
/*********************************************************************
 *  made by : Kees Lemmens
 *  version : 0.1
 *  date    : May 1999
 *
 *  class name  : ColorTables
 *  function    : Setup the color tables as used in old Sepran (plcoli.f)
 * 
 *  Modified    : January 2004 JB
 *  Removed fill of first element of colors array
 *  for  color set 0 with  Color.white.
 *  
 *********************************************************************/

import java.awt.Color;
import java.awt.Graphics;

class ColorTables
{
   static final int COLSIZ = 41;
   static final int NRSETS =  4;
   private Color[][] colors;
   protected boolean fixedTable = false;
   protected int colorset = 0;
   int lastcolor = 0;
   
   public void BuildColorTables()
   {
      // This coloring schema was taken from the rather grubby stuff
      // in the Fortran routine plcoli.f from Sepran !!
      // setup the color tables (NRSETS)
      int tabsize,gamma,colnr,curcol;
      colors = new Color[NRSETS][COLSIZ];
  
      // color set 0 : blue -> red -> yellow
      colnr   = 0;
      tabsize = 10; // The first stupid table is shorter than the rest ...

      gamma   = 256/tabsize;

//      colors[0][colnr++] = Color.white;
      for(int m=0; m<tabsize;m++)
      {
	 colors[0][colnr        ] = new Color(gamma*m,      0,255-gamma*m);
	 colors[0][colnr+tabsize] = new Color(    255,gamma*m,          0);
	 colnr++;
      }
      colors[0][colnr+tabsize] = Color.yellow;

      // color set 1 : blue -> red -> yellow
      colnr   = 0;
      tabsize = 20;
      gamma   = 256/tabsize;

      for(int m=0; m<tabsize;m++)
      {
	 colors[1][colnr        ] = new Color(gamma*m,      0,255-gamma*m);
	 colors[1][colnr+tabsize] = new Color(    255,gamma*m,          0);
	 colnr++;
      }
      colors[1][colnr+tabsize] = Color.yellow;

      // color set 2 : green -> yellow -> red
      colnr   = 0;
      tabsize = 20;
      gamma   = 256/tabsize;

      for(int m=0; m<tabsize;m++)
      {
	 colors[2][colnr        ] = new Color(gamma*m,        255,0);
	 colors[2][colnr+tabsize] = new Color(    255,255-gamma*m,0);
	 colnr++;
      }
      colors[2][colnr+tabsize] = Color.red;

      // color set 3 : white -> gray -> black
      colnr   = 0;
      tabsize = 40;
      gamma   = 256/tabsize;

      for(int m=0; m<tabsize;m++)
      {
	 curcol = 255-gamma*m;
	 colors[3][colnr++] = new Color(curcol,curcol,curcol);
      }
   }
   
   public void setColor(Graphics g, int color)
   {
      if(color != lastcolor)
      {
	 g.setColor(translateColor(color));
	 lastcolor = color;
      }
   }
   
   private Color translateColor(int color)
   {
      if      (color == 0)
	return  Color.black;
      else if(fixedTable == true)
      {	
	 // The first stupid table is shorter than the rest ...
	 if (color > 22) color -= 22;
	 while(color > COLSIZ) color -= COLSIZ;
	 return colors[colorset][color - 1]; 
      }
      else if (color <=  22)
	return colors[0][color       - 1];
      else if (color <=  63)
	return colors[1][color -  22 - 1];
      else if (color <= 104)
	return colors[2][color -  63 - 1];
      else
	return colors[3][color - 104 - 1];
   }
}

