 
/*********************************************************************
 *  made by : Kees Lemmens
 *  version : 0.1
 *  date    : May 1999
 *
 *  class name  : JsepView
 *  function    : Main and toplevel part of the Sepran Viewer as applet
 *
 * The toplevel window contains 2 regions : canvas and control,
 * each of them implemented in a separate class. The menu header 
 * sits in separate frame as applets don't allow menubars in a browser.
 *
 * modified : July 1999 KL
 * Complete rewrite of scrolling and scaling classes
 *
 * modified : July 1999 KL 
 * Added Big Endian data reader
 * 
 *********************************************************************/

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.StringTokenizer;
import java.applet.*;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.MenuBar;
import java.awt.Panel;
import java.awt.PrintJob;
import java.awt.Graphics;
import java.awt.event.*;
import java.util.Properties;

public class JsepView extends Applet     // use for applets
implements Runnable
{
   // Some predefined settings :

   static final String TITLE = "Java Sepran Plotfile Viewer";
   String helpfile = "jsepview.1";   
   int    helpmode = FileViewer.USEURL;
   byte   recordType=RecordReader.NETBINARY;
   
   // And the class variables + methods :

   ViewHeader   header;
   ViewCanvas   canvas;
   ViewControl  control;
   ScrollCanvas viewport;
   Dimension    canvasSize;
   String       filelist[]; // storage for filename(s)
   int          current;
   Thread       thread=null;
   Player       player=null;
   boolean      playmode=false;
   static int nrOfThreads=0;

    // constructor as in application is not allowed in applet !
   //  public JsepView(String argv[])

   public void init()
   {
      int count = 0;

      helpfile = getCodeBase() + helpfile;   // Use URL mode
      current  = 0;

      // Get a parser for the single parameter argv in the HTML file to
      // obtain the playlist (separator character is now "|"):

      StringTokenizer stok = new StringTokenizer(getParameter("argv"),"|");
      Debug.print("Nr of tokens in argv : " + stok.countTokens());
      
      filelist    = new String[stok.countTokens()];

      while(stok.hasMoreTokens())
      {
	 filelist[count] = getCodeBase() + stok.nextToken();
	 if(Debug.LEVEL >=1)
	   Debug.print(filelist[count]);
	 count++;
      }
      
      header   = new ViewHeader(this);
      
      canvasSize = new Dimension(400,400);
      viewport = new ScrollCanvas(this,canvasSize);
      canvas   = viewport.canvas;
      
      control  = new ViewControl(this);
      
      header.add(fakeBar(), "South"); // provide a simple replacement
      
      setLayout(new BorderLayout()); // necessary for applets
      
      add(header,  "North");
      add(viewport,"Center");
      add(control, "South");
      
      setSize(450,450);

//      setTitle(WINDOWTITLE); // not available in applet !
      doLayout();
   }

   public void run() // obligatory for Runnable
   {
      start();
   }

   public void start()
   {
      setVisible(true); // show the top widget 
      if(filelist.length == 0)
	canvas.openPlotSelector();
      else
	header.setCurrentName(filelist[current]);
   }

   public void stop()
   {
      setVisible(false); // hide the top widget (window)
      System.exit(0);
   }

   public void newWindow()
   {
      // not allowed in applet !
   }
   
   public void setMenuBar(MenuBar mb)
   {
      // Menu bars are not allowed in applets as they may confuse the
      // user with the menubar in the browser or appletviewer.

      // Next constructs a menubar in a separate frame floating on top
      // of the browser or appletviewer :
      
      //  Frame frame = new Frame();
      //  frame.setMenuBar(mb);
      //  frame.setSize(300,55);
      //  frame.setVisible(true);
   }
   
   public Panel fakeBar() // this constructs a simple fake replacement
   {
      Panel panel = new Panel();
      Button colorbutton = new Button("Next Colorset");
      Button helpbutton = new Button("Help");
      
      colorbutton.addActionListener
	( new ActionListener()  // provide it as an inner class :
	{
	   public void actionPerformed(ActionEvent e)
	   {  
	      int cur = canvas.getFixedTable();
	      if(cur == -1)
		canvas.setFixedTable(ViewMenu.COLSET1);
	      else if(cur >= ViewMenu.COLSET4)
		canvas.setNormalTable();
	      else
		canvas.setFixedTable(cur + 1);
	      canvas.repaint();
	   }
	});
      
      helpbutton.addActionListener(header.mhelp);
      
      panel.setLayout(new BorderLayout());
      
      panel.add(colorbutton,"West");
      panel.add(helpbutton,"East");
      return panel;
   }
   
   public void setPrevious()
   {
      if(current>0)
      {
	 current--;
	 header.setCurrentName(filelist[current]);
	 canvas.repaint();
      }
   }

   public void togglePlay()
   {
      playmode = !playmode; // toggle
      
      if(playmode == true)
      {
	 control.but[control.PLAY].setBackground(Color.red);
	 player = new Player(this);
      }
      else  if(player != null)
      {
	 control.but[control.PLAY].setBackground(Color.white);
	 player.stop();
      }
   }
   
   public void setNext()
   {
      if(current<filelist.length-1)
      {
	 current++;
	 header.setCurrentName(filelist[current]);
	 canvas.repaint();
      }
   }

   public Frame getParentFrame(Component component)
   {
      while(! (component instanceof Frame))
	component = component.getParent();
      return (Frame)component;
   }
   
   public String getCurrent()
   {
      return filelist[current];   
   }

   // Now some more general but very useful routines for printing, executing
   // commands etc. :
   
   public void printMainWindow(Properties printprefs)
   {
      printComponent( printprefs, this);
   }
   
   public void printComponent(Properties printprefs, Component printable)
   {
      Frame toplevel = getParentFrame(this);
      
      PrintJob job = this.getToolkit().
      	getPrintJob(toplevel,"JsepView",printprefs);
      
      if(job != null)
      {
	 Graphics pg = job.getGraphics();

	 // Center the picture on the page :
	 Dimension tsize = printable.getSize();
	 Dimension psize = job.getPageDimension();

	 pg.translate((psize.width  - tsize.width)/2,
		      (psize.height - tsize.height)/2);

	 printable.printAll(pg);
	 pg.dispose();
	 job.end();
      }    
      System.err.println("Printing window " + printable + " ....");
   }
  
   public void execCommand(String cmdArray[], PrintStream o)
   {
      if(Debug.LEVEL >= 1)
      {
	 System.err.print("Executing system command : ");
	 for(int n=0;n<cmdArray.length;n++)
	   System.err.print(cmdArray[n] + " ");
	 System.err.println();
      }
      
      try
      {
	 String s;
	 Process           p = Runtime.getRuntime().exec(cmdArray);
	 InputStreamReader i = new InputStreamReader(p.getInputStream());
	 BufferedReader pOut = new BufferedReader(i); // command output
	 
	 while ((s = pOut.readLine()) != null)
	   o.println(s);
	 
	 int exit_value = p.waitFor();
	 System.err.println("Exit status : " + exit_value);
      }
      catch (IOException io)
      {	 System.err.println("IOException :" + io);}
      catch (InterruptedException in)
      {  System.err.println("Interrupted Exception :" + in);}
   }

   // And there is also no main as we are an applet now !
}
