      subroutine addscal
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.1  date 15-03-2014 Use original name
!        version  1.0  date 11-12-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add scalar name to the list of scalars
!
! **********************************************************************
!
!                       KEYWORDS
!
!     scalar
!     name
!     add
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusnamef90'
      include 'SPcommon/cpost5f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3159   No  space for scalars left
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'addscal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from addscal'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numscals = numscals+1   ! new sequence number of scalar

!     --- Check available space

      if ( numscals>maxcls ) then

!     --- numscals>maxcls

         call errint ( numscals, 1 )
         call errint ( maxcls, 2 )
         call errsub ( 3159, 2, 0, 0 )
         go to 1000

      end if  ! ( numscals>maxcls )

      chscalars(numscals) = stringim(1:nchar)    ! name of scalar
      dimscals(numscals) = 1         ! dimension of scalar name
      ncharscals(numscals) = nchar   ! number of characters of scalar name
      realrd = numscals+100000000    ! sequence number of scalar

1000  call erclos ( 'addscal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End addscal'

      end if  ! ( debug )

      end
