      subroutine asknxt
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.10 date 15-03-2014 Extra debug
!        version  2.9  date 14-11-2013 new call to readexpression
!        version  2.8  date 20-04-2012 Extra debug
!
!   copyright (c) 1990-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read the next item (keyword, string, or number) from the input
!     If at the call of ASKNXT ityprd is set equal to 1 (common cseprd), a
!     new record is read
!     If ispecl = 4 at input, also the occurence of a - sign not followed
!     by a digit and the occurence of a ) sign is marked
!     If ispecl = 5 at input, the occurence of a ( sign is marked
!     If ispecl = 3 at input, the complete text is interpreted as a string
!     If ispecl = 8 at input, the position of the next equals sign+1 is returned
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchnewf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost1f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!     none
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished, debug
      integer jspecl, isign

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if the loop has been finished (true)
!                    or not (false)
!     isign          Sign read before {
!     jspecl         mod(ispecl,10)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     READCD         Read next card
!     READCN         Read name of constant and substitute value
!     READEXPRESSION Read and evaluate expression between two backslashes
!     READNAME       Read name following %-sign
!     READNM         Read number
!     READQT         Read string between quotes
!     READWR         Read keyword
! **********************************************************************
!
!                       I/O
!
!    Possibly information is read from the standard input file
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1221   Text to be considered as strng is stored in more than one record
! **********************************************************************
!
!                       PSEUDO CODE
!
!    finished := false
!    if ( istop=3 ) then
!       istop := 0
!       finished := true
!    If ( not finished and ityprd=1 ) then
!       Read next record
!    while ( not finished ) do
!       Find next non-blank in record
!       if ( icolum>maxcolum ) then
!          if ( istop=2 ) then
!             Read next record
!             Set kcolum
!             if ( ityprd=0 ) then
!                finished := true
!          else (istop<2)
!             ityprd := 1
!             finished := true
!       else if ( ispecl=3 ) then
!          Put text in string
!          ityprd := 5
!       else if ( next character is letter ) then
!          Read string
!          ityprd := 4
!          finished := true
!       else if ( next character is digit, plus or minus followed by digit
!                 or followed by + folowed by digit ) then
!          Read number
!          Set ityprd
!          finished := true
!       else if ( next character is ' ) then
!          Read string until end quote is found
!          ityprd := 5
!          finished := true
!       else if ( next character is $ ) then
!          Read name of constant (separated by space or end of record)
!          find name in integer or real constants
!          set ityprd
!          finished := true
!       else if ( next character is % ) then
!          Read name of vector (separated by space or end of record)
!          find name in names
!          set ityprd
!          finished := true
!       else if ( mod(ispecl/10,10)=1 and next character is ~ ) then
!          Read name of variable (separated by space or end of record)
!          find name in list of variables
!          set ityprd
!          finished := true
!       else if ( ispecl=4 and next character is - ) then
!          ityprd := 6
!          finished := true
!       else if ( ispecl=4 and next character is ) ) then
!          ityprd := 7
!          finished := true
!       else if ( ispecl=5 and next character is ( ) then
!          ityprd := 8
!          finished := true
!       else if ( ispecl=8 and next character is = ) then
!          ityprd := 9
!          finished := true
!       else if ( ispecl/100=1 and next character is special ) then
!          ityprd = ichar(character)
!          finished := true
!       else if ( next two characters are // ) then
!          Read next record
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'asknxt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from asknxt'
         write(irefwr,1) 'ityprd, istop, numvec, lastitem', &
                          ityprd, istop, numvec, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( istop==3 ) then
         istop = 0
         go to 1000
      end if

      finished = .false.
      jspecl = mod(ispecl,10)
      if ( ityprd==1 ) then

!     --- ityprd = 1, i.e. end of record found, read next record

         call readcd
         if ( ityprd==0 ) finished = .true.

      end if
      if ( ierror/=0 ) go to 1000

200   do while ( .not. finished )

!     --- Find next non-blank in record

         lastitem = lastitem+1
         if ( debug ) then
            write(irefwr,1) 'ispecl, lastitem, typeitems(lastitem)', &
                             ispecl, lastitem, typeitems(lastitem)
            write(irefwr,1) 'jspecl, numvec/2', jspecl, numvec
         end if
         if ( lastitem>nitems ) then

            if ( istop==2 ) then

!           --- istop = 2, read next record

               call readcd
               if ( ityprd==0 ) finished = .true.

            else

!           --- istop<2, end of record reached, return

               ityprd = 1
               finished = .true.

            end if

         else if ( jspecl==3 ) then

!        --- Special situation, the complete text is considered as a string
!            In this case only one record is allowed

            firstcol = firstcolitems(lastitem)
            numrecord = firstcol/240+1
            firstcol = mod(firstcol,240)
            if ( numrecstored-numrecord>0 ) then

!           --- text is stored in more than one record

               call errsub ( 1221, 0, 0, 0 )
               lastcol = lastcolumn(numrecord)
               call errcrd
               finished = .true.
               go to 200

            end if
            lastcol = mod(lastcolitems(nitems),240)
            nchar = lastcol-firstcol+1
            string(1:nchar) = cardimage(numrecord)(firstcol:lastcol)
            ityprd = 5
            finished = .true.
            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd',lastitem, ityprd
               write(irefwr,*) string(1:nchar)
               write(irefwr,1) 'numvec/3', numvec
            end if
            lastitembef = lastitem-1

         else if ( typeitems(lastitem)==3 ) then

!        --- Letter found, read keyword

            call readwr

            finished = .true.
            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd',lastitem, ityprd
               write(irefwr,*) string(1:nchar)
               write(irefwr,1) 'numvec/4', numvec
            end if

         else if ( typeitems(lastitem)==1 .or. &
                   typeitems(lastitem)==2) then

!        --- number found

            call readnm
            lastitembef = lastitem-1
            finished = .true.
            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
               write(irefwr,2) 'realrd', realrd
               write(irefwr,1) 'numvec/5', numvec
            end if

         else if ( typeitems(lastitem)==4 ) then

!        --- ' found, read text till end quote has been found

            call readqt
            lastitembef = lastitem-1
            finished = .true.
            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd',lastitem, ityprd
               write(irefwr,1) 'numvec/6', numvec
               write(irefwr,*) string(1:nchar)
            end if

         else if ( typeitems(lastitem)==9 ) then

!        --- $ found, read name of variable and set ityprd

            call readcn
            lastitembef = lastitem-1
            finished = .true.
            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
               write(irefwr,2) 'realrd', realrd
               write(irefwr,1) 'numvec/7', numvec
               write(irefwr,*) string(1:nchar)
            end if

         else if ( typeitems(lastitem)==10 ) then

!        --- % found, read name of vector or scalar and set ityprd

            call readname
            lastitembef = lastitem-1
            finished = .true.
            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
               write(irefwr,2) 'realrd', realrd
               write(irefwr,1) 'numvec/7a', numvec
               write(irefwr,*) string(1:nchar)
            end if

         else if ( typeitems(lastitem)==17 ) then

!        --- { found, read expression and translate to integer or real
!            Set typrd

            if ( iexpression==1 ) then

!           --- Special case, leave the subroutine with ityprd = 100

               ityprd = 100
               go to 1000

            end if  ! ( iexpression==1 )

            if ( lastitem>1 .and. typeitems(lastitem-1)==11 ) then

!           --- Prior item is minus sign

               isign = -1

            else

!           --- Prior item is not minus sign

               isign = 1

            end if  ! ( typeitems(lastitem-1)==11 )
            call readexpression ( cardim(1:lastp) )
            realrd = realrd*isign
            lastitembef = lastitem-1
            finished = .true.
            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd, realrd', &
                                lastitem, ityprd, realrd
               write(irefwr,1) 'numvec/9', numvec
            end if  ! ( debug )

         else if ( jspecl==4 .and. typeitems(lastitem)==11 ) then

!        --- - found, check if it is not immediately followed by a number

            if ( lastitem<nitems ) then

!           --- - is followed by something

               if ( typeitems(lastitem+1)==1 .or. &
                    typeitems(lastitem+1)==2 .or. &
                    typeitems(lastitem+1)==9 ) then

!              --- - is followed by a number or a $

                  if ( firstcolitems(lastitem)+1/= &
                       firstcolitems(lastitem+1) ) then

!                 --- - is not immediately followed by a number

                     ityprd = 6
                     finished = .true.

                  end if

               else

!              --- - is not followed by a number

                  ityprd = 6
                  finished = .true.

               end if

            else

!           --- - is not followed by something

               ityprd = 6
               finished = .true.

            end if
            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
               write(irefwr,1) 'numvec/10', numvec
            end if
            lastitembef = lastitem-1

         else if ( jspecl==4 .and. typeitems(lastitem)==6 ) then

!        --- ) found

            ityprd = 7
            finished = .true.
            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
               write(irefwr,1) 'numvec/11', numvec
            end if
            lastitembef = lastitem-1

         else if ( jspecl==5 .and. typeitems(lastitem)==5 ) then

!        --- ( found

            ityprd = 8
            finished = .true.
            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
            end if
            lastitembef = lastitem-1

         else if ( jspecl==8 .and. typeitems(lastitem)==13 ) then

!        --- = found

            ityprd = 9
            finished = .true.
            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
            end if
            lastitembef = lastitem-1

         end if
         if ( ierror/=0 ) go to 1000

      end do

1000  call erclos ( 'asknxt' )
      if ( debug ) then
         write(irefwr,1) 'numvec, lastitem/end', numvec, lastitem
         write(irefwr,*) 'End asknxt'
      end if  ! ( debug )

      end
