      subroutine er0034 ( ibuffr, buffer, isubr, ictime, kmesh, kprob,
     +                    isol, iseqnr, islold, maxvecsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.7  date 15-08-2007 Debug statements
!        version  5.6  date 15-04-2005 Decrease priorities
!        version  5.5  date 14-11-2003 Replace creatvbf by creatv01
!        version  5.4  date 18-03-2002 Call to inirst
!        version  5.3  date 09-07-2001 Extra error message
!        version  5.2  date 21-02-2001 New call to readcr
!        version  5.1  date 08-10-1999 Allow variables
!        version  5.0  date 17-08-1999 Extra parameter maxvecsol
!        version  4.0  date 07-09-1998 Extra parameter islold
!        version  3.2  date 18-06-1998 Include common
!        version  3.1  date 10-11-1997 New call to ini015
!        version  3.0  date 26-10-1997 Extra parameters ibuffr, buffer
!
!   copyright (c) 1991-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   ER0034 creates a number of SEPRAN vectors and fills (a part of)
!   these arrays if required
!   ER0034 replaces the call of the SEPRAN subroutine:
!   CREATV: Same meaning as ER0034, but without reading of the
!           standard input file
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/cbuffr'
      include 'SPcommon/clinsl'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, ictime, kmesh(*), kprob(*), isol(5,*), iseqnr,
     +        ibuffr(*), islold(5,*), maxvecsol
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     ictime         i    Indication if information must be read from standard
!                         input file ( 0 ) or that information read before is
!                         used ( 1 )
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!                         Usually isol has the structure of a solution vector,
!                         hence ISOL is used as actual argument.
!                         isol may contain one vector, in which case isol is an
!                         integer array of length 5.
!                         If isol must contain information of n (>1) vectors,
!                         then isol should be declared as:
!                            INTEGER isol(1;5,1:n)
!                         isol(.,i) refers to the i-th vector, its starting
!                         address is given by isol(1,i).
!     isubr         i/o   Indicates the type of calling subroutine according to
!                         isubr = isubrout+100*(ivecsq-1) with
!                         isubrout sequence number of caling subroutine:
!                             1:  CREATE
!                             2:  PRESDF
!                             3:  CREAVC
!                             4:  PRESTM
!                             5:  ERCRET
!                             6:  ERPRES
!                         ivecsq sequence number of first vector
!                         At output isubr contains the maximum sequence number
!                         of the array created
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     maxvecsol      i    Largest sequence number of vector in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer length, nbufol, ipincr, iprinc, leninp, lenrin, maxlen,
     +        nstart, ipstar, i, numvec, iinpxx, irinxx, ipkmsn, ipcopy,
     +        ivecsq, ipinforinpcre
      logical renum, debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iinpxx         Either iinpcr or iinpdf depending on isubr
!     ipcopy         Starting address of array to be copied in array
!                    IBUFFR
!     ipincr         Starting position of the local array INPCRE in array
!                    IBUFFR
!     ipinforinpcre  Starting position of array INFORINPCRE in IBUFFR
!     ipkmsn         Starting address of array KMESH part n into array
!                    IBUFFR
!     iprinc         Starting position of the local array RINCRE in array
!                    IBUFFR
!     ipstar         temporary starting address in IBUFFR
!     irinxx         Either irincr or irindf depending on isubr
!     ivecsq         Sequence number of vector in ivectr
!     length         Help parameter to indicate the length of arrays
!     leninp         Computed length of integer input array
!     lenrin         Computed length of array user
!     maxlen         Maximum amount of contiguous space in IBUFFR after a
!                    starting address
!     nbufol         help variable to store the old value of nbuffr
!     nstart         Number of positions in array ISTART
!     numvec         Number of vectors to be created by subroutine CREATE
!     renum          Indicates if array INPCRE must be renumbered (true)
!                    or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inigettext, inilen, inidbl

!     CREATV01       Performs the actual creation and filling of vectors
!     ER0049         Reorder the input array INPCRE of subroutine CREATE
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         warnings
!     INI000         Copy integer array to other integer array possibly itself
!     INI015         Extract infor entries of integer and real input arrays from
!                    ISEPIN
!     INI0151        Replace values in rinput corresponding to variables by
!                    actual value
!     INI058         Check how much contiguous space can be found after starting
!                    address
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI062         reserve space for temporary arrays
!     INI069         Reserve space at the end of IBUFFR for a semi-permanent
!                    array
!     INI083         Create new array in IBUFFR in the case that already a
!                    temporary copy has been stored in IBUFFR
!     INI085         Create new array in BUFFER in the case that already a
!                    temporary copy has been stored in BUFFER
!     INICHINPSOL    Adapt input array with respect to variables
!     INIDBL         Transform pointer for IBUFFR to pointer for BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
!     INIPRI         Decrease priorities of arrays
!     INIRST         Reset BUFFER to old value after a call to ini079
!     READCR         Read card input for subroutine CREATE
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array KPROB has not been filled
!      75   Array KMESH has not been filled
!     813   ICTIME out of range
!     814   ICTIME = 1, whereas subroutine PRESTM has not been called before
!    1145   ICTIME = 1, whereas subroutine CREATE has not been called before
!    2104   Vector number of vector to be created too large
!    2260   No input corresponding to sequence number
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Check input

      call eropen ( 'er0034' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0034'
         write(irefwr,1) 'isubr', isubr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kprob(2)/=101 ) then
         call errint ( kprob(1), 1 )
         call errsub ( 26, 1, 0, 0 )
      end if
      if ( kmesh(2)/=100 ) then
         call errint ( kmesh(1), 1 )
         call errsub ( 75, 1, 0, 0 )
      end if
      if ( isubr<3 .and. (ictime<0 .or. ictime>1) ) then
         call errint ( ictime, 1 )
         call errsub ( 813, 1, 0, 0 )
      end if
      if ( isubr<3 .and. ictime==1 ) then
         if ( isubr==1 .and. iinpcr<=0 ) call
     +      errsub ( 1145, 0, 0, 0 )
         if ( isubr==2 .and. iinpdf<=0 ) call
     +      errsub ( 814, 0, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

!     --- Read input and store it in arrays iuser and user

      if ( isubr==1 ) then

!     --- ISUBR = 1, CREATE

         irinxx = irincr
         iinpxx = iinpcr

      else if ( isubr==2 ) then

!     --- ISUBR = 2, PRESDF/TM

         irinxx = irindf
         iinpxx = iinpdf

      else if ( isubr<=4 ) then

!     --- ISUBR>2, PRESBC or CREATN

         call ini015 ( ibuffr, buffer, isubr+1, iseqnr, iinpxx, irinxx )
         if ( iinpxx==0 ) then

!        --- Error: no input found

            call errint ( iseqnr, 1 )
            call errwar ( 2260, 1, 0, 0 )

         end if

      else

!     --- ISUBR>4, PRESBC or CREATN

         call ini015 ( ibuffr, buffer, mod(isubr,100)-1, iseqnr, iinpxx,
     +                 irinxx )
         if ( iinpxx==0 ) then

!        --- Error: no input found

            call errint ( iseqnr, 1 )
            call errwar ( 2260, 1, 0, 0 )

         end if

      end if
      if ( debug )
     +   write(irefwr,1) 'ictime, isubr, iseqnr, iinpxx',
     +                    ictime, isubr, iseqnr, iinpxx

!     --- First reserve space in buffer array

      if ( ictime==0 .and. isubr<=2 ) then

!     --- ictime=0,  read input from standard input file
!         Reserve temporary space

         lenrin = 1000
         call ini069 ( ibuffr, ipinforinpcre, lenrin, nbufol )
         length = lenrin * intlen + 10000
         if ( kmesh(28)>0 )
     +      call ini060text ( ibuffr, -kmesh(28 ), 'kmeshn' )
         call ini062 ( ibuffr, length, iprinc )

!        --- Check the actual available space (This might be more than length)

         call ini058 ( ibuffr, iprinc, maxlen )
         nstart = 100
         leninp = ( maxlen - lenrin*intlen - nstart ) / 2
         ipstar = iprinc + lenrin*intlen
         ipincr = ipstar + 100
         if ( kmesh(28)>0 ) then
            ipkmsn = inigettext ( kmesh(28), 'kmeshn' )
         else
            ipkmsn = 1
         end if

!        --- Read input and store in arrays INPCRE and RINCRE

         iprinc = inidbl(iprinc)
         call readcr ( numvec, ibuffr(ipstar), ibuffr(ipincr),
     +                 buffer(iprinc), nstart, leninp, lenrin, isubr,
     +                 kmesh, ibuffr(ipkmsn), iseqnr, kprob,
     +                 ibuffr(ipinforinpcre) )

!        --- Adapt variables

         call inichinpsol ( ibuffr(ipincr), leninp )
         call ini0151 ( buffer(iprinc), ibuffr(ipinforinpcre), lenrin )

!        --- Reorder array INPCRE if necessary

         if ( numvec>1 ) then

!        --- numvec>1, hence possibly incorrect sequence

            renum = .false.
            do i = 1, numvec-1
               if ( ibuffr(ipstar+i-1)>ibuffr(ipstar+i) )
     +              renum = .true.
            end do

            if ( renum ) then

!           --- Renumbering of sequence necessary

               call er0049 ( ibuffr(ipincr), ibuffr(ipstar), numvec,
     +                       ibuffr(ipincr+leninp) )
               ipincr = ipincr + leninp

            end if

         end if

!        --- Fill arrays INPCRE and RINCRE in the final positions in IBUFFR
!            The length is given by leninp and lenrin
!            The new starting positions are computed with the aid of INI093

         if ( lenrin>0 ) then
            call ini085 ( ibuffr, irinxx, lenrin, iprinc, 'rincre' )
         else
            irinxx = 0
         end if
         call ini083 ( ibuffr, iinpxx, leninp, ipincr, 'inpcre' )

!        --- Finally release end of ibuffr

         call inirst ( nbufol )

      end if

!     --- Copy arrays INPCRE and RINCRE to end of IBUFFR.  In this way CREATV
!         cannot overwrite the positions used by these arrays

      if ( iinpxx>0 ) then
         leninp = inilen(iinpxx)
      else
         leninp = 7
      end if
      if ( irinxx>0 ) then
         lenrin = inilen(irinxx)
      else
         lenrin = 0
      end if

      call ini069 ( ibuffr, iprinc, leninp+lenrin, nbufol )
      ipincr = iprinc + lenrin
      ibuffr(iprinc) = 0

!     --- Guarantee that arrays INPCRE and RINCRE are placed into IBUFFR

      if ( iinpxx>0 ) call ini060text ( ibuffr, -iinpxx, 'iinpxx' )
      if ( irinxx>0 ) call ini060text ( ibuffr, -irinxx, 'irinxx' )

!     --- Copy contents of INPCRE and RINCRE

      if ( iinpxx>0 ) then
         ipcopy = inigettext(iinpxx, 'iinpxx')
         call ini000 ( ibuffr, ibuffr, ipcopy, ipincr, leninp, 1 )

!        --- Adapt variables

         call inichinpsol ( ibuffr(ipincr), leninp )

      else
         ibuffr(ipincr+1) = 1
         ibuffr(ipincr+2) = 0
         ibuffr(ipincr+3) = 1
         ibuffr(ipincr+4) = 1
         ibuffr(ipincr+5) = 0
         ibuffr(ipincr+6) = 0
      end if
      if ( lenrin>0 ) then
         ipcopy = inigettext(irinxx, 'irinxx')
         call ini000 ( ibuffr, ibuffr, ipcopy, iprinc, lenrin, 1 )
      end if

!     --- Create vectors

      ibuffr(ipincr) = -isubr
      iprinc = inidbl(iprinc)
      ivecsq = isubr/100
      numvec = ibuffr(ipincr+1)
      if ( ivecsq+numvec>maxvecsol ) then

!     --- Too much vectors must be created

         call errint ( ivecsq+1, 1 )
         call errint ( ivecsq+numvec, 2 )
         call errint ( maxvecsol, 3 )
         call errsub ( 2104, 3, 0, 0 )

      end if
      if ( ierror/=0 ) go to 1000

      call creatv01 ( ibuffr, buffer, kmesh, kprob, isol,
     +                ibuffr(ipincr), buffer(iprinc), islold )
      if ( isubr==1 .or. isubr==3 .or. mod(isubr,100)==5 ) then

!     --- ISUBR = 1, CREATE

         irincr = irinxx
         iinpcr = iinpxx

      else

!     --- ISUBR = 2, PRESDF/TM

         irindf = irinxx
         iinpdf = iinpxx

      end if
      if ( isubr>4 ) isubr = ibuffr(ipincr)

!     --- Restore nbuffr

      call inirst ( nbufol )

!     --- Decrease priorities

      call inipri

1000  call erclos ( 'er0034' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End er0034'

      end if  ! ( debug )

      end
