      subroutine er0085 ( ichois, usol, indprf, kprobf, nodes, coor, &
                          amin, amax, ndim, nnodes )
! ====================================================================== &
!
!        programmer     Guus Segal
!        version 1.1    date 26-12-1998 Include commons
!        version 1.0    date 03-10-1994
!
!   copyright (c) 1994-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute minimum and maximum of vector in nodal points given by array nodes
!   Depending on ichois a user function subroutine may be called
! **********************************************************************
!
!                       KEYWORDS
!
!   maximum
!   minimum
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, nnodes, nodes(nnodes), indprf, kprobf(*), ndim
      double precision coor(ndim,*), usol(*), amin, amax

!     amax    o    Maximum value in solution array
!     amin    o    Minimum value in solution array
!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     ichois  i    Choice parameter
!     indprf  i    indication if kprob part f has been filled (>0) or not (<0)
!                  If <0 -indprf contains the number of unknowns per point
!     kprobf  i    KPROB part f: contains the number of unknowns per point
!                  accumulated
!     ndim    i    Dimension of the space.
!     nnodes  i    Number of nodes in nodes
!     nodes   i    Contains the node numbers
!     usol    i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nusol, inunkp, i, inode, istart
      double precision x, y, z, funval

!     funval  Function value
!     i       Counting variable
!     inode   Node sequence number
!     inunkp  Number of degrees of freedom in first point
!     istart  First position used at a part
!     nusol   Total number of unknowns
!     x       x co-ordinate of point
!     y       y co-ordinate of point
!     z       z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision funbnd

!     FUNBND  Function subroutine to compute a value
!     INI005  Compute mininimum and maximum value of a real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    if ( ichois=1 ) then
!       if ( indprf<0 ) then
!          Compute minimum and maximum of array usol of length -indprf*nnodes
!       else ( indprf>0 )
!          Compute minimum and maximum of array usol of length kprobf(nnodes+1)
!    else ( ichois=2 )
!       inunkp = -indprf
!       amin := rinfin
!       amax := -rinfin
!       For i := 1 (1) nnodes
!          inode := nodes(i)
!          if ( indprf>0 ) then
!             inunkp := kprobf(i+1) - kprobf(i)
!          if ( inunkp>0 ) then
!             func_val := funbnd(x,y,z,usol(inode),inunkp)
!             amin := min(amin,func_val)
!             amax := max(amax,func_val)
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      if ( ichois.eq.1 ) then

!     --- ichois = 1, standard minimum and maximum value

         if ( indprf.lt.0 ) then
            nusol = -indprf*nnodes
         else
            nusol = kprobf(nnodes+1)
         end if

!        --- Compute minimum and maximum of array usol

         call ini005 ( usol, amin, amax, nusol )

      else

!     --- ichois = 2, call function subroutine

         inunkp = -indprf
         amin   = rinfin
         amax   = -rinfin
         y = 0d0
         z = 0d0

         do i = 1, nnodes

            inode = nodes(i)
            if ( indprf.gt.0 ) then
               inunkp = kprobf(i+1) - kprobf(i)
               istart = kprobf(i)+1
            else
               istart = inunkp*(i-1)+1
            end if
            if ( inunkp.gt.0 ) then
               x = coor(1,inode)
               if ( ndim.gt.1 ) y = coor(2,inode)
               if ( ndim.gt.2 ) z = coor(3,inode)
               funval = funbnd ( x, y, z, usol(istart), inunkp )
               amin = min(amin,funval)
               amax = max(amax,funval)
            end if

         end do

      end if

      end
