      subroutine erchcordispl ( ibuffr, buffer, kmesh, kprob, isol, &
                                factor )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.2  date 09-11-2012 Long length ibuffr
!        version  1.1  date 17-05-2008 to0023-->tolincomb
!        version  1.0  date 16-02-2001
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Change the coordinates as stored in coor (kmesh) using factor times the
!    displacement stored in isol
! **********************************************************************
!
!                       KEYWORDS
!
!     displacement
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cnamtbf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*)
      integer kmesh(*), kprob(*), isol(5)
      double precision buffer(*), factor

!     buffer        i/o   Buffer array
!     factor         i    Multiplication factor
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     kmesh         i/o   Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipcoor, ipusol
      integer indprp, ndim, npoint, nphys, indprh, indprf

!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprh         indication if kprob part h has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     ipcoor         starting address of co-ordinates array in BUFFER
!     ipusol         Starting address of solution array in BUFFER
!     ndim           Dimension of the space.
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI065         compute pointers to degrees of freedom of vectors
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     TOLINCOMB      Store linear combination a vec1 + b vec2 into vec3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2297   olution vector does not have ndim degrees of freedom per point
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ndim := kmesh(6)
!     Activate solution vector
!     Check if it has the same dimensions as the coordinates vector
!     subtract coor - usol and store in usol
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erchcordispl' )

!     --- ndim := kmesh(6)

      ndim = kmesh(6)
      npoint = kmesh(8)

!     --- Activate solution vector

      call ini065 ( ibuffr, isol, kprob, indprf, indprh, indprp, nphys )

!     --- Check if it has the same dimensions as the coordinates vector

      if ( nphys/=ndim .or. indprf/=0 .or. isol(5)/=ndim*npoint ) &
         then

!     --- Coordinates vector has not the same structure as the solution vector

         call errint ( ndim, 1 )
         call errchr ( namarr(isol(1)), 1 )
         call errsub ( 2297, 1, 0, 1 )
         go to 1000

      end if

!     --- add factor * usol to coor and store in coor

      call ini060text ( ibuffr, -kmesh(23), 'coor' )
      ipcoor = inidgttext ( kmesh(23), 'coor' )
      ipusol = inidgttext ( isol(1), 'usol' )
      call tolincomb ( buffer(ipcoor), buffer(ipusol), buffer(ipcoor), &
                       1d0, factor, 0d0, ndim*npoint, ibuffr, 0, &
                       .true., 3 )

1000  call erclos ( 'erchcordispl' )
      end
