      subroutine ercomp ( ibuffr, buffer, kprob, isepi2, isepi3, nsubr,
     +                    maxi3, kmesh, manag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 06-07-2011 Use allocate
!        version  4.0  date 29-11-2006 Extra parameter manag
!        version  3.5  date 20-10-2006 iniget-->inigettext
!
!   copyright (c) 1993-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the main program SEPCOMP from the standard input file
!    Store the result in IBUFFR and references in isepi2/3 or ISEPIN
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/cbuffr'
      include 'SPcommon/cseqnr'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprob(*), nsubr, maxi3, isepi2(4,nsubr), isepi3(3,maxi3),
     +        kmesh(*), ibuffr(*), manag(*)
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     isepi2         i    Block 2 of array ISEPIN
!                         See subroutine ini009 for a description
!     isepi3         i    Block 3 of array ISEPIN
!                         See subroutine ini016 for a description
!     kmesh          i    Standard SEPRAN array, containing information of
!                         the mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     manag          i    Manager array that sets the parameters in subroutine
!                         SEPSTR
!                         See subroutine READSP for a description
!     maxi3          i    Maximum length of first parameter in two-dimensional
!                         array isepi3
!     nsubr          i    Number of different subroutines that may store their
!                         information in ISEPIN
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nprob, nbufol, iprnsp, ipos,
     +        niinp, nrinp, iptemp, error
      integer, allocatable :: iincom(:)
      double precision, allocatable :: rincom(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     iincom         Local integer input array
!     ipos           Position in array ISEPI3
!     iprnsp         Starting address of array rinspc in IBUFFR
!     iptemp         Temporary starting address of array in IBUFFR
!     nbufol         help variable to store the old value of nbuffr
!     niinp          Length of integer input array
!     nprob          Number of various problems
!     nrinp          Length of real input array
!     rincom         Local double precision input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inigettext, inidbl, inidgttext

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI080         Create permanent space in array BUFFER
!     INI081         Create permanent space in array IBUFFR
!     INIDBL         Transform pointer for IBUFFR to pointer for BUFFER
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     READSC         Read the input for the structure of the main program
!                    sepcomp
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READCM
!    The first record must have the following shape:
!    STRUCTURE
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1556   Sequence number exceeds maximum
!    1872   Sequence number used more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercomp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercomp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nprob  = kprob(40)
      nbufol = nbuffr
      niinp = 20000
      nrinp = 1000

      allocate ( iincom(niinp), rincom(nrinp), stat = error )
      if ( error/=0 ) call eralloc ( error, niinp+nrinp, 'iincom' )
      if ( ierror/=0 ) go to 1000
      iseqcomp = 1

      iprnsp= inidbl(iprnsp)
      call readsc ( ibuffr, buffer, iincom, rincom, niinp, nrinp,
     +              nprob, kmesh, manag )

!     --- Store input arrays as permanent arrays in buffer

      isepi2(3,10) = max ( isepi2(3,10), iseqcomp )
      if ( iseqcomp>isepi2(1,10) ) then

!     --- Sequence number exceeds maximum

         call errint ( iseqcomp, 1 )
         call errint ( isepi2(1,10), 2 )
         call errint ( iseqcomp, 3 )
         call errchr ( 'STRUCTURE', 1 )
         call errsub ( 1556, 3, 0, 1 )

      end if
      ipos = isepi2(4,10) - 1 + iseqcomp
      if ( isepi3(1,ipos)/=0 ) then

!     --- Error sequence number already in use

          call errint ( iseqcomp, 1 )
          call errchr ( 'STRUCTURE', 1 )
          call errsub ( 1872, 1, 0, 1 )

      end if
      call ini081 ( ibuffr, isepi3(1,ipos), niinp, 'iinstruc' )
      if ( nrinp>0 )
     +    call ini080 ( ibuffr, isepi3(2,ipos), nrinp, 'rinstruc' )

!     --- Copy contents to final positions

      iptemp = inigettext(isepi3(1,ipos), 'temp1')
      ibuffr(iptemp:iptemp+niinp-1) = iincom(1:niinp)
      if ( nrinp>0 ) then
         iptemp = inidgttext(isepi3(2,ipos), 'temp2')
         buffer(iptemp:iptemp+nrinp-1) = rincom(1:nrinp)
      end if

      deallocate ( iincom, rincom, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iincom' )

1000  call erclos ( 'ercomp' )
      nbuffr = nbufol
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercomp'

      end if  ! ( debug )

      end
