      subroutine erdivmatdiag ( ibuffr, buffer, matr, isdiag )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.0  date 09-11-2012 Remove intmat
!        version  1.0  date 10-09-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a (diagonal) matrix by a diagonal matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     subdivision
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*)
      integer matr(*), isdiag(5)
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     isdiag         i    Standard SEPRAN array containing information
!                         about the diagonal matrix
!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipmatr, ipdiag
      integer nusol
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipdiag         Starting address of diagal matrix
!     ipmatr         Starting address of matrix
!     nusol          Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     TODIVVEC       Divide 2 vectors components wise, i.e. y_i = x1_i / x2_i
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1296   Complex matrices have not yet been implemented in this subroutine
!    2733   Only diagonal matrices have been implemented in this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erdivmatdiag' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erdivmatdiag'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( matr(2)==110 ) then

!     --- Matrix is diagonal matrix, simple case

         if ( mod(matr(3),10)/=0 ) then

!        --- Diagonal matrix is complex, not yet implemented

            call errsub ( 1296, 0, 0, 0 )
            go to 1000

         end if  ! ( mod(matr(3),10)/=0 )

!        --- Activate arrays

         call ini060text ( ibuffr, matr(1), 'matrix_1' )
         call ini060text ( ibuffr, isdiag(1), 'diag_matrix' )

!        --- Compute starting address

         ipmatr = inidgttext ( matr(1), 'matrix_1' )
         ipdiag = inidgttext ( isdiag(1), 'diag_matrix' )

!        --- Apply multiplication

         nusol = matr(5)
         call todivvec ( buffer(ipmatr), buffer(ipmatr), &
                         buffer(ipdiag), nusol )

      else if ( matr(2)==103 ) then

!     --- Matrix is large matrix

         call errsub ( 2733, 0, 0, 0 )
         go to 1000

      end if  ! ( matr(2)==110 )

1000  call erclos ( 'erdivmatdiag' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erdivmatdiag'

      end if

      end
