      subroutine ererrl ( errlin, ints, nints, reals, nreal, chars, &
                          nchar, numint, numrea, numcha, ipttrg, line )
! ====================================================================== &
!
!        programmer    S.Jansen
!        version  2.13 date 15-11-2012 Long integers
!        version  2.12 date 04-04-2003 Make length of lines equal to 80
!        version  2.11 date 19-01-1999 include commons
!
!   copyright (c) 1989-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     ERERRL prints one line of an error message, thereby substituting
!     integers, reals and character strings.
!     If $, & or % is preceded by ~, the rest of the corresponding integers
!     reals and strings is printed as a list from new lines
! **********************************************************************
!
!                       KEYWORDS
!
!     error_manipulation
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmachnf90'
      include 'SPcommon/cdebugf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character*(*) chars(*), errlin
      character*160 line
      integer (kind=8) ints(*)
      integer ipttrg, nchar, nints, nreal, numcha, numint,  numrea
      double precision reals(*)

!     chars   i    Extra information as character strings.
!     errlin  i    One line of the error message
!     ints    i    Extra information as integers.
!     ipttrg  i/o  Pointer to first position not filled in LINE
!     line    i/o  Text line
!     nchar   i    Number of character strings.
!     nints   i    Number of integers.
!     nreal   i    Number of reals.
!     numcha  i    Number of character strings in error message.
!     numint  i/o  Number of integers in error message
!     numrea  i/o  Number of reals in error message
!     reals   i    Extra information as reals.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character fmtcha*7, fmtint*7, fmtout*8, fmtrea*18
      logical hrdret
      integer i, iendpo, ifact, ifpt, ilast, ilpt, iptend, iptsrc, &
              istapo, leneff, length, lnsize, numdec, itype, ih
      logical nxtsgn, poinfn, tildef

!     fmtcha  variable format to write string to line
!     fmtint  variable format to write integer to line
!     fmtout  variable format to write line to stdout
!     fmtrea  variable format to write real to line
!     hrdret  logical whether hard return is found at end of errlin
!     i       loop counter
!     iendpo  end position of number in line
!     ifact   first 10 power greater than passed integer
!     ifpt    position of the first non-blank character in ERRLIN
!     ih      General integer help variable
!     ilast   last blank on position 80 or before, first non-blank beyond
!             found last blank
!     ilpt    position of the last  non-blank character in ERRLIN
!     iptend  last non blank character in line from error text file.
!     iptsrc  pointer to current character in line from error text file.
!     istapo  start position of number in line
!     itype   Defines the type of numbers to be printed as a block
!             0: nothing
!             1: integers
!             2: reals
!             3: characters
!     leneff  effective string length, i.e. without trailing blanks.
!     length  field length of integer, real or string.
!     lnsize  length of output line
!     numdec  number of decimals of real variable
!     nxtsgn  true: next character is $- or &-sign
!     poinfn  true if period was found in series of &-signs
!     tildef  if true: a tilde has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     None.
! **********************************************************************
!
!                       I/O
!
!     ERERRL  writes to the default output device.
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None.
! **********************************************************************
!
!                       PSEUDO CODE
!
!     tilde_found := false
!     Search last non blank in line from error text file
!     While not at last character of line from error text file do
!        itype := 0
!        Take next character
!        Case next character of
!        -Character==~ : tilde_found := true
!        -Character==$ and tilde_found
!           Fill rest of line with spaces
!           itype := 1
!        -Character==& and tilde_found
!           Fill rest of line with spaces
!           itype := 2
!        -Character==% and tilde_found
!           Fill rest of line with spaces
!           itype := 3
!        -Character==$ : substitute with passed integer
!           tilde_found := false
!           Goto end of sequence of $-signs and count them
!           Take next passed integer
!           If there is one then
!              Determine available space and if it is not enough determine
!                  the required space
!              Make format descriptor for writing integer to line buffer
!              Write integer to line buffer
!              Set pointer to line buffer
!           Endif
!        -Character==& : substitute with passed real
!           tilde_found := false
!           Goto end of sequence of &-signs and count them
!           Take next passed real
!           If there is one then
!              Determine available space and if it is not enough determine
!                  the required space
!              If point was found then f-format
!                 Read number of &-signs after period
!                 Make format descriptor for writing real to line buffer
!              Else
!                 Make sure length is enough
!                 Make format descriptor for writing real to line buffer
!              Endif
!              Write real to line buffer
!              Set pointer to line buffer
!        -Character==% : substitute with passed character string
!           tilde_found := false
!           Goto end of sequence of %-signs and count them
!           Take next passed character string
!           If there is one then
!              Determine available space and if it is not enough determine
!                  the required space
!              Determine the effective string length
!                  (=length without trailing blanks).
!              Make format descriptor for writing character string
!                  to line buffer
!              Write character string to line buffer
!              Set pointer to line buffer
!        -Character==@ : hard return, fill with spaces
!           tilde_found := false
!        -other : ordinary character, copy it to line buffer
!           tilde_found := false
!        If a complete line is filled then
!           Search last blank on position 80 or before
!           Write line
!           Search first non-blank beyond found last blank
!           Shift not printed part of line to beginning
!           Reset remaining part of line
!           if ( tilde_found ) then
!              Print list of items depending on itype
!           tilde_found := false
!        Endif
!     Endwhile
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
!     --- use 80 columns only

      lnsize = 80

!     --- Move errlin one position to the left

      do i = 1, lnsize
         ih = i+1
         errlin(i:i) = errlin(ih:ih)
      end do
      errlin(81:81) = ' '

!     --- search last non blank in line from error text file

      iptsrc = 0
      iptend = len(errlin)
      hrdret = .false.
      tildef = .false.
100   if ( iptend>0 ) then
         if ( errlin(iptend:iptend)=='@' ) then
            hrdret = .true.
            iptend = iptend - 1
            goto 100
         else if ( errlin(iptend:iptend)==' ' ) then
            iptend = iptend - 1
            goto 100
         end if
      end if
      if ( hrdret ) then
         iptend = iptend + 1
         errlin(iptend:iptend) = '@'
      end if
      itype  = 0

!     --- while not at last character of line from error text file

110   if ( iptsrc<=iptend ) then

!     --- take next character

         iptsrc = iptsrc + 1
         ipttrg = ipttrg + 1

!        --- case next character of

         if ( errlin(iptsrc:iptsrc)=='~' ) then

!        --- tilde found

            tildef = .true.
            line(ipttrg:ipttrg) = errlin(iptsrc:iptsrc)

         else if ( tildef .and. errlin(iptsrc:iptsrc)=='$' ) then

!        --- ~$ found

            itype = 1
            line(ipttrg-1:160) = ' '
            ipttrg = lnsize+1

         else if ( tildef .and. errlin(iptsrc:iptsrc)=='&' ) then

!        --- ~& found

            itype = 2
            line(ipttrg-1:160) = ' '
            ipttrg = lnsize+1

         else if ( tildef .and. errlin(iptsrc:iptsrc)=='%' ) then

!        --- ~% found

            itype = 3
            line(ipttrg-1:160) = ' '
            ipttrg = lnsize+1

         else if ( errlin(iptsrc:iptsrc)=='$' ) then

!        --- character==$ : substitute with passed integer

            tildef = .false.
            istapo = iptsrc
            nxtsgn = .true.

!           --- goto end of sequence of $-signs

150         if (nxtsgn) then
               iptsrc = iptsrc + 1
               if (errlin(iptsrc:iptsrc)=='$') then
                  nxtsgn = .true.
               else
                  nxtsgn = .false.
               end if
               goto 150
            end if

!           --- take next passed integer

            iptsrc = iptsrc - 1
            iendpo = iptsrc
            numint = numint + 1
            if (numint<=nints) then

!           --- determine available space and if it is not enough determine
!               the required space

               length = iendpo - istapo + 1
               if ( ints(numint)>=0 ) then
                  if ( ints(numint)>=10.0**length ) then
                     length = 0
                     ifact  = 1
160                  length = length + 1
                     ifact  = ifact * 10
                     if ( iinfin / ifact>=10 ) then
                        if ( ints(numint)>=ifact ) goto 160
                     else
                        length = length + 1
                     end if
                  end if
               else
                  if ( abs(ints(numint))>=10.0**(length-1) ) then
                     length = 1
                     ifact  = -1
170                  length = length + 1
                     ifact  = ifact * 10
                     if ( iinfin / (-ifact)>=10 ) then
                        if ( ints(numint)<=ifact ) goto 170
                     else
                        length = length + 1
                     end if
                  end if
               end if

!              --- make format descriptor for writing integer to line buffer

               write (fmtint,'(''(i'',i4,'')'')') length

!              --- write integer to line buffer

               istapo = ipttrg
               iendpo = istapo+length-1
               write (line(istapo:iendpo),fmtint) ints(numint)

!              --- set pointer to line buffer

               ipttrg = ipttrg + length - 1
            end if

         else if (errlin(iptsrc:iptsrc)=='&' ) then

!        --- character==& : substitute with passed real

            tildef = .false.
            istapo = iptsrc
            nxtsgn = .true.
            poinfn = .false.

!           --- goto end of sequence of &-signs

200         if (nxtsgn) then
               iptsrc = iptsrc + 1
               if (errlin(iptsrc:iptsrc)=='&') then
                  nxtsgn = .true.
               else
                  if ( errlin(iptsrc:iptsrc)=='.' ) then
                     if ( iptsrc<iptend ) then
                        ih = iptsrc+1
                        if ( errlin(ih:ih)=='&' ) then
                           poinfn = .true.
                        end if
                     end if
                  end if
                  nxtsgn = .false.
               end if
               goto 200
            end if

!           --- take next passed real

            iptsrc = iptsrc - 1
            iendpo = iptsrc
            numrea = numrea + 1
            if (numrea<=nreal) then

!           --- determine available space and if it is not enough determine
!               the required space

               length = iendpo - istapo + 1

!              --- if point was found then f-format

               if ( poinfn ) then
                  iptsrc = iptsrc + 1

!                 --- read number of &-signs after period

                  nxtsgn = .true.
210               if ( nxtsgn ) then
                     iptsrc = iptsrc + 1
                     if ( errlin(iptsrc:iptsrc)/='&' ) then
                        nxtsgn = .false.
                     end if
                     goto 210
                  end if

!                 --- make format descriptor for writing real to line buffer

                  numdec = length
                  iptsrc = iptsrc - 1
                  length = iptsrc - iendpo - 1
                  write ( fmtrea, '(''(f'',i3,''.'',i3,'')'')' ) &
                        numdec+length+1, length
                  length = numdec+length+1

               else

!              --- make sure length is enough

                  if ( length==1 ) length = 10
                  length = max ( 7, length )+1

!                 --- make format descriptor for writing real to line buffer

                  numdec = length - 7
                  if ( reals(numrea)<0 ) length = length + 1
                  write (fmtrea,'(''(e'',i4,''.'',i3,'')'')') &
                         length, numdec
               end if

!              --- write real to line buffer

               istapo = ipttrg
               iendpo = istapo+length-1
               write (line(istapo:iendpo),fmtrea) reals(numrea)

!              --- set pointer to line buffer

               ipttrg = ipttrg + length - 1
            end if

         else if (errlin(iptsrc:iptsrc)=='%' ) then

!        --- character==% : substitute with passed character string

            tildef = .false.
            istapo = iptsrc
            nxtsgn=.true.

!           --- goto end of sequence of %-signs

250         if (nxtsgn) then
               iptsrc = iptsrc + 1
               if (errlin(iptsrc:iptsrc)=='%') then
                  nxtsgn = .true.
               else
                  nxtsgn = .false.
               end if
               goto 250
            end if

!           --- take next passed character string

            iptsrc = iptsrc - 1
            iendpo = iptsrc
            numcha = numcha + 1
            if (numcha<=nchar) then

!           --- determine available space and if it is not enough determine
!               the required space

               length = iendpo - istapo + 1

!              --- determine the effective string length
!                  (= length without trailing blanks).

               leneff = len(chars(numcha))
270            if ( leneff>0 ) then
                  if ( chars(numcha)(leneff:leneff)==' ' ) then
                     leneff = leneff - 1
                     goto 270
                  end if
               end if
               if (length<leneff) then
                  length = leneff
                  iendpo = istapo+length-1
               end if

!              --- make format descriptor for writing character string
!                  to line buffer

               write (fmtcha,'(''(a'',i4,'')'')') max(1,leneff)

!              --- write character string to line buffer

               istapo = ipttrg
               iendpo = istapo+length-1
               write (line(istapo:iendpo),fmtcha) chars(numcha)

!              --- set pointer to line buffer

               ipttrg = ipttrg + length - 1

            end if

         else if (errlin(iptsrc:iptsrc)=='@' ) then

!        --- character==@ : hard return, fill with spaces

            tildef = .false.
            line(ipttrg:160) = ' '
            ipttrg = lnsize

         else

!        --- other : ordinary character, copy it to line buffer

            tildef = .false.
            line(ipttrg:ipttrg) = errlin(iptsrc:iptsrc)

         end if

!        --- if one line is filled then

         if ( ipttrg>lnsize ) then

!        --- search last blank on position 80 or before

            ilast = lnsize+1
300         if ( line(ilast:ilast)/=' ' ) then
               ilast = ilast - 1
               if ( ilast>0 ) goto 300
            end if

!           --- search last non-blank before last blank

310         if ( line(ilast:ilast)==' ' ) then
               ilast = ilast - 1
               if ( ilast>0 ) goto 310
            end if
            if ( ilast==0 ) ilast=lnsize

!           --- write line ( if not blank )

            if ( line(1:ilast)/=' ' ) then
               if ( icarcn==0 ) then
                  write (fmtout,'(''(a'',i2,'')'')') max(1,ilast)
                  write (irefwr,fmtout) line(1:ilast)
               else
                  write (fmtout,'(''(1x,a'',i2,'')'')') max(1,ilast)
                  write (irefwr,fmtout) line(1:ilast)
               end if
            else
               ifpt = 1
320            if ( ( errlin(ifpt:ifpt)==' ' ) .and. &
                    ( ifpt<=iptend ) ) then
                  ifpt = ifpt + 1
                  goto 320
               end if
               ilpt = iptend
               if ( errlin(ifpt:ilpt)=='@' ) write ( irefwr, * )
            end if

!           --- search first non-blank beyond found last blank

            ilast = ilast + 1
330         if ( line(ilast:ilast)==' ' ) then
               ilast = ilast + 1
               if ( ilast<=ipttrg ) goto 330
            end if

!           --- shift not printed part of line to beginning

            do i = ilast, ipttrg
               ih = i-ilast+1
               line(ih:ih) = line(i:i)
            end do

!           --- reset remaining part of line

            ipttrg = ipttrg - ilast + 1
            line(ipttrg+1:160) = ' '
            if ( tildef ) then

!           --- tilde found, print list depending on itype

               if ( itype==1 ) then
                  write(irefwr,400) (ints(i),i=numint+1,nints)
400               format(6x,4i8,4x,4i8)
                  numint = nints
               else if ( itype==2 ) then
                  write(irefwr,410) (reals(i),i=numrea+1,nreal)
410               format((6x,ss,5(1pe14.6e2)))
                  numrea = nreal
               else if ( itype==3 ) then
                  write(irefwr,420) (chars(i),i=numcha+1,nchar)
420               format(6x,a)
                  numcha = nchar
               end if
            end if
            if ( ipttrg==0 ) then
               tildef = .false.
            else if ( line(ipttrg:ipttrg)/='~' ) then
               tildef = .false.
            end if

         end if

!        --- endwhile

         goto 110

      end if

      if ( line==' ' ) ipttrg = 0

      end
