      subroutine erfillnodobst ( kmeshm, iobst, icurvsobs, kmesh, nodes, &
                                 nnodes, iwork )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 05-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for obstacle problems (curves only)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     obstacle
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm(*), iobst, icurvsobs(2,*), kmesh(*), nodes(*), &
              nnodes, iwork(*)

!     icurvsobs      i    Array containing information of the active obstacles.
!                         The array corresponds to all curves separately
!                         icurvsobs(1,i) = 1 means that the corresponding point
!                         is active; 0 means not active
!     iobst          i    Obstacle sequence number
!     iwork         i/o   integer work array of length npoint
!                         iwork(i) = 0, means node i has not yet been filled
!                         iwork(i)>0, means node i has been filled
!                         The value defines the new node number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     nnodes        i/o   Number of nodes
!     nodes          o    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, icurnr, ncurvs, i1, i2, k, node
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icurnr         Curve sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     k              Counting variable
!     ncurvs         Number of curves in mesh
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erfillnodobst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillnodobst'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      ncurvs = kmesh(11)

!     --- Fill curves

      istar3 = kmeshm(2)-1
      do icurnr = 1, ncurvs
         i1 = kmeshm(icurnr+4)
         i2 = kmeshm(icurnr+5)-1

         do k = i1, i2-1
            if ( icurvsobs(1,k)==iobst ) then

!           --- Obstacle point found

               node = kmeshm(istar3+k-1)
               if ( iwork(node)==0 ) then

!              --- Node is new

                  iwork(node) = 1
                  nnodes    = nnodes + 1
                  nodes(nnodes) = node

               end if
            end if
         end do

      end do

1000  call erclos ( 'erfillnodobst' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillnodobst'

      end if  ! ( debug )

      end
