      subroutine erfillstrtkmc ( iwork, kmesh )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.1  date 07-05-2001 Correction  for inpelm not constant
!        version  1.0  date 02-01-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array iwork with number of nodes per element as well as starting
!    addresses in array kmeshc
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(2,*), kmesh(*)

!     iwork          o    integer work array of size 2 x nelem
!                         iwork2 is filled in the following way:
!                         iwork2(1,ielem) contains number of nodes of element
!                         iwork2(2,ielem) contains number of starting position
!                                         of element ielem in array kmeshc
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, nelgrp, ndim, inelem, kelma, kelmf, kelmg, i, &
              irelem, istart, ielem, ishape, ielgrp

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     ishape         Shape number of elements
!     istart         First position used at a part
!     kelma          Indicates if KMESH part a has been filled (>0)
!                    or not (0)
!                    If kelma>0, kelma is the starting address of KMESH
!                    part a in KMESH
!     kelmf          Indicates if KMESH part f has been filled (>0) or
!                    not (0)
!                    If kelmf>0, kelmf is the starting address of KMESH
!                    part f in KMESH
!     kelmg          Indicates if KMESH part g has been filled (>0)
!                    or not (0)
!                    If kelmg>0, kelmg is the starting address of KMESH
!                    part g in KMESH
!     ndim           Dimension of the space.
!     nelgrp         Number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      inpelm = kmesh(4)
      nelgrp = kmesh(5)
      ndim   = kmesh(6)
      inelem = kmesh(9)
      kelma  = kmesh(15)
      kelmf  = kmesh(20)
      kelmg  = kmesh(21)
      do i = 1, inelem
         iwork(1,i) = 0
         iwork(2,i) = 0
      end do

      istart = 1
      ielem = 0
      do ielgrp = 1, nelgrp
         ishape = kmesh(kelmf+ielgrp-1)
         if ( kelma.gt.0 ) &
            inpelm = kmesh(kelma+ielgrp)-kmesh(kelma+ielgrp-1)
         if ( kelmg.gt.0 ) inelem = kmesh(kelmg+ielgrp-1)
         if ( ndim.eq.1 .or. &
              ndim.eq.2 .and. ishape.gt.2 .or. &
              ndim.eq.3 .and. ishape.gt.10 ) then

!        --- elements of highest dimension, Fill iwork

            do irelem = 1, inelem
               iwork(1,irelem+ielem) = inpelm
               iwork(2,irelem+ielem) = istart
               istart = istart+inpelm
            end do

         else

!        --- elements of lower dimension, raise istart

            istart = istart+inpelm*inelem

         end if
         ielem = ielem+inelem

      end do

      end
