      subroutine ermultmatcons ( ibuffr, buffer, matr, const, intmat )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.2  date 29-04-2014 Correction jmetod
!        version  1.1  date 09-11-2012 Long length ibuffr
!        version  1.0  date 10-09-2004
!
!   copyright (c) 2004-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Multiply a (diagonal) matrix by a constant
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     multiplication
!     constant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*)
      integer matr(*), intmat(*)
      double precision buffer(*), const

!     buffer        i/o   Buffer array
!     const          i    Multiplication constant
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     intmat         i    Standard SEPRAN array of length 5 containing
!                         information concerning the structure of the matrix.
!                         See the Programmer's Guide 13.5 for a
!                         complete description.
!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipvector
      integer jmetod, iprob, nusol
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iprob          Actual problem number
!     ipvector       Starting address of array vector
!     jmetod         Indicates type of storage scheme for the matrix
!     nusol          Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext
      integer inidln

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIDLN         Compute length of array in BUFFER
!     TOVECMULCONS   Multiply vector by a constant
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     309   The arrays MATR1 and MATR2 correspond to different storage methods
!    1296   Complex matrices have not yet been implemented in this subroutine
!    2686   MATR is not of type matrix
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'ermultmatcons' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ermultmatcons'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( matr(2)==110 ) then

!     --- Matrix is diagonal matrix, simple case

         if ( mod(matr(3),10)/=0 ) then

!        --- Diagonal matrix is complex, not yet implemented

            call errsub ( 1296, 0, 0, 0 )
            go to 1000

         end if  ! ( mod(matr(3),10)/=0 )

!        --- Activate array

         call ini060text ( ibuffr, matr(1), 'matrix_1' )

!        --- Compute starting address

         ipvector = inidgttext ( matr(1), 'matrix_1' )

!        --- Apply multiplication

         nusol = matr(5)
         call tovecmulcons ( const, buffer(ipvector), nusol )

      else if ( matr(2)==103 ) then

!     --- Matrix is large matrix

         jmetod = mod(intmat(1),1000)
         iprob  = intmat(1) / 1000 + 1
         if ( mod(matr(1),1000)/=jmetod ) then
            call errint ( mod(matr(1),1000), 1 )
            call errint ( matr(1)/1000+1, 2 )
            call errint ( jmetod, 3 )
            call errint ( iprob, 4 )
            call errsub ( 309, 4, 0, 0 )
            go to 1000
         end if

!        --- First the internal matrix
!            Activate array

         call ini060text ( ibuffr, matr(3), 'matrix_3' )

!        --- Compute starting address

         ipvector = inidgttext ( matr(3), 'matrix_3' )

!        --- Apply multiplication

         nusol = inidln(matr(3))
         call tovecmulcons ( const, buffer(ipvector), nusol )

!        --- Next the boundary part

         if ( matr(5)>0 ) then

!        --- Activate array

            call ini060text ( ibuffr, matr(5), 'matrix_5' )

!           --- Compute starting address

            ipvector = inidgttext ( matr(5), 'matrix_5' )

!           --- Apply multiplication

            nusol = inidln(matr(5))
            call tovecmulcons ( const, buffer(ipvector), nusol )

         end if  ! ( matr(5)>0 )

      else

!     --- Array is not of type matrix

         call errint ( matr(2), 1 )
         call errsub ( 2686, 1, 0, 0 )
         go to 1000

      end if  ! ( matr(2)==110 )

1000  call erclos ( 'ermultmatcons' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ermultmatcons'

      end if

      end
