      subroutine ermultveccons ( ibuffr, buffer, ivectr, const )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.1  date 09-11-2012 Long length ibuffr
!        version  1.0  date 24-09-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Multiply a SEPRAN vector by a constant
!
! **********************************************************************
!
!                       KEYWORDS
!
!     multiplication
!     vector
!     constant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*)
      integer ivectr(*)
      double precision buffer(*), const

!     buffer        i/o   Buffer array
!     const          i    Multiplication constant
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     ivectr         i    standard sepran array denoting USOL or vector of
!                         special structure defined in nodal points
!                         This vector is multiplied by const
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipusol
      integer itypvc, nusol
      logical debug, sepranvector

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipusol         Starting address of solution array in BUFFER
!     itypvc         Type of sepran vector
!     nusol          Total number of unknowns
!     sepranvector   if true the vector is a known seran vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     TOVECMULCONS   Multiply vector by a constant
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check array
!     Activate array
!     Compute starting address
!     Actual multiplication
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'ermultveccons' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ermultveccons'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check array

      itypvc = ivectr(2)
      sepranvector = itypvc==110 .or. itypvc==115 .or. &
                     itypvc==116 .or. itypvc==126 .or. &
                     itypvc==129
      if ( .not.sepranvector ) then

!     --- Not a known sepran vector

         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )
         go to 1000

      end if  ! ( .not.sepranvector )

!     --- Activate array

      call ini060text ( ibuffr, ivectr(1), 'ivectr' )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting address

      ipusol = inidgttext ( ivectr(1), 'ivectr' )
      if ( ierror/=0 ) go to 1000

!     --- Actual multiplication

      nusol = ivectr(5)
      call tovecmulcons ( const, buffer(ipusol), nusol )

1000  call erclos ( 'ermultveccons' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ermultveccons'

      end if

      end
