      subroutine ersurfsnod ( kmesh, kmeshn, nsurfaces, nodes, &
                              nnodes, isurfaces, kmesht, kmeshc )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  3.0  date 03-01-2012 Remove iwork
!        version  2.0  date 28-12-2010 extra parameters
!        version  1.0  date 11-02-2005
!
!   copyright (c) 2005-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all nodes in the surfaces stored in array surfaces
!     in array nodes
!     The number of nodes in nodes is stored in nnodes
!     At this moment the surfaces must be part of kmeshn
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kmeshn(*), nsurfaces, nodes(*), &
              nnodes, isurfaces(*), kmesht(*), kmeshc(*)

!     isurfaces      i    Array containing the surfaces from which the nodes
!                         must be found
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     kmesht         i    kmesh part t, contains element numbers of elements
!                         with highest dimensionality
!     nnodes         o    Number of nodes in nodes
!     nodes          o    Contains the node numbers that must be filled
!     nsurfaces      i    Number of surfaces in array isurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSURFSNOD1    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!     ERSURFSNOD2    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array iwork
!    nnodes := 0
!    For all surfaces isurnr in isurfaces do
!       inelem := number of elements in isurnr
!       inpelm := number of nodes in elements in isurnr
!       istart := starting address of surface in kmeshc
!       For all nodes inode in surface do
!          if ( iwork(inode)=0 ) then
!             iwork(inode):=1
!             nnodes := nnodes+1
!             nodes(nnodes) := inode
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'ersurfsnod' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ersurfsnod'
         call prinin ( isurfaces, nsurfaces, 'isurfaces' )

      end if
      if ( ierror/=0 ) go to 1000

      nnodes = 0

      if ( kmesh(28)>0 ) then

!     --- kmeshn has been filled

         call ersurfsnod1 ( nsurfaces, isurfaces, kmeshn, kmesh, &
                            nodes, nnodes )

      else

!     --- kmeshn has not been filled

         call ersurfsnod2 ( nsurfaces, isurfaces, kmesht, kmesh, &
                            nodes, nnodes, kmeshc )

      end if  ! ( kmesh(28)>0 )

1000  call erclos ( 'ersurfsnod' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ersurfsnod'

      end if

      end
