      subroutine ertext ( mesnum, nints, nreal, nchar )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.8  date 06-10-2006 Remove stop
!        version  1.7  date 19-01-1999 include commons
!        version  1.6  date 30-04-1995 Improved layout
!        version  1.5  date 26-12-1994 New commons cmessc and cmessg
!        version  1.4  date 24-12-1992 Set @ in column 80
!        version  1.3  date 16-12-1992 New call of ererrl
!        version  1.2  date 17-10-1992 Removal of readi4
!        version  1.1  date 23-06-1992 New common cmessg
!        version  1.0  date 18-06-1991
!
!   copyright (c) 1991-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     ertext prints a message from the menu message file, but without
!     extra texts. In this way it is fit for interactive input
! **********************************************************************
!
!                       KEYWORDS
!
!     message_manipulation
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmesscf90'
      include 'SPcommon/cmessgf90'
      include 'SPcommon/cmachnf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mesnum, nints, nreal, nchar

!     mesnum   i    Error number.
!     nchar    i    Number of character strings.
!     nints    i    Number of integers.
!     nreal    i    Number of reals.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer      i, ifrom, istart(21), ito ,maxerr, ipttrg, &
                   linenr, numint, numlin, numrea, &
                   numcha, iref, idum
      parameter    ( iref=17)
      logical      nxtlin, exist
      character*80 errlin
      character * 160 line

!     errlin   Contains one line of the message
!     exist    Flag: true= error number exists in error text file.
!     i        loop variable
!     idum     Integer dummy parameter
!     ifrom    information in array istart is read from position ifrom
!     ipttrg  pointer to first position not filled in line buffer
!     iref     Reference to error message file
!     istart   integer array containing a part of the starting records
!              of the messages.
!     ito      information in array istart is read to position ito.
!     line    line buffer
!     linenr   record number of message line
!     maxerr   1 + maximum number of messages that are stored
!              in the message file.
!     numcha   number of character strings in a message
!     numint   number of integers in a message
!     numlin   number of lines of a message
!     numrea   number of reals in a message
!     nxtlin   true: scan next line, false :don't scan next line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERERRL   Scan message line
!     READFL   Read from a SEPRAN direct access file
! **********************************************************************
!
!                       I/O
!
!     ertext writes to the default output device.
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     read maximum message number from error text file ;
!     if (message number correct) then
!        read starting record number ;
!        calculate number of lines ;
!        if ( number of lines>0 ) then
!           for each line do
!              read line ;
!              process and print line ;
!        else
!           print warning : message does not exist ;
!     else
!        print warning : message number out of range ;
!     if (number of integers, reals and/or strings not correct) then
!        print warning ;
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      exist = .true.

!     --- read largest message number permitted

      ifrom = 1
      ito   = 1
      linenr= 1
      idum  = 0

      call readfl ( linenr, ifrom, ito, idum, iref, istart )
      if ( ierror/=0 ) go to 1000
      maxerr = istart(1)

      if ( mesnum>0 .and. mesnum<=maxerr ) then

!     --- Read starting record of message

         linenr = 2 + (mesnum-1)/nrec4
         ito    = mod(mesnum,nrec4)+1
         if (ito==1) ito = nrec4+1
         call readfl (linenr, ifrom, ito, idum, iref, istart)
         linenr = istart(ito-1)
         numlin = istart(ito) - linenr

         if ( numlin>0 ) then

!        --- Read and print message

            i=linenr
            numint=0
            numrea=0
            numcha=0
            nxtlin=.true.
            ipttrg=0
            line=' '

  100       if (nxtlin) then

!           --- read one message line

               read ( iref17, rec=i ) errlin
               errlin(80:80) = '@'

!              --- scan message line for integer fields ($), &
!                  real fields (&) and character fields (%), and
!                  write line to output device.

               call ererrl ( errlin,ints,nints,reals,nreal,chars, &
                             nchar,numint,numrea,numcha,ipttrg,line )

               i = i + 1
               nxtlin = i<=(linenr+numlin-1)
               goto 100
            end if
            if ( icarcn==0 ) then
               if ( ipttrg>0 ) write (irefwr,'(1x,a)') line(1:ipttrg)
            else
               if ( ipttrg>0 ) write (irefwr,'(2x,a)') line(1:ipttrg)
            end if

         else

!        --- Message not present in the file

            write (irefwr,1100) 'ERTEXT', mesnum
            write (irefwr,1200) (ints(i),i=1,nints)
            write (irefwr,1300) (reals(i),i=1,nreal)
            write (irefwr,1600) (chars(i),i=1,nchar)
            exist = .false.

         end if

      else

!     --- Message number out of range

         write (irefwr,1100) 'ERTEXT', mesnum
         write (irefwr,1200) (ints(i),i=1,nints)
         write (irefwr,1300) (reals(i),i=1,nreal)
         write (irefwr,1600) (chars(i),i=1,nchar)
         exist = .false.

      end if

!     --- warning if number of integers, reals, or strings not correct
!         (if error number exists)

      if (exist) then
         if(numint/=nints.or.numrea/=nreal.or.numcha/=nchar)then
            write (irefwr,'(//1x,78(''*'')/)')
            write (irefwr,1400) 'ERTEXT',mesnum,numint,nints,numrea, &
                                nreal,numcha,nchar
            write (irefwr,1200) (ints(i),i=1,nints)
            write (irefwr,1300) (reals(i),i=1,nreal)
            write (irefwr,1600) (chars(i),i=1,nchar)
            write (irefwr,'(/1x,78(''*'')//)')
         end if
      end if
 1100 format (' *** WARNING : '/ &
              ' ',a,' number ',i6,' has not yet been added', &
              ' to the error message file')
 1200 format (' *** Ints  : ',5i13)
 1300 format (' *** Reals : ',5e13.7)
 1400 format ( &
       ' *** INTERNAL ERROR : number of integers, reals,', &
       ' or strings in message '/ &
       ' not equal to number of integers, reals, or strings', &
       ' in program.'/ &
       ' ',a,' number : ',i5/ &
       ' number of integers in message :',i4, &
       ', in program :',i4/ &
       ' number of reals in message    :',i4, &
       ', in program :',i4/ &
       ' number of strings in message  :',i4, &
       ', in program :',i4)
 1600 format (' *** Strings : '/(1x,a78))

 1000 end
