      function getikelma ( kmesh, ielgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with ikelma as defined for kprobe, i.e. number of local nodes
!     accumulated of all element groups before ielgrp
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: kmesh(*), ielgrp
      integer :: getikelma

!     getikelma      o    Value of ikelma
!     ielgrp         i    element group number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: kelma, npelm
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     kelma          Indicates if KMESH part a has been filled (>0)
!                    or not (0)
!                    If kelma>0, kelma is the starting address of KMESH
!                    part a in KMESH
!     npelm          Number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getikelma' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getikelma'
         write(irefwr,1) 'ielgrp', ielgrp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npelm = kmesh(4)
      kelma = kmesh(15)

      if ( kelma==0 ) then
         getikelma = (ielgrp-1)*npelm
      else
         getikelma = kmesh(kelma+ielgrp-1)
      end if  ! ( kelma==0 )

1000  call erclos ( 'getikelma' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'getikelma', getikelma
         write(irefwr,*) 'End getikelma'

      end if  ! ( debug )

      end function getikelma
