      function getindvar ( varname, loopname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the integer in the block constants with name varname
!     If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) varname, loopname
      double precision getindvar

!     getindvar      o    Value of variable
!     loopname       i    Name of loop variable
!     varname        i    Name of variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnameindvar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
!     GETNAMEINDVAR  Get the sequence number corresponding to the scalar in
!                    the block variables with name varname
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2157   variable name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chscalars
!     If ( name in chscalars ) then
!        getindvar := value
!     else
!        error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iseq = getnameindvar ( varname, loopname )
      if ( iseq>0 ) then
         getindvar = scalars(iseq)
      else
         getindvar = 0
         call eropen ( 'getindvar' )
         call errchr ( varname, 1 )
         call errwar ( 2157, 0, 0, 1 )
         call erclos ( 'getindvar' )
      end if
      end
