      subroutine getnodeneighs ( ibuffr, kmesh, inode, neigh, numneigh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-11-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get all neighbors of node inode and store in array neigh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     neighbour
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), inode, neigh(*), numneigh

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inode          i    Node sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     neigh          o    contains neighbors of node
!     numneigh       o    number of neighbors of a point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmse
      integer mmkmeshe
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipkmse         Starting address of array KMESH part e
!     mmkmeshe       Memory management address of kmesh part e
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETELMSNODE1   Find all elements corresponding to nodal point inode
!     INIACTMK       General memory management subroutine to activate or create
!                    a permanent array
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getnodeneighs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getnodeneighs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill or activate array kmesh part e

      call iniactmk ( ibuffr, 1, 19, kmesh, kmesh, mmkmeshe )

!     --- Get neighbors

      ipkmse = inigettext ( mmkmeshe, 'kmeshe' )
      call getelmsnode1 ( ibuffr(ipkmse), inode, neigh, numneigh, &
                          kmesh(8) )

1000  call erclos ( 'getnodeneighs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getnodeneighs'

      end if  ! ( debug )

      end
