/*
 *  Copyright (c) 1991  Philips Export B.V.
 * 
 *  Philips Research Laboratories
 *  Eindhoven, The Netherlands
 *
 *  Author    : G.F.M. Haenen
 *              F.W.G.M. Wouters
 *
 *  Date      : July, 1991
 *
 *  Modified  : Kees Lemmens, June 9, 1999
 *            - fixed compiler warnings and some small bugs
 *
 *  Function  : Translates plotting primitives to HPGL.
 *
 *  Comment   : These HPGL commands are first written into buffers,
 *              (one buffer for each pen-color) before writing them to
 *              a file. Writing to file is done color for color.
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/file.h>
#include <time.h>
#include <unistd.h>
#include "plotfun.h"

#define CHUNK    4096
#define P1X      100
#define P1Y      100
#define PSIZE    7200
#define XOFFSET  100.0
#define YOFFSET  100.0
#define PENS     7

static FILE   *fp;
static struct string_t cmd[PENS];
static char   str[MAXCOMMAND];
static int    curpen;
static window_t *usr;

static void cpycmd(str)
char *str;
{
    int l = strlen(str);

    if (cmd[curpen].alloc - cmd[curpen].length <= l)
    {
        cmd[curpen].alloc += CHUNK;
        cmd[curpen].str = (char *)realloc(cmd[curpen].str, cmd[curpen].alloc);
    }

    memcpy(&cmd[curpen].str[cmd[curpen].length], str, l+1);
    cmd[curpen].length += l;
}

void hpgl_header(char *hpglfile, window_t *dim)
{
    time_t timeloc;
    float step, margin2;

    margin2 = ((dim->xmax-dim->xmin) / 4.0) + dim->xmin;
    step = (dim->ymax-dim->ymin) / 30.0;
    (void) time(&timeloc);

    fprintf(fp, "%cI81;;17:%cN;19:\n", 0x1b2e, 0x1b2e);
    fprintf(fp, "DF;PW0;RO90;IP%d,%d,%d,%d;", P1X, P1Y, P1X+PSIZE, P1Y+(int)(1.5*PSIZE));
    fprintf(fp, "IW%d,%d,%d,%d;", P1X, P1Y, P1X+PSIZE, P1Y+(int)(1.5*PSIZE));
    fprintf(fp, "SC%.1f,%.1f,%.1f,%.1f;\n",dim->xmin, dim->xmax, dim->ymin, dim->ymax*1.5);
    /* This is obviously wrong and so I changed it into something that
     * at any rate satisfied the compiler (KL) :
     * fprintf(fp, "LT;SP1;PA;\n", 0.0, usr.width, 0.0, usr.height);
    */
    fprintf(fp, "LT;SP1;PA;\n");
    fprintf(fp, "DT%c;\n",03);
    fprintf(fp, "SI0.5,0.6;PU%.1f,%.1f;LBSEPRAN%c;SI;\n",dim->xmin*1.2, dim->ymax+13.0*step,03);
    fprintf(fp, "PU%.1f,%.1f;LBUserid%c;\n",dim->xmin*1.2, dim->ymax+11.0*step,03);
    fprintf(fp, "PU%.1f,%.1f;LBFilename%c;\n",dim->xmin*1.2, dim->ymax+10.0*step,03);
    fprintf(fp, "PU%.1f,%.1f;LBDate/Time%c;\n",dim->xmin*1.2, dim->ymax+9.0*step,03);
    fprintf(fp, "PU%.1f,%.1f;LB: %s%c;\n",margin2, dim->ymax+11.0*step,
                 getenv("USER"),03);
    fprintf(fp, "PU%.1f,%.1f;LB: %s%c;\n",margin2, dim->ymax+10.0*step,
                 hpglfile,03);
    fprintf(fp, "PU%.1f,%.1f;LB: %s%c;\n",margin2, dim->ymax+9.0*step,
                 ctime(&timeloc),03);
    fprintf(fp, "PU%.1f,%.1f;PD%.1f,%.1f;\n",0.0,dim->ymax+7.0*step,
                                        dim->xmax,dim->ymax+7.0*step);
}

static void hpgl_trailer()
{
    fprintf(fp, "SP0;\n");
    fclose(fp);
    infoMessage("");
}

void hpgl_finish()
{
    int i;

    for (i = 0; i < PENS; i++)
        if (cmd[i].length)
            fprintf(fp, "SP%d;\n%s", i + 1, cmd[i].str);

    hpgl_trailer();
}

int hpgl_init(window_t *dim,filnam_t *fileinfo)
{
   int  number = 0;
   char string[80], hpglfile[80];
   int  i;
   
   for (i = 0; i < PENS; i++)
   {
      cmd[i].alloc = cmd[i].length = 0;
      cmd[i].str   = (char *) malloc((size_t) CHUNK);
   }
   
   if(strlen(fileinfo->outname) == 0)
   {
      do
	sprintf(hpglfile, "%s%03d_%02d.hpgl", fileinfo->name,
		fileinfo->version, ++number);
      while (access(hpglfile, R_OK) == F_OK && number < 100);
   }
   else
     strncpy(hpglfile,fileinfo->outname,79);
   
   if ((fp = fopen(hpglfile, "w")) == NULL)
   {
      sprintf(string, "Unable to open hpgl-file '%s'", hpglfile);
      errorMessage(string);
      return(FALSE);
   }
   sprintf(string, "Creating hpgl file: %s...", hpglfile);
   infoMessage(string);
   
   hpgl_header(hpglfile,dim);

   usr = dim; // remember it for clipping ! 
   return(TRUE);
}

void hpgl_line(x1, y1, x2, y2)
double x1, y1, x2, y2;
{
    if (clip_line(&x1, &y1, &x2, &y2, usr))
    {
        sprintf(str, "PU%.2f,%.2f;PD%.2f,%.2f;\n", x1, y1, x2, y2);
        cpycmd(str);
    }
}

void hpgl_color(iclr)
int iclr;
{
    curpen = iclr % PENS;     
}
        
void hpgl_polyline(x, y, n)
double x[], y[];
int n;
{
    int i;
 
    if (clip_polygon(x, y, &n, usr))
    {
        sprintf(str, "PU%.2f,%.2f;PM0;PD%.2f,%.2f", x[0], y[0], x[1], y[1]);
        cpycmd(str);
        for (i = 2; i < n; i++)
        {
            sprintf(str, ",%.2f,%.2f", x[i], y[i]);
            cpycmd(str);
        }
        sprintf(str, ";PM2;EP;\n");
        cpycmd(str);
    }
}
 
void hpgl_polyfill(x, y, n)
double x[], y[];
int    n;
{
    int i;
 
    if (clip_polygon(x, y, &n, usr))
    {
        sprintf(str, "PU%.2f,%.2f;PM0;PD%.2f,%.2f", x[0], y[0], x[1], y[1]);
        cpycmd(str);
        for (i = 2; i < n; i++)
        {
            sprintf(str, ",%.2f,%.2f", x[i], y[i]);
            cpycmd(str);
        }
        sprintf(str, ";PM2;FP;\n");
        cpycmd(str);
    }
}
 
void hpgl_text(x, y, height, angle, string)
double x, y, height, angle;
char   *string;
{
     line_text(x, y, height, angle, string);
}

void hpgl_symbol(x, y, num)
double x, y;
int    num;
{
     line_symbol(x, y, num);
}
