/*
 *   Copyright (c) 1999 Kees Lemmens, TU Delft
 * 
 * 
 * Date     : June , 1999
 * 
 * Function : Wrapper functions around the basic input routines.
 * The standard input routines should be callable from Fortran as well
 * and thus have a nasty syntax. It also provides a uniform set of
 * (C-style) functions no matter what the plotformat of the input is.
 *
 * Determination of the input type is done in the openfile() function,
 * after which control is passed to the appropriate file handling routines :
 * either CRAY, native Sepran-Fortran, Fortran emulation (fortranbin-io.c)
 * or Java binary format.
 */

#define FORTRAN  1
#define JAVABIN  2

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "plotfun.h"
#include "readfun.h"

/* global variables */

// None !!!!!!!!!!!!!!!!

int closeinput(int type)
{
   static int ios;
   switch (type)
   {
    case FORTRAN : plrdcl  (&ios); break;
    case JAVABIN : jplrdcl (&ios); break;
   }   
   return ios;
}

int openinput(char *name, filnam_t *info)
{
   static int ios;
   FILE *in;
   int  fillen;
   int firstbyte;
   
   /* determine the input type if not yet set: */

   if(info->intype == 0)
   {
      if((in = fopen(name,"r")) == NULL)
	return -1;
   
      fread(&firstbyte,sizeof(int),1,in);
      
      switch(firstbyte)
      {
       case 0x00000018 : /* Fortran Binary on native platform */
	 fprintf(stderr,
		 "Fortran binary detected on native platform.\n");
	 info->intype = FORTRAN;
	 break;
       case 0x20202031 : /* ASCII (either little or big endian) */
       case 0x31202020 :
	 fprintf(stderr,
		 "ASCII input format detected.\n");
	 info->intype = FORTRAN;
	 break;
       case 0x626a0700 :
	 info->intype = JAVABIN;
	 fprintf(stderr,
		 "Java portable binary detected on little-endian platform.\n");
	 break;
       case 0x00076a62 :
	 fprintf(stderr,
		 "Java portable binary detected on big-endian platform.\n");
	 info->intype = JAVABIN;
	 break;
       default:
	 fprintf(stderr,"Input format not recognized\n");
	 exit(1);
      }
      fclose(in);
   }
   
   /* And reopen the file in the standard old-fashioned Fortran way : */
   fillen = strlen(name);
   switch (info->intype)
   {
    case FORTRAN:
      plrdop (name, &fillen, &ios);
        break;
    case JAVABIN:
       jplrdop (name, &fillen, &ios);
      break;
    default:
      ios = -1;
   }
   return ios;
}

int readrecord(seprec_t *s, int type)
{
   static int ios;
   
   switch (type)
   {
    case FORTRAN:
      plrd01 (&s->command, &s->x, &s->y, &s->a, &s->h, &s->nchar, &ios);
      break;
    case JAVABIN:
      jplrd01(&s->command, &s->x, &s->y, &s->a, &s->h, &s->nchar, &ios);
      break;
    default:
      ios = -1;
   }
   return ios;
}

int readstring(int len, char *string, int type)
{   
   static int ios;
   
   switch (type)
   {
    case FORTRAN:
      plrd06 (&len, string, &ios);
      break;
    case JAVABIN:
      jplrd06(&len, string, &ios);
      break;
    default:
      ios = -1;
   }   
   return ios;
}

int readpolygon(int nr, float polyg[], int type)
{
   static int ios;
   
   switch (type)
   {
    case FORTRAN:
      plrd07 (&nr, polyg, &ios);
      break;
    case JAVABIN:
      jplrd07(&nr, polyg, &ios);
      break;
    default:
      ios = -1;
   }
   return ios;
}

