      subroutine ini081ext ( ibuffr, mmindex, length, name )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 28-09-2012 Jump in case ierror#0
!        version  1.0  date 31-03-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new sepran integer array
!     Clear this array
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     clear
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), mmindex, length
      character (len=*) name

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     length         i    Help parameter to indicate the length of arrays
!                         In this case it contains the length of the vector in
!                         integers
!     mmindex        i    memory management sequence number of array
!     name           i    Name of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipindex
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipindex        Starting address of array with memory management sequence
!                    number mmindex
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI007         clear integer space
!     INI081         Create permanent space in array IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create array
!     Compute starting address
!     Clear array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini081ext' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini081ext'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Create array

      call ini081 ( ibuffr, mmindex, length, name )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting address

      ipindex = inigettext ( mmindex, name )

!     --- Clear array

      call ini007 ( ibuffr(ipindex), length )

1000  call erclos ( 'ini081ext' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ini081ext'

      end if

      end
