      subroutine ini083 ( ibuffr, index, length, ipcopy, namear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.10 date 06-04-2013 Correction integers
!        version  1.9  date 21-11-2012 Long integers
!        version  1.8  date 28-09-2012 Correction largest integer
!
!   copyright (c) 1997-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute starting position of a new or already existing array in buffer
!     array ibuffr
!     The array has already been constructed in IBUFFR from the temporary
!     position IPCOPY, which implies that there is already space enough
!     in IBUFFR.
!     Furthermore the array must be copied from the positions IPCOPY to
!     the new positions.
!     INI083 replaces the old subroutine INI001 (for new arrays), combined
!     with INI000.
!     If the array did not exist, or the old array took too few positions to
!     keep the new one, then space in array IBUFFR is reserved
!     if no space is visible in array IBUFFR, INI083 starts the new array from
!     position IPCOPY.
!     INI083 is identical to subroutine INI053 as described in the Programmer's
!     Guide, except that the parameter name is not longer used.
!     The name of the calling subroutine is extracted from common blocks
!     CMESSC and CMESSG, so it is supposed that the subroutine EROPEN has been
!     used
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cnamtbf90'
      include 'SPcommon/cmessgf90'
      include 'SPcommon/cmesscf90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/carbacf90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) ipcopy
      integer index, length, ibuffr(*)
      character *(*) namear

!     ibuffr i/o Integer buffer array in which all large data is stored
!     index  i/o The value of index is the reference number of the
!                array of which the starting position must be computed with
!                respect to array INFOR in common block IINFOR
!                This means that the positions (i,INDEX) (i=1,2,3) in INFOR
!                contain all the information about the storage of the array
!     ipcopy  i  Temporary address in IBUFFR, from which the contents of the
!                array have been stored
!     length  i  length of the array in words
!     namear  i  Name of array
!                This name is used in the table if the array did not yet exist
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) indfor, ipstar, j
      integer lengol, i, iextra, lenchr
      logical newarr

!     i         General loop variable
!     iextra    Extra space needed at both sides of the array for control
!               integers
!     indfor    Help parameter to store the record number from which the
!               array has been stored in file 3
!     ipstar    computed starting position in array IBUFFR
!     j         General help variable
!     lenchr    Number of characters in name of array
!     lengol    Length of the preceding array corresponding to index
!     newarr    If true, the array is new
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRINTLONG     Put long integer in error message
!     ERRSUB         Error messages
!     INICHK1        Check IBUFFR
!     PRINBF         Print and check length of array IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     760   iinfor too large
!    1765   length of array less than one
!    3116   Memory needed exceeds largest integer
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( idebug>0 ) then
!        Set iextra = extra space needed
!        check all control integers
!     If ( Array existed before and array in IBUFFR and
!                                 old length<=new length ) then
!         Compute IPSTAR
!         Copy array
!     else if ( enough space in IBUFFR after IBFREE ) then
!          IPSTAR = IBFREE
!          Raise IBFREE
!          Copy array
!     else
!          Set IPSTAR = ICOPY
!     If ( array did not exist before ) then
!         Raise IINFOR
!         INDEX = IINFOR
!     Fill array INFOR
!     Set INBACK ( INDEX ) negative if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini083' )
      if ( ierror/=0 ) go to 1000

      if ( idebug==0 ) then
         iextra = 0
      else
         iextra = intlen
         if ( ifirws==0 ) then
            ifirws = 1
         else
            ifirws = 2
         end if
         call inichk1(ibuffr)
      end if
      if ( length<=0 ) then

!     --- length<=0, error

         call errint ( length, 1 )
         call errchr ( namear, 1 )
         call errsub ( 1765, 1, 0, 1 )

      end if
      if ( ierror/=0 ) go to 1000

      newarr = .true.
      indfor = 0
      if ( index>0 .and. index<=iinfor ) then

!     --- 0<index<=iinfor  array has been filled before

         indfor = infor ( index )
         lengol = infor1 ( 1,index )
         newarr = .false.

      end if

      if ( indfor>0 .and. length<=lengol ) then

!     --- indfor>0  and length>lengol   old positions are used

         ipstar = indfor
         if ( ipcopy>ipstar ) then

!        --- Copy forwards

             do i = 0, length-1
                ibuffr(ipstar+i) = ibuffr(ipcopy+i)
             end do

         else if ( ipcopy<ipstar ) then

!        --- Copy backwards

             do i = length, 1, -1
                ibuffr(ipstar+length-i) = ibuffr(ipcopy+length-i)
             end do
         end if

      else

!     --- length>lengol or indfor<0
!         new array is bigger than preceding one, &
!         or array is stored in backing storage
!         old positions are deleted, new positions are used
!         check free positions

         j = ibfree+length+iextra*2
         if ( j>0 .and. j<=nbuffr ) then

!        --- enough free space available   ( standard situation )

            ipstar = ibfree
            ibfree = ibfree+length+iextra*2
            if ( intlen==2 .and. mod (ibfree,2_8) ==0 ) &
                ibfree = ibfree+1
            if ( ipcopy>ipstar ) then

!           --- Copy forwards

                do i = 0, length-1
                   ibuffr(ipstar+i) = ibuffr(ipcopy+i)
                end do

            else if ( ipcopy<ipstar ) then

!           --- Copy backwards

                do i = length, 1, -1
                   ibuffr(ipstar+length-i) = ibuffr(ipcopy+length-i)
                end do
            end if

         else

!        --- Not enough space at the end of IBUFFR
!            Set IPSTAR = IPCOPY

            ipstar = ipcopy
            j = ibfree
            ibfree = max(ibfree,ipstar+length+iextra*2)

            if ( ibfree<=0 ) then

!           --- Error not enough space in ibuffr and extension is not
!               possible

               call errintlong ( ibfree, 1 )
               call errint ( length, 2 )
               call errsub ( 3116, 2, 0, 0 )
               go to 1000

            end if  ! ( ibfree<=0 )

            if ( intlen==2 .and. mod (ibfree,2_8) ==0 ) &
                ibfree = ibfree+1

         end if

      end if

      if ( index<=0 .or. index>iinfor ) then

!     --- New array

         iinfor = iinfor+1
         if  ( iinfor>numarrs ) then

!        --- iinfor too large
!            Check if some array has been deleted, if not error

            do i = 1, numarrs
               if ( infor1(1,i)==0 ) then

!              --- infor1(1,i) = 0, means that array i is free

                  index = i
                  iinfor = numarrs
                  go to 150

               end if
            end do

!           --- iinfor too large error

            call errint ( iinfor, 1 )
            call errint ( numarrs, 2 )
            call errsub ( 760, 2, 0, 0 )

         end if
         if ( ierror/=0 ) go to 1000
         index = iinfor

      end if

150   infor(index) = ipstar
      infor1(1,index) = length
      infor1(2,index) = 10
      if ( inback (index)>0 ) inback (index) = -inback (index)

      call prinbf ( ibfree-1 )
      if ( ierror/=0 ) go to 1000

      if ( idebug>0 ) then
         ibuffr(ipstar-1) = index
         ibuffr(ipstar+length) = index
      end if

      if ( ioutp>2 ) then
         write(irefwr,*) ' Create array ', namear, &
                         ' with sequence number', index, &
                         ' and starting address', infor(index), &
                         ' in subroutine ', &
                         namsub(levsub)//':ini083'
      end if

      if ( newarr ) then

!     --- Fill names table

         lenchr = min ( 16, len(namear) )
         namarr(index) = namear(1:lenchr)

      end if

1000  call erclos ( 'ini083' )

      end
