      subroutine ini084 ( ibuffr, index, length, namear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 07-10-2014 Extra debug
!        version  2.3  date 07-02-2014 Correction in case not enough space
!        version  2.2  date 21-11-2012 Long integers
!
!   copyright (c) 1997-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend existing array such that the new length becomes LENGTH
!     The first part of the array has already been constructed in IBUFFR
!     from the temporary position IPCOPY. It must be copied to the new
!     positions.
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cmessgf90'
      include 'SPcommon/cmesscf90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cnamtbf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index, length, ibuffr(*)
      character (len=*) namear

!     ibuffr i/o Integer buffer array in which all large data is stored
!     index  i/o The value of index is the reference number of the
!                array of which the starting position must be computed with
!                respect to array INFOR in common block IINFOR
!                This means that the positions (i,INDEX) (i=1,2,3) in INFOR
!                contain all the information about the storage of the array
!     length  i  length of the array in words
!     namear  i  Name of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical newarr, finished
      integer (kind=8) ipcopy, ih
      integer i, iextra, lenchr, lencop, error
      integer, allocatable :: iwork(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     finished       if true the extension is finished
!     i              General loop variable
!     iextra         Extra space needed at both sides of the array for control
!                    integers
!     ih             General help variable
!     ipcopy         Starting address of array to be copied
!     iwork          integer work array to store the part to be copied
!     lenchr         Number of characters in name of array
!     lencop         Length of part to be copied
!     newarr         Indicates if the array is made at a complete new place
!                    (true) or is a straight-forward extansion of the
!                    existing one
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext
      integer inilen

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI081         Create permanent space in array IBUFFR
!     INICHK1        Check IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
!   none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1765   length of array less than one
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( idebug>0 ) then
!        Set iextra = extra space needed
!        check all control integers
!     Make IPCOPY semi permanent by putting information in array INFOR
!          in common block CARRAY, without raising IBFREE
!     Reserve space for new array ( subroutine INI091, starting address IPSTAR )
!     Recompute IPCOPY (It might have been reorganised)
!     If ( IPCOPY # IPSTAR ) then
!         Copy contents of first array to new positions
!     If ( INDEX>index of semi-permanent array ) then
!         INDEX = INDEX - 1
!     Decrease IINFOR
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini084' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini084'
         write(irefwr,1) 'length, index', length, index
         write(irefwr,1) 'infor', infor(index), infor1(1:2,index)
  1      format ( a, 1x, (10i11) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( idebug==0 ) then
         iextra = 0
      else
         iextra = intlen
         if ( ifirws==0 ) then
            ifirws = 1
         else
            ifirws = 2
         end if
         call inichk1(ibuffr)
      end if

      if ( debug ) write(irefwr,1) 'iextra, ifirws', iextra, ifirws

      if ( length<=0 ) then

!     --- length<=0, error

         call errint ( length, 1 )
         call errchr ( namear, 1 )
         call errsub ( 1765, 1, 0, 1 )

      end if
      if ( ierror/=0 ) go to 1000

      newarr = .false.
      call ini060text ( ibuffr, index, namear )
      if ( ierror/=0 ) go to 1000

      ipcopy = inigettext ( index, namear )
      lencop = inilen(index)

      if ( debug ) write(irefwr,1) 'ipcopy, lencop', ipcopy, lencop

      finished = .false.

!     --- Check if array can be extended without any action

      ih = ipcopy+lencop+iextra*2
      if ( intlen==2 .and. mod(ih,2_8)==0 ) ih = ih + 1

      if ( debug ) &
         write(irefwr,1) 'ih, ibfree, end, nbuffr', &
                          ih, ibfree, ipcopy+length+iextra*2, nbuffr

      if ( ibfree<=ih .and. ipcopy+length+iextra*2<=nbuffr ) then

!     --- index is the last array in IBUFFR and there is sufficient space

         ibfree = ipcopy+length+iextra*2
         if ( intlen==2 .and. mod(ibfree,2_8)==0 ) ibfree = ibfree + 1
         finished = .true.

      else

!     --- Check if there is sufficient contiguous space

         do i = 1, iinfor
            if ( infor(i)>ipcopy ) ih = min ( ih,infor(i) )
         end do
         if ( ih-ipcopy-2*iextra>length ) then

!        --- Sufficient space has been found

            finished = .true.

         end if

      end if

      if ( debug ) write(irefwr,*) 'finished ', finished

      if ( finished ) then

!     --- Sufficient space has been found

         infor1(1,index) = length
         infor1(2,index) = max(10,infor1(2,index))
         if ( idebug>1 ) then
            ibuffr(ipcopy+lencop+1:ipcopy+length) = iinfin
            ibuffr(max(nbufw1,ipcopy+length+1):nbufw2) = iinfin
            nbufw1 = 0
            nbufw2 = 0
         end if
         if ( idebug>=1 ) then
            ibuffr(ipcopy+length) = index
         end if

      else

!     --- Not enough space found
!         Copy old array to temporary array

         allocate ( iwork(lencop), stat = error )
         if ( error/=0 ) call eralloc ( error, lencop, 'iwork' )
         if ( ierror/=0 ) go to 1000

         iwork(1:lencop) = ibuffr(ipcopy:ipcopy+lencop-1)

         call ini081 ( ibuffr, index, length, namear )
         if ( ierror/=0 ) go to 1000

         ipcopy = inigettext ( index, namear )
         ibuffr(ipcopy:ipcopy+lencop-1) = iwork(1:lencop)

         deallocate ( iwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )

      end if  ! ( finished )

      if ( debug .or. ioutp>2 ) then
         write(irefwr,*) ' Create array ', namear, &
                         ' with sequence number', index, &
                         ' and starting address', infor(index), &
                         ' in subroutine ', &
                         namsub(levsub)//':ini084'
      end if

      if ( newarr .and. .not.finished ) then

!     --- Fill names table

         lenchr = min ( 16, len(namear) )
         namarr(index) = namear(1:lenchr)

      end if

1000  call erclos ( 'ini084' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ini084'

      end if  ! ( debug )

      end
