      subroutine iniactm26 ( ibuffr, buffer, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-02-2012 Extension with 3d levelset
!        version  1.0  date 09-06-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmesh part ad subpart 9: iobstfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     element
!     node
!     level_set
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*)
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh         i/o   Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kelmad
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     kelmad         memory management reference number of array KMESH part ad
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     PRBOUNLEVSET   Fill array iobstfaces with edges corresponding to the
!                    boundary of the region with elements with levelset>0
!     PRBOUNLEVSET3D Fill array iobstfaces with faces corresponding to the
!                    boundary of the region with elements with levelset>0
!     TOPFILLKMAD9   Fill array kmesh ad  part 9: obstfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniactm26' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniactm26'
         write(irefwr,1) 'kmesh(64), kmesh(6)', kmesh(64), kmesh(6)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kelmad = kmesh(52)
      if ( debug ) write(irefwr,1) 'kelmad', kelmad
      if ( kelmad>0 .and. kmesh(kelmad+8)>0 ) then

!     --- Array kmesh ad already exists

         if ( debug ) &
            write(irefwr,1) 'kmesh(kelmad+8)', kmesh(kelmad+8)
         call ini060text ( ibuffr, kmesh(kelmad+8), 'iseqicross' )

      else

!     --- Array kmesh ad part 9 does not exist, create in case of levelset

         select case ( kmesh(6) )

         case(2)

!        --- ndim = 2, use edges

            call prbounlevset ( kmesh, ibuffr )

         case(3)

!        --- ndim = 3, use faces

            call prbounlevset3d ( kmesh, ibuffr )

         end select  ! case ( kmesh(6) )

      end if

1000  call erclos ( 'iniactm26' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniactm26'

      end if  ! ( debug )

      end
