      subroutine iniactnmeshl ( ibuffr, buffer, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 28-12-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates nmesh part l
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*)
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh         i/o   Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipnmpl, ipnmpc, ipnmpd, ipkmd1, ipkmd2, ipiwork
      integer kelmaa, lpnmpl, lpnmplest, &
              i, npoint, medpel, melpnd, nedges

!     i              Counting variable
!     ipiwork        Starting address of array iwork in array IBUFFR
!     ipkmd1         Starting address of KMESH part d1 in array IBUFFR
!     ipkmd2         Starting address of KMESH part d2 in array IBUFFR
!     ipnmpc         Pointer to nmesh part c
!     ipnmpd         Pointer to nmesh part d
!     ipnmpl         Pointer to nmesh part l
!     kelmaa         Starting position of KMESH part AA in array KMESH
!     lpnmpl         length of array nmeshl
!     lpnmplest      estimated length of array nmeshl
!     medpel         maximum number of edges in one element
!     melpnd         maximum number of elements connected to a node
!     nedges         number of edges
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060         Reactivate existing array in IBUFFR
!     INI062         reserve space for temporary arrays
!     INI083         Create new array in IBUFFR in the case that already a
!                    temporary copy has been stored in IBUFFR
!     INIACTKMESHD   Fill array kmesh part d
!     INIGET         Compute pointer of array in IBUFFR
!     TOPEDG06       creates nmesh part l
!     TOPFILLNMESHC  Fill array nmesh parts c and d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First check if nmeshc, nmeshd, kmeshd exist
!     Activate old arrays nmeshc, nmeshd, kmeshd
!     Create temporary space for nmeshl, do worst case estimate
!     compute starting addresses
!     Fill nmesh part l (nmeshl) with actual data
!     Make nmeshl permanent
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniactnmeshl' )
      kelmaa = kmesh(48)-1

!     --- First check if nmeshc, nmeshd, kmeshd exist

      if ( kmesh(18)==0 ) call iniactkmeshd ( ibuffr, kmesh )  ! kmeshd
      if ( kmesh(kelmaa+33)==0 ) &
         call topfillnmeshc ( ibuffr, buffer, kmesh )  ! nmeshc/d

      npoint = kmesh(8)          ! number of nodes
      melpnd = kmesh(kelmaa+14)  ! maximum number of elements per node

!     --- Activate old arrays nmeshc, nmeshd, kmeshd

      call ini060 ( ibuffr, kmesh(kelmaa+33) )   ! nmeshc
      call ini060 ( ibuffr, kmesh(kelmaa+34) )   ! nmeshd
      call ini060 ( ibuffr, kmesh(18) )          ! kmeshd

      if ( melpnd==0 ) then

!     --- melpnd = 0, compute

         ipkmd1 = iniget(kmesh(18))   ! starting address of kmesh part d1
         do i = 1, npoint
            melpnd = max(melpnd,ibuffr(ipkmd1+i)-ibuffr(ipkmd1+i-1))
         end do
         kmesh(kelmaa+14) = melpnd

      end if
      medpel = kmesh(kelmaa+22)

!     --- Create temporary space for nmeshl, do worst case estimate
!         number of nodes * maximum number of elements connected to one
!         node * &
!         maximum number of edges connected to one element

      lpnmplest = npoint*melpnd*medpel + 2*npoint
      lpnmpl = lpnmplest
      nedges =  kmesh(kelmaa+4)

      call ini062 ( ibuffr, lpnmplest+nedges, ipnmpl )
      ipiwork = ipnmpl+lpnmplest

!     --- compute starting addresses

      ipnmpc = iniget(kmesh(kelmaa+33))
      ipnmpd = iniget(kmesh(kelmaa+34))
      ipkmd1 = iniget(kmesh(18))
      ipkmd2 = ipkmd1+npoint+1

!     --- Fill nmesh part l (nmeshl) with actual data

      call topedg06 ( ibuffr(ipnmpc), kmesh, kmesh(kelmaa+1), &
                      ibuffr(ipnmpl), lpnmpl, ibuffr(ipnmpd), &
                      ibuffr(ipkmd1), ibuffr(ipkmd2), &
                      ibuffr(ipiwork) )

      if ( lpnmpl>lpnmplest ) then

!     --- Estimated length too short

         call errint ( lpnmplest, 1 )
         call errint ( lpnmpl, 2 )
         call errchr ( 'nmeshl', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Make nmeshl permanent

      call ini083 ( ibuffr, kmesh(kelmaa+42), lpnmpl, ipnmpl, &
                    'nmeshl' )

1000  call erclos ( 'iniactnmeshl' )

      end
