      subroutine inichk1 ( ibuffr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 08-03-2013 Correction check
!        version  1.5  date 21-11-2012 Long integers
!        version  1.4  date 20-07-2011 Extra jump
!
!   copyright (c) 1997-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Check all control integers in array IBUFFR
!   Set tail of IBUFFR to infinity if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     debug
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cnamtbf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*)

!    ibuffr   i/o  Integer array in which all large data is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipoint, ihbuf
      integer i, iarsv
      logical check

!     check          Help parameter for if statement
!     i              General loop variable
!     iarsv          Help variable to compute the new value of narbuf
!     ihbuf          Help parameter to store one of the nbufw2/3 values
!     ipoint         Address in array IBUFFR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRINTLONG     Put long integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1809   Leading control integer incorrect
!    1810   Trailing control integer incorrect
!    1814   Control integers of work space incorrect
!    1815   Control integers of work space created by ini079 incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   For i := 1 (1) iinfor
!      if ( infor(i)>0 ) then
!         check if array i is preceded and followed by control integer i
!   if ( idebug>1 and nbuffr>nbufls ) then
!      Set last part of IBUFFR to iinfin
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( idebug==0 ) go to 1000

!     --- Check control integers

      nbufmx = max(nbuffr,nbufmx)
      do i = 1, iinfor
         if ( infor(i)>0 ) then

!        --- Array i is stored in IBUFFR

            ipoint = infor(i) - 1
            if ( ibuffr(ipoint)/=i ) then

!           --- Error 1809: leading control integer incorrect

               call eropen ( 'inichk1' )
               call errint ( i, 1 )
               call errint ( ibuffr(ipoint), 2 )
               call errint ( i, 3 )
               call errintlong ( ipoint, 4 )
               call errintlong ( infor(i) + infor1(1,i), 5 )
               call errchr ( namarr(i), 1 )
               call errsub ( 1809, 5, 0, 1 )
               call erclos ( 'inichk1' )
               go to 1000

            end if

            ipoint = infor(i) + infor1(1,i)
            if ( ibuffr(ipoint)/=i ) then

!           --- Error 1810  Trailing control integer incorrect

               call eropen ( 'inichk1' )
               call errint ( i, 1 )
               call errint ( ibuffr(ipoint), 2 )
               call errint ( i, 3 )
               call errintlong ( ipoint, 4 )
               call errintlong ( infor(i) - 1, 5 )
               call errchr ( namarr(i), 1 )
               call errsub ( 1810, 5, 0, 1 )
               call erclos ( 'inichk1' )
               go to 1000

            end if

         end if

      end do

      if ( nbufw1>0 ) then

!     --- nbufw1>0, check work space

         if ( nbufw3==0 ) then
            ihbuf = nbufw2
         else
            ihbuf = nbufw3
         end if
         if ( ibuffr(nbufw1)/=numarrs+1 .or. &
              ibuffr(ihbuf)/=numarrs+1 ) then
            call eropen ( 'inichk1' )
            call errint ( ibuffr(nbufw1), 1 )
            call errint ( ibuffr(ihbuf), 2 )
            call errint ( numarrs+1, 3 )
            call errintlong ( nbufw1, 4 )
            call errintlong ( ihbuf, 5 )
            call errintlong ( nbufw1+1, 6 )
            call errintlong ( ihbuf-nbufw1-1, 7 )
            call errsub ( 1814, 7, 0, 0 )
            call erclos ( 'inichk1' )
            go to 1000
         end if

      end if

      check = narbuf>0
      if ( check ) check = nbuffr>=iarbuf(narbuf,1)

      if ( check ) then

!     --- Some of the arrays have been removed, decrease narbuf

         iarsv = 0
         do i = narbuf, 1, -1
            if ( nbuffr<iarbuf(i,1) ) then
               iarsv = i
               go to 120
            end if
         end do
120      narbuf = iarsv

      end if

      if ( narbuf>0 ) then

!     --- narbuf>0, check the narbuf arrays created by ini079

         do i = 1, narbuf
            if ( ibuffr(iarbuf(i,1))/=1600+i .or. &
                 ibuffr(iarbuf(i,2))/=1600+i ) then
               call eropen ( 'inichk1' )
               call errint ( i, 1 )
               call errint ( ibuffr(iarbuf(i,1)), 2 )
               call errint ( ibuffr(iarbuf(i,2)), 3 )
               call errint ( 1600+i, 4 )
               call errintlong ( iarbuf(i,1), 5 )
               call errintlong ( iarbuf(i,2), 6 )
               call errintlong ( iarbuf(i,1)+1, 6 )
               call errintlong ( iarbuf(i,2)-iarbuf(i,1)-1, 7 )
               call errsub ( 1815, 8, 0, 0 )
               call erclos ( 'inichk1' )
               go to 1000
            end if
         end do

      end if

      nbufls = min(nbufls,nbuffr)

      if ( idebug>1 .and. nbufls>nbuffr ) then

!     --- Last part of IBUFFR has not yet been set to IINFIN

         ibuffr(nbufls+1:nbuffr) = iinfin
         nbufls = nbuffr

      end if
      nbufw1 = 0
      nbufw2 = 0
      nbufw3 = 0

1000  end
