      subroutine inicopreal2din ( nodes, nnodes, coor, ndim, coorcopy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the nnodes coordinates indicated by array nodes
!     from array coor into array coorcopy of size nnodes x ndim
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     node
!     coordinate
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(nnodes), ndim
      double precision coor(ndim,*), coorcopy(ndim,nnodes)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coorcopy       o    Array of size ndim x nnodes containing the copied
!                         coordinates
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers that must be copied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicopreal2din' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicopreal2din'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      coorcopy(1:ndim,1:nnodes) = coor(1:ndim,nodes(1:nnodes))

1000  call erclos ( 'inicopreal2din' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inicopreal2din'

      end if  ! ( debug )

      end
