      subroutine inicoprinput ( ibuffr, buffer, rinput, mmrinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 10-10-2012
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy sepran real array referenced by mmrinput into rinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), mmrinput
      double precision buffer(*), rinput(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     mmrinput       i    Memory management sequence number with respect to
!                         rinput
!     rinput         o    real input array to be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) iprinput
      integer length
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iprinput       Starting address of array rinput
!     length         Help parameter to indicate the length of arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext
      integer inidln

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIDLN         Compute length of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicoprinput' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicoprinput'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      length = inidln(mmrinput)

!     --- Guarantee that array IINPUT is placed into IBUFFR

      call ini060text ( ibuffr, -mmrinput, 'rinput' )

!     --- Copy contents of IINPUT

      iprinput = inidgttext ( mmrinput, 'rinput' )

      rinput(1:length) = buffer(iprinput:iprinput+length-1)

1000  call erclos ( 'inicoprinput' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inicoprinput'

      end if  ! ( debug )

      end
