      subroutine inifilename ( indfile, namefile, filename, nchar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create filename from indfile and namefile
!
! **********************************************************************
!
!                       KEYWORDS
!
!     file
!     name
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cfilenamesf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: indfile
      integer, intent(out) :: nchar
      character (len=*) :: namefile, filename

!     filename       o    Name of file
!     indfile        i    Sequence number of file in array filenames
!     namefile       i    Default name of the file
!     nchar          o    Number of characters in a filename
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer length
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     length         length of string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inifilename' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inifilename'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indfile==0 ) then

!     --- indfile = 0, make file name default

         filename = namefile
         nchar = len(namefile)

      else

!     --- indfile>0, use file name from cfilenames

         filename = file_names(indfile)
         length = len(filename)
         nchar = index ( filename, ' ' )-1
         if ( nchar==0 ) nchar = length

      end if  ! ( indfile==0 )

1000  call erclos ( 'inifilename' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inifilename'

      end if  ! ( debug )

      end
