      subroutine inimultadd3 ( ibuffr, buffer, iu1, iu2, iu3, &
                               a1, a2, a3, iures )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 23-07-2007
!
!   copyright (c) 2007-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute a1 u1 + a2 u2 + a3 u3 and store in ures
!     ures may be one of the three vectors
!     The vectors must be sepran solution vectors with the same length
! **********************************************************************
!
!                       KEYWORDS
!
!     multiplication
!     linear_combination
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iu1(5), iu2(5), iu3(5), iures(5), ibuffr(*)
      double precision a1, a2, a3, buffer(*)

!     a1             i    multiplication constant
!     a2             i    multiplication constant
!     a3             i    multiplication constant
!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iu1            i    First vector
!     iu2            i    Second vector
!     iu3            i    Third vector
!     iures         i/o   Result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipu1, ipu2, ipu3, ipures
      integer nusol, i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ipu1           Starting address of array u1
!     ipu2           Starting address of array u2
!     ipu3           Starting address of array u3
!     ipures         Starting address of array ures
!     nusol          Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI080         Create permanent space in array BUFFER
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     PRMULTADD3     Compute a1 u1 + a2 u2 + a3 u3 and store in ures
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2888   The length of three input vectors is not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inimultadd3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inimultadd3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays iu1/2/3

      call ini060text ( ibuffr, iu1(1), 'iu1' )
      call ini060text ( ibuffr, iu2(1), 'iu2' )
      call ini060text ( ibuffr, iu3(1), 'iu3' )

!     --- Get length of arrays

      nusol = iu1(5)
      if ( iu2(5)/=nusol .or. iu3(5)/=nusol ) then

!     --- error: arrays do not have the same length

         call errchr ( 'iu1', 1 )
         call errchr ( 'iu2', 2 )
         call errchr ( 'iu3', 3 )
         call errint ( iu1(5), 1 )
         call errint ( iu2(5), 2 )
         call errint ( iu3(5), 3 )
         call errsub ( 2888, 3, 0, 3 )
         go to 1000

      end if  ! ( iu2(5)/=nusol .or. iu3(5)/=nusol )

!     --- Create or activate result array

      call ini080 ( ibuffr, iures(1), nusol, 'iures' )
      do i = 2, 5
         iures(i) = iu1(i)
      end do  ! i = 2, 5

!     --- Compute starting addresses

      ipu1 = inidgttext ( iu1(1), 'iu1' )
      ipu2 = inidgttext ( iu2(1), 'iu2' )
      ipu3 = inidgttext ( iu3(1), 'iu3' )
      ipures = inidgttext ( iures(1), 'iures' )
      call prmultadd3 ( nusol, buffer(ipu1), buffer(ipu2), buffer(ipu3), &
                        a1, a2, a3, buffer(ipures) )

1000  call erclos ( 'inimultadd3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inimultadd3'

      end if  ! ( debug )

      end
