      function ininelmsurf ( ibuffr, kmesh, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 31-07-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of elements in surface
!     Warning this subroutine uses memory management subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     element
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ininelmsurf, ibuffr(*), kmesh(*), isurnr

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     ininelmsurf    o    Number of elements in surface
!     isurnr         i    Surface sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmsn
      integer nsurfs, kelmt

!     ipkmsn         Starting address of array KMESH part n in array
!                    IBUFFR
!     kelmt          Indicates if array KMESH part t has been filled >0, &
!                    or not 0
!                    If kelmt>0, kelmt denotes the starting address of
!                    KMESH part t in KMESH
!     nsurfs         number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060         Reactivate existing array in IBUFFR
!     INIGET         Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      75   Array KMESH has not been filled
!    1058   kmeshn has not been filled
!    2282   surface number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ininelmsurf' )
      ininelmsurf = 0
      if ( kmesh(2)/=100 ) then

!     --- Array kmesh has not been filled

         call errint ( kmesh(2), 1 )
         call errsub ( 75, 1, 0, 0 )
         go to 1000

      end if

      nsurfs = kmesh(12)
      if ( isurnr<=0 .or. isurnr>nsurfs ) then

!     --- surface number out of range

         call errint ( isurnr, 1 )
         call errint ( nsurfs, 2 )
         call errsub ( 2282, 2, 0, 0 )
         go to 1000

      end if

      if ( kmesh(28)/=0 ) then

!     --- kmesh(28)>0, i.e.kmesh part n exists
!         Activate array and compute starting address

         call ini060 ( ibuffr, -kmesh(28) )
         ipkmsn = iniget(kmesh(28))
         ininelmsurf = ibuffr(ipkmsn-1+nsurfs+isurnr)

      else if ( kmesh(13)==0 ) then

!     --- No volume elements, use kmesh part t

         kelmt = kmesh(36)-2+2*isurnr
         ininelmsurf = kmesh(kelmt+1)-kmesh(kelmt)+1

      else

!     --- kmeshn has not been filled

         call errsub ( 1058, 0, 0, 0 )
         go to 1000

      end if

1000  call erclos ( 'ininelmsurf' )
      end
