      function ininelzsurf ( ibuffr, kmesh, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 31-07-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of elements in height direction of pipe surface
!     Warning this subroutine uses memory management subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     element
!     pipe
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ininelzsurf, ibuffr(*), kmesh(*), isurnr

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     ininelzsurf    o    Number of elements in height direction of pipe surface
!     isurnr         i    Surface sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmss
      integer nsurfs, isurf

!     ipkmss         Starting address of array KMESH part s in array
!                    IBUFFR
!     isurf          Type of surface with respect to mesh generator
!     nsurfs         number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060         Reactivate existing array in IBUFFR
!     INIGET         Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      75   Array KMESH has not been filled
!    2282   surface number out of range
!    2347   Surface is not of type pipe surface
!    2348   Array kmeshs has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ininelzsurf' )
      ininelzsurf = 0
      if ( kmesh(2)/=100 ) then

!     --- Array kmesh has not been filled

         call errint ( kmesh(2), 1 )
         call errsub ( 75, 1, 0, 0 )
         go to 1000

      end if

      nsurfs = kmesh(12)
      if ( isurnr<=0 .or. isurnr>nsurfs ) then

!     --- surface number out of range

         call errint ( isurnr, 1 )
         call errint ( nsurfs, 2 )
         call errsub ( 2282, 2, 0, 0 )
         go to 1000

      end if

      if ( kmesh(33)/=0 ) then

!     --- kmesh(33)>0, i.e.kmesh part s exists
!         Activate array and compute starting address

         call ini060 ( ibuffr, -kmesh(33) )
         ipkmss = iniget(kmesh(33))+nsurfs+(isurnr-1)*8

         isurf = ibuffr(ipkmss+1)
         if ( isurf/=10 .and. isurf/=12 ) then

!        --- Surface is not of type pipe surface

            call errint ( isurnr, 1 )
            call errsub ( 2347, 1, 0, 0 )
            go to 1000

         end if
         ininelzsurf = ibuffr(ipkmss+8)

      else

!     --- kmeshs has not been filled

         call errsub ( 2348, 0, 0, 0 )
         go to 1000

      end if

1000  call erclos ( 'ininelzsurf' )
      end
