      function ininodezerlev ( ibuffr, kmesh, ilev )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 19-10-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with node at boundary defined by zero levelset
!     1d only
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     node
!     level_set
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ininodezerlev, ibuffr(*), kmesh(*), ilev

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     ilev           i    Sequence number of level being considered
!     ininodezerlev  o    Node number found
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipipointobst
      integer mmipointobst
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipipointobst   Starting address of array ipointobst
!     mmipointobst   memory management sequence number of array ipointobst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer ininodezerlev1
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIACTMK       General memory management subroutine to activate or create
!                    a permanent array
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     ININODEZERLEV1 Return with node at boundary defined by zero levelset
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ininodezerlev' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ininodezerlev'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ininodezerlev = 0  ! initialization

      if ( kmesh(6)/=1 ) then

!     --- Error: ndim#1

         call errint ( kmesh(6), 1 )
         call errint ( 1, 2 )
         call errint ( 1, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( kmesh(6)/=1 )

!     --- activate or create array ipointobst

      mmipointobst = 0
      call iniactmk ( ibuffr, 5, 1, kmesh, kmesh, mmipointobst )

!     --- Get starting address

      ipipointobst = inigettext ( mmipointobst, 'ipointobst' )

!     --- Find node number

      ininodezerlev = ininodezerlev1 ( ibuffr(ipipointobst), ilev )

1000  call erclos ( 'ininodezerlev' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ininodezerlev'

      end if  ! ( debug )

      end
