      subroutine ininodonedge ( iedge, inode1, inode2, nmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find other end node on edge if one of the end nodes and the ege number
!     are given
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iedge, inode1, inode2, nmeshc(*)

!     iedge          i    edge sequence number
!     inode1         i    First node in edge
!     inode2         o    Last node in edge
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, nnode_edge
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipstart        Starting address in array
!     nnode_edge     number of nodes corresponding to edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2659   Node is not end point of edge
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ininodonedge' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ininodonedge'

      end if
      if ( ierror.ne.0 ) go to 1000

      ipstart = nmeshc(iedge)      ! starting address of iedge in nmeshc
      nnode_edge = nmeshc(ipstart) ! number of nodes on iedge
      if ( nmeshc(ipstart+1).eq.inode1 ) then

!     --- node1 is starting node, so node2 is end node

         inode2 = nmeshc(ipstart+nnode_edge)

      else if ( nmeshc(ipstart+nnode_edge).eq.inode1 ) then

!     --- node1 is end node, so node2 is start node

         inode2 = nmeshc(ipstart+1)

      else

!     --- node1 is not end node of edge

         call errint ( inode1, 1 )
         call errint ( iedge, 2 )
         call errsub ( 2659, 2, 0, 0 )

      end if  ! ( nmeshc(ipstart+1).eq.inode1 )

1000  call erclos ( 'ininodonedge' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ininodonedge'

      end if

      end
