      subroutine ininonempty ( kprob, iprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-05-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find problem number with non-empty elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     find
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprob(*), iprob

!     iprob          o    Problem number found
!                         If 0, no problem number found
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielgrp, nprob, ipkprb, nelgrp, kprobc, itype
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ielgrp         element group number
!     ipkprb         Starting address of array KPROB for actual problem
!                    number - 1
!     itype          Table containing information about the numbers behind
!                    the keywords in the table code
!     kprobc         Starting address of KPROB part c in KPROB
!     nelgrp         Number of element groups
!     nprob          Number of various problems
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ininonempty' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ininonempty'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      nelgrp = kprob(8)   ! number of element groups
      nprob = kprob(40)
      iprob = 0
      ipkprb = 0
      do i = 1, nprob

         kprobc = kprob(ipkprb+16)  ! start of kprob part c

!        --- For all element groups do

         do ielgrp = 1, nelgrp

            itype = kprob(kprobc+ielgrp-1)  ! type number of element
            if ( itype==0 ) go to 100       ! empty type, jump to next problem

         end do  ! ielgrp = 1, nelgrp

!        --- If the program reaches this position all elements are non-empty
!            Problem found return

         iprob = i
         go to 1000

100      ipkprb = ipkprb + kprob ( 3+ipkprb )

      end do  ! i = 1, nprob

1000  call erclos ( 'ininonempty' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ininonempty'

      end if  ! ( debug )

      end
