      subroutine ininped ( kmesh, ielgrp, nndped, nspecp )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.10 date 22-05-2014 Extension with shape 22
!        version  1.9  date 10-02-2012 use getnpelmelgrp
!        version  1.8  date 27-11-2011 use function getshapeelgrp
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fetch number of nodes per edge in elementgroup ielgrp
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), ielgrp, nndped, nspecp

!     ielgrp     i    element group number
!     kmesh      i    Standard SEPRAN array containing information of the mesh
!     nndped     i    number of nodes in one edge
!     nspecp     i    number of spectral points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer dnoped(22), ishape

!     dnoped     Data nodes per edge
!     ishape     Type number of elements in the mesh or element group
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getshapeelgrp

!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     GETSHAPEELGRP  Determine shape number for an element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
!    1692   type of spectral element has not been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data dnoped / 2, 3, 2, 3, 2,   3, 3, 4, 2, 2, &
                    2, 3, 2, 3, 3,   3, 2, 2, 0, 0, &
                    2, 3 /
! ======================================================================
!
      call eropen ( 'ininped' )

      ishape = getshapeelgrp ( kmesh, ielgrp )
      if ( ishape<=0 .and. nspecp==0 ) then

         nndped = 2

      else if ( (ishape>=50 .and. ishape<=60 .or. &
                 ishape>70 .and. ishape<=80 .or. &
                 ishape==109 .or. ishape==134) &
                .and. nspecp==0 ) then

         nndped = 2

      else if (nspecp==0 .and. ishape<=22 ) then

         nndped = dnoped(ishape)
         if ( nndped==0 ) then

!        --- Shape number not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )

         end if

      else if ( nspecp>0 .and. &
              ( ishape==5 .or. ishape==13) ) then
         nndped = nspecp

      else

         call errint ( ishape, 1 )
         call errint ( nspecp, 2 )
         call errsub ( 1692, 2, 0, 0 )

      end if
      call erclos ( 'ininped' )
      end
