      subroutine inistr ( number, kprob, itypvc, ivec, iprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 04-03-2011 Extension with probextend
!        version  1.2  date 12-12-2005 Fortran 90
!        version  1.1  date 25-05-1996 (itypvc set at input)
!
!   copyright (c) 1993-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the structure of the vector with exactly NUMBER unknowns per point
!     If this type does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     structure
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, kprob(*), itypvc, ivec, iprob

!     iprob          i    Actual problem number
!     itypvc        i/o   Indication of the type of solution vector
!                         If itypvc = 110 at input then the output vector may be
!                         a vector of type 110 or 115 else of type 115 only
!                         Output:
!                         110: solution vector
!                         115: vector of special structure
!     ivec           o    Sequence number of vector of special structure
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ipkprb, nunkp, kproba, ndtvec, kprobe, kelgrp
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ipkprb         Starting address of array KPROB for actual problem
!                    number minus 1
!     kelgrp         sum over all standard elements of number of points in
!                    standard elements
!     kproba         starting address in kprob part a
!     kprobe         starting address in kprob part e
!     ndtvec         Number of vectors of special structure
!     nunkp          Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniprb

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPRB         Get value of ipkprb as function of iprob
!     PROBEXTEND     Extend problem definition with vectors of special structure
!                    of exactly number degrees of freedom
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( solution vector contains exactly NUMBER unknowns per point ) then
!        itypvc := 110
!     else
!        ivec := 0
!        For i := 1 (1) NDTVEC, while ( ivec=0 ) do
!           If ( vector of special structure i has exactly NUMBER unknowns
!                per point then )
!              ivec := i
!              itypvc := 115
!        If ( ivec=0 ) then
!           Error: no vector found
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inistr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inistr'

      end if
      if ( ierror/=0 ) go to 1000

      ipkprb = iniprb ( iprob, kprob )
      nunkp  = kprob(ipkprb+4)
      kproba = kprob(ipkprb+14)
      if ( itypvc==110 .and. kproba==0 .and. nunkp==number ) then

!     --- nunkp = number, vector is of type solution vector

         itypvc = 110

      else

!     --- vector is not of type solution vector
!         Check if there is a vector of special structure with NUMBER
!         unknowns per point

         ivec = 0
         ndtvec = kprob(ipkprb+6)
         kprobe = kprob(ipkprb+18)
         kelgrp = kprob(ipkprb+39)
         do i = 1, ndtvec
            nunkp = - kprob(ipkprb+kprobe-1+kelgrp*ndtvec+i)
            if ( nunkp==number ) then

!           --- Vector of special structure found

               itypvc = 115
               ivec = i

            end if

         end do  ! i = 1, ndtvec

         if ( ivec==0 ) then

!        --- No vector found

            call probextend ( kprob, number )
            ivec = kprob(6)
            itypvc = 115

         end if  ! ( ivec==0 )

      end if

1000  call erclos ( 'inistr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inistr'

      end if

      end
