      subroutine inizerolev ( ipointobst, istart, numbndobst, ilev )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-12-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of nodes in zero level set and starting
!     position in ipointobst
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointobst(*), ilev, istart, numbndobst

!     ilev           i    Sequence number of level being considered
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     istart         o    Starting position of zero level set in ipointobst
!     numbndobst     o    Number of nodal points on the boundary of the obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numpntobst, numinnobst
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inizerolev' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inizerolev'
         write(irefwr,1) 'ilev', ilev
         call prinin ( ipointobst, 10, 'ipointobst' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get starting address and number of points in boundary

      istart = ipointobst(ilev)
      numpntobst = ipointobst(istart)
      numinnobst = ipointobst(istart+1)
      numbndobst = ipointobst(istart+2)
      istart = istart+3+numpntobst+numinnobst

1000  call erclos ( 'inizerolev' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inizerolev'

      end if  ! ( debug )

      end
