      program makef7
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 06-10-2006 Remove asknxt
!        version  1.3  date 28-12-2000 Length of records extended to 240
!        version  1.2  date 08-02-1994 Replace ASK000 by ASKNXT
!
!   copyright (c) 1992-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     this program reads the menu messages from the input file
!     and puts them in file 17
! **********************************************************************
!
!                       KEYWORDS
!
!     create_file
!     menu
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxmes, i, inext, imessg, nmessg, ios, ifrom, ito, &
              idum, iref, iref95, iref96, icount
      logical finished
      parameter ( maxmes=4000 )
      integer istart(maxmes)

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     icount         Counter
!     idum           General dummy variable
!     ifrom          Array ISTART is written from position IFROM to ITO
!     imessg         Last message that has been read
!     inext          Next record number to which a message must be written
!     ios            Integer giving the value of the i/o status specifier
!     iref           SEPRAN local reference number to which the message file is
!                    coupled. In SEPRAN this is file 17. The actual reference
!                    number may be different (iref17).
!                    This number is set by INITMD
!     iref95         The unit containing the input data file
!     iref96         The unit to contain the messages
!                    reported by this program
!     istart         In this array the starting position of the messages
!                    on the file are stored.
!                    istart is stored in the records 2-(MAXMES+NREC4-1)/NREC4+1
!                    of the direct access file
!     ito            Array ISTART is written from position IFROM to ITO
!     maxmes         Maximum number of messages plus one that may be stored in
!                    the message file
!     nmessg         Error number read from the standard the input file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     FINISH         Stop SEPRAN
!     INIFIL         Open files
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITMD         Set machine-dependent quantities
!     WRITFL         Write array to SEPRAN direct access file
! **********************************************************************
!
!                       I/O
!
!     Program MAKEF7 requires an input file consisting of records of
!     at most 80 characters (Standard card input)
!     The input should have the following structure:
!     For each message:
!     Error number followed by colon:
!        The message number must start in column 1
!     Followed by
!     Text describing the message
!        The text must start after column 1 (So from column 2) and may not
!        exceed column 79.
!        The number of lines to be used for the message is arbitrary
!     Error numbers must be given in the natural sequence, however,
!     gaps between messages are permitted
!     MAKEF7 produces 2 types of output:
!     Output to the standard output file
!        This output contains messages in the input file
!        and furthermore produces a list of gaps between message numbers
!     Output in the direct access output file
!        The direct access output file is a file with record length of 80 bytes
!        The first record contains one number: the largest message number
!        that is stored in this file
!        The next (MAXMES+NREC4-1)/NREC4 records contain the starting record
!        numbers of the messages on the direct access file consequetively.
!        NREC4 defines the number of integers that can be stored in one record
!        of the message file.
!        If a message is not available, the ending record number of the
!        preceding message plus one is stored.
!        In this way the number of records needed for message number i is
!        equal to istart(i+1)-istart(i)
!        Finally the last records contain the messages sequentially
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1    First column is not zero, but no number in input record
!    2    Error number smaller than preceding one
!    3    Error number larger than MAXMES:increase MAXMES
!    4    Error in the input file
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Initialize common blocks
!      Open message file
!      Initialise current message number and currect record number
!      While (input not exhausted)
!         Read record from the input file
!         If (first column non-empty) then
!            find message number
!            Check consistency of message number
!            If (message number has wrong value) then
!               Print message
!               stop
!            Else if (message number has correct value) then
!               Fill information in array istart
!         else if (first column empty) then
!            write record to message file
!            raise current record number
!      Fill final positions in array ISTART
!      Write array ISTART to records 2-(MAXMES+NREC4-1)/NREC4
!      Write first record to message file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false.
  1   format ( a, 1x, (10i6) )

!     --- SET common blocks (machine-dependent)

      call initcb
      call initmd
      iref = 17

!     --- Open files (message file)

      call inifil(-iref)

!     --- Open the input file to be read

      iref95 = 95
      open ( iref95, file='message.dat' )
      iref96 = 96
      open ( iref96, file='message.out' )

!     --- Initialize parameters inext and imessg

      inext  = 2 + (maxmes+nrec4-1)/nrec4
      imessg = 0
      istart(1) = inext

!     --- Read messages from the input file until the input is exhausted

200   read ( iref95, 210, iostat = ios ) cards(1:80)
210   format(a80)
      if ( debug ) then
         write(irefwr,1) 'ios', ios
         write(irefwr,*) cards(1:80)
      end if  ! ( debug )
      if ( ios==0 ) then

!     --- ios = 0, so record has been read
!         Check if the first column is blank

         if ( cards(1:1)/=' ' ) then

!        --- cards(1:1) # blank, so number has been read
!            Interpret this number by subroutine ASKNXT

            if ( cards(1:1)<'0' .or. cards(1:1)>'9' ) then

!           --- Error 1:  inum=0, hence no number could be found

               write(iref96,220) imessg,(cards(i:i),i=1,80)
220            format(///' *******Error number 1 in program MAKEF7'///, &
                         ' First column of message is not blank,', &
                         ' but no message number could be found'// &
                         ' Preceding message number read is:', i5/ &
                         ' record read is'//1x,80a1)
               stop

            else

!           --- cards(1:1) # blank, so number has been read
!               Interpret this number

               icount = 1
               finished = .false.
               do while ( .not. finished )
                  icount = icount+1
                  if ( cards(icount:icount)<'0' .or. &
                       cards(icount:icount)>'9' ) &
                     finished  = .true.
               end do
               icount = icount-1
               if ( icount==1 ) then
                  read(cards(1:1),221) nmessg
221               format(i1)
               else if ( icount==2 ) then
                  read(cards(1:2),222) nmessg
222               format(i2)
               else if ( icount==3 ) then
                  read(cards(1:3),223) nmessg
223               format(i3)
               else if ( icount==4 ) then
                  read(cards(1:4),224) nmessg
224               format(i4)
               else
                  read(cards(1:5),225) nmessg
225               format(i5)
               end if

            end if  ! ( cards(1:1)<'0' .or. cards(1:1)>'9' )
            if ( nmessg>=maxmes ) then

!           --- Error 3:  nmessg>=maxmes

               write(iref96,230) nmessg,maxmes,(cards(i:i),i=1,80)
230            format(///' *******Error number 3 in program MAKEF7'///, &
                         ' Error number read is larger than MAXMES'/ &
                         ' Raise MAXMES'// &
                         ' Error number read is:', i5/ &
                         ' MAXMES is:', i5/ &
                         ' record read is'//1x,80a1)
               stop

            else if ( nmessg<=imessg ) then

!           --- Error 2:  nmessg<=imessg

               write(iref96,240) nmessg,imessg,(cards(i:i),i=1,80)
240            format(///' *******Error number 2 in program MAKEF7'///, &
                         ' Message number read is not larger than', &
                         ' preceding message number read'// &
                         ' Error number read is:', i5/ &
                         ' Preceding message number read is:', i5/ &
                         ' record read is'//1x,79a1)
               stop

            end if

            if ( nmessg>imessg+1 ) then

!           --- nmessg>imessg+1, i.e. Gap found

               write(iref96,250) imessg+1, nmessg-1
250            format(' Message numbers not in use:', i6, ' ...', i6 )

!              --- Fill array ISTART

               do i = imessg+1, nmessg-1
                  istart(i) = inext
               end do
            end if
            imessg = nmessg
            istart(imessg) = inext

         else

!        --- First column is blank, hence a message record has been found
!            Write message to file

            if ( debug ) write(irefwr,1) 'iref17, inext', iref17, inext
            write ( iref17, rec = inext ) cards(1:80)
            inext = inext + 1

         end if
         go to 200

      else if ( ios<0 ) then

!     --- End of file reached
!         Fill rest of array ISTART and write to file 4

         istart(nmessg+1) = inext
         inext = 2
         ifrom = 1
         ito   = nmessg+1
         if ( debug ) then
            write(irefwr,1) 'inext, ifrom, ito, iref', &
                             inext, ifrom, ito, iref
         end if  ! ( debug )
         call writfl ( inext, ifrom, ito, idum, iref, istart )

!        --- write first record to messages file

         istart(1) = nmessg
         ifrom = 1
         ito   = 1
         inext = 1
         if ( debug ) then
            write(irefwr,1) 'inext, ifrom, ito, iref', &
                             inext, ifrom, ito, iref
         end if  ! ( debug )
         call writfl ( inext, ifrom, ito, idum, iref, istart )

      else

!     --- Error 4:  Error in input file

         write(iref96,300) imessg,(cards(i:i),i=1,80)
300      format(///' *******Error number 4 in program MAKEF7'///, &
                   ' Error in input file', &
                   ' Preceding message number read is:', i5/ &
                   ' record read is'//1x,80a1)

      end if

      close ( iref96 )

      call finish(0)

      end
