      subroutine intx0206 ( icountkey, icode, table, &
                            tabmax, code, finished, &
                            kposut, nstrng, strout, iretur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 01-06-2013 Allow variable strings
!        version  1.1  date 03-07-2012 Extra debug
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword = text
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kposut, icode, icountkey(*), nstrng, iretur
      character (len=*) code(*), strout(nstrng)
      double precision table(*), tabmax(*)
      logical finished

!     code           i    Table with keywords
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     icode          i    Position number of keyword found in table code
!     icountkey     i/o   Array that counts the number of times a keyword has
!                         been found
!     iretur        i/o   Return value of subroutine INTX02
!                         Possible values:
!                         -2:  end of file has been reached
!                         -1:  error has been found
!                          0:  normal return
!     kposut        i/o   last position used in array strout
!     nstrng         i    Length of array strout
!     strout        i/o   Contains series of strings
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     READSTRINGS    Check if name read corresponds to one of the strings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1628   Keyword found more than one for the same problem
!    1879   Not a text found after the keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0206' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0206'
         write(irefwr,1) 'iretur, icode', iretur, icode
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call asknxt
      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if

      if ( debug ) write(irefwr,1) 'ityprd', ityprd

      if ( ityprd==4 ) then

!     --- text found, check if it is one of the strings in chstrings

         call  readstrings

         if ( debug ) write(irefwr,2) 'realrd', realrd
         if ( ityprd==15 .or. ityprd==16 ) then

!        --- string found

            table(icode) = realrd

         else

!        --- error no string found

            iretur = -1

         end if  ! ( ityprd==15 .or. ityprd==16 )

      else if ( ityprd==5 ) then

         kposut = kposut + 1
         if ( debug ) write(irefwr,1) 'kposut, nstrng', kposut, nstrng
         if ( kposut>nstrng ) go to 1000
         strout(kposut) = string(1:nchar)
         tabmax(icode) = nchar
         table(icode) = kposut

      else

!     --- not a text found

         iretur = -1

      end if  ! ( ityprd==4 )

      if ( iretur==-1 ) then

!     --- not a text found

         call errchr ( code(icode), 1 )
         call errsub ( 1879, 0, 0, 1 )
         finished = .true.

      end if  ! ( iretur==-1 )

1000  call erclos ( 'intx0206' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iretur', iretur
         write(irefwr,*) 'End intx0206'

      end if

      end
