      subroutine intx0211 ( icountkey, icode, table, nout, tabout, &
                            tabmin, tabmax, code, readnx, &
                            itype, iposut, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 10-11-2014 Check tabout
!        version  1.2  date 04-11-2009 Minor change in debug
!        version  1.1  date 21-10-2007 Debug information
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by a series of positive integers
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iposut, nout, icode, icountkey(*), itype(*), ihelp(*)
      character (len=*) code(*)
      double precision table(*), tabmin(*), tabmax(*), tabout(nout)
      logical readnx

!     code           i    Table with keywords
!     icode          i    Position number of keyword found in table code
!     icountkey     i/o   Array that counts the number of times a keyword has
!                         been found
!     ihelp          i    Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
!     iposut        i/o   Position in array tabout
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     nout           i    Number of entries in tabout
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inumlc, num, maxval
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     inumlc         Number of numbers read in a record
!     maxval         Maximum value allowed for the integer read
!     num            Number of numbers read between brackets
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTX05         Read a series of integers
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0211' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0211'
         write(irefwr,1) 'icode, iposut', icode, iposut
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      num = 0
      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if
      table(icode) = iposut+1

!     --- Next read integer numbers until closing bracket or
!         other type of separator is found

      inumlc = min(nint(tabmin(icode)),nout-iposut)
      maxval = nint(tabmax(icode))

      if ( iposut+1>nout ) then
         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000
      end if  ! ( iposut>nout )

      call intx05 ( tabout(iposut+1), inumlc, num, readnx, &
                    itype(icode)==11, maxval, code(icode), &
                    iposut+1 )
      ihelp(icode) = num
      iposut = iposut+num

1000  call erclos ( 'intx0211' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,2 ) 'table(icode)', table(icode)
         call prinrl ( tabout(iposut+1-num), num, 'tabout' )
         write(irefwr,*) 'readnx', readnx
         write(irefwr,1) 'iposut', iposut
         write(irefwr,*) 'End intx0211'

      end if

      end
