      subroutine intx0217 ( icountkey, icode, table, nout, tabout, &
                            tabmin, tabmax, code, finished, readnx, &
                            iposut, ihelp, nmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 10-11-2014 Check tabout
!        version  1.1  date 14-05-2010 Correction 1d
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by a series of series of numbers
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iposut, nout, icode, icountkey(*), ihelp(*), nmax
      character (len=*) code(*)
      double precision table(*), tabmin(*), tabmax(*), tabout(nout)
      logical finished, readnx

!     code           i    Table with keywords
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     icode          i    Position number of keyword found in table code
!     icountkey     i/o   Array that counts the number of times a keyword has
!                         been found
!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of
!                         numbers read
!     iposut        i/o   Position in array tabout
!     nmax           i    maximum number of numbers allowed
!     nout           i    Number of entries in tabout
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ndim, inumlc, num, maxnum
      logical debug, close

!     close          Indicates if the end of a sequence of numbers has been
!                    found (true) or not (false)
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     inumlc         Number of numbers read in a record
!     maxnum         Maximum number of numbers to be read
!     ndim           Dimension of the space.
!     num            Number of numbers read between brackets
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTX04         Read a series of numbers until ) or test is found
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1459   not a series of numbers found after keyword
!    1460   number of numbers between brackets incorrect
!    1462   too much series of numbers read
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0217' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0217'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndim = nint(tabmin(icode))
      nmax = nint(tabmax(icode))

      if ( debug ) write(irefwr,1) 'ndim, nmax', ndim, nmax
      num = 0
      close = .false.
      table(icode) = iposut+1
      readnx = .false.
      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if

      do while ( .not. close )

!     --- end of series has not yet been found
!         Read next series

         if ( ndim==1 ) then
            maxnum = 1        ! maximum number of numbers to be read
         else
            maxnum = ndim+1   ! one extra for check on brackets
         end if  ! ( ndim==1 )

         if ( iposut+1>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         call intx04 ( tabout(iposut+1), maxnum, inumlc, &
                       iposut+1 )
         if ( debug ) call prinrl ( tabout(iposut+1), inumlc, 'tabout' )

         if ( inumlc==0 ) then

!        --- end of series found

            if ( num==0 ) then

!           --- No series found

               call errchr ( code(icode), 1 )
               call errsub ( 1459, 0, 0, 1 )
               call errcrd
               ihelp(icode) = 0

            else

!           --- Fill table

               tabmax(icode) = num

            end if
            close = .true.

         else

!        --- next series found

            if ( inumlc/=ndim ) then

!           --- Not exactly ndim numbers between brackets

               call errint ( inumlc, 1 )
               call errint ( ndim, 2 )
               call errsub ( 1460, 2, 0, 0 )
               call errcrd

            end if
            iposut = iposut + ndim
            if ( iposut>nout ) go to 1000
            num = num + 1
            if ( num>nmax ) then

!           --- Too much series of numbers read

               call errchr ( code(icode), 1 )
               call errint ( num, 1 )
               call errint ( nmax, 2 )
               call errsub ( 1462, 2, 0, 1 )
               call errcrd
               finished = .true.

            end if

         end if

      end do  ! while ( .not. close )
      readnx = .false.

1000  call erclos ( 'intx0217' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx0217'

      end if

      end
