      subroutine intx0219 ( icountkey, icode, table, nout, tabout, &
                            tabmin, tabmax, code, readnx, &
                            iposut, ihelp, nmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 10-01-2015 extra debug
!        version  1.3  date 10-11-2014 Check tabout
!        version  1.2  date 04-11-2009 Minor change in debug
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by a series of numbers
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nout, icode
      integer, intent(inout) :: iposut, icountkey(*), ihelp(*)
      integer, intent(out) :: nmax
      character (len=*), intent(in) :: code(*)
      double precision, intent(in) :: tabmin(*)
      double precision, intent(inout) :: table(*), tabmax(*), &
                                         tabout(nout)
      logical, intent(inout) :: readnx

!     code           i    Table with keywords
!     icode          i    Position number of keyword found in table code
!     icountkey     i/o   Array that counts the number of times a keyword has
!                         been found
!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of
!                         numbers read
!     iposut        i/o   last position used in array tabout
!     nmax           o    maximum number of numbers found
!     nout           i    Number of entries in tabout
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     table         i/o   Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inumlc
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     inumlc         Number of numbers read in a record
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTX02SUB      Interpret series of numbers read
!     INTX04         Read a series of numbers until ) or test is found
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0219' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0219'
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nmax = nint(tabmin(icode))
      if ( debug ) write(irefwr,1) 'nout, nmax', nout, nmax
      table(icode) = iposut+1
      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if
      if ( nout<nmax+1 ) then

         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000

      end if  ! ( nout<nmax+1 )

      call intx04 ( tabout(iposut+1), nmax+1, inumlc, iposut+1 )

!     --- interpret series of numbers

      call intx02sub ( icode, nout, tabout, inumlc, nmax, &
                       tabmax, code, readnx, iposut, ihelp )

1000  call erclos ( 'intx0219' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd', ityprd
         write(irefwr,*) 'readnx', readnx
         write(irefwr,*) 'End intx0219'

      end if

      end
