      subroutine intx0220 ( icountkey, icode, table, nout, tabout, &
                            tabmin, tabmax, code, readnx, nmcode, &
                            itype, iposut, ihelp, subkey, codemn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read single keyword inwhich case the table must be set to -100000
!     or keyword followed by agiven other keyword and a series of
!     integers
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iposut, nout, icode, icountkey(*), itype(*), ihelp(*), &
              nmcode(*)
      character (len=*) code(*), subkey, codemn(*)
      double precision table(*), tabmin(*), tabmax(*), tabout(nout)
      logical readnx

!     code           i    Table with keywords
!     codemn         i    Table with main keywords
!     icode          i    Position number of keyword found in table code
!     icountkey     i/o   Array that counts the number of times a keyword has
!                         been found
!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of
!                         numbers read
!     iposut        i/o   last position used in array tabout
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     nmcode         i    Table containing the number of significant characters
!                         in the main keyword table codemn, including those of
!                         the extra texts
!     nout           i    Number of entries in tabout
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     subkey         i    Sub (second) keyword
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numkey, ih
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ih             Help variable to store a constant temporarily
!     numkey         Number of significant characters in sub keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     INTX0211       Read keyword followed by a series of positive integers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1467   keyword has been found more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0220' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0220'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Read next item

      call asknxt
      if ( ityprd.ne.4 ) then

!     --- not a keyword found, hence single keyword situation

         table(icode) = -100000d0
         readnx = .false.
         go to 1000

      else

!     --- keyword found, check if it is the special one

         ih = nint(-table(icode))
         if ( ih.lt.0 ) then
            call errchr ( code(icode), 1 )
            call errsub ( 1467, 0, 0, 1 )
            call errcrd
            go to 1000
         end if
         subkey = codemn(ih)
         numkey = nmcode(ih)
         if ( string(1:numkey).ne.subkey(1:numkey) ) then

!        --- not special keyword found, hence single keyword situation

            table(icode) = -100000d0
            readnx = .false.
            go to 1000

         else

!        --- Special keyword found, read integers
!            This is the situation of itype(icode) = 11, except
!            for positiveness
!            Jump to itype(icode) = 11

            call  intx0211 ( icountkey, icode, table, nout, tabout, &
                             tabmin, tabmax, code, readnx, &
                              itype, iposut, ihelp )

         end if

      end if

1000  call erclos ( 'intx0220' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx0220'

      end if

      end
