      subroutine intx0222 ( icountkey, icode, table, nout, tabout, &
                            tabmin, tabmax, code, readnx, nmcode, &
                            itype, iposut, subkey, codemn, ihelp, &
                            nmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-11-2014 Check tabout
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by possibly one or two special keywords
!     that are skipped and then by a series of numbers
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/creadinfof90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iposut, nout, icode, icountkey(*), itype(*), ihelp(*), &
              nmcode(*), nmax
      character (len=*) code(*), subkey, codemn(*)
      double precision table(*), tabmin(*), tabmax(*), tabout(nout)
      logical readnx

!     code           i    Table with keywords
!     codemn         i    Table with main keywords
!     icode          i    Position number of keyword found in table code
!     icountkey     i/o   Array that counts the number of times a keyword has
!                         been found
!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of
!                         numbers read
!     iposut        i/o   last position used in array tabout
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     nmax           i    maximum number of numbers allowed
!     nmcode         i    Table containing the number of significant characters
!                         in the main keyword table codemn, including those of
!                         the extra texts
!     nout           i    Number of entries in tabout
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     subkey         i    Sub (second) keyword
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih, inumlc, irep, numkey
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ih             Help variable to store a constant temporarily
!     inumlc         Number of numbers read in a record
!     irep           Number of local repetitions
!     numkey         Number of significant characters in sub keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTX0211       Read keyword followed by a series of positive integers
!     INTX02SUB      Interpret series of numbers read
!     INTX04         Read a series of numbers until ) or test is found
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1461   incorrect keyword found after keyword
!    1467   keyword has been found more than once
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0222' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0222'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call asknxt
      irep = 0
      if ( itype(icode)==22 .and. icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else if ( itype(icode)==22  ) then
         icountkey(icode) = icountkey(icode)+1
      end if
760   if ( ityprd==4 ) then

!     --- keyword found, check if it is the special one

         ih = nint(-table(icode))
         if ( ih<0 ) then
            call errchr ( code(icode), 1 )
            call errsub ( 1467, 0, 0, 1 )
            call errcrd
            go to 1000
         end if
         subkey = codemn(ih+irep)
         numkey = nmcode(ih+irep)
         if ( string(1:numkey)/=subkey(1:numkey) ) then

!        --- string not recognized

            call errchr ( code(icode), 1 )
            call errchr ( string(1:nchar), 2 )
            call errchr ( subkey, 3 )
            call errsub ( 1461, 0, 0, 3 )
            call errcrd
            go to 1000

         end if
         irep = irep + 1
         call asknxt
         if ( irep==1 ) go to 760

      end if

!     --- The rest of the record should contain the numbers to be
!         read. In fact this is the same situation as for
!         itype(icode) = 19, so we jump to that part of the code
!         or in the case 24 as 11, so we jump to that part of the code

      if ( itype(icode)==24 ) then

         readnx = .false.
         call intx0211 ( icountkey, icode, table, nout, tabout, &
                         tabmin, tabmax, code, readnx, &
                         itype, iposut, ihelp )
         go to 1000

      end if
      nmax = nint(tabmin(icode))
      table(icode) = iposut+1

      if ( iposut+1>nout ) then
         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000
      end if  ! ( iposut>nout )

      tabout(iposut+1) = realrd
      infotabout(iposut+1) = ityprd
      if ( ityprd==2 .or. ityprd==3 ) then
         call intx04 ( tabout(iposut+2), nmax, inumlc, &
                       iposut+2 )
         inumlc = inumlc + 1
      else
         inumlc = 0
      end if

!     --- interpret series of numbers

      call intx02sub ( icode, nout, tabout, inumlc, nmax, &
                       tabmax, code, readnx, iposut, ihelp )

1000  call erclos ( 'intx0222' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx0222'

      end if

      end
