      subroutine intx02error ( keywordfound, iretur, minusg, nncode, &
                               code, codemn, finished, nmain, ncodmx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Give error message that keyword is not recognized
!     Help subroutine for intx02ext1
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nncode, iretur, minusg, nmain, ncodmx(*)
      character (len=*) code(nncode), codemn(*)
      logical keywordfound, finished

!     code           i    Table with keywords
!     codemn         i    Table with main keywords
!     finished       i    Indicates if the loop has finished (true) or not
!                         (false)
!     iretur        i/o   Return value of subroutine INTX02
!                         See subroutine INTX02
!     keywordfound   i    If true a keyword has been found
!     minusg         i    Indicates if a minus sign has been found (-1) or
!                         not (1)
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nmain          i    Number of entries in codemn
!     nncode         i    Number of entries in code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1868   Text not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx02error' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx02error'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( .not. keywordfound .and. iretur==-10000 ) then

!     --- iretur = -10000, so we must return with iretur = -10001
!         minus sign must be read again

         if ( minusg==-1 ) lastitem = lastitem-1
         lastitem = lastitem-1  ! last item must be reread
         iretur = -10001
         go to 1000

      end if  ! ( iretur==-10001 )
      do i = 1, abs(nmain)
         call errchr ( codemn(i), i )
      end do
      do i = 1, nncode
         call errchr ( code(i)(1:ncodmx(i)), i+abs(nmain) )
      end do
      call errsub ( 1868, 0, 0, abs(nmain)+nncode )
      call errcrd
      iretur = -1
      finished = .true.

1000  call erclos ( 'intx02error' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx02error'

      end if  ! ( debug )

      end
