      subroutine intx02ext1 ( code, ncode, nncode, itype, table, codemn, &
                              nmcode, nmain, iretur, tabmin, tabmax, &
                              tabout, nout, strout, nstrng, ncodmx, &
                              iseqkeyw, nseqkeyw, errorcode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.7  date 27-03-2015 Extension with type 45
!        version  3.6  date 10-01-2015 Allow expressions
!        version  3.5  date 08-01-2015 Extension with type 44
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read rest of record and check if keywords from the code table followed
!     by a number are available
!     It is assumed that either the record has alrady been read or that ityprd
!     is set equal to 1
!     The subroutine starts with reading the next item but reads only one record
!     If a keyword from the code table is found, the value following it is
!     read and put in the right position of table
!     It is checked if the value read is within a given range
!     If an item is not recognized an error message is given
!     If a main keyword is found or end of record the subroutine returns
!     This is the old subroutine intx02, extended with a sequence array
!     defining in which sequence options are found
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconstf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nncode, ncode(nncode), itype(*), nmain, &
                             nmcode(*), nout, nstrng, ncodmx(nncode)
      integer, intent(inout) :: iretur, errorcode
      integer, intent(out) :: iseqkeyw(*), nseqkeyw
      character (len=*), intent(in) :: code(nncode), codemn(*)
      character (len=*), intent(out) :: strout(nstrng)
      double precision, intent(inout) :: table(nncode), tabmin(nncode), &
                                         tabmax(nncode)
      double precision, intent(out) :: tabout(nout)

!     code           i    Table containing the texts to be compared
!                         n texts have been stored in code
!     codemn         i    Table with main keywords
!     errorcode     i/o   If 0 at input an error message is given if a keyword
!                         is not recognized
!                         If>0, the subroutine does not give an error message, &
!                         but instead returns with -1
!     iretur        i/o   Return code. Possible values
!                         -10000  At input means that intx02ext1 returns if
!                              the first item read is not a keyword
!                              The return code will be -10001 in that case
!                         -11  At input means that asknxt has already been
!                              called before
!                         -10  At input means that no error message is given if
!                              a single real or integer is found
!                              The corresponding value is stored in tabout, next
!                              position
!                              iretur gets the value -10-number of numbers found
!                         -2   End of file has been reached
!                         -1   An error has been detected
!                          0   Normal end of subroutine
!                         >0   Main keyword number iretur has been found
!     iseqkeyw       o    Array of size nseqkeyw that contains the sequence in
!                         which keywords are found in a record
!                         isekeyw(i)>0, means i-th keyword found is the
!                                       iseqkeyw(i)-th keyword in the table
!                         isekeyw(i)<0, means keyword refers to number found
!                                       -isekeyw(i) refers to position in
!                                       tabout
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!                         Possible values (short description):
!                         -1, keyword, skip
!                          0, keyword = integer
!                          1, keyword = real
!                          2, keyword, Set table to 1
!                          3, keyword or keyword = integer
!                          4, keyword or keyword = integer, &
!                             indicate both situations
!                          5, keyword = keyword
!                          6, keyword = 'text'
!                          8, keyword = real, repeatedly
!                          9, ci or -ci, i.e. keyword provided by sign
!                             followed by integer number repeatedly
!                         10, keyword iseq1 = j1, keyword iseq2 = j2, ...
!                         11, keyword = i1, i2, i3, ...  all positive integer
!                         12, keyword iseq1 = j1, keyword iseq2 = j2, ...
!                             or keyword = j1 (iseq1 is set to 1)
!                         13, keyword = keyword1 [keyword2], the last may
!                             be empty
!                         14, c1, -c3, c4, -c9, c5, i.e. list of keywords
!                             followed by sequence number (or c1 to c2)
!                         15, c1 v1 c2 v2 List of keyword followed by sequence
!                             number followed by another keyword followed by
!                             sequence number and this repeatedly
!                         16, keyword = real
!                             or keyword (value = real)
!                             or keyword (func = i)
!                             or keyword (time_func = i)
!                         17, keyword = ((x1,y1,..), (x2,y2,..)  ... (xn,yn,..))
!                         18, keyword = P1, P2, ... (or P1 to P2)
!                         19, keyword = ( x1, y1, z1 )
!                             All may be expressions
!                         20, keyword or keyword keyword = i1, i2, ...
!                         21, keyword (ax+by+cz=d)
!                         22, keyword [keyword1] [keyword2] = ( x1, y1, z1 )
!                         23, keyword keyword  set tabel to 1
!                         24, keyword [keyword1] [keyword2] = ( i1, i2, i3 )
!                         25, keyword iseq1 = real
!                             or keyword iseq1 (value = real)
!                             or keyword iseq1 (func = i)
!                             or keyword iseq1 (time_func = i)
!                         26, keyword = complex
!                             or keyword (value = complex)
!                             or keyword (func = i)
!                             or keyword (time_func = i)
!                             with complex = real or (real,real)
!                         27, keyword = series of:
!                             real
!                             or keyword (value = real)
!                             or keyword (func = i)
!                             or keyword (time_func = i)
!                         28, keyword = i1, i2, i3, ...  all integer
!                         29, keyword iseq = [keyword1] i1, i2, i3, ...  all
!                             integer
!                         30, keyword iseq1 = c1, keyword iseq2 = c2, ...
!                         31, keyword [iseq] = c1, c2, ... or c1 to c2
!                             for example curves 1 = c1 to c2 or curves = c1
!                         32, keyword [(i1, i2, ..)] = sj, sn, ... or
!                             keyword = sj, sn, ...[, nodes =(i1, i2, ..)]
!                         33, keyword iseq = ( i1, i2, i3, ... ) all integer
!                         34, keyword iseq1 = j1, keyword iseq2 = j2, ...
!                             Different input/output than 10
!                         35, keyword iseq1 = real keyword iseq2 = real
!                             or keyword iseq1 (value = real)
!                             or keyword iseq1 (func = i)
!                             or keyword iseq1 (time_func = i)
!                             or keyword iseq2 (value = real)
!                             or keyword iseq2 (func = i)
!                             or keyword iseq2 (time_func = i)
!                         36, keyword = integer or %name
!                         37, keyword iseq [to iseq1] = [( i1, i2, i3, ... )]
!                             all integer
!                         38, keyword = integer or vector name
!                         39, keyword = integer or scalar name
!                         40, keyword = integer or vector name repeatedly
!                         41, keyword iseq [to iseq1] = [keyword1] i1, i2, i3, &
!                             ...  all integer
!                         42, keyword iseq [to iseq1]
!                         43, keyword = constant, scalar or vector name
!                         44, see 2, but after reading the keyword, reading
!                             is stopped
!                         45, keyword = integer or scalar name repeatedly
!                         Possible values (long description):
!                         itype(i) = -1, no number, the information must be
!                                       skipped
!                                       input:  code(i) keyword
!                                       output: - &
!                         itype(i) = 0, number i is of type integer
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value
!                                               tabmax(i) maximum value
!                                       output: table(i)  value
!                         itype(i) = 1, number i is of type real
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value
!                                               tabmax(i) maximum value
!                                       output: table(i)  value
!                         itype(i) = 2, no number but in table the value 1 must
!                                       be set
!                                       input:  code(i) keyword
!                                       output: table(i): 1
!                         itype(i) = 3, either an integer number or no number
!                                       at all
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value
!                                               tabmax(i) maximum value
!                                       output: table(i)  value or nothing
!                         itype(i) = 4, either an integer number or no number
!                                       at all
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value
!                                               tabmax(i) maximum value
!                                       output: table(i) = -1000 keyword without
!                                                          number
!                                               table(i) = iposut keyword with
!                                                          number
!                                               tabout(iposut) = value
!                         itype(i) = 5, number is given in the form of a keyword
!                                       input:  code(i) keyword
!                                               tabmin(i) starting position
!                                                         allowed
!                                                         keywords in codemn
!                                               tabmax(i) end position allowed
!                                                         keywords in codemn
!                                               codemn(tabmin(i) to tabmax(i))
!                                       output: table(i) = relative position of
!                                                          keyword found in
!                                                          codemn i.e. from 1 to
!                                                          tabmax(i)-tabmin(i)+1
!                         itype(i) = 6, text between quotes
!                                       input:  code(i) keyword
!                                       output: table(i) kposut
!                                               if ( >100000000 )
!                                               reference to chstrings
!                                               tabmax(i) number of characters
!                                               in text
!                                               strout(kposut) = string
!                         itype(i) = 8, special situation of keyword followed
!                                       by number, repeatedly
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value of
!                                                         reals
!                                               tabmax(i) maximum value of
!                                                         reals
!                                       output: table(i) = iposut
!                                               tabmax(i) = m
!                                                   (number of reals read)
!                                               tabout(iposut to iposut+m-1) = &
!                                                   values
!                         itype(i) = 9, special situation of keyword followed
!                                       by integer number, repeatedly
!                                       the keyword may be provided by a sign, &
!                                       like -c8
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value of
!                                                         integers
!                                               tabmax(i) maximum value of
!                                                         integers
!                                       output: table(i) = iposut
!                                               tabmax(i) = m
!                                                   (number of integers read)
!                                               tabout(iposut to iposut+m-1) = &
!                                                   values
!                         itype(i) =10, special situation of keyword followed
!                                       by sequence number followed by number, &
!                                       for example inpvc i = j
!                                       In this case the number positions
!                                       in the arrays code itype etc. is equal
!                                       to the range of the sequence numbers
!                                       input:  code(i to i+max-1) keyword
!                                               max is tabmax(i)-tabmin(i)+1
!                                               tabmin(i) minimum value i
!                                               tabmax(i) maximum value i
!                                               tabmin(i+1) minimum value j
!                                               tabmax(i+1) maximum value j
!                                               itype(i+1) = 0: j is integer
!                                               itype(i+1) = 1: j is real
!                                       output: table(i to i+max-1) contain the
!                                               values of j for respectively
!                                               k=tabmin(i) to k=tabmax(i)
!                         itype(i) =11, special situation of keyword followed by
!                                       series of positive integer numbers
!                                       Brackets are treated as separator
!                                       input:  code(i) keyword
!                                               tabmin(i) maximum number of
!                                                         integers
!                                               tabmax(i) maximum value of
!                                                         integers
!                                       output: table(i) = iposut
!                                               tabmax(i) = m
!                                                    (number of integers read)
!                                               tabout(iposut to iposut+m-1) = &
!                                                     values
!                         itype(i) =12, special situation of keyword followed
!                                       by sequence number followed by number, &
!                                       for example inpvc i = j or alternatively
!                                       keyword followed by number
!                                       for example inpvc = j. i = 1 in that
!                                       case
!                                       Rest see itype(i) = 10
!                         itype(i) =13, special situation of keyword followed
!                                       by one or two keywords, for example:
!                                       renumber = cuthill profile
!                                       input:  code(i) keyword
!                                               code(i+1) same keyword
!                                               itype(i+1) ipostyp
!                                               itype(ipostyp+2*j-2) ipmn1_j
!                                               itype(ipostyp+2*j-1) ipmn2_j
!                                               tabmin(i) ipmn1_1
!                                               tabmax(i) ipmn2_1
!                                               codemn(ipmn1_1 to ipmn2_1)
!                                        contains the list of first keywords
!                                               allowed codemn(ipmn1_j to
!                                               ipmn2_j) contains the list
!                                               of second keywords allowed
!                                               corresponding to first keyword
!                                               "j", where j is the relative
!                                               seq. number
!                                               If to first keyword j no second
!                                               keyword corresponds ipmn1_j and
!                                               ipmn2_j must be 0
!                                       output: table(i) = j1
!                                               table(i+1) = j2
!                                               j1, j2 denote the relative ref.
!                                               number corresponding to first
!                                               resp. second keyword
!                         itype(i) =14, special situation of keyword followed
!                                       by number, followed by same keyword
!                                       followed by number and so on, for
!                                       example: p1, p3, p5, p8, p2
!                                       The keywords may be preceded by a minus
!                                       sign, hence c1, c3, -c5, c8 etc., &
!                                       in which the sequence number
!                                       gets the minus sign
!                                       input:  code(i) keyword
!                                               tabmin(i) maximum number of
!                                                         integers
!                                               tabmax(i) maximum value of
!                                                         integers
!                                       output: table(i) = iposut
!                                               tabmax(i) = m
!                                                   (number of integers read)
!                                               tabout(iposut to iposut+m-1) = &
!                                                   values
!                         itype(i) =15, special situation of keyword followed
!                                       by number, followed by same keyword
!                                       followed by number etc.
!                                       However, in this case it is possible
!                                       that the keyword number combination
!                                       is followed by other keywords, before it
!                                       is followed by the next keyword number
!                                       combination
!                                       Example: c1 v1 + c2 v2
!                                       In this case both c and v get type
!                                       number 15
!                                       In this case in tabout exactly tabmin(i)
!                                       positions are reserved. The rest of the
!                                       input is exactly the same as for
!                                       itype(i) = 14
!                         itype(i) =16, special situation of keyword followed
!                                       by either a value or the possibilities
!                                       value = v, func = ifunc, or
!                                       time_func = ifunc
!                                       input:  code(i) keyword
!                                       output: table(i) = ifunc (>0) or
!                                               -iposut
!                                               tabout(iposut) = value
!                                               if time_func = ifunc is given
!                                               table(i) gets the value
!                                               -ifunc-100
!                         itype(i) =17, special situation of keyword followed by
!                                       series of series of numbers between
!                                       brackets. Each series of local numbers
!                                       itself must be between brackets.
!                                       The number o fnumbers in the inner
!                                       series must be constant.
!                                       input:  code(i) keyword
!                                               tabmin(i) number of numbers in
!                                                         inner series
!                                               tabmax(i) maximum number of
!                                                         series
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m-1)
!                                               values
!                                               tabmax(i) m
!                         itype(i) =18, special situation of keyword
!                                       followed by a series of the same
!                                       keywords followed by sequence numbers
!                                       hence we have the situation keyword
!                                       followed by possibility 14
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m-1)
!                                               values
!                                               tabmax(i) m
!                         itype(i) =19, special situation of keyword followed by
!                                       a series of numbers possibly enclosed by
!                                       brackets
!                                       input:  code(i) keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m-1)
!                                               values
!                                               tabmax(i) m
!                         itype(i) =20, special situation of keyword which has
!                                       to be set or the same keyword followed
!                                       by another special keyword
!                                       followed by integers
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i)  iposut
!                                               or -100000 if subkeyword is not
!                                               present
!                                               tabout(iposut to iposut+m-1)
!                                               values
!                                               tabmax(i) m
!                         itype(i) =21, special situation of keyword followed by
!                                       (ax+by+cz=d), with a, b, c and d numbers
!                                       if a, b or c is omitted 1 or -1
!                                       is supposed
!                                       At output table(i) contains the starting
!                                       position of the 4 numbers a,b,c and d in
!                                       that sequence in tabout
!                         itype(i) =22, special situation of keyword followed by
!                                       possibly some other special keywords
!                                       followed by a series of numbers
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m)
!                                               values read
!                                               tabmax(i) m
!                         itype(i) =23, special situation of keyword followed
!                                       by a special keyword that must
!                                       be skipped
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special
!                                               sub keyword
!                                       output: table(i): 1
!                         itype(i) =24, special situation of keyword followed by
!                                       possibly some other special keywords
!                                       followed by a series of positive
!                                       integers
!                                       See 22
!                         itype(i) =25, special situation of keyword followed by
!                                       sequence number followed by either a
!                                       value or the possibilities
!                                       value = v, func = ifunc, &
!                                       or time_func = ifunc (See 10 and 16)
!                                       input:  code(i to i+max-1) keyword
!                                               max is tabmax(i)-tabmin(i)+1
!                                               tabmin(i) minimum value i
!                                               tabmax(i) maximum value i
!                                       output: table(k) = ifunc (>0)
!                                               or -iposut
!                                               tabout(iposut) = value
!                                               if time_func = ifunc is given
!                                               table(k) gets the value
!                                               -ifunc-100 for respectively
!                                               k=tabmin(i) to k=tabmax(i)
!                         itype(i) =26, special situation of keyword followed
!                                       by either a value or the possibilities
!                                       value = v, func = ifunc, or
!                                       time_func = ifunc with v complex
!                                       input:  code(i) keyword
!                                       output: table(i) = ifunc (>0) or -iposut
!                                               tabout(iposut,iposut+1) = value
!                                               if time_func = ifunc is given
!                                               table(i) gets the value
!                                               -ifunc-100
!                         itype(i) =27, special situation of keyword followed
!                                       by a series of: value or value = v, &
!                                       func = ifunc, or time_func = ifunc
!                                       input:  code(i) keyword
!                                               tabmin(i) number of values
!                                               expected
!                                       output: table(i) = iposut
!                                               tabout(iposut+2*i-2) = ifunc_i
!                                               if time_func = ifunc is given
!                                               -ifunc_i-100
!                                               if value is given then 0
!                                               tabout(iposut+2*i-1) = value
!                         itype(i) =28, special situation of keyword followed by
!                                       series of integer numbers. See 11
!                         itype(i) =29, special situation of keyword followed by
!                                       sequence number possibly followed by
!                                       another keyword followed by integers
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                                         |numbers|
!                                       output: table(i) = iposut
!                                               tabmax(i) = m (number of
!                                               integers read)
!                                               tabout(iposut to iposut+m-1) = &
!                                               values
!                         itype(i) =30, special situation of keyword followed
!                                       by sequence number followed by
!                                       keyword and integer number, &
!                                       for example obs i = cj or alternatively
!                                       keyword followed by keyword and integer
!                                       number
!                                       for example obs = cj. i = 1 in that case
!                                       input:  itype(i+1) = iposmain
!                                               codemn(iposmain) special
!                                               sub keyword
!                                       Rest see itype(i) = 10, 12
!                         itype(i) =31, special situation of keyword followed by
!                                       possibly an integer followed by
!                                       followed by a series of the same
!                                       keywords followed by sequence numbers
!                                       hence we have the situation keyword
!                                       possibly followed by an integer followed
!                                       by possibility 18
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special
!                                               sub keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m-1)
!                                               values
!                                               tabmax(i) m
!                                               tabmin(i) integer after keyword
!                                                         if not found: 0
!                         itype(i) =32, special situation of keyword followed by
!                                       possibly a series of integers
!                                       followed by one or two of the same
!                                       keywords followed by sequence numbers
!                                       followed by possibly the text nodes
!                                       followed by integers
!                                       If nodes is found the first series may
!                                       not be used
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read in the
!                                               nodes part
!                                               tabmax(i) maximum value of the
!                                               numbers after the sub keyword
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m+1)
!                                               values
!                                               the first n contain the number
!                                               after the subkeyword
!                                               the last m the values
!                                               after nodes
!                                               tabmax(i) n*10000+m
!                         itype(i) =33, special situation of keyword followed by
!                                       sequence number followed by integers
!                                       input:  code(i) keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i) = iposut
!                                               tabmax(i) = m
!                                               (number of integers read)
!                                               tabout(iposut to iposut+m-1) = &
!                                               values
!                         itype(i) =34, special situation of keyword followed
!                                       by sequence number followed by number, &
!                                       for example inpvc i = j
!                                       In this case 2 positions
!                                       in the arrays code itype etc. are used
!                                       i must be between 1 and max
!                                       input:  code(i to i+1) keyword
!                                               tabmax(i) maximum value i
!                                               tabmin(i+1) minimum value j
!                                               tabmax(i+1) maximum value j
!                                               itype(i+1) = 0: j is integer
!                                               itype(i+1) = 1: j is real
!                                               table(i)<=0
!                                       output: table(i) iposut
!                                               tabout(iposut to iposut+max-1)
!                                               values of j
!                                               If sequence is not read
!                                               tabout(iposut+i-1) = -1d30
!                         itype(i) =35, special situation of keyword followed by
!                                       sequence number followed by either a
!                                       value or the possibilities
!                                       value = v, func = ifunc, &
!                                       or time_func = ifunc (See 10 and 16)
!                                       then
!                                       followed by the same keyword followed by
!                                       sequence number followed by either a
!                                       value or the possibilities
!                                       value = v, func = ifunc, &
!                                       or time_func = ifunc (See 10 and 16)
!                                       then
!                                       input:  code(i ) keyword
!                                               tabmin(i) minimum value i
!                                               tabmax(i) maximum value i
!                                       output: table(i) = iposut
!                                               tabout(iposut to iposut+3) = &
!                                               values
!                         itype(i) =36, number i is of type integer or %name
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value
!                                               tabmax(i) maximum value
!                                       output: table(i)  value
!                         itype(i) =37, special situation of keyword followed by
!                                       sequence number to other sequence number
!                                       followed by integers between brackets
!                                       The second sequence number may absent as
!                                       well as the numbers between brackets
!                                       input:  code(i) keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i) = iposut
!                                               tabmax(i) = m (number of
!                                               integers read)
!                                               tabout(iposut to iposut+m-1) = &
!                                               values
!                                               The first two always refer to
!                                        the sequence numbers
!                         itype(i) = 38, number i is vector name or number
!                                       input:  code(i) keyword
!                                       output: table(i)  value
!                         itype(i) = 39, number i is scalar name or number
!                                       input:  code(i) keyword
!                                       output: table(i)  value
!                         itype(i) = 40, number i is vector name or number
!                                        may appear more than once
!                                       input:  code(i) keyword
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m-1) = &
!                                               values
!                                               tabmax(i) number of vectors
!                         itype(i) =41, special situation of keyword followed by
!                                       sequence number possibly followed by
!                                       another keyword followed by integers
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                                         |numbers|
!                                       output: table(i) = iposut
!                                               tabmax(i) = m (number of
!                                               integers read)
!                                               tabout(iposut to iposut+m-1) = &
!                                               values
!                                               The first two always refer to
!                                               the sequence numbers
!                         itype(i) =42, special situation of keyword followed by
!                                       sequence number or iseq1 to iseq2
!                                       input:  code(i) keyword
!                                               tabmax(i) maximum value of
!                                                         |numbers|
!                                       output: table(i) = iposut
!                                               tabout(iposut to iposut+1) = &
!                                               values
!                         itype(i) = 43, number i is scalar/vector name or number
!                                       input:  code(i) keyword
!                                       output: table(i)  position in set
!                                               of constants/scalars/vectors
!                                               scalars are counted from first
!                                               position in constants
!                                               in case of vector 10000 is added
!                                               to the position
!                         itype(i) = 45, number i is scalar name or number
!                                        may appear more than once
!                                       input:  code(i) keyword
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m-1) = &
!                                               values
!                                               tabmax(i) number of scalars
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     ncodmx         i    Table containing the maximum number of characters in
!                         the keyword table code. If the keyword contains more
!                         characters a warning is given.
!     nmain          i    Number of main keywords stored in codemn
!                         codemn itself may be larger because of the possibility
!                         keyword = text
!                         If nmain<0 and a main keyword has been found, &
!                         icolum is reset to the preceding value, which implies
!                         that the main keyword may be read again
!     nmcode         i    Table containing the number of significant characters
!                         in the main keyword table codemn, including those of
!                         the extra texts
!     nncode         i    Number of texts stored in code
!     nout           i    Length of array tabout
!     nseqkeyw       o    Number of entries in iseqkeyw
!     nstrng         i    Length of array strout
!     strout         o    Contains series of strings, see itype
!     table         i/o   Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax         i    Contains the maximum values allowed in the table
!                         In case of the possibility keyword = text it contains
!                         the starting positions of the texts in codemn
!     tabmin         i    Contains the minimum values allowed in the table
!                         In case of the possibility keyword = text it contains
!                         the end positions of the texts in codemn
!     tabout         o    Contains series of numbers, see itype
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished, readnx, debug, keywordfound
      integer maxcode, maxmultiple, maxrepeat
      parameter ( maxcode = 200, maxmultiple = 5, maxrepeat = 1000 )
      integer icode, iposut, minusg, ispecs, ihelp(maxcode), &
              kposut, icountkey(maxcode), lastitemsav, &
              pointar(nncode), numberkeyw(maxmultiple), icount
      double precision valuekeyw(maxrepeat, maxmultiple)

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     icode          Position number of keyword found in table code
!     icount         Counter, number of keywords with itype = 8 or 9
!     icountkey      Array that counts the number of times a keyword has been
!                    found
!     ihelp          Help array to store number of appearances of keywords that
!                    may be used more than once or number of numbers read
!     iposut         last position used in array tabout
!     ispecs         Help variable to save the value of ispecl
!     keywordfound   If true a keyword has been found
!     kposut         last position used in array strout
!     lastitemsav    Help parameter to save the value of lastitem
!     maxcode        Maximum length of array code allowed
!     maxmultiple    Declared size of array pointar
!     maxrepeat      Declared size of first entry of array numberkeyw
!     minusg         Indicates if a minus sign has been found (-1) or not (1)
!     numberkeyw     Array of size maxmultiple containing the number of values
!                    stored in array valuekeyw for each type
!                    The sequence is defined by array pointar
!     pointar        array of size n containing the sequence numbers of
!                    all keywords with respect to arrays numberkeyw and
!                    valuekeyw
!                    if 0, no reference is necessary
!     readnx         Indicates if the next item has not been read (true) or has
!                    been (false)
!     valuekeyw      Array of size maxrepeat x maxmultiple containing
!                    all values corresponding to the special keywords of
!                    type 8 and 9
!                    valuekeyw(i,j) corresponds to ith value for j-th keyword
!                    where the sequence of pointar is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTX02COFSCAL  Special case: coefficient is a scalar
!     INTX02ERROR    Give error message that keyword is not recognized
!     INTX02FILLTAB  Fill array tabmax for subroutine intx02ext1
!     INTX02INIT     Initialize some parameters and perform some checks for
!                    intx02ext1
!     INTX02KEYW     Check if string read is (main) keyword
!     INTX02MINUS    Check for the presence of minus sign and closing brackets
!     INTX02NEXT     Keyword found, read next item
!     INTX02NOKEYW   Treat the situation in intx02ext1 that no keyword is found
!     INTX02NUMBER   Read number in case iretur = -10
!     INTX02SKIP     Read next item in case iretur#11
!     INTX02TYPE89   There are keywords of type 8 and 9
!                    Fill information read into table, tabmax and tabout
!     PRININ         print 1d integer array
!     READNEXTITEM   Read next item and interpret
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    finished := false
!    Clear array ihelp
!    While ( not finished ) do
!       read next item
!       if ( end of file found ) then
!          iretur := -2
!          finished := true
!       else if ( end of record found ) then
!          iretur := 0
!          finished := true
!       else if ( item # keyword ) then
!          Error message
!          iretur := -1
!          finished := true
!       else if ( item = main keyword ) then
!          set iretur
!          finished := true
!       else if ( item = keyword from code ) then
!          icode := sequence number of keyword in code table
!          if ( itype(icode) # 2 and # -1 ) then
!             read next item
!             if ( itype(icode) = 0, 1, 3 or 4 and next item is a number ) then
!                Check type of number
!                Fill number in table
!             else if ( itype(icode) = 5 or 13 and next item is a text ) then
!                Check type of text
!                Fill number in table
!                if ( itype(icode) = 13 ) then
!                   If ( second keyword should be present ) then
!                      Read next keyword
!                      Check type of text
!                      Fill number in table
!             else if ( itype(icode) = 6 ) then
!                Read and store text
!             else if ( itype(icode) = 3 or 4 and no number found ) then
!                skip
!             else if ( itype(icode) = 10, 12, 30 and 34
!                       and next item is integer number ) then
!                Check sequence number
!                Read next item
!                Check if this is a number in the correct range
!                Fill number in table
!             else if ( itype(icode) = 11 or 28 or 29 ) then
!                Read and check series of integers between brackets
!                Store integers in tabout
!                Set table(i) and tabmax(i)
!             else if ( itype(icode) = 14, 18 or 31 ) then
!                if ( itype(icode) = 18 or 31 ) then
!                   Set keyword to subkeyword
!                While ( next item = keyword with number icode ) do
!                   Read next item (must be integer) and store in tabout
!                   Read next item
!                Set table(i) and tabmax(i)
!             else if ( itype(icode) = 15 ) then
!                if ( ihelp(icode)=0 ) then
!                   Fill table(icode)
!                Read next item (must be integer) and store in tabout
!                Raise ihelp
!             else if ( itype(icode) = 16 ) then
!                Read next item
!                if ( item = 'value' or 'func' or 'time_func' ) then
!                   read next item
!                Fill table and if necessary tabout
!             else if ( itype(icode) = 17 ) then
!                Read series of numbers until new item is found
!                Fill table and tabout
!             else if ( itype(icode) = 19 ) then
!                Read series of numbers until new item is found
!                Fill table and tabout
!             else if ( itype(icode) = 20 ) then
!                Check if keyword is followed by special keyword
!                If ( not special keyword found ) then
!                   Set table(icode)
!                else
!                   Read and store integers
!             else if ( itype(icode) = 21 ) then
!                Read ax+by+sz=d
!                Store result in posout
!             else if ( itype(icode) = 22 or 24 ) then
!                Read next item
!                if ( next item = special keyword ) then
!                   read next item
!                if ( next item = special keyword ) then
!                   read next item
!                Read numbers and store in posout
!             else if ( itype(icode) = 23 ) then
!                Read next item
!                If ( next item is special keyword ) then
!                   skip
!                Set table(icode)
!             else if ( itype(icode) = 25 and next item is integer number ) then
!                Check sequence number
!                Read next item
!                if ( item = 'value', 'func' or 'time_func' ) then
!                   read next item
!                Fill table and if necessary tabout
!             else if ( itype(icode) = 26 ) then
!                Repeat until different item is found
!                   Read next item
!                   if ( item = 'value' or 'func' or 'time_func' ) then
!                      read next item
!                   Fill tabout and tabmax
!             else if ( itype(icode) = 32 ) then
!                Read information after keyword
!                Set table(icode) and fill tabmax and tabout
!             else
!                error message
!                iretur := -1
!                finished := true
!          else if ( itype(icode) = 2 ) then
!             Put a 1 in the table
!       else ( keyword not recognized )
!          iretur := -1
!          finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx02ext1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx02ext1'
         write(irefwr,1) 'iretur, nncode, ityprd, lastitem', &
                          iretur, nncode, ityprd, lastitem
         write(irefwr,1) 'istop, nout', istop, nout
         call prinin ( itype, nncode, 'itype' )
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d18.10) )

      end if
      if ( ierror/=0 ) go to 1000

      finished = .false.
      readnx = .false.
      keywordfound = .false.
      lastitemsav = lastitem

!     --- keyitems(1:nitems) = 0            ! mark as not found

      if ( iretur/=-11 ) call intx02skip()  ! read next item

      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

!     --- Initialize some arrays and parameters
!         Perform some checks
!         Compute icount

      call intx02init ( nncode, maxcode, ihelp, icountkey, nout, &
                        pointar, numberkeyw, maxmultiple, iposut, &
                        ispecs, kposut, nseqkeyw, icount, itype )

!     --- Loop over all items in record

      do while ( .not. finished )

!     --- read next item

         if ( ierror/=0 ) go to 1000
         minusg = 1
         ispecl = 4
         if ( debug ) then
            write(irefwr,*) 'readnx ', readnx
            write(irefwr,1) 'istop, lastitem/1a', istop, lastitem
            write(irefwr,1) 'iposut, ityprd', iposut, ityprd
            if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         if ( readnx .or. ityprd==20 ) then

!        --- Read next item, might be an expression
!            However keywords have higher priority than reals

            call readnextitem ( code, ncode, nncode, codemn, nmcode, &
                                nmain, iretur, ncodmx, finished, icode )
            if ( debug ) then

               write(irefwr,*) 'after readnextitem'
               write(irefwr,1) 'ityprd, istop, lastitem/2', &
                                ityprd, istop, lastitem
               if ( ityprd==4 ) &
                  write(irefwr,*) 'string ', string(1:nchar)

            end if  ! ( debug )

            if ( ityprd==19 .and. errorcode==1 ) then

!           --- Expression found (not allowed), set errorcode negative

               errorcode = -1
               go to 1000

            end if  ! ( ityprd==19 )

         end if  ! ( readnx .or. ityprd==20 )
         if ( ierror/=0 ) go to 1000

         if ( ityprd==11 .and. (itype(1)==16 .or. itype(1)==26 ) ) then

!        --- Special case of a scalar found in combination with a
!            coefficient

            if ( debug ) write(irefwr,*) 'before intx02cofscal'
            call intx02cofscal ( iseqkeyw, nseqkeyw, readnx, table(1) )
            if ( debug ) write(irefwr,*) 'coefficient is scalar'
            go to 1000

         else if ( ityprd==100 ) then

!        --- Special case of a vector found

            if ( debug ) write(irefwr,*) 'vector found'
            ityprd = 2
            call intx02next ( code, ncode, nncode, itype, table, &
                              codemn, nmcode, iretur, tabmin, &
                              tabmax, tabout, nout, strout, &
                              nstrng, iseqkeyw, nseqkeyw, icode, &
                              keywordfound, minusg, icountkey, &
                              pointar, finished, ihelp, maxrepeat, &
                              numberkeyw, readnx, maxmultiple, &
                              valuekeyw, iposut, kposut, lastitemsav, &
                              errorcode )
            go to 1000

         end if  ! ( ityprd==11 .and. (itype(1)==16 .or. itype(1)==26 ) )

!        --- Check for the presence of a minus sign or closing brackets
!            The last ones are skipped and a new item is read

         if ( debug ) write(irefwr,*) 'before intx02minus'
         call intx02minus ( readnx, minusg )
         if ( ierror/=0 ) go to 1000
         if ( debug ) then
            write(irefwr,1) 'iretur, ityprd, minusg, lastitem/3', &
                             iretur, ityprd, minusg, lastitem
            if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)

         end if  ! ( debug )

         if ( ityprd==0 ) then

!        --- end of file found< stop reading

            if ( iretur>-10 ) iretur = -2
            finished = .true.

         else if ( ityprd==1 ) then

!        --- end of record found, stop reading

            if ( iretur>-10 ) iretur = 0
            finished = .true.

         else if ( iretur<=-10 .and. iretur>-10000 .and. &
                 (ityprd==2 .or. ityprd==3 .or. ityprd==11 .or. &
                  ityprd==19) ) then

!        --- iretur = -10 and number found, fill tabout

            if ( debug ) write(irefwr,*) 'before intx02number'
            call intx02number ( tabout, iposut, iseqkeyw, nseqkeyw, &
                                nout, iretur )
            if ( ierror/=0 ) go to 1000
            if ( debug ) write(irefwr,1) 'iretur', iretur

         else if ( ityprd/=4 .and. ityprd/=100 ) then

!        --- item # keyword: give error message or special return

            call intx02error ( keywordfound, iretur, minusg, nncode, &
                               code, codemn, finished, nmain, ncodmx )
            go to 1000

         else

!        --- Keyword found, check if it is a main keyword

            if ( debug ) write(irefwr,*) 'before intx02keyw'
            call intx02keyw ( code, ncode, nncode, codemn, nmcode, &
                              nmain, iretur, ncodmx, finished, &
                              icode, &
                              itype, nout, tabout, iposut, icountkey, &
                              table, readnx, tabmax )
            if ( debug ) then
               write(irefwr,*) 'after intx02keyw'
               write(irefwr,1) 'icode, lastitem, iretur', &
                                icode, lastitem, iretur
               write(irefwr,1) 'ityprd, iposut, index_rdarr(3)', &
                                ityprd, iposut, index_rdarr(3)
               write(irefwr,2) 'realrd', realrd
               write(irefwr,*) 'finished', finished
            end if  ! ( debug )
            if ( ierror/=0 ) go to 1000

            if ( .not. finished ) then

!           --- not main keyword found

               if ( icode>0 ) then

!              --- keyword found

                  if ( itype(icode)==44 ) then

!                 --- itype(icode) = 44, keyword found, return

                     table(icode) = 1d0
                     go to 1000

                  end if  ! ( itype(icode)==44 )

                  if ( debug ) write(irefwr,*) 'before intx02next'
                  call intx02next ( code, ncode, nncode, itype, table, &
                                    codemn, nmcode, iretur, tabmin, &
                                    tabmax, tabout, nout, strout, &
                                    nstrng, iseqkeyw, nseqkeyw, icode, &
                                    keywordfound, minusg, icountkey, &
                                    pointar, finished, ihelp, maxrepeat, &
                                    numberkeyw, readnx, maxmultiple, &
                                    valuekeyw, iposut, kposut, &
                                    lastitemsav, errorcode )
                  if ( debug ) then
                     write(irefwr,*) 'after intx02next'
                     write(irefwr,1) 'iretur, lastitem', &
                                      iretur, lastitem
                  end if  ! ( debug )

               else

!              --- keyword not recognized
!                  might be an expression or some special case, &
!                  but also an error

                  if ( debug ) write(irefwr,*) 'before intx02nokeyw'
                  call intx02nokeyw ( keywordfound, iretur, minusg, &
                                      nout, iposut, iseqkeyw, tabout, &
                                      nseqkeyw, finished, errorcode, &
                                      code, codemn, nncode, itype, &
                                      nmain, readnx, table )
                  if ( ierror/=0 .or. errorcode/=0 ) go to 1000

               end if  ! ( icode>0 )

            end if  ! ( .not. finished )

         end if  ! ( ityprd==0 )

         lastitemsav = lastitem

      end do  ! while ( .not. finished )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'icount', icount
      if ( icount>0 ) then

!     --- icount>0, hence there are keywords of type 8 and 9
!         Fill information read into table, tabmax and tabout

         call intx02type89 ( numberkeyw, icount, nncode, table, &
                             tabmax, tabout, pointar, valuekeyw, &
                             maxrepeat, iposut, nout )

      end if  ! ( icount>0 )

!     --- Fill tabmax

      call intx02filltab ( nncode, tabmax, tabmin, ihelp, itype, &
                           code )

1000  call erclos ( 'intx02ext1' )
      if ( ierror/=0 ) iretur = -1
      ispecl = ispecs
      if ( debug ) then
         write(irefwr,1) 'istop, lastitem, ityprd', &
                          istop, lastitem, ityprd
         write(irefwr,*) 'End intx02ext1'
      end if  ! ( debug )

      end
