      subroutine intx02filltab ( nncode, tabmax, tabmin, ihelp, itype, &
                                 code )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array tabmax for subroutine intx02ext1
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ihelp(*), nncode, itype(*)
      double precision tabmin(nncode), tabmax(nncode)
      character (len=*) code(nncode)

!     code           i    Table with keywords
!     ihelp          i    Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     nncode         i    Number of entries in code
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1440   Keyword occurs too many times
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx02filltab' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx02filltab'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nncode

         if ( ihelp(i)>0 ) then

!        --- ihelp(i)>0, fill tabmax(i)

            tabmax(i) = ihelp(i)
            if ( itype(i)==32 ) ihelp(i) = mod(ihelp(i),10000)
            if ( (itype(i)==29 .or. itype(i)==33 ) .and. &
                  ihelp(i)>nint(tabmin(i))+1 .or. &
                 (itype(i)<29 .or. itype(i)==30 .or. &
                  itype(i)==32 .or. itype(i)>33 ) .and. &
                  ihelp(i)>nint(tabmin(i)) ) then

!           --- Error: more numbers found than permitted

               call errchr ( code(i), 1 )
               call errint ( ihelp(i), 1 )
               call errint ( nint(tabmin(i)), 2 )
               call errsub ( 1440, 2, 0, 1 )
               call errcrd

            end if

         else if ( itype(i)==11 .or. itype(i)==14 .or. &
                   itype(i)==15 .or. itype(i)>17 .and. &
                   itype(i)<=20 .or. itype(i)==22 .or. &
                   itype(i)==24 .or. itype(i)==28 ) then

            tabmax(i) = 0d0

         end if

      end do

1000  call erclos ( 'intx02filltab' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx02filltab'

      end if  ! ( debug )

      end
