      subroutine intx02type89 ( numberkeyw, icount, nncode, table, &
                                tabmax, tabout, pointar, valuekeyw, &
                                maxrepeat, iposut, nout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     There are keywords of type 8 and 9
!     Fill information read into table, tabmax and tabout
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nncode, numberkeyw(*), icount, pointar(*), maxrepeat, &
              iposut, nout
      double precision table(nncode), tabmax(nncode), tabout(nout), &
                       valuekeyw(maxrepeat,*)

!     icount         i    Counter, number of keywords with itype = 8 or 9
!     iposut        i/o   Position in array tabout
!     maxrepeat      i    Declared size of first entry of array numberkeyw
!     nncode         i    Number of entries in code
!     nout           i    Number of entries in tabout
!     numberkeyw     i    Contains the number of values stored in array
!                         valuekeyw
!     pointar        i    array of size n containing the sequence numbers of
!                         all keywords with respect to arrays numberkeyw and
!                         valuekeyw
!                         if 0, no reference is necessary
!     table         i/o   Contains the numbers read for the various keywords
!     tabmax        i/o   Contains maximum values allowed in table
!     tabout        i/o   Contains series of numbers
!     valuekeyw      i    Array to store all values corresponding to one
!                         specific keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, k, i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iseq           Sequence number
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx02type89' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx02type89'
         call prinin ( numberkeyw, icount, 'numberkeyw' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nncode

         if ( pointar(i)>0 ) then

!        --- Sequence number found

            iseq = pointar(i)
            if ( numberkeyw(iseq)>0 ) then

!           --- keyword appears more than once

               tabmax(i) =  numberkeyw(iseq)  ! number of numbers read
               table(i) = iposut+1  ! refers to starting position in tabout
               if ( iposut+numberkeyw(iseq)>nout ) then

!              --- Error: tabout too small

                  call errint ( nout, 1 )
                  call errsub ( 1468, 1, 0, 0 )
                  go to 1000

               end if  ! ( iposut+numberkeyw(iseq)>nout )

!              --- Copy information to tabout

               do k = 1, numberkeyw(iseq)
                  tabout(iposut+k) = valuekeyw(k,iseq)
               end do  ! k = 1, numberkeyw(iseq)
               iposut = iposut+numberkeyw(iseq)

            end if  ! ( numberkeyw(iseq)>0 )

         end if  ! ( pointar(i)>0 )

      end do  ! i = 1, nncode

1000  call erclos ( 'intx02type89' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx02type89'

      end if  ! ( debug )

      end
