      subroutine intx04 ( dnumbr, maxnum, inum, istartinfo )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 10-01-2015 Allow expressions
!        version  2.4  date 14-05-2010 Correction 1d
!        version  2.3  date 04-11-2009 Minor change in debug
!
!   copyright (c) 1993-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine INTX04 is used to read a series of numbers
!   The series is closed by a ) sign, end of record or a text
!   The reading of numbers is stopped if
!    a ) sign is found
!    an end of record is found
!    maxnum numbers have been read
!   After reading the numbers, icolum points to the next position to be read
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/creadinfof90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: maxnum, istartinfo
      integer, intent(out) :: inum
      double precision, intent(out) :: dnumbr(maxnum)

!     dnumbr         o    Array to be filled with the numbers
!     inum           o    Number of numbers that have been actually read
!     istartinfo     i    Starting address to put information in array
!                         infotabout
!     maxnum         i    Maximum number of numbers to be read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished, debug, openfound
      integer kcolsv

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     kcolsv         Help parameter to save the old value of kcolum
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READREAL       Read a real or real expression from the input file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear array dnumbr
!   finished  := false
!   inum   := 0
!   ispecl := 4
!   while ( not finished ) do
!      Read next item (ASKNXT)
!      if ( ityprd = 2 or 3 ) then
!         inum := inum+1
!      else if ( ityprd = 0, 1, 4, 5, 6 or 7 ) then
!         finished := true
!      if ( inum = maxnum ) then
!         finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx04' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx04'
         write(irefwr,1) 'lastitem', lastitem
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear array dnumbr

      dnumbr = 0

      finished  = .false.
      inum   = 0
      ispecl = 4
      kcolsv = kcolum
      if ( typeitems(lastitem+1)==13 ) lastitem = lastitem+1  ! skip = &
      if ( debug ) write(irefwr,1) 'lastitem/1', lastitem

200   if ( .not. finished ) then

!     --- While loop
!         Read next item

         if ( typeitems(lastitem+1)==5 ) then
            lastitem = lastitem+1
            openfound = .true.
         else
            openfound = .false.
         end if  ! ( typeitems(lastitem+1)==5 )

         if ( debug ) then
            write(irefwr,1) 'lastitem, typeitems(lastitem+1)', &
                             lastitem, typeitems(lastitem+1)
            write(irefwr,*) 'openfound ', openfound
         end if  ! ( debug )

         if ( typeitems(lastitem+1)==6 ) then

!        --- Next item = closing bracket, mark and return

            lastitem = lastitem+1
            ityprd = 7

         else

!        --- Standard case, read real or scalar

            call readreal ( openfound, .true. )

         end if  ! ( typeitems(lastitem+1)==6 )

         if ( debug ) then
            write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
            write(irefwr,2) 'realrd ', realrd
         end if  ! ( debug ) then

         if ( ityprd==2 .or. ityprd==3 ) then

!        --- ityprd = 2 or 3, number found

            inum = inum + 1
            dnumbr(inum) = realrd

         else if ( ityprd==11 .or. ityprd==19 ) then

!        --- ityprd = 11, scalar found or
!            ityprd = 19, expression found

            inum = inum + 1
            dnumbr(inum) = nint(realrd)
            infotabout(istartinfo+inum-1) = ityprd

         else if ( ityprd<=1 .or. ityprd>=4 .and. ityprd<=5 .or. &
                   ityprd==7 .or. ityprd==20 ) then

!        --- ityprd = 0, 1 or 4-7, i.e. end of file, end of record or
!            ) sign found

            finished = .true.

         end if

         if ( openfound .and. typeitems(lastitem)==6 .or. inum>=maxnum ) &
              then

!        --- ) found or inum = maxnum, stop reading

            finished = .true.

         end if

         go to 200

      end if
      ispecl = 0
      kcolum = min ( kcolum, kcolsv )

1000  call erclos ( 'intx04' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( dnumbr, inum, 'dnumbr' )
         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
         write(irefwr,*) 'End intx04'

      end if  ! ( debug )

      end
