      subroutine intxstring ( code, nncode, ncodmx, icode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if one of the keywords is String
!     In that case it might be a string name
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nncode, ncodmx(nncode), icode
      character (len=*) code(nncode)

!     code           i    Table with keywords
!     icode          o    Return code
!                         Indicates position of keyword in array code
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nncode         i    Number of entries in code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ivalrd, jcode, valmax
      logical debug, vectorfound

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ivalrd         Integer value read
!     jcode          Special value of icode corresponding to vector or string
!     valmax         Maximum value allowed for the integer read
!     vectorfound    If true vector or string is a keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READSTRINGS    Check if name read corresponds to one of the strings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2951   Sequence number of vectors out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intxstring' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intxstring'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nncode

         vectorfound = ncodmx(i)==6 .and. code(i)=='STRING'
         jcode = i
         if ( vectorfound ) exit

      end do  ! i = 1, nncode

      if ( debug ) then
         write(irefwr,1) 'jcode', jcode
         write(irefwr,*) 'vectorfound', vectorfound
      end if  ! ( debug )

      if ( vectorfound ) then

!     --- The keyword string is present

         if ( debug ) write(irefwr,*) 'vectorfound/2'
         call readstrings
         if ( debug ) &
            write(irefwr,1) 'After readstrings, ityprd', ityprd
         if ( ityprd==16 ) then

!        --- string name found

            icode = jcode
            ivalrd = nint(realrd-100000000d0)
            ivalrd = mod(ivalrd,10000)
            ivalrd = mod(ivalrd,5000)

            valmax = numstrings
            if ( debug ) write(irefwr,1) 'ivalrd', ivalrd
            if ( ivalrd<0 .or. ivalrd>valmax ) then

!           --- string number out of range

               call errint ( ivalrd, 1 )
               call errint ( valmax, 2 )
               call errsub ( 2951, 2, 0, 0 )
               call errcrd

            end if  ! ( ivalrd<=0 .or. ivalrd>numstrings )

         end if  ! ( ityprd==2 )

      end if  ! ( vectorfound )

1000  call erclos ( 'intxstring' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intxstring'

      end if  ! ( debug )

      end
