      module sepmodulecmat

      use sepmodulematstr
      use sepmodulecomio
      use sepmodulecpack

      double complex, pointer :: cxglob(:)   ! Contains the part of the global
                                              ! solution that is required in
                                              ! this cpu

      type cmatrix

!     --- Defines a complex compact matrix in SEPRAN style

         double complex, pointer :: S(:)  ! the internal matrix
         double complex, pointer :: MD(:) ! the diagonal of the preco mat
         double complex, pointer :: DLshift(:) ! diagonal for laplace shift
         integer, pointer :: diag(:)        ! position of diagonal elements
                                            ! for row-compact storage
         integer, pointer :: cols(:), rows(:)  ! columns and row definitions
         integer, pointer :: Mcols(:), Mrows(:) ! columns and row definitions
                                               ! for preconditioning matrix
         integer, pointer :: Mdiag(:)         ! position of diagonal elements
                                              ! for row-compact storage
         integer :: iseqmt ! Sequence number of matrix in array realmt
         integer :: iseqim ! Sequence number of integer matrix description
         integer :: istartsol  ! Control parameter concerning the start
                               ! of the conjugate process.
                               ! Possibilities:
                               ! 0: The process starts with vector 0
                               ! 1: The process starts with an explicitly
                               !    given start vector.
         integer :: keep                       ! See sepmodulesolve
         integer :: neq                        ! number of equations
         integer :: nnz                        ! number of non-zero elements
         integer :: type_storage               ! storage tyoe
         integer :: type_prec                  ! type of preconditioning
         integer :: scaling                    ! 0 is not scaling
                                               ! 1: Only row scaling
                                               ! 2: Row and column scaling
                                               ! 3: Only column scaling
         integer :: matmultsp                  ! 0 is standard matrix vector
                                               !   multiplication
                                               ! 1: special for AL
         integer :: nblocks                    ! number of blocks
         integer, pointer :: iblocks(:)        ! blocks
         integer :: lmax                       ! second dimension of block
         double complex, pointer :: MDB(:,:)   ! the diagonal of the
                                               ! block preco mat

!        --- Information with respect to projection vectors

         integer :: nproj                      ! number of projection vectors
         logical :: deflation                  ! if true deflation
         integer, pointer :: projcols(:), projrows(:)  ! def for proj vectors
         double complex, pointer :: projU(:)  ! sparse projection vectors
         double complex, pointer :: projAu(:)! sparse projection vectors Au
         double complex, pointer :: projE(:)  ! Cholesky decomposition
!                                               ! of the matrix E
         double complex, pointer :: workstorage(:), work1(:)  ! work arrays

!        --- In case of parallel computations the off-diagonal matrix
!            blocks are stored in rowsout, colsout and Aout

         integer, pointer :: colsout(:), rowsout(:)  ! columns and row
                                               ! definitions
         double complex, pointer :: Aout(:)  ! matrix elements
         integer :: nglob                      ! global number of unknowns

!        --- Information for linear solvers

         double precision :: adapt    ! parameter to adapt diagonal for preco
         double precision :: factor   ! factor for MILU process
         double precision :: eps      ! Local accuracy
         double precision :: eps1     ! Help variable for the termination
                                      ! criterion of the CG-process
                                      ! If eps1 = 0, the termination
                                      ! criterion is based upon the relative
                                      ! error
         double precision :: eps2     ! Help variable for the termination
                                      ! criterion of the CG-process
                                      ! If eps2 = 0, the termination
                                      ! criterion is based upon the absolute
                                      ! error
         double precision :: alammin  ! minimum eigenvalue to be used in
                                      ! termination criterion
         double precision :: subsolve ! relative accuracy for subsolver
         double precision :: maxlev   ! minimum value in case of constraint
                                      ! overrelaxation
         double precision :: minlev   ! minimum value in case of constraint
                                      ! overrelaxation
         integer :: iabserr           ! check on absolute error
         integer :: iaccuracy         ! defines how accuracy is stored
         integer :: iconstrain        ! indicates if constraints should be
                                      ! applied.
         integer :: idegfd            ! Indicates the degree of freedom that
                                      ! must be restricted
         integer :: iminimum          ! indicates how the minimum must be
                                      ! computed.
         integer :: imaximum          ! indicates how the maximum must be
                                      ! computed.
         integer :: iprint            ! print level
         integer :: irelerr           ! relative/abs error
         integer :: itotit            ! Accumulated number of iterations
         integer :: newmumps          ! if >0 the matrix structure is new
                                      ! otherwise an old structure is used
         integer :: nmax              ! maximum number of iterations
         integer :: ntreigv           ! used to estimate eigenvalues
         integer :: ntrunk            ! residual is made orthogonal to
                                      ! ntrunk vectors
         integer :: numiter           ! number of iterations executed
         integer :: itererr           ! error in case of no convergence
                                      !   0 no error
                                      !   2 error: no convergence
                                      !   4 error: maximal accuracy reached
         integer :: aterror           ! Defines what to do in case of an
                                      ! error: Possible values:
                                      !   0: stop
                                      !   1: return
         integer :: prlump            ! See sepmodulesolve
         integer :: startvec          ! Defines start vector
                                      !   0:  vector 0
                                      !   1:  given start vector.
                                      !   2:  random vector
         integer :: typesolv          ! Defines type of solver
                                      ! see inpsol(3) (read03)
         integer :: typeparsolv       ! Defines type of parallel solver
                                      !   1:  block jacobi
                                      !   2:  gcr
         integer :: ldim              ! parameter l in bicgstab(l)
         integer :: imax              ! Maximal dimension of Krylov space
         integer :: ninner            ! Number of inner iterations
         logical :: eistat            ! if true eisenstat preconditioning
         logical :: precon            ! if true standard preconditioning
         logical :: globshadow        ! if true the shadow vectors must be
                                      ! computed globally in the parallel
                                      ! case
         logical :: decomp            ! if true the decomposition of
                                      ! the matrix must be computed
         logical :: symmetric         ! if true the matrix is symmetric

       ! Arrays with respect to parallel computations (N-N coupling)

         integer, pointer :: innercontr(:)
         integer, pointer :: sendinfo(:)

      end type cmatrix

!     --- Definition of boundary condition matrix S_ip

      type cpmatrix

         integer :: nrow                       ! number of rows in
                                               ! internal matrix
         integer :: neq                        ! number of boundary conditions
         integer :: n                          ! size of complete vector
         integer, pointer :: cols(:), rows(:)  ! columns and row definitions
         integer, pointer :: colsp(:), rowsp(:) ! columns and row definitions
         double complex, pointer :: B(:)       ! boundary matrix
         double complex, pointer :: Bip(:)     ! boundary matrix S_ip
         double complex, pointer :: Bpi(:)     ! boundary matrix S_pi
         double complex, pointer :: Bpp(:)     ! boundary matrix S_pp

      end type cpmatrix

!        --- Define matrix vector multiplication interface for this type

      interface operator (*)
         module procedure cmplmatvec
      end interface

!     --- Define solution of incomplete system

      interface operator (/)
         module procedure matdiv  ! A y = x  or U y = x
         module procedure divmat  ! L y = x (Eisenstat)
      end interface

!     --- Define solution of system of linear equations

      interface operator (.div.)
         module procedure cmatsolve  ! A x = y
      end interface

!     --- Define matrix vector multiplication interface for
!         projection vectors
!         y = P x = A .P. x
!         y = P^T x = A .PT. x
!         y = AP x = A .AP. x
!         y = AP^T x = A .APT. x
!         y = E^-1 x = A .Einv. x
!         y = P E^-1 P^T x = A .CPROJ. x   ( coarse grid)
!         y = x - P E^-1 AP^T x = A .PROJ. x (deflation)
!         y = x - AP E^-1 P^T x = A .PROJT. x (deflation transpose)

      interface operator (.PROJ.)
         module procedure PROJ
      end interface
      interface operator (.PROJT.)
         module procedure PROJT
      end interface
      interface operator (.CPROJ.)
         module procedure CPROJ
      end interface

!     --- Define actual matrix vector multiplication

      contains

!     --- Matrix vector multiplication

      function cmplmatvec ( A, x ) result(y)

      implicit none
      type (cmatrix), intent(in) :: A     ! matrix
      double complex, intent(in) :: x(A%neq)      ! vector
      double complex :: y(A%neq)                  ! result


      if ( parallel .and. typecoupl==1 ) then

!     --- parallel case
!         Copy cxglob from other blocks and send x to other blocks

         call sepsolcopsol ( x, cxglob )

!        --- Standard part matrix vector multiplication

         call sepcmatvec ( A, x, y )

!        --- Extra part, due to parts of vector stored in neighbor blocks

         call sepcmatoutvec ( A, cxglob, y )

      else

!     --- serial case

         call sepcmatvec ( A, x, y )

      end if  ! ( parallel )

      end function cmplmatvec

      function cmatsolve ( A, y ) result(x)  ! A x = y

      implicit none
      type (cmatrix), intent(in) :: A     ! matrix
      double complex, intent(in) :: y(A%neq)      ! right-hand side
      double complex :: x(A%neq)                  ! result

      call sepcsolvemeth ( A, x, y )

      end function cmatsolve

!     --- Inverse Matrix vector multiplication for a compact matrix
!         ILU

      function matdiv ( x, A ) result(y)

      implicit none
      type (cmatrix), intent(in) :: A      ! matrix
      double complex, intent(in) :: x(A%neq)      ! vector
      double complex :: y(A%neq)                  ! result M y = x

      call sepcmatdiv ( A, x, y )
      if ( A%nproj>0 .and. .not. A%deflation ) then

         A%workstorage = A .CPROJ. x
         y = y + A%workstorage

      end if  ! ( A%nproj>0 .and. .not. A%deflation )

      end function matdiv

      function divmat ( A, x ) result(y)

      implicit none
      type (cmatrix), intent(in) :: A      ! matrix
      double complex, intent(in) :: x(A%neq)      ! vector
      double complex :: y(A%neq)                  ! result M y = x

      call sepcdivmat ( A, x, y, A%neq )

      end function divmat

!     --- Parallel matrix vector multiplication

      function parmat ( A, x, cxglob ) result(y)

      implicit none
      type (cmatrix), intent(in) :: A         ! matrix
      double complex, intent(in) :: x(A%neq), cxglob(A%nglob)  ! vector
      double complex :: y(A%neq)                     ! result

      integer :: n

      n = A%neq

      call sepcmatvec ( A, x, y )
      call sepcmatoutvec ( A, cxglob, y )

      end function parmat

!     --- projection matrix vector multiplication P^T x

      function PTmatvec ( A, x ) result(y)

      implicit none
      type (cmatrix), intent(in) :: A     ! matrix
      double complex, intent(in) :: x(A%neq)      ! vector
      double complex :: y(A%nproj)                ! result
      integer :: i, j, idegfd
      double complex :: h

      do i = 1, A%nproj

         h = 0d0

         do j = A%projrows(i)+1, A%projrows(i+1)
            idegfd = A%projcols(j)
            h = h + A%projU(j)*x(idegfd)
         end do

         y(i) = h

      end do

      end function PTmatvec

!     --- projection matrix vector multiplication AP^T x

      function APTmatvec ( A, x ) result(y)

      implicit none
      type (cmatrix), intent(in) :: A     ! matrix
      double complex, intent(in) :: x(A%neq)      ! vector
      double complex :: y(A%nproj)                ! result
      integer :: i, j, idegfd
      double complex :: h

      do i = 1, A%nproj

         h = 0d0

         do j = A%projrows(i)+1, A%projrows(i+1)
            idegfd = A%projcols(j)
            h = h + A%projAU(j)*x(idegfd)
         end do

         y(i) = h

      end do

      end function APTmatvec

!     --- projection matrix vector multiplication  y = AP x

      function APmatvec ( A, x ) result(y)

      implicit none
      type (cmatrix), intent(in) :: A     ! matrix
      double complex :: y(A%neq)       ! result
      double complex, intent(in) :: x(A%nproj)    ! vector
      integer :: i, j, idegfd
      double complex :: h

      y = 0d0
      do i = 1, A%nproj

         h = x(i)

         do j = A%projrows(i)+1, A%projrows(i+1)
            idegfd = A%projcols(j)
            y(idegfd) = y(idegfd)+A%projAu(j)*h
         end do

      end do

      end function APmatvec

!     --- projection matrix vector multiplication  x = P y

      function Pmatvec ( A, y ) result(x)

      implicit none
      type (cmatrix), intent(in) :: A     ! matrix
      double complex :: x(A%neq)       ! result
      double complex, intent(in) :: y(A%nproj)    ! vector
      integer :: i, j, idegfd
      double complex :: h

      x = 0d0

      do i = 1, A%nproj

         h = y(i)

         do j = A%projrows(i)+1, A%projrows(i+1)
            idegfd = A%projcols(j)
            x(idegfd) = x(idegfd) + A%proju(j)*h
         end do

      end do

      end function Pmatvec

!     --- Multiplication with inverse of E matrix (projection)

      function Einverse ( A, x ) result(y)

      implicit none
      type (cmatrix), intent(in) :: A     ! matrix
      double complex :: y(A%nproj)       ! result
      double complex, intent(in) :: x(A%nproj)    ! vector

      call sepcsolvsm ( A%nproj, A%projE, y, x )

      end function Einverse

!     --- Norm of vector x (serial)

      function cnorm ( x, n )

      implicit none
      integer, intent(in) :: n
      double complex, intent(in) :: x(n)
      double precision :: cnorm

      cnorm = sqrt(abs(dot_product (x,x)))

      end function cnorm

!     --- Global norm of vector x over all processors (parallel)

      function glnorm ( x, n )

      implicit none
      integer, intent(in) :: n
      double complex, intent(in) :: x(n)
      double precision :: glnorm

      double complex :: sum1, norm1

      sum1 = dot_product (x,x)
      call prparglsum ( sum1, norm1, 2, SPDOUBLE )
      glnorm = sqrt(abs(norm1))

      end function glnorm

!     --- Global dot product of vectors x an y over all processors (parallel)

      function gldotprod ( x, y, n )
      integer, intent(in) :: n
      double complex, intent(in) :: x(n), y(n)
      double complex :: gldotprod

      double complex :: sum1

      sum1 = dot_product (x,y)
      call prparglsum ( sum1, gldotprod, 2, SPDOUBLE )

      end function gldotprod

!     --- Computation of residual

      function cresidual ( A, x, b ) result(y)

      implicit none
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%neq), b(A%neq)
      double complex :: y(A%neq), work(A%neq)
      logical :: eistat
      integer :: iprec

      iprec = A%type_prec

      eistat = iprec==2 .or. iprec==4 .or. iprec==7
      y = A*x                     ! y = A x
      if ( eistat ) y = y+2d0*x   ! correction Eisenstat
      y = b-y                     ! residual

      if ( A%nproj>0 .and. A%deflation ) y =  A .projT. y
      if ( eistat ) y = A/y       ! Pre multiply residual by E^(-1)
                                  ! in case of Eisenstat

      end function cresidual

      function projT ( A, x ) result(y)
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%neq)
      double complex :: y(A%neq)
      double complex :: work(A%nproj)

!     --- Deflation transpose:    y = x - AP E^-1 P^T x

      call sepcprojT ( A, x, y )

      end function projT

      function proj ( A, x ) result(y)
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%neq)
      double complex :: y(A%neq)
      double complex :: work(A%nproj)

!     --- Deflation:  y = x - P E^-1 AP^T x

      call sepcproj ( A, x, y )

      end function proj

      function cproj ( A, x ) result(y)
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%neq)
      double complex :: y(A%neq)
      double complex :: work(A%nproj)

!     --- Coarse grid  y = P E^-1 P^T x

      call sepcprojc ( A, x, y )

      end function cproj

      end module sepmodulecmat
