module sepmoduleoldrouts

!    Contains call to old subroutines as described in the Programmer's Guide

   contains

      subroutine adapboun ( inpada, rinada, kmeshlc, kproblc, isol, &
                            icurves, iread )
      use sepmodulemain
      implicit none
      integer, intent(in) :: inpada(*), kmeshlc, kproblc, icurves(*), iread
      integer, target :: isol
      double precision, intent(in) :: rinada(*)
      logical :: changed, changeinput
      integer :: iseqvecs(1)
      call eropen ( 'adapboun' )
      kmesh2d(1) = kmeshlc
      kprob2d(1) = kproblc
      kmesh => kmesh2d(1)
      kprob => kprob2d(1)
      isolut3d(1,1) = isol
      isolut2d => isolut3d(:,1)

      iseqvecs(1) = 1
      call adapbndb ( inpada, rinada, icurves, iread, changed, changeinput, &
                      1, iseqvecs )
      isol = isolut2d(1)
      call erclos ( 'adapboun' )
      end subroutine adapboun

      subroutine adapmesh ( kmeshlc, kproblc, intmatlc, isol, inpada, &
                            rinada, inpmsh, rinmsh, icurvs, iread )
      use sepmodulemain
      implicit none
      integer, intent(in) :: inpada(*), inpmsh(*), icurvs(*), iread
      integer, target ::  kmeshlc, kproblc, intmatlc, isol(*)
      double precision :: rinada(*), rinmsh(*)
      integer :: iseqvecs(1)
      call eropen ( 'adapmesh' )

      intmat1d(1:1) = intmatlc
      isolut2d => isol(1:2)
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      kprob2d(1) = kproblc
      kprob => kprob2d(1)
      iseqvecs(1) = 1
      call mshadap ( inpada, rinada, inpmsh, rinmsh, icurvs, iread, &
                     1, iseqvecs )
      call erclos ( 'adapmesh' )
      end subroutine adapmesh

      subroutine addmat ( kproblc, matr1, matr2, intmatlc, &
                          alpha1, alpha2, beta1, beta2 )
      use sepmodulemain
      use sepmoduleintmat
      implicit none
      integer, intent(in) :: kproblc, matr1
      integer, intent(inout) :: matr2
      integer, target :: intmatlc
      double precision, intent(in) :: alpha1, alpha2, beta1, beta2
      call eropen ( 'addmat' )
      intmat => intmatlc
      kprob2d(1) = kproblc
      kprob => kprob2d(1)
      call sepaddmat ( matr1, matr2, alpha1, alpha2, beta1, beta2 )
      call erclos ( 'addmat' )
      end subroutine addmat

      subroutine addmt1 ( kproblc, matr1, matr2, matr3, &
                          matr4, intmat1, intmat2, alpha, beta, gamma )
      double complex, intent(in) :: alpha, beta, gamma
      integer, intent(in) :: kproblc, matr1, matr2, matr3, intmat1
      integer, intent(out) :: intmat2, matr4
      call eropen ( 'addmt1' )
      call sepaddmat1 ( kproblc, matr1, matr2, matr3, &
                        matr4, intmat1, intmat2, alpha, beta, gamma )
      call erclos ( 'addmt1' )
      end subroutine addmt1

      subroutine difiar ( intmt1, intmt2 )
      implicit none
      integer, intent(in) :: intmt1, intmt2
      call eropen ( 'difiar' )
      call sepdifiar ( intmt1, intmt2 )
      call erclos ( 'difiar' )
      end

      subroutine algebr ( ichois, idegfd, ivec1, ivec2, ivec3, &
                          kmeshlc, kproblc, a, b, p, q, ipoint )
      use sepmodulemain
      implicit none
      double precision, intent(inout) :: a,b,p,q
      integer, intent(in) :: ichois, idegfd, ivec1, ivec2, kmeshlc, &
              kproblc
      integer, intent(inout) :: ivec3, ipoint
      call eropen ( 'algebr' )
      kprob2d(1) = kproblc
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      call sepfillkmesh(kmesh)
      kprob => kprob2d(1)
      call sepalgebr ( ichois, idegfd, ivec1, ivec2, ivec3, &
                       a, b, p, q, ipoint )
      call erclos ( 'algebr' )
      end subroutine algebr

      function anorm ( ichois, ichnrm, jdegfd, kmeshlc, &
                       kproblc, isol1, isol2, ielhlp )
      use sepmodulekprob
      use sepmodulemain
      implicit none
      integer, intent(in) :: ichois, ichnrm, jdegfd, kproblc, isol1, &
              isol2, ielhlp
      integer, target :: kmeshlc
      double precision :: anorm
      double precision :: sepnorm
      call eropen ( 'anorm' )

      kprob2d(1) = kproblc
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      call sepfillkmesh(kmesh)
      kprob => kprob2d(1)
      anorm = sepnorm ( ichois, ichnrm, jdegfd, isol1, isol2, ielhlp )
      call erclos ( 'anorm' )
      end function anorm

      function bounin ( ichois, irule, idim, number, kmesh, kprob, &
                        icrv1, icrv2, ivectr, iuser, user )
      implicit none
      integer, intent(in) :: ichois, irule, idim, number, kmesh, &
              kprob, icrv1, icrv2, ivectr, iuser(*)
      double precision, intent(in) :: user(*)
      double precision :: bounin
      double precision :: value
      call eropen ( 'bounin' )
      call bouniv ( ichois, irule, idim, number, kmesh, kprob, &
                    icrv1, icrv2, ivectr, iuser, user, value )
      bounin = value
      call erclos ( 'bounin' )
      end function bounin

      subroutine bouniv ( ichois, irule, idim, number, kmeshlc, kproblc, &
                          icrv1, icrv2, ivectr, iuser, user, value )
      implicit none
      integer, intent(in) :: ichois, irule, idim, number, kmeshlc, &
              kproblc, icrv1, icrv2, ivectr, iuser(*)
      double precision, intent(in) :: user(*)
      double precision, intent(out) :: value
      call eropen ( 'bouniv' )
      call sepbouniv ( ichois, irule, idim, number, kmeshlc, kproblc, &
                       icrv1, icrv2, ivectr, iuser, user, value )
      call erclos ( 'bouniv' )
      end subroutine bouniv

      subroutine build ( iinbldlc, matr, intmatlc, kmeshlc, kproblc, irhsdlc, &
                         massmtr, iusol, islold, iuserloc, userloc )
      implicit none
      integer, intent(in) :: iinbldlc(*), iusol, islold, kproblc, kmeshlc
      integer, intent(inout) :: irhsdlc
      integer, target :: intmatlc, matr, massmtr, iuserloc(*)
      double precision, target :: userloc(*)
      call eropen ( 'build' )
      call sepbuildold ( iinbldlc, matr, intmatlc, kmeshlc, kproblc, &
                         irhsdlc,  massmtr, iusol, islold, iuserloc, &
                         userloc )
      call erclos ( 'build' )
      end subroutine build

      subroutine bvalue ( ichois, jdimpr, kmeshlc, kproblc, isol, &
                          value, icrv1, icrv2, idegfd, jstap )
      use sepmodulemain
      implicit none
      integer, intent(in) :: ichois, jdimpr, icrv1, icrv2, idegfd, jstap, &
                             kproblc, kmeshlc
      integer, intent(inout) :: isol
      double precision, intent(in) :: value
      call eropen ( 'bvalue' )
      kprob2d(1) = kproblc
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      call sepfillkmesh(kmesh)
      kprob => kprob2d(1)
      call sepbvalue ( ichois, jdimpr, isol, &
                       value, icrv1, icrv2, idegfd, jstap )
      call erclos ( 'bvalue' )
      end subroutine bvalue

      subroutine chancf ( iuser, user, kprob, kmesh, iinfil )
      implicit none
      integer, intent(in) :: iinfil(*)
      integer, target :: kmesh, kprob, iuser(*)
      double precision, target :: user(*)
      call eropen ( 'chancf' )
      call sepchancf ( iuser, user, kprob, kmesh, iinfil )
      call erclos ( 'chancf' )
      end subroutine chancf

      subroutine difrar ( matr1, matr2 )
      implicit none
      integer, intent(in) :: matr1, matr2
      call eropen ( 'difrar' )
      call sepdifrar ( matr1, matr2 )
      call erclos ( 'difrar' )
      end subroutine difrar

      subroutine manmat ( iinmat, rinmat, matr1, matr2 )
      implicit none
      integer, intent(in) :: iinmat(*), matr1, matr2
      double precision, intent(in) :: rinmat(*)
      call eropen ( 'manmat' )
      call sepmanmat ( iinmat, rinmat, matr1, matr2 )
      call erclos ( 'manmat' )
      end

      subroutine mapcom ( iinmap, kprob, intmat1, intmat2, matr1, matr2 )
      implicit none
      integer, intent(in) :: iinmap(*), intmat1, matr1, kprob
      integer, intent(inout) :: intmat2, matr2
      call eropen ( 'mapcom' )
      call sepmapcom ( iinmap, intmat1, intmat2, matr1, matr2 )
      call erclos ( 'mapcom' )
      end subroutine mapcom

end module sepmoduleoldrouts
