module sepmoduleplot
!
!  Replaces old commons cgrens, ccolor, cplaf1, cplaff, cplafo, cplot,
!                       cplpos, cpost5
!
   use sepmodulecons

   integer, parameter :: ncolor = 145  ! CONSTANT! Number of colors in table

   integer :: curcol    ! current color of output device
   integer :: dcolor    ! Default (current) color to draw lines and text in
                        ! 0          Do not use any color handling
   integer :: expx      ! Defines the multiplication exponent along x-axis
   integer :: expy      ! Defines the multiplication exponent along y-axis
   integer :: idirec    ! Direction of plotting used:
                        !  1          Normal direction
                        ! -1          Rotated over 90 degrees
   integer :: ilzoom    ! Level of zooming ( is used in computation of actfac:
                        ! actfac = factor ** -(ilzoom-1)
   integer :: ioutsd    ! Used when JTEST = 1. Meanings are:
                        ! 0 Both new and preceding point are within rectangle
                        ! 1 New point is within, preceding outside
                        ! 2 New is outside, preceding inside
                        ! 3 Both new and preceding point are outside rectangle
   integer :: iplpic    ! Picture number for SEPRAN neutral plot file
   integer :: irotat    ! Indication whether plots may be rotated over 90
                        ! degrees in order to save plotting paper or not.
                        ! Possible values:
                        ! 0   Rotation is allowed
                        ! 1   No rotation is allowed
                        ! 2   PLots are always rotated
   integer :: isigx     ! Number of significant digits (floating) or
                        ! decimal places (fixed point) along x-axis
   integer :: isigy     ! Same for the y-axis
   integer :: jkader    ! At input jkader has the following meaning:
                        ! >= 0 (standard) A plot frame is plotted without scales
                        !  -1  No plot frame nor scales
                        !  -2  A plot frame is plotted as well as axis with scales
                        !  -3  No plot frame is plotted, the axis are plotted with
                        !      scales
                        !      Locally jkader gets the value kader when a plot frame
                        !      must be plotted
   integer :: jmark     ! Flags, used to suppress drawing of scale-values:
                        !  0-99      Not of interest for this routine
                        !   100      Suppress values along the x-axis
                        !  1000      Same along left y-axis
                        ! 10000      Same along right y-axis (if selected)
   integer :: jmax      ! Indication if the parameters xmin, xmax, ymin and ymax
                        ! must be computed by the subroutine (0) or are given
                        ! by the user (1)
   integer :: jtest     ! Test parameter for the window. Possibilities:
                        ! 0          No tests performed
                        ! 1          Test if point are within rectangle.
   integer :: jtimes    ! Parameter to indicate whether one or more plots are made
                        ! in one picture. Possibilities
                        !    0 Only one plot is made
                        !    1 First plot in a sequence
                        !    2 Next plot in a sequence
                        !    3 Final plot in a sequence
                        ! >=11 See jtimes-10, however, the plotting of axis and arrows in
                        !      the standard plotting subroutines (except PLOTFN) is suspended
                        !      This possibility is meant for program SEPPOST.
                        !      In this program the axis and arrows are plotted later on
   integer :: kiesx     ! Indication if axmin and axmax must be computed (-1)
                        ! or not (Any other value)
                        ! When > 0 then numbers will be fixed-point, else floating
   integer :: kiesy     ! See kiesx now for y-axis
   integer :: lzoom     ! Indicates if zooming should be applied (1) or not (0)
   integer :: maxplots  ! Defines the number of integers that are used in
                        ! the files sepplot.xxx + 3
                        ! So 0 means sepplot.xxx
                        ! 1 means sepplot.xxxx
                        ! Only non-negative values  are allowed
   integer :: nstepx    ! Number of scale values to be plotted along x-axis
   integer :: nstepy    ! Same for the y-axis

   double precision :: actfac  ! actual zooming factor
   double precision :: ale     ! Length of the plot in the horizontal
                               ! direction in plotting units
   double precision :: alefig  ! Length of actual figure in cm (i.e. without
                               ! margins and frame)
   double precision :: aleng   ! Length of plot in cm's
   double precision :: alengm  ! Maximal length of plot paper in centimeters
   double precision :: axmax   ! largest value to be plotted along the x-axis
   double precision :: axmin   ! smallest value to be plotted along the x-axis
   double precision :: aymax   ! largest value to be plotted along the y-axis
   double precision :: aymin   ! smallest value to be plotted along the y-axis
   double precision :: botm    ! Margin in cm around plotting area (bottom)
   double precision :: cm      ! Multiplication factor to transfer plotting
                               ! units in centimeters
   double precision :: leftm   ! Margin in cm around plotting area (left)
   double precision :: marlft  ! Margin at left side of actual figure
                               ! (but within frame)
   double precision :: marund  ! Margin at lower side of actual figure
                               ! (but within frame)
   double precision :: rightm  ! Margin in cm around plotting area (right)
   double precision :: shiftx
   double precision :: shifty
   double precision :: small   ! Width of small paper in centimeters
   double precision :: stepx   ! Step size along x-axis
   double precision :: stepy   ! Step size along y-axis
   double precision :: topm    ! Margin in cm around plotting area (top)
   double precision :: xcentr  ! Centre of zooming region
   double precision :: xfct    ! Multiplication factors in x-direction.
                               ! Such that all coordinates will be transformed into the
                               ! unit square (0,1) x (0,1)
   double precision :: xmaxor  ! original value of xmax
   double precision :: xmax    ! maximum x-value of plot region
   double precision :: xmaxt   ! Only points with x<=xmaxt are plotted
   double precision :: xmaxzm  ! x-coordinate of right upper zooming point
   double precision :: xminor  ! original value of xmin
   double precision :: xmin    ! minimum x-value of plot region
   double precision :: xmint   ! Only points with x>=xmint are plotted
   double precision :: xminzm  ! x-coordinate of left under zooming point
   double precision :: xpleft  ! x-coordinate of left under point in region
                               ! including the margin in cm
   double precision :: xprec   ! x-coordinate of the last point that has been plotted
   double precision :: wid     ! Width of the plot in the vertical
                               ! direction in plotting units
   double precision :: widfig  ! Width of actual figure in cm
                               ! (i.e. without margins and frame)
   double precision :: wide    ! Width of wide paper in centimeters
   double precision :: width   ! Width of plot in cm's
   double precision :: ycentr  ! Centre of zooming region
   double precision :: yfct    ! See xfct, now in y-direction
   double precision :: ymax    ! maximum y-value of plot region
   double precision :: ymaxor  ! original value of ymax
   double precision :: ymaxt   ! Only points with y<=ymaxt are plotted
   double precision :: ymaxzm  ! y-coordinate of right upper zooming point
   double precision :: ymin    ! minimum y-value of plot region
   double precision :: yminor  ! original value of ymin
   double precision :: ymint   ! Only points with y>=ymint are plotted
   double precision :: yminzm  ! y-coordinate of left under zooming point
   double precision :: ypleft  ! y-coordinate of left under point in region
                               ! including the margin in cm
   double precision :: yprec   ! y-coordinate of the last point that has been plotted
   double precision :: zfact   ! zooming factor (generic)
   double precision :: zmax    ! maximum z-value of plot region
   double precision :: zmin    ! minimum z-value of plot region

   real :: oldfac     ! Contains the font width of the last font size used in cm
   real :: xorig
   real :: yorig
   real :: x0
   real :: y0
   real :: transx
   real :: transy

   real :: colors(3,ncolor) ! RGB values (between 0 and 1) of all colors to be used
                          ! color "1" is the default color (white for screen,
                          ! black for paper), rest has to be rendered to give
                          ! "as closely as possible" the same colors.

   character (len=32) :: namecomponents(maxcomponents) ! contains the names of
                         ! the components of the arrays in names
                         ! sequentially
   character (len=32) :: namefuncs(maxfuncs)           ! contains the names of
                         ! the function arrays of type 117 (funcy)
   character (len=32) :: names(maxvc1)                 ! contains the names of
                         ! the arrays as given by the user in the statements
                         ! NAME Vi = ....

   character (len=80) :: identf      ! Text to be plotted as identification at
                                     ! all plots
   character (len=80) :: textid(2)   ! Contains identification texts
                         ! 1: Text to be plotted as plot identification
                         ! 2: Text: levels
   character (len=80) :: textx       ! Text to be plotted along x--axis
   character (len=80) :: texty       ! Text to be plotted along y--axis

end module sepmoduleplot
