module sepmoduletime

!   --- Definition of some quantities for time integration subroutines

   use sepmodulecons

   type equationtim

      integer :: iconsmass   ! If 1 the mass matrix is constant
                             ! otherwise 0
      integer :: iconsrhs    ! Information concerning the right-hand
                             ! side i.e. the function f (no effect of boundary
                             ! conditions)
                             ! 0:  The right-hand side is time-dependent
                             ! 1:  The right-hand side is time-independent
                             ! 2:  The right-hand side is zero
      integer :: iconsstiff  ! If 1 the stiffness matrix is constant
                             ! otherwise 0
      integer :: imas        ! Parameter imas for building of mass matrix
      integer :: imasstot    ! Indicates if the matrix M+dtS must be
                             ! created (0) or not (1)
      integer :: ireadsl     ! Parameter with respect to the reading of
                             ! information for subroutine solvel.
                             ! If ireadsl = -1, only defaults are used
      integer :: iseqfr      ! sequence number of first vector with
                             ! respect to array islold. It is supposed that
                             ! the unknown vectors to be solved are iseqfr,
                             ! iseqfr+1, ... , iseqfr+numprb-1 in that sequence
      integer :: iseqaccel   ! sequence number of acceleration vector
      integer :: iseqaddrhs  ! sequence number of rhs term for adding
      integer :: iseqmt  ! equence number of matrix and
                             ! right-hand side in set of matrices
      integer :: iseqveloc   ! sequence number of velocity vector
      integer :: isubdiv     ! indicates if quadratic elements must
                             ! be treated as a conglomerate of linear elements
                             ! (1) or as quadratic elements itself (0)
      integer :: lenextra    ! Length of array extratim
      integer :: limit       ! limit for equation
      integer :: iseqreac    ! sequence number of reaction vector

      integer :: extratim(6) ! Pos. 1 Contains the type of action
                            !        Possible values:
                            !        -1: last action
                            !         8: call to deriv
                            ! Depending on the type of action extra positions
                            ! are needed
                            ! In case of DERIV the positions 2 - 6 are used:
                            ! 2:  Sequence number coefficients
                            ! 3:  IPROB
                            ! 4:  IVECTOR Sequence number output vector
                            ! 5:  ISEQNR Sequence number derivatives
                            ! 6:  ISCAL

   end type equationtim


   integer :: icalltim      ! Sequence number of call to timprb
   integer :: icall_outtim  ! defines whether OUTTIM must be called (1)
                            ! or not (0)
   integer :: ichtim        ! Type of integration method
                            ! 0: Time stepping with constant time step, where
                            !    in each time step a stationary problem is
                            !    solved
                            ! 1: Euler/Trapezoid rule with self-selecting
                            !    step-size
                            ! 2: Rozenbrock/Wanner rule with self-selecting
                            !    step-size
                            ! 3: Runge-Kutta-Fehlberg (1-2 order) with
                            !    self-selecting step-size
                            ! 4: Runge-Kutta-Fehlberg (3-4 order) with
                            !    self-selecting step-size
                            ! 5: Euler implicit, time-step given by the user
                            ! 6: Implicit trapezoid rule, time-step given by
                            !    the user
                            ! 7: Euler explicit, time-step given by the user
                            ! 8: Runge-Kutta 4 explicit, time-step given by
                            !    the user
                            ! 9: Implicit (modified) theta method
                            !    (0<theta<=1),
                            !    theta and time-step given by the user
                            !    (includes 5,6)
                            !10: Fractional step method
                            !11: Central differences for second order
                            !    time-derivatives, time-step given by the
                            !    user
                            !12: Generalized theta method
                            !13: Second order Gear
                            !14: Newmark method for second order
                            !    time-derivatives
                            !15: Generalized alpha method for second for
                            !    second order time-derivatives
   integer :: icons         ! Indication if the matrices and time-step are
                            ! constant (1) or not (0)
   integer :: ifilltim      ! Indicates if the user builds the matrix
                            ! and right-hand side by a call to filtim (1) or
                            ! not (0)
   integer :: ifrac         ! Fractional step sequence number
   integer :: iflag         ! Return code, with indication of how the process
                            ! has terminated.
                            ! Possible values:
                            ! 3:  Successfull return
                            ! 4:  The process has stopped because of errors or
                            !     inaccuracy
   integer :: ioutact       ! If 1 printing may be performed, if 0 not
   integer :: ipiusrlc      ! Position in iuser
   integer :: iprinttim     ! Defines the amount of output produced
                            ! in case of a stationary solution
                            ! Possible values:
                            ! -1:  All output information is suppressed,
                            !      even the message of no-convergence
                            !  0:  No information about the iteration
                            !      process is printed except in the case
                            !      of an error
                            !  1:  The number of iterations performed is
                            !      printed
                            !  2:  Some extra information about the iteration
                            !      process is printed for each iteration
                            !  3:  See 2, in this case also the ratio of two
                            !      succeeding iterations is printed
                            !  100:  Contains the maximum of preceding values
                            !      and also prints the solution in each
                            !      iteration
   integer :: iprobtim      ! Porblem number
   integer :: ipuserlc      ! Position in user
   integer :: ireadbctim    ! Indicates how boundary conditions must
                            ! be computed. This parameter is only used if
                            ! itimbc<2.
                            ! Possible values:
                            !   0: Read input from the standard input file
                            !      during the first time step
                            !  -1: Copy boundary conditions from array islold
                            !      Only if itimbc=1
                            ! <-10:Copy boundary conditions from array islold
                            !      Sequence number in array islold is equal
                            !      to |ireadbc+10|. So actually a different
                            !      solution vector is used to store the
                            !      boundary conditions
                            !      Only if itimbc = 0
                            !  >0: The input has been read by SEPSTN
                            !      ireadbc indicates the corresponding
                            !      sequence number
   integer :: ireturtim     ! Indicates if the subroutine must stop
                            ! if no convergence has been found (0) or that
                            ! control must be given back to the main program (1)
                            ! Only if stationary solution is required
   integer :: iseqdt        ! Sequence number in isol of diagonal matrix
                            ! containing the local time steps per point
   integer :: iseqfrtim     ! sequence number of first vector
   integer :: iseqouttim    ! sequence number of output block
   integer :: iseqreactim   ! sequence number of reaction vector
   integer :: itimbc        ! Indicates how often boundary conditions
                            ! must be computed. Possible values:
                            ! 0: Boundary conditions are time-dependent and
                            !    must be filled each step
                            ! 1: Boundary conditions are time-independent and
                            !    must be filled only once
                            ! 2: Boundary conditions are not filled by this
                            !    subroutine, but by subroutine filtim
                            !    Only in combination with ifiltm=1
   integer :: itimcf        ! Indicates how coefficients are treated.
                            ! Possible values:
                            ! 0: Coefficients are filled by the user in the
                            !    arrays iuser and user. This may be done in
                            !    subroutine filtim.
                            !    The parameter ireadcf is not used.
                            ! 1: Coefficients are not filled in iuser and
                            !    user but in the buffer array instead.
                            !    The parameter ireadcf is used.
   integer :: itimdep       ! If 0 no time-dependence has been detected
   integer :: itiminterval  ! Defines the sequence number of the actual time
                            ! interval with respect to a series of
                            ! (dt1, dt2, ... ) and (tend1, tend2, ..)
                            ! Hence 1 refers to dt1, tend1 etc
   integer :: itimloop      ! Indicates the status of the time loop
                            ! 0  Time loop not active
                            ! 1  First time step
                            ! 2  Rest of time steps
   integer :: keeptim       ! If 1 the time t is not changed, otherwise
                            ! it is raised
                            ! If 2 the time t is first reset and then raised
                            ! again
   integer :: lineartim     ! Indicates if the process must be
                            ! considered as linear, i.e. with linearized
                            ! coefficients or as strictly non-linear, which
                            ! implies that in each time step iterations
                            ! are carried out.
                            ! Possible values:
                            ! 0,1: linear
                            ! 2: standard non-linear iteration
                            ! 3: updated Lagrange
                            ! 4: updated Lagrange, without mesh deformation
   integer :: maxcalltim    ! Maximum number of calls to timprb in one time step
   integer :: maxitertim    ! maximum number of iterations per time step
   integer :: minitertim    ! minimum number of iterations per time step
   integer :: maxoldtim     ! Maximum number of vectors allowed in islold
   integer :: ncopeqtim     ! Number of coupled problems to be solved
   integer :: ndt           ! Number of different time steps
   integer :: nfrac         ! Number of fraction steps
   integer :: nhist         ! Number of points in which the time history
                            ! must be printed
   integer :: nitertim      ! Iteration sequence number
   integer :: numoldtim     ! Number of vectors stored in array islold


   integer, pointer :: ireadcftim(:)  ! May be considered as an array ireadcf
                            ! with length ncopeq
                            ! This array contains the parameters iread for the
                            ! coefficients
                            ! ireadcf(j): Indicates how coefficients must be
                            ! read
                            !  Possible values:
                            !  0: Read input from the standard input file
                            !     during the first time step
                            !     Each problem requires his own input block.
                            ! >0: The input has been read by SEPSTN

   double precision :: absaccurtim  ! Absolute criterion for stationary solution
   double precision :: arrendtime(10) ! ndt end times
   double precision :: beta_g      ! coefficients for generalized alpha and
   double precision :: gamma_g     ! newmark
   double precision :: alpha_f
   double precision :: alpha_m
   double precision :: arrtheta(10) ! nfrac thetas
   double precision :: arrtimestep(10) ! ndt time steps
   double precision :: dt           ! Time step
   double precision :: epstim       ! accuracy for time integration in case of
                                    ! a self-selecting step size
   double precision :: epstimsol    ! accuracy for linear solver if an
                                    ! iterative method is used
   double precision :: relaccurtim  ! Relative criterion for stationary solution
   double precision :: t            ! Actual time
   double precision :: t0           ! Time at first call of soltim
   double precision :: tend         ! End time for the numerical integration
   double precision :: tendnext     ! Next value of tend
   double precision :: theta        ! Parameter theta for the theta method
   double precision :: tout         ! The integration is carried out from
                                    ! initial time to tout
   double precision :: toutend      ! End time for time-dependent output
   double precision :: toutinit     ! Initial time for time-dependent output
   double precision :: toutnext     ! Next time for time-dependent output
   double precision :: toutstep     ! Time step for time-dependent output
   double precision :: tstep        ! Time step for the numerical integration
   double precision :: tthreshold   ! Threshold time. Divergence of the
                                    ! stationary solution is only checked
                                    ! when t>tthreshold

   double precision, pointer :: coorhist(:,:)  ! co-ordinates of points for
                                    !  which the time history must be printed
                                    ! first position length ndim
                                    ! second position: length nhist

   type (equationtim), pointer :: Teq(:)

   logical :: stationarysol ! If true the solution is stationary and it is
                            ! checked if a stationary solution has been
                            ! reached
   logical :: nocomputationtim ! If true only the time is raised and the time
                            !  parameters are set. No computation is performed
   logical :: usectimen     ! If true time parameters are copied from ctimen
   logical :: complrun      ! If true a complete time loop must be carried out,
                            ! if false one time step only
   logical :: oldmethod     ! If true soltim closes if tout is reached

!  --- Some local arrays for subroutine septimrout

   integer :: inpsoltim(ninpsol)    !  User input array for subroutine SOLVE
   integer :: nodhist(maxhist)      !  contains the nodes of the history points
   integer :: iinbldtim(15)         !  User input array for subroutine BUILD
   double precision :: rinsoltim(nrinsol)! User input array for subroutine SOLVE
   double precision :: diffsl(maxprb,3) ! information about the convergence of
                                    !  the iteration process is stored for each
                                    !  coupled equation
                                    ! DIFFSL is filled as follows:
                                    ! diffsl(i,1)   maximum difference between
                                    !               present and preceding
                                    !               iteration,
                                    !               i.e. || u(n) - u(n-1) ||
                                    ! diffsl(i,2)   value of diffsl(i,1) in the
                                    !               preceding iteration,
                                    !               i.e. || u(n-1) - u(n-2) ||
                                    ! diffsl(i,3)   value of diffsl(i,2) in the
                                    !               preceding iteration,
                                    !               i.e. || u(n-2) - u(n-3) ||
                                    ! The parameter i indicates the sequence
                                    ! number of the equation

   integer, pointer :: iintimcp(:)  ! Copy of local array iintim

end module sepmoduletime
