module sepmoduletypes

!  information of element types like boundary points subdivision etc

   integer :: iboun3(142)
   integer :: ishstr(20)
   integer :: ishsub(206)
   integer :: itetra(352)
   integer :: itypbs(3,8)
   integer :: ityps(24)
   integer :: ivertx12(4)
   integer :: ivertx14(8)
   integer :: numpel(20)
   integer :: numsel(20)

!
!      Description
!
!     iboun3  This array describes, together with itypbs the outer boundary
!             of the surface elements of the surfaces. The starting position
!             of a boundary is given in itypbs(i,3)
!             The actual contents are the relative nodal points, with
!             respect to the element, of the surfaces
!             The boundary of a three-dimensional element consists of
!             either 4 triangles (in case of tetrahedras) or 6 quadrilaterals
!             (in case of hexahedras)
!             Internal nodes at the boundaries are also stored, hence
!             the boundaries corresponding to the shape numbers are mapped
!             as follows:
!             type 11:    4 elements of shape 3
!             type 12:    4 elements of shape 4
!             type 13:    6 elements of shape 5
!             type 14:    6 elements of shape 6
!             type 15:    4 elements of shape 7
!             type 16:    4 elements of shape 7
!             type 17:    4 elements of shape 3
!             type 18:    6 elements of shape 5
!             The information of types 15 and 16 is identical so no
!             extra storage is needed
!     ishstr  ISHSTR(i) contains the start position of the subdivided
!             elements in array ISHSUB. i refers to the shape number of
!             the not subdivided elements
!
!     ISHSUB  Contains the subdivision of the elements sequentially
!     itetra  In this array the subdivision of three-dimensional elements of
!             some shape in simple linear elements is stored.
!             The numbers refer to the local numbers in the element of type
!             ishape
!             The information in itetra is stored as follows:
!             Pos 1-4 contain the 4 nodes of elements of shape 11
!             Pos 5-40 contain information of elements of shape 12
!                     in the following way:
!                Pos. 5-8  4 vertex nodes (subdivision in one linear element)
!                Pos. 9-40 8 groups of 4 nodes each forming a linear
!                          tetrahedron in this element
!             Pos 41-64 contain information of elements of shape 13
!                       The brick is subdivided into 6 tetrahedral elements
!                       of 4 nodes each
!             Pos 65-280 contain information of elements of shape 14
!                     in the following way:
!                Pos. 65-88 The brick is subdivided into 6 tetrahedral elements
!                           of 4 nodes each, corresponding to the vertices
!                           only
!                Pos. 89-280 The brick is subdivided into 48 tetrahedral
!                           elements of 4 nodes each, corresponding to all
!                           points in the brick. This is done by
!                           considering the brick as a set of 8 subbricks
!                           each of which is subdivided into 6 tetrhedras
!             At this moment only the type numbers 11-14 are stored in itetra
!             The starting address of information of elements of type ishape
!             can be found in ityps. This means that information for type
!             number ishape can be found from position ityps(ishape)+1
!             Pos 281-292 contain information of elements of shape 21
!                     3 groups of 4 nodes each forming a linear
!                          tetrahedron in this element
!             Pos.293-352 Contains the subdivision of a quadratic hexahedron in
!                         6 quadratic tetrahedrons
!     itypbs  This element describes a part of the boundaries of three-
!             dimensional elements (type numbers 11-18)
!             Contents:
!             itypbs(1,i)  Number of nodes at boundary of surfaces of element
!                          type i-10
!             itypbs(2,i)  Number of surfaces of element type i-10
!             itypbs(3,i)  Starting position of information of boundary surface
!                          of element type i-10 in array iboun3
!                          Internal nodes at the boundary are also included
!     ityps   Array with starting positions in array itetra
!             ityps(ishape) gives the starting position of elements of type
!             ishape in array in itetra minus one
!             At this moment only type numbers 11-14 are stored in itetra
!             The other positions in ityps contain a zero
!     ivertx12   Contains the relative vertex numbers of the 4 vertices of
!                a quadratic tetrahedron
!     ivertx14   Contains the relative vertex numbers of the 8 vertices of
!                a tri-quadratic hexahedron
!
!     numpel     NUMPEL(i) contains the number of nodes in the subelements,
!                elements of shape i are subdivided into
!
!     numsel     NUMSEL(i) contains the number of subelements, elements of
!                shape i are subdivided into
end module sepmoduletypes
