      subroutine msh071 ( ibuffr, buffer, ichois, iref, kmesh, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 21-11-2012 Long integers
!        version  3.1  date 09-05-2012 Extra debug
!        version  3.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1991-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read information of the mesh from the file indicated by iref
!   This subroutine replaces the old subroutine MESHRD with a minor extension
!   (ihelp)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cmachtf90'
      include 'SPcommon/cdebug_routsf90'
      include 'SPcommon/cpackagef90'
      include 'SPcommon/csepmainf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, iref, kmesh(*), ihelp(10), ibuffr(*)
      double precision buffer(*)

!     buffer i/o   Real array in which all large data is stored
!     ibuffr i/o   Integer array in which all large data is stored
!     ichois  i    choice parameter,  possibilities:
!                 >=0    mesh is read,  Subroutine TOPOL is called with ichois
!                                         as first parameter
!                    -1    mesh is read,  Subroutine TOPOL is skipped
!                    -2    mesh is read,  Subroutine TOPOL is called
!                          Parameter ichois depends on version number of
!                          file meshoutput
!     ihelp   o    Contains the input written by MSH059 on the second record
!                  after nvolms
!     iref    i    dataset reference number for the output
!                  when iref = 0 the standard output file is assumed
!                  when iref>=0 the file is supposed to be formatted, &
!                                 else an unformatted file is assumed
!     kmesh   o    standard sepran array containing information of the mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipcoor, ipwork, ipkmsc, ipkmsj, ipiinput, &
                       iprinput, ipkmsh, ipkmsm, ipicrv, ipcurv, &
                       ipkmsp, ipkmsq, ipkmsr, ipkmss, ipkmsv, &
                       ipkmsw1, ipkmsw2, ipkmsx, iprmsx, ipiwrk, &
                       ipkmsn, ipkmsk, ipcurvefacts
      integer irefc, i, ios, ndim, npelm, nboun, meshtp, nelgrp, &
              npoint, nelem, nuspnt, ncurvs, nvolms, lengpc, length, &
              lengav, kelmt, n, ivers, lenkmss, npointlc, &
              jchois(10), nsurfs, kspec, lniinp, lnrinp, lenkmsk, &
              ivolnr, isurnr, lenkmsh, lenkmsq, lenkmsr, &
              lenkmsv, nintprop, nrealprop, &
              lenkmsx, index, inpelm, iobst, lenin, &
              lenrl, numobst, isurf, nextra, nwork, &
              ioutpsav, mmiinput
      double precision dummy(1)
      logical debug, renumber
      character (len=80) text

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     dummy          Dummy parameter
!     i              Counting variable
!     index          Memory management sequence number
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     ios            IOSTAT variable, returns from reading or writing
!     ioutpsav       Help parameter to store the value of ioutp
!     ipcoor         starting address of co-ordinates array in BUFFER
!     ipcurv         starting address array curves containing the coordinates of
!                    the curves
!     ipcurvefacts   Starting address of CURVEFACTS in IBUFFR
!     ipicrv         starting address array containing the information of
!                    the curves (number of nodal points in each curve)
!     ipiinput       Starting address of array iinput in array IBUFFR
!     ipiwrk         Starting address of work array iwork in IBUFFR
!     ipkmsc         Starting address of array KMESH part c into array IBUFFR
!     ipkmsh         Starting address of array KMESH part h into array IBUFFR
!     ipkmsj         Starting address of array KMESH part j into array IBUFFR
!     ipkmsk         Starting address of array KMESH part k into array IBUFFR
!     ipkmsm         Starting address of array KMESH part m into array IBUFFR
!     ipkmsn         Starting address of array KMESH part n into array IBUFFR
!     ipkmsp         Starting address of array KMESH part p into array IBUFFR
!     ipkmsq         Starting address of array KMESH part q into array IBUFFR
!     ipkmsr         Starting address of array KMESH part r into array IBUFFR
!     ipkmss         Starting address of array KMESH part s into array IBUFFR
!     ipkmsv         Starting address of array KMESH part v in array IBUFFR
!     ipkmsw1        Starting address of array INTPROP in array IBUFFR
!     ipkmsw2        Starting address of array REALPROP part v in array IBUFFR
!     ipkmsx         Starting address of array KMESH part x into array IBUFFR
!     iprinput       Starting address of array rinput in array BUFFER
!     iprmsx         Starting address of array KMESH part x into array BUFFER
!     ipwork         Starting address in array work of work array
!     irefc          dataset reference number for reading (>0 formatted, &
!                    <0 unformatted)
!     isurf          Type of surface with respect to mesh generator
!     isurnr         Surface sequence number
!     ivers          Version sequence number
!     ivolnr         Volume sequence number
!     jchois         Parameter ichois for TOPOL
!     kelmt          Indicates if array KMESH part t has been filled >0,or not 0
!                    If kelmt>0, kelmt denotes the starting address of
!                    KMESH part t in KMESH
!     kspec          Indicates if spectral elements have been created (>0) or
!                    not (=0). If kspec>0, then kspec gives the starting
!                    position of the spectral variables in KMESH
!     lengav         Available length for an array
!     lengpc         Length of KMESH part c
!     length         Help parameter to indicate the length of arrays
!     lenin          Number of integers in one row of kmeshx
!     lenkmsh        Length of array kmeshh
!     lenkmsk        Length of array kmeshk
!     lenkmsq        Length of array kmeshq
!     lenkmsr        Length of array kmeshr
!     lenkmss        Length of array kmeshs
!     lenkmsv        Length of array KMESH part v
!     lenkmsx        Length of array KMESH part x
!     lenrl          Number of reals in one row of rmeshx
!     lniinp         Available space for array IINPUT
!     lnrinp         Available space for array RINPUT
!     meshtp         Type number of elements in the mesh or element group
!     mmiinput       Memory management sequence number with respect to
!                    iinput
!     n              Help variable to store ncurvs, nsurfs or nvolms
!     nboun          Number of boundary points at an element
!     ncurvs         Number of curves in mesh
!     ndim           Dimension of the space.
!     nelem          Number of elements
!     nelgrp         Number of element groups
!     nextra         Number of extra variables stored in kmeshh(5,.)
!     nintprop       Number of integer properties per element group
!     npelm          Number of points per element
!     npoint         Number of nodal points
!     npointlc       Local number of points
!     nrealprop      Number of real properties per element group
!     nsurfs         Number of surfaces in mesh
!     numobst        number of obstacles
!     nuspnt         Number of user points
!     nvolms         number of volumes
!     nwork          Declared length of array iwork
!     renumber       If true standard renumbering must be applied
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidbl, inigettext, inidgttext
      integer inilen

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI062         reserve space for temporary arrays
!     INI080         Create permanent space in array BUFFER
!     INI081         Create permanent space in array IBUFFR
!     INIDBL         Transform pointer for IBUFFR to pointer for BUFFER
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
!     MSH000         Fill some parameters from array IINPUT
!     MSH021         Fill first part of array KMESH
!     MSH022         Read KMESH part c and COOR from a file
!     MSH070         read kmesh part j from the file indicated by irefc
!     MSHCHECKELMS   Check if two-dimensional mesh contains hanging nodes
!     MSHDEL         Delete array KMESH and corresponding sub arrays
!     MSHFILCR       Fill array icurvs and curves
!     MSHFILLSURNOD  Create and fill array surfnodes
!     MSHFILSR       Fill array kmesh part h with respect to surfaces
!     MSHFILVL       Fill array kmesh part h with respect to volumes
!     MSHFLKMP       Fill array kmesh part p
!     MSHOBSIN33     Fill information in kmesh part x concerning one obstacle,3D
!     MSHPOSJAC      Change sequence of nodes in elements with negative Jacobian
!     MSHRDKMESHLMN  Read kmesh parts l m and n from the file meshoutput
!     MSHRDMESHDIM   Read first records from file meshoutput
!     MSHREADINT     Read integer array from file irefc
!     MSHREADKMESHT  Read kmesh part t from file meshoutput
!     MSHREADREAL    Read real array from file irefc
!     MSHUSERMESH    Create extra surfaces, curves and user points in a user
!                    mesh if necessary
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
!     TOPOL          Fill integer information corresponding to the mesh in KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       4   Number of element groups read is incorrect
!      20   nboun has wrong value
!      62   dimension of space (ndim) has wrong value
!      73   Maximal number of points per element is incorrect
!     119   The number of curves is incorrect
!     150   The number of surfaces is incorrect
!     192   MESHTP has incorrect value
!    1380   The number of user points is incorrect
!    1836   The number of elements is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ndim=2 ) then
!        Fill array KMESH part P
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh071' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh071'
         write(irefwr,1) 'iref', iref
         ioutpsav = ioutp
         ioutp = 1
  1      format ( a, 1x, (10i9) )

      end if
      if ( ierror/=0 ) go to 1000
100   format ( a )

      irefc = iref
      if ( iref==0 ) irefc = irefre
      if ( kmesh(2)==100 ) call mshdel ( ibuffr, kmesh )
      if ( debug ) write(irefwr,1) 'irefc', irefc

!     --- Read dimension parameters and first text records

      call mshrdmeshdim ( irefc, ndim, npelm, nboun, meshtp, &
                          nelgrp, npoint, nelem, nuspnt, ncurvs, &
                          nsurfs, nvolms, ihelp, ivers )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'ndim, npelm, nboun, meshtp, nelgrp, npoint', &
                          ndim, npelm, nboun, meshtp, nelgrp, npoint
         write(irefwr,1) 'nelem, nuspnt, ncurvs, nsurfs, nvolms', &
                          nelem, nuspnt, ncurvs, nsurfs, nvolms
         call prinin ( ihelp, 10, 'ihelp' )
      end if  ! ( debug )

!     --- Check parameters

      if ( ndim<=0 .or. ndim>3 ) then
         call errint ( ndim, 1 )
         call errsub ( 62, 1, 0, 0 )
         go to 1000
      end if
      if ( npelm<=0 .or. npelm>5000 ) then
         call errint ( npelm, 1 )
         call errint ( 1000, 2 )
         call errsub ( 73, 2, 0, 0 )
      end if
      if ( nelgrp<=0 .or. nelgrp>10000 ) then
         call errint ( nelgrp, 1 )
         call errint ( 1000, 2 )
         call errsub ( 4, 2, 0, 0 )
      end if
      if ( nelem<1 ) then
         call errint ( nelem, 1 )
         call errint ( 100000, 2 )
         call errsub ( 1836, 2, 0, 0 )
      end if
      if ( nuspnt<0 .or. nuspnt>10*npoint ) then
         call errint ( nuspnt, 1 )
         call errint ( 10*npoint, 2 )
         call errsub ( 1380, 2, 0, 0 )
      end if
      if ( ncurvs<0 .or. ncurvs>10000 ) then
         call errint ( ncurvs, 1 )
         call errint ( 10000, 2 )
         call errsub ( 119, 2, 0, 0 )
      end if
      if ( nsurfs<0 .or. nsurfs> 10000) then
         call errint ( nsurfs, 1 )
         call errint ( 10000, 2 )
         call errsub ( 150, 2, 0, 0 )
      end if
      if ( nboun<0 .or. nboun>npelm ) then
         call errint ( nboun, 1 )
         call errint ( npelm, 2 )
         call errsub ( 20, 2, 0, 0 )
      end if
      if ( meshtp<0 .or. meshtp>100 ) then
         call errint ( meshtp, 1 )
         call errint ( 100, 2 )
         call errint ( 0, 3 )
         call errsub ( 192, 3, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      kmesh( 6) = ndim
      kmesh( 4) = npelm
      kmesh( 5) = nelgrp
      kmesh( 8) = npoint
      kmesh( 9) = nelem
      kmesh(10) = nuspnt
      kmesh(11) = ncurvs
      kmesh(12) = nsurfs
      kmesh(13) = nvolms

      if ( debug .or. ioutp>=1 ) &
           write(irefwr,110) ndim, npelm, nboun, meshtp, nelgrp, npoint, &
                             nelem, nuspnt, ncurvs, nsurfs, nvolms
110   format(' ndim =', i3, '   npelm =', i5, '   nboun =', i5, &
             '    meshtp =', i4/ ' nelgrp =', i5, &
             ' npoint =', i8, '   nelem =', i8, '   nuspnt =', i6/ &
             ' ncurvs =', i6, '    nsurfs =', i6, '  nvolms =', i6 )
      call ini062 ( ibuffr, 4*nelgrp, ipwork )
      if ( ierror/=0 ) go to 1000

!     --- Fill the first part of array kmesh

      if ( debug ) write(irefwr,*) 'before msh021'
      call msh021 ( kmesh, npelm, nboun, meshtp, nelgrp, ibuffr(ipwork), &
                    irefc, nelem, lengpc, ihelp )

      call ini080 ( ibuffr, kmesh(23), ndim*npoint, 'coor' )
      call ini081 ( ibuffr, kmesh(17), lengpc, 'kmeshc' )

      ipcoor = inidgttext ( kmesh(23), 'coor' )
      ipkmsc = inigettext ( kmesh(17), 'kmeshc' )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'before msh022'
      call msh022 ( irefc, ndim, npoint, lengpc, buffer(ipcoor), &
                    ibuffr(ipkmsc), kmesh, ivers )
      if ( ierror/=0 ) go to 1000

!     --- Read kmesh parts l, m and n

      if ( debug ) write(irefwr,*) 'before mshrdkmeshlmn'
      call mshrdkmeshlmn ( ibuffr, kmesh(26), irefc, ivers, nuspnt, &
                           ncurvs, npoint, ihelp, nsurfs, nvolms, &
                           nelem )

      if ( ivers>=1 .and. ihelp(5)>0 ) then

!     --- Read kmesh part j

         if ( ivers>=7 ) then

!        --- Read and skip comment record

            if ( irefc>=0 ) then
               read (irefc,100,iostat=ios) text
            else
               read(-irefc,iostat=ios) text
            end if
            if ( debug ) write(irefwr,*) text
         end if
         if ( ichois==-2 ) then

!        --- ichois = -2, read kmeshj

            call ini081 ( ibuffr, kmesh(24), npoint, 'kmeshj' )
            call ini062 ( ibuffr, npoint, ipwork  )
            ipkmsj = inigettext ( kmesh(24), 'kmeshj' )
            if ( ierror/=0 ) go to 1000
            call msh070 ( ibuffr(ipkmsj), ibuffr(ipwork), npoint, &
                          irefc )

         else

!        --- Read dummy

            if ( irefc>=0 ) then
                read (irefc,*,iostat=ios) (n,i=1,npoint)
            else
                read(-irefc,iostat=ios) n
            end if

         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- READ KMESH t if necessary

      kelmt = kmesh(36)
      if ( kelmt>0 ) then

!     --- kelmt>0, read kmesh t

         if ( debug ) write(irefwr,*) 'before mshreadkmesht'
         call mshreadkmesht ( kmesh, kelmt, nvolms, nsurfs, ncurvs, &
                              ios, irefc, debug, ivers, nelem )

      end if
      if ( debug ) write(irefwr,*) 'after mshreadkmesht'

!     --- Read spectral part of KMESH if necessary

      kspec = kmesh(38)

      if ( debug ) write(irefwr,1) 'kspec, ivers', kspec, ivers

      if ( kspec>0 ) then

!     --- Read spectral part

         if ( irefc>=0 ) then

            if ( ivers>=7 ) read (irefc,100,iostat=ios) text
            read(irefc, *) ( kmesh( kspec + i ), i = 0 , 4 )

         else

            if ( ivers>=7 ) read(-irefc,iostat=ios) text
            read(-irefc)  ( kmesh( kspec + i ), i = 0 , 4 )

         end if
         if ( debug ) write(irefwr,*) text

      end if
      if ( ivers>1 ) then

!     --- At least version 2, read arrays iinput and rinput

         if ( irefc>=0 ) then
            if ( ivers>=7 ) read (irefc,100,iostat=ios) text
            read(irefc,*) lniinp, lnrinp, lenkmsh, lenkmsq, lenkmsr, &
                          lenkmss
         else
            if ( ivers>=7 ) read(-irefc,iostat=ios) text
            read(-irefc)  lniinp, lnrinp, lenkmsh, lenkmsq, lenkmsr, &
                          lenkmss
         end if
         if ( debug ) then
            write(irefwr,*) text
            write(irefwr,1) 'lniinp, lnrinp, lenkmsh, lenkmsq, lenkmsr', &
                             lniinp, lnrinp, lenkmsh, lenkmsq, lenkmsr
            write(irefwr,1) 'lenkmss', lenkmss
         end if  ! ( debug )
         if ( lniinp>0 ) then

!        --- Read iinput and rinput

            call ini081 ( ibuffr, kmesh(34), lniinp, 'iinput' )
            call ini080 ( ibuffr, kmesh(35), lnrinp, 'rinput' )
            ipiinput = inigettext ( kmesh(34), 'iinput' )
            iprinput = inidgttext ( kmesh(35), 'rinput' )
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (ibuffr(ipiinput+i),i=0,lniinp-1)
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (buffer(iprinput+i),i=0,lnrinp-1)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (ibuffr(ipiinput+i),i=0,lniinp-1)
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (buffer(iprinput+i),i=0,lnrinp-1)
            end if
            if ( debug ) write(irefwr,*) text

         end if
         if ( lenkmsq>0 ) then
            call ini081 ( ibuffr, kmesh(31), lenkmsq, 'kmeshq' )
            ipkmsq = inigettext ( kmesh(31), 'kmeshq' )
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (ibuffr(ipkmsq+i),i=0,lenkmsq-1)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (ibuffr(ipkmsq+i),i=0,lenkmsq-1)
            end if
            if ( debug ) write(irefwr,*) text
         end if
         if ( lenkmsr>0 ) then
            call ini081 ( ibuffr, kmesh(32), lenkmsr, 'kmeshr' )
            ipkmsr = inigettext ( kmesh(32), 'kmeshr' )
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (ibuffr(ipkmsr+i),i=0,lenkmsr-1)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (ibuffr(ipkmsr+i),i=0,lenkmsr-1)
            end if
            if ( debug ) write(irefwr,*) text
         end if
         if ( lenkmss>0 ) then
            call ini081 ( ibuffr, kmesh(33), lenkmss, 'kmeshs' )
            ipkmss = inigettext ( kmesh(33), 'kmeshs' )
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (ibuffr(ipkmss+i),i=0,lenkmss-1)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (ibuffr(ipkmss+i),i=0,lenkmss-1)
            end if
            if ( debug ) write(irefwr,*) text
         end if

         if ( lenkmsh>0 ) then

!        --- create array kmesh part h

            length = 6*(nsurfs+nvolms+1)
            call ini081 ( ibuffr, kmesh(22), length, 'kmeshh' )
            ipkmsh = inigettext ( kmesh(22), 'kmeshh' )
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (ibuffr(ipkmsh+i),i=0,length-1)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (ibuffr(ipkmsh+i),i=0,length-1)
            end if
            if ( debug ) write(irefwr,*) text
            n = ibuffr(ipkmsh+4)
            length = n
            ipkmsm = inigettext ( kmesh(27), 'kmeshm' )
            ibuffr(ipkmsh+2) = 0
            call ini081 ( ibuffr, ibuffr(ipkmsh+2), ncurvs*2, 'icurvs' )
            ibuffr(ipkmsh+3) = 0
            ibuffr(ipkmsh+4) = 0
            call ini080 ( ibuffr, ibuffr(ipkmsh+4), length, 'curves' )
            ibuffr(ipkmsh+5) = 0
            call ini080 ( ibuffr, ibuffr(ipkmsh+5), length/ndim, &
                          'curvefacts' )
            ipicrv = inigettext ( ibuffr(ipkmsh+2), 'icurvs' )
            ipcurv = inidgttext ( ibuffr(ipkmsh+4), 'curves' )
            ipcurvefacts = inidgttext ( ibuffr(ipkmsh+5) ,'curvefacts' )
            ipcoor = inidgttext ( kmesh(23), 'coor' )
            ipkmsm = inigettext ( kmesh(27), 'kmeshm' )

!           --- Fill arrays icurvs and curves

            do i = 1, ncurvs
               ibuffr(ipicrv+ncurvs+i-1) = 0
            end do
            if ( ivers<=2 ) then
               call mshfilcr ( ibuffr(ipicrv), buffer(ipcurv), &
                               ibuffr(ipkmsm), buffer(ipcoor), ncurvs, &
                               ndim )
               do i = 0, length/ndim-1
                  buffer(ipcurvefacts+i) = 0d0
               end do
            else
               if ( ivers<=5 ) then
                  lengav = ncurvs
               else
                  lengav = 2*ncurvs
               end if
               if ( irefc>=0 ) then
                  read(irefc,*) (ibuffr(ipicrv+i),i=0,lengav-1)
                  read(irefc,*) (buffer(ipcurv+i),i=0,length-1)
                  if ( ivers>7 ) read(irefc,*) &
                     (buffer(ipcurvefacts+i),i=0,length/ndim-1)
               else
                  read(-irefc) (ibuffr(ipicrv+i),i=0,lengav-1)
                  read(-irefc) (buffer(ipcurv+i),i=0,length-1)
                  if ( ivers>7 ) read(-irefc) &
                     (buffer(ipcurvefacts+i),i=0,length/ndim-1)
               end if
            end if
            if ( ihelp(8)==1 ) then

!           --- Fill surfaces

               do isurnr = 1, nsurfs
                  ipkmsh = inigettext ( kmesh(22), 'kmeshh' )-1
                  ipkmss = inigettext ( kmesh(33), 'kmeshs' )-1
                  isurf = ibuffr(ipkmss+nsurfs+2+(isurnr-1)*8)
                  if ( isurf>0 .and. isurf/=7 .and. &
                       isurf/=11 .and. isurf/=12 .and. &
                       ibuffr(ipkmsh+isurnr*6+3)>0 ) then
                     if ( ivers<=7 ) then
                        npointlc = ibuffr(ipkmsh+isurnr*6+2)/ndim
                        npointlc = npointlc/intlen
                        nelem =  ibuffr(ipkmsh+isurnr*6+4)
                        if ( isurf==2 ) then
                           nextra = npointlc
                        else if ( isurf==7 .or. isurf==11 .or. &
                                  isurf==12 ) then
                           nextra = kmesh(11)
                        else
                           nextra = 0
                        end if
                     else
                        if ( irefc>=0 ) then
                           read(irefc,*) npointlc, nelem, nextra
                        else
                           read(-irefc)  npointlc, nelem, nextra
                        end if
                     end if
                     if ( npointlc>0 ) then

!                    --- Non-empty surface

                        index = 0
                        call ini080 ( ibuffr, index, npointlc*ndim, &
                                      'coor-loc' )
                        ipkmsh = inigettext ( kmesh(22), 'kmeshh' )-1
                        ibuffr(ipkmsh+isurnr*6+1) = index
                        index = 0
                        call ini081 ( ibuffr, index, nelem, &
                                      'kmeshc-loc' )
                        ipkmsh = inigettext ( kmesh(22), 'kmeshh' )-1
                        ibuffr(ipkmsh+isurnr*6+2) = index
                        index = 0
                        call ini081 ( ibuffr, index, npointlc, &
                                      'mapar' )
                        ipkmsh = inigettext ( kmesh(22), 'kmeshh' )-1
                        ibuffr(ipkmsh+isurnr*6+3) = index
                        ibuffr(ipkmsh+isurnr*6+4) = 0
                        if ( nextra>0 ) then
                           index = 0
                           call ini081 ( ibuffr, index, nextra, &
                                         'renum' )
                           ipkmsh = inigettext ( kmesh(22), 'kmeshh' )-1
                           ibuffr(ipkmsh+isurnr*6+5) = index
                        else
                           ibuffr(ipkmsh+isurnr*6+5) = 0
                        end if
                        ipkmsh = inigettext ( kmesh(22), 'kmeshh' )
                        length = npointlc*ndim*intlen + &
                                 nelem + npointlc+npoint
                        call ini062 ( ibuffr, length, ipiwrk )
                        ipwork = inidbl(ipiwrk)
                        ipiwrk = ipiwrk + npointlc*ndim*intlen
                        ipcoor = inidgttext ( kmesh(23), 'coor' )
                        ipkmsc = inigettext ( kmesh(17), 'kmeshc' )
                        ipkmsh = inigettext ( kmesh(22), 'kmeshh' )
                        ipkmss = inigettext ( kmesh(33), 'kmeshs' )
                        ipkmsn = 1
                        if ( kmesh(28)>0 ) &
                           ipkmsn = inigettext ( kmesh(28), 'kmeshn' )
                        call mshfilsr ( ibuffr, buffer, isurnr, &
                                        buffer(ipwork), ibuffr(ipiwrk), &
                                        ibuffr(ipiwrk+npointlc), &
                                        ibuffr(ipiwrk+npointlc+npoint), &
                                        buffer(ipcoor), ibuffr(ipkmsc), &
                                        ibuffr(ipkmss), ibuffr(ipkmsh), &
                                        ndim, ibuffr(ipkmsn), &
                                        kmesh(kmesh(36)), kmesh, irefc )

                     end if

                  end if

               end do

!              --- Fill volumes

               do ivolnr = 1, nvolms
                  isurnr = nsurfs+ivolnr
                  ipkmsh = inigettext ( kmesh(22), 'kmeshh' )-1
                  if ( ibuffr(ipkmsh+isurnr*6+3)>0 ) then
                     if ( ivers<=7 ) then
                        npointlc = ibuffr(ipkmsh+isurnr*6+6)
                        nelem =  ibuffr(ipkmsh+isurnr*6+4)
                     else
                        if ( irefc>=0 ) then
                           read(irefc,*) npointlc, nelem
                        else
                           read(-irefc)  npointlc, nelem
                        end if
                     end if
                     index = 0
                     call ini080 ( ibuffr, index, npointlc*ndim, &
                                   'coor-loc' )
                     ipkmsh = inigettext ( kmesh(22), 'kmeshh' )-1
                     ibuffr(ipkmsh+isurnr*6+1) = index
                     index = 0
                     call ini081 ( ibuffr, index, nelem, &
                                   'kmeshc-loc' )
                     ipkmsh = inigettext ( kmesh(22), 'kmeshh' )-1
                     ibuffr(ipkmsh+isurnr*6+2) = index
                     index = 0
                     call ini081 ( ibuffr, index, npointlc, &
                                   'mapar' )
                     ipkmsh = inigettext ( kmesh(22), 'kmeshh' )-1
                     ibuffr(ipkmsh+isurnr*6+3) = index
                     ibuffr(ipkmsh+isurnr*6+4) = 0
                     ibuffr(ipkmsh+isurnr*6+5) = 0
                     length = npointlc*intlen*ndim + &
                              nelem + npointlc + npoint
                     call ini062 ( ibuffr, length, ipiwrk )
                     ipwork = inidbl(ipiwrk)
                     ipiwrk = ipiwrk + npointlc*intlen*ndim
                     ipcoor = inidgttext ( kmesh(23), 'coor' )
                     ipkmsc = inigettext ( kmesh(17), 'kmeshc' )
                     ipkmsh = inigettext ( kmesh(22), 'kmeshh' )
                     ipkmsn = 1
                     if ( kmesh(28)>0 ) &
                        ipkmsn = inigettext ( kmesh(28),'kmeshn' )
                     call mshfilvl ( ibuffr, buffer, ivolnr, &
                                     buffer(ipwork), ibuffr(ipiwrk), &
                                     ibuffr(ipiwrk+npointlc), &
                                     ibuffr(ipiwrk+npointlc+npoint), &
                                     buffer(ipcoor), ibuffr(ipkmsc), &
                                     ibuffr(ipkmsh), ndim, &
                                     kmesh(kmesh(36)), kmesh, irefc )

                  end if

               end do

            end if

         end if

      end if
      if ( kmesh(27)>0 .and. ndim==2 ) then

!     --- ndim = 2
!         Fill array KMESH part P

         call ini080 ( ibuffr, kmesh(30), 2*ncurvs, 'kmeshp' )
         ipicrv = 1
         ipcurv = 1
         if ( kmesh(22)==0 ) lenkmsh = 0
         if ( lenkmsh>0 ) then
            ipkmsh = inigettext ( kmesh(22), 'kmeshh' )
            ipicrv = inigettext ( ibuffr(ipkmsh+2), 'icurvs' )
            ipcurv = inidgttext ( ibuffr(ipkmsh+4), 'curves' )
         end if
         ipcoor = inidgttext ( kmesh(23), 'coor' )
         ipkmsm = inigettext ( kmesh(27), 'kmeshm' )
         ipkmsp = inidgttext ( kmesh(30), 'kmeshp' )
         call mshflkmp ( buffer(ipcoor), ibuffr(ipkmsm), buffer(ipkmsp), &
                         ncurvs, lenkmsh, ibuffr(ipicrv), &
                         buffer(ipcurv) )

      end if

      if ( ivers>=4 ) then

!     --- Version 4, read two extra records

         if ( irefc>=0 ) then
            if ( ivers>=7 ) read (irefc,100,iostat=ios) text
            read ( irefc, *, iostat = ios ) kmesh(7), kmesh(44)
            read ( irefc, *, iostat = ios ) ihelp(1)
         else
            if ( ivers>=7 ) read(-irefc,iostat=ios) text
            read(-irefc, iostat = ios) kmesh(7), kmesh(44)
            read(-irefc, iostat = ios) ihelp(1)
         end if
         if ( debug ) then
            write(irefwr,*) text
            write(irefwr,*) 'kmesh(7), kmesh(44), ihelp(1)', &
                             kmesh(7), kmesh(44), ihelp(1)
         end if  ! ( debug )

         if ( kmesh(7)>0 ) then

!        --- Read kmesh part k

            lenkmsk = kmesh(7)+kmesh(44)
            call ini081 ( ibuffr, kmesh(25), lenkmsk, 'kmeshk' )
            ipkmsk = inigettext ( kmesh(25), 'kmeshk' )
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (ibuffr(ipkmsk+i),i=0,lenkmsk-1)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (ibuffr(ipkmsk+i),i=0,lenkmsk-1)
            end if
            if ( debug ) write(irefwr,*) text

         end if

      end if
      if ( ivers>=5 ) then

!     --- Version 5, read kmesh parts v and w if they exist

         if ( irefc>=0 ) then
            if ( ivers>=7 ) read (irefc,100,iostat=ios) text
            read(irefc,*) (ihelp(i),i=1,10)
         else
            if ( ivers>=7 ) read(-irefc,iostat=ios) text
            read(-irefc) (ihelp(i),i=1,10)
         end if
         if ( debug ) then
            write(irefwr,*) text
            call prinin ( ihelp, 10, 'ihelp' )
         end if  ! ( debug )

         if ( ihelp(1)>0 ) then

!        --- kmesh part v exists

            if ( irefc>=0 ) then
               read(irefc,*) lenkmsv
            else
               read(-irefc) lenkmsv
            end if
            call ini080 ( ibuffr, kmesh(45), lenkmsv, 'kmeshv' )
            ipkmsv = inigettext ( kmesh(45), 'kmeshv' )
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (ibuffr(ipkmsv+i),i=0,lenkmsv-1)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (ibuffr(ipkmsv+i),i=0,lenkmsv-1)
            end if
            if ( debug ) write(irefwr,*) text

         end if

         if ( ihelp(2)>0 ) then

!        --- kmesh part w exists

            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) nintprop, nrealprop
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) nintprop, nrealprop
            end if
            if ( debug ) write(irefwr,*) text
            if ( kmesh(46)==0 ) then
               kmesh(46) = kmesh(3)+1
               kmesh(kmesh(46)) = nintprop
               kmesh(kmesh(46)+1) = nrealprop
               kmesh(kmesh(46)+2) = 0
               kmesh(kmesh(46)+3) = 0
               call prinlg ( 'kmesh', kmesh(3)+4, dummy, kmesh )
            end if
            if ( nintprop>0 ) then

!           --- nintprop>0, Read integer properties

               call ini081 ( ibuffr, kmesh(kmesh(46)+2), &
                             nintprop*nelgrp, 'intprop' )
               ipkmsw1 = inigettext ( kmesh(kmesh(46)+2), 'intprop' )
               if ( irefc>=0 ) then
                  read(irefc,*) &
                     (ibuffr(ipkmsw1+i),i=0,nintprop*nelgrp-1)
               else
                  read(-irefc) (ibuffr(ipkmsw1+i),i=0,nintprop*nelgrp-1)
               end if

            end if
            if ( nrealprop>0 ) then

!           --- nrealprop>0, Read real properties

               call ini080 ( ibuffr, kmesh(kmesh(46)+3), &
                             nrealprop*nelgrp, 'realprop' )
               ipkmsw2 = inidgttext ( kmesh(kmesh(46)+3), 'kmeshw2' )
               if ( irefc>=0 ) then
                  read(irefc,*) &
                     (buffer(ipkmsw2+i),i=0,nrealprop*nelgrp-1)
               else
                  read(-irefc) &
                     (buffer(ipkmsw2+i),i=0,nrealprop*nelgrp-1)
               end if

            end if

         end if

         if ( ihelp(3)>0 ) then

!        --- kmesh part x exists

            kmesh(42) = ihelp(3)
            lenrl = 3*ndim
            lenin = 6*(ndim-1)
            lenkmsx = ihelp(6)
            if ( lenkmsx==0 ) lenkmsx = kmesh(42)*(lenin+intlen*lenrl)
            call ini081 ( ibuffr, kmesh(47), lenkmsx, 'kmeshx' )
            iprmsx = inigettext ( kmesh(47), 'kmeshx' )
            ipkmsx = iprmsx+lenrl*intlen*kmesh(42)
            iprmsx = inidbl(iprmsx)
            lenkmsx = lenkmsx-lenrl*intlen*kmesh(42)
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (ibuffr(ipkmsx+i),i=0,lenkmsx-1)
               read(irefc,*) (buffer(iprmsx+i),i=0,lenrl*kmesh(42)-1)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (ibuffr(ipkmsx+i),i=0,lenkmsx-1)
               read(-irefc) (buffer(iprmsx+i),i=0,lenrl*kmesh(42)-1)
            end if
            if ( debug ) write(irefwr,*) text
            if ( ndim==3 ) then

!           --- ndim = 3, also read coordinates and topology of the obstacles

               numobst = kmesh(42)
               do iobst = 1, numobst
                  iprmsx = inigettext ( kmesh(47), 'kmeshx' )
                  ipkmsx = iprmsx+lenrl*intlen*kmesh(42)
                  isurnr = nsurfs+1
                  inpelm = ibuffr(ipkmsx-1+12*(iobst-1)+10)
                  nelem  = ibuffr(ipkmsx-1+12*(iobst-1)+11)
                  npoint = ibuffr(ipkmsx-1+12*(iobst-1)+12)
                  index = 0
                  length = inpelm*nelem
                  call ini081 ( ibuffr, index, length, 'kmeshc_obst' )
                  iprmsx = inigettext ( kmesh(47), 'kmeshx' )
                  ipkmsx = iprmsx+lenrl*intlen*kmesh(42)
                  ibuffr(ipkmsx-1+12*(iobst-1)+8) = index
                  ipkmsc = inigettext ( index, 'index' )
                  if ( irefc>=0 ) then
                     read(irefc,*) (ibuffr(ipkmsc+i),i=0,length-1)
                  else
                     read(-irefc) (ibuffr(ipkmsc+i),i=0,length-1)
                  end if
                  index = 0
                  length = ndim*npoint
                  call ini080 ( ibuffr, index, length, 'coor_obst' )

!                 --- Reserve work space

                  nwork = 11*npoint+1+4*nelem
                  call ini062 ( ibuffr, nwork, ipiwrk )
                  iprmsx = inigettext ( kmesh(47), 'kmeshx' )
                  ipkmsx = iprmsx+lenrl*intlen*kmesh(42)
                  iprmsx = inidbl(iprmsx)
                  ipcoor = inidgttext ( index, 'coor' )
                  ibuffr(ipkmsx-1+12*(iobst-1)+9) = index
                  if ( irefc>=0 ) then
                     read(irefc,*) (buffer(ipcoor+i),i=0,length-1)
                  else
                     read(-irefc) (buffer(ipcoor+i),i=0,length-1)
                  end if

!                 --- Fill part of rmeshx if necessary, because it has not
!                     been filled actually
!                     This is for example the case if the mesh is not created by
!                     sepmesh

                  call mshobsin33 ( ibuffr(ipkmsx), buffer(iprmsx), &
                                    buffer(ipcoor), ibuffr(ipkmsc), &
                                    isurnr, inpelm, nelem, npoint, &
                                    iobst, ibuffr(ipiwrk), nwork )

               end do

            end if

!           --- Clear reference to array iobgrid

            do i = 1, kmesh(42)
               ibuffr(ipkmsx+(i-1)*lenin+ndim+2) = 0
            end do

         end if
         if ( ihelp(9)>0 ) then

!        --- ihelp(9)>0, Read volume_surfs
!            Warning this subroutine uses memory management

            text = '//Volume_surfs'
            if ( debug ) write(irefwr,*) text
            call mshreadint ( ibuffr, irefc, text, kmesh(57), name10, &
                              'volume_surfs' )

         end if
         if ( ihelp(7)>0 ) then

!        --- ihelp(7)>0, Read outer curves
!            Warning this subroutine uses memory management

            text = '//Outercurves'
            if ( debug ) write(irefwr,*) text
            call mshreadint ( ibuffr, irefc, text, kmesh(58), name10, &
                              'ioutercurvs' )

         end if
         if ( ihelp(8)>0 ) then

!        --- ihelp(8)>0, Read normals
!            Warning this subroutine uses memory management

            text = '//Normals'
            if ( debug ) write(irefwr,*) text
            call mshreadreal ( ibuffr, buffer, irefc, text, kmesh(61), &
                               name10, 'normal' )
            if ( kmesh(13)>0 .and. kmesh(61)>0 ) then

!           --- There are volumes, fill array surfnodes

               call mshfillsurnod ( ibuffr, kmesh )

            end if

         end if

         if ( ihelp(10)>0 ) then

!        --- ihelp(10)>0, Read curvesup
!            Warning this subroutine uses memory management

            text = '//Curveusp'
            if ( debug ) write(irefwr,*) text
            call mshreadint ( ibuffr, irefc, text, kmesh(62), name10, &
                              'curveusp' )

         end if

      end if

      renumber = ichois/=-1
      if ( mainsubr==6 ) then

!     --- mainsubr = 6, call by sepmakeparmesh
!         Always a renumbering is applied based on Cuthill-McKee

         renumber = .true.

      end if  ! ( mainsubr==6 )

      if ( renumber ) then

!     --- Perform renumbering if ichois # -1

         if ( .not. parallel .and. ivers>=4 .and. ichois==-2 ) then
            jchois(1) = 1000
         else if ( ivers>=1 .and. ichois==-2 ) then
            jchois(1) = 0
         else if ( ichois==-2 ) then
            jchois(1) = 9
         else
            jchois(1) = ichois
         end if

         if ( mainsubr==6 ) then

!        --- mainsubr = 6, call by sepmakeparmesh
!            Always a renumbering is applied based on Cuthill-McKee

            jchois(1) = -10   ! 10 positions are filled by the user
            jchois(2) = 1     ! Cuthill-Mckee algorithm
            jchois(3) = 2     ! Use only the renumbered algorithm
            jchois(4) = 0     ! iprint
            jchois(5) = 3     ! Priority for vertex nodes
            jchois(6) = 3     ! Priority for mid point nodes
            jchois(7) = 3     ! Priority for centroids
            jchois(8) = 0     ! no renumbering per level
            jchois(9) = 0     ! Automatic start
            jchois(10) = 1    ! start value (not used)

         end if  ! ( mainsubr==6 )
         call topol ( ibuffr, jchois, kmesh )

      end if

      kmesh(2) = 100

      if ( check_hanging_nodes ) then

!     --- check_hanging_nodes is true, check mesh (hanging nodes)

         call mshcheckelms ( ibuffr, kmesh )

      end if

      if ( ndim==3 .and. kmesh(58)==0 .and. &
          (ncurvs<nsurfs .and. ncurvs<3 .or. nuspnt<1) ) then

!     --- ncurvs<nsurfs or nuspnt<ncurvs
!         Special situation, presumably this mesh has been created by
!         a third party mesh generator
!         Create extra surfaces, curves and user points if necessary
!         Make sure that all elements have positive jacobian

         call mshposjac ( ibuffr, buffer, kmesh )

         call mshusermesh ( ibuffr, kmesh )

      end if

!     --- Fill array intarmsh partly

      mmiinput = kmesh(34)
      if ( mmiinput<=0 ) go to 1000
      if ( inilen(mmiinput)<20 ) go to 1000  ! no input in iinput

      call ini060text ( ibuffr, mmiinput, 'iinput' )
      ipiinput = inigettext ( mmiinput, 'iinput' )
      call msh000 ( kmesh, ibuffr(ipiinput), jchois, 1 )

1000  call erclos ( 'msh071' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh071'
         ioutp = ioutpsav

      end if

      end
