      subroutine mshcopred ( ibuffr, buffer, kmesh1, kmesh2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-05-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the essential parts of array kmesh1 into kmesh2
!     Only those parts needed for subroutine refine are copied
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh1(*), kmesh2(*), ibuffr(*)
      double precision buffer(*)

!     buffer    i/o   Real buffer array in which all large data is stored
!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     kmesh1     i    KMESH array to be copied
!     kmesh2    i/o   Copy of array KMESH
!                     All positions must have been initialized before
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, kelmi, kelmm, kelmc, kelmk, kelml, kelmn, &
              mmiinput, mmrinput, kelmq, kelmr, kelms

!     i          Counting variable
!     kelmc      Memory management sequence number of array kmeshc
!     kelmi      Memory management sequence number of array coor
!     kelmk      Memory management sequence number of array kmeshk
!     kelml      Memory management sequence number of array kmeshl
!     kelmm      Memory management sequence number of array kmeshm
!     kelmn      Memory management sequence number of array kmeshn
!     kelmq      Memory management sequence number of array kmeshq
!     kelmr      Memory management sequence number of array kmeshr
!     kelms      Memory management sequence number of array kmeshs
!     mmiinput   Memory management sequence number with respect to iinput
!     mmrinput   Memory management sequence number with respect to rinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICOPARR      Copy one sepran array into the other, which may already
!                    have been created before.
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Check length of array kmesh2
!    Save memory management positions of old arrays of kmesh2
!    Copy parts of array kmesh
!    Copy kmesh part i (co-ordinates)
!    Copy kmesh part c (topology)
!    Copy kmesh part k (levels)
!    Copy kmesh part m (curve description)
!    Copy kmesh part n (surface description)
!    Copy kmesh parts q, r and s (description of curves and so on)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcopred' )

!     --- Check length of array kmesh2

      call prinlg ( 'kmesh2', kmesh1(3), buffer, kmesh2 )

!     --- Save memory management positions of old arrays of kmesh2

      kelmi = kmesh2(23)
      kelmc = kmesh2(17)
      kelmk = kmesh2(25)
      kelml = kmesh2(26)
      kelmm = kmesh2(27)
      kelmn = kmesh2(28)
      kelmq = kmesh2(31)
      kelmr = kmesh2(32)
      kelms = kmesh2(33)
      mmiinput = kmesh2(34)
      mmrinput = kmesh2(35)

!     --- Copy parts of array kmesh

      do i = 2, kmesh1(3)
         kmesh2(i) = kmesh1(i)
      end do

!     --- restore positions old arrays in ibuffr

      kmesh2(23) = kelmi
      kmesh2(17) = kelmc
      kmesh2(18) = 0
      kmesh2(19) = 0
      kmesh2(22) = 0
      kmesh2(24) = 0
      kmesh2(25) = kelmk
      kmesh2(26) = kelml
      kmesh2(27) = kelmm
      kmesh2(28) = kelmn
      kmesh2(29) = 0
      kmesh2(31) = kelmq
      kmesh2(32) = kelmr
      kmesh2(33) = kelms
      kmesh2(34) = mmiinput
      kmesh2(35) = mmrinput
      kmesh2(37) = 0
      kmesh2(43) = 0
      kmesh2(48) = 0
      kmesh2(49) = 0
      kmesh2(50) = 0
      kmesh2(51) = 0
      kmesh2(52) = 0
      kmesh2(53) = 0
      kmesh2(54) = 0
      kmesh2(55) = 0
      kmesh2(58) = 0
      kmesh2(60) = 0
      kmesh2(61) = 0

!     --- Copy kmeshl

      call inicoparr ( ibuffr, kmesh1(26), kmesh2(26), 'kmeshl-1', &
                       'kmeshl-2' )

!     --- Copy arrays kmesh parts q r and s if available

      if ( kmesh1(31)>0 ) then

!     --- kmeshq exists, copy
!         make sure old kmeshq is in ibuffr

         call inicoparr ( ibuffr, kmesh1(31), kmesh2(31), 'kmeshq-1', &
                          'kmeshq-2' )

      end if

      if ( kmesh1(32)>0 ) then

!     --- kmeshr exists, copy
!         make sure old kmeshr is in ibuffr

         call inicoparr ( ibuffr, kmesh1(32), kmesh2(32), 'kmeshr-1', &
                          'kmeshr-2' )

      end if

      if ( kmesh1(33)>0 ) then

!     --- kmeshs exists, copy
!         make sure old kmeshs is in ibuffr

         call inicoparr ( ibuffr, kmesh1(33), kmesh2(33), 'kmeshs-1', &
                          'kmeshs-2' )

      end if

!     --- Copy arrays iinput and rinput if available

      if ( kmesh1(34)>0 .and. kmesh1(35)>0 ) then

         call inicoparr ( ibuffr, kmesh1(34), kmesh2(34), 'iinput-1', &
                          'iinput-2' )

         call inicoparr ( ibuffr, kmesh1(35), kmesh2(35), 'rinput-1', &
                          'rinput-2' )

      end if
      call erclos ( 'mshcopred' )

      end
