      subroutine mshdel ( ibuffr, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.6  date 21-10-2005 correction kmesh(46)
!        version  2.5  date 08-12-2003 Separate subroutine for kmesh ad
!        version  2.4  date 04-10-2003 Replace part of subroutine by mshdelkmehh
!        version  2.3  date 01-01-2003 Replace part of subroutine by mshdelsel
!        version  2.2  date 22-11-2002 Extra part of kmesh
!        version  2.1  date 30-05-2002 Extra part of kmesh
!        version  2.0  date 03-04-1999 Extra parameter ibuffr
!        version  1.0  date 27-02-1997
!
!   copyright (c) 1997-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Delete array KMESH and all of its corresponding arrays
!
! **********************************************************************
!
!                       KEYWORDS
!
!     delete
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), ibuffr(*)

!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     kmesh     i/o   KMESH array to be deleted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, kelmab

!     i              Counting variable
!     kelmab         Starting position of KMESH part AB in array KMESH
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI066TEXT     delete temporary array
!     MSHDELKMESHAD  Delete kmesh part ad
!     MSHDELKMESHH   Delete kmesh part h and subarrays
!     MSHDELSEL      Delete a selected set of subarrays of kmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      75   Array KMESH has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshdel' )

      if ( kmesh(2)/=100 ) then

!     --- kmesh has not been filled

         call errint ( kmesh(2), 1 )
         call errsub ( 75, 1, 0, 0 )
         go to 1000

      end if
      if ( kmesh(17)>0 ) &
         call ini066text ( ibuffr, kmesh(17), 'kmeshc' )

!     --- Delete kmeshh and related arrays

      call mshdelkmeshh ( ibuffr, kmesh )

      if ( kmesh(23)>0 ) &
         call ini066text ( ibuffr, kmesh(23), 'coor' )
      do i = 26, 28
         if ( kmesh(i)>0 ) &
            call ini066text ( ibuffr, kmesh(i), 'kmeshl' )
      end do
      do i = 30, 35
         if ( kmesh(i)>0 ) &
            call ini066text ( ibuffr, kmesh(i), 'kmeshp' )
      end do
      if ( kmesh(45)>0 ) &
         call ini066text ( ibuffr, kmesh(45), 'kelmv' )
      if ( kmesh(47)>0 ) &
         call ini066text ( ibuffr, kmesh(47), 'kelmx' )

      kelmab = kmesh(49)-1
      if ( kelmab>0 ) then
         do i = 1, 15
            if ( kmesh(kelmab+i)>0 ) &
               call ini066text ( ibuffr, kmesh(kelmab+i), 'kelmab' )
         end do
      end if

!     --- Delete kmeshad and related arrays

      call mshdelkmeshad ( ibuffr, kmesh )

      if ( kmesh(57)>0 ) &
         call ini066text ( ibuffr, kmesh(57), 'kelmag' )
      if ( kmesh(62)>0 ) &
         call ini066text ( ibuffr, kmesh(62), 'kelmak' )

!     --- Remove a selective part of kmesh

      call mshdelsel ( ibuffr, kmesh )

      do i = 4, kmesh(3)
         kmesh(i) = 0
      end do
      kmesh(2) = 0
1000  call erclos ( 'mshdel' )

      end
