      subroutine prcopsepvec4 ( func, usol, kprobp, npoint, idegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy one degree of freedom of sepran vector in user vector
!     All nodes in the mesh are used
!     The number of degrees of freedom per point is variable
!     If a node is not filled the value 0 is substituted
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     sepran_vector
!     user_vector
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: npoint, idegfd, kprobp(npoint,*)
      double precision, intent(in) :: usol(*)
      double precision, intent(out) :: func(npoint)

!     func           o    Contains the function values
!     idegfd         i    Degree of freedom sequence number
!     kprobp         i    SEPRAN array KPROB part p containing the positions of
!                         the unknowns per point
!     npoint         i    Number of nodal points
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ipos
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopsepvec4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopsepvec4'
         write(irefwr,1) 'npoint,idegfd', npoint,idegfd
         call prinin ( kprobp, npoint, 'kprobp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      func = 0d0

      do i = 1, npoint

         ipos = kprobp(i,idegfd)

         if ( ipos>0 ) func(i) = usol(ipos)

      end do  ! i = 1, npoint

1000  call erclos ( 'prcopsepvec4' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( func, npoint, 'func' )
         write(irefwr,*) 'End prcopsepvec4'

      end if  ! ( debug )

      end
