      subroutine prcopymmsep ( ibuffr, mmvec1, mmvec2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 16-12-2005
!
!   copyright (c) 2005-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy vector corresponding to memory management sequence number 1
!     into vector corresponding to memory management sequence number 2
!     If the second is not zero both must have equal length
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons3f90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cnamtbf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), mmvec1, mmvec2

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     mmvec1         i    memory management number of vector to be copied
!     mmvec2        i/o   memory management number of vector to which vec1
!                         must be copied.
!                         If 0, a new array is created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipvec1, ipvec2
      integer length, i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ipvec1         Starting address of array VECTOR1
!     ipvec2         Starting address of array VECTOR2
!     length         Help parameter to indicate the length of arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inilen
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI081         Create permanent space in array IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2796   Different length for arrays to be copied on each other
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopymmsep' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopymmsep'

      end if
      if ( ierror/=0 .or. mmvec1==mmvec2 ) go to 1000

!     --- Activate vector 1

      call ini060text ( ibuffr, mmvec1, 'vector_1' )
      if ( ierror/=0 ) go to 1000
      length = inilen(mmvec1)

      if ( mmvec2<=0 .or. mmvec2>iinfor ) then

!     --- vector 2 has not been filled
!         create new array

         call ini081 ( ibuffr, mmvec2, length, 'vector_2' )

      else

!     --- vector 2 has been filled
!         check length of array and activate

         if ( inilen(mmvec2)/=length ) then

!        --- Array length not the same

            call errint ( mmvec1, 1 )
            call errint ( length, 2 )
            call errint ( mmvec2, 3 )
            call errint ( inilen(mmvec2), 4 )
            call errchr ( namarr(mmvec1), 1 )
            call errchr ( namarr(abs(mmvec2)), 2 )
            call errsub ( 2796, 4, 0, 2 )
            go to 1000

         end if  ! ( inilen(mmvec2)/=length )
         call ini060text ( ibuffr, mmvec2, 'vector_2' )

      end if  ! ( mmvec2<=0 .or. mmvec2>iinfor )

!     --- Compute starting addresses

      ipvec1 = inigettext ( mmvec1, 'vector_1' )
      ipvec2 = inigettext ( mmvec2, 'vector_2' )

!     --- Copy

      do i = 0, length-1
         ibuffr(ipvec2+i) = ibuffr(ipvec1+i)
      end do  ! i = 0, length-1

1000  call erclos ( 'prcopymmsep' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopymmsep'

      end if

      end
