      subroutine prcopyus2vecml ( sepvector, inpvec, nodes, nnodes, &
                                  nunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy user vector to sepran vector
!     The user vector has nunkp unknowns per point and is defined in a set of
!     nodes only
!     The output vector is defined in all nodes and has exactly nunkp
!     degrees of freedom per point
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  nnodes, nodes(nnodes), nunkp
      double precision sepvector(nunkp,*), inpvec(nunkp,nnodes)

!     inpvec         i    Input vector of length nunkp x nnodes containing the
!                         vector to be copied
!     nnodes         i    Number of nodes
!     nodes          i    Contains the node numbers that must be copied
!     nunkp          i    Maximum number of unknowns in a point
!     sepvector      o    sepran vector consisting of nunkp degrees of freedom
!                         per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopyus2vecml' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopyus2vecml'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      sepvector(1:nunkp,nodes(1:nnodes)) = inpvec(1:nunkp,1:nnodes)

1000  call erclos ( 'prcopyus2vecml' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopyus2vecml'

      end if  ! ( debug )

      end
