      function prderuserp ( ibuffr, buffer, isol, kmesh, kprob, iuserp, &
                            ix )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-07-2015 Jump in case of error
!        version  1.0  date 05-11-2009
!
!   copyright (c) 2009-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the derivative of the solution vector in
!     a given user point in direction ix
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(*), kmesh(*), kprob(*), iuserp, ix
      double precision prderuserp, buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     iuserp         i    User point sequence number
!     ix             i    indication what type of derivative must be computed
!                         See DERIVA
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     prderuserp     o    value of the derivative
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, iinder(15), ioutvc(5), iuser(1), iskpel(1), &
              icurvs(1)
      double precision user(1), output
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     icurvs         List of curves to be used
!     iinder         User input array for subroutine DERIV
!     ioutvc         Standard SEPRAN array containing information
!                    concerning the interpolated function on the line.
!                    ioutvc is an array of structure 119
!                    Both the one-dimensional co-ordinates as the function
!                    values are stored in the array corresponding to ioutvc
!     iskpel         Array containing the element groups to be skipped
!     iuser          Integer user array to pass user information from
!                    main program to subroutine. (Dummy)
!     node           Node number
!     output         Dummy
!     user           Real user array to pass user information from
!                    main program to subroutine. (Dummy)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer prnodeuserp
      double precision prvalnode

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI066TEXT     delete temporary array
!     PRINTVBF       Print a SEPRAN array
!     PRNODEUSERP    Return with nodal point number corresponding to user point
!     PRVALNODE      Get the value of the solution vector in a given node
!     TO0100         Performs the actual computation of the derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get node number corresponding to user point
!     Compute derivatives
!     Get value in node
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prderuserp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prderuserp'
         write(irefwr,1) 'iuserp, ix', iuserp, ix
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get node number corresponding to user point

      node = prnodeuserp ( ibuffr, kmesh, iuserp )
      if ( debug ) write(irefwr,1) 'node', node
      if ( ierror/=0 ) go to 1000

!     --- Compute derivatives

      iinder = 0         ! clear array
      ioutvc = 0         ! clear output vector
      iinder(1) = 4      ! calling subroutine
      iinder(2) = 1      ! output vector is of special structure
      iinder(4) = 1      ! icheld
      iinder(5) = ix     ! ix
      iinder(6) = 1      ! jdegfd
      iinder(7) = 1      ! ivec
      iinder(10) = 1     ! get from solution vector
      call to0100 ( ibuffr, buffer, iinder, ioutvc, kmesh, kprob, &
                    isol, iuser, user, output, iskpel, icurvs )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         call  printvbf ( ibuffr, buffer, ioutvc, kmesh, kprob, &
                          'outvc' )

!     --- Get value in node

      prderuserp = prvalnode ( ibuffr, buffer, ioutvc, kprob, node )
      if ( ierror/=0 ) go to 1000

!     --- remove output vector

      call ini066text ( ibuffr, ioutvc(1), 'outvc' )

1000  call erclos ( 'prderuserp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prderuserp'

      end if  ! ( debug )

      end
