      subroutine prfillnodsurfs ( ibuffr, kmesh, inum, inpsep, &
                                  nodes, nnodes, iplace )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 17-10-2013 Extra debug
!        version  2.1  date 21-11-2012 Long integers
!        version  2.0  date 03-04-2012 Extra parameter iplace
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes with nodal points as defined in array inpsep
!     Special case: nodes in surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), inpsep(*), nodes(*), nnodes, inum, &
              iplace

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prfillnodes
!     inum           i    See inpsep
!     iplace         i    Position where nodes must be filled
!                         See subroutine prfillnodes
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nnodes         o    Number of nodes
!     nodes          o    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipouter, ipkmsn, ipkmsc
      integer numnod, isrf1, isrf2, nsur
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipkmsc         Starting address of array KMESH part c
!     ipkmsn         Starting address of array KMESH part n
!     ipouter        Starting address of array outerboun
!     isrf1          First surface number
!     isrf2          Last surface number
!     nsur           Number of surfaces surrounding a volume
!     numnod         If>0, only numnod nodes per element must be used
!                    In that case ihelp is used
!                    If 0 all nodes are used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSURFSNOD     Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIACTM0       Actual body of subroutine INIACTMK
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRFILLNODSURF1 Fill array nodes for set of surfaces
!     PRFILLNODSURF2 Fill array nodes for set of surfaces
!                    Only a restricted number of nodes per element is used
!     PRFILLNODSURF3 Fill array nodes for all outer surfaces
!     PRFILLNODSURF5 Fill array nodes for set of surfaces
!                    Only a restricted number of nodes per element is used
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillnodsurfs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillnodsurfs'
         write(irefwr,1) 'inum, kmesh(28)', inum, kmesh(28)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kmesh(28)>0 ) &
         call ini060text ( ibuffr, -kmesh(28), 'kmeshn' )
      call ini060text ( ibuffr, -kmesh(17), 'kmeshc' )

      ipkmsc = inigettext ( kmesh(17), 'kmeshc' )
      ipkmsn = 1
      if ( kmesh(28)>0 ) &
         ipkmsn = inigettext ( kmesh(28), 'kmeshn' )

      if ( inum==-1 .or. iplace==24 ) then

!     --- inum = -1, special case for create
!         isrf1, 2 in inpsep(1:2)
!         information of skipping is stored in inpsep(3...)

         isrf1 = inpsep(1)
         isrf2 = inpsep(2)
         numnod = inpsep(3)
         if ( debug ) &
            write(irefwr,1) 'isrf1, isrf2, numnod', isrf1, isrf2, numnod

         if ( isrf1==-1 ) then

!        --- Special case isrf1 = -1: all outer boundaries
!            First activate or create the outer boundary (kmesh part ah)

            call iniactm0 ( ibuffr, ibuffr, 29, kmesh, kmesh )
            ipouter = inigettext ( kmesh(58), 'outerboun' ) ! starting address

            call prfillnodsurf3 ( kmesh, ibuffr(ipkmsn), nodes, nnodes, &
                                  inpsep(3), inpsep(4), &
                                  ibuffr(ipouter) )

         else if ( iplace==24 ) then

!        --- New definition of create input

            numnod = inpsep(2)
            if ( numnod==0 ) then

!           --- Standard case, all nodes in surfaces

               call ersurfsnod ( kmesh, ibuffr(ipkmsn), inpsep(1), &
                                 nodes, nnodes, inpsep(3), &
                                 kmesh(kmesh(36)), ibuffr(ipkmsc) )

            else

!           --- Special case, restricted set of nodes in surfaces

               call prfillnodsurf5 ( kmesh, inpsep(1), inpsep(3), &
                                     ibuffr(ipkmsn), nodes, nnodes, &
                                     kmesh(kmesh(36)), ibuffr(ipkmsc), &
                                     inpsep(2), inpsep(3+inpsep(1)) )

            end if  ! ( numnod==0 )

         else if ( numnod==0 ) then

!        --- Standard case, all nodes in surfaces isrf1 to isrf2

            call prfillnodsurf1 ( kmesh, isrf1, isrf2, ibuffr(ipkmsn), &
                                  nodes, nnodes, kmesh(kmesh(36)), &
                                  ibuffr(ipkmsc) )

         else

!        --- Special case, restricted set of nodes in surfaces isrf1 to isrf2

            call prfillnodsurf2 ( kmesh, isrf1, isrf2, ibuffr(ipkmsn), &
                                  nodes, nnodes, kmesh(kmesh(36)), &
                                  ibuffr(ipkmsc), inpsep(3), inpsep(4) )

         end if  ! ( isrf1==-1 )

      else if ( inum>0 ) then

!     --- inum>0, Use inpsep which contains the surface numbers

         call ersurfsnod ( kmesh, ibuffr(ipkmsn), inum, nodes, &
                           nnodes, inpsep, kmesh(kmesh(36)), &
                           ibuffr(ipkmsc) )

      else if ( inum==0 ) then

!     --- inum = 0, inpsep contains old array icurvs

         nsur = inpsep(1)
         if ( nsur==0 ) nsur = 1
         if ( debug ) then
            write(irefwr,1) 'nsur', nsur
            call prinin ( inpsep(2), nsur, 'surfaces' )
         end if  ! ( debug )

         if ( nsur>0 ) then

!        --- inpsep(1)>= 0, surface numbers stored from position 2

            call ersurfsnod ( kmesh, ibuffr(ipkmsn), nsur, nodes, &
                              nnodes, inpsep(2), kmesh(kmesh(36)), &
                              ibuffr(ipkmsc) )

         else

!        --- inpsep(1)<0, surface numbers from inpsep(2 to 3)

            isrf1 = inpsep(2)
            isrf2 = inpsep(3)
            if ( debug ) write(irefwr,1) 'isrf1, isrf2', isrf1, isrf2

            call prfillnodsurf1 ( kmesh, isrf1, isrf2, ibuffr(ipkmsn), &
                                  nodes, nnodes, kmesh(kmesh(36)), &
                                  ibuffr(ipkmsc) )

         end if  ! ( ichoice>0 )

      end if  ! ( inum>0 )

1000  call erclos ( 'prfillnodsurfs' )
      if ( debug ) then

!     --- Debug information

         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End prfillnodsurfs'

      end if  ! ( debug )

      end
