      subroutine prfindcompcur ( istore, icount, kmeshq, icurcompound, &
                                 iwork, ncurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-02-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find compound curve corresponding to set of curves
!     If not found return with zero
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     common
!     compound_curve
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(*), icount, kmeshq(*), icurcompound, &
              iwork(*), ncurvs

!     icount         i    Number of curves in istore
!     icurcompound   o    Sequence number of compound curve
!                         If no compound curve is found: 0
!     istore         i    Contains the set of curves which might form a compound
!                         curve
!     iwork               integer work array of length ncurvs
!                         is used to mark the curves in istore
!     kmeshq         i    KMESH part q, contains the subcurve numbers of
!                         each curve
!     ncurvs         i    Number of curves in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numcurves, ipstart, icurnr, jcurnr
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     icurnr         Curve sequence number
!     ipstart        Starting address in array
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     numcurves      Number of curves in compound curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindcompcur' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindcompcur'

      end if
      if ( ierror/=0 ) go to 1000

      icurcompound = 0

!     --- Clear array iwork

      do i = 1, ncurvs
         iwork(i) = 0
      end do  ! i = 1, ncurvs

!     --- Set iwork 1 for all curves in istore

      do i = 1, icount
         iwork(istore(i)) = 1
      end do  ! i = 1, icount

      do icurnr = 1, ncurvs          ! loop over all curves in kmeshq

         numcurves = kmeshq(icurnr+1)-kmeshq(icurnr) ! number of subcurves

         if ( numcurves==icount ) then

!        --- Compound curve found, with the same number of subcurves
!            This is a candidate

            ipstart = kmeshq(icurnr)    ! starting address in kmeshq -1
            do i = 1, numcurves

               jcurnr = abs(kmeshq(ipstart+i))  ! curve in compound curve
               if ( iwork(jcurnr)==0 ) go to 200 ! curves are not the
                                                   ! same: jump
            end do  ! i = 1, numcurves

!           --- Since the code reaches this position we have found the
!               curve

            icurcompound = icurnr  ! store curve number
            go to 1000

200      end if  ! ( numcurves==icount )

      end do  ! icurnr = 1, ncurvs

1000  call erclos ( 'prfindcompcur' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindcompcur'

      end if

      end
