      subroutine prfindelclear ( nodes, nnodes, iwork, kmeshd, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-12-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Clear array iwork
!     Only the few entries that are filled by prinnodeelms2 are set to 0
! **********************************************************************
!
!                       KEYWORDS
!
!     clear
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodes(*), nnodes, iwork(*), kmeshd(*), npoint

!     iwork         i/o   integer work array that must be set to zero
!                         for all elements corresponding to nodes
!                         in array nodes
!     kmeshd         i    KMESH part d
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, inode, ipstart, nelements, i, ielem
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ielem          Element number
!     inode          Node sequence number
!     ipstart        Starting address in array
!     nelements      number of elemenst corresponding to a node
!     node           Loop variable to find node numbers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     See subroutine prinnodeelms2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindelclear' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindelclear'

      end if
      if ( ierror/=0 ) go to 1000

      do node = 1, nnodes-1

         inode = nodes(node)
         ipstart = kmeshd(inode)+npoint+1
         nelements = kmeshd(inode+1)-kmeshd(inode)
         do i = 1, nelements
            ielem = kmeshd(ipstart+i)
            iwork(ielem) = 0
         end do  ! i = 1, nelements

      end do  ! inode = 1, nnodes-1

1000  call erclos ( 'prfindelclear' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindelclear'

      end if

      end
