      subroutine prfindsurfnode ( ibuffr, inode, kmesh, numsurfaces, &
                                  surfacesfound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 08-04-2005
!
!   copyright (c) 2005-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if node inode is in one or more of the surfaces
!     and return with corresponding surface numbers
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), inode, kmesh(*), numsurfaces, surfacesfound(*)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inode          i    Node sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     numsurfaces    o    Number of surfaces stored in array surfacesfound
!     surfacesfound  o    Array with surface numbers in which inode can be found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmsn, ipkmsr
      integer nsurfs
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipkmsn         Starting address of array KMESH part n
!     ipkmsr         Starting address of array KMESH part r
!     nsurfs         Number of surfaces in mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRFINDSURFNOD1 Check if node inode is in one or more of the surfaces
!                    and return with corresponding surface numbers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract constants
!     activate arrays
!     Compute starting addresses
!     Fill surfacesfound
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindsurfnode' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindsurfnode'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extract constants

      nsurfs = kmesh(12)  ! number of surfaces
      if ( nsurfs==0 ) go to 1000

!     --- activate arrays

      call ini060text ( ibuffr, kmesh(28), 'kmeshn' )
      if ( kmesh(32)>0 ) call ini060text ( ibuffr, kmesh(32), 'kmeshr' )

!     --- Compute starting addresses

      ipkmsn = inigettext ( kmesh(28), 'kmeshn' )
      ipkmsr = 1
      if ( kmesh(32)>0 ) ipkmsr = inigettext ( kmesh(32), 'kmeshr' )

!     --- Fill surfacesfound

      call prfindsurfnod1 ( inode, ibuffr(ipkmsn), nsurfs, numsurfaces, &
                            surfacesfound, ibuffr(ipkmsr), kmesh )

1000  call erclos ( 'prfindsurfnode' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindsurfnode'

      end if

      end
