      subroutine prgetvalarr ( buffer, ibuffr, kprob, isol, nodes, &
                               nnodes, idegfd, uservec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the contents of a sepran solution array into a user
!     array for a set of nodes and a specific degree of freedom
! **********************************************************************
!
!                       KEYWORDS
!
!     get_parameter
!     value
!     variable
!     array
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kprob(*), nnodes, nodes(nnodes), idegfd, &
              isol(5)
      double precision buffer(*), uservec(nnodes)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     idegfd         i    Degree of freedom sequence number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!                         The corresponding array contains the values that must
!                         be copied
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers that must be filled
!     uservec        o    The values are copied in this array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipusol, ipkprf
      integer indprf, indprp, nunkp, nphys
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     ipkprf         Starting address of array KPROB part f
!     ipusol         Starting address of solution array
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     nunkp          Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext, inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCOPARR   Copy the contents of array uold into array unew for
!                    all nodes stored in array nodes and the degrees of
!                    freedom indicated in kdeg
!                    uold a sepran array, unew is a user array
!     EROPEN         Produces concatenated name of local subroutine
!     INI061         compute pointers to degrees of freedom of vectors
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prgetvalarr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prgetvalarr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate array isol and corresponding arrays

      call ini061 ( ibuffr, isol, kprob, indprf, indprp, &
                    nunkp, nphys )
      if ( ierror/=0 ) go to 1000

!     --- Get starting addresses

      ipusol = inidgttext ( isol(1), 'usol' )
      ipkprf = 1
      if ( indprf>0 ) ipkprf = inigettext ( indprf, 'kprobf' )

      call erfillcoparr ( nodes, idegfd, buffer(ipusol), uservec, &
                          indprf, ibuffr(ipkprf), nphys, nnodes )

1000  call erclos ( 'prgetvalarr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prgetvalarr'

      end if  ! ( debug )

      end
