      subroutine princonsta ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-04-2013 Loop in case of series
!        version  1.0  date 26-02-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print value of constant from block constants
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     constant
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/csepcmf90'
      include 'SPcommon/cusnamef90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readcostring
!     istart        i/o   Starting address in array inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, nch, itype, numarrays
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iseq           Sequence number
!     itype          Type of quantity to be printed
!     nch            Number of characters in a string
!     numarrays      Number of indexed vectors in vector array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'princonsta' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from princonsta'
         call prinin ( inpsep, 2, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itype = inpsep(1)   ! type of item
      iseq = inpsep(2)  ! sequence number

      if ( iseq>5000 ) then

!     --- Complete array

         iseq = iseq-5000
         if ( itype==2 ) then
            numarrays = dimcons(iseq)
         else
            numarrays = dimcons(maxints+iseq)
         end if  ! ( itype==2 )

      else

!     --- Not an array

         numarrays = 1

      end if  ! ( iseq>5000 )

      if ( debug ) write(irefwr,1) 'iseq, numarrays', iseq, numarrays

      if ( itype==2 ) then

!     --- integer found

         nch = index ( chincons(iseq), ' ' ) -1
         if ( nch<0 ) nch = 32
         if ( numarrays==1 ) then
            write(irefwr,100) chincons(iseq)(1:nch), incons(iseq)
100         format ( a,' = ', 3x, i8 )
         else
            call prinin ( incons(iseq), numarrays, &
                          chincons(iseq)(1:nch-3) )
         end if  ! ( numarrays==1 )

      else if ( itype==3 ) then

!     --- real found

         nch = index ( chrlcons(iseq), ' ' ) -1
         if ( nch<0 ) nch = 32

         if ( debug ) then
             write(irefwr,1) 'nch', nch
             write(irefwr,*) 'chrlcons ', chrlcons(iseq)(1:nch)
         end if  ! ( debug )

         if ( numarrays==1 ) then
            write(irefwr,110) chrlcons(iseq)(1:nch), rlcons(iseq)
110         format ( a,' = ', 3x, ss, 1pe13.5e2 )
         else
            call prinrl ( rlcons(iseq), numarrays, &
                          chrlcons(iseq)(1:nch-3) )
         end if  ! ( numarrays==1 )

      else

!     --- string found

         nch = index ( texts(iseq), ' ' ) -1
         if ( nch<0 ) nch = 80
         write(irefwr,*) texts(iseq)(1:nch)

      end if  ! ( itype==12 )

!     --- Raise istart

      istart = istart+3

1000  call erclos ( 'princonsta' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End princonsta'

      end if  ! ( debug )

      end
