      function prnodeuserp ( ibuffr, kmesh, iuserp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 05-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with nodal point number corresponding to user point
!
! **********************************************************************
!
!                       KEYWORDS
!
!     user_point
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer prnodeuserp, ibuffr(*), kmesh(*), iuserp

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iuserp         i    User point sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     prnodeuserp    o    nodal point number corresponding to user point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmsl
      integer nuspnt
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipkmsl         Starting address of array KMESH part l
!     nuspnt         Number of user points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3036   user point number has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prnodeuserp' )
      prnodeuserp = 0
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prnodeuserp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nuspnt = kmesh(10)  ! number of user points

      if ( iuserp<1 .or. iuserp>nuspnt ) then

!     --- Error user point number out of range

         call errint ( iuserp, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 3036, 2, 0, 0 )
         go to 1000

      end if  ! ( iuserp<1 .or. iuserp>nuspnt )

!     --- Activate kmesh part L

      call ini060text ( ibuffr, kmesh(26), 'kmeshl' )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting address

      ipkmsl = inigettext ( kmesh(26), 'kmeshl' )

!     --- Fill node number

      prnodeuserp = ibuffr(ipkmsl+iuserp-1)

1000  call erclos ( 'prnodeuserp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prnodeuserp'

      end if  ! ( debug )

      end
