      subroutine prodelsel ( ibuffr, kprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-03-2007 Clear kprob(48)
!        version  1.0  date 12-04-2003
!
!   copyright (c) 2003-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Delete a selective part of array KPROB and all of its corresponding arrays
!
! **********************************************************************
!
!                       KEYWORDS
!
!     delete
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprob(*), ibuffr(*)

!     ibuffr    i/o  Integer buffer array in which all large data is stored
!     kprob     i/o   KPROB array to be deleted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nprob, iprob, ipkprb, kprobe, ndtvec, kelgrp

!     i          Counting variable
!     ipkprb     Starting address of array KPROB for actual problem number - 1
!     iprob      Actual problem number
!     kelgrp     sum over all standard elements of number of points in standard
!                elements
!     kprobe     starting address in kprob part e
!     ndtvec     Number of vectors of special structure
!     nprob      Number of various problems
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniprb

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI066TEXT     delete temporary array
!     INIPRB         Get value of ipkprb as function of iprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array KPROB has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prodelsel' )
      if ( kprob(2)/=101 ) then

!     --- kprob has not been filled

         call errint ( kprob(2), 1 )
         call errsub ( 26, 1, 0, 0 )
         go to 1000
      end if
      nprob = kprob(40)
      do iprob = 1, nprob
         ipkprb = iniprb(iprob,kprob)
         kprobe = kprob(ipkprb+18)
         if ( kprobe>0 ) then

!        --- Delete KPROB part e

            ndtvec = kprob(ipkprb+6)
            kelgrp = kprob(ipkprb+39)
            kprobe = kprobe+ndtvec*kelgrp+ipkprb
            do i = 0, ndtvec-1
               if ( kprob(kprobe+i)>0 ) &
                  call ini066text ( ibuffr, kprob(kprobe+i), 'kprobe' )
            end do
         end if
         if ( kprob(ipkprb+19)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+19), 'kprobf' )
         if ( kprob(ipkprb+20)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+20), 'kprobg' )
         if ( kprob(ipkprb+21)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+21), 'kprobh' )
         if ( kprob(ipkprb+22)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+22), 'kprobi' )
         if ( kprob(ipkprb+24)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+24), 'kprobk' )
         if ( kprob(ipkprb+25)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+25), 'kprobl' )
         if ( kprob(ipkprb+26)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+26), 'kprobm' )
         if ( kprob(ipkprb+27)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+27), 'kprobn' )
         if ( kprob(ipkprb+35)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+35), 'kprobp' )
         if ( kprob(ipkprb+36)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+36), 'kprobq' )
         if ( kprob(ipkprb+37)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+37), 'kprobr' )
         if ( kprob(ipkprb+38)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+38), 'kprobs' )
         if ( kprob(ipkprb+43)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+43), 'kprobt' )
         if ( kprob(ipkprb+44)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+44), 'kprobu' )
         if ( kprob(ipkprb+46)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+46), 'kprobv' )
         kprob(ipkprb+48) = 0  ! Set maxcontc to 0
         if ( kprob(ipkprb+47)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+47), 'kprobw' )
         do i = ipkprb+51, ipkprb+51
            if ( kprob(i)>0 )  then

!           --- Fictitious domain part

               call ini066text ( ibuffr, kprob(ipkprb+kprob(i)+3), &
                                 'kprobfict' )

            end if
         end do
         if ( kprob(ipkprb+53)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+53), 'kprobaa' )
         if ( kprob(ipkprb+55)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+55), 'blockunkinfo' )
         if ( kprob(ipkprb+56)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+56), 'sendinfo' )
         if ( kprob(ipkprb+57)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+57), 'innercontr' )
         if ( kprob(ipkprb+58)>0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+58), 'kprobab' )

      end do

1000  call erclos ( 'prodelsel' )
      end
