      subroutine prparglob ( n, rhsd, parinfo, icontext )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 31-03-2015 Use prparglob1
!        version  2.0  date 07-10-2014 New parameter list
!        version  1.3  date 24-03-2013 Use inibsendmpi
!
!   copyright (c) 2004-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Send information to neighbouring processors in order to give all
!     local arrays the global value in the common points
!     only unprescribed unknowns are considered
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     global
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: n, icontext, parinfo(*)
      double precision, intent(inout) :: rhsd(n)

!     icontext       i    Tag for sending and receiving, is used as check
!     n              i    Size of the matrix
!     parinfo        i    Contains information about special parallel arrays
!                         See subroutine probparinfo1
!     rhsd          i/o   Real vector of length n containing the global
!                         vector to be used in parallel computations
!                         At input it contains the local values
!                         At output it contains the global values
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipblock, ipsend
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipblock        starting address of array blockunkinfo
!     ipsend         Starting address of array sendinfo
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRPARGLOB1     Send information to neighbouring processors in order to
!                    give all local arrays the global value in the common points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prparglob' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prparglob'
         write(irefwr,1) 'n', n
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      ipblock = parinfo(2)
      ipsend = parinfo(3)

      call prparglob1 ( n, rhsd, parinfo, icontext, ipsend, ipblock )

1000  call erclos ( 'prparglob' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prparglob'

      end if

      end
