      subroutine prplotzerolev1 ( coor, nmeshc, obstfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-06-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot zero level set curve
!     It is assumed that the plot is already active and hence opened
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     level_set
!     plot
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshc(*), obstfaces(*)
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     obstfaces      o    Array containing the face or edge numbers
!                         corresponding to the boundary of the obstacle
!                         Storage:
!                         Pos. 1 to numobst: starting addresses of obstacle
!                         Per obstacle:
!                         number of faces, followed by face numbers
!                         In this case obstacle is zero level set
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilev, istart, numedges, k, iedge, ipstart, inpelm, node, i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iedge          edge sequence number
!     ilev           Sequence number of level set being considered
!     inpelm         number of points per element
!     ipstart        Starting address in array
!     istart         First position used at a part
!     k              Counting variable
!     node           Node number
!     numedges       number of edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotzerolev1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplotzerolev1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ilev = 1  ! for the moment only one level set

      istart = obstfaces(ilev)
      numedges = obstfaces(istart)

      do k = 1, numedges     ! loop over edges

         iedge = obstfaces(istart+k)  ! edge
         ipstart = nmeshc(iedge)
         inpelm = nmeshc(ipstart)     ! number of nodes in edge
         node = nmeshc(ipstart+1)     ! first node of edge

!        --- plot with pen up to first node

         call plafep ( 4, coor(1,node), coor(2,node), 0d0, 0d0, 0 )

!        --- plot with pen down to other nodes

         do i = 2, inpelm

            node = nmeshc(ipstart+i)     ! next node of edge
            call plafep ( 5, coor(1,node), coor(2,node), 0d0, 0d0, 0 )

         end do  ! i = 2, inpelm

      end do  ! k = 1, numedges

1000  call erclos ( 'prplotzerolev1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prplotzerolev1'

      end if  ! ( debug )

      end
