      subroutine prveccombinat1 ( ibuffr, buffer, isol, iseqvec, vecs, &
                                  numvecrhs, kmesh, lenvec, istore, &
                                  store, icount, numrealrhs, nunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate right-hand side of vector expression
!     Special case: the complete real vector must be computed
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     expression
!     real
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(5,*), iseqvec, vecs(*), numvecrhs, &
              kmesh(*), lenvec, istore(*), icount, numrealrhs, nunkp
      double precision buffer(*), store(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     icount         i    Computed length of array istore
!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     isol           i    Standard SEPRAN array containing information of the
!                         respect solution array
!     istore         i    Contains the expression as stored in common
!                         cexpression, where all scalars and reals
!                         have been computed and stored in store
!                         References to these values are given by
!                         their position in store plus 1000
!                         The values of the vectors are stored in
!                         array vectors
!                         References to these vectors are given by
!                         their position in vectors plus 10000
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     lenvec         i    Length of left-hand side vector in expression
!     numrealrhs     i    Number of reals stored in store
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     nunkp          i    Maximum number of unknowns in a point
!     store          i    Contains real information of the expression
!                         See subroutine READEXPR, array work
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipvec
      integer error
      double precision, allocatable :: vectors(:,:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     ipvec          Starting address of output vector in buffer
!     vectors        Array of size lenvec x numvecrhs to store the vectors
!                    of the right-hand side in the nodes required
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     PREVALVECEXPR  Evaluate vector expression and fill actual values of
!                    vectors
!     PRVECFILL      Fill all vectors of right-hand side of expression
!                    in array vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prveccombinat1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prveccombinat1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( vectors(lenvec,numvecrhs), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lenvec*numvecrhs, 'vectors' )
      if ( ierror/=0 ) go to 1000

!     --- Fill all vectors

      call prvecfill ( ibuffr, buffer, isol, iseqvec, vecs, &
                       numvecrhs, kmesh, vectors, lenvec, nunkp )

!     --- Fill output vector

      ipvec = inidgttext ( isol(1,iseqvec), names(iseqvec) )

      call prevalvecexpr ( buffer(ipvec), vectors, lenvec, &
                           numvecrhs, istore, store, icount, &
                           numrealrhs, names(iseqvec) )

!     --- Deallocate vectors

      if ( allocated(vectors) ) deallocate ( vectors, stat = error )

1000  call erclos ( 'prveccombinat1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prveccombinat1'

      end if  ! ( debug )

      end
