      subroutine putscal ( varname, iseq, value )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Put the value of the scalar iseq in the block constants
!
! **********************************************************************
!
!                       KEYWORDS
!
!     fill_scalar
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) varname
      integer iseq
      double precision value

!     iseq           i    Sequence number
!     value          i    Value of variabe
!     varname        i    Name of variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqloc
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iseqloc        Local sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2157   variable name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chscalars
!     If ( name in chscalar ) then
!        put value in common
!     else
!        error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'putscal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from putscal'
         write(irefwr,1) 'iseq ', iseq
         write(irefwr,2) 'value', value
  1      format ( a, 1x, (10i9) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqloc = mod(iseq,100000000)
      iseqloc = mod(iseq,5000)

      if ( iseqloc>maxints+maxreals ) iseqloc = iseqloc-maxints-maxreals

      if ( iseqloc>0 ) then
         scalars(iseqloc) = value
      else
         call errchr ( varname, 1 )
         call errsub ( 2157, 0, 0, 1 )
      end if

1000  call erclos ( 'putscal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End putscal'

      end if  ! ( debug )

      end
