      function readarrindex()
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 10-04-2013 Extra debug
!        version  1.1  date 20-04-2012 Extra debug
!        version  1.0  date 03-02-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if the name of a constant vector or varibale is followed
!     by an index
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     constant
!     variable
!     array
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cpost5f90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/cwarnf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer readarrindex

!     readarrindex   o    Returns with index
!                         0:   no index found
!                         >0:  constant index found with value sequence number
!                              of integer
!                         <0:  scalar index found with -value sequence number
!                              of scalar
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, ncharsav, mxwarnsav
      character (len=240) stringsav, stringimsav
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iseq           Sequence number, see readarrindex
!     mxwarnsav      Help parameter to save the value of mxwarn
!     ncharsav       Help parameter to save the value of nchar
!     stringimsav    Help parameter to save the value of stringim
!     stringsav      Help parameter to save the value of string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getint, getnamevar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETINT         Get the value of an integer in the input block constants
!     GETNAMEVAR     Return with sequence number of variable in scalars
!     PRININ         print 1d integer array
!     READWR         Read keyword
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readarrindex' )
      debug = .false. .and. ioutp>=0

      stringsav = string
      stringimsav = stringim
      ncharsav = nchar
      mxwarnsav = mxwarn
      mxwarn = 0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readarrindex'
         write(irefwr,*) 'string ', string(1:nchar)
         write(irefwr,1) 'lastitem, nitems, ityprd', &
                          lastitem, nitems, ityprd
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseq = 0  ! sequence number found

      if ( lastitem>nitems-3 ) go to 1000  ! not enough items to be found

      if ( typeitems(lastitem+1)==5 .and. &
           typeitems(lastitem+3)==6 ) then

!     --- ( ? ) found, check the value of ?

         if ( typeitems(lastitem+2)==1 ) then

!        --- typeitems(lastitem+2)=1, hence integer found

            iseq = nint(realitems(lastitem+2))
            if ( debug ) write(irefwr,1) 'integer: iseq', iseq

         else if ( typeitems(lastitem+2)==3 ) then

!        --- typeitems(lastitem+2)=3, hence word found
!            First check if this is an integer constant

            lastitem = lastitem+2
            call readwr
            if ( debug ) write(irefwr,*) 'string ', string(1:nchar)
            iseq = getint ( string(1:nchar) )
            if ( debug ) write(irefwr,1) 'integer const: iseq', iseq

!           --- Next check if this is a variable

            if ( iseq==0 ) iseq = -getnamevar ( string(1:nchar) )
            if ( debug ) write(irefwr,1) 'variable: iseq', iseq
            lastitem = lastitem-2

         end if  ! ( typeitems(lastitem+1)==1 )

      end if  ! ( typeitems(lastitem+1)==5 ... )

      if ( iseq/=0 ) lastitem = lastitem+3

1000  call erclos ( 'readarrindex' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iseq, lastitem, ityprd', &
                          iseq, lastitem, ityprd
         write(irefwr,*) 'End readarrindex'

      end if  ! ( debug )

      readarrindex = iseq
      string = stringsav
      stringim = stringimsav
      nchar = ncharsav
      mxwarn = mxwarnsav

      end
