      subroutine readb5 ( ibuffr, numarr, intmat )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.2  date 04-06-2014 Allow -102 in intmat(2)
!        version  5.1  date 15-11-2012 Long integers
!        version  5.0  date 14-04-1999 Extra parameter ibuffr
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   readb5 reads array of intmat type from backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cfile2f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr, intmat(*), ibuffr(*)

!    ibuffr   i/o  Integer buffer array in which all large data is stored
!    intmat    o   Standard SEPRAN array containing information of the structure
!                  of the matrix
!    numarr    i   Array number with respect to the index in common cfile2
!                  for array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) nbufol, ipwork, ipmat, istart
      integer length, index, ihelp(2), k
      character *8 text

!     ihelp   Help array to store the length of the two parts of intmat
!     index   Record number with respect to file 2 from which array
!             must be read
!     ipmat   Starting position of part of INTMAT in IBUFFR
!     ipwork  Starting position of part of array in IBUFFR
!     istart  Help parameter to compute the position in IBUFFR, where the
!             length of a part is stored
!     k       General loop variable
!     length  Length of array KMESH
!     nbufol  Old value of nbuffr
!     text    String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     INI000  Copy integer array to other integer array possibly itself
!     INI069  Reserve space at the end of IBUFFR for a semi-permanent array
!     INI081  Create permanent space in array IBUFFR
!     INIGET  Compute pointer of array in IBUFFR
!     INSTOP  Stop the program
!     READFL  Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readb5' )
      length = iwork3 ( 2*numarr )
      index  = iwork3 ( 2*numarr-1 )
      if ( abs(intmat(2))/=102 ) then
         do k = 1, 5
            intmat(k) = 0
         end do
      end if

!     --- Reserve space in IBUFFR

      call ini069 ( ibuffr, ipwork, length, nbufol )
      if ( ierror/=0 ) go to 1000

!     --- Read all information from file 2

      call readfl ( index, 1, length, 1, 2, ibuffr(ipwork) )
      if ( ierror/=0 ) go to 1000
      istart = ipwork+4

      do k = 1, 2
         ihelp(k) = 0
         if ( ibuffr(ipwork+k+1)>0 ) then

!        --- intmat part k exists

            istart  = istart + 2
            ihelp(k) = ibuffr(istart)
            istart = istart + ihelp(k)
            write(text,110) k
110         format('intmat-',i1)
            call ini081 ( ibuffr, intmat(k+2), ihelp(k), text )

         else

            intmat(k+2) = 0

         end if

      end do
      if ( ierror/=0 ) go to 1000

      intmat(1) = ibuffr(ipwork)
      intmat(2) = ibuffr(ipwork+1)
      intmat(5) = ibuffr(ipwork+4)
      istart = ipwork+5

!     --- Replace all information

      do k = 1, 2

         if ( ihelp(k)>0 ) then

!        --- intmat part k exists

            istart  = istart + 2
            ipmat = iniget(intmat(k+2))
            call ini000 ( ibuffr(istart), ibuffr(ipmat), 1, 1, &
                          ihelp(k), 0 )
            istart = istart + ihelp(k)

         end if

      end do

      nbuffr = nbufol
      call erclos ( 'readb5' )
1000  if ( ierror/=0 ) call instop
      end
