      subroutine readb7 ( ibuffr, numarr, iarray )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.1  date 15-11-2012 Long integers
!        version  3.0  date 14-04-1999 Extra parameter ibuffr
!        version  2.3  date 24-07-1998 include commons
!
!   copyright (c) 1988-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     readb7 reads integer user array from backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cfile2f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr, iarray(*), ibuffr(*)

!    iarray    o   Integer user array
!    ibuffr   i/o  Integer buffer array in which all large data is stored
!    numarr    i   Array number with respect to the index in common cfile2
!                  for array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) nbufol, ipwork
      integer length, index

!     index       Record number with respect to file 2 from which array
!                 must be read
!     ipwork      Starting position of part of array in IBUFFR
!     length      Length of array KMESH
!     nbufol      Old value of nbuffr
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     INI000    Copy integer array to other integer array possibly itself
!     INI069    Reserve space at the end of IBUFFR for a semi-permanent array
!     INSTOP    Stop the program
!     READFL    Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readb7' )
      length = iwork3 ( 2*numarr )
      index  = iwork3 ( 2*numarr-1 )

!     --- Reserve space in IBUFFR

      call ini069 ( ibuffr, ipwork, length, nbufol )
      if ( ierror/=0 ) go to 1000

!     --- Read all information from file 2

      call readfl ( index, 1, length, 1, 2, ibuffr(ipwork) )
      if ( ierror/=0 ) go to 1000

!     --- Replace all information

      call ini000 ( ibuffr(ipwork+2), iarray, 1, 1, length-2, 0 )

1000  if ( ierror/=0 ) call instop
      nbuffr = nbufol
      call erclos ( 'readb7' )
      end
