      subroutine readcn
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 29-02-2008 Replace part of body by call subroutine
!        version  2.1  date 22-11-2004 Fortran 90
!        version  2.0  date 01-01-2003 Complete revision
!
!   copyright (c) 1995-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read the name of a constant behind the $ sign and set ityprd as well
!   as realrd
!   The name of the constant may contain digits, letters or an underscore
!   The separation character is either the space or end of record sign
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!     none
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, n, minusg
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              General loop variable
!     minusg         Indicates if a minus sign has been found (-1) or
!                    not (1)
!     n              Number of names that are known
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     READNAME1      read the name of a constant behind the $ or % sign
!     READSTRCONST   Check if name read corresponds to one the integer or
!                    real constants
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1608   Name found after $ or \ contains more than 32 characters
!    1609   Name found does not belong to chincons or chrlcons
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readcn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readcn'
         write(irefwr,1) 'lastitem, lastitembef', lastitem, lastitembef
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read name following $-sign and detect if it is preceded by a - sign

      call readname1 ( minusg )
      if ( debug ) then
         write(irefwr,1) 'minusg', minusg
         write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- name has been found, check to which group it belongs

      if ( nchar>32 ) then
         call errchr ( string(1:nchar), 1 )
         call errint ( nchar, 1 )
         call errwar ( 1608, 1, 0, 1 )
         call errcrd
         nchar = 32
      end if

!     --- Check if name belongs to the list of constants
!         If so return with ityprd/=0

      call readstrconst
      if ( debug ) write(irefwr,1) 'ityprd', ityprd
      if ( ityprd/=4 ) go to 1000

!     --- Name has not been found

      n = 1
      do i = 1, numints
         call errchr ( chincons(i), n+i )
      end do
      n = n+numints
      do i = 1, numreals
         call errchr ( chrlcons(i), n+i )
      end do
      n = n+numreals
      call errchr ( string(1:nchar), 1 )
      call errsub ( 1609, 0, 0, n )
      call errcrd

1000  call erclos ( 'readcn' )
      if ( minusg<0 ) realrd = -realrd
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lastitem', lastitem
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End readcn'

      end if  ! ( debug )

      end
