      subroutine readexpress3 ( iwork, lenwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-02-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove superfluous open and close brackets
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), lenwork

!     iwork         i/o   integer work array, see subroutine readexpressnew
!     lenwork       i/o   Length of array work
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ipositions(2,lenwork), ishift
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ipositions     Indicates the positions of openings and closings
!                    brackets in iwork
!                    ipositions(1,i) position of i-th open bracket
!                    ipositions(2,i) position of corresponding close bracket
!     ishift         Defines shift in array iwork, once brackets are removed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     READEXPRESS4   Remove brackets around a single number
!     READEXPRESS5   Remove superfluous openings and closings brackets
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readexpress3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readexpress3'
         call prinin ( iwork, lenwork, 'iwork/start' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Remove superfluous brackets

      if ( iwork(1)==0 ) then

!     --- iwork(1) = 0, remove pos. 1

         lenwork = lenwork-1
         do i = 1, lenwork
            iwork(i) = iwork(i+1)
         end do  ! i = 1, lenwork

      end if  ! ( iwork(1)==0 )

!     --- Remove brackets around a single number, &
!         i.e. the combination ( + number ) is replaced by number
!         or in case it is the first number: by + number

      ishift = 1
      do while ( ishift>0 )
         call readexpress4 ( iwork, lenwork, ipositions, ishift )
      end do  ! while ( ishift>0 )

!     --- Remove superfluous open and close bracktes

      call readexpress5 ( iwork, lenwork )

1000  call erclos ( 'readexpress3' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iwork, lenwork, 'iwork/end' )
         write(irefwr,*) 'End readexpress3'

      end if  ! ( debug )

      end
