      subroutine readsc ( ibuffr, buffer, inpsep, rinsep, niinp, nrinp,
     +                    nprob, kmesh, manag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.14 date 26-05-2011 New call to askvec
!        version  5.13 date 23-02-2011 Extension with WRITE_TO_FILE
!        version  5.12 date 12-02-2011 Extension with readscmatrix
!        version  5.11 date 13-11-2010 Extension with readscbouncond
!
!   copyright (c) 1993-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for the structure of the main program sepcomp or freeboun
!     The input is stored in array inpsep and the real array rinsep
!     The input must be enclosed between STRUCTURE and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons1'
      include 'SPcommon/csepch'
      include 'SPcommon/cseprd'
      include 'SPcommon/cseprdnew'
      include 'SPcommon/cconst'
      include 'SPcommon/cuscons'
      include 'SPcommon/cusname'
      include 'SPcommon/csepmain'
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cscalvec'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), inpsep(*), niinp, nrinp, nprob, kmesh(*),
     +        manag(*)
      double precision buffer(*), rinsep(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         o    The integer input read is stored in array INPSEP
!                         See subroutine TOSEPCOMSUB for a description
!     kmesh          i    Standard SEPRAN array, containing information of the
!                         mesh
!     manag          i    Manager array that sets the parameters in subroutine
!                         SEPSTR
!                         See subroutine READSP for a description
!     niinp         i/o   Length of integer input array inpsep
!                         At input: maximum available length
!                         At output: used length
!     nprob          i    Number of various problems
!     nrinp         i/o   Length of real input array rinsep
!                         At input: maximum available length
!                         At output: used length
!     rinsep         o    Real input array. The storage depends on iinsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished, outfnd, instat, timefnd, readnx, debug
      integer nncode, nout, nstrng
      parameter ( nncode=98, nout=200, nstrng=2 )
      character (len=37) code(nncode)
      character (len=50) strout(nstrng)
      integer ncode(nncode), ncodmx(nncode)
      double precision tabout(nout)
      integer numscl, istart, numvec, icode, number, iseqif,
     +        i, istrin, iscal, iwhile(10), iseqwh, iseqfr,
     +        ifreest, ndim, iloop(10), ilooptm(1), iseqlp,
     +        iseqtm, iseqfor, iforloop(10), ibeginfor, iendfor,
     +        istepfor, ivar, iforvar(10), nch, nelgrp, leninpsep,
     +        iarrif(10), iretur, istartbef, errorcode

!     code           Table with main keywords
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if the loop has been finished (true) or
!                    not (false)
!     i              Counting variable
!     iarrif         Array of size 10 in which information of the if
!                    statements is stored in the following way:
!                    iarrif(i) = starting address of if part in inpsep
!     ibeginfor      First value of for loop
!     icode          Return code with respect to subroutine INTX00
!                    Indicates position in array code
!     iendfor        Last value of for loop
!     iforloop       Array of size 10 in which information of the loops
!                    is stored in the following way:
!                    iloop(i) = starting address of for loop in inpsep
!     iforvar        Array of size 10 in which the scalar number of the
!                    loop variable of the active loop is stored
!     ifreest        ifreest = starting address of stationary free boundary
!                    in inpsep
!     iloop          Array of size 10 in which information of the loops
!                    is stored in the following way:
!                    iloop(i) = starting address of while loop in inpsep
!     ilooptm        Array of size 1 in which information of the time loops
!                    is stored in the following way:
!                    ilooptm(i) = starting address of while loop in inpsep
!     instat         If true an instationary free boundary loop is active
!     iretur         Return value for subroutine askcos
!                    input value -10, which means that if a text is recognized,
!                    an error message is given
!     iscal          Sequence number of scalar
!     iseqfor        Present level of for loop
!     iseqfr         Present level of stationary free boundary loop
!     iseqif         Present level of if loop
!     iseqlp         Present level of loop
!     iseqtm         Present level of time loop
!     iseqwh         Present level of while loop
!     istart         Starting address in array INPSEP
!     istartbef      Help variable to store the value of istart before call
!     istepfor       Step of for loop
!     istrin         Starting address in array RINSEP
!     ivar           Sequence number of variable
!     iwhile         Array of size 10 in which information of the while loops
!                    is stored in the following way:
!                    iwhile(i) = starting address of while loop in inpsep
!     leninpsep      Length of part of array inpsep
!     nch            Number of characters in a string
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     ncodmx         Table containing the number of maximum characters in
!                    the keyword table code
!     ndim           Dimension of space
!     nelgrp         Number of element groups
!     nncode         Number of keywords in code
!     nout           Length of array tabout
!     nstrng         Length of array strout
!     number         Scalar sequence number read
!     numscl         Number of scalars read
!     numvec         Number of vectors stored in isol
!     outfnd         Call to output found (true) or not (false)
!     readnx         Indicates if the next item has not been read (true) or has
!                    been (false)
!     strout         Contains names of texts read
!     tabout         Output array for subroutine INTX02
!     timefnd        If true, time integration has been found in loop
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer intx00

!     ASKCOS         Interpret options after compute_scalar
!     ASKNXT         Read next item from the standard input file
!     ASKPRIGEN      Interpret the options after the keyword print
!     ASKPRISCA      Interpret options after print_scalar
!     ASKPRIVEC      Interpret options after print_vector or print text
!     ASKSCA         Interpret options after SCALAR =
!     ASKSCL         Read and interpret integer sequence number after keyword
!     ASKVEC         Interpret options after compute_vector
!     ASKVECIN       Interpret the options after the keyword input_vector
!     ASKWRT         Interpret options after WRITE_VECTOR or READ_VECTOR
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INTX00         Compare text in string with table (variable length)
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READFOR        Read keywords after the keyword for
!     READSCBOUNCOND Read keywords after the keyword PRESCRIBE_BOUNDARY_COND
!     READSCBOUNINT  Read keywords after the keyword BOUNDARY_INTEGRAL
!     READSCBUBBLE   Read keywords after the keyword COMPUTE_BUBBLE
!     READSCCHCOEF   Read keywords after the keyword CHANGE_COEFFICIENTS
!     READSCCHPROB   Read keywords after the keyword PROBLEM
!     READSCCHSTRUCT Read keywords after the keyword CHANGE_STRUCTURE
!     READSCCOPYVEC  Read keywords after the keyword COPY_VECTOR
!     READSCCREATFVC Read keywords after the keyword create_force_vector
!     READSCDEFMESH  Read keywords after the keyword DEFORM_MESH
!     READSCDERIV    Read keywords after the keyword DERIVATIVES
!     READSCEIGVAL   Read keywords after the keyword COMPUTE_EIGENVALUES
!     READSCENDFLOOP Read keywords after the keywords
!                    END_STATIONARY_FREE_BOUNDARY,
!                    END_INSTATIONARY_FREE_BOUNDARY
!     READSCIF       Read the input following the following the keyword IF
!     READSCINTEGRAL Read keywords after the keyword INTEGRAL
!     READSCINTERMSH Read keywords after the keyword INTERCHANGE_MESH
!     READSCINTERPOL Read keywords after the keyword INTERPOLATE
!     READSCINTERSEC Read keywords after the keyword INTERSECTION
!     READSCLEVMESH  Read the input following the following the keywords
!                    MAKE_LEVELSET_MESH, REMOVE_LEVELSET_MESH
!     READSCLINSOL   Read keywords after the keyword SOLVE_LINEAR_PROBLEM
!     READSCMATRIX   Read keywords after the keyword MATRIX_STRUCTURE
!     READSCMVBOUN   Read the input after the keyword MOVE_BOUNDARY
!     READSCMVOBST   Read the input following the keyword move_obstacle
!     READSCNLNSOL   Read keywords after the keyword SOLVE_NONLINEAR_PROBLEM
!     READSCOBSTMESH Read keywords after the keywords MAKE_OBSTACLE_MESH,
!                    REMOVE_OBSTACLE_MESH
!     READSCPLOT3D   Read keywords after the keyword PLOT_3D
!     READSCPLOTBN   Read keywords after the keyword PLOT_BOUNDARY
!     READSCPLOTCN   Read keywords after the keyword PLOTCN
!     READSCPLOTFN   Read keywords after the keyword PLOT_FUNCTION
!     READSCPLOTINT  Read keywords after the keyword PLOT_INTERSECTION
!     READSCPLOTMESH Read keywords after the keyword PLOT_MESH
!     READSCPLOTTIM  Read the options following the keyword PLOT_TIME_HISTORY
!     READSCPLOTVC   Read keywords after the keyword PLOT_VECTOR
!     READSCPRESMESH Read keywords after the keyword PRESENT_MESH
!     READSCPSI      Read the input following the following the keyword
!                    COMPUTE_PSI_FILM
!     READSCREADMESH Read keywords after the keyword READ_MESH
!     READSCREFENT   Read keywords after the keyword REF_ENTHALPY_INTEGRATION
!     READSCREFMESH  Read keywords after the keyword REFINE_MESH
!     READSCSETTIME  Read information found after the keyword set_time
!     READSCSTANDARD Read keywords after a number of standard keywords
!     READSCSTLOOP   Read keywords after the keyword START_(TIME)_LOOP
!     READSCSTOC     Read the input after the keyword stochastic_investment
!     READSCSTRESS   Read keywords after the keyword COMPUTE_PRINCIPAL_STRESSES
!     READSCTIME     Read keywords after the keyword TIME_INTEGRATION
!     READSCTIMHIS   Read keywords after the keyword TIME_HISTORY
!     READSCUSEROUT  Read keywords after the keyword USER_OUTPUT
!     READSCWHILE    Read keywords after the keyword WHILE
!     READSCWRITFILE Read keywords after the keyword WRITE_TO_FILE
!     READSCWRITMESH Read keywords after the keyword WRITE_MESH
! **********************************************************************
!
!                       I/O
!
!     READSC requires input from the standard input file
!     The following input is recognized:
!     STRUCTURE    (has been read already be readal)
!         start of input
!      the next records are interpreted in the sequence they are read
!      the program is carried out in the same sequence
!
!       MATRIX_STRUCTURE, options (all in one line)
!       PRESCRIBE_BOUNDARY_CONDITIONS, options (all in one line)
!       CREATE_VECTOR, options (all in one line)
!       CREATE_FORCE_VECTOR, options (all in one line)
!
!       SOLVE_LINEAR_SYSTEM, options (all in one line)
!       SOLVE_NONLINEAR_SYSTEM, options (all in one line)
!       SOLVE_TIME_DEPENDENT_PROBLEM, options (all in one line)
!       SOLVE_BEARING, options (all in one line)
!       STOCHASTIC_INVESTMENT, options (all in one line)
!
!       DERIVATIVES, options (all in one line)
!       INTEGRAL, options (all in one line)
!       BOUNDARY_INTEGRAL, options (all in one line)
!
!       COMPUTE_EIGENVALUES, options (all in one line)
!       COMPUTE_CAPACITY, sequence_number = k (Default 1)
!       SOLVE_INVERSE_PROBLEM, sequence_number = k (Default 1)
!       COMPUTE_CONTACT_SURFACE, sequence_number = k (Default 1)
!       COMPUTE_PRINCIPAL_STRESSES, options (all in one line)
!       COMPUTE_ENTHALPY, options (all in one line)
!
!       OUTPUT, options (all in one line)
!       NO_OUTPUT
!
!       COMPUTE_VECTOR i, options (all in one line)
!       VECTOR i, options (all in one line)
!       COMPUTE_SCALAR i, options (all in one line)
!       SCALAR i = choice, (all in one line)
!       COPY VECTOR 1 DEGFD1 = j VECTOR 2 DEGFD2 = k
!       COPY_COOR VECTOR i, PROBLEM p
!       CHANGE_PROBLEM, vector=i, problem=p (Default 1,1)
!
!       PRINT_SCALAR i1, i2, ..., options (all in one line)
!       PRINT_VECTOR i, options (all in one line)
!       PRINT_TEXT, 'text between quotes'
!       PRINT_TIME
!       PRINT
!
!       PLOT_VECTOR V1, options
!       PLOT_TENSOR V1, options
!       PLOT_CONTOUR V1, options
!       PLOT_COLOURED_LEVELS V1, options
!       PLOT_BOUNDARY, options
!       PLOT_MESH, options
!       PLOT_FUNCTION, options
!       PLOT_TIME_HISTORY, options
!       PLOT_INTERSECTION, options
!       OPEN_PLOT
!       CLOSE_PLOT
!
!       WRITE_VECTOR j, sequence_number = i, save_administration
!       READ_VECTOR j, sequence_number = i
!       INPUT_VECTOR j, file= 'file name', type_of_vector
!
!       DEFORM_MESH, options (all in one line)
!       CHANGE_COORDINATES, sequence_number = k (Default 1)
!       REFINE_MESH, options (all in one line)
!       PRESENT_MESH = 2 (Default 1)
!       INTERCHANGE_MESH = (i,j) (Default 1,2)
!       WRITE_MESH, options (all in one line)
!       READ_MESH, options (all in one line)
!       COPY_MESH i to j
!       MOVE_BOUNDARY, options
!
!       INTERPOLATE V1,  options
!       INTERSECTION, options
!
!       MOVE_OBSTACLE i, options (all in one line)
!       MAKE_OBSTACLE_MESH, options (all in one line)
!       REMOVE_OBSTACLE_MESH, options (all in one line)
!
!       MAKE_LEVELSET_MESH, options (all in one line)
!       REMOVE_LEVELSET_MESH, options (all in one line)
!
!       USER_OUTPUT, options
!
!       CHANGE_STRUCTURE_OF_MATRIX, SEQ_STRUCTURE = s
!       CHANGE_COEFFIENTS, options (all in one line)
!       NEW_PROBLEM_DESCRIPTION, SEQUENCE_NUMBER = s
!       READ_SOLUTIONS
!
!       CREATE_REACTION_FORCES, options (all in one line)
!
!       SET_TIME i
!       TIME_HISTORY, options (all in one line)
!
!       Structure commands:
!       WHILE ( boolean_expr(k) ) DO
!          ....
!       END_WHILE
!       FOR %var = i1 STEP i2 TO i3
!          ....
!       END_FOR
!       START_STATIONARY_FREE_BOUNDARY_LOOP, sequence_number = k
!          ....
!       END_STATIONARY_FREE_BOUNDARY_LOOP
!       START_INSTATIONARY_FREE_BOUNDARY_LOOP, sequence_number = k
!          ....
!          TIME_INTEGRATION, sequence_number = s, vector = i
!          ....
!       END_INSTATIONARY_FREE_BOUNDARY_LOOP
!       START_LOOP, sequence_number = k
!          ....
!       END_LOOP
!       START_TIME_LOOP, sequence_number = k
!          ....
!          TIME_INTEGRATION, sequence_number = s, vector = i
!     or   ENTHALPY_INTEGRATION, sequence_number = s, vector = i
!     or   REF_ENTHALPY_INTEGRATION, seq_enthalpy = s, seq_refine = j
!          ....
!       END_TIME_LOOP
!       STOP
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     133   while loop not closed by end_while
!     167   END_WHILE found but no corresponding WHILE statement
!    1075   END_LOOP found but no corresponding START_LOOP statement
!    1076   END_TIME_LOOP found but no corresponding START_TIME_LOOP statement
!    1868   Text not recognized
!    1927   START_FREE_BOUNDARY not closed by END_FREE_BOUNDARY
!    1928   free boundaries not allowed in SEPCOMP
!    2119   end_for found but no corresponding for
!    2120   more than 10 nested for loops
!    2121   for variable already in use in nested for loop
! **********************************************************************
!
!                       PSEUDO CODE
!
!    finished := false
!    output_found := false
!    iseqwhile := 0
!    iseqfor   := 0
!    iseqstatf := 0
!    iseqstatl := 0
!    iseqif := 0
!    while ( not finished ) do
!       read next record
!       interpret record
!       if ( item # keyword ) then
!          error
!       else if ( keyword = STOP) then
!          set stop
!       else if ( keyword = END) then
!          finished := true
!          close open while loops
!          close open stationary free boundary loop
!       else
!          read options
!    if ( not output_found )
!       Set call to standard output
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code(1:10) / 'PRESCRIBE_BOUNDARY_CONDITIONS',
     +            'SOLVE_LINEAR_SYSTEM', 'SOLVE_NONLINEAR_SYSTEM',
     +            'CREATE_VECTOR', 'OUTPUT', 'COMPUTE_SCALAR',
     +            'PRINT_SCALAR', 'DERIVATIVES', 'COMPUTE_VECTOR',
     +            'PRINT_VECTOR' /
      data code(11:20) / 'INTEGRAL', 'SCALAR',
     +            'CHANGE_STRUCTURE', 'COPY_COOR',
     +            'PRINT_TEXT', 'BOUNDARY_INTEGRAL', 'WRITE_VECTOR',
     +            'READ_VECTOR', 'SOLVE_TIME_DEPENDENT_PROBLEM',
     +            'USER_OUTPUT' /
      data code(21:30) / 'WHILE', 'END_WHILE',
     +            'START_STATIONARY_FREE_BOUNDARY',
     +            'END_STATIONARY_FREE_BOUNDARY', 'CHANGE_COEFFICIENTS',
     +            'START_INSTATIONARY_FREE_BOUNDARY_LOOP',
     +            'END_INSTATIONARY_FREE_BOUNDARY_LOOP',
     +            'TIME_INTEGRATION', 'INPUT_VECTOR', 'OPEN_PLOT' /
      data code(31:40) / 'CLOSE_PLOT', 'PLOT_BOUNDARY', 'REFINE_MESH',
     +            'PRESENT_MESH', 'WRITE_MESH', 'INTERPOLATE',
     +            'FOR', 'END_FOR', 'PLOT_VECTOR', 'PLOT_CONTOUR' /
      data code(41:50) / 'COMPUTE_CONTACT_SURFACE', 'INTERCHANGE_MESH',
     +            'CREATE_FORCE_VECTOR', 'START_LOOP', 'END_LOOP',
     +            'START_TIME_LOOP', 'END_TIME_LOOP',
     +            'CHANGE_COORDINATES', 'COMPUTE_EIGENVALUES',
     +            'CHANGE_PROBLEM' /
      data code(51:60) / 'COMPUTE_CAPACITY',
     +            'SOLVE_INVERSE_PROBLEM', 'DEFORM_MESH',
     +            'PRINT_TIME', 'COMPUTE_PRINCIPAL_STRESSES',
     +            'PLOT_COLOURED_LEVELS', 'PLOT_TENSOR',
     +            'MOVE_OBSTACLE', 'MAKE_OBSTACLE_MESH',
     +            'REMOVE_OBSTACLE_MESH' /
      data code(61:70) / 'NO_OUTPUT', 'PLOT_MESH',
     +            'CREATE_REACTION_FORCES', 'STOP', 'READ_MESH',
     +            'SET_TIME', 'VECTOR', 'V',
     +            'MAKE_LEVELSET_MESH', 'REMOVE_LEVELSET_MESH' /
      data code(71:80) / 'ENTHALPY_INTEGRATION', 'COMPUTE_ENTHALPY',
     +            'INTERSECTION', 'TIME_HISTORY',
     +            'NEW_PROBLEM_DESCRIPTION', 'REF_ENTHALPY_INTEGRATION',
     +            'PRESSURE_CORRECTION', 'NAVIER_STOKES',
     +            'SOLVE_BEARING', 'READ_SOLUTIONS' /
      data code(81:90) / 'PLOT_FUNCTION',
     +            'S', 'IF', 'END_IF', 'COPY', 'PRINT', 'MOVE_BOUNDARY',
     +            'COMPUTE_PSI_FILM', 'STOCHASTIC_INVESTMENT',
     +            'COPY_MESH' /
      data code(91:98) / 'COMPUTE_BUBBLE', 'PLOT_TIME_HISTORY',
     +            'MATRIX_STRUCTURE', 'WRITE_TO_FILE', 'CLOSE_FILE',
     +            'PLOT_3D', 'PLOT_INTERSECTION', 'END' /
      data ncode /  5,  9,  9,  8,  4,   11,  9,  5, 11,  9,  ! 10
     +              5,  6, 10,  7,  8,   10,  9,  8,  9,  8,  ! 20
     +              5,  9, 10,  8, 11,   12, 10,  8,  5,  6,  ! 30
     +              7,  9,  5,  6,  8,    7,  3,  6,  8,  8,  ! 40
     +             11,  8,  8,  8,  7,    8,  7, 11, 11, 11,  ! 50
     +             11, 10,  6,  9, 16,    7,  8,  9,  8, 10,  ! 60
     +              6,  9, 11,  4,  9,    7,  6,  1,  8, 10,  ! 70
     +              6, 11,  9,  8,  8,    8, 12,  5, 10,  8,  ! 80
     +              8,  1,  2,  6,  3,    5,  9, 10, 12,  9,  ! 90
     +             11,  8,  6,  8,  8,    7,  8,  3 /
      data ncodmx/ 29, 19, 22, 13,  6,   14, 12, 11, 14, 12,  ! 10
     +              8,  6, 26,  9, 10,   17, 12, 11, 28, 11,  ! 20
     +              5,  9, 35, 33, 19,   37, 35, 16, 12,  9,  ! 30
     +             10, 13, 11, 12, 10,   11,  3,  7, 11, 12,  ! 40
     +             23, 16, 19, 10,  9,   15, 14, 18, 19, 14,  ! 50
     +             16, 21, 11, 10, 26,   20, 11, 13, 18, 20,  ! 60
     +              9,  9, 23,  4,  9,    8,  6,  1, 18, 21,  ! 70
     +             20, 16, 12, 12, 23,   24, 19, 13, 13, 14,  ! 80
     +             14,  1,  2,  6,  4,    5, 13, 16, 21,  9,  ! 90
     +             14, 17, 16, 14, 10,    7, 17,  3 /

!     code    Table with main keywords
!             Values:
!             1:  PRESCRIBE_BOUNDARY_CONDITIONS
!             2:  SOLVE_LINEAR_SYSTEM
!             3:  SOLVE_NONLINEAR_SYSTEM
!             4:  CREATE_VECTOR
!             5:  OUTPUT
!             6:  COMPUTE_SCALAR
!             7:  PRINT_SCALAR
!             8:  DERIVATIVES
!             9:  COMPUTE_VECTOR
!            10:  PRINT_VECTOR
!            11:  INTEGRAL
!            12:  SCALAR
!            13:  CHANGE_STRUCTURE
!            14:  COPY_COOR
!            15:  PRINT_TEXT
!            16:  BOUNDARY_INTEGRAL
!            17:  WRITE_VECTOR
!            18:  READ_VECTOR
!            19:  SOLVE_TIME_DEPENDENT_PROBLEM
!            20:  USER_OUTPUT
!            21:  WHILE
!            22:  END_WHILE
!            23:  START_STATIONARY_FREE_BOUNDARY
!            24:  END_STATIONARY_FREE_BOUNDARY
!            25:  CHANGE_COEFFICIENTS
!            26:  START_STATIONARY_FREE_BOUNDARY_LOOP
!            27:  END_STATIONARY_FREE_BOUNDARY_LOOP
!            28:  TIME_INTEGRATION
!            29:  INPUT_VECTOR
!            30:  OPEN_PLOT
!            31:  CLOSE_PLOT
!            32:  PLOT_BOUNDARY
!            33:  REFINE_MESH
!            34:  PRESENT_MESH
!            35:  WRITE_MESH
!            36:  INTERPOLATE
!            37:  FOR
!            38:  END_FOR
!            39:  PLOT_VECTOR
!            40:  PLOT_CONTOUR
!            41:  COMPUTE_CONTACT_SURFACE
!            42:  INTERCHANGE_MESH
!            43:  CREATE_FORCE_VECTOR
!            44:  START_LOOP
!            45:  END_LOOP
!            46:  START_TIME_LOOP
!            47:  END_TIME_LOOP
!            48:  CHANGE_COORDINATES
!            49:  COMPUTE_EIGENVALUES
!            50:  CHANGE_PROBLEM
!            51:  COMPUTE_CAPACITY
!            52:  SOLVE_INVERSE_PROBLEM
!            53:  DEFORM_MESH
!            54:  PRINT_TIME
!            55:  COMPUTE_PRINCIPAL_STRESSES
!            56:  PLOT_COLOURED_LEVELS
!            57:  PLOT_TENSOR
!            58:  MOVE_OBSTACLE
!            59:  MAKE_OBSTACLE_MESH
!            60:  REMOVE_OBSTACLE_MESH
!            61:  NO_OUTPUT
!            62:  PLOT_MESH
!            63:  CREATE_REACTION_FORCES  Not used
!            64:  STOP
!            65:  READ_MESH
!            66:  SET_TIME
!            67:  VECTOR
!            68:  V
!            69:  MAKE_OBSTACLE_MESH
!            70:  REMOVE_OBSTACLE_MESH
!            71:  ENTHALPY_INTEGRATION
!            72:  COMPUTE_ENTHALPY
!            73:  INTERSECTION
!            74:  TIME_HISTORY
!            75:  NEW_PROBLEM_DESCRIPTION
!            76:  REF_ENTHALPY_INTEGRATION
!            77:  PRESSURE_CORRECTION
!            78:  NAVIER_STOKES
!            79:  SOLVE_BEARING
!            80:  READ_SOLUTIONS
!            81:  PLOT_FUNCTION
!            82:  S
!            83:  IF
!            84:  END_IF
!            85:  COPY
!            86:  PRINT
!            87:  MOVE_BOUNDARY
!            88:  COMPUTE_PSI_FILM
!            89:  STOCHASTIC_INVESTMENT
!            90:  COPY_MESH
!            91:  COMPUTE_BUBBLE
!            92:  PLOT_TIME_HISTORY
!            93:  MATRIX_STRUCTURE
!            94:  WRITE_TO_FILE
!            95:  CLOSE_FILE
!            96:  PLOT_3D
!            97:  PLOT_INTERSECTION
!            98:  END
! ======================================================================
!
      call eropen ( 'readsc' )

      debug = .false.

      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'Debug information from readsc'
  1      format ( a, 1x, (10i6) )

      end if

!     --- Initialize parameters

      nelgrp = kmesh(5)
      ndim   = kmesh(6)
      numvec = nprob
      numscl = 0
      istart = 2
      istrin = 1
      iseqwh = 0
      iseqif = 0
      iseqfr = 0
      iseqlp = 0
      iseqtm = 0
      iseqfor = 0
      errorcode = 0
      finished  = .false.
      outfnd = .false.
      instat = .false.
      timefnd = .false.
      ivectors(1:nprob) = 1

!     --- While loop finished

      do while ( .not. finished )

!     --- Check array lengths

         call inichkarlen ( istart, niinp, 'inpsep' )
         call inichkarlen ( istrin,  nrinp, 'rinsep' )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then

!        --- Debug information

            call prinin ( inpsep, istart, 'inpsep' )
            call prinrl ( rinsep, istrin, 'rinsep' )

         end if  ! ( debug )

!        --- Read next record

         ityprd = 1
         ispecl = 9    ! a keyword may contain digits and underscores
         call asknxt
         ispecl = 0
         if ( debug ) write(irefwr,1) 'ityprd', ityprd
         if ( ityprd/=4 ) then

!        --- error: not a keyword found

            do i = 1, nncode
               call errchr ( code(i), i )
            end do
            call errsub ( 1868, 0, 0, nncode )
            call errcrd
            go to 1000

         end if
         if ( debug ) write(irefwr,*) 'keyword ', string(1:nchar)

         if ( string(1:nchar)=='PLOT_3D' ) then

!        --- Special case, keyword contains a number

            icode = 96

         else

!        --- Standard case

            icode = intx00 ( code, ncode, nncode, 1, ncodmx )

         end if  ! ( string(1:nchar)=='PLOT_3D' )

         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,1) 'icode', icode

         if ( mainsubr==2 .and. (icode==23 .or. icode==24 .or.
     +        icode==26 .or. icode==27) ) then
             call errsub ( 1928, 0, 0, 0 )
             go to 1000
         end if
         if ( debug )
     +      write(irefwr,1) 'icode, istart, lastitem',
     +                       icode, istart, lastitem

         select case ( icode )

            case(0)

!           --- Code 0, jump

            case(1, 4)

!           --- Code 1, essential boundary conditions or create

               call readscbouncond ( ibuffr, buffer, istart, inpsep,
     +                               tabout, icode, nout, strout, nprob,
     +                               numvec, rinsep, istrin, kmesh )

            case(5,14,19,41,48,50,51,52,71,72,77,78,79,80)

!           --- icode = 5, 14, 19, 41, 48, 50, 51, 52, 71, 72, 77, 78, 79, 80
!               read standard options

               call readscstandard ( istart, inpsep, tabout, outfnd,
     +                               icode, nout, strout, nprob,
     +                               numvec, errorcode )

            case(2)

!           --- icode = 2, linear solver

               call readsclinsol ( istart, inpsep, tabout, icode,
     +                             nout, strout, nprob, numvec, kmesh )

            case(3)

!           --- icode = 3, non-linear solver

               call readscnlnsol ( istart, inpsep, tabout, icode,
     +                             nout, strout, nprob, numvec,
     +                             rinsep, istrin )

            case(6)

!           --- icode = 6, compute scalar

               call askscl ( number, 0, maxscalars, code(icode), icode )
               if ( ierror/=0 ) go to 1000
               inpsep(istart) = icode
               iretur = -10  ! standard case, allow a real to be read
               call askcos ( number, inpsep(istart+1),
     +                       numscl, kmesh, nelgrp, istart, iretur )

            case(7)

!           --- icode = 7, print scalar

               call askprisca ( inpsep, istart, icode )

            case(8)

!           --- icode = 8, derivatives

               call readscderiv ( istart, inpsep, tabout, icode,
     +                            nout, strout, nprob, numvec, numscl )

            case(9,67,68)

!           --- icode = 9, 67, 68 compute vector

               call askscl ( number, 1, maxvectors, code(icode), icode )
               if ( ierror/=0 ) go to 1000
               istartbef = istart
               call askvec ( number, inpsep(istart), rinsep,
     +                       numvec, istrin, kmesh, istart, nprob )
               if ( debug ) then

!              --- debug information

                  write(irefwr,*) 'istart = ', istartbef
                  call prinin ( inpsep(istartbef), 15,
     +                          'inpsep from istart' )

               end if
               if ( ierror/=0 ) go to 1000

            case(10,15)

!           --- icode = 15, print text or icode = 10, print vector

               call askprivec ( inpsep, istart, rinsep, number, icode,
     +                          istrin, kmesh, code )

            case(11)

!           --- icode = 11, integral

               call readscintegral ( istart, inpsep, tabout, icode,
     +                               nout, strout, 0 )

            case(12,82)

!           --- icode = 12, 82 SCALAR found, read sequence number and value
!               or icode = 12 if compute_scalar is replaced by s

               call askscl ( iscal, 1, maxscalars, code(icode), icode )
               if ( ierror/=0 ) go to 1000
               call inichkarlen ( istart+104, niinp, 'inpsep' )
               if ( ierror/=0 ) go to 1000
               call asksca ( iscal, inpsep, rinsep,
     +                       numscl, istrin, leninpsep, icode,
     +                       kmesh, nelgrp, istart,
     +                       nout, strout, tabout )
               if ( ierror/=0 ) go to 1000

            case(13)

!           --- icode = 13, CHANGE_STRUCTURE found, read sequence number

               call readscchstruct ( istart, inpsep, tabout, icode,
     +                               nout, strout )

            case(16)

!           --- icode = 16, BOUNDARY_INTEGRAL

               call readscbounint ( istart, inpsep, tabout, icode,
     +                              nout, strout, 0, kmesh )

            case(17,18)

!           --- icode = 17, 18 WRITE_VECTOR or READ_VECTOR found

               call askscl ( number, 1, maxvectors, code(icode), icode )
               if ( ierror/=0 ) go to 1000
               inpsep(istart) = icode
               call askwrt ( inpsep(istart+1), number, icode )
               if ( ierror/=0 ) go to 1000
               numvec = max(numvec,number)
               istart = istart + 4

            case(20)

!           --- icode = 20, USER_OUTPUT found, read sequence number
!                           and possibly extra integers and scalars

               call readscuserout ( istart, inpsep, tabout, icode,
     +                              nout, strout )

            case(21)

!           --- icode = 21 WHILE found

               call readscwhile ( istart, inpsep, tabout, icode,
     +                            nout, strout, iseqwh, iwhile, rinsep,
     +                            niinp, istrin )

            case(22)

!           --- icode = 22 END_WHILE found

               if ( iseqwh>0 ) then
                  inpsep(istart) = icode
                  inpsep(istart+1) = iwhile(iseqwh)
                  istart = istart + 2
                  inpsep(iwhile(iseqwh)+2) = istart
                  iseqwh = iseqwh - 1
               else
                  call errsub ( 167, 0, 0, 0 )
               end if

            case(23,26,44,46)

!           --- icode = 23,26 START_(IN)STATIONARY_FREE_BOUNDARY found or
!                       44, 46 START_(TIME_)LOOP

               call readscstloop ( istart, inpsep, tabout, icode, nout,
     +                             strout, iseqfr, instat, iseqlp,
     +                             iseqtm, iloop, ilooptm, timefnd,
     +                             ifreest, rinsep, istrin )

            case(24,27)

!           --- icode = 24, 27 END_(IN)STATIONARY_FREE_BOUNDARY found

               call readscendfloop ( istart, inpsep, icode, iseqfr,
     +                               instat, timefnd, ifreest )

            case(25)

!           --- icode = 25 CHANGE_COEFFICIENTS found

               call readscchcoef ( istart, inpsep, tabout, icode,
     +                             nout, strout, nprob )

            case(28)

!           --- icode = 28 TIME_INTEGRATION found

               call readsctime ( istart, inpsep, tabout, icode,
     +                           nout, strout, instat, timefnd )

            case(29)

!           --- icode = 29, INPUT_VECTOR found, read sequence number,
!                           file name and type

               call askvecin ( inpsep(istart+1), numvec, ndim )
               if ( ierror/=0 ) go to 1000
               inpsep(istart) = icode
               istart = istart+5

            case(30,31)

!           --- icode = 30, 31 OPEN_PLOT, CLOSE_PLOT found

               inpsep(istart) = icode
               istart = istart+1

            case(32)

!           --- icode = 32 PLOT_BOUNDARY found

               call readscplotbn ( istart, inpsep, tabout, icode,
     +                             nout, strout, istrin, rinsep )

            case(33)

!           --- icode = 33 REFINE_MESH found

               call readscrefmesh ( istart, inpsep, tabout, icode,
     +                              nout, strout )

            case(34)

!           --- icode = 34 PRESENT_MESH found

               call readscpresmesh ( istart, inpsep, icode,
     +                               code(icode) )

            case(35)

!           --- icode = 35 WRITE_MESH found

               call readscwritmesh ( istart, inpsep, tabout, icode,
     +                               nout, strout )

            case(36)

!           --- icode = 36 INTERPOLATE found

               call readscinterpol ( istart, inpsep, tabout, icode,
     +                               nout, strout )

            case(37)

!           --- icode = 37 FOR found

               call readfor ( ivar, ibeginfor, istepfor, iendfor )
               if ( ierror/=0 ) go to 1000
               inpsep(istart) = icode
               inpsep(istart+1) = ibeginfor
               inpsep(istart+2) = iendfor
               inpsep(istart+3) = istepfor
               inpsep(istart+4) = ivar
               inpsep(istart+5) = 0
               iseqfor = iseqfor+1
               if ( iseqfor>10 ) then
                  call errint ( iseqfor, 1 )
                  call errint ( 10, 2 )
                  call errsub ( 2120, 2, 0, 0 )
               end if
               iforloop(iseqfor) = istart
               iforvar(iseqfor) = ivar
               do i = 1, iseqfor-1
                  if ( iforvar(i)==ivar ) then

!                 --- variable already in use

                     call errint ( ivar, 1 )
                     nch = index ( chscalars(i), ' ' ) -1
                     if ( nch<0 ) nch = 32
                     call errchr ( chscalars(i)(1:nch), 1 )
                     call errsub ( 2121, 1, 0, 1 )

                  end if
               end do
               istart = istart+6

            case(38)

!           --- icode = 38 END_FOR found

               if ( iseqfor>0 ) then
                  inpsep(istart) = icode
                  inpsep(istart+1) = iforloop(iseqfor)
                  istart = istart + 2
                  inpsep(iforloop(iseqfor)+5) = istart
                  iseqfor = iseqfor - 1
               else
                  call errsub ( 2119, 0, 0, 0 )
               end if

            case(39,57)

!           --- icode = 39 PLOT_VECTOR found

               call readscplotvc ( istart, inpsep, istrin, tabout,
     +                             rinsep, ndim, icode, strout, nout )
               if ( ierror/=0 ) go to 1000

            case(40,56)

!           --- icode = 40: PLOT_CONTOUR or 56: PLOT_COLOURED_LEVELS found

               call readscplotcn ( istart, inpsep, istrin, tabout,
     +                             rinsep, ndim, icode, strout, nout )
               if ( ierror/=0 ) go to 1000

            case(42, 90)

!           --- icode = 42 INTERCHANGE_MESH found, read 2 integer numbers
!                          or none

               call readscintermsh ( istart, inpsep, icode, readnx,
     +                               code(icode) )

            case(43)

!           --- icode = 43, create_force_vector

               call readsccreatfvc ( istart, inpsep, tabout, icode,
     +                               nout, strout, nprob,
     +                               numvec )

            case(45)

!           --- icode = 45 END_LOOP found

               if ( iseqlp>0 ) then
                  inpsep(istart) = icode
                  inpsep(istart+1) = iloop(iseqlp)+3
                  istart = istart + 2
                  inpsep(iloop(iseqlp)+2) = istart
                  iseqlp = iseqlp - 1
               else
                  call errsub ( 1075, 0, 0, 0 )
               end if

            case(47)

!           --- icode = 47 END_TIME_LOOP found

               if ( iseqtm>0 ) then
                  inpsep(istart) = icode
                  inpsep(istart+1) = ilooptm(iseqtm)+3
                  istart = istart + 2
                  inpsep(ilooptm(iseqtm)+2) = istart
                  iseqtm = iseqtm - 1
                  timefnd = .false.
                  instat = .false.
               else
                  call errsub ( 1076, 0, 0, 0 )
               end if

            case(49)

!           --- icode = 49, COMPUTE_EIGENVALUES found, read sequence number
!                           and other options

               call readsceigval ( istart, inpsep, tabout, icode,
     +                             nout, strout, nprob, numvec )

            case(53)

!           --- icode = 53, DEFORM_MESH found, read vector number
!                           and scale

               call readscdefmesh ( istart, inpsep, tabout, icode,
     +                              nout, strout, istrin, rinsep )

            case(54)

!           --- icode = 54, PRINT_TIME found

               inpsep(istart) = icode
               istart = istart + 1

            case(55)

!           --- icode = 55, COMPUTE_PRINCIPAL_STRESSES found

               call readscstress ( istart, inpsep, tabout, icode,
     +                             strout, nout, numvec )

            case(58)

!           --- icode = 58, MOVE_OBSTACLE found

               call readscmvobst ( istart, inpsep, tabout, icode,
     +                             kmesh, nout, istrin, rinsep, strout,
     +                             code(58)(1:ncodmx(58)) )
               if ( ierror/=0 ) go to 1000

            case(59,60)

!           --- icode = 59, 60 MAKE_OBSTACLE_MESH or REMOVE_OBSTACLE_MESH found

               call readscobstmesh ( istart, inpsep, tabout, icode,
     +                               nout, strout )

            case(61)

!           --- icode = 61, NO_OUTPUT found
!               by setting outfnd to TRUE the default output will not
!               be generated and there is no output generated by this
!               keyword

               outfnd = .true.

            case(62)

!           --- icode = 62 PLOT_MESH found

               call readscplotmesh ( istart, inpsep, tabout, icode,
     +                               nout, strout, istrin, rinsep,
     +                               ndim )

            case(63)

!           --- icode = 63 not yet defined

            case(64)

!           --- icode = 64, STOP found

               inpsep(istart) = icode
               istart = istart+1

            case(65)

!           --- icode = 65, READ_MESH found

               call readscreadmesh ( istart, inpsep, tabout, icode,
     +                               nout, strout )

            case(66)

!           --- icode = 66, SET_TIME found

               call readscsettime ( istart, inpsep, istrin, rinsep,
     +                              icode )

            case(69,70)

!           --- icode = 69, 70 MAKE_LEVELSET_MESH or REMOVE_LEVELSET_MESH found

               call readsclevmesh ( istart, inpsep, tabout, icode,
     +                              nout, strout )

            case(73)

!           --- icode = 73, INTERSECTION found

               call readscintersec ( istart, inpsep, rinsep, tabout,
     +                               icode, nout, strout, kmesh,
     +                               numscl, istrin )

            case(74)

!           --- icode = 74, INTERSECTION found

               call readsctimhis ( istart, inpsep, tabout, icode, nout,
     +                             strout, kmesh, rinsep, istrin )

            case(75)

!           --- icode = 75, NEW_PROBLEM_DESCRIPTION found, read sequence number

               call readscchprob ( istart, inpsep, tabout, icode,
     +                             nout, strout )

            case(76)

!           --- icode = 76, REF_ENTHALPY_INTEGRATION found, read sequence
!               numbers

               call readscrefent ( istart, inpsep, tabout, icode,
     +                             nout, strout )

            case(81)

!           --- icode = 81, PLOT_FUNCTION found, read options

               call readscplotfn ( istart, inpsep, tabout, icode, nout,
     +                             strout, numvec, kmesh, nstrng )

            case(83)

!           --- icode = 83 IF found

               call readscif ( istart, inpsep, tabout, icode,
     +                         nout, strout, iseqif, iarrif, rinsep,
     +                         niinp, istrin )

            case(84)

!           --- icode = 84 END_IF found

               if ( iseqif>0 ) then
                  inpsep(istart) = icode
                  istart = istart + 1
                  inpsep(iarrif(iseqif)+2) = istart
                  iseqif = iseqif - 1
               else
                  call errsub ( 167, 0, 0, 0 )
               end if

            case(85)

!           --- icode = 85, COPY VECTOR found, read sequence numbers

               call readsccopyvec ( istart, inpsep, tabout, icode,
     +                              nout, strout, code(icode) )

            case(86)

!           --- icode = 86, PRINT found

               call askprigen ( inpsep, istart, rinsep, number,
     +                          istrin, kmesh, code )

            case(87)

!           --- icode = 87, Move_boundary found

               call readscmvboun ( inpsep, istart, nprob, numvec,
     +                             icode, tabout, strout, nout )

            case(88)

!           --- icode = 88, COMPUTE_PSI_FILM

               call readscpsi ( inpsep, istart, icode, tabout,
     +                          strout, nout, kmesh )

            case(89)

!           --- icode = 89, stochastic_investment

               call readscstoc ( inpsep, istart, icode, rinsep, istrin,
     +                           tabout, strout, nout, kmesh )

            case(91)

!           --- icode = 91, COMPUTE_BUBBLE found

               call readscbubble ( istart, inpsep, rinsep, tabout,
     +                             icode, nout, strout, istrin )

            case(92)

!           --- icode = 92, PLOT_TIME_HISTORY found

               call readscplottim ( inpsep, istart, rinsep, istrin )

            case(93)

!           --- icode = 93, MATRIX_STRUCTURE found

               call readscmatrix ( inpsep, istart )

            case(94)

!           --- icode = 94, WRITE_TO_FILE found

               call readscwritfile ( inpsep, istart )

            case(95)

!           --- icode = 95, CLOSE_FILE found, raise istart

               inpsep(istart) = icode
               istart = istart+1

            case(96)

!           --- icode = 96, PLOT_3D found

               call readscplot3d ( istart, inpsep, tabout,
     +                             icode, strout, nout )

            case(97)

!           --- icode = 97, PLOT_INTERSECTION found

               call readscplotint ( inpsep, istart, rinsep, istrin,
     +                              tabout, icode, strout, nout )

            case(98)

!           --- icode = 98, END found

               finished = .true.
               do i = iseqwh, 1, -1
                  call errint ( iseqwh, 1 )
                  call errwar ( 133, 1, 0, 0 )
                  inpsep(istart) = 22
                  inpsep(istart+1) = iwhile(i)
                  istart = istart + 2
                  inpsep(iwhile(i)+2) = istart
               end do
               if ( iseqfr>0 ) then
                  call errwar ( 1927, 0, 0, 0 )
                  inpsep(istart) = 24
                  inpsep(istart+1) = ifreest+3
                  istart = istart + 2
                  inpsep(ifreest+2) = istart
               end if

         end select  ! case ( icode )

      end do  ! while ( .not. finished )

      if ( manag(7)>0 .and. .not. outfnd ) then

!     --- output not found, fill default

         call inichkarlen ( istart+4, niinp, 'inpsep' )
         if ( ierror/=0 ) go to 1000
         inpsep(istart) = 5
         inpsep(istart+1) = -1
         inpsep(istart+2) = -1
         inpsep(istart+3) = -1
         istart = istart + 4

      end if

      niinp = istart
      nrinp = istrin - 1
      inpsep(1) = numvec
      inpsep(niinp) = -1

1000  call erclos ( 'readsc' )

      if ( debug ) then
         call prinin ( inpsep, istart, 'inpsep' )
         write(irefwr,*) 'End readsc'
      end if  ! ( debug )

      end
