      subroutine readscalrhs ( number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 01-12-2014 Extension with min/max
!        version  1.3  date 29-04-2014 Extension with boundary_sum
!        version  1.2  date 17-04-2014 Extension with intersection_integral
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if right-hand side of equals sign corresponds to
!     a known function that produces a scalar
! **********************************************************************
!
!                       KEYWORDS
!
!     scalar
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number

!     number         o    If the right-hand side corresponds to a known
!                         function, the sequence number of this function
!                         is returned, otherwise 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode=29
      integer ncode(nncode), ncodmx(nncode), i
      character (len=21) code(nncode)
      logical debug

!     code           Table with keywords
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     ncodmx         Table containing the maximal number of  characters in
!                    the keyword table code
!     nncode         Number of entries in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'NORM_DIF', 'NORM', 'AVERAGE', 'DOT_PRODUCT', &
                  'MIN_MAX', 'SCAL_MAX', 'EXTRACT_VALUE', 'COOR_MIN', &
                  'COOR_MAX', 'ABS_VALUE', 'NODE', 'USER_POINT', &
                  'ONE_NORM', 'TWO_NORM', 'INF_NORM', 'MEAN_ONE_NORM', &
                  'MEAN_TWO_NORM', 'MEAN_AREA', 'MIN_AREA', &
                  'POINT_NUMBER', 'ZERO_LEVEL_SET', 'MEAN_VALUE', &
                  'STANDARD_DEVIATION', 'INTEGRAL', 'BOUNDARY_INTEGRAL', &
                  'BOUNDARY_SUM', 'INTERSECTION_INTEGRAL', 'MIN', &
                  'MAX' /
      data ncode / 8, 4, 4, 3, 5,   7,10, 8, 8, 5, &
                   4, 7, 4, 4, 4,   8, 8, 8, 6, 5, &
                   4, 8, 8, 5,12,  12, 6, 3, 3 /
      data ncodmx / 8, 4, 7, 11, 7,   8,13, 8, 8, 9, &
                    4,10, 8, 8, 8,   13,13, 9, 8,12, &
                   14,10,18, 8,17,   12,21, 3, 3 /

!     code   Table with keywords corresponding to options of COMPUTE_SCALAR
!             Values:
!             1:  NORM_DIFFERENCE
!             2:  NORM
!             3:  AVERAGE
!             4:  DOT_PRODUCT
!             5:  MIN_MAX
!             6:  SCAL_MAX
!             7:  EXTRACT_VALUE
!             8:  COOR_MIN
!             9:  COOR_MAX
!            10:  ABS_VALUE
!            11:  NODE
!            12:  USER_POINT
!            13:  ONE_NORM
!            14:  TWO_NORM
!            15:  INF_NORM
!            16:  MEAN_ONE_NORM
!            17:  MEAN_TWO_NORM
!            18:  MEAN_AREA
!            19:  MIN_AREA
!            20:  POINT_NUMBER
!            21:  ZERO_LEVEL_SET
!            22:  MEAN_VALUE
!            23:  STANDARD_DEVIATION
!            24:  INTEGRAL
!            25:  BOUNDARY_INTEGRAL
!            26:  BOUNDARY_SUM
!            27:  INTERSECTION_INTEGRAL
!            28:  MIN
!            29:  MAX
! ======================================================================
!
      call eropen ( 'readscalrhs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscalrhs'
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,1) 'lastitem, ityprd, nchar', &
                          lastitem, ityprd, nchar
         write(irefwr,*) 'string ', string(1:nchar)
         write(irefwr,*) 'End readscalrhs'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      number = 0  ! initialize number

      if ( typeitems(lastitem)==3 ) then

!     --- string found, &
!         check if it is one of the known functions

         if ( debug ) then
            write(irefwr,1) 'ityprd, nchar', ityprd, nchar
            write(irefwr,*) 'string', string(1:nchar)
         end if  ! ( debug )

         do i = 1, nncode

            if ( nchar>=ncode(i) .and. nchar<=ncodmx(i) ) then

!           --- Number of characters in correct range

               if ( string(1:nchar)==code(i)(1:nchar) ) then

!              --- string matches keyword

                  number = i
                  exit

               end if  ! ( string(1:nchar)==code(i)(1:nchar) )

            end if  ! ( nchar>=ncode(i) .and. nchar<=ncodmx(i) )

         end do  ! i = 1, nncode

      end if  ! ( typeitems(lastitem)==3 )

1000  call erclos ( 'readscalrhs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lastitem, number', lastitem, number
         write(irefwr,*) 'End readscalrhs'

      end if  ! ( debug )

      end
