      subroutine readseqintgr ( table, iinput, tabmin, icode, defvalue, &
                                name )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the value of the integer variable read in iinput
!     If no sequence number is found, use default value
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/creadinfof90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision table(*), tabmin(*)
      integer iinput, icode, defvalue
      character (len=*) name

!     defvalue       i    Default value
!     icode          i    Sequence number of variable in table and tabmin
!     iinput         o    Entry of integer input array referring to variable
!                         If value stored in table(icode) iinput contains the
!                         entry in rinput where the value is stored
!                         If the variable is a scalar minus the entry
!                         in scalars is stored in iinput
!     name           i    Name of parameter for debugging
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmin         i    Contains minimum values allowed in table
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iseq           Sequence number in set of scalars
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readseqintgr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readseqintgr'
         write(irefwr,1) 'icode, infotable(icode)', &
                          icode, infotable(icode)
         write(irefwr,2) 'table(icode), tabmin(icode)', &
                          table(icode), tabmin(icode)
         write(irefwr,*) 'name ', name
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( infotable(icode)==11 ) then

!     --- variable found

         iseq = nint(table(icode))
         iinput = -iseq-maxints-maxreals

      else if ( table(icode)>=tabmin(icode) ) then

!     --- value is given

         iinput = nint(table(icode))

      else

!     --- value is not given, use default

         iinput = defvalue

      end if  ! ( infotable(icode)==11 )

1000  call erclos ( 'readseqintgr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinput', iinput
         write(irefwr,*) 'End readseqintgr'

      end if  ! ( debug )

      end
