      subroutine readstrings
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if name read corresponds to one the strings
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     keyword
!     string
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, iseq
      character (len=80) namecheck
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iseq           Sequence number
!     namecheck      Help parameter to store a name
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer readarrindex

!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READARRINDEX   Check if the name of a constant vector or varibale is
!                    followed by an index
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readstrings' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readstrings'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numstrings

         nch = ncharstrings(i)
         index_rdarr(1:5) = 0

         if ( nchar==nch ) then

            namecheck = chstrings(i)(1:nchar)
            call askupc ( namecheck )
            if ( string(1:nchar)==namecheck(1:nchar) ) then

!           --- name of string found

               iseq = readarrindex()
               if ( debug ) write(irefwr,1) 'iseq', iseq
               index_rdarr(3) = abs(iseq)

               if ( iseq>0 ) then

!              --- iseq>0, string index is integer

                  iseq = iseq-1  ! correction for adding

               else if ( iseq<0 ) then

!              --- iseq>0, string index is scalar

                  iseq = 0

               else if ( dimstrings(i)>1 ) then

!              --- iseq = 0 but indexed string, &
!                  hence complete string must be used

                  iseq = 5000  ! to indicate complete string

               else

!              --- Standard string

                  iseq = 0

               end if  ! ( iseq>0 )

               if ( debug ) &
                  write(irefwr,1) 'ispecl, iseq, i', ispecl, iseq, i

               if ( ispecl==10 .or. ispecl==15 .or. ispecl==19 ) then

                  index_rdarr(1) = i+iseq
                  ityprd = 15

               else

                  realrd = i+iseq+100000000d0
                  if ( index_rdarr(3)>0 ) &
                     realrd = realrd+10000*index_rdarr(3)
                  index_rdarr(1) = i+iseq
                  index_rdarr(3) = 0
                  ityprd = 16

               end if  ! ( ispecl==10 .or. ispecl==15 .or. ispecl==19 )

               if ( debug ) write(irefwr,1) 'string, ityprd', i, ityprd
               go to 1000

            end if  ! ( string(1:nchar)==namecheck(1:nchar) )

         end if

      end do  ! i = 1, numstrings

1000  call erclos ( 'readstrings' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readstrings'

      end if  ! ( debug )

      end
