      subroutine readvecnum ( ivecnm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 07-04-2008 New way of dealoin with vector names
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 15-06-1999
!
!   copyright (c) 1999-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the vector number behind the letter V and store in ivecnm
!   This subroutine is a help subroutine for seppost
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cpost1f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivecnm

!     ivecnm     o  Vector number read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put string in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READNAMEVEC    Read name of vector in common cpost5
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!     659   Vector or constant number read is incorrect
!     660   Vector number read in input record does not correspond to a
!           predefined array
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ivecnm := 1
!     if ( ityprd#2 ) then
!        error: no number found behind V
!     else ( ityprd=2 )
!        ivecnm := vector number
!        Check if ivecnm in correct range
!        Check if there is an array corresponding to V_ivecnm
!        ivecnm := irefar(ivecnm)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readvecnum' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readvecnum'
         write(irefwr,1) 'ityprd, lastitem/1', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ityprd==4 .and. string(1:2)=='V ' ) call asknxt
      if (debug ) write(irefwr,1) 'ityprd, lastitem/2', ityprd, lastitem
      ivecnm = 1
      if ( ityprd==4 ) then

!     --- String found, check if it is a vector name

         call readnamevec

      end if  ! ( ityprd==4 )

      if ( ityprd/=2 .and. ityprd/=10 ) then

!     --- Not a number found after V

         call errchr ( 'V', 1 )
         call errsub ( 574, 0, 0, 1 )
         call errcrd
         go to 1000

      else

!     --- Number found, check

         ivecnm = nint(realrd)
         if ( ivecnm<0 .or. ivecnm>2000 ) then

!        --- ivecnm out of range

            call errint ( ivecnm, 1 )
            call errint ( 2000, 2 )
            call errsub ( 659, 2, 0, 0 )
            call errcrd
            go to 1000

         end if
         if ( ivecnm<=1000 ) then

!        --- ivecnm<=1000

            if ( irefar(ivecnm)==0 ) then

!           --- There is no array corresponding to ivecnm

               call errint ( ivecnm, 1 )
               call errsub ( 660, 1, 0, 0 )
               call errcrd
               go to 1000

            else

               ivecnm = irefar(ivecnm)

            end if

         end if

      end if

1000  call erclos ( 'readvecnum' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readvecnum'

      end if  ! ( debug )

      end
