      subroutine readvectnum ( ivecnm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-09-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read vector name or Vi or V%vectorname
!     Return with sequence number of vector name
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost1f90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepmainf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivecnm

!     ivecnm        i/o   vector sequencer number with respect to isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer minval, maxval, ispeclsav
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ispeclsav      Help parameter to save the value of ispecl
!     maxval         Maximum value allowed for the integer read
!     minval         Minimum value allowed for the integer read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READNAMEVEC    Read name of vector in common cpost5
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2951   Sequence number of vectors out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readvectnum' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readvectnum'
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read next item, should be a string

      call asknxt

      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem/2', ityprd, lastitem
         write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

      if ( ityprd/=4 ) then

!     --- ityprd#4, not a string found, return

         lastitem = lastitembef
         go to 1000

      end if  ! ( ityprd/=4 )

      if ( string(1:nchar)=='V' ) then

!     --- Letter V found, check if it is followed by a %-symbol or
!         a number

         if ( typeitems(lastitem+1)==10 ) then

!        --- % found, read vector number

            call asknxt
            go to 200

         end if  ! ( typeitems(lastitem+1)==10 )

!        --- Reread item with number

         lastitem = lastitembef
         ispeclsav = ispecl
         ispecl = 9
         call asknxt

         if ( debug ) then
            write(irefwr,1) 'ityprd, lastitem/3', ityprd, lastitem
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         ispecl = ispeclsav
         call readnamevec    ! name of vector
         if ( ityprd==2 ) go to 200   ! vector name found
         lastitem = lastitembef+1     ! reset to position V

         if ( typeitems(lastitem+1)==1 ) then

!        --- number found, read vector number

            call asknxt

         else

!        --- not a number found, reset lastitem

            lastitem = lastitembef

         end if  ! ( typeitems(lastitem+1)==1 ) then

      else

!     --- Other string found, include numbers if necessary

         lastitem = lastitembef
         ispeclsav = ispecl
         ispecl = 9
         call asknxt

         if ( debug ) then
            write(irefwr,1) 'ityprd, lastitem/4', ityprd, lastitem
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         ispecl = ispeclsav
         call readnamevec    ! name of vector
         if ( ityprd/=2 ) lastitem = lastitembef  ! reset lastitem

      end if  ! ( string(1:nchar)=='V' ) then

200   if ( ityprd==2 ) then

!     --- number found, store in ivecnm

         ivecnm = nint(realrd)
         if ( debug ) write(irefwr,1) 'ivecnm', ivecnm
         maxval = numvec
         minval = 1
         if ( maxval==0 ) maxval = maxvc1

         if ( mainsubr==4 ) then

!        --- In case of call by seppost we have to decrease the values by 1

            maxval = numvec-1
            minval = 0

         end if  ! ( mainsubr==4 )

         if ( ivecnm<minval .or. ivecnm>maxval ) then

!        --- Vector number out of range

            call errint ( ivecnm, 1 )
            call errint ( maxval, 2 )
            call errsub ( 2951, 2, 0, 0 )
            call errcrd

         end if  ! ( ivecnm<=0 .or. ivecnm>numvec )

      end if  ! ( ityprd==2 )

1000  call erclos ( 'readvectnum' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ivecnm', ivecnm
         write(irefwr,*) 'End readvectnum'

      end if  ! ( debug )

      end
