      subroutine msh477( coor, i, j, k, e1, e2, e3, xm, ym, zm )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 03-09-2010 New release
!        version  1.0  date 30-12-1992
!
!   copyright (c) 1992-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine the vector (e1,e2,e3) which is perpen- &
!     dicular to the plane of the points i-j-k. Also point (xm,ym,zm)
!     which is a mean value between the origin of the circumcircle of
!     the triangle and the barycenter of the triangle is calculated.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3,*), e1,e2,e3, xm,ym,zm
      integer i,j,k

!     coor    i  coordinate array
!     e1      o  first component of normal vector
!     e2      o  second component of normal vector
!     e3      o  third component of normal vector
!     i       i  first node of triangle
!     j       i  second node of triangle
!     k       i  third node of triangle
!     xm      o  x-component of centre of circumcircle of triangle
!     ym      o  y-component of centre of circumcircle of triangle
!     zm      o  z-component of centre of circumcircle of triangle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision a1, a2, a3, a4, a5, a6, a7, a8, a9, &
                       b1, b2, b3, det, eleng, &
                       xi, xj, xk, yi, yj, yk, zi, zj, zk

!     a1      matrix entry
!     a2      matrix entry
!     a3      matrix entry
!     a4      matrix entry
!     a5      matrix entry
!     a6      matrix entry
!     a7      matrix entry
!     a8      matrix entry
!     a9      matrix entry
!     b1      rhs component
!     b2      rhs component
!     b3      rhs component
!     det     determinant of matrix
!     eleng   euclidian length of vector
!     xi      coordinate
!     xj      coordinate
!     xk      coordinate
!     yi      coordinate
!     yj      coordinate
!     yk      coordinate
!     zi      coordinate
!     zj      coordinate
!     zk      coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   deconcatenate name of string of calling subroutines
!     EROPEN   concatenate name to string of calling subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh477' )

!     --- Determine coordinates of the three nodes

      xi = coor(1,i)
      yi = coor(2,i)
      zi = coor(3,i)
      xj = coor(1,j)
      yj = coor(2,j)
      zj = coor(3,j)
      xk = coor(1,k)
      yk = coor(2,k)
      zk = coor(3,k)

!     --- Determine the "inward pointing normal"

      e1 = (yj-yi)*(zk-zi) - (yk-yi)*(zj-zi)
      e2 = (zj-zi)*(xk-xi) - (zk-zi)*(xj-xi)
      e3 = (xj-xi)*(yk-yi) - (xk-xi)*(yj-yi)

!     --- Normalize

      eleng = sqrt( e1*e1 + e2*e2 + e3*e3 )
      e1 = e1 / eleng
      e2 = e2 / eleng
      e3 = e3 / eleng

!     --- Matrix

      a1 = e1
      a2 = e2
      a3 = e3
      a4 = xi-xj
      a5 = yi-yj
      a6 = zi-zj
      a7 = xi-xk
      a8 = yi-yk
      a9 = zi-zk

!     --- Right-hand-side

      b1 = xi*e1 + yi*e2 + zi*e3
      b2 = ( xi*xi - xj*xj + yi*yi - yj*yj + zi*zi - zj*zj ) / 2.d0
      b3 = ( xi*xi - xk*xk + yi*yi - yk*yk + zi*zi - zk*zk ) / 2.d0

!     --- Solve system

      det  =  a1 * ( a5 * a9 - a6 * a8 ) &
            - a2 * ( a4 * a9 - a6 * a7 ) &
            + a3 * ( a4 * a8 - a5 * a7 )
      xm   =  b1 * ( a5 * a9 - a6 * a8 ) &
            - a2 * ( b2 * a9 - a6 * b3 ) &
            + a3 * ( b2 * a8 - a5 * b3 )
      ym   =  a1 * ( b2 * a9 - a6 * b3 ) &
            - b1 * ( a4 * a9 - a6 * a7 ) &
            + a3 * ( a4 * b3 - b2 * a7 )
      zm   =  a1 * ( a5 * b3 - b2 * a8 ) &
            - a2 * ( a4 * b3 - b2 * a7 ) &
            + b1 * ( a4 * a8 - a5 * a7 )
      xm  =    xm / det
      ym  =    ym / det
      zm  =    zm / det
      call erclos( 'msh477' )
      end
