      subroutine msh479( kstapl, kstap, line, nlind, coor, &
                         jn, itri, npoint, icheck )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 02-09-2010 New release
!        version  1.1  date 26-03-2008 Use smaller "box" in process
!        version  1.0  date 30-12-1992
!
!   copyright (c) 1992-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to check whether a line of the boundary has a common
!     point with one of the new triangles.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kstapl(*), kstap, line(*), nlind, itri(*), jn, npoint, &
              icheck
      double precision coor( 3,npoint )

!     coor       i    coordinate array
!     icheck     o    if a point j is inside the new element than icheck
!                        gets the node number j
!                     else if a line crosses the new element then at output
!                        icheck =-1
!                     else
!                        tetrahedron is allowed : icheck = 0
!     itri       i    array containing for each point ipnt indication where
!                     the point is situated
!                     if point ipnt belongs to the actual boundary then
!                        itri(ipnt)>0
!                     else if point ipnt is inside the volume then
!                        itri(ipnt)=-1
!                     else if point ipnt is outside then
!                        itri(ipnt) = 0
!     jn         i    node to be considered in combination with
!                     the first triangle of kstapl
!     kstap      i    number of triangles in kstapl
!     kstapl     i    actual array with boundary triangles
!     line            helparray to store the lines of the boundary
!     nlind           declared length of array line
!     npoint     i    length of array itri
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, i3, i4, ia, ib, il, j, je, &
              k1, k2, k3, ks, nline, ncheck
      double precision ext, &
              xd1, xd2, xd3, yd1, yd2, yd3, zd1, zd2, zd3, &
              xmin, xmax, ymin, ymax, zmin, zmax, xmind, xmaxd, &
              ymind, ymaxd, zmind, zmaxd, a1, a2, a3, b1, b2, b3, &
              c1, c2, c3, d1, d2, d3, det, xj, yj, zj, ski, eta, zta, t, &
              xa, ya, za, xb, yb, zb, xlm, ylm, zlm, &
              xn, yn, zn, x1, x2, x3, x4, y1, y2, y3, y4, z1, z2, z3, z4

!     a1      coefficient for determinant computation
!     a2      coefficient for determinant computation
!     a3      coefficient for determinant computation
!     b1      coefficient for determinant computation
!     b2      coefficient for determinant computation
!     b3      coefficient for determinant computation
!     c1      coefficient for determinant computation
!     c2      coefficient for determinant computation
!     c3      coefficient for determinant computation
!     d1      coefficient for determinant computation
!     d2      coefficient for determinant computation
!     d3      coefficient for determinant computation
!     det     variable to store determinant value
!     eta     barycentre coordinate
!     ext     barycentre coordinate
!     i1      first node triangle
!     i2      second node triangle
!     i3      third node triangle
!     i4      fourth node tetrahedron
!     ia      begin point line
!     ib      end point of line
!     il      line loop number
!     j       loop variable
!     je      end loop variable
!     k1      triangle node
!     k2      triangle node
!     k3      triangle node
!     ks      loop variable
!     ncheck  variable to store length of array line used
!     nline   number of lines stored in array line
!     ski     barycentric coordinate
!     t       parameter for crossing point
!     x1      x-coordinate
!     x2      x-coordinate
!     x3      x-coordinate
!     x4      x-coordinate
!     xa      x-coordinate first point line
!     xb      x-coordinate last point line
!     xd1     x-coordinate first node triangle
!     xd2     x-coordinate second node triangle
!     xd3     x-coordinate third node triangle
!     xj      x-coordinate
!     xlm     x-coordinate
!     xmax    help variable for surrounding cube
!     xmaxd   help variable for surrounding cube
!     xmin    help variable for surrounding cube
!     xmind   help variable for surrounding cube
!     xn      x-coordinate point to be considered
!     y1      y-coordinate
!     y2      y-coordinate
!     y3      y-coordinate
!     y4      y-coordinate
!     ya      y-coordinate
!     yb      y-coordinate
!     yd1     y-coordinate
!     yd2     y-coordinate
!     yd3     y-coordinate
!     yj      y-coordinate
!     ylm     y-coordinate
!     ymax    y-coordinate
!     ymaxd   y-coordinate
!     ymin    y-coordinate
!     ymind   y-coordinate
!     yn      y-coordinate
!     z1      z-coordinate
!     z2      z-coordinate
!     z3      z-coordinate
!     z4      z-coordinate
!     za      z-coordinate
!     zb      z-coordinate
!     zd1     z-coordinate
!     zd2     z-coordinate
!     zd3     z-coordinate
!     zj      z-coordinate
!     zlm     z-coordinate
!     zmax    z-coordinate
!     zmaxd   z-coordinate
!     zmin    z-coordinate
!     zmind   z-coordinate
!     zn      z-coordinate
!     zta     z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSH610         Determine intersection of triangle and line
!     MSH636         Compute determinant of matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2958   jn out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check whether some boundary point is "inside" new element
!     Determine the extreme values
!     Check value of determinant : if too small then point not allowed
!     Run through all points and check the local coordinates
!     If all coordinates are between 0 and 1 : Point is "inside"
!     Fill array line with all lines in boundary
!     Check whether all lines have no points in common with
!     new boundaries
!     Run through all lines skip all lines outside "block" , and
!     check for the three faces
!     Check for intersection points
!     Finally check whether the three new lines have no points in
!     common with all faces of the boundary
!     If no point inside and no crossing lines or faces set icheck 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh479' )

!     --- Check whether some boundary point is "inside" new element

      icheck = -1

      if ( jn>0.and.jn<=npoint ) then

!     --- 0<jn<=npoint

         xn = coor( 1,jn )
         yn = coor( 2,jn )
         zn = coor( 3,jn )
         i4 = jn

      else

!     --- jn out of range

         call errint ( jn, 1 )
         call errint ( npoint, 2 )
         call errsub ( 2958, 2, 0, 0 )
         go to 1000

      end if

      i1 = kstapl( 1 )
      i2 = kstapl( 2 )
      i3 = kstapl( 3 )

!     --- Coordinates basispoints
!         --- i1:

      xd1 = coor( 1,i1 )
      yd1 = coor( 2,i1 )
      zd1 = coor( 3,i1 )

!     --- i2:

      xd2 = coor( 1,i2 )
      yd2 = coor( 2,i2 )
      zd2 = coor( 3,i2 )

!     --- i3:

      xd3 = coor( 1,i3 )
      yd3 = coor( 2,i3 )
      zd3 = coor( 3,i3 )

!     --- Determine the extreme values
!         to make a box

      xmin = min(xd1,xd2,xd3,xn)
      ymin = min(yd1,yd2,yd3,yn)
      zmin = min(zd1,zd2,zd3,zn)
      xmax = max(xd1,xd2,xd3,xn)
      ymax = max(yd1,yd2,yd3,yn)
      zmax = max(zd1,zd2,zd3,zn)

!     --- Enlarge the box

      xmind = xmin - 1D-4*(xmax-xmin)
      xmaxd = xmax + 1D-4*(xmax-xmin)
      ymind = ymin - 1D-4*(ymax-ymin)
      ymaxd = ymax + 1D-4*(ymax-ymin)
      zmind = zmin - 1D-4*(zmax-zmin)
      zmaxd = zmax + 1D-4*(zmax-zmin)

!     --- Determine volume of tet:

      a1 = xd2 - xd1
      a2 = yd2 - yd1
      a3 = zd2 - zd1
      b1 = xd3 - xd1
      b2 = yd3 - yd1
      b3 = zd3 - zd1
      c1 =  xn - xd1
      c2 =  yn - yd1
      c3 =  zn - zd1
      call msh636(a1,a2,a3,b1,b2,b3,c1,c2,c3,det)

!     --- Check value of determinant : if too small then point not allowed

      if ( abs(det)<=0d0 ) then
           icheck = -2
           goto 1000
      end if

!     --- Run through all points and check that none of these points
!         is inside the new tetrahedron:

      do j = 1 , npoint

!     --- if ( j==265 ) then
!            write(*,*) 'geval 265'
!         end if
!         --- If node j is in outside region skip:

          if ( itri(j)==0 ) goto 100

!         --- If node j is one of the corners of the tet skip:

          if ( j==i1.or.j==i2.or.j==i3.or.j==i4 ) goto 100

!         --- If coordinates are such that node j is outside tet skip:

          xj = coor( 1,j )

!         --- Check x:

          if ( xj<xmind ) goto 100
          if ( xj>xmaxd ) goto 100
          yj = coor( 2,j )

!         --- Check y:

          if ( yj<ymind ) goto 100
          if ( yj>ymaxd ) goto 100
          zj = coor( 3,j )

!         --- Check z:

          if ( zj<zmind ) goto 100
          if ( zj>zmaxd ) goto 100
          d1 = xj - xd1
          d2 = yj - yd1
          d3 = zj - zd1

!         --- Check barycentric coordinates:

          call msh636(d1,d2,d3,b1,b2,b3,c1,c2,c3,ski)
          ski = ski / det
          if ( ski<0d0.or.ski>1d0 ) goto 100
          call msh636(a1,a2,a3,d1,d2,d3,c1,c2,c3,eta)
          eta = eta / det
          if ( eta<0d0.or.eta>1d0 ) goto 100
          call msh636(a1,a2,a3,b1,b2,b3,d1,d2,d3,zta)
          zta = zta / det
          if ( zta<0d0.or.zta>1d0 ) goto 100
          ext = 1d0 - ski - eta - zta
          if ( ext<0d0.or.ext>1d0 ) goto 100

!         --- Point is "inside"

          icheck = j
          goto 1000
100   end do

!     --- Fill array line with all lines in boundary

      nline = 0
      do ks = 2 , kstap
           k1 = kstapl( 3*ks - 2 )
           k2 = kstapl( 3*ks - 1 )
           k3 = kstapl( 3*ks     )
           if ( k1>k2 ) then
                line(2*nline+1) = k1
                line(2*nline+2) = k2
                nline = nline + 1
           end if
           if ( k2>k3 ) then
                line(2*nline+1) = k2
                line(2*nline+2) = k3
                nline = nline + 1
           end if
           if ( k3>k1 ) then
                line(2*nline+1) = k3
                line(2*nline+2) = k1
                nline = nline + 1
           end if
      end do

!     --- Length of array line enough ?

      ncheck = 2 * nline
      call inichkarlen ( ncheck, nlind, 'line' )
      if ( ierror/=0 ) go to 1000

!     --- Check whether all lines have no points in common with
!         new boundaries.
!         Run through all lines skip all lines outside "block" , and
!         check for the three faces

      do il = 1 , nline
          ia = line(2*il-1)
          ib = line(2*il  )
          xa = coor( 1,ia )
          ya = coor( 2,ia )
          za = coor( 3,ia )
          xb = coor( 1,ib )
          yb = coor( 2,ib )
          zb = coor( 3,ib )
          xlm = min(xa,xb)
          if ( xlm>xmax ) goto 300
          xlm = max(xa,xb)
          if ( xlm<xmin ) goto 300
          ylm = min(ya,yb)
          if ( ylm>ymax ) goto 300
          ylm = max(ya,yb)
          if ( ylm<ymin ) goto 300
          zlm = min(za,zb)
          if ( zlm>zmax ) goto 300
          zlm = max(za,zb)
          if ( zlm<zmin ) goto 300

!         --- First triangle

          je = 1
          if ( ia==i1.or.ia==i2.or.ia==jn ) je = 0
          if ( ib==i1.or.ib==i2.or.ib==jn ) je = 0
          if ( je==1 ) then
               call msh610(xa,ya,za,xb,yb,zb,xd1,yd1,zd1,xd2,yd2,zd2, &
                           xn,yn,zn,t)
               if ( t<1d0 ) goto 1000
          end if

!         --- Second triangle

          je = 1
          if ( ia==i2.or.ia==i3.or.ia==jn ) je = 0
          if ( ib==i2.or.ib==i3.or.ib==jn ) je = 0
          if ( je==1 ) then
               call msh610(xa,ya,za,xb,yb,zb,xd2,yd2,zd2,xd3,yd3,zd3, &
                           xn,yn,zn,t)
               if ( t<1d0 ) goto 1000
          end if

!         --- Third triangle

          je = 1
          if ( ia==i3.or.ia==i1.or.ia==jn ) je = 0
          if ( ib==i3.or.ib==i1.or.ib==jn ) je = 0
          if ( je==1 ) then
               call msh610(xa,ya,za,xb,yb,zb,xd3,yd3,zd3,xd1,yd1,zd1, &
                           xn,yn,zn,t)
               if ( t<1d0 ) goto 1000
          end if

300   end do

!     --- Finally check whether the three new lines have no points in
!         common with all faces of the boundary
!         --- Compute the coordinates of the four nodes of the tet

      x1 = xd1
      y1 = yd1
      z1 = zd1
      x2 = xd2
      y2 = yd2
      z2 = zd2
      x3 = xd3
      y3 = yd3
      z3 = zd3
      x4 = xn
      y4 = yn
      z4 = zn

!     --- Run through all faces and check for the lines
!         1 - 4, 2 - 4, 3 - 4

      do ks = 2 , kstap

!     --- Nodes of boundary triangle

         k1 = kstapl( 3*ks - 2 )
         k2 = kstapl( 3*ks - 1 )
         k3 = kstapl( 3*ks     )

!        --- If i4 is node of this boundary triangle : ready

         if ( i4==k1.or.i4==k2.or.i4==k3 ) goto 500

!        --- Compute coordinates of nodes of boundary triangle

         xd1 = coor( 1,k1 )
         yd1 = coor( 2,k1 )
         zd1 = coor( 3,k1 )
         xd2 = coor( 1,k2 )
         yd2 = coor( 2,k2 )
         zd2 = coor( 3,k2 )
         xd3 = coor( 1,k3 )
         yd3 = coor( 2,k3 )
         zd3 = coor( 3,k3 )

!        --- Check line i1 - i4
!            Only check if i1 is not a node of the triangle

         if ( i1/=k1.and.i1/=k2.and.i1/=k3 ) then

!        --- Use coordinates line: i1 - i4

            call msh610(x1,y1,z1,x4,y4,z4,xd1,yd1,zd1,xd2,yd2,zd2, &
                        xd3,yd3,zd3,t)
            if ( t<1d0 ) goto 1000
         end if

!        --- Check line i2 - i4
!            Only check if i2 is not a node of the triangle

         if ( i2/=k1.and.i2/=k2.and.i2/=k3 ) then

!        --- Use coordinates line: i2 - i4

            call msh610(x2,y2,z2,x4,y4,z4,xd1,yd1,zd1,xd2,yd2,zd2, &
                        xd3,yd3,zd3,t)
            if ( t<1d0 ) goto 1000
         end if

!        --- Check line i3 - i4
!            Only check if i3 is not a node of the triangle

         if ( i3/=k1.and.i3/=k2.and.i3/=k3 ) then

!        --- Use coordinates line: i3 - i4

            call msh610(x3,y3,z3,x4,y4,z4,xd1,yd1,zd1,xd2,yd2,zd2, &
                        xd3,yd3,zd3,t)
            if ( t<1d0 ) goto 1000
         end if

500   end do

!     --- No point inside found and no crossing lines or faces found

      icheck = 0
1000  call erclos( 'msh479' )
      end
