      subroutine msh480ext ( xp1, xp2, xt1, xt2, xt3  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 29-08-2014 Correction fact2
!        version  1.0  date 06-12-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     perform extra check on intersection of line and triangle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     line
!     triangle
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision xp1(3), xp2(3), xt1(3), xt2(3), xt3(3)

!     xp1            i    coordinates of first point of line
!     xp2            i    coordinates of second point of line
!     xt1            i    coordinates of first point of triangle
!     xt2            i    coordinates of second point of triangle
!     xt3            i    coordinates of third point of triangle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision fact1, fact2(2)

!     fact1          multiplication factor
!     fact2          multiplication factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHCROSSSURF   Find the intersection of the lines xp1-xp2 and the
!                    triangle x1,x2,x3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3157   Internal error in msh480
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh480ext' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh480ext'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call mshcrosssurf ( xp1, xp2, xt1, xt2, xt3, fact1, fact2, &
                          1d-10 )

      if ( fact1<0d0 .or. fact1>1d0 ) then
         call errsub ( 3157, 0, 0, 0 )
      end if  ! ( (fact1<0d0 .or. fact1>1d0) ... )

1000  call erclos ( 'msh480ext' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh480ext'

      end if  ! ( debug )

      end
