      subroutine msh604 ( coor, npoint, xmin, xmax, ymin, ymax, &
                          zmin, zmax )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 03-09-2010 New release
!        version  2.1  date 27-06-2008 textual adjustments
!        version  2.0  date 13-04-2005 Update code
!        version  1.2  date 19-02-1993 New layout
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine extreme values of given volume
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3,*), xmin, xmax, ymin, ymax, zmin, zmax
      integer npoint

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     npoint  i    number of given surface-nodes
!     xmax    o    Maximum x-value
!     xmin    o    Minimum x-value
!     ymax    o    Maximum y-value
!     ymin    o    Minimum y-value
!     zmax    o    Maximum z-value
!     zmin    o    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision x, y, z

!     i       Counting variable
!     x       x co-ordinate of point
!     y       y co-ordinate of point
!     z       z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set max values

      xmax = -rinfin
      ymax = -rinfin
      zmax = -rinfin

!     --- Set min values

      xmin = -xmax
      ymin = -ymax
      zmin = -zmax

      do i = 1, npoint
         x = coor( 1, i )
         y = coor( 2, i )
         z = coor( 3, i )
         if ( xmin>x ) xmin = x
         if ( xmax<x ) xmax = x
         if ( ymin>y ) ymin = y
         if ( ymax<y ) ymax = y
         if ( zmin>z ) zmin = z
         if ( zmax<z ) zmax = z
      end do

      end
