      subroutine msh611 ( x1, y1, z1, x2, y2, z2, x3, y3, z3, xs, ys, &
                          zs, t )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 03-09-2010 New release
!        version  1.2  date 13-04-2005 Update code
!        version  1.1  date 28-03-1999 Include commons
!        version  1.0  date 14-12-1987
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine whether point (xs,ys,zs) is inside angle
!     ( 2 - 1 - 3 ) or not .
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1, y1, z1, x2, y2, z2, x3, y3, z3, xs, ys, zs, t

!     t          o    if ( 0<t<1 ) then there is a common
!                     point , else t = 1000
!     x1         i    x-coordinate of vertex 1
!     x2         i    x-coordinate of vertex 2
!     x3         i    x-coordinate of vertex 3
!     xs         i    x-coordinate intersection point
!     y1         i    y-coordinate of vertex 1
!     y2         i    y-coordinate of vertex 2
!     y3         i    y-coordinate of vertex 3
!     ys         i    y-coordinate intersection point
!     z1         i    z-coordinate of vertex 3
!     z2         i    z-coordinate of vertex 3
!     z3         i    z-coordinate of vertex 3
!     zs         i    z-coordinate intersection point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision a1, a2, a3, angle1, angle2, anrm, b1, b2, bnrm, &
                       c1, c2, c3, cnrm, eps, b3

!     a1         dif variable
!     a2         dif variable
!     a3         dif variable
!     angle1     angle
!     angle2     angle
!     anrm       euclidian norm of differences vector
!     b1         dif variable
!     b2         dif variable
!     b3         dif variable
!     bnrm       euclidian norm of differences vector
!     c1         dif variable
!     c2         dif variable
!     c3         dif variable
!     cnrm       euclidian norm of differences vector
!     eps        local accuracy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      eps = 10d2 * epsmac

      a1 = x2 - x1
      a2 = y2 - y1
      a3 = z2 - z1

      anrm = sqrt( a1*a1 + a2*a2 + a3*a3 )

      b1 = x3 - x1
      b2 = y3 - y1
      b3 = z3 - z1

      bnrm = sqrt( b1*b1 + b2*b2 + b3*b3 )

      angle1 = ( a1*b1 + a2*b2 + a3*b3 ) / ( anrm * bnrm )

      c1 = xs - x1
      c2 = ys - y1
      c3 = zs - z1

      cnrm = sqrt( c1*c1 + c2*c2 + c3*c3 )

      if ( cnrm<eps ) return

      angle2 = ( a1*c1 + a2*c2 + a3*c3 ) / ( anrm * cnrm )

      if ( angle2<angle1 - eps )  t = 1000.d0

      end
