      subroutine msh612 ( kelem, nelem, i1, i2, i3, jpn, &
                          kstapl, kstap, itri )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 03-09-2010 New release
!        version  2.1  date 27-06-2008 Textual adjustments
!        version  2.0  date 13-04-2005 Update code
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to fill new element in array KELEM and to adjust the
!     arrays KSTAPL and ITRI
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kelem(4,*), kstapl(3,*), itri(*), nelem, &
              i1, i2, i3, jpn, kstap

!     i1      i    node of new element
!     i2      i    node of new element
!     i3      i    node of new element
!     itri    i    array containing for each node nk whether
!                  that node is on the actual boundary ( i.e.
!                  itri(nk)>0 ) or not (i.e. itri(nk) = 0 )
!     jpn     i    node of new element
!     kelem  i/o   New KMESH part c with respect to the surface elements
!                  All elements have the same number of nodes
!     kstap  i/o   number of elements in kstapl
!     kstapl i/o   array containing the actual boundary
!     nelem  i/o   Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jstap, id1, id2, id3, ib, j1, j2, j3, ja, jb, jc, &
              jd, je, jf

!     ib        loop variable
!     id1       helpvariable for exclusion
!     id2       helpvariable for exclusion
!     id3       helpvariable for exclusion
!     j1        nodenumber boundary triangle
!     j2        nodenumber boundary triangle
!     j3        nodenumber boundary triangle
!     ja        removal indicator
!     jb        removal indicator
!     jc        removal indicator
!     jd        removal indicator
!     je        removal indicator
!     jf        removal indicator
!     jstap     triangle counter kstapl
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      jstap = 0

      nelem = nelem + 1

      kelem( 1, nelem ) = i1
      kelem( 2, nelem ) = i2
      kelem( 3, nelem ) = i3
      kelem( 4, nelem ) = jpn

!     --- Adjust kstapl and itri

      id1   = 1
      id2   = 1
      id3   = 1

      do ib = 2, kstap

          j1 = kstapl( 1, ib )
          j2 = kstapl( 2, ib )
          j3 = kstapl( 3, ib )

          ja = 1
          if ( j1==i1.or.j1==i2.or.j1==i3.or.j1==jpn ) ja = 0
          jb = 1
          if ( j2==i1.or.j2==i2.or.j2==i3.or.j2==jpn ) jb = 0
          jc = 1
          if ( j3==i1.or.j3==i2.or.j3==i3.or.j3==jpn ) jc = 0

          if ( ja==0 .and. jb==0 .and. jc==0 ) then

!         --- Triangle has to be excluded

               itri(j1) = itri(j1) - 1
               itri(j2) = itri(j2) - 1
               itri(j3) = itri(j3) - 1
               if ( id1==1 ) then
                    jd = 1
                    if (j1==i1.or.j1==i2.or.j1==jpn) jd=0
                    je = 1
                    if (j2==i1.or.j2==i2.or.j2==jpn) je=0
                    jf = 1
                    if (j3==i1.or.j3==i2.or.j3==jpn) jf=0

                    if (jd==0.and.je==0.and.jf==0) id1=0
               end if
               if ( id2==1 ) then
                    jd = 1
                    if (j1==i2.or.j1==i3.or.j1==jpn) jd=0
                    je = 1
                    if (j2==i2.or.j2==i3.or.j2==jpn) je=0
                    jf = 1
                    if (j3==i2.or.j3==i3.or.j3==jpn) jf=0

                    if (jd==0.and.je==0.and.jf==0) id2=0
               end if
               if ( id3==1 ) then
                    jd = 1
                    if (j1==i3.or.j1==i1.or.j1==jpn) jd=0
                    je = 1
                    if (j2==i3.or.j2==i1.or.j2==jpn) je=0
                    jf = 1
                    if (j3==i3.or.j3==i1.or.j3==jpn) jf=0

                    if (jd==0.and.je==0.and.jf==0) id3=0
               end if
          else
               jstap = jstap + 1
               kstapl( 1, jstap ) = j1
               kstapl( 2, jstap ) = j2
               kstapl( 3, jstap ) = j3
          end if
      end do

!     --- Check whether 0 , 1 , 2 or 3 triangles have already been
!         cancelled

      if ( id1==1 ) then
           jstap = jstap + 1

           kstapl( 1, jstap ) = i1
           kstapl( 2, jstap ) = i2
           kstapl( 3, jstap ) = jpn

           itri( i1)  = itri( i1) + 1
           itri( i2)  = itri( i2) + 1
           itri(jpn)  = itri(jpn) + 1
      end if
      if ( id2==1 ) then
           jstap = jstap + 1

           kstapl( 1, jstap ) = i2
           kstapl( 2, jstap ) = i3
           kstapl( 3, jstap ) = jpn

           itri( i2)  = itri( i2) + 1
           itri( i3)  = itri( i3) + 1
           itri(jpn)  = itri(jpn) + 1
      end if
      if ( id3==1 ) then
           jstap = jstap + 1

           kstapl( 1, jstap ) = i3
           kstapl( 2, jstap ) = i1
           kstapl( 3, jstap ) = jpn

           itri( i3 ) = itri( i3 )  + 1
           itri( i1 ) = itri( i1 )  + 1
           itri(jpn ) = itri(jpn )  + 1
      end if
      kstap = jstap
      end
