      subroutine msh615( coor  , kstapl, kstap , i1, i2, i3, &
                         iex1  , iex2  , iex3  , &
                         angle1, angle2, angle3 )
! ======================================================================
!
!        programmer    niek praagman
!        version  3.0  date 03-09-2010 New release
!        version  2.0  date 08-04-2005 Update code
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine for triangle i1 - i2 - i3 the other triangles
!     in the present boundary where i1 - i2 , i2 - i3 and i3 - i1 are
!     sides off. Also the angles between these triangles and the original
!     triangle are computed.
! **********************************************************************
!
!                       KEYWORDS
!
!     neighbour
!     angle
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kstapl(3,*), kstap , i1, i2, i3, iex1  , iex2  , iex3
      double precision coor(*), angle1, angle2, angle3

!     angle1          o   corner between basistriangle and iex1 triangle
!     angle2          o   corner between basistriangle and iex2 triangle
!     angle3          o   corner between basistriangle and iex3 triangle
!     coor            i   coordinate array
!     i1              i   first node basistriangle
!     i2              i   second node basistriangle
!     i3              i   third node basistriangle
!     iex1            i   extra node connected with i2-i3
!     iex2            i   extra node connected with i3-i1
!     iex3            i   extra node connected with i1-i2
!     kstap           i   number of elements (triangles) in kstapl
!     kstapl          i   boundary triangles array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ies, ih1, ih2, ih3, ii1, ii2, ii3, istop, &
              jh1, jh2, jh3, ji
      double precision olda

!     i       loop variable
!     ies     helpvariable for extra node
!     ih1     helpindicator
!     ih2     helpindicator
!     ih3     helpindicator
!     ii1     help for node number
!     ii2     help for node number
!     ii3     help for node number
!     istop   stopindicator
!     jh1     helpindicator
!     jh2     helpindicator
!     jh3     helpindicator
!     ji      indicator
!     olda    old angle value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH613         Compute angles between faces and lines of tetrahedron
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Initialize extra corners:

      iex1 = 0
      iex2 = 0
      iex3 = 0

!     --- Initialize values for the angles:

      angle1 = -2d0
      angle2 = -2d0
      angle3 = -2d0

!     --- Find extra values iexj and anglej for j=1,2,3

      do i = 2, kstap

!     --- Set helpvariable istop:

         istop = 0

!        --- Determine node numbers triangle:

         ii1 = kstapl( 1, i )
         ii2 = kstapl( 2, i )
         ii3 = kstapl( 3, i )

!        --- Check whether this triangle has a common side with
!            the basis triangle

         ji = 1
         if ( i1==ii1 .or. i1==ii2 .or. i1==ii3 ) ji = 0

!        --- Basisnode i1?

         if ( ji==0 ) then

!        --- Node i1 found

              ih1 = 1
         else

!        --- Not equal i1:

              ih1 = 0
         end if
         ji = 1
         if ( i2==ii1 .or. i2==ii2 .or. i2==ii3 ) ji = 0

!        --- Basisnode i2?

         if ( ji==0 ) then

!        --- Node i2 found

              ih2 = 1
         else

!        --- Not equal i2:

              ih2 = 0
         end if
         ji = 1
         if ( i3==ii1 .or. i3==ii2 .or. i3==ii3 ) ji = 0

!        --- Basisnode i3?

         if ( ji==0 ) then

!        --- Node i3 found

              ih3 = 1
         else

!        --- Not equal i3:

              ih3 = 0
         end if

!        --- Next check for two nodes:

         if ( ih1+ih2+ih3 ==2 ) then

!        --- One of the sides: Check the tetrahedron

            istop = 0
            if ( ih1==0 ) then
               if ( iex1>0 ) then
                    istop = 1
                    ies   = iex1
                    olda  = angle1
               end if
               jh1 = 1
               if ( ii1==i2 .or. ii1==i3 ) jh1 = 0
               jh2 = 1
               if ( ii2==i2 .or. ii2==i3 ) jh2 = 0
               jh3 = 1
               if ( ii3==i2 .or. ii3==i3 ) jh3 = 0
               if ( jh1==1 ) iex1 = ii1
               if ( jh2==1 ) iex1 = ii2
               if ( jh3==1 ) iex1 = ii3

!              --- Determine angle1:

               call msh613(i1,i2,i3,iex1,coor,angle1)

!              --- Check for mor possibilities:

               if ( istop==1 ) then
                  if ( olda>angle1 ) then
                       angle1 = olda
                       iex1   = ies
                  end if
               end if
            else if ( ih2==0 ) then
               if ( iex2>0 ) then
                    istop = 1
                    ies   = iex2
                    olda  = angle2
               end if
               jh1 = 1
               if ( ii1==i1 .or. ii1==i3 ) jh1 = 0
               jh2 = 1
               if ( ii2==i1 .or. ii2==i3 ) jh2 = 0
               jh3 = 1
               if ( ii3==i1 .or. ii3==i3 ) jh3 = 0
               if ( jh1==1 ) iex2 = ii1
               if ( jh2==1 ) iex2 = ii2
               if ( jh3==1 ) iex2 = ii3

!              --- Determine angle2:

               call msh613(i2,i3,i1,iex2,coor,angle2)

!              --- Check for more possibilities:

               if ( istop==1 ) then
                  if ( olda>angle2 ) then
                     angle2 = olda
                     iex2   = ies
                  end if
               end if
            else if ( ih3==0 ) then
               if ( iex3>0 ) then
                    istop = 1
                    ies   = iex3
                    olda  = angle3
               end if
               jh1 = 1
               if ( ii1==i1 .or. ii1==i2 ) jh1 = 0
               jh2 = 1
               if ( ii2==i1 .or. ii2==i2 ) jh2 = 0
               jh3 = 1
               if ( ii3==i1 .or. ii3==i2 ) jh3 = 0
               if ( jh1==1 ) iex3 = ii1
               if ( jh2==1 ) iex3 = ii2
               if ( jh3==1 ) iex3 = ii3

!              --- Determine angle3:

               call msh613(i3,i1,i2,iex3,coor,angle3)

!              --- Check for more possibilities:

               if ( istop==1 ) then
                  if ( olda>angle3 ) then
                     angle3 = olda
                     iex3   = ies
                  end if
               end if
           end if
        end if
      end do

      end
