      subroutine msh619 ( kelem, nelem, npoint, nipnt, iwork, ibuurp, &
                          leng )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.1  date 03-09-2010 New release
!        version  2.0  date 14-04-2005 Update code
!        version  1.2  date 28-03-1999 Include commons
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to fill array iwork which contains the number of
!     neighbour points for each point accumulated.
!     Also array ibuurp which contains the nodal point  numbers of the
!     neighbours is filled in the case of tetrahedrons.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, npoint, nipnt, leng, &
              kelem(*), iwork(npoint), ibuurp(*)

!     ibuurp     o    neighbour nodal points
!     iwork      o    array containing pointers for array ibuurp
!     kelem      i    element array
!     leng      i/o   input: declared length array ibuurp
!                     output: needed length array ibuurp
!     nelem      i    Number of elements
!     nipnt      i    number of boundary points
!     npoint     i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, is, isum, j

!     debug      if true debug statements are carried out otherwise
!     i          loop variable
!     i1         node
!     i2         node
!     i3         node
!     i4         node
!     is         counter
!     isum       sum variable
!     j          loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     INSTOP    Stop the program
!     MSH620    Place neighbour in work array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Start setting debug

      debug = .false.

!     --- Clear array iwork

      do i = 1 , npoint
         iwork(i)=0
      end do

!     --- Number of appearances in elements:

      do i = 1, nelem*4
         j        = kelem(i)
         iwork(j) = iwork(j)+1
      end do

!     --- All boundary points + 2

      do i = 1, nipnt
         iwork(i) = iwork(i) + 2
      end do

!     --- Rearrange and accumulate

      isum=0

!     --- Run through all nodes:

      do i = 1, npoint
         iwork(i) = iwork(i)+isum
         isum     = iwork(i)
      end do
      if (debug ) then

!     --- Message concerning length:

         write(irefwr,*) 'Length kstapl needed in msh619 is',isum
      end if
      if ( isum>leng ) then
           write(irefwr,*) ' Declaration error in subroutine msh619 '
           write(irefwr,*) ' Please inform SEPRA '
           write(irefwr,*) ' Declared length',leng,'needed',isum
           call instop
      end if

!     --- Clear array with neighbournodes ibuurp

      do i = 1, isum
         ibuurp(i) = 0
      end do

!     --- Length needed is isum

      leng = isum

!     --- Fill array ibuurp

      do i = 1 , nelem
         is = 4*(i-1)
         i1 = kelem( is + 1 )
         i2 = kelem( is + 2 )
         i3 = kelem( is + 3 )
         i4 = kelem( is + 4 )

!        --- Place neighbours:

         call msh620( ibuurp, iwork, i1, i2 )
         call msh620( ibuurp, iwork, i1, i3 )
         call msh620( ibuurp, iwork, i1, i4 )
         call msh620( ibuurp, iwork, i2, i1 )
         call msh620( ibuurp, iwork, i2, i3 )
         call msh620( ibuurp, iwork, i2, i4 )
         call msh620( ibuurp, iwork, i3, i1 )
         call msh620( ibuurp, iwork, i3, i2 )
         call msh620( ibuurp, iwork, i3, i4 )
         call msh620( ibuurp, iwork, i4, i1 )
         call msh620( ibuurp, iwork, i4, i2 )
         call msh620( ibuurp, iwork, i4, i3 )
      end do

      end
