      subroutine msh628( kelem, nelem, istart, ibuur )
! ======================================================================
!
!        programmer    niek praagman
!        version  3.0  date 08-09-2010 New release
!        version  2.0  date 15-04-2005 Update code
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refill array kelem. Linear elements are changed in quadratic elems
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, kelem(*), ibuur(*), istart(*)

!     ibuur       i   neighbour array
!     istart      i   array with startaddresses
!     kelem      i/o  element array
!     nelem       i   number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, i3, i4, i5, i6, i7, i8, i9, i10, &
              ielem, k

!     i1          nodenumber
!     i10         nodenumber
!     i2          nodenumber
!     i3          nodenumber
!     i4          nodenumber
!     i5          nodenumber
!     i6          nodenumber
!     i7          nodenumber
!     i8          nodenumber
!     i9          nodenumber
!     ielem       loopvariable
!     k           counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH629    Determine number of midpoint of linesegment
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do ielem = 1, nelem
         k = 4 * ( nelem - ielem )
         i1  = kelem( k + 1 )
         i3  = kelem( k + 2 )
         i5  = kelem( k + 3 )
         i10 = kelem( k + 4 )

!        --- Determine middle points

          call msh629( i1, i2,  i3, istart, ibuur )
          call msh629( i3, i4,  i5, istart, ibuur )
          call msh629( i5, i6,  i1, istart, ibuur )
          call msh629( i1, i7, i10, istart, ibuur )
          call msh629( i3, i8, i10, istart, ibuur )
          call msh629( i5, i9, i10, istart, ibuur )

!         --- Place all middle nodenumbers in element
!             Find startaddress

          k   = 10 * ( nelem - ielem )

!         --- Place all middel nodes

          kelem( k +  1 ) =  i1
          kelem( k +  2 ) =  i2
          kelem( k +  3 ) =  i3
          kelem( k +  4 ) =  i4
          kelem( k +  5 ) =  i5
          kelem( k +  6 ) =  i6
          kelem( k +  7 ) =  i7
          kelem( k +  8 ) =  i8
          kelem( k +  9 ) =  i9
          kelem( k + 10 ) = i10
      end do

      end
