      subroutine mshphil3d ( kbndpt, nbound, kmeshcloc, &
                             jnew, nipnt, fix_pnt, &
                             numnodextcrvs, curvenumbers, coorcurvs, &
                             surfnumbers  )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.5  date 29-05-2014 Extension of timebefore
!        version  2.4  date 06-12-2013 New call to msh476/670
!        version  2.3  date 20-11-2013 New call to msh601
!
!   copyright (c) 2006-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any1 means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to make a mesh of tetrahedrons for a volume which is
!     given by the triangles of kbndpt.(Alternative routine for mshg3d)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     tetrahedron
!     tesselation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(3,*), kmeshcloc(4,*), nbound, nipnt, &
              numnodextcrvs(*), curvenumbers(*), surfnumbers(*)
      double precision fix_pnt(4,nextrapnts), coorcurvs(3,*)
      logical jnew

!     coorcurvs      i    array containing the coordinates of the nodes on the
!                         internal curves
!     curvenumbers   i    array containing the curvenumbers of the extra
!                         internal curves
!     fix_pnt        i    array with coordinates and coarsenesses of
!                         prescribed internal fixed points
!     jnew           i    logical parameter to indicate whether the mesh
!                         has to be created (TRUE) or whether the mesh
!                         has to be repositioned (FALSE).
!     kbndpt         i    array containing the boundary triangles
!     kmeshcloc         o    element array
!     nbound         i    number of triangles in boundary
!     nipnt          i    number of points in boundary ; only needed if
!                         jnew = 'false'
!     numnodextcrvs  i    array with number of nodes on internal curves
!     surfnumbers    i    Array of size nextrasrfs containing the surface
!                         sequence numbers of each extra surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, ia, ib, ic, icall, ifout, jsurf, j, &
              jcheck, idr1, idr2, idr3, ielem, iex, iex1, iex2, iex3, &
              iherha, ii1, ii2, ii3, ipoint, irepos, istep, itessel, &
              itot, itry, inegat, iloop, ja, jaold, jas, jdrie, jelem, &
              jh, jh1, jh2, jh3, jherha, jmax, jmin, jnear, jold, jpn, &
              jstep, kdrie, kstap, kstmax, kherha, leng, lenkstapl, &
              lenline, n1, n2, n3, jelvol, nc1, ncube, nelhelp, nelold, &
              nextra, nkmeshc, nochan, nol, nptold, np, npunt, ntot, &
              nul, nx1, ny1, nz1, nexpnt, ntotal, numelm, nelmfix, &
              ifrato, nipcrv, nextratri, &
              lkbndpt, npndef, npntmp, neldef, neltmp, lnkbno
      double precision adis, afst1, afst2, afst3, alp, alp1, &
             angle1, bndmin, bndmax, angle2, angle3, bdis, cdis, &
             coa, coars, cros, ddis, dismax, dismin, &
             dista, e1, e2, e3, edis, factor, fdis, hh, &
             verh, vermax, vermin, ratio, ratiob, ratiop, ratold, &
             ratiom, refval, tran, vol, volmax, volref, &
             voluma, volumb, volume, volumi, &
             x1, x2, x3, y1, y2, y3, z1, z2, z3, t, dx, dy, dz, dis, th, &
             xhalf, xmax, xmin, xmint, xm, xn, xnx, xpt, xstart, xz, &
             yhalf, ymax, ymin, ymint, ym, yn, yny, ypt, ystart, yz, &
             zhalf, zmax, zmin, zmint, zm, zn, znz, zpt, zstart, zz, &
             maxcub, mincub, xminvec(4)
      integer, allocatable :: icube(:), jcube(:), itri(:), line(:)
      integer, allocatable :: kbnold(:,:), meshtmp(:,:), kstapl(:,:)
      double precision, allocatable :: chcoar(:), coarse(:), &
                                                      cube(:)
      double precision, allocatable :: coortmp(:,:), userco(:,:)
      double precision :: timebefore(2)

!     adis           distance in tetrahedron
!     afst1          distance between two points
!     afst2          distance between two points
!     afst3          distance between two points
!     alp            local coarseness in boundary triangle
!     alp1           extra local coarseness
!     angle1         computed angle
!     angle2         computed angle
!     angle3         computed angle
!     bdis           local distance
!     bndmax         max ratio volume element / local volume indication
!     bndmin         min ratio volume element / local volume indication
!     cdis           local distance
!     chcoar         allocatable help array for coarsenesses of points, and
!                    reference values for volumes
!     coa            coarseness
!     coars          coarseness
!     coarse         help array to store the local coarseness per point
!     coortmp        ?
!     cros           product of minimal and maximal ratio of local volumes
!     cube           allocatable help array of length ncube containing the
!                    required coarsenesses of the cubes
!     ddis           distance
!     dis            ?
!     dismax         max distance
!     dismin         min distance
!     dista          help distance
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     e1             x-component of normal on triangle
!     e2             y-component of normal on triangle
!     e3             z-component of normal on triangle
!     edis           local distance in tetrahedron
!     factor         multiplication factor for new point distance
!     fdis           local distance in tetrahedron
!     hh             helpparameter for accuracy
!     i              loop variable
!     i1             node number
!     i2             node number
!     i3             node number
!     i4             node number
!     ia             node number
!     ib             node number
!     ic             node number
!     icall          Call sequence number for present subroutine
!     icube          allocatable help array of length npointlc containing for each
!                    node its cube number
!     idr1           local number of boundary triangle
!     idr2           local number of boundary triangle
!     idr3           local number of boundary triangle
!     ielem          number of element
!     iex            neighbour node number
!     iex1           neighbour node
!     iex2           neighbour node
!     iex3           neighbour node
!     ifout          counter of errors
!     ifrato         indicator for first ratio computation
!     iherha         indicator for iterations
!     ii1            node
!     ii2            node
!     ii3            node
!     iloop          loop variable
!     inegat         counter for negative volumes
!     ipoint         node number
!     irepos         indicator whether repositioning is wanted or not
!     istep          number of special adjustments
!     itessel        counter number of tesselation calls
!     itot           local total sum
!     itri           help array of length npunt, containing informa- &
!                    tion whether a nodal point is in the actual
!                    boundary or not
!     itry           number of tries
!     j              help variable for loops
!     ja             indicator
!     jaold          indicator of loop that succeeded present loop
!     jas            help indicator
!     jcheck         indicator whether new tetrahedron is permitted or not
!     jcube          allocatable help array of length ncube indicating whether a
!                    cube is outside the volume ( 0 ), a boundary cube
!                    ( 1 ) or inside ( 2 ) the volume
!     jdrie          triangle number
!     jelem          local element number
!     jelvol         number of element with "large" negative ratio
!     jh             local node
!     jh1            local node
!     jh2            local node
!     jh3            local node
!     jherha         number of iterations
!     jmax           element number of (relative) largest element
!     jmin           element number of (relative) smallest element
!     jnear          help indicator
!     jold           old node number
!     jpn            local node number
!     jstep          number of special emergency adjustments
!     jsurf          surface number
!     kbnold         help array with starting boundary triangles
!     kdrie          local triangle number
!     kherha         counter for repeatings
!     kstap          actual number of triangles in boundary array kstapl
!     kstapl         help array for temporary boundary elements
!     kstmax         max length of kstapl used
!     leng           length used for repositioning
!     lenkstapl      allocated length kstapl array
!     lenline        allocated length line array
!     line           temporary help array for boundary lines
!     lkbndpt        length nodes in kbndpt
!     lnkbno         length starting array of boundary triangles
!     maxcub         maximum value in array cube
!     meshtmp        ?
!     mincub         minimum value in array cube
!     n1             number of points in x-direction
!     n2             number of points in y-direction
!     n3             number of points in z-direction
!     nc1             number of reference cube point is situated in
!     ncube          number of cubes for total volume
!     neldef         ?
!     nelhelp        help variable for surface number
!     nelmfix        number of elements fixed by input
!     nelold         number of elements in preceding loop
!     neltmp         ?
!     nexpnt         counter for extra interior points
!     nextra         help variable for length of arrays
!     nextratri      number of extra triangles in extra surfaces
!     nipcrv         number of points on internal curves
!     nkmeshc        number of elements
!     nochan         number of successive failures during division
!     nol            difference of elements
!     np             number of nodes
!     npndef         ?
!     npntmp         ?
!     nptold         old number of points
!     npunt          estimated number of points
!     ntot           l2-sum of differences
!     ntotal         total number of nodes to be used in Tesselation
!     nul            difference in nbr of points after two successive steps
!     numelm         variable for barycenter test (no of element)
!     nx1             number of points in x-direction for reference cubes
!     ny1             number of points in y-direction for reference cubes
!     nz1             number of points in z-direction for reference cubes
!     ratio          value of 3 * (radius inner sphere / radius outer sphere)
!     ratiob         help value to store extreme ratio value
!     ratiom         variable to store the mean ratio value
!     ratiop         ratioindicator stored result
!     ratold         ratio of run before last run
!     refval         minimal reference value for volume
!     t              ?
!     th             ?
!     tran           ratio value for transformation to unit cube in first octant
!     userco         help array to store coordinates submitted userpoints
!     verh           ratio of volumes
!     vermax         max of ratio's of volumes in present loop
!     vermin         min of ratio's of volumes in present loop
!     vol            volume value
!     volmax         max volume value
!     volref         reference volume value
!     voluma         volume value of present element
!     volumb         volume value of total volume considered
!     volume         volume value of present element
!     volumi         min volume value
!     x1             ?
!     x2             ?
!     x3             ?
!     xhalf          x-coordinate of mid-point
!     xm             x-coordinate
!     xmax           max x-value in region considered
!     xmin           min x-value in region considered
!     xmint          min x-value of all usersubmitted points
!     xminvec        Contains the vector xmint, ymint, zmint and tran
!     xn             x-coordinate
!     xnx            x-coordinate
!     xpt            x-coordinate
!     xstart         help x-variable to consider region
!     xz             x-coordinate of barycenter
!     y1             ?
!     y2             ?
!     y3             ?
!     yhalf          y-coordinate of mid-point
!     ym             y-coordinate
!     ymax           max y-value in region considered
!     ymin           min y-value in region considered
!     ymint          min y-value of all usersubmitted points
!     yn             y-coordinate
!     yny            y-coordinate
!     ypt            y-coordinate
!     ystart         help y-value to consider region
!     yz             y-coordinate of barycenter
!     z1             ?
!     z2             ?
!     z3             ?
!     zhalf          z-coordinate of mid-point
!     zm             z-coordinate
!     zmax           max z-value in region considered
!     zmin           min z-value in region considered
!     zmint          min z-value of all usersubmitted points
!     zn             z-coordinate
!     znz            z-coordinate
!     zpt            z-coordinate
!     zstart         help z-value to consider region
!     zz             z-coordinate of barycenter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     INICHKARLEN    Check length of array and give error message 550 if
!                    arraysize is too small
!     MSH476         Construct a tetrahedronal mesh given the points in 3D
!                    by Delaunay
!     MSH480         check whether boundary does not intersect itself
!     MSH601         Check boundary and determine coarsenesses
!     MSH603         compute the distance between two points i and j
!     MSH604         Determine extreme coordinate values for volume
!     MSH605         Determine extreme distances for volume
!     MSH606         Determine coarseness for help-cubes
!     MSH607         check whether point is inside or outside volume
!     MSH608         Place all triangles of boundary in right order
!     MSH609         Determine centre of circumcircle of triangle
!     MSH610         Determine intersection of triangle and line
!     MSH612         fill element in array and adjust
!     MSH613         compute the angle between triangle i1 - i2 - i3 and
!                    triangle i4 - i2 - i3
!     MSH615         compute special neighbour points of triangle
!     MSH616         Check whether new point is visible
!     MSH617         Determine nearest boundary point to new created point
!     MSH618         Place element in element array, adjust boundary and fill
!                    coor
!     MSH619         Determine all the neighbours of each point
!     MSH621         Make repositioning of mesh
!     MSH631         Check whether all lines of new tetrahedron are allowed
!     MSH632         compute volume of tetrahedron
!     MSH633         Place first boundary triangle on last positions
!     MSH634         Compute reference volume values
!     MSH637         Check boundary triangles
!     MSH638         Skip local (bad) part of mesh and start again
!     MSH639         Check whether all triangles are unique
!     MSH640         Check all internal angles of tetrahedron
!     MSH647         Check whether boundary does not intersect itself
!     MSH649         Renumber kelem, kstap, coor, so that only used nodes
!                    occur in array kelem
!     MSH650         Cancel all elements with at least one point in common with
!                    the boundary , but keep the points
!     MSH651         Check intersections for new proposed tetrahedron (debug)
!     MSH652         Check if all tetrahedrons of kmeshcloc are disjoint
!     MSH655         Place special internal nodes and fill surrounding
!                    special fixed elements
!     MSH656         Place special nodes and elements for internal lines
!     MSH661         Compute ratio of inner and outer sphere of tetrahedron
!     MSH663         Skip if possible bad tetrahedron by joining nodes.
!     MSH665         Skip bad (flat) tetrahedron by rearranging the nodes.
!                    Try to redivide bunches of three or four tetrahedrons.
!     MSH670         Construct a mesh given the points and using an adjusted
!                    Delaunay method
!     MSH672         Check all mutual distances of the boundary elements of a
!                    volume in relation to their coarsenesses
!     MSH677         Compute surface of triangle given by three nodes in 3D
!     MSHCHECKTETS   Compute the minimal ratio between radius of inner sphere
!                    and outer sphere of all tets in a volume
!     MSHGENESTIMATE Compute estimates for npointlc and nelemlc
!     MSHO86         check whether division of multihedron is possible
!                    with extra nodes
!     MSHTRANS3DSUR  Transform 3d region to region of unit length in the
!                    first octant
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     499   Internal SEPRAN error
!     923   number of points too large
!     924   number of elements too large
!     926   internal error in boundary, stop
!     927   too many repositionings required
!     928   Length help array not enough
!     951   Intersection in boundary : Fatal !
!    1091   Entering total division
!    1644   The number of extra points needed is too large
!    2059   kstap is odd
!    2905   problem while creating the mesh
!    2906   Internal error in mesh generator, jpn = 0
!    2907   Internal error in mesh generator, ielem = 0
!    2908   More than 500 redivisions in mesh generator
!    2913   Error with barycenters
!    2961   No mesh solution found in mshphil3d
!    2962   Elements of bad shape found
!    2963   Elements may have bad shape
!    2964   Error in mesh generation (general3d)
!    2965   Division not too successfull
!    2966   Local ratio problem
!    2967   Internal error
!    2976   Fixed internal surface too near to either the original boundary
!    2977   fixed points, lines and surfaces to close to each other
!    2978   Intersection Error
!    3060   Mesh in volume may be bad
!    3061   No node found in mesh generation
!    3149   Internal error in mshphil3d
! **********************************************************************
!
!                       PSEUDO CODE
!
!     While boundary triangles are available do
!       Run through list of boundary triangles
!       Try for first boundary triangle to form a tetrahedron with one of
!       the other boundary points it is connected with
!       If tetrahedron is formed go to next boundary triangle else
!       Create new point in interior (run along inside pointing
!       normal starting from barycentre of triangle)
!       Check whether this new tetrahedron is allowed
!       If not, check whether tetrahedron can be formed with one of the
!          points of the opposite side
!          If so go to next boundary triangle
!          if not try (sequential first) new boundary triangle
!       If new tetrahedron is allowed
!          Check whether new point is not too near to old point
!          If too near create tetrahedron with old point
!       If "unresolvable" triangles remain than do special actions
!          -  Compute barycenter of remaining points and form tetrahedrons
!          -  Skip region around not solvable interior and try again
!          -  Find helppoint in interior
!          -  Use all points created and try to create elements using
!             Delauney properties
!     Endwhile
!     If all elements have a positive volume than make a repositioning
!     Check resulting mesh for
!        elements with negative volume (not allowed !)
!        bad shape (for instance too small volumes or sides or angles)
!     If not allowable elements exist, remove and start division again for
!     local area around rejected elements, adjust and improve
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshphil3d' )

      debug = .false.

!     --- Info:

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshphil3d'
      end if

!     --- Set timevalue

      if ( itime >0 ) call ersettime ( timebefore )

!     --- Initialize parameters:

      numelm   =    0
      nelmfix  =    0
      itessel  =    0
      jelem    =    0
      jmin     =    0
      jmax     =    0
      vermin   =  1d0
      vermax   =  1d0
      bndmin   =  1d0
      bndmax   =  1d0
      ratiop   = -2d0
      ratold   = -1d0

!     --- Debug info:

      if ( debug ) then
         write(irefwr,*) 'Number of submitted nodes in boundaries'
         write(irefwr,*) 'is',npointlc,'with coordinates'
         do i=1, npointlc
            x1 = coor3d(1,i)
            y1 = coor3d(2,i)
            z1 = coor3d(3,i)
            write(irefwr,*) 'Node',i,'crds',x1,y1,z1
         end do
      end if

!     --- Start: replace all nodes "temporarily" to the first
!         (positive) unity-octant, with as "left-under" point (1,1,1):
!     --- Allocate space for help array usernodes:

      allocate( userco( 3, nuspnt ), stat = error )
      if ( error>0 ) then
         call eralloc ( error, 3*nuspnt, 'userco' )
         go to 1150
      end if

!     --- Check for nodes on extra curves:

      nipcrv = 0
      if ( nextracrvs>0 ) then
         do i=1, nextracrvs
            nipcrv = nipcrv + numnodextcrvs(i)
         end do
      end if

!     --- Give info:

      if ( debug ) write(irefwr,*) 'extra internal linenodes =',nipcrv

!     --- Make transformation to unitycube in first octant:

      call mshtrans3dsur( coor3d, npointlc, xmint, ymint, zmint, tran, &
                          nextrapnts, fix_pnt, nextracrvs, numnodextcrvs, &
                          coorcurvs, userco, nuspnt )
      xminvec(1) = xmint
      xminvec(2) = ymint
      xminvec(3) = zmint
      xminvec(4) = tran

!     --- Give info for all internal lines:

      if ( debug ) then

!     --- all internal lines transformed:

         write(irefwr,*) '(transformed) Nodes of Internal lines'
         do i=1,nipcrv
            x1 = coorcurvs(1,i)
            y1 = coorcurvs(2,i)
            z1 = coorcurvs(3,i)
            write(irefwr,*) 'Node',i,'crds',x1,y1,z1
         end do
         write(irefwr,*) 'After transformation: npointlc= ',npointlc
         do i=1, npointlc
            x1 = coor3d(1,i)
            y1 = coor3d(2,i)
            z1 = coor3d(3,i)
            write(irefwr,*) 'Node',i,'crds',x1,y1,z1
         end do
         write(irefwr,*) 'All these transformed nodes Ok? '
      end if

!     --- Compute some estimated lengths

      nextra = max ( 15 , nint ( sqrt( dble(nipnt)/4d0 ) + 2d0 ) )

!     --- Estimate number of points and elements

      call mshgenestimate ( nextra, npunt, nkmeshc, 2*nipnt-4 )

!     --- Estimated length of array kstapl is lenkstapl

      lenkstapl =  3 * npunt + 100

!     --- Estimated length of array line is lenline

      lenline  = 50 * nextra * nextra

!     --- Allocate space for help array line

      if ( debug ) then
         write(irefwr,*) 'Allocated length array line =',lenline
      end if

!     --- Give info:

      if ( debug ) then
         write(irefwr,*) 'Alloc length array kstapl = ',3 * lenkstapl
         write(irefwr,*) 'Two dimensions kstapl = (3,',lenkstapl,')'
      end if

!     --- Allocate space for help array kstapl

      allocate( kstapl(3,lenkstapl), line(lenline), itri(npunt), &
                stat = error )
      if ( error>0 ) then
         call eralloc ( error, 3*lenkstapl+lenline+npunt, 'kstapl' )
         go to 1150
      end if

!     --- Check for full generation or only repositioning:

      if ( .not. jnew ) goto 550

!     --- Check boundary elements (they should not intersect each other)
!         (In total submitted boundary contains nbound triangles)

      call msh480 ( kbndpt, nbound, line, lenline, coor3d, ivolnr, &
                    xminvec, 0 )
      if ( ierror/=0 ) go to 1150

!     --- Never occurring error:

      if ( nbound<0 ) then

!     --- something is terribly wrong with boundary mesh, stop

         call errint ( ivolnr, 1 )
         call errsub ( 951, 1, 0, 0 )
         go to 1150

      end if

!     --- Length of itri is always smaller than max of nodes:

      call inichkarlen ( max(nbound,npointlc,npunt), npunt, 'itri' )
      if ( ierror/=0 ) go to 1150

!     --- Check whether each line in boundarytriangle array kbndpt is double
!         and determine for each point a coarseness-value:
!     --- Before the check find number of extra triangles in kbndpt due
!         to internal surfaces:

      nextratri = 0

!     --- Determine boundary triangles extra surfaces:

      do i=1, nextrasrfs
         nextratri = nextratri + numelmextsurfs(i)
      end do

!     --- Allocate help array coarse with length npunt:

      allocate( coarse(npunt), stat = error )
      if ( error>0 ) then
         call eralloc ( error, npunt, 'coarse' )
         go to 1150
      end if

!     --- Check whether each line in kbndpt is double or not and
!         determine for each point a coarseness value

      call msh601 ( kbndpt, nbound-nextratri, itri, kstapl, coarse, &
                    coor3d, npointlc, ivolnr, lenkstapl, xminvec )
      if ( ierror/=0 ) go to 1150

!     --- Determine extreme coordinate-values for this volume

      call msh604 ( coor3d, npointlc, xmin, xmax, ymin, ymax, zmin, zmax )

!     --- Determine max and min distance for total area

      call msh605 ( coarse, npointlc, dismin, dismax )

!     --- Determine global frame for coarseness

      nx1 = nint( ((xmax-xmin) / dismax) + 0.6d0 )
      ny1 = nint( ((ymax-ymin) / dismax) + 0.6d0 )
      nz1 = nint( ((zmax-zmin) / dismax) + 0.6d0 )

!     --- Number of cubes in coarseness frame:

      ncube = nx1 * ny1 * nz1

!     --- Allocate space for cube related help arrays:

      allocate( cube(ncube), jcube(ncube), icube(npunt), stat = error)
      if ( error>0 ) then
         call eralloc (  error, 3*ncube, 'cube' )
         go to 1150
      end if

!     --- Determine for each cube-centre a dis value

      xstart = ( xmax + xmin - (nx1+0.1d0)*dismax ) / 2.d0
      ystart = ( ymax + ymin - (ny1+0.1d0)*dismax ) / 2.d0
      zstart = ( zmax + zmin - (nz1+0.1d0)*dismax ) / 2.d0

!     --- Determine coarseness value for each cube

      call msh606 ( npointlc, coor3d, dismax, xstart, ystart, zstart, &
                    nx1, ny1, nz1, icube, coarse, cube, jcube, &
                    nextrapnts, fix_pnt )

!     --- Deallocate help array coarse:

      deallocate ( coarse, stat = error )
      if ( error>0 ) then
         call erdealloc ( error, 'coarse' )
         go to 1150
      end if

!     --- Coarseness values cubes:

      mincub  = 100000d0
      maxcub  =      0d0

!     --- Determine min and max:

      do i = 1, ncube
         if (abs(cube(i))>0d0) then
            mincub = min(mincub,cube(i))
            maxcub = max(maxcub,cube(i))
         end if
      end do

!     --- Compute reference volume value for each cube
!         Allocate help array chcoar, length ncube, for reference values:

      allocate( chcoar( ncube ), stat = error )
      if ( error>0 ) then
         call eralloc ( error, ncube, 'chcoar' )
         go to 1150
      end if

!     --- Use chcoar to fill cube array

      call msh634 ( cube, ncube, chcoar, refval )

!     --- Set number of old points

      nipnt = npointlc

!     --- Clear array itri

      do i = 1, npunt
          itri(i) = 0
      end do

!     --- Place boundary elements in special array and fill itri
!         (Internal surfaces are treated later)

      kstap = nbound - nextratri

!     --- Get all triangles and place in kstapl, at the same time fill
!         indicationarray itri:

      do i = 1, kstap
         do j = 1, 3
            ipoint       = kbndpt(j,i)
            kstapl(j,i)  = ipoint
            itri(ipoint) = itri(ipoint) + 1
         end do
      end do

!     --- Check whether all elements are given right and adjust
!         array kstapl eventually

      call msh639 ( kstapl, kstap, itri )

!     --- Check declared length of kstapl:

      if ( kstap>lenkstapl ) then
         call errint ( lenkstapl,  1 )
         call errint ( kstap    ,  2 )
         call errint ( ivolnr   ,  3 )
         call errsub ( 928, 3, 0,  0 )
         go to 1150
      end if

!     --- Set number of elements:

      nelemlc  = 0

!     --- Check if needed, respectively for fixed nodes, lines and elements:

      if ( debug ) then
         write(irefwr,*) 'Contents kstapl first check'
         do i=1, kstap
            i1 = kstapl(1,i)
            i2 = kstapl(2,i)
            i3 = kstapl(3,i)
            write(irefwr,*) i,i1,i2,i3
         end do
         write(irefwr,*) 'Ok voor kstapl? '
         write(irefwr,*) 'First use msh608, next msh637'
         call msh608 ( kstapl, kstap, coor3d, xstart )
         write(irefwr,*) 'Contents kstapl second check'
         do i=1, kstap
            i1 = kstapl(1,i)
            i2 = kstapl(2,i)
            i3 = kstapl(3,i)
            write(irefwr,*) i,i1,i2,i3
         end do
         write(irefwr,*) 'Ok kstapl second check? '
         write(irefwr,*) 'Next msh637'
         call msh637 ( kstapl, kstap, line, line(kstap+11) )
         write(irefwr,*) 'After msh637 still ok?'
      end if

!     --- Check for fixed points:

      if ( nextrapnts>0 ) then

!     --- Place special nodes and fill the surrounding
!         special elements:

         call msh655 ( npointlc, coor3d   , xstart , ystart, zstart, &
                       dismax, fix_pnt, nextrapnts, icube , &
                       nx1  , ny1, kmeshcloc , nelemlc , &
                       kstapl, kstap  , itri, &
                       kbndpt, 3 * nbound  )

!        --- Adjust number of initial ("fixed") points and
!            initial ("fixed") elements

         nipnt    = npointlc
         nelmfix  = nelemlc
      end if

!     --- Check whether fixed lines are submitted

      if ( nextracrvs>0 ) then

!     --- Place special nodes and fill the surrounding
!         special elements for all internal fixed-lines:

         lkbndpt = 3 * nbound + nextrapnts
         call msh656 ( npointlc, coor3d , xstart , ystart, zstart, dismax, &
                       numnodextcrvs, curvenumbers, coorcurvs, &
                       nextracrvs, icube, nx1, ny1, kmeshcloc , nelemlc , &
                       kstapl, kstap , itri, kbndpt, lkbndpt )

!        --- Check again whether prescribed boundary is still closed

         if ( debug ) then
            write(irefwr,*) 'Contents kstapl after call msh656'
            do i=1, kstap
               i1 = kstapl(1,i)
               i2 = kstapl(2,i)
               i3 = kstapl(3,i)
               write(irefwr,*) i,i1,i2,i3
            end do
            write(irefwr,*) 'Ok voor kstapl? Check using msh637'
         end if

!        --- Check orientation of volumeboundary triangles:

         call msh608 ( kstapl, kstap, coor3d, xstart )

!        --- Check closure of boundary triangles:

         call msh637 ( kstapl, kstap, line, line(kstap+11) )

!        --- Adjust number of initial ("fixed") points and
!            initial ("fixed") elements

         nipnt    = npointlc
         nelmfix  = nelemlc
      end if

!     --- Fixed elements:

      if ( nextratri>0 ) then

!     --- Run through all special triangles in fixed surfaces that are
!         fully inside and fill up surrounding space with special
!         (tetrahedrons) elements:

         do i = nbound-nextratri+1, nbound

!        --- Determine basistriangle:

            i1 = kbndpt(1,i)
            i2 = kbndpt(2,i)
            i3 = kbndpt(3,i)
            if ( debug ) write(irefwr,*) 'Fixed elmnt',i,'=',i1,i2,i3

!           --- Find coordinates barycenter

            call msh609(coor3d,i1,i2,i3,e1,e2,e3,xm,ym,zm)

!           --- Determine coarseness needed

            alp = cube(icube(i1)) + cube(icube(i2)) + cube(icube(i3))

!           --- Create new point (use tuningvalue 0.27)

            alp = 0.27D0 * alp

!           --- Disturbfactor

            hh = 1D0 + 1D-8

!           --- Compute startcoordinates:

            xn = (xm + 1.2 * alp * e1) * hh
            yn = (ym + 1.2 * alp * e2) * hh
            zn = (zm + 1.2 * alp * e3) * hh

!           --- Check whether new point is inside volume:

            call msh607(xn,yn,zn,xstart,coor3d,kstapl,kstap,ja)

!           --- ok?

            if ( ja==1 ) then

!           --- Position ok!

               xn = (xm + alp * e1) * hh
               yn = (ym + alp * e2) * hh
               zn = (zm + alp * e3) * hh

            else if ( ja==0 ) then

!           --- Point is not allowed: try other direction

               if ( debug ) then
                  write(irefwr,*) 'Surface point element internal '
                  write(irefwr,*) 'surface not allowed. Problem?'
                  write(irefwr,*) 'Crds',xn,yn,zn
               end if

!              --- Alternative direction:

               xn = (xm - 1.2 * alp * e1) * hh
               yn = (ym - 1.2 * alp * e2) * hh
               zn = (zm - 1.2 * alp * e3) * hh

!              --- Check for this node internal:

               call msh607(xn,yn,zn,xstart,coor3d,kstapl,kstap,ja)

!              --- Check for allowed:

               if ( ja==1 ) then

!              --- Position ok!

                  xn = (xm - alp * e1) * hh
                  yn = (ym - alp * e2) * hh
                  zn = (zm - alp * e3) * hh

               else if ( ja==0 ) then
                  if ( debug ) then
                     write(irefwr,*) 'Alternative also not allowed '
                     write(irefwr,*) xn,yn,zn
                  end if
                  jsurf = 0
                  nelhelp = numelmextsurfs(1)
100               jsurf = jsurf + 1
                  if ( i-nbound+nextratri>nelhelp ) then
                     nelhelp = nelhelp + numelmextsurfs(jsurf+1)
                     goto 100
                  end if
                  jsurf = surfnumbers(jsurf)

!                 --- Error: intersections

                  call errint ( jsurf, 1 )
                  call errsub ( 2976, 1, 0, 0 )
                  go to 1150

               end if

            end if
            npointlc = npointlc + 1

!           --- Fill array coor:

            coor3d(1,npointlc) = xn
            coor3d(2,npointlc) = yn
            coor3d(3,npointlc) = zn

!           --- Fill array kelem

            nelemlc = nelemlc + 1

!           --- Element is:

            kmeshcloc( 1, nelemlc ) = i1
            kmeshcloc( 2, nelemlc ) = i2
            kmeshcloc( 3, nelemlc ) = i3
            kmeshcloc( 4, nelemlc ) = npointlc

!           --- Compute volume:

            call msh632( coor3d, i1, i2, i3, npointlc, volumb)

!           --- Adjust kstapl and itri

            if ( volumb>0 ) then
               kstap = kstap + 1
               kstapl( 1, kstap ) = i1
               kstapl( 2, kstap ) = i2
               kstapl( 3, kstap ) = npointlc

               kstap = kstap + 1
               kstapl( 1, kstap ) = i2
               kstapl( 2, kstap ) = i3
               kstapl( 3, kstap ) = npointlc

               kstap = kstap + 1
               kstapl( 1, kstap ) = i3
               kstapl( 2, kstap ) = i1
               kstapl( 3, kstap ) = npointlc

               kstap = kstap + 1
               kstapl( 1, kstap ) = i1
               kstapl( 2, kstap ) = i3
               kstapl( 3, kstap ) = i2
            else
               kmeshcloc( 1, nelemlc ) = i2
               kmeshcloc( 2, nelemlc ) = i1
               kmeshcloc( 3, nelemlc ) = i3
               kmeshcloc( 4, nelemlc ) = npointlc

               kstap = kstap + 1
               kstapl( 1, kstap ) = i2
               kstapl( 2, kstap ) = i1
               kstapl( 3, kstap ) = npointlc

               kstap = kstap + 1
               kstapl( 1, kstap ) = i3
               kstapl( 2, kstap ) = i2
               kstapl( 3, kstap ) = npointlc

               kstap = kstap + 1
               kstapl( 1, kstap ) = i1
               kstapl( 2, kstap ) = i3
               kstapl( 3, kstap ) = npointlc

               kstap = kstap + 1
               kstapl( 1, kstap ) = i1
               kstapl( 2, kstap ) = i2
               kstapl( 3, kstap ) = i3
            end if

!           --- Adjust itri:

            itri(npointlc) = itri(npointlc) + 3
            itri(i1)     = itri(i1)     + 3
            itri(i2)     = itri(i2)     + 3
            itri(i3)     = itri(i3)     + 3

!           --- Determine cube for the new point

            n1 = int ( (xn-xstart) / dismax )
            n2 = int ( (yn-ystart) / dismax )
            n3 = int ( (zn-zstart) / dismax )

!           --- Compute correct cubenumber:

            nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1

!           --- Place cubenumber:

            icube(npointlc) = nc1

!           --- Debug info:

            if ( debug ) then

!           --- Compute ratio:

               call msh661( coor3d, i1, i2, i3, npointlc, ratio )
               write(irefwr,*) 'Internal surface example element',nelemlc
               write(irefwr,*) 'standard ratio = ',ratio
            end if
         end do

!        --- More debug info concerning the nodes:

         if ( debug ) then
            write(irefwr,*) 'Coordinates nodes after positioning of'
            write(irefwr,*) 'internal surfaces'
            do i = 1, npointlc
               xn = coor3d(1,i)
               yn = coor3d(2,i)
               zn = coor3d(3,i)
               write(irefwr,*) i,xn,yn,zn
            end do
         end if

!        --- Check that all triangles are mutually disjunct:

         call msh480 ( kstapl, kstap, line, lenline, coor3d, ivolnr, &
                       xminvec, 1 )
         if ( ierror/=0 ) go to 1150

!        --- Check for intersections:

         numelm = 0
         call msh652 (kmeshcloc, nelemlc, coor3d, npointlc, numelm)
         if ( numelm>0 ) then

!        --- Error: intersection of fixed elements

            write(irefwr,*) 'Intersections due to internal surfaces!'
            write(irefwr,*) 'adjust your input'
            call errsub ( 2977, 0, 0, 0 )
            go to 1150
         end if

!        --- Set new number of elements and nodes:

         nelmfix = nelemlc
         nipnt   = npointlc
      end if

!     --- Check ordering of kstapl: all normals should point inside!

      if ( debug ) then
         write(irefwr,*) 'Check kstapl just before call to msh608'
         do i=1, kstap
            i1 = kstapl(1,i)
            i2 = kstapl(2,i)
            i3 = kstapl(3,i)
            write(irefwr,*) i,i1,i2,i3
         end do
         write(irefwr,*) 'Contents kstapl ok?'
      end if

!     --- Check all normals in boundary:

      call msh608 ( kstapl, kstap, coor3d, xstart )

!     --- Ordering of the elements:

      if ( debug ) then
         write(irefwr,*) 'Check kstapl after call to msh608'
         do i=1, kstap
            i1 = kstapl(1,i)
            i2 = kstapl(2,i)
            i3 = kstapl(3,i)
            write(irefwr,*) i,i1,i2,i3
         end do
         write(irefwr,*) 'Contents kstapl ok?'
      end if

!     --- Make final check on closure of volume
!         (all lines should appear only twice and in opposite direction)

      call inichkarlen ( 2*kstap+11, lenline, 'line' )
      if ( ierror/=0 ) go to 1150

!     --- Perform the check whether prescribed boundary is closed

      if ( debug ) write(irefwr,*) 'Is prescribed boundary closed?'
      call msh637 ( kstapl, kstap, line, line(kstap+1) )
      if ( debug ) write(irefwr,*) 'After msh637, ierror =',ierror
      if ( ierror>0 ) goto 1000

!     --- Check mutual distances of the boundary elements in relation to
!         their coarsenesses; this option is not yet used!

      ifout = 1

!     --- For the future (check mutual distances of boundary):

      if ( ifout==0 ) then
         do j = 1, kstap

!        --- Consider triangle j:

           call msh672 ( j, coor3d,npointlc,kstapl,kstap,xstart,jcheck )
           if ( jcheck>0 ) then
             ifout = ifout + 1
             write(irefwr,*) 'Triangles',j,'and',jcheck,'are too close'
             write(irefwr,*) 'This may cause problems in the generation'
             write(irefwr,*) 'Please adjust your input'
             write(irefwr,*) 'Execution goes on however:'
             write(irefwr,*) 'results may be erroneous'
           end if
         end do
         if ( ifout>0 ) then
            write(irefwr,*) 'Find better input of your mesh'
            write(irefwr,*) 'Number of wrong positions is',ifout
            go to 1150
         end if
      end if

!     --- Part for division
!         nochan : parameter to count changings without creating an element
!         iherha : parameter to count number of redivisions
!         jold   : parameter to store element number to be changed
!         jherha : parameter to count whether jold is the same in two
!                  successive steps
!         nelold : number of elements in mesh in old loop
!         nptold : number of nodes in mesh in old loop
!         istep  : number of repeatings of general division loop
!         jstep  : number of repetitions of "very special" loop

      nochan = 1
      iherha = 0
      jold   = 0
      jherha = 0
      nelold = 0
      nptold = 0
      istep  = 0
      jstep  = 0
      inegat = 0

!     --- Use kstmax for max length used array kstapl:

      kstmax = 0

!     --- Start determination of internal tetrahedrons
!         Allocate space for help array boundary kbnold:

      lnkbno = kstap
      allocate( kbnold( 3, kstap ), stat = error )
      if ( error>0 ) then
         call eralloc ( error, 3*kstap, 'kbnold' )
         go to 1150
      end if

!     --- Copy boundary array:

      kbnold(1,1:kstap) = kstapl(1,1:kstap)
      kbnold(2,1:kstap) = kstapl(2,1:kstap)
      kbnold(3,1:kstap) = kstapl(3,1:kstap)
      if ( debug ) then
         write(irefwr,*) 'The',kstap,'boundary triangles of vol=',ivolnr
         do i=1, kstap
            i1 = kstapl(1,i)
            i2 = kstapl(2,i)
            i3 = kstapl(3,i)
            write(irefwr,*) 'Triangle',i,'nodes',i1,i2,i3
            call msh677 ( coor3d, i1, i2, i3, adis)
            write(irefwr,*) 'Surface of triangle is',adis
         end do
         write(irefwr,*) 'All triangles ok?'
      end if
300   factor = 1.00d0

!     --- Always check length kstapl

      if ( kstap>kstmax ) then
         kstmax = kstap
         if ( kstap>lenkstapl ) then
            write(irefwr,*) 'kstap =',kstap
            write(irefwr,*) 'Too large arraylength needed! Stop'
            go to 1150
         end if
      end if

!     --- Part for development and trouble shooting: not for
!         external use
!         --- Part with checks for debugging, normally to be skipped:

      debug = .false.

!     --- Check boundary

      if ( debug .and. ivolnr==-16 .and. nelemlc>1089 ) then
           write(irefwr,*) 'nel and noch=',nelemlc,nochan,'to msh637'
           if ( nochan==0 ) then
              write(irefwr,*) 'new element found'
              write(irefwr,*) (kmeshcloc(j,nelemlc),j=1,4)
              write(irefwr,*) 'contents kstapl'
              do i=1, kstap
                 write(*,*) (kstapl(j,i),j=1,3)
              end do
           end if
           call msh637 ( kstapl,kstap,line,line(kstap+11) )
           if ( ierror>0 ) then
              write(irefwr,*) 'ierror=',ierror,'msh637'
              go to 1150
           end if
      end if

!     --- For checking: stop at specified element number:

      if ( nelemlc==-1 .and. nochan==0 ) then
         write(irefwr,*) 'nelemlc=',nelemlc,'and nochan =',nochan
      end if

!     --- Negative elements never allowed:

      if ( debug .and. nochan==0 .and. nelemlc>0 ) then
         i1 = kmeshcloc(1,nelemlc)
         i2 = kmeshcloc(2,nelemlc)
         i3 = kmeshcloc(3,nelemlc)
         i4 = kmeshcloc(4,nelemlc)
         call msh661( coor3d, i1, i2, i3, i4, ratio )
         call msh632( coor3d, i1, i2, i3, i4, volumb)
         if ( volumb<0 ) then
            write(irefwr,*) 'Check for element created',nelemlc
            write(irefwr,*) 'nochan = ',nochan,'kstap =',kstap
            write(irefwr,*) 'Volume =',volumb,'ratio =',ratio
            write(irefwr,*) 'knopen',i1,i2,i3,i4
            write(irefwr,*) 'crds',i1,'are',(coor3d(j,i1),j=1,3)
            write(irefwr,*) 'crds',i2,'are',(coor3d(j,i2),j=1,3)
            write(irefwr,*) 'crds',i3,'are',(coor3d(j,i3),j=1,3)
            write(irefwr,*) 'crds',i4,'are',(coor3d(j,i4),j=1,3)
            if ( i1>nipnt.and.i2>nipnt.and.i3>nipnt.and.i4>nipnt ) then
               write(irefwr,*) 'Element is fully inside: check later'
            end if

!           --- Write nodes, elements and boundary triangles

            write(irefwr,*) nelemlc,'Elements'
            do i=1, nelemlc
               write(irefwr,*) 'el',i,'nodes',(kmeshcloc(j,i),j=1,4)
            end do
            write(irefwr,*) npointlc,'Nodes'
            do i=1, npointlc
               if ( itri(i)>0 ) &
                    write(irefwr,*) 'crds',i,'are',(coor3d(j,i),j=1,3)
            end do
            write(irefwr,*) kstap,'resting boundary triangles'
            do i=1,kstap
               write(irefwr,*) 'tri',i,'with',(kstapl(j,i),j=1,3)
            end do
            write(irefwr,*) 'Routine msh637'
            call msh637 ( kstapl,kstap,line,line(kstap+11) )

!           --- Check new kstapl

            write(irefwr,*) 'and also use msh647'
            call msh647( kstapl,kstap,coor3d,kstap,1,1,3 )
            write(irefwr,*) 'After msh647 npointlc=',npointlc
            write(irefwr,*) 'Everything ok, stop'

!           --- Finally check the barycenters:

            numelm = 0
            call msh652( kmeshcloc, nelemlc, coor3d, npointlc, numelm)
            if ( numelm>0 ) then
               write(irefwr,*) 'Na msh652 numelm =',numelm
               write(irefwr,*) 'Error stop'
               go to 1150
            end if
         end if

!        --- Another check

         if ( debug .and. nochan==0 .and. volumb<0 ) then
            numelm = 0
            call msh652 (kmeshcloc, nelemlc, coor3d, npointlc, numelm)
            if ( numelm>0 ) then
               write(irefwr,*) 'fout, numelm =',numelm
               write(irefwr,*) 'stop execution'
               write(irefwr,*) 'if available use other mesh'

!              --- if possible use old mesh

               go to 1150
            end if
            write(irefwr,*) 'next check is msh637'
            call msh637 ( kstapl,kstap,line,line(kstap+11) )

!           --- Check new kstapl

            write(irefwr,*) 'and also use msh647'
            call msh647( kstapl,kstap,coor3d,kstap,1,1,3 )
            write(irefwr,*) 'After msh647 npointlc=',npointlc
            write(irefwr,*) 'Everything ok, stop'
            go to 1150
         end if
         if ( debug ) then
            write(irefwr,*) 'next check is msh637'
            call msh637 ( kstapl,kstap,line,line(kstap+11) )

!           --- Check new kstapl

            write(irefwr,*) 'and also use msh647'
            call msh647( kstapl,kstap,coor3d,kstap,1,1,3 )
            write(irefwr,*) 'After msh647 npointlc=',npointlc
            write(irefwr,*) 'Everything ok, go on for new element'
         end if
      end if
      if ( debug .and. nelemlc==-5 ) then

!     --- Check all tets (very timeconsuming!!):

         numelm = 0
         call msh652 (kmeshcloc, nelemlc, coor3d, npointlc, numelm)
         write(irefwr,*) 'Barycheck routine msh652:nelemlc=', nelemlc
         if ( numelm==0 ) then
            write(irefwr,*) 'After msh652 No errors so far'
         else
            write(irefwr,*) 'Error msh652: Value numelm =',numelm
            write(irefwr,*) 'Check your input'
            go to 1150
         end if
      end if
      if ( debug .and. nochan==0 ) then
         write(irefwr,*) 'ivolnr = ',ivolnr
         if ( 0>1 ) then
            write(irefwr,*) 'Number nodes is',npointlc
            do i=1, npointlc
               if ( itri(i)>0 ) then
                  x1 = coor3d(1,i)
                  y1 = coor3d(2,i)
                  z1 = coor3d(3,i)
                  write(irefwr,*) 'Node',i,'crds',x1,y1,z1
               end if
            end do
         end if
         write(irefwr,*) 'Last element created'
         write(irefwr,*) nelemlc,(kmeshcloc(i,nelemlc),i=1,4)
         i1 = kmeshcloc(1,nelemlc)
         i2 = kmeshcloc(2,nelemlc)
         i3 = kmeshcloc(3,nelemlc)
         i4 = kmeshcloc(4,nelemlc)
         call msh661( coor3d, i1, i2, i3, i4, ratio )
         call msh632( coor3d, i1, i2, i3, i4, volumb)
         write(irefwr,*) 'elem',nelemlc,'ratio=',ratio,volumb
         call msh480 ( kstapl, kstap, line, lenline, coor3d, ivolnr, &
                       xminvec, 0 )
         if ( ierror/=0 ) go to 1150
         write(irefwr,*) 'controle via msh652'
         numelm = 0
         call msh652 (kmeshcloc, nelemlc, coor3d, npointlc, numelm)
         if ( numelm>0 ) then
            write(irefwr,*) 'Barycheck routine msh652:nelemlc=', nelemlc
            write(irefwr,*) 'Numelm = ',numelm
            i1 = kmeshcloc(1,nelemlc)
            i2 = kmeshcloc(2,nelemlc)
            i3 = kmeshcloc(3,nelemlc)
            i4 = kmeshcloc(4,nelemlc)
            write(irefwr,*) 'Nodes',i1,i2,i3,i4
            go to 1150
         end if

!        --- Present boundary:

         write(irefwr,*) 'Temporary contents of kstapl ='
         do i = 1, kstap
            i1 = kstapl ( 1, i )
            i2 = kstapl ( 2, i )
            i3 = kstapl ( 3, i )
            write(irefwr,*) 'Newtri',i,'Nodes',i1,i2,i3
         end do
         do i = 1, npointlc
            if ( itri(i)/=0 ) then
               write(irefwr,*) 'Array itri',i,'value =',itri(i)
            end if
         end do
         write(irefwr,*) 'nelemlc is',nelemlc,'Ok?'
      end if
      if ( nelemlc>0 .and. nochan==0 .and. debug ) then
         write(irefwr,*) nelemlc,'elements found'
         do ielem = 1, nelemlc
            i1 = kmeshcloc(1,ielem)
            i2 = kmeshcloc(2,ielem)
            i3 = kmeshcloc(3,ielem)
            i4 = kmeshcloc(4,ielem)
            write(irefwr,*) 'ielem =',ielem,'elem',i1,i2,i3,i4
         end do
      end if

!     --- Boundary check:

      if ( debug .and. (nelemlc-10*(nelemlc/10)==-6) ) then
         write(irefwr,*) 'check boundary kstapl voor nelemlc= ',nelemlc
         write(irefwr,*) 'kstap = ',kstap
         call msh637 ( kstapl, kstap, line, line(kstap+11) )
         numelm = 0
         call msh652 (kmeshcloc, nelemlc, coor3d, npointlc, numelm)
         write(irefwr,*) 'Barycheck routine msh652:nelemlc=', nelemlc
         write(irefwr,*) 'Numelm = ',numelm
         if ( numelm>0 ) then
            i1 = kmeshcloc(1,nelemlc)
            i2 = kmeshcloc(2,nelemlc)
            i3 = kmeshcloc(3,nelemlc)
            i4 = kmeshcloc(4,nelemlc)
            write(irefwr,*) 'Nodes',i1,i2,i3,i4
            go to 1150
         end if
      end if
      if ( debug ) then

!     --- Option find element with specified number:

         if ( nochan==0 .and. nelemlc>0 ) then
            write(irefwr,*) 'nelemlc = ',nelemlc,'nochan = ',nochan
            write(irefwr,*) 'nodes : ',(kmeshcloc(i,nelemlc),i=1,4)
         else
            write(irefwr,*) 'nochan = ',nochan,' nelemlc = ',nelemlc
         end if
      end if

!     --- Option check created mesh sofar (very time consuming):

      if ( debug .and. (nelemlc - (nelemlc/10)*10)==-5 ) then
         call msh652 (kmeshcloc, nelemlc, coor3d, npointlc, numelm)
         if ( numelm>0 ) then
            call errsub ( 2978, 0, 0, 0 )
            go to 1150
         end if
      end if

!     --- Check after creation of each new element:

      if ( debug ) then
         if ( nochan==0 .and. nelemlc>0 ) then
            write(irefwr,*) 'Overall check for nelemlc = ',nelemlc
            numelm = 0
            call msh652 (kmeshcloc, nelemlc, coor3d, npointlc, numelm)
            write(irefwr,*) 'Barycheck routine msh652:nelemlc=', nelemlc
            write(irefwr,*) 'Numelm = ',numelm
            if ( numelm>0 ) go to 1150
         end if

!        --- Option check for special ratio in new element:

         if ( nelemlc<0 .and. nochan==0 ) then
            i1 = kmeshcloc(1,nelemlc)
            i2 = kmeshcloc(2,nelemlc)
            i3 = kmeshcloc(3,nelemlc)
            i4 = kmeshcloc(4,nelemlc)
            call msh661( coor3d, i1, i2, i3, i4, ratio )
            call msh632( coor3d, i1, i2, i3, i4, volumb)
            if ( ratio<=0 ) then
               write(irefwr,*) 'Creation bad shaped element vol=',volumb
               write(irefwr,*) 'ratio = ',ratio,' element ',nelemlc
               write(irefwr,*) 'nodes',i1,i2,i3,i4
               go to 1150
            end if
         end if

!        --- Option print last element created:

         if ( nochan==0 .and. nelemlc==-1 ) then
            write(irefwr,*) 'Elmt',nelemlc,'nodes',(kmeshcloc(i,nelemlc),i=1,4)
         end if

!        --- Option give message if no boundaryelements are left

         if ( kstap==0 ) then
           write(irefwr,*) 'kstapl empty reached'
         end if

!        --- Check barycenters of all so far created elements
!            (very time consuming!)

         if ( kstap==0 .and. nelemlc<-1 .or. &
              nochan==0 .and. nelemlc==-8254 ) then
            numelm = 0
            call msh652 (kmeshcloc, nelemlc, coor3d, npointlc, numelm)
            write(irefwr,*)'Barycentercheck msh652:nelemlc=', nelemlc
            if ( numelm>0 )then

!           --- numelm>0, error!

               call errint ( numelm, 1 )
               call errsub ( 2913, 1, 0, 0 )
            end if  ! ( numelm>0 )
         end if

!        --- Check: ratio and volume of last created element:

         if ( nelemlc>0 .and. nochan==0 ) then
            i1 = kmeshcloc(1,nelemlc)
            i2 = kmeshcloc(2,nelemlc)
            i3 = kmeshcloc(3,nelemlc)
            i4 = kmeshcloc(4,nelemlc)
            call msh661( coor3d, i1, i2, i3, i4, ratio )
            call msh632( coor3d, i1, i2, i3, i4, volumb)
            if ( ratio<0 ) then
               write(irefwr,*) 'Creation bad shaped element vol=',volumb
               write(irefwr,*) 'ratio = ',ratio,' element ',nelemlc
               write(irefwr,*) 'nodes',i1,i2,i3,i4
            end if
         end if
      end if

!     --- End special checks for SEPRAN debugging
!         Skip part until here for externals
!         --- Determination of new elements:
!         Check value of kstap: it should always be even:

      if ( mod(kstap,2)/=0 ) then

!     --- internal error, kstap is, but should never be, odd

         call errint ( kstap , 1 )
         call errint ( npointlc, 2 )
         call errint ( nelemlc , 3 )
         call errint ( ivolnr, 4 )
         call errsub ( 2059, 4, 0, 0 )
         go to 1150
      end if

!     --- Change values in case of no convergence:

      if ( nochan>kstap ) then
         factor = 0.6d0
      end if

!     --- Check number of nodes:

      if ( npointlc>npunt ) then
         call errint ( npunt,1 )
         call errint ( npointlc,2 )
         call errint ( ivolnr, 3 )
         call errsub ( 923, 3, 0, 0 )
      end if

!     --- Check number of elements:

      if ( nelemlc>nkmeshc ) then
         call errint ( nkmeshc,1 )
         call errint ( nelemlc,2 )
         call errint ( ivolnr, 3 )
         call errsub ( 924, 3, 0, 0 )
      end if
      leng = kstap

!     --- Check length of kstapl:

      if ( leng>lenkstapl ) then
         call errint ( lenkstapl,1 )
         call errint ( leng  ,2 )
         call errint ( ivolnr, 3 )
         call errsub ( 928, 3, 0, 0 )
      end if

!     --- If error => Stop!

      if ( ierror/=0 ) go to 1150

!     --- Check situation : no direct division possible ?

      if ( nochan>2*kstap + 1 ) then

!     --- Several possibilities: assume that finally a small body
!         for which it is impossible to find a division in tetrahedrons
!         is left; In case of debugging print the characteristic values

         if ( debug ) then
            write(irefwr,*) 'iherha = ',iherha,'nochan = ',nochan
            write(irefwr,*) 'nelemlc  = ',nelemlc, 'npointlc = ',npointlc
            write(irefwr,*) 'Problem with',kstap,' triangles'
            do i=1,kstap
               i1 = kstapl(1,i)
               i2 = kstapl(2,i)
               i3 = kstapl(3,i)
               write(irefwr,*) 'Tri',i,'-',i1,i2,i3
            end do
            do i=1,npointlc
               if ( itri(i)>0 ) then
                  write(irefwr,*) 'node',i,'coords',(coor3d(j,i),j=1,3)
                  write(irefwr,*) 'node',i,'has',itri(i),'neighbours'
               end if
            end do
         end if

!        --- Eventually check boundary and barycenters: be sure that
!            the starting conditions for this final step are ok:

         if ( debug ) then
            write(irefwr,*) 'Problematic division: final tesselation'
            write(irefwr,*) 'values nochan and kstap:',nochan,kstap
            write(irefwr,*) 'nipnt  = ',nipnt
            write(irefwr,*) 'npointlc = ',npointlc
            write(irefwr,*) 'nelemlc  = ',nelemlc
            write(irefwr,*) 'Check kstapl using msh637'

!           --- Check array kstapl:

            call msh637 ( kstapl, kstap, line, line(kstap+11) )
            write(irefwr,*) 'After msh637: Check the elements'
            numelm = 0
            call msh652( kmeshcloc, nelemlc, coor3d, npointlc, numelm )
            if (numelm>0 ) then
               i1 = kmeshcloc(1,numelm)
               i2 = kmeshcloc(2,numelm)
               i3 = kmeshcloc(3,numelm)
               i4 = kmeshcloc(4,numelm)

!              --- Error: barycentre is inside other element

               write(irefwr,*) 'nodes',i1,i2,i3,i4
            else
               write(irefwr,*) 'After msh652: No problem so far'
            end if
         end if

!        --- Construct a new internal node:

         if ( debug ) then
            write(irefwr,*) 'No node found: use tri-construction'
            write(irefwr,*) 'npointlc =',npointlc
            write(irefwr,*) 'nelemlc  =',nelemlc
            write(irefwr,*) 'kstap  =',kstap
            write(irefwr,*) 'ivolnr =',ivolnr
         end if
         j  = 0
         ja = 0

!        --- Run through all boundary triangles untill an inside node
!            is found:(Place that node and if the new tet is allowed
!            return to the main division strategy
!            (else if not succesfull finally try tesselation strategy)

         do while ( j<kstap .and. ja==0 )

!        --- Consider triangle with number j:

            j  = j + 1

!           --- Compute nodes:

            i1 = kstapl(1,1)
            i2 = kstapl(2,1)
            i3 = kstapl(3,1)

!           --- Try to create an extra internal point for this triangle

            if ( debug ) then
               call msh677( coor3d, i1, i2, i3, adis )
               write(irefwr,*) 'j=',j,'triangle',i1,i2,i3,'surf=',adis
            end if

!           --- Determine unit normal and startpoint of line
!               using triangle i1 - i2 - i3

            call msh609(coor3d,i1,i2,i3,e1,e2,e3,xm,ym,zm)

!           --- x-coordinates

            x1 = coor3d(1,i1)
            x2 = coor3d(1,i2)
            x3 = coor3d(1,i3)

!           --- y-coordinates

            y1 = coor3d(2,i1)
            y2 = coor3d(2,i2)
            y3 = coor3d(2,i3)

!           --- z-coordinates

            z1 = coor3d(3,i1)
            z2 = coor3d(3,i2)
            z3 = coor3d(3,i3)

!           --- differences

            dx   = x1 - x2
            dy   = y1 - y2
            dz   = z1 - z2

!           --- Euclidian distance

            dis  = sqrt(dx*dx + dy*dy + dz*dz)

!           --- Compute barycentre of this triangle:

            xz = ( x1 + x2 + x3 ) / 3d0
            yz = ( y1 + y2 + y3 ) / 3d0
            zz = ( z1 + z2 + z3 ) / 3d0

!           --- Find coordinates startpoint:

            xm = xz + 1d-5 * dis * e1
            ym = yz + 1d-5 * dis * e2
            zm = zz + 1d-5 * dis * e3

!           --- Check whether point 'm' belongs to volume

            call msh607(xm,ym,zm,xstart,coor3d,kstapl,kstap,ja)
            if ( ja==1 ) then
               if ( debug ) &
                  write(irefwr,*) 'Helppoint of triangle',j,'internal'
            else
               if ( debug ) &
                  write(irefwr,*) 'Helppoint triangle',j,'not internal'

!              --- Prepare for next boundary triangle

               itri(i1) = itri(i1) - 1
               itri(i2) = itri(i2) - 1
               itri(i3) = itri(i3) - 1
               call msh633(kstapl,kstap,itri)
            end if
         end do

!        --- End of do-loop: either an inside node has been found (ja==1)
!            or for non of the boundary triangles a node is found (ja==0)
!            If ja==0 there is an accuracy problem! In that case
!            go, as kind of emergency to last possibility i.e. Delaunay.

         if ( ja==0 ) then
            if ( debug ) then
               write(irefwr,*) 'Accuracy problems in mshphil3d'
               write(irefwr,*) 'Execution goes on using Delaunay'
               write(irefwr,*) 'Please inform SEPRA'
               write(irefwr,*) 'kstap =',kstap,'nelemlc =',nelemlc
            end if
            itessel = 0
            goto 320
         end if

!        --- Point found: find coordinates "endpoint" line:

         xn = xz + 0.5 * dis * e1
         yn = yz + 0.5 * dis * e2
         zn = zz + 0.5 * dis * e3

!        --- Check whether new point is also inside:

         if ( debug ) then
            write(irefwr,*) 'Extra trick emergency'
            write(irefwr,*) 'Startpnt =',xm,ym,zm
            write(irefwr,*) 'Endpoint =',xn,yn,zn
            write(irefwr,*) 'For triangle',i1,i2,i3
         end if
         ja = 0
         call msh607(xn,yn,zn,xstart,coor3d,kstapl,kstap,ja)
         if ( debug .and. ja==1 ) then
            write(irefwr,*) 'Endpoint also inside!'
         end if
         th = 1.1d0

!        --- Run through all remaining boundary triangles:

         if ( debug ) write(irefwr,*) 'Remaining',kstap,'triangles'
         do i = 1, kstap
            ia = kstapl ( 1, i )
            ib = kstapl ( 2, i )
            ic = kstapl ( 3, i )
            if ( debug ) &
               write(irefwr,*) 'Check BndTri',i,'Nodes',ia,ib,ic

!           --- x-coordinates:

            x1 = coor3d(1,ia)
            x2 = coor3d(1,ib)
            x3 = coor3d(1,ic)

!           --- y-coordinates:

            y1 = coor3d(2,ia)
            y2 = coor3d(2,ib)
            y3 = coor3d(2,ic)

!           --- z-coordinates:

            z1 = coor3d(3,ia)
            z2 = coor3d(3,ib)
            z3 = coor3d(3,ic)

!           --- Common point with line?

            call msh610(xn, yn, zn, xm, ym, zm, x1, y1, z1, &
                        x2, y2, z2, x3, y3, z3, t)
            if ( t>=0d0 .and. t<=1d0 ) then

!           --- Intersection with this triangle:

               if ( t<1d-5 .or. abs(t-th)<1d-3) then

!              --- "Double" intersection: skip:

                  if ( debug ) write(irefwr,*) 'Double!'
                  ja = 0
                  th = 0d0
               else if ( t<th ) then

!              --- New and better value

                  th = t
               else
                  if ( debug ) then
                     write(irefwr,*) 'Intersection',ia,ib,ic,'th=',th
                  end if
               end if
            end if
         end do

!        --- At least one of the triangles should have theoretically an
!            intersection point with the line n - m:

         if ( ja==1 .and. th>1d0 ) then

!        --- ja==1: endpoint is also inside

            if ( debug ) write(irefwr,*) 'Inside and th=',th
            th = 0.98d0
         else if ( th<=0 .or. th>1d0 .and. ja==0 ) then
            call errint ( ivolnr, 1 )
            call errwar ( 3060, 1, 0, 0 )

!           --- Give coordinates:

            if ( debug ) then
               do i = 1, npointlc
                  if ( itri(i)/=0 ) then
                     x1 = coor3d(1,i)
                     y1 = coor3d(2,i)
                     z1 = coor3d(3,i)
                     write(irefwr,*) i,itri(i),x1,y1,z1
                  end if
               end do
            end if

!           --- If already a mesh has been created: use that one
!               taking the emergency exit

            if ( ratiop>0 ) then
                 vermin = 0.5 * ratiop
                 goto 1050
            end if

!           --- No meshresult at all:

            call errint ( ivolnr, 1 )
            call errwar ( 3061, 1, 0, 0 )
            itessel = 0
            goto 320
         end if

!        --- Place new point "halfway"

         if ( debug ) &
            write(irefwr,*) 'Place new point halfway, th=',th
         ja = 0
         if ( th>0 .and. th<1d0 ) then

!        --- Check this new point:

            xn = xz + th * .49d0 * dis * e1
            yn = yz + th * .49d0 * dis * e2
            zn = zz + th * .49d0 * dis * e3

!           --- Check:

            call msh607(xn,yn,zn,xstart,coor3d,kstapl,kstap,ja)
            if ( debug ) then
               write(irefwr,*) 'Check, whether new node is inside'
               if ( ja==1 ) then
                  write(irefwr,*) 'Yes, new node is inside'
               else
                  write(irefwr,*) 'No, new node not inside! Emergency!'
               end if
            end if
         else
            write(irefwr,*) 'Error in Emergency! Stop!'
            write(irefwr,*) 'Strange situation. Please inform SEPRA'
            write(irefwr,*) 'Leave mshphil3d'
            go to 1150
         end if
         if ( ja==1 ) then

!        --- Normal situation: new node is inside

            coor3d(1,npointlc+1) = xn
            coor3d(2,npointlc+1) = yn
            coor3d(3,npointlc+1) = zn

!           --- Compute ratio and volume of new(?) elmnt:

            call msh661( coor3d, i1, i2, i3, npointlc+1, ratio  )
            call msh632( coor3d, i1, i2, i3, npointlc+1, volume )
            if ( debug ) then
               write(irefwr,*) 'New node seems Ok'
               write(irefwr,*) 'Nodes',npointlc+1,i1,i2,i3
               write(irefwr,*) 'ratio=',ratio,'volume=',volume
            end if

!           --- Check new tetrahedron for savety:

            call msh631( kstapl, kstap, line, lenline, coor3d, xn, yn, &
                         zn, npointlc+1, itri, npointlc+1, jcheck )
            if ( jcheck>0 ) then

!           --- Non expected situation

               call errint ( jcheck, 1 )
               call errsub ( 3149, 1, 0, 0 )
               go to 1150

            else if ( jcheck==0 ) then

!           --- New tetrahedron is i1,i2,i3,npointlc+1 ;
!               fill arrays coor, kmeshcloc and adjust kstapl and itri

               if ( debug ) then
                  write(irefwr,*) 'Check kstapl, kstap=',kstap
                  do i = 1, kstap
                     ia = kstapl ( 1, i )
                     ib = kstapl ( 2, i )
                     ic = kstapl ( 3, i )
                     write(irefwr,*) i,'Nodes',ia,ib,ic
                  end do

!                 --- Remaining boundary nodes:

                  do i = 1, npointlc
                     if ( itri(i)/=0 ) then
                        write(irefwr,*) 'itri',i,'value =',itri(i)
                     end if
                  end do
               end if

!              --- Adjust itri values:

               itri(i1) = itri(i1) - 1
               itri(i2) = itri(i2) - 1
               itri(i3) = itri(i3) - 1
               if ( debug ) write(irefwr,*) 'msh618',i1,i2,i3,npointlc+1
               call msh618( coor3d, npointlc, kmeshcloc, nelemlc, kstapl, &
                            kstap, itri, i1, i2, i3, xn, yn, zn )

!              --- Check boundary:

               if ( debug ) then

!              --- Show new kstapl

                  write(irefwr,*) 'After halfway solution: nelemlc=',nelemlc
                  write(irefwr,*) 'new kstapl for kstap =',kstap
                  do i = 1, kstap
                     ia = kstapl ( 1, i )
                     ib = kstapl ( 2, i )
                     ic = kstapl ( 3, i )
                     write(irefwr,*) i,'Nodes',ia,ib,ic
                  end do

!                 --- Remaining boundary nodes:

                  do i = 1, npointlc
                     if ( itri(i)/=0 ) then
                        write(irefwr,*) 'itri',i,'value =',itri(i)
                     end if
                  end do
                  call msh637 ( kstapl,kstap,line,line(kstap+11) )

!                 --- Check new kstapl

                  call msh647( kstapl,kstap,coor3d,kstap,1,1,3 )
                  write(irefwr,*) 'After msh647 npointlc=',npointlc
                  write(irefwr,*) 'Everything Ok?'
               end if
               if ( debug ) then

!              --- Check this result for barycenters:

                  write(irefwr,*) 'Temporary: controle via msh652'
                  write(irefwr,*) 'Last element created is',nelemlc
                  numelm = 0
                  call msh652 (kmeshcloc, nelemlc, coor3d, npointlc, numelm)
                  write(irefwr,*) 'Barycheck msh652:nelemlc=', nelemlc
                  write(irefwr,*) 'Numelm = ',numelm
                  if ( numelm>0 ) then
                     i1 = kmeshcloc(1,numelm)
                     i2 = kmeshcloc(2,numelm)
                     i3 = kmeshcloc(3,numelm)
                     i4 = kmeshcloc(4,numelm)
                     write(irefwr,*) 'Nodes',i1,i2,i3,i4
                     write(irefwr,*) 'Stop execution'
                     go to 1150
                  end if
               end if
               nochan  = 0

!              --- Determine cube for new point

               if ( debug ) then
                  write(irefwr,*) 'Coordinates new point',npointlc
                  write(irefwr,*) xn,yn,zn
               end if
               n1 = int ( (xn-xstart) / dismax )
               n2 = int ( (yn-ystart) / dismax )
               n3 = int ( (zn-zstart) / dismax )
               nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1
               icube(npointlc) = nc1

!              --- Go on with normal execution

               goto 300
            end if
         end if
         if ( jcheck==-1 ) then

!        --- Try Delaunay tesselation:

            itri(i1) = itri(i1) - 1
            itri(i2) = itri(i2) - 1
            itri(i3) = itri(i3) - 1
            call msh633(kstapl,kstap,itri)
            if ( debug ) then
               write(irefwr,*) 'Try Delaunay tesselation in mshphil3d'
            end if
         end if

!        --- Still no solution found!

320      if ( debug ) then
            write(irefwr,*) 'Warning:itessel =',itessel
            write(irefwr,*) 'still no solution'
         end if
         itessel = itessel + 1
         if ( itessel>10 ) then
            if ( ratiop>0 ) then
               vermin = 0.5 * ratiop
               goto 1050
            else
               write(irefwr,*) 'No solution found, and itessel>10'
               write(irefwr,*) 'Execution stopped, leave mshphil3d'
               write(irefwr,*) 'If your input is OK, inform SEPRA'
               go to 1150
            end if
         end if
         if ( debug ) then
            write(irefwr,*) 'Not expected Complex situation, (msh650)'
            write(irefwr,*) 'Please inform SEPRA'
         end if

!        --- Complex situation, try fully different method
!            Cancel all elements connected to kstap but keep points

         call msh650 ( kmeshcloc, nelemlc, nelmfix, kstap, kstapl, &
                       npointlc, coor3d, itri )

!        --- Check number of elements (not too large?):

         if ( nelemlc>nkmeshc ) then
            call errint ( nkmeshc,1 )
            call errint ( nelemlc,2 )
            call errint ( ivolnr, 3 )
            call errsub ( 924, 3, 0, 0 )
         end if

!        --- remove the nodes that are not used and renumber the
!            remaining points accordingly in kmeshcloc, kstapl, coor
!            and icube

         call msh649 ( kmeshcloc, nelemlc, kstap, kstapl, coor3d, npointlc, &
                       nipnt, itri, icube, nx1, ny1, &
                       xstart, ystart, zstart, dismax )
         call inichkarlen ( npointlc, npunt, 'icube' )
         if ( ierror/=0 ) go to 1150
         goto 360

!        --- Cancel all elements connected to kstap but keep points

350      call msh650( kmeshcloc, nelemlc, nelmfix, kstap, kstapl, npointlc, &
                      coor3d, itri)
360      jstep = 1

!        --- Present

         leng = kstap

!        --- Check length of kstapl:

         if ( leng>lenkstapl ) then
            call errint ( lenkstapl, 1 )
            call errint ( leng     , 2 )
            call errint ( ivolnr   , 3 )
            call errsub ( 928, 3, 0, 0 )
            go to 1150
         end if

!        --- Make division using the 3D tesselation property
!            as long as kstap is not zero, add new points
!            if necessary

         nexpnt = 0

!        --- Tesselation is very time consuming:

         nochan = 0

!        --- If iherha not too large: call to msh476

         if ( iherha>10 .and. ratiop>0 ) then

!        --- Escape and use "old" mesh

            vermin = 0.5 * ratiop
            goto 1050
         else if ( iherha>50 ) then

!        --- Skip, error: no division has been found.

            call errint ( iherha, 1 )
            call errint ( kstap, 2 )
            call errsub ( 2905, 2, 0, 0 )
            go to 1150
         end if

!        --- Try to find solution by tesselation:

         if ( debug ) then
            write(irefwr,*) 'msh476, 670: Tesselation,kstap = ',kstap
            write(irefwr,*) 'and npointlc =',npointlc,'nelemlc=',nelemlc
         end if
  370    if ( kstap/=0 ) then
            ntotal = npointlc + nexpnt
            if ( debug ) then

!           --- Check all elements

               call msh652( kmeshcloc, nelemlc, coor3d, npointlc, numelm )
            end if

!           --- Check call to Delaunay routine:
!               Use either msh670 or msh476

            if ( ntotal<0 ) then
               if ( debug ) write(irefwr,*) 'call msh670'
               call msh670 ( kstap, kstapl, coor3d, ntotal, &
                             nelemlc, kmeshcloc, itri, line, &
                             lenline, nipnt , xstart, &
                             xn, yn, zn, ivolnr, xminvec )
            else
               if ( debug ) write(irefwr,*) 'call msh476'
               call msh476 ( kstap, kstapl, coor3d, ntotal, &
                             nelemlc, kmeshcloc, itri, &
                             lenline, nipnt, ivolnr, xminvec )
            end if

            if ( ierror/=0 ) go to 1150

!           --- set check variable numelm

            if ( kstap<0 ) then

!           --- the boundary is not correct, throw it out

               kstap = -kstap
               goto 350
            else if ( kstap==0 ) then
               goto 300
            else if ( kstap>0 ) then

!           --- This is obsolete nexpnt can never be more then 1
!               however keep it, part may be needed later

               if ( nexpnt>4 ) then

!              --- Error, even with five points added, no
!                  solution possible, give message
!                  and stop execution

                  call errint ( nexpnt, 1 )
                  call errsub ( 1644, 1, 0, 0 )
                  go to 1150
               end if

!              --- Remove the nodes that are not used

               call msh649 ( kmeshcloc, nelemlc, kstap, kstapl, coor3d, &
                             npointlc, nipnt, itri, icube, nx1, ny1, &
                             xstart, ystart, zstart, dismax )

!              --- Add internal node if kstap>4

               if ( kstap>4 ) call msho86( kstap , kstapl, coor3d, &
                                           npointlc, nexpnt, itri, 0 )
               npointlc = npointlc + nexpnt

!              --- Try again

               goto 350
            end if

!           --- This is obsolete nexpnt can never be more than 1
!               however keep it may be needed later

            if ( nexpnt>8 ) then

!           --- Warning:

               call errwar ( 1091, 0, 0, 0 )

!              --- Try total division
!                  Clear array itri

               do i=1,npunt
                  itri(i) = 0
               end do

!              --- Place original boundary elements in special array and
!                  fill itri

               kstap = nbound
               do i = 1, nbound
                  do j = 1, 3
                     ipoint       = kbndpt(j,i)
                     kstapl(j,i)  = ipoint
                     itri(ipoint) = itri(ipoint) + 1
                  end do
               end do

!              --- Check whether all elements are given right and adjust
!                  array kstapl

               call msh639( kstapl, kstap, itri )
               nelemlc = 0
            end if
            if ( itessel<14 ) goto 370

!           --- In total nexpnt points added

            npointlc = npointlc + nexpnt

!           --- To many local tesselations: try instead of a local
!               a total tesselation and use original boundary array:

            kstap = nbound

!           --- Place old boundary again in kstapl

            do i=1, nbound
               do j=1, 3
                  kstapl(j,i) = kbndpt(j,i)
               end do
            end do

!           --- Start with zero elements

            nelemlc = 0

!           --- Clear array itri:

            do i=1, npointlc
               itri(i) = 0
            end do

!           --- Call Delaunay routine: (Use msh476 or msh670)

            if ( npointlc<0 ) then
               if ( debug ) write(irefwr,*) 'msh670 total volume'
               call msh670 ( kstap, kstapl, coor3d, npointlc, &
                             nelemlc, kmeshcloc, itri, line, &
                             lenline, nipnt , xstart, &
                             xn, yn, zn, ivolnr, xminvec )
            else
               if ( debug ) write(irefwr,*) 'msh476 total volume'
               call msh476 ( kstap, kstapl, coor3d, npointlc, &
                             nelemlc, kmeshcloc, itri, &
                             lenline, nipnt, ivolnr, xminvec )
            end if
            nochan = 0
            if ( ierror/=0 ) go to 1150

!           --- Go on in original setting

         else
            call errint ( ivolnr, 1 )
            call errsub ( 2961, 1, 0, 0 )
            go to 1150
         end if
      end if

!     --- "Normal" situation: create new element, augment nochan:

      nochan = nochan + 1

!     --- Check for ready

      if ( kstap==0 ) goto 500

!     --- Transform using boundary triangles in tetrahedrons by the advancing
!         front method, run through kstapl

      i1 = kstapl(1,1)
      i2 = kstapl(2,1)
      i3 = kstapl(3,1)

!     --- Change array itri

      itri(i1) = itri(i1) - 1
      itri(i2) = itri(i2) - 1
      itri(i3) = itri(i3) - 1

!     --- First check whether tetrahedron can be formed with other
!         boundary triangles and compute "helpangles"

      call msh615( coor3d, kstapl, kstap, i1, i2, i3, iex1, iex2, iex3, &
                   angle1, angle2, angle3 )

!     --- Check whether all neighbours have been found:

      if ( iex1==0 .or. iex2==0 .or. iex3==0 ) then

!     --- iex1==0 or iex2==0 or iex3==0: error, stop

         call errsub ( 499, 0, 0, 0 )
         go to 1150
      end if

!     --- Check and, if necessary adjust, the three help angles:

      j = 0
      call msh661( coor3d, i1, i2, i3, iex1, ratio )
      if ( ratio<0 ) then
         angle1=-1d0
      else if ( nochan>kstap .and. ratio>0 .and. ratio<0.01d0 ) then

!     --- Try eventually a sliver and a remake using msh665:

         j = iex1
      end if
      call msh661( coor3d, i1, i2, i3, iex2, ratio )
      if ( ratio<0 ) then
         angle2=-1d0
      else if ( nochan>kstap .and. ratio>0 .and. ratio<0.01d0 ) then

!     --- Try eventually a sliver and a remake using msh665:

         j = iex2
      end if
      call msh661( coor3d, i1, i2, i3, iex3, ratio )
      if ( ratio<0 ) then
         angle3=-1d0
      else if ( nochan>kstap .and. ratio>0 .and. ratio<0.01d0 ) then

!     --- Try eventually a sliver and a remake using msh665:

         j = iex3
      end if
      if ( nochan<kstap ) then
         j=0
      else if ( j>0 ) then

!     --- Check whether triangle i1-i2-i3 is an original boundary triangle
!         and if so skip:

         do i=1, lnkbno
            if ( i1==kbnold(1,i) .and. i2==kbnold(2,i) .and. &
                 i3==kbnold(3,i) .or. &
                 i2==kbnold(1,i) .and. i3==kbnold(2,i) .and. &
                 i1==kbnold(3,i) .or. &
                 i3==kbnold(1,i) .and. i1==kbnold(2,i) .and. &
                 i2==kbnold(3,i) ) then
               j = 0
            end if
         end do
      end if

!     --- Check neighbours:

      iex = 0

!     --- Check for forced situation (not at start!)

      if ( iex1==iex2 .and. iex1==iex3 .and. iex1>nipnt ) then

!     --- Forced situation: no choice left

         call msh661(coor3d,i1,i2,i3,iex1,ratio)
         if ( debug .and. ratio<0.1d0 ) then
            write(irefwr,*) nelemlc+1,'Forced element, ratio = ',ratio
            write(irefwr,*) 'nodes are',i1,i2,i3,iex1
         end if
         call msh612(kmeshcloc,nelemlc,i1,i2,i3,iex1,kstapl,kstap,itri)
         nochan = 0

!        --- First check whether element can be removed:

         if ( ratio<1D-1 ) then

!        --- Try to skip element or to make a redivision

            jelem = nelemlc

!           --- Set number of nodes: np

            np = npointlc
            call msh665(kmeshcloc,nelemlc,nelmfix,jelem,ja,coor3d,npointlc)
            if ( np/=npointlc ) then

!           --- Node removed, refill icube:

               do j = 1, npointlc
                  n1 = int ( (coor3d(1,j)-xstart)/dismax )
                  n2 = int ( (coor3d(2,j)-ystart)/dismax )
                  n3 = int ( (coor3d(3,j)-zstart)/dismax )

                  nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1
                  icube(j) = nc1
               end do
            end if
         end if

!        --- Accepted: Return to 300 for next triangle

         goto 300
      end if

!     --- Use value 0.25: Tuning parameter

      if ( iex1==iex2 .and. angle1>0.25d0 .and. angle2>0.25d0) &
           iex = iex1
      if ( iex2==iex3 .and. angle2>0.25d0 .and. angle3>0.25d0) &
           iex = iex2
      if ( iex3==iex1 .and. angle3>0.25d0 .and. angle1>0.25d0) &
           iex = iex3

!     --- Two neighbours, pos volume and good ratio?

      if ( iex>0 ) then
         call msh661( coor3d, i1, i2, i3, iex, ratio )

!        --- Check ratio (should always be positive)

         if ( ratio>0.3d0 ) then

!        --- Check visibility of iex

            call msh631( kstapl, kstap, line, lenline, coor3d, &
                         xn, yn, zn, iex, itri, npointlc, jcheck )
            if ( jcheck==0 ) then
               call msh612(kmeshcloc,nelemlc,i1,i2,i3,iex,kstapl,kstap,itri)
               nochan = 0
               if ( debug .and. iherha==0 ) &
                  write(irefwr,*) nelemlc,'via iex, ratio = ',ratio
               goto 300
            end if
         end if
      end if

!     --- angle1,2 and 3: tuning values.

      if ( angle1> -0.1d0 ) then

!     --- Check tet: i1 - i2 - i3 - iex1 ?

         xhalf = ( coor3d(1,i1) + coor3d(1,i2) + coor3d(1,i3) + &
                   coor3d(1,iex1) ) / 4d0
         yhalf = ( coor3d(2,i1) + coor3d(2,i2) + coor3d(2,i3) + &
                   coor3d(2,iex1) ) / 4d0
         zhalf = ( coor3d(3,i1) + coor3d(3,i2) + coor3d(3,i3) + &
                   coor3d(3,iex1) ) / 4d0
         call msh607(xhalf,yhalf,zhalf,xstart,coor3d,kstapl,kstap,jas)
         if ( jas==1 ) then

!        --- New tetrahedron check (ratio and) volume:

            call msh632(coor3d,i1,i2,i3,iex1,volume)
            if ( volume>0 ) then

!           --- ratio is checked later:

               call msh631(kstapl,kstap,line,lenline,coor3d,xn,yn,zn, &
                           iex1,itri,npointlc,jcheck)
               if ( jcheck==0 ) then

!              --- New element:

                  call msh612(kmeshcloc,nelemlc,i1,i2,i3,iex1,kstapl, &
                              kstap,itri)
                  nochan = 0
                  if ( debug .and. iherha==0 ) then
                     call msh661(coor3d,i1,i2,i3,iex1,ratio )
                     write(irefwr,*) nelemlc,'angle1 ratio = ',ratio
                  end if
                  goto 300
               end if
            end if
         end if
      end if
      if ( angle2>-0.1 ) then

!     --- Check tet: i1 - i2 - i3 - iex2 ?
!         Permitted ?

         xhalf = ( coor3d(1,i1) + coor3d(1,i2) + coor3d(1,i3) + &
                   coor3d(1,iex2) ) / 4d0
         yhalf = ( coor3d(2,i1) + coor3d(2,i2) + coor3d(2,i3) + &
                   coor3d(2,iex2) ) / 4d0
         zhalf = ( coor3d(3,i1) + coor3d(3,i2) + coor3d(3,i3) + &
                   coor3d(3,iex2) ) / 4d0
         call msh607(xhalf,yhalf,zhalf,xstart,coor3d,kstapl,kstap,jas)
         if ( jas==1 ) then

!        --- New tetrahedron, check (ratio and) volume:

            call msh632(coor3d,i1,i2,i3,iex2,volume)
            if ( volume>0 ) then

!           --- Ratio is checked later

               call msh631(kstapl,kstap,line,lenline,coor3d,xn,yn,zn, &
                           iex2,itri,npointlc,jcheck)
               if ( jcheck==0 ) then

!              --- New element:

                  call msh612(kmeshcloc,nelemlc,i1,i2,i3,iex2,kstapl, &
                              kstap,itri)
                  nochan = 0
                  if ( debug .and. iherha==0 ) then
                     call msh661(coor3d,i1,i2,i3,iex2,ratio )
                     write(irefwr,*) nelemlc,'angle2 ratio = ',ratio
                  end if
                  goto 300
               end if
            end if
         end if
      end if
      if ( angle3>-0.1 ) then

!     --- Check: i1 - i2 - i3 - iex3 ?
!         Permitted ?

         xhalf = ( coor3d(1,i1) + coor3d(1,i2) + coor3d(1,i3) + &
                   coor3d(1,iex3) ) / 4d0
         yhalf = ( coor3d(2,i1) + coor3d(2,i2) + coor3d(2,i3) + &
                   coor3d(2,iex3) ) / 4d0
         zhalf = ( coor3d(3,i1) + coor3d(3,i2) + coor3d(3,i3) + &
                   coor3d(3,iex3) ) / 4d0
         call msh607(xhalf,yhalf,zhalf,xstart,coor3d,kstapl,kstap,jas)
         if ( jas==1 ) then

!        --- New tetrahedron, check (ratio and) volume:

            call msh632(coor3d,i1,i2,i3,iex3,volume)
            if ( volume>0 ) then

!           --- Ratio is checked later:

               call msh631(kstapl,kstap,line,lenline,coor3d,xn,yn,zn, &
                           iex3,itri,npointlc,jcheck)
               if ( jcheck==0 ) then
                  call msh612(kmeshcloc,nelemlc,i1,i2,i3,iex3,kstapl, &
                              kstap,itri)
                  nochan = 0
                  if ( debug .and. iherha==0 ) then
                     call msh661(coor3d,i1,i2,i3,iex3,ratio )
                     write(irefwr,*) nelemlc,'angle3 ratio = ',ratio
                  end if
                  goto 300
               end if
            end if
         end if
      end if

!     --- Special situation?

      if ( j>0 ) then

!     --- Create element: i1-i2-i3-j

         call msh661( coor3d, i1, i2, i3, j, ratio )
         if ( debug ) write(irefwr,*) 'Special nel=',nelemlc,'rat=',ratio

!        --- Check position of nodes:

         call msh677( coor3d, i1, i2, j, adis )
         call msh677( coor3d, i2, i3, j, bdis )
         call msh677( coor3d, i3, i1, j, cdis )

!        --- Check barycenter tet: i1 - i2 - i3 - j

         xhalf = ( coor3d(1,i1) + coor3d(1,i2) + coor3d(1,i3) + &
                   coor3d(1,j) ) / 4d0
         yhalf = ( coor3d(2,i1) + coor3d(2,i2) + coor3d(2,i3) + &
                   coor3d(2,j) ) / 4d0
         zhalf = ( coor3d(3,i1) + coor3d(3,i2) + coor3d(3,i3) + &
                   coor3d(3,j) ) / 4d0
         call msh607(xhalf,yhalf,zhalf,xstart,coor3d,kstapl,kstap,jas)
         if ( adis>1d-8 .and. bdis>1d-8 .and. cdis>1d-8 .and. &
              jas==1) then
            call msh612(kmeshcloc,nelemlc,i1,i2,i3,j,kstapl,kstap,itri)
            if ( debug ) write(irefwr,*) 'Elmnt',i1,i2,i3,j,'created'
            nochan = 0

!           --- Check whether element can be replaced using neighbours:

            if ( ratio<0.1d0 ) then
               if ( debug ) &
                  write(irefwr,*) 'Bad ratio: improve element directly'
               jelem = nelemlc

!              --- Set number of nodes: np

               np = npointlc
               call msh665( kmeshcloc, nelemlc, nelmfix, jelem, ja, &
                            coor3d, npointlc )
               if ( np/=npointlc ) then

!              --- Number of nodes changed, refill icube:

                  do j = 1, npointlc
                     n1 = int ( (coor3d(1,j)-xstart) / dismax )
                     n2 = int ( (coor3d(2,j)-ystart) / dismax )
                     n3 = int ( (coor3d(3,j)-zstart) / dismax )
                     nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1
                     icube(j) = nc1
                  end do
               end if
               i1 = kmeshcloc(1,nelemlc)
               i2 = kmeshcloc(2,nelemlc)
               i3 = kmeshcloc(3,nelemlc)
               i4 = kmeshcloc(4,nelemlc)
               call msh661(coor3d,i1,i2,i3,i4,ratio)
               if ( debug ) &
                  write(irefwr,*) 'New?',i1,i2,i3,i4,'ratio =',ratio
            end if
            goto 300
         end if
         if ( debug ) write(irefwr,*) 'Special not used'
      end if

!     --- No special reason for tetrahedron with boundary triangles ;
!         determine for triangle i1 - i2 - i3 coordinates of centre of
!         circumcircle

      call msh609(coor3d,i1,i2,i3,e1,e2,e3,xm,ym,zm)

!     --- Coarseness triangle

      alp    = cube(icube(i1)) + cube(icube(i2)) + cube(icube(i3))
      alp    = factor * alp / 3.0d0
      alp1   = alp
      coa    = alp

!     --- First estimate of point

      alp = 0.8d0 * alp

!     --- Coordinates:

      xn = xm + alp * e1
      yn = ym + alp * e2
      zn = zm + alp * e3

!     --- Check whether point belongs to volume

      ja = 0
      call msh607(xn,yn,zn,xstart,coor3d,kstapl,kstap,ja)
      jaold = ja
      if ( ja==1 ) then

!     --- Point (xn,yn,zn) belongs to volume.
!         Determine cube for this point

         n1 = int ( (xn-xstart) / dismax )
         n2 = int ( (yn-ystart) / dismax )
         n3 = int ( (zn-zstart) / dismax )

!        --- Cubenumber:

         nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1

!        --- Set value coa

         if ( nc1>ncube ) then
            coa = alp1
         else
            coa = factor * cube(nc1)
         end if

!        --- Finally take mean value:

         alp = 0.5d0 * ( coa + alp1 )
      end if

!     --- Coarseness:

      coa = alp

!     --- Determine on basis of this alp-value new point

      xn = xm + alp * e1
      yn = ym + alp * e2
      zn = zm + alp * e3

!     --- Check whether this point belongs to volume:

      call msh607(xn,yn,zn,xstart,coor3d,kstapl,kstap,ja)

!     --- Determine extra values for case that new point has to be placed

      xnx = xm + 0.78 * alp * e1
      yny = ym + 0.78 * alp * e2
      znz = zm + 0.78 * alp * e3

!     --- Check whether new point is allowed or is there a common
!         point with another triangle (with number kdrie)

      kdrie = 0
      jnear = 0
      call msh616(coor3d,i1,i2,i3,kdrie,kstapl,kstap,xn,yn,zn,0)

!     --- Both points inside?

      jaold = jaold * ja

!     --- If all surrounding triangles are "divergent": (tuning 0.4)

      if ( jaold==1 .and. kdrie==0 .and. &
           angle1<-0.4 .and. angle2<-0.4 .and. angle3<-0.4 ) then

!     --- Not too near to other nodes?

         dista = 1.2d0 * coa
         call msh617(coor3d,jpn,npointlc,itri,i1,i2,i3,xnx,yny,znz,dista)

!        --- Check ratio dista / coa: (tuning 0.85)

         if ( dista/coa>0.85d0 ) then

!        --- Special situation, if allowed place new node:

            jpn = npointlc + 1

!           --- Fill coordinates array:

            coor3d(1,jpn) = xnx
            coor3d(2,jpn) = yny
            coor3d(3,jpn) = znz

!           --- Check new tetrahedron i1,i2,i3,jpn
!               First compute "new" angles:

            call msh613(jpn,i2,i1,iex3,coor3d,angle1)
            call msh613(jpn,i3,i2,iex1,coor3d,angle2)
            call msh613(jpn,i1,i3,iex2,coor3d,angle3)

!           --- Check intersections with old elements

            call msh631(kstapl,kstap,line,lenline, coor3d, &
                        xnx,yny,znz,0,itri,npointlc,jcheck)

!           --- If both checks ok, create element

            if ( angle1<0.98d0.and.angle2<0.98d0.and. &
                 angle3<0.98d0.and.jcheck==0 .or. &
                 jcheck==0 .and. nochan>kstap ) then

!           --- New tetrahedron is i1,i2,i3,npointlc+1 ; fill arrays coor,
!               kmeshcloc and adjust kstapl and itri

               call msh618(coor3d,npointlc,kmeshcloc,nelemlc,kstapl,kstap, &
                           itri,i1,i2,i3,xnx,yny,znz)
               nochan = 0

!              --- Determine cube for new point

               n1 = int ( (xnx-xstart) / dismax )
               n2 = int ( (yny-ystart) / dismax )
               n3 = int ( (znz-zstart) / dismax )

!              --- Cubenumber

               nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1

!              --- Set number

               icube(npointlc) = nc1

               call msh661( coor3d, i1, i2, i3, npointlc, ratio )

               if ( debug .and. iherha==0 ) then
                  call msh661( coor3d, i1, i2, i3, npointlc, ratio )
                  write(irefwr,*) nelemlc,'div method, ratio = ',ratio
               end if
               goto 300
            end if
         end if
      end if

!     --- If jaold==0 and kdrie==0: point is near to the boundary!
!         Better tet with another triangle ?

      if ( jaold==0 .and. kdrie==0 ) jnear = 1

!     --- Replace with adjusted values and check again:

      xn = xnx
      yn = yny
      zn = znz

!     --- Point allowed?

      if ( kdrie==0 ) then
         call msh631(kstapl,kstap,line,lenline,coor3d,xn,yn,zn,0,itri, &
                     npointlc,jcheck)

!        --- If boundary problems: replace triangle

         if ( jcheck/=0 .or. jnear==1 ) then

!        --- Replace triangle

            call msh633(kstapl,kstap,itri)

!           --- Next triangle

            goto 300
         end if
      end if

!     --- Triangle kdrie:

      if ( kdrie>0 ) then

!     --- There is a common point with triangle kdrie

         itry = 0

!        --- Try again:

420      itry = itry + 1

!        --- Nodes triangle kdrie

         ii1 = kstapl( 1, kdrie )
         ii2 = kstapl( 2, kdrie )
         ii3 = kstapl( 3, kdrie )

!        --- Check possibility of tet with one of these (old) points
!            ( Check whether points are allowed and take the best ratio )

         jh1 = 1
         if ( i1==ii1.or.i2==ii1.or.i3==ii1 ) jh1 = 0
         jh2 = 1
         if ( i1==ii2.or.i2==ii2.or.i3==ii2 ) jh2 = 0
         jh3 = 1
         if ( i1==ii3.or.i2==ii3.or.i3==ii3 ) jh3 = 0

!        --- Initialize jpn and vermin:

         jpn    = 0
         vermin = 0d0

!        --- Point ii1 ?

         if ( jh1/=0 ) then

!        --- Check position and ratio of node ii1:

            call msh661(coor3d, i1, i2, i3, ii1, ratio )

!           --- ratio ok?

            if ( ratio>vermin ) then
               vermin = ratio
               jpn    = ii1
            end if
         end if

!        --- Point ii2 ?

         if ( jh2/=0 ) then

!        --- Check position and ratio of node ii2:

            call msh661(coor3d, i1, i2, i3, ii2, ratio )

!           --- ratio ok?

            if ( ratio>vermin ) then
               vermin = ratio
               jpn    = ii2
            end if
         end if

!        --- Point ii3 ?

         if ( jh3/=0 ) then

!        --- Check position and ratio of node ii3:

            call msh661(coor3d, i1, i2, i3, ii3, ratio )

!           --- ratio ok?

            if ( ratio>vermin ) then
               vermin = ratio
               jpn    = ii3
            end if
         end if

!        --- Point jpn allowed, else place triangle at bottom of kstapl
!            (At least one of the points has a positive ratio, jpn=/0!)

         jdrie = 0

!        --- check

         if ( jpn>0 ) then
            xn = coor3d(1,jpn)
            yn = coor3d(2,jpn)
            zn = coor3d(3,jpn)
            call msh616(coor3d,i1,i2,i3,jdrie,kstapl,kstap,xn,yn,zn,jpn)
         end if

!        --- jdrie?

         if ( jdrie>0 .and. jpn>0 ) then
            idr1 = kstapl( 1, jdrie )
            idr2 = kstapl( 2, jdrie )
            idr3 = kstapl( 3, jdrie )
            jh   = 1
            if ( idr1==jpn.or.idr2==jpn.or.idr3==jpn ) jh = 0

!           --- new triangle to be considered?

            if ( jh/=0 ) then
               kdrie = jdrie
               if ( itry<3 ) goto 420

!              --- Problems? For safety set jpn = 0

               jpn = 0
            end if
         end if

!        --- node jpn?

         if ( jpn>0 ) then
            call msh631(kstapl,kstap,line,lenline,coor3d,xn,yn,zn, &
                        jpn,itri,npointlc,jcheck)
         else
            jcheck = -1
         end if

!        --- For savety check that barycenter is inside:

         if ( jcheck==0 ) then

!        --- Check tet: i1 - i2 - i3 - iex1 ?

            xhalf = ( coor3d(1,i1) + coor3d(1,i2) + coor3d(1,i3) + &
                      coor3d(1,jpn) ) / 4d0
            yhalf = ( coor3d(2,i1) + coor3d(2,i2) + coor3d(2,i3) + &
                      coor3d(2,jpn) ) / 4d0
            zhalf = ( coor3d(3,i1) + coor3d(3,i2) + coor3d(3,i3) + &
                      coor3d(3,jpn) ) / 4d0

!           --- Inside:

            call msh607(xhalf,yhalf,zhalf,xstart,coor3d,kstapl,kstap,jas)

!           --- If not inside, not allowed

            if ( jas==0 ) jcheck = -1
         end if

!        --- For safety check the ratio:

         if ( jcheck==0 ) then
            call msh661(coor3d,i1,i2,i3,jpn,ratio )
            if ( ratio<0 ) jcheck = -1
            if ( ratio<0.1d0 .and. nochan<3 ) jcheck = -1
         end if

!        --- If all checks passed:

         if ( jcheck==0 ) then

!        --- New tetrahedron

            call msh612(kmeshcloc,nelemlc,i1,i2,i3,jpn,kstapl,kstap,itri)
            if ( debug .and. iherha==0 ) &
               write(irefwr,*) nelemlc,'old point, ratio = ',ratio
            nochan = 0
         end if
      else

!     --- New point not to near to old point ? Run through array itri

         dista = 1.2d0 * coa

!        --- Find number and distance nearest point in boundary:

         call msh617(coor3d,jpn,npointlc,itri,i1,i2,i3,xn,yn,zn,dista)

!        --- Check whether nearest point is visible:

         if ( jpn>0 ) then

!        --- Check all "new" edges with respect to present boundary

            kdrie = 0

!           --- Coordinates new (old) point:

            xpt = coor3d(1,jpn)
            ypt = coor3d(2,jpn)
            zpt = coor3d(3,jpn)

!           --- Check edges:

            call msh616( coor3d, i1, i2, i3, kdrie, kstapl, kstap, &
                         xpt, ypt, zpt, jpn )

!           --- Intersection with triangle kdrie

            if ( kdrie>0 ) dista = coa
         end if

!        --- Place or not:

         if ( dista>0.8 * coa ) then

!        --- Place new node:

            jpn = npointlc + 1

            coor3d(1,jpn) = xn
            coor3d(2,jpn) = yn
            coor3d(3,jpn) = zn

!           --- Check new tetrahedron i1,i2,i3,jpn
!               Compute angles:

            call msh613(jpn,i2,i1,iex3,coor3d,angle1)
            call msh613(jpn,i3,i2,iex1,coor3d,angle2)
            call msh613(jpn,i1,i3,iex2,coor3d,angle3)

!           --- Intersections with old elements?

            call msh631(kstapl,kstap,line,lenline, coor3d, &
                        xnx,yny,znz,0,itri,npointlc,jcheck)

!           --- Both checks ok?

            if ( angle1<0.98d0.and.angle2<0.98d0.and. &
                 angle3<0.98d0.and.jcheck==0 .or. &
                 jcheck==0 .and. nochan>kstap ) then

!           --- New tetrahedron is i1,i2,i3,npointlc+1 ; fill arrays coor,
!               kmeshcloc and adjust kstapl and itri

               call msh618( coor3d, npointlc, kmeshcloc, nelemlc, kstapl, kstap, &
                            itri, i1, i2, i3, xnx, yny, znz )
               if ( debug .and. iherha==0 ) then
                  call msh661( coor3d, i1, i2, i3, npointlc, ratio )
                  write(irefwr,*) nelemlc,'new point, ratio = ',ratio
               end if
               nochan = 0

!              --- Determine cube for this new node

               n1 = int ( (xnx-xstart) / dismax )
               n2 = int ( (yny-ystart) / dismax )
               n3 = int ( (znz-zstart) / dismax )

!              --- Cube number

               nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1

!              --- Fill icube

               icube(npointlc) = nc1
            end if
         else

!        --- Small distance to boundary. Nearest boundary point is jpn, &
!            tetrahedron allowed ?

            if ( jpn==0 ) then
               call errsub ( 2906, 0, 0, 0 )
               go to 1150
            end if

!           --- Check possibility tetrahedron with old node jpn:
!               Check visibility and ratio

            call msh661(coor3d,i1,i2,i3,jpn,ratio)
            call msh631(kstapl,kstap,line,lenline,coor3d,xpt,ypt,zpt, &
                        jpn,itri,npointlc,jcheck)

!           --- Next check whether element to be created does
!               not cause intersection problems with neighbour elements
!               represented by iex1, iex2 and iex3

            ja = 0

!           --- consider "new" neighbour elements:

            if ( jcheck==0.and.ratio>0.1d0 ) then
               volumb = -1d0
               if (angle1>-1d0 .and. iex1/=jpn) &
                    call msh632(coor3d,i2,i3,iex1,jpn,volumb)
               if (volumb>0) ja = 1
               if (angle2>-1d0 .and. iex2/=jpn) &
                    call msh632(coor3d,i3,i1,iex2,jpn,volumb)
               if (volumb>0) ja = 1
               if (angle3>-1d0 .and. iex3/=jpn) &
                    call msh632(coor3d,i1,i2,iex3,jpn,volumb)
               if ( volumb>0) ja = 1
            end if

!           --- Finally check barycenter new element

            if ( jcheck==0.and.ja==0.and.debug ) then
               numelm=0
               call msh651 ( i1, i2, i3, jpn, kmeshcloc, nelemlc, &
                             coor3d, npointlc, numelm )
               if ( debug .and. numelm>0 ) then
                  write(irefwr,*) '651: Error, element',i1,i2,i3,jpn
                  write(irefwr,*) i1,'met',(coor3d(j,i1),j=1,3)
                  write(irefwr,*) i2,'met',(coor3d(j,i2),j=1,3)
                  write(irefwr,*) i3,'met',(coor3d(j,i3),j=1,3)
                  write(irefwr,*) jpn,'met',(coor3d(j,jpn),j=1,3)
               end if
            end if

!           --- Check whether none of the barycentres of the
!               old elements is inside the new element

            numelm = 0
            if ( jcheck==0.and.ja==0 ) &
               call msh651( i1, i2, i3, jpn, kmeshcloc, nelemlc, &
                            coor3d, npointlc, numelm )
            if ( numelm>0 ) ja=1

!           --- If visible, ratio ok and no intersections:
!               create tetrahedron

            if ( jcheck==0.and.ratio >0.1d0.and.ja==0 .or. &
                 jcheck==0.and.nochan>kstap.and.ja==0 ) then

!           --- New tetrahedron: i1,i2,i3 with old point jpn

               if ( debug ) write(irefwr,*) 'new element',i1,i2,i3,jpn
               call msh612(kmeshcloc,nelemlc,i1,i2,i3,jpn,kstapl,kstap, &
                           itri)
               if ( debug .and. iherha==0 ) write(irefwr,*) &
                  nelemlc,'With old node ',jpn,'ratio = ',ratio
               nochan = 0
            end if
         end if
      end if

!     --- If no change: Change array kstapl

      if ( nochan>0 ) call msh633(kstapl,kstap,itri)

!     --- Array kstapl empty?

      if ( kstap>0 ) goto 300

!     --- Check itri (all elements should be zero)

500   itot = 0

!     --- Check contents of itri (should all be zero)

      if ( debug ) &
         write(irefwr,*) 'After label 500 npointlc, nelemlc =',npointlc,nelemlc
      do i = 1, npointlc
         itot = itot + abs(itri(i))
      end do

!     --- itri should have npointlc zero entries

      if ( itot/=0 ) then
         call errint ( itot,1 )
         call errint ( ivolnr, 2 )
         call errsub ( 926, 2, 0, 0 )
      end if

!     --- Determine characteristic ratio value vermin for comparison
!         reasons:

      vermin = rinfin

!     --- Compute all ratio's

      do ielem = 1, nelemlc
         i1 = kmeshcloc (1,ielem)
         i2 = kmeshcloc (2,ielem)
         i3 = kmeshcloc (3,ielem)
         i4 = kmeshcloc (4,ielem)
         call msh661 ( coor3d, i1, i2, i3, i4, ratio  )
         if ( debug .and. ratio<0 ) then
            write(irefwr,*) 'Warning: iherha = ',iherha
            write(irefwr,*) 'Tetra',ielem,' - ',i1,i2,i3,i4
            write(irefwr,*) 'negative ratio =',ratio
         end if
         vermin = min( ratio, vermin )
      end do  ! ielem = 1, nelemlc

!     --- Give message:

      if ( iherha==0 .and. debug ) then
         write(irefwr,*) 'End first subdivision, ratio = ',vermin
      end if

!     --- Solution found, if first solution or a better solution than
!         the old one save new solution before extra actions are performed:

      if ( ratiop<-1d0 ) then

!     --- Allocate space for help arrays coortmp and meshtmp:
!         Length coortmp is npointlc:

         npndef = npointlc + int(125 + npointlc/10)
         npntmp = npointlc

!        --- Length meshtmp is nelemlc:

         neldef = nelemlc + int(125 + nelemlc/10)
         neltmp = nelemlc

!        --- Allocate

         allocate( coortmp(3,npndef), meshtmp(4,neldef), stat=error )
         if ( error>0 ) then
            call eralloc ( error, 3*npndef+4*neldef, 'coortmp' )
            go to 1150
         end if

!        --- Fill help arrays with Coordinates:

         do i=1,3
            coortmp(i,1:npointlc) = coor3d(i,1:npointlc)
         end do

!        --- Elements:

         do i=1,4
            meshtmp(i,1:nelemlc ) = kmeshcloc(i,1:nelemlc)
         end do
         ratiop = vermin
         if ( debug ) write(irefwr,*) 'Start, iherha=0, ratio = ',ratiop
      else if ( vermin>0 .and. ratiop<vermin ) then

!     --- "Better mesh found" place in temporary arrays
!         --- Coordinates:

         do i=1,3
            coortmp(i,1:npointlc) = coor3d(i,1:npointlc)
         end do

!        --- Elements:

         do i=1,4
            meshtmp(i,1:nelemlc ) = kmeshcloc(i,1:nelemlc)
         end do
         if ( debug ) then
            write(irefwr,*) 'End phil3D:New mesh better',npointlc,nelemlc
            write(irefwr,*) 'Old mesh, npointlc =',npntmp,'nelemlc=',neltmp
            write(irefwr,*) 'Old ratio =',ratiop,'new ratio =',vermin
         end if
         ratiop = vermin
         npntmp = npointlc
         neltmp = nelemlc
      else

!     --- New mesh is not yet(?) better

         if ( debug ) then
            write(irefwr,*) 'New mesh not yet(?) better'
            write(irefwr,*) 'New mesh: npointlc =',npointlc,'nelemlc=',nelemlc
            write(irefwr,*) 'Old mesh: npointlc =',npntmp,'nelemlc=',neltmp
            write(irefwr,*) 'Old ratio =',ratiop,'new ratio =',vermin
         end if
      end if

!     --- Start checking the elements:

      if ( debug ) then

!     --- Contents after division:

         write(irefwr,*) 'Elements after first division: '
         vol    = 0
         volumb = 0

!        --- Compute volumes and ratio's

         do ielem = 1, nelemlc
            i1 = kmeshcloc( 1, ielem )
            i2 = kmeshcloc( 2, ielem )
            i3 = kmeshcloc( 3, ielem )
            i4 = kmeshcloc( 4, ielem )

            call msh632( coor3d, i1, i2, i3, i4, voluma )
            call msh661( coor3d, i1, i2, i3, i4, ratio  )
            vol    = vol + abs(voluma)
            volumb = volumb + voluma

            if ( debug ) then
               write(irefwr,*) ' elnum',ielem,'=',i1,i2,i3,i4
               write(irefwr,*) ' and ratio = ',ratio
            end if
         end do

         vol    = vol    / 6d0
         volumb = volumb / 6d0
         write(irefwr,*) 'nelemlc and corresponding volume ',nelemlc,vol

         if ( abs(vol-volumb)> 100 * epsmac ) then
            write(irefwr,*) 'negative volumes? volumb =',volumb
         end if
      end if

!     --- Set number of basic elements that are fixed:

      jelem = nelmfix

!     --- Check all free elements for ratio and try to improve
!         using msh665:

      do while ( nelemlc>jelem )

         jelem = jelem + 1

         i1 = kmeshcloc(1,jelem)
         i2 = kmeshcloc(2,jelem)
         i3 = kmeshcloc(3,jelem)
         i4 = kmeshcloc(4,jelem)

         call msh661(coor3d,i1,i2,i3,i4,ratio)

!        --- Check for use of msh663

         itot = nelemlc

!        --- Try to remove one of the six sides of the tet

         if ( ratio<0.1d0 ) then
            call msh663( coor3d, nipnt, npointlc, kmeshcloc, nelemlc, jelem)
         end if

!        --- Succesfull? else second check!

         if ( itot==nelemlc .and. ratio<0.35d0 ) then

!        --- Set number of nodes: np

            np = npointlc
            call msh665( kmeshcloc, nelemlc, nelmfix, jelem, ja, &
                         coor3d, npointlc )
            if ( np/=npointlc ) then

!           --- Nodes changed, adjust cube reference values:

               do j = 1, npointlc
                  n1 = int ( (coor3d(1,j)-xstart) / dismax )
                  n2 = int ( (coor3d(2,j)-ystart) / dismax )
                  n3 = int ( (coor3d(3,j)-zstart) / dismax )

                  nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1
                  icube(j) = nc1
               end do
            end if
         end if
      end do
      iherha = iherha + 1
      if ( debug ) then
         write(irefwr,*) 'After first calls to msh665',nelemlc,' elements'
         volumb = 0
         do ielem = 1, nelemlc
            i1 = kmeshcloc( 1, ielem )
            i2 = kmeshcloc( 2, ielem )
            i3 = kmeshcloc( 3, ielem )
            i4 = kmeshcloc( 4, ielem )
            call msh661( coor3d, i1, i2, i3, i4, ratio )
            call msh632( coor3d, i1, i2, i3, i4, voluma)
            volumb = volumb + abs(voluma)
         end do
         volumb = volumb / 6d0
         write(irefwr,*) 'Total volume = ',volumb
      end if

!     --- Check number of nodes and elements:

      if ( npointlc>npunt ) then
         call errint ( npunt,1 )
         call errint ( npointlc,2 )
         call errint ( ivolnr, 3 )
         call errsub ( 923, 3, 0, 0 )
      end if
      if ( nelemlc>nkmeshc ) then
         call errint ( nkmeshc,1 )
         call errint ( nelemlc,2 )
         call errint ( ivolnr, 3 )
         call errsub ( 924, 3, 0, 0 )
      end if

!     --- Find present ratio's and if possible and allowed stop division:

      vermin = 1d0

!     --- Run through all elements:

      do ielem = nelmfix+1, nelemlc
         i1 = kmeshcloc( 1, ielem )
         i2 = kmeshcloc( 2, ielem )
         i3 = kmeshcloc( 3, ielem )
         i4 = kmeshcloc( 4, ielem )

         call msh661( coor3d, i1, i2, i3, i4, ratio )

         if ( ratio<vermin ) then
            vermin = ratio
            jelem  = ielem
         end if
      end do

!     --- If smallest ratio>0.25 escape:

      if ( iherha>5 .and. vermin>0.25d0 ) then

!     --- Mesh is more than acceptable

          if (debug) write(irefwr,*) 'escape, vermin =',vermin
          goto 900
      end if

!     --- If smallest ratio is not too bad and iherha great, escape
!         but give warning:

      if ( iherha> 20 .and. vermin>0.20d0 .or. &
           iherha> 30 .and. vermin>0.10d0 .or. &
           iherha> 50 .and. vermin>0.05d0 ) then

!     --- Escape with warning:

         call errwar ( 2962, 0, 0, 0 )
         if ( debug ) write(irefwr,*) 'Escape smallest ratio = ',vermin
         goto 900
      end if

!     --- Check volumes and ratio's of all tetrahedrons

      inegat =   0
      jelvol =   0
      iloop  =   0

!     --- Check all nodes of tet in one plane and repair:

      if ( abs(vermin)<=1d-12 ) then

!     --- Element jelem has four nodes in a plane, try to replace:

         if ( debug) write(irefwr,*) 'Four nodes in a plane'

!        --- Set number of nodes: np

         np = npointlc
         if ( jelem>nelmfix ) then

!        --- First try msh663:

            call msh663( coor3d, nipnt, npointlc, kmeshcloc, nelemlc, jelem)
            if ( np==npointlc ) then
                 call msh665( kmeshcloc, nelemlc, nelmfix, jelem, ja, &
                              coor3d, npointlc )
            end if

!           --- If number of points changed, then adjust cube values:

            if ( np/=npointlc ) then
               do j = 1, npointlc
                  n1 = int ( (coor3d(1,j)-xstart) / dismax )
                  n2 = int ( (coor3d(2,j)-ystart) / dismax )
                  n3 = int ( (coor3d(3,j)-zstart) / dismax )

                  nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1
                  icube(j) = nc1
               end do
            end if
         end if
      else
         jelem = 0
      end if

      do while ( jelvol==0 .and. jelem==0 .and. iloop<100 )
         jelem  = -1
         vermin = 1d0
         iloop  = iloop + 1

!        --- Use eventually for extra debugging:

         vol = 0d0

         ib = 0

         do ielem = nelmfix+1, nelemlc
            i1 = kmeshcloc( 1, ielem )
            i2 = kmeshcloc( 2, ielem )
            i3 = kmeshcloc( 3, ielem )
            i4 = kmeshcloc( 4, ielem )

            call msh661( coor3d, i1, i2, i3, i4, ratio )
            call msh632( coor3d, i1, i2, i3, i4, voluma)

            if ( ratio<0.1d0 .and. ib==0 ) ib=ielem

            if ( ratio<vermin ) then
                 jelem  = ielem
                 vermin = ratio
            end if

            vol = vol + abs(voluma/6d0)
         end do

         if ( debug ) write(irefwr,*) 'Position volume = ',vol

!        --- Reorder elements:

         if ( iherha>5 .and. ib>0 .and. vermin>0d0 ) then

!        --- Find elementnumber to change:

            ielem = nelemlc

            do while ( ielem>0 )
               i1 = kmeshcloc( 1, ielem )
               i2 = kmeshcloc( 2, ielem )
               i3 = kmeshcloc( 3, ielem )
               i4 = kmeshcloc( 4, ielem )

               call msh661( coor3d, i1, i2, i3, i4, ratio )

               if ( ratio>0.1d0 ) then

!              --- Number found:

                  kmeshcloc( 1, ielem ) = kmeshcloc( 1, ib )
                  kmeshcloc( 2, ielem ) = kmeshcloc( 2, ib )
                  kmeshcloc( 3, ielem ) = kmeshcloc( 3, ib )
                  kmeshcloc( 4, ielem ) = kmeshcloc( 4, ib )

                  jelem = ielem

                  kmeshcloc( 1, ib ) = i1
                  kmeshcloc( 2, ib ) = i2
                  kmeshcloc( 3, ib ) = i3
                  kmeshcloc( 4, ib ) = i4

                  ielem = 0
               else

!              --- Find next elementnumber to replace

                  ielem = ielem-1
                  if ( ielem==0 ) then
                     call errsub ( 2907, 0, 0, 0 )
                     go to 1150
                  end if
               end if
           end do
         end if
         if ( debug ) write(irefwr,*) 'vermin = ',vermin,'vol=',vol

         if ( vermin<0d0 ) jelvol = jelem

         if ( jelvol==0 .and. jelem>0 ) then
            if ( iherha<10 .and. vermin<0.15d0 .or. &
                 iherha<20 .and. vermin<0.10d0 .or. &
                 iherha<30 .and. vermin<0.05d0 .or. &
                 iloop <10 ) then

!           --- Try to reorder all the elements around element jelem

               if ( debug ) &
                    write(irefwr,*) 'Remove element with ratio ',vermin
               i1 = kmeshcloc( 1, jelem )
               i2 = kmeshcloc( 2, jelem )
               i3 = kmeshcloc( 3, jelem )
               i4 = kmeshcloc( 4, jelem )

               ja    = 0
               itot  = nelemlc

               if ( jelem>nelmfix ) then

!              --- Set number of nodes: np

                  np = npointlc
                  call msh665( kmeshcloc, nelemlc, nelmfix, jelem, ja, &
                               coor3d, npointlc )
                  if ( np/=npointlc ) then

!                 --- Nodes changed, find new cube values:

                     do j = 1, npointlc
                        n1 = int ( (coor3d(1,j)-xstart) / dismax )
                        n2 = int ( (coor3d(2,j)-ystart) / dismax )
                        n3 = int ( (coor3d(3,j)-zstart) / dismax )

                        nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1
                        icube(j) = nc1
                     end do
                  end if

!                 --- Check new created elements in case of debugging:

                  if ( debug ) then
                    numelm = 0
                    call msh652( kmeshcloc, nelemlc, coor3d, npointlc, numelm )
                    if ( numelm>0 ) then
                      write(irefwr,*) 'After calls to msh665 and msh652'
                      write(irefwr,*) 'Problems in element',numelm
                      call errint ( numelm, 1 )
                      call errsub ( 2913, 1, 0, 0 )
                      go to 1150
                    end if
                  end if

!                 --- Check situation, consider value ja:

                  if ( ja==0 ) then

!                 --- Boundary with reordering, succes?

                     call msh661(coor3d,i1,i2,i3,i4, ratio)
                     if ( ratio>vermin+0.001d0 ) jelem = 0
                  else if ( ja==1 .or. ja==2 ) then
                     jelem = 0
                  else if ( ja==3 ) then

!                 --- Reordering succesfull?

                     if ( itot>nelemlc ) jelem = 0
                  end if
               end if
            end if
         end if
      end do
      if ( debug ) write(irefwr,*) 'Next step'
      if ( jelvol==0 ) then

!     --- Reset jelem if needed i.e. in case -1:

         if ( jelem==-1 ) jelem = 0
            if ( debug ) then

!           --- Compute reference volume

            voluma = 0d0

!           --- Run through all elements

            do i = 1, nelemlc
               i1 = kmeshcloc(1,i)
               i2 = kmeshcloc(2,i)
               i3 = kmeshcloc(3,i)
               i4 = kmeshcloc(4,i)

!              --- Compute volume

               call msh632(coor3d, i1, i2, i3, i4, vol)

!              --- Add tet to total volume

               voluma = voluma + abs(vol)
            end do

!           --- Reference volume computation (later use??)

            voluma = voluma / 6d0
            write(irefwr,*) 'Reference volume is voluma'
         end if
      end if

!     --- If negative element:

      if ( jelvol/=0 ) jelem = jelvol

!     --- Check for ready

      if ( jelem==0 .and. vermin>0.25d0 .and. iherha>10 ) goto 900
      if ( iherha> 20 .and. vermin>0.20d0 .or. &
           iherha>100 .and. vermin>0.10d0 ) goto 900
      if ( iherha>200 .and. vermin>0 ) then

!     --- Too many redivisions

         if ( ratiop>0 ) then

!        --- Use escape mode:

            ratio = 0.5 * ratiop
            goto 1050
         else if ( jelvol==0 ) then

!        --- Give warning

            call errint ( iherha, 1 )
            call errwar ( 2963, 1, 0, 0 )
            goto 900
         else

!        --- Stop, give error message

            call errsub ( 2908, 0, 0, 0 )
            go to 1150
         end if
      end if
      if ( ierror/=0 ) go to 1150

!     --- Check ratios of spheres, of sides, absolute length of sides, &
!         repeats and negative elementvolumes:

      if ( jelem==0 .or. jelem==nelemlc .or. iherha>30 ) then

!     --- Check ratio's of inner and outer sphere for each element:

         kherha = 0

520      ifrato =  0
         vermin = 10d0

         kherha = kherha + 1
         jelem  = 0

         if ( kherha<10 ) then

!        --- Volume for debugging

            voluma =  0d0
            volumb =  0d0
            vermin = 10d0

            do ielem = nelmfix+1, nelemlc

               i1 = kmeshcloc( 1, ielem )
               i2 = kmeshcloc( 2, ielem )
               i3 = kmeshcloc( 3, ielem )
               i4 = kmeshcloc( 4, ielem )

               call msh632( coor3d, i1, i2, i3, i4, vol )
               call msh661( coor3d, i1, i2, i3, i4, ratio )

               if ( vol<volumb ) then

                  jelem  = ielem
                  volumb = vol
                  vermin = ratio

               end if

               voluma = voluma + abs(vol)

            end do

            voluma = voluma / 6d0

            if ( vermin<-0.0 ) then
               jelvol = jelem
            else
               jelem  = 0
            end if

            if ( jelem==0 ) then

               do ielem = nelmfix+1, nelemlc
                  i1 = kmeshcloc( 1, ielem )
                  i2 = kmeshcloc( 2, ielem )
                  i3 = kmeshcloc( 3, ielem )
                  i4 = kmeshcloc( 4, ielem )
                  call msh661( coor3d, i1, i2, i3, i4, ratio )

!                 --- Check position of tetrahedron:

                  if ( abs(ratio)<0.15d0 ) then
                     ja   = 0
                     itot = nelemlc
                     np   = npointlc
                     jelem = ielem
                     call msh663( coor3d, nipnt, npointlc, kmeshcloc, nelemlc, &
                                  jelem)
                     if ( itot==nelemlc .and. ratio>0 ) then

!                    --- No removal, try to make a redivision

                        jelem = ielem
                        if ( jelem>nelmfix ) then

!                       --- Set number of nodes: np

                           np = npointlc
                           call msh665( kmeshcloc, nelemlc, nelmfix, &
                                        jelem, ja, coor3d, npointlc )
                           if ( np/=npointlc ) then
                              do j = 1, npointlc
                                 n1 = int ( (coor3d(1,j)-xstart)/dismax )
                                 n2 = int ( (coor3d(2,j)-ystart)/dismax )
                                 n3 = int ( (coor3d(3,j)-zstart)/dismax )

                                 nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1
                                 icube(j) = nc1
                              end do
                           end if
                        end if
                     end if

!                    --- Check for restarting:

                     if ( itot>nelemlc .or. ja==1 .or. ja==2 ) goto 520
                  end if
                  if ( max(i1,i2,i3,i4)>nipnt ) then

!                 --- Check ratio

                     if ( ratio<vermin ) then
                        vermin = ratio
                        jelem  = ielem
                        if ( ifrato==0 .and. ratio<0.2d0 ) &
                             ifrato = ielem
                     end if
                  end if
               end do

!              --- Check for ready:

               if ( vermin>0.35d0 .and. iherha> 5 ) goto 900
               if ( vermin>0.15d0 .and. iherha>10 ) jelem = 0
               if ( jelem>0 ) then
                  if ( iherha>10 .and. vermin>0d0 .and. ifrato>0 ) &
                       jelem = ifrato

!                 --- Compute ratio of jelem

                  i1 = kmeshcloc( 1, jelem )
                  i2 = kmeshcloc( 2, jelem )
                  i3 = kmeshcloc( 3, jelem )
                  i4 = kmeshcloc( 4, jelem )

!                 --- Ratio

                  call msh661( coor3d, i1, i2, i3, i4, ratio )
                  if ( iherha>10 .and. ratio>0.02d0 .and. ifrato>0 ) &
                       jelem = ifrato
                  if ( iherha>150 .and. ratio>0.001d0 ) jelem = 0

!                 --- Eventually for the future:
!                     Run through all elements and replace
!                     call msh662( coor3d, npointlc, kmeshcloc, nelemlc, jelem )

               end if
            end if
         end if
      end if

!     --- Ready, check jelem number:

      if ( jelem>0 .and. mod(iherha,2)==0 .and. iherha>10 ) then

!     --- Adjust surroundings of element jelem

         call msh638(kmeshcloc,nelemlc,jelem,kstapl,kstap,nipnt)

!        --- Check present length of kstapl

         leng = kstap

!        --- check

         if ( leng>lenkstapl ) then
            call errint ( lenkstapl, 1 )
            call errint ( leng     , 2 )
            call errint ( ivolnr   , 3 )
            call errwar ( 928, 3, 0, 0 )
         end if

!        --- Remove the nodes that are not used and renumber the
!            remaining points accordingly in kmeshcloc, kstapl, coor
!            and icube

         call msh649( kmeshcloc, nelemlc , kstap , kstapl, coor3d  , npointlc, &
                      nipnt , itri  , icube , nx1    , ny1    , &
                      xstart, ystart, zstart, dismax )

!        --- Make another run for this subvolume

         nochan = 0
      end if

!     --- Check for ready:

      if ( kstap>0 ) goto 300

!     --- kstap=0 and no error?

      if ( ierror/=0 ) go to 1150

!     --- End of new loop

550   if ( .not. jnew ) then

!     --- Set declared length:

         lenkstapl = 3 * npunt + 100
         irepos =  1
      end if

!     --- Compare new values with old values

      nol  = nelold - nelemlc
      nul  = nptold - npointlc
      ntot = nol * nol + nul * nul

!     --- Set istep variable

      if ( ntot==0 .and. iherha>2 ) then
         istep = istep + 1
      else
         istep = 0
      end if

!     --- if too many runs, no repositioning

      if ( iherha>100 .and. inegat==0 ) then

!     --- Stop redivision and submit warning

         call errint ( iherha, 1 )
         call errint ( ivolnr, 2 )
         call errwar ( 927, 2, 0, 0 )
         go to 900
      else if ( iherha>200 ) then
         call errsub ( 2964, 0, 0, 0 )
         go to 1150
      end if
      nelold = nelemlc
      nptold = npointlc

!     --- Repositioning if elements are ok, &
!         check the element ratio's and volumes first

      volumi = rinfin
      volmax = 0d0
      vol    = 0d0
      jelem  = 0
      ratio  = 1d0

!     --- Run through all elements to compute ratio and volume:

      do ielem = 1, nelemlc
         i1 = kmeshcloc(1, ielem)
         i2 = kmeshcloc(2, ielem)
         i3 = kmeshcloc(3, ielem)
         i4 = kmeshcloc(4, ielem)

!        --- compute volume

         call msh632(coor3d,i1,i2,i3,i4,voluma)

         voluma = voluma / 6d0

!        --- Compute ratio

         call msh661( coor3d, i1, i2, i3, i4, adis )

!        --- Check for smallest?

         if ( adis<ratio ) ratio = adis
         if ( voluma<volumi ) then
            jelem = ielem
            volumi = voluma
         end if
         if ( voluma>volmax ) volmax = voluma
         vol = vol + abs(voluma)
      end do

      if ( debug ) then

!     --- Give debug information:

         write(irefwr,*) 'To repositioning with total vol = ',vol
         write(irefwr,*) 'Element',jelem,'smallest volume = ',volumi
         write(irefwr,*) 'Smallest ratio = ',ratio

!        --- consider smallest element

         if ( volumi<0d0 ) then

!        --- There are still negative parts

            write(irefwr,*) 'In mshphil3d'
            write(irefwr,*) 'Volume mshphil3d (not yet Ok!) = ',vol
         else

!        --- No negative volumes:

            write(irefwr,*) 'In mshphil3d'
            write(irefwr,*) 'Volume (mshphil3d, no neg elts) = ',vol
         end if  ! ( volumi<0 )
      end if  ! ( debug )

!     --- Repositioning, dependent on size smallest element

      if ( volumi< 10 * epsmac .and. jnew ) then

!     --- Negative volume, no repositioning

         irepos = 0
      else

!     --- Positive volume, repositioning

         irepos = 1
         inegat = 0

!        --- Check barycenters

         if ( numelm==0 .and. debug ) then
            call msh652( kmeshcloc, nelemlc, coor3d, npointlc, numelm )
         end if
      end if

!     --- No repos if more than 100 repeats:

      if ( irepos==1 .and. iherha<100 ) then

!     --- Repositioning

         if ( debug ) then
            write(irefwr,*) 'To repositioning ratio = ',ratio
            write(irefwr,*) 'and iherha = ',iherha
         end if
         leng = 3 * lenkstapl

!        --- Fill neighbour array:

         call msh619(kmeshcloc,nelemlc,npointlc,nipnt,itri,kstapl,leng)

!        --- Determine coars, use first triangle

         i1 = kmeshcloc(1,1)
         i2 = kmeshcloc(2,1)
         i3 = kmeshcloc(3,1)

!        --- Compute distances sides first triangle

         call msh603(i1,i2,coor3d,afst1)
         call msh603(i2,i3,coor3d,afst2)
         call msh603(i3,i1,coor3d,afst3)

!        --- Compute mean value

         coars = ( afst1 + afst2 + afst3 ) / 3d0

!        --- Repositioning

         call msh621( coor3d, npointlc, nipnt, itri, kstapl, coars, &
                      kmeshcloc, nelemlc )
         if ( jnew ) then

!        --- Adjust icube values (position of point may have been changed)

            do i = nipnt + 1 , npointlc
               xz = coor3d( 1, i )
               yz = coor3d( 2, i )
               zz = coor3d( 3, i )

!              --- Determine cube for this point

               n1 = int ( (xz-xstart) / dismax )
               n2 = int ( (yz-ystart) / dismax )
               n3 = int ( (zz-zstart) / dismax )

!              --- Compute cube number

               nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1

!              --- Adjust value

               icube( i ) = nc1
            end do

!           --- Reset array itri

            do i= 1 , npointlc
               itri(i) = 0
            end do
         end if
      end if

      if ( .not. jnew ) go to 1000

!     --- Check result of division

      volumb =  0
      volumi =  rinfin
      volmax = -rinfin
      vermin =  1d0
      vermax =  1d0
      adis   =  1d1

!     --- Check the elements

      do ielem = 1 , nelemlc

!     --- Find nodes element

          i1 = kmeshcloc(1,ielem)
          i2 = kmeshcloc(2,ielem)
          i3 = kmeshcloc(3,ielem)
          i4 = kmeshcloc(4,ielem)

!         --- Make choice: at least "partial" internal or
!             completely internal elements?
!             Partial:

          jcheck = max( i1, i2, i3, i4 )

!         --- Alternative full internal
!             jcheck = min( i1, i2, i3, i4 )

          call msh632(coor3d,i1,i2,i3,i4,voluma)

!         --- Volumes

          voluma = voluma / 6d0

!         --- Total sum

          volumb = volumb + abs(voluma)

!         --- min and max

          if ( voluma<volumi ) volumi = voluma
          if ( voluma>volmax ) volmax = voluma

!         --- ref value

          volref = ( chcoar(icube(i1))+chcoar(icube(i2))+ &
                     chcoar(icube(i3))+chcoar(icube(i4)) ) / 4d0

!         --- compute ratio

          verh  =  voluma / volref

!         --- check max or min

          if ( jcheck>nipnt ) then
             if ( verh<vermin ) then
                vermin = verh
                jmin   = ielem
             else if ( verh>vermax ) then
                vermax = verh
                jmax   = ielem
             end if
          else

!         --- boundary elements

             if ( verh<bndmin ) then
                 bndmin = verh
             else if ( verh>bndmax ) then
                 bndmax = verh
             end if
          end if

!         --- Check ratio

          call msh661( coor3d, i1, i2, i3, i4, ratio )

!         --- Compare

          if ( ratio<adis ) then
             adis  = ratio
             jelem = ielem
          end if
      end do

!     --- Debug message

      if ( debug ) then
         write(irefwr,*) 'After repositioning: ratio= ',adis
         write(irefwr,*) 'and volume =',volumb
      end if

!     --- Check whether mesh found is acceptable:

      if ( volumi<0d0 ) then
         inegat = inegat + 1
         if ( vermin>0.01d0 ) vermin = 0.01d0
      else if ( iherha>5 .and. adis>0.15d0 ) then

!     --- Mesh seems to be ok

         goto 900
      else if ( vermin>min(0.05d0, bndmin) .and. &
                vermax<max(20d0, bndmax) .and. &
                adis  >0.35d0 .and. iherha>5 ) then

!     --- Special case: boundary included

         if ( debug ) write(irefwr,*) 'Boundary included'
         goto 900
      end if

!     --- Regard convergence, compare vermin to factor 0.01 or 0.03, &
!         not necessary but nice

      if ( numelm==0 .and. inegat==0 ) then
         if ( ( iherha>100 .and. adis>0.01d0 ) .or. &
              ( iherha> 20 .and. adis>0.05d0 ) .or. &
              ( iherha>  5 .and. jstep>0 .and. istep>1 .and. &
                vermin>0.03d0 .and. adis>0.01d0 ) ) then

!        --- Return. Division ok but not really successfull

             call erreal ( adis, 1 )
             call errwar ( 2965, 0, 1, 0 )
             goto 900
         end if
      end if

!     --- Check max and min ratio's:

      if (vermax<  20d0 .and. iherha>25 .and. inegat==0) jmax = 0
      if (vermin>0.05d0 .and. iherha>25 .and. inegat==0) jmin = 0

!     --- Compute relative importance:

      cros = vermax * vermin

!     --- Determine element jelem

      jelem = 0
      if ( cros>2d0 .and. vermax>20d0 ) then
           jelem = jmax
      else if ( vermin<0.05d0 ) then
           jelem = jmin
      end if

!     --- If jelem = 0 and irepos = 0 then check for angles

      if ( debug .and. jelem==0 .and. irepos==0 ) then

!     --- run through all elements

         do ielem = nelemlc , 1 , -1
            i1 = kmeshcloc( 1, ielem )
            i2 = kmeshcloc( 2, ielem )
            i3 = kmeshcloc( 3, ielem )
            i4 = kmeshcloc( 4, ielem )
            if ( max( i1, i2, i3, i4 )>nipnt ) then
               call msh603( i1, i2, coor3d, adis )
               call msh603( i1, i3, coor3d, bdis )
               call msh603( i1, i4, coor3d, cdis )
               call msh603( i2, i3, coor3d, ddis )
               call msh603( i2, i4, coor3d, edis )
               call msh603( i3, i4, coor3d, fdis )

!              --- check angles

               call msh640( adis, ddis, bdis, ielem, jelem )
               call msh640( adis, edis, cdis, ielem, jelem )
               call msh640( edis, ddis, fdis, ielem, jelem )
               call msh640( bdis, fdis, cdis, ielem, jelem )
               if ( jelem/=0 ) goto 612
            end if
         end do
      end if

!     --- Check element jelem:

612   if ( jelem/=0 ) then

!     --- Nodes:

         i1 = kmeshcloc( 1, jelem )
         i2 = kmeshcloc( 2, jelem )
         i3 = kmeshcloc( 3, jelem )
         i4 = kmeshcloc( 4, jelem )

!        --- Maxnode:

         ielem = max( i1, i2, i3, i4 )

!        --- Ratio

         call msh661( coor3d, i1, i2, i3, i4, ratio )

!        --- If boundary element then angles remain the same

         if ( ielem<=nipnt ) jelem = 0
      end if

!     --- If necessary adjust element jelem

      if ( jelem==jold ) then
           jherha = jherha + 1
      else
           jold   =  jelem
           jherha =    0
      end if

!     --- Check for eternal loop

      if ( jherha>5 .and. inegat==0 ) then

!     --- Possible eternal loop: leave routine

         ratio = 1d0

!        --- Run through all elements

         do ielem = nelmfix+1, nelemlc
            i1 = kmeshcloc( 1, ielem )
            i2 = kmeshcloc( 2, ielem )
            i3 = kmeshcloc( 3, ielem )
            i4 = kmeshcloc( 4, ielem )
            call msh661( coor3d, i1, i2, i3, i4, ratio )
            if ( ratio<vermin ) vermin = ratio
         end do

!        --- Check for result:

         if ( iherha>50 .or. vermin<0 ) then
            call errint ( iherha, 1 )
            call errint ( ivolnr, 2 )
            call erreal ( vermin, 1 )
            call errwar ( 2966, 2, 1, 0 )
         end if
         if ( vermin<0 ) then
            write(irefwr,*) 'Error: Please inform SEPRA'
         end if
         goto 1000
      end if

!     --- In case of repositioning: Check all ratio's

      if ( jelem==0 ) then
         vermin = 10d0
         ratio  =  1d0
         do ielem = nelmfix+1, nelemlc
            i1 = kmeshcloc( 1, ielem )
            i2 = kmeshcloc( 2, ielem )
            i3 = kmeshcloc( 3, ielem )
            i4 = kmeshcloc( 4, ielem )
            call msh661( coor3d, i1, i2, i3, i4, ratio )
            if ( ratio<vermin ) vermin = ratio
         end do

!        --- Check whether this mesh is better:

         if ( vermin>ratiop ) then

!        --- Better mesh:

            if ( debug ) then
               write(irefwr,*) 'ratio new mesh (reposresult) ',vermin
               write(irefwr,*) 'ratio',vermin,'>old ratio',ratiop
               write(irefwr,*) 'safe the better one'
            end if

!           --- Use "best" mesh:
!               Set equal to new number of points:

            npntmp = npointlc
            if ( npntmp>npndef ) then
               write(irefwr,*) ' npntmp =',npntmp
               write(irefwr,*) ' npndef =',npndef
               write(irefwr,*) 'Error in allocation temp pointsarray'
               write(irefwr,*) 'Please inform SEPRA'
               write(irefwr,*) 'Execution stopped'
               go to 1150
            end if

!           --- Coordinates:

            do i=1,3
               coortmp(i,1:npointlc) = coor3d(i,1:npointlc)
            end do

!           --- Set number of elements:

            neltmp = nelemlc
            if ( neltmp>neldef ) then
               write(irefwr,*) 'Error in allocation temp elemarray'
               write(irefwr,*) 'neltmp =',neltmp,'neldef =',neldef
               write(irefwr,*) 'Please inform SEPRA'
               write(irefwr,*) 'Execution stopped'
               go to 1150
            end if

!           --- Elements:

            do i=1,4
               meshtmp(i,1:nelemlc ) = kmeshcloc(i,1:nelemlc)
            end do
            ratiop = vermin
         end if
      end if

      if ( jelem/=0 ) then
         if ( vermin<0.1d0 .or. inegat/=0 ) then
            call msh638(kmeshcloc,nelemlc,jelem,kstapl,kstap,nipnt)

!           --- Check used length

            leng = kstap
            if ( leng>lenkstapl ) then
               call errint ( lenkstapl, 1 )
               call errint ( leng  , 2 )
               call errint ( ivolnr, 3 )
               call errwar ( 928, 3, 0, 0 )
            end if
         else if ( iherha>100 ) then

!        --- Too many repeats and/or repositionings

            call errint ( iherha, 1 )
            call errint ( ivolnr, 2 )
            call errwar ( 927, 2, 0, 0 )
            go to 1150
         end if

!        --- Renumber kmeshcloc, kstapl, coor, to contain only used nodes

         call msh649( kmeshcloc, nelemlc, kstap, kstapl, coor3d, npointlc, &
                      nipnt, itri, icube, nx1, ny1, &
                      xstart, ystart, zstart, dismax )

!        --- Make another run for this subvolume

         nochan = 0
         goto 300
      else if (iherha<5 ) then
         goto 500
      else if ( nelemlc/=nelold .or. npointlc/=nptold ) then
         ratold = vermin
         goto 500
      else if ( vermin<0.35d0 ) then

!     --- Results of repositioning (not yet?) stable?

         if ( debug ) then
            write(irefwr,*) 'Repos not yet stable, ratio =',vermin
            write(irefwr,*) 'and old ratio =',ratold
            write(irefwr,*) 'nelemlc  = ',nelemlc ,'nelold =',nelold
            write(irefwr,*) 'npointlc = ',npointlc,'nptold =',nptold
         end if
         if ( vermin>ratold + 1d-4 ) ratold = vermin
         goto 500
      end if
 900  continue

!     --- Finally try to upgrade the mesh:

      jherha = 0

      do while ( jherha==0 )
         jherha = 1
         jelem  = nelmfix
         ja     = 0
         do while ( nelemlc>jelem )
            jelem = jelem + 1

!           --- Nodes:

            i1 = kmeshcloc(1,jelem)
            i2 = kmeshcloc(2,jelem)
            i3 = kmeshcloc(3,jelem)
            i4 = kmeshcloc(4,jelem)

!           --- Ratio:

            call msh661(coor3d,i1,i2,i3,i4,ratio)

!           --- Again try to improve:

            if ( ratio<0.7d0 .and. jelem>nelmfix ) then

!           --- Set number of nodes: np

               np = npointlc
               call msh665( kmeshcloc, nelemlc, nelmfix, jelem, ja, &
                            coor3d, npointlc )
               if ( np/=npointlc ) then

!              --- Adjusted mesh, adjust cube numbers:

                  do j = 1, npointlc

!                 --- Find cubenumber for each node:

                  n1 = int ( (coor3d(1,j)-xstart) / dismax )
                  n2 = int ( (coor3d(2,j)-ystart) / dismax )
                  n3 = int ( (coor3d(3,j)-zstart) / dismax )

!                 --- Determine cube number:

                  nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1
                  icube(j) = nc1
                  end do
               end if
            end if
            if ( ja==1 .or. ja==2 ) jherha = 0
         end do
      end do
1000  if ( ierror/=0 ) go to 1100

!     --- Last step, try once more to improve the elements

      jelem = nelmfix

!     --- Run again through all non-fixed elements

      do while ( nelemlc>jelem )
         jelem = jelem + 1
         i1 = kmeshcloc(1,jelem)
         i2 = kmeshcloc(2,jelem)
         i3 = kmeshcloc(3,jelem)
         i4 = kmeshcloc(4,jelem)
         call msh661(coor3d,i1,i2,i3,i4,ratio)
         if ( ratio<0.5d0 ) then

!        --- Set number of nodes: np

            np = npointlc
            if ( ratio<0.1d0 ) then
               call msh663( coor3d, nipnt, npointlc, kmeshcloc, nelemlc, jelem)
            end if
            if ( np==npointlc ) then
                 call msh665( kmeshcloc, nelemlc, nelmfix, jelem, ja, &
                              coor3d, npointlc )
            end if
            if ( np/=npointlc ) then

!           --- Determine corresponding node for each triangle:

               do j = 1, npointlc
                  n1 = int ( (coor3d(1,j)-xstart) / dismax )
                  n2 = int ( (coor3d(2,j)-ystart) / dismax )
                  n3 = int ( (coor3d(3,j)-zstart) / dismax )

                  nc1 = 1 + n1 + n2*nx1 + n3*nx1*ny1
                  icube(j) = nc1
               end do
            end if
         end if
      end do

!     --- Compute smallest ratio:

      vermin = 1d0
      do ielem=1, nelemlc
         i1 = kmeshcloc(1,ielem)
         i2 = kmeshcloc(2,ielem)
         i3 = kmeshcloc(3,ielem)
         i4 = kmeshcloc(4,ielem)
         call msh661(coor3d,i1,i2,i3,i4,ratio)
         if ( ratio<vermin ) vermin = ratio
      end do

1050  continue ! Also used as emergency exit
      if (  .not.  jnew  ) goto 1100
      if ( ratiop>vermin ) then

!     --- Use the saved mesh as final mesh, because it is
!         the best one found

         if ( debug ) write(irefwr,*) 'Use mesh saved, ratio =',ratiop

!        --- Coordinates:

         npointlc = npntmp
         do i=1,3
            coor3d(i,1:npointlc) = coortmp(i,1:npointlc)
         end do

!        --- Elements:

         nelemlc = neltmp
         do i=1,4
            kmeshcloc(i,1:nelemlc ) = meshtmp(i,1:nelemlc)
         end do

!        --- Skip boundary elements

         kstap = 0
      end if

!     --- For debugging:

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshphil3d, repeats: ',iherha
         write(irefwr,*) 'number of boundary points nipnt ',nipnt
         write(irefwr,*) 'number of nodes in mesh is ',npointlc
         write(irefwr,*) 'number of elements is ',nelemlc

         if ( kstap>0 ) then

!        --- kstap>0 no final solution for kstapl array:

            write(irefwr,*) 'Kstap/=0: Error. Boundary elements: '
            do i = 1, kstap
               i1 = kstapl( 1,i )
               i2 = kstapl( 2,i )
               i3 = kstapl( 3,i )
               write(irefwr,*) i,i1,i2,i3
            end do
            write(irefwr,*) 'And coordinates'
            do i=1, npointlc
               if ( itri(i)>0 ) then
                  xn = coor3d(1,i)
                  yn = coor3d(2,i)
                  zn = coor3d(3,i)
                  write(irefwr,*) i,xn,yn,zn
               end if
            end do
            call msh637 ( kstapl, kstap, line, line(kstap+11) )
            write(irefwr,*) 'After msh637: ierror = ',ierror
         end if

         volumb =  0

         volumi =  rinfin
         volmax = -rinfin

         vermin =  1d0
         vermax =  1d0

!        --- Check the elements

         do ielem = 1 , nelemlc

            i1 = kmeshcloc(1,ielem)
            i2 = kmeshcloc(2,ielem)
            i3 = kmeshcloc(3,ielem)
            i4 = kmeshcloc(4,ielem)

            call msh661(coor3d,i1,i2,i3,i4,ratio)
            if ( ratio<0.1d0 ) then
               write(irefwr,*) 'element ',ielem,' ratio = ',ratio
               write(irefwr,*) 'nodes',i1,i2,i3,i4
            end if

!           --- Compute voluma of element

            call msh632(coor3d,i1,i2,i3,i4,voluma)

!           --- Check for negative volume:

            if ( voluma<0d0 ) then
               write(irefwr,*) 'vol element ',ielem,'=',voluma
            end if

            jcheck = min(i1,i2,i3,i4)

            voluma = voluma / 6d0

            volumb = volumb + abs(voluma)

            if ( voluma<volumi ) volumi = voluma
            if ( voluma>volmax ) volmax = voluma

            volref = ( chcoar(icube(i1))+chcoar(icube(i2))+ &
                       chcoar(icube(i3))+chcoar(icube(i4)) ) / 4d0

            verh  =  voluma / volref

            if ( verh  <vermin ) then
                 vermin = verh
                 jmin   = ielem
            end if

            if ( verh  >vermax ) then
                 vermax = verh
                 jmax   = ielem
            end if

         end do
         write(irefwr,*) 'At the end of mshphil3d volume =',volumb

!        --- Check quality of tets

         call mshchecktets ( 11, 4, kmeshcloc, nelemlc, ivolnr, coor3d )
      end if

!     --- End local debugging
!         Make backtransformation for the nodes:

      coor3d(1,1:npointlc) = (coor3d(1,1:npointlc) - 1d0)*tran + xmint
      coor3d(2,1:npointlc) = (coor3d(2,1:npointlc) - 1d0)*tran + ymint
      coor3d(3,1:npointlc) = (coor3d(3,1:npointlc) - 1d0)*tran + zmint

      if ( nelemlc>6*npointlc - 3*nipnt .and. nelmfix==0 ) then
         call errint ( nelemlc, 1 )
         call errint ( npointlc-nipnt, 2 )
         call errint ( npointlc, 3 )
         call errint ( nipnt, 4 )
         call errint ( nbound, 5 )
         call errint ( 6*npointlc-3*nipnt, 6 )
         call errint ( 6*npointlc-3*nipnt-nelemlc, 7 )
         call errsub ( 2967, 7, 0, 0 )
      end if

!     --- Deallocate local help arrays:

      deallocate( cube, jcube, icube, chcoar, kbnold, stat = error )
      if ( error>0 ) then
         call erdealloc ( error, 'cube' )
         go to 1150
      end if

!     --- Check the last mesh and delete the temporary help arrays for
!         coor and mesh

      if ( neltmp>0 ) then

!     --- Check

         deallocate( coortmp, stat = error )
         if ( ierror>0 ) then
            write(irefwr,*) 'deallocation error coortmp'
            go to 1150
         end if
         deallocate( meshtmp, stat = ierror )
         if ( error>0 ) then
            call erdealloc ( error, 'meshtmp' )
            go to 1150
         end if
      end if
      if ( itime==1 ) &
           call printtime ( 'Mshphil3d')
      debug = .false.
      if ( debug )  then

!     --- Debug information

         write(irefwr,*) 'Coordinates of ',npointlc,' nodes'
         do i=1, npointlc
            write(irefwr,*) i,(coor3d(j,i),j=1,3)
         end do
         write(irefwr,*) 'In total',nelemlc,'Elements'
         do i=1, nelemlc
            write(irefwr,*) i,(kmeshcloc(j,i),j=1,4)
         end do
      end if

      if ( debug ) then

!     --- Check positioning of elements

         numelm = 0
         write(irefwr,*) 'Check barycenters of all elements'
         call msh652 (kmeshcloc, nelemlc, coor3d, npointlc, numelm)
         if ( numelm>0 ) then
            write(irefwr,*) 'numelm=',numelm
            write(irefwr,*) 'Barycenter error; Stop'
            go to 1150
         end if
         write(irefwr,*) 'End debug information mshphil3d'
      end if

!     --- Test Delaunay for new ideas:

      if ( debug ) then
         write(irefwr,*) '01-12-2010:Extra Delaunaytest'
         write(irefwr,*) 'To find new ideas at end mshphil3d'
         write(irefwr,*) 'Start with old tetrahedrons'
         ratiob = 1d0

         do i = 1, nelemlc
            i1 = kmeshcloc(1,i)
            i2 = kmeshcloc(2,i)
            i3 = kmeshcloc(3,i)
            i4 = kmeshcloc(4,i)
            call msh661(coor3d,i1,i2,i3,i4,ratio)

            if ( ratio<ratiob ) ratiob = ratio

            if ( debug ) then
               write(irefwr,*) 'Elmnt',i,'has ratio =', ratio
               write(irefwr,*) 'Points',(kmeshcloc(j,i),j=1,4)
            end if
         end do
         write(irefwr,*) 'smallest ratio in classical mesh:',ratiob
         kstap  = nbound
         icall  = 0

!        --- Find new division in elements

         do while ( kstap>0 )

!        --- Use original boundary array and fill kstapl:

            kstap = nbound

            do i=1, nbound
            do j=1, 3
               kstapl(j,i) = kbndpt(j,i)
            end do
            end do

!           --- Start from "scratch"

            nelemlc = 0
            icall = icall + 1

!           --- Call Delaunay routine:

            if ( icall>5 ) then
               write(irefwr,*) 'icall>5:No solution by Delaunay476/670'
               write(irefwr,*) 'Please inform SEPRA'
               go to 1150
            end if
            if ( debug ) then
               write(irefwr,*) 'nodes in boundary for msh670'
               write(irefwr,*) 'number of nodes is',npointlc
               do i=1, npointlc
                  x1 = coor3d(1,i)
                  y1 = coor3d(2,i)
                  z1 = coor3d(3,i)
                  write(irefwr,222) 'Node',i,'crds',x1,y1,z1
               end do
               write(irefwr,*) 'Contents kstapl just before 670'
               write(irefwr,*) 'number of boundary triangles is',kstap
               do i=1, kstap
                  i1 = kstapl(1,i)
                  i2 = kstapl(2,i)
                  i3 = kstapl(3,i)
                  write(irefwr,*) 'elmnt',i,'nodes',i1,i2,i3
               end do
            end if

!           --- Check ordering of kstapl

            x1 = (xstart - 1d0)*tran + xmint

!           --- Use this startvalue

            call msh608 ( kstapl, kstap, coor3d, x1 )

!           --- Show new kstapl

            if ( debug ) then
               write(irefwr,*) 'kstapl after msh608 just before 670'
               write(irefwr,*) 'number of boundary triangles is',kstap
               do i=1, kstap
                  i1 = kstapl(1,i)
                  i2 = kstapl(2,i)
                  i3 = kstapl(3,i)
                  write(irefwr,*) 'elmnt',i,'nodes',i1,i2,i3
               end do
            end if

!           --- Make choice for routine

            if ( 0>1 ) then
               call msh670 ( kstap, kstapl, coor3d, npointlc, &
                             nelemlc, kmeshcloc, itri, line, lenline, &
                             nipnt , x1, xn, yn, zn, ivolnr, xminvec )
            else
               call msh476 ( kstap, kstapl, coor3d, npointlc, &
                             nelemlc, kmeshcloc, itri, &
                             lenline, nipnt, ivolnr, xminvec )
            end if

            if ( ierror/=0 ) go to 1150

!           --- Give messages on results:

            if ( kstap>0 ) then
               write(irefwr,*) 'After msh476/670, kstap =',kstap
               write(irefwr,*) 'remaining triangles'
               do i = 1, kstap
                  i1 = kstapl(1,i)
                  i2 = kstapl(2,i)
                  i3 = kstapl(3,i)
                  write(irefwr,*) i,i1,i2,i3
               end do
               write(irefwr,*) 'add new node'
               i = nint ( (kstap+1) / 2d0 )
               i1 = kstapl(1,i)
               i2 = kstapl(2,i)
               i3 = kstapl(3,i)
               write(irefwr,*) i,i1,i2,i3

!              --- Add all barycenters of remaining boundary triangles:

               call msh609(coor3d,i1,i2,i3,e1,e2,e3,xm,ym,zm)
               npointlc = npointlc + 1
               coor3d(1,npointlc) = xm
               coor3d(2,npointlc) = ym
               coor3d(3,npointlc) = zm
               write(irefwr,*) npointlc,'Crds',xm,ym,zm
               write(irefwr,*) 'Nu is npointlc =',npointlc
            end if
            if ( debug .and. kstap==0 ) then

!           --- Check result:

               numelm = 0
               call msh652 (kmeshcloc, nelemlc, coor3d, npointlc, numelm)
               write(irefwr,*) 'Controle msh652, kstap=',kstap
               write(irefwr,*) 'numelm = ',numelm
               write(irefwr,*) 'nelemlc = ',nelemlc,' and npointlc =',npointlc
               write(irefwr,*) 'Compute the volume, callnr =',icall

               volumb = 0
               ratiob = 1d0

               do ielem = 1 , nelemlc

!              --- Nodes

                  i1 = kmeshcloc(1,ielem)
                  i2 = kmeshcloc(2,ielem)
                  i3 = kmeshcloc(3,ielem)
                  i4 = kmeshcloc(4,ielem)

!                 --- Compute volume and ratio

                  call msh632(coor3d,i1,i2,i3,i4,voluma)
                  call msh661(coor3d,i1,i2,i3,i4,ratio)
                  if ( ratio<ratiob ) ratiob = ratio
                  if ( ratio<0.01d0 ) then
                     write(irefwr,*) 'element ',ielem,' ratio = ',ratio
                     write(irefwr,*) 'nodes',i1,i2,i3,i4
                  end if

!                 --- Neg volume give message:

                  if ( voluma<0d0 ) then
                     write(irefwr,*) 'vol element ',ielem,'=',voluma
                  end if
                  volumb = volumb + abs(voluma)
               end do
               volumb = volumb / 6d0
               write(irefwr,*) 'After Delaunay, volume =',volumb
            end if
         end do

!        --- Check quality of tets

         call mshchecktets ( 11, 4, kmeshcloc, nelemlc, ivolnr, coor3d )
         write(irefwr,*) 'End extra qualitytest after Delaunay'
      end if

!     --- Extra repositioning at the end:

      j = 0
      ratiob = 0d0
      do while ( ratiob<0.6d0 .and. j<5 )
         if ( debug ) write(irefwr,*) 'Quality test run =',j
         j     = j + 1
         jelem = 0

!        --- Loop:

         do while ( nelemlc>jelem )
            jelem = jelem + 1

!           --- Nodes:

            i1 = kmeshcloc(1,jelem)
            i2 = kmeshcloc(2,jelem)
            i3 = kmeshcloc(3,jelem)
            i4 = kmeshcloc(4,jelem)

!           --- Ratio:

            call msh661(coor3d,i1,i2,i3,i4,ratio)

!           --- Redivision by msh665?

            if ( ratio<0.7d0 .and. jelem>nelmfix ) then

!           --- Set number of nodes: np

               np = npointlc
               call msh665( kmeshcloc, nelemlc, nelmfix, jelem, ja, &
                            coor3d, npointlc )
            end if
         end do

         if ( debug ) write(irefwr,*) 'Compute volume and ratio again'

         volumb = 0
         ratiob = 1d0

         do ielem = 1 , nelemlc

            i1 = kmeshcloc(1,ielem)
            i2 = kmeshcloc(2,ielem)
            i3 = kmeshcloc(3,ielem)
            i4 = kmeshcloc(4,ielem)

            call msh661(coor3d,i1,i2,i3,i4,ratio)
            if ( ratio<ratiob ) ratiob = ratio
            if ( debug .and. ratio<1d-3 ) then
               write(irefwr,*) 'element ',ielem,' ratio = ',ratio
               write(irefwr,*) 'nodes',i1,i2,i3,i4
            end if
            call msh632(coor3d,i1,i2,i3,i4,voluma)
            if ( voluma<0d0 ) then
               write(irefwr,*) 'vol element ',ielem,'=',voluma
            end if
            volumb = volumb + abs(voluma)
         end do
         volumb = volumb / 6d0
         if ( debug ) then
            write(irefwr,*) 'After msh665 and before repos'
            write(irefwr,*) 'Volume =',volumb,'ratio = ',ratiob
         end if

         leng = 3 * lenkstapl
         call msh619(kmeshcloc,nelemlc,npointlc,nipnt,itri,kstapl,leng)

!        --- Determine value for coars

         i1 = kmeshcloc(1,1)
         i2 = kmeshcloc(2,1)
         i3 = kmeshcloc(3,1)

!        --- Compute sidelengths of the edges of the triangle

         call msh603(i1,i2,coor3d,afst1)
         call msh603(i2,i3,coor3d,afst2)
         call msh603(i3,i1,coor3d,afst3)

!        --- Mean value

         coars = ( afst1 + afst2 + afst3 ) / 3d0

!        --- Perform repositioning

         call msh621( coor3d, npointlc, nipnt, itri, kstapl, coars, &
                      kmeshcloc, nelemlc )

!        --- Check result after repositioning

         volumb = 0
         ratiop = 1d0
         do ielem = 1 , nelemlc

!        --- Nodes:

            i1 = kmeshcloc(1,ielem)
            i2 = kmeshcloc(2,ielem)
            i3 = kmeshcloc(3,ielem)
            i4 = kmeshcloc(4,ielem)

!           --- ratio of this element:

            call msh661(coor3d,i1,i2,i3,i4,ratio)
            if ( ratio< ratiop ) ratiop = ratio
            if ( debug .and. ratio<=1d-3 ) then
               write(irefwr,*) 'After repos ',ielem,' ratio = ',ratio
               write(irefwr,*) 'nodes',i1,i2,i3,i4
            end if

!           --- volume of this element

            call msh632(coor3d,i1,i2,i3,i4,voluma)
            if ( voluma<0d0 ) then
               write(irefwr,*) 'Neg vol element ',ielem,'=',voluma
            end if
            volumb = volumb + abs(voluma)
         end do
         volumb = volumb / 6d0
         if ( debug ) then
            write(irefwr,*) 'After repos volume = ',volumb
            write(irefwr,*) 'smallest ratio = ',ratiop
            write(irefwr,*) 'Next extra calls to msh665'
         end if

!        --- Extra runs:

         jelem = 0

!        --- Do until etc

         do while ( nelemlc>jelem )
            jelem = jelem + 1

!           --- determine nodes element

            i1 = kmeshcloc(1,jelem)
            i2 = kmeshcloc(2,jelem)
            i3 = kmeshcloc(3,jelem)
            i4 = kmeshcloc(4,jelem)

!           --- compute ratio

            call msh661(coor3d,i1,i2,i3,i4,ratio)

!           --- check value

            if ( ratio<0.7d0 .and. jelem>nelmfix ) then

!           --- Set number of nodes: np

               np = npointlc
               call msh665( kmeshcloc, nelemlc, nelmfix, jelem, ja, &
                            coor3d, npointlc )
            end if
         end do

!        --- Results after application of msh665:

         if ( debug ) &
            write(irefwr,*) 'Compute ratio and volume after msh665'
         volumb = 0
         ratiop = 1d0
         do ielem = 1 , nelemlc
            i1 = kmeshcloc(1,ielem)
            i2 = kmeshcloc(2,ielem)
            i3 = kmeshcloc(3,ielem)
            i4 = kmeshcloc(4,ielem)
            call msh661(coor3d,i1,i2,i3,i4,ratio)
            if ( debug .and. ratio<ratiop ) ratiop = ratio
            if ( debug .and. ratio<1d-3 ) then
               write(irefwr,*) 'Element ',ielem,' ratio = ',ratio
               write(irefwr,*) 'nodes',i1,i2,i3,i4
            end if
            call msh632(coor3d,i1,i2,i3,i4,voluma)
            if ( voluma<0d0 ) then
               write(irefwr,*) 'vol element ',ielem,'=',voluma
            end if
            volumb = volumb + abs(voluma)
         end do
         volumb = volumb / 6d0
      end do ! repos
      if ( debug ) then
         write(irefwr,*) 'Before renumbering and transformation'
         write(irefwr,*) 'Volume =',volumb,' ratio =',ratiop
      end if

!     --- Finally renumber kmeshcloc, kstapl and coor3d, in order to contain
!         only nodenumbers that are really used:

      call msh649( kmeshcloc, nelemlc, kstap, kstapl, coor3d, npointlc, nipnt, &
                   itri, itri, nx1, ny1, xstart, ystart, zstart, dismax )

!     --- Final information

      if ( debug .or. 1<0 ) then
         write(irefwr,*) 'End mshphil3D: Elements nelemlc=',nelemlc
         write(irefwr,*) 'Number of nodes npointlc=',npointlc
         write(irefwr,*) 'Coordinates found'
         do i=1, npointlc
            x1 = coor3d(1,i)
            y1 = coor3d(2,i)
            z1 = coor3d(3,i)
            write(irefwr,222) 'Node',i,'crds',x1,y1,z1
         end do
         write(irefwr,*) 'Elements found'
         ratiom = 1d0
         do ielem = 1 , nelemlc
            i1 = kmeshcloc(1,ielem)
            i2 = kmeshcloc(2,ielem)
            i3 = kmeshcloc(3,ielem)
            i4 = kmeshcloc(4,ielem)
            call msh661(coor3d, i1, i2, i3, i4, ratio )
            if ( ratio<ratiom ) ratiom = ratio
            write(irefwr,223) 'El nodes',i1,i2,i3,i4,'ratio =',ratio
         end do
         write(irefwr,*) 'Smallest ratio is',ratiom
222      format(a,(i8),1x,a,(3f15.8))
223      format(a,(4i8),1x,a,f15.8)
      end if
1100  continue

!     --- Finally deallocate the last help arrays:

      deallocate( kstapl, itri, line, userco, stat = error )
      if ( error>0 ) call erdealloc ( error, 'kstapl' )

1150  call erclos ( 'mshphil3d' )

!     --- For debugging results:

      debug = .false.
      if ( debug ) then
         write(irefwr,*) 'Date 07 - january - 2011'
         write(irefwr,*) 'Information 3d meshgeneration MSHPHIL3D:'
         write(irefwr,*) 'Number of iterations:',iherha
         volumb = 0D0
         ratiop = 1D0
         ratiom = 0D0
         do ielem = 1 , nelemlc
            i1 = kmeshcloc(1,ielem)
            i2 = kmeshcloc(2,ielem)
            i3 = kmeshcloc(3,ielem)
            i4 = kmeshcloc(4,ielem)
            call msh661(coor3d,i1,i2,i3,i4,ratio)
            ratiom = ratiom + ratio
            if ( ratio<ratiop ) ratiop = ratio
            call msh632(coor3d,i1,i2,i3,i4,voluma)
            volumb = volumb + abs(voluma)
         end do

!        --- Compute meanratio:

         ratiom = ratiom/nelemlc
         write(irefwr,*) 'End of routine after backtransformation'
         write(irefwr,*) 'Finally: all tet-ratios are>=',ratiop
         write(irefwr,*) 'meanvalue for all ratio values= ',ratiom
         write(irefwr,*) 'and sum of all tet-volumes  is ',volumb/6d0
      end if

!     --- Print time used:

      if ( itime>0 ) call printtime ( 'mshphil3d')

!     --- Leave routine:

      end
