      subroutine msh635 ( kbound, kbond, nextracrvs, &
                          numnodextcurvsl, itri, kstapl, coor, &
                          coorcurvs, istart, jstart )
! ======================================================================
!
!        programmer    Wendy Swolfs
!        version  2.0  date 08-09-2010 New release
!        version  1.0  date 13-05-2009
!
!   copyright (c) 2009-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to insert the intermediate nodes of internal
!     curves after the generation back into the quadratic boundary
!     elements
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     tetrahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbound(*), kbond(*), nextracrvs, itri(*), &
              numnodextcurvsl(*), kstapl(*), istart, jstart
      double precision coor(3,*), coorcurvs(3,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect
!                         to the global numbering
!     coorcurvs      i    array containing the coordinates of the nodes on the
!                         internal curves
!     istart        i/o   Starting address
!     itri           i    helparray of length npoint, containing information
!                         whether a nodal point is in the actual
!                         boundary or not
!     jstart        i/o   starting address in array kbound
!     kbond          i    help array for linear boundary triangles
!     kbound         i    array containing the boundary triangles
!     kstapl              help array for temporary boundary elements
!     nextracrvs     i    number of extra internal curves
!     numnodextcurvsl i   array with number of nodes on internal curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numnodes, icurnr, iel, i1, i2, i3, i, kstart

!     i              Counting variable
!     i1             Node number
!     i2             Node number
!     i3             Node number
!     icurnr         Curve sequence number
!     iel            Element number
!     kstart         Starting address
!     numnodes       Number of nodes of an internal curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      kstart = 0

!     --- Run through all internal curves

      do icurnr = 1, nextracrvs

!     --- Find length of curve

         numnodes = numnodextcurvsl(icurnr)

!        --- Run through all nodes

         do iel = 1, numnodes - 1

!        --- Find begin and end node

            i1 = kbond(istart + 1)
            i3 = kbond(istart + 2)

!           --- Place new node i2:

            call msh629( i1, i2,  i3, itri, kstapl )

!           --- Set all values in array kbound

            kbound( jstart + 1 ) = i1
            kbound( jstart + 2 ) = i2
            kbound( jstart + 3 ) = i3

!           --- Set coordinates i2 back to coordinates of curve

            do i = 1, 3
               coor( i, i2 ) =  coorcurvs( i, kstart + 2)
            end do

!           --- Jump to next positions

            istart = istart + 1
            jstart = jstart + 2
            kstart = kstart + 2
         end do

!        --- Jump to next internal curve

         istart = istart + 1
         jstart = jstart + 1
         kstart = kstart + 1
      end do

      end
