      subroutine msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                          ia, ib, ic, ik, il, im, is1, is2 )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 14-10-2010 New call
!        version  1.0  date 18-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine whether the lines of triangle with nodes
!     (ia, ib, ic) are disjunct with triangle (ik, il, im) except for edges
!     and nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     3d
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivolnr, itri1, itri2, ia, ib, ic, ik, il, im, is1, is2, &
              iprob
      double precision coor(3,*)

!     coor      i    standard SEPRAN coordinate array
!     ivolnr    i    Volume sequence number
!     ia        i    node of triangle and line
!     ib        i    node of triangle and line
!     ic        i    node of triangle and line
!     ik        i    node of triangle
!     il        i    node of triangle
!     im        i    node of triangle
!     iprob     o    error indicator: if 0 mesh is ok, &
!                    if>0: node iprob is wrong node
!     is1       i    triangle number first triangle (for error messages)
!     is2       i    triangle number second triangle (for error messages)
!     itri1     i    indicator for subsurfacegroup (SEPRAN counting)
!     itri2     i    indicator for subsurfacegroup (SEPRAN counting)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iha, ihb, ihc
      double precision t

!     iha            indicator for coinciding nodes
!     ihb            indicator for coinciding nodes
!     ihc            indicator for coinciding nodes
!     t              relative position of common point of line and triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH648         Computes intersection coordinate of line and triangle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     950   There is an intersection of two triangles
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check each of the three lines for intersection with the triangle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh657' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh657'
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Start assuming no problemnodes

      iprob = 0

!     --- Set controle values:

      iha = 0
      ihb = 0
      ihc = 0

!     --- Find situation (no double nodes?):

      if ( ia==ik .or. ia==il .or. ia==im ) iha = 1
      if ( ib==ik .or. ib==il .or. ib==im ) ihb = 1
      if ( ic==ik .or. ic==il .or. ic==im ) ihc = 1
      if ( iha==0 .and. ihb==0 ) then
         call msh648( ia,ib, ik,il,im, coor, t )
         if ( 0<t .and. t<1d0 ) iprob = iprob + 1
      end if
      if ( ihb==0 .and. ihc==0 ) then
         call msh648( ib,ic, ik, il, im, coor, t )
         if ( 0<t .and. t<1d0 ) iprob = iprob + 10
      end if
      if ( ihc==0 .and. iha==0 ) then
         call msh648( ic,ia, ik, il, im, coor, t )
         if ( 0<t .and. t<1d0 ) iprob = iprob + 100
      end if

1000  call erclos ( 'msh657' )
      if ( debug .and. iprob>0 ) then

!     --- Debug information

         write(irefwr,*) 'End msh657, node with error =',iprob
         write(irefwr,*) 'Basistri',is1,'kn',ia,ib,ic
         if ( iprob==1 ) then
            write(irefwr,*) 'line',ia,'- ',ib
         else if ( iprob==10 ) then
            write(irefwr,*) 'line',ib,'- ',ic
         else if ( iprob==11 ) then
            write(irefwr,*) 'lines',ia,'- ',ib
            write(irefwr,*) 'and  ',ib,'- ',ic
         else if ( iprob==100 ) then
            write(irefwr,*) 'line',ic,'- ',ia
         else if ( iprob==101 ) then
            write(irefwr,*) 'lines',ia,'- ',ib
            write(irefwr,*) 'and  ',ic,'- ',ia
         else if ( iprob==110 ) then
            write(irefwr,*) 'lines',ib,'- ',ic
            write(irefwr,*) 'and  ',ic,'- ',ia
         end if
         write(irefwr,*) 'intersects with tri',is2,'nodes',ik,il,im
      end if  ! ( debug )

      end
