      subroutine msh658 ( i1, coor, ik, il, im, in, io, ip, ja )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 14-10-2010 New release
!        version  1.0  date 19-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if node i1 is inside "element" ik,il,im,in,io,ip
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
!     inside
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i1, ik, il, im, in, io, ip, ja
      double precision coor(3,*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     i1         i    cornernode to be considered
!     ik         i    node of quadratic triangle to be considered
!     il         i    node of quadratic triangle to be considered
!     im         i    node of quadratic triangle to be considered
!     in         i    node of quadratic triangle to be considered
!     io         i    node of quadratic triangle to be considered
!     ip         i    node of quadratic triangle to be considered
!     ja         o    indicator whether i1 is inside
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision volume, xa, ya, za, xb, yb, zb,  t, xinf, &
                       x1, x2, x3, x4, x5, x6, &
                       y1, y2, y3, y4, y5, y6, &
                       z1, z2, z3, z4, z5, z6

!     t           helpvariable
!     volume      help to store volume of tetrahedron
!     x1          x-coordinate
!     x2          x-coordinate
!     x3          x-coordinate
!     x4          x-coordinate
!     x5          x-coordinate
!     x6          x-coordinate
!     xa          x-coordinate
!     xb          x-coordinate
!     xinf        x value 'infinity'
!     y1          y-coordinate
!     y2          y-coordinate
!     y3          y-coordinate
!     y4          y-coordinate
!     y5          y-coordinate
!     y6          y-coordinate
!     ya          y-coordinate
!     yb          y-coordinate
!     z1          z-coordinate
!     z2          z-coordinate
!     z3          z-coordinate
!     z4          z-coordinate
!     z5          z-coordinate
!     z6          z-coordinate
!     za          z-coordinate
!     zb          z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH610         Determine intersection of triangle and line
!     MSH632         compute volume of tetrahedron
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Consider line from node i1 to "infinity".
!     Check number of crossings of line with the eight surfaces that
!     constitute the volume together
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     None
! ======================================================================
!
!     --- Set the variable xinf:

      xinf   = -1000000.10d0

!     --- Run through all eight "faces" of the quadratic element and
!         check for line i1 to infinity

!         --- Determine coordinates node i1:

      xb = coor( 1, i1 )
      yb = coor( 2, i1 )
      zb = coor( 3, i1 )

!     --- Determine node "infinity"

      xa =  xinf
      ya =  yb + 5.55551d0
      za =  zb + 6.66661d0

!     --- Determine coordinates of all nodes:

      x1 = coor( 1, ik )
      y1 = coor( 2, ik )
      z1 = coor( 3, ik )

      x2 = coor( 1, il )
      y2 = coor( 2, il )
      z2 = coor( 3, il )

      x3 = coor( 1, im )
      y3 = coor( 2, im )
      z3 = coor( 3, im )

      x4 = coor( 1, in )
      y4 = coor( 2, in )
      z4 = coor( 3, in )

      x5 = coor( 1, io )
      y5 = coor( 2, io )
      z5 = coor( 3, io )

      x6 = coor( 1, ip )
      y6 = coor( 2, ip )
      z6 = coor( 3, ip )

!     --- Check common points line a - b with the eight faces

      t = - 1000d0
      ja = 0

!     --- Face 1 - 3 - 5

      call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x3,y3,z3, &
                        x5,y5,z5,t)
      if ( t<1d0 ) ja = ja + 1

!     --- Face 1 - 5 - 6

      call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x5,y5,z5, &
                        x6,y6,z6,t)
      if ( t<1d0 ) ja = ja + 1

!     --- Face 1 - 2 - 3

      call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x2,y2,z2, &
                  x3,y3,z3,t)
      if ( t<1d0 ) ja = ja + 1

!     --- Face 3 - 4 - 5

      call msh610(xa,ya,za,xb,yb,zb,x3,y3,z3,x4,y4,z4, &
                        x5,y5,z5,t)
      if ( t<1d0 ) ja = ja + 1

!     --- Face 1 - 6 - 2

      call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x6,y6,z6, &
                        x2,y2,z2,t)
      if ( t<1d0 ) ja = ja + 1

!     --- Face 3 - 2 - 4

      call msh610(xa,ya,za,xb,yb,zb,x3,y3,z3,x2,y2,z2, &
                        x4,y4,z4,t)
      if ( t<1d0 ) ja = ja + 1

!     --- Face 5 - 4 - 6

      call msh610(xa,ya,za,xb,yb,zb,x5,y5,z5,x4,y4,z4, &
                  x6,y6,z6,t)
      if ( t<1d0 ) ja = ja + 1

!     --- Face 6 - 4 - 2

      call msh610(xa,ya,za,xb,yb,zb,x6,y6,z6,x4,y4,z4, &
                        x2,y2,z2,t)
      if ( t<1d0 ) ja = ja + 1

!     --- Give message:

      if ( ja==1.or.ja==3.or.ja==5 ) then

         if ( ja==300 ) then

!        --- For debugging:
!            Odd number of intersections, point i1 is inside

             write(irefwr,*) 'Node',i1,'inside',ik,il,im,in,io,ip
             call msh632( coor, ik, il, im, io, volume)
             write(irefwr,*) 'Volume',ik,il,im,io,'vol = ',volume
             call msh632( coor, in, io, im, il, volume)
             write(irefwr,*) 'Volume',in,io,im,il,'vol = ',volume
         end if
         ja = 1
      else if ( ja==2 ) then

!     --- Even number: not inside

         ja = 0
      end if

!     --- Ready msh658

      end
