      subroutine sepbuildalloc ( kprob, larray, liarry, iboun )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Allocate temporary arrays in for subroutine build
!
! **********************************************************************
!
!                       KEYWORDS
!
!     allocate
!     mass_matrix
!     right_hand_side
!     stiffness_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: kprob(*)
      integer, intent(out) :: larray, liarry, iboun

!     iboun          o    boundary element group number
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     larray         o    length of array coeffarray in element subroutines
!     liarry         o    length of the integer array icoeffarray
!! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenindex3, lenindex4, lenarray, error, num
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     lenarray       Length of array coeffarr
!     lenindex3      Length of array index3
!     lenindex4      Length of array index4
!     num            Number of element vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMGRINF       Fill general information for a specific element group
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuildalloc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuildalloc'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!        --- Special actions for negative type numbers

      liarry = 1
      larray = 1
      if ( itype<0 ) then
         ielrou = -itype
         if ( nbcons>0) then
            iboun = ielgrp-nelgrp-numnat+nbcons
            if ( iboun>0) iboun = kprob(ipkprb+kprobo+iboun-1)
         end if
         icount = inpelm

      else

!     --- Compute some parameters depending on the type number

         call elmgrinf ( larray, liarry, kmeshm, ndim )

      end if
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,*) 'after elmgrinf'
         write(irefwr,1) 'icount/5', icount
      end if  ! ( debug )

!        --- allocate arrays for element subroutines

      lenarray = larray+7425

      jmethod = intmatinf(2)

      if ( jmethod>=13 .and. jmethod<=16 .or. &
           jmethod>=24 .and. jmethod<=27 ) then

!     --- In case of block preconditioners for the Navier-Stokes elements
!         or shifted Laplace preconditioner, extra space
!         we need extra positions to store some extra matrices

         idefcr = idefcr+1

      end if  ! ( jmethod>=13 .and. jmethod<=16 ... )

      lenindex4 = max(numold,1)*(1+inpelm)
      if ( debug ) then
         write(irefwr,1) 'larray, liarry, lenarray', &
                          larray, liarry, lenarray
         write(irefwr,1) 'inpelm, icount', &
                          inpelm, icount
         write(irefwr,1) 'lenindex4', lenindex4
      end if  ! ( debug )
      error = 0
      allocate ( coeffarr(lenarray), stat=error )
      if ( error/=0 )  call eralloc ( error, lenarray, 'coeffarr' )
      allocate ( icoeffarr(liarry), stat=error )
      if ( error/=0 )  call eralloc ( error, liarry, 'icoeffarr' )

      allocate ( elemms(1:icount,1:icount), stat=error )
      if ( error/=0 )  call eralloc ( error, icount, 'elemms' )
      allocate ( indexren(1:icount), stat=error )
      if ( error/=0 )  call eralloc ( error, icount, 'indexren' )
      allocate ( index5(1:icount), stat=error )
      if ( error/=0 )  call eralloc ( error, icount, 'index5' )
      allocate ( index2(1:icount), stat=error )
      if ( error/=0 )  call eralloc ( error, icount, 'index2' )
      allocate ( index4(lenindex4), stat=error )
      if ( error/=0 )  call eralloc ( error, icount, 'index4' )
      if ( ierror/=0 ) go to 1000

      num = max(numrhs,idefcr+1)

!     --- Real right-hand side: use elemvc

      allocate ( elemvc2d(icount*icompl,num), stat=error )
      if ( error/=0 ) call eralloc ( error, icount*num, 'elemvc2d' )
      allocate ( elemmt2d(icount*icompl,icount,idefcr+1), stat=error )
      if ( error/=0 ) call eralloc ( error, icount*(idefcr+1), 'elemmt2d' )
      elemvc1 => elemvc2d(:,1)
      elemvc2 => elemvc2d(:,2)
      elemvc => elemvc1
      elemmt1 => elemmt2d(:,:,1)
      elemmt2 => elemmt2d(:,:,2)
      elemmt => elemmt1

      if ( jmethod>=13 .and. jmethod<=16 .or. &
           jmethod>=24 .and. jmethod<=27 ) then

!     --- Reset idefcr

         idefcr = idefcr-1

      end if  ! ( jmethod>=13 .and. jmethod<=16 ... )

1000  call erclos ( 'sepbuildalloc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbuildalloc'

      end if  ! ( debug )

      end subroutine sepbuildalloc
