      subroutine addvec
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.2  date 26-06-2014 Set namevfound
!        version  1.1  date 15-03-2014 Use original name
!        version  1.0  date 11-12-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add vector name to the list of vectors
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector_name
!     name
!     add
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleread
      use sepmodulecpack
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3158   No  space for vectors left
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'addvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from addvec'
         write(irefwr,1) 'numvec', numvec
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numvec = numvec+1   ! new sequence number of vector

!     --- Check available space

      if ( numvec>maxvec ) then

!     --- numvec>maxvec

         call errint ( numvec, 1 )
         call errint ( maxvec, 2 )
         call errsub ( 3158, 2, 0, 0 )
         go to 1000

      end if  ! ( numvec>maxvec )

      names(numvec) = stringim(1:nchar)    ! name of vector
      dimnames(numvec) = 1         ! dimension of vector name
      ncharnames(numvec) = nchar   ! number of characters of vector name
      realrd = numvec              ! sequence number of vector
      if ( string(1:nchar)=='V' ) namevfound = .true.

1000  call erclos ( 'addvec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numvec, nchar', numvec, nchar
         write(irefwr,*) 'vector ', names(numvec)
         write(irefwr,*) 'End addvec'

      end if  ! ( debug )

      end
