      subroutine ellglp(x, m)
! ====================================================================== &
!
!     programmer Luc Timmermans
!     version 1.1    date  06-03-1996 Layout
!     version 1.0    date  16-11-1990
!
!   copyright (c) 1990-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!***********************************************************************
!
!                         DESCRIPTION
!
!     compute the Legendre Gauss-Lobatto collocation points
!
!***********************************************************************
!
!                      KEYWORDS
!
!   legendre
!   polynomial
! ********************************************************************
!
!                 INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer m
      double precision x(m)

!     m    i  number of points
!     x    o  array of length m containing the Legendre Gauss-Lobatto
!             points
! ********************************************************************
!
!                    COMMON BLOCKS
!
!***********************************************************************
!
!     LOCAL PARAMETERS

      integer i , j, k, nh
      double precision pnp1p, pdnp1p, pnp, pdnp, pnm1p, pdnm1, pnp1m, &
                       pdnp1m, pnm, pdnm, pnm1m, poly, pder, pnm1, &
                       pnp1, pdnp1, pn, pdn, det, rp, rn, a, b, rm, &
                       dth, cd, sd, cs, ss, cssave, y, recsum, dely, eps

!     a
!     b
!     cd
!     cs
!     cssave
!     dely
!     det
!
!     dth
!     eps
!     i
!     j
!     k
!     nh
!     pder
!     pdn
!     pdnm
!     pdnm1
!     pdnp
!     pdnp1
!     pdnp1m
!     pdnp1p
!     pn
!     pnm
!     pnm1
!     pnm1m
!     pnm1p
!     pnp
!     pnp1
!     pnp1m
!     pnp1p
!     poly
!     recsum
!     rm
!     rn
!     rp
!     sd
!     ss
!     y
!***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ELLEGP   compute Legendre polynomials
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
! ====================================================================== &
!
!     --- compute the parameters in the Legendre polynomial

      call ellegp(m, pnp1p, pdnp1p, pnp, pdnp, pnm1p, pdnm1, 1d0)
      call ellegp(m, pnp1m, pdnp1m, pnm, pdnm, pnm1m, pdnm1, -1d0)
      det = pnp*pnm1m - pnm*pnm1p
      rp = -pnp1p
      rn = -pnp1m
      a = (rp*pnm1m - rn*pnm1p)/det
      b = (rn*pnp - rp*pnm)/det

      rm = m
      x(m) = 1d0
      nh = m/2

!     --- set-up rercursion relation for initial guess for the
!         Gauss-Lobatto roots

      dth = 4d0*atan(1d0)/(2d0*rm - 1d0)
      cd = cos(2d0*dth)
      sd = sin(2d0*dth)
      cs = cos(dth)
      ss = sin(dth)

!     --- compute the first half of the Gauss-Lobatto roots by polynomial
!         deflation

      do 300 j = 2, nh
         y = cs

         do 200 k = 1, 10
            call ellegp(m, pnp1, pdnp1, pn, pdn, pnm1, pdnm1, y)
            poly = pnp1 + a*pn + b*pnm1
            pder = pdnp1 + a*pdn + b*pdnm1
            recsum = 0d0

            do 100 i = 1, j-1
                   recsum = recsum + 1d0/(y - x(m+1-i))
100         continue

            dely = -poly/(pder - recsum*poly)
            y = y + dely
            eps = 1d-12
            if ( abs(dely).lt.eps ) go to 250
200      continue
250      continue

         x(m+1-j) = y
         cssave = cs*cd - ss*sd
         ss = cs*sd+ss*cd
         cs = cssave
300   continue

      x(1) = -1d0

!     --- use symmetry for the second half of the Gauss-Lobatto roots

      do 400 i = 2, nh
         x(i) = -x(m+1-i)
400   continue

!     --- fill middle point if necessary

      if ( mod(m,2).eq.1 ) x(nh+1) = 0d0

      end
