      subroutine er0085 ( ichois, amin, amax, nnodes )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.1    date 26-12-1998 Include commons
!        version 1.0    date 03-10-1994
!
!   copyright (c) 1994-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute minimum and maximum of vector in nodal points given by array nodes
!   Depending on ichois a user function subroutine may be called
! **********************************************************************
!
!                       KEYWORDS
!
!   maximum
!   minimum
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, nnodes
      double precision amin, amax

!     amax    o    Maximum value in solution array
!     amin    o    Minimum value in solution array
!     ichois  i    Choice parameter
!     nnodes  i    Number of nodes in nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inunkp, i, inode, istart
      double precision x, y, z, funval

!     funval  Function value
!     i       Counting variable
!     inode   Node sequence number
!     inunkp  Number of degrees of freedom in first point
!     istart  First position used at a part
!     x       x co-ordinate of point
!     y       y co-ordinate of point
!     z       z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision funbnd

!     FUNBND  Function subroutine to compute a value
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    if ( ichois=1 ) then
!       if ( indprfo<0 ) then
!          Compute minimum and maximum of array unew of length -indprfo*nnodes
!       else ( indprfo>0 )
!          Compute minimum and maximum of array unew of length kprobfo(nnodes+1)
!    else ( ichois=2 )
!       inunkp = -indprfo
!       amin := rinfin
!       amax := -rinfin
!       For i := 1 (1) nnodes
!          inode := nodes(i)
!          if ( indprfo>0 ) then
!             inunkp := kprobfo(i+1) - kprobfo(i)
!          if ( inunkp>0 ) then
!             func_val := funbnd(x,y,z,unew(inode),inunkp)
!             amin := min(amin,func_val)
!             amax := max(amax,func_val)
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ichois==1 ) then

!     --- ichois = 1, standard minimum and maximum value

         if ( indprfo<0 ) then
            nusolo = -indprfo*nnodes
         else
            nusolo = kprobfo(nnodes+1)
         end if

!        --- Compute minimum and maximum of array unew

         amin = minval(unew(1:nusolo))
         amax = maxval(unew(1:nusolo))

      else

!     --- ichois = 2, call function subroutine

         inunkp = -indprfo
         amin   = rinfin
         amax   = -rinfin
         y = 0d0
         z = 0d0

         do i = 1, nnodes

            inode = nodes(i)
            if ( indprfo>0 ) then
               inunkp = kprobfo(i+1) - kprobfo(i)
               istart = kprobfo(i)+1
            else
               istart = inunkp*(i-1)+1
            end if
            if ( inunkp>0 ) then
               x = coor(1,inode)
               if ( ndim>1 ) y = coor(2,inode)
               if ( ndim>2 ) z = coor(3,inode)
               funval = funbnd ( x, y, z, unew(istart), inunkp )
               amin = min(amin,funval)
               amax = max(amax,funval)
            end if

         end do

      end if

      end
