      subroutine erfill119 ( isol, numsol, numsubcurvs, outvec, numnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array outvec with the information of array ivectr of type 119
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numsol, isol(numsol), numsubcurvs, &
                             numnodes
      double precision, intent(out) :: outvec(numnodes,numsol+1)

!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     numnodes       i    Number of nodal points in array outvec
!     numsol         i    Total number of solution vectors in isol
!     numsubcurvs    i    Number of subcurves
!     outvec         o    Contains the output vector
!                         (.,1) the x-coordinates
!                         (.,2..) the values
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, isubcv, n, ipos
      integer, pointer :: infsubc(:)

!     i              Counting variable
!     infsubc        Contains number of nodes per segment
!     ipos           Starting position in array usol
!     istart         First position used at a part
!     isubcv         Subcurve sequence number
!     n              number of nodes in subcurve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfill119' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfill119'
         call prinin1 ( isol, numsol, 5, 'isol' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over vectors

      do i = 1, numsol

         infsubc => ks(isol(i))%isol
         usol => ks(isol(i))%sol

!        --- Loop over subcurves

         istart = 1
         ipos = 1
         do isubcv = 1, numsubcurvs

            n = infsubc(isubcv)
            if ( i==1 ) outvec(istart:istart+n-1,1) = usol(1:n) ! coordinates
            ipos = ipos+n  ! raise ipos

!           --- Copy function values

            outvec(istart:istart+n-1,i+1) = usol(ipos:ipos+n-1)
            ipos = ipos+n  ! raise ipos
            istart = istart+n

         end do  ! isubcv = 1, numsubcurvs

      end do  ! i = 1, numsol

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( outvec, numsol+1, numnodes, 'outvec' )
         write(irefwr,*) 'End erfill119'

      end if  ! ( debug )
      call erclos ( 'erfill119' )

      end subroutine erfill119
