      subroutine erfillcoparr ( kdeg, uservec, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-01-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the contents of array uold into array uservec for
!     all nodes stored in array nodes and the degrees of freedom indicated
!     in kdeg
!     uservec is user array of length nnodes
!     uold is a sepran solution array
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kdeg, nnodes
      double precision :: uservec(*)

!     kdeg           i    Only the KDEGth degree of freedom
!     nnodes         i    Number of nodal points in array nodes
!     uservec       i/o   New solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, i

!     i              Counting variable
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1131   Degree of freedom required is larger than number of unknowns
!           per point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcoparr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcoparr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kdeg>nphysi ) then
         call errint ( kdeg, 1 )
         call errint ( nphysi, 2 )
         call errsub ( 1131, 2, 0, 0 )
         go to 1000
      end if  ! ( kdeg>nphysi )

!     --- Distinguish between the various cases

      if ( indprfi==0 ) then

!     --- constant number of degrees of freedom in uold

         do i = 1, nnodes
            node = nodes(i)
            uservec(i) = uold((node-1)*nphysi+kdeg)
         end do  ! i = 1, nnodes

      else

!     --- variable number of degrees of freedom in uold

         do i = 1, nnodes

            node = nodes(i)
            if ( kprobfi(node+1)>=kprobfi(node)+kdeg ) then

!           --- Degree of freedom available in uold

               uservec(i) = uold(kprobfi(node)+kdeg)

            end if  ! ( kprobfi(node+1)>=kprobfi(node)+kdeg )

         end do  ! i = 1, nnodes

      end if  ! ( indprfi==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillcoparr'

      end if  ! ( debug )
      call erclos ( 'erfillcoparr' )

      end
