      subroutine ermess ( mesnum, nints, nreal, nchar, type )
! ======================================================================
!
!        programmer    S.Jansen / H.Hanzon
!        version  2.10 date 15-07-2015 Change of format
!        version  2.9  date 15-11-2012 Long integers
!        version  2.8  date 03-08-2011 Extra debug
!
!   copyright (c) 1990-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     ERMESS prints a (warning or error) message.
!
! **********************************************************************
!
!                       KEYWORDS
!
!     error_manipulation
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemach
      use sepmodulecomio
      use sepmoduleerr
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mesnum, nchar, nints, nreal
      character (len=*) type

!     mesnum  i    Message number.
!     nchar   i    Number of character strings.
!     nints   i    Number of integers.
!     nreal   i    Number of reals.
!     type    i    Type of message: error or warning.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, idum, ifrom, ipttrg, iref, istart(21), ito, lenspe, &
              linenr, maxerr, numcha, numint, numlin, numrea, numspe, &
              nch
      logical nxtlin, exist
      parameter ( lenspe = 2 )
      character (len=81) errlin, messpe(lenspe)
      character (len=162) line

!     errlin         Contains one line to be printed
!     exist          Flag: true= error number exists in error text file.
!     i              loop variable
!     idum           Integer dummy parameter
!     ifrom          information in array istart is read from position ifrom
!     ipttrg         pointer to first position not filled in line buffer
!     iref           Reference to error message file
!     istart         integer array containing a part of the starting records
!                    of the messages.
!     ito            information in array istart is read to position ito.
!     lenspe         length of special message array.
!     line           line buffer
!     linenr         record number of message line
!     maxerr         1 + maximum number of messages that are stored
!                    in the message file.
!     messpe         special error message:
!                    'Failure on open of error text file'.
!     nch            Number of characters in a string
!     numcha         number of character strings in a message
!     numint         number of integers in a message
!     numlin         number of lines of a message
!     numrea         number of reals in a message
!     numspe         the number of the special error message.
!     nxtlin         true: scan next line, false :don't scan next line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERERRL         scans and prints one line of a message
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
!     PRININLONG     print 1d integer array
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!     ERMESS writes to the default output device.
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    Message number not yet added to the Error Text File.
!    Number of integers, reals or strings in message not correct.
! **********************************************************************
!
!                       PSEUDO CODE
!
!     read maximum message number from error text file ;
!     if (message number correct) then
!        read starting record number ;
!        calculate number of lines ;
!        if ( number of lines>0 ) then
!           for each line do
!              read line ;
!              process and print line ;
!        else
!           print warning : message does not exist ;
!     else
!        print warning : message number out of range ;
!     if (number of integers, reals and/or strings not correct) then
!        print warning ;
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data messpe / &
       ' Failure on open of error message text file ', &
       ' unit= $, status= $'/
! ======================================================================
!
      debug = .false.

      if ( debug ) then

         write(irefwr,1) 'nints, nreal, nchar', nints, nreal, nchar
         write(irefwr,1) 'levsub, mesnum', levsub, mesnum
         call prininlong ( ints, nints, 'ints' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

!     --- initialize some parameters

      numspe = -1
      iref   = 4
      exist  = .true.
      errlin = ' '

      write (irefwr,'(//80(''*'')/)')

!     --- print the module name

      if ( levsub==0 ) then
         write (irefwr,100) type, mesnum, 'main'
100      format (' *** ',a,' number ',i5,' , in module ',a/)
      else
         nch = index(namsub(levsub),' ')-1  ! last non-blank
         if ( nch==-1 ) nch = len(namsub(levsub))
         write (irefwr,100) type, mesnum, namsub(levsub)(1:nch)
      end if

!     --- read largest message number permitted

      if ( debug ) write(irefwr,1) 'mesnum, numspe', mesnum, numspe
      if ( mesnum==numspe ) then

!     --- special error message: failure in open of text file.

         maxerr = numspe + 1

      else

         ifrom = 1
         ito   = 1
         linenr= 1
         idum  = 0
         call readfl ( linenr, ifrom, ito, idum, iref, istart )
         maxerr = istart(1)
         if ( debug ) call prinin ( istart, 1, 'istart' )

      end if
      if ( debug ) &
         write(irefwr,1) 'mesnum, maxerr, numspe', &
                          mesnum, maxerr, numspe

      if ( mesnum>0 .and. mesnum<=maxerr .or. mesnum==numspe ) &
          then

!     --- Read starting record of message

         if ( mesnum==numspe ) then

!        --- special error message: open failure of error text file

            numlin = lenspe
            linenr = 1

         else

            linenr = 2 + (mesnum-1)/nrec4
            ito    = mod(mesnum,nrec4)+1
            if ( debug ) write(irefwr,1) 'linenr, ito', linenr, ito
            if (ito==1) ito = nrec4+1
            if ( debug ) write(irefwr,1) 'ifrom, ito', ifrom, ito
            call readfl (linenr, ifrom, ito, idum, iref, istart)
            linenr = istart(ito-1)
            numlin = istart(ito) - linenr
            if ( debug ) call prinin ( istart, 21, 'istart' )

         end if
         if ( debug ) write(irefwr,1) 'numlin, linenr', numlin, linenr

         if ( numlin>0 ) then

!        --- Read and print message

            i=linenr
            numint=0
            numrea=0
            numcha=0
            nxtlin=.true.
            ipttrg=0
            line=' '

200         if ( nxtlin ) then

!           --- read one message line

               if (mesnum==numspe) then

!              --- special message: failure in open of text file

                  errlin = messpe(i)

               else

                  read ( irefer, rec=i ) errlin(1:80)

               end if

!              --- scan message line for integer fields ($), &
!                  real fields (&) and character fields (%), and
!                  write line to output device.

               call ererrl ( errlin,ints,nints,reals,nreal,chars, &
                             nchar,numint,numrea,numcha,ipttrg,line )

               i = i + 1
               nxtlin = i<=(linenr+numlin-1)
               goto 200
            end if
            if ( icarcn==0 ) then
               if ( ipttrg>0 ) write (irefwr,'(a)') line(1:ipttrg)
            else
               if ( ipttrg>0 ) write (irefwr,'(a)') line(1:ipttrg)
            end if

         else

!        --- Message not present in the file

            write (irefwr,210) type, mesnum
210         format ('*** WARNING : '/ &
                    ' ',a,' number ',i6,' has not yet been added', &
                    ' to the error message file')
            write (irefwr,220) (ints(i),i=1,nints)
220         format ('*** Ints  : ',5i13)
            write (irefwr,230) (reals(i),i=1,nreal)
230         format ('*** Reals : ',5e13.7)
            write (irefwr,240) (chars(i),i=1,nchar)
240         format ('*** Strings : '/(1x,a78))
            exist = .false.

         end if

      else

!     --- Message number out of range

         write (irefwr,210) type, mesnum
         write (irefwr,220) (ints(i),i=1,nints)
         write (irefwr,230) (reals(i),i=1,nreal)
         write (irefwr,240) (chars(i),i=1,nchar)
         exist = .false.

      end if

!     --- warning if number of integers, reals, or strings not correct
!         (if error number exists)

      if ( exist ) then
         if ( numint/=nints .or. numrea/=nreal .or. &
              numcha/=nchar ) then
            write (irefwr,'(//1x,78(''*'')/)')
            write (irefwr,250) type,mesnum,numint,nints,numrea,nreal, &
                                numcha,nchar
250         format ( '*** INTERNAL ERROR : number of integers, reals,', &
                     ' or strings in message '/ &
                     'not equal to number of integers, reals, or', &
                     ' strings in program.'/ &
                     a,' number : ',i5/ &
                     'number of integers in message :',i4, &
                     ', in program :',i4/ &
                     'number of reals in message    :',i4, &
                     ', in program :',i4/ &
                     'number of strings in message  :',i4, &
                     ', in program :',i4)
            write (irefwr,220) (ints(i),i=1,nints)
            write (irefwr,230) (reals(i),i=1,nreal)
            write (irefwr,240) (chars(i),i=1,nchar)
            write (irefwr,'(/80(''*'')//)')
         end if
      end if

!     --- write trace back of subroutines

      call ertrace
      write (irefwr,'(/80(''*'')//)')

      end
