      subroutine getelmsnodeh1 ( inode, ielems, nelms )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all elements corresponding to nodal point inode
!     Only elements of dimension ndim are used
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     node
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode, ielems(*), nelms

!     ielems         o    Array of size nelms with the elements containing
!                         inode
!     inode          i    Node sequence number
!     nelms         i/o   At input: Declared length of array ielems
!                         At output: Number of elements found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelmsinp, minshape, i, icount, ielem, ielgrp, istart

!     i              Counting variable
!     icount         Counter
!     ielem          Element number
!     ielgrp         element group number
!     istart         First position used at a part
!     minshape       Lowest shape number for this dimension
!     nelmsinp       nelms at input
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getelmsnodeh1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getelmsnodeh1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find all elements connected to inode

      nelmsinp = nelms
      istart = kmshd1(inode)
      nelms = kmshd1(inode+1)-istart

      call inichkarlen ( nelms, nelmsinp, 'ielems' )  ! check length
      if ( ierror/=0 ) go to 1000

      ielems(1:nelms) = kmshd2(istart+1:istart+nelms)

      if ( nelgrp>1 ) then

!     --- nelgrp>1, hence possibly shape numbers of lower dimension
!         Find minimum number of nodes per element for ndim

         select case ( ndim )
         case(1)
            minshape = 1
         case(2)
            minshape = 3
         case(3)
            minshape = 11
         end select  ! case ( ndim )

!        --- Remove elements with lower dimension

         icount = 0
         do i = 1, nelms

            ielem = ielems(i)
            ielgrp = sepelgrpelem(ielem)
            ishape = sepgetshape ( ielgrp )

            if ( ishape>=minshape ) then
               icount = icount+1
               ielems(icount) = ielem
            end if  ! ( ishape>=minshape )

         end do  ! i = 1, nelms
         nelms = icount

      end if  ! ( nelgrp>1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getelmsnodeh1'

      end if  ! ( debug )
      call erclos ( 'getelmsnodeh1' )

      end
