      function getkproba ( ivec, ielgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with starting address in kproba or kprobe for
!     vector of type ivec and element group ielgrp
!     if 0, all nodes have the same number of unknowns
! **********************************************************************
!
!                       KEYWORDS
!
!     element_group
!     problem_definition
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ivec, ielgrp
      integer :: getkproba

!     getkproba      o    Value of specific kproba
!     ielgrp         i    element group number
!     ivec           i    Sequence number of vector of special structure
!                         0 means solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ikelm

!     ikelm          Last position used in kmeshc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getkproba' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getkproba'
         write(irefwr,1) 'ielgrp, ivec, indpra', ielgrp, ivec, indpra
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kelma==0 ) then
         ikelm = (ielgrp-1)*npelm
      else
         ikelm = kmesha(ielgrp)
      end if  ! ( kelma==0 )
      getkproba = 0
      if ( ivec==0 ) then

!     --- solution vector

         if ( indpra>0 ) then
            getkproba = ikelm+1
            kprobatmp => kproba(ielgrp)%arr
         end if  ! ( indpra>0 )

      else

!     --- vector of special structure with sequence number ivec

         kprobe2 => kprobpart%kprobe2
         if ( kprobe2(ivec)<0 ) then
            getkproba = ikelm+1
            kprobatmp => kprobe(ikelm+1:,ivec)
         end if  ! ( kprobe2(ivec)<0 )

      end if  ! ( ivec==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'getkproba', getkproba
         write(irefwr,*) 'End getkproba'

      end if  ! ( debug )
      call erclos ( 'getkproba' )

      end function getkproba
