      function getnameindreal ( realname, loopname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the sequence number corresponding to the real in the block constants
!     with name realname
!     If the name does not exist getnameindreal is not changed
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) realname, loopname
      integer getnameindreal

!     getnameindreal o    Sequence number of real with respect to array
!                         chrlcons
!     loopname       i    Name of loop variable
!     realname       i    Name of real
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, nch1, iloop
      character (len=80) rname1, rname2

!     i              General loop variable
!     iloop          Value of loop variable
!     nch            Number of characters in a string
!     nch1           Number of characters in a string
!     rname1         Name of real translated to capitals
!     rname2         Name of real translated to capitals
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision getvar

!     ASKUPC         Translate all letters to upper case
!     GETVAR         Get the value of a scalar in the input block variables
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) numreals do
!        if ( intname = chrlcons(i) ) then
!           getnameindreal := i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      getnameindreal = 0
      nch1 = index ( realname, ' ' ) -1
      if ( nch1==-1 ) nch1 = len(realname)
      rname1 = realname(1:nch1)
      call askupc(rname1)
      iloop = nint(getvar(loopname))
      do i = 1, numreals
         nch = ncharcons(i+maxints)
         if ( nch1==nch ) then
            rname2 = chrlcons(i)(1:nch1)
            call askupc(rname2)
            if ( rname1(1:nch1)==rname2(1:nch1) ) then

!           --- name of real found

               getnameindreal = i+iloop-1
               go to 1000

            end if
         end if
      end do

1000  end
