      subroutine getvecnameseq ( vectorname, ivecnm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the sequence number corresponding to the vector with vector name
!     If the name does not exist ivecnm is not changed
! **********************************************************************
!
!                       KEYWORDS
!
!     name
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) :: vectorname
      integer, intent(inout) :: ivecnm

!     ivecnm        i/o   vector sequencer number with respect to isol
!     vectorname     i    Name of vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, nch1
      character (len=80) vname1, vname2

!     i              Counting variable
!     nch            Number of characters in a string
!     nch1           Number of characters in first word
!     vname1         Name of variable translated to capitals
!     vname2         Name of variable translated to capitals
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getvecnameseq' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getvecnameseq'
         write(irefwr,*) 'vectorname ', vectorname
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      nch1 = index ( vectorname, ' ' ) -1
      if ( nch1==-1 ) nch1 = len(vectorname)
      vname1 = vectorname(1:nch1)
      call askupc(vname1)

      ivecnm = 0

      if ( debug ) write(irefwr,1) 'numvec', numvec

      do i = 1, numvec

         nch = ncharnames(i)

         if ( nch1==nch ) then
            vname2 = names(i)(1:nch1)
            call askupc(vname2)
            if ( vname1(1:nch1)==vname2(1:nch1) ) then

!           --- name of vector found

               ivecnm = i
               exit

            end if
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ivecnm', ivecnm
         write(irefwr,*) 'End getvecnameseq'

      end if  ! ( debug )
      call erclos ( 'getvecnameseq' )

      end subroutine getvecnameseq
