      subroutine iniac0 ( compl, number, idegfd, ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 01-04-1999 Extra parameter ibuffr
!        version  1.2  date 24-02-1996 Jump in case of error
!        version  1.1  date 15-07-1994 New definition of ivectr(3)
!        version  1.0  date 30-12-1992
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Activate arrays ISOL, KPROB parts F, H and P, COOR
!    Compute compl, nphysi, ivec. Check NUMBER
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekprob
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: number, ivec, idegfd
      logical compl

!     compl     i/o   indication whether the arithmetic is real (false)
!                     or complex (true)
!     idegfd     o    Degree of freedom corresponding to number
!                     In the real case idegfd is number in the complex case it
!                     corresponds to the degree of freedom number refers to
!     ivec       o    Sequence number of vector of special structure
!     number     i    The number-th unknown in each nodal point of the array
!                     is considered. If the array is complex number=2*i-1
!                     corresponds to the real part of array i and number=2*i
!                     to the imaginary part of array i
!                     If compl=true at input number refers to the complete
!                     unknown
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numb
      logical compsv

!      compsv   Value of compl at input
!      numb     numb gives the number of the unknown to be taken in each
!               nodal point. numb is completely dependent on number
!               If number = 0 then numb = 1 else numb = number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF1   compute pointers to degrees of freedom of vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     102   NUMBER has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniac0' )

!     --- Compute starting positions of arrays

      compsv = compl
      call sepactsolbf1 ( isolut )
      if ( ierror/=0 ) go to 1000

      compl = ks(isolut)%compl==1
      ivec = ks(isolut)%ivec

!     --- solution vector or vector of special structure

      numb = max( 1, number )
      if ( (.not. compl .or. compsv) .and. ( numb<0 .or.numb>nphysi ) ) then
         call errint ( numb, 1 )
         call errint ( nphysi, 2 )
         call errsub ( 102, 2, 0, 0 )
      end if
      if ( .not. compsv .and. compl .and. ( numb<0 .or. numb>2*nphysi ) ) then
         call errint ( numb, 1 )
         call errint ( 2*nphysi, 2 )
         call errsub ( 102, 2, 0, 0 )
      end if
      if ( .not. compsv .and. compl ) then
         idegfd = (numb+1)/2
      else
         idegfd = numb
      end if

1000  call erclos ( 'iniac0' )
      end
