      subroutine iniclearbnsol ( isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 21-11-2012 Long integers
!        version  1.2  date 27-09-2010 Make work arrays local
!        version  1.1  date 30-03-2007 Extension wih 3d
!
!   copyright (c) 2007-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set all values of the solution array equal to 0 at the boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     clear
!     solution_vector
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isol(*)

!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!                         The value of this vector at the boundary is set to 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nnodes

!     nnodes         Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     EROUTCURVNOD   Find all nodes in the outer curves and store in array nodes
!     EROUTSURFNOD   Find all nodes in the outer surfaces and store in array
!                    nodes
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICLEARBOUN   Make solution array equal to 0 for all nodes
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPFILLOUTERB  Fill outer boundary of mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store all nodes of the boundary in array nodes
!     Make solution equal to 0 for all these nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniclearbnsol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniclearbnsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- create or activate kmesh part ah with outer boundary

      if ( mmouter==0 ) call sepfillouterb

      call sepactsolbf2 ( isol )

!     --- Reserve space for temporary arrays

      allocate ( nodes(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting addresses

      usol => ks(isol(1))%sol

!     --- Store all nodes of the boundary in array nodes

      if ( ndim==2 ) then

!     --- ndim = 2

         call eroutcurvnod ( nnodes, nodes )

      else if ( ndim==3 ) then

!     --- ndim = 3

         call eroutsurfnod ( nnodes, nodes )

      else

!     --- Other dimensions not yet implemented

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( ndim==2 )

!     --- Make solution equal to 0 for all these nodes

      call iniclearboun ( nnodes )

!     --- Remove semi-permanent arrays

      deallocate ( nodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniclearbnsol'

      end if  ! ( debug )
      call erclos ( 'iniclearbnsol' )

      end
