      subroutine inicreatvec1 ( isol, name )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-02-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create space for a vector with one degree of freedom per point
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol
      character (len=*) name

!     isol          i/o   Standard SEPRAN array containing information of
!                         the vector to be created
!     name           i    Name of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc, ivec

!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INISTR         find structure of vector
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicreatvec1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicreatvec1'

      end if
      if ( ierror/=0 ) go to 1000

!     --- First find the vector of special structure with one degree
!         of freedom per point

      itypvc = 0
      iprob = 1
      call inistr ( 1, itypvc, ivec )
      if ( ierror/=0 ) go to 1000

!     --- Next we create the vector

      call sepcreasolbf1 ( isol, 115, ivec, name, 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inicreatvec1'

      end if
      call erclos ( 'inicreatvec1' )

      end subroutine inicreatvec1
