      subroutine inimap2 ( npointglob, mapglob, npointloc, maploc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-10-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute mapping from global to local array
!     Points that are not present in local array get the value 0
!     Hence the inverse of the local to global mapping is stored
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     global
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npointglob, mapglob(npointglob), npointloc, &
              maploc(npointloc)

!     mapglob        o    Contains the local nodal point numbers corresponding
!                         to nodes in the global mesh
!                         If gobal node is not present in local mesh the
!                         corresponding value in mapglob is 0
!     maploc         i    Contains the global nodal point numbers of the
!                         nodes in the local mesh
!     npointglob     i    Number of points in gobal mesh
!     npointloc      i    Number of points in local mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inimap2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inimap2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array mapglob

      do i = 1, npointglob
         mapglob(i) = 0
      end do  ! i = 1, npointglob

!     --- Map array maploc into array mapglob

      do i = 1, npointloc
         mapglob(maploc(i)) = i
      end do  ! i = 1, npointglob

1000  call erclos ( 'inimap2' )
      if ( debug ) then

!     --- Debug information

         call prinin ( mapglob, npointglob, 'mapglob' )
         write(irefwr,*) 'End inimap2'

      end if  ! ( debug )

      end
