      subroutine iniproj ( x, x1, x2, xp, alpha )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date  23-08-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the projection of the point x onto the line through x1 and x2
!   Return with the projection and a parameter alpha indicating at which side
!   of x1 and x2 the prjected point is lying
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     projection
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision x(2), x1(2), x2(2), xp(2), alpha

!     alpha      o    Multiplication factor for parameter along element
!                     If between 0 and 1 the projection is in the element
!     x          i    co-ordinates of point to be projected
!     x1         i    coordinates of first point in line
!     x2         i    coordinates of last point in line
!     xp         o    coordinates of projection
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision det, beta

!     beta       Multiplication factor to compute projection
!     det        Determinant
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1643   Distance between 2 points is 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The projection is computed as follows:
!
!    t = (x2-x1, y2-y1)
!    n = (y2-y1, x1-x2)
!    x + beta n = x1 + alpha t
!
!    hence
!
!    | x |       | y2-y1 |   |x1|         |x2-x1|
!    |   | + beta|       | = |  | + alpha |     |
!    | y |       | x1-x2 |   |y1|         |y2-y1|
!
!    So
!
!   |alpha|    1  | x2-x1  y2-y1 | |x-x1|
!   |     | = --- |              | |    |
!   |beta |   det | y1-y2  x2-x1 | |y-y1|
!
!   det = (y2-y1)**2 + (x2-x1)**2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      det = (x2(1)-x1(1))**2 + (x2(2)-x1(2))**2
      if ( det.le.0d0 ) then
         call eropen ( 'iniproj' )
         call errsub ( 1643, 0, 0, 0 )
         call erclos ( 'iniproj' )
         alpha = 0d0
         beta = 0d0
         go to 1000
      end if
      det = 1d0/det
      alpha = det*( (x2(1)-x1(1))*(x(1)-x1(1)) + &
                    (x2(2)-x1(2))*(x(2)-x1(2)) )
      beta = det*( (x1(2)-x2(2))*(x(1)-x1(1)) + &
                   (x2(1)-x1(1))*(x(2)-x1(2)) )
1000  xp(1) = x(1) + beta*(x2(2)-x1(2))
      xp(2) = x(2) + beta*(x1(1)-x2(1))
      end
