      subroutine inireducescal ( iscalin, iscal, numscalars )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the reduced value of iscalin which indicates the first
!     sequence number of a (series of) scalars, as well as the dimension
!     of the series
! **********************************************************************
!
!                       KEYWORDS
!
!     fill_scalar
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iscalin, iscal, numscalars

!     iscal          o    Actual sequence number of scalar after reduction
!     iscalin        i    Sequence number of scalar before reduction
!     numscalars     o    Number of scalars in case it is a series
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getscalindex

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETSCALINDEX   Return with adapted index of scalar
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inireducescal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inireducescal'
         write(irefwr,1) 'iscalin', iscalin
   1     format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iscal = getscalindex(iscalin)

      if ( debug ) write(irefwr,1) 'iscal', iscal

      if ( iscal<0 ) then

!     --- iscal < 0, series of scalars

         iscal = -iscal
         if ( iscal>5000 ) iscal = iscal-5000
         numscalars = dimscals(iscal)

      else

!     --- iscal > 0, only one scalar

         numscalars = 1

      end if  ! ( iscal<0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iscal, numscalars', iscal, numscalars
         write(irefwr,*) 'End inireducescal'

      end if  ! ( debug )
      call erclos ( 'inireducescal' )

      end
