      subroutine iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.2    date   16-07-1998 Implement long lines
!        version 1.1    date   03-07-1997 Remove redundant assignment
!        version 1.0    date   07-03-1997
!
!
!   copyright (c) 1997-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Write record to file irefmesh in sepran input format
!    Spaces are removed from numbers and records are split if longer than 80
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer ilast, indent, irefmesh
      character *1000 wrtrecord

!     ilast      i    Last curve (surface) number read
!     indent     i    number of spaces for the indent
!     irefmesh   i    Reference number of file to which the mesh description
!                     must be written
!     wrtrecord  i    Contains record to be written
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icolum, icount, ifirst, k
      character *1000 copyrec

!     copyrec    Contains adapted copy of the record to be written
!     icolum     Next column number that must be read in the character array
!     icount     If zero it concerns the first line otherwise a
!                continuation line
!     ifirst     First position in wrtrecord to be used
!     k          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1681   No , found whereas record longer than 79
! **********************************************************************
!
!                       PSEUDO CODE
!
!     While last position + indent > 80
!        split record
!        write record
!        recompute record
!     write last record
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      copyrec = ' '
      copyrec(indent+1:indent+ilast) = wrtrecord(1:ilast)
      icount = 0
      ifirst = 1
100   if ( indent+ilast-ifirst.gt.79 ) then

!     --- While last position + indent > 80

         icolum = 78
         do k = icolum, indent+1, -1
            if ( copyrec(k:k).eq.',' ) then

!           --- , found, split record
!                 write record

               write(irefmesh,200) copyrec(1:k-1),'//'

!              --- recompute record

               ifirst = ifirst+k-indent
               copyrec = ' '
               if ( icount.eq.0 ) then
                  indent = indent+3
                  icount = 1
               end if
               copyrec(indent+1:indent+ilast-ifirst+1) = &
                  wrtrecord(ifirst:ilast)
               go to 100
            end if

         end do
         call eropen ( 'iniwrrcd' )
         call errsub ( 1681, 0, 0, 0 )
         call erclos ( 'iniwrrcd' )

      end if
      icolum= indent+ilast-ifirst+1
      if ( icount.gt.0 ) indent = indent-3
      write(irefmesh,200) copyrec(1:icolum)
200   format(a,a)

      ilast = 0

      end
