      subroutine mshcop2 ( iseqkm1, iseqkm2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-07-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy part of kmesh1 to kmesh2
!     Parts: coor, c,k,l,m,n
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqkm1, iseqkm2

!     iseqkm1        i    Sequence number with respect to km of first mesh
!     iseqkm2        i    Sequence number with respect to km of second mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcop2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcop2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy coordinates

      if ( km(iseqkm2)%kmesh(23)>0 ) deallocate(km(iseqkm2)%coor)
      length = size(km(iseqkm1)%coor,2)
      allocate ( km(iseqkm2)%coor(ndim,length) )
      km(iseqkm2)%coor = km(iseqkm1)%coor
      km(iseqkm2)%kmesh(23) = length*ndim

!     --- Copy kmeshc

      if ( km(iseqkm2)%kmesh(17)>0 ) deallocate(km(iseqkm2)%kmeshc)
      length = size(km(iseqkm1)%kmeshc )
      allocate ( km(iseqkm2)%kmeshc(length) )
      km(iseqkm2)%kmeshc = km(iseqkm1)%kmeshc
      km(iseqkm2)%kmesh(17) = length

!     --- If nsplit>1, copy kmeshk

      if ( km(iseqkm1)%kmesh(44)>1 ) then

!     --- Copy kmeshk

         if ( associated(km(iseqkm2)%kmeshk) ) deallocate(km(iseqkm2)%kmeshk)
         length = size(km(iseqkm1)%kmeshk )
         allocate ( km(iseqkm2)%kmeshk(length) )
         km(iseqkm2)%kmeshk = km(iseqkm1)%kmeshk
         km(iseqkm2)%kmesh(25) =  length

      end if

      if ( km(iseqkm1)%kmesh(26)/=0 ) then

!     --- Copy kmeshl

         if ( km(iseqkm2)%kmesh(26)>0 ) deallocate(km(iseqkm2)%kmeshl)
         length = size(km(iseqkm1)%kmeshl )
         allocate ( km(iseqkm2)%kmeshl(length) )
         km(iseqkm2)%kmeshl = km(iseqkm1)%kmeshl
         km(iseqkm2)%kmesh(26) =  length

      end if

      if ( km(iseqkm1)%kmesh(27)/=0 ) then

!     --- Copy kmeshm

         if ( km(iseqkm2)%kmesh(27)>0 ) deallocate(km(iseqkm2)%kmeshm)
         length = size(km(iseqkm1)%kmeshm )
         allocate ( km(iseqkm2)%kmeshm(length) )
         km(iseqkm2)%kmeshm = km(iseqkm1)%kmeshm
         km(iseqkm2)%kmesh(27) =  length

      end if

      if ( km(iseqkm1)%kmesh(28)/=0 ) then

!     --- Copy kmeshn

         if ( km(iseqkm2)%kmesh(28)>0 ) deallocate(km(iseqkm2)%kmeshn)
         length = size(km(iseqkm1)%kmeshn )
         allocate ( km(iseqkm2)%kmeshn(length) )
         km(iseqkm2)%kmeshn = km(iseqkm1)%kmeshn
         km(iseqkm2)%kmesh(28) =  length

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcop2'

      end if  ! ( debug )
      call erclos ( 'mshcop2' )

      end subroutine mshcop2
