      subroutine mshcop3 ( iseqkm1, iseqkm2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-07-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy part of kmesh1 to kmesh2
!     Parts: q,r,s,iinput,rinput, kmesht, kmeshquant
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqkm1, iseqkm2

!     iseqkm1        i    Sequence number with respect to km of first mesh
!     iseqkm2        i    Sequence number with respect to km of second mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMESHS  Make relation between kmeshs and kmeshs2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcop3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcop3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( km(iseqkm1)%kmesh(31)>0 ) then

!     --- Copy kmeshq

         if ( km(iseqkm2)%kmesh(31)>0 ) &
            deallocate(km(iseqkm2)%kmeshq)
         length = size(km(iseqkm1)%kmeshq )
         allocate ( km(iseqkm2)%kmeshq(length) )
         km(iseqkm2)%kmeshq = km(iseqkm1)%kmeshq
         km(iseqkm2)%kmesh(31) =  length

      end if

      if ( km(iseqkm1)%kmesh(32)>0 ) then

!     --- Copy kmeshr

         if ( km(iseqkm2)%kmesh(32)>0 ) &
            deallocate(km(iseqkm2)%kmeshr)
         length = size(km(iseqkm1)%kmeshr )
         allocate ( km(iseqkm2)%kmeshr(length) )
         km(iseqkm2)%kmeshr = km(iseqkm1)%kmeshr
         km(iseqkm2)%kmesh(32) =  length

      end if

      if ( km(iseqkm1)%kmesh(33)>0 ) then

!     --- Copy kmeshs

         if ( km(iseqkm2)%kmesh(33)>0 ) &
            deallocate(km(iseqkm2)%kmeshs)
         length = size(km(iseqkm1)%kmeshs )
         allocate ( km(iseqkm2)%kmeshs(length) )
         km(iseqkm2)%kmeshs = km(iseqkm1)%kmeshs
         km(iseqkm2)%kmesh(33) =  length
         call sepfillkmeshs ( km(iseqkm)%kmeshs(km(iseqkm)%kmesh(12)+2) )

      end if

      if ( km(iseqkm1)%kmesh(34)>0 ) then

!     --- Copy iinputkm

         if ( associated(km(iseqkm2)%iinputkm) ) &
            deallocate(km(iseqkm2)%iinputkm)
         length = size(km(iseqkm1)%iinputkm )
         allocate ( km(iseqkm2)%iinputkm(length) )
         km(iseqkm2)%iinputkm = km(iseqkm1)%iinputkm
         km(iseqkm2)%kmesh(34) =  length

      end if

      if ( km(iseqkm1)%kmesh(35)>0 ) then

!     --- Copy rinputkm

         if ( associated(km(iseqkm2)%rinputkm) ) &
            deallocate(km(iseqkm2)%rinputkm)
         length = size(km(iseqkm1)%rinputkm )
         allocate ( km(iseqkm2)%rinputkm(length) )
         km(iseqkm2)%rinputkm = km(iseqkm1)%rinputkm
         km(iseqkm2)%kmesh(35) =  length

      end if

      if ( km(iseqkm1)%kmesh(36)>0 ) then

!     --- Copy kmesht

         if ( associated(km(iseqkm2)%kmesht) ) &
            deallocate(km(iseqkm2)%kmesht)
         length = size(km(iseqkm1)%kmesht,2 )
         allocate ( km(iseqkm2)%kmesht(2,length) )
         km(iseqkm2)%kmesht = km(iseqkm1)%kmesht
         km(iseqkm2)%kmesh(36) = length

      end if

      if ( km(iseqkm1)%kmesh(66)>0 ) then

!     --- Copy kmeshquant

         if ( associated(km(iseqkm2)%kmeshquant) ) &
            deallocate(km(iseqkm2)%kmeshquant)
         length = size(km(iseqkm1)%kmeshquant )
         allocate ( km(iseqkm2)%kmeshquant(length) )
         km(iseqkm2)%kmeshquant = km(iseqkm1)%kmeshquant
         km(iseqkm2)%kmesh(66) = length

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcop3'

      end if  ! ( debug )
      call erclos ( 'mshcop3' )

      end subroutine mshcop3
