      subroutine mshwrinp ( iinput, rinput, irefmesh, krenum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.2  date 23-09-2009 Extension with maxratio
!        version  2.1  date 17-03-2006 Extension with 3d interface elements
!
!   copyright (c) 1997-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write contents of arrays iinput and rinput in original form back to
!     an input file with reference number irefmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), irefmesh, krenum(*)
      double precision rinput(*)

!     iinput         i    integer input array
!     irefmesh       i    Reference number of file to which the mesh description
!                         must be written
!     krenum         i    Array containing information about renumbering
!                         See subroutine msh000
!     rinput         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, indent, ilast, jmaxin, iinp, jlast
      double precision c
      character (len=1000) wrtrecord

!     c              real help parameter to store a constant
!     i              Counting variable
!     iinp           last position used in array iinput
!     ilast          Last position used in wrtrecord
!     indent         number of spaces for the indent
!     j              Counting variable
!     jlast          iinput(20) refers to new part in iinput
!     jmaxin         if>0 its value indicates where the maximum numbers
!                    of points, curves, surfaces and volumes are stored
!                    in iinput
!     wrtrecord      Contains record to be written
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPUTIN       Put integer number in wrtrecord from position ilast
!     INIPUTRL       Put real number in wrtrecord from position ilast
!     INIPUTXT       Put text in wrtrecord from position ilast
!     INIWRRCD       Write record to file irefmesh in sepran input format
!     MSHWRCURVES    Write parts of arrays iinput and rinput corresponding to
!                    curves in original form back to an input file
!     MSHWRELGRPS    Write parts of arrays iinput and rinput corresponding to
!                    element groups in original form back to an input file
!     MSHWRINTERFC   Write parts of array iinput corresponding to
!                    interface elements in original form back to an input file
!     MSHWRPLOT      Write parts of arrays iinput and rinput corresponding to
!                    plot in original form back to an input file
!     MSHWRRENUM     Write parts of arrays iinput and rinput corresponding to
!                    renumber in original form back to an input file
!     MSHWRSURFACES  Write parts of arrays iinput and rinput corresponding to
!                    surfaces in original form back to an input file
!     MSHWRVOLUMES   Write parts of arrays iinput and rinput corresponding to
!                    volumes in original form back to an input file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    write record with meshnd
!    if ( jcoarse=1 ) then
!       write coarse record
!    if ( jmaxin>0 ) then
!       write max records
!    write record with points
!    write user points
!    write record with curves
!    write curves
!    if ( nsurfs>0 ) then
!       write record with surfaces
!       write surfaces
!    if ( nvolms>0 ) then
!       write record with volumes
!       write volumes
!    if ( nlines>0 ) then
!       write record with meshline
!       write line elements
!    if ( nsurel>0 ) then
!       write record with meshsurf
!       write surface elements
!    if ( nvolel>0 ) then
!       write record with meshvolume
!       write volume elements
!    if ( jconnect>0 ) then
!       write record with meshconnect
!       write connection elements
!    if ( istartinterface>0 ) then
!       write record with interface elements
!       write interface elements
!    write record with renumber or norenumber
!    if ( iskiptopol>0 )
!       write record with notopology
!    if ( jextra>0 ) then
!       write record with intermediate points
!       write intermediate points
!    write plot record
!    if ( irefin>0 )
!       write refine record
!    if ( itrans>0 )
!       write transform record
!    if ( icheck>0 )
!       write check_level record
!    write record with end
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshwrinp' )

!     --- write record with meshnd

      indent = 0
      ilast = 0
      call iniputxt ( 'mesh', wrtrecord, ilast )
      call iniputin ( ndim, wrtrecord, ilast )
      call iniputxt ( 'd', wrtrecord, ilast )
      call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
      indent = 3
      if ( jcoars==1 ) then

!     --- jcoarse=1
!         write coarse record

         call iniputxt ( 'coarse ( unit = ', wrtrecord, ilast )
         call iniputrl ( rinput(3+ndim*nuspnt), wrtrecord, ilast )
         call iniputxt ( ', maxratio = ', wrtrecord, ilast )
         call iniputrl ( rinput(4+(ndim+1)*nuspnt), wrtrecord, ilast )
         call iniputxt ( ' )', wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

      end if
      if ( iinput(13)>0 ) then

!     --- jmaxin>0
!         write max records

         jmaxin = iinput(13)
         call iniputxt ( 'maxpoints = ', wrtrecord, ilast )
         call iniputin ( iinput(jmaxin), wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
         call iniputxt ( 'maxcurves = ', wrtrecord, ilast )
         call iniputin ( iinput(jmaxin+1), wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
         call iniputxt ( 'maxsurfaces = ', wrtrecord, ilast )
         call iniputin ( iinput(jmaxin+2), wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
         call iniputxt ( 'maxvolumes = ', wrtrecord, ilast )
         call iniputin ( iinput(jmaxin+3), wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

      end if

!     --- write record with points

      call iniputxt ( 'points', wrtrecord, ilast )
      call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

!     --- write user points

      indent = 6
      do i = 1, nuspnt
         c = 0d0
         do j = 1, ndim
            c = c+abs(rinput(2+j+(i-1)*ndim))
         end do
         if ( i==1 .or. i==nuspnt .or. c>0d0 .or. jcoars==1 ) then

!        --- First or last point or point with coordinates not equal to 0

            call iniputxt ( 'p', wrtrecord, ilast )
            call iniputin ( i, wrtrecord, ilast )
            call iniputxt ( ' = ( ', wrtrecord, ilast )
            call iniputrl ( rinput(3+(i-1)*ndim), wrtrecord, ilast )
            do j = 2, ndim
               call iniputxt ( ', ', wrtrecord, ilast )
               call iniputrl ( rinput(2+j+(i-1)*ndim), wrtrecord, &
                               ilast )
            end do
            if ( jcoars==1 ) then
               call iniputxt ( ', ', wrtrecord, ilast )
               call iniputrl ( rinput(3+i+ndim*nuspnt), wrtrecord, &
                               ilast )
            end if
            call iniputxt ( ' )', wrtrecord, ilast )
            call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

         end if

      end do

!     --- write curves

      call mshwrcurves ( iinput, rinput, irefmesh )

!     --- write surfaces

      call mshwrsurfaces ( iinput, irefmesh )

!     --- write volumes

      call mshwrvolumes ( iinput, irefmesh )

!     --- Write line, surface and volume elements

      call mshwrelgrps ( iinput, rinput, irefmesh )

!     --- if ( jconnect>0 ) then
!            write record with meshconnect
!            write connection elements

      if ( iinput(15)>0 ) then
         jlast = iinput(15)
         if ( iinput(jlast-1)>0 ) then

!        --- istartinterface>0

            call mshwrinterfc ( iinput, wrtrecord, irefmesh, jlast )

         end if

      end if

!     --- if ( iskiptopol>0 )
!            write record with notopology

      indent = 3
      if ( iinput(14)>0 ) then

!     --- jextra>0 ) then
!           write record with intermediate points

         call iniputxt ( 'intermediate points', wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
         indent = 6
         iinp = iinput(14)-5

!        --- write intermediate points

         call iniputxt ( 'sidepoints = ', wrtrecord, ilast )
         call iniputin ( iinput(iinp+1), wrtrecord, ilast )
         if ( iinput(iinp+2)==-1 ) then
            call iniputxt ( ', midpoints = filled ', wrtrecord, ilast )
         end if
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
      end if
      if ( iinput(9)/=0 ) then

!     --- write plot record

         call mshwrplot ( iinput, rinput, irefmesh )

      end if

!     --- write record with renumber or norenumber

      if ( krenum(1)/=0 ) call mshwrrenum ( jrenum, irefmesh )
      indent = 3

!     --- if ( irefin>0 )
!            write refine record
!         if ( itrans>0 )
!            write transform record

      if ( icheck>0 ) then

!     --- if ( icheck>0 )
!            write check_level record

         call iniputxt ( 'check_level = ', wrtrecord, ilast )
         call iniputin ( icheck, wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

      end if

!     --- end statement

      indent = 0
      wrtrecord(1:3) = 'end'
      ilast = 3
      call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

!     --- write record with end

      call erclos ( 'mshwrinp' )
      end
