      subroutine prcopyvec2us ( sepvector, outvec, nodes, nnodes, nunkp, &
                                idegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-03-2014 debug statements
!        version  1.0  date 27-12-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy part of sepran vector to user vector
!     The user vector has 1 unknown per point and is defined in a set of
!     nodes only
!     The sepran vector is defined in all nodes and has exactly nunkp
!     degrees of freedom per point
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  nnodes, nodes(nnodes), nunkp, idegfd
      double precision sepvector(nunkp,*), outvec(nnodes)

!     idegfd         i    Degree of freedom sequence number
!     nnodes         i    Number of nodes
!     nodes          i    Contains the node numbers that must be copied
!     nunkp          i    Maximum number of unknowns in a point
!     outvec         o    Output vector of length nnodes containing the
!                         copied vector
!     sepvector      i    sepran vector consisting of nunkp degrees of freedom
!                         per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopyvec2us' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopyvec2us'
         write(irefwr,1) 'idegfd, nunkp, nnodes', idegfd, nunkp, nnodes
         call prinin ( nodes, nnodes, 'nodes' )
         call prinrl1 ( sepvector, nnodes, nunkp, 'sepvector' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      outvec(1:nnodes) = sepvector(idegfd,nodes(1:nnodes))

1000  call erclos ( 'prcopyvec2us' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( outvec, nnodes, 'outvec' )
         write(irefwr,*) 'End prcopyvec2us'

      end if  ! ( debug )

      end
