      subroutine prfillnodelgrp ( inpsep, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 03-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for element groups
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nodes(*), nnodes

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prfillnodes
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numnod, ielgrp, istart, inpelm
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ielgrp         element group number
!     inpelm         number of points per element
!     istart         First position used at a part of kmeshc
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means node i has not yet been filled
!                    iwork(i)>0, means node i has been filled
!                    The value defines the new node number
!     numnod         If>0, only numnod nodes per element must be used
!                    In that case ihelp is used
!                    If 0 all nodes are used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnumelmelgrp, getnpelmelgrp

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILLNODSKMC1 Fill array nodes for elements 1 to inelem
!     ERFILLNODSKMC2 Fill array nodes for elements 1 to inelem
!                    Special case oly a restricted number of nodes per element
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETNPELMELGRP  Fetch number of points per element in element group ielgrp
!     GETNUMELMELGRP Determine number of elements in element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      67   ielgrp has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillnodelgrp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillnodelgrp'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0
      nnodes = 0

!     --- Get parameters from inpsep

      istart = 0
      ielgrp = inpsep(1)
      numnod = inpsep(2)
      if ( debug ) write(irefwr,1) 'ielgrp, numnod', ielgrp, numnod

      if ( ielgrp<=0 .or. ielgrp>nelgrp ) then
         call errint ( ielgrp, 1 )
         call errint ( nelgrp, 2 )
         call errsub ( 67, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

!     --- compute inpelm and number of elements
!         compute the number of elements  inelem

      do i = 1, ielgrp-1
         inelem = Topgrp(i)%inelem
         inpelm = Topgrp(i)%inpelm
         istart = istart + inpelm * inelem
      end do

      inelem = Topgrp(ielgrp)%inelem
      inpelm = Topgrp(ielgrp)%inpelm
      if ( ierror/=0 ) go to 1000

      if ( numnod==0 ) then

!     --- numnod = 0, fill all nodes per element

         call erfillnodskmc1 ( nodes, nnodes, kmeshc(1+istart), &
                               inelem, inpelm, iwork )

      else

!     --- numnod>0, fill some nodes per element

         call erfillnodskmc2 ( nodes, nnodes, kmeshc, inelem, &
                               inpelm, iwork, numnod, inpsep(3), &
                               istart )

      end if  ! ( numnod==0 )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillnodelgrp'

      end if  ! ( debug )
      call erclos ( 'prfillnodelgrp' )

      end
