      subroutine prinsl ( ivectr, kmeshlc, kproblc, iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 29-05-2014 remove call to time subroutine
!        version  1.3  date 21-11-2012 Long integers
!        version  1.2  date 12-08-1999 New call to erclmn
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   print real/complex  sepran array except the large matrix
!   standard format 5d16.9 is used
!   prinsl writes a solution in character format to a file,
!   readsl reads the solution
!         ks(ivectr)%typevec=110,115 allowed
! **********************************************************************
!
!                       KEYWORDS
!
!     output
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshlc, kproblc, ivectr, iref

!     iref       i    Reference number of file to which solution must be written
!                     possibilities:  see user manual
!                     if iref<0, the file is written unformatted, otherwise it
!                                  is supposed to be formatted
!     ivectr     i    integer array denoting the array to be printed
!     kmeshlc    i    array containing information of the mesh
!     kproblc    i    array containing information of degrees of freedom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ihelp(8)

!     i              Counting variable
!     ihelp          o    integer work array of length 8 to store
!                         quantities corresponding to ivectr
!                         1: compl
!                         2: iprob
!                         3: ivec
!                         4: jcart
!                         5: nphys
!                         6: nusol
!                         7: typevec
!                         8: iseqkp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinsl' )
      if ( ierror/=0 ) go to 1000
  1   format(5d16.9)

      usoltemp => ks(ivectr)%sol

      if ( iref>0 ) then
         write(iref,100) ihelp
         write(iref,1) usoltemp
100      format(5i6)
      else
         write(-iref) ihelp
         write(-iref,1) usoltemp
      end if

1000  call erclos ( 'prinsl' )
      end subroutine prinsl
