      subroutine printtime ( name, timebefore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 01-03-2004 Extension with wall clock time
!        version  1.0  date 12-08-1999
!
!   copyright (c) 1999-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the CPU time used in a subroutine
!     Replaces part of the actions taken by askclk
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     timing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpu
      use sepmodulecomio
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character name *(*)
      double precision :: timebefore(2)

!     name           i   Name of calling subroutine  (Literal constant)
!     timebefore     i   Time at entrance of subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer secday, sechour, secmin
      double precision tday, thour, tmin, tsec

!     secday         Number of seconds in a day
!     sechour        Number of seconds in an hour
!     secmin         Number of seconds in a minute
!     tday           Number of days
!     thour          Number of hours
!     tmin           Number of minutes
!     tsec           Number of seconds
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKCLK         print elapsed time
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call askclk
      if ( cputiming ) then

!     --- CPU time

         if ( tim3<1d1 ) then
            write(irefwr,1) name, tim3-timebefore(1), tim3
  1         format('cputime: ',a,  f8.4, '  Total', f8.4 )
         else if ( tim3<1d3 ) then
            write(irefwr,2) name, tim3-timebefore(1), tim3
  2         format('cputime: ',a,  f10.4, '  Total', f10.4 )
         else if ( tim3<1d5 ) then
            write(irefwr,3) name, tim3-timebefore(1), tim3
  3         format('cputime: ',a,  f12.4, '  Total', f12.4 )
         else if ( tim3<1d7 ) then
            write(irefwr,4) name, tim3-timebefore(1), tim3
  4         format('cputime: ',a,  f14.4, '  Total', f14.4 )
         else
            write(irefwr,5) name, tim3-timebefore(1), tim3
  5         format('cputime: ',a,  f17.4, '  Total', f17.4 )
         end if  ! ( tim3<1d1 )

      else

!     --- Wall clock time

         secday = 24*3600
         sechour = 3600
         secmin = 60
         tday = int(tim3wall/secday)
         thour = tim3wall-tday*secday
         tmin = thour
         thour = int(thour/sechour)
         tmin = tmin-thour*sechour
         tsec = tmin
         tmin = int(tmin/secmin)
         tsec = tsec-tmin*secmin

         if ( tim3wall<1d1 ) then
            write(irefwr,11) name, tim3-timebefore(1), tim3, &
                             tim3wall-timebefore(2),  tim3wall, &
                         nint(tday), nint(thour), nint(tmin), nint(tsec)
 11         format('cputime: ',a, f8.4, '  Total', f8.4, &
                   ' Wallclock', f8.4, '  Total', f8.4, &
             ' ( ', i3.3, ':', i2.2, ':', i2.2, ':', i2.2, ')' )
         else if ( tim3wall<1d3 ) then
            write(irefwr,12) name, tim3-timebefore(1), tim3, &
                             tim3wall-timebefore(2),  tim3wall, &
                         nint(tday), nint(thour), nint(tmin), nint(tsec)
 12         format('cputime: ',a, f10.4, '  Total', f10.4, &
                   ' Wallclock', f10.4, f10.4, &
             ' ( ', i3.3, ':', i2.2, ':', i2.2, ':', i2.2, ')' )
         else if ( tim3wall<1d5 ) then
            write(irefwr,13) name, tim3-timebefore(1), tim3, &
                             tim3wall-timebefore(2),  tim3wall, &
                         nint(tday), nint(thour), nint(tmin), nint(tsec)
 13         format('cputime: ',a, f12.4, '  Total', f12.4, &
                   ' Wallclock', f12.4, f12.4, &
             ' ( ', i3.3, ':', i2.2, ':', i2.2, ':', i2.2, ')' )
         else if ( tim3wall<1d7 ) then
            write(irefwr,14) name, tim3-timebefore(1), tim3, &
                             tim3wall-timebefore(2),  tim3wall, &
                         nint(tday), nint(thour), nint(tmin), nint(tsec)
 14         format('cputime: ',a, f14.4, '  Total', f14.4, &
                   ' Wallclock', f14.4, f14.4, &
             ' ( ', i3.3, ':', i2.2, ':', i2.2, ':', i2.2, ')' )
         else
            write(irefwr,15) name, tim3-timebefore(1), tim3, &
                             tim3wall-timebefore(2),  tim3wall, &
                         nint(tday), nint(thour), nint(tmin), nint(tsec)
 15         format('cputime: ',a, f17.4, '  Total', f17.4, &
                   ' Wallclock', f17.4, f17.4, &
             ' ( ', i3.3, ':', i2.2, ':', i2.2, ':', i2.2, ')' )
         end if  ! ( tim3wall<1d1 )
      end if  ! ( cputiming )
      end
