      subroutine prveccombinat ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 04-09-2016 New parameter list
!        version  1.0  date 22-12-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate right-hand side of vector expression
!
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleplot
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine ASKVEC
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqvec, iseqcoor, numarrays, iseqarrsav

!                    they are not
!     iseqarrsav     Help parameter to store old value of iseqarray
!     iseqcoor       Sequence number of coordinates vector in isol
!     iseqvec        sequence number of vector in left-hand side with respect
!                    to isol
!     numarrays      Number of indexed vectors in vector array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     PRVECCOMBINAT0 Evaluate right-hand side of vector expression
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( left-hand-side vector exists ) then
!        activate vector
!     else
!        create vector depending on right-hand side
!     Fill set of nodes where vector must be computed
!     Compute length of vector
!     Translate expression to a set of reals and a new set of
!     temporary vectors
!     Evaluate expression
!     Store result in vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prveccombinat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prveccombinat'
         write(irefwr,1) 'iseqkp', iseqkp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqvec = inpsep(3)  ! sequence number of vector in left-hand side
      iseqcoor = 0         ! no coordinates

!     --- Check number of arrays at left-hand side

      numarrays = 1
      if ( iseqvec>0 ) call inireducevec ( iseqvec, iseqvec, numarrays )

      if ( debug ) write(irefwr,1) 'iseqvec', iseqvec

      iseqarrsav = iseqarray  ! save old value of iseqarray

!     --- Loop over all subarrays

      do iseqarray = 1, numarrays

         iseqvec = iseqvec+iseqarray-1
         call prveccombinat0 ( inpsep, iseqvec, iseqcoor )

      end do  ! iseqarray = 1, numarrays

      iseqarray = iseqarrsav ! reset value

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numvecmn', numvecmn
         write(irefwr,*) 'End prveccombinat'

      end if  ! ( debug )
      call erclos ( 'prveccombinat' )

      end
