      subroutine prveccombinat0 ( inpsep, iseqvec, iseqcoor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 04-09-2016 New parameter list
!        version  1.3  date 12-02-2015 New call to prstorevec
!        version  1.2  date 17-01-2015 Extra debug
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate right-hand side of vector expression
!
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecons
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmoduleplot
      use sepmodulepost
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), iseqvec, iseqcoor

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine ASKVEC
!     iseqcoor       i    Sequence number of coordinates vector in isolut
!     iseqvec        i    Sequence number of vector to be created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numvecrhs, numrealrhs, icount, istore(100), &
              vecs(100), itypvc, ivec, lenvec, nch, nunkploc, icompl
      double precision store(100)
      logical :: completevec, complexvec

!     completevec    If true the complete vector must be filled
!     complexvec     If true the vector to be filled is complex
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     icount         Computed length of array istore
!     istore         Contains the expression as stored in common
!                    cexpression, where all scalars and reals
!                    have been computed and stored in store
!                    References to these values are given by
!                    their position in store plus 1000
!                    The values of the vectors are stored in
!                    array vectors
!                    References to these vectors are given by
!                    their position in vectors plus 10000
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     lenvec         Length of vector to be created
!     nch            Number of characters in a string
!     numrealrhs     Number of reals stored in store
!     numvecrhs      Number of vectors in right-hand side
!     nunkploc       Local value of nunkp
!     store          Contains values of reals and scalars, see istore
!     vecs           In this array of size numvecrhs the sequence numbers of
!                    the right-hand side vectors are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRCOPYCOORBCK  Copy coordinates stored in solution vector back
!     PRININ         print 1d integer array
!     PRSTOREVEC     Adapt vector sequence number and copy coor if necessary
!                    in new vector
!     PRVECCHECKLEN  Check if all vectors at right-hand side have the same
!                    length as the vector on the right-hand side
!     PRVECCOMBINAT1 Evaluate right-hand side of vector expression
!                    Complete real vector
!     PRVECCOMBINAT2 Evaluate right-hand side of vector expression
!                    Complete complex vector
!     PRVECCOMBINAT3 Evaluate right-hand side of vector expression
!                    Partial real vector
!     PRVECFIND      Find the type of vector to be created for the
!                    left-hand side of an expression
!     PRVECSTRHS     Translate expression to a set of reals and a new set of
!                    temporary vectors
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3162   Option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( left-hand-side vector exists ) then
!        activate vector
!     else
!        create vector depending on right-hand side
!     Fill set of nodes where vector must be computed
!     Compute length of vector
!     Translate expression to a set of reals and a new set of
!     temporary vectors
!     Evaluate expression
!     Store result in vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prveccombinat0' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prveccombinat0'
         write(irefwr,1) 'iseqvec, maxvc1, iseqkp', iseqvec, maxvc1, iseqkp
         call prinin ( inpsep, 10, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Translate expression to a set of reals and a new set of
!         temporary vectors

      vecs = 0
      call prvecstrhs ( inpsep(8)-200000000, numvecrhs, numrealrhs, &
                        icount, istore, store, vecs )
      call inichkarlen ( icount, 100, 'istore' )
      call inichkarlen ( numrealrhs, 100, 'store' )
      call inichkarlen ( numvecrhs, 100, 'vecs' )
      if ( ierror/=0 ) go to 1000
      icompl = 0

!     --- Create left-hand side vector

      if ( iseqvec>maxvc1 ) then

!     --- iseqvec>maxvc1, hence special vector
!         Store temporarily into isolut

         call prstorevec ( iseqcoor, iseqvec, iseqvec )

         if ( debug ) then
            write(irefwr,1) 'iseqvec, iseqcoor', iseqvec, iseqcoor
            call prinin ( isolut2d(iseqvec), 5, 'isolut_coor' )
         end if  ! ( debug )
         indprfo = indprf
         nunkploc = nunkp

      else if ( inpsep(5)>0 .and. isolut2d(iseqvec)>0 .and. &
                isolut2d(iseqvec)<=lastsol ) then

!     --- degfd>0 and vector exists

         call sepactsolbf2 ( isolut2d(iseqvec) )
         complexvec = ks(isolut2d(iseqvec))%compl==1

      else

!     --- find vector depending on right-hand side

         call prvecfind ( inpsep, ivec, itypvc, iseqvec, &
                          vecs, numvecrhs, complexvec )
         if ( debug ) write(irefwr,*) 'complexvec/2', complexvec

!        --- Create vector

         if ( complexvec ) icompl = 1 ! Complex vector
         call sepcreasolbf1 ( isolut2d(iseqvec), itypvc, ivec, &
                              names(iseqvec), icompl )
         nunkploc = nphyso

         if ( debug ) then
            write(irefwr,1) 'nunkploc, indprfo, isolut2d(iseqvec), iseqkp', &
                             nunkploc, indprfo, isolut2d(iseqvec), iseqkp
         end if  ! ( debug )

      end if  ! ( iseqvec>maxvc1 )
      if ( ierror/=0 ) go to 1000
      numvecmn = max(numvecmn, iseqvec)
      numvec = max(numvec, iseqvec)

      if ( ioutp>=0 .and. itimloop<=1 ) then

         nch = index(names(iseqvec),' ')-1
         if ( nch<=0 ) nch = len(names(iseqvec))
         write ( irefwr, 100 ) names(iseqvec)(1:nch), iseqvec
100      format ( ' Create vector ', a, &
                  ' with sequence number ', i3, ' by algebraic', &
                  ' manipulations' )

      end if  ! ( ioutp>=0 .and. itimloop<=1 )

      if ( debug ) write(irefwr,*) 'complexvec/3', complexvec

      completevec = inpsep(14)==0 .and. inpsep(15)==0 .and. &
                    inpsep(5)==0

      if ( debug ) write(irefwr,*) 'completevec', completevec

      if ( completevec ) then

!     --- Complete vector
!         Check if all vectors at the right-hand side have the same length
!         as the left-hand side

         call prvecchecklen ( iseqvec, vecs, numvecrhs, &
                              nunkploc, completevec )
         if ( ierror/=0 ) go to 1000

!        --- Compute length of vector

         lenvec = ks(isolut2d(iseqvec))%nusol

         if ( complexvec ) then

!        --- Complex vectors

            call prveccombinat2 ( iseqvec, vecs, numvecrhs, lenvec, &
                                  istore, store, icount, numrealrhs )

         else

!        --- Real vectors

            call prveccombinat1 ( iseqvec, vecs, &
                                  numvecrhs, lenvec, istore, &
                                  store, icount, numrealrhs, nunkploc )

         end if  ! ( complexvec )

      else

!     --- Part of vector

         if ( complexvec ) then

!        --- Complex vectors

            call errsub ( 3162, 0, 0, 0 )
            go to 1000

         else

!        --- Real vectors

            call prveccombinat3 ( iseqvec, vecs, numvecrhs, &
                                  istore, store, icount, numrealrhs, inpsep )

         end if  ! ( complexvec )

      end if  ! ( completevec )

      if ( iseqcoor>0 ) then

!     --- coor has been changed, copy back into kmesh

         call prcopycoorbck ( iseqcoor, inpsep(3) )

!        --- Remove from isolut

         call seprmsolinfo ( isolut2d(iseqcoor) )
         if ( numvec==iseqcoor ) numvec = numvec-1

      end if  ! ( iseqcoor>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numvec', numvec
         write(irefwr,*) 'End prveccombinat0'

      end if  ! ( debug )
      call erclos ( 'prveccombinat0' )

      end subroutine prveccombinat0
