      subroutine prveccombinat2 ( iseqvec, vecs, numvecrhs, lenvec, &
                                  istore, store, icount, numrealrhs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate right-hand side of vector expression
!     Special case: the complete complex vector must be computed
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     expression
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqvec, vecs(*), numvecrhs, &
              lenvec, istore(*), icount, numrealrhs
      double precision store(*)

!     icount         i    Computed length of array istore
!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     istore         i    Contains the expression as stored in common
!                         cexpression, where all scalars and reals
!                         have been computed and stored in store
!                         References to these values are given by
!                         their position in store plus 1000
!                         The values of the vectors are stored in
!                         array vectors
!                         References to these vectors are given by
!                         their position in vectors plus 10000
!     lenvec         i    Length of left-hand side vector in expression
!     numrealrhs     i    Number of reals stored in store
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     store          i    Contains real information of the expression
!                         See subroutine READEXPR, array work
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double complex, allocatable :: vectors(:,:)

!     vectors        Array of size lenvec x numvecrhs to store the vectors
!                    of the right-hand side in the nodes required
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PREVALVECCEXP  Evaluate vector expression and fill actual values
!                    of vectors (complex case)
!     PRVECFILLC     Fill all vectors of right-hand side of expression
!                    in array vectors (complex case)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prveccombinat2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prveccombinat2'
         write(irefwr,1) 'lenvec, numvecrhs', lenvec, numvecrhs
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( vectors(lenvec,numvecrhs), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lenvec*numvecrhs, 'vectors' )
      if ( ierror/=0 ) go to 1000

!     --- Fill all vectors

      call prvecfillc ( iseqvec, vecs, numvecrhs, vectors, lenvec )

!     --- Fill output vector

      usoltemp => ks(isolut2d(iseqvec))%sol

      call prevalveccexp ( usoltemp, vectors, lenvec, &
                           numvecrhs, istore, store, icount, &
                           numrealrhs, names(iseqvec) )

!     --- Deallocate vectors

      if ( allocated(vectors) ) deallocate ( vectors, stat = error )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prveccombinat2'

      end if  ! ( debug )
      call erclos ( 'prveccombinat2' )

      end subroutine prveccombinat2
