      subroutine prvecfind ( inpsep, ivec, itypvc, iseqvec, &
                             vecs, numvecrhs, complexvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 04-09-2016 New parameter list
!        version  1.1  date 12-02-2015 Use prvecfind1
!        version  1.0  date 22-12-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the type of vector to be created for the left-hand side
!     This depends on the right-hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleinput
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), ivec, itypvc, iseqvec, vecs(*), numvecrhs
      logical complexvec

!     complexvec     o    If true the vector to be filled is complex
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine ASKVEC
!     iseqvec        i    Starting address of sequence numbers of vectors
!     itypvc         o    Indication of the type of solution vector:
!                         110: solution vector
!                         115, 116: vectors of special structure
!     ivec           o    Sequence number of vector of special structure
!     numvecrhs      i    Number of vectors in vecs
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PRVECFIND1     Find the type of vector to be created for the left-hand side
!                    This depends on the right-hand side
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfind' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfind'
         write(irefwr,1) 'iseqvec', iseqvec
         call prinin1 ( isolut, 4, 5, 'isolut' )
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprob = inpsep(2)
      ivec = inpsep(7)

      complexvec = .false.

      if ( debug ) write(irefwr,1) 'iprob, ivec', iprob, ivec

      if ( iprob==0 .and. ivec==0 .and. numvecrhs>0 ) then

!     --- Both iprob = 0 and ivec = 0, hence check the vectors in the
!         right-hand side of the expression

         call prvecfind1 ( ivec, itypvc, iseqvec, &
                           vecs, numvecrhs, complexvec )

      else

!     --- either iprob or ivec given

         iprob = max(1,iprob)
         itypvc = 110
         if ( ivec>0 ) itypvc = 115

      end if  ! ( iprob==0 .and. ivec==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'complexvec ', complexvec
         write(irefwr,1) 'iprob, ivec, itypvc', iprob, ivec, itypvc
         write(irefwr,*) 'End prvecfind'

      end if  ! ( debug )
      call erclos ( 'prvecfind' )

      end
