      subroutine prvecfind1 ( ivec, itypvc, iseqvec, vecs, &
                              numvecrhs, complexvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 04-09-2016 New parameter list
!        version  1.0  date 09-02-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the type of vector to be created for the left-hand side
!     This depends on the right-hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec, itypvc, iseqvec, vecs(*), numvecrhs
      logical complexvec

!     complexvec     o    If true the vector to be filled is complex
!     iseqvec        i    Starting address of sequence numbers of vectors
!     itypvc         o    Indication of the type of solution vector:
!                         110: solution vector
!                         115, 116: vectors of special structure
!     ivec           o    Sequence number of vector of special structure
!     numvecrhs      i    Number of vectors in vecs
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenvec, iseqv, i, nunkploc, lenpcomp, &
              lenpcomp2, iseqfinal
      character (len=32) name1, name2, nameiseqv

!                    they are not
!     i              Counting variable
!     iseqfinal      Sequence number of vector in isolut that must be used
!                    to define the type of vector
!     iseqv          Sequence number of vector in isolut
!     lenpcomp       Length of solution array for one component
!     lenpcomp2      Length of solution array for different array
!                    (one component)
!     lenvec         Length of left-hand side vector in expression
!     name1          Name of first vector in rhs
!     name2          Name of second vector in rhs
!     nameiseqv      Name of vector inlrhs
!     nunkploc       Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERROR3161      Print error message 3161
!     INISTREXT      Find the structure of the vector with exactly
!                    NUMBER unknowns per point
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfind1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfind1'
         call ertrace
         write(irefwr,1) 'iseqvec, maxvc1, numvecrhs, iseqkp', &
                          iseqvec, maxvc1, numvecrhs, iseqkp
         call prinin ( isolut2d, iseqvec, 'isolut' )
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      complexvec = .false.

      if ( debug ) write(irefwr,1) 'npoint, ndim', npoint, ndim

!     --- check the vectors in the
!         right-hand side of the expression

      if ( debug ) call prinin ( vecs, numvecrhs, 'vectors' )
      iseqv = vecs(1)
      if ( iseqvec<=maxvc1 ) then
         nameiseqv = names(iseqvec)
      else
         nunkploc = 1
         select case (iseqvec-maxvc1)
            case(1)
               nameiseqv = 'coor'
               nunkploc = ndim
            case(2)
               nameiseqv = 'x_coor'
            case(3)
               nameiseqv = 'y_coor'
            case(4)
               nameiseqv = 'z_coor'
         end select  ! case (iseqvec)
         iseqfinal = iseqvec
      end if  ! ( iseqvec<=maxvc1 )

      if ( debug ) write(irefwr,1) 'iseqv, maxvc1', iseqv, maxvc1

      if ( iseqv<=maxvc1 ) then

!     --- The first vector is an existing vector

         iseqkp = ks(isolut2d(iseqv))%iseqkp
         call sepactsolbf2 ( isolut2d(iseqv) )
         nunkploc = nunkpi
         lenvec = ks(isolut2d(iseqv))%nusol
         lenpcomp = lenvec/nunkploc
         iseqfinal = iseqv
         name1 = names(iseqv)
         complexvec = ks(isolut2d(iseqv))%compl==1
         itypvc = ks(isolut2d(iseqv))%typevec
         iprob = ks(isolut2d(iseqv))%iprob

         if ( debug ) &
            write(irefwr,1) 'iseqv, iprob, ivec, itypvc, iseqkp', &
                             iseqv, iprob, ivec, itypvc, iseqkp

      else if ( iseqv==maxvc1+1 ) then

!     --- The first vector is coor

         lenvec = npoint*ndim
         lenpcomp = npoint
         nunkploc = ndim
         name1 = 'coor'
         iseqfinal = iseqv

      else if ( iseqv<=maxvc1+1+ndim ) then

!     --- The first vector is x_coor (or y or z)

         lenvec = npoint
         lenpcomp = npoint
         nunkploc = 1
         select case(iseqv-maxvc1-1)
         case(1)
            name1 = 'x_coor'
         case(2)
            name1 = 'y_coor'
         case(3)
            name1 = 'z_coor'
         end select  ! case(iseqv-maxvc1-1)
         iseqfinal = iseqv

      end if  ! ( iseqvec<=maxvc1 )

      if ( debug ) then
         write(irefwr,1) 'iseqv, lenvec, lenpcomp, nunkploc', &
                          iseqv, lenvec, lenpcomp, nunkploc
         write(irefwr,*) 'name1 ', name1
      end if  ! ( debug )

!     --- Check if the rest of the vectors have the same length
!         If necessary define new length

      do i = 2, numvecrhs

         iseqv = vecs(i)

         if ( debug ) write(irefwr,1) 'i, iseqv, maxvc1', i, iseqv, maxvc1

         if ( iseqv<=maxvc1 ) then

!        --- The first vector is an existing vector

            call sepactsolbf2 ( isolut2d(iseqv) )
            length = ks(isolut2d(iseqv))%nusol
            lenpcomp2 = length/nunkpi

            if ( debug ) &
               write(irefwr,1) 'i, iseqv, length, lenvec, iseqkp', &
                                i, iseqv, length, lenvec, iseqkp

            if ( length/=lenvec ) then

!           --- The length of both vectors is different

               if ( lenpcomp==lenpcomp2 ) then

!              --- Per component we have the same length

                  if ( nunkpi>nunkploc .and. nunkploc==1 ) then

!                 --- New length found, previous one was 1

                     nunkploc = nunkpi
                     lenvec = length
                     iseqfinal = iseqv
                     name1 = names(iseqv)

                  else if ( nunkpi/=1 ) then

!                 --- The new vector has different length than the previous
!                     one

                     name2 = names(iseqv)
                     call error3161 ( lenvec, length, name1, name2, &
                                      nameiseqv )
                     go to 1000

                  end if  ! ( nunkpi>nunkploc .and. nunkploc==1 )

               else

!              --- The new vector has different length than the previous
!                  one

                   name2 = names(iseqv)
                   call error3161 ( lenvec, length, name1, name2, &
                                    nameiseqv )
                   go to 1000

               end if  ! ( lenpcomp==lenpcomp2 )

            end if  ! ( length/=lenvec )

            complexvec = ks(isolut2d(iseqv))%compl==1 .or. complexvec

         else if ( iseqv==maxvc1+1 ) then

!        --- The first vector is coor

            length = npoint*ndim
            name2 = 'coor'

            if ( length/=lenvec ) then

!           --- The length of both vectors is different

               if ( lenpcomp==npoint ) then

!              --- Per component we have the same length

                  if ( ndim>nunkploc .and. nunkploc==1 ) then

!                 --- New length found, previous one was 1

                     nunkploc = ndim
                     lenvec = length
                     iseqfinal = iseqv
                     name1 = 'coor'

                  else if ( ndim/=1 ) then

!                 --- The new vector has different length than the previous
!                     one

                     call error3161 ( lenvec, length, name1, name2, &
                                      nameiseqv )
                     go to 1000

                  end if  ! ( nunkpi>nunkploc .and. nunkploc==1 )

              else

!             --- The new vector has different length than the previous
!                 one

                  call error3161 ( lenvec, length, name1, name2, &
                                   nameiseqv )
                  go to 1000

               end if  ! ( lenpcomp==lenpcomp )

            end if  ! ( length/=lenvec )

         else if ( iseqv<=maxvc1+1+ndim ) then

!        --- The first vector is x_coor (or y or z)

            length = npoint
            select case(iseqv-maxvc1-1)
            case(1)
               name2 = 'x_coor'
            case(2)
               name2 = 'y_coor'
            case(3)
               name2 = 'z_coor'
            end select  ! case(iseqv-maxvc1-1)

            if ( debug ) write(irefwr,1) 'length, lenvec', length, lenvec

            if ( length/=lenvec ) then

!           --- The length of both vectors is different

               if ( lenpcomp==npoint ) then

!              --- Per component we have the same length

                  if ( ndim>nunkploc .and. nunkploc==1 ) then

!                 --- New length found, previous one was 1

                     nunkploc = 1
                     lenvec = length
                     iseqfinal = iseqv
                     name1 = name2

                  else

!                 --- The new vector has different length than the previous
!                     one

                     call error3161 ( lenvec, length, name1, name2, &
                                      nameiseqv )
                     go to 1000

                  end if  ! ( nunkpi>nunkploc .and. nunkploc==1 )

               else

!              --- The new vector has different length than the previous
!                  one

                   call error3161 ( lenvec, length, name1, name2, &
                                    nameiseqv )
                   go to 1000

               end if  ! ( lenpcomp==lenpcomp )

            end if  ! ( length/=lenvec )

         end if  ! ( iseqvec<=maxvc1 )

         if ( debug ) write(irefwr,1) 'length', length

      end do  ! i = 2, numvecrhs

!     --- define iprob etc.

      if ( debug ) write(irefwr,1) 'iseqfinal, maxvc1', iseqfinal, maxvc1

      if ( iseqfinal<=maxvc1 ) then

!     --- standard array

         iprob = ks(isolut2d(iseqfinal))%iprob
         ivec = ks(isolut2d(iseqfinal))%ivec
         itypvc = 110
         if ( ivec>0 ) itypvc = 115

      else

!     --- Coordinates array

         call inistrext ( nunkploc, itypvc, ivec )

      end if  ! ( iseqfinal<=mavc1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'complexvec ', complexvec
         write(irefwr,1) 'iprob, ivec, itypvc, iseqkp', &
                          iprob, ivec, itypvc, iseqkp
         write(irefwr,*) 'End prvecfind1'

      end if  ! ( debug )
      call erclos ( 'prvecfind1' )

      end
