      subroutine prvecstrhs ( seqexpress, numvecrhs, numrealrhs, icount, &
                              istore, store, vecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 26-10-2015 Correction time
!        version  1.1  date 11-03-2014 replace inpsep by seqexpress
!        version  1.0  date 22-12-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Translate expression to a set of reals and a new set of
!     temporary vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     expression
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexpr
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: seqexpress
      integer, intent(out) :: numvecrhs, numrealrhs, icount, istore(*), &
                              vecs(*)
      double precision, intent(out) :: store(*)

!     icount         o    Length of arrays istore and store
!     istore         o    Contains the expression as stored in common
!                         cexpression, where all scalars and reals
!                         have been computed and stored in store
!                         References to these values are given by
!                         their position in store plus 1000
!                         The values of the vectors are stored in
!                         array vectors
!                         References to these vectors are given by
!                         their position in vectors plus 10000
!     numrealrhs     o    Number of reals stored in store
!     numvecrhs      o    Number of vectors in right-hand side
!                         Length of array vecs
!     seqexpress     i    Sequence number of expression in common cexpression
!     store          o    Contains values of reals and scalars, see istore
!     vecs           o    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istart, iend, i

!     i              Counting variable
!     iend           Last position used at a part
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getvecindex
      double precision getscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETSCAL        Return with value of scalar
!     GETVECINDEX    Compute the adapted value of ivec
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecstrhs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecstrhs'
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = istexpress(seqexpress-1)+1  ! starting address in intexpress
      iend = istexpress(seqexpress)        ! end address

      if ( debug ) then
         write(irefwr,1) 'seqexpress, istart, iend', &
                          seqexpress, istart, iend
         call prininlarge ( intexpress(istart), iend-istart+1, &
                            'intexpress' )

      end if  ! ( debug )

      numvecrhs = 0
      numrealrhs = 0
      icount = 0
      do i = istart, iend

         if ( debug ) &
            write(irefwr,1) 'i, intexpress(i)', i, intexpress(i)

         if ( intexpress(i)>150000000 ) then

!        --- Vector found, store in array vecs

            numvecrhs = numvecrhs+1
            vecs(numvecrhs) = getvecindex(intexpress(i))
            icount = icount+1
            istore(icount) = numvecrhs+10000

            if ( debug ) then
               write(irefwr,1) 'numvecrhs, icount', numvecrhs, icount
               write(irefwr,1) 'vecs(numvecrhs), istore(icount)', &
                                vecs(numvecrhs), istore(icount)
            end if  ! ( debug )

         else if ( intexpress(i)>1000 ) then

!        --- Real of scalar found, store in array store

            numrealrhs = numrealrhs+1
            icount = icount+1
            istore(icount) = numrealrhs+1000

            if ( debug ) &
               write(irefwr,1) 'numrealrhs, icount', numrealrhs, icount

            if ( intexpress(i)>=100000000 ) then

!           --- Scalar found

               store(numrealrhs) = getscal(intexpress(i))

            else

!           --- Real found

               store(numrealrhs) = realexpress(intexpress(i)-1000)

            end if  ! ( intexpress(i)>100000000 )

         else

!        --- operator found, store in array istore

            icount = icount+1
            istore(icount) = intexpress(i)

         end if  ! ( intexpress(i)>150000000 )

      end do  ! i = istart, iend

1000  if ( debug ) then

!     --- Debug information

         call prinin ( vecs, numvecrhs, 'vectors' )
         call prinrl ( store, numrealrhs, 'reals' )
         call prinin ( istore, icount, 'istore' )
         write(irefwr,*) 'End prvecstrhs'

      end if  ! ( debug )
      call erclos ( 'prvecstrhs' )

      end
