      subroutine putconvararray ( iseqarrglob, array, name, numcomps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the values of a constant or scalar array from
!     a local array into cons
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     array
!     constant
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqarrglob, numcomps
      double precision array(numcomps)
      character (len=*) name

!     array          i    Local array from which the values must be copied
!     iseqarrglob    i    Sequence number of array as stored in inpsep
!     name           i    Name of array
!     numcomps       i    Number of components
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqarr

!     iseqarr        Local value of iseqarrglob
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3136   constant or scalar does not correspond to an array
!    3137   dimension of constant, vector or scalar not correct
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'putconvararray' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from putconvararray'
         write(irefwr,1) 'iseqarrglob', iseqarrglob
  1      format ( a, 1x, (10i11) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqarr = mod(iseqarrglob,100000000)
      if ( iseqarr<5000 ) then

!     --- error: index does not belong to an array

         call errchr ( name, 1 )
         call errint ( iseqarr, 1 )
         call errsub ( 3136, 1, 0, 1 )
         go to 1000

      end if  ! ( iseqarr<5000 )

      if ( debug ) write(irefwr,1) 'iseqarr', iseqarr

      iseqarr = mod(iseqarr,5000)

      if ( iseqarr>maxints+maxreals ) then

!     --- scalar array found

         iseqarr = iseqarr-maxints-maxreals
         if ( dimscals(iseqarr)/=numcomps ) then

!        --- dimension incorrect

            call errchr ( name, 1 )
            call errint ( dimscals(iseqarr), 1 )
            call errint ( numcomps, 2 )
            call errsub ( 3137, 2, 0, 1 )
            go to 1000

         end if  ! ( dimscals(iseqarr)/=numcomps )

         scalars(iseqarr:iseqarr+numcomps-1) = array(1:numcomps)

      else

!     --- constants array found

         if ( dimcons(iseqarr)/=numcomps ) then

!        --- dimension incorrect

            call errchr ( name, 1 )
            call errint ( dimcons(iseqarr), 1 )
            call errint ( numcomps, 2 )
            call errsub ( 3137, 2, 0, 1 )
            go to 1000

         end if  ! ( dimcons(iseqarr)/=numcomps )

         if ( iseqarr>maxints ) then

!        --- real found

            iseqarr = iseqarr-maxints
            rlcons(iseqarr:iseqarr+numcomps-1) = array(1:numcomps)

         else

!        --- integer found

            incons(iseqarr:iseqarr+numcomps-1) = nint(array(1:numcomps))

         end if  ! ( iseqarr>maxints )

      end if  ! ( iseqarr>maxints+maxreals )

1000  call erclos ( 'putconvararray' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End putconvararray'

      end if  ! ( debug )

      end
