      subroutine readb7 ( numarr, iarray )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.1  date 15-11-2012 Long integers
!        version  3.0  date 14-04-1999 Extra parameter ibuffr
!        version  2.3  date 24-07-1998 include commons
!
!   copyright (c) 1988-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     readb7 reads integer user array from backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulefiles
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr, iarray(*)

!    iarray    o   Integer user array
!    numarr    i   Array number with respect to the index in common cfile2
!                  for array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer index
      integer, allocatable :: iwork(:)

!     index          Record number with respect to file 2 from which array
!                    must be read
!     iwork          integer work array to store records read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readb7' )
      length = iwork3 ( 2*numarr )
      index  = iwork3 ( 2*numarr-1 )

      allocate ( iwork(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Read all information from file 2

      call readfl ( index, 1, length, 1, 2, iwork )
      if ( ierror/=0 ) go to 1000

!     --- Replace all information

      iarray(1:length-2) = iwork(3:length)

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'readb7' )
      end subroutine readb7
