      subroutine readcostring ( iseq, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 06-12-2013 Extra error message data not recognized
!        version  1.0  date 01-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read part strings of input block CONSTANTS
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     constant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleread
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseq, iwork(*)

!     iseq           i    Sequence numberof vector in VECOLD/ISLOLD
!     iwork         i/o   integer work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq1

!     iseq1          Shifted sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1879   Not a text found after the keyword
!    2068   sequence number of the vector read is incorrect
!    2143   Sequence number already in use
!    3156   Text not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readcostring' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readcostring'
         write(irefwr,1) 'nextxt', nextxt
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iseq==0 ) iseq = nextxt
      iseq1 = iseq+maxints+maxreals+maxcls+maxvc1

      if ( iwork(iseq1)/=0 ) then

!     --- error, sequence number already in use

         call errint ( iseq, 1 )
         call errchr ( 'strings' , 1 )
         call errsub ( 2143, 1, 0, 1 )

      end if

      iwork(iseq1) = 1
      numstrings = max(numstrings,iseq)
      if ( numstrings>=maxstrings ) then

!     --- Too much vector_names

         call errint ( numstrings, 1 )
         call errint ( maxvc1-1, 2 )
         call errsub ( 2068, 2, 0, 0 )
         call errcrd

      else

!     --- store name into texts

         chstrings(iseq)(1:32) = stringim(1:32)
         ncharstrings(iseq) = nchar
         if ( debug ) then
            write(irefwr,1) 'iseq, nchar', iseq, nchar
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

!        --- Read string

         call asknxt
         if ( debug ) write(irefwr,1) 'ityprd', ityprd
         if ( ityprd==5 ) then

!        --- text found

            instring(iseq) = nextxt
            texts(nextxt) = string(1:nchar)
            nextxt = nextxt+1

         else if ( ityprd==1 ) then

!        --- no text found, just raise nextxt

            instring(iseq) = nextxt
            texts(nextxt) = ''
            nextxt = nextxt+1

         else

!        --- not a text found

            call errchr ( 'TEXT', 1 )
            call errsub ( 1879, 0, 0, 1 )
            call errcrd

         end if  ! ( ityprd==5 )

         if ( lastitem<nitems ) then

!        --- Error text not recognized

            call errsub ( 3156, 0, 0, 0 )
            call errcrd
            go to 1000

         end if  ! ( lastitem<nitems )

      end if

1000  call erclos ( 'readcostring' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readcostring'

      end if  ! ( debug )

      end
