      subroutine readdf ( iref, defar, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-10-1992 New message numbers
!        version  1.0  date 13-12-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Reads defaults from the file with reference number iref
!       It is assumed that subroutine errchr has been called before with the
!       name of environment file as first character variable
! **********************************************************************
!
!                       KEYWORDS
!
!     default
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iref, n
      double precision defar(n)

!     defar   o   Contains the defaults that have been reed from the file
!     iref    i   File reference number from which the defaults must be read
!     n       i   Number of defaults to be read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, irefsv

!    inum     Number of integers read
!    irefsv   Help variable to save irefre temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  Error messages
!     ERTEXT  Write message from error message file to screen
!     READSR  Read a series of numbers from the file
! **********************************************************************
!
!                       I/O
!
!    Input is read from the environment file
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   1822:  environment file incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Save reference number for reading from screen
!   Set reading reference number for reading from environment file
!   Read numbers from environment file
!   If ( not enough numbers are available ) then
!      error: incorrect environment file
!   Reset reference number for reading from screen
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readdf' )

!     --- Make sure that end of record is skipped

      irefsv = irefre
      irefre = iref

      istop = 2
      call readsr ( defar, n, inum )
      istop = 0

      if ( inum.lt.n ) then

!     --- Error environment file is incorrect

         call errsub ( 1822, 0, 0, 1 )
         call ertext ( 1, 0, 0, 0 )
         read(*,*)
         go to 1000

      end if

!     --- reset istop

1000  irefre = irefsv

      call erclos ( 'readdf' )
      end
