      subroutine readinteger ( intread, minval, maxval, name )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read next item of input file and check if it is an integer with
!     values between minval and maxval
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intread, minval, maxval
      character *(*) name

!     intread        o    Integer read
!     maxval         i    Maximum value allowed for the integer read
!     minval         i    Minimum value allowed for the integer read
!     name           i    Name of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1870   Integer number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!     read next item
!     Check if it is of type integer
!     Set value
!     Check if the value is between minval and maxval
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readinteger' )

!     --- read next item

      call asknxt

!     --- Check if it is of type integer

      if ( ityprd.ne.2 ) then

!     --- item read is not of type integer

         call errchr ( name, 1 )
         call errsub ( 574, 0, 0, 1 )
         call errcrd
         intread = minval
         go to 1000

      end if

!     --- Set value

      intread = nint(realrd)

!     --- Check if the value is between minval and maxval

      if ( intread.lt.minval .or. intread.gt.maxval ) then

!     --- intread out of range

         call errchr ( name, 1 )
         call errint ( intread, 1 )
         call errint ( minval, 2 )
         call errint ( maxval, 3 )
         call errsub ( 1870, 3, 0, 1 )
         call errcrd
         intread = minval
         go to 1000

      end if
1000  call erclos ( 'readinteger' )
      end
