      subroutine readm4 ( ivers, stoper )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 02-04-2008 Fortran 90
!        version  2.0  date 02-01-2003 Complete revision
!        version  1.3  date 28-12-2000 Length of records extended to 240
!        version  1.2  date 10-02-1994 New comments
!        version  1.1  date 06-11-1992 Remove text in case of incorrect file
!        version  1.0  date 17-10-1992
!
!   copyright (c) 1992-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read second line in SEPRAN environment file. This line contains the
!   version number of the file. Print a warning if the number is too low
!   If no version number is found the next line is interpreted
! **********************************************************************
!
!                       KEYWORDS
!
!     environment_file
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character *(*) stoper
      integer ivers

!    ivers    o   Version number of environment file
!    stoper   o   Type of operating system
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical found

!     found          If true already a possibility has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     READCD  Read a record from the standard input file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   Error:    No colon found
!   Warning:  Old version of SEPRAN environment file
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Read next record (comments excluded):  READCD
!      Find colon
!      Read item after colon
!      If ( item = real ) then
!         version number found
!         if  version number too small ) give warning
!         Store version number into ivers
!      else ( item = text ) then
!         give warning
!         ivers := 1
!         stoper := text
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Read record (Do not translate into capitals)

      ispecl = 7
      ityprd = 1
      call readcd
      ispecl = 6

!     --- Find colon

      found = .false.
      do while ( .not. found .and. lastitem<nitems-1 )
         lastitem = lastitem+1
         if ( typeitems(lastitem)==16 ) found =.true.
      end do
      if ( .not. found ) then

!     --- No colon found:

         write(*,100) cardimage(1)(1:lastcolumn(1))
100      format ( ' Line in the environment file does not contain a', &
                  ' colon(:). '/' Execution is terminated'/ &
                  ' The line read is:'/1x,a )
         stop

      end if
      lastitem = lastitem+1

      if ( typeitems(lastitem)==3 ) lastitem = lastitem+1

      if ( typeitems(lastitem)==1 ) then

!     --- Version number found after colon
!         Check this number

         ivers = nint(realitems(lastitem))
         if ( ivers>2 ) then

!        --- ivers>2

            write(*,110) ivers, 2, cardimage(1)(1:lastcolumn(1))
110         format(//' The version number found in the environment', &
                     ' file is ', i3/' whereas the present version', &
                     ' should be at most', i2/ &
                     ' The line read in the environment file is:'/a/ &
                     ' Execution is terminated' )
            stop

         else if ( ivers<2 ) then

!        --- ivers<2

            write(*,120) ivers, 2, cardimage(1)(1:lastcolumn(1))
120         format(//' The version number found in the environment', &
                     ' file is ', i3/' whereas the present version', &
                     ' should be', i2/ &
                     ' The line read in the environment file is:'//a// &
                     ' Update your environment file as soon as', &
                     ' possible'// )

         end if

      else

         ivers = 1
         write(*,130) 2
130      format(//' No version number found in the environment', &
                  ' file'/ ' whereas the present version should be',i2/ &
                  ' Update your environment file as soon as', &
                  ' possible'// )

         stoper = cardimage(1)(1:lastcolumn(1))

      end if

!     --- Fill text

      end
