      subroutine readseqvar ( name, rinput, iinput, istrin, defvalue )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 14-04-2013 Allow constants
!        version  2.1  date 31-07-2012 Extra debug
!        version  2.0  date 30-01-2012 Extra parameter defvalue
!
!   copyright (c) 2011-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the value of the variable read in rinput
!     If no sequence number is found, check if the variable is known
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision rinput(*), defvalue
      integer iinput, istrin
      character (len=*) name

!     defvalue       i    Default value
!     iinput         o    Entry of integer input array referring to variable
!                         If value stored in table(icode) iinput contains the
!                         entry in rinput where the value is stored
!                         If the variable is a scalar minus the entry
!                         in scalars is stored in iinput
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINPUT
!                         If a value is stored in rinput, istrin is raised
!     name           i    Name of variable
!                         If no input is found, it is checked if this name
!                         is stored in the set of scalars
!     rinput        i/o   Real input array
!                         If a real is read the value is stored in
!                         rinput(istrin)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq

!     iseq           Sequence number in set of scalars
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnamevar, getnameint, getnamereal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETNAMEINT     Return with sequence number of integer in incons
!     GETNAMEREAL    Return with sequence number of real in rlcons
!     GETNAMEVAR     Return with sequence number of variable in scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readseqvar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readseqvar'
         write(irefwr,1) 'icode, infotable(icode), istrin', &
                          icode, infotable(icode), istrin
         write(irefwr,2) 'table(icode), tabmin(icode)', &
                          table(icode), tabmin(icode)
  1      format ( a, 1x, (10i9) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( infotable(icode)==11 ) then

!     --- variable found

         iseq = nint(table(icode))
         iinput = iseq+maxints+maxreals

      else if ( table(icode)>=tabmin(icode) ) then

!     --- value is given

         rinput(istrin) = table(icode)
         iinput = -istrin
         istrin = istrin+1

      else

!     --- value is not given, test if there is a variable with name
!         name

         iseq = getnamevar ( name )
         if ( debug ) write(irefwr,1) 'iseq', iseq
         if ( iseq>0 ) then

!        --- iseq>0, variable name found

            iinput = iseq+maxints+maxreals+100000000

         else

!        --- iseq = 0, variable name not found
!            Check if it is an integer or real

            iseq = getnameint ( name )
            if ( iseq>0 ) then
               iinput = iseq+100000000  ! integer found
            else
               iseq = getnamereal ( name )
               if ( iseq>0 ) then
                  iinput = iseq+maxints+100000000  ! real found
               end if  ! ( iseq>0 )
            end if  ! ( iseq>0 )

            if ( iseq==0 ) then

!           --- Use default value

               rinput(istrin) = defvalue
               iinput = -istrin
               istrin = istrin+1

            end if  ! ( iseq==0 )

         end if  ! ( iseq>0 )

      end if  ! ( infotable(icode)==11 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinput, istrin', iinput, istrin
         write(irefwr,*) 'End readseqvar'

      end if  ! ( debug )
      call erclos ( 'readseqvar' )

      end subroutine readseqvar
