      subroutine readstrfunc ( numfun, stringlc, ncharlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 10-04-2014 Extra parameters
!        version  1.2  date 27-10-2011 Extension with erfc
!        version  1.1  date 30-06-2011 Extension with abs
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if name read corresponds to one the known functions
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleread
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numfun, ncharlc
      character (len=ncharlc) stringlc

!     ncharlc        i    Number of characters in stringlc
!     numfun         o    Sequence number of function in accordance
!                         with iwork (see readconstant)
!                         If 0, no function has been found
!     stringlc       i    String to be analyzed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncode1
      parameter ( ncode1=14 )
      integer nchars(ncode1)
      character (len=10) code1(ncode1)
      logical found

!     code1          Table with keywords corresponding to special functions
!     found          If true already a possibility has been found
!     nchars         Contains number of characters in functions
!     ncode1         Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'SQRT', 'SIN', 'COS', 'EXP', 'TAN', 'ASIN', 'ACOS', &
                   'ATAN', 'LOG', 'ABS', 'ERFC', 'ARCSIN', 'ARCCOS', &
                   'ARCTAN' /
      data nchars / 4, 3, 3, 3, 3,  4, 4, 4, 3, 3, &
                    4, 6, 6, 6 /
! ======================================================================
!
      call eropen ( 'readstrfunc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readstrfunc'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      found = .false.
      numfun = 0       ! number of functions checked

      do while ( numfun<ncode1 .and. .not. found )

!     --- Function not yet found, raise numfun

         numfun = numfun+1
         if ( stringlc(1:ncharlc)==code1(numfun)(1:nchars(numfun)) ) then

!        --- Function found

            if ( numfun>11 ) numfun = numfun-6  ! correction arc --> a
            if ( nitems>lastitem .and. typeitems(lastitem+1)==5 ) &
               found = .true.   ! next item is a (

         end if  ! ( string(1:nchar)==code(numfun)(1:nchar) )

      end do  ! while ( numfun<ncode .and. .not. found )

      if ( found ) then

!     --- Function found, raise numfun by 9

         numfun = 9+numfun

      else

!     --- No function found, set numfun 0

         numfun = 0

      end if  ! ( found )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numfun', numfun
         write(irefwr,*) 'End readstrfunc'

      end if  ! ( debug )
      call erclos ( 'readstrfunc' )

      end
