      subroutine sepcrdiagmatr ( matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 240-04-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create a new diagonal matrix in realmt
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: matr

!     matr         i/o   Defines matrix information by integer sequence number
!                        If it already exists the old info is removed
!                        New matr info is created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart
      type (globmat), pointer :: S
      type (intmatrix), pointer :: IM
      type (rmatrix), pointer :: Ar
      type (cmatrix), pointer :: Ac

!     Ac             Refers to realmt(matr) (complex matrix)
!     Ar             Refers to realmt(matr) (real matrix)
!     IM             Refers to intmt(intmat)
!     S              pointer to realmt(iseqmt)
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCRMATPART1  create a new array in realmt without allocation of space
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcrdiagmatr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcrdiagmatr'
         write(irefwr,1) 'lastmatrix, nummat, matr', &
                          lastmatrix, nummat, matr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcrmatpart1 ( matr )

      IM => intmt(intmat)

      S => realmt(matr)

      length = 0
      if ( associated(IM%colsout) ) length = length+size(IM%colsout)
      if ( associated(IM%colsfi) ) length = length+2*size(IM%colsfi)

      nullify ( S%S_pi, S%S_ip, S%S_pp, S%S_ic, S%matinv, S%Dlapshift  )
      nullify ( S%S_if, S%S_fi, S%S_iout )

      S%diagonal = .true.
      length = length+IM%nusol

      allocate ( S%S_ii(length) )
      ipstart = IM%nusol

      if ( associated(IM%colsfi) ) then

!     --- There are interface unknowns

         length = size(IM%colsfi)
         S%S_if => S%S_ii(ipstart+1:ipstart+length)
         ipstart =ipstart+length
         S%S_fi => S%S_ii(ipstart+1:ipstart+length)
         ipstart =ipstart+length

      end if  ! ( associated(IM%colsfi) )

      if ( associated(IM%colsout) )  then
         length = size(IM%colsout)
         S%S_iout => S%S_ii(ipstart+1:ipstart+length)
      end if  ! ( associated(IM%colsout) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'matr, lastmatrix, matr', &
                          matr, lastmatrix, matr
         write(irefwr,*) 'End sepcrdiagmatr'

      end if  ! ( debug )
      call erclos ( 'sepcrdiagmatr' )

      end subroutine sepcrdiagmatr
