      subroutine sepcrintmt ( intmatlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-12-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create a new array in intmt
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: intmatlc

!     intmatlc     i/o   Defines matrix structure by integer sequence number
!                        If it already exists the old info is removed
!                        New intmat info is created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     SEPRMINTMT     Remove array with sequence number iseq from intmt
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3223   all arrays in use
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcrintmt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcrintmt'
         call ertrace
         write(irefwr,1) 'intmatlc, lastintmat, numintmat', &
                          intmatlc, lastintmat, numintmat
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( intmatlc>0 .and. intmatlc<=lastintmat ) call seprmintmt ( intmatlc )

      if ( nextfreeintmt<=numintmat ) then

!     --- nextfreeintmt in proper range

         intmatlc = nextfreeintmt
         do i = intmatlc+1, numintmat
            if ( intmatarr(i)==0 ) then
               nextfreeintmt = i
               exit
            end if  ! ( intmatarr(i)==0 )
         end do  ! i = intmatlc+1, numintmat
         lastintmat = max(lastintmat, intmatlc)

      else

!     --- no free entries left

         call errchr ( 'intmat', 1 )
         call errsub ( 3223, 0, 0, 1 )
         go to 1000

      end if  ! ( nextfreeintmt<=numintmat )

      IM => intmt(intmatlc)

      IM%iseqkp = iseqkp
      IM%icounter = 0
      IM%spectral = .false.
      intmatarr(intmatlc) = 1
      lastintmat = max(lastintmat, intmatlc)
      iseqim = intmatlc
      IM%ninterface = 0
      IM%nbound = nbound
      IM%npboun = npboun
      IM%nrusol = nrusol
      IM%nusol = nusol
      IM%signmethod = 0
      IM%iprob = iprob
      IM%typecoupl = 0
      IM%nu = 0
      IM%np = 0

      nullify ( IM%rowsii, IM%colsii, IM%rowsip, IM%colsip, &
                IM%rowspp, IM%colspp, IM%rowsic, IM%colsic, &
                IM%rowsif, IM%colsif, IM%rowsfi, IM%colsfi, &
                IM%rowsout, IM%colsout, IM%storeinfo, IM%sendunknowns, &
                IM%recunknowns, IM%diag, IM%rowsQ, IM%colsQ, &
                IM%rowsG, IM%colsG, IM%rowsDG, IM%colsDG, &
                IM%rowsPM, IM%colsPM, IM%diag, IM%diagQ, &
                IM%rowsiiprec, IM%colsiiprec, IM%diagprec )
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'intmatlc, lastintmat', intmatlc, lastintmat
         write(irefwr,*) 'End sepcrintmt'

      end if  ! ( debug )
      call erclos ( 'sepcrintmt' )

      end subroutine sepcrintmt
