      subroutine sepcrmeshinfo ( kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create a new array in km
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: kmesh

!     kmesh        i/o   Defines mesh by integer sequence number
!                        If it already exists the old mesh info is removed
!                        New meshinfo is created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i         Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3223   all arrays in use
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcrmeshinfo' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcrmeshinfo'
         call ertrace
         write(irefwr,1) 'kmesh, lastmesh, nummesh, nextfreekm', &
                          kmesh, lastmesh, nummesh, nextfreekm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kmesh>0 .and. kmesh<=lastmesh ) call seprmmeshinfo ( kmesh )

      if ( nextfreekm<=nummesh ) then

!     --- nextfreekm in proper range

         kmesh = nextfreekm
         do i = kmesh+1, nummesh
            if ( mesharr(i)==0 ) then
               nextfreekm = i
               exit
            end if  ! ( mesharr(i)==0 )
         end do  ! i = kmesh+1, nummesh
         lastmesh = max(lastmesh, kmesh)

      else

!     --- no free entries left

         call errchr ( 'kmesh', 1 )
         call errsub ( 3223, 0, 0, 1 )
         go to 1000

      end if  ! ( nextfreekm<=nummesh )

      km(kmesh)%icounter = 0
      mesharr(kmesh) = 1
      lastmesh = max(lastmesh, kmesh)

      if ( debug ) write(irefwr,1) 'kmesh', kmesh

      allocate ( km(kmesh)%kmesh(80), stat = error )
      if ( error/=0 ) call eralloc ( error, 80, 'km(kmesh)%kmesh' )
      km(kmesh)%kmesh = 0

      nullify(km(kmesh)%kmesha)
      nullify(km(kmesh)%kmeshb)
      nullify(km(kmesh)%kmeshf)
      nullify(km(kmesh)%kmeshg)
      nullify(km(kmesh)%kmeshu)
      nullify(km(kmesh)%kmesht)

      nullify(km(kmesh)%coor)
      nullify(km(kmesh)%kmeshc)
      nullify(km(kmesh)%kmeshd)
      nullify(km(kmesh)%kmshd1)
      nullify(km(kmesh)%kmshd2)
      nullify(km(kmesh)%kmeshe)
      nullify(km(kmesh)%kmshe1)
      nullify(km(kmesh)%kmshe2)
      nullify(km(kmesh)%kmeshh)
      nullify(km(kmesh)%curvesmsh)
      nullify(km(kmesh)%icurvsmsh)
      nullify(km(kmesh)%icurvsobst2d)
      nullify(km(kmesh)%iactobst)
      nullify(km(kmesh)%iobscurv)
      nullify(km(kmesh)%curvefacts)
      nullify(km(kmesh)%kmeshj)
      nullify(km(kmesh)%kmeshk)
      nullify(km(kmesh)%kmeshl)
      nullify(km(kmesh)%kmeshm)
      nullify(km(kmesh)%kmeshn)
      nullify(km(kmesh)%kmesho)
      nullify(km(kmesh)%kmsho1)
      nullify(km(kmesh)%kmsho2)
      nullify(km(kmesh)%elsize)
      nullify(km(kmesh)%elsize)
      nullify(km(kmesh)%kmeshq)
      nullify(km(kmesh)%kmeshr)
      nullify(km(kmesh)%kmeshs)
      nullify(km(kmesh)%kmeshs2)
      nullify(km(kmesh)%iinputkm)
      nullify(km(kmesh)%rinputkm)
      nullify(km(kmesh)%kmeshv)
      nullify(km(kmesh)%kmshv1)
      nullify(km(kmesh)%kmshv2)
      nullify(km(kmesh)%kmeshx)
      nullify(km(kmesh)%kmeshx)
      nullify(km(kmesh)%kmeshz)
      nullify(km(kmesh)%levelstruc)
      nullify(km(kmesh)%surfnodes)
      nullify(km(kmesh)%normal)
      nullify(km(kmesh)%pointlevel)
      nullify(km(kmesh)%rmeshx)
      nullify(km(kmesh)%intprop)
      nullify(km(kmesh)%realprop)
      nullify(km(kmesh)%ielgrp_nodes)
      nullify(km(kmesh)%node_elgrps)
      nullify(km(kmesh)%transinfo)
      nullify(km(kmesh)%blockinfo)
      nullify(km(kmesh)%outerboun)
      nullify(km(kmesh)%curveusrp)
      nullify(km(kmesh)%iouterbounseq)
      nullify(km(kmesh)%kmeshquant)
      nullify(km(kmesh)%mappar)
      nullify(km(kmesh)%rennested)
      iseqkm = kmesh

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'kmesh, lastmesh', kmesh, lastmesh
         write(irefwr,*) 'End sepcrmeshinfo'

      end if  ! ( debug )
      call erclos ( 'sepcrmeshinfo' )

      end subroutine sepcrmeshinfo
