      subroutine sepcrprobinfo ( kproblc, numprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create a new array in kp
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numprob
      integer, intent(inout) :: kproblc

!     kproblc      i/o   Defines problem structure by integer sequence number
!                        At output its value is iseqkp
!     numprob       o    Number of problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i         Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
!     SEPRMPROBINFO  Remove array with sequence number kproblc from kp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3223   all arrays in use
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcrprobinfo' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcrprobinfo'
         call ertrace
         write(irefwr,1) 'lastprob, nummesh, numprob, kproblc', &
                          lastprob, nummesh, numprob, kproblc
         call prinin ( probarr, lastprob, 'probarr' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kproblc>0 .and. kproblc<=lastprob ) &
         call seprmprobinfo ( kproblc )

      if ( nextfreekp<=nummesh ) then

!     --- nextfreekp in proper range

         kproblc = nextfreekp
         do i = kproblc+1, nummesh
            if ( probarr(i)==0 ) then
               nextfreekp = i
               exit
            end if  ! ( probarr(i)==0 )
         end do  ! i = kproblc+1, nummesh
         lastprob = max(lastprob, kproblc)

      else

!     --- no free entries left

         call errchr ( 'intmat', 1 )
         call errsub ( 3223, 0, 0, 1 )
         go to 1000

      end if  ! ( nextfreekp<=nummesh )
      iseqkp = kproblc

      kp(iseqkp)%icounter = 0
      kp(iseqkp)%iseqkm = iseqkm
      probarr(iseqkp) = 1
      lastprob = max(lastprob, iseqkp)

      if ( debug ) write(irefwr,1) 'iseqkp', iseqkp

      allocate ( kp(iseqkp)%kprobsub(numprob+1), stat = error )
      if ( error/=0 ) call eralloc ( error, numprob+1, 'kp(iseqkp)%kprobloc' )

      do i = 1, numprob
         allocate ( kp(iseqkp)%kprobsub(i)%kprobloc(lenkprobfp) )
         kp(iseqkp)%kprobsub(i)%kprobloc = 0
         kp(iseqkp)%kprobsub(i)%kprobloc(1) = lenkprobfp
         kp(iseqkp)%kprobsub(i)%kprobloc(2) = 101
         nullify ( kp(iseqkp)%kprobsub(i)%iinputloc, &
                   kp(iseqkp)%kprobsub(i)%rinputloc, &
                   kp(iseqkp)%kprobsub(i)%intarprb, &
                   kp(iseqkp)%kprobsub(i)%kproba, &
                   kp(iseqkp)%kprobsub(i)%kprobb, &
                   kp(iseqkp)%kprobsub(i)%kprobc, &
                   kp(iseqkp)%kprobsub(i)%kprobd, &
                   kp(iseqkp)%kprobsub(i)%kprobe, &
                   kp(iseqkp)%kprobsub(i)%kprobe2, &
                   kp(iseqkp)%kprobsub(i)%kprobe3, &
                   kp(iseqkp)%kprobsub(i)%kprobf, &
                   kp(iseqkp)%kprobsub(i)%kprobg, &
                   kp(iseqkp)%kprobsub(i)%kprobh, &
                   kp(iseqkp)%kprobsub(i)%kprobhinv, &
                   kp(iseqkp)%kprobsub(i)%kprobi, &
                   kp(iseqkp)%kprobsub(i)%kprobj, &
                   kp(iseqkp)%kprobsub(i)%kprobn, &
                   kp(iseqkp)%kprobsub(i)%kprobo, &
                   kp(iseqkp)%kprobsub(i)%kprobp, &
                   kp(iseqkp)%kprobsub(i)%kprobr, &
                   kp(iseqkp)%kprobsub(i)%kprobs, &
                   kp(iseqkp)%kprobsub(i)%kprobvp1, &
                   kp(iseqkp)%kprobsub(i)%kprobvp2, &
                   kp(iseqkp)%kprobsub(i)%kprobwp1, &
                   kp(iseqkp)%kprobsub(i)%kprobwp2, &
                   kp(iseqkp)%kprobsub(i)%kprobx, &
                   kp(iseqkp)%kprobsub(i)%kproby, &
                   kp(iseqkp)%kprobsub(i)%kprobfict, &
                   kp(iseqkp)%kprobsub(i)%kprobz, &
                   kp(iseqkp)%kprobsub(i)%probzarr, &
                   kp(iseqkp)%kprobsub(i)%kprobaa, &
                   kp(iseqkp)%kprobsub(i)%kprobab, &
                   kp(iseqkp)%kprobsub(i)%kprobac, &
                   kp(iseqkp)%kprobsub(i)%kprobad, &
                   kp(iseqkp)%kprobsub(i)%kprobpar, &
                   kp(iseqkp)%kprobsub(i)%onnz, &
                   kp(iseqkp)%kprobsub(i)%ninterfdeg, &
                   kp(iseqkp)%kprobsub(i)%blockinfor, &
                   kp(iseqkp)%kprobsub(i)%nnz, &
                   kp(iseqkp)%kprobsub(i)%kprobhglob, &
                   kp(iseqkp)%kprobsub(i)%nrusollocs, &
                   kp(iseqkp)%kprobsub(i)%kprobhdd, &
                   kp(iseqkp)%kprobsub(i)%kprobhddinv, &
                   kp(iseqkp)%kprobsub(i)%kprobfglobal, &
                   kp(iseqkp)%kprobsub(i)%kprobper, &
                   kp(iseqkp)%kprobsub(i)%rprobs, &
                   kp(iseqkp)%kprobsub(i)%coortr, &
                   kp(iseqkp)%kprobsub(i)%blockunkinfo, &
                   kp(iseqkp)%kprobsub(i)%innercontr, &
                   kp(iseqkp)%kprobsub(i)%sendinfo, &
                   kp(iseqkp)%kprobsub(i)%sendinfopar, &
                   kp(iseqkp)%kprobsub(i)%innercontru, &
                   kp(iseqkp)%kprobsub(i)%sendinfou, &
                   kp(iseqkp)%kprobsub(i)%innercontrp, &
                   kp(iseqkp)%kprobsub(i)%sendinfop )

      end do  ! i = 1, numprob
      kp(iseqkp)%nprob = numprob

      nullify ( kp(iseqkp)%iinputkp,  kp(iseqkp)%rinputkp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iseqkp, lastprob', iseqkp, lastprob
         write(irefwr,*) 'End sepcrprobinfo'

      end if  ! ( debug )
      call erclos ( 'sepcrprobinfo' )

      end subroutine sepcrprobinfo
