      subroutine seperr1556 ( isubr, iseqnr, namesubr, ipos )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.0  date 26-08-2011
!
!   copyright (c) 2011-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if sequence number for reading is in the right range
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isubr, iseqnr, ipos
      character (len=*) namesubr

!     ipos           o    Position of input arrays in sepmoduleinput
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                         1:  CREATE
!     namesubr       i    Name of subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1556   Sequence number exceeds maximum
!    1872   Sequence number used more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( iseqnr>iinputarrs(isubr)%maxsub ) then

!     --- Sequence number exceeds maximum

         call errint ( iseqnr, 1 )
         call errint ( iinputarrs(isubr)%maxsub, 2 )
         call errint ( iseqnr, 3 )
         call errchr ( namesubr, 1 )
         call errsub ( 1556, 3, 0, 1 )
         go to 1000

      end if

      iinputarrs(isubr)%maxcall = max ( iinputarrs(isubr)%maxcall, iseqnr )
      ipos = iinputarrs(isubr)%istart - 1 + iseqnr
      if ( in(ipos)%used ) then

!     --- Error sequence number already in use

         call errint ( iseqnr, 1 )
         call errchr ( namesubr, 1 )
         call errsub ( 1872, 1, 0, 1 )

      end if

1000  end
