      subroutine sepfillkmeshd
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kmesh part d
!     kmesh part d contains the elements corresponding to each node
!     Old subroutine iniactkmeshd
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lengd2
      integer, allocatable :: markelem(:)

!     lengd2         length of second part of array KMESH part d
!     markelem       Work array to mark elements that must be used
!                    0: element is skipped
!                    1: element is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMESHD1 Compute length of KMESH part D2, fill KMESH part D1
!     SEPFILLKMESHD2 Fill KMESH part D2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create work space for kmesh part d1
!     Activate arrays
!     Fill kmesh part d1 and compute length of d2
!     Create permanent space for kmesh part d and copy kmesh d part 1
!     Create work space
!     Activate arrays
!     Fill kmesh part d2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmeshd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmeshd'
         write(irefwr,1) 'kelmd, iseqkm, npoint', kelmd, iseqkm, npoint
  1      format ( a, 1x, (10i10) )

      end if
      if ( kelmd>0 ) go to 1000

!     --- subroutine topelm compute elements corresponding to nodal points

      if ( associated(kmshd1) ) nullify(kmshd1)
      allocate ( markelem(nelem), kmshd1(npoint+1), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem+npoint, 'markelem' )
      if ( ierror/=0 ) go to 1000

      markelem = 1

!     --- Fill kmesh part d1 and compute length of d2

      if ( debug ) write(irefwr,*) 'before sepfillkmeshd1'
      call sepfillkmeshd1 ( markelem )

      lengd2 = kmshd1(1+npoint)
      if ( debug ) &
         write(irefwr,1) 'npoint, lengd2, iseqkm', npoint, lengd2, iseqkm
      kelmd = lengd2+npoint+1
      allocate ( km(iseqkm)%kmeshd(kelmd) )
      km(iseqkm)%kmshd1 => km(iseqkm)%kmeshd(1:npoint+1)
      km(iseqkm)%kmshd2 => km(iseqkm)%kmeshd(npoint+2:kelmd)
      km(iseqkm)%kmshd1 = kmshd1
      deallocate ( kmshd1 )
      kmeshd => km(iseqkm)%kmeshd

      kmshd1 => km(iseqkm)%kmshd1
      kmshd2 => km(iseqkm)%kmshd2

!     --- Fill kmesh part d2

      call sepfillkmeshd2 ( markelem )

      deallocate ( markelem, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'markelem' )

1000  if ( debug ) write(irefwr,*) 'End sepfillkmeshd'
      call erclos ( 'sepfillkmeshd' )

      end subroutine sepfillkmeshd
