      subroutine sepfillkmesho2 ( lengo, lardim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kmesh part o2 with neighboring elements of elements
!     only two- and three dimensional elements are permitted
!     Old subroutine toplm4
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     neighbor
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lengo, lardim

!     lardim  i    largest dimension of elements used
!     lengo   i    length of kmesh part o
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mshbn1(6), mshbn2(6), mshbnd(90), mshbn3(8), inodp, &
              kbound, i1, i2, icoun1, icoun2, nodp, k, jnpelm, jkelmc, &
              inodp1, inodp2, ibound, istar1, istar2, n1, n2, k2, kelem, &
              k1, jelem, iielem, ielem, ikelmc, ikelmo, &
              inboun, jstep, npboun, nprnst, &
              nbound, kstep, i, istart, ielgrp, mbound, inpelm

!     i       Counting variable
!     i1
!     i2
!     ibound
!     icoun1
!     icoun2
!     ielem   Element number
!     ielgrp  element group number
!     iielem  Relative element number in element group
!     ikelmc  position in kmesh part c
!     ikelmo  position in kmesh part o2 minus 1
!     inboun
!     inodp   Nodal point number
!     inodp1
!     inodp2
!     inpelm  number of points per element
!     istar1
!     istar2
!     istart
!     jelem   do loop variable with respect to the elements
!     jkelmc  position in kmesh part c
!     jnpelm  Number of nodes in an element
!     jstep   Step in loop
!     k       Counting variable
!     k1
!     k2
!     kbound
!     kelem   New KMESH part c with respect to the surface elements
!             All elements have the same number of nodes
!     kstep
!     mbound
!     mshbn1  mshbn1(i) contains the number of vertices of a face of elements
!             with shape number i+10
!     mshbn2  mshbn2(i) contains the starting address of elements
!             with shape number i+10 in array mshbnd
!     mshbn3  contains the relative nodal point numbers of the vertices of
!             the elements in the following way:
!             lardim = 2:
!                mshbn3(i) = vertex i+1 (i=1,2,3)
!                mshbn3(4) = vertex 1
!             lardim = 3:
!                mshbn3(i) = vertex i (i=1,..,8)
!     mshbnd  contains the relative nodal point numbers of the faces of the
!             elements with shape number i+10 consecutively
!     n1
!     n2
!     nbound
!     nodp
!     npboun
!     nprnst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear KMESH_part_o2 and IWORK2
!   if ( lardim=2 and kelma>0 and nelgrp>1 ) then
!      fill iwork1 with number of boundaries of the elements
!   if ( lardim=2 and nelgrp>1 ) then
!      Fill array kmeshcpoint with starting addresses in kmeshc
!   ikelmc:=1;  ielem:=0
!   For ielgrp := 1 (1) nelgrp do
!      nbound := kmesh_o1(ielgrp)
!      inpelm := number of nodes in element
!      inelem := number of elements in group
!      ishape := shape number of element
!      if ( elements can not be treated ) then
!         ielem := ielem + inelem
!         ikelmc := ikelmc + inelem * inpelm
!      else ( elements can be treated )
!         For iielem := 1 (1) inelem do
!            ielem := ielem + 1
!            if ( lardim<=2 ) then
!               inodp1 = first vertex
!               for ibound := 1 (1) nbound do
!                  inodp2 := next vertex
!                  Fill iwork2 with all elements connected to inodp2
!                  For all elements connected to inodp1 do
!                     if ( element#ielem and iwork2(element)#0 ) then
!                        Find common boundary of elements (if existing)
!                        and store in KMESH_O2
!                  Clear iwork2
!                  inodp1 := inodp2
!            else ( lardim=3 )
!               for ibound := 1 (1) nbound do
!                  inodp1 := first vertex in boundary
!                  Fill iwork2 for all elements connected to inodp1 with 1
!                  for all other vertices do
!                     inodp2 := next vertex
!                     For all elements connected to inodp2 do
!                        iwork2(element) := iwork2(element)+1
!                  iwork2(ielem) := 0
!                  for all elements connected to inodp1 do
!                     if ( iwork2(element) = number of vertices of a face )
!                        put element number into kmesh_o2
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data mshbn1 / 3, 3, 4, 4, 3, 3 /, &
           mshbn2 / 0, 12, 24, 48, 12, 12 /
      data mshbnd / 2,3,4, 1,3,4, 1,2,4, 1,2,3, &
                    3,5,10, 1,5,10, 1,3,10, 1,3,5, &
                    1,2,3,4, 1,2,6,5, 2,3,7,6, 3,4,8,7, 4,8,5,1, &
                    5,6,7,8, &
                    1,3,9,7, 1,3,21,19, 3,9,27,21, 9,7,25,27, &
                    7,25,19,1, 19,21,27,25, &
                    1,2,3,  4,5,6,  1,2,5,4, 2,3,6,5, 3,1,4,6 /
! ======================================================================
!
      call eropen ( 'sepfillkmesho2' )

      mbound = kmsho1(1)
      if ( nelgrp==1 .or. kelma==0 ) kstep = npelm/mbound
      allocate ( iwork1(nelem), iwork2(nelem),stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'iwork1' )
      if ( ierror/=0 ) go to 1000

!     --- clear kmsho2, iwork2

      kmsho2(1:lengo-nelgrp) = 0
      iwork2 = 0

!     --- fill array iwork1 with number of boundaries of the elements

      if ( lardim==2 .and. kelma>0 .and. nelgrp>1 ) then

!     --- kelma>0, nelgrp>1 and lardim = 2

         istart = 0
         do ielgrp = 1, nelgrp
            nbound = kmsho1(ielgrp)
            inelem = kmeshg(ielgrp)
            do iielem = 1, inelem
               iwork1(istart+iielem) = nbound
            end do
            istart = istart+inelem
         end do

      end if

!     --- loop over element groups

      ikelmc = 1
      ielem = 0
      ikelmo = 0
      do ielgrp = 1, nelgrp
         nbound = kmsho1(ielgrp)
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         if ( ishape>18 .and. ishape/=21  .and. ishape/=50 ) then

!        --- Shape number not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if

         if ( nbound==0 .or. lardim==3 .and. ishape<11 .or. &
              ishape>=50 ) then

!        --- these type of elements can not be treated

            ielem = ielem+inelem
            ikelmc = ikelmc+inelem*inpelm

         else

!        --- elements that can be treated

            if ( lardim<=2 ) then

!           --- lardim<=2

               if ( nspec==0 ) then

!              --- nspec = 0, standard case

                  inboun = inpelm
                  if ( kelmb>0) inboun = kmeshb(ielgrp)
                  jstep = inboun/nbound

               else if ( ishape==3 ) then

!              --- ishape=3 and spectral elements

                  inboun = inpelm
                  if ( kelmb>0) inboun = kmeshb(ielgrp)
                  jstep = inboun/nbound
                  if ( jstep==5 ) jstep = 4

               else if ( lardim==1 ) then

!              --- nspec>0, spectral elements, ndim = 1

                  mshbn3(1) = 1
                  mshbn3(2) = nspec+2
                  npboun = 2
                  nprnst = 0

               else

!              --- nspec>0, spectral elements, ndim = 2

                  mshbn3(1) = nspec+2
                  mshbn3(2) = (nspec+2)**2
                  mshbn3(3) = (nspec+2)**2 - nspec-1
                  mshbn3(4) = 1
                  kstep = nspec

               end if

            else

!           --- lardim = 3

               if ( nspec>0 ) then

!              --- Spectral elements

                  mshbn3(1) = 1
                  mshbn3(2) = nspec+2
                  mshbn3(3) = (nspec+2)**2
                  mshbn3(4) = (nspec+2)**2 - nspec-1
                  do i = 1, 4
                     mshbn3(i+4) = mshbn3(i) + (nspec+1) * (nspec+2)**2
                  end do

               end if
               if ( ishape<20 ) then

!              --- Standard tetrahedron or hexahedron

                  npboun = mshbn1(ishape-10)
                  nprnst = mshbn2(ishape-10)

               else

!              --- ishape = 21, prism

                  npboun = 5
                  nprnst = 72

               end if

            end if

!           --- loop over elements

            do iielem = 1, inelem
               ielem = ielem+1

!              --- loop over boundaries

               if ( lardim==1 ) then

!              --- lardim = 1

                  inodp1 = kmeshc(ikelmc)
                  istar1 = kmshd1(inodp1)
                  n1 = kmshd1(inodp1+1)-istar1
                  do k = 1, n1
                     kelem = kmshd2(istar1+k)
                     if ( kelem/=ielem ) kmsho2(ikelmo+1) = kelem
                  end do
                  inodp1 = kmeshc(ikelmc+inpelm-1)
                  istar1 = kmshd1(inodp1)
                  n1 = kmshd1(inodp1+1)-istar1
                  do k = 1, n1
                     kelem = kmshd2(istar1+k)
                     if ( kelem/=ielem ) kmsho2(ikelmo+2) = kelem
                  end do

               else if ( lardim==2 ) then

!              --- lardim = 2

                  inodp1 = kmeshc(ikelmc)

!                 --- loop over sides

                  do ibound = 1, nbound
                     if ( ishape/=3 .and. nspec>0 ) then
                        inodp2 = kmeshc(ikelmc-1+mshbn3(ibound))
                     else if ( ibound/=nbound ) then
                        inodp2 = kmeshc(ikelmc+ibound*jstep)
                     else
                        inodp2 = kmeshc(ikelmc)
                     end if
                     istar1 = kmshd1(inodp1)
                     istar2 = kmshd1(inodp2)
                     n1 = kmshd1(inodp1+1)-istar1
                     n2 = kmshd1(inodp2+1)-istar2

!                    --- compare common elements not equal to ielem

                     if ( nelgrp>1 ) then
                        do k2 = 1, n2
                           kelem = kmshd2(istar2+k2)
                           iwork2(kelem) = iwork1(kelem)
                        end do
                     else
                        do k2 = 1, n2
                           iwork2( kmshd2(istar2+k2) ) = nbound
                        end do
                     end if
                     do k1 = 1, n1
                        jelem = kmshd2(istar1+k1)
                        if ( jelem/=ielem .and. iwork2(jelem)/=0 ) then

!                       --- common element found

                           if ( kelma==0 .or. nelgrp==1 ) then
                              jkelmc = (jelem-1)*inpelm
                           else
                              jkelmc = kmeshcpoint(jelem)
                              jnpelm = kmeshcpoint(jelem+1)-jkelmc
                              mbound = iwork1(jelem)
                              kstep = jnpelm/mbound
                           end if
                           do k = 1, mbound
                              nodp = kmeshc(jkelmc+(k-1)*kstep+1)
                              if ( nodp==inodp1 ) then
                                 icoun1 = k
                              else if ( nodp==inodp2 ) then
                                 icoun2 = k
                              end if
                           end do
                           i1 = min(icoun1, icoun2)
                           i2 = max(icoun1, icoun2)
                           if ( i2-i1>1 ) then
                              kbound = mbound
                           else
                              kbound = i1
                           end if
                           kmsho2(ikelmo+ibound) = jelem+ &
                                                   kbound*(nelem+1)
                           goto 240
                        end if
                     end do

!                    --- clear iwork2

240                  do k2 = 1, n2
                        iwork2(kmshd2(istar2+k2)) = 0
                     end do
                     inodp1 = inodp2
                  end do

               else

!              --- lardim = 3

                  istart = nprnst
                  do ibound = 1, nbound
                     nodp = mshbnd(istart+1)
                     if ( nspec>0 ) nodp = mshbn3(nodp)
                     inodp1 = kmeshc(ikelmc-1+nodp)

!                    --- inodp1 is first nodal point at ibound-th boundary
!                        of element

                     istar1 = kmshd1(inodp1)
                     n1 = kmshd1(inodp1+1)-istar1
                     do k1 = 1, n1
                        iwork2(kmshd2(istar1+k1)) = 1
                     end do
                     if ( ishape==21 ) then

!                    --- ishape = 21, boundaries 1 and 2 contain 3 points
!                        boundaries 3-5: 4 points

                        if ( ibound<=2 ) then
                           npboun = 3
                        else
                           npboun = 4
                        end if
                     end if

!                    --- set positions in iwork2 corresponding to all
!                        elements that have point inodp1 in common
!                        equal to one
!                        this value is raised for all other points of
!                        the boundary
!                        when the value of iwork2(jelem) is equal to npboun
!                        then jelem has all points of the boundary in common
!                        and hence jelem is the neighbor of element ielem
!                        boundary ibound

                     do inodp = 2, npboun
                        nodp = mshbnd(istart+inodp)
                        if ( nspec>0 ) nodp = mshbn3(nodp)
                        inodp2 = kmeshc(ikelmc-1+nodp)
                        istar2 = kmshd1(inodp2)
                        n2 = kmshd1(inodp2+1)-istar2
                        do k1 = 1, n2
                           kelem = kmshd2(istar2+k1)
                           if ( iwork2(kelem)/=0 ) &
                                 iwork2(kelem) = iwork2(kelem)+1
                        end do
                     end do
                     iwork2(ielem) = 0
                     jelem = 0
                     do k1 = 1, n1
                        kelem = kmshd2(istar1+k1)
                        if ( iwork2(kelem)==npboun ) jelem = kelem
                        iwork2(kelem) = 0
                     end do
                     kmsho2(ikelmo+ibound) = jelem
                     istart = istart+npboun
                  end do
               end if
               ikelmo = ikelmo+nbound
               ikelmc = ikelmc+inpelm
            end do
         end if
      end do

      deallocate ( iwork1, iwork2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork1' )

1000  call erclos ( 'sepfillkmesho2' )

      end subroutine sepfillkmesho2
