      subroutine sepfillnmesha
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create nmesh parts a and b
!     Old subroutine iniactnmesha
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     face
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lpnmpb, medpfa, mfapel, lpnmpa

!     lpnmpa         length of array nmesha
!     lpnmpb         length of array nmeshb
!     medpfa         maximum number of edges per face
!     mfapel         maximum number of faces in one element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFA01        creates the faces part 1 pointers and initialization
!     SEPFA02        creates the faces part 2 actual face labels and lmeshb
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmesh parts c and d
!     SEPFILLNMESHH  Fill array nmesh part h
!     SEPPRINNMESHA  print nmesh part a
!     SEPPRINNMESHB  print nmesh part b
!     SEPPRINNMESHC  print nmesh part c
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( maximum number of faces per element>0 ) then
!        activate old arrays nmeshd, nmeshh
!        Create temporary space and initialize first part (nmesha)
!        Initialize array nmesha
!        Make the array permanent
!        Fetch nmesh part a, d and h again
!        Create temorary space for nmeshb with medpfa extra space
!        Fetch pointers to nmesh part a, d and h
!        Fill nmesha with actual data and create face information
!        array nmeshb
!        Make nmeshb permanent
!     else
!        Fill information with zero faces
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnmesha' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnmesha'
         write(irefwr,1) 'kelmaa, nelem, npoint', kelmaa, nelem, npoint
  !       call sepprinkmeshc
  !       call prinkmeshh
  1      format ( a, 1x, (10i6) )

      end if

      if ( kelmaa==0 ) then

!     --- Kmesh part aa has not been filled

         if ( debug ) write(irefwr,*) 'fill kmeshaa'
         call sepfillkmeshaa

      end if
      if ( ierror/=0 ) go to 1000

      nullify ( nmesha, nmeshb )
      if ( associated(km(iseqkm)%nmesha) ) &
         deallocate(km(iseqkm)%nmesha, km(iseqkm)%nmeshb)

!     --- Arrays nmesha, b do not exist, create

      mfapel = nmesh(19)  ! maximum number of faces per element
      if ( debug ) write(irefwr,*) 'mfapel, nelem ', mfapel, nelem
      if ( mfapel>0 ) then

!     --- activate old arrays nmeshd, nmeshh

         if ( nmesh(34)==0 ) call sepfillnmeshc
         if ( ierror/=0 ) go to 1000
         if ( nmesh(38)==0 ) call sepfillnmeshh
         if ( ierror/=0 ) go to 1000

!        --- Create temporary space and initialize first part (nmesha)

         lpnmpa = 2*nelem+2*nelem*mfapel  ! overestimated length of nmesha
         if ( debug ) &
            write(irefwr,1) 'nelem, mfapel, lpnmpa', nelem, mfapel, lpnmpa

         allocate ( nmesha(lpnmpa), stat = error )
         if ( error/=0 ) call eralloc ( error, lpnmpa, 'nmesha' )
         if ( ierror/=0 ) go to 1000

!        --- Initialize array nmesha
!            For each element faces are created and face labels and
!            rotations are set to zero

         call sepfa01
         lpnmpa = length
         if ( ierror/=0 ) go to 1000
         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'lpnmpa ',lpnmpa
            call prinin ( nmesha, nelem, 'nmesha part 1' )

         end if

!        --- Create temorary space for nmeshb with medpfa extra space
!            for help array

         medpfa = nmesh(21)  ! maximum number of edges per face
         lpnmpb = nelem*mfapel*(medpfa+2)  ! overestimated length of nmeshb

         allocate ( nmeshb(lpnmpb), stat = error )
         if ( error/=0 ) call eralloc ( error, lpnmpb, 'nmeshb' )
         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,*) 'medpfa, lpnmpb ',medpfa, lpnmpb

!        --- Fill nmesha with actual data and create face information
!            array nmeshb

         call sepfa02 ( medpfa )
         lpnmpb = length
         if ( ierror/=0 ) go to 1000

!        --- Make  nmesha and nmeshb permanent

         allocate ( km(iseqkm)%nmesha(lpnmpa), km(iseqkm)%nmeshb(lpnmpb) )
         km(iseqkm)%nmesha = nmesha(1:lpnmpa)
         km(iseqkm)%nmeshb = nmeshb(1:lpnmpb)
         nmesh(5) = nfaces
         nmesh(31) = lpnmpa
         nmesh(32) = lpnmpb

         if ( debug ) then

!        --- Debug information

            call sepprinnmeshc
            write(irefwr,*) 'nfaces ', nfaces
            call sepprinnmesha
            call sepprinnmeshb
            write(irefwr,*) 'End sepfillnmesha'

         end if

         deallocate ( nmesha, nmeshb, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmesha' )

         nmesha => km(iseqkm)%nmesha
         nmeshb => km(iseqkm)%nmeshb

      else

!     --- no faces exist

         nmesh(5) = 0
         nmesh(31) = 0
         nmesh(32) = 0

      end if

1000  call erclos ( 'sepfillnmesha' )

      end subroutine sepfillnmesha
