      subroutine sepfillnodcont ( inpsep, nodes, nnodes, kfill )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for contact problems depending on kfill
!     Old subroutine prfillnodcont
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nodes(*), nnodes, kfill

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prfillnodes
!     kfill          i    Indication how the nodal points must be filled into
!                         array NODES
!                         See subroutine prfillnodes
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iconct

!                    they are not
!     iconct         Sequence number of contact
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLNODCON1 Fill array nodes for contact problems depending on kfill
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnodcont' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodcont'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nnodes = 0
      if ( kp(iseqkp)%kprobsub(1)%kprobloc(46)==0 ) go to 1000  ! no contact

!     --- Get parameters from inpsep

      iconct = inpsep(1)

!     --- actual call

      call sepfillnodcon1 ( iconct, kfill, nodes, nnodes )

1000  call erclos ( 'sepfillnodcont' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnodcont'

      end if  ! ( debug )

      end subroutine sepfillnodcont
