      subroutine sepfillnodsrf2 ( isrf1, isrf2, nodes, nnodes, numnod, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for set of surfaces
!     Only a restricted number of nodes per element is used
!     Old subroutine prfillnodsurf2
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isrf1, isrf2, nodes(*), nnodes, numnod, ihelp(*)

!     ihelp          i    contains the realtive node numbers per element
!                         that should be used in case numnod>0
!     isrf1          i    First surface number
!     isrf2          i    Last surface number
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
!     numnod         i    If>0, only numnod nodes per element must be used
!                         In that case ihelp is used
!                         If 0 all nodes are used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsurfaces, i
      integer, allocatable :: surfaces(:)

!                    they are not
!     i              Counting variable
!     nsurfaces      Number of surfaces in array surfaces
!     surfaces       Contains the surface numbers to be used for filling nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPSURFSNOD3   Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!                    It is assumed that kmesh part n has been filled
!                    Only a restricted number of nodes per element is used
!     SEPSURFSNOD4   Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!                    It is assumed that kmesh part n has not been filled
!                    Only a restricted number of nodes per element is used
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     155   first surface number has incorrect value
!     156   last surface number has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnodsrf2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodsrf2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( isrf1<=0 .or. isrf1>nsurfs ) then
         call errint ( isrf1, 1 )
         call errint ( nsurfs, 2 )
         call errsub ( 155, 2, 0, 0 )
         isrf1 = 1
      end if

      if ( isrf2<=isrf1 .or. isrf2>nsurfs ) then
         call errint ( isrf2, 1 )
         call errint ( isrf1, 2 )
         call errint ( nsurfs, 3 )
         call errsub ( 156, 3, 0, 0 )
         isrf2 = isrf1
      end if

      nsurfaces = isrf2-isrf1+1

      allocate ( surfaces(nsurfaces), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nsurfaces, 'surfaces' )
      if ( ierror/=0 ) go to 1000

      do i = 1, nsurfaces
         surfaces(i) = isrf1+i-1
      end do  ! i = 1, nsurfaces

      nnodes = 0

      if ( kelmn>0 ) then

!     --- kmeshn has been filled

         call sepsurfsnod3 ( nsurfaces, surfaces, nodes, nnodes, &
                             numnod, ihelp )

      else

!     --- kmeshn has not been filled

         call sepsurfsnod4 ( nsurfaces, surfaces, nodes, nnodes, &
                             numnod, ihelp )

      end if  ! ( kelmn>0 )

      deallocate ( surfaces, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'surfaces' )

1000  call erclos ( 'sepfillnodsrf2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnodsrf2'

      end if  ! ( debug )

      end subroutine sepfillnodsrf2
