      integer function sepkeywordscal ( )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if one of the keywords is S or Scalar
!     In that case it might be a scalar name with number
!     Old subroutine intxscalar
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical ::  scalarfound

!     i              Counting variable
!     scalarfound    If true vector or scalar is a keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READSTRSCALS   Check if name read corresponds to one of the scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2951   Sequence number of vectors out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepkeywordscal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepkeywordscal'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icode = 0
      do i = 1, nncode

         scalarfound = ncode(i)==1 .and. code(i)=='S' .or. &
                       ncodmx(i)==6 .and. code(i)=='SCALAR'
         jcode = i
         if ( scalarfound ) exit

      end do  ! i = 1, nncode

      if ( scalarfound ) then

!     --- The keyword scalar is present

         call readstrscals
         if ( ityprd==11 ) then

!        --- scalar name found

            icode = jcode
            ivalrd = nint(realrd-100000000d0)
            ivalrd = mod(ivalrd,10000)
            ivalrd = mod(ivalrd,5000)

            if ( ivalrd<=maxcls ) then

!           --- Real scalar found

               valmax = numscals
               if ( ivalrd<0 .or. ivalrd>valmax ) then

!              --- Scalar number out of range

                  call errint ( ivalrd, 1 )
                  call errint ( valmax, 2 )
                  call errsub ( 2951, 2, 0, 0 )
                  call errcrd

               end if  ! ( ivalrd<=0 .or. ivalrd>numvec )

            else

!           --- Integer scalar found

               valmax = numintscals
               if ( ivalrd-maxcls>valmax ) then

!              --- Scalar number out of range

                  call errint ( ivalrd, 1 )
                  call errint ( valmax, 2 )
                  call errsub ( 2951, 2, 0, 0 )
                  call errcrd

               end if  ! ( ivalrd<=0 .or. ivalrd>numvec )

            end if  ! ( ivalrd<maxcls )

         end if  ! ( ityprd==2 )

      end if  ! ( scalarfound )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepkeywordscal'

      end if  ! ( debug )
      call erclos ( 'sepkeywordscal' )

      sepkeywordscal = icode

      end function sepkeywordscal
