      integer function sepmainkeywnn ( )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if string read is (main) keyword
!     If so go to 1000 with icode >0 or finished true
!     Old subroutine intx00keyw
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: check

!     check          Help parameter for if statement
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepkeywordscal, sepkeywordstr, sepkeywordvect

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READSTRCONST   Check if name read corresponds to one of the integer or
!                    real constants
!     SEPKEYWORDSCAL Check if one of the keywords is S or Scalar
!     SEPKEYWORDSTR  Check if one of the keywords is String
!     SEPKEYWORDVECT Check if one of the keywords is V or Vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmainkeywnn' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmainkeywnn'
         write(irefwr,1) 'lastitem, lastitembef', lastitem, lastitembef
         write(irefwr,1) 'numchar, ityprd', numchar, ityprd
         write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d13.7) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if string is a standard keyword

      icode = sepkeyword ()
      if ( debug ) write(irefwr,1) 'after sepkeyword', icode
      if ( icode>0 ) go to 1000  ! keyword found

!     --- String is not a standard keyword
!         Check if one of the keywords is V or Vector
!         In that case it might be a vector name with number

      icode = sepkeywordvect()
      if ( debug ) write(irefwr,1) 'after sepkeywordvect', icode
      if ( icode>0 ) go to 1000  ! keyword found

!     --- String is not a standard keyword nor a vector name
!         Check if one of the keywords is S or Scalar
!         In that case it might be a scalar name with number

      icode = sepkeywordscal()
      if ( debug ) write(irefwr,1) 'after sepkeywordscal: icode', icode
      if ( icode>0 ) go to 1000  ! keyword found

!     --- String is not recognized yet
!         Might be a string

      icode = sepkeywordstr()
      if ( debug ) write(irefwr,1) 'after sepkeywordstr: icode', icode
      if ( icode>0 ) go to 1000  ! keyword found

!     --- String is not recognized yet
!         Might be a constant
!         If it is a constant and it is preceded by a minus sign,
!         then multiply by -1

      call readstrconst
      check = lastitem>1 .and. (ityprd==2 .or. ityprd==3)
      if ( check ) check = typeitems(lastitem-1)==11
      if ( check ) then

         if ( index_rdarr(3)>0 ) then
            index_rdarr(3)=-index_rdarr(3)
         else
            realrd = -realrd
         end if  ! ( index_rdarr(3)>0 )

      end if  ! ( check )

!     --- In case string is not one of the previous options, check if it
!         is a keyword that is not spelled completely by comparing the
!         first characters only

      if ( debug ) write(irefwr,1) 'ityprd', ityprd
      if ( ityprd==4 ) icode = sepkeyword2()  ! still a string

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icode, lastitem, ityprd, index_rdarr(3)', &
                          icode, lastitem, ityprd, index_rdarr(3)
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End sepmainkeywnn'

      end if  ! ( debug )
      call erclos ( 'sepmainkeywnn' )

      sepmainkeywnn = icode

      end function sepmainkeywnn
