      subroutine seppointkmeshc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array pointkmsc with starting addresses of element groups
!     in kmeshc (minus 1)
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     mesh
!     pointer
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ielgrp, inpelm

!     ielgrp         element group number
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppointkmeshc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppointkmeshc'
         call ertrace
         write(irefwr,1) 'size(pointkmsc) ', size(pointkmsc)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      pointkmsc(1) = 0
      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         pointkmsc(ielgrp+1) = pointkmsc(ielgrp)+inelem*inpelm

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppointkmeshc'

      end if  ! ( debug )
      call erclos ( 'seppointkmeshc' )

      end subroutine seppointkmeshc
