      subroutine sepprocopy2 ( kprob2, nprob1, iseqkp1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create kprob2 and copy parts of kprob
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: kprob2, nprob1, iseqkp1

!     iseqkp1        i    Sequence number of array to be copied in kp
!     kprob2        i/o   Contains copy of old problem
!     nprob1         i    Number of problems in array to be copied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, leng, npointlc, ih, ivec

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ivec           Sequence number of vector of special structure
!     leng           Help variable to compute the length of some arrays
!     npointlc       Local number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCRPROBINFO  create a new array in kp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprocopy2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprocopy2'
         write(irefwr,1) 'nprob1', nprob1
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcrprobinfo ( kprob2, nprob1 )

      length = size(kp(iseqkp1)%iinputkp)

      if ( debug ) write(irefwr,1) 'length', length

      allocate ( kp(iseqkp)%iinputkp(length) )
      kp(iseqkp)%iinputkp = kp(iseqkp1)%iinputkp

      if ( associated(kp(iseqkp1)%rinputkp) ) then
         length = size(kp(iseqkp1)%rinputkp)
         allocate ( kp(iseqkp)%rinputkp(length) )
         kp(iseqkp)%rinputkp = kp(iseqkp1)%rinputkp
      else
         nullify(kp(iseqkp)%rinputkp)
      end if  ! ( associated(kp(iseqkp1)%rinputkp) )

!     --- Copy parts of array kprob

      do iprob = 1, nprob1
         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         kprobloc = kp(iseqkp1)%kprobsub(iprob)%kprobloc

         if ( kprobloc(11)>0 ) then

!        --- Copy kprob x

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobx)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobx(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobx = &
               kp(iseqkp1)%kprobsub(iprob)%kprobx

         end if  ! ( kprobloc(11)>0 )

         if ( kprobloc(14)>0 ) then

!        --- Copy kprob a

            length = size(kp(iseqkp1)%kprobsub(iprob)%kproba)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kproba(length) )
            do i = 1, length
               if ( associated(kp(iseqkp1)%kprobsub(iprob)%kproba(i)%arr) ) then
                  leng = size(kp(iseqkp1)%kprobsub(iprob)%kproba(i)%arr)
                  allocate ( kp(iseqkp)%kprobsub(iprob)%kproba(i)%arr(leng) )
                  kp(iseqkp)%kprobsub(iprob)%kproba(i)%arr = &
                     kp(iseqkp1)%kprobsub(iprob)%kproba(i)%arr
               else
                  nullify(kp(iseqkp)%kprobsub(iprob)%kproba(i)%arr)
               end if  ! ( associated(kp(iseqkp1)%kprobsub(iprob)%kproba(i)...
            end do  ! i = 1, length

         end if  ! ( kprobloc(14)>0 )

         if ( kprobloc(15)>0 ) then

!        --- Copy kprob b

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobb)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobb(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobb = &
               kp(iseqkp1)%kprobsub(iprob)%kprobb

         end if  ! ( kprobloc(15)>0 )

!        --- Copy kprob c

         length = size(kp(iseqkp1)%kprobsub(iprob)%kprobc)
         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobc(length) )
         kp(iseqkp)%kprobsub(iprob)%kprobc = &
            kp(iseqkp1)%kprobsub(iprob)%kprobc

         if ( kprobloc(17)>0 ) then

!        --- Copy kprob d

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobd)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobd(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobd = &
               kp(iseqkp1)%kprobsub(iprob)%kprobd

         end if  ! ( kprobloc(17)>0 )

         if ( kprobloc(18)>0 ) then

!        --- Copy kprob e

            leng = size(kp(iseqkp1)%kprobsub(iprob)%kprobe,1)
            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobe,2)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobe(leng,length) )
            kp(iseqkp)%kprobsub(iprob)%kprobe = &
               kp(iseqkp1)%kprobsub(iprob)%kprobe
            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobe2)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobe2(length), &
                       kp(iseqkp)%kprobsub(iprob)%kprobe3(length))
            kp(iseqkp)%kprobsub(iprob)%kprobe2 = &
               kp(iseqkp1)%kprobsub(iprob)%kprobe2
            ndtvec = kprobloc(6)
            do ivec = 1, ndtvec
               nullify(kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr)
               if ( kp(iseqkp)%kprobsub(iprob)%kprobe2(ivec)>0 ) then

!              --- part is stored in ibuffr, copy

                  if ( associated(kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr) ) &
                     deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr )
                  length = size(kp(iseqkp1)%kprobsub(iprob)%kprobe3(ivec)%arr)
                  allocate ( kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)% &
                             arr(length)  )
                  kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr = &
                     kp(iseqkp1)%kprobsub(iprob)%kprobe3(ivec)%arr

               end if
            end do  ! ivec = 1, ndtvec

         end if  ! ( kprobloc(18)>0 )

         if ( kprobloc(19)>0 ) then

!        --- Copy kprob f

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobf)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobf(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobf = &
               kp(iseqkp1)%kprobsub(iprob)%kprobf

         end if  ! ( kprobloc(19)>0 )

         if ( kprobloc(22)>0 ) then

!        --- Copy kprob i

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobi)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobi(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobi = &
               kp(iseqkp1)%kprobsub(iprob)%kprobi

         end if  ! ( kprobloc(22)>0 )

         if ( kprobloc(23)>0 ) then

!        --- Copy kprob j

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobj,1)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobj(length,2) )
            kp(iseqkp)%kprobsub(iprob)%kprobj = &
               kp(iseqkp1)%kprobsub(iprob)%kprobj

         end if  ! ( kprobloc(25)>0 )

         if ( kprobloc(27)>0 ) then

!        --- Copy kprob n

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobn,2)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobn(ndim,length) )
            kp(iseqkp)%kprobsub(iprob)%kprobn = &
               kp(iseqkp1)%kprobsub(iprob)%kprobn
            allocate ( kp(iseqkp)%kprobsub(iprob)%coortr(maxtrn,length) )
            kp(iseqkp)%kprobsub(iprob)%coortr = &
               kp(iseqkp1)%kprobsub(iprob)%coortr

         end if  ! ( kprobloc(27)>0 )

         if ( kprobloc(28)>0 ) then

!        --- Copy kprob o

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobo)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobo(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobo = &
               kp(iseqkp1)%kprobsub(iprob)%kprobo

         end if  ! ( kprobloc(28)>0 )

         if ( kprobloc(35)>0 ) then

!        --- Copy kprob p

            npointlc = size(kp(iseqkp1)%kprobsub(iprob)%kprobp,1)
            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobp,2)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobp(npointlc,length) )
            kp(iseqkp)%kprobsub(iprob)%kprobp = &
               kp(iseqkp1)%kprobsub(iprob)%kprobp

         end if  ! ( kprobloc(35)>0 )

         if ( kprobloc(37)>0 ) then

!        --- Copy kprob r

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobr)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobr(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobr = &
               kp(iseqkp1)%kprobsub(iprob)%kprobr

         end if  ! ( kprobloc(37)>0 )

         if ( kprobloc(38)>0 ) then

!        --- Copy kprob s

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobs)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobs(length) )
               kp(iseqkp)%kprobsub(iprob)%kprobs = &
                  kp(iseqkp1)%kprobsub(iprob)%kprobs
            length = size(kp(iseqkp1)%kprobsub(iprob)%rprobs,1)
            allocate ( kp(iseqkp)%kprobsub(iprob)%rprobs(length,2) )
               kp(iseqkp)%kprobsub(iprob)%rprobs = &
                  kp(iseqkp1)%kprobsub(iprob)%rprobs

         end if  ! ( kprobloc(38)>0 )

         if ( kprobloc(46)>0 .and. iprob==1 ) then

!        --- Copy kprob v

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobvp1,1)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobvp1(length,3) )
            kp(iseqkp)%kprobsub(iprob)%kprobvp1 = &
               kp(iseqkp1)%kprobsub(iprob)%kprobvp1
            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobvp2)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobvp2(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobvp2 = &
               kp(iseqkp1)%kprobsub(iprob)%kprobvp2

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobwp1,1)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobwp1(length,4) )
            kp(iseqkp)%kprobsub(iprob)%kprobwp1 = &
               kp(iseqkp1)%kprobsub(iprob)%kprobwp1
            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobwp2)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobwp2(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobwp2 = &
               kp(iseqkp1)%kprobsub(iprob)%kprobwp2

         end if  ! ( kprobloc(46)>0 )
         if ( kprobloc(50)>0 ) then

!        --- Copy kprob h

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobh)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobh(length), &
                       kp(iseqkp)%kprobsub(iprob)%kprobhinv(length))
            kp(iseqkp)%kprobsub(iprob)%kprobh = &
               kp(iseqkp1)%kprobsub(iprob)%kprobh
            kp(iseqkp)%kprobsub(iprob)%kprobhinv = &
               kp(iseqkp1)%kprobsub(iprob)%kprobhinv

         end if  ! ( kprobloc(50)>0 )

         if ( kprobloc(51)>0 ) then

!        --- Copy kprob y

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobz)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobz(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobz = &
               kp(iseqkp1)%kprobsub(iprob)%kprobz
            kprobz => kp(iseqkp)%kprobsub(iprob)%kprobz
            if ( kprobz(4)>0 ) then
               length = kprobz(4)
               allocate ( kp(iseqkp)%kprobsub(iprob)%kprobfict(2,length) )
               kp(iseqkp)%kprobsub(iprob)%kprobfict = &
                  kp(iseqkp1)%kprobsub(iprob)%kprobfict
            end if  ! ( kprobz(4)>0 )

         end if  ! ( kprobloc(51)>0 )

         if ( kprobloc(52)>0 ) then

!        --- Copy kprob z

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobz)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobz(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobz = &
               kp(iseqkp1)%kprobsub(iprob)%kprobz
            kprobz => kp(iseqkp)%kprobsub(iprob)%kprobz
            do i = 1, size(kprobz)
               if ( kprobpart%kprobz(i)>0 ) then
                  length = size(kp(iseqkp1)%kprobsub(iprob)%probzarr(i)%arr)
                  allocate (kp(iseqkp)%kprobsub(iprob)%probzarr(i)%arr(length) )
                  kp(iseqkp)%kprobsub(iprob)%probzarr(i)%arr = &
                     kp(iseqkp1)%kprobsub(iprob)%probzarr(i)%arr
               else
                  ih = -kprobpart%kprobz(i)
                  kp(iseqkp)%kprobsub(iprob)%probzarr(i)%arr => &
                     kp(iseqkp)%kprobsub(iprob)%probzarr(ih)%arr
               end if  ! ( kprobpart%kprobz(i)>0 )
            end do  ! i = 1, size(kprobz)

         end if  ! ( kprobloc(52)>0 )

         if ( kprobloc(53)>0 ) then

!        --- Copy kprob aa

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobaa)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobaa(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobaa = &
               kp(iseqkp1)%kprobsub(iprob)%kprobaa

         end if  ! ( kprobloc(53)>0 )

         if ( kprobloc(58)>0 ) then

!        --- Copy kprob ab

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobab)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobab(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobab = &
               kp(iseqkp1)%kprobsub(iprob)%kprobab

         end if  ! ( kprobloc(58)>0 )

         if ( kprobloc(59)>0 ) then

!        --- Copy kprob ac

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobac)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobac(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobac = &
               kp(iseqkp1)%kprobsub(iprob)%kprobac

         end if  ! ( kprobloc(59)>0 )

         if ( kprobloc(60)>0 ) then

!        --- Copy kprob ad

            length = size(kp(iseqkp1)%kprobsub(iprob)%kprobad)
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobad(length) )
            kp(iseqkp)%kprobsub(iprob)%kprobad = &
               kp(iseqkp1)%kprobsub(iprob)%kprobad

         end if  ! ( kprobloc(60)>0 )

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprocopy2'

      end if  ! ( debug )
      call erclos ( 'sepprocopy2' )

      end subroutine sepprocopy2
