      subroutine sepreadnext ( finished, ihelp, lastitemsav )
! ======================================================================
!
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Keyword found, read next item
!     Help subroutine for sepreadsetkeyw
!     Old subroutine intx02next
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: ihelp(*), lastitemsav
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
!     lastitemsav   i/o   Help parameter to save the value of lastitem
!                         This is used in case an expression is found, to
!                         restart with the begin of the expression
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, jump, m, nmax
      logical ::  vectorfound
      character (len=240) subkey

!     iseq           Sequence number
!     jump           if 0 no error has been found, if 1 array tabout
!                    is too small
!     m              Number of integers read
!     nmax           maximum number of numbers allowed
!     subkey         Sub (second) keyword
!     vectorfound    If true the last item read is a vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINRL         Print 1d real vector
!     SEPREADTYPE01  Read keyword or keyword = number
!     SEPREADTYPE05  Read keyword followed by one or two keywords
!     SEPREADTYPE06  Read keyword = text
!     SEPREADTYPE08  Read keyword = number repeatedly
!     SEPREADTYPE09  Read keyword = number repeatedly
!     SEPREADTYPE10  Read keyword followed by sequence number and number
!                    or number
!     SEPREADTYPE11  Read keyword followed by a series of positive integers
!     SEPREADTYPE15  Read keyword followed by an integer, repeatedly
!     SEPREADTYPE16  Read keyword followed by a value or value = v, or func = i
!     SEPREADTYPE17  Read keyword followed by a series of series of numbers
!     SEPREADTYPE18  Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 14, 18 and 31
!     SEPREADTYPE19  Read keyword followed by a series of numbers
!     SEPREADTYPE20  Read single keyword inwhich case the table must be set
!                    to -100000 or keyword followed by agiven other keyword
!                    and a series of integers
!     SEPREADTYPE21  Read keyword followed by (ax+by+cz=d)
!     SEPREADTYPE22  Read keyword followed by possibly one or two special
!                    keywords that are skipped and then by a series of numbers
!     SEPREADTYPE23  Read keyword followed by possibly a special keyword that
!                    is skipped
!                    table(icode) is set equal to 1
!     SEPREADTYPE25  Read keyword followed by a sequence number followed by
!                    a value(1) or value(1) = v, or func = i
!     SEPREADTYPE27  Read keyword followed bya series of values or value = v, &
!                    or func = i
!     SEPREADTYPE29  Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 29, 33 and 37
!     SEPREADTYPE32  Read keyword like:
!                    surfaces (i1, i2, ... in ) = s1 to s2 or
!                    surfaces = s1 to s2, nodes (i1, i2, ... in )
!     SEPREADTYPE35  Read keyword followed by a sequence number followed by
!                              a value or value = v, or func = i
!                              and again the same keyword followed  by
!                              a sequence number followed by
!                              a value or value = v, or func = i
!     SEPREADTYPE38  Read number or vector name (itype=38, 40)
!     SEPREADTYPE39  Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 39
!     SEPREADTYPE43  Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 43
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1469   Array strout too small
!    1628   Keyword found more than once for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadnext' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadnext'
         write(irefwr,1) 'iretur', iretur
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      vectorfound = ityprd==2

      nseqkeyw = nseqkeyw+1
      iseqkeyw(nseqkeyw) = icode
      keywordfound = .true.
      jump = 0

      if ( debug ) then
        write(irefwr,1) 'nseqkeyw, icode, itype(icode), minusg', &
                         nseqkeyw, icode, itype(icode), minusg
        write(irefwr,1) 'istop, lastitem/3', istop, lastitem
      end if  ! ( debug )

         if ( icode>0 ) then
            if ( iseqkeywitem(icode)==0 ) &
               iseqkeywitem(icode) = lastitem  ! store position
         end if  ! ( icode>0 )

      select case ( itype(icode) )

         case(:-1,7)

!        --- itype(icode) = -1,7: skip

         case(0,1,3,4,36)

!        --- itype(icode) = 0, 1, 3, 4, 36: Read keyword or
!            keyword = number

            call sepreadtype01 ( finished )
            if ( ierror/=0 ) go to 1000
            if ( iposut>nout ) jump = 1

         case(2)

!        --- itype(icode) = 2: set table = 1

            if ( debug ) &
               write(irefwr,*) 'itype(icode) = 2: set table = 1'
            table(icode) = 1d0
            if ( icountkey(icode)/=0 ) then
                call errchr ( code(icode), 1 )
                call errsub ( 1628, 0, 0, 1 )
            else
               icountkey(icode) = icountkey(icode)+1
            end if

         case(5,13)

!        --- itype(icode) = 5 or 13, hence the keyword must be followed by
!            one or two keywords

            call sepreadtype05 ( finished )

         case(6)

!        --- itype(icode) = 6, text

            call sepreadtype06 ( finished )
            if ( kposut>nstrng ) then

!           --- Error: kposut too large

               call errint ( nstrng, 1 )
               call errsub ( 1469, 1, 0, 0 )

            end if  ! ( kposut>nstrng )

         case(8)

!        --- itype(icode) = 8, keyword = real, repeatedly

            iseq = pointar(icode)
            call inichkarlen ( numberkeyw(iseq)+1, maxrepeat, 'valuekeyw' )
            if ( ierror/=0 ) go to 1000
            call sepreadtype08 ( finished )

         case(9)

!        --- itype(icode) = 9, keyword = positive integer, repeatedly
!            keyword may be provided with minus sign

            iseq = pointar(icode)
            call inichkarlen ( numberkeyw(iseq)+1, maxrepeat, 'valuekeyw' )
            if ( ierror/=0 ) go to 1000
            call sepreadtype09 ( finished )

         case(10, 12, 30, 34)

!        --- itype(icode) = 10, hence the keyword must be followed by
!            a sequence number and a number
!            or itype(icode) = 12, 30:
!            the sequence number may be not present

            call sepreadtype10 ( finished, subkey, vectorfound )

         case(11, 28)

!        --- itype(icode) = 11 or 28, &
!            hence the keyword must be followed by
!            a series of (positive) integers

            call sepreadtype11 ( ihelp )
            if ( iposut>nout ) jump = 1

         case(14,18,31)

!        --- itype(icode) = 14, 18, 31 hence the keyword must be followed
!            by a positive integer and possibly the same keyword with
!            another integer and so on

            call sepreadtype18 ( jump, ihelp, finished )

         case(15)

!        --- itype(icode) = 15, hence the keyword must be followed by
!            an integer. This situation may be repeated

            call sepreadtype15 ( finished, ihelp )
            if ( debug ) then
               write(irefwr,*) 'after sepreadtype15'
               write(irefwr,1) 'iposut', iposut
               write(irefwr,2) 'tabout', tabout(iposut-1)
            end if  ! ( debug )
            if ( iposut>nout ) jump = 1

         case(16,26)

!        --- itype(icode) = 16, hence the keyword must be followed by
!                               a value or value = v, or func = i

            call sepreadtype16
            if ( iposut>nout ) jump = 1

         case(17)

!        --- itype(icode) = 17, hence the keyword must be followed by
!                               a series of series of numbers

            call sepreadtype17 ( finished, ihelp, nmax )
            if ( iposut>nout ) jump = 1

         case(19)

!        --- itype(icode) = 19, hence the keyword must be followed by
!                               a series of numbers

            if ( debug ) call prinrl ( tabmax, icode, 'tabmax' )
            call sepreadtype19 ( ihelp, nmax )
            if ( nout<nmax+1 .or. iposut>nout ) jump = 1

         case(20)

!        --- itype(icode) = 20, hence the keyword is a single keyword in
!                               which case the table must be set to
!                               -100000 or the keyword is followed by a
!                               given other keyword and a series of
!                               integers

            call sepreadtype20 ( ihelp, subkey )

         case(21)

!        --- itype(icode) = 21, hence the keyword is followed by
!                               (ax+by+cz=d)

            call sepreadtype21
            if ( iposut>nout ) jump = 1

         case(22, 24)

!        --- itype(icode) = 22, 24 hence the keyword is followed by
!                                  possibly one or two special keywords
!                                  that are skipped and then by a
!                                  series of numbers

            call sepreadtype22 ( subkey, ihelp, nmax )

         case(23)

!        --- itype(icode) = 23, hence the keyword is followed by possibly
!                               a special keyword that is skipped
!                               table(icode) is set equal to 1

            call sepreadtype23 ( subkey )

         case(25)

!        --- itype(icode) = 25, hence the keyword must be followed by
!                               a sequence number followed by
!                               a value or value = v, or func = i

            call sepreadtype25 ( finished )
            if ( iposut>nout ) jump = 1

         case(27)

!        --- itype(icode) = 27, hence the keyword must be followed by
!                               a series of values or value = v, &
!                               or func = i

            call sepreadtype27
            if ( iposut+2>nout ) jump = 1

         case(29,33,37,41,42)

!        --- itype(icode) = 29, 31, 33, 41, 42
!            hence the keyword must be followed by a sequence number
!            possibly a subkeyword and then a series of integers

            call sepreadtype29 ( jump, ihelp, finished )
            if ( jump/=0 ) jump = 1

         case(32)

!        --- type = 32, keyword like:
!            surfaces (i1, i2, ... in ) = s1 to s2 or
!            surfaces = s1 to s2, nodes (i1, i2, ... in )

            call sepreadtype32 ( finished, ihelp, subkey, m )
            if ( iposut+m+2>nout ) jump = 1

         case(35)

!        --- itype(icode) = 35, hence the keyword must be followed by
!                               a sequence number followed by
!                               a value or value = v, or func = i
!                               and again the same keyword followed  by
!                               a sequence number followed by
!                               a value or value = v, or func = i

            call sepreadtype35 ( finished )
            if ( iposut>nout ) jump = 1

         case(38, 40)

!        --- itype(icode) = 38, hence the keyword must be followed by
!            an integer number or a vector name

            call sepreadtype38 ( lastitemsav )

         case(39, 45)

!        --- itype(icode) = 39, hence the keyword must be followed by
!            an integer number or a scalar name

            call sepreadtype39

         case(43)

            call sepreadtype43

      end select  ! case ( itype(icode) )

      if ( jump/=0 ) then

!     --- jump#0, hence tabout too small

         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )

      end if  ! ( jump/=0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'table(icode)', table(icode)
         write(irefwr,1) 'icode, iretur, lastitem, ityprd', &
                          icode, iretur, lastitem, ityprd
         write(irefwr,*) 'readnx', readnx
         write(irefwr,*) 'End sepreadnext'

      end if  ! ( debug )
      call erclos ( 'sepreadnext' )

      end subroutine sepreadnext
