      subroutine sepreadtype01 ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword or keyword = number
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0201
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical finished

!     finished       i    Indicates if the loop has finished (true) or not
!                         (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ispeclsav, lastitemsav, ityprdsav
      double precision realsav
      logical ::  openfound

!     ispeclsav      Help parameter to save the value of ispecl
!     ityprdsav      Help parameter to save the value of ityprd
!     lastitemsav    Help parameter to save the value of lastitem
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     realsav        Help parameter to save the value of realrd
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READNAMEVEC    Read name of vector in common cpost5
!     READREAL       Read a real or real expression from the input file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1468   Array tabout too small
!    1628   Keyword found more than once for the same problem
!    1869   No number found after keyword
!    1870   Integer number read out of range
!    1871   Real number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype01' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype01'
         write(irefwr,1) 'ityprd, icode, itype(icode), lastitem', &
                          ityprd, icode, itype(icode), lastitem
         write(irefwr,1) 'icountkey(icode)', icountkey(icode)
         write(irefwr,*) 'string ', string
         if ( ityprd==2 .or. ityprd==3 ) &
            write(irefwr,2) 'realrd', realrd
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( itype(icode)==0 .or. itype(icode)==1 .or. &
           itype(icode)==3 .or. itype(icode)==36 ) then

!     --- Integer or real expected

         if ( debug ) then
            write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
            write(irefwr,*) 'string ', string(1:nchar)
         end if   ! ( debug )

         if ( ityprd==4 ) then

!        --- string found

            if ( debug ) &
               write(irefwr,1) 'lastitem, typeitems(lastitem)', &
                                lastitem, typeitems(lastitem)
            if ( lastitem<nitems .and. &
                 typeitems(lastitem+1)==13 ) lastitem = lastitem+1  ! = found
            if ( debug ) &
               write(irefwr,1) 'lastitem, typeitems(lastitem+1)', &
                                lastitem, typeitems(lastitem+1)
            if ( lastitem<nitems .and. &
                 typeitems(lastitem+1)==10 ) then

!           --- % found, read item

               call asknxt
               go to 200

            else if ( lastitem<nitems .and. &
                      typeitems(lastitem+1)==3 ) then

!           --- string found, check if it is a vectorname

               ispeclsav = ispecl
               ispecl = 9
               call asknxt
               ispecl = ispeclsav
               if ( debug ) write(irefwr,*) 'string ', string(1:nchar)
               call readnamevec
               if ( debug ) &
                  write(irefwr,1) 'ityprd, lastitem/2', &
                                   ityprd, lastitem
               if ( ityprd/=2 ) then

!              --- Not a vector name found, must be an expression

                  lastitem = lastitembef
                  openfound = .false.
                  call readreal ( openfound, .false. )

               end if  ! ( ityprd/=2 )
               if ( debug ) &
                  write(irefwr,1) 'ityprd, lastitem/3', &
                                   ityprd, lastitem

            else

!           --- Not a string found, must be an expression

               openfound = .false.
               call readreal ( openfound, .false. )
               if ( debug ) write(irefwr,*) 'openfound ', openfound

            end if  ! ( typeitems(lastitem)==10 )

         else

!        --- Not a keyword read
!            Save values

            lastitemsav = lastitem
            ityprdsav = ityprd
            realsav = realrd

            if ( ityprd==2 .or. ityprd==3 ) then

!           --- Real or integer found

               if ( lastitem>=nitems ) go to 200  ! no further reading
               lastitem = lastitembef   ! start with the keyword again

            end if  ! ( ityprd==2 .or. ityprd==3 )
            if ( debug ) &
               write(irefwr,1) 'lastitem, ityprd/1', lastitem, ityprd
            openfound = .false.

!           --- Read real or expression

            call readreal ( openfound, .false. )

            if ( lastitem<lastitemsav ) then

!           --- Not a real or expression fund, must be a vectorname
!               Restore values

               lastitem = lastitemsav
               ityprd = ityprdsav
               realrd = realsav

            end if  ! ( lastitem<lastitemsav )
            if ( debug ) &
               write(irefwr,1) 'lastitem, ityprd/2', lastitem, ityprd

         end if  ! ( ityprd==4 )

      else

!     --- Complex situation, read next item

         call asknxt

      end if  ! ( itype(icode)==0 .or. itype(icode)==1 )

200   if ( debug ) then
         write(irefwr,1) 'ityprd', ityprd
         write(irefwr,2) 'realrd', realrd
         write(irefwr,1) 'icode, icountkey(icode), itype(icode)', &
                          icode, icountkey(icode), itype(icode)
      end if  ! ( debug )

      if ( icountkey(icode)/=0 ) then

!     --- Keyword found more than once
!         In case of create_vector vector = ... and keyword is a vector
!         we jump

         if ( debug ) write(irefwr,*) 'before errorcode>0'
         if ( errorcode>0 .and. &
              code(icode)(1:6)=='VECTOR' .or. code(icode)(1:6)=='V' &
              .and. (typeitems(2)==3 .and. typeitems(3)==13 .or. &
              typeitems(2)==3 .and. typeitems(3)==1 &
              .and. typeitems(4)==13 ) )  then

!        --- create_vector vector = ... and keyword is a vector
!            we jump with error code

            errorcode = -1
            go to 1000

         else

!        --- error message

            call errchr ( code(icode), 1 )
            call errsub ( 1628, 0, 0, 1 )

         end if  ! ( code(icode)(1:6)=='VECTOR' .or. code(icode)(1:6)=='V'.. )

      else
         icountkey(icode) = icountkey(icode)+1
      end if
      if ( debug ) &
         write(irefwr,1) 'itype(icode), ityprd', itype(icode), ityprd

      if ( itype(icode)==0 .and. ityprd==11 ) then

!     --- Variable found

         table(icode) = realrd

      else if ( itype(icode)==1 .and. ityprd==3 .or. &
                ityprd==2 .and. itype(icode)>=0 .or. &
                ityprd==10 .and. itype(icode)==36 .or. &
                ityprd==11 .and. itype(icode)==0 .or. &
                ityprd==11 .and. itype(icode)==1 ) then

!     --- Number of correct type found

         if ( itype(icode)==4 ) then
            iposut = iposut + 1

            if ( iposut>nout ) then
               call errint ( nout, 1 )
               call errsub ( 1468, 1, 0, 0 )
               go to 1000
            end if  ! ( iposut>nout )

            tabout(iposut) = realrd
            table(icode) = iposut
         else
            table(icode) = realrd
         end if

         if ( debug ) write(irefwr,2) 'table(icode)', table(icode)

!        --- Check values

         if ( itype(icode)==0 .or. ityprd==10 ) then

!        --- Integer value read

            valmin = nint(tabmin(icode))
            valmax = nint(tabmax(icode))
            if ( ityprd==11 ) ivalrd = ivalrd-100000000
            ivalrd = nint(realrd)

            if ( debug ) &
               write(irefwr,1) 'valmin, valmax, ivalrd', &
                                valmin, valmax, ivalrd

            if ( ivalrd>5000 .and. valmax< 5000 ) then
               ivalrd = mod(ivalrd,10000)  ! value may correspond to vector
               ivalrd = mod(ivalrd,5000)   ! correction for a series of vectors
            end if  ! ( ivalrd>10000 .and. valmax< 10000 )

            if ( ivalrd<valmin .or. ivalrd>valmax ) then
               call errchr ( code(icode), 1 )
               call errint ( ivalrd, 1 )
               call errint ( valmin, 2 )
               call errint ( valmax, 3 )
               call errsub ( 1870, 3, 0, 1 )
               call errcrd
            end if

         else if ( ityprd/=11 ) then

!        --- Real value read

            if ( realrd<tabmin(icode) .or. &
                 realrd>tabmax(icode) ) then
               call erreal ( realrd, 1 )
               call erreal ( tabmin(icode), 2 )
               call erreal ( tabmax(icode), 3 )
               call errsub ( 1871, 0, 3, 0 )
               call errcrd
            end if

         end if

      else if ( itype(icode)==0 ) then

!     --- Not an integer number found

         call errchr ( code(icode), 1 )
         call errsub ( 574, 0, 0, 1 )
         call errcrd
         finished = .true.

      else if ( itype(icode)==1 ) then

!     --- Not a real number found

         call errchr ( code(icode), 1 )
         call errsub ( 1869, 0, 0, 1 )
         call errcrd
         finished = .true.

      else if ( itype(icode)==3 ) then

!     --- No number found, permitted in this case

         readnx = .false.

      else if ( itype(icode)==4 ) then

!     --- No number found, permitted in this case

         table(icode) = -1d3
         readnx = .false.

      end if

      if ( debug ) write(irefwr,1) 'ityprd, icode', ityprd, icode

      infotable(icode) = ityprd
      indexread(icode) = index_rdarr(1)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
         write(irefwr,*) 'End sepreadtype01'

      end if
      call erclos ( 'sepreadtype01' )

      end subroutine sepreadtype01
