      subroutine sepreadtype25 ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by a sequence number followed by
!     a value(1) or value(1) = v, or func = i
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0225
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ifunc, iseqmn, iseqmx, iseq
      double precision value(1)

!     ifunc          Parameter k after FUNC = &
!     iseq           Sequence number read after keyword
!     iseqmn         Minimum value allowed of sequence number read after keyword
!     iseqmx         Maximum value allowed of sequence number read after keyword
!     value          Array containing the real or complex value read
!                    (2 positions)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFN         Read items of the shape number, VALUE=number or FUNC=i
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1880   Not a sequence number found after the keyword
!    1881   The sequence number found after the keyword is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype25' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype25'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call asknxt
      if ( ityprd/=2 ) then

!     --- Not a sequence number found after the keyword

         call errchr ( code(icode), 1 )
         call errsub ( 1880, 0, 0, 1 )
         call errcrd
         iretur = -1
         finished = .true.
         go to 1000

      end if
      iseq = 1
      if ( ityprd==2 ) iseq = nint(realrd)

!     --- Sequence number found, check if it is in the correct range

      iseqmn = nint(tabmin(icode))
      iseqmx = nint(tabmax(icode))
      if ( iseq<iseqmn .or. iseq>iseqmx ) then

         call errchr ( code(icode), 1 )
         call errint ( iseq, 1 )
         call errint ( iseqmn, 2 )
         call errint ( iseqmx, 3 )
         call errsub ( 1881, 3, 0, 1 )
         call errcrd
         iseq = iseqmn

      end if

!     --- Read next item

      call readfn ( ifunc, value, .false. )
      if ( ifunc>0 .or. ifunc<-100 ) then

!     --- function found

         table(icode+iseq-iseqmn) = ifunc

      else

!     --- value(1) found

         iposut = iposut+1

         if ( iposut>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         table(icode+iseq-iseqmn) = -iposut
         tabout(iposut) = value(1)

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadtype25'

      end if
      call erclos ( 'sepreadtype25' )

      end subroutine sepreadtype25
