      subroutine sepreadtype27
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by a series of values or value = v, &
!     or func = i
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0227
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ifunc
      double precision value(1)
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     i              Counting variable
!     ifunc          Parameter k after FUNC = &
!     value          Array containing the real or complex value read
!                    (2 positions)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFN         Read items of the shape number, VALUE=number or FUNC=i
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype27' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype27'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      compl = .false.
      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if
      table(icode) = iposut+1

      do i = 1, nint(tabmin(icode))
         call readfn ( ifunc, value, compl )
         if ( iposut+2>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )
         tabout(iposut+1) = ifunc
         tabout(iposut+2) = 0d0
         if ( ifunc==0 ) tabout(iposut+2) = value(1)
         iposut = iposut + 2
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadtype27'

      end if
      call erclos ( 'sepreadtype27' )

      end subroutine sepreadtype27
