      subroutine sepreadtype35 ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by a sequence number followed by
!                              a value or value = v, or func = i
!                              and again the same keyword followed  by
!                              a sequence number followed by
!                              a value or value = v, or func = i
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0235
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ifunc, iseq, iseqmn, iseqmx
      double precision value(1)
      logical ::  seqk1, seqk2

!     i              Counting variable
!     ifunc          Parameter k after FUNC = &
!     iseq           Sequence number read after keyword
!     iseqmn         Minimum value allowed of sequence number read after keyword
!     iseqmx         Maximum value allowed of sequence number read after keyword
!     seqk1          Indicates if sequence number 1 has been found
!     seqk2          Indicates if sequence number 2 has been found
!     value          Array containing the real or complex value read
!                    (2 positions)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFN         Read items of the shape number, VALUE=number or FUNC=i
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1880   Not a sequence number found after the keyword
!    1881   The sequence number found after the keyword is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype35' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype35'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      seqk1 =  .false.
      seqk2 =  .false.
      call asknxt
      if ( ityprd/=2 ) then

!     --- Not a sequence number found after the keyword

         call errchr ( code(icode), 1 )
         call errsub ( 1880, 0, 0, 1 )
         call errcrd
         iretur = -1
         finished = .true.
         go to 1000

      end if
      iseq = 1
      if ( ityprd==2 ) iseq = nint(realrd)

!     --- Sequence number found, check if it is in the correct range

      iseqmn = nint(tabmin(icode))
      iseqmx = nint(tabmax(icode))
      if ( iseq<iseqmn .or. iseq>iseqmx ) then

         call errchr ( code(icode), 1 )
         call errint ( iseq, 1 )
         call errint ( iseqmn, 2 )
         call errint ( iseqmx, 3 )
         call errsub ( 1881, 3, 0, 1 )
         call errcrd
         iseq = iseqmn

      end if

      if ( iseq==1) then
         seqk1 = .true.
      else if ( iseq==2 ) then
         seqk2 = .true.
      end if

!     --- Read next item

      if( seqk1 .and. .not. seqk2   .or. &
          seqk2 .and. .not. seqk1) then
         iposut = iposut + 1
         table(icode) = iposut

         if ( iposut+3>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         do i = 1, 4
            tabout(iposut-1+i) =0d0
         end do
      end if

      call readfn ( ifunc, value, .false. )
      if ( ifunc>0 .or. ifunc<-100 ) then

!     --- function found

         if ( iposut+(iseq-1)*2>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         tabout(iposut+(iseq-1)*2) = ifunc

      else

!     --- value found

         if ( iposut+(iseq-1)*2+1>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         tabout(iposut+(iseq-1)*2) = -1d0
         tabout(iposut+1+(iseq-1)*2) = value(1)

      end if

      if ( seqk1 .and. seqk2 ) then
         iposut = iposut + 3
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadtype35'

      end if
      call erclos ( 'sepreadtype35' )

      end subroutine sepreadtype35
