      subroutine seprmsolinfo ( ivectlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove array with sequence number ivectlc from ks
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     remove
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: ivectlc

!     ivectlc      i/o   Defines solution vector by integer sequence number
!                        At output its value is 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i         Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprmsolinfo' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprmsolinfo'
         call ertrace
         write(irefwr,1) 'lastsol, ivectlc', lastsol, ivectlc, nextfreesol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 .or. ivectlc<=0 .or. ivectlc>lastsol ) go to 1000
      if ( solarr(ivectlc)==0 ) go to 900

      iseqsols = ivectlc

      if ( debug ) then
         write(irefwr,1) 'iseqsols', iseqsols
         write(irefwr,*) 'sol', associated(ks(iseqsols)%sol), &
                                size(ks(iseqsols)%sol)
         write(irefwr,*) 'isol', associated(ks(iseqsols)%isol), &
                                 size(ks(iseqsols)%isol)
      end if  ! ( debug )

      if ( associated(ks(iseqsols)%sol) ) deallocate(ks(iseqsols)%sol)
      if ( associated(ks(iseqsols)%isol) ) deallocate(ks(iseqsols)%isol)

      if ( iseqsols==lastsol ) then

!     --- iseqsols = lastsol, decrease lastsol

         lastsol = lastsol-1

         do i = lastsol, 1, -1
            if ( solarr(lastsol)==0 ) then
               lastsol = lastsol-1
            else
               exit
            end if  ! ( solarr(lastsol)==0 )
         end do  ! i = lastsol, 1, -1

      end if  ! ( iseqsols==lastsol )
      solarr(iseqsols) = 0
      nextfreesol = min(nextfreesol,iseqsols)

900   ivectlc = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iseqsols, lastsol, nextfreesol', &
                          iseqsols, lastsol, nextfreesol
         call prinin ( solarr, lastsol, 'solarr' )
         write(irefwr,*) 'End seprmsolinfo'

      end if  ! ( debug )
      call erclos ( 'seprmsolinfo' )

      end subroutine seprmsolinfo
