      subroutine sepstorereal ( inpsep, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store value of real or information of the corresponding scalar
!     or expression in inpsep and rinsep
!     Old subroutine intxstorereal
! **********************************************************************
!
!                       KEYWORDS
!
!     storage
!     real
!     scalar
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: istrin
      integer, intent(out) :: inpsep
      double precision, intent(out) :: rinsep(*)

!     inpsep         o    Contains the integer part of the information read
!                         Possible values:
!                         0   value is 0
!                         >200000000  sequence number of expression + 200000000
!                         >100000000  sequence number of scalar + 100000000
!                         >0          sequence number in rinsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!                         is raised if information is stored in istrin
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstorereal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstorereal'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( infotabout(iposut)==0 ) then

!     --- Real found, store negative address in inpsep and value in rinsep

         rinsep(istrin) = tabout(iposut)  ! value
         inpsep = istrin                  ! address
         istrin = istrin+1                ! raise istrin

      else

!     --- expression or scalar found, store in inpsep

         inpsep = nint(tabout(iposut))

      end if  ! ( infotabout(iposut)==0 )

      iposut = iposut+1  ! raise iposut

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepstorereal'

      end if  ! ( debug )
      call erclos ( 'sepstorereal' )

      end subroutine sepstorereal
