      subroutine sepstrelgrp ( number, ivec, jvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if vector of special structure has a constant number of
!     number unknowns per point for all non-empty elements
!     Old subroutine inistrelgrp
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: number, ivec, jvec

!     ivec           o    Sequence number of vector of special structure
!                         if 0, no vector found
!     jvec           i    present sequence number of vector of special stucture
!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ikelm, indprelc, ielgrp, itype, jnunkp, i
      integer, pointer :: kprobclc(:)

!     i              Counting variable
!     ielgrp         element group number
!     ikelm          Last position used for element group
!     inpelm         number of points per element
!     itype          type number of element
!     jnunkp         Number of degrees of freedom in point
!     indprelc       starting address in kprob part e
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all element groups do
!        jnunkp := number of degrees of freedom per point in vector
!                  of special structure jvec
!        if ( jnunkp#number ) then
!           return
!     ivec := jvec
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstrelgrp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstrelgrp'
         call ertrace
         write(irefwr,1) 'number, jvec', number, jvec
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extract constants

      inpelm = npelm      ! number of points per element
      indpre = kprobloc(18)  ! start of kprob part e
      kelgrp = kprobloc(39)  ! sum of number of points in all element groups
      kprobclc => kprobpart%kprobc
      ivec = 0            ! return value of ivec
      if ( debug ) &
         write(irefwr,1) 'inpelm, nelgrp, kelma, indprelc', &
                          inpelm, nelgrp, kelma, indprelc

      if ( indpre>0 ) then
         ndtvec = kprobloc(6)
         kelgrp = kprobloc(39)
         kprobe => kprobpart%kprobe
         kprobe2 => kprobpart%kprobe2
         kprobe3 => kprobpart%kprobe3
      end if  ! ( indpre>0 )

!     --- For all element groups do

      do ielgrp = 1, nelgrp

!     --- inunkp := number of degrees of freedom per point in solution vector

         itype = kprobclc(ielgrp)  ! type number of element
         if ( itype<0 ) go to 200       ! skip elements with type number<1
         if ( kelma==0) then

!        --- kelma = 0, constant number of nodes per element

            ikelm = (ielgrp-1)*inpelm

         else

!        --- kelma>0, variable number of nodes per element

            ikelm = kmesha(ielgrp)
            inpelm = Topgrp(ielgrp)%inpelm

         end if
         if ( debug ) &
          write(irefwr,1) 'ielgrp, itype, ikelm, inpelm', &
                           ielgrp, itype, ikelm, inpelm

!        --- jnunkp := number of degrees of freedom per point in vector
!                     of special structure jvec

         do i = 1, inpelm

   !         jnunkp = kprobloc(indprelc-1+ikelm+kelgrp*(jvec-1)+i)
            jnunkp = kprobe(ikelm+i,jvec)
            if ( debug ) write(irefwr,1) 'i, jnunkp',i, jnunkp
            if ( jnunkp/=number ) then

!           --- jnunkp#number
!               return

               go to 1000

            end if  ! ( jnunkp/= number )

         end do  ! i = 1, inpelm

200   end do  ! ielgrp = 1, nelgrp

!     --- ivec := jvec

      ivec = jvec    ! apparantly vector found

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepstrelgrp'

      end if
      call erclos ( 'sepstrelgrp' )

      end
