      subroutine sepstrslelgrp ( number, itypvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if solution vector has a constant number of number unknowns
!     per point for all non-empty elements
!     Old subroutine inistrslelgrp
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, itypvc

!     itypvc         o    Indicates if the vector required is solution vector
!                         110: solution vector
!                           0: not solution vector
!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ielgrp, itype, inunkp, jnunkp, i, ikelm
      integer, pointer :: kprobclc(:)

!     i              Counting variable
!     ielgrp         element group number
!     ikelm          Last position used for element group
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in point
!     itype          type number of element
!     jnunkp         Number of degrees of freedom in point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     itypvc = 0
!     For all element groups do
!        inunkp := number of degrees of freedom per point in solution vector
!        if ( inunkp not constant ) then
!           error
!              return
!        else if ( inunkp/=number ) then
!           Vector is not solution vector
!              return
!     itypvc = 110
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstrslelgrp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstrslelgrp'
         write(irefwr,1) 'number', number
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extract constants

      inpelm = npelm   ! number of points per element
      indpra = kprobloc(14)  ! start of kprob part a
      kprobclc => kprobpart%kprobc
      if ( debug ) &
         write(irefwr,1) 'inpelm, nelgrp, kelma, indpra', &
                          inpelm, nelgrp, kelma, indpra

!     --- For all element groups do

      do ielgrp = 1, nelgrp

!     --- inunkp := number of degrees of freedom per point in solution vector

         if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr
         itype = kprobclc(ielgrp)  ! type number of element
         if ( itype<0 ) go to 200       ! skip elements with type number<1
         if ( kelma==0) then

!        --- kelma = 0, constant number of nodes per element

            ikelm = (ielgrp-1)*inpelm

         else

!        --- kelma>0, variable number of nodes per element

            ikelm = kmesha(ielgrp)
            inpelm = Topgrp(ielgrp)%inpelm

         end if
         inunkp = kprobatmp(1)  ! number of unknowns in node 1
         do i = 2, inpelm
            jnunkp = kprobatmp(i)  ! number of unknowns in node i
            if ( jnunkp/=inunkp ) inunkp = -1  ! mark inunkp if not constant
         end do  ! i = 2, inpelm

         if ( debug ) &
          write(irefwr,1) 'ielgrp, itype, ikelm, inpelm, inunkp', &
                           ielgrp, itype, ikelm, inpelm, inunkp

         if ( inunkp/=number ) then

!        --- inunkp # number, so solution vector is not the one we are looking
!            for: jump

            itypvc = 0
            go to 1000

         end if  ! ( inunkp==-1 )

!        --- ivec := jvec

         itypvc = 110  ! apparantly vector found

200   end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'itypvc', itypvc
         write(irefwr,*) 'End sepstrslelgrp'

      end if
      call erclos ( 'sepstrslelgrp' )

      end
