      subroutine septransf1 ( lnkmsc, lncoor, lnkmsm, lnkmsn, kmeshcnew, &
                              kmeshmnew, kmeshnnew, coornew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-07-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy new kmeshc, coor, kmeshm and kmeshn to old arrays
!
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     mesh
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lnkmsc, lncoor, lnkmsm, lnkmsn, kmeshcnew(lnkmsc), &
                 kmeshmnew(lnkmsm), kmeshnnew(lnkmsn)
      double precision :: coornew(ndim,lncoor/ndim)

!     coornew        i    Array containing the new coordinates of the nodes
!     kmeshcnew      i    New kmesh part c
!     kmeshmnew      i    New array kmeshm
!     kmeshnnew      i    New array kmeshn
!     lncoor         i    declared length for coor, resp coorh
!     lnkmsc         i    length of kelem array (kmesh part c)
!     lnkmsm         i    length of curves array (kmesh part m)
!     lnkmsn         i    length of surfaces array (kmesh part n)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMESHC  Make relation between kmeshc and Topgr
!     SEPPOINTKMESHC Fill array pointkmsc with starting addresses of element
!                    groups in kmeshc (minus 1)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septransf1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septransf1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- KMESH part c (elements)
!         Store this array on a permanent position

      deallocate ( km(iseqkm)%kmeshc )
      allocate ( km(iseqkm)%kmeshc(lnkmsc) )
      km(iseqkm)%kmeshc = kmeshcnew
      kmeshc => km(iseqkm)%kmeshc
      km(iseqkm)%kmesh(17) = lnkmsc
      call sepfillkmeshc   ! Make relation between kmeshc and Topgrp
      call seppointkmeshc  ! Fill array pointkmsc

!     --- KMESH part i (coordinates)
!         Store on a permanent position

      deallocate ( km(iseqkm)%coor )
      allocate ( km(iseqkm)%coor(ndim,npoint) )
      km(iseqkm)%coor = coornew
      coor => km(iseqkm)%coor

!     --- KMESH part m (curves)
!         Store on a permanent position

      if ( kelmm/=0 ) then
         deallocate ( km(iseqkm)%kmeshm )
         allocate ( km(iseqkm)%kmeshm(lnkmsm) )
         km(iseqkm)%kmeshm = kmeshmnew
         kmeshm => km(iseqkm)%kmeshm
         km(iseqkm)%kmesh(27) = lnkmsm
      end if  ! ( kelmm/=0 )

!     --- KMESH part n (surfaces)

      if ( ndim==3 .and. kelmn/=0 ) then
         deallocate ( km(iseqkm)%kmeshn )
         allocate ( km(iseqkm)%kmeshn(lnkmsn) )
         km(iseqkm)%kmeshn = kmeshnnew
         kmeshn => km(iseqkm)%kmeshn
         km(iseqkm)%kmesh(28) = lnkmsn
      end if  ! ( kelmn/=0 )

!     --- KMESH part h is not longer available

      kelmh = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septransf1'

      end if  ! ( debug )
      call erclos ( 'septransf1' )

      end subroutine septransf1
