      subroutine topq13 ( neleml, kline )

! =======================================================================
!
!        programmer   Niek Praagman
!
!
!        version 1.1    date   27-05-1994 Determine ishape also if one group
!        version 1.0    date   05-05-1994
!
!
!
!   copyright (c) 1994-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Fill line elements array in case that surface and/or volume elements
!     need not to be subdivided. Divide quadratic line elements (2 and 3D)
!     into elements of linear shape.
!
! ***********************************************************************
!
!     KEYWORDS
!
!     2d
!     3d
!     line_element
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS

      integer neleml, kline(*)

!     kline   o   Array containing "line"elements
!     neleml  o   Number of line elements in kline
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer inpelm, ielgrp, ikelmc, iielem, j, k

!     ielgrp         Element group number (loop variable)
!     iielem         Relative element number with respect to element group
!     ikelmc         position in kmesh part c
!     inpelm         Number of nodes in an element
!     j              Count helpvariable
!     k              Count helpvariable
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     Run through all element groups
!
!       Determine for each group shape number
!
!       Check whether line elements and if so store in line elements array
!
! =======================================================================

      ikelmc  = 0
      neleml = 0

!     Loop over element groups

      do 200 ielgrp = 1, nelgrp

!        Compute ishape number of element group IELGRP

         inpelm = Topgrp(ielgrp)%inpelm
         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem

         if ( ishape==1 ) then

!           Linear line element, copy vertices

            j = 2 * neleml

            do 100 iielem = 1, 2*inelem

                kline( j + iielem ) = kmeshc( ikelmc + iielem )

100         continue

         else if ( ishape==2 ) then

!           Quadratic line element, store vertices only

            do 150 iielem = 1 , inelem

                j = 2 * neleml + 2 * ( iielem - 1 )
                k =     ikelmc  + 3 * ( iielem - 1 )

                kline( j + 1 ) = kmeshc( k + 1 )
                kline( j + 2 ) = kmeshc( k + 3 )

150         continue

         end if

         neleml = neleml + inelem

         ikelmc = ikelmc + inpelm * inelem

200   continue

      end
