      subroutine writb3 ( inum, numarr, namevc, ihelp )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.4  date 21-11-2012 Long integers
!        version  3.3  date 04-04-2001 Corrections for more problems
!        version  3.2  date 27-06-2000 New definitions
!
!   copyright (c) 1982-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     array kprob is written to backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     write
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulexxx
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inum, numarr, ihelp(*)
      character (len=*) namevc

!    ihelp           Work array of length NREC2 which is used as a buffer
!    inum       i    Choice parameter at the input of subroutine WRITBS
!                    When INUM<=0, KPROB is written on the next free positions
!                    of file 2. When INUM = -1, the subroutine prints a text, as
!                    described in the Programmer's Guide.
!                    When INUM>0, KPROB is written on positions of a preceding
!                    written array, with array number INUM.
!    namevc     i    Name of the array to be written to file 2.
!                    Character variable, is used for printing only
!    numarr     o    At output NUMARR gets the sequence number of KPROB in
!                    file 2.
!                    NUMARR must be a variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, lenglc, lengkb, k, iwork(3), i, idummy

!     ivec        Loop variable: gives vector number of array of special
!                 structure
!     iwork       Work array of length 2 in which extar information to be
!                 written is stored
!     k           General loop variable
!     lengkb      Length of array KPROB
!     lenglc      Length of local array in KPROB
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     WRIT00         writes array in buffered form to file 2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The complete array is written to file 2
!     The subroutine finds which parts of KPROB have been filled and which
!     have not been filled.
!     The actual writing is performed by subroutine WRIT00, which
!     uses a buffer of length NREC2
!     In order that preceding versions of file 2 still can be read or written
!     for each part 2 extra positions are written. The contents of these
!     two positions are completely arbitrary.
!     These 2 positions are written before each part, except the first one,
!     however, for part n only one extra position is written and for the
!     remaining parts no extra positions are used
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'writb3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from writb3'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- compute length of array

      iwork(1) = 1
      iseqkp = kprob
      nprob = kp(iseqkp)%nprob

!     --- Compute number of problems

      lengkb = 5
      do iprob = 1, nprob
         lengkb = lengkb+kp(iseqkp)%kprobsub(iprob)%kprobloc(3)
      end do
      lengkb = lengkb+size(kp(iseqkp)%iinputkp)
      if ( associated(kp(iseqkp)%rinputkp) ) &
         lengkb = lengkb+2*size(kp(iseqkp)%rinputkp)

!     --- Compute complete length of arrays

      length = lengkb
      if ( debug ) write(irefwr,1) 'length/1', length

!     --- Loop over all problems

      do iprob = 1, nprob

         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         kproba => kprobpart%kproba

!        --- Kprob part a

         indpra = kprobloc(14)
         numnat = kprobloc(12)
         nbcons = kprobloc(32)
         if ( indpra>0 ) then
            do i = 1, nelgrp+numnat-nbcons
               kprobatmp => kproba(i)%arr
               length = length+2+size(kprobatmp)
            end do  ! i = 1, nelgrp+numnat-nbcons
         end if  ! ( indpra>0 )

!        --- Kprob part b

         indprb = kprobloc(15)
         if ( indprb>0 ) then
            kprobb => kprobpart%kprobb
            length = length+2+size(kprobb)
         end if  ! ( indprb>0 )

!        --- Kprob part c

         kprobc => kprobpart%kprobc
         length = length+2+size(kprobc)

!        --- Kprob part d

         indprd = kprobloc(17)
         if ( indprd>0 ) then
            kprobd => kprobpart%kprobd
            length = length+2+size(kprobd)
         end if  ! ( indprd>0 )

!        --- Kprob part i

         indpri = kprobloc(22)
         if ( indpri>0 ) then
            kprobi => kprobpart%kprobi
            length = length+2+size(kprobi)
         end if  ! ( indpri>0 )

!        --- Kprob part j

         if ( kprobloc(23)>0 ) then
            kprobj => kprobpart%kprobj
            length = length+2+size(kprobj)
         end if  ! ( kprobloc(23)>0 )

         ndtvec = kprobloc(6)
         if ( ndtvec>0 ) then

!        --- ndtvec>0  kprob part e

            kprobe => kprobpart%kprobe
            length = length+size(kprobe)+2
            kprobe2 => kprobpart%kprobe2
            kprobe3 => kprobpart%kprobe3
            length = length+size(kprobe2)+2
            do ivec = 1, ndtvec
               indpre = kprobe2(ivec)
               if ( indpre>0 ) then
                  length = length+size(kprobe3(ivec)%arr)+2
                  if ( debug ) &
                     write(irefwr,1) 'ivec, length, lenglc', &
                                      ivec, length, size(kprobe3(ivec)%arr)+2
               end if  ! ( indpre>0 )

            end do

         end if
         if ( debug ) write(irefwr,1) 'length/2', length

!        --- Kprob parts f

         if ( kprobloc(19)>0 ) then
            kprobf => kprobpart%kprobf
            length = length+2+size(kprobf)
         end if  ! ( kprobloc(19)>0 )
         if ( debug ) write(irefwr,1) 'length/3', length

!        --- Kprob parts n, p-s, iinput and rinput

         if ( kprobloc(27)>0 ) then

!        --- Kprob part n

            kprobn => kprobpart%kprobn
            length = length+2+size(kprobn)
            coortr => kprobpart%coortr
            length = length+2+size(coortr)*intlen

         end if  ! ( kprobloc(27)>0 )

!        --- Kprob part o

         if ( kprobloc(28)>0 ) then
            kprobo => kprobpart%kprobo
            length = length+2+size(kprobo)
         end if  ! ( kprobloc(28)>0 )

!        --- Kprob part p

         if ( kprobloc(35)>0 ) then
            kprobp => kprobpart%kprobp
            length = length+2+size(kprobp)
         end if  ! ( kprobloc(35)>0 )
         if ( debug ) write(irefwr,1) 'length/5', length

         if ( debug ) write(irefwr,1) 'length/6', length
!        do k = 36, 36
!            if ( kprobloc(k)>0 ) length = length + &
!                 infor1(1,kprobloc(k))+1
!         end do
         if ( debug ) write(irefwr,1) 'length/7', length

!        --- Kprob part s

         if ( kprobloc(38)>0 ) then
            kprobs => kprobpart%kprobs
            length = length+2+size(kprobs)
            rprobs => kprobpart%rprobs
            length = length+2+size(rprobs)*intlen
         end if  ! ( kprobloc(38)>0 )
         if ( debug ) write(irefwr,1) 'length/8', length

!        --- Kprob part v

         if ( kprobloc(46)>0 .and. iprob==1 ) then
            kprobvp1 => kprobpart%kprobvp1
            length = length+2+size(kprobvp1)
            kprobvp2 => kprobpart%kprobvp2
            length = length+2+size(kprobvp2)
         end if  ! ( kprobloc(46)>0 )

!        --- Kprob part w

         if ( kprobloc(47)>0 .and. iprob==1 ) then
            kprobwp1 => kprobpart%kprobwp1
            length = length+2+size(kprobwp1)
            kprobwp2 => kprobpart%kprobwp2
            length = length+2+size(kprobwp2)
         end if  ! ( kprobloc(47)>0 )
         if ( debug ) write(irefwr,1) 'length/10', length

!        --- Kprob part h

         if ( kprobloc(50)>0 ) then
            kprobh => kprobpart%kprobh
            length = length+4+size(kprobh)*2
         end if  ! ( kprobloc(50)>0 )

!        --- Kprob part y

         if ( kprobloc(51)>0 ) then
            kproby => kprobpart%kproby
            length = length+2+size(kproby)
            if ( kproby(4)>0 ) length = length+2+2*kproby(4)
         end if  ! ( kprobloc(51)>0 )
         if ( debug ) write(irefwr,1) 'length/11', length

      end do

      if ( debug ) write(irefwr,1) 'length', length

!     --- write parts of arrays to backing storage
!         Array KPROB:

      iwork(1) = 0
      iwork(2) = 101
      call writ00( inum, numarr, namevc, iwork, length, 1, 2, &
                   ihelp )
      iwork(1) = nprob
      iwork(2) = size(kp(iseqkp)%iinputkp)
      iwork(3) = 0
      if ( associated(kp(iseqkp)%rinputkp) ) &
         iwork(3) = size(kp(iseqkp)%rinputkp)*2
      call writ00( inum, numarr, namevc, iwork, length, 2, 3, &
                   ihelp )

!     --- iinput and rinput

      lenglc = size(kp(iseqkp)%iinputkp)
      call writ00( inum, numarr, namevc, kp(iseqkp)%iinputkp, length, 2, &
                   lenglc,ihelp )
      if ( associated(kp(iseqkp)%rinputkp) ) then
         lenglc = 2*size(kp(iseqkp)%rinputkp)
         call writ00( inum, numarr, namevc, kp(iseqkp)%rinputkp, length, 2, &
                      lenglc,ihelp )
      end if  ! ( associated(kp(iseqkp)%rinputkp) )

!     --- kproblocs

      do iprob = 1, nprob
         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         lenglc = kprobloc(3)
         call writ00( inum, numarr, namevc, kprobloc, length, 2, &
                      lenglc,ihelp )
         if ( debug ) call prinin ( kprobloc, lenglc, 'kprobloc' )
      end do

!     --- Loop over the problems

      do iprob = 1, nprob

         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         kproba => kprobpart%kproba

!        --- Kprob part a

         indpra = kprobloc(14)
         numnat = kprobloc(12)
         nbcons = kprobloc(32)
         if ( indpra>0 ) then
            do i = 1, nelgrp+numnat-nbcons
               kprobatmp => kproba(i)%arr
               lenglc = size(kprobatmp)
               iwork(2) = lenglc
               call writ00(inum, numarr, namevc, iwork, length, 2, &
                           2, ihelp)
               call writ00( inum, numarr, namevc, kprobatmp, length, 2, &
                            lenglc,ihelp )
            end do  ! i = 1, nelgrp+numnat-nbcons
         end if  ! ( indpra>0 )

!        --- Kprob part b

         indprb = kprobloc(15)
         if ( indprb>0 ) then
            kprobb => kprobpart%kprobb
            lenglc = size(kprobb)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobb, length, 2, &
                         lenglc,ihelp )
         end if  ! ( indprb>0 )

!        --- Kprob part c

         kprobc => kprobpart%kprobc
         lenglc = size(kprobc)
         iwork(2) = lenglc
         call writ00(inum, numarr, namevc, iwork, length, 2, &
                     2, ihelp)
         call writ00( inum, numarr, namevc, kprobc, length, 2, &
                      lenglc,ihelp )

!        --- Kprob part d

         indprd = kprobloc(17)
         if ( indprd>0 ) then
            kprobd => kprobpart%kprobd
            lenglc = size(kprobd)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobd, length, 2, &
                         lenglc,ihelp )
         end if  ! ( indprd>0 )

!        --- Kprob part e

         ndtvec = kprobloc(6)

!        --- Kprob part e

         if ( ndtvec>0 ) then

!        --- ndtvec>0  kprob part e

            kprobe => kprobpart%kprobe
            lenglc = size(kprobe)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00(inum, numarr, namevc, kprobe, &
                        length, 2, lenglc, ihelp)
            kprobe2 => kprobpart%kprobe2
            kprobe3 => kprobpart%kprobe3
            lenglc = size(kprobe2)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00(inum, numarr, namevc, kprobe2, &
                        length, 2, lenglc, ihelp)
            do ivec = 1, ndtvec
               indpre = kprobe2(ivec)

               if ( indpre>0 ) then

!              --- indpre>0  hence part in ibuffr or on file 3

                  lenglc = size(kprobe3(ivec)%arr)
                  iwork(2) = lenglc
                  call writ00(inum, numarr, namevc, iwork, length, 2, &
                              2, ihelp)
                  call writ00(inum, numarr, namevc, kprobe3(ivec)%arr, &
                              length, 2, lenglc, ihelp)
               end if

            end do

         end if

!        --- Kprob parts f

         if ( kprobloc(19)>0 ) then
            kprobf => kprobpart%kprobf
            lenglc = size(kprobf)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobf, length, 2, &
                         lenglc,ihelp )
         end if  ! ( kprobloc(19)>0 )

!        --- Kprob part i

         indpri = kprobloc(22)
         if ( indpri>0 ) then
            kprobi => kprobpart%kprobi
            lenglc = size(kprobi)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobi, length, 2, &
                         lenglc,ihelp )
         end if  ! ( indpri>0 )

!        --- Kprob part j

         if ( kprobloc(23)>0 ) then
            kprobj => kprobpart%kprobj
            lenglc = size(kprobj)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobj, length, 2, &
                         lenglc,ihelp )
         end if  ! ( kprobloc(23)>0 )

!        --- Kprob parts n, p-s, iinput and rinput

         if ( kprobloc(27)>0 ) then

            kprobn => kprobpart%kprobn
            lenglc = size(kprobn)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobn, length, 2, &
                         lenglc,ihelp )

            coortr => kprobpart%coortr
            lenglc = size(coortr)*intlen
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, coortr, length, 2, &
                         lenglc,ihelp )
         end if

!        --- Kprob part o

         if ( kprobloc(28)>0 ) then
            kprobo => kprobpart%kprobo
            lenglc = size(kprobo)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobo, length, 2, &
                         lenglc,ihelp )
         end if  ! ( kprobloc(28)>0 )

!        --- Kprob part p

         if ( kprobloc(35)>0 ) then
            kprobp => kprobpart%kprobp
            lenglc = size(kprobp)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobp, length, 2, &
                         lenglc,ihelp )
         end if  ! ( kprobloc(35)>0 )

!        --- Kprob part s

         if ( kprobloc(38)>0 ) then
            kprobs => kprobpart%kprobs
            lenglc = size(kprobs)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobs, length, 2, &
                         lenglc,ihelp )

            rprobs => kprobpart%rprobs
            lenglc = size(rprobs)*intlen
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, rprobs, length, 2, &
                         lenglc,ihelp )
         end if  ! ( kprobloc(38)>0 )


!        --- Kprob part v

         if ( kprobloc(46)>0 .and. iprob==1 ) then
            kprobvp1 => kprobpart%kprobvp1
            lenglc = size(kprobvp1)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobvp1, length, 2, &
                         lenglc,ihelp )
            kprobvp1 => kprobpart%kprobvp1
            lenglc = size(kprobvp2)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobvp2, length, 2, &
                         lenglc,ihelp )
            kprobwp1 => kprobpart%kprobwp1
            lenglc = size(kprobwp1)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobwp1, length, 2, &
                         lenglc,ihelp )
            kprobwp1 => kprobpart%kprobwp1
            lenglc = size(kprobwp2)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobwp2, length, 2, &
                         lenglc,ihelp )
         end if  ! ( kprobloc(46)>0 )

!        --- Kprob part h

         if ( kprobloc(50)>0 ) then
            kprobh => kprobpart%kprobh
            lenglc = size(kprobh)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobh, length, 2, &
                         lenglc,ihelp )
            kprobhinv => kprobpart%kprobhinv
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kprobhinv, length, 2, &
                         lenglc,ihelp )
         end if  ! ( indprd>0 )

!        --- Kprob part y

         if ( kprobloc(51)>0 ) then
            kproby => kprobpart%kproby
            lenglc = size(kproby)
            iwork(2) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        2, ihelp)
            call writ00( inum, numarr, namevc, kproby, length, 2, &
                         lenglc,ihelp )
            if ( kproby(4)>0 ) then
               kprobfict => kprobpart%kprobfict
               lenglc = size(kprobfict)
               iwork(2) = lenglc
               call writ00(inum, numarr, namevc, iwork, length, 2, &
                           2, ihelp)
               call writ00( inum, numarr, namevc, kprobfict, length, 2, &
                            lenglc,ihelp )
            end if  ! ( kproby(4)>0 )
         end if  ! ( kprobloc(51)>0 )

      end do

!     --- close writing by writing nothing but using ifirst = 3 in writ00

      call writ00 ( inum, numarr, namevc, idummy, length, 3, &
                    0, ihelp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End writb3'

      end if  ! ( debug )
      call erclos ( 'writb3' )

      end subroutine writb3
