      subroutine writb5 ( inum, numarr, namevc, intmatlc, iwork )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.2  date 13-12-2017 New storage of intmat
!        version  3.1  date 21-11-2012 Long integers
!        version  3.0  date 03-04-1999 Extra parameter ibuffr
!
!   copyright (c) 1984-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     writb5 writes array of intmat type to backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) namevc
      integer :: inum, numarr, intmatlc, iwork(*)

!     intmatlc     i    Standard SEPRAN array containing information of the
!                     solution
!     inum       i    Choice parameter at the input of subroutine WRITBS
!                     When INUM<=0, ISOL is written on the next free positions
!                     of file 2. When INUM = -1, the subroutine prints a text,
!                     as described in the Programmer's Guide.
!                     When INUM>0, ISOL is written on positions of a preceding
!                     written array, with array number INUM.
!     iwork           Work array of length NREC2 which is used as a buffer
!     namevc     i    Name of the array to be written to file 2.
!                     Character variable, is used for printing only
!     numarr     o    At output NUMARR gets the sequence number of ISOL in
!                     file 2.
!                     NUMARR must be a variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ihelp(2), iprio, lenglc, k, intmattmp(5)

!     ihelp          Help array to store two positions from infor into the
!                    backing storage in order to have the same files for old
!                    and new versions of the backing storage subroutines
!     intmattmp      Temperary array to similate old intmat
!     iprio          Help variable to store the old priority temporarily
!                    The new priority is made equal to the old one
!     k              General loop variable
!     lenglc         Length of local array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     WRIT00         writes array in buffered form to file 2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'writb5' )
      write(irefwr,*) 'writb5 is not longer available'
      call ertrace
      call instop

      iprob = intmt(intmatlc)%iprob

!     --- Compute length of array

      length = 5
!      lenglc = size(intmt(intmatlc)%intmatinf)
  !    length = lenglc+7
   !   intmatinf => intmt(intmatlc)%intmatinf
      intmattmp(1) = intmatlc
      intmattmp(2) = 102
      intmattmp(3) = 1
      intmattmp(4:5) = 0

!     --- Write part of array to backing storage

      call writ00 ( inum, numarr, namevc, intmattmp, length, 1, 5, iwork )

!     --- Get intmat arrays into ibuffr

      ihelp(1) = intmatlc
   !   ihelp(2) = lenglc
      call writ00 ( inum, numarr, namevc, ihelp, length, 2, 2, &
                    iwork )
  !    call writ00 ( inum, numarr, namevc, intmatinf, &
  !                  length, 2, lenglc, iwork )

!     --- close writing by writing nothing but using ifirst = 3 in writ00

   !   call writ00 ( inum, numarr, namevc, intmatinf, length, 3, &
   !                 0, iwork )
      call erclos ( 'writb5' )
      end subroutine writb5
