      subroutine writfl ( index, iunder, iupper, ichois, iref, iarray )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.11 date 27-02-2014 Debug statements
!        version  3.10 date 04-06-2007 Fortran 90
!        version  3.9  date 23-11-2002 Only print message if ioutp>=4
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     write iarray from core to SEPRAN direct access file
!
! **********************************************************************
!
!                       KEYWORDS
!
!     direct_access
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      use sepmoduleerr
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iarray(*), index, iunder, iupper, ichois, iref

!     iarray    array in which the input is written depending on the
!               values of ichois and iref
!     ichois    choice parameter,  possibilities
!               1 when iref = 1  irecp1 gets the value of index
!                 when iref>1  ireci gets the value of the last record
!                                written after this subroutine plus one
!               2 when iref = 1  irecp2 gets the value of index
!                 when iref>1  ireci gets the value of the last record
!                                written after this subroutine plus one
!     index     record number from which iarray has to be written
!     iref      dataset reference number indicating the type of file
!               iref = 1, 2, 3, 4 or 17
!     iunder    first element in iarray to be written
!     iupper    last element in iarray to be written
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jref, nrec, n, iho, i, iht, ios

!     i              General loop variable
!     iho            lower index of array to be written in present record
!     iht            upper index of array to be written in present record
!     ios            IOSTATus variable
!     jref           reference number
!     n              number of records to be written
!     nrec           record length of file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1801   Error in writing file
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'writfl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from writfl'
         write(irefwr,1) 'iref, index, iunder, iupper, ierror', &
                          iref, index, iunder, iupper, ierror
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 .and. iref/=4 ) go to 1000

      if ( iref==1 ) then

!     --- iref = 1

         jref = iref1
         nrec = nrec1
         if ( ichois==1 ) then
            irecp1 = index
         else
            irecp2 = index
         end if

      else if ( iref==2 ) then

!     --- iref = 2

         jref = iref2
         nrec = nrec2
         irec2 = index+(iupper-iunder)/nrec2+1
         ifree2 = max(ifree2,irec2)

      else if ( iref==3 ) then

!     --- iref = 3

         jref = iref3
         nrec = nrec3
         irec3 = index+(iupper-iunder)/nrec3+1
         if ( ioutp>=2 .and. irec3>ifree3 ) then

!        --- File 3 is extended

            write(irefwr,100) irec3, namsub(levsub)
100         format ( ' Last record used in file 3 is ', i6, &
                     '  in subroutine ', a )

         end if
         ifree3 = max(ifree3,irec3)

      else if ( iref==4 ) then

!     --- iref = 4

         jref = irefer
         nrec = nrec4

      else

!     --- iref = 17

         jref = iref17
         nrec = nrec4

      end if

      if ( debug ) write(irefwr,1) 'jref, nrec', jref, nrec

      if ( nrec==0 ) then
         write(*,110) iref
110      format ( ///' ************************************'// &
                     ' fatal error in subroutine writfl '/ &
                     ' File with symbolic reference number',i2/ &
                     ' has not been opened, whereas you try to'/ &
                     ' write to this file'// &
                     ' Execution is terminated' // &
                     ' ************************************')
         call instop
      end if
      n = (iupper-iunder)/nrec+1
      iho = iunder

      if ( debug ) write(irefwr,1) 'n, iho', n, iho

      if ( debug .or. ioutp>=4 ) then

!     --- Extra writes to print all record numbers that are used

         write(irefwr,120) namsub(levsub), iref, nrec, index, index+n-1
120      format(' Writing to file in subroutine ',a/ &
                ' File with symbolic ref. number',i2,'  record length', &
                i5, '  records', i4, ' to ', i4 )

      end if

      do i = 1, n
         iht = min(iupper,iho+nrec-1)
         write(jref, rec=index+i-1, iostat=ios) iarray(iho:iht)
         if ( ios/=0 ) then
            call errint ( iref, 1 )
            call errint ( ios, 2 )
            call errsub ( 1801, 2, 0, 0 )
         end if
         iho = iht+1
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End writfl'

      end if  ! ( debug )
      call erclos ( 'writfl' )

      end
