      subroutine askpri ( inpsep, rinsep, number, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.5  date 16-04-2014 Extension with append
!        version  6.4  date 01-06-2013 Allow variable strings
!        version  6.3  date 21-09-2012 Extension with minval and maxval
!
!   copyright (c) 1993-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the options after the keyword print_scalar i or print_vector i
!     and store in inpsep and rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduleplot
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), number, istrin
      double precision rinsep(*)

!     inpsep  o    The integer input read is stored in array INPSEP
!                  Array INPSEP is filled as follows:
!                  1: Scalar sequence number
!                  2: Position in text array
!                  3: Number of characters in text
!                  4:  ISTRIN (Starting address in rinsep)
!                  5:  ICHOICE (inppri(2), see prinvc)
!                  6:  IDIM    (inppri(3), see prinvc)
!                  7:  IQUANT  (inppri(4), see prinvc)
!                  8:  IDEGFD  (inppri(5), see prinvc)
!                  9:  ISEQ    (inppri(6), see prinvc)
!                  10: IREGION (inppri(7), see prinvc)
!                  11: IHEADER (inppri(8), see prinvc)
!                  12: ICOOR (inppri(9), see prinvc)
!                  13: IFORMAT (inppri(10), see prinvc)
!                  14: iequidistant (1: equidistant found)
!                  15: inamefile (0: no filename)
!                  16: file_seq_number (-1: none)
!                  17: file_format (0: standard)
!                  18: MINVAL
!                  19: MAXVAL
!                  20: NREAD   (number of curves, surfaces etc)
!                  21: IPOS position of start of curves etc
!                  22: appendmode
!                      0: not
!                      1: append
!                  23:22+NREAD  If NREAD>0 followed by NREAD curves etc.
!                  If IREGION#0, 6 positions in rinsep are used
!                  If iequidistant#0, 9 positions in rinsep are used
!     istrin i/o   Starting address of information for this subroutine in RINSEP
!     number  i    Scalar sequence number
!     rinsep i/o   Array containing the real input for this subroutine
!                  This array is filled if iregion#0
!                  In that case positions istrin to istrin+5 contain
!                  xmin, xmax, ymin, ymax, zmin, zmax
!                  and also if iequidistant#0
!                  then both xmin, ... are stored as well as dx, dy, dz
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=26, nncode1=12
      character (len=20) code2(nncode2)
      character (len=10) code1(nncode1)
      integer itext, ncode2(nncode2), ncode1(nncode1), itype2(nncode2), &
              nread, k, ncodemx2(nncode2), ichoice, idim, iquant, &
              idegfd, iseq, iregion, iheader, icoor, iformat, &
              iequidist, icodesav
      double precision tabmin2(nncode2), tabmax2(nncode2)

!     code1          Table with subkeywords
!     code2          Table with keywords
!     ichoice        Choice parameter with respect to PRINVC
!                    Possibilities:
!                    0:   The nodal point number is printed followed by the
!                         coordinates and then the values in the nodal point
!                    1:   The nodal point number is printed followed by the
!                         values in the nodal point
!     icodesav       Help parameter to save the value of icode
!     icoor          Type of coordinate system
!                    1: Cartesian
!                    2: Cylindrical
!                    3: Polar
!     idegfd         Degree of freedom in VECTOR(IVEC) that must be used
!     idim           Indicates which quantities must be printed
!                    Possible values:
!                    0: The complete vector is printed
!                    1: The vector is only printed in user points.
!                       These user points must be defined in array ICURVS
!                    2: The vector is only printed in user curves.
!                       These curves must be defined in array ICURVS
!                    3: The vector is only printed in user surfaces.
!                       These surfaces must be defined in array ICURVS
!                    4: The vector is only printed in user volumes.
!                       These volumes must be defined in array ICURVS
!     iequidist      If 1 an equidistant grid is used, if 0 not
!     iformat        Format used for printing
!                    1: e13.5
!                    2: e16.8
!     iheader        indicates if the header must be printed (0) or not (1)
!     iquant         indicates which quantity must be considered
!                    Possible values:
!                    0:  The contents of the vector are considered as stored in
!                        the vector corresponding to IVECTR i.e. without
!                        transformation.
!                    1:  The normal component of a vector along the curve
!                        or surface is considered .
!                    2:  The tangential component(s) of a vector along the
!                        curve or surface is considered .
!     iregion        indicates if the complete region must be printed
!                    (0) or that the printing is restricted to a sub region, &
!                    defined by the parameters xmin, xmax, ymin, ymax, zmin, &
!                    zmax in the common block CPLOT.
!                    Possible values:
!                    0:  The complete region is printed
!                    1:  The region is restricted to xmin<=x<=xmax, &
!                        ymin<=y<=ymax, zmin<=z<=zmax
!     iseq           indicates the sequence in which the coordinates
!                    along the surface must be ordered in case idim=3
!                    Possible values:
!                    1:  increasing x, for fixed x increasing y, &
!                        for fixed y increasing z
!                    2:  increasing y, for fixed y increasing z, &
!                        for fixed z increasing x
!                    3:  increasing z, for fixed z increasing x, &
!                        for fixed x increasing y
!                    4:  sequence of nodal point numbers
!     itext          Position in array TEXTS
!     itype2         Table containing information about the numbers behind
!                    the keywords in the table code2
!     k              Counting variable
!     ncode1         Table containing the number of significant letters in code1
!     ncode2         Table containing the number of significant characters in
!                    the keyword table code2
!     ncodemx2       Maximum number of characters in a keyword
!     nncode1        Number of entries in array code1
!     nncode2        Number of entries in array code2
!     nread          Number of items read
!     tabmax2        Contains maximum values allowed in table
!     tabmin2        Contains minimum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READREALVAR    Put the value of the variable read in rinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     askpri requires input from the standard input file
!     The following input is recognized
!       PRINT VECTOR i, options (all in one line)
!           where options may have the following contents
!           text = t, with t some text between quotes
!           region = (xmin,xmax,ymin,ymax,zmin,zmax)
!           points = p1, p2, p3, ... or alternatively:
!           curves = c1, c2, cn, ... or alternatively:
!           surfaces = s1, s3, s5, ... or surfaces = s1 to s2
!           volumes = v1, v3, v5, ... or volumes = v1 to v2
!           zero_levelset i
!           degfd = k
!           normal_component or alternatively
!           tangential_component
!           suppress_coordinates
!           suppress_header
!           suppress_nodes
!           suppress_coordinates
!           sequence = (y)
!           type_coordinates = cartesian/polar
!           type_format = i
!           equidistant_grid, distance = ( dx, dy, dz )
!           file = '...'
!           file_seq_number = i
!           file_format = form
!           nodes = (i,j,k,...)
!           min_value = ..
!           max_value = ..
!           append
!       This program only interprets the part options
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     665   Number of numbers found in REGION = ( ..., ...,    ) not 2,4 or 6
!    1579   More than one of POINTS, CURVES or SURFACES found
!    1879   Not a text found after the keyword
!    1887   Number of texts read too large
!    2227   Only one of a combination of keywords found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'CURVES', 'DEGFD', 'NORMAL_COMPONENT', 'POINTS', &
                  'REGION', 'SEQUENCE', 'SUPPRESS_COORDINATES', &
                  'SUPPRESS_HEADER', 'SURFACES', &
                  'TANGENTIAL_COMPONENT', 'TEXT', 'TYPE_COORDINATES', &
                  'TYPE_FORMAT', 'VOLUMES', 'EQUIDISTANT_GRID', &
                  'DISTANCE', 'SUPPRESS_ZEROS', &
                  'SUPPRESS_NODES', 'FILE_SEQ_NUMBER', &
                  'FILE_FORMAT', 'FILE', 'ZERO_LEVELSET', 'NODES', &
                  'MIN_VALUE', 'MAX_VALUE', 'APPEND' /
      data itype2   /18, 0, 2,18,19,   5, 2, 2,18, 2, &
                     6, 5, 0,18, 2,  17, 2, 2,39, 5, &
                     6, 0,28, 1, 1,   2 /
      data ncode2   / 3, 3, 4, 4, 3,   3,11,11, 4, 4, &
                     4, 9, 9, 4, 4,   4,12,12, 8, 8, &
                     4, 6, 4, 3, 3,   3 /
      data ncodemx2 / 6, 5,16, 6, 6,   8,20,15, 8,20, &
                     4,16,11, 7,14,   8,14,14,15,11, &
                     4,13, 5, 9, 9,   6 /
      data tabmin2  / 1d2, 1d0, 0d0, 1d2, 6d0,  4d0, 0d0, 0d0, 1d2, 0d0, &
                     0d0, 8d0, 1d0, 1d2, 0d0,  0d0, 0d0, 0d0, 0d0, 0d0, &
                     0d0, 1d0, 1d2, -1d20, -1d20, 0d0 /
      data tabmax2  / 1d4, 1d2, 0d0, 1d4, 1d9,  7d0, 0d0, 0d0, 1d4, 0d0, &
                     0d0, 9d0, 4d0, 1d4, 0d0,  0d0, 0d0, 0d0, 0d0, 0d0, &
                     0d0, 1d0, 1d0, 1d20, 1d20, 0d0 /
      data code1/ 'C', 'P', 'S', 'X', 'Y', 'Z', 'N', 'CARTESIAN', &
                  'POLAR', 'V', 'STANDARD', 'MATLAB' /
      data ncode1  / 1, 1, 1, 1, 1,   1, 1, 4, 4, 1, &
                     4, 4 /

!     code2    Table with main keywords
!             Values:
!             1:  CURVES
!             2:  DEGFD
!             3:  NORMAL_COMPONENT
!             4:  POINTS
!             5:  REGION
!             6:  SEQUENCE
!             7:  SUPPRESS_COORDINATES
!             8:  SUPPRESS_HEADER
!             9:  SURFACES
!            10:  TANGENTIAL_COMPONENT
!            11:  TEXT
!            12:  TYPE_COORDINATES
!            13:  TYPE_FORMAT
!            14:  VOLUMES
!            15:  EQUIDISTANT_GRID
!            16:  DISTANCE
!            17:  SUPPRESS_ZEROS
!            18:  SUPPRESS_NODES
!            19:  FILE_SEQ_NUMBER
!            20:  FILE_FORMAT
!            21:  FILE
!            22:  ZERO_LEVELSET
!            23:  NODES
!            24:  MIN_VALUE
!            25:  MAX_VALUE
!            26:  APPEND
!     code1   Table with subkeywords
!             Values:
!             1:  C
!             2:  P
!             3:  S
!             4:  X
!             5:  Y
!             6:  Z
!             7:  N
!             8:  CARTESIAN
!             9:  POLAR
!            10:  V
!            11:  STANDARD
!            12:  MATLAB
! ======================================================================
!
      call eropen ( 'askpri' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askpri'
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         write(irefwr,1) 'number, icode', number, icode
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      icodesav = icode

!     --- Copy arrays in code, itype, codesub and so on

      call sepcopycodesub ( code1, nncode1, ncode1 )
      call sepcopycode ( code2, nncode2, ncode2, ncodemx2 )
      call sepcopytype ( nncode2, itype2, tabmin2, tabmax2 )
      nmain = 0

      inpsep(1:22) = 0
      itext = 0
      iretur = 0
      nread = 0

      if ( icode==10 .or. icode==114 ) then

         inpsep(15:20) = 0
         inpsep(16) = -1        ! file_seq_number

      end if

      table(1:nncode) = -1d0

      if ( icode==15 ) then

!     --- print text
!         Read text

         if ( ityprd==4 ) then

!        --- text has been found

            call asknxt
            if ( ityprd==5 ) then

               if ( nextxt>ntexts ) then
                  call errint ( ntexts, 1 )
                  call errsub ( 1887, 1, 0, 0 )
                  call errcrd
               else
                  itext = nextxt
                  texts(nextxt) = string(1:nchar)
                  nextxt = nextxt + 1
               end if

            else

!           --- not a text found

                call errchr ( 'TEXT', 1 )
                call errsub ( 1879, 0, 0, 1 )
                call errcrd

            end if

         else

!        --- No text found

            nchar = 0

         end if
         inpsep(1) = itext
         inpsep(2) = nchar

      end if  ! ( icode==15 )

      if ( debug ) write(irefwr,1) 'lastitem', lastitem

!     --- print vector or print scalar

      ichoice = 1
      idim    = 0
      iquant  = 0
      idegfd  = 0
      iseq    = 0
      iregion = 0
      iheader = 0
      nchar   = 0
      icoor   = 1
      iformat = 1
      iequidist = 0

      if ( icode==10 .or. icode==114 ) then
         do k = 1, 3
            rinsep(istrin+2*k-2) = -rinfin
            rinsep(istrin+2*k-1) = rinfin
         end do
      end if

      table(1) = -1d0
      table(4) = -2d0
      table(9) = -3d0
      table(14) = -10d0
      table(24) = -1d30
      table(25) = -1d30
      tabmin(16) = ndim  ! ndim numbers (distance)
      tabmin(20) = 11d0  ! start of sub keywords for file_format
      tabmax(1) = ncurvs
      tabmax(4) = nuspnt
      tabmax(5) = 1d9
      tabmax(9) = nsurfs
      tabmax(14) = nvolms
      tabmax(16) = 1d0   ! number of series (distance)
      tabmax(20) = 12d0  ! end of sub keywords for file_format
      tabmax(23) = 100*npoint
      table(26) = 0d0   ! append

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      icode = icodesav

      if ( table(1)>=0d0 ) then

!     --- Curves = ( c1, c2, c3, ... ) found

         idim = 2
         iposut = nint(table(1))
         nread  = nint(tabmax(1))

      end if
      if ( table(4)>=0d0 ) then

!     --- Points = ( p1, p2, p3, ... ) found

         if ( idim/=0 ) then

!        --- Curves has already been found

            call errsub ( 1579, 0, 0, 0 )

         end if
         idim = 1
         iposut = nint(table(4))
         nread  = nint(tabmax(4))

      end if
      if ( table(9)>=0d0 ) then

!     --- Surfaces = ( s1, s2, s3, ... ) found

         if ( idim/=0 ) then

!        --- Points or curves has already been found

            call errsub ( 1579, 0, 0, 0 )

         end if
         idim = 3
         iposut = nint(table(9))
         nread  = nint(tabmax(9))

      end if
      if ( table(14)>=0d0 ) then

!     --- Volumes = ( v1, v2, v3, ... ) found

         if ( idim/=0 ) then

!        --- Points or curves has already been found

            call errsub ( 1579, 0, 0, 0 )

         end if
         idim = 4
         iposut = nint(table(14))
         nread  = nint(tabmax(14))

      end if
      if ( table(22)>=0d0 ) then

!     --- zero_levelset found

         if ( idim/=0 ) then

!        --- Points or curves has already been found

            call errsub ( 1579, 0, 0, 0 )

         end if
         idim = 20
         inpsep(20)  = nint(table(22))
         inpsep(21)  = 0  ! to avoid later problems in erscal

      end if  ! ( table(22)>=0d0 )

      if ( table(23)>=0d0 ) then

!     --- nodes found

         if ( idim/=0 ) then

!        --- Points or curves has already been found

            call errsub ( 1579, 0, 0, 0 )

         end if
         idim = 23
         iposut = nint(table(23))
         nread  = nint(tabmax(23))

      end if  ! ( table(22)>=0d0 )

      if ( idim>0 .and. idim<20 .or. idim==23 ) then

!     --- Store curve, point or surface numbers

         inpsep(20) = nread
         do k = 1, nread
            inpsep(23:22+nread) = nint(tabout(iposut:iposut+nread-1))
         end do

      end if  ! ( idim>0 .and. idim<20 )
      if ( table(2)>=0d0 ) idegfd = nint(table(2))
      if ( table(3)>=0d0 ) iquant = 1
      if ( table(10)>=0d0 ) iquant = 2

      if ( icode==10 .or. icode==114 ) then

!     --- Print vector found, check on min_value and max_value

         icode = 24
         call readrealvar ( rinsep, inpsep(18), istrin )
         icode = 25
         call readrealvar ( rinsep, inpsep(19), istrin )
         icode = icodesav

      end if  ! ( icode==10 .or. icode==114 )

      if ( (icode==10 .or. icode==114) .and. table(5)>=0d0 ) then

!     --- REGION = ( xmin, xmax, ymin, ymax, zmin, zmax )
!         Place values in RINSEP

         iposut = nint(table(5))
         nread  = nint(tabmax(5))
         if ( nread<2 .or. nread==3 .or. nread==5 ) then
            call errint ( nread, 1 )
            call errsub ( 665, 1, 0, 0 )
            call errcrd
         end if
         iregion = 1

!        --- Fill numbers in rinsep

         rinsep(istrin) = tabout(iposut)
         rinsep(istrin+1) = tabout(iposut+1)
         if ( nread>2 ) then
            rinsep(istrin+2) = tabout(iposut+2)
            rinsep(istrin+3) = tabout(iposut+3)
            if ( nread>4 ) then
               rinsep(istrin+4) = tabout(iposut+4)
               rinsep(istrin+5) = tabout(iposut+5)
            end if
         end if

      end if
      if ( table(6)>=0d0 ) iseq = nint(table(6))
      if ( table(7)>=0d0 ) ichoice = 0
      if ( table(8)>=0d0 ) iheader = 1
      if ( table(11)>=0d0 ) then

!     --- Text = 'text' found

         iposut = nint(table(11))
         nchar  = nint(tabmax(11))
         itext = nextxt
         texts(nextxt) = strout(iposut)(1:nchar)
         nextxt = nextxt + 1

      end if
      if ( table(12)>=0d0 ) then
         icoor = nint(table(12))
         if ( icoor==2 ) icoor = 3
      end if
      if ( table(13)>=0d0 ) iformat = nint(table(13))
      if ( table(15)>0d0 ) then

!     --- table(15)>0, hence equidistant grid found

         iequidist = 1
         if ( table(16)<0d0 ) then

!        --- No distances found

            call errchr ( code(15)(1:ncodemx2(15)), 1 )
            call errchr ( code(16)(1:ncodemx2(16)), 2 )
            call errsub ( 2227, 0, 0, 2 )

         else

!        --- Distances found

            iposut = nint(table(16))
            do k = 0, ndim-1
               rinsep(istrin+6+k) = tabout(iposut+k)
            end do

         end if
         if ( table(5)<0d0 ) then

!        --- No region found

            call errchr ( code(15)(1:ncodemx2(15)), 1 )
            call errchr ( code(5)(1:ncodemx2(5)), 2 )
            call errsub ( 2227, 0, 0, 2 )

         end if

      end if  ! ( table(15)>0d0 )
      if ( table(17)>0d0 ) ichoice = ichoice+100  ! suppress_zeros
      if ( table(18)>0d0 ) ichoice = ichoice+10   ! suppress_nodes
      if ( (icode==10 .or. icode==15 .or. icode==114) .and. &
           table(21)>=0d0 ) then

!     --- File = 'name' found

         iposut = nint(table(21))
         if ( iposut>100000000 ) then

!        --- Text is a string variable
!            Store nextxt in inpsep(15)

            inpsep(15) = instring(iposut-100000000)

         else

!        --- Standard case, text found, store in array texts

            nchar = nint(tabmax(21))
            texts(nextxt) = strout(iposut)(1:nchar)
            inpsep(15) = nextxt  ! position of name in array texts
            nextxt = nextxt + 1

         end if  ! ( iposut>100000000 )

         if ( table(19)>0d0 ) inpsep(16) = nint(table(19)) ! file_seq_number

      end if  ! ( icode==10 .or. icode==114 .and. table(21)>=0d0 )
      if ( table(20)>0d0 ) inpsep(17) = nint(table(20))-1  ! file_format
      inpsep(22) = nint(table(26))  ! append mode

      if ( icode==15 ) then

!     --- icode = 15, print text

         inpsep(4) = inpsep(15)
         inpsep(5) = inpsep(22)

      else

!     --- icode#15

         inpsep(1) = number
         inpsep(2) = itext
         inpsep(3) = nchar
         inpsep(21) = 23
      end if  ! ( icode/=15 )
      if ( icode==10 .or. icode==114 ) then

!     --- icode = 10, print vector

         inpsep(4) = istrin
         inpsep(5) = ichoice
         inpsep(6) = idim
         inpsep(7) = iquant
         inpsep(8) = idegfd
         inpsep(9) = iseq
         inpsep(10) = iregion
         inpsep(11) = iheader
         inpsep(12) = icoor
         inpsep(13) = iformat
         inpsep(14) = iequidist
         if ( iregion/=0 ) istrin = istrin+6
         if ( iequidist/=0 ) istrin = istrin+3

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icode' ,icode
         call prinin ( inpsep, 22+nread, 'inpsep' )
         if ( (icode==10 .or. icode==114) .and. inpsep(10)>0 ) then
            call prinrl ( rinsep(istrin-6), 6, 'rinsep' )
         end if  ! ( icode==10 .or. icode==114 .and. inpsep(10)>0 )

         write(irefwr,*) 'End askpri'

      end if
      call erclos ( 'askpri' )

      end subroutine askpri
