      subroutine askprikmesh ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 04-05-2014 Remove istart
!        version  1.0  date 06-11-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the information following print kmesh and store in inpsep
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         For a description, see subroutine askprisca
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askprikmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askprikmesh'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call asknxt  ! read next item
      if ( debug ) then
         write(irefwr,1) 'ityprd, nchar', ityprd, nchar
         write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

!     --- Fill inpsep

      inpsep(1) = 0  ! No variables
      inpsep(2) = -1002  ! print kmesh

      if ( ityprd/=4 ) then

!     --- No letter found

         inpsep(3) = 0  ! print complete kmesh

      else

!     --- letter found

         if ( string(1:nchar)=='C' .or. string(1:nchar)=='G' ) then
            inpsep(3) = 1   ! print complete kmesh c and g
         else if ( string(1:nchar)=='D' ) then
            inpsep(3) = 2   ! print complete kmesh d
         else if ( string(1:nchar)=='E' ) then
            inpsep(3) = 3   ! print complete kmesh e
         else if ( string(1:nchar)=='J' ) then
            inpsep(3) = 4   ! print complete kmesh j
         else if ( string(1:nchar)=='H' ) then
            inpsep(3) = 5   ! print complete kmesh h
         else if ( string(1:nchar)>='L' .and. string(1:nchar)<='N'  ) &
            then
            inpsep(3) = 6   ! print complete kmesh l, m and n
         else if ( string(1:nchar)=='O' ) then
            inpsep(3) = 7   ! print complete kmesh o
         else if ( string(1:nchar)=='K' ) then
            inpsep(3) = 8   ! print complete kmesh k
         else if ( string(1:nchar)>='Q' .and. string(1:nchar)<='S'  ) &
            then
            inpsep(3) = 9   ! print complete kmesh q, r and s
         else if ( string(1:nchar)=='IINPUT' .or. &
                   string(1:nchar)=='RINPUT' ) then
            inpsep(3) = 30  ! print complete kmesh iinput/rinput
         else if ( string(1:nchar)=='T' ) then
            inpsep(3) = 31  ! print complete kmesh t
         else if ( string(1:nchar)=='U' ) then
            inpsep(3) = 32  ! print complete kmesh u
         else if ( string(1:nchar)=='V' ) then
            inpsep(3) = 33  ! print complete kmesh v
         else if ( string(1:nchar)=='W' ) then
            inpsep(3) = 34  ! print complete kmesh w
         else if ( string(1:nchar)=='X' ) then
            inpsep(3) = 35  ! print complete kmesh x
         else if ( string(1:nchar)=='Z' ) then
            inpsep(3) = 36  ! print complete kmesh z
         else if ( string(1:nchar)=='Y' ) then
            inpsep(3) = 37  ! print complete kmesh y
         else if ( string(1:nchar)=='I' ) then
            inpsep(3) = 38  ! print complete kmesh i
         else if ( string(1:nchar)=='NMESHC' ) then
            inpsep(3) = 39  ! print complete kmesh nmeshc
         else if ( string(1:nchar)=='NMESHD' ) then
            inpsep(3) = 40  ! print complete kmesh nmeshd
         else if ( string(1:nchar)=='NMESHL' ) then
            inpsep(3) = 41  ! print complete kmesh nmeshl
         else if ( string(1:nchar)=='AG' .or. &
                   string(1:nchar)=='VOLUME_SURFS' ) then
            inpsep(3) = 42  ! print complete kmesh ag
         else if ( string(1:nchar)=='NMESHH' ) then
            inpsep(3) = 43  ! print complete kmesh nmeshh
         else if ( string(1:nchar)=='NMESHA' ) then
            inpsep(3) = 44  ! print complete kmesh nmesha
         else if ( string(1:nchar)=='NMESHB' ) then
            inpsep(3) = 45  ! print complete kmesh nmeshb
         else if ( string(1:nchar)=='NMESHF' ) then
            inpsep(3) = 46  ! print complete kmesh nmeshf
         else if ( string(1:nchar)=='NMESHG' ) then
            inpsep(3) = 47  ! print complete kmesh nmeshg
         else if ( string(1:nchar)=='NMESHK' ) then
            inpsep(3) = 48  ! print complete kmesh nmeshk
         else if ( string(1:nchar)=='BLOCKINFO' .and. &
                   string(1:nchar)=='TRANSPORTINFO') then
            inpsep(3) = 49  ! print complete kmesh block/transportinfo
         else if ( string(1:nchar)=='IOUTERCURVS' .or. &
                   string(1:nchar)=='IOUTERSURFS' ) then
            inpsep(3) = 50  ! print complete ioutercurvs/surfs
         else if ( string(1:nchar)=='AI' .or. &
                   string(1:nchar)=='SURFNODES' ) then
            inpsep(3) = 59  ! print complete kmesh ai
         else if ( string(1:nchar)=='AJ' .or. &
                   string(1:nchar)=='NORMALS' ) then
            inpsep(3) = 60  ! print complete kmesh aj
         else if ( string(1:nchar)=='AL' ) then
            inpsep(3) = 61  ! print complete kmesh al
         else if ( string(1:nchar)=='AE' ) then
            inpsep(3) = 62  ! print complete kmesh ae
         else if ( string(1:nchar)=='A' .or. string(1:nchar)=='B' .or. &
                   string(1:nchar)=='F' ) then
            inpsep(3) = 63  ! print complete kmesh a, b and f
         else if ( string(1:nchar)=='NMESHI' ) then
            inpsep(3) = 64  ! print complete kmesh nmeshi
         else if ( string(1:nchar)=='QUANT' ) then
            inpsep(3) = 66  ! print complete kmesh_quant
         end if  ! ( string(1:nchar)=='C' .or. string(1:nchar)=='G' )

      end if  ! ( ityprd/=4 )

1000  call erclos ( 'askprikmesh' )
      if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(0), 4, 'inpsep' )
         write(irefwr,*) 'End askprikmesh'

      end if  ! ( debug )

      end
