      subroutine asksca4 ( inpsep, rinsep, istrin, leninpsep, icode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.6  date 12-02-2015 New call to readexpress
!        version  3.5  date 08-12-2013 Extension with integer scalars
!        version  3.4  date 20-04-2012 Indexed scalars
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the part following scalar i = ... in case
!     a keyword has been found
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons 
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istrin, leninpsep, icode
      double precision rinsep(*)

!     icode          i    Return code with respect to subroutine INTX00
!                         Indicates position in array code
!     inpsep         o    The integer input read is stored in array INPSEP
!                         Array INPSEP is filled as described in asksca
!     istrin        i/o   Next free position in RINSEP
!     leninpsep      o    Length of this part of array inpsep
!     rinsep         o    Real input array, the value is store in rinsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ifound, ifunc, ipos
      logical ::  finished, openfound, closefound, vecfound

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     finished       If true the do-loop is finished
!     ifound         Number of items found after the keywords MIN or MAX
!     ifunc          Output parameter of elflc8 to be used if ind(number)>4000
!                    It indicates if the parameter which with the function
!                    is multiplied is a function (>0) or a constant (<0)
!     ipos           Position in array inpsep
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     vecfound       If true the expression contains vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     READEXPRESS    Interprete part of input as number or expression
!     READREALSCAL   Read integer, real or scalar
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!     898   Parameter in FUNC = k non-positive
!    1533   Number of items read after keyword incorrect
!    1869   No number found after keyword
!    2968   Keyword not followed by openings brackett
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'asksca4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from asksca4'
         write(irefwr,1) 'icode, lastitem', icode, lastitem
         write(irefwr,1) 'istrin', istrin
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      inpsep(2) = icode
      leninpsep = 3
      closefound = .false.

      select case ( icode )

         case(0)

!        --- icode = 0 no action

         case(1)

!        --- icode = 1, VALUE found, read value

            call readrealscal
            if ( ityprd==102 ) ityprd = 2  ! vector found
            if ( debug ) write(irefwr,1) 'ityprd', ityprd
            if ( ityprd/=2 .and. ityprd/=3 ) then

!           --- Not a number found after VALUE

               call errchr ( string(1:nchar), 1 )
               call errsub ( 1869, 0, 0, 1 )
               call errcrd

            end if
            rinsep(istrin) = realrd
            inpsep(3) = istrin
            istrin = istrin + 1
            go to 1000

         case(2)

!        --- icode = 2, FUNC found, read IFUNC

            call asknxt
            if ( ityprd==2 ) then

!           --- Integer found

               ifunc = nint(realrd)
               if ( ifunc<=0 .or. ifunc>20000 ) then

!              --- IFUNC out of range

                  call errint ( ifunc, 1 )
                  call errint ( 20000, 2 )
                  call errsub ( 898, 2, 0, 0 )
                  call errcrd

               end if

            else if ( ityprd==11 ) then

!           --- scalar found

               ifunc = nint(realrd)-100000000

            else

!           --- Not an integer found

               call errchr ( string(1:nchar), 1 )
               call errsub ( 574, 0, 0, 1 )
               call errcrd

            end if
            inpsep(3) = ifunc
            go to 1000

         case(3,4)

!        --- icode = 3 or 4, MIN or MAX found
!            Next item must be (

            ifound = 0
            ipos = 4
            finished = .false.
            lastitem = lastitem+1
            if ( typeitems(lastitem)/=5 ) then

!           --- Error min or max not followed by (

               call errchr ( string(1:nchar), 1 )
               call errsub ( 2968, 0, 0, 1 )
               call errcrd
               go to 1000

            end if  ! ( typeitems(lastitem)/=5 )
            if ( debug ) write(irefwr,*) 'min or max'

            do while ( .not. finished )

!           --- while loop finished

               if ( ifound>100 ) then
                  if ( icode==3 ) then
                     call errchr ( 'MIN', 1 )
                  else
                     call errchr ( 'MAX', 1 )
                  end if
                  call errint ( ifound, 1 )
                  call errint ( 2, 1 )
                  call errint ( 100, 1 )
                  call errsub ( 1533, 3, 0, 1 )
                  call errcrd
                  finished = .true.
                  go to 1000
               end if

               openfound = .true.
               call readexpress ( openfound, closefound, .true., &
                                  .false., .false., vecfound )
               if ( debug ) then
                  write(irefwr,*) 'after readexpress'
                  write(irefwr,1) 'ityprd', ityprd
                  write(irefwr,2) 'realrd', realrd
                  write(irefwr,*) 'openfound, closefound ', &
                                   openfound, closefound
               end if  ! ( debug )

               if ( closefound ) then

!              --- end of record or ) found

                  finished = .true.

               end if  ! ( closefound )

               if ( ityprd==3 ) then

!              --- + followed by integer, value found

                  inpsep(ipos) = istrin
                  istrin = istrin+1
                  rinsep(istrin) = realrd
                  ipos = ipos+1

               else if ( ityprd==11 .or. ityprd==19 ) then

!              --- scalar or expression found

                  inpsep(ipos) = nint(realrd)
                  if ( debug ) &
                     write(irefwr,1) 'ipos, inpsep(ipos)', &
                                      ipos, inpsep(ipos)
                  ipos = ipos+1

               end if  ! ( ityprd==3 )
               ifound = ifound + 1

!              --- Skip , &

 !              if ( typeitems(lastitem)==26 ) lastitem = lastitem+1  ! , &

            end do  ! while ( .not. finished )

            inpsep(3) = ifound
            leninpsep = ipos-1
            if ( ifound<2 ) then
               if ( icode==3 ) then
                  call errchr ( 'MIN', 1 )
               else
                  call errchr ( 'MAX', 1 )
               end if
               call errint ( ifound, 1 )
               call errint ( 2, 1 )
               call errint ( 100, 1 )
               call errsub ( 1533, 3, 0, 1 )
               call errcrd
            end if

         case(7)

!        --- icode = 7 get_time

            inpsep(2) = 9   ! get_time

      end select  ! case ( icode )

1000  call erclos ( 'asksca4' )
      if ( debug ) then

!     --- Debug information

         call prininlarge ( inpsep, leninpsep, 'inpsep' )
         write(irefwr,*) 'End asksca4'

      end if

      end
