      subroutine askvec25 ( inpsep, numpar, table2, nelgrp, tabout, &
                            tabmx2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-12-2013 New storage of information
!        version  1.0  date 07-05-2010
!
!   copyright (c) 2010-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store information about skip element groups
!     with respect to askvec
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), numpar, nelgrp
      double precision table2(*), tabout(*), tabmx2(*)

!     inpsep         o    The integer input read is stored in array INPSEP
!                         See subroutine askvec for a description
!     numpar         i    Number of parameters in inpsep before information
!                         concerning the element groups
!     nelgrp         i    Number of element groups
!     table2         i    Contains values of items read corresponding to code2
!     tabmx2         i    Table containing the maximum values of the quantities
!                         given in code2
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipos, k, iwork(nelgrp), numelgrps, numskip

!     ipos           Position in array tabout
!     iwork          integer work array of length nelgrp
!                    indicates if an element group must be used (0) or skipped
!                    (1)
!     k              Counting variable
!     numelgrps      Number of element groups read
!     numskip        Number of skip element groups read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvec25' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvec25'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numpar = numpar+1
      inpsep(14) = numpar               ! start in inpsep
      if ( table2(22)>0d0 ) then

!     --- element groups = given

         ipos  = nint(table2(22))       ! start in tabout
         numelgrps = nint(tabmx2(22))   ! number of groups to be used
         inpsep(numpar) = numelgrps     ! fill number of groups to be used

!        --- Fill element groups to be used

         inpsep(numpar+1:numpar+numelgrps) = &
            nint(tabout(ipos:ipos+numelgrps-1))
         numpar = numpar+numelgrps      ! raise numpar

      else

!     --- skip element groups = given

         numskip = nint(tabmx2(23))  ! number of groups to be skipped
         numelgrps = 0               ! number of groups to be used
         ipos = nint(table2(23))     ! start in tabout
         iwork(1:nelgrp) = 0         ! mark as to be used
         iwork(nint(tabout(ipos:ipos+numskip-1))) = 1  ! mark to be skipped

         if ( debug ) call prinin ( iwork, nelgrp, 'iwork' )

!        --- Loop over all element groups

         do k = 1, nelgrp

            if ( iwork(k)==0 ) then

!           --- element group must be used

               numelgrps = numelgrps+1
               inpsep(numpar+numelgrps) = k  ! store group

            end if  ! ( iwork(k)==0 )

         end do  ! k = 1, nelgrp
         inpsep(numpar) = numelgrps     ! fill number of groups to be used
         numpar = numpar+numelgrps      ! raise numpar

      end if  ! ( table2(22)>0d0 )

1000  call erclos ( 'askvec25' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numelgrps', numelgrps
         call prinin ( inpsep, numpar, 'inpsep' )
         write(irefwr,*) 'End askvec25'

      end if  ! ( debug )

      end
