      subroutine askveccurvs ( inpsep, table2, tabmx2, ncurvs, numpar, &
                               tabout, iposcur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 17-01-2015 Extra parameter iposcur
!        version  1.1  date 30-12-2013 New storage of information
!        version  1.0  date 07-05-2010
!
!   copyright (c) 2010-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret information about curves
!     with respect to subroutine askvec
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), ncurvs, numpar, i, icurnr, iposcur
      double precision table2, tabmx2, tabout(*)

!     inpsep         o    The integer input read is stored in array INPSEP
!                         See subroutine askvec for a description
!     iposcur        i    Position in inpsep where information of
!                         start address of curves must be stored
!     numpar         i    Number of parameters in inpsep before information
!                         concerning the curves
!     ncurvs         i    Number of curves in mesh
!     table2         i    Contains values of items read corresponding to code2
!     tabmx2         i    Table containing the maximum values of the quantities
!                         given in code2
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposut, iwork(ncurvs), ncurves

!     i              Counting variable
!     icurnr         Curve sequence number
!     iposut         Position in array tabout
!     iwork          integer work array of length ncurvs
!                    iwork(i) = 0, means curve number i not yet used, &
!                               1, means in use
!     ncurves        Number of curves that is read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1949   Curve number found more than once in a set of curve numbers
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askveccurvs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askveccurvs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

!     --- interpret information

      iposut = nint(table2)
      ncurves = nint(tabmx2)
      inpsep(iposcur) = numpar        ! starting address
      inpsep(numpar) = ncurves   ! number of curves

!     --- Store curve numbers

      do i = 1, ncurves

         icurnr = abs(nint(tabout(iposut+i-1)))
         inpsep(numpar+i) =icurnr

         if ( iwork(icurnr)/=0 ) then

!        --- Error curve number appeared more than once

            call errint ( icurnr, 1 )
            call errsub ( 1949, 1, 0, 0 )

         end if  ! ( iwork(icurnr)/=0 )

         iwork(icurnr) = 1

      end do  ! i = 1, ncurves

      numpar = numpar+ncurves  ! raise numpar

1000  call erclos ( 'askveccurvs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askveccurvs'

      end if  ! ( debug )

      end
