      subroutine askvecin ( inpsep, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 03-07-2012 Use subroutine intxstoretext
!        version  2.2  date 13-11-2010 error 1488 --> errkeywexcl
!        version  2.1  date 11-09-2008 Allow vector names
!
!   copyright (c) 1996-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Interpret the options after the keyword input_vector
!    and store in inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), ndim

!     inpsep  o    The integer input read is stored in array INPSEP
!                  Array INPSEP is filled as follows:
!                  1:  Sequence number of output vector
!                  2:  Number of unknowns per point
!                      if <0 -number of unknowns per element
!                  3:  Position of file name in text array
!                  4:  number of characters of file name
!     ndim    i    Dimension of space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, ind(4), k, lastitemsav, ispeclsav, icodesav

!     icodesav       Help parameter to save the value of icode
!     ind            Array of length 4 that is used to store sequence numbers
!                    that have been found
!     iseq           sequence number of vector
!     ispeclsav      Help parameter to save the value of ispecl
!     k              Counting variable
!     lastitemsav    Help parameter to save the value of lastitem
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     INTXSTORETEXT  Fill reference to text and number of characters in inpsep
!     READNAMEVEC    Read name of vector in common cpost5
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     askvecin requires input from the standard input file
!     The following input is recognised:
!       INPUT_VECTOR j, file= 'file name', type_of_vector
!           where type_of_vector may be:
!           scalar
!           vector
!           tensor
!           element_wise
!           number = n
!       This program interprets the text after input_vector
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1881   The sequence number found after the keyword is incorrect
!    1993   No file name found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read sequence number of output vector
!     Interpret rest of record
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecin' )
      icodesav = icode
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecin'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode = 6
      code(1) = 'SCALAR'
      code(2) = 'VECTOR'
      code(3) = 'TENSOR'
      code(4) = 'ELEMENT_WISE'
      code(5) = 'FILE'
      code(6) = 'NUMBER'
      tabmin(1:5) = 0d0
      tabmin(6) = 1d0
      tabmax(1:5) = 0d0
      tabmax(6) = 1d2
      itype(1:4) = 2
      itype(5) = 6
      itype(6) = 0
      ncode(1:3) = 3
      ncode(4:6) = 4
      ncodmx(1:6) = 6
      ncodmx(4) = 12
      ncodmx(5) = 4
      nmain = 0

!     --- read sequence number of output vector

      iseq = 1
      call asknxt

      if ( debug ) write(irefwr,1) 'ityprd/1', ityprd

      if ( ityprd==4 ) then

!     --- ityprd = 4, string found, check if it is a vector name

         lastitemsav = lastitem
         lastitem = lastitem-1   ! reread item since it may contain digits
         ispeclsav = ispecl
         ispecl = 9
         call asknxt
         ispecl = ispeclsav
         if ( debug ) then
            write(irefwr,1) 'ityprd/2', ityprd
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )
         call readnamevec    ! name of vector
         if ( ityprd/=2 ) lastitem = lastitemsav

      end if  ! ( ityprd==4 )
      if ( debug ) write(irefwr,1) 'ityprd/3', ityprd

      if ( ityprd/=2 ) then

!     --- item read is not an integer

         call errchr ( 'input_vector', 1 )
         call errsub ( 574, 0, 0, 1 )
         go to 1000

      else

         iseq = nint(realrd)

!        --- integer found, check value

         if ( iseq<1 .or. iseq>maxvec ) then

!        --- integer out of range

            call errint ( iseq, 1 )
            call errint ( 1, 2 )
            call errint ( maxvec, 3 )
            call errchr ( code(1), 1 )
            call errsub ( 1881, 3, 0, 1 )
            call errcrd
         end if

      end if

      inpsep(1) = iseq
      inpsep(2) = 1
      ivectors(iseq) = 1
      numvec = max(numvec,iseq)

!     --- Interpret rest of record

      do k = 1, nncode
         table(k) = -1d0
      end do

      iretur = 0
      call sepreadsetkeyw

      iseq = 0
      do k = 1, 4
         if ( table(k)>0d0  ) then
            iseq= iseq+1
            ind(iseq) = k
         end if
      end do
      if ( iseq>1 ) call errkeywexcl ( code, ncodmx, ind(1), ind(2) )
      do k = 1, 3
         if ( table(k)>0d0 .and. table(6)>0d0 ) &
            call errkeywexcl ( code, ncodmx, k, 6 )
      end do
      if ( table(2)>0d0 ) then
         inpsep(2) = ndim
      else if ( table(3)>0d0 ) then
         if ( ndim==2 ) then
            inpsep(2) = 3
         else if ( ndim==3 ) then
            inpsep(2) = 6
         end if
      else if ( table(4)>0d0 ) then
         if ( table(6)>0d0 ) then
            inpsep(2) = -nint(table(6))
         else
            inpsep(2) = -1
         end if
      else if ( table(6)>0d0 ) then
         inpsep(2) = nint(table(6))
      end if
      if ( table(5)<=0d0 ) then

!     --- no file name found

         call errsub ( 1993, 0, 0, 0 )
         call errcrd

      else

!     --- file name found

         call intxstoretext ( inpsep(3), table(5), tabmax(5), &
                              strout )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askvecin'

      end if  ! ( debug )
      call erclos ( 'askvecin' )
      icode = icodesav

      end subroutine askvecin
