      subroutine askvecrhs ( inpsep, istart, table, number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret input following vi = right_hand_side
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: number
      integer, intent(out) :: inpsep(0:*)
      integer, intent(inout) :: istart
      double precision, intent(in) :: table(*)

!     inpsep         o    The integer input read is stored in array INPSEP of
!                         length 10
!                         Array INPSEP is filled as follows:
!                         0: Type of function to be called
!                            112:  right-hand side
!                         1: output vector
!                         2: problem number
!                         3: seq_coef
!     istart        i/o   Next free position in inpsep
!     number         i    Vector sequence number
!     table          i    Contains the numbers read for the various keywords
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecrhs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecrhs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(0) = 112               ! icode for right_hand_side
      inpsep(1) = number            ! output vector
      inpsep(2) = nint(table(18))   ! problem
      inpsep(3) = nint(table(32))   ! sequence number of coefficients
      istart = istart + 4

1000  call erclos ( 'askvecrhs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askvecrhs'

      end if  ! ( debug )

      end subroutine askvecrhs
