      function elfilllinear ( iuser, ipos, ishift, isubtract )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set elfilllinear equal to value following linearization
!     Raise ipos and ishift
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     scalar
!     string
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iuser(*), isubtract
      integer, intent(inout) :: ipos, ishift
      integer :: elfilllinear

!     elfilllinear   o    Value of parameter following linearization
!     ipos          i/o   Position in array iuser
!                         Is raised by 1
!     ishift        i/o   Shift of specific variable with respect to iuser
!                         Is raised by 1
!     isubtract      i    Value which must be subtracted from the sequence
!                         number in code to get the correct parameter
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode=2
      integer :: ncode(nncode)
      character (len=6) :: code (nncode)

!     code           Table with keywords
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     nncode         Number of entries in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer elgetintcoef

!     ELGETINTCOEF   Give elgetintcoef the actual integer value corresponding to
!                    iuser(ipos)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'PICARD', 'NEWTON' /
      data ncode / 6, 6 /
! ======================================================================
!
      call eropen ( 'elfilllinear' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfilllinear'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      elfilllinear = elgetintcoef ( iuser, ipos, ishift, code, nncode, &
                                    ncode, isubtract, 'linearization' )

1000  call erclos ( 'elfilllinear' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elfilllinear'

      end if  ! ( debug )

      end function elfilllinear
