      subroutine eradapbn
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-08-2016 use sepmoduleinput
!        version  2.8  date 22-11-2012 Long integers
!        version  2.7  date 26-08-2011 Simplification of subroutine
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the adaptation of the boundaries from the
!    standard input file
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     change_boundary
!     free_boundary
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparm, leng1, nrlparm
      integer, allocatable :: inpada(:), icurvs(:)
      double precision, allocatable :: rinada(:)
      parameter ( nparm=20, nrlparm=17 )

!     icurvs         array containing curve numbers
!     inpada         User input array for subroutine ADAPBOUN
!     leng1          Length of local co-ordinates array
!     nparm          Maximum length of inpada
!     nrlparm        Number of entries in rinada
!     rinada         User input array for subroutine ADAPBOUN
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into buffer array
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     READADBN       Read the input for the subroutine ADAPBOUN
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READADBN
!    The first record must have the following shape:
!    ADAPT_BOUNDARY [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eradapbn' )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays inpada

      allocate ( inpada(nparm+2*ncurvs), rinada(nrlparm), &
                 icurvs(2*ncurvs), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nparm+nrlparm+4*ncurvs, 'inpada' )
      if ( ierror/=0 ) go to 1000

      iseqadapbn = iseqadapbn + 1

      call readadbn ( inpada, rinada, icurvs, 2, iseqadapbn )
      if ( ierror/=0 ) go to 1000

      length = inpada(2)+inpada(15)/100+inpada(17)
      leng1 = 20+inpada(17)
      inpada(nparm+1:nparm+length) = icurvs(1:length)

!     --- Make input arrays permanent

      call ercopyinput ( 16, iseqadapbn, 'ADAPT_BOUNDARY', inpada, &
                         rinada, inpada, leng1+length, nrlparm, 0 )

      deallocate ( inpada, rinada, icurvs, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'inpada' )

1000  call erclos ( 'eradapbn' )
      end
