      subroutine ercapacity
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-08-2016 use sepmoduleinput
!        version  1.3  date 26-08-2011 Simplification of subroutine
!        version  1.2  date 09-07-2001 Include common cseqnr
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read the input for the subroutine to compute the capacity
!    The input is stored in the arrays iinput and rinput
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     capacity
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp, niinploc
      parameter ( niinp=8 )
      integer iinput(niinp)
      double precision :: rinput

!     iinput         integer input array
!     niinp          Length of integer input array
!     niinploc       Contains local value of niinp (may be changed)
!     rinput         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     EROPEN         Produces concatenated name of local subroutine
!     READCAPACITY   Read the input for the subroutine to compute the capacities
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READCAPACITY
!    The first record must have the following shape:
!    CAPACITY [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercapacity' )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays iinput and rinput

      iseqcapacity = iseqcapacity + 1

      niinploc = niinp
      call readcapacity ( iinput, 2, iseqcapacity, niinploc )

!     --- Make input arrays permanent

      call ercopyinput ( 24, iseqcapacity, 'CAPACITY', iinput, &
                         rinput, iinput, niinploc, 0, 0 )

1000  call erclos ( 'ercapacity' )
      end
