      subroutine erchcf ( iinput, rinput, niinp, nparm, nelgrp, compl, &
                          iend )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 17-08-2013 Debug statements
!        version  2.1  date 08-09-2000 Correction for inext=0
!        version  2.0  date 07-09-1998 Remove parameter nbngrp
!
!   copyright (c) 1991-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      This subroutine translates the contents of the arrays IUSER and USER
!      back into the arrays IINPUT and RINPUT
!      In this way subroutine READ02 can be used to update the coefficients
!      Help subroutine for subroutine CHNCOF
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, nelgrp, iinput(4,niinp), iend, &
              nparm(nelgrp)
      double precision rinput(niinp*2)
      logical compl

!    compl     i   Indication if the coefficients to be filled are real
!                  (false) or complex (true)
!    iend      i   Last position used in IUSER
!    iinput    o   Integer input array of length 4 x niinp that is
!                  filled as follows:
!                  iinput(1,i)>0:  Coefficient is function or integer
!                                    parameter
!                  iinput(1,i)<0:  Coefficient is equal to preceding coeff
!                              = 0:  The coefficient is a constant whose value
!                                    is stored in rinput(i)
!                  iinput(2,i):      iref, see FIL103
!                  iinput(3,i):      information of second coefficient, see
!                                    FIL103
!                  iinput(4,i):      information of third coefficient, see
!                                    FIL103
!                  Information must be stored in iinput sequentially:
!                  first element group one ,then two etc. and finally the
!                  boundary element groups
!    nelgrp    i   Number of element groups+boundary groups + global groups
!    niinp     i   Length parameter for iinput
!    nparm     o   Array of length nelgrp + nbngrp containing the number of
!                  parameters (coefficients) for each element group or
!                  boundary element group.
!                  Number of coefficients for element group ielgrp is
!                  equal to nparm(ielgrp), &
!                  Number of coefficients for boundary element group ibngrp is
!                  equal to nparm(ibngrp+nelgrp)
!    rinput    o   Real input array of length 2*niinp that must be filled with
!                  real information of the coefficients (sequentially)
!                  See iinput
!                  In the complex case each coefficient occupies two real
!                  positions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, icount, jparm , istart, ifunc, inext, ielgrp

!    i        General loop variable
!    icount   Counting variable, position in IUSER
!    ielgrp   Counting variable, element group number
!    ifunc    Counting variable, position in IINPUT, RINPUT
!    inext    Starting position of next element group in IUSER
!    istart   Counting variable, position in IINPUT, RINPUT
!    jparm    Counting variable, contains number of coefficients for element
!             group
!    k        General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear the arrays IINPUT, RINPUT and NPARM
!   Set counting parameters: ICOUNT:=6+nelgrp, ISTART:=0
!   Loop over element groups:
!   for ielgrp := 1 (1) NELGRP+NBNGRP
!      if ( ielgrp<nelgrp+nbngrp ) then
!         inext := iuser(ielgrp+6)
!      else ( ielgrp=nelgrp+nbngrp )
!         inext :=iend+1
!      jparm := 0
!      while ( icount<inext ) do
!         jparm := jparm+1
!         istart:= istart+1
!         ifunc := iuser(icount)
!         icount := icount+1
!         if ( ifunc<0 ) then
!            iinput(1,istart) := 0
!            rinput(istart)   := user(-ifunc)
!         else if ( ifunc=2001 or 2002 ) then
!            iinput(1,istart) := ifunc
!            icount := icount+1
!         else if ( ifunc=2003 ) then
!            iinput(1,istart) := ifunc
!            iinput(2,istart) := iuser(icount)
!            iinput(3,istart) := iuser(icount+1)
!            icount := icount+2
!         else if ( ifunc=2004 ) then
!            iinput(1,istart) := ifunc
!            iinput(2,istart) := iuser(icount)
!            iinput(3,istart) := iuser(icount+1)
!            icount := icount+2
!            ifunc := iuser(icount)
!            icount := icount+1
!            if ( ifunc<0 ) then
!               iinput(4,istart) := 0
!               rinput(istart)   := user(-ifunc)
!            else ( ifunc>=0 ) then
!               iinput(4,istart) := ifunc
!         else ( ifunc>=0 and ifunc<2001 or ifunc>2004) then
!            iinput(1,istart) := ifunc
!      nparm(ielgrp) := jparm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erchcf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erchcf'
         call prinin ( iuser, iuser(4), 'iuser' )
         if ( user(4)>0d0 ) call prinrl ( user, nint(user(4)), 'user' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear the arrays IINPUT, RINPUT and NPARM

      do i = 1, niinp
         do k = 1, 4
            iinput(k,i) = 0
         end do
         rinput(i) = 0d0
         rinput(i+niinp) = 0d0
      end do

      do i = 1, nelgrp
         nparm(i) = 0
      end do

!     --- Set counting parameters

      icount = 6+nelgrp
      istart = 0
      inext = 0

!     --- Loop over element groups:

      do ielgrp = 1, nelgrp

         if ( ielgrp<nelgrp ) then

!        --- ielgrp<nelgrp+nbngrp

            inext = iuser(ielgrp+6)
            if ( inext==0 ) then

!           --- inext = 0, search for larger value

               do i = ielgrp+1, nelgrp
                  inext = iuser(i+6)
                  if ( inext>0 ) go to 100
               end do

            end if

         else

!        --- ielgrp = nelgrp

            inext = iend+1

         end if

100      jparm = 0

!        --- while loop  icount<inext

150      if ( icount<inext .or. &
              icount>0 .and. icount<iend+1 .and. inext==0 ) then

!        --- icount<=inext

            jparm = jparm+1
            istart = istart+1
            ifunc = iuser(icount)
            icount = icount+1
            if ( ifunc<0 ) then

!           --- ifunc<0

               iinput(1,istart) = 0
               if ( compl ) then
                  rinput(2*istart-1) = user(-ifunc)
                  rinput(2*istart)   = user(-ifunc+1)
               else
                  rinput(istart) = user(-ifunc)
               end if

            else if ( ifunc==2001 .or. ifunc==2002 ) then

!           --- ifunc = 2001 or 2002

               iinput(1,istart) = ifunc
               icount = icount+1

            else if ( ifunc==2003 ) then

!           --- ifunc = 2003

               iinput(1,istart) = ifunc
               iinput(2,istart) = iuser(icount)
               iinput(3,istart) = iuser(icount+1)
               icount = icount+2

            else if ( ifunc==2004 ) then

!           --- ifunc = 2004

               iinput(1,istart) = ifunc
               iinput(2,istart) = iuser(icount)
               iinput(3,istart) = iuser(icount+1)
               icount = icount+2
               ifunc = iuser(icount)
               icount = icount+1
               if ( ifunc<0 ) then

!              --- ifunc<0

                  iinput(4,istart) = 0
                  if ( compl ) then
                     rinput(2*istart-1) = user(-ifunc)
                     rinput(2*istart)   = user(-ifunc+1)
                  else
                     rinput(istart) = user(-ifunc)
                  end if

               else

!              --- ifunc>0

                  iinput(4,istart) = ifunc

               end if

            else

!           --- ifunc>0 and ifunc<2001 or ifunc>2004

               iinput(1,istart) = ifunc

            end if

            go to 150

         end if

         nparm(ielgrp) = jparm

      end do

1000  call erclos ( 'erchcf' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( iinput, istart, 4, 'iinput' )
         call prinin ( nparm, ielgrp, 'nparm' )
         call prinrl ( rinput, istart, 'rinput' )
         write(irefwr,*) 'End erchcf'

      end if  ! ( debug )
      end
