      subroutine erchcoor
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-08-2016 use sepmoduleinput
!        version  1.7  date 28-03-2013 set nchancoor
!        version  1.6  date 09-03-2013 Correction length code
!
!   copyright (c) 1998-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning change_coordinates from the standard input file
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     change
!     change_mesh
!     coordinate
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleinput
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nchancoor, isubr
      integer :: chacor(0:40)

!     chacor         integer work array of length 40 to store the information
!                     concerning change_coordinates
!                     For each data record 4 positions are required, at most 10
!                     different cases are allowed
!                     Contents:
!                     (1,i):  ifunc_chan
!                     (2,i):  itype
!                     (3,i):  ifirst
!                     (4,i):  ilast
!     i              Loop variable
!     isubr          Indicates the sequence number of the subroutine
!     nchancoor      number of data records for change coordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     EROPEN         Produces concatenated name of local subroutine
!     MSHR04         Interpret information following the keyword
!                    change_coordinates
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READ02
!    The first record must have the following shape:
!    CHANGE_COORDINATES, [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Interpret first record
!   Check IPROB and ISEQ
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erchcoor' )

!     --- Read rest of record

      isubr = 22
      nncode = 1
      code(1) = 'SEQUENCE'
      codemain(1) = 'END'
      ncodemain(1) = 3
      ncode(1) = 3
      ncodmx(1) =15
      itype(1) = 0
      table(1) = iseqchcoor  + 1
      tabmin(1) = 1d0
      tabmax(1) = iinputarrs(isubr)%maxsub
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      iseqchcoor  = nint(table(1))
      if ( ierror/=0 ) go to 1000

      nmain = 1
      nchancoor = 0
      call mshr04 ( chacor(1), nchancoor, nelem )

!     --- Make input arrays permanent

      chacor(0) = nchancoor
      call ercopyinput ( isubr, iseqchcoor, 'CHANGE_COORDINATES', chacor, &
                         table, chacor(0), 1+nchancoor*4, 0, 0 )

1000  call erclos ( 'erchcoor' )

      end
