      subroutine ercret ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 26-08-2016 use sepmoduleinput
!        version  4.5  date 24-12-2014 New definition clinsl
!        version  4.4  date 05-13-2013 Extension with change_vector
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call subroutine CREATN to create vectors
!     Fill array IINCRT with the values stored in INPSEP
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleinput
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         0             Type of routine
!                                         4: create
!                                       108: change
!                         1  ISEQNR     Sequence number corresponding to input
!                                       If iseqnr = -1, the next one will be
!                                       used
!                         2  IPROB      Problem sequence number
!                                       Not used
!                         3  IVECTOR    Vector sequence number (first vector)
!                                       If ivector = -1, the same value as iprob
!                                        will be used
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, ivec, iseqnr, ivect1, icall, &
              maxcall, istrin, nreal, numint, ipos, i, isubr
      integer, pointer :: iinput(:), iinputinf(:)
      double precision, pointer :: rinput(:)

!                    they are not
!     i              Counting variable
!     icall          Call sequence number for present subroutine
!     iinput         integer input array (temporary fake array)
!     iinputinf      integer input array referring to scalars
!                    (temporary fake array)
!     ipos           Position of input arrays in sepmoduleinput
!     iseq           Sequence number read
!     iseqnr         Sequence number with respect to the storage in
!                    sepmoduleinput
!     istrin         Starting address of information for this subroutine
!                    in RINSEP
!     isubr          Indicates the type of calling subroutine for er0034
!                    Possible values
!                    1:  CREATE
!                    2:  PRESDF/TM
!                    3:  CREATEN
!                    4:  presbc
!     ivec           Sequence number of vector of special structure
!     ivect1         Sequence number of first vector
!     maxcall        Maximum value of icall in sepmoduleinp
!     nreal          Number of reals stored
!     numint         Number of integers stored
!     rinput         Real input array (temporary fake array)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ER0034         Performs the actual tasks for CREAT and PRESDF
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIVECTR       Return with sequence number of vector
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1556   Sequence number exceeds maximum
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ivect1 /0/
! ======================================================================
!
      call eropen ( 'ercret' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercret'
         write(irefwr,1) 'istartmn', istartmn
  1      format ( a, 1x, (10i6) )
         call prinin ( inpsep(0), 10, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqnr = 1
      iseq   = inpsep(1)
      if ( iseq>0 ) iseqnr = iseq

      ivect1 = min ( ivect1 + 1, maxvc1 )
      ivec = inivectr(inpsep(3))
      if ( ivec/=-1 ) ivect1 = ivec
      if ( debug ) &
         write(irefwr,1) 'iseq, iseqnr, ivect1, ivec', &
                          iseq, iseqnr, ivect1, ivec

      if ( iseq/=-10 ) then

!     --- Standard case, use sequence number

         isubr  = 5+100*(ivect1-1)
         call er0034 ( isubr, 0, isolut2d, iseqnr, isolut2d, 100 )
         numvecmn = max(numvecmn,mod(isubr,10000))
         istartmn = istartmn + 4

      else

!     --- New case, all input in inpsep and rinsep

         numint = inpsep(5)
         nreal = inpsep(6)
         istrin = inpsep(7)
         if ( debug ) then
            write(irefwr,1) 'numint, nreal, istrin', &
                             numint, nreal, istrin
            call prinin ( inpsep(1), 7+numint+nreal, 'inpsep' )
            if ( nreal>0 ) &
               call prinrl ( rinsep(istrin), nreal, 'rinsep' )
         end if  ! ( debug )

         icall = iinputarrs(4)%maxcall
         call sepgetinpfinf ( 4, icall, ipos, maxcall )
         if ( in(ipos)%used ) then

!        --- Input already in use, raise icall

            if ( icall<iinputarrs(4)%maxsub ) then

               icall = icall+1
               ipos = ipos+1

            else

!           --- No more space for storage

               call errint ( icall+1, 1 )
               call errint ( iinputarrs(4)%maxsub, 2 )
               call errsub ( 1556, 2, 0, 0 )
               go to 1000

            end if  ! ( icall<iinputarrs(4)%maxsub )

         end if  ! ( in(ipos)%used )

         in(ipos)%used = .true.  ! mark as used

         allocate ( in(ipos)%iinput(numint+7), stat=error )
         if ( error/=0 ) call eralloc ( error, numint+7, 'iinput' )
         if ( ierror/=0 ) go to 1000

         iinput => in(ipos)%iinput

         if ( nreal>0 ) then

!        --- There are real arrays

            allocate ( re(ipos)%rinput(nreal), inf(ipos)%iinputf(nreal), &
                       stat=error )
            if ( error/=0 ) call eralloc ( error, 2*nreal, 'rinput' )
            if ( ierror/=0 ) go to 1000

            rinput => re(ipos)%rinput
            iinputinf => inf(ipos)%iinputf

         end if  ! ( nreal>0 )
         if ( debug ) write(irefwr,1) 'icall, maxcall', icall, maxcall

!        --- Fill input array with first part and copy rest

         iinput(1) = -1          ! -1: create;  -2 prescribe_bc
         iinput(2) = 1           ! number of vectors
         if ( inpsep(4)==0 ) then
            iinput(3) = 1           ! solution vector
         else
            iinput(3) = 2           ! vector of special structure
         end if  ! ( ivec==0 )
         iinput(4) = inpsep(2)   ! problem number
         iinput(5) = inpsep(4)   ! ivec
         iinput(6) = 0           ! icompl
         iinput(7) = 1           ! number of cases

         iinput(8:7+numint) = inpsep(8:7+numint)

         if ( debug ) call prinin ( iinput, 7+numint, 'iinput' )

         if ( nreal>0 ) then

!        --- There is real input

            rinput(1:nreal) = rinsep(istrin:istrin+nreal-1)
            iinputinf(1:nreal) = inpsep(8+numint:7+numint+nreal)
            do i = 1, nreal
               if ( iinputinf(i)>0 ) rinput(i) = scalars(iinputinf(i))
            end do  ! i = 1, nreal

         end if  ! ( nreal>0 )

!        --- Fill vector

         isubr = 8       ! indicates call from structure
         if ( inpsep(0)==108 ) isubr = 9  ! change_vector
         isubr  = isubr+100*(ivect1-1)
         iseqnr = icall

         call er0034 ( isubr, 0, isolut2d, iseqnr, isolut2d, 100 )

         numvecmn = max(numvecmn,mod(inpsep(3),5000))
         istartmn = istartmn + 8 + numint+nreal

!        --- Remove input from sepmoduleinput

         in(ipos)%used = .false.
         deallocate ( in(ipos)%iinput )
         if ( nreal>0 ) deallocate ( re(ipos)%rinput, inf(ipos)%iinputf )

      end if  ! ( iseq/=-10 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartmn, numvecmn', istartmn, numvecmn
         write(irefwr,*) 'End ercret'

      end if  ! ( debug )
      call erclos ( 'ercret' )

      end
