      subroutine erfillcf703
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for elements of type 703 (Convection-diffusion)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulekmesh
      use sepmodulestrings
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCFCONV   Fill array iincof/rincof for convection part of elements
!     ERFILLCFDIF    Fill array iincof/rincof for diffusion part of elements
!     ERFILLCFRULE   Fill array iincof/rincof part integration rule and
!                    coordinate system
!     ERFILLCFSRHOCP Fill array iincof/rincof for source and rho cp part
!                    of elements
!     ERFILLDEFCOF4  Check if keyword is found in list of constants or strings
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!
!     The default input is:
!     icoef1 = typediff
!     icoef2 = supg
!     icoef3 = integration_rule
!     icoef4 = coordinate_system
!     icoef5 = type_convection
!     coeff6-11: diffusion input
!     coeff12-14: convection input
!     coef15 = source
!     coef16 = density
!     coef17 = heat_capacity
!     icoef20= applyupw
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcf703' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcf703'
         write(irefwr,1) 'iposin, iposin, ndim', iposin, iposin, ndim
         call prinin ( imark, 20, 'imark/start' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Coefficient 6-11 (Diffusion)

      if ( imark(6)==0 ) then

!     --- Diffusion has not been filled yet

         call erfillcfdif

      end if  ! ( iposin==iposinsav )
      if ( debug ) write(irefwr,1) 'iposin, iposin/1', iposin, iposin

!     --- Next velocity
!         Coefficient 12-14 (velocity)

      call erfillcfconv ( 12 )

!     --- Coefficient 15-17 (source, density and heat_capacity)

      call erfillcfsrhocp ( 15 )

      icoef = 2     ! Upwind parameter

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof4 ( codegen(4)(1:ncodegen(4)) )

      end if  ! ( imark(icoef)==0 )

      icoef = 5     ! type_convection

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof4 ( codegen(6)(1:ncodegen(6)) )

      end if  ! ( imark(icoef)==0 )

      icoef = 20    ! apply_upwind

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof4 (codegen(9)(1:ncodegen(9)) )

      end if  ! ( imark(icoef)==0 )

!     --- integration rule and coordinate system

      call erfillcfrule

      iincof(iposin+1) = -2        ! end of information

1000  call erclos ( 'erfillcf703' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin, iposin', iposin, iposin
         call prinin ( imark, 20, 'imark/end' )
         write(irefwr,*) 'End erfillcf703'

      end if  ! ( debug )

      end subroutine erfillcf703
