      subroutine erfillcfdif
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 16-02-2015 Extra debug
!        version  1.0  date 17-12-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for diffusion part of elements of type 800
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
!     diffusion
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmodulecoeffs
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iposinsav, typediff

!     i              Counting variable
!     iposinsav      Parameter to save old value of ipos
!     typediff       Indicates the type of diffusion
!                    1:  diffusion is a scalar
!                    2:  diffusion is a diagonal tensor
!                    3:  diffusion is a symmetric tensor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcfdif' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcfdif'
         write(irefwr,1) 'iposin, ndim', iposin, ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icoef = 6
      iposinsav = iposin
      call erfilldefcof1 ( 'diffusion' )

      if ( debug ) write(irefwr,1) 'iposin', iposin

      if ( iposin>iposinsav ) then

!     --- Diffusion found
!         Set other coefficients corresponding to Diffusion

         do i = 2, ndim
            icoef = 9+(i-2)*2
            iincof(iposin+2) = icoef  ! Sequence number coefficient
            iincof(iposin+3) = 6      ! refers to coef 6
            iincof(iposin+1) = 9      ! coefficient is the same as coef 6
            iposin = iposin + 3
         end do  ! i = 2, ndim
         typediff = 1

      else

!     --- Not diffusion found, check for diff_x, diff_y ...

         call erfilldefcof1 ( codediffus(2) )
         if ( ndim>1 ) then

            if ( imark(9)==0 ) then

!           --- Coefficient icoef has not been filled

               icoef = 9
               call erfilldefcof1 ( codediffus(3) )

            end if  ! ( imark(9)==0 )

         end if  ! ( ndim>1 )
         if ( ndim>2 ) then

            if ( imark(11)==0 ) then

!           --- Coefficient icoef has not been filled

               icoef = 11
               call erfilldefcof1 ( codediffus(4) )

            end if  ! ( imark(11)==0 )

         end if  ! ( ndim>2 )
         typediff = 2

      end if  ! ( iposin>iposinsav )

      if ( iposin==iposinsav ) then

!     --- Not diffusion or diff_x ... found, check for diff_xx ..

         icoef = 6
         call erfilldefcof1 ( codediffus(5) )
         if ( ndim>1 ) then

            if ( imark(7)==0 ) then

!           --- Coefficient icoef has not been filled

               icoef = 7
               call erfilldefcof1 ( codediffus(6) )

            end if  ! ( imark(7)==0 )

            if ( imark(9)==0 ) then

!           --- Coefficient icoef has not been filled

               icoef = 9
               call erfilldefcof1 ( codediffus(8) )

            end if  ! ( imark(9)==0 )

         end if  ! ( ndim>1 )

         if ( ndim>2 ) then

            if ( imark(8)==0 ) then

!           --- Coefficient icoef has not been filled

               icoef = 8
               call erfilldefcof1 ( codediffus(7) )

            end if  ! ( imark(8)==0 )

            if ( imark(10)==0 ) then

!           --- Coefficient icoef has not been filled

               icoef = 10
               call erfilldefcof1 ( codediffus(9) )

            end if  ! ( imark(10)==0 )

            if ( imark(11)==0 ) then

!           --- Coefficient icoef has not been filled

               icoef = 11
               call erfilldefcof1 ( codediffus(10) )

            end if  ! ( imark(11)==0 )

         end if  ! ( ndim>2 )
         typediff = 3

      end if  ! ( iposin==iposinsav )

!     --- Store typediff in icoef1

      iincof(iposin+1) = 1        ! Type input = integer
      iincof(iposin+2) = 1        ! Sequence number coefficient
      iincof(iposin+3) = typediff
      iposin = iposin+3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillcfdif'

      end if  ! ( debug )
      call erclos ( 'erfillcfdif' )

      end subroutine erfillcfdif
