      subroutine erfillcfgamma ( icoefgamma )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for gamma part of elements of type 800/7xx
!     3 consecutive positions are filled
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulekmesh
      use sepmodulestrings
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: icoefgamma

!     icoefgamma     i    Coefficient sequence number for gamma
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF2  Check if keyword is found in list of vectors
!                    If so fill value in iincof/rincof
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcfgamma' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcfgamma'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( imark(icoefgamma)==0 ) then

!     --- Coefficient icoef has not been filled

         if ( ndim==1 ) then

!        --- ndim = 1, only one component

            icoef = icoefgamma
            call erfilldefcof1 ( 'gamma' )

         else

!        --- ndim>1, more components

            icoef = icoefgamma
            call erfilldefcof2 ( 'gamma' )

         end if  ! ( ndim==1 )
         if ( debug ) write(irefwr,1) 'iposin, iposin/2', iposin, iposin

      end if  ! ( imark(12)==0 )

!     --- Coefficient icoefgamma (x_gamma)

      icoef = icoefgamma
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( 'x_gamma' )
         if ( debug ) write(irefwr,1) 'iposin, iposin/3', iposin, iposin

      end if  ! ( imark(icoef)==0 )

      if ( ndim>1 ) then

!     --- Coefficient 19 (y_gamma)

         icoef = icoefgamma+1
         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( 'y_gamma' )

         end if  ! ( imark(icoef)==0 )

      end if  ! ( ndim>1 )
      if ( debug ) write(irefwr,1) 'iposin, iposin/4', iposin, iposin

      if ( ndim>2 ) then

!     --- Coefficient 20 (z_gamma)

         icoef = icoefgamma+2
         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( 'z_gamma' )

         end if  ! ( imark(icoef)==0 )

      end if  ! ( ndim>2 )

1000  call erclos ( 'erfillcfgamma' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillcfgamma'

      end if  ! ( debug )

      end subroutine erfillcfgamma
