      subroutine erfillvc ( vector, nodes, numnodes, uoldcp )
! ======================================================================
!
!        programmer    Guus Segal
!        version 2.1   date   12-07-1998 Correction inusol
!        version 2.0   date   20-02-1998 Remove kprobh
!        version 1.0   date   01-11-1997
!
!
!
!   copyright (c) 1997-1998 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the contents of array vector into array uoldcp for all points
!     in array nodes and all degrees of freedom
!     Real case
! **********************************************************************
!
!                       KEYWORDS
!
!   create
!   real
!   vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, nodes(numnodes)
      double precision vector(*), uoldcp(numnodes,*)

!    nodes     i   Contains the node numbers that must be filled
!    numnodes  i   Number of nodes stored in array NODES
!    uoldcp    o   Contains the copied solution
!    vector    i   Contains the solution to be copied
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node, j, inusol, inunkp
!    i         General loop variable
!    inunkp    Number of unknowns in a point
!    inusol    Positions of degree of freedom in array IVECTR
!    j         General loop variable
!    node      Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       PSEUDO CODE
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Distinguish between the various cases

      if ( indprpi.gt.0 ) then

!     --- indprpi>0, degrees of freedom are stored in kprobpi
!          Loop over nodes

         do j = 1, nphysi

            do i = 1, numnodes
               node   = nodes(i)
               inusol = kprobpi(node,j)
               if ( inusol.gt.0 ) uoldcp(i,j) = vector(inusol)

            end do

         end do

      else if ( indprfi.gt.0 ) then

!     --- indprfi>0, Non-constant number of degrees of freedom per point

         do i = 1, numnodes

            node   = nodes(i)
            inusol = kprobfi(node)
            inunkp = kprobfi(node+1) - inusol
            do j = 1, inunkp
               uoldcp(i,j) = vector(inusol+j)
            end do

         end do

      else

!     --- Constant number of degrees of freedom per point

         do i = 1, numnodes
            node   = nodes(i)
            do j = 1, nphysi
               uoldcp(i,j) = vector((node-1) * nphysi + j)
            end do

         end do

      end if

      end
