      subroutine erpresscor
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-08-2016 use sepmoduleinput
!        version  1.2  date 26-08-2011 Simplification of subroutine
!        version  1.1  date 02-12-2005 Use erreadcopy
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read the input for pressure correction
!    The input is stored in the arrays iinput and rinput
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     pressure_correction
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp, iseqpresscor, nrinp, niinploc, nrinploc
      parameter ( niinp=12, nrinp=1 )
      integer iinput(niinp)
      double precision rinput(nrinp)

!     iinput         integer input array
!     iseqpresscor   Sequence number of input for pressure correction
!     niinp          Length of integer input array
!     niinploc       Contains local value of niinp (may be changed)
!     nrinp          Length of real input array
!     nrinploc       Contains local value of nrinp (may be changed)
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into buffer array
!     EROPEN         Produces concatenated name of local subroutine
!     READPRESSCOR   Read the input for pressure correction
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READPRESSCOR
!    The first record must have the following shape:
!    PRESSURE_CORRECTION [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     reserve space for input arrays
!     Read rest of input until END has been found
!     Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iseqpresscor /0/
! ======================================================================
!
      call eropen ( 'erpresscor' )

!     --- Read input and store in arrays iinput and rinput

      iseqpresscor = iseqpresscor + 1

      niinploc = niinp
      nrinploc = nrinp
      call readpresscor ( iinput, rinput, 2, &
                          iseqpresscor, niinploc, nrinploc )

!     --- Make input arrays permanent

      call ercopyinput ( 29, iseqpresscor, 'pressure correction', iinput, &
                         rinput, iinput, niinploc, nrinploc, 0 )

      call erclos ( 'erpresscor' )
      end
