      subroutine ersolv
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-08-2016 use sepmoduleinput
!        version  2.8  date 30-07-2014 Use common comcons1
!        version  2.7  date 22-11-2012 Long integers
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the linear solver from the standard input file
!    Store the result in sepmoduleinput
!    Remark: if ersolv is adapted, also adapt ersolvin !!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleinput
      use sepmodulecons
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iproblc, ninsol, nrisol
      integer, allocatable :: iinsol(:)
      double precision, allocatable :: rinsol(:)
      save iproblc

!                    they are not
!     iinsol         Contains information concerning the linear solver
!                    See the Programmers Guide for a description
!     iproblc        Actual problem number
!     ninsol         Length of array INPSOL
!     nrisol         Length of array RINSOL
!     rinsol         Contains information concerning the linear solver
!                    See the Programmers Guide for a description
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     READ03         Read card input for subroutine TO0089
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READ03
!
!    The first record must have the following shape:
!
!    SOLVE [,PROBLEM = j], [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iproblc /0/
! ======================================================================
!
      call eropen ( 'ersolv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ersolv'
         write(irefwr,1) 'iseqsolv', iseqsolv
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays iinsol and rinsol
!         reserve space in buffer array for the arrays IINSOL and RINSOL
!         In order to prevent overwriting of these arrays by READ03
!         they are created at the end of IBUFFR

      ninsol = ninpsol
      nrisol = nrinsol

      allocate ( iinsol(ninsol), rinsol(nrisol), stat = error )
      if ( error/=0 ) call eralloc ( error, ninsol+nrisol, 'iinsol' )
      if ( ierror/=0 ) go to 1000

      iseqsolv = iseqsolv + 1

      if ( debug ) write(irefwr,1) 'iseqsolv/1', iseqsolv

      iinsol(1) = ninsol
      call read03 ( iinsol, rinsol, 2, iseqsolv, &
                    iproblc, nprob, ninsol, nrisol )
      if ( debug ) &
         write(irefwr,1) 'iseqsolv/2, ninsol, nrisol', iseqsolv, ninsol, nrisol

!     --- Make input arrays permanent

      call ercopyinput ( 8, iseqsolv, 'iinsol', iinsol, &
                         rinsol, iinsol, ninsol, nrisol, 0 )

      deallocate ( iinsol, rinsol, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinsol' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ersolv'

      end if  ! ( debug )
      call erclos ( 'ersolv' )

      end subroutine ersolv
