      subroutine erstatfr
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-08-2016 use sepmoduleinput
!        version  2.5  date 26-08-2011 Simplification of subroutine
!        version  2.4  date 09-07-2001 Include common cseqnr
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read the input for the stationary free boundary subroutine statfree
!    The input is stored in the arrays iinput and rinput
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_mesh
!     change_mesh
!     free_boundary
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp, nrinp, niinploc, nrinploc
      parameter ( niinp=20, nrinp=1 )
      integer iinput(niinp)
      double precision rinput(nrinp)

!     iinput         integer input array
!     niinp          Length of integer input array
!     niinploc       Contains local value of niinp (may be changed)
!     nrinp          Length of real input array
!     nrinploc       Contains local value of nrinp (may be changed)
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into buffer array
!     EROPEN         Produces concatenated name of local subroutine
!     READSTFR       Read the input for the subroutine STATFREE
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READADMS
!    The first record must have the following shape:
!    STATIONARY_FREE_BOUNDARY [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erstatfr' )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays iinput and rinput

      iseqstatfr = iseqstatfr + 1

      niinploc = niinp
      nrinploc = nrinp
      call readstfr ( iinput, rinput, 2, iseqstatfr, &
                      niinploc, nrinploc )

!     --- Make input arrays permanent

      call ercopyinput ( 18, iseqstatfr, 'STATIONARY_FREE_BOUNDARY', &
                         iinput, rinput, iinput, niinploc, nrinploc, 0 )

1000  call erclos ( 'erstatfr' )
      end
