      function getnamereal ( realname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 10-04-2013 Correction indexed reals
!        version  1.3  date 10-04-2013 Loop of indexed reals
!        version  1.2  date 20-04-2012 Indexed reals
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Get the sequence number corresponding to the real in the block constants
!    with name realname
!    If the name does not exist getnamereal is not changed
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleread
      use sepmoduletexts
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) realname
      integer getnamereal

!     getnamereal   o   Sequence number of real with respect to array
!                       chrlcons
!     realname      i   Name of real
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, nch1
      character (len=80) rname1, rname2

!     i              General loop variable
!     nch            Number of characters in a string
!     nch1           Number of characters in a string
!     rname1         Name of real translated to capitals
!     rname2         Name of real translated to capitals
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) numreals do
!        if ( intname = chrlcons(i) ) then
!           getnamereal := i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getnamereal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getnamereal'
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      getnamereal = 0
      nch1 = index ( realname, ' ' ) -1
      if ( nch1==-1 ) nch1 = len(realname)
      rname1 = realname(1:nch1)
      call askupc(rname1)
      do i = 1, numreals
         nch = ncharcons(i+maxints)
         if ( nch1==nch ) then
            rname2 = chrlcons(i)(1:nch1)
            call askupc(rname2)
            if ( rname1(1:nch1)==rname2(1:nch1) ) then

!           --- name of real found

               getnamereal = i

               if ( dimcons(maxints+i)>0 ) then

!              --- Constant is an array

                  if ( iseqarray>0 ) then

!                 --- constants array

                     getnamereal = i+iseqarray-1

                  else if ( iseqarray<0 ) then

!                 --- constants array, but no sequence number given
!                     in this case an array is permitted because iseqarray<0

                     getnamereal = i+5000

                  else if ( nitems>lastitem+2 .and. &
                            typeitems(lastitem+1)==5 .and. &  ! (
                            typeitems(lastitem+2)==1 .and. &  ! integer
                            typeitems(lastitem+3)==6 ) then ! )

!                 --- indexed array found, read number

                     getnamereal = i+nint(realitems(lastitem+2))-1
                     lastitem = lastitem+3

                  else

!                 --- constants array

!                     getnamereal = i+5000

                  end if  ! ( iseqarray>0 )

               end if  ! ( dimcons(maxints+i)>1 )
               exit

            end if
         end if
      end do

1000  call erclos ( 'getnamereal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getnamereal'

      end if  ! ( debug )

      end
