      subroutine incv01 ( ibinas )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 30-01-2016 New call to msh059
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.0  date 06-10-2006 Extra parameters
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    program incv01 is an interactive program to make a conversion
!    of the SEPRAN meshoutput file from ASCII to binary or vice-versa
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleread
      use sepmodulemach
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibinas

!     ibinas         i    Indicates if a file must be translated from binary to
!                         ASCII or vice-versa
!                         Possible values:
!                         0:  Translate from binary to ASCII
!                         1:  Translate from ASCII to binary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irefsv, ihelp(10)
      character (len=100) namesv

!     ihelp      Help array to store information read by the mesh
!                This information is written by msh059
!     irefsv     Help variable to save a reference number
!     namesv     Save of previous name
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERTEXT         Produce menu
!     INIFIL         Open files
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
!     SEPMSH059      Write information of the mesh to the file indicated by iref
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incv01' )

!     --- Save old name of meshoutput and ask for actual name

      namesv = name10
      irefsv = iref10
      call errchr ( name10, 1 )
      call ertext ( 14+ibinas, 0, 0, 1 )

!     --- Read answer

      ityprd = 1
      ispecl = 3
      call asknxt
      if ( ityprd==5 ) name10 = string(1:nchar)
      ispecl = 0

!     --- Open old file

      if ( ibinas==0 ) then

!     --- ibinas = 0, binary file

         iref10 = -abs(iref10)

      else

!     --- ibinas = 1, ASCII file

         iref10 = abs(iref10)

      end if
      ierror = -1
      call inifil(-10)
      if ( ierror>0 ) then
         iref10 = irefsv
         name10 = namesv
         call ertext ( 1, 0, 0, 0 )
         read(*,*)
         go to 1000
      end if

!     --- Read file and close file

      call ertext ( 16, 0, 0, 0 )
      call msh071 ( -1, iref10, ihelp )
      iref10 = abs(iref10)
      close ( iref10 )
      if ( ierror>0 ) then
         iref10 = irefsv
         name10 = namesv
         call ertext ( 1, 0, 0, 0 )
         read(*,*)
         go to 1000
      end if

!     --- ask for name of new file

      call errchr ( name10, 1 )
      call ertext ( 15-ibinas, 0, 0, 1 )

!     --- Read answer

      ityprd = 1
      ispecl = 3
      call asknxt
      if ( ityprd==5 ) name10 = string(1:nchar)
      ispecl = 0

!     --- Open new file

      iref10 = abs(iref10)+1

      if ( ibinas==0 ) then

!     --- ibinas = 0, binary file

         iref10 = abs(iref10)

      else

!     --- ibinas = 1, ASCII file

         iref10 = -abs(iref10)

      end if
      ierror = -1
      call inifil(10)
      if ( ierror>0 ) then
         iref10 = irefsv
         name10 = namesv
         call ertext ( 1, 0, 0, 0 )
         read(*,*)
         go to 1000
      end if

!     --- Write new file, close file

      call ertext ( 17, 0, 0, 0 )
      call sepmsh059 ( iref10, ihelp )
      iref10 = abs(iref10)
      close ( iref10 )
      iref10 = irefsv

1000  call erclos ( 'incv01' )
      end subroutine incv01
