      subroutine plsepd
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 25-11-2012 Long integers
!        version  2.3  date 31-01-2005 Remove file 17
!        version  2.2  date 06-02-2004 Remove superfluous computers
!
!   copyright (c) 1991-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Interactive subroutine to plot the SEPRAN neutral plot files
!   It is called by program sepdisplay, and actually replaces the
!   complete body of sepdisplay
! **********************************************************************
!
!                       KEYWORDS
!
!     interactive
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleread
      use sepmodulemach
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ready, check, readsb, stop
      integer iseqpl, iseqps, numfil, numfsv, iseq, i, ios
      character (len=2) possav
      character (len=60) namefl
      integer lastps, ifirst

!    check     Logical indicating if file exists (true) or not (false)
!    i         General loop variable
!    ifirst    Indicates if the first plot must be made (0) or not (1)
!    ios       IOSTAT variable
!    iseq      Sequence number of plot
!    iseqpl    Sequence number of HPGL plot
!    iseqps    Sequence number of Postscript plot
!    lastps    Last non-blank position in namefl (without the extension)
!    namefl    Name of default SEPRAN neutral plotting file
!    numfil    Number of plot files found
!    numfsv    Help variable to store the preceding value of numfil temporarily
!    possav    Help variable to store the preceding value of posdev temporarily
!    readsb    Indicates if the sub loop has been finished (true) or not (false)
!    ready     Indicates if the loop has been finished (true) or not (false)
!    stop      Indicates if a stop has been found in the sub menu (true) or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERTEXT         Produce menu
!     FINISH         Stop SEPRAN
!     INDFDV         Define plotting device
!     INICLS         Clear screen
!     INIFIL         Open files
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITDF         Set defaults for program SEPPOST
!     INITMD         Set machine-dependent quantities
!     PLAF           Plot file
!     PLMN00         Produce sub menu plotting of separate plot files or zooming
!     SEPSTART       Standard SEPRAN starting subroutine
!     STARTSEPRAN    Create buffer array and fill part of sepmodulebuf
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   start SEPRAN
!   ready := false
!   namefl := namepl
!   numfil := Number of files namefl.***
!   seq_num_ps := 0
!   seq_num_pl := 0
!   if ( numfil=0 ) then
!      print warning
!   While (not ready ) do
!      print main menu
!      read answer
!      if ( answer = "none" ) then
!         Start procedure of plotting the files
!         Parameters: namefl, numfil, seq_num_ps, seq_num_pl, stop
!         If ( stop ) then
!            ready := true
!      else if ( answer = a ) then
!         make a display plot of all numfil files namefl
!      else if ( answer = c ) then
!         make a hard copy of all numfil files namefl
!      else if ( answer = h ) then
!         print the help menu
!      else if ( answer = n ) then
!         Ask for new name of SEPRAN neutral file
!         namefl := new_name
!         numfil := Number of files namefl.***
!         if ( numfil=0 ) then
!            print error message
!            reset namefl
!            reset numfil
!      else if ( answer = p ) then
!         print and carry out menu define plotting devices
!      else if ( answer = q ) then
!         ready := true
!      Close SEPRAN
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ifirst /0/
! ======================================================================
!
!     --- Start SEPRAN

      if ( startcalled/=99991 ) call startsepran
      call initcb
      call initmd

!     --- Open read and write files

      irefre = 5
      irefwr = 6

!     --- Open standard message file and error message file

      call inifil(4)

!     --- Standard SEPRAN start subroutine

      call sepstart ( 10100, 1, 0, 0 )
      ioutp = -1
      call eropen ( 'sepdisplay:plsepd' )

!     --- Set defaults

      call initdf
      posdev(1:2) = devdis(1:2)
      iplotf = 0
      iseqpl = 0
      iseqps = 0
      istop = 3

!     --- Compute number of present SEPRAN neutral plotting files

      lastps = index ( namepl, ' ' ) - 1
      ready = .false.

!     --- While loop ready

      numfil = 0
      namefl = ' '
      namefl(1:lastps+4) = namepl(1:lastps)//'.001'

100   if ( .not. ready ) then

         numfil = numfil + 1
         write(namefl(lastps+2:lastps+4), 110 ) numfil
110      format ( i3.3 )
         inquire ( file=namefl(1:lastps+4), exist=check )
         if ( .not. check ) then

!        --- file does not exist, decrease numfil

            numfil = numfil - 1
            ready = .true.

         end if
         go to 100

      end if
      namefl(1:lastps+4) = namepl(1:lastps)//'.001'

      if ( numfil==0 ) then

!     --- Warning: no file namefl found

         call inicls
         call errchr ( namefl, 1 )
         call ertext ( 2, 0, 0, 1 )
         read(*,*)

      end if

!     --- While loop ready

      ready = .false.
200   if ( .not. ready ) then

!     --- Write main menu

         call inicls
         do 210 i = 1 ,3
            call errchr ( namefl(1:lastps), i )
            call errint ( numfil, i )
210      continue
         call errchr ( namefl(1:lastps), 4 )
         call ertext ( 23, 3, 0, 4 )

!        --- Read answer

         ityprd = 1
         ispecl = 1
         call asknxt

         if ( ispecl==2 ) then

!        --- Blank record found hence RETURN
!            Start plotting of series of pictures

            if ( numfil==0 ) then

!           --- No files available, return to main menu

               go to 200

            else

               if ( ifirst==0 ) then

!              --- ifirst = 0, first plot to be made

                   ifirst = 1

               end if
               call plmn00 ( numfil, iseqps, iseqpl, stop )
               if ( stop ) ready = .true.
               go to 200

            end if

         else if ( ityprd/=4 ) then

!        --- Not a text found, return to main menu

            go to 200

         end if

!        --- Character found, detect

         if ( string(1:1)=='A' ) then

!        --- Plot all files

            do 220 iseq = 1, numfil

               if ( ifirst==0 ) then

!              --- ifirst = 0, first plot to be made

                   ifirst = 1

               end if

               call plaf ( iseq )

               if ( posdev(1:1)=='h' .or. posdev(1:2)=='xg' ) then

!              --- in case of a display, a stop is required after each
!                  picture, at least for HP

215               read(*,*,iostat=ios)
                  if ( ios/=0 ) go to 215

               end if
               if ( ierror/=0 ) then

!              --- Error found in plot file

                  call ertext ( 1, 0, 0, 0 )
                  read(*,*)
                  ierror = 0
                  nerror = 0
                  go to 200

               end if

220         continue

         else if ( string(1:1)=='C' ) then

!        --- hard copy of plot

            possav = posdev
            posdev = devhcp
            do 230 iseq = 1, numfil
               call plaf ( iseq )
               if ( ierror/=0 ) then

!              --- Error found in plot file

                  call ertext ( 1, 0, 0, 0 )
                  read(*,*)
                  ierror = 0
                  nerror = 0

               end if

               if ( posdev(1:2)=='pl' .or. posdev(1:2)=='ps' .or. &
                    posdev(1:2)=='pc' .or. posdev(1:2)=='pw' ) then

!              --- Post script file has been made

                  iseqps = iseqps + 1
                  string(1:8) = 'sepposc.'
                  write(string(9:11),110) iseqps
                  write(*,*) ' Picture', iseq, ' has been stored in', &
                             ' file ', string(1:11)
               else

!              --- HPGL file has been made

                  iseqpl = iseqpl + 1
                  string(1:8) = 'sephpgl.'
                  write(string(9:11),110) iseqpl
                  write(*,*) ' Picture', iseq, ' has been stored in', &
                             ' file ', string(1:11)

               end if

230         continue
            posdev = possav

         else if ( string(1:1)=='H' ) then

!        --- Help menu

            call inicls
            call ertext ( 30, 0, 0, 0 )
            read (*,*)
            call inicls
            call ertext ( 31, 0, 0, 0 )
            read (*,*)

         else if ( string(1:1)=='N' ) then

!        --- Ask for new name of SEPRAN neutral file

            numfsv = numfil
240         call ertext ( 28, 0, 0, 0 )
            ispecl = 3
            ityprd = 1
            call asknxt
            if ( ityprd/=5 .or. nchar==0 ) go to 240
            lastps = nchar
            namefl = ' '
            namefl(1:lastps+4) = string(1:nchar)//'.001'
            numfil = 0
            readsb = .false.

250         if ( .not. readsb ) then

               numfil = numfil + 1
               write(namefl(lastps+2:lastps+4), 110 ) numfil
               inquire ( file=namefl(1:lastps+4), exist=check )
               if ( .not. check ) then

!              --- file does not exist, decrease numfil

                  numfil = numfil - 1
                  readsb = .true.

               end if
               go to 250

            end if
            namefl(lastps+2:lastps+4) = '001'

            if ( numfil==0 ) then

!           --- Error: no file namefl found

               numfil = numfsv
               lastps = index ( namepl, ' ' ) - 1
               call inicls
               call errchr ( namefl, 1 )
               call errchr ( namepl(1:lastps), 2 )
               call ertext ( 3, 0, 0, 2 )
               read(*,*)
               namefl = ' '
               namefl(1:lastps) = namepl(1:lastps)

            else

!           --- namepl gets the new value

               namepl = ' '
               namepl(1:lastps) = namefl(1:lastps)

            end if

         else if ( string(1:1)=='P' ) then

!        --- Define plotting device

            call indfdv ( 0, 0 )

         else if ( string(1:1)=='Q' ) then

!        --- Quit

            ready = .true.

         end if

         go to 200

      end if

      call erclos ( 'sepdisplay:plsepd' )

!     --- Close SEPRAN

      call finish ( 0 )

      end subroutine plsepd
