      subroutine poswin ( posold, iuser, user )
! ======================================================================
!
!        programmer    Arno van Duin
!        version  1.7  date 30-08-2006 New definition of cpost5
!        version  1.6  date 11-01-1993 added indicator for mincolour by Arno
!        version  1.5  date 04-01-1993 Errors in error message calls corrected
!
!   copyright (c) 1992-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interactive subroutine for the program SEPPOST.
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interactive
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmoduleread
      use sepmodulemach
      use sepmoduleplot
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*)
      double precision user(*)
      character*2 posold

!     iuser   i    Integer information concerning the coefficients
!     posold  i    Name of POSDEV from environment file
!     user    o    Real information concerning the coefficients
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical check, endsub, ready, subsub, ok
      double precision t
      integer i, inum, istrf, lastps, iref, ikeuze
      character*76 namefl
      character*2 possav
      save possav

!     check   Logical denoting whether file exists (true) or not (false)
!     endsub  Logical denoting whether sub-window is handled
!     i       General loop variable.
!     ikeuze  Choise entered by the user
!     inum    Output parameter of subroutine ASK000, contains the number
!             of numbers read.
!     iref    File reference number
!     istrf   Vector number for streamfunction
!     lastps  Last position in string
!     namefl  Name of the file
!     ok      Checks whether coefficient is filled (true) or not (false)
!     possav  Help variable to store the preceding value of posdev temporarily
!     ready   Logical denoting whether main-window is handled
!     subsub  Logical denoting whether sub-window is handled
!     t       Actual time read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT  Get next word
!     ERCARD  Prints the last record read and underlines the errorneous part
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRCHR  Fill characters in character error array
!     ERREAL  Fill reals in real error array
!     ERRINT  Fill integers in integer error array
!     ERRSUB  Error messages
!     ERRWAR  Display warnings
!     ERTEXT  Display text window
!     INICLS  Clear screen
!     INITDF  Initialise some constpstants and arrays
!     PLAF    Display plot on screen
!     PLAFEP  Close plot
!     PRBISC  Find time step corresponding to actual time
!     PRESET  Reset parameters in common blocks equal to old defaults
!     PRSET   Set parameters in common blocks equal to new defaults
!     READSR  Read a series of numbers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     433     Plot file has already been opened before
!     659     ISTRF out of range (<0 or >1000)
!     660     There is no array coresponding to ISTRF
!     672     Too many vectors per time step
!     717     ISTRF is out of range (<0 or >1000) for COMPUTE
!    1472     Plot file is opened and closed, without making any plot
!    1473     Plot file has not been opened
!    1501     dielectric constpstant has zero value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     write main menu
!     case ikeuze of
!     1) write Compute menu
!        case ikeuze of
!        1) while not ready do
!              write Compute Stream Function Menu
!              case ikeuze of
!              1) get start node
!              2) get stream function value
!              3) ready
!              endcase
!           endwhile
!        2) while not ready do
!              write Compute Velocity Profile menu
!              case ikeuze of
!              1) get DEGFD
!              2) get x-coordinate of origin
!              3) get y-coordinate of origin
!              4) get angle
!              5) ready
!              endcase
!           endwhile
!        3) Intersection (2D)
!        4) Intersection (3D)
!        5) ready
!     2) get colours for Define Colour Table
!     3) while not ready do
!           write Define Plot Parameters
!           case ikeuze of
!           i) get parameter i
!           endcase
!        endwhile
!     4) get number of Vi
!        get name of Vi
!     5) display plot menu
!        case ikeuze of
!        1) display plot contour menu
!        2) display plot coloured levels menu
!        3) display plot vector menu
!        4) display plot function menu
!        5) display plot velocity profile menu
!        6) display plot intersection menu
!        7) display plot boundary function menu
!        8) display plot field menu
!        9) display plot track menu
!        10) display 3d plot menu
!        11) display 3d coloured plot menu
!        12) display plot mesh menu
!        13) display plot Vi mesh menu
!        14) display plot curves menu
!        15) display plot Vi curves menu
!        16) display plot points menu
!        17) display plot text menu
!        18) display plot polygon menu
!        19) open plot
!        10) close plot
!        21) get plot identification text
!        22) end
!        endcase
!     6) get number of Vi
!        print Vi
!     7) reset colour table
!     8) reset plot parameters
!     9) Time= &
!     10) Time History
!     11) Make hard copy of plot on standard sized paper (A4)
!     12) Make hard copy of plot on wide paper (A3)
!     13) Make hard copy of plot on A0 plotter
!     14) ready
!     endcase
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'poswin' )

!     --- remove all present SEPRAN neutral plotting files

      lastps = index ( namepl, ' ' ) + 3
      iplpic = 0

!     --- While loop ready

      i = 0
      iref = abs(irefpl)
      namefl(1:lastps) = namepl(1:lastps-4)//'.001'

      ready = .false.
200   if ( .not. ready ) then

         i = i + 1
         write(namefl(lastps-3:lastps), 210 ) i
210      format ( i3.3 )
         inquire ( file=namefl(1:lastps), exist=check )
         if ( check ) then

!        --- file exists, delete

            open ( unit=iref, file=namefl(1:lastps) )
            close ( unit=iref, status='delete' )

         else

!        --- File does not exist, ready

            ready = .true.

         end if
         go to 200

      end if

!     --- Construct name of SEPRAN neutral plotting file

      namefl(1:lastps) = namepl(1:lastps-4)//'.001'

!     --- While loop ready

      ready = .false.
300   if ( .not. ready ) then

!     --- Write main menu

         call inicls
         call ertext ( 101, 0, 0, 0 )

!        --- Read answer

         iinputpst(1) = -1
         endsub = .false.
         ityprd = 1
         call asknxt
         if ( ityprd==2 ) then
            ikeuze = int(realrd)
            if ( ikeuze==1 ) then

!           --- Compute

!               --- get Vi number

               write(*,*)'Give number of Vi:'
               ityprd = 1
               call asknxt
               subsub = .true.
               istrf = int(realrd)
               if ( istrf<0 .or. istrf>1000 ) then

!              --- istrf out of range

                  call errint ( istrf, 1 )
                  call errint ( 1000, 2 )
                  call errsub ( 717, 2, 0, 0 )
                  call ercard ( kcolum, icolum )

               else
                  if ( irefar(istrf)==0 ) then

!                 --- irefar(istrf) = 0, so a new vector must be created

                     numvec = numvec + 1
                     if ( numvec>maxvc1 ) then

!                    --- Too much vectors per time step

                        call errint ( numvec, 1 )
                        call errint ( maxvc1, 2 )
                        call errsub ( 672, 2, 0, 0 )
                        call ercard ( kcolum, icolum )
                     else
                        irefar(istrf) = numvec
                        iinputpst(16) = irefar(istrf)
                        subsub = .false.
                     end if
                  end if
               end if

310            if ( .not. endsub ) then

!              --- write Compute menu

                  call inicls
                  call ertext ( 102, 0, 0, 0 )

!                 --- Read answer

                  ityprd = 1
                  call asknxt
                  ikeuze = int(realrd)
                  if ( ikeuze==1 ) then

!                 --- Compute Stream Function

!                     --- get Vj number

                     write(*,*)'Give number of Vj:'
                     ityprd = 1
                     call asknxt
                     subsub = .true.
                     istrf = int(realrd)
                     if ( istrf<0 .or. istrf>1000 ) then

!                    --- istrf out of range

                        call errint ( istrf, 1 )
                        call errint ( 1000, 2 )
                        call errsub ( 717, 2, 0, 0 )
                        call ercard ( kcolum, icolum )

                     else
                        if ( irefar(istrf)==0 ) then

!                       --- irefar(istrf) = 0, so a new vector must be
!                           created

                           numvec = numvec + 1
                           if ( numvec>maxvc1 ) then

!                          --- Too much vectors per time step

                              call errint ( numvec, 1 )
                              call errint ( maxvc1, 2 )
                              call errsub ( 672, 2, 0, 0 )
                              call ercard ( kcolum, icolum )
                           else
                              irefar(istrf) = numvec
                           end if
                        end if
                        iinputpst(2) = irefar(istrf)
                        subsub = .false.
                     end if

!                    --- write Compute Stream Function menu

311                  if ( .not. subsub ) then
                        call errint ( iinputpst(17), 1 )
                        call erreal ( rinputpst(6), 1 )
                        call inicls
                        call ertext ( 103, 1, 1, 0 )

!                       --- Read answer

                        ityprd = 1
                        call asknxt
                        ikeuze = int(realrd)
                        if ( ikeuze==1 ) then
                           write(*,*)'Give start node number:'
                           ityprd = 1
                           call asknxt
                           iinputpst(17) = int(realrd)
                        else if ( ikeuze==2 ) then
                           write(*,*)'Give stream function value:'
                           ityprd = 1
                           call asknxt
                           rinputpst(6) = realrd
                        else if ( ikeuze==3 ) then
                           subsub = .true.
                           iinputpst(1) = 203
                        end if
                        goto 311
                     end if

                  else if ( ikeuze==2 ) then

!                 --- Compute Velocity Profile

!                     --- get Vj number

                     write(*,*)'Give number of Vj:'
                     ityprd = 1
                     call asknxt
                     subsub = .true.
                     istrf = int(realrd)
                     if ( istrf<0 .or. istrf>1000 ) then

!                    --- istrf out of range

                        call errint ( istrf, 1 )
                        call errint ( 1000, 2 )
                        call errsub ( 717, 2, 0, 0 )
                        call ercard ( kcolum, icolum )

                     else
                        if ( irefar(istrf)==0 ) then

!                       --- irefar(istrf) = 0, so a new vector must be
!                           created

                           numvec = numvec + 1
                           if ( numvec>maxvc1 ) then

!                          --- Too much vectors per time step

                              call errint ( numvec, 1 )
                              call errint ( maxvc1, 2 )
                              call errsub ( 672, 2, 0, 0 )
                              call ercard ( kcolum, icolum )
                           else
                              irefar(istrf) = numvec
                           end if
                        end if
                        iinputpst(2) = irefar(istrf)
                        subsub = .false.
                     end if

!                    --- write Compute Velocity Profile menu

312                  if ( .not. subsub ) then
                        call errint ( iinputpst(8), 1 )
                        call erreal ( rinputpst(7), 1 )
                        call erreal ( rinputpst(8), 2 )
                        call erreal ( rinputpst(9), 3 )
                        call inicls
                        call ertext ( 104, 1, 3, 0 )

!                       --- Read answer

                        ityprd = 1
                        call asknxt
                        ikeuze = int(realrd)
                        if ( ikeuze==1 ) then
                           write(*,*)'Give DEGFD:'
                           ityprd = 1
                           call asknxt
                           iinputpst(8) = int(realrd)
                        else if ( ikeuze==2 ) then
                           write(*,*)'Give x-origin:'
                           ityprd = 1
                           call asknxt
                           rinputpst(7) = realrd
                        else if ( ikeuze==3 ) then
                           write(*,*)'Give y-origin:'
                           ityprd = 1
                           call asknxt
                           rinputpst(8) = realrd
                        else if ( ikeuze==4 ) then
                           write(*,*)'Give angle:'
                           ityprd = 1
                           call asknxt
                           rinputpst(9) = realrd
                        else if ( ikeuze==5 ) then
                           subsub = .true.
                           iinputpst(1) = 201
                        end if
                        goto 312
                     end if

                  else if ( ikeuze==3 ) then

!                 --- Compute Intersection (2D)

!                     --- get Vj number

                     write(*,*)'Give number of Vj:'
                     ityprd = 1
                     call asknxt
                     subsub = .true.
                     istrf = int(realrd)
                     if ( istrf<0 .or. istrf>1000 ) then

!                    --- istrf out of range

                        call errint ( istrf, 1 )
                        call errint ( 1000, 2 )
                        call errsub ( 717, 2, 0, 0 )
                        call ercard ( kcolum, icolum )

                     else
                        if ( irefar(istrf)==0 ) then

!                       --- irefar(istrf) = 0, so a new vector must be
!                           created

                           numvec = numvec + 1
                           if ( numvec>maxvc1 ) then

!                          --- Too much vectors per time step

                              call errint ( numvec, 1 )
                              call errint ( maxvc1, 2 )
                              call errsub ( 672, 2, 0, 0 )
                              call ercard ( kcolum, icolum )
                           else
                              irefar(istrf) = numvec
                           end if
                        end if
                        iinputpst(2) = irefar(istrf)
                        subsub = .false.
                     end if

!                    --- write Intersection (2D) menu

313                  if ( .not. subsub ) then
                        call errint ( iinputpst(8), 1 )
                        call erreal ( rinputpst(7), 1 )
                        call erreal ( rinputpst(8), 2 )
                        call erreal ( rinputpst(9), 3 )
                        call inicls
                        call ertext ( 104, 1, 3, 0 )

!                       --- Read answer

                        ityprd = 1
                        call asknxt
                        ikeuze = int(realrd)
                        if ( ikeuze==1 ) then
                           write(*,*)'Give DEGFD:'
                           ityprd = 1
                           call asknxt
                           iinputpst(8) = int(realrd)
                        else if ( ikeuze==2 ) then
                           write(*,*)'Give x-origin:'
                           ityprd = 1
                           call asknxt
                           rinputpst(7) = realrd
                        else if ( ikeuze==3 ) then
                           write(*,*)'Give y-origin:'
                           ityprd = 1
                           call asknxt
                           rinputpst(8) = realrd
                        else if ( ikeuze==4 ) then
                           write(*,*)'Give angle:'
                           ityprd = 1
                           call asknxt
                           rinputpst(9) = realrd
                        else if ( ikeuze==5 ) then
                           subsub = .true.
                           iinputpst(1) = 201
                        end if
                        goto 313
                     end if

                  else if ( ikeuze==4 ) then

!                 --- Compute Intersection (3D)

!                     --- get Vj number

                     write(*,*)'Give number of Vj:'
                     ityprd = 1
                     call asknxt
                     subsub = .true.
                     istrf = int(realrd)
                     if ( istrf<0 .or. istrf>1000 ) then

!                    --- istrf out of range

                        call errint ( istrf, 1 )
                        call errint ( 1000, 2 )
                        call errsub ( 717, 2, 0, 0 )
                        call ercard ( kcolum, icolum )

                     else
                        if ( irefar(istrf)==0 ) then

!                       --- irefar(istrf) = 0, so a new vector must be
!                           created

                           numvec = numvec + 1
                           if ( numvec>maxvc1 ) then

!                          --- Too much vectors per time step

                              call errint ( numvec, 1 )
                              call errint ( maxvc1, 2 )
                              call errsub ( 672, 2, 0, 0 )
                              call ercard ( kcolum, icolum )
                           else
                              irefar(istrf) = numvec
                           end if
                        end if
                        iinputpst(2) = irefar(istrf)
                        subsub = .false.
                     end if

!                    --- write Intersection (3D) menu

314                  if ( .not. subsub ) then
                        call errint ( iinputpst(8), 1 )
                        call erreal ( rinputpst(17), 1 )
                        call erreal ( rinputpst(18), 2 )
                        call erreal ( rinputpst(19), 3 )
                        call erreal ( rinputpst(20), 4 )
                        call inicls
                        call ertext ( 105, 1, 4, 0 )

!                       --- Read answer

                        ityprd = 1
                        call asknxt
                        ikeuze = int(realrd)
                        if ( ikeuze==1 ) then
                           write(*,*)'Give DEGFD:'
                           ityprd = 1
                           call asknxt
                           iinputpst(8) = int(realrd)
                        else if ( ikeuze==2 ) then
                           write(*,*)'Give a:'
                           ityprd = 1
                           call asknxt
                           rinputpst(17) = realrd
                        else if ( ikeuze==3 ) then
                           write(*,*)'Give b:'
                           ityprd = 1
                           call asknxt
                           rinputpst(18) = realrd
                        else if ( ikeuze==4 ) then
                           write(*,*)'Give c:'
                           ityprd = 1
                           call asknxt
                           rinputpst(19) = realrd
                        else if ( ikeuze==5 ) then
                           write(*,*)'Give d:'
                           ityprd = 1
                           call asknxt
                           rinputpst(20) = realrd
                        else if ( ikeuze==6 ) then
                           subsub = .true.
                           iinputpst(1) = 201
                        end if
                        goto 314
                     end if

                  else if ( ikeuze==5 ) then
                     endsub = .true.
                  end if

                  goto 310
               end if
            else if ( ikeuze==2 ) then

!           --- perform Define Colour Table action

               write(*,*)'Give colour numbers:'
               ityprd = 1
               call readsr ( getal, 100, inum )
               do 320 i = 1, inum
                  indcol(i+5) = int(getal(i))
320            continue
               iinputpst(62) = 0
               iinputpst(63) = 0
               call prset

            else if ( ikeuze==3 ) then

330            if ( .not. endsub ) then

!              --- write Define Plot Parameters menu

                  call inicls
                  call errint ( isigy, 1 )
                  call errint ( iinputpst(22), 2 )
                  call errint ( dcolor, 3 )
                  call errint ( iinputpst(23), 4 )
                  call erreal ( rinputpst(9), 1 )
                  call erreal ( rinputpst(7), 2 )
                  call erreal ( rinputpst(8), 3 )
                  call erreal ( rinputpst(22), 4 )
                  call erreal ( axmin, 5 )
                  call erreal ( rinputpst(10), 6 )
                  call erreal ( axmax, 7 )
                  call erreal ( rinputpst(11), 8 )
                  call erreal ( aymin, 9 )
                  if ( iinputpst(40)==0 ) then
                     call errchr ( 'noaxis', 1 )
                  else
                     call errchr ( 'axis', 1 )
                  end if
                  if ( iinputpst(31)==0 ) then
                     call errchr ( 'nobold', 2 )
                  else
                     call errchr ( 'bold', 2 )
                  end if
                  if ( iinputpst(28)==0 ) then
                     call errchr ( 'noboundaries', 3 )
                  else
                     call errchr ( 'boundaries', 3 )
                  end if
                  if ( iinputpst(29)==0 ) then
                     call errchr ( 'nocontract', 4 )
                  else
                     call errchr ( 'contract', 4 )
                  end if
                  if ( iinputpst(33)==0 ) then
                     call errchr ( 'noelement', 5 )
                  else
                     call errchr ( 'element', 5 )
                  end if
                  call ertext ( 106, 4, 9, 5 )

                  call errint ( iinputpst(27), 1 )
                  call errint ( nstepx, 2 )
                  call errint ( iinputpst(26), 3 )
                  call errint ( nstepy, 4 )
                  call errint ( abs(mod(iinputpst(6),1000)), 2 )
                  call errint ( iinputpst(21), 6 )
                  call errint ( kiesx, 7 )
                  call errint ( isigx, 8 )
                  call errint ( kiesy, 9 )
                  call erreal ( rinputpst(12), 1 )
                  call erreal ( aymax, 2 )
                  call erreal ( rinputpst(3), 3 )
                  call erreal ( rinputpst(16), 4 )
                  call erreal ( xmin, 5 )
                  call erreal ( xmax, 6 )
                  call erreal ( rinputpst(1), 7 )
                  call erreal ( ymin, 8 )
                  call erreal ( ymax, 9 )
                  call erreal ( rinputpst(2), 10 )
                  if ( irotat==0 ) then
                     call errchr ( 'norotate', 1 )
                  else if ( irotat==1 ) then
                     call errchr ( 'norotate', 1 )
                  else
                     call errchr ( 'rotate', 1 )
                  end if
                  if ( iinputpst(30)==0 ) then
                     call errchr ( 'noinner', 2 )
                  else
                     call errchr ( 'inner', 2 )
                  end if
                  if ( iinputpst(35)==0 ) then
                     call errchr ( 'nomark', 3 )
                  else
                     call errchr ( 'mark', 3 )
                  end if
                  if ( iinputpst(34)==0 ) then
                     call errchr ( 'nonode', 4 )
                  else
                     call errchr ( 'node', 4 )
                  end if
                  if ( iinputpst(39)==0 ) then
                     call errchr ( ' ', 5 )
                  else
                     call errchr ( textid(1), 5 )
                  end if
                  if ( iinputpst(33)==0 ) then
                     call errchr ( 'nonumber', 6 )
                  else
                     call errchr ( 'number', 6 )
                  end if
                  if ( iinputpst(37)==0 ) then
                     call errchr ( ' ', 7 )
                  else
                     call errchr ( textx, 7 )
                  end if
                  if ( iinputpst(38)==0 ) then
                     call errchr ( ' ', 8 )
                  else
                     call errchr ( texty, 8 )
                  end if
                  call ertext ( 107, 9, 10, 8 )

!                 --- Read answer

                  ityprd = 1
                  call asknxt
                  ikeuze = int(realrd)
                  if ( ikeuze==1 ) then
                     write(*,*)'Give angle:'
                     ityprd = 1
                     call asknxt
                     rinputpst(9) = realrd
                  else if ( ikeuze==2 ) then
                     if ( iinputpst(40)==0 ) then
                        iinputpst(40) = 1
                     else
                        iinputpst(40) = 0
                     end if
                  else if ( ikeuze==3 ) then
                     if ( iinputpst(31)==0 ) then
                        iinputpst(31) = 1
                     else
                        iinputpst(31) = 0
                     end if
                  else if ( ikeuze==4 ) then
                     if ( iinputpst(28)==0 ) then
                        iinputpst(28) = 1
                     else
                        iinputpst(28) = 0
                     end if
                  else if ( ikeuze==5 ) then
                     write(*,*)'Give colour:'
                     ityprd = 1
                     call asknxt
                     dcolor = int(realrd)
                  else if ( ikeuze==6 ) then
                     if ( iinputpst(29)==0 ) then
                        iinputpst(29) = 1
                     else
                        iinputpst(29) = 0
                     end if
                  else if ( ikeuze==7 ) then
                     if ( iinputpst(33)==0 ) then
                        iinputpst(33) = 1
                     else
                        iinputpst(33) = 0
                     end if
                  else if ( ikeuze==8 ) then
                     write(*,*)'Give x-coordinate of eye point:'
                     ityprd = 1
                     call asknxt
                     rinputpst(10) = realrd
                     if ( iinputpst(53)>=10 ) iinputpst(53)=iinputpst(53)-10
                  else if ( ikeuze==9 ) then
                     write(*,*)'Give y-coordinate of eye point:'
                     ityprd = 1
                     call asknxt
                     rinputpst(11) = realrd
                     if ( iinputpst(53)>=10 ) iinputpst(53)=iinputpst(53)-10
                  else if ( ikeuze==10 ) then
                     write(*,*)'Give z-coordinate of eye point:'
                     ityprd = 1
                     call asknxt
                     rinputpst(12) = realrd
                     if ( iinputpst(53)>=10 ) iinputpst(53)=iinputpst(53)-10
                  else if ( ikeuze==11 ) then
                     write(*,*)'Give factor:'
                     ityprd = 1
                     call asknxt
                     rinputpst(2) = realrd
                  else if ( ikeuze==12 ) then
                     write(*,*)'Give height:'
                     ityprd = 1
                     call asknxt
                     rinputpst(16) = realrd
                  else if ( ikeuze==13 ) then
                     if ( iinputpst(30)==0 ) then
                        iinputpst(30) = 1
                     else
                        iinputpst(30) = 0
                     end if
                  else if ( ikeuze==14 ) then
                     write(*,*)'Give length:'
                     ityprd = 1
                     call asknxt
                     rinputpst(1) = realrd
                  else if ( ikeuze==15 ) then
                     if ( iinputpst(35)==0 ) then
                        iinputpst(35) = 1
                     else
                        iinputpst(35) = 0
                     end if
                  else if ( ikeuze==16 ) then
                     write(*,*)'Give maxcolour:'
                     ityprd = 1
                     call asknxt
                     iinputpst(27) = int(realrd)
                     iinputpst(63) = 1
                  else if ( ikeuze==17 ) then
                     write(*,*)'Give mincolour:'
                     ityprd = 1
                     call asknxt
                     iinputpst(26) = int(realrd)
                     iinputpst(62) = 1
                  else if ( ikeuze==18 ) then
                     write(*,*)'Give ncolour:'
                     ityprd = 1
                     call asknxt
                     iinputpst(6) = int(realrd)
                  else if ( ikeuze==19 ) then
                     if ( iinputpst(34)==0 ) then
                        iinputpst(34) = 1
                     else
                        iinputpst(34) = 0
                     end if
                  else if ( ikeuze==20 ) then
                     if ( iinputpst(33)==0 ) then
                        iinputpst(33) = 1
                     else
                        iinputpst(33) = 0
                     end if
                  else if ( ikeuze==21 ) then
                     write(*,*)'Give number format Nx:'
                     ityprd = 1
                     call asknxt
                     kiesx = int(realrd)
                  else if ( ikeuze==22 ) then
                     write(*,*)'Give number format Mx:'
                     ityprd = 1
                     call asknxt
                     isigx = int(realrd)
                  else if ( ikeuze==23 ) then
                     write(*,*)'Give number format Ny:'
                     ityprd = 1
                     call asknxt
                     kiesy = int(realrd)
                  else if ( ikeuze==24 ) then
                     write(*,*)'Give number format My:'
                     ityprd = 1
                     call asknxt
                     isigy = int(realrd)
                  else if ( ikeuze==25 ) then
                     write(*,*)'Give x-coordinate of origin:'
                     ityprd = 1
                     call asknxt
                     rinputpst(7) = realrd
                  else if ( ikeuze==26 ) then
                     write(*,*)'Give y-coordinate of origin:'
                     ityprd = 1
                     call asknxt
                     rinputpst(8) = realrd
                  else if ( ikeuze==27 ) then
                     write(*,*)'Give picture number i:'
                     ityprd = 1
                     call asknxt
                     iinputpst(22) = int(realrd)
                  else if ( ikeuze==28 ) then
                     write(*,*)'Give total number of pictures n:'
                     ityprd = 1
                     call asknxt
                     iinputpst(23) = int(realrd)
                  else if ( ikeuze==29 ) then
                     write(*,*)'Give reference value:'
                     ityprd = 1
                     call asknxt
                     rinputpst(22) = realrd
                  else if ( ikeuze==30 ) then
                     write(*,*)'Give minimum x-value of region:'
                     ityprd = 1
                     call asknxt
                     xmin = realrd
                  else if ( ikeuze==31 ) then
                     write(*,*)'Give maximum x-value of region:'
                     ityprd = 1
                     call asknxt
                     xmax = realrd
                  else if ( ikeuze==32 ) then
                     write(*,*)'Give minimum y-value of region:'
                     ityprd = 1
                     call asknxt
                     ymin = realrd
                  else if ( ikeuze==33 ) then
                     write(*,*)'Give maximum y-value of region:'
                     ityprd = 1
                     call asknxt
                     ymax = realrd
                  else if ( ikeuze==34 ) then
                     if ( irotat==1 .or. irotat==0 ) then
                        irotat = 2
                     else if (irotat==2 ) then
                        irotat = 1
                     end if
                  else if ( ikeuze==35 ) then
                     write(*,*)'Give x-upper of scales:'
                     ityprd = 1
                     call asknxt
                     axmax = realrd
                  else if ( ikeuze==36 ) then
                     write(*,*)'Give x-under of scales:'
                     ityprd = 1
                     call asknxt
                     axmin = realrd
                  else if ( ikeuze==37 ) then
                     write(*,*)'Give y-upper of scales:'
                     ityprd = 1
                     call asknxt
                     aymax = realrd
                  else if ( ikeuze==38 ) then
                     write(*,*)'Give y-under of scales:'
                     ityprd = 1
                     call asknxt
                     aymin = realrd
                  else if ( ikeuze==39 ) then
                     write(*,*)'Give step-x:'
                     ityprd = 1
                     call asknxt
                     nstepx = int(realrd)
                  else if ( ikeuze==40 ) then
                     write(*,*)'Give step-y:'
                     ityprd = 1
                     call asknxt
                     nstepy = int(realrd)
                  else if ( ikeuze==41 ) then
                     write(*,*)'Give symbol:'
                     ityprd = 1
                     call asknxt
                     iinputpst(21) = int(realrd)
                  else if ( ikeuze==42 ) then
                     write(*,*)'Give text:'
                     ityprd = 1
                     call asknxt
                     textid(1) = cardim
                     if ( textid(1)==' ' ) then
                        iinputpst(39) = 0
                     else
                        iinputpst(39) = lastp
                     end if
                  else if ( ikeuze==43 ) then
                     write(*,*)'Give textx:'
                     ityprd = 1
                     call asknxt
                     textx = cardim
                     if ( textx==' ' ) then
                        iinputpst(37) = 0
                     else
                        iinputpst(37) = lastp
                     end if
                  else if ( ikeuze==44 ) then
                     write(*,*)'Give texty:'
                     ityprd = 1
                     call asknxt
                     texty = cardim
                     if ( texty==' ' ) then
                        iinputpst(38) = 0
                     else
                        iinputpst(38) = lastp
                     end if
                  else if ( ikeuze==45 ) then
                     write(*,*)'Give yfact:'
                     ityprd = 1
                     call asknxt
                     rinputpst(1) = realrd
                  else if ( ikeuze==46 ) then
                     endsub = .true.
                  end if
                  goto 330
               end if
               call prset
            else if ( ikeuze==4 ) then

!           --- get Vi number

               write(*,*)'Give number of Vi:'
               ityprd = 1
               call asknxt
               istrf = int(realrd)
               if ( istrf<0 .or. istrf>1000 ) then

!              --- istrf out of range

                  call errint ( istrf, 1 )
                  call errint ( 1000, 2 )
                  call errsub ( 659, 2, 0, 0 )
                  call ercard ( kcolum, icolum )
               else
                  if ( irefar( istrf)==0 ) then

!                 --- There is no array coresponding to istrf

                     call errint ( istrf, 1 )
                     call errsub ( 660, 1, 0, 0 )
                     call ercard ( kcolum, icolum )
                  else
                     iinputpst(16) = irefar(istrf)

!                    --- get name

                     write(*,*)'Give name of V',istrf,':'
                     ityprd = 1
                     call asknxt
                     names(irefar(istrf)) = cardim(1:30)
                  end if
               end if
            else if ( ikeuze==5 ) then

!           --- write Plot menu

               call inicls
               call ertext ( 108, 0, 0, 0 )

!              --- Read answer

               ityprd = 1
               call asknxt
               ikeuze = int(realrd)
               if ( ikeuze==1 ) then

!              --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  subsub = .true.
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(2) = irefar(istrf)
                     subsub = .false.
                  end if

!                 --- write Plot Contour menu

351               if ( .not. subsub ) then
                     call errint ( iinputpst(8), 1 )
                     call errint ( abs(mod(iinputpst(6),1000)), 2 )
                     call errint ( iinputpst(7), 3 )
                     call erreal ( rinputpst(4), 1 )
                     call erreal ( rinputpst(5), 2 )
                     call inicls
                     call ertext ( 109, 3, 2, 0 )

!                    --- Read answer

                     ityprd = 1
                     call asknxt
                     ikeuze = int(realrd)
                     if ( ikeuze==1 ) then
                        write(*,*)'Give DEGFD:'
                        ityprd = 1
                        call asknxt
                        iinputpst(8) = int(realrd)
                     else if ( ikeuze==2 ) then
                        write(*,*)'Give number of levels:'
                        ityprd = 1
                        call asknxt
                        iinputpst(6) = int(realrd)
                     else if ( ikeuze==3 ) then
                        write(*,*)'Give levels:'
                        ityprd = 1
                        call readsr ( contln(6), 100, inum )
                        iinputpst(6) = inum
                        iinputpst(14) = 1
                     else if ( ikeuze==4 ) then
                        write(*,*)'Give minimum level:'
                        ityprd = 1
                        call asknxt
                        iinputpst(12) = 1
                        rinputpst(4) = realrd
                     else if ( ikeuze==5 ) then
                        write(*,*)'Give maximum level:'
                        ityprd = 1
                        call asknxt
                        iinputpst(13) = 1
                        rinputpst(5) = realrd
                     else if ( ikeuze==6 ) then
                        write(*,*)'Give smoothing factor:'
                        ityprd = 1
                        call asknxt
                        iinputpst(7) = int(realrd)
                     else if ( ikeuze==7 ) then
                        subsub = .true.
                        iinputpst(1) = 112
                     end if

                     goto 351
                  end if
               else if ( ikeuze==2 ) then

!              --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  subsub = .true.
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(2) = irefar(istrf)
                     subsub = .false.
                  end if

!                 --- write Plot Coloured Levels menu

352               if ( .not. subsub ) then
                     call errint ( iinputpst(8), 1 )
                     call errint ( abs(mod(iinputpst(6),1000)), 2 )
                     call erreal ( rinputpst(4), 1 )
                     call erreal ( rinputpst(5), 2 )
                     call inicls
                     call ertext ( 110, 2, 2, 0 )

!                    --- Read answer

                     ityprd = 1
                     call asknxt
                     ikeuze = int(realrd)
                     if ( ikeuze==1 ) then
                        write(*,*)'Give DEGFD:'
                        ityprd = 1
                        call asknxt
                        iinputpst(8) = int(realrd)
                     else if ( ikeuze==2 ) then
                        write(*,*)'Give number of levels:'
                        ityprd = 1
                        call asknxt
                        iinputpst(6) = int(realrd)
                     else if ( ikeuze==3 ) then
                        write(*,*)'Give levels:'
                        ityprd = 1
                        call readsr ( contln(6), 100, inum )
                        iinputpst(6) = inum
                        iinputpst(14) = 1
                     else if ( ikeuze==4 ) then
                        write(*,*)'Give minimum level:'
                        ityprd = 1
                        call asknxt
                        iinputpst(12) = 1
                        rinputpst(4) = realrd
                     else if ( ikeuze==5 ) then
                        write(*,*)'Give maximum level:'
                        ityprd = 1
                        call asknxt
                        iinputpst(13) = 1
                        rinputpst(5) = realrd
                     else if ( ikeuze==6 ) then
                        subsub = .true.
                        iinputpst(1) = 113
                     end if

                     goto 352
                  end if
               else if ( ikeuze==3 ) then

!              --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  subsub = .true.
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(2) = irefar(istrf)
                     subsub = .false.
                  end if

!                 --- write Plot Vector menu

353               if ( .not. subsub ) then
                     call errint ( iinputpst(8), 1 )
                     call errint ( iinputpst(9), 2 )
                     call inicls
                     call ertext ( 111, 2, 0, 0 )

!                    --- Read answer

                     ityprd = 1
                     call asknxt
                     ikeuze = int(realrd)
                     if ( ikeuze==1 ) then
                        write(*,*)'Give DEGFD1:'
                        ityprd = 1
                        call asknxt
                        iinputpst(8) = int(realrd)
                     else if ( ikeuze==2 ) then
                        write(*,*)'Give DEGFD2:'
                        ityprd = 1
                        call asknxt
                        iinputpst(9) = int(realrd)
                     else if ( ikeuze==3 ) then
                        subsub = .true.
                        iinputpst(1) = 110
                     end if

                     goto 353
                  end if
               else if ( ikeuze==4 ) then

!              --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  subsub = .true.
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(1) = 120
                     iinputpst(2) = irefar(istrf)
                     subsub = .false.
                  end if
               else if ( ikeuze==5 ) then

!              --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  subsub = .true.
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(2) = irefar(istrf)
                     subsub = .false.
                  end if

!                 --- write Plot Velocity Profile menu

354               if ( .not. subsub ) then
                     call errint ( iinputpst(8), 1 )
                     call erreal ( rinputpst(7), 1 )
                     call erreal ( rinputpst(8), 2 )
                     call erreal ( rinputpst(9), 3 )
                     call inicls
                     call ertext ( 112, 1, 3, 0 )

!                    --- Read answer

                     ityprd = 1
                     call asknxt
                     ikeuze = int(realrd)
                     if ( ikeuze==1 ) then
                        write(*,*)'Give DEGFD:'
                        ityprd = 1
                        call asknxt
                        iinputpst(8) = int(realrd)
                     else if ( ikeuze==2 ) then
                        write(*,*)'Give x-coordinate of origin:'
                        ityprd = 1
                        call asknxt
                        rinputpst(7) = realrd
                     else if ( ikeuze==3 ) then
                        write(*,*)'Give y-coordinate of origin:'
                        ityprd = 1
                        call asknxt
                        rinputpst(8) = realrd
                     else if ( ikeuze==4 ) then
                        write(*,*)'Give angle:'
                        ityprd = 1
                        call asknxt
                        rinputpst(9) = realrd
                     else if ( ikeuze==5 ) then
                        subsub = .true.
                        iinputpst(1) = 121
                     end if

                     goto 354
                  end if
               else if ( ikeuze==6 ) then

!              --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  subsub = .true.
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(2) = irefar(istrf)
                     subsub = .false.
                  end if

!                 --- write Plot Intersection menu

355               if ( .not. subsub ) then
                     call errint ( iinputpst(8), 1 )
                     call erreal ( rinputpst(7), 1 )
                     call erreal ( rinputpst(8), 2 )
                     call erreal ( rinputpst(9), 3 )
                     call inicls
                     call ertext ( 112, 1, 3, 0 )

!                    --- Read answer

                     ityprd = 1
                     call asknxt
                     ikeuze = int(realrd)
                     if ( ikeuze==1 ) then
                        write(*,*)'Give DEGFD:'
                        ityprd = 1
                        call asknxt
                        iinputpst(8) = int(realrd)
                     else if ( ikeuze==2 ) then
                        write(*,*)'Give x-coordinate of origin:'
                        ityprd = 1
                        call asknxt
                        rinputpst(7) = realrd
                     else if ( ikeuze==3 ) then
                        write(*,*)'Give y-coordinate of origin:'
                        ityprd = 1
                        call asknxt
                        rinputpst(8) = realrd
                     else if ( ikeuze==4 ) then
                        write(*,*)'Give angle:'
                        ityprd = 1
                        call asknxt
                        rinputpst(9) = realrd
                     else if ( ikeuze==5 ) then
                        subsub = .true.
                        iinputpst(1) = 121
                     end if

                     goto 355
                  end if
               else if ( ikeuze==7 ) then

!              --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(1) = 120
                     iinputpst(2) = irefar(istrf)
                     write(*,*)'Give curve numbers'
                     ityprd = 1
                     call readsr ( getal, 100, inum )
                     do 356 i = 1, inum
                        icurvs(i+1) = int(getal(i))
356                  continue
                     icurvs(1) = inum
                  end if
               else if ( ikeuze==8 ) then

!              --- plot field

!                  --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(1) = 118
                     iinputpst(2) = irefar(istrf)

!                    --- get Vi number

                     write(*,*)'Field lines from (part of) boundary (0)', &
                               ' or point (1):'
                     ityprd = 1
                     call asknxt
                     if ( int(realrd)==0 ) then

!                    --- display boundary menu

                        if ( iinputpst(5)<=0 ) iinputpst(5) = 1
                        if ( iinputpst(5)>=7 ) iinputpst(5) = 1
                        if ( iinputpst(5)==4 ) iinputpst(5) = 1
                        if ( iinputpst(5)==5 ) iinputpst(5) = 2
                        if ( iinputpst(5)==6 ) iinputpst(5) = 3
                        if ( iinputpst(18)==0 ) iinputpst(18) = 1
                        if ( iinputpst(19)==0 ) iinputpst(19) = 2
                        if ( abs(rinputpst(6))<=0d0 ) rinputpst(6) = 1d0
                        subsub = .false.

357                     if ( .not. subsub ) then
                           call errint ( iinputpst(18), 1 )
                           call errint ( iinputpst(19), 2 )
                           call erreal ( rinputpst(6), 1 )
                           if ( iinputpst(18)/=iinputpst(19) ) then
                              if ( iinputpst(5)==1 .or. &
                                   iinputpst(5)==4 ) then
                                 iinputpst(5) = iinputpst(5) + 1
                              end if
                           end if
                           if ( iinputpst(5)==2 .or. &
                                iinputpst(5)==5 ) then
                              call errchr ( 'FROM', 1 )
                           else if ( iinputpst(5)==3 .or. &
                                    iinputpst(5)==6 ) then
                              call errchr ( 'TOWARDS', 1 )
                           else
                              call errchr ( 'THROUGH', 1 )
                           end if
                           call inicls
                           call ertext ( 120, 2, 1, 1 )

!                          --- Read answer

                           ityprd = 1
                           call asknxt
                           ikeuze = int(realrd)
                           if ( ikeuze==1 ) then
                              write(*,*) &
                                 'Give user point number of first', &
                                 ' boundary point:'
                              ityprd = 1
                              call asknxt
                              iinputpst(18) = int(realrd)
                           else if ( ikeuze==2 ) then
                              write(*,*) &
                                 'Give user point number of last ', &
                                 'boundary point:'
                              ityprd = 1
                              call asknxt
                              iinputpst(19) = int(realrd)
                           else if ( ikeuze==3 ) then
                              write(*,*) &
                                 'Give flux-cut between to consecutive', &
                                 ' field lines (>0):'
                              ityprd = 1
                              call asknxt
                              rinputpst(6) = realrd
                           else if ( ikeuze==4 ) then
                              if ( iinputpst(5)==1 ) then
                                 iinputpst(5) = 2
                              else if ( iinputpst(5)==2 ) then
                                 iinputpst(5) = 3
                              else if ( iinputpst(5)==3 ) then
                                 iinputpst(5) = 1
                              end if
                           else if ( ikeuze==5 ) then
                              write(*,*) &
                                 'Give the value for each element ', &
                                 'group ( ',iuser(2),' values):'
                              ityprd = 1
                              call readsr ( user(6), 100, inum )
                           else if ( ikeuze==6 ) then
                              ok = .true.
                              do 3570 i = 1, iuser(2)
                                 ok = ok .and. ( abs(user(5+i))<=0d0 )
3570                          continue
                              if ( ok ) then
                                 subsub = .true.
                                 iinputpst(1) = 118
                              else
                                 call errsub ( 1501, 0, 0, 0 )
                                 read (*,'(a)') cardim
                                 ierror = 0
                                 nerror = 0
                              end if
                           end if

                           goto 357
                        end if

                     else

!                    --- display point menu

                        if ( iinputpst(5)<=0 ) iinputpst(5) = 4
                        if ( iinputpst(5)>=7 ) iinputpst(5) = 4
                        if ( iinputpst(5)==1 ) iinputpst(5) = 4
                        if ( iinputpst(5)==2 ) iinputpst(5) = 5
                        if ( iinputpst(5)==3 ) iinputpst(5) = 6
                        subsub = .false.

358                     if ( .not. subsub ) then
                           call erreal ( rinputpst(51), 1 )
                           call errint ( iinputpst(52), 2 )
                           if ( iinputpst(5)==2 .or. &
                                iinputpst(5)==5 ) then
                              call errchr ( 'FROM', 1 )
                           else if ( iinputpst(5)==3 .or. &
                                    iinputpst(5)==6 ) then
                              call errchr ( 'TOWARDS', 1 )
                           else
                              call errchr ( 'THROUGH', 1 )
                           end if
                           call inicls
                           call ertext ( 121, 0, 2, 1 )

!                          --- Read answer

                           ityprd = 1
                           call asknxt
                           ikeuze = int(realrd)
                           iinputpst(49) = 1
                           if ( ikeuze==1 ) then
                              write(*,*) &
                                 'Give x-coordinate of starting point:'
                              ityprd = 1
                              call asknxt
                              rinputpst(51) = realrd
                           else if ( ikeuze==2 ) then
                              write(*,*) &
                                 'Give y-coordinate of starting point:'
                              ityprd = 1
                              call asknxt
                              rinputpst(52) = realrd
                           else if ( ikeuze==3 ) then
                              if ( iinputpst(5)==4 ) then
                                 iinputpst(5) = 5
                              else if ( iinputpst(5)==5 ) then
                                 iinputpst(5) = 6
                              else if ( iinputpst(5)==6 ) then
                                 iinputpst(5) = 4
                              end if
                           else if ( ikeuze==4 ) then
                              write(*,*) &
                                 'Give the value for each element ', &
                                  'group ( ',iuser(2),' values):'
                              ityprd = 1
                              call readsr ( user(6), 100, inum )
                           else if ( ikeuze==5 ) then
                              ok = .true.
                              do 3580 i = 1, iuser(2)
                                 ok = ok .and. ( abs(user(5+i))<=0d0 )
3580                          continue
                              if ( ok ) then
                                 subsub = .true.
                                 iinputpst(1) = 118
                              else
                                 call errsub ( 1501, 0, 0, 0 )
                                 read (*,'(a)') cardim
                              end if
                           end if

                           goto 358
                        end if

                     end if

                  end if
               else if ( ikeuze==9 ) then

!              --- plot track

!                  --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  subsub = .true.
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(2) = irefar(istrf)
                     subsub = .false.
                  end if

!                 --- fill defaults

                  iinputpst(48) = 0
                  iinputpst(49) = 2
                  iinputpst(50) = 0
                  iinputpst(52) = 1
                  rinputpst(21) = rinfin
                  rinputpst(51) = 0d0
                  rinputpst(52) = 0d0

!                 --- write Plot Track menu

3590              if ( .not. subsub ) then
                     call erreal ( rinputpst(51), 1 )
                     call erreal ( rinputpst(52), 2 )
                     call erreal ( rinputpst(21), 3 )
                     if ( iinputpst(52)==1 ) then
                        call errchr ( 'y', 1 )
                        call errchr ( '+', 2 )
                     else if ( iinputpst(52)==2 ) then
                        call errchr ( 'x', 1 )
                        call errchr ( '+', 2 )
                     else if ( iinputpst(52)==3 ) then
                        call errchr ( 'y', 1 )
                        call errchr ( '-', 2 )
                     else if ( iinputpst(52)==4 ) then
                        call errchr ( 'x', 1 )
                        call errchr ( '-', 2 )
                     end if
                     if ( iinputpst(50)==0 ) then
                        call errchr ( 'are not printed', 3 )
                     else
                        call errchr ( 'are printed', 3 )
                     end if
                     call inicls
                     call ertext ( 122, 0, 3, 3 )

!                    --- Read answer

                     ityprd = 1
                     call asknxt
                     ikeuze = int(realrd)
                     if ( ikeuze==1 ) then
                        write(*,*)'Give x-coordinate:'
                        ityprd = 1
                        call asknxt
                        rinputpst(51) = realrd
                     else if ( ikeuze==2 ) then
                        write(*,*)'Give y-coordinate:'
                        ityprd = 1
                        call asknxt
                        rinputpst(52) = realrd
                     else if ( ikeuze==3 ) then
                        write(*,*)'Give maximum time:'
                        ityprd = 1
                        call asknxt
                        rinputpst(21) = realrd
                     else if ( ikeuze==4 ) then
                        iinputpst(52) = iinputpst(52) + 1
                        if ( iinputpst(52)==5 ) iinputpst(52) = 1
                     else if ( ikeuze==5 ) then
                        if ( iinputpst(50)==0 ) iinputpst(50) = 1
                        if ( iinputpst(50)==1 ) iinputpst(50) = 0
                     else if ( ikeuze==6 ) then
                        write(*,*) &
                           'Give the value for each element group ', &
                           '( ',iuser(2),' values):'
                        ityprd = 1
                        call readsr ( user(6), 100, inum )
                     else if ( ikeuze==7 ) then
                        ok = .true.
                        do 3591 i = 1, iuser(2)
                           ok = ok .and. ( abs(user(5+i))<=0d0 )
3591                    continue
                        if ( ok ) then
                           subsub = .true.
                           iinputpst(1) = 119
                        else
                           call errsub ( 1501, 0, 0, 0 )
                           read (*,'(a)') cardim
                           ierror = 0
                           nerror = 0
                        end if
                     end if

                     goto 3590
                  end if
               else if ( ikeuze==10 ) then

!              --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  subsub = .true.
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(2) = irefar(istrf)
                     subsub = .false.
                  end if

!                 --- write 3D Plot menu

359               if ( .not. subsub ) then
                     call errint ( iinputpst(24), 1 )
                     call errint ( iinputpst(25), 2 )
                     call inicls
                     call ertext ( 113, 2, 0, 0 )

!                    --- Read answer

                     ityprd = 1
                     call asknxt
                     ikeuze = int(realrd)
                     if ( ikeuze==1 ) then
                        write(*,*)'Give LINDIREC:'
                        ityprd = 1
                        call asknxt
                        iinputpst(24) = int(realrd)
                     else if ( ikeuze==2 ) then
                        write(*,*)'Give number of lines:'
                        ityprd = 1
                        call asknxt
                        iinputpst(25) = int(realrd)
                     else if ( ikeuze==3 ) then
                        subsub = .true.
                        iinputpst(1) = 115
                     end if

                     goto 359
                  end if
               else if ( ikeuze==11 ) then

!              --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  subsub = .true.
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(1) = 116
                     iinputpst(2) = irefar(istrf)
                     subsub = .false.
                  end if
               else if ( ikeuze==12 ) then

!              --- write Plot Mesh menu

                  subsub = .false.
410              if ( .not. subsub ) then
                     if ( iinputpst(36)==1 )then
                        call errchr ( 'ON', 1 )
                     else
                        call errchr ( 'OFF', 1 )
                     end if
                     call inicls
                     call ertext ( 114, 0, 0, 1 )

!                    --- Read answer

                     ityprd = 1
                     call asknxt
                     ikeuze = int(realrd)
                     if ( ikeuze==1 ) then
                        write(*,*)'Give element groups to be skipped'
                        ityprd = 1
                        call readsr ( getal, 100, inum )
                        do 411 i = 1, inum
                           igroup(i) = int(getal(i))
411                     continue
                        iskip = 1
                        iinputpst(32) = inum
                     else if ( ikeuze==2 ) then
                        if ( iinputpst(36)==0 ) then
                           iinputpst(36) = 1
                        else
                           iinputpst(36) = 0
                        end if
                     else if ( ikeuze==3 ) then
                        subsub = .true.
                        iinputpst(1) = 101
                     end if

                     goto 410
                  end if
               else if ( ikeuze==13 ) then

!              --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(1) = 101
                     iinputpst(2) = irefar(istrf)
                     subsub = .false.
                  end if
               else if ( ikeuze==14 ) then
                  iinputpst(1) = 103
               else if ( ikeuze==15 ) then

!              --- get Vi number

                  write(*,*)'Give number of Vi:'
                  ityprd = 1
                  call asknxt
                  istrf = int(realrd)
                  if ( istrf<0 .or. istrf>1000 ) then

!                 --- istrf out of range

                     call errint ( istrf, 1 )
                     call errint ( 1000, 2 )
                     call errsub ( 717, 2, 0, 0 )
                     call ercard ( kcolum, icolum )

                  else
                     if ( irefar(istrf)==0 ) then

!                    --- irefar(istrf) = 0, so a new vector must be created

                        numvec = numvec + 1
                        if ( numvec>maxvc1 ) then

!                       --- Too much vectors per time step

                           call errint ( numvec, 1 )
                           call errint ( maxvc1, 2 )
                           call errsub ( 672, 2, 0, 0 )
                           call ercard ( kcolum, icolum )
                        else
                           irefar(istrf) = numvec
                        end if
                     end if
                     iinputpst(1) = 103
                     iinputpst(2) = irefar(istrf)
                     subsub = .false.
                  end if
               else if ( ikeuze==16 ) then
                  iinputpst(1) = 102
               else if ( ikeuze==17 ) then
                  write(*,*)'Give text:'
                  ityprd = 1
                  call asknxt
                  textid(1) = cardim
                  if ( textid(1)==' ' ) then
                     iinputpst(39) = 0
                  else
                     iinputpst(39) = lastp
                  end if
                  write(*,*)'Give coordinates of position of text:'
                  ityprd = 1
                  call readsr ( rinputpst(7), 2, inum )
                  if ( inum==0 ) then
                     rinputpst(7) = 0
                     rinputpst(8) = 0
                  end if
                  iinputpst(1) = 125
               else if ( ikeuze==18 ) then
                  write(*,*)'Give coordinates:'
                  ityprd = 1
                  call readsr ( rinputpst(51), 50, inum )
                  iinputpst(43) = inum/2
                  iinputpst(1) = 126
               else if ( ikeuze==19 ) then

!              --- OPEN PLOT
!                  Check value of jtimes, and set this value to 1

                  if ( jtimes/=10 ) then

!                 --- Error 433:  Plot file has already been opened before

                     call errwar ( 433, 0, 0, 0 )

                  else

                     possav = posdev
                     posdev = posold
                     iplotf = 0
                     jtimes = 11

                  end if
               else if ( ikeuze==20 ) then

!              --- CLOSE PLOT
!                  Check value of jtimes, and set this value to 3

                  if ( jtimes==11 ) then

!                 --- Error 276:  Plot file is opened and closed, without
!                                 making any plot

                     call errwar ( 1472, 0, 0, 0 )
                     jtimes = 10

                  else if ( jtimes==10 ) then

!                 --- Error 276:  Plot file has not been opened

                     call errwar ( 1473, 0, 0, 0 )

                  else

!                 --- Close Plot dataset

                     jtimes = 13
                     call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
                     jtimes = 10
                     close ( iref )
                     posdev = possav

!                    --- Display Plot

                     call plaf ( 1 )
                     iplotf = 1

                  end if
               else if ( ikeuze==21 ) then
                  write(*,*)'Give identification text:'
                  ityprd = 1
                  call asknxt
                  textid(1) = cardim
                  if ( textid(1)==' ' ) then
                     iinputpst(39) = 0
                  else
                     iinputpst(39) = lastp
                  end if
                  write(*,*)'Give coordinates of position of ', &
                            'identification text:'
                  ityprd = 1
                  call readsr ( rinputpst(7), 2, inum )
                  if ( inum==0 ) then
                     rinputpst(7) = 0
                     rinputpst(8) = 0
                  end if
                  iinputpst(1) = 127
               else if ( ikeuze==22 ) then
                  endsub = .true.
               end if
            else if ( ikeuze==6 ) then

!           --- get Vi number

               write(*,*)'Give number of Vi:'
               ityprd = 1
               call asknxt
               subsub = .true.
               istrf = int(realrd)
               if ( istrf<0 .or. istrf>1000 ) then

!              --- istrf out of range

                  call errint ( istrf, 1 )
                  call errint ( 1000, 2 )
                  call errsub ( 717, 2, 0, 0 )
                  call ercard ( kcolum, icolum )

               else
                  if ( irefar(istrf)==0 ) then

!                 --- irefar(istrf) = 0, so a new vector must be created

                     numvec = numvec + 1
                     if ( numvec>maxvc1 ) then

!                    --- Too much vectors per time step

                        call errint ( numvec, 1 )
                        call errint ( maxvc1, 2 )
                        call errsub ( 672, 2, 0, 0 )
                        call ercard ( kcolum, icolum )
                     else
                        irefar(istrf) = numvec
                     end if
                  end if
                     iinputpst(2) = irefar(istrf)
                  subsub = .false.
               end if

!              --- write Print menu

360            if ( .not. subsub ) then
                  if ( iinputpst(10)==1 ) then
                     call errchr ( 'ON', 1 )
                  else
                     call errchr ( 'OFF', 1 )
                  end if
                  call erreal ( xmin, 1 )
                  call erreal ( xmax, 2 )
                  call erreal ( ymin, 3 )
                  call erreal ( ymax, 4 )
                  call inicls
                  call ertext ( 115, 0, 4, 1 )

!                 --- Read answer

                  ityprd = 1
                  call asknxt
                  ikeuze = int(realrd)
                  if ( ikeuze==1 ) then
                     if ( iinputpst(10)==0 ) then
                        iinputpst(10) = 1
                     else
                        iinputpst(10) = 0
                     end if
                  else if ( ikeuze==2 ) then
                     write(*,*)'Give minimum of x-coordinate:'
                     ityprd = 1
                     call asknxt
                     xmin = realrd
                  else if ( ikeuze==3 ) then
                     write(*,*)'Give maximum of x-coordinate:'
                     ityprd = 1
                     call asknxt
                     xmax = realrd
                  else if ( ikeuze==4 ) then
                     write(*,*)'Give minimum of y-coordinate:'
                     ityprd = 1
                     call asknxt
                     ymin = realrd
                  else if ( ikeuze==5 ) then
                     write(*,*)'Give maximum of y-coordinate:'
                     ityprd = 1
                     call asknxt
                     ymax = realrd
                  else if ( ikeuze==6 ) then
                     subsub = .true.
                     iinputpst(1) = 2
                  end if

                  goto 360
               end if
            else if ( ikeuze==7 ) then

!           --- reset Colour Table

               call preset ( 2 )
            else if ( ikeuze==8 ) then

!           --- reset Plot Parameters Table

               call preset ( 1 )
               call initdf
               iinputpst(1) = -1
               rinputpst(10) = 0d0
               rinputpst(11) = 0d0
               rinputpst(12) = 0d0
            else if ( ikeuze==9 ) then

!           --- write Time= menu

               inum = 0
               subsub = .false.
               getal(1) = time(1)
               getal(2) = time(2)
               getal(3) = 1d0
390            if ( .not. subsub ) then
                  call erreal ( getal(1), 1 )
                  call erreal ( getal(2), 2 )
                  call errint ( int(getal(3)), 1 )
                  call inicls
                  call ertext ( 116, 1, 2, 0 )

!                 --- Read answer

                  ityprd = 1
                  call asknxt
                  ikeuze = int(realrd)
                  if ( ikeuze==1 ) then
                     write(*,*)'Give t0:'
                     ityprd = 1
                     call asknxt
                     getal(1) = realrd
                     inum = max ( inum, 1 )
                  else if ( ikeuze==2 ) then
                     write(*,*)'Give t1:'
                     ityprd = 1
                     call asknxt
                     getal(2) = int(realrd)
                     inum = max ( inum, 2 )
                  else if ( ikeuze==3 ) then
                     write(*,*)'Give tstep:'
                     ityprd = 1
                     call asknxt
                     getal(3) = realrd
                     inum = max ( inum, 3 )
                  else if ( ikeuze==4 ) then
                     subsub = .true.
                     t = getal(1)

!                    --- Find the corresponding time step itime

                     call prbisc ( t, time, iltime(1), ntime )
                     if ( inum==1 ) then

!                    --- inum = 1, so only one number:  TIME  = t0

                        iltime(2) = iltime(1)
                        iltime(3) = 1
                     else
                        t = getal(2)
                        call prbisc ( t, time, iltime(2), ntime )
                        if ( inum==2 ) then

!                       --- inum = 2, so two numbers:  TIME  = t0,t1

                           iltime(3) = 1
                        else

!                       --- inum = 3, so three numbers: TIME = (t0,t1,iltime)

                           iltime(3) = int(getal(3))

                        end if
                     end if
                  end if

                  goto 390
               end if
            else if ( ikeuze==10 ) then

!           --- Time History

!               --- get Vi number

               write(*,*)'Give number of Vi:'
               ityprd = 1
               call asknxt
               subsub = .true.
               istrf = int(realrd)
               if ( istrf<0 .or. istrf>1000 ) then

!              --- istrf out of range

                  call errint ( istrf, 1 )
                  call errint ( 1000, 2 )
                  call errsub ( 717, 2, 0, 0 )
                  call ercard ( kcolum, icolum )

               else
                  if ( irefar(istrf)==0 ) then

!                 --- irefar(istrf) = 0, so a new vector must be created

                     numvec = numvec + 1
                     if ( numvec>maxvc1 ) then

!                    --- Too much vectors per time step

                        call errint ( numvec, 1 )
                        call errint ( maxvc1, 2 )
                        call errsub ( 672, 2, 0, 0 )
                        call ercard ( kcolum, icolum )
                     else
                        irefar(istrf) = numvec
                     end if
                  end if
                  iinputpst(2) = irefar(istrf)
                  subsub = .false.
               end if

!              --- write Time History menu

               getal(1) = 0
               getal(2) = 0
               getal(3) = 0
               getal(4) = time(1)
               getal(5) = time(ntime)
               iinputpst(1) = 300
400            if ( .not. subsub ) then
                  call erreal ( getal(4), 1 )
                  call erreal ( getal(5), 2 )

                  call inicls
                  call ertext ( 117, 0, 2, 0 )

!                 --- Read answer

                  ityprd = 1
                  call asknxt
                  ikeuze = int(realrd)
                  if ( ikeuze==1 ) then
                     iinputpst(20) = 1
                     subsub = .true.
                  else if ( ikeuze==2 ) then
                     iinputpst(20) = 0
                     subsub = .true.
                  else if ( ikeuze==3 ) then
                     iinputpst(20) = 6
                     subsub = .true.
                  else if ( ikeuze==4 ) then
                     iinputpst(20) = 5
                     subsub = .true.
                  else if ( ikeuze==5 ) then
                     iinputpst(20) = 2
                     subsub = .true.
                     ikeuze = 0
401                  if ( ikeuze/=5 ) then
                        call errint ( iinputpst(8), 1 )
                        call erreal ( getal(1), 1 )
                        call erreal ( getal(2), 2 )
                        call erreal ( getal(3), 3 )
                        call inicls
                        call ertext ( 118, 1, 3, 0 )

!                       --- Read answer

                        ityprd = 1
                        call asknxt
                        ikeuze = int(realrd)
                        if ( ikeuze==1 ) then
                           write(*,*)'Give DEGFD:'
                           ityprd = 1
                           call asknxt
                           iinputpst(8) = int(realrd)
                        else if ( ikeuze==2 ) then
                           write(*,*)'Give x-coordinate of point:'
                           ityprd = 1
                           call asknxt
                           getal(1) = realrd
                        else if ( ikeuze==3 ) then
                           write(*,*)'Give y-coordinate of point:'
                           ityprd = 1
                           call asknxt
                           getal(2) = realrd
                        else if ( ikeuze==4 ) then
                           write(*,*)'Give z-coordinate of point:'
                           ityprd = 1
                           call asknxt
                           getal(3) = realrd
                        end if
                        goto 401
                     end if
                  else if ( ikeuze==6 ) then
                     iinputpst(20) = 11
                     subsub = .true.
                  else if ( ikeuze==7 ) then
                     iinputpst(20) = 10
                     subsub = .true.
                  else if ( ikeuze==8 ) then
                     iinputpst(20) = 16
                     subsub = .true.
                  else if ( ikeuze==9 ) then
                     iinputpst(20) = 15
                     subsub = .true.
                  else if ( ikeuze==10 ) then
                     iinputpst(20) = 12
                     subsub = .true.
402                  if ( ikeuze/=5 ) then
                        call errint ( iinputpst(8), 1 )
                        call erreal ( getal(1), 1 )
                        call erreal ( getal(2), 2 )
                        call erreal ( getal(3), 3 )
                        call inicls
                        call ertext ( 118, 1, 3, 0 )

!                       --- Read answer

                        ityprd = 1
                        call asknxt
                        ikeuze = int(realrd)
                        if ( ikeuze==1 ) then
                           write(*,*)'Give DEGFD:'
                           ityprd = 1
                           call asknxt
                           iinputpst(8) = int(realrd)
                        else if ( ikeuze==2 ) then
                           write(*,*)'Give x-coordinate of point:'
                           ityprd = 1
                           call asknxt
                           getal(1) = realrd
                        else if ( ikeuze==3 ) then
                           write(*,*)'Give y-coordinate of point:'
                           ityprd = 1
                           call asknxt
                           getal(2) = realrd
                        else if ( ikeuze==4 ) then
                           write(*,*)'Give z-coordinate of point:'
                           ityprd = 1
                           call asknxt
                           getal(3) = realrd
                        end if
                        goto 402
                     end if
                  else if ( ikeuze==11 ) then
                     write(*,*)'Give t0:'
                     ityprd = 1
                     call asknxt
                     getal(4) = realrd
                  else if ( ikeuze==12 ) then
                     write(*,*)'Give t1:'
                     ityprd = 1
                     call asknxt
                     getal(5) = realrd
                  else if ( ikeuze==13 ) then
                     subsub = .true.
                     iinputpst(1) = 300
                  end if

                  goto 400
               end if

!              --- Find the corresponding time step itime

               t = getal(4)
               call prbisc ( t, time, iltime(1), ntime )
               t = getal(5)
               call prbisc ( t, time, iltime(2), ntime )
            else if ( ikeuze==11 ) then
               possav = posdev
               posdev = 'p'
               iplotf = 0
               call plaf(1)
               iplotf = 1
               posdev = possav
            else if ( ikeuze==12 ) then
               possav = posdev
               posdev = 'p0'
               iplotf = 0
               call plaf(1)
               iplotf = 1
               posdev = possav
            else if ( ikeuze==13 ) then
               possav = posdev
               posdev = 'p'
               iplotf = 0
               call plaf(1)
               iplotf = 1
               posdev = possav
            else if ( ikeuze==14 ) then
               iinputpst(1) = 0
            end if

         end if

!        --- endwhile

         ready = ( iinputpst(1)/=-1 )
         goto 300
      end if

      call erclos ( 'poswin' )

      end
