      subroutine prbearing ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 01-09-2016 New parameter list
!        version  1.2  date 20-03-2015 Use prsetname
!        version  1.1  date 30-06-2011 Allow defualt values without input
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform the complete iteration for one time step
!     of the mass conservation scheme of Kumar
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_conservation
!     bearing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer,intent(inout) :: iinput(*)

!     iinput         i    integer input array
!                         Contains specific input for the bearing
!                         contents:
!                         1:  sequence number of bearing input
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: ninpbear=10, nrinbear=1
      integer :: inpbear(ninpbear), iseqnr, leninp, lenrnp
      double precision :: rinbear(nrinbear)

!                    they are not
!     inpbear        Integer input array for bearing subroutine
!                    See subroutine prbearmasscon for a description
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     leninp         Computed length of integer input array inpbear
!     lenrnp         Computed length of real input array rinbear
!     ninpbear       Declared length of integer input array inpbear
!     nrinbear       Declared length of real input array rinbear
!     rinbear        Real input array for bearing subroutine
!                    See subroutine prbearmasscon for a description
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: readconstvar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     IINPUTSEQ      Read contents of input block with sequence number iseqnr
!                    corresponding to main keyword name
!     PRBEARMASSCON  Perform the complete iteration for one time step
!                    of the mass conservation scheme of Kumar
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRSETNAME      Set sequence number of vector with specific name if it
!                    does not exist otherwise get sequence number
!     READCONSTVAR   Read constant (or variable) from  list of constants
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prbearing' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prbearing'
         call prinin ( iinput, 1, 'iinput' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Read input for bearing and store in array inpbear

      iseqnr = iinput(1)    ! sequence number input
      leninp = ninpbear
      lenrnp = nrinbear

      call iinputseq ( 30, iseqnr, inpbear, &
                       rinbear, leninp, lenrnp, 'bearing' )

      if ( leninp<=0 ) then

!     --- No input block found, use defaults, see subroutine readbearing

         inpbear = 1
         call prsetname ( 'pressure', inpbear(3) )

         call prsetname ( 'reac_force', inpbear(5) )
         inpbear(6) = 10
         inpbear(7) = 0
         inpbear(9:10) = 0

         rinbear(1) = readconstvar ( 'p_cavity', -1d10, 1d10 )

      else

!     --- Input block found

         if ( debug ) then
            call prinin ( inpbear, leninp, 'inpbear' )
            call prinrl ( rinbear, lenrnp, 'rinbear' )
         end if  ! ( debug )

      end if  ! ( inpbear<=0 )

      call prbearmasscon ( inpbear, rinbear )

      istartmn = istartmn + 2

1000  call erclos ( 'prbearing' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prbearing'

      end if

      end
