      subroutine prbisc( t, time, itimelc, ntime )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.1  date 26-10-1990 Delete names
!        version  1.0  date 16-08-1988
!
!   copyright (c) 1988-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Help subroutine for subroutine READ05
!     computes the time step itimelc that is closest to the
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itimelc, ntime
      double precision time(1000), t

!    itimelc     o   Actual time step corresponding to t
!    ntime     i   Number of time steps
!    t         i   Actual time, from which itimelc must be computed
!    time      i   Array of length 1000 containing the actual time at the
!                  various time steps
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iunder, iupper, ihalf

!     ihalf          Value half between iunder and iupper
!     iunder         Under boundary for itimelc
!     iupper         Upper boundary for itimelc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prbisc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prbisc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call inichkarlen ( ntime, 1000, 'time' )
      if ( ierror/=0 ) go to 1000

      if ( ntime<=1 ) then

!     --- ntime<=1, no time dependence

         itimelc = 1

      else

         iunder = 1
         iupper = ntime

!        --- Loop

100      if ( iupper - iunder==1 ) then

!        --- iupper - iunder = 1, find which one is closest to t

            if ( abs ( t-time(iupper) )<abs ( t -time(iunder) ) ) then

!           --- | t - time(iupper) |<| t - time(iunder) |

               itimelc = iupper
            else

!           --- | t - time(iupper) |>| t - time(iunder) |

               itimelc = iunder
            end if
         else

!        --- iupper - iunder>1, bisection

            ihalf = ( iupper - iunder ) / 2 + iunder
            if ( time(ihalf)<=t ) then

!           --- |time(ihalf)|<t, raise iunder

               iunder = ihalf
            else

!           --- |time(ihalf)|>t, raise iupper

               iupper = ihalf
            end if
            go to 100
         end if
      end if

1000  call erclos ( 'prbisc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prbisc'

      end if  ! ( debug )

      end
