      subroutine prchanprob ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 25-11-2012 Long integers
!        version  1.2  date 06-02-2012 Allow indexed vectors
!        version  1.1  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Change the problem number of the vector isolut2d if possible
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulemnsub
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    Integer input array that defines the actions to be
!                         performed.
!                         Contents:
!                         Pos. 1  not used
!                         Pos. 2  New problem number for the solution vector
!                         Pos. 3  Vector sequence number with respect to isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivecseq, i, ivec, kpivec, itypvc

!     i              Counting variable
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     ivecseq        Sequence number of vector to be considered with respect
!                    to the solution array (isol or isolut2d)
!     kpivec         Starting address of information concerning arrays of
!                    special structure in KPROB
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIVECTR       Return with sequence number of vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2114   Vector is not of type 110 or 115
!    2115   Length of vector for both problems numbers not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prchanprob' )
      iprob = inpsep(2)
      ivecseq = inivectr(inpsep(3))
      itypvc = ks(isolut2d(ivecseq))%typevec
      if ( itypvc/=110 .and. itypvc/=115 ) then

!     --- Only vectors of type 110 or 115 permitted

         call errint ( ivecseq, 1 )
         call errint ( itypvc, 2 )
         call errchr ( names(ivecseq), 1 )
         call errsub ( 2114, 2, 0, 1 )

      end if
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kprobd => kprobpart%kprobd
      kprobe2 => kprobpart%kprobe2
      kprobe3 => kprobpart%kprobe3
      if ( itypvc==110 ) then

!     --- Solution vector

         nusol = kprobloc(5)
         ivec = 0

      else

!     --- Vector of special structure

         ivec = ks(isolut2d(ivecseq))%ivec
         nphys = kprobe2(ivec)
         if ( nphys<0 ) then

!        --- nphys<0, constant number of degrees of freedom per point

            nphys = -nphys
            nusol = nphys * npoint

         else

!        --- nphys>0, number of degrees of freedom per point not constant

            indprf = nphys
            nphys = maxval(kprobe(:,ivec))
            kprobf => kprobe3(ivec)%arr
            nusol = kprobf(1+npoint)-kprobf(1)

         end if
         if ( nusol/=ks(isolut2d(ivecseq))%nusol ) then

!        --- Length for both problem numbers not the same

            call errint ( ivecseq, 1 )
            call errint ( isolut2d(ivecseq), 2 )
            call errint ( nusol, 3 )
            call errint ( ks(isolut2d(ivecseq))%nusol, 4 )
            call errchr ( names(ivecseq), 1 )
            call errsub ( 2115, 4, 0, 1 )

         end if

      end if

!     --- Change problem number

      ks(isolut2d(ivecseq))%iprob = iprob
      ks(isolut2d(ivecseq))%ivec = ivec
      istartmn = istartmn + 4

      call erclos ( 'prchanprob' )
      end
