      subroutine prcomm ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 20-08-2016 use sepmoduleinput
!        version  3.0  date 17-12-2015 New parameter list
!        version  2.6  date 25-11-2012 Long integers
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Call subroutine COMMAT to change the structure of the large matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep  i    Sequence number of input for COMMAT
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lniincommat
      parameter ( lniincommat=25 )
      integer iincop(lniincommat), jmetod, &
              ipos, i, iseqintmat, lencheck, maxcal

!     i              General loop parameter
!     iincop         Copy of input array extended with defaults
!     ipos           Position of input arrays in sepmoduleinput
!     iseqintmat     Sequence number with respect to array intmat
!     jmetod         Indicates type of storage scheme for the matrix
!     lencheck       Length of iincommt as stored in ibuffr
!     lniincommat    Declared length of iincommt
!     maxcal         Maximum number of coefficient inputs read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHINPSOL    Adapt input array with respect to variables
!     PRININ         print 1d integer array
!     SEPCOMMAT      Actual body of COMMAT and MATSTRUC
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2896   Parameters must be equal
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get jmetod for sequence number inpsep
!     For iprob := 1 (1) nprob do
!        if ( jmetod(iprob)>0 ) then
!           call incommat
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcomm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcomm'
         call prinin ( inpsep, 2, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get jmetod for sequence number inpsep

      call sepgetinpfinf ( 3, inpsep(1), ipos, maxcal )
      lencheck = size(in(ipos)%iinput)
      if ( lencheck/=lniincommat*nprob ) then

!     --- Error lencheck and lniincommat must be equal

         call errint ( lencheck, 1 )
         call errint ( lniincommat*nprob, 2 )
         call errchr ( 'lencheck', 1 )
         call errchr ( 'lniincommat', 2 )
         call errsub ( 2896, 2, 0, 2 )
         go to 1000

      end if  ! ( lencheck/=lniincommat )

      iseqintmat = inpsep(2)-1

!     --- Loop over problems

      do iprob = 1, nprob
         jmetod = in(ipos)%iinput((iprob-1)*lniincommat+2)

         if ( debug ) write(irefwr,1) 'iprob, jmetod', iprob, jmetod
         if ( jmetod>0 ) then
            do i = 1, lniincommat
               iincop(i) = in(ipos)%iinput((iprob-1)*lniincommat+i)
            end do

!           --- Adapt variables

            call inichinpsol ( iincop, lniincommat )
            intmat => intmat1d(iprob+iseqintmat)
            call sepcommat ( iincop )
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcomm'

      end if  ! ( debug )
      call erclos ( 'prcomm' )

      end
