      subroutine prcompcoor ( coor1, coor2, npoint, inodes1, inodes2, &
                              ndim, same )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compare coordinates in two meshes
!     node numbers may be different
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     compare
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, inodes1(npoint), inodes2(npoint), ndim
      double precision coor1(ndim,npoint), coor2(ndim,npoint)
      logical same

!     coor1          i    Array containing the coordinates of the first mesh
!     coor2          i    Array containing the coordinates of the second mesh
!     inodes1        i    Ordering of coor1 for increasing coordinates
!     inodes2        i    Ordering of coor2 for increasing coordinates
!     ndim           i    Dimension of the space.
!     npoint         i    Number of nodal points
!     same           o    If true the arrays are the same
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, l, i, j
      double precision eps

!     eps            Local accuracy
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompcoor' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompcoor'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find maximum value of coordinate in coor1, to compute accuracy

      eps = maxval ( abs(coor1) ) * 1d-10
      same = .true.

      do i = 1, npoint
         k = inodes1(i)
         l = inodes2(i)
         do j = 1, ndim

            if ( abs(coor1(j,k)-coor2(j,l))>eps ) then

!           --- Distance larger than eps

               if ( .not. same ) &
                  write(irefwr,*) 'Coordinates are not the same'
               same = .false.
               write(irefwr,2) 'coor1', coor1(:,k)
               write(irefwr,2) 'coor2', coor1(:,l)
               go to 1000

            end if  ! ( abs(coor1(j,k)-coor2(j,l))>eps )

         end do  ! j = 1, ndim

      end do  ! i = 1, npoint

1000  call erclos ( 'prcompcoor' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompcoor'

      end if  ! ( debug )

      end
