      subroutine prcompenthalpy ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 17-09-2004 Complete revision
!        version  1.0  date 09-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the enthalpy, given the temperature
!
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     temperature
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)

!     iinput         i    Integer input array with the following contents
!                         1: IREAD Defines how the input must be read
!                            Possible values:
!                            0   The input is read from the standard input file
!                            >0  The input as read from the standard input file
!                                by subroutine SEPSTM is used. The value of
!                                iread defines the sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinpent, nrinpent
      parameter ( niinpent=10, nrinpent=10 )
      integer iinenthalpy(niinpent), iseqnr, leninp, lenrnp, &
              solutionmethod
      double precision rinenthalpy(nrinpent)

!                    they are not
!     iinenthalpy    Integer input array with respect to enthalpy
!                    computations
!                    For a definition of this array see subroutine
!                    readenthalpy (array iinput)
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     leninp         Computed length of integer input array
!     lenrnp         Computed length of real input array
!     niinpent       Declared length of array iinenthalpy
!     nrinpent       Declared length of array rinenthalpy
!     rinenthalpy    Real input array with respect to enthalpy
!                    computations
!                    For a definition of this array see subroutine
!                    readenthalpy (array iinput)
!     solutionmethod Defines type of solution method
!                    1: overrelaxation
!                    2: newton
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     IINPUTSEQ      Read contents of input block with sequence number iseqnr
!                    corresponding to main keyword name
!     PRCOMPENTHALP1 Compute the enthalpy, given the temperature
!                    Overrelaxation approach
!     PRCOMPENTHALP2 Compute the enthalpy, given the temperature
!                    Newton approach
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read input for enthalpy problem and store in arrays
!     iinenthalpy, rinenthalpy
!     Compute enthalpy depending on solutionmethod
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompenthalpy' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompenthalpy'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Read input for enthalpy problem and store in arrays
!         iinenthalpy, rinenthalpy

      iseqnr = iinput(1)
      leninp = niinpent
      lenrnp = nrinpent
      call iinputseq ( 26, iseqnr, iinenthalpy, &
                       rinenthalpy, leninp, lenrnp, 'COMPUTE_ENTHALPY' )
      if ( ierror.ne.0 ) go to 1000
      if ( debug ) call prinin ( iinenthalpy, 6, 'iinenthalpy' )

      solutionmethod = iinenthalpy(6)
      if ( solutionmethod==1 ) then

!     --- Overrelaxation

         call prcompenthalp1 ( isolut2d, rinenthalpy, numvecmn, iinenthalpy )

      else if ( solutionmethod==2 ) then

!     --- Newton

         call prcompenthalp2 ( numvecmn, iinenthalpy, 41 )

      else

!     --- Wrong value

         call errchr ( 'solutionmethod', 1 )
         call errint ( solutionmethod, 1 )
         call errint ( 1, 2 )
         call errint ( 2, 3 )
         call errsub ( 1378, 3, 0, 1 )

      end if  ! ( solutionmethod==1 )
      istartmn = istartmn + 2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompenthalpy'

      end if
      call erclos ( 'prcompenthalpy' )
      end
