      subroutine prcopycoor1 ( usol, kprobf, indprf, nphys )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 03-12-2013 Extra debug
!        version  1.1  date 11-04-2008 Correction coor(i,j)
!        version  1.0  date 30-01-2008
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates into a vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: kprobf(*), indprf, nphys
      double precision :: usol(*)

!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     usol           o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, istart

!     i              Counting variable
!     istart         First position used at a part
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopycoor1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopycoor1'
         write(irefwr,1) 'indprf, npoint, ndim', indprf, npoint, ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprf==0 ) then

!     --- indprf = 0, constant number of degrees of freedom

         do i = 1, npoint

            do j = 1, ndim
               usol((i-1)*nphys+j) = coor(j,i)
            end do  ! j = 1, ndim

         end do  ! i = 1, npoint

      else

!     --- indprf>0, variable number of degrees of freedom
!         Loop over all nodes

         istart = kprobf(1)  ! first position in kprobf
         do i = 1, npoint

            if ( kprobf(i+1)-istart>0 ) then

!           --- There is at least one unknown in node

               do j = 1, ndim
                  usol(istart+j) = coor(j,i)
               end do  ! j = 1, ndim

            end if  ! ( inunkp>0 )
            istart = kprobf(i+1)  ! next value in kprobf

         end do  ! i = 1, npoint
         if ( debug ) call prinrl ( usol, istart, 'coor_cop' )

      end if  ! ( indprf==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopycoor1'

      end if  ! ( debug )
      call erclos ( 'prcopycoor1' )

      end
