      subroutine prcopycoorbck ( iseqcoor, iseqinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 30-12-2013 New definition of vectors
!        version  2.1  date 25-11-2012 Long integers
!        version  2.0  date 18-01-2012 Extra parameter iseqinp
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Copy coordinates stored in solution vector back
!      into coordinates array
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     coordinate
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqcoor, iseqinp

!     iseqcoor       i    Sequence number of coordinates vector is isolut2d
!     iseqinp        i    Sequence number of input vector in isolut2d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRCOPYCOORBCK1 Copy coordinates from a vector into coor
!                    All components
!     PRCOPYCOORBCK2 Copy coordinates from a vector into coor
!                    One component only
!     PRININ         print 1d integer array
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopycoorbck' )
      debug = .false.  .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopycoorbck'
         call prinin ( isolut2d(iseqcoor), 5, 'isolut2d(iseqcoor)' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepactsolbf2 ( isolut2d(iseqcoor) )
      usoltemp => ks(isolut2d(iseqcoor))%sol
      if ( ierror/=0 ) go to 1000

!     --- Copy

      if ( iseqinp<0 .or. iseqinp>maxvc1+3 ) then

!     --- Error: iseqinp out of range

         call errint ( iseqinp, 1 )
         call errint ( maxvc1+1, 2 )
         call errint ( maxvc1+ndim, 3 )
         call errchr ( 'iseqinp', 1 )
         call errsub ( 1378, 3, 0, 1 )
         go to 1000

      else if ( iseqinp==maxvc1+1 ) then

!     --- iseqinp = -1000 coordinates

         call prcopycoorbck1 ( usoltemp, kprobfi, indprfi, nphysi )

      else

!     --- iseqinp = -1001 ... -1003 Hence one of the coordinates

         call prcopycoorbck2 ( usoltemp, kprobfi, indprfi, &
                               nphysi, iseqinp-maxvc1-1 )

      end if  ! ( iseqinp==-1000 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopycoorbck'

      end if  ! ( debug )
      call erclos ( 'prcopycoorbck' )

      end subroutine prcopycoorbck
