      subroutine prcorrinterp11 ( istore, lensto, iboun, ibounnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate special points that where outside the computational
!     region before move of boundary and are inside afterwards
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     level_set
!     node
!     intersection
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(*), lensto, iboun, ibounnew

!     iboun          i    Node number corresponding to levelset zero (old mesh)
!     ibounnew       i    Node number corresponding to levelset zero (new mesh)
!     istore         i    Contains the nodes that should be adapted
!     lensto         i    Number of nodes in istore
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, i

!     i              Counting variable
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all nodes node in store do
!        Copy boundary value of uold into unew
!        Adpat mesh velocity
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcorrinterp11' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcorrinterp11'
         call prinin ( istore, lensto, 'istore' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First fill the concentration at the interface and
!         compute the mesh velociy at the interface

      unew(ibounnew) = uold(iboun)

!     --- For all nodes node in store do

      do i = 1, lensto

!     --- Compute coordinate of node in old mesh by x_node = meshvel_node*dt

         node = istore(lensto)
         unew(node) = uold(iboun)
         if ( debug ) then
            write(irefwr,1) 'i, node, iboun', i, node, iboun
            write(irefwr,2) 'coornew(1,node), coorold(1,iboun)', &
                             coornew(1,node), coorold(1,iboun)
            write(irefwr,2) 'unew(node)', unew(node)
         end if  ! ( debug )

      end do  ! i = 1, lensto

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcorrinterp11'

      end if  ! ( debug )
      call erclos ( 'prcorrinterp11' )

      end
