      subroutine prcorrinterp3 ( coorold, interfaceold, nofaceold, &
                                 coornew, interfacenew, nofacenew, &
                                 oldnodes, meshvel, dt, lensto, &
                                 istore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-04-2013 New parameter list
!        version  1.0  date 24-02-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Correct interpolation in case nodes of the mesh crossed
!     the old object (3d)
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     level_set
!     node
!     intersection
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nofaceold, interfaceold(nofaceold), nofacenew, &
              interfacenew(nofacenew), lensto, istore(*), oldnodes(*)
      double precision coorold(3,*), coornew(3,*), meshvel(3,*), dt

!     coornew        i    Array containing the new coordinates of the nodes
!     coorold        i    Array containing the coordinates in the old mesh
!     dt             i    Time step
!     interfacenew   i    Contains node numbers of new interface
!     interfaceold   i    Contains node numbers of old interface
!     istore         i    work array of length npoint to store nodes for which
!                         phi1 and phi2 have different sign
!     lensto         i    Number of nodes in istore
!     meshvel       i/o   Contains mesh velocity
!     nofacenew      i    Number of nodes in interfacenew
!     nofaceold      i    Number of nodes in interfaceold
!     oldnodes       o    Contains the set of old nodes corresponding to
!                         the nodes in interfacenew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRLEVINTERFCP3 Interpolate the concentration at the new interface from
!                    values at the old interface in case of a level set method
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcorrinterp3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcorrinterp3'
         write(irefwr,1) 'lensto', lensto
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First fill the concentration at the interface and
!         compute the mesh velocity at the interface

      if ( debug ) write(irefwr,*) 'first call to prlevinterfcp'
      call prlevinterfcp3 ( coorold, interfaceold, nofaceold, &
                            coornew, interfacenew, nofacenew, &
                            oldnodes, meshvel, dt )

      if ( lensto>0 ) then

!     --- Lensto>0, it must also be done for nodes outside the interface

         if ( debug ) write(irefwr,*) 'second call to prlevinterfcp'
         call prlevinterfcp3 ( coorold, interfaceold, nofaceold, &
                               coornew, istore, lensto, &
                               oldnodes(nofacenew+1), meshvel, dt )

      end if  ! ( lensto>0 )
1000  call erclos ( 'prcorrinterp3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcorrinterp3'

      end if  ! ( debug )

      end
