      subroutine prfindclosnod1 ( cells, nactive, markcells, nx, ny )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-03-2013 Correction declaration
!        version  1.0  date 16-11-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array cells with neighbors of old cells which have not been
!     treated yet
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     cell
!     find
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nactive, cells(2,*), nx, ny, markcells(nx,ny)

!     cells         i/o   At input: all active cells
!                         At output all non-marked neighbors of these cells
!     markcells     i/o   Array of size nx * ny to mark cells that have
!                         already been used
!     nactive       i/o   Number of active cells
!                         At input the old number
!                         At output the new one
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nactcp, cellscop(2,nactive), indi, indj, k, i, j

!     cellscop       Copy of the input array cells
!     i              Counting variable
!     indi           i-index
!     indj           j-index
!     j              Counting variable
!     k              Counting variable
!     nactcp         copy of input value of nactive
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindclosnod1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindclosnod1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy old array cells in cellscop

      nactcp = nactive
      cellscop(1:2,1:nactive) = cells(1:2,1:nactive)

!     --- Fill array cells with neighbors of old cells which have not been
!         treated yet

      nactive = 0

      do k = 1, nactcp

         indi = cellscop(1,k)
         indj = cellscop(2,k)

         do i = max(1,indi-1), min(nx,indi+1)
            do j = max(1,indj-1), min(ny,indj+1)

               if ( markcells(i,j)==0 ) then

!              --- New cell found, store

                  markcells(i,j) = 1
                  nactive = nactive+1
                  cells(1,nactive) = i
                  cells(2,nactive) = j

               end if  ! ( markcells(i,j)==0 )

            end do  ! j = max(1,indj-1), min(ny,indj+1)
         end do  ! i = max(1,indi-1), min(nx,indi+1)

      end do  ! k = 1, nactcp

1000  call erclos ( 'prfindclosnod1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindclosnod1'

      end if  ! ( debug )

      end
