      subroutine priavs ( jtime, iseq, time, nameavs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.6  date 09-10-2013 Use inifilename
!        version  2.5  date 10-04-2014 Extra debug
!        version  2.4  date 23-01-2014 extension filenm
!
!   copyright (c) 1997-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Write solution array or array of special structure to file sepavs_priavs.yyy
!   This possibility is meant for a SEPRAN-AVS interface
!   The SEPRAN mesh is mapped onto a mesh of linear triangles or tetrahedrons
!   only
! **********************************************************************
!
!                       KEYWORDS
!
!     avs
!     output
!     vector
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: jtime, iseq, nameavs
      double precision time

!     iseq           i    Sequence number of vector to be written
!     jtime          i    Integer time sequence number
!     nameavs        i    Sequence number of name of avs file in array
!                         file_names
!     time           i    Actual time
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=110) filenm
      character (len=100) filename
      integer iresul(5,3), nphys, iref, nchar, npelmlc, nelemlc

!     filename       Name of file
!     filenm         Name of file
!     iref           File reference number
!     iresul         Output array of length 5,1 if ichois=2 and otherwise of
!                    length 5,3
!                    The array is filled as follows:
!                    ,1 corresponds to the "vertex" mesh array (ichois=2,3)
!                    ,2 corresponds to the "triangle" mesh array (ichois=1,3)
!                    ,3 corresponds to the solution array (ichois=1,3)
!                    1,1  INDVER  sequence number with respect to INFOR
!                                 for the "vertex" mesh array
!                    2,1  NPELM   number of points per element for this mesh
!                    3,1  NELEM   number of points per element for this mesh
!                    4,1  NEWVER  indicates if the mesh is new (1) or is
!                                 in fact the standard mesh (0)
!                    5,1  - &
!                    1,2  INDTRI  sequence number with respect to INFOR
!                                 for the "triangle" mesh array
!                    2,2  NPELM   number of points per element for this mesh
!                    3,2  NELEM   number of points per element for this mesh
!                    4,2  NEWTRI  indicates if the mesh is new (1) or is
!                                 in fact the standard mesh (0) or the mesh
!                                 created by the "vertex" mesh
!                    5,2  NPHYS   Max. number of degrees of freedom
!                    1,3  INDSOL  sequence number with respect to INFOR
!                                 for the new solution array
!                    2,3  NEWSOL  indicates if the solution array is new (1) or
!                                 is in fact the standard solution array (0)
!                    3,3  INDPRF  sequence number of KPROBF
!                    4,3  INDPRH  sequence number of KPROBH
!                    5,3  INPPRP  sequence number of KPROBP
!     nchar          Number of characters in a word
!     nelemlc        Number of elements
!     npelmlc        Number of points per element
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIFILENAME    Create filename from indfile and namefile
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     PRAVS0         Actual writing of solution array for AVS
!     PRAVS4         Map mesh on appropriate mesh and map solution on this mesh
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
!    Output is written to the file sepavs_priavs.yyy
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Open file sepavs_priavs.yyy
!   Map mesh on appropriate mesh and map solution on this mesh
!   Write solution and mesh information to sepavs_priavs.yyy
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'priavs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from priavs'
         write(irefwr,1) 'nameavs, jtime, iseq', nameavs, jtime, iseq
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Open file sepavs_priavs.yyy

      iref = inisetnewref()
      call inifilename ( nameavs, 'sepavs', filename, nchar )
      nchar = nchar+1
      filename(nchar:nchar) = '_'

      if ( debug ) then

         write(irefwr,1) 'nchar', nchar
         write(irefwr,*) 'filename ', filename(1:nchar)

      end if  ! ( debug )

      write ( filenm, 100 ) filename(1:nchar), jtime, iseq
100   format ( a,i3.3,'.',i3.3 )
      open ( unit=iref, file=filenm )

!     --- Map mesh on appropriate mesh and map solution on this mesh

      call pravs4 ( iresul, nphys )
      if ( ierror/=0 ) go to 1000

!     --- Write solution and mesh information to sepavs_priavs.yyy

      usoltemp => ks(iresul(1,3))%sol
      npelmlc = iresul(2,2)
      nelemlc = iresul(3,2)

      if ( debug ) &
         write(irefwr,1) 'ndim, npoint, nelemlc, npelmlc, nphys, iseq', &
                          ndim, npoint, nelemlc, npelmlc, nphys, iseq

      call pravs0 ( iref, coor, ndim, npoint, nelemlc, npelmlc, &
                    kmeshc, usoltemp, nphys, iseq, time )
      close ( iref )
      call inirmref ( iref )
      if ( iresul(2,3)/=0 ) call seprmsolinfo ( iresul(1,3) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End priavs'

      end if  ! ( debug )
      call erclos ( 'priavs' )

      end subroutine priavs
