      subroutine printb ( itim, ichois, funcx, funcy, ndim, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 14-10-2004 Extension with ichois = 3
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.0  date 20-08-1988
!
!   copyright (c) 1988-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Help subroutine for program SEPPOST
!   This subroutine is used to print results of the time history
!   during each time-step
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itim, ichois, ndim
      double precision coor(ndim), funcx, funcy

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     funcx      i    X-value to be printed  (usually the time)
!     funcy      i    Y-value to be printed
!     ichois     i    Choice parameter indicating which quantity is printed
!                     Possible values:
!                     0   Print max Vi
!                     1   Print min Vi
!                     5   Print max abs( Vi )
!                     6   Print min abs( Vi )
!     itim       i    Relative time-step number (>= 1 )
!     ndim       i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!      i        Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( itim.eq.1 ) then

!     --- itim = 1,  Print heading

         if ( ichois.eq.0 ) then
            write(irefwr, 100)
100         format ( ' Time      Max (Vi)             Co-ordinates ')
         else if ( ichois.eq.1 ) then
            write(irefwr, 110)
110         format ( ' Time      Min (Vi)             Co-ordinates ')
         else if ( ichois.eq.2 ) then
            write(irefwr, 120)
120         format ( ' Time      solution             Co-ordinates ')
         else if ( ichois.eq.3 ) then
            write(irefwr, 130)
130         format ( ' Time      x-value ')
         else if ( ichois.eq.5 ) then
            write(irefwr, 150)
150         format ( ' Time      Max abs (Vi)             Co-ordinates')
         else if ( ichois.eq.6 ) then
            write(irefwr, 160)
160         format ( ' Time      Min abs (Vi)             Co-ordinates')
         end if
      end if
      write ( irefwr, 200 ) funcx, funcy, ( coor(i), i = 1,ndim )
200   format ( d10.3, 6d14.7 )
      end
