      subroutine printimehis1 ( idegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 16-04-2014 Extra parameters idegfd, nphysi
!        version  1.0  date 19-05-2010
!
!   copyright (c) 2010-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write contents of time history array to file
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmodulecons
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idegfd

!     idegfd         i    Degree of freedom sequence number
!                         in the time history
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printimehis1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printimehis1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( idegfd==0 ) then

!     --- idegfd = 0, print all components

         do i = 1, istephis
            write(irefwr,100) rvectim(1:npoint_hist*nphysi+1,i)
100         format ( 40(1pe12.4e2) )
         end do  ! i = 1, nstephis

      else

!     --- idegfd>0, print one component

         do i = 1, istephis
            write(irefwr,100) rvectim(1,i), &
              (rvectim(1+idegfd+(j-1)*nphysi,i), j=1, npoint_hist)
         end do  ! i = 1, nstephis

      end if  ! ( idegfd==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printimehis1'

      end if  ! ( debug )
      call erclos ( 'printimehis1' )

      end
