      subroutine prlistphipnt2d ( lensto, istore, interfacenew, nofacenew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-12-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with set of nodes with different phi signs
!     2d only  Interface points are exluded
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     level_set
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lensto, istore(*), interfacenew(*), nofacenew

!     interfacenew   i    Contains node numbers of new interface
!     istore         o    work array of length npoint to store nodes for which
!                         phi1 and phi2 have different sign
!     lensto         o    Number of nodes in istore
!     nofacenew      i    Number of nodes in interfacenew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     iwork          integer work array of length npoint (basis mesh)
!                    iwork(i) = 1 means node is on interface: 0 not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prlistphipnt2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prlistphipnt2d'
         call prinrl ( phi1, npoint, 'phi1' )
         call prinrl ( phi2, npoint, 'phi2' )
         call prinin ( interfacenew, nofacenew, 'interfacenew' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lensto = 0

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iwork = 0  ! Clear iwork

!     --- Set all iwork = 1 for all nodes in interfacenew

      do i = 1, nofacenew
         if ( interfacenew(i)<=npoint ) iwork(interfacenew(i)) = 1
      end do  ! i = 1, nofacenew

!     --- For all nodes node in original mesh do

      do i = 1, npoint

         if ( iwork(i)==0 ) then

            if ( phi1(i)*phi2(i)<=0d0 ) then

!           --- phi1 and phi2 have different sign, store node

               lensto = lensto+1
               istore(lensto) = i

            end if  ! ( phi1(i)*phi2(i)<=0d0 ) then

         end if  ! ( iwork(i)==0 )

      end do  ! i = 1, npoint

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prlistphipnt2d'

      end if  ! ( debug )
      call erclos ( 'prlistphipnt2d' )

      end
