      subroutine prlistphipoint ( npoint, lensto, istore, iboun )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-11-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with set of nodes with different phi signs
!     1d only  Interface points are exluded
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     level_set
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, lensto, istore(*), iboun

!     iboun          i    boundary point where level set equals 0 (new mesh)
!     istore         o    work array of length npoint to store nodes for which
!                         phi1 and phi2 have different sign
!     lensto         o    Number of nodes in istore
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all nodes node in original mesh do
!        if ( phi1(node)*phi2(node)<0 ) then
!           add node to list
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prlistphipoint' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prlistphipoint'
         call prinrl ( phi1, npoint, 'phi1' )
         call prinrl ( phi2, npoint, 'phi2' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lensto = 0

!     --- For all nodes node in original mesh do

      do i = 1, npoint

         if ( i/=iboun .and. phi1(i)*phi2(i)<=0d0 ) then

!        --- phi1 and phi2 have different sign, store node

            lensto = lensto+1
            istore(lensto) = i

         end if  ! ( phi1(i)*phi2(i)<-1d-6 )

      end do  ! i = 1, npoint

1000  call erclos ( 'prlistphipoint' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prlistphipoint'

      end if  ! ( debug )

      end
