      subroutine prmakesignvec2 ( uin, uout, n, umult, nunkp, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-05-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new vector V by either multiplication of the sign of
!     input vector 1 by input vector 2
! **********************************************************************
!
!                       KEYWORDS
!
!     sign
!     multiplication
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, nunkp, inpsep(*)
      double precision uout(nunkp,n), uin(n), umult(nunkp,n)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prmanv for a description
!     n              i    number of elements in sign vector
!     nunkp          i    Maximum number of unknowns in a point
!     uin            i    sign vector
!     umult          i    Multiplication vector
!     uout           o    Output vector (sign of uin times umult)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ispec

!     i              Counting variable
!     ispec          Parameter in inpsep denoting special options
!                      0:  Compute V1 * V2
!                      1:  Compute sign(V1) * V2
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakesignvec2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakesignvec2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ispec = inpsep(7)       ! ispec, defines type of multiplication

      select case ( ispec )

      case (0)

!     --- V1 * V2

         do j = 1, nunkp

            do i = 1, n
               uout(j,i) = uin(i)*umult(j,i)
            end do  ! i = 1, n

         end do  ! j = 1, nunp

      case (1)

!     --- sign(V1) * V2

         do j = 1, nunkp

            do i = 1, n
               if ( abs(uin(i))> 0d0 ) then
                  uout(j,i) = sign ( 1d0, uin(i) )*umult(j,i)
               else
                  uout(j,i) = 0d0
               end if  ! ( abs(uin(i))> 0d0 ) then
            end do  ! i = 1, n

         end do  ! j = 1, nunp

      end select  ! case ( ispec )

1000  call erclos ( 'prmakesignvec2' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakesignvec2'

      end if  ! ( debug )

      end
