      function prminarea ( )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 20-07-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the minimal area of all elements of highest dimension in mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     volume
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision prminarea

!     prminarea     o    Computed mean area
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision minar
      double precision, allocatable :: areas(:)

!     areas          Contains the areas of the elements
!     i              Counting variable
!     minar          Help parameter to compute minimal area
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRAREAS        Compute areas of all elements of highest dimension
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prminarea' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prminarea'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- allocate work array

      allocate ( areas(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'areas' )

!     --- Compute areas of all elements of highest dimension

      call prareas ( areas )

!     --- Compute minimum of these areas and number of elements of highest
!     --- dimension

      minar = 1d10   ! computes minar of areas

      do i = 1, nelem

         if ( areas(i)>0d0 ) then

!        --- area>0, hence element of highest dimension

            minar = min(minar,areas(i))

         end if  ! ( areas(i)>0d0 )

      end do  ! i = 1, nelem

!     --- Compute mean value

      prminarea = minar

!     --- deallocate work array

      deallocate ( areas, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'areas' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prminarea'

      end if  ! ( debug )
      call erclos ( 'prminarea' )

      end function prminarea
