      subroutine probobstnew ( meshorig, meshobst, ireadstruc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 11-12-2017 New storage of intmat2d
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 22-11-2011
!
!   copyright (c) 2011-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob and intmat2d for new mesh in case of obstacle or level set
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     level_set
!     obstacle
!     problem_definition
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshorig, meshobst, ireadstruc

!     ireadstruc     i    Sequence number of input for matrix structure
!     meshobst       i    Sequence number of obstacle mesh to be used
!     meshorig       i    Sequence number of original mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idum(1), iseqprob
      logical :: debugloc

!     debugloc       If true local debug statements are carried out otherwise
!                    they are not
!     idum           General dummy variable
!     iseqprob       Sequence number of problem input
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCOMGEN       Compute matrix structure for free boundary problem
!     PROBDFEXT      Reads and stores problem definition
!     PROCOPY1       Copy important information of problem definition
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPPRINKPROB   Print array KPROB
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probobstnew' )
      debug = .false. .and. ioutp>=0
      debugloc = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probobstnew'
         write(irefwr,1) 'ireadstruc', ireadstruc
         write(irefwr,1) 'meshnr, meshobst, meshorig', &
                          meshnr, meshobst, meshorig
         write(irefwr,1) 'kmesh2d(meshobst)', kmesh2d(meshobst)
         write(irefwr,1) 'kprob2d(meshorig), kprob2d(meshobst)', &
                          kprob2d(meshorig), kprob2d(meshobst)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy old kprob to new one

      call sepfillkmesh(kmesh)

      call procopy1 ( kprob2d(meshorig), kprob2d(meshobst) )

      if ( debug ) write(irefwr,*) 'before probdf'
      iseqprob = 1
      kprob => kprob2d(meshobst)
      call probdfext ( 3, iseqprob )
      if ( ierror/=0 ) go to 1000

      if ( debugloc ) then
         write(irefwr,*) 'kprob_orig'
         call sepfillkmesh(kmesh2d(meshorig))
         kprob => kprob2d(meshorig)
         call sepprinkprob ( 0 )
         kmesh => kmesh2d(meshobst)
         call sepfillkmesh(kmesh)
         write(irefwr,*) 'kprob_obst'
         kprob => kprob2d(meshobst)
         call sepprinkprob ( 0 )
      end if  ! ( debug )

!     --- Fill matrix structure

      intmat1d => intmat2d(:,meshobst)
      call incomgen ( ireadstruc, intmat2d(1,meshorig) )
      iinmap(2) = 2
      meshnr = meshobst

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probobstnew'

      end if  ! ( debug )
      call erclos ( 'probobstnew' )

      end
