      subroutine probstcopsol ( meshorig, meshobst, map1d )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-04-2013 Use prcopsol
!        version  1.0  date 22-11-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solutions from old mesh onto new mesh if possible
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     level_set
!     obstacle
!     copy
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulemnsub
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshorig, meshobst
      type (mapping) :: map1d

!     map1d         i/o   contains information of the mapping
!     meshobst       i    Sequence number of obstacle mesh to be used
!     meshorig       i    Sequence number of original mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOPSOL       Interpolate a series of solution vectors from one mesh
!                    to the other
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probstcopsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probstcopsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numvecmn

         call prcopsol ( meshorig, meshobst, map1d, i, .true. )

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probstcopsol'

      end if  ! ( debug )
      call erclos ( 'probstcopsol' )

      end
