      subroutine prplotfn ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 09-01-2015 change of inpsep
!        version  2.0  date 12-05-2014 Complete new subroutine
!        version  1.2  date 06-02-2012 Allow indexed vectors
!
!   copyright (c) 2007-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot function along curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     plot_function
!     driving_subroutine
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplotfn
      use sepmodulemain
      use sepmoduleint2d
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotfn
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRPLOTFN1      Plot function along curve
!     PRPLOTFN2      Plot function along line in surface
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPPLOTFUNC2   Plot function along curve or line
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotfn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplotfn'
         write(irefwr,1) 'istartmn', istartmn
         call prinin ( inpsep, 16, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( itimloop==0 .or. ioutact==1 ) then

!     --- Actual plots

         numvecs = inpsep(1)
         istartvecs = inpsep(2)
         idegfd = inpsep(3)

!        --- Activate and fill kmesh

         call sepfillkmesh ( kmesh )

         if ( inpsep(14)==1 ) then

!        --- type 1, line and possibly surface given

            call prplotfn2 ( inpsep, rinsep )
            if ( numelm==0 ) go to 900

         else

!        --- type 0, curves given

            call prplotfn1 ( inpsep )

         end if  ! ( inpsep(14)==1 )

!        --- Make plot

         call  sepplotfunc2 ( inpsep )

      end if  ! ( itimloop==0 .or. ioutact==1 )

!     --- Raise istartmn

900   istartmn = istartmn+15+inpsep(1)+inpsep(4)+inpsep(10)+inpsep(12)

1000  call erclos ( 'prplotfn' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartmn', istartmn
         write(irefwr,*) 'End prplotfn'

      end if  ! ( debug )

      end subroutine prplotfn
