      subroutine prstoccompvel ( iseqvel, iseqdiff, iseqw1, numnodes, curves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 23-12-2011 replace mmnodes by nodes
!        version  1.0  date 16-11-2009
!
!   copyright (c) 2009-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the velocity vector along the free surface
!     Defined by difference subdivided by components of the grad vector
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     free_surface
!     stochastic
!     investment
!     velocity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulesmf
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqvel, iseqdiff, iseqw1, numnodes, curves(*)

!     curves         i    Contains curves of free boundary
!     iseqdiff       i    sequence number of difference vector in isolut2d
!     iseqvel        i    sequence number of velocity in isolut2d
!     iseqw1         i    sequence number of w1 in isolut2d
!     numnodes       i    Number of nodal points in array nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iskpel(1), icurvs(1)

!                    they are not
!     icurvs         dummy
!     iskpel         Array containing the element groups to be skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINTVCCURV    Print the values of a vector along the curve icurnr
!     PRSTOCCOMPVEL1 Compute diff/velocity and store in velocity at free
!                    interface
!     SEPFILLSMF1    Fill part of input for subroutine SEPBUILD in SMF
!     TO0100         Performs the actual computation of the derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The difference vector is defined along the free surface
!     Let v be the gradient of the solution
!     Then the velocity vector is defined as diff/veloc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstoccompvel' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstoccompvel'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute gradient of vector w1

      iinder = 0              ! clear array
      iinder(1) = 4           ! calling subroutine
      iinder(2) = 1           ! output vector is of special structure
      iinder(4) = 2           ! icheld
      iinder(5) = 1           ! ix
      iinder(6) = 1           ! jdegfd
      iinder(7) = 5           ! ivec
      iinder(8) = numvecmn+2    ! number of vectors stored in isolut2d
      iinder(10) = iseqw1     ! get from solution vector

      call sepfillsmf1 ( isolut2d, maxvec )
      SMF%ioutvc =>  isolut2d(iseqvel)
      call to0100 ( iskpel, icurvs )

      if ( debug ) &
         call printvccurv ( isolut2d(iseqvel), curves(1), 'grad w1' )

!     --- Compute diff / gradient of vector w1 on interface
!         Store in array iseqvel

      call prstoccompvel1 ( numnodes, iseqdiff, iseqvel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstoccompvel'

      end if  ! ( debug )
      call erclos ( 'prstoccompvel' )

      end subroutine prstoccompvel
