      subroutine prstream ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute stream function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     compute
!     incompressible
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmodulemnsub
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine askvecstr (from pos. 1)
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqin, iseqout, itypvc, ichels, ivec, ielhlp(1)
      double precision :: strm0

!                    they are not
!     ichels         dummy
!     ielhlp         Dummy
!     iseqin         Sequence number of input vector in isol
!     iseqout        Sequence number of output vector in isol
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     strm0          start value for stream function
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCREASTRUC   find structure of vector
!     SEPSTREAM      Actual body of STREAM
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstream' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstream'
         call prinin ( inpsep, 4, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqin = inpsep(2)   ! input vector
      iseqout = inpsep(1)  ! output vector

      if ( debug ) then
         write(irefwr,1) 'iseqin, iseqout', iseqin, iseqout
         call prinin ( isolut2d(iseqin), 5, 'isol_in' )
      end if  ! ( debug )

!     --- Compute start value of stream function

      strm0 = 0d0
      if ( inpsep(4)>0 ) strm0 = rinsep(inpsep(4))

      istartmn = istartmn+5

!     --- Compute ivec

      itypvc = 110
      iprob = isolut2d(iseqin)/1000+1
      call sepcreastruc ( 1, itypvc, ivec )

      ichels = 1   ! stream function
      call sepstream ( 0, ichels, ivec, isolut2d(iseqout), &
                       inpsep(3), strm0, isolut2d(iseqin), ielhlp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstream'

      end if  ! ( debug )
      call erclos ( 'prstream' )

      end
