      subroutine prtimehisscal
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill time and value for all scalars in time history at the start of
!     the time loop
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_history
!     scalar
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmodulecons
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iscal

!     iscal          Sequence number of scalar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRTIMEHISSCAL1 Fill time and value for scalar iscal into vector
!     PRTIMHISNSTEP  Estimate number of time steps
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimehisscal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimehisscal'
         write(irefwr,1) 'numscals', numscals
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Estimate number of time steps

      call prtimhisnstep ( nstephiscal )
      istephiscal = 1           ! first time step

      if ( debug ) write(irefwr,1) 'nstephiscal', nstephiscal

!     --- Loop over all active scalars

      do iscal = 1, numscals

         if ( scaltim(iscal)%nstephiscal==0 ) then

!        --- scaltim(iscal) has not been filled yet, create array

            allocate ( scaltim(iscal)%arr(2,nstephiscal) )
            scaltim(iscal)%nstephiscal = nstephiscal
            if ( debug ) write(irefwr,1) 'iscal', iscal

            rvectim => scaltim(iscal)%arr
            call prtimehisscal1 ( iscal )

         end if  ! ( scaltim(iscal)==0 )

      end do  ! iscal = 1, numscals

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtimehisscal'

      end if  ! ( debug )
      call erclos ( 'prtimehisscal' )

      end subroutine prtimehisscal
