      subroutine prtimehisscal2
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill time and value for all scalars in time history during
!     the time loop
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_history
!     scalar
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmodulecons
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iscal
      double precision, allocatable :: work(:,:)

!     iscal          Sequence number of scalar
!     work           work array to store a vector temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRTIMEHISSCAL1 Fill time and value for scalar iscal into vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimehisscal2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimehisscal2'
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Raise time step

      istephiscal = istephiscal+1
      if ( istephiscal>nstephiscal ) then

!     --- Number of time steps too large, enlarge real arrays

         if ( debug ) then
            write(irefwr,*) 'Number of time steps too large'
            write(irefwr,*) 'Real array is extended'
            write(irefwr,1) 'nstephiscal', nstephiscal
         end if  ! ( debug )

         nstephiscal = nstephiscal*2  ! double nstephiscal

         do iscal = 1, numscals

            if ( scaltim(iscal)%nstephiscal>0 ) then

!           --- scaltim(iscal) has been filled yet, extend array

               rvectim => scaltim(iscal)%arr
               scaltim(iscal)%nstephiscal = nstephiscal

               allocate ( work(2,nstephiscal/2), stat = error )
               if ( error/=0 ) call eralloc ( error, nstephiscal, 'work' )
               if ( ierror/=0 ) go to 1000

!              --- Copy array temporarily in work

               work = rvectim

!              --- Create array with larger length

               deallocate ( scaltim(iscal)%arr )
               allocate ( scaltim(iscal)%arr(2,nstephiscal) )
               rvectim => scaltim(iscal)%arr

!              --- Copy work back in new defined array

               rvectim(:,1:nstephiscal/2) = work

               deallocate ( work, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'work' )
               if ( ierror/=0 ) go to 1000

!              --- Fill value

               call prtimehisscal1 ( iscal )

            end if  ! ( scaltim(iscal)%nstephiscal>0 ) then

         end do  ! iscal = 1, numscals

      else

!     --- Number of time steps not too large, fill values

         do iscal = 1, numscals

            if ( scaltim(iscal)%nstephiscal>0 ) then

!           --- scaltim(iscal) has been filled yet, fill value

               rvectim => scaltim(iscal)%arr
               call prtimehisscal1 ( iscal )

            end if  ! ( scaltim(iscal)%nstephiscal>0 ) then

         end do  ! iscal = 1, numscals

      end if  ! ( istephiscal>nstephiscal )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtimehisscal2'

      end if  ! ( debug )
      call erclos ( 'prtimehisscal2' )

      end subroutine prtimehisscal2
