      subroutine prtimhislev ( isol, fval, xval, intersect )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 25-08-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the x-value in the one-dimensional space where the function
!     takes value fval
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     interpolation
!     level
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol
      double precision fval, xval
      logical intersect

!     fval           i    level for which the x-value must be computed
!     intersect      o    if true fval is in the reach of the function
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!                         isol must be of type 119, i.e. the result of
!                         a 1D interpolation
!     xval           o    computed x-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoor, numsubcurv

!     ncoor          Number of nodes along 1D line
!     numsubcurv     Number of subcurves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOMPPOSLEV   Find x-position of function corresponding to f = c
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1585   SEPRAN arrays of this type  are not yet allowed in this subroutine
!    2670   Intersection line contains more than 1 subcurves
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimhislev' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimhislev'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( ks(isol)%typevec/=119 ) then

!     --- Only arrays of type 119 are allowed

         call errint ( ks(isol)%typevec, 1 )
         call errsub ( 1585, 1, 0, 0 )
         go to 1000

      end if  ! ( ks(isol)%typevec/=119 )

      numsubcurv = ks(isol)%n
      if ( numsubcurv/=1 ) then

!     --- Only arrays of type 119 are allowed

         call errint ( numsubcurv, 1 )
         call errsub ( 2670, 1, 0, 0 )
         go to 1000

      end if  ! ( numsubcurv/=1 )

      isoltemp => ks(isol)%isol
      ncoor = isoltemp(1)
      usoltemp => ks(isol)%sol
      xcoor => usoltemp(1:ncoor)
      xfunc => usoltemp(ncoor+1:)
      call ercompposlev ( xcoor, xfunc, ncoor, fval, xval, intersect )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtimhislev'

      end if
      call erclos ( 'prtimhislev' )

      end subroutine prtimhislev
