      subroutine read00 ( iincommt, nprob, iseqnr, isubr, lniincommat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.16 date 21-04-2015 Allow print_level 4
!        version  6.15 date 06-09-2014 Use mpistart
!        version  6.14 date 11-07-2014 Extension with assymetric
!
!   copyright (c) 1988-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reads the input for subroutine COMMAT.
!     The input must be enclosed between MATRIX and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulecpack
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nprob, lniincommat, iincommt(lniincommat,nprob), isubr, &
              iseqnr

!     iincommt       o    Integer input array containing information concerning
!                         the structure of the large matrix
!                         iincommt must be filled as follows:
!                         1: Not used
!                         2: jmethod parameter indicating the structure of
!                            the large matrix
!                            Possible values
!                             1    symmetric profile matrix
!                             2    non-symmetric profile matrix
!                             3    symmetric complex profile matrix
!                             4    non-symmetric complex profile matrix
!                             5    symmetric compact matrix
!                             6    non-symmetric compact matrix
!                             7    symmetric complex compact matrix
!                             8    non-symmetric complex compact matrix
!                            13    simple, q matrix symmetric L = G^T
!                            14    simple, q matrix unsymmetric L = G^T
!                            15    simple, q matrix symmetric L # G^T
!                            16    simple, q matrix unsymmetric L # G^T
!                            19    hermitian complex profile matrix
!                            20    full symmetric real matrix
!                            21    full non-symmetric real matrix
!                            22    full symmetric complex matrix
!                            23    full non-symmetric complex matrix
!                            24-27  See 5-8, extra diagonal is stored
!                                  Used for shifted laplace preconditioner
!                            35-38 See 13-16 however instead of DG, the
!                                  pressure mass matrix is stored
!                            49    hermitian complex profile matrix, no-split
!                            52    row compact for mumps (unsymmetric)
!                            53    row compact for mumps (symmetric)
!                            54    row compact for mumps (symmetric, pos def)
!                            55    row compact for mumps (petsc)
!                            56    row compact for mumps (petsc, no sepran mat)
!                         3: iprint defines the print level:
!                            0: no extra information is printed
!                            1: The contents of array intmat corresponding to
!                               the internal unknowns is printed
!                            2: The complete contents of array intmat
!                               is printed
!                            10-12: See 0-2, however also the structure of the
!                               matrix is printed formally
!                               only available for jmethod=1-4
!                         5: iprob      Problem number
!                         6: fillin_extra defines if the structure of the matrix
!                            must be created such that extra fill-in is possible
!                            Possible values:
!                            0: No extra fill-in
!                            1: Special case for discontinuous Navier-Stokes
!                               equations
!                               The connections of the centroids of the elements
!                               with the centroids of neighboring elements
!                               are also filled in the structure for al physical
!                               variables iphys1 to iphys2
!                            2: All neighbors of neighbors are used as extra
!                               fill-in
!                            3: Combination of 1 and 2
!                            fillin_extra >0 only makes sense for a compact
!                            storage in combination with a preconditioned solver
!                          7: iphys1 (See 6)
!                          8: iphys2 (See 6)
!                          9: Type of mesh, possible values:
!                             0: is standard mesh, i.e. the mesh as constructed
!                                by SEPMERSH
!                             1: Finite element mesh, where the standard mesh
!                                is a spectral mesh
!                         10: num_dec Number of unknowns that is considered to
!                             be decoupled
!                         11: istart_dec Starting address of unknowns that are
!                             considered to be decoupled
!                         12: skip_boundary_conditions
!                             0: part of boundary conditions is taken into
!                                account
!                             1: part corresponding to boundary conditions is
!                                skipped
!                         13: use_petsc
!                             0: standard case
!                             1: use petsc
!                             1: use mumps
!                         14: type of parallel coupling
!                             0: neumann-neumann
!                             1: dirichlet-dirichlet
!                         15: storage scheme for direct method
!                             0: default
!                             1: profile
!                             2: nested_dissection
!                             3: minimal_degree
!                         istart_dec ... istart_dec+num_dec-1: sequence numbers
!                             of unknowns that are considered to be decoupled
!     iseqnr        i/o   Sequence number with respect to the storage in array
!                         ISEPIN
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                         0   Subroutine TO0092
!                         1   Subroutine READAL
!     lniincommat    i    Length of first index of iincommat
!                         If this number is extended also adapt:
!                         read00, sepstmbf, to0092, incomgen, linrefbf, to0090
!     nprob          i    Number of various problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=26, nextra=13
      character (len=29) code2(nncode2)
      character (len=19) codemn(nextra)
      integer ncode2(nncode2), i, method, iprob, scheme, &
              ncodmx2(nncode2), nmcode(nextra), &
              ielgrp, jmethod, lenmeshlc, istart, &
              numdec, lastentryused, iseqkm1, iseqkm2
      logical finished, linear

!     code2          Table with keywords
!     codemn         Table with main keywords
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     ielgrp         element group number
!     iprob          Actual problem number
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     istart         Starting address of unknowns that are
!                    considered to be decoupled
!     jmethod        mod(method,100)
!     lastentryused  Last entry used in iincommt before set of numbers for
!                    decoupled degrees of freedom
!     lenmeshlc      Length of array kmesh
!     linear         Indicates whether the element is a linear simplex (true)
!                    or not (false)
!     method         Type of method read
!     ncode2          Table containing the number of significant characters in
!                    the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nextra         Number of extra keywords in codemn
!     nmcode         Table containing the number of significant characters in
!                    the main keyword table codemn, including those of
!                    the extra texts
!     nncode2        Number of entries in table code2
!     numdec         Number of unknowns that is considered to be decoupled
!     scheme         Type of storage scheme as read form input file,
!                    see read00method
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRKEYWEXCLUS  Check if two keywords are mutually exclusive
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     READ00METHOD   Translate keywords found in input file into the parameter
!                    jmethod
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPPETSCSTART  Start petsc
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!    Input is read from the standard input file
!    The following input is recognized
!    MATRIX [ SEQUENCE_NUMBER = s]
!       METHOD = i, PROBLEM = j, PRINT_LEVEL = p, EXTRA_FILLIN = f,
!       PHYS = (p1 TO p2 ), NOSPLIT, BLOCK_STORAGE = i,
!       DECOUPLED_DEGFD ( d1, d2, .. ), SKIP_BOUNDARY_CONDITIONS,
!       SYMMETRIC, UNSYMMETRIC, REAL, COMPLEX,
!       INCOMPRESSIBILITY_SYM, INCOMPRESSIBILITY_UNSYM, ASSEMBLE_PRECON,
!       REACTION_FORCE, SHIFTED_LAPLACE, POSITIVE_DEFINITE, NO_SEPRAN_MATRIX
!       STORAGE_SCHEME = k, PARALLEL_COUPLING = dd/nn
!          with k =  COMPACT
!                    PROFILE
!                    SIMPLE
!                    ROW_COMPACT
!                    VECTOR_COMPUTER, &
!                    AUGMENTED_LAGRANGE
!                    AMG_ROW_COMPACT
!                    MUMPS
!    END
!    The option METHOD = i, is old fashioned
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     306   method has incorrect value
!     572   first input record of next part of SEPSTR is not MATRIX
!    1631   iphys2<iphys1
!    1632   Number after keyword incorrect
!    1858   no method found for a problem number, method = 2 is assumed
!    1859   The keyword method has been found more than once
!    1860   No problem number read in present record
!    2111   matrix structure for a FEM grid required, but there is no spectral
!           mesh
!    2112   matrix structure for a FEM grid required, but there is no FEM mesh
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array IINCOMMT with default values
!     If ( isubr=0 ) then
!        Read first main keyword (MATRIX)
!     Read rest of first record (sequence number)
!     finished := false
!     While ( not finished ) do
!        Read next keyword
!        if ( keyword = END ) then
!           finished := true
!        else
!           Read rest of information
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'METHOD', 'PROBLEM', 'NOSPLIT', 'PRINT_LEVEL', &
                  'EXTRA_FILLIN', 'PHYS', 'TO', 'xxxx', &
                  'MESH', 'DECOUPLED_DEGFD', &
                  'SKIP_BOUNDARY_CONDITIONS', 'STORAGE_SCHEME', &
                  'SYMMETRIC', 'COMPLEX', 'REAL', &
                  'UNSYMMETRIC', 'INCOMPRESSIBILITY_SYMMETRIC', &
                  'INCOMPRESSIBILITY_UNSYMMETRIC', 'ASSEMBLE_PRECON', &
                  'REACTION_FORCE', 'SHIFTED_LAPLACE', 'PETSC', &
                  'MUMPS', 'POSITIVE_DEFINITE', 'NO_SEPRAN_MATRIX', &
                  'PARALLEL_COUPLING' /
      data ncode2 / 4, 4, 4, 4, 5,   4, 2, 5, 4, 6, &
                   8, 5, 4, 4, 4,   6,20,20, 6, 8, &
                   5, 5, 5, 5, 6,   3 /
      data ncodmx2 / 6, 7, 7, 11, 12,   4, 2, 13, 4, 15, &
                   24,14, 9,  7,  4,  11, 27,29,11, 15, &
                   15, 5, 5, 17, 16,  17 /
      data codemn / 'FEM_MESH', 'SEM_MESH', 'COMPACT', 'PROFILE', &
                    'SCHUR_COMPLEMENT', 'ROW_COMPACT', &
                    'VECTOR_COMPUTER', 'AUGMENTED_LAGRANGE', &
                    'AMG_ROW_COMPACT', 'SIMPLE', 'MUMPS', &
                    'NEUMANN_NEUMANN', 'DIRICHLET_DIRICHLET' /
      data nmcode / 7, 7, 4, 4,   5, 10, 4, 4, 7, &
                    7, 5, 1, 1 /

!     code2    Table with keywords
!             Values:
!              1: METHOD
!              2: PROBLEM
!              3: NOSPLIT
!              4: PRINT_LEVEL
!              5: EXTRA_FILLIN
!              6: PHYS
!              7: TO
!              8: xxxx
!              9: MESH
!             10: DECOUPLED_DEGFD
!             11: SKIP_BOUNDARY_CONDITIONS
!             12: STORAGE_SCHEME
!             13: SYMMETRIC
!             14: COMPLEX
!             15: REAL
!             16: UNSYMMETRIC
!             17: INCOMPRESSIBILITY_SYM
!             18: INCOMPRESSIBILITY_UNSYM
!             19: REACTION_FORCE
!             20: SHIFTED_LAPLACE
!             21: ASYMMETRIC
!             22: PETSC
!             23: MUMPS
!             24: POSITIVE_DEFINITE
!             25: NO_SEPRAN_MATRIX
!             26: PARALLEL_COUPLING
!     codemn  Table with subkeywords
!              1: FEM_MESH
!              2: SEM_MESH
!              3: COMPACT
!              4: PROFILE
!              5: SCHUR_COMPLEMENT
!              6: ROW_COMPACT
!              7: VECTOR_COMPUTER
!              8: AUGMENTED_LAGRANGE
!              9: AMG_ROW_COMPACT
!             10: SIMPLE
!             11: MUMPS
!             12: NEUMANN_NEUMANN
!             13: DIRICHLET_DIRICHLET
! ======================================================================
!
      call eropen ( 'read00' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read00'
         write(irefwr,1) 'isubr', isubr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      linear = .false.
      lastentryused = 15   ! last entry used in iincommt
      do ielgrp = 1, nelgrp
         ishape = sepgetshape ( ielgrp )
         if ( ishape==9 .or. ishape==17 ) linear = .true.
      end do

      scheme = 20 ! default

      iincommt = 0
      do i = 1, nprob
         iincommt(1,i) = 8
         iincommt(5,i) = i
         iincommt(7,i) = ndim+1
         if ( linear ) then
            iincommt(8,i) = ndim+1
         else
            iincommt(8,i) = 2*ndim+1
         end if
      end do

      if ( isubr==0 ) then

!     --- isubr = 0, direct call by TO0092
!         Read and check first record

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then

!        --- Not a keyword found

            call errsub ( 572, 0, 0, 0 )
            call errcrd

         else if ( string(1:4)/='MATR' ) then

!        --- Incorrect keyword found

            call errsub ( 572, 0, 0, 0 )
            call errcrd

         end if

      end if

!     --- Read rest of record

      call sepreadseqnr ( iseqnr )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'iseqnr', iseqnr

      finished = .false.

!     --- Fill code etc

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopycodesub ( codemn, nextra, nmcode )
      call sepsetmainend

      tabmin(1:nncode) = 0d0
      tabmin(6:7) = 1d0  ! PHYS and TO
      tabmin(9) = 1d0    ! MESH
      tabmin(10) = 5d0   ! DECOUPLED_DEGFD

      tabmax(1:nncode) = 0d0
      tabmax(1) = 306d0  ! METHOD
      tabmax(2) = 1d0    ! PROBLEM
      tabmax(4) = 13d0   ! PRINT_LEVEL
      tabmax(5) = 7d0    ! EXTRA_FILLIN
      tabmax(6:7) = 1d2  ! PHYS and TO
      tabmax(8) = 1d0
      tabmax(9) = 2d0    ! MESH

      itype(1:nncode) = 2          ! keyword found
      itype(1:2) = 0     ! integer
      itype(4:8) = 0     ! integer
      itype(9) = 5       ! keyword = keyword
      itype(10) = 11     ! keyword = i1, i2, i3, ...
      itype(12) = 5      ! keyword = keyword
      itype(26) = 5      ! keyword = keyword

!     --- While loop finished

      do while ( .not. finished )

!     --- Read next keyword

         ityprd = 1
         table(1:nncode) = 0d0
         tabmax(2) = nprob
         tabmax(10) = 10d0
         tabmin(12) = 3d0 ! start of sub keyword for storage scheme in codemn
         tabmax(12) = 11d0 ! end of sub keyword for storage scheme in codemn
         tabmin(26) = tabmax(12)+1d0 ! PARALLEL_COUPLING
         tabmax(26) = tabmin(26)+1d0
         iretur = 0
         call sepreadsetkeyw

         if ( debug ) call prinrl ( table, nncode, 'table' )
         if ( ierror/=0 ) go to 1000
         if ( iretur==1 ) then

!        --- end found

            finished = .true.

         else

            method = nint ( table(1) )
            iprob  = nint ( table(2) )
            jmethod = method
            if ( jmethod==51 .or. jmethod==11 ) jmethod = 6
            if ( jmethod==12 ) jmethod = 8
            if ( debug ) write(irefwr,1) 'jmethod', jmethod
            if ( jmethod==0 ) then

!           --- Not jmethod found, check other keywords

               call read00method ( table(12), jmethod, code(12), ncodmx, &
                                   codemn, scheme )

            else if ( jmethod<0 .or. jmethod>16 .and. &
                      jmethod<20 .or. jmethod>27 ) then

!           --- jmethod found, but wrong value

               call errint ( method, 1 )
               call errint ( max(iprob,1), 2 )
               call errsub ( 306, 2, 0, 0 )

            else

!           --- jmethod found
!               other keywords to compute jmethod should not be present
!               Check on these keywords

               do i = 12, 20
                  if ( table(i)>0d0 ) &
                     call errkeywexcl ( code, ncodmx, 1, i )

               end do  ! i = 12, 20

            end if  ! ( jmethod==0 )

            if ( nprob>1 .and. iprob==0 ) then
               call errsub ( 1860, 0, 0, 0 )
               go to 1000
            end if  ! ( nprob>1 .and. iprob==0 )

            iprob = max(1,iprob)
            iprob = min(nprob,iprob)
            if ( iincommt(2,iprob)/=0 ) then
               call errint ( iprob, 1 )
               call errwar ( 1859, 1, 0, 0 )
            end if

            iincommt(2,iprob) = jmethod
            iincommt(15,iprob) = scheme
            iincommt(3,iprob) = nint(table(4))
            if ( iincommt(3,iprob)>4 .and. iincommt(3,iprob)<10 ) then
               call errint ( iincommt(3,iprob), 1 )
               call errchr ( code(4), 1 )
               call errsub ( 1632, 1, 0, 1 )
            end if
            iincommt(5,iprob) = iprob
            if ( table(5)>0d0 ) then
               iincommt(6,iprob) = nint(table(5))
               if ( table(6)>0d0 ) iincommt(7,iprob) = nint(table(6))
               if ( table(7)>0d0 ) then
                  iincommt(8,iprob) = nint(table(7))
               else
                  iincommt(8,iprob) = iincommt(7,iprob)
               end if
               if ( iincommt(8,iprob)<iincommt(7,iprob) ) then

!              --- error phys2<phys1

                  call errint ( iincommt(8,iprob), 1 )
                  call errint ( iincommt(7,iprob), 2 )
                  call errint ( iprob, 3 )
                  call errsub ( 1631, 3, 0, 0 )
               end if

            end if

            if ( table(9)>0d0 ) then

!           --- MESH found

               iseqkm1 = kmesh2d(1)
               iseqkm2 = kmesh2d(2)
               if ( nint(table(9))==1 ) then

!              --- FEM mesh found, check if there is a spectral and a FEM mesh

                  if ( km(iseqkm2)%kmesh(38)==0 ) then

!                 --- No spectral mesh

                     call errwar ( 2111, 0, 0, 0 )

                  else

                     if ( -km(iseqkm2)%kmesh(38)/=km(iseqkm1)%kmesh(38) .or. &
                          km(iseqkm2)%kmesh(8)/=km(iseqkm1)%kmesh(8) .or. &
                          km(iseqkm2)%kmesh(9)<=km(iseqkm1)%kmesh(9) ) then

!                    --- error no fem mesh available

                        call errsub ( 2112, 0, 0, 0 )

                     else

!                    --- everything looks ok

                        iincommt(9,iprob) = 1

                     end if

                  end if
               end if

            end if  ! ( table(9)>0d0 )

            if ( table(10)>0d0 ) then

!           --- DECOUPLE_DEGFD found

               numdec = nint(tabmax(10))  ! number of items
               istart = lastentryused+1   ! starting address of information
               iposut = nint(table(10))   ! starting address in tabout
               iincommt(10,iprob) = numdec
               iincommt(11,iprob) = istart

!              --- Check array length

               call inichkarlen ( istart+numdec-1, lniincommat, &
                                  'iincommt' )
               if ( ierror/=0 ) go to 1000

               do i = 0, numdec-1
                  iincommt(istart+i,iprob) = nint(tabout(iposut+i))
               end do  ! i = 0, numdec-1

            end if  ! ( table(10)>0d0 )
            iincommt(12,iprob) = nint(table(11))  ! skip_boundary_conditions

!           --- Check if the keywords petsc and mumps are found
!               These are mutually exclusive

            call errkeywexclus ( table, code, ncodmx, 22, 23 ) ! petsc/mumps
            call errkeywexclus ( table, code, ncodmx, 13, 16 ) ! (un)symmetric

            if ( nint(table(22))==1 ) then

!           --- petsc found

               iincommt(13,iprob) = 1   ! use petsc
               if ( table(25)>0d0 ) then
                  iincommt(2,iprob) = 56   ! jmethod petsc, no sepran matrix
               else
                  iincommt(2,iprob) = 55   ! petsc jmethod
               end if  ! ( table2(25)>0d0 )

               if ( .not. mpistart ) then

!              --- mpi has not started, start petsc and make mpistart true

                  call seppetscstart
                  mpistart = .true.

               end if  ! ( .not mpistart )

            else if ( nint(table(23))==1 ) then

!           --- mumps found

               iincommt(13,iprob) = 2   ! use mumps
               if ( nint(table(13))==1 ) then

!              --- symmetric found

                  if ( nint(table(24))==1 ) then
                     iincommt(2,iprob) = 54   ! symmetric positive definite
                  else
                     iincommt(2,iprob) = 53   ! symmetric
                  end if  ! ( nint(table(24))==1 )

               else

!              --- unsymmetric found

                  iincommt(2,iprob) = 52   ! unsymmetric

               end if  ! ( nint(table(13))==1 )

            end if  ! ( nint(table(22))==1 )

!           --- Parallel coupling

            if ( nint(table(26))>0 ) iincommt(14,iprob) = nint(table(26))-1

         end if

      end do  ! while ( .not. finished )

!     --- Check array iincommt

      if ( iseqnr==1 ) then
         do iprob = 1, nprob
            if ( iincommt(2,iprob)==0 ) then
               call errint ( iprob, 1 )
               call errwar ( 1858, 1, 0, 0 )
               iincommt(2,iprob) = 2
            end if
         end do
      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( iincommt, nprob, lniincommat, 'iincommt' )
         write(irefwr,*) 'End read00'

      end if  ! ( debug )
      call erclos ( 'read00' )

      end subroutine read00
