      subroutine read02_304
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-01-2015 Extension with restriction
!        version  1.0  date 30-10-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 304
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     code       contains keywords recognized for coefficients type 304
!                1: RESTRICTION
!                2: CAPILLARY_RESTRICTION
!                3: OIL_SUPPLY_PRESSURE
!     codemain      contains subkeywords
!                1: LINEAR
!                2: NON_LINEAR
! ======================================================================
!
      call eropen ( 'read02_304' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_304'
         write(irefwr,1) 'lastitem, iposin', lastitem, iposin
         call prinin ( iincof, iposin, 'iincof' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( codereynoldsbn, nncodereynoldsbn, &
                         nscodereynoldsbn, ncodereynoldsbn )
      call sepcopycodesub ( codelin, nncodelin, nscodelin )

      itype = 44                 ! read keyword only
      table = -1d0
      tabmin = 0d0
      tabmax = 1d10

      itype(1) = 5               ! RESTRICTION
      tabmin(1) = 1d0
      tabmax(1) = tabmin(1)+1d0

      nmain = 0
      iretur = -11  ! keyword has been read

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4

      icoef = 0

      if ( table(1)>0d0 ) then

!     --- RESTRICTION found

         icoef = 1
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(1))-1
         iposin = iposin+3
         go to 1000

      end if  ! ( table(1)>0d0 )

      do i = 2, 3

        if ( table(i)>0d0 ) then

!       --- CAPILLARY_RESTRICTION, OIL_SUPPLY_PRESSURE

           icoef = i
           call read02fillc
           exit

         end if  ! ( table(i)>0d0 )

      end do  ! i = 2, 3

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iincof, iposin, 'iincof' )
         write(irefwr,*) 'End read02_304'

      end if  ! ( debug )
      call erclos ( 'read02_304' )

      end subroutine read02_304
