      subroutine read02_325
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-10-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 325
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     READ02FILLV    Fill information concerning coefficient into iincof/rincof
!                    Special case of a vector of coefficients
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02_325' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_325'
         write(irefwr,1) 'lastitem, iposin', lastitem, iposin
         write(irefwr,1) 'ndim, ielgrp', ndim, ielgrp
         call prinin ( iincof, iposin, 'iincof' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncodemain=nncodebear+nncodelinear

      itype = 44                 ! read keyword only
      table = -1d0
      tabmin = 0d0
      tabmax = 1d10

      itype(1) = 5               ! TYPE_OF_BEARING
      tabmin(1) = 1d0
      tabmax(1) = tabmin(1)+nncodebear-1
      itype(2) = 5               ! LINEARIZATION
      tabmin(2) = tabmax(1)+1d0
      tabmax(2) = tabmin(2)+nncodelinear-1
      tabmax(3:4) = 4d0          ! INTEGRATION_RULE; TRANSFORMATION
      tabmin(5) = 1d-30
      tabmax(5) = 1d2

      nmain = 0
      iretur = -11  ! keyword has been read

      call sepcopycode ( codereynolds, nncodereynolds, &
                         nscodereynolds, ncodereynolds )
      codesub(1:nncodebear) = codebear
      codesub(nncodebear+1:nncodebear+2) = codelinear(2:3)
      ncodesub(1:nncodebear) = nscodebear
      ncodesub(nncodebear+1:nncodebear+2) = nscodelinear(2:3)

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncodereynolds, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4

      icoef = 0

      do i = 1, 2

         if ( table(i)>0d0 ) then

!        --- TYPE_OF_BEARING,  LINEARIZATION found

            icoef = i
            iincof(iposin+1) = 1      ! integer found
            iincof(iposin+2) = icoef
            iincof(iposin+3) = nint(table(i))-1
            iposin = iposin+3
            exit

         end if  ! ( table(i)>0d0 )

      end do  ! i = 1, 2
      if ( icoef>0 ) go to 1000

      do i = 3, 4

         if ( table(i)>0d0 ) then

!        --- INTEGRATION_RULE, TRANSFORMATION found

            icoef = i
            iincof(iposin+1) = 1      ! integer found
            iincof(iposin+2) = icoef
            iincof(iposin+3) = nint(table(i))-1
            iposin = iposin+3
            exit

         end if  ! ( table(i)>0d0 )

      end do  ! i = 3, 4
      if ( icoef>0 ) go to 1000

      do i = 5, 9

        if ( table(i)>0d0 ) then

!       --- LAYER_THICKNESS, VISCOSITY, SQUEEZE, CONSTANT, REFERENCE_PRESSURE

           icoef = i+1
           call read02fillc
           exit

         end if  ! ( table(i)>0d0 )

      end do  ! i = 5, 9
      if ( icoef>0 ) go to 1000

      if ( table(10)>0d0 ) then

!     --- VELOCITY found

         icoef = 11
         call read02fillv ( ndim )
         imark(icoef) = 0

      else

!     --- VELOCITY not found

        do i = 11, 12

           if ( table(i)>0d0 .and. imark(11)==0 ) then

!          --- U_VELOCITY, V_VELOCITY found

              icoef = i
              call read02fillc
              exit

           end if  ! ( table(10)>0d0 )

        end do  ! i = 11, 12

      end if  ! ( table(10)>0d0 )
      if ( icoef>0 ) go to 1000

      do i = 13, 14

         if ( table(i)>0d0 ) then

!        --- DENSITY, RT found

            icoef = i+1
            call read02fillc
            exit

         end if  ! ( table(12)>0d0 )

      end do  ! i = 13, 14
      if ( icoef>0 ) go to 1000

      if ( table(15)>0d0 ) then

!     --- FORCE found

         icoef = 16
         call read02fillv ( ndim )
         imark(icoef) = 0

      else

!     --- FORCE not found

        do i = 16, 17

           if ( table(i)>0d0 .and. imark(16)==0 ) then

!          --- X_FORCE, Y_FORCE found

              icoef = i
              call read02fillc
              exit

           end if  ! ( table(10)>0d0 )

        end do  ! i = 11, 12

      end if  ! ( table(15)>0d0 )
      if ( icoef>0 ) go to 1000

      if ( table(18)>0d0 ) then

!     --- UPPER_FACE_VELOCITY found

         icoef = 19
         call read02fillv ( ndim )
         imark(icoef) = 0

      else

!     --- UPPER_FACE_VELOCITY not found

        do i = 19, 20

           if ( table(i)>0d0 .and. imark(19)==0 ) then

!          --- X_UPPER_FACE_VELOCITY, Y_UPPER_FACE_VELOCITY found

              icoef = i
              call read02fillc
              exit

           end if  ! ( table(10)>0d0 )

        end do  ! i = 19, 20

      end if  ! ( table(18)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iincof, iposin, 'iincof' )
         write(irefwr,*) 'End read02_325'

      end if  ! ( debug )
      call erclos ( 'read02_325' )

      end subroutine read02_325
