      subroutine read02_701
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 16-02-2015 Correction irule
!        version  1.0  date 30-10-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 701
!     Poisson equation
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     READ02FILLI    Read and fill information concerning integer coefficient
!                    into iincof
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     The default input is:
!     icoef3 = integration_rule
!     icoef4 = coordinate_system
!     coef 6 = source
!     coef 7 = density
!     coef 8 = heat_capacity
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     code       contains keywords recognized for coefficients type 701
!                 1: SOURCE
!                 2: INTEGRATION_RULE
!                 3: COORDINATE_SYSTEM
!                 4: DENSITY
!                 5: HEAT_CAPACITY
! ======================================================================
!
      call eropen ( 'read02_701' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_701'
         write(irefwr,1) 'ndim, lastitem', ndim, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode=5

      code(1:3) = codegen(1:3)
      ncode(1:3) = nscodegen(1:3)
      ncodmx(1:3) = ncodegen(1:3)
      code(4) = codegen(5)
      ncode(4) = nscodegen(5)
      ncodmx(4) = ncodegen(5)
      code(5) = codegen(8)
      ncode(5) = nscodegen(8)
      ncodmx(5) = ncodegen(8)
      call sepcopycodesub ( codecsys, nncodecsys, nscodecsys )

      itype(1:nncode) = 44
      table(1:nncode) = -1d20
      tabmin(1:nncode) = -1d20
      tabmax(1:nncode) = 1d20
      nmain = 0
      iretur = -11  ! keyword has been read
      ityprd = 4    ! indicates that the first item is a string

      itype(2) = 0  ! INTEGRATION_RULE
      tabmin(2) = 0d0
      tabmax(2) = 4d0

      itype(3) = 5 ! COORDINATE_SYSTEM
      tabmin(3) = 1d0
      tabmax(3) = tabmin(3)+nncodecsys-1

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4

      icoef = 0

      if ( table(1)>-1d19 ) then

!     --- SOURCE found

         icoef = 6
         call read02fillc
         go to 1000

      end if  ! ( table(1)>0d0 )
      if ( icoef>0 ) go to 1000

      if ( table(2)>0d0 ) then

!     --- table(2)> 0, INTEGRATION_RULE

         icoef = 3
         call read02filli ( table(2), 0 )
         go to 1000

      end if  ! ( table(2)>0d0 )

      if ( table(3)>0d0 ) then

!     --- table(3)> 0, COORDINATE_SYSTEM

         icoef = 4
         call read02filli ( table(3), 1 )
         go to 1000

      end if  ! ( table(3)>0d0 )

      if ( table(4)>=0d0 ) then

!     --- table(4)> 0, DENSITY

         icoef = 7
         call read02fillc
         go to 1000

      end if  ! ( table(4)>0d0 )

      if ( table(5)>=0d0 ) then

!     --- table(5)> 0, heat_capacity

         icoef = 8
         call read02fillc
         go to 1000

      end if  ! ( table(5)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icoef', icoef
         write(irefwr,*) 'End read02_701'

      end if  ! ( debug )
      call erclos ( 'read02_701' )

      end subroutine read02_701
