      subroutine read02_900
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-01-2015 Extension with compute_pressure
!        version  1.0  date 22-12-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type numbers 900:903
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icoefforce, iforce, ih

!     i              Counting variable
!     icoefforce     Coefficient sequence number w.r.t. force vector
!     iforce         Sequence number of FORCE in code
!     ih             Help variable to store a constant temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READ02_FORCE   Read coefficients for force vector part
!     READ02CODE     Copy num positions of codegen into code
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!

!     code       contains keywords recognized for coefficients type 703
!                 1: MODEL
!                 2: INTEGRATION_RULE
!                 3: COORDINATE_SYSTEM
!                 4: LINEARIZATION
!                 5: PENALTY
!                 6: DENSITY
!                 7: ANGULAR_VELOCITY
!                 8: FORCE
!                 9: X_FORCE
!                10: Y_FORCE
!                11: Z_FORCE
!                12: VISCOSITY
!                13: POWER_N
!                14: LAMBDA
!     codemain      contains subkeywords
!                 1: NEWTONIAN
!                 2: POWER_LAW
!                 3: CARREAU
!                 4: PLASTICO_VISCOUS
!                 5: SPECIAL
!                 6: DIAGONAL
!                 7: CARTESIAN
!                 8: AXI_SYMMETRIC
!                 9: POLAR
!                10: NONE
!                11: PICARD
!                12: NEWTON
!                13: INCORRECT_PICARD
! ======================================================================
!
      call eropen ( 'read02_900' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_900'
         write(irefwr,1) 'ndim, lastitem', ndim, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode=14
      nncodesub=13

      itype(1:nncode) = 44
      table(1:nncode) = -1d20
      tabmin(1:nncode) = -1d20
      tabmax(1:nncode) = 1d20
      nmain = 0
      iretur = -11  ! keyword has been read

      tabmin(5) = 0d0        ! penalty
      tabmin(12:14) = 1d-20  ! viscosity parameters be positive

      code(1:14) = codenavstoks(1:14)
      ncode(1:14) = nscodenavstoks(1:14)
      ncodmx(1:14) = ncodenavstoks(1:14)

      codesub(1:6) = codemodel(1:6)
      ncodesub(1:6) = nscodemodel(1:6)

      itype(1) = 5   ! MODEL
      tabmin(1) = 1d0
      tabmax(1) = tabmin(1)+5d0

      itype(2) = 0   ! INTEGRATION_RULE
      tabmin(2) = 0d0
      tabmax(2) = 4d0
      call read02code ( 2, codegen(2), nscodegen(2), ncodegen(2), 1 )

      itype(3:4) = 5   ! COORDINATE_SYSTEM
      tabmin(3) = tabmax(1)+1d0
      tabmax(3) = tabmin(3)+2d0
      ih = 6
      codesub(ih+1:ih+nncodecsys) = codecsys
      ncodesub(ih+1:ih+nncodecsys) = nscodecsys
      ih = ih+nncodecsys
      codesub(ih+1:ih+nncodelinear) = codelinear
      ncodesub(ih+1:ih+nncodelinear) = nscodelinear

      tabmin(4) = tabmax(3)+1d0   ! LINEARIZATION
      tabmax(4) = tabmin(4)+3d0

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4

      icoef = 0

      do i = 1, 2

         if ( table(i)>0d0 ) then

!        --- MODEL, INTEGRATION_RULE  found

            icoef = i+1
            iincof(iposin+1) = 1      ! integer found
            iincof(iposin+2) = icoef
            iincof(iposin+3) = nint(table(i))
            if ( i==1 .and. iincof(iposin+3)==6 ) iincof(iposin+3)=10 !diagonal
            iposin = iposin+3
            go to 1000

         end if  ! ( table(i)>0d0 )

      end do  ! i = 1, 2

      do i = 2, 4

         if ( table(i)>0d0 ) then

!        --- COORDINATE_SYSTEM, LINEARIZATION  found

            icoef = i+1
            iincof(iposin+1) = 1      ! integer found
            iincof(iposin+2) = icoef
            iincof(iposin+3) = nint(table(i))-1
            iposin = iposin+3
            go to 1000

         end if  ! ( table(i)>0d0 )

      end do  ! i = 1, 4

      do i = 5, 6

         if ( table(i)>0d0 ) then

!        --- penalty, density

            icoef = i+1
            call read02fillc
            go to 1000

         end if  ! ( table(i)>0d0 )

      end do  ! i = 5, 6

      if ( table(7)>-1d19 ) then

!     --- ANGULAR_VELOCITY found

         icoef = 8
         call read02fillc
         go to 1000

      end if  ! ( table(11)>0d0 )

!     --- Force vector

      iforce = 8
      icoefforce = 9
      call read02_force ( iforce, icoefforce )
      if ( icoef>0 ) go to 1000

      do i = 12, 14

         if ( table(i)>0d0 ) then

!        --- viscosity, power_n, lambda

            icoef = i
            call read02fillc
            go to 1000

         end if  ! ( table(i)>0d0 )

      end do  ! i = 12, 14

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iincof, iposin, 'iincof' )
         call prinrl ( rincof, iposrn, 'rincof' )
         write(irefwr,*) 'End read02_900'

      end if  ! ( debug )
      call erclos ( 'read02_900' )

      end subroutine read02_900
