      subroutine read03defect ( isubeq, inpsol, rinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read all information corresponding to the sub-keyword DEFECT_CORRECTION
!     corresponding to the main keyword SOLVE
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     solve
!     defect_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubeq, inpsol(*)
      double precision rinsol(*)

!     inpsol        i/o   Contains integer information concerning the solution
!                         process.
!                         For a description see subroutine READ03
!     isubeq         i    Sequence number of subequation
!     rinsol        i/o   Array containing the real user input if iread=-1
!                         For a description see subroutine READ03
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       DEFECT_CORRECTION, options
!         with options empty or a combination of the following possibilities:
!          accuracy = eps
!          max_iter = m
!          print_level = p
!          solve_accracy = eps
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read03defect' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read03defect'

      end if
      if ( ierror.ne.0 ) go to 1000

      nncode = 4
      code(1) = 'MAX_ITER'
      code(2) = 'ACCURACY'
      code(3) = 'PRINT_LEVEL'
      code(4) = 'SOLVE_ACCURRACY'
      ncode(1) = 6
      ncode(2) = 3
      ncode(3) = 4
      ncode(4) = 8
      ncodmx(1:2) = 8
      ncodmx(3) = 1
      ncodmx(4) = 15
      itype(1:3) = 0
      itype(2) = 1
      itype(4) = 1
      tabmin(1:4) = 0d0
      tabmax(1) = 1d3
      tabmax(2) = 1d-1
      tabmax(3) = 4d0
      tabmax(4) = 0.5d0
      table(1) = 1d0
      table(2) = 1d-2
      table(3) = 0d0
      table(4) = 1d-1
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( iretur.eq.-1 ) go to 1000
      if ( table(1).gt.0d0 ) &
         inpsol(ninpsol1*isubeq+21) = nint(table(1))
      rinsol(nrinsol1*isubeq+9) = table(2)
      if ( nint(table(3)).ne.0 ) &
         inpsol(ninpsol1*isubeq+6) = nint(table(3))
      rinsol(nrinsol1*isubeq+10) = table(4)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read03defect'

      end if
      call erclos ( 'read03defect' )

      end subroutine read03defect
