      subroutine read06 ( ivecnm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.27 date 26-10-2011 Extra debug
!        version  5.26 date 13-11-2010 error 1488 --> errkeywexcl
!        version  5.25 date 12-03-2008 New way of defining scalars
!
!   copyright (c) 1988-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Help subroutine for subroutine READ05.
!   Reads the rest of the input for a record of the shape prin/plot vi =  ..
!   The part until vi is read by READ05.
!   READ05 is a help subroutine for the program SEPPOST which
!   performs the actual postprocessing for SEPRAN.
!   The contents of the record are filled in the arrays iinputpst, rinputpst
!   and CONTLN according to the description given in subroutine READ05.
!   Besides that it is possible that input is stored in common block CPLOT.
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduleplot
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  :: ivecnm

!     ivecnm i/o   Vector sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=139, nncode2=39
      character (len=21) code1(nncod1)
      character (len=14) code2(nncode2)
      integer ncode1(nncod1), ncode2(nncode2), itype1(nncod1), i, &
              nread, k, inum, ncod1m(nncod1), idim, iposcv, nreadcv, &
              indtext
      double precision tabmn1(nncod1), tabmx1(nncod1)

!     code1          Table containing all main keywords
!     code2          Table containing all  sub keywords
!     i              Counting variable
!     idim           Indicates which quantities must be printed
!                    Possible values:
!                    0: The complete vector is printed
!                    1: The vector is only printed in user points.
!                       These user points must be defined in array ICURVS
!                    2: The vector is only printed in user curves.
!                       These curves must be defined in array ICURVS
!                    3: The vector is only printed in user surfaces.
!                       These surfaces must be defined in array ICURVS
!     indtext        Contains sequence number of the string TEXT in table
!     inum           Number of numbers read in a record
!     iposcv         Position in array tabout with respect to icurvs
!     itype1         Contains the information of the type of keyword for INTX02
!     k              Counting variable
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in code1
!     ncode2         Table containing the number of significant characters in
!                    the keyword table code2
!     nncod1         Number of items in codea
!     nncode2        Number of items in code2
!     nread          Number of numbers read
!     nreadcv        Number of numbers read for curves
!     tabmn1         Contains the minimum values of constpstants for intx02
!     tabmx1         Contains the maximum values of constpstants for intx02
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PLCOMPKIES     Compute number of digits before and after the decimal point
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    This subroutine reads the part options for SEPPOST
!    The following texts are recognized:
!    ANGLE = a
!    ARC_SCALE = (smin, smax)
!    AXIS
!    BNDPART = P1 [,P2]  (Plot field only)
!    BLOCK_MODE = m
!    BOLD
!    BOUNDARIES
!    BOT_TEXT_SIZE = h
!    COLOUR = c
!    CONTOUR
!    CONTRACT
!    COORDINATES ( (x1,y1), (x2,y2), ... , (xn,yn) )
!    CURVES (C1, C2, ... Cn )
!    CURVE_COLOUR
!    DEGFD [i] = k
!    DIR_ROTATE = i
!    DISTANCE = ( dx, dy, dz )
!    ELEMENTS
!    ENTHALPY_VECTOR = Vj
!    EPS_TIME_STEP = eps
!    EQUIDISTANT_GRID
!    EYE POINT = (xe, ye, ze )
!    FACTOR = f
!    FILE_MESH_OUT
!    FILE_SOLUT_OUT
!    FLUX = v  (plot field)
!    FUNCTION
!    FROM  (plot field)
!    GAMMA = g
!    GROUND_VALUE = g
!    HEIGHT = h
!    INNER
!    INTERSECT_ANGLE = alpha
!    ISTEP_COLOUR
!    ISTEP_SYMBOL
!    IVEC = i
!    LENGTH = l
!    LEVELS = (q1, q2, ... qn)
!    LEV_TEXT_SIZE = h
!    LINDIREC = l
!    MARK
!    MAXCOLOUR = m
!    MAXLEVEL  = m
!    MESH_IN = m
!    MESH_OUT = m
!    MESH
!    MINCOLOUR = m
!    MINLEVEL  = m
!    NCOLOUR = n
!    NEGPOS_LEVELS
!    NLEVEL = n
!    NMARK  = n
!    NNEGLEV = n
!    NOAXIS
!    NOBOLD
!    NOBOUNDARIES
!    NOCONTRACT
!    NODE
!    NOELEMENTS
!    NOINNER
!    NOMARK
!    NONODE
!    NONUMBER
!    NOPLOT_LEGENDA
!    NOPLOT_SCALES
!    NORMAL_COMPONENT
!    NOROTATE
!    NPOSLEV = n
!    NSTEP = n
!    NUMBER
!    NUMBER FORMAT = ( nx, mx, ny, my )
!    NUMBUNKNOWNS = n
!    NUM_ROTATIONS = n
!    NUM_TEXT_SIZE = h
!    NVIEW = n
!    ONE_PICTURE
!    ORIGIN = (ox, oy [, oz])
!    POLAR_SYSTEM
!    PICT i OF n
!    PLANE ( ax+by+sz=d )
!    PLOT_BOX
!    PLOT_COORDINATES
!    PLOT_LEGENDA
!    PLOT_ROWS
!    PLOT_SCALES
!    PLOTFM = l
!    POINTS = P1, P2, P3, ...
!    PRINT TRACK
!    PROFILE
!    PROJECTION POINT = (xp,yp,zp)
!    PSTART = (x1,y1,[z1], ... , xn, yn,[zn] )
!    REFERENCE = r
!    REGION = ( xmin, xmax, ymin, ymax )
!    REMOVE_PLOT_BOX
!    RENUMBERED NODES
!    ROTATE
!    ROUNDED_LEVELS
!    SCALES = (xunder, xupper, yunder, yupper )
!    SEQUENCE = X (or Y, Z, R)
!    SKIP ELEMENT GROUPS ( G1, G2, ... Gn )
!    SMOOTHING FACTOR = s
!    START NODE = s
!    START_SURFACE = (Si, Sj)
!    STEPS = (stepx, stepy )
!    STREAM FUNCTION VALUE = s
!    SUPPRESS_COORDINATES
!    SUPPRESS_NODES
!    SUPPRESS_HEADER
!    SURFACES (S1, S2, ... Sn )
!    SYMBOL = s
!    TANGENTIAL_COMPONENT
!    TANG_X = ( x, y, z )
!    TANG_Y = ( x, y, z )
!    TEXT = '...'
!    TEXT_LEVELS = '...'
!    TEXT_SIZE = s
!    TEXTX = '...'
!    TEXTY = '...'
!    TMAX = t
!    TOWARDS
!    TRANSPARENT
!    TRANSFORMATION = tr    (CARTESIAN or PLANE_ORIENTED)
!    TSTEP_PRINT
!    TYPE_FUNC = log
!    TYPE_INTERPOLATION = linear / bilinear / average / area_average /
!                         dist_average
!    TYPE_SCALES = linlin / loglin / linlog / loglog
!    TYPE_TANG = xy / yz / xz / proj_xy / proj_yz / proj_xz
!    TYPE_TIME_INTEGRATION = heun / runge_kutta / trapezoid
!    USE_POROSITY
!    USER_COORDINATES
!    Vj
!    VALUES = (Vi, Vj, .... )
!    VELOCITY_VECTOR = Vj
!    VOLUMES = (Vi, Vj, .... )
!    XY_ANGLE = alpha
!    YFACT = y
!    XSCALE = x
!    YSCALE = y
!    ZSCALE = z
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     665   Number of numbers found in REGION = ( ..., ...,    ) not 2,4 or 6
!     679   Number of numbers found after origin is not equal to 2
!     753   The number of numbers between the brackets after NUMBER FORMAT
!           ( ... ) is incorrect
!     756   The number of numbers found between the brackets after
!           SCALE = ( ... ) is not 4
!     757   The number of numbers found between the brackets after
!           STEP ( ... ) is not 2
!     851   Number of co-ordinates found after EYE POINT = is not equal to 3
!     854   Number of co-ordinates found after PROJECTION POINT = is not 3
!    1134   keyword not available in combination with present keyword
!    1285   Picture number read is incorrect
!    1460   number of numbers between brackets incorrect
!    1579   More than one of POINTS, CURVES or SURFACES found
!    1623   TRANSFORMATION = PLANE_ORIENTED while NUMUNKNOWNS<3
!    1834   Contour levels have already been defined
!    2202   The combination skip element groups and volumes not allowed
!    2227   Only one of a combination of keywords found
!    2358   Only one of a couple of keywords found
!    2359   Number of numbers found after keyword incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data (code1(k),k=1,50) &
                 /'ANGLE', 'AXIS', 'BNDPART', 'BOLD', 'BOUNDARIES', &
                  'COLOUR', 'CONTRACT', 'COORDINATES', 'CURVES', &
                  'DEGFD', 'DEGFD', 'ELEMENTS', 'EYE', 'FACTOR', &
                  'FLUX', 'FROM', 'HEIGHT', 'INNER', 'LENGTH', 'LEVELS', &
                  'LINDIREC', 'MARK', 'MAXCOLOUR', 'MAXLEVEL', &
                  'MESH_IN', 'MINCOLOUR', 'MINLEVEL', 'NCOLOUR', &
                  'NLEVEL', 'NMARK', 'NOAXIS', 'NOBOLD', &
                  'NOBOUNDARIES', 'NOCONTRACT', &
                  'NODE', 'NOELEMENTS', 'NOINNER', 'NOMARK', 'NONODE', &
                  'NONUMBER', 'NOROTATE', 'NSTEP', 'NUMBER', &
                  'NUMBUNKNOWNS', 'NVIEW', 'OF', 'ORIGIN', &
                  'PICT', 'PLANE', 'PLOTFM' /
      data (code1(k),k=51,100) / 'PRINT', &
                  'PROJECTION', 'PSTART', 'REFERENCE', 'REGION', &
                  'RENUMBERED', 'ROTATE', 'SCALES', 'SEQUENCE', 'SKIP', &
                  'SMOOTHING', 'START_SURFACE', 'STEPS', 'STREAM', &
                  'SYMBOL', 'TEXTX', 'TEXTY', 'TEXT_SIZE', &
                  'TMAX', 'TOWARDS', 'YFACT', &
                  'NORMAL_COMPON', 'SUPPRESS_COOR', 'SURFACES', &
                  'TANGENTIAL_CO', 'XY_ANGLE', 'INTERSECT_ANG', &
                  'GROUND_VALUE', 'ZSCALE', 'BLOCK_MODE', 'YSCALE', &
                  'TRANSPARENT', 'POINTS', 'SUPPRESS_HEAD', &
                  'TRANSFORMATIO','NOPLOT_LEGEND', 'NOPLOT_SCALES', &
                  'PLOT_LEGENDA', 'PLOT_SCALES', 'ARC_SCALE', &
                  'POLAR_SYSTEM', 'VELOCITY_VECTOR', &
                  'ENTHALPY_VECTOR', 'GAMMA', 'IVEC', 'CONTOUR', &
                  'FUNCTION', 'PROFILE', 'VOLUMES', 'USE_POROSITY' /
      data (code1(k),k=101,139) / 'TYPE_SCALES', 'NEGPOS_LEVELS', &
                  'ROUNDED_LEVELS', 'NPOSLEV', 'NNEGLEV', 'ONE_PICTURE', &
                  'ISTEP_COLOUR', 'ISTEP_SYMBOL', 'DIR_ROTATE', &
                  'NUM_ROTATIONS', 'VALUES', 'TYPE_FUNC', 'TSTEP_PRINT', &
                  'V', 'MESH_OUT', 'MESH', 'EQUIDISTANT_GRID', &
                  'DISTANCE', 'CURVE_COLOUR', 'TYPE_INTERPOLATION', &
                  'PLOT_ROWS', 'START', 'TYPE_TIME_INTEGRATION', &
                  'EPS_TIME_STEP', 'TANG_X', 'TANG_Y', 'TYPE_TANG', &
                  'XSCALE', 'FILE_MESH_OUT', 'FILE_SOLUT_OUT', &
                  'PLOT_BOX', 'REMOVE_PLOT_BOX', 'TEXT_LEVELS', &
                  'PLOT_COORDINATES', 'USER_COORDINATES', &
                  'LEV_TEXT_SIZE', 'NUM_TEXT_SIZE', &
                  'TEXT', 'SUPPRESS_NODES' /
      data ncode1 /  3,  2,  3,  4,  4,  3,  4,  4,  6,  3, & !10
                     3,  4,  3,  3,  3,  4,  3,  3,  3,  5, & !20
                     3,  4,  6,  6,  7,  6,  6,  4,  4,  4, & !30
                     4,  6,  6,  5,  4,  5,  5,  5,  5,  5, & !40
                     5,  4,  6,  6,  4,  2,  4,  4,  5,  6, & !50
                     5,  4,  5,  3,  3,  3,  3,  3,  3,  4, & !60
                     5,  9,  4,  6,  3,  5,  5,  8,  4,  3, & !70
                     2,  8, 11,  4, 12,  5,  6,  5,  4,  5, & !80
                     4,  8,  5, 11,  8, 11, 11,  9,  9,  6, & !90
                     8, 15,  5,  5,  4,  7,  8,  7,  6,  7, & !100
                     8,  6,  6,  6,  6,  5,  9,  9,  7,  7, & !110
                     3,  8,  8,  1,  8,  4,  4,  4,  9,  8, & !120
                     8,  4,  9,  8,  6,  6,  8,  5,  8,  8, & !130
                     8, 13,  8,  9,  9,  8,  8,  4, 12 /    !140
      data ncod1m /  5,  4,  7,  4, 10,  6,  8, 11,  6,  5, & !10
                     5,  8,  3,  6,  4,  4,  6,  5,  6,  6, & !20
                     8,  4,  9,  8,  7,  9,  8,  7,  6,  5, & !30
                     6,  6, 12, 10,  4, 10,  7,  6,  6,  8, & !40
                     8,  4,  6, 12,  5,  2,  6,  4,  5,  6, & !50
                     5, 10,  6,  9,  6, 10,  6,  6,  8,  4, & !60
                     9, 13,  5,  6,  6,  5,  5,  9,  4,  7, & !70
                     5, 16, 20,  8, 20,  8, 15, 12,  6, 10, & !80
                     6, 11,  5, 15, 14, 14, 13, 12, 11, 10, & !90
                    12, 15, 15,  5,  4,  7,  8,  7,  7, 12, & !100
                    11, 13, 14,  7,  7, 11, 12, 12, 10, 13, & !110
                     6,  9, 11,  1,  8,  4, 16,  8, 12, 18, & !120
                     9,  5, 21, 13,  6,  6,  9,  6, 13, 14, & !130
                     8, 15, 11, 16, 16, 13, 13,  4, 14 /    !140
      data itype1 /  1,  2, 18,  2,  2,  1,  2, 17, 18, 12, & !10
                     0,  2, 22,  1,  1,  2,  1,  2,  1, 19, & !20
                     1,  2,  0,  1,  0,  0,  1,  0,  0,  0, & !30
                     2,  2,  2,  2,  2,  2,  2,  2,  2,  2, & !40
                     2,  0, 20,  0,  0,  0, 19,  0, 21,  1, & !50
                    23, 22, 19,  1, 19, 23,  2, 19,  5, 24, & !60
                    24, 18, 19, 22,  0,  6,  6,  1,  1,  2, & !70
                     1,  2,  2, 18,  2,  1,  1,  1,  1,  1, & !80
                     1,  2, 18,  2,  5,  2,  2,  2,  2, 19, & !90
                     2, 18, 18,  1,  0, -1, -1, -1, 18,  2, & !100
                     5,  2,  2, 38,  0,  2,  0,  0,  0,  0, & !110
                    40,  5,  1, 36,  0,  2,  2, 17,  0,  5, & !120
                     2, 24,  5,  1, 19, 19,  5,  1,  6,  6, & !130
                     2,  2,  6,  2,  2,  1,  1,  6,  2 /    !140
      data tabmn1 /   0d0, 0d0, 2d0, 0d0, 0d0, 1d0, 0d0,  2d0, 1d2, 1d0, &
                      1d0, 0d0, 3d0,-1d9,-1d9, 0d0,1d-1,  0d0, 5d0, 1d2, &
                      1d0, 0d0, 2d0,-1d9, 1d0, 1d0,-1d9,  1d0, 2d0, 0d0, &
                      0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,  0d0, 0d0, 0d0, &
                      0d0, 1d0, 4d0, 1d0, 1d0, 1d0, 4d0,  1d0, 0d0, 5d0, &
                      0d0, 3d0, 1d2, 0d0, 6d0, 0d0, 0d0,  4d0, 7d0, 1d2, &
                      1d0, 2d0, 2d0, 1d0, 1d0, 0d0, 0d0, 1d-2,-1d9, 0d0, &
                      1d-5,0d0, 0d0, 1d2, 0d0, 0d0, 0d0,-1d10,1d-3, 1d0, &
                      1d-3,0d0, 1d2, 0d0, 2d0, 0d0, 0d0,  0d0, 0d0, 2d0, &
                      0d0, 1d0, 1d0, 1d0, 0d0, 0d0, 0d0,  0d0, 1d2, 0d0, &
                     21d0, 0d0, 0d0, 2d0, 2d0, 0d0, 0d0,  0d0, 1d0, 1d0, &
                      1d2, 0d0,1d-9, 0d0, 1d0, 0d0, 0d0,  0d0, 0d0, 0d0, &
                      0d0, 1d0, 0d0,1d-7, 3d0, 3d0, 0d0,1d-10, 0d0, 0d0, &
                      0d0, 0d0, 0d0, 0d0, 0d0,1d-2,1d-2,  0d0, 0d0 /
      data tabmx1 / 360d0, 0d0, 1d4, 0d0, 0d0, 1d2, 0d0,  0d0, 1d4, 2d0, &
                      1d2, 0d0, 0d0, 1d9, 1d9, 0d0, 2d0,  0d0, 1d2, 1d9, &
                      5d0, 0d0, 5d2, 1d9, 1d1, 5d2, 1d9,  1d2, 1d2, 1d4, &
                      0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,  0d0, 0d0, 0d0, &
                      0d0, 1d2, 1d1, 1d1, 4d0, 2d1, 1d9,  2d1, 0d0, 1d2, &
                      0d0, 1d9, 1d9, 1d9, 1d9, 0d0, 0d0,  1d9, 1d1, 1d4, &
                      5d0, 1d4, 5D2, 1d5, 2d1, 0d0, 0d0,  1d0,1d20, 0d0, &
                      1d5, 0d0, 0d0, 1d4, 0d0,18d1,18d1, 1d10, 1d3, 2d0, &
                      1d3, 0d0, 1d4, 0d0, 3d0, 0d0, 0d0,  0d0, 0d0, 0d0, &
                      0d0, 1d2, 1d2, 2d0, 1d2, 0d0, 0d0,  0d0, 1d4, 0d0, &
                     25d0, 0d0, 0d0, 1d2, 1d2, 0d0, 1d1,  1d1, 3d0, 1d2, &
                      1d4, 0d0, 1d9, 2d3, 1d1, 0d0, 0d0,  0d0, 1d2, 0d0, &
                      0d0, 1d5, 0d0, 1d0, 1d9, 1d9, 0d0, 1d10, 0d0, 0d0, &
                      0d0, 0d0, 0d0, 0d0, 0d0, 1d1, 1d1,  0d0, 0d0 /
      data code2 / 'POINT', 'CARTESIAN', 'PLANE_ORIENTED', 'TRACK', &
                  'NODES', 'C', 'X', 'Y', 'Z', 'R', 'ELEMENT', 'GROUPS', &
                  'FACTOR', 'NODE', 'FUNCTION', 'VALUE', 'S', &
                  'FORMAT', 'P', 'V', 'LINLIN', 'LOGLIN', 'LINLOG', &
                  'LOGLOG', 'LOG', 'LINEAR', 'BILINEAR', 'AVERAGE', &
                  'AREA_AVERAGE', 'DIST_AVERAGE', 'HEUN', &
                  'RUNGE_KUTTA', 'TRAPEZOID', 'XY', 'YZ', 'XZ', &
                  'PROJ_XY', 'PROJ_YZ', 'PROJ_XZ' /
      data ncode2 / 2, 4, 5, 4, 5, 1, 1, 1, 1, 1, &
                   4, 4, 3, 3, 4, 3, 1, 3, 1, 1, &
                   6, 6, 6, 6, 3, 4, 6, 6,10,10, &
                   4, 5, 5, 2, 2, 2, 7, 7, 7 /

!     code2    Table containing all  sub keywords
!             Contents
!              1:  POINT
!              2:  CARTESIAN
!              3:  PLANE_ORIENTED
!              4:  TRACK
!              5:  NODES
!              6:  C
!              7:  X
!              8:  Y
!              9:  Z
!             10:  R
!             11:  ELEMENT
!             12:  GROUPS
!             13:  FACTOR
!             14:  NODE
!             15:  FUNCTION
!             16:  VALUE
!             17:  S
!             18:  P
!             19:  FORMAT
!             20:  V
!             21:  LINLIN
!             22:  LOGLIN
!             23:  LINLOG
!             24:  LOGLOG
!             25:  LOG
!             26:  LINEAR
!             27:  BILINEAR
!             28:  AVERAGE
!             29:  AREA_AVERAGE
!             30:  DIST_AVERAGE
!             31:  HEUN
!             32:  RUNGE_KUTTA
!             33:  TRAPEZOID
!             34:  XY
!             35:  YZ
!             36:  XZ
!             37:  PROJ_XY
!             38:  PROJ_YZ
!             39:  PROJ_XZ
!     code1   Table containing all main keywords
!             Contents
!              1:  ANGLE
!              2:  AXIS
!              3:  BNDPART
!              4:  BOLD
!              5:  BOUNDARIES
!              6:  COLOUR
!              7:  CONTRACT
!              8:  COORDINATES
!              9:  CURVES
!             10:  DEGFD
!             11:  DEGFD
!             12:  ELEMENTS
!             13:  EYE
!             14:  FACTOR
!             15:  FLUX
!             16:  FROM
!             17:  HEIGHT
!             18:  INNER
!             19:  LENGTH
!             20:  LEVELS
!             21:  LINDIREC
!             22:  MARK
!             23:  MAXCOLOUR
!             24:  MAXLEVEL
!             25:  MESH_IN
!             26:  MINCOLOUR
!             27:  MINLEVEL
!             28:  NCOLOUR
!             29:  NLEVEL
!             30:  NMARK
!             31:  NOAXIS
!             32:  NOBOLD
!             33:  NOBOUNDARIES
!             34:  NOCONTRACT
!             35:  NODE
!             36:  NOELEMENTS
!             37:  NOINNER
!             38:  NOMARK
!             39:  NONODE
!             40:  NONUMBER
!             41:  NOROTATE
!             42:  NSTEP
!             43:  NUMBER
!             44:  NUMBUNKNOWNS
!             45:  NVIEW
!             46:  OF
!             47:  ORIGIN
!             48:  PICT
!             49:  PLANE
!             50:  PLOTFM
!             51:  PRINT
!             52:  PROJECTION
!             53:  PSTART
!             54:  REFERENCE
!             55:  REGION
!             56:  RENUMBERED
!             57:  ROTATE
!             58:  SCALES
!             59:  SEQUENCE
!             60:  SKIP
!             61:  SMOOTHING
!             62:  START_SURFACE
!             63:  STEPS
!             64:  STREAM
!             65:  SYMBOL
!             66:  TEXTX
!             67:  TEXTY
!             68:  TEXT_SIZE
!             69:  TMAX
!             70:  TOWARDS
!             71:  YFACT
!             72:  NORMAL_COMPONENT
!             73:  SUPPRESS_COORDINATES
!             74:  SURFACES
!             75:  TANGENTIAL_COMPONENT
!             76:  XY_ANGLE
!             77:  INTERSECT_ANGLE
!             78:  GROUND_VALUE
!             79:  ZSCALE
!             80:  BLOCK_MODE
!             81:  YSCALE
!             82:  TRANSPARENT
!             83:  POINTS
!             84:  SUPPRESS_HEADER
!             85:  TRANSFORMATION
!             86:  NOPLOT_LEGENDA
!             87:  NOPLOT_SCALES
!             88:  PLOT_LEGENDA
!             89:  PLOT_SCALES
!             90:  ARC_SCALES
!             91:  POLAR_SYSTEM
!             92:  VELOCITY_VECTOR
!             93:  ENTHALPY_VECTOR
!             94:  GAMMA
!             95:  IVEC
!             96:  CONTOUR
!             97:  FUNCTION
!             98:  PROFILE
!             99:  VOLUMES
!            100:  USE_POROSITY
!            101:  TYPE_SCALES
!            102:  NEGPOS_LEVELS
!            103:  ROUNDED_LEVELS
!            104:  NPOSLEV
!            105:  NNEGLEV
!            106:  ONE_PICTURE
!            107:  ISTEP_COLOUR
!            108:  ISTEP_SYMBOL
!            109:  DIR_ROTATE
!            110:  NUM_ROTATIONS
!            111:  VALUES
!            112:  TYPE_FUNC
!            113:  TSTEP_PRINT
!            114:  V
!            115:  MESH_OUT
!            116:  MESH
!            117:  EQUIDISTANT_GRID
!            118:  DISTANCE
!            119:  CURVE_COLOUR
!            120:  TYPE_INTERPOLATION
!            121:  PLOT_ROWS
!            122:  START
!            123:  TYPE_TIME_INTEGRATION
!            124:  EPS_TIME_STEP
!            125:  TANG_X
!            126:  TANG_Y
!            127:  TYPE_TANG
!            128:  XSCALE
!            129:  FILE_MESH_OUT
!            130:  FILE_SOLUT_OUT, &
!            131:  PLOT_BOX
!            132:  REMOVE_PLOT_BOX
!            133:  TEXT_LEVELS
!            134:  PLOT_COORDINATES
!            135:  USER_COORDINATES
!            136:  LEV_TEXT_SIZE
!            137:  NUM_TEXT_SIZE
!            138:  TEXT
!            139:  SUPPRESS_NODES
! ======================================================================
!
      call eropen ( 'read06' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read06'
         write(irefwr,1) 'lastitem/1, ityprd', lastitem, ityprd
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read rest of information from input record

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )
      call sepcopytype ( nncod1, itype1, tabmn1, tabmx1 )
      call sepcopycodesub ( code2, nncode2, ncode2 )

      indtext =  138    ! sequence number of text in code1
      iinputpst(27) = 21
      iinputpst(99) = 0    ! ICHOICE_INTP3D
      iinputpst(102) = 0   ! User coordinates
      table = -1d0
      table(1)  =   0d0
      table(3)  =  -2d0
      table(9)  =  -6d0
      table(13) =  -1d0
      table(17) = 0.25d0    ! Default height of characters
      table(21) =   2d0
      table(24) =  -1d20
      table(27) =  -1d20
      table(43) =  -18d0
      table(44) =   1d0
      table(51) =  -4d0
      table(52) =  -2d0
      table(56) =  -5d0
      table(60) = -11d0
      table(61) = -13d0
      table(62) = -17d0
      table(64) = -15d0
      table(68) = 0.15d0    ! default text size
      table(74) = -17d0
      table(76) =  60d0
      table(77) =  45d0
      table(78) =   0d0
      table(79) =   1d0
      table(80) =   2d0
      table(81) = 0.5d0
      table(83) = -19d0
      table(92) = -20d0
      table(93) = -20d0
      table(94) = rpost(1)
      table(99) = -20d0
      table(101) = 1d0
      table(104) = 11d0
      table(105) = 11d0
      table(109) = 1d0
      table(110) = 1d0
      table(111) = -20d0
      table(112) = 1d0
      table(114) = ivecnm
      table(119) = 0d0
      table(121) = 0d0
      table(122) = -14d0
      table(124) = 1d-2
      table(128) = 1d0
      table(129) = -1d0
      table(130) = -1d0
      table(136) = 0.15d0    ! default text size levels
      table(137) = 0.25d0    ! default text size numbers

      tabmin(10) =  1d0
      tabmin(24) = -1d20
      tabmin(27) = -1d20
      tabmin(111) = dble(min(100,maxvc1))
      tabmin(118) = ndim
      tabmax(8)  = 25d0
      tabmax(9)  =  1d4
      tabmax(10) =  2d0
      tabmax(13) =  1d20
      tabmax(20) =  1d20
      tabmax(43) =  1d1
      tabmax(47) =  1d20
      tabmax(52) =  1d20
      tabmax(53) =  1d20
      tabmax(55) =  1d20
      tabmax(56) =  1d20
      tabmax(58) =  1d9
      tabmax(60) =  1d4
      tabmax(61) =  5d0
      tabmax(62) =  1d4
      tabmax(63) =  5d2
      tabmax(74) =  1d4
      tabmax(90) =  1d10
      tabmax(99) =  1d4
      tabmin(101) = 21d0
      tabmax(101) = tabmin(101)+3d0
      tabmax(111) = dble(maxvc1)
      tabmin(112) = tabmax(101)+1d0
      tabmax(112) = tabmin(112)
      tabmax(118) = 1d0
      tabmin(120) = tabmax(112)+1d0
      tabmax(120) = tabmin(120)+4d0
      tabmin(123) = tabmax(120)+1d0
      tabmax(123) = tabmin(123)+2d0
      tabmax(125) = 1d9
      tabmax(126) = 1d9
      tabmin(127) = tabmax(123)+1d0
      tabmax(127) = tabmin(127)+5d0
      iretur = 0
      nmain = 0
      if ( debug ) write(irefwr,*) 'before intx02'
      call sepreadsetkeyw
      if ( debug ) then
         write(irefwr,1) 'iretur', iretur
         call prinrl ( table, nncode, 'table' )
      end if  ! ( debug )
      if ( iretur==-1 ) go to 1000
      idim = 0

!     --- Put all computed results into common blocks
!         angle, axis, bold, boundaries, colour, contract, elements

      if ( table(1)>=0d0 ) rinputpst(9) = table(1)
      if ( table(2)>=0d0 ) iinputpst(40) = 1
      if ( table(4)>=0d0 ) iinputpst(31) = 1
      if ( table(5)>=0d0 ) iinputpst(28) = 1
      if ( table(6)>=0d0 ) dcolor = nint(table(6))
      if ( table(7)>=0d0 ) iinputpst(29) = 1
      if ( table(12)>=0d0 ) iinputpst(33) = 1

!     --- factor, flux, height, inner, length, lindirec, mark

      if ( table(14)>=0d0 ) rinputpst(3) = table(14)
      if ( table(15)>=0d0 ) rinputpst(6) = table(15)
      rinputpst(16) = table(17)
      if ( table(18)>=0d0 ) iinputpst(30) = 1
      if ( table(19)>=0d0 ) rinputpst(1) = table(19)
      if ( table(21)>=0d0 ) iinputpst(24) = nint(table(21))
      if ( table(22)>=0d0 ) iinputpst(35) = 1

!     --- nmark, noaxis, nobold, noboundaries, nocontract, mode, noelements

      if ( table(30)>=0d0 ) iinputpst(51) = nint(table(30))
      if ( table(31)>=0d0 ) iinputpst(40) = 0
      if ( table(32)>=0d0 ) iinputpst(31) = 0
      if ( table(33)>=0d0 ) iinputpst(28) = 0
      if ( table(34)>=0d0 ) iinputpst(29) = 0
      if ( table(35)>=0d0 ) iinputpst(34) = 1
      if ( table(36)>=0d0 ) iinputpst(33) = 0

!     --- noinner, nomark, nonode, nonumber, norotate, nstep, numbunknowns

      if ( table(37)>=0d0 ) iinputpst(30) = 0
      if ( table(38)>=0d0 ) iinputpst(35) = 0
      if ( table(39)>=0d0 ) iinputpst(34) = 0
      if ( table(40)>=0d0 ) iinputpst(41) = 0
      if ( table(41)>=0d0 ) irotat = 1
      if ( table(42)>=0d0 ) iinputpst(25) = nint(table(42))
      if ( table(44)>=0d0 ) iinputpst(46) = nint(table(44))

!     --- nview, plotfm, print track, reference, renumbered nodes

      if ( table(45)>=0d0 ) iinputpst(52) = nint(table(45))
      if ( table(50)>=0d0 ) rinputpst(1) = table(50)
      if ( table(51)>=0d0 ) iinputpst(50) = nint(table(51))
      if ( table(54)>=0d0 ) rinputpst(22) = table(54)
      if ( table(56)>=0d0 ) iinputpst(36) = 1

!     --- rotate, smoothing factor, start node, stream, symbol, tmax, yfact

      if ( table(57)>=0d0 ) irotat = 2
      if ( table(61)>=0d0 ) iinputpst(7) = &
           nint(tabout(nint(table(61))))
      if ( table(122)>=0d0 ) iinputpst(17) = &
           nint(tabout(nint(table(122))))
      if ( table(64)>=0d0 ) rinputpst(6) = tabout(nint(table(64)))
      if ( table(65)>=0d0 ) iinputpst(21) = nint(table(65))
      if ( table(69)>=0d0 ) rinputpst(21) = table(69)
      if ( table(71)>=0d0 ) rinputpst(2) = table(71)

      if ( debug ) write(irefwr,*) 'after 71'

!     --- normal_component, suppress_coordinates, tangential_component, &
!         transparent, suppress_header

      if ( table(72)>=0d0 ) iinputpst(64) = 1
      if ( table(73)>=0d0 ) iinputpst(66) = 1
      if ( table(75)>=0d0 ) iinputpst(65) = 1
      if ( table(82)>=0d0 ) iinputpst(69) = 1
      if ( table(84)>=0d0 ) iinputpst(70) = 1

!     --- noplot_legenda, noplot_scales, plot_legenda, plot_scales

      if ( table(86)>=0d0 ) iinputpst(71) = 1
      if ( table(87)>=0d0 ) iinputpst(72) = 1
      if ( table(88)>=0d0 ) iinputpst(71) = 0
      if ( table(89)>=0d0 ) iinputpst(72) = 0

!     --- degfd1/2

      if ( table(10)>=0d0 ) iinputpst(8) = nint(table(10))
      if ( table(11)>=0d0 ) iinputpst(9) = nint(table(11))

!     --- maxcolour, maxlevel, mesh, mincolour, minlevel

      if ( table(23)>=0d0 ) then
         iinputpst(27) = nint(table(23))
         iinputpst(63) = 1
      end if
      if ( table(24)>=-1d19 ) then
         rinputpst(5) = table(24)
         iinputpst(13) = 1
      end if
      if ( table(26)>=0d0 ) then
         iinputpst(26) = nint(table(26))
         iinputpst(62) = 1
      end if
      if ( table(27)>=-1d19 ) then
         rinputpst(4) = table(27)
         iinputpst(12) = 1
      end if

      if ( table(3)>=0d0 ) then

!     --- bndpart = p1, p2

         if ( iinputpst(5)==4 ) iinputpst(5) = 1
         if ( iinputpst(5)==5 ) iinputpst(5) = 2
         if ( iinputpst(5)==6 ) iinputpst(5) = 3
         iposut = nint(table(3))
         nread  = nint(tabmax(3))
         iinputpst(18) = nint(tabout(iposut))
         if ( nread==2 ) then
            iinputpst(19) = nint(tabout(iposut+1))
         else
            iinputpst(19) = iinputpst(18)
         end if

      end if

      if ( debug ) write(irefwr,*) 'after 3'

      if ( table(8)>=0d0 ) then

!     --- co-ordinates
!         Fill co-ordinates in rinputpst (At most 50)

         iposut = nint(table(8))
         nread  = nint(tabmax(8))

         do k = 1, nread*2
            rinputpst(50+k) = tabout(iposut+k-1)
         end do

         iinputpst(43) = nread

      end if

      if ( table(9)>=0d0 ) then

!     --- CURVES = (C1, C3, ... )

         idim   = 2
         iposcv = nint(table(9))
         nreadcv = nint(tabmax(9))

      end if

      if ( table(13)>=0d0 ) then

!     --- eye point = (xe,ye,ze)

         iposut = nint(table(13))
         nread  = nint(tabmax(13))
         if ( nread/=3 ) then
            call errint ( nread, 1 )
            call errint ( 3, 2 )
            call errsub ( 851, 2, 0, 0 )
            call errcrd
            iretur = -1
         end if
         do k = 1, nread
            rinputpst(k+9) = tabout(iposut+k-1)
         end do
         if ( iinputpst(53)>=10 ) iinputpst(53) = iinputpst(53) - 10

      end if

      if ( table(16)>=0d0 ) then

!     --- from

         if ( iinputpst(5)==0 .or. iinputpst(5)==1 ) iinputpst(5) = 2
         if ( iinputpst(5)==4 ) iinputpst(5) = 5

      end if

      if ( table(20)>=0d0 ) then

!     --- levels
!         Fill levels in array contln from position 6

         iposut = nint(table(20))
         nread  = nint(tabmax(20))
         do k = 1, nread
            contln(5+k) = tabout(iposut+k-1)
         end do
         iinputpst(6) = nread
         iinputpst(14) = 1

      end if

      if ( debug ) write(irefwr,*) 'after 20'

      if ( table(28)>=0d0 .or. table(29)>=0d0 ) then

!     --- ncolour, nlevel

         if ( iinputpst(6)/=0 ) then

!        --- Warning the number of contour lines has already been defined
!            This information is skipped

            call errwar ( 1834, 0, 0, 0 )
            call errcrd

         else

            if ( table(28)>=0d0 ) then
               iinputpst(6) = nint(table(28))
            else
               iinputpst(6) = nint(table(29))
            end if

         end if

      end if

      if ( table(43)<-1d3 ) then

!     --- number

         iinputpst(41) = 1

      end if

      if ( table(43)>=0d0 ) then

!     --- number format

         iposut = nint(table(43))
         nread  = nint(tabmax(43))
         if ( nread/=4 ) then
            call errint ( inum, 1 )
            call errint ( 4, 2 )
            call errsub ( 753, 2, 0, 0 )
            call errcrd
            iretur = -1
         end if
         kiesx = nint(tabout(iposut))
         isigx = nint(tabout(iposut+1))
         kiesy = nint(tabout(iposut+2))
         isigy = nint(tabout(iposut+3))

      end if

      if ( table(47)>=0d0 ) then

!     --- origin

         iposut = nint(table(47))
         nread  = nint(tabmax(47))
         if ( nread<2 .or. nread>3 ) then
            call errint ( nread, 1 )
            call errsub ( 679, 1, 0, 0 )
            call errcrd
            iretur = -1
         end if
         rinputpst(7) = tabout(iposut)
         rinputpst(8) = tabout(iposut+1)
         if ( nread==3 ) then

!        --- 3 numbers read, this means that we are using a plane

            rinputpst(35) = tabout(iposut+2)
            iinputpst(99) = 100

         end if

      end if

      if ( table(48)>=0d0 ) then

!     --- pict i of n

         iinputpst(22) = nint(table(48))
         iinputpst(23) = nint(table(46))
         if ( iinputpst(22)<0 .or. iinputpst(22)>iinputpst(23) )then
            call errint ( iinputpst(22), 1 )
            call errint ( iinputpst(23), 2 )
            call errsub ( 1285, 2, 0, 0 )
            call errcrd
            iretur = -1
         end if

      end if

      if ( table(49)>=0d0 ) then

!     --- plane ( ax+by+cz=d)

         iposut = nint(table(49))
         do k = 1, 4
            rinputpst(16+k) = tabout(iposut+k-1)
         end do

      end if

      if ( table(52)>=0d0 ) then

!     --- PROJECTION POINT = ( xp, yp, zp )

         iposut = nint(table(52))
         nread  = nint(tabmax(52))
         if ( nread/=3 ) then
            call errint ( nread, 1 )
            call errsub ( 854, 1, 0, 0 )
            call errcrd
            iretur = -1
         end if
         rinputpst(13) = tabout(iposut)
         rinputpst(14) = tabout(iposut+1)
         rinputpst(15) = tabout(iposut+2)

      end if

      if ( table(53)>=0d0 ) then

!     --- PSTART = ( x1st, y1st, (z1st)... xnst, ynst, (znst) )
!         Place values in rinputpst starting at position 51

         iposut = nint(table(53))
         nread  = nint(tabmax(53))
         iinputpst(49) = nread
         do k = 1, nread
            rinputpst(50+k) = tabout(iposut+k-1)
         end do

      end if

      if ( table(55)>=0d0 ) then

!     --- REGION = ( xmin, xmax, ymin, ymax, zmin, zmax )
!         Place values in rinputpst

         iposut = nint(table(55))
         nread  = nint(tabmax(55))
         jmax = 1
         iinputpst(11) = 1
         if ( nread<2 .or. nread==3 .or. nread==5 ) then
            call errint ( nread, 1 )
            call errsub ( 665, 1, 0, 0 )
            call errcrd
            go to 1000
         end if

!        --- Fill numbers in xmin to zmax

         xmin = tabout(iposut)
         xmax = tabout(iposut+1)
         if ( nread>2 ) then
            ymin = tabout(iposut+2)
            ymax = tabout(iposut+3)
            if ( nread>4 ) then
               zmin = tabout(iposut+4)
               zmax = tabout(iposut+5)
            end if
         end if

      end if

      if ( debug ) write(irefwr,*) 'after 55'

      if ( table(58)>=0d0 ) then

!     --- SCALES ( xunder, ... , yupper )

         iposut = nint(table(58))
         nread  = nint(tabmax(58))
         if ( nread/=4 ) then
            call errint ( nread, 1 )
            call errsub ( 756, 1, 0, 0 )
            call errcrd
            iretur = -1
         end if
         axmin = tabout(iposut)
         axmax = tabout(iposut+1)
         aymin = tabout(iposut+2)
         aymax = tabout(iposut+3)
         if ( kiesx==-1 ) &
            call plcompkies ( axmin, axmax, kiesx, isigx )
         if ( kiesy==-1 ) &
            call plcompkies ( aymin, aymax, kiesy, isigy )

      end if

      if ( debug ) write(irefwr,*) 'after 58'

      if ( table(59)>=0d0 ) then

!     --- SEQUENCE = X, Y, Z or R

         iposut = nint(table(59))
         if ( iposut==4 ) iposut = 1
         iinputpst(10) = iposut

      end if

      if ( table(60)>=0d0 ) then

!     --- SKIP ELEMENT GROUPS (g1, g2, ...gn)

         iposut = nint(table(60))
         nread  = nint(tabmax(60))
         do k = 1, nread
            igroup(k) = nint(tabout(iposut+k-1))
         end do
         iinputpst(32) = nread

      end if

      if ( table(62)>=0d0 ) then

!     --- START_SURFACES = (Si to Sj )

         iposut = nint(table(62))
         nread = nint(tabmax(62))
         iinputpst(95) = 2
         iinputpst(96) = nint(tabout(iposut))
         if ( nread==2 ) then
            iinputpst(97) = nint(tabout(iposut+1))
         else
            iinputpst(97) = iinputpst(96)
         end if

      else

!     --- No START_SURFACES

         iinputpst(95) = 1

      end if

      if ( table(63)>=0d0 ) then

!     --- STEPS (.. , .. )

         iposut = nint(table(63))
         nread  = nint(tabmax(63))
         if ( nread/=2 ) then
            call errint ( nread, 1 )
            call errsub ( 757, 1, 0, 0 )
            call errcrd
            iretur = -1
         end if
         nstepx = nint(tabout(iposut))
         nstepy = nint(tabout(iposut+1))

      end if

!     --- textx, texty, text, text_levels

      if ( table(66)>=0d0 ) then
         iposut = nint(table(66))
         nchar  = nint(tabmax(66))
         textx = ' '
         textx(1:nchar) = strout(iposut)(1:nchar)
         iinputpst(37) = nchar
      end if
      if ( table(67)>=0d0 ) then
         iposut = nint(table(67))
         nchar  = nint(tabmax(67))
         texty = ' '
         texty(1:nchar) = strout(iposut)(1:nchar)
         iinputpst(38) = nchar
      end if
      if ( table(133)>=0d0 ) then

!     --- text_levels found

         iposut = nint(table(133))
         nchar  = nint(tabmax(133))
         textid(2) = ' '
         textid(2)(1:nchar) = strout(iposut)(1:nchar)
         iinputpst(101) = nchar   ! number of characters

      else

!     --- text_levels not found

         textid(2) = 'LEVELS'
         iinputpst(101) = 6   ! number of characters

      end if

      if ( table(indtext)>=0d0 ) then
         iposut = nint(table(indtext))
         nchar  = nint(tabmax(indtext))
         textid(1) = ' '
         textid(1)(1:nchar) = strout(iposut)(1:nchar)
         iinputpst(39) = nchar
      end if

      if ( table(70)>=0d0 ) then

!     --- TOWARDS found for PLOT FIELD, adjust ICHOIS

         if ( iinputpst(5)==0 .or. iinputpst(5)==1 ) iinputpst(5) = 3
         if ( iinputpst(5)==4 ) iinputpst(5) = 6

      end if

      if ( table(74)>=0d0 ) then

!     --- SURFACES = (S1, S3, ... )

         if ( idim/=0 ) then

!        --- Curves has already been found

            call errsub ( 1579, 0, 0, 0 )

         end if
         idim   = 3
         iposcv = nint(table(74))
         nreadcv = nint(tabmax(74))

      end if
      if ( table(83)>=0d0 ) then

!     --- POINTS = (P1, P3, ... )

         if ( idim/=0 ) then

!        --- Curves has already been found

            call errsub ( 1579, 0, 0, 0 )

         end if
         idim   = 1
         iposcv = nint(table(83))
         nreadcv = nint(tabmax(83))

      end if
      if ( table(85)>=2d0 ) then

!     --- TRANSFORMATION = PLANE_ORIENTED

         if ( nint(table(44))<3 ) then

!        --- Number of unknowns less than 3

            call errint ( nint(table(44)), 1 )
            call errsub ( 1623, 1, 0, 0 )

         end if
         iinputpst(46) = iinputpst(46)+10000

      end if
      if ( idim/=0 ) then

!     --- Store curve, point or surface numbers

         icurvs(1) = nreadcv
         do k = 1, nreadcv
            icurvs(1+k) = nint(tabout(iposcv+k-1))
         end do

      end if
      iinputpst(67) = idim

!     --- XY_ANGLE, INTERSECT_ANGLE, GROUND_VALUE, ZSCALE, &
!         BLOCK_MODE, YSCALE

      rinputpst(23) = table(76)
      rinputpst(24) = table(77)
      rinputpst(25) = table(78)
      rinputpst(26) = table(79)
      rinputpst(27) = table(81)
      iinputpst(68) = nint(table(80))
      if ( table(90)>0d0 ) then
         iposut = nint(table(90))
         nreadcv = nint(tabmax(90))
         if ( nreadcv/=2 ) then

!        --- nreadcv # 2

            call errint ( nreadcv, 1 )
            call errint ( 2, 2 )
            call errsub ( 1460, 2, 0, 0 )

         else
            iinputpst(73) = 1
            rinputpst(28) = tabout(iposut)
            rinputpst(29) = tabout(iposut+1)
         end if
      end if
      if ( table(91)>0d0 ) iinputpst(74) = 3

!     --- information about  mach number computation

      if ( table(92)>0d0 ) then

!     --- velocity vector found

         iposut = nint(table(92))+1
         iinputpst(2) = nint(tabout(iposut))

      end if

      if ( table(93)>0d0 ) then

!     --- enthalpy vector found

         iposut = nint(table(93))+1
         iinputpst(3) = nint(tabout(iposut))

      end if
      rpost(1) = table(94)
      if ( table(95)>0d0 ) iinputpst(75) = nint(table(95))
      ivecnm = nint(table(114))
      if ( table(100)>0d0 ) iinputpst(76) = 1
      iinputpst(77) = nint(table(101))-1
      if ( iinputpst(77)>1 ) iinputpst(77) = iinputpst(77)+8
      if ( table(102)>0d0 ) then

!     --- negpos_levels found, set ncntln

         if ( table(103)>0d0 ) then

!        --- rounded_levels found

            iinputpst(6) = -10001

         else

            iinputpst(6) = -10000

         end if

      else if ( table(103)>0d0 ) then

!     --- rounded_levels found

         iinputpst(6) = iinputpst(6)-1000

      end if
      if ( table(104)>0d0 ) iinputpst(78) = nint(table(104))
      if ( table(105)>0d0 ) iinputpst(79) = nint(table(105))
      if ( table(106)>0d0 ) iinputpst(80) = 1
      if ( table(107)>0d0 ) iinputpst(81) = nint(table(107))
      if ( table(108)>0d0 ) iinputpst(82) = nint(table(108))
      iinputpst(83) = nint(table(109))
      iinputpst(84) = nint(table(110))
      if ( table(99)>0d0 ) then

!     --- volumes found

         if ( iinputpst(32)>0 ) then

!        --- error combination skip element groups and volumes not allowed

            call errwar ( 2202, 0, 0, 0 )

         end if
         iposut = nint(table(99))
         iinputpst(85) = nint(tabmax(99))
         do i = 1, iinputpst(85)
            igroup(i) = nint(tabout(iposut+i-1))
         end do

      end if
      if ( table(111)>0d0 ) then

!     --- values found

         iposut = nint(table(111))
         iinputpst(85) = nint(tabmax(111))
         do i = 1, iinputpst(85)
            igroup(i) = nint(tabout(iposut+i-1))+1
         end do

      end if
      iinputpst(86) = nint(table(112))
      if ( table(113)>0d0 ) then

!     --- TSTEP_PRINT found

         rinputpst(30) = table(113)
         iinputpst(87) = 1

      end if
      iinputpst(88) = nint(table(25))
      iinputpst(89) = nint(table(115))
      if ( table(116)>=0d0 ) iinputpst(48) = 1
      if ( table(117)>=0d0 ) then

!     --- equidistant grid found

         iinputpst(91) = 1
         if ( table(118)<0d0 ) then

!        --- No distances found

            call errchr ( code1(117)(1:ncod1m(117)), 1 )
            call errchr ( code1(118)(1:ncod1m(118)), 2 )
            call errsub ( 2227, 0, 0, 2 )

         else

!        --- Distances found

            iposut = nint(table(118))
            do i = 0, ndim-1
               rinputpst(31+i) = tabout(iposut+i)
            end do

         end if
         if ( table(55)<0d0 ) then

!        --- No region found

            call errchr ( code1(117)(1:ncod1m(117)), 1 )
            call errchr ( code1(55)(1:ncod1m(55)), 2 )
            call errsub ( 2227, 0, 0, 2 )

         end if

      else if ( table(118)>0d0 ) then

!     --- No equidistant grid found

         call errchr ( code1(117)(1:ncod1m(117)), 2 )
         call errchr ( code1(118)(1:ncod1m(118)), 1 )
         call errsub ( 2227, 0, 0, 2 )
         go to 1000

      end if
      iinputpst(92) = nint(table(119))
      iinputpst(93) = nint(table(120))
      iinputpst(94) = nint(table(121))

!     --- TYPE_TIME_INTEGRATION = heun / runge_kutta / trapezoid

      if ( nint(table(123))==-1 ) then
         iinputpst(98) = 1
      else
         iinputpst(98) = nint(table(123))
      end if

!     --- EPS_TIME_STEP = eps

      rinputpst(34) = table(124)

!     --- TANG_X = ( x, y, z )
!         TANG_Y = ( x, y, z )
!         TYPE_TANG = xy / yz / xz / proj_xy / proj_yz / proj_xz

      if ( table(125)>0 ) then

!     --- tang_x found, this means that also tang_y must be present
!         but not type_tang

         if ( table(127)>0 ) then

!        --- type_tang found

            call errchr ( code1(127)(1:ncod1m(127)), 1 )
            call errchr ( code1(125)(1:ncod1m(125)), 2 )
            call errsub ( 1134, 0, 0, 2 )
            call errcrd
            iretur = -1

         end if

         if ( table(126)<=0 ) then

!        --- tang_y not found

            call errchr ( code1(125)(1:ncod1m(125)), 1 )
            call errchr ( code1(126)(1:ncod1m(126)), 2 )
            call errsub ( 2358, 0, 0, 2 )
            call errcrd
            iretur = -1

         else

!        --- tang_x and tang_y found

            iposut = nint(table(125))
            nread  = nint(tabmax(125))
            if ( nread/=3 ) then
               call errint ( nread, 1 )
               call errint ( 3, 2 )
               call errchr ( code1(125)(1:ncod1m(125)), 1 )
               call errsub ( 2359, 2, 0, 1 )
               call errcrd
               iretur = -1
            end if
            rinputpst(36) = tabout(iposut)
            rinputpst(37) = tabout(iposut+1)
            rinputpst(38) = tabout(iposut+2)

            iposut = nint(table(126))
            nread  = nint(tabmax(126))
            if ( nread/=3 ) then
               call errint ( nread, 1 )
               call errint ( 3, 2 )
               call errchr ( code1(126)(1:ncod1m(126)), 1 )
               call errsub ( 2359, 2, 0, 1 )
               call errcrd
               iretur = -1
            end if
            rinputpst(39) = tabout(iposut)
            rinputpst(40) = tabout(iposut+1)
            rinputpst(41) = tabout(iposut+2)
            iinputpst(99) = iinputpst(99)+70

         end if

      else if ( table(126)>0d0 ) then

!     --- tang_y found, but not tang_x

         call errchr ( code1(126)(1:ncod1m(126)), 1 )
         call errchr ( code1(125)(1:ncod1m(125)), 2 )
         call errsub ( 2358, 0, 0, 2 )
         call errcrd
         iretur = -1

      else if ( table(127)>0d0 ) then

!     --- type_tang found

         iinputpst(99) = iinputpst(99)+10*nint(table(127))

      end if

!     --- xscale

      rinputpst(42) = table(128)

      if ( table(129)>0d0 ) then

!     --- file_mesh_out found, store in textx (is not used in this case)

         if ( table(37)>0d0 ) &
            call errkeywexcl ( code1, ncod1m, 37, 129 )

         iposut = nint(table(129))
         nchar  = nint(tabmax(129))
         textx = ' '
         textx(1:nchar) = strout(iposut)(1:nchar)
         iinputpst(37) = nchar

      end if

      if ( table(130)>0d0 ) then

!     --- file_solut_out found, store in texty (is not used in this case)

         if ( table(38)>0d0 ) &
            call errkeywexcl ( code1, ncod1m, 38, 130 )

         iposut = nint(table(130))
         nchar  = nint(tabmax(130))
         texty = ' '
         texty(1:nchar) = strout(iposut)(1:nchar)
         iinputpst(38) = nchar

      end if

      if ( table(131)>0d0 ) jkader = -5
      if ( table(132)>0d0 ) jkader = -1
      rinputpst(43) = table(68)      ! text_size
      if ( table(134)>0d0 ) iinputpst(102) = 1  ! plot coordinates
      rinputpst(44) = table(136)      ! lev_text_size
      rinputpst(45) = table(137)      ! num_text_size
      if ( table(139)>=0d0 ) iinputpst(103) = 1  ! suppress_nodes

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iretur, ivecnm', iretur, ivecnm
         call prinin ( iinputpst, 103, 'iinputpst' )
         call prinrl ( rinputpst, 45, 'rinputpst' )
         write(irefwr,*) 'End read06'

      end if  ! ( debug )
      call erclos ( 'read06' )

      end subroutine read06
