      subroutine read08 ( kcode, value )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.3  date 20-04-2012 Extra debug
!        version  3.2  date 23-03-2008 Extra debug
!        version  3.1  date 01-09-2006 Extension of cpost1
!
!   copyright (c) 1989-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Analyse keyword after time history (t0,t1) for subroutine read05
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kcode
      double precision value

!     kcode          i    Return code for subroutine READ05
!                         If -1 an error has occured, otherwise the procedure
!                         is correct
!     value          o    Value read after POSITION, VALUE = &
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod3=2, nncod2=4
      character (len = 8) code3(nncod3), code2(nncod2)
      integer ncode3(nncod3), ncode2(nncod2), lcode, i, inum, &
              ncod3m(nncod3), ncod2m(nncod2)

!     code2          Table with keywords
!     code3          Table with keywords
!     i              Counting variable
!     inum           Number of numbers read in a record
!     lcode          return code produced by intx00
!     ncod2m         Table containing the number of maximum characters in
!                    the keyword table code2
!     ncod3m         Table containing the number of maximum characters in
!                    the keyword table code3
!     ncode2         Table containing the number of significant letters in code2
!     ncode3         Table containing the number of significant letters in code3
!     nncod2         Number of strings in code2
!     nncod3         Number of strings in code3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRBISC         computes the time step itime that is closest to the
!                    actual time t by the method of bisection
!     READSR         Read a series of numbers from the input (texts are
!                    neglected)
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
!    This subroutine interprets the part of the records for seppost
!    following TIME HISTORY
!    The following items are recognized:
!    [ (t0,t1) ] ,options
!    with options:
!    PRINT MIN Vi
!    PRINT MAX Vi
!    PRINT MIN ABS Vi
!    PRINT MAX ABS Vi
!    PRINT position, value = a Vi
!    PRINT POINT(x,y,z) Vi [,DEGFD=k]
!    PLOT MIN Vi
!    PLOT MAX Vi
!    PLOT MIN ABS Vi
!    PLOT MAX ABS Vi
!    PLOT position, value = a Vi
!    PLOT POINT(x,y,z) Vi [,DEGFD=k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     683   Wrong word found after TIME HISTORY
!     693   No numbers read after TIME HISTORY POINT
!    1190   subkeyword not recognized
!    1869   No number found after keyword
!    2671   No keyword found after keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code3 / 'PLOT', 'PRINT' /, ncode3 / 2, 2 /, ncod3m / 4, 5 /
      data code2 / 'MIN', 'MAX', 'POINT', 'POSITION' /, &
           ncode2 / 3, 3, 4, 4 /, &
           ncod2m / 3, 3, 5, 8 /

!     code3   Table of first set of keywords.
!             Possible values
!             1:  PLOT
!             2:  PRINT
!     code2   Table of second set of keywords.
!             Possible values
!             1:  MIN
!             2:  MAX
!             3:  POINT
!             4:  POSITION
! ======================================================================
!
      call eropen ( 'read08' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read08'
         write(irefwr,1) 'ityprd, lastitem, typeitems(lastitem)', &
                          ityprd, lastitem, typeitems(lastitem)
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iinputpst(1) = 300
      ierpri = 1
      kcode = -1
      call asknxt

      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem, typeitems(lastitem)/1', &
                          ityprd, lastitem, typeitems(lastitem)
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

      iltime(1) = 1
      iltime(2) = ntime
      if ( ityprd==2 .or. ityprd==3 ) then

!     --- number t0 found

         getal(1) = realrd
         getal(2) = 1d10
         call asknxt
         if ( ityprd==2 .or. ityprd==3 ) then
            getal(2) = realrd
            call asknxt
         end if
         do i = 1, 2
            call prbisc ( getal(i), time, iltime(i), ntime )
         end do

      end if
      if ( ityprd/=4 ) then

!     --- Error: Not a keyword found

         call errsub ( 683, 0, 0, 0 )
         call errcrd
         go to 1000

      else

!     --- keyword found, check next keywords

         call sepcopycode ( code3, nncod3, ncode3, ncod3m )
         lcode = sepsubkeyw()
         if ( debug ) write(irefwr,1) 'lcode', lcode
         if ( lcode==0 ) go to 1000
         if ( lcode==1 ) then

!        --- PL found, i.e. PLOT check rest of string

            kcode = 10

         else

!        --- PR found, i.e. PRINT check rest of string

            kcode = 0

         end if
         if ( debug ) write(irefwr,1) 'kcode', kcode

!        --- read next keyword

         call asknxt
         if ( debug ) then
            write(irefwr,1) 'ityprd, lastitem, typeitems(lastitem)/2', &
                             ityprd, lastitem, typeitems(lastitem)
            if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         call sepcopycode ( code2, nncod2, ncode2, ncod2m )
         lcode = sepsubkeyw()
         if ( debug ) write(irefwr,1) 'lcode', lcode

         if ( lcode==0 ) then
            kcode = -1
            go to 1000
         end if
         if ( lcode==1 ) then

!        --- MIN found

            kcode = kcode + 1

         else if ( lcode==3 ) then

!        --- POINT found

            kcode = kcode + 2

!           --- Read numbers between brackets

            call readsr ( rinputpst(10), 4, inum )
            if ( inum==0 .or. inum>=4 ) then
               call errint ( inum, 1 )
               call errsub ( 693, 1, 0, 0 )
               call errcrd
               kcode = -1
            end if
            go to 1000

         else if ( lcode==4 ) then

!        --- POSITION found, read value = ...

            kcode = kcode + 3
            call asknxt
            if  ( ityprd==4 ) then

!           --- string found, must be VALUE

               if ( string(1:3)/='VAL' ) then

!              --- Incorrect keyword found

                  call errchr ( string(1:nchar), 1 )
                  call errchr ( 'POSITION', 2 )
                  call errchr ( 'VALUE', 3 )
                  call errsub ( 1190, 0, 0, 3 )
                  call errcrd
                  go to 1000

               end if  ! ( string(1:3)/='VAL' )
               call asknxt
               if ( ityprd<2 .or. ityprd>3 ) then

!              --- No number found after value = &

                  call errchr ( 'POSITION', 1 )
                  call errsub ( 1869, 0, 0, 1 )
                  call errcrd
                  go to 1000

               end if  ! ( ityprd<2 .or. ityprd>3 )
               value = realrd

            else

!           --- No keyword found

               call errchr ( 'POSITION', 1 )
               call errsub ( 2671, 0, 0, 1 )
               call errcrd
               go to 1000

            end if  !  ( ityprd==4 )

         end if

!        --- Check for presence of ABS

         call asknxt
         if ( ityprd==4 ) then
            if ( string(1:3)=='ABS' ) then
               kcode = kcode + 5
               call asknxt
            end if
         end if

      end if

1000  if ( kcode/=-1 ) iinputpst(20) = kcode
      if ( typeitems(lastitem)==3 ) ityprd = 4
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd, lastitem, typeitems(lastitem)/3', &
                          ityprd, lastitem, typeitems(lastitem)
         write(irefwr,1) 'kcode', kcode
         write(irefwr,*) 'End read08'

      end if  ! ( debug )
      call erclos ( 'read08' )

      end
