      subroutine read74sub
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-04-2012 Indexed vectors
!        version  1.0  date 28-11-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read names of vectors from the sepcomp.out file
!     Store in array names
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmodulecomio
      use sepmodulemach
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ios, numvecsav
      character (len=80) textloc

!     i              Counting variable
!     ios            IOSTAT variable, returns from reading or writing
!     numvecsav      Help parameter to store the value of numvec
!     textloc        String to store a local text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read74sub' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read74sub'

      end if
      if ( ierror/=0 ) go to 1000
      numvecsav = numvec

!     --- Read numvec, lenipost, lenrpost

      if ( iref74>0 ) then

!     --- formatted

         read(iref74,100, iostat=ios) textloc
         if ( ios/=0 ) go to 1000
100      format(a)
         read ( iref74,*, iostat=ios ) numvec, (ipost(i),i=1,2)
         if ( ios/=0 ) go to 1000

!        --- Read names of arrays

         if ( numvec>0 ) then
            read(iref74,100, iostat=ios) textloc
            if ( ios/=0 ) go to 1000
            read ( iref74,110, iostat=ios ) (names(i),i=1,numvec)
110         format(a32)
            if ( ios/=0 ) go to 1000
         end if

!        --- Read ipost/rpost

         if ( ipost(1)>0 ) then
            read(iref74,100, iostat=ios) textloc
            if ( ios/=0 ) go to 1000
            read ( iref74,*, iostat=ios ) (ipost(i),i=1,ipost(1))
            if ( ios/=0 ) go to 1000
         end if
         if ( ipost(2)>0 ) then
            read(iref74,100,iostat=ios) textloc
            if ( ios/=0 ) go to 1000
            read ( iref74,*, iostat=ios ) (rpost(i),i=1,ipost(2))
            if ( ios/=0 ) go to 1000
         end if

      else

!     --- unformatted

         read(-iref74, iostat=ios) textloc
         if ( ios/=0 ) go to 1000
         read ( -iref74, iostat=ios ) numvec, (ipost(i),i=1,2)
         if ( ios/=0 ) go to 1000
         if ( numvec>0 ) read(-iref74, iostat=ios) textloc

!        --- Read names of arrays

         do i = 1, numvec
            read ( -iref74,iostat=ios ) names(i)
            if ( ios/=0 ) go to 1000
         end do

!        --- Read ipost/rpost

         if ( ipost(1)>0 ) then
            read(-iref74, iostat=ios) textloc
            read ( -iref74, iostat=ios ) (ipost(i),i=1,ipost(1))
            if ( ios/=0 ) go to 1000
            if ( ios/=0 ) go to 1000
         end if
         if ( ipost(2)>0 ) then
            read(-iref74, iostat=ios) textloc
            if ( ios/=0 ) go to 1000
            read ( -iref74, iostat=ios ) (rpost(i),i=1,ipost(2))
            if ( ios/=0 ) go to 1000
         end if

      end if

!     --- Set ncharnames

      do i = 1, numvec
         ncharnames(i) = index ( names(i), ' ' ) -1
         if ( ncharnames(i)<0 ) ncharnames(i) = 32
      end do  ! i = 1, numvec

!     --- To avoid destruction of vectors that are defined already we
!         make numvec the maximum of old value and parts read

      numvec = max(numvec,numvecsav)

1000  call erclos ( 'read74sub' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read74sub'

      end if

      end
