      subroutine readc0fill ( numint, ndegfd, nread, icompl, &
                              inforinpcre, rincre, inpcre, nreal )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-04-2012 Extension with series of surfaces
!        version  1.0  date 06-07-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information read by readc0 and interpreted by readc0interpr
!     in inpcre
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numint, ndegfd, nread, icompl, inforinpcre(*), inpcre(*), &
              nreal
      double precision rincre(*)

!     icompl         i    Indicates if vector is real (0) or complex (1)
!     inforinpcre    o    Array of length of rincre that contains a 0 if the
!                         number stored in rincre is a real and a reference
!                         to scalars if it is a scalar
!     inpcre         o    The integer input read is stored in array INPCRE from
!                         position 1. Hence in the call INPCRE(ISTART), with
!                         ISTART the starting position must be used.
!                         INPCRE will be filled in correspondence with array
!                         INPCRE for subroutine CREATV
!     ndegfd         i    Number of degrees of freedom per point
!     nread          i    Number of items read
!     nreal         i/o   Number of reals stored in RINCRE
!     numint        i/o   Number of integers stored
!     rincre        i/o   Double precision array containing the values of the
!                         constant if KFUN<0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, m, inum

!     i              Counting variable
!     inum           Number of surfaces
!     m              number of items
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1069   A quadratic profile may only be described for one degree of freedom
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readc0fill' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readc0fill'
         write(irefwr,1) 'ndegfd, nread', ndegfd, nread
         call prinin ( iwork, 3, 'iwork' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numint = 1
      if ( ndegfd==0 ) then

!     --- All degrees of freedom

         inpcre(1) = 0
         if ( iwork(1)>=9001 .and. iwork(1)<=9004 ) inpcre(1) = 1

      else if ( ndegfd==1 ) then

!     --- One degree of freedom

         inpcre(1) = iwork(100)

      else if ( iwork(1)>=9001 .and. iwork(1)<=9004 ) then

!     --- Quadratic profile may only be given for one degree of freedom

         call errsub ( 1069, 0, 0, 0 )

      else

!     --- Specific degrees of freedom

         inpcre(1) = -ndegfd
         do i = 1, ndegfd
            inpcre(i+1) = iwork(99+i)
         end do
         numint = 1 + ndegfd

      end if

!     --- Indication of how to be filled

      numint = numint + 1
      if ( nread>0 ) then

!     --- nread>0, value read

         inpcre(numint) = -(nreal+1)
         nreal = nreal + nread

      else if ( iwork(1)==0 ) then

!     --- VALUE nor FUNC found, Set VALUE equal to zero

         inpcre(numint) = -(nreal+1)
         inforinpcre(nreal+1) = 0
         rincre(nreal+1) = 0d0
         if ( icompl==1 ) then
            inforinpcre(nreal+2) = 0
            rincre(nreal+2) = 0d0
         end if
         nreal = nreal + 1 + icompl

      else

!     --- FUNC found

         inpcre(numint) = iwork(1)
         if ( iwork(1)>=9001 .and. iwork(1)<=9004 ) then

!        --- QUADRATIC found, store MAX

            inpcre(numint+1) = nreal+1
            nreal = nreal+1
            numint = numint+1
            if ( iwork(1)==9003 ) then

!           --- quadratic with mean value and band width

               nreal = nreal+1

            else if ( iwork(1)==9004 ) then

!           --- quadratic with mean value, difference and band width

               nreal = nreal+2

            end if  ! ( iwork(1)==9003 )

         else if ( iwork(1)>3000 .and. iwork(1)<4000 ) then

!        --- old vectors found, fill the vector sequence numbers

            m = iwork(111)
            numint = numint+1
            inpcre(numint) = m
            do i = 1, m
               inpcre(numint+i) = iwork(111+i)
            end do
            numint = numint+m

         end if

      end if

!     --- Fill indication in which nodal points the vector must be filled

      numint = numint + 1
      inpcre(numint) = iwork(2)
      m = iwork(3)

      if ( debug ) then
         call prinin ( inpcre, numint, 'inpcre/1' )
         write(irefwr,1) 'm, iwork(2)', m, iwork(2)
      end if  ! ( debug ) then

      select case ( iwork(2) )

      case (1)

!     --- POINTS

         numint = numint + 1
         inpcre(numint) = m
         inpcre(numint+1:numint+m) = nint(work(100:99+m))
         numint = numint + m

      case(2)

!     --- curves

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(5)
         inpcre(numint+3) = iwork(3)
         numint = numint + 3

      case(3)

!     --- surfaces

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(5)
         inpcre(numint+3) = m
         numint = numint + 3
         inpcre(numint+1:numint+m) = nint(work(100:99+m))
         numint = numint + m

      case(4)

!     --- nodes

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(5)
         inpcre(numint+3) = max(1,iwork(3))
         numint = numint + 3

      case(5, 7)

!     --- elements, volumes

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(5)
         inpcre(numint+3) = iwork(3)
         numint = numint + 3
         inpcre(numint+1:numint+m) = nint(work(100:99+m))
         numint = numint + m

      case(6)

!     --- element groups

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(3)
         numint = numint + 2
         inpcre(numint+1:numint+m) = nint(work(100:99+m))
         numint = numint + m

      case(8)

!     --- USER POINTS with step

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(5)
         inpcre(numint+3) = iwork(3)
         numint = numint + 3

      case(9:20)

!     --- obstacle i, contact i or no_contact i or file_nodal_value or
!         file_element_value or global group or in_xx_obstacles found or
!         zero_levelset

         inpcre(numint+1) = iwork(3)
         numint = numint + 1

      case(22)

!     --- series of curves

         inpcre(numint+1) = iwork(3)
         m = iwork(4)
         inpcre(numint+2) = m
         inpcre(numint+3:numint+m+2) = iwork(5:4+m)
         numint = numint + 2+m

      case(24, 25)

!     --- series of surfaces or volumes

         inum = iwork(4)    ! number of surfaces
         inpcre(numint+1) = inum
         inpcre(numint+2) = m     ! number of local nodes
         inpcre(numint+3:numint+inum+2) = iwork(5:4+inum)  ! surfaces
         if ( debug ) &
            call prinin ( inpcre(numint+1), inum+2, 'inpcre_sur/1' )
         numint = numint + 2+inum
         inpcre(numint+1:numint+m-1) = nint(work(100:99+m))  ! rel nodes
         if ( debug ) &
            call prinin ( inpcre(numint+1), m, 'inpcre_sur/2' )
         numint = numint + m
         if ( debug ) write(irefwr,1) 'numint, inum, m', numint, inum, m

      end select ! case ( iwork(2) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readc0fill'

      end if  ! ( debug )
      call erclos ( 'readc0fill' )

      end subroutine readc0fill
