      subroutine readconsdef ( realname, value, valdef )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-01-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if real constant can be found in part constants of input file
!     If so, set constant otherwise use default value
! **********************************************************************
!
!                       KEYWORDS
!
!     name
!     constant
!     get_parameter
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision value, valdef
      character (len=*) realname

!     realname       i    Name of real
!     valdef         i    Default value
!     value          o    Value of real
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnamereal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETNAMEREAL    Return with sequence number of real in rlcons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readconsdef' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readconsdef'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get sequence number of real

      iseq = getnamereal ( realname )
      if ( iseq>0 ) then

!     --- Sequence number>0, hence constant found, fill value

         value = rlcons(iseq)

      else

!     --- Sequence number = 0, use default value

         value = valdef

      end if  ! ( iseq>0 )

1000  call erclos ( 'readconsdef' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readconsdef'

      end if  ! ( debug )

      end
