      subroutine readfor ( ivar, ibeginfor, istepfor, iendfor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 19-04-2014 Use readstrnames
!        version  1.2  date 08-12-2013 Extension with integer scalars
!        version  1.1  date 06-02-2012 Debug
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keywords after the keyword for
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivar, ibeginfor, istepfor, iendfor

!     ibeginfor      o    First value of for loop
!     iendfor        o    Last value of for loop
!     istepfor       o    Step of for loop
!     ivar           o    Sequence number of variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ispeclsav, icodesav

!     icodesav       Help parameter to save the value of icode
!     ispeclsav      Help parameter to save the value of ispecl
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ADDINTSCAL     Add integer scalar name to the list of scalars
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     READSTRNAMES   Check if string belongs to the list of constants
!                    variables or vectors
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2117   not %var found after for
!    2118   no integer after variable in for loop
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Read name of variable
!      Read beginloop
!      Read endloop and loopstep
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readfor' )
      debug = .false. .and. ioutp>=0
      icodesav = icode
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readfor'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call asknxt

      if ( debug ) write(irefwr,1) 'ityprd', ityprd

      if ( ityprd==4 ) then

!     --- word found, check if it is a variable
!         it might be one containing a number, so we have to reread

         lastitem = lastitembef
         ispeclsav = ispecl
         ispecl = 9
         call asknxt
         if ( debug ) write(irefwr,1) 'ityprd/2', ityprd
         ispecl = ispeclsav
         call readstrnames
         if ( debug ) write(irefwr,1) 'ityprd/3', ityprd

      end if  ! ( ityprd==4 ) then

      iretur = 0

      if ( ityprd==4 ) then

!     --- string found that is not in the list of known variables

         call addintscal  ! add to list of integer scalars

      else if ( ityprd/=11 ) then

!     --- Not variable found

         call errchr ( string(1:nchar), 1 )
         call errsub ( 2117, 0, 0, 1 )
         call errcrd
         go to 1000

      end if  ! ( ityprd==4 )

!     --- set ivar

      ivar = nint(realrd) - 100000000

      if ( debug ) write(irefwr,1) 'ivar', ivar

      iscalars(ivar) = 1

!     --- Read begin step

      call asknxt
      if ( ityprd/=2 .and. ityprd/=11 ) then

!     --- No integer found

         call errsub ( 2118, 0, 0, 0 )
         call errcrd
         ibeginfor = 1

      else

!     --- integer found

         ibeginfor = nint(realrd)

      end if

!     --- read endloop and step

      code(1) = 'STEP'
      code(2) = 'TO'
      ncode(1) = 4
      ncode(2) = 2
      ncodmx(1:2) = ncode(1:2)
      itype(1:2) = 0
      tabmin(1:2) = 1d0
      tabmax(1) = 1d3
      tabmax(2) = 1d6

      table(1) = 1d0
      table(2) = ibeginfor
      tabmin(2) = ibeginfor
      nmain = 0
      call sepreadsetkeyw
      istepfor =  nint(table(1))
      iendfor =  nint(table(2))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readfor'

      end if  ! ( debug )
      call erclos ( 'readfor' )
      icode = icodesav

      end subroutine readfor
