      subroutine readin ( iinint, isubr, iseqnr, nelgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 21-10-2011 Extension with levelset
!        version  2.0  date 26-08-2011 Remove tabout from parameter list
!        version  1.4  date 08-04-2002 Extension of iinint
!
!   copyright (c) 1993-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Read the input for the subroutine INTEGR
!      The input is stored in array IININT
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinint(*), isubr, iseqnr, nelgrp

!    iinint  o   The integer input read is stored in array IININT
!                Array IININT is filled as follows:
!                1  ICHELI, choice parameter for the element subroutines
!                   Default value: 1
!                2  JDEGFD,  choice parameter for the element subroutines
!                   Default value: 1
!                3  MAXMIN, Indication if only the integral must be computed or
!                           also minimum and maximum values of the integrals
!                           per element. Possible values:
!                           0:  Only the integral is computed
!                           1:  The integral as well as the minimum and maximum
!                               value per element are computed
!                           2:  The integral as well as the minimum and maximum
!                               absolute value of the integral per element are
!                               computed
!                   Default value: 0
!                4  ALLELEMENTS Indication if all elements are used
!                          or not
!                          Possible values:
!                          -2: all elements corresponding to part
!                              levelset not active
!                          -1: all elements corresponding to part
!                              levelset active
!                           0: all elements are used
!                           >0: Some element groups are skipped
!                           In that case the positions ALLELEMENTS to
!                           ALLELEMENTS+NELGRP-1
!                           (NELGRP is number of element groups)
!                           are considered as an array IELHLP to be
!                           filled by the user in the following way:
!                           In IELHLP(IELGRP)  ( 1<=IELGRP<=NELGRP)
!                           the user must store, whether elements with
!                           standard element sequence number IELGRP
!                           are added to the large matrix and large vector or
!                           not.
!                           When IELHLP(IELGRP) = 0 the elements with standard
!                           element sequence number IELGRP are skipped, when
!                           IELHLP(IELGRP) = 1 these elements are added.
!                   Default value: 0
!                5  NUMVEC  Number of vectors that are stored in array
!                           INPVEC
!                           Compare with subroutine BUILD
!                           If NUMVEC=0, it is supposed that one vector
!                           should be used
!                   Default value: 0
!                6  ISEQ    Sequence number of input vector
!                   Default   value: 1
!                7  ILEV,   Level set number (ichoice<0)
!    iseqnr i/o  Sequence number with respect to the storage in array ISEPIN
!    isubr   i   Indicates by which subroutine READDR is called
!                Possible values:
!                1:  DERIVS,  read record with DERIVATIVES
!                2:  ERDERI,  do not read record with DERIVATIVES
!    nelgrp  i   Number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=9, nncod2=3
      integer :: ncode1(nncode1), itype1(nncode1), &
                 ncode2(nncod2), ncodmx1(nncode1)
      integer :: k, iskip
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=20) :: code1(nncode1)
      character (len=3) :: code2(nncod2)

!     code1         Table with keywords corresponding to options
!     code2         Table with the keywords following MAXMIN_ELEMENTS
!                   Values:
!                   1: no
!                   2: yes
!                   3: absolute
!     iskip         Defines where the information of the element groups must
!                   be stored
!     itype1        itype array for code1 (See intx02)
!     k             Counting variable
!     ncode1        Table containing the number of significant characters in
!                   the keyword table code1
!     ncode2        See ncode1, now for table code2
!     ncodmx1       Table containing the number of maximum characters in
!                   the keyword table code1
!     nncod2        Number of entries in table code2
!     nncode1       Number of entries in table code1
!     tabmax1       Table with maximum values of numbers corresponding to code1
!     tabmin1       Table with minimum values of numbers corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     readin requires input from the standard input file
!     The following input is recognized:
!     INTEGRAL [, SEQUENCE_NUMBER = s]      (mandatory):
!        start of input
!      optional records:
!        ICHELI = s                               (Default 1 )
!        ELEMENT_GROUPS = (s1, s2, ...)           (Default all)
!        SKIP_ELEMENT_GROUPS = (s1, s2, ... )     (Default none)
!        DEGREE_OF_FREEDOM = d                    (Default 1)
!        MAXMIN_ELEMENTS = choice                 (Default no)
!            with choice = no
!                          yes
!                          abs
!        NUMVEC = n                               (Default 0)
!        SEQ_INPUT_VECTOR = i                     (Default 1)
!        ACTIVE_LEVEL_SET i
!        NON_ACTIVE_LEVEL_SET i
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     650   First record read by subroutine is not equal to SOLVE
!    1900   Both element groups and skip element groups given
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (derivatives)
!    Read rest of first record (sequence number, problem number )
!    Read the rest of the records by intx02 unitl END is found
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'ICHELI', 'ELEMENT_GROUPS', 'SKIP_ELEMENT_GROUPS', &
                  'DEGREE_OF_FREEDOM', 'MAXMIN_ELEMENTS', 'NUMVEC', &
                  'SEQ_INPUT_VECTOR', &
                  'ACTIVE_LEVEL_SET', 'NON_ACTIVE_LEVEL_SET' /
      data ncode1 / 4, 4,  4, 2, 5,  4, 6, 5, 8 /
      data ncodmx1/ 6,14, 19,17,15,  6,16,16,20 /
      data itype1 / 0,11, 11, 0, 5,  0, 0, 0, 0 /
      data tabmin1 / 1d0, 1d0, 1d0, 1d0, 1d0,   1d0, 1d0, 1d0, 1d0 /
      data tabmax1 / 1d3, 1d0, 1d0, 1d1, 3d0,   1d2, 1d2, 1d0, 1d0 /
      data code2 / 'NO', 'YES', 'ABS' /
      data ncode2 / 2, 3, 3 /

!     code1   Table with keywords corresponding to options
!             Values:
!             1:  ICHELI
!             2:  ELEMENT_GROUPS
!             3:  SKIP_ELEMENT_GROUPS
!             4:  DEGREE_OF_FREEDOM
!             5:  MAXMIN_ELEMENTS
!             6:  NUMVEC
!             7:  SEQ_INPUT_VECTOR
!             8:  ACTIVE_LEVEL_SET
!             9:  NON_ACTIVE_LEVEL_SET
! ======================================================================
!
      call eropen ( 'readin' )

      iinint(1:5) = 0
      iinint(1:2) = 1
      iinint(6) = 1
      iskip = 7
      iinint(iskip:iskip+nelgrp-1) = 0

      if ( isubr==1 ) then

!     --- Call by INTEG, read first record  (INTEGRAL)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'INTEGRAL', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:5)/='INTEG' ) then
            call errchr ( 'INTEGRAL', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

      call sepreadseqnr (iseqnr  )   ! read sequence number
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      call sepcopycodesub ( code2, nncod2, ncode2 )
      call sepsetmainend                  ! Fill END as main keyword

      table(1:nncode) = -1d20
      tabmax(2) = nelgrp
      tabmax(3) = nelgrp
      tabmin(2) = nelgrp
      tabmin(3) = nelgrp
      istop = 2
      iretur = 0
      call sepreadsetkeyw

!     --- Check which keywords are mutually exclusive

      if ( table(8)>0d0 .and. table(9)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 8, 9 )

      istop = 0
      if ( table(1)>0d0 )  iinint(1) = nint(table(1))
      if ( table(4)>0d0 )  iinint(2) = nint(table(4))
      if ( table(5)>0d0 )  iinint(3) = nint(table(5))
      if ( table(6)>0d0 )  iinint(5) = nint(table(6))
      if ( table(7)>0d0 )  iinint(6) = nint(table(7))

      if ( table(2)>0d0 .or. table(3)>0d0 ) then

!     --- Element groups must be skipped

         iskip = 7
         iinint(4) = iskip
         if ( table(2)>0d0 ) then

!        --- element groups = given

            do k = 1, nelgrp
               iinint(iskip-1+k) = 0
            end do
            do k = 1, nint(table(2))
               iinint(iskip-1+nint(tabout(k))) = 1
            end do

         else

!        --- skip element groups = given

            do k = 1, nelgrp
               iinint(iskip-1+k) = 1
            end do
            do k = 1, nint(table(3))
               iinint(iskip-1+nint(tabout(k))) = 0
            end do

         end if
         if ( table(2)>0d0 .and. table(3)>0d0 ) &
              call errsub ( 1900, 0, 0, 0 )

      else

         iinint(4) = 0

      end if

      if ( table(8)>0d0 ) then

!     --- Active part of level set

         iinint(4) = -1              ! ichoice
         iinint(7) = nint(table(8))  ! level set

      else if ( table(9)>0d0 ) then

!     --- Active part of level set

         iinint(4) = -2              ! ichoice
         iinint(7) = nint(table(9)) ! level set

      end if  ! ( table(8)>0d0 )

1000  call erclos ( 'readin' )
      end subroutine readin
