      subroutine readnlloc ( iinsol, rinsol, istart, iposrl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-01-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read local options for non-linear equations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iposrl, iinsol(*), istart
      double precision rinsol(*)

!     iinsol        i/o   Contains information concerning the non-linear solver
!                         See readnl
!     iposrl        i/o   Position in array rinsol
!     istart        i/o   Starting address in array iinsol
!     rinsol        i/o   Contains information concerning the non-linear solver
!                         See readnl
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     READNLFILL     Fill arrays itype2, tabmn2, tabmx2 for subroutine readnl
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readnlloc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readnlloc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill code and codesub and related arrays

      call readnlfill

      table(4:6) = -1d0
      table(7) = 0d0
      table(10) = 1d0
      table(11) = 0d0
      table(12:13) = -1d0
      table(14:15) = 0d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( iretur==-1 ) go to 1000

      if ( table(4)>0d0 ) then
         iposrl = iposrl + 1
         rinsol(iposrl) = table(4)
         iinsol(istart+1) = iposrl
      end if
      if ( table(5)>=0d0 ) then
         iposrl = iposrl + 1
         rinsol(iposrl) = table(5)
         iinsol(istart+2) = iposrl
      end if

      iinsol(istart) = nint(table(6))
      iinsol(istart+3) = nint(table(7))
      iinsol(istart+10) = nint(table(10))-1
      iinsol(istart+11) = nint(table(11))
      iinsol(istart+14) = nint(table(14))
      iinsol(istart+15) = nint(table(15))
      if ( table(12)>=0d0 .or. table(13)>=0d0 ) then
         if ( table(12)>=0d0 .and. table(13)>=0d0 ) &
            call errkeywexcl ( code, ncodmx, 12, 13 )

         if ( table(12)>=0d0 ) then

!        --- absolute residual criterion

            iposrl = iposrl + 1
            rinsol(iposrl) = table(12)
            iinsol(istart+12) = iposrl
            iinsol(istart) = iinsol(istart) + 100

         else

!        --- relative residual criterion

            iposrl = iposrl + 1
            rinsol(iposrl) = table(13)
            iinsol(istart+13) = iposrl
            iinsol(istart) = iinsol(istart) + 200

         end if
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readnlloc'

      end if  ! ( debug )
      call erclos ( 'readnlloc' )

      end subroutine readnlloc
