      subroutine readoutcvec ( vector, nlines )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 01-04-2000 Extra parameter nfictunk
!        version  1.0  date 13-10-1999
!
!   copyright (c) 1999-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read complex sepran solution array from the file sepcomp.out
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex vector(*)
      integer nlines

!     nlines         i    number of lines that are read
!     vector         o    solution vector to be read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inunkp, idum, nusolclassic
      logical skip
      double complex locvalue(20)

!     i              Counting variable
!     idum           General dummy variable
!     inunkp         Number of degrees of freedom in point
!     j              Counting variable
!     locvalue       Local array to store some values
!     nusolclassic   Number of free classical unknowns, i.e ficitious unknowns
!                    and global unknowns excluded
!     skip           If true the next part must be skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( indprpo.ne.0 ) then

!     --- indprpo#0, use kprob part p

         do i = 1, nlines
            if ( indprfo.eq.0 ) then
               skip = .false.
            else if ( kprobfo(i+1)-kprobfo(i).eq.0 ) then
               skip = .true.
            else
               skip = .false.
            end if
            if ( .not. skip ) then
               if ( iref74.gt.0 ) then
                  read(iref74,*) idum, (locvalue(j),j=1,nphyso)
               else
                  read(-iref74) idum, (locvalue(j),j=1,nphyso)
               end if
               do j = 1, nphyso
                  if ( kprobpo(i,j).ne.0 ) then
                     vector(kprobpo(i,j)) = locvalue(j)
                  end if
               end do

            end if
         end do

      else if ( indprfo.ne.0 ) then

!     --- indprfo#0, use kprob part f

         do i = 1, nlines
            inunkp = kprobfo(i+1)-kprobfo(i)
            if ( inunkp.gt.0 ) then
               if ( iref74.gt.0 ) then
                  read(iref74,*) idum, (vector(j+kprobfo(i)),j=1,inunkp)
               else
                  read(-iref74) idum, (vector(j+kprobfo(i)),j=1,inunkp)
               end if
            end if
         end do
      else

!     --- Standard situation, indprfo=0 and indprpo=0

         do i = 1, nlines
            if ( iref74.gt.0 ) then
               read(iref74,*) idum, (vector(j+(i-1)*nphyso),j=1,nphyso)
            else
               read(-iref74) idum, (vector(j+(i-1)*nphyso),j=1,nphyso)
            end if
         end do

      end if

      nusolclassic = nusolo-nfictunk-nglunk
      if ( nfictunk.gt.0 ) then

!     --- nfictunk > 0, read fictitious unknowns

         if ( iref74.gt.0 ) then
            read(iref74,*) idum, (vector(j+nusolclassic),j=1,nfictunk)
         else
            read(-iref74) idum, (vector(j+nusolclassic),j=1,nfictunk)
         end if

      end if

      nusolclassic = nusolclassic+nfictunk
      if ( nglunk.gt.0 ) then

!     --- nglunk > 0, read global unknowns

         if ( iref74.gt.0 ) then
            read(iref74,*) idum, (vector(j+nusolclassic),j=1,nglunk)
         else
            read(-iref74) idum, (vector(j+nusolclassic),j=1,nglunk)
         end if

      end if

      end
