      subroutine readscbreak ( lastloop, iseqloop, iwhile, iseqwh, &
                               ifreest, iforloop, iseqfor, ilooptm, &
                               iseqtm, istart, inpsep, iloop, iseqlp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-09-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information of the break option depending on the type of loop
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     loop
!     break
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lastloop(*), iseqloop, iwhile(*), iseqwh, ifreest, &
              iforloop(*), iseqfor, ilooptm(*), iseqtm, istart, &
              inpsep(*), iloop(*), iseqlp

!     iforloop      i/o   Array of size 10 in which information of the loops
!                         is stored in the following way:
!                         iloop(i) = starting address of for loop in inpsep
!     ifreest       i/o   ifreest = starting address of stationary free boundary
!                         in inpsep
!     iloop         i/o   Array of size 10 in which information of the loops
!                         is stored in the following way:
!                         iloop(i) = starting address of while loop in inpsep
!     ilooptm       i/o   Array of size 1 in which information of the time loops
!                         is stored in the following way:
!                         ilooptm(i) = starting address of while loop in inpsep
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         In this subroutine we store 3 positions from istart
!                         1:  103 to indicate break
!                         2:  icode  defines type of end of loop (see readsc)
!                         3:  starting address in inpsep of loop to be ended
!     iseqfor       i/o   Present level of for loop
!     iseqloop      i/o   Present level of active loop
!     iseqlp        i/o   Present level of loop
!     iseqtm        i/o   Present level of time loop
!     iseqwh        i/o   Present level of while loop
!     istart        i/o   Next position in inpsep
!     iwhile        i/o   Array of size 10 in which information of the while loops
!                         is stored in the following way:
!                         iwhile(i) = starting address of while loop in inpsep
!     lastloop       i    Contains the type of loops that are active
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icode

!     icode          Defines type of loop
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2917   Option not available
!    3115   Break found, but the program is not in a loop
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscbreak' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscbreak'
         write(irefwr,1) 'iseqloop, istart', iseqloop, istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iseqloop==0 ) then

!     --- No loop active

         call errsub ( 3115, 0, 0, 0 )
         go to 1000

      end if  ! ( icode==0 )

      inpsep(istart) = 103  ! icode
      icode = lastloop(iseqloop)
      if ( debug ) write(irefwr,1) 'icode', icode

      select case ( icode )

         case (22)

!        --- End while loop

            inpsep(istart+1) = icode
            inpsep(istart+2) = iwhile(iseqwh)
            istart = istart + 3

         case (24, 27)

!        --- Not implemented

            inpsep(istart+1) = icode
            inpsep(istart+2) = ifreest+3
            istart = istart + 3

         case (38)

!        --- isubr = 38, End for loop

            inpsep(istart+1) = icode
            inpsep(istart+2) = iforloop(iseqfor)
            istart = istart + 3

         case (45)

!        --- isubr = 45, call subroutine TOLOOP and check on convergence

            inpsep(istart+1) = icode
            inpsep(istart+2) = iloop(iseqlp)
            istart = istart + 3

         case (47)

!        --- isubr = 47, End time loop

            inpsep(istart+1) = icode
            inpsep(istart+2) = ilooptm(iseqtm)
            istart = istart + 3

         case default

!        --- icode not implemented

            call errint ( icode, 1 )
            call errsub ( 2917, 1, 0, 0 )

      end select  ! case ( icode )

1000  call erclos ( 'readscbreak' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         call prinin ( inpsep(istart-2), 2, 'inpsep' )
         write(irefwr,*) 'End readscbreak'

      end if  ! ( debug )

      end
