      subroutine readscbubble ( istart, inpsep, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 03-07-2012 Use subroutine intxstoretext
!        version  1.1  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.0  date 01-05-2010
!
!   copyright (c) 2010-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keyword
!     COMPUTE_BUBBLE
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     bubble
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), istrin
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                          1:   Sequence number of vector
!                          2:   DEGFD
!                          3:   Indication if a plot is required (1) or not (0)
!                          4:   icolor
!                          5:   ITEXTX: sequence number of x-text in TEXT array
!                          6:   NTEXTX: number of characters in x-text
!                          7:   ITEXTY: sequence number of y-text in TEXT array
!                          8:   NTEXTY: number of characters in y-text
!                          9:   file_seq_number (-1: none)
!                         10:   istrin
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     rinsep        i/o   The real input read is stored in array INPSEP
!                         This part contains:
!                         1:   yfact
!                         2:   threshold
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=10
      integer ncode1(nncod1), ncod1m(nncod1), iposv
      character (len=9) code1(nncod1)

!     code1          Table with keywords
!     iposv          position of keyword v in array code1
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     INTXSTORETEXT  Fill reference to text and number of characters in inpsep
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     The following input is expected:
!       COMPUTE_BUBBLE, options (all in one line)
!           where options may have the following contents
!           VECTOR = s           (Default first vector)
!           THRESHOLD = t        (Default: 0)
!           PLOT                 (Default: no plot)
!           COLOR = c            (Default: 10)
!           YFACT = y            (Default: 1)
!           TEXTX = '..'         (Default: x)
!           TEXTY = '..'         (Default: y)
!           DEGFD = d            (Default: 1)
!           FILENAME = '...'     (Default: bubble.dat)
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'VECTOR', 'DEGFD', 'PLOT', 'COLOR', 'YFACT', &
                   'THRESHOLD', 'TEXTX', 'TEXTY', 'FILENAME', 'V' /
      data ncode1 / 6, 3, 4, 3, 4,   5, 5, 4, 4, 1 /
      data ncod1m / 6, 5, 4, 5, 5,   9, 5, 5, 8, 1 /
! ======================================================================
!
      call eropen ( 'readscbubble' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscbubble'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )

      iposv = 10         ! position of keyword v

      itype(1:nncode) = 0
      itype(1) = 38
      itype(3) = 2
      itype(5:6) = 1
      itype(7:9) = 6
      itype(10) = 38

!     --- Set defaults in table

      table(1:nncode) = -1d0
      table(2) = 1d0     ! degfd
      table(3) = 0d0     ! no plot
      table(4) = 10d0    ! icolor
      table(5) = 1d0     ! yfact
      table(6) = 0d0     ! threshold

!     --- Fill some values in tabmax

      tabmax(1:nncode) = 0d0
      tabmax(1) = dble(maxvec)  ! vector
      tabmax(iposv) = tabmax(1)     ! v
      tabmax(2) = 1d2               ! degfd
      tabmax(4) = 1d2               ! icolor
      tabmax(5) = 1d4               ! yfact
      tabmax(6) = 1d30              ! threshold

      tabmin(1:nncode) = 0d0
      tabmin(1:2) = 1d0
      tabmin(4) = 1d0
      tabmin(5) = 1d-4
      tabmin(6) = -1d30
      tabmin(10) = 1d0

!     --- interpret options

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( debug ) call prinrl ( table, nncod1, 'table' )

      if ( table(1)>0d0 .and. table(iposv)>0d0 ) &
         call errkeywexcl ( code1, ncod1m, 1, iposv )

      table(1) = max(1d0,table(1),table(iposv))  ! sequence number of vector

      inpsep(istart+1:istart+4) = nint(table(1:4))  ! store in inpsep
      rinsep(istrin:istrin+1) = table(5:6)        ! yfact, threshold

!     --- textx

      call intxstoretext ( inpsep(istart+5), table(7), tabmax(7), &
                           strout )

!     --- texty

      call intxstoretext ( inpsep(istart+7), table(8), tabmax(8), &
                           strout )

!     --- file

      call intxstoretext ( inpsep(istart+9), table(9), tabmax(9), &
                           strout )

      inpsep(istart+10) = istrin  ! starting address rinsep

!     --- Raise positions

      istart = istart + 11  ! Remark: if this number is enlarged, also adapt
                            ! ini016
      istrin = istrin+2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         call prinin ( inpsep(istart-10), 10, 'inpsep' )
         call prinrl ( rinsep(istrin-2), 2, 'rinsep' )
         write(irefwr,*) 'End readscbubble'

      end if  ! ( debug )
      call erclos ( 'readscbubble' )

      end subroutine readscbubble
