      subroutine readsccopyvec ( istart, inpsep, name_keyw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.4  date 25-01-2008 Small adaptation warnings
!        version  1.3  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   COPY_VECTOR
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)
      character (len=*) name_keyw

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2: Sequence number vector1
!                         3: Sequence number vector2
!                         4: Degree of freedom vector 1
!                         5: Degree of freedom vector 2
!     istart        i/o   Next free position in inpsep
!     name_keyw      i    Name of keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=5
      integer ncode1(nncod1), ncod1m(nncod1), ivec, i, n
      character (len=20) code1(nncod1)

!     code1          Table with keywords
!     i              Counting variable
!     ivec           Sequence number of vector of special structure
!     n              number of items read
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       COPY VECTOR 1 DEGFD! = j VECTOR 2 DEGFd1 = k
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1447   The number of vectors found after the keyword is incorrect
!    1448   VECTOR is copied to itself
!    1881   The sequence number found after the keyword is incorrect
!    1886   Vector has not yet been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'VECTOR', 'SEQ_STRUCT', 'DEGFD', 'DEGFD', 'V' /
      data ncode1 /  6, 7, 5, 5, 1 /
      data ncod1m /  6,13, 5, 5, 1 /

!     code1   Table with sub keywords corresponding to copy vector
!             Values:
!             1:  VECTOR
!             2:  SEQ_STRUCTURE
!             3:  DEGFD
!             4:  DEGFD
!             5:  V
! ======================================================================
!
      call eropen ( 'readsccopyvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsccopyvec'
         write(irefwr,1) 'istart, icode', istart, icode
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )

      itype(1:nncode) = 0
      itype(1) = 40
      itype(3) = 10
      itype(5) = 40
      tabmin(1:nncode) = 1d0
      tabmin(1) = 2d0
      tabmin(5) = 2d0
      tabmax(1) = dble(maxvec)
      tabmax(2) = 0d0
      tabmax(3) = 2d0
      tabmax(4) = 10d0
      tabmax(5) = tabmax(1)
      iretur = 0
      table(1:nncode) = -1d0
      nmain = 0
      errorcode = 0

      call sepreadsetkeyw

      if ( debug ) call prinrl ( table, 5, 'table' )

      if ( ierror/=0 ) go to 1000
      if ( table(1)>0d0 .and. table(5)>0d0 ) &
         call errkeywexcl ( code1, ncod1m, 1, 5)

      if ( table(5)>0d0 ) then

!     --- v found, substitute by vector

         table(1) = table(5)
         tabmax(1) = tabmax(5)

      end if  ! ( table(5)>0d0 )
      n = nint(tabmax(1))
      if ( n/=2 ) then

!     --- Error not two vector sequence numbers found after COPY

         call errchr ( name_keyw, 1 )
         call errint ( n, 1 )
         call errint ( 2, 2 )
         call errsub ( 1447, 2, 0, 1 )
         call errcrd

      end if

      iposut = nint(table(1))
      ivec = nint(tabout(iposut))
      if ( ivec>0 ) then
         if ( ivectors(ivec)==0 .and. nwarn<mxwarn ) then
            call errint ( ivec, 1 )
            call errchr ( names(ivec), 1 )
            call errwar ( 1886, 1, 0, 1 )
            call errcrd
         end if
      else
         call errchr ( name_keyw, 1 )
         call errint ( ivec, 1 )
         call errint ( 1, 2 )
         call errint ( maxvec, 3 )
         call errsub ( 1881, 3, 0, 1 )
      end if

      inpsep(istart+1) = ivec

      ivec = nint(tabout(iposut+1))
      if ( ivec>0 ) then
         ivectors(ivec) = 1
         if ( ivec==inpsep(istart+1) .and. nwarn<mxwarn ) then
            call errint ( ivec, 1 )
            call errwar ( 1448, 1, 0, 0 )
            call errcrd
         end if
      else
         call errchr ( name_keyw, 1 )
         call errint ( ivec, 1 )
         call errint ( 1, 2 )
         call errint ( maxvec, 3 )
         call errsub ( 1881, 3, 0, 1 )
      end if

      inpsep(istart+2) = ivec
      inpsep(istart+3) = 0
      inpsep(istart+4) = 0

      if ( table(3)>0 ) inpsep(istart+3) = nint(table(3))
      if ( table(4)>0 ) inpsep(istart+4) = nint(table(4))
      if ( inpsep(istart+3)==0 .and. inpsep(istart+4)>0 ) &
         inpsep(istart+3) = 1
      if ( inpsep(istart+4)==0 .and. inpsep(istart+3)>0 ) &
         inpsep(istart+4) = 1
      if ( debug ) call prinin ( inpsep(istart), 5, 'inpsep' )
      istart = istart + 5

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart, icode', istart, icode
         write(irefwr,*) 'End readsccopyvec'

      end if  ! ( debug )
      call erclos ( 'readsccopyvec' )

      end subroutine readsccopyvec
