      subroutine readsclevel ( inpsep, istart, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 07-03-2012 New call to readseqvector
!        version  1.1  date 30-01-2012 New call to readseqvar
!        version  1.0  date 02-11-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keywords after the keyword LEVELSET_MESH_VELOCITY
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     level_set
!     mesh_velocity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), istrin
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 1-7 are used:
!                         1:  ICODE
!                         2:  MESH1  Sequence number of basis mesh
!                         3:  MESH2  Sequence number of old levelset mesh
!                         4:  MESH3  Sequence number of new levelset mesh
!                         5:  SEQCONC Sequence number of concentration
!                         6:  SEQPHI  Sequence number of new phi
!                         7:  SEQPHIOLD  Sequence number of old phi
!                         8:  SEQMESHVEL Sequence number of mesh velocity
!                         9:  SEQDT Sequence number delta t in real input array
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
!     istrin        i/o   Next free position in rinsep
!     rinsep         o    Array containing the real input for this subroutine
!                         Contents:
!                         1:   dt
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=8
      integer ncode1(nncode1), ncodmx1(nncode1)
      character (len=20) code1(nncode1)

!     code1          Table with keywords
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READSEQVAR     Put the value of the variable read in rinput
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       LEVELSET_MESH_VELOCITY V1, options
!           where options may have the following contents
!           basis_mesh = i
!           mesh_in = i
!           mesh_out = i
!           solution = Vj
!           levelset = Vk
!           old_levelset = Vm
!           time_step = dt
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'BASIS_MESH', 'MESH_IN', 'MESH_OUT', 'SOLUTION', &
                  'LEVELSET', 'OLD_LEVELSET', 'V', 'TIME_STEP' /
      data ncode1  /  3, 5, 5, 3, 3,   3, 1, 4 /
      data ncodmx1 / 10, 7, 8, 8, 8,  12, 1, 9 /
! ======================================================================
!
      call eropen ( 'readsclevel' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsclevel'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode  ! icode (LEVELSET_MESH_VELOCITY)

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )

!     --- Input for subroutine intx02

      itype(1:nncode) = 0
      itype(4:6) = 38  ! vectors
      itype(8) = 1

      tabmin(1:nncode) = 1d0

      tabmax(1:nncode) = 1d2
      tabmax(1:3) = 1d1  ! meshes

      table(1:nncode) = 0d0
      table(1) = 1d0
      table(2) = 2d0
      table(3) = 3d0

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
         call prinrl ( tabout, 1, 'tabout' )
         call prinin ( infotable, nncode, 'infotable' )
      end if  ! ( debug )

      inpsep(istart+1:istart+3) = nint(table(1:3))
      inpsep(istart+4:istart+7) = 0

      call readseqvector ( table(4), 'concentration', inpsep(istart+4), &
                           1 )
      call readseqvector ( table(5), 'phi', inpsep(istart+5), 2 )
      call readseqvector ( table(6), 'phiold', inpsep(istart+6), 3 )
      call readseqvector ( table(7), 'mesh_vel', inpsep(istart+7), 4 )
      icode = 8
      call readseqvar ( 'dt', rinsep, inpsep(istart+8), istrin, 0d0 )
      istart = istart+9

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readsclevel'

      end if  ! ( debug )
      call erclos ( 'readsclevel' )

      end subroutine readsclevel
