      subroutine readscplotbn ( istart, inpsep, istrin, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-11-2004 Fortran 90
!        version  1.0  date 29-11-2002
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   PLOT_BOUNDARY
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     plot
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), istrin
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2:  IREGION
!                         If IREGION#0, 4 positions in rinsep are used
!                         3:  IEXTRA
!                         If IEXTRA#0, 1 extra position in rinsep is used
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       PLOT_BOUNDARY, options
!           where options may have the following contents
!           region = (xmin,xmax,ymin,ymax,zmin,zmax)
!           yfact = y
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscplotbn' )

      inpsep(istart) = icode

      nncode = 2
      code(1) = 'REGION'
      code(2) = 'YFACT'
      ncode(1) = 4
      ncode(2) = 3
      ncodmx(1) = 6
      ncodmx(2) = 5
      itype(1) = 19
      itype(2) = 1
      tabmin(1) = 4d0
      tabmin(2) = 0d0
      tabmax(1) = 1d5
      tabmax(2) = 1d5
      table(1) = 0d0
      table(2) = 0d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( ierror/=0 ) go to 1000
      if ( table(1)>0 ) then
         inpsep(istart+1) = istrin
         do i = 0, 3
            rinsep(istrin+i) = tabout(i+1)
         end do
         istrin = istrin+4
      else
         inpsep(istart+1) = 0
      end if
      if ( table(2)>0 ) then
         inpsep(istart+2) = istrin
         rinsep(istrin) = table(2)
         istrin = istrin+1
      else
         inpsep(istart+2) = 0
      end if
      istart = istart+3

1000  call erclos ( 'readscplotbn' )

      end
