      subroutine readscprinint ( inpsep, istart, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keywords after the keyword PRINT_INTERSECTION
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     print
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), istrin
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 0-6 are used:
!                          0:  ICODE
!                          1:  IVECTOR
!                          2:  IDEGFD
!                          3:  position of file name in array texts
!                          4:  appendmode
!                              0: not
!                              1: append
!                          5:  refers to O_x
!                          6:  refers to O_y
!                          7:  refers to angle
!                          8:  refers to E_x
!                          9:  refers to E_y
!                         10:  refers to length
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
!     istrin        i/o   Next free position in rinsep
!     rinsep         o    Array containing the real input for this subroutine
!                         Contents:
!                         1:   x-coordinate origin
!                         2:   y-coordinate origin
!                         3:   angle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=8
      integer :: ncode1(nncode1), ncodmx1(nncode1)
      double precision :: defvalues(2)
      character (len=20) :: code1(nncode1)
      character (len=3) :: locnames(2)

!     code1          Table with keywords
!     defvalues      Default values
!     locnames       Default names of variables
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCLUS  Check if two keywords are mutually exclusive
!     INTXSTORETEXT  Fill reference to text and number of characters in inpsep
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     READSEQVAR     Put the value of the variable read in rinsep
!     READSEQVARS    Put the values of the variables read in rinsep
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       PRINT_INTERSECTION V1, options
!           where options may have the following contents
!           degfd=i
!           origin = (O_x,O_y)
!           angle = a
!           file = 'file_name'
!           end_point = (E_x, E_y)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'DEGFD', 'FILE', 'ORIGIN', 'ANGLE', 'V', 'APPEND', &
                  'END_POINT', 'LENGTH' /
      data ncode1  /  3, 4, 4, 3,   1, 3, 3, 3 /
      data ncodmx1 /  5, 4, 6, 5,   1, 6, 9, 6 /

!     code   Table with keywords corresponding to options of COMPUTE_SCALAR
!             Values:
!             1:  DEGFD
!             2:  FILE
!             3:  ORIGIN
!             4:  ANGLE
!             5:  V
!             6:  APPEND
!             7:  END_POINT
!             8:  LENGTH
! ======================================================================
!
      call eropen ( 'readscprinint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscprinint'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode

!     --- Input for subroutine intx02

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )

      itype(1:nncode) = 0
      itype(2) = 6     ! file
      itype(3) = 19    ! origin
      itype(4) = 1     ! angle
      itype(5) = 38    ! V
      itype(6) = 2     ! append
      itype(7) = 19    ! end_point
      itype(8) = 1     ! length

      tabmin(1:nncode) = 0d0
      tabmin(1) = 1d0  ! degfd
      tabmin(3) = 2d0  ! max number of coordinates
      tabmin(4) = -360d0  ! minimum angle
      tabmin(5) = 1d0  ! V
      tabmin(7) = 2d0  ! max number of coordinates

      tabmax(1:nncode) = 0d0
      tabmax(1) = 1d2  ! degfd
      tabmax(3) = 1d9  ! max value of coordinates
      tabmax(4) = 360d0  ! maximum angle
      tabmax(7) = 1d9  ! max value of coordinates
      tabmax(8) = 1d9  ! length

      table(1:nncode) = 0d0
      table(4) = -1d20
      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
         call prinin ( infotable, nncode, 'infotable' )
      end if  ! ( debug )

      inpsep(2) = nint(table(1))   ! degfd

!     --- file

      call intxstoretext ( inpsep(3), table(2), tabmax(2), &
                           strout )

      inpsep(1) = nint(table(5))  ! sequence number of vector
      inpsep(4) = nint(table(6))  ! append mode
      inpsep(5) = istrin  ! starting address in rinsep

!     --- Put information of origin in inpsep/rinsep

      locnames(1) = 'O_X'
      locnames(2) = 'O_Y'
      defvalues(1:2) = 0d0
      icode = 3
      call readseqvars ( locnames, rinsep, inpsep(5), istrin, &
                         defvalues, 2 )

!     --- Put information of angle in inpsep/rinsep

      icode = 4
      call readseqvar ( 'angle', rinsep, inpsep(7), istrin, 0d0 )

!     --- Put information of end_point in locsep/rlocsep

      locnames(1) = 'E_X'
      locnames(2) = 'E_Y'
      defvalues(1:2) = 1d10
      icode = 7
      call readseqvars ( locnames, rinsep, inpsep(8), istrin, &
                         defvalues, 2 )

      inpsep(10) = 0  ! length
      if ( inpsep(8)==0 .and. inpsep(9)==0 ) then

!     --- No end point given, check if length is given

         if ( table(8)>0d0 ) then

!        --- length found

            rinsep(istrin) = table(8)
            inpsep(10) = -istrin
            istrin = istrin+1

         end if  ! ( table(8)>0d0 )

      else

!     --- End point given, check if length is not given

         call errkeywexclus ( table, code, ncodmx, 7, 8 )

      end if  ! ( inpsep(9)==0 .and. inpsep(10)==0 )

!     --- Raise istart

      istart = istart+11

1000  if ( debug ) then

!     --- Debug information

         call prininlarge ( inpsep(1), 10, 'inpsep' )
         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End readscprinint'

      end if  ! ( debug )
      call erclos ( 'readscprinint' )

      end subroutine readscprinint
