      subroutine readscrenum ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2021
!
!   copyright (c) 2021-2021  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the information following renumber and
!     store in inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: inpsep(0:*), istart

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         0:  icode (115)
!                         1:  type of renumbering, see sepmodulestrings (coderenum)
!     istart        i/o   Starting address in array INPSEP
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: itype1(nncoderenum)
      double precision :: tabmn1, tabmx1
      integer :: i

!     i              General loop variable
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     tabmn1         Minimum value of items read corresponding to code1
!     tabmx1         Maximum value of items read corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscrenum' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscrenum'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      inpsep(0) = icode+3
      call sepcopycode ( coderenum, nncoderenum, nscoderenum, ncoderenum )
      itype1 = 2
      table = 0d0
      call sepcopytype ( nncoderenum, itype1, tabmn1, tabmx1 )
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      do i = 1, nncoderenum
         if ( table(i)>0d0 ) then
            inpsep(1) = i    ! type of renumbering
            exit
         end if  ! ( table(i)>0d0 ) then
      end do  ! i = 1, nncoderenum
      istart = istart+2  ! raise istart

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, 2, 'inpsep' )
         write(irefwr,*) 'End readscrenum'

      end if  ! ( debug )
      call erclos ( 'readscrenum' )

      end subroutine readscrenum
