      subroutine readscstloop ( istart, inpsep, iseqfr, instat, iseqlp, &
                                iseqtm, iloop, ilooptm, timefnd, ifreest, &
                                rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 13-05-2011 debug false
!        version  1.1  date 22-11-2004 Fortran 90
!        version  1.0  date 29-11-2002
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keywords
!   START_STATIONARY_FREE_BOUNDARY, START_INSTATIONARY_FREE_BOUNDARY, &
!   START_LOOP, START_TIME_LOOP
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     loop
!     stationary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), ifreest, &
              iseqfr, iseqlp, iseqtm, iloop(*), ilooptm(*), istrin
      double precision :: rinsep(*)
      logical instat, timefnd

!     ifreest       i/o   ifreest = starting address of stationary free boundary
!                         in inpsep
!     iloop         i/o   Array of size 10 in which information of the loops
!                         is stored in the following way:
!                         iloop(i) = starting address of while loop in inpsep
!     ilooptm       i/o   Array of size 1 in which information of the time loops
!                         is stored in the following way:
!                         ilooptm(i) = starting address of while loop in inpsep
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2:  sequence number k, if -10 pos. 4 is used
!                         3:  address in inpsep just after the loop
!                         4:  Only used if pos. 2 = -10
!                             Contains starting address of time parameters
!                             in rinsep
!     instat        i/o   If true an instationary free boundary loop is active
!     iseqfr        i/o   Sequence number for input of subroutine STATFREE
!     iseqlp        i/o   Present level of loop
!     iseqtm        i/o   Present level of time loop
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     rinsep        i/o   Array containing the real input for this subroutine
!     timefnd       i/o   If true, time integration has been found in loop
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=7
      integer ncode1(nncode1), ncodmx1(nncode1), istartsav
      character (len=20) code1(nncode1)

!     code1          Table with keywords
!     istartsav      Help variable to save the value of istart
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       START_STATIONARY_FREE_BOUNDARY_LOOP, sequence_number = k
!       START_INSTATIONARY_FREE_BOUNDARY_LOOP, sequence_number = k
!       START_LOOP, sequence_number = k
!       START_TIME_LOOP, options (all in one line)
!         with options:
!            sequence_number = k
!            tinit = t0, tend = t1, tstep = dt
!            toutinit = tout0, toutend = tout1, toutstep =  dtout
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1073   Too much nested loops
!    1074   Too much nested time loops
!    1925   START_FREE_BOUNDARY found, while still one active
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'SEQUENCE_NUMBER', 'TINIT', 'TEND', 'TSTEP', &
                  'TOUTINIT', 'TOUTEND', 'TOUTSTEP' /
      data ncodmx1 /15, 5, 4, 5, 8,  7, 8 /
      data ncode1 / 6, 5, 4, 5, 7,  7, 8 /

!     code   Table with sub keywords corresponding to some keywords
!             Values:
!             1:  SEQUENCE
! ======================================================================
!
      call eropen ( 'readscstloop' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscstloop'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )
         istartsav = istart

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode   ! code
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )

!     --- Set parameters

      itype(1:nncode) = 1     ! all real
      itype(1) = 0  ! sequence_number (integer)

      tabmin(1:nncode) = -1d5    ! minimum value
      tabmin(1) = 1d0  ! minimum sequence_number

      tabmax(1:nncode) = 1d15    ! maximum value
      tabmax(1) = 1d2  ! maximum sequence_number

      table(1:nncode) = -1d20
      table(1) = 1d0  ! sequence_number

      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

      icode = inpsep(istart)
      inpsep(istart+2) = 0     ! return address (to be filled in later on)

      if ( icode==46 .and. maxval(table(2:nncode))>-1d19 ) then

!     --- Start time loop found and also time parameters

         inpsep(istart+1) = -10
         inpsep(istart+3) = istrin

!        --- tinit

         if ( table(2)>-1d19 ) then
            rinsep(istrin) = table(2)
         else
            rinsep(istrin) = 0d0
         end if

!        --- tend

         if ( table(3)>-1d19 ) then
            rinsep(istrin+1) = table(3)
         else
            rinsep(istrin+1) = 1d0
         end if

!        --- tstep

         if ( table(4)>-1d19 ) then
            rinsep(istrin+2) = table(4)
         else
            rinsep(istrin+2) = 0.1d0
         end if

!        --- toutinit

         if ( table(5)>-1d19 ) then
            rinsep(istrin+3) = table(5)
         else
            rinsep(istrin+3) = rinsep(istrin)
         end if

!        --- toutend

         if ( table(6)>-1d19 ) then
            rinsep(istrin+4) = table(6)
         else
            rinsep(istrin+4) = rinsep(istrin+1)
         end if

!        --- toutstep

         if ( table(7)>-1d19 ) then
            rinsep(istrin+5) = table(7)
         else
            rinsep(istrin+5) = rinsep(istrin+2)
         end if

         if ( debug ) call prinrl ( rinsep(istrin), 6, 'rinsep' )

!        --- raise parameters

         istart = istart + 4
         istrin = istrin + 6

!        --- Fill ilooptm

         iseqtm = iseqtm+1
         if ( iseqtm>1 ) then
            call errint ( iseqtm, 1 )
            call errint ( 10, 2 )
            call errsub ( 1074, 2, 0, 0 )
         end if
         ilooptm(iseqtm) = istart
         instat = .true.
         timefnd = .true.

      else

         inpsep(istart+1) = nint(table(1))  !  sequence number
         if ( icode<30 ) then

!        --- Free boundary loop

            iseqfr = iseqfr+1
            if ( iseqfr>1 ) then
               call errsub ( 1925, 0, 0, 0 )
            end if
            ifreest = istart
            if ( icode==26 ) instat = .true.

         else if ( icode==44 ) then

!        --- Loop

            iseqlp = iseqlp+1
            if ( iseqlp>10 ) then
               call errint ( iseqlp, 1 )
               call errint ( 10, 2 )
               call errsub ( 1073, 2, 0, 0 )
            end if
            iloop(iseqlp) = istart

         else

!        --- Time loop

            iseqtm = iseqtm+1
            if ( iseqtm>1 ) then
               call errint ( iseqtm, 1 )
               call errint ( 10, 2 )
               call errsub ( 1074, 2, 0, 0 )
            end if
            ilooptm(iseqtm) = istart
            instat = .true.
            timefnd = .true.

         end if

         istart = istart + 3

      end if  ! ( icode==46 .and. max(table(2):ncode)>-1d10 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(istartsav), istart-istartsav, 'inpsep' )
         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End readscstloop'

      end if  ! ( debug )
      call erclos ( 'readscstloop' )

      end subroutine readscstloop
