      subroutine readscstress ( istart, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-06-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Read the input for the principal stresses with respect to subroutine readsc
!  The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     principal_stress
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 1-5 are used:
!                         1: ICODE
!                         2: ISEQSTRESS
!                         3: ISEQEIGVAL
!                         4: ISEQEIGVEC
!                         5: ICHOICE
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=4
      integer ncode1(nncod1), ncod1m(nncod1)
      character (len=20) code1(nncod1)
      integer ichoice, iscale, iseqeigval, iseqeigvec, i

!     code1          Table with keywords corresponding to
!                    COMPUTE_PRINCIPAL_STRESSES
!     i              Counting variable
!     ichoice        Integer choice parameters for the intermediate file
!     iscale         Indicates the type of scaling that must be applied
!                    Possible values:
!                    0: no scaling
!                    1: always scaling
!                    2: scaling for specified sufraces and volumes
!     iseqeigval     Sequence number of array eigval in isol
!     iseqeigvec     Sequence number of array eigvec in isol
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       COMPUTE_PRINCIPAL_STRESSES, options (all in one line)
!           where options may have the following contents
!           STRESS_VECTOR = j         (Default 1)
!           EIGENVALUES = j1          (Default 0)
!           EIGENVECTORS = j2         (Default 0)
!           SCALING                   (Default no)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2181   Not one of the keywords EOGENVALUES and EIGENVECTORS found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1/ 'STRESS_VECTOR', 'EIGENVALUES', 'EIGENVECTORS', &
                  'SCALING' /
      data ncod1m / 13, 11, 12, 7 /
      data ncode1 /  6,  8,  8, 4 /

!     code1   Table with sub keywords corresponding to keyword
!             COMPUTE_PRINCIPAL_STRESSES
!             Values:
!             1:  STRESS_VECTOR
!             2:  EIGENVALUES
!             3:  EIGENVECTORS
!             4:  SCALING
! ======================================================================
!
      call eropen ( 'readscstress' )

      inpsep(istart) = icode

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )

      itype(1:3) = 0
      itype(4) = 2
      table(1:nncode) = 0d0
      table(1) = 1d0
      tabmin(1:3) = 1d0
      tabmin(4) = 0d0
      tabmax(1:3) = dble(maxvec)
      tabmax(4) = 0d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      inpsep(istart+1) = nint(table(1))
      iseqeigval = nint(table(2))
      iseqeigvec = nint(table(3))
      iscale = nint(table(4))
      inpsep(istart+2) = iseqeigval
      inpsep(istart+3) = iseqeigvec
      if ( iseqeigvec.eq.0 ) then

!     --- No eigenvectors

         if ( iseqeigval.eq.0 ) then

!        --- error: also no eigenvalues

            call errsub ( 2181, 0, 0, 0 )

         end if
         ichoice = 1

      else

!     --- eigenvectors

         if ( iscale.eq.0 ) then
            ichoice = 2
         else
            ichoice = 3
         end if
         if ( iseqeigval.ne.0 ) ichoice = ichoice+2
      end if
      if ( iseqeigval.gt.0 ) then
         numvec = min ( max(numvec,iseqeigval), maxvec )
         if ( iseqeigval.le.maxvec ) ivectors(iseqeigval) = 1
      end if
      if ( iseqeigvec.gt.0 ) then
         numvec = min ( max(numvec,iseqeigvec), maxvec )
         if ( iseqeigvec.le.maxvec ) ivectors(iseqeigvec) = 1
      end if
      inpsep(istart+4) = ichoice
      istart = istart + 5
      call erclos ( 'readscstress' )
      end subroutine readscstress
