      subroutine readscwhile ( istart, inpsep, iseqwh, iwhile, rinsep, &
                               leninpsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 18-12-2013 Start inpsep from 0
!        version  2.1  date 10-09-2008 Debug statements
!        version  2.0  date 08-02-2008 Extra parameters
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   WHILE
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     loop
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), &
              iseqwh, iwhile(*), leninpsep, istrin
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         1: code for while (21)
!                         If boolean_expr(k) is used then:
!                         2: the parameter k in boolean_expr(k)
!                         3: address in inpsep just after the while loop
!                            At present set to 0, filled later on
!                         Otherwise ( boolean expression given)
!                         2: -length, where length is the number of
!                            positions used in inpsep from position 2
!                         2+length+1: address in inpsep just after the while
!                            loop. At present set to 0, filled later on
!     iseqwh        i/o   Present level of while loop
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     iwhile        i/o   Array of size 10 in which information of the while
!                         loops is stored in the following way:
!                         iwhile(i) = starting address of while loop in inpsep
!     leninpsep      i    Length of array inpsep
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ::  booleanfound

!     booleanfound   If true a boolean or boolean expression has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READBOOLEXPR   Read boolean expression between opening and closing
!                    brackets
!     READWR         Read keyword
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       WHILE ( boolean_expr(k) ) DO or WHILE  boolean_expr(k) DO
!             or
!       WHILE ( xxx ) DO, with xxx the boolean expression
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      40   Too much nested while loops
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscwhile' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscwhile'
         write(irefwr,1) 'lastitem', lastitem
         write(irefwr,1) 'icode, istart, iseqwh', icode, istart, iseqwh
  1      format ( a, 1x, (10i6) )
         call prinin ( typeitems, nitems, 'typeitems' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode   ! while in inpsep
      iseqwh = iseqwh+1        ! raise number of while loops

      if ( iseqwh>10 ) then
         call errint ( iseqwh, 1 )
         call errint ( 10, 2 )
         call errsub ( 40, 2, 0, 0 )
      end if

      iwhile(iseqwh) = istart  ! store start of while loop
      istart = istart+1        ! raise istart

!     --- Raise lastitem in case ( is found

      if ( typeitems(lastitem+1)==5 ) lastitem = lastitem+1

!     --- Read next item

      booleanfound = .true.  ! defines if a boolean or boolean_expression is
                             ! present
      if ( typeitems(lastitem+1)==3 ) then

!     --- String found, check if it is BOOLEAN_EXPR

         lastitem = lastitem+1
         call readwr
         if ( nchar>=4 .and. string(1:4)=='BOOL' ) then

!        --- Boolean expression found

            booleanfound = .false.

         end if
         lastitem = lastitem-1

      end if  ! ( typeitems(lastitem)==4 )

      if ( booleanfound ) then

!     --- New method, interpret boolean expression
!         The opening bracket has already been found
!         information from inpsep(2)

         call readboolexpr ( inpsep(2), rinsep, leninpsep-3, istrin )
         length = inpsep(2)
         if ( debug ) write(irefwr,1) 'istart, length', istart, length
         inpsep(1) = -length   ! Set negative length to distinguish
                               ! with old method

      else

!     --- Old method using boolean_expression(k)
!         Read k

         nncode = 2
         code(1) = 'BOOLEAN_EXPR'
         code(2) = 'DO'
         ncode(1) = 4
         ncode(2) = 2
         ncodmx(1) = 12
         ncodmx(2) = 2
         itype(1) = 0
         itype(2) = -1
         tabmin(1) = 1d0
         tabmin(2) = 0d0
         tabmax(1) = 1d3
         tabmax(2) = 0d0
         table(1) = 0d0
         iretur = 0
         nmain = 0
         call sepreadsetkeyw
         if ( ierror/=0 ) go to 1000
         inpsep(1) = nint(table(1))    ! store k
         inpsep(2) = 0
         length = 0

      end if  ! ( booleanfound )

      inpsep(2) = 0  ! return position
      istart = istart + 2 + length

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End readscwhile'

      end if  ! ( debug )
      call erclos ( 'readscwhile' )

      end subroutine readscwhile
