      subroutine readspstart ( manag, iposren, irenum, jrefs, jsepbc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1   date   02-09-2013 Increase max. number of plots
!        version  1.0  date 30-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read part of input defined by keyword START
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulecpack
      use sepmodulemach
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer manag(*), iposren, irenum, jrefs, jsepbc

!     iposren        i    Position of renumber keyword in array code
!     irenum        i/o   Type of renumbering  Possible values
!                         0  No renumbering
!                         1  Cuthill-McKee  optimal profile
!                         2  Sloan  optimal profile
!                         3  Best  optimal profile
!                         4  Cuthill-McKee  optimal band width
!                         5  Sloan  optimal band width
!                         6  Best  optimal band width
!                         7  Cuthill-McKee  always
!                         8  Sloan always
!                         9  Default depending on mesh
!                        11  nested_dissection
!     jrefs         i/o   Indication of the structure of the files sepcomp.out
!                         and sepcomp.inf
!                         Possible values:
!                         -1   Files are used; the defaults from sepran.env
!                             are used
!                         0   Files are not used
!                         1   files are formatted
!                         2   Bulk files are unformatted
!     jsepbc        i/o   Choice parameter indicating if file 2 must be opened
!                         or not
!                         Possible values:
!                         0   File 2 is not used
!                         1   File 2 is new
!                         -1   File 2 is old
!     manag         i/o   Manager array that sets the parameters in subroutine
!                         SEPSTR
!                         See subroutine READSP for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih, iseqout
      character (len=19) namedir

!     ih             Help variable to store a constant temporarily
!     iseqout        Sequence number to define name of output file
!     namedir        Name of temp dir
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     STUNIX1        Rename some files in case of unix and writing to /tmp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     830   Both rotate and norotate found
!    2410   End value of parameter smaller than initial value
!    2783   name_sepout found and name_firstfile or name_secondfile
!    2784   only one of the names name_firstfile and name_secondfile found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readspstart' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readspstart'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      table(1:nncode) = -1d0
      table(18) = 0d0

!     --- read rest of records until END is found

      istop = 2
      iretur = 0
      call sepreadsetkeyw
      istop = 0
      if ( debug ) call prinrl ( table, nncode, 'table' )

!     --- Interpret results

      ih = nint(table(1))
      if ( ih/=-1 ) jsepbc = ih-2

      if ( nint(table(2))==1 ) irotat = 2
      if ( nint(table(3))==1 ) irotat = 1
      if ( table(2)>0d0 .and. table(3)>0d0 ) &
         call errwar ( 830, 0, 0, 0 )

      ih = nint(table(4))
      if ( ih/=-1 ) jrefs = ih-1
      if ( table(16)>0d0 ) then

!     --- cpu_time found

         cputiming = .true.

      else if ( table(17)>0d0 ) then

!     --- wall_clock_time found

         cputiming = .false.

      end if

      ih = nint(table(iposren))
      if ( ih/=-1 ) irenum = ih-1
      if ( debug ) write(irefwr,1) 'irenum', irenum
      if ( irenum>0 .and. irenum<4 ) then

!     --- renumber = cuthill/sloan/best found
!         interpret next keyword

         if ( nint(table(iposren+1))>=1 ) then
            irenum = irenum + 3*(nint(table(iposren+1))-1)
         else
            irenum = irenum + 6
         end if

      else if ( irenum==4 ) then

!     --- renumber = levels found

         irenum = 107

      else if ( irenum==5 ) then

!     --- renumber = nested_dissection found

         irenum = 11

      end if
      if ( table(14)>0d0 ) manag(9) = nint(table(14))
      if ( table(15)>0d0 ) manag(10) = nint(table(15))
      if ( manag(10)<manag(9) ) then

!     --- manag(10)<manag(9)

         call errint ( manag(10), 1 )
         call errint ( manag(9), 2 )
         call errchr ( code(12)(1:ncodmx(12)), 1 )
         call errchr ( code(11)(1:ncodmx(11)), 2 )
         call errsub ( 2410, 2, 0, 2 )

      end if
      if ( table(5)>0d0 ) namef2 = strout(nint(table(5)))

!     --- Check if names of files for sepcomp.out do not conflict

      if ( table(8)>0 .and. (table(10)>0 .or. table(11)>0) ) then

!     --- error both name_sepout found and name_firstfile or name_secondfile
!         These names exclude each other

         call errsub ( 2783, 0, 0, 0 )
         go to 1000

      end if  ! ( table(8)>0 .and. table(10)>0 .or. table(11)>0 )

      if ( table(10)>0 .and. table(11)<=0 .or. &
           table(10)<=0 .and. table(11)>0 ) then

!     --- error only one of the names name_firstfile and name_secondfile
!         found. Both should be present

         call errsub ( 2784, 0, 0, 0 )
         go to 1000

      end if  ! ( table(10)>0 .and. table(11)<=0 .or. ... )
      if ( table(10)>0 ) then

!     --- name_firstfile found, set sequence number to 10

         iseqout = 10

      else

!     --- name_firstfile not found, set sequence number to 8

         iseqout = 8

      end if

      if ( name10(1:8)=='/tmp/SEP' ) then

!     --- file is stored in temporary directory

         namedir = name10(1:16)//'000'
         if ( table(6)>0d0 ) &
            namepl = namedir//'/'//strout(nint(table(6)))
         if ( table(iseqout)>0d0 ) &
            name74 = namedir//'/'//strout(nint(table(iseqout)))
         if ( table(9)>0d0 ) &
            name10 = namedir//'/'//strout(nint(table(9)))
         if ( table(11)>0d0 ) &
            nameout2 = namedir//'/'//strout(nint(table(11)))
         if ( table(12)>0d0 ) &
            namediff = namedir//'/'//strout(nint(table(12)))
         call stunix1 ( namedir )

      else

!     --- file is stored in local directory

         if ( table(6)>0d0 ) namepl = strout(nint(table(6)))
         if ( table(iseqout)>0d0 ) &
            name74 = strout(nint(table(iseqout)))
         if ( table(9)>0d0 ) name10 = strout(nint(table(9)))
         if ( table(11)>0d0 ) nameout2 = strout(nint(table(11)))
         if ( table(12)>0d0 ) namediff = strout(nint(table(12)))

      end if

      if ( table(13)>0d0 ) then

!     --- maxplots found

         if ( nint(table(13))>9999 ) maxplots = 1

      end if
      manag(11) = nint(table(18))
      if ( manag(11)>0 ) then

!     --- manag(11)>0, read from file
!         Detect name input file

         name73 = 'sepcomp.in'
         if ( table(19)>0d0 ) name73 = strout(nint(table(19)))

      else

!     --- manag(11)=0, do not read from file

         if ( table(7)>0d0 ) name73 = strout(nint(table(7)))

      end if  ! ( manag(11)>0 )
      if ( table(20)>0d0 ) manag(7) = 0
      if ( debug ) call prinin ( manag, 11, 'manag' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readspstart'

      end if  ! ( debug )
      call erclos ( 'readspstart' )

      end subroutine readspstart
