      subroutine readtm1 ( icall, iintim, rintim, iseqnr, ndim,&
                           numcop, leniintim, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read first part of array iintim referring to general items
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: icall, iseqnr, ndim
      integer, intent(out) :: iintim(*), numcop, leniintim
      integer, intent(inout) :: istrin
      double precision, intent(out) :: rintim(*)

!     icall          i    Indicates type of calling subroutine
!                         1:  readtm
!                         2:  readscsoltim1
!     iintim         o    Contains the input for time integration
!                         Contents:
!                          1: Number of items filled ( not used here)
!                          2: ichtim: See sepmoduletime
!                          3: lineartim: See sepmoduletime
!                          4: ifilltim: See sepmoduletime
!                          5: ncopeqtim: See sepmoduletime
!                          6: itimbc: See sepmoduletime
!                          7: ireadbctim: See sepmoduletime
!                          8: itimcf: See sepmoduletime
!                          9: iseqfr: See sepmoduletime
!                         10: iconsstiff: See sepmoduletime
!                         11: iconsmass: See sepmoduletime
!                         12: iconsrhs: See sepmoduletime
!                         13: ireadsl: See sepmoduletime
!                         14: imas: See sepmoduletime
!                         15: numoldtim: See sepmoduletime
!                         16: isubdiv: See sepmoduletime
!                         17: nfrac: See sepmoduletime
!                         18: ndt: See sepmoduletime
!                         19: icall_outtim: See sepmoduletime
!                         20: iseqouttim: See sepmoduletime
!                         21: ireturtim: See sepmoduletime
!                         22: iprinttim: See sepmoduletime
!                         23: -:
!                         24: keeptim: See sepmoduletime
!                         25: nhist: See sepmoduletime
!                         26: minitertim:
!                         27: maxoldtim: See sepmoduletime
!                         28: maxitertim: See sepmoduletime
!                         29: stationarysol: See sepmoduletime
!                         30: nocomputationtim: See sepmoduletime
!                         31:
!                         32: iseqreac: See sepmoduletime
!                         33: iseqveloctim: See sepmoduletime
!                         34: iseqacceltim: See sepmoduletime
!                         35: iseqmttim: See sepmoduletime
!                         36: iseqaddrhstim: See sepmoduletime
!                         37: starting address ireadcftim: See sepmoduletime
!                         38: starting address information per equation
!                         39: -
!                         40: -
!                         41: Information concerning epstim
!                         42: Information concerning t0
!                         43: Information concerning toutinit
!                         44: Information concerning toutend
!                         45: Information concerning toutstep
!                         46: Information concerning absaccurtim
!                         47: Information concerning relaccurtim
!                         48: Information concerning tthreshold
!                         49: -
!                         50: -
!                         51: Starting address information concerning tend
!                         52: Starting address information concerning tstep
!                         53: Starting address information concerning theta
!                         54: Starting address information concerning newmark
!                         55: Starting address information concerning gen alpha
!                         56: Starting address information concerning time his
!                         57-60: -
!     iseqnr        i/o   Sequence number with respect to the storage in array
!                         ISEPIN
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     leniintim      o    Length of array iintim
!     ndim           i    Dimension of space
!     numcop         o    number of coupled equations
!     rintim         o    Array RINTIM filled accordinag to iintim
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ih, method, imas, ndt, i, ncoefs
      logical :: stationary

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     imas           Indicates the type of mass matrix
!                    (1=diagonal, 2=consistent)
!     method         see USER INPUT in subroutine ELM600
!     ncoefs         Number of coefficient sequence numbers read
!     ndt            Number of different time steps
!     stationary     If true a stationary solution must be computed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINRL         Print 1d real vector
!     READSEQINTGR   Put the value of the integer variable read in iinput
!                    If no sequence number is found, use default value
!     READSEQVAR     Put the value of the variable read in rinput
!     READSERVAR     Store numitems reals or variables in iinsep
!     READTMTEND     Fill information concerning tend in iintim/rintim
!     READTMTHETA    Fill information concerning theta in iintim/rintim
!     READTMTSTEP    Fill information concerning tstep in iintim/rintim
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1596   Number of coefficients larger than number of coupled problems
!    2039   The combination stationary and non-linear iteration per time-step
!           has not been implemented
!    2041   non-linear_iteration is only implemented for a theta method
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readtm1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readtm1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      leniintim = 60  ! length of array iintim
      iintim(1:60) = 0

!     --- Compute nmain as function of icall

      if ( icall==1 ) then
         nmain = 2
      else
         nmain = 0
      end if  ! ( icall==1 )

!     --- Fill positions of subkeywords

      nncode = nncodetim
      code(1:nncode) = codetim
      ncode(1:nncode) = nscodetim
      ncodmx(1:nncode) = ncodetim
      tabmin(1:nncode) = 0d0
      tabmax(1:nncode) = 1d15

      itype(1:nncode) = 1
      itype(1) = 5      ! METHOD
      tabmin(1) = 1          ! Start method
      tabmax(1) = nncodetimmeth  ! end method
      tabmin(2) = -1d5  ! TINIT
      itype(3:5) = 19   ! TEND, THETA, TSTEP
      tabmin(3:5) = 10d0
      tabmax(4)  = 1d0   ! THETA
      tabmin(6:7) = -1d5 ! TOUTINIT, TOUTEND
      tabmin(8) = 1d-10  ! TOUTSTEP
      tabmin(9) = 1d-16  ! ABS_STATIONARY_ACCURACY
      tabmin(10) = 1d-8  ! REL_STATIONARY_ACCURACY
      tabmax(9:10) = 1d0
      itype(11:13) = 5  ! MASS_MATRIX, STIFFNESS_MATRIX, RIGHT_HAND_SIDE
      tabmin(11) = tabmax(1)+1d0  ! MASS_MATRIX
      tabmax(11) = tabmax(1)+nncodevar-1d0
      tabmin(12) = tabmin(11) ! STIFFNESS_MATRIX
      tabmax(12) = tabmax(11)
      tabmin(13) = tabmin(11) ! RIGHT_HAND_SIDE
      tabmax(13) = tabmax(11)+1d0
      itype(14:17) = 0    ! SEQ_SOLUTION_METHOD, SEQ_BOUNDARY_CONDITIONS, &
                          ! NUMBER_OF_COUPLED_EQUATIONS
      itype(15) = 11      ! SEQ_COEFFICIENTS
      tabmin(14:17) = 1d0
      tabmax(14:16) = 25d0
      tabmin(15)  = 10d0  ! SEQ_COEFFICIENTS
      tabmax(17)  = 10d0  ! NUMBER_OF_COUPLED_EQUATIONS
      itype(18) = 5       ! BOUNDARY_CONDITIONS
      tabmin(18) = tabmax(13)+1d0
      tabmax(18) = tabmax(13)+nncodetimbc
      itype(19:21) = 2    ! LINEAR_SUBELEMENT, DIAGONAL_MASS_MATRIX, &
                          ! CONSISTENT_MASS_MATRIX
      itype(22) = 0       ! SEQ_OUTPUT
      tabmin(22) = 1d0
      tabmax(22) = 10d0
      tabmin(23) = 1d-10  ! ACCURACY
      tabmax(23) = 1d5
      itype(24) = 0       ! PRINT_LEVEL
      tabmin(24) =-1d0
      tabmax(24) = 1d2
      tabmin(25) = 1d0    ! THRESHOLD_TIME
      itype(26) = 5       ! AT_ERROR
      tabmin(26) = tabmax(18)+1d0
      tabmax(26) = tabmax(18)+nncodestop
      itype(27:30) = 2    ! USE_CTIMEN, KEEP_T, NON_LINEAR_ITERATION
      itype(29) = 17      ! PRINT_TIME_HISTORY
      tabmin(29) = ndim
      tabmax(29) = 50d0
      tabmin(31) = 1d-16  ! ABS_ITERATION_ACCURACY
      tabmin(32) = 1d-8   ! REL_ITERATION_ACCURACY
      tabmax(31:32) = 1d0
      itype(33) = 0       ! MAX_ITER
      tabmin(33) = 1d0
      tabmax(33) = 1d3
      itype(34:35) = 2    ! NO_COMPUTATION, REUSE_TIME_PARAMETERS
      tabmin(36:39) =-1d1 ! BETA, GAMMA, ALPHA_F, ALPHA_M
      tabmax(36:39) = 1d1
      itype(40:41) = 2    ! UPDATED_LAGRANGE, SKIP_MESH_DEFORMATION
      itype(43) = 0
      tabmin(42:43) = 1d0 ! MIN_ITER, SEQ_ADD_RHSD
      tabmax(42:43) = 1d2
      itype(44) = 38
      tabmin(44) = 1d0    ! REACTION_FORCE
      tabmax(44) = 1d3
      codemain(1) = 'END'
      codemain(2) = 'EQUATION'
      ncodemain(1) = 3
      ncodemain(2) = 5
      codesub(1:nncodetimmeth) = codetimmeth
      ih = nncodetimmeth
      codesub(ih+1:ih+nncodevar) = codevar
      ih = ih+nncodevar
      codesub(ih+1:ih+nncodetimbc) = codetimbc
      ih = ih+nncodetimbc
      codesub(ih+1:ih+nncodestop) = codestop
      ncodesub(1:nncodetimmeth) = ncodetimmeth
      ih = nncodetimmeth
      ncodesub(ih+1:ih+nncodevar) = ncodevar
      ih = ih+nncodevar
      ncodesub(ih+1:ih+nncodetimbc) = ncodetimbc
      ih = ih+nncodetimbc
      ncodesub(ih+1:ih+nncodestop) = ncodestop

!     --- Put default values into table

      table(1:nncode) = -1d20
      table(1) = 6d0
      table(2) = 0d0
      table(4) = 0d0
      table(11:17) = 1d0
      table(15) = -1d20
      table(18) = 0d0
      table(19) = 0d0
      table(22) = 1d0
      table(23) = 1d-2
      table(24) = 0d0
      table(26) = 0d0
      table(33) = 1d0
      table(42) = 1d0
      table(43:44) = 0d0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
         write(irefwr,1) 'iretur', iretur
      end if  ! ( debug )

!     --- Put information into the arrays iintim and rintim
!         method:

      method = nint(table(1))-1

      if ( method==9 ) then

!     --- Theta method, check if theta = 0

         iposut = nint(table(4))
         if ( iposut>0 ) then

!        --- theta found

            if ( tabout(iposut)<=0d0 ) then
               method = 7  ! use Euler explicit
               table(4) = 0d0  ! reset table(4) to prevent error message
            end if  ! ( tabout(iposut)<=0d0 )

         end if  ! ( iposut>0 ) then

      end if  ! ( method==9 )
      iintim(2) = method
      iintim(3) = 1
      iintim(4) = 0

!     --- numcop

      if ( icall==2 ) table(17) = 0d0
      numcop = nint(table(17))
      iintim(5) = numcop

!     --- itimbc

      iintim(6) = nint(table(18))

!     --- ireadbc

      iintim(7) = nint(table(16))
      if ( iintim(6)==3 ) then

!     --- itimbc corresponds to initial field

         iintim(7) = -1
         iintim(6) = 1

      else if ( iintim(6)==4 ) then

!     --- itimbc corresponds to old vector

         iintim(7) = -10-iintim(7)
         iintim(6) = 0

      else if ( iintim(6)>0 ) then

!     --- standard case

         iintim(6) = iintim(6)-1

      end if

!     --- itimcf

      iintim(8) = 1

!     --- iconsstiff

      iintim(10) = nint(table(12))-1

!     --- iconsmass

      iintim(11) = nint(table(11))-1

!     --- iconsrhs

      iintim(12) = nint(table(13))-1

!     --- ireadsl

      iintim(13) = nint(table(14))

!     --- imas

      if ( table(20)>=0d0 ) then
         if ( table(21)>=0d0 ) call errkeywexcl ( code, ncodmx, 20, 21 )

         imas = 1
      else if ( table(21)>=0d0 ) then
         imas = 2
      else
         imas = 2
      end if
      iintim(14) = imas

!     --- isubdivide

      iintim(16) = nint(table(19))

!     --- tinit

      icode = 2
      call readseqvar ( 'tinit', rintim, iintim(42), istrin, 0d0 )

!     --- end time

      call readtmtend ( ndt, iintim, rintim, leniintim, istrin, iseqnr )

!     --- thetas

      call readtmtheta ( iintim, ndt, rintim, leniintim, istrin, method )

      if ( method==14 ) then

!     --- Newmark method
!         Set beta, gamma

         leniintim = leniintim+1
         iintim(54) = leniintim
         icode = 36
         call readseqvar ( 'beta_newmark', rintim, iintim(leniintim), &
                           istrin, 0.25d0 )
         leniintim = leniintim+1
         icode = 37
         call readseqvar ( 'gamma_newmark', rintim, iintim(leniintim), &
                           istrin, 0.5d0 )

      else if ( method==15 ) then

!     --- generalized alpha method
!         Set beta, gamma, alpha_f and alpha_m

         leniintim = leniintim+1
         iintim(55) = leniintim
         icode = 36
         call readseqvar ( 'beta_g', rintim, iintim(leniintim), istrin, 1d0 )
         leniintim = leniintim+1
         icode = 37
         call readseqvar ( 'gamma_g', rintim, iintim(leniintim), istrin, 1.5d0 )
         leniintim = leniintim+1
         icode = 38
         call readseqvar ( 'alpha_f', rintim, iintim(leniintim), istrin, 0d0 )
         leniintim = leniintim+1
         icode = 39
         call readseqvar ( 'alpha_m', rintim, iintim(leniintim), istrin, -1d0 )

      end if  ! ( method==14 )

      if ( table(28)<=0d0 .and. table(35)<=0d0 ) then

!     --- Not keep_t found
!         time steps

         call readtmtstep ( ndt, iintim, rintim, leniintim, istrin )

!        --- toutinit, toutstep, toutend

         icode = 6
         call readseqvar ( 'toutinit', rintim, iintim(43), istrin, -1d10 )
         icode = 8
         call readseqvar ( 'toutstep', rintim, iintim(45), istrin, -1d10 )
         icode = 7
         call readseqvar ( 'toutend', rintim, iintim(44), istrin, -1d10 )

         iintim(19) = 0  ! icall_outtim
         if ( table(6)>-1d19 .or. table(7)>0d0 .or. &
              table(8)>-1d19 ) iintim(19) = 1  ! at least one of toutxxx found
         if ( iintim(19)==0 ) then

!        --- Check if the parameters toutxxx are found elsewhere

            do i = 43, 45
               if ( iintim(i)<0 ) then
                  if ( rintim(-iintim(i))>-1d9 )iintim(19) = 1
               end if  ! ( iintim(i)<0 )
            end do  ! i = 43, 45

         end if  ! (iintim(19)==0 )

      end if  ! ( table(28)<=0d0 .and. table(35)<=0d0 )

!     --- no-computation

      if ( table(34)>0 ) iintim(30) = 1

!     --- coefficients

      leniintim = leniintim+1
      iintim(37) = leniintim
      if ( numcop==1 ) then

!     --- Only one equation, make default coefficient sequence number 0

         iintim(leniintim) = 0

      else

!     --- More than one equation, set default coefficient sequence numbers

         do i = 1, numcop
            iintim(leniintim-1+i) = i
         end do

      end if  ! ( numcop==1 )

      if ( table(15)>0d0 ) then
         iposut = nint(table(15))
         ncoefs = nint(tabmax(15))
         if ( ncoefs>numcop ) then
            call errint ( ncoefs, 1 )
            call errint ( numcop, 2 )
            call errwar ( 1596, 2, 0, 0 )
         end if
         do i = 1, min(ncoefs,numcop)
            iintim(leniintim-1+i) = nint(tabout(iposut-1+i))
         end do
      end if  ! ( table(15)>0d0 )
      leniintim = leniintim+numcop-1

!     --- seq_output

      iintim(20) = nint(table(22))

!     --- accuracy

      icode = 23
      call readseqvar ( 'epstime', rintim, iintim(41), istrin, 1d-2 )
      icode = 9
      call readseqvar ( 'absepstime', rintim, iintim(46), istrin, 0d0 )

      stationary = iintim(46)>0
      if ( .not. stationary .and. iintim(46)<0 ) &
         stationary = rintim(-iintim(46))>0d0

      icode = 10
      call readseqvar ( 'relepstime', rintim, iintim(47), istrin, 0d0 )
      if ( .not. stationary ) stationary = iintim(47)>0
      if ( .not. stationary .and. iintim(47)<0 ) &
         stationary = rintim(-iintim(47))>0d0
      if ( stationary ) iintim(29) = 1

!     --- print_level, return value and threshold_time

      icode = 24
      call readseqintgr ( iintim(22), -1, 'print_level' )
      icode = 26
      call readseqintgr ( iintim(21), 0, 'at_error' )
      icode = 25
      call readseqvar ( 'threshold_time', rintim, iintim(48), istrin, 0d0 )
      if ( table(27)>0d0 ) iintim(23) = 1
      if ( table(28)>0d0 ) iintim(24) = 1
      if ( table(35)>0d0 ) iintim(24) = 2
      if ( table(28)>0d0 .and. table(35)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 28, 35 )

      if ( table(29)>0 ) then

!     --- Time history

         ih = nint(tabmax(29))
         leniintim = leniintim+1
         call readservar ( table(29), iintim(leniintim), rintim, ndim*ih, istrin )
         iintim(25) = ih
         iintim(56) = leniintim
         leniintim = leniintim+ndim*ih-1

      end if  ! ( table(29)>0 )

      if ( table(30)>0d0 .or. table(40)>0d0 ) then

!     --- non-linear iteration

         if ( iintim(29)>0 ) then

!        --- Can not be combined with stationary solution

            call errwar ( 2039, 0, 0, 0 )

         else

!        --- instationary case

            if ( table(40)>0d0 .and. table(41)>0d0 ) then

!           --- Updated Lagrange without mesh deformation

               iintim(3) = 4

            else if ( table(40)>0d0 ) then

!           --- Updated Lagrange with mesh deformation

               iintim(3) = 3

            else

!           --- Standard non-linear iteration

               iintim(3) = 2

            end if  ! ( table(40)>0d0 )

            iintim(28) = nint(table(33))
            iintim(26) = nint(table(42))
            icode = 31
            call readseqvar ( 'absepsiter', rintim, iintim(46), istrin, 0d0 )
            icode = 32
            call readseqvar ( 'relepsiter', rintim, iintim(47), istrin, 0d0 )
            if ( iintim(46)<0 ) then
               if ( rintim(-iintim(46))<=0d0 ) rintim(-iintim(46)) = 1d-2
            end if  ! ( iintim(46)<0 )

            if ( method<5 .or. method==7 .or. method==8 .or. &
                 method==11 .or. method>12 .and. method<14 .or. &
                 method>15 ) call errsub ( 2041, 0, 0, 0 )

         end if  ! ( iintim(29)>0 )

      end if  ! ( table(30)>0d0 .or. table(40)>0d0 )

      iintim(36) = nint(table(43))  ! seq_add_rhsd
      iintim(32) = nint(table(44))  ! reaction_force

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readtm1'

      end if  ! ( debug )
      call erclos ( 'readtm1' )

      end subroutine readtm1
