      subroutine sepcomppress ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute pressure in case of Navier-Stokes and store in isol
!
! **********************************************************************
!
!                       KEYWORDS
!
!     pressure
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)

!     inpsep         i    See subroutine readscderiv
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iseqpress, iseqvel, ivec, itype, itypvc
      logical :: taylorh

!     i              Loop variable
!     iseqpress      Sequence number of pressure in set of solution arrays
!     iseqvel        Sequence number of velocity in set of solution arrays
!     itype          Type number of element
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     taylorh        If true Taylor-Hood elements are used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOPSEPVEC6   Copy one degree of freedom of sepran vector in user vector
!                    The degree of freedom is not available in all nodes
!     PRDERV         Compute derivatives
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcomppress' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcomppress'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprobc => kprobpart%kprobc

      call sepfillkmesh ( kmesh )

!     --- Activate input vector

      iseqvel = inpsep(7)
      call sepactsolbf2 ( isolut2d(iseqvel) )

!     --- Create pressure vector

      iseqpress = inpsep(3)
      itypvc = 115   ! array of special structure
      ivec = 6       ! structure 6
      call sepcreasolbf1 ( isolut2d(iseqpress), itypvc, ivec, &
                           'pressure', 0 )

!     --- Activate kmesh and kprob

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      call sepfillkprob

!     --- Find type numbers for this problem

      taylorh = .false.
      do i = 1, nelgrp

         itype = kprobc(i)
         if ( itype==903 ) then

!        --- Taylor-Hood element found

            taylorh = .true.
            exit

         end if  ! ( itype==903 )

      end do  ! i = 1, nelgrp

      if ( taylorh ) then

!     --- Taylor Hood element, extract pressure from velocity vector
!         First declare pressure vector

         velocity => ks(isolut2d(iseqvel))%sol
         pressure => ks(isolut2d(iseqpress))%sol

!        --- Copy

         call prcopsepvec6 ( pressure, velocity, ndim+1 )

         istartmn = istartmn+15

      else

!     --- Compute pressure by a call to prderv

         call prderv ( inpsep )

      end if  ! ( taylorh )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcomppress'

      end if  ! ( debug )
      call erclos ( 'sepcomppress' )

      end subroutine sepcomppress
