      subroutine sepcopkmesh ( kmesh1, kmesh2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Delete old kmesh2 if it exists and copy essential information
!     from kmesh1 to kmesh2
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: kmesh1
      integer, intent(out) :: kmesh2

!     kmesh1         i    Standard SEPRAN array containing information of mesh 1
!     kmesh2        i/o   Contains extended copy of old mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istart, iseqkm1, iseqkm2

!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopkmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopkmesh'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepfillkmesh ( kmesh1 )
      iseqkm1 = iseqkm

      call sepcrmeshinfo ( kmesh2 )
      iseqkm2 = iseqkm

!     --- Copy essential parts

      km(iseqkm2)%kmesh(4:6) = km(iseqkm1)%kmesh(4:6)     ! npelm, nelgrp, ndim
      km(iseqkm2)%kmesh(10) = km(iseqkm1)%kmesh(10)       ! nuspnt
      km(iseqkm2)%kmesh(12:14) = km(iseqkm1)%kmesh(12:14) ! nsurfs, nvolms, nelgrp

      if ( km(iseqkm1)%kmesh(15)/=0 ) then

!     --- kmesh part a

         allocate ( km(iseqkm2)%kmesha(size(kmesha)) )
         km(iseqkm2)%kmesha = km(iseqkm1)%kmesha
         km(iseqkm2)%kmesh(15) = nelgrp+1

      end if  ! ( km(iseqkm1)%kmesh(15)/=0 )

!     --- kmesh part f

      allocate ( km(iseqkm2)%kmeshf(size(kmeshf)) )
      km(iseqkm2)%kmeshf = km(iseqkm1)%kmeshf
      km(iseqkm2)%kmesh(20) = size(kmeshf)

      allocate ( km(iseqkm2)%kmeshl(size(kmeshl)) )
      km(iseqkm2)%kmeshl = km(iseqkm1)%kmeshl
      km(iseqkm2)%kmesh(26) = km(iseqkm1)%kmesh(26)  ! kmesh part l

!     --- kmesh part t

      allocate ( km(iseqkm2)%kmesht(size(kmesht,1),size(kmesht,2)) )
      km(iseqkm2)%kmesht = km(iseqkm1)%kmesht
      km(iseqkm2)%kmesh(36) = km(iseqkm1)%kmesh(36)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopkmesh'

      end if  ! ( debug )
      call erclos ( 'sepcopkmesh' )

      end subroutine sepcopkmesh
