      subroutine sepcoptopolsym
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy topology of axi-symmetric mesh and fill a part of new kmesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
!     topology
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesymm
      use sepmodulekmesh
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istart, i, inpelm, istart1

!     i              Counting variable
!     inpelm         number of points per element
!     istart         First position used at a part of kmeshm
!     istart1        First position used at a part of kmeshmtmp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0003         Print KMESH parts c and g
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcoptopolsym' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcoptopolsym'
         call prinin ( nodesnew, npoint, 'nodesnew' )
         write(irefwr,1) 'nelgrp', nelgrp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      length =  size ( kmeshc )

!     --- Create new array kmeshc

      allocate ( kmeshctmp(2*length), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*length, 'kmeshctmp' )
      if ( ierror/=0 ) go to 1000

      if ( nelgrp>1 ) then

!     --- nelgrp>1, fill kmesh part g

         allocate ( km(iseqkm)%kmeshg(nelgrp) )
         km(iseqkm)%kmesh(21) = nelgrp
         km(iseqkm)%kmeshg = 2*kmeshg

      end if  ! ( nelgrp>1 )

!     --- Fill new array kmeshc

      istart = 0
      istart1 = 0
      do i = 1, nelgrp
         inpelm = Topgrp(i)%inpelm  ! number of nodes per group
         inelem = Topgrp(i)%inelem  ! number of elements per group
         kmeshctmp(istart1+1:istart1+inpelm*inelem) = &
            kmeshc(istart+1:istart+inpelm*inelem)
         istart1 = istart1+inpelm*inelem
         kmeshctmp(istart1+1:istart1+inpelm*inelem) = &
            nodesnew(kmeshc(istart+1:istart+inpelm*inelem))
         istart = istart+inpelm*inelem
         istart1 = istart1+inpelm*inelem
      end do  ! i = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         call pr0003 ( kmesh, kmeshctmp )
         write(irefwr,*) 'End sepcoptopolsym'

      end if  ! ( debug )
      call erclos ( 'sepcoptopolsym' )

      end subroutine sepcoptopolsym
