      subroutine sepfillcoeffs ( isubr, iseqcf, ictime )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coefficients in arrays iuser and user
!     Old subroutine tofillcoefs
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     matrix
!     right_hand_side
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleuser
      use sepmodulekprob
      use sepmoduletime
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isubr, iseqcf, ictime

!     ictime         i    Indication if information must be read from standard
!                         input file ( 0 ) or that information read before is
!                         used ( 1 ), see to0089
!     iseqcf         i    Sequence number corresponding to the coefficients
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                         1:    LINSOL
!                         2:    LINSTM
!                         3:    LINPRB
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenius, lenusr

!     lenius         Computed length of array IUSER
!     lenusr         Computed length of array USER
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRINTUSER      Print the contents of the arrays iuser and user
!     SEPFILLCOEFF1  Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     SEPFILLDEFCOF  Fill default coefficients in iuser/user
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillcoeffs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillcoeffs'
         call ertrace
         write(irefwr,1) 'ioutp, itimloop, iseqcf, ictime', &
                          ioutp, itimloop, iseqcf, ictime
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ioutp>=0 .and. itimloop<=1 .and. iseqcf>0 .and. &
           nitercom<=1 ) write ( irefwr, 110 ) iseqcf
110   format ( ' Fill coefficients with sequence number ', i3 )

!     --- Activate arrays

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc

!     --- Fill information of mesh and problem in modules kmesh and kprob

      call sepfillkmesh ( kmesh )
      call sepfillkprob
      nrusol = nusol-nbound-npboun
      if ( iseqcf>0 ) then

!     --- isecf>0,  use icofius(iprob) and icofus(iprob)

         if ( associated(iuser1(iprob)%iuser) ) then
            if ( associated(icofius(iprob)%iuser) ) then
               deallocate ( iuser1(iprob)%iuser, iuser1(iprob)%user )
               lenius = size(icofius(iprob)%iuser)
               lenusr = size(icofius(iprob)%user)
               allocate ( iuser1(iprob)%iuser(lenius), &
                          iuser1(iprob)%user(lenusr) )
               iuser1(iprob)%iuser = icofius(iprob)%iuser
               iuser1(iprob)%user = icofius(iprob)%user
            end if  ! ( associated(icofius(iprob)%iuser) )
         end if  ! ( .not. associated(iuser1(iprob)%iuser) )
         if ( debug ) write(irefwr,1) 'iictime', ictime
         if ( (ictime==0 .or. ictime==2 .or. ictime>=6 ) .and. iseqcf>0 ) &
            call sepfillcoeff1 ( isubr, iseqcf )
         if ( associated(icofius(iprob)%iuser) ) &
            deallocate ( icofius(iprob)%iuser, &
                         icofius(iprob)%user )
         lenius =  size(iuser1(iprob)%iuser)
         lenusr =  size(iuser1(iprob)%user)
         allocate ( icofius(iprob)%iuser(lenius), &
                    icofius(iprob)%user(lenusr) )
         icofius(iprob)%iuser = iuser1(iprob)%iuser
         icofius(iprob)%user = iuser1(iprob)%user
         iuser => iuser1(iprob)%iuser
         user => iuser1(iprob)%user

         if ( debug ) call printuser

      else

!     --- iseqcf=0, hence no coefficients found
!         Define default values if possible

         if ( debug  ) write(irefwr,*) 'before erfilldefcof'
         call sepfilldefcof

      end if  ! ( iseqcf>0 )

      if ( associated(iuser1(iprob)%iuser) ) then
         iuser => iuser1(iprob)%iuser
         user => iuser1(iprob)%user
      end if  ! ( associated(iuser1(iprob)%iuser) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillcoeffs'

      end if
      call erclos ( 'sepfillcoeffs' )

      end subroutine sepfillcoeffs
