      function sepfilldefreal ( namereal, defval )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Give sepfilldefreal either the default value defval or the value of
!     the variable namereal if it exists
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: sepfilldefreal
      double precision, intent(in) :: defval
      character (len=*), intent(in) :: namereal

!     defval         i    Default value
!     namereal       i    Name of real
!     sepfilldefreal o    Value of real at output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepgetnamevar
      double precision :: sepgetvar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPGETNAMEVAR  Get sequence number of scalar, real or integer
!     SEPGETVAR      Get value of scalar, real or integer
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfilldefreal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilldefreal'
         write(irefwr,2) 'defval', defval
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseq = sepgetnamevar ( namereal )

      if ( debug ) write(irefwr,1) 'iseq', iseq

      if ( iseq>0 ) then

!     --- iseq > 0, hence it concerns a variable or constant

         sepfilldefreal = sepgetvar(iseq)

      else

!     --- iseq = 0, use default

         sepfilldefreal = defval

      end if  ! ( iseq>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'sepfilldefreal', sepfilldefreal
         write(irefwr,*) 'End sepfilldefreal'

      end if  ! ( debug )
      call erclos ( 'sepfilldefreal' )

      end function sepfilldefreal
