      subroutine sepfilliinbld ( iinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iinbld from array iinsol
!     Old subroutine to0089consts
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     right_hand_side
!     solve
!     constant
!     extract
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulesmf
      use sepmoduleuser
      use sepmodulekprob
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinsol(*)

!     iinsol         i    Input array defined by the user, see to0089
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: maxvct, nmat, numold, ictime, iseqkmesh, ivecreac, &
                 ivectr

!     ictime         Indication if information must be read from standard
!                    input file ( 0 ) or that information read before is
!                    used ( 1 )
!     iseqkmesh      Sequence number of mesh in kmesh
!     ivecreac       Sequence number of the reaction force vector with
!                    respect to array isol (if >0)
!     ivectr         Sequence number of the solution vector with respect to
!                    array isol
!     maxvct         maximum number of vectors that may be stored in isol
!     nmat           Number of matrices required
!     numold         Number of vectors that are stored in VECOLD
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     TO0089SEMTST   Check arrays kmesh and intmat in case of spectral
!                    element method
!     TO0089TEXT     Write some text information for subroutine to0089 (build)
!                    depending on output parameters
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfilliinbld' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilliinbld'
         call prinin ( iinsol, 18, 'iinsol' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extract constants

      ivectr   = iinsol(6)
      maxvct   = iinsol(7)
      ivecreac = iinsol(10)
      iseqintmat = max(1,iinsol(19))  ! sequence number intmat1d
      ictime = iinsol(20)

      allocate ( skipelgrps(nelgrp+numnat) )

!     --- Fill iinbld

      iinbld = 0
      iinbld(1) = nbld

      if ( iinsol(15)<=1 .and. iinsol(16)<=1 .and. iinsol(18)<1 ) then

!     --- Build matrix and right-hand side

         if ( ictime<4 ) then
            iinbld(2) = 1
         else
            iinbld(2) = 2
         end if  ! ( ictime<4 ) then

      else if ( iinsol(16)<=1 .and. iinsol(18)<1 ) then

!     --- Build right-hand side

         iinbld(2) = 2

      else if (  iinsol(15)<=1 ) then

!     --- Build matrix

         iinbld(2) = 14

      else

!     --- Build nothing

         iinbld(2) = 4

      end if  ! ( iinsol(15)<=1 .and. iinsol(16)<=1 .and. iinsol(18)<1 )

      if ( ivecreac>0 ) iinbld(8) = 1
      iinbld(9) = iprob
      iinbld(10) = maxvct
      iinbld(12) = 1
      iinbld(13) = max(1,min(ivectr,maxvct))
      iinbld(14:15) = iinsol(8:9)
      if ( maxvct==1 ) iinbld(5) = 1

      iseqkmesh = 1

!     --- Perform some checks

      intmat => intmat1d(iseqintmat)
      intmatar => intmat1d(iseqintmat:size(intmat1d))
      keepmat = iinsol(15)
      if ( iinsol(12)>0 ) then

!     --- Finite element preconditioning required, check kmesh

         iseqkmesh = 2
         call to0089semtst ( kmesh2d(meshnr), iseqkmesh, &
                             iinsol(12) )

      end if
      iseqkmesh = iseqkmesh+meshnr-1
      iseqsol = ivectr
      iseqsolut = ivectr
      iseqreac = ivecreac

!     --- make relation between standard arrays and SMF

      numold = max(1,iinbld(10))

      SMF%iusol => isolut2d(ivectr)
      SMF%islold => isolut2d
      stiffmat => stiffm(iprob:iprob+1)
      SMF%irhsd => irhsd
      massmat => massmt(iprob)
      kmesh => kmesh2d(iseqkmesh)

!     --- Write some text information for subroutine to0089
!         depending on output parameters

      if ( debug ) write(irefwr,*) 'before to0089text'
      call to0089text ( iinsol )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinbld, iinbld(1), 'iinbld' )
         write(irefwr,*) 'End sepfilliinbld'

      end if
      call erclos ( 'sepfilliinbld' )

      end subroutine sepfilliinbld
