      subroutine sepfillinpsol
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays inpsol and rinsol with default values
!     These values are extended with variables found before
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLINPSOL1 Fill arrays inpsol and rinsol with default values
!                    extended with given values in the form of scalars or reals
!                    Special case: profile storage (hence direct)
!     SEPFILLINPSOL5 Fill arrays inpsol and rinsol with default values
!                    extended with given values in the form of scalars or reals
!                    Special case: jmethod = 5-8 (compact storage)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillinpsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillinpsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      if ( IM%direct .or. IM%mumps ) then

         call sepfillinpsol1

      else

         call sepfillinpsol5

      end if  ! ( IM%direct )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillinpsol'

      end if  ! ( debug )
      call erclos ( 'sepfillinpsol' )

      end subroutine sepfillinpsol
