      subroutine sepfillinpsol9
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays inpsol and rinsol with default values
!     These values are extended with variables found before
!     Special case: row compact storage (hence over-relaxation)
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepfilldefint, sepgetnamevar
      double precision :: sepgetvar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLDEFINT  Give sepfilldefint either the default value defval or the
!                    value of the variable nameint if it exists
!     SEPGETNAMEVAR  Get sequence number of scalar, real or integer
!     SEPGETVAR      Get value of scalar, real or integer
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillinpsol9' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillinpsol9'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsol(3,1) = 5  ! over-relaxation
      inpsol(5,1) = sepfilldefint ( 'SOL_MAXITER', 0 ) ! maxiter
      inpsol(9,1) = 1  ! istart (old solution)

      iseq = sepgetnamevar ( 'SOL_MINIMUM' )
      if ( iseq>0 ) then

!     --- sol_minimum found

         rinsol(7,1) = sepgetvar(iseq)
         inpsol(18,1) = 1  ! constrain is stored in rinsol(7)
         inpsol(17,1) = 1  ! constrains required (u>=umin)

      end if  ! ( iseq>0 )

      iseq = sepgetnamevar ( 'SOL_MAXIMUM' )
      if ( iseq>0 ) then

!     --- sol_maximum found

         rinsol(8,1) = sepgetvar(iseq)
         inpsol(19,1) = 1  ! constrain is stored in rinsol(8)
         inpsol(17,1) = 2+inpsol(17,1)  ! constrains required

      end if  ! ( iseq>0 )

1000  call erclos ( 'sepfillinpsol9' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillinpsol9'

      end if  ! ( debug )

      end subroutine sepfillinpsol9
