      subroutine sepps1 ( posint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 30-01-2016 New parameter list
!        version  6.13 date 25-11-2012 Long integers
!        version  6.12 date 05-03-2012 Fortran 90
!
!   copyright (c) 1988-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine SEPPS1 performs the post processing of SEPRAN files
!    SEPPS1 is called by program SEPPOST
!    It uses the output written by subroutines OUTPUT or OUTTIM into the
!    files sepcomp.inf and sepcomp.out
!   The file sepcomp.inf is filled as follows:
!          First records array IINPUT as output from subroutine PROBDF
!          IINPUT is written by subroutine SEPSTR
!          Next records:
!          Rec. 1:  idirec, itype
!          Rec. 2:  numvec
!          Rec. 3...: Sequence numbers of arrays that have been
!                     written to file sepcomp.out
!          with
!              idirec:  indication if the file sepcomp.out is unformatted
!                       (1) or formatted (0)
!              itype:   type of solution that is written (1)
!              numvec:  number of vectors that have been written to the
!                       file sepcomp.out
!   The arrays in the file sepcomp.out are written by the subroutine prinsl
!   which allows for formatted arrays only at this moment
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulecons
      use sepmoduleplot
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical posint

!     posint     i    true if seppost is called interactively
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, niter, mmcoor(ntimvc)
      logical meshonly

!     i              Counting variable
!     meshonly       If true only the mesh is plotted
!     mmcoor         Memory management sequence number with respect to
!                    coor
!     niter          Iteration sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FINISH         Stop SEPRAN
!     SEPMAKEREL     Make relation between local and global arrays
!     SEPPS2         Actual body of seppost
!     SEPST1         SEPRAN start and reading of mesh, problem definition and
!                    solution arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data mmcoor / ntimvc*0 /
! ======================================================================
!
!     --- Initialize all arrays

      kmesh2d = 0
      kprob2d = 0
      isolut3d = 0
      meshnr = 1
      call sepmakerel

!     --- Initialize array names

      do i = 1,26
         names(i) (1:8) = 'vector v'
         names(i) (11:30) = '                    '
         write( names(i) (9:10), 90 ) i-1
90       format(i2)
      end do

!     --- Start the program, read the mesh, the problem definition and the
!         solution arrays
      call sepst1 ( time, posint, niter, meshonly )
      if ( ierror/=0 ) go to 1000
      call eropen ( 'seppost')
      call eropen ( 'sepps1')
      call sepps2 ( posint, mmcoor, meshonly )
      call erclos ( 'sepps1')
      call erclos ( 'seppost' )
1000  print *, 'end of seppost'
      call finish(0)
      end
