      subroutine sepps2intersct
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 30-01-2016 Replace meshwrbf by meshwr
!        version  1.2  date 20-06-2013 New definition file ref numbers
!        version  1.1  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Intersect a 3d mesh by a plane and create 2d mesh
!     Map all solutions onto this 2d mesh
!     write mesh and solution arrays for this 2d mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     post_processing
!     write
!     file
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulekprob
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemach
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iref, iplane, &
              isollc(5,100), i, iref74sav, numveclc, ichoicearr(3), &
              iinplt(100), iseqkm1
      double precision rinplt(10)
      logical plot
      integer, pointer :: kmeshlc, kmeshorig
      type (mapping) :: maplc

!     i              Counting variable
!     ichoicearr     Array of size 3 containing the three parts of ichoice
!                    1:  ichoice_plane
!                    2:  ichoice_tang
!                    3:  ichoice_orig
!     iinplt         Input array for subroutine PLOTMS:
!     iplane         Defines direction of plane
!                    1:  plane x = constpstant
!                    2:  plane y = constpstant
!                    3:  plane z = constpstant
!                    4:  general plane
!     iref           File reference number
!     iref74sav      Help variable to store the value of iref74
!     iseqkm1        Sequence number with respect to km of first mesh
!     isollc         Standard SEPRAN array containing information of the
!                    2d velocity vector stored as solution vector
!     kmeshlc        Standard SEPRAN array containing information of the
!                    2d mesh
!     kmeshorig      Old array kmesh
!     maplc          Mapping information
!     numveclc       Number of vectors that have been filled
!     plot           If true the mesh must be plotted
!     rinplt         Input array for subroutine PLOTMS:
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     PRINTERPOL3D   Interpolate all solutions to 2d mesh, defined by
!                    intersection of 3d mesh by plane
!     PROUTPSUB      Write second part of information to file sepcomp.out
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPINTSEC3DMSH Intersect 3d mesh with a plane
!                    create new 2d mesh consisting of triangles
!                    Compute mapping array from 3d mesh to 2d mesh
!     SEPMESHWR      fill the file meshoutput with the mesh
!     SEPPLOTMS      plot mesh
!     SEPPRINOUTP    Write solution vectors to sepcomp.out
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
!     SEPRMPROBINFO  Remove array with sequence number kproblc from kp
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     WRITPROBIINP   Write arrays iinput and rinput corresponding to kprob to
!                    file sepcomp.out
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
!    2832   No file_mesh_out defined in the input file, following the keyword
!           intersect_mesh
!    2833   No file_solut_out defined in the input file, following the keyword
!           intersect_mesh
!    2836   No intersection of surface with plane found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     check input
!     compute 2d mesh and mapping array
!     interpolate all solutions to 2d mesh
!     write 2d mesh to file
!     write 2d solutions to file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepps2intersct' )
      debug = .false.
      plot = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepps2intersct'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- check input

      if ( ndim/=3 ) then

!     --- ndim # 3, not available

         call errint ( ndim, 1 )
         call errint ( 3, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )

      end if  ! ( ndim/=3 )

      if ( iinputpst(37)<=0 ) then

!     --- iinputpst(37)<=0, hence file_mesh_out not read

         call errsub ( 2832, 0, 0, 0 )

      end if  ! ( iinputpst(99)/=100 )
      if ( iinputpst(38)<=0 ) then

!     --- iinputpst(38)<=0, hence file_solut_out not read

         call errsub ( 2833, 0, 0, 0 )

      end if  ! ( iinputpst(99)/=100 )
      if ( ierror/=0 ) go to 1000

      iseqkm1 = kmesh

!     --- compute 2d mesh and mapping array

      kmeshorig => kmesh
      kmeshlc => kmesh2d(maxmesh+1)
      kprob => kprob2d(maxmesh+1)
      ichoicearr(1) = 4  !  ichoice_plane
      ichoicearr(2) = 0  !  ichoice_tang
      ichoicearr(3) = 0  !  ichoice_orig
      call sepintsec3dmsh ( kmeshlc, rinputpst(17), maplc, &
                            ichoicearr, rinputpst(17), iplane )
      if ( ierror==2836 ) then

!     --- error 2836, print error message

         call errsub ( 2836, 1, 1, 0 )

      end if  ! ( ierror==2836 )
      if ( ierror/=0 ) go to 1000

      if ( plot ) then

!     --- Plot is true, plot intersected mesh

         iinplt(1) = 6
         iinplt(2) = 0
         iinplt(3) = 0
         iinplt(4) = 0
         iinplt(5) = 0
         iinplt(6) = 0
         rinplt(1) = 15d0
         rinplt(2) = 1d0
         call sepplotms ( iinplt, rinplt, iskip )

      end if  ! ( plot )

!     --- write 2d mesh to file

      iref = inisetnewref()
      open ( unit=iref, file = textx(1:iinputpst(37)) )

      kmesh => kmeshlc
      call sepfillkmesh(kmesh)
      call sepmeshwr ( iref )
      close ( unit=iref )

!     --- interpolate all solutions to 2d mesh
!         Create new kprob

      call printerpol3d ( kmeshlc, kprob, &
                          isollc, isol, maplc, kmeshorig, numveclc )

!     --- write 2d solutions to file

      iref74sav = iref74
      iref74 = sign(iref,iref74)
      open ( unit = abs(iref74), file = texty(1:iinputpst(38)) )

!     --- write first part to file

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      call sepfillkprob
      call writprobiinp

!     --- write second part to file

      call proutpsub ( numveclc, ipost, names, rpost )

!     --- write vectors

      do i = 1, numveclc
         call sepprinoutp ( isollc(1,i), i-1, 1, 0d0 )
      end do  ! i = 1, numveclc

      close ( abs(iref74) )
      call inirmref ( abs(iref74) )
      iref74 = iref74sav

!     --- Remove 2d mesh and solutions as well as array map

      call seprmmeshinfo ( kmeshlc )
      call seprmprobinfo ( kprob )
      do i = 1, numveclc
         call seprmsolinfo ( isollc(1,i) )
      end do  ! i = 1, numveclc
      if ( associated(maplc%imap) ) deallocate ( maplc%imap )
      if ( associated(maplc%rmap) ) deallocate ( maplc%rmap )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepps2intersct'

      end if  ! ( debug )
      call erclos ( 'sepps2intersct' )

      end subroutine sepps2intersct
