      subroutine sepreadscendif ( inpsep, istart, iseqif, iarrif )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the input due to the keyword
!     END_IF
!     with respect to subroutine readsc
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     if_statement
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), iseqif, iarrif(*)

!     iarrif        i/o   Array of size 10 in which information of the if
!                         statements is stored in the following way:
!                         iarrif(i) = starting address of if part in inpsep
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine READSCWHILE for a description
!                         The only difference is that pos 0 is (code) has value
!                         83
!                         Pos. 1 is used for the length
!                         Pos. 2 for the jump
!                         Pos. 3 to length-2 contains the information
!     iseqif        i/o   Present level of if loop
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3211   END_IF found but no corresponding IF statement
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadscendif' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadscendif'
         write(irefwr,1) 'lastitem', lastitem
         write(irefwr,1) 'icode, istart, iseqif', icode, istart, iseqif
         write(irefwr,1) 'iarrif(iseqif)', iarrif(iseqif)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode

      if ( iseqif>0 ) then

         inpsep(iarrif(iseqif)) = istart
         iseqif = iseqif - 1

      else

         call errsub ( 3211, 0, 0, 0 )

      end if
      istart = istart + 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart, iseqif', istart, iseqif
         call prinin ( inpsep, istart, 'inpsep' )
         write(irefwr,*) 'End sepreadscendif'

      end if  ! ( debug )
      call erclos ( 'sepreadscendif' )

      end subroutine sepreadscendif
