      subroutine sepst1 ( time, posint, niter, meshonly )
! ======================================================================
!
!        programmer    Guus Segal
!        version 10.0  date 30-01-2016 New parameter list
!        version  9.3  date 25-11-2012 Long integers
!        version  9.2  date 16-06-2010 extra debug
!
!   copyright (c) 1988-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       SEPST1 starts the SEPRAN program SEPPOST.
!       It must be called beore all other subroutines.
!       The mesh is supposed to be made by program SEPMESH and stored in the
!       file meshoutput
!       The input for subroutine PROBDF is stored in array iinputlc and read
!       from the file sepcomp.inf
!       The files meshoutput, sepcomp.inf and sepcomp.out are opened, &
!       the file meshoutput is closed.
!       The solution vectors are read from the file sepcomp.out and stored
!       in memory with the aid of subroutine READSL. The information of
!       the storage is placed in array ISOL.
!       The machine dependent reference numbers iref10, iref73 and iref74 get
!       a value
!       SEPST1 replaces the calls of the following SEPRAN subroutines:
!              START:  General SEPRAN starting subroutine
!              MESHRD: Reads mesh.
!              PROBDF: Reads problem definition.
!              READSL: Read SEPRAN solution vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     start
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niter
      double precision time(*)
      logical posint, meshonly

!     meshonly       o    If true only the mesh is plotted
!     niter          o    Maximum number of iterations
!     posint         o    true if seppost is called interactively
!     time           o    Array of length 100 containing the actual time at each
!                         time-step in increasing sequence
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer manag(lenmanag)

!     manag          Manager array that sets the parameters in subroutine
!                    SEPSTL
!                    See subroutine readsp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     READ74         Read solution vectors from sepcomp.out
!     SEPST1SUB1     Perform the first part of subroutine sepst1
!     SEPST1SUB2     Perform the last part of subroutine sepst1
! **********************************************************************
!
!                       I/O
!
!    The files meshoutput, sepcomp.inf and sepcomp.out and if it exists
!    sepcomp.freq are read
!    For a description of meshoutput: see the Programmers Guide Section 3.3
!    For a description of the file sepcomp.inf, see subroutine PROUTP
!    For a description of the file sepcomp.out, see subroutine PRINSL
!    For a description of the file sepcomp.freq, see subroutine FVWRITE
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill common blocks
!     Open files
!     Read array manag
!     Start SEPRAN
!     Open files with mesh and solution
!     Read mesh
!     Read problem definition from sepcomp.inf
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false.

!     --- Perform first part

      call sepst1sub1 ( posint,  meshonly, manag )
      if ( ierror/=0 .or. meshonly ) go to 1000

!     --- Read solution from file 74 with input from file 73

      call read74 ( time, niter, manag )

      if ( debug ) write(irefwr,*) 'after read74'
      if ( ierror/=0 ) go to 1000

!     --- Perform second part

      call sepst1sub2

1000  if ( debug ) write(irefwr,*) 'end of sepst1'
      call erclos ( 'sepst1' )

      end
