      subroutine to0089text ( iinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 24-12-2014 New definition clinsl
!        version  2.1  date 17-12-2009 Less output
!        version  2.0  date 28-03-2007 Remove iinbld
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write some text information for subroutine to0089 (build)
!     depending on output parameters
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     right_hand_side
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmoduletime
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinsol(*)

!     iinsol         i    Input array defined by the user, see to0089
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nch, ivectr, iprecfem, ictime

!     ictime         Indication if information must be read from standard
!                    input file ( 0 ) or that information read before is
!                    used ( 1 ).  See to0089
!     iprecfem       Indicates if a FEM preconditioning must be applied
!                    in case of a spectral mesh
!                    Possible values:
!                    0: no FEM preconditioning
!                    1: FEM preconditioning
!     ivectr         Sequence number of the solution vector with respect to
!                    array isol
!     nch            Number of characters in the name of vector "ivectr"
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0089text' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0089text'

      end if
      if ( ierror/=0 ) go to 1000

      ivectr   = iinsol(6)
      iprecfem = iinsol(12)
      ictime = iinsol(20)
      nch = index(names(ivectr),' ')-1
      if ( nch<=0 ) nch = len(names(ivectr))
      if ( ictime<4 ) then
         if ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 ) then

!        --- Output required

            if ( iprecfem==0 ) then

!           --- Standard finite element method

               write ( irefwr, 140 ) names(ivectr)(1:nch)
140            format ( ' Build matrix and right-hand side for', &
                        ' solution vector ', a )

            else

!           --- Spectral element method

               write ( irefwr, 150 ) names(ivectr)(1:nch)
150            format ( ' Build matrix and right-hand side for', &
                        ' FEM preconditioner ', a )

           end if

         end if

      else

         if ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 ) &
            write ( irefwr, 160 ) names(ivectr)(1:nch)
160      format ( ' Build right-hand side for solution vector ', a )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0089text'

      end if
      call erclos ( 'to0089text' )

      end
