      subroutine to0095 ( iinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 20-08-2016 use sepmoduleinput
!        version  3.5  date 25-11-2012 Long integers
!        version  3.4  date 17-12-2009 Less output
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Solve system of non-linear equations
!    The user input has been read before by subroutine READAL
! **********************************************************************
!
!                       KEYWORDS
!
!     non_linear
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleinput
      use sepmoduletime
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinsol(*)

!    iinsol    i   Input array defined by the user
!                  With this array the user may indicate which actions
!                  he wants to be performed by this subroutine
!                  See readnl
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iread, iseqnr, ivectr, maxvc0, i, &
              ipos, leniin, lenrin, niinp, &
              nrinp, nch, maxcal
      integer, pointer :: iinput(:)
      double precision, pointer :: rinput(:)

!     i              Counting variable
!     iinput         integer input array
!     ipos           Position of input arrays in sepmoduleinput
!     iread          Defines how the input must be read
!                    Possible values:
!                    0:  All SEPRAN input has been read by subroutine SEPSTN
!                        until END_OF_SEPRAN_INPUT or end of file has been found
!                        This input is used
!                    1:  The input is read as described for NLNPRB
!                    2:  The last input read as described for NLNPRB
!                        is used (equivalent with NLNPRB ICTIME=1)
!     iseqnr         Sequence number with respect to the storage in
!                    sepmoduleinput
!     ivectr         Sequence number of the solution vector with respect to
!                    array isolut
!     leniin         Computed length of array iuser
!     lenrin         Computed length of array user
!     maxcal         Maximum number of coefficient inputs read
!     maxvc0         Number of vectors stored in isolut
!     nch            Number of characters in a string
!     niinp          Estimated and actual length of integer input array IINSOL
!     nrinp          Estimated and actual length of real input array RINSOL
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INICHINPSOL    Adapt input array with respect to variables
!     READNL         Read card input for non-linear solver
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     TO0091         Actual solution of system of non-linear equations
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1530   No input for nonlinear equations
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0095' )

!     --- extract constants from iinsol

      iread  = iinsol(2)
      iseqnr = iinsol(3)
      iprob  = iinsol(4)
      ivectr = iinsol(5)
      maxvc0 = iinsol(6)
      if ( ioutp>=0 .and. itimloop<=1 ) then
         nch = index(names(ivectr),' ')-1
         if ( nch<=0 ) nch = len(names(ivectr))
         write(irefwr,100) names(ivectr)(1:nch)
100      format ( ' Compute vector ', a, &
                  ' by solution of non-linear', &
                  ' system of equations ' )

      end if
      if ( iread==0 ) then

!     --- iread = 0, Input has been read by readal

         call sepgetinpfinf ( 12, iseqnr, ipos, maxcal )
         if ( maxcal==0 ) then

!        --- No input for non-linear equations found

            call errsub ( 1530, 0, 0, 0 )
            go to 1000

         end if

         leniin = size( in(ipos)%iinput )
         lenrin = size( re(ipos)%rinput )
         allocate ( iinput(leniin), rinput(lenrin), stat = error )
         if ( error/=0 ) call eralloc ( error, leniin+lenrin, 'iinput' )
         if ( ierror/=0 ) go to 1000

         iinput(1:leniin) = in(ipos)%iinput
         rinput(1:lenrin) = re(ipos)%rinput

      else

!     --- iread # 0, Input must be read

         niinp = 2000
         nrinp = 100

         allocate ( iinput(niinp), rinput(nrinp), stat = error )
         if ( error/=0 ) call eralloc ( error, niinp+nrinp, 'iinput' )
         if ( ierror/=0 ) go to 1000

         iseqnr = 1

         call readnl ( iinput, rinput, 1, iseqnr, &
                       iprob, nprob, niinp, nrinp )
         leniin = niinp

      end if

!     --- Adapt variables

      call inichinpsol ( iinput, leniin )

      iinput(8) = ivectr
      iinput(10) = maxvc0
      call to0091 ( iinput, rinput, iinsol(7) )

      deallocate ( iinput, rinput, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinput' )

1000  call erclos ( 'to0095' )
      end subroutine to0095
