      subroutine to0096 ( usol, number, uout, iskip, iskpar, elgrps )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date 30-12-1995
!
!
!
!   copyright (c) 1995-1995 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Store unknown positioned in center of two-dimensional element only
!   into array uout organized per element
!   This subroutine in fact puts the unknown in an array of type 116
!   with one unknown per element
!   Values that are not available are set equal to -infinity
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision usol(*), uout(*)
      integer number, iskip, iskpar(*), elgrps(*)

!     elgrps     i    Array of length nelgrp that is only used if special=2
!                     In that case elgrps(ielgrp)=0 means skip element
!                     and elgrps(ielgrp)=1 is treat element
!     iskip      i    Indicates if element groups must be skipped (>0)
!                     or not (0)
!     iskpar     i    array of length NELGRP that is only used if ISKIP > 0
!                     It indicates if element group ielgrp must be skipped
!                     iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     number     i    The number-th unknown in each nodal point of the array
!                     is considered. If the array is complex number=2*i-1
!                     corresponds to the real part of array i and number=2*i
!                     to the imaginary part of array i
!     uout       o    Output vector containing the solution in the required
!                     points
!     usol       i    Solution vector
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikelmc, ielem, i, ielgrp, inpelm, irelem, inodp, iusol
      double precision amin

!     amin       Minimum value in solution array
!     i          Counting variable
!     ielem      Element number
!     ielgrp     element group number
!     ikelmc     position in kmesh part c
!     inodp      Nodal point number
!     inpelm     number of points per element
!     irelem     Relative element sequence number with respect to element group
!     iusol      Position in solution vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ikelmc = 0
      ielem  = 0
      uout(1:nelem) = -rinfin
      elgrps(1:nelgrp) = 1
      amin = rinfin
      do ielgrp = 1, nelgrp
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)

!        --- Check elements

         if ( ishape<=5 .or. iskip>0 .and. iskpar(ielgrp)==0 ) &
            then

!        --- This element group must be skipped

            ikelmc = ikelmc+inpelm*inelem
            ielem = ielem+inelem
            elgrps(ielgrp) = 0

         else

!        --- element contains centroid, find unknown in centroid

            do irelem = 1, inelem
               inodp = kmeshc(ikelmc+inpelm)
               ielem = ielem + 1
               ikelmc = ikelmc + inpelm
               if ( indprpi>0 ) then

!              --- indprpi > 0, use array kprobpi

                  iusol = kprobpi(inodp,number)

               else

!              --- indprf > 0, use array kprobfi

                  if ( kprobfi(inodp+1)-kprobfi(inodp)>=number) then
                     iusol = kprobfi(inodp)+number
                  end if

               end if
               if ( iusol>0 ) then
                  uout(ielem) = usol(iusol)
                  amin = min(amin,uout(ielem))
               end if
            end do ! irelem = 1, inelem

         end if

      end do ! ielgrp = 1, nelgrp
      do i = 1, nelem
         if ( uout(i)<=-rinfin ) uout(i) = amin
      end do ! i = 1, nelem

      end
