      subroutine defm01 ( coor, kprobf, npoint, nphys, ndim, &
                          indprf, usol, scale )
! ======================================================================
!
!        programmer     Marcel Rutten
!        version 1.0    date   24-07-1997
!
!
!   copyright (c) 1997-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written consent from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!      DEFM01   transforms coordinates of a mesh using computed displace- &
!               ments in each nodal point. It is called by DEFMSH
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer kprobf(*), ndim, nphys, npoint, indprf
      double precision coor(*), usol(*), scale

!     coor     i/o Physical coordinates of nodal points
!     indprf   i   Index to KPROB, part f
!     kprobf   i   KPROB, part f, contains number of dof in nodes
!     ndim     i   Dimension of space
!     nphys    i   Number of variables in nodes
!     npoint   i   Number of nodal points
!     scale    i   Scaling factor (useful when small displacements are
!                  involved)
!     usol     i   Physical value of dof in each nodal point
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ndegfd, idim, ipoint, idof
      double precision x(3)

!     idim
!     idof
!     ipoint
!     ndegfd
!     x
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Loop over all nodal points:

      do ipoint = 1, npoint

!     --- Fetch the coordinates:

         do idim = 1, ndim
           x(idim) = coor(ndim*(ipoint-1)+idim)
         end do

!        --- Fetch the degrees of freedom and their values:

         if ( indprf>0 ) then
            idof   = kprobf(ipoint) + 1
            ndegfd = kprobf(ipoint + 1) - idof +1
         else
            idof   = (ipoint -1) * nphys + 1
            ndegfd = nphys
         end if

!        --- Transform the original coordinates to new coordinates, using
!            displacements:
!            A second loop over the nodal point coordinates:

         do idim = 1, ndim
            if (ndegfd>0) then

!           --- Possibility 1: no renumbering:

               coor(ndim * (ipoint - 1) + idim) = &
                            usol(idof + idim - 1) * scale + x(idim)
            end if
         end do
      end do
      end
