      subroutine msh001 ( iinput, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version 17.3  date 22-05-2014 Extension with shape 22
!        version 17.2  date 30-01-2014 Correction real properties
!        version 17.1  date 15-11-2012 Long integers
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element shapes in array kmesh
!     Compute kmesh parts a,b and f
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduletexts
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*)
      double precision rinput(*)

!     iinput     i    array containing the integer input sequentially
!     rinput     i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgeg(22), iinner(10), i, ielgrp, inner, &
              jnpelm, igrp, nspgrp, n
      double precision dummy(1)

!     dummy          Dummy parameter
!     i              Counting variable
!     ielgeg         array with number of nodes for each element shape
!     ielgrp         element group number
!     igrp           loop variable for groups of spectral parameters
!     iinner         ?
!     inner          number of internal points in element
!     jnpelm         Number of nodes in an element
!     n              help parameter to compute the length of a part of kmesh
!     nspgrp         Number of spectral groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INCOPY         Copy one integer array into another
!     MSH058         fill the part of KMESH part F that corresponds
!                    to line elements, surface elements and volume elements
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ielgeg / 2, 3, 3, 6, 4,   9, 7,10, 5, 4, &
                    4,10, 8,27,14,  15, 9, 5, 0, 0, &
                    6,18 /, &
           iinner / 0, 0, 0, 0, 0,   1, 1, 1, 1, 1/
! ======================================================================
!
      call eropen ( 'msh001' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh001'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      kelma = 0
      kelmb = 0

      inpelm = 0
      if ( debug ) then
         write(irefwr,1) 'ncurvs, nsurfs, nvolms, nlines, nspec', &
                          ncurvs, nsurfs, nvolms, nlines, nspec
         write(irefwr,1) 'iinp9, iinp15, nelgrpdum', &
                          iinp9, iinp15, nelgrpdum
      end if  ! ( debug )

!     --- Fill first part of array KMESH part f

      if ( associated(kmeshf) ) nullify(kmeshf)
      allocate ( kmeshf(10000), stat = error )
      if ( error/=0 ) call eralloc ( error, 10000, 'kmeshf' )
      if ( ierror/=0 ) go to 1000

      call msh058 ( iinput, ielgeg, iinner )
      if ( ierror/=0 ) go to 1000

      allocate ( km(iseqkm)%kmeshf(nelgrp), stat = error )
      if ( error/=0 ) call eralloc ( error, nelgrp, 'kmeshf' )
      if ( ierror/=0 ) go to 1000
      km(iseqkm)%kmesh(20) = nelgrp

      km(iseqkm)%kmeshf = kmeshf(1:nelgrp)

      deallocate ( kmeshf )
      kmeshf => km(iseqkm)%kmeshf
      kelmf = nelgrp

      if ( .not. associated(Topgrp) ) allocate ( Topgrp(nelgrp) )

      Topgrp(:)%inpelm = npelm

      kelgrp = nelgrp*npelm
      if ( debug ) &
         write(irefwr,1) 'nelgrp, jboun, jnodp, kelgrp', &
                          nelgrp, jboun, jnodp, kelgrp

      if ( jnodp==0 ) then

!     --- jnodp = 0   fill kmesh part a

         kelma = nelgrp+1
         allocate ( km(iseqkm)%kmesha(nelgrp+1) )
         kmesha => km(iseqkm)%kmesha
         km(iseqkm)%kmesh(15) = nelgrp+1
         kmesha(1) = 0
         inpelm = 0

!        --- First for all usual elements

         do ielgrp = 1, nelgrp-nelgrpdum
            ishape = kmeshf(ielgrp)
            if ( debug ) &
               write(irefwr,1) 'ielgrp, ishape', ielgrp, ishape
            if ( ishape<=-1 ) then

!           --- ishape<0

               if ( ishape<=-10 .and. ishape>-10000 ) then
                  jnpelm = -ishape-9
               else if ( ishape==-10001 ) then
                  jnpelm = 2
               else if ( ishape<-10001 ) then
                  jnpelm = ielgeg(-ishape-10001)*2
               else
                  jnpelm = 0
               end if

            else if ( ishape>=70 ) then

!           --- ishape>=70, surface interface elements

               jnpelm = ielgeg(ishape-70)*2

            else if ( ishape>=50 ) then

!           --- ishape>=50, curve interface elements

               jnpelm = (ishape-48)*2

            else if ( ishape>0  .and. &
                      (ishape<19 .or. ishape>20) .and. &
                      ishape<=22 ) then

!           --- ishape>0

               jnpelm = ielgeg(ishape)

            else

!           --- ishape not available

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0)

            end if

            Topgrp(ielgrp)%inpelm = jnpelm

            inpelm = inpelm + jnpelm
            kmesha(ielgrp+1) = inpelm

         end do

!        --- Finally for all dummy element groups

         do i = 1, nelgrpdum

            ielgrp = nelgrp-nelgrpdum+i
            jnpelm = iinput(iinp15+2*i)
            inpelm = inpelm + jnpelm
            kmesha(ielgrp+1) = inpelm
            Topgrp(ielgrp)%inpelm = jnpelm

         end do
         kelgrp = inpelm

      end if

      if ( nelgrp>1 ) then

!     --- nelgrp>1  initialize part g

         kelmg = 1
         allocate ( km(iseqkm)%kmeshg(nelgrp) )
         kmeshg => km(iseqkm)%kmeshg
         kmeshg = 0
         km(iseqkm)%kmesh(21) = nelgrp

      end if

      if ( jboun==0 ) then

!     --- jboun = 0   fill kmesh part b

         kelmb = 1
         allocate ( km(iseqkm)%kmeshb(nelgrp) )
         kmeshb => km(iseqkm)%kmeshb
         km(iseqkm)%kmesh(16) = nelgrp

         do ielgrp = 1,nelgrp

            ishape = kmeshf(ielgrp)

            if ( ishape<=-10000 ) then

!           --- ishape<=-10000

               inner = 0
               inpelm = kmesha(ielgrp+1)-kmesha(ielgrp)

            else if ( ishape<=-1 ) then

!           --- ishape<=-1

               inpelm = 0
               inner = 0

               if ( ishape<-10 ) then

!              --- ishape<-10

                  ishape = -ishape-10
                  inpelm = ishape+1

               end if

            else if ( ishape>=50 ) then

!           --- ishape>=50, special

               inpelm = (ishape-48)*2
               inner = inpelm

            else

!           --- ishape>-1

               inpelm = ielgeg(ishape)
               if ( ishape<=10 ) then
                  inner = iinner(ishape)
               else
                  inner = 0
               end if

            end if

            kmeshb(ielgrp) = inpelm-inner

         end do

      end if

!     --- Set KMESH part t equal to zero

      n = nvolms
      if ( n==0 ) n = nsurfs
      if ( n==0 ) n = ncurvs
      kelmt = 2*n
      km(iseqkm)%kmesh(36) = kelmt
      allocate ( km(iseqkm)%kmesht(2,n) )
      kmesht => km(iseqkm)%kmesht
      kmesht = 0

      if ( nspec/=0 ) then

!     --- fill values for (general) spectral elements in kmesh
!         in this version only one group nspgrp=1) is considered

         kelmu    = 1

         nspgrp = iinput( iinp9 )
         allocate ( km(iseqkm)%kmeshu(1+nspgrp*4) )
         kmeshu => km(iseqkm)%kmeshu
         kmeshu(1) = nspgrp
         km(iseqkm)%kmesh(38) = 1+nspgrp*4

         do igrp = 1 , nspgrp

!        --- Copy each group in kmesh

            do i = 1 , nspgrp * 4
               kmeshu(i+1) = iinput( iinp9 + i )
            end do

         end do

      end if

      if ( nintprop>0 .or. nrealprop>0 ) then

!     --- Properties found, fill array kmesh part w

         if ( debug ) call prinin ( kmeshw(1), 4, 'kmeshw/1')
         allocate ( km(iseqkm)%kmeshw(4) )
         kmeshw => km(iseqkm)%kmeshw
         kmeshw(1) = nintprop
         kmeshw(2) = nrealprop
         if ( nintprop>0 ) then

!        --- integer properties found, create array intprop

            allocate ( km(iseqkm)%intprop(nintprop,nelgrp) )
            intprop => km(iseqkm)%intprop
            call incopy ( iinput(iinp12), intprop, nintprop*nelgrp )
            kmeshw(3) = nintprop*nelgrp

         end if

         if ( nrealprop>0 ) then

!        --- real properties found, create array realprop

            allocate ( km(iseqkm)%realprop(nrealprop,nelgrp) )
            realprop => km(iseqkm)%realprop

            if ( debug ) then
               write(irefwr,1) 'irnp5', irnp5
               call prinrl ( rinput(irnp5), nrealprop*nelgrp, 'realprop' )
            end if  ! ( debug )
            call dcopy ( nrealprop*nelgrp, rinput(irnp5), 1, realprop, 1 )
            kmeshw(4) = nrealprop*nelgrp

         end if

      else

!     --- Create fake kmeshw

         allocate ( km(iseqkm)%kmeshw(4) )
         kmeshw => km(iseqkm)%kmeshw
         kmeshw = 0

         nrealprop = 0
         nintprop = 0
         allocate ( km(iseqkm)%intprop(1,nelgrp) )
         intprop => km(iseqkm)%intprop
         allocate ( km(iseqkm)%realprop(1,nelgrp) )
         realprop => km(iseqkm)%realprop
         kmeshw(3) = nelgrp
         kmeshw(4) = nelgrp
         intprop = 0
         realprop = 0d0

      end if
      kelmw = 4

      if ( nlines>0 ) then

!     --- nlines>0  restore  kmesh part f

         do ielgrp = 1, nelgrp
            ishape = kmeshf(ielgrp)
            if ( ishape<-10 .and. ishape>-10001 ) then
               kmeshf(ielgrp) = -ishape-10
            end if
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         if ( kelma>0 ) call prinin ( kmesha, nelgrp+1, 'kmesha' )
         call prinin ( kmeshf, nelgrp, 'kmeshf' )
         write(irefwr,*) 'End msh001'

      end if
      call erclos ( 'msh001' )

      end subroutine msh001
