      subroutine msh007 ( ndim, curves, iinput, rinput, n, nuspnt, nelm, &
                          ishape, icurve )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 22-02-1999 Include commons
!        version  3.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  3.0  date 05-05-1987
!
!   copyright (c) 1982-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   fill part of array curves   user defined curves
!   n positions are filled
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, n, nuspnt, nelm, ishape, icurve, iinput(*)
      double precision curves(*), rinput(*)

!     curves     o    array to be filled from position 1
!     icurve     i    curve number
!     iinput     i    integer input array
!                     the first two positions of this array contain the point
!                     numbers of the initial and end points
!     ishape     i    number of nodal points in the elements along the curve
!                     minus one
!     n          i    number of points on the line
!     ndim       i    Dimension of the space.
!     nelm       i    number of elements along a curve
!     nuspnt     i    Number of user points
!     rinput     i    Real input array containing the coordinates of the points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, ih, iprec
      double precision dtau, dx, dy, dz, x1, x2,  y1, y2, z1, z2

!     dtau
!     dx         length of vector in x-direction
!     dy         length of vector in y-direction
!     dz         length of vector in z-direction
!     i          Counting variable
!     i1         First point of curve in KMESH part m
!     i2         Last point of curve in KMESH part m
!     ih         Help variable to store a constant temporarily
!     iprec      Curve number of previous curve
!     x1         x-coordinate of first point in curve
!     x2         x-coordinate of last point in curve
!     y1         y-coordinate of first point in curve
!     y2         y-coordinate of last point in curve
!     z1         z-coordinate of first point in curve
!     z2         z-coordinate of last point in curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     127   user point number out of range
!     390   coinciding points in curve
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh007' )

      n = ishape*nelm+1
      i1 = iinput(1)
      if ( i1.le.0 .or. i1.gt.nuspnt ) then
         call errint ( i1, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 127, 2, 0, 0 )
      end if
      iprec = i1
      if ( ierror.ne.0 ) go to 1000
      goto ( 100, 200, 300 ), ndim

!     --- ndim = 1

100    x1 = rinput(i1)
       curves(1) = x1
       do i = 1, nelm
          i2 = iinput(i+1)
          if ( i2.le.0 .or. i2.gt.nuspnt ) then
             call errint ( i2, 1 )
             call errint ( nuspnt, 2 )
             call errsub ( 127, 2, 0, 0 )
          end if
          if ( i2.eq.iprec ) then
             call errint ( i2, 1 )
             call errint ( icurve, 2 )
             call errsub ( 390, 2, 0, 0 )
          end if
          x2 = rinput(i2)
          ih = i*ishape
          curves(1+ih) = x2
          if ( ishape.eq.2 ) then
             curves(ih) = .5d0*(x1+x2)
          else if ( ishape.eq.3 ) then
             dtau = (x2-x1)/3d0
             curves(ih-1) = x1+dtau
             curves(ih) = x1+dtau*2d0
          end if
          iprec = i2
          x1 = x2
      end do
      go to 1000

!     --- ndim = 2

200    x1 = rinput(2*i1-1)
       y1 = rinput(2*i1)
       curves(1) = x1
       curves(2) = y1
       do i = 1, nelm
          i2 = iinput(i+1)
          if ( i2.le.0 .or. i2.gt.nuspnt ) then
             call errint ( i2, 1 )
             call errint ( nuspnt, 2 )
             call errsub ( 127, 2, 0, 0 )
          end if
          if ( i2.eq.iprec ) then
             call errint ( i2, 1 )
             call errint ( icurve, 2 )
             call errsub ( 390, 2, 0, 0 )
          end if
          x2 = rinput(2*i2-1)
          y2 = rinput(2*i2)
          ih = i*ishape
          curves(1+2*ih) = x2
          curves(2+2*ih) = y2
          iprec = i2
          if ( ishape.eq.2 ) then
             curves(2*ih-1) = .5d0*(x1+x2)
             curves(2*ih) = .5d0*(y1+y2)
          else if ( ishape.eq.3 ) then
             dx = (x2-x1)/3d0
             dy = (y2-y1)/3d0
             curves(2*ih-3) = x1+dx
             curves(2*ih-2) = y1+dy
             curves(2*ih-1) = x1+dx*2d0
             curves(2*ih) = y1+dy*2d0
          end if
          x1 = x2
          y1 = y2
      end do
      go to 1000

!     --- ndim = 3

300    x1 = rinput(3*i1-2)
       y1 = rinput(3*i1-1)
       z1 = rinput(3*i1)
       curves(1) = x1
       curves(2) = y1
       curves(3) = z1
       do i = 1, nelm
          i2 = iinput(i+1)
          if ( i2.le.0 .or. i2.gt.nuspnt ) then
             call errint ( i2, 1 )
             call errint ( nuspnt, 2 )
             call errsub ( 127, 2, 0, 0 )
          end if
          if ( i2.eq.iprec ) then
             call errint ( i2, 1 )
             call errint ( icurve, 2 )
             call errsub ( 390, 2, 0, 0 )
          end if
          iprec = i2
          x2 = rinput(3*i2-2)
          y2 = rinput(3*i2-1)
          z2 = rinput(3*i2)
          ih = i*ishape
          curves(1+3*ih) = x2
          curves(2+3*ih) = y2
          curves(3+3*ih) = z2
          if ( ishape.eq.2 ) then
             curves(3*ih-2) = .5d0*(x1+x2)
             curves(3*ih-1) = .5d0*(y1+y2)
             curves(3*ih) = .5d0*(z1+z2)
          else if ( ishape.eq.3 ) then
             dx = (x2-x1)/3d0
             dy = (y2-y1)/3d0
             dz = (z2-z1)/3d0
             curves(3*ih-5) = x1+dx
             curves(3*ih-4) = y1+dy
             curves(3*ih-3) = z1+dz
             curves(3*ih-2) = x1+dx*2d0
             curves(3*ih-1) = y1+dy*2d0
             curves(3*ih) = z1+dz*2d0
          end if

          x1 = x2
          y1 = y2
          z1 = z2
      end do
      go to 1000

1000  call erclos ( 'msh007' )
      end
