      subroutine msh010 ( mapar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 15-11-2012 Long integers
!        version  2.2  date 20-10-2008 Allow isurf = 24
!        version  2.1  date 18-04-2006 Debug statements
!
!   copyright (c) 1996-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill array mapar for surfaces that not yet have been treated before
!   This subroutine is only used if volume elements are present
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mapping_array
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mapar(*)

!     mapar          o    mapping array: contains the mapping from local to
!                         global numbering of the nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, lenkmsc, istart, npointlc, isurf, &
              ivolnr

!     i              Counting variable
!     istart         Starting position of surface in kmeshn
!     isurf          Type of surface with respect to mesh generator
!     isurnr         Surface sequence number
!     ivolnr         Volume sequence number
!     lenkmsc        Length of array kmeshc
!     npointlc       Local number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For isurnr := 1 (1) nsurfs do
!       if ( kmeshh(5,isurnr+1)<0 ) then
!          Read "kmeshc" into iwork
!          istart := starting address in kmeshn
!          npointlc := number of points in surface
!          for i := 1 (1) length of iwork do
!             mapar(iwork(i)) := kmeshn(istart+i-1)
!          write mapar to file 3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh010' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh010'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nvolms>0 ) then

!     --- nvolms>0, standard situation

         do isurnr = 1, nsurfs

            isurf = kmeshs(nsurfs+2+(isurnr-1)*8)
            if ( isurf==24 ) isurf = 7    ! compound surface
            if ( kmeshh(4,isurnr+1)<0 .and. isurf/=7 .and. &
                 isurf/=11 .and. isurf/=12 ) then

!           --- Read "kmeshc" into iwork

               lenkmsc = size(kmhsur(isurnr)%kmeshcsur)
               npointlc = kmeshh(1,isurnr+1)

               istart = 2*nsurfs
               do i = 1, isurnr-1
                  istart = istart + kmeshn(i)*kmeshn(i+nsurfs)
               end do

               do i = 1, lenkmsc
                  mapar(kmhsur(isurnr)%kmeshcsur) = &
                     kmeshn(istart+1:istart+lenkmsc)
               end do

!              --- write mapar to file 3

               kmeshh(4,isurnr+1) = abs(kmeshh(4,isurnr+1))
               kmhsur(isurnr)%maparsur(1:npointlc) = mapar(1:npointlc)

            end if

         end do

      else

!     --- nvolms = 0, special situation

         do isurnr = 1, nsurfs

            isurf = kmeshs(nsurfs+2+(isurnr-1)*8)
            if ( isurf==24 ) isurf = 7    ! compound surface
            if ( kmeshh(4,isurnr+1)<0 .and. isurf/=7 .and. &
                 isurf/=11 .and. isurf/=12 ) then

!           --- Read "kmeshc" into iwork

               npointlc = kmeshh(1,isurnr+1)

!              --- write mapar to file 3

               kmeshh(4,isurnr+1) = abs(kmeshh(4,isurnr+1))
               kmhsur(isurnr)%maparsur(1:npointlc) = mapar(1:npointlc)

            end if

         end do

      end if

!     --- Now consider the volumes that have not been treated yet

      do ivolnr = 1, nvolms

         isurnr = ivolnr+nsurfs
         if ( debug ) then
            write(irefwr,1) 'ivolnr, isurnr', ivolnr, isurnr
         end if  ! ( debug )

         if ( kmeshh(4,isurnr+1)<0 ) then

!        --- Read "kmeshc" into iwork

            npointlc = kmeshh(1,isurnr+1)

!           --- write mapar to file 3

            kmeshh(4,isurnr+1) = abs(kmeshh(4,isurnr+1))
            kmhsur(isurnr)%maparsur(1:npointlc) = mapar(1:npointlc)

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh010'

      end if  ! ( debug )
      call erclos ( 'msh010' )

      end subroutine msh010
