      subroutine msh016 ( iinput, ikelmc, new, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version 12.0  date 02-09-2010 Remove intarmsh
!        version 11.0  date 02-02-1997 Extra parameter intarmsh
!        version 10.5  date 22-03-1996 Allow empty element groups
!
!   copyright (c) 1982-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute the line elements
!    store in arrays coor, kmeshc and kmesh part g
!    the arrays kmesh part l and part m are partly filled
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     line_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical new
      integer iinput(*), ikelmc
      double precision :: rinput(ndim,*)

!     iinput     i    integer input array
!     ikelmc    i/o   last position in kmesh part c
!     new        i    indication whether the mesh is new (true) or not (false)
!     rinput     i    input array rinput
!                     the first position must contain the first co-ordinate
!                     of the first user point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, istar3, iline, ielgrp, ifirst, &
              ilast, j, nodalp, ih1, ih, nsubcv, ic, jcur, kstep, k1, &
              k2, k, jknpt, ih2, iw, lastp, jh, ifirp, nh, i, nsub, &
              nprev, icur, n, l
      integer, allocatable :: iwork(:)

!     i          Counting variable
!     ic         Relative subcurve number
!     icur       Curve sequence number
!     ielgrp     element group number
!     ifirp
!     ifirst     First curve (surface) number read
!     ih         Help variable to store a constant temporarily
!     ih1        Help variable to store a constant temporarily
!     ih2        Help variable to store a constant temporarily
!     iinp       last position used in array iinput
!     ilast      Last curve (surface) number read
!     iline      number of lines
!     istar3     Starting address of kmeshm part 3 in KMESH part M
!     iw         Address in array IWORK
!     iwork      Work space
!     j          Counting variable
!     jcur       Curve sequence number
!     jh
!     jknpt      Nodal point number
!     k          Counting variable
!     k1         First point of curve in KMESH part m
!     k2         Last point of curve in KMESH part m
!                KMESH part t in KMESH
!     kstep      Step in loop
!     l          General loop variable
!     lastp
!     n          Parameter for mesh generation
!     nh         number of nodes along a subcurve
!     nodalp     Nodal point sequence number
!     nprev      Number of elements at preceding curves
!     nsub       Number of elements at a curve accumulated
!     nsubcv     Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     159   number of nodal points along the curves not correct for the
!           creation of line elements in boundary element group
!     421   curves are not subsequent curves, whereas one element must be
!           created along these curves
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh016' )

      allocate ( iwork(lencrv), stat = error )
      if ( error/=0 ) call eralloc ( error, lencrv, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iinp=iinp2
      inpelm = inpelm
      istar3=kmeshm(2)-1
      inodp=1
      ikelmc=0
      iselem=1
      do iline = 1, nlines
         ielgrp=iinput(iinp+1)
         ifirst=iinput(iinp+2)
         ilast=iinput(iinp+3)
         iinp=iinp+4
         if ( kelma/=0 ) inpelm=kmesha(ielgrp+1)- kmesha(ielgrp)
         if ( inpelm==1 ) then

!        --- inpelm = 1  point elements

            n=ilast-ifirst+1
            if ( new .and. nelgrp>1 ) kmeshg(ielgrp) = kmeshg(ielgrp)+n
            do j=ifirst, ilast
               nodalp=kmeshl(j)
               if ( nodalp==0 ) then
                  kmeshl(j)=inodp
                  nodalp=inodp
                  inodp=inodp+1
               end if
               coor(:,nodalp)=rinput(:,j)
               if ( new ) then
                  ikelmc=ikelmc+1
                  kmeshc(ikelmc)=nodalp
               end if
            end do
         else

!        --- fill array kmesh parts l and m
!            and array coor

            do j=ifirst, ilast
               if ( new .or. nsurvl==0 ) then

!              --- new = true  new mesh

                  if ( kelmq==0 ) then
                     nsubcv = 0
                  else
                     nsubcv = kmeshq(j+1) - kmeshq(j)
                  end if

!                 --- Loop over subcurves

                  do ic = 1, max(1, nsubcv)
                     if ( nsubcv==0 ) then
                        jcur = j
                     else
                        jcur = kmeshq( kmeshq(j)+ic )
                     end if

!                    --- Curve jcur

                     if ( jcur>0 ) then
                        kstep = 1
                        k1 = kmeshm(4+jcur)
                        k2 = kmeshm(5+jcur)-1
                     else
                        kstep = -1
                        k1 = kmeshm(5-jcur)-1
                        k2 = kmeshm(4-jcur)
                     end if
                     do 140 k = k1, k2, kstep
                        nodalp=kmeshm(istar3+k)
                        if ( nodalp>0 ) go to 140
                        if ( nodalp<0 ) then

!                       --- nodalp<0

                           nodalp=-nodalp
                           jknpt=kmeshl(nodalp)
                           if ( jknpt==0 ) then

!                          --- jknpt = 0  new point

                              kmeshl(nodalp)=inodp
                           else

!                          --- jknpt>0  old point

                              kmeshm(istar3+k)=jknpt
                              go to 140
                           end if
                        end if

!                       --- nodalp = 0  new point

                        kmeshm(istar3+k)=inodp
                        coor(:,inodp)=curvesmsh(:,k)
                        inodp=inodp+1
140                  continue
                  end do
               else

!              --- new = false    old mesh, change coordinates

                  do k = kmeshm(j+4), kmeshm(j+5)-1
                     nodalp=kmeshm(istar3+k)
                     inodp = max ( inodp, nodalp )
                     coor(:,nodalp)=curvesmsh(:,k)
                  end do
                  inodp = inodp + 1
               end if
            end do
            if ( new ) then

!           --- fill kmesh part c  with elements
!               first array iwork is filled

               iw=1
               n=0
               nsub=0
               nprev=0
               do j=ifirst, ilast
                  if ( kelmq/=0 ) nsubcv = kmeshq(j+1) - kmeshq(j)

!                 --- Loop over subcurves

                  do ic = 1, max(1, nsubcv)
                     if ( nsubcv==0 ) then
                        jcur = j
                     else
                        jcur = kmeshq( kmeshq(j)+ic )
                     end if
                     icur = jcur

!                    --- Curve jcur

                     k1 = kmeshm(4+abs(jcur))
                     k2 = kmeshm(5+abs(jcur))-1
                     if ( jcur>0 ) then
                        do k = k1, k2
                           iwork(iw+k-k1) = kmeshm(istar3+k)
                        end do
                     else
                        do k = k1, k2
                           iwork(iw+k-k1) = kmeshm(istar3+k2+k1-k)
                        end do
                     end if
                     iw=iw+k2-k1
                  end do

!                 --- Check if end of series curves is found by finding start
!                     position of next curve when j # ilast and comparing with
!                     end position of present curve

                  lastp = iwork(iw)
                  if ( j/=ilast ) then

!                 --- j # ilast, find next point

                     jh = j+1
                     if ( kelmq/=0 ) nsubcv=kmeshq(jh+1)-kmeshq(jh)
                     if ( nsubcv==0 ) then
                        jcur = jh
                     else
                        jcur = kmeshq( kmeshq(jh) + 1 )
                     end if

!                    --- Curve jcur

                     if ( jcur>0 ) then
                        ifirp = kmeshm(istar3+kmeshm(4+jcur))
                     else
                        ifirp = kmeshm(istar3+kmeshm(5-jcur)-1)
                     end if
                  end if
                  if ( j==ilast .or. ifirp/=lastp ) then

!                 --- end of series curves found because either j=ilast
!                     or next curve starts with point different from endpoint
!                     of curve

                     iw=iw-1
                     if ( inpelm==0 ) then

!                    --- inpelm = 0  one element only

                        if ( j/=ilast ) then
                           nh=0
                           call errint ( ielgrp, 1 )
                           call errint ( j, 2 )
                           call errint ( ilast, 3 )
                           call errsub ( 421, 3, 0, 0 )
                        else
                           if ( iwork(1)==iwork(iw+1) ) then
                              inpelm=iw
                           else
                              inpelm=iw+1
                           end if
                           nh=1
                           kelgrp=kelgrp+inpelm
                           npelm=max(inpelm, npelm)
                           do k=ielgrp, nelgrp
                              kmesha(k+1)=kmesha(k+1)+inpelm
                           end do
                        end if
                     else

!                    --- inpelm>0

                        nh=iw/(inpelm-1)
                        if ( iw/=nh*(inpelm-1) ) then
                           call errint ( ielgrp, 1 )
                           call errint ( iw, 2 )
                           call errint ( nh*(inpelm-1), 3 )
                           call errint ( 1, 4 )
                           call errsub ( 159, 4, 0, 0 )
                        end if
                     end if
                     ih=0
                     do i=1, nh
                        do k=1, inpelm
                           kmeshc(ikelmc+k)=iwork(ih+k)
                        end do
                        ikelmc=ikelmc+inpelm
                        ih=ih+inpelm-1
                     end do
                     n=n+nh
                     iw=1
                  end if
                  if ( nsurfs+nvolms==0 ) then

!                 --- nsurfs = 0, and nvolms = 0, hence the mesh is restricted
!                     to curves
!                     Fill array KMESH part t

                     if ( inpelm==0 ) then
                        nsub = 1
                     else
                        nsub = (k2-k1)/(inpelm-1)
                     end if
                     kmesht(1,abs(icur)) = iselem+nprev
                     kmesht(2,abs(icur)) = iselem+nprev+nsub-1
                     nprev = nprev + nsub
                  end if
               end do
               if ( new .and. nelgrp>1 ) kmeshg(ielgrp) =  kmeshg(ielgrp)+n
            end if
         end if
         iselem=iselem+n
      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'msh016' )
      end subroutine msh016
