      subroutine msh030gen ( rinput, j1, j2, x1, x2, ndim, edge, &
                             edgemn, rincor, nuspnt, icurve, icurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 10-02-2015 New parameter list
!        version  1.2  date 08-12-2014 Use msh030gen1
!        version  1.1  date 30-08-2013 Extra debug
!
!   copyright (c) 2012-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine number of elements to be created along
!     general part for some types of curves
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer j1, j2, ndim, nuspnt, icurve, icurnr
      double precision rincor(*), rinput(ndim,nuspnt), x1(ndim), &
                       x2(ndim), edge, &
                       edgemn

!     edge           i    Unit length defined for coarseness
!     edgemn         o    Mean length of edges
!     icurnr         i    Curve sequence number
!     icurve         i    Type of curve
!     j1             i    Index of first point
!     j2             i    Index of second point
!     ndim           i    Dimension of the space.
!     nuspnt         i    Number of user points
!     rincor         i    Part of array rinput refering to coarseness
!     rinput         i    array containing on the first nuspnt*ndim positions
!                         coordinates , on position nuspnt+1 the unit edge
!                         length and on the next nuspnt positions the defined
!                         coarseness of the user points
!     x1             o    coordinates of first point
!     x2             o    coordinates of second point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: edge1, edge2

!     edge1          Length of first edge
!     edge2          Length of last edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH030GEN1     Compute edge1/2 and edgemn for one line segment
!                    corresponding to cline
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030gen' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030gen'
         write(irefwr,1) 'icurnr, icurve', icurnr, icurve
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill x1, x2 and compute edge1, edge2 and edgemn

      call msh030gen1 ( j1, j2, nuspnt, rinput, ndim, rincor, &
                        edge, edge1, edge2, edgemn, icurnr, &
                        x1, x2 )

1000  call erclos ( 'msh030gen' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh030gen'

      end if  ! ( debug )

      end
