      subroutine msh030gen1 ( j1, j2, nuspnt, rinput, ndim, rincor, &
                              edge, edge1, edge2, edgemn, icurnr, &
                              x1, x2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute edge1/2 and edgemn for one line segment corresponding to cline
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: j1, j2, nuspnt, ndim, icurnr
      double precision, intent(in) :: rinput(ndim,*), rincor(*), edge
      double precision, intent(out) :: edge1, edge2, edgemn, x1(ndim), &
                                       x2(ndim)

!     edge           i    Unit length defined for coarseness
!     edge1          o    Length of first edge
!     edge2          o    Length of last edge
!     edgemn         o    Mean length of edges
!     icurnr         i    Curve sequence number
!     j1             i    Index of first point
!     j2             i    Index of second point
!     ndim           i    Dimension of the space.
!     nuspnt         i    Number of user points
!     rincor         i    Part of array rinput refering to coarseness
!     rinput         i    array containing on the first nuspnt*ndim positions
!                         coordinates , on position nuspnt+1 the unit edge
!                         length and on the next nuspnt positions the defined
!                         coarseness of the user points
!     x1             o    coordinates of first point
!     x2             o    coordinates of second point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: coars1, coars2

!     coars1         Coarseness in first point
!     coars2         Coarseness in second point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1783   User point number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030gen1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030gen1'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- test user points

      if ( j1<=0 .or. j1>nuspnt ) then
         call errint ( j1, 1 )
         call errint ( nuspnt, 2 )
         call errint ( icurnr, 3 )
         call errsub ( 1783, 3, 0, 0 )
         go to 1000
      end if
      if ( j2<=0 .or. j2>nuspnt ) then
         call errint ( j2, 1 )
         call errint ( nuspnt, 2 )
         call errint ( icurnr, 3 )
         call errsub ( 1783, 3, 0, 0 )
         go to 1000
      end if
      if ( debug ) write(irefwr,1) 'j1, j2', j1, j2

      x1 = rinput(1:ndim,j1)
      x2 = rinput(1:ndim,j2)
      if ( debug ) write(irefwr,2) 'x1, x2', x1, x2

      coars1 = rincor(j1+1)
      coars2 = rincor(j2+1)

      edge1 = coars1*edge
      edge2 = coars2*edge
      edgemn = 0.5d0*(edge1+edge2)

1000  call erclos ( 'msh030gen1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'coars1, coars2', coars1, coars2
         write(irefwr,*) 'End msh030gen1'

      end if  ! ( debug )

      end subroutine msh030gen1
