      subroutine msh033 ( kmeshm, kmeshs, ncurvs, nsurfs, &
                          ileft, iwork, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-10-2008 Allow isurf = 24
!        version  1.0  date 20-10-1993
!
!   copyright (c) 1993-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    This subroutine sets positions in iwork corresponding to the outer
!    boundary of surface ileft equal to -1
! **********************************************************************
!
!                       KEYWORDS
!
!     connect
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm(*), kmeshs(*), ileft, nsurfs, &
              ncurvs, npoint, iwork(npoint+ncurvs)

!     ileft   i    Surface number of "left" surface to be connected
!     iwork  i/o   Work array of length npoint+ncurvs
!                  At input the array is equal to zero
!                  At output iwork(i) = -1, if point i at the outer boundary
!     kmeshm  i    KMESH part m, contains nodal point numbers of the curves
!     kmeshs  i    Kmesh part s, contains information about the various
!                  surfaces. See the PG 13.1
!     ncurvs  i    Number of curves in mesh
!     npoint  i    Number of nodal points
!     nsurfs  i    Number of surfaces in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, isurf, noutcv, icrvn, istar3, k

!     i       Counting variable
!     icrvn   Curve number
!     istar3  Starting address of KMESHM part 3 in KMESH part M
!     istart  Starting address in array KMESHS
!     isurf   Type of surface with respect to mesh generator
!     k       Counting variable
!     noutcv  Number of outer curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- First the curve numbers of the outer boundary are stored in
!         iwork

      istart = nsurfs+1+8*(ileft-1)
      isurf = kmeshs(istart+1)
      if ( isurf==24 ) isurf = 7    ! compound surface
      if ( isurf==7 .or. isurf==11 .or. isurf==12 ) then

!     --- surface consists of subsurfaces

         noutcv = kmeshs(istart+3)
         do 100 i = 1, noutcv
            iwork(npoint+i) = kmeshs(kmeshs(ileft)+i)
100      continue
      else

!     --- standard surface
!         Set iwork(i) equal to 1, if i is an outer
!         curve and to 2 if it is a double and hence inner curve

         do 110 i = 1, kmeshs(ileft+1)-kmeshs(ileft)
            icrvn = abs(kmeshs(kmeshs(ileft)+i))
            iwork(icrvn) = iwork(icrvn) + 1
110      continue

!        --- Fill iwork positions 1 to noutcv with curve numbers
!            Compute noutcv

         noutcv = 0
         do 120 i = 1, ncurvs
            if ( iwork(i)==1 ) then

!           --- Outer curve found

               noutcv = noutcv + 1
               iwork(noutcv) = i
            end if
120      continue

!        --- move curve numbers to final positions

         do 130 i = noutcv, 1, -1
            iwork(npoint+i) = iwork(i)
130      continue

!        --- Clear first part of iwork

         do 140 i = 1, min(ncurvs,npoint)
            iwork(i) = 0
140      continue
      end if

!     --- Set iwork(ipoint) for outer curves

      istar3 = kmeshm(2) - 1
      do 200 i = 1, noutcv
         icrvn = iwork(npoint+i)
         do 180 k = kmeshm(4+icrvn), kmeshm(5+icrvn)-1
            iwork(kmeshm(istar3+k)) = -1
180      continue
200   continue
      end
