      subroutine msh035 ( kbndpt, mapar, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.0  date 02-09-2010 Remove intarmsh
!        version  7.2  date 18-08-2008 Fortran 90
!        version  7.1  date 26-03-2004 Correction negative isurnr
!
!   copyright (c) 1985-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array mapar and the arrays kmeshn, kmeshm, kmeshl for
!     the surfaces corresponding to the volume
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), kbndpt(*), mapar(*)

!     iinput         i    array iinput containing the user input
!                         with respect to the volume
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary
!     mapar          o    mapping array: contains the mapping from local to
!                         global numbering of the nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, ndefsf, kstart, nposrf, nsubsf, istmsr, jstmsr, &
              j, knodp, mstart, isubsf, npoloc, &
              inplms, nelloc, index, k, ielem, nstart, ipos, iposm, &
              jsurnr
      integer, allocatable :: iwork(:), iwork1(:), ipoint(:)

!     i              Counting variable
!     ielem          Element number
!     index
!     inplms         Number of points in surface element
!     ipoint         Work array of length npomxs+nelmax to store the local
!                    mapping from subsurfaces to surfaces
!     ipos
!     iposm          ?
!     istart         First position used at a part
!     istmsr         ?
!     isubsf         ?
!     iwork          work array of length nsurfs
!     iwork1         work array of length nsurfs
!     j              Counting variable
!     jstmsr         ?
!     jsurnr         Surface sequence number
!     k              Counting variable
!     knodp          Nodal point number
!     kstart         Start value of parameter
!     mstart         ?
!     ndefsf         Number of curves in surface
!     nelloc         Local number of elements
!     npoloc         Local number of nodal points
!     nposrf         ?
!     nstart
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inidln

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIDLN         Compute length of array in BUFFER
!     MSH035SUB      fill array mapar and the arrays kmeshn, kmeshm, kmeshl for
!                    one node in surface corresponding to volume
!     MSH036         fill array mapar position iknpt with jknpt
!     MSH044         fill negative values in kmeshm if possible
!     MSH057         fill array mapar and the arrays kmeshn, kmeshm, kmeshl
!                    for a volume
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh035' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh035'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,1) 'ivolnr, nsurfs, npoint', &
                                    ivolnr, nsurfs, npoint

      allocate ( iwork(nsurfs), iwork1(nsurfs), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nsurfs, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- clear arrays iwork and mapar

      mapar(1:npoint) = 0
      iwork(1:nsurfs) = 0

!     --- fill starting addresses minus one of the surfaces in kmeshn in iwork1

      istart = 2*nsurfs
      do i = 1, nsurfs
         iwork1(i) = istart
         istart = istart+kmeshn(i)*kmeshn(i+nsurfs)
      end do

      ndefsf = iinput(3)
      kstart = 0

      do i = 1, ndefsf

         isurnr = mod(iinput(3+i)-1,nsurfs)+1
         istart = iwork1(isurnr)
         nposrf = kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
         if ( kelmr==0 ) then

!        --- kelmr = 0, hence there are no surfaces consisting of subsurfaces

            nsubsf = 0

         else

!        --- kelmr>0, hence there are surfaces consisting of subsurfaces

            istmsr = kmeshr(isurnr)
            nsubsf = kmeshr(isurnr+1) - istmsr
            jstmsr = kmeshr(nsurfs+1) - nsurfs - 1 + istmsr

         end if
         if ( debug ) then

!        --- debug = true, debug prints

            write(irefwr,1) 'i, isurnr, nsubsf', i, isurnr, nsubsf
            write(irefwr,1) 'iwork(isurnr), nposrf', &
                             iwork(isurnr), nposrf

         end if

         if ( nsubsf==0 ) then

!        --- nsubsf = 0, one surface

            if ( iwork(isurnr)==0 ) then

!           --- surface has not been used yet

               iwork(isurnr) = i

               do j = 1, nposrf

                  call  msh035sub ( j, kmeshl, kmeshm, kmeshn, kbndpt, &
                                    mapar, npoint, istart, kstart, &
                                    isurnr, ivolnr )
                  if ( ierror/=0 ) go to 1000

               end do  ! j = 1, nposrf

            else

!           --- surface has been used already,
!               fill negative or positive values in mapar

              mstart = iwork1(isurnr)
              do j = 1, nposrf
                 knodp = kbndpt(kstart+j)
                 jnodp = kmeshn(mstart+j)
                 if (  jnodp>0 ) then

!                --- jnodp>0, kmeshn has been filled

                    if ( mapar(knodp)<-npoint ) &
                           kmeshn( -mapar(knodp)-npoint ) = jnodp
                    call msh036 ( mapar, knodp, jnodp, npoint, &
                                  isurnr, ivolnr )
                    if ( ierror/=0 ) go to 1000

                 else

!                --- jnodp<0,  kmeshn has not yet been filled for this
!                                surface, which means that we have a double
!                                surface
!                                Fill mapar if necessary

                       if ( mapar(knodp)==0 ) mapar(knodp)  = &
                                - npoint - mstart - j
                 end if

              end do

            end if

         else

!        --- nsubsf>0, surface of subsurfaces

            isurf = kmeshs2(1,isurnr)
            do isubsf = 1, nsubsf

!           --- Read local mapping array into ipoint

               isurnr = kmeshr(istmsr+isubsf)
               jsurnr = abs(isurnr)
               npoloc = kmeshh(1,jsurnr+1)
               inplms = kmeshn(jsurnr)
               nelloc = kmeshn(jsurnr+nsurfs)

               allocate ( ipoint(npoloc+nelloc), stat = error )
               if ( error/=0 ) call eralloc ( error, npoloc+nelloc, 'ipoint' )
               if ( ierror/=0 ) go to 1000

               index = kmeshr(jstmsr+isubsf)
               if ( debug ) then

!              --- debug = true, debug prints

                  write(irefwr,1) &
                     'isubsf, isurnr, npoloc, nelloc, isurf', &
                      isubsf, isurnr, npoloc, nelloc, isurf

               end if

               call readfl ( index, 1, npoloc+nelloc, 1, 3, ipoint )

               istart = iwork1(jsurnr)
               if ( debug ) then
                  write(irefwr,1) 'jsurnr, iwork(jsurnr)', &
                                   jsurnr, iwork(jsurnr)
                  write(irefwr,1) 'istart', istart
                  call prinin ( ipoint, npoloc, 'ipoint_nodes' )
                  call prinin ( ipoint(npoloc+1), nelloc, 'ipoint_els' )
                  call prinin1 ( kmeshn(istart+1), nelloc, inplms, &
                                 'kmeshn/sub' )
               end if  ! ( debug )

               if ( iwork(jsurnr)==0 ) then

!              --- surface has not been used yet

                  iwork(jsurnr) = i
                  do k = 1, nelloc
                     ielem = ipoint(npoloc+k)
                     nstart = (ielem-1)*inplms + kstart
                     if ( debug ) then

!                    --- debug = true, debug prints

                        write(irefwr,1) 'k, ielem', k, ielem

                     end if
                     do j = 1, inplms

                        call msh035sub ( j, kmeshl, kmeshm, kmeshn, &
                                         kbndpt, mapar, npoint, istart, &
                                         nstart, isurnr, ivolnr )
                        if ( ierror/=0 ) go to 1000

                     end do
                     istart = istart + inplms

                  end do

               else

!              --- surface has been used already,
!                  fill negative or positive values in mapar

                  mstart = iwork1(jsurnr)
                  do k = 1, nelloc
                    ielem = ipoint(npoloc+k)
                    nstart = (ielem-1)*inplms + kstart
                    do j = 1, inplms
                       knodp = kbndpt(nstart+j)
                       jnodp = kmeshn(mstart+j)
                       if ( jnodp>0 ) then

!                      --- jnodp>0  Surface has been filled

                          if ( mapar(knodp)<-npoint ) &
                               kmeshn( -mapar(knodp)-npoint ) = jnodp
                          call msh036 ( mapar, knodp, jnodp, npoint, &
                                        isurnr, ivolnr )
                          if ( ierror/=0 ) go to 1000

                       else

!                      --- jnodp<=  0,  kmeshn has not yet been filled for
!                                         this surface, which means that we
!                                         have a double surface
!                                         Fill mapar if necessary

                          if ( mapar(knodp)==0 ) mapar(knodp)  = &
                               -npoint - mstart - j
                       end if

                    end do
                    mstart = mstart + inplms

                 end do

               end if

               deallocate ( ipoint, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'ipoint' )
               if ( ierror/=0 ) go to 1000

            end do  ! isubsf = 1, nsubsf

         end if  ! ( nsubsf==0 )
         kstart = kstart+nposrf

      end do  ! i = 1, ndefsf

!     --- fill array mapar

      call msh057 ( mapar, kmeshn, kmeshm, kmeshl, npoint, inodp )
      if ( ierror/=0 ) go to 1000

!     --- fill surfaces in kmeshn

      kstart = 0
      do i = 1, ndefsf
         isurnr = mod(iinput(3+i)-1,nsurfs)+1
         if ( debug ) write(irefwr,1) 'i,isurnr', i, isurnr
         istart = iwork1(isurnr)
         nposrf = kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
         if ( kelmr==0 ) then

!        --- kelmr = 0, hence there are no surfaces consisting of subsurfaces

            nsubsf = 0

         else

!        --- kelmr>0, hence there are surfaces consisting of subsurfaces

            istmsr = kmeshr(isurnr)
            nsubsf = kmeshr(isurnr+1) - istmsr
            jstmsr = kmeshr(nsurfs+1) - nsurfs - 1 + istmsr

         end if

         if ( nsubsf==0 ) then

!        --- nsubsf = 0, one surface

            do j = 1, nposrf
               knodp = mapar(kbndpt(kstart+j))
               ipos = kmeshn(istart+j)
               kmeshn(istart+j) = knodp
               if ( ipos<-1000000000 ) then

!              --- ipos<-1000000000, refers to user point

                  iposm = -ipos-1000000000
                  kmeshl(iposm) = knodp

               else if ( ipos<0 ) then

                  iposm = kmeshm(-ipos)
                  kmeshm(-ipos) = knodp
                  if ( iposm<0 ) then
                     kmeshl(-iposm) = knodp
                  end if

               end if

            end do

         else

!        --- nsubsf>0, surface of subsurfaces

            do isubsf = 1, nsubsf

!           --- Read local mapping array into ipoint

               isurnr = kmeshr(istmsr+isubsf)
               jsurnr = abs(isurnr)
               inplms = kmeshn(jsurnr)
               nelloc = kmeshn(jsurnr+nsurfs)
               npoloc = kmeshh(1,jsurnr+1)

               allocate ( ipoint(npoloc+nelloc), stat = error )
               if ( error/=0 ) call eralloc ( error, npoloc+nelloc, 'ipoint' )
               if ( ierror/=0 ) go to 1000

               index = kmeshr(jstmsr+isubsf)
               call readfl ( index, 1, npoloc+nelloc, 1, 3, ipoint )
               istart = iwork1(jsurnr)
               do k = 1, nelloc
                  ielem = ipoint(npoloc+k)
                  nstart = (ielem-1)*inplms + kstart
                  do j = 1, inplms
                     knodp = mapar(kbndpt(nstart+j))
                     ipos = kmeshn(istart+j)
                     kmeshn(istart+j) = knodp
                     if ( ipos<-1000000000 ) then

!                    --- ipos<-1000000000, refers to user point

                        iposm = -ipos-1000000000
                        kmeshl(iposm) = knodp

                     else if ( ipos<0 ) then

                        iposm = kmeshm(-ipos)
                        kmeshm(-ipos) = knodp
                        if ( iposm<0 ) then
                           kmeshl(-iposm) = knodp
                        end if
                     end if
                  end do
                  istart = istart + inplms
               end do

               deallocate ( ipoint, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'ipoint' )
               if ( ierror/=0 ) go to 1000

            end do
         end if
         kstart = kstart+nposrf

      end do

!     --- fill array kmeshm if necessary

      call msh044 ( kmeshm, kmeshl )

!     --- Fill array kmeshn for subsurfaces if there are surfaces consisting of
!         subsurfaces

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then
         call prinin ( mapar, inodp-1, 'mapar' )
         write(irefwr,*) 'End msh035'
      end if  ! ( debug )
      call erclos ( 'msh035' )

      end
