      subroutine msh042 ( iinput, iinp, curves, iposc, ndim, n, icurvs, &
                          icurve, icurnr, tau, ispinp, ncurvs, &
                          rinput, irnp2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.3  date 06-06-2013 Debug statements
!        version  4.2  date 12-01-2009 New call to mshc01
!        version  4.1  date 12-10-2002 New storage composite curves
!
!   copyright (c) 1985-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    copy co-ordinates of curves that have been constructed
!    into curves consisting of curves
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     spline
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, iposc, ndim, n, icurvs(*), icurve, &
              icurnr, ispinp(*), ncurvs, irnp2
      double precision curves(*), tau(*), rinput(*)

!     curves    i/o   array containing the coordinates of the curves
!     icurnr     i    Curve sequence number
!     icurve     i    Type of curve
!     icurvs     i    array containing the number of points in the curves
!                     accumulated
!     iinp       i    starting position in array iinput
!     iinput     i    new array iinput
!     iposc      i    next position in array curves to be filled
!     irnp2      i    first pos. in rinput definition of factors
!     ispinp          Work array of length n+2 to be used for spline
!                     generation, where n is numbner of nodes along curve
!     n          o    number of points in curve
!     ncurvs     i    Number of curves in mesh
!     ndim       i    Dimension of the space.
!     rinput    i/o   standard real input array for subroutine mesh
!                     it is supposed that the first five positions have been
!                     deleted
!     tau             Work array of length n to store the relative distances
!                     between points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kposc, i, jcurve, kcurve, jposc, jstart, k, j, iratio, &
              nelm, ishape, ncrvs, iposcsav
      double precision factor, h1, h2, rinloc(1)

!     factor         Multiplication factor
!     h1             Length of first element along curve
!     h2             Length of last element along curve
!     i              Counting variable
!     iposcsav       Help variable to store the old value of iposc
!     iratio         Ratio as defined in input
!     ishape         Shape number of elements
!     j              Counting variable
!     jcurve         Curve number
!     jposc          Position in array curves
!     jstart         starting position of curve i in kmeshm
!     k              Counting variable
!     kcurve         abs(jcurve)
!     kposc          Position in array curves
!     ncrvs          number of curves the curve consists of
!     nelm           number of elements along a curve
!     rinloc         the first position contains alpha for splines
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH004         Fill array tau
!     MSHC01         Fill array curves with splines
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh042' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh042'
         write(irefwr,1) 'iposc, icurve, icurnr, ndim', &
                          iposc, icurve, icurnr, ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iposcsav = iposc
      iposc = ndim*(iposc-1)+1

      kposc = iposc
      if ( icurve==7 ) then
         ncrvs = iinput(iinp-1)
      else
         ncrvs = iinput(iinp+2)
      end if  ! ( icurve==7 )

      if ( debug ) write(irefwr,1) 'ncrvs', ncrvs

      do i = 1, ncrvs

         if ( icurve==7 ) then
            jcurve = iinput(iinp+i-1)
         else
            jcurve = iinput(iinp+i+2)
            icurvs(ncurvs+abs(jcurve)) = 2
         end if  ! ( icurve==7 )

         if ( debug ) write(irefwr,1) 'i, jcurve', i, jcurve

         kcurve = abs(jcurve)
         if ( kcurve==1 ) then
            jposc = 0
            n = icurvs(1)
         else
            jposc = icurvs(kcurve-1)
            n = icurvs(kcurve) - jposc
         end if  ! ( kcurve==1 )

         if ( debug ) write(irefwr,1) 'n, jposc', n, jposc

         jposc = jposc*ndim+1
         if(jcurve>0) then
            jstart = jposc
         else
            jstart = jposc+(n-1)*ndim
         end if  ! (jcurve>0)
         if ( i>1 .and. icurve/=7 ) kposc = kposc-ndim

         if ( debug ) write(irefwr,1) 'jposc, kposc', jposc, kposc

!        --- copy co-ordinates of curves

         if ( jcurve>0 ) then

!        --- copy forwards

            curves(kposc:kposc+ndim*n-1) = curves(jposc:jposc+ndim*n-1)
            if ( debug ) &
               call prinrl1 ( curves(kposc), n, ndim, 'curve' )
            kposc = kposc+ndim*n

         else

!        --- copy backwards

            do j = 1, n
               do k = 1, ndim
                  curves(kposc+k-1) = curves(jstart+k-1)
               end do
               kposc = kposc+ndim
               jstart = jstart-ndim
            end do

         end if

      end do

      n = (kposc-iposc)/ndim

      if ( debug ) then
         write(irefwr,1) 'n', n
         call prinrl1 ( curves(iposc), n, ndim, 'composite curve' )
      end if  ! ( debug )

      if ( icurve==14 ) then

!     --- icurve = 14, construct spline through curve

         ishape = iinput(iinp-3)
         nelm = (n-1) /ishape
         iratio = 2
         h1 = (curves(iposc+ndim)-curves(iposc))**2
         if ( ndim>=2 ) h1 = h1+(curves(iposc+ndim+1)-curves(iposc+1))**2
         if ( ndim>=3 ) h1 = h1+(curves(iposc+ndim+2)-curves(iposc+2))**2
         h1 = sqrt(h1)
         h2 = (curves(kposc-1)-curves(kposc-1-ndim))**2
         if ( ndim>=2 ) h2 = h2+(curves(kposc-2)-curves(kposc-2-ndim))**2
         if ( ndim>=3 ) h2 = h2+(curves(kposc-3)-curves(kposc-3-ndim))**2
         h2 = sqrt(h2)
         factor = h1/h2
         rinput(irnp2+icurnr-1) = factor

!        --- Store subdivision of nodes in array tau

         call msh004 ( tau, ishape, nelm, n, iratio, factor, 0 )

!        --- Compute spline and store in curves
!            ipispl = itype=2, n, + [1..n]
!            rinput = curves

         ispinp(1) = 32
         ispinp(2) = n
         do i = 1, n
            ispinp(2+i) = i
         end do
         rinloc(1) = 0.5d0
         call mshc01 ( ispinp, curves(iposc), curves(iposc), &
                       ndim, n, tau, rinloc, icurnr )

      end if

      iposc = iposcsav

1000  call erclos ( 'msh042' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh042'

      end if  ! ( debug )

      end
