      subroutine msh054
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 27-11-2013 Maximize number of entries in message
!        version  4.0  date 02-09-2010 Remove intarmsh
!        version  3.1  date 08-09-1997 Restore negative values
!
!   copyright (c) 1987-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if all user points, curves and surfaces have been filled
!     If not, give a warning.  Negative values are set to zero
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     curve
!     mesh
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, icurve, ih, ikelmc, icount, n
      logical found, detect

!     detect     If true a warning for that curve or surface has been detected
!     found      If true a warning of that type has been detected
!     i          Counting variable
!     icount     number of unknowns per element
!     icurve     Curve number
!     ih         Help variable to store a constant temporarily
!     ikelmc     position in kmesh part c
!     istart     First position used at a part
!     n          number of nodes along curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integers in error message
!     ERRWAR  Warnings
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   180   User point has not been filled
!   585   Surface has not been filled
!   587   Curve has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh054' )

!     --- First check user points

      found = .false.
      icount = 0
      do i = 1, nuspnt
         if ( kmeshl(i)<=0 ) then
            icount = icount+1
            if ( icount<1000 ) call errint ( i, icount )
            found = .true.
         end if
      end do
      if ( found ) &
         call errwar ( 180, min(icount,1000), 0, 0 )

!     --- Next check curves if nsurel>0

      found = .false.
      icount = 0
      if ( nsurel>0 ) then
         istart = kmeshm(2)-1
         do icurve = 1, ncurvs
            n = kmeshm(5+icurve)-kmeshm(4+icurve)
            detect = .false.
            if ( n<=0 ) detect = .true.
            ih = istart+kmeshm(icurve+4)
            do i = 0, n-1
               if ( kmeshm(ih+i)<=0 ) then
                  detect = .true.
               end if
            end do
            if ( detect ) then
               icount = icount+1
               if ( icount<1000 ) call errint ( icurve, icount )
               found = .true.
            end if
         end do
         if ( found ) &
            call errwar ( 587, min(icount,1000), 0, 0 )

      end if

!     --- Finally check surfaces if nvolel>0

      found = .false.
      icount = 0
      if ( nvolel>0 ) then
         ikelmc = 2*nsurfs
         do isurf = 1, nsurfs
            inpelm = kmeshn(isurf)
            nelemlc = kmeshn(isurf+nsurfs)
            detect = .false.
            if ( inpelm<=0 .or. nelemlc<=0 ) detect = .true.
            do i = 1, nelemlc*inpelm
               if ( kmeshn(ikelmc+i)<=0 ) then
                  detect = .true.
               end if
            end do
            if ( detect ) then
               icount = icount+1
               if ( icount<1000 ) call errint ( isurf, icount )
               found = .true.
            end if
            ikelmc = ikelmc+inpelm*nelemlc
         end do
         if ( found ) &
            call errwar ( 585, min(icount,1000), 0, 0)

      end if

      call erclos ( 'msh054' )

      end
