      subroutine msh062 ( iinput, iseqsurfs, ipointsurf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 02-11-2012 Debug statements
!        version  2.4  date 20-10-2008 Allow isurf = 24
!        version  2.3  date 27-05-2002 Allow negative surfaces
!
!   copyright (c) 1987-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array kmeshr with surface numbers of the subsequent surfaces
!     Help subroutine for subroutine MSH060
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iseqsurfs(*), ipointsurf(*)

!     iinput         i    Array containing information of the surfaces
!                         See Prog. Guide 3.1.2
!     ipointsurf     i    Array containing the relative starting addresses
!                         of all surfaces in the part of iinput referring
!                         to surfaces
!     iseqsurfs      i    Contains new sequence of surface numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, istart, i, ndefsf, nrow, k, ic, jsurnr, &
              ksurnr, jstart, nsubsf, lengav

!     i              Counting variable
!     ic             Relative subcurve number
!     iinp           last position used in array iinput
!     istart         First position used at a part
!     jstart         starting position of curve i in kmeshm
!     jsurnr         Surface sequence number
!     k              Counting variable
!     ksurnr         absolute value of surface sequence number
!     lengav         available space for array KMESH part r
!     ndefsf         Number of surfaces in surface
!     nrow           Number of rows of surfaces in ordened surface of
!                    surfaces
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh062' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh062'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array kmeshr

      kmeshr(1:nsurfs+1) = 0
      lengav = 100*nsurfs

!     --- Loop over surfaces
!         Fill number of subsurfaces in kmeshr

      do i = 1, nsurfs

         isurnr = iseqsurfs(i)
         iinp = ipointsurf(isurnr)
         isurf = iinput(iinp)
         if ( isurf==24 ) isurf = 7  ! compound surface
         nsubsf = 0

         if ( debug ) &
            write(irefwr,1) 'i, isurnr, isurf', i, isurnr, isurf

         if ( isurf==7 .or. isurf==11 ) then

!        --- surface consisting of subsurfaces

            if ( isurf==7 ) then

!           --- isurf = 7   non-ordered surface

               ndefsf = iinput(iinp+2)
               nrow = 0

            else

!           --- isurf = 11  ordered surface

               nrow = iinput(iinp+2)
               ndefsf = 0
               do k = 1, nrow
                  ndefsf = ndefsf + iinput(iinp+2+k)
               end do

            end if

!           --- Loop over subsurfaces

            do ic = 1 , ndefsf
               jsurnr = iinput(iinp+ic+2+nrow)
               ksurnr = abs(jsurnr)
               nsubsf = nsubsf+max(1,kmeshr(ksurnr))
            end do
            kmeshr(isurnr) = nsubsf

         end if

      end do  ! i = 1, nsurfs

      if ( debug ) call prinin ( kmeshr, nsurfs+1, 'kmeshr' )

!     --- Construct kmeshr part a positions 1 to nsurfs+1

      istart = nsurfs + 1
      do i = 1, nsurfs
         nsubsf = kmeshr(i)
         kmeshr(i) = istart
         istart = istart+nsubsf
      end do
      kmeshr(nsurfs+1) = istart

      if ( debug ) call prinin ( kmeshr, nsurfs+1, 'kmeshr/2' )

      call inichkarlen ( istart-1, lengav, 'kmeshr' )
      if ( ierror/=0 ) go to 1000

!     --- Fill second part of kmeshr

      do i = 1, nsurfs

         isurnr = iseqsurfs(i)
         iinp = ipointsurf(isurnr)
         isurf = iinput(iinp)
         if ( isurf==24 ) isurf = 7  ! compound surface
         istart = kmeshr(isurnr)
         if ( isurf==7 .or. isurf==11 ) then

!        --- surface consisting of subsurfaces

            if ( isurf==7 ) then

!           --- isurf = 7   non-ordered surface

               ndefsf = iinput(iinp+2)
               nrow = 0

            else

!           --- isurf = 11  ordered surface

               nrow = iinput(iinp+2)
               ndefsf = 0
               do k = 1, nrow
                  ndefsf = ndefsf + iinput(iinp+2+k)
               end do

            end if

!           --- Loop over subsurfaces

            do ic = 1 , ndefsf
               jsurnr = iinput(iinp+ic+2+nrow)
               ksurnr = abs(jsurnr)
               if ( isurf==7 ) jsurnr = ksurnr
               jstart = kmeshr(ksurnr)
               nsubsf = kmeshr(ksurnr+1) - jstart
               if ( nsubsf==0 ) then

!              --- nsubsf = 0  subsurface does not contain sub subsurfaces

                  istart = istart + 1
                  kmeshr(istart) = jsurnr

               else

!              --- nsubsf>0  subsurface contains sub subsurfaces

                  if ( jsurnr>0 ) then
                     do k = 1, nsubsf
                        kmeshr(istart+k) = kmeshr(jstart+k)
                     end do
                  else
                     do k = 1, nsubsf
                        kmeshr(istart+k) = -kmeshr(jstart+nsubsf+1-k)
                     end do
                  end if
                  istart = istart + nsubsf
               end if
            end do

         end if

      end do  ! i = 1, nsurfs

      istart = kmeshr(nsurfs+1)

!     --- Set last part of kmeshr equal to zero

      length = istart - nsurfs - 1
      kmeshr(istart+1:istart+length) = 0
      length = istart

!     --- Put kmeshr into ibuffr

      if ( length>nsurfs+1 ) then
         kelmr = 2*length-nsurfs-1
         allocate ( km(iseqkm)%kmeshr(kelmr) )
         km(iseqkm)%kmeshr = kmeshr(1:kelmr)
      else
         kelmr = 0
      end if  ! ( length>nsurfs+1 )

      deallocate ( kmeshr, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshr' )
      kmeshr => km(iseqkm)%kmeshr

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         call prinin ( kmeshr, istart-1, 'kmeshr' )
         write(irefwr,*) 'End msh062'

      end if  ! ( debug )
      call erclos ( 'msh062' )

      end
