      subroutine msh355 ( weddir, conedge, iel, leftover, dir, force )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  1.2  date 29-09-2000 Correction subscripts weddir
!        version  1.1  date 28-11-1997  Orientation of el stored in conedge
!        version  1.0  date 03-10-1997
!
!   copyright (c) 1997-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!        subroutine to set the diagonal of the side indicated by
!        leftover of a wedge
!        based on the direction and wether this should be forced
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     pipe
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer weddir(7,*), conedge(6,*), iel, leftover, dir, force

!     conedge  o    Array containing connectivity of the sides of elements in
!                   kmscbt
!     dir      i    Direction that will be assigned to leftover depending on
!                   force
!     force    i    Indicates whether this is a suggestion and can be changed
!                   or if it has to be obayed (value of dir)
!     iel      i    Element that iel belongs to
!     leftover i    side of iel that has to be filled with a direction
!     weddir   o    Direction of the diagonals per wedge
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer oel, oedge, dir1, iside

!     dir1     Temporary direction
!     iside    Looping variable for sides
!     oedge    Edge at which oel is connected
!     oel      Other element connected to iel at side leftover
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      oel = conedge(leftover,iel)
      oedge = conedge(leftover+3,iel)

      if (weddir(4,oel).eq.2 .and. force.eq.0) then

!     --- in this case you might want to adjust the direction of
!         the diagonal about to be set (you can change it since
!         force.eq.0)  If the diagonal of the other element is
!         the same for the two sides, chose dir such that the
!         third diagonal of the other element is in oposite
!         direction.

         dir1 = 0
         do iside = 1,3
            if (weddir(iside,oel).ne.0 .and. dir1.eq.0) then
               dir1 = weddir(iside,oel)
            else if(weddir(iside,oel).ne.0 .and. &
                   weddir(iside,oel).eq.dir1) then
               dir = dir1
            end if
         end do
      end if

      weddir(leftover,iel)   = dir * sign(1,oedge)
      weddir(abs(oedge),oel) =-dir * sign(1,oedge)
      weddir(4,iel)          = weddir(4,iel) + 1
      weddir(4,oel)          = weddir(4,oel) + 1

      end
