      subroutine msh3dq ( isurf, coor, kmeshc, npoint, plotfm, yfact, &
                          inpelm, nelem )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.9  date 20-10-2008 Minor change
!        version  1.8  date 13-03-2006 Fortran 90
!        version  1.7  date 15-10-2004 Debug option
!
!   copyright (c) 1990-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      plot one surface in 3-D (no hidden lines)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3, *), plotfm, yfact
      integer isurf, inpelm, kmeshc(inpelm,*), npoint, nelem

!     coor         i   array containing the coordinates of the surface
!     inpelm       i   number of nodal points in standard element
!     isurf        i   surface number
!     kmeshc       i   kmesh part c
!     nelem        i   number of elements in submesh
!     npoint       i   number of nodal points in surface
!     plotfm       i   length of the plot in centimeters
!     yfact        i   multiplication factor for the y-coordinates
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision disx, disy
      double precision xs, ys, zs, xp, yp
      double precision xminp, xmaxp, yminp, ymaxp, xminsv, &
                       yminsv, zminsv, xmaxsv, ymaxsv, zmaxsv
      integer i, nbpoin, ikn, in, is, ikader, imark, iwork(npoint)

!     disx           ?
!     disy           ?
!     i              Counting variable
!     ikader         Help variable to store jkader temporarily
!     ikn            ?
!     imark          ?
!     in             (i-1) x lrow
!     is             ?
!     iwork          integer work array of length npoint
!                    This array is used to mark node numbers that have been
!                    plotted by a 1
!     nbpoin         Number of boundary points of element
!     xmaxp          maximum x-co-ordinate of projection
!     xmaxsv         Help variable to save the value of xmax
!     xminp          minimum x-co-ordinate of projection
!     xminsv         Help variable to save the value of xmin
!     xp             x-coordinate in plotting units
!     xs             ?
!     ymaxp          maximum y-co-ordinate of projection
!     ymaxsv         Help variable to save the value of ymax
!     yminp          minimum y-co-ordinate of projection
!     yminsv         Help variable to save the value of ymin
!     yp             y-coordinate in plotting units
!     ys             ?
!     zmaxsv         Help variable to save the value of zmax
!     zminsv         Help variable to save the value of zmin
!     zs             ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS       Resets old name of previous subroutine of higher level
!     EROPEN       Produces concatenated name of local subroutine
!     ERRINT       Put integer in error message
!     ERRSUB       Error messages
!     PL0000       Compute length and width of the plot
!     PLAFEP       General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     913     ?
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('msh3dq')
      debug = .false.
      if ( debug ) then

!     --- Node numbers must be plotted, clear iwork

         iwork = 0

      end if  ! ( debug )

!     --- compute xmin, xmax, ymin, ymax, aleng and width

      xmin  =  rinfin
      ymin  =  xmin
      zmin  =  xmin
      xmax  = -xmin
      ymax  =  xmax
      zmax  =  xmax
      do i = 1 , npoint
         xmin  =  min ( xmin, coor(1,i) )
         ymin  =  min ( ymin, coor(2,i) )
         zmin  =  min ( zmin, coor(3,i) )
         xmax  =  max ( xmax, coor(1,i) )
         ymax  =  max ( ymax, coor(2,i) )
         zmax  =  max ( zmax, coor(3,i) )
      end do
      xminsv = xmin
      yminsv = ymin
      zminsv = zmin
      xmaxsv = xmax
      ymaxsv = ymax
      zmaxsv = zmax
      xmin = 0d0
      ymin = 0d0
      zmin = 0d0
      xmax = xmaxsv-xminsv
      ymax = ymaxsv-yminsv
      zmax = zmaxsv-zminsv

!     --- Compute extreme values

      xminp = xmin + 0.77d0 * ymin
      xmaxp = xmax + 0.77d0 * ymax
      yminp = zmin + 0.45d0 * ymin
      ymaxp = zmax + 0.45d0 * ymax
      disx  = ( xmaxp - xminp ) / 10d0
      disy  = ( ymaxp - yminp ) / 10d0
      xmin  = xminp - disx
      xmax  = xmaxp + disx
      ymin  = yminp - disy
      ymax  = ymaxp + disy
      call pl0000 ( plotfm, yfact )

!     --- start picture

      ikader  =  jkader
      if ( ikader==0 ) jkader = 10 + isurf
      call plafep(1, aleng, width, 0d0, 0d0, 0)
      jkader = ikader
      imark  = jmark
      jmark  = 3

!     --- Compute number of boundary points in an element

      if ( inpelm==3 .or. inpelm==4 .or. inpelm==6 .or. &
           inpelm==8   ) then
           nbpoin = inpelm
      else if ( inpelm==7 .or. inpelm==9  ) then
           nbpoin = inpelm - 1
      else

!     --- Only the possibilities inpelm = 3, 4, 6, 7,8 and 9 are supported in
!         MSH3DQ

           call errint ( 1, inpelm )
           call errsub ( 913, 1, 0, 0 )
           go to 1000
      end if

!     --- Run through all elements, plot the boundaries

      do i = 1 , nelem

!     --- Start with last boundary point of element

          is = kmeshc(nbpoin,i)
          xs = coor(1,is)-xminsv
          ys = coor(2,is)-yminsv
          zs = coor(3,is)-zminsv

!         --- Compute projection of this point

          xp = xs + 0.77d0 * ys
          yp = zs + 0.45d0 * ys

!         --- Move pen to this point

          call plafep(4, xp, yp, 0d0, 0d0, 0)

!         --- Run through all points of boundary of element

          do in = 1 , nbpoin
              ikn = kmeshc(in,i)
              xs  = coor(1,ikn)-xminsv
              ys  = coor(2,ikn)-yminsv
              zs  = coor(3,ikn)-zminsv

!             --- Compute projection

              xp  = xs + 0.77d0 * ys
              yp  = zs + 0.45d0 * ys

!             --- Plot to this point

              call plafep ( 5, xp, yp, 0d0, 0d0, 0 )
              if ( debug ) then

!             --- Debug = true, plot number

                 if ( iwork(ikn)==0 ) &
                    call plafep ( 7, xp, yp, 0.02d0, 0.02d0, ikn )
                 iwork(ikn) = 1

              end if  ! ( debug )

          end do  ! in = 1 , nbpoin

      end do

      jmark  =  imark

!     --- close plot dataset

      call plafep(2, 0d0, 0d0, 0d0, 0d0, 0)

!     --- close error file

1000  call erclos('msh3dq')
      end
