      subroutine msh463( ia, ib, npt, coor, niedge, ndim, factor )
! =======================================================================
!
!
!        programmer     Niek Praagman
!        version 1.0    date   20-03-1995
!
!
!
!   copyright (c) 1995 - 1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ***********************************************************************
!
!    DESCRIPTION
!
!    Compute coordinates of niedge points between nodes ia and ib
!
! ***********************************************************************
!
!    KEYWORDS
!
!    mesh_generation
!    element
!    boundary
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision coor(*), factor(0:*)
      integer npt, ia, ib, niedge, ndim

!     coor      i,o  array containing the coordinates
!
!     factor     i   work array with the distribution of the points
!                    along sides, length: niedge + 2
!
!     ia         i   starting node of line
!
!     ib         i   end node of line
!
!     ndim       i   dimension of space considered
!
!     niedge     i   number of internal nodes on line
!
!     npt        i   node number of node(s) to be created
!
! ********************************************************************
!
!     COMMON BLOCKS
!
! ********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS   Resets old name of previous subroutine of higher level
!     EROPEN   Produces concatenated name of local subroutine
!
! ********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, iposa, iposb, npoint, idim
      double precision dx(3)

!     dx         coordinate distances
!     i          loop variable
!     idim       loop variable for dimension
!     iposa      pointer for point A
!     iposb      pointer for point B
!     npoint     actual number of node to be placed
!
! ********************************************************************
!
!     I/O
!
! ********************************************************************
!
!     ERROR MESSAGES
!
! ********************************************************************
!
!     PSEUDO CODE
!
!     Compute distance vector B - A
!
!     Place niedge points from A to B
!
! =======================================================================

      call eropen( 'msh463' )

!     Compute coordinates of all points between, (start with smallest
!     node number)

      if ( ia.lt.ib ) then

         iposa = ndim * ( ia-1 )
         iposb = ndim * ( ib-1 )

      else

         iposa = ndim * ( ib-1 )
         iposb = ndim * ( ia-1 )

      end if

      do 410 idim = 1 , ndim

         dx(idim) = (coor(iposb+idim) - coor(iposa+idim)) / 2d0

410   continue

      do 440 i = 1 , niedge

          npoint = npt + i - 1

          do 420 idim = 1 , ndim

              coor( ndim * (npoint-1) + idim ) = &
                    coor( iposa+idim ) + &
                    ( 1d0 + factor(i) ) * dx(idim)

420       continue

440   continue

      call erclos( 'msh463' )

      end
