      subroutine msh469( i, k, coor, kstapl, kstap, ja )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 26-04-1994 Complete new version
!        version  1.0  date 01-01-1989
!
!   copyright (c) 1989-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine whether line i - k is permitted.
!     With other words : Belongs lines i - k  to area considered. The
!     area is given by the line elements in kstapl.
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i, ja, k, kstap, kstapl(2,kstap)
      double precision coor(2,*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     i       i    node number to be considered
!     ja      o    if ja = 1 then point is not visible
!                  if ja = 0 then point is visible
!     k       i    node number to be considered
!     kstap   i    number of boundary segments
!     kstapl  i    array with nodes of boundary segments
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, ih, li
      double precision x1, x2, xh, xk, y1, y2, yh, yk

!     i1      point number of first point of boundary segment
!     i2      point number of second point of boundary segment
!     ih      common point indicator ( 0 : yes, 1 : no )
!     li      loop variable
!     x1      x-coordinate of first point of boundary segment
!     x2      x-coordinate of second point of boundary segment
!     xh      x-coordinate of first point of line under consideration
!     xk      x-coordinate of second point of line under consideration
!     y1      y-coordinate of first point of boundary segment
!     y2      y-coordinate of second point of boundary segment
!     yh      y-coordinate of first point of line under consideration
!     yk      y-coordinate of second point of line under consideration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH470  determine whether two line segments have a point in common
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check whether line has any point in common with the boundary
!     PSEUDO CODE
!     Take the line segment indicated
!     Do for all boundary segments
!        If line segment has any point in common with the boundary segment
!           then  Return
!        Endif
!     Enddo
!     Conclude : no points in common
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ja = 1

      xh = coor(1,i)
      yh = coor(2,i)

      xk = coor(1,k)
      yk = coor(2,k)

      do 10 li = 1, kstap

          i1 = kstapl(1,li)
          i2 = kstapl(2,li)

          x1 = coor(1,i1)
          y1 = coor(2,i1)

          x2 = coor(1,i2)
          y2 = coor(2,i2)

          ih = 1

          ja = 1

          if ( k.eq.i1 ) ja = 0
          if ( k.eq.i2 ) ja = 0
          if ( i.eq.i1 ) ja = 0
          if ( i.eq.i2 ) ja = 0

          if ( ja.eq.1 ) &
               call msh470( xh, yh, xk, yk, x1, y1, x2, y2, ih )

          if ( ih.eq.0 ) goto 100

10    continue

!     --- no common point found with all boundary parts

      ja = 0

100   continue

      end
