      subroutine msh470( xi, yi, xj, yj, x1, y1, x2, y2, ih )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.0  date 27-04-1994  new method, params s and t
!        version  3.1  date 19-04-1994  small adjustments
!        version  3.0  date 20-03-1994  update of routine snijpt for SEPRAN
!        version  2.0  date 27-07-1991  update
!        version  1.1  date 17-06-1991  general revision by Arno van Duin
!        version  1.1  date 11-01-1988
!
!   copyright (c) 1987-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to check whether line i-j and line 1-2 have a point in
!     common.
! **********************************************************************
!
!                       KEYWORDS
!
!     crossing
!     common_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ih
      double precision x1, x2, xi, xj, y1, y2, yi, yj

!     ih      o    indicator of result
!                     ih = 0 : there is a common point
!                     ih = 1 : there is no common point
!     x1      i    x-coordinate of point 1
!     x2      i    x-coordinate of point 2
!     xi      i    x-coordinate of point i
!     xj      i    x-coordinate of point j
!     y1      i    x-coordinate of point 1
!     y2      i    x-coordinate of point 2
!     yi      i    x-coordinate of point i
!     yj      i    x-coordinate of point j
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps, det, s, t

!     det    determinant of system of 2 * 2 equations
!     eps    accuracy parameter
!     s      parameter of common point on line 1 - 2
!     t      parameter of common point on line i - j
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Use parameter representation for line i - j
!     ( x )   ( xi )        ( xj - xi )
!     (   ) = (    )  + t * (         )
!     ( y )   ( yi )        ( yj - yi )
!     and for line 1 - 2
!     ( x )   ( x1 )        ( x2 - x1 )
!     (   ) = (    )  + s * (         )
!     ( y )   ( y1 )        ( y2 - y1 )
!     Determine ( s, t ) of intersection point of the lines
!     if 0 < s < 1 and 0 < t < 1 then segments have a common point
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set accuracy

      eps    = 10 * epsmac

!     --- Set parameter ih : (1 means there is no common point)

      ih     = 1

      det = ( xj - xi ) * ( y2 - y1 ) - ( x2 - x1 ) * ( yj - yi )

      if ( abs(det).gt.eps ) then

!     --- Compute coordinates intersection point

         t =   ( y2 - y1 ) * ( x1 - xi ) - ( x2 - x1 ) * ( y1 - yi )
         t =     t / det

         s =   ( yj - yi ) * ( x1 - xi ) - ( xj - xi ) * ( y1 - yi )
         s =     s / det

!        --- Check where the points are situated

         if ( 0d0.lt.t .and. 1d0.gt.t .and. &
              0d0.lt.s .and. 1d0.gt.s ) ih = 0

      end if

      end
