      subroutine msh484( kelem, i, niedge, nisurf, nivolm )
! =======================================================================
!
!
!         programmer   Niek Praagman
!         version 1.1  date  11-07-1995 Declaration adjustment for internal
!                                      nodes)
!         version 1.0  date  07-04-1995
!
!
!
!   copyright (c) 1995 - 1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Fill kelem with a spectral hexahedron
!
! ***********************************************************************
!
!     KEYWORDS
!
!     mesh_generation
!     spectral
!     hexahedron
!
! ***********************************************************************
!
!
!     INPUT / OUTPUT PARAMETERS

      implicit none
      integer kelem(*), i(27), niedge, nisurf, nivolm

!     i        i    array in which all the important nodes
!                   for the spectral element are given
!     kelem    o    array to store new (spectral) element
!     niedge   i    number of internal nodes on line part
!     nisurf   i    number of internal nodes on surface part
!     nivolm   i    number of internal nodes in volume part
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer ihlp1, ihlp2, jpos, k, kk, kl, km, line, lsurf, newplm, &
              npt, ih, inode(576)

!     ih        helpvariable for running through node array
!     ihlp1     helpvariable in first plane
!     ihlp2     helpvariable for last plane
!     inode     array with the local numbering of a surface
!     jpos      pointer in kelem
!     k         loop variable
!     kk        loop variable for first direction
!     kl        loop variable for second direction
!     km        loop variable for third direction
!     line      help variable
!     lsurf     number of points in one plane
!     newplm    number of nodes in new element
!     npt       actual node number of internal node
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS    deconcatenate name from calling sequence
!     EROPEN    concatenate name to calling sequence
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP    stop execution of program
!     MSH492    determine correct ordering of nodal points of quadrilateral
! ***********************************************************************
!
!     ERROR MESSAGES
!
!     973   Required option has not yet been implemented
!     974   Too many side points
! ***********************************************************************
!
!     METHOD
!
!     Run through all (2+niedge) "planes" of spectral hexahedron and
!     fill the nodenumbers in array kelem
!
! =======================================================================

      call eropen( 'msh484' )

!     Determine the total number of nodes

      newplm = 8 + 12 * niedge + 6 * nisurf + nivolm

!     Assume that all quantities are positive

      if ( niedge.le.0 .or. nisurf.le.0 .or. &
           nivolm.le.0 ) then

           call errsub ( 973, 0, 0, 0 )
           call instop

      end if

      line   =      2   +   niedge
      lsurf  =   line   *   line
      ihlp1  =  lsurf   -   line
      ihlp2  = newplm   -   lsurf

!     Place the eight corners

      kelem(                 1 )  =  i(1)
      kelem( line              )  =  i(2)
      kelem( ihlp1 +         1 )  =  i(4)
      kelem( lsurf             )  =  i(3)
      kelem( ihlp2 +         1 )  =  i(5)
      kelem( ihlp2 + line      )  =  i(6)
      kelem( ihlp2 + ihlp1 + 1 )  =  i(8)
      kelem( newplm            )  =  i(7)

!     Place the line nodes

      do 100 k = 1 , niedge

!         Side i(1) - i(2)

          if ( i(1).lt.i(2) ) then
             kelem( 1 + k ) = i(9) + k - 1
          else
             kelem( 1 + k ) = i(9) + niedge - k
          end if

!         Side i(4) - i(3)

          if ( i(4).lt.i(3) ) then
             kelem( lsurf - line + 1 + k ) = i(11) + k - 1
          else
             kelem( lsurf - line + 1 + k ) = i(11) + niedge - k
          end if

!         Side i(5) - i(6)

          if ( i(5).lt.i(6) ) then
             kelem( ihlp2 + 1 + k ) = i(17) + k - 1
          else
             kelem( ihlp2 + 1 + k ) = i(17) + niedge - k
          end if

!         Side i(8) - i(7)

          if ( i(8).lt.i(7) ) then
             kelem( newplm - line + 1 + k ) = i(19) + k - 1
          else
             kelem( newplm - line + 1 + k ) = i(19) + niedge - k
          end if

!         Side i(1) - i(4)

          if ( i(1).lt.i(4) ) then
             kelem( k * line + 1 ) = i(12) + k - 1
          else
             kelem( k * line + 1 ) = i(12) + niedge - k
          end if

!         Side i(2) - i(3)

          if ( i(2).lt.i(3) ) then
             kelem( ( k+1 ) * line ) = i(10) + k - 1
          else
             kelem( ( k+1 ) * line ) = i(10) + niedge - k
          end if

!         Side i(5) - i(8)

          if ( i(5).lt.i(8) ) then
             kelem( ihlp2 + 1 + k * line ) = i(20) + k - 1
          else
             kelem( ihlp2 + 1 + k * line ) = i(20) + niedge - k
          end if

!         Side i(6) - i(7)

          if ( i(6).lt.i(7) ) then
             kelem( ihlp2 + ( k+1 ) * line ) = i(18) + k - 1
          else
             kelem( ihlp2 + ( k+1 ) * line ) = i(18) + niedge - k
          end if

!         Side i(1) - i(5)

          if ( i(1).lt.i(5) ) then
             kelem( 1 + k * lsurf ) = i(13) + k - 1
          else
             kelem( 1 + k * lsurf ) = i(13) + niedge - k
          end if

!         Side i(2) - i(6)

          if ( i(2).lt.i(6) ) then
             kelem( k * lsurf + line ) = i(14) + k - 1
          else
             kelem( k * lsurf + line ) = i(14) + niedge - k
          end if

!         Side i(4) - i(8)

          if ( i(4).lt.i(8) ) then
             kelem( (k+1) * lsurf - line + 1 ) = i(16) + k - 1
          else
             kelem( (k+1) * lsurf - line + 1 ) = i(16) + niedge - k
          end if

!         Side i(3) - i(7)

          if ( i(3).lt.i(7) ) then
             kelem( ( k+1 ) * lsurf ) = i(15) + k - 1
          else
             kelem( ( k+1 ) * lsurf ) = i(15) + niedge - k
          end if

100   continue

!     Next place the internal nodes of the six faces

!     Bottom face : i(1) - i(2) - i(3) - i(4)

      jpos  =  2 + niedge
      npt   =  i(21) - 1
      ih    =  1

!     Determine the situation

      if ( niedge.gt.24 ) then
           call errint ( niedge, 1 )
           call errint ( 24, 2 )
           call errsub ( 974, 2, 0, 0 )
           call instop
      end if

      call msh492( i(1), i(2), i(3), i(4), niedge, inode )

      do 220 kl = 1 , niedge

         jpos = jpos + 2

         do 210 kk = 1 , niedge

            kelem( jpos ) = npt  + inode( ih )
            ih            = ih   + 1
            jpos          = jpos + 1

210      continue

220   continue

!     Front face : i(1) - i(2) - i(6) - i(5)

      jpos  =  2 + niedge
      npt   =  i(22) - 1
      ih    =  1

!     Determine the situation

      call msh492( i(1), i(2), i(6), i(5), niedge, inode )

      do 240 kl = 1 , niedge

         jpos = jpos + lsurf - niedge

         do 230 kk = 1 , niedge

            kelem( jpos ) = npt  + inode( ih )
            ih            = ih   + 1
            jpos          = jpos + 1

230      continue

240   continue

!     Right face : i(2) - i(3) - i(7) - i(6)

      jpos  =  lsurf
      npt   =  i(23) - 1
      ih    =  1

!     Determine the situation

      call msh492( i(2), i(3), i(7), i(6), niedge, inode )

      do 260 kl = 1 , niedge

         jpos = jpos + 2 * line

         do 250 kk = 1 , niedge

            kelem( jpos ) = npt  + inode( ih )
            ih            = ih   + 1
            jpos          = jpos + line

250      continue

260   continue

!     Back face : i(4) - i(3) - i(7) - i(8)

      jpos  =  lsurf
      npt   =  i(24) - 1
      ih    =  1

!     Determine the situation

      call msh492( i(4), i(3), i(7), i(8), niedge, inode )

      do 280 kl = 1 , niedge

         jpos = jpos + lsurf - niedge

         do 270 kk = 1 , niedge

            kelem( jpos ) = npt  + inode( ih )
            ih            = ih   + 1
            jpos          = jpos + 1

270      continue

280   continue

!     Left face : i(1) - i(4) - i(8) - i(5)

      jpos  =  lsurf + 1 - line
      npt   =  i(25) - 1
      ih    =  1

!     Determine the situation

      call msh492( i(1), i(4), i(8), i(5), niedge, inode )

      do 300 kl = 1 , niedge

         jpos = jpos + 2 * line

         do 290 kk = 1 , niedge

            kelem( jpos ) = npt  + inode( ih )
            ih            = ih   + 1
            jpos          = jpos + line

290      continue

300   continue

!     Top face : i(5) - i(6) - i(7) - i(8)

      jpos  =  ( 1 + niedge ) * lsurf + line
      npt   =  i(26) - 1
      ih    =  1

!     Determine the situation

      call msh492( i(5), i(6), i(7), i(8), niedge, inode )

      do 320 kl = 1 , niedge

         jpos = jpos + 2

         do 310 kk = 1 , niedge

            kelem( jpos ) = npt  + inode( ih )
            ih            = ih   + 1
            jpos          = jpos + 1

310      continue

320   continue

!     Finally place the internal nodes of the volume

      npt = i(27)
      jpos = lsurf - line

      do 430 km = 1 , niedge

         jpos = jpos + 2 * line

         do 420 kl = 1 , niedge

            jpos = jpos + 2

            do 410 kk = 1 , niedge

               kelem( jpos ) = npt
               npt           = npt  + 1
               jpos          = jpos + 1

410         continue

420      continue

430   continue

      call erclos( 'msh484' )

      end
