      subroutine msh494( kelem, i, niedge, nisurf, nivolm )
! =======================================================================
!
!
!         programmer   Niek Praagman
!         version 1.0  date  11-04-1995
!
!
!
!   copyright (c) 1995 - 1995  "Ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Refill kmesh part c with a new spectral tetrahedronal element
!
! ***********************************************************************
!
!     KEYWORDS
!
!     mesh_generation
!     spectral
!     tetrahedron
!
! ***********************************************************************
!
!
!     INPUT / OUTPUT PARAMETERS

      implicit none
      integer kelem(*), i(*), niedge, nisurf, nivolm

!     i        i    helparray with all the important nodes for the
!                   new element
!     kelem    o    array with ordered nodes of new element
!     niedge   i    number of internal nodes on line part
!     nisurf   i    number of internal nodes on surface part
!     nivolm   i    number of internal nodes in volume part
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer iface, il, ipos

!     iface     loop variable
!     il        loop variable
!     ipos      pointer in array kelem
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
!
! ***********************************************************************
!
!     METHOD
!
!     Check whether situation is in accordance with what can be handled
!
!     Determine the ordering of the triangular elements
!
!     Create new spectral triangular elements
!
! =======================================================================

!     Place the nodes on the edges of the basic triangle

      kelem( 1 )  = i(1)

      if ( i(1).lt.i(3) ) then

         do 110 il = 1 , niedge

            kelem( 1+il ) = i(2) - 1 + il

110      continue

      else

         do 120 il = 1 , niedge

            kelem( 1+il ) = i(2) + niedge - il

120      continue

      end if

      kelem( 2 + niedge )  = i(3)

      if ( i(3).lt.i(5) ) then

         do 130 il = 1 , niedge

            kelem( 2+niedge+il ) = i(4) - 1 + il

130      continue

      else

         do 140 il = 1 , niedge

            kelem( 2+niedge+il ) = i(4) + niedge - il

140      continue

      end if

      kelem( 3 + 2 * niedge )  = i(5)

      if ( i(5).lt.i(1) ) then

         do 150 il = 1 , niedge

            kelem( 3+2*niedge+il ) = i(6) - 1 + il

150      continue

      else

         do 160 il = 1 , niedge

            kelem( 3+2*niedge+il )  = i(6) + niedge - il

160      continue

      end if

!     Run through all the lines to the top of the tetrahedron
!     and place the nodes accordingly

      ipos = 3 * ( 1 + niedge )

      do 200 il = 1 , niedge

         if ( i(1).lt.i(10) ) then

            kelem( ipos+1 ) = i(7) - 1 + il

         else

            kelem( ipos+1 ) = i(7) + niedge - il

         end if

         if ( i(3).lt.i(10) ) then

            kelem( ipos+2 ) = i(8) - 1 + il

         else

            kelem( ipos+2 ) = i(8) + niedge - il

         end if

         if ( i(5).lt.i(10) ) then

            kelem( ipos+3 ) = i(9) - 1 + il

         else

            kelem( ipos+3 ) = i(9) + niedge - il

         end if

         ipos = ipos + 3

200   continue

      kelem( ipos+1 ) = i(10)
      ipos = ipos + 1

!     Next place (if available) the internal points, &
!     first the four faces

      if ( nisurf.gt.0 ) then

         do 320 iface = 1 , 4

            do 310 il = 1 , nisurf

               kelem( ipos+il ) = i(10+iface) + il - 1

310         continue

            ipos = ipos + nisurf

320      continue

      end if

!     Finally the volume points

      if ( nivolm.gt.0 ) then

!        Place the internal points

         do 400 il = 1 , nivolm

            kelem( ipos + il ) = i(15) - 1 + il

400      continue

      end if

      end
