      subroutine mshactmpr ( iinput, jnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 25-11-2012 Long integers
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.4  date 17-11-2008 Correction in case nbndpt=0
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate subarrays of kmesh part h used by mshmpr
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical jnew
      integer iinput(*)

!     iinput         i    integer input array
!     jnew           i    indication whether the mesh is new (true) or old
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, i, ifirst, jinp, kinp, ifirstparm, ilast, itype, &
              j, jns, index, isubsf, isursf, nsubsf, jsurnr, ih
      logical changed

!     changed        If true an issue has been changed
!     i              Counting variable
!     ifirst         First curve (surface) number read
!     ifirstparm     Parameter ifirst for subroutine
!     ih             Help variable to store a constant temporarily
!     iinp           last position used in array iinput
!     ilast          Last curve (surface) number read
!     index          Help parameter to store a value
!     isubsf         Sequence number of subsurface
!     isursf         Surface sequence number
!     itype          Type number of element
!     j              Counting variable
!     jinp           last position used in array iinput
!     jns            nsurfs+ivolnr
!     jsurnr         Surface sequence number
!     kinp           position in array iinput
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH018         compute position of surface in array iinput
!     MSHACT010      Activate subarrays of kmesh part h for subroutine msh010
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    See subroutine mshmpr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set some parameters

      call eropen ( 'mshactmpr' )
      iinp = iinp4

!     --- First set index positions in kmeshh with respect to kbndpt arrays
!         negative in order to know if a surface has been filled (>0)
!         or not (<0)

      do i  = 1, nsurfs+nvolms
         kmeshh(4,i+1) = -kmeshh(4,i+1)
      end do

      do i = 1, nsurel

         ifirst = iinput(iinp+2)
         ilast = iinput(iinp+3)
         iinp = iinp+4

         changed = .false.
         do jsurnr = ifirst, ilast

            isurnr = jsurnr
            if ( kmeshs2(1,isurnr)==20 ) then

!           --- Surface is of type sphere, use the surface from which it must
!               be copied

               isurnr = kmeshs2(2,isurnr)
               changed = .true.

            end if
            if ( jnew) then

!           --- jnew = true   new mesh

               if ( changed ) then
                  ifirstparm = 0
               else
                  ifirstparm = ifirst-isurnr
               end if
               call msh018 ( iinput(iinp3), isurnr, ifirstparm, jinp)
               itype  = iinput(jinp+iinp3)
               if ( itype==0 ) go to 100   ! skip empty surface
               if ( itype==24 ) itype = 7
               npointlc = abs(kmeshh(1,isurnr+1))
               index = kmeshh(4,isurnr+1)
               if ( index<=0 ) then

!              --- index<0, hence surface has not been treated yet

                  if ( itype==7 .or. itype==11 .or. itype==12 ) then

!                 --- itype = 7, 11 or 12  surface of subsurfaces
!                     Perform a loop over the subsurfaces

                     ih = kmeshr(1+isurnr)
                     nsubsf = kmeshr(isurnr+1) - ih
                     do isubsf = 1, nsubsf
                        isursf = abs(kmeshr(ih+isubsf))
                        call msh018 ( iinput(iinp3), isursf, 0, kinp )
                        itype  = iinput(kinp+iinp3)
                        npointlc = kmeshh(1,isursf+1)
                        index = kmeshh(4,isursf+1)
                        if ( index<0 ) then

!                       --- index<0, hence surface has not been treated yet

                           index = - index
                           kmeshh(4,isursf+1) = index

                           if ( kmeshh(3,isursf+1)>0 ) &
                              deallocate ( kmhsur(isursf)%maparsur )
                           allocate ( kmhsur(isursf)%maparsur(npointlc) )
                           kmeshh(3,isursf+1) = npointlc

                        end if

                     end do

                  else

!                 --- itype # 7, 11 or 12  Standard surface

                     kmeshh(4,isurnr+1) = -kmeshh(4,isurnr+1)

                     if ( kmeshh(3,isurnr+1)>0 ) &
                        deallocate ( kmhsur(isurnr)%maparsur )
                     allocate ( kmhsur(isurnr)%maparsur(npointlc) )
                     kmeshh(3,isurnr+1) = npointlc

                  end if

               end if

            end if

100      end do

      end do

      iinp = iinp6

      do ivolnr = 1, nvolel
         ifirst = iinput(iinp+2)
         ilast  = iinput(iinp+3)
         iinp   = iinp+4
         do j = ifirst,ilast
            jns = j+nsurfs
            kmeshh(4,jns+1) = -kmeshh(4,jns+1)

            if ( jnew ) then

!           --- jnew = true   new mesh

               npointlc = kmeshh(1,jns+1)

!              --- Create array mapar

               if ( kmeshh(3,jns+1)>0 ) &
                  deallocate ( kmhsur(jns)%maparsur )
               allocate ( kmhsur(jns)%maparsur(npointlc) )
               kmeshh(3,jns+1) = npointlc

            end if

         end do

      end do
      if ( (jnew .or. nvolms>0) .and. itrans==0 .and. &
           irefin==0 ) then

!     --- Write array mapar for surfaces that not have been treated yet

         call mshact010

      end if

!     --- Reset kmeshh

      do i = 1, nsurfs
         kmeshh(4,i+1) = abs( kmeshh(4,i+1) )
      end do

      call erclos ( 'mshactmpr' )
      end
