      subroutine msharr ( coor, t1, t2, coor3d, npoint, bcord, &
                          kbndpt, nbndpt )
! ======================================================================
!
!        programmer    guus segal
!        version  3.0  date 02-09-2013 Extra parameters kbndpt, nbndpt
!        version  2.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  2.0  date 10-02-1985
!
!   copyright (c) 1984-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     map two-dimensional array coor into three-dimensional array coor3d
!     with the aid of the transformation as stored in t1 and t2
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nbndpt, kbndpt(nbndpt)
      double precision coor(2,*), t1(3), t2(3), coor3d(3,npoint), &
                       bcord(3,nbndpt)

!     bcord          i    array containing the coordinates of the boundaries
!     coor           i    contains co-ordinates of nodes in 2-d plane
!     coor3d         o    array containing the co-ordinates in 3-d plane
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     npoint         i    Number of nodal points
!     t1             i    First  tangential vector
!     t2             i    Second tangential vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nodp, i
      double precision eta, xi

!     eta            eta coordinate of node
!     i              Counting variable
!     nodp           Nodal point sequence number
!     xi             xi coordinate of node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msharr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msharr'
         call prinrl ( t1, 3, 't1' )
         call prinrl ( t2, 3, 't2' )
         call prinrl1 ( coor, npoint, 2, 'coor' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Map coordinates from 2d to 3d

      do nodp = 1,npoint
         xi=coor(1,nodp)
         eta=coor(2,nodp)
         coor3d(:,nodp)=xi*t1+eta*t2+bcord(:,1)
      end do  ! nodp = 1,npoint

!     --- put bcord back into coor3d

      do i = 1, nbndpt
         nodp = kbndpt(i)
         coor3d(:,nodp) = bcord(:,i)
      end do  ! i = 1, nbndpt

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( coor3d, npoint, 3, 'coor3d' )
         write(irefwr,*) 'End msharr'

      end if  ! ( debug )
      call erclos ( 'msharr' )

      end
