      subroutine mshc04 ( solve, inum, ndim, arc, amat, rhs1, rhs2, &
                          rhs3, iinput, coor, alpha )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 2.1    date   16-04-1996 Sander de Snoo: update
!        version 2.0    date   17-11-1995 Sander de Snoo: complete revision
!        version 1.0    date   25-06-1989
!
!
!   copyright (c) 1989-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine spline through points given in array IINPUT in case
!     of a closed contour
! **********************************************************************
!
!                       KEYWORDS
!
!     spline
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision solve(*), arc(*), amat(*), rhs1(*), rhs2(*), &
                       rhs3(*), coor(*), alpha
      integer inum, ndim, iinput(*)

!     alpha   i    spline parameter
!     amat    o    matrix array
!     arc     o    arc length array
!     coor   i/o   coordinate array
!     iinput  i    information array
!     inum    i    number of basis points for spline-curve
!     ndim    i    Dimension of the space.
!     rhs1    o    right hand side array
!     rhs2    o    right hand side array
!     rhs3    o    right hand side array
!     solve   o    solution array
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dx,dy,dz,x,y,z,xe,ye,ze,hs,hi1,hi,c1,c2,cf
      double precision x1,x2,x3,y1,y2,y3,z1,z2,z3,cx,cy,cz,bb,d1
      double precision rx,ry,rz
      integer i,ipoint

!     bb      local bbar value
!     c1      coefficient
!     c2      coefficient
!     cf      coefficient
!     cx      help  coefficient
!     cy      help coefficient
!     cz      help coefficient
!     d1      array of length modecl x modecl contaning the values of the
!             derivatives of the basisfunctions in the x-direction
!     dx      distance in x-direction
!     dy      distance in y-direction
!     dz      distance in z-direction
!     hi      local arc length
!     hi1     local arc length
!     hs      step length
!     i       Counting variable
!     ipoint  nodal point number
!     rx      help variable
!     ry      help variable
!     rz      help variable
!     x       local x co-ordinate
!     x1      local x-co-ordinate
!     x2      local x-co-ordinate
!     x3      local x-co-ordinate
!     xe      x co-ordinate
!     y       local y co-ordinate
!     y1      local y co-ordinate
!     y2      local y co-ordinate
!     y3      local y co-ordinate
!     ye      y co-ordinate
!     z       local z co-ordinate
!     z1      local z co-ordinate
!     z2      local z co-ordinate
!     z3      local z co-ordinate
!     ze      z co-ordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  Error messages
!     INSTOP  Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     908   Spline contour not closed
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine the system of equations, in all internal points using
!     the fact that the solution is periodic.
!     Solve system of equations by direct elimination , use the profile
!     of the mainly three-diagonal matrix, and find the coefficients of
!     the Spline on each subinterval.
!     See for the complete method the technical description.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Place coordinates in array solve

      do 100 i=1,inum

          ipoint = iinput(i+2) - 1

          solve(        i ) = coor( ndim * ipoint + 1 )
          solve( 4*inum+i ) = coor( ndim * ipoint + 2 )
          if ( ndim.eq.3 ) solve( 8*inum+i ) = coor( ndim * ipoint + 3 )

100   continue

      i  = iinput(3) - iinput(2+inum)
      if ( i.ne.0 ) then
         call eropen ( 'mshc04' )
         call errsub ( 908, 0, 0, 0 )
         call erclos ( 'mshc04' )
         call instop
      end if

      hs = 1000d0

      do 200 i = 1 , inum - 1

          x = solve(            i )
          y = solve( 4 * inum + i )
          if(ndim.eq.3) z = solve( 8 * inum + i )

          xe = solve(            i + 1 )
          ye = solve( 4 * inum + i + 1 )
          if(ndim.eq.3) ze = solve( 8 * inum + i + 1 )

          dx = x - xe
          dy = y - ye
          if (ndim.eq.3 ) then
              dz = z - ze
          else
              dz = 0
          end if

          arc(i) = sqrt( dx*dx + dy*dy + dz*dz )

          if ( arc(i)**alpha.lt.hs ) hs = arc(i)**alpha

200   continue

      hs = hs / 1000.

!     --- Determine system of equations

      do 300 i = 1 , inum - 1

          if ( i.eq.1 ) then
               hi1 = arc(inum-1)**alpha
          else
               hi1 = arc(i-1)**alpha
          end if

          hi  = arc(i  )**alpha

          c1 = ( hi1*hi1 - hs*hs ) / ( 3*hi1 )

          c2 = ( hi *hi  - hs*hs ) / ( 3*hi  )

          cf =   hi1 + hi - c1 - c2

          amat(3*i-2) = c1
          amat(3*i-1) = cf
          amat(3*i  ) = c2

          if ( i.eq.1 ) then
               x1  =  solve(inum-1)
          else
               x1  =  solve( i - 1 )
          end if

          x2  =  solve( i     )
          x3  =  solve( i + 1 )

          rhs1(i) = (x3-x2)/hi - (x2-x1)/hi1

          if ( i.eq.1 ) then
               y1  =  solve( 5*inum     - 1 )
          else
               y1  =  solve( 4*inum + i - 1 )
          end if

          y2  =  solve( 4*inum + i     )
          y3  =  solve( 4*inum + i + 1 )

          rhs2(i) = (y3-y2)/hi - (y2-y1)/hi1

          if ( ndim.eq.3 ) then

               if ( i.eq.1 ) then
                    z1 = solve( 9*inum - 1 )
               else
                    z1  =  solve( 8*inum + i - 1 )
               end if

               z2  =  solve( 8*inum + i     )
               z3  =  solve( 8*inum + i + 1 )

               rhs3(i) = (z3-z2)/hi - (z2-z1)/hi1

          end if

300   continue

!     --- Solve sytem of equations in two sweeps
!         First sweep

      do 400 i = 2 , inum - 2

          cf = amat(3*i-2) / amat(3*i-4)

          amat(3*i-2) = -cf * amat(3*i-5)
          amat(3*i-1) = amat(3*i-1)-cf*amat(3*i-3)

          rhs1(i)     = rhs1(i) - cf*rhs1(i-1)
          rhs2(i)     = rhs2(i) - cf*rhs2(i-1)
          if ( ndim.eq.3 ) rhs3(i)     = rhs3(i) - cf*rhs3(i-1)

400   continue

      amat(3*inum-8) = amat(3*inum-6) + amat(3*inum-8)
      amat(3*inum-6) = 0

!     --- Last row

      do 410 i = 1 , inum - 3

          cf = amat(3*inum-3) / amat(3*i-1)

          amat(3*inum-4) = amat(3*inum-4) - cf * amat(3*i-2)
          amat(3*inum-3) = -cf * amat(3*i)
          rhs1(  inum-1) = rhs1(  inum-1) - cf * rhs1(  i  )
          rhs2(  inum-1) = rhs2(  inum-1) - cf * rhs2(  i  )
          if ( ndim.eq.3 ) &
               rhs3(  inum-1) = rhs3(  inum-1) - cf * rhs3(  i  )

410   continue

      amat(3*inum-3) = amat(3*inum-5) + amat(3*inum-3)

!     --- Two equations in two unknowns

      cf = amat(3*inum-3) / amat(3*inum-7)
      amat(3*inum-4) = amat(3*inum-4) - cf * amat(3*inum-8)
      rhs1( inum-1 ) = rhs1( inum-1 ) - cf * rhs1( inum-2 )
      rhs2( inum-1 ) = rhs2( inum-1 ) - cf * rhs2( inum-2 )
      if ( ndim.eq.3 ) &
           rhs3( inum-1 ) = rhs3( inum-1 ) - cf * rhs3( inum-2 )

!     --- Compute first c - values

      cx                = rhs1( inum-1 ) / amat(3*inum-4)
      cy                = rhs2( inum-1 ) / amat(3*inum-4)
      if ( ndim.eq.3 ) &
           cz           = rhs3( inum-1 ) / amat(3*inum-4)

      solve( 3*inum-1 ) = cx
      solve( 7*inum-1 ) = cy
      if (ndim.eq.3) solve(11*inum-1 ) = cz

      solve( 3*inum-2 ) = ( rhs1(inum-2)-amat(3*inum-8)*cx ) / &
                            amat(3*inum-7)
      solve( 7*inum-2 ) = ( rhs2(inum-2)-amat(3*inum-8)*cy ) / &
                            amat(3*inum-7)
      if ( ndim.eq.3 ) &
           solve(11*inum-2 ) = ( rhs3(inum-2)-amat(3*inum-8)*cz ) / &
                                 amat(3*inum-7)

!     --- Compute remaining values

      do 420 i = 1 , inum - 3

          rx = rhs1(inum-2-i)-amat(3*inum-6-3*i)*solve(3*inum-1-i)- &
                              amat(3*inum-8-3*i)*cx
          solve( 3*inum-2-i ) = rx / amat(3*inum-7-3*i)

          ry = rhs2(inum-2-i)-amat(3*inum-6-3*i)*solve(7*inum-1-i)- &
                              amat(3*inum-8-3*i)*cy
          solve( 7*inum-2-i ) = ry / amat(3*inum-7-3*i)

          if ( ndim.eq.3 ) then

          rz = rhs3(inum-2-i)-amat(3*inum-6-3*i)*solve(11*inum-1-i)- &
                              amat(3*inum-8-3*i)*cz
          solve(11*inum-2-i ) = rz / amat(3*inum-7-3*i)

          end if

420   continue

!     --- Boundary conditions

      solve( 3*inum) = solve( 2*inum+1)
      solve( 7*inum) = solve( 6*inum+1)
      if (ndim.eq.3) solve(11*inum) = solve(10*inum+1)

!     --- Compute other coefficients

      do 600 i = 2 , inum

        hi = arc(i-1)**alpha
        solve( 3*inum+i-1)=(solve( 2*inum+i)-solve( 2*inum+i-1))/(3*hi)
        solve( 7*inum+i-1)=(solve( 6*inum+i)-solve( 6*inum+i-1))/(3*hi)
        if ( ndim.eq.3 ) &
        solve(11*inum+i-1)=(solve(10*inum+i)-solve(10*inum+i-1))/(3*hi)

600   continue

!     --- Determine bbar and finally the b values

      do 700 i = 2 , inum

          hi = arc(i-1)**alpha

          c1 = solve(2*inum+i-1)
          d1 = solve(3*inum+i-1)
          dx = solve(i)-solve(i-1)

          bb = dx / hi - c1*(hi-hs) - d1*(hi*hi-hs*hs)

          solve(  inum+i-1) = bb - hs*c1 - hs*hs*d1

          c1 = solve(6*inum+i-1)
          d1 = solve(7*inum+i-1)
          dy = solve(4*inum+i)-solve(4*inum+i-1)

          bb = dy / hi - c1*(hi-hs) - d1*(hi*hi-hs*hs)

          solve(5*inum+i-1) = bb - hs*c1 - hs*hs*d1

          if ( ndim.eq.3 ) then

          c1 = solve(10*inum+i-1)
          d1 = solve(11*inum+i-1)
          dz = solve( 8*inum+i)-solve( 8*inum+i-1)

          bb = dz / hi - c1*(hi-hs) - d1*(hi*hi-hs*hs)

          solve(9*inum+i-1) = bb - hs*c1 - hs*hs*d1

          end if

700   continue

      end
