      subroutine mshc06 ( iinput, rinput, nuspnt, ndim, work, tau, &
                          ishape, nelm )
! ======================================================================
!
!        programmer    Sander de Snoo
!        version  1.0  date 14-04-1996
!
!   copyright (c) 1996-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine tau for cspline
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     spline
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision rinput(*),tau(*),work(*)
      integer iinput(*), nuspnt, ndim, ishape, nelm

!     iinput     i    standard input arrays for subroutine
!                     mesh , it is supposed that the first five
!                     positions have been deleted
!     ishape     i    indicator
!                     1  :  linear elements
!                     2  :  quadratic elements
!                     3  :  cubic elements
!     ndim       i    Dimension of the space.
!     nelm       i    number of elements on curve
!     nuspnt     i    Number of user points
!     rinput     i    standard input arrays for subroutine
!                     mesh , it is supposed that the first five
!                     positions have been deleted
!     tau             work array to store the relative arc lengths of
!                     the elements
!     work            Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision edge, stotal, slocal, ntotal, nlocal, dtau, &
              fcorr,x1,x2,y1,y2,z1,z2,edge1,edge2,sbegin,g,ds
      integer naant, itype, iline, ih, n, npoint, i, j1, j2, &
              itau, iseg, j

!     ds         distance in t-coordinate from begin segment
!     dtau       difference of tau values
!     edge       global coarseness parameter
!     edge1      coarseness at begin segment
!     edge2      coarseness at end segment
!     fcorr      correction for difference between coarseness and true size
!     g          (real) number of elements from begin segment to new point
!     i          Counting variable
!     ih         Help variable
!     iline      position of the user points in iinput
!     iseg       loop variable for line segments
!     itau       local position indicator in tau array
!     itype      type of the spline
!     j          Counting variable
!     j1         pointer in array rinput to userpoint begin segment
!     j2         pointer in array rinput to userpoint end segment
!     n          number of spline segments
!     naant      number of user points on spline
!     nlocal     (real) number of elements on a piece
!     npoint     number of points on the spline
!     ntotal     total number of points (real; not rounded)
!     sbegin     s-value of begin point
!     slocal     local arclength
!     stotal     total arclength
!     x1         x-coordinate first point
!     x2         x-coordinate second point
!     y1         y-coordinate first point
!     y2         y-coordinate second point
!     z1         z-coordinate first point
!     z2         z-coordinate second point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      naant = iinput( 6)
      itype = iinput( 5)

      if ( mod(itype,10).eq.4 ) naant = naant - 2

      iline = 6
      ih    = nuspnt * ndim + 2
      edge  = rinput( ih - 1 )

      stotal =  0
      ntotal =  0
      n      =  naant - 1

      y1 = 0d0
      y2 = 0d0

      z1 = 0d0
      z2 = 0d0

!     --- compute total length and number of elements

      do 100 i = 1 , n

!     --- compute length of segment

          j1 = iinput(iline + i )
          j2 = iinput(iline + i + 1 )

          x1 =  rinput(ndim*(j1-1)+1)
          x2 =  rinput(ndim*(j2-1)+1)

          if ( ndim.ne.1 ) then
            y1 = rinput(ndim*(j1-1)+2)
            y2 = rinput(ndim*(j2-1)+2)
            if ( ndim.eq.3 ) then
               z1 = rinput(ndim*(j1-1)+3)
               z2 = rinput(ndim*(j2-1)+3)
            end if
          end if

          slocal = sqrt((x2-x1)**2+(y2-y1)**2+(z2-z1)**2)

!         --- coarseness at begin/end segment

          edge1 = rinput(ih+j1-1)*edge
          edge2 = rinput(ih+j2-1)*edge

!         --- (real) number of elements on segment

          if (abs(edge1/edge2-1d0).ge.epsmac) then
             nlocal = slocal/(edge1-edge2)*log(edge1/edge2)
          else
             nlocal = 2.*slocal/(edge1+edge2)
          end if

          stotal = stotal + slocal

          ntotal = ntotal + nlocal

100   continue

!     --- Line has several segments, determine total number of points

         npoint    = ishape * nelm + 1

!        --- scaling needed to get a integer number of elements

         fcorr     = ntotal/nelm

         tau ( 1 ) = 0d0
         tau ( npoint ) = 1d0

!        --- Several segments in one curve
!            --- Run through all segments

         sbegin = 0d0
         ntotal = 0d0
         itau   = 1+ishape

         do 400 iseg = 1 , naant - 1

!        --- Determine begin and end of segment

            j1 = iinput( iseg + 6 )
            j2 = iinput( iseg + 7 )

            x1 = rinput( ndim*(j1-1) + 1 )
            x2 = rinput( ndim*(j2-1) + 1 )

            if ( ndim.gt.1 ) then

               y1 = rinput( ndim*(j1-1) + 2 )
               y2 = rinput( ndim*(j2-1) + 2 )

               if ( ndim.eq.3 ) then

                  z1 = rinput( ndim*j1 )
                  z2 = rinput( ndim*j2 )

               end if

            end if

!           --- Determine local length

            slocal = sqrt((x2-x1)**2+(y2-y1)**2+(z2-z1)**2)

!           --- Determine coarseness in both points

            edge1 = rinput(ih+j1-1)*edge*fcorr
            edge2 = rinput(ih+j2-1)*edge*fcorr
            if (abs(edge1/edge2-1d0).ge.epsmac) then
               nlocal = slocal/(edge1-edge2)*log(edge1/edge2)
            else
               nlocal = 2.*slocal/(edge1+edge2)
            end if

            i = 1

!           --- Fill tau for segment using exponential varying coarseness

200         continue

            if (int(ntotal+i).gt. &
               (ntotal+nlocal)*(1d0+epsmac)) goto 210

!           --- number of elements from begin segment to new point

              g = int(ntotal+i)-ntotal

!             --- distance from begin segment to new point

              if (abs(edge1/edge2-1d0).ge.epsmac) then
                ds = edge1*nlocal/log(edge2/edge1)* &
                       ((edge2/edge1)**(g/nlocal)-1d0)
              else
                ds = (edge1+0.5*g/nlocal*(edge2-edge1))*g
              end if

              i = i + 1
              tau(itau)=(sbegin + ds)/stotal

              itau = itau + ishape
            goto 200

210         continue

            ntotal = ntotal + nlocal
            sbegin = sbegin + slocal

400      continue

         if ( ishape.gt.1 ) then

!        --- ishape > 1, Fill points in between according to isubd
!             The relative division is stored in work

            do 380 i = 1 , npoint-1 , ishape
               dtau   = tau(i+ishape)-tau(i)
               do 370 j = 1, ishape-1
                  tau(i+j) = tau(i) + dtau * work(j+1)
370            continue
380         continue

          end if

      end
