      subroutine mshcheckelms2
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 20-05-2005 Fortran 90
!        version  1.1  date 10-11-2004 Extra print
!        version  1.0  date 03-04-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if all faces in the three-dimensional mesh correspond to 2 elements
!    faces may only be connected to one element if all nodes are on the
!    boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     element
!     node
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iface, nelem_face, inode1, ipstart, nnodes
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     iface          face sequence number
!     inode1         First node in face
!     ipstart        Starting address in array
!     iwork          integer work array of length npoint
!                    iwork(i) = 0 means that a node is not on a curve
!                    iwork(i) = 1 means that a node is on a curve
!     nelem_face     number of elements corresponding to face
!     nnodes         Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2237   face connected to more than two elements
!    2265   Hanging node found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize parameters
!     Clear iwork
!     Set iwork = 1 for all points in surfaces
!     For all faces do
!        nelem_face := number of elements connected to the face
!        if ( nelem_face>2 ) then
!           error: too many elements
!        else if ( nelem_face=1 ) then
!           if ( not all nodes on the boundary ) then
!              error: hanging node found
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcheckelms2' )

!     --- Initialize parameters

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear iwork

      iwork = 0

!     --- Set iwork = 1 for all points in a surface

      length = 2*nsurfs
      do i = 1, nsurfs
         length = length+kmeshn(i)*kmeshn(i+nsurfs)
      end do
      do i = 2*nsurfs+1, length
         if ( kmeshn(i)>0 ) iwork(kmeshn(i)) = 1
      end do

!     --- Loop over the faces
!         if ( nelem_face>2 ) then
!           error: too many elements
!         else if ( nelem_face=1 ) then
!           if ( not both nodes on the boundary ) then
!              error: hanging node found

      do iface = 1, nfaces

!     --- nelem_face := number of elements connected to the face

         ipstart = nmeshf(iface)
         nelem_face = nmeshf(ipstart)
         if ( nelem_face==1 ) then

!        --- nelem_face=1, find all nodes

            ipstart = nmeshg(iface)
            nnodes = nmeshg(ipstart)

            do i = 1, nnodes

               inode1 = nmeshg(ipstart+i)
               if ( iwork(inode1)==0 ) then

!              --- Not all nodes on the boundary

                  call errint ( iface, 1 )
                  call errint ( inode1, 2 )
                  call errsub ( 2265, 2, 0, 0 )

               end if

            end do
            if ( ierror==2265 ) &
              call prinin ( nmeshg(ipstart+1), nnodes, 'nodes of face' )

         else if ( nelem_face>2 ) then

!        --- nelem_face>2: error

            call errint ( iface, 1 )
            do i = 1, nelem_face
               call errint ( nmeshf(ipstart+i), 1+i )
            end do
            call errsub ( 2237, 1+nelem_face, 0, 0 )

         end if

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'mshcheckelms2' )
      end
