      subroutine mshcopymesh ( kmeshold, kmeshnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-09-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy mesh from one mesh array into the other and remove
!     old mesh
!     It is assumed that kmeshold and kmeshnew have the same structure
!     and that kmeshold has been filled before
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshold(*), kmeshnew(*)

!     kmeshnew       i    New array kmesh
!     kmeshold      i/o   Old array kmesh that must be reused to store the
!                         contents of kmeshnew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ndim, kelmt, lenkmesh, lenkmesht

!     i              Counting variable
!     kelmt          Indicates if array KMESH part t has been filled >0, &
!                    or not 0
!                    If kelmt>0, kelmt denotes the starting address of
!                    KMESH part t in KMESH
!     lenkmesh       Size of array kmesh2d
!     lenkmesht      length of array kmesht
!     ndim           Dimension of the space.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCOPREM      Copy newentry in oldentry
!     MSHDELKMESHH   Delete kmesh part h and subarrays
!     MSHDELSEL      Delete a selected set of subarrays of kmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcopymesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcopymesh'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Delete part of kmesh old, including parts in ibuffr

      call mshdelsel ( kmeshold )

!     --- Copy parts of kmeshnew into kmeshold

      kmeshold(4:13) = kmeshnew(4:13)

!     --- kmeshc..e (includes parts in ibuffr)

      do i = 17, 19
         if ( kmeshnew(i)/=kmeshold(i) ) &
            call mshcoprem ( kmeshnew(i), kmeshold(i) )
      end do  ! i = 17, 19

      if ( kmeshnew(22)/=kmeshold(22) ) then

!     --- Remove kmesh part h

         call mshdelkmeshh
         kmeshold(22) = kmeshnew(22)

      end if  ! ( kmeshnew(22)/=kmeshold(22) )

!     --- kmeshi..rinput

      do i = 23, 35
         if ( kmeshnew(i)/=kmeshold(i) ) &
            call mshcoprem ( kmeshnew(i), kmeshold(i) )
      end do  ! i = 23, 35

!     --- kmesht

      kelmt = kmeshnew(36)
      ndim = kmeshnew(6)
      lenkmesht = kmeshnew(10+ndim)*2  ! length of kmesht
      kmeshold(kelmt:kelmt+lenkmesht-1) = &
         kmeshnew(kelmt:kelmt+lenkmesht-1)

      kmeshold(37:44) = kmeshnew(37:44)

!     --- kmeshv..x

      do i = 45, 47
         if ( kmeshnew(i)/=kmeshold(i) ) &
            call mshcoprem ( kmeshnew(i), kmeshold(i) )
      end do  ! i =  45, 47

!     --- Copy rest of kmeshnew

      lenkmesh = kmeshnew(3)
      kmeshold(48:lenkmesh) = kmeshnew(48:lenkmesh)

1000  call erclos ( 'mshcopymesh' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcopymesh'

      end if  ! ( debug )

      end
