      subroutine mshd3 ( coor, kmeshc, ipoly, jpoly, lpoly, nr, &
                         izwaar, itria, npt, nhelp, coarse, rim )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 5.0    date   30-12-1995 New call to mshd03
!        version 4.1    date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 4.0    date   27-08-1986
!        version 4.1    date   17-02-1989 Deletion of superfluous parameters
!
!
!
!   copyright (c) 1983-1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!                         DESCRIPTION
!
!    subroutine for the creation of triangles in an arbitrary
!    polygon
! ********************************************************************
!
!                      KEYWORDS
!
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      implicit none
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      integer kmeshc(*), ipoly(*), jpoly(*), lpoly(*), nr(*), izwaar(*), &
              itria, npt, nhelp
      double precision coor(2,*), coarse(*), rim

!     c          i
!     coarse     i    Array containing the coarseness per point
!     coor       o    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ipoly      i
!     itria      i
!     izwaar     i
!     jpoly      i
!     kmeshc     o    kmesh part c, contains the nodal points corresponding to
!                     the elements
!     lpoly      i
!     nhelp      i
!     npt        i    total number of points in the mesh
!     nr         i
!     rim        i    length of smallest edge along the boundary
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer ipolc, ipoln, lpolc, jpoln, n, i, j, i1, i2, i3, &
              ih, ind_j
      double precision  cs
      double precision :: work(lnwrk2)

!     cs         cosine of angle
!     i          Counting variable
!     i1
!     i2
!     i3
!     ih         Help variable to store a constant temporarily
!     ind_j
!     ipolc      counting variable for ipoly
!     ipoln      length of ipoly
!     j          Counting variable
!     jpoln      length of jpoly
!     lpolc      counting variable for lpoly
!     n          length of polygon
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     MSHD01    Divides a concave polygon in two new but smaller polygons
!     MSHD03    Divides a convex polygon into two new but smaller convex
!               polygons
!     MSHG01    Compute the cosine of an angle
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      call eropen ( 'mshd3' )

!     --- first the polygon is checked for "concave" points .
!         if there are any the polygon is divided in subpolygons until
!         only "convex" polygons are left .
!         start cycle "concave"

      ipoln=0
      ih = 2*nhelp+1

!     --- do while (lpoly(1).gt.0)

100   continue

        jpoln=0
        lpolc=1

!       --- do while (lpoly(lpolc).gt.0)

110     continue

!       --- n gives length of polygon to be considered

          n=lpoly(lpolc)

          call mshd01(lpoly,jpoly,ipoly,lpolc,jpoln,ipoln,coor,nr, &
                      izwaar,npt,coarse,rim,work(ih))
          lpolc=lpolc+1+n

          if (lpoly(lpolc).gt.0) goto 110

!         --- lpoly(lpolc) = 0 , cycle through lpoly ready .
!             check whether jpoly is empty

        if ( jpoln.gt.0 ) then

!       --- jpoln not zero, &
!           there is still the possibility of concave points
!           refill lpoly with the contents of jpoly

          do 120 i=1,jpoln
            lpoly(i)=jpoly(i)
120       continue
          lpoly(jpoln+1)=0

        end if

        if (jpoln.gt.0) goto 100

!     --- ipoly has been filled and
!          ipoln gives the total length of array ipoly

      ipoly(ipoln+1)=0

!     --- fill cosine array (first time only):

      j=1

!     --- do while( ipoly(j).gt.0 )

200   continue

        n=ipoly(j)

        do 210 i=1,n
          nr(i+1)=ipoly(j+i)
210     continue

        nr(1)=nr(n+1)
        nr(n+2)=nr(2)

        do 220 i=1,n
            i1=nr(i)
            i2=nr(i+1)
            i3=nr(i+2)
            call mshg01(coor(1,i1),coor(2,i1),coor(1,i2),coor(2,i2), &
                        coor(1,i3),coor(2,i3),cs)
            work(j+i)=cs
220     continue

        j=j+1+n

        if ( ipoly(j).gt.0 ) goto 200

!     --- ipoly(j)=0 no polygons left, &
!         start creation of triangles

      ind_j = 4*nhelp+1
      ih = 2*nhelp+1

!     --- do while ( ipoly(1).gt.0 )

300   continue

        ipolc=1
        jpoln=0

!       --- do while ( ipoly(ipolc).gt.0 )

310     continue

!       --- divide convex polygon

          call mshd03(coor,kmeshc,itria,ipoly,ipolc,nr,work,jpoly,jpoln, &
                      work(ind_j),work(ih),npt,coarse)
          ipolc=ipolc+ipoly(ipolc)+1

          if ( ipoly(ipolc).gt.0 ) goto 310

!       --- array ipoly is empty , consider jpoly

        if ( jpoln.gt.0 ) then

!       --- jpoly not empty , next cycle

          do 320 i=1,jpoln
            ipoly(i)=jpoly(i)
            work(i)=work(ind_j+i-1)
320       continue
          ipoly(jpoln+1)=0

        end if
        if ( jpoln.gt.0 ) goto 300

!     --- jpoly empty , hence division ready
!         all polygons in triangles

      call erclos ( 'mshd3' )

      end
