      subroutine mshd7 ( kmeshc, coor, nelem, ndim )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.2  date 15-11-2006 debug statements
!        version  2.1  date 01-12-2002 New formula
!        version  2.0  date 04-05-2002 Complete revision
!
!   copyright (c) 2002-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put coordinates of extra centre point in quadratic triangle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     centre
!     coordinate
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, ndim, kmeshc(7,*)
      double precision coor(ndim,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     ndim           i    Dimension of the space.
!     nelem          i    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nod1, nod2, nod3, nod4, nod5, nod6, nodc, j
      double precision oneninth

!     i              Counting variable
!     j              Counting variable
!     nod1           Node number of first node in triangle
!     nod2           Node number of second node in triangle
!     nod3           Node number of third node in triangle
!     nod4           Node number of fourth node in triangle
!     nod5           Node number of fifth node in triangle
!     nod6           Node number of sixth node in triangle
!     nodc           Node number of centroid in tetrahedron
!     oneninth       1/9
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all elements ielem do
!        Compute coordinates of centroid
!        Put coordinates into inpelm-th point of element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshd7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshd7'
         call prinin1 ( kmeshc, nelem, 7, 'kmeshc' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      oneninth = 1d0/9d0

!     --- For all elements ielem do

      do i = 1, nelem
         nod1 = kmeshc(1,i)
         nod2 = kmeshc(2,i)
         nod3 = kmeshc(3,i)
         nod4 = kmeshc(4,i)
         nod5 = kmeshc(5,i)
         nod6 = kmeshc(6,i)
         nodc = kmeshc(7,i)

!        --- Compute coordinates of centroid
!            Put coordinates into inpelm-th point of element

         do j = 1, ndim
            coor(j,nodc) = oneninth*(-coor(j,nod1)-coor(j,nod3) &
                                     -coor(j,nod5)+ &
                           4d0*(coor(j,nod2)+coor(j,nod4)+coor(j,nod6)))
         end do

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( coor, nodc, ndim, 'coor' )
         write(irefwr,*) 'End mshd7'

      end if  ! ( debug )
      call erclos ( 'mshd7' )

      end
