      subroutine mshfilvl ( ivolnr, irefc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 15-11-2012 Long integers
!        version  2.0  date 01-08-2001 Extra parameter ibuffr,
!                                      new definition kmesh part h
!        version  1.2  date 03-02-2001 Extension for empty volumes
!
!   copyright (c) 1996-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Store contents of volume in file 3 and references in kmesh part h
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivolnr, irefc

!     irefc          i    dataset reference number for reading (>0 formatted, &
!                         <0 unformatted)
!     ivolnr         i    Volume sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, npointlc, istart, iend
      double precision, allocatable :: work(:,:)
      integer, allocatable :: iwork(:), mapar(:), maparinv(:)

!     i              Counting variable
!     iend           Last position used in array
!     istart         First position used  in array
!     iwork               integer work array of length inpelm*inelem
!     k              Counting variable
!     mapar          i    mapping array: contains the mapping from local to
!                         global numbering of the nodal points
!     maparinv            Work array of length npoint (local) containing the
!                         inverse of mapar
!     npointlc       Local number of points
!     work                Work array to store the local co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     DCOPY          Copy double precision dx to double precision dy
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     SEPMSHEXTKT    Compute starting and ending address of elements in kmeshc
!                    by using the information stored in kmesh and kmesht
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read array mapar
!     Compute the inverse of mapar
!     Store local co-ordinates in work
!     Write local co-ordinates to file 3
!     ielem1 := kmesht(1,ivolnr)
!     ielem2 := kmesht(2,ivolnr)
!     Compute starting address of element ielem1 in kmeshc
!     Compute end address of element ielem2 in kmeshc
!     Store local element numbers in iwork
!     write local kmeshc to file 3
!     write array mapar to file 3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Read array mapar

      npointlc = kmeshh(1,nsurfs+ivolnr+1)

      allocate ( iwork(nelem*npelm), mapar(npointlc), &
                 maparinv(npoint), work(ndim,npointlc), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npointlc*2+npoint+nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Read array mapar

      if ( irefc>0 ) then
         read(irefc,*) (mapar(i),i=1,npointlc)
      else
         read(-irefc) (mapar(i),i=1,npointlc)
      end if

!     --- Compute the inverse of mapar

      if ( mapar(1)>0 ) then

!     --- Standard situation, the surface has been filled

         do i = 1, npointlc
            maparinv(mapar(i)) = i
         end do

      else

!     --- The surface has not been filled

         do i = 1, npointlc
            maparinv(i) = i
            mapar(i) = i
         end do

      end if

!     --- Store local co-ordinates in work

      do i = 1, npointlc
         do k = 1, ndim
            work(k,i) = coor(k,mapar(i))
         end do
      end do

!     --- Write local co-ordinates to file 3

      call dcopy ( ndim*npointlc, work, 1, kmhsur(nsurfs+ivolnr)%coorsur, 1 )

!     --- Compute starting address of element ielem1 in kmeshc
!         Compute end address of element ielem2 in kmeshc

      call sepmshextkt ( ivolnr, istart, iend )

!     --- Store local element numbers in iwork

      do i = istart, iend
         iwork(i-istart+1) = maparinv(kmeshc(i))
      end do

!     --- write local kmeshc to file 3

      i = iend-istart+1
      kmhsur(nsurfs+ivolnr)%kmeshcsur(1:i) = iwork(1:i)

!     --- write array mapar to file 3

      kmhsur(nsurfs+ivolnr)%maparsur(1:npointlc) = mapar(1:npointlc)

      deallocate ( iwork, work, mapar, maparinv, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  end subroutine mshfilvl
