      subroutine mshiinpext ( iinput, rinput, krenum, kcoars, &
                              iinputnw, rinputnw, ichois, lniinp, &
                              lnrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 25-11-2012 Long integers
!        version  3.0  date 30-07-2012 New parameter list
!        version  2.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend arrays iinput and rinput with extra curves and surfaces
!     The reason is that a complete surface is splitted into subparts
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, iinput(*), krenum(*), kcoars, &
              iinputnw(*), lniinp, lnrinp
      double precision rinput(*), rinputnw(*)

!     ichois     i    Choise parameter JCHOIS in subroutine MESH
!                     Possible values
!                     0  standard method,  input from standard input file
!                        the mesh is created for the first time
!                     1  input is read from the arrays iinput and rinput
!                        the mesh is created for the first time
!                     2  input is read from the standard input file and stored
!                        in the arrays iinput and rinput
!                        the mesh is created for the first time
!                     3  input is read from the arrays iinput and rinput
!                        the co-ordinates of the existing mesh are changed
!                        the topological description remains unchanged
!                     4  input is read from the arrays iinput and rinput.
!                        the coordinates and the topological description of the
!                        mesh are changed, however the input as stored in iinput
!                        and rinput is the same as in a preceding call of mesh.
!                        The number of points and curves remains the same, as
!                        well as the number of elements along the curves.
!                        The coordinates of the user points and the curves are
!                        changed by a call of subroutine chanbn and as a
!                        consequence the number elements in the surfaces may be
!                        changed.
!                        This possibility must be preceded by calls of
!                        subroutine mesh and subroutine chanbn
!                     5  See jchois = 4, however, the topological description
!                        remains unchanged
!                     6  Only the arrays iinput and rinput are filled
!                        The subroutine is returned without actually creating a
!                        mesh
!                     7  The input must be stored in the arrays IINPUT and
!                        RINPUT
!                        The subroutine creates the curves and stores the
!                        information of curves and user points via KMESH
!                        After this step the subroutine returns
!                        This possibility is meant for program SEPGEOM
!                    10  The boundary of the mesh has been changed by subroutine
!                        ADAPBN. Information with respect to this change
!                        has been stored in the arrays ICURVS, CURVES,IINPUT
!                        and RINPUT, which are all part of array KMESH
!                        The input/output parameters IINPUT and RINPUT are not
!                        used.
!                        The topological description of the mesh may be changed
!                    11  See ICHOIS=10, however, the topological description
!                        may not be changed
!     iinput     i    integer input array, standard for subroutine MESH
!                     The array is filled as defined from position 6
!     iinputnw   o    Adapted (or copied) new array iinput
!     kcoars     i    If 1 coarseness is used
!     krenum     o    Array of length 10 with information about the
!                     renumbering.
!                     Array must be filled as follows:
!                      1: if>=0, only position 1 is filled.
!                         This position contains the parameter method according
!                         to the old SEPRAN definition, i.e.
!                         krenum = 9: Use default depending on mesh
!                         else:
!                         krenum = ispecial*100 + method + 3 * ipref
!                         ispecial = 0: standard
!                                    1: ivertx=1, imidp=1, icentr=3
!                         If<0, -krenum(1) contains the sequence number
!                         of the last entry filled by the user
!                         All other values are filled with defaults
!                      2: method Indication of the renumbering method used
!                                The algorithms are compared to the original
!                                numbering:
!                                0  No renumbering
!                                1  Cuthill-Mckee algorithm
!                                2  Sloan algorithm
!                                3  Both algorithms,  the best is used
!                                9  Choose depending on mesh
!                         Default: 9
!                      3: ipref  Indication of the criterion to be used to find
!                                the best algorithm:
!                                0  Choose the smallest profile
!                                1  Choose the smallest band width
!                                2  Use only the renumbered algorithm
!                                   (method=1 or 2)
!                         Default: 0
!                      4: iprint Indicates if information about the renumbering
!                                must always be printed (1) or only depending on
!                                ioutp (0)
!                         Default: 0
!                      5: ivertx Priority for vertex nodes (1-3)
!                         Default: 3
!                      6: imidp  Priority for mid point nodes (1-3)
!                         Default: 3
!                      7: icentr Priority for centroids (1-3)
!                         Default: 3
!                      8: ilevel Indicates if renumbering of specia nodes must
!                                be performed per level (1) or not (0)
!                         Default: 0
!                      9: istmet Indicates how the start of the renumbering
!                                algorithm must be performed
!                                0  Automatic
!                                1  Start user point given
!                                2  Start curve given
!                                3  Start surface given
!                         Default: 0
!                     10: istval Value of start point, curve f surface
!                         Default: 1
!     lniinp     i    Available space for array IINPUT
!     lnrinp     i    Available space for array RINPUT
!     rinput     i    real input array, standard for subroutine MESH
!                     The array is filled as defined from position 6
!     rinputnw   o    Adapted (or copied) new array rinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lniinpextra, lenpnt, lnrinpextra, ncurvsext, &
              nsurfsext, nuspntext, nextrasrfssav
      integer, allocatable :: infcurves(:,:), infsurfaces(:,:)
      double precision, allocatable :: cooruserp(:,:), factors(:), &
                                       ccoarse(:), ctangent(:,:)

!     ccoarse        Array containing the coarseness of the user points
!     cooruserp      Array containing the co-ordinates of the user points
!                    x_i = coorusrp(1,i);  y_i = coorusrp(2,i)
!     ctangent       Array containing the factors t_0 and t_1 of the curves
!                    t_0_i = ctangent(1,i);  t_1_i = ctangent(2,i)
!     factors        Array containing the factors of the curves
!     infcurves      Array containing the information of the curves
!                    infcurves(1,i) refers to curve i
!     infsurfaces    Array containing the information of the surfaces
!                    infsurfaces(1,i) refers to surface i
!     lenpnt         Number of entries in local point type arrays
!     lniinpextra    Extra length of array iinput in integers
!     lnrinpextra    Extra length of array rinput in reals
!     ncurvsext      Number of curves including extra curves
!     nextrasrfssav  Help parameter to save the value of nextrasrfs
!     nsurfsext      Number of surfaces including extra surfaces
!     nuspntext      Number of user points including extra user points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSH000         Fill some parameters from array IINPUT
!     MSHNEWINP1     Extend parts of local subarrays of iinput and rinput with
!                    extra information
!     MSHNEWINP2     Extend parts of local subarrays of iinput and rinput with
!                    extra information
!     MSHNEWSURF     Check if new surfaces, curves and user points must be
!                    created because a surface actually consists of subparts
!     PLAXIS01       Split arrays iinput and rinput into various parts
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     fill some parameters from array iinput
!     Check if there are surfaces that must be created by new curves and
!     surfaces
!     If ( new curves must be created ) then
!        split iinput, rinput into subarrays
!        Store which parts are mapped
!        Create new arrays iinput and rinput
!        fill some parameters from array iinput
!     Create mesh
!     If ( new curves have been created ) then
!        Remove extra user points, curves and surfaces
!        Map subarrays of kmesh back to correct size
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshiinpext' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshiinpext'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nextrasrfssav = nextrasrfs

!     --- fill some parameters from array iinput

      jcoars = kcoars
      call msh000 ( iinput, krenum, ichois )

!     --- Check if there are surfaces that must be created by new curves and
!         surfaces

      if ( iinp3>0 ) then

!     --- iinp3>0, hence there are surfaces
!         At this moment only in case of a sphere
!         extra points, curves and surfaces are created

          call mshnewsurf ( iinput )

      end if

!     --- new curves must be created

      nuspntext = nuspnt+nextrapnts
      ncurvsext = ncurvs+nextracrvs
      nsurfsext = nsurfs+nextrasrfs
      lenpnt = 3*nuspntext
      lencrv = 3*ncurvsext
      length = ndim*lenpnt+maxlencur*lencrv+maxlensur*nsurfsext+ &
               lencrv+lenpnt+2*lencrv

      allocate ( cooruserp(ndim,lenpnt), infcurves(maxlencur,lencrv), &
                 infsurfaces(maxlensur,nsurfsext), factors(lencrv), &
                 ccoarse(lenpnt), ctangent(2,lencrv), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'cooruserp' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'ndim, nuspntext, ncurvsext, nsurfsext', &
                          ndim, nuspntext, ncurvsext, nsurfsext
         write(irefwr,1) 'nextrapnts, nextracrvs, nextrasrfs', &
                          nextrapnts, nextracrvs, nextrasrfs
         write(irefwr,1) 'maxlencur, maxlensur', maxlencur, maxlensur
      end if  ! ( debug )

!     --- Split arrays iinput and rinput into various parts

      call plaxis01 ( iinput, rinput, cooruserp, infcurves, infsurfaces, &
                      ccoarse, factors, ctangent, lenpnt, nsurfsext )

!     --- Create new arrays iinput and rinput
!         First extend local arrays

      call mshnewinp1 ( cooruserp, infcurves, infsurfaces, &
                        ccoarse, factors, ctangent, &
                        lniinpextra, lnrinpextra, lenpnt, nsurfsext )

!     --- Check array length

      call inichkarlen ( iinp8+lniinpextra, lniinp, 'iinputnw' )
      call inichkarlen ( irnp3+lnrinpextra, lnrinp, 'rinputnw' )
      if ( ierror/=0 ) go to 1000

!     --- Copy information to new arrays iinput and rinput

      call mshnewinp2 ( cooruserp, infcurves, infsurfaces, &
                        ccoarse, factors, ctangent, iinput, &
                        rinput, iinputnw, rinputnw, lenpnt, nsurfsext )

      deallocate ( cooruserp, infcurves, infsurfaces, factors, &
                   ccoarse, ctangent, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'cooruserp' )
      nextrasrfs = nextrasrfssav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshiinpext'

      end if  ! ( debug )
      call erclos ( 'mshiinpext' )

      end
