      subroutine mshmpr ( iinput, infel, lengc, lengpm, lengmx, jnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version 14.1  date 12-03-2015 Extra debug
!        version 14.0  date 02-09-2010 Remove intarmsh
!        version 13.1  date 18-08-2008 Extra debug statements
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill the arrays mapar, kmesh part g, kmeshl, kmeshm, kmeshn
!     raise inodp and iselem
!     a part of array infel is filled
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical jnew
      integer iinput(*), lengmx, lengc, lengpm, infel(*)

!     iinput     i    integer input array
!     infel      i    array containing some information of how the elements
!                     created are stored on file 3
!     jnew       i    indication whether the mesh is new (true) or old
!     lengc     i/o   computed length of kmesh part c
!     lengmx     i    Maximum length of local array kmeshc
!     lengpm     i    computed length of kmesh part m
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indsur, i, iselemsav
      integer, allocatable :: iwork(:), mapar(:), kbndpt(:)

!     i              Counting variable
!     indsur         Position in array infel
!     iselemsav      Help variable to store value of iselem
!     iwork          work array of length
!                    max ( 2 * npomxs, nelmax + 2*max(ncurvs,nsurfs) )
!     kbndpt         array containing the nodal point numbers of the boundary
!     mapar          mapping array: contains the mapping from local to
!                    global numbering of the nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH010         Fill array mapar with the global numbers for surfaces not
!                    used as standard elements
!     MSH019         fill inner boundaries in kmesh part m
!     MSHMPRSUR      fill the arrays mapar, kmeshl, kmeshm for surface elements
!                    in case surface elements are connected to an element group
!     MSHMPRSUREL    Fill array infel and raise kmeshg for surface elements
!                    in case surface elements are connected to an element group
!     MSHMPRVOL      fill the arrays mapar, kmesh part g, kmeshl, kmeshm, kmeshn
!                    raise inodp and iselem
!                    a part of array infel is filled
!                    Volume elements only
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set some parameters

      call eropen ( 'mshmpr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmpr'
  1      format ( a, 1x, (10i6) )

      end if

      indsur = 5
      if ( nvolms==0 ) npomax = npomxs
      if ( debug ) &
          write(irefwr,1) 'nsurfs, nvolms, itrans, npomax', &
                           nsurfs, nvolms, itrans, npomax

!     --- First set index positions in kmeshh with respect to kbndpt arrays
!         negative in order to know if a surface has been filled (>0)
!         or not (<0)

      allocate ( iwork(nsurfs+nvolms), mapar(npomax), &
                 kbndpt(nbnmax+lengmx), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nsurfs+nvolms+npomax+nbnmax+lengmx, 'iwork' )
      if ( ierror/=0 ) go to 1000

      do i  = 1, nsurfs+nvolms
         kmeshh(4,i+1) = -kmeshh(4,i+1)
         iwork(i) = kmeshh(4,i+1)
      end do

      if ( debug ) call prinin ( iwork, nsurfs+nvolms, 'iwork' )

!     --- Set iselem

      iselemsav = iselem
      call mshmprsurel ( iinput, infel, jnew, indsur )
      if ( debug ) write(irefwr,*) 'iselem/1', iselem

      do i  = 1, nsurfs+nvolms
         kmeshh(4,i+1) = iwork(i)
      end do

!     --- Fill surface elements

      iselem = iselemsav
      call mshmprsur ( iinput, lengc, kbndpt, mapar, jnew )
      if ( debug ) write(irefwr,*) 'iselem/2', iselem

!     --- Fill volume elements

      call mshmprvol ( iinput, infel, lengc, kbndpt, &
                       mapar, jnew, indsur )
      if ( debug ) then
         write(irefwr,1) 'inodp, iselem', inodp, iselem
         call prinin ( mapar, inodp-1, 'mapar/2' )
      end if  ! ( debug )

      if ( (jnew .or. nvolms>0) .and. itrans==0 .and. &
           irefin==0 ) then

!     --- Write array mapar for surfaces that not have been treated yet

         call msh010 ( mapar )

      end if

!     --- Reset kmeshh

      do i = 1, nsurfs
         kmeshh(4,i+1) = abs( kmeshh(4,i+1) )
      end do

      if ( debug ) then
         write(irefwr,1) 'npoint, nelem', inodp-1, iselem-1
         call prinin ( mapar, inodp-1, 'mapar/3' )
      end if  ! ( debug )

      if ( jnew ) then

!     --- jnew is true, fill internal curves in kmeshm

         call msh019 ( iinput, lengpm )
         nelem = iselem-1
         npoint = inodp-1

      end if

      if ( kelmg>0 ) then

!     --- Kelmg > 0, kmesh part g exists

         Topgrp(:)%inelem = kmeshg(:)

      else

!     --- Kelmg = 0, kmesh part g does not exist

         Topgrp(:)%inelem = nelem

      end if  ! ( kelmg>0 )

      deallocate ( iwork, mapar, kbndpt, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then
         write(irefwr,1) 'npoint, nelem', npoint, nelem
         call prinin ( kmeshl, nuspnt, &
                       'nodal point numbers of user points' )
         call pr0008 ( kmeshm, ncurvs )
         write(irefwr,*) 'End mshmpr'
      end if  ! ( debug )
      call erclos ( 'mshmpr' )

      end
