      subroutine mshnewinp1 ( coorusr, infcur, infsur, &
                              ccoarse, factors, ctangent, &
                              lniinpextra, lnrinpextra, lenpnt, lensurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.1  date 31-12-2001 New input for isopar
!        version  1.0  date 16-07-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend parts of local subarrays of iinput and rinput with extra
!     information
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lniinpextra, lenpnt, &
              lensurfs, lnrinpextra, infcur(maxlencur,lencrv), &
              infsur(maxlensur,lensurfs)
      double precision coorusr(ndim,lenpnt), ccoarse(lenpnt), &
                       factors(lencrv), ctangent(2,lencrv)

!     ccoarse        o    Array containing the coarseness of the user points
!     coorusr        o    Array containing the co-ordinates of the user points
!                         x_i = coorusr(1,i);  y_i = coorusr(2,i)
!     ctangent       o    Array containing the factors t_0 and t_1 of the curves
!                         t_0_i = ctangent(1,i);  t_1_i = ctangent(2,i)
!     factors        o    Array containing the factors of the curves
!     infcur         o    Array containing the information of the curves
!                         infcur(1,i) refers to curve i
!     infsur         o    Array containing the information of the surfaces
!                         infsur(1,i) refers to surface i
!     lenpnt         i    Number of entries in local point type arrays
!     lensurfs       i    Number of entries in local surface type arrays
!     lniinpextra    o    Extra length of array iinput in integers
!     lnrinpextra    o    Extra length of array rinput in reals
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurve, j, ic, &
              icurnrnew, ifirst, ishift, isubcv, ncurvsnew, &
              nsubcv, nsurfsnew, nuspntnew, icurnrcircle, &
              ndefsf, jsurnr, ksurnr, iseq, nrow
      logical part4, extracurves

!     extracurves    If true extra curves must be created
!     i              Counting variable
!     ic             Relative subcurve number
!     icurnrcircle   Curve number of circle
!     icurnrnew      New curve sequence number
!     icurve         Type of curve
!     ifirst         First curve (surface) number read
!     iseq           Sequence number
!     ishift         Shift
!     isubcv         Subcurve sequence number with direction
!     j              Counting variable
!     jsurnr         Surface sequence number
!     ksurnr         absolute value of surface sequence number
!     ncurvsnew      Last curve sequence number used
!     ndefsf         Number of curves in surface
!     nrow           Number of rows of surfaces in ordened surface of
!                    surfaces
!     nsubcv         Number of subcurves of a curve
!     nsurfsnew      Last surface sequence number used
!     nuspntnew      Last user point sequence number used
!     part4          If true paver will be called
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2085   Curve number not positive
!    2086   Point number defining centre of sphere not positive
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For all surfaces do
!       if ( isurf = 20 ) then
!          extend arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshnewinp1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshnewinp1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'nuspnt, ncurvs, nsurfs', &
                          nuspnt, ncurvs, nsurfs

      nuspntnew = nuspnt
      ncurvsnew = ncurvs
      nsurfsnew = nsurfs
      lniinpextra = 0
      lnrinpextra = 0
      nextracrvs = 0
      extracurves = .true.

!     --- Check if paver is used

      part4 = .false.
      do isurnr = 1, nsurfs
         if ( infsur(1,isurnr)==19 ) part4 = .true.
      end do

!     --- For all surfaces do

      do isurnr = 1, nsurfs

         isurf = infsur(1,isurnr)

         if (debug ) write(irefwr,1) 'isurnr, isurf', isurnr, isurf

         if ( isurf==20 ) then

!        --- isurf = 20, sphere found

            itypesurf = infsur(6,isurnr)
            if (debug ) write(irefwr,1) 'itypesurf', itypesurf

!           --- First we define the extra points

            if ( itypesurf==1 ) then

!           --- Complete sphere
!               first 5 extra points, three on the circle and 2 on top and below
!               The coordinates are computed later on

               nextrapnts = 5

            else if ( itypesurf==2 ) then

!           --- Upper half sphere
!               first 4 extra points, three on the circle and 1 on top
!               The coordinates are computed later on

               nextrapnts = 4

            else

!           --- Lower half sphere
!               first 4 extra points, three on the circle and 1 below
!               The coordinates are computed later on

               nextrapnts = 4

            end if

            coorusr(:,nuspnt+1:nuspnt+nextrapnts) = 0d0
            ccoarse(nuspnt+1:nuspnt+nextrapnts) = 1d0
            lnrinpextra = lnrinpextra+(ndim+jcoars)*nextrapnts

!           --- Next the extra curves
!               The first 4 curves are parts of the circle
!               They get the following information:
!               icurve (18), ishape, nelm (now 0), icurnr (from circle)
!               Sub sequence number (i), first point, last point, icentre

            icurnrcircle = infsur(4,isurnr)

            if ( debug ) write(irefwr,1) 'icurnrcircle', icurnrcircle

            if ( icurnrcircle<=0 ) then

!           --- error curve number not positive

               call errint ( 1, 1 )
               call errint ( isurnr, 2 )
               call errint ( icurnrcircle, 3 )
               call errchr ( 'SPHERE', 1 )
               call errsub ( 2085, 3, 0, 1 )
               go to 1000

            end if

!           --- Find centre of sphere

            icentre = infsur(5,isurnr)
            if ( debug ) write(irefwr,1) 'icentre', icentre

            if ( icentre==0 ) then

!           --- icentre = 0, check if the centre can be detected from the
!               circle

               icurve = infcur(1,icurnrcircle)
               if ( debug ) write(irefwr,1) 'icurve', icurve

               if ( icurve==17 ) then

!              --- circle detected

                  icentre = infcur(5,icurnrcircle)

               else if ( icurve==7 ) then

!              --- curve of curves detected, check if one of them is of
!                  type arc, and find centre

                  nsubcv = infcur(4,icurnrcircle)
                  if ( debug ) write(irefwr,1) 'nsubcv', nsubcv

                  if ( nsubcv==4 ) then

!                 --- The curve already exists of 4 subcurves, hence no
!                     extra curves are needed

                     nextrapnts = nextrapnts-3  ! the first three are not
                                                  ! necessary
                     extracurves = .false.        ! no extra curves

                  end if  ! ( nsubcv==4 )

                  do ic = 1, nsubcv

                     isubcv = abs(infcur(4+ic,icurnrcircle))
                     if ( debug ) &
                        write(irefwr,1) 'ic, isubcv, infcur(1,isubcv)', &
                                         ic, isubcv, infcur(1,isubcv)

                     if ( infcur(1,isubcv)==3 .or. &
                          infcur(1,isubcv)==6 ) then

!                    --- arc found

                         icentre = abs(infcur(7,isubcv))

                         exit

                     end if

                  end do

               end if

            end if

            if ( debug ) then
               write(irefwr,1) 'icentre', icentre
               write(irefwr,*) 'extracurves', extracurves
            end if  ! ( debug )

            if ( icentre<=0 ) then

!           --- Error: icentre = 0

               call errint ( icentre, 1 )
               call errint ( isurnr, 2 )
               call errsub ( 2086, 2, 0, 0 )
               go to 1000

            end if

            ishape = infcur(2,icurnrcircle)
            ifirst = infcur(6,icurnrcircle)

            if ( debug ) &
               write(irefwr,1) 'ishape, ifirst', ishape, ifirst

            if ( extracurves ) then

!           --- Extra curves needed

               do i = 1, 4

                  infcur(1,ncurvsnew+i) = 18
                  infcur(2,ncurvsnew+i) = ishape
                  infcur(3,ncurvsnew+i) = 0
                  infcur(4,ncurvsnew+i) = icurnrcircle
                  infcur(5,ncurvsnew+i) = i
                  if ( i==1 ) then
                     infcur(6,ncurvsnew+i) = ifirst
                  else
                     infcur(6,ncurvsnew+i) = nuspntnew+i-1
                  end if
                  if ( i==4 ) then
                     infcur(7,ncurvsnew+i) = ifirst
                  else
                     infcur(7,ncurvsnew+i) = nuspntnew+i
                  end if
                  infcur(8,ncurvsnew+i) = icentre

                  if ( debug ) then
                     write(irefwr,1) 'i, ncurvsnew+i', i, ncurvsnew+i
                     call prinin ( infcur(1,ncurvsnew+i), 8, 'infcur' )
                  end if  ! ( debug )

               end do
               nextracrvs = 4
               lniinpextra = lniinpextra+4*8

            end if  ! ( extracurves )

            if ( itypesurf/=3 ) then

!           --- itypesurf = 1, 2: 4 curves to above
!               These curves are of type 19, since the upper point as well as
!               the coarseness must be detected
!               They get the following information:
!               icurve (19), ishape, nelm (now 0), icurnr (circle)
!               icurnr (first subcurve of circle)
!               direction (1), first point, last point, icentre

               ishape = infcur(2,icurnrcircle)
               ifirst = infcur(6,icurnrcircle)
               do i = 1, 4
                  icurnrnew = ncurvsnew+i+nextracrvs
                  infcur(1,icurnrnew) = 19     ! type of curve
                  infcur(2,icurnrnew) = ishape ! shape of elements
                  infcur(3,icurnrnew) = 0      ! nelm
                  infcur(4,icurnrnew) = icurnrcircle  ! icurnr (circle)
                  infcur(6,icurnrnew) = 1      ! direction
                  infcur(9,icurnrnew) = icentre
               end do  ! i = 1, 4

               if ( extracurves ) then

!              --- Circle is 1 curve

                  do i = 1, 4

                     icurnrnew = ncurvsnew+i+nextracrvs
                     infcur(5,icurnrnew) = ncurvsnew+1! first subcurve of circle
                     infcur(7,icurnrnew) = infcur(6,ncurvsnew+i) ! first point
                     infcur(8,icurnrnew) = nuspntnew+4 ! last point

                     if ( debug ) then
                        write(irefwr,1) 'i, icurnrnew', i, icurnrnew
                        call prinin ( infcur(1,icurnrnew), 9, 'infcur' )
                     end if  ! ( debug )

                  end do  ! i = 1, 4

               else

!              --- Circle consists of 4 subcurves

                  do i = 1, 4

                     infcur(5,icurnrnew) = 0 ! first subcurve of circle
                     icurnr = abs(infcur(4+i,icurnrcircle)) ! subcurve
                     infcur(7,icurnrnew) = infcur(6,icurnr) ! first point
                     infcur(8,icurnrnew) = nuspntnew+1 ! last point

                     if ( debug ) then
                        write(irefwr,1) 'i, icurnrnew', i, icurnrnew
                        call prinin ( infcur(1,icurnrnew), 9, 'infcur' )
                     end if  ! ( debug )

                  end do  ! i = 1, 4

               end if  ! ( extracurves )

               nextracrvs = nextracrvs+4
               lniinpextra = lniinpextra+4*9

            end if

            if ( itypesurf/=2 ) then

!           --- itypesurf = 1, 3: 4 curves to below
!               These curves are of type 19, since the lower point as well as
!               the coarseness must be detected
!               They get the following information:
!               icurve (19), ishape, nelm (now 0), &
!               icurnr (first subcurve of circle)
!               direction (-1), first point, last point, icentre

               ishape = infcur(2,icurnrcircle)
               ifirst = infcur(6,icurnrcircle)
               do i = 1, 4
                  icurnrnew = ncurvsnew+i+nextracrvs
                  infcur(1,icurnrnew) = 19
                  infcur(2,icurnrnew) = ishape
                  infcur(3,icurnrnew) = 0
                  infcur(4,icurnrnew) = icurnrcircle
                  infcur(5,icurnrnew) = ncurvsnew+1
                  infcur(6,icurnrnew) = -1
                  infcur(7,icurnrnew) = infcur(6,ncurvsnew+i)
                  infcur(8,icurnrnew) = nuspntnew+nextrapnts
                  infcur(9,icurnrnew) = icentre

                  if ( debug ) then
                     write(irefwr,1) 'i, icurnrnew', i, icurnrnew
                     call prinin ( infcur(1,icurnrnew), 9, 'infcur' )
                  end if  ! ( debug )
               end do
               nextracrvs = nextracrvs+4
               lniinpextra = lniinpextra+4*9

            end if  ! ( itypesurf/=2 )

            if ( extracurves ) then

!           --- The last extra curve is a curve of curves defined by the
!               4 subparts of the circle

               nextracrvs = nextracrvs+1
               lniinpextra = lniinpextra+8
               icurnrnew = ncurvsnew+nextracrvs
               infcur(1,icurnrnew) = 7
               infcur(2,icurnrnew) = ishape
               infcur(3,icurnrnew) = 0
               infcur(4,icurnrnew) = 4
               do i = 1, 4
                  infcur(4+i,icurnrnew) = ncurvsnew+i
               end do
               infsur(8,isurnr) = icurnrnew

!              --- The curve number corresponding to the original curve
!                  must be replaced in all surfaces by the new one

               do i = 1, nsurfs
                  isurf = infsur(1,i)
                  if ( isurf/=7 .and. isurf/=11 .and. isurf/=12 .and. &
                       isurf/=20 ) then

!                 --- there are subcurves to be changed

                     ndefsf = infsur(3,i)
                     do j = 1, ndefsf
                        icurnr = infsur(3+j,i)
                        if ( icurnr==icurnrcircle ) &
                           infsur(3+j,i) = icurnrnew
                     end do
                  end if
               end do

            end if  ! ( extracurves )

!           --- Finally we define the extra surfaces

            if ( itypesurf/=1 ) then

!           --- itypesurf = 2, 3: surface through the circle plane
!               This surface is created by triangle
!               The following information is stored
!               isurf (3), ishape, ndefsf(1), icurnr

               if ( infsur(10,isurnr)/=0 ) then

!              --- surface number given by the user

                  jsurnr = infsur(10,isurnr)

               else

!              --- surface number not given by the user

                  nsurfsnew = nsurfsnew+1
                  jsurnr = nsurfsnew
                  infsur(10,isurnr) = jsurnr
                  lniinpextra = lniinpextra+4

               end if
               infsur(1,jsurnr) = 3
               infsur(2,jsurnr) = infsur(2,isurnr)
               infsur(3,jsurnr) = 1
               infsur(4,jsurnr) = icurnrcircle

            end if
            if ( itypesurf/=3 ) then

!           --- itypesurf = 1, 2: 4 upper surfaces
!               This surface is created by isopar
!               The following information is stored
!               isurf (18), ishape, ndefsf(3), 3 curve numbers, mapping (1), &
!               icentre

               ksurnr = nsurfsnew
               do i = 1, 4
                  nsurfsnew = nsurfsnew+1
                  infsur(1,nsurfsnew) = 18
                  infsur(2,nsurfsnew) = infsur(2,isurnr)
                  infsur(3,nsurfsnew) = 3
                  infsur(4,nsurfsnew) = ncurvsnew+i
                  infsur(5,nsurfsnew) = ncurvsnew+i+5
                  infsur(6,nsurfsnew) = -(ncurvsnew+i+4)
                  infsur(7,nsurfsnew) = 1
                  infsur(8,nsurfsnew) = icentre
                  infsur(9,nsurfsnew) = 0
               end do
               infsur(5,nsurfsnew) = ncurvsnew+i
               lniinpextra = lniinpextra+4*9

!              --- Make an extra surface of the 4 subsurfaces

               if ( infsur(9,isurnr)/=0 ) then

!              --- surface number given by the user

                  jsurnr = infsur(9,isurnr)
                  lniinpextra = lniinpextra+6

               else

!              --- surface number not given by the user

                  nsurfsnew = nsurfsnew+1
                  jsurnr = nsurfsnew
                  infsur(9,isurnr) = jsurnr
                  lniinpextra = lniinpextra+7

               end if
               infsur(1,jsurnr) = 7
               infsur(2,jsurnr) = infsur(2,isurnr)
               infsur(3,jsurnr) = 4
               do i = 1, 4
                  infsur(3+i,jsurnr) = ksurnr+i
               end do

            end if
            if ( itypesurf/=2 ) then

!           --- itypesurf = 1, 3: 4 lower surfaces
!               This surface is created by isopar
!               The following information is stored
!               isurf (18), ishape, ndefsf(3), 3 curve numbers, mapping (1), &
!               icentre

               if ( itypesurf==1 ) then
                  ishift = 4
               else
                  ishift = 0
               end if
               ksurnr = nsurfsnew
               do i = 1, 4
                  nsurfsnew = nsurfsnew+1
                  infsur(1,nsurfsnew) = 18
                  infsur(2,nsurfsnew) = infsur(2,isurnr)
                  infsur(3,nsurfsnew) = 3
                  infsur(4,nsurfsnew) = ncurvsnew+i
                  infsur(5,nsurfsnew) = ncurvsnew+i+5+ishift
                  infsur(6,nsurfsnew) = -(ncurvsnew+i+4+ishift)
                  infsur(7,nsurfsnew) = 1
                  infsur(8,nsurfsnew) = icentre
                  infsur(9,nsurfsnew) = 0
               end do
               infsur(5,nsurfsnew) = ncurvsnew+i+ishift
               lniinpextra = lniinpextra+4*9

!              --- Make an extra surface of the 4 subsurfaces

               if ( itypesurf==3 ) then
                  iseq = 9
               else
                  iseq = 10
               end if

               if ( infsur(iseq,isurnr)/=0 ) then

!              --- surface number given by the user

                  jsurnr = infsur(iseq,isurnr)
                  lniinpextra = lniinpextra+6

               else

!              --- surface number not given by the user

                  nsurfsnew = nsurfsnew+1
                  jsurnr = nsurfsnew
                  infsur(iseq,isurnr) = jsurnr
                  lniinpextra = lniinpextra+7

               end if
               infsur(1,jsurnr) = 7
               infsur(2,jsurnr) = infsur(2,isurnr)
               infsur(3,jsurnr) = 4
               do i = 1, 4
                  infsur(3+i,jsurnr) = ksurnr+i
               end do

            end if

!           --- Finally combine all surfaces to one

            nsurfsnew = nsurfsnew+1
            infsur(7,isurnr) = nsurfsnew
            infsur(1,nsurfsnew) = 7
            infsur(2,nsurfsnew) = 0
            infsur(3,nsurfsnew) = 2
            infsur(4,nsurfsnew) = infsur(9,isurnr)
            infsur(5,nsurfsnew) = infsur(10,isurnr)
            lniinpextra = lniinpextra+5
            nextrasrfs = nsurfsnew-nsurfs

!           --- Replace old surface in all surface of surface records by new one

            do i = 1, nsurfs
               isurf = infsur(1,i)
               if ( isurf==7 .or. isurf==11 .or. isurf==12 ) then

!              --- there are subsurfaces to be changed

                  if ( isurf==7 ) then
                     ndefsf = infsur(3,i)
                     nrow = 0
                  else
                     nrow = infsur(3,i)
                     ndefsf = 0
                     do j = 1, nrow
                        ndefsf = ndefsf+infsur(3+j,i)
                     end do
                  end if
                  do j = 1, ndefsf
                     jsurnr = infsur(3+j+nrow,i)
                     if ( jsurnr==isurnr ) &
                        infsur(3+j+nrow,i) = nsurfsnew
                  end do

               end if

            end do

            do i = ncurvsnew, ncurvsnew+nextracrvs
               factors(i) = 0d0
               ctangent(1,i) = 0d0
               ctangent(2,i) = 0d0
            end do
            lnrinpextra = lnrinpextra+3*nextracrvs+4*nextrasrfs
            if ( part4 ) lnrinpextra = lnrinpextra+4*nextracrvs
            ncurvsnew = ncurvsnew+nextracrvs
            nuspntnew = nuspntnew+nextrapnts

         end if

      end do
      nuspnt = nuspntnew
      ncurvs = ncurvsnew
      nsurfs = nsurfsnew

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshnewinp1'

      end if  ! ( debug )
      call erclos ( 'mshnewinp1' )

      end
