      subroutine msho63 ( isurf, coor1, coor2, number )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.1  date 19-10-1993 Stop execution if points are
!                                      different)
!        version  1.0  date 30-09-1993
!
!   copyright (c) 1993-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compare the coordinates of two points that should be the same
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurf, number
      double precision coor1(3), coor2(3)

!    coor1    i    coordinate array for first point
!    coor2    i    coordinate array for second point
!    isurf    i    number of surface considered
!    number   i    corner to be compared
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision e1, e2, e3

!     e1      accuracy in x
!     e2      accuracy in y
!     e3      accuracy in z
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   Deconcatenate name subroutine from string
!     EROPEN   Concatenate name of subroutine to string of names
!     ERRINT   Place integer in errormessage array
!     ERRSUB   Submit error message
!     INSTOP   Stop execution SEPRAN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     956     Coordinates of submitted points are not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msho63' )

      e1 = sqreps * ( abs(coor2(1)) + abs(coor1(1)) )
      e2 = sqreps * ( abs(coor2(2)) + abs(coor1(2)) )
      e3 = sqreps * ( abs(coor2(3)) + abs(coor1(3)) )

      if ( e1.lt.epsmac ) e1 = sqreps
      if ( e2.lt.epsmac ) e2 = sqreps
      if ( e3.lt.epsmac ) e3 = sqreps

      if ( abs(coor2(1) - coor1(1) ).gt.e1 .or. &
           abs(coor2(2) - coor1(2) ).gt.e2 .or. &
           abs(coor2(3) - coor1(3) ).gt.e3 ) then

!     --- Points do not coincide : fatal error

           call errint( number, 1 )
           call errint( isurf , 2 )

           call errsub ( 956, 2, 0, 0 )
           call instop

      end if

      call erclos ( 'msho63' )

      end
