      subroutine msho71 ( npx, npy, kelem, nelem, npelm )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 1.0    date   27-06-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     MSHO71 constructs a triangular or quadrilateral grid if only the
!     nodes in a domain are given in a structured way.
!     It is assumed that the points are given in NPY rows of NPX points
!     the first row in the x-direction is point 1 to point NPX, &
!     the second row is NPX+1 till 2*NPX and so on.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     2d
!     structured
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer npx, npy, kelem(*), nelem, npelm

!     kelem   o    array containing the nodes of the triangles.
!                  length of this array npelm * nelem.
!     nelem   i    number of elements to be created
!     npelm   i    number of points in element
!                  3 : linear triangle
!                  4 : linear quadrangle
!     npx     i    number of points on lines in x-direction
!     npy     i    number of points on lines in y-direction
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, i3, i4, ielem, jc, jlm, jr

!     i1           first node number of local quadrangle
!     i2           second node number of local quadrangle
!     i3           third node number of local quadrangle
!     i4           fourth nodal numbers of local quadrangle
!     ielem        temporary number of elements
!     jc           column number considered
!     jlm          position in array kelem to be filled
!     jr           row number considered
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    deconcatenate name from string of calling subroutines
!     EROPEN    concatenate name to string of calling subroutines
!     INSTOP    stop execution of SEPRAN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through lines parallel x-direction
!
!     Create triangles or quadrangles in kelem
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msho71' )

      ielem = 0

!     --- Run through all rows

      do 200 jr = 1 , npy - 1

!     --- Run through all columns

          do  100 jc = 1 , npx - 1

!         --- Determine four corner nodes

               i1 = ( jr-1 ) * npx + jc
               i2 = i1 + 1
               i3 = i1 + npx
               i4 = i3 + 1

!              --- Fill elements in kelem

               if ( npelm.eq.3 ) then

                    jlm = 3 * ielem

                    kelem( jlm + 1 ) = i1
                    kelem( jlm + 2 ) = i2
                    kelem( jlm + 3 ) = i3
                    kelem( jlm + 4 ) = i2
                    kelem( jlm + 5 ) = i4
                    kelem( jlm + 6 ) = i3

                    ielem = ielem + 2

               else if ( npelm.eq.4 ) then

                    jlm = 4 * ielem

                    kelem( jlm + 1 ) = i1
                    kelem( jlm + 2 ) = i2
                    kelem( jlm + 3 ) = i4
                    kelem( jlm + 4 ) = i3

                    ielem = ielem + 1

               end if

100       continue

200   continue

      if ( nelem.ne.ielem ) then

           write( *, *) ' Number of elements expected was ',nelem
           write( *, *) ' Number of elements found was ',ielem
           write( *, *) ' Error, execution stopped '
           write( *, *) ' Please, inform SEPRA '
           call instop

      end if

      call erclos( 'msho71' )

      end
