      subroutine msho81 ( i, k, coor, nbndpt, ja )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 1.0    date   27-06-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine whether line i - k is permitted.
!     With other words : Belongs lines i - k totally to the area given
!     by the boundary nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     visibility
!     mesh_generation
!     2d
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer i, ja, k, nbndpt
      double precision coor( 2 , * )

!     coor    i    array containing the Euclidian coordinates of the
!                  nodes
!     i       i    first node of line piece to be considered
!     ja      o    indicator : if ja = 1 then point is not visible
!                              if ja = 0 then point is visible
!     k       i    second node of line piece to be considered
!     nbndpt  i    length of actual boundary
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, ih, li
      double precision x1, x2, xh, xk, y1, y2, yh, yk

!     i1      point number of first point of boundary segment
!     i2      point number of second point of boundary segment
!     ih      common point indicator ( 0 : yes, 1 : no )
!     li      loop variable
!     x1      x-coordinate of first point of boundary segment
!     x2      x-coordinate of second point of boundary segment
!     xh      x-coordinate of first point of line under consideration
!     xk      x-coordinate of second point of line under consideration
!     y1      y-coordinate of first point of boundary segment
!     y2      y-coordinate of second point of boundary segment
!     yh      y-coordinate of first point of line under consideration
!     yk      y-coordinate of second point of line under consideration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSHO75  determines whether two line segments have a point in common
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check whether line i-k has any point in common with the boundary
!     lines
!
!     PSEUDO CODE
!
!     Take line segment
!     Do for all points on the boundary
!        Take boundary line from this boundary point to the next
!        If line segment has any point in common with the boundary then
!           Return
!        Endif
!     Enddo
!     Conclude : no points in common
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ja = 1

      xh = coor( 1 , i )
      yh = coor( 2 , i )

      xk = coor( 1 , k )
      yk = coor( 2 , k )

      do 100 li = 1, nbndpt

          i1 = li
          i2 = li + 1
          if ( i2.gt.nbndpt ) i2 = 1

          x1 = coor( 1 , i1 )
          y1 = coor( 2 , i1 )

          x2 = coor( 1 , i2 )
          y2 = coor( 2 , i2 )

          ih = 1

          ja = 1

          if ( k.eq.i1 ) ja = 0
          if ( k.eq.i2 ) ja = 0
          if ( i.eq.i1 ) ja = 0
          if ( i.eq.i2 ) ja = 0

          if ( ja.eq.1 ) then
             call msho75 ( xh, yh, xk, yk, x1, y1, x2, y2, ih )
          end if

          if ( ih.eq.0 ) goto 200

100   continue

!     --- no common point found with all boundary parts

      ja = 0

200   continue

      end
