      subroutine msho85( coor, npoint, kelem, nelem, kbound, nbound )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.1  date 15-09-1996 Check the constructed elements
!        version  1.0  date 28-06-1996
!
!   copyright (c) 1996-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove the extra coordinates and elements in case of an unstructured
!     mesh with artificial constructed boundary and check the elements
!     constructed
! **********************************************************************
!
!                       KEYWORDS
!
!     unstructured
!     coordinate
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nelem, nbound, kelem( 4, nelem), &
              kbound(3,*)
      double precision coor( 3, * )

!     coor   i,o   array with all coordinates of the nodes
!     kbound  o    array with all boundary triangles
!     kelem  i,o   element array
!     nbound  o    number of boundary triangles in kbound
!     nelem  i,o   total number of elements
!     npoint i,o   total number of nodes in array coor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i , ielem, jelem, ibound, i1, i2, i3, i4, j
      double precision volume, vol

!     i       loop variable
!     i1      node number
!     i2      node number
!     i3      node number
!     i4      node number
!     ibound  new number of boundary pieces
!     ielem   loop variable for elements
!     j       loop variable
!     jelem   new number of elements
!     vol     volume of element
!     volume  sum of volume of all elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERREAL    Put real in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INSTOP  stop execution of SEPRAN
!     MSH632  compute volume of tetrahedron
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1647   Wrong element found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Rearrange coor, i.e. cancel the help points
!     Rearrange kelem
!     Determine boundary pieces
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Rearrange coor : (cancel the "help" points)

      do 110 i = 1 , npoint - 8

         do 100 j = 1 , 3

            coor( j, i ) = coor( j, i+8 )

100      continue

110   continue

!     --- Rearrange kelem

      jelem  = 0
      ibound = 0

      do 200 ielem = 1 , nelem

         i1 = kelem( 1, ielem )
         i2 = kelem( 2, ielem )
         i3 = kelem( 3, ielem )
         i4 = kelem( 4, ielem )

         if ( i1.lt.9 .or. i2.lt.9 .or. i3.lt.9 .or. i4.lt.9 ) then

!        --- Cancel element

            if ( i1.gt.8 .and. i2.gt.8 .and. i3.gt.8 ) then

                 ibound = ibound + 1
                 kbound( 1, ibound ) = i1 - 8
                 kbound( 2, ibound ) = i2 - 8
                 kbound( 3, ibound ) = i3 - 8

            else if ( i1.gt.8 .and. i2.gt.8 .and. i4.gt.8 ) then

                 ibound = ibound + 1
                 kbound( 1, ibound ) = i1 - 8
                 kbound( 2, ibound ) = i2 - 8
                 kbound( 3, ibound ) = i4 - 8

            else if ( i1.gt.8 .and. i3.gt.8 .and. i4.gt.8 ) then

                 ibound = ibound + 1
                 kbound( 1, ibound ) = i1 - 8
                 kbound( 2, ibound ) = i3 - 8
                 kbound( 3, ibound ) = i4 - 8

            else if ( i2.gt.8 .and. i3.gt.8 .and. i4.gt.8 ) then

                 ibound = ibound + 1
                 kbound( 1, ibound ) = i2 - 8
                 kbound( 2, ibound ) = i3 - 8
                 kbound( 3, ibound ) = i4 - 8

            end if

         else

!        --- Element is ok, store in new array

            jelem = jelem + 1

            kelem( 1, jelem ) = i1 - 8
            kelem( 2, jelem ) = i2 - 8
            kelem( 3, jelem ) = i3 - 8
            kelem( 4, jelem ) = i4 - 8

         end if

200   continue

      npoint = npoint - 8
      nelem  = jelem
      nbound = ibound

!     --- Compute volumes

      volume = 0d0

      do 500 jelem = 1 , nelem

         i1 = kelem( 1, jelem )
         i2 = kelem( 2, jelem )
         i3 = kelem( 3, jelem )
         i4 = kelem( 4, jelem )

         call msh632( coor, i1, i2, i3, i4, vol )

         if ( vol.lt.0d0 ) then

!        --- Wrong element, give message

            call errint ( i1, 1 )
            call errint ( i2, 2 )
            call errint ( i3, 3 )
            call errint ( i4, 4 )
            call erreal ( vol, 1 )
            call errsub ( 1647, 4, 1, 0 )

         end if

         volume = volume + vol

500   continue

      if ( nerror.gt.0 ) call instop

      end
