      subroutine mshobsfl30 ( nx, ny, nz, iobgrid, nelem, kmeshc, &
                              coor, inpelm, xmin, ymin, zmin, &
                              dx, dy, dz, eps, leniobgrid )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-11-2012 Remove input parameter debug
!        version  2.1  date 21-01-2004 print --> write
!        version  2.0  date 14-11-2001 Extra parameter leniobgrid
!
!   copyright (c) 2001-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iobgrid for all cells that are crossed by an
!     obstacle surface element
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, leniobgrid, inpelm, nelem, &
              kmeshc(inpelm,nelem), &
              iobgrid(0:nx-1,0:ny-1,0:nz-1,leniobgrid)
      double precision xmin, ymin, zmin, dx, dy, dz, coor(3,*), eps

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dx             i    length of vector in x-direction
!     dy             i    length of vector in y-direction
!     dz             i    length of vector in z-direction
!     eps            i    Local accuracy
!     inpelm         i    number of points per element
!     iobgrid       i/o   In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements of the obstacle
!     leniobgrid    i/o   Declared length of last dimension of iobgrid
!     nelem          i    Number of elements
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     xmin           i    Minimum x-value
!     ymin           i    Minimum y-value
!     zmin           i    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxpelm
      parameter ( maxpelm=3 )
      integer i1(maxpelm+1), i2(maxpelm+1), i3(maxpelm+1), ielem, &
              i1max, i1min, i2max, i2min, i3max, i3min, i, j, inodp, &
              leniobgridact
      double precision x(maxpelm+1), y(maxpelm+1), z(maxpelm+1), alpha, &
                       beta, gamma
      logical intern, i1equal, i2equal, i3equal

!     alpha          multiplication factor
!     beta           multiplication factor
!     gamma          multiplication factor
!     i              Counting variable
!     i1             Array containing the "i" values of the vertices
!     i1equal        if true all inidces in i-drection are equal
!     i1max          minimum i-value
!     i1min          maximum i-value
!     i2             Array containing the "j" values of the vertices
!     i2equal        if true all inidces in j-drection are equal
!     i2max          minimum j-value
!     i2min          maximum j-value
!     i3             Array containing the "k" values of the vertices
!     i3equal        if true all inidces in k-drection are equal
!     i3max          minimum k-value
!     i3min          maximum k-value
!     ielem          Element number
!     inodp          Nodal point number
!     intern         if true there may be parts of the triangle that are
!                    internal and correspond to a different cell than the edges
!     j              Counting variable
!     leniobgridact  Actual length of last dimension of iobgrid
!     maxpelm        maximum number of points per element
!     x              Contains the x-coordinates of the vertices
!     y              Contains the y-coordinates of the vertices
!     z              Contains the z-coordinates of the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHOBSFL1      Fills array iobsgrid for one cell
!     MSHOBSFL31     Fill array iobgrid for all edges of an obstacle element in
!                    case the variation in all directions is unequal to zero
!     MSHOBSFL3SF    Fill internal part of one element in the obstacle surfaces
!                    into iobgrid
!     MSHOBSFL3XC    Fill array iobgrid for all edges of an obstacle element in
!                    case the variation in i-drection of all elements is zero
!     MSHOBSFL3YC    Fill array iobgrid for all edges of an obstacle element in
!                    case the variation in j-drection of all elements is zero
!     MSHOBSFL3ZC    Fill array iobgrid for all edges of an obstacle element in
!                    case the variation in k-drection of all elements is zero
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2254   inpelm # 3
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfl30' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsfl30'
         write(irefwr,1) 'inpelm, nelem', inpelm, nelem
         write(irefwr,1) 'nx, ny, nz', nx, ny, nz
         write(irefwr,1) 'leniobgrid', leniobgrid
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inpelm.ne.3 ) then

!     --- Only triangles have been implemented

         call errsub ( 2254, 0, 0, 0 )
         go to 1000

      end if

!     --- For all elements in surface do

      leniobgridact = 0
      do ielem = 1, nelem

!     --- Loop over the nodes in the element

         do j = 1, inpelm

!        --- Get node, Compute x1,y1,z1 and i1, i2 and i3

            inodp = kmeshc(j,ielem)
            x(j) = coor(1,inodp)
            y(j) = coor(2,inodp)
            z(j) = coor(3,inodp)

!           --- alpha := (x1-xmin)/dx;
!               beta  := (y1-ymin)/dy;
!               gamma := (z1-zmin)/dz;
!               i1 := alpha1; i2:= beta1; i3 = gamma;

            alpha = (x(j)-xmin)/dx
            beta  = (y(j)-ymin)/dy
            gamma = (z(j)-zmin)/dz
            i1(j) = min(int(max(alpha,0d0)),nx-1)
            i2(j) = min(int(max(beta ,0d0)),ny-1)
            i3(j) = min(int(max(gamma,0d0)),nz-1)

         end do
         i1(inpelm+1) = i1(1)
         i2(inpelm+1) = i2(1)
         i3(inpelm+1) = i3(1)
         x(inpelm+1) = x(1)
         y(inpelm+1) = y(1)
         z(inpelm+1) = z(1)

         if ( debug ) then

!        --- debug is true, print info

            write(irefwr,1) 'ielem', ielem
            call prinin(kmeshc(1,ielem),inpelm,'kmeshc')
            call prinrl(x,inpelm,'x')
            call prinrl(y,inpelm,'y')
            call prinrl(z,inpelm,'z')
            call prinin(i1,inpelm,'i1')
            call prinin(i2,inpelm,'i2')
            call prinin(i3,inpelm,'i3')

         end if

!        --- Check which of the i's are equal for all elements

         i1min = i1(1)
         i1max = i1(1)
         i2min = i2(1)
         i2max = i2(1)
         i3min = i3(1)
         i3max = i3(1)
         do j = 2, inpelm
            i1min = min(i1min,i1(j))
            i1max = max(i1max,i1(j))
            i2min = min(i2min,i2(j))
            i2max = max(i2max,i2(j))
            i3min = min(i3min,i3(j))
            i3max = max(i3max,i3(j))
         end do
         i1equal = i1min.eq.i1max
         i2equal = i2min.eq.i2max
         i3equal = i3min.eq.i3max

!        --- First mark all corners

         do j = 1, inpelm
            call mshobsfl1 ( nx, ny, nz, iobgrid, i1(j), i2(j), i3(j), &
                             ielem, leniobgrid, leniobgridact )
         end do
         intern = .false.

!        --- Now detect all cells that are crossed

         if ( i1equal .and. i2equal .and. i3equal ) then

!        --- Most simple situation, only one cell; no action necessary

         else if ( i1equal ) then

!        --- All i1's are equal, check i2 and i3

            if ( i2equal ) then

!           --- All i1's and i2's are equal, only the i3'th differ

               do i = i3min, i3max
                  call mshobsfl1 ( nx, ny, nz, iobgrid, i1(1), i2(1), i, &
                                   ielem, leniobgrid, leniobgridact )
               end do

            else if ( i3equal ) then

!           --- All i1's and i3's are equal, only the i2'th differ

               do i = i2min, i2max
                  call mshobsfl1 ( nx, ny, nz, iobgrid, i1(1), i, i3(1), &
                                   ielem, leniobgrid, leniobgridact )
               end do

            else

!           --- All i1's are the same but i2-th and i3-th differ

               call mshobsfl3xc ( nx, ny, nz, iobgrid, ielem, y, z, &
                                  i1, i2, i3, inpelm, ymin, zmin, dy, &
                                  dz, leniobgrid, leniobgridact )
               intern = .true.

            end if

         else if ( i2equal ) then

!        --- All i2's are equal, check i1 and i3

            if ( i3equal ) then

!           --- All i2's and i3's are equal, only the i1'th differ

               do i = i1min, i1max
                  call mshobsfl1 ( nx, ny, nz, iobgrid, i, i2(1), i3(1), &
                                   ielem, leniobgrid, leniobgridact )
               end do

            else

!           --- All i2's are the same but i1-th and i3-th differ

               call mshobsfl3yc ( nx, ny, nz, iobgrid, ielem, x, z, &
                                  i1, i2, i3, inpelm, xmin, zmin, dx, &
                                  dz, leniobgrid, leniobgridact )
               intern = .true.

            end if

         else if ( i3equal ) then

!        --- All i3's are the same but i1-th and i2-th differ

            call mshobsfl3zc ( nx, ny, nz, iobgrid, ielem, x, y, &
                               i1, i2, i3, inpelm, xmin, ymin, dx, &
                               dy, leniobgrid, leniobgridact )
            intern = .true.

         else

!        --- Most complicated situation, all i's are different

            call mshobsfl31 ( nx, ny, nz, iobgrid, ielem, x, y, z, &
                              i1, i2, i3, inpelm, xmin, ymin, zmin, dx, &
                              dy, dz, eps, leniobgrid, leniobgridact )
            intern = .true.

         end if
         if ( intern ) then

!        --- intern = true, fill internal part of surface element

            call mshobsfl3sf ( nx, ny, nz, iobgrid, ielem, x, y, z, &
                               i1, i2, i3, inpelm, xmin, ymin, zmin, &
                               dx, dy, dz, leniobgrid, leniobgridact )

         end if
         if ( ierror.ne.0 ) go to 1000

      end do
      leniobgrid =  leniobgridact

1000  call erclos ( 'mshobsfl30' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobsfl30'

      end if  ! ( debug )

      end
