      subroutine mshobstiin1 ( iinput, rinput, iinputnew, rinputnew, &
                               nspecialpnts, specialpoints, iposrinp, &
                               curves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 15-08-1999
!
!   copyright (c) 1999-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute number of extra user points and fill information from old arrays
!   iinput and rinput to new arrays iinputnew and rinput with respect to
!   user points
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nspecialpnts, specialpoints(7,*), iinput(*), &
              iinputnew(*), iposrinp
      double precision curves(2,*), rinputnew(*), rinput(*)

!     curves         i    array containing the coordinates of the curves
!     iinput         i    integer input array
!     iinputnew      i    ?
!     iposrinp       i    ?
!     nspecialpnts   i    Number of special points
!     rinput         i    Real input array
!     rinputnew      i    ?
!     specialpoints  i    This array of size 3 x nspecialpnts contains
!                         information about the special points on the curve
!                         specialpoints(1,i) contains the position in curves
!                                           of special point i
!                         specialpoints(2,i) contains first curve number in the
!                                           free surface connected to point i
!                         specialpoints(3,i) contains second curve number in the
!                                           free surface connected to point i
!                                           If 0, there is only one curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ipos

!     i              Counting variable
!     ipos           Position in array
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute number of extra user points
!     Copy first parts of iinput and rinput
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute number of extra user points
!         Copy first parts of iinput and rinput

      nextrapnts = 0
      do i = 1, iinp1-1
         iinputnew(i) = iinput(i)
      end do
      do i = 1, irnp1-1
         rinputnew(i) = rinput(i)
      end do
      do i = 1, nuspnt*ndim
         rinputnew(irnp1-1+i) = rinput(irnp1-1+i)
      end do
      iposrinp = irnp1+nuspnt*ndim-1
      do i = 1, nspecialpnts
         if ( specialpoints(7,i)==0 ) then
            nextrapnts = nextrapnts+1
            specialpoints(7,i) = nuspnt+nextrapnts
            ipos = specialpoints(1,i)
            do j = 1, ndim
               rinputnew(iposrinp+j) = curves(j,ipos)
            end do
            iposrinp = iposrinp+ndim
         end if
      end do
      iinputnew(2) = nuspnt+nextrapnts
      if ( jcoars==1 ) then

!     --- Copy coarseness from rinput into rinputnew

         iposrinp = iposrinp+1
         rinputnew(iposrinp) = rinput(irnp1+ndim*nuspnt)
         do i = 1, nuspnt
            rinputnew(iposrinp+i) = rinput(irnp1+ndim*nuspnt+i)
         end do
         iposrinp = iposrinp+nuspnt
         do i = 1, nextrapnts
            rinputnew(iposrinp+i) = 1d0
         end do
         iposrinp = iposrinp+nextrapnts

      end if
      iposrinp = iposrinp+1

      end
