      subroutine mshobstiin3 ( iinput, iinputnew, iposiinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 15-08-1999
!
!   copyright (c) 1999-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Copy parts of iinput with respect to surfaces from input into
!   iinputnew
!   Change contents of the surfaces if necessary
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinputnew(*),  iposiinp

!     iinput         i    integer input array
!     iinputnew      i    ?
!     iposiinp       i    Starting address in array iinput with respect to the
!                         extra user points (internal points)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, i, istart, &
              istartold, nextra, ipos, nshift, isub, isubcv, &
              inpncv, itype, iuserp2, ncopy, ncrvs, &
              istartcurv, nsubcv, numsurfs, numcurvs, iinp, jlast, &
              iwork(nsurfs), ipointsurf(nsurfs), iworkcurve(3,ncurvs), &
              ipointcurvnw(ncurvs), icurvsurface(ncurvs)
      logical changed, first

!     changed        If true an issue has been changed
!     first          If true it concerns the first step
!     i              Counting variable
!     icurvsurface   ?
!     iinp           last position used in array iinput
!     inpncv         Number of points in curve
!     iobst          Obstacle sequence number
!     ipointcurvnw   ?
!     ipointsurf     Array containing the relative starting addresses
!                    of all surfaces in the part of iinput referring
!                    to surfaces
!     ipos           Position in array
!     istart         First position used at a part
!     istartcurv     ?
!     istartold      Old starting address in array
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isubcv         Subcurve sequence number with direction
!     itype          Table containing information about the numbers behind
!                    the keywords in the table code
!     iuserp2        Second user point
!     iwork          integer work array
!     iworkcurve     integer work array
!     jlast          iinput(20) refers to new part in iinput
!     ncopy          ?
!     ncrvs          Number of curves
!     nextra         array containing information about segment endings and
!                    whether segments have already been loaded into the
!                    complist(1, segnum) > 10 or have already been plotted
!                    (2, segnum) > 10.
!     nshift         shift
!     nsubcv         Number of subcurves of a curve
!     numcurvs       Number of curves
!     numsurfs       number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array iwork with information about surfaces that are connected
!     to one of the obstacles (0 not connected, 1 connected)
!     For isurnr := 1 (1) number_of_surfaces do
!        if ( iwork(isurnr)=0 ) then
!           Copy information of surface from iinput to iinputnew
!        else
!           Store all curves that span the surface into icurvsurface (kmeshs)
!           changed := false
!           For all curves in icurvsurface do
!              itype := type of curve in iinputnew
!              if ( itype=composite curve ) then
!                 changed := true
!                 Replace curve number by its subcurves if these
!                 subcurves have 10000 or 10003
!           if ( not changed ) then
!              Copy information of surface from iinput to iinputnew
!           else ( changed )
!              For all curves in icurvsurface do
!                 Store type, initial point and end point in iworkcurve(1-3,.)
!              Construct new surface:
!                 While ( there are still curves to do: )
!                    Initial curve := next curve to do
!                    Find end_point
!                    While ( end_point is point of a remaining curve )
!                       take new curve
!                       Find end_point
!                 Put information in iinputnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill array iwork with information about surfaces that are connected
!         to one of the obstacles (0 not connected, 1 connected)

      iwork = 0
      do iobst = 1,  numobst
         istart = abs(kmeshx(6,iobst))
         if ( istart>0 ) then
            numsurfs = kmeshxsur(istart)
            do i = 1, numsurfs
               iwork(kmeshxsur(istart+i)) = 1
            end do
         end if
      end do

!     --- fill array ipointsurf

      iinp = iinp3
      do isurnr = 1, nsurfs
         ipointsurf(isurnr) = iinp
         call msh049 ( iinput, iinp, iinput(iinp) )
      end do

!     --- Loop over surfaces

      do isurnr = 1, nsurfs
         istartold = ipointsurf(isurnr)
         if ( iwork(isurnr)==0 ) then

!        --- iwork(isurnr)=0, surface will not be changed
!            Copy information of surface from iinput to iinputnew

            if ( isurnr<nsurfs ) then
               ncopy = ipointsurf(isurnr+1)-istartold
            else
               ncopy = iinp5-istartold
            end if
            do i = 0, ncopy-1
               iinputnew(iposiinp+i) = iinput(istartold+i)
            end do
            iposiinp = iposiinp+ncopy

         else

!        --- iwork(isurnr)=1, surface may be changed
!            Store all curves that span the surface into icurvsurface (kmeshs)

            istart = kmeshs(isurnr)
            numcurvs = kmeshs(isurnr+1)-istart
            do i = 1, numcurvs
               icurvsurface(i) = kmeshs(istart+i)
            end do
            changed = .false.

!           --- For all curves in icurvsurface do

            nextra = 0
            do i = 1, numcurvs
               icurnr = icurvsurface(i)
               istartcurv = ipointcurvnw(abs(icurnr))

!              --- itype := type of curve in iinputnew

               itype = iinputnew(istartcurv)
               if ( itype==7 ) then

!              --- itype=composite curve

                  changed = .true.
                  nsubcv = iinputnew(istartcurv+3)
                  first = .true.

!                 --- Replace curve number by its subcurves if these
!                     subcurves have 10000 or 10003

                  do isub = 1, nsubcv
                     if ( icurnr>0 ) then
                        isubcv = iinputnew(istartcurv+3+isub)
                     else
                        isubcv = -iinputnew(istartcurv+3+nsubcv+1-isub)
                     end if
                     itype = iinputnew(ipointcurvnw(abs(isubcv)))
                     if ( itype==10000 .or. itype==10002 ) then

!                    --- itype = 10000 or 10002, replace or add

                        if ( first ) then

!                       --- first time for this composite curve, i.e. replace

                           first = .false.
                           icurvsurface(i) = isubcv

                        else

!                       --- next time for this composite curve, i.e. add

                           nextra = nextra+1
                           icurvsurface(numcurvs+nextra) = isubcv

                        end if

                     end if

                  end do

               end if

            end do

            if ( .not. changed ) then

!           --- not changed
!               Copy information of surface from iinput to iinputnew

               if ( isurnr<nsurfs ) then
                  ncopy = ipointsurf(isurnr+1)-istartold
               else
                  ncopy = iinp5-istartold
               end if
               do i = 0, ncopy-1
                  iinputnew(iposiinp+i) = iinput(istartold+i)
               end do
               iposiinp = iposiinp+ncopy

            else

!           --- changed
!               Construct new surface
!               For all curves in icurvsurface do
!                  Store type, initial point and end point in iworkcurve(1-3,.)

               numcurvs = numcurvs+nextra
               do i = 1, numcurvs
                  icurnr = abs(icurvsurface(i))
                  istartcurv = ipointcurvnw(icurnr)
                  iworkcurve(1,i) = iinputnew(istartcurv)
                  if ( icurnr<=ncurvs ) then

!                 --- Curve is old curve, hence point numbers are stored in
!                     curveusrp

                     istart = curveusrp(icurnr)
                     inpncv = curveusrp(icurnr+1)-istart
                     iworkcurve(2,i) = curveusrp(istart+1)
                     iworkcurve(3,i) = curveusrp(istart+inpncv)

                  else

!                 --- Curve is new curve of type>=10000

                     iworkcurve(2,i) = iinputnew(istartcurv+3)
                     iworkcurve(3,i) = iinputnew(istartcurv+4)

                  end if

               end do

!              --- Construct new surface

               ncrvs = 0
               iinputnew(iposiinp) = iinput(istartold)
               iinputnew(iposiinp+1) = iinput(istartold+1)
               do while ( ncrvs<numcurvs )

!              --- While ( there are still curves to do )
!                  Initial curve := next curve to do

                  do i = 1, numcurvs
                     icurnr = icurvsurface(i)
                     if ( icurnr/=0 ) then

!                    --- Next curve found

                        ipos = i
                        go to 100

                     end if

                  end do

100               ncrvs = ncrvs+1
                  iinputnew(iposiinp+ncrvs+2) = icurnr
                  icurvsurface(ipos) = 0

!                 --- Find end_point

                  if ( icurnr>0 ) then
                     iuserp2 = iworkcurve(3,ipos)
                  else
                     iuserp2 = iworkcurve(2,ipos)
                  end if

!                 --- While ( end_point is point of a remaining curve )
!                        take new curve
!                        Find end_point
!                        Put information in iinputnew

200               do i = 1, numcurvs
                     icurnr = abs(icurvsurface(i))
                     if ( icurnr/=0 ) then
                        if ( iworkcurve(2,i)==iuserp2 ) then

!                       --- End point is begin point of icurnr

                           ncrvs = ncrvs+1
                           iinputnew(iposiinp+ncrvs+2) = icurnr
                           icurvsurface(i) = 0
                           go to 200

                        else if ( iworkcurve(3,i)==iuserp2 ) then

!                       --- End point is end point of icurnr

                           ncrvs = ncrvs+1
                           iinputnew(iposiinp+ncrvs+2) = -icurnr
                           icurvsurface(i) = 0
                           go to 200

                        end if

                     end if
                  end do

               end do
               iinputnew(iposiinp+2) = ncrvs
               iposiinp = iposiinp+ncrvs+3

            end if

         end if

      end do

!     --- Copy rest of iinput

      nshift = iposiinp-iinp5
      if ( iinputnew(9)<0 ) iinputnew(9) = iinputnew(9)-nshift
      if ( iinputnew(14)>16 ) iinputnew(14) = iinputnew(14)+nshift
      do i = iinp5, iinp8
         iinputnew(i+nshift) = iinput(i)
      end do
      if ( iinputnew(15)>16 ) then
         iinputnew(15) = iinputnew(15)+nshift
         jlast = iinputnew(15)-5
         do i = 4, 7
            if ( iinputnew(jlast+i)>0 ) &
               iinputnew(jlast+i) = iinputnew(jlast+i)+nshift
         end do
      end if
      iposiinp = iinp8+nshift

      end
