      subroutine mshobstiinp ( iinput, rinput, &
                               nextracurves, specialpnt, curvinfo, &
                               newcurves, nspecialpnts )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 15-11-2012 Long integers
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 15-08-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Create new array iinput and rinput with respect to free surface
!   intersecting an obstacle
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), nextracurves, specialpnt(*), curvinfo(*), &
              newcurves(*), nspecialpnts
      double precision rinput(*)

!     curvinfo       i    array INFOCURVE
!     iinput         i    integer input array
!     newcurves      i    memory management reference number of array NEWCURVES
!     nextracurves   i    Number of extra curves to be created
!     nspecialpnts   i    Number of special points
!     rinput         i    Real input array
!     specialpnt     i    This array of size 3 x nspecialpnts contains
!                          information about the special points on the curve
!                          specialpnt(1,i) contains the position in curves
!                                            of special point i
!                          specialpnt(2,i) contains first curve number in the
!                                            free surface connected to point i
!                          specialpnt(3,i) contains second curve number in the
!                                            free surface connected to point i
!                                            If 0, there is only one curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposiinp, iposrinp, &
              krenum(10), lenginput, lengrinput, &
              lencurv, lengav, lengq
      integer, allocatable :: iinputnew(:), icurvsnew(:)
      double precision, allocatable :: rinputnew(:), curvesnew(:,:)

!     curvesnew      ?
!     icurvsnew      ?
!     iinputnew      ?
!     iposiinp       Starting address in array iinput with respect to the
!                    extra user points (internal points)
!     iposrinp       ?
!     krenum         Array of length 10 with information about the
!                    renumbering, see msh000
!     lencurv        ?
!     lengav         Available length for an array
!     lenginput      ?
!     lengq          Computed length of array kmeshq
!                    If lengq = 0, array kmeshq has not been filled
!     lengrinput     ?
!     rinputnew      ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH000         Fill some parameters from array IINPUT
!     MSH061         fill array KMESHQ with curve numbers of the subsequent
!                    curves
!     MSHOBSTIIN1    ?
!     MSHOBSTIIN2    ?
!     MSHOBSTIIN3    ?
!     MSHSEQCURVS    Compute sequence in which curves must be created
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extend number of user points and fill first part of rinputnew
!     Extend and copy curves, fill new arrays icurvsnew and curvesnew
!     Adapt array iinputnew and rinputnew for the surfaces
!     Fill final parts of iinputnew and rinputnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstiinp' )

!     --- Extend number of user points and fill first part of rinputnew

      call msh000 ( iinput, krenum, 0 )
      lenginput = size ( iinputkm )+nextracurves*5
      lengrinput = size ( rinputkm )+ &
               nextracurves*3+ &
               nspecialpnts*(ndim+jcoars)

      allocate ( iinputnew(lenginput), rinputnew(lengrinput), &
                 curvesnew(2,size(curvesmsh,2)), &
                 icurvsnew(2*(ncurvs+nextracurves)), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lenginput+lengrinput+size(curvesmsh)+ &
                        ncurvs+nextracurves, &
                       'iinputnew' )
      if ( ierror/=0 ) go to 1000
      call mshobstiin1 ( iinput, rinput, iinputnew, &
                         rinputnew, nspecialpnts, &
                         iposrinp, curvesmsh )
      if ( ierror/=0 ) go to 1000

!     --- Extend and copy curves, fill new arrays icurvsnew and curvesnew
!         Fill final parts of rinputnew

      call mshobstiin2 ( iinput, rinput, iinputnew, &
                         rinputnew, specialpnt, &
                         iposrinp, iposiinp, curvinfo, &
                         newcurves, nextracurves, &
                         newcurves(1+ncurvs), icurvsnew,curvesnew )
      if ( ierror/=0 ) go to 1000

!     --- Adapt array iinputnew and rinputnew for the surfaces
!         Fill final parts of iinputnew

      call mshobstiin3 ( iinput, iinputnew, iposiinp )
      if ( ierror/=0 ) go to 1000

!     --- Create permanent new arrays iinput, rinput, icurvs and curves

      ncurvs = iinputnew(3)
      nuspnt = iinputnew(2)
      km(iseqkm)%kmesh(10) = nuspnt
      km(iseqkm)%kmesh(11) = ncurvs
      lencurv = icurvsnew(ncurvs*2)

      if ( associated(km(iseqkm)%iinputkm) ) &
         deallocate ( km(iseqkm)%iinputkm, km(iseqkm)%rinputkm )
      allocate ( km(iseqkm)%iinputkm(iposiinp), &
                 km(iseqkm)%rinputkm(iposrinp) )
      km(iseqkm)%kmesh(34) = iposiinp
      km(iseqkm)%kmesh(35) = iposrinp

      if ( associated(km(iseqkm)%icurvsmsh) ) &
         deallocate ( km(iseqkm)%icurvsmsh, km(iseqkm)%curvesmsh )
      allocate ( km(iseqkm)%icurvsmsh(2*ncurvs), &
                 km(iseqkm)%curvesmsh(ndim,lencurv) )

!     --- Copy information from temporary positions to new ones

      iinputkm => km(iseqkm)%iinputkm
      rinputkm => km(iseqkm)%rinputkm
      iinputkm = iinputnew(1:iposiinp)
      icurvsmsh => km(iseqkm)%icurvsmsh
      icurvsmsh(1:2*ncurvs) = icurvsnew(1:2*ncurvs)
      curvesmsh => km(iseqkm)%curvesmsh
      rinputkm = rinputnew(1:iposrinp)
      curvesmsh(1:ndim,1:lencurv) = curvesnew(1:ndim,1:lencurv)

!     --- Refill array kmeshq
!         Detect sequence of curves and store in array iseqcurvs

      allocate ( iseqcurvs(ncurvs), kmeshq(40*ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, ncurvs, 'iseqcurvs' )
      if ( ierror/=0 ) go to 1000

      if ( ierror/=0 ) go to 1000
      call mshseqcurvs ( icurvsmsh )
      if ( ierror/=0 ) go to 1000

!     --- Fill new array kmeshq

      call msh061 ( icurvsmsh )
      if ( ierror/=0 ) go to 1000

      deallocate ( iseqcurvs, kmeshq, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iseqcurvs' )
      if ( ierror/=0 ) go to 1000

1000  call erclos ( 'mshobstiinp' )

      end subroutine mshobstiinp
