      subroutine mshobstmarkel5 ( ielemobstcp, lenielemobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-10-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Recompute array ielemobstcp in the sense that only elements inside or
!    outside the obstacle are marked
!    Elements that cross the boundary are marked as external
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     element
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielemobstcp(*), lenielemobst

!     ielemobstcp   i/o   Array containing information about the elements in an
!                         obstacle
!                         The first numobst positions contain the starting
!                         positions of each obstacle sequentially
!                         Per obstacle the first three positions contain
!                         1. The number of elements that are completely inside
!                            the obstacle
!                         2. The number of elements that are not completely
!                            inside the element but with all vertices inside
!                            the element
!                         3. The number of elements that are partly inside
!                            the obstacle
!                            elements that contain only boundary points of the
!                            obstacle are not considered
!                         These positions are followed by the element numbers
!                         in the sequence defined by the first three positions
!     lenielemobst   o    Computed length of array ielemobstcp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iobst, istartelem, istartelem1, nh, numinelmobst

!     i              Counting variable
!     iobst          Obstacle sequence number
!     istartelem     Starting address in element array
!     istartelem1    Starting address in element array
!     nh             Help parameter, number of elements in the boundary of the
!                    obstacle
!     numinelmobst   Number of elements that are completely within the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize some parameters
!     For all obstacles do
!        Copy part corresponding to internal elements from old positions
!        to new ones
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      istartelem = numobst+1
      istartelem1 = istartelem

!     --- Loop over obstacles

      do iobst = 1, numobst
         numinelmobst = ielemobstcp(istartelem)+ielemobstcp(istartelem+1)
         nh = ielemobstcp(istartelem+2)
         ielemobstcp(istartelem1) = numinelmobst
         ielemobstcp(istartelem1+1) = 0
         ielemobstcp(istartelem1+2) = 0
         istartelem1 = istartelem1+3
         istartelem = istartelem+3
         do i = 0, numinelmobst-1
            ielemobstcp(istartelem1+i) = ielemobstcp(istartelem+i)
         end do
         istartelem1 = istartelem1+numinelmobst
         istartelem = istartelem+numinelmobst+nh

      end do
      lenielemobst = istartelem1

      end
