      subroutine mshori2d ( kmeshc, inpelm, inelem, coor, ishape, &
                            ichnge, jac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 10-03-2000 Extra parameters ichnge and jac
!        version  1.0  date 11-10-1998
!
!   copyright (c) 1998-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    If the orientation of the elements is clockwise, change them in
!    anti-clockwise by reordening of the node numbers
!    2D only
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     mesh
!     orientation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, inelem, kmeshc(inpelm,inelem), ishape, ichnge
      double precision coor(2,*), jac

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ichnge     o    indicator how many elements were changed
!     inelem     i    Number of elements in a specific element group
!     inpelm     i    number of points per element
!     ishape     i    Shape number of elements
!     jac        o    jacobian of element
!     kmeshc    i/o   kmesh part c, contains the nodal points corresponding
!                     to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, ihelp(10), inboun
      double precision t1(2), t2(2)

!     i          Counting variable
!     ielem      Element number
!     ihelp      integer work array to store the node numbers of one element
!     inboun     Number of nodes at boundary of element
!     t1         First tangential vector
!     t2         Second tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Check direction of first element by computing the outer product
!      of the first 2 vectors
!      If ( direction negative ) then
!         Renumber nodes in kmeshc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      inboun = inpelm
      ichnge = 0
      go to ( 130, 140, 130, 140, 140, 180, 130, 130 ), ishape-2

!     --- ishape = 3, 5, 9, 10 linear triangles and quadrilaterals

130   t1(1) = coor(1,kmeshc(2,1))-coor(1,kmeshc(1,1))
      t1(2) = coor(2,kmeshc(2,1))-coor(2,kmeshc(1,1))
      t2(1) = coor(1,kmeshc(3,1))-coor(1,kmeshc(2,1))
      t2(2) = coor(2,kmeshc(3,1))-coor(2,kmeshc(2,1))
      if ( ishape.ge.9 ) inboun = inpelm-1
      go to 1000

!     --- ishape = 4, 6, 7 quadratic triangles and quadrilaterals

140   t1(1) = coor(1,kmeshc(3,1))-coor(1,kmeshc(1,1))
      t1(2) = coor(2,kmeshc(3,1))-coor(2,kmeshc(1,1))
      t2(1) = coor(1,kmeshc(5,1))-coor(1,kmeshc(3,1))
      t2(2) = coor(2,kmeshc(5,1))-coor(2,kmeshc(3,1))
      if ( ishape.ge.6 ) inboun = inpelm-1
      go to 1000

!     --- ishape = 8 special triangle

180   t1(1) = coor(1,kmeshc(4,1))-coor(1,kmeshc(1,1))
      t1(2) = coor(2,kmeshc(4,1))-coor(2,kmeshc(1,1))
      t2(1) = coor(1,kmeshc(7,1))-coor(1,kmeshc(7,1))
      t2(2) = coor(2,kmeshc(7,1))-coor(2,kmeshc(7,1))
      inboun = inpelm-1
      go to 1000

1000  jac = t1(1)*t2(2)-t1(2)*t2(1)
      if ( jac.lt.0d0 ) then

!     --- Jacobian has negative direction, change the node numbers

         ichnge = 1

         do ielem = 1, inelem
            do i = 2, inboun
               ihelp(i) = kmeshc(i,ielem)
            end do
            do i = 2, inboun
               kmeshc(i,ielem) = ihelp(inboun+2-i)
            end do
         end do

      end if

      end
