      subroutine mshori3d ( kmeshc, inpelm, inelem, coor, ishape, &
                            ichnge, jac, ivol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.5  date 22-05-2014 Extension with shape 22
!        version  3.4  date 17-02-2006 Extension with old_general
!        version  3.3  date 25-01-2006 Jump in case inelem<=1
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make the orientation of the elements such that the "volume" is always
!     positive
!     3D only
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh
!     orientation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, inelem, kmeshc(inpelm,inelem), ishape, ichnge, &
              ivol
      double precision coor(3,*), jac

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ichnge     i    indicator how many elements were changed
!     inelem     i    Number of elements in a specific element group
!     inpelm     i    number of points per element
!     ishape     i    Shape number of elements
!     ivol       i    Defines type of volume generator
!     jac        o    jacobian of element
!     kmeshc    i/o   kmesh part c, contains the nodal points corresponding
!                     to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical same

!     same       If true all elements have the same orientation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHORI3D11     Make the orientation of the elements such that the
!                    "volume" is always positive for linear tetrahedrons
!     MSHORI3D12     Make the orientation of the elements such that the
!                    "volume" is always positive for quadratic tetrahedrons
!     MSHORI3D13     Make the orientation of the elements such that the
!                    "volume" is always positive for linear hexahedrons
!     MSHORI3D14     Make the orientation of the elements such that the
!                    "volume" is always positive for quadratic hexahedrons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     298   code for generation of volumes incorrect
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Check direction by computing the Jacobian of a tetrahedron with 4 nodes
!      defined in the element
!      If ( Jacobian negative ) then
!         Renumber nodes in kmeshc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshori3d' )

      if ( ivol<0 .or. ivol>11 ) then
         call errint ( ivol, 1 )
         call errsub ( 298, 1, 0, 0 )
         go to 1000
      end if
      if ( inelem<=1 ) go to 1000

      ichnge = 0
      same = ivol==1 .or. ivol==4 .or. ivol==5 .or. ivol==9 .or. &
             ivol>=10

      select case ( ishape )

      case (11, 18)

!     --- ishape = 11, 18 linear tetrahedrons

         call mshori3d11 ( kmeshc, inpelm, inelem, coor, ichnge, &
                           jac )

      case (12)

!     --- ishape = 12 quadratic tetrahedrons

         call mshori3d12 ( kmeshc, inpelm, inelem, coor, ichnge, &
                           jac )

      case (13, 17)

!     --- ishape = 13 trilinear hexahedrons

         call mshori3d13 ( kmeshc, inpelm, inelem, coor, ichnge, &
                           jac, same )

      case (14)

!     --- ishape = 14 triquadratic hexahedrons

         call mshori3d14 ( kmeshc, inpelm, inelem, coor, ichnge, &
                           jac, same )

      case (15, 16, 19:22)

!     --- ishape = 15,16, 19-22: prisms, do not change direction

      case default

!     --- shape number not implemented

         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )

      end select  ! case ( ishape )

1000  call erclos ( 'mshori3d' )

      end
