      subroutine mshpavcf ( coor, ncoor, nel, kbndpt, &
                            crd, ncrd, ndim, height )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  2.0  date 22-09-2010 Remove work array
!        version  1.0  date 27-05-1997
!
!   copyright (c) 1997-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!        copies the relevant nodes from crd to coor
!        relevant nodes are nodes referenced in kmeshclc
!        kmeshclc is adjusted to point to the nodes in coor
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoor, nel, ncrd, ndim, kbndpt(*)
      double precision coor(ndim,*), crd(4,*), height

!     coor     o    coordinates that will be filled
!     crd      i    coordinates of the nodes, not all nodes are used
!     height   i    height of the plane, generally not used
!     kbndpt  i/o   nodes stored in the boundary, adjusted for nodes not used
!     ncoor    o    number of nodes in coor
!     ncrd     i    number of nodes in crd
!     ndim     i    dimension
!     nel     i/o   number of elements in kmeshclc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, lookup(ncrd)

!     i        counting variable
!     j        counting variable
!     lookup   integer work array of length ncrd containing the lookup
!              table, lookup(i) has the new node number of node i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- put lookup to zero

      do i = 1, ncrd
         lookup(i) = 0
      end do

!     --- indicate a node is used by making it 1

      do i = 1, 3*nel
         lookup(kmeshclc(i)) = 1
      end do

!     --- fill coor and make lookup into a lookuptable

      j=1
      do i = 1, ncrd
         if ( lookup(i)==1 ) then

!        --- this is a used node, put in coor

            coor(1,j) = crd(1,i)
            coor(2,j) = crd(2,i)

!           --- one elevation for the surface, not good

            if (ndim==3) coor(3,j) = height
            lookup(i) = j
            j = j+1
         end if
      end do
      ncoor = j-1

!     --- rewrite kmeshclc

      do i=1,3*nel
         kmeshclc(i) = lookup(kmeshclc(i))
      end do

!     --- rewrite kbndpt

      do i=1,nbndpt
         kbndpt(i) = lookup(kbndpt(i))
      end do

      end
