      subroutine mshr11(bcord,tau,n,m,irand)
! ======================================================================
!
!        programmer    guus segal
!        version  1.3  date 27-06-2013 Debug statements
!        version  1.2  date 06-11-2010 Use exact bound for array tau
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1982-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array tau for rectangular grid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     rectangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, m, irand
      double precision bcord(2,*), tau(2*irand*(n+m)+1)

!     bcord          i    array containing the coordinates of the boundaries
!     irand          i    number of points on the side of an element - 1
!     m              i    number of elements in the "second" direction
!     n              i    number of elements in the "first" direction
!     tau            o    in array tau the relative arclength of each side of
!                         the "rectangle" are stored. see meshr1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, ibcord, ih, irnd2, j, k ,m2, n2
      double precision x1(2), dx(2), s, x2(2)

!     dx             length of vector in x-direction
!     i1             Loop variable
!     i2             Loop variable
!     ibcord         ?
!     ih             Help variable to store a constant temporarily
!     irnd2          ?
!     j              Counting variable
!     k              Number of nodes along side
!     m2             ?
!     n2             ?
!     s              ?
!     x1             coordinates of prior point
!     x2             coordinates of present point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshr11' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshr11'
         write(irefwr,1) 'irand, m, n', irand, m, n
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- array tau giving the relative arclengths in the local boundary
!         is computed

      ibcord = 1
      irnd2=2*irand
      m2=irand*m
      n2=irand*n
      ih=1
      x1=bcord(:,1)

      if ( debug ) then
         write(irefwr,1) 'ibcord, irnd2, m2, n2', ibcord, irnd2, m2, n2
         write(irefwr,2) 'x1', x1
      end if  ! ( debug )

      tau = 0d0     ! Clear array tau

!     --- Fill all tau-values:

      do i1 = 1, 2  ! i1 = 1: horizontal;  i1 = 2: vertical

         k = n2  ! number of nodes along side 1 or 3 (horizontal)

         do i2 = 1, 2  ! i2 = 1: sides 1, 2;  i2 = 2 sides 3 , 4

!        --- Next side

            s=0d0

            do j = 1, k

               ih=ih+1            ! next node on side
               x2(:)=bcord(:,ih)  ! coordinates
               dx=x2-x1           ! distance
               s=s+sqrt(dot_product(dx,dx)) !length
               tau(ibcord+j)=s    ! store length
               x1=x2              ! copy new node into old one

            end do  ! j= 1, k

            k=k-1

            if ( i1==1 ) then

!           --- i1=1, hence sides 1 and 2: positive direction

               do j = 1, k
                  tau(ibcord+j)=tau(ibcord+j)/s
               end do   ! j= 1, k

            else

!           --- i1=2, hence sides 3 and 4: negative direction

               do j = 1, k
                  tau(ibcord+j)=1d0-tau(ibcord+j)/s
               end do   ! j= 1, k

            end if  ! ( i1==1 )

            ibcord=ibcord+k+1
            k=m2  ! number of nodes along side 2 or 4 (vertical)

         end do  ! i2=1,2

      end do  ! i1=1,2

1000  call erclos ( 'mshr11' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( tau, 2*irand*(n+m)+1, 'tau' )
         write(irefwr,*) 'End mshr11'

      end if  ! ( debug )

      end
