      subroutine mshrd2paver ( i, work, rpin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill rpin in case of paver
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: i
      double precision, intent(in) :: work(*)
      double precision, intent(inout) :: rpin(4,*)

!     i              i    Sequence number of local curve
!     rpin          i/o   Array containing the thicknesses and factors
!                         of certain curves
!     work           i    Work array
!                         See subroutine mshr01
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, icurnr, jcurnr

!     icurnr         Curve sequence number
!     j              Counting variable
!     jcurnr         curve number of next curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2paver' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2paver'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Paver has been called, fill rpin(1-4,.)

      icurnr = nint(work(24+i*5))
      if ( icurnr<0 ) then

!     --- Negative curve number, fill rpin(1-2,.)

         do j = 1, 2
            rpin(j,-icurnr) = work(24+(i-1)*5+j)
         end do
         if ( i==1 ) then

!        --- First curve, fill defaults

            do j = 1, 2
              if ( rpin(j,-icurnr)<-9d19 ) rpin(j,-icurnr) = 1d0

            end do

         else

!        --- Next curve, fill defaults

            jcurnr =  nint(work(24+(i-1)*5))
            if ( jcurnr<0 ) then

!           --- Previous curve has negative value

               do j = 1, 2
                 if ( rpin(j,-icurnr)<-9d19 ) rpin(j,-icurnr) = rpin(j,-jcurnr)
               end do

            else

!           --- Previous curve has positive value

               do j = 1, 2
                 if ( rpin(j,-icurnr)<-9d19 ) rpin(j,-icurnr) = rpin(j+2,jcurnr)
               end do

            end if  ! ( jcurnr<0 )

         end if  ! ( i==1 )

      else

!     --- Positive curve number, fill rpin(3-4,.)

         do j = 3, 4
            rpin(j,icurnr) = work(24+(i-1)*5+j)
         end do
         if ( i==1 ) then

!        --- First curve, fill defaults

            do j = 3, 4
              if ( rpin(j,icurnr)<-9d19 ) rpin(j,icurnr) = 1d0
            end do

         else

!        --- Next curve, fill defaults

            jcurnr =  nint(work(24+(i-1)*5))
            if ( jcurnr<0 ) then

!           --- Previous curve has negative value

               do j = 3, 4
                 if ( rpin(j,icurnr)<-9d19 ) rpin(j,icurnr) = rpin(j-2,-jcurnr)
               end do

            else

!           --- Previous curve has positive value

               do j = 3, 4
                 if ( rpin(j,icurnr)<-9d19 ) rpin(j,icurnr) = rpin(j,jcurnr)
               end do

            end if  ! ( jcurnr<0 )

         end if  ! ( i==1 )

      end if

1000  call erclos ( 'mshrd2paver' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd2paver'

      end if  ! ( debug )

      end subroutine mshrd2paver
