      subroutine mshrd2rect ( iinp, iinput, n, m, idiag, isurnr, ismoot, &
                              jinp, ncur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information for rectangle in array iinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: n, m, isurnr, ismoot, jinp, ncur
      integer, intent(inout) :: iinp, iinput(*), idiag

!     idiag         i/o   Position of diagonal element in matrix
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See subroutine mshrea
!     ismoot         i    Smoothing parameter, the iteration process is stopped
!                         if the difference between two grids is less than
!                         10^-ismoot
!     isurnr         i    Surface sequence number
!     jinp           i    last position used in array iinput with respect to
!                         iprec
!     m              i    Parameter m in surface record
!     n              i    Parameter n in surface record
!     ncur           i    Number of curves surrounding a surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1345   incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2rect' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2rect'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( idiag/=2 ) idiag = 0
      iinput(iinp) = n
      iinput(iinp+1) = m
      iinp = iinp+2
      iinput(jinp+2) = iinput(jinp+2)+10000*idiag
      if ( ismoot>0 ) then
         iinput(jinp) = 17
         iinput(iinp) = ismoot
         iinput(iinp+1) = 0
         iinp = iinp+2
      end if
      if ( n<0 .or. n>100000 .or. ncur/=4 .and. n==0 ) then

!     --- Error 1345:   N out of range

         call errint ( n, 1 )
         call errint ( 100000, 2 )
         call errint ( isurnr, 3 )
         call errchr ( 'N', 1 )
         call errsub ( 1345, 3, 0, 1 )

      end if
      if ( m<0 .or. m>100000 .or. ncur/=4 .and. m==0 ) then

!     --- Error 1345:   M out of range

         call errint ( m, 1 )
         call errint ( 100000, 2 )
         call errint ( isurnr, 3 )
         call errchr ( 'M', 1 )
         call errsub ( 1345, 3, 0, 1 )

      end if

1000  call erclos ( 'mshrd2rect' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd2rect'

      end if  ! ( debug )

      end subroutine mshrd2rect
