      subroutine mshrd2shape ( iinput, iinp, trian )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-01-2015 New call to readreal
!        version  1.0  date 21-11-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read or set shape number of surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iinput(*)
      integer, intent(in) :: iinp
      logical, intent(out) :: trian

!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See Programmer's Guide
!     trian          o    Indication whether triangles (true) or quadrilaterals
!                         are created (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nncodelc
      parameter ( nncodelc=9 )
      integer :: ncodelc(nncodelc), ncodmxlc(nncodelc)
      character (len=29) codelc(nncodelc)
      logical ::  openfound

!     codelc         Table with keywords
!     ncodelc        Table containing the number of significant characters
!                    in the keyword table codelc
!     ncodmxlc       Table containing the maximal number of  characters in
!                    the keyword table codelc
!     nncodelc       Number of entries in codelc
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READREAL       Read a real or real expression from the input file
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     128   code for generation of elements (ishape) is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc / 'TRIANGLE', 'QUADRILATERAL', &
                  'LINEAR_TRIANGLE', 'QUADRATIC_TRIANGLE', &
                  'LINEAR_QUADRILATERAL', 'QUADRATIC_QUADRILATERAL', &
                  'EXTENDED_QUADRATIC_TRIANGLE', &
                  'EXTENDED_LINEAR_QUADRILATERAL', &
                  'EXTENDED_LINEAR_TRIANGLE' /
      data ncodelc  /  3, 7,10,13,10,  13,14,19,19 /
      data ncodmxlc /  8,13,15,18,20,  23,27,29,24 /

!     codelc           Table with keywords
!                    The following keywords are available:
!                     1: TRIANGLE
!                     2: QUADRILATERAL
!                     3: LINEAR_TRIANGLE
!                     4: QUADRATIC_TRIANGLE
!                     5: LINEAR_QUADRILATERAL
!                     6: QUADRATIC_QUADRILATERAL
!                     7: EXTENDED_QUADRATIC_TRIANGLE
!                     8: EXTENDED_LINEAR_QUADRILATERAL
!                     9: EXTENDED_LINEAR_TRIANGLE
! ======================================================================
!
      call eropen ( 'mshrd2shape' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2shape'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ishape = shape_sur     ! Default shape number

!     --- determine shape number

      openfound = .false.
      call readreal ( openfound, .false. )

      if ( debug ) write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
      if ( ityprd==2 ) then

!     --- shape number found

         ishape = nint(realrd)

         if ( ishape<=2 .or. ishape>10 ) then
            call errint ( ishape, 1 )
            call errsub ( 128, 1, 0, 0 )
            call errcrd
         end if

      else if ( typeitems(lastitem+1)==3 ) then

!     --- string found, check if it is one of the possible shapes

         call asknxt
         if ( debug ) then
            write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         call sepcopycode ( codelc, nncodelc, ncodelc, ncodmxlc )
         ierpri = 0
         iretur = sepsubkeyw()
         if ( iretur>7 ) then

!        --- extended_linear..

            ishape = iretur+1

         else if ( iretur>2 ) then

!        --- standard shapes

            ishape = iretur

         else if ( iretur>0 ) then

!        --- linear/quadratic

            ishape = iretur+2

         else

!        --- not shape found, reset lastitem

            lastitem = lastitem-1

         end if  ! ( iretur>7 )

      end if  ! ( ityprd==2 )

      iinput(iinp+1) = ishape
      trian = ishape<=4 .or.ishape>6 .and. ishape/=9

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ishape, iinp', ishape, iinp
         call prinin ( iinput, iinp+1, 'iinput' )
         write(irefwr,*) 'End mshrd2shape'

      end if  ! ( debug )
      call erclos ( 'mshrd2shape' )

      end subroutine mshrd2shape
