      subroutine mshrd2trans ( icopy, isurnr, isurf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-01-2015 New call to readreal
!        version  1.0  date 21-11-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read rest of surface for translate, rotate, reflect or similar
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer,intent(in) :: isurnr, isurf
      integer,intent(out) :: icopy

!     icopy          o    Number of volume to be copied
!     isurf          i    Type of surface with respect to mesh generator
!     isurnr         i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ::  openfound

!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREAL       Read a real or real expression from the input file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     147   wrong character in surface record.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2trans' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2trans'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read surface to be "copied"

      call asknxt
      if ( ityprd==4 ) then

!     --- Text found, this must be S

         if ( nchar/=1 .or. string(1:1)/='S' ) then

!        --- Error 147: Not an S found

            call errchr ( string(1:1), 1 )
            call errint ( isurnr, 1 )
            call errsub ( 147, 1, 0, 1 )
            call errcrd
            go to 1000

         else

            openfound = .false.
            call readreal ( openfound, .false. )

         end if

      end if
      if ( ityprd/=2 ) then

!     --- Not an integer found

         call errchr ( string(1:1), 1 )
         call errint ( isurnr, 1 )
         call errsub ( 147, 1, 0, 1 )
         call errcrd
         go to 1000

      end if
      icopy = nint(realrd)

1000  call erclos ( 'mshrd2trans' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isurf, icopy', isurf, icopy
         write(irefwr,*) 'End mshrd2trans'

      end if  ! ( debug )

      end subroutine mshrd2trans
