      subroutine mshrd2type ( iinput, iinp, isurnr, isurf, adaptiinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read type of surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer,intent(in) :: isurnr, iinp
      integer,intent(out) :: isurf
      integer,intent(inout) :: iinput(*)
      logical,intent(inout) :: adaptiinput

!     adaptiinput   i/o   If true iinput must be adapted
!     iinp           i    last position used in array iinput
!     iinput        i/o   integer input array
!     isurf          o    Type of surface with respect to mesh generator
!     isurnr         i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1349   Not a text found after Si =.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2type' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2type'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      call asknxt
      if ( ityprd/=4 ) then

!     --- Not a text found after Si =

         call errint ( isurnr, 1 )
         call errsub ( 1349, 1, 0, 0 )
         call errcrd
         go to 1000

      end if

      ierpri = 1
      isurf = sepsubkeyw ()

      if ( isurf==0 ) go to 1000
      if ( isurf==20 )  adaptiinput = .true.

      iinput(iinp) = isurf
      iinput(iinp+1) = 0
      iinput(iinp+2) = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isurf, isurnr', isurf, isurnr
         write(irefwr,*) 'End mshrd2type'

      end if  ! ( debug )
      call erclos ( 'mshrd2type' )

      end subroutine mshrd2type
