      subroutine mshrd7 ( iinput, nvolel, nvolms, inpmax, nelgrp, inpvol, &
                          nintprop, nrealprop, intprop, realprop )
! ======================================================================
!
!        programmer    Guus Segal
!        version 11.0  date 21-11-2014 New parameter list
!        version 10.7  date 27-01-2014 Correction real properties
!        version 10.6  date 17-02-2006 Extension with old_general
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read cards with volume elements until a card with
!   renumber, plot or end is found
!   fill array iinput with the information of the volume elements
!   iinput must be filled from position 1
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpmax, iinput(inpmax), inpvol(*), nvolel, &
              nelgrp, nvolms, nintprop, &
              nrealprop, intprop(10,*)
      double precision realprop(10,*)

!     iinput    i/o   integer input array of length iinpmax
!     inpmax     i    maximal space for iinput
!     inpvol     i    array iinput containing information of the volumes
!     intprop   i/o   Integer array with properties
!     nelgrp    i/o   number of element groups
!     nintprop  i/o   Number of integer properties per element group
!     nrealprop i/o   Number of real properties per element group
!     nvolel     o    number of parts where volume elements are defined
!     nvolms     i    number of volumes
!     realprop  i/o   Real array with properties
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncodelc=6
      character (len=13) codelc(nncodelc)
      integer ncodelc(nncodelc), ncodmxlc(nncodelc), iprop, &
              i, ielgrp, ifirst, iinp, ilast, ivolm, m, &
              iwork(2*nvolms), nwarnlc
      double precision :: rprop
      logical ::  newelgrp

!     codelc         Table with keywords
!     i              Counting variable
!     ielgrp         element group number
!     ifirst         First volume number read
!     iinp           last position used in array iinput
!     ilast          Last volume number read
!     iprop          Integer property
!     ivolm          Type of volume with respect to mesh generator
!     iwork          work array of length 2*nvolms
!                    iwork(i), i=1(1)nsurfs contains the shape numbers of the
!                       volumes
!                    iwork(i+nvolms), i=1(1)nvolms is used to indicate if a
!                       volume has already been used for an element group (>0)
!                       or not (0)
!     m              number of items read
!     ncodelc        Table containing the number of significant characters in
!     ncodmxlc       Table containing the maximal number of  characters in
!     newelgrp       If true a new element group is found
!     nncodelc       Number of entries in code
!     nwarnlc        Number of local warnings
!     rprop          Real property
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     MSH050         Raise iinp as function of ivolm
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    The subroutine reads input records of the shape:
!    VELM i = (V1, V2 ), int_properity j = k, real_property l = r
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      68   element group number not equal to preceding element group number or
!           preceding element group number plus one
!     142   Internal error, too few space for integers
!     296   first volume number has incorrect value
!     297   last volume number has incorrect value
!     298   code for generation of volumes incorrect
!    1078   Integer property has different values for the same group
!    1082   Real property has different values for the same group
!    1545   Volumes have been used more than once to create element groups
!    1837   Present element group number is equal to preceding one, whereas a
!           new type of elements is introduced
!    2199   Volume not connected to element group
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc / 'VELM', 'INT_PROPERTY', 'INT_PROPERTY', &
                    'REAL_PROPERTY', 'REAL_PROPERTY', 'V' /
      data ncodelc / 3,  3,  3,  3,  3,   1 /
      data ncodmxlc/ 4, 12, 12, 13, 13,   1 /

!     codelc     Table with keywords
!              The following types of texts are recognised:
!              1:   VELM
!              2:   INT_PROPERTY
!              3:   INT_PROPERTY
!              4:   REAL_PROPERTY
!              5:   REAL_PROPERTY
!              6:   V
! ======================================================================
!
      call eropen ( 'mshrd7' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd7'
         write(irefwr,1) 'nrealprop', nrealprop
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( codelc, nncodelc, ncodelc, ncodmxlc )

      itype(1:nncode) = 0
      itype(2) = 34
      itype(4) = 34
      itype(5) = 1
      itype(6) = 14
      tabmin(1:nncode) = -1d9
      tabmin(1) = 0d0
      tabmin(2) = 1d0
      tabmin(4) = 1d0
      tabmin(6) = 2d0
      tabmax(1:nncode) = 1d9
      tabmax(1) = 1d3
      tabmax(2) = 1d1
      tabmax(4) = 1d1
      tabmax(6) = 0d0

      iinp = 1
      do i = 1, nvolms
         ivolm = inpvol(iinp)

         if ( ivolm<0 .or. ivolm>11 ) then
            call errint ( ivolm, 1 )
            call errsub ( 298, 1, 0, 0 )
            go to 1000
         end if

         if ( ivolm>=6 .and. ivolm<=8 ) then

!        --- Volume is copied, hence use shape number of previous volume

            iwork(i) = iwork(inpvol(iinp+2))

         else

!        --- Volume is not copied, new shape number

            iwork(i) = inpvol(iinp+1)

         end if
         iwork(i+nvolms) = 0
         call msh050 ( inpvol, iinp, ivolm )
      end do
      nvolel = 0
      iinp = 0

!     --- read next record

      ityprd = 1

200   table = 0d0
      tabmax(6) = nvolms
      nmain = nncodemain
      call  sepreadsetkeyw
      icode = iretur
      if ( icode>0 ) go to 900

      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
      end if  ! ( debug )

!     --- detect the volume elements

      nvolel = nvolel+1
      ielgrp = nint(table(1))
      if ( iinp==0 ) then

!     --- No records with respect to surface elements read, so nelgrp must
!         be raised

         if ( ielgrp==nelgrp ) then

!        --- Error 1837: ielgrp = nelgrp

            call errint ( ielgrp, 1 )
            call errsub ( 1837, 1, 0, 0 )
            call errcrd

         end if
         nelgrp = nelgrp + 1
         newelgrp = .true.

      else if ( ielgrp>nelgrp ) then

!     --- Records have been read and ielgrp>nelgrp, raise nelgrp

         nelgrp = nelgrp+1
         newelgrp = .true.

      else

         newelgrp = .false.

      end if
      if ( ielgrp<nelgrp ) then
         call errint ( ielgrp, 1 )
         call errint ( nelgrp-1, 2 )
         call errsub ( 68, 2, 0, 0 )
         call errcrd
      end if
      iposut = nint(table(6))
      m = nint(tabmax(6))
      ifirst = nint(tabout(iposut))
      if ( m==1 ) then
         ilast = ifirst
      else
         ilast = nint(tabout(iposut+1))
      end if
      if ( ifirst<=0 ) then
         call errint ( ifirst, 1 )
         call errint ( nvolms, 2 )
         call errsub ( 296, 2, 0, 0 )
         call errcrd
      end if
      if ( ilast<ifirst ) then
         call errint ( ifirst, 1 )
         call errint ( ilast, 2 )
         call errint ( nvolms, 3 )
         call errsub ( 297, 3, 0, 0 )
         call errcrd
      end if
      if ( newelgrp ) then

!     --- new element group, set integer properties

         iposut = nint(table(2))
         if ( iposut>0 ) then

!        --- integer property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- integer property found

                  nintprop = max(i,nintprop)
                  intprop(i,ielgrp) = nint(tabout(iposut+i-1))

               else
                  intprop(i,ielgrp) = -1000000000
               end if

            end do

         end if

!        --- set real properties

         iposut = nint(table(4))
         if ( debug ) then
            write(irefwr,1) 'iposut, nout', iposut, nout
            call prinrl ( tabout(iposut), 10, 'tabout' )
         end if  ! ( debug )

         if ( iposut>0 ) then

!        --- real property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- real property found

                  nrealprop = max(i,nrealprop)
                  realprop(i,ielgrp) = tabout(iposut+i-1)

               else
                  realprop(i,ielgrp) = -1d30
               end if

            end do

         end if

      else

!     --- old element group, check integer properties

         iposut = nint(table(2))
         if ( iposut>0 ) then

!        --- integer property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- integer property found

                  nintprop = max(i,nintprop)
                  iprop = nint(tabout(iposut+i-1))
                  if ( intprop(i,ielgrp)<=-1000000000 ) then
                     intprop(i,ielgrp) = iprop
                  else if ( intprop(i,ielgrp)/=iprop ) then

!                 --- Property is not the same as given before

                     call errint ( i, 1 )
                     call errint ( ielgrp, 2 )
                     call errint ( iprop, 3 )
                     call errint ( intprop(i,ielgrp), 4 )
                     call errsub ( 1078, 4, 0, 0 )

                  end if

               end if

            end do

         end if

         iposut = nint(table(4))
         if ( iposut>0 ) then

!        --- real property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- real property found

                  nrealprop = max(i,nrealprop)
                  rprop = tabout(iposut+i-1)
                  if ( realprop(i,ielgrp)<=-1d30 ) then
                     realprop(i,ielgrp) = rprop
                  else if ( abs(realprop(i,ielgrp)-rprop)> &
                            sqreps*rprop ) then

!                 --- Property is not the same as given before

                     call errint ( i, 1 )
                     call errint ( ielgrp, 2 )
                     call erreal ( rprop, 1 )
                     call erreal ( realprop(i,ielgrp), 2 )
                     call errsub ( 1082, 2, 2, 0 )

                  end if

               end if

            end do

         end if

      end if

      if ( iinp+4>inpmax ) then

!     --- iinp>inpmax

         call errint ( inpmax, 1 )
         call errint ( iinp, 2 )
         call errsub ( 142, 2, 0, 0 )
         call errcrd
         go to 1000

      end if

      do i = ifirst, ilast

         if ( iwork(i+nvolms)==0 ) then

!        --- volume has not yet been used

            iwork(i+nvolms) = ielgrp

         else

!        --- error volume has been used before

            call errint ( i, 1 )
            call errint ( iwork(i+nvolms), 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 1545, 3, 0, 0 )

         end if

      end do

      iinput(iinp+1) = iwork(ifirst)
      iinput(iinp+2) = ielgrp
      iinput(iinp+3) = ifirst
      iinput(iinp+4) = ilast
      iinp = iinp+4

      goto 200
 900  icode = icode - 1

      nwarnlc = 0
      do i = 1, nvolms

         if ( iwork(i+nvolms)==0 ) then

!        --- volume i is not connected to an element group

            nwarnlc = nwarnlc+1
            call errint ( i, nwarnlc )

         end if
         if ( nwarnlc>0 ) call errwar ( 2199, nwarnlc, 0, 0 )

      end do

1000  call erclos ( 'mshrd7' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nrealprop', nrealprop
         write(irefwr,*) 'End mshrd7'

      end if  ! ( debug )

      end
