      subroutine mshrddummy ( nelgrp, iinputdummy, nelgrpdum, &
                              leniinputdummy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information with respect to dummy elements and store in array
!     iinputdummy
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelgrp, iinputdummy(*), nelgrpdum, leniinputdummy

!     iinputdummy    o    Integer array in which the number of nodes per dummy
!                         element are stored
!     leniinputdummy i    Declared length of array iinputdummy
!     nelgrp         i    Number of element groups
!     nelgrpdum      o    Number of dummy element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     The following input is recognized
!
!     delm i = ( npelm = j )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     While not a main record has been found do
!        Read next record
!        Interpret next record
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrddummy' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrddummy'
         write(irefwr,*) 'nelgrp ', nelgrp

      end if
      icode = 0
      nelgrpdum = 0
      ityprd = 1

      nncode = 2
      itype(1:2) = 0
      code(1) = 'DELM'
      code(2) = 'NPELM'
      ncode(1) = 3
      ncode(2) = 4
      ncodmx(1) = 4
      ncodmx(2) = 5
      nmain = nncodemain

!     --- While not a main record has been found do

      do while ( icode==0 )

!     --- Read next record

         tabmin(1) = nelgrp+1    ! New element group number must nelgrp+1
         tabmin(2) = 1d0         ! minimum number of nodes
         tabmax(1) = nelgrp+1    ! New element group number must nelgrp+1
         tabmax(2) = 1000d0      ! maximum number of nodes
         table(2) = 1d0
         call sepreadsetkeyw
         if ( icode/=0 .or. ierror/=0 ) go to 1000

         nelgrp = nelgrp+1
         nelgrpdum = nelgrpdum+1

         if ( nelgrpdum>leniinputdummy ) then

!        --- nseqkeyw > maxkeyw, array too small

            call errint ( nelgrpdum, 1 )
            call errint ( leniinputdummy, 2 )
            call errchr ( 'iinputdummy', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

         iinputdummy(nelgrpdum) = nint(table(2))

      end do

1000  if ( icode>0 ) icode = icode-1
      if ( debug ) write(irefwr,*) 'End mshrddummy'
      call erclos ( 'mshrddummy' )

      end
