      subroutine mshrea ( iinput, rinput, lengrl, lengin, lnrinp, &
                          lniinp, cunit, adaptiinput, maxratio )
! ======================================================================
!
!        programmer    Guus Segal
!        version 18.1  date 03-03-2015 Extra debug
!        version 18.0  date 21-11-2014 Use subroutines mshreadxxx
!        version 17.6  date 14-05-2014 New call to mshrd1
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read cards and store in array ibuffr
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lengrl, lengin, lniinp, lnrinp, iinput(lniinp)
      double precision rinput(lnrinp), cunit, maxratio
      logical adaptiinput

!     adaptiinput    o    If true iinput must be adapted
!     cunit          i    Unit length in the case of coarseness
!     iinput         o    The integer description of the input is stored in
!                         IINPUT See Programmer's Guide    integer input array
!     lengin         o    computed length of array iinput
!     lengrl         o    computed length of array rinput
!     lniinp         i    estimated length of array iinput
!     lnrinp         i    estimated length of array rinput
!     maxratio       o    Maximal ratio between succeeding elements
!     rinput         o    The real description of the input is stored in RINPUT
!                         See Programmer's Guide
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncodelc=21, nextra=4, leniinputdummy=10
      character (len=18) codelc(nncodelc+nextra)
      integer ncodelc(nncodelc+nextra), &
              lengintf, iinputdummy(leniinputdummy), &
              ncodmxlc(nncodelc+nextra), leng, &
              iwork(60), lenipoint, i1, i2
      integer, allocatable :: ipoint(:,:)
      logical composrf

!     codelc         Table with keywords
!     composrf       If true composite elements have been found
!     i1             Help variable to compute a constant
!     i2             Help variable to compute a constant
!     iinputdummy    Integer array in which the number of nodes per dummy
!                    element are stored
!     ipoint         pointer array: contains the starting positions of the
!                    curves (1), surfaces (2) and volumes(3) in array
!                    iinput
!     iwork          Integer work array to be used by mshrd5
!                    At this moment 40 positons are used to store ichangecr
!                    and 20 positions to store iobstacle
!     leng           Help variable to compute the length of some arrays
!     lengintf       Length of part corresponding to interface elements
!     leniinputdummy Declared length of array iinputdummy
!     lenipoint      Length of array ipoint
!     ncodelc        Table containing the number of significant characters in
!                    the keyword table codelc
!     ncodmxlc         Table containing the maximal number of  characters in
!                    the keyword table codelc
!     nextra         Number of extra positions in arrays with main keywords to
!                    store subsubkeywords
!     nncodelc       length of array codelc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSH037         fill array iinput with information of elements when no
!                    card with mesh.... is read
!     MSHRD0         Read information of user points
!     MSHRD3         Read information of line elements
!     MSHRD4         Read information of surface elements
!     MSHRD5         Fill last part of arrays iinput and rinput
!     MSHRD7         Read information of volume elements
!     MSHRD8         Fill information about mesh connect elements
!     MSHRD9         Fill information about interface elements
!     MSHRDDUMMY     Fill information about dummy element groups
!     MSHREADCUR     Read information of curves
!     MSHREADSUR     Read information of surfaces
!     MSHREADVOL     Read information of volumes
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    This subroutine reads the mesh input file from the part points
!    The following input is recognized:
!       points
!          data records
!       curves
!          data records
!       surfaces
!          data records
!       volumes
!          data records
!       meshline
!          data records
!       meshsurf
!          data records
!       meshvolume
!          data records
!       meshconnect
!          data records
!       meshdummy
!          data records
!       interface_elements
!          data records
!       change_coordinates
!          data records
!       obstacles
!          data records
!       renumber, options
!       norenumber
!       notopology
!       intermediate points
!          data records
!       plot, options
!       refine, options
!       transform, options
!       check_level = i
!       write_faces
!       parallel
!       end
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     142   Internal error, too few space for integers
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc / 'SURFACES', 'VOLUMES', 'MESHLINE', 'MESHSURF', &
                    'MESHVOLUME', 'RENUMBER', 'PLOT   ', 'END    ', &
                    'MESHCONNECT', 'NORENUMBER', 'INTERMEDIATE', &
                    'REFINE', 'TRANSFORM', 'CHECK_LEVEL', 'NOTOPOLOGY', &
                    'INTERFACE_ELEMENTS', 'CHANGE_COORDINATES', &
                    'OBSTACLES', 'WRITE_FACES', 'PARALLEL', 'MESHDUMMY', &
                     nextra*' ' /
      data ncodelc / 4, 4, 7, 7, 7,   5, 4, 3, 7, 5, &
                     7, 6, 6, 4, 5,   7, 9, 5, 9, 5, &
                     7, &
                     nextra*0 /
      data ncodmxlc/ 8, 7, 8, 8,10,   8, 4, 3,11,10, &
                    12, 6, 9,11,10,  18,18, 9,11, 8, &
                     9, &
                     nextra*0 /

!    codelc      In this table the significant characters of the various
!              texts after renumber are stored
!              The following types of texts are recognized:
!              1:   SURFACES
!              2:   VOLUMES
!              3:   MESHLINE
!              4:   MESHSURF
!              5:   MESHVOLUME
!              6:   RENUMBER
!              7:   PLOT
!              8:   END
!              9:   MESHCONNECT
!             10:   NORENUMBER
!             11:   INTERMEDIATE POINTS
!             12:   REFINE
!             13:   TRANSFORM
!             14:   CHECK_LEVEL
!             15:   NOTOPOLOGY
!             16:   INTERFACE_ELEMENTS
!             17:   CHANGE_COORDINATES
!             18:   OBSTACLES
!             19:   WRITE_FACES
!             20:   PARALLEL
!             21:   MESHDUMMY
!     icode          return code,  possibilities:
!                     0       next card:  surfaces
!                     1       next card:  volumes
!                     2       next card:  meshline
!                     3       next card:  meshsurface
!                     4       next card:  meshvolume
!                     5       next card:  renumber
!                     6       next card:  plot
!                     7       next card:  end
!                     8       next card:  meshconnect
!                     9       next card:  norenumber
!                    10       next card:  intermediate points
!                    11       next card:  refine
!                    12       next card:  transform
!                    13       next card:  check_level
!                    14       next card:  notopol
!                    15       next card:  interface_elements
!                    16       next card:  change_coordinates
!                    17       next card:  obstacles
!                    18       next card:  write_faces
!                    19       next card:  parallel
!                    20       next card:  meshdummy
! ======================================================================
!
!     --- Clear some standard parameters

      call eropen ( 'mshrea' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrea'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      nncodemain = nncodelc
      codemain(1:nncodemain+nextra) = codelc(1:nncodemain+nextra)
      ncodemain(1:nncodemain+nextra) = ncodelc(1:nncodemain+nextra)
      ncodmxmain(1:nncodemain+nextra) = ncodmxlc(1:nncodemain+nextra)

!     --- Clear first part of iinput

      iinput(1:15) = 0

!     --- Set some parameters

      nelgrp = 0
      nlines = 0
      nsurel = 0
      nvolel = 0
      nvolms = 0
      adaptiinput = .false.
      composrf = .false.
      nelgrpdum = 0
      lengin = 15
      maxratio = 1d0

      lenipoint = max(maxcrv, maxsrf, maxvlm)
      allocate ( ipoint(lenipoint,3), intprop(10,lenipoint), &
                 realprop(10,lenipoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 23*lenipoint, 'ipoint' )
      if ( ierror/=0 ) go to 1000

      iinput(11) = jcoars
      intprop = 0
      realprop = 0d0
      ipoint = 0

      call inichkarlen ( 2+(ndim+2)*maxpnt, lnrinp, 'rinput' )
      if ( ierror/=0 ) go to 1000

!     --- read point cards

      call mshrd0 ( rinput(3), ndim, maxpnt, &
                    rinput(3+ndim*maxpnt), iinput )
      lengrl = 2 + maxpnt * (ndim+jcoars)+2*jcoars

      if ( debug ) then
         write(irefwr,*) 'ndim, maxpnt', ndim, maxpnt
         call prinin ( iinput, 15, 'iinput/1' )
      end if  ! ( debug )

      call inichkarlen ( lengrl+1+3*maxcrv, lniinp, 'rinput' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'icode', icode

      if ( icode==-1 ) then

!     --- read curve cards

         call mshreadcur ( iinput, rinput, lengrl, lengin, &
                           cunit, maxratio, ipoint )

      else

         nuspnt = iinput(2)
         lengrl = 2 + nuspnt * (ndim+jcoars)+2*jcoars

         if ( jcoars/=0 ) then

!        --- If jcoars = 1, the second part of RINPUT is put in the correct
!            positions

            i1 = 3+ndim*maxpnt
            i2 = 4+ndim*nuspnt
            rinput(i2:i2+nuspnt-1) = rinput(i1:i1+nuspnt-1)
            if ( debug ) &
               write(irefwr,2) 'cunit, maxratio', cunit, maxratio

            rinput(3+ndim*nuspnt) = cunit
            rinput(4+(ndim+1)*nuspnt+1) = maxratio

         end if

      end if  ! ( icode==-1 )

      if ( debug ) call prinin ( iinput, lengin, 'iinput/2' )

      if ( icode==0 ) then

!     --- read surface cards  when icode = 0

         call mshreadsur ( iinput, lengin, rinput, lengrl, composrf, &
                           ipoint, lenipoint, adaptiinput )

      end if

      if ( debug ) then
         write(irefwr,1) 'icode/2', icode
         call prinin ( iinput, lengin, 'iinput/2a' )
      end if  ! ( debug )

      if( icode==1 ) then

!     --- read volume cards when icode = 1

         call mshreadvol ( iinput, lengin, lenipoint, ipoint )

      end if

      if ( debug ) then
         write(irefwr,1) 'icode/3', icode
         call prinin ( iinput, lengin, 'iinput/3' )
      end if  ! ( debug )

      nintprop = 0
      nrealprop = 0
      if ( icode>4 ) then

!     --- icode>4  no mesh card found

         call msh037 ( iinput, nlines, nsurel, nvolel )
         lengin = lengin+4

      else

!     --- icode = 2 or 3, mesh cards found

         if ( icode==2 ) then

!        --- icode = 2  line elements

            call mshrd3 ( iinput(lengin+1), nlines, 10*lenipoint, nelgrp, &
                          nintprop, nrealprop, intprop, realprop, iinput )
            lengin = lengin+4*nlines
            if ( debug ) &
               write(irefwr,1) 'nintprop, nrealprop', nintprop, nrealprop

         end if

         if ( debug ) then
            write(irefwr,1) 'icode/4', icode
            call prinin ( iinput, lengin, 'iinput/4' )
         end if  ! ( debug )

         if ( icode==3 ) then

!        --- icode = 3; read cards for surface elements

            call mshrd4 ( iinput(lengin+1), nsurel, iinput, &
                          lniinp-lengin, nelgrp, iinput(iinp3), nintprop, &
                          nrealprop, intprop(1,nelgrp+1), &
                          realprop(1,nelgrp+1), nvolms )
            lengin = lengin+4*nsurel
            if ( debug ) &
               write(irefwr,1) 'nintprop, nrealprop/2', nintprop, nrealprop

         end if

         if ( debug ) write(irefwr,1) 'icode/5', icode

         if ( icode==4 ) then

!        --- icode = 4,  Read mesh volume cards

            if ( debug ) write(irefwr,*) 'before mshrd7'
            call mshrd7 ( iinput(lengin+1), nvolel, nvolms, &
                          lniinp-lengin, &
                          nelgrp, iinput(iinp5), nintprop, nrealprop, &
                          intprop(1,nelgrp+1), realprop(1,nelgrp+1) )
            lengin = lengin+4*nvolel
            if ( debug ) &
               write(irefwr,1) 'nintprop, nrealprop/3', nintprop, nrealprop

         end if

      end if

      if ( debug ) write(irefwr,1) 'icode/6', icode

      if ( icode==8 ) then

!     --- icode = 8   mesh connect

         if ( debug ) write(irefwr,1) 'lengin before mshrd8', lengin
         call mshrd8 ( iinput, iinput(lengin+1), nelgrp, lniinp-lengin, &
                       leng )
         lengin = lengin+leng

      end if
      if ( debug ) write(irefwr,1) 'icode/8', icode
      lengintf = 0
      if ( icode==15 ) then

!     --- icode = 15   interface elements

         call mshrd9 ( ipoint, nelgrp, lenipoint*3, iinput, lengintf )

      end if
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'icode/9', icode
      if ( icode==20 ) then

!     --- meshdummy found, store information in array iinputdummy

         call mshrddummy ( nelgrp, iinputdummy, nelgrpdum, &
                           leniinputdummy )

      end if
      if ( ierror/=0 ) go to 1000

!     --- fill last positions of iinput and rinput

      if ( debug ) write(irefwr,*) 'before mshrd5'
      call mshrd5 ( iinput, rinput, lengin, lniinp, lengrl, ipoint, &
                    lengintf, iwork, nextra, iwork(41), iinputdummy )

      if ( lengin>lniinp ) then
         call errint ( lniinp, 1 )
         call errint ( lengin, 2 )
         call errsub ( 142, 2, 0, 0 )
         go to 1000
      end if

      if ( maxpnt/=1000 .or. maxcrv/=1000 .or. maxsrf/=1000 &
           .or. maxvlm/=500 ) then

!     --- Defaults can not be used, hence fill maxpnt, etc. in iinput

         iinput(13) = lengin + 6
         iinput(lengin+1) = maxpnt
         iinput(lengin+2) = maxcrv
         iinput(lengin+3) = maxsrf
         iinput(lengin+4) = maxvlm
         lengin = lengin + 4

      end if
      if ( lengin+4>lniinp ) then
         call errint ( lniinp, 1 )
         call errint ( lengin+4, 2 )
         call errsub ( 142, 2, 0, 0 )
         go to 1000
      end if

      deallocate ( ipoint, intprop, realprop, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ipoint' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinput, lengin, 'iinput' )
         call prinrl ( rinput, lengrl, 'rinput' )
         write(irefwr,*) 'End mshrea'

      end if
      call erclos ( 'mshrea' )

      end subroutine mshrea
