      subroutine mshreadcur ( iinput, rinput, lengrl, lengin, &
                              cunit, maxratio, ipoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-11-2014
!
! !   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information of curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iinput(*), lengin, lengrl
      integer, intent(out) :: ipoint(*)
      double precision, intent(in) :: cunit, maxratio
      double precision, intent(inout) :: rinput(*)

!     cunit          i    Unit length for coarseness
!     iinput        i/o   integer input array
!                         See subroutine mshrea
!     ipoint         o    pointer array: contains the starting positions of the
!                         curves in array iinput
!     lengin        i/o   Length of integer array iinput
!     lengrl        i/o   Length of real array rinput
!                         (in double precision variables)
!     maxratio       i    Maximal ratio between succeeding elements
!     rinput        i/o   Real input array
!                         See subroutine mshrea
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lengrlus
      integer :: inpmax, i1, i2
      integer, allocatable :: iinputloc(:)

!     i1             Help variable to compute a constant
!     i2             Help variable to compute a constant
!     iinputloc      Local array iinput
!     inpmax         Available space for array iinput
!     lengrlus       computed length of array rinput with respect to user points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH014         Rearrange information in IINPUT and RINPUT wrt curves
!     MSHRD1         Read information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreadcur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreadcur'
         call prinin ( iinput, 15, 'iinput' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpmax = 10000
      allocate ( iinputloc(inpmax), stat = error )
      if ( error/=0 ) call eralloc ( error, inpmax, 'iinputloc' )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      ndim = iinput(1)
      nuspnt = iinput(2)
      jcoars = iinput(11)

      call mshrd1 ( iinputloc, rinput(lengrl+1), &
                    ipoint, inpmax, lengin, rinput(lengrl+1+maxcrv), &
                    rinput(3), rinput(3+ndim*maxpnt), cunit )

      if ( debug ) then
         write(irefwr,1) 'lengin after mshrd1', lengin
         call prinin ( iinput, 15, 'iinput/2' )
      end if  ! ( debug)

      if ( jcoars/=0 ) then

!     --- If jcoars = 1, the second part of RINPUT is put in the correct
!         positions

         i1 = 3+ndim*maxpnt
         i2 = 4+ndim*nuspnt
         rinput(i2:i2+nuspnt-1) = rinput(i1:i1+nuspnt-1)
         if ( debug ) write(irefwr,2) 'cunit, maxratio', cunit, maxratio

         rinput(3+ndim*nuspnt) = cunit
         rinput(4+(ndim+1)*nuspnt+1) = maxratio

      end if
      lengrlus = 2 + nuspnt * (ndim+jcoars)+2*jcoars

!     --- Rearrange rinput

      rinput(lengrlus+1:lengrlus+ncurvs) = &
         rinput(lengrl+1:lengrl+ncurvs)
      rinput(lengrlus+1+ncurvs:lengrlus+3*ncurvs) = &
         rinput(lengrl+1+maxcrv:lengrl+maxcrv+2*ncurvs)
      lengrl = lengrlus+ncurvs*3

!     --- Rearrange information in IINPUT with respect to curves

      call msh014 ( iinput, iinputloc, ipoint, ncurvs, ndim )
      if ( ierror/=0 ) go to 1000

      lengin = lengin + 14
      iinp3 = 1

!     --- Fill parameters

      iinput(3) = ncurvs

      deallocate ( iinputloc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinputloc' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icode/1, lengin', icode, lengin
         call prinin ( iinput, lengin, 'iinput/end' )
         write(irefwr,*) 'End mshreadcur'

      end if  ! ( debug )
      call erclos ( 'mshreadcur' )

      end subroutine mshreadcur
