      subroutine mshreadsur ( iinput, lengin, rinput, lengrl, composrf, &
                              ipoint, lenipoint, adaptiinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information of surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lenipoint
      integer, intent(inout) :: iinput(*), lengin, lengrl, &
                                ipoint(lenipoint,2)
      double precision, intent(inout) :: rinput(*)
      logical, intent(inout) :: composrf, adaptiinput

!     adaptiinput   i/o   If true iinput must be adapted
!     composrf       i    If true composite elements have been found
!     iinput        i/o   integer input array
!                         See subroutine mshrea
!     ipoint         o    pointer array: contains the starting positions of the
!                         curves (1) and surfaces (2) in array iinput
!     lengin        i/o   Length of integer array iinput
!     lengrl        i/o   Length of real array rinput
!                         (in double precision variables)
!     lenipoint      i    Length of array ipoint
!     rinput        i/o   Real input array
!                         See subroutine mshrea
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: leng, i
      integer :: inpmax
      integer, allocatable :: iinputloc(:)
      logical :: part4, changeinp

!     changeinp      if true extra curves have been created
!     i              Counting variable
!     iinputloc      Local array iinput
!     inpmax         Available space for array iinput
!     leng           Help variable to compute the length of some arrays
!     part4          If true rinput part 4 is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH015         Rearrange information in IINPUT with respect to surfaces
!     MSHRD2         Read information of surfaces
!     MSHREADSURCHA  Change contents of array iinput and ipoint due to special
!                    surfaces
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreadsur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreadsur'
         write(irefwr,1) 'lengrl', lengrl
         call prinin ( iinput, lengin, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpmax = 10000
      allocate ( iinputloc(inpmax), stat = error )
      if ( error/=0 ) call eralloc ( error, inpmax, 'iinputloc' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( ipoint, ncurvs, 'icurvs' )

      nsurfs = 0

      call mshrd2 ( iinputloc, rinput(lengrl+1), &
                    ipoint(1,2), inpmax, leng, ipoint(1,1), &
                    composrf, rinput(lengrl+1+4*maxsrf), part4, &
                    adaptiinput, lenipoint, changeinp )
      leng = leng-1

      if ( part4 ) then

!     --- starting address of thickness and factors for each curve

         irnp4 = lengrl+1+4*nsurfs

!        --- copy rpin to rinput

         do i = 1, 4*ncurvs
            rinput(lengrl+i+4*nsurfs) = rinput(lengrl+4*maxsrf+i)
         end do
         lengrl = lengrl + 4 * ncurvs

      end if
      lengrl = lengrl + 4 * nsurfs

      if ( changeinp ) then

!     --- The input for curves may be changed, because curves have been created
!         by surfaces

         call mshreadsurcha ( iinput, iinputloc, ipoint, lenipoint, &
                              leng, lengin, nsurfs, lengrl, rinput, &
                              maxcrv )

      end if  ! ( changeinp )

!     --- Rearrange information in IINPUT with respect to surfaces

      call msh015 ( iinput, iinputloc, ipoint(1,2), nsurfs, lengin )
      if ( ierror/=0 ) go to 1000
      iinp3 = lengin+1
      lengin = lengin+leng

      deallocate ( iinputloc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinputloc' )

!     --- Fill parameters

      iinput(4) = nsurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshreadsur'

      end if  ! ( debug )
      call erclos ( 'mshreadsur' )

      end subroutine mshreadsur
