      subroutine mshreadvol ( iinput, lengin, lenipoint, ipoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information of volumes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lenipoint
      integer, intent(inout) :: iinput(*), lengin, ipoint(lenipoint,3)

!     iinput        i/o   integer input array
!                         See subroutine mshrea
!     ipoint        i/o   pointer array: contains the starting positions of the
!                         curves (1), surfaces (2) and volumes(3) in array
!                         iinput
!     lengin        i/o   Length of integer array iinput
!     lenipoint      i    Length of array ipoint
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: leng
      integer :: inpmax
      integer, allocatable :: iinputloc(:)

!     iinputloc      Local array iinput
!     inpmax         Available space for array iinput
!     leng           Help variable to compute the length of some arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH027         Rearrange information in IINPUT with respect to volumes
!     MSHRD6         Read information of volumes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreadvol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreadvol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpmax = 10000
      allocate ( iinputloc(inpmax), stat = error )
      if ( error/=0 ) call eralloc ( error, inpmax, 'iinputloc' )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      nuspnt = iinput(2)
      ncurvs = iinput(3)
      nsurfs = iinput(4)

      call mshrd6 ( iinputloc, &
                    ipoint(1,3), inpmax, leng, ipoint(1,2) )
      leng = leng-1

!     --- Rearrange information in IINPUT with respect to volumes

      call msh027 ( iinput, iinputloc, ipoint(1,3), nvolms, lengin )
      iinp5 = lengin+1
      lengin = lengin+leng

      deallocate ( iinputloc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinputloc' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshreadvol'

      end if  ! ( debug )
      call erclos ( 'mshreadvol' )

      end subroutine mshreadvol
