      subroutine mshrmobstmshp ( kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Remove most mesh subarrays that have been created while making
!    an obstacle mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     delete
!     mesh
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh

!     kmesh         i/o   Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, kelmad

!     i              Counting variable
!     kelmad         memory management reference number of array KMESH part ad
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHDELSEL      Delete a selected set of subarrays of kmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrmobstmshp' )

!     --- Remove subarrays

      iseqkm = kmesh
      if ( km(iseqkm)%kmesh(17)>0 ) then
         deallocate ( km(iseqkm)%kmeshc )
         km(iseqkm)%kmesh(17) = 0
      end if  ! ( km(iseqkm)%kmesh(17)>0 )

!     --- kmeshl/m/n

      if ( km(iseqkm)%kmesh(26)>0 ) then
         deallocate ( km(iseqkm)%kmeshl )
         km(iseqkm)%kmesh(26) = 0
      end if  ! ( km(iseqkm)%kmesh(26)>0 )
      if ( km(iseqkm)%kmesh(27)>0 ) then
         deallocate ( km(iseqkm)%kmeshm )
         km(iseqkm)%kmesh(27) = 0
      end if  ! ( km(iseqkm)%kmesh(27)>0 )
      if ( km(iseqkm)%kmesh(28)>0 ) then
         deallocate ( km(iseqkm)%kmeshn )
         km(iseqkm)%kmesh(28) = 0
      end if  ! ( km(iseqkm)%kmesh(28)>0 )

!     --- kmesh ad

      if ( associated(km(iseqkm)%ielemobst) ) &
         deallocate ( km(iseqkm)%ielemobst )

!     --- Remove selected part of kmesh

      call mshdelsel ( kmesh )

      call erclos ( 'mshrmobstmshp' )
      end
