      subroutine mshspadapcoor ( kmeshcnew, kmeshmnew, coornew, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-02-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt the coordinates of midside points of triangles in case the
!     boundary is curved and spectral points are placed
!     Restriction: at this moment displacement of nodes is only programmed
!     for triangles with an odd number of nodes on the edges
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     spectral
!     global
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), kmeshmnew(*), iinput(*)
      double precision coornew(2,*)

!     coornew          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iinput         i    integer input array
!     kmeshcnew         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshmnew         i    KMESH part m, contains nodal point numbers of the
!                         curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, i, i1, i2, icurnr, ielgrp, &
              ikelmc, istar3, k
      integer, allocatable :: iwork(:)

!     i              General loop variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icurnr         Curve sequence number
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     iwork          integer work array of length npoint
!                    If iwork(i) = 1, node i corresponds to a point on a
!                    curve, otherwise (0) it is an internal point
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     MSHSPADAPCOOR1 Adapt the coordinates of midside points of triangles in
!                    case the boundary is curved and spectral points are placed
!                    One element group only
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2298   Adaptation of curvature for intermediate points only implemented
!           for odd numbers of intermediate points
! **********************************************************************
!
!                       PSEUDO CODE
!
!     niedge := iinput(1)
!     if ( niedge even ) warning
!     Clear array iwork
!     Set iwork(i) 1 for all nodes i on curves
!     For ielgrp := 1 (1) nelgrp do
!        ishape = kmeshf(ielgrp)
!        if ( ishape = 3 ) then
!           adapt coordinates of all triangles in element group
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshspadapcoor' )
      if ( ndim/=2 ) go to 1000

!     --- niedge := iinput(1)

      niedge = iinput(1)
      if ( mod(niedge,2)==0 ) then

!     --- if ( niedge even ) warning

         call errint ( niedge, 1 )
         call errwar ( 2298, 1, 0, 0 )
         go to 1000

      end if
      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

!     --- Set iwork(i) 1 for all nodes i on curves

      istar3 = kmeshmnew(2)-1
      do icurnr = 1, ncurvs
         i1 = kmeshmnew(icurnr+4)
         i2 = kmeshmnew(icurnr+5)-1
         do k = i1, i2
            i = kmeshmnew(istar3+k)
            if ( i>0 ) iwork(i) = 1
         end do

      end do

!     --- For ielgrp := 1 (1) nelgrp do

      ikelmc = 1
      inpelm = npelm
      inelem = nelem
      do ielgrp = 1, nelgrp

!     --- ishape = kmeshf(ielgrp)

         ishape = kmeshf(ielgrp)
         if ( kelmg>0 ) inelem = kmeshg(ielgrp)
         if ( kelma>0 ) inpelm = kmesha(ielgrp+1)-kmesha(ielgrp)
         if ( ishape==3 ) then

!        --- ishape = 3
!            adapt coordinates of all triangles in element group

            call mshspadapcoor1 ( kmeshcnew(ikelmc), coornew, inpelm, &
                                  iwork )

         end if
         ikelmc = ikelmc+inelem*inpelm

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'mshspadapcoor' )
      end
