      subroutine mshspadapcoor1 ( kmeshcnew, coornew, inpelm, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 24-09-2003 New call to chksbm
!        version  2.0  date 22-03-2003 New update method (Paul Bonnier)
!        version  1.0  date 02-02-2001
!
!   copyright (c) 2001-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt the coordinates of midside points of triangles in case the
!     boundary is curved and spectral points are placed
!     One element group only
!     Restriction: at this moment displacement of nodes is only programmed
!     for triangles with an odd number of nodes on the edges
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     spectral
!     global
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshcnew(inpelm,inelem), iwork(*)
      double precision coornew(2,*)

!     coornew        i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     inpelm         i    number of points per element
!     iwork          i    integer work array of length npoint
!                         If iwork(i) = 1, node i corresponds to a point on a
!                         curve, otherwise (0) it is an internal point
!     kmeshcnew      i    kmesh part c, contains the nodal points
!                         corresponding to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ielem, ielem1, iedge, iedge1, ifirst, ilast, imid, &
              imid1, ind(4), niter, maxiter, iedge2, mid, nside, &
              ifirst1, ilast1, ip, imethod, ielem0
      double precision xmean, ymean, dx, eps, diff, udisplacement(2), &
                       vdisplacement(2), tang(2), normal1(2), &
                       leng, alpha, alpha1, xmean1, ymean1, x, y, &
                       xc, yc, radius, beta1, dy, dl, dist, ddist, &
                       dist1, xn, yn, xi, eta
      logical changed
      integer, allocatable :: iwork1(:,:)

!     alpha          multiplication factor
!     alpha1         multiplication factor
!     beta1          multiplication factor
!     changed        If true coordinates have been changed
!     ddist          distance of point to a line
!     diff           Help parameter to compute the difference of two
!                    parameters
!     dist           Distance of point to line from (x1,y1) to (x2,y2)
!                    line: a*x + b*y - c = 0
!                    where (a,b) is the normal vector to the line
!                    a=-(y2-y1), b=(x2-x1), scaled to unit length
!                    a=-(y2-y1)/ sqrt( (x2-x1)^2 + (y2-y1)^2 )
!                    b= (x2-x1)/ sqrt( (x2-x1)^2 + (y2-y1)^2 )
!                    c= a*x1 + b*y1  (or a*x2 + b*y2)
!                    dist = a*x + b*y - c : distance of point (x,y) to line
!     dist1          distance of point to line (before point is moved)
!     dl             length of vector
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     eps            Local accuracy
!     eta            Coordinate in reference element
!     i              Counting variable
!     iedge          edge sequence number
!     iedge1         edge sequence number
!     iedge2         edge sequence number
!     ielem          Element number
!     ielem0         element number of the first element connected to
!                    point imid
!     ielem1         First element number in surface
!     ifirst         First node on edge
!     ifirst1        First node on edge
!     ilast          Last node on edge
!     ilast1         Last node on edge
!     imethod        Defines whether the new method (1) is used or the old one
!                    (2)
!     imid           Mid side node
!     imid1          Mid side node
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3,4
!     ip             nodal point number
!     iwork1         integer work array of size 4 x npoint
!                    iwork1(1,i) contains the first element corresponding
!                                to node i
!                    iwork1(2,i) contains the corresponding edge sequence
!                                number
!                    iwork1(3,i) contains the second element corresponding
!                                to node i
!                    iwork1(4,i) contains the corresponding edge sequence
!                                number
!                    iwork1 is only filed for mid-side points
!     j              Counting variable
!     leng           Help parameter to indicate the length of vectors
!     maxiter        maximum number of iterations
!     mid            Defines relative position of mid-side points on edge
!     niter          Iteration sequence number
!     normal1        First normal vector
!     nside          Number of ndoes along edge
!     radius         Radius of sphere or circle
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     udisplacement  Array containing the displacement of a point compared
!                    to the non-curved shape
!     vdisplacement  Array containing the displacement of a point compared
!                    to the non-curved shape
!     x              x co-ordinate of point
!     xc             x co-ordinate of entroid
!     xi             Coordinate in reference element
!     xmean          mean values of x co-ordinates
!     xmean1         mean values of x co-ordinates
!     xn             x-component of normal vector
!     y              y co-ordinate of point
!     yc             y co-ordinate of entroid
!     ymean          mean values of y co-ordinates
!     ymean1         mean values of y co-ordinates
!     yn             y-component of normal vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHKSBM         Check volume of submesh
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     243   Elements with more than 15 nodes not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!                    Old Method
!
!    First only the midpoints are treated
!    For each midpoint we proceed as follows
!       Find the two elements it is lying in
!       t = tangental vector along the edge corresponding to the midpoint
!       n = normal vector
!       alpha = sum of displacements of the midpoints of the other 4 edges
!               of the two triangles in the direction of the normal
!               a displacement is the difference between the actual position
!               of the point and the mean value of begin and end point
!       move mid point over a distance alpha/2 in the direction of the normal
!
!    Next the other points on the edge are moved
!    For each edge
!       alpha is displacement of mid point in the directin of the normal
!       if ( |alpha|>eps ) then
!          assume that there is an arc defined by the midpoint and the two end
!          points of the edge
!          Move the other points on the arc by defining a line from the centroid
!          through the point
!          To compute the centroid we use:
!
!          x Centroid
!          |\                      M_O is original mid point
!          | \                     M   is moved mid point
!          |  \
!          |   \                   d = e+a
!        e |    \  d               d^2 = e^2+a^2+2ae
!          |     \                 c^2+e^2 = d^2
!          |      \                Hence
!          |   c   \               c^2 = a^2+2ae
!      M_o x-------x  end point    e = ( c^2-a^2)/2a
!          |      /                r = a+e
!       a  |   /
!          | /  b
!       M  x
!
!    To place the other points use the same arc:
!
!          x Centroid
!          |\
!          | \
!          |  \
!          |   \                 x_n - x_c + r/a ( x_p - x_c)
!          |    \
!          |     \
!          |      \
!          |       \ x_p
!      M_o x--------x-----------x
!          |         \       /
!       a  |          x x_n
!          |     /
!       M  x
!
!
!           New method, due to Paul Bonnier
!
!   Consider the normal displacements of the other sides of the quadrilateral
!   around the centre point and compute the mean value, see Figure
!
!   5       8       9
!   *-------*-------* &
!   | \             |
!   |   \     jEl   |
!   |     \         |
!   *6      *4      *7
!   |   iEl   \     |
!   |           \   |
!   |             \ |
!   *-------*-------* &
!   1       2       3
!
!   We start from element iEl. If node 2 goes outwards, then node 4 will
!   go inwards. The same is true for node 6. The same is true for the
!   other element jEl, but of course for this element the direction
!   of the edge 3-4-5 is negative compared to element iEl
!   To compute the displacement of node 4 we proceed as follows:
!   d4 = ( -(d2+d6) + (d7+d8) ) / 2
!   Now node 4 is moved such that the distance to line 3-5 is equal to d4
!   iEl = first  element connected to node 4 = iWork(1,imid)
!   jEl = second element connected to node 4 = iWork(3,imid)
!
!
!                       Pseudo code
!
!    Clear iwork1
!    For all midside points do
!       Fill iwork1
!    dx := minimum length of sides
!    nside := niedge+1
!    mid := niedge/2+1
!    diff := dx*10
!    eps := dx*0.05
!    changed := false
!    if ( old method ) then
!    while ( diff<eps ) do
!       diff := 0
!       For all elements in element group do
!          For all edges in element do
!             imid = node in middle of face
!             if ( iwork(imid) = 0 ) then
!                Find two elements corresponding to node
!                udisplacement = mean displacement of 4 surrounding midpoints
!                if ( |udisplacement|>eps ) then
!                   vdisplacement = displacement of point
!                   diff = |udisplacement-vdisplacement|
!                   move point
!                   if ( diff>eps ) then
!                      changed := true
!    else ( new method )
!    while ( diff<eps ) do
!       diff := 0
!       For all elements in element group do
!          For all edges in element do
!             imid = node in middle of face
!             if ( iwork(imid) = 0 ) then
!                Find two elements corresponding to node
!                dist = average distance to line of 4 surrounding mid-points
!                if ( |dist|>eps ) then
!                   dist1 = current distance of point
!                   diff = |dist-dist1|
!                   move point
!                   if ( diff>eps ) then
!                      changed := true
!    if ( changed ) then
!       adapt rest of points along edges
!        determine internal nodes for 15-noded elements
!        based on 6-noded shape functions
!
!
!         ^eta
!         |
!         9
!         | \
!         |  \
!        10   8
!         |    \
!         |     \
!         |      \
!        11  15   7               xi(15) = 0.25, eta(15) = 0.5
!         |        \
!         |         \
!         |          \
!        12  13  14   6           xi(13) = 0.25, eta(13) = 0.25
!         |            \          xi(14) = 0.50, eta(14) = 0.25
!         |             \
!         |              \
!         1---2---3---4---5 ----> xi
!         0  .25 .5  .75 1.0      xi
!       Typical shape function corner 1 : 2*(1-xi-eta)*(0.5-xi-eta)
!       Typical shape mid point       3 : 4*(  xi    )*(1.0-xi-eta)
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 2, 3, 1, 2 /
! ======================================================================
!
      call eropen ( 'mshspadapcoor1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshspadapcoor1'

      end if

      allocate ( iwork1(4,npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork1' )
      if ( ierror/=0 ) go to 1000

!     --- Clear iwork1

      iwork1 = 0

!     --- nside := niedge+2
!         mid := niedge/2+1

      nside = niedge+1
      mid = niedge/2+1

!     --- For all midside points do
!         Fill iwork1
!         dx := minimum length of sides

      dx = rinfin
      do ielem = 1, inelem

         do iedge = 1, 3

            ifirst = kmeshcnew(1+(iedge-1)*nside,ielem)
            imid = kmeshcnew(1+mid+(iedge-1)*nside,ielem)
            ilast = kmeshcnew(1+(ind(iedge)-1)*nside,ielem)
            if ( iwork1(1,imid)==0 ) then

!           --- First element corresponding to this edge

               iwork1(1,imid) = ielem
               iwork1(2,imid) = iedge

            else

!           --- Second element corresponding to this edge

               iwork1(3,imid) = ielem
               iwork1(4,imid) = iedge

            end if
            dx = min ( dx, (coornew(1,ifirst)-coornew(1,ilast))**2+ &
                           (coornew(2,ifirst)-coornew(2,ilast))**2 )

         end do

      end do
      dx = sqrt(dx)

!     --- diff := dx*10
!         eps := dx*0.05
!         changed := false

      diff = 10*dx
      eps = 0.05*dx
      changed = .false.
      niter = 1
      maxiter = 10  ! raised from 5 to 10
      imethod = 1   ! defines the type of method used
                    ! 1 is new method, 2 is old method

      if ( imethod==1 ) then

!     --- New method
!         while ( diff>eps ) do

         do while ( diff>eps .and. niter<=maxiter )

!        --- diff := 0

            diff = 0d0
            niter = niter+1

!           --- For all elements in element group do

            do ielem = 1, inelem

!           --- For all edges in element do

               do iedge = 1, 3

!              --- imid = node in middle of face

                  imid = kmeshcnew(1+mid+(iedge-1)*nside,ielem)
                  if ( iwork(imid)==0 ) then

!                 --- iwork(imid) = 0, hence no boundary point
!                     Find two elements corresponding to node
!                     udisplacement = mean displacement of 4 surrounding
!                     midpoints

                      dist       = 0d0
                      ifirst     = kmeshcnew(1+(iedge-1)*nside,ielem)
                      ilast      = kmeshcnew(1+(ind(iedge)-1)*nside,ielem)
                      tang(1)    = coornew(1,ilast)-coornew(1,ifirst)
                      tang(2)    = coornew(2,ilast)-coornew(2,ifirst)
                      leng     = 1d0/sqrt(tang(1)**2+tang(2)**2)
                      normal1(1) = -tang(2)*leng
                      normal1(2) =  tang(1)*leng
                      xmean = 0.5d0*(coornew(1,ifirst)+coornew(1,ilast))
                      ymean = 0.5d0*(coornew(2,ifirst)+coornew(2,ilast))

!                     --- ielem0 is the lower element number of the
!                         two connecting elements.

                      ielem0 = iwork1(1,imid)

!                     --- only perform the operations when the current
!                         element ielem is the lower element number ielem0
!                         otherwise, normal1 is in the opposit direction
!                         alternative: always determine normal1() on
!                         element iwork1(1,imid)

                      if (ielem0==ielem) then

                         do i = 1, 3, 2
                            ielem1 = iwork1(i,imid)
                            iedge1 = iwork1(i+1,imid)
                            do j = 0, 1
                               iedge2 = ind(iedge1+j)
                               ifirst1 = &
                                  kmeshcnew(1+(iedge2-1)*nside,ielem1)
                               imid1 = &
                                  kmeshcnew(1+mid+(iedge2-1)*nside,ielem1)
                               ilast1 = &
                                  kmeshcnew(1+(ind(iedge2)-1)*nside,ielem1)
                               xmean1 = 0.5d0* &
                                        (coornew(1,ifirst1)+coornew(1,ilast1))
                               ymean1 = 0.5d0* &
                                        (coornew(2,ifirst1)+coornew(2,ilast1))

                               dx = coornew(1,ilast1)-coornew(1,ifirst1)
                               dy = coornew(2,ilast1)-coornew(2,ifirst1)
                               dl = 1/sqrt(dx*dx+dy*dy)   ! length

!                              --- normal vector (xn,yn) points to the left
!                                  when looking from ifirst1 to ilast1

                               xn = -dy*dl
                               yn =  dx*dl

!                              --- distance is positive when point is to the
!                                  left of line

                               ddist = &
                                  xn * (coornew(1,imid1)-coornew(1,ifirst1)) + &
                                  yn * (coornew(2,imid1)-coornew(2,ifirst1))

!                              --- adapt distance to move point imid

                               dist = dist+ddist
                            end do ! j

!                           --- after first element, switch sign of distance

                            if (i==1) dist = -dist
                         end do ! i
                         dist  = 0.5d0*dist

                         if ( abs(dist)>eps ) then

!                        --- dist1: current distance of point imid to
!                            the line ifirst - ilast

                            dist1 = &
                               normal1(1)*(coornew(1,imid)-coornew(1,ifirst))+ &
                               normal1(2)*(coornew(2,imid)-coornew(2,ifirst))

!                           --- move point

                            coornew(1,imid) = normal1(1)*dist+xmean
                            coornew(2,imid) = normal1(2)*dist+ymean

                            diff = max ( diff, abs(dist1-dist) )

!                           --- if ( diff>eps ) then
!                               changed := true

                           if ( diff>eps ) changed = .true.

                        end if

                     end if ! ielem1=ielem

                  end if

               end do

            end do

         end do ! while

      else

!     --- old method
!         while ( diff>eps ) do

         do while ( diff>eps .and. niter<=maxiter )

!        --- diff := 0

            diff = 0d0
            niter = niter+1

!           --- For all elements in element group do

            do ielem = 1, inelem

!           --- For all edges in element do

               do iedge = 1, 3

!              --- imid = node in middle of face

                  imid = kmeshcnew(1+mid+(iedge-1)*nside,ielem)
                  if ( iwork(imid)==0 ) then

!                 --- iwork(imid) = 0, hence no boundary point
!                     Find two elements corresponding to node
!                     udisplacement = mean displacement of 4 surrounding
!                     midpoints

                      alpha = 0d0
                      ifirst = kmeshcnew(1+(iedge-1)*nside,ielem)
                      ilast = kmeshcnew(1+(ind(iedge)-1)*nside,ielem)
                      tang(1) = coornew(1,ilast)-coornew(1,ifirst)
                      tang(2) = coornew(2,ilast)-coornew(2,ifirst)
                      leng = 1d0/sqrt(tang(1)**2+tang(2)**2)
                      normal1(1) = -tang(2)*leng
                      normal1(2) =  tang(1)*leng
                      xmean = 0.5d0*(coornew(1,ifirst)+coornew(1,ilast))
                      ymean = 0.5d0*(coornew(2,ifirst)+coornew(2,ilast))

                      do i = 1, 3, 2
                         ielem1 = iwork1(i,imid)
                         iedge1 = iwork1(i+1,imid)
                         do j = 0, 1
                            iedge2 = ind(iedge1+j)
                            ifirst1 = kmeshcnew(1+(iedge2-1)*nside,ielem1)
                            imid1 = &
                               kmeshcnew(1+mid+(iedge2-1)*nside,ielem1)
                            ilast1 = &
                               kmeshcnew(1+(ind(iedge2)-1)*nside,ielem1)
                            xmean1 = 0.5d0* &
                                     (coornew(1,ifirst1)+coornew(1,ilast1))
                            ymean1 = 0.5d0* &
                                     (coornew(2,ifirst1)+coornew(2,ilast1))
                            udisplacement(1) = coornew(1,imid1)-xmean1
                            udisplacement(2) = coornew(2,imid1)-ymean1
                            alpha = alpha+ &
                                    udisplacement(1)*normal1(1)+ &
                                    udisplacement(2)*normal1(2)
                         end do
                     end do
                     alpha = 0.5d0*alpha

                     if ( niter>1 ) alpha = alpha / 1.25d0 ! PGB test

                     if ( abs(alpha)>eps ) then

!                    --- |udisplacement|>eps
!                        vdisplacement = displacement of point
!                        diff = |udisplacement-vdisplacement|
!                        move point

                         vdisplacement(1) = coornew(1,imid)-xmean
                         vdisplacement(2) = coornew(2,imid)-ymean
                         alpha1 = vdisplacement(1)*normal1(1)+ &
                                  vdisplacement(2)*normal1(2)
                         diff = max ( diff, abs(alpha1-alpha) )
                         coornew(1,imid) = normal1(1)*alpha+xmean
                         coornew(2,imid) = normal1(2)*alpha+ymean

!                        --- if ( diff>eps ) then
!                            changed := true

                        if ( diff>eps ) changed = .true.

                     end if

                  end if

               end do

            end do

         end do ! while

      end if

      if ( changed .and. niedge>1 ) then

!     --- changed = true: adapt rest of points along edges
!         For all elements in element group do

         do ielem = 1, inelem

!        --- For all edges in element do

            do iedge = 1, 3

!           --- imid = node in middle of face

               imid = kmeshcnew(1+mid+(iedge-1)*nside,ielem)
               if ( iwork(imid)==0 ) then

!              --- iwork(imid) = 0, hence no boundary point and point not
!                  treated yet

                   iwork(imid) = 2
                   ifirst = kmeshcnew(1+(iedge-1)*nside,ielem)
                   ilast = kmeshcnew(1+(ind(iedge)-1)*nside,ielem)
                   tang(1) = coornew(1,ilast)-coornew(1,ifirst)
                   tang(2) = coornew(2,ilast)-coornew(2,ifirst)
                   leng = 1d0/sqrt(tang(1)**2+tang(2)**2)
                   normal1(1) = -tang(2)*leng
                   normal1(2) =  tang(1)*leng
                   xmean = 0.5d0*(coornew(1,ifirst)+coornew(1,ilast))
                   ymean = 0.5d0*(coornew(2,ifirst)+coornew(2,ilast))
                   x = coornew(1,imid)
                   y = coornew(2,imid)
                   udisplacement(1) = x-xmean
                   udisplacement(2) = y-ymean
                   alpha = udisplacement(1)*normal1(1)+ &
                           udisplacement(2)*normal1(2)
                   if ( abs(alpha)>eps ) then

!                  --- |udisplacement|>eps
!                      adapt other points

                       leng = 0.5d0/leng
                       beta1 = (leng**2-alpha**2)/(2d0*alpha)
                       xc = xmean - beta1*normal1(1)
                       yc = ymean - beta1*normal1(2)
                       radius = sqrt((x-xc)**2+(y-yc)**2)
                       do j = 1, niedge

                          if ( j/=mid ) then

!                         --- Not the midside point because it has already
!                             been adapted

                             ip = kmeshcnew(j+1+(iedge-1)*nside,ielem)
                             x = coornew(1,ip)
                             y = coornew(2,ip)
                             alpha = radius/sqrt((x-xc)**2+(y-yc)**2)
                             coornew(1,ip) = xc + alpha*(x-xc)
                             coornew(2,ip) = yc + alpha*(y-yc)

                          end if

                       end do

                  end if

               end if

            end do ! iedge

!           --- finally internal nodes
!               At this moment only implemented for 15-noded elements
!               determine internal nodes for 15-noded elements
!               based on 6-noded shape functions
!                 ^eta
!                 |
!                 9
!                 | \
!                 |  \
!                10   8
!                 |    \
!                 |     \
!                 |      \
!                11  15   7               xi(15) = 0.25, eta(15) = 0.5
!                 |        \
!                 |         \
!                 |          \
!                12  13  14   6           xi(13) = 0.25, eta(13) = 0.25
!                 |            \          xi(14) = 0.50, eta(14) = 0.25
!                 |             \
!                 |              \
!                 1---2---3---4---5 ----> xi
!                 0  .25 .5  .75 1.0      xi
!               Typical shape function corner 1 : 2*(1-xi-eta)*(0.5-xi-eta)
!               Typical shape mid point       3 : 4*(  xi    )*(1.0-xi-eta)

            if ( inpelm==15 ) then

!           --- inpelm = 15

               do j = 13, 15

                 xi = 0.25d0
                 eta= 0.25d0
                 if (j==14) xi  = 0.5d0
                 if (j==15) eta = 0.5d0
                 xn=0
                 yn=0
                 xn= 2d0*(1d0-xi-eta)*(0.5d0-xi-eta) * &
                     coornew(1,kmeshcnew( 1,ielem)) &
                    -2d0*xi *(0.5d0-xi  ) * coornew(1,kmeshcnew( 5,ielem)) &
                    -2d0*eta*(0.5d0-eta ) * coornew(1,kmeshcnew( 9,ielem)) &
                    +4d0*xi *(1d0-xi-eta) * coornew(1,kmeshcnew( 3,ielem)) &
                    +4d0*xi *(   eta    ) * coornew(1,kmeshcnew( 7,ielem)) &
                    +4d0*eta*(1d0-xi-eta) * coornew(1,kmeshcnew(11,ielem))
                 yn= 2d0*(1d0-xi-eta)*(0.5d0-xi-eta) * &
                     coornew(2,kmeshcnew( 1,ielem)) &
                    -2d0*xi *(0.5d0-xi  ) * coornew(2,kmeshcnew( 5,ielem)) &
                    -2d0*eta*(0.5d0-eta ) * coornew(2,kmeshcnew( 9,ielem)) &
                    +4d0*xi *(1d0-xi-eta) * coornew(2,kmeshcnew( 3,ielem)) &
                    +4d0*xi *(   eta    ) * coornew(2,kmeshcnew( 7,ielem)) &
                    +4d0*eta*(1d0-xi-eta) * coornew(2,kmeshcnew(11,ielem))
                 i = kmeshcnew(j,ielem)
                 coornew(1,i) = xn
                 coornew(2,i) = yn

               end do

            else if ( inpelm>15 ) then

!           --- Not yet implemented

               call errint ( inpelm, 1 )
               call errwar ( 243, 1, 0, 0 )

            end if  ! ( inpelm==15 )

         end do ! ielem

      end if

!     --- Check volume of each element

      if ( inpelm==6 .or. inpelm==12 ) then

!     --- For Plaxis mesh generator a dummy routine is used for chksbm
!         call to chksbm blocked

        call chksbm ( kmeshcnew, inelem, 4, inpelm, 0, coornew, 2, 1, npoint )

      end if

      deallocate ( iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork1' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshspadapcoor1'

      end if
      call erclos ( 'mshspadapcoor1' )

      end
