      subroutine mshv4 ( coor, kmeshc, ipoly, lpoly, nr, izicht, &
                         ivier, npt, coarse, rim )
! ======================================================================
!
!        programmer    Niek Praagman
!        version 3.2   date 28-12-1998 layout
!        version 3.1   date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version 3.0   date 28-08-1986
!
!   copyright (c) 1984-1998 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   mshv4 is the driver for the case that an arbitrary polygon has
!   to be divided in quadrilaterals .
! **********************************************************************
!
!                       KEYWORDS
!
!    mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), lpoly(*), ipoly(*), nr(*), izicht(*), ivier, &
              npt
      double precision coor(*), coarse(*), rim

!     coarse     i    Array containing the coarseness per point
!     coor       o    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ipoly      i
!     ivier      i
!     izicht     i
!     kmeshc     o    kmesh part c, contains the nodal points corresponding
!                     to the elements
!     lpoly      i
!     npt        i    total number of points in the mesh
!     nr         i
!     rim        i    length of smallest edge along the boundary
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, i5, i6, iconc, ih, ipoln, j, j1, j2, &
              j3, j4, jn, jw, k, n, n1, nelloc, nh, nt
      double precision c1, c2, c3, cs, dx, dy, verh, w, wa, x1, x2, x3, &
                       x4, y1, y2, y3, y4, zijde1, zijde2, zijde3
      double precision :: work(lnwrk2)

!     c1
!     c2
!     c3
!     cs
!     dx         length of vector in x-direction
!     dy         length of vector in y-direction
!     i          Counting variable
!     i1
!     i2
!     i3
!     i4
!     i5
!     i6
!     iconc
!     ih         Help variable to store a constant temporarily
!     ipoln      length of ipoly
!     j          Counting variable
!     j1
!     j2
!     j3
!     j4
!     jn
!     jw
!     k          Counting variable
!     n
!     n1
!     nelloc      Number of elements
!     nh
!     nt
!     verh
!     w
!     wa
!     x1
!     x2
!     x3
!     x4
!     y1
!     y2
!     y3
!     y4
!     zijde1
!     zijde2
!     zijde3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     ERRWAR    Print warnings
!     INSTOP    Stop the program
!     MSHG01    Compute the cosine of an angle
!     MSHG02    Check whether angle is greater or smaller than 180 degrees
!     MSHV01    Put quadrilateral into kmeshc
!     MSHV02
!     MSHV04    divides a convex polygon into two new smaller
!               polygons by drawing the "most-appropriate" split-line
!     MSHV05    divides a concave polygon in two new ones by choosing the
!               most appropriate splitting line
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     350   Number of points in boundary too small
!     359   Number of vertices of boundaries for a submesh odd
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshv4' )

!     --- start the division of the polygons in lpoly

100   ipoln = 0
      jn = 1
      n = lpoly(jn)

 10   if ( n.le.3 ) then

!     --- if n < 4 input is wrong , hence stop execution

         call errint ( n, 1 )
         call errint ( 4, 2 )
         call errsub ( 350, 2, 0, 0 )
         go to 1000
      end if

!     --- check whether n is even

      n1 = n/2
      n1 = 2*n1
      n1 = n-n1
      if ( n1.ne.0 ) then

!     --- number of points odd , hence stop

         call errint ( n, 1 )
         call errsub ( 359, 1, 0, 0 )
         go to 1000

       end if

!      --- if n = 4 then ready

      if ( n.eq.4 ) goto 600

!     --- if n = 6 then a special division is required

      if ( n.eq.6 ) goto 800

!     --- n > 6 and  even
!         check if polygon is convex

      do i = 1, n
         nr(i+1) = lpoly(i+jn)
      end do

      nr(1) = nr(n+1)
      nr(n+2) = nr(2)
      wa = -1d0
      jw = 0
      iconc = 0

      do i = 1, n
         i1 = nr(i)
         i2 = nr(i+1)
         i3 = nr(i+2)
         x1 = coor(2*i1-1)
         x2 = coor(2*i2-1)
         x3 = coor(2*i3-1)
         y1 = coor(2*i1)
         y2 = coor(2*i2)
         y3 = coor(2*i3)
         call mshg02(x1, y1, x2, y2, x3, y3, ih, w)
         if ( ih.ne.0 ) then

!        --- ih = 1   so point is concave

            iconc = iconc+1
            if ( w.ge.wa ) then
               wa = w
               jw = i
            end if
         end if
      end do

      if (jw.eq.0) goto 200

!     --- jw # 0  concave , in jw the most
!         (call mshv05 to split the polygon)

      call mshv05(jw, nr, ipoly, ipoln, iconc, izicht, n, work, npt, coor, &
                  coarse, rim)

      goto 900

!     --- polygon is convex
!         ( fill array c with the cosines of the angles of the points)

200   do i = 1, n
         i1 = nr(i)
         i2 = nr(i+1)
         i3 = nr(i+2)
         x1 = coor(2*i1-1)
         x2 = coor(2*i2-1)
         x3 = coor(2*i3-1)
         y1 = coor(2*i1)
         y2 = coor(2*i2)
         y3 = coor(2*i3)
         call mshg01(x1, y1, x2, y2, x3, y3, cs)
         work(i+1) = cs
      end do

      work(1) = work(n+1)
      work(n+2) = work(2)

!     --- check if a quadrilateral of 4 succeeding boundary points
!         can be created

      i = 0

220   i = i+1
      if (i.eq.(n+1)) goto 300
      c1 = work(i+1)
      if (c1.lt.-0.05d0) goto 220

!     --- angle smaller than 95 degrees , consider the neighbour points

      c2 = work(i+2)
      c3 = work(i)

      if (c3.gt.c2) goto 225

!     --- if ( c2+c1 ) <  200 degrees :   = > cut off

      if (c2.lt.-0.94d0) goto 220

      if ((c2+c1).gt.-0.15d0) goto 230
      goto 220

!     --- if ( c3+c1 ) <  200 degrees  :  = > cut off

225   if (c3.lt.-0.94d0) goto 220

      if ((c3+c1).gt.-0.15d0) goto 240
      goto 220

!     --- check the length of the sides of the new quadrangle
!         ( i-1 , i , i+1 , i+2 is considered )

230   j = i+3
      if (i.eq.n) j = 3
      j1 = nr(i)
      j2 = nr(i+1)
      j3 = nr(i+2)
      j4 = nr(j)

!     --- side 1

      dx = coor(2*j2-1)-coor(2*j1-1)
      dy = coor(2*j2)-coor(2*j1)
      zijde1 = sqrt(dx*dx+dy*dy)

!     --- side 2

      dx = coor(2*j3-1)-coor(2*j2-1)
      dy = coor(2*j3)-coor(2*j2)
      zijde2 = sqrt(dx*dx+dy*dy)

!     --- side 3

      dx = coor(2*j4-1)-coor(2*j3-1)
      dy = coor(2*j4)-coor(2*j3)
      zijde3 = sqrt(dx*dx+dy*dy)

!     --- relations

      verh = zijde1/zijde2
      if (verh.lt.0.8d0.or.verh.gt.1.25d0) goto 220
      verh = zijde2/zijde3
      if (verh.lt.0.8d0.or.verh.gt.1.25d0) goto 220

!     --- quadrangle o.k.

      call mshv01(ivier, j1, j2, j3, j4, kmeshc)

!     --- renumber

      nh = n-2
      ih = i+1
      k = ipoln+1
      do j = 1, nh
          ih = ih+1
          if (ih.gt.n) ih = ih-n
          ipoly(k+j) = nr(ih+1)
      end do
      ipoly(k) = nh
      ipoln = ipoln+nh+1

      goto 900

!     --- (consider i-2 , i-1 , i , i+1 )

240   j = i-1
      if (i.eq.1) j = n
      j1 = nr(j)
      j2 = nr(i)
      j3 = nr(i+1)
      j4 = nr(i+2)

!     --- side 1

      dx = coor(2*j2-1)-coor(2*j1-1)
      dy = coor(2*j2)-coor(2*j1)
      zijde1 = sqrt(dx*dx+dy*dy)

!     --- side 2

      dx = coor(2*j3-1)-coor(2*j2-1)
      dy = coor(2*j3)-coor(2*j2)
      zijde2 = sqrt(dx*dx+dy*dy)

!     --- side 3

      dx = coor(2*j4-1)-coor(2*j3-1)
      dy = coor(2*j4)-coor(2*j3)
      zijde3 = sqrt(dx*dx+dy*dy)

!     --- compute quotients

      verh = zijde2/zijde1
      if ( verh.lt.0.8d0 .or. verh.gt.1.25d0 ) goto 220

      verh = zijde3/zijde2
      if ( verh.lt.0.8d0 .or. verh.gt.1.25d0 ) goto 220

!     --- quadrangle o.k.

      call mshv01(ivier, j1, j2, j3, j4, kmeshc)

!     --- renumber

      nh = n-2
      ih = i
      k = ipoln+1

      do j = 1, nh
          ih = ih+1
          if (ih.gt.n) ih = ih-n
          ipoly(k+j) = nr(ih+1)
      end do

      ipoly(k) = nh
      ipoln = ipoln+nh+1

      goto 900

!     --- call mshv04 to find most appropriate splitting line for polygon

300   nt = n

      call mshv04(ipoly, ipoln, nr, work, nt, npt, coor, coarse)

      goto 900

!     --- n = 4   = >  fill  kmeshc

600   i1 = lpoly(jn+1)
      i2 = lpoly(jn+2)
      i3 = lpoly(jn+3)
      i4 = lpoly(jn+4)

      call mshv01(ivier, i1, i2, i3, i4, kmeshc)

      goto 900

!     --- n = 6   = > special treatment

800   i1 = lpoly(jn+1)
      i2 = lpoly(jn+2)
      i3 = lpoly(jn+3)
      i4 = lpoly(jn+4)
      i5 = lpoly(jn+5)
      i6 = lpoly(jn+6)

      call mshv02(ivier, i1, i2, i3, i4, i5, i6, kmeshc, coor, npt, work)

      goto 900

!     --- ready with this polygon , anything left in lpoly ?

900   jn = jn+n+1
      n = lpoly(jn)

      if (n.gt.0) goto 10

!     --- n = 0   = >  lpoly is empty
!         is ipoly empty too ?

      if (ipoln.eq.0) goto 950

!     --- ipoln not zero , not yet ready
!         fill lpoly

      do i = 1, ipoln
         lpoly(i) = ipoly(i)
      end do

      lpoly(ipoln+1) = 0

!     --- start again

      goto 100

950   nelloc = ivier-1

!     --- check all quadrangles and give a warning when the angles are not
!            between the right boundaries

      do i = 1, nelloc
         i1 = kmeshc(4*i-3)
         i2 = kmeshc(4*i-2)
         i3 = kmeshc(4*i-1)
         i4 = kmeshc(4*i)
         x1 = coor(2*i1-1)
         x2 = coor(2*i2-1)
         x3 = coor(2*i3-1)
         x4 = coor(2*i4-1)
         y1 = coor(2*i1)
         y2 = coor(2*i2)
         y3 = coor(2*i3)
         y4 = coor(2*i4)
         call mshg02(x1, y1, x2, y2, x3, y3, ih, w)
         if (ih.eq.1) call errwar ( 374, 0, 0, 0 )
         call mshg01(x1, y1, x2, y2, x3, y3, w)
         if (w.lt.-.94d0) call errwar ( 373, 0, 0, 0 )
         call mshg02(x2, y2, x3, y3, x4, y4, ih, w)
         if (ih.eq.1) call errwar ( 374, 0, 0, 0 )
         call mshg01(x2, y2, x3, y3, x4, y4, w)
         if (w.lt.-.94d0) call errwar ( 373, 0, 0, 0 )
         call mshg02(x3, y3, x4, y4, x1, y1, ih, w)
         if (ih.eq.1) call errwar ( 374, 0, 0, 0 )
         call mshg01(x3, y3, x4, y4, x1, y1, w)
         if (w.lt.-.94d0) call errwar ( 373, 0, 0, 0 )
         call mshg02(x4, y4, x1, y1, x2, y2, ih, w)
         if (ih.eq.1) call errwar ( 374, 0, 0, 0 )
         call mshg01(x4, y4, x1, y1, x2, y2, w)
         if (w.lt.-.94d0) call errwar ( 373, 0, 0, 0 )

      end do

1000  call erclos ( 'mshv4' )
      if ( ierror.ne.0 ) call instop
      end
