      subroutine mshz05 ( trian, n1, n2, n3, n4, npelm, npedge, iblend, &
                          melem, mpoint )
! ======================================================================
!
!        programmer    Zdenek Vlasek
!        version  2.0  date 20-09-2010 Remove work arrays
!        version  1.1  date 24-03-1992 bug when trian and iblend=2,3
!        version  1.0  date 14-02-1990
!
!   copyright (c) 1990-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   To estimate all upper bounds for mshzde.
!   This subroutine is  n o t  called from the pack mshzde.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical trian
      integer n1, n2, n3, n4, npelm, npedge, iblend, melem, mpoint

!     iblend         i    User parameter for blending
!     melem          o    Upper bound for nelem
!     mpoint         o    Upper bound for npoint
!     n1             i    Number of elements along the  south  side
!                         of the current quadrilateral
!     n2             i    Number of elements along the  east  side
!                         of the current quadrilateral
!     n3             i    Number of elements along the  north side
!                         of the current quadrilateral
!     n4             i    Number of elements along the  west side
!                         of the current quadrilateral
!     npedge         i    Number of points per one element edge, both ends
!                         included
!     npelm          i    Number of points per element
!     trian          i    Indication whether triangles (true) or quadrilaterals
!                         are created (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer m1, m2, m3, m4, nedg, nsum, nbcord

!     m1             A permutation of n1,n2,n3,n4
!     m2             See m1
!     m3             See m1
!     m4             See m1
!     nbcord         Number of user's boundary points, see mshzde
!     nedg           Number of edges per element, = 3 or 4
!     nsum           n1+n2+n3+n4
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical mshz06

!     MSHZ06         Logical function. Is the number even ?
!     MSHZ13         Swap two integers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Estimation used here is exact for "easy" algorithm for quadrilater
!     For triangles supposes each quadrilteral be split in two triangles
!     For recursive algorithm the estimation is pessimistic, &
!     but still not safe.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      nsum = n1 + n2 + n3 + n4
      nbcord = nsum * ( npedge - 1 ) + 1

!-- create in  m1, m2, m3, m4  the standard situation
      m1 = n1
      m2 = n2
      m3 = n3
      m4 = n4
   10 continue
      if ( m1>m3 ) call mshz13 ( m1, m3 )
      if ( m4>m2 ) call mshz13 ( m4, m2 )
      if ( m3 - m1<m2 - m4 ) then
         call mshz13 ( m1, m4 )
         call mshz13 ( m2, m3 )
      end if
      if ( trian .and. .not. mshz06 ( m1+m2+m3+m4 ) ) then
         m1 = m1 + 1
         goto 10
      end if

!-- fill  melem
      if ( iblend==0 .or. iblend==1 .or. trian ) then

!     --- rather pessimistic estimation

         melem = ( max ( m1, m3 ) + 1 ) * ( max ( m2, m4 ) + 1 ) * 7 / 3
      else

!     --- exact value

         melem = m4 * m3 + m1 * ( m3 - m1 + m2 - m4 ) / 2
      end if
      if ( trian ) melem = 2 * melem

!-- fill  mpoint
      if ( trian ) then
         nedg = 3
      else
         nedg = 4
      end if

!-- count all vertices
      if ( iblend==0 .or. iblend==1 .or. trian ) then

!     --- unexact

         if ( trian ) then
            mpoint = melem / 2 + nbcord
         else
            mpoint = melem + nbcord
         end if
      else

!     --- exact

         mpoint = ( m4 + 1 ) * ( m3 + 1 ) &
             + ( m1 - 1 ) * ( m3 - m1 + m2 - m4 ) / 2 + ( m2 - m4 )
      end if

!-- add inner points on element boundaries
      mpoint = mpoint + ( melem * ( npedge - 2 ) * nedg &
                   + ( m1 + m2 + m3 + m4 ) * ( npedge - 2 ) ) / 2

!-- add inner element points
      mpoint = mpoint + melem * ( npelm - nedg * ( npedge - 1 ) )

!-- add point #1 which is stored twice
      mpoint = mpoint + 1
      end
