      subroutine mshz07 ( a1, b1, a2, b2, mrec, iQ, &
         nparts, kStore, mpoint, npoint, Coor, k1, k2, Ref, &
         nbcord, Tau, iblend )
!
! **********************************************************************
!
!       programmer      Zdenek Vlasek
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                  PURPOSE
!
!   Create new meshpoints on a line connecting two given points, &
!   the line may be curved depending on the mapping used.
!
!   When iblend=0 or 2:  local interpolation by blending inside iQ
!
!                        l o c a l  ( with respect to iQ ) reference
!                        coordinates are [a1,b1], [a2,b2], &
!                        these points need not be nodal points
!
!                        boundary points of iQ are mapped on the
!                        reference rectangle equidistantly
!
!                        k1, k2, Ref(*,*), Tau(*,*) are ignored
!
!              =1 or 3:  global interpolation by blending
!
!                        g l o b a l  nodenumbers are  k1, k2
!
!                        the values  a1, b1, a2, b2   are ignored
!
!                        only points recorded in Ref can be connected
!
!   The connecting line should be divided by the new meshpoints into
!   nparts  parts.
!
!   The intermediate points are new, store them in Coor(npoint+1 .. ), &
!               for global interpolation fill also Ref (npoint+1 .. ), &
!   store their global nodenumbers in  kStore ( 1 .. nparts-1 ), &
!   update also the value  npoint.
!
! ********************************************************************
!
!                  INPUT / OUTPUT  PARAMETERS

      integer mrec, iQ ( mrec ), nparts, kStore ( nparts ), &
         mpoint, npoint, k1, k2, nbcord, iblend

      double precision a1, b1, a2, b2, &
         Coor ( 2, mpoint ), Ref ( 2, mpoint ), Tau ( 2, nbcord )

!
!     a1,b1,a2,b2
!              i    see above.  Reference coordinates of beginning and
!                   the end of the line.  Needed only for iblend=0 or 2
!     Coor     io   Real array (2,npoint).  To be updated here.
!                   Physical coordinates of all meshpoints
!     iblend   i    User's choice, see mshzde
!     iQ       i    Integer array of length mrec. This record describes
!                   boundary of the quadrilateral region ( see mshz21 ).
!                   Needed only for iblend=0 or 2
!     k1, k2   i    Global nodenumbers of beginning and end of the line.
!                   Needed only for iblend=1 or 3
!     kStore   o    Integer array. The global nodenumbers of new points, &
!                   it should be filled here, starting from kStore(1).
!     mpoint   i    Upper bound for npoint
!     mrec     i    Upper bound for length of iQ
!     nbcord   i    Number of user's boundary points, see mshzde
!     nparts   i    Number of parts of the line, thus  nparts-1 of new
!                   points should be created
!     npoint   io   Current number of all meshpoints. To be updated here
!     Ref      io   Real array (2,mpoint). Reference coordinates of all
!                   meshpoints, the inner-element points are not needed, &
!                   see mshz21. Needed only for iblend=1 or 3
!     Tau      i    Real array (2,nbcord). Reference nonequidistant
!                   coordinates of user's boundary points, see mshz21.
!                   Needed only for iblend=1 or 3
!
! **********************************************************************
!
!                  LOCAL VARIABLES

      integer ipt
      double precision da, db, aa1, bb1, aa2, bb2

!     aa1, bb1   Reference coordinates of point k1
!     aa2, bb2   Reference coordinates of point k2
!     da, db     Steps in the reference plane
!     ipt        Loop control for new points
!
! **********************************************************************
!
!                  SUBROUTINES CALLED
!
!     mshz04   To display an error message and to stop the run
!     mshz08   Global ( using the original boundary ) blending transform
!     mshz16   Local ( using the current boundary ) blending transform
!
! **********************************************************************
!
!                  ERROR MESSAGES are self-explaining
!
! **********************************************************************
!
!                  METHOD
!
!     local interpolation: the line [a1,b1], [a2,b2] in the reference
!                          plane is divided equidistantly, &
!                          for each point: mshz16 is called the perform
!                                          the mapping
!
!     global interpolation: the line connecting the meshpoints  k1, k2
!                           in the reference plane Ref is divided
!                           equidistantly, &
!
!                           for each point: mshz08 is called the perform
!                                           the mapping, &
!
!                           after the mapping the situation in the
!                           physical plane is:  the density of the
!                           new points is as nonequidistant as the
!                           density of the original boundary points
!
!            notice: boundary points in Ref are equidistant and thus
!                  only the equidistant division of k1, k2 is reasonable
!
!                  the nonequidistant boundary points in Tau are
!                  passed into mshz08 to improve the mapping
!
!                  it seems easier to keep in Ref the nonequidistant
!                  boundary coordinates, but the author knows no
!                  suitable method how to divide k1,k2 in this case
!
!    information on the mapping: see mshz16 resp. mshz08
!
! **********************************************************************
!

      if ( nparts.lt.1 ) call mshz04 ( '# mshz07: nparts<1' )

      if ( mod ( iblend, 2 ).eq.0 ) then

!       *  l o c a l   and   e q u i d i s t a n t   blending in iQ, &
!       *  it is slow because the value of mapping in projections
!       *  are not known, they must be interpolated

!       *  ( array Ref not used )

         da = ( a2 - a1 ) / nparts
         db = ( b2 - b1 ) / nparts

         do 10 ipt = 1, nparts - 1

!          * only for ipt=1 is iQ new
!               for other  ipt  subr mshz16 uses saved values

            call mshz16 ( a1 + dble(ipt) * da, b1 + dble(ipt) * db, &
                          mrec, iQ, ipt.eq.1, &
                          mpoint, npoint, Coor )

!          * npoint was incremented and Coor extended
!           in subr mshz16

            kStore ( ipt ) = npoint

   10    continue

      else

!       * g l o b a l   and   n o n e q u i d i s t a n t   blending
!            in the original quadrilateral, &
!            Ref(*,*) ... reference coordinates of points, &
!                     they are equidistant on the boundary
!
!            Coor(*,*) contains their images, &
!            Tau(*,*) ... reference coordinates of boundary points
!                     of the original quadrilateral, &
!                     they are not equidistant on the boundary
!
!       * translate the nodenumbers k1, k2
!        into global reference coordinates, stored in Ref

         aa1 = Ref ( 1, k1 )
         bb1 = Ref ( 2, k1 )
         aa2 = Ref ( 1, k2 )
         bb2 = Ref ( 2, k2 )

!       * steps in reference plane are equidistant
         da = ( aa2 - aa1 ) / nparts
         db = ( bb2 - bb1 ) / nparts

         do 20 ipt = 1, nparts - 1

!          * create one point

            call mshz08 ( aa1 + dble(ipt) * da, bb1 + dble(ipt) * db, &
                          mpoint, npoint, Coor, Ref, &
                          nbcord, Tau, mrec, iQ )

!          * npoint was incremented, Coor and Ref extended in mshz08

            kStore ( ipt ) = npoint

   20    continue

      end if

      end
