      subroutine mshzde ( new, trian, npelm, icurve, iblend, n1, n2, n3, &
                          n4, nbcord, bCord, mpoint, melem, kBndPt, &
                          npoint, nelem, Coor, kmeshc2d )
! ======================================================================
!
!        programmer    Zdenek Vlasek
!        version  3.1  date 29-05-2014 Extension of timebefore
!        version  3.0  date 20-09-2010 Remove work arrays
!        version  2.7  date 12-08-1999 Replace askclk by printtime
!
!   copyright (c) 1990-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Submesh generator for 2-dim quadrilateral region.
!   The subroutine subdivides the region, the element type:
!       triangle      with 3, 6, 7 or 10      nodes
!       quadrilateral with 4, 9, 16, 25, ...  nodes
!   User gives all boundary nodes, type of element and the number of
!   elements to be created along each side of the quadrilateral.
!   For quadrilateral elements: n1+n2+n3+n4 must be even
!   and there is no other restriction concerning n1, n2, n3, n4.
!   It is not suitable for too curved quadrilateral regions
!   and for very irregular distribution of boundary nodes, &
!   in which cases some general mesh generator should be used.
!   If possible, the regular division is performed.
!   For quadrialteral elements, the topology of the mesh does not depend
!   on the geometry of the region ( in this software ).
!   For triangular elements, the topology of the mesh may be given by
!   user, if stored (untouched!) after a previous call of mshzde.
!   The recursive algorithm used may result in an irregular numbering of
!   the new meshpoints or the elements.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical new, trian
      integer npelm, icurve, iblend, n1, n2, n3, n4, nbcord, &
              mpoint, melem, kBndPt ( nbcord ), npoint, nelem, &
              kmeshc2d ( npelm, melem )
      double precision bCord ( 2, nbcord ), Coor ( 2, mpoint )
      double precision :: timebefore(2)

!     bcord   i    Real array (2,nbcord).  User's boundary coordinates
!                  ( including inner points on boundary, &
!                    in the natural order, &
!                    positive ( =anti-clockwise ) oriented
!                    or negative ( =clockwise ) oriented, &
!                    last point is the same as the first point )array containing
!                    the coordinates of the boundaries
!     coor    o    Real array (2,mpoint).
!                  Physical coordinates of all meshpoints, &
!                  the input array bCord is also
!                  contained there ( without its last point ).
!                  of the nodal points with respect to the global numbering
!     iblend  i    User's choice:
!                  0  many  l o c a l  mappings are used:
!                     boundary nodes of any current quadrilateral or
!                     any quadrilateral element are mapped onto its
!                     reference rectangle   e q u i d i s t a n t l y, &
!                     Coon's blending formula is used for interpolation
!                     in the interior
!                     ( not suitable for a nonuniform distribution of
!                       boundary points )
!                  1  only one - g l o b a l - mapping is used:  user's
!                     boundary nodes are mapped onto the reference
!                     rectangle "Tau"   n o n e q u i d i s t a n t l y, &
!                     proportionaly to the length of arc, &
!                     and at the same time they are mapped onto the
!                     reference rectangle "Ref" equidistantly, then
!                     Coon's formula is used to map "Ref" into the
!                     physical "xy" plane, but the coefficients are
!                     pre-transformed this way:
!                     for given position [a,b] in "Ref" we find the
!                     four projections onto the boundary in "Ref"
!                     [a,    0], [a,    1], [0,    b], [1,    b]
!                     and then the four corresponding points
!                     [tau0, 0], [tau1, 1], [0, sig0], [1, sig1]
!                     on the boundary in "Tau", &
!                     ( it may happen tau0<>tau1, sig0<>sig1 ), now
!                     in "Tau" we connect the opposite points by
!                     straight lines and the coordinates [ aa, bb ]
!                     of their intersection point in "Tau"
!                     are used in Coon's formula instead of the
!                     "Ref"-coordinates [a,b]
!                     ( this approach is used in other Sepran parts )
!                  2  as iblend=0 but an "easy" topology is used
!                     for quadrilateral elements
!                     ( not suitable for long strips with big n1-n3 ), &
!                     for triangular elements the value iblend=2
!                             is useless, it works like iblend=0
!                  3  as iblend=1 but an easy topology is used
!                     for quadrilateral elements
!                     ( not suitable for long strips with big n1-n3 , &
!                     for triangular elements the value iblend=3
!                             is useless, it works like iblend=1
!     icurve  i    User's choice:
!                  0   all new element edges must be  s t r a i g h t
!                     ( it may cause problems when boundary is curved )
!                  1  all new element edges which contain no boundary
!                     point must be straight
!                     ( it may cause problems when boundary is curved ), &
!                     the edges connected to boundary may be curved
!                  2  all new element edges may be curved, &
!                     in accordance with the mapping chosen by iblend
!     kbndpt  o    Integer array of length nbcord.
!                  Global nodenumbers for the user's boundary
!                  points, ordered in accordance with the
!                  user's array bCord(2,nbcord)
!     kmeshc2d  i/o   Integer array (npelm,melem).
!                  Global nodenumbers for each element, &
!                  for new=.true.  it is output parameter, &
!                  for new=.false. it is input parameter
!     melem   i    User's upper bound for the number of elements
!                  ( subroutine mshz05 can be called to estimate it )
!     mpoint  i    User's upper bound for the number of points, plus 1.
!     n1      i    User's choice:
!                  number of elements along the  s/e/n/west  side
!                  of the given quadrilateral.  For quadrilateral
!                  elements their sum must be even.
!     n2      i    See n1
!     n3      i    See n1
!     n4      i    See n1
!     nbcord  i    Number of user's boundary points, &
!                  including the intermediate points, &
!                  including the last point equal to the first
!                  it must be   nbcord = (n1+n2+n3+n4)*(npedge-1) + 1
!     nelem  i/o   The number of elements created (output parameter).
!                  When new=.false.: nelem is also input parameter, &
!                  it must keep the corresponding value.
!     new     i    User's choice:
!                  .true.    when a new topology may be created
!                  .false.   when the topology is submitted by user
!                            in the array kmeshc2d, &
!                            it this case user must also submit the
!                            corresponding values npoint, nelem, &
!                            mshzde should re-calculate the Coor
!                            ( this option is  n o t  supported for
!                            quadrilateral elements, in this case
!                            the topology is determined by values
!                            n1, n2, n3, n4 only )
!     npelm   i    The number of points per element, &
!                  including all internal points. This number is
!                  u n i q u e  for each element shape.
!     npoint i/o   The number of points in Coor (output parameter).
!                  When new=.false.: npoint is also input parameter, &
!                                 it must keep the corresponding value.
!     trian   i    Indication whether triangles (true) or quadrilaterals
!                  are created (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipoint, iorien, npedge, mrec, mstk, kpoint, &
              liq, lir, lis, lit, liu, lstk, lref, ltau, lfree, &
              ielem, ipelm, npelmsv, i
      integer, allocatable :: iwork(:)
      double precision, allocatable :: work(:)

!     i              Counting variable
!     ielem          Element number
!     iorien         =1 for anticklocwise orentation, =-1 for clockwise
!     ipelm          local point number in the element
!     ipoint         global node number
!     iwork          integer work array
!     kpoint
!     lfree          the first free position in the array iWork resp. Work
!     liq            the location of scratch array iQ in iWork ( see mshz21 )
!     lir            the location of scratch array iR in iWork ( see mshz21 )
!     lis            the location of scratch array iS in iWork ( see mshz21 )
!     lit            the location of scratch array iT in iWork ( see mshz21 )
!     liu            the location of scratch array iU in iWork ( see mshz21 )
!     lref           the location of scratch array  Ref  in Work, &
!                    needed when iblend = 1 or 3  ( see mshz21 )
!     lstk           the location of scratch array iStk in iWork
!     ltau           the location of scratch array  Tau  in Work, &
!                    needed when iblend = 1 or 3  ( see mshz21 )
!     mrec           an upper bound for the length of the record iQ, &
!                                 ( see mshz21 to know more about iQ )
!     mstk           an upper bound for the number of records
!                    in the stack iStk ( see mshz21 to know more about iStk )
!     npedge         number of points per one element edge
!                    ( example: for linear triangle  npedge=2 )
!     npelmsv        Help variable to save npelm
!     work           real work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSHR21         Fill array kmeshc2dc for centroid points in 5-point elements
!     MSHZ01         Double-faced subroutine:
!                    To check an element created: angles along its boundary
!                    To determine the orientation of the whole boundary
!     MSHZ08         Global ( using the original boundary ) blending transform
!     MSHZ10         to initialize coordinates of points on the boundary of
!                    the reference rectangle [0,1] * [0,1]
!     MSHZ11         initialization of the record iQ for the first time
!     MSHZ21         Solver, it divides the original quadrilateral region into
!                    q u a d r i l a t e r a l  elements
!     MSHZ25         Solver, it divides the original quadrilateral region into
!                    t r i a n g u l a r  elements
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     360   Number of nodes larger than estimated
!    1382   Value of NPELM is not yet implemented for this subroutine
!    1383   Value of ICURVE is not yet implemented for this subroutine
!    1384   Value of IBLEND is not yet implemented for this subroutine
!    1385   At least one of the number of elements along a side is less than one
!    1386   The total number of elements along the sides of the region is not
!           even, whereas quadrilaterals must be created.
!    1387   The number of points at the side is not correct
! **********************************************************************
!
!                       PSEUDO CODE
!
!   The whole pack of subroutines called directly or indirectly from
!   mshzde works this way:
!   a/ In this master subroutine mshzde the following is to be done:
!          checks of input values, &
!          allocations of scratch arrays in iWork(*) and Work(*), &
!          initialization of some scratch arrays, &
!          call of one of the  t w o   s o l v e r s   doing the job:
!                  mshz21 ... for quadrilateral elements
!                  mshz25 ... for triangular elements
!          ( the two solvers use many common subroutines )
!   b/ In the solver mshz21 resp. mshz25:
!      That problem ( represented by  n1,   n2,   n3,   n4 , &
!                              for  south  east  north  west ) is solved
!      which is passed by the calling subroutine mshzde
!      or which is taken from the top of the stack.
!      The problem is transformed by a cyclic or by a reverse orderings
!      of the four sides onto a standard problem  ( see the inequalities
!      in the comments in mshz21/mshz25 ).
!      Now, some  s p e c i a l  cases are solved immediately using
!      a simple idea how to subdivide the quadrilateral region:
!      -------- For quadrilateral elements the special cases are:
!      n1=n3, n2=n4              ... regular division into n2 layers, &
!                                    ( see subroutine mshz22 ), &
!                                    regular division of each layer
!                                    ( see subroutine mshz14 )
!      n1=1   n2    n3    n4=1   ... irregular division into
!                                    smaller quadrilaterals
!                                 ( see diagrams in subroutine mshz12 ), &
!                                    each of them can be divided in
!                                    the regular way (the case above)
!      -------- For triangular elements the special cases are:
!      n2=n4                     ... regular division into n2 layers, &
!                                    interpolation from n1 to n3
!                                    ( see subroutine mshz22 ), &
!                                    irregular division of each layer
!                                    ( see subroutine mshz15 )
!      n1=1   n2    n3    n4=1   ... irregular division into
!                                    smaller quadrilaterals
!                                 ( see diagrams in subroutine mshz12 ), &
!                                    each of them can be divided in
!                                    the regular way (the case above).
!      The  o t h e r  cases, both for quadrilateral and triangular
!      elements,  are solved depending on user's choice:
!      iblend=0 or 1 :  by cutting the region roughly half-to-half
!                       ( see diagram in the subroutine mshz24 ).
!                       This aproach leads to a recursive algorithm, &
!                       but in fact, no recursive call is used.
!                       Instead, the two parts are pushed onto
!                       a stack of quadrilateral regions waiting for
!                       farther processing.
!                       The solver turns a loop until the stack is empty
!      iblend=2 or 3 :  the "easy" straighforward algorithm is applied
!                       ( see diagrams in the subroutine mshz12 ).
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshzde' )
      if ( itime>0 ) call ersettime ( timebefore )

!     --- check  npelm  and fill  npedge

      npelmsv = npelm
      if ( trian ) then

         if ( npelm==3 ) then
            npedge = 2
         else if ( npelm==6 .or. npelm==7 ) then
            npedge = 3
         else if ( npelm==10 ) then
            npedge = 4
         else
            call errint ( npelm, 1 )
            call errsub ( 1382, 1, 0, 0 )
            go to 1000
         end if

      else

!     --- quadrilateral,  npelm = npedge**2

         if ( npelm==9 ) then
            npedge = 3
         else if ( npelm==4 .or. npelm==5 ) then
            npedge = 2
            npelm = 4
         else
            call errint ( npelm, 1 )
            call errsub ( 1382, 1, 0, 0 )
            go to 1000
         end if

      end if

!     --- check  icurve, iblend

      if ( icurve<0 .or. icurve>2 ) then
         call errint ( icurve, 1 )
         call errsub ( 1383, 1, 0, 0 )
         go to 1000
      end if

      if ( iblend<0 .or. iblend>3 ) then
         call errint ( iblend, 1 )
         call errsub ( 1384, 1, 0, 0 )
         go to 1000
      end if

!     --- check n1, n2, n3, n4

      if ( min ( n1, n2, n3, n4 )<1 ) then
         call errint ( n1, 1 )
         call errint ( n2, 2 )
         call errint ( n3, 3 )
         call errint ( n4, 4 )
         call errsub ( 1385, 4, 0, 0 )
         go to 1000

      else if ( .not. trian .and. mod(n1+n2+n3+n4, 2)/=0 ) then
         call errint ( n1, 1 )
         call errint ( n2, 2 )
         call errint ( n3, 3 )
         call errint ( n4, 4 )
         call errsub ( 1386, 4, 0, 0 )
         go to 1000

      end if

!     --- check  nbcord

      if ( nbcord/=(n1+n2+n3+n4) * (npedge-1) + 1 ) then
         call errint ( n1, 1 )
         call errint ( n2, 2 )
         call errint ( n3, 3 )
         call errint ( n4, 4 )
         call errint ( nbcord, 5 )
         call errint ( npedge, 6 )
         call errsub ( 1387, 6, 0, 0 )
         go to 1000

      end if

!     --- fill  kBndpt  as if the boundary were anticlockwise oriented
!         ( here, we wont to use the more common subr mshz01 )
!         ( for clockwise orientation the  kBndPt  will be changed later )

      do ipoint = 1, nbcord
         kBndPt ( ipoint ) = ipoint
      end do

!     --- check the sum of inner angles of the boundary, &
!         fill  iorien  by +-1 : the orientation of the boundary

      iorien = 0
      call mshz01 ( iorien, nbcord-1, 0, nbcord-1, kBndPt, &
                    nbcord, bCord )

!     --- allocations in iWork(*)
!         estimations of mrec, mstk ( the same are in subr mshz05 )
!         if you correct them then correct both occurencies !!!

      mrec = 14 + 2 * ( max(n1,n3) + max(n2,n4) ) * ( npedge - 1 ) + 1
      mstk = int ( 2d0 &
         + 2d0 * log10 ( ( n1 + n2 + n3 + n4 ) / 4d0 ) / log10 ( 2d0 ) )

      liQ = 1
      liR = liQ + mrec
      liS = liR + mrec
      liT = liS + mrec
      liU = liT + mrec
      lstk = liU + mrec
      lfree = lstk + mstk * mrec

      allocate ( iwork(lfree), stat = error )
      if ( error/=0 ) call eralloc ( error, lfree, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- allocations in Work(*):  Ref(*,*) and Tau(*,*), when needed

      if ( mod ( iblend, 2 )==0 ) then

!     --- no need for Ref and Tau

         lref = 1
         ltau = 1
         lfree = 1

      else

         lref = 1
         ltau = lref + 2 * mpoint
         lfree = ltau + 2 * nbcord

      end if

      allocate ( work(lfree), stat = error )
      if ( error/=0 ) call eralloc ( error, lfree, 'work' )
      if ( ierror/=0 ) go to 1000

!     --- initialize  iQ ( * )  alias  iWork ( liQ .. )
!         ( put there values corresponding to the given boundary )

      if ( iorien==1 ) then
         call mshz11 ( n1, n2, n3, n4, npedge, mrec, iWork ( liQ ) )
      else

!     --- let the user's  data be clockwise oriented, but
!          let the working data be anticlockwise oriented

         call mshz11 ( n4, n3, n2, n1, npedge, mrec, iWork ( liQ ) )
      end if

!     --- initialize  npoint, Coor ( 2, * ), eventually correct the  kBndPt
!         ( put there values corresponding to the given boundary )
!         for  subr mshz08  we store in Coor the point No.1 twice, &
!         to enable interpolation on the boundary of the original region

      if ( .not. new ) then

!     --- restore kmeshc2d, for the restart:
!         ( eliminate the effect of the loops 410, 400 below )
!         in this case npoint, nelem are also input parameters, &
!                      they must keep the last output value
!         ( remark: the input value nelem is used only here )

         do ielem = 1, nelem
            do ipelm = 1, npelm

!           --- ( remark: the input value npoint is used only here )

               if ( kmeshc2d ( ipelm, ielem )==nbcord ) &
                    kmeshc2d ( ipelm, ielem ) = npoint + 1
            end do
         end do

      end if

      npoint = nbcord
      if ( npoint>mpoint ) then
         call errint ( npoint, 1 )
         call errint ( mpoint, 2 )
         call errsub ( 360, 2, 0, 0 )
         go to 1000
      end if

      if ( iorien==1 ) then

!     --- normal case, the anticlockwise orientation

         do ipoint = 1, nbcord
            Coor ( 1, ipoint ) = bCord ( 1, ipoint )
            Coor ( 2, ipoint ) = bCord ( 2, ipoint )
         end do
         kBndPt ( nbcord ) = 1
      else

!     --- reverse ( clockwise ) orientation of the boundary:
!         let the user's  data be clockwise oriented, but
!         let the working data be anticlockwise oriented

         do ipoint = 1, nbcord
            Coor ( 1, ipoint ) = bCord ( 1, nbcord+1-ipoint )
            Coor ( 2, ipoint ) = bCord ( 2, nbcord+1-ipoint )
            kBndPt ( nbcord+1-ipoint) = ipoint
         end do
         kBndPt ( 1 ) = 1
      end if

!     --- now there is no pointer to the extra point Coor ( *, nbcord ), &
!         but the point may be used in  subr mshz08
!         initialize  Ref ( 2, * )  and  Tau ( 2, nbcord ), &
!         alias  Work ( lref ),  Work ( ltau )

      if ( mod ( iblend, 2 )==1 ) then

         call mshz10 ( mrec, iWork ( liQ ), &
                       Coor, Work ( lref ), &
                       nbcord, Work ( ltau ) )

!        --- call mshz08  to initialize its local saved variables
!            ( the negative npoint causes the initialization take place
!              instead of a blending transformation )

         kpoint = -npoint
         call mshz08 ( 0d0, 0d0, npoint, kpoint, Coor, Work ( lref ), &
            nbcord, Work ( ltau ), mrec, iWork ( liQ ) )

      end if

!     --- and now, call the proper solver, &
!         the solver has no access to user's  n1, n2, n3, n4, bCord, kBndPt
!         values n1, n2, n3, n4 are not passed directly, they are hidden:
!         for anticlockwise orientation there is:
!             n1=iQ(2)=iWork(liQ+1),  n2=iQ(3),  n3=iQ(4),  n4=iQ(5)
!         for clockwise orientation there is:
!             n4=iQ(2)=iWork(liQ+1),  n3=iQ(3),  n2=iQ(4),  n1=iQ(5), &

      if ( trian ) then

!     --- topology may depend on mapping

         call mshz25 ( new, trian, npelmsv, npedge, &
            mpoint, melem, mrec, mstk, &
            npoint, nelem, kmeshc2d, &
            iWork ( lstk ), iWork ( liQ ), iWork ( liR ), &
            iWork ( liS ), iWork ( liT ), iWork ( liU ), &
            Coor, Work ( lref ), &
            nbcord, icurve, Work ( ltau ), iblend )

      else

!     --- topology does not depend on the mapping

         call mshz21 ( new, trian, npelmsv, npedge, &
            mpoint, melem, mrec, mstk, &
            npoint, nelem, kmeshc2d, &
            iWork ( lstk ), iWork ( liQ ), iWork ( liR ), &
            iWork ( liS ), iWork ( liT ), iWork ( liU ), &
            Coor, Work ( lref ), &
            nbcord, icurve, Work ( ltau ), iblend )

      end if

!     --- remove the extra global point (# nbcord) from Coor(*,*)

      Coor ( 1, nbcord ) = Coor ( 1, npoint )
      Coor ( 2, nbcord ) = Coor ( 2, npoint )

      do ielem = 1, nelem
         do ipelm = 1, npelm
            if ( kmeshc2d ( ipelm, ielem )==npoint ) &
               kmeshc2d ( ipelm, ielem ) = nbcord
         end do
      end do

      npoint = npoint - 1
      if ( icurve==20 .and. npelm>=6 ) then
         do i = 2, nbcord-2, 2
            coor(1,i) = 0.5d0*(coor(1,i-1)+coor(1,i+1))
            coor(2,i) = 0.5d0*(coor(2,i-1)+coor(2,i+1))
         end do
      end if

      deallocate ( iwork, work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'mshzde' )
      if ( itime==1 ) call printtime ( 'Quadrilateral')
      npelm = npelmsv
      if ( npelm==5 ) call mshr21 ( kmeshc2d, npoint, nelem, npelm )

      end
