      subroutine sepextkmesht ( isurnrfirst, isurnrlast, istart, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute starting and ending address of elements in kmeshc by using
!     the information stored in kmesh and kmesht
!     More than one surface may be used, but all surfaces must have the
!     same number of elements
!     The number of nodes per element as well as the number of elements is
!     returned
!     Old subroutine mshextkmesht
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     fill
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnrfirst, isurnrlast, istart, inpelm

!     inpelm         o    number of points per element
!     istart         o    First position in array kmeshc
!     isurnrfirst    i    First surface number
!     isurnrlast     i    Last surface number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, ielem1, ielem2, ielgrp, nelemgrp, jnpelm, jelgrp

!                    they are not
!     ielem          Element number
!     ielem1         First element number in surface
!     ielem2         Last element number in surface
!     ielgrp         element group number
!     jelgrp         Actual element group number
!     jnpelm         Number of nodes in an element
!     nelemgrp       Number of elements in element group
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     272   number of nodal points in elements of the same group for the
!           generation of essential boundary conditions, not the same.
! **********************************************************************
!
!                       PSEUDO CODE
!
!      ielem1 := kmesht(1,isurnrfirst)
!      ielem2 := kmesht(2,isurnrlast)
!      Compute starting address of element ielem1 in kmeshc
!      Check if all elements have the same number of nodes per element
!      inelem := ielm2-ielem1+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! ======================================================================
!
      call eropen ( 'sepextkmesht' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepextkmesht'
         write(irefwr,1) 'isurnrfirst, isurnrlast', &
                          isurnrfirst, isurnrlast
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ielem1 = kmesht(1,isurnrfirst)
      ielem2 = kmesht(2,isurnrlast)
      inpelm = npelm

      if ( debug ) write(irefwr,1) 'ielem1, ielem2', ielem1, ielem2

      if ( ielem1<=0 ) then

!     --- ielem1 <= 0, hence surface has not been filled; return

         inpelm = 0
         inelem = 0
         istart = 0
         go to 1000

      end if

!     --- Compute starting address of element ielem1 in kmeshc
!         Compute end address of element ielem2 in kmeshc

      if ( debug ) write(irefwr,1) 'nelgrp', nelgrp

      if ( nelgrp==1 ) then

!     --- Only one element group

         istart = inpelm*(ielem1-1)+1

      else

!     --- More element groups

         istart = 0
         ielem  = 0
         do ielgrp = 1, nelgrp
            inpelm = sepgetinpelm ( ielgrp )
            jelgrp = ielgrp
            inelem = kmeshg(ielgrp)
            if ( ielem+inelem>=ielem1 ) go to 100
            ielem = ielem+inelem
            istart = istart+inpelm*inelem
         end do

100      istart = istart+(ielem1-ielem-1)*inpelm+1
         nelemgrp = inelem+(ielem1-ielem-1)

         if ( ielem2-ielem1+1>nelemgrp ) then

!        --- the last surface belongs to another element group

            jnpelm = inpelm
            ielem = ielem+inelem

            if ( debug ) then

!           --- Debug statements

               write(irefwr,*) 'nelemgrp, jelgrp, nelgrp ', &
                                nelemgrp, jelgrp, nelgrp

            end if

            do ielgrp = jelgrp+1, nelgrp
               if ( kelma>0 ) inpelm = kmesha(ielgrp+1)-kmesha(ielgrp)
               inelem = kmeshg(ielgrp)
               if ( inpelm.ne.jnpelm ) then

!              --- Error jnpelm = inpelm

                  call errint ( isurnrfirst, 1 )
                  call errint ( isurnrlast, 2 )
                  call errint ( jelgrp, 3 )
                  call errint ( jnpelm, 4 )
                  call errint ( ielgrp, 5 )
                  call errint ( jnpelm, 6 )
                  call errsub ( 272, 6, 0, 0 )

               end if
               if ( ielem+inelem>=ielem2 ) go to 200
               ielem = ielem+inelem

            end do

         end if

      end if

200   inelem = ielem2-ielem1+1

1000  call erclos ( 'sepextkmesht' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart, inelem, inpelm', istart, inelem, inpelm
         write(irefwr,*) 'End sepextkmesht'

      end if  ! ( debug )

      end subroutine sepextkmesht
