      subroutine sepflkmeshad2 ( ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part ad as well as the subarrays ipointobst and
!     ielemobst (2d only)
!     Old subroutine mshfillkmeshad
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice

!     ichoice        i    Choice parameter: defines which part of kmesh part ad
!                         must be filled
!                         Possible values:
!                         0: the complete array
!                         1: part 1 (ipointobst)
!                         2: parts 1,2 (ielemobst)
!                         3: part 1-4 (icrossobst,rcrossobst)
!                         -2: parts 2 only (ielemobst)
!                         -3: part 3,4 only (icrossobst,rcrossobst)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     SEPFILLCRSOBS  Fill arrays icrossobst and rcrossobst (2d)
!     SEPFILLELEMOBS Fill array ielemobst (2d)
!     SEPFLPOINTOBS2 Fill array ipointobst (2d)
!     SEPPLOTOBST    Print and plot information about the obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2207   There are no obstacles
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numobst := kmesh(42)
!     for iobst := 1 (1) numobst do
!        Mark all nodes of mesh in obstacle
!        Mark all elements to be in/out or partly in obstacle
!        Mark all nodes that are in obstacle and only in elements that are
!        completely in obstacle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepflkmeshad2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepflkmeshad2'
         write(irefwr,1) 'ichoice', ichoice
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- initializations

      if (debug)  write(irefwr,1) 'numobst', numobst
      if ( numobst==0 ) then

!     --- No obstacles

         call errwar ( 2207, 0, 0, 0 )
         go to 1000

      end if
      if ( ichoice>=0 ) call sepflpointobs2
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'kelmad', kelmad
         call prinin ( kmeshad, 5, 'kmesh part ad' )
      end if  ! ( debug )

!     --- Mark all elements to be in/out or partly in obstacle

      if ( ichoice==0 .or. ichoice>=2 .or. ichoice==-2 ) then

!     --- ichois = 0 or 2, fill array ielemobst
!         Mark all elements to be in/out or partly in obstacle

         call sepfillelemobs

      end if

      if ( ichoice>=-2 .and. (debug .or. plotobst .or. printobst) ) then

!     --- debug = true, plot nodes and elements with a color

         call sepplotobst ( debug, ichoice )

      end if

      if ( ichoice==0 .or. ichoice>=3 .or. ichoice==-3 ) then

!     --- ichois = 0 or 3, fill array icrossobst,rcrossobst

         call sepfillcrsobs

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepflkmeshad2'

      end if
      call erclos ( 'sepflkmeshad2' )

      end subroutine sepflkmeshad2
