      subroutine sepmshchacr1 ( numnodes, icall, itypepos, &
                                ifirst, ilast )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Overestimate numnodes for subroutine mshchacr
!     This is used to declare arrays iwork and nodes
!     Old subroutine mshchacr1
! **********************************************************************
!
!                       KEYWORDS
!
!     estimate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, icall, itypepos, ifirst, ilast

!     icall          i    See subroutine mshchacr
!     ifirst         i    First curve (surface) number read
!     ilast          i    Last curve (surface) number read
!     itypepos       i    Type of position parameter
!                         See subroutine mshchacr
!     numnodes       o    Number of nodal points in array nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart

!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmshchacr1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmshchacr1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( icall==5 .and. mod(itypepos,100)==0 ) then

!     --- itypepos = 0, all points

         numnodes = npoint

      else if ( icall==5 .and. itypepos==1 .or. &
                icall==1 .and. itypepos==101 ) then

!     --- itypepos = 1, user points from ifirst to ilast

         numnodes = ilast-ifirst+1

      else if ( icall==5 .and. &
                (itypepos>=2 .and. itypepos<=4 .or. itypepos==101) ) then

!     --- itypepos = 2, curves from ifirst to ilast

         numnodes = npoint

      else if ( icall==2 .and. itypepos==102 ) then

!     --- itypepos = 102, curves from ifirst to ilast

         if ( ifirst==1 ) then
            istart = 0
         else
            istart = icurvsmsh(ifirst-1)
         end if
         numnodes = icurvsmsh(ilast)-istart

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmshchacr1'

      end if  ! ( debug )
      call erclos ( 'sepmshchacr1' )

      end
