      subroutine sepmshextkt ( isurnr, istart, iend )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute starting and ending address of elements in kmeshc by using
!    the information stored in kmesh and kmesht
!     Old subroutine mshextkt
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     fill
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr, istart, iend

!     iend       o    Last position used in array
!     istart     o    First position in array
!     isurnr     i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, ielem1, ielem2, inpelm, ielgrp

!     ielem      Element number
!     ielem1     First element number in surface
!     ielem2     Last element number in surface
!     ielgrp     element group number
!     inpelm     number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      ielem1 := kmesht(1,isurnr)
!      ielem2 := kmesht(2,isurnr)
!      Compute starting address of element ielem1 in kmeshc
!      Compute end address of element ielem2 in kmeshc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

      ielem1 = kmesht(1,isurnr)
      ielem2 = kmesht(2,isurnr)

!     --- Compute starting address of element ielem1 in kmeshc
!         Compute end address of element ielem2 in kmeshc

      if ( nelgrp==1 ) then

!     --- Only one element group

         istart = npelm*(ielem1-1)+1
         iend   = npelm*ielem2

      else

!     --- More element groups

         istart = 0
         ielem  = 0
         do ielgrp = 1, nelgrp
            inpelm = Topgrp(ielgrp)%inpelm
            inelem = Topgrp(ielgrp)%inelem
            if ( ielem+inelem>=ielem1 ) go to 100
            ielem = ielem+inelem
            istart = istart+inpelm*inelem
         end do
100      istart = max(1,istart+(ielem1-ielem-1)*inpelm+1)
         iend = max(0,istart+(ielem2-ielem1+1)*inpelm-1)

      end if

      end
