      subroutine sepmshnors0
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute normals on boundary in R^3 and store in array normal
!     Old subroutine mshnors0
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     outer_boundary
!     mesh
!     normal_vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurvs(2), ipstart, j, istart, node, normaldr
      double precision :: norm(3,npoint), weight(npoint)

!     i              Counting variable
!     icurvs         Integer array to be used by ERCOMNOR
!     ipstart        Starting address in array
!     istart         First position used at a part
!     j              Counting variable
!     node           Node number
!     norm           Work array of length ndim * npoint containing the
!                    normal in the nodal points
!     normaldr       If 1 the outward directed normal is required, if -1
!                    the inward directed normal
!     weight         Work array of length npoint to be used by ERCOMNOR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINNORMAL3D1  Print the array normals for all outer surfaces
!     SEPCOMNOR      Compute the averaged normals in all points on the surfaces
!                    isrf1 to isrf2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all surfaces do
!        compute normal per surface
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmshnors0' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmshnors0'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- For all surfaces do
!            compute normal per surface

      do isurnr = 1, nsurfs

!     --- Compute normals and store in array norm

         icurvs(1) = 0
         icurvs(2) = isurnr
         normaldr = 1
         call sepcomnor ( norm, weight, icurvs, normaldr )

!        --- Copy computed norm in array normal

         ipstart = surfnodes(isurnr)
         nnodes = surfnodes(ipstart)
         istart = ipstart-nsurfs-isurnr
         do i = 1, nnodes
            node = surfnodes(ipstart+i)
            do j = 1, 3
               normal(j,istart+i) = norm(j,node)
            end do
         end do

      end do  ! icurnr = 1, ncurvs

      if ( debug ) then

!     --- debug true, debug statements

         call prinnormal3d1 ( nsurfs, surfnodes, normal )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmshnors0'

      end if
      call erclos ( 'sepmshnors0' )

      end
