      subroutine sepobstmarkpt1 ( iobst, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reduced version of subroutine mshobstmarkpt in case of open
!     obstacles lying in x, y or z direction
!     Old subroutine sepobstmarkpt1
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst, iwork(*)

!     iobst          i    Obstacle sequence number
!     iwork               Work array of length npoint to store the boundary
!                         nodes temporarily
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xmin, xmax, ymin, ymax, x, y, &
                       eps, xminorig, yminorig
      integer i, inode, numpntobst

!     eps            Accuracy
!                    Points that are at a distance less than eps of the
!                    obstacle are defined as part of the obstacle
!     i              Counter
!     inode          Node sequence number
!     numpntobst     Number of nodal points inside the obstacle
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xminorig       Minimum x-value to compute ipcell, jpcell
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     yminorig       Minimum y-value to compute ipcell, jpcell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Extract xmin,xmax,ymin,ymax from obstacle array
!    numpntobst := 0
!    For inode := 1 (1) npoint do
!       (x,y) := coordinates of node
!       If ( (x,y) inside (xmin,xmax;ymin,ymax) )
!             numpntobst := numpntobst+1
!             ipointobst(numpntobst) := inode
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepobstmarkpt1' )

!     --- initialize parameters

      numpntobst = 0

!     --- Extract xmin,xmax,ymin,ymax from obstacle array

      xminorig = rmeshx(1,iobst)
      xmax     = rmeshx(2,iobst)
      yminorig = rmeshx(3,iobst)
      ymax     = rmeshx(4,iobst)
      eps = rinfin
      do i = 1, ncurvs
         eps = min(eps,elsize(1,i),elsize(2,i))
      end do
      eps = eps*accuracy_obstacle
      xmin = xminorig-eps
      xmax = xmax    +eps
      ymin = yminorig-eps
      ymax = ymax    +eps

!     --- For inode := 1 (1) npoint do

      do inode = 1, npoint

!     --- (x,y) := coordinates of node

         x = coor(1,inode)
         y = coor(2,inode)

         if ( x.ge.xmin .and. x.le.xmax .and. &
              y.ge.ymin .and. y.le.ymax ) then
         numpntobst=numpntobst+1
         iwork(numpntobst)=inode
         end if

      end do

!     --- Fill four positions of ipointobst

      ipointobst(1) = numpntobst
      ipointobst(2) = 0
      ipointobst(3) = 0
      ipointobst(4) = 0
      do i = 1, numpntobst
         ipointobst(4+i) = iwork(i)
      end do

      call erclos ( 'sepobstmarkpt1' )
      end
