      subroutine msh057 ( mapar, kmeshn, kmeshm, kmeshl, npoint, nodp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 21-08-2008 Extension with internal points...
!        version  2.2  date 14-11-2002 Layout
!        version  2.1  date 22-07-1998 Include common
!
!   copyright (c) 1988-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array mapar and the arrays kmeshn, kmeshm, kmeshl for a volume
!     Help subroutine for subroutine msh035
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm(*), kmeshn(*), mapar(*), kmeshl(*), npoint, nodp

!     kmeshl    i/o   KMESH part l, contains nodal point numbers of the user
!                     points
!     kmeshm    i/o   KMESH part m, contains nodal point numbers of the curves
!     kmeshn    i/o   KMESH part n, contains nodal point numbers of the surfaces
!     mapar      o    mapping array: contains the mapping from local to
!                     global numbering of the nodal points
!     nodp      i/o   Nodal point number
!     npoint     i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inodp, jnodp, knodp, lnodp, nnodp

!     i          Counting variable
!     inodp      Nodal point number
!     jnodp      Nodal point number
!     knodp      Nodal point number
!     lnodp      Nodal point number
!     nnodp      Nodal point number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh057' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh057'
         write(irefwr,1) 'npoint, nodp', npoint, nodp
         call prinin ( mapar, npoint, 'mapar' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      do i = 1, npoint

         inodp=mapar(i)

         if ( inodp==0 ) then
            mapar(i) = nodp
            nodp = nodp+1

         else if ( inodp<-npoint ) then

!        --- inodp corresponds to a nodal point that has not been filled, &
!            although it is a boundary point

            jnodp = -inodp - npoint

!           --- Check if the corresponding position in kmeshn has been filled

            knodp = kmeshn(jnodp)
            if ( knodp>0 ) then

!           --- kmeshn has been filled, copy in mapar

               mapar(i) = knodp

            else if ( knodp<-1000000000 ) then

!           --- knodp<-1000000000:  kmeshn has not been filled, &
!               but refers to kmeshl

               lnodp = kmeshl(-knodp-1000000000)
               if ( lnodp>0 ) then

!              --- kmeshl has been filled
!                  Fill kmeshn and mapar

                  kmeshn(jnodp) = lnodp
                  mapar(i) = lnodp

               else

!              --- lnodp = 0, fill kmeshl, kmeshm and kmeshn

                  kmeshl(-knodp-1000000000) = nodp
                  kmeshn(jnodp)  = nodp
                  mapar(i)       = nodp
                  nodp = nodp + 1

               end if  ! ( lnodp>0 )

            else if ( knodp<0 ) then

!           --- knodp<0:  kmeshn has not been filled, but refers to kmeshm
!               Check kmeshm

               lnodp = kmeshm(-knodp)
               if ( lnodp>0 ) then

!              --- kmeshm has been filled
!                  Fill kmeshn and mapar

                  kmeshn(jnodp) = lnodp
                  mapar(i) = lnodp

               else if ( lnodp<0 ) then

!              --- lnodp<0:  kmeshm has not been filled but refers to kmeshl

                  nnodp = kmeshl(-lnodp)
                  if ( nnodp>0 ) then

!                 --- kmeshl has been filled, Copy into kmeshm and kmeshn

                     kmeshm(-knodp) = nnodp
                     kmeshn(jnodp) = nnodp
                     mapar(i) = nnodp

                  else

!                 --- nnodp = 0, fill kmeshl, kmeshm and kmeshn

                     kmeshl(-lnodp) = nodp
                     kmeshm(-knodp) = nodp
                     kmeshn(jnodp)  = nodp
                     mapar(i)       = nodp
                     nodp = nodp + 1

                  end if

               else

!              --- lnodp = 0:  kmeshm has not been filled

                  kmeshm(-knodp) = nodp
                  kmeshn(jnodp)  = nodp
                  mapar(i)       = nodp
                  nodp = nodp + 1

               end if

            else

!           --- knodp = 0:  kmeshn has not been filled

               kmeshn(jnodp)  = nodp
               mapar(i)       = nodp
               nodp = nodp + 1

            end if
         else if ( inodp<0 ) then
            mapar(i)=mapar(-inodp)
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'npoint, nodp', npoint, nodp
         call prinin ( mapar, npoint, 'mapar' )
         write(irefwr,*) 'End msh057'

      end if  ! ( debug )
      call erclos ( 'msh057' )

      end subroutine msh057
