      subroutine msh303 ( inpelm, kmeshc, nx, ny, nz, nelem )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 1.3    date   28-12-1996 Extension with shape 18
!        version 1.2    date   01-09-1995 Extension with shape 17
!        version 1.1    date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 1.0    date   03-12-1984
!
!   copyright (c) 1982-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array kmeshc with the nodal point numbers of the elements
!    sequentially.four possibilities are considered
!    tetrahedron (linear)    :   inpelm =  4
!    tetrahedron (quadratic) :   inpelm = 10
!    hexahedron  (linear)    :   inpelm =  8, 9
!    hexahedron  (quadratic) :   inpelm = 27
! **********************************************************************
!
!                       KEYWORDS
!
!   mesh
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer kmeshc(*), inpelm, nx, ny, nz, nelem

!     inpelm  i    number of points per element
!     kmeshc  o    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     nelem   i    Number of elements
!     nx      i    number of points in x direction
!     ny      i    number of points in y direction
!     nz      i    number of points in z direction
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ik, nxy, k, kxy, j, ih, i, ipnt1, ipnt2, ipnt3, ipnt4, &
              ipnt5, ipnt6, ipnt7, ipnt8, ipnt9, ipnt10, ipnt11, ipnt12, &
              ipnt13, ipnt14, ipnt15, ipnt16, ipnt17, ipnt18, ipnt19, &
              ipnt20, ipnt21, ipnt22, ipnt23, ipnt24, ipnt25, ipnt26, &
              ipnt27, jh

!     i       Counting variable
!     ih      Help variable to store a constant temporarily
!     ik      ?
!     ipnt1   ?
!     ipnt10  ?
!     ipnt11  ?
!     ipnt12  ?
!     ipnt13  ?
!     ipnt14  ?
!     ipnt15  ?
!     ipnt16  ?
!     ipnt17  ?
!     ipnt18  ?
!     ipnt19  ?
!     ipnt2   ?
!     ipnt20  ?
!     ipnt21  ?
!     ipnt22  ?
!     ipnt23  ?
!     ipnt24  ?
!     ipnt25  ?
!     ipnt26  ?
!     ipnt27  ?
!     ipnt3   ?
!     ipnt4   ?
!     ipnt5   ?
!     ipnt6   ?
!     ipnt7   ?
!     ipnt8   ?
!     ipnt9   ?
!     j       Counting variable
!     jh      corresponds to h,  see jsigma
!     k       Counting variable
!     kxy     ?
!     nxy     ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     285   number of nodal points nx,ny or nz even in the call of mesh (3d)
!     358   The number of nodal points per element is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh303' )
      if ( inpelm.eq.4 .or. inpelm.eq.5 ) then

!     --- inpelm = 4 or 5

         ik=0
         nelem=6*(nx-1)*(ny-1)*(nz-1)
         nxy=nx*ny
         do k=1,nz-1
            kxy=(k-1)*nxy
            do j=1,ny-1
               ih=(j-1)*nx+kxy
               do i=1,nx-1
                  ipnt1=i+ih
                  ipnt2=ipnt1+1
                  ipnt3=ipnt2+nx
                  ipnt4=ipnt3-1
                  ipnt5=ipnt1+nxy
                  ipnt6=ipnt2+nxy
                  ipnt7=ipnt3+nxy
                  ipnt8=ipnt4+nxy
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt8
                  kmeshc(3+ik)=ipnt5
                  kmeshc(4+ik)=ipnt7
                  ik=ik+inpelm
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt4
                  kmeshc(3+ik)=ipnt8
                  kmeshc(4+ik)=ipnt7
                  ik=ik+inpelm
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt3
                  kmeshc(3+ik)=ipnt4
                  kmeshc(4+ik)=ipnt7
                  ik=ik+inpelm
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt2
                  kmeshc(3+ik)=ipnt3
                  kmeshc(4+ik)=ipnt7
                  ik=ik+inpelm
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt6
                  kmeshc(3+ik)=ipnt2
                  kmeshc(4+ik)=ipnt7
                  ik=ik+inpelm
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt5
                  kmeshc(3+ik)=ipnt6
                  kmeshc(4+ik)=ipnt7
                  ik=ik+inpelm
               end do
            end do
         end do

      else if ( inpelm.eq.8 .or. inpelm.eq.9 ) then

!     --- inpelm = 8 or 9

         ik=0
         nelem=(nx-1)*(ny-1)*(nz-1)
         nxy=nx*ny
         do k=1,nz-1
            kxy=(k-1)*nxy
            do j=1,ny-1
               ih=(j-1)*nx+kxy
               do i=1,nx-1
                  kmeshc(1+ik)=i+ih
                  kmeshc(2+ik)=i+1+ih
                  kmeshc(3+ik)=i+1+nx+ih
                  kmeshc(4+ik)=i+nx+ih
                  kmeshc(5+ik)=i+nx*ny+ih
                  kmeshc(6+ik)=i+1+nx*ny+ih
                  kmeshc(7+ik)=i+1+(ny+1)*nx+ih
                  kmeshc(8+ik)=i+(ny+1)*nx+ih
                  ik=ik+inpelm
               end do
            end do
         end do

      else if ( inpelm.eq.10 ) then

!     --- inpelm = 10

         ik=0
         nelem=(nx-1)*(ny-1)*(nz-1)/8
         nelem=6*nelem
         if ( nx-(nx/2)*2.eq.0 .or. &
              ny-(ny/2)*2.eq.0 .or. &
              nz-(nz/2)*2.eq.0 ) &
             call errsub ( 285, 0, 0, 0 )
         nxy=nx*ny
         do k=1,nz-2,2
            kxy=(k-1)*nxy
            do j=1,ny-2,2
               ih=(j-1)*nx+kxy
               do i=1,nx-2,2
                  ipnt1=i+ih
                  ipnt2=ipnt1+1
                  ipnt3=ipnt2+1
                  ipnt4=ipnt1+nx
                  ipnt5=ipnt4+1
                  ipnt6=ipnt5+1
                  ipnt7=ipnt4+nx
                  ipnt8=ipnt7+1
                  ipnt9=ipnt8+1
                  ipnt10=ipnt1+nxy
                  ipnt11=ipnt2+nxy
                  ipnt12=ipnt3+nxy
                  ipnt13=ipnt4+nxy
                  ipnt14=ipnt5+nxy
                  ipnt15=ipnt6+nxy
                  ipnt16=ipnt7+nxy
                  ipnt17=ipnt8+nxy
                  ipnt18=ipnt9+nxy
                  ipnt19=ipnt10+nxy
                  ipnt20=ipnt11+nxy
                  ipnt21=ipnt12+nxy
                  ipnt22=ipnt13+nxy
                  ipnt23=ipnt14+nxy
                  ipnt24=ipnt15+nxy
                  ipnt25=ipnt16+nxy
                  ipnt26=ipnt17+nxy
                  ipnt27=ipnt18+nxy
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt13
                  kmeshc(3+ik)=ipnt25
                  kmeshc(4+ik)=ipnt22
                  kmeshc(5+ik)=ipnt19
                  kmeshc(6+ik)=ipnt10
                  kmeshc(7+ik)=ipnt14
                  kmeshc(8+ik)=ipnt26
                  kmeshc(9+ik)=ipnt23
                  kmeshc(10+ik)=ipnt27
                  ik=ik+10
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt4
                  kmeshc(3+ik)=ipnt7
                  kmeshc(4+ik)=ipnt16
                  kmeshc(5+ik)=ipnt25
                  kmeshc(6+ik)=ipnt13
                  kmeshc(7+ik)=ipnt14
                  kmeshc(8+ik)=ipnt17
                  kmeshc(9+ik)=ipnt26
                  kmeshc(10+ik)=ipnt27
                  ik=ik+10
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt5
                  kmeshc(3+ik)=ipnt9
                  kmeshc(4+ik)=ipnt8
                  kmeshc(5+ik)=ipnt7
                  kmeshc(6+ik)=ipnt4
                  kmeshc(7+ik)=ipnt14
                  kmeshc(8+ik)=ipnt18
                  kmeshc(9+ik)=ipnt17
                  kmeshc(10+ik)=ipnt27
                  ik=ik+10
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt2
                  kmeshc(3+ik)=ipnt3
                  kmeshc(4+ik)=ipnt6
                  kmeshc(5+ik)=ipnt9
                  kmeshc(6+ik)=ipnt5
                  kmeshc(7+ik)=ipnt14
                  kmeshc(8+ik)=ipnt15
                  kmeshc(9+ik)=ipnt18
                  kmeshc(10+ik)=ipnt27
                  ik=ik+10
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt11
                  kmeshc(3+ik)=ipnt21
                  kmeshc(4+ik)=ipnt12
                  kmeshc(5+ik)=ipnt3
                  kmeshc(6+ik)=ipnt2
                  kmeshc(7+ik)=ipnt14
                  kmeshc(8+ik)=ipnt24
                  kmeshc(9+ik)=ipnt15
                  kmeshc(10+ik)=ipnt27
                  ik=ik+10
                  kmeshc(1+ik)=ipnt1
                  kmeshc(2+ik)=ipnt10
                  kmeshc(3+ik)=ipnt19
                  kmeshc(4+ik)=ipnt20
                  kmeshc(5+ik)=ipnt21
                  kmeshc(6+ik)=ipnt11
                  kmeshc(7+ik)=ipnt14
                  kmeshc(8+ik)=ipnt23
                  kmeshc(9+ik)=ipnt24
                  kmeshc(10+ik)=ipnt27
                  ik=ik+10
               end do
            end do
         end do

      else if ( inpelm.eq.27 ) then

!     --- inpelm = 27

         ik=0
         nelem=(nx-1)*(ny-1)*(nz-1)/8
         if ( nx-(nx/2)*2.eq.0 .or. &
              ny-(ny/2)*2.eq.0 .or. &
              nz-(nz/2)*2.eq.0 ) &
             call errsub ( 285, 0, 0, 0 )
         nxy=nx*ny
         do k=1,nz-2,2
            kxy=(k-1)*nxy
            do j=1,ny-2,2
               ih=(j-1)*nx+kxy
               do i=1,nx-2,2
                  kmeshc(1+ik)=i+ih
                  kmeshc(2+ik)=i+1+ih
                  kmeshc(3+ik)=i+2+ih
                  kmeshc(4+ik)=i+nx+ih
                  kmeshc(5+ik)=i+1+nx+ih
                  kmeshc(6+ik)=i+2+nx+ih
                  kmeshc(7+ik)=i+2*nx+ih
                  kmeshc(8+ik)=i+2*nx+1+ih
                  kmeshc(9+ik)=i+2*nx+2+ih
                  do jh=1,18
                      kmeshc(9+ik+jh)=kmeshc(ik+jh)+nxy
                  end do
                  ik=ik+27
               end do
            end do
         end do

      else

!     --- inpelm = incorrect

         call errint ( inpelm, 1)
         call errsub ( 358, 1, 0, 0 )

      end if

      call erclos ( 'msh303' )
      end
