      subroutine msh317 ( kbndpt, nx, ny, nz, iinput, iwork, nsurfs )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.0  date 26-08-2002 Extra parameters and orientation
!        version  3.3  date 04-05-2002 Interchange triangles
!        version  3.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  3.1  date 11-03-1990 Change kbndpt(ii+5) and ii+6
!                                      after do )
!        version  3.0  date 22-01-1988 Reorganization of sequence
!        version  2.0  date 03-03-1986
!
!   copyright (c) 1982-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      fill kbndpt with the boundary elements in case of inpelm = 14 or 15
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh
!     mesh_generation
!     surface
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(14,*), nx, ny, nz, iinput(*), iwork(*), nsurfs

!     iinput         i    integer input array for the mesh generator
!                         only the part referred to the surface numbers
!                         corresponding to the brick is used
!     iwork               integer work array of the same length as kbndpt
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary stored per boundary element
!     nsurfs         i    Number of surfaces in mesh
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nstepx, kcht, kvlak1, kvlak2, kvlak3, ii, ihulp, &
              no6, n1, n2, m1, m2, nxy, ij, ijw, locsur, ih, iorien

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ihulp          Help variable
!     ii             Help variable
!     ij             help parameter
!     ijw            help parameter
!     iorien         Defines orientation of the base vectors in 3D plots
!                    1 x-y-z
!                    2 z-x-y
!                    3 y-z-x
!     j              Counting variable
!     kcht           help parameter
!     kvlak1         help parameter
!     kvlak2         help parameter
!     kvlak3         help parameter
!     locsur         Local surface number with respect to brick
!     m1             (n1-1)/2 i.e. number of elements in "1"-direction
!     m2             (n2-1)/2 i.e. number of elements in "2"-direction
!     n1             Number of points in "1"-direction
!     n2             Number of points in "2"-direction
!     no6            help parameter
!     nstepx         Number of steps in x-direction
!     nxy            nx * ny
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh317' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'In msh317'

      end if
      nstepx = 3*nx-1
      kcht = 5*nx*ny-2*nx-2*ny+1
      kvlak1 = nx*ny+(nx-1)*(ny-1)/2
      kvlak2 = (nx-1)*ny/2+(ny-1)*nx/2+(nx-1)*(ny-1)/4
      kvlak3 = nx*ny

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'nx, ny, nz, nstepx, kcht, kvlak1, kvlak2,', &
                         ' kvlak3 ',nx, ny, nz, nstepx, kcht, &
                         kvlak1, kvlak2, kvlak3

      end if

      ih = 0
      do locsur = 1, 6

!     --- Set n1 and n2 depending on locsur

         iorien = (iinput(locsur)-1)/nsurfs
         if ( locsur.eq.1 .or. locsur.eq.6 ) then
            n1 = nx
            n2 = ny
         else if ( locsur.eq.2 .or. locsur.eq.4 ) then
            n1 = nx
            n2 = nz
         else
            n1 = ny
            n2 = nz
         end if
         nxy = nx*ny
         m1 = (n1-1)/2
         m2 = (n2-1)/2

         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'locsur, iorien, nx, ny, nz, n1, n2, nxy ', &
                             locsur, iorien, nx, ny, nz, n1, n2, nxy

         end if

!        --- Fill array iwork with local nodal point numbers

         go to ( 110, 120, 130, 140, 150, 160 ), locsur

!        --- locsur.eq.1

110      do i = 1, kvlak1
            iwork(i) = i
         end do
         go to 200

!        --- locsur = 2 (1-2-6-5)

120      do j = 1, nz-2, 2
            do i = 1, nx-2, 2
               ii = i+(j-1)*kcht/2
               ijw = i+(j-1)*(n1+m1)
               iwork(ijw) = ii
               iwork(ijw+1) = ii+1
               iwork(ijw+2) = ii+2
               iwork(ijw+n1+m1+2) = ii+2+kvlak1+kvlak2
               iwork(ijw+2*n1+2*m1+2) = ii+2+kcht
               iwork(ijw+n1+m1+1) = ii+1+kvlak1+kvlak2
               iwork(ijw+n1-(i-1)/2) = ii+kvlak1-(i-1)/2
               iwork(ijw+2*n1+2*m1+1) = ii+1+kcht
               iwork(ijw+2*n1+2*m1) = ii+kcht
               iwork(ijw+n1+m1) = ii+kvlak1+kvlak2
               iwork(ijw+2*n1+m1-(i-1)/2) = &
                  ii+kvlak1+kvlak2+kvlak3-(i-1)/2
            end do
         end do
         go to 200

!        --- locsur = 3 (2-3-7-6)

130      do j = 1, nz-2, 2
            do i = 1, ny-2, 2
               ii = nx+(i-1)*nstepx/2+(j-1)*kcht/2
               ijw = i+(j-1)*(n1+m1)
               iwork(ijw) = ii
               iwork(ijw+1) = ii+nstepx/2
               iwork(ijw+2) = ii+nstepx
               iwork(ijw+n1+m1+2) = ii+kvlak2+nstepx*(ny-i)/2+(i+2)*nx
               iwork(ijw+2*n1+2*m1+2) = iwork(ijw+2)+kcht
               iwork(ijw+n1+m1+1) = iwork(ijw+n1+m1+2)-nx
               iwork(ijw+n1-(i-1)/2) = ii+ &
                  nstepx*(ny-i)/2+(5*(nx-1)/2+1)*(i+1)/2
               iwork(ijw+2*n1+2*m1+1) = iwork(ijw+2*n1+2*m1+2)-nstepx/2
               iwork(ijw+2*n1+2*m1) = iwork(ijw+2*n1+2*m1+2)-nstepx
               iwork(ijw+n1+m1) = iwork(ijw+n1+m1+1)-nx
               iwork(ijw+2*n1+m1-(i-1)/2) = ii+nstepx*(ny-i)/2+ &
                             kvlak2+kvlak3+(nx-1)*2+1+((i-1)*(5*nx-3))/4
            end do
         end do
         go to 200

!        --- locsur = 4 (4-3-7-8)

140      do j = 1, nz-2, 2
            do i = 1, nx-2, 2
               ii = kvlak1-nx+i+(j-1)*kcht/2
               ijw = i+(j-1)*(n1+m1)
               iwork(ijw) = ii
               iwork(ijw+1) = ii+1
               iwork(ijw+2) = ii+2
               iwork(ijw+n1+m1+2) = ii+kvlak2+kvlak3+2
               iwork(ijw+2*n1+2*m1+2) = ii+2+kcht
               iwork(ijw+n1+m1+1) = iwork(ijw+n1+m1+2)-1
               iwork(ijw+n1-(i-1)/2) = ii+(nx-i)/2+kvlak2+1
               iwork(ijw+2*n1+2*m1+1) = iwork(ijw+2*n1+2*m1+2)-1
               iwork(ijw+2*n1+2*m1) = iwork(ijw+2*n1+2*m1+2)-2
               iwork(ijw+n1+m1) = ii+kvlak2+kvlak3
               iwork(ijw+2*n1+m1-(i-1)/2) = ii+(nx-i)/2+1+ &
                   2*kvlak2+kvlak3
            end do
         end do
         go to 200

!        --- locsur = 5 (1-4-8-5)

150      do j = 1, nz-2, 2
            do i = 1, ny-2, 2
               ii = 1+(i-1)*nstepx/2+(j-1)*kcht/2
               ijw = i+(j-1)*(n1+m1)
               iwork(ijw) = ii
               iwork(ijw+1) = ii+nstepx/2
               iwork(ijw+2) = ii+nstepx
               iwork(ijw+n1+m1+2) = ii+nstepx*(ny-i)/2+kvlak2+(i+2)*nx
               iwork(ijw+2*n1+2*m1+2) = ii+nstepx+kcht
               iwork(ijw+n1+m1+1) = iwork(ijw+n1+m1+2)-nx
               iwork(ijw+n1-(i-1)/2) = ii+2*nx-1+(ny-i)*nstepx/2+ &
                              ((i-1)*(5*nx-3))/4
               iwork(ijw+2*n1+2*m1+1) = iwork(ijw+2*n1+2*m1+2)-nstepx/2
               iwork(ijw+2*n1+2*m1) = iwork(ijw+2*n1+2*m1+2)-nstepx
               iwork(ijw+n1+m1) = iwork(ijw+n1+m1+1)-nx
               iwork(ijw+2*n1+m1-(i-1)/2) = ii+3*(nx-1)/2+kvlak2+kvlak3+ &
                                   (ny-i)*nstepx/2+1+((i-1)*(5*nx-3))/4
            end do
         end do
         go to 200

!        --- locsur = 6 (5-6-7-8)

160      no6 = (ny-1)/2*(nx-1)*7
         ihulp = (nz-1)/2*kcht

         do i = 1, no6
             iwork(i) = i+ihulp
         end do
         go to 200

!        --- Fill kbndpt depending on the orientation

200      if ( debug ) then

!        --- Debug information

            call prinin ( iwork, n1*n2+m1*m2*2, 'iwork' )

         end if

         go to ( 210, 220, 230, 240, 250, 260, 270, 280 ), iorien+1

!        --- iorien = 1

210      do i = 1, m1
            do j = 1, m2
               ij = i+(j-1)*m1+ih
               ijw = 2*i-1+(2*j-2)*(n1+m1)
               kbndpt( 8,ij) = iwork(ijw+2)
               kbndpt( 9,ij) = iwork(ijw+2+n1+m1)
               kbndpt(10,ij) = iwork(ijw+2+2*n1+2*m1)
               kbndpt(11,ij) = iwork(ijw+1+n1+m1)
               kbndpt(12,ij) = iwork(ijw)
               kbndpt(13,ij) = iwork(ijw+1)
               kbndpt(14,ij) = iwork(ijw+n1-(i-1))
               kbndpt( 1,ij) = iwork(ijw+2*n1+2*m1)
               kbndpt( 2,ij) = iwork(ijw+n1+m1)
               kbndpt( 3,ij) = iwork(ijw)
               kbndpt( 4,ij) = iwork(ijw+1+n1+m1)
               kbndpt( 5,ij) = iwork(ijw+2+2*n1+2*m1)
               kbndpt( 6,ij) = iwork(ijw+1+2*n1+2*m1)
               kbndpt( 7,ij) = iwork(ijw+2*n1+m1-(i-1))
            end do
         end do
         go to 300

!        --- iorien = 2

220      do i = 1, m2
            do j = 1, m1
               ij = i+(j-1)*m2+ih
               ijw = n1*(2*i-1)-2*j+2+2*m1*(i-1)
               kbndpt( 8,ij) = iwork(ijw+2*n1+2*m1)
               kbndpt( 9,ij) = iwork(ijw+2*n1+2*m1-1)
               kbndpt(10,ij) = iwork(ijw+2*n1+2*m1-2)
               kbndpt(11,ij) = iwork(ijw+n1+m1-1)
               kbndpt(12,ij) = iwork(ijw)
               kbndpt(13,ij) = iwork(ijw+n1+m1)
               kbndpt(14,ij) = iwork(ijw+2*m1+n1+j-1)
               kbndpt( 1,ij) = iwork(ijw-2)
               kbndpt( 2,ij) = iwork(ijw-1)
               kbndpt( 3,ij) = iwork(ijw)
               kbndpt( 4,ij) = iwork(ijw+n1+m1-1)
               kbndpt( 5,ij) = iwork(ijw+2*n1+2*m1-2)
               kbndpt( 6,ij) = iwork(ijw+n1+m1-2)
               kbndpt( 7,ij) = iwork(ijw+m1+j-1)
            end do
         end do
         go to 300

!        --- iorien = 3

230      do i = 1, m1
            do j = 1, m2
               ij = i+(j-1)*m1+ih
               ijw = (n1+m1)*(n2-2*j+2)-2*i+2-m1
               kbndpt( 8,ij) = iwork(ijw-2)
               kbndpt( 9,ij) = iwork(ijw-2-n1-m1)
               kbndpt(10,ij) = iwork(ijw-2-2*n1-2*m1)
               kbndpt(11,ij) = iwork(ijw-1-m1-n1)
               kbndpt(12,ij) = iwork(ijw)
               kbndpt(13,ij) = iwork(ijw-1)
               kbndpt(14,ij) = iwork(ijw-n1+(i-1))
               kbndpt( 1,ij) = iwork(ijw-2*n1-2*m1)
               kbndpt( 2,ij) = iwork(ijw-n1-m1)
               kbndpt( 3,ij) = iwork(ijw)
               kbndpt( 4,ij) = iwork(ijw-1-m1-n1)
               kbndpt( 5,ij) = iwork(ijw-2-2*n1-2*m1)
               kbndpt( 6,ij) = iwork(ijw-1-2*n1-2*m1)
               kbndpt( 7,ij) = iwork(ijw-2*n1-m1+(i-1))
            end do
         end do
         go to 300

!        --- iorien = 4

240      do i = 1, m2
            do j = 1, m1
               ij = i+(j-1)*m2+ih
               ijw = (n1+m1)*(n2-2*i+1)+2*j-1
               kbndpt( 8,ij) = iwork(ijw-2*n1-2*m1)
               kbndpt( 9,ij) = iwork(ijw-2*n1-2*m1+1)
               kbndpt(10,ij) = iwork(ijw-2*n1-2*m1+2)
               kbndpt(11,ij) = iwork(ijw+1-n1-m1)
               kbndpt(12,ij) = iwork(ijw)
               kbndpt(13,ij) = iwork(ijw-n1-m1)
               kbndpt(14,ij) = iwork(ijw-n1-2*m1-(j-1))
               kbndpt( 1,ij) = iwork(ijw+2)
               kbndpt( 2,ij) = iwork(ijw+1)
               kbndpt( 3,ij) = iwork(ijw)
               kbndpt( 4,ij) = iwork(ijw+1-n1-m1)
               kbndpt( 5,ij) = iwork(ijw-2*n1-2*m1+2)
               kbndpt( 6,ij) = iwork(ijw+2-n1-m1)
               kbndpt( 7,ij) = iwork(ijw-m1-(j-1))
            end do
         end do
         go to 300

!        --- iorien = 5

250      do i = 1, m2
            do j = 1, m1
               ij = i+(j-1)*m2+ih
               ijw = 2*j-1+(2*i-2)*(n1+m1)
               kbndpt( 8,ij) = iwork(ijw+2*n1+2*m1)
               kbndpt( 9,ij) = iwork(ijw+1+2*n1+2*m1)
               kbndpt(10,ij) = iwork(ijw+2+2*n1+2*m1)
               kbndpt(11,ij) = iwork(ijw+1+n1+m1)
               kbndpt(12,ij) = iwork(ijw)
               kbndpt(13,ij) = iwork(ijw+n1+m1)
               kbndpt(14,ij) = iwork(ijw+2*n1+m1-(j-1))
               kbndpt( 1,ij) = iwork(ijw+2)
               kbndpt( 2,ij) = iwork(ijw+1)
               kbndpt( 3,ij) = iwork(ijw)
               kbndpt( 4,ij) = iwork(ijw+1+n1+m1)
               kbndpt( 5,ij) = iwork(ijw+2+2*n1+2*m1)
               kbndpt( 6,ij) = iwork(ijw+2+n1+m1)
               kbndpt( 7,ij) = iwork(ijw+n1-(j-1))
            end do
         end do
         go to 300

!        --- iorien = 6

260      do i = 1, m1
            do j = 1, m2
               ij = i+(j-1)*m1+ih
               ijw = n1*(2*j-1)-2*i+2+2*(j-1)*m1
               kbndpt( 8,ij) = iwork(ijw-2)
               kbndpt( 9,ij) = iwork(ijw+n1+m1-2)
               kbndpt(10,ij) = iwork(ijw+2*n1+2*m1-2)
               kbndpt(11,ij) = iwork(ijw+n1+m1-1)
               kbndpt(12,ij) = iwork(ijw)
               kbndpt(13,ij) = iwork(ijw-1)
               kbndpt(14,ij) = iwork(ijw+m1+(i-1))
               kbndpt( 1,ij) = iwork(ijw+2*n1+2*m1)
               kbndpt( 2,ij) = iwork(ijw+n1+m1)
               kbndpt( 3,ij) = iwork(ijw)
               kbndpt( 4,ij) = iwork(ijw+n1+m1-1)
               kbndpt( 5,ij) = iwork(ijw+2*n1+2*m1-2)
               kbndpt( 6,ij) = iwork(ijw+2*n1+2*m1-1)
               kbndpt( 7,ij) = iwork(ijw+n1+2*m1+(i-1))
            end do
         end do
         go to 300

!        --- iorien = 7

270      do i = 1, m2
            do j = 1, m1
               ij = i+(j-1)*m2+ih
               ijw = (n1+m1)*(n2-2*i+2)-2*j+2-m1
               kbndpt( 8,ij) = iwork(ijw-2*n1-2*m1)
               kbndpt( 9,ij) = iwork(ijw-2*n1-2*m1-1)
               kbndpt(10,ij) = iwork(ijw-2*n1-2*m1-2)
               kbndpt(11,ij) = iwork(ijw-1-n1-m1)
               kbndpt(12,ij) = iwork(ijw)
               kbndpt(13,ij) = iwork(ijw-n1-m1)
               kbndpt(14,ij) = iwork(ijw-2*n1-m1+(j-1))
               kbndpt( 1,ij) = iwork(ijw-2)
               kbndpt( 2,ij) = iwork(ijw-1)
               kbndpt( 3,ij) = iwork(ijw)
               kbndpt( 4,ij) = iwork(ijw-1-n1-m1)
               kbndpt( 5,ij) = iwork(ijw-2*n1-2*m1-2)
               kbndpt( 6,ij) = iwork(ijw-n1-m1-2)
               kbndpt( 7,ij) = iwork(ijw-n1+(j-1))
            end do
         end do
         go to 300

!        --- iorien = 8

280      do i = 1, m1
            do j = 1, m2
               ij = i+(j-1)*m1+ih
               ijw = 2*i-1+(n2-2*j+1)*(n1+m1)
               kbndpt( 8,ij) = iwork(ijw+2)
               kbndpt( 9,ij) = iwork(ijw-n1-m1+2)
               kbndpt(10,ij) = iwork(ijw-2*n1-2*m1+2)
               kbndpt(11,ij) = iwork(ijw-n1-m1+1)
               kbndpt(12,ij) = iwork(ijw)
               kbndpt(13,ij) = iwork(ijw+1)
               kbndpt(14,ij) = iwork(ijw-m1-(i-1))
               kbndpt( 1,ij) = iwork(ijw-2*n1-2*m1)
               kbndpt( 2,ij) = iwork(ijw-n1-m1)
               kbndpt( 3,ij) = iwork(ijw)
               kbndpt( 4,ij) = iwork(ijw-n1-m1+1)
               kbndpt( 5,ij) = iwork(ijw-2*n1-2*m1+2)
               kbndpt( 6,ij) = iwork(ijw-2*n1-2*m1+1)
               kbndpt( 7,ij) = iwork(ijw-n1-2*m1-(i-1))
            end do
         end do
         go to 300

300      if ( debug ) then

!        --- Debug information

            call prinin1 ( kbndpt(1,ih+1), m1*m2*2, 7, 'kbndpt' )

         end if
         ih = ih+m1*m2

      end do

      call erclos ( 'msh317' )
      end
