      subroutine msh353 ( kbndpt, linear, trian, kmscbt, kbndbt, npobot, &
                          nelz, nelbot, inpelmbot, nsurbt, nbndpt, &
                          kmscps, found, klp, nlp, inpelmsur )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.0  date 15-12-2002 Extra param's for prisms
!        version  4.0  date 29-04-1999 Extra param's for hollow pipes (BvR)
!        version  3.0  date 05-11-1997 Allow for alternating diagonals (BvR)
!        version  2.0  date 27-10-1988 Extension with pipesurface elements
!                                      in order to allow for surfaces of
!                                      surfaces
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.0  date 06-08-1987
!
!   copyright (c) 1987-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array kbndpt with nodal point numbers in elements in
!    surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical trian
      integer kbndpt(*), kmscbt(*), kbndbt(*), kmscps(*), found, &
              inpelmbot, linear, nbndpt, nelbot, nelz, npobot, nsurbt, &
              klp(2,*), nlp, inpelmsur

!     found          i    If 1 the diagonals are in alternating order, &
!                         if 0, they are not
!     inpelmbot      i    Number of nodes in surface elements
!     inpelmsur      i    Number of nodes per element of pipe surface
!     kbndbt         o    Array KBNDPT for bottom surface
!                         Contains the local nodal point numbers of the
!                         boundary of the bottom surface
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary
!     klp            i    In this array the relative node numbers with respect
!                         to array kbndbt of first and last point of a
!                         consecutive part are stored. The number
!                         of parts the boundary consists of is nlp
!                         Hence klp(1,1) = 1, klp(2,1) denotes the end point of
!                         the first part of the boundary
!                         klp(1,2) contains the node number of the first point
!                         of the second part. This must be klp(2,1)+1
!                         klp(2,nlp) contains the length of kbndbt
!     kmscbt         i    Kmesh part c corresponding to bottom surface
!                         Contains the local nodal point numbers of the bottom
!                         surface
!     kmscps         i    Kmesh part c corresponding to pipe surface
!                         This array is required to check the direction of
!                         diagonals in the pipe surface
!     linear         i    Indication of the type of elements to be created
!                         Linear elements:     linear=1
!                         Quadratic elements:  linear=2
!     nbndpt         o    Number of nodal points in boundary, length of array
!                         kbndpt
!     nelbot         i    Number of elements in bottom surface
!     nelz           i    Number of line elements in vertical direction
!     nlp            i    Number of disjoint parts the boundary of the bottom
!                         surface consists of
!     npobot         i    Number of points of bottom surface
!     nsurbt         i    Number of elements in boundary of bottom surface
!     trian          i    Indication of the type of elements to be created
!                         Tetrahedrons:  trian = true
!                         Bricks:        trian = false
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, ih1, ih2, ihalf, ikelsf, istep, j, nh, nstart, &
              ilp

!     i          Counting variable
!     ih         Help variable to store a constant temporarily
!     ih1        Help variable to store a constant temporarily
!     ih2        Help variable to store a constant temporarily
!     ihalf
!     ikelsf
!     ilp        Counting variable
!     istep      Step size
!     j          Counting variable
!     nh         number of nodes along a subcurve
!     nstart     Number of positions in array ISTART
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- First copy bottom surface

      do i = 1, nelbot*inpelmbot
         kbndpt(i) = kmscbt(i)
      end do

!     --- Next copy top surface

      istep = linear*npobot
      if ( trian .and. found.eq.0 ) &
         istep = istep+nelbot*(1+6*(linear-1))
      ih1 = nelz*istep
      ih2 = nelbot*inpelmbot
      do i = 1,nelbot*inpelmbot
         kbndpt(i+ih2) = kmscbt(i)+ih1
      end do

!     --- Finally envelope

      ih2 = 2*ih2
      nstart = ih2
      i = 1
      if ( inpelmsur.eq.3 ) then

!     --- inpelmsur = 3  linear triangles
!         First the lowest cross-section

         do ilp = 1,nlp
            do j = klp(1,ilp),klp(2,ilp)

!           --- Check direction of diagonal, compare with msh352
!               ikelsf is start adress in kmscps minus one

            ikelsf = 6 * (i-1)
            if ( kmscps(ikelsf+4).eq.i+1 ) then

!           --- small = true, normal direction

               if ( kmscps(ikelsf+2).eq.i ) then

!              --- orientation of diagonal from i to i+1+istep
!                  ___________
!                  |1     3  /|
!                  |       / 5|
!                  |      /   |
!                  |    /     |
!                  |2 / 6   4 |
!                  _/_________

                  kbndpt(ih2+1) = kbndbt(j)+istep
                  kbndpt(ih2+2) = kbndbt(j)
                  kbndpt(ih2+3) = kbndbt(j+1)+istep
                  kbndpt(ih2+4) = kbndbt(j+1)
                  kbndpt(ih2+5) = kbndbt(j+1)+istep
                  kbndpt(ih2+6) = kbndbt(j)

               else

!              --- orientation of diagonal from i+1 to i+istep
!                  ___________
!                  |\   1    3|
!                  |  \       |
!                  |5   \     |
!                  |      \   |
!                  |       \ 2|
!                  |6     4 \ |
!                  ___________

                  kbndpt(ih2+1) = kbndbt(j)+istep
                  kbndpt(ih2+2) = kbndbt(j+1)
                  kbndpt(ih2+3) = kbndbt(j+1)+istep
                  kbndpt(ih2+4) = kbndbt(j+1)
                  kbndpt(ih2+5) = kbndbt(j)+istep
                  kbndpt(ih2+6) = kbndbt(j)

               end if

            else

!           --- small = false, surface corresponds to negative subsurface

               if ( kmscps(ikelsf+5).eq.i+1 ) then

!              --- orientation of diagonal from i+1 to i+istep
!                  ___________
!                  |\   6    4|
!                  |  \       |
!                  |2   \     |
!                  |      \   |
!                  |       \ 5|
!                  |1     3 \ |
!                  ___________

                  kbndpt(ih2+1) = kbndbt(j)
                  kbndpt(ih2+2) = kbndbt(j)+istep
                  kbndpt(ih2+3) = kbndbt(j+1)
                  kbndpt(ih2+4) = kbndbt(j+1)+istep
                  kbndpt(ih2+5) = kbndbt(j+1)
                  kbndpt(ih2+6) = kbndbt(j)+istep

               else

!              --- orientation of diagonal from i+1 to i+istep
!                  ___________
!                  |6     4  /|
!                  |       / 2|
!                  |      /   |
!                  |    /     |
!                  |5 / 1   3 |
!                  _/_________

                  kbndpt(ih2+1) = kbndbt(j)
                  kbndpt(ih2+2) = kbndbt(j+1)+istep
                  kbndpt(ih2+3) = kbndbt(j+1)
                  kbndpt(ih2+4) = kbndbt(j+1)+istep
                  kbndpt(ih2+5) = kbndbt(j)
                  kbndpt(ih2+6) = kbndbt(j)+istep

               end if
            end if
            ih2 = ih2+6
            i = i+1
            end do
         end do
         nh = nsurbt*6

      else if ( inpelmsur.eq.4 ) then

!     --- inpelmsur = 4  linear quadrilaterals
!         First the lowest cross-section

         do ilp = 1,nlp
            do j = klp(1,ilp),klp(2,ilp)

!           --- Check if the surface is positive or negative

            ikelsf = 4 * (i-1)
            if ( kmscps(ikelsf+2).gt.kmscps(ikelsf+1) ) then

!           --- Positive surface

               kbndpt(ih2+1) = kbndbt(j)
               kbndpt(ih2+2) = kbndbt(j+1)
               kbndpt(ih2+3) = kbndbt(j+1)+istep
               kbndpt(ih2+4) = kbndbt(j)+istep
            else

!           --- Negative surface

               kbndpt(ih2+2) = kbndbt(j)
               kbndpt(ih2+1) = kbndbt(j+1)
               kbndpt(ih2+4) = kbndbt(j+1)+istep
               kbndpt(ih2+3) = kbndbt(j)+istep
            end if
            ih2 = ih2+4
            i = i+1
            end do
         end do
         nh = nsurbt*4

      else if ( inpelmsur.eq.6 ) then

!     --- inpelmsur = 6  quadratic triangles
!         First the lowest cross-section

         ihalf = npobot+3*nelbot
         do ilp = 1,nlp
            do j = klp(1,ilp),klp(2,ilp),2

!           --- Check direction of diagonal, compare with msh352
!               ikelsf is start adress in kmscps minus one

            ikelsf = 6 * (i-1)
            if ( kmscps(ikelsf+7).eq.i+2 ) then

!           --- small = true, normal direction

               kbndpt(ih2+1) = kbndbt(j)+istep
               kbndpt(ih2+2) = kbndbt(j)+ihalf
               kbndpt(ih2+3) = kbndbt(j)
               kbndpt(ih2+4) = kbndbt(j+1)+ihalf
               kbndpt(ih2+5) = kbndbt(j+2)+istep
               kbndpt(ih2+6) = kbndbt(j+1)+istep
               kbndpt(ih2+7) = kbndbt(j+2)
               kbndpt(ih2+8) = kbndbt(j+2)+ihalf
               kbndpt(ih2+9) = kbndbt(j+2)+istep
               kbndpt(ih2+10) = kbndbt(j+1)+ihalf
               kbndpt(ih2+11) = kbndbt(j)
               kbndpt(ih2+12) = kbndbt(j+1)

            else

!           --- small = false, surface corresponds to negative subsurface

               kbndpt(ih2+1) = kbndbt(j)
               kbndpt(ih2+2) = kbndbt(j)+ihalf
               kbndpt(ih2+3) = kbndbt(j)+istep
               kbndpt(ih2+4) = kbndbt(j+1)+ihalf
               kbndpt(ih2+5) = kbndbt(j+2)
               kbndpt(ih2+6) = kbndbt(j+1)
               kbndpt(ih2+7) = kbndbt(j+2)+istep
               kbndpt(ih2+8) = kbndbt(j+2)+ihalf
               kbndpt(ih2+9) = kbndbt(j+2)
               kbndpt(ih2+10) = kbndbt(j+1)+ihalf
               kbndpt(ih2+11) = kbndbt(j)+istep
               kbndpt(ih2+12) = kbndbt(j+1)+istep
            end if
            ih2 = ih2+12
            i = i+2
            end do
         end do
         nh = nsurbt*12

      else if ( inpelmsur.eq.9 ) then

!     --- inpelmsur = 9  quadratic quadrilaterals
!         First the lowest cross-section

         do ilp = 1,nlp
            do j = klp(1,ilp),klp(2,ilp),2
            ikelsf = 9 * (i-1)/2
            if ( kmscps(ikelsf+2).gt.kmscps(ikelsf+1) ) then

!           --- Positive surface

               kbndpt(ih2+1) = kbndbt(j)
               kbndpt(ih2+2) = kbndbt(j+1)
               kbndpt(ih2+3) = kbndbt(j+2)
               kbndpt(ih2+4) = kbndbt(j+2)+npobot
               kbndpt(ih2+5) = kbndbt(j+2)+istep
               kbndpt(ih2+6) = kbndbt(j+1)+istep
               kbndpt(ih2+7) = kbndbt(j)+istep
               kbndpt(ih2+8) = kbndbt(j)+npobot
               kbndpt(ih2+9) = kbndbt(j+1)+npobot
            else

!           --- Negative surface

               kbndpt(ih2+3) = kbndbt(j)
               kbndpt(ih2+2) = kbndbt(j+1)
               kbndpt(ih2+1) = kbndbt(j+2)
               kbndpt(ih2+8) = kbndbt(j+2)+npobot
               kbndpt(ih2+7) = kbndbt(j+2)+istep
               kbndpt(ih2+6) = kbndbt(j+1)+istep
               kbndpt(ih2+5) = kbndbt(j)+istep
               kbndpt(ih2+4) = kbndbt(j)+npobot
               kbndpt(ih2+9) = kbndbt(j+1)+npobot
            end if
            ih2 = ih2+9
            i = i+2
            end do
         end do
         nh = nsurbt*9
      end if

!     --- Copy the last nelz-1 cross-sections from the first one

      do i = 1,nelz-1
         ih = i*istep
         do j = 1,nh
            kbndpt(ih2+j) = kbndpt(nstart+j)+ih
         end do
         ih2 = ih2+nh
      end do
      nbndpt = ih2

      end
