      subroutine msh3dlinearbnd ( itriaold, itrianew, npointold, &
                                  npointnew, map, inpelm, coorold, &
                                  coornew, nelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map the boundary consisting of quadratic triangle with 7 points
!     onto a boundary consisting of linear elements only
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     boundary
!     triangle
!     map
!     linear
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, itriaold(inpelm,*), itrianew(3,*), npointold, &
              npointnew, map(npointold), nelem
      double precision coorold(3,*), coornew(3,*)

!     coornew        o    Array containing the coordinates of the vertices
!     coorold        i    Array containing the coordinates of the original
!                         7-point triangles
!     inpelm         i    number of points per element
!     itrianew       o    Contains the topology of the new triangle surface
!                         consisting of the vertices only
!     itriaold       i    Contains the topology of the old triangle surface
!                         containing all nodes
!     map            o    Array of size npointold containing the new node
!                         numbers of the triangles
!                         map(i) = 0, means that node i is not a vertex
!                                > 0, gives the vertex sequence number
!     nelem          i    Number of elements
!     npointnew      o    New number of nodes (= vertices)
!     npointold      i    Number of nodes in original mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode, j, ivertx, ielem

!     i              Counting variable
!     ielem          Element number
!     inode          Node sequence number
!     ivertx         node number of vertex
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2174   Wrong shape for surface elements
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array map
!     inode := 0
!     For all elements ielem do
!        For all vertices ivertx do
!           if ( map(ivertx) = 0 ) then
!              inode := inode+1
!              map((ivertx) := inode
!              coornew(ivertx) := coorold(ivertx)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh3dlinearbnd' )
      if ( inpelm.ne.7 ) then

!     --- shape number not available

         call errint ( inpelm, 1 )
         call errsub ( 2174, 1, 0, 0 )
         go to 1000

      end if

!     --- Clear array map

      do i = 1, npointold
         map(i) = 0
      end do

!     --- inode := 0

      inode = 0

!     --- For all elements ielem do

      do ielem = 1, nelem

!     --- For all vertices ivertx do

         do i = 1, 3

            ivertx = itriaold(2*i-1,ielem)
            if ( map(ivertx).eq.0 ) then

!           --- map(ivertx) = 0, vertex has not been renumbered yet
!               inode := inode+1
!               map((ivertx) := inode
!               coornew(inode) := coorold(ivertx)

               inode = inode+1
               map(ivertx) = inode
               do j = 1, 3
                  coornew(j,inode) = coorold(j,ivertx)
               end do

            end if
            itrianew(i,ielem) = map(ivertx)

         end do

      end do

      npointnew = inode

1000  call erclos ( 'msh3dlinearbnd' )
      end
