      subroutine msh3dquadpip ( kmeshc, inpelm, nelem, nedges, nfaces, &
                                nmesha, nmeshd, npoint, ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmeshc in the specific case of a pipe and quadratic tetrahedrons of
!     shape 15 or 16
!     It has been assumed that kmeshc has been filled before for the
!     corresponding linear tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     pipe
!     tetrahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), inpelm, nelem, nedges, nfaces, nmeshd(*), &
              ishape, nmesha(*), npoint

!     inpelm         i    number of points per element
!     ishape         i    Shape number of elements
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!                         At input the linear tetrahedrons with corresponding
!                         node numbers
!                         At output the tetrahedrons of shape 15/16 with the new
!                         numbering
!     nedges         i    number of edges of volume
!     nelem          i    Number of elements of volume
!     nfaces         i    Number of faces of volume
!     nmesha         i    nmesh part a
!     nmeshd         i    nmesh part d
!     npoint        i/o   Number of nodal points of volume
!                         At input for the linear subdivision
!                         At output for the quadratic elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, istartold, istartnew, ielem, npointedges, &
              npointfaces

!     i              Counting variable
!     ielem          Element number
!     istart         First position used at a part
!     istartnew      New starting address in array kmeshc
!     istartold      Old starting address in array kmeshc
!     npointedges    Number of vertices + number of edges in volume
!     npointfaces    Number of vertices + number of edges + number of faces
!                    in volume
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh3dquadpip' )

!     --- For ielem := nelem (-1) 1

      npointedges = npoint+nedges
      npointfaces = npointedges+nfaces
      do ielem = nelem, 1, -1

!     --- Compute old starting address in kmeshc of element ielem
!         Compute new starting address in kmeshc of element ielem

         istartold = (ielem-1)*4
         istartnew = (ielem-1)*inpelm

!        --- Copy old tetrahedron to new positions

         kmeshc(istartnew+10) = kmeshc(istartold+4)
         do i = 3, 1, -1
            kmeshc(istartnew+2*i-1) = kmeshc(istartold+i)
         end do

!        --- Fill edges

         istart = nmeshd(ielem)
         kmeshc(istartnew+2) = abs(nmeshd(istart+1))+npoint
         kmeshc(istartnew+4) = abs(nmeshd(istart+5))+npoint
         kmeshc(istartnew+6) = abs(nmeshd(istart+4))+npoint
         kmeshc(istartnew+7) = abs(nmeshd(istart+3))+npoint
         kmeshc(istartnew+8) = abs(nmeshd(istart+2))+npoint
         kmeshc(istartnew+9) = abs(nmeshd(istart+6))+npoint

!        --- Fill faces

         istart = nmesha(ielem)
         kmeshc(istartnew+11) = abs(nmesha(istart+4))+npointedges
         kmeshc(istartnew+12) = abs(nmesha(istart+1))+npointedges
         kmeshc(istartnew+13) = abs(nmesha(istart+2))+npointedges
         kmeshc(istartnew+14) = abs(nmesha(istart+3))+npointedges

         if ( ishape.eq.16 ) then

!        --- Put node in centroid

            kmeshc(istartnew+inpelm) = npointfaces+ielem

         end if

      end do
      npoint = npointfaces
      if ( ishape.eq.16 ) npoint = npoint+nelem

      call erclos ( 'msh3dquadpip' )
      end
