      subroutine msh431(i1,i2,base1,base2,help,new,inew,kwad)
! * &
! ***********************************************************************
! * &
! *   Parameters
! * &
      double precision base1,base2,help(*)
      integer i1,i2,new,inew,kwad
! * &
! ***********************************************************************
! *                                                                     * &
! *   Subroutine to determine the node number of new created point on   * &
! *   line i1 - i2.                                                     * &
! *                                                                     * &
! ***********************************************************************
!                                                                       * &
!                                                                       * &
!         programmer   niek praagman                                    * &
!                                                                       * &
!         version 4.0  date  10-01-1990 Relative and absolute case)      * &
!         version 3.1  date  18-12-1989 Renamed version of 421    )      * &
!         version 3.0  date  05-12-1989 addition of quadratic case)      * &
!         version 2    date  23-11-1989                                   * &
!         version 1    date  10-02-1988                                   * &
!                                                                       * &
!                                                                       * &
!                                                                       * &
!   copyright (c) 1988 - 1990  "Ingenieursbureau SEPRA"                 * &
!   permission to copy or distribute this software or documentation     * &
!   in hard or soft copy granted only by written license obtained       * &
!   from "Ingenieursbureau SEPRA".                                      * &
!   all rights reserved. no part of this publication may be reproduced, * &
!   stored in a retrieval system (e.g. , in memory, disk or core)       * &
!   or be transmitted by any means, electronic, mechanical, photocopy,  * &
!   recording, or otherwise, without written permission from the        * &
!   publisher.                                                          * &
!                                                                       * &
! ***********************************************************************
! *                                                                     * &
! *   Meaning of parameters :                                           * &
! *                                                                     * &
! *   i1,i2          (i)  :  nodenumbers of line to be considered       * &
! *   base1,base2    (i)  :  basis numbers on which array help is filled* &
! *   help           (i)  :  array with values  ia * base1 + ib *base2 +* &
! *                          inew                                       * &
! *   new            (i)  :  total number of new created points         * &
! *   inew           (o)  :  node number of (first new) point on line   * &
! *                          i1 - i2                                    * &
! *                          if  inew = 0  then no point has been       * &
! *                          created yet on this line                   * &
! *   kwad           (i)  :  indicator whether linear or quadratic ele- * &
! *                          ments are considered                       * &
! *                                                                     * &
! ***********************************************************************
! * &
! *   Local variables
! * &
      double precision getal,snew,rest
      integer istep,i
! * &
! ***********************************************************************
! * &
! *   Compute base
! * &
      if ( i1.lt.i2 ) then
           getal = i1 * base1 + i2 * base2
      else
           getal = i2 * base1 + i1 * base2
      end if
! * &
      snew = new + 1d0
! * &
      if ( kwad.eq.1 ) then
           istep = 1
      else if ( kwad.eq.2 ) then
           istep = 2
      end if
! * &
      do 100 i = 1 , new , istep
! * &
          rest =  help(i) - getal
          if ( 0.lt.rest .and. snew.gt.rest ) then
! * &
               inew = i
               return
! * &
          end if
! * &
100   continue
! * &
! *   No point found , hence not yet created
! * &
      inew = 0
! * &
      end
