      subroutine msh433( kelem, nelem, coor, coorh, npoint, new, &
                         crit, help, base1, base2, meshtp, npelm, &
                         kelemh, nelhel, irelat )
! =======================================================================
!
!
!         programmer   niek praagman
!
!         version 4.2  date  08-11-1993 meshtp -199 adjusted
!         version 4.1  date  05-07-1991 crit 1.5 replaced by 0.5d0
!         version 4.0  date  10-01-1990 Relative and absolute crit
!         version 3.0  date  18-12-1989 Renamed version msh423
!         version 2.0  date  06-12-1989 New linear version
!         version 1    date  06-03-1988
!
!
!
!   copyright (c) 1988-1993  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Compute whether an elementside is too long with respect to a given
!     criterion in array crit. If so then an extra new point is placed.
!     New coordinates are placed in coorh and new elements in kelemh.
!     Only linear elements are considered (meshtp=-9,1,3)
!
! ***********************************************************************
!
!     KEYWORDS
!
!     mesh_generation
!     linear
!     refine
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none

      double precision coor(*), coorh(*), crit(*), help(*), base1, &
             base2
      integer kelem(*), nelem, npoint, new, meshtp, npelm, kelemh(*), &
              nelhel, irelat

!     base1   i    basisvalue for new points
!
!     base2   i    basisvalue for new points
!
!     coor    i    old coordinate array
!
!     coorh   o    new coordinate array
!
!     crit    i    criterion array specified by user
!
!     help         helparray for points
!
!     irelat  i    indicator for refine-criterium
!
!     kelem   i    old element array
!
!     kelemh  o    new element array
!
!     meshtp  i    Sepran meshtype of elements considered
!
!     nelem   i    number of elements
!
!     new     o    number of new created points
!
!     npelm   i    number of nodes in element
!
!     npoint  i    old number of nodes
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS

      integer kwad, i, i1, i2, i3, i4, i5, i6, ipo, ipo1, ipo2, ipo3, &
              inew, jnew, j1, j2, j3, j4, j5, ih, iknp, iref, ipoint

      double precision x1, x2, x3, y1, y2, y3, r1, r2, r3, &
              cri1, cri2, cri3, xj1, yj1, xj3, yj3, xj4, yj4, &
              xj5, yj5, disa, disb

!      cri1   criterium value in node 1
!
!      cri2   criterium value in node 2
!
!      cri3   criterium value in node 3
!
!      disa   Euclidian length of line 1
!
!      disb   Euclidian length of line 2
!
!      i      loop variable
!
!      i1     node number of first node
!
!      i2     node number of secon node
!
!      i3     node number of third node
!
!      i4     new node number of fourth node
!
!      i5     new node number of fifth node
!
!      i6     new node number of sixth node
!
!      ih     help node number
!
!      iknp   node number
!
!      inew   new node number
!
!      ipo    help variable for determination on which sides
!             new points have to be created
!
!      ipo1   help for first side
!
!      ipo2   help for second side
!
!      ipo3   help for third side
!
!      ipoint local point number
!
!      iref   check variable whether criterium has been met
!
!      j1     helpvariable if not all sides have to be refined
!
!      j2     helpvariable if not all sides have to be refined
!
!      j3     helpvariable if not all sides have to be refined
!
!      j4     helpvariable if not all sides have to be refined
!
!      j5     helpvariable if not all sides have to be refined
!
!      jnew   extra new node number
!
!      kwad   quadratic indicator
!
!      nelhel helpvariable for number of elements
!
!      r1     euclidian distance first side
!
!      r2     euclidian distance second side
!
!      r3     euclidian distance third side
!
!      x1     x-coordinate node 1
!
!      x2     x-coordinate node 2
!
!      x3     x-coordinate node 3
!
!      xj1    x-coordinate extra point 1
!
!      xj3    x-coordinate extra point 3
!
!      xj4    x-coordinate extra point 4
!
!      xj5    x-coordinate extra point 5
!
!      y1     y-coordinate node 1
!
!      y2     y-coordinate node 2
!
!      y3     y-coordinate node 3
!
!      yj1    y-coordinate extra point 1
!
!      yj3    y-coordinate extra point 3
!
!      yj4    y-coordinate extra point 4
!
!      yj5    y-coordinate extra point 5
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS    deconcatenate name of calling string
!     EROPEN    concatenate name to string of calling subroutines
!     ERRINT    place value in integer error array
!     ERRSUB    submit error message
!     MSH431    check whether new point has to be placed on line
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
!      958    wrong (not supported) value for meshtp
!
! ***********************************************************************
!
!     PSEUDO CODE
!
! =======================================================================

      call eropen( 'msh433' )

!     Set linear parameter kwad

      kwad = 1

!     Check each side of each element , and place coordinates of
!     new points in array coorh

      if ( meshtp.eq.3 ) then

!          npelm = 3

           do 100 i=1,nelem

               i1 = kelem(3*i-2)
               i2 = kelem(3*i-1)
               i3 = kelem(3*i  )

               x1 = coor(2*i1-1)
               y1 = coor(2*i1  )
               x2 = coor(2*i2-1)
               y2 = coor(2*i2  )
               x3 = coor(2*i3-1)
               y3 = coor(2*i3  )

               r1 = sqrt( (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1) )
               r2 = sqrt( (x3-x2)*(x3-x2)+(y3-y2)*(y3-y2) )
               r3 = sqrt( (x1-x3)*(x1-x3)+(y1-y3)*(y1-y3) )

               cri1 = ( crit(i1) + crit(i2) ) / 2.d0
               cri2 = ( crit(i2) + crit(i3) ) / 2.d0
               cri3 = ( crit(i3) + crit(i1) ) / 2.d0


               ipo  =  0
               ipo1 =  0
               ipo2 =  0
               ipo3 =  0

               iref =  0

!              Check criterium

               if ( irelat.eq.1 .and. cri1.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r1.gt.cri1 ) iref = 1

               if ( iref.eq.1 ) then
                    ipo  = ipo + 1
                    ipo1 = 1
                    inew = 0
                    call msh431(i1,i2,base1,base2,help,new,inew,kwad)
                    if ( inew.eq.0 ) then
                         new = new + 1
                         if ( i1.lt.i2 ) then
                         help(new) = i1 * base1 + i2 * base2 + new
                         else
                         help(new) = i2 * base1 + i1 * base2 + new
                         end if
                         ipoint = npoint + new
                         coorh( 2*ipoint-1 ) = ( x1 + x2 ) / 2d0
                         coorh( 2*ipoint   ) = ( y1 + y2 ) / 2d0
                         crit (   ipoint   ) =   cri1
                    end if
               end if

               iref =  0

!              Check criterium

               if ( irelat.eq.1 .and. cri2.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r2.gt.cri2 ) iref = 1

               if ( iref.eq.1 ) then
                    ipo  = ipo + 1
                    ipo2 = 1
                    inew = 0
                    call msh431(i2,i3,base1,base2,help,new,inew,kwad)
                    if ( inew.eq.0 ) then
                         new = new + 1
                         if ( i2.lt.i3 ) then
                         help(new) = i2 * base1 + i3 * base2 + new
                         else
                         help(new) = i3 * base1 + i2 * base2 + new
                         end if
                         ipoint = npoint + new
                         coorh( 2*ipoint-1 ) = ( x2 + x3 ) / 2d0
                         coorh( 2*ipoint   ) = ( y2 + y3 ) / 2d0
                         crit (   ipoint   ) =   cri2
                    end if
               end if

               iref =  0

!              Check criterium

               if ( irelat.eq.1 .and. cri3.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r3.gt.cri3 ) iref = 1

               if ( iref.eq.1 ) then
                    ipo  = ipo + 1
                    ipo3 = 1
                    inew = 0
                    call msh431(i3,i1,base1,base2,help,new,inew,kwad)
                    if ( inew.eq.0 ) then
                         new = new + 1
                         if ( i3.lt.i1 ) then
                         help(new) = i3 * base1 + i1 * base2 + new
                         else
                         help(new) = i1 * base1 + i3 * base2 + new
                         end if
                         ipoint = npoint + new
                         coorh( 2*ipoint-1 ) = ( x1 + x3 ) / 2d0
                         coorh( 2*ipoint   ) = ( y1 + y3 ) / 2d0
                         crit (   ipoint   ) =   cri3
                    end if
               end if

               if ( ipo.eq.0 ) then

                    nelhel             = nelhel + 1
                    kelemh(3*nelhel-2) = i1
                    kelemh(3*nelhel-1) = i2
                    kelemh(3*nelhel  ) = i3

               else if ( ipo.eq.1 ) then

!                   Only one side too long

                    if ( ipo1.eq.1 ) then
                         j1 = i1
                         j2 = i2
                         j3 = i3

                    else if ( ipo2.eq.1 ) then
                         j1 = i2
                         j2 = i3
                         j3 = i1

                    else if ( ipo3.eq.1 ) then
                         j1 = i3
                         j2 = i1
                         j3 = i2
                    end if

                    call msh431(j1,j2,base1,base2,help,new,inew,kwad)
                    j4 = npoint + inew
                    nelhel = nelhel + 1
                    kelemh(3*nelhel-2) = j1
                    kelemh(3*nelhel-1) = j4
                    kelemh(3*nelhel  ) = j3
                    nelhel = nelhel + 1
                    kelemh(3*nelhel-2) = j4
                    kelemh(3*nelhel-1) = j2
                    kelemh(3*nelhel  ) = j3

               else if ( ipo.eq.2 ) then

!                   Two sides too long

                    if ( (ipo1 + ipo2).eq.2 ) then
                          j1 = i1
                          j2 = i2
                          j3 = i3
                    end if

                    if ( (ipo2 + ipo3).eq.2 ) then
                          j1 = i2
                          j2 = i3
                          j3 = i1
                    end if

                    if ( (ipo3 + ipo1).eq.2 ) then
                          j1 = i3
                          j2 = i1
                          j3 = i2
                    end if

                    call msh431(j1,j2,base1,base2,help,new,inew,kwad)
                    j4 = npoint + inew
                    call msh431(j2,j3,base1,base2,help,new,inew,kwad)
                    j5 = npoint + inew

                    nelhel = nelhel + 1
                    kelemh(3*nelhel-2) = j2
                    kelemh(3*nelhel-1) = j5
                    kelemh(3*nelhel  ) = j4

                    xj1   = coorh(2*j1-1)
                    yj1   = coorh(2*j1  )
                    xj3   = coorh(2*j3-1)
                    yj3   = coorh(2*j3  )
                    xj4   = coorh(2*j4-1)
                    yj4   = coorh(2*j4  )
                    xj5   = coorh(2*j5-1)
                    yj5   = coorh(2*j5  )
                    disa = (xj5-xj1)*(xj5-xj1)+(yj5-yj1)*(yj5-yj1)
                    disb = (xj4-xj3)*(xj4-xj3)+(yj4-yj3)*(yj4-yj3)

                    if ( disa.lt.disb ) then
                         nelhel = nelhel + 1
                         kelemh(3*nelhel-2) = j1
                         kelemh(3*nelhel-1) = j4
                         kelemh(3*nelhel  ) = j5
                         nelhel = nelhel + 1
                         kelemh(3*nelhel-2) = j1
                         kelemh(3*nelhel-1) = j5
                         kelemh(3*nelhel  ) = j3
                    else
                         nelhel = nelhel + 1
                         kelemh(3*nelhel-2) = j1
                         kelemh(3*nelhel-1) = j4
                         kelemh(3*nelhel  ) = j3
                         nelhel = nelhel + 1
                         kelemh(3*nelhel-2) = j4
                         kelemh(3*nelhel-1) = j5
                         kelemh(3*nelhel  ) = j3
                    end if

               else if ( ipo.eq.3 ) then

                    call msh431(i1,i2,base1,base2,help,new,inew,kwad)
                    i4 = npoint + inew
                    call msh431(i2,i3,base1,base2,help,new,inew,kwad)
                    i5 = npoint + inew
                    call msh431(i3,i1,base1,base2,help,new,inew,kwad)
                    i6 = npoint + inew

                    nelhel = nelhel + 4
                    kelemh(3*nelhel-11) = i1
                    kelemh(3*nelhel-10) = i4
                    kelemh(3*nelhel- 9) = i6
                    kelemh(3*nelhel- 8) = i4
                    kelemh(3*nelhel- 7) = i2
                    kelemh(3*nelhel- 6) = i5
                    kelemh(3*nelhel- 5) = i4
                    kelemh(3*nelhel- 4) = i5
                    kelemh(3*nelhel- 3) = i6
                    kelemh(3*nelhel- 2) = i5
                    kelemh(3*nelhel- 1) = i3
                    kelemh(3*nelhel   ) = i6

               end if

100        continue

      else if ( meshtp.eq.1 ) then

!          npelm = 2 (line element)

           do 200 i=1,nelem

               i1 = kelem(2*i-1)
               i2 = kelem(2*i  )

               x1 = coor(2*i1-1)
               y1 = coor(2*i1  )
               x2 = coor(2*i2-1)
               y2 = coor(2*i2  )

               r1 = sqrt( (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1) )

               cri1 = ( crit(i1) + crit(i2) ) / 2.d0

               iref =  0

!              Check criterium

               if ( irelat.eq.1 .and. cri1.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r1.gt.cri1 ) iref = 1

               if ( iref.eq.1 ) then
                    inew = 0
                    call msh431(i1,i2,base1,base2,help,new,inew,kwad)
                    if ( inew.eq.0 ) then
                         new = new + 1
                         if ( i1.lt.i2 ) then
                         help(new) = i1 * base1 + i2 * base2 + new
                         else
                         help(new) = i2 * base1 + i1 * base2 + new
                         end if
                         ipoint = npoint + new
                         coorh( 2*ipoint-1 ) = ( x1 + x2 ) / 2d0
                         coorh( 2*ipoint   ) = ( y1 + y2 ) / 2d0
                         crit (   ipoint   ) =   cri1
                    end if

                    jnew = npoint + new
                    nelhel = nelhel + 2
                    kelemh( 2*nelhel - 3 ) = i1
                    kelemh( 2*nelhel - 2 ) = jnew
                    kelemh( 2*nelhel - 1 ) = jnew
                    kelemh( 2*nelhel     ) = i2

               else

                    nelhel = nelhel + 1
                    kelemh( 2*nelhel - 1 ) = i1
                    kelemh( 2*nelhel     ) = i2

               end if

200        continue

      else if ( meshtp.eq.-9 ) then

!          one large line element (nelem=1) with npelm points

           ih   =  1

           do 300 iknp = 1 , npelm - 1

               i1 = kelem(iknp  )
               i2 = kelem(iknp+1)

               x1 = coor(2*i1-1)
               y1 = coor(2*i1  )
               x2 = coor(2*i2-1)
               y2 = coor(2*i2  )

               r1 = sqrt( (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1) )

               cri1 = ( crit(i1) + crit(i2) ) / 2.d0

               iref =  0

!              Check criterium

               if ( irelat.eq.1 .and. cri1.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r1.gt.cri1 ) iref = 1

               if ( iref.eq.1 ) then
                    inew = 0
                    call msh431(i1,i2,base1,base2,help,new,inew,kwad)
                    if ( inew.eq.0 ) then
                         new = new + 1
                         if ( i1.lt.i2 ) then
                         help(new) = i1 * base1 + i2 * base2 + new
                         else
                         help(new) = i2 * base1 + i1 * base2 + new
                         end if
                         ipoint = npoint + new
                         coorh( 2*ipoint-1 ) = ( x1 + x2 ) / 2d0
                         coorh( 2*ipoint   ) = ( y1 + y2 ) / 2d0
                         crit (   ipoint   ) =   cri1

                         jnew   = ipoint

                    else

                         jnew   = npoint + inew

                    end if

                    kelemh( ih     ) = i1
                    kelemh( ih + 1 ) = jnew
                    kelemh( ih + 2 ) = i2
                    ih               = ih + 2

               else

                    kelemh( ih     ) = i1
                    kelemh( ih + 1 ) = i2
                    ih               = ih + 1

               end if

300       continue

!         Special situation
!         same number of elements, i.e. 1 but different number
!         of nodes

          nelhel = 1
          npelm  = ih

      else

          call errint(  meshtp  , 1 )
          call errsub ( 958, 1, 0, 0 )

      end if

      call erclos( 'msh433' )

      end
