      subroutine msh434(kelem,nelem,coor,coorh,npoint,new,crit, &
                        help,base1,base2,meshtp,irelat)
! * &
! ***********************************************************************
! *                                                                     * &
! *   Subroutine to compute whether an elementside is too long with     * &
! *   respect to a given criterion in array crit. If so then an extra   * &
! *   new point is placed. New coordinates are placed in coorh.         * &
! *   Only quadratic elements are considered (meshtp = 2 , 4 )          * &
! *                                                                     * &
! ***********************************************************************
!                                                                       * &
!                                                                       * &
!         programmer   niek praagman                                    * &
!                                                                       * &
!         version 3.1    date  05-07-1991  crit 1.5 replaced by 0.5d0 )  * &
!         version 3.0    date  10-01-1990 relative and absolute crit)    * &
!         version 2.0    date  06-12-1989 complete new version)          * &
!                                                                       * &
!                                                                       * &
!                                                                       * &
!   copyright (c) 1988-1990  "ingenieursbureau sepra"                   * &
!   permission to copy or distribute this software or documentation     * &
!   in hard or soft copy granted only by written license obtained       * &
!   from "ingenieursbureau sepra".                                      * &
!   all rights reserved. no part of this publication may be reproduced, * &
!   stored in a retrieval system (e.g. , in memory, disk or core)       * &
!   or be transmitted by any means, electronic, mechanical, photocopy,  * &
!   recording, or otherwise, without written permission from the        * &
!   publisher.                                                          * &
!                                                                       * &
! ***********************************************************************
! * &
! *   GLOBAL parameters
! * &
      double precision coor(*),coorh(*),crit(*),help(*),base1,base2
      integer kelem(*),nelem,npoint,new,meshtp,irelat
! *                                                                     * &
! *   Meaning of parameters :                                           * &
! *                                                                     * &
! *   kelem   (i)  :  element array                                     * &
! *   nelem   (i)  :  number of elements                                * &
! *   coor    (i)  :  old coordinate array                              * &
! *   coorh   (o)  :  new coordinate array                              * &
! *   npoint  (i)  :  old number of nodes                               * &
! *   new     (o)  :  number of new created points                      * &
! *   crit    (i)  :  criterion array specified by user                 * &
! *   help         :  helparray for points                              * &
! *   base1   (i)  :  basisvalue for new points                         * &
! *   base2   (i)  :  basisvalue for new points                         * &
! *   meshtp  (i)  :  Sepran meshtype of elements considered            * &
! *   irelat  (i)  :  indicator for refine criterium
! *                                                                     * &
! ***********************************************************************
! * &
! *   Local variables
! * &
      integer kwad,i,i1,i2,i3,i4,i5,i6,inew,iref
      double precision x1,y1,x2,y2,x3,y3,x4,y4,x5,y5,x6,y6,r1,r2,r3
      double precision cri1,cri2,cri3
! * &
! ***********************************************************************
! * &
! *   Set value kwad for quadratic case
! * &
      kwad = 2
! * &
! *   Check each side of each element , and place coordinates of
! *   new points in array coorh
! * &
      if ( meshtp.eq.4 ) then
! * &
! *        npelm = 6 (quadratic triangle)
! * &
           do 100 i = 1 , nelem
! * &
               i1 = kelem(6*i-5)
               i2 = kelem(6*i-4)
               i3 = kelem(6*i-3)
               i4 = kelem(6*i-2)
               i5 = kelem(6*i-1)
               i6 = kelem(6*i  )
! * &
               x1 = coor(2*i1-1)
               y1 = coor(2*i1  )
! * &
               x2 = coor(2*i2-1)
               y2 = coor(2*i2  )
! * &
               x3 = coor(2*i3-1)
               y3 = coor(2*i3  )
! * &
               x4 = coor(2*i4-1)
               y4 = coor(2*i4  )
! * &
               x5 = coor(2*i5-1)
               y5 = coor(2*i5  )
! * &
               x6 = coor(2*i6-1)
               y6 = coor(2*i6  )
! * &
               r1 = sqrt( (x3-x1)*(x3-x1)+(y3-y1)*(y3-y1) )
               r2 = sqrt( (x5-x3)*(x5-x3)+(y5-y3)*(y5-y3) )
               r3 = sqrt( (x1-x5)*(x1-x5)+(y1-y5)*(y1-y5) )
! * &
               cri1 = ( crit(i1) + crit(i3) ) / 2.d0
               cri2 = ( crit(i3) + crit(i5) ) / 2.d0
               cri3 = ( crit(i5) + crit(i1) ) / 2.d0
! * &
               iref =  0
! * &
               if ( irelat.eq.1 .and. cri1.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r1.gt.cri1 ) iref = 1
! * &
               if ( iref.eq.1 ) then
! * &
                    inew = 0
! * &
                    call msh431(i1,i3,base1,base2,help,new,inew,kwad)
! * &
                    if ( inew.eq.0 ) then
! * &
! *                      Place new points
! * &
                         call msh435(i1,i2,i3,new,base1,base2,help, &
                                     npoint,x1,x2,x3,y1,y2,y3,coorh, &
                                     crit)
                         new = new + 2
! * &
                    end if
               end if
! * &
               iref =  0
! * &
               if ( irelat.eq.1 .and. cri2.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r2.gt.cri2 ) iref = 1
! * &
               if ( iref.eq.1 ) then
! * &
                    inew = 0
! * &
                    call msh431(i3,i5,base1,base2,help,new,inew,kwad)
! * &
                    if ( inew.eq.0 ) then
! * &
! *                      Place new points
! * &
                         call msh435(i3,i4,i5,new,base1,base2,help, &
                                     npoint,x3,x4,x5,y3,y4,y5,coorh, &
                                     crit)
                         new = new + 2
! * &
                    end if
               end if
! * &
               iref =  0
! * &
               if ( irelat.eq.1 .and. cri3.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r3.gt.cri3 ) iref = 1
! * &
               if ( iref.eq.1 ) then
! * &
                    inew = 0
! * &
                    call msh431(i5,i1,base1,base2,help,new,inew,kwad)
! * &
                    if ( inew.eq.0 ) then
! * &
! *                      Place new points
! * &
                         call msh435(i5,i6,i1,new,base1,base2,help, &
                                     npoint,x5,x6,x1,y5,y6,y1,coorh, &
                                     crit)
                         new = new + 2
! * &
                    end if
               end if
! * &
100        continue
! * &
      else if ( meshtp.eq.2 ) then
! * &
! *        npelm = 3 (quadratic line element)
! * &
           do 200 i = 1 , nelem
! * &
               i1 = kelem(3*i-2)
               i2 = kelem(3*i-1)
               i3 = kelem(3*i  )
! * &
               x1 = coor(2*i1-1)
               y1 = coor(2*i1  )
! * &
               x2 = coor(2*i2-1)
               y2 = coor(2*i2  )
! * &
               x3 = coor(2*i3-1)
               y3 = coor(2*i3  )
! * &
               r1 = sqrt( (x3-x1)*(x3-x1)+(y3-y1)*(y3-y1) )
! * &
               cri1 = ( crit(i1) + crit(i3) ) / 2.d0
! * &
               iref =  0
! * &
               if ( irelat.eq.1 .and. cri1.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r1.gt.cri1 ) iref = 1
! * &
               if ( iref.eq.1 ) then
! * &
                    inew = 0
                    call msh431(i1,i3,base1,base2,help,new,inew,kwad)
! * &
                    if ( inew.eq.0 ) then
! * &
! *                      Place new points
! * &
                         call msh435(i1,i2,i3,new,base1,base2,help, &
                                     npoint,x1,x2,x3,y1,y2,y3,coorh, &
                                     crit)
                         new = new + 2
! * &
                    end if
! * &
               end if
! * &
200        continue
! * &
      else
! * &
          call ertrap('REFLOK  (msh434)',477,2,meshtp,0,0)
! * &
      end if
! * &
      end
