      subroutine msh436(kelem,nelem,coorh,npoint,new,nextra,crit, &
                        help,base1,base2,meshtp,kelemh,nelhel,irelat)
! * &
! ***********************************************************************
! *                                                                     * &
! *   Subroutine to compute whether elementsides of an element are too  * &
! *   long with respect to a given criterion in array CRIT.             * &
! *   If so then extra nodal points are placed inside the element.      * &
! *   New coordinates are placed in COORH and new elements in KELEMH.   * &
! *   Only quadratic elements are considered (meshtp = 2 , 4 )          * &
! *                                                                     * &
! ***********************************************************************
!                                                                       * &
!                                                                       * &
!         programmer   niek praagman                                    * &
!                                                                       * &
!         version 2.1    date  05-07-1991  crit 1.5 replaced by 0.5d0 )  * &
!         version 2.0    date  10-02-1990                                 * &
!         version 1.0    date  06-12-1989                                 * &
!                                                                       * &
!                                                                       * &
!                                                                       * &
!   copyright (c) 1989-1990  "ingenieursbureau sepra"                   * &
!   permission to copy or distribute this software or documentation     * &
!   in hard or soft copy granted only by written license obtained       * &
!   from "ingenieursbureau sepra".                                      * &
!   all rights reserved. no part of this publication may be reproduced, * &
!   stored in a retrieval system (e.g. , in memory, disk or core)       * &
!   or be transmitted by any means, electronic, mechanical, photocopy,  * &
!   recording, or otherwise, without written permission from the        * &
!   publisher.                                                          * &
!                                                                       * &
! ***********************************************************************
! * &
! *   GLOBAL parameters
! * &
      double precision coorh(*),crit(*),help(*),base1,base2
      integer kelem(*),nelem,npoint,new,nextra,meshtp
      integer kelemh(*),nelhel,irelat
! *                                                                     * &
! *   Meaning of parameters :                                           * &
! *                                                                     * &
! *   kelem   (i)  :  element array                                     * &
! *   nelem   (i)  :  number of elements                                * &
! *   coorh   (o)  :  new coordinate array                              * &
! *   npoint  (i)  :  old number of nodes                               * &
! *   new     (i)  :  number of new created points on the lines         * &
! *   nextra  (o)  :  number of new created points in the elements      * &
! *   crit    (i)  :  criterion array specified by user                 * &
! *   help         :  helparray for points                              * &
! *   base1   (i)  :  basisvalue for new points                         * &
! *   base2   (i)  :  basisvalue for new points                         * &
! *   meshtp  (i)  :  Sepran meshtype of elements considered            * &
! *   irelat  (i)  :  indicator for refinement criterium
! *                                                                     * &
! ***********************************************************************
! * &
! *   Local parameters
! * &
      integer kwad,i,i1,i2,i3,i4,i5,i6,ipo,ipo1,ipo2,ipo3,inew
      integer j1,j2,j3,j4,j5,j6,j7,j8,j9,j10,j11,j12,j13,j14,j15, iref
      double precision x1,y1,x3,y3,x5,y5,r1,r2,r3
      double precision cri1,cri2,cri3
! * &
! ***********************************************************************
! * &
      kwad = 2
! * &
! *   Check each side of each element , and place coordinates of
! *   new points in array coorh
! * &
      if ( meshtp.eq.4 ) then
! * &
! *        npelm = 6 (quadratic triangle)
! * &
           do 100 i = 1 , nelem
! * &
               i1 = kelem(6*i-5)
               i2 = kelem(6*i-4)
               i3 = kelem(6*i-3)
               i4 = kelem(6*i-2)
               i5 = kelem(6*i-1)
               i6 = kelem(6*i  )
! * &
               x1 = coorh(2*i1-1)
               y1 = coorh(2*i1  )
! * &
               x3 = coorh(2*i3-1)
               y3 = coorh(2*i3  )
! * &
               x5 = coorh(2*i5-1)
               y5 = coorh(2*i5  )
! * &
               r1 = sqrt( (x3-x1)*(x3-x1)+(y3-y1)*(y3-y1) )
               r2 = sqrt( (x5-x3)*(x5-x3)+(y5-y3)*(y5-y3) )
               r3 = sqrt( (x1-x5)*(x1-x5)+(y1-y5)*(y1-y5) )
! * &
               cri1 = ( crit(i1) + crit(i3) ) / 2.d0
               cri2 = ( crit(i3) + crit(i5) ) / 2.d0
               cri3 = ( crit(i5) + crit(i1) ) / 2.d0
! * &
               ipo  =  0
               ipo1 =  0
               ipo2 =  0
               ipo3 =  0
! * &
               iref = 0
! * &
               if ( irelat.eq.1 .and. cri1.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r1.gt.cri1 ) iref = 1
! * &
               if ( iref.eq.1 ) then
                    ipo  = ipo + 1
                    ipo1 = 1
               end if
! * &
               iref = 0
! * &
               if ( irelat.eq.1 .and. cri2.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r2.gt.cri2 ) iref = 1
! * &
               if ( iref.eq.1 ) then
                    ipo  = ipo + 1
                    ipo2 = 1
               end if
! * &
               iref = 0
! * &
               if ( irelat.eq.1 .and. cri3.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r3.gt.cri3 ) iref = 1
! * &
               if ( iref.eq.1 ) then
                    ipo  = ipo + 1
                    ipo3 = 1
               end if
! * &
               if ( ipo.eq.0 ) then
! * &
                    nelhel             = nelhel + 1
                    call msh406(kelemh,nelhel,i1,i2,i3,i4,i5,i6)
! * &
               else if ( ipo.eq.1 ) then
! * &
! *                 Only one side too long
! * &
                    if ( ipo1.eq.1 ) then
                         j1 = i1
                         j2 = i2
                         j3 = i3
                         j4 = i4
                         j5 = i5
                         j6 = i6
! * &
                    else if ( ipo2.eq.1 ) then
                         j1 = i3
                         j2 = i4
                         j3 = i5
                         j4 = i6
                         j5 = i1
                         j6 = i2
! * &
                    else if ( ipo3.eq.1 ) then
                         j1 = i5
                         j2 = i6
                         j3 = i1
                         j4 = i2
                         j5 = i3
                         j6 = i4
                    end if
! * &
! *                 Determine extra node numbers
! * &
                    call msh431(j1,j3,base1,base2,help,new,inew,kwad)
! * &
                    if ( j1.lt.j3 ) then
                        j7 = npoint + inew
                        j8 = j7 + 1
                    else
                        j8 = npoint + inew
                        j7 = j8 + 1
                    end if
! * &
                    nextra = nextra + 1
                    j9     = npoint + new + nextra
! * &
! *                 Fill coorh for point j9
! * &
                    coorh(2*j9-1)=(coorh(2*j2-1)+coorh(2*j5-1))/2d0
                    coorh(2*j9  )=(coorh(2*j2  )+coorh(2*j5  ))/2d0
! * &
! *                 Fill crit for point j9
! * &
                    crit (j9) = (crit(j2)+crit(j5))/2d0
! * &
! *                 Fill new elements
! * &
                    nelhel = nelhel + 1
                    call msh406(kelemh,nelhel,j1,j7,j2,j9,j5,j6)
                    nelhel = nelhel + 1
                    call msh406(kelemh,nelhel,j2,j8,j3,j4,j5,j9)
! * &
               else if ( ipo.eq.2 ) then
! * &
! *                 Two sides too long
! * &
                    if ( (ipo1 + ipo2).eq.2 ) then
! * &
                          j1 = i1
                          j2 = i2
                          j3 = i3
                          j4 = i4
                          j5 = i5
                          j6 = i6
! * &
                    else if ( (ipo2 + ipo3).eq.2 ) then
! * &
                          j1 = i3
                          j2 = i4
                          j3 = i5
                          j4 = i6
                          j5 = i1
                          j6 = i2
! * &
                    else if ( (ipo3 + ipo1).eq.2 ) then
! * &
                          j1 = i5
                          j2 = i6
                          j3 = i1
                          j4 = i2
                          j5 = i3
                          j6 = i4
! * &
                    end if
! * &
! *                 Determine extra node numbers
! * &
                    call msh431(j1,j3,base1,base2,help,new,inew,kwad)
! * &
                    if ( j1.lt.j3 ) then
                         j7  = npoint + inew
                         j8  = j7 + 1
                    else
                         j8  = npoint + inew
                         j7  = j8 + 1
                    end if
! * &
                    call msh431(j3,j5,base1,base2,help,new,inew,kwad)
! * &
                    if ( j3.lt.j5 ) then
                         j9  = npoint + inew
                         j10 = j9 + 1
                    else
                         j10 = npoint + inew
                         j9  = j10 + 1
                    end if
! * &
                    nextra = nextra + 1
                    j11    = npoint + new + nextra
! * &
                    nextra = nextra + 1
                    j12    = npoint + new + nextra
! * &
! *                 Place coordinates in coorh for j11 and j12
! * &
                    coorh(2*j11-1)=(coorh(2*j2-1)+coorh(2*j4-1))/2d0
                    coorh(2*j11  )=(coorh(2*j2  )+coorh(2*j4  ))/2d0
! * &
                    crit(j11) = (crit(j2)+crit(j4))/2d0
! * &
                    coorh(2*j12-1)=(coorh(2*j2-1)+coorh(2*j5-1))/2d0
                    coorh(2*j12  )=(coorh(2*j2  )+coorh(2*j5  ))/2d0
! * &
                    crit(j12) = (crit(j2)+crit(j5))/2d0
! * &
! *                 Fill new elements
! * &
                    nelhel = nelhel + 1
                    call msh406(kelemh,nelhel,j1,j7,j2,j12,j5,j6)
                    nelhel = nelhel + 1
                    call msh406(kelemh,nelhel,j2,j8,j3,j9,j4,j11)
                    nelhel = nelhel + 1
                    call msh406(kelemh,nelhel,j2,j11,j4,j10,j5,j12)
! * &
               else if ( ipo.eq.3 ) then
! * &
! *                 Determine the extra node numbers
! * &
                    call msh431(i1,i3,base1,base2,help,new,inew,kwad)
! * &
                    if ( i1.lt.i3 ) then
                         j7  = npoint + inew
                         j8  = j7 + 1
                    else
                         j8  = npoint + inew
                         j7  = j8 + 1
                    end if
! * &
                    call msh431(i3,i5,base1,base2,help,new,inew,kwad)
! * &
                    if ( i3.lt.i5 ) then
                         j9  = npoint + inew
                         j10 = j9 + 1
                    else
                         j10 = npoint + inew
                         j9  = j10 + 1
                    end if
! * &
                    call msh431(i5,i1,base1,base2,help,new,inew,kwad)
! * &
                    if ( i5.lt.i1 ) then
                         j11 = npoint + inew
                         j12 = j11 + 1
                    else
                         j12 = npoint + inew
                         j11 = j12 + 1
                    end if
! * &
                    j13 = nextra + 1 + new + npoint
                    j14 = nextra + 2 + new + npoint
                    j15 = nextra + 3 + new + npoint
! * &
                    nextra = nextra + 3
! * &
! *                 Place coordinates for internal points
! * &
                    coorh(2*j13-1)=(coorh(2*i2-1)+coorh(2*i6-1))/2d0
                    coorh(2*j13  )=(coorh(2*i2  )+coorh(2*i6  ))/2d0
! * &
                    crit(j13) = (crit(i2)+crit(i6))/2d0
! * &
                    coorh(2*j14-1)=(coorh(2*i2-1)+coorh(2*i4-1))/2d0
                    coorh(2*j14  )=(coorh(2*i2  )+coorh(2*i4  ))/2d0
! * &
                    crit(j14) = (crit(i2)+crit(i4))/2d0
! * &
                    coorh(2*j15-1)=(coorh(2*i4-1)+coorh(2*i6-1))/2d0
                    coorh(2*j15  )=(coorh(2*i4  )+coorh(2*i6  ))/2d0
! * &
                    crit(j15) = (crit(i4)+crit(i6))/2d0
! * &
! *                 Fill new elements
! * &
                    nelhel = nelhel + 1
                    call msh406(kelemh,nelhel,i1,j7,i2,j13,i6,j12)
                    nelhel = nelhel + 1
                    call msh406(kelemh,nelhel,i2,j8,i3,j9,i4,j14)
                    nelhel = nelhel + 1
                    call msh406(kelemh,nelhel,i2,j14,i4,j15,i6,j13)
                    nelhel = nelhel + 1
                    call msh406(kelemh,nelhel,i6,j15,i4,j10,i5,j11)
! * &
               end if
! * &
100        continue
! * &
      else if ( meshtp.eq.2 ) then
! * &
! *        npelm = 3 (quadratic line element)
! * &
           do 200 i = 1 , nelem
! * &
               i1 = kelem(3*i-2)
               i2 = kelem(3*i-1)
               i3 = kelem(3*i  )
! * &
               x1 = coorh(2*i1-1)
               y1 = coorh(2*i1  )
! * &
               x3 = coorh(2*i3-1)
               y3 = coorh(2*i3  )
! * &
               r1 = sqrt( (x3-x1)*(x3-x1)+(y3-y1)*(y3-y1) )
! * &
               cri1 = ( crit(i1) + crit(i3) ) / 2.d0
! * &
               iref = 0
! * &
               if ( irelat.eq.1 .and. cri1.gt.0.5d0 ) iref = 1
               if ( irelat.eq.2 .and.   r1.gt.cri1 ) iref = 2
! * &
               if ( iref.eq.1 ) then
                    inew = 0
                    call msh431(i1,i3,base1,base2,help,new,inew,kwad)
! * &
                    if ( i1.lt.i3 ) then
                         j4 = npoint + inew
                         j5 = j4 + 1
                    else
                         j5 = npoint + inew
                         j4 = j5 + 1
                    end if
! * &
                    nelhel = nelhel + 2
! * &
                    kelemh( 3*nelhel - 5 ) = i1
                    kelemh( 3*nelhel - 4 ) = j4
                    kelemh( 3*nelhel - 3 ) = i2
                    kelemh( 3*nelhel - 2 ) = i2
                    kelemh( 3*nelhel - 1 ) = j5
                    kelemh( 3*nelhel     ) = i3
! * &
               else
! * &
                    nelhel = nelhel + 1
                    kelemh( 3*nelhel - 2 ) = i1
                    kelemh( 3*nelhel - 1 ) = i2
                    kelemh( 3*nelhel     ) = i3
! * &
               end if
! * &
200        continue
! * &
      else
! * &
          call ertrap('REFLOC  (msh436)',477,2,meshtp,0,0)
! * &
      end if
! * &
      end
