      subroutine msh444 ( kmeshc , nelem , istart, ibrp, ibrpnt, &
                          kelemh, ishape, npoint )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 2.1    date   18-04-1997 Add eropen/erclos
!        version 2.0    date   14-02-1995 Complete redesign
!        version 1.0    date   27-03-1992
!
!
!
!   copyright (c) 1992-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Run through all triangles (ishape=3) or all tetrahedrons (ishape=11)
!     and place nodes in ibrpnt and istart
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     mesh
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer kmeshc(*), nelem, istart(*), ibrp, ibrpnt(*), kelemh(*), &
              ishape, npoint

!     ibrp     i    number of neighbour positions per point
!     ibrpnt  i/o   array to store the neighbours
!     ishape  i   type of elements to consider
!     istart  i/o   array to indicate starting positions in ibrpnt
!     kelemh  i   array to store extra points
!     kmeshc  i   element array
!     nelem   i   number of elements
!     npoint  i   number of nodes in starting mesh
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4

!    i       loop variable
!    i1      node number
!    i2      node number
!    i3      node number
!    i4      node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   Resets old name of previous subroutine of higher level
!     EROPEN   Produces concatenated name of this subroutine
!     MSH459  Make reservation for barycenter of triangle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all elements
!     Check each face of each element , and place (if necessary)
!     numbers in arrays istart and ibrpnt
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh444' )
      if ( ishape.eq.3 ) then

!     --- Triangles

         do 300 i = 1 , nelem

             i1 = kmeshc( 3*i - 2 )
             i2 = kmeshc( 3*i - 1 )
             i3 = kmeshc( 3*i     )

             call msh459( ibrp, ibrpnt, istart, kelemh, i1, i2, i3, &
                          npoint )

300      continue

      else if ( ishape.eq.11 ) then

!     --- Tetrahedrons

         do 1100 i = 1 , nelem

!        --- Store each triangular face

             i1 = kmeshc(4*i-3)
             i2 = kmeshc(4*i-2)
             i3 = kmeshc(4*i-1)
             i4 = kmeshc(4*i  )

             call msh459( ibrp, ibrpnt, istart, kelemh, i1, i2, i3, &
                          npoint )

             call msh459( ibrp, ibrpnt, istart, kelemh, i1, i2, i4, &
                          npoint )

             call msh459( ibrp, ibrpnt, istart, kelemh, i1, i3, i4, &
                          npoint )

             call msh459( ibrp, ibrpnt, istart, kelemh, i2, i3, i4, &
                          npoint )

1100      continue

      end if
      call erclos ( 'msh444' )

      end
