      subroutine mshadd2dnod  ( trianglenode, trianglenodenw, &
                                npointnew, ielemnew, &
                                coornew, iwork, iedges, nmeshc, &
                                icrossobst, rcrossobst, ielem, &
                                ikelmcnew, istarticross, inpelm, &
                                skipnod,eps, fluidnd, &
                                inelem1, shifted )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  2.2  date 19-05-2005 New calls to subroutines
!        version  2.1  date 05-04-2005 Fortran 90
!        version  2.0  date 23-06-2004 Extra parameters
!        version  1.1  date 07-04-2004 Extra parameters
!        version  1.0  date 26-09-2002
!
!   copyright (c) 2002-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a triangle if necessary into two, three or four subtriangles and
!     indicate which of these triangles is in or out the obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, trianglenode(inpelm), trianglenodenw(inpelm,4), &
              npointnew, ielemnew, iwork(*), &
              iedges(3,*), nmeshc(*), shifted(*), &
              icrossobst(*), ielem, ikelmcnew, istarticross, &
              skipnod(*), fluidnd(3,*), inelem1
      double precision coornew(2,*), rcrossobst(*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     eps            i    Local accuracy
!     fluidnd        i    Array containing the fluidnodes that lie on the
!                         obstacle
!     icrossobst     i    array containing information about the cross section
!                         of obstacles with edges
!                         The storage of icrossobst is as follows
!                         Pos. 1 to numobst (number of obstacles):
!                               icrossobst(iobst) = istartobst (starting address
!                               of obstacle iobst in array
!                         Per obstacle from position istartobst:
!                         Number of cross sections, followed by 2 positions for
!                         each cross section consisting of the edge sequence
!                         number and a reference to rcrossobst
!                         This part is followed by the number of elements
!                         that is intersected by the obstacle followed by
!                         for each element: element number, number of
!                         intersections and sequence numbers of intersections
!     iedges        i/o   Work array of size nedges
!                         If iedges(i) = 0, no point has been created in edge i
!                         If iedges(i)>0, a point has been created in edge i, &
!                         with sequence number iedges(i)
!     ielem          i    Element number
!     ielemnew      i/o   Sequence number of element in new mesh
!     ikelmcnew     i/o   Starting address of next element in kmeshcnew
!     inelem1        i    ?
!     inpelm         i    number of points per element
!     istarticross   i    Starting address of obstacle in array icrossobst
!     iwork          i    integer work array of size nelem
!                         iwork(ielem) contains the starting address of element
!                         ielem in array icrossobst
!                         If 0, the element is not present in icrossobst
!     nmeshc         i    nmesh part c
!     npointnew     i/o   Sequence number of last generated node
!     rcrossobst     i    Real array corresponding to icrossobst
!                         For each cross section a real factor f is stored
!                         defining the intersection of the edge with the
!                         obstacle in the following way
!                         x_intersect = x1 + f * (x2-x1)
!                         where x1 and x2 are the first and last point of the
!                         edge respectively
!     shifted        i    Array of npoint long in which the nodes are marked
!                         which are shifted onto the obstacle
!     skipnod        i    Array of npoint long in which the nodes are marked
!                         which are not allowed to be moved
!     trianglenode   i    Array of nodal point numbers of the corners of
!                         the original triangle.
!     trianglenodenw o    Array of nodal point numbers of the corners of
!                         the new subtriangles
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer edgenode(5,3), istart, icross, nodebegin, nodeend, &
              ipstart, i, iedge, nextranodes, ireledge, &
              npointnewsave, j, ifacenode(4)
      double precision eps, fact

!     edgenode       Array with all nodes on the crossed edge
!                    edgenode(1,1) first node on crossed edge
!                    edgenode(2,1) last node on crossed edge
!                    edgenode(3,1) new node on crossed edge
!     fact           interpolation factor
!     i              Counting variable
!     icross         position of iedge in array icrossobst
!     iedge          edge sequence number
!     ifacenode      ?
!     ipstart        Starting address in array
!     ireledge       relative edge sequence number
!     istart         First position used at a part
!     j              Counting variable
!     nextranodes    Number of extra nodes in element
!     nodebegin      node at the begin of a crossed edge
!     nodeend        node at the end of a crossed edge
!     npointnewsave  ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INSTOP         Stop the program
!     MSHOBSTMS2D04  Subdivide triangle into two triangles
!     MSHOBSTMS2D04Q Subdivide triangle into two triangles
!                    Quadratic 7 point elements
!     MSHOBSTMS2D05  Subdivide triangle into three triangles
!     MSHOBSTMS2D05Q Subdivide triangle into three triangles
!                    Quadratic 7 point elements
!     MSHOBSTMS2D06  Subdivide triangle into four triangles
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     eps := 0.2
!     nextranodes = 0
!     do i = 1,number of intersections
!        iedge := edge which is intersected
!        fact := interpolation factor
!        if ( cross section with edge close to end point of edge ) then
!           mark corresponding node as on boundary
!        else ( i-th edge is intersected )
!           if (skipnode==1) then
!             node is already shifted in shift2dnod
!           else
!             nextranodes := nextranodes+1
!             edgenode(1,i) := first point of edge
!             edgenode(2,i) := last point on edge
!             if( first time edge is intersected)
!                mark edge as intersected
!                introduce the intersection node
!                calculate its coordinates
!                mark it as boundary node
!                store interpolation data
!             end if
!             edgenode(3,i) := intersection node
!          end if
!        end if
!     end do
!     subdivide triangle into nextranode+1 subtriangles
!     mark each subtriangle as inside or outside
!     update pointers ikelmcnew and ielemnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshadd2dnod' )

!     --- nextranodes := 0

      nextranodes = 0

      istart = iwork(ielem)

!     --- do i = 1,nintersect

      do i = 1,icrossobst(istart)

         ireledge = icrossobst(istart+i)
         icross  = istarticross+1+2*(ireledge-1)

!        --- iedge := edge which is intersected

         iedge = icrossobst(icross)
         fact  = rcrossobst(icrossobst(icross+1))
         ipstart = nmeshc(iedge)

         if ( fact<=1d0-eps .and. fact>=eps ) then
            nodebegin = nmeshc(ipstart+1)
            if (inpelm==3) then
            nodeend = nmeshc(ipstart+2)
            else
            nodeend = nmeshc(ipstart+3)
            end if

            if (skipnod(nodebegin)/=3 .and. &
                skipnod(nodeend)/=3) then

!           --- i-th edge is intersected
!               increment nextranodes
!               edgenode(1,nextranodes) := first point of edge
!               edgenode(2,nextranodes) := last point on edge

               nextranodes = nextranodes+1
               edgenode(1,nextranodes) = nodebegin
               edgenode(2,nextranodes) = nodeend

               if ( iedges(1,iedge)==0 ) then

!              --- -- New point
!                     npointnew := npointnew+1
!                     c Fill coordinates of cross section in coornew(npointnew)
!                     c mark node as boundary node

                  if (inpelm==3) then
                     npointnew = npointnew+1
                     iedges(1,iedge) = npointnew
                  else if (inpelm==7) then
                     iedges(2,iedge) = npointnew+1
                     iedges(3,iedge) = npointnew+2
                     npointnew=npointnew+2
                     npointnewsave = npointnew
                     npointnew = nmeshc(ipstart+2)
                     iedges(1,iedge) = npointnew
                  end if
                  coornew(1,npointnew) = (1d0-fact)* &
                                      coornew(1,edgenode(1,nextranodes)) &
                                     +fact* &
                                      coornew(1,edgenode(2,nextranodes))
                  coornew(2,npointnew) = (1d0-fact)* &
                                      coornew(2,edgenode(1,nextranodes)) &
                                     +fact* &
                                      coornew(2,edgenode(2,nextranodes))
                 npointnew=npointnewsave

               end if

!              --- edgenode(3,nextranodes) := cross section node

               if (inpelm==3) then
                  edgenode(3,nextranodes) = iedges(1,iedge)
               else if (inpelm==7) then
                  edgenode(3,nextranodes) = iedges(2,iedge)
                  edgenode(4,nextranodes) = iedges(1,iedge)
                  edgenode(5,nextranodes) = iedges(3,iedge)
               end if
            end if
         end if
      end do

      if (inpelm==7) then

         if (nextranodes==1) then

            do i=1,2
                npointnew=npointnew+1
                ifacenode(i)=npointnew
            end do

         else if(nextranodes==2) then

            do i=1,4
                npointnew=npointnew+1
                ifacenode(i)=npointnew
            end do

         end if

      end if

      if ( nextranodes==0 ) then

!     --- nextranodes=0
!         subdivide triangle into 1 triangle and mark it as inside or outside

         do i = 1, inpelm
            trianglenodenw(i,1) = trianglenode(i)
         end do

      else if ( nextranodes==1 ) then

!     --- nextranodes=1
!         subdivide triangle into 2 triangles and mark them

         if (inpelm==3) then

            call mshobstms2d04 ( trianglenode, trianglenodenw, &
                                 edgenode )

         else if(inpelm==7) then

            call mshobstms2d04q ( trianglenode, trianglenodenw, &
                                  edgenode, ifacenode, coornew )

            if (shifted(trianglenodenw(1,1))==1) then
               inelem1=inelem1+1
               do i=1,3
                  fluidnd(i,inelem1)=trianglenodenw(i,2)
               end do
            end if

         end if

      else if ( nextranodes==2 ) then

!     --- nextranodes=2
!         subdivide triangle into 3 triangles and mark them

         if(inpelm==3) then

           call mshobstms2d05 ( trianglenode, trianglenodenw, edgenode, &
                                coornew, 5 )

         else if(inpelm==7) then

            call mshobstms2d05q (trianglenode, trianglenodenw, &
                                 edgenode, coornew, ifacenode)

            inelem1=inelem1+1
            do i=1,3
               fluidnd(i,inelem1)=trianglenodenw(2+i,1)
            end do

         end if

      else

!     --- nextranodes=3
!         subdivide triangle into 4 triangles and mark them

         write(*,*)'three intersections in one triangle'
         write(*,*)'this is not yet possible'
         call instop
         call mshobstms2d06 ( trianglenode, trianglenodenw, edgenode, &
                              coornew, 5 )
      end if

      do j=1,inpelm
         do i=1, nextranodes+1
            imap(trianglenodenw(j,i),1)=-1
         end do
      end do

      ikelmcnew = ikelmcnew+(nextranodes+1)*inpelm
      ielemnew = ielemnew+nextranodes

      call erclos ( 'mshadd2dnod' )
      end
