      subroutine mshanalconnect ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 26-11-2013 Extra test curves around volume
!        version  1.1  date 27-11-2012 Long integers
!        version  1.0  date 13-11-2002
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Analyse all connections between entities of largest dimension, &
!     i.e. volumes or surfaces or curves
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     volume
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)

!     iinput         i    integer input array, standard for subroutine MESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenvolusp
      integer, allocatable :: volusp(:)

!     lenvolusp      Length of array volusp
!     volusp         array containing the user points corresponding to
!                    the volumes in the following way:
!                    volusp(1)=nvolms+1
!                    number of user points in volume i:
!                    volusp(i+1)-volusp(i)
!                    user points in volume i are stored in volusp from
!                    position volusp(i)+1 to volusp(i+1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHANALVOL     Check if all volumes are coupled because they have
!                    common user points
!     MSHANALVOLCUR  Check if surface surrounding general is closed by checking
!                    the number of curves in the surfaces
!     MSHUSPVOLMS    Find all user points connected to the volumes and store in
!                    array volusp
!     PR0012         Print accumulated array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( there are volumes ) then
!        Find all surfaces, curves and user points connected to each volume
!        Check if volumes are connected
!     else if ( there are surfaces ) then
!        Find all curves and user points connected to each surface
!        Check if surfaces are connected
!     else ( there are curves only ) then
!        Find all user points connected to each curve
!        Check if curves are connected
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshanalconnect' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshanalconnect'

      end if
      if ( nwarn>mxwarn ) go to 1000

      if ( nvolms>0 ) then

!     --- there are volumes
!         Find all user points connected to each volume
!         Create space for array volusp

         lenvolusp = nvolms*nuspnt+nvolms+1
         allocate ( volusp(lenvolusp), stat = error )
         if ( error/=0 ) call eralloc ( error, lenvolusp, 'volusp' )
         if ( ierror/=0 ) go to 1000

         call mshuspvolms ( volusp, lenvolusp )
         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'User points corresponding to volumes'
            call pr0012 ( nvolms, volusp, volusp )

         end if

!        --- Check if volumes, surfaces and curves are connected
!            At this moment only connection between volumes is tested

         call mshanalvol ( volusp, nvolms, nuspnt )

!        --- Check if the surface surrounding general is closed

         call mshanalvolcur ( iinput )

!        --- else if ( there are surfaces ) then
!               Find all curves and user points connected to each surface
!               Check if surfaces are connected
!            else ( there are curves only ) then
!               Find all user points connected to each curve
!               Check if curves are connected

         deallocate ( volusp, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'volusp' )

      end if  ! nvolms>0

1000  if ( debug ) write(irefwr,*) 'End mshanalconnect'
      call erclos ( 'mshanalconnect' )

      end
