      subroutine mshchplane ( nnodes, nodes, coor, isurnr, normal, &
                              planefound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 11-02-2005 Extra parameter planefound
!        version  1.0  date 24-03-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if all nodes in surface are lying in a plane
!
! **********************************************************************
!
!                       KEYWORDS
!
!     normal_vector
!     plane
!     surface
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(*), isurnr
      double precision coor(3,*), normal(3)
      logical planefound

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     isurnr         i    Surface sequence number
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers in the plane
!     normal         i    Contains the normal on the plane
!     planefound    i/o   Indicates if an error message should be given or not
!                         If at input planefound = false an error message is
!                         given when the coordinates are not in a plane
!                         Otherwise (planefound=true at input) not an error
!                         message is given but the parameter planefound at
!                         output is true if the coordinates are in a plane and
!                         false if not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, i
      double precision sum,  d

!     d              The surface is defined as n_x x + n_y y + n_z z = d
!     i              Counting variable
!     inode          node in edge
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2639   Surface is not a plane
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The general formula for a plane is defined by
!        (x, y, z) n = d
!
!     So all we have to do is to check if d is more or less constant
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshchplane' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr, *) 'Debug information from mshchplane'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- check if d is more or less constant

      sum = 0d0
      do i = 1, nnodes
         inode = nodes(i)
         sum = sum+coor(1,inode)*normal(1)+coor(2,inode)*normal(2)+ &
               coor(3,inode)*normal(3)
      end do  ! i = 1, nnodes
      d = sum/nnodes

      do i = 1, nnodes

         inode = nodes(i)
         sum = d-coor(1,inode)*normal(1)-coor(2,inode)*normal(2)- &
                 coor(3,inode)*normal(3)
         if ( abs(sum).gt.sqreps ) then

!        --- sum > threshold, hence not a plane

            if ( planefound ) then

!           --- planefound = true, return with planefound false

               planefound = .false.

            else

!           --- planefound = false, give error message

               call errint ( isurnr, 1 )
               call errsub ( 2639, 1, 0, 0 )

            end if  ! ( planefound )
            go to 1000

         end if  ! ( abs(sum).gt.sqr )

      end do  ! i = 1, nnodes

1000  call erclos ( 'mshchplane' )

      if ( debug ) then

!     --- Debug information

         write(irefwr, *) 'End mshchplane'

      end if

      end
