      subroutine mshconnectsurf ( ileft, iright, inpelm, ilefst, irigst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute first part for connection of surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     connect
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ileft, iright, inpelm, ilefst, irigst

!     ilefst         o    Starting point of "left" surface in KMESH part n
!     ileft          i    Left surface in connection element
!     inpelm         o    number of points per element in surface
!     iright         i    Right surface in connection element
!     irigst         o    Starting point of "right" surface in KMESH part n
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurfleft, isurfright, iprec, j
      logical copy

!     copy           If true one surface is a copy of the other one
!     iprec          Surface number of surface to be copied etc.
!     isurfleft      Type of surface of "left" surface
!     isurfright     Type of surface of "right" surface
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2435   Surfaces do not have the same topology
!    2436   Surfaces do not have the same topology
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshconnectsurf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshconnectsurf'
         write(irefwr,1) 'ileft, iright', ileft, iright
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpelm = kmeshn(ileft)
      inelem = kmeshn(ileft+nsurfs)

      if ( debug ) write(irefwr,1) 'inpelm, inelem', inpelm, inelem

      if ( inpelm/=kmeshn(iright) .or. &
           inelem/=kmeshn(iright+nsurfs) ) then

!     --- Surfaces have different topology

         call errint ( ileft, 1 )
         call errint ( iright, 2 )
         call errint ( inpelm, 3 )
         call errint ( kmeshn(iright), 4 )
         call errint ( inelem, 5 )
         call errint ( kmeshn(iright+nsurfs), 6 )
         call errsub ( 2435, 6, 0, 0 )
         go to 1000

      end if

!     --- Detect the type of surface of both surfaces

      isurfleft = kmeshs(nsurfs+1+(ileft-1)*8+1)
      isurfright = kmeshs(nsurfs+1+(iright-1)*8+1)
      if ( debug ) write(irefwr,1) 'isurfleft, isurfright', &
                                    isurfleft, isurfright
      copy = .true.
      if ( isurfleft==2 .or. isurfleft==3 .or. &
           isurfleft==18 .or. isurfleft==19 ) then

!     --- isurfleft is of type general, triangle, isopar or paver
!         other surface must be a copy

         copy = isurfright==8 .or. isurfright==9 .or. &
                isurfright==14 .or. isurfright==16
         if ( copy ) then

!        --- isurfright might be a copy of isurfleft

            iprec = kmeshs(nsurfs+1+(iright-1)*8+3)
            if ( debug ) write(irefwr,1) 'iprec', iprec
            if ( iprec/=ileft ) copy = .false.

         end if

      end if
      if ( .not. copy ) then

!     --- Surfaces may have a different topology

         call errint ( ileft, 1 )
         call errint ( iright, 2 )
         call errsub ( 2436, 2, 0, 0 )
         go to 1000

      end if

      copy = .true.
      if ( isurfright==2 .or. isurfright==3 .or. &
           isurfright==18 .or. isurfright==19 ) then

!     --- isurfleft is of type general, triangle, isopar or paver
!         other surface must be a copy

         copy = isurfleft==8 .or. isurfleft==9 .or. &
                isurfleft==14 .or. isurfleft==16
         if ( copy ) then

!        --- isurfleft might be a copy of isurfright

            iprec = kmeshs(nsurfs+1+(ileft-1)*8+3)
            if ( debug ) write(irefwr,1) 'iprec', iprec
            if ( iprec/=iright ) copy = .false.

         end if

      end if

      if ( .not. copy ) then

!     --- Surfaces may have a different topology

         call errint ( ileft, 1 )
         call errint ( iright, 2 )
         call errsub ( 2436, 2, 0, 0 )
         go to 1000

      end if

!     --- Compute starting addresses in array KMESHN

      ilefst = 2*nsurfs
      do j = 1, ileft-1
         ilefst = ilefst + kmeshn(j)*kmeshn(j+nsurfs)
      end do
      irigst = 2*nsurfs
      do j = 1, iright-1
         irigst = irigst + kmeshn(j)*kmeshn(j+nsurfs)
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshconnectsurf'

      end if  ! ( debug )
      call erclos ( 'mshconnectsurf' )

      end
