      subroutine mshconnectwork ( iwork, ileft, iexclu, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-01-2013 extra debug
!        version  1.0  date 18-06-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iwork for connection of points in surface elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     connect
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ileft, iexclu, iinput(*)

!     iexclu         i    Indicates which points are excluded
!                         See connection elements or periodical boundary
!                         conditions
!     iinput         i    integer input array containing the curves to be
!                         excluded
!     ileft          i    Left curve in special element
!     iwork          o    integer work array
!                         If IWORK(i) = 0, node i has not been found
!                         in the left surface yet
!                         If IWORK(i)>0, the node has been used and elements
!                         containing this node as starting point are skipped
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k, icrvn, istar3

!     icrvn          Curve number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH033         Fill array iwork for outer boundaries
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshconnectwork' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshconnectwork'
         write(irefwr,1) 'iexclu', iexclu
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iwork(1:npoint) = 0

      if ( iexclu==-1 ) then

!     --- iexclu = 1, the outer boundary must be skipped

         call msh033 ( kmeshm, kmeshs, ncurvs, nsurfs, &
                       ileft, iwork, npoint )

      else if ( iexclu>0 ) then

!     --- iexclu>0, skip given curves
!         Set iwork(ipoint) for curves

          istar3 = kmeshm(2)-1
          do j = 1, iexclu
             icrvn = iinput(j)
             do k = kmeshm(4+icrvn), kmeshm(5+icrvn)-1
                iwork(kmeshm(istar3+k)) = -1
             end do
         end do

      end if

1000  call erclos ( 'mshconnectwork' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshconnectwork'

      end if  ! ( debug )

      end
