      subroutine mshcoorcentr ( kmeshc, coor, nelem, ndim, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-05-2002 Complete revision
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put coordinates of extra centre point in quadratic tetrahedron
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     centre
!     coordinate
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, ndim, inpelm, kmeshc(inpelm,*)
      double precision coor(ndim,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     inpelm         i    Number of nodes in element (new one)
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     ndim           i    Dimension of the space.
!     nelem          i    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nod1, nod2, nod3, nod4, nodc, j

!     i              Counting variable
!     j              Counting variable
!     nod1           Node number of first vertex in tetrahedron
!     nod2           Node number of second  vertexin tetrahedron
!     nod3           Node number of third vertex in tetrahedron
!     nod4           Node number of fourth vertex in tetrahedron
!     nodc           Node number of centroid in tetrahedron
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all elements ielem do
!        Compute coordinates of centroid
!        Put coordinates into inpelm-th point of element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcoorcentr' )

!     --- For all elements ielem do

      do i = 1, nelem
         nod1 = kmeshc(1,i)
         nod2 = kmeshc(3,i)
         nod3 = kmeshc(5,i)
         nod4 = kmeshc(10,i)
         nodc = kmeshc(inpelm,i)

!        --- Compute coordinates of centroid
!            Put coordinates into inpelm-th point of element

         do j = 1, ndim
            coor(j,nodc) = 0.25d0*(coor(j,nod1)+coor(j,nod2)+ &
                                   coor(j,nod3)+coor(j,nod4))
         end do

      end do

      call erclos ( 'mshcoorcentr' )

      end
