      subroutine mshcr02 ( coorol, coornw, usol, anorm, npoint, indprf, &
                           kprobf, number, iwork, nunkp, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 29-12-2008 Fortran 90
!        version  2.0  date 05-11-2008 Remove kprobh
!        version  1.0  date 08-04-1996
!
!   copyright (c) 1996-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the co-ordinates of the new boundary in msh047 in the case
!   iadapt = 2 i.e. x_new is computed by a user function subroutine funccr
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, indprf, kprobf(*), number, iwork(*), nunkp
      double precision coorol(2,*), coornw(2,*), usol(*), anorm(2,*), &
                       work(*)

!     anorm      i    In this array the normal vector is stored
!                     anorm(j,i) contains the j th component of the normal
!                     vector in the i-th point
!     coornw     o    Array containing the new co-ordinates along the boundary
!     coorol     i    Array containing the old co-ordinates along the boundary
!     indprf     i    indication if kprob part f has been filled (>0) or not (0)
!     iwork      i    In this array the nodal point numbers of the nodes
!                     along the boundary are stored
!     kprobf     i    KPROB part f: contains the number of unknowns per point
!                     accumulated
!     npoint     i    Number of nodal points along the boundary
!     number     i    The number-th unknown in each nodal point of the array
!                     is considered.
!     nunkp      i    Maximal number of unknowns per point
!     usol       i    Solution vector
!     work            Work array of length nunkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inusol, inodp, inunkp

!     i          Counting variable
!     inodp      Nodal point number
!     inunkp     Number of degrees of freedom in first point
!     inusol     Position of unknown in array USOL
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     FUNCCR    User subroutine for changing the boundary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The co-ordinates along the boundary are updated according to the
!    formula:
!    x_new(i) = x_old(i) + u(i) * n(i)
!    For all points i along the boundary do
!       inodp := nodal point number
!       For all unknowns in inodp do
!          inusol := sequence number of unknown
!          u := usol(inusol)
!          store u in work
!       Adapat boundary by FUNCCR
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      inunkp = nunkp
      do i = 1, npoint
         inodp = iwork(i)
         if ( indprf==0 ) then
            inusol = (inodp-1)*inunkp
         else
            inusol = kprobf(inodp)
            inunkp = kprobf(inodp+1)-inusol
         end if
         inusol = inusol+number-1
         work(1:inunkp) = usol(inusol+1:inusol+inunkp)
         call funccr ( coorol, coornw, work, inodp, i, anorm, npoint )
      end do  ! i = 1, npoint
      end
