      subroutine mshcuradapend ( curves, iinput, ndim, rinput, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-09-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Change coordinates of end points of curve by the coordinates as
!     stored in array rinput
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     curve
!     free_boundary
!     free_surface
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, iinput(*), n
      double precision curves(ndim,*), rinput(ndim,*)

!     curves        i/o   array containing the coordinates of the curves
!                         At input the coordinates to be redistributed
!                         At output the new coordinates
!     iinput         i    integer input array corresponding to the mesh
!                         generation , see PG
!                         The first position must refer to the present curve
!     n              i    number of basis functions for each component
!     ndim           i    Dimension of the space.
!     rinput         i    Real input array corresponding to the mesh
!                         generation , see PG
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurve, nuserp, ibegin, iend, i

!     i              Counting variable
!     ibegin         First user point of curve
!     icurve         Type of curve
!     iend           Last user point of curve
!     nuserp         Number of user points along curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1383   Value of ICURVE is not yet implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find first and last user point number
!     Adapt curves
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcuradapend' )

      icurve = abs(iinput(1))

!     --- Find first and last user point number

      if ( icurve.eq.8 ) then

!     --- icurve = 8, parameter function

         ibegin = abs(iinput(5))
         iend = abs(iinput(6))

      else if ( icurve.eq.10 .or.icurve.eq.11 ) then

!     --- icurve = 10, 11 translate or rotate

         nuserp = iinput(4)
         ibegin = abs(iinput(6))
         iend = abs(iinput(6+nuserp-1))

      else

!     --- other value of icurve, not implemented

         call errint ( icurve, 1 )
         call errsub ( 1383, 1, 0, 0 )
         go to 1000

      end if

!     --- Adapt curves

      do i = 1, ndim
         curves(i,1) = rinput(i,ibegin)
         curves(i,n) = rinput(i,iend)
      end do

1000  call erclos ( 'mshcuradapend' )

      end
