      subroutine mshds1 ( bcord1, bcord2, imax, kbndbt, dz )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.1    date   22-09-1995 New definition of imax
!        version 1.0    date   12-11-1994
!
!
!
!   copyright (c) 1994-1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!   Compute the average distance between the surfaces described by bcord1
!   and bcord2 in the points 1 and imax at the boundary
! ********************************************************************
!
!                      KEYWORDS
!
!    distance
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer imax(3), kbndbt(*)
      double precision bcord1(3,*), bcord2(3,*), dz

!     bcord1  i    Co-ordinates of nodal points in bottom surface
!     bcord2  i    Co-ordinates of nodal points in top surface
!     dz      o    Distance between under and upper face
!     imax    i    In this array the indices with respect to boundary points
!                  in the surfaces are stored
!                  imax(1) stores the node number of the first point (usually 1)
!                  imax(2) stores the node number with respect to the xsi dir.
!                          The xsi direction is defined as P2-P1, where P2 is
!                          given by bcord(kbndbt(imax(2))
!                  imax(3) stores the node number with respect to the first eta
!                          dir.
!                          The eta direction is defined as P3-P1, where P3 is
!                          given by bcord(kbndbt(imax(3))
!     kbndbt  i    Array KBNDPT for bottom surface
!                  Contains the local nodal point numbers of the boundary of
!                  the bottom surface
! ********************************************************************
!
!                    COMMON BLOCKS
!
! ********************************************************************
!
!                   LOCAL PARAMETERS
!
      integer i, j

!     i       Counting variable
!     j       Counting variable
! ********************************************************************
!
!                   SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
! ======================================================================
!
      i = kbndbt(imax(2))
      j = kbndbt(imax(1))
      dz = 0.5d0 * ( sqrt ( (bcord1(1,j)-bcord2(1,j))**2 + &
                            (bcord1(2,j)-bcord2(2,j))**2 + &
                            (bcord1(3,j)-bcord2(3,j))**2 ) + &
                     sqrt ( (bcord1(1,i)-bcord2(1,i))**2 + &
                            (bcord1(2,i)-bcord2(2,i))**2 + &
                            (bcord1(3,i)-bcord2(3,i))**2 ) )
      end
