      subroutine mshds2 ( bcord1, bcord3, imax, kbndbt, dz, alpha )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 27-11-2012 Long integers
!        version  1.1  date 22-09-1995 New definition of imax
!        version  1.0  date 12-11-1994
!
!   copyright (c) 1994-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the factor alpha defining the average distance between cross section
!   and under surface in the points 1 and imax at the boundary, &
!   divided by the distance between upper and under surface
! **********************************************************************
!
!                       KEYWORDS
!
!     distance
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imax(3), kbndbt(*)
      double precision bcord1(3,*), bcord3(3,*), dz, alpha

!     alpha   o    multiplication factor to be used between under surface
!                  and upper surface
!                  The ksi-eta co-ordinates in a coross section are computed by
!                  xsieta = (1-alpha)*xsieta_bot _ alpha*xsieta_top
!     bcord1  i    Co-ordinates of nodal points in bottom surface
!     bcord3  i    Co-ordinates of nodal points in envelopping pipe surface
!     dz      i    Distance between under and upper face
!     imax    i    In this array the indices with respect to boundary points
!                  in the surfaces are stored
!                  imax(1) stores the node number of the first point (usually 1)
!                  imax(2) stores the node number with respect to the xsi dir.
!                          The xsi direction is defined as P2-P1, where P2 is
!                          given by bcord(kbndbt(imax(2))
!                  imax(3) stores the node number with respect to the first eta
!                          dir.
!                          The eta direction is defined as P3-P1, where P3 is
!                          given by bcord(kbndbt(imax(3))
!     kbndbt  i    Array KBNDPT for bottom surface
!                  Contains the local nodal point numbers of the boundary of
!                  the bottom surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision h

!     h       Help variable to store a constant temporarily
!     i       Counting variable
!     j       Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( abs(dz)<=0d0 ) then

!     --- under and upper surface are identical

         alpha = 1d0
      else

!     --- under and upper surface are not identical

         i = kbndbt(imax(2))
         j = kbndbt(imax(1))
         h = 0.5d0 * ( sqrt ( (bcord1(1,j)-bcord3(1,imax(1)))**2 + &
                              (bcord1(2,j)-bcord3(2,imax(1)))**2 + &
                              (bcord1(3,j)-bcord3(3,imax(1)))**2 ) + &
                       sqrt ( (bcord1(1,i)-bcord3(1,imax(2)))**2 + &
                              (bcord1(2,i)-bcord3(2,imax(2)))**2 + &
                              (bcord1(3,i)-bcord3(3,imax(2)))**2 ) )
         alpha = h / dz
      end if
      end
