      subroutine mshfaceedges ( nmeshb, nfaces, nedges, ifacedges, &
                                iworkfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array faceedges with all boundary faces corresponding to the
!     boundary edges
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     pipe
!     tetrahedron
!     boundary
!     triangle
!     face
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshb(*), nfaces, nedges, ifacedges(2,nedges), &
              iworkfaces(*)

!     ifacedges      o    Work array in which all the faces corresponding to
!                         the boundary edges i are stored in
!                         ifaceedges(1,i) and ifaceedges(2,i)
!                         if i is not a boundary edge ifaceedges(.,i) = 0
!     iworkfaces     i    Integer work array of size nfaces indicating the
!                         number of elements connected to the faces
!     nedges         i    number of edges in volume
!     nfaces         i    Number of faces in volume
!     nmeshb         i    nmesh part b in volume
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iface, j, istart, iedge

!     i              Counting variable
!     iedge          edge sequence number
!     iface          Face sequence number
!     istart         First position used at a part
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array ifacedges
!     For all faces do
!        if ( face = boundary face ) then
!           fill face number in ifacedges
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfaceedges' )

!     --- Clear array ifacedges

      do i = 1, nedges
         ifacedges(1,i) = 0
         ifacedges(2,i) = 0
      end do

!     --- For all faces do

      do iface = 1, nfaces

         if ( iworkfaces(iface).eq.1 ) then

!        --- face = boundary face
!            fill face number in ifacedges

            istart = nmeshb(iface)

            do j = 1, 3
               iedge = abs(nmeshb(istart+j))
               if ( ifacedges(1,iedge).eq.0 ) then
                  ifacedges(1,iedge) = iface
               else
                  ifacedges(2,iedge) = iface
               end if
            end do

         end if

      end do

      call erclos ( 'mshfaceedges' )
      end
