      subroutine mshfil12 ( iinput, iposi, nptot, neltot )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-11-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array KBNDPT for a composite surface
!     and write to file 3
!     Type surface: 12 (pipe surface)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iposi, nptot, neltot

!     iinput        i     Array containing input for the composite surface
!     iposi         i     next free position of iinput
!     neltot        i     Total number of elements in composite surface
!     nptot         i     Total number of nodes in composite surface, &
!                         centroids excluded
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isub, irow, isubsf, jsurf, kstart, m1, n1, &
              ndefsf, nrow, ns

!     irow           Row number in rows of surfaces in ordered surface of
!                    surfaces
!     isub           Loop variable
!     isubsf         Subsurface sequence number
!     jsurf
!     kstart
!     m1             Number of elements in vertical direction per row of
!                    surfaces
!     n1             Number of elements in horizontal direction per row of
!                    surfaces
!     ndefsf         Number of subsurfaces in surface
!     nrow           Number of rows of surfaces in ordered surface of surfaces
!     ns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1392   Not all subsurfaces of ordered pipe surface are of type pipe surface
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfil12' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfil12'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      nrow = iinput(iposi+2)
      kstart = iposi + nrow + 2
      isubsf = abs( iinput(kstart+1) )
      ns = nsurfs + 2 + (isubsf-1) * 8
      ishape = kmeshs(ns+1)

      neltot = 0
      if ( debug ) write(irefwr,1) 'nrow, ishape, isubsf', &
                                    nrow, ishape, isubsf
      if ( ishape==7 ) then

!     --- ishape = 7, compute total number of elements

         n1 = 0
         m1 = 0
         do irow = 1, nrow
            ndefsf = mod(iinput(iposi+irow+2),10000)
            do isub = 1, ndefsf
               isubsf = abs( iinput(kstart+isub) )
               ns = nsurfs + 2 + (isubsf-1) * 8
               jsurf  = kmeshs(ns)
               if ( debug ) &
                  write(irefwr,1) 'irow, isub, isubsf, jsurf', &
                                   irow, isub, isubsf, jsurf
               if ( jsurf/=10 .and. jsurf/=12 ) then

!              --- wrong type of subsurface

                  call errint ( isub, 1 )
                  call errint ( irow, 2 )
                  call errsub ( 1392, 2, 0, 0 )
                  go to 1000

               end if
               nmes = kmeshs(ns+6)
               mmes = kmeshs(ns+7)
               neltot = neltot+2*nmes*mmes
               if ( irow==1 ) n1 = n1+nmes
               if ( debug ) write(irefwr,1) 'n ,m, neltot, n1', &
                                             nmes ,mmes, neltot, n1

            end do
            m1 = m1+mmes

         end do
         nptot = (2*n1+1)*(2*m1+1)

      end if

1000  if ( debug ) then

         write(irefwr,*) 'End mshfil12'

      end if  ! ( debug )
      call erclos ( 'mshfil12' )

      end
