      subroutine mshfillkmeshs2 ( curvstosurfs, nsurfs, lncurvstosurfs, &
                                  lencurvstosf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 08-04-2005 Extra parameter lencurvstosf
!        version  1.1  date 13-10-2004 Extension of curvstosurfs
!        version  1.0  date 26-03-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the length of array curvstosurfs
!     This means the number of curves stored in this array
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     outer_boundary
!     surface
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfs, curvstosurfs(nsurfs,*), lncurvstosurfs, &
              lencurvstosf

!     curvstosurfs   i    Array of size nsurfs x nsurfs
!                         curvstosurfs(i,j) contains the j-th curve
!                         corresponding to surface i
!                         if 0 no more curves can be found
!     lencurvstosf   i    Length of second dimension of curvstosurfs
!     lncurvstosurfs o    Length of array curvstosurfs
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillkmeshs2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillkmeshs2'

      end if
      if ( ierror/=0 ) go to 1000
      lncurvstosurfs = 0
      do i = 1, nsurfs
         do j = 1, lencurvstosf
            if ( curvstosurfs(i,j)==0 ) go to 100
            lncurvstosurfs = lncurvstosurfs+1
         end do  ! j = 1, nsurfs
100   end do  ! i = 1, nsurfs

1000  call erclos ( 'mshfillkmeshs2' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfillkmeshs2'

      end if

      end
