      subroutine mshfl3 ( kbndlc, kbndpt, icurvs, ipoint, nloc, mloc, &
                          ntot, iboun, nrel, npoloc, idirec, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 27-11-2012 Long integers
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.1  date 27-10-2008 New call to mshfl5
!
!   copyright (c) 1987-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill arrays KBNDPT and ipoint for one surface
!     isurf = 11:  ordened
!    Help subroutine for subroutine MSHFIL
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*), icurvs(*), ipoint(*), iboun(*), &
              kbndlc(*), nloc, mloc, ntot, nrel, npoloc, idirec, &
              iwork(*)

!     iboun    i  Array of length nboun containing the curve numbers of the
!                 boundary of the local surface
!     icurvs  i/o Array of length ncurvs containing information of the curves
!                 with respect to kbndpt
!                 icurvs is filled as follows:
!                 icurvs(i) = 0    Curve i is not present
!                 icurvs(i)>0    Curve i has not been used before
!                                  The value gives the starting position in
!                                  array kbndpt
!                 icurvs(i)<0    Curve i has been used before
!                                  Minus the value gives the starting position
!                                  in array kbndpt
!     idirec   i  Gives direction of subsurfaces
!                 idirec = 1:  normal direction  ("left to right")
!                 idirec = 2:  reversed direction ("right to left")
!     ipoint   i  Help array of length npoloc, to store the new nubering of
!                 the nodes
!     iwork    i  Work array of length NCURVS.
!                 IWORK is filled as follows:
!                 If iwork(i) = 0, then curve i does not belong to the surface
!                 If iwork(i)#0, then curve iwork(i) belongs to the surface
!                 The sign of iwork(i) indicates the direction of storage in
!                 KBNDPT
!     kbndlc   i  Local array KBNDPT corresponding to surface
!     kbndpt  i/o Array containing the boundary points
!                 of the composite surface (curves are counted for only once)
!     mloc     i  Number of elements in "vertical direction of local surface"
!     nloc     i  Number of elements in "horizontal direction of local surface"
!     npoloc   i  Number of nodes in local surface
!     nrel     i  Relative element number of at most left element at the
!                 bottom of local surface, with respect to one row of
!                 elements
!     ntot     i  Number of elements in "horizontal direction" of row of
!                 local surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ifirst, ishfgl, ishflc, j, m, n, &
              nloc1, nr, nrel1, nt, ntot1

!     i          General loop variable
!     ifirst     Input parameter for subroutine MSFL5
!                Indicates the last position used in kbndlc
!     ishfgl     Shift for global numbering
!                This shift indicates the global nodal point numbering
!                before the present row
!     ishflc     Shift for local numbering
!                This shift indicates the local nodal point numbering
!                before the present row
!     j          General loop variable
!     m          Parameter for mesh generation
!     n          Parameter for mesh generation
!     nloc1      Number of elements in "horizontal" direction of local surface
!     nr         Relative nodal point number of at most left nodal point in
!                local surface minus one
!                This nodal point number is counted for one row of surfaces
!                only. So in fact it indicates the relative "n" position in a
!                row of surfaces
!     nrel1      Number of elements in the under row of a complete row of
!                surfaces
!     nt         Number of nodes in the under row of a complete row of
!                surfaces
!     ntot1      Number of elements in "horizontal" direction of row of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSHFL5    Fill array kbndpt
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!  --------------------------------------------------------------------------- &
!  |                 |                         |                             |
!  |                 |                         |                             |
!  |                 |                         |                             |
!  |                 |                         |                             |
!  --------------------------------------------------------------------------- &
!  |                       |                        |                        |
!  |                       |                        |                        |
!  |                       |                        |                        |
!  |                       |                        |                        |
!  |                       |                        |                        |
!  |                       |                        |                        |
!  |mloc                   |  local surface         |                        |
!  |                       |                        |                        |
!  |                       |                        |                        |
!  |                       |                        |                        |
!  |                       |                        |                        |
!  --------------------------------------------------------------------------- &
!                          nrel      nloc                                  ntot
!   Type of numbering used:
!   Standard elements of shape 3 and 5 (linear elements)
!  n+1                  n+2                 n+3                      n+4 ...
!   ____________________________________________________________________
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |                     |                    |                       |
!   -------------------------------------------------------------------- &
!   1                     2                    3                       4 . . n
!   Standard elements of shape 4 and 6 (Quadratic elements)
!  2n+1     2n+2        2n+3      2n+4      2n+5        2n+6         2n+7...
!   ____________________________________________________________________
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |n+1     n+2          |n+3     n+4         | n+5    n+6        n+7 | ...
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |                     |                    |                       |
!   -------------------------------------------------------------------- &
!   1         2           3         4          5          6            7 ... n
!   Standard elements of shape 7 (Quadratic triangles with exta point)
!  2s+1     2s+2        2s+3      2s+4      2s+5        2s+6         2s+7...
!   ____________________________________________________________________
!   |                     |                    |                       |
!   | l+1                 | l+2                |  l+3                  | ...
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |k+1    k+2           |k+3  k+4            | k+5    k+6        k+7 | ...
!   |                     |                    |                       |
!   |                     |                    |                       |
!   |          n+1        |         n+2        |         n+3           | ...
!   |                     |                    |                       |
!   -------------------------------------------------------------------- &
!   1         2           3         4          5          6            7 ... n
!   n:   Number of nodes in horizontal direction of lower boundary
!   k:   n+(n-1)/2
!   l:   k+n
!   s:   l+(n-1)/2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- First compute some constants depending on the type of element

      if ( ishape==3 .or. ishape==5 ) then

!     --- ishape = 3, 5:  Linear elements

         n = nloc+1
         m = mloc+1
         nr = nrel
         nt = ntot + 1
      else

!     --- ishape = 4, 6, 7:  Quadratic elements

         n = nloc*2+1
         m = mloc*2+1
         nr = 2*nrel
         nt = 2*ntot + 1
       end if
       if ( ishape==3 .or. ishape==4 .or. ishape==7 ) then

!      --- ishape = 3, 4 or 7  triangular elements

          nrel1 = 2 * nrel
          nloc1 = 2 * nloc
          ntot1 = 2 * ntot
       else

!      --- ishape = 5 or 6  quadrilateral elements

          nrel1 = nrel
          nloc1 = nloc
          ntot1 = ntot
       end if

!      --- Fill array ipoint

      if ( ishape==7 ) then

!     --- ishape = 7, 7-point triangles
!                     These elements require a special treatment, because
!                     the extra centroid nodes introduce a new type of numbering
!                     See method

         if ( idirec==1 ) then

!        --- idirec = 1, standard direction

            ishflc = 0
            ishfgl = nr + npointlc

!           --- Loop over all element rows in one surface

            do j = 1, mloc

!           --- Under boundary of row of elements:

               do i = 1, n
                  ipoint(ishflc+i) = i + ishfgl
               end do
               ishflc = ishflc + n
               ishfgl = ishfgl + nt-nrel

!              --- First row of centroids:

               do i = 1, nloc
                  ipoint(ishflc+i) = i + ishfgl
               end do
               ishflc = ishflc + nloc
               ishfgl = ishfgl + ntot + nrel

!              --- Mid row of elements:

               do i = 1, n
                  ipoint(ishflc+i) = i + ishfgl
               end do
               ishflc = ishflc + n
               ishfgl = ishfgl + nt-nrel

!              --- Second row of centroids:

               do i = 1, nloc
                  ipoint(ishflc+i) = i + ishfgl
               end do
               ishflc = ishflc + nloc
               ishfgl = ishfgl + ntot + nrel
               if ( j==mloc ) then

!              --- j = mloc, i.e. last row of surfaces in a local row
!                  In this case also the upper row must be taken into account

                  do i = 1, n
                     ipoint(ishflc+i) = i + ishfgl
                  end do
               end if
            end do
         else

!        --- idirec = -1, reversed direction

            ishflc = 0
            ishfgl = nr + npointlc

!           --- Loop over all element rows in one surface

            do j = 1, mloc

!           --- Under boundary of row of elements:

               do i = 1, n
                  ipoint(ishflc+i) = ishfgl + n+1 - i
               end do
               ishflc = ishflc + n
               ishfgl = ishfgl + nt-nrel

!              --- First row of centroids:

               do i = 1, nloc
                  ipoint(ishflc+i) = ishfgl + nloc+1 - i
               end do
               ishflc = ishflc + nloc
               ishfgl = ishfgl + ntot + nrel

!              --- Mid row of elements:

               do i = 1, n
                  ipoint(ishflc+i) = ishfgl + n+1 - i
               end do
               ishflc = ishflc + n
               ishfgl = ishfgl + nt-nrel

!              --- Second row of centroids:

               do i = 1, nloc
                  ipoint(ishflc+i) = ishfgl + nloc+1 - i
               end do
               ishflc = ishflc + nloc
               ishfgl = ishfgl + ntot + nrel
               if ( j==mloc ) then

!              --- j = mloc, i.e. last row of surfaces in a local row
!                  In this case also the upper row must be taken into account

                  do i = 1, n
                     ipoint(ishflc+i) = ishfgl + n+1 - i
                  end do
               end if
            end do
         end if
      else

!     --- Other types of elements

         if ( idirec>0 ) then

!        --- idirec = 1   normal direction

            do j = 1, m
               do i = 1, n
                  ipoint( (j-1)*n + i ) = npointlc + nr + (j-1)*nt + i
               end do
            end do
         else

!        --- idirec = -1   reversed direction

            do j = 1, m
               do i = 1, n
                  ipoint( (j-1)*n + i ) = npointlc+nr+(j-1)*nt-i+n+1
               end do
            end do
         end if
      end if

!     --- Fill elements in array ipoint

      if ( idirec>0 ) then

!     --- idirec = 1   normal direction

        do j = 1, mloc
           do i = 1, nloc1
              ipoint(npoloc+(j-1)*nloc1+i) = (j-1)*ntot1+nrel1+i+nelemlc
           end do
        end do
      else

!     --- idirec = -1   reversed direction

        do j = 1, mloc
           do i = 1, nloc1
               ipoint(npoloc+(j-1)*nloc1+i) = (j-1)*ntot1+nrel1+ &
                      nloc1+1-i+nelemlc
           end do
        end do
      end if

!     --- Finally fill array kbndpt

      ifirst = 0
      call mshfl5 ( iboun, kmeshm, icurvs, nboun, kbndpt, kbndlc, &
                    ipoint, iwork, ifirst, 0 )
      end
