      subroutine mshkmeshrdir1 ( isubsf, kmeshsp2, iwork2, kmeshsp3, &
                                 nouter, istart, debuglc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 28-01-2009 Extra debug
!        version  1.0  date 01-08-2002
!
!   copyright (c) 2002-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store the outer boundaries of a subsurface into array iwork2
!     Provide them with a sign such that all curves have the same orientation
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     orientation
!     direction
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubsf, kmeshsp2(8,*), iwork2(*), kmeshsp3(*), nouter, &
              istart
      logical debuglc

!     debuglc        i    Indicates if the file exist (true) or not
!     istart         i    First position used at a part
!     isubsf         i    Sequence number of subsurface
!     iwork2         o    integer work array containing the outer curves of
!                         the surface provided with a sign
!     kmeshsp2       i    Second part of kmeshs, referring to surfaces
!     kmeshsp3       i    Third part of kmeshs, containg curve numbers
!     nouter         i    Number of outer boundaries
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, nh, nh1, itypsf, jsubsf

!     itypsf         Indicates type of surface generator
!     jsubsf         Sequence number of subsurface
!     k              Counting variable
!     nh             help variable
!     nh1            help variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Detect itypsf and jsubsf
!     Store curve numbers of sub surface in iwork2
!     provided with a sign
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshkmeshrdir1' )
      debug = debuglc
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshkmeshrdir1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Detect itypsf and jsubsf

      itypsf = kmeshsp2(1,isubsf)
      if ( itypsf==8 .or. itypsf==9 .or. &
           itypsf==14 .or. itypsf==9 ) then
         jsubsf = kmeshsp2(3,isubsf)
         itypsf = kmeshsp2(1,jsubsf)
      else
         jsubsf = isubsf
      end if

!     --- Store curve numbers of sub surface in iwork2
!         provided with a sign

      if ( itypsf==10 ) then

!     --- Special case of pipe surface
!         curves with 1 and 4 with plus, other 2 with minus sign

         nh = kmeshsp2(3,jsubsf)
         do k = 1, nh
            iwork2(k) = kmeshsp3(istart+k)
         end do
         nh1 = nh
         nh = kmeshsp2(4,jsubsf)
         do k = nh1+1, nh1+nh
            iwork2(k) = -kmeshsp3(istart+k)
         end do
         nh1 = nh+nh1
         nh = kmeshsp2(5,jsubsf)
         do k = nh1+1, nh1+nh
            iwork2(k) = -kmeshsp3(istart+k)
         end do
         nh1 = nh+nh1
         do k = nh1+1, nouter
            iwork2(k) = kmeshsp3(istart+k)
         end do

      else

!     --- Standard case

         do k = 1, nouter
            iwork2(k) = kmeshsp3(istart+k)
         end do

      end if
      if ( debug ) then

!     --- debug statements

         write(irefwr,*) 'isubsf, jsubsf, itypsf ', &
                          isubsf, jsubsf, itypsf
         call prinin ( iwork2, nouter, 'local boundary')
         call prinin ( kmeshsp2(1,isubsf), nouter, &
                       'kmeshs part')

      end if

1000  call erclos ( 'mshkmeshrdir1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshkmeshrdir1'

      end if  ! ( debug )

      end
