      subroutine mshnegordsurf1 ( ipoint, kbndpt, ishape, nelloc, &
                                  inplmt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 13-11-2002
!
!   copyright (c) 2002-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorder array kbndpt for negative subsurfaces of a compound
!     ordered surface
!     Actual lower subroutine of mshnegordsurf that carries out the
!     reordering
! **********************************************************************
!
!                       KEYWORDS
!
!     compound_surface
!     ordering
!     reordering
!     surfaces
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipoint(*), ishape, nelloc, inplmt, kbndpt(inplmt,*)

!     inplmt         i    Number of points in element of surface corresponding
!                         to volume
!     ipoint         i    Contains the element numbers of the subsurface
!                         with respect to array kbndpt
!     ishape         i    Shape number of elements
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         boundary
!     nelloc         i    Local number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istore(10), i, j, ielem

!     i              Counting variable
!     ielem          Element number
!     istore         Work array to store the nodes of one element
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshnegordsurf1' )

      if ( ishape.eq.5 ) then

!     --- 4 point quads
!         loop over elements

          do i = 1, nelloc
             ielem = ipoint(i)
             do j = 1, 4
                istore(j) = kbndpt(j,ielem)
             end do
             kbndpt(1,ielem) = istore(2)
             kbndpt(2,ielem) = istore(1)
             kbndpt(3,ielem) = istore(4)
             kbndpt(4,ielem) = istore(3)

          end do

      else if ( ishape.eq.6 ) then

!     --- 4 point quads
!         loop over elements

          do i = 1, nelloc
             ielem = ipoint(i)
             do j = 1, 8
                istore(j) = kbndpt(j,ielem)
             end do
             kbndpt(1,ielem) = istore(3)
             kbndpt(3,ielem) = istore(1)
             kbndpt(4,ielem) = istore(8)
             kbndpt(5,ielem) = istore(7)
             kbndpt(7,ielem) = istore(5)
             kbndpt(8,ielem) = istore(4)

          end do

      else

!     --- Shape number not yet implemented

         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )

      end if
      call erclos ( 'mshnegordsurf1' )
      end
