      subroutine mshobstmap2d ( npoint, npointmap, nmap, nmapmax, &
                                node1, node2, inode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2001
!
!   copyright (c) 2001-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays imap2 and rmap2 for a new point inode which is the mean
!     value of two already existing nodes node1 and node2 stored in imap/rmap
! **********************************************************************
!
!                       KEYWORDS
!
!     mapping_array
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, npointmap, nmap, nmapmax, &
              node1, node2, inode

!     inode          i    Node sequence number
!     nmap           i    length of first parameter of imap and rmap
!     nmapmax       i/o   Actual value of nmap
!     node1          i    Node of face
!     node2          i    Node of face
!     npoint         i    Number of nodal points
!     npointmap      i    First index of array imap2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nodp, ih, nh, numnodes, nnodes, iwork(nmap)
      double precision rwork(nmap)
      logical found

!     found          If true already a possibility has been found
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     iwork          integer work array of length nmap to store node numbers
!     j              Counting variable
!     nh             Help variable to store a constant temporarily
!     nnodes         Number of nodes
!     nodp           Nodal point sequence number
!     numnodes       Number of nodal points
!     rwork          real work array of length nmap to store coefficients
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store coefficients and nodes corresponding to node1 in iwork, rwork
!     For all nodes corresponding to node2 do
!        if ( attached node not in iwork ) then
!           add node to iwork
!           add coefficient to rwork
!        else if ( attached node in iwork ) then
!           add coefficient to old position in rwork
!     Copy information from iwork to imap
!     Copy information from rwork to rmap multiplied by 0.5
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstmap2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstmap2d'
         call ertrace
         write(irefwr,1) 'nmap, npoint, node1', nmap, npoint, node1
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store coefficients and nodes corresponding to node1 in iwork, rwork

      nnodes = imap(node1,1)
      if ( debug )  write(irefwr,1) 'nnodes', nnodes
      do i = 1, nnodes
         iwork(i) = imap(node1,i+1)
         rwork(i) = rmap(node1,i)
      end do
      numnodes = nnodes

!     --- For all nodes corresponding to node2 do

      nh = imap(node2,1)
      do i = 1, nh
         nodp =  imap(node2,i+1)
         found = .false.
         do j = 1, nnodes
            if ( nodp==iwork(j) ) then

!           --- Node already present in iwork

               ih = j
               found = .true.
               go to 100

            end if

         end do

100      if ( found ) then

!        --- attached node in iwork
!            add coefficient to old position in rwork

            rwork(ih) = rwork(ih)+rmap(node2,i)

         else

!        --- attached node not in iwork
!            add node to iwork
!            add coefficient to rwork

            numnodes = numnodes+1
            iwork(numnodes) = nodp
            rwork(numnodes) = rmap(node2,i)

         end if

      end do

      if ( numnodes>nmap ) then

!     --- nmap too small

         call errint ( numnodes, 1 )
         call errint ( nmap, 2 )
         call errchr ( 'imap2', 1 )
         call errsub ( 550, 2, 0, 1 )
         imap2(inode,1) = 0
         go to 1000

      end if
      nmapmax = max(nmapmax,numnodes)

!     --- Copy information from iwork to imap
!         Copy information from rwork to rmap multiplied by 0.5

      imap2(inode,1) = numnodes
      do i = 1, numnodes
         imap2(inode,i+1) = iwork(i)
         rmap2(inode,i) = rwork(i)*0.5d0
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstmap2d'

      end if  ! ( debug )
      call erclos ( 'mshobstmap2d' )

      end subroutine mshobstmap2d
