      subroutine mshobstms3d06b ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew, &
                                  nedgenode )
! ======================================================================
!
!        programmer    Koos Meijerink
!        version  3.0  date 19-05-2005 Extra parameter nedegenode
!        version  2.0  date 21-01-2002 Remove coornew
!        version  1.0  date 11-07-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide tetrahedron with three crossed edges, all in one plane, &
!     into four tetrahedrons.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(4), tetnodenew(16), nedgenode, &
              edgenode(nedgenode,3), isort(4), &
              ledgenode(2,3), nelemnew

!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     isort          i    if isort(i)>isort(j) then tetnode(i) is at least on
!                         the same number of crossed edges as tetnode(j)
!     ledgenode      i    for i=1,2 and 1<j< number of crossed edges
!                         ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                         in other words:
!                         edgnode(i,j) = tetnode(ledgenode(i,j))
!     nedgenode      i    Length of first index of edgenode
!     nelemnew       o    Number of elements in new mesh
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, &
              trans(4), transi(4), transtemp, transposj, &
              tetnodet(4), edgenodet(6)
      integer pedgenodet(4,4)
      logical oddpairintchan

!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     i              Counting variable
!     j              Counting variable
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     pedgenodet     the local edge number = local edgenode number
!                    for the edge [local tetnode i, local tetnode j]
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     trans          Transformation array. trans(i) is the transformed
!                    local position of the i-th corner of the original
!                    tetrahedron.
!     transi         Inverse of trans
!     transposj      The transformed local edge node number of edge node j
!     transtemp      temporary storage
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Define transformation(=rotation) to move node isort(1) to node 4.
!
!     Transform tetrahedron tetnode to tetnodet and the position of the
!     intersection points edgenode(3,.) to edgenodet(.).
!
!     Split tetrahedron into four subtetrahedrons.
!
!     Calculate volumes of tetrahedrons and compare.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data pedgenodet /0,1,3,4, &
                       1,0,2,5, &
                       3,2,0,6, &
                       4,5,6,0/
! ======================================================================
!
!     --- pedgenodet(i,j) is the midpoint number for the edge [node i,node j]

      call eropen ( 'mshobstms3d06b' )

!     --- Define transformation to move node isort(1) to node 4
!         -- Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move node isort(1)

      if( isort(1)/=trans(4) ) then
         transtemp = trans(4)
         trans(4) = isort(1)
         trans(isort(1)) = transtemp
         oddpairintchan = .not.oddpairintchan
      end if

!     --- make pairinterchanges even

      if( oddpairintchan ) then
         transtemp = trans(1)
         trans(1) = trans(2)
         trans(2) = transtemp

      end if

!     --- transi := inverse of trans

      do i = 1,4
         transi(trans(i)) = i
      end do

!     --- Transform tetrahedron tetnode to tetnodet

      do i = 1,4
         tetnodet(i) = tetnode(trans(i))
      end do

!     --- Transform the intersection points to their transformed positions

      do j = 1,3
         transposj = pedgenodet(transi(ledgenode(1,j)), &
                                transi(ledgenode(2,j)))
         edgenodet(transposj) = edgenode(3,j)
      end do

!     --- Split tetrahedron into four new tetrahedrons.

      tetnodenew(2) = tetnodet(4)
      tetnodenew(3) = edgenodet(1)
      tetnodenew(4) = edgenodet(2)
      tetnodenew(1) = edgenodet(3)
      tetnodenew(5) = tetnodet(4)
      tetnodenew(6) = edgenodet(1)
      tetnodenew(7) = tetnodet(1)
      tetnodenew(8) = edgenodet(3)
      tetnodenew(9) = tetnodet(4)
      tetnodenew(10) = tetnodet(3)
      tetnodenew(11) = edgenodet(2)
      tetnodenew(12) = edgenodet(3)
      tetnodenew(13) = tetnodet(4)
      tetnodenew(14) = edgenodet(1)
      tetnodenew(15) = edgenodet(2)
      tetnodenew(16) = tetnodet(2)

      nelemnew = 4

      call erclos ( 'mshobstms3d06b' )

      end
