      subroutine mshobstms3d07b ( tetnode, tetnodenew, edgenode, &
                                  ledgenode, nelemnew )
! ======================================================================
!
!        programmer    Koos Meijerink
!        version  2.2  date 19-05-2005 Reset of edgenode
!        version  2.1  date 07-04-2004 Extension of edgenode (RvL)
!        version  2.0  date 21-01-2002 Remove coornew
!        version  1.0  date 11-07-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron with four crossed edges which form a closed
!     curve.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(4),tetnodenew(24), edgenode(3,4), &
              ledgenode(2,4), nelemnew

!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     ledgenode      i    for i=1,2 and 1<j< number of crossed edges
!                         ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                         in other words:
!                         edgnode(i,j) = tetnode(ledgenode(i,j))
!     nelemnew       o    Number of elements in new mesh
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, &
              trans(4), transi(4),transtemp, transposj, switch, &
              tetnodet(4), edgenodet(6), &
              tetnodettemp, lowedgenode
      integer pedgenodet(4,4)
      logical oddpairintchan, transpos2

!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     i              Counting variable
!     j              Counting variable
!     lowedgenode    The number of the intersection node with the lowest local
!                    edge node number
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     pedgenodet     the local edge number = local edgenode number
!                    for the edge [local tetnode i, local tetnode j]
!     switch         if oddpairinterchanges .true. then switch = 1
!                    else switch = 0
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     tetnodettemp   temporary storage for a tetnodet element
!     trans          Transformation array. trans(i) is the transformed
!                    local position of the i-th corner of the original
!                    tetrahedron.
!     transi         Inverse of trans
!     transpos2      transpos2 = .true. if there is an edge node on local edge 2
!     transposj      The transformed local edge node number of edge node j
!     transtemp      temporary storage
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!                   the new tetrahedrons
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine intersection point with lowest node index
!
!     Define transformation to move this node to local edge position 1.
!
!     Transform tetrahedron tetnode to tetnodet and the position of the
!     intersection points edgenode(3,.) to edgenodet(.)
!
!     If intersection points at positions 1,3,5,6 transform tetrahedron such
!     that they get into positions 1,2,4,6
!
!     Split tetrahedron into six subtetrahedrons, taking into account the
!     determinant of the transformation.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data pedgenodet /0,1,3,4, &
                       1,0,2,5, &
                       3,2,0,6, &
                       4,5,6,0/
! ======================================================================
!
      call eropen ( 'mshobstms3d07b' )

!     --- Search for the intersection node with the lowest node index

      lowedgenode = 1
      if( edgenode(3,2)<edgenode(3,lowedgenode)) lowedgenode = 2
      if( edgenode(3,3)<edgenode(3,lowedgenode)) lowedgenode = 3
      if( edgenode(3,4)<edgenode(3,lowedgenode)) lowedgenode = 4

!     --- Move the intersection node with the lowest node index
!         to local intersection point 1:
!         -- Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move node ledgenode(1,lowedgenode) to node 1 and
!              node ledgenode(2,lowedgenode) to node 2

!         -- move node ledgenode(1,lowedgenode) to node 1

      if( ledgenode(1,lowedgenode)/=trans(1) ) then
         transtemp = trans(1)
         trans(1) = ledgenode(1,lowedgenode)
         trans( ledgenode(1,lowedgenode)) = transtemp
         oddpairintchan = .not.oddpairintchan
      end if

!     --- move node ledgenode(2,lowedgenode) to node 2

      if (ledgenode(2,lowedgenode)/=trans(2) ) then
         transtemp = trans(2)
         trans(2) = ledgenode(2,lowedgenode)
         trans(trans(ledgenode(2,lowedgenode))) = transtemp
         oddpairintchan = .not.oddpairintchan

      end if

!     --- transi := inverse of trans

      do i = 1,4
         transi(trans(i)) = i
      end do

!     --- Transform tetrahedron tetnode to tetnodet

      do i = 1,4
         tetnodet(i) = tetnode(trans(i))
      end do

!     --- Move the intersection points to their transformed positions

      transpos2 = .false.
      do j = 1,4

         transposj = pedgenodet(transi(ledgenode(1,j)), &
                            transi(ledgenode(2,j)))
         if( transposj==2) transpos2 = .true.
         edgenodet(transposj) = edgenode(3,j)
      end do

!     --- There are two possible intersection node configurations:
!         1,2,4,6 and 1,3,5,6.
!         Transform the second configuration into the first.

      if( .not.transpos2 ) then

!     --- move edgenodet(5) to intersection point 2

            tetnodettemp = tetnodet(3)
            tetnodet(3) = tetnodet(4)
            tetnodet(4) = tetnodettemp
            oddpairintchan = .not.oddpairintchan
            edgenodet(2) = edgenodet(5)
            edgenodet(4) = edgenodet(3)
      end if
      if( oddpairintchan ) then
         switch = 1
      else
         switch = 0
      end if

!     --- Split tetrahedron

      tetnodenew(1+switch) = edgenodet(4)
      tetnodenew(2-switch) = edgenodet(1)
      tetnodenew(3) = edgenodet(6)
      tetnodenew(4) = tetnodet(4)
      tetnodenew(6-switch) = edgenodet(4)
      tetnodenew(7) = edgenodet(1)
      tetnodenew(8) = edgenodet(6)
      tetnodenew(10-switch) = edgenodet(1)
      if( edgenodet(4)<edgenodet(6) ) then
         tetnodenew(5+switch)  = tetnodet(3)

         tetnodenew(9+switch)  = edgenodet(4)
         tetnodenew(11) = tetnodet(1)
         tetnodenew(12) = tetnodet(3)
      else
         tetnodenew(5+switch)  = tetnodet(1)

         tetnodenew(9+switch)  = tetnodet(3)
         tetnodenew(11) = edgenodet(6)
         tetnodenew(12) = tetnodet(1)
      end if

      tetnodenew(13+switch) = edgenodet(2)
      tetnodenew(14-switch) = edgenodet(1)
      tetnodenew(15) = edgenodet(6)
      tetnodenew(16) = tetnodet(3)
      tetnodenew(18-switch) =  edgenodet(2)
      tetnodenew(19) = edgenodet(1)
      tetnodenew(20) = edgenodet(6)

      tetnodenew(22-switch) = edgenodet(1)
      if( edgenodet(2)<edgenodet(6) ) then
         tetnodenew(17+switch) = tetnodet(4)

         tetnodenew(21+switch)  = edgenodet(2)
         tetnodenew(23) = tetnodet(2)
         tetnodenew(24) = tetnodet(4)
      else
         tetnodenew(17+switch) = tetnodet(2)

         tetnodenew(21+switch)  = tetnodet(4)
         tetnodenew(23) = edgenodet(6)
         tetnodenew(24) = tetnodet(2)
      end if

      nelemnew = 6

      call erclos ( 'mshobstms3d07b' )

      end
