      subroutine mshobstms3d08 ( tetnode, tetnodenew, edgenode, &
                                 isort, ledgenode, nelemnew, nedgenode )
! ======================================================================
!
!        programmer    Koos Meijerink
!        version  3.0  date 19-05-2005 Extra parameter nedegenode
!        version  2.1  date 07-04-2004 Extension of edgenode (RvL)
!        version  2.0  date 21-01-2002 Remove coornew
!        version  1.0  date 11-07-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron with five crossed edges into seven tetrahedrons.
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(4),tetnodenew(28), nedgenode, &
              edgenode(nedgenode,5), isort(4), &
              ledgenode(2,5), nelemnew

!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     isort          i    if isort(i)>isort(j) then tetnode(i) is at least on
!                         the same number of crossed edges as tetnode(j)
!     ledgenode      i    for i=1,2 and 1<j< number of crossed edges
!                         ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                         in other words:
!                         edgnode(i,j) = tetnode(ledgenode(i,j))
!     nedgenode      i    Length of first index of edgenode
!     nelemnew       o    Number of elements in new mesh
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, &
              trans(4), transi(4),transtemp, transposj, switch, &
              tetnodet(4), edgenodet(6), &
              tetnodettemp, edgenodettemp
      integer pedgenodet(4,4)
      logical oddpairintchan

!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     edgenodettemp  temporary storage for a edgenodet element
!     i              Counting variable
!     j              Counting variable
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     pedgenodet     the local edge number = local edgenode number
!                    for the edge [local tetnode i, local tetnode j]
!     switch         if oddpairinterchanges .true. then switch = 1
!                    else switch = 0
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     tetnodettemp   temporary storage for a tetnodet element
!     trans          array containing the inverse of the transformation
!     transi         Inverse of trans
!     transposj      The transformed local edge node number of edge node j
!     transtemp      temporary storage
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!                   the new tetrahedrons
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Define transformation to move node isort(4) to node 1
!                               and node isort(3) to node 2.
!
!     Transform tetrahedron tetnode to tetnodet and the position of the
!     intersection points edgenode(3,.) to edgenodet(.)
!
!     If edgenodet(2)>edgenodet(5) interchange these nodes
!
!     Split tetrahedron into seven subtetrahedrons.
!
!     Calculate volumes of tetrahedrons and compare
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data pedgenodet /0,1,3,4, &
                       1,0,2,5, &
                       3,2,0,6, &
                       4,5,6,0/
! ======================================================================
!
!     --- Define transformation to move node isort(4) to node 1
!                                      and node isort(3) to node 2
!         -- Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move node isort(4) to node 1

      if( isort(4)/=trans(1) ) then
         transtemp = trans(1)
         trans(1) = isort(4)
         trans(isort(4)) = transtemp
         oddpairintchan = .not.oddpairintchan

       end if

!      --- move node isort(3) to node 2

      if (isort(3)/=trans(2) ) then
         transtemp = trans(2)
         trans(2) = isort(3)
         trans(trans(isort(3))) = transtemp
         oddpairintchan = .not.oddpairintchan

      end if

!     --- transi := inverse of trans

      do i = 1,4
         transi(trans(i)) = i
      end do

!     --- Transform tetrahedron tetnode to tetnodet

      do i = 1,4
         tetnodet(i) = tetnode(trans(i))
      end do

!     --- Transform the intersection points to their transformed positions

      do j = 1,5
         transposj = pedgenodet(transi(ledgenode(1,j)), &
                            transi(ledgenode(2,j)))
         edgenodet(transposj) = edgenode(3,j)
      end do

!     --- If edgenodet(2)>edgenodet(5) interchange these nodes

      if( edgenodet(2)>edgenodet(5) ) then
         tetnodettemp = tetnodet(3)
         tetnodet(3) = tetnodet(4)
         tetnodet(4) = tetnodettemp
         edgenodettemp = edgenodet(2)
         edgenodet(2) = edgenodet(5)
         edgenodet(5) = edgenodettemp
         edgenodettemp = edgenodet(3)
         edgenodet(3) = edgenodet(4)
         edgenodet(4) = edgenodettemp
         oddpairintchan = .not.oddpairintchan
      end if

!     --- Split tetrahedron
!         -- If pairinterchanges odd interchange first and second node of
!         each new tetrahedron.

      if( oddpairintchan ) then
         switch = 1
      else
         switch = 0
      end if

      tetnodenew(1+switch) = tetnodet(1)
      tetnodenew(2-switch) = edgenodet(1)
      tetnodenew(3) = edgenodet(3)
      tetnodenew(4) = edgenodet(4)
      tetnodenew(6-switch) = edgenodet(2)
      tetnodenew(7) = edgenodet(1)
      tetnodenew(8) = edgenodet(3)
      tetnodenew(5+switch) = edgenodet(4)
      tetnodenew(9+switch)  = tetnodet(2)
      tetnodenew(10-switch) = edgenodet(2)
      tetnodenew(11) = edgenodet(1)
      tetnodenew(12) = edgenodet(5)
      tetnodenew(14-switch)  = edgenodet(4)
      tetnodenew(15) = edgenodet(2)
      tetnodenew(16) = edgenodet(1)
      tetnodenew(13+switch) = edgenodet(5)
      if( edgenodet(3)<edgenodet(4) ) then
         tetnodenew(17+switch)  = tetnodet(3)
         tetnodenew(18-switch) = edgenodet(3)
         tetnodenew(19) = edgenodet(2)
         tetnodenew(20) = tetnodet(4)
         tetnodenew(22-switch)  = edgenodet(4)
         tetnodenew(23) = edgenodet(3)
         tetnodenew(24) = edgenodet(2)
         tetnodenew(21+switch) = tetnodet(4)
         tetnodenew(25+switch)  = edgenodet(4)
         tetnodenew(26-switch) = edgenodet(5)
         tetnodenew(27) = edgenodet(2)
         tetnodenew(28) = tetnodet(4)
      else
         tetnodenew(17+switch)  = tetnodet(3)
         tetnodenew(18-switch) = edgenodet(3)
         tetnodenew(19) = edgenodet(2)
         tetnodenew(20) = edgenodet(4)
         tetnodenew(22-switch)  = tetnodet(3)
         tetnodenew(23) = tetnodet(4)
         tetnodenew(24) = edgenodet(2)
         tetnodenew(21+switch) = edgenodet(4)
         tetnodenew(25+switch)  = edgenodet(5)
         tetnodenew(26-switch) = tetnodet(4)
         tetnodenew(27) = edgenodet(2)
         tetnodenew(28) = edgenodet(4)
      end if

      nelemnew = 7

      end
