      subroutine mshoutercheck
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 27-11-2012 Long integers
!        version  1.0  date 17-10-2003
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the outer boundary of a mesh from the single edges /faces in the
!    mesh and compare with the previously computed outer boundary
!    The 3d part has not yet been implemented
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     boundary
!     outer_boundary
!     curve
!     surface
!     check
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncurvstmp
      integer, allocatable :: kmeshmsave(:)

!     kmeshmsave     Old array kmeshm
!     ncurvstmp      Number of curves kmeshmtmp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHOUTERCHK2D  Compute the outer boundary of a 2d mesh from the single
!                    edges in the mesh and compare with the curves stored in
!                    ioutercurvs
!     SEPFILLKMESHM  Make relation between kmeshm and Curvesub
!     SEPFILLKMESHMN Fill array kmesh part m with information of all the curves
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
!    2465   Array kmesh part m has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ndim := dimension of space
!     if ( ndim = 2 ) then
!        Copy kmeshm temporarily to kmeshmsave
!        Compute new kmeshm from the single edges of the mesh
!        Compare old and new kmeshm
!        Remove new kmeshm
!        Copy kmeshmsave back to kmeshm
!     else ( ndim # 2 )
!        error: not yet implemented
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshoutercheck' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshoutercheck'

      end if
      if ( ierror/=0 ) go to 1000
!     --- ndim := dimension of space

      if ( ndim==2 ) then

!     --- ndim = 2
!         Copy kmeshm temporarily to kmeshmsave

         length = size(kmeshm)
         if ( length==0 ) then

!        --- kmeshm has not been filled

            call errsub ( 2465, 0, 0, 0 )
            go to 1000

         end if
         allocate ( kmeshmsave(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'kmeshmsave' )
         if ( ierror/=0 ) go to 1000
         kmeshmsave = kmeshm
         ncurvstmp = ncurvs

!        --- Compute new kmeshm from the single edges of the mesh

         call sepfillkmeshmn
         if ( ierror/=0 ) go to 1000

!        --- Compare old and new kmeshm

         call mshouterchk2d ( kmeshmsave, kmeshm, ncurvstmp, npoint, ncurvs )
         if ( ierror/=0 ) go to 1000

!        --- Reset new kmeshm

         nullify(kmeshm)
         deallocate ( km(iseqkm)%kmeshm )
         allocate ( km(iseqkm)%kmeshm(size(kmeshmsave)) )
         km(iseqkm)%kmesh(27) = size(kmeshmsave)

!        --- Copy kmeshmsave back to kmeshm

         kmeshm => km(iseqkm)%kmeshm
         kmeshm = kmeshmsave
         ncurvs = ncurvstmp
         call sepfillkmeshm

         deallocate ( kmeshmsave, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshmsave' )

      else  ! ( ndim/=2 )

!     --- ndim # 2
!         error: not yet implemented

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( ndim==2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshoutercheck'

      end if
      call erclos ( 'mshoutercheck' )

      end subroutine mshoutercheck
