      subroutine mshparn ( maparinv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-02-2003
!
!   copyright (c) 2003-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make local array kmeshn from global array kmeshn using array maparinv
!     This is used for subdivision of region in subregions in case of
!     parallel computing
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     parallel
!     subdivision
!     surface
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maparinv(*)

!     maparinv       i    Contains mapping from global numbering to local
!                         numbering
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      integer, pointer :: kmeshlocn(:)

!     i              Counting variable
!     kmeshlocn      Local KMESH part n
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparn' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparn'
         write(irefwr,1) 'nsurfs', nsurfs
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kmeshlocn => km(iseqkm)%kmeshn

!     --- copy kmeshn to kmeshlocn

      kmeshlocn(1:2*nsurfs) = kmeshn(1:2*nsurfs)

!     --- Compute length of part 3 of kmeshn

      length = 0
      do i = 1, nsurfs
         length = length+kmeshn(i)*kmeshn(i+nsurfs)
      end do
      if ( debug ) then
         call prinin ( kmeshn, 2*nsurfs+length, 'kmeshn' )
         call prinin ( maparinv, length, 'maparinv' )
      end if  ! ( debug )

!     --- Store new kmesh partn

      do i = 2*nsurfs+1, 2*nsurfs+length
         if ( kmeshn(i)>0 ) then
            kmeshlocn(i) = maparinv(kmeshn(i))
         else
            kmeshlocn(i) = 0
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmeshlocn, 2*nsurfs+length, 'kmeshnlocn' )
         write(irefwr,*) 'End mshparn'

      end if  ! ( debug )
      call erclos ( 'mshparn' )

      end subroutine mshparn
