      subroutine mshparncorr2 ( nfaces, nmeshf, list, numfacesurf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-04-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a list of faces that appear only once
!     These belong to the surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     2d
!     3d
!     parallel
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nfaces, nmeshf(*), list(*), numfacesurf

!     list           o    List of element numbers in kmeshncopy with respect to
!                         kmeshn
!     nfaces         i    Number of faces
!     nmeshf         i    nmesh part f, contains element numbers corresponding
!                         to faces
!     numfacesurf    o    Number of faces in istore to be considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iface, istart, numelms

!     iface          Face sequence number
!     istart         First position used at a part
!     numelms        Number of elements connected to face
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparncorr2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparncorr2'
         write(irefwr,*) 'nmesh f'
         call  prinnmeshx ( nfaces, nmeshf )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numfacesurf = 0

!     --- Loop over the faces

      do iface = 1, nfaces

         istart = nmeshf(iface)
         numelms = nmeshf(istart)
         if ( numelms==1 ) then

!        --- Face connected to one element

            numfacesurf = numfacesurf+1
            list(numfacesurf) = iface

         end if  ! ( numelms==1 )

      end do  ! iface = 1, nfaces

1000  call erclos ( 'mshparncorr2' )
      if ( debug ) then

!     --- Debug information

         call prinin ( list, numfacesurf, 'list of faces' )
         write(irefwr,*) 'End mshparncorr2'

      end if  ! ( debug )

      end
