      recursive subroutine mshparncorr6 ( kmeshncopy, istore, iloc, &
                                          inpelm, icount, list, &
                                          numfacesurf, kmeshn, iend, &
                                          nodesurf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-02-2012 Remove statement
!        version  1.0  date 05-04-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove those elements from kmeshn that are not faces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     2d
!     3d
!     parallel
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, icount, numfacesurf, kmeshncopy(inpelm,icount), &
              istore(inpelm,numfacesurf), iloc, list(icount), &
              kmeshn(inpelm,*), iend, nodesurf(*)

!     icount         i    Number of non-zero elements in kmeshncopy
!                         to be considered
!     iend           o    Last position used in istore
!     iloc           i    Sequence number of local node in element
!     inpelm         i    number of points per element
!     istore         i    Contains the part of the set of faces to be considered
!     kmeshn        i/o   KMESH part n, contains nodal point numbers of the
!                         surfaces
!     kmeshncopy     i    Sorted copy of array kmeshn with all zero elements
!                         removed
!     list           i    List of element numbers in kmeshncopy with respect to
!                         kmeshn
!     nodesurf      i/o   Array of length npoint containing the number
!                         of times a node in a surface appears
!                         If a node is cleared the number is decreased
!     numfacesurf    i    Number of faces in istore to be considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, ielem, inode, i
      logical whilecond

!     i              Counting variable
!     ielem          Element number
!     inode          Node sequence number
!     istart         Next position to be used in istore
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPARNCORR1   Clear one node in element in kmesh in the case that surface
!                    elements do not belong to the set of outer faces
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparncorr6' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparncorr6'
         write(irefwr,1) 'iloc, inpelm, icount, numfacesurf', &
                          iloc, inpelm, icount, numfacesurf

         call prinin1 ( kmeshncopy, icount, inpelm, 'kmeshncopy' )
         call prinin ( list, icount, 'list' )
         call prinin1 ( istore, numfacesurf, inpelm, 'istore/1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = 1

!     --- Loop over elements in kmeshncopy

      do ielem = 1, icount

         inode = kmeshncopy(iloc,ielem)
         if ( debug ) then
            write(irefwr,1) 'ielem, inode, iloc, istart', &
                             ielem, inode, iloc, istart
            call prinin ( istore(iloc,istart), 3, 'istore' )
         end if  ! ( debug )
         do while ( inode>istore(iloc,istart) )

            istart = istart+1

            if ( istart>numfacesurf ) then

!           --- End of faces, clear rest of kmeshn (one node per element)

               do i = ielem, icount

                  call mshparncorr1 ( kmeshn(1,list(ielem)), inpelm, &
                                      nodesurf )

               end do  ! i = ielem, icount
               if ( debug ) &
                  call prinin ( list(ielem), icount-ielem+1, 'list/0' )
               go to 1000

            end if  ! ( istart>numfacesurf )

         end do  ! while ( inode>istore(iloc,istart) )

         if ( debug ) then
            write(irefwr,1) 'ielem, inode, istart, iloc', &
                             ielem, inode, istart, iloc
            call prinin ( istore(iloc,istart), inpelm, 'istore' )
         end if  ! ( debug )

         if ( inode<istore(iloc,istart) ) then

!        --- inode<istore(iloc,istart), so not a face

            call mshparncorr1 ( kmeshn(1,list(ielem)), inpelm, &
                                nodesurf )
            iend = 0

            if ( debug ) write(irefwr,1) 'element/1', list(ielem)

         else

!        --- inode = istore(iloc,istart), check on rest of nodes
!            Find last node with the same end number

            iend = istart
            do while ( iend<numfacesurf )

               if ( inode==istore(iloc,iend+1) ) then
                  iend = iend+1
               else
                  go to 100
               end if  ! ( inode==istore(iloc,iend+1) )

            end do  ! while ( iend<numfacesurf )

100         if ( debug ) write(irefwr,1) 'iend', iend

            if ( istart==iend ) then

!           --- istart = iend, one node in common

               do i = iloc+1, inpelm

                  if ( kmeshncopy(i,ielem)/=istore(i,istart) ) then

!                 --- kmeshn does not correspond to face, set equal to 0

                     call mshparncorr1 ( kmeshn(1,list(ielem)), inpelm, &
                                         nodesurf )
                     if ( debug ) &
                        write(irefwr,1) 'element/2', list(ielem)

                  end if  ! ( kmeshn(i,list(ielem))/=istore(i,istart) )

               end do  ! i = iloc+1, inpelm
               istart = istart+1
               iend = 1

            else

!           --- istart>iend, use next nodes

               call mshparncorr6 ( kmeshncopy(1,ielem), &
                                   istore(1,istart), iloc+1, &
                                   inpelm, 1, list(ielem), &
                                   iend-istart+1, kmeshn, iend, &
                                   nodesurf )

            end if  ! ( istart==iend )
            if ( debug ) write(irefwr,1) 'istart', istart

            if ( istart>numfacesurf ) go to 1000

         end if  ! ( inode<istore(1,istart) ) then

      end do  ! ielem = 1, icount

1000  call erclos ( 'mshparncorr6' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshparncorr6'

      end if  ! ( debug )

      end
