      subroutine mshpav2fillbnd ( nowbnd, maxbounds, bndcoorold, &
                                  bndcoornew, crd, nloop, bndangle, &
                                  lnnowbnd, layer, maxlen, zero )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays bndcoorold and bndcoornew with old and new coordinates of
!     all boundaries
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxbounds, nowbnd(maxbounds,*), nloop, lnnowbnd, layer
      double precision crd(4,*), bndcoorold(2*lnnowbnd,*), &
                       bndcoornew(4*lnnowbnd,*), bndangle(lnnowbnd,*), &
                       maxlen
      logical zero(*)

!     bndangle       i    Array containing the angles corresponding to all nodes
!                         of a boundary loop
!     bndcoornew     o    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     o    Contains the coordinates of the previous (old)
!                         boundary during the paver process
!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     layer          i    Present layer sequence number
!     lnnowbnd       i    Declared length of second index of array nowbnd
!     maxbounds      i    Max number of loops in nowbnd
!     maxlen         i    Maximum amount of contiguous space in IBUFFR after a
!                         starting address
!     nloop          i    number of boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     zero           o    If true all nodes along the boundary have thickness
!                         zero
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ibndnr, numnodes

!     ibndnr         i'th boundary in nowbnd
!     numnodes       Number of nodal points in boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPAVCB       Fills the array bndcr with all the coordinates that
!                    are positioned on the edge of that loop
!     MSHPAVMNCOOR   Compute the new coordinates that arise if we create a new
!                    layer of elements alomg one boundary loop without actually
!                    computing the elements or taking into account angles
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For ibndnr := 1 (1) nloop do
!        Fill array bndcoorold with coordinates of old boundary
!        If ( there are nodes along boundary with thickness > 0 ) then
!            Compute coordinates of new row of points and
!            corresponding angles
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpav2fillbnd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpav2fillbnd'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- For ibndnr := 1 (1) nloop do

      do ibndnr = 1, nloop

!     --- Array bndcoorold is filled with all the coordinates of the
!         boundary part we are dealing with
!         The first coordinate is also placed at the end
!         Array nowbnd is extended with the first node as last one
!         numnodes contains the number of nodes in the boundary part + 1

         call mshpavcb ( nowbnd, bndcoorold(1,ibndnr), crd, ibndnr, &
                         numnodes, maxbounds, zero(ibndnr) )
         if ( debug ) &
            write(irefwr,*) 'ibndnr, nloop, numnodes ', &
                             ibndnr, nloop, numnodes
         if ( .not. zero(ibndnr) ) then

!        --- There is at least one point at the boundary with thickness larger
!            than zero, hence a new layer of elements may be formed

            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'After mshpavcb '
               call prinrl1 ( bndcoorold(1,ibndnr), numnodes, 2, &
                              'bndcoorold' )

            end if
            numnodes = numnodes-1   ! reset numnodes to actual value

!           --- Compute coordinates of new row of points and
!               corresponding angles

            call mshpavmncoor ( numnodes, bndcoorold(1,ibndnr), crd, &
                                bndcoornew(1,ibndnr), &
                                bndangle(1,ibndnr), nowbnd, maxbounds, &
                                ibndnr, layer, maxlen )
            if ( ierror.ne.0 ) go to 1000

         end if  ! ( .not. zero )

      end do  ! ibndnr = 1, nloop

1000  call erclos ( 'mshpav2fillbnd' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpav2fillbnd'

      end if

      end
