      subroutine mshpavcleargrd ( grid, nx, ny, gridfilled, &
                                  lengridfilled )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-07-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Clear array grid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, grid(0:nx,0:ny), lengridfilled, gridfilled(*)

!     grid          i/o   Work array of size at most 101 x 101 to store
!                         information about the boundary in relation with
!                         arrays gridinfo and rgridinfo
!                         The actual size is (0:nx, 0:ny) and (ix,iy) refers
!                         to the cell ( xmin+ix*dx, ymin+iy*dy) x
!                         ( xmin+(ix+1)*dx, ymin+(iy+1)*dy)
!                         if grid(ix,iy) = 0, no boundary node is stored in cell
!                         (ix,iy), if > 0 information about the boundary cells
!                         is stored in array grdinfo from position
!                         grid(ix,iy)
!     gridfilled    i/o   Work array of size 2 x lengridfilled, indicating which
!                         cells in array grid have value larger than zero
!                         gridfilled(1,i) contains ix and
!                         gridfilled(2,i) contains iy for a cell with
!                         grid(ix,iy) > 0
!     lengridfilled  i    Contains the declared length of array gridfilled
!     nx             i    number of cells in x direction in array grid
!     ny             i    number of cells in y direction in array grid
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavcleargrd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavcleargrd'

      end if

!     --- Clear array grid

      do i = 0, nx
         do j = 0, ny
            grid(i,j) = 0
         end do
      end do

!     --- Clear array gridfilled

      do i = 1, lengridfilled
         gridfilled(i) = 0
      end do
      call erclos ( 'mshpavcleargrd' )
      if ( debug ) write(irefwr,*) 'End mshpavcleargrd'

      end
