      subroutine mshpavcopy ( newbnd, bndcoornew, crd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill new coordinates in bndcoornew by copying from crd
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: newbnd(0:*)
      double precision, intent(in) :: crd(4,*)
      double precision, intent(inout) :: bndcoornew(4,*)

!     bndcoornew    i/o   Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     newbnd         i    temp array containing the boundary nodes for one loop
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numnodes, i

!     i              Counting variable
!     numnodes       Number of nodal points in array bndcoornew
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavcopy' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavcopy'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numnodes = newbnd(0)
      do i = 1, numnodes
         bndcoornew(1:4,i) = crd(1:4, newbnd(i))
      end do  ! i = 1, numnodes

1000  call erclos ( 'mshpavcopy' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( bndcoornew, numnodes, 4, ' bndcoornew' )
         write(irefwr,*) 'End mshpavcopy'

      end if  ! ( debug )

      end subroutine mshpavcopy
