      subroutine mshpavdelcolel ( kmeshc, nelprev, nel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-05-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove collapsed elements in kmeshc by setting corresponding entries to 0
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
!     paver
!     delete
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(3,*), nel, nelprev

!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!                         Entries corresponding to collapsed elements are set
!                         to 0
!     nel            i    number of elements in kmeshc
!     nelprev        i    Only the elements with number larger than nelprev
!                         are checked
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, inode1, inode2, inode3

!     ielem          Element number
!     inode1         First node in triangle
!     inode2         Second node in triangle
!     inode3         Third node in triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all elements with number larger than nelprev do
!        if ( element is collapsed ) then
!           set kmeshc to 0 for this element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavdelcolel' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavdelcolel'

      end if
      if ( ierror.ne.0 ) go to 1000

      do ielem = nelprev+1, nel

         inode1 = kmeshc(1,ielem)
         inode2 = kmeshc(2, ielem )
         inode3 = kmeshc(3, ielem )
         if ( inode1.eq.inode2 .or. inode1.eq.inode3 .or. &
              inode2.eq.inode3 ) then

!        --- Element is collapsed, so remove

            kmeshc(1,ielem) = 0
            kmeshc(2,ielem) = 0
            kmeshc(3,ielem) = 0

         end if  !( inode1.eq.inode2 .or. inode1.eq.inode3 ... )

      end do  ! ielem = nelprev+1, nel

1000  call erclos ( 'mshpavdelcolel' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavdelcolel'

      end if

      end
