      subroutine mshpavedgelen ( bndcoornew, numnodes, nodenrs, &
                                 imark, edgelength, ratio )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute edgelength of edges on new boundary if corresponding to new points
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, nodenrs(*), imark(*)
      double precision bndcoornew(4,*), edgelength(*), ratio(*)

!     bndcoornew     i    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     edgelength     o    Contains the length of the edges along the present
!                         boundary (squared)
!     imark          i    Array indicating for each point in a boundary loop
!                         how it should be treated in the following way
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     nodenrs        i    Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     numnodes       i    Number of nodal points in array nodes
!     ratio          o    Contains the ratio of the length of the edge and the
!                         length of the previous edge along the boundary
!                         (squared)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iprev, inode, ind
      double precision xpnew, ypnew, xnew, ynew, dsnew, edgelen, &
                       edgeprev

!     dsnew          Length of edge (squared)
!     edgelen        Length of edge (squared)
!     edgeprev       Length of previous edge (squared)
!     i              Counting variable
!     ind            sequence number of edge
!     inode          Node sequence number
!     iprev          Previous value used
!     xnew           New x-coordinate
!     xpnew          Previous x-coordinate
!     ynew           New y-coordinate
!     ypnew          Previous y-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store edge length in array edgelength (squared)
!     Compute ratio of edge length
!     (1 if length is 0, because node may  not be removed
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavedgelen' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavedgelen'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Store edge length in array edgelength (squared)

      xpnew = bndcoornew(1,1)
      ypnew = bndcoornew(2,1)
      iprev = nodenrs(1)
      do i = 1, numnodes

         ind = i+1
         if ( ind.gt.numnodes ) ind = 1
         inode = nodenrs(ind)
         xnew = bndcoornew(1,ind)
         ynew = bndcoornew(2,ind)
         if ( imark(ind).eq.0 .or. iprev.eq.inode ) then

!        --- previous point on old boundary or element has been skipped
!            Set edgelength equal to 0

            edgelength(i) = 0d0

         else

!        --- Standard case

            dsnew = (xnew-xpnew)**2+(ynew-ypnew)**2
            edgelength(i) = dsnew

         end if  ! ( imark(ind).eq.0 .or. iprev.eq.inode )
         xpnew = xnew
         ypnew = ynew
         iprev = inode

      end do  ! i = 1, numnodes

!     --- Compute ratio of edge length
!         (1 if length is 0, because node may  not be removed

      edgeprev = edgelength(numnodes)
      do i = 1, numnodes
         edgelen = edgelength(i)
         if ( edgelen.le.0d0 .or. edgeprev.le.0d0 ) then
            ratio(i) = 1d0
         else
            ratio(i) = edgelen/edgeprev
         end if
         edgeprev = edgelen
      end do  ! i = 1, numnodes

1000  call erclos ( 'mshpavedgelen' )

      if ( debug ) then

!     --- Debug information

         call prinrl ( edgelength, numnodes, 'edgelength' )
         write(irefwr,*) 'End mshpavedgelen'

      end if

      end
