      subroutine mshpavmncoor ( inbndnod, bndcoorold, crd, bndcoornew, &
                                bndangle, nowbnd, maxbounds, ibndnr, &
                                layer, maxlen )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-07-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the new coordinates that arise if we create a new layer of
!     elements along one boundary loop without actually computing the
!     elements or taking into account angles
!     Also the angles are stored
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inbndnod, maxbounds, nowbnd(maxbounds,*), ibndnr, layer
      double precision  bndcoorold(2,*), crd(4,*), bndcoornew(4,*), &
                        bndangle(*), maxlen

!     bndangle       o    Array containg the angles corresponding to all nodes
!                         of a boundary loop
!     bndcoornew     o    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    real work array containing the
!                         boundary coordinates of the i'th boundary
!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     ibndnr         i    i'th boundary in nowbnd being looked at
!     inbndnod       i    number of coordinates of nodes in bndcoorold
!     layer          i    Present layer sequence number
!     maxbounds      i    maximum loops in nowbnd in paver
!     maxlen         i    Maximum length of edges in the surface
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (maxbounds,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ind, i, indnr
      double precision prevcoor(2), nodecoor(2), nextcoor(2), t1(2), &
                       t2(2), n1(2), n2(2), norm1, norm2, lng, n(2), &
                       norm, fact, hi, factor

!     fact           factor with which the thickness grows
!     factor         Maximum length of an edge corrected by sqrt(2)
!     hi             height of the mesh in node indnr
!     i              Counting variable
!     ind            sequence number of point along boundary loop
!     indnr          node number of the i'th node in ibndnr
!     lng            multiplication factor
!     n              average normal
!     n1             normal on first tangent
!     n2             normal on second tangent
!     nextcoor       contains the coordinates of the second point
!     nodecoor       contains the coordinates of the first point
!     norm           length of n
!     norm1          length of n1
!     norm2          length of n2
!     prevcoor       contains the coordinates of the previous point
!     t1             First tangential vector
!     t2             Second tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store coordinate of last point in prevcoor
!     for all the edges on the boundary stored in bndcoorold
!        Store nodes of actual edge in nodecoor and nextcoor
!        Compute n1, n2, t1 and t2
!        Compute the angle
!        Compute new coordinates
!        Store angle and coordinates into arrays bndangle and bndcoornew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmncoor' )
      debug = .false.
      if ( layer>=10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmncoor'
         call prinrl1 ( bndcoorold, inbndnod, 2, 'bdncri' )
         call prinin ( nowbnd, ibndnr, 'nowbnd' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      factor = maxlen/1.4d0

!     --- Store coordinate of last point in prevcoor

      prevcoor(1) = bndcoorold(1,inbndnod)
      prevcoor(2) = bndcoorold(2,inbndnod)

!     --- for all the edges on the boundary stored in bndcoorold

      do ind = 1, inbndnod

!     --- Store nodes of actual edge in nodecoor and nextcoor
!         Compute n1, n2, t1 and t2
!         t1 is the tangential vector from previous node to present node
!         t2 is the tangential vector from present node to next node
!         n1 and n2 the corresponding normals

         do i = 1, 2
            nodecoor(i) = bndcoorold(i,ind)
            nextcoor(i) = bndcoorold(i,ind+1)
            t1(i) = nodecoor(i) - prevcoor(i)
            t2(i) = nextcoor(i) - nodecoor(i)
         end do
         n1(1) = -t1(2)
         n1(2) =  t1(1)
         n2(1) = -t2(2)
         n2(2) =  t2(1)
         norm1 = 1d0/(sqrt(n1(1)**2+n1(2)**2))
         norm2 = 1d0/(sqrt(n2(1)**2+n2(2)**2))
         do i = 1, 2
            n1(i) = n1(i) * norm1
            t1(i) = t1(i) * norm1
            n2(i) = n2(i) * norm2
            t2(i) = t2(i) * norm2
         end do

!        --- Store the average norm in n

         do i = 1, 2
            n(i) = n1(i) + n2(i)
         end do
         norm = 1d0/sqrt(n(1)**2+n(2)**2)
         n(1) = n(1)*norm
         n(2) = n(2)*norm

!        --- Compute a multiplication factor for the line perpendicular
!            to the boundary
!            This factor is equal to 1 in case of a straight line, &
!            but more than one for a curved part
!            The maximum value is 2

         lng = 1d0/max(0.5d0,(n1(1)*n(1)+n1(2)*n(2)))

!        --- Store inner product of n1 and t2 in angle
!            So actually the angle in this point is equal to angle + 90 degrees
!            In fact this is the cosine of the angle, hence 0 for 90 degrees
!            > 0 for angles less than 90 degrees and < 0 for angles more than
!            90 degrees (120 degrees: -0.5)

         bndangle(ind) = n1(1)*t2(1)+n1(2)*t2(2)
         if ( debug .and. layer>=10000 ) then

!        --- Debug information

            write(irefwr,1) 'ind', ind
            write(irefwr,2) 'lng, n1, n2, n', lng, n1,n2,n
            write(irefwr,2) 't1, t2', t1, t2
            write(irefwr,2) 'angle', bndangle(ind)

         end if

         indnr = abs(nowbnd(ibndnr,ind+1))  ! node number of next node
         fact = crd(4,indnr)
         hi   = min ( crd(3,indnr), factor )    ! width corresponding to indnr
         if ( debug .and. layer>=10000 ) then
            write(irefwr,1) 'indnr', indnr
            write(irefwr,2) 'fact, hi', fact, hi
         end if  ! ( debug .and. layer>=10000 )

!        --- see whether a new node will be created, thick > 0

         if ( hi>0d0 ) then

!        --- create new coordinates
!            The new node is created along the normal pointing from the
!            present point with a length adapted to hi

            bndcoornew(1,ind) = nodecoor(1) + (hi*n(1))*lng
            bndcoornew(2,ind) = nodecoor(2) + (hi*n(2))*lng
            bndcoornew(3,ind) = min(hi*fact, factor)
            bndcoornew(4,ind) = fact

         else

!        --- reuse the old node

            bndcoornew(1,ind) = nodecoor(1)
            bndcoornew(2,ind) = nodecoor(2)
            bndcoornew(3,ind) = hi
            bndcoornew(4,ind) = fact

         end if
         if ( debug .and. layer>=10000 ) then

!        --- Debug information

            write(irefwr,1) 'ind', ind
            write(irefwr,2) 'nodecoor', nodecoor
            write(irefwr,2) 'bndcoornew ', bndcoornew(1,ind), &
                             bndcoornew(2,ind)

         end if

!        --- Copy last created coordinates in prevcoor

         prevcoor(1) = nodecoor(1)
         prevcoor(2) = nodecoor(2)

      end do

      call erclos ( 'mshpavmncoor' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( bndcoornew, inbndnod, 4, 'bndcoornew' )
         call prinrl ( bndangle, inbndnod,  'bndangle' )
         write(irefwr,*) 'End mshpavmncoor'

      end if

      end
