      subroutine mshpavmninter0 ( numnodes, bndcoornew, imark, nodenrs, &
                                  layer, bndcoorold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-08-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Smooth new boundary in the following way
!     if the ratio between new edge and previous edge is less than 0.83 and
!     the edge itself has length less than 0.715 or more than 1.4 times one of
!     its neighbours the edge is removed provided the new boundary is not too
!     close to the previous boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, imark(*), nodenrs(*), layer
      double precision bndcoornew(4,*), bndcoorold(2,*)

!     bndcoornew    i/o   Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    Contains the coordinates of the previous (old)
!                         boundary during the paver process
!     imark          i    Array indicating for each point in a boundary loop
!                         how it should be treated in the following way:
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     layer          i    Present layer sequence number
!     nodenrs       i/o   Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     numnodes       i    number of coordinates of nodes in bndcoorold
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iremove(0:numnodes+1), icount
      double precision ratio(0:numnodes+1), ratioedge(0:numnodes+1), &
                       edgelength(0:numnodes+1), sqrratio(0:numnodes+1)

!     edgelength     Contains the length of the edges along the present
!                    boundary (squared)
!     i              Counting variable
!     icount         Number of nodes that may be removed (actual length of
!                    iremove)
!     iremove        Contains node numbers of nodes that may be removed
!     ratio          Contains the ratio between edges on old and new boundary
!                    (squared)
!     ratioedge      Contains the ratio of the length of the edge and the
!                    length of the previous edge along the boundary
!                    (squared)
!     sqrratio       Contains sqrt(ratio)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCONTREAL    Check control integer around a real work array
!     ERRCONTROL     Check control integers around integer array
!     MSHPAVEDGELEN  Compute edgelength of edges on new boundary if
!                    corresponding to new points
!     MSHPAVMNRATIO  Compute ratio between edges on old and new boundary
!     MSHPAVMNREMPNT Remove point from new boundary if possible
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store ratio of subsequent distances in array ratio
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmninter0' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmninter0'
         write(irefwr,*) 'layer ', layer
         call prinin ( nodenrs, numnodes, 'nodenrs' )
         call prinin ( imark, numnodes, 'imark' )

      end if

!     --- Set control integers

      iremove(0) = 1009
      iremove(numnodes+1) = 1009
      ratio(0) = 1006d0
      ratio(numnodes+1) = 1006d0
      ratioedge(0) = 1007d0
      ratioedge(numnodes+1) = 1007d0
      edgelength(0) = 1008d0
      edgelength(numnodes+1) = 1008d0
      sqrratio(0) = 1009d0
      sqrratio(numnodes+1) = 1009d0

!     --- Store ratio of subsequent distances in array ratio (squared)

      call mshpavmnratio ( bndcoorold,  bndcoornew, numnodes, &
                           nodenrs, imark, ratio(1) )
      if ( debug ) then
         do i = 1, numnodes
           sqrratio(i) = sqrt(ratio(i))
         end do
         call prinrl ( sqrratio, numnodes, 'ratio' )
      end if

!     --- Compute edge length along new boundary

      call mshpavedgelen ( bndcoornew, numnodes, nodenrs, &
                           imark, edgelength(1), ratioedge(1) )
      if ( debug ) then
         call prinrl ( edgelength(1), numnodes, 'edgelength^2' )
         call prinrl ( ratioedge(1), numnodes, 'ratioedge^2' )
      end if

!     --- Store node numbers of nodes that are candidate to be removed
!         This is the case if the ratio between old and new edge is less than
!         0.83 and the edge itself has length less than 0.715 or more than 1.4
!         times one of its neighbours

      icount = 0
      do i = 1, numnodes
         if ( ratio(i).lt.0.7d0 .and. (ratioedge(i).lt.0.5d0 .or. &
              ratioedge(i).gt.2d0) ) then
            icount = icount+1
            iremove(icount) = i
         end if
      end do  ! i = 1, numnodes
      if ( debug .and. icount.gt.0 ) then
         call prinin ( iremove(1), icount, 'iremove' )
      end if

!     --- Try to remove the nodes stored in iremove.
!         This is only allowed if the "new" edge is not close to the old one

      call mshpavmnrempnt ( iremove(1), icount, numnodes, bndcoornew, &
                            nodenrs, layer, bndcoorold, &
                            edgelength(1) )

      call erclos ( 'mshpavmninter0' )

!     --- Check control integers

      call errcontrol ( iremove(0), iremove(numnodes+1), 1009, &
                        'iremove' )
      call errcontreal ( ratio(0), ratio(numnodes+1), &
                         1006d0, 'ratio' )
      call errcontreal ( ratioedge(0), ratioedge(numnodes+1), &
                         1007d0, 'ratioedge' )
      call errcontreal ( edgelength(0), edgelength(numnodes+1), &
                         1008d0, 'edgelength' )
      call errcontreal ( sqrratio(0), sqrratio(numnodes+1), &
                         1009d0, 'sqrratio' )

      if ( debug ) then

!     --- Debug information

         call prinin ( nodenrs, numnodes, 'nodenrs' )
         write(irefwr,*) 'End mshpavmninter0'

      end if

      end
