      subroutine mshpavmninter1 ( numnodes, bndcoorold, bndcoornew, crd, &
                                  bndangle, nowbnd, maxbounds, &
                                  ibndnr, imarkinters )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-08-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     mark all lines from old to new point if they intersect with
!     previous line and next line
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, maxbounds, nowbnd(maxbounds,*), ibndnr, &
              imarkinters(2,*)
      double precision bndcoorold(2,*), bndcoornew(4,*), bndangle(*), &
                       crd(4,*)

!     bndangle       i    Array containg the angles corresponding to all nodes
!                         of a boundary loop
!     bndcoornew     i    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    real work array containing the
!                         boundary coordinates of the i'th boundary
!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     ibndnr         i    i'th boundary in nowbnd being looked at
!     imarkinters    o    array of size 2 x numnodes indicating if two
!                         lines from old boundary to new boundary intersect
!                         imarkinters(1,i) > 0 means that the lines i-1 and i
!                         intersect (0 not)
!                         imarkinters(2,i) > 0 means that the lines i and i+1
!                         intersect (0 not)
!     maxbounds      i    maximum loops in nowbnd in paver
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (maxbounds,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     numnodes       i    number of coordinates of nodes in bndcoorold
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, ind, jnd
      double precision x1(2), x2(2), x3(2), x4(2), hi, angle, &
                       fact1, fact2
      logical zero

!     angle          angle between two tangents in a boundary node
!     fact1          multiplication factor
!     fact2          multiplication factor
!     hi             height of the mesh in node indnr
!     ind            Sequence number along boundary loop
!     inode          Node sequence number
!     jnd            jth node in jbndnr
!     x1             coordinates of first point
!     x2             coordinates of second point
!     x3             coordinates of third point
!     x4             coordinates of foruth point
!     zero           if true the new line has length zero
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCROSSLINE1  Find the intersection of the lines x1-x2 and x3-x4
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmninter1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmninter1'

      end if

!     --- Clear array imarkinters

      do ind = 1, numnodes
         imarkinters(1,ind) = 0
         imarkinters(2,ind) = 0
      end do

!     --- First mark all lines from old to new point if they intersect with
!         previous line and next line
!         Store coordinates of last point in x1 (old) and x2 (new)

      x1(1) = bndcoorold(1,numnodes)
      x1(2) = bndcoorold(2,numnodes)
      x2(1) = bndcoornew(1,numnodes)
      x2(2) = bndcoornew(2,numnodes)
      inode = abs(nowbnd(ibndnr,numnodes))  ! node number of last node
      hi = crd(3,inode)
      if ( hi.gt.0d0 ) then
         zero = .false.
      else
         zero = .true.
      end if

!     --- Loop over all nodes on the boundary part

      jnd = numnodes
      do ind = 1, numnodes
         inode = abs(nowbnd(ibndnr,ind+1))  ! node number of node
         hi = crd(3,inode)
         if ( hi.le.0d0 ) then

!        --- hi = 0, imark(ind) := 0

            zero = .true.

         else  ! ( hi.gt.0d0 )

!        --- hi > 0, check intersections and angles

            angle = max ( bndangle(jnd), bndangle(ind) )
            if ( angle.gt.0.0001d0 ) then

!           --- Convex at most one new point
!               First check if neighbouring lines from old to new point
!               intersect
!               Start with previous line to present one

               if ( zero ) then

!              --- Previous line has length zero: no intersection

                  x1(1) = bndcoorold(1,ind)
                  x1(2) = bndcoorold(2,ind)
                  x2(1) = x1(1)
                  x2(2) = x1(2)

               else  ! ( not zero )

!              --- Previous line has length more than zero: check intersection

                  x3(1) = bndcoorold(1,ind)
                  x3(2) = bndcoorold(2,ind)
                  x4(1) = bndcoornew(1,ind)
                  x4(2) = bndcoornew(2,ind)
                  call mshcrossline1 ( x1, x2, x3, x4, fact1, fact2, &
                                       sqreps )
                  if ( debug ) write(irefwr,*) 'ind, fact1, fact2 ', &
                                                ind, fact1, fact2

                  if ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps ) then

!                 --- The lines x1-x2 and x3-x4 intersect

                     imarkinters(1,ind) = 1
                     imarkinters(2,jnd) = 1

                  end if  ! ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps )
                  x1(1) = x3(1)
                  x1(2) = x3(2)
                  x2(1) = x4(1)
                  x2(2) = x4(2)

               end if  ! ( zero )
               zero = .false.

            else  ! ( angle.le.0.0001d0 )

!           --- Concave region, no intersection

               imarkinters(1,ind) = 0
               imarkinters(2,jnd) = 0
               x1(1) = bndcoorold(1,ind)
               x1(2) = bndcoorold(2,ind)
               x2(1) = bndcoornew(1,ind)
               x2(2) = bndcoornew(2,ind)
               zero = .false.

            end if  ! ( angle.gt.0.0001d0 )

         end if  ! ( hi.le.0d0 )
         jnd = ind

      end do  ! ind = 1, numnodes

      call erclos ( 'mshpavmninter1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavmninter1'

      end if

      end
