      subroutine mshpavmninter2 ( numnodes, bndcoorold, bndcoornew, crd, &
                                  bndangle, nowbnd, maxbounds, &
                                  ibndnr, imark, ncrd, nodenrs, &
                                  imarkinters, layer, nloop, markboun, &
                                  lnnowbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-08-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array imark for all nodes along the boundary
!     If necessary adapt bndcoornew and nodenrs
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, maxbounds, nowbnd(maxbounds,*), ibndnr, nloop, &
              imark(*), ncrd, nodenrs(*), imarkinters(2,*), layer, &
              lnnowbnd, markboun(lnnowbnd,nloop,*)
      double precision bndcoorold(2,*), bndcoornew(4,*), bndangle(*), &
                       crd(4,*)

!     bndangle       i    Array containg the angles corresponding to all nodes
!                         of a boundary loop
!     bndcoornew     i    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    real work array containing the
!                         boundary coordinates of the i'th boundary
!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     ibndnr         i    i'th boundary in nowbnd being looked at
!     imark          o    Array indicating for each point in a boundary loop
!                         how it should be treated in the following way:
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     imarkinters    i    array of size 2 x numnodes indicating if two
!                         lines from old boundary to new boundary intersect
!                         imarkinters(1,i) > 0 means that the lines i-1 and i
!                         intersect (0 not)
!                         imarkinters(2,i) > 0 means that the lines i and i+1
!                         intersect (0 not)
!     layer          i    Present layer sequence number
!     lnnowbnd       i    Declared length of second index of array nowbnd
!     markboun       i    Three diemsional array of size lnnowbnd x nloop x 3
!                         with the following contents
!                         markboun(i,j,k) refers to relative node i along the
!                         j-th boundary in nowbnd
!                         k has the following meaning.
!                         if Pos. 1 = 0 then the node is free otherwise (1)
!                         it is clcoe to another node.
!                         In that case position 2 contains the boundary number
!                         of that other node and position 3 the relative
!                         position  of the node along that boundary
!     maxbounds      i    maximum loops in nowbnd in paver
!     ncrd          i/o   Last node number used
!     nloop          i    number of boundaries in nowbnd
!     nodenrs        o    Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (maxbounds,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     numnodes       i    number of coordinates of nodes in bndcoorold
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, ntri, ind, jnd, i, icount, ncrdprev
      double precision hi, n(2), len, angle, n1(2), x, y

!     angle          angle between two tangents in a boundary node
!     hi             height of the mesh in node indnr
!     i              Counting variable
!     icount         Counter
!     ind            Sequence number along boundary loop
!     inode          Node sequence number
!     jnd            jth node in jbndnr
!     len            length of edge
!     n              average normal
!     n1             normal on an edge
!     ncrdprev       Number of nodes in crd before the call of this subroutine
!     ntri           number of extra triangles
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmninter2' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmninter2'
         write(irefwr,1) 'layer, ncrd ', layer, ncrd
         call prinin1 ( imarkinters, numnodes, 2, 'imarkinters' )
         call prinrl1 ( bndcoornew, numnodes, 4, 'bndcoornew' )
  1      format ( a, 1x, (10i6) )

      end if

!     --- Loop over all nodes in the boundary part

      jnd = numnodes
      ncrdprev = ncrd
      do ind = 1, numnodes
         inode = abs(nowbnd(ibndnr,ind+1))  ! node number of node
         hi = crd(3,inode)
         if ( hi.le.0d0 .or. markboun(ind,ibndnr,1).gt.0 ) then

!        --- hi = 0, imark(ind) := 0

            imark(ind) = 0
            nodenrs(ind) = inode
            bndcoornew(1,ind) = bndcoorold(1,ind)
            bndcoornew(2,ind) = bndcoorold(2,ind)
            bndcoornew(3,ind) = 0d0

         else  ! ( hi>0 and markboun(ind,ibndnr,1)=0 )

!        --- hi > 0, check intersections and angles

            angle = bndangle(ind)
            if ( angle.gt.0.0001d0 ) then

!           --- Convex at most one new point
!               Check if there is an intersection in this point

               if ( imarkinters(1,ind).eq.0 .and. &
                    imarkinters(2,ind).eq.0 ) then

!              --- No intersection point, one new point

                  imark(ind) = 1
                  ncrd = ncrd+1
                  nodenrs(ind) = ncrd

               else  ! not ( imarkinters(1,ind).eq.0 ...

!              --- Complicated situation, there is an intersection point
!                  If previous point is skipped, this point must be used
!                  If previous point is not skipped and there is an
!                  intersection with previous point: skip point
!                  If previous point is not skipped and the angle is smaller
!                  than next angle skip, otherwise do not skip

                  if ( ind.eq.1 ) then

!                 --- First point, special situation

                     if ( imarkinters(1,ind).ne.0 ) then

!                    --- Intersection with previous point, mark to be skipped

                        imark(ind) = 1
                        nodenrs(ind) = 0

                     else

!                    --- No intersection with previous point

                        imark(ind) = 1
                        ncrd = ncrd+1
                        nodenrs(ind) = ncrd

                     end if

                  else  ! not ( ind.eq.1 )

!                 --- Other point, standard

                     if ( imarkinters(1,ind).ne.0 ) then

!                    --- there is an intersection with previous point skip

                        imark(ind) = 1
                        nodenrs(ind) = nodenrs(ind-1)

                     else

!                    --- intersection with next point:
!                        check angles, skip smallest one

                        if ( bndangle(ind).lt.bndangle(ind+1) ) then
                           imark(ind) = 1
                           ncrd = ncrd+1
                        else
                           imark(ind) = 1
                           ncrd = ncrd+1
                        end if
                        nodenrs(ind) = ncrd

                     end if

                  end if  ! ( ind.eq.1 )

               end if  ! ( imarkinters(1,ind).eq.0 ...

            else  ! not ( angle.gt.0.0001d0 )

!           --- Concave region, no intersection, but possibly more points
!               First we compute the angle between n1 and n2
!               The actual angle is equal to this one + pi

               n(1) = bndcoornew(1,ind)-bndcoorold(1,ind)
               n(2) = bndcoornew(2,ind)-bndcoorold(2,ind)
               len = 1d0/sqrt(n(1)**2+n(2)**2)
               n(1) = n(1)*len
               n(2) = n(2)*len
               n1(1) = -(bndcoorold(2,ind)-bndcoorold(2,jnd))  ! -t1(2)
               n1(2) =  (bndcoorold(1,ind)-bndcoorold(1,jnd))  !  t1(1)
               len = 1d0/sqrt(n1(1)**2+n1(2)**2)
               n1(1) = n1(1)*len
               n1(2) = n1(2)*len
               angle = max(n(1)*n1(1)+n(2)*n1(2),-1d0)
               angle = min(angle,1d0)
               angle = acos(angle)

!              --- We try to subdivide this part into angles of approximately
!                  pi/6 radians, hence we get two angles of 90 degrees and the
!                  rest approximately 60 degrees

               ntri = 1+int(angle/pi*6d0)
               imark(ind) = max(1,ntri)
               ncrd = ncrd+1
               nodenrs(ind) = ncrd

            end if  ! ( angle.gt.0.0001d0 )

         end if  ! ( hi.gt.0d0 )

         jnd = ind

      end do  ! ind = 1, numnodes

      if ( nodenrs(numnodes).eq.0 ) then

!     --- All new nodes 0, set all to ncrd+1

         ncrd = ncrd+1
         nodenrs(numnodes) = ncrd

      else if ( nodenrs(1).eq.0 ) then

!     --- nodenrs(1) = 0, hence first and last node must be the same
!         in order to have the first node number the smallest one we
!         perform the following trick:
!         All node numbers equal to ncrd are made equal to 0, then
!         All node numbers larger than 0 are raised by one
!         Finally all node numbers equal to 0 are made equal to ncrdprev+1

         do i = 1, numnodes
            if ( nodenrs(i).eq.ncrd ) nodenrs(i) = 0
            if ( nodenrs(i).gt.ncrdprev ) nodenrs(i) = nodenrs(i)+1
         end do
         do i = 1, numnodes
            if ( nodenrs(i).eq.0 ) nodenrs(i) = ncrdprev+1
         end do

      end if  ! ( nodenrs(numnodes).eq.0 )
      if (debug) call prinin ( nodenrs, numnodes, 'nodenrs/2' )

!     --- Compute mean value of coordinates in case of same nodal point numbers

      ind = 1                  ! entry along boundary
      inode = nodenrs(1)       ! last new node number
      nodenrs(numnodes+1) = 0  ! the position after the last entry is made
                               ! equal to zero for ease of programming
      do while ( ind.lt.numnodes )

         icount = 1  ! number of entries with the same node number

         if ( nodenrs(ind+icount).eq.inode ) then

!        --- Subsequent entries have the same node nrs
!            Check if there are more entries with this same node number

            icount = icount+1
            do while ( nodenrs(ind+icount).eq.inode )
               icount = icount+1
            end do

!           --- icount is the number of entries with the same node number
!               compute the mean value of the coordinates and store in x, y

            x = 0d0
            y = 0d0
            do i = 0, icount-1
               x = x + bndcoornew(1,ind+i)
               y = y + bndcoornew(2,ind+i)
            end do
            x = x/icount
            y = y/icount

!           --- Replace the coordinates in these points by their mean value

            do i = 0, icount-1
               bndcoornew(1,ind+i) = x
               bndcoornew(2,ind+i) = y
            end do

         end if  ! ( nodenrs(ind+icount).eq.inode )
         ind = ind+icount
         inode = nodenrs(ind)

      end do  ! while ( ind.lt.numnodes )

      call erclos ( 'mshpavmninter2' )

      if ( debug ) then

!     --- Debug information

         call prinin ( imark, numnodes, 'imark' )
         call prinin ( nodenrs, numnodes, 'nodenrs' )
         call prinrl1 ( bndcoornew, numnodes, 4, 'bndcoornew' )
         write(irefwr,*) 'End mshpavmninter2'

      end if

      end
