      subroutine mshpavmnratio ( bndcoorold,  bndcoornew, numnodes, &
                                 nodenrs, imark, ratio )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute ratio between edges on old and new boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     boundary
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, nodenrs(*), imark(*)
      double precision bndcoorold(2,*), bndcoornew(4,*), ratio(*)

!     bndcoornew     i    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    Contains the coordinates of the previous (old)
!                         boundary during the paver process
!     imark          i    Array indicating for each point in a boundary loop
!                         how it should be treated in the following way:
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     nodenrs        i    Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     numnodes       i    Number of nodal points in array nodes
!     ratio          o    Contains the ratio between edges on old and new
!                         boundary (squared)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iprev, inode, ind
      double precision xpold, ypold, xpnew, ypnew, xold, yold, xnew, &
                       ynew, dsold, dsnew

!     dsnew          Length of edge on new boundary (squared)
!     dsold          Length of edge on old boundary (squared)
!     i              Counting variable
!     ind            Sequence number along boundary loop
!     inode          Node sequence number
!     iprev          Previous node number
!     xnew           x-coordinate along new boundary
!     xold           x-coordinate along old boundary
!     xpnew          Previous x-coordinate along new boundary
!     xpold          Previous x-coordinate along old boundary
!     ynew           y-coordinate along new boundary
!     yold           y-coordinate along old boundary
!     ypnew          Previous y-coordinate along new boundary
!     ypold          Previous y-coordinate along old boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmnratio' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmnratio'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Store ratio of subsequent distances in array ratio (squared)

      xpold = bndcoorold(1,1)
      ypold = bndcoorold(2,1)
      xpnew = bndcoornew(1,1)
      ypnew = bndcoornew(2,1)
      iprev = nodenrs(1)
      do i = 1, numnodes

         ind = i+1
         if ( ind.gt.numnodes ) ind = 1
         xold = bndcoorold(1,ind)
         yold = bndcoorold(2,ind)
         inode = nodenrs(ind)
         xnew = bndcoornew(1,ind)
         ynew = bndcoornew(2,ind)
         if ( imark(ind).eq.0 .or. iprev.eq.inode ) then

!        --- previous point on old boundary or element has been skipped
!            Set ratio equal to 1

            ratio(i) = 1d0

         else

!        --- Standard case

            dsold = (xold-xpold)**2+(yold-ypold)**2
            dsnew = (xnew-xpnew)**2+(ynew-ypnew)**2
            ratio(i) = dsnew/dsold

         end if  ! ( imark(ind).eq.0 .or. iprev.eq.inode )
         xpold = xold
         ypold = yold
         xpnew = xnew
         ypnew = ynew
         iprev = inode

      end do  ! i = 1, numnodes

1000  call erclos ( 'mshpavmnratio' )

      if ( debug ) then

!     --- Debug information

         call prinrl ( ratio, numnodes, 'ratio' )
         write(irefwr,*) 'End mshpavmnratio'

      end if

      end
