      subroutine mshpavmnsmoot0 ( crd, layer, ibndnr, nel, &
                                  kmeshc, newbnd, nelprev, &
                                  deleted, imark, ielem, iseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if element may be removed without problem
!     This means that we want to identify inode2 and inode3 and replace
!     them by a new coordinate, which is the mean value of the two
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     smoothing
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer layer, ibndnr, nel, kmeshc(3,*), &
              newbnd(*), imark(*), nelprev, ielem, iseq
      double precision crd(4,*)
      logical deleted

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     deleted       i/o   If true an element has been deleted
!     ibndnr         i    i'th boundary in nowbnd
!     ielem          i    Element number
!     imark          i    Array indicating for each point in a boundary loop
!                         how it should be treated in the following way:
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     iseq           i    Sequence number of node inode1 in kmeshc for element
!                         ielem (inode1 corresponds to old boundary)
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     layer          i    Present layer sequence number
!     nel            i    number of elements in the topology
!     nelprev        i    Number of old elements stored in kmeshc
!     newbnd         i    temp array containing the boundary nodes for one loop
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode1, inode2, inode3, iplus(4), jelem, iseq1, iseq2, &
              jnode1, jnode2, jnode3, i, inode
      double precision x(2), angle, area
      logical remove

!     angle          angle between two edges
!     area           Area of element
!     i              Counting variable
!     inode          Node sequence number
!     inode1         First node in triangle
!     inode2         Second node in triangle
!     inode3         First node in second triangle
!     iplus          Array containing i+1 modulo 3
!     iseq1          Sequence number of inode1 in element jelem
!     iseq2          Sequence number of inode2 in element jelem
!     jelem          element number (not ielem)
!     jnode1         first node in element jelem
!     jnode2         second node in element jelem
!     jnode3         thirs node in element jelem
!     remove         if true element is removed
!     x              co-ordinates of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHANGLE       Compute the cosine of the angle between the nodes with
!                    coordinates x1, x2 and x3, where the angle is defined
!                    as x3-x1-x2
!     MSHAREA        Compute the area of the triangle spanned by x1-x2-x3
!                    The sign of the are defines the direction
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2477   Element number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!     remove := true
!     Get three nodes of element
!     Compute mean value of nodes inode2 and inode3
!     For all new elements jelem not equal to ielem do
!        if ( both inode2 and inode3 in jelem ) then
!           if ( angle of jelem from old to new boundary and back is to
!                large ) then
!              remove := false
!        else if ( one of the nodes inode2 and inode3 in jelem ) then
!           if ( element can not be removed without effecting jelem ) then
!              remove := false
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iplus / 2, 3, 1, 2 /
! ======================================================================
!
      call eropen ( 'mshpavmnsmoot0' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmnsmoot0'
         write(irefwr,*) 'layer, ibndnr ', layer, ibndnr
         write(irefwr,*) 'ielem, iseq ', ielem, iseq

      end if
      if ( ierror.ne.0 ) go to 1000
      if ( ielem.eq.0 ) then
         call errint ( ielem, 1 )
         call errsub ( 2477, 1, 0, 0 )
         go to 1000
      end if
      remove = .true.

!     --- Get three nodes of element

      inode1 = kmeshc(iseq,ielem)
      inode2 = kmeshc(iplus(iseq), ielem )
      inode3 = kmeshc(iplus(iseq+1), ielem )

!     --- Compute mean value of nodes inode2 and inode3

      x(1) = 0.5d0*(crd(1,inode2)+crd(1,inode3))
      x(2) = 0.5d0*(crd(2,inode2)+crd(2,inode3))
      if ( debug ) &
         write(irefwr,*) 'inode1, inode2, inode3 ', &
                          inode1, inode2, inode3

!     --- Loop over all new elements

      jelem = nelprev
      do while ( jelem.lt.nel .and. remove )

         jelem = jelem+1
         iseq1 = 0
         iseq2 = 0
         if ( jelem.ne.ielem .and. kmeshc(1,jelem).gt.0 ) then

!        --- Element not equal to ielem

            do i = 1, 3
               if ( kmeshc(i,jelem).eq.inode2 ) iseq1 = i
               if ( kmeshc(i,jelem).eq.inode3 ) iseq2 = i
            end do  ! i = 1, 3

            if ( iseq1.gt.0 .and. iseq2.gt.0 ) then

!           --- Special case, both nodes in other triangle
!               Check the angle of that triangle

               do i = 1, 3
                  inode = kmeshc(i,jelem)
                  if ( imark(inode).le.0 ) then

!                 --- Node on old boundary, store sequence number

                     iseq = i

                  end if  !  ( imark(inode).eq.1 )

               end do  ! i = 1, 3
               jnode1 = kmeshc(iseq,jelem)
               jnode2 = kmeshc(iplus(iseq), jelem )
               jnode3 = kmeshc(iplus(iseq+1), jelem )

!              --- Compute angle jnode3-jnode1-jnode2

               call mshangle ( crd(1,jnode1), crd(1,jnode2), &
                               crd(1,jnode3), angle, 2 )
               if ( debug ) then
                  write(irefwr,*) 'iseq1, iseq2, angle ', &
                                   iseq1, iseq2, angle
                  write(irefwr,*) 'jnode1, jnode2, jnode3 ', &
                                   jnode1, jnode2, jnode3
               end if
               if ( angle.lt.0.89d0 ) then

!              --- angle too large, do not remove

                  remove = .false.

               end if

            else if ( iseq1.gt.0 .or. iseq2.gt.0 ) then

!           --- One of the nodes in triangle
!               Check the effect of replacement

               iseq = max(iseq1,iseq2)
               jnode1 = kmeshc(iseq,jelem)
               jnode2 = kmeshc(iplus(iseq), jelem )
               jnode3 = kmeshc(iplus(iseq+1), jelem )

!              --- First we test if the area remains positive

               call msharea ( x, crd(1,jnode2), &
                              crd(1,jnode3), area )
               if ( debug ) then
                  write(irefwr,*) 'iseq1, iseq2, area ', &
                                   iseq1, iseq2, area
                  write(irefwr,*) 'jnode1, jnode2, jnode3 ', &
                                   jnode1, jnode2, jnode3
               end if
               if ( area.le.0d0 ) then

!              --- area negative, do not remove

                  remove = .false.

               else  ! ( area.gt.0d0 )

!              --- Next we check the angles ( less than 135 degrees)

                  call mshangle ( x, crd(1,jnode2), &
                                  crd(1,jnode3), angle, 2 )
                  if ( debug ) write(irefwr,*) 'ange/1 ' ,angle
                  if ( angle.gt.-0.7d0 ) &
                     call mshangle ( crd(1,jnode2), x, &
                                     crd(1,jnode3), angle, 2 )
                  if ( debug ) write(irefwr,*) 'ange/2 ' ,angle
                  if ( angle.gt.-0.7d0 ) &
                     call mshangle ( crd(1,jnode3), x, &
                                     crd(1,jnode2), angle, 2 )
                  if ( debug ) write(irefwr,*) 'ange/3 ' ,angle
                  if ( angle.le.-0.7d0 ) then

!                 --- angle too large, do not remove

                     remove = .false.

                  end if  ! ( angle.le.-0.7d0 )

               end if  ! ( area.le.0d0 )

            end if  ! ( iseq1.gt.0 .and. iseq2.gt.0 )

         end if  ! ( jelem.ne.ielem )

      end do  !  while ( jelem.lt.nel .and. remove )

      if ( remove ) then

!     --- element can be removed without effecting neighbour
!         elements
!         remove element and adapt newbnd

         crd(1,inode2) = x(1)
         crd(2,inode2) = x(2)
         deleted = .true.
         do jelem = nelprev+1, nel
            if ( jelem.eq.ielem ) then

!           --- Delete element

               kmeshc(1,ielem) = 0
               kmeshc(2,ielem) = 0
               kmeshc(3,ielem) = 0

            else  ! ( jelem.ne.ielem )

!           --- replace inode3 by inode2

               do i = 1, 3
                  if ( kmeshc(i,jelem).eq.inode3 ) &
                     kmeshc(i,jelem) = inode2
               end do  ! i = 1, 3

            end if  ! ( jelem.eq.ielem )

         end do  !   jelem = nelprev+1, nel

!        --- Adapt newbnd

         do i = 1, newbnd(1)
            if ( newbnd(i+1).eq.inode3 ) newbnd(i+1) = inode2
         end do

      end if  ! ( remove )

1000  call erclos ( 'mshpavmnsmoot0' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavmnsmoot0'

      end if

      end
