      subroutine mshpiplintets3 ( npobot, nelbot, kmeshc, ikelm, &
                                  inpelm, kmscbt, weddir, istep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-12-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill bottom layer of elements in case of a pipe consisting of
!     linear tetrahedrons
!     Each wedge is subdivided into 3 tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     element
!     topology
!     pipe
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istep, npobot, nelbot, kmeshc(*), ikelm, inpelm, &
              kmscbt(*), weddir(7,*)

!     ikelm         i/o   Last position used in kmeshc
!     inpelm         i    number of points per element
!     istep          o    Step size of nodes in one row to the new row of
!                         elements
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmscbt         i    Kmesh part c corresponding to bottom surface
!                         Contains the local nodal point numbers of the bottom
!                         surface
!     nelbot         i    Number of elements in bottom surface
!     npobot         i    Number of points of bottom surface
!     weddir         i    Direction of the diagonals per wedge
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikelbt, iielem, k, i1(6), i, oedge, iedge

!     i              Counting variable
!     i1             Work array to store the nodes of one bottom triangle twice
!     iedge          edge sequence number
!     iielem         Relative element number in element group
!     ikelbt         Last position used in kmscbt
!     k              Counting variable
!     oedge          edge opposing the node connecting iedge and iedge+1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpiplintets3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpiplintets3'
         write(irefwr,1) 'npobot, nelbot, inpelm, ikelm', &
                          npobot, nelbot, inpelm, ikelm
  1      format ( a, 1x, (10i6) )

      end if
      istep = npobot
      ikelbt = 0

!     --- Loop over elements in bottom cross-section

      do iielem = 1, nelbot

!     --- i1(k) is filled with nodes of bottom triangle iielem,twice

         do k = 1, 3
            i1(k) = kmscbt(ikelbt+k)
            i1(k+3) = kmscbt(ikelbt+k)
         end do
         ikelbt = ikelbt+3

!        --- find edge with down-up up-down (1 -1) transition in weddir

         iedge = 3
         do i = 1,2
            if (weddir(i,iielem).eq.1 .and. &
                weddir(i+1,iielem).eq.-1) iedge = i
         end do

!        --- add first element, base is bottom element

         do k = 1,  3
            kmeshc(ikelm + k + 1) = i1(k)
         end do

!        --- top node is the node above iedge+1
!            top node of others as well

         do k = 0,2
            kmeshc(ikelm + k*inpelm + 1) = i1(iedge+1)+istep
         end do

!        --- edge opposing the node connecting iedge and iedge+1

         oedge = iedge+2 - ((iedge+2-1)/3)*3

!        --- fill two node of the other two elements

         kmeshc(ikelm + 1*inpelm + 2) = i1(oedge+1)
         kmeshc(ikelm + 1*inpelm + 3) = i1(oedge)
         kmeshc(ikelm + 2*inpelm + 2) = i1(oedge)+istep
         kmeshc(ikelm + 2*inpelm + 3) = i1(oedge+1)+istep

         if (weddir(oedge,iielem).eq.1) then
            kmeshc(ikelm + 1*inpelm + 4) = i1(oedge+1)+istep
            kmeshc(ikelm + 2*inpelm + 4) = i1(oedge)
         else
            kmeshc(ikelm + 1*inpelm + 4) = i1(oedge)+istep
            kmeshc(ikelm + 2*inpelm + 4) = i1(oedge+1)
         end if

         ikelm = ikelm+3*inpelm

      end do

      if ( debug ) then
         call prinin1 ( kmeshc, ikelm/inpelm, inpelm, 'kmeshc' )
         write(irefwr,*) 'End mshpiplintets3'
      end if  ! ( debug )
      call erclos ( 'mshpiplintets3' )

      end
