      subroutine mshrdrenum ( iinput, lengin, renum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 15-03-2014 Set iretur
!        version  2.0  date 28-03-2013 Extra parameter renum
!        version  1.0  date 13-01-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read rest of input following the keyword renumber
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     mesh
!     read
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*), lengin
      logical renum

!     iinput        i/o   integer input array
!     lengin        i/o   Length of integer array iinput
!     renum         i/o   If true renumber has been found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nrenum=19
      double precision tabmn1(nrenum), tabmx1(nrenum)
      character (len=17) code1(nrenum)
      integer itype1(nrenum), ncode1(nrenum), ncodm1(nrenum), nuspnt, &
              ncurvs, nsurfs, method, ipref, ivertx, imidp, iprint, &
              icentr, ilevel, istmet, istval, i

!     code1          In this table the significant characters of the various
!                    texts after renumber are stored
!     i              Counting variable
!     icentr         priority number corresponding to centroid
!     ilevel         If 1 renumbering must be performed per level, if 0 not
!     imidp          priority number corresponding to mid point
!     ipref          Sequence number of preference
!                    0:   profile
!                    1:   band
!                    2:   best
!     iprint         If 1 information must be printed
!     istmet         Indicates the method for computing the start for
!                    renumbering
!                    0:  default
!                    1:  user point
!                    2:  curve
!                    3:  surface
!     istval         Depending on istmet start value for user point, curve
!                    or surface
!     itype1         Indicates the type of action corresponding to keywords in
!                    code1
!     ivertx         priority number corresponding to vertex
!     method         indicates type of renumbering algorithm
!     ncode1         number of significant characters in table code1
!     ncodm1         maximum number of characters in table code1
!     ncurvs         Number of curves in mesh
!     nrenum         Number of entries in table code1
!     nsurfs         Number of surfaces in mesh
!     nuspnt         Number of user points
!     tabmn1         Minimum value of items read corresponding to code1
!     tabmx1         Maximum value of items read corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1547   keyword method=i in combination with new keyword for renumber
!    1548   method keyword found more than once for renumber
!    1549   preference keyword found more than once for renumber
!    1550   start value keyword found more than once for renumber
!    1551   start value keyword found for renumber but not start itself
!    1552   keyword found more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'METHOD', 'NOT', 'CUTHILL_MCKEE', 'SLOAN', 'BEST', &
                   'PROFILE', 'BAND', 'ALWAYS', 'LEVELS', 'PRINT', &
                   'VERTEX_SEQ', 'MID_POINT_SEQ', 'CENTROID_SEQ', &
                   'START', 'P', 'C', 'S', 'ELEMENT_GROUP', &
                   'NESTED_DISSECTION' /
      data itype1 / 0, 2, 2, 2, 2,   2, 2, 2, 2, 2, &
                    0, 0, 0, 2, 0,   0, 0, 2, 2 /
      data ncode1 / 3, 3, 3, 3, 4,   4, 4, 2, 3, 3, &
                    4, 3, 4, 4, 1,   1, 1, 4, 4 /
      data ncodm1 / 6, 3,13, 5, 4,   7, 4, 6, 6, 5, &
                   10,13,12, 5, 1,   1, 1,14,17 /
      data tabmn1 / 0d0, 0d0, 0d0, 0d0, 0d0,  0d0, 0d0, 0d0, 0d0, 0d0, &
                    1d0, 1d0, 1d0, 0d0, 1d0,  1d0, 1d0, 0d0, 0d0 /
      data tabmx1 / 9d0, 0d0, 0d0, 0d0, 0d0,  0d0, 0d0, 0d0, 0d0, 0d0, &
                    3d0, 3d0, 3d0, 0d0, 0d0,  0d0, 0d0, 0d0, 0d0 /

!    code1     In this table the significant characters of the various
!              texts after renumber are stored
!              The following types of texts are recognised:
!              1:   method
!              2:   not
!              3:   cuthill_mckee
!              4:   sloan
!              5:   best
!              6:   profile
!              7:   band
!              8:   always
!              9:   levels
!             10:   print
!             11:   vertex_seq
!             12:   mid_point_seq
!             13:   centroid_seq
!             14:   start
!             15:   p
!             16:   c
!             17:   s
!             18:   element_groups
!             19:   nested_dissection
! ======================================================================
!
      call eropen ( 'mshrdrenum' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdrenum'

      end if

      if ( renum ) then

!     --- renum has been found more than once

         call errchr ( string(1:nchar), 1 )
         call errwar ( 1552, 0, 0, 1 )

      end if
      renum = .true.

      nuspnt = iinput(2)
      ncurvs = iinput(3)
      nsurfs = iinput(4)

      call sepcopycode ( code1, nrenum, ncode1, ncodm1 )
      call sepcopytype ( nrenum, itype1, tabmn1, tabmx1 )
      nmain = nncodemain

      table(1:nncode) = -1d19
      tabmax(15) = nuspnt
      tabmax(16) = ncurvs
      tabmax(17) = nsurfs
      iretur = 0
      call sepreadsetkeyw
      method = 9
      ipref  = 0
      ivertx = 3
      imidp  = 3
      iprint = 0
      icentr = 3
      ilevel = 0
      istmet = 0
      istval = 0
      if ( table(1)>-1d0 ) then

!     --- method = i found, old-fashioned way

         do i = 2, nrenum
            if ( table(i)>-1d0 ) then

!           --- old fashioned way, may not be combined with new way

               call errchr ( code(i), 1 )
               call errwar ( 1547, 0, 0, 1 )

            end if
         end do
         method = nint(table(1))
         ipref  = method/3
         method = method-3*ipref

      end if
      if ( table(2)>-1d0 ) method = 0
      do i = 3, 5
         if ( table(i)>-1d0 ) then
            if ( method/=9 ) then
               call errchr ( code(i), 1 )
               call errwar ( 1548, 0, 0, 1 )
            end if
            method = i-2
         end if
      end do
      if ( table(18)>-1d0 ) method = 4
      if ( table(19)>-1d0 ) method = 5
      if ( table(6)>-1d0 ) ipref = -1
      do i = 7, 8
         if ( table(i)>-1d0 ) then
            if ( ipref/=0 ) then
               call errchr ( code(i), 1 )
               call errwar ( 1549, 0, 0, 1 )
            end if
            ipref = i-6
         end if
      end do
      ipref = max(0,ipref)
      if ( table(9)>-1d0 ) ilevel = 1
      if ( table(10)>-1d0 ) iprint = 1
      if ( table(11)>-1d0 ) ivertx = nint(table(11))
      if ( table(12)>-1d0 ) imidp  = nint(table(12))
      if ( table(13)>-1d0 ) icentr = nint(table(13))
      if ( table(14)>-1d0 ) istmet = -1
      do i = 15, 17
         if ( table(i)>-1d0 ) then
            if ( istmet>0 ) then
               call errchr ( code(i), 1 )
               call errwar ( 1550, 0, 0, 1 )
            else if ( istmet/=-1 ) then
               call errchr ( code(i), 1 )
               call errwar ( 1551, 0, 0, 1 )
            end if
            istmet = i-14
            istval = nint(table(i))
         end if
      end do

!     --- Fill renumbering array

      if ( method==0 ) then
         iinput(10) = method
      else
         iinput(10) = -lengin-6
         iinput(lengin+1) = method
         iinput(lengin+2) = ipref
         iinput(lengin+3) = iprint
         iinput(lengin+4) = ivertx
         iinput(lengin+5) = imidp
         iinput(lengin+6) = icentr
         iinput(lengin+7) = ilevel
         iinput(lengin+8) = istmet
         iinput(lengin+9) = istval
         iinput(lengin+10) = 0
         lengin = lengin + 10
      end if

      call erclos ( 'mshrdrenum' )
      if ( debug ) write(irefwr,*) 'End mshrdrenum'

      end
