      subroutine mshrefconnect ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 27-11-2012 Long integers
!        version  2.1  date 18-06-2012 New call to msh041
!        version  2.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc case that the complete mesh is refined
!     In this case we only fill the connection elements
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     connect
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*)

!     iinput         i    integer input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, lengc1, lengc, ikelmc

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     lengc          computed length of kmesh part c
!     lengc1         computed length of extended kmesh part c
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH038         Check the input for connected elements
!     MSH041         fill array kmeshc for connected elements
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefconnect' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefconnect'
         call prinin ( iinput, 20, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      if ( iinput(12)>0 ) then

!     --- There are connection elements

         do ielgrp = 1, nelgrp

            ishape = kmeshf(ielgrp)
            if ( ishape==-10001 ) kmeshg(ielgrp) = 0

         end do
         lengc1 = size ( kmeshc )
         lengc  = lengc1
         if ( ierror/=0 ) go to 1000

         call msh038 ( iinput(iinp7), lengc )
         if ( ierror/=0 ) go to 1000

!        --- Extend array kmeshc with space for connection elements

         if ( debug ) write(irefwr,1) 'lengc, length', lengc, length
         if ( lengc>lengc1 ) then

!        --- lengc > length, extend

            allocate ( kmeshctmp(lengc1) )
            kmeshctmp = km(iseqkm)%kmeshc(1:lengc1)
            deallocate ( km(iseqkm)%kmeshc )
            allocate ( km(iseqkm)%kmeshc(lengc) )
            kmeshc => km(iseqkm)%kmeshc
            kmeshc(1:lengc1) = kmeshctmp
            deallocate(kmeshctmp)

         end if  ! ( lengc>length )

!        --- Define work array of length npoint + ncurvs

         ikelmc = lengc1
         call msh041 ( iinput(iinp7), ikelmc )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefconnect'

      end if
      call erclos ( 'mshrefconnect' )

      end
