      subroutine mshrefcoor10 ( kmeshclc, first_node, fillmap  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-06-2014 extra parameters
!        version  1.0  date 26-09-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the coordinates for the "face" points in case of extended linear
!     triangles of shape 10
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     node
!     refine
!     linear
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshclc(4), first_node
      logical :: fillmap

!     fillmap        o    If true array imap must be filled
!     first_node    i/o   Node number of next node to be created
!     kmeshclc       i    kmesh part c, contains the nodal points
!                         corresponding to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, inode1, inode2, inode3
      double precision onethird

!     inode          Node sequence number
!     inode1         Node sequence number
!     inode2         Node sequence number
!     inode3         Node sequence number
!     onethird       1/3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The extended linear triangle has 4 nodes according to
!     The edges are defined as: 1-2,  2-3,  3-1
!     Let the corresponding nodes be e1, e2 and e3
!
!                  3
!                  x
!                 / \
!                /   \
!            e3 x     x e2
!              /   +   \
!             /     4   \
!            x-----x-----x
!           1     e1     2
!
!     The three face nodes are defined centres of the 3 triangles
!     f_1 in 1 e1 e3
!     f_2 in 2 e2 e1
!     f_3 in 3 e3 e2
!
!     for i := 1 (1) 3 do
!        inode := nodenumbers(ielem)+i-1
!        inode1 := 2*(i+2)-th node in element
!        inode2 := (2*i)-th node in element
!        Fill 3 face nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefcoor10' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefcoor10'

      end if
      if ( ierror/=0 ) go to 1000
      onethird = 1d0/3d0

!     --- inode := nodenumbers(ielem)+i-1
!         inodei := i-th node in element
!         Fill 4 face nodes

      inode = first_node
      inode1 = kmeshclc(1)
      inode2 = kmeshclc(2)
      inode3 = kmeshclc(3)

!     --- f_1 in 1 e1 e3

      coor(1:ndim,inode) = onethird*(2d0*coor(1:ndim,inode1)+ &
                      0.5d0*(coor(1:ndim,inode2)+coor(1:ndim,inode3)))

      if ( fillmap ) then
         imap(inode,1) = 3
         imap(inode,2) = inode1
         imap(inode,3) = inode2
         imap(inode,4) = inode3
         rmap(inode,1) = 2d0*onethird
         rmap(inode,2:3) = 0.5d0*onethird
      end if  ! ( fillmap )

!     --- f_2 in 2 e2 e1

      inode = inode+1
      coor(1:ndim,inode) = onethird*(2d0*coor(1:ndim,inode2)+ &
                      0.5d0*(coor(1:ndim,inode1)+coor(1:ndim,inode3)))

      if ( fillmap ) then
         imap(inode,1) = 3
         imap(inode,2) = inode2
         imap(inode,3) = inode1
         imap(inode,4) = inode3
         rmap(inode,1) = 2d0*onethird
         rmap(inode,2:3) = 0.5d0*onethird
      end if  ! ( fillmap )

!     --- f_3 in 3 e3 e2

      inode = inode+1
      coor(1:ndim,inode) = onethird*(2d0*coor(1:ndim,inode3)+ &
                      0.5d0*(coor(1:ndim,inode1)+coor(1:ndim,inode2)))

      if ( fillmap ) then
         imap(inode,1) = 3
         imap(inode,2) = inode3
         imap(inode,3) = inode1
         imap(inode,4) = inode2
         rmap(inode,1) = 2d0*onethird
         rmap(inode,2:3) = 0.5d0*onethird
      end if  ! ( fillmap )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefcoor10'

      end if
      call erclos ( 'mshrefcoor10' )

      end
