      subroutine mshrefcoor5 ( kmeshclc, first_node, fillmap  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-06-2014 extra parameters
!        version  1.0  date 20-09-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the coordinates for the "face" points in case of bilinear
!     quadrilaterals of shape 5
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     node
!     refine
!     linear
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshclc(4), first_node
      logical :: fillmap

!     fillmap        o    If true array imap must be filled
!     first_node    i/o   Node number of next node to be created
!     kmeshclc       i    kmesh part c, contains the nodal points
!                         corresponding to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, inode1, inode2, inode3, inode4

!     inode          Node sequence number
!     inode1         Node sequence number
!     inode2         Node sequence number
!     inode3         Node sequence number
!     inode4         Node sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The quadrilateral has 4 nodes according to
!
!            4           3
!            x-----------x
!            |           |
!            |           |
!            |           |
!            |           |
!            |           |
!            x-----------x
!           1            2
!
!     The face nodes are defined in the centre
!
!     inode := nodenumbers(ielem)
!     inodei := i-th node in element
!     coor(inode) := (sum coor(inodei))/4
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefcoor5' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefcoor5'

      end if
      if ( ierror/=0 ) go to 1000

!     --- inode := nodenumbers(ielem)
!         inodei := i-th node in element
!         coor(inode) := (sum coor(inodei))/4

      inode = first_node
      inode1 = kmeshclc(1)
      inode2 = kmeshclc(2)
      inode3 = kmeshclc(3)
      inode4 = kmeshclc(4)
      coor(1:ndim,inode) = 0.25d0*(coor(1:ndim,inode1)+ &
         coor(1:ndim,inode2)+coor(1:ndim,inode3)+coor(1:ndim,inode4))

      if ( fillmap ) then
         imap(inode,1) = 4
         imap(inode,2) = inode1
         imap(inode,3) = inode2
         imap(inode,4) = inode3
         imap(inode,5) = inode4
         rmap(inode,1:4) = 0.25d0
      end if  ! ( fillmap )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefcoor5'

      end if
      call erclos ( 'mshrefcoor5' )

      end
