      subroutine mshrefcoorad ( iinput, rinput, &
                                rinputavail, lencurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 03-10-2003
!
!   copyright (c) 2003-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates on curves in case of refinement in the case that
!     a functional description of the curve is available
!     At this moment the subroutine is limimted to 2d and only in case of
!     arcs or curves created by funccv
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), lencurvs
      double precision rinput(*)
      logical rinputavail

!     iinput         i    integer input array
!     lencurvs       i    Overestimate of the number of points along the new
!                         curves
!     rinput         i    Real input array
!     rinputavail    i    If true, the input array rinput is available
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, istart, i, numnodes, j, jn, icurve, istar3, nodp
      double precision curves(2,lencurvs), curvesnew(2,lencurvs)
      logical filled

!     curves         array containing the coordinates of a curve before
!                    refinement
!     curvesnew      array containing the coordinates of a curve after
!                    refinement
!     filled         If true the curve has been filled
!     i              Counting variable
!     icurve         Type of curve
!     iinp           last position used in array iinput
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     j              Counting variable (here dummy, must be set to 0)
!     jn             Counting variable (here dummy, must be set to 0)
!     nodp           Nodal point sequence number
!     numnodes       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH046         Raise iinp as function of icurve
!     MSH4152DARC    compute the coordinates and place the midpoints
!                    in all parts of an arc
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ndim=2 and rinputavail ) then
!        for all curves do
!           if ( type of curve is arc or made by funccv ) then
!              if ( all nodes of the curve have been filled ) then
!                 Copy all coordinates in array curves
!                 if ( curve is of type arc ) then
!                    Compute new coordinates of curves
!                 else ( curve is made by funccv )
!                    Store factors t in curvefacts
!                    Recompute factors t
!                    Compute new coordinates of curves
!                    Store factors t back in rinput
!                 Copy new coordinates back in coor
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefcoorad' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefcoorad'

      end if
      if ( ierror/=0 ) go to 1000

      if ( ndim==2 .and. rinputavail ) then

!     --- ndim=2 and rinputavail

      iinp = iinp1          ! starting address of curves in iinput
      istar3 = kmeshm(2)-1

!     --- for all curves do

         do icurnr = 1, ncurvs
            icurve = iinput(iinp)  ! type of curve
            if ( icurve==3 .or. icurve==6 .or. icurve==8 .or. &
                 icurve==9 .or. icurve==17 ) then

!           --- type of curve is arc or made by funccv
!               Check if all nodes in curve have been filled

               nnodes = kmeshm(icurnr+5)-kmeshm(icurnr+4) !number of nodes in
                                                          ! curve
               istart = istar3+kmeshm(icurnr+4) ! first position of curve in
                                                ! kmeshm
               filled = .true.
               do i = 0, nnodes-1
                  if ( kmeshm(istart+i)<=0 ) filled = .false.
               end do
               if ( debug ) write(irefwr,*) 'icurnr, icurve, nnodes ', &
                  icurnr, icurve, nnodes
               if ( filled ) then

!              --- all nodes of the curve have been filled
!                  Copy all coordinates in array curves

                   if ( debug ) &
                      call prinin ( kmeshm(istart), nnodes, 'kmeshm' )
                   numnodes = nnodes/2+1
                   do i = 1, numnodes
                      nodp = kmeshm(istart+i*2-2)
                      curves(1,i) = coor(1,nodp)
                      curves(2,i) = coor(2,nodp)
                   end do
                   if ( debug ) &
                      call prinrl1 ( curves, numnodes, 2, 'curves' )
                   if ( icurve==3 .or. icurve==6 .or. icurve==17 ) &
                      then

!                  --- curve is of type arc
!                      Compute new coordinates of curves

                     j = 0
                     jn = 0
                     call msh4152darc ( curves, curvesnew, numnodes, &
                                        iinput(iinp+4), icurve, &
                                        rinput(irnp1), j, jn )

                   else  ! ( icurve==8 .or. icurve==69 )

!                  --- curve is made by funccv
!                      Store factors t in curvefacts
!                      Recompute factors t
!                      Compute new coordinates of curves
!                      Store factors t back in rinput
!                      This part will be filled later on if kmeshh is refilled
!                      See mshrefbd

                   end if  ! ( icurve==3 .or. icurve==6 .or. icurve==17 )

!                  --- Copy new coordinates back in coor

                   do i = 2, nnodes, 2
                      nodp = kmeshm(istart+i-1)
                      coor(1,nodp) = curvesnew(1,i)
                      coor(2,nodp) = curvesnew(2,i)
                   end do
                   if ( debug ) &
                      call prinrl1 ( curvesnew, nnodes, 2, 'curvesnew' )

               end if  ! ( filled )

            end if  ! ( icurve==3 ... )

!           --- raise iinp

            call msh046 ( iinput, iinp, icurve )

         end do  ! icurnr = 1, ncurvs

      end if  ! ( ndim==2 .and. rinputavail ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefcoorad'

      end if
      call erclos ( 'mshrefcoorad' )

      end
