      subroutine mshrefcoorsf ( ielem, nodenumbers, fillmap, nmapout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-06-2014 extra parameters
!        version  1.2  date 10-02-2012 use getnpelmelgrp
!        version  1.1  date 27-11-2011 use function getshapeelgrp
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the coordinates for the "face" points in for all 2d elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     surface
!     mesh
!     coordinate
!     node
!     refine
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielem, nodenumbers(*), nmapout
      logical :: fillmap

!     fillmap        o    If true array imap must be filled
!     ielem          i    Element number
!     nmapout        o    Size of last dimension of rmap
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ipstart, ielgrp, iielem

!     ielgrp         element group number
!     iielem         Relative element number in element group
!     inpelm         number of points per element
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHREFCOOR10   Fill the coordinates for the "face" points in case of
!                    extended linear triangles of shape 10
!     MSHREFCOOR4    Fill the coordinates for the "face" points in case of
!                    quadratic triangle of shape 4
!     MSHREFCOOR5    Fill the coordinates for the "face" points in case of
!                    bilinear quadrilaterals of shape 5
!     MSHREFCOOR6    Fill the coordinates for the "face" points in case of
!                    bi-quadratic quadrilaterals of shape 6
!     MSHREFCOOR7    Fill the coordinates for the "face" points in case of
!                    extended quadratic triangle of shape 7
!     MSHREFCOOR9    Fill the coordinates for the "face" points in case of
!                    extended bilinear quadrilaterals of shape 9
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefcoorsf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefcoorsf'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      inpelm = npelm
      inelem = nelem

!     --- For all element groups do

      ipstart = 1
      do ielgrp = 1, nelgrp

!     --- ishape := shape number of element

         ishape = kmeshf(ielgrp)
         if ( kelmg>0 ) inelem = kmeshg(ielgrp)
         if ( kelma>0 ) inpelm = kmesha(ielgrp+1)-kmesha(ielgrp)
         if ( ishape>3 ) then

!        --- ishape between 4 and 10

            do iielem = 1, inelem

               ielem = ielem+1

               go to ( 140, 150, 160, 170, 180, 190, 200 ), &
                  ishape-3

!              --- ishape=4, quadratic triangle, 3 face nodes

140            call mshrefcoor4 ( kmeshc(ipstart), &
                                  nodenumbers(ielem), fillmap )
               go to 300

!              --- ishape=5, quadrilateral, 1 face node

150            call mshrefcoor5 ( kmeshc(ipstart), &
                                  nodenumbers(ielem), fillmap )
               nmapout = max(nmapout,4)
               go to 300

!              --- ishape=6, bi-quadratic quadrilateral, 8 face nodes

160            call mshrefcoor6 ( kmeshc(ipstart), &
                                  nodenumbers(ielem), fillmap )
               nmapout = max(nmapout,4)
               go to 300

!              --- ishape=7, extended quadratic triangle, 6 face nodes

170            call mshrefcoor7 ( kmeshc(ipstart), &
                                  nodenumbers(ielem), fillmap )
               nmapout = max(nmapout,3)
               go to 300

!              --- ishape=8 not implemented

180            call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

!              --- ishape=9, extended quadrilateral, 4 face nodes

190            call mshrefcoor9 ( kmeshc(ipstart), &
                                  nodenumbers(ielem), fillmap )
               nmapout = max(nmapout,4)
               go to 300

!              --- ishape=10, extended triangle, 3 face nodes

200            call mshrefcoor10 ( kmeshc(ipstart), &
                                   nodenumbers(ielem), fillmap )
               nmapout = max(nmapout,3)

300            ipstart = ipstart+inpelm
               if ( ierror/=0 ) go to 1000

            end do  ! iielem = 1, inelem

         else

!        --- Shape not between 4 and 10

            ipstart = ipstart+inelem*inpelm
            ielem = ielem+inelem

         end if  ! ( ishape>3 )

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefcoorsf'

      end if
      call erclos ( 'mshrefcoorsf' )

      end
