      subroutine mshrefkmeshc11 ( kmeshc, kmeshcnew, nelem, nelprev, &
                                  nmeshd, nodenumbers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 11, i.e. linear tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     linear
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(4,*), kmeshcnew(4,*), nelem, nodenumbers(*), &
              nelprev, nmeshd(*)

!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nelem          i    Number of elements
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     nmeshd         i    nmesh part d
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge, edge(6), i

!     edge           Contains the edge node numbers on the edges
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The linear tetrahedron has nodes 1 2 3 and 4
!     The edges are defines as: 1-2 2-4 4-1 1-3 2-3 and 4-3
!     Let the corresponding nodes be e1, e2 ... e6
!
!     The faces as 1-2-4  2-3-4  3-1-4  1-3-2
!
!     Let the top be node 4, hence the bottom triangle = 1-2-3
!
!     The 8 elements become:
!
!     1   e1  e4  e3
!     2   e5  e1  e2
!     3   e4  e5  e6
!     e3  e2  e6   4
!
!     e1  e4  e3  e2
!     e2  e4  e5  e1
!     e3  e4  e6  e2
!     e4  e6  e2  e5
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        fill first element
!        ielemnew := ielemnew+1
!        fill second element
!        ielemnew := ielemnew+1
!        fill third element
!        ielemnew := ielemnew+1
!        fill fourth element
!        ielemnew := ielemnew+1
!        fill fifth element
!        ielemnew := ielemnew+1
!        fill sixth element
!        ielemnew := ielemnew+1
!        fill seventh element
!        ielemnew := ielemnew+1
!        fill eigth element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc11' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc11'
         write(irefwr,*) 'nelem, nelprev ', nelem, nelprev
         call prinin1 ( kmeshc, nelem, 4, 'kmeshc/volume' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, nelem

!     --- ielemnew := ielemnew+1
!         fill first element
!         ielemnew := ielemnew+1
!         fill second element
!         ielemnew := ielemnew+1
!         fill third element
!         ielemnew := ielemnew+1
!         fill fourth element
!         ielemnew := ielemnew+1
!         fill fifth element
!         ielemnew := ielemnew+1
!         fill sixth element
!         ielemnew := ielemnew+1
!         fill seventh element
!         ielemnew := ielemnew+1
!         fill eigth element

         ipstart = nmeshd(nelprev+ielem)
         do i = 1, 6
            iedge = abs(nmeshd(ipstart+i))
            edge(i) = nodenumbers(iedge)
         end do
         ielemnew = ielemnew+1

!        --- 1   e1  e4  e3

         kmeshcnew(1,ielemnew) = kmeshc(1,ielem)
         kmeshcnew(2,ielemnew) = edge(1)
         kmeshcnew(3,ielemnew) = edge(4)
         kmeshcnew(4,ielemnew) = edge(3)
         ielemnew = ielemnew+1

!        --- 2   e5  e1  e2

         kmeshcnew(1,ielemnew) = kmeshc(2,ielem)
         kmeshcnew(2,ielemnew) = edge(5)
         kmeshcnew(3,ielemnew) = edge(1)
         kmeshcnew(4,ielemnew) = edge(2)
         ielemnew = ielemnew+1

!        --- 3  e4  e5  e6

         kmeshcnew(1,ielemnew) = kmeshc(3,ielem)
         kmeshcnew(2,ielemnew) = edge(4)
         kmeshcnew(3,ielemnew) = edge(5)
         kmeshcnew(4,ielemnew) = edge(6)
         ielemnew = ielemnew+1

!        --- e3  e2  e6 4

         kmeshcnew(1,ielemnew) = edge(3)
         kmeshcnew(2,ielemnew) = edge(2)
         kmeshcnew(3,ielemnew) = edge(6)
         kmeshcnew(4,ielemnew) = kmeshc(4,ielem)
         ielemnew = ielemnew+1

!        --- e1  e4  e3  e2

         kmeshcnew(1,ielemnew) = edge(1)
         kmeshcnew(2,ielemnew) = edge(4)
         kmeshcnew(3,ielemnew) = edge(3)
         kmeshcnew(4,ielemnew) = edge(2)
         ielemnew = ielemnew+1

!        --- e2  e4  e5  e1

         kmeshcnew(1,ielemnew) = edge(2)
         kmeshcnew(2,ielemnew) = edge(4)
         kmeshcnew(3,ielemnew) = edge(5)
         kmeshcnew(4,ielemnew) = edge(1)
         ielemnew = ielemnew+1

!        --- e3  e4  e6  e2

         kmeshcnew(1,ielemnew) = edge(3)
         kmeshcnew(2,ielemnew) = edge(4)
         kmeshcnew(3,ielemnew) = edge(6)
         kmeshcnew(4,ielemnew) = edge(2)
         ielemnew = ielemnew+1

!        --- e4  e6  e2  e5

         kmeshcnew(1,ielemnew) = edge(4)
         kmeshcnew(2,ielemnew) = edge(6)
         kmeshcnew(3,ielemnew) = edge(2)
         kmeshcnew(4,ielemnew) = edge(5)

      end do  ! ielem = 1, nelem

1000  call erclos ( 'mshrefkmeshc11' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew, ielemnew, 3, 'kmeshcnew/surface' )
         write(irefwr,*) 'End mshrefkmeshc11'

      end if

      end
