      subroutine mshrefkmeshc3e ( kmeshc, kmeshcnew, ielemnew, edge )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refine one element in case of a complete refinement
!     Element of shape 3, i.e. linear triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     linear
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(3), kmeshcnew(3,*), ielemnew, edge(3)

!     edge           i    Contains the edge node numbers on the 3 edges
!     ielemnew       i    Sequence number of element in new mesh
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The linear triangle has nodes 1 2 and 3
!     The edges are defines as: 1-2,  2-3,  3-1
!     Let the corresponding nodes be e1, e2 and e3
!
!            3
!            x
!           / \
!       e3 0   0  e2
!         /     \
!        x---0---x
!        1   e1  2
!
!     Then the new triangles become:
!     1 e1 e3;  2 e2 e1; 3 e3 e2; e1 e2 e3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc3e' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc3e'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- 1 e1 e3

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(1)
      kmeshcnew(2,ielemnew) = edge(1)
      kmeshcnew(3,ielemnew) = edge(3)

!     --- 2 e2 e1

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(2)
      kmeshcnew(2,ielemnew) = edge(2)
      kmeshcnew(3,ielemnew) = edge(1)

!     --- 3 e3 e2

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(3)
      kmeshcnew(2,ielemnew) = edge(3)
      kmeshcnew(3,ielemnew) = edge(2)

!     --- e1 e2 e3

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = edge(1)
      kmeshcnew(2,ielemnew) = edge(2)
      kmeshcnew(3,ielemnew) = edge(3)

1000  call erclos ( 'mshrefkmeshc3e' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefkmeshc3e'

      end if

      end
