      subroutine mshrefkmeshc5e ( kmeshc, kmeshcnew, ielemnew, edge, &
                                  iface  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refine one element in case of a complete refinement
!     Element of shape 5, i.e. bi-linear quadrilaterals
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     linear
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(4), kmeshcnew(4,*), ielemnew, edge(4), iface

!     edge           i    Contains the edge node numbers on the 4 edges
!     ielemnew       i    Sequence number of element in new mesh
!     iface          i    Face sequence number
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The bi-linear quadrilateral has nodes 1 2 3 and 4
!     The edges are defines as: 1-2,  2-3,  3-4, 4-1
!     Let the corresponding nodes be e1, e2, e3 and e4
!     The face nodes are defined in the centre
!
!            4     e3    3
!            x-----0-----x
!            |           |
!            |           |
!        e4  0     +     0  e2
!            |           |
!            |           |
!            x-----0-----x
!           1     e1     2
!
!
!     Then the new quadrilaterals become:
!     1 e1 f1 e4;  2 e2 f1 e1; 3 e3 f1 e2; 4 e4 f1 e3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc5e' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc5e'
         call prinin ( kmeshc, 4, 'original element' )
         call prinin ( edge , 4, 'edge' )
         write(irefwr,*) iface

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- 1 e1 f1 e4

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(1)
      kmeshcnew(2,ielemnew) = edge(1)
      kmeshcnew(3,ielemnew) = iface
      kmeshcnew(4,ielemnew) = edge(4)

!     --- 2 e2 f1 e1

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(2)
      kmeshcnew(2,ielemnew) = edge(2)
      kmeshcnew(3,ielemnew) = iface
      kmeshcnew(4,ielemnew) = edge(1)

!     --- 3 e3 f1 e2

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(3)
      kmeshcnew(2,ielemnew) = edge(3)
      kmeshcnew(3,ielemnew) = iface
      kmeshcnew(4,ielemnew) = edge(2)

!     --- 4 e4 f1 e3

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(4)
      kmeshcnew(2,ielemnew) = edge(4)
      kmeshcnew(3,ielemnew) = iface
      kmeshcnew(4,ielemnew) = edge(3)

1000  call erclos ( 'mshrefkmeshc5e' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew(1,ielemnew-3), 4, 4, 'new elements' )
         write(irefwr,*) 'End mshrefkmeshc5e'

      end if

      end
