      subroutine mshrefkmeshc9 ( kmeshc, kmeshcnew, nelem, nelprev, &
                                 nmeshd, nodenumbers, nedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 9, i.e. extended bi-linear quadrilaterals
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(5,*), kmeshcnew(5,*), nelem, nodenumbers(*), &
              nelprev, nmeshd(*), nedges

!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nedges         i    number of edges
!     nelem          i    Number of elements
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     nmeshd         i    nmesh part d
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge, edge(4), i, iface

!     edge           Contains the edge node numbers on the 4 edges
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     iface          Face sequence number
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The bi-linear quadrilateral has nodes 1 2 3 and 4
!     The edges are defined as: 1-2,  2-3,  3-4, 4-1
!     Let the corresponding nodes be e1, e2, e3 and e4
!     The face nodes are defined in the centres of the 4 quads:
!     f_1 in 1 e1 5 e4
!     f_2 in 2 e2 5 e1
!     f_3 in 3 e3 5 e2
!     f_4 in 4 e4 5 e3
!
!            4     e3    3
!            x-----0-----x
!            |           |
!            |           |
!        e4  0     +     0  e2
!            |      5    |
!            |           |
!            x-----0-----x
!           1     e1     2
!
!
!     Then the new quadrilaterals become:
!     1 e1 5 e4 f1
!     2 e2 5 e1 f2
!     3 e3 5 e2 f3
!     4 e4 5 e3 f4
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        fill first element
!        ielemnew := ielemnew+1
!        fill second element
!        ielemnew := ielemnew+1
!        fill third element
!        ielemnew := ielemnew+1
!        fill fourth element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc9' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc9'
         write(irefwr,*) 'nelem, nelprev ', nelem, nelprev
         call prinin1 ( kmeshc, nelem, 3, 'kmeshc/surface' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, nelem

!     --- ielemnew := ielemnew+1
!         fill first element
!         ielemnew := ielemnew+1
!         fill second element
!         ielemnew := ielemnew+1
!         fill third element
!         ielemnew := ielemnew+1
!         fill fourth element

         ipstart = nmeshd(nelprev+ielem)
         do i = 1, 4
            iedge = abs(nmeshd(ipstart+i))
            edge(i) = nodenumbers(iedge)
         end do
         iface = nodenumbers(nedges+ielem+nelprev)  ! node number of face
         ielemnew = ielemnew+1

!        --- 1 e1 5 e4 f1

         kmeshcnew(1,ielemnew) = kmeshc(1,ielem)
         kmeshcnew(2,ielemnew) = edge(1)
         kmeshcnew(3,ielemnew) = kmeshc(5,ielem)
         kmeshcnew(4,ielemnew) = edge(4)
         kmeshcnew(5,ielemnew) = iface
         ielemnew = ielemnew+1

!        --- 2 e2 5 e1 f2

         kmeshcnew(1,ielemnew) = kmeshc(2,ielem)
         kmeshcnew(2,ielemnew) = edge(2)
         kmeshcnew(3,ielemnew) = kmeshc(5,ielem)
         kmeshcnew(4,ielemnew) = edge(1)
         kmeshcnew(5,ielemnew) = iface+1
         ielemnew = ielemnew+1

!        --- 3 e3 5 e2 f3

         kmeshcnew(1,ielemnew) = kmeshc(3,ielem)
         kmeshcnew(2,ielemnew) = edge(3)
         kmeshcnew(3,ielemnew) = kmeshc(5,ielem)
         kmeshcnew(4,ielemnew) = edge(2)
         kmeshcnew(5,ielemnew) = iface+2
         ielemnew = ielemnew+1

!        --- 4 e4 5 e3 f4

         kmeshcnew(1,ielemnew) = kmeshc(4,ielem)
         kmeshcnew(2,ielemnew) = edge(4)
         kmeshcnew(3,ielemnew) = kmeshc(5,ielem)
         kmeshcnew(4,ielemnew) = edge(3)
         kmeshcnew(5,ielemnew) = iface+3

      end do  ! ielem = 1, nelem

1000  call erclos ( 'mshrefkmeshc9' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew, ielemnew, 4, 'kmeshcnew/surface' )
         write(irefwr,*) 'End mshrefkmeshc9'

      end if

      end
