      subroutine mshrefkmeshcl2 ( kmeshc, kmeshcnew, nelem, nelprev, &
                                  nmeshd, nodenumbers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-03-2005 Fortran 90
!        version  1.0  date 14-09-2003
!
!   copyright (c) 2003-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 2, i.e. quadratic line elements
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     quadratic
!     1d
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(3,*), kmeshcnew(3,*), nelem, nodenumbers(*), &
              nelprev, nmeshd(*)

!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nelem          i    Number of elements
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     nmeshd         i    nmesh part d
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge

!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        kmeshcnew(1,ielemnew) := kmeshc(1,ielem)
!        iedge := edge_number of element
!        kmeshcnew(2,ielemnew) := nodenumbers(iedge)
!        kmeshcnew(3,ielemnew) := kmeshc(2,ielem)
!        ielemnew := ielemnew+1
!        kmeshcnew(1,ielemnew) := kmeshc(2,ielem)
!        kmeshcnew(2,ielemnew) := nodenumbers(iedge)+1
!        kmeshcnew(3,ielemnew) := kmeshc(3,ielem)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshcl2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshcl2'

      end if
      if ( ierror/=0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, nelem

!     --- ielemnew := ielemnew+1
!         kmeshcnew(1,ielemnew) := kmeshc(1,ielem)
!         iedge := edge_number of element
!         kmeshcnew(2,ielemnew) := nodenumbers(iedge)
!         kmeshcnew(3,ielemnew) := kmeshc(2,ielem)
!         ielemnew := ielemnew+1
!         kmeshcnew(1,ielemnew) := kmeshc(2,ielem)
!         kmeshcnew(2,ielemnew) := nodenumbers(iedge)+1
!         kmeshcnew(3,ielemnew) := kmeshc(3,ielem)

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshc(1,ielem)
         kmeshcnew(3,ielemnew) = kmeshc(2,ielem)
         kmeshcnew(1,ielemnew+1) = kmeshc(2,ielem)
         kmeshcnew(3,ielemnew+1) = kmeshc(3,ielem)
         ipstart = nmeshd(nelprev+ielem)
         iedge = nmeshd(ipstart+1)
         if ( iedge>0 ) then

!        --- edge in positive direction

            kmeshcnew(2,ielemnew) = nodenumbers(iedge)
            kmeshcnew(2,ielemnew+1) = nodenumbers(iedge)+1

         else

!        --- edge in negative direction

            kmeshcnew(2,ielemnew) = nodenumbers(-iedge)+1
            kmeshcnew(2,ielemnew+1) = nodenumbers(-iedge)

         end if
         ielemnew = ielemnew+1

      end do  ! ielem = 1, nelem

1000  call erclos ( 'mshrefkmeshcl2' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefkmeshcl2'

      end if

      end
