      subroutine mshshiftnod1 ( skipnod, bndnod, iworkpoint )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  2.1  date 22-11-2011 Debug information
!        version  2.0  date 23-04-2003 Extra parameters
!        version  1.1  date 26-11-2002 ncurvs used
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if there are nodes on the boundary and mark them.
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer skipnod(*), bndnod, iworkpoint(*)

!     bndnod         i    Node that lies on boundary of the fluid domain and
!                         on the obstacle boundary
!     iworkpoint     i    Work array of length npointnew
!                         This array is filled as follows:
!                         iworkpoint(i) = 0 if node i outside obstacle
!                         iworkpoint(i) = 1 if node i inside obstacle
!                         iworkpoint(i) = 2 if node i inside obstacle (near
!                                           boundary)
!                         iworkpoint(i) = 3 if node i on obstacle
!     skipnod       i/o   Array of npoint long in which the nodes are marked
!                         which are not allowed to be moved
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  inpelm, ireledge, ielem, isurnr, istart, inode,  icurv

!     icurv          Curve number
!     ielem          Element number
!     inode          Node number
!     inpelm         Number of nodal points per element
!     ireledge       Relative edge sequence number
!     istart         Starting address in array
!     isurnr         Surface number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2174   Wrong shape for surface elements
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshiftnod1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshiftnod1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim==2 ) then

         istart = kmeshm(2)
         inpelm=0

!        --- loop over curves

         do icurv=1,ncurvs
            istart=istart+inpelm
            inpelm=kmeshm(5+icurv)-kmeshm(5+icurv-1)

!           --- loop over nodes

            do inode=1,inpelm

               if ( iworkpoint(kmeshm(istart-1+inode))==3 ) then

!              --- Node lies on curve and on obstacle

                  bndnod=kmeshm(istart-1+inode)
               end if

!              --- Mark node if it lies on curve

               skipnod(kmeshm(istart-1+inode))=1

            end do

         end do

      else if ( ndim==3 ) then

         istart = 2*nsurfs

!        --- For all surfaces do
!            loop over surfaces

         do isurnr = 1, nsurfs
            inpelm = kmeshn(isurnr)
            inelem = kmeshn(isurnr+nsurfs)
            if  (  inpelm/=3 .and. inpelm/=7 .and. &
                   inelem/=0 ) then

!           --- shape is not triangular

               call errint ( inpelm, 1 )
               istart=istart+inelem*inpelm
               go to 100
            end if

!           --- loop over elements

            do ielem = 1, inelem

!           --- loop over nodes

               do ireledge = 1, inpelm
                  inode = kmeshn(istart+ &
                                (ielem-1)*inpelm+ireledge)

!                 --- mark nodes on surface

                  skipnod(inode)=1
               end do
            end do
            istart=istart+inelem*inpelm
100      end do

         call errwar ( 2174, 1, 0, 0 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshiftnod1'

      end if  ! ( debug )
      call erclos ( 'mshshiftnod1' )

      end
