      subroutine mshshiftnod3 ( iwork, istarticross, skipnod, marknod )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.3  date 22-11-2011 Debug information
!        version  1.2  date 23-04-2003 Reordering
!        version  1.1  date 26-11-2002 Extension for quadratic elements
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if all nodes of an element are marked to be shifted onto the
!     obstacle and if that is the case then mark one node as 'not to be
!     shifted'.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), istarticross, skipnod(*), marknod(*)

!     istarticross   i    Starting address of obstacle in array icrossobst
!     iwork          i    integer work array of size nelem
!                         iwork(ielem) contains the starting address of element
!                         ielem in array icrossobst
!                         If 0, the element is not present in icrossobst
!     marknod        i    Mark nodes that will be shifted
!     skipnod       i/o   Array of npoint long in which the nodes are marked
!                         which are not allowed to be moved
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  istart, icross, inode(4), ipstart, i, iedge, &
               ireledge, jnode, idx2(4), nmeshc1, nmeshc2, iielem

!     i              Counting variable
!     icross         Position of iedge in array icrossobst
!     idx2           Array containing the indices of the vertices
!     iedge          Edge sequence number
!     iielem         Relative element number in element group
!     inode          Node sequence number
!     ipstart        Starting address in array
!     ireledge       Relative edge sequence number
!     istart         First position used at a part
!     jnode          Node number
!     nmeshc1        Help variable
!     nmeshc2        Help variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx2 /1,3,5,10/
! ======================================================================
!
      call eropen ( 'mshshiftnod3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshiftnod3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ielem  = 0
      istart = 0

!     --- loop over element groups

      do ielgrp = 1, nelgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)

!              --- if shape is triangular or tetrahedron

         if (ndim==2) then
            if (ishape==3 .or. ishape==7 .or. &
             ishape==11.or. ishape==16)  then

!           --- loop over elements

               do iielem=1, inelem

                  ielem=ielem+1

                  if (iwork(ielem)/=0) then
                     istart = iwork(ielem)

!                    --- do i = 1,nintersect

                     if (icrossobst(istart)>1) then
                        do i = 1,icrossobst(istart)
                           ireledge = icrossobst(istart+i)
                           icross  = istarticross+1+2*(ireledge-1)
                           iedge = icrossobst(icross)
                           ipstart = nmeshc(iedge)

!                          --- check if all nodes of element are potentially
!                              --- shifted

                           nmeshc1=nmeshc(ipstart+1)
                           if (ishape==3) then
                              nmeshc2=nmeshc(ipstart+2)
                           else if (ishape==7) then
                              nmeshc2=nmeshc(ipstart+3)
                           end if
                           if (i==1) then
                              inode(1) = nmeshc2
                              inode(2) = nmeshc1
                           else if (nmeshc2/=inode(1) .and. &
                                    nmeshc2/=inode(2) ) then
                              inode(3)=nmeshc2
                              jnode=nmeshc1
                           else
                              inode(3)=nmeshc1
                              jnode=nmeshc2
                           end if
                        end do

!                       --- mark one node fixed

                        if ( marknod(inode(1))>0 .and. &
                             marknod(inode(2))>0 .and. &
                             marknod(inode(3))>0 ) then
                           skipnod(jnode)=2
                        end if
                     end if

                  end if
               end do !end elementgroup

            else
               ielem=ielem+inelem
            end if !end if ishape==....

         else if (ndim==3) then

!        --- Three dimensional situation

            if (ishape==11.or. ishape==16)  then
               do ielem=1,inelem
                  do i=1,4
                     if (inpelm==4) then
                        inode(i)=kmeshc(istart+(ielem-1)*inpelm+i)
                     else
                        inode(i)=kmeshc(istart+(ielem-1)* &
                                 inpelm+idx2(i))
                     end if
                  end do

!                 --- mark one node fixed

                  if ( marknod(inode(1))>0 .and. &
                       marknod(inode(2))>0 .and. &
                       marknod(inode(3))>0 .and. &
                       marknod(inode(4))>0 ) then
                     if (skipnod(inode(1))/=0) then
                        skipnod(inode(1))=2
                     else if (skipnod(inode(2))/=0) then
                        skipnod(inode(2))=2
                     else if (skipnod(inode(3))/=0) then
                        skipnod(inode(3))=2
                     else
                        skipnod(inode(4))=2
                     end if
                  end if
               end do !end 3D element loop
            else
               istart=istart+inelem*inpelm
            end if

         end if !end ndim=3

      end do !end elementgroup loop

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshiftnod3'

      end if  ! ( debug )
      call erclos ( 'mshshiftnod3' )

      end
