      subroutine mshsl2 ( i, icurve, coor, iinp, iinput, rinput, nelm, &
                          icurvs, elsize )
! ======================================================================
!
!        programmer     Sander de Snoo
!        version 3.1    date   15-09-1997 Lesser accuracy in choice
!        version 3.0    date   24-10-1996 Remove curstr
!        version 2.1    date   18-10-1996 Correction for quadratic elements
!        version 2.0    date   11-09-1996 Extra parameter elsize
!        version 1.0    date   19-04-1996
!
!   copyright (c) 1996-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine to compute nelm for a free boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!   adapt_boundary
!   free_boundary
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision coor(*), rinput(*), elsize(2,*)
      integer icurve, iinput(*), nelm, i, icurvs(0:*), iinp

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     elsize     i    Contains kmesh part p i.e.
!                     elsize(1,i) element length of first element of curve i
!                     elsize(2,i) element length of last element of curve i
!     i          i    sequence number of curve
!     icurve     i    Type of curve
!     icurvs     i    indication of position of curves in array coor
!     iinp       i    last position used in array iinput
!     iinput     i    integer input array
!     nelm       o    number of elements on curve
!     rinput     i    Real input array
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x1,x2,y1,y2,z1,z2,edge1,edge2, slocal, sleng
      integer n,j1,j2,j,cord1, nuspnt, ndim, np, jcoars, ishape

!     cord1      position of first points of curve in array coor
!     edge1      coarseness at begin of curve
!     edge2      coarseness at end of curve
!     ishape     Shape number of elements
!     j          Counting variable
!     j1         user points at begin curve
!     j2         user points at end curve
!     jcoars     If 1 coarseness is defined, if 0 not
!     n          number of points on a curve
!     ndim       Dimension of the space.
!     np
!     nuspnt     Number of user points
!     sleng      total length of curve
!     slocal     length of part of a curve
!     x1         x-coordinate
!     x2         x-coordinate
!     y1         y-coordinate
!     y2         y-coordinate
!     z1         z-coordinate
!     z2         z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      ndim   = iinput(1)
      nuspnt = iinput(2)
      jcoars = iinput(11)

!     --- compute length curve

      sleng =  0
      n     = icurvs(i)-icurvs(i-1)
      cord1 = icurvs(i-1)*ndim
      y1 = 0d0
      y2 = 0d0

      z1 = 0d0
      z2 = 0d0
      do 100 j = 1 , n-1

          x1 =  coor(cord1+ndim*(j-1)+1)
          x2 =  coor(cord1+ndim* j   +1)

          if ( ndim.ne.1 ) then
            y1 =  coor(cord1+ndim*(j-1)+2)
            y2 =  coor(cord1+ndim* j   +2)
            if ( ndim.eq.3 ) then
              z1 =  coor(cord1+ndim*(j-1)+3)
              z2 =  coor(cord1+ndim* j   +3)
            end if
          end if

          slocal = sqrt((x2-x1)**2+(y2-y1)**2+(z2-z1)**2)

          sleng = sleng + slocal

100   continue

!     --- compute coarseness in begin/end point curve

      if ( jcoars.eq.0 ) then

!     --- coarseness has not been defined

         edge1 = elsize(1,i)
         edge2 = elsize(2,i)

      else

!     --- coarseness has been given

         if (icurve.eq.1 .or. icurve.eq.3 .or. icurve.eq.5 .or. &
             icurve.eq.6 .or. icurve.eq.8 .or. icurve.eq.9) then
           j1=iinput(iinp+4)
           j2=iinput(iinp+5)
           if (icurve.eq.5) then
             np = iinput(iinp+6)
           else if (icurve.eq.6) then
             np = iinput(iinp+7)
           else if (icurve.eq.9) then
             np = iinput(iinp+6)
           end if
         else if (icurve.eq.4 .or. icurve.eq.12) then
           j1=iinput(iinp+6)
           j2=iinput(iinp+iinput(iinp+5)-1)
           np=iinput(iinp+5+iinput(iinp+5))
         end if

         edge1=rinput(ndim*nuspnt+j1+1)*rinput(ndim*nuspnt+1)
         edge2=rinput(ndim*nuspnt+j2+1)*rinput(ndim*nuspnt+1)

      end if

!     --- compute nelm

      ishape = iinput(iinp+1)
      if (abs(edge1/edge2-1d0).ge.0.01d0) then
         nelm = nint(sleng/(edge1-edge2)*log(edge1/edge2)/ishape)
      else
         nelm = nint(2d0*sleng/(edge1+edge2)/ishape)
      end if

      nelm = max ( nelm, 1 )

      if ( np.eq.2 ) then

!     --- np = 2 , odd number of points

         if ( mod(nelm,2).ne.0 ) nelm = nelm+1

      else if ( np.eq.3 ) then

!     --- np = 3 , even number of points

         if ( mod(nelm,2).eq.0 ) nelm = nelm+1

      end if

!     --- Fill result in iinput

      iinput(iinp+2) = nelm

      end
