      subroutine mshsurfaces3 ( nouterfaces, outerfaces, &
                                surffaces, kmeshnold, nextra )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 19-05-2004 Change for error message 2630
!        version  1.0  date 03-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend array kmeshn with extra surface consisting of all outer faces
!     that are not yet in a surface
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     mesh
!     outer_boundary
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nouterfaces, outerfaces(nouterfaces), &
              surffaces(*), nextra, inpelm, kmeshnold(*)

!     kmeshnold      i    Old array kmeshn
!     nextra         i    Number of outer faces that do not belong to a surface
!     nouterfaces    i    Number of outer faces (length of outerfaces)
!     outerfaces     i    Array containing the face numbers of the outer faces
!     surffaces      i    Array containing the face numbers of the elements
!                         stored in array kmeshn
!                         Exactly the sequence of kmeshn is used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, isurnr, iielem, iface, j, &
              ipstart, list(2), iwork(nfaces)
      double precision dummy(1)

!     dummy          Dummy parameter
!     i              Counting variable
!     iface          Face sequence number
!     iielem         Relative element number in element group
!     ipstart        Starting address in array
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array of length nfaces
!                    for each outer face iface iwork(iface) is set to -1
!                    if iface belongs to a surface it is reset to 0
!     j              Counting variable
!     list           Dummy parameter for subroutine pr0009
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PR0009         Print information of surfaces
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2625   Not all faces in the surfaces have the same number of nodes
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork(1:nfaces)
!     For all faces iface in array outerfaces do
!        iwork(iface) := -1
!     For all surfaces stored in kmeshn and surffaces do
!        For all faces iface in surface do
!           iwork(iface) := 0
!     Copy old part of kmeshn to new positions
!     Fill information of extra surface in first part of kmeshn
!     For all faces iface in array outerfaces do
!        if ( iwork(iface)#0 ) then
!           put face in kmeshn
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces3'

      end if
      if ( ierror/=0 ) go to 1000

      inpelm = kmeshnold(1)

!     --- Clear array iwork(1:nfaces)

      iwork = 0

!     --- For all faces iface in array outerfaces do
!            iwork(iface) := -1

      iwork(outerfaces) = -1

!     --- For all surfaces stored in kmeshn and surffaces do

      istart = 0
      do isurnr = 1, nsurfs

         inelem = kmeshnold(isurnr+nsurfs)

!        --- For all faces iface in surface do

         do iielem = 1, inelem

!        --- iwork(iface) := 0

            iface = surffaces(istart+iielem)
            if ( iwork(iface)<0 ) then

!           --- Outer surface found

               iwork(iface) = 0

            end if

         end do  ! iielem = 1, inelem
         istart = istart+inelem

      end do  ! isurnr = 1, nsurfs

!     --- Copy old kmeshn to new positions
!         Fill information of extra surface in first part of kmeshn
!         First positions 1 to nsurfs

      do i = 1, nsurfs
         kmeshn(i) = kmeshnold(i)
      end do
      kmeshn(nsurfs+1) = inpelm

!     --- Next positions nsurfs+1 to 2*nsurfs

      do i = nsurfs+1, 2*nsurfs
         kmeshn(i+1) = kmeshnold(i)
      end do
      kmeshn(2*nsurfs+2) = nextra

!     --- Finally last part

      istart = 0
      do i = 1, nsurfs
         istart = istart+kmeshnold(i)*kmeshnold(i+nsurfs)
      end do  ! i = 1, nsurfs

      do i = 2*nsurfs+1, 2*nsurfs+istart
         kmeshn(i+2) = kmeshnold(i)
      end do
      nsurfs = nsurfs+1
      istart = istart+2*nsurfs

!     --- For all faces iface in array outerfaces do

      do i = 1, nouterfaces

         iface = outerfaces(i)

         if ( iwork(iface)/=0 ) then

!        --- iwork(iface)#0
!            put face in kmeshn

            ipstart = nmeshg(iface)
            if ( nmeshg(ipstart)/=inpelm ) then

!           --- Number of nodes in face not inpelm

               call errint ( iface, 1 )
               call errint ( nmeshg(ipstart), 2 )
               call errint ( inpelm, 3 )
               call errsub ( 2625, 3, 0, 0 )
               go to 1000

            end if  ! ( nmeshg(ipstart)/=inpelm )

            do j = 1, inpelm
               kmeshn(istart+j) = nmeshg(ipstart+j)
            end do  ! j = 1, inpelm
            istart = istart+inpelm
            if ( debug ) &
               call prinin ( nmeshg(ipstart+1), inpelm, 'face' )

         end if  ! ( iwork(iface)/=0 )

      end do  ! i = 1, nouterfaces

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'old array kmeshn'
         call pr0009 ( 0, kmeshnold, dummy, list, list(1), &
                       list(1), nsurfs-1 )
         write(irefwr,*) 'new array kmeshn'
         call pr0009 ( 0, kmeshn, dummy, list, list(1), list(1), &
                       nsurfs )
         write(irefwr,*) 'End mshsurfaces3'

      end if
      call erclos ( 'mshsurfaces3' )

      end
