      subroutine mshvlmchksrf ( jsurnr, coorlc, kbndptloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check whether elements of surface do not intersect each other
!
! **********************************************************************
!
!                       KEYWORDS
!
!     volume
!     surface
!     mesh
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jsurnr, kbndptloc(*)
      double precision coorlc(*)

!     coorlc         i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     jsurnr         i    Surface sequence number
!     kbndptloc      i    array containing the nodal point numbers of the
!                         boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsubsf, nelloc, isur, isubsf, leng2, istart
      integer, allocatable :: numsubsf(:)

!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isur           surface number
!     leng2          Length of the second part of the large matrix that
!                    has to be skipped
!     nelloc         Local number of elements
!     nsubsf         Number of subsurfaces
!     numsubsf       Array containing the number of elements in the subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH647         Check whether boundary does not intersect itself
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshvlmchksrf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshvlmchksrf'
         write(irefwr,1) 'jsurnr', jsurnr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nsubsf = 0
      if ( kelmr/=0 ) nsubsf = kmeshr(jsurnr+1) - kmeshr(jsurnr)
      if ( debug ) &
         write(irefwr,1) 'jsurnr, kelmr, nsubsf', &
                          jsurnr, kelmr, nsubsf

      allocate ( numsubsf(max(1,nsubsf)), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( max(1,nsubsf), 1 )
         call errint ( error, 2 )
         call errchr ( 'numsubsf', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

      if ( nsubsf==0 ) then

!     --- nsubsf = 0, No subsurfaces

         numsubsf( 1 ) = kmeshh(2,jsurnr+1)

      else

!     --- Surface consists of subsurfaces
!         Read information of subsurfaces and store in numsubsf

         istart = kmeshr(jsurnr)

         do isubsf = 1, nsubsf

             isur = abs( kmeshr(istart+isubsf) )
             numsubsf( isubsf ) = kmeshh(2,isur+1)

             if ( isubsf>1 ) &
                numsubsf(isubsf) = numsubsf(isubsf) + numsubsf(isubsf-1)

          end do

      end if

      call msh647 ( kbndptloc, ny, coorlc, numsubsf, max(1,nsubsf), &
                    ivolnr, inplmt )

      deallocate ( numsubsf, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'numsubsf', 1 )
         call errsub ( 2911, 1, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshvlmchksrf'

      end if  ! ( debug )
      call erclos ( 'mshvlmchksrf' )

      end
