      subroutine mshvolboun ( iinput, iposi, nsubsf, ndefsf, &
                              coorlc, ivlold, surfnumbers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 27-11-2012 Long integers
!        version  3.0  date 24-08-2010 Removal of work arrays
!        version  2.0  date 19-07-2008 Extra parameters
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read boundary for the generation of volume generators
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iposi, &
              nsubsf, ndefsf, ivlold, surfnumbers(*)
      double precision :: coorlc(*)

!     coorlc         i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iinput         i    integer input array    see subroutine mesh
!                         iinput contains only the part with respect to the
!                         definition of the volumes
!     iposi          i    next free position of iinput
!     ivlold        i/o   sequence number of volume to be "copied"
!     ndefsf         i    Number of curves in surface
!     nsubsf         o    Number of subsurfaces
!     surfnumbers    i    Array of size nextrasrfs containing the surface
!                         sequence numbers of each extra surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposkbn, j, iposj, iv, &
              istart, jstart, ih, isubsf, isur, index, leng1, &
              npoloc, i, iunder, iupper, ih1
      integer, allocatable :: locnum(:)
      double precision, allocatable :: coor1(:)

!     coor1        Array with the coordinates of the submesh
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     index          Record sequence number
!     iposj          Position in array iinput
!     iposkbn        Starting address of present surface in kbndptlc
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isur           surface number
!     iunder         Lower bound of array
!     iupper         Upper bound of array
!     iv             loop variable
!     j              Counting variable
!     jstart         starting position of curve i in kmeshm
!     leng1          length indicator points
!     locnum         Contains local numbering of nodes in submesh
!     npoloc         Number of nodes in subsurface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH050         Raise iinp as function of ivolm
!     MSHVOLBOUN1    Fill array kbndptlc with surfaces that define the boundary
!                    of a volume
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     298   code for generation of volumes incorrect
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshvolboun' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshvolboun'
         write(irefwr,1) 'ivol, iposi', ivol, iposi
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( ivol<0 .or. ivol>11 ) then
         call errint ( ivol, 1 )
         call errsub ( 298, 1, 0, 0 )
         go to 1000
      end if

!     --- Read kmeshc of boundary

      iposkbn = 1
      do j = 1, ndefsf

         isurnr = iinput(iposi+2+j)
         isurnr = mod(isurnr-1,nsurfs)+1
         if ( debug ) write(irefwr,1) 'j, isurnr', j, isurnr
         call mshvolboun1 ( kbndptlc, iposkbn )
         nbndpt = max(nbndpt,iposkbn-1)

      end do  ! j = 1, ndefsf

      if ( nextrasrfs>0 ) then

!     --- nextrasrfs>0, hence extra surfaces
!         Fill numelmextsurfs with number of elements in extra surfaces

         do j = 1, nextrasrfs
            isurnr = surfnumbers(j)
            if ( debug ) write(irefwr,1) 'j, isurnr', j, isurnr
            numelmextsurfs(j) = kmeshh(2,isurnr+1)

            if ( debug ) then
               write(irefwr,1) 'numelmextsurfs(j)', numelmextsurfs(j)
            end if  ! ( debug )

         end do  ! j = 1, nextrasrfs

      end if  ! ( nextrasrfs>0 )

      if ( ivol>=6 .and. ivol<=8 ) then

!     --- Surface and volume to be "copied" are needed
!         (Use coor to store surface)

         ndefsf = iinput(iposi+2)
         ivlold = iinput(iposi+3+ndefsf)

!        --- Determine start position iposj

         iposj  =  iinp5

         do j = 1, ivlold-1
            iv = iinput(iposj)
            call msh050 ( iinput, iposj, iv )
         end do

         ndefsf = iinput(iposj+2)
         iunder = 1
         nsubsf = 0

         do j = 1, ndefsf

            isurnr = mod(iinput(iposj+2+j)-1,nsurfs)+1
            if ( kelmr/=0 ) &
                 nsubsf = kmeshr(isurnr+1) - kmeshr(isurnr)
            length = abs(kmeshh(1,isurnr+1))*ndim
            iupper = iunder+length-1

            if ( nsubsf==0 ) then

!           --- nsubsf = 0, No subsurfaces

               call dcopy ( length, kmhsur(isurnr)%coorsur, &
                            1, coorlc(iunder), 1 )
            else

!           --- Surface consists of subsurfaces
!               Read information of subsurfaces and store in coor

               istart = kmeshr(isurnr)
               jstart = kmeshr(nsurfs+1) - nsurfs-1 + istart

               do isubsf = 1, nsubsf

                  isur = abs ( kmeshr(istart+isubsf) )
                  index = kmeshr(jstart+isubsf)
                  npoloc = kmeshh(1,isur+1)
                  leng1 = npoloc*ndim

                  allocate ( locnum(npoloc), coor1(leng1), &
                             stat = error )
                  if ( error/=0 ) then

!                 --- error#0, so not enough space

                     call errint ( npoloc+leng1, 1 )
                     call errint ( error, 2 )
                     call errchr ( 'locnum', 1 )
                     call errsub ( 2910, 2, 0, 1 )
                     go to 1000

                  end if  ! ( error/= 0 )

!                 --- Read local numbering and store in array locnum of
!                     length npoloc

                  call readfl ( index, 1, npoloc, 1, 3, locnum )

!                 --- Read local array coor and store in array coor1

                  call dcopy ( leng1, kmhsur(isur)%coorsur, 1, coor1, 1 )

!                 --- Put co-ordinates from local array coor into global coor

                  do i = 1, npoloc

                     ih = ndim * (i-1)
                     ih1 = ndim * (locnum(i)-1)+iunder-1

                     coorlc(ih1+1:ih1+ndim) = coor1(ih+1:ih+ndim)

                  end do

                  deallocate ( locnum, coor1, stat = error )
                  if ( error/= 0 ) then

!                 --- error#0, problem with deallocate

                     call errint ( error, 1 )
                     call errchr ( 'coor1', 1 )
                     call errsub ( 2911, 1, 0, 1 )
                     go to 1000

                  end if  ! ( error/= 0 )

               end do

            end if

            iunder = iupper+1

         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kbndptlc, iposkbn/inplmt, inplmt, 'kbndptlc' )
         call prinrl1 ( coorlc, nbcrd, ndim, 'coorlc' )
         write(irefwr,*) 'End mshvolboun'

      end if  ! ( debug )
      call erclos ( 'mshvolboun' )

      end
