      subroutine mshwrsurfaces ( iinput, irefmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.1  date 25-09-2008 Extension with frame surf
!        version  1.0  date 17-01-2003
!
!   copyright (c) 2003-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write parts of arrays iinput and rinput corresponding to surfaces
!     in original form back to an input file with reference number irefmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     write
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), irefmesh

!     iinput         i    integer input array
!     irefmesh       i    Reference number of file to which the mesh description
!                         must be written
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsurs, ntypesur
      parameter ( nsurs=23, ntypesur=3 )
      integer ntxtsur(nsurs), ntxttypsur(ntypesur), ilast, indent, iinp, &
              i, j, ndefsf, irow, nrow, istart
      character (len=17) txtsurfs(nsurs), txttypsurfs(ntypesur)
      character (len=1000) wrtrecord

!     i              Counting variable
!     iinp           last position used in array iinput
!     ilast          Last position used in wrtrecord
!     indent         number of spaces for the indent
!     irow           Row sequence number in case of ordered surfaces
!     istart         First position used at a part
!     j              Counting variable
!     ndefsf         Number of curves in surface
!     nrow           Number of rows of surfaces in ordered surface of
!                    surfaces
!     nsurs          Number of surface generators
!     ntxtsur        Help array to store the length of the names of the
!                    surface generators
!     ntxttypsur     Help array to store the length of the names of the
!                    types of surfaces
!     ntypesur       Number of type of surfaces
!     txtsurfs       Help array to store the names of the surface
!                    generators
!     txttypsurfs    Help array to store the names of the types of surfaces
!     wrtrecord      Contains record to be written
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPUTIN       Put integer number in wrtrecord from position ilast
!     INIPUTXT       Put text in wrtrecord from position ilast
!     INIWRRCD       Write record to file irefmesh in sepran input format
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data txtsurfs / 'rectangle', 'general', 'triangle', &
                      'quadrilateral', 'user', ' ', 'surfaces', &
                      'translate s', 'rotate s', 'pipesurface', &
                      'ordered surfaces', ' ', 'coons', 'similar s', &
                      'parsurf', 'reflect', 'rectangle', 'isopar', &
                      'paver', 'sphere', 'copy_surface s', 'framesurf', &
                      'triangle' /
      data ntxtsur / 9, 7, 8, 13, 4,   1, 8, 11, 8, 11, &
                    16, 1, 5,  9, 7,   7, 9,  6, 5,  6, &
                    14, 9, 8 /
      data txttypsurfs / 'sphere', 'upper_half_sphere', &
                         'lower_half_sphere' /
      data ntxttypsur / 6,17,17 /
! ======================================================================
!
      call eropen ( 'mshwrsurfaces' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshwrsurfaces'
  1      format ( a, 1x, (10i6) )

      end if
      indent = 3
      ilast = 0

      if ( nsurfs==0 ) go to 1000

!     --- nsurfs>0
!         write record with surfaces

      call iniputxt ( 'surfaces', wrtrecord, ilast )
      call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
      indent = 6
      iinp = iinp3
      do i = 1, nsurfs
         isurf = iinput(iinp)
         if ( debug ) write(irefwr,1) 'iinp, isurf', iinp, isurf
         if ( isurf/=0 ) then

!        --- Non empty surface

            call iniputxt ( 's', wrtrecord, ilast )
            call iniputin ( i, wrtrecord, ilast )
            call iniputxt ( ' = ', wrtrecord, ilast )
            call iniputxt ( txtsurfs(isurf)(1:ntxtsur(isurf)), &
                            wrtrecord, ilast )
            call iniputxt ( ' ', wrtrecord, ilast )
            ndefsf = mod(iinput(iinp+2),10000)
            if ( isurf<7 .or. isurf==10 .or. isurf==13 .or. &
                 isurf==15 .or. isurf>16 .and. isurf/=21 ) then

!           --- write shape number

               call iniputin ( iinput(iinp+1), wrtrecord, ilast )
               call iniputxt ( ' ( ', wrtrecord, ilast )

            end if
            if ( isurf==5 ) then

!           --- user, write nelem and npoint

               call iniputxt ( 'nelem = ', wrtrecord, ilast )
               call iniputin ( iinput(iinp+3+ndefsf), wrtrecord, ilast )
               call iniputxt ( ', npoint = ', wrtrecord, ilast )
               call iniputin ( iinput(iinp+4+ndefsf), wrtrecord, ilast )
               do j = 1, ndefsf
                  call iniputxt ( ', c', wrtrecord, ilast )
                  call iniputin ( iinput(iinp+2+j), wrtrecord, ilast )
               end do
               call iniputxt ( ' )', wrtrecord, ilast )

            else if ( isurf==7 ) then

!           --- surface of surfaces, write surface numbers

               call iniputxt ( ' ( ', wrtrecord, ilast )
               do j = 1, ndefsf
                  call iniputxt ( 's', wrtrecord, ilast )
                  call iniputin ( iinput(iinp+2+j), wrtrecord, ilast )
                  call iniputxt ( ', ', wrtrecord, ilast )
               end do
               ilast = ilast-2
               call iniputxt ( ' )', wrtrecord, ilast )

            else if ( isurf==8 .or. isurf==9 .or. isurf==14 .or. &
                      isurf==16 .or. isurf==21 ) then

!           --- translate, rotate, similar or reflect, write surface number

               if ( isurf==21 ) ilast = ilast-1
               call iniputin ( iinput(iinp+3+ndefsf), wrtrecord, ilast )
               call iniputxt ( ' ( ', wrtrecord, ilast )

!              --- write curve numbers

               do j = 1, ndefsf
                  if ( iinput(iinp+2+j)<0 ) &
                     call iniputxt ( '-', wrtrecord, ilast )
                  call iniputxt ( 'c', wrtrecord, ilast )
                  call iniputin ( abs(iinput(iinp+2+j)), wrtrecord, &
                                  ilast )
                  call iniputxt ( ', ', wrtrecord, ilast )
               end do
               ilast = ilast-2
               if ( isurf/=21 ) call iniputxt ( ' )', wrtrecord, ilast )

            else if ( isurf==11 .or. isurf==12 ) then

!           --- ordered surface of surfaces, write surface numbers

               call iniputxt ( ' ( ', wrtrecord, ilast )
               nrow = iinput(iinp+2)
               istart = iinp+2+nrow
               do irow = 1, nrow

                  ndefsf = iinput(iinp+2+irow)
                  call iniputxt ( ' ( ', wrtrecord, ilast )
                  do j = 1, ndefsf
                     if ( iinput(istart+j)<0 ) &
                        call iniputxt ( '-', wrtrecord, ilast )
                     call iniputxt ( 's', wrtrecord, ilast )
                     call iniputin ( abs(iinput(istart+j)), wrtrecord, &
                                     ilast )
                     call iniputxt ( ', ', wrtrecord, ilast )
                  end do
                  ilast = ilast-2
                  call iniputxt ( ' ) ', wrtrecord, ilast )
                  istart = istart+ndefsf

               end do
               call iniputxt ( ' )', wrtrecord, ilast )

            else if ( isurf==20 ) then

!           --- write information of sphere

               call iniputxt ( 'c', wrtrecord, ilast )
               call iniputin ( iinput(iinp+3), wrtrecord, ilast )
               call iniputxt ( ', ', wrtrecord, ilast )
               call iniputxt ( 'centre = p', wrtrecord, ilast )
               call iniputin ( iinput(iinp+4), wrtrecord, ilast )
               call iniputxt ( ', ', wrtrecord, ilast )
               call iniputxt ( 'type = ', wrtrecord, ilast )
               itypesurf = max(1,iinput(iinp+5))
               call iniputxt ( &
                  txttypsurfs(itypesurf)(1:ntxttypsur(itypesurf)), &
                  wrtrecord, ilast )

               call iniputxt ( ', copy = S', wrtrecord, ilast )
               call iniputin ( iinput(iinp+6), wrtrecord, ilast )
               call iniputxt ( ', gencurve = C', wrtrecord, ilast )
               call iniputin ( iinput(iinp+7), wrtrecord, ilast )
               call iniputxt ( ', subsurfaces = (S', wrtrecord, ilast )
               call iniputin ( iinput(iinp+8), wrtrecord, ilast )
               call iniputxt ( ', S', wrtrecord, ilast )
               call iniputin ( iinput(iinp+9), wrtrecord, ilast )
               call iniputxt ( ' )', wrtrecord, ilast )
               call iniputxt ( ' )', wrtrecord, ilast )

            else

!           --- write curve numbers

               do j = 1, ndefsf
                  if ( iinput(iinp+2+j)<0 ) &
                     call iniputxt ( '-', wrtrecord, ilast )
                  call iniputxt ( 'c', wrtrecord, ilast )
                  call iniputin ( abs(iinput(iinp+2+j)), wrtrecord, &
                                  ilast )
                  call iniputxt ( ', ', wrtrecord, ilast )
               end do
               ilast = ilast-2
               call iniputxt ( ' )', wrtrecord, ilast )

            end if
            call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

         end if
         call msh049 ( iinput, iinp, isurf )

      end do

1000  if ( debug ) write(irefwr,*) 'End mshwrsurfaces'
      call erclos ( 'mshwrsurfaces' )

      end
