      subroutine mesh3dprinin ( coor, nbound, fix_pnt, &
                                curvenumbers, coorcurvs, surfnumbers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-07-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print input of subroutine mesh3d
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     tetrahedron
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbound, curvenumbers(*), surfnumbers(*)
      double precision coor(3,*), fix_pnt(*), coorcurvs(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the boundary points
!                         global numbering
!     coorcurvs      i    Contains the coordinates of the extra curves
!     curvenumbers   i    Contains the extra curve numbers
!     fix_pnt        i    Real array of size 4 x nfixpnt containing the
!                         coordinates and coarseness of the fixed points
!                         in the mesh
!     nbound         i    Number of elements in kbndptlc
!     surfnumbers    i    Array of size nextrasrfs containing the surface
!                         sequence numbers of each extra surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  istart, i, numnodes

!     i              Counting variable
!     istart         First position used at a part
!     numnodes       Number of nodal points in array nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     Print of the input
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mesh3dprinin' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mesh3dprinin'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,1) 'ivolnr, nbound, nbndpt', ivolnr, nbound, nbndpt
      call prinrl1 ( coor, nbcrd, 3, 'bcord' )
      call prinin1 ( kbndptlc, nbound, inpelm, 'kbndptlc' )
      write(irefwr,1) 'nextrapnts, nextracrvs, nextrasrfs', &
                       nextrapnts, nextracrvs, nextrasrfs
      if ( nextrapnts>0 ) &
         call prinrl1 ( fix_pnt, nextrapnts, 4, 'fixed points' )
      if ( nextracrvs>0 ) then

!     --- Extra curves, print information

         call prinin ( curvenumbers, nextracrvs, 'extra curves' )
         write(irefwr,*) 'coordinates of extra curves'
         istart = 1
         do i = 1, nextracrvs
            write(irefwr,1) 'curve', curvenumbers(i)
            numnodes = numnodextcurvs(i)
            call prinrl1 ( coorcurvs(1,istart), numnodes, 3, &
                           'coordinates' )
            istart = istart+numnodes
         end do  ! i = 1, nextracrvs

      end if  ! ( nextracrvs>0 )

      if ( nextrasrfs>0 ) then

!     --- Extra surfaces, print information

         call prinin ( surfnumbers, nextrasrfs, 'extra surfaces' )
         call prinin ( numelmextsurfs, nextrasrfs, &
                       'number of elements in extra surfaces' )

      end if  ! ( nextrasrfs>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mesh3dprinin'

      end if  ! ( debug )
      call erclos ( 'mesh3dprinin' )

      end
