      subroutine meshwrpar3
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-01-2016 New parameter list
!        version  1.0  date 15-05-2012
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write global array kmesh to meshoutput in case of refinement
!     Parallel case
! **********************************************************************
!
!                       KEYWORDS
!
!     write
!     mesh
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemach
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ihelp(9), npointlc, npointsav, &
              lenkmeshc, lenkmesh, nelemlc, iseqkm1, iseqkm2
      integer, pointer :: kmeshnew

!     ihelp          integer work array
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kmeshnew       New array kmesh
!     lenkmesh       Size of array kmesh
!     lenkmeshc      length of array kmeshc
!     nelemlc        Number of elements
!     npointlc       Number of nodal points
!     npointsav      Help parameter to save the old value of npoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPMSH059      Write information of the mesh to the file indicated by iref
!     SEPRMMESHINFO  Remove mesh information
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'meshwrpar3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from meshwrpar3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize arrays

      kmeshnew => kmesh2d(maxmesh+1)
      call sepcrmeshinfo ( kmeshnew )
      iseqkm2 = iseqkm
      ihelp = 0

      call sepfillkmesh(kmesh)
      iseqkm1 = iseqkm
      kmeshpar => km(iseqkm1)%kmeshpar
      km(iseqkm2)%kmeshpar => km(iseqkm1)%kmeshpar
      km(iseqkm2)%kmesh(65) = -10

!     --- Create new kmesh

      km(iseqkm2)%kmesh(4:14) = km(iseqkm1)%kmesh(4:14)
      nelgrp = km(iseqkm1)%kmesh(5)
      npointlc = kmeshpar(1)
      km(iseqkm2)%kmesh(8) = npointlc

!     --- kmesha/b

      km(iseqkm2)%kmesh(15:16) = -abs(km(iseqkm1)%kmesh(15:16))
      if ( km(iseqkm1)%kmesh(15)/=0 ) km(iseqkm2)%kmesha => kmesha
      if ( km(iseqkm1)%kmesh(16)/=0 ) km(iseqkm2)%kmeshb => kmeshb

!     --- kmeshc

      lenkmeshc = size(kmeshcglob)-nelgrp
      km(iseqkm2)%kmesh(17) = -lenkmeshc+nelgrp
      km(iseqkm2)%kmeshc => kmeshcglob(nelgrp+1:nelgrp+lenkmeshc)

!     --- skip kmeshd/e
!         kmeshf

      km(iseqkm2)%kmesh(20) = -abs(km(iseqkm1)%kmesh(20))
      if ( km(iseqkm1)%kmesh(20)/=0 ) km(iseqkm2)%kmeshf => kmeshf

!     --- skip kmeshh
!         coor

      km(iseqkm2)%kmesh(23) = -abs(kmeshpar(2))  ! coorglob
      km(iseqkm2)%coor => coorglob  ! coor

!     --- skip kmeshj/k/o iinput/rinput
!         kmeshl/m/n

      km(iseqkm2)%kmesh(26:28) = -abs(kmeshpar(3:5))
      km(iseqkm2)%kmeshl => km(iseqkm1)%kmeshlglob  ! kmeshl
      km(iseqkm2)%kmeshm => km(iseqkm1)%kmeshmglob  ! kmeshm
      km(iseqkm2)%kmeshn => km(iseqkm1)%kmeshnglob  ! kmeshn

!     --- kmeshq/r/s/t

      km(iseqkm2)%kmeshq => km(iseqkm1)%kmeshq
      km(iseqkm2)%kmesh(31) = -abs(km(iseqkm1)%kmesh(31))
      km(iseqkm2)%kmeshr => km(iseqkm1)%kmeshr
      km(iseqkm2)%kmesh(32) = -abs(km(iseqkm1)%kmesh(32))
      km(iseqkm2)%kmeshs => km(iseqkm1)%kmeshs
      km(iseqkm2)%kmesh(33) = -abs(km(iseqkm1)%kmesh(33))
      km(iseqkm2)%kmesht => km(iseqkm1)%kmesht  ! kmesht
      km(iseqkm2)%kmesh(36) = -abs(km(iseqkm1)%kmesh(36))

      if ( km(iseqkm1)%kmesh(21)>0 ) then

!     --- kmesh(21)>0, hence fill kmesh part g

         km(iseqkm2)%kmeshg => kmeshcglob(1:nelgrp)
         nelemlc = sum(km(iseqkm2)%kmeshg(1:nelgrp))
         km(iseqkm2)%kmesh(21) = -nelgrp

      else

!     --- kmesh(21)=0, use nelemlc

         nelemlc = kmeshcglob(1)

      end if  ! ( kmesh(21)>0 )

      km(iseqkm2)%kmesh(9) = nelemlc

!     --- write mesh to meshoutput

      call sepfillkmesh(kmeshnew)
      call sepmsh059 ( iref10, ihelp )
      call seprmmeshinfo ( kmeshnew )

      call sepfillkmesh(kmesh)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End meshwrpar3'

      end if  ! ( debug )
      call erclos ( 'meshwrpar3' )

      end
