      subroutine msh043case5 ( iinput, iposi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 24-08-2010 Removal of work3
!        version  2.1  date 08-04-2009 Reset number of internal points,
!                                      curves and surfaces (ws)
!        version  2.0  date 19-07-2008 Extra parameter kmeshm
!
!   copyright (c) 2005-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     initialize some parameters for mshvlm
!     Case 5-10: subroutines general, rotate, translate
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iposi

!     iinput         i    integer input array containing the information
!                         of volumes
!     iposi          i    next free position of iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, i, nbc, numextrelems

!     i              Counting variable
!     istart         First position used at a part
!     nbc            Number of points along a side
!     numextrelems   Number of elements in the internal surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inidln

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIDLN         Compute length of array in BUFFER
!     MSHGENESTIMATE Compute estimates for npointlc and nelemlc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute the number of elements and points in the boundary
!     Number of points in the boundary surface: nx
!     Number of elements in the boundary surface: ny
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh043case5' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh043case5'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      isurnr = iinput(4)
      ny = kmeshh(2,isurnr+1)
      nbndpt = ny*kmeshh(6,isurnr+1)
      if ( debug ) write(irefwr,1) 'isurnr, nbndpt', isurnr, nbndpt
      nx = abs(kmeshh(1,isurnr+1))

      nmes = max ( 15, nint( 2d0 + sqrt(nx/4d0) ) )

!     --- estimate number of points and elements

      call mshgenestimate ( nmes, npointlc, nelemlc, ny )

      nbcrd  =   nx
      if ( debug ) write(irefwr,1) 'nbcrd', nbcrd

      if ( ivol==5 .or. ivol>=10 ) then

         leniwk = 19 * npointlc + 20 + 3 * nbndpt + 10 + 1000 * nmes * nmes
         lenwrk =  2 * npointlc

      else

         lenwrk =  3 * nbcrd

      end if

!     --- Reset number of extra points, curves and surfaces and
!         related values

      nextrapnts = 0
      ncoarstart = 0
      nextracrvs = 0
      ncurvstart = 0
      lenextracrvs = 0
      nextrasrfs = 0
      nsurfstart = 0
      lenextraboun = 0

      if ( ivol==10 ) then

!     --- ivol = 10, extra internal points, curves or surfaces

         nextrapnts    = iinput(5)   ! number of extra points
         nextracrvs    = iinput(6)   ! number of extra curves
         nextrasrfs    = iinput(7)   ! number of extra surfaces
         ncoarstart = iposi+7     ! starting address of extra
                                     ! points in iinput

         nextracrvs = nextracrvs  ! number of extra curves
         ncurvstart = ncoarstart+nextrapnts  ! starting address of extra
                                     ! curves in iinput

         nsurfstart = ncurvstart+nextracrvs  ! starting address of extra
                                     ! surfaces in iinput
         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'number of internal points', nextrapnts
            write(irefwr,1) 'starting address in iinput', &
                            ncoarstart
            write(irefwr,1) 'number of internal curves', nextracrvs
            write(irefwr,1) 'starting address in iinput', &
                            ncurvstart
            write(irefwr,1) 'number of internal surfaces', nextrasrfs
            write(irefwr,1) 'starting address in iinput', &
                            nsurfstart

         end if  ! ( debug )

         if ( nextracrvs>0 ) then

!        --- nextracurvs>0, hence there are extra curves
!            Compute the number of nodes in these curves and store
!            in lenextracrvs

            istart = ncurvstart-iposi+1  ! local shift
            lenextracrvs = 0   ! length of extra curves
            do i = 0, nextracrvs-1

               icurnr = abs(iinput(istart+i))
               if ( debug ) write(irefwr,1) 'i, icurnr', i, icurnr
               nbc = kmeshm(5+icurnr) - kmeshm(4+icurnr)
               lenextracrvs = lenextracrvs+nbc

            end do  ! i = 1, nextracrvs

         else

!        --- No extra curves

            lenextracrvs = 0

         end if  ! ( nextracrvs>0 )

         lenextracrvs = lenextracrvs  ! Number of nodes in extra curves

         if ( nextrasrfs>0 ) then

!        --- There are extra surfaces, compute numextrelems

            numextrelems = 0
            istart = nsurfstart-iposi  ! local shift
            do i = 1, nextrasrfs
               isurnr = iinput(istart+i)
               if ( debug ) write(irefwr,1) 'i, isurnr', i, isurnr
               numextrelems = numextrelems+ &
                  kmeshh(2,isurnr+1)*kmeshh(6,isurnr+1)
            end do  ! i = 1, nextrasrfs

         end if  ! ( nextrasrfs>0 )

      end if  ! ( ivol==10 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numextrelems', numextrelems
         write(irefwr,*) 'End msh043case5'

      end if
      call erclos ( 'msh043case5' )

      end subroutine msh043case5
