      subroutine mshadapcent12 ( coor, kmeshc, imark, ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates of 3d mesh once the mesh has been deformed
!     Goal is to keep mid-side points ate the centre of the sides
!     and centre points in the centroid
!     One element of shape 12, 15 or 16 only
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     smoothing
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), imark(*), ishape
      double precision coor(3,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     imark         i/o   Indicates if nodes may be moved (0) or not (1)
!     ishape         i    Shape number of elements
!     kmeshc         i    contains the nodal point number
!                         corresponding to the element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision fact

!     fact           multiplication factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshadapcent12' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshadapcent12'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First put all nodes on the mid sides of the edges in the centre

      if ( imark(kmeshc(2))==0 ) coor(1:3,kmeshc(2)) = &
         0.5d0*(coor(1:3,kmeshc(1))+coor(1:3,kmeshc(3)))
      if ( imark(kmeshc(4))==0 ) coor(1:3,kmeshc(4)) = &
         0.5d0*(coor(1:3,kmeshc(3))+coor(1:3,kmeshc(5)))
      if ( imark(kmeshc(6))==0 ) coor(1:3,kmeshc(6)) = &
         0.5d0*(coor(1:3,kmeshc(1))+coor(1:3,kmeshc(5)))
      if ( imark(kmeshc(7))==0 ) coor(1:3,kmeshc(7)) = &
         0.5d0*(coor(1:3,kmeshc(1))+coor(1:3,kmeshc(10)))
      if ( imark(kmeshc(8))==0 ) coor(1:3,kmeshc(8)) = &
         0.5d0*(coor(1:3,kmeshc(3))+coor(1:3,kmeshc(10)))
      if ( imark(kmeshc(9))==0 ) coor(1:3,kmeshc(9)) = &
         0.5d0*(coor(1:3,kmeshc(5))+coor(1:3,kmeshc(10)))

      if ( ishape>12 ) then

!     --- ishape = 15 or 16, centers of the faces

         fact = 1d0/3d0
         if ( imark(kmeshc(11))==0 ) coor(1:3,kmeshc(11)) = fact* &
           (coor(1:3,kmeshc(2))+coor(1:3,kmeshc(4))+coor(1:3,kmeshc(6)))
         if ( imark(kmeshc(12))==0 ) coor(1:3,kmeshc(12)) = fact* &
           (coor(1:3,kmeshc(2))+coor(1:3,kmeshc(7))+coor(1:3,kmeshc(8)))
         if ( imark(kmeshc(13))==0 ) coor(1:3,kmeshc(13)) = fact* &
           (coor(1:3,kmeshc(4))+coor(1:3,kmeshc(8))+coor(1:3,kmeshc(9)))
         if ( imark(kmeshc(14))==0 ) coor(1:3,kmeshc(14)) = fact* &
           (coor(1:3,kmeshc(6))+coor(1:3,kmeshc(9))+coor(1:3,kmeshc(7)))

         if  ( ishape>16 ) then

!        --- ishape = 16,  centroid

            if ( imark(kmeshc(15))==0 ) coor(1:3,kmeshc(15)) = 0.25d0* &
               (coor(1:3,kmeshc(1))+coor(1:3,kmeshc(3))+ &
                coor(1:3,kmeshc(5))+coor(1:3,kmeshc(10)))

         end if  !  ( ishape>16 )

      end if  ! ( ishape>12 )

1000  call erclos ( 'mshadapcent12' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshadapcent12'

      end if  ! ( debug )

      end
