      subroutine mshcheckcircle ( inode1, origin, radius, eps, &
                                  coor, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 06-04-2005 Extra debug information
!        version  1.0  date 13-10-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if node curvesnode(i) is lying on circle defined by origin and
!     radius
! **********************************************************************
!
!                       KEYWORDS
!
!     centre
!     radius
!     check
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr, inode1
      double precision radius, eps, origin(3), coor(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     eps            i    Local accuracy
!     inode1         i    Node on circle
!     isurnr         i    Surface sequence number
!     origin         i    Contains the coordinates of the origin of the circle
!     radius         i    Radius of sphere or circle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision c1

!     c1             Help variable to store the computed local distance
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2640   Boundary of surface is not a (part of a ) circle
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcheckcircle' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcheckcircle'
100      format ( a, 1x, (10i6) )
110      format ( a, 1x, (5d12.4) )
         write(irefwr,110) 'origin', origin
         write(irefwr,110) 'radius, eps', radius, eps
         write(irefwr,100) 'inode1', inode1
         write(irefwr,110) 'coor', coor(1,inode1), coor(2,inode1), &
                           coor(3,inode1)

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      c1 =  (coor(1,inode1)-origin(1))**2+ &
            (coor(2,inode1)-origin(2))**2+ &
            (coor(3,inode1)-origin(3))**2

      if ( abs(c1-radius)>eps ) then

!     --- Not all points on circle

         write(irefwr,110) 'c1, c1-radius', c1, c1-radius
         call errint ( isurnr, 1 )
         call errsub ( 2640, 1, 0, 0 )
         go to 1000

      end if  ! ( abs(c1-radius)>eps )

1000  call erclos ( 'mshcheckcircle' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcheckcircle'

      end if

      end
