      subroutine mshcheckcompcr ( check, nextracurvs, lensubcurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-11-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if surfaces have more than one intersecting curve with other
!     surfaces.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nextracurvs, lensubcurvs
      logical check

!     check          o    Indicates if there are surfaces with at least 2
!                         common curves
!     lensubcurvs    o    Extra length required for compound curves
!     nextracurvs    o    Number of extra curves to be created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, numcurves, iwork(ncurvs), ipstart, &
              jpstart, jsurnr, ncount, numcrv

!     i              Counting variable
!     ipstart        Starting address in array
!     isurnr         Surface sequence number
!     iwork          integer work array of length ncurvs
!                    if iworK(i) = 1, curve i belongs to surface isurnr
!                    otherwise not
!     jpstart        Starting address in array
!     jsurnr         Surface sequence number
!     ncount         Counter
!     numcrv         number of curves in surface jsurnr
!     numcurves      Number of curves in surface isurnr
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     check := false
!     nextracurvs := 0
!     lensubcurvs := 0
!     Clear array iwork(1:ncurvs)
!     for isurnr := 1 (1) nsurfs do
!        Set iwork(i) 1 for all curves in isurnr
!        For jsurnr := isurnr+1 (1) nsurfs do
!           ncount := number of common curves of isurnr and jsurnr
!           check := ncount>1
!           raise lensubcurvs
!        Clear iwork
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcheckcompcr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcheckcompcr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- check := false
!         nextracurvs := 0
!         lensubcurvs := 0

      check = .false.
      nextracurvs = 0
      lensubcurvs = 0

!     --- Clear array iwork(1:ncurvs)

      do i = 1, ncurvs
         iwork(i) = 0
      end do  ! i = 1, ncurvs

!     --- for isurnr := 1 (1) nsurfs do

      do isurnr = 1, nsurfs

!     --- Set iwork(i) 1 for all curves in isurnr

         numcurves = kmeshs(isurnr+1)-kmeshs(isurnr)  ! number of curves in
                                                      ! surface isurnr
         ipstart = kmeshs(isurnr)  ! starting address of curves in isurnr
         do i = 1, numcurves
            iwork(abs(kmeshs(ipstart+i))) = 1
         end do  ! i = 1, numcurves

!        --- For jsurnr := isurnr+1 (1) nsurfs do

         do jsurnr = isurnr+1, nsurfs

!        --- ncount := number of common curves of isurnr and jsurnr
!            check := ncount>1

            ncount = 0
            numcrv = kmeshs(jsurnr+1)-kmeshs(jsurnr)  ! number of curves in
                                                      ! surface jsurnr
            jpstart = kmeshs(jsurnr)  ! starting address of curves in jsurnr
            do i = 1, numcrv
               if ( iwork(abs(kmeshs(jpstart+i)))==1 ) &
                  ncount = ncount+1
            end do  ! i = 1, numcrv

            if ( ncount>1 ) then

!           --- More than one curve found, check := true

               check = .true.
               lensubcurvs = lensubcurvs+ncount
               nextracurvs = nextracurvs+1

            end if  ! ( ncount>1 )

         end do  ! jsurnr = isurnr+1, nsurfs

!        --- Clear iwork

         do i = 1, numcurves
            iwork(abs(kmeshs(ipstart+i))) = 0
         end do  ! i = 1, numcurves

      end do  ! isurnr = 1, nsurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'check ', check
         write(irefwr,1010) 'lensubcurvs, nextracurvs', &
                             lensubcurvs, nextracurvs
1010     format ( a, 1x, 10i6 )
         write(irefwr,*) 'End mshcheckcompcr'

      end if
      call erclos ( 'mshcheckcompcr' )

      end
