      subroutine mshchecktria ( ishape, inpelm, kmeshc, nelem, isurnr, &
                                coor )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.2  date 22-01-2014 New call to msho33
!        version  1.1  date 02-09-2013 Correction format
!        version  1.0  date 17-01-2007
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the minimal ratio between radius of inner circle and
!     outer circle of all triangles in a surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     check
!     quality
!     triangle
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishape, inpelm, kmeshc(inpelm,*), nelem, isurnr
      double precision coor(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points  of the surface
!     inpelm         i    number of points per element
!     ishape         i    Shape number of elements
!     isurnr         i    Surface sequence number
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements of the surface
!     nelem          i    Number of elements of the surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, i3, ielem
      double precision vermin, ratio

!     i1             Index of vertex in element
!     i2             Index of vertex in element
!     i3             Index of vertex in element
!     ielem          Element number
!     ratio          ratio of radius of inner sphere and outer sphere of element
!     vermin         minimum value of ratio
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHO33         Determine the ratio (2*Rout/Rin) for all triangles of mesh
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshchecktria' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshchecktria'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find positions of vertices depending on shape of elements

      select case ( ishape )

         case(:2, 5, 6, 9, 11: )

!        --- Not a triangle, jump

            go to 1000

         case(3, 10)

!        --- Linear triangle

            i1 = 1
            i2 = 2
            i3 = 3

         case(4, 7)

!        --- Quadratic triangle

            i1 = 1
            i2 = 3
            i3 = 5

         case(8)

!        --- Triangle with 4 nodes per edge

            i1 = 1
            i2 = 4
            i3 = 7

      end select  ! case ( ishape )

      vermin = rinfin

      do ielem = 1, nelem

         call msho33 ( coor, ratio, kmeshc(i1,ielem), kmeshc(i2,ielem), &
                       kmeshc(i3,ielem), isurnr, 0, coor, coor, &
                       coor, coor(1), coor(1), coor(1), 0 )

         vermin = min(ratio,vermin)
         if ( debug .and. ratio<0.1d0 ) then

            write(irefwr,*) '1: Element ',ielem,' has ratio ', ratio
            call prinin ( kmeshc(1,ielem), inpelm, 'nodes' )

         end if

      end do  ! ielem = 1, nelem

      if ( vermin>0d0 ) then

!     --- Only if the ratio is positive it will be printed

         write(irefwr,100) isurnr, vermin
100      format ( 'smallest ratio of elements of surface ', i6, &
                  ' is ', ss, 1pe15.6e2 )

      end if  ! ( vermin>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshchecktria'

      end if  ! ( debug )
      call erclos ( 'mshchecktria' )

      end
