      subroutine mshchsurf01 ( coorol, coornw, uloc, anorm, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the co-ordinates of the new 3D boundary in the case
!     iadapt = 1: x_new = x_old + factor u n
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     free_boundary
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint
      double precision coorol(3,npoint), coornw(3,npoint), &
                       uloc(npoint), anorm(3,npoint)

!     anorm          i    In this array the normal vector is stored
!                         anorm(j,i) contains the j th component of the normal
!                         vector in the i-th integration point
!     coornw         o    Array containing the new co-ordinates along the
!                         boundary
!     coorol         i    Array containing the old co-ordinates along the
!                         boundary
!     npoint         i    Number of nodal points along the boundary
!     uloc           i    Array of size npoint containing the
!                         solution in the points of the boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The co-ordinates along the boundary are updated according to the
!    formula:
!    x_new(i) = x_old(i) + u(i) * n(i)
!    For all points i along the boundary do
!       x_new(i) := x_old(i) + uloc(i) * n(i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      coornw(1,:) = coorol(1,:) + uloc*anorm(1,:)
      coornw(2,:) = coorol(2,:) + uloc*anorm(2,:)
      coornw(3,:) = coorol(3,:) + uloc*anorm(3,:)

      end
