      subroutine mshcompdispl21 ( markcurv, marknodes, coornew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates of curves of which at least one point has
!     been moved in a changing boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer markcurv(*), marknodes(*)
      double precision  coornew(*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     markcurv       i    Array of size ncurvs that marks all curves that
!                         have been adapted in the moving boundary
!                         markcurv(icurnr) = 0: curve has not been moved
!                         markcurv(icurnr) = 1: curve has been moved
!     marknodes      i    Array of size npoint that marks all nodes that
!                         have been adapted in the moving boundary
!                         marknodes(node) = 0: node has not been moved
!                         marknodes(node) = 1: node has been moved
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numcurvs, adapcurvs(ncurvs)

!     adapcurvs      Array of size ncurvs containing the curves that
!                    must be adapted
!     numcurvs       Number of curves stored in adapcurvs
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHFINDADCURV  Find all curves that have at least one node that has
!                    been moved during a free boundary problem
!     MSHREDISCUR    Redistribute nodes along set of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find all curves of which at least one node has been moved
!     Redistribute nodes of these curves
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcompdispl21' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcompdispl21'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find all curves of which at least one node has been moved

      call mshfindadcurv ( markcurv, marknodes, numcurvs, adapcurvs, &
                           coornew )
      if ( debug ) call prinin ( adapcurvs, numcurvs, 'adapcurvs' )

!     --- Redistribute nodes of these curves

      call mshrediscur ( numcurvs, adapcurvs, coornew )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcompdispl21'

      end if  ! ( debug )
      call erclos ( 'mshcompdispl21' )

      end
