      subroutine mshcoorref ( iedges, nedges, coor, nmeshc, ndim, fillmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 23-04-2012 Extra debug
!        version  2.0  date 03-05-2005 Extra parameter fillmap
!        version  1.0  date 20-10-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend array coornew with the coordinates of the extra points
!     that are created by subdividing the edges
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iedges(*), nedges, nmeshc(*), ndim
      double precision coor(ndim,*)
      logical fillmap

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!                         At input coor contains the nodes of the old mesh
!                         At output the coordinates of the refined mesh are
!                         added
!     iedges         i    Work array of size nedges
!                         If iedges(i)>0, edge i must be splitted and
!                          the node number will be iedges(i)+npoint
!     fillmap        i    if true arrays imap and rmap must be filled,
!                         if false not
!     ndim           i    Dimension of the space.
!     nedges         i    number of edges
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, newnode, ipstart, nnodes, inode1, inode2

!     iedge          edge sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     newnode        node number of new node
!     nnodes         Number of nodes in edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2740   more than 2 nodes in edge
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all edges iedge do
!        if ( iedges(iedge)>0 ) then
!           newnode := iedges(iedge)+npoint
!           inode1 := first node of edge
!           inode2 := second node of edge
!           coor(newnode) := (coo(inode1)+coor(inode2))/2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcoorref' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcoorref'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- For all edges iedge do

      do iedge = 1, nedges

         if ( iedges(iedge)>0 ) then

!        --- iedges(iedge)>0
!            newnode := iedges(iedge)+npoint
!            inode1 := first node of edge
!            inode2 := second node of edge
!            coor(newnode) := (coor(inode1)+coor(inode2))/2

            newnode = iedges(iedge)       ! node number of new node
            ipstart = nmeshc(iedge)       ! starting address of edge in nmeshc
            nnodes = nmeshc(ipstart)      ! number of nodes in edge

            if ( nnodes>2 ) then

!           --- More than 2 nodes in edge

               call errint ( iedge, 1 )
               call errint ( nnodes, 2 )
               call errint ( 2, 3 )
               call errsub ( 2740, 3, 0, 0 )
               go to 1000

            end if  ! ( nnodes>2 )

            inode1 = nmeshc(ipstart+1)     ! first node in edge
            inode2 = nmeshc(ipstart+2)     ! second node in edge
            coor(:,newnode) = 0.5d0*(coor(:,inode1)+coor(:,inode2))

            if ( debug ) then
               write(irefwr,1) 'iedge, newnode, inode1, inode2', &
                                iedge, newnode, inode1, inode2
               write(irefwr,2) 'coor', coor(:,newnode)
            end if  ! ( debug )

            if ( fillmap ) then

!           --- fillmap is true, fill arrays imap and rmap

               imap(newnode,1) = 2
               imap(newnode,2) = inode1
               imap(newnode,3) = inode2
               rmap(newnode,1) = 0.5d0
               rmap(newnode,2) = 0.5d0

            end if  ! ( fillmap )

         end if  ! ( iedges(iedge)>0 )

      end do  ! iedge = 1, nedges

1000  call erclos ( 'mshcoorref' )

      if ( debug ) then

!     --- Debug information

      write(irefwr,2) 'coor', coor(:,145)
         write(irefwr,*) 'End mshcoorref'

      end if

      end
