      subroutine mshcopsol2d1 ( usol2d, usol3d, npoint, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution from3d mesh onto 2d mesh given the node numbers
!     of the surfaces to which the solution must be copied
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     surface
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, iwork(*)
      double precision usol2d(*), usol3d(*)

!     iwork          i    Contain the local node numbers corresponding to the
!                         global ones
!                         if zero node is not local
!                         length of array iwork is npoint
!     npoint         i    Number of nodal points
!     usol2d         o    Solution vector in surface
!     usol3d         i    Solution vector in 3d space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nunkp2d, nunkp3d, i, j, nodp, inusol2d, inusol3d
      integer, pointer :: kproblc

!     i              Counting variable
!     inusol2d       Starting address (-1) of node in array usol2d
!     inusol3d       Starting address (-1) of node in array usol3d
!     j              Counting variable
!     kproblc        Contains information of the problem definition
!     nodp           Nodal point sequence number
!     nunkp2d        Maximum number of unknowns per point in 2d mesh
!     nunkp3d        Maximum number of unknowns per point in 3d mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcopsol2d1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcopsol2d1'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      kproblc => kprob2d(maxmesh+1)
      iseqkp = kproblc
      kprobpart => kp(iseqkp)%kprobsub(1)
      kprobloc => kprobpart%kprobloc

!     --- Get constants

      nunkp2d = kprobloc(4)
      nunkp3d = nunkpi

!     --- Copy solution

      if ( indprfi==0 ) then

!     --- indprfi = 0, constant number of unknowns in usol3d

         do i = 1, npoint

            if ( iwork(i)>0 ) then

!           --- Node in surface found

               nodp = iwork(i)
               inusol3d = nunkp3d*(i-1)
               inusol2d = nunkp2d*(nodp-1)
               do j = 1, nunkp2d
                  usol2d(inusol2d+j) = usol3d(inusol3d+j)
               end do  ! j = 1, nunkp2d

            end if  ! ( iwork(i)\=0 )

         end do  ! i = 1, npoint

      else

!     --- indprfi # 0, non-constant number of unknowns in usol3d

         if ( kprobloc(19)==0 ) then

!        --- kprobloc(19) = 0, constant number of unknowns in usol2d

            do i = 1, npoint

               if ( iwork(i)>0 ) then

!              --- Node in surface found

                  nodp = iwork(i)
                  inusol3d = kprobfi(i)
                  inusol2d = nunkp2d*(nodp-1)
                  do j = 1, nunkp2d
                     usol2d(inusol2d+j) = usol3d(inusol3d+j)
                  end do  ! j = 1, nunkp2d

               end if  ! ( iwork(i)\=0 )

            end do  ! i = 1, npoint

         else

!        --- kprobloc(19) # 0, non-constant number of unknowns in usol2d

            do i = 1, npoint

               if ( iwork(i)>0 ) then

!              --- Node in surface found

                  nodp = iwork(i)
                  inusol3d = kprobfi(i)
                  inusol2d = kprobfo(nodp)
                  do j = 1, nunkp2d
                     usol2d(inusol2d+j) = usol3d(inusol3d+j)
                  end do  ! j = 1, nunkp2d

               end if  ! ( iwork(i)\=0 )

            end do  ! i = 1, npoint

         end if  ! ( kprobloc(19)==0 )

      end if  ! ( indprfi==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'nunkp2d, nunkp3d', nunkp2d, nunkp3d
         write(irefwr,*) 'End mshcopsol2d1'

      end if
      call erclos ( 'mshcopsol2d1' )

      end
