      subroutine mshcorrfrend ( coorol, coornw, inpada, rinada, n, &
                                kmeshm, coorold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Correct end points of free boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpada(*), n, kmeshm(*)
      double precision coorol(2,*), coornw(2,*), rinada(*), coorold(2,*)

!     coornw        i/o   Array containing the new co-ordinates along the
!                         boundary
!     coorol         i    Array containing the old co-ordinates along the
!                         boundary
!     coorold        i    Contains the coordinates of the original mesh
!     inpada         i    User input array for subroutine ADAPBOUN
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     n              i    number of points along the boundary
!     rinada         i    User input array for subroutine ADAPBOUN
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iangl1, iangl2, icurnr

!     iangl1         Indicates how the starting point must be adapted, &
!                    see INPADA
!     iangl2         Indicates how the end point must be adapted, see INPADA
!     icurnr         Curve sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCORREND     Correct end points of free boundary
!     MSHPROJEND     correct end points for free separation points by projection
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Extract parameters from inpada and rinada
!    Adapt end points as function of iexcl1/2
!    Adapt end points if angles have been described
!    if ( iangl1/2<0 ) then
!       project end point onto prescribed curve
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcorrfrend' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcorrfrend'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Extract parameters from inpada and rinada

      iangl1 = inpada(13)
      iangl2 = inpada(14)
      if ( debug ) write(irefwr,1) 'iangl1, iangl2', iangl1, iangl2

!     --- Adapt end points as function of iexcl1/2

      call mshcorrend ( coorol, coornw, inpada, rinada, n )
      if ( debug ) call prinrl1 ( coornw, n, 2, 'coornw' )

!     --- Project end points onto prescribed curves if required

      if ( iangl1<0 ) then

!     --- iangle1<0, point must ly along curve |iangl1|

         icurnr = abs(iangl1)
         call mshprojend ( coorol, coornw, icurnr, 1, kmeshm, &
                           coorold )

      end if
      if ( iangl2<0 ) then

!     --- iangle2<0, point must ly along curve |iangl2|

         icurnr = abs(iangl2)
         call mshprojend ( coorol, coornw, icurnr, n, kmeshm, &
                           coorold )

      end if

1000  call erclos ( 'mshcorrfrend' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcorrfrend'

      end if  ! ( debug )

      end
