      subroutine mshfillcoorkmc ( kmeshc2dlc, isurfaces, &
                                  nsurfaces, coor2d, normallc, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshc2dlc and coor for a three-dimensional surface mesh
!     mapped onto a two-dimesional mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
!     2d
!     3d
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc2dlc(*), isurfaces(*), nsurfaces, iwork(*)
      double precision coor2d(2,*), normallc(3)

!     coor2d         o    Coordinates of local 2d mesh with local numbering
!     isurfaces      i    Array containing the surfaces from which information
!                         must be found
!     iwork          i    Contain the local node numbers corresponding to the
!                         global ones
!                         if zero node is not local
!                         length of array iwork is npoint
!     kmeshc2dlc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements of the 2d mesh
!                         local numbering
!     normallc       i    normal vector (normal to plane)
!     nsurfaces      i    Number of surfaces in array isurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jsurnr, isurnr, istart, inpelm, ikelmc
      double precision trans(3,3)

!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     jsurnr         Surface sequence number
!     trans          array containing the inverse of the transformation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIINFSURF     Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
!     INIMAPINT1     Map integer array corresponding to global mesh into integer
!                    array corresponding to local mesh using an array with
!                    local node numbers corresponding to the global mesh
!     MSHMAPCR3D2D   Map coordinates from coor into coor2d using given
!                    transformation and the numbering stored in iwork
!     MSHTRANS3D2D   Compute the transformation from 3d to 2d for a plane
!                    given the normal
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all surfaces isurnr in isurfaces do
!        Copy topology from kmeshn into kmeshc2dlc using the new
!        numbering stored in iwork
!     Compute the transformation from 3d to 2d
!     Map coordinates from coor into coor2d using transformation
!     and the numbering stored in iwork
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillcoorkmc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillcoorkmc'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- For all surfaces isurnr in isurfaces do

      ikelmc = 1  ! starting address in kmeshc2dlc
      do jsurnr = 1, nsurfaces

         isurnr = isurfaces(jsurnr)

!        --- Copy topology from kmeshn into kmeshc2dlc using the new
!            numbering stored in iwork

         call iniinfsurf ( kmeshn, istart, inpelm, inelem, nsurfs, &
                           isurnr )
         if ( debug ) &
            write(irefwr,100) 'jsurnr, isurnr, istart, inpelm, inelem', &
                               jsurnr, isurnr, istart, inpelm, inelem

!        --- Map kmeshn into kmeshc2dlc

         call inimapint1 ( kmeshn(istart+1), kmeshc2dlc(ikelmc), iwork, &
                           inpelm*inelem )
         if ( debug ) then
            call prinin1 ( kmeshn(istart+1), inelem, inpelm, 'kmeshn' )
            call prinin1 ( kmeshc2dlc(ikelmc), inelem, inpelm, 'kmeshc2dlc' )
         end if  ! ( debug )

!        --- Raise ikelmc

         ikelmc = ikelmc+inelem*inpelm

      end do  ! jsurnr = 1, nsurfaces

!     --- Compute the transformation from 3d to 2d

      call mshtrans3d2d ( normallc, trans )

!     --- Map coordinates from coor into coor2d using transformation
!         and the numbering stored in iwork

      call mshmapcr3d2d ( coor, coor2d, npoint, iwork, trans )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfillcoorkmc'

      end if
      call erclos ( 'mshfillcoorkmc' )

      end
