      subroutine mshfillkmeshme ( ncurvs, nextracurvs, ipointer, &
                                  subcurves, kmeshm, kmeshmnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend array kmesh part m with new (compound) curves
!     These curves are considered as internal
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     compound_curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, nextracurvs, ipointer(*), subcurves(*), &
              kmeshm(*), kmeshmnew(*)

!     ipointer       i    pointer of length nextracurvs
!                         contains starting addresses in array subcurves
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshmnew      o    New array kmeshm
!     ncurvs         i    Number of curves in mesh
!     nextracurvs    i    Number of extra curves to be created
!     subcurves      i    Contains subcurve numbers of compound curves
!                         sequentially
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ninner, ncurvsnew, i, istart, istartp3, icurnr, jcurnr, &
              numpnt, numpoints, jstart, istar3, j, k

!     i              Counting variable
!     icurnr         Curve sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     istartp3       Starting address of KMESHM part 4 in KMESH part M
!     j              Counting variable
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     jstart         starting position of curve i in kmeshm
!     k              Counting variable
!     ncurvsnew      Last curve sequence number used
!     ninner         Number of inner curves
!     numpnt         number of points in icurnr
!     numpoints      Number of points in compound curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute ninner and store in kmeshmnew(1,4)
!     Fill kmeshmnew(2)
!     Copy kmeshm part 2 to kmeshmnew part 2
!     Extend kmeshmnew part 2 with new curves
!     Copy kmeshm part 3 to kmeshmnew part 3
!     Extend kmeshmnew part 3 with new curves
!     Fill kmeshmnew(3)
!     Copy kmeshm part 4 to kmeshmnew part 4
!     Extend kmeshmnew part 4 with new curves
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillkmeshme' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillkmeshme'
         call pr0008 ( kmeshm, ncurvs )
         call prinin ( kmeshm, 20, 'kmeshm' )
         call prinin ( ipointer, nextracurvs, 'ipointer' )
         call prinin ( subcurves, ipointer(nextracurvs), 'subcurves' )

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Compute ninner and store in kmeshmnew(1,4)

      ninner = kmeshm(1)+nextracurvs
      kmeshmnew(1) = ninner
      kmeshmnew(4) = ninner
      ncurvsnew = ncurvs+nextracurvs

!     --- Fill kmeshmnew(2)

      kmeshmnew(2) = ncurvsnew+6

!     --- Copy kmeshm part 2 to kmeshmnew part 2

      do i = 5, ncurvs+5
         kmeshmnew(i) = kmeshm(i)
      end do  ! i = 1, ncurvs+1

!     --- Copy kmeshm part 3 to kmeshmnew part 3

      do i = kmeshm(2), kmeshm(3)-1
         kmeshmnew(i+nextracurvs) = kmeshm(i)
      end do  ! i = kmeshm(2), kmeshm(3)-1

!     --- Extend kmeshmnew part 2 with new curves
!         Extend kmeshmnew part 3 with new curves

      istar3 = kmeshm(2)-1
      istartp3 = kmeshm(3)-1+nextracurvs
      if ( debug ) &
         write(irefwr,1010) 'istar3, istartp3, nextracurvs', &
                             istar3, istartp3, nextracurvs
      istart = 0
      do i = 1, nextracurvs
         numpoints = 0
         do j = istart+1, ipointer(i)
            icurnr = subcurves(j)      ! curve number
            jcurnr = abs(icurnr)       ! |icurnr|
            if ( debug ) &
               write(irefwr,1010) 'i, istart, j, icurnr', &
                                   i, istart, j, icurnr
            numpnt = kmeshm(5+jcurnr)-kmeshm(4+jcurnr) !  number of points in
                                                       !  icurnr
            numpoints = numpoints + numpnt ! number of points in compound curve
            if ( icurnr.gt.0 ) then

!           --- icurnr > 0, copy curve forwards

               jstart = istar3+kmeshm(4+jcurnr)-1
               do k = 1, numpnt
                  kmeshmnew(istartp3+k) = kmeshm(jstart+k)
               end do  ! k = 1, numpnt

            else

!           --- icurnr < 0, copy curve backwards

               jstart = istar3+kmeshm(5+jcurnr)
               do k = 1, numpnt
                  kmeshmnew(istartp3+k) = kmeshm(jstart-k)
               end do  ! k = 1, numpnt

            end if  ! ( icurnr.gt.0 )
            istartp3 = istartp3+numpnt

         end do  ! j = istart+1, ipointer(i)
         istart = ipointer(i)
         kmeshmnew(ncurvs+5+i) = kmeshmnew(ncurvs+4+i)+numpoints

      end do  ! i = 1, nextracurvs

!     --- Fill kmeshmnew(3)

      kmeshmnew(3) = istartp3+1

!     --- Copy kmeshm part 4 to kmeshmnew part 4

      istar3 = kmeshm(3)-1
      do i = 1, ninner
         kmeshmnew(i+istartp3) = kmeshm(i+istar3)
      end do  ! i = kmeshm(2), kmeshm(3)-1

!     --- Extend kmeshmnew part 4 with new curves

      istartp3 = istartp3+ninner-nextracurvs
      do i = 1, nextracurvs
         kmeshmnew(i+istartp3) = ncurvs+i
      end do  ! i =nextracurvs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1010) 'nextracurvs', &
                             nextracurvs
1010     format ( a, 1x, 10i6 )
         call prinin ( kmeshmnew, 20, 'kmeshmnew' )
         call pr0008 ( kmeshmnew, ncurvs+nextracurvs )
         write(irefwr,*) 'End mshfillkmeshme'

      end if
      call erclos ( 'mshfillkmeshme' )

      end
