      subroutine mshfillkmeshso ( ncurves, istorecurves, imarkcurves, &
                                  kmeshm, imarknodes, icount, icurves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Start with open curve
!     Find all next curves, store in icurves
!     raise icount and mark curves
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     curve
!     ordering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurves, istorecurves(*), imarkcurves(*), kmeshm(*), &
              imarknodes(*), icount, icurves(*)

!     icount        i/o   Number of open curves stored
!     icurves        i    array containing a series of curve numbers
!     imarkcurves   i/o   Work array of length ncurves
!                         if > 0, curve i is in boundary and not yet used
!     imarknodes     i    Work array of length npoint
!                         In imarknodes(i) it is stored how many times node i
!                         is an initial or end point of a curve
!     istorecurves   i    Work array of length ncurves
!                         Contains the curve numbers of the boundary
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     ncurves        i    Number of curves that must be adapted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurnr, ifirst, ilast, nextnode, istar3
      logical finished

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     icurnr         Curve sequence number
!     ifirst         Position of first node in curve
!     ilast          Position of last node in curve
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     nextnode       next node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2739   icount > ncurves
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillkmeshso' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillkmeshso'
         call prinin ( istorecurves, ncurves, 'istorecurves' )

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      istar3 = kmeshm(2)-1
      do i = 1, ncurves
         icurnr = istorecurves(i)    ! curve sequence number
         if ( imarkcurves(icurnr).ne.0 ) then

!        --- New curve

            ifirst = kmeshm(4+icurnr)   ! position of first node
            ilast = kmeshm(5+icurnr)-1  ! position of last node
            if ( imarknodes(kmeshm(ifirst+istar3)).eq.1 ) then

!           --- Open curve found

               nextnode = kmeshm(ilast+istar3)   ! next node in row
               go to 100

            else if ( imarknodes(kmeshm(ilast+istar3)).eq.1 ) then

!           --- Open curve found

               icurnr = -icurnr
               nextnode = kmeshm(ifirst+istar3)   ! next node in row
               go to 100

            end if  ! ( imarknodes(kmeshm(ifirst+istar3)).eq.1 )

         end if  ! ( imarkcurves(icurnr).eq.1 )

      end do  ! i = 1, ncurves

100   imarkcurves(abs(icurnr)) = 0        ! curve is marked as used
      icount = icount+1
      icurves(icount) = icurnr
      if ( debug ) &
         write(irefwr,*) 'icurnr, icount, nextnode ', &
                          icurnr, icount, nextnode

!     --- Find all next curves, store in icurves
!         raise icount and mark curves

      finished = .false.
      do while ( .not. finished )

         do i = 1, ncurves
            icurnr = istorecurves(i)    ! curve sequence number
            if ( imarkcurves(icurnr).ne.0 ) then

!           --- Free curve found
!               check if it starts or ends with nextnode

               ifirst = kmeshm(4+icurnr)   ! position of first node
               ilast = kmeshm(5+icurnr)-1  ! position of last node

               if ( debug ) then
                  write(irefwr,*) 'icurnr, ifirst, ilast ', &
                                   icurnr, ifirst, ilast
                  write(irefwr,*) 'nodes ', &
                     kmeshm(ifirst+istar3),kmeshm(ilast+istar3)
               end if  ! ( debug )

               if ( kmeshm(ifirst+istar3).eq.nextnode ) then

!              --- First node is the next one

                  nextnode = kmeshm(ilast+istar3)
                  go to 200

               else if ( kmeshm(ilast+istar3).eq.nextnode ) then

!              --- Last node is the next one

                  nextnode = kmeshm(ifirst+istar3)
                  icurnr = -icurnr
                  go to 200

               end if  ! ( kmeshm(ifirst+istar3).eq.nextnode )

            end if  ! ( imarkcurves(icurnr).ne.1 )

         end do  ! i = 1, ncurves

         finished = .true.
         go to 300  ! if the program comes here, no curve has been found

200      imarkcurves(abs(icurnr)) = 0        ! curve is marked as used
         icount = icount+1
         icurves(icount) = icurnr
         if ( imarknodes(nextnode).eq.1 ) finished = .true.
         if ( debug ) &
            write(irefwr,*) 'icurnr, icount, nextnode ', &
                             icurnr, icount, nextnode

300      if ( icount.gt.ncurves ) then

!        --- Internal error, icount to large

            call errint ( icount, 1 )
            call errint ( ncurves, 2 )
            call errsub ( 2739, 2, 0, 0 )
            go to 1000

         end if  ! ( icount.gt.ncurves )

      end do  ! while ( .not. finished )

1000  call erclos ( 'mshfillkmeshso' )

      if ( debug ) then

!     --- Debug information

         call prinin ( icurves, icount, 'icurves' )
         write(irefwr,*) 'End mshfillkmeshso'

      end if

      end
