      subroutine mshgetelm3d ( nmeshe, kmeshd, fluidnd, inelem1, npoint, &
                               inode, indelm, shifted)
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 25-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Check for coupling elements (3D)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer fluidnd(7,*), nmeshe(*), inelem1, kmeshd(*), npoint, &
              inode, indelm(*), shifted(*)

!     fluidnd        o    Array that contains the nodal numbers of the coupling
!                         elements in case of fluid-structure computations
!     indelm         i    Array of length nelem in which elements are marked
!                         of which at least one node is shifted
!     inelem1       i/o   Counter for number of coupling elements
!     inode          i    Node number
!     kmeshd         i    KMESH part d
!     nmeshe         i    nmesh part e contains the node sequence numbers of
!                         the elements
!     npoint         i    Number of nodal points
!     shifted        i    Array of npoint long in which the nodes are marked
!                         which are shifted onto the obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i, j, k, ielem, nrelem, idx2(4), idx(4), istart, &
               jtrans(7,4), itran, minnod, maxnod, cnt
      logical found

!     cnt            ?
!     found          If true already a possibility has been found
!     i              Counting variable
!     idx            Array with indices of the vertices
!     idx2           Help parameter to store an index
!     ielem          Element number
!     istart         First position used at a part
!     itran          ?
!     j              Counting variable
!     jtrans         ?
!     k              Counting variable
!     maxnod         ?
!     minnod         ?
!     nrelem         Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data  idx /1,3,5,10/
      data  jtrans /1, 2, 3, 4, 5, 6, 11, &
                    1, 2, 3, 8, 10, 7, 12, &
                    5, 6, 1, 7, 10, 9, 14, &
                    3, 4, 5, 9, 10, 8, 13 /
! ======================================================================
!
      call eropen ( 'mshgetelm3d' )

!     --- loop over elements connected to node i

      nrelem=kmeshd(inode+1)-kmeshd(inode)
      do k=1,nrelem
         ielem=kmeshd(npoint+1+kmeshd(inode)+k)
         istart=nmeshe(ielem)

!        --- loop over vertices of element

         idx2(1)=0
         idx2(2)=0
         idx2(3)=0
         idx2(4)=0
         cnt=1
         do j=1,4
            if( shifted(nmeshe(istart+idx(j))).eq.1) then
               idx2(cnt)=j
               cnt=cnt+1
            end if
         end do

         if (idx2(4).ne.0) then
            write(*,*)'4 nodes shifted onto obstacle'
            write(*,*)'this is not allowed (mshgetelm3d)'
            call instop
         end if

!        --- if 3 nodes on obstacle are found

         if (idx2(1).gt.0.and. &
             idx2(2).gt.0.and. &
             idx2(3).gt.0) then

!        --- get node with lowest and highest index

            minnod = min ( idx2(3), min(idx2(1),idx2(2)) )
            maxnod = max ( idx2(3), max(idx2(1),idx2(2)) )

!           --- determine which face lies on boundary

            if (minnod.eq.1) then
               if (maxnod.eq.3) then
                  itran = 1
               else
                  do i=1,3
                     if (idx2(i).ne.1.and.idx2(i).ne.4) then
                        if (idx2(i).eq.2) itran = 2
                        if (idx2(i).eq.3) itran = 3
                     end if
                  end do
               end if
            else
               itran = 4
            end if

!           --- check if coupling element has found before

            found=.false.
            do i=1,inelem1
               if (nmeshe(istart+jtrans(7,itran)).eq.fluidnd(7,i)) &
                  found=.true.
            end do

!           --- if coupling element has been found before

            if (.not.found) then

               inelem1=inelem1+1

!              --- get new coupling element

               do i=1,7
                  fluidnd(i,inelem1)=nmeshe(istart+jtrans(i,itran))
               end do

            end if

         end if

         indelm(ielem)=1

      end do

      call erclos ( 'mshgetelm3d' )
      end
