      subroutine mshinterfcoor ( inodpbef, kmeshcloc1, kmeshcloc2, &
                                 coorloc1, coorloc2, isurnr1, isurnr2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-03-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new coordinates created by interface elements into
!     array coor
! **********************************************************************
!
!                       KEYWORDS
!
!     interface_element
!     mesh
!     mesh_generation
!     coordinate
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodpbef, kmeshcloc1(*), kmeshcloc2(*), isurnr1, isurnr2
      double precision coorloc1(3,*), coorloc2(3,*)

!     coorloc1       i    Local array coor for surface 1
!     coorloc2       i    Local array coor for surface 2
!     inodpbef       i    First nodal point to be used for extra points
!     isurnr1        i    Surface sequence number
!     isurnr2        i    Surface sequence number
!     kmeshcloc1     i    Local array kmeshc for surface 1
!     kmeshcloc2     i    Local array kmeshc for surface 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer l, istart1, istart2, j, inpelm, istartloc1, &
              istartloc2, ielem, nodeloc, inodp

!     ielem          Element number
!     inodp          Nodal point number
!     inpelm         number of points per element
!     istart1        First position used in kmeshn for surface 1
!     istart2        First position used in kmeshn for surface 2
!     istartloc1     Starting address in local array kmeshcloc1
!     istartloc2     Starting address in local array kmeshcloc2
!     j              Counting variable
!     l              General loop variable
!     nodeloc        Contain the global node numbers corresponding to the
!                    local ones
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get constants
!     Compute starting addresses of surfaces
!     Fill coor
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshinterfcoor' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshinterfcoor'
         write(irefwr,1) 'isurnr1, isurnr2', isurnr1, isurnr2
         write(irefwr,1) 'inodpbef', inodpbef
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute starting addresses of surfaces

      istart1 = 2*nsurfs          ! starting address surface 1
      istart2 = istart1           ! starting address surface 2
      do j = 1, isurnr1-1
         istart1 = istart1+kmeshn(j)*kmeshn(j+nsurfs)
      end do  ! j = 1, isurnr1-1
      do j = 1, isurnr2-1
         istart2 = istart2+kmeshn(j)*kmeshn(j+nsurfs)
      end do  ! j = 1, isurnr2-1
      inpelm = kmeshn(isurnr1)
      inelem = kmeshn(isurnr1+nsurfs)
      if ( debug ) then
         write(irefwr,1) 'istart1, istart2, inpelm, inelem', &
                          istart1, istart2, inpelm, inelem
         call prinin1 ( kmeshn(istart1+1), inelem, inpelm, 'surface 1' )
         call prinin1 ( kmeshn(istart2+1), inelem, inpelm, 'surface 2' )
         call prinin1 ( kmeshcloc1, inelem, inpelm, 'loc_surface 1' )
         call prinin1 ( kmeshcloc2, inelem, inpelm, 'loc_surface 2' )
      end if  ! ( debug )

!     --- Fill coor

      istartloc1 = 0
      istartloc2 = 0
      do ielem = 1, inelem

         do j = 1, inpelm
            inodp = kmeshn(istart1+j)
            if ( inodp>=inodpbef ) then

!           --- New node, coordinates must be filled

               nodeloc = kmeshcloc1(istartloc1+j)
               do l = 1, 3
                  coor(l,inodp) = coorloc1(l,nodeloc)
               end do  ! l = 1, ndim

            end if

         end do  ! j = 1, inpelm
         istart1 = istart1+inpelm
         istartloc1 = istartloc1+inpelm
         do j = 1, inpelm
            inodp = kmeshn(istart2+j)
            if ( inodp>=inodpbef ) then

!           --- New node, coordinates must be filled

               nodeloc = kmeshcloc2(istartloc2+j)
               do l = 1, 3
                  coor(l,inodp) = coorloc2(l,nodeloc)
               end do  ! l = 1, ndim

            end if
         end do  ! j = 1, inpelm
         istart2 = istart2+inpelm
         istartloc2 = istartloc2+inpelm

      end do  ! ielem = 1, inelem

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshinterfcoor'

      end if
      call erclos ( 'mshinterfcoor' )

      end
