      subroutine mshmakeouter ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute outer boundary curves for 2d mesh
!     and store the curve numbers in kmesh part AL
!     Outer boundary is coupled to a set of element groups
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     boundary
!     curve
!     2d
!     outer_boundary
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)

!     iinput         i    Integer input array describing which element
!                         groups must be coupled to a sequence number
!                         For a description see subroutine readmshextra
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icase, ncases, istart, jstart

!     icase          Case sequence number
!     istart         First position used at a part
!     jstart         starting position
!     ncases         Number of cases
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHMAKEOUTERSB Compute outer boundary (curves or surfaces) for 2d or 3d
!                    mesh and store the curve or surface numbers in
!                    kmesh part AL
!     PRININ         print 1d integer array
!     SEPPRINKMESHAL Print the contents of array kmeshal
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmakeouter' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmakeouter'
         call prinin ( iinput, iinput(5)-1, 'iinput' )

      end if
      if ( ierror/=0 ) go to 1000
      ncases = iinput(2)  ! number of input parts defining outer boundaries
      istart = iinput(3)  ! Starting address part outer boundaries
      do icase = 1, ncases
         jstart = iinput(istart+icase-1) ! starting address icase-th outer bound
         if ( debug ) write(irefwr,*) 'ncases, icase, istart, jstart ', &
            ncases, icase, istart, jstart
         call mshmakeoutersb ( iinput(jstart), icase )
      end do  ! icase = 1, ncases

1000  if ( debug ) then

!     --- Debug information

         call sepprinkmeshal
         write(irefwr,*) 'End mshmakeouter'

      end if
      call erclos ( 'mshmakeouter' )

      end
