      subroutine mshmapcr3d2d ( coor3d, coor2d, npoint, iwork, trans )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map coordinates from coor3d into coor2d using given transformation
!     and the numbering stored in iwork
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
!     2d
!     3d
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, iwork(*)
      double precision coor3d(3,*), coor2d(2,*), trans(3,3)

!     coor2d         o    Coordinates of local 2d mesh with local numbering
!     coor3d         i    Coordinates of global 3d mesh with global numbering
!     iwork          i    Contain the local node numbers corresponding to the
!                         global ones
!                         if zero node is not local
!                         length of array iwork is npoint
!     npoint         i    Number of nodal points in global mesh
!     trans          i    array containing the inverse of the transformation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nodp, j, k
      double precision sum

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     nodp           Nodal point sequence number
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) npoint do
!        if ( iwork(i)>0 ) then
!           Map coordinates from coor3d to coor2d
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmapcr3d2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmapcr3d2d'

      end if
      if ( ierror/=0 ) go to 1000

!     --- For i := 1 (1) npoint do

      do i = 1, npoint

         if ( iwork(i)>0 ) then

!        --- iwork(i)>0
!            Map coordinates from coor3d to coor2d

            nodp = iwork(i)     ! new node number
            do k = 1, 2

               sum = 0d0
               do j = 1, 3
                  sum = sum + coor3d(j,nodp)*trans(k,j)
               end do
               coor2d(k,nodp) = sum

            end do  ! k = 1, 2

         end if  ! ( iwork(i)>0 )

      end do  ! i = 1, npoint

1000  call erclos ( 'mshmapcr3d2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmapcr3d2d'

      end if

      end
