      subroutine mshmprvol ( iinput, infel, lengc, kbndpt, &
                             mapar, jnew, indsur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 15-11-2012 Long integers
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 17-04-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill the arrays mapar, kmesh part g, kmeshl, kmeshm, kmeshn
!     raise inodp and iselem
!     a part of array infel is filled
!     Volume elements
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical jnew
      integer iinput(*), kbndpt(*), mapar(*), &
              lengc, infel(*), indsur

!     iinput         i    integer input array for subroutine mesh
!     indsur        i/o   Position in array infel
!     infel         i/o   array containing some information of how the elements
!                         created are stored on file 3
!     jnew           i    indication whether the mesh is new (true) or old
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary
!     lengc          i    computed length of kmesh part c
!     mapar         i/o   mapping array: contains the mapping from local to
!                         global numbering of the nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iposiinpcrv, iposiinppnt
      integer iinp, ielgrp, ifirst, numnodes, &
              ilast, jinp, leng, j, jns, kinp, ivolnum

!     ielgrp         element group number
!     ifirst         First curve (surface) number read
!     iinp           last position used in array iinput
!     ilast          Last curve (surface) number read
!     iposiinpcrv    Starting address in array iinput with respect to the
!                    extra user curves (internal curves)
!     iposiinppnt    Starting address in array iinput with respect to the
!                    extra user user points (internal points)
!     ivolnum        Loop variable
!     j              Counting variable
!     jinp           position in array iinput
!     jns            nsurfs+ivolnr
!     kinp           position in array iinput
!     leng           Help variable to compute the length of some arrays
!     numnodes       Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH029         compute position of volume in array iinput
!     MSH035         fill array mapar and the arrays kmeshn, kmeshm, kmeshl for
!                    the surfaces corresponding to the volume
!     MSHMAPAR35     Put nodal point numbers of extra points into array kmeshl
!     MSHMAPCURVS    Put nodal point numbers of extra curves into array kmeshm
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmprvol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmprvol'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      iinp = iinp6
      if ( debug ) write(irefwr,1) 'inodp, iselem', inodp, iselem

      do ivolnum = 1, nvolel

         ielgrp = iinput(iinp+1)
         ifirst = iinput(iinp+2)
         ilast  = iinput(iinp+3)

         if ( debug ) then

!        --- debug = true, carry out debug statements

            write(irefwr,1) 'ivolnum, ielgrp, ifirst, ilast', &
                             ivolnum, ielgrp, ifirst, ilast

         end if  ! ( debug )

         iinp = iinp+4
         if ( kelma>0 ) inpelm = kmesha(ielgrp+1) -  kmesha(ielgrp)
         nelemlc = 0

         do j = ifirst, ilast

            jns = j+nsurfs
            infel(indsur) = jns
            indsur = indsur+1
            ivolnr = j
            kmeshh(4,jns+1) = -kmeshh(4,jns+1)

            if ( jnew ) then

!           --- jnew = true   new mesh
!               First compute position of volume in array iinput

               call msh029 ( iinput(iinp5), j, ifirst-j, jinp )

               ivol = iinput(jinp+iinp5)
               numnodes = kmeshh(1,jns+1)
               nbndpt = abs(kmeshh(4,jns+1))+1
               nelemlc = kmeshh(2,jns+1)
               leng = nelemlc*inpelm
               if ( debug ) &
                  write(irefwr,1) 'ivol, numnodes, nbndpt, nelemlc', &
                                   ivol, numnodes, nbndpt, nelemlc

!              --- nvolms>0, fill array KMESH part t with volumes

               kmesht(1,j) = iselem
               kmesht(2,j) = iselem+nelemlc-1

               if ( ivol==10 ) then

!              --- ivol = 10, extra surfaces, curves and points

                  kinp = iinp5+jinp+3
                  nextrapnts = iinput(kinp+1)
                  nextracrvs = iinput(kinp+2)
                  iposiinppnt = kinp+4
                  iposiinpcrv = iposiinppnt+nextrapnts

               else

!              --- ivol # 10, standard case no extra surfaces etc.

                  nextrapnts = 0
                  nextracrvs = 0
                  iposiinppnt = 1
                  iposiinpcrv = 1

               end if  ! ( ivol==10 )

               lengc = lengc+leng
               kbndpt(1:nbndpt) = kmhsur(jns)%kbndptsur(1:nbndpt)

!              --- Store actual value of nbndpt
!                  The last position of kbndpt contains lenextracrvs
!                  See msh012

               lenextracrvs = kmhsur(jns)%kbndptsur(nbndpt)
               nbndpt = nbndpt-lenextracrvs-1
               lenextracrvs = lenextracrvs-nextrapnts

               if ( debug ) then

!              --- debug = true, carry out debug statements

                  write(irefwr,1) 'nextrapnts, nextracrvs', &
                                   nextrapnts, nextracrvs
                  write(irefwr,1) 'lenextracrvs, nbndpt', &
                                   lenextracrvs, nbndpt
                  write(irefwr,1) 'j', j
                  if ( inpelm==4 .or. inpelm==5 ) then
                     inplmt = 3
                  else if ( inpelm==10 ) then
                     inplmt = 6
                  else if ( inpelm==8 .or. inpelm==9 ) then
                     inplmt = 4
                  else if ( inpelm==14 .or. inpelm==15 ) then
                     inplmt = 7
                  else
                     inplmt = 9
                  end if
                  call prinin1 ( kbndpt, nbndpt/inplmt, inplmt, &
                                'kbndpt' )

               end if

               npoint = numnodes

!              --- fill array mapar and the arrays kmeshn, kmeshm, kmeshl for
!                  the surfaces corresponding to the volume

               call msh035 ( kbndpt, mapar, iinput(jinp+iinp5) )
               if ( debug ) call prinin ( mapar, numnodes, 'mapar' )

               if ( nextrapnts>0 ) then

!              --- nextrapnts>0, we have to fill the internal nodes into
!                  array kmeshl

                  call mshmapar35 ( mapar, kmeshl, &
                                    iinput(iposiinppnt), &
                                    nextrapnts, &
                                    kbndpt(nbndpt+1) )

               end if  ! ( nextrapnts>0 )

               if ( nextracrvs>0 ) then

!              --- nextracrvs>0, we have to fill the internal curves into
!                  array kmeshm

                  call mshmapcurvs ( nextracrvs, iinput(iposiinpcrv), &
                                     kmeshm, kmeshl, &
                                     kbndpt(nbndpt+1+nextrapnts), &
                                     mapar, ivolnum )

               end if  ! ( nextracrvs>0 )

               kmhsur(jns)%maparsur(1:numnodes) = mapar(1:numnodes)
               if ( nelgrp>1 ) kmeshg(ielgrp) = kmeshg(ielgrp)+nelemlc
               iselem = iselem+nelemlc

            end if

         end do  ! j = ifirst, ilast

      end do  ! ivolnum = 1, nvolel

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmprvol'

      end if
      call erclos ( 'mshmprvol' )

      end subroutine mshmprvol
