      subroutine mshobst01 ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-09-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute some parameters for subroutine mshobstms
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)

!     iinput         i    integer input array
!                         See subroutine mshobstms23d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numbnelmobst, numcross
      logical :: fillcross

!     fillcross      if true crossobst must be filled
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numcross       Number of cross sections
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMS01    Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFLIELEMOBST Fill or activate array kmesh ad  part 2: ielemobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobst01' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobst01'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      call sepflielemobst
      fillcross = kelmad==0
      if ( .not. fillcross ) fillcross = kmeshad(3)==0
      if ( fillcross ) call sepfillkmad3
      if ( ierror/=0 ) go to 1000

!     --- Compute some parameters

      call mshobstms01 ( ielemobst, numobst, numbnelmobst, &
                         icrossobst, numcross )

      iinput(9) = numbnelmobst
      iinput(10) = numcross

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobst01'

      end if  ! ( debug )
      call erclos ( 'mshobst01' )

      end
