      subroutine mshobst2dnm ( kmeshcnew, coornew, kmeshnew, nmap, &
                               iwork, iworkpoint, iedges, edgemoved, &
                               ikelmcnew, npointnew, ielemnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-09-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new mesh from a fixed mesh with an obstacle crossing the mesh
!     2d only
!     Some points may already have been moved
!     Main subroutine calling mshobst2dcr
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), nmap, kmeshnew, &
              iwork(nelem,2), iworkpoint(*), iedges(*), &
              edgemoved(*), ikelmcnew, npointnew, ielemnew
      double precision coornew(2,*)

!     coornew        o    Array containing the new coordinates of the nodes
!     edgemoved     i/o  Array of size nedges indicating if nodes on an edge
!                         have been moved (1) or not (0)
!     iedges        i/o   Work array of size nedges
!                         If iedges(i) = 0, no point has been created in edge i
!                         If iedges(i)>0, a point has been created in edge i,
!                         with sequence number iedges(i)
!     ielemnew       i    Sequence number of element in new mesh
!     ikelmcnew     i/o   Starting address of next element in kmeshcnew
!     iwork         i/o   integer work array of length nelem*2
!                         See subroutine mshobstms2d
!     iworkpoint    i/o   Work array of length npointnew*2
!                         See subroutine mshobstms2d
!     kmeshcnew      o    New kmesh part c
!     kmeshnew      i/o   New array kmesh
!     nmap           i    length of first parameter of imap and rmap
!     npointnew     i/o   Sequence number of last generated node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, ielgrp, irelem, i, ikelmc, ilow, ielemtop, &
              ielembot, isurnr, inelem2, inpelm, iseqkm2

!     i              Counting variable
!     ielem          Element number
!     ielembot       Element sequence number of element with lowest number in
!                    surface isurnr
!     ielemtop       Element sequence number of element with largest number in
!                    surface isurnr
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     ilow           help variable to compute smallest element number
!     inelem2        Help variable for determination of number of elements
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     iseqkm2        Sequence number with respect to km of second mesh
!     isurnr         Surface sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHOBSTMS2D03  Subdivide a triangle if necessary into triangles
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFINDLOWSUR  Find surface (curve) with lowest non-zero entry in kmesht
!     SEPFLIELEMOBST Fill or activate array kmesh ad  part 2: ielemobst
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobst2dnm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobst2dnm'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc
      call sepflpointobst
      call sepflielemobst
      if ( kmeshad(3)==0 .or. kmeshad(5)/=kmeshad(8) ) call sepfillkmad3

      iseqkm2 = kmeshnew

!     --- Initialize some parameters

      ikelmc = 0
      ikelmcnew = 0
      inelem2 = 0
      ielem = 0
      ielemnew = 0

!     --- Find surface with lowest positive element number
!         This is necessary to fill kmesh part t

      call sepfindlowsur ( nsurfs, ielembot, ielemtop, isurnr )

      do ielgrp = 1, nelgrp

!     --- inelem := number of elements in group ielgrp
!         inpelm := number of nodes per element in group ielgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         if ( debug ) &
            write(irefwr,1) 'ielgrp, ishape, inpelm, inelem', &
                             ielgrp, ishape, inpelm, inelem
         if ( ishape>3 .and. ishape/=7 ) then

!        --- shape number not 3 or 7, not yet implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if  ! ( ishape>3 .and. ishape/=7 )

         if ( ishape<3 ) then

!        --- Line elements

            kmeshcnew(ikelmcnew+1:ikelmcnew+inpelm*inelem) = &
               kmeshc(ikelmc+1:ikelmc+inpelm*inelem)
            ikelmc = ikelmc+inpelm*inelem
            ikelmcnew = ikelmcnew+inpelm*inelem

         else

!        --- Surface elements
!            For irelem := 1 (1) inelem do

            do irelem = 1, inelem

!           --- ielem := ielem+1
!               ielemnew := ielemnew+1

               ielem = ielem+1
               ielemnew = ielemnew+1

               if ( debug ) then
                  write(irefwr,1) 'ielem, irelem, ielemnew', &
                                   ielem, irelem, ielemnew
                  write(irefwr,1) 'iwork(ielem,1)', iwork(ielem,1)
               end if  ! ( debug )
               if ( ielem==ielembot ) then

!              --- Lowest element of surface, refill kmesht

                  km(iseqkm2)%kmesht(1,isurnr) = ielemnew

               end if

               if ( iwork(ielem,1)==0 ) then

!              --- ielem completely in obstacle or completely outside obstacle
!                  copy element from kmeshc to kmeshcnew

                  kmeshcnew(ikelmcnew+1:ikelmcnew+inpelm) = &
                     kmeshc(ikelmc+1:ikelmc+inpelm)
                  ikelmc = ikelmc+inpelm
                  ikelmcnew = ikelmcnew+inpelm

               else

!              --- ielem is intersected by obstacle
!                  create new elements
!                  copy these elements to kmeshcnew
!                  Raise ielemnew

                  call mshobstms2d03 ( kmeshc(ikelmc+1), &
                                       kmeshcnew(ikelmcnew+1), &
                                       iworkpoint, npointnew, &
                                       ielemnew, coornew, iwork, &
                                       iedges, nmeshc, &
                                       icrossobst, rcrossobst, ielem, &
                                       ikelmcnew, nmap, nelem, edgemoved )
                  ikelmc = ikelmc+inpelm
                  if ( debug ) then
                     write(irefwr,1) 'ielem, irelem, npointnew', &
                                      ielem, irelem, npointnew
                     write(irefwr,1) 'ikelmcnew', ikelmcnew
                  end if  ! ( debug )

               end if
               if ( ielem==ielemtop ) then

!              --- Top element of surface, refill kmesht

                  km(iseqkm2)%kmesht(2,isurnr) = ielemnew

!                 --- Raise ielembot and ielemtop and recompute isurnr

                  ilow = nelem
                  do i = 1, nsurfs

                     if ( kmesht(1,i)<ilow .and. &
                          kmesht(2,i)>ielemtop ) then

!                    --- new surface found

                        isurnr = i
                        ilow = km(iseqkm2)%kmesht(1,i)

                     end if

                  end do
                  ielembot = km(iseqkm2)%kmesht(1,isurnr)
                  ielemtop = km(iseqkm2)%kmesht(2,isurnr)

               end if

            end do

         end if  ! ( ishape<3 )

         if ( kelmg>0 ) then

!        --- Raise kmesh part g

            km(iseqkm2)%kmeshg(ielgrp) = ielemnew-inelem2
            inelem2=ielemnew

         end if

      end do !end loop over element groups

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobst2dnm'

      end if  ! ( debug )
      call erclos ( 'mshobst2dnm' )

      end
