      subroutine mshobstms07 ( kmeshnew, coornew, kmeshcnew, &
                               kmeshnnew, kmeshmnew, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 19-09-2013 New parameter list
!        version  4.0  date 22-11-2011 New parameter list
!        version  3.0  date 20-09-2006 Extra parameters
!
!   copyright (c) 2004-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make new arrays permanent for subroutines mshobstms and refineloc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshnew, iinput(*), &
              kmeshcnew(*), kmeshnnew(*), kmeshmnew(*)
      double precision coornew(ndim,*)

!     coornew        i    Array containing the new coordinates of the nodes
!     fillmap        i    If true, array map will be filled
!     iinput         i    integer input array
!                         See subroutine mshobstms23d
!     kmeshcnew      i    New kmesh part c
!     kmeshmnew      i    New array kmeshm
!     kmeshnew      i/o   New array kmesh
!     kmeshnnew      i    New array kmeshn
!     map            i    contains information of the mapping
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npointnew, lenkmscnew, lenkmsmnew, lenkmsnnew

!     lenkmscnew     Length of array kmshcnew
!     lenkmsmnew     Length of array kmshmnew
!     lenkmsnnew     Length of array kmshnnew
!     npointnew      Sequence number of last generated node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPRBUF     Activate or create the array referred to by the memory
!                    management index mmindex and copy the contents
!                    of array to buffer
!     SEPFILLKMESHC  Make relation between kmeshc and Topgr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms07' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms07'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm = kmeshnew

      npointnew = iinput(4)
      lenkmsmnew = iinput(7)
      lenkmsnnew = iinput(8)
      lenkmscnew = iinput(12)
      if ( km(iseqkm)%kmesh(23)>0 ) deallocate(km(iseqkm)%coor)
      allocate(km(iseqkm)%coor(ndim,npointnew))
      coor => km(iseqkm)%coor
      coor = coornew(:,1:npointnew)
      km(iseqkm)%kmesh(23) = npointnew

      if ( km(iseqkm)%kmesh(17)>0 ) deallocate(km(iseqkm)%kmeshc)
      allocate(km(iseqkm)%kmeshc(lenkmscnew))
      km(iseqkm)%kmeshc = kmeshcnew(1:lenkmscnew)
      km(iseqkm)%kmesh(17) = lenkmscnew
      call sepfillkmeshc

      if ( ndim>=2 ) then

!     --- ndim = 2

         if ( km(iseqkm)%kmesh(27)>0 ) deallocate(km(iseqkm)%kmeshm)
         allocate(km(iseqkm)%kmeshm(lenkmsmnew))
         km(iseqkm)%kmeshm = kmeshmnew(1:lenkmsmnew)
         km(iseqkm)%kmesh(27) = lenkmsmnew

      end if  ! ( ndim>=2 )

      if ( ndim==3 ) then

!     --- ndim = 3

         if ( km(iseqkm)%kmesh(28)>0 ) deallocate(km(iseqkm)%kmeshn)
         allocate(km(iseqkm)%kmeshn(lenkmsnnew))
         km(iseqkm)%kmeshn = kmeshnnew(1:lenkmsnnew)
         km(iseqkm)%kmesh(28) = lenkmsnnew

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms07'

      end if
      call erclos ( 'mshobstms07' )

      end
