      subroutine mshobstms08 ( kmeshnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 22-08-2012 Copy kmeshquant
!        version  1.1  date 04-10-2006 Refill kmeshm in R^3
!        version  1.0  date 25-10-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Delete some parts of kmeshnew and copy parts l and m from kmeshorig
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshnew

!     kmeshnew      i/o   New array kmesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqkm1

!     iseqkm1        Sequence number with respect to km of first mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHDELSEL      Delete a selected set of subarrays of kmesh
!     SEPCRMESHINFO  Create permanent space for mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms08' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms08'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm1 = iseqkm

      iseqkm = 0
      if ( kmeshnew>0 .and. kmeshnew<=lastmesh ) iseqkm = kmeshnew

      if ( iseqkm>0 ) then

!     --- kmeshnew has been filled before
!         delete some arrays that will be created completely new again

         call mshdelsel ( kmeshnew )

!        --- kmeshnew m, n

         if ( km(iseqkm)%kmesh(27)>0 ) deallocate ( km(iseqkm)%kmeshm )
         if ( km(iseqkm)%kmesh(28)>0 ) deallocate ( km(iseqkm)%kmeshn )
         km(iseqkm)%kmesh(27:28) = 0

      else

!     --- kmeshnew has not been filled before, create

         call sepcrmeshinfo ( kmeshnew )

      end if  ! ( iseqkm>0 )

!     --- Copy kmeshl and in R^3 kmeshm

      if ( kelml/=0 ) then

!     --- kelml/=0, hence kmeshl exists

         length = size(kmeshl)
         allocate ( km(iseqkm)%kmeshl(length))
         km(iseqkm)%kmeshl = kmeshl
         km(iseqkm)%kmesh(26) = length

      end if  ! ( kelml/=0 )

      if ( kelman>0 ) then

!     --- kmesh_quan exists, copy

         length = size(kmeshquant)
         allocate ( km(iseqkm)%kmeshquant(length))
         km(iseqkm)%kmeshquant = kmeshquant
         km(iseqkm)%kmesh(66) = length

      end if

      if ( associated(icurvsmsh) ) then
         if ( .not. associated(km(iseqkm)%icurvsmsh) ) then
            length = size(icurvsmsh)
            allocate (km(iseqkm)%icurvsmsh(length) )
            km(iseqkm)%icurvsmsh = icurvsmsh
            length = size(curvesmsh,2)
            allocate (km(iseqkm)%curvesmsh(ndim,length) )
            km(iseqkm)%curvesmsh = curvesmsh
            curvesmsh => km(iseqkm1)%curvesmsh
         end if  ! ( .not. associated(km(iseqkm)%icurvsmsh) )
      end if  ! ( associated(icurvsmsh) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms08'

      end if
      call erclos ( 'mshobstms08' )

      end
