      subroutine mshobstms2d05s ( trianglenode, trianglenodenw, &
                                  edgenode, ifacenode )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 22-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide quadratic triangle into three triangles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     quadratic
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer trianglenode(7), trianglenodenw(7,*), edgenode(5,2), &
              ifacenode(4)

!     edgenode       i    Array with nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     ifacenode      i    ?
!     trianglenode   i    Array of nodal point numbers of the corners of
!                         the original triangle
!     trianglenodenw o    Array of nodal point numbers of the corners of
!                         the new triangles.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, idx(3), edgenodet(2,2), &
              triangtemp(7), iseq, idx1(2,2), itrans(7,3), iseq1

!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     i              Counting variable
!     idx            Array containing the indices of the non-vertices
!     idx1           Array for renumbering of the linear basis functions
!     iseq           Sequence number
!     iseq1          Sequence number
!     itrans         ?
!     triangtemp     Temporary array to store the element nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     node1 := common node of edgenode(1,1), edgenode(2,1), edgenode(1,2)
!              and edgenode(2,2)
!     iseq := sequence number of node1 in trianglenode
!     node2 := next node in trianglenode
!     node3 := last node in trianglenode
!     if ( node2 in ( edgenode(1,1),edgenode(2,1) ) then
!        node4 := edge(3,1)
!        node5 := edge(3,2)
!     else ( node2 in ( edgenode(1,2),edgenode(2,2) )
!        node4 := edge(3,2)
!        node5 := edge(3,1)
!     triangle1 := (node1,node4,node5)
!     if ( angle(node5-node4,node4-node2) > angle(node3-node5,node5-node4)) then
!        triangle2 := (node4,node2,node3)
!        triangle3 := (node3,node5,node4)
!     else ( angle(node5-node4,node4-node2) < angle(node3-node5,node5-node4) )
!        triangle2 := (node5,node4,node2)
!        triangle3 := (node2,node3,node5)
!
!      1                      1
!      |\                     |\
!      | \                    | \                 5                     5
!      |  \5                  |  \ 5            /\                    /\
!      |  /\                  |  /            /   \                 / / \
!      |/   \                 |/          4  |\    \            4  |     \
!   4  |     \             4  |              |   \  \              | /    \
!      |      \  -->                         |       \     or      |       \
!      |       \                             |      \ \            |/       \
!      |        \                            ---------- ---------- &
!      ---------- 2          3          2          3
!      2          3
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1, 3, 5/
      data idx1 / 1, 2, 2, 1/
      data itrans /1, 2, 3, 4, 5, 6, 7, &
                   3, 4, 5, 6, 1, 2, 7, &
                   5, 6, 1, 2, 3, 4, 7/
! ======================================================================
!
!     --- find node connected to two intersected edges

      if ( edgenode(1,1).eq.edgenode(1,2) .or. &
           edgenode(1,1).eq.edgenode(2,2) ) then
         triangtemp(1) = edgenode(1,1)
      else
         triangtemp(1) = edgenode(2,1)
      end if

!     --- find sequence number of this node

      do i = 1, 3
         if ( trianglenode(idx(i)).eq.triangtemp(1) ) then

!        --- Node found

            iseq = i
            go to 100
         end if
      end do
 100  continue

      do i=1,7
         triangtemp(i) = trianglenode(itrans(i,iseq))
      end do

!     --- write(*,*)'oldelm', (triangtemp(i), i=1,7)

!         --- triangle1 := (node1,node4,node5)

      if ( ( triangtemp(1).eq.edgenode(1,1).and. &
             triangtemp(3).eq.edgenode(2,1) ).or. &
           ( triangtemp(1).eq.edgenode(2,1).and. &
             triangtemp(3).eq.edgenode(1,1) ) ) then
         iseq1=1
      else
         iseq1=2
      end if

      do i=1,2
         if (triangtemp(1).eq.edgenode(1,i)) then
            edgenodet(1,idx1(i,iseq1))=edgenode(3,i)
            edgenodet(2,idx1(i,iseq1))=edgenode(5,i)
         else
            edgenodet(1,idx1(i,iseq1))=edgenode(5,i)
            edgenodet(2,idx1(i,iseq1))=edgenode(3,i)
         end if
      end do

      trianglenodenw(1,1) = triangtemp(1)
      trianglenodenw(2,1) = edgenodet(1,1)
      trianglenodenw(3,1) = triangtemp(2)
      trianglenodenw(4,1) = triangtemp(7)
      trianglenodenw(5,1) = triangtemp(6)
      trianglenodenw(6,1) = edgenodet(1,2)
      trianglenodenw(7,1) = ifacenode(1)

      if (triangtemp(2).le.triangtemp(6))then

         trianglenodenw(1,2) = triangtemp(2)
         trianglenodenw(2,2) = ifacenode(4)
         trianglenodenw(3,2) = triangtemp(5)
         trianglenodenw(4,2) = edgenodet(2,2)
         trianglenodenw(5,2) = triangtemp(6)
         trianglenodenw(6,2) = triangtemp(7)
         trianglenodenw(7,2) = ifacenode(2)

         trianglenodenw(1,3) = triangtemp(2)
         trianglenodenw(2,3) = edgenodet(2,1)
         trianglenodenw(3,3) = triangtemp(3)
         trianglenodenw(4,3) = triangtemp(4)
         trianglenodenw(5,3) = triangtemp(5)
         trianglenodenw(6,3) = ifacenode(4)
         trianglenodenw(7,3) = ifacenode(3)

      else if (triangtemp(6).le.triangtemp(2)) then

         trianglenodenw(1,2) = triangtemp(2)
         trianglenodenw(2,2) = edgenodet(2,1)
         trianglenodenw(3,2) = triangtemp(3)
         trianglenodenw(4,2) = ifacenode(4)
         trianglenodenw(5,2) = triangtemp(6)
         trianglenodenw(6,2) = triangtemp(7)
         trianglenodenw(7,2) = ifacenode(2)

         trianglenodenw(1,3) = triangtemp(3)
         trianglenodenw(2,3) = triangtemp(4)
         trianglenodenw(3,3) = triangtemp(5)
         trianglenodenw(4,3) = edgenodet(2,2)
         trianglenodenw(5,3) = triangtemp(6)
         trianglenodenw(6,3) = ifacenode(4)
         trianglenodenw(7,3) = ifacenode(3)

      end if

      end
