      subroutine mshobstms3dq4 ( tetnode, tetnodenew, edgenode, &
                                 isort, nelemnew, centroid, &
                                 midfacenodes, facenode, fluidnd )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 12-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a quadratic tetrahedron with only one crossed edge
!     into 2 tetrahedrons. Subroutine based on mshobstms3d04.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(15),tetnodenew(15,2), edgenode(5,3), isort(4), &
              nelemnew, midfacenodes(4), centroid(*), facenode(4,4), &
              fluidnd(*)

!     centroid       i    ?
!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     facenode       i    ?
!     fluidnd        i    Array containing the fluidnodes that lie on the
!                         obstacle
!     isort          i    if isort(i)>isort(j) then tetnode(i) is at least on
!                         the same number of crossed edges as tetnode(j)
!     midfacenodes   i    ?
!     nelemnew       o    Number of new elements
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i, j, trans(4), transtemp, transtot(15,12), itran, &
               tetnodet(15), edgenodet(2), facenodet(2,2), idx(2), &
               facetrans(4)
      logical oddpairintchan, flip

!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     facenodet      ?
!     facetrans      ?
!     flip           ?
!     i              Counting variable
!     idx            ?
!     itran          ?
!     j              Counting variable
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     trans          Transformation array. trans(i) is the transformed
!                    local position of the i-th corner of the original
!                    tetrahedron.
!     transtemp      temporary storage
!     transtot       ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!                   the new tetrahedrons
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! --- Transform tetrahedron {tetnode(i),i=1,4} to {tetnodet(i),i=1,4}
!     such that
!     tetnodet(1) = edgenode(1,1) and tetnodet(2) = edgenode(1,2).
! --- Split tetrahedron into
!        tetrahedron1 = { tetnodet(1), edgenode(3,1), &
!                       tetnodet(3), tetnodet(4) }
!     and
!        tetrahedron2 = { edgenode(3,1), tetnodet(2), &
!                       tetnodet(3), tetnodet(4) }
!
!
!     We have the following situation
!
!
!                 tetnodet(4)
!
!                    /|\
!                   / | \ \
!                  /  |  \  \
!                 /   |   \   \  tetnodet(3)
!                /    |    \ _/|
!               /     |.  __X  |
!              /  .   | _/   \ |
!             /_______|/______\|
!
!    tetnodet(1)   edgenode(3,1)   tetnodet(2)
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     trans: transtot
!      1234: 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
!      2314: 3, 4, 5, 6, 1, 2, 8, 9, 7, 10, 11, 13, 14, 12, 15
!      3124: 5, 6, 1, 2, 3, 4, 9, 7, 8, 10, 11, 14, 12, 13, 15
!      1423: 1, 7, 10, 8, 3, 2, 6, 9, 4, 5, 12, 14, 13, 11, 15
!      2143: 3, 2, 1, 7, 10, 8, 4, 6, 9, 5, 12, 11, 14, 13, 15
!      4213: 10, 8, 3, 2, 1, 7, 9, 4, 6, 5, 12, 13, 11, 14, 15
!      1342: 1, 6, 5, 9, 10, 7, 2, 4, 8, 3, 14, 11, 13, 12, 15
!      3412: 5, 9, 10, 7, 1, 6, 4, 8, 2, 3, 14, 13, 12, 11, 15
!      4132: 10, 7, 1, 6, 5, 9, 8, 2, 4, 3, 14, 12, 11, 13, 15
!      2431: 3, 8, 10, 9, 5, 4, 2, 7, 6, 1, 13, 12, 14, 11, 15
!      3241: 5, 4, 3, 8, 10, 9, 6, 2, 7, 1, 13, 11, 12, 14, 15
!      4321: 10, 9, 5, 4, 3, 8, 7, 6, 2, 1, 13, 14, 11, 12, 15
      data transtot /1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, &
                     3, 4, 5, 6, 1, 2, 8, 9, 7, 10, 11, 13, 14, 12, 15, &
                     5, 6, 1, 2, 3, 4, 9, 7, 8, 10, 11, 14, 12, 13, 15, &
                     1, 7, 10, 8, 3, 2, 6, 9, 4, 5, 12, 14, 13, 11, 15, &
                     3, 2, 1, 7, 10, 8, 4, 6, 9, 5, 12, 11, 14, 13, 15, &
                     10, 8, 3, 2, 1, 7, 9, 4, 6, 5, 12, 13, 11, 14, 15, &
                     1, 6, 5, 9, 10, 7, 2, 4, 8, 3, 14, 11, 13, 12, 15, &
                     5, 9, 10, 7, 1, 6, 4, 8, 2, 3, 14, 13, 12, 11, 15, &
                     10, 7, 1, 6, 5, 9, 8, 2, 4, 3, 14, 12, 11, 13, 15, &
                     3, 8, 10, 9, 5, 4, 2, 7, 6, 1, 13, 12, 14, 11, 15, &
                     5, 4, 3, 8, 10, 9, 6, 2, 7, 1, 13, 11, 12, 14, 15, &
                     10, 9, 5, 4, 3, 8, 7, 6, 2, 1, 13, 14, 11, 12, 15/
      data idx /2, 1/
! ======================================================================
!
      call eropen ( 'mshobstms3dq4' )

      write(*,*)'elemnod',tetnode(1), tetnode(3), tetnode(5),tetnode(10)

!     --- Define transformation to move edgenode(3,1) to edge 1
!         This will be done by moving
!         1)  tetnode(isort(4)) to position 1
!         2)  tetnode(isort(3)) to position 2

!         --- Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move tetnode(isort(4)) to position 1

         if( isort(4).ne.1 ) then
            transtemp = trans(1)
            trans(1) = isort(4)
            trans(isort(4)) = transtemp
            oddpairintchan = .not.oddpairintchan

         end if

!        --- move tetnode(isort(3)) to position 2

         if( isort(3).ne.trans(2) ) then
            transtemp = trans(2)
            trans(2) = isort(3)

!           --- note that at this stage trans is still its own inverse

            trans(trans(isort(3))) = transtemp
            oddpairintchan = .not.oddpairintchan

         end if

!        --- make pairinterchanges even

      if( oddpairintchan ) then
         transtemp = trans(3)
         trans(3) = trans(4)
         trans(4) = transtemp
      end if

      if (trans(4).eq.4) then
         if (trans(1).eq.1) itran=1
         if (trans(1).eq.2) itran=2
         if (trans(1).eq.3) itran=3
      else if (trans(4).eq.3) then
         if (trans(1).eq.1) itran=4
         if (trans(1).eq.2) itran=5
         if (trans(1).eq.4) itran=6
      else if (trans(4).eq.2) then
         if (trans(1).eq.1) itran=7
         if (trans(1).eq.3) itran=8
         if (trans(1).eq.4) itran=9
      else if (trans(4).eq.1) then
         if (trans(1).eq.2) itran=10
         if (trans(1).eq.3) itran=11
         if (trans(1).eq.4) itran=12
      end if

!     --- Transform quadratic tetrahedron tetnode to tetnodet

      do i = 1,15
         tetnodet(i) = tetnode(transtot(i,itran))
         write(*,*)'tetnodet', tetnodet(i)
      end do

!     --- Transform the intersection points to their new positions

      if (edgenode(1,1).eq.tetnodet(1)) then
         edgenodet(1) = edgenode(3,1)
         edgenodet(2) = edgenode(5,1)
         flip=.false.
      else if (edgenode(2,1).eq.tetnodet(1)) then
         edgenodet(1) = edgenode(5,1)
         edgenodet(2) = edgenode(3,1)
         flip=.true.
      end if

      do j=1,4
         facetrans(j)=0
         do i=1,4
            if (midfacenodes(i).eq.tetnodet(10+j)) facetrans(j)=i
         end do
      end do

!     --- Transform faces

      do j=1,2
         do i=1,2
            if (.not.flip) facenodet(i,j)=facenode(i,facetrans(j))
            if (flip) facenodet(i,j)=facenode(idx(i),facetrans(j))
         end do
      end do

!     --- Subdivide tetrahedron into two tetrahedrons

      tetnodenew(1,1) = tetnodet(1)
      tetnodenew(2,1) = edgenodet(1)
      tetnodenew(3,1) = tetnodet(2)
      tetnodenew(4,1) = tetnodet(11)
      tetnodenew(5,1) = tetnodet(5)
      tetnodenew(6,1) = tetnodet(6)
      tetnodenew(7,1) = tetnodet(7)
      tetnodenew(8,1) = tetnodet(12)
      tetnodenew(9,1) = tetnodet(9)
      tetnodenew(10,1) = tetnodet(10)
      tetnodenew(11,1) = facenodet(1,1)
      tetnodenew(12,1) = facenodet(1,2)
      tetnodenew(13,1) = tetnodet(15)
      tetnodenew(14,1) = tetnodet(14)
      tetnodenew(15,1) = centroid(1)

      tetnodenew(1,2) = tetnodet(2)
      tetnodenew(2,2) = edgenodet(2)
      tetnodenew(3,2) = tetnodet(3)
      tetnodenew(4,2) = tetnodet(4)
      tetnodenew(5,2) = tetnodet(5)
      tetnodenew(6,2) = tetnodet(11)
      tetnodenew(7,2) = tetnodet(12)
      tetnodenew(8,2) = tetnodet(8)
      tetnodenew(9,2) = tetnodet(9)
      tetnodenew(10,2) = tetnodet(10)
      tetnodenew(11,2) = facenodet(2,1)
      tetnodenew(12,2) = facenodet(2,2)
      tetnodenew(13,2) = tetnodet(13)
      tetnodenew(14,2) = tetnodet(15)
      tetnodenew(15,2) = centroid(2)

      nelemnew = 2

      fluidnd(1)=tetnodet(2)
      fluidnd(2)=tetnodet(11)
      fluidnd(3)=tetnodet(5)
      fluidnd(4)=tetnodet(9)
      fluidnd(5)=tetnodet(10)
      fluidnd(6)=tetnodet(12)
      fluidnd(7)=tetnodet(15)

      call erclos ( 'mshobstms3dq4' )

      end
