      subroutine mshobstms3dq7d ( tetnodet, tetnodenew, facenodet, &
                                  centroid,  edgenodet )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 15-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron with four crossed edges which form a closed
!     curve.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnodet(15),tetnodenew(15,6), edgenodet(2,6), &
              facenodet(4,4), centroid(*)

!     centroid       i    Array containing the centroid numbers of the
!                         subtetrahedrons
!     edgenodet      i    Array of global nodal point numbers of point on edge i
!                         after transformation
!     facenodet      i    Array containing the centroids of the faces of
!                         the subtetrahedrons
!     tetnodenew     i    New kmesh part c
!                         In this case only the new elements are stored
!     tetnodet       i    Array of global nodal point numbers of the corners of
!                         the transformed original tetrahedron
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!                   the new tetrahedrons
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms3dq7d' )

         tetnodenew(1,1) = tetnodet(2)
         tetnodenew(2,1) = tetnodet(11)
         tetnodenew(3,1) = tetnodet(6)
         tetnodenew(4,1) = tetnodet(14)
         tetnodenew(5,1) = tetnodet(9)
         tetnodenew(6,1) = tetnodet(15)
         tetnodenew(7,1) = facenodet(4,1)
         tetnodenew(8,1) = edgenodet(2,3)
         tetnodenew(9,1) = edgenodet(2,6)
         tetnodenew(10,1) = tetnodet(5)
         tetnodenew(11,1) = centroid(7)
         tetnodenew(12,1) = facenodet(2,1)
         tetnodenew(13,1) = facenodet(1,4)
         tetnodenew(14,1) = centroid(8)
         tetnodenew(15,1) = centroid(1)

      if( tetnodet(6).lt.tetnodet(9) ) then

         tetnodenew(1,2) = tetnodet(1)
         tetnodenew(2,2) = edgenodet(1,1)
         tetnodenew(3,2) = tetnodet(2)
         tetnodenew(4,2) = tetnodet(11)
         tetnodenew(5,2) = tetnodet(6)
         tetnodenew(6,2) = edgenodet(1,3)
         tetnodenew(7,2) = tetnodet(7)
         tetnodenew(8,2) = facenodet(4,2)
         tetnodenew(9,2) = facenodet(4,4)
         tetnodenew(10,2) = tetnodet(10)
         tetnodenew(11,2) = facenodet(1,1)
         tetnodenew(12,2) = facenodet(3,2)
         tetnodenew(13,2) = centroid(9)
         tetnodenew(14,2) = facenodet(3,4)
         tetnodenew(15,2) = centroid(2)

         tetnodenew(1,3) = tetnodet(2)
         tetnodenew(2,3) = tetnodet(11)
         tetnodenew(3,3) = tetnodet(6)
         tetnodenew(4,3) = facenodet(4,4)
         tetnodenew(5,3) = tetnodet(10)
         tetnodenew(6,3) = facenodet(4,2)
         tetnodenew(7,3) = tetnodet(15)
         tetnodenew(8,3) = tetnodet(14)
         tetnodenew(9,3) = edgenodet(1,6)
         tetnodenew(10,3) = tetnodet(9)
         tetnodenew(11,3) = centroid(9)
         tetnodenew(12,3) = centroid(7)
         tetnodenew(13,3) = facenodet(2,4)
         tetnodenew(14,3) = centroid(10)
         tetnodenew(15,3) = centroid(3)

      else

         tetnodenew(1,2)  = tetnodet(1)
         tetnodenew(2,2)  = edgenodet(1,1)
         tetnodenew(3,2)  = tetnodet(2)
         tetnodenew(4,2)  = tetnodet(11)
         tetnodenew(5,2)  = tetnodet(6)
         tetnodenew(6,2)  = edgenodet(1,3)
         tetnodenew(7,2)  = facenodet(4,4)
         tetnodenew(8,2)  = tetnodet(15)
         tetnodenew(9,2)  = tetnodet(14)
         tetnodenew(10,2)  = tetnodet(9)
         tetnodenew(11,2)  = facenodet(1,1)
         tetnodenew(12,2)  = centroid(9)
         tetnodenew(13,2)  = centroid(7)
         tetnodenew(14,2)  = facenodet(2,4)
         tetnodenew(15,2)  = centroid(2)

         tetnodenew(1,3)  = tetnodet(1)
         tetnodenew(2,3)  = tetnodet(7)
         tetnodenew(3,3)  = tetnodet(10)
         tetnodenew(4,3)  = facenodet(4,2)
         tetnodenew(5,3)  = tetnodet(2)
         tetnodenew(6,3)  = edgenodet(1,1)
         tetnodenew(7,3)  = facenodet(4,4)
         tetnodenew(8,3)  = edgenodet(1,6)
         tetnodenew(9,3)  = tetnodet(15)
         tetnodenew(10,3)  = tetnodet(9)
         tetnodenew(11,3)  = facenodet(3,2)
         tetnodenew(12,3)  = facenodet(3,4)
         tetnodenew(13,3)  = centroid(10)
         tetnodenew(14,3)  = centroid(9)
         tetnodenew(15,3)  = centroid(3)

      end if

      tetnodenew(1,4) = tetnodet(2)
      tetnodenew(2,4) = facenodet(4,2)
      tetnodenew(3,4) = tetnodet(10)
      tetnodenew(4,4) = edgenodet(2,5)
      tetnodenew(5,4) = tetnodet(8)
      tetnodenew(6,4) = tetnodet(12)
      tetnodenew(7,4) = tetnodet(15)
      tetnodenew(8,4) = edgenodet(1,6)
      tetnodenew(9,4) = tetnodet(13)
      tetnodenew(10,4) = tetnodet(9)
      tetnodenew(11,4) = facenodet(2,2)
      tetnodenew(12,4) = centroid(10)
      tetnodenew(13,4) = facenodet(1,3)
      tetnodenew(14,4) = centroid(11)
      tetnodenew(15,4) = centroid(4)

      if( tetnodet(8).lt.tetnodet(9) ) then

         tetnodenew(1,5) = tetnodet(8)
         tetnodenew(2,5) = edgenodet(1,5)
         tetnodenew(3,5) = tetnodet(3)
         tetnodenew(4,5) = edgenodet(2,1)
         tetnodenew(5,5) = tetnodet(2)
         tetnodenew(6,5) = tetnodet(12)
         tetnodenew(7,5) = facenodet(4,3)
         tetnodenew(8,5) = tetnodet(4)
         tetnodenew(9,5) = facenodet(4,1)
         tetnodenew(10,5) = tetnodet(5)
         tetnodenew(11,5) = facenodet(1,2)
         tetnodenew(12,5) = facenodet(3,3)
         tetnodenew(13,5) = facenodet(3,1)
         tetnodenew(14,5) = centroid(12)
         tetnodenew(15,5) = centroid(5)

         tetnodenew(1,6) = tetnodet(2)
         tetnodenew(2,6) = tetnodet(15)
         tetnodenew(3,6) = tetnodet(9)
         tetnodenew(4,6) = tetnodet(13)
         tetnodenew(5,6) = tetnodet(8)
         tetnodenew(6,6) = tetnodet(12)
         tetnodenew(7,6) = facenodet(4,1)
         tetnodenew(8,6) = edgenodet(2,6)
         tetnodenew(9,6) = facenodet(4,3)
         tetnodenew(10,6) = tetnodet(5)
         tetnodenew(11,6) = centroid(11)
         tetnodenew(12,6) = centroid(8)
         tetnodenew(13,6) = facenodet(2,3)
         tetnodenew(14,6) = centroid(12)
         tetnodenew(15,6) = centroid(6)

      else

         tetnodenew(1,5) = tetnodet(8)
         tetnodenew(2,5) = edgenodet(1,5)
         tetnodenew(3,5) = tetnodet(3)
         tetnodenew(4,5) = edgenodet(2,1)
         tetnodenew(5,5) = tetnodet(2)
         tetnodenew(6,5) = tetnodet(12)
         tetnodenew(7,5) = tetnodet(13)
         tetnodenew(8,5) = facenodet(4,3)
         tetnodenew(9,5) = tetnodet(15)
         tetnodenew(10,5) = tetnodet(9)
         tetnodenew(11,5) = facenodet(1,2)
         tetnodenew(12,5) = facenodet(2,3)
         tetnodenew(13,5) = centroid(12)
         tetnodenew(14,5) = centroid(11)
         tetnodenew(15,5) = centroid(5)

         tetnodenew(1,6) = tetnodet(2)
         tetnodenew(2,6) = tetnodet(15)
         tetnodenew(3,6) = tetnodet(9)
         tetnodenew(4,6) = facenodet(4,3)
         tetnodenew(5,6) = tetnodet(3)
         tetnodenew(6,6) = edgenodet(2,1)
         tetnodenew(7,6) = facenodet(4,1)
         tetnodenew(8,6) = edgenodet(2,6)
         tetnodenew(9,6) = tetnodet(4)
         tetnodenew(10,6) = tetnodet(5)
         tetnodenew(11,6) = centroid(12)
         tetnodenew(12,6) = centroid(8)
         tetnodenew(13,6) = facenodet(3,3)
         tetnodenew(14,6) = facenodet(3,1)
         tetnodenew(15,6) = centroid(6)

      end if

      call erclos ( 'mshobstms3dq7d' )
      end
