      subroutine mshobstsurfs ( kmeshnnew, lenkmsnnew, &
                                iedges, ielmfaces, idimedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-11-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new array kmeshn in the case that extra points at the surface
!     are created when a new mesh is created due to intersection with an
!     obstacle (3D only)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     surface
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshnnew(*),  idimedges, iedges(idimedges,*), lenkmsnnew, &
              ielmfaces(4,*)

!     idimedges      i    Length of first dimension of array iedges
!                         (linear:1, quadratic: 3)
!     iedges         i    Work array of size nedges
!     ielmfaces      i    Contains the faces corresponding to the elements
!     kmeshnnew      o    New array kmeshn
!     lenkmsnnew     i    Length of array kmshnnew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nedgenode, inpelm

!     inpelm         number of points per element
!     nedgenode      Length of first index of edgenode
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHOBSTSURF    Create new array kmeshn in the case that extra points at
!                    the surface are created when a new mesh is created due to
!                    intersection with an obstacle (3D only)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2751   This number of nodes per element in surface is not programmed yet
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstsurfs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstsurfs'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      inpelm = maxval(kmeshn(1:nsurfs))

      if ( inpelm==3 ) then

!     --- Linear elements, hence length of edgenode is 3

         nedgenode = 3

      else if ( inpelm==6 .or. inpelm==7 ) then

!     --- Quadratic elements, hence length of edgenode is 5

         nedgenode = 5

      else

!     --- Number of nodes incorrect

         call errint ( inpelm, 1 )
         call errint ( 1, 2 )
         call errsub ( 2751, 2, 0, 0 )
         go to 1000

      end if  ! ( inpelm==3 )
      if ( debug ) write(irefwr,1) 'nedgenode', nedgenode

      call mshobstsurf ( kmeshnnew, lenkmsnnew, iedges, ielmfaces, idimedges, &
                         nedgenode )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstsurfs'

      end if  ! ( debug )
      call erclos ( 'mshobstsurfs' )

      end
