      subroutine mshpavbnopp ( inode, nmeshl, numnodes, lenopposite, &
                               nodopposite, markedges, nmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all nodes on opposite boundary connected to node 1 in array
!     nodopposite
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     edge
!     boundary
!     opposite
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode, nmeshl(*), numnodes, lenopposite, nodopposite(*), &
              markedges(*), nmeshc(*)

!     inode          i    Node sequence number
!     lenopposite    i    Length of array nodopposite
!     markedges      i    array of length nedges indicating whether a
!                         edge is internal or on the boundary
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
!     nodopposite    o    Array of length lenopposite containing numnodes
!                         nodes on opposite boundary
!     numnodes       o    Number of nodal points in array nodopposite
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, numedges, i, iedge

!     i              Counting variable
!     iedge          edge sequence number
!     ipstart        Starting address in array
!     numedges       number of edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     ININODONEDGE   Find other end node on edge if one of the end nodes and
!                    the ege number are given
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2661   No triangle could be found corresponding to given edges
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavbnopp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavbnopp'

      end if
      if ( ierror.ne.0 ) go to 1000

      ipstart = nmeshl(inode)
      numedges = nmeshl(ipstart)  ! number of edges connected to node 1
      numnodes = 0                ! number of nodes on opposite boundary
                                  ! connected to inode
      do i = 1, numedges

         iedge = nmeshl(ipstart+i)  ! edge connected to node 1
         if ( markedges(iedge).eq.-2 ) then

!        --- edge connects two opposite boundaries
!            store end node in nodopposite

            numnodes = numnodes+1
            call inichkarlen ( numnodes, lenopposite, 'nodopposite' )
            if ( ierror.ne.0 ) go to 1000

            call ininodonedge ( iedge, inode, nodopposite(numnodes), &
                                nmeshc )

         end if  ! ( markedges(iedge).eq.-2 )

      end do  ! i = 1, numedges

      if ( numnodes.le.1 ) then

!     --- Less than 2 nodes on opposite boundary (should be impossible)

         call errint ( inode, 1 )
         call errint ( numnodes, 2 )
         call errsub ( 2661, 2, 0, 0 )
         go to 1000

      end if  ! ( numnodes.le.1 )

1000  call erclos ( 'mshpavbnopp' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavbnopp'

      end if

      end
