      subroutine mshpavbnquad3 ( itrian1, itrian2, iedge1, iedge5, &
                                 iedge3, nmeshh, nmeshd, markedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-03-2009 Jump in case quadrilateral does not exist
!        version  1.1  date 10-03-2005 Fortran 90
!        version  1.0  date 07-07-2004
!
!   copyright (c) 2004-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find triangle corresponding to two edges as well as opposite triangle
!     corresponding to "diagonal" edge
!     Find boundary edge corresponding to second triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     boundary
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itrian1, itrian2, iedge1, iedge5, iedge3, nmeshh(*), &
              nmeshd(*), markedges(*)

!     iedge1         i    edge sequence number of boundary edge
!     iedge3         o    edge sequence number of opposite boundary edge
!     iedge5         i    edge sequence number of "diagonal" edge
!     itrian1        o    Sequence number of first triangle
!     itrian2        o    Sequence number of second triangle
!     markedges      i    array of length nedges indicating whether a
!                         edge is internal or on the boundary
!                         -2:  internal edge between 2 boundary nodes
!                         -1:  boundary edge
!                          0:  standard internal edge
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     nmeshh         i    nmesh part h, contains element numbers corresponding
!                         to edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, i, iedge

!     i              Counting variable
!     iedge          edge sequence number
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer initrianedge

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INITRIANEDGE   Find triangle corresponding to 2 given edges
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavbnquad3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavbnquad3'
         write(irefwr,1) 'iedge1, iedge5', iedge1, iedge5
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      iedge3 = 0

!     --- itrian1 := triangle spanned by e1, e4 and e5

      itrian1 = initrianedge ( iedge1, iedge5, nmeshh )
      if ( ierror/=0 .or. itrian1==0 ) go to 1000

!     --- itrian2 := other triangle connected to edge 5

      ipstart = nmeshh(iedge5)
      if ( itrian1==nmeshh(ipstart+1) ) then

!     --- itrian2 := nmeshh(ipstart+2)

         itrian2 = nmeshh(ipstart+2)

      else

!     --- itrian2 := nmeshh(ipstart+1)

         itrian2 = nmeshh(ipstart+1)

      end if  ! ( itrian1==nmeshh(ipstart+1) )

!     --- e3 := boundary edge in itrian2

      ipstart = nmeshd(itrian2)
      do i = 1, 3
         iedge = abs(nmeshd(ipstart+i))
         if ( markedges(iedge)==-1 ) iedge3 = iedge
      end do

1000  call erclos ( 'mshpavbnquad3' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'itrian1, itrian2', itrian1, itrian2
         write(irefwr,1) 'iedge3', iedge3
         write(irefwr,*) 'End mshpavbnquad3'

      end if

      end
