      subroutine mshpavsubtrian ( itrian, nmeshc, iworkedge, kmeshc, &
                                  nel, coor, nmeshd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if triangle is candidate for subdivision into more triangles
!     and if so subdivide this triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     paver
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itrian, nmeshc(*), iworkedge(*), kmeshc(3,*), nel, &
              nmeshd(*)
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     itrian         i    sequence number of triangle
!     iworkedge      i    work array of length nedges to be used to store
!                         new node numbers on edges
!                          0:  not an extra node
!                         >0:  node number of extra node
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nel           i/o   number of elements in the topology
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsubdiv, ipstart, iedge, iedgediv(2), inodediv(2), i
      double precision area, areabefore, areaend
      logical checkarea

!     area           Area of element
!     areabefore     Area of region before subdivision
!     areaend        Area of region after subdivision
!     checkarea      If true the area must be checked
!     i              Counting variable
!     iedge          edge sequence number
!     iedgediv       Contains edge numbers of the edges to be subdivided
!     inodediv       Contains extra node numbers on the edges to be
!                    subdivided
!     ipstart        Starting address in array
!     nsubdiv        number of edges to be subdivided
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRAREA        Print error message if area is negative
!     MSHAREA        Compute the area of the triangle spanned by x1-x2-x3
!                    The sign of the are defines the direction
!     MSHPAVSUB1TRIA Subdivide one triangle into 2, because one of the edges is
!                    splitted, into 2 parts
!     MSHPAVSUB3TRIA Subdivide triangle into 3 triangles because 2 edges have
!                    been splitted into 2 parts
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nsubdiv := number of edges that must be sudivided
!     if ( nsubdiv = 1 ) then
!        Subdivide triangle into 2 triangles
!     else if ( nsubdiv = 2 ) then
!        Subdivide triangle into 3 triangles
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavsubtrian' )
      debug = .false.
      checkarea = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavsubtrian'
         write(irefwr,*) 'itrian ', itrian
         call prinin ( kmeshc(1,itrian), 3, 'triangle' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- nsubdiv := number of edges that must be sudivided

      nsubdiv = 0
      ipstart = nmeshd(itrian)           ! starting address of triangle
      do i = 1, 3

         iedge = abs(nmeshd(ipstart+i))  ! edge in triangle
         if ( iworkedge(iedge).gt.0 ) then

!        --- iworkedge(iedge) > 0, edge must be subdivided

            nsubdiv = nsubdiv+1
            iedgediv(nsubdiv) = iedge
            inodediv(nsubdiv) = iworkedge(iedge)

         end if  ! ( iworkedge(iedge).gt.0 )

      end do  ! i = 1, 3

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'nusbdiv ', nsubdiv
         if ( nsubdiv.eq.1 ) then

!        --- nsubdiv = 1

            write(irefwr,*) 'iedgediv, inodediv ', &
               iedgediv(1), inodediv(1)

         else if ( nsubdiv.eq.2 ) then

!        --- nsubdiv = 1

            write(irefwr,*) 'iedgediv ', iedgediv, '   inodediv ', &
               inodediv

         end if  ! ( nsubdiv.eq.1 )

      end if  ! ( debug )
      if ( nsubdiv.gt.0 .and. checkarea ) then
         call msharea ( coor(1,kmeshc(1,itrian)), &
                        coor(1,kmeshc(2,itrian)), &
                        coor(1,kmeshc(3,itrian)), areabefore )
         if ( areabefore.le.0d0 ) call  errarea ( areabefore, itrian )
      end if  ! ( nsubdiv.gt.0 )

      if ( nsubdiv.eq.1 ) then

!     --- nsubdiv = 1
!         Subdivide triangle into 2 triangles

         call mshpavsub1tria ( iedgediv(1), nmeshc, itrian, kmeshc, nel, &
                               inodediv(1) )

         if ( checkarea ) then

!        --- Debug information

            call msharea ( coor(1,kmeshc(1,itrian)), &
                           coor(1,kmeshc(2,itrian)), &
                           coor(1,kmeshc(3,itrian)), areaend )
            if ( areaend.le.0d0 ) call errarea ( areaend, itrian )
            call msharea ( coor(1,kmeshc(1,nel)), &
                           coor(1,kmeshc(2,nel)), &
                           coor(1,kmeshc(3,nel)), area )
            if ( area.le.0d0 ) call errarea ( area, nel )
            areaend = areaend+area
            if ( abs(areaend-areabefore).gt.sqreps*areaend ) then

!           --- Error area before not same as area after

               write(irefwr,*) 'areabefore, areaend ', &
                                areabefore, areaend
               call prinin ( kmeshc(1,itrian), 3, 'triangle 1' )
               call prinin ( kmeshc(1,nel), 3, 'triangle 2' )

            end if  ! ( abs(areaend-areabefore).gt.sqreps*areaend )

         end if  ! ( debug )

      else if ( nsubdiv.eq.2 ) then

!     --- nsubdiv = 2
!         Subdivide triangle into 3 triangles

         call mshpavsub3tria ( iedgediv, nmeshc, itrian, kmeshc, nel, &
                               inodediv, coor )

         if ( checkarea ) then

!        --- Debug information

            call msharea ( coor(1,kmeshc(1,itrian)), &
                           coor(1,kmeshc(2,itrian)), &
                           coor(1,kmeshc(3,itrian)), areaend )
            if ( areaend.le.0d0 ) call errarea ( areaend, itrian )
            nel = nel-1
            call msharea ( coor(1,kmeshc(1,nel)), &
                           coor(1,kmeshc(2,nel)), &
                           coor(1,kmeshc(3,nel)), area )
            if ( area.le.0d0 ) call errarea ( area, nel )
            areaend = areaend+area
            nel = nel+1
            call msharea ( coor(1,kmeshc(1,nel)), &
                           coor(1,kmeshc(2,nel)), &
                           coor(1,kmeshc(3,nel)), area )
            if ( area.le.0d0 ) call errarea ( area, nel )
            areaend = areaend+area
            if ( abs(areaend-areabefore).gt.sqreps*areaend ) then

!           --- Error area before not same as area after

               write(irefwr,*) 'areabefore, areaend ', &
                                areabefore, areaend
               call prinin ( kmeshc(1,itrian), 3, 'triangle 1' )
               nel = nel-1
               call prinin ( kmeshc(1,nel), 3, 'triangle 2' )
               nel = nel+1
               call prinin ( kmeshc(1,nel), 3, 'triangle 3' )

            end if  ! ( abs(areaend-areabefore).gt.sqreps*areaend )

         end if  ! ( debug )

      end if  ! ( nsubdiv.eq.1 ) then

1000  call erclos ( 'mshpavsubtrian' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavsubtrian'

      end if

      end
