      subroutine mshplaxis31( ntri, ico, coor, xn )

! ======================================================================
!
!        programmer    Paul Bonnier (Plaxis)
!        version  1.0  date 17-11-2008        Adjusted NP
!
!   copyright (c) 2008-2008  "Plaxis BV and Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Plaxis BV or Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine the avarage value of the normals on asurface of triangles
!     in a 3D-space.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     normal
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!      None
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ntri, ico(3,ntri)
      double precision coor(3,*), xn(3)

!     coor           i    3D array containing the coordinates of the nodes
!     ico            i    array containing the nodes of the triangles
!     ntri           i    number of triangles in surface
!     xn             o    avarage (to all triangles) normal vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, n1, n2, n3
      double precision a(3), b(3), c(3), w, wl, x

!     a         vector from node 1 to node 2 of triangle
!     b         vector from node 1 to node 3 of triangle
!     c         crossproduct vector ( a X b )
!     i         loop counter
!     j         loop counter
!     n1        node number of triangle
!     n2        node number of triangle
!     n3        node number of triangle
!     x         variable to store length
!     w         total weight
!     wl        local weight (=area/2 of triangle)
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! ***********************************************************************
!
!     Pseudo code
!
!
! ***********************************************************************

!     --- Set vectors to 0

      xn = 0
      w  = 0
!     --- Run through all triangles
      do i = 1, ntri
!       --- Find the three nodes
        n1 = ico(1,i)
        n2 = ico(2,i)
        n3 = ico(3,i)
!       --- Find the two base vectors
        do j=1,3
          a(j) = coor(j,n2) - coor(j,n1)   ! vector from node 1 to node 2
          b(j) = coor(j,n3) - coor(j,n1)   ! vector from node 1 to node 3
        end do
!       --- Compute crossproduct
        call eloutr ( a, b, c )
!       --- Find weight for vector c:
        wl = dot_product ( c, c )
        wl = sqrt( wl )
!       --- Add to total weight
        w  = w  + wl
!       --- Add components to total "vector"
        xn = xn + c
      end do

!     --- Adjust vector for total weight:
      xn = xn / w
!     --- Normalize:
      x = dot_product ( xn, xn )
      if ( x>0 ) xn= xn / sqrt(x)

      end ! mshplaxis31
