      subroutine mshplaxread ( isurnr, nnodes, ntria, iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-06-2013 Extra parameter iref
!        version  1.0  date 18-07-2007
!
!   copyright (c) 2007-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read number of nodes and triangles that defines the frame for the
!     submesh generator framesurf
!     Open the file
!     The file to be read is framesurf_x, where x is the surface number
!     No leading zeros in x!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr, nnodes, ntria, iref

!     iref           o    file reference number
!     isurnr         i    Surface sequence number
!     nnodes         o    Number of nodes in triangle mesh
!     ntria          o    number of triangles in triangle mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenname, ios
      character (len=13) filename
      logical check

!     check          Indicates if the file exist (true) or not
!     filename       Name of file
!     ios            IOSTAT variable, returns from reading or writing
!     lenname        Length of name
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCHKINT       Check if integer is between two values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
!     This subroutine reads a file with name framesurf_x
!     where x is the surface number
!     The file must be organized as follows:
!     Line 1: n (number of points), m  ( number of triangles)
!     Line 2 ... n+1:  node number followed by 3 coordinates
!     Line n+2, n+m+1: 3 node numbers that form a triangle
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     146   surface number has incorrect value
!    1669   file does not exist
!    2706   error in opening file
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create file name
!     Check if file exists
!     Read contents of file
!     Check contents
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshplaxread' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshplaxread'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iref = inisetnewref()  ! reference number of file

!     --- Create file name

      filename = 'framesurf_'
      if ( isurnr<0 .or. isurnr>=10000 ) then

!     --- Surf number larger than 9999, not yet implemented

         call errint ( isurnr, 1 )
         call errint ( 10000, 2 )
         call errsub ( 146, 2, 0, 0 )
         go to 1000

      else if ( isurnr<10 ) then

!     --- Surf number between 1 and 9

         lenname = 10
         write(filename(10:lenname),100) isurnr
100      format(i1)

      else if ( isurnr<100 ) then

!     --- Surf number between 10 and 99

         lenname = 11
         write(filename(10:lenname),110) isurnr
110      format(i2)

      else if ( isurnr<1000 ) then

!     --- Surf number between 100 and 999

         lenname = 12
         write(filename(10:lenname),120) isurnr
120      format(i3)

      else

!     --- Surf number between 1000 and 9999

         lenname = 13
         write(filename(10:lenname),130) isurnr
130      format(i4)

      end if  ! ( isurnr0 .or. isurnr>=10000 )

!     --- Check if file exists

      inquire ( file=filename, exist = check )
      if ( .not. check ) then

!     --- Error: file does not exits

         call errchr ( filename, 1 )
         call errsub ( 1669, 0, 0, 1 )
         go to 1000

      end if  ! ( .not. fileexist )
      open ( unit = iref, file = filename, iostat = ios )
      if ( ios/=0 ) then

!     --- error in opening file

         call errint ( ios, 1 )
         call errchr ( filename, 1 )
         call errsub ( 2706, 1, 0, 1 )
         go to 1000

      end if

!     --- Read number of nodes and number of triangles

      read(iref,*) nnodes, ntria
      call erchkint ( nnodes, 1, 10000, 'number of nodes' )
      call erchkint ( ntria, 1, 10000, 'number of triangles' )

1000  call erclos ( 'mshplaxread' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isurnr, nnodes, ntria', isurnr, nnodes, ntria
         write(irefwr,*) 'End mshplaxread'

      end if  ! ( debug )

      end
