      subroutine mshpointgrps ( nprocessors, pointgrps, &
                                npointproc, nworkprocs )
! ======================================================================
!
!        programmer    Dragan Vidovic
!        version  1.0  date 26-09-2005
!
!   copyright (c) 1983-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Defines submeshes such that each submesh is connected
!
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulework
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nprocessors, pointgrps(*), npointproc, nworkprocs

!     npointproc     i    Number of nodes per processor (approximately)
!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
!     nworkprocs     i    reserved size of matrix connects
!     pointgrps      i    for each point contains the number of submesh it
!                         belongs to
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ithres, ithres1, nspare, ibig, i, j, nsmallest, ismallest, &
              ismconn, nsmconn, ihlp, &
              totcount, node, nset, jcount, mset, nsetl, icountl, msetl
      logical newsub, startlayer
      integer :: grpsize(npoint), sparenodes(npoint), connected(npoint), &
                 connects(nworkprocs,nworkprocs)
      integer, pointer :: istorel(:), ihelpl(:), itemp(:)

!     connected      connected(i)=j if spare node i is connected to submesh j
!     connects       connects(i,j)=1 if submeshes i and j are connected
!     grpsize        array containing sizes of submeshes
!     i              Counting variable
!     ibig           big integer number
!     icountl        number of local layers
!     ihelpl         integer work array
!     ihlp           help variable
!     ismallest      index of the smallest submesh
!     ismconn        The submesh to which the smallest submesh will be joinded
!     istorel        Contains the last local level set
!     itemp          integer work array
!     ithres         Submesh number
!     ithres1        Submesh number
!     j              Counting variable
!     jcount         Counter
!     mset           number of nodal points in a submesh
!     msetl          local number of nodal points
!     newsub         wether to start a new submesh
!     node           Node number
!     nset           Number of elements in a set of points
!     nsetl          Number of elements in a local set of points
!     nsmallest      size of the smallest submesh
!     nsmconn        size of the submesh with index ismconn
!     nspare         number of spare nodes
!     sparenodes     array of spare nodes not belonging to any submesh
!     startlayer     wether to start a submesh with a layer or with a node
!     totcount       total number of points used in submeshes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPNEXTLEVEL   Find neighbors corresponding to a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! 10 While not all points were used
!    start a new submesh
!    200 find a new layer
!    check if the layer is connected or find a connected chunk and put the
!                                rest in array sparenodes
! put the connected chunk in the submesh if there is space and go to 200
!                               or go to 10
! while there are too many submeshs
!    find the smallest submesh
!    find the smallest submesh connected to the smallest submesh
!    join these two submeshes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpointgrps' )
      debug = .false.

      allocate ( marknodes(npoint), ihelp(npoint), istore(npoint), &
                 ihelpl(npoint), istorel(npoint), itemp(npoint) )
      startlayer = .false.
      ibig = 99999
      pointgrps(1:npoint) = ibig
      marknodes = 0
      connects = 0
      nspare = 0
      totcount = 0
      ithres = 0
      jcount = 0
      if (startlayer) then

!     --- Start from the first connected chunk of the first layer
!         defined in kmeshk

         nset = kmeshk(1)
         marknodes(kmeshj(2:nset)) = 2
         marknodes(kmeshj(1)) = 1
         nsetl = 1
         istorel(1) = kmeshj(1)
         icountl = 1

5        if ( nsetl>0 ) then

!        --- Find a 1d layer in the 2d layer

            lastlevel => istorel(1:nsetl)
            nextlevel => ihelpl
            call sepnextlevel ( 1, msetl )
            nsetl = msetl
            istorel(1:nsetl) = ihelpl(1:nsetl)
            icountl = icountl + nsetl
            goto 5
         end if

         if ( icountl<nset ) then

!        --- The initial layer is disconnected; use only the first chunk;
!            put the rest in the sparenodes array

            if (debug) then
               write(irefwr,*) 'disconnected starting layer. Size', &
                               nset, ', chunk size', icountl
            end if

            do i = 1, nset
               if ( marknodes(kmeshj(i))==1 ) then
                  nset = nset + 1
                  istore(nset) = kmeshj(i)
                  pointgrps(kmeshj(i)) = 1
               end if
            end do
         else

!        --- The initial layer is connected

            if (debug) then
               write(irefwr,*) 'Starting layer size:', nset
            end if

            istore(1:nset) = kmeshj(1:nset)
            pointgrps(istore(1:nset)) = 1
         end if

!        --- Reset marknodes

         marknodes(ihelp(1:nset)) = 0

      else

!     --- Start from the first renumbered node

         node = kmeshj(1)
         nset = 1
         pointgrps(node) = 1
         istore(1) = node

      end if

10    if ( totcount+jcount<npoint ) then

!     --- Start a new submesh

         if ( ithres>0 ) then
            grpsize(ithres) = jcount
            if (debug) then
               write(irefwr,*) ithres, '. submesh size:', &
                               grpsize(ithres), ', max:', npointproc
               write(irefwr,*) 'starting a new submesh'
            end if
         end if
         totcount = totcount + jcount
         ithres = ithres + 1
         ithres1 = ithres + 1
         jcount = nset
         mset = nset

200      if ( mset>0 ) then

!        --- Find neighbors of set in istore, put in ihelp

            lastlevel => istore(1:nset)
            nextlevel => ihelp
            itemp = marknodes      ! save marknodes in itemp
            marknodes = pointgrps(1:npoint) ! copy pointgrps in marknodes
            call sepnextlevel ( ithres, mset )
            pointgrps(1:npoint) = marknodes ! copy new marknodes in pointgrps
            marknodes = itemp ! reset marknodes

            if ( mset>0 ) then

!           --- Check if the layer is connected

               marknodes(ihelp(2:mset)) = 2
               marknodes(ihelp(1)) = 1
               nsetl = 1
               istorel(1) = ihelp(1)
               icountl = 1

300            if ( nsetl>0 ) then

!              --- Find a 1d layer in the 2d layer

                  lastlevel => istorel(1:nsetl)
                  nextlevel => ihelpl
                  call sepnextlevel ( 1, msetl )
                  nsetl = msetl
                  istorel(1:nsetl) = ihelpl(1:nsetl)
                  icountl = icountl + nsetl
                  goto 300
               end if

               if ( jcount+icountl>=npointproc .and. &
                        totcount+jcount+icountl<npoint ) then
                  newsub = .true.
                  connects(ithres,ithres1) = 1
                  connects(ithres1,ithres) = 1
                  if (debug) then
                     write(irefwr,*) 'submesh full: size', jcount, &
                         ', new layer size:', icountl, ', max:', &
                            npointproc
                  end if
               else
                  newsub = .false.
                  jcount = jcount+icountl
                  if (debug) then
                     write(irefwr,*) 'Layer size:', icountl, &
                      ', total submesh size:', jcount
                  end if
               end if

               if ( icountl<mset ) then

!              --- The layer is disconnected; use only the first chunk;
!                  put the rest in the sparenodes array

                  if (debug) then
                     write(irefwr,*) 'disconnected layer. Size', mset, &
                     ', chunk size', icountl
                  end if

                  nset = 0
                  do i = 1, mset
                     if ( marknodes(ihelp(i))==1 ) then
                        nset = nset + 1
                        istore(nset) = ihelp(i)
                        if ( newsub ) then
                           pointgrps(ihelp(i)) = ithres1
                        end if
                     else
                        nspare = nspare + 1
                        sparenodes(nspare) = ihelp(i)
                        connected(nspare) = ithres
                        pointgrps(ihelp(i)) = ibig
                     end if

                  end do
               else
                  do i = 1, mset
                     istore(i) = ihelp(i)
                     if ( newsub ) then
                        pointgrps(ihelp(i)) = ithres1
                     end if
                  end do
                  nset = mset
               end if

!              --- Reset marknodes

               marknodes(ihelp(1:mset)) = 0
               if ( newsub ) then
                  goto 10
               end if

            end if

            goto 200

         end if

!        --- mset==0

         if ( totcount+jcount<npoint ) then

!        --- find where to start a new submesh

            do i=1,nspare
               if ( pointgrps(sparenodes(i))==ibig ) then
                  node = sparenodes(i)
                  connects(ithres1,connected(i)) = 1
                  connects(connected(i),ithres1) = 1
                  go to 600
               end if
600         end do

!           --- Find a connected chunk in sparenodes to start with

            do i = 1,nspare
               marknodes(sparenodes(i)) = 2
            end do
            marknodes(node) = 1
            nsetl = 1
            istorel(1) = node
            icountl = 1

700         if ( nsetl>0 ) then

!           --- Find a layer in sparenodes

               lastlevel => istorel(1:nsetl)
               nextlevel => ihelpl
               call sepnextlevel ( 1, msetl )
               nsetl = msetl
               do i=1,nsetl
                  istorel(i) = ihelpl(i)
               end do
               icountl = icountl + nsetl
               goto 700
            end if

            nset = 0

            do i=1,nspare
               if ( marknodes(sparenodes(i))==1 ) then
                  pointgrps(sparenodes(i)) = ithres1
                  nset = nset + 1
                  istore(nset) = sparenodes(i)
               end if
            end do

!           --- reset marknodes

            do i = 1,nspare
               marknodes(sparenodes(i)) = 0
            end do

            if (debug) then
               write(irefwr,*) &
                  'Boundary reached, there are no more points here'
            end if
            goto 10
         end if
      end if

      grpsize(ithres) = jcount

      if (debug) then
         write(irefwr,*) ithres, '. submesh size:',grpsize(ithres), &
                         ', max:', npointproc
         nset = 0
         do i=1,ithres
            nset = nset + grpsize(i)
         end do
         write(irefwr,*) 'total', nset, ' out of', npoint
      end if

800   if ( ithres>nprocessors ) then

!     --- there are too many submeshes; find the smallest one and join it to the
!         smallest other submesh to which it is connected

         nsmallest = grpsize(1)
         ismallest = 1
         do i=2, ithres
            if ( grpsize(i)<nsmallest ) then
               nsmallest = grpsize(i)
               ismallest = i
            end if
         end do

         nsmconn = npoint+1

         do i=1, ithres
            if ( connects(ismallest,i)==1 ) then
               if ( grpsize(i)<nsmconn ) then
                  nsmconn = grpsize(i)
                  ismconn = i
               end if
            end if
         end do

!        --- Joinn submeshes ismallest and ismconn

         if ( ismallest<ismconn ) then
            ihlp = ismallest
            ismallest = ismconn
            ismconn = ihlp
         end if
         do i=1,npoint
            if ( pointgrps(i)==ismallest ) then
               pointgrps(i) = ismconn
            else if ( pointgrps(i)>ismallest ) then
               pointgrps(i) = pointgrps(i) - 1
            end if
         end do
         grpsize(ismconn) = grpsize(ismconn) + grpsize(ismallest)
         do i=1,ithres
            if ( connects(i,ismallest)==1 .and. i/=ismconn ) then
               connects(i,ismconn) = 1
               connects(ismconn,i) = 1
            end if
         end do
         do i=ismallest+1,ithres
            grpsize(i-1)=grpsize(i)
            do j=1,ithres
               connects(j,i-1) = connects(j,i)
               connects(i-1,j) = connects(i,j)
            end do
         end do
         ithres = ithres - 1

         if (debug) then
            write(irefwr,*) 'Joined groups', ismconn, ' and', ismallest
            write(irefwr,*) 'New group sizes:', (grpsize(i),i=1,ithres)
         end if
         go to 800

      end if
      deallocate ( marknodes, ihelp, istore, ihelpl, istorel, itemp )

      if ( debug ) then

!     --- Debug information

         call prinin ( pointgrps, npoint, 'pointgrps' )

      end if  ! ( debug )
      call erclos ( 'mshpointgrps' )

      end
