      subroutine mshrdmeshinput ( iinput, rinput, ichois, jlocal, &
                                  ireadr, krenum, lengin, lengrl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 30-08-2013 Extra debug
!        version  3.0  date 30-07-2012 Remove ibuffr,buffer from parameter list
!        version  2.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read input from standard input file
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      use sepmoduledebug
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, iinput(*), ireadr, &
              krenum(*), lengin, lengrl
      double precision rinput(*)
      logical jlocal

!     ichois         i    Choice parameter, see subroutine meshbody
!     iinput         o    integer input array for subroutine MESH
!                         The array is filled as described from position 6
!     ireadr         i    If 1 the first record has been read by READSP
!     jlocal        i/o   indication whether iinput and rinput are local arrays
!                         (jlocal=true), i.e. stored in IBUFFR or not (false)
!     krenum        i/o   Array of length 10 with information about the
!                         renumbering.
!     lengin         o    Length of integer array iinput
!     lengrl         o    Length of real array rinput
!                         (in double precision variables)
!     rinput         o    real input array for subroutine MESH
!                         The array is filled as described from position 6
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lniinp, lnrinp
      double precision cunit, maxratio
      logical :: adaptiinput
      integer, allocatable :: iinputcp(:)
      double precision, allocatable :: rinputcp(:)

!     adaptiinput    If true iinput must be adapted
!     cunit          Unit length for coarseness
!     iinputcp       Copy of array iinput
!     lniinp         Available space for array IINPUT
!     lnrinp         Available space for array RINPUT
!     maxratio       Maximal ratio between succeeding elements
!     rinputcp       Copy of array rinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHIINPEXT     Extend arrays iinput and rinput with extra curves and
!                    surfaces
!     MSHRDA         Read first part of the mesh input and set some constants
!     MSHRDB         reads all records with respect to the option NODAL_POINTS
!     MSHREA         Read mesh input
!     MSHWRITIRINP   write the arrays iinput and rinput created by sepmesh
!                    to a file
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     WRITMSHI       Write arrays iinput and rinput in original format to
!                    a file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrdmeshinput' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdmeshinput'
         write(irefwr,1) 'lengin, lengrl, ireadr', lengin, lengrl, ireadr
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- jread = true   read input from standard input file
!                        First compute some constants

      call mshrda ( ndim, cunit, maxpnt, maxcrv, maxsrf, &
                    maxvlm, jcoars, ireadr, ispecial, &
                    iscale, itype_scale, maxratio )

      if ( debug ) then
         write(irefwr,1) 'jcoars, maxpnt, maxcrv, maxsrf, maxvlm', &
                          jcoars, maxpnt, maxcrv, maxsrf, maxvlm
         write(irefwr,1) 'ispecial, iscale, itype_scale, ndim', &
                          ispecial, iscale, itype_scale, ndim
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lniinp = lengin  ! available space for iinput
      lnrinp = lengrl  ! available space for rinput

      if ( ispecial==1 ) then

!     --- Special case: the user defines the mesh by giving the co-ordinates
!         of the nodes himself

         call mshrdb ( ndim, iinput, rinput, lengrl, &
                       lengin, lnrinp, lniinp )
         go to 1000

      end if

      call mshrea ( iinput, rinput, lengrl, lengin, &
                    lnrinp, lniinp, cunit, adaptiinput, maxratio )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         call prinin ( iinput, lengin, 'iinput/1' )
         call prinrl ( rinput, lengrl, 'rinput/1' )
      end if  ! ( debug )

      if ( write_iinput_mesh ) then

!     --- write_iinput_mesh = true, write input and rinput to file
!         meshinprinp

         call mshwritirinp ( iinput, rinput, lengin, lengrl )

      end if  ! ( write_iinput_mesh )

      if ( adaptiinput ) then

!     --- The input array must be extended with extra curves and surfaces
!         First the extension takes place in new positions

         allocate ( iinputcp(lengin), rinputcp(lengrl), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, lengin+lengrl, 'iinputnw' )
         if ( ierror/=0 ) go to 1000

         iinputcp(1:lengin) = iinput(1:lengin)
         rinputcp(1:lengrl) = rinput(1:lengrl)

         call mshiinpext ( iinputcp, rinputcp, krenum, jcoars, &
                           iinput, rinput, ichois, lniinp, lnrinp )

         if ( idebug>0 .and. ioutp>0 ) then

!        --- Debugging and printing are both activated
!            write information about new mesh

            call writmshi ( iinput, rinput, 6 )

         end if

         lengin = iinp8
         lengrl = irnp3

         deallocate ( iinputcp, rinputcp, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iinputnw' )

      end if  ! ( adaptiinput )

      jlocal = .true.

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinput, lengin, 'iinput' )
         call prinrl ( rinput, lengrl, 'rinput' )
         write(irefwr,*) 'End mshrdmeshinput'

      end if  ! ( debug )
      call erclos ( 'mshrdmeshinput' )

      end
