      subroutine mshreadkmesht ( ios, irefc, ivers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-05-2012 Extra debug
!        version  1.0  date 21-03-2007
!
!   copyright (c) 2007-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read kmesh part t from file meshoutput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ios, irefc, ivers

!     ios           i/o   IOSTAT variable, returns from reading or writing
!     irefc          i    dataset reference number for reading (>0 formatted, &
!                         <0 unformatted)
!     ivers          i    Version sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ilast, ifirst, n
      character (len=80) text

!     i              Counting variable
!     ifirst         First entry in row i of kmesh part t
!     ilast          Last entry in row i of kmesh part t
!     n              number of volumes/surfaces/curves
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2869   ifirst out of range
!    2870   ilast<ifirst or ilast> nelem
!    2871   ifirst = 0 and ilast # 0
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreadkmesht' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreadkmesht'
  1      format ( a, 1x, (10i9) )
100      format ( a )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read kmesht

      n = nvolms
      if ( n==0 ) n = nsurfs
      if ( n==0 ) n = ncurvs
      if ( debug ) write(irefwr,1) 'n', n
      if ( irefc>=0 ) then
         if ( ivers>=7 ) read (irefc,100,iostat=ios) text
         read(irefc, *) kmesht
      else
         if ( ivers>=7 ) read(-irefc,iostat=ios) text
         read(-irefc)  kmesht
      end if
      if ( debug ) then
         write(irefwr,*) text
         write(irefwr,1) 'irefc, ivers', irefc, ivers
         call prinin1 ( kmesht, n, 2, 'kmesht' )
      end if  ! ( debug )

!     --- Check kmesht

      do i = 1, n
         ifirst = kmesht(1,i)
         if ( ifirst<0 .or. ifirst>nelem ) then

!        --- error: ifirst out of range

            call errint ( i, 1 )
            call errint ( ifirst, 2 )
            call errint ( nelem, 3 )
            call errsub ( 2869, 3, 0, 0 )

         end if  ! ( ifirst/=ilast+1)

         ilast = kmesht(2,i)
         if ( ilast<ifirst .or. ilast>nelem ) then

!        --- error: ilast<ifirst or ilast> nelem

            call errint ( i, 1 )
            call errint ( ilast, 2 )
            call errint ( ifirst, 3 )
            call errint ( nelem, 4 )
            call errsub ( 2870, 4, 0, 0 )

         end if  ! ( ilast<ifirst .or. ilast>nelem )

         if ( ifirst==0 .and. ilast/=0 ) then

!        --- error: ifirst=0, but ilast is not

            call errint ( i, 1 )
            call errsub ( 2871, 1, 0, 0 )

         end if  ! ( ifirst==0 .and. ilast/=0 )

      end do  ! i = 1, n

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshreadkmesht'

      end if  ! ( debug )
      call erclos ( 'mshreadkmesht' )

      end
