      subroutine mshreflocinp ( iinputrefloc, typerefine, irefarray )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2008 Use subroutine checkrange
!        version  1.0  date 20-10-2004
!
!   copyright (c) 2004-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extract parameters from input array for subroutine refineloc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     input
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputrefloc(*), typerefine, irefarray

!     iinputrefloc   i    Integer input array for subroutine inirefloc
!                         This array is filled as follows:
!                         Pos. 1 last entry that has been filled by the user
!                         Pos. 2 irefarray  (Default value: 1 )
!                         Pos. 3 typerefine  (Default value: 1 )
!     irefarray      o    Sequence number of marker array in array isol
!     typerefine     o    Type of criterion to be used
!                         Possible values
!                         1: The marker array in usol is filled with 0 and 1
!                            All elements indicated by 1 are refined
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leniinput
      parameter (leniinput=2)
      integer iinput(leniinput), i

!     i              Counting variable
!     iinput         integer input array
!     leniinput      Length of array iinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHECKRANGE     Check range of integer parameter
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set default values
!     Replace by input from iinputrefloc
!     extract parameters
!     test parameters
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreflocinp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreflocinp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set default values

      do i = 1, leniinput
         iinput(i) = 1
      end do  ! i = 1, leniinput

!     --- Replace by input from iinputrefloc

      do i = 2, min(leniinput+1,iinputrefloc(1))
         iinput(i-1) = iinputrefloc(i)
      end do  ! i = 2, min(leniinput+1,iinputrefloc(1))

!     --- extract parameters

      typerefine = iinput(1)
      irefarray = iinput(2)

!     --- test parameters

      call checkrange ( typerefine, 0, 1, 2, 'typerefine', &
                        'iinputrefloc' )

      call checkrange ( irefarray, 1, 100, 3, 'irefarray', &
                        'iinputrefloc' )

1000  call erclos ( 'mshreflocinp' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshreflocinp'

      end if

      end
