      subroutine mshrefmap ( ielemold, ielemnew, refmapold, refmapnew, &
                             refarray, ireflvl, ielem )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 21-04-2005
!
!   copyright (c) 2005-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array refmap. For each element of the newest mesh, array refmap
!     contains 3 integers: element number of new mesh, element number in
!     basis mesh (mother element), required level of refinement of mother
!     element
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielem, ielemnew, ielemold, ireflvl, refmapnew(2,*), &
              refmapold(2,*), refarray(*)

!     ielem          i    Element number
!     ielemnew       i    Sequence number of element in new mesh
!     ielemold       i    Sequence number of element in old mesh
!     ireflvl        i    Current level of refinement
!     refarray       i    Reference array containing the required level
!                         of refinement per element of the basis mesh
!     refmapnew     i/o   Mapping array
!     refmapold      i    Mapping array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nnewelems, istart

!     i              Counting variable
!     istart         starting postition for writing to array refmap
!     nnewelems      Number of new elements after refinement
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefmap' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefmap'
         print *, 'ielemold, ielemnew, ielem, ireflvl', &
                   ielemold, ielemnew, ielem, ireflvl
         call prinin(refarray, 50, 'refarray')
      end if
      if ( ierror/=0 ) go to 1000

!     --- If ireflvl==1 array refmap does not yet exist. It does not need to
!         since all information needed is to be found in refarray

      nnewelems = ielemnew - ielemold
      if (debug) print *, 'nnewelems', nnewelems
      istart = ielemold
      if (ireflvl==1) then
         do i = 1, nnewelems
            refmapnew(1,istart+i) = ielem
            refmapnew(2,istart+i) = refarray(ielem)
            if (debug) print *, 'refmapnew', &
                     refmapnew(1,istart+i), refmapnew(2,istart+i)
         end do
      else
         do i = 1, nnewelems
            refmapnew(1,istart+i) = refmapold(1,ielem)
            refmapnew(2,istart+i) = refmapold(2,ielem)
            if (debug) print *, 'refmapnew', &
                     refmapnew(1,istart+i), refmapnew(2,istart+i)
         end do
      end if

1000  call erclos ( 'mshrefmap' )
      if ( debug ) then

!     --- Debug information

         call prinin(refmapnew, 750, 'refmapnew')
         write(irefwr,*) 'End mshrefmap'

      end if

      end
