      subroutine mshrefmapping ( nelem_basis, nodeextrael, npoint_basis, &
                                 itempold, itempnew, &
                                 reflvl, first, refine, npoint_new )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.1  date 20-10-2005 Obsolete arrays are deleted (JB)
!        version  1.0  date 25-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new temperature vector and map the solution vector from the
!     old mesh to the new mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem_basis, reflvl, &
              npoint_basis, nodeextrael(0:*), &
              itempold(5), npoint_new, itempnew(5)
      logical first, refine

!     first          i    If true it concerns the first step
!     itempnew       i    Temperature at the current time level
!     itempold       i    Temperature at the previous time level
!     nelem_basis    i    Number of elements in basis mesh
!     nodeextrael    i    Array of size maxreflvl that contains the number
!                         of extra nodes per element defined as function
!                         of the refinement level
!     npoint_basis   i    Number of points in basis mesh
!     npoint_new     i    Number of points of the new mesh
!     refine         i
!     reflvl         i    Level of refinement
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHREFCOPY     Copy a refinement array such that all elements marked
!                    as intersected are 'normalized'
!     MSHREFMAPPING1 Map the solution vector from the old mesh to the new mesh
!     PRINRL         Print 1d real vector
!     SEPCRSOLINFO   Create a new vector in ks
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     initialize some arrays
!     copy all values for the nodes of the basis mesh
!     for each element of the basis mesh and all points of the old mesh
!     that are not mapped yet
!        copy nodes in case refinement old mesh>=refinement new mesh
!        interpolate nodes when refinement new mesh<refinement old mesh
!     end
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefmapping' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefmapping'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Create itempnew

      itempnew(2:4) = itempold(2:4)
      call sepcrsolinfo ( itempnew(1), npoint_new, 0 )
      itempnew(5) = npoint_new
      tempnew => ks(itempnew(1))%sol
      tempold => ks(itempold(1))%sol

!     --- When the mesh is not refined, it suffices to make a copy
!         of the old temperature vector.
!         Otherwise, perform a mapping from the old mesh to the new one.

      if (refine) then

!     --- Both refineold as well as refinenew will contain
!         refinement levels that are reflvl + 1 in case these elements
!         are intersected by the moving interface, For our mapping, &
!         this information is not essential. Even more so, it affects
!         our mapping. Therefore, we create a local copy of both arrays
!         in which we set all values of reflvl + 1 equal to reflvl!

         allocate ( refinenewloc(size(refinenew)), &
                    refineoldloc(size(refineold)) )
         refinenewloc = refinenew
         refineoldloc = refineold

         call mshrefcopy (refinenewloc,refinenew, &
                          reflvl, nelem_basis)

         if (first) then

!        --- If this is the first time this routine is called, array refineold
!            might contain refinement information, used to activate the enthalpy
!            solver. This information is unwanted here, so we temporarily
!            continue with an emptied version. If we don't, we will interpolate
!            between non existing nodes, since intnodesold has not been filled
!            in accordance with refineold (we might not have actually refined
!            the
!            mesh yet, but just pretended to have done so, based on the initial
!            solution!)

             refineoldloc(1:nelem_basis) = 0

         else

            call mshrefcopy (refineoldloc,refineold, &
                             reflvl, nelem_basis)

         end if

!        --- Perform the actual mapping based on the local refine arrays

         call mshrefmapping1(npoint_basis, tempnew, &
                          tempold, nelem_basis, &
                          refineoldloc,refinenewloc, &
                          reflvl, intnodesold, &
                          intnodesnew, nodeextrael, parentnodes )

!        --- Remove local permanent arrays

         deallocate ( refineoldloc, refinenewloc )

      else

!     --- Copy temperature to temperature_new

         tempnew = tempold

      end if ! refine

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'npoint_new', npoint_new
         call prinrl(tempnew, npoint_new, 'tempnew (m)')
         call prinrl(tempold, npoint_new, 'tempold (m)')
         write(irefwr,*) 'End mshrefmapping'

      end if
      call erclos ( 'mshrefmapping' )

      end subroutine mshrefmapping
