      subroutine mshrefpar ( kmeshnew, &
                             newtransinfo, lennewtrans, newblockinfo, &
                             leng2, npoint_glob, newmapping, istorest )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 04-05-2015 Fill globnodesnew
!        version  1.1  date 05-08-2014 set pointers
!        version  1.0  date 25-04-2012
!
!   copyright (c) 2012-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make arrays transinfo, blockinfo and kmeshpar parmanent
!
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lennewtrans, kmeshnew, &
              leng2, newblockinfo(leng2), npoint_glob, &
              newmapping(*), istorest(4,*), newtransinfo(lennewtrans)

!     istorest       o    array of length (4,iacnodes) containing information
!                         for the various blocks
!                         istorest(1,iblock):  starting address of block iblock
!                         istorest(2,iblock):  number of edges of block iblock
!                         istorest(3,iblock):  starting address in transportinfo
!                         istorest(4,iblock):  sequence number of neighbor
!     kmeshnew       i    New array kmesh
!     leng2          i    Length of array newblockinfo
!     lennewtrans    i    Length of array newtransinfo
!     newblockinfo   i    New array blockinfo
!     newmapping     i    contains global node numbers for refined mesh
!     newtransinfo   i    New array transportinfo
!     npoint_glob   i/o   Number of nodes in global mesh
!                         Input: in old mesh
!                         Output: in new mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenkmeshm, lenkmeshn, iseqkm2, &
                 lenkmeshcglob, lenkmeshcglnw, lenkmeshc, lenkmeshmglob
      integer, allocatable :: kmeshmglobnew(:), kmeshcglobnew(:)
      integer, pointer :: kmeshparnew(:)

!     iseqkm2        Sequence number with respect to km of second mesh
!     kmeshcglobnew  Array kmeshcglob corresponding to new mesh
!     kmeshmglobnew  Array kmeshmglob corresponding to new mesh
!     kmeshparnew    kmeshpar for new mesh
!     lenkmeshc      Length of new array kmeshc
!     lenkmeshcglnw  Length of new array kmeshcglob
!     lenkmeshcglob  Length of array kmeshcglob
!     lenkmeshm      Length of array kmeshm
!     lenkmeshmglob  Length of array kmeshmglob
!                    In first instance declared length
!                    Finally computed length
!     lenkmeshn      Length of array kmeshn
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHREFPAR1     Fill array mappingnew with the global node numbers
!     MSHREFPAR12    Copy global arrays coor_glob, kmeshm/n_glob from node 1
!                    to other nodes
!     MSHREFPAR13    Receive global arrays coor_glob, kmeshm/n_glob from node 1
!     MSHREFPAR14    Fill array globnodesnew
!     MSHREFPAR2     Send local arrays newmapping, coor, kmeshmnew and kmeshnnew
!                    to node 1
!     MSHREFPAR3     Receive local arrays newmapping, coor, kmeshm and kmeshn
!                    from other nodes
!                    Merge results with global arrays
!     PRININ         print 1d integer array
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPPRINKMESH   Print array KMESH
!     SEPPRINKMESHP1 Print contents of array kmeshpar
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar', inodenr
         call sepprinkmesh ( 6 )
         call sepprinkmesh ( 49 )
         if ( inodenr==1 ) call sepprinkmeshp1
         write(irefwr,1) 'lennewtrans, leng2', lennewtrans, leng2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm2 = kmeshnew

!     --- Make arrays newtransinfo and newblockinfo permanent

      if ( associated(km(iseqkm2)%transinfo) ) &
         deallocate ( km(iseqkm2)%transinfo)
      allocate ( km(iseqkm2)%transinfo(lennewtrans) )
      km(iseqkm2)%transinfo = newtransinfo(1:lennewtrans)
      km(iseqkm2)%kmesh(53) = lennewtrans

      if ( associated(km(iseqkm2)%blockinfo) ) &
         deallocate ( km(iseqkm2)%blockinfo)
      allocate ( km(iseqkm2)%blockinfo(leng2) )
      km(iseqkm2)%blockinfo = newblockinfo(1:leng2)
      km(iseqkm2)%kmesh(54) = leng2

      if ( ierror/=0 ) go to 1000

!     --- Compute global node numbers and store in newmapping

      if ( debug ) write(irefwr,*) 'before mshrefpar1'
      call mshrefpar1 ( iseqkm2, newmapping, istorest, &
                        newtransinfo, mappingparcp, &
                        npoint_glob )
      if ( debug ) write(irefwr,*) 'after mshrefpar1'

!     --- Fill array kmeshnewpar

      if ( associated(km(iseqkm2)%kmeshpar) ) &
         deallocate ( km(iseqkm2)%kmeshpar)
      allocate ( km(iseqkm2)%kmeshpar(10) )
      km(iseqkm2)%kmesh(65) = 10
      kmeshparnew => km(iseqkm2)%kmeshpar
      kmeshparnew = 0

      kmeshparnew(1) = npoint_glob  ! global number of nodes

      length = km(iseqkm2)%kmesh(8)
      if ( associated(km(iseqkm2)%mappingpar) ) &
         deallocate ( km(iseqkm2)%mappingpar)
      allocate ( km(iseqkm2)%mappingpar(length) )
      km(iseqkm2)%mappingpar = newmapping(1:length)
      kmeshparnew(6) = length

!     --- Fill array globnodesnew

      call mshrefpar14 ( iseqkm2 )

!     --- Copy kmesh part l from old kmesh

      length = km(iseqkm2)%kmesh(10)
      if ( associated(km(iseqkm2)%kmeshlglob) ) &
         deallocate ( km(iseqkm2)%kmeshlglob)
      allocate ( km(iseqkm2)%kmeshlglob(length) )
      km(iseqkm2)%kmeshlglob = kmeshlglob(1:length)
      kmeshparnew(3) = length

!     --- Reserve space for coornew

      if ( debug ) &
         write(irefwr,1) 'ndim, npoint_glob, iseqkm2, inodenr', &
                          ndim, npoint_glob, iseqkm2, inodenr

      length = npoint_glob
      if ( associated(km(iseqkm2)%coorglob) ) &
         deallocate ( km(iseqkm2)%coorglob)
      allocate ( km(iseqkm2)%coorglob(ndim,npoint_glob) )
      kmeshparnew(2) = ndim*npoint_glob

      if ( debug ) call prinin ( newmapping, npoint, 'newmapping' )

!     --- coor_glob, kmeshl_glob, kmeshc_glob, kmeshm_glob, kmeshn_glob

      if ( inodenr>1 ) then

!     --- inodenr>1, send arrays coor, kmeshc, kmeshm and kmeshn to node 1

         if ( debug ) write(irefwr,*) 'before mshrefpar2'
         call mshrefpar2 ( iseqkm2, km(iseqkm2)%coor, newmapping, &
                           km(iseqkm2)%kmeshm, &
                           km(iseqkm2)%kmeshn, km(iseqkm2)%kmeshc )
         if ( debug ) write(irefwr,1) 'iseqkm2/2', iseqkm2

      else

!     --- inodenr = 1, receive arrays coor, kmeshm and kmeshn from other nodes

         lenkmeshcglob = size ( kmeshcglob )
         lenkmeshmglob = size ( kmeshmglob )*2
         lenkmeshcglnw = max(2*lenkmeshcglob,ndim**2*lenkmeshcglob)

         allocate ( kmeshmglobnew(lenkmeshmglob), &
                    kmeshcglobnew(lenkmeshcglnw), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, lenkmeshmglob+lenkmeshcglnw, &
                           'kmeshmglobnew' )
         if ( ierror/=0 ) go to 1000

         lenkmeshm = size ( km(iseqkm2)%kmeshm )
         lenkmeshn = 0
         if ( ierror/=0 ) go to 1000

         if ( debug ) write(irefwr,*) 'before mshrefpar3', inodenr
         call mshrefpar3 ( iseqkm2, km(iseqkm2)%coorglob, &
                           km(iseqkm2)%kmeshm, &
                           lenkmeshm, lenkmeshn, km(iseqkm2)%coor, &
                           newmapping,  kmeshmglobnew, &
                           lenkmeshmglob, kmeshcglobnew, &
                           km(iseqkm2)%kmeshc, lenkmeshcglnw )
         if ( ierror/=0 ) go to 1000

!        --- Store array kmeshmglobnew in ibuffr

         length = lenkmeshmglob
         if ( associated(km(iseqkm2)%kmeshmglob) ) &
            deallocate ( km(iseqkm2)%kmeshmglob)
         allocate ( km(iseqkm2)%kmeshmglob(length) )
         km(iseqkm2)%kmeshmglob = kmeshmglobnew(1:length)
         kmeshparnew(4) = length

!        --- Store array kmeshcglobnew in ibuffr

         length = lenkmeshcglnw
         if ( kmeshparnew(7)>0 ) deallocate ( km(iseqkm2)%kmeshcglob)
         allocate ( km(iseqkm2)%kmeshcglob(length) )
         km(iseqkm2)%kmeshcglob = kmeshcglobnew(1:length)
         kmeshparnew(7) = length

         deallocate ( kmeshmglobnew, kmeshcglobnew, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshmglobnew' )

!        --- Copy global arrays coor_glob, kmeshm/n_glob to other nodes

         call mshrefpar12 ( iseqkm2 )

      end if  ! ( inodenr>1 )

!     --- Receive global arrays coor_glob, kmeshm/n_glob from node 1

      if ( debug ) write(irefwr,*) 'before mshrefpar13'
      if ( inodenr>1 ) call mshrefpar13 ( iseqkm2 )

!     --- Copy mappingnew to reference in kmeshpar(9)

      length = km(iseqkm2)%kmesh(8)
      if ( associated(km(iseqkm2)%mappingparcp) ) &
         deallocate ( km(iseqkm2)%mappingparcp)
      allocate ( km(iseqkm2)%mappingparcp(length) )
      km(iseqkm2)%mappingparcp = newmapping(1:length)
      kmeshparnew(9) = length

1000  if ( debug ) then

!     --- Debug information

         call sepfillkmesh(kmeshnew)
         call sepprinkmesh ( 49 )
         if ( inodenr==1 ) call sepprinkmeshp1
         write(irefwr,*) 'End mshrefpar'

      end if  ! ( debug )
      call erclos ( 'mshrefpar' )

      end
