      subroutine mshrefpar13 ( iseqkm2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Receive global arrays coor_glob, kmeshm/n_glob from node 1
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     parallel
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqkm2

!     iseqkm2        i    Sequence number with respect to km of second mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iblock, icontext, lencoor, lenkmeshm, lenkmeshn
      integer, allocatable :: iwork(:)
      integer, pointer :: kmeshparnew(:)

!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     iwork          integer work array to store information sent by other nodes
!     kmeshparnew    kmeshpar for new mesh
!     lencoor        Length of array COOR
!     lenkmeshm      Length of array kmeshm
!     lenkmeshn      Length of array kmeshn
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIRECEIVEMPI  Receive information from other node (mpi)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar13' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar13'
         if ( debug ) write(irefwr,1) 'iseqkm2, inodenr', iseqkm2, inodenr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Receive coor array

      lencoor = size ( km(iseqkm2)%coorglob )
      kmeshparnew => km(iseqkm2)%kmeshpar

      if ( debug ) write(irefwr,1) 'lencoor', lencoor, inodenr

      icontext = 27
      call inireceivempi ( km(iseqkm2)%coorglob, lencoor, SPDOUBLE, iblock, &
                           icontext )

      if ( debug ) write(irefwr,1) 'iblock, lencoor', iblock, lencoor, inodenr

!     --- allocate buffer array iwork to receive information

      length = 4*lencoor
      allocate ( iwork(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Receive kmeshm

      icontext = 28
      lenkmeshm = length
      call inireceivempi ( iwork, lenkmeshm, SPINT, iblock, &
                           icontext )
      call inichkarlen ( lenkmeshm, length, 'kmeshm' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'iblock, lenkmeshm', iblock, lenkmeshm, inodenr

!     --- Create kmeshm

      if ( associated(km(iseqkm2)%kmeshmglob) ) &
         deallocate ( km(iseqkm2)%kmeshmglob)
      allocate ( km(iseqkm2)%kmeshmglob(lenkmeshm) )
      kmeshparnew(4) = lenkmeshm
      km(iseqkm2)%kmeshmglob = iwork(1:lenkmeshm)

      if ( ndim>2 ) then

!     --- ndim> 2, kmesh part n

         icontext = 29
         lenkmeshn = length
         call inireceivempi ( iwork, lenkmeshn, SPINT, iblock, &
                              icontext )
         call inichkarlen ( lenkmeshn, length, 'kmeshn' )
         if ( ierror/=0 ) go to 1000

         if ( associated(km(iseqkm2)%kmeshnglob) ) &
            deallocate ( km(iseqkm2)%kmeshnglob)
         allocate ( km(iseqkm2)%kmeshnglob(lenkmeshn) )
         kmeshparnew(5) = lenkmeshn
         km(iseqkm2)%kmeshnglob = iwork(1:lenkmeshn)

      end if  ! ( ndim>2 )

!     --- Remove buffer array

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefpar13'

      end if  ! ( debug )
      call erclos ( 'mshrefpar13' )

      end subroutine mshrefpar13
