      subroutine mshrefpar14 ( iseqkm2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array globnodesnew
!
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
!     mesh
!     node
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqkm2

!     iseqkm2        i    Sequence number with respect to km of second mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: npointglobnw, i
      integer, pointer :: kmeshparnew(:), globnodesnew(:), mappingparnew(:)
      integer :: istart, icount, iblock, icontext, npointlc
      integer, allocatable :: mappingnew(:)

!     globnodesnew   array globnodes for new mesh
!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     icount         Counter
!     istart         First position used at a part
!     kmeshparnew    kmeshpar for new mesh
!     mappingnew     Array containing all mapping arrays for all cpu's
!                    Contents:
!                    Pos. 1-iacnodes: starting addresses per cpu
!                    For each cpu from its starting address:
!                    number of nodes in cpu, followed by mapping array
!     mappingparnew  Array mappingpar for new mesh
!     npointglobnw   New number of global points
!     npointlc       Local number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBCASTMPI    Wrapper for MPI_BCAST
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     INISENDMPI     Send information to other node (mpi)
!     MSHREFPAR15    Fill array globnodes
!     PRINGLOBNODES  Print the contents of array globnodes
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     create array globnodesnew as permanent array
!     if ( inodenr==1 ) then
!        activate old globnodes
!        receive arrays newmapping from other blocks
!        Fill array globnodesnew
!        send arrays globnodesnew to other blocks
!     else
!        send arrays newmapping from other blocks
!        receive array globnodesnew from block 1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar14' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar14'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      kmeshparnew => km(iseqkm2)%kmeshpar

      npointlc = km(iseqkm2)%kmesh(8)
      npointglobnw = kmeshparnew(1)
      if ( debug ) then
         write(irefwr,*) 'old globnodes'
         call pringlobnodes ( globnodes, globnodes(1+iacnodes) )
      end if  ! ( debug )

!     --- create array globnodesnew as permanent array


      length = npointglobnw+iacnodes
      if ( associated(km(iseqkm2)%globnodes) ) &
         deallocate ( km(iseqkm2)%globnodes)
      allocate ( km(iseqkm2)%globnodes(length) )
      globnodesnew => km(iseqkm2)%globnodes
      kmeshparnew(8) = length
      mappingparnew => km(iseqkm2)%mappingpar

      if ( inodenr==1 ) then

!     --- node number 1 (host)
!         activate old globnodes

         allocate ( mappingnew(npointglobnw*iacnodes), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, npointglobnw*iacnodes, 'mappingnew' )
         if ( ierror/=0 ) go to 1000

!        --- Store new mapping array into mappingnew

         istart = iacnodes+1
         mappingnew(1) = istart         ! start of cpu 1
         mappingnew(istart) = npointlc    ! number of nodes
         mappingnew(istart+1:istart+npointlc) = &
            mappingparnew(1:npointlc) ! mapping array
         if ( debug ) call prinin (mappingnew, istart+npointlc, 'mappingnew' )
         istart = istart+npointlc+1        ! raise istart

!        --- receive arrays newmapping from other blocks

         icontext = 91
         do i = 2, iacnodes

            icount = npointglobnw*iacnodes-istart-1
            call inireceivempi ( mappingnew(istart+1), icount, SPINT, &
                                 iblock, icontext )
            mappingnew(istart) = icount    ! number of nodes
            mappingnew(iblock) = istart    ! starting addressof cpu iblock
            istart = istart+icount+1        ! raise istart

         end do  ! i = 2, iacnodes

!        --- Fill array globnodesnew

         call mshrefpar15 ( globnodesnew(1+iacnodes), &
                            globnodesnew, npointglobnw, &
                            mappingnew )

      else

!     --- node number>1
!         send arrays newmapping from other blocks

         icontext = 91
         call inisendmpi ( mappingparnew, npointlc, SPINT, 1, icontext )

      end if  ! ( inodenr==1 )

!     --- send arrays globnodesnew to all blocks from host

      call inibcastmpi ( globnodesnew, npointglobnw+iacnodes, SPINT )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'new globnodes'
         call pringlobnodes ( globnodesnew, globnodesnew(1+iacnodes) )
         write(irefwr,*) 'End mshrefpar14'

      end if  ! ( debug )
      call erclos ( 'mshrefpar14' )

      end subroutine mshrefpar14
