      subroutine mshrefpar15 ( globnodes, numnodesnew, npointglobnw, &
                               mapping )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array globnodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
!     mesh
!     node
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: npointglobnw, mapping(*)
      integer, intent(out) :: globnodes(npointglobnw), &
                              numnodesnew(iacnodes)

!     globnodes      o    Array of size npoint_glob containing the numbering of
!                         the global nodes under the condition that
!                         first all nodes of the first cpu are numbered, &
!                         then of the second one and so on.
!                         A global node has only one sequence number
!                         defined by the lowest cpu it is positioned on
!                         In this case it refers to the new mesh
!     mapping        i    contains global node numbers for old mesh
!     npointglobnw   i    New number of global points
!     numnodesnew    o    Contains the number of nodes that correspond
!                         to a cpu accumulated
!                         Nodes that belong to a cpu with lower number are
!                         excluded
!                         In this case it refers to the new mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icpu, icount, istart, npointloc, locnode, nodp

!     icount         Counter
!     icpu           Loop variable over the various cpu's
!     istart         First position of information for a cpu
!     locnode        Loop variable for local nodes
!     nodp           Nodal point sequence number
!     npointloc      Number of points in local mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar15' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar15'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      globnodes = 0
      icount = 0

!     --- For all blocks do

      do icpu = 1, iacnodes

         istart = mapping(icpu)
         npointloc = mapping(istart)

         if ( debug ) then
            write(irefwr,1) 'icpu, istart, npointloc', icpu, istart, npointloc
            call prinin ( mapping(istart+1), npointloc, 'mapping' )
         end if  ! ( debug )

!        --- For locnode := 1 (1) npoint do

         do locnode = 1, npointloc

!        --- nodp = maploc(i)

            nodp = mapping(istart+locnode)
            if ( globnodes(nodp)==0 ) then

!           --- Node is new
!               icount := icount+1; globnodes(nodp):=icount

               icount = icount+1
               globnodes(nodp) = icount

            end if  ! ( globnodes(nodp)==0 )

         end do  ! locnode = 1, npointloc
         numnodesnew(icpu) = icount

      end do  ! icpu = 1, iacnodes

1000  if ( debug ) then

!     --- Debug information

         call prinin ( numnodesnew, icount, 'numnodesnew' )
         call prinin ( globnodes, npointglobnw, 'globnodes' )
         write(irefwr,*) 'End mshrefpar15'

      end if  ! ( debug )
      call erclos ( 'mshrefpar15' )

      end subroutine mshrefpar15
