      subroutine mshrefpar2 ( iseqkm2, icoor, newmapping, &
                              kmeshmnew, kmeshnnew, kmeshcnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Send local arrays newmapping, coor, kmeshmnew and kmeshnnew to node 1
!
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
!     coordinate
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqkm2, icoor(*), newmapping(*), kmeshmnew(*), &
              kmeshnnew(*), kmeshcnew(*)

!     icoor          i    Contains the coordinates stored in an integer array
!     iseqkm2        i    Sequence number with respect to km of second mesh
!     kmeshcnew      i    KMESH part c, contains topology
!     kmeshmnew      i    New array kmeshm
!     kmeshnnew      i    New array kmeshn
!     newmapping     i    contains global node numbers for refined mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npointlc, lenbufarray, icontext, npointold, &
              npointnew, lenbuf, lenkmeshc, lenkmeshm, lenkmeshn
      integer, allocatable :: ibufarray(:)
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_size

!     ibufarray      Buffer array to store information to be sent or received
!     icontext       Tag for sending and receiving, is used as check
!     lenbuf         Local length of ibufarray
!     lenbufarray    Length of ibufarray
!     lenkmeshc      Length of kmeshc
!     lenkmeshm      Length of kmeshmnew
!     lenkmeshn      length of array kmeshnnew
!     npointlc       Number of new nodal points
!     npointnew      Number of nodes in refined mesh
!     npointold      Number of nodes in old mesh
!     send_buffer    send buffer
!     send_buffer_size  Length of the send buffer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     MSHREFPAR8     Find single curves that are changed by refinement
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar2'
         call pr0008 ( kmeshmnew,  km(iseqkm2)%kmesh(11) )
         call prinin ( newmapping,  km(iseqkm2)%kmesh(8), 'newmapping' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nelgrp =  km(iseqkm2)%kmesh(5)
      ndim =  km(iseqkm2)%kmesh(6)
      npointnew =  km(iseqkm2)%kmesh(8)
      npointold = npoint
      npointlc = npointnew-npointold
      lenkmeshc = size ( km(iseqkm2)%kmeshc )
      lenkmeshm = size ( km(iseqkm2)%kmeshm )
      lenkmeshn = 0
      if ( kelmn/=0 ) lenkmeshn = size ( km(iseqkm2)%kmeshn )
      lenbufarray = npointlc*ndim*intlen+2+lenkmeshm+lenkmeshn+ &
                    npointlc+lenkmeshc+nelgrp

      if ( debug ) &
         write(irefwr,1) 'ndim, npointnew, npointold, lenbufarray', &
                          ndim, npointnew, npointold, lenbufarray

      allocate ( ibufarray(lenbufarray), stat = error )
      if ( error/=0 ) call eralloc ( error, lenbufarray, 'ibufarray' )
      if ( ierror/=0 ) go to 1000

!     --- Store npoint (local) in ibufarray(1)

      ibufarray(1) = npointnew  ! number of nodes in submesh
      ibufarray(2) = npointlc     ! number of new nodes in submesh

!     --- Copy coordinates array in ibufarray from position 3
!         Only new nodes are stored

      ibufarray(3:npointlc*ndim*intlen+2) = &
         icoor(npointold*ndim*intlen+1:npointnew*ndim*intlen)
      lenbufarray = npointlc*ndim*intlen+2
      if ( debug ) write(irefwr,1) 'lenbufarray/1', lenbufarray

!     --- Copy newmapping in ibufarray (only new nodes)

      ibufarray(lenbufarray+1:lenbufarray+npointlc) = &
          newmapping(npointold+1:npointnew)
      lenbufarray = lenbufarray+npointlc
      if ( debug ) write(irefwr,1) 'lenbufarray/2', lenbufarray

!     --- Copy kmesh part g in ibufarray

      if ( nelgrp==1 ) then

!     --- Only one element groep, store nelem

         ibufarray(lenbufarray+1) =  km(iseqkm2)%kmesh(9)  ! nelem
         lenbufarray = lenbufarray+1

      else

!     --- More element groups, store kmesh part g

         ibufarray(lenbufarray+1:lenbufarray+nelgrp) = &
            km(iseqkm2)%kmeshg(1:nelgrp)
         lenbufarray = lenbufarray+nelgrp

      end if  ! ( nelgrp==1 )

!     --- Copy kmesh part c in ibufarray
!         use global numbering

      ibufarray(lenbufarray+1:lenbufarray+lenkmeshc) = &
         newmapping(kmeshcnew(1:lenkmeshc))

      if ( debug ) then
         write(irefwr,1) 'lenbufarray', lenbufarray
         call prinin ( ibufarray(lenbufarray+1-nelgrp), nelgrp, &
                       'kmeshg' )
         call prinin ( ibufarray(lenbufarray+1), lenkmeshc, 'kmeshc' )
      end if  ! ( debug )

      lenbufarray = lenbufarray+lenkmeshc

!     --- Copy those parts of kmeshmnew in ibufarray that correspond to curves
!         that are changed

      call mshrefpar8 ( kmeshmnew, newmapping, &
                        iseqkm2, lenbuf, ibufarray(lenbufarray+1) )
      lenbufarray = lenbufarray+lenbuf
      if ( debug ) write(irefwr,1) 'lenbufarray/3', lenbufarray

      if ( lenkmeshn>0 ) then

!     --- lenkmeshn>0 copy kmeshnnew in ibufarray

         write(irefwr,*) '3d has not yet been implemented in mshrefpar2'
         write(irefwr,*) 'Please contact SEPRA'
         ierror = 1
         go to 1000

      end if  ! ( lenkmeshn>0 )
      if ( debug ) write(irefwr,1) 'lenbufarray/4', lenbufarray
      send_buffer_size = 4*lenbufarray

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_size, 1 )

      allocate(send_buffer(send_buffer_size))

      call inibufattmpi ( send_buffer, send_buffer_size )

      icontext = 25
      call inibsendmpi ( ibufarray, lenbufarray, SPINT, 1, icontext )

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

      deallocate ( ibufarray, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ibufarray' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefpar2'

      end if  ! ( debug )
      call erclos ( 'mshrefpar2' )

      end subroutine mshrefpar2
