      subroutine mshrefparents ( parentnodes, edgenode, nedgenode, &
                                 npoint_basis )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 26-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   fill array parentnodes with the vertex numbers of the 'parents' of
!   an added node
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nedgenode, edgenode(3,nedgenode), parentnodes(2,*), &
              npoint_basis

!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     nedgenode      i    number of added (internal) nodes for this
!                         level of refinement
!     npoint_basis   i    npoint of the basis mesh
!     parentnodes   i/o   Array with for all added nodes the node
!                         numbers of the parents
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefparents' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefparents'
         print *, 'nedgenode, npoint_basis', nedgenode, npoint_basis

      end if
      if ( ierror/=0 ) go to 1000

      do i = 1, nedgenode
         parentnodes(1,edgenode(3,i)-npoint_basis) = edgenode(1,i)
         parentnodes(2,edgenode(3,i)-npoint_basis) = edgenode(2,i)
         if (debug) then
            call prinin(edgenode(1,nedgenode), 3, 'edgenode')
            print *, 'parentnodes(i) ', &
                      parentnodes(1,edgenode(3,i)-npoint_basis), &
                      parentnodes(2,edgenode(3,i)-npoint_basis)
         end if

      end do

1000  call erclos ( 'mshrefparents' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefparents'

      end if

      end
