      subroutine mshsetmeshnr ( mesh_new )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-05-2005 Replaced==bij/=(Brusche)
!        version  1.0  date 19-05-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set mesh number mesh_new equal to first free mesh number
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mesh_new

!     mesh_new       o    Next free mesh number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icount
      logical :: new

!     icount         Counter
!     new            indication whether the mesh is new (true) or not
!                    (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2769   No free meshes available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsetmeshnr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsetmeshnr'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      mesh_new = 0       ! start value
      icount = 0         ! counter for array kmesh
      do while ( mesh_new==0 .and. icount<maxmesh )

!     --- Not a free mesh number found, raise icount

         icount = icount+1

         if ( debug ) &
            write(irefwr,1) 'icount, kmesh2d(icount)', &
                             icount, kmesh2d(icount)
         new = kmesh2d(icount)<1 .or. kmesh2d(icount)>lastmesh

         if ( debug ) write(irefwr,*) 'new ', new

         if ( new ) then

!        --- Mesh icount is still free

            mesh_new = icount

         end if  ! ( kmesh(1,icount)/=0 )

      end do  ! while ( mesh_new==0 .and. icount<maxmesh )

      if ( mesh_new==0 ) then

!     --- No free mesh number found

         call errint ( maxmesh, 1 )
         call errsub ( 2769, 1, 0, 0 )

      end if  ! ( mesh_new==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsetmeshnr'

      end if
      call erclos ( 'mshsetmeshnr' )

      end
