      subroutine mshshift2dnod ( numnodesb, nodesmv, nodesfac, coornew, &
                                 skipnod, nmap, ndim, iworkpoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-09-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Move nodes along edges or curves depending on skipnod
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodesb, nodesmv(2,numnodesb), nmap, skipnod(*), &
              ndim, iworkpoint(nmap,*)
      double precision nodesfac(numnodesb), coornew(ndim,*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     iworkpoint     i    Work array of length npointnew
!                         This array is filled as follows:
!                         iworkpoint(i) = 0 if node i outside obstacle
!                         iworkpoint(i) = 1 if node i inside obstacle
!                         iworkpoint(i) = 2 if node i inside obstacle (near
!                                           boundary)
!                         iworkpoint(i) = 3 if node i on obstacle
!     ndim           i    Dimension of the space.
!     nmap           i    length of first parameter of imap and rmap
!     nodesfac       i    Contains factor for moving along edge, measured from
!                         first node
!     nodesmv        i    Contains set of nodes to be moved
!                         (1,.) node to be moved
!                         (2,.) second node on edge
!     numnodesb      i    Number of nodes in nodesmv
!     skipnod       i/o   Array of npoint long in which the nodes are marked
!                         which are not allowed to be moved
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node1, node2, iobst
      double precision fact

!     fact           factor defining intersection point
!     i              Counting variable
!     iobst          Obstacle sequence number
!     node1          Node of face
!     node2          Node of face
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2dnod' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2dnod'
         call prinin1 ( nodesmv, numnodesb, 2, '  node1      node2' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iobst = 1

      do i = 1, numnodesb

         node1 = nodesmv(1,i)
         node2 = nodesmv(2,i)
         fact = nodesfac(i)

         if (debug) then
            write(irefwr,1) 'node1, node2', node1, node2
            write(irefwr,2) 'coors_1', coornew(:,node1)
            write(irefwr,2) 'coors_2', coornew(:,node2)
            write(irefwr,2) 'fact', fact
         end if

         coornew(:,node1) = (1d0-fact)*coornew(:,node1) + &
                             fact*coornew(:,node2)

         if (debug) then
            write(irefwr,2) 'coors end (new)', coornew(:,node1)
         end if

         iworkpoint(node1,1) = 3  ! mark node as on the obstacle
         iworkpoint(node1,2) = iobst
         skipnod(node1) = 3     ! mark node as shifted

!        --- Store mapping information for shifted node

         imap(node1,1) = 2
         imap(node1,2) = node1
         imap(node1,3) = node2
         rmap(node1,1) = 1d0-fact
         rmap(node1,2) = fact

      end do  ! i = 1, numnodesb

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshift2dnod'

      end if  ! ( debug )
      call erclos ( 'mshshift2dnod' )

      end
