      subroutine mshsubvol ( kmeshc, nselem, jshape, jnpelm, isurnr, &
                             coor, nldim, valmin, valmax, value, npointlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 17-07-2011 Complete revision: use mshsubvol1
!        version  1.0  date 15-12-2006
!
!   copyright (c) 2006-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute volume of submesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     volume
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nselem, jnpelm, kmeshc(jnpelm,nselem), jshape, isurnr, &
              nldim, npointlc
      double precision coor(nldim,npointlc), valmin, valmax, value

!     coor       i     array of length ndim x npoint containing the co-ordinates
!                      of the nodal points with respect to the global numbering
!     isurnr     i     Surface sequence number
!     jnpelm     i     Number of nodes in an element
!     jshape     i     Shape number of element
!     kmeshc     i     kmesh part c, contains the nodal points corresponding
!                      to the elements
!     nldim      i     Dimension of space
!     npointlc   i     Number of nodes
!     nselem     i     Number of elements
!     valmax     o     Maximum value of computed integral
!     valmin     o     Minimum value of computed integral
!     value      o     Computed integral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, allocatable :: areas(:)

!     areas          Contains the areas of the elements of the submesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     MSHSUBVOL1     Compute volume of elements in submesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsubvol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsubvol'
         call ertrace
         write(irefwr,1) 'nselem', nselem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( areas(nselem) , stat = error )
      if ( error/=0 ) call eralloc ( error, nselem, 'areas' )
      call mshsubvol1 ( kmeshc, nselem, jshape, jnpelm, isurnr, &
                        coor, nldim, areas, npointlc )
      if ( ierror/=0 ) go to 1000
      valmin = minval(areas)
      valmax = maxval(areas)
      value = sum(areas)

      deallocate ( areas, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'areas' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsubvol'

      end if  ! ( debug )
      call erclos ( 'mshsubvol' )

      end
