      subroutine mshsurfaces8 ( kmeshn, nsurfs, nsurfsold, &
                                surffaces, iworksurfs, icount )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 26-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all subsurfaces corresponding to ihelp(isurnr) = 1
!     into iworksurfs and put the starting addresses into ihelp
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     mesh
!     compound_surface
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshn(*), nsurfs, nsurfsold, surffaces(*), &
              iworksurfs(*), icount

!     icount         o    Number of surfaces stored in iworksurfs
!     ihelp         i/o   integer work array of length nsurfs
!                         If surface i is single ihelp(i) = 0
!                         If surface i is possibly compound ihelp(i) = -1
!                         If surface i is compound but all faces refer to the
!                         same surfaces: ihelp(i) = 1
!                         At output ihelp(i) > 0 refers to a stating address
!                         in array iworksurfs containing the subsurfaces of
!                         the surface
!                         ihelp(i) = -10 corresponds to a complicated situation
!     ihelp1         o    integer work array of length nsurfs
!                         containing the number of subsurfaces
!                         corresponding to all surfaces with ihelp(i)>0
!     iwork          i    integer work array of length nfaces
!                         iwork(iface) contains the number of surfaces
!                         face iface is stored in
!     iwork1         i    integer work array of length nfaces
!                         contains for each face iface with iwork(iface)>1 a
!                         starting address in iwork2
!     iwork2         i    integer work array of length leniwork2
!                         contains for each face iface with iwork(iface)>1 the
!                         sequence numbers of the surfaces iface is lying in
!     iworksurfs     o    Array in which the subsurfaces corresponding to a
!                         surface are stored
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     nsurfs         i    number of surfaces
!     nsurfsold      i    Number of surfaces in the mesh read
!     surffaces      i    Array containing the face numbers of the elements
!                         stored in array kmeshn
!                         Exactly the sequence of kmeshn is used
!                         In this case it refers to kmeshn before extension
!                         with extra outer surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurnr, istart, i, iface, inelem, isurface, &
              kstart, numsurfaces
      logical found

!     found          If true already a possibility has been found
!     i              Counting variable
!     iface          Face sequence number
!     inelem         Number of elements in a specific element group
!     istart         First position used at a part
!     isurface       surface sequence number
!     isurnr         Surface sequence number
!     kstart         Start value
!     numsurfaces    Number of surfaces coupled to face
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all surfaces isurnr do
!        if ( Composite surface found ) then
!           find all subsurfaces and check if one of them is single
!           fill ihelp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces8' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces8'

      end if
      if ( ierror.ne.0 ) go to 1000
      icount = 0
      istart = 0

!     --- For all surfaces isurnr do

      do isurnr = 1, nsurfsold

         inelem = kmeshn(isurnr+nsurfs)
         ihelp1(isurnr) = 0
         if ( ihelp(isurnr).eq.1 ) then

!        --- Composite surface found, find all subsurfaces and check if
!            one of them is single

            found = .false.
            iface = surffaces(istart+1)  ! face number first element
            numsurfaces = iwork(iface)   ! number of surfaces corresponding to
                                         ! this face
            kstart = iwork1(iface)
            do i = 1, numsurfaces

               isurface = iwork2(kstart+i-1)
               if ( ihelp(isurface).eq.0 ) then

!              --- Single subsurface found

                  found = .true.
                  icount = icount+1
                  iworksurfs(icount) = isurface
                  ihelp(isurnr) = icount
                  ihelp1(isurnr) = 1
                  go to 300

               end if  ! ( ihelp(isurface).eq.0 )

            end do  ! i = 1, numsurfaces
300         if ( .not. found ) then

!           --- The surface is not created from a single surface
!               This situation is treated elsewhere

               ihelp(isurnr) = -10

            end if  ! ( .not. found )

         end if  ! ( ihelp(isurnr).eq.1 )
         istart = istart+inelem

      end do  ! isurnr = 1, nsurfsold

1000  call erclos ( 'mshsurfaces8' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsurfaces8'

      end if

      end
