      subroutine mshsurfaces9 ( kmeshn, nsurfs, nsurfsold, surffaces, &
                                iworksurfs, icount, nsurfsnew, &
                                extsurffaces, jcount, lenextsurffac, &
                                ipoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-11-2004 Fortran 90
!        version  1.1  date 04-08-2004 Correction for complex surfaces
!        version  1.0  date 26-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     All surfaces with ihelp(i)=-1 refer to several groups of surfaces
!     If all these subsurfaces are single we store these subsurfaces in
!     iworksurfs and adapt ihelp
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     mesh
!     compound_surface
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshn(*), nsurfs, nsurfsold, surffaces(*), &
              iworksurfs(*), icount, nsurfsnew, extsurffaces(*), jcount, &
              lenextsurffac, ipoint(*)

!     extsurffaces   o    Array of length lenextsurffac containing the
!                         face sequence numbers of the new surfaces to be
!                         created
!     icount        i/o   Number of surfaces stored in iworksurfs
!     ihelp         i/o   integer work array of length nsurfs
!                         If surface i is single ihelp(i) = 0
!                         If surface i is possibly compound ihelp(i) = -1
!                         At output ihelp(i) > 0 refers to a stating address
!                         in array iworksurfs containing the subsurfaces of
!                         the surface
!                         ihelp(i) = -10 corresponds to a complicated situation
!     ihelp1        i/o   integer work array of length nsurfs
!                         containing the number of subsurfaces
!                         corresponding to all surfaces with ihelp(i)>0
!     ipoint         o    Array of estimated length 11*nsurfsold that is used to
!                         store the end addresses of each new surface in array
!                         extsurffaces
!                         The positions 1 to nsurfs get the value 0
!     iwork          i    integer work array of length nfaces
!                         iwork(iface) contains the number of surfaces
!                         face iface is stored in
!     iwork1         i    integer work array of length nfaces
!                         contains for each face iface with iwork(iface)>1 a
!                         starting address in iwork2
!     iwork2         i    integer work array of length leniwork2
!                         contains for each face iface with iwork(iface)>1 the
!                         sequence numbers of the surfaces iface is lying in
!     iworksurfs    i/o   Array in which the subsurfaces corresponding to a
!                         surface are stored
!                         Starting add
!     jcount        i/o   Last position used in extsurffaces
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     lenextsurffac  i    Length of array extsurffaces
!     nsurfs         i    number of surfaces
!     nsurfsnew     i/o   Last surface sequence number used
!     nsurfsold      i    Number of surfaces in the mesh read
!     surffaces      i    Array containing the face numbers of the elements
!                         stored in array kmeshn
!                         Exactly the sequence of kmeshn is used
!                         In this case it refers to kmeshn before extension
!                         with extra outer surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurnr, istart, i, iface, inelem, isurface, iielem, &
              kstart, numsurfaces, icompare(nsurfs), kcount
      logical found, newsurface

!     found          If true already a possibility has been found
!     i              Counting variable
!     icompare       Work array of length nsurfs
!                    icompare(i) = 0 if surface i is not coupled to the first
!                    face and icompare(i) = 1 otherwise
!     iface          Face sequence number
!     iielem         Relative element number in element group
!     inelem         Number of elements in a specific element group
!     istart         First position used at a part
!     isurface       surface sequence number
!     isurnr         Surface sequence number
!     kcount         Counter, contains value of jcount at start of surface
!     kstart         Start value of parameter
!     newsurface     if true a new surface has been created
!     numsurfaces    Number of surfaces coupled to face
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all surfaces isurnr do
!        if ( Remaining double surface found ) then
!           This surface refers to several groups of surfaces
!           If all these subsurfaces are single we store these subsurfaces
!           otherwise we have a situation that is treated elsewhere
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces9' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces9'

      end if
      if ( ierror.ne.0 ) go to 1000

      istart = 0
      kcount = 0

!     --- For all surfaces isurnr do

      do isurnr = 1, nsurfsold

         inelem = kmeshn(isurnr+nsurfs)
         kcount = jcount
         newsurface = .false.
         if ( debug ) &
            write(irefwr,*) 'isurnr, inelem, jcount ', &
                             isurnr, inelem, jcount

         if ( ihelp(isurnr).eq.-1 ) then

!        --- Remaining double surface
!            This surface refers to several groups of surfaces
!            If all these subsurfaces are single we store these subsurfaces
!            otherwise we have a situation that is treated elsewhere

            do i = 1, nsurfs
               icompare(i) = 0
            end do  ! i = 1, nsurfs
            ihelp1(isurnr) = 0
            ihelp(isurnr) = icount+1

            if ( debug ) &
               write(irefwr,*) 'isurnr, icount, inelem ', &
                                isurnr, icount, inelem

            do iielem = 1, inelem

!           --- iface := surffaces(istart+iielem)
!               Find single surface corresponding to this face
!               Add to list if necessary

               iface = surffaces(istart+iielem) ! next face
               numsurfaces = iwork(iface)   ! number of surfaces corresponding
                                            ! to this face
               if ( numsurfaces.eq.1 ) then

!              --- Special case of a single face
!                  All these single faces are combined into a new surface

                  if ( icompare(isurnr).eq.0 ) then

!                 --- Surface has not been treated yet
!                     Create new surface and store information

                     icompare(isurnr) = 1  ! surface has been stored
                     nsurfsnew = nsurfsnew+1  !new surface number
                     ihelp1(isurnr) = ihelp1(isurnr)+1  ! number of surfs
                     icount = icount+1  ! position in iworksurfs
                     iworksurfs(icount) = nsurfsnew ! new surface in iworksurfs
                     newsurface = .true.

                     if ( debug ) then
                        write(irefwr,*) 'single face: nsurfsnew ', &
                           nsurfsnew
                        write(irefwr,*) 'icount ', icount
                     end if  ! ( debug )

                  end if  ! ( icompare(isurnr).eq.0 )
                  jcount = jcount+1
                  if ( jcount.gt.lenextsurffac ) then

!                 --- array length too small

                     call errint ( jcount, 1 )
                     call errint ( lenextsurffac, 2 )
                     call errchr ( 'extsurffaces', 1 )
                     call errsub ( 550, 2, 0, 1 )
                     go to 1000

                  end if
                  extsurffaces(jcount) = iface

               else

!              --- Standard case of more subsurfaces

                  kstart = iwork1(iface)
                  found = .false.
                  do i = 1, numsurfaces

                     isurface = iwork2(kstart+i-1)
                     if ( ihelp(isurface).eq.0 ) then

!                    --- Single surface found, check if it is a new one

                        found = .true.
                        if ( icompare(isurface).eq.0 ) then

!                       --- New surface, add to list

                           icount = icount+1  ! position in iworksurfs
                           iworksurfs(icount) = isurface  ! new surface in
                                                          ! iworksurfs
                           icompare(isurface) = 1  ! surface has been stored
                           ihelp1(isurnr) = ihelp1(isurnr)+1  ! number of surfs

                           if ( debug ) then
                              write(irefwr,*) 'more surfaces '
                              write(irefwr,*) 'single surface ', &
                                 isurface
                              write(irefwr,*) 'icount ', icount
                           end if  ! ( debug )

                        end if  ! ( icompare(isurface).eq.0 )
                        go to 400

                     end if  ! ( ihelp(isurface).eq.0 )

400                  if ( .not. found ) then

!                    --- The surface is not created from a single surface
!                        This situation is treated elsewhere

                        ihelp(isurnr) = -10
                        if ( newsurface ) then

!                       --- Newsurface has been created but must be deleted
!                           Reset nsurfsnew and jcount

                           jcount = kcount
                           nsurfsnew = nsurfsnew-1
                           icount = icount-1

                        end if  ! ( newsurface )
                        go to 500

                     end if  ! ( .not. found )

                  end do  ! i = 1, numsurfaces

               end if  ! ( numsurfaces.eq.1 )

            end do  ! iielem = 1, inelem
            if ( jcount.gt.kcount ) ipoint(nsurfsnew) = jcount

         end if  ! ( ihelp(isurnr).eq.-1 )
500      istart = istart+inelem

      end do  ! isurnr = 1, nsurfsold

1000  call erclos ( 'mshsurfaces9' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'jcount, kcount ', jcount, kcount
         call prinin ( ipoint, nsurfsnew, 'ipoint' )
         write(irefwr,*) 'End mshsurfaces9'

      end if

      end
