      subroutine mshsurfinfo ( nsurfaces, isurfaces, ishapelc, &
                               nelemlc, npelmlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get information concerning a number of surfaces stored in array
!     isurfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     shape
!     3d
!     number_of_nodes
!     number_of_elements
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfaces, isurfaces(*), ishapelc, nelemlc, npelmlc

!     ishapelc       o    local shape number
!     isurfaces      i    Array containing the surfaces from which information
!                         must be found
!     nelemlc        o    Local number of elements
!     npelmlc        o    Local number of nodes per element
!     nsurfaces      i    Number of surfaces in array isurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jsurnr, isurnr, istart, inpelm

!     inpelm         number of points per element
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     jsurnr         Surface sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIINFSURF     Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1347   Number of nodes per element not the same in set of surfaces
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nelemlc := 0
!     For all surfaces isurnr in isurfaces do
!        find inpelm and inelem for surface
!        nelemlc := nelemlc+inelem
!     Find shape number corresponding to isurnr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfinfo' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfinfo'
100      format ( a, 1x, (10i6) )
         call prinin ( isurfaces, nsurfaces, 'isurfaces' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- nelemlc := 0

      nelemlc = 0

!     --- For all surfaces isurnr in isurfaces do

      do jsurnr = 1, nsurfaces

         isurnr = isurfaces(jsurnr)

!        --- find inpelm and inelem for surface

         call iniinfsurf ( kmeshn, istart, inpelm, inelem, nsurfs, &
                           isurnr )

         if ( debug ) then
            write(irefwr,100) 'jsurnr, isurnr, inpelm, inelem, nsurfs', &
                               jsurnr, isurnr, inpelm, inelem, nsurfs
         end if  ! ( debug )

         if ( jsurnr==1 ) then

!        --- First surface, set npelmlc

            npelmlc = inpelm

         else

!        --- Next surface, check npelmlc

            if ( inpelm/=npelmlc ) then

!           --- npelmlc # inpelm: error

               call errint ( isurfaces(1), 1 )
               call errint ( npelmlc, 2 )
               call errint ( isurnr, 3 )
               call errint ( inpelm, 4 )
               call errsub ( 1347, 4, 0, 0 )

            end if  ! ( inpelm/=npelmlc )

         end if  ! ( jsurnr==1 )

!        --- nelemlc := nelemlc+inelem

         nelemlc = nelemlc+inelem

      end do  ! jsurnr = 1, nsurfaces

!     --- Find shape number corresponding to isurnr

      if ( kelms>0 ) then

!     --- Kmeshs has been filled, extract information from kmeshs

         ishapelc = kmeshs(nsurfs+1+8*(isurnr-1)+2)

      else

!     --- Kmeshs has not been filled, compute information from inpelm

         select case ( inpelm )

            case(:2, 11:)

!           --- inpelm<=2 or >- 11, not possible

               ishapelc = 0

            case(3)

!           --- inpelm = 3, triangle

               ishapelc = 3

            case(4)

!           --- inpelm = 4, quadrilateral

               ishapelc = 5

            case(5)

!           --- inpelm = 5, quadrilateral

               ishapelc = 9

            case(6)

!           --- inpelm = 6, triangle

               ishapelc = 4

            case(7)

!           --- inpelm = 7, triangle

               ishapelc = 7

            case(9)

!           --- inpelm = 9, quadrilateral

               ishapelc = 6

            case(10)

!           --- inpelm = 10, triangle

               ishapelc = 8

         end select  ! case ( inpelm )

      end if  ! ( kelms>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'nelemlc, npelmlc, ishapelc',&
                            nelemlc, npelmlc, ishapelc
         write(irefwr,*) 'End mshsurfinfo'

      end if
      call erclos ( 'mshsurfinfo' )

      end
