      subroutine mshtaucurv ( nnodes, nodes, coor, ndim, tau )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute distribution of nodes along curve
!     Store in array tau
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(nnodes), ndim
      double precision tau(nnodes), coor(ndim,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in curve
!     nodes          i    Contains the node numbers of the curve
!     tau            o    array of length nnodes with the relative distances
!                         between points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, nodp, nodpbef
      double precision dist

!     dist           Distance between two points
!     i              Counting variable
!     k              Counting variable
!     nodp           Nodal point sequence number
!     nodpbef        Prior nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     tau(1) := 0
!     For i := 2, nnodes do
!        dist := || coor(node_i) - coor(node_i-1) ||
!        tau(i) := tau(i-1) + dist
!     For i := 2, nnodes do
!        tau(i) := tau(i)/tau(nnodes)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshtaucurv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshtaucurv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- tau(1) := 0

      tau(1) = 0d0
      nodpbef = nodes(1)

!     --- For i := 2, nnodes do

      do i = 2, nnodes

!     --- dist := || coor(node_i) - coor(node_i-1) ||
!         tau(i) := tau(i-1) + dist

         nodp = nodes(i)
         dist = 0d0
         do k = 1, ndim
            dist = dist + (coor(k,nodp)-coor(k,nodpbef))**2
         end do  ! k = 1, ndim
         tau(i) = tau(i-1)+sqrt(dist)
         nodpbef = nodp

      end do  ! i = 2, nnodes

!     --- For i := 2, nnodes do
!            tau(i) := tau(i)/tau(nnodes)

      dist = tau(nnodes)
      tau(2:nnodes-1) = tau(2:nnodes-1)/dist
      tau(nnodes) = 1d0

1000  call erclos ( 'mshtaucurv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshtaucurv'

      end if  ! ( debug )

      end
