      subroutine mshvolbcord ( bcord, iinput, iposi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 23-08-2010 Remove arrays iwork/work from parm list
!        version  1.0  date 18-07-2008
!
!   copyright (c) 2008-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Fill coordinates of boundary of volume into array bcord
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     boundary
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iposi
      double precision bcord(*)

!     bcord          o    array containing the coordinates of the boundaries
!     iinput         i    input array    see subroutine mesh
!                         iinput contains only the part with respect to the
!                         definition of the volumes
!     iposi          i    next free position of iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ndefsf, j, iunder, iupper, nsubsf

!     iunder         Lower bound of array
!     iupper         Upper bound of array
!     j              Counting variable
!     ndefsf         Number of surfaces in volume
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHIORCR       Rearrange array bcord such that the new orientation of the
!                    surface is taken into account
!     MSHVOLBCORD1   Fill coordinates of boundary of volume into array bcord
!                    One surface only
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshvolbcord' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshvolbcord'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndefsf = iinput(iposi+2)   ! number of surfaces that defines the boundary
      iunder = 1                 ! start address in bcord
      nsubsf = 0                 ! number of subsurfaces

      do j = 1, ndefsf

         isurnr = iinput(iposi+2+j)
         iorien = (isurnr-1)/nsurfs
         isurnr = mod(isurnr-1,nsurfs)+1
         if ( kelmr/=0 ) nsubsf = kmeshr(isurnr+1) - kmeshr(isurnr)
         if ( debug ) then

!        --- Debug statements

            write(irefwr,*) 'Surface ', j, ' is ', isurnr
            write(irefwr,*) 'Number of subsurfaces ', nsubsf

         end if
         call mshvolbcord1 ( bcord, nsubsf, iunder, iupper )
         if ( iorien>0 ) &
            call mshiorcr ( iorien, j, nx, ny, nz, bcord(iunder), &
                            ishape )
         iunder = iupper+1

      end do  ! j = 1, ndefsf

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iupper, iupper/3', iupper, iupper/3
         call prinrl1 ( bcord, iupper/3, 3, 'bcord' )
         write(irefwr,*) 'End mshvolbcord'

      end if  ! ( debug )
      call erclos ( 'mshvolbcord' )

      end
