      subroutine mshvolboun1 ( kbndpt, iposkbn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-08-2010 Removal of work arrays
!        version  1.0  date 18-07-2008
!
!   copyright (c) 2008-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kbndpt with surfaces that define the boundary of
!     a volume
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*), iposkbn

!     iposkbn       i/o   Starting address of present surface in kbndpt
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsubsf, istart, jstart, &
              leng2, npoloc, nelloc, i, k, ih1, jsur, &
              ih, isubsf, isur, index, iposkbnbef, ikelmc
      integer, allocatable :: locnum(:), loctopol(:)

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     ikelmc         position in kmesh part c
!     index          Record sequence number
!     iposkbnbef     Previous value of iposkbn
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isur           surface number
!     jstart         starting position
!     jsur           surface number provided with sign
!     k              Counting variable
!     leng2          length indicator elements
!     locnum         Contains local numbering of nodes in submesh
!     loctopol       Contains local topology of submesh
!     nelloc         Local number of elements
!     npoloc         Number of nodes in subsurface
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshvolboun1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshvolboun1'
         write(irefwr,1) 'isurnr', isurnr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ishape = ishape-8   ! shape of surface elements
      nsubsf = 0                ! number of subsurfaces
      iposkbnbef = iposkbn      ! previous value of iposkbn
      if ( debug ) &
         write(irefwr,1) 'nsurfs, ndim, ishape, inplmt', &
                          nsurfs, ndim, ishape, inplmt

      if ( kelmr/=0 ) nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)
      if ( debug ) write(irefwr,1) 'nsubsf', nsubsf
      if ( nsubsf==0 ) then

!     --- nsubsf = 0, No subsurfaces

         length = size(kmhsur(isurnr)%kmeshcsur)
         kbndpt(iposkbn:iposkbn+length-1) = kmhsur(isurnr)%kmeshcsur
         iposkbn = iposkbn+length

      else

!     --- Surface consists of subsurfaces
!         Read information of subsurfaces and store in locnum and loctopol

         istart = kmeshr(isurnr)
         jstart = kmeshr(nsurfs+1) - nsurfs-1 + istart
         ih = 0

         do isubsf = 1, nsubsf

            jsur = kmeshr(istart+isubsf)
            isur = abs(jsur)
            index = kmeshr(jstart+isubsf)
            npoloc = kmeshh(1,isur+1)
            nelloc = kmeshh(2,isur+1)
            leng2 = inplmt*nelloc

            allocate ( locnum(npoloc+nelloc), loctopol(leng2), &
                       stat = error )
            if ( error/=0 ) then

!           --- error#0, so not enough space

               call errint ( npoloc+nelloc+leng2, 1 )
               call errint ( error, 2 )
               call errchr ( 'locnum', 1 )
               call errsub ( 2910, 2, 0, 1 )
               go to 1000

            end if  ! ( error/= 0 )

            if ( debug ) &
               write(irefwr,1) 'isubsf, jsur, npoloc, nelloc', &
                                isubsf, jsur, npoloc, nelloc

            ih = ih+nelloc

!           --- Read local numbering and store in array locnum

            call readfl ( index, 1, npoloc+nelloc, 1, 3, locnum )

            if ( debug ) then
               call prinin ( locnum, npoloc, 'node-map' )
               call prinin ( locnum(1+npoloc), nelloc, 'el-map' )
            end if  ! ( debug )

!           --- Read local array kmeshc and store in kbndpt

            loctopol(1:leng2) = kmhsur(isur)%kmeshcsur
            if ( debug ) &
               call prinin1 ( loctopol, nelloc, inplmt, &
                              'kmeshc_loc' )

!           --- Put local array kmeshc into global kmeshc

            do i = 1, nelloc

               ikelmc = (locnum(npoloc+i)-1 ) * inplmt
               ih1 = ( i-1 ) * inplmt
               if ( jsur>0 ) then

!              --- jsur>0, standard case, positive direction

                  do k  = 1, inplmt
                     kbndpt(iposkbn+ikelmc+k-1) = &
                        locnum(loctopol(ih1+k))
                  end do

               else

!              --- jsur<0, special case, reversed direction

                  if ( ishape<=5 ) then

!                 --- ishape = 3, 4, 5 all points must be reversed

                     kbndpt(iposkbn+ikelmc+1-1) = &
                        locnum(loctopol(ih1+1))
                     do k  = 2, inplmt
                        kbndpt(iposkbn+ikelmc+inplmt+2-k-1) = &
                           locnum(loctopol(ih1+k))
                     end do

                  else

!                 --- ishape>5 last point must be kept

                     kbndpt(iposkbn+ikelmc+1-1) = &
                        locnum(loctopol(ih1+1))
                     do k  = 2, inplmt-1
                        kbndpt(iposkbn+ikelmc+inplmt+1-k-1) = &
                           locnum(loctopol(ih1+k))
                     end do
                     k = inplmt
                     kbndpt(iposkbn+ikelmc+k-1) = &
                        locnum(loctopol(ih1+k))

                  end if

               end if

            end do

            deallocate ( locnum, loctopol, stat = error )
            if ( error/= 0 ) then

!           --- error#0, problem with deallocate

               call errint ( error, 1 )
               call errchr ( 'locnum', 1 )
               call errsub ( 2911, 1, 0, 1 )
               go to 1000

            end if  ! ( error/= 0 )

         end do
         iposkbn = iposkbn+inplmt*ih

      end if
      ishape = ishape+8   ! shape of surface elements

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposkbnbef, iposkbn, inplmt', &
                          iposkbnbef, iposkbn, inplmt
         call prinin1 ( kbndpt(iposkbnbef), &
                        (iposkbn-iposkbnbef)/inplmt, &
                        inplmt, 'kbndpt' )
         write(irefwr,*) 'End mshvolboun1'

      end if  ! ( debug )
      call erclos ( 'mshvolboun1' )

      end
