      subroutine prmarkskipelm ( imark, ielemobst, iobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark array imark for all elements that are completely inside
!     the obstacle
!     It is supposed that imark has been initialized before
! **********************************************************************
!
!                       KEYWORDS
!
!     obstacle
!     element
!     mark
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imark(*), ielemobst(*), iobst

!     ielemobst      i    Array containing information about the elements in an
!                         obstacle
!                         The first numobst positions contain the starting
!                         positions of each obstacle sequentially
!                         Per obstacle the first three positions contain
!                         1. The number of elements that are completely inside
!                            the obstacle
!                         2. The number of elements that are not completely
!                            inside the element but with all vertices inside
!                            the element
!                         3. The number of elements that are partly inside
!                            the obstacle
!                            elements that contain only boundary points of the
!                            obstacle are not considered
!                         These positions are followed by the element numbers
!                         in the sequence defined by the first three positions
!     imark         i/o   Marker array of length nelem
!                         imark must have been initialized by 0
!                         imark(i) = 0: element i not inside obstacle
!                         imark(i) = 1: element i inside obstacle
!     iobst          i    Obstacle sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, numinelmobst, i

!     i              Counting variable
!     istart         First position used at a part
!     numinelmobst   Number of elements that are completely within the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmarkskipelm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmarkskipelm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = ielemobst(iobst)
      numinelmobst = ielemobst(istart)+ielemobst(istart+1)
      istart = istart+2
      do i = 1, numinelmobst
         imark(ielemobst(istart+i)) = 1
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmarkskipelm'

      end if
      call erclos ( 'prmarkskipelm' )

      end
