      subroutine sepadapcenter
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt center of elements in case of mesh deformation
!     This assures that a centroid is still a center of an element
!     Old subroutine mshadapcenter
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     deformation
!     centre
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ikelmc, ielgrp

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHD7          Places coorindates of centroids of 7-point triangles
!                    in array coor
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all element groups do
!        if ( center must be adapted ) then
!           For all elements in center do
!              adapt coordinates of center
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepadapcenter' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepadapcenter'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'inpelm, nelgrp, ndim, inelem', &
                          inpelm, nelgrp, ndim, nelem
         write(irefwr,1) 'kelma, kelmf, kelmg', kelma, kelmf, kelmg
      end if  ! ( debug )

      ikelmc = 1             ! starting address used in kmeshc

!     --- For all element groups do

      do ielgrp = 1, nelgrp

         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm

         if ( debug ) &
            write(irefwr,1) 'ielgrp, ishape, inelem', &
                             ielgrp, ishape, inelem

!        --- if ( center must be adapted ) then
!               For all elements in center do
!                  adapt coordinates of center

         if ( ishape==7 ) then

!        --- shape = 7, adapt center

            call mshd7 ( kmeshc(ikelmc), coor, inelem, ndim )

         end if  ! ( ishape==7 )

         ikelmc = ikelmc+inelem*inpelm  ! raise ikelmc

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepadapcenter'

      end if  ! ( debug )
      call erclos ( 'sepadapcenter' )

      end
