      subroutine sepfindlowsur ( ncursur, ielembot, ielemtop, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find surface (curve) with lowest non-zero entry in kmesht
!     Return with lowest and highest element for this surface
!     Old subroutine mshfindlowsur
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncursur, ielembot, ielemtop, isurnr

!     ielembot       o    Element sequence number of element with lowest number
!                         in surface isurnr
!     ielemtop       o    Largest element number in surface isurnr
!     isurnr         o    Surface sequence number with lowest value in kmesht
!     ncursur        i    number of surfaces or curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ilow

!     i              Counting variable
!     ilow           elements are numbered from element number ilow
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfindlowsur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfindlowsur'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ilow = nelem+1

      do i = 1, ncursur
         if ( kmesht(1,i)<ilow ) then

!        --- new surface found

            isurnr = i
            ilow = kmesht(1,i)

         end if
      end do
      ielembot = kmesht(1,isurnr)
      ielemtop = kmesht(2,isurnr)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ielembot, ielemtop', ielembot, ielemtop
         write(irefwr,*) 'End sepfindlowsur'

      end if  ! ( debug )
      call erclos ( 'sepfindlowsur' )

      end
