      subroutine seplevcross1d ( iseqlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill arrays icrossobst and rcrossobst (1d)
!     Old subroutine mshlevcross1d
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     element
!     node
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqlc

!     iseqlc         o    Sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenicrossobst, lenrcrossobst

!     lenicrossobst  Computed length of array icrossobst
!     lenrcrossobst  Computed length of array rcrossobst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPLEVELCREL1D Compute all information concerning kmesh ad parts 3 and 4
!     SEPPLOTINTERS  plot and print intersection points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplevcross1d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplevcross1d'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- define work space to store rcrossobst and icrossobst

      if ( associated(icrossobst) ) nullify(icrossobst, rcrossobst)
      allocate ( icrossobst(3*nelem+2*numobst), rcrossobst(nelem), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 3*nelem+2*numobst, 'icrossobst' )
      if ( ierror/=0 ) go to 1000

!     --- compute starting addresses

      usoltemp => ks(isolut)%sol

!     --- Compute arrays icrossobst and rcrossobst

      lenicrossobst = 0
      lenrcrossobst = 0
      call seplevelcrel1d ( usoltemp, lenicrossobst, lenrcrossobst )

!     --- Make the arrays rcrossobst, icrossobst permanent

      if ( lenrcrossobst>0 ) then

!     --- Array may be created

         length = lenrcrossobst
         if ( associated(km(iseqkm)%rcrossobst) ) &
            deallocate(km(iseqkm)%rcrossobst)
         allocate ( km(iseqkm)%rcrossobst(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'rcrossobst' )
         if ( ierror/=0 ) go to 1000

         km(iseqkm)%rcrossobst = rcrossobst(1:length)
         deallocate ( rcrossobst )

         rcrossobst => km(iseqkm)%rcrossobst
         kmeshad(4) = length

      else

!     --- No intersections found, delete previous array if it exists

         if ( associated(km(iseqkm)%rcrossobst) ) &
            deallocate(km(iseqkm)%rcrossobst)
         nullify ( rcrossobst )
         kmeshad(4) = 0

      end if

      length = lenicrossobst
      allocate ( km(iseqkm)%icrossobst(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'icrossobst' )
      if ( ierror/=0 ) go to 1000

      km(iseqkm)%icrossobst = icrossobst(1:length)
      deallocate ( icrossobst )

      icrossobst => km(iseqkm)%icrossobst
      kmeshad(3) = length

      kmeshad(8) = iseqlc

      if ( lenrcrossobst>0 .and. &
           (debug .or. plotobst .or. printobst) ) then

!     --- debug = true, plot and print intersection points

         call sepplotinters

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lenicrossobst, lenrcrossobst', &
                          lenicrossobst, lenrcrossobst
         write(irefwr,*) 'End seplevcross1d'

      end if  ! ( debug )
      call erclos ( 'seplevcross1d' )

      end
