      subroutine er0040 ( inodes, nnodes, number, vector, funcx, funcy )
! ======================================================================
!
!        programmer    Guus Segal
!        version 3.0   date 05-02-1999 Extra parameters
!        version 2.0   date 15-11-1998 Complete revision
!        version 1.2   date 27-11-1992 Removal of superfluous parameters
!
!   copyright (c) 1985-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute values of vector in nodal points of curves/surfaces (real case)
!     values are stored in funcy, co-ordinates in funcx
! **********************************************************************
!
!                       KEYWORDS
!
!    copy
!    curve
!    surface
!    value
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer inodes(*), nnodes, number
      double precision vector(*), funcx(*), funcy(*)

!     funcx      o    user array of type 117, where the coordinates
!                     are stored
!     funcy      o    user array of type 117, where the degrees of freedom
!                     are stored
!     inodes    i/o   Integer array containing the node numbers that must be
!                     filled. If no unknowns are available in the node the
!                     node is removed
!     nnodes    i/o   Number of nodes
!                     At input number of nodes in the set
!                     At output number of nodes in the set with unknowns
!     number     i    the number-th degree of freedom in each nodal point
!                     is considered
!                     when number=0 all degrees of freedom are stored
!     vector     i    array containing the vector from which the values
!                     must be computed
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount, inode, inunkp, inusol, j, numb1, k

!     i          Counting variable
!     icount     Counter
!     inode      Node sequence number
!     inunkp     Number of degrees of freedom in point
!     inusol     Position of unknown in array vector
!     j          Counting variable
!     k          Counting variable
!     numb1      Sequence number of degree of freedom
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( number>0 or nphysi=1 ) then
!        For all nodes in inodes do
!           Copy one value into array funcy
!     else ( nphysi>1 ) then
!        For all nodes in inodes do
!           Copy all values into array funcy
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      inunkp = nphysi
      if ( indprpi.gt.0 .and. number.eq.0 ) then

!     --- indprpi > 0, Use array kprobpi (all unknowns)

         icount = 0
         do i = 1, nnodes
            inode = inodes(i)
            inusol = kprobpi(inode,1)
            do j = 1, ndim
               funcx(icount*ndim+j) = coor(j,inode)
            end do
            icount = icount+1
            do k = 1, nphysi
               inusol = kprobpi(inode,k)
               funcy((i-1)*nphysi+k) = vector(inusol)
            end do
            inodes(icount) = inode
         end do

      else if ( indprpi.gt.0 .and. number.gt.0 ) then

!     --- indprpi > 0, Use array kprobpi, only one unknown

         icount = 0
         do i = 1, nnodes
            inode = inodes(i)
            inusol = kprobpi(inode, number)
            if ( inusol.gt.0 ) then
               do j = 1, ndim
                  funcx(icount*ndim+j) = coor(j,inode)
               end do
               icount = icount+1
               funcy(icount) = vector(inusol)
               inodes(icount) = inode
            end if
         end do

      else if ( number.gt.0 .or. nphysi.eq.1 ) then

!     --- only one degree of freedom per nodal point

         numb1 = max(1,number)
         icount = 0
         do i = 1, nnodes

            inode = inodes(i)
            if ( indprfi.eq.0 ) then
               inusol = (inode-1)*nphysi+numb1
            else
               inusol = kprobfi(inode)
               inunkp = kprobfi(inode+1)-inusol
               inusol = inusol+numb1
            end if
            if ( inunkp.gt.0 ) then
               do j = 1, ndim
                  funcx(icount*ndim+j) = coor(j,inode)
               end do
               icount = icount+1
               funcy(icount) = vector(inusol)
               inodes(icount) = inode
            end if

         end do

         nnodes = icount

      else if ( indprfi.eq.0 ) then

!     --- number = 0  store all degrees of freedom
!         indprfi = 0, the number of degrees of freedom per point is constant

         do i = 1, nnodes

            inode = inodes(i)
            inusol = (inode-1)*nphysi
            do j = 1, nphysi
               funcy((i-1)*nphysi+j) = vector(inusol+j)
            end do
            do j = 1, ndim
               funcx((i-1)*ndim+j) = coor(j,inode)
            end do

         end do

      else

!     --- number = 0  store all degrees of freedom
!         indprfi # 0, the number of degrees of freedom per point is not constant

         icount = 0
         do i = 1, nnodes

            inode = inodes(i)
            inusol = kprobfi(inode)
            inunkp = kprobfi(inode+1)-inusol
            if ( inunkp.gt.0 ) then
               do j = 1, ndim
                  funcx(icount*ndim+j) = coor(j,inode)
               end do
               icount = icount+1
               do j = 1, inunkp
                  funcy((icount-1)*nphysi+j) = vector(inusol+j)
               end do
               inodes(icount) = inode
            end if

         end do

         nnodes = icount

      end if

      end
