      subroutine er0058 ( iintbn, ivectr, icurvs, value )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.9  date 29-05-2014 remove call to time subroutine
!        version  4.8  date 30-11-2012 Long integers
!        version  4.7  date 05-10-2011 New call to er0009
!
!   copyright (c) 1987-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute a boundary integral over a part of the boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecactl
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iintbn(*), ivectr, icurvs(*)
      double precision value(*)

!     icurvs  i   Integer array containing the curve numbers for the
!                 computation of the boundary integral. A negative curve
!                 number indicates that the curve must be computed in
!                 the reversed direction.
!     iintbn  i   Integer input array in which the user indicates what type
!                 of integral must be computed.
!                 IINTBN must be filled as follows:
!                 IINTBN(1) Type of calling subroutine
!                           1:  BOUNIN
!                           2:  BOUNIV
!                           3:  INTBOU
!                           4:  INTBND
!                 IINTBN(2) (ICHINT) indicates what type of integral has to
!                           be computed. Possibible values:
!                           1          /  f u  ds
!                           2          /  f  u.n  ds
!                           3          /  f  u.t  ds
!                           4          /  f  n. sigma .n  ds
!                           5          /  f  n. sigma .t  ds
!                           6          /  f  n. sigma     ds
!                           7          /  f  u  n  ds
!                           8          sum f(i) u(i)
!                           9          /  f ds
!                           The possibilities 3 to 6 have only been implemented
!                           for NDIM = 2, i.e. two-dimensional regions
!                           Default value: 1
!                 IINTBN(3) (ICHFUN) indication of the value of f.
!                           Possibible values:
!                           0         f = 1
!                           >0        f = func ( ichfun, x, y, z ) (real case)
!                                     or
!                                     f = cfunc ( ichfun, x, y, z ) (complex
!                                         case)
!                                     at this moment only the combination f#1
!                                     and ichint=1 is permitted
!                           Default value: 0
!                 IINTBN(4) (IRULE) The numerical integration rule to be carried
!                           out depends on IRULE. Possibilities:
!                           Possibible values:
!                           For curves only (i.e. 2D):
!                           1         trapezoid rule,  cartesian co-ordinates
!                           2         trapezoid rule,  axisymmetric co-ordinates
!                           3         simpson's rule   cartesian co-ordinates
!                           4         simpson's rule   axisymmetric co-ordinates
!                           For surfaces only (i.e. 3D):
!                           1         Newton cotes rule, i.e. integration rule
!                                     based on the type of approximation used
!                                     in the element
!                           Default value: 1
!                 IINTBN(5) (IDIM) This parameter indicates if curves (1)
!                           or surfaces (2) must be computed
!                           Default value: NDIM-1
!                 IINTBN(6) (NUMBER) This parameter indicates which unknown
!                           (physical degree of freedom) must be used from
!                           the (solution) vector u (IVECTR)
!                           If ICHINT = 2, 3 or 7, it is supposed that the
!                           degrees of freedom are equal to NUMBER, NUMBER+1 and
!                           NUMBER+2 (3D)
!                           If ICHINT = 4, 5 or 6, it is supposed that the
!                           degrees of freedom are equal to NUMBER, NUMBER+1 and
!                           NUMBER+2 (2D) or NUMBER to NUMBER+5 (3D)
!                           Default value: 1
!                 IINTBN(7) (NCURVES) Indicates the number of curves or surfaces
!                           in array ICURVS that must be used for the
!                           integration
!                           Default value: 1
!     ivectr  i    standard sepran array referring to usol or vector of special
!                  structure defined in nodal points
!     value   o    array containing the computed value of the integral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer itypvc, i, ichfun, ichint, number, numcrv

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     i              Counting variable
!     ichfun         choice parameter for the computation of f
!                    Possibible values:
!                    0         f = 1
!                    >0        f = func ( ichfun, x, y, z ) (real case)
!                                  or
!                              f = cfunc ( ichfun, x, y, z ) (complex case)
!                              at this moment only the combination f#1
!                              and ichint=1 is permitted
!     ichint         choice parameter for the type of integral
!                    Possibible values:
!                    1          /  f u  ds
!                    2          /  f  u.n  ds
!                    3          /  f  u.t  ds
!                    4          /  f  n. sigma .n  ds
!                    5          /  f  n. sigma .t  ds
!                    6          /  f  n. sigma     ds
!                    7          /  f  u  n  ds
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and number=2*i
!                    to the imaginary part of array i
!     numcrv         Number of curves in icurvs
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0009         compute a boundary integral over curves, real case
!     ER0079         compute a boundary integral over curves, complex case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!      57   irule has wrong value
!      60   Input solution array has not been filled or is of the wrong
!     102   NUMBER has wrong value
!     388   ichois = 2,  whereas the dimension of the space is not equal to 2
!     389   ichois = 3,  whereas the dimension of the space is equal to 1
!     410   ichois has incorrect value in the case that the dimension of
!           the space is not equal to 2
!     451   ichois has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0058' )

!     --- checks of the parameters

      ichint = iintbn(2)
      if ( ichint/=9 ) then

!     --- ichint # 9, use usol

         itypvc = ks(ivectr)%typevec
         if ( itypvc/=110 .and. itypvc/=115 ) then
            call errint ( itypvc, 1 )
            call errsub ( 60, 1, 0, 0 )
         end if

      end if
      ichfun = iintbn(3)
      irule  = iintbn(4)
      number = iintbn(6)
      numcrv = iintbn(7)
      if ( ichfun>0 .and. ichint/=1 ) then
         call errint ( ichfun, 1 )
         call errint ( ichfun, 2 )
         call errint ( ichint, 3 )
         call errsub ( 451, 3, 0, 0 )
      end if
      if ( ichint<1 .or. ichint>9 ) then
         call errint ( ichint, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      if ( ichfun<0 ) then
         call errint ( ichfun, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      if ( irule<1 .or. irule>4 ) then
         call errint ( irule, 1 )
         call errsub ( 57, 1, 0, 0 )
      end if
      if ( ichint==2 .and. ndim/=2 ) then
         call errsub ( 388, 0, 0, 0 )
      end if
      if ( ichint==3 .and. ndim==1 ) then
         call errsub ( 389, 0, 0, 0 )
      end if
      if ( ichint>=4 .and. ndim<2 .or. ndim>4 ) then
         call errint ( ichint, 1 )
         call errsub ( 410, 1, 0, 0 )
      end if

!     --- reserve space for standard arrays

      if ( ichint==9 ) then

!     --- ichint = 9, no solution vector needed

         compl = .false.

      else

!     --- ichint # 9, solution vector needed

         compl = ks(ivectr)%compl==1
         if ( nphysi<number .or. number<=0 )  then
            call errint ( number, 1 )
            call errint ( nphysi, 2 )
            call errsub ( 102, 2, 0, 0 )
         end if

      end if

      if ( ichint/=9 ) usolt => ks(ivectr)%sol

!     --- actual computation

      npointcac = npoint
      if ( ierror/=0 ) go to 1000
      if ( compl ) then

!     --- Complex case

         call er0079 ( ichint, irule, numcrv, usolt, number, &
                       ichfun, value, icurvs )

      else

!     --- Real case

         call er0009 ( ichint, irule, numcrv, usolt, number, &
                       ichfun, value, icurvs )

      end if

1000  call erclos ( 'er0058' )
      end subroutine er0058
