      subroutine erintg ( iintbn, iintlc, icrv1, icrv2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 08-05-1994 new definition of iintlc, &
!                                      extra parameters icrv1 and icrv2)
!        version  1.0  date 23-06-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau  SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine erintg is used to fill array iintlc with the values
!    of iintbn extended with default values
!    ERINTB is a help subroutine for subroutine INTBOU
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iintbn(*), iintlc(*), icrv1, icrv2

!     icrv1   i    First curve number
!     icrv2   i    Last curve number
!     iintbn  i   Integer input array in which the user indicates what type
!                 of integral must be computed.
!                 IINTBN must be filled as follows:
!                 IINTBN(1) gives the last position of array IINTBN that has
!                           been filled by the user. For all other positions
!                           only defaults are used.
!                           So: if IINTBN(1)=0 or 1 only defualts are used
!                 IINTBN(2) (ICHINT) indicates what type of integral has to
!                           be computed. Possibible values:
!                           1          /  f u  ds
!                           2          /  f  u.n  ds
!                           3          /  f  u.t  ds
!                           4          /  f  n. sigma .n  ds
!                           5          /  f  n. sigma .t  ds
!                           6          /  f  n. sigma     ds
!                           7          /  f  u  n  ds
!                           The possibilities 2 to 7 have only been implemented
!                           for NDIM = 2, i.e. two-dimensional regions
!                           Default value: 1
!                 IINTBN(3) (ICHFUN) indication of the value of f.
!                           Possibible values:
!                           0         f = 1
!                           >0        f = func ( ichfun, x, y, z ) (real case)
!                                     or
!                                     f = cfunc ( ichfun, x, y, z ) (complex
!                                         case)
!                                     at this moment only the combination f#1
!                                     and ichint=1 is permitted
!                           Default value: 0
!                 IINTBN(4) (IRULE) The numerical integration rule to be carried
!                           out depends on IRULE. Possibilities:
!                           Possibible values:
!                           For curves only (i.e. 2D):
!                           1         trapezoid rule,  cartesian co-ordinates
!                           2         trapezoid rule,  axisymmetric co-ordinates
!                           3         simpson's rule   cartesian co-ordinates
!                           4         simpson's rule   axisymmetric co-ordinates
!                           For surfaces only (i.e. 3D):
!                           1         Newton cotes rule, i.e. integration rule
!                                     based on the type of approximation used
!                                     in the element
!                           Default value: 1
!                 IINTBN(5) (IDIM) This parameter is used for curves only
!                           It indicates if the curves are given in the normal
!                           order (1) or in reversed order (-1)
!                           Default value: 1
!                 IINTBN(6) (NUMBER) This parameter indicates which unknown
!                           (physical degree of freedom) must be used from
!                           the (solution) vector u (IVECTR)
!                           If ICHINT = 2, 3 or 7, it is supposed that the
!                           degrees of freedom are equal to NUMBER, NUMBER+1 and
!                           NUMBER+2 (3D)
!                           If ICHINT = 4, 5 or 6, it is supposed that the
!                           degrees of freedom are equal to NUMBER, NUMBER+1 and
!                           NUMBER+2 (2D) or NUMBER to NUMBER+5 (3D)
!                           Default value: 1
!                 IINTBN(7) (ICRV1) This parameter is used in combination with
!                           ICRV2
!                           Default value: 1
!                 IINTBN(8) (ICRV2) This parameter is used in combination with
!                           ICRV1.
!                           In 2D the integral is computed over the curves
!                           ICRV1 to ICRV2 (IDIM=1) or over these curves
!                           in reversed order (IDIM=-1). In the last case the
!                           starting point is the last point of ICRV1
!                           When ICHINT=3 the tangential vector is taken into
!                           the direction of the curves. When ICHINT=2, the
!                           curve must be such that the surface is at the left- &
!                           hand side of the curves.
!                           In 3D the integral is computed over the sufaces
!                           ICRV1 to ICRV2.
!                           Default value: 1
!     iintlc  o   Input array for subroutine ER0058 to be filled as follows:
!                 1:  ISUBR (3)
!                 2:  ICHINT
!                 3:  ICHFUN
!                 4:  IRULE
!                 5:  IDIM
!                 6:  NUMBER
!                 7:  NUMCRV
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numcrv

!     i       Counting variable
!     numcrv  number of parts where curves are given for generation
!             of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     155   first surface number has incorrect value
!     156   last surface number has incorrect value
!     185   first curve number (icrv1) is out of range
!     186   last curve number (icrv2) is out of range
!     419   NDIM # 2
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erintg' )
      if ( ndim/=2 .and. ndim/=3 ) then
         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )
      end if

!     --- Fill iintlc with defaults

      do 100 i = 2, 6
         iintlc(i) = 1
100   continue
      iintlc(1) = 3
      do 110 i = 2, min(6,iintbn(1))
         iintlc(i) = iintbn(i)
110   continue
      icrv1 = 1
      icrv2 = 1
      if ( iintbn(1)>=7 ) icrv1 = iintbn(7)
      if ( iintbn(1)>=8 ) icrv2 = iintbn(8)
      numcrv = abs(icrv2-icrv1)+1
      iintlc(7) = numcrv

!     --- Check icrv1 and icrv2

      if ( ndim==2 ) then

!     --- ndim = 2, curves

         if ( icrv1<1 .or. icrv1>ncurvs ) then
            call errint ( icrv1, 1 )
            call errint ( ncurvs, 2 )
            call errsub ( 185, 2, 0, 0 )
         end if
         if ( icrv2<icrv1 .or. icrv2>ncurvs ) then
            call errint ( icrv2, 1 )
            call errint ( icrv1, 2 )
            call errint ( ncurvs, 3 )
            call errsub ( 186, 3, 0, 0 )
         end if
      else

!     --- ndim = 3, surfaces

         iintlc(5) = 2
         if ( icrv1<1 .or. icrv1>nsurfs ) then
            call errint ( icrv1, 1 )
            call errint ( nsurfs, 2 )
            call errsub ( 155, 2, 0, 0 )
         end if
         if ( icrv2<icrv1 .or. icrv2>nsurfs ) then
            call errint ( icrv2, 1 )
            call errint ( icrv1, 2 )
            call errint ( nsurfs, 3 )
            call errsub ( 156, 3, 0, 0 )
         end if
      end if
      call erclos ( 'erintg' )
      end
