      subroutine ermapdelouter ( imap, rmap, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-08-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove all entries in imap/rmap with negative weights in order to
!     prevent extrapolation outside the original domain
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     delete
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, imap(npoint,*)
      double precision rmap(npoint,*)

!     imap          i/o   Mapping array
!                         Array imap is filled as follows:
!                         imap(i,1) = 0 means node i has not been interpolated
!                         yet
!                         imap(i,1) = n (>0) means node i must be interpolated
!                                     with n points
!                         imap(i,2-n+1) contain the nodes in the old mesh
!                                     required for the interpolation
!     npoint         i    Number of nodal points
!     rmap          i/o   Mapping array
!                         If imap(i,1) = n (>0) then the positions
!                         rmap(i,1-n) contain the weight factor for the
!                         interpolation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ninterpol, j
      logical negative

!     i              Counting variable
!     j              Counting variable
!     negative       if true a negative interpolation value has been found
!     ninterpol      number of points used for the interpolation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ermapdelouter' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ermapdelouter'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Loop over all nodes

      do i = 1, npoint

         ninterpol = imap(i,1)
         negative = .false.
         do j = 1, ninterpol

            if ( rmap(i,j).le.-1d-1 ) then

!           --- negative entry found, set negative

               negative = .true.

            end if  ! ( rmap(i,j).le.-1d-6 )

         end do  ! j = 1, ninterpol

         if ( negative ) then

!        --- negative entry found, set ninterpol to 0

            imap(i,1) = 0

         end if  ! ( negative )

      end do  ! i = 1, npoint

1000  call erclos ( 'ermapdelouter' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ermapdelouter'

      end if

      end
