      subroutine erput01 ( usol, inodes, funcy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 13-04-2002 New parameter list
!        version  1.0  date 23-10-1998
!
!   copyright (c) 1998-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Copy contents of array funcy for all nodes defined in inodes into
!   array usol (Real case)
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     user_vector
!     sepran_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*)
      double precision usol(*), funcy(*)

!     funcy          i    user array of type 117, where the degrees of freedom
!                         of the points are stored
!                         funcy must be an array of the type produced by COMPCR
!                         or COMPSR
!     inodes         i    integer array containing the node numbers
!                         corresponding to the arrays funcy in the same sequence
!                         from position 6
!                         This must be an array of type 128 as produced by
!                         compsr
!     usol          i/o   Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inunkp, iunk, k, ndegfd, nnodes, node

!     i          Counting variable
!     inunkp     Number of degrees of freedom in point
!     iunk       Position of degree of freedom in node
!     k          Counting variable
!     ndegfd     Number of degrees of freedom per point
!     nnodes     Number of nodes
!     node       Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2021   Degree of freedom not available in node
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erput01' )
      nnodes = inodes(5)
      ndegfd = nint(funcy(4))
      inunkp = nphysi
      if ( indprpi.gt.0 ) then

!     --- Array kprob part p has been filled

         do i = 1, nnodes
            node = inodes(i+5)
            do k = 1, ndegfd
               iunk = kprobpi(node,k)
               if ( iunk.eq.0 ) then

!              --- Unknown not present in point

                  call errint ( k, 1 )
                  call errint ( node, 2 )
                  call errint ( ndegfd, 3 )
                  call errsub ( 2021, 3, 0, 0 )

               else

                  usol(iunk) = funcy(5+(i-1)*ndegfd+k)

               end if

            end do
         end do

      else if ( indprfi.gt.0 ) then

!     --- Array kprob part p has not been filled
!         The number of degrees of freedom per point is not constant

         do i = 1, nnodes
            node = inodes(i+5)
            iunk = kprobfi(node)
            inunkp = kprobfi(node+1) - iunk
            if ( inunkp.lt.ndegfd ) then

!           --- Unknown not present in point

                  call errint ( inunkp+1, 1 )
                  call errint ( node, 2 )
                  call errint ( ndegfd, 3 )
                  call errsub ( 2021, 3, 0, 0 )

            end if
            do k = 1, ndegfd
               usol(iunk+k) = funcy(5+(i-1)*ndegfd+k)
            end do
         end do

      else

!     --- Array kprob part p has not been filled
!         The number of degrees of freedom per point is constant

         do i = 1, nnodes
            node = inodes(i+5)
            do k = 1, ndegfd
               usol((node-1)*inunkp+k) = funcy(5+(i-1)*ndegfd+k)
            end do
         end do

      end if
      call erclos ( 'erput01' )

      end
