      subroutine incpcm ( n, intdi1, intmt1, intdi2, intmt2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-04-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau  SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       incpcm is used to copy the contents of an array of type
!       INTMAT with method = 5, into an array of type INTMAT with method = 9
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, intdi1(n+1), intmt1(*), intdi2(n+1), intmt2(*)

!     intdi1  i   Array to be transformed (INTMAT part 1, first part, &
!                 method = 5 )
!     intdi2  o   Transformed array (INTMAT part 1, first part, method = 9 )
!     intmt1  i   Array to be transformed (INTMAT part 1, second part, &
!                 method = 5 )
!     intmt2  o   Transformed array (INTMAT part 1, second part, method = 9 )
!     n       i   Number of degrees of freedom corresponding to intmt1/2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icol, j, k, iwork(n)

!    icol     Column number
!    iwork    Work array of length n
!    j        General loop variable
!    k        General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Compute INTDI2:
!      Set IWORK := 0
!      For k := 2, n do
!         For j := 1, INTDI1(k+1) - INTDI1(k)
!            column number := INTMT1( INTDI1(k)+j )
!            IWORK(column number) := IWORK(column number) + 1
!      INTDI2(1) := 0
!      For k := 1, n do
!         INTDI2(k+1) := INTDI2(k) + IWORK(k) + INTDI1(k+1)-INTDI1(k)
!    Fill IWORK for each row with number of columns in under triangle by
!    copying information from INTMT1: IWORK(k) := INTDI1(k+1)-INTDI1(k) (k=1,n)
!    Copy contents of INTMT1 into INTMT2:
!      For k := 2, n do
!         For j := 1, INTDI1(k+1) - INTDI1(k)
!            INTMT2 ( INTDI2(k)+j ) := INTMT1 ( INTDI1(k)+j)
!    Fill upper triangle of matrix INTMT2:
!      For k := 2, n do
!         For j := 1, INTDI1(k+1) - INTDI1(k)
!            column number := INTMT1( INTDI1(k)+j )
!            IWORK(column number) := IWORK(column number)  + 1
!            INTMT2( INTDI2(column number)+IWORK(column number) ) := k
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute INTDI2:
!           Set IWORK := 0

      iwork = 0

!     --- Fill IWORK with number of columns in upper triangle for each row

      do k = 2, n
         do j = 1, intdi1(k+1) - intdi1(k)
            icol = intmt1(intdi1(k)+j)
            iwork(icol) = iwork(icol) + 1
         end do
         end do

!        --- Fill array INTDI2

      intdi2(1) = 0
      do k = 1, n
         intdi2(k+1) = intdi2(k) + iwork(k) + intdi1(k+1) - intdi1(k)
      end do

!     --- Fill IWORK for each row with number of columns in under triangle

      do k = 1, n
         iwork(k) = intdi1(k+1) - intdi1(k)
      end do

!     --- Copy contents of INTMT1 into INTMT2:

      do k = 2, n
         do j = 1, iwork(k)
            intmt2 ( intdi2(k)+j ) = intmt1 ( intdi1(k)+j )
         end do
      end do

!     --- Fill upper triangle of matrix INTMT2:

      do k = 2, n
         do j = 1, intdi1(k+1) - intdi1(k)
            icol = intmt1(intdi1(k)+j)
            iwork(icol) = iwork(icol) + 1
            intmt2( intdi2(icol)+iwork(icol) ) = k
         end do
      end do
      end
