      subroutine incpus ( iincop, isol, routcp, ichois, &
                          inode, idegfd, icompl, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-07-1994 New definition of ivectr(3
!        version  1.0  date 15-12-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau  SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine INCPUS is a help subroutine for subroutine COPYUS
!    With this subroutine the input is checked and some parameters are set
! **********************************************************************
!
!                       KEYWORDS
!
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iincop(*), isol, ichois, inode, &
              idegfd, icompl, istart
      double precision routcp(*)

!     ichois   o  Choice parameter, see IINCOP
!     icompl   o  Indicates if vector is real (0) or complex (1), see IINCOP
!     idegfd   o  Degree of freedom, see IINCOP
!     iincop   i  User input array
!                 In this array the user defines which output the user requires
!                 from the solution. The output may be restricted to a user
!                 point, a curve, a surface or to the complete mesh etc.
!                 IINCOP must be filled as follows:
!                 Pos. 1   Number of entries that are filled by the user
!                          All other positions are defined by default values
!                          So if IINCOP(1) = 0 or 1 only defaults are used
!                 Pos. 2   ICHOIS: Defines what type of nodes must be used.
!                          Possible values:
!                          0: copy contents of isol into ROUTCP
!                          1: copy contents of ROUTCP into isol
!                             In this case the user must give IPROB (IINCOP(5))
!                             a value
!                             The array ISOL is supposed to be a solution vector
!                          2: Copy the values of the unknowns in user point
!                             IPOINT into ROUTCP
!                          10: See 0, COPYUS has been called by COPYUV, i.e.
!                              KMESH has not been filled
!                          11: See 1, COPYUS has been called by COPYUV, i.e.
!                              KMESH has not been filled
!                          Default value: 0
!                 Pos. 3   IDEGFD: Defines the specific unknowns in each point
!                          that must be copied into ROUTCP
!                          If IDEGFD = 0, all unknowns are copied
!                          Default value: 0
!                 Pos. 4   INODE:  Defines the user point in which the unknown
!                          must be selected (ICHOIS=2)
!                          Default value: 1
!                 Pos. 5   IPROB Problem number, must only be filled
!                          for ichois = 1
!                          Default value: 1
!                 Pos. 6   ICOMPL Indication if the vector is real (0) or
!                          complex, must only be filled for ichois = 1
!                          Default value: 0
!                 Pos. 7   ISTART, Starting position in array ROUTCP, with
!                          respect to the information.
!                          Default value: 6
!     inode    o  User point number, see IINCOP
!     isol     i  Standard SEPRAN solution array of type 110, 115 or 116
!     istart   o  Starting position in ROUTCP, see IINCOP
!     routcp   i  User array in which the computed value(s) is (are)  stored
!                 ROUTCP is a variable length array which implies that the
!                 first position must be filled by the user with the declared
!                 length
!                 It is advised to set the third position equal to 0
!                 The output is always stored from position 6!
!                 If ICHOIS = 1, ROUTCP is an intput array to be filled
!                 from position 6.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ninput
      parameter ( ninput=7 )
      integer i, input(ninput)

!    i          General loop variable
!    input      Copy of array IINCOP extended with defaults
!    ninput     Length of array INPUT
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERREAL  Puts values of reals in integer error array
!     ERRINT  Puts values of integers in integer error array
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong
!     287   Array IINCOP(1) has incorrect value
!     295   ICHOIS (IINCOP(2)) has incorrect value
!     699   Problem number incorrect
!    1213   IDEGFD (IINCOP(3)) has incorrect value
!    1214   IDEGFD (IINCOP(4)) has incorrect value
!    1215   IPROB (IINCOP(5)) has incorrect value
!    1216   Position 1 of ROUTCP < 6
!    1218   ICOMPL (IINCOP(6)) < 0 or > 1
!    1284   ISTART (IINCOP(7)) < 6
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incpus' )

!     --- Test array iincop

      if ( iincop(1)<0 .or. iincop(1)>ninput ) then

!     --- Error 287: iincop(1) incorrect

         call errint ( iincop(1), 1 )
         call errint ( ninput, 2 )
         call errsub ( 287, 2, 0, 0 )
      end if

!     --- Fill array input:

      do 100 i = 1, ninput
         input(i) = 0
100   continue
      input(4) = 1
      input(5) = 1
      input(7) = 6
      do 110 i = 2, min(ninput,iincop(1))
         input(i) = iincop(i)
110   continue

!     --- Perform tests

      ichois = input(2)
      if ( ichois<10 ) then
      else
         ichois = ichois-10
      end if
      if ( ichois<0 .or. ichois>2 ) then

!     --- Error 295: iincop(2) incorrect

         call errint ( ichois, 1 )
         call errint ( 2, 2 )
         call errsub ( 295, 2, 0, 0 )
      end if
      idegfd = input(3)
      if ( ichois/=1 .and. idegfd<0 ) then

!     --- Error 1213: iincop(3) incorrect

         call errint ( idegfd, 1 )
         call errsub ( 1213, 1, 0, 0 )
      end if
      inode  = input(4)
      if ( ichois==2 .and. inode<0 .or. inode>nuspnt ) then

!     --- Error 1214: iincop(4) incorrect

         call errint ( inode, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 1214, 2, 0, 0 )
      end if
      if ( ichois==1 ) then

!     --- ichois = 1

         iprob = input(5)
         icompl = input(6)
         if ( iprob<1 .or. iprob>nprob ) then

!        --- Error 1215: iincop(5) incorrect

            call errint ( iprob, 1 )
            call errint ( nprob, 2 )
            call errsub ( 1215, 2, 0, 0 )
         end if
         if ( icompl<0 .or. icompl>1 ) then

!        --- Error 1218: iincop(6) incorrect

            call errint ( icompl, 1 )
            call errsub ( 1218, 1, 0, 0 )
         end if
      else

!     --- ichois # 1

         if ( ks(isol)%typevec/=110 .and. ks(isol)%typevec/=115 ) then
            call errint ( ks(isol)%typevec, 1 )
            call errsub ( 60, 1, 0, 0 )
         end if
         iprob = ks(isol)%iprob
         if ( iprob<1 .or. iprob>nprob )  then

!        --- Error 699:  iprob < 1 or iprob > nprob

            call errint ( iprob, 1 )
            call errint ( nprob, 2 )
            call errsub ( 699, 2, 0, 0 )
         end if
         icompl = ks(isol)%compl
      end if
      istart = input(7)
      if ( abs(istart)<=5 ) then

!     --- Error: |istart| < 6

         call errint ( istart, 1 )
         call errsub ( 1284, 1, 0, 0 )
      end if
      if ( routcp(1)<istart ) then

!     --- Error 1216: routcp(1) < 6

         call erreal ( routcp(1), 1 )
         call errint ( istart, 1 )
         call errsub ( 1216, 1, 1, 0 )
      end if
      call erclos ( 'incpus' )
      end
