      subroutine pr0004 ( jmethod, nrusol, intmt1, intmt2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 17-12-2015 New parameter list
!        version  2.6  date 28-07-2014 Extension with jmethod = 55
!        version  2.5  date 27-02-2014 Extra error message
!
!   copyright (c) 1982-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print first part of array intmat
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jmethod, nrusol, intmt1(*), intmt2(*)

!     intmt1   i  first part of array intmat part 1
!     intmt2   i  second part of array intmat part 1
!     jmethod   i  parameter jmethod in subroutine commat
!                 indicating the type of storage
!     nrusol   i  number of free degrees of freedom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isum, n, m, k

!     i              Counting variable
!     isum           Integer variable to compute a sum
!     k              Counting variable
!     m              number of items to be printed in one line
!     n              number of items to be printed for one row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1406   The storage scheme is not implemented in this routine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr0004' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pr0004'
         call prinin ( intmt1, nrusol+1, 'intmt1' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( jmethod )
      case(1:12, 51:55)

!     --- jmethod = 1:12 or 51:54   compact storage

         write(irefwr,200)
200      format( ' compact storage of matrix'/ &
                 ' rows i coupled with internal degrees of freedom' )
         isum=0
         do i=1,nrusol
            n=intmt1(i+1)-isum
            m=min(10,n)
            write(irefwr,210) i,(intmt2(isum+k),k=1,m)
210         format(1x,i6,':',1x,10i6)
            if(n>m) write(irefwr,220) (intmt2(isum+k),k=m+1,n)
220         format(9x,10i6)
            isum=isum+n
         end do

      case(56)

!     --- jmethod = 56   petsc storage, only the number of non-zero elements
!                        per column is stored

         write(irefwr,300)
300      format( ' petsc storage of matrix' )

         call prinin ( intmt1, nrusol, &
                       'number of non-zero elements per row' )

      case default

!     --- Value of jmethod not yet available

         call errint ( jmethod, 1 )
         call errsub ( 1406, 1, 0, 0 )

      end select  ! case ( jmethod )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pr0004'

      end if  ! ( debug )
      call erclos ( 'pr0004' )

      end
