      subroutine pr0030 ( ichois, kmeshl, coor, list, nlist, ndim, &
                          nuspnt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-06-1993
!
!   copyright (c) 1993-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Print (selected) information of user points
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, kmeshl(*), list(*), nlist, ndim, nuspnt
      double precision coor(ndim,*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     ichois  i    Choice parameter
!                  Possible values:
!                  0:   print nodal point numbers of all user points
!                  1:   print nodal point numbers of all user points
!                       and print the co-ordinates also
!                  10,11: See 0, 1, however in this case only the user points
!                       with user point sequence number between list(1)
!                       and list(2) are printed
!                  20,21: See 0, 1, however in this case only the user points
!                       with user point sequence number in list(1) to
!                       list(nlist) are printed
!     kmeshl  i    KMESH part l, contains nodal point numbers of the user points
!     list    i    See ichois
!     ndim    i    Dimension of the space.
!     nlist   i    Length of array list in case ichois>19
!     nuspnt  i    number of user points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical pricor, chslst
      integer i, iunder, iupper, nodusr, nodp, j, ilist

!     chslst  Indicate if the nodal point numbers must be found from
!             list (true) or not (false)
!     i       Counting variable
!     ilist   ichois/20
!     iunder  lower bound for loop variable
!     iupper  upper bound for loop variable
!     j       Counting variable
!     nodp    Nodal point sequence number
!     nodusr  User point sequence number
!     pricor  Indicates if co-ordinates must be printed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Print heading
!   pricor := co-ordinates must be printed
!   choose_list := ichois >= 20
!   Define range of loop (iunder, iupper)
!   For i := iunder (1) iupper do
!      if ( choose_list ) then
!         nodusr := list(i)
!      else ( not choose_list )
!         nodusr := i
!      nodp := kmeshl(i)
!      if ( pricor ) then
!         write nodp and co-ordinates
!      else ( not pricor )
!         write nodp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
100   format ( '  User point    Nodal point' )
110   format ( '  User point    Nodal point        Co-ordinates' )
120   format ( i11, 5x, i11, ss, 3(1pe13.4e2) )

!     --- Print heading

      pricor = mod(ichois,10).eq.1
      if ( pricor ) then
         write ( irefwr,110 )
      else
         write ( irefwr,100 )
      end if

!     --- Set choose_list, iunder, iupper

      ilist = ichois/10
      chslst = ilist.eq.2
      if ( ilist.eq.0 ) then
         iunder = 1
         iupper = nuspnt
      else if ( ilist.eq.1 ) then
         iunder = list(1)
         iupper = list(2)
      else
         iunder = 1
         iupper = nlist
      end if

!     --- loop over user points

      do 200 i = iunder, iupper
         if ( chslst ) then
            nodusr = list(i)
         else
            nodusr = i
         end if
         nodp = kmeshl(nodusr)
         if ( pricor ) then
            write(irefwr,120) nodusr, nodp, (coor(j,nodp),j=1,ndim)
         else
            write(irefwr,120) nodusr, nodp
         end if
200   continue
      end
