      subroutine pr2026 ( lenkmtri )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-04-2005 Replace body by new subroutine
!        version  1.1  date 11-01-199 Extension for more parts
!        version  1.0  date 06-12-1996
!
!   copyright (c) 1996-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute length of array kmeshtri, which is used to subdivide an
!     existing 2D mesh into three-point triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenkmtri

!     lenkmtri   o    Length of array kmeshtri
!                     If 0, no subdivision is required
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp
      logical subdiv

!     ielgrp     element group number
!     subdiv     If true the elements must be subdivided
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR2026SUB      Raise lenkmtri as function of shape number and
!                    Set subdiv if necessary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    subdiv := false
!    lenkmtri := 0
!    For all element groups do
!       inelem := number of elements in element group
!       if ( ishape#3 ) then
!          raise lenkmtri
!          subdiv := true
!       else
!          raise lenkmtri
!    if ( not subdiv ) then
!       lenkmtri := 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr2026' )
      subdiv = .false.

!     --- For all element groups do

      lenkmtri = 0
      do ielgrp = 1, nelgrp

!     --- inelem := number of elements in element group

         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         call pr2026sub ( ishape, lenkmtri, subdiv, inelem )

      end do
      if ( nsplit==1 .and. .not. subdiv ) lenkmtri = 0
      call erclos ( 'pr2026' )
      end
