      subroutine pr2037 ( kmeshtri, minnode, maxnode, nelemlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 05-01-2003 Extension for prisms
!        version  1.1  date 18-12-2002 Check of shapes
!        version  1.0  date 14-02-1998
!
!   copyright (c) 1998-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subdivide a mesh consisting of tetrahedrons and/or hexahedrons
!    into four-point tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduletypes
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshtri(*), minnode, maxnode, nelemlc

!     kmeshtri       o    Kmesh part c for triangular mesh
!     maxnode        i    Maximum nodal point number in part
!     minnode        i    Minimum nodal point number in part
!     nelemlc        i    Local number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, kelmctri, j, inpelm, ielgrp

!     i          Counting variable
!     ielgrp     element group number
!     inpelm     number of points per element
!     j          Counting variable
!     kelmctri   Last position used in kmeshtri
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For all element groups do
!       inpelm := number of nodes in element group
!       inelem := number of elements in element group
!       if ( inpelm=4 ) then
!          Copy kmeshc to kmeshtri
!       else
!          Subdivide kmeshc into kmeshtri
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr2037' )

      nelemlc = 0

!     --- For all element groups do

      kelmc = 0
      kelmctri = 0
      do ielgrp = 1, nelgrp

!     --- inpelm := number of nodes in element group
!         inelem := number of elements in element group

         ishape = kmeshf(ielgrp)
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         if ( kmeshc(kelmc+1)<minnode .or. &
              kmeshc(kelmc+1)>maxnode ) then

!        --- element group does not belong to sub part

            go to 100

         end if
         if ( ishape==11 ) then

!        --- inpelm=4 Copy kmeshc to kmeshtri

            do i = 1, inelem*4
               kmeshtri(kelmctri+i) = kmeshc(kelmc+i)
            end do
            kelmctri = kelmctri+4*inelem

         else if ( ishape==18 ) then

!        --- ishape=18 Skip centroid

            do i = 0, inelem-1
               do j = 1, 4
                  kmeshtri(kelmctri+j+4*i) = kmeshc(kelmc+inpelm*i+j)
               end do
            end do
            kelmctri = kelmctri+4*inelem

         else if ( ishape==13 .or. ishape==17 ) then

!        --- inpelm=8 Subdivide elements into 6 tetrahedrons

            do i = 0, inelem-1
               do j = 1, 24
                  kmeshtri(kelmctri+j+24*i) = &
                     kmeshc(kelmc+inpelm*i+itetra(40+j))
               end do
            end do
            kelmctri = kelmctri+24*inelem

         else if ( ishape==12 .or. ishape==15 .or. ishape==16 ) then

!        --- inpelm=10, 14, 15 Subdivide elements into 8 tetrahedrons

            do i = 0, inelem-1
               do j = 1, 48
                  kmeshtri(kelmctri+j+48*i) = &
                     kmeshc(kelmc+inpelm*i+itetra(8+j))
               end do
            end do
            kelmctri = kelmctri+48*inelem

         else if ( ishape==14 ) then

!        --- inpelm=27 Subdivide elements into 48 tetrahedrons

            do i = 0, inelem-1
               do j = 1, 192
                  kmeshtri(kelmctri+j+192*i) = &
                     kmeshc(kelmc+inpelm*i+itetra(88+j))
               end do
            end do
            kelmctri = kelmctri+192*inelem

         else if ( ishape==21 ) then

!        --- inpelm=6 Subdivide elements into 3 tetrahedrons

            do i = 0, inelem-1
               do j = 1, 12
                  kmeshtri(kelmctri+j+12*i) = &
                     kmeshc(kelmc+inpelm*i+itetra(280+j))
               end do
            end do
            kelmctri = kelmctri+12*inelem

         else if ( ishape>18 ) then

!        --- Shape number not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if
100      kelmc = kelmc+inpelm*inelem

      end do
      nelemlc = kelmctri/4

1000  call erclos ( 'pr2037' )

      end
