      subroutine prgather
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 20-08-2016 use sepmoduleinput
!        version  2.0  date 30-01-2016 New parameter list
!        version  1.2  date 09-05-2012 Extra debug
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine SEPPS1 performs the post processing of SEPRAN files
!    SEPPS1 is called by program SEPPOST
!    It uses the output written by subroutines OUTPUT or OUTTIM into the
!    files sepcomp.inf and sepcomp.out
!   The file sepcomp.inf is filled as follows:
!          First records array IINPUT as output from subroutine PROBDF
!          IINPUT is written by subroutine SEPSTR
!          Next records:
!          Rec. 1:  idirec, itype
!          Rec. 2:  numvec
!          Rec. 3...: Sequence numbers of arrays that have been
!                     written to file sepcomp.out
!          with
!              idirec:  indication if the file sepcomp.out is unformatted
!                       (1) or formatted (0)
!              itype:   type of solution that is written (1)
!              numvec:  number of vectors that have been written to the
!                       file sepcomp.out
!   The arrays in the file sepcomp.out are written by the subroutine prinsl
!   which allows for formatted arrays only at this moment
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmodulemeshinf
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      integer ispace(500)

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, niter, initer(10), ivec, iseqlc, isubr, iter, &
              lastpart
      logical filusr

!     filusr         Indication if arrays IUSER and USER have been filled
!                    explictly in the call (true) or that they must be found
!                    from IBUFFR (false)
!     i              Counting variable
!     initer         Iteration numbers
!     iseqlc           Sequence number
!     isubr          Indicates the type of calling subroutine
!                    Possible values
!                    1:  CREATE
!     iter           Iteration sequence number
!     ivec           Sequence number of vector of special structure
!     j              Counting variable
!     lastpart       Number of solution vectors in all time steps read from
!                    a sepcomp_par.xxx file
!     niter          Iteration sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     FINISH         Stop SEPRAN
!     INIFIL         Open files
!     INIRMF         Remove file if it exists
!     PRGATHERREAD   Reads and compiles sepcomp_par files
!     PROUTP         Performs the actual write actions for OUTPUT
!     READALFAKE     A part of readal routine
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     WRITPROBIINP   Write arrays iinput and rinput corresponding to kprob to
!                    file sepcomp.out
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data initer/10*0/
! ======================================================================
!
!     --- Initialize all arrays

      isol = 0
      kmesh2d = 0
      kprob2d = 0
      kmesh2d(1:maxmesh) = 0
      kmesh => kmesh2d(1)
      kprob => kprob2d(1)
      isolut3d = 0
      meshnr = 1
      isolut2d => isolut3d(:,1)
      irefwr = 6
      if ( kmesh<=0 .or. kmesh>lastmesh ) kmesh = 0
      nextfreekm = 1
      iseqkm = kmesh
      if ( iseqkm==0 ) call sepcrmeshinfo ( kmesh )
      km(iseqkm)%kmesh = 0

!     --- Initialize array names

      do i = 1,26
         names(i) (1:8) = 'vector v'
         names(i) (11:30) = '                    '
         write( names(i) (9:10), 90 ) i-1
90       format(i2)
      end do

!     --- Start the program, read the mesh, the problem definition and the
!         solution arrays

      ispace(1) = 1498
      call prgatherread ( ispace, time, niter, lastpart )
      if ( ierror/=0 ) go to 1000

!     --- Write array iinput to file sepcomp.out

      call inirmf ( 74 )
      call inifil ( 74 )
      call writprobiinp

!     --- Fake reading of the input file as is doe by readal
!         This is necessary in order to call proutp
!         It creates the default input for proutp

      call readalfake ( 1 )

      call sepfillkmesh(kmesh)

      iseqlc   = 1
      ivec   = 1
      isubr = 3
      filusr = .false.
      do i = 1, lastpart/numvec

!     --- New time step set initer

         do j = 1, 10
            initer(j) = 0
         end do
         initer(iseqlc) = initer(iseqlc) + 1
         iter = initer(iseqlc)

!        --- Write the arrays into sepcomp.out

         call proutp ( isubr, time(i), lastpart+(i-1)*numvec+1, &
                       iseqlc, iter, filusr, ivec )
      end do  ! i = 1, lastpart/numvec

1000  write(irefwr,*) 'end of gather'

      call erclos ( 'prgather')
      call erclos ( 'sepcombineoutexe' )
      call finish(0)
      end subroutine prgather
