      subroutine prgeomtriangle ( coor, xh, yh, kmeshtri, dphidx, &
                                  dphidy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute geometrical quantities of triangle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshtri(3)
      double precision coor(2,*), dphidx(3), dphidy(3), xh(3), yh(3)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!                         Contains coordinates of mesh in which interpolation
!                         must take place
!     dphidx         o    Contains the x-derivatives of the basis functions
!     dphidy         o    Contains the y-derivatives of the basis functions
!     kmeshtri       i    Contains 3 nodes of the triangle
!     xh             o    x-coordinates of vertices
!     yh             o    y-coordinates of vertices
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision delta

!     delta          jacobian of element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prgeomtriangle' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prgeomtriangle'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      xh = coor(1,kmeshtri)
      yh = coor(2,kmeshtri)
      dphidy(1) = xh(3) - xh(2)
      dphidy(3) = xh(2) - xh(1)
      dphidx(1) = yh(2) - yh(3)
      dphidx(3) = yh(1) - yh(2)
      delta = dphidx(3)*dphidy(1)-dphidx(1)*dphidy(3)
      dphidy(1) = dphidy(1) / delta
      dphidy(3) = dphidy(3) / delta
      dphidy(2) = -dphidy(1) - dphidy(3)
      dphidx(1) = dphidx(1) / delta
      dphidx(3) = dphidx(3) / delta
      dphidx(2) = -dphidx(1) - dphidx(3)

1000  call erclos ( 'prgeomtriangle' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prgeomtriangle'

      end if  ! ( debug )

      end
