      subroutine princapvec ( inppri, nelectrodes, usol, nunkp, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 09-06-2010 Change of layout
!        version  1.1  date 28-07-2001 Extra option iformat
!        version  1.0  date 29-10-1999
!
!   copyright (c) 1999-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print values of capcacity vector (real case)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     capacity
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inppri(*), nelectrodes, nunkp
      double precision usol(*)
      character *(*) text

!     inppri         i    Integer array in which the user indicates which
!                         quantities must be printed by subroutine PRINBN
!                         The entries of INPPRI must be filled as follows
!                         1:  Indicates calling subroutine:
!                             1: PRINVC
!                             2: PRINCR
!                         2:  ICHOIS, indicates the type of print
!                             Possibilities:
!                             0:   The nodal point number is printed followed by
!                                  the values in the nodal point
!                             1:   The nodal point number is printed followed by
!                                  the co-ordinates and then the values in the
!                                  nodal point
!                         3:  IDIM, indicates if userpoints (idim=1), curves
!                             (idim=2) or surfaces (idim=3) must be printed.
!                             The user point, curve or surface numbers must be
!                             stored in array ICURVS
!                         4:  IQUANTITY, indicates which quantity must be
!                             printed
!                             Possible values:
!                             0:  The contents of the vector are printed as
!                                 stored in the vector corresponding to IVECTR
!                                 i.e. without transformation.
!                             1:  The normal component of a vector along the
!                                 curve or surface is printed.
!                             2:  The tangential component(s) of a vector along
!                                 the curve or surface is printed.
!                         5:  NUMBER, indicates which degree of freedom in each
!                             point must be printed. This parameter must be seen
!                             in connection with IQUANTITY.
!                             Possible values:
!                             0:  If iquantity=0, all degrees of freedom are
!                                 printed in the nodes defined by icurvs, &
!                                 if iquantity>0, the vector from which the
!                                 normal or tangential components must be
!                                 computed is supposed to be a n-dimensional
!                                 vector with first degree of freedom 1, &
!                                 second degree of freedom 2 and third degree of
!                                 freedom 3. In this case it is necessary
!                                 that IDIM = NDIM (dimension of space)
!                             i>0: If iquantity=0, the degree of freedom i is
!                                 printed
!                                 if iquantity>0, the vector from which the
!                                 normal or tangential components must be
!                                 computed is supposed to be a n-dimensional
!                                 vector with first degree of freedom i, &
!                                 second degree of freedom i+1 and third
!                                 degree of freedom i+2. In this case it is
!                                 necessary that IDIM = NDIM
!                                 (dimension of space)
!                         6:  ISEQ, indicates the sequence in which the
!                             co-ordinates along the surface must be ordered
!                             in case idim=3
!                             Possible values:
!                             1:  increasing x, for fixed x increasing y, &
!                                 for fixed y increasing z
!                             2:  increasing y, for fixed y increasing z, &
!                                 for fixed z increasing x
!                             3:  increasing z, for fixed z increasing x, &
!                                 for fixed x increasing y
!                         7:  IREGION, indicates if the complete region must be
!                             printed (0) or that the printing is restricted to
!                             a sub region, defined by the parameters xmin, xmax
!                             ymin, ymax, zmin, zmax in the comon block CPLOT.
!                             Possible values:
!                             0:  The complete region is printed
!                             1:  The region is restricted to xmin<=x<=xmax, &
!                                 ymin<=y<=ymax, zmin<=z<=zmax
!                            -1:  The region is restricted to x=xmin
!                            -2:  The region is restricted to y=ymin
!                            -3:  The region is restricted to z=zmin
!                            -11: The region is restricted to x>=xmin
!                            -12: The region is restricted to y>=ymin
!                            -13: The region is restricted to z>=zmin
!                            -21: The region is restricted to x<=xmin
!                            -22: The region is restricted to y<=ymin
!                            -23: The region is restricted to z<=zmin
!                         8:  IHEADER, indicates if the header must be printed
!                             (0) or not (1)
!                         9:  ICOOR, type of co-ordinate system
!                             1: Cartesian (x,y)
!                             3: Polar (r, phi)
!                         10: IFORMAT  Format used for printing
!                             1: e13.5
!                             2: e16.8
!     nelectrodes    i    Number of electrodes
!     nunkp          i    Maximum number of unknowns in a point
!     text           i    String variable containing some text
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount, number, j, k
      logical heading

!     heading        If true a heading must be printed
!     i              Counting variable
!     icount         Counter
!     j              Counting variable
!     k              Counting variable
!     number         See array inppri
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    Print of the vector
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Extract constants from inppri
!    Write heading
!    print values
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Extract constants from inppri

      number = inppri(5)
      heading = inppri(8)==0

!     --- Write heading

      if ( heading ) then
         write(irefwr,100) text
100      format(/' vector ',a/)
      else
         write(irefwr,*)
      end if
      write(irefwr,110)
110   format ( 'electrodes       values' )

      icount = 0
      if ( number==0 ) then

!     --- number>0: all unknowns

         do i = 1, nelectrodes
            do j = i+1, nelectrodes
               write(irefwr,120) i, j, (usol(icount+k),k=1,nunkp)
120            format ( 2i5,':',ss,30(1pe13.5e2) )
               icount = icount+nunkp
            end do
         end do

      else

!     --- number>0: only 1 unknown

         do i = 1, nelectrodes
            do j = i+1, nelectrodes
               write(irefwr,120) i, j, usol(icount+number)
               icount = icount+nunkp
            end do
         end do

      end if

      end
