      subroutine pringrid2d ( nx, ny, istart, kmeshac, xmin, xmax, &
                              ymin, ymax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print part of kmesh ac corresponding to 1 subpart of mesh (2d)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, istart, kmeshac(*)
      double precision xmin, xmax, ymin, ymax

!     istart         i    Starting position in kmeshac
!     kmeshac        i    Integer array containing the information of the
!                         subdivision of the mesh in cells
!                         This part refers to parts 2 and 3
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     xmax           i    Maximum x-value
!     xmin           i    Minimum x-value
!     ymax           i    Maximum y-value
!     ymin           i    Minimum y-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icell, jcell, indk, istartpart2, npoint, i
      double precision distx, disty

!     distx          Distance in x-direction
!     disty          Distance in y-direction
!     i              Counting variable
!     icell          Cell number of cell under consideration
!     indk           k-index
!     istartpart2    First position used at part 2
!     jcell          Cell number of cell under consideration
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pringrid2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pringrid2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      distx = (xmax-xmin)/nx
      disty = (ymax-ymin)/ny

      do icell = 1, nx
         do jcell = 1, ny
            indk = (jcell-1)*nx+icell
            istartpart2 = kmeshac(istart+indk-1)
            npoint = kmeshac(istart+indk)-istartpart2
            if ( npoint>0 ) then

!           --- npoint>0

               write(irefwr,110) icell, jcell, &
                  xmin+(icell-1)*distx,xmin+icell*distx, &
                  ymin+(jcell-1)*disty,ymin+jcell*disty, &
                  (kmeshac(istartpart2+i-1), i = 1, npoint )
110            format ( 'Nodes in cell (', i3,',',i3, ')', &
                        2('(', 2(ss,1pe10.2e2),')')/ &
                        (10i8))
            end if
         end do
      end do

1000  call erclos ( 'pringrid2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pringrid2d'

      end if  ! ( debug )

      end
