      subroutine pringrid3d ( nx, ny, nz, istart, kmeshac, xmin, xmax, &
                              ymin, ymax, zmin, zmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-09-2012 Extra debug
!        version  1.0  date 07-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print part of kmesh ac corresponding to 1 subpart of mesh (2d)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, istart, kmeshac(*)
      double precision xmin, xmax, ymin, ymax, zmin, zmax

!     istart         i    Starting position in kmeshac
!     kmeshac        i    Integer array containing the information of the
!                         subdivision of the mesh in cells
!                         This part refers to parts 2 and 3
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     xmax           i    Maximum x-value
!     xmin           i    Minimum x-value
!     ymax           i    Maximum y-value
!     ymin           i    Minimum y-value
!     zmax           i    Maximum z-value
!     zmin           i    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icell, jcell, kcell, indk, istartpart2, npoint, i
      double precision distx, disty, distz

!     distx          Distance in x-direction
!     disty          Distance in y-direction
!     distz          Distance in z-direction
!     i              Counting variable
!     icell          Cell number of cell under consideration
!     indk           k-index
!     istartpart2    First position used at part 2
!     jcell          Cell number of cell under consideration
!     kcell          Cell number of cell under consideration
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pringrid3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pringrid3d'
         write(irefwr,1) 'istart', istart
         write(irefwr,1) 'nx, ny, nz', nx, ny, nz
         write(irefwr,2) 'xmin, ymin, zmin', xmin, ymin, zmin
  1      format ( a, 1x, (10i8) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      distx = (xmax-xmin)/nx
      disty = (ymax-ymin)/ny
      distz = (zmax-zmin)/nz

      if ( debug ) &
         write(irefwr,2) 'distx, disty, distz', distx, disty, distz

      do icell = 1, nx
         do jcell = 1, ny
            do kcell = 1, nz
               indk = (kcell-1)*nx*ny + (jcell-1)*nx+icell
               istartpart2 = kmeshac(istart+indk-1)
               npoint = kmeshac(istart+indk)-istartpart2

               if ( npoint>0 ) then

!              --- npoint>0

                  write(irefwr,120) icell, jcell, kcell, &
                     xmin+(icell-1)*distx,xmin+icell*distx, &
                     ymin+(jcell-1)*disty,ymin+jcell*disty, &
                     zmin+(kcell-1)*distz,zmin+kcell*distz, &
                     (kmeshac(istartpart2+i-1), i = 1, npoint )
120               format ( 'Nodes in cell (',i2,',',i2,',',i2,')'/ &
                        3('(', 2(ss,1pe10.2e2),')')/ &
                           (10i8))
               end if
            end do
         end do
      end do

1000  call erclos ( 'pringrid3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pringrid3d'

      end if  ! ( debug )

      end
