      subroutine pringridinfo ( nx, ny, grid, gridinfo, nowbnd, &
                                imxlps, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-07-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Print the contents of the arrays grid and gridinfo in a suitable form
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     print
!     grid
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, grid(0:nx,0:ny), gridinfo(*), imxlps, &
              nowbnd(imxlps,*)
      character *(*) text

!     grid           i    Work array of size at most 101 x 101 to store
!                         information about the boundary in relation with
!                         arrays gridinfo and rgridinfo
!                         The actual size is (0:nx, 0:ny) and (ix,iy) refers
!                         to the cell ( xmin+ix*dx, ymin+iy*dy) x
!                         ( xmin+(ix+1)*dx, ymin+(iy+1)*dy)
!                         if grid(ix,iy) = 0, no boundary node is stored in cell
!                         (ix,iy), if > 0 information about the boundary cells
!                         is stored in array grdinfo from position
!                         grid(ix,iy)
!     gridinfo       i    In this array the boundary points corresponding to a
!                         specific grid cell in grid are stored
!                         istart = grid(ix,iy)
!                         In gridinfo(istart) the number of boundary points of
!                         cell (ix,iy) are stored (if > 0). This is followed
!                         by 2 positions for each boundary point
!                         The pair consists of the loop sequence number in the
!                         boundary where loop refers to a closed loop and
!                         a sequence number in the loop
!     imxlps         i    maximum loops in nowbnd in paver
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     nx             i    number of cells in x direction
!     ny             i    number of cells in y direction
!     text           i    String variable containing some text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lennodes
      parameter ( lennodes = 100 )
      integer i, j, istart, k, numnodes, nodes(lennodes), ind, &
              iloop

!     i              Counting variable
!     iloop          Loop sequence number of closed boundary loop
!     ind            Sequence number along closed boundary loop
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     lennodes       Length of array nodes
!     nodes          Contains the node numbers that must be filled
!     numnodes       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pringridinfo' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pringridinfo'

      end if
      write(irefwr,*) text
      write(irefwr,*) 'Arrays grid and gridinfo'
      write(irefwr,*) 'grid_index  node numbers'

      do i = 0, nx

         do j = 0, ny
            istart = grid(i,j)

            if ( istart.gt.0 ) then

!           --- There is at least one node in the cell

               numnodes = gridinfo(istart)
               if ( numnodes.gt.lennodes ) then

!              --- Error nodes too large

                  call errint ( numnodes, 1 )
                  call errint ( lennodes, 2 )
                  call errchr ( 'nodes', 1 )
                  call errsub ( 550, 2, 0, 1 )
                  go to 1000

               end if

               do k = 1, numnodes
                  iloop = gridinfo(istart+2*(k-1)+1)
                  ind = gridinfo(istart+2*(k-1)+2)
                  nodes(k) = nowbnd(iloop,ind+1)
               end do
               write(irefwr,100) i, j, (nodes(k),k=1,numnodes)
100            format ( 2i5, 4x, 100i6 )

            end if  ! ( istart.gt.0 )

         end do  ! j = 0, ny

      end do  ! i = 0, nx

1000  call erclos ( 'pringridinfo' )
      if ( debug ) write(irefwr,*) 'End pringridinfo'

      end
