      subroutine prinintmat1 ( storeinfo )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print array storeinfo
!
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
!     parallel
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: storeinfo(*)

!     storeinfo      i    Array to store all information that is received
!                         from other blocks
!                         See subrutine incommatpar3
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iblock, istart, ninterf

!     iblock         Block sequence number
!     istart         First position used at a part
!     ninterf        Number of unknowns in interf (interface)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0005         Print INTMAT corresponding to boundary conditions
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinintmat1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinintmat1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,*) 'information stored in storeinfo'

      do iblock = 1, iacnodes

         istart = storeinfo(iblock)
         if ( istart>0 ) then

!        --- istart>0, information from iblock

            write(irefwr,1) 'information from block', iblock
            ninterf = storeinfo(istart)
            istart = istart+1
            call prinin ( storeinfo(istart), ninterf, &
                          'global unknown numbers of interface' )
            istart = istart+ninterf
            write(irefwr,*) 'matrix S_if'
            write(irefwr,*) 'row numbers refer to local unknowns'
            write(irefwr,*) 'column numbers refer to global unknowns'
            call pr0005 ( ninterf, storeinfo(istart), &
                          storeinfo(istart+ninterf+1), 0 )

         end if  ! ( istart>0 )

      end do  ! iblock = 1, iacnodes

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinintmat1'

      end if  ! ( debug )
      call erclos ( 'prinintmat1' )

      end subroutine prinintmat1
