      subroutine priniobgrid ( iobgrid, nx, ny, iobgrid1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array iobgrid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, iobgrid(0:nx-1,0:ny-1), iobgrid1(*)

!     iobgrid        i    position i,j contains:
!                         -1:  cell inside obstacle
!                          0:  cell outside obstacle
!                         >0:  cell contains part of the obstacle
!                         The value contains a starting address in iobgrid1
!     iobgrid1       i    Must be used in connection with iobgrid
!                         iobgrid contains the starting addresses for each cell
!                         i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, istart, n

!     i              Counting variable
!     istart         Starting address in iobgrid1
!     j              Counting variable
!     n              number of curve parts in cell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'priniobgrid' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from priniobgrid'
         call prinin1 ( iobgrid, ny, nx, 'iobgrid' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,*) 'edges in cells'

      do i = 0, nx-1

         do j = 0, ny-1

            if ( iobgrid(i,j)>0 ) then

               istart = iobgrid(i,j)

               if ( debug ) write(irefwr,1) 'i, j, istart', i, j, istart

               n = iobgrid1(istart)

               write(irefwr,100) i, j, iobgrid1(istart+1:istart+n)
100            format ( 'cell (', i3, ',', i3, '): ', 20i4 )

            else if ( iobgrid(i,j)<0 ) then

!           --- iobgrid<0, hence inside obstacle

               write(irefwr,100) i, j, iobgrid(i,j)

            end if  ! ( iobgrid(i,j)>0 )

         end do  ! j = 0, ny

      end do  ! i = 0, nx

1000  call erclos ( 'priniobgrid' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End priniobgrid'

      end if  ! ( debug )

      end
