      subroutine prinkprobv ( nsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-03-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print contents of array kprobv
!     Information about contact points
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
!     problem_definition
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfs

!     kprobvp1       i    First part of array KPROB part v
!                         Length: 3*nsurfs
!                         Contents:
!                         Pos. (i,1): number of contact points in
!                                     contact surface i
!                         Pos. (i,2): number of points in contact
!                                     surface i that make nocontact
!                         Pos. (i,3): starting address of contact points
!                                     for i^th contact surface in part 2
!     kprobvp2       i    Second part of array KPROB part v
!                         contains the nodal points in the surfaces
!                         that make contact in the sequence defined by
!                         part 1 followed by the nodes that make no contact
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ncontact, nnocontact, ipstart, k

!     i              Counting variable
!     ipstart        Starting address in array
!     k              Counting variable
!     ncontact       Number of contact points
!     nnocontact     Number of no-contact points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinkprobv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinkprobv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write ( irefwr, 340 )
340   format(' information about the contact points')
      do i = 1, nsurfs

         ncontact = kprobvp1(i,1)
         nnocontact = kprobvp1(i,2)
         ipstart = kprobvp1(i,3)

         if ( ipstart>0 ) then

            write ( irefwr, 350 ) i, ncontact, nnocontact
350         format(' contact surface nr.  ', i4/ &
                   ' number of points that make contact = ',i6/ &
                   ' number of points that make no contact = ', &
                   i6 )
            if ( ncontact>0 ) write ( irefwr, 360 ) &
               (kprobvp2(ipstart+k),k=0,ncontact-1)
360         format(' contact points  '/ (1x,10i6))
            ipstart = ipstart+ncontact
            if ( nnocontact>0 ) write ( irefwr, 370 ) &
               (kprobvp2(ipstart+k),k=0,nnocontact-1)
370         format(' no-contact points  '/ (1x,10i6))

         end if  ! ( ipstart>0 )

      end do  ! i = 1, nsurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinkprobv'

      end if  ! ( debug )
      call erclos ( 'prinkprobv' )

      end
