      subroutine prinkprobw ( nsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-03-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print contents of array kprobw
!     Information about contact elements
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
!     problem_definition
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfs

!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ncontactelm, nocontactelm, isurnr, ipstart, k

!     i              Counting variable
!     ipstart        Starting address in array
!     isurnr         Surface sequence number
!     k              Counting variable
!     ncontactelm    Number of contact elements
!     nocontactelm   Number of no-contact elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinkprobw' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinkprobw'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write ( irefwr, 380 )
380   format(' information about the contact elements')

      do i = 1, nsurfs
         ncontactelm  = kprobwp1(i,1)
         nocontactelm = kprobwp1(i,2)
         isurnr       = kprobwp1(i,3)
         ipstart      = kprobwp1(i,4)
         if ( ipstart>0 ) then
            write ( irefwr, 390 ) i, isurnr, ncontactelm, &
               nocontactelm
390         format(' contact surface  ', i4/ &
                   ' surface number = ',i6/ &
                   ' number of elements that make', &
                   '  contact = ', i6/ &
                   ' number of elements that make', &
                   ' no contact = ', i6 )

            if ( ncontactelm>0 ) write ( irefwr, 400 ) &
               (kprobwp2(k+ipstart),k=0,ncontactelm-1)
400         format(' contact elements  '/ (1x,10i6))
            ipstart = ipstart+ncontactelm
            if ( nocontactelm>0 ) write ( irefwr, 410 ) &
               (kprobwp2(k+ipstart),k=0,nocontactelm-1)
410         format(' no-contact elements  '/ (1x,10i6))
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinkprobw'

      end if  ! ( debug )
      call erclos ( 'prinkprobw' )

      end
