      subroutine prinnodeelms ( inode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 10-11-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print all elements corresponding to a node
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode

!     inode          i    Node sequence number
!                         All elements corresponding to this node are printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, nelements, i

!     i              Counting variable
!     ipstart        Starting address in array
!     nelements      Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMSHD1  Fill part of kmshd1 for one element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate arrays
!     Compute starting addresses
!     Actual printing
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinnodeelms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinnodeelms'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kelmd==0 ) call sepfillkmshd1

!     --- Actual printing

!     --- Compute starting position and number of elements

      ipstart = kmshd1(inode)
      nelements = kmshd1(inode+1)-kmshd1(inode)

!     --- Write elements

      write(irefwr,100) inode, (kmshd2(ipstart+i), i = 1, nelements)
100   format ( 'elements corresponding to node ', i8, ':' / (10i8) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinnodeelms'

      end if
      call erclos ( 'prinnodeelms' )

      end
