      subroutine prinnodeelms2 ( kmeshd, inode, npoint, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-11-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Raise iwork for all elements corresponding to a node
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshd(*), inode, npoint, iwork(*)

!     inode          i    Node sequence number
!                         All elements corresponding to this node are printed
!     iwork         i/o   integer work array
!                         iwork(i) contains the number of nodes from array
!                         nodes connected to element i
!     kmeshd         i    KMESH part d
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, nelements, i, ielem

!     i              Counting variable
!     ielem          Element number
!     ipstart        Starting address in array
!     nelements      Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute starting position and number of elements
!     Raise iwork for all elements corresponding to inode
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinnodeelms2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinnodeelms2'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Compute starting position and number of elements

      ipstart = kmeshd(inode)+npoint+1
      nelements = kmeshd(inode+1)-kmeshd(inode)

!     --- Raise iwork for all elements corresponding to inode

      do i = 1, nelements
         ielem = kmeshd(ipstart+i)
         iwork(ielem) = iwork(ielem)+1
      end do  ! i = 1, nelements

1000  call erclos ( 'prinnodeelms2' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinnodeelms2'

      end if

      end
