      subroutine prinnormal3d1 ( nsurfs, surfnodes, normal )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the array normals for all outer surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     normal_vector
!     outward_normal
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfs, surfnodes(*)
      double precision normal(3,*)

!     normal         i    Contains the normals on the surfaces in exactly the
!                         same sequence as the nodes in array surfnodes
!     nsurfs         i    number of surfaces
!     surfnodes      i    Contains the node numbers corresponding to the
!                         surfaces
!                         Pos. 1 ... nsurfs starting addresses to part 2
!                         Part2: for each surface:
!                         number of nodes followed by node numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, isurnr, ipstart, i, j, nnodes

!     i              Counting variable
!     ipstart        Starting address in array
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     j              Counting variable
!     nnodes         Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinnormal3d1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinnormal3d1'
100      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      write(irefwr,*) 'Normals corresponding to surfaces'
      istart = 0
      do isurnr = 1, nsurfs
         write(irefwr,100) 'surface', isurnr
         ipstart = surfnodes(isurnr)
         nnodes = surfnodes(ipstart)
         do i = 1, nnodes
            write(irefwr,200) surfnodes(ipstart+i), &
                              (normal(j,istart+i),j=1,3)
200         format ( i8, ':', 3d15.5 )
         end do
         istart = istart+nnodes
      end do

1000  call erclos ( 'prinnormal3d1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinnormal3d1'

      end if

      end
