      subroutine prinnowbnd ( nowbnd, nloop, imxlps, layer )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-02-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Print array nowbnd that is used in subroutine mshpav
!      This array describes the new boundary during the mesh generation
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     boundary
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imxlps, nowbnd(imxlps,*), nloop, layer

!     imxlps         i    maximum loops in nowbnd in paver
!     layer          i    Present layer sequence number
!     nloop          i    number of closed boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nbndpt, j

!     i              Counting variable
!     j              Counting variable
!     nbndpt         Number of nodal points in boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinnowbnd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinnowbnd'

      end if
      write(irefwr,*) 'Nowbnd in layer ', layer
      do i = 1, nloop
         nbndpt = nowbnd(i,1)
         write(irefwr,110) i, (nowbnd(i,j+1),j=1,min(nbndpt,10))
110      format(i7,':',10i7)
         if ( nbndpt.gt.10 ) &
            write(irefwr,120) (nowbnd(i,j+1),j=11,nbndpt)
120      format(8x,10i7)

      end do

      call erclos ( 'prinnowbnd' )
      if ( debug ) write(irefwr,*) 'End prinnowbnd'

      end
