      subroutine prinsfobst ( coor, npoint, nelem, kmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-06-2013 New definition file ref numbers
!        version  1.0  date 20-07-2001 Copied from plotsfobst
!
!   copyright (c) 2001-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Print the information of an obstacle to a file in geomview format
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     write
!     openview
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nelem, kmeshc(3,*)
      double precision coor(3,npoint)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, inodp, nnodes, icount, iref, iwork(npoint)
      character (len=80) filename
      save icount

!     filename       Name of file
!     i              Counting variable
!     icount         Counter
!     inodp          Nodal point number
!     iref           File reference number
!     iwork          integer work array of length npoint to mark the
!                    points in the obstacle (>0)
!     j              Counting variable
!     k              Counting variable
!     nnodes         Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data icount /0/
! ======================================================================
!
      call eropen( 'prinsfobst' )

!     --- Open file

      icount = icount+1
      filename = 'openview_'
      write(filename(10:12),100) icount
100   format(i3.3)
      filename(13:16) = '.off'
      iref = inisetnewref()
      open ( unit =iref, file = filename(1:16) )

!     --- Clear array iwork

      iwork = 0

!     --- Fill iwork with an 1 for all nodes that are present in the obstacle

      nnodes = 0
      do i = 1, nelem
         do j = 1, 3
            inodp = kmeshc(j,i)
            if ( iwork(inodp)==0 ) then

!           --- Nodal point is new

               iwork(inodp) = 1
               nnodes = nnodes+1

            end if

         end do
      end do

!     --- write number of nodes and elements

      write(iref,110) nnodes, nelem
110   format('OFF',/,2i6,' 0')

!     --- Set new nodal point numbers in iwork and write coordinates

      nnodes = 0
      do i = 1, npoint
         if ( iwork(i)==1 ) then

!        --- Nodal point is new

            iwork(i) = nnodes
            write(iref,120) (coor(j,i),j=1,3)
            nnodes = nnodes+1
120         format(3e15.6)

         end if
      end do

!     --- write elements

      do i = 1, nelem
         write(iref,130) (iwork(kmeshc(k,i)),k=1,3)
130      format('3 ', 3i6)
      end do

      close ( unit = iref )
      call inirmref ( iref )

      call erclos( 'prinsfobst' )
      end
