      subroutine prinsurf ( isurnr, npoint, nelem, ndim, inpelm, &
                            coor, kmeshc, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-06-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of one surface
!     It concerns co-ordinates and topology
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     write
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr, npoint, nelem, ndim, inpelm, kmeshc(inpelm,*)
      double precision coor(ndim,*)
      character *(*) text

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     inpelm         i    number of points per element
!     isurnr         i    Surface sequence number
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     ndim           i    Dimension of the space.
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
!     text           i    String variable containing the heading to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      write(irefwr,*) text
      write(irefwr,100) isurnr
100   format ( 'Co-ordinates of surface ', i3)
      do i = 1, npoint
         write(irefwr,110) i, (coor(j,i), j = 1, ndim )
110      format( i6, 3 (ss, 1pe14.6e2) )
      end do
      write(irefwr,200) isurnr
200   format ( 'Node numbers of surface ', i3)
      do i = 1, nelem
         write(irefwr,210) i, (kmeshc(j,i), j = 1, inpelm )
210      format( i6, 5x, 27i6 )
      end do

      end
