      subroutine prinsurfcoor ( isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 23-12-2011 new call to ersurfsnod
!        version  1.1  date 28-12-2010 new call to ersurfsnod
!        version  1.0  date 25-03-2009
!
!   copyright (c) 2009-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the coordinates of surface isurnr in R^3
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     surface
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr

!     isurnr         i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nnodes, isurfaces(1)
      integer, allocatable :: nodes(:)

!     isurfaces      Array containing the surfaces from which information
!                    must be found
!     nnodes         Number of nodes
!     nodes          Contains the node numbers that must be printed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINSURFCOOR1  Print the coordinates of surface isurnr in R^3
!     SEPSORTCOOR    Sort nodal points with respect to increasing co-ordinates
!     SEPSURFSNOD    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinsurfcoor' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinsurfcoor'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate(nodes(npoint),stat = error)
      if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Find nodes in surface

      isurfaces(1) = isurnr
      call sepsurfsnod ( 1, nodes, nnodes, isurfaces )
      if ( ierror/=0 ) go to 1000

!     --- Sort nodes

      call sepsortcoor ( 3, nnodes, nodes )
      if ( ierror/=0 ) go to 1000

!     --- Print coordinates

      write(irefwr,1) 'Coordinates of surface', isurnr
      call prinsurfcoor1 ( coor, nodes, nnodes )

      deallocate ( nodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinsurfcoor'

      end if  ! ( debug )
      call erclos ( 'prinsurfcoor' )

      end
