      subroutine printcoorlist ( coor, ndim, list, nlist, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-08-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the coordinates stored in the list of nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, list(*), nlist
      double precision coor(ndim,*)
      character *(*) text

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     list           i    List of nodal points to be printed
!     ndim           i    Dimension of the space.
!     nlist          i    Length of array list
!     text           i    String variable containing the text to be printed
!                         as heading
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printcoorlist' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printcoorlist'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      write(irefwr,*) text
      do i = 1, nlist
         write(irefwr,100) list(i), (coor(j,list(i)),j=1,ndim)
100      format(i6,' :  ', 3d15.6 )
      end do  ! i = 1, nlist

1000  call erclos ( 'printcoorlist' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printcoorlist'

      end if

      end
