      subroutine printcurvs ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-01-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print a set of curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart

!     inpsep         i    The integer input read is stored in array INPSEP
!                         see subroutine askprincurv
!     istart         i    First position used at a part
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numcurvs, nparts, nnodes
      integer, allocatable :: nodes(:), ipoint(:)
      character (len=9) text

!     i              Counting variable
!     ipoint         Array of length nparts+1 containing the starting
!                    addresses in array inodes of the various parts
!                    the boundary consists of.
!                    ipoint(1) = 1, &
!                    number of nodes at part i = ipoint(i+1)-ipoint(i)
!     nnodes         Number of nodes in nodes
!     nodes          Contains the node numbers of the curve
!     nparts         number of parts the boundary consists of
!     numcurvs       number of curves in icurves
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printcurvs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printcurvs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- allocate work arrays

      allocate ( nodes(npoint), ipoint(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

      numcurvs = inpsep(2)  ! number of curves to be printed

      text(1:6) = 'curve '

!     --- lopp over curves

      do i = 1, numcurvs

!     --- Get node numbers of curve

         call sepfillnodes ( 2, 1, inpsep(i+2), &
                             nodes, nnodes, ipoint, nparts )

         write(text(7:9),100) inpsep(i+2)  ! put curve number in text
100      format(i3)

         call prinin ( nodes, nnodes, text )  ! print curve

      end do  ! i = 1, numcurvs

      istart = istart+3+numcurvs  ! raise istart

      deallocate ( nodes, ipoint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printcurvs'

      end if  ! ( debug )
      call erclos ( 'printcurvs' )

      end
