      subroutine printhis ( nhist, nodhist, t, ncopeq, isol, first )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 30-11-2012 Long integers
!        version  2.0  date 13-04-1999 Extra parameters ibuffr/buffer
!        version  1.1  date 24-01-1998 Correction non-renumbered arrays
!
!   copyright (c) 1997-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Print the time history in a number of nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_history
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nhist, nodhist(nhist), ncopeq, isol(ncopeq)
      double precision t
      logical first

!     first      i    If true it concerns the first time step
!     isol       i    Standard SEPRAN array containing information of the
!                     solution array
!     ncopeq     i    Number of coupled equations
!     nhist      i    Number of points in which a time history is required
!     nodhist    i    Contains nodal points in which the time history is
!                     required
!     t          i    actual time
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, ieq, nodp, istart

!     i          Counting variable
!     ieq        Equation sequence number
!     istart     First position used at a part
!     k          Counting variable
!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   If ( first ) then
!      write heading
!   For ieq := 1 (1) ncopeq do
!      Activate array
!      For i := 1 (1) nhist do
!         nodp := nodhist(i)
!         Print values in nodp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printhis' )
      if ( first ) then

!     --- Write heading

         write(irefwr,100) nhist, ncopeq
100      format(' Time history for ',i2, ' nodes in ', i2,' equations'/ &
                ' For each time step the sequence is:'/ &
                ' Loop over equations '/ &
                ' Per equation: loop over nodes'/ &
                ' Per node: loop over unknowns'// &
                ' Time  eqn   node    values' )

      end if
      do ieq = 1, ncopeq

!     --- Activate array

         call sepactsolbf2 ( isol(ieq) )

         usoltemp => ks(isol(ieq))%sol
         do i = 1, nhist
            nodp = nodhist(i)
            if ( indprfi>0 ) then
               istart = kprobfi(nodp)
               nunkpi = kprobfi(nodp+1)-istart
            else
               istart = (nodp-1)*nunkpi
            end if
            if ( nunkpi>0 ) then
               write(irefwr,200) t, ieq, nodp, &
                  (usoltemp(istart+k),k=1,nunkpi)
200            format ( f10.4, i4, i7, 40d14.6 )

            end if
         end do
      end do
      call erclos ( 'printhis' )
      end
