      subroutine printzerolev ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 16-04-2014 Extension with append
!        version  1.1  date 20-06-2013 New definition file ref numbers
!        version  1.0  date 03-07-2012
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the edges (2d) or faces (3d) of the zero level set
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduletime
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart

!     inpsep         i    The integer input read is stored in array INPSEP
!                         The first position refers to this subroutine
!                         See subroutine askprizerolev
!     istart        i/o   Starting address in array INPVEC in tosepcomsub
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icount, irefwrsav, nchar
      character (len=80) filename

!     filename       Name of file
!     icount         Counter
!     irefwrsav      Help variable to store the value of irefwr
!     nchar          Number of characters in a word
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICLOSEFILE   Close file and reset irefwr
!     INIOPENFILE    Open serial file and set irefwr
!     PRININ         print 1d integer array
!     PRINTZEROLEV2  Print the edges (2d) or faces (3d) of the zero level set
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data icount /0/
! ======================================================================
!
      call eropen ( 'printzerolev' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printzerolev'
         call prinin ( inpsep, 4, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inpsep(2)>0 ) then

!     --- inpsep(2)>0, write to file

         if ( t<=t0 ) then

!        --- t = t0, set icount to 0

            icount = 0

         else

!        --- t>t0, raise icount

            icount = icount+1

         end if  ! ( t<=t0 )

!        --- Create file name

         nchar = inpsep(3)
         filename = texts(inpsep(2))(1:nchar)
         write(filename(nchar+1:nchar+5),100) icount
100      format ( '_',i4.4 )

         call iniopenfile ( 0, irefwrsav, inpsep(4), filename )

      end if  ! ( inpsep(2)>0 )

      if ( ndim>=2 ) then

!     --- ndim = 2 or 3

!         --- Actual subroutine

         call printzerolev2 ( inpsep, irefwr )

      end if  ! ( ndim==2 )

      if ( inpsep(2)>0 ) call iniclosefile ( irefwrsav )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printzerolev'

      end if  ! ( debug )
      istart = istart+5
      call erclos ( 'printzerolev' )

      end subroutine printzerolev
