      subroutine prmachn1 ( npoint, veloc, enth, machno, ipost, rpost )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-03-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Actual body of prmachno.f: compute the machnumber in all points of
!    the mesh, from the given velocity and enthalpy
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          npoint, ipost(*)
      double precision veloc(2,npoint), enth(npoint), machno(npoint), &
                       rpost(*)

!     enth       i    Array containing the enthalpy at every cell
!     ipost      i    Integer array containing information required for
!                     seppost
!                     See the Programmers Guide for a description
!     machno     o    Array containing the mach number at every nodal point
!     npoint     i    Number of nodal points
!     rpost      i    Real array containing information required for
!                     seppost
!                     See the Programmers Guide for a description
!     veloc      i    Array containing the velocity
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer          i, idimchoice
      double precision machref, spgamma

!     i          Counting variable
!     idimchoice Dimensionless (0) or dimensionful computation (1)
!     machref    Reference Mach number
!     spgamma    Specific heat ratio
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The mach number is computed from, with ux, uy, h dimensionless, &
!
!     M = u/a = machref*sqrt((ux^2+uy^2) / h)
!
!     When ux, uy, h are dimensionful, the Mach number is computed from
!
!     M = u/a = sqrt((ux^2+uy^2)/((gamma-1)h))
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      idimchoice = ipost(6)

!     --- Dimensionless input quantities

      if ( idimchoice.eq.0 ) then
         machref = rpost(2)

         do i = 1, npoint
            machno(i) = &
            machref*sqrt( (veloc(1,i)**2+veloc(2,i)**2)/enth(i) )
         end do

      else

!     --- Dimensionful input quantities

         spgamma = rpost(1)

         do i = 1, npoint
            machno(i) = sqrt( (veloc(1,i)**2+veloc(2,i)**2)/( &
                        (spgamma-1.0d0)*enth(i)) )
         end do

      end if

      end
