      subroutine proutpmain ( iavs, isubract, ivectr, nproblc, numveclc, &
                              filusr, icounter, iseqlc, iter, nameavs, &
                              namedx, nametec, one_file, timelc, &
                              nameparaview, separate_elgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.2  date 24-12-2014 New definition clinsl
!        version  4.1  date 16-04-2014 New call to prtecplot
!        version  4.0  date 09-04-2014 Put all variables in one file
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Main body subroutine with respect to subroutine PROUTP
!     This subroutine performs the actual writing and if necessary
!     computes derived quantities
! **********************************************************************
!
!                       KEYWORDS
!
!     output
!     write
!     tecplot
!     avs
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulepost
      use sepmodulecons
      use sepmoduleuser
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleinput
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iavs, isubract, ivectr, nproblc, numveclc, icounter, &
              iseqlc, iter, nameavs, namedx, &
              one_file, nametec, nameparaview
      double precision timelc

!     filusr         i    Indication if arrays IUSER and USER have been filled
!                         explictly in the call (true) or that they must be
!                         found from IBUFFR (false)
!     iavs           i    Indicates which kind of output must be created.
!                         This parameter consists of three parts:
!                         TOSEPPOST, TOAVS and TODX according to
!                         TOSEPPOST+2*TOAVS+4*TODX+8*TOTECPLOT+16**TOparaview
!                         Possible values for TOSEPPOST
!                         1:  Create output for SEPPOST only in the
!                             form of file sepcomp.out
!                         0:  Do not create output for SEPPOST
!                         Possible values for TOAVS
!                         1:  Create output for AVS only in the form
!                             of a file sepavs_xxx.yyy
!                             With xxx a time sequence number of the
!                             shape 001, 002 etc
!                             and yyy a vector sequence number of the
!                             shape 001, 002 etc
!                         0:  Do not create output for avs
!                         Possible values for TODX
!                         1:  Create output for OPEN_DX only in the form
!                             of a file sepodx_xxx.yyy
!                             With xxx a time sequence number of the
!                             shape 001, 002 etc
!                             and yyy a vector sequence number of the
!                             shape 001, 002 etc
!                         0:  Do not create output for open dx
!                         Possible values for TOparaview
!                         1:  Create output for paraview (avs format) in the
!                             form of a file sepavs_xxx.inp
!                             With xxx a time sequence number of the
!                             shape 001, 002 etc
!                             All vectors are stored in the same file
!                         0:  Do not create output for paraview
!     icounter       i    Integer time sequence number
!     iseqlc         i    Sequence number for array to be written
!     isubract       i    Parameter isubr_act, see proutpsub1
!     iter           i    Iteration sequence number
!     ivectr         i    sequence number of first vector to be used
!                         in the array isol2d(.,j) for output purposes.
!                         IVECTR-1 refers to the vector number in SEPPOST
!     nameavs        i    Sequence number of name of avs file in array
!                         file_names
!     namedx         i    Sequence number of name of open dx file in array
!                         file_names
!     nameparaview   i    Sequence number of name of paraview file in array
!                         file_names
!     nametec        i    Sequence number of name of tecplot file in array
!                         file_names
!     nproblc        i    Number of various problems
!     numveclc       i    Number of vectors that have been filled
!     one_file       i    Indicates if all time steps must be stored in one file
!                         (1) or not (0)
!     separate_elgrp i    If true, each element group is written to a different
!                         output file
!     timelc         i    Actual time
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, lenius, lenusr, ivec, nch, iproblc, itypvc
      logical seppst, avs, opendx, tecplot, paraview
      logical filusr, separate_elgrp, filled
      integer, target :: itemp

!     avs            Indicates if avs output must be generated (true)
!                    or not (false)
!     filusr         ?
!     i              Counting variable
!     iproblc        Actual problem number
!     itemp          Index for temporary array
!     ivec           Sequence number of vector of special structure
!     lenius         Computed length of array IUSER
!     lenusr         Computed length of array USER
!     nch            Number of characters in a string
!     opendx         Indicates if open_dx output must be generated (true)
!                    or not (false)
!     paraview       Indicates if paraview must be generated (true)
!                    or not (false)
!     separate_elgrp If true, each element group is written to a different
!                    output file
!     seppst         Indicates if seppost output must be generated (true)
!                    or not
!     tecplot        Indicates if tecplot output must be generated (true)
!                    or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDERV         Performs subsequent calls of subroutine DERIVA
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRCOPYVECS     Copy the contents of vector iold into vector inew
!     PRIAVS         Write solution array or array of special structure to file
!                    sepavs_xxx.yyy (AVS file)
!     PROUTPSUB      Write second part of information to file sepcomp.out
!     PRTECPLOT      Write solution array or array of special structure to file
!                    septec.yyy (Tecplot file)
!     SEP2ODX        Write solution arrays to OPEN_DX format file
!     SEP2PARAVIEW   Write solution arrays to PARAVIEW format file
!     SEP2PARAVIEWEL Write solution arrays and mesh for each element group to a
!                    set of files in avs format such that it can be read by
!                    paraview
!     SEPPRINOUTP    Write solution vectors to sepcomp.out
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     822   Vector to be written by OUTPUT or OUTTIM has not been filled.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'proutpmain' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from proutpmain'
         write(irefwr,1) 'iavs, ivectr, nproblc', iavs, ivectr, nproblc
         write(irefwr,1) 'ifoutp, isubract', ifoutp, isubract
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      seppst = mod(iavs,2)==1
      avs    = mod(iavs/2,2)==1
      opendx = mod(iavs/4,2)==1
      tecplot = mod(iavs/8,2)==1
      paraview = mod(iavs/16,2)==1

      if ( debug ) then
         write(irefwr,*) 'seppst, avs, opendx, tecplot, paraview', &
                          seppst, avs, opendx, tecplot, paraview
      end if  ! ( debug )

      if ( seppst .and. ifoutp==0 ) then

!     --- Write second part of information to file 74
!         First numvec and lenipost, lenrpost

         call proutpsub ( numvec, ipost, names, rpost )

      end if
      if ( debug ) write(irefwr,*) 'after proutpsub'
      if ( ifoutp==0 .and. isubract==3 ) ifoutp = 1

!     --- write solution array to file sepcomp.out

      do iproblc = ivectr, ivectr+nproblc-1
         filled = isol2d(iproblc)>0 .and. isol2d(iproblc)<=lastsol
         if ( filled ) filled = solarr(isol2d(iproblc))>0
         if ( filled ) then
            itypvc = ks(isol2d(iproblc))%typevec
            if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=116 .and. &
                 itypvc/=127 .and. itypvc/=129 .and. itypvc/=801 ) &
               filled = .false.
         end if  ! ( filled )
         if ( .not. filled ) then

!        --- Array isol2d(iproblc) has not been filled

            nch = index(names(iproblc),' ')-1
            if ( nch<=0 ) nch = len(names(iproblc))
            call errchr ( names(iproblc)(1:nch), 1 )
            call errint ( iproblc, 1 )
            call errint ( iproblc-ivectr+1, 2 )
            call errwar ( 822, 2, 0, 1 )

         else

!        --- Array has been filled
!            First check if it must be transformed (i.e. deleting Lagrangian
!            multipliers )

            iprob = ks(isol2d(iproblc))%iprob
            iseqkp = kprob
            kprobpart => kp(iseqkp)%kprobsub(iprob)
            kprobloc => kprobpart%kprobloc
            if ( kprobloc(54)==1 .and. ks(isol2d(iproblc))%typevec==110 ) then

!           --- There are Lagrangian multipliers that must be deleted
!               Copy solution vector to intermediate vector of special
!               structure ndtvec

               ivec = kprobloc(6)
               itemp = 0
               isolut => itemp
               call prcopyvecs ( isol2d(iproblc), isolut, ivec )

!              --- write vector

               if ( seppst ) &
                  call sepprinoutp ( isolut, iproblc+iseqlc-1, iter, timelc )
               if ( avs ) call priavs ( icounter, iproblc, timelc, nameavs )

!              --- Delete temporary vector

               call seprmsolinfo ( itemp )

            else

!           --- Standard case

               isolut => isol2d(iproblc)
               if ( seppst ) &
                  call sepprinoutp ( isolut, iproblc+iseqlc-1, iter, timelc )
               if ( avs ) call priavs ( icounter, iproblc, timelc, nameavs )

            end if

         end if

      end do
      if ( opendx ) &
         call sep2odx ( isol2d, timelc, &
                        icounter, iseqlc, ivectr, ivectr+nproblc-1, namedx )

      if ( paraview ) then

!     --- Paraview output file (avs format)

         if ( separate_elgrp ) then

!        --- All element groups are stored separately

            call sep2paraviewel ( isol2d, icounter, &
                                  iseqlc, ivectr+nproblc-1, nameparaview )

         else

!        --- All element groups are stored as a whole

            call sep2paraview ( isol2d, icounter, &
                                iseqlc, ivectr+nproblc-1, nameparaview )

         end if  ! ( separate_elgrp )

      end if  ! ( paraview )

      if ( tecplot ) then
         call prtecplot ( isol2d, icounter, nametec, one_file, nproblc )
      end if  ! ( tecplot )

!     --- Calls of DERIVA

      if ( numveclc>0 ) then

!     --- numveclc>0, compute starting addresses for iuser and user

         iseqlc = nproblc
         iprob = ks(isol2d(iinputout(21)))%iprob

         if ( .not. filusr .and. associated(iuser1(iprob)%iuser) ) then

!        --- not filusr: iuser and user may be stored in IBUFFR
!            iuser1(iprob)>0, IUSER and USER are filled

            iuser => iuser1(iprob)%iuser
            user => iuser1(iprob)%user

            if ( debug ) write(irefwr,*) 'before erderv/1'
            call erderv ( numveclc, iinputout(16), &
                          iavs, iseqlc, icounter, timelc, iter, nameavs )

         else

!        --- filusr: iuser and user are explicitly given

            if ( debug ) write(irefwr,*) 'before erderv/2'
            call erderv ( numveclc, iinputout(16), iavs, &
                          iseqlc, icounter, timelc, iter, nameavs )

         end if
         if ( debug ) write(irefwr,*) 'after erderv'

      end if

!     --- Final statements, write output_sequence_number, iteration_number
!                           and timelc and 7 dummies

      ifoutp = ifoutp + 1

      if ( isubract==1 .and. associated(iinputout) ) deallocate ( iinputout )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End proutpmain'

      end if
      call erclos ( 'proutpmain' )

      end subroutine proutpmain
