      subroutine proutpnoinput ( isubr, timelc, iter )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 21-11-2012 Long integers
!        version  1.2  date 16-04-2007 Extension with paraview
!        version  1.1  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     writes the solution sas stored in the solution arrays
!     corresponding to ISOL to the file sepcomp.out
!     No input is required
! **********************************************************************
!
!                       KEYWORDS
!
!     output
!     read
!     write
!     tecplot
!     avs
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulemain
      use sepmoduleplot
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, iter
      double precision timelc

!    isubr     i   Indicates the type of calling subroutine
!                  isubr consists of 2 parts isubr_act and nvect according to
!                  isubr = isubr_act + 10*nvect
!                  Possible values for isubr_act
!                  1:  OUTPUT
!                  2:  OUTTIM
!                  3:  OUTSOL
!                  4:  OUTPOS
!                  5:  TUDFINVOL
!                  nvect is only used if isubr_act=5 and it indicates the number
!                  of vectors to be written
!    iter      i   Iteration number
!    timelc    i   Actual time corresponding to this call of PROUTP
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numveclc, mmiinput, nprob, icounter, iavs, iseq, &
              isubract, nameavs, namedx, ivectr, iseqnr, &
              one_file, nametec, nameparaview
      logical filusr
      save nprob, numveclc, icounter, iavs

!     filusr         Indication if arrays IUSER and USER have been filled
!                    explictly in the call (true) or that they must be found
!                    from IBUFFR (false)
!     iavs           Indicates which kind of output must be created.
!                    This parameter consists of three parts:
!                    TOSEPPOST, TOAVS and TODX according to
!                    TOSEPPOST+2*TOAVS+4*TODX
!                    Possible values for TOSEPPOST
!                    1:  Create output for SEPPOST only in the
!                        form of file sepcomp.out
!                    0:  Do not create output for SEPPOST
!                    Possible values for TOAVS
!                    1:  Create output for AVS only in the form
!                        of a file sepavs_xxx.yyy
!                        With xxx a time sequence number of the
!                        shape 001, 002 etc
!                        and yyy a vector sequence number of the
!                        shape 001, 002 etc
!                    0:  Do not create output for avs
!                    Possible values for TODX
!                    1:  Create output for OPEN_DX only in the form
!                        of a file sepodx_xxx.yyy
!                        With xxx a time sequence number of the
!                        shape 001, 002 etc
!                        and yyy a vector sequence number of the
!                        shape 001, 002 etc
!                    0:  Do not create output for open dx
!     icounter       Integer time sequence number
!     iseq           Sequence number for array to be written
!     iseqnr         Sequence number with respect to the storage in array ISEPIN
!     isubract       Parameter isubr_act, see isubr
!     ivectr         sequence number of first vector to be used
!                    in the array isolut2d(.,j) for output purposes.
!                    IVECTR-1 refers to the vector number in SEPPOST
!     mmiinput       Help variable to store the reference number of array IINPUT
!                    in array INFOR in common block CARRAY
!     nameavs        Sequence number of name of avs file in array
!                    file_names
!     namedx         Sequence number of name of open dx file in array
!                    file_names
!     nameparaview   Sequence number of name of paraview file in array
!                    file_names
!     nametec        Sequence number of name of tecplot file in array
!                    file_names
!     nprob          Number of various problems
!     numveclc       Number of derivative vectors that have been written to file
!                    sepcomp.out
!     one_file       Indicates if all time steps must be stored in one file (1)
!                    or not (0)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INSTOP         Stop the program
!     PROUTPMAIN     Main body subroutine with respect to subroutine PROUTP
!     PROUTPSUB1     Set some parameters depending on the input list
! **********************************************************************
!
!                       I/O
!
!   The solution is written to file "iref74"
!   The arrays in the file sepcomp.out are written by the subroutine prinsl
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'proutpnoinput' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from proutpnoinput'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set some parameters depending on the input list

      call proutpsub1 ( icounter, isubract, &
                        iseq, isubr, numveclc, iavs, iseqnr )

!     --- set other parameters

      iavs     = 1
      iseq     = 0
      numveclc = 0
      nameavs  = 0
      namedx   = 0
      nametec  = 0
      nameparaview = 0
      nprob    = numvec
      one_file = 0
      ivectr   = 1
      filusr = .false.
      mmiinput = 0

!     --- Main body of subroutine

      isol2d => isolut2d
      call proutpmain ( iavs, isubract, ivectr, &
                        nprob, numveclc, mmiinput, &
                        filusr, icounter, iseq, iter, nameavs, &
                        namedx, nametec, one_file, timelc, &
                        nameparaview, .false. )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End proutpnoinput'

      end if
      call erclos ( 'proutpnoinput' )

      end subroutine proutpnoinput
