      subroutine proutpsub ( numvec, ipost, names, rpost )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-09-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write second part of information to file sepcomp.out
!
! **********************************************************************
!
!                       KEYWORDS
!
!     output
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numvec, ipost(*)
      double precision rpost(*)
      character (len=*) names(*)

!     ipost          i    Integer array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
!     names          i    Array of vector names to be written to file
!     numvec         i    Number of vectors stored in isol
!     rpost          i    Real array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      character (len=80) textloc

!     i              Counting variable
!     textloc        String to store a local text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'proutpsub' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from proutpsub'

      end if
      if ( ierror/=0 ) go to 1000

      if ( iref74>0 ) then

!     --- iref74>0, formatted

         write( iref74, 200 ) &
                '//General_info:   numvec, lenipost, lenrpost '
200      format(a)
         write( iref74, 210 ) numvec, (ipost(i),i=1,2)
210      format(10i9)

!        --- names of arrays

         if ( numvec>0 ) then
            write( iref74, 200 ) '//Names_of_vectors'
            do i = 1, numvec
               write( iref74, 220 ) names(i)
220            format(a)
            end do
         end if

!        --- ipost and rpost

         if ( ipost(1)>0 ) then
            write( iref74, 200 ) '//Ipost'
            write( iref74, 210 ) (ipost(i),i=1,ipost(1))
         end if
         if ( ipost(2)>0 ) then
            write( iref74, 200 ) '//Rpost'
            write( iref74, 230 ) (rpost(i),i=1,ipost(2))
230         format(5d16.8)
         end if

      else

!     --- iref74<0, unformatted

         textloc = '//General_info:   numvec, lenipost, lenrpost '
         write( -iref74 ) textloc
         write( -iref74 ) numvec, (ipost(i),i=1,2)

         if ( numvec>0 ) then
            textloc = '//Names_of_vectors'
            write( -iref74 ) textloc
            do i = 1, numvec
               write( -iref74 ) names(i)
            end do
         end if

!        --- ipost and rpost

         if ( ipost(1)>0 ) then
            textloc = '//Ipost'
            write( -iref74 ) textloc
            write( -iref74 ) (ipost(i),i=1,ipost(1))
         end if
         if ( ipost(2)>0 ) then
            textloc = '//Rpost'
            write( -iref74 ) textloc
            write( -iref74 ) (rpost(i),i=1,ipost(2))
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End proutpsub'

      end if
      call erclos ( 'proutpsub' )

      end
