      subroutine prsegi (slevel, coor, fval, knodp, iout, NIOUT, &
                 xcint, fcint)
!************************************************************************
!******
!       version 1.4   date   22-08-1991 Zdenek: no dimension, cdc, ce
!****** Version 1.3
!******
!****** Programmer : Jos van Kan
!******
!****** Date 18 may 1988
!******
!****** Last Change : 19 may 1988
!******
!************************************************************************
!******
!****** SEGINT takes the output from ellint (that is all elements that
!****** are intersected by a straight line y = slevel) and calculates
!****** all intersection points per element with that line and also
!****** the linearly interpolated function values. So all internal
!****** intersection points are double and the boundary points of any
!****** segment are single. This seems a bit wasteful, but in fact it is
!****** a cheap way (you don't have to check boundaries and all that sort
!****** of rot) to identify separate segments and boundaries. This, however
!****** is the task of gathsg.
!****** coor, fval and knodp have their usual meaning. (input)
!****** iout and NIOUT are input parameters (output from ellint), xcint and
!****** fcint contain the interpolated xcoordinates and functionvalues
!****** respectively. The number of points in xcint and fcint is just
!****** 2*NIOUT.
!******
!************************************************************************
!******
!****** Dictionary      i/o      meaning
!******
!****** slevel           i       y value with which elements are intersected
!******
!****** coor             i       nodal coordinates
!******
!****** fval             i       nodal functionvalues
!******
!****** knodp            i       nodal point numbers of elements
!******
!****** iout             i       array of elements intersecting y = slevel
!******                          (see ellint)
!******
!****** NIOUT                    number of elements in iout.
!******
!****** xcint            o       array of interpolated x coordinates (unsorted)
!******
!****** fcint            o       array of interpolated function values
!******
!************************************************************************
!******
!****** Local Variables
!******
!****** i            counter
!******
!****** ielem        element number of element under consideration
!******
!****** c1o, c2o     x-coordinates of intersection
!******
!****** f1o, f2o     interpolated functionvalues at intersection
!******
!************************************************************************
      implicit none
      double precision slevel, coor(*), fval(*), xcint(*), fcint(*)
      integer knodp(*), iout(*), NIOUT
      double precision c1o, c2o, f1o, f2o
      integer i, ielem
!******
!****** For  all elements that intersect the line y = slevel ...
!******
      do 100 i = 1, NIOUT
!******
!****** get element number
!******
         ielem = iout (i)
!******
!****** Calculate the intersection points and interpolated function
!****** values
!******
         call prcutl (ielem, slevel, coor, fval, knodp, &
                      c1o, c2o, f1o, f2o)
!******
!****** ... and store them
!******
         xcint (2 * i - 1) = c1o
         xcint (2 * i) = c2o
         fcint (2 * i - 1) = f1o
         fcint (2 * i) = f2o
100   continue
!******  end do
      return
      end
