      subroutine prsepblock1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-01-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make mesh input file for a set of blocks
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     write
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numlayers

!     numlayers      Number of blocks that are created
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSEPBLOCKCONS Write part constant to output file
!     PRSEPBLOCKCURS Write part curves to output file
!     PRSEPBLOCKELGS Write part element groups to output file
!     PRSEPBLOCKEND  Write last part to mesh input file
!     PRSEPBLOCKPNTS Write part points to output file
!     PRSEPBLOCKSURS Write part surfaces to output file
!     PRSEPBLOCKVOLS Write part volumes to output file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsepblock1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsepblock1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Write block constants

      call prsepblockcons ( numlayers )

!     --- Write block points

      call prsepblockpnts ( numlayers )

!     --- Write block curves

      call prsepblockcurs ( numlayers )

!     --- Write block surfaces

      call prsepblocksurs ( numlayers )

!     --- Write block volumes

      call prsepblockvols ( numlayers )

!     --- Write block element groups

      call prsepblockelgs ( numlayers )

!     --- Write end block

      call prsepblockend

1000  call erclos ( 'prsepblock1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsepblock1'

      end if  ! ( debug )

      end
