      subroutine prsepblock2
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-02-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make problem input file for a set of blocks
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     write
!     read
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numlayers

!     numlayers      Number of blocks that are created
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getint

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETINT         Get the value of an integer in the input block constants
!     PRSEPBLOCKCOEF Write part coefficients to problem file
!     PRSEPBLOCKENDP Write last part problem to problem file
!     PRSEPBLOCKPROB Write part problem to problem file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsepblock2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsepblock2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numlayers = getint('num_layers')
      if ( numlayers==0 ) numlayers=1

!     --- Write block constants

      write(irefwr,*) '# block.prb'
      write(irefwr,*) 'set output none'
      write(irefwr,*) "include 'block.input'"
      write(irefwr,*) 'debug_parameters'
      write(irefwr,*) '   write_matlab_mat'
      write(irefwr,*) 'end'

!     --- Write block problem

      call prsepblockprob ( numlayers )

!     --- Write block coefficients

      call prsepblockcoef ( numlayers )

!     --- Write final block

      call prsepblockendp

1000  call erclos ( 'prsepblock2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsepblock2'

      end if  ! ( debug )

      end
