      subroutine prsw02 ( inodes, ipoint, nparts, iregion, coor, ndim, &
                          idim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-05-1995 Complete new subroutine
!
!   copyright (c) 1985-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Reduce series of nodes stored in set inodes to those node
!    lying in a given region
! **********************************************************************
!
!                       KEYWORDS
!
!     region
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*), ipoint(*), nparts, iregion, ndim, idim
      double precision coor(ndim,*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     idim    i    If idim=2, nparts contains the number of parts and array
!                  ipoint has been filled, otherwise nparts contains the number
!                  of nodes and ipoint is not used.
!     inodes i/o   Array containing the list of nodal points that must be
!                  considered
!                  At input it contains all nodes that must be checked, &
!                  at output only the nodal points that are within the
!                  required region are stored sequenctially
!     ipoint i/o   Array of length nparts+1 containing the starting addresses
!                  in array inodes of the various parts the boundary consists
!                  of.
!                  ipoint(1) = 1, &
!                  number of nodes at part i = ipoint(i+1)-ipoint(i)
!     iregion  i   indicates if the complete region must be printed
!                  (0) or that the printing is restricted to a sub region, &
!                  defined by the parameters xmin, xmax, ymin, ymax, zmin, &
!                  zmax in the comon block CPLOT.
!                  Possible values:
!                   0:  The complete region is printed
!                   1:  The region is restricted to xmin<=x<=xmax, &
!                       ymin<=y<=ymax, zmin<=z<=zmax
!                  -1:  The region is restricted to x=xmin
!                  -2:  The region is restricted to y=ymin
!                  -3:  The region is restricted to z=zmin
!                  -11: The region is restricted to x>=xmin
!                  -12: The region is restricted to y>=ymin
!                  -13: The region is restricted to z>=zmin
!                  -21: The region is restricted to x<=xmin
!                  -22: The region is restricted to y<=ymin
!                  -23: The region is restricted to z<=zmin
!     ndim    i    Dimension of the space.
!     nparts  i    number of parts the boundary consists of
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, inew, inodp, ipart, j, iend, numpart
      logical found
      double precision x(2,3), eps

!     eps     Local accuracy
!     found   If true the node belongs to the region
!     i       Counting variable
!     iend    Last position used at a part
!     inew    New address in array inodes
!     inodp   Nodal point number
!     ipart   Sequence number of part
!     istart  First position used at a part
!     j       Counting variable
!     numpart Number of parts
!     x       Array containing the bounds of the region according to:
!             x(1,1) <= x <= x(2,1)
!             x(1,2) <= y <= x(2,2)
!             x(1,3) <= z <= x(2,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Set start values
!   Fill array x with bounds for co-ordinates
!   For ipart := 1 (1) nparts do
!      For all nodes inode in part ipart
!         if ( inode in region ) then
!            Fill inode in inodes
!      Set ipoint(ipart)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set start values

      istart = 1
      inew   = 1
      eps    = 1d-5

!     --- Fill array x with bounds for co-ordinates

      do 100 j = 1, ndim
         x(1,j) = -rinfin
         x(2,j) =  rinfin
100   continue
      if ( iregion.eq.1 ) then
         x(1,1) = xmin
         x(1,2) = ymin
         x(1,3) = zmin
         x(2,1) = xmax
         x(2,2) = ymax
         x(2,3) = zmax
      else if ( iregion.eq.-1 ) then
         x(1,1) = xmin-eps
         x(2,1) = xmax+eps
      else if ( iregion.eq.-2 ) then
         x(1,2) = ymin-eps
         x(2,2) = ymax+eps
      else if ( iregion.eq.-3 ) then
         x(1,3) = zmin-eps
         x(2,3) = zmax+eps
      else if ( iregion.eq.-11 ) then
         x(1,1) = xmin
      else if ( iregion.eq.-12 ) then
         x(1,2) = ymin
      else if ( iregion.eq.-13 ) then
         x(1,3) = zmin
      else if ( iregion.eq.-21 ) then
         x(2,1) = xmax
      else if ( iregion.eq.-22 ) then
         x(2,2) = ymax
      else if ( iregion.eq.-23 ) then
         x(2,3) = zmax
      end if

!     --- Loop over parts

      if ( idim.eq.2 ) then
         numpart = nparts
      else
         numpart = 1
      end if
      do 200 ipart = 1, numpart

!     --- Loop over nodes in part ipart

         if ( idim.eq.2 ) then
            iend = ipoint(ipart+1)-1
         else
            iend = nparts
         end if
         do 150 i = istart, iend
            inodp = inodes(i)
            found = .true.
            do 130 j = 1, ndim
               found = found .and. coor(j,inodp).ge.x(1,j) .and. &
                       coor(j,inodp).le.x(2,j)
130         continue
            if ( found ) then

!           --- Node is in region

               inodes(inew) = inodp
               inew = inew + 1
            end if
150      continue
         istart = iend + 1
         if ( idim.eq.2 ) then
            ipoint(ipart+1) = inew
         else
            nparts = inew-1
         end if
200   continue
      end
