      subroutine prsw06c6 ( iformat, nnodes, usol, kprobf, heading )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     complex case
!     Special case, do not print coordinates
!     Do not print node numbers
!     Non constant number of unknowns per point
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, iformat, kprobf(*)
      complex * 16 usol(*)
      logical heading

!     heading        i    If true a heading must be printed
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     nnodes         i    Number of nodes in inodes
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inunkp, istart

!     i              Counting variable
!     inunkp         Number of degrees of freedom in first point
!     istart         First position used at a part
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!     Print of vector
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06c6' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06c6'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( heading ) write(irefwr,270)
270   format('   values'/)

      do i = 1, nnodes
         istart = kprobf(i)
         inunkp = kprobf(i+1)-istart
         if ( iformat==1 ) then
            write(irefwr,280) &
               (usol(j+istart), j=1,inunkp)
         else if ( iformat==2 ) then
            write(irefwr,281) &
               (usol(j+istart), j=1,inunkp)
         else if ( iformat==3 ) then
            write(irefwr,282) &
               (usol(j+istart), j=1,inunkp)
         else
            write(irefwr,283) &
               (usol(j+istart), j=1,inunkp)
         end if
      end do
280   format(1x,ss,2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
281   format(1x,ss,2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
282   format(1x,ss,99('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
283   format(1x,ss,99('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))

1000  call erclos ( 'prsw06c6' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06c6'

      end if  ! ( debug )

      end
