      subroutine prsw06sup04 ( inodes, nnodes, usol, coor, &
                               iformat, heading, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-12-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     Nodes with a zero solution are skipped
!     real case
!     Special case ichoice_print=1, only one unknown
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*), nnodes, iformat, ndim
      double precision usol(*), coor(ndim,*)
      logical heading

!     coor           i    array of length ndim x npoint containing the
!                         coordinates of the nodal points with respect to the
!                         global numbering
!     heading        i    If true a heading must be printed
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in inodes
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, icount, ind(nnodes)

!     i              Counting variable
!     icount         Number of nodes to be printed
!     ind            Integer array of length icount containing the
!                    indices of the nodes to be printed
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06sup04' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06sup04'

      end if
      if ( ierror/=0 ) go to 1000

!     --- only one unknown
!         First store the indices of non-zero entries

      icount = 0     ! number of non-zero entries
      do i = 1, nnodes

         if ( abs(usol(i))>0d0 ) then

!        --- usol(i) # 0, store sequence number in ind

            icount = icount+1
            ind(icount) = i

         end if  ! ( abs(usol(i)>0d0 )

      end do  ! i = 1, nnodes

      if ( ndim==1 ) then
         if ( heading ) write(irefwr,330)
         if ( iformat==1 .or. iformat==3 ) then
            write(irefwr,340) (inodes(ind(i)), &
                               coor(1,inodes(ind(i))), &
                               usol(ind(i)), i=1,icount)
         else
            write(irefwr,341) (inodes(ind(i)), &
                               coor(1,inodes(ind(i))), &
                               usol(ind(i)), i=1,icount)
         end if
      else if ( ndim==2 ) then
         if ( heading ) write(irefwr,350)
         if ( iformat==1 .or. iformat==3 ) then
            write(irefwr,360) (inodes(ind(i)), &
                              (coor(j,inodes(ind(i))),j=1,2), &
                               usol(ind(i)), i=1,icount)
         else
            write(irefwr,361) (inodes(ind(i)), &
                              (coor(j,inodes(ind(i))),j=1,2), &
                               usol(ind(i)), i=1,icount)
         end if
      else
         if ( heading ) write(irefwr,370)
         if ( iformat==1 .or. iformat==3 ) then
            write(irefwr,380) (inodes(ind(i)), &
                              (coor(j,inodes(ind(i))),j=1,3), &
                               usol(ind(i)), i=1,icount)
         else
            write(irefwr,381) (inodes(ind(i)), &
                              (coor(j,inodes(ind(i))),j=1,3), &
                               usol(ind(i)), i=1,icount)
         end if
      end if

330   format(' node       x           value')
340   format((1x,i9,ss,1pe13.5e2,1pe13.5e2))
341   format((1x,i9,ss,1pe16.8e2,1pe16.8e2))
350   format(' node       x            y         value')
360   format((1x,i9,ss,1pe13.5e2,1pe13.5e2,1pe13.5e2))
361   format((1x,i9,ss,1pe16.8e2,1pe16.8e2,1pe16.8e2))
370   format(' node       x            y           z      ')
380   format((1x,i9,ss,4(1pe13.5e2)))
381   format((1x,i9,ss,4(1pe16.8e2)))

1000  call erclos ( 'prsw06sup04' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06sup04'

      end if

      end
