      subroutine prtomatlab ( solution, npoint, nunkp, iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-06-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write solution stored in columns per degree of freedom
!     to Matlab input file
! **********************************************************************
!
!                       KEYWORDS
!
!     matlab
!     output
!     vector
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nunkp, iref
      double precision solution(nunkp,npoint)

!     iref           i    File reference number
!     npoint         i    Number of nodal points
!     nunkp          i    Maximum number of unknowns in a point
!     solution       i    Array of size npoint x nunkp containing
!                         the solution to be written
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtomatlab' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtomatlab'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, npoint
         write(iref,200) (solution(j,i), j=1,nunkp)
200      format ( 1x,ss,80(1pe13.5e2) )
      end do  ! i = 1, npoint

1000  call erclos ( 'prtomatlab' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtomatlab'

      end if  ! ( debug )

      end
