      subroutine sepcopycoorus ( iincop, usercoor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coordinates from array kmesh into user array usercoor
!     Old subroutine copycoorus
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     user_vector
!     sepran_vector
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iincop(*)
      double precision usercoor(*)

!     iincop       i    User input array
!                       In this array the user defines which coordinates
!                       the user requires from the mesh. The output may be
!                       restricted to a user point, a curve, a surface or to
!                       the complete mesh
!                       IINCOP must be filled as follows:
!                       Pos. 1   Number of entries that are filled by the user
!                                All other positions are defined by default
!                                values
!                                So if IINCOP(1) = 0 or 1 only defaults are used
!                       Pos. 2   ICHOIS: Defines what type of nodes must be
!                                copied
!                                Possible values:
!                                0: copy all coordinates into ROUTCP
!     usercoor     o    User array in which the cooordinates are  stored
!                       USERCOOR is a variable length array which implies that
!                       the first position must be filled by the user with the
!                       declared length
!                       It is advised to set the third position equal to 0
!                       The  always cooordinates are stored from position 6!
!                       The sequence is x-coordinate of the first point, &
!                       followed by y-coordinate and z-co-ordinate.
!                       Then the same for the second point and so on.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate coor array
!     compute starting address
!     Actual copying
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopycoorus' )

!     --- Actual copying

      length = size(coor)
      call prinlg ( 'usercoor', -length-5, usercoor, iincop )
      call dcopy ( length, coor, 1, usercoor(6), 1 )
      call erclos ( 'sepcopycoorus' )
      end subroutine sepcopycoorus
