      subroutine sepcopytoprec5 ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-02-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy symmetric matrix into preconditioning matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRECCOPY       Copy matrix amat into matrix premat, where premat may have
!                    a different structure.
!                    Both matrices are stored as compact matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopytoprec5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopytoprec5'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iseqpre/=iseqintmat ) then

!     --- the structure of the preconditioning
!         matrix and the standard matrix is different

         call preccopy ( A%neq, A%rows, A%Mrows, A%S, &
                         A%MD, A%cols, A%Mcols )

      else

!     --- both matrices have the same structure
!         Copy matrix into premat

         A%MD = A%S

      end if  ! ( iseqpre/=iseqintmat )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopytoprec5'

      end if  ! ( debug )
      call erclos ( 'sepcopytoprec5' )

      end subroutine sepcopytoprec5
