      subroutine sepderaver
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate values in global vertices in region by using the mean value
!     of two neighboring nodes
!     A global vertex is defined as a point that is lying in one element only
!     Remark: this subroutine is only programmed under the following
!             restrictions:
!           - one unknown per point
!           - only 4-point quadrilaterals in the mesh
!     Old subroutine to0111
! **********************************************************************
!
!                       KEYWORDS
!
!     average
!     derivative
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulebuild
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indlc(0:5), i, j, ikelmc, indexi

!     i              Counting variable
!     ikelmc         position in kmesh part c
!     indexi         Index of vertex in element
!     indlc          Integer array containing the cyclic permutation of 1,2,3,4
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRWAR    Print warnings
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1624   Weighting in glocal vertices has not yet been implemented for this
!           case
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Test on restrictions
!    Fill array indlc
!    For all nodes i do
!       If ( weight(i)=1 ) then
!          ielem = element node i is lying in
!          Interpolate solution in node i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderaver' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderaver'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( npelm/=4 .or. kelma/=0 .or. indprfo/=0 ) then

!     --- This case has not yet been implemented, so skip subroutine

         call errwar ( 1624, 0, 0, 0 )
         go to 1000
      end if

!     --- Fill array indlc

      do i = 1, npelm
         indlc(i) = i
      end do
      indlc(0) = npelm
      indlc(npelm+1) = 1

!     --- Loop over nodes

      do i = 1, npoint
         if ( abs(weight(i)-1d0)<sqreps ) then
            ielem = kmshd2(kmshd1(i)+1)
            ikelmc = (ielem-1)*npelm

            do j = 1, npelm
               if ( i==kmeshc(ikelmc+j) ) then
                  indexi = j
                  go to 120
               end if
            end do  ! j = 1, npelm

120         vectout(i) = 0.5d0*(vectout(kmeshc(ikelmc+indlc(indexi+1)))+ &
                             vectout(kmeshc(ikelmc+indlc(indexi-1))))
         end if
      end do  ! i = 1, npoint

1000  call erclos ( 'sepderaver' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderaver'

      end if  ! ( debug )

      end subroutine sepderaver
