      subroutine sepderelgrp
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize some parameters for one element group
!     Help subroutine for deriv
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleelm
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: larray, liarry, lenindex3, lenarray

!     larray         length of array coeffarray in element subroutines
!     lenarray       Length of array coeffarr
!     lenindex3      Length of array index3
!     liarry         length of the integer array icoeffarray
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMGRINF       Fill general information for a specific element group
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPDERELINIT   initialize some parameters for one element group in
!                    subroutine deriv
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderelgrp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderelgrp'
         write(irefwr,1) 'icouno', icouno
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

!     --- Initialize some parameters for one element group

      call sepderelinit

      if ( itype<=0 ) go to 1000  ! skip element group

!     --- allocate arrays for element subroutines
!         In this case icount is correct

      liarry = 1
      larray = 1

!     --- Compute some parameters depending on the type number

      call elmgrinf ( larray, liarry, kmeshm, ndim )
      lenarray = larray+7425
      allocate ( indoldsols(1)%index3(icount), stat=error )
      if ( error/=0 ) call eralloc ( error, inpelm, 'index4' )
      if ( ierror/=0 ) go to 1000

      error = 0
      allocate ( coeffarr(lenarray), stat=error )
      if ( error/=0 )  call eralloc ( error, lenarray, 'coeffarr' )
      allocate ( icoeffarr(liarry), stat=error )
      if ( error/=0 )  call eralloc ( error, liarry, 'icoeffarr' )

      allocate ( index5(1:icouno), stat=error )
      if ( error/=0 )  call eralloc ( error, icouno, 'index5' )
      allocate ( index2(icount), stat=error )
      if ( error/=0 )  call eralloc ( error, icount, 'index2' )
      allocate ( index4(max(1,numold),inpelm+1), stat=error )
      if ( error/=0 )  call eralloc ( error, icount, 'index4' )
      if ( ierror/=0 ) go to 1000

      indoldsols(1)%index3 => index2

!     --- allocate arrays for element subroutines

      allocate ( elemvc1(icouno*icompl) )
      allocate ( elemwg(icouno) )
      elemvc1 = 0d0
      elemwg = 1d0

      lenindex3 = max(nindex+inpelm, inpelm*nunkp)
      if ( debug ) write(irefwr,1) 'lenindex3', lenindex3, numold
      allocate ( index3(max(1,numold),lenindex3), stat=error )
      if ( error/=0 ) call eralloc ( error, lenindex3, 'index3' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderelgrp'

      end if  ! ( debug )
      call erclos ( 'sepderelgrp' )

      end subroutine sepderelgrp
