      subroutine sepderindex5 ( ikelmc, ipkproba, ioutvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array index5 with sequence numbers of the element
!     with respect to the output vector
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ikelmc, ipkproba, ioutvc

!     ikelmc         i    Last position in KMESH part c that has been used
!     ioutvc         i    Standard SEPRAN array containing information
!                         of tghe output vector
!     ipkproba       i    start address in kprob part a for element group
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPDERINDEX    fill index2 with positions of degrees of freedom in element
!                    ielem as stored in vector denoted by ivectr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderindex5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderindex5'
         write(irefwr,1) 'iseqi1, iseqi2', iseqi1, iseqi2
         call prinin ( E%islold(iseqi1), 5, 'E%islold' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ks(ioutvc)%typevec==115 ) then
         kprobatmp => kprobe(ipkproba:,ivctst)
      else
         if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr
      end if  ! ( ks(ioutvc)%typevec==115 )

      allocate ( indexh(inpelm+1) )
      posinp = posout
      index25 => index5
      kprobflc => kprobfo
      call sepderindex ( ioutvc, nunkpo, indprfo )
      deallocate ( indexh )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderindex5'

      end if  ! ( debug )
      call erclos ( 'sepderindex5' )

      end subroutine sepderindex5
