      subroutine sepderintout ( ioutvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array ioutvc with information that is used to compute icouno
!     Used by subroutine sepderintegral
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmodulebuild
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: ioutvc

!     ioutvc         i    Standard SEPRAN array to be used only to store
!                         information to compute icouno
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderintout' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderintout'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcrsolinfo ( ioutvc, 0, 0 )

      if ( ichtst==1 .or. ichtst==4 .and. ivctst==0 .or. &
           ichtst==8 ) then

!     --- ichtst = 1, 8 output vector is of type 110

         itypvo = 110
         ks(ioutvc)%ivec = 0

      else if ( ichtst==2 .or. ichtst==4 ) then

!     --- ichtst = 2, 4 output vector is of type 115

         itypvo = 115
         ks(ioutvc)%ivec = abs(ivctst)

      else if ( ichtst==3 ) then

!     --- ichtst = 3, output vector is of type 116

         itypvo = 116
         ks(ioutvc)%ivec = abs(ivctst)
         ks(ioutvc)%n = abs(ivctst)

      else if ( ichtst==6 ) then

!     --- ichtst = 6, output vector is of type 126

         itypvo = 126
         ks(ioutvc)%ivec = abs(ivctst)

      else if ( ichtst==7 ) then

!     --- ichtst = 7, output vector is of type 129

         itypvo = 129
         ks(ioutvc)%ivec = abs(ivctst)

      else if ( ichtst==9 ) then

!     --- ichtst = 9, output vector is of type 136

         itypvo = 136
         ks(ioutvc)%ivec = abs(ivctst)

      end if
      ks(ioutvc)%iprob = iprob
!      if ( ks(ioutvc)%typevec/=itypvo ) ioutvc(1) = 0

      compli = ks(isolut2d(iseqi1))%compl==1

      if ( ivctst<0 ) then
         complo = .true.
      else if ( ivctst==0 ) then
         complo = compli
      else
         complo = .false.
      end if

      if ( complo ) then
         ks(ioutvc)%compl = 1
         icompl = 2
      else
         ks(ioutvc)%compl = 0
         icompl = 1
      end if
      ks(ioutvc)%typevec = itypvo

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderintout'

      end if  ! ( debug )
      call erclos ( 'sepderintout' )

      end subroutine sepderintout
