      subroutine sepderintoutvc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create output vector for sepcompderint and
!     Activate input vector
!     Output vector is of type 136 (integers define per element)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulemain
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: ioutvc

!     ioutvc         Pointer referring to SMF%ioutvc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCRSOLINFO   Create a new vector in ks
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderintoutvc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderintoutvc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ioutvc => SMF%ioutvc

!     --- Activate input vector

      call sepactsolbf2 ( isolut2d(iseqi1) )

!     --- Create space for output vector

      nunkpo = iinder(7)
      nusolo = nunkpo*nelem

      length = nusolo

      call sepcrsolinfo ( ioutvc, 0, length )
      ks(ioutvc)%typevec = 136
      ks(ioutvc)%compl = 0
      ks(ioutvc)%nphys = nunkpo
      ks(ioutvc)%iprob = iprob
      ks(ioutvc)%nusol = nusolo
      ks(ioutvc)%n = ivctst
      ks(ioutvc)%ivec = ivctst

      if ( iinder(9)==0 ) then
         isoltemp => ks(ioutvc)%isol
         isoltemp = 0
      end if  ! ( iinder(9)==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderintoutvc'

      end if  ! ( debug )
      call erclos ( 'sepderintoutvc' )

      end subroutine sepderintoutvc
