      subroutine sepderoutvc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create output vector for sepcompder and
!     Activate input vector
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulemain
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ih, i, ivec
      integer, pointer :: ioutvc
      logical :: new

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ioutvc         Pointer referring to SMF%ioutvc
!     ivec           Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCREA129I     Create integer part of vector of type 129
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCRSOLINFO   Create a new vector in ks
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderoutvc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderoutvc'
         write(irefwr,1) 'iseqi1, iseqi2, ichderiv', iseqi1, iseqi2, ichderiv
         write(irefwr,1) 'ivctst, iprob', ivctst, iprob
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ioutvc => SMF%ioutvc
      if ( debug )   write(irefwr,1) 'ioutvc', ioutvc

!     --- Activate input vector

      call sepactsolbf2 ( isolut2d(iseqi1) )

      compli = ks(isolut2d(iseqi1))%compl==1
      if ( ivctst<0 ) then
         complo = .true.
         icompl = 1
      else if ( ivctst==0 ) then
         complo = compli
         icompl = ks(isolut2d(iseqi1))%compl
      else
         complo = .false.
         icompl = 0
      end if

      ivec = abs(ivctst)
      select case ( ichderiv )

      case(0)

!     --- ichderiv = 0; output vector is of type 110

         itypvo = 110
         call sepcreasolbf1 ( ioutvc, itypvo, ivec, 'outvec', icompl )
         nusolo = ks(ioutvc)%nusol

      case(1, 3)

!     --- ichderiv = 1, 3; output vector is of type 115

         itypvo = 115
         if ( ivctst==0 ) itypvo = 110
         call sepcreasolbf1 ( ioutvc, itypvo, ivec, 'outvec', icompl )
         nusolo = ks(ioutvc)%nusol

      case(2)

!     --- ichderiv = 2; output vector is of type 116

         itypvo = 116
         indprfo = 0
         indprpo = 0
         nunkpo = abs(ivctst)
         call sepcreasolbf1 ( ioutvc, 116, ivec, 'outvec', 0 )
         nusolo = ks(ioutvc)%nusol

      case(5)

!     --- ichderiv = 5; output vector is of type 126

         itypvo = 126
         indprfo = 0
         indprpo = 0
         nunkpo = abs(ivctst)
         call sepcreasolbf1 ( ioutvc, 126, ivec, 'outvec', 0 )
         nusolo = ks(ioutvc)%nusol

      case(6)

!     --- ichderiv = 1; output vector is of type 129

         itypvo = 129

!         First reserve space for the first 4*nelgrp positions:
!         Per element group:
!         1: number of integration points
!         2: number of unknowns per point
!         3: number of elements in group
!         4: starting address of first element in real part minus 1

         new = ioutvc<=0 .or. ioutvc>lastsol
         if ( .not. new ) new = solarr(ioutvc)==0
         if ( .not. new ) then

!        --- Old vector, remove

            if (  ks(ioutvc)%typevec==129 ) &
               call seprmsolinfo ( ioutvc )

         end if  ! ( .not. new )
         call sepcrsolinfo ( ioutvc, 0, 4*nelgrp )
         isoltemp => ks(ioutvc)%isol
         indprfo = 0
         indprpo = 0
         nunkpo = abs(ivctst)
         length = size(isoltemp)
         call ercrea129i ( isoltemp, nunkpo, length )

!        --- Store real part in array with memory management number ioutvc+1

         nusolo = length
         allocate ( ks(ioutvc)%sol(length) )
         ks(ioutvc)%iprob = iprob
         ks(ioutvc)%ivec = ivec
         ks(ioutvc)%n = ivec
         ks(ioutvc)%nphys = m
         ks(ioutvc)%nusol = nusolo
         ks(ioutvc)%typevec = itypvo
         usoltemp => ks(ioutvc)%sol

      end select  ! case ( ichderiv )

      if ( iinder(9)==0 ) then
         ks(ioutvc)%sol = 0d0
      end if  ! ( iinder(9)==0 )

      if ( indprf>0 ) kprobf => kprobpart%kprobf

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'indprf, indprfi, indprfo', indprf, indprfi, indprfo
         write(irefwr,1) 'nunkpo, nusolo, itypvo', nunkpo, nusolo, itypvo
         write(irefwr,1) 'ioutvc'
         write(irefwr,*) 'End sepderoutvc'

      end if  ! ( debug )
      call erclos ( 'sepderoutvc' )

      end subroutine sepderoutvc
