      subroutine sepdertype
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Analyze information from backing storage (file 1), as stored in
!     istfl1 for standard elements with standard element number > 99
!     for given icheld the parameters ichtst, ivctsts
!     must be equal for all itype of the problem (and equal to
!     ichderiv+1, ivec for type numbers less than 100
!     Old subroutine to0026
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulebuild
      use sepmodulecpack
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jtest, indtvc, ncheld, ivcder, ivec, ichder
      integer :: istfl1(2*nrec1)
      logical :: stanelfn

!     ichder         Local value of ichderiv
!     indtvc         Number of vectors of special structure
!     istfl1         Array in which information from FILE 1 is stored
!     ivcder         is used tgo compute ivctst
!     ivec           Sequence number of vector of special structure
!     jtest          = 0, all type numbers < 100
!                    > 99, at least one type number with type number jtest
!                    all with respect to one standard element
!     ncheld         Number of choices for derivative subroutine
!     stanelfn       If true the problem contains standard elements with
!                    type numbers > 99
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TO0025         information from backing storage (file 1) is stored in
!                    array istfl1 for a standard problem with standard problem
!                    number > 99
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     227   Parameter ichois has different values for different element types
!     228   Parameter ivec has different values for different element types
!     406   Value of icheld not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepdertype' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepdertype'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ivec = iinder(7)

      ichtst = ichderiv+1

      ifirst = 0
      jtest = 0

!     --- Test if there are element groups with type number >= 100

      stanelfn = .false.
      ivctst = ivec
      do ielgrp = 1, nelgrp
         itype = kprobc(ielgrp)
         if ( itype>=100 ) stanelfn = .true.
      end do

      if ( debug ) &
         write(irefwr,1) 'ichtst, ivctst, nelgrp', ichtst, ivctst, nelgrp

      do ielgrp = 1, nelgrp

         if ( debug ) write(irefwr,1) 'ielgrp', ielgrp
         if ( skipelgrps(ielgrp)==1 ) then

!        --- Element group must not be skipped

            itype = kprobc(ielgrp)

            if ( debug ) then
               write(irefwr,1) 'itype, jtest', itype, jtest
               write(irefwr,*) 'stanelfn', stanelfn
            end if  ! ( debug )

            if ( .not. stanelfn .and. itype<100 .and. itype>0 ) then

!           --- set test values for itype < 100

               if ( jtest<=0 ) then

!              --- jtest < =  0

                  ichtst = ichderiv+1
                  ivctst = ivec
                  jtest = itype

               else

!              --- jtest > 0

!                  compare values
!                  ichderiv+1 different for itype and itype1( = jtest)

                  if ( ichderiv+1/=ichtst ) then
                     call errint ( ichderiv+1, 1 )
                     call errint ( ichtst, 2 )
                     call errint ( itype, 3 )
                     call errsub ( 227, 3, 0, 0 )
                  end if

!                 --- ivec different for itype and itype1( = jtest)

                  if (ivec/=ivctst) then
                     call errint ( ivec, 1 )
                     call errint ( ivctst, 2 )
                     call errint ( itype, 3 )
                     call errsub ( 228, 3, 0, 0 )
                  end if
               end if

            else if ( itype>=100 ) then

!           --- itype> = 100

               call to0025 ( itype, ifirst, ipos, istfl1, ielgrp )
               indtvc = istfl1(ipos+1)
               ncheld = istfl1(ipos+4+indtvc)
               if ( ncheld>0 .and. icheld>ncheld ) then

!              --- possibility icheld not supported by standard problem itype

                  call errint ( icheld, 1 )
                  call errint ( itype, 2 )
                  call errint ( ielgrp, 3 )
                  call errsub ( 406, 3, 0, 0 )

               else if ( ncheld>0 ) then

                  ichder = istfl1(ipos+indtvc+3+2*icheld)
                  ivcder = istfl1(ipos+indtvc+4+2*icheld)

                  if ( debug ) write(irefwr,1) 'ichder, ivcder', ichder, ivcder
                  if ( ichder==3 .and. ivcder>1000 ) then
                     if ( ivcder==1001 ) then
                        ivcder = ndim
                     else if ( ivcder==1002 ) then
                        if ( ndim==1 ) then
                           ivcder = 1
                        else if ( ndim==2 ) then
                           ivcder = 3
                        else if ( ndim==3 ) then
                           ivcder = 6
                        end if
                     else if ( ivcder==1003 ) then
                        if ( ndim==1 ) then
                           ivcder = 0
                        else if ( ndim==2 ) then
                           ivcder = 1
                        else if ( ndim==3 ) then
                           ivcder = 3
                        end if
                     else if ( ivcder==1004 ) then
                        ivcder = ndim**2
                     else if ( ivcder==1005 ) then
                        ivcder = 2*ndim+1
                     else if ( ivcder==1006 ) then
                        if ( ndim<=2 ) then
                           ivcder = 0
                        else if ( ndim==3 ) then
                           ivcder = 3
                        end if
                     end if
                  end if
                  if ( debug ) write(irefwr,1) 'ndim, ivcder', ndim, ivcder

                  if (jtest<=0) then

!                 --- set test values for itype > =  100

                     ichtst = ichder
                     ivctst = ivcder
                     jtest = itype

                   else

!                  --- compare values
!                      ichderiv+1 different for itype and itype1( = jtest)

                     if ( ichder/=ichtst ) then
                        call errint ( ichder, 1 )
                        call errint ( ichtst, 2 )
                        call errint ( itype, 3 )
                        call errsub ( 227, 3, 0, 0 )
                     end if

!                    --- ivec different for itype and itype1( = jtest)

                     if (ivcder/=ivctst)then
                        call errint ( ivcder, 1 )
                        call errint ( ivctst, 2 )
                        call errint ( itype, 3 )
                        call errsub ( 228, 3, 0, 0 )
                     end if
                  end if
               end if
            end if
         end if  ! ( skipelgrps(ielgrp)==1
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ivctst', ivctst
         write(irefwr,*) 'End sepdertype'

      end if  ! ( debug )
      call erclos ( 'sepdertype' )

      end subroutine sepdertype
