      subroutine sepintsectlin1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a list of all triangles that intersect the line y=0 and store
!     the corresponding edges and x-values
! **********************************************************************
!
!                       KEYWORDS
!
!     triangle
!     intersection
!     line
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplotfn
      use sepmoduleint2d
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, icount
      double precision :: ymin1, ymax1

!     i              Counting variable
!     icount         Counter
!     k              Counting variable
!     ymax1           Maximum y-value
!     ymin1           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
!     PRSORT         Sort real array for increasing sequence
!     SEPFILLMAP2D   Fill mapping array for line intersecting a surface
!     SEPREMDOUBLE   Remove double points in array iseqx
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintsectlin1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintsectlin1'
         write(irefwr,1) 'inelem', inelem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( listel(3,inelem), listx(2,inelem), stat = error )
      if ( error/=0 ) call eralloc ( error, 5*inelem, 'listel' )
      if ( ierror/=0 ) go to 1000

      listel = 0
      listx = 0

!     --- For all elements

      numelm = 0
      do i = 1, inelem

         x(1:3) = coor2d(1,trian2d(1:3,i))
         y(1:3) = coor2d(2,trian2d(1:3,i))
         ymin1 = minval(y(1:3))
         ymax1 = maxval(y(1:3))

         if ( ymin1*ymax1<=0d0 ) then

!        --- intersection found

            numelm = numelm+1
            listel(1,numelm) = i

!           --- Find corresponding edges and y-values

            x(4) = x(1)
            y(4) = y(1)
            icount = 0
            do k = 1, 3

               if ( y(k)*y(k+1)<=0d0 ) then

!              --- Line intersects edge

                  icount = icount+1
                  listel(1+icount,numelm) = k
                  listx(icount,numelm) = x(k)+y(k)/(y(k)-y(k+1))*(x(k+1)-x(k))

               end if  ! ( x(k)*x(k+1)<=0d0 )

               if ( icount==2 ) go to 100

            end do  ! k = 1, 3

100         if ( icount==1 ) then

!           --- Intersection in one point, copy

               listel(3,numelm) = listel(2,numelm)
               listx(2,numelm) = listx(1,numelm)

            end if  ! ( icount==1 )

         end if  ! ( ymin1*ymax1<=0d0 )

      end do  ! i = 1, inelem

      if ( debug ) then
         call prinin1 ( listel, numelm, 3, 'listel' )
         call prinrl1 ( listx, numelm, 2, 'listx' )
      end if  ! ( debug )

      if ( numelm==0 ) go to 900

      allocate ( iseqx(2*numelm), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*numelm, 'iseqx' )
      if ( ierror/=0 ) go to 1000

!     --- Sort y-coordinates in ascending order

      call prsort ( listx, iseqx, 2*numelm)
      if ( debug ) call prinin ( iseqx, 2*numelm, 'iseqx' )

!     --- Remove double points

      call sepremdouble

!     --- Fill mapping array

      if ( numint>1 ) then

!     --- intersection with more than 1 point

         call sepfillmap2d

      else

!     --- No intersection with more than 1 point
!         Set numelm = 0 in order to skip plotting

         numelm = 0

      end if  ! ( numint>1 )

      deallocate ( iseqx, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iseqx' )

900   deallocate ( listel, listx, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'listel' )

1000  call erclos ( 'sepintsectlin1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintsectlin1'

      end if  ! ( debug )

      end subroutine sepintsectlin1
