      subroutine sepintsectline
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute intersection of a line with a set of triangles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     line
!     2d
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmoduleplotfn
      use sepmoduleint2d
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision :: dx(2), si, co, xi, yi

!     co             Cosine of angle
!     dx             length of vector in x-direction
!     i              Counting variable
!     si             Sine of angle
!     xi             x-coordinate
!     yi             y-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
!     SEPINTSECTLIN1 Make a list of all triangles that intersect the line y=0
!                    and store the corresponding edges and x-values
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute angle angle of line
!     Map coordinates of triangles by rotating over angle degrees and
!     shifting the origin
!     Make a list of all triangles that intersect the line x=0 and store
!     the corresponding edges and y-values
!     Sort the intersection into a set of lines
!     Map back
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintsectline' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintsectline'
         write(irefwr,2) 'x1', x1
         write(irefwr,2) 'x2', x2
         call prinrl1 ( coor2d, nnodes, 2, 'coor2d' )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute angle angle of line

      dx = x2-x1
      angle = acos(dx(1)/sqrt(dx(1)**2+dx(2)**2))  ! angle in radians
      if ( dx(2)<0d0 ) angle = angle+pi            ! correction for quadrant
      si = sin(angle)
      co = cos(angle)

      if ( debug ) then
         write(irefwr,2) 'dx', dx
         write(irefwr,2) 'angle, si, co', angle, si, co
      end if  ! ( debug )


!     --- Map coordinates of triangles by rotating over angle degrees and
!         shifting the origin

      coor2d(1,1:nnodes) =  coor2d(1,1:nnodes)-x1(1)
      coor2d(2,1:nnodes) =  coor2d(2,1:nnodes)-x1(2)

      do i = 1, nnodes
         xi = coor2d(1,i)
         yi = coor2d(2,i)
         coor2d(1,i) = xi*co + yi*si
         coor2d(2,i) = -xi*si + yi*co
      end do  ! i = 1, nnodes


      if ( debug ) call prinrl1 ( coor2d, nnodes, 2, 'coor2d_map' )

!     --- Make a list of all triangles that intersect the line x=0 and store
!         the corresponding edges and y-values
!         Sort the intersection into a set of lines

      call sepintsectlin1

!     --- Map back

      do i = 1, numint
         imapfn(2,i) = nodes(imapfn(2,i))
         if ( imapfn(1,i)>1 ) imapfn(3,i) = nodes(imapfn(3,i))
      end do  ! i = 1, numint

1000  call erclos ( 'sepintsectline' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( imapfn, numint, 3, 'imapfn' )
         write(irefwr,*) 'End sepintsectline'

      end if  ! ( debug )

      end subroutine sepintsectline
