      subroutine sepmapglobvec ( isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map local vector referred to by isol into global vector
!     Only for host
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     solution_vector
!     vector
!     global
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol

!     isol           i    Sequence number of vector in array ks
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ivec, typevec

!     ivec           Sequence number of vector of special structure
!     typevec        Type of vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPMAPGLOBVEC1 Map local vector referred to by isol into global vector
!                    Only for host
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2708   vectors of this type not implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmapglobvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmapglobvec'
         write(irefwr,1) 'iseqkm, isol, iseqkp', iseqkm, isol, iseqkp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nodes => km(iseqkm)%mappar(inodenr)%arr
      kprobfglobal => kprobpart%kprobfglobal
      typevec = ks(isol)%typevec
      usol => ks(isol)%sol
      usolglob => ks(isol)%solglob
      npoint = km(iseqkm)%kmesh(8)

      if ( debug ) write(irefwr,1) 'typevec, npoint', typevec, npoint

      if ( typevec==110 ) then

!     --- solution vector

         kprobloc => kprobpart%kprobloc
         indprf = kprobloc(19)
         nunkp = kprobloc(4)
         ivec = 0
         if ( debug ) write(irefwr,1) 'indprf, nunkp', indprf, nunkp

         call sepmapglobvec1 ( 0 )

      else if ( typevec==115 ) then

!     --- vector of special structure

         ivec = ks(isol)%ivec
         kprobe2 => kprobpart%kprobe2
         nunkp = -kprobe2(ivec)
         indprf = min(nunkp,0)

         call sepmapglobvec1 ( ivec )

      else

!     --- other type of vector not yet implemented

         call errint ( typevec, 1 )
         call errsub ( 2708, 1, 0, 0 )

      end if  ! ( typevec==110 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmapglobvec'

      end if  ! ( debug )
      call erclos ( 'sepmapglobvec' )

      end subroutine sepmapglobvec
