      subroutine sepprcrossobst
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the arrays icrossobst and rcrossobst
!     Old subroutine princrossobst
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     obstacle
!     intersection
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, i, istarticross, numintersect, iedge, &
              ielem, j, numelms, istart, node1, node2, numnodes, nedgeslc
      double precision fact

!     fact           factor
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          element number
!     iobst          Obstacle sequence number
!     istart         Starting address in nmeshc
!     istarticross   Starting address of obstacle in array icrossobst
!     j              Counting variable
!     nedgeslc       Local number of edges
!     node1          end node of edge
!     node2          end node of edge
!     numelms        number of elements that are intersected by the obstacle
!     numintersect   Number of intersections
!     numnodes       Number of nodal points in edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      write(irefwr,*) ' Information of cross sections of obstacle', &
                      ' with edges'
      do iobst = 1, numobst

         write(irefwr,100) iobst
100      format ( /' Obstacle ', i4/ &
                   ' edge number  factor     nodes'/ )
         istarticross = icrossobst(iobst)
         numintersect = icrossobst(istarticross)
         do i = 1, numintersect

            iedge = icrossobst(istarticross+1)
            istart = nmeshc(iedge)
            numnodes = nmeshc(istart)
            node1 = nmeshc(istart+1)
            node2 = nmeshc(istart+numnodes)
            fact = rcrossobst(icrossobst(istarticross+2))
            istarticross = istarticross+2
            write(irefwr,110) iedge, fact, node1, node2
110         format ( i6, f15.6, 2i6 )

         end do  ! i = 1, numintersect

         if ( ndim>1 ) then

!        --- ndim>1

            write(irefwr,*) 'elements with cross sections'
            write(irefwr,*) 'element      sequence numbers '
            numelms = icrossobst(istarticross+1)
            istarticross = istarticross+1
            do i = 1, numelms

               ielem = icrossobst(istarticross+1)
               nedgeslc = icrossobst(istarticross+2)
               istarticross = istarticross+2
               write(irefwr,120) ielem, &
                  (icrossobst(istarticross+j), j = 1, nedgeslc)
120            format ( i6, ': ',10i6 )
               istarticross = istarticross+nedgeslc

            end do  ! i = 1, numelms

         end if  ! ( ndim>1 )

      end do  ! iobst = 1, numobst
      end
