      subroutine sepprinielobst ( ielemobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array ielemobst
!     Old subroutine prinielobst
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     print
!     element
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ielemobst(*)

!     ielemobst      i    Array containing information about the elements in an
!                         obstacle
!                         The first numobst positions contain the starting
!                         positions of each obstacle sequentially
!                         Per obstacle the first three positions contain
!                         1. The number of elements that are completely inside
!                            the obstacle
!                         2. The number of elements that are not completely
!                            inside the element but with all vertices inside
!                            the element
!                         3. The number of elements that are partly inside
!                            the obstacle
!                            elements that contain only boundary points of the
!                            obstacle are not considered
!                         These positions are followed by the element numbers
!                         in the sequence defined by the first three positions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, istart, numinelmobst, numbnelmobst, numobst

!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numinelmobst   Number of elements that are completely within the obstacle
!     numobst        number of obstacles
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinielobst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinielobst'
         call ertrace
         write(irefwr,1) 'iseqkm', iseqkm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numobst = km(iseqkm)%kmesh(42)
      if ( debug ) write(irefwr,1) 'numobst', numobst
      if ( numobst>0 ) &
          write(irefwr,*) 'Information of elements in obstacles'

      do iobst = 1, numobst

         write(irefwr,*) 'Obstacle number ', iobst
         istart = ielemobst(iobst)
         numinelmobst = ielemobst(istart)+ielemobst(istart+1)
         numbnelmobst = ielemobst(istart+2)
         istart = istart+3
         if ( numinelmobst>0 ) then

!        --- Internal elements

            call prinin ( ielemobst(istart), numinelmobst, &
                          'Elements completely inside the obstacle' )
            istart = istart+numinelmobst

         end if
         if ( numbnelmobst>0 ) then

!        --- Internal points near boundary

            call prinin ( ielemobst(istart), numbnelmobst, &
                          'Elements partly inside the obstacle' )

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinielobst'

      end if  ! ( debug )
      call erclos ( 'sepprinielobst' )

      end subroutine sepprinielobst
