      subroutine sepprinipobst ( ipointobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array ipointobst
!     Old subroutine prinipobst
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     print
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ipointobst(*)

!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, istart, numpntobst, numinnobst, &
              numbndobst, numinonobst, numobst

!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numinonobst    Number of nodal points on the boundary of the obstacle
!                    which are only in elements completely inside the obstacle
!     numobst        number of obstacles
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinipobst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinipobst'
         call ertrace

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numobst = km(iseqkm)%kmesh(42)
      if ( numobst>0 ) &
          write(irefwr,*) 'Information of points in obstacles'

      do iobst = 1, numobst

         write(irefwr,*) 'Obstacle number ', iobst
         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         numinonobst = ipointobst(istart+3)
         istart = istart+4
         if ( numpntobst>0 ) then

!        --- Internal points

            call prinin ( ipointobst(istart), numpntobst, &
                          'Points completely inside the obstacle' )
            istart = istart+numpntobst

         end if
         if ( numinnobst>0 ) then

!        --- Internal points near boundary

            call prinin ( ipointobst(istart), numinnobst, &
                          'Points near boundary of the obstacle' )
            istart = istart+numinnobst

         end if
         if ( numbndobst>0 ) then

!        --- Boundary points

            call prinin ( ipointobst(istart), numbndobst, &
                          'Points on boundary of the obstacle' )
            istart = istart+numbndobst

         end if
         if ( numinonobst>0 ) then

!        --- Boundary points inside obstacle

            call prinin ( ipointobst(istart), numinonobst, &
                          'Points on boundary inside the obstacle' )

         end if

      end do
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinipobst'

      end if  ! ( debug )
      call erclos ( 'sepprinipobst' )

      end subroutine sepprinipobst

