      subroutine sepprinkmeshp1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-08-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print contents of array kmeshpar if it exists
!     Old subroutine prinkmeshpar1
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     print
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idum(1)

!     idum           General dummy variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0008         Print information of curves
!     PR0009         Print information of surfaces
!     PRINGLOBNODES  Print the contents of array globnodes
!     PRININ         print 1d integer array
!     PRINKMESHCGLOB Print array kmeshcglob containing global topology array
!                    in parallel case
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinkmeshp1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinkmeshp1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- In case kmeshpar does not exist, jump from subroutine

      if ( km(iseqkm)%kmesh(65)==0 ) go to 1000
      if ( debug ) call prinin ( kmeshpar, 10, 'kmeshpar' )

      write(irefwr,100) npointglob
100   format ( 'Contents of kmeshpar'/ 'npoint_global = ', i8 )

      if ( kmeshpar(2)>0 ) &
         call prinrl1 ( coorglob, npointglob, ndim, 'coor_global' )

      if ( kmeshpar(3)>0 ) &
         call prinin ( kmeshlglob, nuspnt, 'user_points_global' )

      if ( kmeshpar(4)>0 ) then

!     --- kmeshpar(4)>0, kmesh part m

         write(irefwr,*) 'global nodal points corresponding to curves'
         call pr0008 ( kmeshmglob, ncurvs )

      end if  ! ( kmeshpar(4)>0 )

      if ( kmeshpar(5)>0 ) then

!     --- kmeshpar(5)>0, kmesh part n

         write(irefwr,*) 'global nodal points corresponding to surfaces'
         call pr0009 ( 0, kmeshnglob, idum, idum, idum(1), &
                       ndim, nsurfs )

      end if  ! ( kmeshpar(5)>0 )

!     --- Print mapping

      call prinin ( mappingpar, npoint, 'global node numbers' )

      if ( kmeshpar(7)>0 ) call prinkmeshcglob

      if ( kmeshpar(8)>0 ) &
         call pringlobnodes ( globnodes, globnodes(1+iacnodes) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinkmeshp1'

      end if  ! ( debug )
      call erclos ( 'sepprinkmeshp1' )

      end subroutine sepprinkmeshp1
