      subroutine sepprinmat6p ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print preconditioning matrix that is stored as row-compact real matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat   ! definition of matrices
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, jstart, m, jend, j

!     i              Counting variable
!     j              Counting variable
!     jend           end address
!     jstart         starting position
!     m              number of items to be printed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinmat6p' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinmat6p'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,125)
125   format(//'matrix with row and column numbers'/)

!     --- Row compact storage

      do i = 1, A%neq
         jstart = A%Mrows(i)+1
         jend = A%Mrows(i+1)
         m = min(jstart+3,jend)
         write(irefwr,200) i, ( A%Mcols(j), A%MD(j), j = jstart, m )
200      format(1x,i6,':',4(i5,':',ss,1pe12.5e2))
         if ( m<jend ) &
            write(irefwr,210) ( A%Mcols(j), A%MD(j), j = m+1, jend )
210      format((8x,4(i5,':',ss,1pe12.5e2)))

      end do  ! i = 1, A%neq

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinmat6p'

      end if  ! ( debug )
      call erclos ( 'sepprinmat6p' )

      end subroutine sepprinmat6p
