      subroutine sepprinoutp ( ivectr, iseqlc, iter, timelc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print real/complex  sepran array except the large matrix
!     standard format 5d16.9 is used
!     This subroutine is meant to be used in combination with subroutine
!     prinoutp.
!     prinoutp writes a solution in character format to a file,
!     readsl reads the solution
!         ks(ivectr)%typevec=110,115 allowed
!     Old subroutine prinoutp
! **********************************************************************
!
!                       KEYWORDS
!
!     output
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      use sepmodulemach
      use sepmoduleplot
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivectr, iseqlc, iter
      double precision timelc

!     iseqlc           i    Sequence number
!     iter           i    Iteration sequence number
!     ivectr         i    standard sepran array denoting usol or vector of
!                         special structure defined in nodal points
!     timelc         i    Actual time
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ihelp(7), nlines, nlinesred
      logical compl
      character (len=340) textloc
      character (len=80) textlc

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     i              Counting variable
!     ihelp          integer work array
!     nlines         number of lines that are possibly available
!     nlinesred      number of lines that are written
!     textlc         String to store a local text
!     textloc        String to store a local text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRINOUTCVEC    Write complex sepran solution vector to file sepcomp.out
!     PRINOUTVEC     Write real sepran solution vector to file sepcomp.out
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2299   nphysi>20
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinoutp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinoutp'
         call ertrace
         write(irefwr,1) 'npoint, nelem, ivectr', npoint, nelem, ivectr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,1) 'ks(ivectr)%typevec', ks(ivectr)%typevec
      if ( ks(ivectr)%typevec==116 .or. ks(ivectr)%typevec==126 .or. &
           ks(ivectr)%typevec==801 ) then
         indprfi = 0
         indprpi = 0
         nphysi = ks(ivectr)%ivec
      else if ( ks(ivectr)%typevec==129 ) then
         indprfi = 0
         indprpi = 0
         nphysi = ks(ivectr)%nphys
         if ( debug ) write(irefwr,1) 'nphysi', nphysi
      else
         call sepactsolbf2 ( ivectr )
      end if

      nusoli = ks(ivectr)%nusol
      if ( ierror/=0 ) go to 1000

!     --- Write first part for vector

      textloc = ' '
      if ( nphysi>20 ) then

!     --- nphysi>20, not yet implemented

         call errint ( nphysi, 1 )
         call errint ( 20, 2 )
         call errwar ( 2299, 2, 0, 0 )
         nphysi = 20

      end if

      if ( nphysi==1 ) then

!     --- nphysi = 1, name of component is name of vector

         textloc(1:17) = names(iseqlc+1)(1:17)

      else

!     --- nphysi>1, name of components

         do i = 1, nphysi

            if ( infocomponents(iseqlc+1,2)>0 ) then
               textloc(17*(i-1)+1:17*i) = &
                  namecomponents(infocomponents(iseqlc+1,2)+i-1)(1:17)
            else if ( i<10 ) then
               write(textloc(17*(i-1)+1:17*i),100) i
100            format('   Component_',i1,'   ')
            else
               write(textloc(17*(i-1)+1:17*i),105) i
105            format('   Component_',i2,'  ')
            end if

         end do  ! i = 1, nphysi

      end if
      do i = 1, 7
         ihelp(i) = 0
      end do
      if ( ks(ivectr)%typevec==110 .or. ks(ivectr)%typevec==115 ) then

!     --- Solution vector or vector special structure defined per element

         nlines = npoint
         nlinesred = nlines
         if ( indprfi/=0 ) then
            nlinesred = 0
            do i = 1, nlines
               if ( kprobfi(i+1)-kprobfi(i)/=0 ) &
                  nlinesred = nlinesred+1
            end do
         end if

      else if ( ks(ivectr)%typevec==127 ) then

!     --- Vector defined per edge

         nlines = nmesh(4)
         nlinesred = nlines

      else if ( ks(ivectr)%typevec==801 ) then

!     --- Capacity vector

         nlines = (nphysi*(nphysi-1))/2
         nlinesred = nlines
         nphysi = ks(ivectr)%nusol/nphysi

      else

!     --- Vector defined per element

         nlines = nelem
         nlinesred = nlines

      end if
      if ( iref74>0 ) then

!     --- iref74> 0, formatted

         write(iref74,110) names(iseqlc+1)
110      format('//Vector_name: ',a)
         if ( ks(ivectr)%typevec==110 .or. ks(ivectr)%typevec==115 ) then
            write(iref74,120) '//Points'
120         format(a)
         else if ( ks(ivectr)%typevec==127 ) then
            write(iref74,120) '//Faces'
         else
            write(iref74,120) '//Elements'
         end if
         write(iref74,120) '//Comment: the next 3 data lines contain'
         write(iref74,120) &
         '//Comment: sequence_number, iteration_number, time_level, 7*0'
         write(iref74,120) '//Comment: isol (5 integers for seppost)'
         write(iref74,120) '//Comment: nlines, ncomponents, nglobal'
         write(iref74,130) iseqlc, iter, timelc, (ihelp(i),i=1,7)
130      format(2i9, d15.6, 7i9)

         write(iref74,140) ks(ivectr)%compl, ks(ivectr)%iprob, &
                           ks(ivectr)%ivec, ks(ivectr)%jcart, &
                           ks(ivectr)%nphys, ks(ivectr)%nusol, &
                           ks(ivectr)%typevec, ks(ivectr)%iseqkp
140      format(5i9)
         write(iref74,140) nlinesred, nphysi, nglunk, nfictunk
         write(iref74,120) '//No    '// textloc(1:17*nphysi)

      else

!     --- iref74< 0, unformatted

         textlc = '//Vector_name: '//names(iseqlc+1)
         write(-iref74) textlc
         if ( ks(ivectr)%typevec==110 .or. ks(ivectr)%typevec==115 ) then
            textlc = '//Points'
         else if ( ks(ivectr)%typevec==127 ) then
            textlc = '//Faces'
         else
            textlc = '//Elements'
         end if
         write(-iref74) textlc
         textlc = '//Comment: the next 3 data lines contain'
         write(-iref74) textlc
         textlc = &
        '//Comment: sequence_number, iteration_number, time_level , 7*0'
         write(-iref74) textlc
         textlc = '//Comment: isol (5 integers for seppost)'
         write(-iref74) textlc
         textlc = '//Comment: nlines, ncomponents, nglobal'
         write(-iref74) textlc
         write(-iref74) iseqlc, iter, timelc, (ihelp(i),i=1,7)

         write(-iref74) ks(ivectr)%compl, ks(ivectr)%iprob, &
                        ks(ivectr)%ivec, ks(ivectr)%jcart, &
                        ks(ivectr)%nphys, ks(ivectr)%nusol, &
                        ks(ivectr)%typevec, ks(ivectr)%iseqkp

         write(-iref74) nlinesred, nphysi, nglunk, nfictunk
         textlc ='//No    '
         write(-iref74) textlc(1:8), textloc(1:16*nphysi)

      end if
      compl = ks(ivectr)%compl==1
      usoltemp => ks(ivectr)%sol

      if ( compl ) then

!     --- Complex vector

         call prinoutcvec ( usoltemp, nlines )

      else

!     --- Real vector

         call prinoutvec ( usoltemp, nlines )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinoutp'

      end if  ! ( debug )
      call erclos ( 'sepprinoutp' )

      end subroutine sepprinoutp
