      subroutine sepprintmat ( matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print large matrix
!     Old subroutine prinmt
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulematr
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: matr

!     matr    i   Standard SEPRAN array containing information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINDIAGMAT    Print diagonal matrix
!     SEPPRINTMATB   Print part of matrix corresponding to boundary conditions
!     SEPPRINTMATC   print large complex matrix
!     SEPPRINTMATP   Print part of matrix corresponding to parallel part
!     SEPPRINTMATR   print large real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprintmat' )
      debug = debug_prinmtbf
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprintmat'
         write(irefwr,*) 'diagonal ', realmt(matr)%diagonal

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( realmt(matr)%diagonal ) then

!      --- Diagonal matrix

         call prindiagmat ( matr )

      else

!      --- Standard matrix

         if ( debug ) write(irefwr,*) 'complexmat ', realmt(matr)%complexmat
         if ( realmt(matr)%complexmat ) then
            call sepprintmatc ( matr )
         else
            call sepprintmatr ( matr )
         end if  ! ( realmt(matr(1))%complmat )

         if ( nbound>0 ) call sepprintmatb ( matr )
         if ( debug ) write(irefwr,*) 'parallel ', parallel
         if ( parallel ) call sepprintmatp ( matr )

      end if  ! ( realmt(matr)%diagonal )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprintmat'

      end if  ! ( debug )
      call erclos ( 'sepprintmat' )

      end
