      subroutine sepprintmatr ( matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-03-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print large real matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: matr

!     matr       i    Standard SEPRAN array containing information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM
      type (rmatrix), pointer :: Ar
      type (globmat), pointer :: A

!     A              Structure of type globmat containing the necessary
!                    information of the matrix
!     Ar             Structure of type rmatrix containing the necessary
!                    information of the matrix
!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINMTDIR      print large matrix in case of a direct storage
!     PRINMTSIMPLE   Print matrix corresponding to simple or al storage
!     SEPFILLMATR    Fill parameters in module sepmodulematr
!     SEPPRINMATCOMP print large matrix in case of a real compact storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprintmatr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprintmatr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      A => realmt(matr)
      IM => intmt(A%intmatref)
      Ar => A%S_i

      if ( debug ) write(irefwr,*) 'IM%direct, IM%simple ', IM%direct, IM%simple

      if ( IM%complexm .and. IM%direct ) then

!     --- Direct method

         call sepfillmatr ( matr )
         call prinmtdir ( S_ii )

      else if ( IM%simple ) then

!     --- Simple or schur algorithm

         call prinmtsimple ( matr )

      else if ( .not. IM%nosepmat ) then

!     --- compact storage

         call sepprinmatcomp ( 0d0, Ar )

      end if  ! ( IM%direct )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprintmatr'

      end if  ! ( debug )
      call erclos ( 'sepprintmatr' )

      end subroutine sepprintmatr
