      subroutine sepstream5 ( knpelm, lstep, jnodp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if start node of stream function is a vertex
!
! **********************************************************************
!
!                       KEYWORDS
!
!     compute
!     incompressible
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: knpelm, lstep, jnodp

!     jnodp          i    Nodal point sequence number
!     knpelm         i    Last vertex in element
!     lstep          i    Step in element for use of vertices only
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ikelmc, i, inodp, jtest

!     i              Counting variable
!     ikelmc         position in kmesh part c
!     inodp          Nodal point number
!     jtest          if 2 no error has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     220   specified nodal point number is not a vertex of element
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstream5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstream5'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- position of nodal points of ielem  : ikelmc

      ikelmc = pointkmsc(ielgrp)+(ielem-1-pointnel(ielgrp))*inpelm
      jtest = 0

!     --- Check if start node is vertex

      do i = 1, knpelm, lstep

!     --- find vertex in element ielem

         inodp=kmeshc(ikelmc+i)
         if ( inodp==jnodp ) then
            jtest = 2
            exit
         end if  ! ( inodp==jnodp )

      end do

      if (jtest/=2) then
         call errint ( jnodp, 1 )
         call errint ( ielem, 2 )
         call errsub ( 220, 2, 0, 0 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepstream5'

      end if  ! ( debug )
      call erclos ( 'sepstream5' )

      end subroutine sepstream5
