      subroutine to0076 ( Binv, xstart, ystart, zstart, b, xloc, iloc, &
                          ndim )
! ======================================================================
!
!        programmer    Arno van Duin
!        version  1.3  date 09-03-2013 Correction declaration
!        version  1.2  date 30-11-2012 Long integers
!        version  1.1  date 91-03-21 adjusted for NDIM = 3
!
!   copyright (c) 1990-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   The local coordiantes of the point ( xstart, ystart ) are determined
!   as well as the number of the edge on which it lies
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iloc, ndim
      double precision b(ndim), Binv(ndim,ndim), xloc(ndim), xstart, &
                       ystart, zstart

!     b              i    real vector, the translation vector of the mapping
!     binv           i    real matrix, the inverse of the mapping to a reference
!                         element
!     iloc           o    integer, the number of the edge on which the point
!                         lies
!     ndim           i    Dimension of the space.
!     xloc           o    real array, the local coordinates of (xstart,ystart)
!     xstart         i    real, the x-coordinate of the point
!     ystart         i    real, the y-coordinate of the point
!     zstart         i    real, the z-coordinate of the point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xx(3)

!     xx             ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELM655         performs matrix multiplications
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                                        _   _
!     xx          real array, containing x - b
!                              METHOD
!     ^    -1  _   _        ^                        T
!     x = B  ( x - b )      x  = ( xloc, yloc, zloc ) , &
!                           _                             T
!                           x = ( xstart, ystart, zstart )
!
!     NDIM = 2
!                           ^
!     edge_number = 0    if x is in the interior of the reference element
!
!                           ^   ^
!                 = 1    if x + x - 1 = 0
!                            1   2
!                           ^
!                 = 2    if x = 0
!                            1
!                           ^
!                 = 3    if x = 0
!                            2
!
!     NDIM = 3
!                           ^
!     edge_number = 0    if x is in the interior of the reference element
!
!                           ^   ^   ^
!                 = 1    if x + x + x - 1 = 0
!                            1   2   3
!                           ^
!                 = 2    if x = 0
!                            1
!                           ^
!                 = 3    if x = 0
!                            2
!                           ^
!                 = 4    if x = 0
!                            3
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0076' )

!     --- _   _
!         determine x - b

      xx(1) = xstart - b(1)
      if ( ndim>1 ) xx(2) = ystart - b(2)
      if ( ndim>2 ) xx(3) = zstart - b(3)

!     --- ^        __
!         determine x = Binv xx

      call elm655 ( Binv, ndim, ndim, xx, ndim, 1, xloc )

!     --- ^
!         presume x lies in the middle of the element -> iloc = 0

      iloc = 0

!     --- ^          ^
!         if x = 0 then x lies on edge
!          2                       3

      if ( xloc(2)<1d-25) then
         iloc = 3

!        --- ^          ^
!            if x = 0 then x lies on edge
!             1                       2

      else
         if ( xloc(1)<1d-25) then
            iloc = 2

!           --- if NDIM = 2 then

         else
            if ( ndim==2 ) then

!           --- ^   ^              ^
!               if x + x - 1 = 0 then x lies on edge
!                1   2                           1

               if ( abs( xloc(1) + xloc(2)-1d0)<1d-25 ) then
                  iloc = 1
               end if

!              --- if NDIM = 3 then

            else if ( ndim==3 ) then

!           --- ^          ^
!               if x = 0 then x lies on edge
!                3                       4

               if ( xloc(3)<1d-25) then
                  iloc = 4

!                 --- ^   ^   ^              ^
!                     else if x + x + x - 1 = 0 then x lies on edge
!                      1   2   3                           1

               else if (abs(xloc(1)+xloc(2)+xloc(3)-1d0)<1d-25) then
                  iloc = 1
               end if
            end if
         end if
      end if

      call erclos ( 'to0076' )

      end
