      subroutine to0085 ( iinput, outval, iskip )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.12 date 11-12-2017 New storage of intmat
!        version  3.11 date 06-08-2015 New definition istold
!        version  3.10 date 28-03-2015 Extra debug
!
!   copyright (c) 1991-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine TO0085 is used to compute the integral over a region
!     TO0085 is called by the subroutines VOLINT, VOLUME and INTEGR
!     These subroutines are only "head" subroutines, the actual task is
!     performed by TO0085
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulemain
      use sepmodulebuild
      use sepmodulekprob
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulecpack
      use sepmodulesmf
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iskip(*)
      double precision outval(*)

!                    they are not
!     iinput  i    Integer input array provided by the user
!                  See subroutine READIN
!     iskip   i    See subroutine INTEGSBF
!     outval  o    In this array the computed integral is stored as well as
!                  the minimal and maximum values per element if MAXMIN>0:
!                  OUTVAL(1): integral
!                  OUTVAL(2): minimum value of the integrals per element
!                  OUTVAL(3): maximum value of the integrals per element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichoice, itypvc
      integer, allocatable :: iworkel(:)

!     ichoice        Choice parameter, see array IINPUT
!     iworkel        Work array of length nelem to indicate if an element may
!                    be used (0) or must be skipped (1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPFILLMATSTR2 Fill parameters in module sepmodulematstr
!     SEPFILLOLDSOL  Fill old solutions as well as array flipflop in
!                    sepmodulebuild
!     SEPFILLOLDSOLS Fill array istold with information of old solution vectors.
!                    Reserve space for the necessary arrays
!     SEPINITBUILD   Allocate some arrays for subroutine to0050
!                    Initialize some parameters
!     SEPINTBODY     Compute quantities per element using one or more
!                    input vectors to create integral
!     SEPINTFILLELG  Reserve space in array IBUFFR with respect to
!                    subroutine to0085
!     SEPNULLIFYELM  Nullify all pointers associated with E
!     SEPREACSTART1  Start of assembly process
!                    Make all initializations, fill modules and declare arrays
!     SEPRMMATSTR2   nullify arrays in module sepmodulematstr
!     TO0085FILLEL   Fill array workel with information which elements must be
!                    skipped and which not
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0085' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0085'
         call prinin ( iinput, 10, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting address of kprob

      isubr = 0

!     --- Set constants from array iinput

      icheli = iinput(2)
      jdegfd = iinput(3)
      maxmin = iinput(4)
      ichoice = iinput(5)
      numold = max(1,iinput(6))
      iseqin = max(1,iinput(7))
      numol0 = iinput(6)
      if ( debug ) then

         write(irefwr,1) 'icheli, jdegfd, maxmin, ichoice, numold', &
                          icheli, jdegfd, maxmin, ichoice, numold
         write(irefwr,1) 'iseqin', iseqin

      end if  ! ( debug )

      itypvc = ks(SMF%islold(iseqin))%typevec
      if ( itypvc/=110 .and. itypvc/=115 ) then

!     --- Input vector is of the wrong type

         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )
         go to 1000

      end if

      iprob  = ks(SMF%islold(iseqin))%iprob

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kprobc => kprobpart%kprobc
      kprobj => kprobpart%kprobj
      kmeshloc => kmesh

!     --- Fill modules sepmodulekmesh and sepmodulekprob

      bufchanged = .false.
      kprobf => kprobpart%kprobf
      indprf = kprobloc(19)
      call sepfillkprob

      call sepnullifyelm  ! nullify all pointers in E

!     --- Copy information in cactl

      npointcac = npoint
      nelemcac  = nelem
      inpelm = npelm
      maxunk = 0
      numns = numnat+nelgrp
      ndimlc = ndim

!     --- Create array iworkel

      allocate ( iworkel(nelem), skipelgrps(nelgrp) )
      call to0085fillel ( iworkel, ichoice, iinput(8) )
      if ( debug ) call prinin ( iworkel, nelem, 'iworkel' )

      skipelgrps = 1
      if ( ichoice>0 ) skipelgrps = iskip(1:nelgrp)

!     --- Allocate some arrays for subroutine sepbuild
!         Initialize some parameters
!         These parameters and arrays are all used by element subroutines

      call sepinitbuild
      if ( ierror/=0 ) go to 1000

!     --- Fill some parameters in sepmodulematstr

      call sepfillmatstr2

!     --- Compute pointers for array islold

      numtypesols = 1  ! number of different types of old vectors (at least 1)
      oldsols(1)%nunkp = nunkp

      if ( debug ) then
         write(irefwr,1) 'numtypesols, numold', numtypesols, numold
         write(irefwr,*) 'oldsol ', oldsol
      end if  ! ( debug )

!     --- Fill array istold with information of old solution vectors.
!         Reserve space for the necessary arrays

      call sepfilloldsols ( SMF%islold, 1 )

!     --- Fill pointers in sepmodulebuild for subroutine build
!         Create relation between arrays and buffer

      call sepfilloldsol
      if ( ierror/=0 ) go to 1000

      icompl = ks(SMF%islold(iseqin))%compl+1

!     ---  Start of assembly process
!          Make some initializations, fill modules and declare arrays

      if ( debug ) write(irefwr,*) 'before sepreacstart1'
      call sepreacstart1
      if ( ierror/=0 ) go to 1000

!     --- Initialize some parameters for the element groups

      if ( debug ) write(irefwr,*) 'before sepintfillelg'
      call sepintfillelg

!     --- compute the integral

      if ( debug ) write(irefwr,*) 'before sepintbody'
      call sepintbody ( outval, iworkel )

!     --- deallocate arrays

      if ( debug ) write(irefwr,*) 'before deallocate'
      deallocate ( istold, istold1, oldsols, indoldsols, iworkel, &
                   skipelgrps, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'istoldlc' )

      call seprmmatstr2  ! nullify arrays in modulematstr

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( outval, 2, 'outval' )
         write(irefwr,*) 'End to0085'

      end if  ! ( debug )
      call erclos ( 'to0085' )

      end
