      subroutine to0114 ( nodes, listelms, skiparr, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-10-2012 Debug statements
!        version  1.0  date 11-04-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Make listelms of element groups and elements for all nodal points in nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     element
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nnodes, nodes(nnodes), skiparr(nelgrp)
      integer, intent(out) :: listelms(nelem)

!     listelms       o    Array of length nelem, containing information about the
!                         elements
!                         if listelms(i) = 0, the element must be skipped,
!                         if listelms(i) = 1, the element must be taken into account
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Array of length nnodes containing th nodal point
!                         numbers of all nodes along the curves to be considered
!     skiparr        i    if skiparr(i)=0, element group i must be skipped
!                         otherwise not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iielem, ielgrp, istart, nneigh, ineigh, j, nodp
      integer, allocatable :: ireflg(:)

!                    they are not
!     i              Counting variable
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ineigh         Element that contains nodal point number that is considered
!     ireflg         Work array of length nelem
!                    ireflg(ielem) contains the element group sequence number
!                    of element ielem
!     istart         Starting address in array kmshd2
!     j              Counting variable
!     nneigh         Number of elements a nodal point is lying in
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Fill array ref_element_groups with element numbers corresponding
!    to elements
!    Clear array listelms
!    For all nodes along the curve do
!       Find all admissable elements that are connected to point
!       Store element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0114' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0114'
         call ertrace
         call prinin ( nodes, nnodes, 'nodes' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array ref_element_groups with element numbers corresponding
!         to elements

      allocate ( ireflg(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'ireflg' )
      if ( ierror/=0 ) go to 1000

      iielem = 0
      do ielgrp = 1, nelgrp
         inelem = sepgetinelem ( ielgrp )
         ireflg(iielem+1:iielem+inelem) = ielgrp
         iielem = iielem + inelem
      end do

      listelms = 0  ! Clear array listelms

!     --- For all nodes along the curve do

      do j = 1, nnodes

!     --- Find all admissible elements for the node

         nodp = nodes(j)
         if ( nodp>0 ) then
            istart = kmshd1(nodp)
            nneigh = kmshd1(nodp+1) - istart
            do i = 1, nneigh

               ineigh = kmshd2(istart+i)
               ielgrp = ireflg(ineigh)
               if ( skiparr(ielgrp)>0 ) listelms(ineigh) = 1

            end do  ! i = 1, nneigh
         end if  ! ( nodp>0 )

      end do  ! i = 1, nneigh

      deallocate ( ireflg, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ireflg' )

1000  call erclos ( 'to0114' )
      if ( debug ) then

!     --- Debug information

         call prinin ( listelms, nelem, 'listelms' )
         write(irefwr,*) 'End to0114'

      end if  ! ( debug )

      end
