      subroutine tointersecint ( inpsep, values, result, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute intersection of line with 2d mesh and compute the
!     corresponding line integral
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
!     vector
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: values(*), rinsep(*)
      double precision, intent(out) :: result

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscprinint
!     result         o    Result of computation
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscprinint
!     values         i    Double precision or complex array in which the real
!                         or complex information concerning the line is stored
!                         See subroutine readscprinint (rinsep)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: idegfd, ichois, &
                 iskip, ioutvc(5), ihelp(1), ivalue

!     ichois         Choice parameter, see subroutine intp2dbf
!     idegfd         Degree of freedom sequence number
!     ihelp          dummy
!     ioutvc         Standard SEPRAN array containing information
!                    concerning the interpolated function on the line.
!                    ioutvc is an array of structure 119
!                    Both the one-dimensional co-ordinates as the function
!                    values are stored in the array corresponding to ioutvc
!     iskip          Indicates if element groups must be skipped (>0)
!                    or not (0)
!     ivalue         Integer value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: tolineint

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRREDUCINTERS  Check if end point of line is given and if so reduce array
!                    usol to part defined by this length if length smaller
!                    than original length
!     SEPINTP2D      Interpolate 2D region with straight line
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     TOLINEINT      Compute line integral over vector of type 119
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tointersecint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tointersecint'
         call prinin ( inpsep(1), 5, 'inpsep' )
         call prinin ( isolut, 5, 'isolut' )
         call prinrl ( values, 3, 'values' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      idegfd = inpsep(2)

!     --- Compute intersection

      ichois = 1   ! line is defined by origin and angle
      iskip = 0    ! do not skip element groups

      ioutvc(1) = 0
      call sepintp2d ( ichois, ivalue, values, idegfd, &
                       ioutvc, iskip, ihelp )

!     --- Reduce line in case end point is given

      call prreducinters ( inpsep, ioutvc, 1, rinsep, values )

      if ( debug ) call prinin ( ioutvc, 5, 'ioutvc' )

!     --- Compute line integral

      result = tolineint ( ioutvc, values )

!     --- Remove array ioutvc

      call seprmsolinfo ( ioutvc(1) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tointersecint'

      end if  ! ( debug )
      call erclos ( 'tointersecint' )

      end
