      function tolineint1 ( x, func, numnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the 1d integral over func, where the coordinates are given in x
!     The trapezoid rule is used
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numnodes
      double precision, intent(in) :: x(numnodes), func(numnodes)
      double precision :: tolineint1

!     func           i    contains function values to be integrated
!     numnodes       i    Number of nodal points
!     tolineint1     o    Computed integral
!     x              i    contains the coordinates
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision :: h, subint

!     h              length of integration interval
!     i              Counting variable
!     subint         Sub integral
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3180   Number of nodes along line less than 2
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tolineint1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tolineint1'
         call prinrl ( x, numnodes, 'x' )
         call prinrl ( func, numnodes, 'func' )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( numnodes<=1 ) then

!     --- Error: numnodes too small

         call errint ( numnodes, 1 )
         call errsub ( 3180, 1, 0, 0 )
         go to 1000

      end if  ! ( numnodes<=1 )

      subint = 0d0
      do i = 1, numnodes-1
         h = x(i+1)-x(i)
         subint = subint+0.5d0*h*(func(i+1)+func(i))
      end do  ! i = 1, numnodes

      tolineint1 = subint

1000  call erclos ( 'tolineint1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'tolineint1', tolineint1
         write(irefwr,*) 'End tolineint1'

      end if  ! ( debug )

      end
