      subroutine topmaptetra ( kmeshnew, subdiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-11-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map 3d mesh onto a mesh consisting of linear tetrahedrons only
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshnew
      logical subdiv

!     kmeshnew       o    New array kmesh
!     subdiv         o    If true the elements must be subdivided
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp
      double precision volume

!     ielgrp         element group number
!     volume         Volume of mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision volmesh

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPRINKMESH   Print array KMESH
!     SEPRELKMESH    Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
!     TOPMAPTETRA1   Map 3d mesh onto a mesh consisting of linear tetrahedrons
!                    only
!     VOLMESH        Compute the volume of a region
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check if mesh must be sudivided or not
!     if ( mesh must be subdivided ) then
!        subdiv := true
!        subdivide mesh into tetrahedrons
!     else ( mesh must not be subdivided )
!        subdiv := false
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topmaptetra' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topmaptetra'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d16.8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if mesh must be sudivided or not

      if ( debug ) write(irefwr,1) 'npelm', npelm
      subdiv = .false.
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         if ( ishape==12 .or. ishape>=14 .and. ishape<=16 ) &
            subdiv = .true.
      end do  ! ielgrp = 1, nelgrp
      if ( debug ) write(irefwr,*) 'subdiv ', subdiv

      if ( subdiv ) then

!     --- subdiv true, make a mesh consisting of linear tetrahedrons
!         and hexahedrons only
!         surface elements are skipped

         call topmaptetra1 ( kmeshnew )

      end if  ! ( subdiv )

1000  if ( debug ) then

!     --- Debug information

         if ( subdiv ) then
            call sepprinkmesh ( 1 )
            call sepprinkmesh ( 38 )
            volume = volmesh()
            write(irefwr,2) 'volume old mesh', volume
            call seprelkmesh ( kmeshnew )
            call sepprinkmesh ( 1 )
            call sepprinkmesh ( 38 )
            volume = volmesh()
            call seprelkmesh ( kmesh )
            write(irefwr,2) 'volume new mesh', volume
         end if  ! ( subdiv )
         write(irefwr,*) 'End topmaptetra'

      end if  ! ( debug )
      call erclos ( 'topmaptetra' )

      end subroutine topmaptetra
