      subroutine writesoltofile ( ivectr, namevector, namefile )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 20-06-2013 New definition file ref numbers
!        version  1.3  date 30-08-2006 New definition of cpost5
!        version  1.2  date 26-04-2006 Minor change
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write one solution array to file "filename" referred to by iref74
!     File has the structure of sepcomp.out
! **********************************************************************
!
!                       KEYWORDS
!
!     write
!     solution_vector
!     output
!     file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulemain
      use sepmodulecons
      use sepmodulemach
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivectr(*)
      character (len=*) namevector, namefile

!     ivectr         i    standard sepran array denoting USOL or vector of
!                         special structure defined in nodal points
!     namefile       i    String variable containing name of the file
!                         to which the result is written
!     namevector     i    Name of vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iref74sav
      character (len=32) namessav

!     iref74sav      Is used to save the value of iref74
!     namessav       Is used to save the value of names(1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     PROUTPSUB      Write second part of information to file sepcomp.out
!     SEPPRINOUTP    Write solution vectors to sepcomp.out
!     WRITPROBIINP   Write arrays iinput and rinput corresponding to kprob to
!                    file sepcomp.out
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     open file
!     write first part to file
!     write second part to file
!     write solution to file
!     close file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'writesoltofile' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from writesoltofile'

      end if
      if ( ierror/=0 ) go to 1000

!     --- open file

      iref74sav = iref74
      iref74 = sign(inisetnewref(),iref74)
      open ( unit = abs(iref74), file = namefile )

!     --- write first part to file

      if ( debug ) write(irefwr,*) 'before writprobiinp'
      call writprobiinp

!     --- write second part to file

      numvec = 1
      namessav = names(1)
      names(1) = namevector
      if ( debug ) write(irefwr,*) 'before proutpsub'
      call proutpsub ( numvec, ipost, names, rpost )

!     --- write solution to file

      if ( debug ) write(irefwr,*) 'before prinoutp'
      call sepprinoutp ( ivectr, 0, 1, 0d0 )
      names(1) = namessav

!     --- close file

      call inirmref ( abs(iref74) )
      close ( abs(iref74) )
      iref74 = iref74sav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End writesoltofile'

      end if
      call erclos ( 'writesoltofile' )

      end subroutine writesoltofile
