      subroutine er0060 ( number, x     , y     , nh    , n     , &
                          coor  , iwork, vector, kprobf, &
                          ikprbf, nunkp , factor )
! =======================================================================
!
!        programmer     Guus Segal
!        version 1.2    date   22-08-1995 Spectral ordering added
!        version 1.1    date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 1.0    date   17-10-1988
!
!
!
!   copyright (c) 1988 - 1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g.,  in memory,  disk,  or core)
!   or be transmitted by any means,  electronic,  mechanical,  photocopy
!   recording,  or otherwise,  without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     KEYWORDS
!
!     fill_function
!     1d
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     fill arrays x and y with co-ordinates and function
!     values along the complete mesh, for the special case ndim = 1
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer number, nh, n, ikprbf, nunkp, kprobf(*), &
              iwork(nh)
      double precision coor(*), vector(*), x(*), y(*), factor

!   coor      i   array containing the x-coordinates of the mesh
!   factor    i   the function values are multiplied by factor
!   ikprbf    i   indication whether kprobf has been filled (ikprbf>0)
!                 or not (ikprbf=0)
!   iwork     i   array containing the nodal points to be considered
!   kprobf    i   array kprob part f
!   n         o   actual number of points in x and y
!   nh        i   maximal number of points in x and y
!   number    i   the numberth degree of freedom in each nodal point
!                 is considered
!   nunkp     i   maximal number of degrees of freedom in a nodal point
!   vector    i   array containing the vector from which the function y
!                 must be computed
!   x         o   array containing the x-co-ordinates of the function to
!                 be plotted (arc length)
!   y         o   array containing the function values to be plotted
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer inunkp, iknpt, iknpt1, inusol

!     iknpt      node number to be considered
!     iknpt1     loop variable
!     inunkp     local number of unknowns
!     inusol     Address of unknown in solution array
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     Run through all nodes sequentially and determine the corresponding
!     value in the array vector depending on the number of unknowns
!     in each node
!
! =======================================================================
!
      inunkp = nunkp

      n = 0

      do 100 iknpt1 = 1, nh

         iknpt = iwork(iknpt1)

         if(ikprbf.eq.0) then
            inusol = (iknpt-1)*nunkp+number
         else
            inusol = kprobf(iknpt)
            inunkp = kprobf(iknpt+1)-inusol
            inusol = inusol+number
         end if

         if(inunkp.gt.0) then
            n = n+1
            x(n) = coor(iknpt)
            y(n) = vector(inusol)*factor
         end if

100   continue

      end
