      subroutine indfdv ( ichois, iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 17-10-2000 minor correction
!        version  1.4  date 20-04-1999 Inlcude commons
!        version  1.3  date 28-06-1994 extra postscript (color) devices
!
!   copyright (c) 1992-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subroutine to choose interactively the plotting output device
!    as well as the hard copy plotting output device
!    Depending on the parameter ichois, it also possible to read the defaults
!    from the file with reference number iref or to dump the defaults on this
!    file
!    If defaults are read, it is assumed that subroutine errchr has been
!    called before with the name of environment file as first character
!    variable
! **********************************************************************
!
!                       KEYWORDS
!
!     device
!     machine_dependent
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, iref

!    ichois   i  Choice parameter, defines the type of action.
!                Possible values:
!                0:  Make menu and ask the input from the user interactively
!                1:  Read default values from file wih reference number iref
!                2:  Save default values from file wih reference number iref
!    iref     i  File reference number with respect to defaults file
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ready
      double precision defar(1)
      character * 2 defchr(2)
      integer ivers, irefsv

!     defar          Array in which the number defaults are read
!                    Refar is filled as follows:
!                    1:  version number
!     defchr         Array in which the character defaults are read
!                    Refchr is filled as follows:
!                    1:  posdev
!                    2:  devhcp
!     irefsv         Help variable to save irefwr temporarily
!     ivers          Version number of file
!     ready          Indicates if the loop has been finished (true)
!                    or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERTEXT         Produce menu
!     INICLS         Clear screen
!     READDF         Read defaults
!     READI4         Reads character defaults from file
! **********************************************************************
!
!                       I/O
!
!    The subroutine writes a menu to the screen and reads the users answer
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    -
! **********************************************************************
!
!                       PSEUDO CODE
!
!   if ( ichois=0 ) then
!      ready := false
!      while (not ready) do
!         write menu
!         read answer
!         if ( answer=p ) then
!            switch type of projection
!         else if ( answer=a ) then
!            set view point to automatic
!         else if ( answer=r ) then
!            reset view point
!         else if ( answer=v ) then
!            read view point
!         else if ( answer=q ) then
!            ready := true
!   else if ( ichois=1 ) then
!      read default values
!   else ( ichois=2 )
!      save default values
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'indfdv' )
      if ( ichois.eq.0 ) then

!     --- ichois = 0, make menu

         ready = .false.

!        --- While loop ready

100      if ( .not. ready ) then

!        --- Write menu

            call inicls
            posdev = 'v'
            write (irefwr,110) posdev, devhcp
110         format(' Choose a hard copy unit (present type: ',a2,')'/ &
                   ' code   meaning' )

            write(irefwr,130)
130         format('             Hard copy types:'/ &
                   '   pc   Postscript color file (portrait mode)'/ &
                   '   pw   Postscript color landscape'/ &
                  /'   q   Return to main menu'/ &
                      ' Enter your choice: ')
            ityprd = 1
            call asknxt
            if ( ityprd.eq.4 ) then

!           --- Character found, detect

               if ( string(1:1).eq.'Q' ) then

!              --- Q found, return to main menu

                  ready = .true.

                else if ( string(1:2).eq.'PC' ) then
                   devhcp(1:2) = 'pc'
                else if ( string(1:2).eq.'PW' ) then
                   devhcp(1:2) = 'pw'
               end if

            end if
            go to 100
         end if
      else if ( ichois.eq.1 ) then

!     --- ichois = 1, read defaults from file iref

         call readdf ( iref, defar, 1 )
         if ( ierror.eq.0 ) then

!        --- Environment file correct

            ivers  = nint(defar(1))
            call readi4 ( iref, defchr, 2 )
            posdev = defchr(1)
            devhcp = defchr(2)
         end if
      else

!     --- ichois = 2, save defaults on file iref

         irefsv = irefwr
         irefwr = iref
         ivers = 1
         call errint ( ivers, 1 )
         call errchr ( posdev, 1 )
         call errchr ( devhcp, 2 )
         call ertext ( 26, 1, 0, 2 )
         irefwr = irefsv
      end if
      call erclos ( 'indfdv' )
      end
