      subroutine pl0002
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  3.0  date 12-04-1988 Reduction of parameter list
!        version  2.0  date 14-02-1985
!
!   copyright (c) 1982-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the parameters xmin, xmax, ymin and ymax depending on the
!     the value of jmax as well as on the values of xmin ... ymax
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inodp, istar3, ixmax, iymax
      double precision x, y

!     i          Counting variable
!     inodp      Nodal point number
!     istar3     Starting address of KMESHM part 3 in KMESH part M
!     ixmax      Indication if the x mininimum and maximum values must be
!                computed (0) or not (1)
!     iymax      Indication if the y mininimum and maximum values must be
!                computed (0) or not (1)
!     x          x-coordinate of point
!     y          y-coordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pl0002' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pl0002'
         call ertrace

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Perform the initializations

      istar3 = kmeshm(2)
      length = kmeshm(3)-istar3
      istar3 = istar3-1

!     --- Set ixmax and iymax

      ixmax = 0
      iymax = 0
      if ( jmax/=0 ) then

!     --- jmax = 1, so in fact the user has given the values of xmin etc.
!                   However, if the values of xmin, xmax etc are equal to
!                   plus or minus infinity then no actual filling took place
!                   This may be the case if only one set has been filled

         if ( xmin>-rinfin .and. xmax<rinfin ) ixmax = 1
         if ( ymin>-rinfin .and. ymax<rinfin ) iymax = 1
         if ( ixmax==1 .and. iymax==1 ) go to 1000
      end if

!     --- Inititiate xmin, xmax, ymin and ymax if necessary

      if ( ixmax==0 ) then
         xmin=rinfin
         xmax=-xmin
      end if
      if ( iymax==0 ) then
         ymin=xmin
         ymax=xmax
      end if

!     --- Loop over nodal points at the curves (Hence at least the boundary)

      do 100 i = 1, length

!     --- Find nodal point number

         inodp=kmeshm(istar3+i)
         if ( inodp>0 ) then

!        --- inodp > 0, hence point exists

            if ( ixmax==0 ) then
               x = coor(1,inodp)
               xmin = min ( xmin,x )
               xmax = max ( xmax,x )
            end if
            if ( iymax==0 ) then
               y = coor(2,inodp)
               ymin = min ( ymin,y )
               ymax = max ( ymax,y )
            end if
         end if
100   continue

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pl0002'

      end if  ! ( debug )
      call erclos ( 'pl0002' )

      end subroutine pl0002
