      subroutine pl0005 ( x, y, alpha, z2, inumbr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.11 date 30-04-2010 Debug statements
!        version  5.10 date 01-09-2006 Extension of cpost1
!        version  5.9  date 14-04-2000 Update error message 111
!        version  5.8  date 22-02-1996 Remove VIp correction
!        version  5.7  date 09-02-1996 Correction because of VIp
!
!   copyright (c) 1984-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Start plot, compute parameters and plot PLOTFRAME
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     start
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x, y, alpha, z2
      integer inumbr

!     alpha   i
!     inumbr  i
!     x       i
!     y       i
!     z2      i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision w, hx, dx, s, dy, hy, h, wid1
      double precision ddelta
      integer is

!     ddelta
!     dx
!     dy
!     h
!     hx
!     hy
!     is
!     s
!     w
!     wid1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     PL0006         Plot frame work (General case)
!     PLAFP6         Low level standard machine-dependent plotting subroutine
!     PLOTBOX        Plot small box below the picture
!     PLOTKD         User defined subroutine to plot the frmae work
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     108   xmin>=xmax
!     109   ymin>=ymax
!     110   width of plot is too small
!     111   width of plot is too large
!     112   length of plot is too large
!     113   length of plot is too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pl0005' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pl0005'
         write(irefwr,1) 'irotat', irotat
         write(irefwr,2) 'xmin, xmax, ymin, ymax', &
                          xmin, xmax, ymin, ymax
         write(irefwr,2) 'x, y', x, y
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- call plotstart and compute scale factors

      if ( xmin>=xmax ) &
         call errsub ( 108, 0, 0, 0)
      ddelta = 0.001d0 * (ymin + ymax)
      if ( debug ) write(irefwr,2) 'ddelta', ddelta

      if ( ymin+ddelta>ymax-ddelta ) then

!     --- ymin = ymax, correct both in order to get always a plot

          ymin = ymin - ddelta
          ymax = ymax + ddelta

      end if

      if ( ymin>=ymax ) &
         call errsub ( 109, 0, 0, 0)
      if ( y<=0 ) then

!     --- error 110:  width too small

         call erreal ( y, 1 )
         call erreal ( 0d0, 2 )
         call errsub ( 110, 0, 2, 0 )
      end if
      if ( x<=0 ) then

!     --- error 113:  aleng too small

         call erreal ( x, 1 )
         call erreal ( 0d0, 2 )
         call errsub ( 113, 0, 2, 0 )
      end if
      if ( ierror/=0 ) go to 1000
      if ( y>=x .and.irotat/=1 .or. irotat==2 ) then

!     --- length<width, change roles

         ale=y
         wid=x
         idirec=-1

      else

!     --- length>=width, normal direction

         ale=x
         wid=y
         idirec=1

      end if

      if ( debug ) then
         write(irefwr,1) 'idirec', idirec
         write(irefwr,2) 'ale, wid', ale, wid
      end if  ! ( debug )

      if ( irotat/=1 .and. irotat/=2 .and. ale<=small ) then

!     --- length<small cm, change roles

         w=ale
         ale=wid
         wid=w
         idirec=-idirec

      end if

      if ( wid<=small .and. ale<=alengm ) then

!     --- wid<=small cm,  small paper

         is=1

      else

!     --- wid>small cm,  wide paper

         is=2
         if ( irotat/=1 .and. irotat/=2 .and. ale<=wide ) then

!        --- length<wide cm, change roles

            w=ale
            ale=wid
            wid=w
            idirec=-idirec
         end if
         if ( wid>1d5 ) then

!        --- error 111:  width too large

            call erreal ( wid, 1 )
            call errsub ( 111, 0, 1, 0 )
         end if
         if ( ale>1d5 ) then

!        --- error 112:  length too large

            call erreal ( ale, 1 )
            call erreal ( 0d0, 2 )
            call erreal ( alengm, 3 )
            call errsub ( 112, 0, 3, 0 )
         end if
      end if
      if ( ierror/=0 ) go to 1000
      hx = ale+5d0
      dx = max ( wid+5d0, 18d0 )
      if ( jkader>0 ) then
         if ( idirec==-1 ) then
            hx = hx+5d0
         else
            dx = dx+5d0
         end if
      end if
      if ( hx<dx ) hx = dx
      s = is
      if ( debug ) write(irefwr,2) 's, hx', s, hx
      call plafp6( 1, s, hx )
      if ( debug ) write(irefwr,*) 'after plafp6/1'

      dx = 0.0001*( xmax-xmin )
      dy = 0.0001*( ymax-ymin )
      xfct = ( xmax-xmin )/x
      yfct = ( ymax-ymin )/y
      xmint = xmin-dx
      ymint = ymin-dy
      xmaxt = xmax+dx
      ymaxt = ymax+dy
      alpha = xfct/yfct
      jtest = inumbr
      xprec = xmin
      yprec = ymin
      ioutsd = 0

!     --- plot kader

      hx = 18d0
      hy = 5d0
      dy = hy/3d0
      h = 2d0*dy
      wid = wid+2.5d0
      wid1 = wid
      if ( jkader>0 ) then

!     --- Plot standard frame work

         call pl0006 ( wid1, hx, dx, hy, dy, h, z2 )

      else if ( jkader==-4 ) then

!     --- Special case, call user defined frame work

         call plotkd ( idirec, wid1 )

      else if ( jkader==-5 ) then

!     --- Special case, frame work for seppost

         call plotbox

      end if

1000  call erclos ( 'pl0005' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pl0005'

      end if  ! ( debug )

      end
