      subroutine pl0018 ( str, npnt, contln, ncntln, coor, indcol, &
                          ndim )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.4  date 22-09-1994 Introduce epsmax in test fmax>fmin
!        version  1.3  date 27-05-1994 Remove shift of color
!        version  1.2  date 31-03-1994 Better representation of extreme
!                                      values on line element)
!        version  1.1  date 12-08-1993 Restore dcolor, do not plot lines
!                                      with length 0)
!        version  1.0  date 19-05-1993
!
!   copyright (c) 1993-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute contour lines for one line element
!     plot the intervals of the line bordered by the contour levels
!     by double plotting with colours as indicated by indcol
! **********************************************************************
!
!                       KEYWORDS
!
!     colour
!     contour_level
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npnt(2), indcol(*), ndim, ncntln
      double precision coor(ndim,*), str(2), contln(*)

!     contln    i   array containing the contour levels
!     coor      i   array containing the co-ordinates of the nodal points
!                   The x-coordinate of point i is stored in coor(1,i)
!                   The y-coordinate of point i is stored in coor(2,i)
!                   If ndim=3, coor(3,i) is not used
!     indcol    i   array containing the colour numbers of the various
!                    contour levels
!     ncntln    i   number of contour levels
!     ndim      i   First parameter in the definition of coor
!     npnt      i   nodal point numbers of the two nodal points of line
!     str      i/o  array containing the function values in the points npnt
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x1(2), x2(2), fmin, fmax, xst(2), xen(2), &
                       val1, val2, dif
      integer i, ip(2), icolsv
      logical ready

!     dif     Difference in function values
!     fmax    Greatest value in nodes
!     fmin    Smallest value in nodes
!     i       Counting variable
!     icolsv  Help variable to store the value of dcolor temporarily
!     ip      Array to store the node numbers
!     ready   Logical to indicate whether ready or not
!     val1    Help value to determine start of present coloured interval
!     val2    Help value to determine end of present coloured interval
!     x1      x-co-ordinates of the two nodes
!     x2      y-co-ordinates of the two nodes
!     xen     coordinates of last point of interval to be plotted
!     xst     coordinates of starting point of interval
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLAFEP  General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Sort the function values for increasing value
!     store the coordinates of the nodes in the same sequence
!     Run through all contour values and plot those parts of the line
!     that are within the present band. All parts that are outside the
!     contourbands, i.e. smaller than contln(1) or greater than contln(n)
!     are plotted with respectively color(1) or color(n+1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Determine max and min of the two function values

      if ( str(2).gt.str(1) ) then
           ip(1) = npnt(1)
           ip(2) = npnt(2)
           fmax  = str(2)
           fmin  = str(1)
      else
           ip(1) = npnt(2)
           ip(2) = npnt(1)
           fmax  = str(1)
           fmin  = str(2)
      end if

!     --- Determination of co-ordinates
!         the x- and y-co-ordinates are stored in the arrays x1 and x2
!         in the same sequence as the function values in str

      do 120 i = 1, 2
         x1(i) = coor(1,ip(i))
         x2(i) = coor(2,ip(i))
120   continue
      icolsv = dcolor

!     --- Check which situation has to be considered

      if ( fmax.gt.fmin+epsmac ) then

!     --- Run through all colours

         dif    = fmax - fmin

!        --- Check first special situation that fmin is smaller than contln(1)

         val1 = ( contln(1) - fmin ) / dif
         val2 = ( contln(1) - fmax ) / dif
         if ( val1.gt.0 ) then

!        --- Start by plotting first part
!            (Whole line or only part)

            xst(1) = x1(1)
            xst(2) = x2(1)
            if ( val2.le.0 ) then

!           --- Only part is outside

               xen(1) = x1(1) + val1 * ( x1(2) - x1(1) )
               xen(2) = x2(1) + val1 * ( x2(2) - x2(1) )
            else if ( val2.gt.0 ) then

!           --- Total line is outside

               xen(1) = x1(2)
               xen(2) = x2(2)
            end if

!           --- Plot

            dcolor = indcol(1)
            if( abs(xst(1)-xen(1)).gt.0d0 .or. &
                abs(xst(2)-xen(2)).gt.0d0 )then
                call plafep (  4, xst(1), xst(2), 0d0, 0d0, 0 )
                call plafep ( 23, xen(1), xen(2), 0d0, 0d0, 0 )
            end if
         end if
         do 300 i = 1, ncntln-1
            val1 =  (contln(i  ) - fmin) / dif
            val2 =  (contln(i+1) - fmin) / dif
            if ( val1.lt.0 ) then

!           --- Contln.lt.fmin

               xst(1) = x1(1)
               xst(2) = x2(1)
            else if ( val1.gt.1d0 ) then

!           --- Contln.gt.fmax

                xst(1) = x1(2)
                xst(2) = x2(2)
            else
                xst(1) = x1(1) + val1 * (x1(2)-x1(1))
                xst(2) = x2(1) + val1 * (x2(2)-x2(1))
            end if
            if ( val2.lt.0 ) then

!           --- Contln.lt.fmin

                xen(1) = x1(1)
                xen(2) = x2(1)
            else if ( val2.gt.1d0 ) then

!           --- Contln.gt.fmax

                xen(1) = x1(2)
                xen(2) = x2(2)
            else
                xen(1) = x1(1) + val2 * (x1(2)-x1(1))
                xen(2) = x2(1) + val2 * (x2(2)-x2(1))
            end if

!           --- Plot those parts that are within the interval
!               (Use option 23 , i.e. disturbed double lines)

            dcolor = indcol(i+1)
            if( abs(xst(1)-xen(1)).gt.0d0 .or. &
                abs(xst(2)-xen(2)).gt.0d0 )then
               call plafep (  4, xst(1), xst(2), 0d0, 0d0, 0 )
               call plafep ( 23, xen(1), xen(2), 0d0, 0d0, 0 )
            end if
300      continue

!        --- Check finally situation that fmax is greater than contln(ncntln)

         val1 = ( contln(ncntln) - fmin ) / dif
         val2 = ( contln(ncntln) - fmax ) / dif
         if ( val2.lt.0 ) then

!        --- Determine whether whole line or only part has to be plotted

            xen(1) = x1(2)
            xen(2) = x2(2)
            if ( val1.ge.0 ) then

!           --- Only part is outside

               xst(1) = x1(1) + val1 * ( x1(2) - x1(1) )
               xst(2) = x2(1) + val1 * ( x2(2) - x2(1) )
            else if ( val1.lt.0 ) then

!           --- Total line is outside

               xst(1) = x1(1)
               xst(2) = x1(2)
            end if

!           --- Plot

            dcolor = indcol(ncntln+1)
            if( abs(xst(1)-xen(1)).gt.0d0 .or. &
                abs(xst(2)-xen(2)).gt.0d0 )then
                call plafep (  4, xst(1), xst(2), 0d0, 0d0, 0 )
                call plafep ( 23, xen(1), xen(2), 0d0, 0d0, 0 )
            end if
         end if
      else

!     --- Constant value
!         Determine which color for the whole line

         ready = .false.
         if ( fmax.le.contln(1) ) then
            dcolor = indcol(1)
            ready  = .true.
         else if ( fmin.gt.contln(ncntln) ) then
            dcolor = indcol(ncntln+1)
            ready  = .true.
         end if
         i = 1
400      continue
         if ( .not. ready ) then
              if ( fmin.le.contln(i+1) ) then
                 dcolor = indcol( i+1 )
                 ready  = .true.
              else
                 i = i + 1
              end if
              goto 400
         end if

!        --- Plot this interval

         xst(1) = x1(1)
         xst(2) = x2(1)
         xen(1) = x1(2)
         xen(2) = x2(2)

!        --- Plot line with disturbed double lines (special option 23)

         call plafep (  4, xst(1), xst(2), 0d0, 0d0, 0 )
         call plafep ( 23, xen(1), xen(2), 0d0, 0d0, 0 )
      end if
      dcolor = icolsv
      end
