      subroutine pl3d01 ( iplots, rplots, iincop, rincop )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.3    date   04-06-1998 Extension of rplots with (13)
!        version 1.2    date   05-12-1995 Correction rincop(2)
!        version 1.1    date   28-02-1995 Extension with transparent
!        version 1.0    date   21-01-1995
!
!
!
!   copyright (c) 1995-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill arrays iincop and rincop with values of iplots and rplots
!   extended with defaults. Help subroutine for plotf3
! **********************************************************************
!
!                       KEYWORDS
!
!   3d
!   hidden_line
!   plot
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmoduleconsts
      implicit none
      integer iplots(*), iincop(*)
      double precision rplots(*), rincop(*)

!     iincop  o   Copy of iplots extended with defaults
!     iplots  i   Integer control array in which the user defines which choices
!                 he wants for this subroutine
!                 IPLOTS must be filled  by the user in the following way:
!                 Position:
!                 1:  Defines the highest entry number of the array that has
!                     been filled by the user. This means that the user must
!                     fill all positions 1 to IPLOTS(1) in array IPLOTS.
!                     For all other positions default values are used.
!                     Hence if IPLOTS(1) = 0 or 1 only defaults are used.
!                     Remark: if IPLOTS(i)=0, also for position i the
!                             default is taken.
!                 2:  (linopt) defines the types of lines to be drawn
!                     Possible values:
!                     1: All lines are drawn parallel to the x-axis
!                     2: 2 sets of perpendicular lines are drawn
!                        The first set has the direction defined by rplots(3)
!                     Default value: 1
!                 3:  (nlines) defines the number of lines per cell minus 1
!                     A cell is defined as a square with sides dx, where dx
!                     is defined by the formula:
!                     npoint = (a/dx+1) (b/dx+1), where a and b are the length
!                     and the width of the smallest rectangle enclosing
!                     the region
!                     Default value: 1
!                 4:  (iblock) defines the mode in which the lines are plotted
!                     Possible values:
!                     1: sheet mode
!                     2: block mode
!                     Default value: 1
!                 5:  (number) the number'th physical unknown in each node is
!                     considered When this unknown does not appear in a nodal
!                     point, the point is skipped
!                     When array ISOL corresponds to a complex array, &
!                     NUMBER=2*i-1 corresponds to the real part of unknown i
!                     and NUMBER=2*i to the imaginary part of unknown i
!                     Default value: 1
!                 6:  (iskip) indicates if element groups must be skipped (1)
!                     or not (0)
!                     If iskip=1 array ielskp must be filled
!                     Default value: 0
!                 7:  (itransparent) indicates if the hidden line plot must be
!                     made (0) or a transparent plot (1)
!                     Default value: 0
!     rincop  o   Copy of rplots extended with defaults
!     rplots  i   Double precision input array that must be filled as follows:
!                 Values 0 are replaced by the default value
!                 1:    orientation of projection in degrees
!                       Default: 0
!                 2:    angle between projected y-axis and x-axis in degrees
!                       angle must be between 10 and 170
!                       Default: 60
!                 3:    orientation of primary intersection lines in degrees
!                       Default: 0
!                 4:    width of the plot in cm
!                       Default: 15 cm
!                 5:    height of the plot in cm
!                       Default: 15 cm
!                 6:    xmin
!                       Default: automatically computed
!                 7:    xmax
!                       Default: automatically computed
!                 8:    ymin
!                       Default: automatically computed
!                 9:    ymax
!                       Default: automatically computed
!                10:    ground value to be used in block mode
!                       all function values  at the boundary are
!                       connected to this value. ground value should
!                       be <= the minimal function value.
!                       this amount. If ground value >= 0 the subroutine
!                       calculates a value:
!                       if all function values are >= 0: GV = 0.
!                       if some function values are <0: GV = fmin
!                11:    Scalefactor for of y-axis.
!                       Default 0.5.
!                12:    Scalefactor for of z-axis.
!                       If one of the scales are <= 0 the default
!                       value is set.
!                       Default 1.
!                13:    Multiplication factor for y-coordinates
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i       Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!   none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i = 1, 5
         iincop(i) = 1
      end do
      iincop(6) = 0
      iincop(7) = 0
      do i = 1, 12
         rincop(i) = 0d0
      end do
      rincop(13) = 1d0
      rincop(2) = 60d0
      rincop(4) = defplo
      rincop(5) = defplo
      rincop(11) = 0.5d0
      do i = 1, min(6,iplots(1))
         iincop(i) = iplots(i)
      end do
      rincop(1) = rplots(1)
      if ( rplots(2).ge.10d0 .and. rplots(2).le.170d0 ) &
           rincop(2) = rplots(2)
      rincop(3) = rplots(3)
      if ( rplots(4).gt.3d0 ) rincop(4) = rplots(4)
      if ( rplots(5).gt.3d0 ) rincop(5) = rplots(5)
      rincop(6) = rplots(6)
      rincop(7) = rplots(7)
      rincop(8) = rplots(8)
      rincop(9) = rplots(9)
      rincop(10) = rplots(10)
      if ( rplots(11).gt.0d0 ) rincop(11) = rplots(11)
      if ( rplots(12).gt.0d0 ) rincop(12) = rplots(12)
      if ( rplots(13).gt.0d0 ) rincop(13) = rplots(13)
      end
