      subroutine plaf ( ipict )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-01-1993 minor change in layout
!        version  1.0  date 13-07-1991
!
!   copyright (c) 1991-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine plaf is used to plot the picture stored in the file
!   namepl."ipict", where namepl is the generic name of the plot file
!   and ipict is the picture number in the form 001, 002 etc.
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     read
!     zooming
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipict

!     ipict         i   Picture number (also file number)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ios, iref, lastps, ierrtn
      character * 60 namefl

!     ierrtn       Error return code with the following meaning:
!                  0:  No error has been found
!                  1:  The file with reference number |irefpl| is empty
!                  2:  An incorrect record has been read in the plot file
!                  3:  The plot file does not end with the correct record
!                  4:  The plot file contains a string with too much characters
!                  5:  The plot file contains a polygon consisting of too many
!                      points
!     ios          IOSTAT variable, returns from reading or writing
!     iref         File reference number for plotting file
!     lastps       Last non-blank position in namefl
!     namefl       Name of default SEPRAN neutral plotting file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PLAFRD         Read and plot SEPRAN neutral file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     184   Error in opening of plot file
!    1709   Plot file is empty
!    1710   Plot file is incorrect
!    1711   Plot file end with incorrect record
!    1712   Too much characters in a string
!    1713   Too many nodes in a polygon
! **********************************************************************
!
!                       PSEUDO CODE
!
!   open plot file
!   read and plot contents with the aid of subroutine PLAFRD
!   close plot file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plaf' )

!     --- construct name of plot file by concatenation of namepl and picture
!         number

      lastps = index ( namepl, ' ' ) - 1
      namefl = ' '
      namefl(1:lastps) = namepl(1:lastps)
      namefl(lastps+1:lastps+1) = '.'
      write ( namefl(lastps+2:lastps+4), 100 ) ipict
100   format ( i3.3 )
      lastps = lastps + 4

!     --- Open plot file

      iref = abs(irefpl)
      if ( irefpl.lt.0 ) then

!     --- irefpl < 0, file is unformatted

         open ( unit=iref, file=namefl(1:lastps), iostat=ios, &
                status='old', form='unformatted' )

      else

!     --- irefpl > 0, file is formatted

         open ( unit=iref, file=namefl(1:lastps), iostat=ios, &
                status='old' )

      end if

      if ( ios.ne.0 ) then

!     --- Error on opening of file
!         Presumably the file does not exist

         call errchr ( namefl(1:lastps), 1 )
         call errint ( ios, 1 )
         call errsub ( 184, 1, 0, 1 )
         go to 1000

      end if

!     --- Read and plot PLOT file

      call plafrd ( ierrtn )
      if ( ierrtn.gt.0 ) then

!     --- Error in PLOT file

         call errchr ( namefl(1:lastps), 1 )
         call errsub ( 1708+ierrtn, 0, 0, 1 )

      end if

!     --- Close plot file

      close ( unit = iref )
1000  call erclos ( 'plaf' )
      end
