      subroutine plaf01 ( x, y, xp, yp, xpboun, ypboun, iside )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 09-01-1993 Extra parameter iside, &
!                                      Use marlft and marund
!        version  4.2  date 10-05-1990
!
!   copyright (c) 1983-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Translate user co-ordinates into plotter co-ordinates (cm's)
!     Clipping is applied if required
!     If jtest > 0, it is checked if the point is within the rectangle
!     xmin,xmax -- ymin,ymax. Otherwise clipping is performed.
! **********************************************************************
!
!                       KEYWORDS
!
!     clipping
!     plot
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x, y, xp, yp, xpboun, ypboun
      integer iside

!     iside   o    Indicates at which side the cross section is positioned
!                  Possible values:
!                  0: There is no cross section point computed in this call
!                  1: The cross section point is at side y=ymint
!                  2: The cross section point is at side x=xmaxt
!                  3: The cross section point is at side y=ymaxt
!                  4: The cross section point is at side x=xmint
!     x       i    x-co-ordinate of point from which the co-ordinates in
!                  plot units must be computed
!     xp      i    x-coordinate in plotting units
!     xpboun  i    x-coordinate of point at the boundary of the plotting region
!     y       i    y-co-ordinate of point from which the co-ordinates in
!                  plot units must be computed
!     yp      i    y-coordinate in plotting units
!     ypboun  i    y-coordinate of point at the boundary of the plotting region
!                  if the new or the preceding point is outside the region.
!                  This point is computed by intersection of boundary with the
!                  line x,y -- xprec,yprec
!                  xpboun and ypboun are only filled if jtest=1 or 2
!                  If jtest = 1, the pen goes up to this point before leaving
!                  this subroutine, if jtest = 2 he does not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x00, x1, x2, y00, y1, y2

!     x00     x-coordinate of point outside rectangle
!     x1      x-coordinate of point within rectangle
!     x2      help variable
!     y00     y-coordinate of point outside rectangle
!     y1      y-coordinate of point within rectangle
!     y2      help variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLAFP6  Standard machine dependent plotting
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Copy x and y into x1 and y1

      x1 = x
      y1 = y
      iside = 0
      if ( jtest.gt.0 ) then

!     --- jtest > 0, so check if point is within region

         if ( x1.ge.xmint .and. x1.le.xmaxt .and. y1.ge.ymint &
             .and. y1.le.ymaxt ) then

!        --- x, y within rectangle

            if ( ioutsd.le.1 ) then

!           --- preceding point within rectangle

               ioutsd = 0
            else

!           --- preceding point outside rectangle

               ioutsd = 1
               x00 = xprec
               y00 = yprec
            end if
         else

!        --- x, y outside rectangle

            if ( ioutsd.eq.-1 .or. ioutsd.ge.2 ) then

!           --- preceding point outside rectangle or did not exist

               ioutsd = 3
            else

!           --- preceding point inside rectangle

               ioutsd = 2
               x00 = x1
               y00 = y1
               x1 = xprec
               y1 = yprec
            end if
         end if
         if ( ioutsd.eq.1 .or. ioutsd.eq.2 ) then

!        --- ioutsd = 1 or 2, i.e. one of the points inside and one outside
!            the rectangle. Compute intersection point

            if ( x00.lt.xmint .or. x00.gt.xmaxt ) then

!           --- x00 outside region, compute new value

               if ( x00.lt.xmint ) then

!              --- x00 < xmint

                  x2 = xmint
                  iside = 4
               else

!              --- x00 > xmaxt

                  x2 = xmaxt
                  iside = 2
               end if
               if ( abs(x00-x1).gt.0d0 ) &
                  y00 = (x1*y00-x00*y1-x2*(y00-y1))/(x1-x00)
               x00 = x2
            end if
            if ( y00.lt.ymint .or. y00.gt.ymaxt ) then

!           --- y00 outside region, compute new value

               if ( y00.lt.ymint ) then

!              --- y00 < ymint

                  y2 = ymint
                  iside = 1
               else

!              --- y00 > ymaxt

                  y2 = ymaxt
                  iside = 3
               end if
               if ( abs(y00-y1).gt.0d0 ) &
                  x00 = (x1*y00-x00*y1-y2*(x1-x00))/(y00-y1)
               y00 = y2
            end if
            if ( idirec.ge.0 ) then

!           --- idirec = 1

               xp = (x00-xmin)/xfct+marlft
               yp = (y00-ymin)/yfct+marund
            else

!           --- idirec = -1

               xp = (y00-ymin)/yfct+marund
               yp = (xmax-x00)/xfct+marlft
            end if
            if ( ioutsd.eq.1 ) then

!           --- ioutsd = 1, new point within, old point outside rectangle
!               Store intersection point into xp1 and yp1
!               If jtest = 1, go to intersection point with pen up

               xpboun = xp
               ypboun = yp
               if ( jtest.eq.1 ) call plafp6 ( 3, xp, yp )
            end if
         end if
      end if

!     --- Compute new value of xp and yp if new point is within rectangle

      if ( jtest.eq.0 .or. ioutsd.le.1 ) then

!     --- either jtest=0 (no testing) or ioutsd = 0 or 1 (inside rectangle)

         if ( idirec.ge.0 ) then

!        --- idirec = 1

            xp = (x1-xmin)/xfct+marlft
            yp = (y1-ymin)/yfct+marund
         else

!        --- idirec = -1

            xp = (y1-ymin)/yfct+marund
            yp = (xmax-x1)/xfct+marlft
         end if
      end if
      xprec = x
      yprec = y
      end
