      subroutine plafep ( ichois, x, y, z1, z2, inumbr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.20 date 04-06-2010 Give size of cross
!        version  7.19 date 30-04-2010 Debug statements
!        version  7.18 date 26-10-2009 Correction negative number of digits
!        version  7.17 date 07-03-2007 Correction numbers
!        version  7.16 date 18-01-2001 Remove double lines
!
!   copyright (c) 1984-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    General SEPRAN low level plotting subroutine
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, inumbr
      double precision x, y, z1, z2

!     ichois  i    Choice parameter
!                  Possible values:
!                   6: Plot cross ate center x.y and size z1
!                  24: PLot number in the form 10^i
!     inumbr  i
!     x       i
!     y       i
!     z1      i
!     z2      i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=80) charac
      double precision alpha, xp, yp, xpboun, ypboun, xh, yh, zh2, zh1, &
                       a1, a2, b, x1, x2, y1, y2, a, c, xp1, yp1, h1, &
                       h2, fpn, sizetxt
      integer nchar, i1, iform, defcol, ktimes, iside
      save alpha,xp,yp

!     a              Help variable to store quantity
!     a1
!     a2
!     alpha
!     b
!     c
!     charac
!     defcol
!     fpn
!     h1
!     h2
!     i1
!     iform   parameter to indicate the height of the number to be plotted
!     iside   Indicates at which side the cross section is positioned
!             Possible values:
!             0: There is no cross section point computed in this call
!             1: The cross section point is at side y=ymint
!             2: The cross section point is at side x=xmaxt
!             3: The cross section point is at side y=ymaxt
!             4: The cross section point is at side x=xmint
!     ktimes  mod(itimes,10)
!     nchar   Number of characters in a word
!     sizetxt        Size of text in cm's
!     x1
!     x2
!     xh
!     xp      x-coordinate in plotting units
!     xp1
!     xpboun  x-coordinate of point at the boundary of the plotting region
!     y1
!     y2
!     yh
!     yp      y-coordinate in plotting units
!     yp1
!     ypboun  y-coordinate of point at the boundary of the plotting region
!     zh1
!     zh2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRWAR  Print warnings
!     PL0005  Start plot, compute parameters and plot PLOTFRAME
!     PLAF01  Translate user coordinates into plotter coordinates
!     PLAF02  Convert integer number to character format
!     PLAF03  Convert real number to character format
!     PLAFP3  Machine dependent plotting of symbols
!     PLAFP6  Standard machine dependent plotting
!     PLAFP7  Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     408   Plotting of text is not possible anymore in PLAFEP
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plafep' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plafep'
         write(irefwr,1) 'ichois, inumbr', ichois, inumbr
         write(irefwr,1) 'jtimes', jtimes
         write(irefwr,2) 'x, y, z1, z2', x, y, z1, z2
         write(irefwr,2) 'alpha', alpha
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      go to (  10,  20,  30,  40,  50,  60,  70,  80,  70, 100, &
              100, 100, 100, 100, 100, 160, 170, 180, 180, 200, &
              210, 220, 230, 240 ), ichois

!     --- ichois=1    call plotstart and compute scale factors

 10      if ( mod(jtimes,10)<=1 ) call pl0005 ( x,y,alpha,z2,inumbr )
         if ( debug ) write(irefwr,*) 'after pl0005'
         if ( mod(jtimes,10)==1 ) jtimes = jtimes+1
         go to 1000

!        --- ichois=2    close plotdataset, depending on jtimes

 20      ktimes = mod(jtimes,10)
         if ( ktimes==0 .or. ktimes==3 ) call plafp6(5,0d0,0d0)
         go to 1000

!        --- ichois=3    plot x and y arrows

 30      if ( inumbr>0 ) then
            defcol = dcolor
            dcolor = inumbr
         end if
         if ( idirec>=0 ) then

!        --- idirec=1; x horizontal, y vertical

            call plafp7(1d0,0.25d0,'X',0d0,0.25d0,1)
            call plafp6(3,1.25d0,.375d0)
            call plafp6(2,2d0,.375d0)
            call plafp6(2,1.75d0,0.5d0)
            call plafp6(2,1.75d0,0.25d0)
            call plafp6(2,2d0,.375d0)

!           --- y arrow

            call plafp7(0.5d0,1d0,'Y',0d0,0.25d0,1)
            call plafp6(3,.375d0,1.25d0)
            call plafp6(2,.375d0,2d0)
            call plafp6(2,0.25d0,1.75d0)
            call plafp6(2,0.5d0,1.75d0)
            call plafp6(2,.375d0,2d0)
         else

!        --- idirec=-1;  x vertical, y horizontal

            call plafp7(1d0,wid+2d0,'Y',-90d0,0.25d0,1)
            call plafp6(3,1.25d0,wid+2.125d0)
            call plafp6(2,2d0,wid+2.125d0)
            call plafp6(2,1.75d0,wid+2.25d0)
            call plafp6(2,1.75d0,wid+2d0)
            call plafp6(2,2d0,wid+2.125d0)

!           --- x arrow

            call plafp7(0.25d0,wid+1.5d0,'X',-90d0,0.25d0,1)
            call plafp6(3,.375d0,wid+1.25d0)
            call plafp6(2,.375d0,wid+0.5d0)
            call plafp6(2,0.25d0,wid+0.75d0)
            call plafp6(2,0.5d0,wid+0.75d0)
            call plafp6(2,.375d0,wid+0.5d0)
         end if

!        --- reset default color

         if (inumbr>0) dcolor = defcol
         go to 1000

!        --- ichois=4   plot with pen up

 40      call plaf01(x,y,xp,yp,xpboun,ypboun,iside)
         if ( jtest==0 .or. ioutsd<=2 ) call plafp6(3,xp,yp)
         go to 1000

!        --- ichois=5   plot with pen down

 50      if (inumbr>0) then
            defcol = dcolor
            dcolor = inumbr
         end if
         call plaf01(x,y,xp,yp,xpboun,ypboun,iside)
         if ( jtest==0 .or. ioutsd<=2 ) call plafp6(2,xp,yp)
         if (inumbr>0) dcolor = defcol
         go to 1000

!        --- ichois=6   plot star

 60      if (inumbr>0) then
            defcol = dcolor
            dcolor = inumbr
         end if
         call plaf01(x,y,xp,yp,xpboun,ypboun,iside)
         if ( z1>0d0 ) then
            sizetxt = 0.5d0*z1
         else
            sizetxt = 0.075d0
         end if  ! ( z1>0d0 )
         if ( jtest<=0 .or. ioutsd<2 ) then
            call plafp6(3,xp+sizetxt,yp+sizetxt)
            call plafp6(2,xp-sizetxt,yp-sizetxt)
            call plafp6(3,xp-sizetxt,yp+sizetxt)
            call plafp6(2,xp+sizetxt,yp-sizetxt)
            call plafp6(3,xp      ,yp      )
         end if
         if (inumbr>0) dcolor = defcol
         go to 1000

!        --- ichois=7,9  plot integer number

 70      call plaf01(x,y,xp,yp,xpboun,ypboun,iside)
         if ( jtest>0 .and. ioutsd>=2 ) go to 1000
         fpn=inumbr
         iform=1
         if (ichois==9) iform=2
         h1=xp+z1
         h2=yp+z2
         nchar=0
         call plaf02(fpn,charac,-1,nchar)
         if ( idirec==1 ) call plafp7 ( h1, h2, charac, 0d0, &
              0.15d0*iform, nchar )
         h1=xp+z2
         h2=yp-z1
         if (idirec==-1) call plafp7(h1,h2,charac,-90d0,0.15d0*iform, &
             nchar)
         go to 1000

!        --- ichois=8   plot vector

 80      call plaf01(x,y,xp,yp,xpboun,ypboun,iside)
         if ( jtest>0 .and. ioutsd>=2 ) go to 1000
         call plafp6(3,xp,yp)
         call plaf01(z1,z2,xp1,yp1,xpboun,ypboun,iside)
         call plafp6(2,xp1,yp1)
         if ( jtest>0 .and. ioutsd>=2 ) go to 1000
         a=(xp1-xp)*(xp1-xp)+(yp1-yp)*(yp1-yp)
         if (a<0.01025d0) go to 1000

!        --- when the length is larger than 0.01025 cm, plot arrow point
!            restrict length of arrow point to 0.25 cm
!            length of arrow is (length of vector)/4

            c=4d0
            if (a>1d0) c=4d0*sqrt(a)
            x2=(x+z1*(c-1d0))/c
            y2=(y+z2*(c-1d0))/c
            x1=x2-(y2-z2)/2d0*alpha
            y1=y2+(x2-z1)/(2d0*alpha)
            jtest=1
            call plaf01(x1,y1,xp,yp,xpboun,ypboun,iside)
            call plafp6(2,xp,yp)
            x1=x2+(y2-z2)/2d0*alpha
            y1=y2-(x2-z1)/(2d0*alpha)
            call plaf01(x1,y1,xp,yp,xpboun,ypboun,iside)
            if ( jtest>0 .and. ioutsd==3 ) go to 1000
            call plafp6(2,xp,yp)
            call plafp6(2,xp1,yp1)
            go to 1000

!           --- ichois=10,11,12,13,14 or 15   plot number with centre x,y

100      call plaf01(x,y,xp,yp,xpboun,ypboun,iside)
         if ( jtest>0 .and. ioutsd>2 ) go to 1000
         if ( ichois>11 ) go to 108

!        --- ichois=10,11  plot integer number

            fpn=inumbr
            nchar=0
            b=z1
            call plaf02(fpn,charac,-1,nchar)
            iform=ichois-9
104         a2=-0.075d0*iform
            a1=nchar*a2+iform*0.0375d0
            if (b<0) a2=a2-z2
            if (b>0) a1=a1-z2
            if (idirec==1) then
               call plafp7(xp+a1,yp-0.075d0,charac,0d0, &
                           0.15d0*iform,nchar)
            else
               call plafp7(xp+a2,yp-a1,charac,-90d0,0.15d0*iform,nchar)
            end if
            go to 1000

!           --- ichois=12,13,14 or 15 plot real number

108         i1=abs(inumbr)
            b=inumbr
            fpn=z1
            nchar=0
            if (ichois<=13) then

!           --- ichois=12,13   fixed format

               iform=ichois-11
               call plaf02(fpn,charac,i1,nchar)
               if ( debug ) then
                  write(irefwr,1) 'i1, nchar', i1, nchar
                  write(irefwr,*) 'charac ', charac(1:nchar)
                  write(irefwr,2) 'fpn', fpn
               end if  ! ( debug )
               go to 104

            else

!           --- ichois=14,15   floating point format

               iform=ichois-13
               call plaf03(fpn,charac,i1,nchar,1d0)
               go to 104

            end if

!           --- ichois=16      plot with pen down over z1,z2 inches

160      call plaf01(x,y,xp,yp,xpboun,ypboun,iside)
         if ( jtest>0 .and. ioutsd==2 ) go to 1000
         zh1=z1
         zh2=z2
         if (idirec==1) call plafp6(2,xp+zh1,yp+zh2)
         if (idirec==-1) call plafp6(2,xp+zh2,yp-zh1)
         go to 1000

!        --- ichois=17,  plot markers with user-defined coordinates

170      call plaf01 (x,y,xp,yp,xpboun,ypboun,iside)
         if ( jtest==0 .or. ioutsd<2 ) call plafp3 (inumbr,xp,yp)
         go to 1000

!        --- ichois=18,19   plot text

180      call errwar ( 408, 0, 0, 0 )
         go to 1000

!        --- ichois=20  change xmint,ymint,xmaxt,ymaxt

200      xmint=x
         xmaxt=y
         ymint=z1
         ymaxt=z2
         go to 1000

!        --- ichois = 21  plot double line with pen down

210      xh=xp
         yh=yp
         call plaf01(x,y,xp,yp,xpboun,ypboun,iside)
         if ( jtest>0 .and. ioutsd>=1 ) go to 1000
         call plafp6(3,xh          ,yh          )
         call plafp6(2,xp          ,yp          )
         go to 1000

!        --- ichois = 22    Change current default drawing color

220      dcolor = inumbr

!        --- ichois = 23  plot (disturbed) double line with pen down

230      xh=xp
         yh=yp
         call plaf01(x,y,xp,yp,xpboun,ypboun,iside)
         if ( jtest>0 .and. ioutsd>=1 ) go to 1000

!        --- introduce offset, so that curves will
!            not be overwritten by standard curves

         call plafp6(3,xh+2.5d0*0.01025d0,yh+2.5d0*0.01025d0)
         call plafp6(2,xp+2.5d0*0.01025d0,yp+2.5d0*0.01025d0)
         call plafp6(3,xh+3.0d0*0.01025d0,yh+3.0d0*0.01025d0)
         call plafp6(2,xp+3.0d0*0.01025d0,yp+3.0d0*0.01025d0)
         call plafp6(3,xp+3.5d0*0.01025d0,yp+3.5d0*0.01025d0)
         call plafp6(2,xh+3.5d0*0.01025d0,yh+3.5d0*0.01025d0)
         go to 1000

!        --- ichois=24  plot real number in the form 10^i

240      call plaf01(x,y,xp,yp,xpboun,ypboun,iside)
         if ( jtest>0 .and. ioutsd>=2 ) go to 1000
         fpn=inumbr
         if ( idirec==1 ) then

!        --- normal direction

            nchar = 0
            call plaf02(fpn,charac,-1,nchar)
            a2=-0.075d0
            a1=(5+nchar)*a2
            charac(1:2) = '10'
            nchar = 2
            call plafp7 ( xp+a1, yp+a2, charac, 0d0, 0.3d0, nchar )
            nchar = 0
            call plaf02(fpn,charac,-1,nchar)
            a1 = a1+0.4d0
            a2 = a2+0.3d0
            call plafp7 ( xp+a1, yp+a2, charac, 0d0, 0.15d0, nchar )

         else

!        --- rotated

            nchar = 0
            call plaf02(fpn,charac,-1,nchar)
            a2=-0.075d0
            a1=(5+nchar)*a2
            charac(1:2) = '10'
            nchar = 2
            call plafp7 ( xp+a2, yp-a1, charac, 0d0, 0.3d0, nchar )
            nchar = 0
            call plaf02(fpn,charac,-1,nchar)
            a1 = a1+0.3d0
            a2 = a2-0.4d0
            call plafp7 ( xp+a2, yp-a1, charac, 0d0, 0.15d0, nchar )

         end if
         go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plafep'

      end if  ! ( debug )
      call erclos ( 'plafep' )

      end
