      subroutine plafp7 (x, y, text, angle, height, nchar)
! ======================================================================
!
!        programmer    Guus Segal
!        version 3.22  date 11-03-2013 Correction declaration text
!        version 3.21  date   06-02-2004 Remove superfluous computers
!        version 3.20  date   20-04-1999 Inlcude commons
!
!
!   copyright (c) 1986-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   All rights reserved. No part of this publication may be reproduced,
!   stored in a retrieval system (e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!    subroutine to plot a text starting on (x,y)
!
!    machine dependent subroutine
!    starbase/ hp-gl/ tektronix/ plot10/ apollo/ calcomp/ gks/ cgi
!    PC with VGA or EGA
!
!
! **********************************************************************
!
!                 Input/output parameters

      use sepmodulecomio
      use sepmodulemach
      use sepmoduleplot
      implicit none
      double precision x, y, angle, height
      integer nchar
      character (len=*) text
!
!     x,y       i   give the co-ordinates of the at most left under point of
!                   the first character. The pen moves with pen up to (X,Y).
!                   Double precision parameters.
!     text      i   Text to be plotted. Character variable
!                   The text may contain at most 80 characters
!     angle     i   The text is plotted under an angle of angle degrees.
!                   Double precision parameter
!     height    i   The height of the text is equal to HEIGHT centimeters.
!                   Double precision parameter
!     nchar     i   Number of characters in the text to be plotted
!
! **********************************************************************
!
!                 SEPRAN commons
!
! **********************************************************************
!
!                 Generally used local variables

      real sx, sy, sangle, sheigt
      integer imode, iref

!    iref         Reference number for SEPRAN neutral plot file
!    sx,sy        new x and y coordinates, after addition of new origin
!    sangle       single precision equivalent of angle
!    sheigt       single precision equivalent of height
!    imode        first parameter for unformatted file
!
! **********************************************************************
!
!                 Machine-dependent actions
!
!    In order to get the correct package, the user should remove the
!    comments before the calls of the packages
!
!    The following specific comments are used
!
!    chps   HP-Unix   StarBase version
!    chpw   HP-Unix   Workstation (300/400)
!    chgl   HP-GL plotter version
!    ctek   Tektronix 4010 version (direct)
!    cpl1   Plot-10 version (through library)
!    capo   Apollo version
!    calc   Calcomp version (with slight modifications for some versions)
!    cgks   GKS version
!    cvga   PC, running FTN77/386 with VGA or EGA card
!    ccgi   PC, running UNIX with CGI package
!    cpos   PostScript output
!    cown   Local version, not supported by SEPRAN
!
! ======================================================================
!
      if ( nchar.eq.0 ) return
      iref = abs(irefpl)
      sx = x+xorig
      sy = y+yorig
      sangle = angle
      sheigt = height

!
! **********************************************************************
!   some implementations need explicit setting of the default color
!

      if ((posdev(1:2).ne.'ps' .and. posdev(1:2).ne.'pl') .and. &
          dcolor.gt.0) call plcol0 (dcolor)

!
! **********************************************************************
!
      if (iplotf.eq.1 .and. irefpl.lt.0 .or. posdev(1:2).eq.'fb') then
!
!       *  write binary (unformatted) to file 8
!
         imode=6
         write (iref,err=6000) &
            imode,sx,sy,sangle,sheigt,nchar + 100 * dcolor
         write (iref,err=6000) text(1:nchar)
      else if ( iplotf.eq.1 .or. posdev(1:2).eq.'fa' .or. &
                posdev(1:2).eq.'f1' ) then
!
!       *  write ascii (formatted) to file 8
!
         if ( dcolor.le.9 ) then
            write (iref,1000,err=6000) &
             sx,sy,sangle,sheigt,nchar + 100 * dcolor
1000        format ('6',4(1x,f8.3),1x,i3)
         else
            write (iref,1001,err=6000) &
             sx,sy,sangle,sheigt,nchar + 100 * dcolor
1001        format ('6',4(1x,f8.3),1x,i5)
         end if
         write (iref,1010,err=6000) text(1:nchar)
1010     format (a)
      end if
      if ( posdev(1:1).eq.'f' ) then
         return

! **********************************************************************
!
!   *  PostScript plotter output to file:
!
      else if ( posdev(1:2).eq.'ps' .or. posdev(1:2).eq.'pl' &
           .or. posdev(1:2).eq.'pc' .or. posdev(1:2).eq.'pw' ) then
!
         call plpos7 ( sx, sy, text, sangle, sheigt, nchar )
         return
!
! **********************************************************************
!
!   *  Local version not supported by SEPRAN
!
      else if (posdev(1:2).eq.'ow') then
!
         call plown7 ( sx, sy, text, sangle, sheigt, nchar )
         return
!
! **********************************************************************
!
!   *  HP-GL plotter output
!
      else if (posdev(1:1).eq.'p') then
         call plhgl7 ( sx, sy, text, sangle, sheigt, nchar )
         return

! **********************************************************************
!
!    Unknown choice, unsupported or commented out of compiled version
!
      else
         write (irefwr,9900) posdev
9900     format (' plafp7 - device ',a,' not supported in this version')
         stop
      end if
!
6000  continue
      end
