      subroutine plbc37 ( coor, procor, kmeshc, nelem, ihelp, help, &
                          line, nline, v, icolor, jcolor, nprcor )
! =======================================================================
!
!
!         programmer     Niek Praagman
!         version 2.2    date  21-04-1995 New call of plt353
!         version 2.1    date  21-11-1994 Norms
!         version 2.0    date  30-12-1992 Extra parameter nprcor, make view
!                                        local)
!         version 1.1    date  27-06-1992 Initialise tst and ten
!         version 1.0    date  09-04-1990 almost copy of CONT04,DBWRIZA
!
!
!   copyright (c) 1990 - 1995   "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Plot projection of 3-D object (lines are given in array line) with
!     hidden lines removed (triangles are given in kmeshc)
!
! ***********************************************************************
!
!     KEYWORDS
!
!     projection
!     hidden_line
! ***********************************************************************
!
!     INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer kmeshc(*), line(*), ihelp(*), nelem, nline, icolor, &
              jcolor, nprcor
      double precision coor(*), procor(nprcor,*), help(*), v(3)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     help    i    help array with x-coordinates of triangles
!     icolor  i    color of visible lines
!     ihelp   i    help array with ordering of array help
!     jcolor  i    color of non-visible lines
!     kmeshc  i    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     line    i    array containing all the lines to be considered and
!                  probably to be drawn
!     nelem   i    number of elements
!     nline   i    number of lines stored in array line
!     nprcor  i    dimension of first index of procor
!                  if nprcor = 3, pl0014 is called by a colour subroutine, &
!                  which implies that also the distance to the view-point is
!                  computed, if nprcor = 2, only the projection is computed
!     procor  i    coordinate array for projected points
!                  procor(1,i) contains the x-coordinate of the projected point
!                  procor(2,i) contains the y-coordinate of the projected point
!                  procor(3,i) contains the distance of point to the view-point
!                  i is the point number
!     v       i    array containing the co-ordinates of the view point
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      double precision tnv, tst, ten, xa, ya, xb, yb, xma, &
                       xk, xl, xm, view(20)
      integer istart, i, ip, jp, idr, iview, iv, ivis, ikl, kp, lp, mp

!     i       loop variable
!     idr     loop variable for triangles
!     ikl     indicator for triangle process
!     ip      first node of line
!     istart  indicator where loop for triangles should start
!     iv      loop variable for array view
!     iview   number of "pieces" in array view
!     ivis    indicator whether line is visible, non-visible or partly
!             visible
!     jp      second node of line
!     kp      first node of triangle
!     lp      second node of triangle
!     mp      third node of triangle
!     ten     t-coordinate of end of local line part
!     tnv     temporary end t-coordinate
!     tst     t-coordinate of start of local line part
!     view    helparray of 20 positions
!     xa      x-coordinate first point line
!     xb      x-coordinate second point line
!     xk      x-coordinate of first node of triangle
!     xl      x-coordinate of second node of triangle
!     xm      x-coordinate of third node of triangle
!     xma     maximum of x-coordinates of triangle
!     ya      y-coordinate of first point line
!     yb      y-coordinate of second point line
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS  Deconcatenate name from calling string
!     EROPEN  Concatenate name to calling string
!     PLAFEP  General SEPRAN low level plotting subroutine
!     PLBC30  Plot part of line in projection plane
!     PLT352  Cancel part of a segment of line that is not visible
!     PLT353  Determine whether part of a line is covered by a triangle
!
! ***********************************************************************
!
!     I/O
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Run through all lines
!
!         Run through all triangles
!
!             Determine whether line is visible
!
!         End triangles
!
!         Plot all visible parts with special colour and plot all
!         non-visible parts with another colour
!
!     End lines
!
! =======================================================================
!
      call eropen( 'plbc37' )

      istart = 1

      tst = 0d0
      ten = 0d0

!     --- Run through all lines

      do 400 i = 1 , nline

          ip = line( 2 * i - 1 )
          jp = line( 2 * i     )

          ivis = 0

          call plt352( ivis, tst, ten, view, iview )

          xa = procor( 1, ip )
          ya = procor( 2, ip )
          xb = procor( 1, jp )
          yb = procor( 2, jp )

          ikl = 0

          do 100 idr = istart , nelem

              kp = kmeshc( 3 * idr - 2 )
              lp = kmeshc( 3 * idr - 1 )
              mp = kmeshc( 3 * idr     )

!             --- Check whether the remaining part can be cancelled

              xma = help(ihelp(idr))

              if ( xma.gt.xb ) goto 200

!             --- Consider the first triangles

              if ( ikl.eq.0 ) then

!             --- Determine maximal x of triangle

                   xk = procor( 1, kp )
                   xl = procor( 1, lp )
                   xm = procor( 1, mp )

                   xma = max( xk, xl, xm )

                   if ( xma.lt.xa ) then

                        istart = idr + 1

                   else

                        ikl = 1

                   end if

              end if

              call plt353( v, ip, jp, kp, lp, mp, coor, procor, ivis, &
                           tst, ten, nprcor, 1 )

              if ( ivis.gt.2 ) then

                   call plt352( ivis, tst, ten, view, iview )

              else if ( ivis.eq.2 ) then

!             --- Line is not visible

                   if ( jcolor.gt.0 ) then

!                  --- Plot non-visible line with color jcolor

                        call plafep( 4, xa, ya, 0d0, 0d0, 0      )
                        call plafep( 5, xb, yb, 0d0, 0d0, jcolor )

                   end if

!                  --- Ready for this line

                   goto 400

              end if

100       continue

200       continue

!         --- Plot the visible part(s?) of the line

          tnv = 0d0

          do 300 iv = 1 , iview

              tst = view(2*iv-1)
              ten = view(2*iv  )

              if ( jcolor.gt.0 ) then

!             --- Plot the non-visible part (if any)

                   if ( ( tst - tnv ).gt.5.d-3 ) &
                        call plbc30( tnv, tst, xa, ya, xb, yb, jcolor )

              end if

              tnv = ten

!             --- Plot visible part with color icolor

              if ( ( ten - tst ).gt.5.d-3 ) &
                   call plbc30( tst, ten, xa, ya, xb, yb, icolor )

300       continue

!         --- Check last part and plot eventually

          if ( jcolor.gt.0 ) then

               if ( (1d0 - tnv).gt.5d-3 ) &
                    call plbc30( tnv, 1d0, xa, ya, xb, yb, jcolor )

          end if

400   continue

      call erclos( 'plbc37' )

      end
