      subroutine plbc42 ( kmeshc, nelem , procor, ihelp, help, &
                          ispace, nprcor, kelem )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 18-11-1994
!
!   copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to select all lines with respect to their x-coordinate
!     from small too large
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), nelem, ihelp(*), ispace(*), nprcor, kelem(*)
      double precision procor(nprcor,*), help(*)

!     help    i    double precision help array of length iline
!     ihelp   i    integer helparray of length 2*iline
!     ispace  i    integer work array for sorting
!     kelem   o    triangle array
!     kmeshc  i    part c of SEPRAN array kmesh
!     nelem   i    Number of elements
!     nprcor  i    Dimension of first index of procor
!                  If nprcor = 3, pl0014 is called by a colour subroutine, &
!                  which implies that also the distance to the view-point is
!                  computed, if nprcor = 2, only the projection is computed
!     procor  i    coordinate array for projected points
!                  procor(1,i) contains the x-coordinate of the projected point
!                  procor(2,i) contains the y-coordinate of the projected point
!                  procor(3,i) contains the distance of point to the view-point
!                  i is the point number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4

!     i       loop variable
!     i1      first node of quadrilateral
!     i2      second node of quadrilateral
!     i3      third node of quadrilateral
!     i4      fourth node of quadrilateral
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLBC33    Sort all triangles with respect to the x-coordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all quadrilaterals and place triangles in array KELEM
!     Rearrange the contents of KELEM such that the triangles are sorted
!     with respect to the smallest x-coordinate
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Run through all quadrilaterals and place the nodes of two new
!         triangles in array kelem

      do 100 i = 1 , nelem
          i1 = kmeshc( 4 * i - 3 )
          i2 = kmeshc( 4 * i - 2 )
          i3 = kmeshc( 4 * i - 1 )
          i4 = kmeshc( 4 * i     )
          kelem( 6 * i - 5 ) = i1
          kelem( 6 * i - 4 ) = i2
          kelem( 6 * i - 3 ) = i3
          kelem( 6 * i - 2 ) = i1
          kelem( 6 * i - 1 ) = i3
          kelem( 6 * i     ) = i4
100   continue

!     --- Next rearrange triangle array with respect to x-coordinate

      call plbc33 ( kelem, 2 * nelem, procor, ihelp, help, ispace, &
                    nprcor )
      end
