      subroutine plbcminmax ( usol, number, nphys, amin, amax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-10-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute minimum and maximum value of usol(number,*), where usol
!     is defined per element (type 116) with nphys degrees of freedom per point
! **********************************************************************
!
!                       KEYWORDS
!
!     maximum
!     minimum
!     solution_vector
!     vector_defined_per_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, nphys
      double precision usol(nphys,*), amin, amax

!     amax           o    Maximum value in solution array
!     amin           o    Minimum value in solution array
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ikelmc, ielem, ielgrp, irelem
      double precision  value

!     ielem          Element number
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     value          Value of solution in element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plbcminmax' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plbcminmax'

      end if
      amin = rinfin
      amax = -rinfin
      ikelmc = 0
      ielem  = 1
      do ielgrp = 1, nelgrp
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         if ( ishape<3 .or. ishape>9 ) then
            ielem = ielem+inelem
            ikelmc = ikelmc+inpelm*inelem
            inelem = 0
         end if
         do irelem = 1, inelem
            value = usol(number,ielem)
            amin = min ( amin, value )
            amax = max ( amax, value )
            ikelmc = ikelmc + inpelm
            ielem  = ielem  + 1
         end do
      end do

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plbcminmax'

      end if
      call erclos ( 'plbcminmax' )

      end
