      subroutine plcon1 ( kmeshc, coor, usol, contln, ncntln, nelem, &
                          indcol, iplotc, nline, kline, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.0  date 20-01-2015 New parameter list
!        version  7.0  date 21-05-1993 Extra line elements, extra parameters
!        version  6.0  date 06-01-1993 Extra parameters iplotc and work,
!                                      New call to pl0008)
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill space between contour lines with colours
!     It has been assumed that the mesh consists of trianges only, &
!     and that the solution array contains one unknown in each point
!     Plot line elements too if nline>0.
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nelem, ncntln, kmeshc(3,nelem), indcol(*), &
                             iplotc, nline, kline(2,*), npoint
      double precision, intent(in) :: usol(npoint), contln(ncntln), &
                                      coor(2,npoint)

!     contln         i    User array of length ncntln + 5 (if used) in which
!                         the user is supposed to have stored the contour
!                         levels from position 6
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     indcol         i    array containing the colour indices for the various
!                         levels
!     iplotc         i    Colour of the contour lines, 0 = none
!     kline          i    Array for extra line elements
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     ncntln         i    number of contour levels
!     nelem          i    Number of elements
!     nline          i    Number of line elements to be plotted
!     npoint         i    Number of nodal points
!     usol           i    array containing the values of the function to be
!                         plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ndim, ielem
      double precision :: str(3)

!     ielem          Element number
!     ndim           Dimension of the space.
!     str            Array of length 3 containing the function values in the
!                    vertices of the triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0008         Fill contour levels for one triangle
!     PL0018         Compute and plot contour levels for lines
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plcon1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plcon1'
         write(irefwr,1) 'nelem, nline', nelem, nline
         call prinin1 ( kmeshc, nelem, 3, 'kmeshc' )
         call prinrl ( usol, npoint, 'usol')
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndim = 2
      do ielem = 1, nelem

         str(1:3) = usol(kmeshc(1:3,ielem))
         call pl0008 ( str, kmeshc(1,ielem), contln, ncntln, coor, &
                       indcol, ndim, 0, iplotc )

      end do  ! ielem = 1, nelem

!     --- If necessary plot line elements

      do ielem = 1, nline

         str(1:2) = usol(kline(1:2,ielem))
         call pl0018 ( str, kline(1,ielem), contln, ncntln, coor, &
                       indcol, ndim )

      end do  ! ielem = 1, nline

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plcon1'

      end if  ! ( debug )
      call erclos ( 'plcon1' )

      end subroutine plcon1
