      subroutine plconvertnum ( dnumbr, isig, text, nchar, iform )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Convert real number to text with isig digits behind decimal point
!
! **********************************************************************
!
!                       KEYWORDS
!
!     number
!     conversion
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isig, nchar, iform
      double  precision dnumbr
      character (len=*) text

!     dnumbr         i    Number to be converted
!     iform          i    Defines format to be used
!                         <0:  non-significant zeros are removed
!                         >=0: non-significant zeros are kept
!     isig           i    Number of points behind decimal point
!     nchar          o    Number of characters in text
!     text           o    String representation of dnumbr
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3054   Number too large for fixed format
!    3055   Number of digits after decimal point not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plconvertnum' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plconvertnum'
         write(irefwr,1) 'isig', isig
         write(irefwr,2) 'dnumbr', dnumbr
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( abs(dnumbr)>1d8 ) then

!     --- Number too large for fixed format

         call erreal ( dnumbr, 1 )
         call errsub ( 3054, 0, 1, 0 )
         go to 1000

      end if  ! ( abs(dnumbr)>1d8 )

      select case ( isig )

         case(1)

!        --- isig = 1, one digit behind decimal point

            write(text,110) dnumbr
110         format ( f16.1 )

         case(2)

!        --- isig = 2, 2 digits behind decimal point

            write(text,120) dnumbr
120         format ( f16.2 )

         case(3)

!        --- isig = 3, 3 digits behind decimal point

            write(text,130) dnumbr
130         format ( f16.3 )

         case(4)

!        --- isig = 4, 4 digits behind decimal point

            write(text,140) dnumbr
140         format ( f16.4 )

         case(5)

!        --- isig = 5, 5 digits behind decimal point

            write(text,150) dnumbr
150         format ( f16.5 )

         case(6)

!        --- isig = 6, 6 digits behind decimal point

            write(text,160) dnumbr
160         format ( f16.6 )

         case default

!        --- isig not yet implemented

            call errint ( isig, 1 )
            call errsub ( 3055, 1, 0, 0 )
            go to 1000

      end select  ! case ( isig )
      if ( debug ) write(irefwr,*) text

      text = adjustl(text)            ! remove leading blanks
      nchar = index ( text, ' ' )-1   ! number of non-blank characters

      if ( iform<0 ) then

!     --- iform<0, remove non-significant zeros

         do while ( text(nchar:nchar)=='0' )
            nchar = nchar-1
         end do  ! while ( text(nchar:nchar)=='0' )
         if ( text(nchar:nchar)=='.' ) nchar = nchar-1
         if (  debug ) write(irefwr,*) 'text ', text(1:nchar)

      end if  ! ( iform<0 )

1000  call erclos ( 'plconvertnum' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) text(1:nchar)
         write(irefwr,*) 'End plconvertnum'

      end if  ! ( debug )

      end
