      subroutine plelemskip
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 25-04-2010 New call to plaf05
!        version  2.0  date 07-02-2003 Extra parameter kmeshm
!        version  1.1  date 28-02-2001 Correction text to be plotted
!
!   copyright (c) 2000-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot elements that must be skipped
!     Two-dimensional case only
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     skip
!     plot
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iscolr, inpelm, ikader, ielgrp, ikelmc, ielem, iielem, j, nodp
      double precision x1(2,9)

!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikader         Help variable to store jkader temporarily
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     iscolr         Help variable to store the value of dcolor temporarily
!     j              Counting variable
!     nodp           Nodal point sequence number
!     x1             help array to store the coordinates of the nodes of an
!                    element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAF05         Plot polygon
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     SEPPLOTBN      Plot boundary of region
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plelemskip' )

!     --- open plot

      jkader = -1
      textx = 'x'
      texty = 'y'
      call pl0003 ( ikader, 1, 15d0, 1d0, 10 )

!     --- plot elements

      iscolr = dcolor
      ikelmc = 0
      ielem  = 0
      do ielgrp = 1, nelgrp

!     --- Determine number of elements and number of nodes in element

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         nbounelm = sepgetbounelm(ielgrp)
         do iielem = 1, inelem

            ielem = ielem+1
            if ( kprobaa(ielem)/=0 ) then
               dcolor = 45
               do j = 1, nbounelm
                  nodp = kmeshc(ikelmc+j)
                  x1(1,j) = coor(1,nodp)
                  x1(2,j) = coor(2,nodp)
               end do
               call plaf05 ( 1, dcolor, nbounelm, x1, 2 )
               call plafep(4, x1(1,1), x1(2,1), 0d0, 0d0, 0)
            end if
            ikelmc = ikelmc+inpelm

         end do

      end do

!     --- plot boundary

      dcolor = 1
      call sepplotbn (  0, 3, .true. )

!     --- plot text

      dcolor = iscolr
      call plafp7 ( 7.5d0, 18.5d0, 'elements that must be skipped', 0d0, &
                    0.25d0, 29 )

!     --- close plot dataset

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
      call erclos ( 'plelemskip' )

      end
