      subroutine plelm1par ( elements )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-02-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot elements in a one or two-dimensional mesh
!     If this subroutine is changed also change plelm1 !!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     2d
!     plot
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer elements(*)

!     elements       i    Array of length nelem indicating if element ielem
!                         must be plotted (>0) or not (0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ilow, ielgrp, nboun, nisrf, ikelmc

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     ilow           elements are numbered from element number ilow
!     inpelm         number of points per element
!     nboun          Number of boundary points at an element
!     nisrf          Help variable for number of extra nodes in surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLELEMPAR      Plot elements for one element group (2d)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plelm1par' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plelm1par'

      end if

      ilow   = 1
      ikelmc = 1

      do ielgrp = 1 , nelgrp

!     --- Determine shape number and check whether 2d element

         ishape = kmeshf(ielgrp)

!        --- Determine number of elements and number of nodes in element

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         nboun = inpelm
         if ( kelmb/=0 ) then
            if ( kmeshb(ielgrp)>0 ) nboun = kmeshb(ielgrp)
         end if
         if ( kelmu>0 .and. ndim>1 .and. ishape.gt.0 ) then

!        --- kelmu > 0, Spectral elements
!            determine niedge and nisurf

            niedge = kmeshu(2)
            nisurf = kmeshu(3)
            if ( nisurf==-1 ) then

               if ( ishape==3 ) then
                    nisrf = 1
               else if ( ishape==5 ) then
                    nisrf = niedge * niedge
               end if

            else

               nisrf = nisurf

            end if

            if ( ishape==1 ) then

!           --- Line mesh (Special situation)

               nboun  = inpelm

            else if ( ishape==3 ) then

!           --- Triangles, delete internal points

               nboun = inpelm - nisrf

            else if ( ishape==50 ) then

!           --- Interface elements (Special situation)

               nboun  = inpelm

            else

!           --- Quadrilaterals

               if ( nisrf==0 ) then

!              --- Row by row, no internal points

                  nboun = -1

               else

!              --- Internal points

                  nboun = -2

               end if

            end if

         end if

         call plelempar ( coor, kmeshc(ikelmc), ilow, inelem, &
                          inpelm, nboun, ndim, elements )

!        --- Raise number of elements and start position

         ilow   = ilow   +          inelem
         ikelmc = ikelmc + inpelm * inelem

      end do

      if ( debug ) write(irefwr,*) 'End plelm1par'
      call erclos ( 'plelm1par' )

      end
