      subroutine plfl00 ( npoint, ibrpnt, nelem, ibrelm, kelem, lbrpnt, &
                          nbndpt, ihelp, coor, istart, iend, kmeshl )
! ======================================================================
!
!        programmer    Arno van Duin
!        version  1.3  date 27-09-2005 New call to plbc03
!        version  1.2  date 20-01-1996 New calls to some subroutines
!        version  1.1  date 03-07-1992 Replace hlpnode and lownode by
!                                      hlpnod, lownod
!        version  1.0  date 12-07-1991
!
!   copyright (c) 1991-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine the boundary nodal point numbers and arrange them in a
!     counter clockwise order
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(*)
      integer ibrelm(*), ibrpnt(*), iend, ihelp(*), istart, kelem(*), &
              kmeshl(*), lbrpnt, nbndpt(*), nelem, npoint

!     coor    i    KMESH part i, see P.G. 13.1.6.
!     ibrelm  o    work array
!     ibrpnt  o    work array, the number of elements a node belongs to
!     iend    i    last user point of boundary part
!     ihelp   o    work array
!     istart  i    first user point of boundary part
!     kelem   i    KMESH part m, see P.G. 13.1.8.
!     kmeshl  i    KMESH part l, see P.G. 13.1.8.
!     lbrpnt  o    number of boundary (nodal)points
!     nbndpt  o    array with the begin- and end-points of the boundary
!     nelem   i    number of elements
!     npoint  i    number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer hlpnod, i, iel, i1, i2, i3, ikn1, ikn2, indend, &
              indstr, itel, itelh, lownod, lowpt2
      double precision x1, xikn1, xikn2, y1, yikn1, yikn2

!     hlpnod  temporary storage of a node number
!     i       loop variable
!     i1      node number of first node of element
!     i2      node number of second node of element
!     i3      node number of third node of element
!     iel     loopvariable (element number)
!     ikn1    node number
!     ikn2    node number
!     indend  node number of last point of boundary
!     indstr  node number of first point of boundary
!     itel    cumulative number of elements a node belongs to
!     itelh   temporary storage parameter
!     lownod  node number of node with lowest y-coordinate
!     lowpt2  node number of neighbour node of LOWNOD of most horizontal
!             boundary element
!     x1      x-coordinate of LOWNOD
!     xikn1   x-coordinate of neighbour node of LOWNOD
!     xikn2   x-coordinate of other neighbour node of LOWNOD
!     y1      y-coordinate of LOWNOD
!     yikn1   y-coordinate of neighbour node of LOWNOD
!     yikn2   y-coordinate of other neighbour node of LOWNOD
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INSTOP    Stop the program
!     PLBC02    fill the element number in the right position of array ibrelm
!               for point i
!     PLBC03    fill array nbndpt with all the boundary elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1503   User point is not a nodal point
!    1504   user point does not lie on the boundary
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Fill array IBRPNT with the number of elements a node belongs to
!    Fill array IBRELM using IBRPNT
!    Fill arrat NBNDPT using IBRPNT and IBRELM
!    Put the elements of NBNDPT in the right order:
!       Fill array IHELP with the neighbour nodes of the nodes
!          ( neighbour1 of node(i) = IHELP(2*i-1)
!            neighbour2 of node(i) = IHELP(2*i) )
!       To ensure counterclockwise direction:
!          Determine node with lowest y-coordinate (LOWNOD)
!          Determine neighbournode of LOWNOD with most horizontal position
!           to LOWNOD
!          Determine right order of these two nodes (lowest x-coordinate first)
!       Determine first node of boundary
!       Walk along boundary until first node is encountered
!       Fill NBNDPT
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plfl00' )

!     --- reset array ibrpnt

      do i = 1, npoint + 1
         ibrpnt(i) = 0
      end do

!     --- fill ibrpnt

      do iel = 1, nelem
         i1 = kelem(3*iel-2)
         i2 = kelem(3*iel-1)
         i3 = kelem(3*iel)
         ibrpnt(i1) = ibrpnt(i1) + 1
         ibrpnt(i2) = ibrpnt(i2) + 1
         ibrpnt(i3) = ibrpnt(i3) + 1
      end do

!     --- rearrange

      itel=0
      do i = 1, npoint
         itelh = itel + 1
         itel  = itel + ibrpnt(i)
         ibrpnt(i) = itelh
      end do
      ibrpnt(npoint+1)=itel+1

!     --- reset array ibrelm

      do i = 1, itel
         ibrelm(i) = 0
      end do

!     --- fill ibrelm

      do iel = 1, nelem
         i1 = kelem(3*iel-2)
         i2 = kelem(3*iel-1)
         i3 = kelem(3*iel  )
         call plbc02 ( ibrelm, ibrpnt, i1, iel )
         call plbc02 ( ibrelm, ibrpnt, i2, iel )
         call plbc02 ( ibrelm, ibrpnt, i3, iel )
      end do

!     --- first fill of nbndpt

      call plbc03 ( ibrpnt, ibrelm, npoint, kelem, lbrpnt, &
                    nbndpt )

!     --- reset work array ihelp

      do i = 1, 2 * npoint
         ihelp(i) = 0
      end do

!     --- fill array ihelp and determine node with lowest y-coordinate

      lownod = 0
      do i = 1, lbrpnt
         ikn1 = nbndpt(2*i-1)
         ikn2 = nbndpt(2*i)
         if ( lownod==0 ) lownod = ikn1
         if ( ihelp(2*ikn1-1)==0 ) then
            ihelp(2*ikn1-1) = ikn2
         else
            ihelp(2*ikn1) = ikn2
         end if
         if ( ihelp(2*ikn2-1)==0 ) then
            ihelp(2*ikn2-1) = ikn1
         else
            ihelp(2*ikn2) = ikn1
         end if
         if ( coor(2*ikn1)<coor(2*lownod) ) lownod = ikn1
         if ( coor(2*ikn2)<coor(2*lownod) ) lownod = ikn2
      end do

!     --- determine neighbour node of lownod with most horizontal boundary part

      x1 = coor(2*lownod-1)
      y1 = coor(2*lownod)
      ikn1 = ihelp(2*lownod-1)
      ikn2 = ihelp(2*lownod)
      xikn1 = coor(2*ikn1-1)
      yikn1 = coor(2*ikn1)
      xikn2 = coor(2*ikn2-1)
      yikn2 = coor(2*ikn2)
      if ( abs(x1-xikn1)*abs(y1-yikn2) < &
           abs(y1-yikn1)*abs(x1-xikn2) ) then
         lowpt2 = ikn2
      else
         lowpt2 = ikn1
      end if
      if ( coor(2*lownod-1)>coor(2*lowpt2-1) ) then
         ikn1 = lowpt2
         ikn2 = lownod
      else
         ikn1 = lownod
         ikn2 = lowpt2
      end if

!     --- find first node that belongs to the boundary

      if ( istart==0 .and. iend==0 ) then
         indstr = 1
80       if ( ihelp(2*indstr-1)==0 ) then
            indstr = indstr + 1
            go to 80
         end if
         indend = indstr
      else
         indstr = kmeshl(istart)
         indend = kmeshl(iend)
         if ( indstr==0 ) then
            call errint ( istart, 1 )
            call errsub ( 1504, 1, 0, 0 )
            call instop
         end if
         if ( ihelp(2*indstr-1)==0 ) then
            call errint ( istart, 1 )
            call errsub ( 1503, 1, 0, 0 )
            call instop
         end if
         if ( indend==0 ) then
            call errint ( iend, 1 )
            call errsub ( 1504, 1, 0, 0 )
            call instop
         end if
         if ( ihelp(2*indend-1)==0 ) then
            call errint ( iend, 1 )
            call errsub ( 1503, 1, 0, 0 )
            call instop
         end if
      end if

!     --- walk along boundary from lownod until start node is encountered

90    if ( ikn1/=indstr ) then
         hlpnod = ikn1
         ikn1 = ikn2
         if ( ihelp(2*ikn1-1)==hlpnod ) then
            ikn2 = ihelp(2*ikn1)
         else
            ikn2 = ihelp(2*ikn1-1)
         end if
         go to 90
      end if

!     --- fill nbndpt

      i = 1
100   if ( i<=lbrpnt ) then
         nbndpt(2*i-1) = ikn1
         nbndpt(2*i)   = ikn2
         if ( ikn2==indend ) lbrpnt = i
         hlpnod       = ikn1
         ikn1          = ikn2
         if ( ihelp(2*ikn1-1)==hlpnod ) then
            ikn2 = ihelp(2*ikn1)
         else
            ikn2 = ihelp(2*ikn1-1)
         end if
         i = i + 1
         go to 100
      end if
      call erclos ( 'plfl00' )
      end
