      subroutine plmn00 ( numfil, iseqps, iseqpl, stop )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.3    date 21-02-1999 Minor corrections
!        version 1.2    date 21-01-1993 New method of raising seq
!        version 1.1    date 11-01-1993 Keep zooming as default
!        version 1.0    date 01-01-1993
!
!   copyright (c) 1993-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Plot individual SEPRAN neutral plotting files in the user defined
!   sequence. Perform zooming if required.
!   This subroutine is interactive and corresponds to the submenu
!   plotting of separate files called by plsepd (SEPDISPLAY)
! **********************************************************************
!
!                       KEYWORDS
!
!   interactive
!   plot
!   zooming
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmoduleread
      use sepmodulemach
      use sepmoduleplot
      implicit none
      integer iseqpl, iseqps, numfil
      logical stop

!    iseqpl   Sequence number of HPGL plot
!    iseqps   Sequence number of Postscript plot
!    numfil   Number of plot files found
!    stop     Indicates if a stop has been found in the sub menu (true) or not
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ready, zoomdf
      integer iseq, iseqnw, lastps, ierrtn, nexseq
      character *2 possav
      character * 60 namefl
      save zoomdf
      data zoomdf /.false./

!     ierrtn  Error return code with the following meaning:
!             -1:  Error in opening of plot file
!              0:  No error has been found
!              1:  The file with reference number |irefpl| is empty
!              2:  An incorrect record has been read in the plot file
!     iseq    Sequence number plot file
!     iseqnw  New sequence number read
!     lastps  Last non-blank position in namefl
!     namefl  Name of default SEPRAN neutral plotting file
!     nexseq  Next sequence number for file to be plotted
!     possav  Help variable to store the preceding value of posdev temporarily
!     ready   Indicates if the loop has been finished (true) or not (false)
!     zoomdf  Indicates if zooming defaults are set for the present file
!             (true) or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT  Read next item from the standard input file
!     ERRCHR  Put character in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     ERTEXT  Produce menu
!     INICLS  Clear screen
!     PL0007  Initialise zooming parameters
!     PLAF    Plot file
!     PLMN01  Make menu define zooming parameters
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!  1709   Plot file is empty
!  1710   Plot file is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   ready := false
!   seq_number := 1
!   next_seq_number := 1
!   if ( first call ) then
!      zoom := false
!      zoom_level := 0
!      zoom_def := false
!   stop := false
!   while ( not ready ) do
!      print sub menu
!      read answer
!      If ( answer = 'none' ) then
!         seq_number := next_seq_number
!         make plot
!         next_seq_number := next_seq_number + 1
!      else if ( answer = number ) then
!         check number
!         seq_number := number
!         next_seq_number := number
!      else if ( answer = b or z ) then
!         zoom := true
!         if ( not zoom_def ) then
!            Set zooming defaults
!            zoom_def := true
!         if ( answer = z ) then
!            zoom_level := zoom_level + 1
!         else ( answer = b )
!            zoom_level := zoom_level - 1
!      else if ( answer = c ) then
!         make hard copy
!      else if ( answer = d ) then
!         if ( not zoom_def ) then
!            Set zooming defaults
!            zoom_def := true
!         Define zooming parameters
!      else if ( answer = h ) then
!         print help menu
!      else if ( answer = k ) then
!         ready := true
!      else if ( answer = o ) then
!         zoom := false
!         zoom_level := 0
!      else if ( answer = q ) then
!         ready := true
!         zoom := false
!         zoom_level := 0
!      else if ( answer = s ) then
!         ready := true
!         stop := true
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iseq   = 1
      nexseq = 1
      if ( .not. zoomdf ) then

!     --- first call, set parameters for zooming

         lzoom  = 0
         ilzoom = 0

      end if

      ready  = .false.
      stop   = .false.
      lastps = index ( namepl, ' ' ) - 1

!     --- While loop ready

100   if ( .not. ready ) then

!     --- Write sub menu

         namefl = ' '
         namefl(1:lastps+1) = namepl(1:lastps)//'.'
         write ( namefl(lastps+2:lastps+4),110 ) iseq
110      format ( i3.3 )
         call inicls
         call errchr ( namepl(1:lastps), 1 )
         call errchr ( namefl(1:lastps+4), 4 )
         write ( namefl(lastps+2:lastps+4),110 ) nexseq
         call errchr ( namefl(1:lastps+4), 6 )
         call errint ( numfil, 1 )
         call errint ( 0, 2 )
         if ( lzoom.eq.1 ) then

!        --- Zooming activated, hence option zooming off makes sence

            call errint ( ilzoom, 2 )
            call errchr ( 'o', 2 )
            call errchr ( 'set zooming off', 3 )
            call errchr ( 'uses last defined zooming options', 5 )
            call errchr ( 'uses last defined zooming options', 7 )

         else

!        --- Zooming not activated

            call errchr ( ' ', 2 )
            call errchr ( ' ', 3 )
            call errchr ( ' ', 5 )
            call errchr ( ' ', 7 )

         end if

         call ertext ( 29, 2, 0, 7 )

!        --- Read answer

         ityprd = 1
         ispecl = 1
         call asknxt

         if ( ispecl.eq.2 ) then

!        --- Blank record found hence RETURN
!            Start plotting of one picture

            iseq = nexseq
            call plaf ( iseq )
            if ( ierror.ne.0 ) then

!           --- Error found in plot file

               call ertext ( 1, 0, 0, 0 )
               read(*,*)
               ierror = 0
               nerror = 0

            end if

            if ( iseq.lt.numfil ) nexseq = nexseq + 1
            go to 100

         end if

         if ( ityprd.eq.2 ) then

!        --- Number read, set iseq

            iseqnw = nint(realrd)
            if ( iseqnw.le.0 .or. iseqnw.gt.numfil ) then
               call errint ( iseqnw, 1 )
               call errint ( numfil, 2 )
               call errchr ( namepl(1:lastps), 1 )
               call ertext ( 4, 2, 0, 1 )
            else
               iseq   = iseqnw
               nexseq = iseqnw
            end if
            zoomdf = .false.
            go to 100

         else if ( ityprd.ne.4 ) then

!        --- No text found, repeat menu

            go to 100

         end if
         if ( string(1:1).eq.'B' .or. string(1:1).eq.'Z' ) then

!        --- Letters B or Z found, activate zooming

            lzoom = 1
            if ( .not. zoomdf ) then

!           --- Zooming defaults are not yet defined

               call pl0007 ( iseq, ierrtn )
               if ( ierrtn.gt.0 ) then

!              --- Error in plot file

                  call errchr ( namefl(1:lastps), 1 )
                  call errsub ( 1708+ierrtn, 0, 0, 1 )

               end if
               if ( ierrtn.ne.0 ) then
                  call ertext ( 1, 0, 0, 0 )
                  go to 100
               end if
               zoomdf = .true.

            end if
            if ( string(1:1).eq.'Z' ) then
               ilzoom = ilzoom + 1
            else
               ilzoom = ilzoom - 1
            end if

         else if ( string(1:1).eq.'C' ) then

!        --- Letter C found, make hard copy

            possav = posdev
            posdev = devhcp
            call plaf ( iseq )
            if ( ierror.ne.0 ) then

!           --- Error found in plot file

               call ertext ( 1, 0, 0, 0 )
               read(*,*)
               ierror = 0
               nerror = 0

            end if

            if ( posdev(1:2).eq.'pl' .or. posdev(1:2).eq.'ps' ) then

!           --- Post script file has been made

               iseqps = iseqps + 1
               string(1:8) = 'sepposc.'
               write(string(9:11),110) iseqps
               write(*,*) ' Picture', iseq, ' has been stored in', &
                          ' file ', string(1:11)
            else

!           --- HPGL file has been made

               iseqpl = iseqpl + 1
               string(1:8) = 'sephpgl.'
               write(string(9:11),110) iseqpl
               write(*,*) ' Picture', iseq, ' has been stored in', &
                          ' file ', string(1:11)
            end if

            posdev = possav

         else if ( string(1:1).eq.'D' ) then

!        --- Define zooming parameters

            if ( .not. zoomdf ) then

!           --- Zooming defaults are not yet defined

               call pl0007 ( iseq, ierrtn )
               if ( ierrtn.gt.0 ) then

!              --- Error in plot file

                  call errchr ( namefl(1:lastps), 1 )
                  call errsub ( 1708+ierrtn, 0, 0, 1 )

               end if
               if ( ierrtn.ne.0 ) then
                  call ertext ( 1, 0, 0, 0 )
                  go to 100
               end if
               zoomdf = .true.

            end if
            call plmn01

         else if ( string(1:1).eq.'H' ) then

!        --- Help menu

            call inicls
            call ertext ( 32, 0, 0, 0 )
            read (*,*)

         else if ( string(1:1).eq.'K' ) then

!        --- Return to main menu, but keep zooming parameters

            ready = .true.

         else if ( string(1:1).eq.'O' ) then

!        --- Set zooming off

            lzoom  = 0
            ilzoom = 0

         else if ( string(1:1).eq.'Q' ) then

!        --- Return to main menu

            lzoom  = 0
            ilzoom = 0
            ready = .true.

         else if ( string(1:1).eq.'S' ) then

!        --- Stop

            zoomdf = .false.
            ready = .true.
            stop  = .true.

         end if

         go to 100

      end if

      end
