      subroutine plotcheckreg ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of sub-arrays and ivecseq
!     Set region if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision , intent(in) :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotcn
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscplotcn
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2830   Clipping only limited to 2d
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotcheckreg' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotcheckreg'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inpsep(2)>0 .and. ndim==2 ) then

!     --- inpsep(2)>0, region is given; ndim = 2

         jmax = 1
         xmin = rinsep(inpsep(2))
         xmax = rinsep(inpsep(2)+1)
         ymin = rinsep(inpsep(2)+2)
         ymax = rinsep(inpsep(2)+3)
         if ( debug ) &
            write(irefwr,2) 'xmin, xmax, ymin, ymax', &
                             xmin, xmax, ymin, ymax

      else if ( inpsep(2)>0 ) then

!     --- inpsep(2)>0, region is given; ndim # 2
!         not yet implemented

         call errint ( ndim, 1 )
         call errwar ( 2830, 1, 0, 0 )

      end if  ! ( inpsep(2)>0 .and. ndim==2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotcheckreg'

      end if  ! ( debug )
      call erclos ( 'plotcheckreg' )

      end subroutine plotcheckreg
