      subroutine plotcompscal2 ( exp1, exp3, minout, maxout, step, ndec, &
                                 shift )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Scale numbers for scales, since they are too large
!
! **********************************************************************
!
!                       KEYWORDS
!
!     scale
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer exp1, exp3, ndec
      double precision minout, maxout, step, shift

!     exp1          i/o   Exponent to be used for these values
!     exp3           i    Exponent corresponding to valmax-valmin
!     maxout        i/o   Scaled valmax
!     minout        i/o   Scaled valmin
!     ndec          i/o   Number of digits to be used in step
!     shift          o    Shift of numbers if step is much lower than numbers
!     step          i/o   Step along axis
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer exp2
      double precision diff, mult

!     diff           Difference between valmax and valmin
!     exp2           Help exponent
!     mult           Multiplication factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inipl2

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPL2         compute exponent of real number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotcompscal2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotcompscal2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      diff = max(abs(minout),abs(maxout))
      exp2 = inipl2 ( diff )   ! compute exponent
      if ( exp1==0 ) then

!     --- exp1 = 0, scale

         if ( exp2-exp3>4 ) then

!        --- Difference between exponents too large, scale on basis of
!            largest number

            exp1 = exp2-4

         else

            exp1 = exp3-1

         end if  ! ( exp2-exp3>4 )
         mult = 10d0**(-exp1)
         minout = minout*mult
         maxout = maxout*mult
         step = step*mult
         if ( step<0.999d0 ) then
            ndec = -inipl2 ( step )
         else
            ndec = 0
         end if  ! ( step<0.999d0 )

      else if ( exp1>0 ) then

!     --- exp1>0, use shift

         shift = 0.5d0*(minout+maxout)
         minout = minout-shift
         maxout = maxout-shift

      end if  !( exp1==0 )

1000  call erclos ( 'plotcompscal2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotcompscal2'

      end if  ! ( debug )

      end
