      subroutine plotfunc11 ( nfunc, funcx, funcy, ny, ncolours, &
                              colours )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-10-2012 extra debug
!        version  1.0  date 25-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Actual plot of function values
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nfunc,  ny, ncolours, colours(*)
      double precision funcx(nfunc), funcy(nfunc,ny)

!     colours        i    Contains the set of colors to be used for each
!                         function
!     funcx          i    Set of values in x-direction
!     funcy          i    ny sets of values in y-direction
!     ncolours       i    Number of colors stored in colours
!     nfunc          i    Number of values per function
!     ny             i    number of functions to be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision multx, multy, x, y

!     i              Counting variable
!     j              Counting variable
!     multx          Multiplication factor in x-direction
!     multy          Multiplication factor in y-direction
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotfunc11' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotfunc11'
         call prinrl ( funcx, nfunc, 'funcx' )
         call prinrl1 ( funcy, ny, nfunc, 'funcy' )
         write(irefwr,1) 'ncolours, ny', ncolours, ny
         call prinin ( colours, ncolours, 'colours' )
         write(irefwr,2) 'axmin, axmax, aymin, aymax', &
                          axmin, axmax, aymin, aymax
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      multx = 10d0**(-expx)
      multy = 10d0**(-expy)
      if ( debug ) write(irefwr,2) 'multx, multy', multx, multy

      do i = 1, ny
         if ( i<=ncolours ) then
            dcolor = colours(i)
         else
            dcolor = 0
         end if  ! ( i<=ncolours )
         x = funcx(1)*multx
         y = funcy(1,i)*multy

         if ( debug ) then
            write(irefwr,1) 'i, dcolor', i, dcolor
            write(irefwr,2) 'x, y', x, y
         end if  ! ( debug )

         call plafep ( 4, x, y, 0d0, 0d0, 0 )  ! pen up

         do j = 2, nfunc
            x = funcx(j)*multx
            y = funcy(j,i)*multy
            call plafep ( 5, x, y, 0d0, 0d0, 0 )  ! pen down
         end do  ! j = 2, nfunc

      end do  ! i = 1, ny

1000  call erclos ( 'plotfunc11' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotfunc11'

      end if  ! ( debug )

      end
