      subroutine plotnewbnd ( nodenrs, inbndnod, layer, &
                              bndcoornew, text1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-10-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot the new boundary during the creation of the mesh in a specific
!     iteration
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     boundary
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inbndnod, nodenrs(*), layer
      double precision bndcoornew(4,*)
      character *(*) text1

!     bndcoornew     i    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     inbndnod       i    number of coordinates of nodes in bndcri
!     layer          i    Present layer sequence number
!     nodenrs        i    Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     text1          i    Text to be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, j, lentext
      character *100 text

!     j              Counting variable
!     lentext        Number of characters in text
!     node           Node number
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0000         Compute length and width of the plot
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Open plot
!     Plot boundary
!     Close plot
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotnewbnd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotnewbnd'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Open plot

      call pl0000 ( 15d0, 1d0 )
      call plafep( 1, aleng, width, 1d0, 15d0, 1 )

!     --- Go to last node with pen up

      j = inbndnod
      node = nodenrs(j)
      call plafep( 4, bndcoornew(1,j), bndcoornew(2,j), 0d0, 0d0, 0 )
      call plafep( 6, bndcoornew(1,j), bndcoornew(2,j), 0d0, 0d0, 63 )
      call plafep ( 7, bndcoornew(1,j), bndcoornew(2,j), -0.06d0, &
                    0.06d0, node )
      if ( debug ) &
         write(irefwr,*) 'node ', node, bndcoornew(1,j), bndcoornew(2,j)

!     --- Go to all other nodes with pen down

      do j = 1, inbndnod
         node = nodenrs(j)
         call plafep( 5, bndcoornew(1,j), bndcoornew(2,j), 0d0, 0d0, 0 )
         call plafep( 6, bndcoornew(1,j), bndcoornew(2,j), 0d0, 0d0, 63)
         if ( mod(j,5).eq.0 ) &
            call plafep ( 7, bndcoornew(1,j), bndcoornew(2,j), -0.06d0, &
                          0.06d0, node )
         if ( debug ) &
            write(irefwr,*) 'node ', node, bndcoornew(1,j), &
                            bndcoornew(2,j)
      end do

!     --- Close plot

      lentext = len(text1)+1
      text = text1
      text(lentext+1:lentext+18) = '        loop '
      write(text(lentext+19:lentext+24),200) layer
200   format(i6)
      call plafp7 ( 1d0, 1d0, text, 0d0, 0.25d0, lentext+24 )
      call plafep( 2, 0d0, 0d0, 0d0, 0d0, 0 )

1000  call erclos ( 'plotnewbnd' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotnewbnd'

      end if

      end
