      subroutine plotscales
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot scales depending on minimum and maximum values
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     plot_axis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision valmin, valmax

!     valmax         Maximum value of function
!     valmin         Minimum value of function
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLOTFUNC1SCAL  Compute scales to be used for plot function
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotscales' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotscales'
         write(irefwr,1) 'kiesx, kiesy', kiesx, kiesy
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute parameters to be used along axis

      if ( kiesx==-1 ) then

!     --- kiesx = -1, automatic scaling along x-axis

         call plotfunc1scal ( axmin, axmax, valmin, valmax, stepx, &
                              isigx, expx, shiftx )
         axmin = valmin
         axmax = valmax
         nstepx = nint((axmax-axmin)/stepx)

         if ( debug ) then
            write(irefwr,1) 'isigx, expx', isigx, expx
            write(irefwr,2) 'axmin, axmax, stepx, shiftx', &
                             axmin, axmax, stepx, shiftx
         end if  ! ( debug )

      end if  ! ( kiesx==-1 )

      if ( kiesy==-1 ) then

!     --- kiesy = -1, automatic scaling along y-axis

         call plotfunc1scal ( aymin, aymax, valmin, valmax, stepy, &
                              isigy, expy, shifty )
         aymin = valmin
         aymax = valmax
         nstepy = nint((aymax-aymin)/stepy)

         if ( debug ) then
            write(irefwr,1) 'isigy, expy', isigy, expy
            write(irefwr,2) 'aymin, aymax, stepy, shifty', &
                             aymin, aymax, stepy, shifty
         end if  ! ( debug )

      end if  ! ( kiesx==-1 )

1000  call erclos ( 'plotscales' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotscales'

      end if  ! ( debug )

      end
