      subroutine plotxaxis ( axhulp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 12-05-2014 Correction position x-text
!        version  1.1  date 27-07-2010 Correction integer case
!        version  1.0  date 24-05-2010
!
!   copyright (c) 2010-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     plot the x-axis of a function plot
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     axis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision axhulp

!     axhulp         i    Defines the step along the x-axis
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inumbr, nchar, iside
      double precision ydigits, y, x, sizetxt, xdown, lenmarker, &
                       xp, yp, xpboun, ypboun, x1, yshift

!     i              Counting variable
!     inumbr         number to be plotted
!     iside          dummy
!     lenmarker      Length of marker
!     nchar          Number of characters in a word
!     sizetxt        Size of text in cm's
!     x              x co-ordinate of point
!     x1             x-coordinate of point
!     xdown          shift in x-direction
!     xp             x-coordinate in plotting units
!     xpboun         x-coordinate of point at the boundary of the plotting
!                    region
!     y              y co-ordinate of point
!     ydigits        y shift
!     yp             y-coordinate in plotting units
!     ypboun         y-coordinate of point at the boundary of the
!                    plotting region
!     yshift         Shift in y direction (plotting coordinates)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAF01         Translate user coordinates into plotter coordinates
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLOTINTNUMBER  Plot an integer number
!     PLOTNUMBER     Plot a real number
!     PLOTTEXT1      Plot a text
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotxaxis' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotxaxis'
         write(irefwr,1) 'isigx', isigx
         write(irefwr,2) 'axhulp', axhulp
         write(irefwr,2) 'xmint, xmaxt', xmint, xmaxt
         write(irefwr,2) 'axmin, axmax, aymin, aymax', &
                          axmin, axmax, aymin, aymax
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      sizetxt = 0.2d0
      xdown = 0.4d0
      lenmarker = 0.08d0

!     --- First plot line of x-axis

      y = aymin
      call plafep (4, axmax, y, 0d0, 0d0, 0 )    ! pen up
      call plafep (5, axmin, y, 0d0, 0d0, 0 )    ! pen down

!     --- draw values and markers along the x-axis

      do i = 0, nstepx

         x = axmin+i*axhulp
         if ( debug ) write(irefwr,2) 'x, number', x, axmin+i*stepx

         call plafep ( 4, x, y, 0d0, 0d0, 0 )        ! pen up
         call plafep (16, x, y, 0d0, lenmarker, 0)   ! plot marker 0.08

!        --- Plot number

         if ( isigx==0 ) then

!        --- No digits behind decimal point, i.e. integer number

            inumbr = nint(axmin+i*stepx)
            call plotintnumber ( x, y, inumbr, sizetxt, 1, &
                                 0d0, -xdown, nchar )

         else

!        --- Digits behind decimal point, i.e. real number

            call plotnumber ( x, y, axmin+i*stepx, isigx, sizetxt, &
                              1, 0d0, -xdown, -1, nchar )

         end if  ! ( isigx==0 )

      end do  ! i = 0, nstepx

!     --- Next plot line of upper x-axis

      y = aymax
      call plafep (4, axmax, y, 0d0, 0d0, 0 )    ! pen up
      call plafep (5, axmin, y, 0d0, 0d0, 0 )    ! pen down

!     --- draw markers along the x-axis

      do i = 0, nstepx

         x = axmin+i*axhulp
         if ( debug ) write(irefwr,2) 'x, number', x, axmin+i*stepx

         call plafep ( 4, x, y, 0d0, 0d0, 0 )     ! pen up
         call plafep (16, x, y, 0d0, -0.08d0,0)   ! plot marker

      end do  ! i = 0, nstepx

      if ( expx/=0 ) then

!     --- Exponent found. plot value of exponent

         ydigits = aymin-yfct  ! digits below x-axis
         x = axmax-xfct

!        --- Plot x marker

         x1 = x+xfct*sizetxt*.35d0
         y = ydigits+yfct*sizetxt*.35d0
         call plafep ( 6, x1, y, 0.7d0*sizetxt, 0d0, 0 )

!        --- Find xp and yp corresponding to this position

         call plaf01 ( x1, ydigits, xp, yp, xpboun, ypboun, iside )
         xp = xp+sizetxt

         call plottext1 ( '10', 2, sizetxt, 2, xp, yp, 0d0, 0d0, 0d0 )

         call plotintnumber ( x1, ydigits, expx, sizetxt, 2, &
                              2d0*sizetxt, sizetxt, nchar )

      end if  ! ( expx/=0 )

!     --- Plot text

      ydigits = aymin-yfct  ! digits below x-axis
      inumbr = len_trim(textx)
      x = 0.5d0*(xmaxt+xmint-inumbr*0.15d0*xfct)
      yshift = -sizetxt

!     --- Find xp and yp corresponding to this position

      call plaf01 ( x, ydigits, xp, yp, xpboun, ypboun, iside )

      call plottext1 ( textx, inumbr, sizetxt, 2, xp, yp, 0d0, yshift, &
                       0d0 )

1000  call erclos ( 'plotxaxis' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotxaxis'

      end if  ! ( debug )

      end
