      subroutine plotyaxis ( ayhulp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-05-2014 New position of text
!        version  1.0  date 24-05-2010
!
!   copyright (c) 2010-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     plot the y-axis of a function plot
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     axis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision ayhulp

!     ayhulp         i    Defines the step along the y-axis
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inumbr, nchar, iside
      double precision ydigits, y, x, sizetxt, lenmarker, yleft, xshift, &
                       yshift, xp, yp, xpboun, ypboun, x1

!     i              Counting variable
!     inumbr         number to be plotted
!     iside          dummy
!     lenmarker      Length of marker
!     nchar          Number of characters in a word
!     sizetxt        Size of text in cm's
!     x              x co-ordinate of point
!     x1             x co-ordinate of point
!     xp             x-coordinate in plotting units
!     xpboun         x-coordinate of point at the boundary of the plotting
!                    region
!     xshift         Shift in x direction (plotting coordinates)
!     y              y co-ordinate of point
!     ydigits        y shift
!     yleft          shift of y-coordinate
!     yp             y-coordinate in plotting units
!     ypboun         y-coordinate of point at the boundary of the
!                    plotting region
!     yshift         Shift in y direction (plotting coordinates)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAF01         Translate user coordinates into plotter coordinates
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLOTINTNUMBER  Plot an integer number
!     PLOTNUMBER     Plot a real number
!     PLOTTEXT1      Plot a text
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotyaxis' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotyaxis'
         write(irefwr,1) 'isigy', isigy
         write(irefwr,2) 'ayhulp', ayhulp
         write(irefwr,2) 'ymint, ymaxt', ymint, ymaxt
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      sizetxt = 0.2d0
      lenmarker = 0.08d0
      yleft = 0.1d0

!     --- First plot line of y-axis

      x = axmin
      call plafep (4, x, aymax, 0d0, 0d0, 0 )    ! pen up
      call plafep (5, x, aymin, 0d0, 0d0, 0 )    ! pen down

!     --- draw values and markers along the x-axis

      ydigits = x-xfct*.4d0  ! digits left of y-axis
      if ( debug ) write(irefwr,2) 'ydigits', ydigits

      do i = 0, nstepy

         y = aymin+i*ayhulp
         if ( debug ) write(irefwr,2) 'y, number', y, aymin+i*stepy

         call plafep ( 4, x, y, 0d0, 0d0, 0 )          ! pen up
         call plafep ( 16, x, y, lenmarker, 0d0, 0 )   ! plot marker

!        --- Plot number

         if ( isigy==0 ) then

!        --- No digits behind decimal point, i.e. integer number

            inumbr = nint(aymin+i*stepy)
            call plotintnumber ( x, y, inumbr, sizetxt, 3, &
                                 -yleft, -0.5d0*sizetxt, nchar )

         else

!        --- Digits behind decimal point, i.e. real number

            call plotnumber ( x, y, aymin+i*stepy, isigy, sizetxt, &
                              3, -yleft, -0.5d0*sizetxt, -1, nchar )

         end if  ! ( isigy==0 )

      end do  ! i = 0, nstepy

!     --- Next plot line of right y-axis

      x = axmax
      call plafep (4, x, aymax, 0d0, 0d0, 0 )    ! pen up
      call plafep (5, x, aymin, 0d0, 0d0, 0 )    ! pen down

      if ( expy/=0 ) then

!     --- Exponent found. plot value of exponent

         ydigits = aymax+0.1*yfct  ! digits above y-axis
         x = axmin

!        --- Plot x marker

         x1 = x+xfct*sizetxt*.35d0
         y = ydigits+yfct*sizetxt*.35d0
         call plafep ( 6, x1, y, 0.7d0*sizetxt, 0d0, 0 )

!        --- Find xp and yp corresponding to this position

         call plaf01 ( x1, ydigits, xp, yp, xpboun, ypboun, iside )

         xp = xp+sizetxt

         call plottext1 ( '10', 2, sizetxt, 2, xp, yp, 0d0, 0d0, 0d0 )

         call plotintnumber ( x1, ydigits, expy, sizetxt, 2, &
                              2d0*sizetxt, sizetxt, nchar )

      end if  ! ( expx/=0 )

!     --- Plot text

      ydigits = aymax+yfct*0.5d0  ! digits above upper x-axis
      inumbr = len_trim(texty)
      x = axmin
      y = aymax

!     --- Find xp and yp corresponding to this position

      call plaf01 ( x, y, xp, yp, xpboun, ypboun, iside )

      xshift = -0.5d0
      yshift = 0.5d0
      call plottext1 ( texty, inumbr, sizetxt, 1, xp, yp, xshift, &
                       yshift, 0d0 )

      x = axmin
      y = 0.5d0*(aymax+aymin)

!     --- Find xp and yp corresponding to this position

      call plaf01 ( x, y, xp, yp, xpboun, ypboun, iside )

      xshift = -nchar*sizetxt
      yshift = 0d0
      call plottext1 ( texty, inumbr, sizetxt, 1, xp, yp, xshift, &
                       yshift, 90d0 )

1000  call erclos ( 'plotyaxis' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotyaxis'

      end if  ! ( debug )

      end
