      subroutine plscal
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 01-09-2006 Debug information
!        version  1.0  date 11-11-1990
!
!   copyright (c) 1990-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!        Plot subroutine to plot the scale fators
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     scale
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=18) charac
      double precision scalex, scaley, xp

!     charac         Help variable to store a number in character form
!     scalex         Scale factor in x-direction
!     scaley         Scale factor in y-direction
!     xp             Plotting x-position in cm's
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFP7         Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plscal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plscal'

      end if  ! ( debug )
      if ( ierror/=0 .or. jkader>=0 .or. jkader<=-5 ) go to 1000

!     --- Plot scales:

      scalex = 1d0 / xfct
      scaley = 1d0 / yfct
      xp = max ( 16d0, ale+5.0d0 )
      if ( scalex<1d-3 .or. scalex>1d3 .or. scaley<1d-3 .or. &
           scaley>1d3 ) then

!     --- Plot in floating precision

         charac(1:8) = 'scalex: '
         write ( charac(9:18),100 ) scalex
100      format(ss,1pe10.3e2)
         call plafp7 ( xp, 1.0d0, charac, 0d0, 0.25d0, 18 )
         charac(1:8) = 'scaley: '
         write ( charac(9:18),100 ) scaley
         call plafp7 ( xp, 1.5d0, charac, 0d0, 0.25d0, 18 )
      else

!     --- Plot in fixed precision

         charac(1:8) = 'scalex: '
         write ( charac(9:18),110 ) scalex
110      format(f10.3)
         call plafp7 ( xp, 1.0d0, charac, 0d0, 0.25d0, 18 )
         charac(1:8) = 'scaley: '
         write ( charac(9:18),110 ) scaley
         call plafp7 ( xp, 1.5d0, charac, 0d0, 0.25d0, 18 )
      end if

      if ( itimdep/=0 ) then

!     --- Time dependent solution, plot time

         if ( abs(t)>0d0 .and. t<1d-3 .or. t>1d3 ) then

!        --- Plot in floating precision

            charac(1:8) = 'time t: '
            write ( charac(9:18),100 ) t
            call plafp7 ( xp, 0.5d0, charac, 0d0, 0.25d0, 18 )

         else

!        --- Plot in fixed precision

            charac(1:8) = 'time t: '
            write ( charac(9:18),110 ) t
            call plafp7 ( xp, 0.5d0, charac, 0d0, 0.25d0, 18 )

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plscal'

      end if  ! ( debug )
      call erclos ( 'plscal' )

      end
