      subroutine plsepps
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 02-09-2013 Change of maximum number of plots
!        version  1.4  date 22-08-2013 Change loop
!        version  1.3  date 25-03-2005 Remove /mtp from file name
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Translate all sepran plot file sepplot.xxx into a set of postcript files
!   of the name sepposc.xxx.eps
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulemach
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished, check
      integer numfil, iseq, numdigits
      real xminrd, xmaxrd, yminrd, ymaxrd
      character (len=60) namefl
      integer lastps

!     check          Indicates if the file exist (true) or not
!     finished       Indicates if the loop has finished (true)
!                    or not (false)
!     iseq           Sequence number of plot
!     lastps         Last non-blank position in namefl
!     namefl         Name of default SEPRAN neutral plotting file
!     numdigits      Number of digits in plot file
!     numfil         Number of plot files found
!     xmaxrd         Maximum x-value read in file
!     xminrd         Minimum x-value read in file
!     ymaxrd         Maximum y-value read in file
!     yminrd         Minimum y-value read in file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     FINISH         Stop SEPRAN
!     INIFIL         Open files
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITDF         Set defaults for program SEPPOST
!     INITMD         Set machine-dependent quantities
!     PLAFPS         read a sepran neutral plot file and translate the contents
!                    to postscript format
!     PLAFSCANFILE   Read all records from a sepran neutral file and detect
!                    minimum and maximum x and y values
!     SEPSTART       Standard SEPRAN starting subroutine
!     STARTSEPRAN    Create buffer array and fill part of sepmodulebuf
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1669   file does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
!   start SEPRAN
!   finished := false
!   namefl := namepl
!   numfil := Number of files namefl.***
!   if ( numfil=0 ) then
!      print error
!   Translate files
!   Close SEPRAN
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false.
   1  format ( a, 1x, (10i6) )

!     --- Start SEPRAN

      if ( startcalled/=99991 ) call startsepran
      call initcb
      call initmd

!     --- Open read and write files

      irefre = 5
      irefwr = 6

!     --- Open standard message file and error message file

      call inifil(4)

!     --- Standard SEPRAN start subroutine

      call sepstart ( 10100, 1, -1, 0 )
      ioutp = -1
      call eropen ( 'seppostscript' )
      call eropen ( 'plsepps' )

!     --- Set defaults

      call initdf
      posdev(1:2) = devdis(1:2)
      iplotf = 0

!     --- Compute number of present SEPRAN neutral plotting files

      lastps = index ( namepl, ' ' ) - 1

      if ( namepl(1:5)=='/tmp/' ) then

!     --- namepl starts with /tmp
!         remove first part of name

         namepl(1:lastps-20) = namepl(21:lastps)
         lastps = lastps-20
         namepl(lastps+1:100) = ' '

      end if  ! ( namepl(1:5)=='/tmp/' )
      if ( debug ) write(irefwr,*) 'namepl ', namepl(1:lastps)

      finished = .false.

!     --- While loop finished

      numfil = 0
      namefl = ' '
      namefl(1:lastps+4) = namepl(1:lastps)//'.001'

      if ( debug ) write(irefwr,*) 'namefl ', namefl(1:lastps+4)

      numdigits = 3

      inquire ( file=namefl(1:lastps+4), exist=check )

      if ( debug ) write(irefwr,*) 'check ', check

      if ( .not.check ) then

!     --- The file does not exist, perhaps the extension is .0001

         namefl(1:lastps+5) = namepl(1:lastps)//'.0001'
         inquire ( file=namefl(1:lastps+5), exist=check )
         if ( check ) numdigits = 4

         if ( debug ) then
            write(irefwr,*) 'namefl ', namefl(1:lastps+5 )
            write(irefwr,*) 'check ', check
         end if  ! ( debug )

      end if  ! ( .not.check )

      if ( debug ) write(irefwr,1) 'numdigits', numdigits

      do while ( .not. finished )

         numfil = numfil + 1
         if ( numdigits==3 ) then
            write(namefl(lastps+2:lastps+4), 110 ) numfil
110         format ( i3.3 )
            inquire ( file=namefl(1:lastps+4), exist=check )
         else
            write(namefl(lastps+2:lastps+5), 120 ) numfil
120         format ( i4.4 )
            inquire ( file=namefl(1:lastps+5), exist=check )
         end if  ! ( numdigits==3 )
         if ( .not. check ) then

!        --- file does not exist, decrease numfil

            numfil = numfil - 1
            finished = .true.

         end if

      end do  ! while ( .not. finished )

      if ( numdigits==3 ) then
         namefl(1:lastps+4) = namepl(1:lastps)//'.001'
      else
         namefl(1:lastps+5) = namepl(1:lastps)//'.0001'
      end if  ! ( numdigits==3 )

      if ( numfil==0 ) then

!     --- Error: no file namefl found

         call errchr ( namefl, 1 )
         call errsub ( 1669, 0, 0, 1 )
         go to 1000

      end if

!     --- Scan and plot file

      posdev = 'pc'
      do iseq = 1, numfil
         call plafscanfile ( iseq, xminrd, xmaxrd, yminrd, ymaxrd, &
                             numdigits )
      end do
      do iseq = 1, numfil
         call plafps ( iseq, xminrd, xmaxrd, yminrd, ymaxrd, numdigits )
      end do

!     --- write one line to file  mpeg_input.2

      open ( unit=31, file = 'mpeg_input.2' )
      write(31,200) numfil
200   format ( 'sepposc.*.ppm [001-',i3.3,']' )
      close ( 31 )
1000  call erclos ( 'plsepps' )
      call erclos ( 'seppostscript' )

!     --- Close SEPRAN

      call finish ( 0 )

      end subroutine plsepps
