      subroutine plt353 ( v,ip,jp,kp,lp,mp,coor,procor,ivis,tst,ten, &
                          nprcor,isitua )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.0  date 21-04-1995 Coordinate basis added, &
!                                      extra parameter
!        version  4.0  date 30-12-1992 Extra parameter nprcor
!        version  3.4  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  3.3  date 24-01-1991 error in ityp=10 removed
!        version  3.2  date 03-06-1990 hard boundaries for t-values in
!                                      type = 9 and 11
!        version  3.1  date 24-01-1989
!
!   copyright (c) 1989-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine whether part of line is covered by triangle kp,lp,mp in
!     projection-plane.
! **********************************************************************
!
!                       KEYWORDS
!
!     hidden_line
!     covering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ip, jp, kp, lp, mp, ivis, nprcor, isitua
      double precision coor(3,*), procor(nprcor,*), v(*), tst, ten

!     coor    i    array of length npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     ip      i    nodal point numbers of line to be considered
!     isitua  i    indicator for coordinate basis
!                    1 :   x - y - z
!                    2 :   z - x - y
!                    3 :   y - z - x
!     ivis    o    indicator whether line is
!                  visible                     :  ivis = 1
!                  not visible                 :  ivis = 2
!                  begin or end part visible   :  ivis = 3
!                  middle part not visible     :  ivis = 4
!     jp      i    nodal point numbers of line to be considered
!     kp      i    nodes of triangle to be considered
!     lp      i    nodes of triangle to be considered
!     mp      i    nodes of triangle to be considered
!     nprcor  i    Dimension of first index of procor
!                  If nprcor = 3, pl0014 is called by a colour subroutine, &
!                  which implies that also the distance to the view-point is
!                  computed, if nprcor = 2, only the projection is computed
!     procor  i    coordinate array for projected points
!                  procor(1,i) contains the x-coordinate of the projected point
!                  procor(2,i) contains the y-coordinate of the projected point
!                  procor(3,i) contains the distance of point to the view-point
!                  i is the point number
!     ten     i    t value end of visible part
!     tst     i    t value start of visible part
!     v       i    array containing the co-ordinates of the view point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps, x1, y1, x2, y2, x3, y3, surf, xa, ya, xb, &
                       yb, xmintr, xmaxtr, ymintr, ymaxtr, xminli, &
                       xmaxli, yminli, ymaxli, ski1a, ski2a, ski3a, &
                       ski1b, ski2b, ski3b, aps, skmina, skminb, xp, yp, &
                       zp, t1, t2, t3, tmax, tmin, xaa, yaa, zaa, xbb, &
                       ybb, zbb, tmid
      integer itypa, itypb, ityp, jes, isnij, isn, inum, in(3)

!     aps     local accuracy
!     eps     local accuracy
!     in      orderings array coordinates
!     inum
!     isn     ?
!     isnij   ?
!     ityp    ?
!     itypa   ?
!     itypb   ?
!     jes     ?
!     ski1a   ?
!     ski1b   ?
!     ski2a   ?
!     ski2b   ?
!     ski3a   ?
!     ski3b   ?
!     skmina  ?
!     skminb  ?
!     surf    ?
!     t1      First component to compute the tangential vector
!     t2      Second component to compute the tangential vector
!     t3      ?
!     tmax    ?
!     tmid    ?
!     tmin    ?
!     x1      x-coordinate of first point in curve
!     x2      x-coordinate of last point in curve
!     x3      ?
!     xa      ?
!     xaa     ?
!     xb      ?
!     xbb     ?
!     xmaxli  ?
!     xmaxtr  ?
!     xminli  ?
!     xmintr  ?
!     xp      ?
!     y1      y-coordinate of first point in curve
!     y2      y-coordinate of last point in curve
!     y3      ?
!     ya      ?
!     yaa     ?
!     yb      ?
!     ybb     ?
!     ymaxli  ?
!     ymaxtr  ?
!     yminli  ?
!     ymintr  ?
!     yp      ?
!     zaa     ?
!     zbb     ?
!     zp      ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLT354  ?
!     PLT355     Compute area coordinates of point in triangle
!     PLT356     Check whether point is real intersection point
!     PLT357     Compute intersection points of line and triangle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      eps =  10 * epsmac

!     --- Determine basic ordering

      if ( isitua.eq.2 ) then

!     --- System is    z - x - y

         in(1) = 3
         in(2) = 1
         in(3) = 2
      else if ( isitua.eq.3 ) then

!     --- System is    y - z - x

         in(1) = 2
         in(2) = 3
         in(3) = 1
      else

!     --- System is    x - y - z (which is also default)

         in(1) = 1
         in(2) = 2
         in(3) = 3
      end if

!     --- Determine projected points

      x1 = procor(1,kp)
      y1 = procor(2,kp)
      x2 = procor(1,lp)
      y2 = procor(2,lp)
      x3 = procor(1,mp)
      y3 = procor(2,mp)

!     --- Compute surface triangle in projection plane

      surf = abs ( x1*(y2-y3) + x2*(y3-y1) + x3*(y1-y2) )
      if ( surf.lt.eps ) then
           ivis = 1
           return
      end if

!     --- Points A and B

      xa =  procor(1,ip)
      ya =  procor(2,ip)
      xb =  procor(1,jp)
      yb =  procor(2,jp)

!     --- Check whether intersection is possible

      xmintr = min( x1,x2,x3 )
      xmaxtr = max( x1,x2,x3 )
      ymintr = min( y1,y2,y3 )
      ymaxtr = max( y1,y2,y3 )
      ivis = 1
      xminli = min( xa,xb )
      if ( xminli.ge.xmaxtr ) return
      xmaxli = max( xa,xb )
      if ( xmaxli.le.xmintr ) return
      yminli = min( ya,yb )
      if ( yminli.ge.ymaxtr ) return
      ymaxli = max( ya,yb )
      if ( ymaxli.le.ymintr ) return

!     --- Compute area coordinates of A and B

      call plt355(x1,x2,x3,y1,y2,y3,xa,ya,ski1a,ski2a,ski3a)
      call plt355(x1,x2,x3,y1,y2,y3,xb,yb,ski1b,ski2b,ski3b)
      aps = 1d0 + eps
      if ( ski1a.lt.-eps .and. ski1b.lt.-eps ) return
      if ( ski1a.gt. aps .and. ski1b.gt. aps ) return
      if ( ski2a.lt.-eps .and. ski2b.lt.-eps ) return
      if ( ski2a.gt. aps .and. ski2b.gt. aps ) return
      if ( ski3a.lt.-eps .and. ski3b.lt.-eps ) return
      if ( ski3a.gt. aps .and. ski3b.gt. aps ) return
      skmina = min (ski1a,ski2a,ski3a)
      skminb = min (ski1b,ski2b,ski3b)
      itypa = 1
      if ( skmina.lt.0d0 ) itypa = 3
      if ( (ip.eq.kp) .or. (ip.eq.lp) .or. (ip.eq.mp) ) itypa=2
      itypb = 1
      if ( skminb.lt.0d0 ) itypb = 3
      if ( (jp.eq.kp) .or. (jp.eq.lp) .or. (jp.eq.mp) ) itypb=2
      ityp  = itypa + 3 * itypb

!     --- Run through all possibilities

      if ( ityp.eq.8 ) then
           ivis = 1
           return
      else if ( ityp.eq.4 ) then
           call plt354(v,coor,kp,lp,mp,ip,xp,yp,zp,jes,in)
           ivis = 2 - jes
           return
      else if ( ityp.eq.5 ) then
           call plt354(v,coor,kp,lp,mp,jp,xp,yp,zp,jes,in)
           ivis = 2 - jes
           return
      else if ( ityp.eq.7 ) then
           call plt354(v,coor,kp,lp,mp,ip,xp,yp,zp,jes,in)
           ivis = 2 - jes
           return
      else if ( ityp.eq.10 ) then
           call plt354(v,coor,kp,lp,mp,ip,xp,yp,zp,jes,in)
           if ( jes.eq.1 ) then
                ivis = 1
                return
           end if
      else if ( ityp.eq.6 ) then
           call plt354(v,coor,kp,lp,mp,jp,xp,yp,zp,jes,in)
           if ( jes.eq.1 ) then
                ivis = 1
                return
           end if
      end if

!     --- Line is partially (?) visible, compute intersection points
!         Edge 1

      call plt357(x2,x3,y2,y3,xa,xb,ya,yb,t1)

!     --- Edge 2

      call plt357(x3,x1,y3,y1,xa,xb,ya,yb,t2)

!     --- Edge 3

      call plt357(x1,x2,y1,y2,xa,xb,ya,yb,t3)
      tmax = max (t1,t2,t3)
      tmin = min (t1,t2,t3)
      if ( ityp.eq.10 ) then

!     --- There is one and only one intersection point

           if ( t1.lt.0d0 ) t1 = 1d3
           if ( t2.lt.0d0 ) t2 = 1d3
           if ( t3.lt.0d0 ) t3 = 1d3
           tst  = min( t1,t2,t3 )
           ivis = 3
           ten  = 1d0
      else if ( ityp.eq.6 ) then

!     --- There is one and only one intersection point

           if ( t1.gt.1d0 ) t1 = -1d3
           if ( t2.gt.1d0 ) t2 = -1d3
           if ( t3.gt.1d0 ) t3 = -1d3
           tst  = 0d0
           ten  = max( t1,t2,t3 )
           ivis =  3
      else if ( ityp.eq.11 ) then
           if ( tmax.gt.1.01d0 .or. tmin.lt.-.01d0 ) then
                ivis = 1
           else

!          --- Check whether point is real intersection point

                isnij=0
                call plt356(tmax,xa,xb,ya,yb,x1,x2,x3,y1,y2,y3,isnij)
                if ( isnij.eq.0 ) then
                     ivis = 1
                     return
                end if

!               --- Check visibility of middle point

                xaa  =  coor( in(1), ip )
                yaa  =  coor( in(2), ip )
                zaa  =  coor( in(3), ip )
                xbb  =  coor( in(1), jp )
                ybb  =  coor( in(2), jp )
                zbb  =  coor( in(3), jp )
                xp   =  xaa + tmax * ( xbb-xaa ) / 2d0
                yp   =  yaa + tmax * ( ybb-yaa ) / 2d0
                zp   =  zaa + tmax * ( zbb-zaa ) / 2d0
                call plt354(v,coor,kp,lp,mp,0,xp,yp,zp,jes,in)
                if ( jes.eq.1 ) then
                     ivis = 1
                     return
                end if
                ivis = 3
                tst  = tmax
                ten  = 1d0
           end if
      else if ( ityp.eq.9 ) then
           if ( tmax.gt.1.01d0 .or. tmin.lt.-.01d0 ) then
                ivis = 1
           else

!          --- Check whether point is real intersection point

                  isnij = 0
                  call plt356(tmin,xa,xb,ya,yb,x1,x2,x3,y1,y2,y3,isnij)
                  if ( isnij.eq.0 ) then
                       ivis = 1
                       return
                  end if

!                 --- Check visibility of middle point

                  xaa  =  coor( in(1), ip )
                  yaa  =  coor( in(2), ip )
                  zaa  =  coor( in(3), ip )
                  xbb  =  coor( in(1), jp )
                  ybb  =  coor( in(2), jp )
                  zbb  =  coor( in(3), jp )
                  xp =  xaa + ( 1d0 + tmin ) * ( xbb-xaa ) / 2d0
                  yp =  yaa + ( 1d0 + tmin ) * ( ybb-yaa ) / 2d0
                  zp =  zaa + ( 1d0 + tmin ) * ( zbb-zaa ) / 2d0
                  call plt354(v,coor,kp,lp,mp,0,xp,yp,zp,jes,in)
                  if ( jes.eq.1 ) then
                       ivis = 1
                       return
                  end if
                  ivis = 3
                  tst  = 0d0
                  ten  = tmin
           end if
      else if ( ityp.eq.12 ) then

!     --- Compute real intersection points

           isnij = 0
           call plt356(t1,xa,xb,ya,yb,x1,x2,x3,y1,y2,y3,isnij)
           if ( isnij.eq.0 ) then
                t1  = -1d0
                isn = 0
           else
                isn = 1
           end if
           isnij = 0
           call plt356(t2,xa,xb,ya,yb,x1,x2,x3,y1,y2,y3,isnij)
           if ( isnij.eq.0 ) then
                t2 = -1d0
           else
                isn = isn + 1
           end if
           isnij = 0
           call plt356(t3,xa,xb,ya,yb,x1,x2,x3,y1,y2,y3,isnij)
           if ( isnij.eq.0 ) then
                t3 = -1d0
           else
                isn = isn + 1
           end if
           if ( isn.eq.0 ) then
                ivis = 1
                return
           else if ( isn.eq.1 ) then

!          --- " Impossible situation "
!                Probable cause : Collinear lines !

                ivis = 1
                return
           else if ( isn.eq.2 ) then
                ivis = 4
                inum = 0
                if ( t1.gt.-5d-1 ) inum = 1
                if ( t2.gt.-5d-1 ) then
                     if ( inum.eq.0 ) then
                          tst = min(t2,t3)
                          ten = max(t2,t3)
                     else
                          tst = min(t1,t2)
                          ten = max(t1,t2)
                     end if
                     inum = 2
                end if
                if ( inum.eq.1 ) then
                     tst = min(t1,t3)
                     ten = max(t1,t3)
                end if
           else if ( isn.eq.3 ) then

!          --- Very special case : Exactly through a cornerpoint

                ivis = 4
                tst  = tmin
                ten  = tmax
           end if

!          --- Check visibility of middle point

           tmid = ( tst + ten ) / 2d0
           xaa  =   coor( in(1), ip )
           yaa  =   coor( in(2), ip )
           zaa  =   coor( in(3), ip )
           xbb  =   coor( in(1), jp )
           ybb  =   coor( in(2), jp )
           zbb  =   coor( in(3), jp )
           xp   =   xaa + tmid * ( xbb - xaa )
           yp   =   yaa + tmid * ( ybb - yaa )
           zp   =   zaa + tmid * ( zbb - zaa )
           call plt354(v,coor,kp,lp,mp,0,xp,yp,zp,jes,in)
           if ( jes.eq.1 ) ivis = 1
      end if
      end
