      subroutine pltbn2 ( ichois, icurvs, curves, ncurvs, plotfm, yfact, &
                          nuspnt, rinput, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.6  date 21-05-2001 New call to pl0001
!        version  5.5  date 29-09-2000 Array bound icurvs
!        version  5.4  date 26-09-1997 Skip subcurves of spline curve
!
!   copyright (c) 1982-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    this subroutine plots the curves and the user points of the mesh
!    when ndim = 2
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     curve
!     plot
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, icurvs(*), nuspnt, ichois, iwork(*)
      double precision curves(2,*), plotfm, yfact, rinput(2,nuspnt)

!     curves   i   array containing the co-ordinates of the curve points
!     ichois   i   Choice parameter indicating what has to be plotted
!                  by this subroutine
!                  ICHOIS consists of 3 parts iplcur, iplnod and iplusp
!                  according to
!                  ICHOIS = iplcur + 10*iplnod + 100*iplusp + 1000*allcurvs
!                  with:
!                  iplcur:  Information of the curves to be plotted
!                    0  No plot of the curve
!                    1  Plot of the curve without curve numbers
!                    2  Plot of the curve with curve numbers
!                  iplnod:  Information of the nodal points to be plotted
!                    0  No plot of the nodal points
!                    1  Plot of the nodal points using crosses
!                    >1 Plot of the curve with symbols (symbol number ichois-1)
!                  iplusp:  Information of the user points to be plotted
!                    0  No plot of the user points
!                    1  Plot of the user points without numbers
!                    2  Plot of the user points with numbers
!                  allcurvs = 0, means use array iwork
!                  allcurvs = 1, means use all curves
!     icurvs   i   array containing the number of nodal points
!                  in the curves  ( accumulated )
!     iwork    i   Array of length ncurvs
!                  If iwork(i)=0 the curve must be plotted, &
!                     iwork(i)=1 not
!     ncurvs   i   number of curves
!     nuspnt   i   Number of user points
!     plotfm   i   length of the plot in centimeters
!     rinput   i   Array containing the coordinates of the user points
!     yfact    i   multiplication factor for the y-coordinates
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurnr, j, ikader, ncount, iuppl, n, iplcur, iplnod, &
              iplusp, iscolr
      double precision x, y, s, sprec, smean, xpr, ypr
      logical plotcurv

!     icurnr    Curve number
!     ikader    Old value of jkader (temporary storage)
!     iplcur    Indication of how the curve must be plotted, see ICHOIS
!     iplnod    Indication of how the nodes must be plotted, see ICHOIS
!     iplusp    Indication of how the user points must be plotted, see ICHOIS
!     iscolr    Help variable to store the old value of dcolor
!     iuppl     Indication if the pen must move to a point with pen up
!               (4) or down (#4)
!     j         Counting variable
!     n         Number of points in the curves that are plotted until now, &
!               plus present curve
!     ncount    Number of points in the curves that are plotted until now
!     plotcurv  If true the curve must be plotted
!     s         Computed actual length of a curve
!     smean     Half the length of the complete curve
!     sprec     Computed length of the curve until present element
!     x         x-Co-ordinates of a point
!     xpr       x-Co-ordinates of preceding point on curve
!     y         y-Co-ordinates of a point
!     ypr       y-Co-ordinates of preceding point on curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PL0000  Compute length and width of plot
!     PL0001  Compute xmin, xmax, ymin and ymax
!     PLAFEP  Actual plotting subroutine
!     PLAFP3  Plot symbols
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pltbn2' )
      iscolr = dcolor

!     --- compute iplcur, iplnod, iplusp, xmin, xmax, ymin, ymax, aleng
!         and width

      iplcur = mod ( ichois,10 )
      iplnod = mod ( ichois/10,10 )
      iplusp = mod ( ichois/100,10 )
      ikader = jkader
      if(jkader.ge.0) jkader = 4
      ncount = icurvs(ncurvs)
      if ( mod(jtimes,10).le.1 ) then
         if ( jmax.eq.0 ) &
            call pl0001 ( curves, 2, ncount, xmin, xmax, ymin, ymax )

         call pl0000 ( plotfm, yfact )

!        --- start picture

         call plafep ( 1, aleng, width, 0d0, 0d0, jmax )

      end if

!     --- plot curves

      if ( iplcur.gt.0 .or. iplnod.gt.0 ) then

!     --- Plot curves and/or nodal points along curves

         ncount = 1
         do icurnr = 1, ncurvs
            n = icurvs(icurnr)
            s = 0d0
            iuppl = 4
            plotcurv = ichois.gt.999
            if ( .not. plotcurv ) plotcurv = iwork(icurnr).eq.0
            if ( plotcurv ) plotcurv = icurvs(ncurvs+icurnr).ne.2
            if ( plotcurv ) then

!           --- plot curve

               do j = ncount, n
                  if ( iscolr.gt.0 ) dcolor = 1+iscolr
                  if ( iplcur.gt.0 ) call plafep ( iuppl, curves(1,j), &
                       curves(2,j), 0d0, 0d0, 0 )
                  if ( iplnod.eq.1 ) then
                     if ( iscolr.gt.0 ) dcolor = 2+iscolr
                     call plafep ( 6, curves(1,j), curves(2,j), &
                                   0d0, 0d0, 0 )
                  else if ( iplnod.gt.1 ) then
                     if ( iscolr.gt.0 ) dcolor = 2+iscolr
                     call plafp3 ( iplnod-1, curves(1,j), curves(2,j) )
                  end if
                  iuppl = 5
               end do

               if ( iplcur.eq.2 ) then

!              --- iplcur = 2, Plot curve numbers
!                  compute length of curve

                  dcolor = iscolr
                  do j = ncount, n
                     x = curves(1,j)
                     y = curves(2,j)
                     if ( j.eq.ncount ) then

!                    --- j = ncount, set s equal to zero

                        s = 0d0

                     else

!                    --- Not the first point, raise s

                        s = s + sqrt ( (xpr-x)**2 +(ypr-y)**2 )

                     end if

                     xpr = x
                     ypr = y

                  end do

                  smean = s * 0.5d0

!                 --- Find element in the center of the boundary

                  do j = ncount, n
                     x = curves(1,j)
                     y = curves(2,j)
                     if ( j.eq.ncount ) then

!                    --- j = ncount, set s equal to zero

                        s     = 0d0
                        sprec = 0d0

                     else

!                    --- Not the first point, raise s

                         s = s + sqrt ( (xpr-x)**2 +(ypr-y)**2 )

                     end if

                     if ( s.gt.smean ) then

!                    --- s > smean, center element found

                        s = (smean-sprec) / (s-sprec)
                        x = xpr + s * (x-xpr)
                        y = ypr + s * (y-ypr)
                        call plafep ( 9, x, y, 0.2d0, 0.2d0, icurnr )
                        go to 150

                     end if

                     sprec = s
                     xpr = x
                     ypr = y

                  end do

150               continue

               end if

            end if
            ncount = n+1

         end do

      end if

      if ( iplusp.gt.0 ) then

!     --- Plot user points

         if ( iscolr.gt.0 ) dcolor = 4+iscolr
         do j = 1, nuspnt
            call plafep ( 6, rinput(1,j), rinput(2,j), 0d0, 0d0, 0 )
         end do

      end if

      if ( iplusp.gt.1 ) then

!     --- Plot numbers at user points

         if ( iscolr.gt.0 ) dcolor = iscolr
         do j = 1, nuspnt
            call plafep ( 9, rinput(1,j), rinput(2,j), 0.2d0, 0.2d0, j )
         end do

      end if

!     --- close plot file

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
      jkader = ikader
      call erclos ( 'pltbn2' )
      dcolor = iscolr

      end
