      subroutine pltbn3 ( ichois, icurvs, curves, ncurvs, plotfm, yfact, &
                          nuspnt, rinput )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.2  date 18-02-2000 Shift coordinates before plotting
!        version  2.1  date 22-07-1998 Include common
!        version  2.0  date 21-08-1992 Complete new parameter list
!        version  1.0  date 10-08-1989
!
!   copyright (c) 1989-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine to plot the projection on a plane of the
!   constituting curves of the mesh when ndim = 3
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     curve
!     plot
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, icurvs(ncurvs), nuspnt, ichois
      double precision curves(3,*), plotfm, yfact, rinput(3,nuspnt)

!     curves   i   array containing the co-ordinates of the curve points
!     ichois   i   Choice parameter indicating what has to be plotted
!                  by this subroutine
!                  ICHOIS consists of 3 parts iplcur, iplnod and iplusp
!                  according to
!                  ICHOIS = iplcur + 10*iplnod + 100*iplusp
!                  with:
!                  iplcur:  Information of the curves to be plotted
!                    0  No plot of the curve
!                    1  Plot of the curve without curve numbers
!                    2  Plot of the curve with curve numbers
!                  iplnod:  Information of the nodal points to be plotted
!                    0  No plot of the nodal points
!                    1  Plot of the nodal points using crosses
!                    >1 Plot of the curve with symbols (symbol number ichois-1)
!                  iplusp:  Information of the user points to be plotted
!                    0  No plot of the user points
!                    1  Plot of the user points without numbers
!                    2  Plot of the user points with numbers
!     icurvs   i   array containing the number of nodal points
!                  in the curves  ( accumulated )
!     ncurvs   i   number of curves
!     nuspnt   i   Number of user points
!     plotfm   i   length of the plot in centimeters
!     rinput   i   Array containing the coordinates of the user points
!     yfact    i   multiplication factor for the y-coordinates
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurve, j, ikader, ncount, iuppl, n, iplcur, iplnod, &
              iplusp, iscolr, i
      double precision x, y, s, xprc, yprc, sprec, smean, xminp, xmaxp, &
                       z, xl, yl, zl, zprc, yminp, ymaxp, xminsv, &
                       yminsv, zminsv, xmaxsv, ymaxsv, zmaxsv

!     i         General loop varibale
!     icurve    Curve number
!     ikader    Old value of jkader (temporary storage)
!     iplcur    Indication of how the curve must be plotted, see ICHOIS
!     iplnod    Indication of how the nodes must be plotted, see ICHOIS
!     iplusp    Indication of how the user points must be plotted, see ICHOIS
!     iscolr    Help variable to store the old value of dcolor
!     iuppl     Indication if the pen must move to a point with pen up
!               (4) or down (#4)
!     j         Counting variable
!     n         Number of points in the curves that are plotted until now, &
!               plus present curve
!     ncount    Number of points in the curves that are plotted until now
!     s         Computed actual length of a curve
!     smean     Half the length of the complete curve
!     sprec     Computed length of the curve until present element
!     x         x-Co-ordinates of a point
!     xl        x-Co-ordinates of a point (after projection)
!     xmaxp     maximum x-co-ordinate of projection
!     xmaxsv    Help variable to save the value of xmax
!     xminp     minimum x-co-ordinate of projection
!     xminsv    Help variable to save the value of xmin
!     xprc      x-Co-ordinates of preceding point on curve
!     y         y-Co-ordinates of a point
!     yl        y-Co-ordinates of a point
!     ymaxp     maximum y-co-ordinate of projection
!     ymaxsv    Help variable to save the value of ymax
!     yminp     minimum y-co-ordinate of projection
!     yminsv    Help variable to save the value of ymin
!     yprc      y-Co-ordinates of preceding point on curve
!     z         z-Co-ordinates of a point
!     zl        z-Co-ordinates of a point (after projection)
!     zmaxsv    Help variable to save the value of zmax
!     zminsv    Help variable to save the value of zmin
!     zprc      z-Co-ordinates of preceding point on curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PL0000  Compute length and width of plot
!     PLAFEP  Actual plotting subroutine
!     PLAFP3  Plot symbols
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pltbn3' )
      iscolr = dcolor

!     --- compute iplcur, iplnod, iplusp, xmin, xmax, ymin, ymax, aleng and
!         width

      iplcur = mod ( ichois,10 )
      iplnod = mod ( ichois/10,10 )
      iplusp = mod ( ichois/100,10 )
      ikader = jkader
      if ( jkader.ge.0 ) jkader = 4

!     --- Compute the minimum and maximum values of x, y and z

       if ( jmax.eq.0 ) then

!      --- compute xmin, xmax, ymin, ymax, aleng and width

         ncount = icurvs(ncurvs)
         xminsv =  rinfin
         yminsv =  xminsv
         zminsv =  xminsv
         xmaxsv = -xminsv
         ymaxsv =  xmaxsv
         zmaxsv =  xmaxsv

         do i = 1 , ncount

            x  =  curves(1,i)
            y  =  curves(2,i)
            z  =  curves(3,i)

            xminsv = min ( xminsv, x )
            yminsv = min ( yminsv, y )
            zminsv = min ( zminsv, z )
            xmaxsv = max ( xmaxsv, x )
            ymaxsv = max ( ymaxsv, y )
            zmaxsv = max ( zmaxsv, z )

         end do
         xmin = 0d0
         ymin = 0d0
         zmin = 0d0
         xmax = xmaxsv-xminsv
         ymax = ymaxsv-yminsv
         zmax = zmaxsv-zminsv

!        --- Compute extreme values of projection

         xminp = xmin + 0.77d0 * ymin
         yminp = zmin + 0.45d0 * ymin
         xmaxp = xmax + 0.77d0 * ymax
         ymaxp = zmax + 0.45d0 * ymax

!        --- Place values in x-y system

         xmin  = xminp - ( xmaxp-xminp ) / 10d0
         ymin  = yminp - ( ymaxp-yminp ) / 10d0
         xmax  = xmaxp + ( xmaxp-xminp ) / 10d0
         ymax  = ymaxp + ( ymaxp-yminp ) / 10d0

      else

!     --- Do not compute xmin, xmax, ymin, ymax

         xminsv = 0d0
         yminsv = 0d0
         zminsv = 0d0

      end if

      call pl0000 ( plotfm, yfact )

!     --- start picture

      call plafep ( 1, aleng, width, 0d0, 0d0, 0 )

!     --- plot curves

      if ( iplcur.gt.0 .or. iplnod.gt.0 ) then

!     --- Plot curves and/or nodal points along curves

         ncount = 1

         do icurve = 1 , ncurvs

            n = icurvs(icurve)
            iuppl = 4

            do j = ncount , n

!           --- compute projection

               xl = curves(1,j)-xminsv
               yl = curves(2,j)-yminsv
               zl = curves(3,j)-zminsv

               xl = xl + 0.77d0 * yl
               zl = zl + 0.45d0 * yl

               if ( iscolr.gt.0 ) dcolor = 1+iscolr
               if ( iplcur.gt.0 ) call plafep ( iuppl, xl, zl, &
                    0d0, 0d0, 0 )
               if ( iplnod.eq.1 ) then
                  if ( iscolr.gt.0 ) dcolor = 2+iscolr
                  call plafep ( 6, xl, zl, 0d0, 0d0, 0 )
               else if ( iplnod.gt.1 ) then
                  if ( iscolr.gt.0 ) dcolor = 2+iscolr
                  call plafp3 ( iplnod-1, xl, zl )
               end if
               iuppl = 5
            end do

            if ( iplcur.eq.2 ) then

!           --- iplcur = 2, Plot curve numbers
!               compute length of curve

               do j = ncount, n

                  x = curves(1,j)-xminsv
                  y = curves(2,j)-yminsv
                  z = curves(3,j)-zminsv

                  if ( j.eq.ncount ) then

!                 --- j = ncount, set s equal to zero

                       s = 0d0

                  else

!                 --- Not the first point, raise s

                       s = s+sqrt((xprc-x)**2+(yprc-y)**2+(zprc-z)**2)

                  end if

                  xprc = x
                  yprc = y
                  zprc = z

               end do

               smean = s * 0.5d0

!              --- Find element in the center of the boundary

               do j = ncount , n

                  x = curves(1,j)-xminsv
                  y = curves(2,j)-yminsv
                  z = curves(3,j)-zminsv

                  if ( j.eq.ncount ) then

!                 --- j = ncount, set s equal to zero

                       s     = 0d0
                       sprec = 0d0

                  else

!                 --- Not the first point, raise s

                       s = s+sqrt((xprc-x)**2+(yprc-y)**2+(zprc-z)**2)

                  end if

                  if ( s.gt.smean ) then

!                 --- s > smean, center element found

                     s = (smean-sprec) / (s-sprec)

                     x = xprc + s * (x-xprc)
                     y = yprc + s * (y-yprc)
                     z = zprc + s * (z-zprc)

                     xl = x + 0.77d0 * y
                     zl = z + 0.45d0 * y

!                    --- Set number of curve

                     call plafep ( 9, xl, zl, 0.2d0, 0.2d0, icurve )
                     go to 150

                  end if

                  sprec = s
                  xprc = x
                  yprc = y
                  zprc = z

               end do

150         end if

            ncount = n+1

         end do

      end if

      if ( iplusp.gt.0 ) then

!     --- Plot user points

         if ( iscolr.gt.0 ) dcolor = 4+iscolr
         do j = 1, nuspnt
            xl = rinput(1,j)-xminsv
            yl = rinput(2,j)-yminsv
            zl = rinput(3,j)-zminsv
            if ( xl.ge.0d0 .and. yl.ge.0d0 .and. zl.ge.0d0 .and. &
                 xl.le.xmax .and. yl.le.ymax .and. zl.le.zmax ) then
               xl = xl + 0.77d0 * yl
               zl = zl + 0.45d0 * yl
               call plafep ( 6, xl, zl, 0d0, 0d0, 0 )
            end if

         end do

      end if

      if ( iplusp.gt.1 ) then

!     --- Plot numbers at user points

         if ( iscolr.gt.0 ) dcolor = iscolr
         do j = 1, nuspnt
            xl = rinput(1,j)-xminsv
            yl = rinput(2,j)-yminsv
            zl = rinput(3,j)-zminsv
            if ( xl.ge.0d0 .and. yl.ge.0d0 .and. zl.ge.0d0 .and. &
                 xl.le.xmax .and. yl.le.ymax .and. zl.le.zmax ) then
               xl = xl + 0.77d0 * yl
               zl = zl + 0.45d0 * yl
               call plafep ( 9, xl, zl, 0.2d0, 0.2d0, j )
            end if

         end do

      end if

!     --- close plot file

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
      jkader = ikader
      call erclos ( 'pltbn3' )
      dcolor = iscolr

      end
