      subroutine pltiso(m,n,sc,xp,ta,solid,base,zmin,xsect, &
                        u,v,t,lt,temp,feu,fev,ubeu,ubev,lbeu, &
                        lbev,ube,lbe)
      implicit double precision (a-h,o-z)
!         version 1.1    date 18-11-1992
      double precision ta(*)
      logical solid,base,xsect
! ********************************************************************
! * &
! *   this subroutine is used to produce an isometric projection
! *   of a surface z =z(x,y) where z is given at points on
! *   a regular mesh of squares. sections are drawn through the
! *   surface parallel to either the x or y axes.
! *   the visibility of each point along a section is examined by
! *   comparing points on the section with those on a visible
! *   profile. visible points are joined by straight lines and the
! *   co-ordinates of the point where a line becomes hidden are
! *   calculated using linear interpolation in the plane of
! *   projection. a full description is given in sections 2
! *   of  1e. the parameters required are m and n the
! *   number of mesh points along the x and y axes respectively , &
! *   a real array , ta(m*n), containing the heights z, a
! *   factor, sc, which determines the size of the projection and
! *   is set in the main program, the length, xp, of mesh
! *   along the x axis in units of z and logical variables
! *   solid, base and xsect. if solid is true the upper profile
! *   only is drawn and a base is drawn at a height determined
! *   by zmin if base is also true. sections are drawn parallel to
! *   the x or y axes depending upon whether xsect is true or false
! *   respectively. these parameters are described on page 7 of  1e
! * &
! *   references
! *    1e an algorithm for the production of an isometric
! *       projection of a three dimensional surface
! * &
! *   remarks
! *    i) installed by niek praagman
! *       version 1.1    date   22-08-1991 Zdenek: no dimension, cdc, ce
! *       last update : 22-03-87
! *   ii) the 2-dimensional array ta(m,n), mentioned in  1e is
! *       replaced by the 1-dimensional array ta(m*n)
! *       ( see also koos de wilde , t.h. delft )
! * &
! *******************************************************************
      double precision    m1,m2,x,y,ang,ss,tt,du,c1,c2
      integer i,j,k,ii,jj,kk,a,b
      logical penup,lower
!
!     set up working space assuming max(m,n) =120. if max(m,n)
!     is greater than 120 the working space must be increased
!     (see note f on page 13.)
!
      double precision u(*),v(*),t(*),lt(*),temp(*),feu(*)
      double precision fev(*),ubeu(*),ubev(*),lbeu(*),lbev(*)
      logical  ube(*),lbe(*)
!
!     set constants. if xsect true then sections are drawn
!     parallel to the x axis otherwise sections are drawn
!     parallel to the y axis.
!
      ang=0.86602540d0
      du=sc*ang*xp/(m-1d0)
      c1=0.5d0*sc*xp/(m-1)
      c2=2d0*c1
      if (.not.xsect) go to 50
      a=m
      b=n
      go to 60
   50 a=n
      b=m
      du=-du
!
!     scan surface in sections determined by xsect starting with
!     the section nearest the viewer.
!
   60 do 100 jj=1,b
        j=b-jj+1
        y=c2*(jj-1)
!
!     scan along section calculating projected co-ordinates u,v.
!     see figure 4b.
!
        do 110 ii=1,a
          i=a-ii+1
          x=c2*(ii-1)
          if (.not.xsect) go to 105
          ita=(j-1)*m+i
          u(ii)=ang*(x-y)
          v(ii)=0.5d0*(x+y)+sc*ta(ita)
          go to 110
  105     ita=(i-1)*m+j
          u(ii)=ang*(y-x)
          v(ii)=0.5d0*(x+y)+sc*ta(ita)
  110     continue
!
!     initialisation, co-ordinates of front edge stored in feu, fev.
!
        lower=.false.
        feu(j)=u(1)
        fev(j)=v(1)
!
!     test for first section. if not first section then test
!     visibility of all points along the section by comparing with
!     points on the visible upper profile t. the updated visible
!     profile is stored in temp until the scan has been completed.
!     co-ordinates ss,tt are calculated when section becomes hidden.
!
        if (j.ne.b) go to 120
!
!     first section drawn in full. set upper and lower visible
!     profiles equal to the initial section. draw vertical lines for
!     solid section if base is set true. store visibility and
!     co-ordinates of section end.
!
        call plt306(u(1),v(1),.true.)
        do 400 k=2,a
          call plt306(u(k),v(k),.false.)
          t(k)=v(k-1)
          lt(k)=t(k)
  400     continue
        if (.not.solid.or..not.base) go to 410
        do 420 kk=1,a
          k=a-kk+1
          call plt306(u(k),c1*(k-1)+zmin*sc,.true.)
          call plt306(u(k),v(k),.false.)
  420     continue
  410   ube(b)=.false.
        lbe(b)=.false.
        ubeu(b)=u(a)
        lbeu(b)=u(a)
        ubev(b)=v(a)
        lbev(b)=v(a)
        go to 100
!
!     first point of a section is always visible as a point on
!     the upper or lower profile. it is stored as a point on both
!     profiles but is drawn as a point on either the upper or lower
!     profile depending upon whether the second point along the
!     section is on the upper or lower profile.
!
  120   penup=v(2).lt.t(2)
        if (.not.penup) call plt306(u(1),v(1),.true.)
        t(1)=v(1)
        temp(1)=v(1)
        do 130 k=2,a
          if ((v(k).lt.t(k).and.penup).or.(v(k).ge.t(k).and..not.penup)) &
            go to 140
!
!     part of the line between k-1,k is hidden. co-ordinates ss,tt
!     where the line becomes hidden are calculated using linear
!     interpolation.
!
          m1=(t(k)-t(k-1))/du
          m2=(v(k)-v(k-1))/du
          ss=u(k)+(t(k)-v(k))/(m2-m1)
          tt=m1*(ss-u(k))+t(k)
          if (.not.penup) go to 150
!
!     line visible between ss,tt and point k which now becomes a
!     point on the upper visible profile.
!
          call plt306(ss,tt,.true.)
          call plt306(u(k),v(k),.false.)
          penup=.false.
          temp(k)=v(k)
          go to 130
!
!     line is hidden between ss,tt and point k.
!
  150     call plt306(ss,tt,.false.)
          penup=.true.
          temp(k)=t(k)
!
!     test for point on lower profile.
!
          if (v(k).le.lt(k)) lower=.true.
          go to 130
  140     if (v(k).ge.t(k)) go to 160
!
!     line between k-1,k is completely hidden.
!
          temp(k)=t(k)
!
!     test for point on lower profile.
!
          if (v(k).le.lt(k)) lower=.true.
          go to 130
!
!     line between k-1,k is completely visible.
!
  160     call plt306(u(k),v(k),.false.)
          temp(k)=v(k)
  130     continue
!
!     visibility of back edge upper profile is stored in
!     ube, co-ordinates are stored in ebeu,ubev. up-date visible
!     upper profile.
!
        ube(j)=penup
        ubeu(j)=u(a)
        ubev(j)=v(a)
        do 210 k=2,a
          t(k)=temp(k-1)
  210     continue
!
!     initialisation for lower profile lt. if lower set true then
!     test for visibility as above.
!
        lt(1)=v(1)
        temp(1)=v(1)
        if (solid.or..not.lower) go to 220
!
!     point occurs on lower profile and solid set false. test for
!     visibility of section end to determine the state of the pen.
!
        penup=v(a).gt.lt(a)
        if (.not.penup) call plt306(u(a),v(a),.true.)
!
!     visibility of back edge lower profile is stored in lbe, &
!     co-ordinates are stored in lbeu,lbev.
!
        lbe(j)=penup
        lbeu(j)=u(a)
        lbev(j)=v(a)
!
!     scan in reverse direction testing visibility of all points
!     along section by comparing with points on the visible lower
!     profile lt. the up-dated lower profile is stored in temp
!     until the scan has been completed. co-ordinates ss,tt are
!     calculated as for the upper profile.
!
        do 230 kk=2,a
          k=a-kk+1
          if ((v(k).gt.lt(k).and.penup).or.(v(k).le.lt(k).and. &
            .not.penup)) go to 240
          m1=(lt(k+1)-lt(k))/du
          m2=(v(k+1)-v(k))/du
          ss=u(k)+(lt(k)-v(k))/(m2-m1)
          tt=m1*(ss-u(k))+lt(k)
          if (.not.penup) go to 250
          call plt306(ss,tt,.true.)
          call plt306(u(k),v(k),.false.)
          penup=.false.
          temp(k)=v(k)
          go to 230
  250     call plt306(ss,tt,.false.)
          penup=.true.
          temp(k)=lt(k)
          go to 230
  240     if (v(k).le.lt(k)) go to 260
          temp(k)=lt(k)
          go to 230
  260     call plt306(u(k),v(k),.false.)
          temp(k)=v(k)
  230     continue
!
!     up-date lower visible profile to include new visible points.
!
        do 300 k=2,a
          lt(k)=temp(k-1)
  300     continue
        go to 100
!
!     no points on lower profile or solid set true. store back
!     edge of lower profile.
!
  220   lbeu(j)=u(a)
        lbev(j)=v(a)
        lbe(j)=.true.
        do 310 kk=2,a
          k=a-kk+2
          lt(k)=lt(k-1)
  310     continue
  100   continue
!
!     complete projection by drawing base and vertical lines if
!     solid and base are set true. a line is drawn along the front
!     and back edges joining adjacent visible section ends.
!
      call plt306(ubeu(b),ubev(b),.true.)
      do 500 kk=2,b
        k=b-kk+1
        if (.not.ube(k)) call plt306(ubeu(k),ubev(k),ube(k+1))
  500   continue
      call plt306(lbeu(b),lbev(b),.true.)
      do 520 kk=2,b
          k=b-kk+1
          if (.not.lbe(k)) call plt306(lbeu(k),lbev(k),lbe(k+1))
  520     continue
      call plt306(feu(1),fev(1),.true.)
      do 550 k=2,b
        call plt306(feu(k),fev(k),.false.)
  550   continue
      if (.not.solid.or..not.base) return
      do 560 kk=2,b
        k=b-kk+1
        call plt306(feu(k),c1*(kk-1)+zmin*sc,.true.)
        call plt306(feu(k),fev(k),.false.)
  560   continue
      call plt306(feu(1),c1*(b-1)+zmin*sc,.true.)
      call plt306(0d0,zmin*sc,.false.)
      call plt306(ubeu(b),c1*(a-1)+zmin*sc,.false.)
      end
