      subroutine pltjac ( acnseg, ackseg, nlines, revrse )
! ======================================================================
!
!        programmer     Jos van Kan
!        version 2.0    date 08-12-1998 rewrite to eliminate
!                                         hidden recursion
!        version 1.1    date 22-03-1996 different implementation
!                                         for reverse case, to eliminate hidden
!                                         recursion.
!        version 1.0    date 20-01-1995
!
!
!
!   copyright (c) 1995-1996 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!  pltjac accumulates the entries of (ac)nseg and (ac)kseg, providing the
!  user with
!  offsets into the arrays kseg and xcint of the iseg(th) segment. Result
!  put in acnseg and ackseg respectively.
!  Acnseg (0) = 0, Acnseg (nlines) = total number of segments (TNSEG).
!  Ackseg (0) = 0, Ackseg (TNSEG) = total number of points.
!  If Reverse is true Acnseg (1) = 1, Acnseg (nlines + 1) = TNSEG + 1.
!                     AcKseg (1) = 1, AcKseg (TNSEG + 1) = TnpointS + 1.
!  in this way you always can go along a line iline starting at
!  AcNseg (iline - inc) + inc to AcNseg (iline) and along a segment
!  kseg starting at AcKseg (kseg - inc) + inc to AcKseg (kseg). Inc will be
!  +1 or -1 according to whether revrse is true or false.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer acnseg (0:*), ackseg (0:*), nlines
      logical revrse

!     ackseg  i/o  i: kseg; o: accumulation of kseg
!     acnseg  i/o  i: nseg; o: accumulation of nseg
!     nlines  i    total number of lines
!     revrse  i    Boolean to indicate whether lines are plotted in reverse
!                  order
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilines, iseg, ntseg

!     ilines  Counting variable
!     iseg    index in cmplst of segment about to be replaced. Cmplst
!             (iseg) contains the segment number, cmplst (iseg + 1)
!             the line number on which this segment lies.
!     ntseg   total number of segments?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if (nlines.eq.0) then
         return
      end if
      if (revrse) then

         do 100 ilines = nlines + 1, 2, -1
            acnseg (ilines) = acnseg (ilines - 1)
100      continue
         acnseg (1) = 1
         do 105 ilines = 2, nlines + 1
            acnseg (ilines) = acnseg (ilines - 1) + acnseg (ilines)
105      continue

         ntseg = acnseg (nlines + 1) - 1

         do 200 iseg = ntseg + 1, 2, -1
            ackseg (iseg) = ackseg (iseg - 1)
200      continue
         ackseg(1) = 1
         do 205 iseg = 2, ntseg + 1
            ackseg (iseg) = ackseg (iseg - 1) + ackseg (iseg)
205      continue
      else
         acnseg (0) = 0
         ackseg (0) = 0
         do 300 ilines = 1, nlines
            acnseg (ilines) = acnseg (ilines - 1) + acnseg (ilines)
300      continue
         ntseg = acnseg (nlines)
         do 400 iseg = 1, ntseg
            ackseg (iseg) = ackseg (iseg - 1) + ackseg (iseg)
400      continue
      end if
      end
