      subroutine pltjli ( xcint, ymin, dy, fcint, ackseg, &
                          ncomp, pltlst, segstr, xcoor, zcoor, inc )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.1  date 21-11-1995 Better error handling
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       pltjli loads "singular" points in block mode that would not get
!       plotted otherwise in the new boundary treatment, since they do
!       not belong to any specific segments. These point are signalled in the
!       array ncomp. If the segment is unconnected from below (ncomp (1,*) = 0)
!       ground points have to be plotted. The routine must not be called
!       in SHEET mode.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepltj
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer pltlst(0:*), ncomp(2, *), pltjcd, ackseg(0:*), segstr
      double precision xcint(*), fcint(*), dy, xcoor(*), zcoor(*), ymin

!     ackseg  i    array of accumulated number of points in every segment
!     dy      i    y increment
!     fcint   i    array of function values
!     inc     i    loading increment. 1 for normal, -1 for backwards
!     ncomp   i    contains information on connected components
!     pltlst  i    array containing segment & line numbers of
!                  segments about to be plotted
!     segstr  o    start of list containing singular points
!     xcint   i    array of x-values
!     xcoor   o    xcoordinates of list
!     ymin    i    minimum value of y coordinate
!     zcoor   o    z coordinates of list
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseg, inxstr, inxend, segst0, seghpt, segtmp, inc, seginx
      double precision xproj, zproj1, y

!     inxend  ?
!     inxstr  ?
!     iseg    index in cmplst of segment about to be replaced. Cmplst
!             (iseg) contains the segment number, cmplst (iseg + 1)
!             the line number on which this segment lies.
!     seghpt  first cell (header) of a segment
!     seginx  ?
!     segst0  ?
!     segtmp  ?
!     xproj   ?
!     y       y-coordinate
!     zproj1  ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLTJCD  To find cellpointer to next cell
!     PLTJMS  to append a created segment to an existing list
!     PLTJNE  to create a new list
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       For all segments on the pltlst:
!       check for singular endpoints (ncomp (1,*) = 0)
!       load singular endpoints segstr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initialize start addresses of list

      call eropen ('pltjli')
      segstr = 0
      segst0 = 0
      seghpt = 0

!     --- Main algorithm. For all segments on the plotting list we check
!         if they have lower component 0, since this indicates a new component.
!         From that new component we plot the two vertical lines connecting the
!         function values to the ground. this is either the minimal function
!         value
!         over the field or 0, whichever is smaller. (See also subroutine
!         decode).
!         This value is stored in fminh.

      do 100 iseg = 1, pltlst (0), 2
         seginx = pltlst (iseg)
         if (ncomp (1, seginx).eq.0) then

!        --- This segment has two singular points
!            We must add those points to segstr

            inxstr = ackseg (seginx - inc) + inc
            inxend = ackseg (seginx)
            call pltjne (segst0, 2)
            if ( ierror/=0 ) goto 99999
            y = ymin + (pltlst (iseg + 1) - 1) * dy
            xcoor (segst0) = xcint (inxstr) * (cth - scfi * sth) &
                    + y *  (sth + scfi * cth)
            xcoor (pltjcd (segst0)) = xcoor (segst0)
            zproj1 = -ssfi * sth * xcint (inxstr) + y * ssfi * cth
            zcoor (segst0) = zproj1 + zscale * fminh
            zcoor (pltjcd (segst0)) = zproj1 + zscale * fcint (inxstr)

!           --- Append this segment to the existing list
!               First save segst0

            segtmp = segst0
            call pltjms (segst0, 0, segstr, seghpt)
            seghpt = segtmp
            call pltjne (segst0, 2)
            if ( ierror/=0 ) goto 99999
            xproj = xcint (inxend) * (cth - scfi * sth) + y * &
                  (sth + scfi * cth)
            xcoor (segst0) = xproj
            xcoor (pltjcd (segst0)) = xproj
            zproj1 = -ssfi * sth * xcint (inxend) + y * ssfi * cth
            zcoor (segst0) = zproj1 + zscale * fminh
            zcoor (pltjcd (segst0)) = zproj1 + zscale * fcint (inxend)
            segtmp = segst0
            call pltjms (segst0, 0, segstr, seghpt)
            seghpt = segtmp
         end if
100   continue
99999 continue
      call erclos ('pltjli')
      end
