      subroutine pltjr1 ( stadr, xcoor, ycoor )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.1  date 21-11-1995 Better error handling
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       pltjr1 removes segments that essentially consist of one point only
!       These may be introduced by pltjvi, when two segments are very close
!       together. This takes up a lot of space in the listpack. So there.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulepltj
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer stadr
      double precision xcoor(*), ycoor(*)

!     stadr   i    start address of the list
!     xcoor   i    array of x coordinates in the listpack.
!     ycoor   i    array of y coordinates in the listpack.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer hptr0, hptr1, cell1

!     cell1   cell number
!     hptr0   predecessor cell of segment to be removed. Its adrg will
!             be updated.
!     hptr1   header cell of segment to be removed. If hptr0 = &
!             nil hptr1 contains the new startaddress on exit
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer pltjca, pltjcd

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLTJCA  to get the next segment
!     PLTJCD  To find cellpointer to next cell
!     PLTJRM  to remove a segment
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       While {List not exhausted} do
!       Get Next Segment
!       If {Segment contains One point} Remove segment
!       If {All segment points are closer to the first point than eps (1d-4)}
!       Remove segment
!       End While
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('pltjr1')
      hptr0 = nil
      hptr1 = stadr
100   if (hptr1.eq.nil) then
         call erclos ('pltjr1')
         return
      end if

!     --- End of list

      cell1 = hptr1
110   cell1 = pltjcd (cell1)
      if (cell1.eq.nil) then

!     --- End of segment. This must be removed

         call pltjrm (hptr0, hptr1)

!        --- If it was the start of the list the start address must be
!            updated

         if ( ierror/=0 ) goto 99999
         if (hptr0.eq.nil) then
            stadr = hptr1
         end if
         go to 100
      else if (abs (xcoor (cell1) - xcoor (hptr1)) + abs (ycoor (cell1) &
         - ycoor(hptr1)).gt.1d-4) then

!     --- At least two different points in this segment
!         Get next segment

         goto 120
      else

!     --- Get next cell

         goto 110
      end if
120   hptr0 = hptr1
      hptr1 = pltjca (hptr1)
      goto 100
99999 continue
      call erclos ('pltjr1')
      end
