      subroutine pltjtr ( coor, fval, npoint, kmeshc, nelem, &
                          rlinp )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.2  date 12-05-2014 call plafep
!        version  1.1  date 11-03-2011 Remove close plot
!        version  1.0  date 02-02-1995
!
!   copyright (c) 1995-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       pltjtr plots a 3D object "transparently" that is, as if it
!       were made of glass. All parts that are hidden from view, &
!       because they are with their "back" to the observer are
!       invisible, but if they should be hidden, because some- &
!       thing is in front, you see them. This is much faster
!       than pltj3d, and also requires no work arrays.
!       The input is exactly the same as for pltj3d, but pltjtr is
!       not sensing parameters like intersect_angle, block_mode and
!       the like. It does not hurt, however to set them.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, kmeshc(*), nelem
      double precision coor (2,*), fval (*), rlinp(14)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     fval    i    function values in nodes
!     kmeshc  i    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     nelem   i    Number of elements
!     npoint  i    Number of nodal points
!     rlinp   i    dp array of 12 elements.
!                  (1) orientation of projection in degrees. Default 0
!                  (2) angle of projected y-axes wrt x-axes (degrees)
!                      Default 60 deg (10<phi<170). Anything outside
!                      this interval (notably phi = 0) defaults to 60.
!                  (4) width of plot in cm (Default 20). (anything<1)
!                  (5) height of plot in cm (Default 20).(anything<1)
!                  (6) xminwindow  minimum dataregion with respect to
!                      the center of the figure to be plotted.
!                  (7) xmaxwindow maximum dataregion with respect to
!                      the center of the figure being plotted.
!                  NB if xmaxwindow - xminwindow<=0 a reasonable
!                     value is calculated by the plotting routine.
!                  (8) yminwindow minimum data region with respect to
!                      the figure being plotted.
!                  (9) ymaxwindow maximum data region with respect to
!                      the figure being plotted.
!                  NB if ymaxwindow - yminwindow<=0 a reasonable
!                     value is calculated by the plotting routine.
!                  (11) Scalefactor for of y-axis. Default 0.5.
!                  (12) Scalefactor for of z-axis. Default 1.
!                       If one of the scales are<=0 the default
!                       value is set.
!                       IMPORTANT: Setting every entry of rlinp 0 gets
!                       default values for all variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, intinp(3), inumbr
      double precision dy

!     dy             ?
!     ielem          Element number
!     intinp         ?
!     inumbr
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLTJDE         Calculates  dy, xmin, xmax, ymin and ymax, fmin, fmax
!     PLTJVS         Checks visibility of triangle, plots it if visible
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      For all elements it is checked whether the triangle {X_0, X_1, &
!      X_2} with X_j = (x_j, y_j, z_j) is with the front or the back
!      towards the viewer by calculating the sign of V.{(X_0 - X_1) x
!      (X_2 - X_1)}. V is the direction of the lines of vision of the
!      parallell projection. If the sign is positive this triangle
!      is plotted, otherwise not.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('pltjtr')
      intinp = 0

      call pltjde (dy, coor, fval, npoint, rlinp, intinp, inumbr)
      call plafep (1, rlinp (4), rlinp (5), 0d0, 0d0, inumbr)

      do ielem = 1, nelem

!     --- pltjvs calculates the quantity sign (det(V, X_0 - X_1, X_2 - X_1))
!         plots the triangle if visible

         call pltjvs (coor, fval, kmeshc, ielem)

      end do

      call erclos ('pltjtr')

      end
