      subroutine pltopbottext ( itext, text, plotfm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot text at top or bottom of plot if required
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     text
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itext
      double precision plotfm
      character (len=*) text(*)

!     itext          i    Number of entries in array text
!                         if 1 only the bottom text is plotted, otherwise
!                         (2) bottom and top text are plotted
!     plotfm         i    Plotting factor. When PLOTFM<0, then the length of
!                         the plot is computed by - (xmax-xmin) * PLOTFM
!                         and the width by - (ymax-ymin) * YFACT * PLOTFM
!                         else the length of the plot is equal to PLOTFM cm
!                         When by some mistake the user has given | PLOTFM | a
!                         value smaller than 1 cm, then the length is chosen
!                         equal to the default length stored in common block
!                         consta
!     text           i    Contains the text to be plotted at bottom (1)
!                         and top (2)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ntextl

!     ntextl         Number of characters in text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFP7         Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pltopbottext' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pltopbottext'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( itext>0 ) then

!     --- itext = 1, 2 Plot bottom text

          ntextl = index(text(1),'   ')-1
          if ( ntextl<=0 ) ntextl = len (text(1))
          call plafp7 ( 1d0, 0.25d0, text(1), 0d0, 0.3d0, ntextl )

      end if

      if ( itext>1 ) then

!     --- itext = 2 Plot text at top

          ntextl = index(text(2),'   ')-1
          if ( ntextl<=0 ) ntextl = len (text(2))
          call plafp7 ( 1d0, plotfm-2d0, text(2), 0d0, 0.3d0, &
                        ntextl )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pltopbottext'

      end if  ! ( debug )
      call erclos ( 'pltopbottext' )

      end
