      subroutine pltr11( x, y, z, iresul, xl, yl, zl, &
                         ski, eta, zta, det )
! ======================================================================
!
!        programmer    niek praagman
!        version  1.1  date 11-02-1993 adjustment to new norms
!        version  1.0  date 31-05-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine to determine whether point (x,y,z) belongs to the
!     tetrahedron given by the points (xl,yl,zl)
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x, y, z, xl(4), yl(4), zl(4), ski, eta, zta, det
      integer iresul

!     det       o   volume of tetrahedron
!     eta       o   local second barycentric coordinate
!     iresul    o   indicator : if 1 point x,y,z belongs to tetrahedron
!                               if 0 point belongs not to tetrahedron
!     ski       o   local first barycentric coordinate
!     x         i   global x-coordinate
!     xl        i   global x-coordinates of nodes of tetrahedron
!     y         i   global y-coordinate
!     yl        i   global y-coordinates of nodes of tetrahedron
!     z         i   global z-coordinate
!     zl        i   global z-coordinates of nodes of tetrahedron
!     zta       o   local third barycentric coordinate
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision a1, a2, a3, b1, b2, b3, c1, c2, c3, d1, d2, d3

!     a1      - &
!     a2      - &
!     a3      - &
!     b1      - &
!     b2      - &
!     b3      -  helpvariables for local differences
!     c1      - &
!     c2      - &
!     c3      - &
!     d1      - &
!     d2      - &
!     d3      - &
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLTR14   determine determinantvalue of 3 X 3 matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine local differences
!     Compute local coordinates
!     Check whether all local values are between 0 and 1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      a1 = xl(2) - xl(1)
      a2 = yl(2) - yl(1)
      a3 = zl(2) - zl(1)
      b1 = xl(3) - xl(1)
      b2 = yl(3) - yl(1)
      b3 = zl(3) - zl(1)
      c1 = xl(4) - xl(1)
      c2 = yl(4) - yl(1)
      c3 = zl(4) - zl(1)
      d1 =  x - xl(1)
      d2 =  y - yl(1)
      d3 =  z - zl(1)

!     --- determine ski , eta and zta

      call pltr14( a1, a2, a3, b1, b2, b3, c1, c2, c3, det )

!     --- ski

      call pltr14( d1, d2, d3, b1, b2, b3, c1, c2, c3, ski )
      ski = ski / det
      if ( abs( ski ).lt.epsmac ) ski = 0
      if ( ski.lt.0d0 .or. ski.gt.1d0 ) then
           iresul = 0
           goto 1000
      end if

!     --- eta

      call pltr14( a1, a2, a3, d1, d2, d3, c1, c2, c3, eta )
      eta = eta / det
      if ( abs( eta ).lt.epsmac ) eta = 0
      if ( eta.lt.0d0 .or. eta.gt.1d0 ) then
          iresul = 0
          goto 1000
      end if

!     --- zta

      call pltr14( a1, a2, a3, b1, b2, b3, d1, d2, d3, zta )
      zta = zta / det
      if ( abs( zta ).lt.epsmac ) zta = 0
      if ( zta.lt.0d0 .or. zta.gt.1d0 ) then
          iresul = 0
          goto 1000
      end if
      if ( ( 1d0 - ski - eta - zta ).lt.0d0 ) then
          iresul = 0
          goto 1000
      end if

!     --- point is inside the tetrahedron , hence set iresul 1

      iresul = 1
1000  continue
      end
