      subroutine propenplot ( bouncurv, plotfm, yfact, ikader, plotboun )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 15-11-2013 Change of parameter plotboun
!        version  1.2  date 12-09-2013 Skip if not first plot
!        version  1.1  date 25-11-2012 Long integers
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Open plot and plot boundary (2d)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ikader, plotboun
      double precision plotfm, yfact
      logical bouncurv

!     bouncurv      i/o   If true at input we might use kmeshh to find
!                         the curves otherwise only kmesh part m is used
!                         At output it has been decided which is used
!                         true: kmeshh,   false:  kmeshm
!     ikader         o    value of the parameter jkader in the common block
!                         cplot at input
!     plotboun       i    Choice parameter consisting of 3 parts: iinner, number, &
!                         isymbl according to:
!                         plotboun = iinner + 10 * number + 100 * isymbl
!                         with:
!                         iinner   Information of how the curves must be plotted
!                           0,1    only the outer boundaries are plotted
!                           2      only the inner boundaries are plotted
!                           3      all boundaries are plotted
!!                          4      no boundaries are plotted
!                         number   Indication if the curves must be provided with
!                                  a curve number (1) or not
!                         isymbl   Indicates if the nodes along the curves must
!                                   be marked with a cross (1) or not (=0)
!     plotfm         i    Plotting factor. When PLOTFM<0, then the length of
!                         the plot is computed by - (xmax-xmin) * PLOTFM
!                         and the width by - (ymax-ymin) * YFACT * PLOTFM
!                         else the length of the plot is equal to PLOTFM cm
!                         When by some mistake the user has given | PLOTFM | a
!                         value smaller than 1 cm, then the length is chosen
!                         equal to the default length stored in common block
!                         consta
!     yfact          i    Multiplication for the y-co-ordinates. With this
!                         parameter the user can influence the scale in
!                         y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ktimes

!     ktimes         mod(itimes,10)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0003         Open plot, plots frame work and computes plotting
!     SEPPLOTBN      Plot boundary of region
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'propenplot' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from propenplot'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Skip in case not the first plot

      ktimes = mod(jtimes,10)
      if ( ktimes>1 ) go to 1000

      bouncurv = .false.
      if ( bouncurv .and. kelmh>0 ) then

!     --- kmesh part h has been filled
!         Check if icurvs and curves have been filled

         if ( kmeshh(3,1)>0 .and. kmeshh(5,1)>0 ) bouncurv = .true.

      end if  ! ( bouncurv .and. kelmh>0 )

      textx = 'x'
      texty = 'y'
      if ( bouncurv ) then

!     --- Use arrays curves and icurvs

         call pl0003 ( ikader, 1, plotfm, yfact, 10 )

      else

!     --- Use arrays coor and kmeshm

         call pl0003 ( ikader, 2, plotfm, yfact, 0 )

      end if

      if ( mod(plotboun,10)<4 ) then

!     --- Plot the boundary of the region

         call sepplotbn ( 0, plotboun, bouncurv )

      end if  ! ( plotboun )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End propenplot'

      end if  ! ( debug )
      call erclos ( 'propenplot' )

      end subroutine propenplot
