      subroutine sepplotbn ( idoubl, jinner, bouncurv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.5  date 11-04-2008 Extra debug
!        version  5.4  date 14-08-2003 Correction closed boundaries
!        version  5.3  date 07-11-2002 Debug statements
!
!   copyright (c) 1982-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   this subroutine plots the boundary of a mesh as given
!   in array kmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idoubl, jinner
      logical bouncurv

!    bouncurv  i   if true array curves has been filled, otherwise arrays
!                  coor and kmeshm must be used
!    idoubl    i   indication whether the boundaries are plotted with
!                  double lines (idoubl = 1) or not (idoubl = 0)
!    jinner    i   Choice parameter consisting of 3 parts: iinner, number,
!                  isymbl according to:
!                  jinner = iinner + 10 * number + 100 * isymbl
!                  with:
!                  iinner   Information of how the curves must be plotted
!                  0,1      only the outer boundaries are plotted
!                    2      only the inner boundaries are plotted
!                    3      all boundaries are plotted
!                  number   Indication if the curves must be provided with
!                           a curve number (1) or not
!                  isymbl   Indicates if the nodes along the curves must be
!                           marked with a cross (1) or not (=0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical inboun, skipcurve
      integer iinner, number, isymbl, nstar3, nstar4, &
              iplot, kinner, istart, icurnr, ilast, iuppl, j, node, i1
      double precision x, y, s, xprec, yprec, sprec, smean

!     i1             Starting position in CURVES for the specific curve
!     icurnr         Curve number
!     iinner         See INPUT/OUTPUT
!     ilast          Last position in KMESHM part m corresponding to curve
!     inboun         Logical indicating if a boundary must be marked as inner
!                    boundary (true) or not (false)
!     iplot          Parameter for subroutine PLAFEP indicating how lines must
!                    be plotted (single=5, double=21)
!     istart         Starting position in KMESM part c for the specific curve
!     isymbl         See INPUT/OUTPUT
!     iuppl          Indication if a the pen must move to a point with pen up
!                    (4) or down (#4)
!     j              Counting variable
!     kinner         Counter for number of inner boundaries
!     node           Nodal point number
!     nstar3         Starting address of KMESHM part 3 in KMESHM containing the
!                    node numbers of the curves
!     nstar4         Starting address of KMESHM part 4 in KMESHM containing the
!                    inner boundaries
!     number         See INPUT/OUTPUT
!     s              Computed actual length of a curve
!     skipcurve      If true curve must be skipped
!     smean          Half the length of the complete curve
!     sprec          Computed length of the curve until present element
!     x              x co-ordinate of point
!     xprec          x co-ordinate of preceding point on curve
!     y              y co-ordinate of point
!     yprec          y co-ordinate of preceding point on curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplotbn' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepplotbn'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute inner, number and isymbl

      isymbl = jinner / 100
      number = (jinner-100*isymbl)/10
      iinner = max ( mod(jinner,10), 1 )

!     --- Compute some constants

      nstar3 = kmeshm(2)-1
      nstar4 = kmeshm(3)
      ninner = kmeshm(1)

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'bouncurv ', bouncurv
         write(irefwr,1) 'isymbl, number, iinner, ninner', &
                          isymbl, number, iinner, ninner
         write(irefwr,1) 'ncurvs, idoubl', ncurvs, idoubl
         call prinin ( kmeshm, 20 ,'first 20 pos. of kmeshm' )
         call prinin ( kmeshm(nstar4), ninner, 'kmeshm_inner' )

      end if

      if ( ninner==0 .and. iinner==2 ) go to 1000
      if ( idoubl==1 ) then
         iplot = 21
      else
         iplot = 5
      end if

!     --- plot boundaries

      kinner = 0
      istart = 1
      i1 = 1

!     --- Loop over curves

      do icurnr = 1, ncurvs
         ilast = kmeshm(5+icurnr)-1
         if ( bouncurv ) then

!        --- bouncurv is true, skip subcurves of SPCURV

            if ( icurvsmsh(ncurvs+icurnr)==2 ) go to 200

         end if
         inboun = .false.
         skipcurve = kinner<ninner
         if ( skipcurve ) skipcurve = abs(kmeshm(nstar4+kinner))==icurnr

         if ( skipcurve ) then

!        --- Curve must be skipped

            if ( iinner/=3 ) then

!           --- inner boundary found

               kinner = kinner+1
               inboun = .true.

            else

!           --- inner boundary found

               if ( kmeshm(nstar4+kinner)<0 ) inboun = .true.
               kinner = kinner+1

            end if

         end if

         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'icurnr, iinner, kinner', &
                             icurnr, iinner, kinner
            write(irefwr,*) 'inboun ', inboun

         end if

         if ( inboun.and.iinner==2 .or. &
              (.not. inboun).and.(iinner==1 .or. iinner==3) ) then

!        --- plot boundary

            iuppl = 4

!           --- Check if boundary has been filled or not
!               If so plot else skip boundary

            if ( bouncurv ) then

!           --- bouncurv = true, use array curves

               if ( debug ) then

!              --- Debug information

                  write(irefwr,1) 'icurnr, i1, icurvsmsh(icurnr)', &
                                   icurnr, i1, icurvsmsh(icurnr)
                  call prinrl1 ( curvesmsh(1,i1), icurvsmsh(icurnr)-i1+1, 2, &
                                 'curves' )

               end if
               do j = i1, icurvsmsh(icurnr)
                  call plafep ( iuppl, curvesmsh(1,j), curvesmsh(2,j), 0d0, &
                                0d0, 0 )
                  iuppl = iplot
               end do
               if ( icurnr==1 .and. ncurvs-ninner==1 ) then

!              --- Only one curve has to be plotted, close curve

                  call plafep ( iuppl, curvesmsh(1,i1), &
                                curvesmsh(2,i1), 0d0, 0d0, 0 )

               end if

            else

!           --- bouncurv = false, use arrays kmeshm and coor

               if ( debug ) then

!              --- Debug information

                  write(irefwr,1) 'icurnr, istart, ilast, nstar3', &
                                   icurnr, istart, ilast, nstar3
                  call prinin ( kmeshm(nstar3+istart), ilast-istart+1, &
                                'nodes on curve' )

               end if
               do j = istart, ilast
                  if ( kmeshm(nstar3+j)<=0 ) goto 290
               end do

               do j = istart, ilast
                  node = kmeshm(nstar3+j)
                  x = coor(1,node)
                  y = coor(2,node)
                  call plafep ( iuppl, x, y, 0d0, 0d0, 0 )
                  iuppl = iplot
               end do

            end if

!           --- Plot numbers along boundary if necessary

            if ( number>0 ) then

!           --- number = 1, plot numbers
!               First compute length of curve

               do j = istart, ilast
                  node = kmeshm(nstar3+j)
                  x = coor(1,node)
                  y = coor(2,node)
                  if ( j==istart ) then

!                 --- j = istart, set s equal to zero

                     s = 0d0

                  else

!                 --- Not the first point, raise s

                     s = s + sqrt ( (xprec-x)**2 +(yprec-y)**2 )

                  end if

                  xprec = x
                  yprec = y

               end do

               smean = s * 0.5d0

!              --- Find element in the center of the boundary

               do j = istart, ilast
                  node = kmeshm(nstar3+j)
                  x = coor(1,node)
                  y = coor(2,node)
                  if ( j==istart ) then

!                 --- j = istart, set s equal to zero

                     s     = 0d0
                     sprec = 0d0

                  else

!                 --- Not the first point, raise s

                     s = s + sqrt ( (xprec-x)**2 +(yprec-y)**2 )

                  end if

                  if ( s>smean ) then

!                 --- s>smean, center element found

                     s = (smean-sprec) / (s-sprec)
                     x = xprec + s * (x-xprec)
                     y = yprec + s * (y-yprec)
                     call plafep ( 9, x, y, 0.2d0, 0.2d0, icurnr )
                     go to 150

                  end if

                  sprec = s
                  xprec = x
                  yprec = y

               end do

            end if

!           --- Plot symbols if necessary

150         if ( isymbl>0 ) then

               if ( bouncurv ) then
                  do j = i1, icurvsmsh(icurnr)
                     call plafep ( 6, curvesmsh(1,j), curvesmsh(2,j), 0d0, &
                                   0d0, 0 )
                  end do
               else
                  do j = istart, ilast
                     node = kmeshm(nstar3+j)
                     x = coor(1,node)
                     y = coor(2,node)
                     call plafep ( 6, x, y, 0d0, 0d0, 0 )
                  end do
               end if

            end if

         end if
200      if ( bouncurv ) i1 = icurvsmsh(icurnr)+1

290      istart = ilast+1

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepplotbn'

      end if  ! ( debug )
      call erclos ( 'sepplotbn' )

      end
