      subroutine sepplotcn ( iplots, rplots, contln, input, text, &
                             ncoorc, coorcn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 17-05-2011 plot boundary zero levelset
!        version  1.2  date 05-12-1999 Extra option with respect to levels
!        version  1.1  date 23-02-1999 Adapt defaults
!
!   copyright (c) 1997-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Driver for SEPRAN contour plotting subroutine
!   Both 2D and 3D contour lines or colored contour levels may be plotted
!   This subroutine replaces the old main SEPRAN subroutines:
!        PLOTC1, PLOTC2, PLOTC3, CONT3C and CONT3D
!   If the length of the arrays IPLOTS and RPLOTS change, the old subroutines
!   must be adapted acoordingly
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     color
!     driving_subroutine
!     plot
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iplots(*), input(*), ncoorc(*)
      double precision rplots(*), coorcn(*), contln(*)
      character *(*) text(*)

!    contln    i   User array of length ncntln + 5 (if used) in which the
!                  user is supposed to have stored the contour levels from
!                  position 6
!    coorcn    o   At output this array contains the co-ordinates of the
!                  points on the contours in the sequnce as given in NCOORC.
!                  These co-ordinates are stored from position 6 in the
!                  sequence (x1,y1), (x2,y2) etc. COORCN is a variable length
!                  array, hence the first position must be filled by the user.
!                  COORCN is only filled if CHOICE (IPLOTS(2)) = 3
!    input     i   This array is only used when JINDCOL (IPLOTS(5)) >0 and
!                  or ISKIP>0
!                  In fact this array consists of three parts:
!                  Part 1 are the first 5 positions. They have to be filled
!                  by the user in the following way:
!                  Pos. 1:  Declared length of array input
!                  Pos. 2-5: 0
!                  Part 2 contains information about the colors to be used.
!                  Part 2 starts at position 6 and the length depends on JINDCOL
!                  If JINDCOL=1, it must be filled by the user with the colors
!                  of the various contour levels in increasing sequence.
!                  The length is equal to the number of contour levels
!                  If JINDCOL=2 it has length 1
!                  In INDCOL(6) the user must store the minimum color required
!                  If JINDCOL=3 it is has length 2
!                  The minimum color must be stored in INDCOL(6) the maximum
!                  one in INDCOL(7)
!                  If the range of the colors is less than the available range
!                  of colors the color numbering is repeated
!                  Part 3 contains information about the element groups to be
!                  skipped. Part 3 is only used if ISKIP>0
!                  The value of ISKIP defines the first position in INPUT
!                  with information about the element groups to be skipped.
!                  Part 3 must be filled as follows:
!                  If INPUT(ISKIP-1+i)=0, element group i must be skipped
!                  If INPUT(ISKIP-1+i)=1, element group i must be taken into
!                  account
!    iplots    i   Integer control array in which the user defines which choices
!                  he wants for this subroutine
!                  IPLOTS must be filled  by the user in the following way:
!                  Position:
!                  1:  Defines the highest entry number of the array that has
!                      been filled by the user. This means that the user must
!                      fill all positions 1 to IPLOTS(1) in array IPLOTS.
!                      For all other positions default values are used.
!                      Hence if IPLOTS(1) = 0 or 1 only defaults are used.
!                      Remark: if IPLOTS(i)=0, also for position i the
!                              default is taken.
!                  2:  (CHOICE) defines the type of contour lines or levels
!                      required. Possible values for CHOICE are:
!                      1:  Contour lines will be drawn. These lines are
!                          provided with a numbered label.
!                          In the case of a 3D grid, contour lines are plotted
!                          on the outer surface only
!                      2:  Contour levels will be colored, i.e. the interior
!                          space between contour levels as well as the region
!                          smaller than the smallest contour level and the
!                          region larger than the largest contour level is
!                          filled with a specific color for that level.
!                      3:  See 1, however, not only are the contour levels
!                          plotted, but also the co-ordinates of these contour
!                          lines are outputed in the arrays COORCN and NCOORC
!                          This possibility is only available in 2D
!                      4:  See 1, however numbers are not plotted
!                      Default value: 1
!                  3:  (NUMBER) the number'th physical unknown in each node is
!                      considered When this unknown does not appear in a nodal
!                      point, the point is skipped
!                      When array ISOL corresponds to a complex array, &
!                      NUMBER=2*i-1 corresponds to the real part of unknown i
!                      and NUMBER=2*i to the imaginary part of unknown i
!                      Default value: 1
!                  4:  (NCNTLN) Parameter indicating the number of contour
!                      levels to be used. Possible values of ncntln are:
!                     >0:  There are ncntln contour levels that must be stored
!                            in array contln from position 6
!                      = 0:  The program computes the contour levels by
!                            dividing the range of the function between maximum
!                            and minimum value into 10 parts in the case of
!                            CHOICE = 1 or 3 and in 20 parts if CHOICE=2
!                     <0>-1000: The program computes the contour levels by
!                            dividing the range of the function between maximum
!                            and minimum value into |NCNTLN| - 1 parts. The
!                            values are chosen such that rounded values of the
!                            contour levels appear
!                     <-1000: The program computes the contour levels by
!                            dividing the range of the function between maximum
!                            and minimum value into |NCNTLN + 1000| - 1 parts.
!                      -10000: separate levels are used for the positive values
!                              as well as for the negative values
!                              The zero level is always plotted
!                            The number of positive levels is stored in pos. 23
!                            The number of negative levels is stored in pos. 24
!                      -10001: see -10000, however, now the values are rounded
!                      Default value: 0
!                  5:  (JINDCOL) Parameter indicating the type of coloring to
!                      be used for the contour lines or levels:
!                      Possible values:
!                      0:  The standard colors are used, i.e. standard color
!                          contour lines in the case of CHOICE=1 or CHOICE=3
!                          and color numbers 1 to number_of_contour_levels in
!                          the case of CHOICE=2.
!                      1:  The user defines the color numbers explicitly
!                          through array INDCOL. This possibility is
!                          moment only available for CHOICE=2
!                      2:  The user defines the minimum color number explicitly
!                          through array INDCOL. This possibility is
!                          only available for CHOICE=2
!                      3:  The user defines the minimum and maximum color
!                          number explicitly through array INDCOL. This
!                          possibility is only available for CHOICE=2
!                      >0:  The visible parts of the contour lines get the
!                          color sequence number JINDCOL CHOICE=1 or 3
!                      Default value: 0
!                  6:  (PROJECT) choice parameter for type of projection
!                      (3D only)
!                       0: parallel projection plot (view point given by the
!                          user in RPLOTS(4-6) )
!                       1: perspective projection plot (view point given by the
!                          user in RPLOTS(4-6) )
!                      10:  parallel projection plot (View point automatic)
!                      11:  perspective projection plot (View point automatic)
!                      Default value: 10
!                  7:  (JSMOOTH) choice parameter for type of smoothing to be
!                      used for the contour lines. At this moment JSMOOTH is
!                      only implemented for CHOICE=1 or 3 in 2-dimensional
!                      grids.
!                      Possible values:
!                      0:  no smoothing
!                      1:  shuman smoothing
!                      2:  |
!                      3:  |  all spline smoothing : the higher the value the
!                      4:  |  more smoothing and the less interpolation of
!                      5:  |  the values.
!                      Default value: 0
!                  8:  (IDOUBLE) indication whether the outer boundary is
!                      plotted as a single line (0) or as double line (1)
!                      (2D only)
!                      Default value: 0
!                  9:  (INNER) indication whether inner boundaries must be
!                      plotted (1) or not (0)
!                      (2D only)
!                      Default value: 0
!                 10:  (PLOT_CONTOUR) indication if the boundaries of the
!                      contourlevels must be plotted (>0) or not (0). The
!                      value of PLOT_CONTOUR defines the color.
!                      This possibility is only available in the case CHOICE=2.
!                      Default value: 0
!                 11:  (TYPE_LEGENDA) defines the type of legenda to be plotted
!                      Possible values:
!                      1:  No legenda
!                      2:  Standard legenda; the text is plotted at the right
!                          side of the picture.
!                      3:  Horizontal legenda; the text plotted below the
!                          picture. Only minimum and maximum values are given.
!                          Only available for CHOICE=2
!                      Default value: 2
!                 12:  (TYPE_SCALES) defines whether the scaling factors are
!                      plotted (1) or not (2)
!                      Default value: 1
!                 13:  (ISKIP) indicates if element groups must be excluded (>0)
!                      or not (0)
!                      If ISKIP>0, the value of ISKIP defines the first position
!                      in INPUT where information about the element groups
!                      to be excluded is stored
!                      Default value: 0
!                 14:  IDEFFM   Indicates if default sizes of the plot are used
!                      (IDEFFM=0) or that the user gives these values
!                      in RPLOTS(1) and RPLOTS(2) (IDEFFM=1)
!                      If IDEFFM=2 the user must also fill RPLOTS(3) with a
!                      premultiplication factor for the solution
!                      The effect of the multiplication factor is that the
!                      contour levels are plotted for the scaled solution, &
!                      the solution itself, however, is not influenced
!                      If the user gives the contour levels explicitly these
!                      contour levels must correspond to the scaled vector
!                      Default value: 0
!                 15:  (IMESH) indicates if the mesh must be plotted in the
!                      picture (>0) or not (0)
!                      If IMESH>0, the value of IMESH defines the color
!                      sequence number with which the mesh must be plotted.
!                      In 3D only the visible parts of the mesh are plotted
!                      with color IMESH
!                      If in 3D CHOICE=2 is used, plotting of the mesh means
!                      plotting of triangles. In the case of quadrilaterals
!                      or quadratic elements this may mean that the mesh is
!                      subdivided into smaller triangles
!                      Default value: 0
!                 16:  (INVIS_MESH) indicates if the invisible part of the
!                      mesh must be plotted in the picture (>0) or not (0)
!                      If INVIS_MESH>0, the value of INVIS_MESH defines the
!                      color sequence number with which the invisible parts
!                      of the mesh must be plotted. (CHOICE=1 and 3D only)
!                      Default value: 0
!                 17:  (INVIS_CONT) indicates if the invisible part of the
!                      contour lines must be plotted in the picture (>0) or not
!                      (0). If INVIS_CONT>0, the value of INVIS_CONT defines the
!                      color sequence number with which the invisible parts
!                      of the contours must be plotted. (CHOICE=1 and 3D only)
!                      Default value: 0
!                 18:  (BACK_COLOUR) indicates if back ground lines must
!                      plotted in the picture (>0) or not (0)
!                      If BACK_COLOUR>0, the value of BACK_COLOUR defines the
!                      color sequence number with which the back ground lines
!                      must be plotted. (CHOICE=1 and 3D only)
!                      Default value: 0
!                 19:  (CURV_COLOUR) indicates if curves must be
!                      plotted in the picture (>0) or not (0)
!                      If CURV_COLOUR>0 and<100, the value of CURV_COLOUR
!                      defines the color sequence number with which the
!                      visible part of the curves must be plotted.
!                      If CURV_COLOUR>100 also the invisible part of the
!                      curves is plotted, the value of
!                      CURV_COLOUR-100 defines the color sequence number
!                      with which the curves must be plotted.
!                      This possibility is only available for 3D
!                 20:  (ITEXT) indicates if text must be plotted in the
!                      picture.
!                      Possible values:
!                      0:  No text at all
!                      1:  Extra text at bottom
!                      2:  Extra text at bottom and top
!                      Default value: 1
!                 21:  (ICOMPT) indicates how the contour levels must be
!                      computed from NCNTLN in the case NCNTLN<=0
!                      0:  The minimum and maximum level are computed
!                      1:  The minimum level is given in contln(6), the
!                          maximum level must be computed
!                      2:  The maximum level is given in contln(7), the
!                          minimum level must be computed
!                      3:  The minimum level is given in contln(6), the
!                          maximum level in contln(7)
!                      Default value: 0
!                 22:  (IORIENT) indicates the orientation of the base vectors
!                      Possible values:
!                      1 : x - y - z system (also default)
!                      2 : z - x - y system
!                      3 : y - z - x system
!                      Default value: 1
!                 23:  (NLEV_POS) indicates the number of positive levels
!                      for ncntln = -10000 or -10001
!                      Default value: 10
!                 24:  (NLEV_NEG) indicates the number of negative levels
!                      for ncntln = -10000 or -10001
!                      Default value: 10
!    ncoorc    o   This variable length array contains at output for every
!                  contour the following information from position 6
!                  For any contour level n+1 positions are used, where n is
!                  the number of parts the contour levels contain. A part is a
!                  continuous line. The contours are stored sequentially, &
!                  starting with contour level 1.
!                  pos. 1  contains the number n for the contour level (n>=1)
!                  pos. 2- n+1 contain of each part the number of points in the
!                  contour level part of which the co-ordinates are stored in
!                  array COORCN.
!                  If the contour part is a closed contour, then minus the
!                  number of points on the contour is stored.
!                  So the information of contour level 1 is stored in NCOORC
!                  from position 6 to position 6 + n1 etc.
!    rplots    i   Double precision input array that must be filled as follows:
!                  1:    plotfm
!                  2:    yfact
!                  3:    scale factor
!                        This parameter is only used if IDEFFM=2
!                  4-6:  eye-point  (Only if PROJECT>9)
!    text      i   Text array containing the texts to be plotted
!                  1:  Text at bottom of the picture
!                  2:  Text at top of the picture
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lnplot
      parameter (lnplot=24)
      integer ilplot(lnplot), i, iplmin(lnplot), iplmax(lnplot)
      double precision rlplot(6)
      character (len=8) chplot(lnplot)

!    chplot   Contains the names of the variables corresponding to array ILPLOT
!    i        General loop variable
!    ilplot   Copy of array iplots extended with all default values
!    iplmax   Contains the maximum admitted values of array ILPLOT
!    iplmin   Contains the minimum admitted values of array ILPLOT
!    lnplot   Length of array ilplot
!    rlplot   Copy of array rplots extended with all default values
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRCHR  Put character in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     PLCN00  Actual contour plotting subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1829   IPLOTS(i) incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear arrays ILPLOT and RLPLOT
!   Copy necessary parts of IPLOTS and RPLOTS into ILPLOT and RLPLOT
!   Extend arrays IPLOTS and RPLOTS with default values
!   Check arrays
!   Call contouring subroutine
! **********************************************************************
!
!                       DATA STATEMENTS
!
!                   1    2     3    4    5    6      7      8    9   10
      data iplmin / 0,   1,    1,-10001, 0,   0,     0,     0,   0,   0, &
                    1,   1,    0,   0,   0,   0,     0,     0,   0,   0, &
                    0,   1,    2,   2 /

!                   1    2     3    4    5    6      7      8    9   10
      data iplmax / lnplot, 4, 100,1000, 100, 11,    5,     1,  11,1000, &
                    3,   2, 1000,   2,1000,1000,  1000,  1000,1000,   2, &
                    3,   3,  200, 200 /

!                            1         2          3          4         5
      data chplot /       ' ', 'CHOICE',  'NUMBER',  'NCNTLN','JINDCOL', &
                    'PROJECT','JSMOOTH', 'IDOUBLE',   'INNER','PLOT_CON' &
                  ,'TYPE_LEG','TYPE_SCA',  'ISKIP',  'IDEFFM',  'IMESH', &
                   'INVIS_ME','INVIS_CO','BACK_COL','CURV_COL', 'ITEXT', &
                    'ICOMPT', 'IORIENT', 'NLEV_POS', 'NLEV_NEG' /

! ======================================================================
!
      call eropen ( 'sepplotcn' )

!     --- Clear arrays ILPLOT and RLPLOT

      ilplot = 0
      ilplot(2)  = 1
      ilplot(3)  = 1
      ilplot(6) = 10
      ilplot(11) = 2
      ilplot(12) = 1
      ilplot(20) = 1
      ilplot(22) = 1
      ilplot(23) = 10
      ilplot(24) = 10
      rlplot = 0d0

!     --- Copy contents from iplots

      do i = 1, min ( iplots(1), lnplot )
         ilplot(i) = iplots(i)
      end do

      if ( ilplot(14)==0 ) then
         rlplot(1) = defplo
         rlplot(2) = 1d0
         rlplot(3) = 1d0
      else if ( ilplot(14)==1 ) then
         rlplot(1) = rplots(1)
         rlplot(2) = rplots(2)
         if ( rlplot(1)>=0d0 .and. rlplot(1)<=0d0 ) rlplot(1)=defplo
         if ( rlplot(2)>=0d0 .and. rlplot(2)<=0d0 ) rlplot(2) = 1d0
         rlplot(3) = 1d0
      else
         rlplot(1) = rplots(1)
         rlplot(2) = rplots(2)
         rlplot(3) = rplots(3)
         if ( rlplot(1)>=0d0 .and. rlplot(1)<=0d0 ) rlplot(1)=defplo
         if ( rlplot(2)>=0d0 .and. rlplot(2)<=0d0 ) rlplot(2) = 1d0
         if ( rlplot(3)>=0d0 .and. rlplot(3)<=0d0 ) rlplot(3) = 1d0
      end if

!     --- Change meaning of ilplot to old value, i.e.
!         0 := 10, 1 := 11, 10 := 0, 11 := 1

      if ( ilplot(6)<10 ) then
         rlplot(4) = rplots(4)
         rlplot(5) = rplots(5)
         rlplot(6) = rplots(6)
         ilplot(6) = ilplot(6) + 10
      else
         ilplot(6) = ilplot(6) - 10
      end if

!     --- Check some values of iplots

      do i = 1, lnplot

         if ( ilplot(i)<iplmin(i) .or. ilplot(i)>iplmax(i) ) then

!        --- Error coefficient i out of range

            call errint ( i, 1 )
            call errint ( iplots(i), 2 )
            call errint ( iplmin(i), 3 )
            call errint ( iplmax(i), 4 )
            call errchr ( chplot(i), 1 )
            call errsub ( 1829, 4, 0, 1 )
         end if

      end do

!     --- Call actual plotting subroutine

      call plcn00 ( ilplot, rlplot, contln, input, text, ncoorc, coorcn )

      call erclos ( 'sepplotcn' )
      end subroutine sepplotcn
