      subroutine sepplotinters
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     plot and print intersection points
!     Old subroutine plotinters
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     node
!     level_set
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduledebug
      use sepmoduleplot
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iscolr, ikader

!     ikader         Help variable to store jkader temporarily
!     iscolr         Help variable to store the value of dcolor temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLELM1         Plot elements in 2D mesh
!     PLOTBN         Plot boundary of region
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPPLOTCROSSOB Plot the contents of the arrays icrossobst and rcrossobst
!     SEPPRCROSSOBST Print the contents of the arrays icrossobst and rcrossobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplotinters' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepplotinters'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- activate arrays

      if ( nmesh(33)==0 ) call sepfillnmeshc

      if ( debug  .or. printobst ) call sepprcrossobst

      if ( debug .or. plotobst .and. ndim>1 .and. ndim<3 ) then

!     --- open plot (ndim>1)

         textx = 'x'
         texty = 'y'
         call pl0003 ( ikader, 1, 15d0, 1d0, 10 )

!        --- plot elements

         jmark = 5
         call plelm1 ( nmesh, coor, .true., .true., .false. )

!        --- plot boundary

         iscolr = dcolor
         dcolor = 1
         call plotbn ( 0, 3, .true. )
         dcolor = iscolr

!        --- plot cross sections with obstacle

         call sepplotcrossob

!        --- close plot dataset

         call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

      end if  ! ( debug .or. plotobst )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepplotinters'

      end if  ! ( debug )
      call erclos ( 'sepplotinters' )

      end subroutine sepplotinters
