      subroutine sepplotm2 ( ichois, iuser, plotfm, yfact )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.10 date 29-05-2014 Extension of timebefore
!        version  1.9  date 15-11-2013 New definition ichois
!        version  1.8  date 12-09-2013 Add plottime
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot two-dimensional mesh
!     Remark:  if this subroutine is changed, also change PLOTM2PAR !!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, iuser(*)
      double precision plotfm, yfact

!     ichois  i    parameter to distinguish between the following
!                  possibilities
!                  ichois consists of the following parts:
!                  plotmesh, fillcl, plotboundouble, plotboun
!                  according to
!                  ichois = plotmesh+5*fillcl+10*plotboundouble+100*plotboun
!                  Meaning of the different parts:
!                  plotmesh:
!                     0:  standard method
!                     1:  only a part of the mesh is plotted.
!                         array iuser is used.  see programmers guide
!                     2:  the mesh is contracted
!                  fillcl:
!                     0:  standard method
!                     1:  the interior of the elements is filled with a color, &
!                         depending on the element group.
!                         Each next elemen group gets the next color from the
!                         color list. The first color is given by the parameter
!                  plotboundouble:
!                     0:  standard method
!                     1:  the boundaries are also plotted with double lines
!                  plotboun:
!                     0,3:  all boundaries are plotted
!                     1:  only the outer boundaries are plotted
!                     2:  only the inner boundaries are plotted
!                     4:  no boundaries are plotted
!     iuser   i    Integer user array to indicate if element group IELGRP must
!                  be skipped (iuser(ielgrp+5)=0) or not (iuser(ielgrp+5)=1)
!     plotfm  i    Plotting factor. When PLOTFM<0, then the length of
!                  the plot is computed by - (xmax-xmin) * PLOTFM
!                  and the width by - (ymax-ymin) * YFACT * PLOTFM
!                  else the length of the plot is equal to PLOTFM cm
!                  When by some mistake the user has given | PLOTFM | a value
!                  smaller than 1 cm, then the length is chosen equal to
!                  the default length stored in common block consta
!     yfact   i    Multiplication for the y-co-ordinates. With this
!                  parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ncontr, part, plboun, plrenm, fillcl, bouncurv
      integer ikader, imark, iscolr, plotboun
      double precision :: timebefore(2)

!     bouncurv       if true the 2D boundaries are plotted using icurvs/curves
!                    otherwise using kmeshm and coor
!     fillcl         Indicates if the elements must be filled with a color
!                    (true) or that only the boundaries must be plotted (false)
!                    If fillcl = true, the color is given by dcolor, the
!                    boundaries get the default color. Otherwise the color of
!                    the boundaries is defined by dcolor
!     ikader         Help variable to store jkader temporarily
!     imark          jmark mod 10
!     iscolr         Help variable to store the value of dcolor temporarily
!     ncontr         indication whether the elements are contracted (false)
!                    or not (true)
!     part           indication whether all elements are plotted (true)
!                    or that some are skipped (false)
!     plboun         Indicates if the boundaries must be plotted (true) or not
!     plotboun       See ichois
!     plrenm         Indicates if the renumbered nodes must be plotted (true)
!                    or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     INITS5         Compute parameters for the plotting subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLELM1         Plot elements in 2D mesh
!     PLOTTIME       Plot the actual time in a plot depending on the parameters
!                    itimdep and t in common ctimen
!     PLPOIN         plot nodal points
!     PROPENPLOT     Open plot and plot boundary (2d)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplotm2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepplotm2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( itime>0 ) call ersettime ( timebefore )
      iscolr = dcolor

      if ( debug ) write(irefwr,*) 'before inits5'
      call inits5 ( iuser, ncontr, jmark, imark, &
                    part, plboun, ichois, plrenm, fillcl )
      plotboun = ichois/100
      if ( plotboun==0 ) plotboun = 3

!     --- Open plot

      bouncurv = part .and. .not. plboun  ! Only in this case we use kmeshh

      if ( iscolr>0 ) then
         dcolor = nelgrp+2+iscolr
      else
         dcolor = 1
      end if

      if ( debug ) write(irefwr,*) 'before propenplot'
      call propenplot ( bouncurv, plotfm, yfact, ikader, plotboun )

!     --- plot nodal points

      if ( plrenm ) npoint = -npoint

      if ( iscolr>0 ) dcolor = iscolr + 1
      if ( debug ) write(irefwr,*) 'before plpoin'
      if ( imark/=2 .and. imark/=5 ) &
         call plpoin ( coor, npoint, kmeshj, 2 )
      npoint = abs(npoint)

!     --- plot elements

      if ( debug ) write(irefwr,*) 'before plelm1'
      call plelm1 ( iuser(6), coor, ncontr, part, fillcl )

!     --- close plot dataset

      if ( debug ) write(irefwr,*) 'before plottime'
      call plottime  ! plot time label if time dependent

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepplotm2'

      end if  ! ( debug )
      call erclmn ( 'sepplotm2', timebefore )
      jkader = ikader
      dcolor = iscolr
      end subroutine sepplotm2
