      subroutine to0030 ( itypvc, ichw, ich1, vectr1, coor, coorc, iwork, &
                          npoint, nelem, nunkp1, ikprf1, kpref1, sch )
! ======================================================================
!
!        programmer    jaap van der zanden
!        version  2.2  date 27-11-1992 Removal of cconst
!        version  2.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  2.0  date 12-03-1991 addition itypvc, coorc;errsub
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   construct vector from vectr1,vectr2 for plotting
!   for vector to be plotted (ich1 from vectr1 together with
!   degree of freedom ich2 of vectr2 form the vector)
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itypvc,ichw,ich1,iwork(*),npoint,nelem,nunkp1, &
              ikprf1,kpref1(*)
      double precision vectr1(*), coor(*), coorc(*), sch

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     coorc      i    array with coordinates of centroid of elements
!     ich1       i    degree of freedom ich1 from vectr1 together with
!                     imaginary part of it form
!                     the vector to be plotted
!     ichw       i    ichw=0 iwork should not be used
!     ikprf1     i    ikprf1=0 number of degrees of freedom is constant (nunkp1)
!                     .ne.0  degrees of freedom from kpref1
!     itypvc     i    Indication of the type of solution vector:
!                     110: solution vector
!                     115, 116: vectors of special structure
!     iwork      i    integer work array
!     kpref1     i    array with renumbering for vectr1 (ikprh1.ne.0)
!     nelem      i    Number of elements
!     npoint     i    Number of nodal points
!     nunkp1     i
!     sch        i    scale factor to be applied to constructed vector
!     vectr1     i    standard sepran vector (type 110,115) defined
!                     in any nodal point, or (type=116) per element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, idegfd, ih, j1, nusol, nusol1
      double precision v1, v2, x, y

!     i          Counting variable
!     idegfd     Degree of freedom sequence number
!     ih         Help variable to store a constant temporarily
!     j1
!     nusol      Total number of unknowns
!     nusol1
!     v1
!     v2
!     x          x co-ordinate of point
!     y          y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRSUB    Error messages
!     PLAFEP    General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     216   degree of freedom does not exist, &
!           parameter idgfd1 or idgfd2 is too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- tests

      call eropen( 'to0030' )
      if ((ikprf1.eq.0).and.(ich1.gt.nunkp1)) then
         call errsub ( 216, 0, 0, 0)
         goto 2001
      end if
      j1=1
      if (itypvc.eq.110 .or. itypvc.eq.115) then
         do 1000 i=1,npoint

!        --- find degree of freedom in vectr1 in point i

            if ((ichw.ne.0).and.(iwork(i).eq.-1)) goto 1000
            if (ikprf1.eq.0) then
               idegfd=(i-1)*nunkp1+ich1
            else
               nusol=kpref1(i)
               nusol1=kpref1(i+1)
               idegfd=nusol+ich1
               j1=0
               if (idegfd.le.nusol1) j1=1
            end if
            if (j1.eq.1) then
               ih=2*idegfd
               v1=vectr1(ih-1)*sch
               v2=vectr1(ih)*sch
               ih=i+i
               x=coor(ih-1)
               y=coor(ih)

!              --- plot vector

               call plafep(8,x,y,x+v1,y+v2,0)
            end if
1000     continue
      else if (itypvc.eq.116) then
         do 2000 i=1,nelem

!        --- find degree of freedom in vectr1 in element i

            idegfd=(i-1)*nunkp1+ich1
            ih=2*idegfd
            v1=vectr1(ih-1)*sch
            v2=vectr1(ih)*sch
            ih=i+i
            x=coorc(ih-1)
            y=coorc(ih)
            if ( x.ge.xmin .and. x.le.xmax .and. &
                 y.ge.ymin .and. y.le.ymax ) then

!           --- plot vector

               call plafep(8,x,y,x+v1,y+v2,0)
            end if
2000     continue
      end if
2001  call erclos( 'to0030' )
      end
