      subroutine topsubdivtri1 ( kmeshc, kmeshctmp, inelem, mult, &
                                 ishape, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide mesh into triangles for one element group
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
!     2d
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inelem, mult, ishape, inpelm, &
                             kmeshc(inpelm,inelem)
      integer, intent(out) :: kmeshctmp(3*mult,inelem)

!     inelem         i    Number of elements in a specific element group
!     inpelm         i    number of points per element
!     ishape         i    Shape number of elements
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshctmp      o    temporary array kmeshc
!     mult           i    Number of triangles an element is subdivided i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topsubdivtri1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topsubdivtri1'
         write(irefwr,1) 'ishape', ishape
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case(ishape)
      case(3, 10)

!     --- ishape = 3 or 10, (extended) linear triangle

         kmeshctmp(1:3,1:inelem) = kmeshc(1:3,1:inelem)

      case(4, 7)

!     --- ishape = 4 or 7, (extended) quadratic triangle

         if ( mult==1 ) then

!        --- mult = 1, use vertices

            do i = 1, 3
               kmeshctmp(i,1:inelem) = kmeshc(2*i-1,1:inelem)
            end do  ! i = 1, 3

         else

!        --- mult = 4, subdivide in 4 elements

            kmeshctmp(1:2,1:inelem) = kmeshc(1:2,1:inelem)
            kmeshctmp(3,1:inelem) = kmeshc(6,1:inelem)
            kmeshctmp(4:6,1:inelem) = kmeshc(2:4,1:inelem)
            kmeshctmp(7:9,1:inelem) = kmeshc(4:6,1:inelem)
            kmeshctmp(10,1:inelem) = kmeshc(2,1:inelem)
            kmeshctmp(11,1:inelem) = kmeshc(6,1:inelem)
            kmeshctmp(12,1:inelem) = kmeshc(4,1:inelem)

         end if  ! ( mult==1 )

      case(5, 9)

!     --- ishape = 5 or 9, (extended) linear quadrilateral

         kmeshctmp(1:3,1:inelem) = kmeshc(1:3,1:inelem)
         kmeshctmp(4,1:inelem) = kmeshc(1,1:inelem)
         kmeshctmp(5:6,1:inelem) = kmeshc(3:4,1:inelem)

      case(6)

!     --- ishape = 6, quadratic quadrilateral

         if ( mult==8 ) then

!        --- mult = 8, subdivide in 8 elements

            kmeshctmp(1:2,1:inelem) = kmeshc(1:2,1:inelem)
            kmeshctmp(3,1:inelem) = kmeshc(9,1:inelem)
            kmeshctmp(4,1:inelem) = kmeshc(1,1:inelem)
            kmeshctmp(5:6,1:inelem) = kmeshc(8:9,1:inelem)
            kmeshctmp(7:9,1:inelem) = kmeshc(2:4,1:inelem)
            kmeshctmp(10,1:inelem) = kmeshc(2,1:inelem)
            kmeshctmp(11,1:inelem) = kmeshc(4,1:inelem)
            kmeshctmp(12,1:inelem) = kmeshc(9,1:inelem)
            kmeshctmp(13:15,1:inelem) = kmeshc(6:8,1:inelem)
            kmeshctmp(16,1:inelem) = kmeshc(6,1:inelem)
            kmeshctmp(17:18,1:inelem) = kmeshc(8:9,1:inelem)
            kmeshctmp(19,1:inelem) = kmeshc(9,1:inelem)
            kmeshctmp(20:21,1:inelem) = kmeshc(4:5,1:inelem)
            kmeshctmp(22:23,1:inelem) = kmeshc(5:6,1:inelem)
            kmeshctmp(24,1:inelem) = kmeshc(9,1:inelem)

         else if ( mult==6 ) then

!        --- mult = 6, subdivide in 6 elements

            kmeshctmp(1:2,1:inelem) = kmeshc(1:2,1:inelem)
            kmeshctmp(3,1:inelem) = kmeshc(8,1:inelem)
            kmeshctmp(4:6,1:inelem) = kmeshc(2:4,1:inelem)
            kmeshctmp(7:9,1:inelem) = kmeshc(4:6,1:inelem)
            kmeshctmp(10,1:inelem) = kmeshc(1,1:inelem)
            kmeshctmp(11:12,1:inelem) = kmeshc(7:8,1:inelem)
            kmeshctmp(13,1:inelem) = kmeshc(2,1:inelem)
            kmeshctmp(14,1:inelem) = kmeshc(6,1:inelem)
            kmeshctmp(15,1:inelem) = kmeshc(8,1:inelem)
            kmeshctmp(16,1:inelem) = kmeshc(2,1:inelem)
            kmeshctmp(17,1:inelem) = kmeshc(6,1:inelem)
            kmeshctmp(18,1:inelem) = kmeshc(4,1:inelem)

         else

!        --- mult = 2, subdivide in 2 elements

            do i = 1, 3
               kmeshctmp(i,1:inelem) = kmeshc(2*i-1,1:inelem)
            end do  ! i = 1, 3
            kmeshctmp(4,1:inelem) = kmeshc(1,1:inelem)
            kmeshctmp(5,1:inelem) = kmeshc(5,1:inelem)
            kmeshctmp(6,1:inelem) = kmeshc(7,1:inelem)

         end if  ! ( mult==8 )

      end select  ! case(ishape)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topsubdivtri1'

      end if  ! ( debug )
      call erclos ( 'topsubdivtri1' )

      end subroutine topsubdivtri1
