      subroutine creatv116grcon ( vector, rincre, kdeg, kfun, nphys, &
                                  ielem, inelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-12-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill vector of type 116 for specific element group
!     Special case a constant is filled
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kdeg, kfun, nphys, ielem, inelem
      double precision vector(nphys,*), rincre(*)

!     ielem          i    Element number
!     inelem         i    Number of elements in a specific element group
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     rincre         i    Double precision array containing the values of the
!                         constant if KFUN<0
!     vector        i/o   Vector to be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision consta

!     consta         constant to be filled
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'creatv116grcon' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from creatv116grcon'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      consta = rincre(-kfun)

      if ( kdeg==0 ) then

!     --- All degrees of freedom

         vector(1:nphys,ielem+1:ielem+inelem) = consta

      else

!     --- One degree of freedom

         vector(kdeg,ielem+1:ielem+inelem) = consta

      end if  ! ( kdeg==0 )

1000  call erclos ( 'creatv116grcon' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End creatv116grcon'

      end if  ! ( debug )

      end
