      subroutine creatvspec116 ( kfun, nphys, usol, icompl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 10-02-2012 use getnpelmelgrp
!        version  2.0  date 27-12-2011 New parameter list
!        version  1.0  date 17-11-2000
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array usol with values in the case that 1000<fkun<2000
!     This means that a loop over the elements is performed and that
!     the subroutines func1b or cfun1b are called
!     This subroutine is ony implemented for kfill = 0 (all nodes)
!     usol must be of type 110 or 115
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kfun, nphys, icompl
      double precision usol(*)

!     icompl         i    Indication if the vector to be created is real (0)
!                         or complex
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     usol          i/o   Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ichoice, ielem, ielgrp, ikelmc, &
              inpelm, iielem, index2(nphys)
      integer, allocatable :: index1(:)

!     i              Counting variable
!     ichoice        Integer choice parameter
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     index1         Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     index2         Array of length icount containing the positions of the
!                    degrees of freedom in the element with respect to USOL
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CFUN1B         User subroutine to fill boundary conditions
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     FUNC1B         User subroutine to fill boundary conditions
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initializations
!    For all element groups do
!       For all elements in element group do
!          Fill index1 and index2
!          call user subroutine
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'creatvspec116' )

!     --- Initializations

      ikelmc = 0
      ichoice = kfun-1000

      allocate ( index1(npelm), stat = error )
      if ( error/=0 ) call eralloc ( error, npelm, 'index1' )
      if ( ierror/=0 ) go to 1000

!     --- For all element groups do

      ielem = 0
      do ielgrp = 1, nelgrp

         inpelm = sepgetinpelm ( ielgrp )
         inelem = sepgetinelem ( ielgrp )

!        --- For all elements in element group do

         do iielem = 1, inelem

!        --- Fill index1 and index2

            ielem = ielem+1
            index1(1:inpelm) = kmeshc(ikelmc+1:ikelmc+inpelm)
            do i = 1, nphys
               index2(i) = (ielem-1)*nphys+i
            end do

!           --- call user subroutine

            if ( icompl==1 ) then

!           --- complex case

               call cfun1b ( ichoice, index1, index2, usol, coor )

            else

!           --- real case

               call func1b ( ichoice, index1, index2, usol, coor )

            end if

            ikelmc = ikelmc+inpelm

         end do

      end do

      deallocate ( index1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'index1' )

1000  call erclos ( 'creatvspec116' )
      end
