      subroutine eligrinf ( larray )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 11-11-2014 Extension with type 700
!        version  2.0  date 21-12-2010 Remove ielrou
!        version  1.3  date 18-10-2006 Type 807
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill general information for a specific element group, depending
!     on the type number. Compute ielrou, larray and liarry
!     Help subroutine for integs
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer larray

!     larray     o    length of array array in element subroutines
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      43   Type number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eligrinf' )
      larray = 0
      if ( itype<=0 ) go to 1000

      select case ( itype )

         case ( 1:101, 104, 105, 108:111, 118, 190, 300, 400, 404, &
                420, 421, 502:504 )

!        --- No action

         case ( 113, 115, 250, 505:507 )

            larray = 320

         case ( 150, 151, 153, 155, 158:161, 163, 325, 700:703, 705, &
                800, 803, 807 )

            larray = 7857

         case ( 900:903, 410 )

            larray = 5778

         case ( 401, 405, 914 )

            notvec=1

         case default

!        --- Type number not available

            call errint ( itype, 1 )
            call errint ( ielgrp, 2 )
            call errsub ( 43, 2, 0, 0)

      end select  ! case ( itype )

1000  call erclos ( 'eligrinf' )

      end
