      subroutine elmsubgl ( coor, elemmt, elemvc, index1, &
                            ishape, index4 )
! ======================================================================
!
!        programmer    Guus Segal
!        version 3.0   date 16-01-1999 Extra parameter index4
!        version 2.0   date 28-12-1998 Remove ndim
!        version 1.0   date 12-09-1998
!
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call specific element subroutine for global unknowns, depending on itype
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!    element_group
!    navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleuser
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), ishape, index4(*)
      double precision elemmt(*), elemvc(*), coor(*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!                     x  = coor (1,i);  y  = coor (2,i);  z  = coor (3,i);
!                      i                 i                 i
!     elemmt     o    Element matrix to be computed
!     elemvc     o    Element vector to be computed
!     index1     i    Array of length inpelm containing the point numbers
!                     of the nodal points in the element
!     index4     i    Array of length inpelm+1 containing the starting
!                     positions of the nodes in array index2 minus 1.
!                     index4(1) = 0, index4(inpelm) = icount
!     ishape     i    Shape number of elements
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELM913    Compute element matrix and vector for given mass flux
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      43   Type number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmsubgl' )
      if ( itype.lt.913 .or. itype.gt.913 ) go to 999

      call elm913 ( coor, elemmt, elemvc, iuser, user, index1, &
                    ishape, index4 )
      goto 1000

!     --- error 43: type number not available

999   call errint ( itype, 1 )
      call errint ( ielgrp, 2 )
      call errsub ( 43, 2, 0, 0 )

1000  if ( ierror.ne.0 ) call instop
      call erclos ( 'elmsubgl' )
      end
