      subroutine er0008 ( usol, ichoice, &
                          ipnt1, ipnt2, idegfd, jstap, value, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.2  date 18-08-1992 iabs==>abs
!        version  4.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  0.4  date 17-06-1987
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of essential boundary conditions in the solution vector
!     for user points     one component only
!     this subroutine is called by bvalue
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical compl
      integer ichoice, ipnt1, ipnt2, idegfd, jstap
      double precision usol(*), value

!     compl      i    indication whether the arithmetic is real (false)
!                     or complex (true)
!     ichoice    i   choice parameter.  possibilities:
!                    0      the degree of freedom gets the constant value of the
!                           parameter value
!                    >0     variable values of the degrees of freedom.
!                           subroutine funcbc is called.
!                    <0     complex values of the degrees of freedom
!                           subroutine cfuncb is called
!     idegfd     i    Degree of freedom sequence number
!     ipnt1      i    all user points with numbers ipnt1 to ipnt2 are used
!                     depending on jstap
!     ipnt2      i    See ipnt1
!     jstap      i    See ipnt1
!     usol      i/o   Solution vector
!     value      i   value of the constant when the boundary condition
!                    is constant
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, ih1, nodenr, j, kstap, n, inusol

!     i          Counting variable
!     ih         Help variable to store a constant temporarily
!     ih1        Help variable to store a constant temporarily
!     inusol     Position in usol
!     j          Counting variable
!     kstap
!     n
!     nodenr     Nodal point number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPINIBVL      actual filling of essential boundary condition for one point
!                    and one degree of freedom only
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      23   first and last user point are not in the same way prescribed
!     180   user point is not a nodal point
!     612   first point number has wrong value
!     613   last point number has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0008' )
      if ( ipnt1<=0 .or. ipnt1>nuspnt ) then

!     --- ipnt1 has wrong value

         call errint ( ipnt1, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 612, 2, 0, 0 )

      end if
      if(ipnt2<=0 .or. ipnt2>nuspnt) then

!     --- ipnt1 has wrong value

         call errint ( ipnt2, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 613, 2, 0, 0 )

      end if

      if ( ierror/=0 ) go to 1000
      kstap=abs(jstap)

!     --- kstap > 0  check

      n=(ipnt2-ipnt1)/(kstap+1)
      if ( n*(kstap+1)/=ipnt2-ipnt1 ) then

!     --- kstap has wrong value

         call errint ( ipnt1, 1 )
         call errint ( ipnt2, 2 )
         call errint ( jstap, 3 )
         call errint ( n, 4 )
         call errsub ( 23, 4, 0, 0 )
         go to 1000

      end if

      if ( jstap>=0 ) then

!     --- jstap >= 0

         kstap=kstap+1
         do i=ipnt1,ipnt2,kstap
            nodenr=kmeshl(i)
            if ( nodenr==0 ) then

!           --- ipnt1 has wrong value

               call errint ( i, 1 )
               call errsub ( 180, 1, 0, 0 )

            else
               inusol = nodenr
               if ( indprp>0 ) inusol = kprobp(inusol,idegfd)
               if ( inusol>0 ) &
                  call sepinibvl ( ichoice, nodenr, usol, inusol, idegfd, &
                                   value, compl )
            end if
         end do

      else

!     --- jstap < 0

         ih=kstap+1
         ih1=ipnt2-1
         do i=ipnt1,ih1,ih
           do j=1,kstap
              nodenr=kmeshl(i+j)
              if ( nodenr==0 ) then

!             --- ipnt1 has wrong value

                 call errint ( i+j, 1 )
                 call errsub ( 180, 1, 0, 0 )

              else
                 if ( indprp>0 ) inusol = kprobp(inusol,idegfd)
                 if ( inusol>0 ) &
                    call sepinibvl ( ichoice, nodenr, usol, inusol, idegfd, &
                                     value, compl )
              end if
           end do
         end do

      end if
1000  call erclos ( 'er0008' )
      end
