      subroutine er0012 ( usol, ichois, isrf1, isrf2, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.5  date 24-07-2015 Replace to0000 by to00002
!        version  4.4  date 25-03-2011 New call to to0000
!        version  4.3  date 03-06-2006 New call to to0000
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of essential boundary conditions in the solution vector
!     for surfaces    elementwise
!     this subroutine is called by bvalue
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     essential_boundary_condition
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical compl
      integer :: ichois, isrf1, isrf2
      double precision usol(*)

!     compl   i    indication whether the arithmetic is real (false)
!                  or complex (true)
!     ichois  i    Choice parameter, see bvalue
!     isrf1   i    First surface number
!     isrf2   i    Last surface number
!     usol   i/o   Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jchois, j, istart, jnpelm, jnelem
      integer, allocatable :: index1(:), index2(:)

!     index1       work array index1  for subroutine cfun1b or func1b
!     index2       work array index2  for subroutine cfun1b or func1b
!     istart       First position used at a part
!     j            Counting variable
!     jchois       ?
!     jnelem       number of elements in an element group for one super element
!     jnpelm       Number of nodes in an element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CFUN1B         User subroutine to fill boundary conditions
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNC1B         User subroutine to fill boundary conditions
!     TO00002        Fill arrays index1 and index2
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     185   first curve number (icrv1) is out of range
!     186   last curve number (icrv2) is out of range
!     272   number of nodal points in elements of the same group for the
!           generation of essential boundary conditions, not the same.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0012' )
      if ( isrf1<=0 .or. isrf1>nsurfs ) then
          call errint ( isrf1, 1 )
          call errint ( nsurfs, 2 )
          call errsub ( 185, 2, 0, 0 )
      end if
      if ( isrf2<=0 .or. isrf2>nsurfs ) then
          call errint ( isrf2, 1 )
          call errint ( isrf1, 2 )
          call errint ( nsurfs, 3 )
          call errsub ( 186, 3, 0, 0 )
      end if
      if ( nunkp==1 ) then
         if ( indprf==0 ) jchois=1
         if ( indprf>0 ) jchois=2
      else
         if ( indprf==0 ) jchois=3
         if ( indprf>0 ) jchois=4
      end if
      itype=0

      allocate ( index1(npelm), index2(npelm*nunkp), stat = error )
      if ( error/=0 ) call eralloc ( error, npelm*nunkp, 'index1' )
      if ( ierror/=0 ) go to 1000

!     --- compute inpelm

      inpelm=kmeshn(isrf1)
      if ( isrf2>isrf1 ) then

!     --- isrf2>isrf1  check inpelm

        do j=isrf1,isrf2
           if ( inpelm/=kmeshn(j) ) then
              call errint ( isrf1, 1 )
              call errint ( isrf2, 2 )
              call errint ( 0, 3 )
              call errint ( inpelm, 4 )
              call errint ( 0, 5 )
              call errint ( kmeshn(j), 6 )
              call errsub ( 272, 6, 0, 0 )
           end if
        end do
      end if

!     --- compute the number of elements  inelem

      inelem=0
      do j=isrf1,isrf2
         inelem=inelem+kmeshn(nsurfs+j)
      end do

!     --- compute starting position in kmeshn

      istart=2*nsurfs
      do j=1,isrf1-1
         jnpelm=kmeshn(j)
         jnelem=kmeshn(nsurfs+j)
         istart=istart+jnpelm*jnelem
      end do

      do ielem=1,inelem
         call to00002 ( jchois, kmeshn, istart, &
                        index1, index2, nelgrp, 0, 0, 0, 0 )
         istart = istart+inpelm
         if ( compl ) then
             call cfun1b ( -ichois, index1, index2, usol, coor )
         else
             call func1b ( ichois, index1, index2, usol, coor )
         end if
      end do

      deallocate ( index1, index2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'index1' )

1000  call erclos ( 'er0012' )
      end
