      subroutine er0024 ( usol, ichois, icrv1, &
                          icrv2, jnpelm, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.4  date 24-07-2015 Replace to0000 by to00002
!        version  5.3  date 25-03-2011 New call to to0000
!        version  5.2  date 03-06-2006 New call to to0000
!
!   copyright (c) 1985-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of essential boundary conditions in the solution vector
!     for curves    elementwise
!     this subroutine is called by bvalue
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     essential_boundary_condition
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical compl
      integer :: ichois, icrv1, icrv2, jnpelm
      double precision usol(*)

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     ichois         i    Choice parameter
!     icrv1          i    First curve number
!     icrv2          i    Last curve number
!     jnpelm         i    Number of nodes in an element
!     usol          i/o   Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jchois, ikelmc, iboun
      integer, allocatable :: index1(:), index2(:), iwork(:)

!     iboun        boundary element group number
!     ikelmc       position in kmesh part c
!     index1       work array index1  for subroutine cfun1b or func1b
!     index2       work array index2  for subroutine cfun1b or func1b
!     iwork        integer work array of length npoint
!     jchois
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CFUN1B         User subroutine to fill boundary conditions
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     FUNC1B         User subroutine to fill boundary conditions
!     SEPER0026      fill nodal points of curves icrv1 to icrv2 in array iwork
!     TO00002        Fill arrays index1 and index2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0024' )

      allocate ( index1(npelm), index2(npelm*nunkp), &
                 iwork(npoint), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npelm*nunkp+npoint, 'index1' )
      if ( ierror/=0 ) go to 1000
      inpelm=jnpelm
      call seper0026 ( icrv1, icrv2, inpelm-1, n, iwork, .false., kmeshm )
      if(nunkp==1) then
         if(indprf==0) jchois=1
         if(indprf>0) jchois=2
      else
         if(indprf==0) jchois=3
         if(indprf>0) jchois=4
      end if
      itype=0
      ikelmc=0
      iboun = 0
      do ielem=1, inelem
         call to00002 ( jchois, iwork, ikelmc, &
                        index1, index2, nelgrp, iboun, 0, 0, 0 )
         ikelmc=ikelmc+inpelm-1
         if ( compl ) then
             call cfun1b ( -ichois, index1, index2, usol, coor )
         else
             call func1b ( ichois, index1, index2, usol, coor )
         end if
      end do

      deallocate ( index1, index2, iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'index1' )

1000  call erclos ( 'er0024' )
      end
