      subroutine er00751 ( usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 01-04-2000 Extension with fictitious unknowns
!        version  1.1  date 28-12-1998 New contents kprobx
!        version  1.0  date 12-09-1998
!
!   copyright (c) 1998-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Print special unknowns (real case)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision usol(*)

!     usol       i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iglgrp, inunkpgr, inusolgr, j, istkprobx, &
              ielem, ifictgrp, inelem, istkproby

!     i              Counting variable
!     ielem          Element number
!     ifictgrp       fictitious element group number
!     iglgrp         global element group number
!     inelem         Number of elements in a specific element group
!     inunkpgr       Number of global unknowns in a group
!     inusolgr       Position of first global unknown in a group with
!                    respect to solution array
!     istkprobx      Start address in kprobx
!     istkproby      Start address in kproby
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      nglgrp   = 0
      nglunk   = 0
      nfictunk = 0
      if ( indprx>0 ) then
         nglgrp = kprobx(1)
         nglunk = kprobx(2)
      end if
      if ( indpry>0 ) nfictunk = kproby(2)
      inusolgr = nusol-nglunk-nfictunk
      istkproby = 1
      if ( nfictunk>0 ) then

!     --- nfictunk > 0, fictitious unknowns

         write(irefwr,100)
100      format( 'fictious unknowns'/ &
                 'group no. element nr. values per point' )
         nfictgrp = kproby(istkproby)
         istkproby = istkproby+10+nfictgrp

         do ifictgrp = 1, nfictgrp

            inunkpgr = kproby(istkproby+2)
            inelem   = kproby(istkproby+11)

            do ielem = 1, inelem

               if ( ielem==1 ) then
                  write(irefwr,110) &
                   ifictgrp,ielem,(usol(i+inusolgr),i=1,min(4,inunkpgr))
110               format ( 1x, i3,':',i4, ss, 5(1pe13.5e2) )
               else
                  write(irefwr,120) &
                     ielem,(usol(i+inusolgr),i=1,min(4,inunkpgr))
120               format ( 5x,i4, ss, 5(1pe13.5e2) )
               end if
               if ( inunkpgr>4 ) write(irefwr,130) &
                  (usol(i+inusolgr), j=5,inunkpgr)
130            format ( (9x,ss,5(1pe13.5e2)) )
               inusolgr = inusolgr+inunkpgr

            end do
            istkproby = istkproby+12

         end do

      end if

      inusolgr = inusolgr+nfictunk

      if ( indprx>0 ) then

!     --- indprx > 0, global unknowns

         istkprobx = 3
         write(irefwr,200)
200      format('global unknowns'/'groupno    values' )
         do iglgrp = 1, nglgrp
            inunkpgr = kprobx(istkprobx+2)
            write(irefwr,210) &
               (iglgrp,usol(i+inusolgr),i=1,min(4,inunkpgr))
210         format(1x,i5,ss,5(1pe13.5e2))
            if ( inunkpgr>2 ) write(irefwr,220) &
               (usol(i+inusolgr), j=5,inunkpgr)
220         format((19x,ss,4(1pe13.5e2)))
            inusolgr = inusolgr+inunkpgr
            istkprobx = istkprobx+6
         end do

      end if

      end
