      subroutine ercreatsub12 ( isol, kvec, kdeg, jpstin, icompl, inpcre )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 29-12-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array vector in case kfill = 12,
!     i.e. from file with node numbers
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulevecs
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), kvec, kdeg, &
              jpstin, icompl, inpcre(*)

!     icompl         i    Indication if the vector to be created is real (0) or
!                         complex
!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     jpstin         i    Help variable to compute new value of ipstin partly
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kvec           i    Loop variable over the vectors to be created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipos
      character (len=80) filename

!     filename       Name of file
!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0090         Fill nodal values from a file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1675   Wrong value of reference to texts
!    1676   kdeg=0 and inunkp>1
!    1677   Complex vectors have not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatsub12' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatsub12'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kdeg==0 ) then

!     --- kdeg = 0, all degrees of freedom

         if ( nunkp>1 ) then

!        --- more than one unknown per point (not yet implemented)

            call errwar ( 1676, 0, 0, 0 )

         end if
         kdeg = 1

      end if
      if ( icompl/=0 ) call errsub ( 1677, 0, 0, 0 )

      ipos = inpcre(jpstin)
      if ( ipos<=0 .or. ipos>=nextxt ) then

!     --- ipos out of range

         call errint ( jpstin, 1 )
         call errint ( ipos, 2 )
         call errint ( nextxt, 3 )
         call errsub ( 1675, 3, 0, 0 )
         go to 1000

      end if
      filename = texts(ipos)

!     --- Compute pointers to arrays

      usoltemp => ks(isol(kvec))%sol

      call er0090 ( filename, usoltemp, kdeg )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatsub12'

      end if  ! ( debug )
      call erclos ( 'ercreatsub12' )

      end
