      subroutine ervl09(nm,n,eps1,d,e,e2,lb,ub,mm,m,w,z, &
                        ierr,rv1,rv2,rv3,rv4,rv5,rv6)
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 01-08-1983
!
!   copyright (c) 1983-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!        Renamed subroutine tsturm
!     this subroutine is a translation of the algol procedure tristurm
!     by peters and wilkinson.
!     handbook for auto. comp., vol.ii-linear algebra, 418-439(1971).
!     this subroutine finds those eigenvalues of a tridiagonal
!     symmetric matrix which lie in a specified interval and their
!     associated eigenvectors, using bisection and inverse iteration.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer m, n, mm, nm, ierr
      double precision d(n), e(n), e2(n), w(mm), z(nm,mm), rv1(n), eps1, &
                       rv2(n), rv3(n), rv4(n), rv5(n), rv6(n), lb, ub

!     d          i    contains the diagonal elements of the input matrix
!     e          i    contains the subdiagonal elements of the input matrix
!                     in its last n-1 positions.  e(1) is arbitrary.
!     e2        i/o   contains the squares of the corresponding elements of e.
!                     e2(1) is arbitrary.
!                     elements of e2, corresponding to elements of e regarded
!                     as negligible, have been replaced by zero causing the
!                     matrix to split into a direct sum of submatrices.
!                     e2(1) is also set to zero.
!     eps1      i/o   an absolute error tolerance for the computed
!                     eigenvalues.  it should be chosen commensurate with
!                     relative perturbations in the matrix elements of the
!                     order of the relative machine precision.  if the
!                     input eps1 is non-positive, it is reset for each
!                     submatrix to a default value, namely, minus the
!                     product of the relative machine precision and the
!                     1-norm of the submatrix.
!                     eps1 is unaltered unless it has been reset to its
!                       (last) default value.
!     ierr       o    is set to
!                     zero       for normal return, &
!                     3*n+1      if m exceeds mm.
!                     4*n+r      if the eigenvector corresponding to the r-th
!                                eigenvalue fails to converge in 5 iterations.
!     lb         i    defines together with ub the interval to be searched for
!                     eigenvalues
!     m          o    number of eigenvalues determined to lie in (lb,ub)
!     mm         i    should be set to an upper bound for the number of
!                     eigenvalues in the interval.  warning. if more than
!                     mm eigenvalues are determined to lie in the interval, &
!                     an error return is made with no values or vectors found.
!     n          i    order of the matrix
!     nm         i    the row dimension of two-dimensional
!                     array parameters as declared in the calling program
!                     dimension statement
!     rv1             work array
!     rv2             work array
!     rv3             work array
!     rv4             work array
!     rv5             work array
!     rv6             work array
!     ub         i    defines together with lb the interval to be searched for
!                     eigenvalues
!                     if lb is not less than ub, no eigenvalues will be found.
!     w          o    contains the m eigenvalues in ascending order if the
!                     matrix does not split.  if the matrix splits, the
!                     eigenvalues are in ascending order for each submatrix.
!                     if a vector error exit is made, w contains those values
!                     already found
!     z          i    contains the associated set of orthonormal eigenvectors.
!                     if an error exit is made, z contains those vectors
!                     already found.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps2, eps3, eps4, t1, t2, norm, &
                       tst1, tst2, u, uk, v, x0, x1, xu
      integer i, ii, ip, isturm, its, j, jj, k, m1, m2, p, q, r, &
              s, group

!     eps2
!     eps3
!     eps4
!     group
!     i
!     ii
!     ip
!     isturm
!     its
!     j
!     jj
!     k
!     m1
!     m2
!     norm
!     p
!     q
!     r
!     s
!     t1
!     t2
!     tst1
!     tst2
!     u
!     uk
!     v
!     x0
!     x1
!     xu
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     the algol procedure sturmcnt contained in tristurm
!     appears in tsturm in-line.
!
!     calls pythag for  sqrt(a*a + b*b) .
!
!     questions and comments should be directed to burton s. garbow, &
!     mathematics and computer science div, argonne national laboratory
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ierr = 0
      t1 = lb
      t2 = ub

!     --- look for small sub-diagonal entries .

      do i = 1, n
         if (i==1) go to 20
         tst1 = abs(d(i)) + abs(d(i-1))
         tst2 = tst1 + abs(e(i))
         if (tst2>tst1) go to 40
   20    e2(i) = 0d0
   40 end do

!     --- determine the number of eigenvalues
!                     in the interval .

      p = 1
      q = n
      x1 = ub
      isturm = 1
      go to 320
   60 m = s
      x1 = lb
      isturm = 2
      go to 320
   80 m = m - s
      if (m>mm) go to 980
      q = 0
      r = 0

!     --- establish and process next submatrix, refining
!                     interval by the gerschgorin bounds .

  100 if (r==m) go to 1001
      p = q + 1
      xu = d(p)
      x0 = d(p)
      u = 0d0
      do q = p, n
         x1 = u
         u = 0d0
         v = 0d0
         if (q==n) go to 110
         u = abs(e(q+1))
         v = e2(q+1)
  110    xu = min(d(q)-(x1+u),xu)
         x0 = max(d(q)+(x1+u),x0)
         if ( abs(v)<=0d0 ) go to 140
      end do
  140 x1 = epsmac*max(abs(xu),abs(x0))
      if (eps1<=0d0) eps1 = -x1
      if (p/=q) go to 180

!     --- check for isolated root within interval .

      if (t1>d(p) .or. d(p)>=t2) go to 940
      r = r + 1
      z(1:n,r) = 0d0
      w(r) = d(p)
      z(p,r) = 1d0
      go to 940
  180 u = q-p+1
      x1 = u * x1
      lb = max(t1,xu-x1)
      ub = min(t2,x0+x1)
      x1 = lb
      isturm = 3
      go to 320
  200 m1 = s + 1
      x1 = ub
      isturm = 4
      go to 320
  220 m2 = s
      if (m1>m2) go to 940

!     --- find roots by bisection .

      x0 = ub
      isturm = 5
      rv5(m1:m2) = ub
      rv4(m1:m2) = lb

!     --- loop for k-th eigenvalue
!                     for k=m2 step -1 until m1 do -- &
!                     (-do- not used to legalize -computed go to-) .

      k = m2
  250    xu = lb

!        --- for i=k step -1 until m1 do -- .

         do ii = m1, k
            i = m1 + k - ii
            if (xu<rv4(i)) then
               xu = rv4(i)
               go to 280
            end if  ! (xu<rv4(i))
         end do
  280    if (x0>rv5(k)) x0 = rv5(k)

!        --- next bisection step .

  300    x1 = (xu + x0) * 0.5d0
         if ((x0 - xu)<=abs(eps1)) go to 420
         tst1 = 2d0 * (abs(xu) + abs(x0))
         tst2 = tst1 + (x0 - xu)
         if ( abs(tst2-tst1)<=0d0 ) go to 420

!        --- in-line procedure for sturm sequence .

  320    s = p - 1
         u = 1d0
         do i = p, q
            if ( abs(u)>0d0 ) go to 325
            v = abs(e(i)) / epsmac
            if ( abs(e2(i))<=0d0 ) v = 0d0
            go to 330
  325       v = e2(i) / u
  330       u = d(i) - x1 - v
            if (u<0d0) s = s + 1
         end do
         go to (60,80,200,220,360), isturm

!        --- refine intervals .

  360    if (s>=k) go to 400
         xu = x1
         if (s>=m1) go to 380
         rv4(m1) = x1
         go to 300
  380    rv4(s+1) = x1
         if (rv5(s)>x1) rv5(s) = x1
         go to 300
  400    x0 = x1
         go to 300

!        --- k-th eigenvalue found .

  420    rv5(k) = x1
      k = k - 1
      if (k>=m1) go to 250

!     --- find vectors by inverse iteration .

      norm = abs(d(p))
      ip = p + 1
      do i = ip, q
         norm = max(norm, abs(d(i)) + abs(e(i)))
      end do

!     --- eps2 is the criterion for grouping, &
!                     eps3 replaces zero pivots and equal
!                     roots are modified by eps3, &
!                     eps4 is taken very small to avoid overflow .

      eps2 = 1d-3 * norm
      eps3 = epsmac*norm
      uk = q - p + 1
      eps4 = uk * eps3
      uk = eps4 / sqrt(uk)
      group = 0
      s = p
      do k = m1, m2
         r = r + 1
         its = 1
         w(r) = rv5(k)
         x1 = rv5(k)

!        --- look for close or coincident roots .

         if (k==m1) go to 520
         if (x1 - x0>=eps2) group = -1
         group = group + 1
         if (x1<=x0) x1 = x0 + eps3

!        --- elimination with interchanges and
!                        initialization of vector .

  520    v = 0d0
         do i = p, q
            rv6(i) = uk
            if (i==p) go to 560
            if (abs(e(i))<abs(u)) go to 540
            xu = u / e(i)
            rv4(i) = xu
            rv1(i-1) = e(i)
            rv2(i-1) = d(i) - x1
            rv3(i-1) = 0d0
            if (i/=q) rv3(i-1) = e(i+1)
            u = v - xu * rv2(i-1)
            v = -xu * rv3(i-1)
            go to 580
  540       xu = e(i) / u
            rv4(i) = xu
            rv1(i-1) = u
            rv2(i-1) = v
            rv3(i-1) = 0d0
  560       u = d(i) - x1 - xu * v
            if (i/=q) v = e(i+1)
  580    end do
         if ( abs(u)<=0d0 ) u = eps3
         rv1(q) = u
         rv2(q) = 0d0
         rv3(q) = 0d0

!        --- back substitution
!                        for i=q step -1 until p do -- .

  600    do ii = p, q
            i = p + q - ii
            rv6(i) = (rv6(i) - u * rv2(i) - v * rv3(i)) / rv1(i)
            v = u
            u = rv6(i)
         end do

!        --- orthogonalize with respect to previous
!                        members of group .

         if (group==0) go to 700
         do jj = 1, group
            j = r - group - 1 + jj
            xu = 0d0
            do i = p, q
               xu = xu + rv6(i) * z(i,j)
            end do
            rv6(p:q) = rv6(p:q) - xu * z(p:q,j)
         end do
  700    norm = 0d0
         do i = p, q
            norm = norm + abs(rv6(i))
         end do
         if (norm>=1d0) go to 840

!        --- forward substitution .

         if (its==20) go to 960
         if (norm>0d0) go to 740
         rv6(s) = eps4
         s = s + 1
         if (s>q) s = p
         go to 780
  740    xu = eps4 / norm
         do i = p, q
            rv6(i) = rv6(i) * xu
         end do

!        --- elimination operations on next vector
!                        iterate .

  780    do i = ip, q
            u = rv6(i)

!           --- if rv1(i-1)==e(i), a row interchange
!                           was performed earlier in the
!                           triangularization process .

            if ( abs(rv1(i-1)-e(i))>0d0 ) go to 800
            u = rv6(i-1)
            rv6(i-1) = rv6(i)
  800       rv6(i) = u - rv4(i) * rv6(i-1)
         end do
         its = its + 1
         go to 600

!        --- normalize so that sum of squares is
!                        1 and expand to full order .

  840    u = 0d0
         do i = p, q
            u = sqrt(u**2+rv6(i)**2)
         end do
         xu = 1d0 / u
         z(1:n,r) = 0d0
         z(p:q,r) = rv6(p:q) * xu
         x0 = x1
      end do
  940 if (q<n) go to 100
      go to 1001

!     --- set error -- non-converged eigenvector .

  960 ierr = 4 * n + r
      go to 1001

!     --- set error -- underestimate of number of
!                     eigenvalues in interval .

  980 ierr = 3 * n + 1
 1001 lb = t1
      ub = t2
      end
