      subroutine ervl25 ( x, del, iwhere, solved, zer, pol, amu )
! ======================================================================
!
!        programmer    Zdenek Vlasek
!        version  1.1  date 05-12-2012 Fortran 90
!        version  1.0  date 19-02-1991
!
!   copyright (c) 1991-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     purpose: rational (2,1) interpolation of  del(x), &
!                see P+R, part 3.2, formulae (3.7)
!     to be called from ervl20 only
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwhere
      double precision x(3), del(3), zer, pol, amu
      logical solved

!     amu            i    ?
!     del            i    ?
!     iwhere         i    ?
!     pol            i    ?
!     solved         o    solved=.false.  (no good solution)
!                         or   solved=.true.,  zer, pol, amu
!     x              i    x co-ordinate of point
!     zer            i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k
      double precision x1, x3, d1, d3, z, z1, z2, &
                       sum, sum2, det, r1, r3
      logical okmu

!     d1
!     d3
!     det
!     k              Counting variable
!     okmu
!     r1
!     r3
!     sum            Help parameter to compute a sum
!     sum2           Help variable to compute a sum
!     x1
!     x3
!     z
!     z1
!     z2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical ok

!     OK             ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Notice:  P+R (3.7) is symmetric with respect to  zer, amu .
!
!       We know values  del(1..3)  of the function (3.7)  in  x(1..3), &
!       this gives three quadratic equations for  zer, pol, amu .
!
!       Subtract the second equation from the first and from the third, &
!       it leads to system of two linear equations for  pol, zer+amu .
!
!       After the linear system is solved, the second equations becomes
!       quadratic with roots  zer, amu (in any order).
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ok (z) = z>z1 .and. z<z2

      k = abs (iwhere)
      solved = .false.

      z1 = x(k) - x(2)
      z2 = x(k+1) - x(2)
      x1 = x(1) - x(2)
      x3 = x(3) - x(2)
      d1 = del(1) - del(2)
      d3 = del(3) - del(2)
      det = x1 * d3 - x3 * d1

!     --- ... det=0 signalizes, that del(x) is linear and then:
!                   if  del(3)-del(1) = x(1)-x(3), then zer=pol=anywhere
!                                                  else no solution

      if ( abs(det)<=0d0 ) goto 100

      r1 = x1 * ( del(1) + x1 )
      r3 = x3 * ( del(3) + x3 )
      sum = ( r1 * d3 - r3 * d1 ) / det
      pol = ( x1 * r3 - x3 * r1 ) / det

      if ( iwhere>0 ) then

!     --- ... check whether  pol  is in  ( x(k), x(k+1) )

        if ( .not. ok(pol) ) goto 100
      end if

      sum2 = 0.5 * sum
      det = sum2 * sum2 - del(2) * pol

!     --- ... if  det<0  no solution exists

      if ( det<0.0 ) goto 100

      det = sqrt(det)
      zer = sum2 + det
      amu = sum2 - det
      okmu = ok (amu)

!     --- ... if both  zer, amu  are in ( x(k), x(k+1) )  or if both are
!             out of it, then no acceptable solution exists

      if ( ok(zer) .eqv. okmu )  goto 100

!     --- ... just one of the pair  zer, amu  is in ( x(k), x(k+1) ) , &
!             it should be  zer

      if ( okmu ) then

!     --- ... swap zer, amu

          z   = zer
          zer = amu
          amu = z
      end if

      amu = amu + x(2)
      zer = zer + x(2)
      pol = pol + x(2)
      solved = .true.

  100 continue
      end
