      subroutine ervl26 ( x, del, pol, solved, zer, amu )
! ======================================================================
!
!        programmer    Zdenek Vlasek
!        version  1.1  date 05-12-2012 Fortran 90
!        version  1.0  date 19-02-1991
!
!   copyright (c) 1991-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     purpose: rational (2,1) but only 2-point interpolation
!                of del(x),  with prescribed value  pol
!                see P+R (3.7) and the whole section 3.2
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x(2), del(2), zer, pol, amu
      logical solved

!     amu            i    ?
!     del            i    ?
!     pol            i    ?
!     solved         i    ?
!     x              i    x co-ordinate of point
!     zer           i/o   solved=.false.  (no solution)
!                         or   solved=.true.,  zer, amu
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision r1, r2, a, b, det
      logical okmu

!     a
!     b
!     det
!     okmu
!     r1
!     r2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical ok

!     OK             ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ok(a) = a>x(1) .and. a<x(2)

!------ statements

      solved = .false.

      r1 = x(1)**2 + del(1) * ( x(1) - pol )
      r2 = x(2)**2 + del(2) * ( x(2) - pol )
      a = ( r2 - r1 ) / ( x(2) - x(1) )
      b = x(1) * a - r1
      a = 0.5 * a
      det = a**2 - b
      if ( det<=0.0 ) goto 100
      det = sqrt (det)
      zer = a + det
      amu = a - det

!     --- ... zer  must be in  ( x(1), x(2) ),  amu  must be out of it

      okmu = ok(amu)
      if ( ok(zer) .eqv. okmu )  goto 100

      if ( okmu ) then

!     --- ... swap zer, amu

          a   = zer
          zer = amu
          amu = a
      end if

      solved = .true.

  100 continue
      end
