      subroutine funada ( v1, v2, v3 )
! ======================================================================
!
!        programmer     Zdenek Vlasek
!        version 1.2    date   27-07-1998 include common
!        version 1.1    date   18-08-1992
!
!
!
!   copyright (c) 1992-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmodulecadapt
      use sepmodulecactl
      implicit none
      double precision v1, v2, v3

!     coor         array containing the co-ordinates
!     v1      i    error indicator, "how many times the local
!                  error exceeds the average"
!     v2           not used
!     v3      o    refinement criterium for refloc, &
!                  "how many times the edge should be shorter"
!                  ( refloc refines each edge for which the
!                    average of this criterion in endpoints is
!                    greater then 1.5 )
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision alpha, expon
      logical first
      double precision expo1, sensit, log2in, shift
      save first, expo1, sensit, log2in, shift
      data first /.true./
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Puts integers in the error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    - &
! **********************************************************************
!
!
!                  METHO
!
!   Let the expected rate of convergence in semi-energy norm is
!       h ** alpha,  where  h  is maximum of diameter of elements.
!
!   Let the error on one specific element is  1, let its diameter is 1.
!
!   Refining each edge of this element into  p  pieces we get
!      error on the old element                     p ** (-alpha) , &
!      error on each of  p**2  subelements roughly  p ** (-alpha-2) .
!
!   Before optimal mesh is found:
!       singularities degrade the convergence rate, alpha = 0.5, &
!                     see [ZhZi88] after (18)
!
!   After optimal mesh was found:
!       alpha = degree of polynomials in shape functions, &
!                     see [ZhZi88] after (16)
!
!   Now, if the ratio of error on old element to error on subelement
!       should be  r, then we must set   p = r ** ( -1/(alpha+2) ).
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- constants to be somehow chosen, they may depend on an expected
!           rate of the convergence

      if ( first ) then

         log2in = 1d0 / log(2d0)

!        --- default values

!              the values are heuristic - better put them in SOLVE ... END, &
!              they cannot satisfy all problems

         expo1 = 0.4d0
         sensit = 1.1d0

         first = .false.

         shift = 0.5d0 - log(sensit) / log(2d0)

      end if

      if ( .not. optim ) then

!     --- improve local error distribution ( ignore global error )

         alpha = 0.5d0

!        --- version [ZhZi88] 15

         expon = 1d0 / alpha

!        --- = 2.0

!              ZV-version

         expon = 1d0 / ( alpha + 2d0 )

!        --- = 0.4

!              file-version

         expon = expo1

      else

!     --- improve global error ( ignore local error distribution )

         if ( itype==100 ) then
            alpha = 1d0
         else if ( itype==104 ) then
            alpha = 2d0
         else
            call errint ( itype, 1 )
            call errsub ( 1427, 1, 0, 0 )
            call instop
         end if

         expon = 1d0 / alpha

!        --- = 1 resp. 0.5

      end if

      v3 = v1 ** expon

!     --- at this stage, we would like to refine this way:
!                          new_edge = old_edge / v3, when v3>sensit
!                          new_edge = old_edge       when v3<sensit

!           take into account the way the refloc works:
!           when v3 <= 1.5 no refinement takes place
!                v3 in (1.5 , 2.5] ==> half once
!                v3 in (2.5 , 3.5] ==> half once
!                v3 in (3.5 , 4.5] ==> half once

      v3 = log(v3) * log2in

!     --- at this stage, we would like to half the edge v3 times

!           shift log2(sensit) onto 0.5

      v3 = v3 + shift

      end
