      subroutine initst
! ======================================================================
!
!        programmer    Guus Segal
!        version 14.1  date 11-12-2017 New storage of intmat
!        version 14.0  date 05-02-2016 New parameter list
!        version 13.8  date 11-04-2015 Remove ibcmat
!
!   copyright (c) 1982-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    initialize some parameters for subroutine BUILD (Old subroutine SYSTEM)
!    Help subroutine for subroutine TO0050
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     initialization
!     matrix
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesol
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k
      integer, pointer :: iusol
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     iusol          SEPRAN reference to solution vector
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      90   array isol has not been filled, or the boundary conditions have
!           not been stored in the solution vector
!      92   wrong values in ielhlp
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'initst' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from initst'
         call prinin ( iinbld, 15, 'iinbld' )

      end if
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)
      iusol => SMF%iusol

!     --- Compute some parameters

      if ( nbound==0 ) effbc = .false.
      numns  = nelgrp+numnat
      if ( nunkp==1 ) ichnun = 1
      if ( nunkp>1 ) ichnun = 3
      if ( indprf>0 ) ichnun = ichnun+1  ! kprobf>0
      compl = IM%complexm

      if ( effbc .or. iinbld(2)==6 ) then

!     --- effbc = true    essential boundary conditions

         if ( iusol<1 .or. iusol>lastsol ) then
            if ( solarr(iusol)==0 ) then
               call errint ( iusol, 1 )
               call errsub ( 90, 1, 0, 0 )
            end if
         end if
      end if

!     --- Set celiar equal to zero

      ind = 0
      ich = 0
      mst = 0
      ist = 0

!     --- Compute some extra parameters

      if ( partad ) then

!     --- partad  =  true  check skipelgrps

         do k = 1, numns
            if ( skipelgrps (k)<0 .or. skipelgrps (k)>1 ) then
               call errint ( k, 1 )
               call errint (skipelgrps(k), 2 )
               call errsub ( 92, 2, 0, 0 )
            end if
         end do

      end if

      maxunk = 0

      if ( compl ) then
         icompl = 2
      else
         icompl = 1
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'clrmat, addmat, clrvec, addvec ', &
                          clrmat, addmat, clrvec, addvec
         write(irefwr,*) 'End initst'

      end if
      call erclos ( 'initst' )

      end subroutine initst
