      subroutine prpcgrad ( iu0, iun, iinsol, rinsol1 )
! ======================================================================
!
!        programmer    Peter Minev
!        version  3.3  date 13-12-2017 New storage of intmat
!        version  3.2  date 29-05-2014 Extension of timebefore
!        version  3.1  date 15-11-2012 Long integers
!
!   copyright (c) 1999-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     finite-element preconditioned conjugate-gradient solver
!
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemain
      use sepmodulekprob
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iu0(*),iun(*), iinsol(*)
      double precision rinsol1(*)

!     iinsol         i    Contains information concerning the solver
!                         Contents
!                         1: maxiter i.e. maximum number of iterations
!                         2: iprint, defines the amount of output
!                            Possible values:
!                            0: no output
!                            1: the number of iterations is printed
!                            2: the convergence of the iteration process is
!                               showed
!     iu0            i    Contain information of the initial approximation
!     iun            o    Contain information of the solution
!     rinsol1        i    Contains information concerning the solver
!                         Contents
!                         1: eps, i.e. accuracy with respect to residual
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: timebefore(2)

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRPCGRAD1      finite-element preconditioned conjugate-gradient solver
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prpcgrad' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prpcgrad'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Create temporary arrays at the end of ibuffr
!         It concerns the arrays R^n, Z^n, V^n, P^n and temp^n
!         They contain the degrees of freedom without boundary conditions
!         Also copy kprobh to end of ibuffr

      iprob = intmt(intmat)%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      call sepfillkprob

!     --- Call to subroutine

      call prpcgrad1 ( iu0, iun, iinsol, rinsol1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prpcgrad'

      end if
      call erclmn ( 'prpcgrad', timebefore )

      end subroutine prpcgrad
