      subroutine sepactsolve ( isol, icompl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2017 New storage of intmat
!        version  1.0  date 24-02-2016
!
!   copyright (c) 2016-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate arrays for subroutine seplinsol
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleproj
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: isol(*)
      integer, intent(in) :: icompl

!     icompl         i    Real: icompl=0,   Complex: icompl=1
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPELGRPNODES  Fill array kmesh part ae with all nodes corresponding
!                    to element groups
!     SEPNODEELGRPS  Fill array kmesh part af with all element groups
!                    corresponding to nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepactsolve' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepactsolve'
         write(irefwr,1) 'imetproj, proj_type, kelmw', &
                          imetproj, proj_type, kelmw
         write(irefwr,1) 'iseqsol', iseqsol
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( imetproj==1 .and. proj_type>2 .and. kelmw>0 ) then

!     --- Projection vectors required (real properties)
!         Create new vectors

         if ( kelmae==0 ) call sepelgrpnodes
         if ( kelmaf==0 ) call sepnodeelgrps

      end if  ! ( imetproj==1 .and. proj_type>2 .and. kelmw>0 ... )

!     --- Create solution arrays

      call sepcreasol110 ( isol(iseqsol), 'solution', icompl )

      if ( iseqres_exact>0 ) &
         call sepcreasol110 ( isol(iseqres_exact), &
                             'res_exact_solution', icompl )
      if ( iseqreac/=0 ) &
         call sepcreasol110 ( isol(abs(iseqreac)), 'reac', icompl )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepactsolve'

      end if  ! ( debug )
      call erclos ( 'sepactsolve' )

      end subroutine sepactsolve
