      subroutine sepbuildessbc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute effect of essential boundary conditions to
!     the right-hand side
!     It is assumed that the solution is in the standard sequence, but
!     that the right-hand sides are inr enumbered form
! **********************************************************************
!
!                       KEYWORDS
!
!     right_hand_side
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision, allocatable :: xcopy(:)

!     i              Counting variable
!     xcopy          Work array to store a renumbered copy of usol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERMATRIXBVEC   Part of matrix vector multiplication with respect to the
!                    boundary conditions
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPMULTBCOND   incorporate effect of boundary conditions
!                    Matrix real, vectors real
!     SEPMULTZBCOND  incorporate effect of boundary conditions
!                    Matrix complex, vectors complex
!     TOREORDC       Map array of solution type from matrix sequence to vector
!                    sequence. Complex version
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuildessbc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuildessbc'
         call ertrace
         write(irefwr,1) 'indprh, numrhs', indprh, numrhs
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprh==0 ) then

!     --- No renumbering

         do i = 1, numrhs

            rhsd =>  rhsds(i)%rh

            if ( compl ) then

!           --- Complex case

            call ermatrixbvec ( usol, rhsd, irowsip, icolsip, S_ip, &
                                compl, compl, -1 )

            else

!           --- Real case

               call sepmultbcond ( usol, rhsd, -1 )

            end if  ! ( compl )

         end do  ! i = 1, numrhs

      else

!     --- Renumbering
!         Copy usol in xcopy

         if ( debug ) write(irefwr,1) 'nusol, icompl', nusol, icompl

         allocate ( xcopy(nusol*icompl), stat = error )
         if ( error/=0 ) call eralloc ( error, nusol*icompl, 'xcopy' )
         if ( ierror/=0 ) go to 1000

         if ( compl ) then

!        --- Complex case

            xcopy(1:nusol*icompl) = usol(1:nusol*icompl)
            call toreordc ( kprobhinv, xcopy, nusol )

         else

!        --- Real case

            xcopy = usol(kprobhinv)

         end if  ! ( compl )

         do i = 1, numrhs

            rhsd =>  rhsds(i)%rh

            if ( compl ) then

!           --- Complex case

               call sepmultzbcond ( xcopy, rhsd, -1, S_ip )

            else

!           --- Real case
               call sepmultbcond ( xcopy, rhsd, -1 )

            end if  ! ( compl )

         end do  ! i = 1, numrhs

         deallocate ( xcopy, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'xcopy' )

      end if  ! ( indprh==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbuildessbc'

      end if  ! ( debug )
      call erclos ( 'sepbuildessbc' )

      end subroutine sepbuildessbc
