      subroutine sepbuildintmat ( iincommat, lniincommat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 18-12-2016
!
!   copyright (c) 2016-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array intmat with default values
!     Set iprob
! **********************************************************************
!
!                       KEYWORDS
!
!     right_hand_side
!     stiffness_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      use sepmodulesmf
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lniincommat
      integer, intent(inout) :: iincommat(lniincommat)

!     iincommat      i    Integer input array containing information concerning
!                         the structure of the large matrix
!                         See subroutine read00 for a description
!     lniincommat    i    Declared length of array iincommat
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOMMAT      Actual body of COMMAT and MATSTRUC
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuildintmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuildintmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( iinbld(1)>0 .and. iinbld(9)>0 ) then
         iprob = iinbld(9)
      else if ( ks(SMF%iusol)%typevec==110 ) then
         iprob = ks(SMF%iusol)%iprob
      else
         iprob = 1
      end if  ! ( ks(SMF%iusol)%typevec==110 )
      iincommat = 0
      iincommat(2) = 2  ! jmethod
      iincommat(5) = iprob

      call sepcommat ( iincommat )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbuildintmat'

      end if  ! ( debug )
      call erclos ( 'sepbuildintmat' )

      end subroutine sepbuildintmat
