      subroutine sepccgrferr ( A, x, b, res, rferr, iter, nglobal, &
                               work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     iter is multiple of sqrt(n)
!     Check error level, i.e. compute new value for rferr
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%neq), b(A%neq), res(A%neq)
      integer, intent(in) :: iter, nglobal
      double precision, intent(inout) :: rferr
      double complex, intent(inout) :: work(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     iter           i    Iteration number
!     nglobal        i    Global number of unknowns. This is an overestimate, &
!                         because unknowns on interfaces are summed up.
!     res            i    Residual
!     rferr         i/o   Maximal accuracy possible for this matrix, &
!                         due to round-off
!     work          i/o   Work array of length n to store the help vectors
!     x              i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h

!                    they are not
!     h              Help variable to store a real
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepccgrferr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepccgrferr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      rferr = iter
      rferr = exp ( (rferr/nglobal) ** 2 )

!     --- Compute Ax, store in work

      work = -cresidual( A, x, b)
      work = work + res

      h = abs(dot_product(work,work))

      rferr = rferr * h

1000  call erclos ( 'sepccgrferr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepccgrferr'

      end if  ! ( debug )

      end subroutine sepccgrferr
