      subroutine sepccopypmatcs ( A, diag, diagshift )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy preconditioning of compact matrix in type matrix
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecmat
      use sepmodulematr
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex, target, intent(in) :: diag(*), diagshift(*)
      type (cmatrix) :: A

!     A              o    Structure of type matrix containing the necessary
!                         information of the matrix
!     diag           i    Diagonal of the large matrix
!     diagshift      i    Diagonal matrix containing the Laplace shift
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepccopypmatcs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepccopypmatcs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store diagonal

      if ( A%type_prec==3 .or.A%type_prec==4 ) then
         length = A%Mrows(nrusol+1)
      else
         length = nrusol
      end if  ! ( A%type_prec==3 .or.A%type_prec==4 )
      A%MD => diag(1:length)

!     --- Define type of scaling

      if ( A%type_prec>0 ) then
         A%scaling = 2
         if ( .not. A%symmetric .and. A%type_prec==3 ) A%scaling = 3
      end if  ! ( iprec>0 )

      A%DLshift => diagshift(1:nrusol)  ! diagonal laplace shift

1000  if ( debug ) then

!     --- Debug information

  !       call prinrl ( A%MD, nrusol, 'diag' )
         write(irefwr,*) 'End sepccopypmatcs'

      end if  ! ( debug )
      call erclos ( 'sepccopypmatcs' )

      end subroutine sepccopypmatcs
