      subroutine sepcmatvec ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Matrix vector multiplication for serial matrices
!     Complex matrix and vector
!     Old subroutine ezmul1
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix_vector_multiplication
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat   ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%neq)
      double complex :: y(A%neq)

!     A              i    Matrix A
!     x              i    vector to be multiplied
!     y              o    result Ax
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCNRCMATVEC  Compute y = A x, where A is a row-compact complex matrix
!     SEPCSCMATVEC   Compute y = A x, where A is symmetric compact matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2917   Option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcmatvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcmatvec'

      end if  ! ( debug )

      if ( ierror/=0 ) go to 1000

      select case (A%type_storage)

      case(3, 7)

!     --- symmetric compact matrix

         call sepcscmatvec ( A, x, y )

      case(4, 8)

!     --- row-compact matrix, non-symmetric structure

         call sepcnrcmatvec ( A, x, y )

      case default

         call errint ( A%type_storage, 1 )
         call errsub ( 2917, 1, 0, 0 )

      end select  ! case (A%type_storage)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcmatvec'

      end if  ! ( debug )
      call erclos ( 'sepcmatvec' )

      end subroutine sepcmatvec
