      subroutine sepcopmatmulr ( A, B )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy information from A with type mat into B with type rmatrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     real
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekprob
      use sepmodulematr
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (globmat) :: A
      type ( rmatrix ) :: B

!     A              i    matrix of type mat to be used in multiplication
!     B              o    internal matrix A_ii stored as type rmatrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipos
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopmatmulr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopmatmulr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(A%intmatref)

      irowsii => IM%rowsii
      icolsii => IM%colsii
      B%neq = IM%nrusol

      if ( IM%nosepmat ) then

!     --- Not a sepran matrix

         B%S => A%S_ii

      else if ( .not. IM%simple ) then

!     --- Row-Compact matrix

         B%S => A%S_ii(1:size(icolsii))

      end if  ! ( IM%direct )
      B%DLshift => A%Dlapshift
      B%symmetric = IM%symmetric

      B%rows => irowsii
      B%cols => icolsii

      if ( size(IM%diag)>0 ) B%diag => IM%diag
      B%Mdiag => IM%diag

!     --- Store type of matrix

      B%type_storage = IM%jmethod
      B%symmetric = IM%symmetric
      B%nglob = B%neq
      B%matmultsp = 0
      if ( parallel ) then

!     --- Parallel case, store matrix S_i out and storage scheme

         B%rowsout => irowsout
         B%colsout => icolsout
         B%nglob = nusolglob
         B%Aout => A%S_iout
         B%innercontr => innercontr

         if ( typecoupl==0 ) then
            B%sendinfo => sendinfo
         else
            B%sendinfo => sendinfopar
         end if  ! ( typecoupl==0 )

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopmatmulr'

      end if  ! ( debug )
      call erclos ( 'sepcopmatmulr' )

      end subroutine sepcopmatmulr
