      subroutine sepcopymatr
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy real R matrix in type matrix (simple)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix
!     real
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPYINPSOL  Put information of inpsol and rinsol in matrix structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopymatr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopymatr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

      R%type_prec = 0
      R%deflation = .false.

!     --- Store type of matrix

      R%neq = np
      R%nglob = np
      R%nproj = 0
      R%prlump = 0
      R%matmultsp = 0
      R%symmetric = IM%Gsymmetric

      R%S => S_R

      if ( R%symmetric ) then
         R%type_storage = 5
      else
         R%type_storage = 6
      end if  ! ( R%symmetric )
      R%scaling = 1    ! matrix is scaled (standard)

      R%rows => IM%rowsDG
      R%cols => IM%colsDG
      R%diag => IM%diagDG
      R%diag => IM%diagDG

      R%Mrows => irowsDGpre
      R%Mcols => icolsDGpre
      R%Mdiag => IM%diagDG
      R%iprint = inpsol(6,3)
      R%factor = rinsol(3,3)
      R%adapt = rinsol(2,3)
      R%matmultsp = 0
      R%iseqmt = iseqmt
      R%iseqim = iseqim

      call sepcopyinpsol ( R, 3 )
      R%simple = .false.

      if ( R%type_prec>0 ) then
         R%scaling = 2
         if ( .not. R%symmetric .and. R%type_prec==3 ) R%scaling = 3
      end if  ! ( iprec>0 )

      if ( parallel ) then

!     --- Parallel case

         R%innercontr => innercontrp
         R%sendinfo => sendinfop
         R%matmultsp = 2
         R%scaling = 0    ! matrix is not scaled
         R%nglob = nusolglob

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopymatr'

      end if  ! ( debug )
      call erclos ( 'sepcopymatr' )

      end subroutine sepcopymatr
