      subroutine sepcopyoldsol ( islold, istold1lc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create relation between solution arrays and buffer
!
! **********************************************************************
!
!                       KEYWORDS
!
!     pointer
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleprobinf
      use sepmodulevecs
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: islold(*), istold1lc(6,*)

!     islold         i    Contains information about old vectors
!     istold1lc        i    Contains information corresponding to old
!                         solutions. See to0050
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, indprfloc, iproblc, ivec, leng
      integer, pointer :: kprobloc1(:)
      type (probinfo), pointer :: kprobpart1

!     indprfloc       indication if kprob part f has been filled (>0)
!                    or not (0) with respect to local array
!     iproblc        Local value of iprob
!     ivec           Sequence number of vector of special structure
!     k              Counting variable
!     kprobloc1      Local array kprobloc
!     kprobpart1     Local array kprobpart
!     leng           Help variable to compute the length of some arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ1        print 2d integer array
!     SEPFILLOLDCVEC Make relation between complvector and module
!                    sepmoduleoldvecs
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopyoldsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopyoldsol'
         call ertrace
         write(irefwr,1) 'numold', numold
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( numold>0 ) then

!     --- There are more old solutions

         if ( debug ) call prinin ( islold, numold, 'islold' )
         do k = 1, numold

            if ( debug ) &
               write(irefwr,1) 'k, islold(k), istold1lc(1,k)', &
                                k, islold(k), istold1lc(1,k)

            if ( islold(k)>0 .and. istold1lc(1,k)>=0 ) then

!           --- Make relation between old solution and oldvecs

               usoltemp => ks(islold(k))%sol
               leng = size(usoltemp)
               oldsols(k)%oldvecs => usoltemp
               indprfloc = istold1lc(2,k)  ! local indprf for this vector
               oldsols(k)%itypvc = ks(islold(k))%typevec
               oldsols(k)%lenoldvec = leng    ! length of array
               oldsols(k)%ndegfd = ks(islold(k))%ivec  ! number of degrees
                                                          ! of freedom

               if ( ks(islold(k))%compl==1 ) then

!              --- Vector is complex, fill oldsols(k)%oldcvecs

                  call sepfilloldcvec ( usoltemp, leng/2, oldsols, k )

               end if  ! ( ks(islold(k))%compl==1 )
      !         if ( indprfloc>0 ) then

!              --- kprobf exists for this vector

                  if ( debug ) write(irefwr,1) 'indprfloc', indprfloc
                  iproblc = ks(islold(k))%iprob
                  kprobpart1 => kp(iseqkp)%kprobsub(iproblc)
                  kprobloc1 => kprobpart1%kprobloc
                  if ( ks(islold(k))%typevec==110 ) then
                     if ( kprobloc1(19)>0 ) &
                       oldsols(k)%kprobf => kprobpart1%kprobf
                     istold1lc(2,k) = kprobloc1(19)
                  else if ( ks(islold(k))%typevec==115 ) then
                     ivec = ks(islold(k))%ivec
                     istold1lc(2,k) = kprobpart1%kprobe2(ivec)
                     indprfloc = istold1lc(2,k)
                     if ( indprfloc>0 ) then
                     oldsols(k)%kprobf => kprobpart1%kprobe3(ivec)%arr
                     end if  ! ( indprfloc>0 )
                  end if  ! ( ks(islold(k))%typevec==110 )

      !         end if  ! ( indprfloc>0 )
               if ( debug ) write(irefwr,1) 'istold1lc(5,k)', istold1lc(5,k)
               oldsols(k)%iseqtype = istold1lc(5,k) ! sequence number with
                                                  ! respect to type of vectors

            else

!           --- Vector is not used

               oldsols(k)%iseqtype = 0

            end if  ! ( islold(k)>0 )

         end do  ! i = 1, numold

      end if  ! ( numold==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopyoldsol'

      end if  ! ( debug )
      call erclos ( 'sepcopyoldsol' )

      end subroutine sepcopyoldsol
