      subroutine sepcscmatvec ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = A x, where A is non-symmetric compact matrix
!     Structure of matrix is symmetric (6)
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat   ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%neq)
      double complex :: y(A%neq)

!     A              i    Matrix A
!     x              i    vector to be multiplied
!     y              o    result Ax
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double complex :: sum1, xi

!     i              General loop variable
!     j              General loop variable
!     k              column number (index in vector)
!     sum1           help variable to compute a sum
!     xi             x(i)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcscmatvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcscmatvec'
         call ertrace
         call prinrl ( x, A%neq, 'x' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First y = D x

      y = A%S(A%diag)*x

!     --- Next the parts L and U

      do i = 1, A%neq

         sum1 = 0d0
         xi = x(i)
         do j = A%rows(i)+2, A%rows(i+1)
            k = A%cols(j)
            sum1 = sum1+A%S(j)*x(k)
            y(k) = y(k)+ A%S(j)*xi
         end do
         y(i) = y(i)+sum1

      end do  ! i = 1, A%neq

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%neq, 'y' )
         write(irefwr,*) 'End sepcscmatvec'

      end if  ! ( debug )
      call erclos ( 'sepcscmatvec' )

      end subroutine sepcscmatvec
