      subroutine sepeffbouncd ( A, u, f )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute effect of boundary conditions in Au and subtract from f
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulematr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (globmat), intent(in) :: A
      double precision, intent(in) :: u(nusol)
      double precision, intent(inout) :: f(nusol)

!     A              i    matrix of type mat to be used in multiplication
!     f             i/o   f := f-A_bnd u
!     u             i/o   vector to be multiplied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: y(:)

!     y              work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMULTBCOND1  Compute y = y + S_ip x_p in real case
!     SEPPARGLOB     Send information to neighboring processors in order to give
!                    all local arrays the global value in the common points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeffbouncd' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeffbouncd'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( y(nusol) )

      y = 0d0
      call sepmultbcond1 ( A, u, y )
      if ( parallel .and. typecoupl==1 ) call sepparglob ( A%S_i, y, nusol )
      f = f - y

      deallocate ( y )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeffbouncd'

      end if  ! ( debug )
      call erclos ( 'sepeffbouncd' )

      end subroutine sepeffbouncd
