      subroutine sepfillmdbuild
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2017 New storage of intmat
!        version  1.0  date 14-01-2016
!
!   copyright (c) 2016-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     initialize some parameters for subroutine BUILD (Old subroutine SYSTEM)
!     Old subroutine initiprob
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     initialization
!     matrix
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1248   Array IINBLD(8) has wrong value (<0 or >1 ), whereas IINBLD(1)>7
!    1249   Array IINBLD(7) has wrong value (<0 or >1 ), whereas IINBLD(1)>6
!    1250   Array IINBLD(6) has wrong value (<0 or >1 ), whereas IINBLD(1)>5
!    1251   Array IINBLD(5) has wrong value (<0 or >1 ), whereas IINBLD(1)>4
!    1252   Array IINBLD(3) has wrong value (<0 or >1 ), whereas IINBLD(1)>2
!    1329   The value of IINBLD(4) is incorrect
!    1480   ISEQIN out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillmdbuild' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillmdbuild'
         write(irefwr,1) 'iprob', iprob
         call prinin ( iinbld, 15, 'iinbld' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Initialize logicals

      clrmat = .true.
      clrvec = .true.
      addmat = .true.
      addvec = .true.
      partad = .false.
      effbc  = .true.
      oldsol = .false.
      clrlpv = .true.

!     --- Compute actual value, depending on the value of iinbld

      imas   = iinbld(4)
      numold = iinbld(10)
      numrhs = iinbld(12)
      iseqin = iinbld(13)
      idefcr = iinbld(14)
      isubdv = iinbld(15)

      iseqprevsol = iseqin  ! store in sepmoduleoldvecs

      if ( idefcr==1 ) then

!     --- Defect correction in combination with super elements is not yet
!         allowed

         if ( numrhs>1 ) call errsub ( 1555, 0, 0, 0 )

      end if

      if ( iinbld(8)>1 .or. iinbld(8)<0 )  then

!     --- iinbld(8)>1 or<0

         call errint ( iinbld(8), 1 )
         call errsub ( 1248, 1, 0, 0 )

      end if
      if ( iinbld(8)==1 ) clrlpv = .false.

      if ( iinbld(7)>1 .or. iinbld(7)<0 ) then

!     --- iinbld(7)>1 or<0

         call errint ( iinbld(7), 1 )
         call errsub ( 1249, 1, 0, 0 )

      end if
      if ( iinbld(7)>=1 ) then
         clrmat = .false.
         clrvec = .false.
      end if

      if ( iinbld(6)>1 .or. iinbld(6)<-2 ) then

!     --- iinbld(6)>1 or<-2

         call errint ( iinbld(6), 1 )
         call errsub ( 1250, 1, 0, 0 )

      end if
      if ( iinbld(6)>=1 ) partad = .true.

      if ( numold==0 ) then

!     --- numold>0, check iinbld(5), i.e. iprec

         if ( iinbld(5)>1 .or. iinbld(5)<0 ) then

!        --- iinbld(5)>1 or<0

            call errint ( iinbld(5), 1 )
            call errsub ( 1251, 1, 0, 0 )

         end if
         if ( iinbld(5)>=1 ) oldsol = .true.

      end if

!     --- check iinbld(3), i.e. effbouncond

      if ( iinbld(3)>1 .or. iinbld(3)<0 ) then

!     --- iinbld(3)>1 or<0

         call errint ( iinbld(3), 1 )
         call errsub ( 1252, 1, 0, 0 )

      end if
      if ( iinbld(3)>=1 ) effbc = .false.

!     --- check iinbld(2), i.e. jchois

      if ( iinbld(2)<=0 .or. iinbld(2)>6 .and. iinbld(2)/=14 &
           .or. iinbld(2)==4 ) then

!     --- iinbld(2) incorrect

         call errint ( iinbld(2), 1 )
         call errsub ( 1329, 1, 0, 0 )

      end if
      if ( iinbld(2)==14 ) then
         addvec = .false.
         clrvec = .false.
         effbc  = .false.
      else if ( iinbld(2)==4 ) then
         addvec = .false.
         clrvec = .false.
         effbc  = .false.
         addmat = .false.
         clrmat = .false.

      else if ( iinbld(2)==6 ) then

!     --- Special case: multiply boundary matrix by solution
!         Is needed for eaction forces

         effbc  = .false.
         addmat = .false.
         clrmat = .false.

      else
         if ( iinbld(2)==3 .or. iinbld(2)==5 )addvec = .false.
         if ( iinbld(2)==2 .or. iinbld(2)==5 ) then
            addmat = .false.
            clrmat = .false.
          end if
      end if
      if ( iinbld(2)>2 ) numrhs = 1
      if ( iseqin<1 .or. iseqin>max(numold,1) ) then
         call errint ( iseqin, 1 )
         call errint ( 1, 2 )
         call errint ( numold, 3 )
         call errsub ( 1480, 3, 0, 0 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'clrmat, addmat, clrvec, addvec ', &
                          clrmat, addmat, clrvec, addvec
         write(irefwr,*) 'End sepfillmdbuild'

      end if
      call erclos ( 'sepfillmdbuild' )

      end subroutine sepfillmdbuild
