      subroutine sepfillsmf ( iinbldlc, irhsdlc, iusol, islold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2017 New storage of intmat
!        version  1.0  date 04-12-2016
!
!   copyright (c) 2016-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all input for subroutine SEPBUILD in SMF
!
! **********************************************************************
!
!                       KEYWORDS
!
!     right_hand_side
!     stiffness_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmoduleintmat
      use sepmodulekmesh
      use sepmodulesmf
      use sepmodulekprob
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, target :: iinbldlc(*), iusol
      integer, target :: irhsdlc(*), islold(*)

!     SMF            o    Contains all input for subroutine SEPBUILD
!     iinbldlc       i    User input array for subroutine SEPBUILD
!     irhsdlc        i    Local array irhsd
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     iusol          i    Position in solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: leniinbld, nmat, nrhs, numold

!     leniinbld      Length of array iinbld
!     nmat           Defines second dimension of array matr (stiffm)
!     nrhs           Defines second dimension of array irhsd
!     numold         Number of vectors that are stored in islold
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLSMF1    Fill part of input for subroutine SEPBUILD in SMF
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillsmf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillsmf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( intmat>0 ) then
         iprob = intmt(intmat)%iprob
      else if ( iusol>0 ) then
         iprob = ks(iusol)%iprob
      else
         iprob = min(nprob,max(1,iprob))
      end if  ! ( intmat>0 )
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      numnat = kprobloc(12)
      leniinbld = min(iinbldlc(1),nbld)

      allocate ( skipelgrps(nelgrp+numnat) )

      if ( iinbldlc(1)<nbld .or. iinbldlc(1)>nbld+nelgrp+numnat ) then

!     --- Probably iinbld has not been filled

         iinbld = 0
         iinbld(1) = nbld
         iinbld(2) = 1       ! JCHOIS
         iinbld(12:13) = 1   ! NRIGHTHS, ISEQIN

      end if  ! ( iinbldlc(1)<nbld .or. iinbldlc(1)>nbld+nelgrp+numnat )
      if ( leniinbld>1 ) iinbld(2:leniinbld) = iinbldlc(2:leniinbld)

      if ( iinbld(6)>0 ) skipelgrps = iinbldlc(nbld+1:nbld+nelgrp+numnat)

      nrhs = iinbld(12)
      nmat = iinbld(14)+1
      numold = max(1,iinbld(10))

      SMF%iusol => iusol
      SMF%irhsd => irhsdlc(1:nrhs)

      call sepfillsmf1 ( islold, numold )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillsmf'

      end if  ! ( debug )
      call erclos ( 'sepfillsmf' )

      end subroutine sepfillsmf
