      subroutine sepfillsolmod2 ( usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill several modules for subroutine seplinsol
!     Old subroutine sepfillsolmod
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulematr
      use sepmodulesolve
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: usol(nusol)

!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iseqlc, lengL
      logical :: compl
      type (rmatrix), pointer :: A
      type (cmatrix), pointer :: B
      type (intmatrix), pointer :: IM

!     A              Structure of type rmatrix containing the necessary
!                    information of the real matrix
!     B              Structure of type cmatrix containing the necessary
!                    information of the complex matrix
!     IM             Refers to intmt(intmat)
!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     i              Counting variable
!     iseqlc         Local sequence number
!     lengL          Length of lower tridiagonal of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLMATSTR2 Fill parameters in module sepmodulematstr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillsolmod2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillsolmod2'
         write(irefwr,1) 'iseqpre, iseqintmat', iseqpre, iseqintmat
  1      format ( a, 1x, (10i11) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then
         call sepfillmatstr2
      end if  ! ( parallel )
      if ( nrusol==0 ) go to 1000

      IM => intmt(intmat)

!     --- Make correction of nrusol in case of parallel computing

      if ( parallel ) then

!     --- Parallel case

         nrusol = IM%nrusol
         indprh = kprobloc(50)
         if ( typecoupl==1 ) indprh = kprobpar(4) ! Dirichlet-Dirichlet coupling

      end if  ! ( parallel )

      if ( iseqpre/=iseqintmat ) then

!     --- iseqpre differs from iseqintmat

         iseqlc = intmatar(iseqpre)
         IM => intmt(iseqlc)

      end if  ! ( iseqpre/=iseqintmat )

      if ( iseqqpre/=iseqintmat ) then

!     --- iseqqpre differs from iseqintmat

         iseqlc = intmatar(iseqqpre)
         IM => intmt(iseqlc)

      end if  ! ( iseqpre/=iseqintmat )

      if ( iseqdgpre/=iseqintmat ) then

!     --- iseqdgpre differs from iseqintmat

         iseqlc = intmatar(iseqdgpre)
         IM => intmt(iseqlc)

      end if  ! ( iseqpre/=iseqintmat )

      if ( iseqpmpre/=iseqintmat ) then

!     --- iseqpmpre differs from iseqintmat

         iseqlc = intmatar(iseqpmpre)
         IM => intmt(iseqlc)

      end if  ! ( iseqpre/=iseqintmat )

      if ( keep==2 .and. .not. IM%direct ) then

!     --- The preconditioning matrix exists and must be reused

         Pre_ii => realmt(iseqmt)%matinv

      end if  ! ( keep==2 .and. .not. IM%direct )

      if ( ilimit>0 ) then

!     --- ilimit>0, compute minimum and maximum of input solution
!         Store in valmin, valmax

         valmin = minval(usol(1:nusol))
         valmax = maxval(usol(1:nusol))

      end if  ! ( ilimit>0 )

!     --- Create usol and rhsd

      if ( .not. IM%direct ) then

!     --- Compact matrix

         if ( realmt(iseqmt)%complexmat ) then
            B => realmt(iseqmt)%Sc_i
            B%scaling = 1          ! matrix is scaled (standard)
            B%prlump = inpsol(33,1)
            B%deflation = .false.
            if ( .not. IM%simple ) then
               B%Mrows => IM%rowsii
               B%Mcols => IM%colsii
            end if  ! ( .not. IM%simple )
         else
            A => realmt(iseqmt)%S_i
            A%scaling = 1          ! matrix is scaled (standard)
            A%prlump = inpsol(33,1)
            A%deflation = .false.
            if ( .not. IM%simple ) then
               A%Mrows => IM%rowsii
               A%Mcols => IM%colsii
            end if  ! ( .not. IM%simple )
         end if  ! ( realmt(iseqmt)%complex )

      end if  ! ( .not. IM%direct )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillsolmod2'

      end if  ! ( debug )
      call erclos ( 'sepfillsolmod2' )

      end subroutine sepfillsolmod2
