      subroutine sepidrshadowvc ( A, sparam, pshadow )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 07-10-2014 New parameter list
!        version  1.0  date 03-04-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      integer, intent(in) :: sparam
      double precision, intent(out) :: pshadow(nrusol,sparam)

!     A              i    Matrix A
!     pshadow        o    Array of size n x s
!                         Contains all shadow vectors of size n each
!                         So pshadow(i,j) contains i^th component of p_j
!     sparam         i    Defines the parameters s in IDR(s)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPIDRSHADOW   Compute shadow vectors p for IDR process
!                    Store in pshadow
!     SEPIDRSHADOWG  Compute shadow vectors p for IDR process
!                    Parallel case, global shadow vectors are created at node 1
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepidrshadowvc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepidrshadowvc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- parallel case

         if ( A%globshadow ) then

!        --- Global shadow vectors

            call sepidrshadowg ( A, sparam, pshadow )

         else

!        --- Local shadow vectors

            call sepidrshadow ( sparam, nrusol, pshadow )

         end if  ! ( inpsol(8)==1 )

      else

!     --- serial case

         call sepidrshadow ( sparam, nrusol, pshadow )

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepidrshadowvc'

      end if  ! ( debug )
      call erclos ( 'sepidrshadowvc' )

      end
