      subroutine sepinibvl ( ichoice, nodenr, usol, inusol, idegfd, &
                             value, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-01-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of essential boundary condition for one point
!     and one degree of freedom only
!     Old subroutine inibvl
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ichoice, inusol, idegfd, nodenr
      double precision :: usol(*), value
      logical :: compl

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     ichoice        i    Integer choice parameter
!                         See subroutine sepbvalue
!     idegfd         i    Degree of freedom sequence number
!     inusol         i    Position of unknown in array USOL
!     nodenr         i    Node number
!     usol           o    Solution vector
!     value          i    Value of boundary condition if constant
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: x, y, z, comval(2)

!     comval         Help variable to store a complex quantity
!     x              x co-ordinate of point
!     y              y co-ordinate of point
!     z              z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: funcbc

!     CFUNCB         Computes the value of the boundary conditions in a point as
!                    function of the co-ordinates (complex case)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FUNCBC         Computes the value of the boundary conditions in a point as
!                    function of the co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepinibvl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepinibvl'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( compl ) then

!     --- Complex case

         x = coor(1,nodenr)
         if ( ndim>1 ) y = coor(2,nodenr)
         if ( ndim>2 ) z = coor(3,nodenr)
         call cfuncb(-ichoice,x,y,z,comval)
         usol(2*inusol-1)=comval(1)
         usol(2*inusol)=comval(2)

      else if ( ichoice<=0 ) then

!     --- Real case, constant value

         usol(inusol)=value

      else

!     --- Real case, function subroutine

         x = coor(1,nodenr)
         if ( ndim>1 ) y = coor(2,nodenr)
         if ( ndim>2 ) z = coor(3,nodenr)
         usol(inusol)=funcbc(ichoice,x,y,z)


      end if  ! ( compl )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepinibvl'

      end if  ! ( debug )
      call erclos ( 'sepinibvl' )

      end subroutine sepinibvl
