      subroutine sepinitbuild
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Allocate some arrays for subroutine to0050
!     Initialize some parameters
! **********************************************************************
!
!                       KEYWORDS
!
!     diagonal_matrix
!     mass_matrix
!     right_hand_side
!     stiffness_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepinitbuild' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepinitbuild'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      numoldvecs = max(1,numold)

      if ( debug ) write(irefwr,1) 'numoldvecs', numoldvecs

      if ( debug )write(irefwr,1) 'allocate ( istold(2,numoldvecs)'
      allocate ( istold(2,numoldvecs), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*numoldvecs, 'istold' )
      if ( debug )write(irefwr,1) 'allocate ( istold1(6,numoldvecs)'
      allocate ( istold1(6,numoldvecs), stat = error )
      if ( error/=0 ) call eralloc ( error, 6*numoldvecs, 'istold1' )
      if ( debug )write(irefwr,1) 'allocate ( oldsols(numoldvecs)'
      allocate ( oldsols(numoldvecs), stat = error )
      if ( error/=0 ) call eralloc ( error, numoldvecs, 'oldsols' )
      if ( debug )write(irefwr,1) 'allocate ( indvector(numoldvecs)'
      allocate ( indvector(numoldvecs), stat = error )
      if ( error/=0 ) call eralloc ( error, numoldvecs, 'indvector' )
      if ( ierror/=0 ) go to 1000

      if ( debug )write(irefwr,1) 'before set to 0'

      istold = 0
      istold1 = 0
      indvector = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepinitbuild'

      end if  ! ( debug )
      call erclos ( 'sepinitbuild' )

      end subroutine sepinitbuild
