      subroutine sepinitinpsol
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     initialize arrays inpsol and rinsol for seplinsol
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepinitinpsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepinitinpsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize array inpsol and rinsol

      inpsol = 0
      rinsol = 0d0

      inpsol(4,1) = 6       ! default preconditioner
      inpsol(11,1) = 10     ! default termination criterion
      inpsol(37,1) = 1      ! IPRINT_SPECTRAL
      inpsol(38,1) = 100    ! MAX_ITER_SPECTRAL
      rinsol(1,1) = 1d-3    ! accuracy
      rinsol(8,1) = 1d-8    ! amax (overrelaxation)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepinitinpsol'

      end if  ! ( debug )
      call erclos ( 'sepinitinpsol' )

      end subroutine sepinitinpsol
