      subroutine sepmanvecs ( ivectr, ivec2, ivec3, inpsep, rinvec, &
                              ichoice, iscal1, iscal2, iinvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Vector/scalar manipulation subroutine
!     Standard case calling sepmanvec
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulecons
      use sepmoduleplot
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivectr, ivec2, ivec3, inpsep(*), &
              ichoice, iscal1, iscal2, iinvec(*)
      double precision rinvec(*)

!     ichoice        i    Integer choice parameter, see prmanv
!     iinvec         i    User input array for subroutine MANVEC
!                         For a description see subroutine MANVECBF
!     inpsep         i    The integer input read is stored in array INPSEP
!                         For a description see subroutine ASKVEC
!     iscal1         i    Sequence number of first scalar
!     iscal2         i    Sequence number of second scalar
!     ivec2          i    Sequence number of second vector
!     ivec3          i    Sequence number of third vector
!     ivectr         i    standard sepran array denoting USOL or vector of
!                         special structure defined in nodal points
!     rinvec         i    Real user input array for subroutine MANVEC
!                         The contents depend on iinvec
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprob, iprobvec, inpsep1(3), i, nch

!     i              Counting variable
!     inpsep1        Array inpsep for prchanprob
!     iprob          Actual problem number
!     iprobvec       Actual problem number as stored in isolut
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCHANPROB     Change problem number of solution vector
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRLINCOMB      Compute the linear combination fact1 * vector1 +
!                    fact2 * vector2
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPMANVEC      Vector manipulation subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmanvecs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmanvecs'
         call ertrace
         write(irefwr,1) 'ichoice, ivectr, ivec2, ivec3', &
                          ichoice, ivectr, ivec2, ivec3
         call prinin ( isolut2d, max(ivectr, ivec2, ivec3), 'isolut2d' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ichoice==27 ) then

!     --- ichoice = 27, linear combination

         if ( abs(rinvec(1)-1d0)<epsmac .and. abs(rinvec(2))<1d-50 ) then

!        --- Special situation of copy vector

            if ( debug ) write(irefwr,1) 'before sepcopyvc'
            call sepcopyvc ( isolut2d(ivec2), isolut2d(ivectr) )
            numvecmn = max(numvecmn,ivectr)
            go to 500  ! jump, since finished

         else  if ( ks(isolut2d(ivec2))%compl==1 .or. &
                    ks(isolut2d(ivec3))%compl==1 ) then

!        --- Linear combination of complex vectors
!            Since the scalars are linear they have to be transformed into complex
!            This is done by moving the second coefficient to positions (3,4)
!            and clear the second coefficient

            rinvec(4) = 0d0
            rinvec(3) = rinvec(2)
            rinvec(2) = 0d0

         else if ( iinvec(3)>0 .and. &
                   ks(isolut2d(ivec2))%nusol/= &
                   ks(isolut2d(max(1,ivec3)))%nusol ) then

!        --- Linear combination of real vectors but with different length of
!            vectors

            if ( debug ) write(irefwr,1) 'before prlincomb'
            call prlincomb ( iinvec, rinvec, &
                             isolut2d(ivec2), isolut2d(max(1,ivec3)), &
                             isolut2d(ivectr), ivectr )
            go to 500  ! jump, since finished

         end if  ! ( abs(rinvec(1)-1d0)<epsmac .and. ... )

      end if  ! ( ichoice==27 )

      if ( ioutp>=0 .and. itimloop<=1 ) then
         if ( ichoice>=25 ) then
            nch = index(names(ivectr),' ')-1
            if ( nch<=0 ) nch = len(names(ivectr))
            write ( irefwr, 100 ) names(ivectr)(1:nch), ivectr
100         format ( ' Create vector ', a, &
                     ' with sequence number ', i3, ' by algebraic', &
                     ' manipulations' )
         else
            nch = index(chscalars(iscal1),' ')-1
            if ( nch<=0 ) nch = len(chscalars(iscal1))
            write ( irefwr, 110 ) chscalars(iscal1)(1:nch), iscal1
110         format ( ' Create scalar ', a, &
                     ' with sequence number ', i3, ' by algebraic', &
                     ' manipulations' )
         end if
      end if
      if ( debug ) then

!     --- Debug information

         call prinin ( iinvec, 20+nelgrp, 'iinvec' )
         call prinrl ( rinvec, 4, 'rinvec' )
         write(irefwr,*) 'ivec2, ivec3, ivectr ', ivec2, ivec3, ivectr

      end if

      if ( debug ) write(irefwr,1) 'before manvecbf'
      call sepmanvec ( iinvec, rinvec, isolut2d(ivec2), &
                       isolut2d(max(1,ivec3)), isolut2d(ivectr) )

      if ( ichoice<=5 ) then

!     --- computation of scalars

         if ( iscal1>0 ) scalars(iscal1) = rinvec(1)
         if ( iscal2>0 ) scalars(iscal2) = rinvec(2)
         numscals = max( numscals, iscal1, iscal2 )

         if ( debug ) then
            write(irefwr,1) 'iscal1, iscal2', iscal1, iscal2
            write(irefwr,2) 'scalars', rinvec(1:2)
         end if  ! ( debug )

         if ( ichoice==3 ) then

!        --- Compute min_max

            if ( inpsep(13)>0 ) then
               do i = 0, ndim-1
                  scalars(inpsep(13)+i) = rinvec(3+i)
               end do
            end if
            if ( inpsep(10)>0 ) then
               do i = 0, ndim-1
                  scalars(inpsep(10)+i) = rinvec(6+i)
               end do
            end if
            numscals = max( numscals, inpsep(9)+ndim-1, inpsep(10)+ndim-1 )

         end if

      else
         numvecmn = max(numvecmn,ivectr)
      end if

500   iprobvec = 0
      if ( isolut2d(ivectr)>0 ) iprobvec = ks(isolut2d(ivectr))%iprob
      iprob = inpsep(2)

      if ( iprob>0 .and. iprobvec/=iprob ) then

!     --- iprob # iprobvec, change problem number of output vector

         inpsep1(2) = iprob
         inpsep1(3) = ivectr
         call prchanprob ( inpsep1 )

      end if  ! ( iprob>0 .and. iprobvec/=iprob )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iprob', iprob
         write(irefwr,*) 'End sepmanvecs'

      end if  ! ( debug )
      call erclos ( 'sepmanvecs' )

      end subroutine sepmanvecs
