      subroutine sepmatoutvec ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = A x, where A is the part of the matrix consisting
!     of the off-diagonal block
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix_vector_multiplication
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat   ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%nglob)
      double precision, intent(inout) :: y(A%neq)

!     A              i    Matrix A
!     x              i    vector to be multiplied
!     y              o    result Ax
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision :: sum1

!     i              General loop variable
!     j              General loop variable
!     k              column number (index in vector)
!     sum1           help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmatoutvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmatoutvec'
         call prinrl ( x, nglob, 'x' )
         call prinrl ( y, A%neq, 'y/1' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Matrix vector multiplication (compact storage)

      do i = 1, A%neq

         sum1 = 0d0
         do j = A%rowsout(i)+1, A%rowsout(i+1)
            k = A%colsout(j)
            sum1 = sum1+A%Aout(j)*x(k)
         end do
         y(i) = y(i)+sum1

      end do  ! i = 1, A%neq

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%neq, 'y/2' )
         write(irefwr,*) 'End sepmatoutvec'

      end if  ! ( debug )
      call erclos ( 'sepmatoutvec' )

      end subroutine sepmatoutvec
