      subroutine sepmultbcond2 ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = y + S_ip x_p in real case
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematr
      use sepmodulemat
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (globmat), intent(in) :: A
      double complex, intent(in) :: x(nusol)
      double complex :: y(nusol)
      type (intmatrix), pointer :: IM

!     A              i    matrix of type mat to be used in multiplication
!     x              i    vector to be multiplied
!     y             i/o   result vector of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, jk

!                    they are not
!     i              Counting variable
!     j              Counting variable
!     jk             Help variable to store an index
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmultbcond2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmultbcond2'
         write(irefwr,1) 'nbound', nbound
         call prinrl ( x(nusol-nbound+1), nbound, 'x-in')
  1      format ( a, 1x, (10i11) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(A%intmatref)

      irowsip => IM%rowsip
      icolsip => IM%colsip

      if ( debug ) then
         call prinin ( irowsip, nbound+1, 'irowsip' )
         call prinin ( icolsip, size(icolsip), 'icolsip' )
         call prinrl ( y, A%nrow, 'y/1' )
      end if  ! ( debug )

      do i = 1, nbound
         do j = irowsip(i)+1, irowsip(i+1)
           jk = icolsip(j)
           y(jk) = y(jk)+A%S_ip(j)*x(nusol-nbound+i)
         end do  ! j = irows(i)+1, irows(i+1)
      end do  ! i = 1, nbound

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, nrusol, 'y/2' )
         write(irefwr,*) 'End sepmultbcond2'

      end if  ! ( debug )
      call erclos ( 'sepmultbcond2' )

      end subroutine sepmultbcond2
