      subroutine sepmultreac1 ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute reaction forces
!     yp = Spf*xf + Spp*xp
!     Matrix real, vectors real
!     Old subroutine er0117
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (globmat), target, intent(in) :: A
      double precision, intent(in) :: x(*)
      double precision, intent(inout) :: y(*)

!     A              i    matrix of type globmat to be used in multiplication
!     x              i    input vector to be multiplied
!     y             i/o   The vector y should exist at input
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ik, j, jk, ypst
      type (pmatrix), pointer :: P

!     P              matrix of type pmatrix used for boundary conditions part
!     i              Counting variable
!     ik             Global sequence number of the i'th d.o.f
!     j              Counting variable
!     jk             Help variable to store an index
!     ypst           Global sequence number of the last free d.o.f
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmultreac1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmultreac1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      ypst = nusol-nbound

      if ( debug ) &
         write(irefwr,1) 'nusol, nbound, ypst', nusol, nbound, ypst
         
      P => A%S_p

      do i = 1, nbound

!     --- set i^th yp to zero

         ik = ypst+i
         y(ik) = 0d0

!        --- add Spf * xf

         do j = P%rows(i)+1, P%rows(i+1)
           jk = P%cols(j)
            y(ik) = y(ik)+P%Bpi(j)*x(jk)
         end do

!        --- add Spp * yp

         do j = P%rowsp(i)+1, P%rowsp(i+1)
           jk = P%colsp(j)
            y(ik) = y(ik)+P%Bpp(j)*x(jk)
         end do

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmultreac1'

      end if  ! ( debug )
      call erclos ( 'sepmultreac1' )

      end subroutine sepmultreac1
