      subroutine sepmumpsslave ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 28-05-2014 Extension of inpsol
!        version  1.0  date 24-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform a repeated call to prmumps on a slave
!     This subroutine is never left except for a stop
! **********************************************************************
!
!                       KEYWORDS
!
!     mumps
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemat
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMUMPS       Solve a linear system of equation by MUMPS
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmumpsslave' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmumpsslave'
         write(irefwr,1) 'inodenr', inodenr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      skipmumps = .true.  ! everything must be skipped

!     --- Infinite loop

      do while ( .true. )
         call sepmumps ( A, usol, rhsd )
      end do  ! while ( .true. )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmumpsslave'

      end if  ! ( debug )
      call erclos ( 'sepmumpsslave' )

      end
