      subroutine sepparalloc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Allocate some global arrays for parallel computing
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipos

!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepparalloc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepparalloc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprobpar => kprobpart%kprobpar
      nglob = kprobpar(2)   ! number of global degrees of freedom
      blockunkinfo => kprobpart%blockunkinfo
      allocate ( sendwork(nglob), xglob(nglob), stat = error )

      if ( error/=0 ) call eralloc ( error, 2*nglob, 'sendwork' )
      if ( ierror/=0 ) go to 1000

!     --- Initialize xglob

      xglob = 0d0
      blockinfor => kprobpart%blockinfor
      kprobhdd => kprobpart%kprobhdd
      kprobhddinv => kprobpart%kprobhddinv
      kprobhglob => kprobpart%kprobhglob
      ninterfdeg => kprobpart%ninterfdeg
      nrusollocs => kprobpart%nrusollocs
      onnz => kprobpart%onnz
      nnz => kprobpart%nnz

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepparalloc'

      end if  ! ( debug )
      call erclos ( 'sepparalloc' )

      end subroutine sepparalloc
