      subroutine sepparccgeps1 ( x, again, repeat1, xnorm, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Recompute accuracy eps during cg iteration
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     accuracy
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      double precision, intent(inout) :: xnorm
      double complex, intent(in) :: x(A%neq)
      logical, intent(out) :: again, repeat1

!     A              i    Matrix A
!     again          o    If true the computation may be repated
!     repeat1       i/o   Logical variable indicating if the accuracy check
!                         must be carried out twice (true) or once (false), &
!                         because the norm of the solution could not be computed
!     x              i    Solution vector
!     xnorm          i    Norm of the estimated solution
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepparccgeps1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepparccgeps1'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      again = .false.
      if ( A%iaccuracy==0 ) then

         repeat1 = .false.

         xnorm = glnorm(x,A%neq)

         if ( debug ) write(irefwr,2) 'xnorm', xnorm

!        --- Set accuracy eps

         A%eps = (A%eps1 + xnorm * A%eps2)
         again = .true.
         goto 1000

      end if

1000  call erclos ( 'sepparccgeps1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepparccgeps1'

      end if  ! ( debug )

      end subroutine sepparccgeps1
