      subroutine sepprecprin2 ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print minimum and maximum element of diagonal of preconditioning matrix
!     Old subroutine precmkprin2
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: imin, imax, i
      double precision :: amin, amax, ah

!     ah             hep variable to store elemnt of matrix
!     amax           maximal element of a matrix
!     amin           minimal element of a matrix
!                    they are not
!     i              Counting variable
!     imax           index of the maximal element of a matrix
!     imin           index of the minimal element of a matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprecprin2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprecprin2'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- print the preconditioner which is used

      select case ( A%type_prec )
         case(1)
            write ( irefwr,200 )
200         format ( ' A diagonal preconditioner is used' )
         case(2)
            write ( irefwr,210 )
210         format ( ' An Eisenstat preconditioner is used' )
         case(3)
            write ( irefwr,220 )
220         format ( ' An incomplete LU-decomposition is used' )
         case(4)
            write ( irefwr,230 )
230         format (' A Symmetric Gauss-Seidel preconditioner is used')
         case(7)
             write ( irefwr,250 ) A%factor
250         format ( ' A MILU Eisenstat preconditioner is used.',/ &
                     ' The average factor is equal to', 1pe10.2e2 )

      end select  ! case ( A%type_prec )

      if ( A%prlump==1) then
         if ( A%symmetric ) then
            write ( irefwr,290 )
         else
            write ( irefwr,295 )
         end if
      end if

290   format ( ' Before the preconditioning, the positive', &
               ' off-diagonal elements are lumped',/ &
               ' to the main diagonal' )

295   format ( ' Before the preconditioning, the off-diagonal', &
               ' elements with the same sign as ' &
              /' the main diagonal element', &
               ' are lumped to the main diagonal' )

!     --- print the maximal en minimal diagonal element of the preconditioner
!         Initializations

      imin = 1
      imax = 1
      amin = A%MD(1)
      amax = A%MD(1)

      do i = 1, A%neq

         if ( A%type_prec==3 .or.  A%type_prec==4 ) then
            ah = A%MD(A%diag(i))
         else
            ah = A%MD(i)
         end if  ! ( A%type_prec==3 .or.  A%type_prec==4 )
         if ( ah<amin) then
            imin = i
            amin = ah
         end if

         if ( ah>amax) then
            imax = i
            amax = ah
         end if

      end do  ! i = 1, A%neq
      write ( irefwr,140 ) amin, imin, A%S(A%diag(imin))
140   format ( //' The minimal diagonal element of premat is ', &
               1pe10.2e2, ' in row ', i7 / &
                ' the diagonal element of the matrix is ', &
               1pe10.2e2 )
      write ( irefwr,150 ) amax, imax, A%S(A%diag(imax))
150   format (   ' The maximal diagonal element of premat is ', &
               1pe10.2e2, ' in row ', i7 / &
                ' the diagonal element of the matrix is ', &
               1pe10.2e2 )

!     --- Next for absolute values

      imin = 1
      imax = 1
      amin = abs(A%MD(1))
      amax = abs(A%MD(1))

      if ( debug ) then
         write(irefwr,1) 'imin, imax, A%neq', imin, imax, A%neq
         write(irefwr,2) 'amin, amax', amin, amax
      end if  ! ( debug )

      do i = 1, A%neq

         if ( A%type_prec==3 .or.  A%type_prec==4 ) then
            ah = abs(A%MD(A%diag(i)))
         else
            ah = abs(A%MD(i))
         end if  ! ( A%type_prec==3 .or.  A%type_prec==4 )
         if ( ah<amin) then
            imin = i
            amin = ah
         end if

         if ( ah>amax) then
            imax = i
            amax = ah
         end if

      end do

      write ( irefwr,360 ) amin, imin, A%S(A%diag(imin))
360   format (   ' The minimal abs(diag) element of premat is ', &
               1pe10.2e2, ' in row ', i7 / &
                ' the diagonal element of the matrix is ', &
               1pe10.2e2 )
      write ( irefwr,370 ) amax, imax, A%S(A%diag(imax))
370   format (   ' The maximal abs(diag) element of premat is ', &
               1pe10.2e2, ' in row ', i7  / &
                ' the diagonal element of the matrix is ', &
               1pe10.2e2 / )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprecprin2'

      end if  ! ( debug )
      call erclos ( 'sepprecprin2' )

      end subroutine sepprecprin2
