      subroutine sepprofgg ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the decomposition of a symmetric positive definite
!     profile matrix
!     Old subroutine profgg
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     positive_definite
!     real
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: timebefore(2)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: seprowsums

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPPRINMAT2    Print inverse matrix that is stored as profile real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1820   The matrix found in the solver is completely zero
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprofgg' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprofgg'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      A%rowmin = 1
      A%rowmax = A%neq
      if ( debug ) call sepprinmat2 ( A )

!     --- compute row norm of part of the matrix in order to be able to check
!         the stopping criterion for the pivot process

      A%rowsum = seprowsums ( A )
      if ( A%rowsum<=0d0 ) then
         call errsub ( 1820, 0, 0, 0 )
         go to 1000
      end if
      A%rowsum = A%rowsum*epsmac

!     --- decomposition

      call sepdecgg ( A )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'G G^t decomposition'
         call sepprinmat2 ( A )
         write(irefwr,*) 'End sepprofgg'

      end if  ! ( debug )
      call erclmn ( 'sepprofgg', timebefore )

      end subroutine sepprofgg
