      subroutine sepprofiledec ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute lU decomposition for system of real linear equations
!     Use Gauss elimination
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
!     direct_method
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulemat
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPYTOPROF  Initialize the LU-decomposition matrix of A for
!                    a profile method
!     SEPCOPYTOPROFS Initialize the LU-decomposition matrix of A for
!                    a profile method (symmetric matrix)
!     SEPPRINTCOND   Print information about the condition number of
!                    the large matrix
!     SEPPROFGG      compute the decomposition of a symmetric positive definite
!                    profile matrix
!     SEPPROFSM      compute the decomposition of a symmetric profile matrix
!     SEPPROFUN      compute the decomposition of an asymmetric profile matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprofiledec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprofiledec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      IM => intmt(iseqim)

!     --- Translate row-compact data structure to profile structure

      nrusol = A%neq

      if ( IM%symmetric ) then

!     --- symmetric matrix
!         Copy matrix into "inverse matrix"

         call sepcopytoprofs ( A )
         if ( ipos==1 ) then

!        --- Matrix positive definite

            call sepprofgg ( A )

         else

!        --- General symmetric matrix

            call sepprofsm ( A )

         end if  ! ( ipos==1 )

      else

!     --- asymmetric matrix
!         Copy matrix into "inverse matrix"

         call sepcopytoprof ( A )
         call sepprofun ( A )

      end if  ! ( IM%symmetric )

      if ( A%iprint>0 ) call sepprintcond ( A )
      A%decomp = .false.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprofiledec'

      end if  ! ( debug )
      call erclos ( 'sepprofiledec' )

      end subroutine sepprofiledec
