      subroutine sepreadinpsol ( inpsl1, rinsl1, iread )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read arrays inpsol and rinsol and store in sepmodulesolve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulesolve
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iread, inpsl1(*)
      double precision :: rinsl1(*)

!     inpsl1         i    Integer input array for subroutine solvel
!                         See solvelbf for a description
!     iread          i    if -2 the input is copied to inpsl1 and rinsl1
!     rinsl1         i    Contains information concerning the linear solver
!                         See the Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqnr, irnsol, ninsol, nrisol, iproblc

!     iproblc        Actual problem number
!     irnsol         Infor entry with respect to array IRNSOL as stored in
!                    IBUFFR
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     ninsol         Length of array INPSOL
!     nrisol         Length of array RINSOL
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     READ03         Read card input for subroutine TO0089
!     SEPINITINPSOL  initialize arrays inpsol and rinsol for seplinsol
!     SEPUPDINPSOL   Update inpsol for special situations
!                    Put parameters in sepmodulesolve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadinpsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadinpsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepinitinpsol  ! initialize inpsol/rinsol

      iproblc  = 1
      iseqnr = 1
      inpsol(1,1) = ninpsol
      ninsol = ninpsol
      nrisol = nrinsol
      call read03 ( inpsol, rinsol, 1, iseqnr, iproblc, nprob, &
                    ninsol, nrisol )

!     --- Adapt variables

      call inichinpsol ( inpsol, ninsol )

      if ( iread==-2 ) then

!     --- iread = -2, copy inpsol and rinsol

         length = min(inpsl1(1), ninsol)
         inpsl1(1) = length
         inpsl1(2:length) = inpsol(2:length,1)
         length = nrisol
         call dcopy ( length, rinsol, 1, rinsl1, 1 )

      end if

      call sepupdinpsol  ! Update inpsol for special situations

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadinpsol'

      end if  ! ( debug )
      call erclos ( 'sepreadinpsol' )

      end subroutine sepreadinpsol
