      function seprowsums ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute rowsum of symmetric profile matrix and store in seprowsums
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: seprowsums

!     A             i/o   Matrix A
!     seprowsums     o    Computed rowsum
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k1, k2, j

!     j              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: dasum

!     DASUM          Blas subroutine computing the sum of absolute values of
!                    vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprowsums' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprowsums'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      seprowsums = 0d0
      k1 = 1
      do j = 1, A%neq
         k2 = A%rows(j+1)
         seprowsums = max(seprowsums,dasum(k2-k1,A%S(k1),1))
         k1 = k2
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seprowsums'

      end if  ! ( debug )
      call erclos ( 'seprowsums' )

      end function seprowsums
