      subroutine sepsimpledpsol ( norm_rhsd, niterp, choice, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve R dp = b_2 - D du by cg type methods
!     Help subroutine for prsimple
!     Old subroutine prsimpledpsol
! **********************************************************************
!
!                       KEYWORDS
!
!     solve
!     simple
!     residual
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulesolve
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niterp, choice
      double precision norm_rhsd, work(np)

!     choice         i    Choice parameter defining whta equation is solved
!                         Possible values
!                        -1:  standard simple method, solve R dp = b_2 - D du
!                         0:  standard simple method, solve -R dp = b_2 - D du
!                         1:  reduced method, solve R dp = b_2
!     niterp        i/o   Number of iterations to solve the p-equations
!     norm_rhsd      i    Norm of right-hand side vector
!     work                Work array of length np to store intermediate vectors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rnorm

!                    they are not
!     rnorm          Norm of vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimpledpsol' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimpledpsol'
         write(irefwr,1) 'choice', choice
         call prinrl ( du, nu, 'du')
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if

      inpsol(5,3) = R%nmax

!     --- Solve R dp = b_2 - D du by cg type methods
!         First compute b_2 - D du

      if ( choice<=0 ) then

!     --- choice = 0, D du must be subtracted
!         Next work := D du

         work(1:np) = D*du
         if ( debug ) call prinrl ( work, np, 'D du')
         if ( ierror/=0 ) go to 1000

!        --- Subtract from rhsp

         rhsp = rhsp - work

      end if  ! ( choice==0 )

      rnorm = norm ( rhsp, C )
      if ( debug ) then
         call prinrl ( rhsp, np, 'rhsp/2')
         write(irefwr,2) 'norm', rnorm
      end if  ! ( debug )

      if ( rnorm<1d-16*norm_rhsd ) then

!     --- To prevent problems in the CG subroutine, we skip solving the
!         pressure part if the right-hand side is too small

         dp = 0d0
         go to 1000

      end if  ! ( rnorm<1d-16*norm_rhsd )

!     --- Next scale the matrix solution and right-hand side by
!         the inverse of the diagonal of the preconditioning matrix
!         This scaling is not done in case of an LDL^t preconditioning
!         Then solve system of equations and scale back

      dp = R .div. rhsp
      if ( debug ) call prinrl ( dp, np, 'dp/1')

      if ( ierror/=0 ) then

!     --- The process halted because of an error

         write(irefwr,*) 'The error occurred in the R-matrix'
         go to 1000

      end if

      niterp = niterp+R%numiter   ! number of iterations carried out

!     --- Since we have to solve -S dp = r_p-Gu*, we multiply dp by -1
!         Only if choice = 0

      if ( choice==0 ) dp = -dp

      if ( debug ) call prinrl ( dp, np, 'dp/2' )

1000  if ( debug ) then
         write(irefwr,*) 'End sepsimpledpsol'
      end if  ! ( debug )
      call erclos ( 'sepsimpledpsol' )

      end
