      subroutine sepsolvenest ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-06-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of real linear equations by Gaussian elimination
!     The nested dissection approach is applied
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
!     direct_method
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmoduleintmat
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Matrix A
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBUILDNEST   replace matrix from row-compact storage to row-compact
!                    storage for nested dissection
!     SEPNESTDEC     lU decomposition for system of real linear equations
!                    Gauss elimination (nested dissection)
!     SEPNESTSOL     Solve system of real linear equations where
!                    the matrix is decomposed before (nested dissection)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsolvenest' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsolvenest'
         write(irefwr,*) 'decomp ', A%decomp

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(A%iseqim)
      if ( IM%nested .and. .not. associated(realmt(iseqmt)%matinv) ) &
           call sepbuildnest ( A )
      A%scaling = 10
      if ( A%decomp ) call sepnestdec ( A )  ! Compute decomposition

      call sepnestsol ( A, usol, rhsd )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsolvenest'

      end if  ! ( debug )
      call erclos ( 'sepsolvenest' )

      end subroutine sepsolvenest
