      subroutine sepupdinpsol
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Update inpsol for special situations
!     Put parameters in sepmodulesolve
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulecpack
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: method
      integer, pointer :: iinsol(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     iinsol         Refers to inpsol(:,1)
!     method         See INPSOL(3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMESHV  Fill array kmeshv with element groups coupled to element
!                    groups
!     SEPFILLSOLVE   Fill parameters from inpsol and rinsol into sepmodulesolve
!     SOLVELCHECK    Perform some checks for the linear solver
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepupdinpsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepupdinpsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

      iinsol => inpsol(:,1)

      method = iinsol(3)
      if ( iinsol(8)==0 ) then
         if ( method==4 ) then
            iinsol(8) = 100
         else
            iinsol(8) = 20
         end if
      end if
      if ( iinsol(13)==0 ) iinsol(13) = 20
      if ( iinsol(14)==0 ) then
         if ( method==1 .or. method==10 ) then
            iinsol(14) = 1
         else
            iinsol(14) = 5
         end if  ! ( method==1 .or. method==10 )
      end if  ! ( iinsol(14)==0 )

      if ( IM%mumps .and. inpsol(3,1)/=19 ) then

!     --- No solve input for mumps, use defaults

         inpsol(3,1) = 19  !  mumps
         inpsol(6,1) = 0   !  print_level
         inpsol(8,1) = 7   !  renumbering

      end if  ! ( IM%mumps .and. inpsol(3,1)/=19 )

      typesolv = inpsol(3,1)
      if ( typesolv==19 ) inpsol(10,1) = max(inpsol(10,1),keepmat)

      call solvelcheck   ! perform some checks
      call sepfillsolve  ! Put parameters in sepmodulesolve

      if ( imetproj==1 .and. .not. parallel ) call sepfillkmeshv

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepupdinpsol'

      end if  ! ( debug )
      call erclos ( 'sepupdinpsol' )

      end subroutine sepupdinpsol
