      subroutine sepzrenumsol ( uinpvec, jtrans, ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Renumber solution vector such that prescribed boundary conditions are
!     stored at the end.
!     Local transformations are carried out if necessary
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     complex
!     ordering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulesolve
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: jtrans, ichoice
      double complex, intent(inout), target :: uinpvec(nusol)

!     ichoice        i    Choice parameter indicating the input vector
!                         it concerns
!                         1: zusol
!                         2: zrhsd
!                         3: zresusolex
!                         4: zusolex
!                         5: zstartres
!     jtrans         i    If > 0 local transformations are carried out if
!                         npltra>0
!     uinpvec       i/o   Solution vector to be transformed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPLOCTRNCM    Compute (back-)transformation in case of local
!                    transformations ( complex case )
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepzrenumsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepzrenumsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Transform essential boundary conditions in case of local transform

      if ( npltra>0 .and. jtrans>0 ) call seploctrncm ( 1, uinpvec )

      if ( indprh>0 ) uinpvec = uinpvec(kprobhinv)  ! reorder uinpvec

      select case ( ichoice )
      case(1)
         zusol => uinpvec
      case(2)
         zrhsd => uinpvec
      case(3)
         zresusolex => uinpvec
      case(4)
         zusolex => uinpvec
      case(5)
         zstartres => uinpvec
      end select  ! case ( ichoice )

1000  call erclos ( 'sepzrenumsol' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepzrenumsol'

      end if  ! ( debug )

      end subroutine sepzrenumsol
