      subroutine to00002 ( ichois, kmeshc, ikelmc, &
                           index1, index2, nelgrp, iboun, &
                           isubel, nsubel, ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-07-2015 Old subroutine to0000
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill arrays index1 and index2 for the element subroutines
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduletypes
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), index1(*), index2(*), &
              ichois, ikelmc, iboun, isubel, &
              nelgrp, nsubel, ishape

!     iboun          i    sequence number for elements of type -2
!                         (boundary conditions of the type u equals unknown
!                         constant)
!     ichois         i    Choice parameter, possibilities
!                         1  nunkp=1, number of degrees of freedom is constant
!                         2  nunkp=1, number of degrees of freedom in the nodal
!                            points not constant
!                         3  nunkp>1,  number of degres of freedom is constant
!                         4  nunkp>1,  number of degrees of freedom not constant
!     ikelmc         i    last position used in kmeshc
!     index1         o    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     index2         o    Array of length icount containing the positions of the
!                         degrees of freedom in the element with respect to usol
!     ishape         i    Shape number of element
!     isubel         i    sub element sequence number
!     kmeshc         i    kmesh part c, contains the nodal points corresponding
!                         to the elements
!     nelgrp         i    Number of element groups
!     nsubel         i    Number of sub elements an element is subdivided into
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jcount, j, kchois, inodp, inusol, inunkp, k, i, ind1(3,4), &
              ind2(4,4), index21(4,6), ind3(3,4), istart

!     i              Counting variable
!     ind1           Array containing the relative sub elements of a quadratic
!                    triangle in the following sense
!                    ind1(j,i) contains the sequence number in kmeshc of the
!                    j-th node of the i-th sub element
!     ind2           Array containing the relative sub elements of a quadratic
!                    quadrilateral in the following sense
!                    ind1(j,i) contains the sequence number in kmeshc of the
!                    j-th node of the i-th sub element
!     ind3           Array containing the relative sub elements of a linear
!                    quadrilateral in the following sense
!                    ind1(j,i) contains the sequence number in kmeshc of the
!                    j-th node of the i-th sub element
!     index21        Array containing the relative sub elements of a linear
!                    prism in the following sense
!                    ind1(j,i) contains the sequence number in kmeshc of the
!                    j-th node of the i-th sub element
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in first point
!     inusol         Position of unknown in array USOL
!     istart         Starting address in array
!     j              Counting variable
!     jcount         Help parameter to compute the centroid of the element
!     k              Counting variable
!     kchois
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1586   Subdivision of elements of this shape have not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind1 / 1, 2, 6, &
                  2, 3, 4, &
                  4, 5, 6, &
                  2, 4, 6 /
      data ind2 / 1, 2, 9, 8, &
                  2, 3, 4, 9, &
                  4, 5, 6, 9, &
                  8, 9, 6, 7 /
      data ind3/ 2, 3, 1,   4, 1, 3,   1, 2, 4,   3, 4, 2 /
      data index21 / 1, 2, 3, 5,   4, 5, 6, 1,    1, 3, 5, 6, &
                     1, 2, 3, 4,   4, 5, 6, 2,    2, 3, 6, 4 /
! ======================================================================
!
      call eropen ( 'to00002' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to00002'
         write(irefwr,1) 'nsubel, inpelm, ishape, isubel, icount', &
                          nsubel, inpelm, ishape, isubel, icount
         write(irefwr,1) 'ichois, indprf, ikelmc', &
                          ichois, indprf, ikelmc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jcount=0
      inunkp = nunkp

      if ( nsubel==1 ) then

!     --- Standard case, no subdivision

         do j = 1, inpelm
            index1(j) = kmeshc(ikelmc+j)
         end do

      else

!     --- Special case, subdivision

         select case ( ishape )

            case ( :1, 8:11, 17:20, 22: )

!           --- error subdivision has not been implemented

               call errint ( ishape, 1 )
               call errsub ( 1586, 1, 0, 0 )
               go to 1000

            case ( 2 )

!           --- quadratic line element

               do j = 1, 2
                  index1(j) = kmeshc(ikelmc+isubel-1+j)
               end do

            case ( 4, 7 )

!           --- quadratic triangle

               do j = 1, 3
                  index1(j) = kmeshc(ikelmc+ind1(j,isubel))
               end do

            case ( 5 )

!           --- bi-linear quadrilateral

               if ( itype==903 ) then

!              --- Special case of type 903

                  do j = 1, 3
                     index1(j) = kmeshc(ikelmc+ind3(j,isubel))
                  end do

               else

!              --- error subdivision has not been implemented

                  call errint ( ishape, 1 )
                  call errsub ( 1586, 1, 0, 0 )
                  go to 1000

               end if  ! ( itype==903 )

            case ( 6 )

!           --- bi-quadratic quadrilateral

               do j = 1, 4
                  index1(j) = kmeshc(ikelmc+ind2(j,isubel))
               end do

            case ( 12, 15, 16 )

!           --- tri-quadratic tetrahedron

               istart = 4+4*isubel
               do j = 1, 4
                  index1(j) = kmeshc(ikelmc+itetra(istart+j))
               end do

            case ( 21 )

!           --- trilinear-prism

               do j = 1, 4
                  index1(j) = kmeshc(ikelmc+index21(j,isubel))
               end do

         end select  ! case ( ishape )

      end if  ! ( nsubel==1 ) then

      kchois=ichois
      if ( itype>0 .and. ielgrp<=nelgrp ) then
         kchois=ichois+4
      end if

      if ( debug ) write(irefwr,1) 'kchois', kchois
      if ( itype==-2 ) then

!     --- type number = -2, boundary conditions of the type u equals
!                           unknown constant

         jcount = inpelm
         if ( nunkp==1 ) then
            do i = 1, inpelm
               index2(i)=index1(i)
            end do
         else if ( ichois==3) then
            do i = 1, inpelm
               index2(i)=(index1(i)-1)*nunkp+iboun
            end do
         else
            do i = 1, inpelm
               index2(i)=kprobf(index1(i))+iboun
            end do
         end if

      else

!     --- other type numbers

         select case ( kchois )

         case(1, 5)

!        --- ichois = 1

            jcount=inpelm
            index2(1:inpelm)=index1(1:inpelm)

         case(2)

!        --- ichois = 2

            do j = 1,inpelm
               inodp=index1(j)
               inusol=kprobf(inodp)
               inunkp=kprobf(inodp+1)-inusol
               if ( inunkp/=0 ) then
                  jcount=jcount+1
                  index2(jcount)=inusol+1
               end if
            end do

         case(3)

!        --- ichois = 3

            do j = 1,inpelm
               inodp=index1(j)
               inusol=(inodp-1)*nunkp
               do k=1,nunkp
                  index2(jcount+k)=inusol+k
               end do
               jcount=jcount+nunkp
            end do

         case(4)

!        --- ichois = 4

            do j = 1,inpelm
               inodp=index1(j)
               inusol=kprobf(inodp)
               inunkp=kprobf(inodp+1)-inusol
               do k=1,inunkp
                  index2(jcount+k)=inusol+k
               end do
               jcount=jcount+inunkp
            end do

         case(6)

!        --- ichois = 2   itype>0

            if ( indprf>0 ) then

!           --- indprf>0

               do j = 1,inpelm
                  inodp=index1(j)
                  inusol=kprobf(inodp)
                  inunkp=kprobatmp(j)
                  if ( inunkp>0 ) then
                     jcount=jcount+1
                     index2(jcount)=inusol+1
                  end if
               end do

            else

!           --- indprf = 0

               do j = 1,inpelm
                  inodp=index1(j)
                  inusol=inodp-1
                  if ( indpra>0 ) inunkp=kprobatmp(j)
                  if ( inunkp>0 ) then
                     jcount=jcount+1
                     index2(jcount)=inusol+1
                  end if
               end do

            end if

         case(7)

!        --- ichois = 7

            do j = 1,inpelm
               inodp=index1(j)
               inusol=(inodp-1)*nunkp
               if ( indpra>0 ) inunkp=kprobatmp(j)
               do k=1,inunkp
                  index2(jcount+k)=inusol+k
               end do
               jcount=jcount+inunkp
            end do

         case(8)

!        --- ichois = 8   itype>0

            if ( indprf>0 ) then

!           --- indprf>0

               do j = 1,inpelm
                  inodp=index1(j)
                  inusol=kprobf(inodp)
                  inunkp=kprobatmp(j)
                  do k=1,inunkp
                     index2(jcount+k)=inusol+k
                  end do
                  jcount=jcount+inunkp
               end do

            else

!           --- indprf = 0

               do j = 1,inpelm
                  inodp=index1(j)
                  inusol=(inodp-1)*nunkp
                  if ( indpra>0 ) inunkp=kprobatmp(j)
                  do k=1,inunkp
                     index2(jcount+k)=inusol+k
                  end do
                  jcount=jcount+inunkp
               end do

            end if

         end select  ! case ( kchois )

      end if

1000  icount=jcount

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icount', icount
         call prinin ( index1, inpelm, 'subelement' )
         call prinin ( index2, jcount, 'index2' )
         write(irefwr,*) 'End to00002'

      end if  ! ( debug )
      call erclos ( 'to00002' )

      end
