      subroutine to0032 ( ichois, vec1, vec2, vec3, c, a, b, ipoint, &
                          kdegfd, nnodes )
! ======================================================================
!
!        programmer    Jaap van der Zanden / Guus Segal
!        version  8.3  date 14-04-1995 Minor changes in comment
!        version  8.2  date 26-12-1994 New commons cmessc and cmessg
!        version  8.1  date 23-06-1992 New common cmessg
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     algebraic manipulations with real vectors
!     Only the possibilities 4, 5, 12, 15, and 16 are treated
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex vec1(*), vec2(*), c
      double precision a, b, vec3(*)
      integer ichois, ipoint, nnodes, kdegfd

!     a       o    Double precision parameters to store minimum and maximum
!     b       o    See a
!     c      i/o   input / output constant (see ichois)
!     ichois  i    Choice parameter. Possible values
!                   4  VEC1(IDEGFD) in IPOINT is stored in c
!                   5  VEC1(IDEGFD) in IPOINT = c
!                  12  Compute the minimum and maximum of abs(vec1) and the
!                      corresponding co-ordinates.
!                      In this case only one degree of freedom per point is
!                      permitted. Furthermore some parameters have the following
!                      special meaning:
!                      vec1:  i  vector from which min and max must be computed
!                      vec2:  i  array containing the co-ordinates
!                      vec3:  o  Output array of length 6, containing the
!                                co-ordinates xmin, ymin, zmin, xmax, ymax, zmax
!                      ipoint:i  Dimension of the space ( 1, 2 or 3)
!                  15  a = || VEC1 - VEC 2 ||  (L1 norm)
!                      ipoint is point where maximum is reached
!                  16  a = || VEC1 ||
!                      ipoint is point where maximum is reached
!     ipoint i/o   nusol for ichois = 2, 7, 10
!     kdegfd  i    Specific degree of freedom to be applied
!     nnodes  i    Number of nodal points
!     vec1   i/o   first input vector
!     vec2   i/o   second input vector
!     vec3   i/o   third input vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, imin, imax, idegfd, inunkp, lunder, lupper, ldegfd
      double precision vv

!    i         Loop varibale
!    idegfd    Position of degree of freedom in vector
!    imax      Nodal point number for which the maximum value is reached
!    imin      Nodal point number for which the minimum value is reached
!    inunkp    Number of degrees of freedom in a point
!    ldegfd    Value of degree of freedom in point, (ldegfd=kdegfd, except
!              in the case kdegfd=0, in which case ldegfd takes all values)
!    lunder    Lower value of ldegfd
!    lupper    Upper value of ldegfd
!    vv        Value of vector in a node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     218   Degree of freedom does not exist
!     219   Degree of freedom is not available in point
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0032' )
      if ( ichois.lt.12 ) then
         if ( indprfi.eq.0 ) then
            call errint ( kdegfd, 1 )
            call errint ( nphysi, 2 )
            call errint ( ipoint, 3 )
            if ( kdegfd.gt.nphysi ) call errsub ( 219, 3, 0, 0 )
            idegfd = (ipoint-1)*nphysi+kdegfd
         else
            if ( kdegfd.gt.kprobfi(ipoint+1)-kprobfi(ipoint)) then
               call errint ( kdegfd, 1 )
               call errint ( kprobfi(ipoint+1)-kprobfi(ipoint), 2 )
               call errint ( ipoint, 3 )
               call errsub ( 219, 3, 0, 0 )
            end if
            idegfd = kprobfi(ipoint)+kdegfd
         end if
         if ( ichois.eq.4 ) then
            c = vec1(idegfd)
         else
            vec1(idegfd) = c
         end if
      else

!     --- Ichois =  12, compute minimum and maximum, plus corresponding
!                       co-ordinates
!         or ICHOIS = 15, 16  Compute maximum norm and corresponding
!                             nodal point for difference of VEC1 and VEC2 (15)
!                             or for IVEC1 alone (16)
!         First compute minimum, maximum and corresponding nodal points

         a =  rinfin
         b = -rinfin
         imin = 0
         imax = 0
         if (indprpi.gt.0) then
            if (kdegfd.gt.nphysi) then
               call errint ( kdegfd, 1 )
               call errint ( nphysi, 2 )
               call errsub ( 218, 2, 0, 0)
               go to 1000
            end if
            if ( kdegfd.eq.0 ) then
               lunder = 1
               lupper = nphysi
            else
               lunder = kdegfd
               lupper = kdegfd
            end if
            do i = 1, nnodes
               do ldegfd = lunder, lupper
                  idegfd = kprobpi(i,ldegfd)
                  if ( idegfd.gt.0 ) then
                     if ( ichois.eq.15 ) then
                        vv = abs( vec1(idegfd)-vec2(idegfd) )
                     else
                        vv = abs( vec1(idegfd) )
                     end if
                     if ( ichois.eq.12 .and. vv.lt.a ) then

!                    --- vv < a, new minimum found

                        a    = vv
                        imin = i
                     end if
                     if ( vv.gt.b ) then

!                    --- vv > b, new maximum found

                        b    = vv
                        imax = i
                     end if
                  end if
               end do
            end do
         else if (indprfi.eq.0) then
            if (kdegfd.gt.nphysi) then
               call errint ( kdegfd, 1 )
               call errint ( nphysi, 2 )
               call errsub ( 218, 2, 0, 0)
               go to 1000
            end if
            if ( kdegfd.eq.0 ) then
               lunder = 1
               lupper = nphysi
            else
               lunder = kdegfd
               lupper = kdegfd
            end if
            do i = 1, nnodes
               do ldegfd = lunder, lupper
                  idegfd = ldegfd + (i-1)*nphysi
                  if ( ichois.eq.15 ) then
                     vv = abs( vec1(idegfd)-vec2(idegfd) )
                  else
                     vv = abs( vec1(idegfd) )
                  end if
                  if ( ichois.eq.12 .and. vv.lt.a ) then

!                 --- vv < a, new minimum found

                     a    = vv
                     imin = i
                  end if
                  if ( vv.gt.b ) then

!                 --- vv > b, new maximum found

                     b    = vv
                     imax = i
                  end if
               end do
            end do
         else
            do i = 1, nnodes
               inunkp = kprobfi(i+1) - kprobfi(i)
               if ( kdegfd.eq.0 ) then
                  lunder = 1
                  lupper = inunkp
               else
                  lunder = kdegfd
                  lupper = kdegfd
               end if
               do ldegfd = lunder, lupper
                  if (ldegfd.le.inunkp) then
                     idegfd = kprobfi(i)+ldegfd
                     if ( ichois.eq.15 ) then
                        vv = abs( vec1(idegfd)-vec2(idegfd) )
                     else
                        vv = abs( vec1(idegfd) )
                     end if
                     if ( ichois.eq.12 .and. vv.lt.a ) then

!                    --- vv < a, new minimum found

                        a    = vv
                        imin = i
                     end if
                     if ( vv.gt.b ) then

!                    --- vv > b, new maximum found

                        b    = vv
                        imax = i
                     end if
                  end if
               end do
            end do
            if (imax.eq.0 ) then
               call errint ( kdegfd, 1 )
               call errint ( nphysi, 2 )
               call errsub ( 218, 2, 0, 0)
            end if
         end if

!        --- Next store the corresponding co-ordinates

         if ( ichois.lt.15 ) then
            do i = 1,ipoint
               vec3(i)   = dble ( vec2((imin-1)*ipoint+i) )
               vec3(i+3) = dble ( vec2((imax-1)*ipoint+i) )
            end do
         else
            ipoint = imax
            a = b
         end if
      end if
1000  call erclos ( 'to0032' )
      end
