      subroutine to0035 ( ichois, idegfd, iu1, iu2, &
                          iu3, const, iconst, iskip, &
                          indelgrp, vector2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.3  date 31-03-2015 Replace er0061/2 by inicopindarr1/2
!        version  5.2  date 09-03-2013 Correction ipgath
!        version  5.1  date 21-11-2012 Long integers
!
!   copyright (c) 1990-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    TO0035 is a vector manipulation subroutine
!    It is called by the SEPRAN subroutines MANVEC, VECMAN, ALGEBR, ANORM,
!    DIFFVC and MANIVC
!    TO0035 is able to manipulate vectors of common type 110, 115, 116
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision const(*)
      integer ichois, idegfd(*), iu1, iu2, iu3, &
              iconst(*), iskip, indelgrp(*)
      logical vector2

!     const         i/o  In- and output real array containing constants for the
!                        manipulation depending on ichois
!     ichois         i   Choice parameter indicating what type of function
!                        is chosen by the user.  Possible values:
!                         1:    Subtract:  VEC3 = VEC1 - CONST(1)
!                         2:    CONST(1) is average value of VEC1
!                               At output IPOINT is number of points
!                         3:    Linear combination
!                               VEC3 = CONST(1) * VEC1 + CONST(2) * VEC2
!                         6:    The minimum value of VEC1 is stored in CONST(1)
!                               The maximum value of VEC1 is stored in CONST(2)
!                               Real vectors only
!                         7:    The inner product VEC1 + VEC2 is stored in
!                               CONST(1)
!                               The number of points used is stored in IPOINT
!                         8:    VEC3 = CONST(1) * |VEC1|
!                               VEC1 is complex, VEC3 is real
!                         9:    VEC3 = CONST(1) * arg( VEC1 )
!                               VEC1 is complex, VEC3 is real
!                               If ICONST(1)=1, VEC3 = 180/pi*CONST(1)*arg(VEC1)
!                         10:   The inner product (VEC1,VEC2) is stored in
!                               CONST(1)
!                               The number of points used is stored in IPOINT
!                               Boundary conditions are excluded
!                         13:   VEC3 is a function of VEC1 and VEC2. The user
!                               subroutine FUNALG or FUNALC is called.
!                         15:   CONST(1) = || VEC1 - VEC 2 ||
!                               The type of norm must be given in ICONST(1).
!                               Possible values:
!                               1       l1-norm
!                               2       l2-norm
!                               3       max-norm
!                               4       l1-norm / number of degrees of freedom
!                               5       l2-norm / number of degrees of freedom
!                              10       max-norm  the nodal point number
!                                       corresponding to the maximum is stored
!                                       in iconst(2)
!                         16:   CONST(1) = || VEC1 ||
!                               For the type of norms available: see ICHOIS = 15
!                         18:   VEC3 = conjg(VEC1)
!                         19:   VEC3 is a function of VEC1 and VEC2. The SEPRAN
!                               subroutine FUNADA is called.
!                         22:   VEC3 is Real part of VEC1 multiplied by CONST(1)
!                         23:   VEC3 is Imaginary part of VEC1 multiplied by
!                               CONST(1)
!                         24:   Linear combination
!                               VEC3 = CONST(1) * VEC1 + CONST(2) * VEC2
!                               with constraint: u3(i)>=const(3)
!                         43:   u3 = function of u1
!                               The type of function is stored in
!     iconst        i/o  In- and output integer array containing constants for
!                        the manipulation depending on ichois
!     idegfd         i   In this array given by the user, he must store the
!                        degrees of freedom that are requested for the special
!                        purpose.
!                        In most applications only one degree of freedom per
!                        point is required, and the array may be replaced by an
!                        integer variable but not by a constant.
!                        If the variable has value 0, then all degrees of
!                        freedom are considered.
!                        If the variable has value -1, then all degrees of
!                        freedom are considered, except those corresponding to
!                        essential boundary conditions
!                        The length of the array depends on ichois:
!                        If ichois = 1-13:  Length 1
!                                      14:         2 or 3 (See iconst(1))
!                                   15-16:         1
!                                   17,18:     ?
!                                      19:         1
!     indelgrp       i   If indelgrp(i) = 1, then element group i must be
!                        skipped, otherwise it must be taken into account
!     iskip          i   If>0, element groups must be skipped
!     iu1          i   First input vector to be manipulated
!                        iu1 must be a SEPRAN array of length 5, which may be
!                        of type 110, 115, 116
!                        If ichois = 17, IVEC1 is supposed to be a more
!                        dimensional array of size 5 x NVEC, otherwise IVEC1
!                        contains only one array
!     iu2          i   Second input vector to be manipulated (if necessary)
!                        If only one array is needed, iu2 may be replaced by a
!                        dummy
!     iu3          i   Third input vector to be manipulated (if necessary)
!                        If only one or two arrays are needed, iu3 may be
!                        replaced by a dummy
!                        definition. See the Programmer's Guide for a complete
!                        description
!     vector2        i   If true the second vector is not 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, itypvc, lendiv, lenhlp, lenwor, &
              nbnout, nnodes, nusout, lenvc1, iprobsav, icompl
      logical compli, new, gather, usevc2, usevc3, scattr
      double precision, pointer :: u1orig(:), u2orig(:), u3orig(:)
      double precision, pointer :: work(:), work1(:), work2(:)
      integer, pointer :: igathr(:), ioutpoints(:), iwork(:)

!     compli         Indication if IVEC1 corresponds to a complex array (true)
!                    or to a real one (false)
!     gather         indication whether gathering has been carried out (true) or
!                    not (false)
!     i              Counting variable
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     igathr         Integer array containing the indices of the positions
!                    in vector to be copied into new compressed array
!     ioutpoints     Output array of length nnodes
!                    If ioutpoints(i)=0, node i must be skipped, else
!                    ioutpoints(i)>0, node i must not be skipped
!     iprobsav       Help parameter to save the value of iprob
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     iwork          Array IWORK is filled by subroutine ER0063 as follows:
!                    IWORK(i)>0, gives start position of unknown in point i
!                    IWORK(i) = 0, unknown is not available in point i
!                    IWORK(i)<0, point is second periodical point
!                    -IWORK(i) refers to first point
!                    Array IWORK is only filled if gather is true
!     lendiv         Length of input vector compared to output vector (1 or 2)
!     lenhlp         Length of a compressed input vector in words
!     lenvc1         Length of vector 1 in words
!     lenwor         Length of the real part in the work space in words
!     nbnout         Number of prescribed boundary conditions in IGATHR
!     new            Indication if the output vector is new (true) and hence
!                    should be cleared or not (false)
!     nnodes         Number of nodal points
!     nusout         Number of degrees of freedom in a solution vector after the
!                    compress operation
!     scattr         If true a scatter operation must be performed
!     usevc2         If true vector 2 is used
!     usevc3         If true vector 3 is used
!     u1           first input vector
!     u1orig       original u1
!     u2           second input vector
!     u2orig       original u2
!     u3           output vector
!     u3orig       original u3
!     work           Work array to store compressed u1
!     work1          Work array to store compressed u2
!     work2          Work array to store compressed u3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0063         Fill index array for gather operation
!     ER0065         General gathering subroutine (complex case)
!     ER0066         General scattering subroutine (complex case)
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERSKIPELGRP    Fill array ioutpoints for all points that are not lying in
!                    element groups that must be skipped
!     INICOPINDARR1  Copy old indexed arrayold into new array arraynew
!     INICOPINDARR2  Copy old arrayold into new indexed array arraynew
!     PRINRL         Print 1d real vector
!     SEPACTSOLBF1   compute pointers to degrees of freedom of vectors
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     TO0023         Perform some vector manipulations for real vectors
!     TO0024         Perform some vector manipulations for complex vectors
!     TO0039         Perform some vector manipulations for complex vectors
!     TOLINCOMB      Store linear combination a u1 + b u2 into u3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0035' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0035'
         write(irefwr,1) 'ichois, idegfd(1), iconst(1)', &
                          ichois, idegfd(1), iconst(1)
         write(irefwr,*) 'vector2 ', vector2
         call prinrl ( const, 4, 'const' )
         write(irefwr,1) 'iu1, iu2, iu3', iu1, iu2, iu3
  1      format ( a, 1x, (10i6) )

      end if

      iprobsav = iprob

!     --- Check type of iu1

      compli = ks(iu1)%compl/=0
      if ( debug ) write(irefwr,*) 'compli ', compli

!     --- Reserve space

      itypvc = ks(iu1)%typevec

      if ( itypvc ==116 ) then
         nphysi = ks(iu1)%ivec
         indprfi = 0
         indprhi = 0
         indprpi = 0
      else
         call sepactsolbf1 ( iu1 )
         if ( ks(iu1)%typevec==110 ) indprhi = kprobloc(50)
         if ( indprhi>0 ) then
            kprobhi => kprobpart%kprobh
            kprobhinvi => kprobpart%kprobhinv
         end if  ! ( indprfi>0 ) then
      end if

      if ( debug ) &
         write(irefwr,1) 'itypvc, indprfi, indprhi ', &
                          itypvc, indprfi, indprhi
      if ( itypvc==116 ) then

!     --- Special case, vector defined per element, set nnodes equal
!         to number of elements

         nnodes = nelem

      else

!     --- Standard case

         nnodes = npoint

      end if
      if ( itypvc==110 ) then
         if ( ichois/=10 ) npbouni = 0
      else
         nboundi = 0
         npbouni = 0
      end if
      nusoli  = size ( ks(iu1)%sol )
      if ( compli ) nusoli = nusoli / 2

      length = 1
      if ( iskip>0 ) length = npoint
      allocate ( ioutpoints(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'ioutpoints' )
      if ( ierror/=0 ) go to 1000

      if ( iskip>0 ) then

!     --- iskip>0, some element groups must be skipped
!         create array ioutpoints

         call erskipelgrp ( indelgrp, ioutpoints )

      end if
      usevc2 = ichois==3  .or. ichois==7 .or. ichois==10 .or. &
               ichois==13 .or. ichois==15 .or. ichois==19 .or. &
               ichois==24 .or. ichois==25
      usevc3 = ichois==1  .or. ichois==3  .or. ichois==8 .or. &
               ichois==9  .or. ichois==13 .or. ichois==18 .or. &
               ichois==19 .or. ichois>=22 .and. ichois<=25 .or. &
               ichois==43
      if ( debug ) write(irefwr,*) 'usevc2, usevc3 ', usevc2, usevc3

      lendiv = 1
      if ( ichois==8 .or. ichois==9 .or. ichois==22 .or. &
           ichois==23 ) lendiv = 2
      if ( usevc3 ) then

!     --- Third vector must be created
!         Check if iu3 has been filled before (new=false) or not (true)

         new = iu3<=0 .or. iu3>lastsol
         if ( .not. new ) new = solarr(iu3)==0

         if ( debug ) write(irefwr,*) 'new', new

         if ( new ) iu3 = 0

!        --- Copy information from iu1 into iu3

         if ( debug ) write(irefwr,1) 'iu3, lendiv', iu3, lendiv
         if ( lendiv==2) then
            icompl = 0
         else
            icompl = ks(iu1)%compl
         end if  ! ( lendiv==2)
         iprob = ks(iu1)%iprob
         call sepcreasolbf1 ( iu3, itypvc, ks(iu1)%ivec, &
                              'iu3', icompl )

!        --- Clear iu3 if it is a new vector

         u3orig => usoltemp
         u3 => u3orig
         if ( new ) u3orig = 0d0

      end if

      if ( debug ) &
         write(irefwr,1) 'indprfi, indprhi, indprpi, nphysi', &
                          indprfi, indprhi, indprpi, nphysi

!     --- Create temporary space

      gather = idegfd(1)>0 .and. (indprfi>0 .or. nphysi>1)
      gather = gather .or. iskip>0
      scattr = gather .and. ichois/=2 .and. ichois/=6 .and. &
               ichois/=7  .and. ichois/=10 .and. ichois/=15 .and. &
               ichois/=16 .and. ichois/=18

      if ( debug ) write(irefwr,*) 'gather, scatter', gather, scattr
      if ( gather ) then

!     --- idegfd(1)>0;
!         ichois # 4 or 5
!         For these values of ichois a gather operation is required

         lenwor = max ( nusoli, nnodes ) * (ks(iu1)%compl+1)
         lenhlp = lenwor
         if ( ichois==1 .or. ichois==2 .or. ichois==7 .or. &
              ichois==10 .or. ichois==15 .or. &
              ichois==18 .or. ichois==43 ) then

!        --- ichois = 1, 2, 7, 10, 15, 18

            lenwor = 2 * lenwor

         else if ( ichois==3 .or. ichois==13 .or. ichois==19 .or. &
                   ichois==24 .or. ichois==25 )then

!        --- ichois = 3, 13, 19, 24 or 25

            lenwor = 4 * lenwor

         else if ( ichois==8 .or. ichois==9 .or. ichois==22 .or. &
                   ichois==23 ) then

!        --- ichois = 8, 9, 22, 23
!            converting complex vector to real

            lenwor = 3 * lenwor / 2

         end if

         length = max(nusoli,nnodes)
         allocate ( work(lenwor), work1(lenwor), work2(lenwor), &
                    igathr(lenhlp), iwork(length), stat = error )
         if ( error/=0 ) call eralloc ( error, lenhlp+2*lenwor, 'work' )
         if ( ierror/=0 ) go to 1000

      end if

      if ( ierror/=0 ) go to 1000

!     --- Set some constants depending on the type of vector

      u1orig => ks(iu1)%sol
      if ( usevc2 ) then
         u2orig => ks(iu2)%sol
      else
         u2orig => u1orig
      end if  ! ( usevc2 )
      u1 => u1orig
      u2 => u2orig

      if ( .not. usevc3 ) u3 => u1
      if ( ichois/=10 .and. idegfd(1)>=0 ) indprhi = 0
      if ( idegfd(1)<0 ) then

!     --- idegfd(1) = -1,  exclude boundary conditions

         nusoli  = nusoli - nboundi - npbouni
         npbouni = 0
         nboundi = 0

      else if ( gather ) then

!     --- idegfd(1)>0, only one degree of freedom
!         First apply gather operation

         call er0063 ( nnodes, idegfd(1), iwork, igathr, nusout, nbnout, &
                       iskip, ioutpoints )

!        --- Recompute nusoli, npbouni and npbouni

         if ( ichois==10 ) then

!        --- ichois = 10

            nusoli  = nusout - nbnout
            nboundi = 0
            indprhi = 0

         else

!        --- ichois = 15 or 16

            nusoli  = nusout
            nboundi = 0

         end if
         npbouni = 0

         if ( compli ) then

!        --- complex case

            call er0065 ( u1, igathr, work, nusoli )
            if ( ichois==3 .or. ichois==7 .or. ichois==10 &
                 .or. ichois==13 .or. ichois==15 &
                 .or. ichois==19 .or. ichois==24 &
                 .or. ichois==25 ) &
            call er0065 ( u2, igathr, work1, nusoli )

         else

!        --- real case

            call inicopindarr1 ( nusoli, work, u1, igathr )
            if ( ichois==3 .or. ichois==7 .or. ichois==10 .or. &
                 ichois==13 .or. ichois==15 .or. &
                 ichois==19 .or. ichois==24 .or. &
                 ichois==25 ) &
               call inicopindarr1 ( nusoli, work1, u2, igathr )

         end if
         u1 => work(1:nusoli)
         u2 => work1(1:nusoli)
         u3 => work2(1:nusoli)

      else if ( ichois==10 ) then

         nboundi = nboundi + npbouni

      end if

      if ( debug ) write(irefwr,*) 'compli', compli

      if ( ichois==8 .or. ichois==9 .or. ichois==22 .or. &
           ichois==23 ) then

!     --- ichois = 8, 9, 22 or 23:  Real vector from complex ones

         call to0039 ( ichois, u1, u1, &
                       u3, const(1), nusoli, nboundi, &
                       indprhi, iconst(1), kprobhinvi )

      else if ( compli ) then

!     --- Complex case

         call to0024 ( ichois, u1, u2, &
                       u3, const(1), const(3), &
                       iconst(1), nusoli, nboundi, indprhi, &
                       iconst, const, kprobhinvi )

      else if ( ichois==3 .or. ichois==24 ) then

!     --- real case linear combination

         call tolincomb ( u1, u2, u3, const(1), const(2), const(3), &
                          nusoli, kprobhinvi, indprhi, vector2, ichois )

      else

!     --- real case

         call to0023 ( ichois, const(1), const(2), &
                       const(3), iconst(1), nusoli, nboundi, &
                       indprhi, kprobhinvi, iconst )

      end if
      if ( debug ) write(irefwr,*) 'scattr', scattr
      if ( scattr ) then

!     --- idegfd1(1)> 0 and ichois = 1,3,8,9,13,18,19,22,23,24
!         scatter operation

         if ( compli .and. (ichois<=3 .or. ichois==24) .or. &
               ichois==18 ) then

!         --- complex case

            call er0066 ( u3orig, igathr, u3, nusoli )

         else

!        --- real case

            call inicopindarr2 ( nusoli, u3orig, u3, igathr )

         end if

      end if
      ks(iu1)%changed = .true.
      if ( usevc2 ) ks(iu2)%changed = .true.

      if ( gather ) then

         deallocate ( work, work1, work2, igathr, ioutpoints, iwork, &
                      stat = error )
         if ( error/=0 ) call erdealloc ( error, 'work' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( gather )

      iprob = iprobsav

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( const, 2, 'const' )
         write(irefwr,*) 'End to0035'

      end if  ! ( debug )
      call erclos ( 'to0035' )

      end subroutine to0035
