      subroutine to0049 ( usol, outvec, idegfd, ndimlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 09-02-2007 Allow variable degrees of freedom
!        version  2.2  date 26-12-1994 New commons cmessc and cmessg
!        version  2.1  date 23-06-1992 New common cmessg
!        version  2.0  date 28-04-1990 Complete redesign
!        version  1.0  date 24-04-1986
!
!   copyright (c) 1986-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine computes:
!                        2         2          2
!     outvec(i) = sqrt( u (i)  +  u (i)   +  u (i) )
!                        1         2          3
!     Real case
!     Restrictions:  The input vector must contain the required degrees of
!                    freedom in each nodal point
!                    The output vector must contain exactly one degree of
!                    freedom in each nodal point
!                    All nodes, includiong boundary nodes are used
!     The output vector must have been cleared before
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndimlc, idegfd(ndimlc)
      double precision usol(*), outvec(*)

!    idegfd   i   Array in which the various degrees of freedom that must be
!                 used are stored. Length ndimlc
!    ndimlc   i   number of degrees of freedom that have been stored in array
!                 idegfd. The summation must be performed over these unknowns
!    nphysi    i   Number of physical unknowns, or maximal degrees of freedom
!                 per point
!    outvec  i/o  Output vector. Must be cleared before this subroutine
!                 Must contain exactly one unknown per point
!    usol     i   Input vector. Must contain exactly nphysi degrees of freedom
!                 per point.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ipsol

!     i              Counting variable
!     ipsol          Address in array isol
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1183   Degree of freedom out of range
!    1441   The degree of freedom is not present in all nodes of the mesh
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0049' )

!     --- Check input

      do j = 1, ndimlc
         if ( idegfd(j)>nphysi ) then
            call errint ( j, 1 )
            call errint ( idegfd(j), 2 )
            call errint ( nphysi, 3 )
            call errsub ( 1183, 3, 0, 0 )
         end if
      end do
      if ( ierror/=0 ) go to 1000

!     --- First add the squares to the output vector

      if ( indprpi==0 ) then

!     --- indprpi = 0, vector of special structure

         do j = 1, ndimlc
            do i = 1, npoint
               outvec(i) = outvec(i) + usol((i-1)*nphysi+idegfd(j))**2
            end do
         end do

      else if ( indprfi==0 ) then

!     --- indprpi>0, input vector has the structure of the solution vector
!         Number of degrees of freedom constant

         do j = 1, ndimlc
            do i = 1, npoint
               outvec(i) = outvec(i) + usol(kprobpi(i,idegfd(j)))**2
            end do
         end do

      else

!     --- indprpi>0, input vector has the structure of the solution vector
!         Number of degrees of freedom not constant per point

         do j = 1, ndimlc

            do i = 1, npoint

               ipsol = kprobpi(i,idegfd(j))

               if ( ipsol<=0 ) then

!              --- Degree of freeom not present

                  call errint ( idegfd(j), 1 )
                  call errsub ( 1441, 1, 0, 0 )
                  go to 1000

               end if  ! ( ipsol<=0 )

               outvec(i) = outvec(i) + usol(ipsol)**2

            end do

         end do
      end if

!     --- Next compute the square root of the output vector

      do i = 1, npoint
         outvec(i) = sqrt(outvec(i))
      end do

1000  call erclos ( 'to0049' )

      end
