      subroutine to0050effbc
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-07-2015 Use sepmodulebuild
!        version  1.2  date 21-04-2015 New definition intmat
!        version  1.1  date 15-11-2012 Long integers
!
!   copyright (c) 2004-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute effect of periodical boundary conditions to right-hand side
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulevecs
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jpos, i

!                    they are not
!     i              Counting variable
!     jpos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     TOST03         Compute effect of jump in periodical boundary conditions to
!                    right-hand side vector (real case)
!     TOST04         Compute effect of jump in periodical boundary conditions to
!                    right-hand side vector (complex case)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0050effbc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0050effbc'
         write(irefwr,1) 'nusol, nbound, npboun', nusol, nbound, npboun
         call prinrl ( rhsd, nusol-nbound-npboun, 'rhsd/out' )
         call prinin ( irowsic, npboun+1, 'irowsic' )
         jpos = irowsic(1)
         length = irowsic(npboun+1)-jpos
         call prinin ( icolsic(jpos+1), length, 'icolsic' )
         call prinrl ( S_ip, length, 'S_ip' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( idefcr==1 ) numrhs = 1

      if (compl) then

         do i = 1, numrhs
            rhsd => rhsds(i)%rh
            call tost04 ( nusol-nbound-npboun+1, rhsd, S1_ic, -1 )
         end do  ! i = 1, numrhs
         if ( idefcr==1 ) then
            rhsd => rhsds(2)%rh
            call tost04 ( nusol-nbound-npboun+1, rhsd, S2_ic, -1 )
         end if  ! ( idefcr==1 )

      else

         do i = 1, numrhs
            rhsd => rhsds(i)%rh
            call tost03 ( nusol-nbound-npboun+1, S1_ic, -1 )
         end do  ! i = 1, numrhs
         if ( idefcr==1 ) then
            rhsd => rhsds(2)%rh
            call tost03 ( nusol-nbound-npboun+1, S2_ic, -1 )
         end if  ! ( idefcr==1 )

      end if  ! (compl)
      if ( idefcr==1 ) numrhs = 2

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( rhsd, nusol-nbound-npboun, 'rhsd/out' )
         write(irefwr,*) 'End to0050effbc'

      end if
      call erclos ( 'to0050effbc' )

      end
