      subroutine to0052 ( irhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  9.0  date 24-07-2015 Use sepmodulebuild
!        version  8.6  date 15-07-2015 Adaptation for complex
!        version  8.5  date 21-04-2015 remove ipoint(5)
!
!   copyright (c) 1988-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorder right-hand sides and
!     start create petsc matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     pointer
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmoduleoldvecs
      use sepmodulebuild
      use sepmodulekprob
      use sepmodulecpack
      use sepmodulecons
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irhsd(5,*)

!     irhsd         i/o   Standard SEPRAN array, containing information of the
!                         large vector(s)
!                         In case of defect correction the right-hand side
!                         consists of two parts:
!                         irhsd(.,i) refers to the right-hand side corresponding
!                                    to matr(.,i)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k
      double precision, allocatable :: work(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     k              Counting variable
!     work           work array for subroutine toordc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PETSCCREAMATS  reserve space for petsc matrix and start creation
!     TOORDC         Map array of solution type from vector sequence to matrix
!                    sequence. Complex version
!     TOORDR         Map array of solution type from vector sequence to matrix
!                    sequence. Real version
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initializations

      call eropen ( 'to0052' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0052'

      end if
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- RIGHT-HAND SIDE

      if ( clrvec .or. addvec .or. effbc.and.nbound>0 ) then

         do k = 1, numrhs

            if ( .not. clrvec .and. indprh>0 .and. &
                 irhsd(2,k)==110 .and. &
                 irhsd(4,k)/1000==iprob-1 ) then

!           --- right-hand side existed before

               allocate ( work(nsol), stat = error )
               if ( error/=0 ) call eralloc ( error, nsol, 'work' )
               if ( ierror/=0 ) go to 1000

               if ( compl ) then

!              --- complex case

                  if ( mod(irhsd(3,k),10)==1 .and. irhsd(5,k)==nsol/2 ) then

!                 --- Renumber rhsd

                     call toordc ( kprobh, rhsds(k)%rh, nsol/2, work )

                  end if  ! ( mod(irhsd(3,k),10)==1 .and. irhsd(5,k)==nsol/2 )

               else

!              --- real case

                  if ( mod(irhsd(3,k),10)==0 .and. irhsd(5,k)==nsol ) then

!                 --- Renumber rhsd

                     call toordr ( kprobh, rhsds(k)%rh, nsol )

                  end if

               end if  ! ( compl )

               deallocate ( work, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'work' )
               if ( ierror/=0 ) go to 1000

            end if  ! ( .not. clrvec .and. indprh>0 .and. ... )

!           --- Clear vector if necessary

            if ( clrvec ) rhsds(k)%rh = 0d0

         end do  ! k = 1, numrhs

      end if  ! ( clrvec.or. addvec .or. effbc.and.nbound>0 )

!     --- Create space for matrix for petsc usage

      if ( IM%nosepmat ) then

!     --- petsc with matrix made directly
!         Must be changed in the future

         if ( parallel ) then

            call petsccreamats ( nrusol, nnz,  onnz, nusolglob )

         else

            call petsccreamats ( nrusol, IM%rowsii, onnz, &
                                 nusolglob )

         end if  ! ( parallel )

      end if  ! ( IM%nosepmat )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0052'

      end if
      call erclos ( 'to0052' )

      end
