      subroutine to00641
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-07-2015 Use sepmodulebuild
!        version  1.1  date 24-03-2000 Correction for non-constant degrees of
!                                      freedom per point
!        version  1.0  date 04-06-1999
!
!   copyright (c) 1999-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array index4 starting position for node in index2
!     This subroutine is only called if to0064 is not called
!     Help subroutine for subroutine TO0050
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     topology
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, inunrf, inusol, inunkp, isum
      logical consta

!     consta         If true the number of unknowns per point is constant
!     inunkp         Number of degrees of freedom in point
!     inunrf         Reference value for inunkp
!     inusol         Position of unknown in array USOL
!     isum           Integer variable to compute a sum
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    if ( indprf=0 ) then
!       index4(1,j) := j*inunkp
!    else ( indprf>0 )
!       Fill index4 using kprobf
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to00641' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to00641'
         write(irefwr,1) 'indprf', indprf
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprf.le.0 ) then

!     --- indprf = 0, number of unknowns constant

         inunkp = icount/inpelm
         do j = 1, inpelm
            index4(1,j) = j*inunkp
         end do
         if ( inunkp.eq.1 ) then
            index4(1,inpelm+1) = 1
         else
            index4(1,inpelm+1) = 2
         end if

      else

!     --- indprf > 0, number of unknowns not constant

         isum = 0
         consta = .true.
         do j = 1, inpelm
            inusol = kprobf(index1(j))
            inunkp = kprobf(index1(j)+1)-inusol
            if ( j.eq.1 ) then
               inunrf = inunkp
            else
               if ( inunrf.ne.inunkp ) consta = .false.
            end if
            isum = isum + inunkp
            index4(1,j) = isum
         end do
         if ( consta .and. inunkp.eq.1 ) then
            index4(1,inpelm+1) = 1
         else if ( consta ) then
            index4(1,inpelm+1) = 2
         else if ( isum.gt.0 ) then
            index4(1,inpelm+1) = -1
         else
            index4(1,inpelm+1) = -2
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to00641'

      end if  ! ( debug )
      call erclos ( 'to00641' )

      end subroutine to00641
