      subroutine to0087 ( vecinp, vecout, indpfi, kprbfi, indpfo, &
                          kprbfo, npoint, nphys )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-10-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!subroutine to0087 is used to map a SEPRAN vector of one structure
!       onto a SEPRAN vector of a different structure
!       Complex case, the real equivalent is TO0082
!       Restriction: this subroutine may only be used if both arrays
!                    correspond to the same mesh and the number of degrees
!                    of freedom per point is identical
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex vecinp(*), vecout(*)
      integer indpfi, kprbfi(*), indpfo, kprbfo(*), npoint, nphys

!     indpfi  i   Indication if the number of unknowns per point is constant
!                 (indpfi=0) or not (indpfi>0) with respect to the input vector
!     indpfo  i   Indication if the number of unknowns per point is constant
!                 (indpfo=0) or not (indpfo>0) with respect to the output vector
!     kprbfi  i   Array KPROB part f for input vector
!     kprbfo  i   Array KPROB part f for output vector
!     nphys   i   Maximal number of unknowns per point
!     npoint  i   Number of nodal points
!     vecinp  i   Input vector to be mapped
!     vecout  o   Output vector, result of the mapping
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nusol

!    i        General loop variable
!    nusol    Number of unknowns in vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY     Copy double precision dx to double precision dy
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1743   The number of unknowns per point is constant for the input
!           vector but not for the output vector
!    1744   The number of unknowns per point is constant for the output
!           vector but not for the input vector
!    1745   The number of unknowns per point for the input vector and
!           output vector are not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
!   If ( number of unknowns per point in input vector is constant ) then
!      Check if number of unknowns per point in output vector is constant
!      Compute total number of unknowns
!   else ( number of unknowns per point in input vector is not constant )
!      Check if number of unknowns per point in output vector is not constant
!      Compare KPRBFI and KPRBFO
!      Compute total number of unknowns
!   If ( input vector is not renumbered ) then
!      If ( output vector is not renumbered ) then
!         Copy input vector into output vector
!      else ( output vector is renumbered ) then
!         Map input vector into output vector
!   else ( input vector is renumbered ) then
!      If ( output vector is not renumbered ) then
!         Map input vector into output vector
!      else ( output vector is renumbered ) then
!         Map input vector into output vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0087' )
      if ( indpfi.eq.0 ) then

!     --- indpfi = 0, number of unknowns per point is constant

         if ( indpfo.ne.0 ) then
            call errsub ( 1743, 0, 0, 0 )
            go to 1000
         end if
         nusol = npoint * nphys

      else

!     --- indpfi > 0, number of unknowns per point is not constant

         if ( indpfo.eq.0 ) then
            call errsub ( 1744, 0, 0, 0 )
            go to 1000
         end if

!        --- Check if kprbfi and kprbfo are identical

         do i = 1, npoint

            if ( kprbfi(i).ne.kprbfo(i) ) then
               call errint ( i, 1 )
               call errint ( kprbfi(i+1)-kprbfi(i), 2 )
               call errint ( kprbfo(i+1)-kprbfo(i), 3 )
               call errsub ( 1745, 3, 0, 0 )
               go to 1000
            end if

         end do
         nusol = kprbfi(npoint+1)

      end if

      call dcopy ( 2*nusol, vecinp, 1, vecout, 1 )

1000  call erclos ( 'to0087' )
      end
