      subroutine tost04 ( ilow, rhsdloc, amatloc, isign )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 24-07-2015 Use sepmodulebuild
!        version  2.2  date 15-11-2012 Long integers
!        version  2.1  date 02-12-1994 New computation of pointer
!
!   copyright (c) 1989-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute effect of periodical boundary conditions to right-hand side
!    in the case of complex arithmetic
!    Compare with subroutine to0010
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     complex
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isign, ilow
      double complex, intent(in) :: amatloc(*)
      double complex, intent(inout) :: rhsdloc(*)

!   amatloc     i    Second part of large matrix corresponding to boundary
!                    conditions
!   ilow        i
!   isign       i    sign of multiplication, for right-hand side isign should
!                    be -1, in matrix-vector multiplication isign = 1
!   rhsdloc       i/o   Right-hand side to be updated
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k

!   i       General loop variable
!   j       General loop variable
!   k       index in array rhsdloc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The right-hand side must be updated in the following way
!               f = f + isign S   c
!                              ir  r
!    The matrix S   is stored in the sequence S
!                ir                            ri
!    Pseudo code
!    For i := 1(1)npboun do
!        for j := irowsic(i)+1,irowsic(i+1)
!           rhsdloc(icolsic(j)) := rhsdloc(icolsic(j))+isign*amatloc(j)*rprobs(i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i = 1, npboun
         do j = irowsic(i)+1, irowsic(i+1)
            k = icolsic(j)
            if ( k>=ilow ) k = kprobs(k-ilow+1)
            rhsdloc(k) = rhsdloc(k)+isign*amatloc(j)*rprobs(i,1)
         end do  ! i = 1, npboun
      end do  ! j = irowsic(i)+1, irowsic(i+1)
      end
