      subroutine ermatvec ( matrix, vector, rhsd, nrows )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute matrix vector multiplication for a full matrix
!     It is supposed that the matrix is stored in the mathematical sequence
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     vector
!     multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nrows
      double precision matrix(nrows,nrows), vector(nrows), rhsd(nrows)

!     matrix         i    nrows by nrows matrix to be used in multiplication
!     nrows          i    size of matrix and right-hand side
!     rhsd           o    Result of multiplication
!     vector         i    vector to be used in multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k
      double precision sum

!     i              Counting variable
!     k              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for i := 1 (1) nrows do
!        rhsd(i) := sum matrix(i,k) vector(k)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ermatvec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ermatvec'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- for i := 1 (1) nrows do

      do i = 1, nrows

!     --- rhsd(i) := sum matrix(i,k) vector(k)

         sum = 0d0
         do k = 1, nrows
            sum = sum + matrix(i,k)*vector(k)
         end do  ! k = 1, nrows
         rhsd(i) = sum

      end do  ! i = 1, nrows

1000  call erclos ( 'ermatvec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ermatvec'

      end if

      end
