      subroutine erperiodreac ( reacvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-04-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Incorporate the effect of periodicity to reaction forces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     reaction_force
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision reacvec(*)

!     reacvec       i/o   Reaction force vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npairs, i, istart, num, j
      double precision sum

!     i              Counting variable
!     istart         First position used at a part
!     j              Counting variable
!     npairs         Number of pairs of degrees of freedom with periodical
!                    boundary conditions
!     num            Number of degrees of freedom per pair
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The idea is the following:
!        each degree of freedom of a set of periodical points refers to the
!        same unknown.
!        However if such a degree of freedom refers to an essential boundary
!        condition the periodicity is not taken into account
!        As a conseqeunce there might be a reaction force in those points
!        which should not be there
!        To solve it the reaction forces in periodical points are added
!        and the sum is used to replace the old reaction force
!     npairs := kprobab(1)
!     for i := 1 (1) npairs do
!        istart := kprobab(1+i)
!        num := kprobab(istart)
!        sum = 0d0
!        for j = 1 (1) num do
!           sum := sum + reacvec(kprobab(istart+j))
!        for j = 1 (1) num do
!           reacvec(kprobab(istart+j)) := sum
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erperiodreac' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erperiodreac'

      end if

!     --- npairs := kprobab(1)

      npairs = kprobab(1)

!     --- for i := 1 (1) npairs do

      do i = 1, npairs

!     --- istart := kprobab(1+i)
!          num := kprobab(istart)
!          sum = 0d0

         istart = kprobab(1+i)
         num = kprobab(istart)
         sum = 0d0
         if ( debug ) write(irefwr,*) 'i, istart, num ', i, istart, num

!        --- for j = 1 (1) num do
!               sum := sum + reacvec(kprobab(istart+j))

         do j = 1, num
            sum = sum + reacvec(kprobab(istart+j))
         end do
         sum = sum/num
         if ( debug ) write(irefwr,*) 'sum ', sum

!        --- for j = 1 (1) num do
!               reacvec(kprobab(istart+j)) := sum

         do j = 1, num
            reacvec(kprobab(istart+j)) = sum
         end do

      end do  ! i = 1, npairs

      if ( debug ) write(irefwr,*) 'End erperiodreac'
      call erclos ( 'erperiodreac' )

      end
