      subroutine prcompgrad ( igrad, iseqlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-07-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute gradient of a vector with one component per point
!     it is assumed that this is the vecor defined by icheld = 2
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     gradient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesmf
      use sepmodulemain
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, target :: igrad
      integer, intent(in) :: iseqlc

!     igrad         i/o   Sepran array containing information of the gradient
!     iseqlc         i    Sequence number of solution vector in isolut2d from
!                         which the gradient must be computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: idum

!                    they are not
!     idum           dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFILLSMF1    Fill part of input for subroutine SEPBUILD in SMF
!     SEPPRINTV      Print solution vector or vector of special structure
!     TO0100         Performs the actual computation of the derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompgrad' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompgrad'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array iinder

      iinder = 0
      iinder(1) = 3   ! call by derivs
      iinder(4) = 2  ! icheld
      iinder(8) = iseqlc   ! numvec

      iinder(10:11) = iseqlc  ! input vector
      if ( debug ) call prinin ( iinder, 15, 'iinder' )

!     --- call actual subroutine

      call sepfillsmf1 ( isolut2d, maxvec )
      SMF%ioutvc => igrad
      call to0100 ( idum, idum )

1000  if ( debug ) then

!     --- Debug information

         call sepprintv ( igrad, 'grad' )
         write(irefwr,*) 'End prcompgrad'

      end if  ! ( debug )
      call erclos ( 'prcompgrad' )

      end
