      subroutine prcomppsimat ( iseqvel, iseqpsi, numold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Build matrix and zero right-hand side for pseudo stream function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     build
!     boundary
!     mesh
!     stream_function
!     free_surface
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulemain
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqvel, iseqpsi, numold

!     iseqpsi        i    sequence number of array psi in isol
!     iseqvel        i    sequence number of velocity in isol
!     numold         i    Number of vectors that are stored in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinbldlc(10), istart

!     iinbldlc       User input array for subroutine BUILD
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBUILD       assemble a system of equations
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPPRINTMAT    Print matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcomppsimat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcomppsimat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iinbldlc = 0       ! clear array
      iinbldlc(1) = 10   ! number of entries filled
      iinbldlc(2) = 1    ! number of entries filled
      iinbldlc(10) = numold   ! number of vectors in isol
      irhsd(1) = 0        ! clear rhs

!     --- Fill coefficients in iuser and user

      allocate ( iuser(13), user(1) )
      iuser = 0                 ! clear iuser
      user = 0d0                ! clear user
      istart = 8                ! start address for boundary group 1
      iuser(1) = 13
      iuser(2) = 2              ! nelgrp+nbgrp
      iuser(3) = 13
      iuser(7) = istart         ! start address for boundary group 1
      istart = istart-1
      iuser(istart+1) = iseqvel ! sequence number of velocity

      call sepfillsmf ( iinbldlc, irhsd, isolut, isolut2d )
      call sepbuild
      deallocate ( iuser, user )

1000  if ( debug ) then

!     --- Debug information

         call sepprintmat ( stiffmat )
         write(irefwr,*) 'End prcomppsimat'

      end if  ! ( debug )
      call erclos ( 'prcomppsimat' )

      end
