      subroutine prdividereac ( ireac, ndegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 11-12-2012 Long integers
!        version  2.0  date 13-02-2006 Extra parameter ndegfd
!        version  1.0  date 25-08-2005
!
!   copyright (c) 2005-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide the reaction force vector by the diagonal mass matrix
!     defined by the surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_matrix
!     subdivision
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ireac, ndegfd

!     ireac          i    Part of the set of solution arrays containing the
!                         information of the reaction force
!     ndegfd         i    Number of degrees of freedom per point in the
!                         reaction force.
!                         This number is supposed to be constant.
!                         Only the first degree of freedom is subdivided by the
!                         mass matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRDIVIDEREAC1  Subdivide the reaction force vector by the diagonal mass
!                    matrix defined by the surface
!                    1 degree of freedom per point
!     PRDIVIDEREAC2  Subdivide the reaction force vector by the diagonal mass
!                    matrix defined by the surface
!                    constant number of degrees of freedom per point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prdividereac' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prdividereac'

      end if
      if ( ierror/=0 ) go to 1000

      reac => ks(ireac)%sol

!     --- Actual computation

      if ( ndegfd==1 ) then

!     --- ndegfd = 1, only one component in the vector

         call prdividereac1 ( npoint, reac, matrixm )

      else

!     --- ndegfd>1, more components in the vector
!         only the first one is subdivided

         call prdividereac2 ( npoint, reac, matrixm, ndegfd )

      end if  ! ( ndegfd==1 )

!     --- remove mass matrix

      deallocate ( matrixm )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prdividereac'

      end if
      call erclos ( 'prdividereac' )

      end subroutine prdividereac
