      function prenthlinesea1 ( deltatemp, tempold, npoint, TOLX )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 19-10-2005
!
!   copyright (c) 2003-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the minimum value of scaling parameter alam
!
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     line_search
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint
      double precision deltatemp(npoint), tempold(npoint), TOLX, &
                       prenthlinesea1

!     deltatemp      i    Search direction vector
!     npoint         i    Number of points
!     prenthlinesea1 o   Minimum value of scaling parameter alam
!     tempold        i    Temperature at the previous time level
!     tolx           i    Maximum tolerated error on the temperature
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision temp, test

!     i              Counting variable
!     temp           Help variable
!     test           Help variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prenthlinesea1' )

      test = 0d0

      do i = 1, npoint

         temp = abs(deltatemp(i))/max(abs(tempold(i)),1d0)
         if (temp>test) test = temp

      end do

      prenthlinesea1 = TOLX/test

      call erclos ( 'prenthlinesea1' )

      end
