      subroutine prenthrefine1 ( meshnr, basis_mesh, mesh_new, nelem_basis, &
                                 iseqtemp )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.3  date 11-12-2017 New storage of intmat
!        version  1.2  date 11-12-2012 Long integers
!        version  1.1  date 20-10-2005 Obsolete arrays are deleted
!                                      mmintnodesold and mmrefineold not
!                                      set to 0 (!)
!
!   copyright (c) 2005-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Swap meshes and arrays and backup some data. Remove unnecesarry
!     arrays
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulecons
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem_basis, basis_mesh, mesh_new, meshnr, iseqtemp

!     basis_mesh     i    Mesh sequence number of the basis mesh
!     iseqtemp       i    Sequence number of temperature in solution vector
!     mesh_new       i    Mesh sequence number of the new mesh
!     meshnr         i    Mesh sequence number
!     nelem_basis    i    Number of elements in basis mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ireadstruc, mesh_old, iseqprob

!     ireadstruc     Sequence number of input for matrix structure
!     iseqprob       Sequence number of problem input
!     mesh_old       Sequence number of old mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCOMGEN       Compute matrix structure for free boundary problem
!     PRININ         print 1d integer array
!     PROBDFEXT      Reads and stores problem definition
!     PROCOPRED      Copy some parts of kprob1 to kprob2
!     SEPRMINTMT     Remove array with sequence number iseq from intmt
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
!     SEPRMPROBINFO  Remove array with sequence number kproblc from kp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prenthrefine1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prenthrefine1'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set meshnr equal to mesh_new and remove meshnr
!         unless meshnr equals basis_mesh


      if ( meshnr/=mesh_new ) then

!     --- meshnr # mesh_new copy some parts of old kprob to new one

         call procopred ( kprob2d(meshnr), &
                          kprob2d(mesh_new) )     ! copy part of kprob

      end if  ! ( meshnr/=mesh_new )
      if ( meshnr/=basis_mesh .and. meshnr/=mesh_new ) then

!     --- meshnr # basis mesh, remove old mesh and problem description

         ks(isolut3d(iseqtemp,mesh_new))%iseqkp = 0
         call seprmmeshinfo ( kmesh2d(meshnr) )  ! Remove kmesh
         call seprmprobinfo ( kprob2d(meshnr) )

!        --- Remove matrix structure

         call seprmintmt ( intmat2d(1,meshnr) )

!        --- Remove old solution arrays for this mesh
!            at this moment only the temperature, later on all arrays

      end if  ! ( meshnr/=basis_mesh )
      ireadstruc = -1           ! eventjes omdat we niet beter weten!!!!!!
      if ( ireadstruc<0 ) then

!     --- ireadstruc = -1, copy jmetod from meshorig to meshnr for all
!         problems

         iseqkp = kprob2d(basis_mesh)
         nprob = kp(iseqkp)%nprob
         intmat2d(1:nprob,mesh_new) = intmat2d(1:nprob,meshnr)

      end if  ! ( ireadstruc<0 )

!     --- Set meshnr equal to new mesh

      mesh_old = meshnr
      meshnr = mesh_new

      if ( debug ) then
         write(irefwr,100) 'meshnr', meshnr
         call prinin(kprob2d( meshnr), 50, 'kprob')
      end if  ! ( debug )

      iseqprob = 1
      kmesh => kmesh2d(meshnr)
      kprob => kprob2d(meshnr)
      call probdfext ( 3, iseqprob )
      if (debug) call prinin(kprob2d( meshnr), 50, 'kprob')

!     --- Actual filling of matrix structure

      intmat1d => intmat2d(:,meshnr)
      kprob => kprob2d(meshnr)
      call incomgen ( ireadstruc, intmat2d(1,mesh_old) )

!     --- Copy refinenew to refineold since refineold will be of
!         importance during the next time step
!         We need not set mmrefineold to 0 since it is saved
!         throughout the computations

      if ( associated(refineold) ) deallocate ( refineold )
      allocate ( refineold(size(refinenew)) )
      refineold = refinenew
      if (debug) &
         call prinin ( refineold, nelem_basis, &
                         'refineold after copying refinenew' )

!     --- Copy intnodesnew to intnodesold since intnodesold will be of
!         importance during the next time step

      if (debug) &
             call prinin ( intnodesnew, nelem_basis, &
                         'intnodesnew before copying intnodesnew' )

!     --- We need not set mmintnodesold to 0 since it is saved
!         throughout the computations

      if ( associated(intnodesold) ) deallocate(intnodesold)
      allocate ( intnodesold(size(intnodesnew)) )
      intnodesold = intnodesnew

      if (debug) then
         call prinin ( intnodesold, nelem_basis, &
                       'intnodesold after copying intnodesnew' )
      end if

!     --- Remove obsolete arrays

      deallocate ( intnodesnew, refinenew )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prenthrefine1'

      end if
      call erclos ( 'prenthrefine1' )

      end
