      subroutine prfilliinbld ( iinbld, iconsrhs, imas, iprob, numold, &
                                iseqveloc, isubdivide )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays iinbld for building of matrix with defaults and
!     some paramters from the input file
!     This subroutine is meant for time-dependent cases
! **********************************************************************
!
!                       KEYWORDS
!
!     default
!     input
!     fill
!     matrix
!     time_dependent
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinbld(15), iconsrhs, imas, iprob, numold, &
              iseqveloc, isubdivide

!     iconsrhs       i    Information concerning the right-hand side
!                         i.e. the function f (no effect of boundary conditions)
!                         0:  The right-hand side is time-dependent
!                         1:  The right-hand side is time-independent
!                         2:  The right-hand side is zero
!     iinbld         o    User input array for subroutine BUILD
!     imas           i    Defines how mass matrix must be build
!                         0: not
!                         1: diagonal
!                         2: consistent
!     iprob          i    Actual problem number
!     iseqveloc      i    Sequence number of velcoity (unknown) vector
!     isubdivide     i    indicates if quadratic elements must be
!                         treated as a conglomerate of linear elements
!                         (1) or as quadratic elements itself (0)
!     numold         i    Number of vectors that are stored in VECOLD
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfilliinbld' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfilliinbld'

      end if
      if ( ierror/=0 ) go to 1000

      iinbld(1) = 15          ! number of entries filled
      if ( iconsrhs==2 ) then
         iinbld(2) = 3        ! build matrix; right-hand side is 0
      else
         iinbld(2) = 1        ! build matrix and right-hand side
      end if
      iinbld(3) = 1           ! do not use essential boundary conditions
      iinbld(4) = imas        ! mass matrix
      iinbld(5) = 0           ! not important
      iinbld(6) = 0           ! allelements (default)
      iinbld(7) = 0           ! addmatvec (default)
      iinbld(8) = 0           ! CLEARLASTRHSD (default)
      iinbld(9) = iprob       ! problem definition number
      iinbld(10) = numold     ! number of old solutions
      iinbld(11) = 0          ! ISPECTRAL (default)
      iinbld(12) = 1          ! number of right-hand sides
      iinbld(13) = iseqveloc  ! ISEQIN, sequence number of velocity
      iinbld(14) = 0          ! no defect correction
      iinbld(15) = isubdivide ! isubdivide, indicates if quadratic elements
                              ! must be treated as a conglomerate of linear
                              ! elements (1)or as quadratic elements itself (0)
      if ( debug ) call prinin ( iinbld, 15, 'iinbld' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfilliinbld'

      end if
      call erclos ( 'prfilliinbld' )

      end
