      subroutine prfillinterf1 ( coor, iobst, ndim, icrossobst, &
                                 rcrossobst, nmeshc, ninterf, &
                                 coorinter )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coordinates of interface nodes into array coorinter
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     level_set
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icrossobst(*), nmeshc(*), iobst, ndim, ninterf
      double precision rcrossobst(*), coor(ndim,*), &
                       coorinter(ndim,ninterf)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coorinter      o    Contains coordinates of the interface nodes
!                         in the local numbering of the interface
!     icrossobst     i    array containing information about the cross section
!                         of obstacles with edges
!                         The storage of icrossobst is as follows
!                         Pos. 1 to numobst (number of obstacles):
!                               icrossobst(iobst) = istartobst (starting address
!                               of obstacle iobst in array
!                         Per obstacle from position istartobst:
!                         Number of cross sections, followed by 2 positions for
!                         each cross section consisting of the edge sequence
!                         number and a reference to rcrossobst
!                         This part is followed by the number of elements
!                         that is intersected by the obstacle followed by
!                         for each element: element number, number of
!                         intersections and sequence numbers of intersections
!     iobst          i    Obstacle sequence number
!     ndim           i    Dimension of the space.
!     ninterf        i    Number of nodes in interface
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     rcrossobst     i    Real array corresponding to icrossobst
!                         For each cross section a real factor f is stored
!                         defining the intersection of the edge with the
!                         obstacle in the following way
!                         x_intersect = x1 + f * (x2-x1)
!                         where x1 and x2 are the first and last point of the
!                         edge respectively
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istarticross, i, iedge, istart, numnodes, node1, node2
      double precision fact

!     fact           factor on edge defining the relative distance between
!                    interface and edge nodes
!     i              Counting variable
!     iedge          edge sequence number
!     istart         Starting address
!     istarticross   Starting address in array icrossobst
!     node1          Node of edge
!     node2          Node of edge
!     numnodes       Number of nodal points in edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillinterf1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillinterf1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istarticross = icrossobst(iobst)
      do i = 1, ninterf

         iedge = icrossobst(istarticross+1)
         istart = nmeshc(iedge)
         numnodes = nmeshc(istart)
         node1 = nmeshc(istart+1)
         node2 = nmeshc(istart+numnodes)
         fact = rcrossobst(icrossobst(istarticross+2))
         coorinter(:,i) = fact*coor(:,node2)+(1d0-fact)*coor(:,node1)
         istarticross = istarticross+2

      end do  ! i = 1, numintersect

1000  call erclos ( 'prfillinterf1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillinterf1'

      end if  ! ( debug )

      end
