      subroutine prkirchback ( iinenthalpy, rinenthalpy, islold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 01-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform temperature from normalized temperature back to
!     physical temperature
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinenthalpy(*), islold(*)
      double precision rinenthalpy(*)

!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     islold        i/o   User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nusol, iseqtemp

!     iseqtemp       Sequence number of enthalpy with respect to array isol
!     nusol          Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRKIRCHOFF     Transform temperature to normalized Kirchoff temperature
!                    and compute enthalpy or transform  normalized Kirchoff
!                    temperature back
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set parameters
!     Activate the temperature vector
!     Compute starting addresses
!     Transform back
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prkirchback' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prkirchback'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set parameters

      iseqtemp = iinenthalpy(3)

!     --- Activate the temperature vector


!     --- Compute starting addresses

      usoltemp => ks(islold(iseqtemp))%sol
      nusol = size ( usoltemp )

!     --- Transform back

      call prkirchoff ( usoltemp, usoltemp, nusol, 1, rinenthalpy )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prkirchback'

      end if
      call erclos ( 'prkirchback' )

      end subroutine prkirchback
