      subroutine prmakedist1d1 ( interf, ninterf, coor, npoint, phi, &
                                 iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make exact distance function in 1d
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     level_set
!     distance
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ninterf, interf(ninterf), npoint, iwork(npoint)
      double precision coor(npoint), phi(npoint)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     interf         i    Contains node numbers of interface
!     iwork          i    integer work array to mark the nodes
!                         -10:  node with phi<0 outside the band
!                          -1:  node with phi<0 inside the band
!                           0:  node on interface
!                           1:  node with phi>0 inside the band
!                          10:  node with phi>0 outside the band
!     ninterf        i    number of nodes in array interf
!     npoint         i    Number of nodal points
!     phi           i/o   contains the distance function
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inodeinterf
      double precision dist, coorint

!     coorint        coordniate of interface
!     dist           Distance between two points
!     i              Counting variable
!     inodeinterf    node number of interface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3117   More than 1 interface point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakedist1d1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakedist1d1'
         call prinin ( interf, ninterf, 'interf' )
         call prinin ( iwork, npoint, 'iwork' )
         call prinrl ( phi, npoint, 'phi' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ninterf>1 ) then

!     --- error more than 1 interface point not yet implemented

         call errint ( ninterf, 1 )
         call errsub ( 3117, 1, 0, 0 )
         go to 1000

      end if  ! ( ninterf>1 )

      inodeinterf = interf(1)
      coorint = coor(inodeinterf)

      if ( debug ) then
         write(irefwr,1) 'inodeinterf', inodeinterf
         write(irefwr,2) 'coorint', coorint
      end if  ! ( debug )

      do i = 1, npoint
         if ( iwork(i)<0 ) then
            dist = abs(coor(i)-coorint)
            phi(i) = -dist
         else if ( iwork(i)>0 ) then
            dist = abs(coor(i)-coorint)
            phi(i) = dist
         end if  ! ( iwork(i)<0 )
      end do  ! i = 1, npoint

1000  call erclos ( 'prmakedist1d1' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( phi, npoint, 'phi' )
         write(irefwr,*) 'End prmakedist1d1'

      end if  ! ( debug )

      end
