      subroutine prmatlab
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 23-02-2016 New parameters list
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 21-12-2008
!
!   copyright (c) 2008-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write matrix and right-hand side to files in matlab format
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matlab
!     matrix
!     right_hand_side
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulematstr
      use sepmodulematr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, allocatable :: rhsdcopy(:)

!                    they are not
!     rhsdcopy       Array to store the renumbered right-hand side
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRBLOCKMATLAB  Print matrices for block preconditioner to files that can
!                    be read by matlab
!     PRMAT2MATLAB   Print a sepran vector in a form suitable for reading
!                    in matlab
!     PRVEC2MATLAB   Print a sepran vector in a form suitable for reading
!                    in matlab
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmatlab' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmatlab'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- write matrix

      select case ( jmethod )

      case( 1,2,3,4,7,8,9,10,11,12,19 )

!     --- jmethod = 1 to 4, direct method, jump
!                   7 to 12 complex or different storage also jump

         go to 1000

      case( 5, 6 )

!     --- jmethod = 5, 6, cg-type method

         call prmat2matlab ( jmethod, nrusol, S_ii, irowsii, 'a.dat' )

      case( 13, 14, 15, 16 )

!     --- jmethod = 13-16, schur storage

         call prblockmatlab

      end select  ! case ( jmethod )

!     --- write right-hand side

      call prvec2matlab ( nrusol, rhsd, 'r.dat' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmatlab'

      end if  ! ( debug )
      call erclos ( 'prmatlab' )

      end
