      subroutine prmatlabmatc ( jmetod, n, diag, amatl, amatu, intmt1, &
                                intmt2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-02-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    print complex compact matrix in a form suitable to be read by matlab
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jmetod, n, intmt1(*), intmt2(*)
      complex *16 diag(*), amatl(*), amatu(*)

!     amatl      i    lower triangular part of the large matrix
!     amatu      i    upper triangular part of the large matrix
!     diag       i    Diagonal of the large matrix
!     intmt1     i    first part of array intmat
!     intmt2     i    part 2 of array intmat
!     jmetod     i    Indicates type of storage scheme for the matrix
!                     if 0, only the diagonal is printed
!     n          i    number of degrees of freedom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jend, j, jstart, ii

!     i              Counting variable
!     ii             address of matrix entry in 1D form
!     j              Counting variable
!     jend           end position of a row
!     jstart         starting position of row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmatlabmatc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmatlabmatc'
         write(irefwr,1) 'n, format_matlab, jmetod', &
                          n, format_matlab, jmetod
         call prinrl ( diag, 10, 'diag' )
         call prinrl ( amatl, 10, 'amatl' )
         call prinrl ( amatu, 10, 'amatu' )
         call prinin ( intmt1, 10, 'intmt1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

100   format(i9,1x,i9,ss,1pe25.16e2, 3x, 1pe25.16e2)
110   format(i9,ss,1pe25.16e2, 3x, 1pe25.16e2)
      if ( format_matlab==0 ) then

!     --- format_matlab = 0, default format defined by Kees Vuik

         write(irefwr,100) (i,i,diag(i),i=1,n)
         if ( jmetod==0 ) go to 1000
         do i=1,n
            jstart=intmt1(i)+1
            jend=intmt1(i+1)
            write(irefwr,100) (i,intmt2(j),amatl(j),j=jstart,jend)
         end do
         if(jmetod/=8) go to 1000
         do i=1,n
            jstart=intmt1(i)+1
            jend=intmt1(i+1)
            write(irefwr,100) (intmt2(j),i,amatu(j),j=jstart,jend)
         end do

      else if ( format_matlab==1 ) then

!     --- format_matlab = 1, compact format defined by Mehfooz ur Rehman

         do i=1,n
            ii=i+(i-1)*n
            write(irefwr,110) ii, diag(i)
         end do
         if ( jmetod==0 ) go to 1000
         do i=1,n
            jstart=intmt1(i)+1
            jend=intmt1(i+1)
            do j=jstart,jend
               ii= i+(intmt2(j)-1)*n
               write(irefwr,110)  ii, amatl(j)
            end do
         end do
         if(jmetod/=8) go to 1000

         do i=1,n
            jstart=intmt1(i)+1
            jend=intmt1(i+1)
            do j=jstart,jend
               ii=intmt2(j)+(i-1)*n
               write(irefwr,110) ii, amatu(j)
            end do

         end do

      end if  ! ( format_matlab==0 )

1000  call erclos ( 'prmatlabmatc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmatlabmatc'

      end if  ! ( debug )

      end
