      subroutine prmatlabmatrix ( matr, filename )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 21-04-2015 New definition intmat
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 30-04-2008
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print a sepran matrix in a form suitable for reading
!     in matlab
!     reading in matlab may be done in the following way:
!     sepget make_matrix.m
!     give the command make_matrix in the matlab command window
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     real
!     print
!     matlab
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: matr(*)
      character (len=*) filename

!     filename       i    Name of file to which the matlab output must be
!                         written
!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRMAT2MATLAB   Print a sepran vector in a form suitable for reading
!                    in matlab
!     SEPFILLMATSTR  Fill parameters in module sepmodulematstr
! **********************************************************************
!
!                       I/O
!
!     The matrix is written to the file with name filename
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmatlabmatrix' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmatlabmatrix'
         call prinin ( matr, 5, 'matr' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepfillmatstr

      if ( debug ) &
         write(irefwr,1) 'jmethod, iprob, nrusol', jmethod, iprob, nrusol

      S_ii => realmt(totmat)%S_ii

      call prmat2matlab ( jmethod, nrusol, S_ii, &
                          irowsii, filename )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmatlabmatrix'

      end if  ! ( debug )
      call erclos ( 'prmatlabmatrix' )

      end subroutine prmatlabmatrix
