      subroutine prsetcells2d ( set, nset, ndim, ishift, nx, indi, &
                                lenset )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-09-2012 Extra parameter lenset
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     make a new set of cells, given the index of the center cell
!     and the shift (2d)
! **********************************************************************
!
!                       KEYWORDS
!
!     cell
!     neighbour
!     interface
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, lenset, set(ndim,lenset), nset, ishift, nx(ndim), &
              indi(ndim)

!     indi           i    contains ijk-index of cell in which point is lying
!     ishift         i    Shift with respect to center cell
!     lenset         i    Declared length of array set
!     ndim           i    Dimension of the space.
!     nset           o    Number of elements in set
!     nx             i    number of points in ndim directions of the grid
!     set            o    contains the ijk innidces of the set of cells
!                         we consider
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsetcells2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsetcells2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      i = indi(1)-ishift
      if ( i>=1 ) then

!     --- lower row

         do j = max(1,indi(2)-ishift), min(nx(2),indi(2)+ishift)
            nset = nset+1
            call inichkarlen ( nset, lenset, 'set' )
            if ( ierror/=0 ) go to 1000
            set(1,nset) = i
            set(2,nset) = j
         end do  ! j = max(1,indi(2)-ishift), min(nx,indi(2)+ishift)

      end if  ! ( i>=1 )

      i = indi(1)+ishift
      if ( i<=nx(1) ) then

!     --- upper row

         do j = max(1,indi(2)-ishift), min(nx(2),indi(2)+ishift)
            nset = nset+1
            call inichkarlen ( nset, lenset, 'set' )
            if ( ierror/=0 ) go to 1000
            set(1,nset) = i
            set(2,nset) = j
         end do  ! j = max(1,indi(2)-ishift), min(nx,indi(2)+ishift)

      end if  ! ( i<=nx(1) )

      j = indi(2)-ishift
      if ( j>=1 ) then

!     --- left_hand side row

         do i = max(1,indi(1)-ishift+1), min(nx(1),indi(1)+ishift-1)
            nset = nset+1
            call inichkarlen ( nset, lenset, 'set' )
            if ( ierror/=0 ) go to 1000
            set(1,nset) = i
            set(2,nset) = j
         end do  ! i = max(1,indi(1)-ishift+1), min(nx(1),indi(1)+ishift-1)

      end if  ! ( j>=1 )

      j = indi(2)+ishift
      if ( j<=nx(2) ) then

!     --- right_hand side row

         do i = max(1,indi(1)-ishift+1), min(nx(1),indi(1)+ishift-1)
            nset = nset+1
            call inichkarlen ( nset, lenset, 'set' )
            if ( ierror/=0 ) go to 1000
            set(1,nset) = i
            set(2,nset) = j
         end do  ! i = max(1,indi(1)-ishift+1), min(nx(1),indi(1)+ishift-1)

      end if  ! ( j<=nx(2) )

1000  call erclos ( 'prsetcells2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsetcells2d'

      end if  ! ( debug )

      end
