      function prtimenthal3eu1 ( gradf, temperature, npoint, &
                                 TOLMIN, f )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 19-10-2005
!
!   copyright (c) 2003-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine whether gradf==0
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint
      double precision gradf(npoint), temperature(npoint), f, TOLMIN
      logical prtimenthal3eu1

!     f              i    0.5d0*|F|^2
!     gradf          i    Gradient of f
!     npoint         i    Number of points
!     prtimenthal3eu1 o    True if gradf ~= 0
!     temperature    i    Temperature vector
!     tolmin         i    Tolerance for measuring gradf==0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision temp, test, den

!     den            Help variable
!     i              Counting variable
!     temp           Help variable
!     test           Help variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimenthal3eu1' )

      test = 0d0
      den  = max(f, 0.5d0*npoint)

      do i = 1, npoint

         temp = abs(gradf(i))*max(abs(temperature(i)),1d0)/den
         if (temp>test) test = temp

      end do

      prtimenthal3eu1 = test<TOLMIN

      call erclos ( 'prtimenthal3eu1' )

      end
