      subroutine prtimpresscor4 ( iq, iseqnr, iproblc, isolp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.1  date 04-10-2010 New call to erpres
!        version  1.0  date 02-12-2005
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store the difference between the present pressure boundary conditions
!     and the previous ones into q
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     navier_stokes_equation
!     pressure_correction
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iq(*), iseqnr, iproblc, isolp(*)

!     iproblc        i    Actual problem number
!     iq             o    Contains information of the update vector q
!                         which is the pressure correction vector
!                         q is filled with the essetnial boundary conditions at
!                         the new time level minus the prior time level
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     isolp          i    Contains information about the prior pressure vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpsep(3), maxveclc, numveclc, iinvec(2), istart
      double precision rinvec(2)

!     iinvec         User input array for subroutine MANVEC
!                    For a description see subroutine MANVEC
!     inpsep         The integer input array for subroutine ERPRES
!     istart         First position used at a part
!     maxveclc         Maximum number of vectors per time-step permitted
!     numveclc         Number of vectors stored in isol
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERPRES         Read essential boundary conditions
!     SEPMANVEC      General subroutine for manipulations with SEPRAN arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill essential boundary conditions in array q
!     Subtract the boundary condition at the previous time-level
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimpresscor4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimpresscor4'

      end if
      if ( ierror/=0 ) go to 1000
      iprob = iproblc
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc

!     --- Jump in case nbound = 0
!         That implies no essential boundary conditions

      if ( kprobloc(10)==0 ) go to 1000

!     --- Fill essential boundary conditions in array q

      inpsep(1) = iseqnr    ! sequence number of input
      inpsep(2) = iprob     ! problem sequence number
      inpsep(3) = 1         ! vector sequence number
      maxveclc = 1            ! number of vectors in iq
      numveclc = 1            ! maximum number of vectors in iq
      istart = 1            ! dummy
      call erpres ( iq, inpsep, maxveclc, numveclc, istart, rinvec )

!     --- Subtract the boundary condition at the previous time-level
!         Since the values outside the boundary conditions are not important
!         we use the complete arrays

      iinvec(1) = 2
      iinvec(2) = 27    ! linear combination
      rinvec(1) = 1d0   ! 1 * iq
      rinvec(2) = -1d0  ! -1 * isolp
      call sepmanvec ( iinvec, rinvec, iq, isolp, iq )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtimpresscor4'

      end if
      call erclos ( 'prtimpresscor4' )

      end
