      subroutine sepcompreac ( isol, matr, iseqsol, iseqreac, irhsdlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute reaction force
!     Old subroutine prcompreac
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulemain
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulevecs
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), matr, iseqsol, iseqreac, irhsdlc

!     irhsdlc        i    Standard SEPRAN array, containing information of the
!                         large vector
!     iseqreac       i    Sequence number of reaction vector in array ISOL
!                         If <0 also the right-hand side contribution is added
!     iseqsol        i    Sequence number of solution array in array ISOL
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: solutvec(:)!, reaction(:)
      integer :: iseqlc

!     iseqlc         Sequence number of array
!     reaction       Refers to the reaction vector
!     solutvec       Refers to the solution vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     SEPCOMPREAC1   Actual computing of reaction force
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPPARALLOC    Allocate some global arrays for parallel computing
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     610   array isol corresponds to a complex array
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcompreac' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcompreac'
         write(irefwr,1) 'iseqsol, iseqreac', iseqsol, iseqreac
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprob = intmt(intmat)%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      npltra = kprobloc(13)
      nusol  = kprobloc(5)
      nbound = kprobloc(10)

      if ( nbound==0 ) go to 1000

      indprh = kprobloc(50)
      nrusol = kprobloc(66)
      kprobhinv => kprobpart%kprobhinv
      if ( parallel ) call sepparalloc

      iseqlc = abs(iseqreac)
      call sepcreasol110 ( isol(iseqlc), 'reac', ks(isol(iseqsol))%compl )
      reac => ks(isol(iseqlc))%sol
      solutvec => ks(isol(iseqsol))%sol
      if ( iseqreac>0 ) rhsd2 => ks(irhsdlc)%sol

      if ( ks(isol(iseqsol))%compl==1 ) then

!     --- Complex vectors are not yet permitted

         call errsub ( 610, 0, 0, 0 )
         go to 1000

      end if  ! ( ks(isol(iseqsol))%compl==1 )

!     --- Compute reaction force

      Smat => realmt(matr)
      call sepcompreac1 ( solutvec, reac, rhsd2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcompreac'

      end if  ! ( debug )
      call erclos ( 'sepcompreac' )

      end subroutine sepcompreac
