      subroutine sepcomptimcof ( ndim, numcop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Allocate some arrays for module sepmoduletime depending on
!     the parameter ireadcftim
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
     integer, intent(in) :: ndim, numcop

!     ndim           i    Dimension of the space.
!     numcop         i    number of coupled equations
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcomptimcof' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcomptimcof'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( .not. associated(ireadcftim) ) then

         allocate ( ireadcftim(1:numcop), stat=error )
         if ( error/=0 ) call eralloc ( error, numcop, 'ireadcftim' )
         if ( ierror/=0 ) go to 1000
         ireadcftim = 0

         if ( nhist>0 ) allocate ( coorhist(ndim,nhist), stat=error )
         if ( error/=0 ) call eralloc ( error, nhist*ndim, 'coorhist' )
         if ( ierror/=0 ) go to 1000

         allocate ( Teq(numcop), stat=error )
         if ( error/=0 )  call eralloc ( error, numcop, 'timeequation' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( .not. associated(ireadcftim) )

1000  call erclos ( 'sepcomptimcof' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcomptimcof'

      end if  ! ( debug )

      end subroutine sepcomptimcof
