      subroutine sepconelgrps ( congrp, ncongrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine what element groups are coupled to element groups.
!     Old subroutine prpjgcon
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: congrp(nelgrp,nelgrp), ncongrp(nelgrp+1)
      
!     congrp         o    See subroutine sepfillkmeshv
!     ncongrp        o    See subroutine sepfillkmeshv
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: check(nelgrp), jglist(nelgrp), &
                 ielgrp, inodp, ielm, i, j, njglist
      integer, allocatable :: gpelm(:)

!     check          Integer array containing information which element groups
!                    the current nodal point lies in.
!                    CHECK(ielgrp) = 1 means that the current point lies in
!                    element group IELGRP.
!     gpelm          Integer array containing information on element group
!                    numbers for each element.
!                    GPELM( IELM ) = element group element IELM lies in.
!     i              Counting variable
!     ielgrp         element group number
!     ielm           Positin in gpelm
!     inodp          Nodal point number
!     j              Counting variable
!     jglist         Integer array containing information which element groups
!                    the current nodal point lies in,
!                    jGLIST( 1 .. nJGLIST ) = element groups which the
!                    current nodal point lies in
!     njglist        Number of element groups the current nodal point lies in
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ2        Print an integer two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepconelgrps' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepconelgrps'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( gpelm(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'gpelm' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array gpelm with element groups element is lying in

      ielm = 0

      do ielgrp = 1, nelgrp

!     --- there are inelem elements in the current element group

         inelem = sepgetinelem(ielgrp)
         gpelm(ielm+1:ielm+inelem) = ielgrp
         ielm = ielm + inelem
         
      end do

!     --- initialize the array with connections between groups to zero

      ncongrp = 0
      congrp = 0

!     --- Determination of element group per node with KMESHD
!         Determination of the connections between element groups
!         Part I: determination of the length of array GRPCON.

      do inodp = 1, npoint

!     --- initialize the array CHECK to zero:
!         for this point no element groups are connected yet

         check = 0
         njglist = 0

!        --- Loop over all elements which point iNODP lies in

         do j = kmshd1(inodp)+1, kmshd1(inodp+1)
            ielm = kmshd2(j)
            ielgrp = gpelm(ielm)
            if ( check(ielgrp)==0 ) then

!           --- A new element group has been found

               check(ielgrp) = 1
               do i = 1, njglist
                  if ( congrp(ielgrp,jglist(i))==0 ) then

!                 --- Connection between element does not exist and
!                     will be stored in CONGRP

                     congrp(ielgrp, jglist(i)) = 1
                     ncongrp(ielgrp+1) = ncongrp(ielgrp+1) + 1
                     congrp(jglist(i), ielgrp ) = 1
                     ncongrp(jglist(i)+1) = &
                     ncongrp(jglist(i)+1) + 1
                  end if
               end do
               njglist = njglist + 1
               jglist(njglist ) = ielgrp  ! list of element groups for node
            end if
         end do
      end do
      
      if ( debug ) call prinin2 ( congrp, nelgrp, nelgrp, 'congrp' )

!     --- Transforming nCONGRP into an index array...

      do ielgrp = 1, nelgrp
         ncongrp(ielgrp+1) = ncongrp(ielgrp+1) + ncongrp(ielgrp)
      end do

      deallocate ( gpelm, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'gpelm' )

1000  call erclos ( 'sepconelgrps' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepconelgrps'

      end if  ! ( debug )

      end subroutine sepconelgrps
