      subroutine sepcoptiminp ( iintim, rintim, ndim, ivector )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy input for time solution from arrays iintim and rintim into
!     module sepmoduletime
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     copy
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iintim(*), ndim, ivector
      double precision, intent(in) :: rintim(*)

!     iintim         i    Contains the input for time integration
!                         See suroutine readtm
!     ivector        i    Sequence number of first vector
!     ndim           i    Dimension of space
!     rintim         i    Contains the real input for time integration
!                         See suroutine readtm
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: idim, i, ipstarteq, ipstartgen, ipstartnewmark, &
                 ipstarttend, ipstarttheta, ipstarttimehis, ipstarttstep, &
                 istart, numcop
      integer :: ipireadcftim

!     i              Loop variable
!     idim           Loop variable
!     ipireadcftim   Starting address information concerning coefficients
!     ipstarteq      Starting address information concerning equations
!     ipstartgen     Starting address information concerning gen alpha
!     ipstartnewmark Starting address information concerning newmark
!     ipstarttend    Starting address information concerning tend
!     ipstarttheta   Starting address information concerning theta
!     ipstarttimehis Starting address information concerning time his
!     ipstarttstep   Starting address information concerning tstep
!     istart         Starting address information concerning extra computations
!     numcop         maximal number of coupled equations
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: getparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETPARM        Return with real defined by inpsep/rinsep
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOMPTIMCOF  Allocate some arrays for module sepmoduletime depending on
!                    the parameter ireadcftim
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcoptiminp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcoptiminp'
         call prinin ( iintim, 70, 'iintim' )
         call prinrl ( rintim, 3, 'rintim' )
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      keeptim = iintim(24)

      ichtim = iintim(2)
      lineartim = iintim(3)
      ifilltim = iintim(4)
      ncopeqtim = max(1,iintim(5))
      itimcf = iintim(8)
      numoldtim = max(1,iintim(15))
      nfrac = iintim(17)
      ndt = iintim(18)
      icall_outtim = iintim(19)
      iseqouttim = iintim(20)
      ireturtim = iintim(21)
      iprinttim = iintim(22)

      usectimen = iintim(23)>0

      nhist = iintim(25)
      minitertim = iintim(26)
      maxoldtim = iintim(27)
      maxitertim = max(1,iintim(28))

      stationarysol = iintim(29)>0
      nocomputationtim = iintim(30)>0
      ipireadcftim = iintim(37)
      ipstarteq = iintim(38)

      epstim = getparm ( iintim, rintim, 41, 'epstime' )

      if ( debug ) write(irefwr,1) 'keeptim', keeptim

      if ( keeptim<2 ) then

!      --- Not reusetime

         t0 = getparm ( iintim, rintim, 42, 'tinit' )
         toutinit = getparm ( iintim, rintim, 43, 'toutinit' )
         if ( toutinit<t0 ) toutinit = t0
         toutend = getparm ( iintim, rintim, 44, 'toutend' )
         toutstep = getparm ( iintim, rintim, 45, 'toutstep' )

         ipstarttend = iintim(51)
         ipstarttstep = iintim(52)
         ipstarttheta = iintim(53)

         if ( debug ) &
            write(irefwr,1) 'ipstarttend, ipstarttstep, ipstarttheta, ndt', &
                             ipstarttend, ipstarttstep, ipstarttheta, ndt

!        --- Fill end times

         if ( ipstarttend>0 ) then

!        --- end times found

            do i = 1, ndt
               arrendtime(i) = getparm ( iintim, rintim, ipstarttend+i-1, &
                                         'tend' )
            end do  ! i = 1, ndt
            tend = arrendtime(ndt)
            if ( debug ) write(irefwr,2) 'tend', tend

         end if  ! ( ipstarttend>0 )

!        --- Fill time steps

         if ( ipstarttstep>0 ) then

!        --- time steps found

            do i = 1, ndt
               arrtimestep(i) = getparm ( iintim, rintim, ipstarttstep+i-1, &
                                          'tstep' )
            end do  ! i = 1, ndt

         end if  ! ( ipstarttstep>0 )

!        --- thetas

         if ( ipstarttheta>0 ) then

!        --- theta found

            do i = 1, nfrac
               arrtheta(i) = getparm ( iintim, rintim, ipstarttheta+i-1, &
                                       'theta' )
            end do  ! i = 1, ndt

         end if  ! ( ipstarttheta>0 )

         iseqfrtim = max(1,ivector)

         if ( nocomputationtim ) go to 1000

      end if  ! ( keeptim<2 )

      absaccurtim = getparm ( iintim, rintim, 46, 'absepstime' )
      relaccurtim = getparm ( iintim, rintim, 47, 'relepstime' )
      tthreshold = getparm ( iintim, rintim, 48, 'threshold_time' )

      if ( ichtim==14 .or. ichtim==15 ) then
         numcop = 4
      else
         numcop = ncopeqtim+maxcalltim
      end if  ! ( ichtim==14 .or. ichtim==15 )

!     --- Allocate some arrays for module sepmoduletime depending on
!         the parameter ireadcftim

      call sepcomptimcof ( ndim, numcop )

      itimbc = iintim(6)
      ireadbctim = iintim(7)
      Teq(:)%iconsstiff = iintim(10)
      Teq(:)%iconsmass = iintim(11)
      Teq(:)%iconsrhs = iintim(12)
      Teq(:)%ireadsl = iintim(13)
      Teq(:)%imas = iintim(14)
      Teq(:)%isubdiv = iintim(16)
      Teq(:)%iseqreac = iintim(32)
      iseqreactim = iintim(32)
      Teq(:)%iseqveloc = iintim(33)
      Teq(:)%iseqaccel = iintim(34)
      Teq(:)%iseqmt = iintim(35)
      Teq(:)%iseqaddrhs = iintim(36)
      Teq(:)%lenextra = 0
      Teq(:)%limit = 0

      Teq(:)%imasstot = 1
      if ( Teq(1)%iconsstiff/=1 .or. Teq(1)%iconsmass/=1 ) &
         Teq(:)%imasstot = 0

      do i = 1, ncopeqtim
         Teq(i)%iseqfr = ivector+i-1
      end do  ! i = 2, ncopeqtim
      iseqfrtim = Teq(1)%iseqfr

      if ( ichtim==14 .or. ichtim==15 ) then
         Teq(2)%iseqfr = iseqfrtim+1
         Teq(3)%iseqfr = iseqfrtim+2
         Teq(4)%iseqfr = iseqfrtim+1
      end if  ! ( ichtim==14 .or. ichtim==15 )

!     --- Fill coefficients

      ireadcftim(1:ncopeqtim) = iintim(ipireadcftim:ipireadcftim+ncopeqtim-1)

      if ( ichtim==5 .or. ichtim==11 .or. ichtim==14 .or. ichtim==15 ) then
         theta = 1d0
      else if ( ichtim==6 ) then
         theta = 0.5d0
      else
         theta = arrtheta(1)
         if ( ichtim/=7 .and. theta<=0d0 ) theta = 1d0
      end if

      if ( ichtim==14 ) then

!     --- Newmark method

         ipstartnewmark = iintim(54)
         beta_g = getparm ( iintim, rintim, ipstartnewmark, 'beta_newmark' )
         gamma_g = getparm ( iintim, rintim, ipstartnewmark+1, 'gamma_newmark' )
         alpha_f = 0d0
         alpha_m = 0d0

      else if ( ichtim==15 ) then

!     --- generalized alpha method

         ipstartgen = iintim(55)
         beta_g = getparm ( iintim, rintim, ipstartgen, 'beta_g' )
         gamma_g = getparm ( iintim, rintim, ipstartgen+1, 'gamma_g' )
         alpha_f = getparm ( iintim, rintim, ipstartgen+2, 'alpha_f' )
         alpha_m = getparm ( iintim, rintim, ipstartgen+3, 'alpha_m' )

      end if  ! ( ichtim==14 )

      if ( nhist>0 ) then

!     --- Time history

         ipstarttimehis = iintim(56)
         do i = 1, nhist
            do idim = 1, ndim
               coorhist(idim,i) = getparm ( iintim, rintim, ipstarttimehis, &
                                            'xxxx' )
               ipstarttimehis = ipstarttimehis+1
            end do  ! idim = 1, ndim
         end do  ! i = 1, nhist

      end if  ! ( nhist>0 )

      if ( iintim(38)>0 ) then

!     --- information of equations found

         length = iintim(38)
         do i = 1, ncopeqtim
            if ( iintim(length)>0 ) Teq(i)%iconsstiff = iintim(length)
            if ( iintim(length+1)>0 ) Teq(i)%iconsmass = iintim(length+1)
            if ( iintim(length+2)>0 ) Teq(i)%iconsrhs = iintim(length+2)
            if ( iintim(length+3)>0 ) Teq(i)%ireadsl = iintim(length+3)
            if ( iintim(length+5)>0 ) Teq(i)%isubdiv = iintim(length+5)
            if ( iintim(length+6)>0 ) Teq(i)%imas = iintim(length+6)
            if ( iintim(length+7)>0 ) Teq(i)%iseqfr = iintim(length+7)
            if ( iintim(length+10)>0 ) Teq(i)%limit = iintim(length+10)
            if ( iintim(length+12)>0 ) Teq(i)%iseqreac = iintim(length+12)
            if ( iintim(length+13)>0 ) then
               Teq(i)%iseqveloc = iintim(length+13)
               if ( ichtim==14 .or. ichtim==15 ) &
                  Teq(2)%iseqfr = iintim(length+13)
            end if  ! ( iintim(length+13)>0 )
            if ( iintim(length+14)>0 ) then
               Teq(i)%iseqaccel = iintim(length+14)
               if ( ichtim==14 .or. ichtim==15 ) &
                  Teq(3)%iseqfr = iintim(length+14)
            end if  ! ( iintim(length+14)>0 )
            if ( iintim(length+15)>0 ) Teq(i)%iseqmt = iintim(length+15)
            if ( iintim(length+16)>0 ) Teq(i)%iseqaddrhs = iintim(length+16)
            istart = iintim(length+9)
            if ( istart>0 ) then
               if ( iintim(istart)>0 ) then
                  Teq(i)%lenextra = 6
                  Teq(i)%extratim(1:6) = iintim(istart:istart+5)
               end if  ! ( iintim(istart)>0 )
            end if  ! ( istart>0 )
            length = length+25
         end do  ! i = 1, ncopeqtim
         iseqfrtim = Teq(1)%iseqfr

      end if  ! ( iintim(38)>0 )
      icalltim = max(1,icalltim)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numoldtim', numoldtim
         write(irefwr,*) 'End sepcoptiminp'

      end if  ! ( debug )
      call erclos ( 'sepcoptiminp' )

      end subroutine sepcoptiminp
