      subroutine seppjshap3 ( lenrtspj )
! ======================================================================
!
!        programmer    Gjalt Wijma
!        version  1.0  date 14-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine determines the shape of the projection vectors.
!     ( where the projection vectors are zero, one, or a equal to a partial
!        solution of the original P.D.E. )
!     Old subroutine prpjshap3
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleproj
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lenrtspj

!     lenrtspj    o  Total length of the temporary array RTSPROJV containing
!                    the solutions in the shale sets.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iset, iproj, nstpoint

!     iproj       Dummy variable, number of a projection vector
!     iset        Dummy variable, set number.
!     nstpoint    Number of points in a set
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: Determining for which sets the projection vectors are
!           one and for which shale sets subsolutions have to be
!           calculated.
!
!     Note that the temporary array RTSPROJV only contains the
!     sub-solutions, whereas the final array PROJVC also contains
!     the constant parts. For this reason both arrays are different
!     in length.
!
!     Semi PSD: prpjSHAP
!
!     We start with checking every projection vector:
!
!     Loop over all projection vectors:
!     DO iPROJ = 1, nPROJ
!        Determine the sand set, iSAND, where projection vector
!        iPROJ is one from array PROJSAND. Set a flag in temporary
!        array ITSPROJV that the projection vector iPROJ is
!        constant one in sand set iSAND.
!
!        The estimated length of the solution array with all
!        projection vectors is increased with the number of
!        degrees of freedom in the current sand set.
!
!        Next we check all shale neighbors to find out where
!        sub-systems have to be solved:
!        LOOP over all shale neighbors
!           IF the shale neighbor is an inclusion with no Dirichlet
!           conditions THEN
!              The projection vector iPROJ is constant one in the
!              shale neighbor. A flag is set in array ITSPROJV.
!
!              The estimated length of the solution array with all
!              projection vectors is increased with the number of
!              degrees of freedom in the current sand set.
!           ELSE
!              A sub-solution has to be calculated for this shale
!              set. This information is stored in array SOLSHALE.
!
!              The number of solutions for this shale set is
!              increased with one:
!              NSOL( iSHALE ) := NSOL( iSHALE ) + 1
!
!              The length of the temporary array RTSPROJV is
!              increased with the number of degrees of freedom in
!              this shale set.
!           END IF
!        END LOOP
!     END DO
!
!     Next we check some things for each shale set:
!     LOOP over all shale sets
!        IF the number of subsolutions is not equal to the number
!        of sand neighbors OR if the shale set has a Dirichlet
!        condition THEN
!           NSOL( iSHALE ) gets a minus sign.
!        END IF
!        The minus sign is used to indicate that this shale set
!        has either at least one sand neighbor with a Dirichlet
!        condition or the shale set has a Dirichlet condition
!        itself.
!     END LOOP
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( print_lv.ge.3 ) then
         write(irefwr,*) ' '
         write(irefwr,*) '   Number of projection vectors: ', nproj
         write(irefwr,*) ' '
      end if

!     --- Clear arrays NSOL and ITSPROJV

      do iset = 1, nset
         nsol( iset ) = 0
         do iproj = 1, nproj
            itsprojv( iproj, iset ) = 0
         end do
      end do

!     --- There are no shale sets to be solved yet:

      nshale = 0

!     --- The total length of the array with all solutions, RTSPROJV, &
!         is set to zero.

      lenrtspj = 0

!     --- The lenght of the array with the projection vectors, &
!         PROJVC, is set to zero

      nnproj = 0

!     --- loop over all projection vectors

      do iproj = 1, nproj

!     --- isand = projsand( iproj )

         iset=iproj
         nstpoint = nsetnode( iset + 1 ) - nsetnode( iset )
         nnproj = nnproj + nstpoint

!        --- The value in this sand layer is equal to one

         itsprojv( iproj, iset ) = 1

        end do
        nnproj=nrusol*nproj

!       --- Check if the number of sand neighbours is equal to the number of
!           solutions for each shale set. And check if the shale set has a
!           Dirichlet condition.

      if ( print_lv.ge.3 ) then

         write(irefwr,*) ' '
         do iproj=1,nproj

            write(irefwr,*) '   projection vector: ', iproj
            iset = iproj
            nstpoint = nsetnode( iset + 1 ) - nsetnode( iset )
            write(irefwr,*) '   aantal onbekenden: ',nstpoint
            write(irefwr,*) '   [ set : type of solution ]'
            do iset =1,nset
               if ( itsprojv( iproj, iset ).eq. 1 ) then
                  write(irefwr,*) '   ', iset, ' :  constant one'
               else
                  write(irefwr,*) '   ', iset, ' :  undefined yet'
               end if
            end do
            write(irefwr,*) ' '
         end do
      end if
      end
