      subroutine sepscalmatprbc ( A, diag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-07-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Rescale the projection vectors with the inverse of the diagonal
!     of the preconditioning matrix.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleproj
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      double precision, intent(inout) :: diag(A%neq)

!     A             i/o   Matrix A
!     diag          i/o   Diagonal of the scaling matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numneigh, i, j, iproj, idegfd

!     i              Counting variable
!     idegfd         Degree of freedom sequence number
!     iproj          Number of a projection vector
!     j              Counting variable
!     numneigh       number of neighbors of a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepscalmatprbc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepscalmatprbc'
         write(irefwr,1) 'A%scaling, A%neq', A%scaling, A%neq
         write(irefwr,*) 'parallel ', parallel
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%scaling>=2 ) then

!     --- column or symmetric scaling, pre-multiply vector z

         if ( parallel ) then

!        --- Scale the projection vector in the present block

            numneigh = blockunkinfo(1)
            A%projU(1:A%neq) = A%projU(1:A%neq) / diag

!           --- Scale the projection vectors from neighboring
!               blocks

            do iproj = 1, numneigh
               do j = A%projrows(iproj)+1, A%projrows(iproj+1)
                  idegfd = A%projcols(j)
                  A%projU(A%neq+j) = A%projU(A%neq+j) / diag(idegfd)
               end do
            end do

         else

!        --- Scale the projection vector

            do i = 1, A%nproj
               do j = A%projrows(i)+1, A%projrows(i+1)
                  idegfd = A%projcols(j)
                  A%projU(j) = A%projU(j) / diag(idegfd)
               end do  ! j = A%projrows(i)+1, A%projrows(i+1)
            end do  ! i = 1, A%nproj

         end if  ! ( parallel )

      end if  ! ( A%scaling>=2 )

      if ( A%scaling<=2 ) then

!     --- row or symmetric scaling, pre-multiply vector Az

         if ( parallel ) then

!        --- Scale the projection vector Az in the present block

            A%projAu(1:A%neq) = A%projAu(1:A%neq)*diag

!           --- Scale the projection vectors Az from neighboring
!               blocks

            do iproj = 1, numneigh
               do j = A%projrowsa(iproj)+1, A%projrowsa(iproj+1)
                  idegfd = A%projcolsa(j)
                  A%projAu(A%neq+j) = A%projAu(A%neq+j)*diag(idegfd)
               end do
            end do

         else

!        --- Scale the projection vector

            do i = 1, A%nproj
               do j = A%projrows(i)+1, A%projrows(i+1)
                  idegfd = A%projcols(j)
                  A%projAu(j) = A%projAu(j)*diag(idegfd)
               end do  ! j = A%projrows(i)+1, A%projrows(i+1)
            end do  ! i = 1, A%nproj

         end if  ! ( parallel )

      end if  ! ( iscaling<=2 )

1000  call erclos ( 'sepscalmatprbc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepscalmatprbc'

      end if  ! ( debug )

      end subroutine sepscalmatprbc
