      subroutine septimeenth3e1 ( iinenthalpy, iter )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Build matrix and right-hand side for enthalpy equation
!     solved by Newtons method
!     Old subroutine prtimenthal3e1
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     vector
!     enthalpy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinenthalpy(*), iter

!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     iter           i    Iteration sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprob, ireadcf, iseqtemp

!     iprob          Actual problem number
!     ireadcf        Sequence number of input for coefficients
!     iseqtemp       Sequence number of temperature in solution vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPBUILD       Build matrix and right-hand side
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPPRINTMAT    Print matrix
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimeenth3e1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimeenth3e1'
         call prinin ( iinenthalpy, 7, 'iinenthalpy' )
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      iprob = intmt(intmat)%iprob  ! problem number
      ireadcf = iinenthalpy(7)  ! sequence number for reading coefficients
      iseqtemp = iinenthalpy(3) ! sequence number of temperature in isolut2d
      if ( debug ) &
         write(irefwr,100) 'iprob, ireadcf, iseqtemp', &
                            iprob, ireadcf, iseqtemp

!     --- Compute iuser1(iprob) and iuser2(iprob), indicating references to
!         start addresses
!         of iuser and user

      call erlins ( 3, iprob, ireadcf )
      iuser => iuser1(iprob)%iuser
      user => iuser1(iprob)%user

!     --- Build matrix G and right-hand side R_2
!         In the first iteration we have to build R_1 too

      iinbldtim(1) = 13    ! number of entries filled
      iinbldtim(2) = 1     ! build matrix and right-hand side
      iinbldtim(3) = 1     ! do not use essential boundary conditions
      iinbldtim(4) = 0     ! no mass matrix
      iinbldtim(5) = 1     ! not important
      iinbldtim(6) = 0     ! allelements (default)
      iinbldtim(7) = 0     ! addmatvec (default)
      iinbldtim(8) = 0     ! CLEARLASTRHSD (default)
      iinbldtim(9) = iprob ! problem definition number
      iinbldtim(10) = numoldtim  ! number of old solutions
      iinbldtim(11) = 0    ! ISPECTRAL (default)
      if ( iter==1 ) then
         iinbldtim(12) = 2    ! number of right-hand side vectors
      else
         iinbldtim(12) = 1    !
      end if  ! ( iter==1 )
      iinbldtim(13) = iseqtemp !  ISEQIN, sequence number of temperature
      if ( debug ) call prinin ( iinbldtim, 13, 'iinbldtim' )

      call sepfillsmf ( iinbldtim, irhsd, isolut2d, isolut2d )
      call sepbuild
      if ( debug ) then

!     --- Debug information

         call sepprintv ( irhsd(1), 'rhs_1' )
         call sepprintv ( irhsd(2), 'rhs_2' )

      end if  ! ( debug )

1000  if ( debug ) then

!     --- Debug information

         call sepprintv ( irhsd(1), 'rhside' )
         call sepprintmat ( stiffmat )
         write(irefwr,*) 'End septimeenth3e1'

      end if  ! ( debug )
      call erclos ( 'septimeenth3e1' )

      end
