      subroutine septimenthal2 ( first, finished, iseqenthalpy, &
                                 iinenthalpy, rinenthalpy, &
                                 solutionmethod, rewind, changestep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     time integration for enthalpy equation using quasi Newton linearization
!     The method of nedjar is applied
!     Old subroutine prtimenthal2
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqenthalpy, iinenthalpy(*), solutionmethod
      double precision, intent(in) :: rinenthalpy(*)
      logical, intent(inout) :: first, finished, rewind
      logical, intent(out) :: changestep

!     changestep     o    If true the time step may be changed
!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not (false)
!     first         i/o   If true it concerns the first time step
!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     iseqenthalpy   i    Sequence number of input for block enthalpy
!     rewind         i    When true, the current time level is rewinded
!                         to a previous time level
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     solutionmethod i    Defines type of solution method
!                          1: overrelaxation
!                          2: nedjar
!                          3: newton
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: joutp, jtime

!     joutp          Help variable to store the value of ioutp
!     jtime          Help variable to store the value of itime
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPTIMCHECK    Perform some checks with respect to time integration input
!                    Fill array iinbldtim
!     SEPTIMENTH2EU  time integration for enthalpy equation using newton
!                    a theta method is used
!     SEPTIMENTH3EU  Time integration of the enthalpy equation
!                    using Newton linearization
!     SEPTIMENTH4EU  Time integration of the enthalpy equation using
!                    Newton linearization
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimenthal2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimenthal2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- set ioutp and itime temporarily

      joutp = ioutp
      ioutp = min(0,ioutp)
      jtime = itime
      itime = 0
      changestep = .true.

!     --- Define parameters and fill arrays

      call  septimcheck ( first )
      if ( keeptim>1 ) t = t - tstep

!     --- Implicit theta method (Euler, Crank Nicolson, fractional step)
!         Stepsize is given by user.

      if ( solutionmethod==2 ) then

!     --- Solutionmethod = 2, nedjar

         call septimenth2eu ( finished, iseqenthalpy, iinenthalpy, &
                              rinenthalpy )

      else

!     --- Solutionmethod = 3, newton

         if ( maxitertim>1 ) then

!        --- maxiter>1, iteration

            call septimenth3eu ( iinenthalpy, rinenthalpy, &
                                 finished, rewind )
         else

!        --- maxiter = 1, no iteration

            call  septimenth4eu ( iinenthalpy, finished )
            changestep = .false.
            rewind = .false.

         end if   ! ( maxitertim>1 )

      end if  ! ( solutionmethod==2 )

!     --- reset ioutp and itime

      first = .false.
      ioutp = joutp
      itime = jtime

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimenthal2'

      end if
      call erclos ( 'septimenthal2' )

      end
