      subroutine septimtheta1 ( finishtm, first, linear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Start the time integration for suboutine septimtheta
!     Set parameters
!     Old subroutines pr110501 and pr110503
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(out) :: finishtm, first, linear

!     finishtm       o    Indicates if the time loop has been finished (true)
!                         or not (false
!     first          o    If true it concerns the first step
!     linear         o    If true the linear time-iteration is used, if false an
!                         iteration per time step is performed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2039   The combination stationary and non-linear iteration per time-step
!           has not been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimtheta1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimtheta1'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'ncopeqtim, numoldtim, maxitertim, ichtim', &
                          ncopeqtim, numoldtim, maxitertim, ichtim
         write(irefwr,*) 'stationarysol', stationarysol
         write(irefwr,1) 'lineartim', lineartim
         write(irefwr,2) 'tthreshold', tthreshold
      end if

      linear = lineartim<=1

      if ( stationarysol .and. .not.linear ) then

!     --- in the stationary case only linear time-stepping has been
!         implemented

         call errwar ( 2039, 0, 0, 0 )
         linear = .true.

      end if
      if ( t<=t0+0.1d0*tstep ) then

!     --- First time step
!         Reset arrays

         first = .true.

         diffsl(1:ncopeqtim,1:3) = 0d0

         nitertim = 0

      else

!     --- Next time step

         first = .false.

      end if

!     --- Set parameters

      ifrac = 1
      finishtm = .false.
      if ( ichtim==5 .or. ichtim==11 ) then
         theta = 1d0
      else if ( ichtim==6 ) then
         theta = 0.5d0
      else
         theta = theta
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'linear', linear
         write(irefwr,*) 'End septimtheta1'

      end if  ! ( debug )
      call erclos ( 'septimtheta1' )

      end subroutine septimtheta1
