      subroutine septimuser ( first, ieqn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.0  date 05-11-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays iuser and user for a specific
!     problem in a sequence of problems
!     Old subroutine prtm03
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleuser
      use sepmoduletime
      use sepmoduleinput
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ieqn
      logical, intent(in) :: first

!     first          i    If true it concerns the first time step
!     ieqn           i    Sequence number of equation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: isubr, lenius, lenusr, istart, maxcal, ipos, lastps
      integer, save :: ipiuser, ipuser

!     ipiuser        Starting address of array iuser
!     ipos           Position of input arrays in sepmoduleinput
!     ipuser         Starting address of IUSER
!     istart         First position used at a part
!     isubr          Indicates the type of calling subroutine
!                    Possible values
!                    1:  First call
!                    3:  Next call
!     lastps         Last position of subarray in iuser
!     lenius         Computed length of array IUSER
!     lenusr         Computed length of array USER
!     maxcal         Maximum number of items read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLDEFCOF  Fill default coefficients in iuser/user
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimuser' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimuser'
  1      format ( a, 1x, (10i6) )
         write(irefwr,1) 'itimcf, ieqn, maxcalltim, iseqcoef', &
                          itimcf, ieqn, maxcalltim, iseqcoef
         write(irefwr,*) 'first ', first

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( itimcf==0 ) then

!     --- itimcf = 0, the arrays iuser and user are used
!         First compute shift for arrays iuser and user

         if ( ieqn>1 ) then

!        --- ieqn>1 , next problem

            lastps = iuserlc(ipiusrlc+2)
            iuserlc(ipiusrlc+lastps) = iuserlc(ipiusrlc)-lastps
            ipiusrlc = ipiusrlc + lastps
            iuser => iuserlc(ipiusrlc:ipiusrlc+iuserlc(ipiusrlc)-1)
            if ( first ) then

!           --- first time step, clear positions 2 to 5 of new array iuserlc

               iuserlc(ipiusrlc+1:ipiusrlc+4) = 0

            end if

            lastps = nint(userlc(ipuserlc+2))
            userlc(ipuserlc+lastps) = userlc(ipuserlc)-lastps
            ipuserlc = ipuserlc + lastps
            user => userlc(ipuserlc:nint(userlc(ipuserlc))-1)
            if ( first ) then

!           --- first time step, clear positions 2 to 5 of new array userlc

               user(2:5) = 0

            end if

         else

            iuser => iuserlc(1:iuserlc(1))
            user => userlc(1:nint(userlc(1)))
            ipiusrlc = 1
            ipuserlc = 1

         end if  ! ( ieqn>1 )

      else

!     --- itimcf = 1, information is stored in the buffer array
!         Coefficients are not filled by filtim but by this subroutine

         iseqcoef = 0
         if ( first .or. maxcalltim>1 ) then

!        --- Reading of the coefficients has to be done only the
!            first time

            if ( ireadcftim(ieqn)>0 ) then
               isubr = 3
            else
               isubr = 1
            end if
            if ( debug ) then
               write(irefwr,1) 'iprobtim', iprobtim
               write(irefwr,*) 'associated(icofius(iprobtim)%iuser)', &
                                associated(icofius(iprobtim)%iuser)
            end if  ! ( debug )
            if ( associated(icofius(iprobtim)%iuser) ) then
               if ( associated(iuser1(iprobtim)%iuser) ) &
                  deallocate ( iuser1(iprobtim)%iuser, &
                               iuser1(iprobtim)%user )
               lenius = size(icofius(iprobtim)%iuser)
               lenusr = size(icofius(iprobtim)%user)
               allocate ( iuser1(iprobtim)%iuser(lenius), &
                          iuser1(iprobtim)%user(lenusr) )
               iuser1(iprobtim)%iuser = icofius(iprobtim)%iuser
               iuser1(iprobtim)%user = icofius(iprobtim)%user
            end if  ! ( associated(icofius(iprobtim)%iuser) )
            if ( debug ) &
              write(irefwr,1) 'ireadcftim/1', ireadcftim(ieqn)
            nitercom = -1  ! Force filling of iuser/user in each step
            iseqcoef = ireadcftim(ieqn)
            if ( debug ) &
               write(irefwr,1) 'iseqcoef, ieqn', iseqcoef, ieqn

            if ( iseqcoef==0 .and. ieqn==1 ) then

!           --- Check if there is an input block coefficients

               call sepgetinpfinf ( 6, 1, ipos, maxcal )
               if ( debug ) write(irefwr,1) 'maxcal, ipos', maxcal, ipos

               if ( in(ipos)%used .and. maxcal>0 ) then

!               --- There is an input block coefficients
!                   Set iseqcoef to 1 in order to call erlins

                  iseqcoef = 1
                  ireadcftim(ieqn) = 1
                  isubr = 3

               end if  ! ( maxcal==1 )

            end if  ! ( iseqcoef==0 .and. ieqn==1 )

            if ( debug ) &
               write(irefwr,1) 'iseqcoef, ieqn, isubr', &
                                iseqcoef, ieqn, isubr

            if ( iseqcoef==0 ) then

!            --- iseqcoef=0, hence no coefficients found
!                Define default values if possible

               iprob = iprobtim
               call sepfilldefcof

            else

               call erlins ( isubr, iprobtim, ireadcftim(ieqn) )

            end if  ! ( iseqcoef==0 )
            nitercom = 0  ! Reset filling of iuser/user

!           --- Copy values of iuser1(iprobtim) and iuser2(iprobtim) into
!               icofius and icofus
!               Reset iuser1(iprobtim) and iuser2(iprobtim)

            if ( associated(icofius(iprobtim)%iuser) ) &
               deallocate ( icofius(iprobtim)%iuser, &
                            icofius(iprobtim)%user )
            lenius =  size(iuser1(iprobtim)%iuser)
            lenusr =  size(iuser1(iprobtim)%user)
            allocate ( icofius(iprobtim)%iuser(lenius), &
                       icofius(iprobtim)%user(lenusr) )
            icofius(iprobtim)%iuser = iuser1(iprobtim)%iuser
            icofius(iprobtim)%user = iuser1(iprobtim)%user
            if ( ncopeqtim/=1 ) &
               nullify ( iuser1(iprobtim)%iuser, iuser1(iprobtim)%user )

         end if

!        --- Copy arrays IUSER and USER

         if ( associated ( icofius(iprobtim)%iuser ) ) then

!        --- icofius(iprobtim)>0, IUSER and USER are filled

            iuser => icofius(iprobtim)%iuser
            user => icofius(iprobtim)%user

         end if

      end if  ! ( itimcf==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimuser'

      end if  ! ( debug )
      call erclos ( 'septimuser' )

      end subroutine septimuser
