      subroutine toaddmatshiftc ( celemmt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-10-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add diagonal element matrix to extra diagonal of complex matrix
!     Used for shifted laplace preconditioner
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex, intent(in) :: celemmt(*)

!     celemmt        i    Diagonal (lumped) element matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, inusol
      double complex, pointer :: lapshiftd(:)

!                    they are not
!     i              Counting variable
!     inusol         Position of unknown in array USOL
!     lapshiftd      diagonal matrix for shifted laplace
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRINCOMPL      Print 1d complex vector
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toaddmatshiftc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toaddmatshiftc'
         call ertrace
         call prinin ( indexren, icount, 'indexren' )
         call princompl ( celemmt, icount, 'celemmt' )
         write(irefwr,1) 'nrusol', nrusol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lapshiftd => Smat%Sc_i%DLshift

      do i = 1, icount

         inusol=indexren(i)
         if ( inusol<=nrusol ) &
            lapshiftd(inusol) = lapshiftd(inusol)+celemmt(i)

      end do  ! i = 1, icount

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toaddmatshiftc'

      end if  ! ( debug )
      call erclos ( 'toaddmatshiftc' )

      end
