      subroutine toaddrhspar
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add parts of right-hand-side at interface to lower blocks
!
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     right_hand_side
!     real
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulevecs
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision, allocatable :: work(:), work1(:)

!                    they are not
!     i              Counting variable
!     work           Work array to be used to transport quantities between
!                    blocks
!     work1          Work array for reordening
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     TOADDRHSPAR1   Add parts of right-hand-side at interface to lower blocks
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toaddrhspar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toaddrhspar'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(nusol), work(2*nusol), &
                 kprobhglobinv(nusolglob), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nusol, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array kprobhglobinv with inverse of array kprobhglob

      kprobhglobinv = 0
      do i = 1, nrusol
         kprobhglobinv(kprobhglob(i)) = i
      end do  ! i = 1, nrusol

!     --- Send and receive information and add to rhsd

      call toaddrhspar1 ( work, rhsd )

      if ( imas==1 .and. typecoupl==1 ) then

!     --- Diagonal mass matrix and d-d coupling
!         First reorder massmat

         matrixm => realmt(massmat)%S_ii
         allocate ( work1(nusol))
         work1 = matrixm(kprobr(1:nusol))
         call toaddrhspar1 ( work, work1 )
         matrixm = work1(kprobhinv(1:nusol))

      end if  ! ( imas==1 .and. typecoupl==1 ) then

      deallocate ( iwork, work, kprobhglobinv, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toaddrhspar'

      end if  ! ( debug )
      call erclos ( 'toaddrhspar' )

      end subroutine toaddrhspar
