      subroutine toaddsif ( amatif, amatiout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add matrix amatif to matrix amatiout
!
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     real
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: amatif(*)
      double precision, intent(out) :: amatiout(*)

!     amatif         i    Matrix part if i.e. internal unknowns coupled
!                         to interface unknowns
!     amatiout       o    Matrix part i out i.e. part of rows with
!                         column numbers outside the present cpu
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ncols, istart, k, icol, iposout

!                    they are not
!     i              Counting variable
!     icol           Local column number
!     iposout        position in array amatiout
!     istart         First position used at a part
!     k              Counting variable
!     ncols          number of non-zero columns in row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toaddsif' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toaddsif'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over all free degrees of freedom

      istart = 0
      do i = 1, nrusol

         ncols = irowsif(i+1)-istart  ! number of non-zero columns in amatif

         if ( ncols>0 ) then

!        --- There are ncols non-zero columns in amatif
!            fill iwork with addresses in amatiout of all off-diagonal
!            elements of row i

            do k = irowsout(i)+1, irowsout(i+1)
               iwork(icolsout(k)) = k
            end do

!           --- Copy row of amatif into amatiout

            do k = 1, ncols
               icol = icolsif(istart+k)           ! local column number
               iposout = iwork(kprobhglob(icol))
               amatiout(iposout) = amatif(istart+k)
            end do  ! k = 1, ncols

            istart = istart+ncols

         end if  ! ( ncols>0 )

      end do  ! i = 1, nrusol

1000  call erclos ( 'toaddsif' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toaddsif'

      end if  ! ( debug )

      end subroutine toaddsif
