      subroutine toelmatvec
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-07-2015 Use sepmodulebuild
!        version  1.0  date 27-08-2005
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Multiply element matrix by part of solution vector and add to
!     right-hand side vector
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     vector
!     element_matrix
!     multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulebuild
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, jpos
      double precision  :: rhsdloc(icount), uloc(icount)

!                    they are not
!     i              Counting variable
!     jpos           Position of unknown in array
!     rhsdloc        Local elelemnt right-hand side vector
!                    Result of element matrix times solution
!     uloc           Local solution vector (just for element)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store part of solution vector in uloc
!     Multiply element matrix by uloc, store in rhsdloc
!     Add rhsdloc to rhsd (skip essential bc's)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toelmatvec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toelmatvec'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Store part of solution vector in uloc

      uloc = usol(index5(1:icount))

!     --- Multiply element matrix by uloc, store in rhsdloc
!         matmul can not be used since elemmt is stored as transposed matrix

      do i = 1, icount
         rhsdloc(i) = dot_product ( elemmt(1:icount,i), uloc )
      end do  ! i = 1, icount

!     --- Add rhsdloc to rhsd (skip essential bc's)

      do i = 1, icount
         jpos = index5(i)
         if ( jpos<=nrusol ) rhsd(jpos) = rhsd(jpos) + rhsdloc(i)
      end do  ! i = 1, icount

1000  call erclos ( 'toelmatvec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toelmatvec'

      end if

      end
