      subroutine chpspn ( ksetnp, mset, istmet, istval, linear, ifirst, ilast )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.6  date 22-05-2014 Extension with shape 22
!        version  4.5  date 10-03-2006 Extension with interface elements
!        version  4.4  date 30-03-2004 Extension with shape 134
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute starting level for renumbering algorithm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: mset, istmet, istval, ifirst, ilast
      integer, target :: ksetnp(npoint)
      logical linear

!     ifirst  i    The nodes from ifirst must be renumbered
!     ilast   i    The nodes until ilast must be renumbered
!     istmet  i    Indicates the method for computing the start for renumbering
!                  0:  default
!                  1:  user point
!                  2:  curve
!                  3:  surface
!                  4  Start point given
!     istval  i    Depending on istmet start value for user point, curve or
!                  surface
!     ksetnp  o    array containing the set of start nodal points
!     linear  o    If true all elements are linear
!     mset    o    Length of array ksetnp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istar3, inodp, inpelm, startnode, istore1(npoint)

!     i              Counting variable
!     inodp          Nodal point number
!     inpelm         number of points per element
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istore1        Work array
!     startnode      Starting node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer sepstartnode

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPSTARTNODE   Find start node in a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find starting set for renumbering
!     Sort starting set
!     Compute rest of levels
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'chpspn' )
      debug = .false.
      do i = ifirst, ilast
         marknodes(i)=0
         ksetnp(i)=i
      end do

!     --- Compute linear

      linear = .true.
      do i = 1, nelgrp

         ishape = kmeshf(i)
         if ( ishape==4 .or. ishape>=6 .and. ishape<10 .or. &
              ishape==12 .or. ishape>=14 .and. ishape<=16 .or. &
              ishape==22 ) then
            linear = .false.
         else if ( ishape==19 .or. ishape==20 .or. &
                   ishape>22 .and. &
                   ishape<50 .or. ishape>59  .and. &
                   ishape<71 .or. ishape>79  .and. &
                   ishape/=109 .and. ishape/=134 ) then

!        --- Shape number not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000
            linear = .false.

         end if

      end do
      if ( debug ) then

!     --- debug statements

         write(irefwr,*) ' in chpspn'
         write(irefwr,*) 'ifirst, ilast, linear,istmet ', &
                          ifirst, ilast, linear,istmet
      end if
      if ( istmet==0 .or. ifirst>1 ) then

!     --- Automatic start

         set => ksetnp(ifirst:ilast)
         startnode = sepstartnode()
         mset = 1
         ksetnp(1) = startnode

      else if ( istmet==1 ) then

!     --- Starting value is a user point

          mset = 1
          ksetnp(1) = kmeshl(istval)

      else if ( istmet==2 ) then

!     --- Starting value is a curve

         istar3 = kmeshm(2)-1
         ksetnp(1) = kmeshm(istval)
         istore1(1:ilast) = 0
         mset = 0
         do i = istar3+kmeshm(4+istval), istar3+kmeshm(5+istval)-1
            inodp = kmeshm(i)
            if ( istore1(inodp)==0 ) then
               istore1(inodp) = 1
               mset = mset + 1
               ksetnp(mset) = inodp
            end if
         end do

      else if ( istmet==3 ) then

!     --- Starting value is a surface

         istore1(1:ilast) = 0
         inpelm = kmeshn(istval)
         inelem = kmeshn(istval+nsurfs)
         istar3 = 2*nsurfs
         do i = 1, istval-1
            istar3 = istar3 + kmeshn(i)*kmeshn(i+nsurfs)
         end do
         mset = 0
         do i = 1+istar3, inpelm*inelem+istar3
            inodp = kmeshn(i)
            if ( istore1(inodp)==0 ) then
               istore1(inodp) = 1
               mset = mset + 1
               ksetnp(mset) = inodp
            end if
         end do

      else

!     --- Starting value is a node

          mset = 1
          ksetnp(1) = istval

      end if

1000  call erclos ( 'chpspn' )
      end
