      subroutine chtmc2 ( ksetnp, istore1, mset, nnodes, npkset, &
                          linear, ifirst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 27-03-2002 Correction in case more parts
!        version  4.0  date 22-11-1996 Extra parameter ifirst
!        version  3.0  date 22-12-1994 New parameter list
!
!   copyright (c) 1982-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    The starting level for the renumbering is found
!
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ksetnp(*), nnodes, mset, npkset, ifirst
      integer, target :: istore1(*)
      logical linear

!     ifirst  i    The nodes from ifirst must be renumbered
!     istore1  o    Contains the last level set
!     ksetnp  i    array containing the set of nodal points
!     linear  i    If true all elements are linear
!     mset    o    number of nodal points in istore1
!     nnodes  i    Number of nodal points
!     npkset  i    Number of elements in ksetnp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ithres, nposet, irep, mins, nbr, nset, msetpr, ilevel
      integer, target :: itemp(npoint)

!     i              Counting variable
!     ilevel         Sequence number of level
!     irep           Loop variable
!     itemp          integer work array
!                    Is used to store the next level
!     ithres         Integer threshold value
!     mins           Index in ksetnp of point with minimal number of neighbors
!     msetpr         Previous value of mset
!     nbr            number of neighbors of nodal point
!     nposet         length of set ksetnp
!     nset           Number of elements in a set of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHTMC1         Compute nodal point with minimum number of neighbors in
!                    set
!     CHTMC3         The starting level set is sorted using the Cuthill-Mckee
!                    algorithm
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPNEXTLEVEL   Find neighbors corresponding to a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Global description:
!    Find node with minimal number of neighbors in ksetnp
!    Find all neighboring levels of this node
!    Sort last set and restrict to level depth
!    Find all neighboring levels of this set
!    The last set is used as starting set
!    Detailed description:
!    nposet := npkset
!    Compute mins such that ksetnp(mins) has minimal number of
!    neighbors in ksetnp
!    nbr := number of neighbors of ksetnp(mins)
!    Set marknodes = 1 for all nodes with marknodes(i) > -10
!    nset := 1
!    istore1(1) := ksetnp(mins)
!    marknodes(istore1(1)) := -1
!    For irep := 1 (1) 2 do
!       ilevel := 0
!       While ( nset>0 ) do
!          mset_previous := nset
!          itemp := set of free neighbors of istore1
!          marknodes(itemp) := -1
!          nset := mset
!          istore1 := itemp
!          ilevel := ilevel + 1
!       If ( irep=1 ) then
!          Set marknodes = 0 for all nodes with marknodes(i) > -10
!          ksetnp := istore1
!          nset := mset_previous
!          if ( nposet > ilevel ) then
!             Sort ksetnp and restrict to at most ilevel positions
!             copy in istore1
!          Set marknodes for istore1
!    mset := mset_previous
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'chtmc2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from chtmc2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ithres = -1
      nposet = npkset

!     --- Compute mins such that ksetnp(mins) has minimal number of
!         neighbors in ksetnp

      call chtmc1 ( nposet, ksetnp(ifirst), mins, nbr )
      mins = mins+ifirst-1
      if ( debug ) then

!     --- debug statements

         write(irefwr,*) ' in chtmc2'
         call prinin ( ksetnp(ifirst), nposet, 'ksetnp' )
      end if

!     --- Set marknodes = 1 for all nodes with marknodes(i) > -10

      do i = ifirst, nnodes
         if ( marknodes(i)>-10 ) marknodes(i) = 1
      end do
      istore1(1) = ksetnp(mins)
      marknodes(istore1(1)) = ithres
      nset = 1
      if ( debug ) then

!     --- debug statements

         write(irefwr,*) ' mins, nbr, istore1(1) ', mins, nbr, istore1(1)
      end if
      do irep = 1, 2
         ilevel = 0

!        --- While loop nset > 0

150      if ( nset>0 ) then
            msetpr = nset
            if ( debug ) then

!           --- debug statements

               write(irefwr,*) 'msetpr, nset ', msetpr, nset
            end if

!           --- itemp := set of free neighbors of istore1
!               marknodes(itemp) := -1

            lastlevel => istore1(1:nset)
            nextlevel => itemp(1:npoint)
            call sepnextlevel ( ithres, mset )
            nset = mset
            do i = 1, mset
               istore1(i) = itemp(i)
            end do
            ilevel = ilevel + 1
            if ( debug ) then

!           --- debug statements

               write(irefwr,*) 'irep, mset, ilevel ', irep, mset, ilevel
               call prinin ( istore1, mset, 'istore1' )
            end if
            go to 150
         end if
         ilevel = ilevel**(ndim-1)
         if ( .not. linear ) ilevel = ilevel*2
         if ( irep==1 ) then

!        --- First step in loop, hence end level of first loop has been found
!            This will be used as start for next computation

            if ( msetpr>ilevel ) then

!           --- Number of nodes in last set is more than number of levels
!               Restrict ksetnp to at most ilevel nodes

               mset = -msetpr
               do i = ifirst, nnodes
                  if ( marknodes(i)>-10 ) marknodes(i) = -3
               end do
               call chtmc3 ( ksetnp, istore1, &
                             mset, nnodes, ifirst )
               nset = min ( mset, ilevel )
               do i = 1, mset
                  istore1(i) = ksetnp(i)
               end do
               if ( debug ) then

!              --- debug statements

                  call prinin ( istore1, mset, 'istore1' )
               end if
            else

!           --- Set nset

               nset = msetpr
            end if
            do i = ifirst, nnodes
               if ( marknodes(i)>-10 ) marknodes(i) = 1
            end do
            do i = 1, nset
               marknodes(istore1(i)) = ithres
            end do
         end if
      end do
      mset = msetpr

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End chtmc2'

      end if  ! ( debug )
      call erclos ( 'chtmc2' )

      end subroutine chtmc2
