      subroutine chtmc3 ( ksetnp, istore1, mset, nnodes, ifirst )
! ======================================================================
!
!        programmer     Guus Segal
!        version 4.0    date   22-11-1996 Extra parameter ifirst
!        version 3.3    date   22-12-1994 New layout
!        version 3.2    date   22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1984-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   The starting level set is sorted using the Cuthill-Mckee algorithm
!   in this way the first level set can be considered as a sorted line
! **********************************************************************
!
!                       KEYWORDS
!
!    renumbering
!    topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ksetnp(*), mset, nnodes, ifirst
      integer, target :: istore1(*)

!     ifirst  i    The nodes from ifirst must be renumbered
!     istore1  i    Contains the level set to be sorted
!     ksetnp  o    array containing the sorted set of nodal points
!     mset   i/o   At input the sign of mset indicates if more than one
!                  part in the set may be considered (>0) or not (<0)
!                  The absolute value contains the number of nodes in istore1
!                  At output it contains the number of nodal points in ksetnp
!     nnodes  i    Number of nodal points
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ithres, ihset, jcount, mins, nbr, nset, k
      logical onepar

!     i       Counting variable
!     ihset   Number of elements in istore1 at input
!     ithres  Integer threshold value
!     jcount  Counter
!     k       Counting variable
!     mins    Index in istore1 of point with minimal number of neighbors
!     nbr     number of neighbors of nodal point
!     nset    Number of elements in a set of points
!     onepar  If true only one part of the boundary may be considered
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHTMC1         Compute nodal point with minimum number of neighbors in set
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPNEXTLEVEL   Find neighbors corresponding to a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    jcount := 1
!    ksetnp(jcount) := istore1(1)
!    if ( mset>1 ) then
!       ksetnp(jcount) := point in istore1 with minimal number of neighbors
!       Set marknodes
!       istore1(1) := ksetnp(jcount)
!       While ( not all points have been used ) do
!          Store all free neighbors of istore1 in ksetnp
!          Raise jcount
!          Put new level in istore1
!          If ( last level empty ) then
!             if ( only one part permitted ) then
!                Set all points have been used: true
!             else ( more parts permitted )
!                Detect rest of the set from marknodes
!                Find node with minimal number of neighbors in this set
!                Store this node in ksetnp and adapt marknodes
!                Raise jcount
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'chtmc3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from chtmc3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( nextlevel(nnodes) )

      onepar = mset<0
      mset = abs(mset)
      ihset = mset
      jcount = 1
      ksetnp(jcount) = istore1(1)
      ithres = -2
      if ( mset>1 ) then

!     --- mset > 1
!         ksetnp(jcount) := point in istore1 with minimal number of neighbors

         call chtmc1 ( mset, istore1, mins, nbr )
         ksetnp(jcount) = istore1(mins)

         do 110 i = 1, mset
            marknodes(istore1(i)) = 0
110      continue
         marknodes(istore1(mins)) = ithres
         istore1(1) = istore1(mins)
         nset = 1

!        --- While ( not all points have been used ) do

200      if ( jcount<ihset ) then

!        --- jcount < ihset, hence not all nodes have been renumbered

            lastlevel => istore1(1:nset)
            call sepnextlevel ( ithres, mset )

!           --- Store all free neighbors of istore1 in ksetnp

            do 220 i = 1, mset
               ksetnp(jcount+i) = nextlevel(i)
220         continue
            jcount = jcount+mset

!           --- Put new level in istore1

            do 230 i = 1, mset
               istore1(i) = nextlevel(i)
230         continue
            nset = mset

            if ( mset==0 ) then

!           --- mset  =  0  last set consists of more parts

               if ( onepar ) then

!              --- Only one part allowed

                  ihset = jcount

               else

!              --- Detect rest of the set from marknodes

                  k = 0
                  do 250 i = ifirst, nnodes
                     if ( marknodes(i)>=0 ) then
                        k = k+1
                        istore1(k) = i
                     end if
250               continue

!                 --- Find node with minimal number of neighbors in this set
!                     Store this node in ksetnp and adapt marknodes

                  call chtmc1 ( k, istore1, mins, nbr )
                  jcount = jcount+1
                  ksetnp(jcount) = istore1(mins)
                  marknodes(istore1(mins)) = ithres
                  istore1(1) = istore1(mins)
                  nset = 1

               end if

            end if

            goto 200

         end if

      end if

      mset = jcount

      deallocate ( nextlevel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End chtmc3'

      end if  ! ( debug )
      call erclos ( 'chtmc3' )

      end subroutine chtmc3
