      subroutine chtmc6err ( iwork, icount, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-07-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print extra information with respect to error 1654
!
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), icount, npoint

!     icount         i    number of nodes in firts connected set
!     iwork          i    integer marker array of size npoint
!                         if iwork(i)<0, node i corresponds to first set
!                         if iwork(i)=0, node i does not belong to set
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical first

!     first          If true the heading of the mesaage must be printed
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'chtmc6err' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from chtmc6err'
         write(irefwr,1) 'icount, npoint', icount, npoint
         call prinin ( iwork, npoint, 'iwork' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

      first = .true.  ! no text has been written

      do i = 1, icount-1

         if ( iwork(i)==0 ) then

            if ( first ) then

!           --- first node that has not been found, write message

               write(irefwr,100)
100            format ( 'The following nodes are not present in the', &
                        ' first connected set' )
               first = .false.  ! heading has been written

            end if  ! ( first )
            write(irefwr,110) i
110         format ( i9 )

         end if  ! ( iwork(i)==0 )

      end do  ! i = 1, icount

      first = .true.  ! no text has been written

      do i = icount, npoint

         if ( iwork(i)<0 ) then

            if ( first ) then

!           --- first node that has not been found, write message

               write(irefwr,200)
200            format ( 'The following nodes are present in the', &
                        ' first connected set' )
               first = .false.  ! heading has been written

            end if  ! ( first )
            write(irefwr,110) i

         end if  ! ( iwork(i)==0 )

      end do  ! i = 1, icount

      call erclos ( 'chtmc6err' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End chtmc6err'

      end if  ! ( debug )

      end
