      subroutine incm01imat1 ( nfdegmid, kfree, nfdeg, &
                               numnei, istore, numneimid )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array irowsii
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nfdegmid, kfree(2,*), nfdeg, numnei, &
              istore(nusol,2), numneimid

!     istore         i    work array of length nusolx2 to store the
!                         non-prescribed neighbour degrees of freedom of a point
!                         (i,1) contains the degree of freedom
!                         (i,2) contains the position of the unknown in the node
!     kfree          i    Array of length nunkp to store the non-prescribed
!                         degrees of freedom in a node
!     nfdeg          i    Number of free degrees of freedom in a point
!     nfdegmid       i    Number of free degrees of freedom corresponding to the
!                         first ndim+1 unknowns
!     numnei         i    Number of neighbors
!     numneimid      i    Number of standard neighbors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isum, knusol, j

!     i              Counting variable
!     isum           Integer variable to compute a sum
!     j              Counting variable
!     knusol         Position of unknown in array USOL
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm01imat1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm01imat1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nfdegmid
         isum   = 0
         knusol = kfree(1,i)
         do j = 1, nfdeg
            if ( kfree(1,j)<knusol ) isum = isum+1
         end do
         do j = 1, numneimid
            if ( istore(j,1)<knusol ) isum = isum+1
         end do
         irowsii(knusol+1) = isum
      end do
      do i = nfdegmid+1, nfdeg
         isum   = 0
         knusol = kfree(1,i)
         do j = 1, nfdeg
            if ( kfree(1,j)<knusol ) isum = isum+1
         end do
         do j = 1, numnei
            if ( istore(j,1)<knusol ) isum = isum+1
         end do
         irowsii(knusol+1) = isum
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End incm01imat1'

      end if  ! ( debug )
      call erclos ( 'incm01imat1' )

      end
