      subroutine incm01node ( kfree, icheck, nfdeg, nfdegmid, &
                              inusol, istore, numnei, ihelpunk, inunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.0  date 12-10-2007
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     put real degrees of freedom of nodal point node into kfree
!     Mark unknowns in icheck
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kfree(2,*), icheck(*), nfdeg, nfdegmid, inunkp, &
              inusol, istore(nusol,2), numnei, ihelpunk(*)

!     icheck        i/o   work array of length nusol to indicate if a degree of
!                         is present in array istore (1) or not (0)
!     ihelpunk       i    Work array indicating if unknowns in coupled nodes are
!                         periodical (0) or not (1)
!     inunkp         i    Number of degrees of freedom in point
!     inusol         i    Position of unknown in array USOL
!     istore        i/o   work array of length nusol to store the non-prescribed
!                         neighbour degrees of freedom of a point
!                         At input this array is 0
!                         At output it contains neighbours of periodical nodes
!     kfree          o    Array of length nunkp to store the non-prescribed
!                         degrees of freedom in a node
!     nfdeg          o    Number of free degrees of freedom in a point
!     nfdegmid       o    Number of free degrees of freedom corresponding to the
!                         first ndim+1 unknowns
!     numnei        i/o   Number of neighbor unknowns stored in istore
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, knusol

!     i              Counting variable
!     knusol         Position of unknown in array USOL
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm01node' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm01node'
         write(irefwr,1) 'indprr, inunkp, inusol', indprr, inunkp, inusol
         write(irefwr,1) 'nrusol', nrusol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprr==0 ) then

!     --- No renumbering of unknowns

         do i = 1, inunkp
            kfree(1,i) = inusol+i
            icheck(inusol+i) = 1
         end do
         nfdeg = inunkp
         if ( extrafillin>=7 ) then
            nfdegmid = ndim+1
         else
            nfdegmid = 0
         end if

      else

!     --- renumbering of unknowns

         do i = 1, inunkp
            knusol = kprobr(inusol+i)

            if ( debug ) write(irefwr,1) 'i, knusol', i, knusol
            if ( knusol<=nrusol ) then

!           --- free unknown found

               if ( ihelpunk(i)==0 ) then

!              --- Standard case, store in kfree

                  nfdeg = nfdeg+1
                  kfree(1,nfdeg) = knusol
                  kfree(2,nfdeg) = i
                  icheck(knusol) = 1

               else

!              --- Unknown differs in periodical nodes, store in istore

                  numnei = numnei+1
                  istore(numnei,1) = knusol
                  istore(numnei,2) = i
                  icheck(knusol) = 1

               end if

            end if
            if ( i==ndim+1 ) nfdegmid = nfdeg

         end do
         if ( extrafillin>=6 ) then
            if ( inunkp<=ndim+1 ) nfdegmid = 0
         else
            nfdegmid = 0
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nfdeg, nfdegmid, numnei', &
                          nfdeg, nfdegmid, numnei
         call prinin1 ( kfree, nfdeg, 2, 'kfree' )
         call prinin1 ( istore, 2, numnei, 'istore' )
         write(irefwr,*) 'End incm01node'

      end if  ! ( debug )
      call erclos ( 'incm01node' )

      end
