      subroutine incommatinter1
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.0  date 29-04-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays irowsfi and irowsif for integer information of matrix
!     corresponding to interface unknowns
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nodp, numintf, numfree, j, inusol, knusol, inunkp, &
                 leninterf, interf(100), ibr, nbr, nfnbr, iknbr, indpres

!     i              Counting variable
!     ibr            Loop variable for neighbors
!     iknbr          Nodal point number of neighbor
!     indpres        All unknowns with sequence number larger than indpres
!                    are prescribed
!     interf         Contains unknown numbers of interface
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     knusol         Position of unknown in array USOL
!     leninterf      Number of interface unknowns in nodal point
!     nbr            number of neighbors of nodal point
!     nfnbr          starting address in array kmshe2
!     nodp           Nodal point sequence number
!     numfree        Number of free unknowns coupled to interface unknowns
!     numintf        Number of interface unknowns in nodal point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     irowsfi := 0; irowsif := 0
!     For all nodes nodp do
!        Store all interface unknowns in array interf
!        If ( there are interface unknowns in nodp ) then
!           numintf := number of interface unknowns in node
!           numfree := number of free unknowns in node
!           For all neighbor nodes ibr of nodp do
!              numfree := numfree + number of free unknowns in ibr
!        For all interface unknowns i in array interf do
!           irowsfi(i):=numfree
!     Accumulate intmat1 a-c
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatinter1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatinter1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      indpres = nrusol+ninterface  ! first index of prescribed unknowns

      if ( debug ) &
         write(irefwr,1) 'npoint, nrusol, ninterface, indpres', &
                          npoint, nrusol, ninterface, indpres

!     --- irowsfi := 0; irowsif := 0

      irowsfi = 0
      irowsif = 0

!     --- For all nodes nodp do

      do nodp = 1, npoint

!     --- Store all interface unknowns in array interf

         inusol = sepgetinusol ( nodp, inunkp )
         numintf = 0

         if ( debug ) &
            write(irefwr,1) 'nodp, inunkp, inusol', nodp, inunkp, inusol

         do j = 1, inunkp

            knusol = kprobh(inusol+j)
            if ( knusol>nrusol .and. knusol<=indpres ) then

!           --- interface unknown found

               numintf = numintf+1
               interf(numintf) = knusol

            end if  ! ( knusol>nrusol .and. knusol<=indpres )

         end do  ! j = 1, inunkp

         if ( numintf>0 ) then

!        --- there are interface unknowns in nodp

            leninterf = numintf
            numfree = 0

            if ( debug ) call prinin ( interf, numintf, 'interf' )

            do j = 1, inunkp

               knusol = kprobh(inusol+j)    ! renumbered unknown
               if ( knusol<=indpres ) numfree = numfree+1  ! not prescribed

            end do  ! j = 1, inunkp

!           --- For all neighbor nodes ibr of nodp do

            nfnbr = kmshe1(nodp)
            nbr = kmshe1(nodp+1)-nfnbr  ! number of neighbors

            do ibr = 1,nbr

               iknbr = kmshe2(nfnbr+ibr)  ! neighbor node

!              --- get the number of d.o.f.'s in this neighboring node

               inusol = sepgetinusol ( iknbr, inunkp )

               if ( debug ) &
                  write(irefwr,1) 'ibr, iknbr, inunkp, inusol', &
                                   ibr, iknbr, inunkp, inusol

               do j = 1, inunkp

                  knusol = kprobh(inusol+j)

                  if ( debug ) write(irefwr,1) 'knusol', knusol
                  if ( knusol<=indpres ) numfree = numfree+1
                  if ( knusol<=nrusol ) then

!                 --- Free degree of freedom found, raise irowsif

                     irowsif(knusol) = irowsif(knusol)+numintf

                  end if  ! ( knusol<=nrusol )

               end do  ! j = 1, inunkp

            end do  ! ibr = 1,nbr

!           --- For all interface unknowns i in array interf do

            do i = 1, leninterf

!           --- irowsfi(i):=numfree

               inusol = interf(i)-nrusol
               irowsfi(inusol) = numfree

            end do  ! i = 1, leninterf

         end if  ! ( numintf>0 )

      end do  ! nodp = 1, npoint

      if ( debug ) call prinin ( irowsfi, ninterface+1, 'irowsfi' )

!     --- Accumulate

      do i = 1, ninterface
         irowsfi(i) = irowsfi(i)+irowsfi(i-1)
      end do  ! i = 1, ninterface

      do i = 1, nrusol
         irowsif(i) = irowsif(i)+irowsif(i-1)
      end do  ! i = 1, nrusol

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irowsfi, ninterface+1, 'irowsfi' )
         call prinin ( irowsif, nrusol+1, 'irowsif' )
         write(irefwr,*) 'End incommatinter1'

      end if  ! ( debug )
      call erclos ( 'incommatinter1' )

      end subroutine incommatinter1
