      subroutine incommatpar2 ( lenstore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute length of new array istore
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: lenstore

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numupp, i, numlow, istart, ninterf, iblock, lenfi

!     i              Counting variable
!     iblock         Block sequence number
!     istart         First position used at a part
!     lenfi          Length of array icolsfi
!     ninterf        Number of unknowns at interface
!     numlow         Number of neighbor blocks with lower number
!     numupp         Number of neighbor blocks with higher number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpar2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpar2'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numlow = blockinfor(1)  ! number of neighbors with smaller number
      numupp = blockinfor(2)  ! number of neighbors with larger number

      lenstore = iacnodes

      if ( debug ) &
          write(irefwr,1) 'numlow, numupp, lenstore', &
                           numlow, numupp, lenstore

      do i = 1, numupp

         iblock = blockinfor(2+i+numlow)
         istart= istore(iblock)
         if ( istart>0 ) then

            ninterf = istore(istart)

            if ( ninterf>0 ) then

!           --- There are interface unknowns
!               Raise lenstore

               istart= istart+ninterf
               lenfi = istore(istart+ninterf+1)
               lenstore = lenstore+2*ninterf+2+lenfi

               if ( debug ) then
                  write(irefwr,1) 'i, istart, ninterf, lenstore', &
                                   i, istart, ninterf, lenstore
                  write(irefwr,1) 'length_fi', lenfi
               end if  ! ( debug )

            end if  ! ( ninterf>0 )

         end if  ! ( istart>0 )

      end do  ! i = 1, numupp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lenstore', lenstore
         write(irefwr,*) 'End incommatpar2'

      end if  ! ( debug )
      call erclos ( 'incommatpar2' )

      end subroutine incommatpar2
