      subroutine inielemnodes ( ksetnp, mset, elements, numelms, numelms1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-02-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subdivide a set of elements into a set of which all nodes are part
!     of a set of nodes and the rest
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element
!     subdivision
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ksetnp(*), mset, elements(*), numelms, &
              numelms1

!     elements      i/o   Array of length numelms containing the element
!                         numbers corresponding to the local mesh
!                         After renumbering of he elements the new sequence is
!                         stored in
!     ksetnp         i    array containing the set of nodal points
!     mset           i    number of nodal points in ksetnp
!     numelms        i    Number of elements in array elements
!     numelms1       o    Number of elements in the first part of elements
!                         So the mesh is subdivided in a set of numelms1
!                         elements and a set of numelms-numelms1 elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, isecond, j, inpelm, ifirst, ielem, ihelp(npoint), &
              istore(nelem)
      logical allnodes

!     allnodes       If true all nodes of the element are part of array ksetnp
!     i              Counting variable
!     ielem          Element number
!     ifirst         Number of elements in first part
!     ihelp          integer work array of length npoint
!                    ihelp(i) = 0 means that node i does not belong to the
!                    set ksetnp
!                    ihelp(i) = 1 means that node i belongs to the set
!                    ksetnp
!     inpelm         number of points per element
!     isecond        Number of elements in second part
!     istart         First position used at a part
!     istore         Work array of length nelem to store a set of elements
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array ihelp (1:npoint)
!     Make ihelp 1 for all nodes in ksetnp
!     ifirst := 0; isecond := 0
!     for i := 1 (1) numelms do
!        ielem := elements(i)
!        if ( all nodes in ielem belong to ksetnp ) then
!           ifirst := ifirst+1
!           elements(ifirst) := ielem
!        else ( not all nodes in ielem belong to ksetnp )
!           isecond := isecond+1
!           istore(isecond) := ielem
!     numelms1 := ifirst
!     Copy istore to elements from position ifirst+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inielemnodes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inielemnodes'
         write(irefwr,*) 'npoint, mset, numelms ',npoint, mset, numelms
         call prinin ( elements, numelms, 'elements' )
         call prinin ( ksetnp, mset, 'ksetnp' )

      end if

!     --- Clear array ihelp (1:npoint)
!         Make ihelp 1 for all nodes in ksetnp

      ihelp = 0
      ihelp(ksetnp(1:mset)) = 1

!     --- ifirst := 0; isecond := 0

      ifirst = 0
      isecond = 0

!     --- for i := 1 (1) numelms do

      do i = 1, numelms

!     --- ielem := elements(i)

         ielem = elements(i)

!        --- check if all nodes in ielem belong to ksetnp

         allnodes = .true.
         istart = kmeshcpoint(ielem)
         inpelm = kmeshcpoint(ielem+1)-istart
         do j = istart+1, istart+inpelm
            if ( ihelp(kmeshc(j))==0 ) then

!           --- Not all nodes belong to set

               allnodes = .false.
               exit

            end if

         end do

         if ( allnodes ) then

!        --- all nodes in ielem belong to ksetnp
!            ifirst := ifirst+1
!            elements(ifirst) := ielem

            ifirst = ifirst+1
            elements(ifirst) = ielem

         else

!        --- not all nodes in ielem belong to ksetnp
!            isecond := isecond+1
!            istore(isecond) := ielem

            isecond = isecond+1
            istore(isecond) = ielem

         end if  ! ( allnodes )

      end do  ! i = 1, numelms

!     --- numelms1 := ifirst
!         Copy istore to elements from position ifirst+1

      numelms1 = ifirst
      elements(ifirst+1:ifirst+isecond) = istore(1:isecond)

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'numelms1 ', numelms1
         call prinin ( elements, numelms, 'elements' )
         write(irefwr,*) 'End inielemnodes'

      end if
      call erclos ( 'inielemnodes' )

      end
