      subroutine init02 ( kprobptmp, ielshape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 28-05-2014 Extra parameter iprob
!        version  6.6  date 01-02-2012 Use printkprobe1
!        version  6.5  date 28-03-2001 Correction printing part p (CvO)
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     write input of subroutine init
!
! **********************************************************************
!
!                       KEYWORDS
!
!     structure
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprobptmp(*), ielshape(2,*)

!     ielshape   i    Array of length 2*(nelgrp+numnat) containing:
!                     (1,i) number of nodes in element group i
!                     (2,i) shape number of elements in element group i
!                     if i>nelgrp: i-nelgrp = boundary group number
!                     otherwise it is the element group number
!     kprobptmp  o    Temporary array KPROB part p
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikelm, i, ns, ik, itype, ielgrp, inpelm, iphys

!     i          Counting variable
!     ielgrp     element group number
!     ik         Help variable
!     ikelm      Starting address in kmesh
!     inpelm     number of points per element
!     iphys      Number of physical variables
!     itype      Type number of element
!     ns         Number of element groups including boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PRINTKPROBE1   Print boundary elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- actual program

      if ( ioutp==0 ) go to 450  ! print only a small part of the information

      ntype = kprobloc(7)
      write ( irefwr,100)
100   format( ' input for subroutine probdf:')
      write ( irefwr,110) nunkp, ndtvec, ntype
110   format(' maximal number of degrees of freedom in a nodal point', &
              i3 / ' number of arrays of special structure', i3/ &
             ' number of different types per standard element', i3 )
      inpelm = npelm
      if ( indpra>0 ) then

!     --- indpra>0, kprob part a

         write(irefwr,200)
200      format(/' number of degrees of freedom in nodal points of ', &
                 'standard elements')
         ikelm = 0
         do ielgrp = 1, nelgrp
            inpelm = ielshape(1,ielgrp)
            kprobatmp => kproba(ielgrp)%arr
            write(irefwr,210) ielgrp, (i,kprobatmp(i),i=1,inpelm)
210         format(i6, 6(i6, ':', i4)/(6x, 6(i6, ':', i4)))
            ikelm = ikelm+inpelm
         end do
         if ( numnat>0 ) write(irefwr,220)
220      format(/' number of degrees of freedom in nodal points of ', &
                 'standard boundary elements'/ &
                 '   (-1 means that it will copied from the internal ', &
                 'element)' )
         do ielgrp = nelgrp+1, nelgrp+numnat
            inpelm = ielshape(1,ielgrp)
            kprobatmp => kproba(ielgrp)%arr
            write(irefwr,210) ielgrp-nelgrp, &
                              (i,kprobatmp(i),i=1,inpelm)
            ikelm = ikelm+inpelm
         end do
      end if

!     --- kprob parts c and d

      ns = nelgrp+numnat
      write(irefwr,230)
230   format(/'  type numbers of standard elements:')
      do ielgrp = 1, ns
         if ( ntype==1 ) then
           write(irefwr,240) ielgrp, kprobc(ielgrp)
240        format(i6, ':', 5x, 6i8/(12x, 6i8))
         else
           write(irefwr,240) ielgrp, (kprobd((i-1)*ns+ielgrp), &
                 i=1,ntype)
         end if
      end do

!     --- kprob part e

      call printkprobe1 ( ielshape )

      if ( numnat>0 ) then

!     --- kprob part j

         write(irefwr,300)
300      format(' number of nodal points in boundary elements')
         write(irefwr,310) (i, kprobj(i,1), i = 1, numnat)
310      format(6(i7, ':', i4))

      end if

      if ( nphys>0 ) then

!     --- nphys>0  write kprob part p

         do iphys = 1, nphys
            write(irefwr, 410) iphys
410         format(' Position of physical unknown', i4, ' in local', &
                   ' nodal points of element groups')
            ikelm = 0
            do ielgrp = 1, ns
               inpelm = ielshape(1,ielgrp)
               itype = kprobc(ielgrp)
               if (itype>=0) then
                  ik = (iphys-1)*inpelm+ikelm
                  write(irefwr,210) ielgrp,(i, kprobptmp(ik+i), i=1,inpelm)
                  ikelm = ikelm+nphys*inpelm
               end if
            end do
         end do
      end if

450   write ( irefwr, 500 ) nusol, iprob
500   format( ' number of degrees of freedom = ', i8, &
              ' in problem ', i2 )
      if ( indprx>0 ) then
         write ( irefwr, 510 ) kprobx(1:2)
510      format(/' number of global groups = ', i3/ &
                 ' number of global degrees of freedom = ', i8)
      end if
      if ( indpry>0 ) then
         write ( irefwr, 520 ) kproby(1:2)
520      format(/' number of fictitous groups = ', i3/ &
                 ' number of fictitous degrees of freedom = ', i8)
      end if

      end
