      subroutine init05 ( ityplc, istore, istfl1, ibuf, istart, &
                          inunkp, indtvc, iphys, jnpelm, jelgrp, &
                          ishape, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.6  date 21-11-2011 Extra debug
!        version  6.5  date 28-12-2006 Extra debug
!        version  6.4  date 23-05-2005 Fortran 90
!
!   copyright (c) 1982-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     information from backing storage (file 1) is stored in array istfl1
!     for a standard element with standard element number>99
! **********************************************************************
!
!                       KEYWORDS
!
!     file_1
!     read
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecomio
      use sepmodulefiles
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ityplc, istore(*), istfl1(*), ibuf, istart, &
              inunkp, indtvc, iphys, jnpelm, jelgrp, ishape, &
              ndim

!     ibuf      i/o   Next position to be used in ISTORE
!     indtvc     i    Number of vectors of special structure
!     inunkp     i    Number of degrees of freedom in first point
!     iphys      i    Number of physical variables
!     ishape     i    Shape number of elements
!     istart    i/o   Next position to be used in array IINPUT
!     istfl1     o    Array in which information from FILE 1 is stored
!     istore     o    Array containing information about standard elements
!                     array istore is filled as follows:
!                     For each element group   (nelgrp standard elements):
!                     pos. 1-ntype   ntype type numbers
!                     next   inunkp, indtvc, iphys
!                     next   if inunkp<=0: jnpelm pos. with degrees of
!                            freedom
!                     next   indtvc*jnpelm pos. with degrees of freedom for
!                            arrays of special structure
!                     next   iphys*jnpelm pos. with information of physical
!                            degrees of freedom
!     ityplc      i    type number of standard element
!     jelgrp     i    element group number
!     jnpelm     i    number of points per element
!     ndim       i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kh1, kh2, jnunkp, ivec, ncheld, i, inundt, meshtp, &
              loctype, indtvcex
      logical type900found, type903found
      save type900found, type903found

!     i              Counting variable
!     indtvcex       Extra vectors of special structure
!     inundt
!     ivec           Sequence number of vector of special structure
!     jnunkp         Number of degrees of freedom in second point
!     kh1            Help parameter to compute a position in KPROB
!     kh2            Help parameter to compute a position in KPROB
!     loctype        mod(ityplc,10000), i.e. the actual type number
!     meshtp         Shape number of element as stored on file 1
!                    If meshtp = -1, the shape of the element is free and
!                    exactly 10 positions are filled in file 1 with information
!                    concerning the unknowns for each inunkp>0
!     ncheld         Number of choices for derivative subroutine
!     type900found   Type number 900 901 or 902 found
!     type903found   Type number 903 found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer init08

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIT01         Store unknowns in ISTORE if ishape = -1
!     INIT08         Compute nunkp as function of inunkp and ndim for
!                    inunkp<-1000
!     PRININ         print 1d integer array
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!    input from file 1 is read
!    File 1 is filled as follows:
! part 1)  records 1 up to krec2-1:  pointers for each standard problem
!          with 99<ityplc<maxtyp  (maximum type number) to part 2
!          For a certain type the pointer may be to another type
!          only if all the information is identical
! part 2)  records krec2,...,9 information on standard problems
!  for each type:
!    pos. 1   inunkp    number of degrees of freedom
!         2   numvsp    number of vectors of special structure
!         3   meshtp    shape number of the element
!         4   nphys     physical variables of solution
!      if inunkp<0    number of degrees of freedom is constant (-inunkp)
!                       if inunkp=-1001 the number of degrees of freedom is
!                       ndim (dimension of space)
!                       if inunkp=-1002 the number of degrees of freedom is
!                       equal to the lower part of a two-tensor.
!                       Hence ndim = 1: 1,  ndim = 2: 3,  ndim = 3: 6
!                       if inunkp=-1003 the number of degrees of freedom is
!                       equal to the lower part of a two-tensor, the main
!                       diagonal excluded.
!                       Hence ndim = 1: 0,  ndim = 2: 1,  ndim = 3: 3
!                       if inunkp=-1004 the number of degrees of freedom is
!                       ndim**2
!                       if inunkp=-1005 the number of degrees of freedom is
!                       2*ndim+1
!                >=0    number of degrees of freedom is not constant, &
!                       and then inunkp must be a pointer to the
!                       position in file 1 where information about the
!                       degrees of freedom can be found (computed by
!                       this program)
!    when numvsp>0:
!         4 + ivec:  inunkp corresponding to ivec th array of special
!                    structure  1<=ivec<=numvsp
!                    (same approach as for degrees of freedom of sol.)
!         5 + numvsp:   ncheld (number of possible values of icheld
!                                 in call of deriva)
!    when  ncheld>0:
!         6 + numvsp + 2 * icheld - 1:  ichder(icheld)
!         6 + numvsp + 2 * icheld - 2:  ivec(icheld)
!                  ichder=1 means vector is like the solution vector
!                         2       ivec'th vector of special structure
!                         3       vector defined per element;
!                                 then ivec is the number of degrees
!                                 of freedom
!                         4       ivec'th vector, but no averaging
!                         if ivec<0 then the vector is complex
!                         (see deriva)
!    followed by (if numvsp>0):
!         ichstr (stream)
!         ivec   (stream)
!                  ichstr=1 means stream function
!                         2       potential
!   when  nphys>0
!         iphys  inunkp corresponding to iphys th array of special
!                structure  1<=iphys<=nphys
!                (same approach as for degrees of freedom of sol.)
!   when any inunkp>0 then followed by
!         information of degrees of freedom per nodal point of
!         an element for solution, vectors of special structure, &
!         and physical degrees of freedom
!   Remark:
!   If meshtp = -1 a special situation occurs. In that case inunkp>0 contains
!   a pointer to a position in file 1 with information about the number of
!   unknowns. The number of actual positions contained in that case is
!   always 10. These ten positions have the following meaning:
!   Pos. 1:  number of unknowns in the vertices of the elements
!        2:  number of unknowns in mid side points
!        3:  number of unknowns in the centroid
!        4:  number of unknowns in the centroids of faces
!        5-10:  not yet defined
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      19   shape number not in correspondence with standard element
!      43   Type number not available
!     550   array length too short
!    1166   inunkp incorrect
!    2218   Taylor-Hood and Crouzeix-Raviart in the same mesh
!    2498   No Navier-Stokes elements found, whereas type 922 is used
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read information concerning type numbers from file 1 if necessary
!     Compute starting address of information concerning the type in array
!     istfl1
!     Read information concerning information corresponding to type numbers
!     from file 1 if necessary
!     Compute starting address of information concerning the type in array
!     istfl1
!     Put information into istore (read information if necessary)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data type900found /.false./, type903found /.false./
! ======================================================================
!
      call eropen ( 'init05' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from init05'
  1      format ( a, 1x, (10i6) )

      end if

      if ( ityplc>=900 .and. ityplc<=902 ) type900found = .true.
      if ( ityplc==903 ) type903found = .true.

!     --- Read information concerning type numbers from file 1 if necessary

      loctype = mod(ityplc,10000)
      indtvcex = 0
      kh1 = (loctype-100)/nrec1+1
      if ( kh1/=irecp1 ) call readfl ( kh1, 1, nrec1, 1, 1, istfl1 )

!     --- Compute starting address of information concerning the type in array
!         istfl1

      ipos = istfl1(loctype-99-nrec1*(kh1-1))
      if ( ipos==0 ) then

!     --- Error 43: type number not available

         call errint ( loctype, 1 )
         call errint ( jelgrp, 2 )
         call errsub ( 43, 2, 0, 0)
         go to 1000

      end if

!     --- Read information concerning information corresponding to type numbers
!         from file 1 if necessary

      kh2 = (ipos-1)/nrec1+1
      if ( kh2/=irecp2 ) &
         call readfl ( kh2, nrec1+1, 2*nrec1, 2, 1, istfl1 )

!     --- Compute starting address of information concerning the type in array
!         istfl1

      ipos   = ipos-(kh2-2)*nrec1
      inunkp = istfl1(ipos)

      indtvc = istfl1(ipos+1)
      if ( ityplc>10000 ) then

!     --- Extra vectors of special structure

         indtvcex = iinputkp(istart)
         istart = istart+1
         indtvc = indtvc+indtvcex

      end if
      meshtp = istfl1(ipos+2)
      iphys  = istfl1(ipos+3)
      if ( debug ) &
         write(irefwr,1) &
            'ityplc, inunkp, indtvc, meshtp, iphys, ishape', &
             ityplc, inunkp, indtvc, meshtp, iphys, ishape

      if ( meshtp>=0 .and. meshtp/=ishape ) then

!     --- error 19: shape number not in correspondence with standard element

         call errint ( meshtp, 1 )
         call errint ( ishape, 2 )
         call errint ( jelgrp, 3 )
         call errsub ( 19, 3, 0, 0 )

      end if

      jnunkp = max ( 0, -inunkp )
      if ( inunkp<-1000 ) jnunkp = init08 ( inunkp, ndim, ishape )
      if ( debug ) write(irefwr,1) 'jnunkp', jnunkp
      if ( ityplc==922 ) then

!     --- Special element for fictitious domain
!         For elements of type 900 junkp = 2*ndim, for elements of type 903
!         it is 2*ndim

         jnunkp = 0
         if ( type903found ) then

!        --- Elements of type 903 found, 2*ndim+1

            inunkp = 2*ndim+1

         else if ( type900found ) then

!        --- Elements of type 900 found, 2*ndim

            inunkp = 2*ndim

         else

!        --- No elements of type 900/903 found, 2*ndim
!            Give warning

            inunkp = 2*ndim
            call errint ( ityplc, 1 )
            call errwar ( 2498, 1, 0, 0 )

         end if

         nunkp = max(nunkp,inunkp)
         meshtp = 1
         jnpelm = 2
         indtvc = 0
         iphys = inunkp

      end if  ! ( ityplc==922 )
      if ( iphys<-1000 ) iphys = init08 ( iphys, ndim, ishape )
      istore(ibuf+1) = jnunkp
      istore(ibuf+2) = indtvc
      istore(ibuf+3) = iphys
      ibuf = ibuf+3
      ipos = ipos+3
      if ( debug ) &
         write(irefwr,1) 'jnunkp, indtvc, iphys, inunkp', &
                          jnunkp, indtvc, iphys, inunkp

      if ( indtvc>0 ) then

!     --- indtvc>0  arrays of special structure

         do ivec = 1, indtvc-indtvcex
            ind(ivec) = istfl1(ipos+ivec)
         end do
         do ivec = indtvc-indtvcex+1, indtvc
            ind(ivec) = 9999
         end do

      end if

      ipos = ipos+indtvc+1-indtvcex

      if ( iphys>0 .and. ityplc/=922 ) then

!     --- iphys>0

         ncheld = istfl1(ipos)
         ipos = ipos+ncheld*2
         if ( indtvc>0 ) ipos = ipos + 2
         do i = 1, iphys
            ind(indtvc+i) = istfl1(ipos+i)
         end do

      end if

      if ( indtvc+iphys>200 ) then

!     --- error 550: array length too short

         call errint ( indtvc+iphys, 1 )
         call errint ( 200, 2 )
         call errchr ( 'ind', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if

      if ( inunkp>0 ) then

!     --- inunkp>0  read degrees of freedom

         if ( ityplc==922 ) then

!        --- Special element for fictitious domain

            if ( type903found ) then

!           --- Elements of type 903 found, 2*ndim+1

               istore(1+ibuf) = 2*ndim+1
               istore(2+ibuf) = ndim+1

            else

!           --- Elements of type 900 found, 2*ndim

               istore(1+ibuf) = 2*ndim
               istore(2+ibuf) = ndim

            end if
            ibuf = ibuf+2

         else if ( inunkp<nrec1 ) then

!        --- error 1166: inunkp incorrect

            call errint ( inunkp, 1 )
            call errint ( nrec1, 2 )
            call errsub ( 1166, 2, 0, 0 )

         else

            kh2 = (inunkp-1)/nrec1+1
            if ( kh2/=irecp2 ) call readfl ( kh2, nrec1+1, 2*nrec1, &
                 2, 1, istfl1 )
            ipos = inunkp-(kh2-2)*nrec1-1

            if ( meshtp>=0 ) then

!           --- meshtp>0, classical situation

               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'ipos, inunkp, ndim, jnpelm ', &
                                   ipos, inunkp, ndim, jnpelm
                  call prinin ( istfl1(ipos+1), jnpelm, 'istfl1' )

               end if

               do i = 1, jnpelm
                  inunkp = istfl1(ipos+i)
                  if ( inunkp<-1000 ) &
                     inunkp = init08 ( istfl1(ipos+i), ndim, meshtp )
                  nunkp = max ( nunkp, inunkp )
                  istore(i+ibuf) = inunkp
               end do

            else

!           --- meshtp = -1, the unknowns are stored in the new way

               if ( debug ) then

!              --- Debug information

                  write(irefwr,1) 'ipos, inunkp, ndim, ishape', &
                                   ipos, inunkp, ndim, ishape
                  call prinin ( istfl1(ipos+1), jnpelm, 'istfl1' )

               end if  ! ( debug )

               call init01 ( ishape, ndim, istore(ibuf+1), &
                             istfl1(ipos+1), jnpelm )

               if ( debug ) then

!              --- Debug information

                  write(irefwr,1) 'ibuf+1', ibuf+1
                  call prinin ( istore(ibuf+1), jnpelm, 'istore' )

               end if  ! ( debug )

               do i = 1, jnpelm
                  inunkp = istore(i+ibuf)
                  nunkp = max ( nunkp, inunkp )
               end do

            end if

            ibuf = ibuf+jnpelm

         end if

      end if

      inunkp = jnunkp
      if ( debug ) write(irefwr,1) 'inunkp, nunkp', inunkp, nunkp

      if ( indtvc>0 ) then

!     --- indtvc>0 arrays of special structure

         do ivec = 1, indtvc

            inundt = ind(ivec)
            if ( inundt<=0 ) then

!           --- inundt<=  0

               if ( inundt<-1000 ) inundt = -init08 ( inundt, ndim, ishape )
               do i = 1, jnpelm
                  istore(i+ibuf) = -inundt
               end do

            else

!           --- inundt>0

               if ( ivec<=indtvc-indtvcex ) then

!              --- Read information from file 1 for the first vectors of
!                  special structure

                  kh2 = (inundt-1)/nrec1+1
                  if ( kh2/=irecp2 ) call readfl ( kh2, nrec1+1, &
                       2*nrec1, 2, 1, istfl1 )
                  ipos = inundt-(kh2-2)*nrec1-1

               end if
               if ( ivec<=indtvc-indtvcex .and. meshtp>=0 ) then

!              --- meshtp>0, classical situation

                  do i = 1, jnpelm
                     istore(i+ibuf) = istfl1(ipos+i)
                     if ( istore(i+ibuf)<-1000 ) istore(i+ibuf) = &
                        init08 ( istfl1(ipos+i), ndim, meshtp )
                  end do

               else if ( ivec>indtvc-indtvcex ) then

!              --- Extra vectors, these are stored in the new way

                     call init01 ( ishape, ndim, istore(ibuf+1), &
                                   iinputkp(istart), jnpelm )
                     istart = istart + 4

               else

!              --- meshtp = -1, the unknowns are stored in the new way

                  call init01 ( ishape, ndim, istore(ibuf+1), &
                                istfl1(ipos+1), jnpelm )
               end if

            end if

            ibuf = ibuf+jnpelm

         end do

      end if

!     --- fill information concerning physical quantities

      if ( ityplc==922 ) then

!     --- Special case: fictitious domain elements

         do ivec = 1, iphys-ndim
            istore(ibuf+1) = ivec
            istore(ibuf+2) = ivec
            ibuf = ibuf+2
         end do
         do ivec = iphys-ndim+1, iphys
            istore(ibuf+1) = ivec
            istore(ibuf+2) = 0
            ibuf = ibuf+2
         end do

      else if ( iphys>0 ) then

         do ivec = 1, iphys
            inundt = ind(ivec+indtvc)
            if ( inundt<=0 ) then

!           --- inundt<=  0

               if ( inundt<-1000 ) inundt = -init08 ( inundt, ndim, ishape )
               do i = 1, jnpelm
                  istore(i+ibuf) = -inundt
               end do

            else

!           --- inundt>0

               kh2 = (inundt-1)/nrec1+1
               if ( kh2/=irecp2 ) call readfl ( kh2, nrec1+1, 2*nrec1, &
                    2, 1, istfl1 )
               ipos = inundt-(kh2-2)*nrec1-1

               if ( meshtp>=0 ) then

!              --- meshtp>0, classical situation

                  do i = 1, jnpelm
                     istore(i+ibuf) = istfl1(ipos+i)
                     if ( istore(i+ibuf)<-1000 ) istore(i+ibuf) = &
                        init08 ( istfl1(ipos+i), ndim, meshtp )
                  end do

               else

!              --- meshtp = -1, the unknowns are stored in the new way

                  call init01 ( ishape, ndim, istore(ibuf+1), &
                                istfl1(ipos+1), jnpelm )

               end if

            end if

            ibuf = ibuf+jnpelm

         end do

      end if

      if ( type900found .and. type903found ) then

!     --- warning: both Taylor Hood and Crouzeix-Raviart elements

         call errwar ( 2218, 0, 0, 0 )

      end if

1000  if ( debug ) write(irefwr,*) 'End init05'
      call erclos ( 'init05' )

      end
