      subroutine init06 ( istore, kprobptmp, ielshape, type922found )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.2  date 17-10-2011 fortran 90
!        version  6.1  date 12-02-2003 Debug statements
!        version  6.0  date 26-01-2001 Extra parameter type922found
!
!   copyright (c) 1982-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     information is stored in array kprob
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(*), kprobptmp(*), ielshape(2,*)
      logical type922found

!     ielshape       i    Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
!     istore         i    Array containing information about standard elements
!                         array istore is filled as follows:
!                         For each element group   (nelgrp standard elements):
!                         pos. 1-ntype ntype type numbers
!                         next    inunkp, indtvc, iphys
!                         next    if inunkp<=0: inpelm pos. with degrees of
!                                 freedom
!                         next    indtvc*inpelm pos. with degrees of freedom for
!                                 arrays of special structure
!                         next    iphys*inpelm pos. with information of physical
!                                 degrees of freedom
!     kprobptmp      i    SEPRAN array KPROB part p containing the positions of
!                         the unkowns per point
!     type922found   i    If true elements of type 922 hve been found, which
!                         means that Lagragian multipliers are defined by means
!                         of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ibuf, ih1, ns, ikprbp, ih, itype, ivec, ielgrp, j, &
              inpelm, inunkp, iphys, junkpt, indtvc, ibngrp

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     ibuf           Next position to be used in istore
!     ielgrp         element group number
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     ikprbp         ?
!     indtvc         Number of vectors of special structure
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in first point
!     iphys          Number of physical variables
!     itype          Type number of element
!     ivec           Sequence number of vector of special structure
!     j              Counting variable
!     junkpt         If 0, the number of degrees of freedom per point is not
!                    constant
!     ns             nelgrp+numnat
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init06' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from init06'
  1      format ( a, 1x, (10i10) )

      end if

!     --- fill rest of array kprob

      ibuf = 0
      ih1 = 0
      ikprbp = 0
      ntype  = iinputkp(1)
      numnat = iinputkp(2)
      nbcons = iinputkp(13)
      junkpt = intarprb(11)
      nphys = intarprb(14)

      if ( debug ) then
         write(irefwr,1) 'nelgrp, ntype, numnat, nbcons, junkpt', &
                          nelgrp, ntype, numnat, nbcons, junkpt
         write(irefwr,1) 'ndtvec, indpra, indprd, indpre', &
                          ndtvec, indpra, indprd, indpre
         write(irefwr,1) 'indprj, nphys', indprj, nphys
      end if  ! ( debug )

      ns = nelgrp+numnat
      do ielgrp = 1, ns
         inpelm = ielshape(1,ielgrp)
         itype = istore(ibuf+1)
         kprobc(ielgrp) = itype
         if ( ntype>1 ) then

!        --- ntype>1, fill part d

            do i = 1, ntype
               itype = istore(ibuf+i)
               kprobd(ielgrp+(i-1)*ns) = itype
            end do

         end if

         ibuf = ibuf+ntype
         if ( debug ) &
             write(irefwr,1) 'ielgrp, inpelm, itype, junkpt, ibuf', &
                              ielgrp, inpelm, itype, junkpt, ibuf

         if ( itype/=-1 ) then

            inunkp = istore(ibuf+1)
            indtvc = istore(ibuf+2)
            iphys = istore(ibuf+3)
            ibuf = ibuf+3
            if ( debug ) &
               write(irefwr,1) 'indpra, ih1, inunkp, iphys, junkpt', &
                                indpra, ih1, inunkp, iphys, junkpt

            if ( junkpt==0 ) then

!           --- junkpt = 0, fill kprob part a

               allocate ( kproba(ielgrp)%arr(inpelm) )
               kprobatmp => kproba(ielgrp)%arr
               if ( inunkp/=0 ) then

!              --- inunkp>0

                  kprobatmp = inunkp

               else

!              --- inunkp = 0

                 kprobatmp = istore(ibuf+1:ibuf+inpelm)
                 if ( debug ) &
                    call prinin ( istore(ibuf+1), inpelm, 'istore' )
                 ibuf = ibuf+inpelm

               end if
               if ( debug ) call prinin ( kprobatmp, inpelm, 'kproba' )

            end if

            if ( ndtvec>0 ) then

!           --- ndtvec>0, fill kprob part e

               if ( indtvc>0 ) then

!              --- indtvc>0

                  if ( type922found ) indtvc = min(indtvc,ndtvec-1)
                  do ivec = 1, indtvc
                     kprobe(ih1+1:ih1+inpelm,ivec) = &
                        istore(ibuf+1:ibuf+inpelm)
                     ibuf = ibuf+inpelm
                  end do

               end if

               if ( indtvc<ndtvec ) &
                  kprobe(ih1+1:ih1+inpelm,indtvc+1:ndtvec) = 0
               if ( type922found .and.ielgrp<=nelgrp ) then

!              --- type922found and internal element
!                  fill krobe with the same information as the solution vector
!                  Boundary elements are skipped

                  if ( junkpt/=0 ) then

!                 --- constant number of unknowns per point

                     kprobe(ih1+1:ih1+inpelm,ndtvec) = inunkp

                  else

!                 --- not constant number of unknowns per point


                     kprobe(ih1+1:ih1+inpelm,ndtvec) = kprobatmp(1:inpelm)

                  end if

               end if

            end if
            if ( nphys>0 ) then

!           --- nphys>0  store information in kprobptmp  (temporary array)

               do i = 1, iphys*inpelm
                  kprobptmp(ikprbp+i) = istore(i+ibuf)
               end do
               ibuf = ibuf+iphys*inpelm
               do i = iphys*inpelm+1,nphys*inpelm
                  kprobptmp(ikprbp+i) = 0
               end do

               if ( iphys==0 ) then

!              --- iphys = 0, no physical degrees of freedom are given
!                  explicitly. Use the available degrees of freedom

                  if ( junkpt==0 ) then

!                 --- junkpt = 0, variable number of degrees of freedom per
!                     point

                     do i = 1, inpelm
                        do j = 1, kprobatmp(i)
                           kprobptmp(ikprbp+i+(j-1)*inpelm) = j
                        end do
                     end do

                  else

!                 --- junkpt>0, constant number of degrees of freedom per
!                     point

                     do i = 1, inpelm
                        do j = 1, inunkp
                           kprobptmp(ikprbp+i+(j-1)*inpelm) = j
                        end do
                     end do

                  end if

               end if

               ikprbp = ikprbp+nphys*inpelm

            end if

         end if

         ih1 = ih1+inpelm

      end do
      if ( indpra>0 ) then

!     --- junkpt = 0, check if kproba may be removed

         junkpt = 1
         do ielgrp = 1, ns

            if ( associated(kproba(ielgrp)%arr) ) then

!           --- kprobatmp exists

               kprobatmp => kproba(ielgrp)%arr
               inpelm = ielshape(1,ielgrp)

               do i = 1, inpelm

                  if ( kprobatmp(i)/=nunkp ) then
                     junkpt = 0
                     exit
                  end if  ! ( kprobatmp(i)/= nunkp )

               end do  ! i = 1, inpelm

            end if  ! ( associated(kproba(i)%arr) )

            if ( junkpt==0 ) exit

         end do  ! ielgrp = 1, ns

         if ( junkpt==1 ) then

!        --- Remove kproba

            do i = 1, ns
               if ( associated(kproba(i)%arr) ) &
                  deallocate ( kp(iseqkp)%kprobsub(iprob)%kproba(i)%arr )
            end do  ! i = 1, ns
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kproba )
            kprobloc(14) = 0
            indpra = 0

         end if  ! ( junkpt==1 )

      end if  ! ( junkpt==0 )

      if ( ndtvec>0 ) kprobe2 = 0 ! clear last part of part e

      if ( debug ) call prinin ( kprobe, size(kprobe), 'kprobe' )

!     --- Fill kprob part j

      do ibngrp = 1, numnat
         kprobj(ibngrp,1) = ielshape(1,ibngrp+nelgrp)
         kprobj(ibngrp,2) = ielshape(2,ibngrp+nelgrp)
      end do

!     --- fill information for boundary conditions of the type u uequals
!         unknown constant

      do ibngrp = numnat+1, numnat+nbcons
         kprobc(nelgrp+ibngrp) = -2
         kprobj(ibngrp,1) = 2
         kprobj(ibngrp,2) = 1
         if ( ntype>1 ) then
            ih = ibngrp+nelgrp
            do i = 1, ntype
               kprobd(ih+(i-1)*(nelgrp+numnat+nbcons)) = -2
            end do
         end if
      end do

      if ( debug ) write(irefwr,*) 'End init06'
      call erclos ( 'init06' )

      end
