      subroutine init07srf ( iinp, ielshape, numsrf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill number of nodes per element and element shapes for
!     boundary groups of type surface in array ielshape
!     New input
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_element
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, ielshape(2,*), numsrf

!     ielshape      i/o   Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
!     iinp          i/o   last position used in array iinputkp
!     numsrf         i    number of parts where surfaces are given for
!                         generation of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, ibngrp, inpelm, i, ielem, ielem1, ielgrp, &
              istart, inum, ifirst, shapenpelm(7), isurnr

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     ielem          Element number
!     ielem1         First element number in surface
!     ielgrp         element group number
!     ifirst         First curve (surface) number read
!     inpelm         number of points per element
!     inum           Number of surfaces
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     j              Counting variable
!     shapenpelm     Array of size 7 containing the shape numbers as
!                    function of npelm-2 (surfaces only)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     271   boundary element group number has wrong value
!    1083   Shape number for boundary group different from before
!    2751   This number of nodes per element in surface is not programmed yet
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data shapenpelm / 3, 5, 0, 4, 7, 0, 6 /
! ======================================================================
!
      call eropen ( 'init07srf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from init07srf'
         write(irefwr,1) 'iinp, numsrf ', iinp, numsrf
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- numsrf>0  surface boundary elements

      do j = 1, numsrf

         ibngrp = iinputkp(iinp+1)
         if ( ibngrp<=0 .or. ibngrp>numnat ) then
            call errint ( ibngrp, 1 )
            call errint ( numnat, 2 )
            call errint ( j, 3 )
            call errchr ( 'surface', 1 )
            call errsub ( 271, 3, 0, 1 )
         end if
         inum = iinputkp(iinp+2)
         ifirst = iinputkp(iinp+3)
         if ( debug ) write(irefwr,1) 'j, ibngrp, inum', j, ibngrp, inum

         if ( kelmn>0 ) then

!        --- kmesh part n exists

            inpelm = kmeshn(ifirst)

         else

!        --- kmesh part n does not exist, use kmesht

            if  ( kelma==0 ) then

!           --- kelma = 0, all groups have the same number of nodes

               inpelm = npelm

            else

!           --- More element groups

               istart = 0
               ielem  = 0
               ielem1 = kmesht(1,ifirst)
               do ielgrp = 1, nelgrp
                  inpelm = Topgrp(ielgrp)%inpelm
                  inelem = Topgrp(ielgrp)%inelem
                  if ( ielem+inelem>=ielem1 ) go to 100
                  ielem = ielem+inelem
                  istart = istart+inpelm*inelem
               end do

100         end if

         end if

         if ( kelms>0 ) then

!        --- kelms> 0, i.e. kmeshs2 exists

            ishape = kmeshs2(2,ifirst)
            do i = 1, inum
               isurnr = iinputkp(iinp+2+i)
               if ( ishape/=kmeshs2(2,isurnr) ) then
                  call errint ( ibngrp, 1 )
                  call errint ( ishape, 2 )
                  call errint ( ielshape(2,ibngrp+nelgrp), 3 )
                  call errint ( j, 4 )
                  call errchr ( 'surface', 1 )
                  call errsub ( 1083, 4, 0, 1 )
               end if
            end do

         else

!        --- kelms> 0, i.e. kmeshs2 does not exist

            if ( inpelm<3 .or. inpelm==5 .or. inpelm==8 .or. &
                 inpelm>9 ) then

!           --- inpelm has wrong value

               call errint ( inpelm, 2 )
               call errint ( ifirst, 1 )
               call errsub ( 2751, 2, 0, 0 )
               go to 1000

            end if  ! ( inpelm<3 .or. inpelm>9 )
            ishape = shapenpelm(inpelm-2)

         end if  ! ( kelms>0 )

         if ( ielshape(2,ibngrp+nelgrp)/=0 ) then

!        --- check if shapes coincide

            if ( ielshape(2,ibngrp+nelgrp)/=ishape ) then
               call errint ( ibngrp, 1 )
               call errint ( ishape, 2 )
               call errint ( ielshape(2,ibngrp+nelgrp), 3 )
               call errint ( i, 4 )
               call errchr ( 'surface', 1 )
               call errsub ( 1083, 4, 0, 1 )
            end if

         end if
         ielshape(1,ibngrp+nelgrp) = inpelm
         ielshape(2,ibngrp+nelgrp) = ishape
         iinp = iinp+2+inum

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp', iinp
         write(irefwr,*) 'End init07srf'

      end if  ! ( debug )
      call erclos ( 'init07srf' )

      end
