      subroutine prinposunknown ( npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 06-04-2005 Part of body --> prfillposunk
!        version  1.0  date 24-01-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    print the position of an unknown with respect to the nodes and also
!    renumbered with respect to the matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint

!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iwork(4,nusol)

!     i              Counting variable
!     iwork          integer work array of size 4*nusol
!                    In this array per unknown the following information
!                    is stored
!                    (1,.) node corresponding to unknown in standard seq
!                    (2,.) local degree of freedom in node corresponding
!                          to unknown in standard sequence
!                    (3,.) node corresponding to unknown if numbering
!                          with respect to matrix is used
!                    (4,.) local degree of freedom in node corresponding
!                          to unknown if numbering
!                          with respect to matrix is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRFILLPOSUNK   Fill array iwork with positions of an unknown with respect
!                    to the nodes and also renumbered with respect to the matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set constants
!     if ( indprp>0 ) then
!        use kprobp to fill iwork
!     else if ( indprf>0 ) then
!        use kprobf to fill iwork
!     else
!        fill iwork
!     Print the contents of iwork
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinposunknown' )

!     --- Fill array iwork with information

      call prfillposunk ( npoint, iwork )

!     --- Print the contents of iwork

      if ( indprh==0 ) then

!     --- indprh = 0, no renumbering

         write(irefwr,100) (i,iwork(1,i),iwork(2,i),i=1,nusol)
100      format ( '    Unknown    node-nr   degfd '/(3i10) )

      else

!     --- indprh>0, renumbering

         write(irefwr,110) (i,iwork(1,i),iwork(2,i), &
                            iwork(3,i),iwork(4,i),i=1,nusol)
110      format ( '             in solution vector     in matrix'/ &
                  '    Unknown    node-nr   degfd   node-nr   degfd '/ &
                  (5i10) )

      end if

      call erclos ( 'prinposunknown' )

      end subroutine prinposunknown
