      subroutine printprobinp07 ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the local transforms part of array
!     iinput with respect to problem definition
!     Raise iinp
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     problem_definition
!     input
!     local_transform
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   next position used in array iinput
!     iinput         i    integer input array to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numcrv, numsrf, ifirst, itrans, ncurvsact, &
              numloccont, numlocobst, iskipboundary, itang, numcurvs, &
              inpelm, jinp, maxtrn

!     i              Counting variable
!     ifirst         First curve (surface) number read
!     inpelm         number of points per element
!     iskipboundary  Defines whther boundaries must be skipped
!     itang          Defines how the first tangential vector must be
!                    defined
!                    0: arbitrary
!                    1: By a line through points p1 and p2
!     itrans         Indicates type of transformation
!     jinp           last position used in array iinput with respect to
!                    iprec
!     maxtrn         Maximum number of real variables per point with
!                    respect to the local transformations
!     ncurvsact      Actual number of curves to be used
!     numcrv         number of parts where curves are given for local
!                    transformations
!     numcurvs       Number of curves
!     numloccont     Number of contacts with local transforms
!     numlocobst     Number of obstacles with local transforms
!     numsrf         number of parts where surfaces are given for
!                    local transformations
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINTPOSINT    Print value of positive integer if>0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printprobinp07' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printprobinp07'
         write(irefwr,1) 'iinp', iinp
         call prinin ( iinput(iinp), 40, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      maxtrn = 9
      jinp = 0

      if ( iinput(9)==0 ) go to 1000

      write(irefwr,*) 'local transforms'

      numcrv = iinput(iinp)
      numsrf = iinput(iinp+1)
      if ( debug ) write(irefwr,1) 'numcrv, numsrf',numcrv, numsrf
      iinp = iinp+2

      if ( numcrv>0 ) then

         write(irefwr,*) 'on curves'

         do i = 1, numcrv

            call printposint ( iinput(iinp), 'jdegfd1' )
            call printposint ( iinput(iinp+1), 'jdegfd2' )

            if ( ndim==2 ) then
               call printposint ( iinput(iinp+2), 'jstep' )
               call printposint ( iinput(iinp+3), 'ifirst' )
               call printposint ( iinput(iinp+4), 'ilast' )
               itrans = iinput(iinp+5)
               call printposint ( itrans, 'itrans' )
               iinp = iinp+6
               if ( itrans==2 ) then
                  call prinin ( iinput(iinp), 4, 'entries of matrix' )
                  iinp = iinp+4
               else if ( itrans==3 ) then
                  call prinin ( iinput(iinp), 8, 'entries of matrix' )
                  iinp = iinp+8
               end if  ! ( itrans==2 )

            else

               call printposint ( iinput(iinp+2), 'jdegfd3' )
               call printposint ( iinput(iinp+3), 'jstep' )
               call printposint ( iinput(iinp+4), 'ifirst' )
               call printposint ( iinput(iinp+5), 'ilast' )
               itrans = iinput(iinp+6)
               call printposint ( itrans, 'itrans' )
               iinp = iinp+7
               if ( itrans==2 ) then
                  call prinin ( iinput(iinp), 9, 'entries of matrix' )
                  iinp = iinp+9
               else if ( itrans==3 ) then
                  call prinin ( iinput(iinp), 16, 'entries of matrix' )
                  iinp = iinp+16
               end if  ! ( itrans==2 )

            end if  ! ( ndim==2 )

         end do  ! i = 1, numcrv

      end if  ! ( numcrv>0 )

      if ( numsrf>0 ) then

         write(irefwr,*) 'on surfaces'

         do i = 1, numsrf

            write(irefwr,1) 'surface', i
            if ( debug ) write(irefwr,1) 'iinp', iinp
            call printposint ( iinput(iinp), 'jdegfd1' )
            call printposint ( iinput(iinp+1), 'jdegfd2' )
            call printposint ( iinput(iinp+2), 'jdegfd3' )
            ifirst = iinput(iinp+3)
            inpelm = kmeshn(ifirst)
            call printposint ( ifirst, 'ifirst' )
            call printposint ( iinput(iinp+4), 'ilast' )
            itrans = iinput(iinp+5)
            call printposint ( itrans, 'itrans' )
            ncurvsact = 0
            iskipboundary = mod(iinput(iinp+6),10)
            call printposint ( iinput(iinp+6), 'iskipnormal' )
            itang = iinput(iinp+7)
            call printposint ( itang, 'itang' )
            if ( mod(iinput(iinp+6),10)>1 ) ncurvsact = -1
            numcurvs = iinput(iinp+inpelm+8)
            iinp = iinp + 8
            if ( debug ) &
               write(irefwr,1) 'itang, iinp', itang, iinp
            call printposint ( numcurvs, 'numcurvs' )

            if ( itang==1 ) then
               call prinin ( iinput(iinp), 2, 'user points' )
               if ( debug ) &
                  write(irefwr,1) 'iinp after user points', iinp

            else if ( itang>1 .and. itang<5 ) then
               if ( debug ) write(irefwr,1) 'iinp itang/1', iinp
            else if ( itang==10 ) then
               numcurvs = iinput(iinp)
               if ( debug ) write(irefwr,1) 'numcurvs', numcurvs
               call prinin ( iinput(iinp+1), numcurvs, 'curves' )

               if ( debug ) then
                  write(irefwr,1) 'iinp, inpelm', iinp, inpelm
                  call prinin ( iinput(iinp+inpelm), &
                                max(2,1+iinput(iinp+inpelm)), &
                                'iinput' )
               end if  ! ( debug )
            end if  ! ( itang==1 )

            if ( debug ) &
               write(irefwr,1) 'iskipboundary, iinp', &
                                iskipboundary, iinp

            if ( iskipboundary>=1 ) then
               numcurvs = iinput(iinp)
               call prinin ( iinput(iinp+1), numcurvs, &
                             'curves to be skipped' )
               if ( iskipboundary>1 ) then
                  jinp = iinp+inpelm
                  if ( itrans==2 ) then
                     jinp = jinp+9
                     maxtrn = 9
                  else if ( itrans==3 ) then
                     jinp = jinp+18
                     maxtrn = 18
                  end if
                  if ( debug ) &
                     write(irefwr,1) 'iinp, jinp, inpelm, maxtrn', &
                                      iinp, jinp, inpelm, maxtrn
                  if ( itang>0 .and. itang<10 ) then
                     jinp = jinp+2
                  else if ( itang>9 ) then
                     jinp = jinp+1+numcurvs
                  end if  ! ( itang>0 .and. itang<10 )
                  if ( debug ) &
                     write(irefwr,1) 'itang, jinp, iinp', itang, jinp, iinp
                  ncurvsact = iinput(jinp+1)
                  jinp = jinp+2
                  if ( debug ) &
                     write(irefwr,1) 'ncurvsact, jinp', ncurvsact, jinp
               else
                  jinp = iinp
               end if  ! ( iskipboundary>1 ) then

               if ( debug ) &
                  write(irefwr,1) 'itang, iinp, jinp', itang, iinp, jinp

            end if  ! ( iskipboundary>1 )
            if ( debug ) &
               write(irefwr,1) 'iinp, jinp after iskipbn', iinp, jinp

            if ( ncurvsact==-1 ) then

!           --- ncurvsact = -1, i.e. there are curves to be excluded or included

               ncurvsact = iinput(iinp)
               call prinin ( iinput(iinp+1), ncurvsact, &
                             'curves to be exluded' )
               iinp = iinp + ncurvsact+1

            end if  ! ( ncurvsact==-1 )

            if ( debug ) &
               write(irefwr,1) 'iskipboundary, itang, iinp', &
                                iskipboundary, itang, iinp

            if ( iskipboundary>0 ) &
               call prinin ( iinput(iinp), inpelm, 'skip nodes' )
            iinp = iinp+inpelm
            if ( debug ) write(irefwr,1) 'iinp', iinp

            if ( itrans==2 ) then
               call prinin ( iinput(iinp), 9, 'entries of matrix' )
               iinp = iinp+9
            else if ( itrans==3 ) then
               call prinin ( iinput(iinp), 16, 'entries of matrix' )
               iinp = iinp+16
            end if  ! ( itrans==2 )
            if ( debug ) write(irefwr,1) 'iinp', iinp

            if ( itang>0 .and. itang<10 ) then
               iinp = iinp+2
            else if ( itang>9 ) then
               iinp = iinp+1+numcurvs
            end if  ! ( itang>0 .and. itang<10 )

            if ( debug ) &
               write(irefwr,1) 'iinp, itrans, inpelm, itang, ncurvsact', &
                                iinp, itrans, inpelm, itang, ncurvsact
            if ( ncurvsact>0 ) iinp = iinp+1+ncurvsact
            if ( debug ) write(irefwr,1) 'iinp', iinp

         end do  ! i = 1, numsrf

      end if  ! ( numsrf>0 )

      numlocobst = iinput(18)
      if ( debug ) write(irefwr,1) 'numlocobst', numlocobst

      if ( numlocobst>0 ) then

         write(irefwr,*) 'on obstacles'

         do i = 1, numlocobst

            call printposint ( iinput(iinp), 'jdegfd1' )
            call printposint ( iinput(iinp+1), 'jdegfd2' )
            call printposint ( iinput(iinp+3), 'jstep' )
            call printposint ( iinput(iinp+4), 'iobst' )
            call printposint ( iinput(iinp+5), 'itrans' )
            iinp = iinp+6

         end do  ! i = 1, numlocobst

      end if  ! ( numlocobst>0 )

      numloccont = iinput(21)

      if ( numloccont>0 ) then

         write(irefwr,*) 'on contacts'

         do i = 1, numloccont

            call printposint ( iinput(iinp), 'jdegfd1' )
            call printposint ( iinput(iinp+1), 'jdegfd2' )
            call printposint ( iinput(iinp+3), 'jstep' )
            call printposint ( iinput(iinp+4), 'icontact' )
            call printposint ( iinput(iinp+5), 'itrans' )
            iinp = iinp+6

         end do  ! i = 1, numloccont

      end if  ! ( numloccont>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp', iinp
         write(irefwr,*) 'End printprobinp07'

      end if  ! ( debug )
      call erclos ( 'printprobinp07' )

      end
