      subroutine printprobinput ( iinput, nprob, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array iinput with respect to problem
!     definition
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     problem_definition
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), nprob
      double precision rinput(*)

!     iinput         i    integer input array array to be printed
!                         This is the input array for subroutine probdf
!     nprob          i    Number of various problems
!     rinput         i    Real input array to be filled by PROREA:
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iprob, iinp, iinpprob

!     iinp           next position used in array iinput for each problem
!     iinpprob       Starting address in iinput for present problem
!     iprob          Actual problem number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINPROPRERINP print the contents of array iinput for periodical
!                    boundary conditions
!     PRINTPROBINP01 Print the contents of the first part of array iinput
!                    with respect to problem definition
!     PRINTPROBINP02 Print the contents of the part type numbers of array
!                    iinput with respect to problem definition
!     PRINTPROBINP03 Print the contents of the natural boundary coditions part
!                    of array iinput with respect to problem definition
!     PRINTPROBINP04 Print the contents of the global elements part of array
!                    iinput with respect to problem definition
!     PRINTPROBINP05 Print the contents of the fictitious elements part of array
!                    iinput with respect to problem definition
!     PRINTPROBINP06 Print the contents of the essential boundary coditions
!                    part of array iinput with respect to problem definition
!     PRINTPROBINP07 Print the contents of the local transforms part of array
!                    iinput with respect to problem definition
!     PRINTPROBINP08 Print the contents of the unknown constants part of array
!                    iinput with respect to problem definition
!     PRINTPROBINP09 Print the contents of the renumbering part of array
!                    iinput with respect to problem definition
!     PRINTPROBINP10 Print the contents of the skip elements part of array
!                    iinput with respect to problem definition
!     PRINTPROBINP11 Print the contents of the level set part of array
!                    iinput with respect to problem definition
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printprobinput' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printprobinput'
         call prinin ( iinput, 10, 'iinput')
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iinpprob = 1
      do iprob = 1, nprob
         write(irefwr,*)
         write(irefwr,1) ' *** Problem', iprob
         write(irefwr,*)
         iinp = 1

!        --- Print the contents of the first part of iinput

         call printprobinp01 ( iinput(iinpprob), iinp )
         if ( debug ) &
            write(irefwr,1) 'iinploc, iinpglob', iinp, iinpprob+iinp-1

!        --- Print the contents of the part type numbers

         call printprobinp02 ( iinput(iinpprob), iinp )
         if ( debug ) &
            write(irefwr,1) 'iinploc, iinpglob', iinp, iinpprob+iinp-1

!        --- Print the contents of the part natural boundary coditions

         call printprobinp03 ( iinput(iinpprob), iinp )
         if ( debug ) &
            write(irefwr,1) 'iinploc, iinpglob', iinp, iinpprob+iinp-1

!        --- Print the contents of the part global elements

         call printprobinp04 ( iinput(iinpprob), iinp )
         if ( debug ) &
            write(irefwr,1) 'iinploc, iinpglob', iinp, iinpprob+iinp-1

!        --- Print the contents of the part fictitious elements

         call printprobinp05 ( iinput(iinpprob), iinp )
         if ( debug ) &
            write(irefwr,1) 'iinploc, iinpglob', iinp, iinpprob+iinp-1

!        --- Print the contents of the part essential boundary coditions

         call printprobinp06 ( iinput(iinpprob), iinp )
         if ( debug ) &
            write(irefwr,1) 'iinploc, iinpglob', iinp, iinpprob+iinp-1

!        --- Print the contents of the part local transforms

         call printprobinp07 ( iinput(iinpprob), iinp )
         if ( debug ) &
            write(irefwr,1) 'iinploc, iinpglob', iinp, iinpprob+iinp-1

!        --- Print the contents of the part unknown constants

         call printprobinp08 ( iinput(iinpprob), iinp )
         if ( debug ) &
            write(irefwr,1) 'iinploc, iinpglob', iinp, iinpprob+iinp-1

!        --- Print the contents of the renumbering part

         call printprobinp09 ( iinput(iinpprob), iinp )
         if ( debug ) &
            write(irefwr,1) 'iinploc, iinpglob', iinp, iinpprob+iinp-1

!        --- Print the contents of the skip elements part

         call printprobinp10 ( iinput(iinpprob), iinp )
         if ( debug ) &
            write(irefwr,1) 'iinploc, iinpglob', iinp, iinpprob+iinp-1

!        --- Print the contents of the level set part

         call printprobinp11 ( iinput(iinpprob), iinp )
         if ( debug ) &
            write(irefwr,1) 'iinploc, iinpglob', iinp, iinpprob+iinp-1

!        --- Print the contents of the part periodical bounday conditions

         if ( iinput(iinpprob+51)>0 ) then

            write(irefwr,*) 'Periodical boundary conditions'
            call prinproprerinp ( iinput(iinpprob+iinp-1), rinput, &
                                  length )
            iinp = iinp+length
            if ( debug ) &
               write(irefwr,1) 'iinploc, iinpglob', &
                                iinp, iinpprob+iinp-1

         end if  ! ( iinput(iinprob+51)>0 )

         iinpprob = iinpprob+iinp-1
         if ( debug ) write(irefwr,1) 'iinpprob', iinpprob

      end do  ! iprob = 1, nprob

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printprobinput'

      end if  ! ( debug )
      call erclos ( 'printprobinput' )

      end
