      subroutine prob01const ( numpnt, numcrv, numsrf, &
                               numobstlc, natconct, natconpt, &
                               natnocel, natnocpt, natcrpnt, natsrpnt, &
                               natobstcross, natlevset )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-03-2012 Extra parameter ivers
!        version  1.0  date 17-05-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill constants for subroutine prob01 (natural boundary conditions)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     boundary_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numpnt, numcrv, numsrf, &
              numobstlc, natconct, natconpt, &
              natnocel, natnocpt, natcrpnt, natsrpnt, &
              natobstcross, natlevset

!     natconct       o    Number of contacts where natural boundary elements
!                         are given
!     natconpt       o    number of parts where contact points are defined as
!                         bound els.
!     natcrpnt       o    Number of curves where point boundary elements
!                         are given
!     natlevset      o    number of parts where zero level set elements
!                         are defined
!     natnocel       o    number of parts where no_contact elements are defined
!     natnocpt       o    number of parts where no_contact points are defined as
!                         boundary elements
!     natobstcross   o    number of parts where cross_section_obstacle elements
!                         are defined
!     natsrpnt       o    Number of surfaces where point boundary elements are
!                         given
!     numcrv         o    number of parts where curves are given for generation
!                         of boundary elements
!     numobstlc        o    number of obstacles
!     numpnt         o    number of parts where points are given for generation
!                         of boundary elements
!     numsrf         o    number of parts where surfaces are given for
!                         generation of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob01const' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob01const'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numnat = iinputkp(2)
      numpnt = iinputkp(3)
      numcrv = iinputkp(4)
      numsrf = iinputkp(5)
      nbcons = iinputkp(13)
      numobstlc  = iinputkp(17)
      natconct = iinputkp(20)
      natconpt = iinputkp(23)
      natnocel = iinputkp(24)
      natnocpt = iinputkp(25)
      natcrpnt = iinputkp(26)
      natsrpnt = iinputkp(27)
      natobstcross = iinputkp(34)
      natlevset = iinputkp(55)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numnat, numpnt, numcrv, numsrf, nbcons', &
                          numnat, numpnt, numcrv, numsrf, nbcons
         write(irefwr,1) 'numobstlc, natconct, natconpt, natnocel', &
                          numobstlc, natconct, natconpt, natnocel
         write(irefwr,1) 'natnocpt, natcrpnt, natsrpnt, natobstcross', &
                          natnocpt, natcrpnt, natsrpnt, natobstcross
         write(irefwr,1) 'natlevset', natlevset
         write(irefwr,*) 'End prob01const'

      end if  ! ( debug )
      call erclos ( 'prob01const' )

      end
