      subroutine prob01crobst1 ( kcrosselem, natobstcross, &
                                 iposgrp, iworkcross, &
                                 nintersect, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-03-2012 New input
!        version  1.0  date 13-12-2000
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill the elements corresponding to cross sections with an
!    obstacle into kprobi and update kprobz
! **********************************************************************
!
!                       KEYWORDS
!
!     fictitious_domain
!     element
!     crossing
!     cross_section
!     crossing_point
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kcrosselem(*), natobstcross, nintersect, &
              iposgrp(*), iworkcross(*), iinp

!     iinp          i/o   last position used in array iinputkp
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!                         In this subroutine the length with respect to an
!                         element group is added to the existing array
!     iworkcross     i    integer work array of length natobstcross that
!                         contains the number of elements per set of boundary
!                         elements corresponding to the intersections
!     kcrosselem     i    array of size nfictunk x 2 with the following contents
!                         kcrosselem(i,1) contains the first node of the element
!                         kcrosselem(i,2) contains the second node of the
!                         element
!     kprobi        i/o   KPROB part i, contains nodal point numbers of
!                         boundary elements
!     kprobz        i/o   Kprob part z
!                         If filled kprob part z has length nelgrp+numnat
!                         For element group an memory management sequence
!     mmrcrosselem   i    memory management sequence number of array rcrosselem
!     natobstcross   i    number of parts where cross_section_obstacle elements
!                         are defined
!     nelgrp         i    number of element groups
!     nintersect     i    Number of intersections found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ibngrp, ikprbi, igrp, ibngrpbef

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     ibngrpbef      ?
!     igrp           Boundary element group number read
!     ikprbi         last position used in array kprob part i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy elements from kcrosselem into kprobi
!     For igrp := 1 (1) natobstcross do
!        adapt iposgrp and kprobz
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( kmeshad(4)<=0 ) go to 1000

!     --- Copy elements from kcrosselem into kprobi

      ibngrp = iinputkp(iinp+1)
      ikprbi = iposgrp(ibngrp)
      do i = 1, nintersect*2
         kprobi(ikprbi+i) = kcrosselem(i)
      end do

!     --- For igrp := 1 (1) natobstcross do

      ibngrpbef = 0
      inelem = 1
      do igrp = 1, natobstcross

!     --- adapt iposgrp and kprobz

         ibngrp = iinputkp(iinp+1)
         ikprbi = iposgrp(ibngrp)
         iinp = iinp+5+iinputkp(iinp+5)
         nintersect = iworkcross(igrp)
         if ( ibngrpbef/=ibngrp .and. ibngrpbef>0 ) then
            probzarr(nelgrp+ibngrp)%arr => probzarr(nelgrp+ibngrpbef)%arr
            kprobz(nelgrp+ibngrp) = -(nelgrp+ibngrpbef)
         end if  ! ( ibngrpbef/=ibngrp .and. ibngrpbef>0 )
         ikprbi = ikprbi+2*nintersect
         iposgrp(ibngrp) = iposgrp(ibngrp)+2*nintersect
         inelem = nintersect+inelem
         ibngrpbef = ibngrp

      end do

1000  end
