      subroutine prob03 ( numcrv, iinp, iposgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array kprob part i with boundary elements corresponding to
!     curve
!     This subroutine is the counterpart of subroutine PROB08, &
!     if PROB08 is changed also PROB03 should be changed and vice versa
!     New input
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numcrv, iinp, iposgrp(*)

!     iinp          i/o   last position used in array iinputkp
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!                         At input the last postions used for an element group
!                         are stored, at output they are updated
!     numcrv         i    number of parts where curves are given for generation
!                         of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenkmeshm3, i, inpelm, inum, &
              ikprbi, ibngrp, npointscurve, icurnr, j
      logical spectral

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     icurnr         Curve sequence number
!     ikprbi         last position used in array kprob part i
!     inpelm         number of points per element
!     inum           Number of curves
!     j              Counting variable
!     lenkmeshm3     Length of kmesh m part 3
!     npointscurve   number of points along a curve
!     spectral       If true spectral elements are used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PROBCURVELM    Fill elements along a curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set the parameter spectral
!     For all groups of curves with boundary conditions do
!        Get information from iinputkp
!        For all curves in group do
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob03' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob03'
  1      format ( a, 1x, (10i6) )

      end if

!     --- Set the parameter spectral

      if ( kelmu>0 ) then
         spectral = .true.
      else
         spectral = .false.
      end if
      lenkmeshm3 = kmeshm(3)-kmeshm(2)

!     --- For all groups of curves with boundary conditions do

      do j = 1, numcrv

!     --- Get information from iinputkp

         ibngrp = iinputkp(iinp+1)
         ikprbi = iposgrp(ibngrp)
         ishape = iinputkp(iinp+2)
         if ( ishape==0 ) then
            iinp = iinp+4
            goto 200
         end if
         if ( spectral ) ishape = 1+kmeshu(2)
         inpelm = ishape+1
         inum = iinputkp(iinp+3)
         if ( debug ) &
            write(irefwr,1) 'j, ibngrp, ishape, inpelm', &
                             j, ibngrp, ishape, inpelm

!        --- Loop over the curves

         do i = 1, inum

!        --- Compute elements and store in kprobi

            icurnr = iinputkp(iinp+3+i)
            call probcurvelm ( icurnr, npointscurve, kprobi, ikprbi, &
                               lenkmeshm3 )
            if ( debug ) then
               write(irefwr,*) 'icurnr, npointscurve, ibngrp, ikprbi ', &
                                icurnr, npointscurve, ibngrp, ikprbi
               call prinin1 ( kprobi(ikprbi+1-npointscurve*inpelm), &
                              npointscurve, inpelm, 'kprobi' )
            end if  ! debug

            iposgrp(ibngrp) = iposgrp(ibngrp)+npointscurve*inpelm

         end do
         iinp = iinp+3+inum

200   end do  ! j = 1, numcrv

      call erclos ( 'prob03' )

      end
