      subroutine prob04cont ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of nodes with essential boundary conditions for
!     contact
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nesconct, nesnocon, icontc

!     i              Counting variable
!     icontc         Sequence number of contact
!     nesconct       Number of contacts at which essential boundary
!                    conditions are given
!     nesnocon       Number of "no"-contacts with essential boundary
!                    conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04cont' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04cont'
         write(irefwr,1) 'nkbnd/1', nkbnd
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      maxcontc = kprobloc(48)
      nesconct = iinput(19)
      nesnocon = iinput(22)
      if ( debug ) &
         write(irefwr,1) 'maxcontc, nesconct, nesnocon, nsurfs', &
                          maxcontc, nesconct, nesnocon, nsurfs
      if ( kprobloc(46)>0 ) then
         kprobvp1 => kp(iseqkp)%kprobsub(1)%kprobvp1
         kprobvp2 => kp(iseqkp)%kprobsub(1)%kprobvp2
      end if  ! ( kprobloc(46)>0 )

      do i = 1, nesconct
         icontc = iinput(iinp+3)
         iinp = iinp+3
         if ( maxcontc>0 .and. kprobloc(46)>0 ) &
            nkbnd = nkbnd+kprobvp1(icontc,1)
         if ( debug ) &
            write(irefwr,1) 'i, icontc, nkbnd', i, icontc, nkbnd
      end do

      do i = 1, nesnocon
         icontc = iinput(iinp+3)
         iinp = iinp+3
         if ( maxcontc>0 .and. kprobloc(46)>0 ) &
            nkbnd = nkbnd+kprobvp1(icontc,2)
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nkbnd/2', nkbnd
         write(irefwr,*) 'End prob04cont'

      end if  ! ( debug )
      call erclos ( 'prob04cont' )

      end
