      subroutine prob04cv1 ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-03-2015 extra debug
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for a curve
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivers, numcrv

!     ivers          Version sequence number
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROB04CVNEW    Compute the number of nodal points with prescribed boundary
!                    conditions for a set of curves
!     PROB04CVOLD    Compute the number of nodal points with prescribed boundary
!                    conditions for curves icrv1 to icrv2
!     SEPFILLOUTERB  Fill outer boundary of mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04cv1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04cv1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      if ( mmouter==0 ) call sepfillouterb

!     --- Actual call

      if ( ierror/=0 ) go to 1000

      numcrv = iinput(7)

      if ( numcrv==0 ) go to 1000  ! jump if no curves are used

      ivers = iinput(15)
      if ( debug ) write(irefwr,*) 'ivers', ivers

      if ( ivers<9 ) then

!     --- Old version

         call prob04cvold ( iinput, iinp, nkbnd )

      else

!     --- New version

         call prob04cvnew ( iinput, iinp, nkbnd )

      end if  ! ( ivers<9 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04cv1'

      end if  ! ( debug )
      call erclos ( 'prob04cv1' )

      end
