      subroutine prob04cvstep ( jstep, iexclu, npointscurve, found, &
                                nodes, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     perform the jstep part of prob04cvold/new
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jstep, iexclu, npointscurve, nodes(*), nkbnd
      logical found

!     found         i/o   If true a warning has been found in a curve
!     iexclu         i    Indicates which points are excluded
!     jstep          i    Step in loop
!     nkbnd         i/o   Number of points with essential boundary conditions
!     nodes         i/o   Contains the node numbers that must be filled
!     npointscurve   i    number of points along a curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kstart, kend, k, kstep, inodp, j

!     inodp          Nodal point number
!     j              Counting variable
!     k              Counting variable
!     kend           End value of loop
!     kstart         Start value of loop
!     kstep          Step in loop
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04cvstep' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04cvstep'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kstep = abs(jstep)+1

      if ( jstep>=0 ) then

!     --- jstep>=  0   end points of intervals
!         Correct for initial and end point in the case that exclude>0

         if ( iexclu==0 ) then

!        --- iexclu  =  0  No excluding of nodal points

            kstart = 1
            kend = npointscurve

         else if ( iexclu==1 ) then

!        --- iexclu  =  1 Exclude initial point

            kstart = 2
            kend = npointscurve

         else if ( iexclu==2 ) then

!        --- iexclu  =  2 Exclude end point

            kstart = 1
            kend = npointscurve-1

         else if ( iexclu==3 ) then

!        --- iexclu  =  3 Exclude initial and end point

            kstart = 2
            kend = npointscurve-1

         end if

         do k = kstart, kend, kstep

            inodp = nodes(k)
            if ( inodp<=0 ) then
               found = .true.
            else
               nkbnd = nkbnd+1
               if ( debug ) &
                  write(irefwr,1) 'nkbnd, inodp/1', nkbnd, inodp
            end if

         end do

      else

!     --- jstep <0      points between intervals

         npointscurve = npointscurve-1
         do k = 1, npointscurve, kstep
            do j = 1, kstep-1
               inodp = nodes(k)
               if ( inodp<=0 ) then
                  found = .true.
               else
                  nkbnd = nkbnd+1
                  if ( debug ) &
                     write(irefwr,1) 'nkbnd, inodp/2', &
                                      nkbnd, inodp
               end if
            end do
         end do

      end if

1000  call erclos ( 'prob04cvstep' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04cvstep'

      end if  ! ( debug )

      end
