      subroutine prob04obstcv ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for obstacle curves
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     curve
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jstep, iobst, numobstlc

!     i              Counting variable
!     iobst          Obstacle sequence number
!     jstep          Step in loop
!     numobstlc      number of obstacles
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROB19         Compute number of boundary conditions for one obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04obstcv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04obstcv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kmeshh(2,1)>0 ) then

!     --- Obstacles are present
!         Activate arrays


         numobstlc = iinput(16)

         do i = 1, numobstlc
            jstep = iinput(iinp+2)
            iobst = iinput(iinp+3)
            iinp = iinp+3
            if ( iactobst(iobst)>0 )  &
               call prob19 ( nkbnd, iobst, jstep, kmeshm, icurvsobst2d, &
                             ncurvs, iobscurv )
         end do

      end if  ! ( kmeshh(1)>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04obstcv'

      end if  ! ( debug )
      call erclos ( 'prob04obstcv' )

      end
