      subroutine prob05cont1 ( iconct, ibit )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential boundary
!     contact
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iconct, ibit

!     ibit           i    Actual bit pattern
!     iconct         i    Sequence number of contact
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ibit2, ih, inodp, inunkp, istart, j, nconpnt

!     ibit2          Maximal bit pattern
!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     istart         First position used at a part
!     j              Counting variable
!     nconpnt        Number of contact points for one contact
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      77   You try to prescribe the wrong degree of freedom
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05cont1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05cont1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inunkp = nunkp
      ibit2  = inibt0(inunkp)
      istart = kprobvp1(iconct,3)-1
      nconpnt = kprobvp1(iconct,1)
      do j = 1, nconpnt
         inodp = kprobvp2(istart+j)
         if ( indprf>0 ) then

!        --- indprf>0  kprob part f contains the number of unknowns

            inunkp = kprobf(inodp+1)-kprobf(inodp)
            ibit2 = inibt0(inunkp)

         end if

         if ( ibit>ibit2 .or. ibit<0 ) then
            call errint ( ibit, 1 )
            call errint ( ibit2, 2 )
            call errint ( inodp, 3 )
            call errwar ( 77, 3, 0, 0 )
         end if

         nkbnd = nkbnd+1
         ih = ibit
         if ( ih==0 ) ih = ibit2
         kprobg(1,nkbnd) = inodp
         kprobg(2,nkbnd) = ih

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05cont1'

      end if  ! ( debug )
      call erclos ( 'prob05cont1' )

      end
