      subroutine prob05cv ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 12-04-2012 Split subroutine
!        version  3.2  date 22-09-2007 Debug information
!        version  3.1  date 14-01-2005 Debug statements
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential
!     boundary conditions for a curve
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iexclu, inum, jstep, kstart, numcrv, &
              npointscurve, ibit2, ibit, nparts, inunkp, &
              nkbndbefore, npointlc
      integer, allocatable :: nodes(:), ipoint(:)

!     i              Counting variable
!     ibit           Actual bit pattern
!     ibit2          Maximal bit pattern
!     iexclu         Indicates which points are excluded
!     inum           Number of curves in input
!     inunkp         Number of degrees of freedom in point
!     ipoint         Array of length nparts+1 containing the starting
!                    addresses in array inodes of the various parts
!                    the boundary consists of.
!                    ipoint(1) = 1, &
!                    number of nodes at part i = ipoint(i+1)-ipoint(i)
!     jstep          Step in loop
!     kstart         Start value of parameter
!     nkbndbefore    Value of nkbnd at input
!     nodes          integer work array of length npoint to store a set of
!                    nodal points
!     nparts         number of parts the boundary consists of
!     npointlc       Number of nodal points
!     npointscurve   number of points along a curve
!     numcrv         number of parts where curves are given for generation
!                    of boundary conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIBT0         Compute bit pattern
!     PROB05CVOUTER  perform the outer boundary part of prob05cvold/new
!     PROB05CVSTEP   perform the jstep part of prob05cvold/new
!     SEPCURVENODES  Store all nodes in one curves in array inodes and gaps in
!                    these curves in array ipoint
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05cv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05cv'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numcrv = iinputkp(7)

      if ( numcrv==0 ) go to 1000  ! jump if no curves are used

      npointlc = max(npoint,kmeshm(3)-kmeshm(2))

      allocate ( nodes(npointlc), ipoint(ncurvs+1), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

      inunkp = nunkp
      ibit2  = inibt0(inunkp)
      if ( debug ) &
         write(irefwr,1) 'ncurvs, numcrv, ibit2', &
                          ncurvs, numcrv, ibit2

      do i = 1, numcrv
         ibit = iinputkp(iinp+1)
         jstep = iinputkp(iinp+2)
         inum = iinputkp(iinp+3)
         if ( debug ) &
            write(irefwr,1) 'i, ibit, jstep, inum', &
                             i, ibit, jstep, inum
         if ( inum>0 ) then

!        --- Standard case, curves from ifirst to ilast

            if ( jstep>0 ) then

!           --- jstep>0,  compute iexclu

               iexclu = jstep/100
               jstep = jstep-100*iexclu
            else
               iexclu = 0
            end if

            nparts = 0
            call sepcurvenodes ( iinputkp(iinp+3), ipoint, nodes, nparts, &
                                 kmeshm )
            npointscurve = ipoint(nparts+1)-1
            if ( ierror/=0 ) go to 1000

            call prob05cvstep ( jstep, iexclu, npointscurve, &
                                nodes, inunkp, ibit, iinp, kstart )

            iinp = iinp+inum+3

         else  ! ( inum = -1 )

!        --- Special case, all outer boundaries

            call prob05cvouter ( npointscurve, nodes, inunkp, ibit, kstart )
            iinp = iinp+3

         end if  ! ( inum>0 )

      end do  ! i = 1, numcrv

      deallocate ( nodes, ipoint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05cv'

      end if
      call erclos ( 'prob05cv' )

      end
