      subroutine prob05inters ( iinp, nodes, ipoint, &
                                nesintersect, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential
!     boundary conditions for intersections
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     essential_boundary_condition
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, &
              nesintersect, npoint, nodes(npoint,nesintersect), &
              ipoint(nesintersect)

!     iinp          i/o   last position used in array iinputkp
!     ipoint         i    Array of length nesintersect containing the
!                         number of nodes for each intersection
!     nesintersect   i    Number of intersections where essential boundary
!                         conditions are given
!     nodes          i    Array of size nesintersect x npoint
!                         nodes(i,1:ipoint(i)) contains the nodal points
!                         corresponding to intersection i
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numsrf, j, nnodes, inunkp, inodp, ibit, &
              ibit2, ih

!     i              Counting variable
!     ibit           Actual bit pattern
!     ibit2          Maximal bit pattern
!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     j              Counting variable
!     nnodes         Number of nodes
!     numsrf         number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      77   You try to prescribe the wrong degree of freedom
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05inters' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05inters'
         write(irefwr,1) 'nkbnd, nesintersect', nkbnd, iinputkp(56)
         write(irefwr,1) 'iinp, npoint', iinp, npoint
         call prinin ( iinputkp(iinp+1), 5, 'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inunkp = nunkp
      ibit2  = inibt0(inunkp)

      if ( debug ) &
         write(irefwr,1) 'inunkp, indprf, ibit2', inunkp, indprf, ibit2

      nesintersect = iinputkp(56)
      do  i = 1, nesintersect

         ibit = iinputkp(iinp+1)
         numsrf = iinputkp(iinp+3)       ! number of surfaces

         nnodes = ipoint(i)  ! number of nodes

         if ( debug ) write(irefwr,1) 'i, ibit, nnodes', i, ibit, nnodes

         do j = 1, nnodes

            inodp = nodes(j,i)

!           --- nodal point found

            if ( indprf>0 ) then

!           --- indprf>0  kprob part f contains the number of unknowns

               inunkp = kprobf(inodp+1)-kprobf(inodp)
               ibit2 = inibt0(inunkp)

            end if  ! ( indprf>0 )

            if ( ibit>ibit2 .or. ibit<0 ) then
               call errint ( ibit, 1 )
               call errint ( ibit2, 2 )
               call errint ( inodp, 3 )
               call errwar ( 77, 3, 0, 0 )
            end if

            ih = ibit
            if ( ih==0 ) ih = ibit2

            nkbnd = nkbnd+1
            kprobg(1,nkbnd) = inodp
            kprobg(2,nkbnd) = ih

         end do  ! j = 1, nnodes

         iinp = iinp+3+numsrf

      end do  ! i = 1, nesintersect

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kprobg, nkbnd, 2, 'kprobg' )
         write(irefwr,1) 'nkbnd/end', nkbnd
         write(irefwr,*) 'End prob05inters'

      end if  ! ( debug )
      call erclos ( 'prob05inters' )

      end
