      subroutine prob05obstcv ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential boundary
!     conditions for obstacle curves
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     curve
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jstep, iobst, numobstlc, ibit, ibit2

!     i              Counting variable
!     ibit           Actual bit pattern
!     ibit2          Maximal bit pattern
!     iobst          Obstacle sequence number
!     jstep          Step in loop
!     numobstlc      number of obstacles
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBT0         Compute bit pattern
!     PROB20         Fill essential boundary conditions for one obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05obstcv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05obstcv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000


      ibit2  = inibt0(nunkp)

      numobstlc = iinputkp(16)

      ibit2  = inibt0(nunkp)

      numobstlc = iinputkp(16)

      do i = 1, numobstlc
         ibit = iinputkp(iinp+1)
         jstep = iinputkp(iinp+2)
         iobst = iinputkp(iinp+3)
         iinp = iinp+3
         if ( iactobst(iobst)>0 ) &
            call prob20 ( iobst, jstep, icurvsobst2d, ncurvs, &
                          ibit, ibit2, kmeshm, iobscurv )
      end do

      do i = 1, numobstlc
         ibit = iinputkp(iinp+1)
         jstep = iinputkp(iinp+2)
         iobst = iinputkp(iinp+3)
         iinp = iinp+3
         if ( iactobst(iobst)>0 ) &
            call prob20 ( iobst, jstep, icurvsobst2d, ncurvs, &
                          ibit, ibit2, kmeshm, iobscurv )
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05obstcv'

      end if  ! ( debug )
      call erclos ( 'prob05obstcv' )

      end
