      subroutine prob05sf ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 03-04-2012 Split subroutine
!        version  4.0  date 27-09-2010 Make work arrays local
!        version  3.3  date 09-12-2009 Correction empty nodes
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential
!     boundary conditions for a surface
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isrf1, numsrf, i, inunkp

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     isrf1          First surface number
!     numsrf         number of parts where surfaces are given for
!                    generation of boundary conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PROB05SFNEW    Fill KPROB part g with information concerning the essential
!                    boundary conditions for a set of surfaces
!     PROB05SFOUTER  Fill KPROB part g with information concerning the essential
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05sf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05sf'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numsrf = iinputkp(8)
      inunkp = nunkp
      if ( debug ) write(irefwr,1) 'numsrf', numsrf

      do i = 1, numsrf

         isrf1 = iinputkp(iinp+2)

         if ( isrf1/=-1 ) then

!        --- Standard case

            call prob05sfnew ( iinp, inunkp )

         else  ! ( isrf1 = -1 )

!        --- Special case, fill all outer surfaces

            call prob05sfouter ( iinp, inunkp )

         end if  ! ( isrf1/=-1 )

      end do  ! i = 1, numsrf

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kprobg, nkbnd, 2, 'kprobg' )
         write(irefwr,*) 'End prob05sf'

      end if
      call erclos ( 'prob05sf' )

      end
