      subroutine prob05up ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential boundary
!     conditions for user points
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numpnt, j, npnt, i, ibit, ibit2, ih, inunkp, inodp

!     i              Counting variable
!     ibit           Actual bit pattern
!     ibit2          Maximal bit pattern
!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     j              Counting variable
!     npnt           Number of points
!     numpnt         number of parts where points are given for generation
!                    of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0, inibt1

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
!     INIBT1         Compute maximum unknown corresponding to bit pattern
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1851   Wrong unknown prescribed in user point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05up' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05up'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numpnt = iinputkp(6)
      inunkp = nunkp
      ibit2  = inibt0(inunkp)
      iinp   = intarprb(4)-1
      if ( debug ) then
         write(irefwr,1) 'numpnt, ibit2, iinp, inunkp', &
                          numpnt, ibit2, iinp, inunkp
      end if  ! ( debug )

      do j = 1, numpnt
         ibit = iinputkp(iinp+1)
         npnt = iinputkp(iinp+2)
         iinp = iinp+2
         do i = 1, npnt

            inodp = kmeshl(iinputkp(iinp+i))
            if ( inodp>0 ) then

!           --- inodp>0, hence node is present in mesh

               if ( indprf>0 ) then

!              --- indprf>0  kprob part f contains the number of unknowns

                  inunkp = kprobf(inodp+1)-kprobf(inodp)
                  ibit2 = inibt0(inunkp)

               end if
               if ( ibit>ibit2 .or. ibit<0 ) then
                  call errint ( inibt1(ibit), 1 )
                  call errint ( iinputkp(iinp+i), 2 )
                  call errint ( inunkp, 3 )
                  call errwar ( 1851, 3, 0 , 0 )
               end if
               nkbnd = nkbnd+1
               ih = ibit
               if ( ih==0 ) ih = ibit2
               kprobg(1,nkbnd) = inodp
               kprobg(2,nkbnd) = ih
               if ( debug ) &
                  write(irefwr,1) 'nkbnd, inodp', nkbnd, inodp

            end if  ! ( inodp>0 )

         end do  ! i = 1, npnt
         iinp = iinp+npnt

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05up'

      end if  ! ( debug )
      call erclos ( 'prob05up' )

      end
