      subroutine prob09s1 ( icurvind, iwork, iskipbn, ncurvsact, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 30-07-2012 correction negative curves
!        version  2.3  date 03-03-2006 Extra debug statements
!        version  2.2  date 23-08-2003 Use positive curve numbers
!
!   copyright (c) 1998-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all points on external curves with a -1 in iwork
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvind(*), iwork(*), iskipbn, &
              ncurvsact, icurvs(*)

!     icurvind       i    Integer work array of length ncurvs that is filled as
!                         follows
!                         icurvind(i) = -1, at the boundary of the surfaces
!                         icurvind(i) = 1, or not in the surfaces icurvind(i)=0
!     icurvs         i    array containing the number of points in the curves
!                         accumulated
!     iskipbn        i    If 1 the outer boundary of the surfaces must be
!                         skipped
!     iwork         i/o   integer work array of length npoint used by prob09sf
!                         iwork(i) = 0, means node has not yet been treated
!                         iwork(i) = -1 means node belongs to outer boundary
!                         At input array iwork is completely zero
!     ncurvsact      i    Actual number of curves to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurnr, istart, istar3, nnodes, jcurnr, kcurnr, &
              isubcv, nsubcv, inode

!     i              Counting variable
!     icurnr         Curve sequence number
!     inode          Node sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     isubcv         Subcurve sequence number with direction
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     kcurnr         abs(jcurnr)
!     nnodes         Number of nodes
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For icurnr := 1 (1) ncurvs do
!        if ( icurnr = external ) then
!           for all nodes nodp on curve icurnr do
!              iwork(nodp):=-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob09s1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob09s1'
         write(irefwr,1) 'iskipbn', iskipbn
  1      format ( a, 1x, (10i6) )
         call pr0008 ( kmeshm, ncurvs )

      end if

!     --- For icurnr := 1 (1) ncurvs do

      if ( debug ) call prinin ( icurvind, ncurvs, 'icurvind' )
      istar3 = kmeshm(2)-1
      if ( iskipbn==1 ) then

!     --- iskipbn = 1, skip outer boundary

         do icurnr = 1, ncurvs

            if ( icurvind(icurnr)==1 ) then

!           --- curve = external

               nnodes = kmeshm(icurnr+5)-kmeshm(icurnr+4)
               istart = istar3+kmeshm(icurnr+4)-1

!              --- for all nodes nodp on curve icurnr do

               do i = 1, nnodes
                  inode = kmeshm(istart+i)
                  if ( inode>0 ) iwork(inode) = -1
               end do

            end if

         end do

      else if ( iskipbn==2 ) then

!     --- iskipbn = 2, skip curves as given in array icurvs

         do jcurnr = 1, ncurvsact

            kcurnr = abs(icurvs(jcurnr))
            if ( kelmq>0 ) then

!           --- There are sub curves

               nsubcv = kmeshq(kcurnr+1)-kmeshq(kcurnr)

            else

!           --- There are no sub curves

               nsubcv = 0

            end if
            if ( debug ) &
               write(irefwr,1) 'jcurnr, kcurnr, nsubcv', &
                                jcurnr, kcurnr, nsubcv
            do isubcv = 1, max(1,nsubcv)

               if ( nsubcv==0 ) then
                  icurnr = kcurnr
               else
                  icurnr = abs(kmeshq(kmeshq(kcurnr)+isubcv))
               end if
               nnodes = kmeshm(icurnr+5)-kmeshm(icurnr+4)
               istart = istar3+kmeshm(icurnr+4)-1

               if ( debug ) then
                  write(irefwr,*) 'isubcv, icurnr, nnodes, istart ', &
                                   isubcv, icurnr, nnodes, istart
                  call prinin ( kmeshm(istart+1), nnodes, 'nodes' )
               end if  ! ( debug )

!              --- for all nodes nodp on curve icurnr do

               do i = 1, nnodes
                  inode = kmeshm(istart+i)
                  if ( inode>0 ) iwork(inode) = -1
               end do

            end do

         end do

      else

!     --- iskipbn = 3, skip outer boundary, except curves given in array icurvs
!         Set corresponding positions in icurvind to 2

         do jcurnr = 1, ncurvsact

            kcurnr = icurvs(jcurnr)
            if ( kelmq>0 ) then

!           --- There are sub curves

               nsubcv = kmeshq(kcurnr+1)-kmeshq(kcurnr)

            else

!           --- There are no sub curves

               nsubcv = 0

            end if

            do isubcv = 1, max(1,nsubcv)

               if ( nsubcv==0 ) then
                  icurnr = kcurnr
               else
                  icurnr = abs(kmeshq(kmeshq(kcurnr)+isubcv))
               end if
               if ( icurvind(icurnr)==1 ) icurvind(icurnr) = 2

            end do

         end do

!        --- Set iwork for all external curves except the ones to be included

         do icurnr = 1, ncurvs

            if ( icurvind(icurnr)==1 ) then

!           --- curve = external

               nnodes = kmeshm(icurnr+5)-kmeshm(icurnr+4)
               istart = istar3+kmeshm(icurnr+4)-1

!              --- for all nodes nodp on curve icurnr do

               do i = 1, nnodes
                  inode = kmeshm(istart+i)
                  if ( inode>0 ) iwork(inode) = -1
               end do

            end if

         end do

!        --- Clear iwork for all external curves to be included
!            In this way end points are taken into account

         do icurnr = 1, ncurvs

            if ( icurvind(icurnr)==2 ) then

!           --- curve must be included

               nnodes = kmeshm(icurnr+5)-kmeshm(icurnr+4)
               istart = istar3+kmeshm(icurnr+4)-1

!              --- for all nodes nodp on curve icurnr do

               do i = 1, nnodes
                  inode = kmeshm(istart+i)
                  if ( inode>0 ) iwork(inode) = -1
               end do

            end if

         end do

!        --- Reset icurvind

         do jcurnr = 1, ncurvsact

            kcurnr = icurvs(jcurnr)
            if ( kelmq>0 ) then

!           --- There are sub curves

               nsubcv = kmeshq(kcurnr+1)-kmeshq(kcurnr)

            else

!           --- There are no sub curves

               nsubcv = 0

            end if

            do isubcv = 1, max(1,nsubcv)

               if ( nsubcv==0 ) then
                  icurnr = kcurnr
               else
                  icurnr = abs(kmeshq(kmeshq(kcurnr)+isubcv))
               end if
               if ( icurvind(icurnr)==2 ) icurvind(icurnr) = 1

            end do

         end do

      end if

      if ( debug ) then

!     --- Debug information

         call prinin ( iwork, npoint, 'iwork' )
         write(irefwr,*) 'End prob09s1'

      end if
      call erclos ( 'prob09s1' )

      end
