      subroutine prob10 ( mappinglc )
! ======================================================================
!
!        programmer    Guus Segal
!        version 16.0  date 22-10-2013 New parameter list
!        version 15.0  date 08-10-2007 Extra parameter kmeshmglob
!        version 14.0  date 24-09-2007 Remove array curves
!
!   copyright (c) 1985-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kprob part n with degrees of freedom
!     and transformation factors (coortr)
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     surface
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulework
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: mappinglc(*)

!     mappinglc        i    Array of size npointglobal
!                         Contains the local nodal point numbers corresponding
!                         to nodes in the global mesh
!                         If gobal node is not present in local mesh the
!                         corresponding value in mappinglc is 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nloccv, iinp, nlocobs, iinp5, nlocsf

!     iinp           last position used in array iinputkp
!     iinp5          Starting address of fifth part in iinputkp
!     iwork2         integer work array of length npoint
!                    iwork2 is used to mark nodes that have been
!                    transformed already
!                    iwork2(i) contains the number of times a node
!                    is transformed
!     nloccv         Number of curves with local transforms
!     nlocobs        Number of obstacles with local transforms
!     nlocsf         Number of surfaces with local transforms
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PROB10CURV     fill kprob part n for curves with degrees of freedom
!                    and transformation factors (coortr)
!     PROB10OBST     fill kprob part n with degrees of freedom
!                    and transformation factors (coortr) in case of an obstacle
!     PROB10SURF     fill kprob part n with degrees of freedom
!                    and transformation factors (coortr) in case of surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2780   parallel version not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob10' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      iinp5 = intarprb(5)
      npltra = 0
      iinp = iinp5-1

      allocate ( iwork2(npointglob), stat = error )
      if ( error/=0 ) call eralloc ( error, npointglob, 'iwork2' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork2 ( no nodes transformed yet)

      iwork2 = 0

      nlocobs = iinputkp(18)

      nloccv = iinputkp(iinp+1)
      nlocsf = iinputkp(iinp+2)
      iinp = iinp+2

      if ( debug ) &
         write(irefwr,1) 'nlocobs, nloccv, nlocsf, iinp', &
                          nlocobs, nloccv, nlocsf, iinp

      if ( nloccv>0 ) then

!     --- curves

         call prob10curv ( iinp, nloccv, mappinglc, iwork2 )

      end if  ! ( nloccv>0 )

      if ( nlocsf>0 ) then

!     --- surfaces

         call prob10surf ( iinp, nlocsf, mappinglc )

      end if

      if ( nlocobs>0 ) then

!     --- obstacles

         if ( parallel ) then

!        --- Parallel computation, we need global coordinates for
!            the transformations: not yet implemented

            call errsub ( 2780, 0, 0, 0 )
            go to 1000

         else

!        --- serial case

            call prob10obst

         end if  ! ( parallel )

      end if

      deallocate ( iwork2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork2' )

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kprobn, npltra, ndim, 'kprobn' )
         call prinrl1 ( coortr, npltra, maxtrn, 'coortr' )

         write(irefwr,*) 'End prob10'

      end if  ! ( debug )
      call erclos ( 'prob10' )

      end subroutine prob10
