      subroutine prob11 ( numsrf, ikprbi, iinp, iposgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute length of array kprob part i with boundary elements
!     corresponding to surfaces
!     This subroutine is the counterpart of subroutine PROB07, &
!     if PROB11 is changed also PROB07 should be changed and vice versa
!     New input
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numsrf, ikprbi, &
              iinp, iposgrp(*)

!     iinp       i    last position used in array iinputkp
!     ikprbi    i/o   last position used in array kprob part i
!     iposgrp   i/o   work array of length numnat in which the starting
!                     addresses of the element groups in KPROB part i are
!                     stored
!                     In this subroutine the length with respect to an
!                     element group is added to the existing array
!     numsrf     i    number of parts where surfaces are given for generation
!                     of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ibngrp, inum, isurnr, ielem, &
              inpelm, j, istart, jnpelm, k, nelemlc
      logical present

!     i              Counting variable
!     ibngrp         Element group sequence number
!     ielem          Loop variable for elements
!     inpelm         number of points per element
!     inum           Number of surfaces
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     j              Counting variable
!     jnpelm         Number of nodes in an element
!     k              Counting variable
!     nelemlc        Local number of elements
!     present        If true the element is lying in the processor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPIINFSURF    Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     270   number of nodal points in elements of the same group for the
!           creation of boundary elements, not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob11' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob11'
         write(irefwr,1) 'numsrf', numsrf
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numsrf

         ibngrp = iinputkp(iinp+1)
         inum = iinputkp(iinp+2)

         nelemlc = 0

         do j = 1, inum

            isurnr = iinputkp(iinp+2+j)

!           --- Find information of surface in kmeshn

            call sepiinfsurf ( istart, inpelm, isurnr )

            if ( debug ) &
               write(irefwr,1) 'j, ibngrp, inpelm, iinp, isurnr', &
                                j, ibngrp, inpelm, iinp, isurnr

            if ( j==1 ) then

!           --- j = 1, first surface in set

               jnpelm = inpelm

            else if ( inpelm/=jnpelm ) then

!           --- j>1, next surface in set and inpelm not jnpelm

               call errint ( inpelm, 1 )
               call errint ( jnpelm, 2 )
               call errint ( ibngrp, 3 )
               call errsub ( 270, 3, 0, 0)
               go to 1000

            end if  ! ( i==1 )

            do ielem = 1, inelem

               present = .true.  ! indicates whether element is in processor

               do k = 1, inpelm
                  if ( kmeshn(istart+k)<=0 ) present = .false. !not in processor
               end do  ! k = 1, inpelm

               if ( present ) nelemlc = nelemlc+1
               istart = istart+inpelm

            end do  ! ielem = 1, inelem

         end do  ! j = 1, inum

         kprobb(ibngrp) = kprobb(ibngrp)+nelemlc
         ikprbi = ikprbi+nelemlc*inpelm
         iposgrp(ibngrp) = iposgrp(ibngrp)+nelemlc*inpelm

         if ( debug ) &
            write(irefwr,1) 'istart, nelemlc, inpelm', &
                             istart, nelemlc, inpelm

         iinp = iinp+2+inum

      end do  ! i = 1, numsrf

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob11'

      end if  ! ( debug )
      call erclos ( 'prob11' )

      end
