      subroutine prob14belms ( iinp, iinp3 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute iinp3 and raise iinp for boundary elements
!     Help routine for prob14
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     initialization
!     natural_boundary_condition
!     boundary_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinp, iinp3

!     iinp          i/o   last position used in array iinputkp
!     iinp3          o    first pos. iinputkp definition surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numbpn, k, numcrv, numsrf, numlevelset, i, inum, numconct, &
              numobst

!     i              Counting variable
!     inum           Number of numbers read in a record
!     k              Counting variable
!     numbpn         Number of parts where point boundary elements are
!                    created
!     numconct       Maximum number of contacts in problem
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
!     numlevelset    Number of different level set functions
!     numobst        number of obstacles
!     numsrf         number of parts where surfaces are given for
!                    generation of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob14belms' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob14belms'
         write(irefwr,1) 'iinp', iinp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- point elements

      iinp3 = iinp
      numbpn = iinputkp(3)  ! number of parts with point boundary elements
      do k = 1, numbpn
         iinp = iinp+2+iinputkp(iinp+1)  ! raise iinp for point boundary elements
      end do  ! k = 1, numbpn
      if ( debug ) write(irefwr,1) 'numbpn, iinp', numbpn, iinp

!     --- curve elements

      numcrv = iinputkp(4)  ! number of parts with curve boundary elements
      do k = 1, numcrv
         iinp = iinp+3+iinputkp(iinp+2)
      end do  ! k = 1, numcrv
      if ( debug ) write(irefwr,1) 'numcrv, iinp', numcrv, iinp

!     --- surface elements

      numsrf = iinputkp(5)  ! number of parts with surface boundary elements
      do k = 1, numsrf
         iinp = iinp+2+iinputkp(iinp+1)
      end do  ! k = 1, numsrf
      if ( debug ) write(irefwr,1) 'numsrf, iinp', numsrf, iinp

!     --- obstacles may be present
!         natobst requires 3 positions

      numobst = iinputkp(17)
      iinp = iinp+3*numobst
      if ( debug ) write(irefwr,1) 'numobst, iinp', numobst, iinp

!     --- contact_elements

      numconct = iinputkp(20)+iinputkp(23)+iinputkp(24)+iinputkp(25)
      iinp = iinp+3*numconct
      if ( debug ) write(irefwr,1) 'numconct, iinp', numconct, iinp

!     --- curv_points and surface points

      numcrv = iinputkp(26)
      numsrf = iinputkp(27)
      do k = 1, numcrv
         iinp = iinp+2+iinputkp(iinp+1)
      end do  ! k = 1, numcrv
      do k = 1, numsrf
         iinp = iinp+2+iinputkp(iinp+1)
      end do  ! k = 1, numsrf
      if ( debug ) &
         write(irefwr,1) 'numcrv, numsrf, iinp', numcrv, numsrf, iinp

!     --- Cross_section_obstacles

      numcrv = iinputkp(34)
      do k = 1, numcrv
         iinp = iinp+5+iinputkp(iinp+4)
      end do  ! k = 1, numcrv
      if ( debug ) write(irefwr,1) 'numcrv, iinp', numcrv, iinp

!     --- Zero level set

      numlevelset = iinputkp(55)
      do i = 1, numlevelset
         inum = iinputkp(iinp+5)
         iinp = iinp+6+inum
      end do  ! i = 1, numlevelset
      if ( debug ) &
         write(irefwr,1) 'numlevelset, iinp', numlevelset, iinp

      if ( debug ) write(irefwr,1) 'iinp3, iinp', iinp3, iinp

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinputkp(iinp3), iinp-iinp3, 'iinputkp bounelm' )
         write(irefwr,*) 'End prob14belms'

      end if  ! ( debug )
      call erclos ( 'prob14belms' )

      end
