      subroutine prob14esssurf ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Raise iinp for essential boundary conditions (subroutine PROBDF)
!     Only surface part
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     initialization
!     essential_boundary_condition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array corresponding to subroutine probdf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numsrf, k, ncurvsact, ifirst, inpelm, istart

!     ifirst         First surface number
!     inpelm         number of points per element
!     istart         First position used at a part
!     k              Counting variable
!     ncurvsact      Actual number of curves to be used
!     numsrf         number of parts where surfaces are given for
!                    generation of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPEXTKMESHT   Compute starting and ending address of elements in kmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob14esssurf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob14esssurf'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numsrf = iinput(8)
      if ( numsrf==0 ) go to 1000

      if ( debug ) then

         write(irefwr,1) 'numsrf', numsrf
         call prinin ( iinput(iinp), 4*numsrf, 'iinput' )

      end if  ! ( debug )

      do k = 1, numsrf

         ifirst = iinput(iinp+1)
         if ( debug ) write(irefwr,1) 'iinp, ifirst', iinp, ifirst
         if ( ifirst==-1 ) then

!        --- Special case of outer_surfaces

            iinp = iinp+3

         else  ! ( ifirst>0 )

!        --- Standard case

            if ( debug ) &
               call prinin ( iinput(iinp), iinput(iinp+1)+3+ &
                             iinput(iinp+2), 'iinput' )
            iinp = iinp + iinput(iinp+1) + iinput(iinp+2) + 3

!           --- skip boundary

            if ( debug ) then
               write(irefwr,1) 'iinp/sur_2', iinp
               call prinin ( iinput(iinp), 2, 'iinput' )
            end if  ! ( debug )
            ncurvsact = iinput(iinp+1)
            iinp = iinp+2+ncurvsact

            if ( debug ) write(irefwr,1) 'iinp/sur_3', iinp

         end if  ! ( ifirst==-1 )

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob14esssurf'

      end if  ! ( debug )
      call erclos ( 'prob14esssurf' )

      end
