      subroutine prob20 ( iobst, jstep, icurvsobs, ncurvs, &
                          ibit, ibit2, kmeshm, iobscurv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 14-08-1999 New definition icurvsobs
!        version  2.0  date 12-06-1998 Extar parameter iobscurv
!        version  1.0  date 29-11-1996
!
!   copyright (c) 1996-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the number of nodes with prescribed boundary conditions on
!   obstacle iobst
!   This subroutine is the counterpart of subroutine PROB20, &
!   if PROB20 is changed also PROB19 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     boundary_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jstep, iobst, icurvsobs(2,*), &
              ibit, ibit2, kmeshm(*), ncurvs, &
              iobscurv(*)

!     ibit       i    Actual bit pattern
!     ibit2      i    Maximal bit pattern
!     icurvsobs  i    Array containing information of he active obstacles.
!                     The array corresponds to all curves separately
!                     icurvsobs(1,i) = 1 means that the corresponding point is
!                     active; 0 means not active
!     iobscurv   i    Array of length ncurvs containing information about the
!                     obstacles
!                     iobscurv(i) = 0: curve i not on obstacle
!                     iobscurv(i) = 1: points of curve i on obstacle
!                     iobscurv(i) = 2: begin and end point of curve i on
!                                      obstacle
!                     iobscurv(i) = 3: begin point of curve i on obstacle
!                     iobscurv(i) = 4: end point of curve i on obstacle
!     iobst      i    Obstacle sequence number
!     jstep      i    Indicates which points must be taken into account
!                     and which points must be skipped (See PG)
!     kmeshm     i    KMESH part m, contains nodal point numbers of the curves
!     ncurvs     i    Number of curves in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i1, i2, nodalp, inunkp, ih, istar3, icurnr

!     i1         First point of curve in icurvsobs
!     i2         Last point of curve in icurvsobs
!     icurnr     Curve sequence number
!     ih         Help variable to store a constant temporarily
!     inunkp     Number of degrees of freedom in point
!     istar3     Starting address of KMESHM part 3 in KMESH part M
!     k          Counting variable
!     nodalp     Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0, inibt1

!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INIBT0    Compute bit pattern
!     INIBT1    Compute maximum unknown corresponding to bit pattern
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1861   Wrong unknown prescribed at curve
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters
!    For all curves do
!       For all nodes along the curve do
!          if ( node is at obstacle iobst ) then
!             nkbnd := nkbnd + 1
!             Fill bit pattern
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob20' )

      istar3 = kmeshm(2)-1
      do icurnr = 1, ncurvs
         i1 = kmeshm(icurnr+4)
         i2 = kmeshm(icurnr+5)-1
         if ( iobscurv(icurnr).gt.0 ) then

!        --- Possibly active points on curve

            if ( jstep.eq.1 ) then

!           --- all nodes must be used

               do k = i1, i2
                  if ( icurvsobs(1,k).eq.iobst ) then
                     nodalp = kmeshm(istar3+k)
                     if ( indprf.gt.0 ) then

!                    --- indprf > 0  kprob part f contains the number of
!                        unknowns

                        inunkp = kprobf(nodalp+1)-kprobf(nodalp)
                        ibit2 = inibt0(inunkp)

                     end if
                     if ( ibit.gt.ibit2 .or. ibit.lt.0 ) then
                        call errint ( inibt1(ibit), 1 )
                        call errint ( k, 2 )
                        call errint ( icurnr, 3 )
                        call errint ( icurnr, 4 )
                        call errint ( inunkp, 5 )
                        call errsub ( 1861, 5, 0, 0 )
                     end if
                     ih = ibit
                     if ( ih.eq.0 ) ih = ibit2
                     nkbnd = nkbnd+1
                     kprobg(1,nkbnd) = nodalp
                     kprobg(2,nkbnd) = ih
                  end if
               end do

            end if

         end if

      end do

      call erclos ( 'prob20' )

      end
