      subroutine prob30 ( natnocel, iinp, iposgrp )
! ======================================================================
!
!        programmer     Guus Segal
!        version 2.0    date   05-05-1998 Use array iposgrp
!        version 1.0    date   12-12-1997
!
!
!   copyright (c) 1997-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array kprob part i with boundary elements
!    corresponding to no_contact surfaces
!     surface
!    This subroutine is the counterpart of subroutine PROB28, &
!    if PROB28 is changed also prob30 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!   problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer natnocel, iinp, iposgrp(*)

!     iinp      i/o   last position used in array iinputkp
!     iposgrp   i/o   work array of length numnat in which the starting
!                     addresses of the element groups in KPROB part i are
!                     stored
!                     At input the last postions used for an element group
!                     are stored, at output they are updated
!     natnocel   i    Number of no_contacts where natural boundary conditions
!                     are given
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iconct, inpelm, istart, ielem, nelcon, nelnocon, &
              isurnr, j, jstart, numconct, k, ibngrp, ikprbi

!     i          Counting variable
!     ibngrp     boundary element group number
!     iconct     Sequence number of contact
!     ielem      Elements sequence number
!     ikprbi     Last position used in kprobi for this group
!     inpelm     number of points per element
!     istart     First position used at a part
!     isurnr     Surface sequence number
!     j          Counting variable
!     jstart     First position used at a part
!     k          Counting variable
!     nelcon     Number of contact elements
!     nelnocon   Number of no_contact elements
!     numconct   Maximum number of contacts in problem
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initializations
!    For all natural boundary contact elements do
!       iblgrp = boundary element group
!       iconct = contact sequence number
!       isurnr := contact surface
!       inpelm := number of nodes per element
!       nelcon := number of contact elements
!       for j := 1 (1) nelcon do
!          Copy elements to kprobi
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- Initializations

      numconct = kprobloc(48)

!     --- For all natural boundary contact elements do

      do i = 1, natnocel
         ibngrp = iinputkp(iinp+1)
         ikprbi = iposgrp(ibngrp)
         iconct = iinputkp(iinp+3)
         iinp = iinp+3
         if ( iconct<=numconct ) then

!        --- For all surfaces do

            istart = kprobwp1(iconct,4)
            inpelm = kprobwp1(iconct,3)
            nelcon = kprobwp1(iconct,1)
            nelnocon = kprobwp1(iconct,2)
            if ( nelnocon>0 ) then

!           --- Contact elements have been found

               jstart = 2*nsurfs
               isurnr = kprobwp1(iconct,3)
               inpelm = kmeshn(isurnr)
               do j = 1, isurnr-1
                  jstart = jstart+kmeshn(j)*kmeshn(j+nsurfs)
               end do
               istart = istart+nelcon-1

!              --- Copy elements to kprobi

               do j = 1, nelnocon
                  ielem = kprobwp2(istart+j)
                  do k = 1, inpelm
                     kprobi(ikprbi+k) = &
                        kmeshn(jstart+k+(ielem-1)*inpelm)
                  end do
                  ikprbi = ikprbi+inpelm
               end do

            end if

         end if
         iposgrp(ibngrp) = ikprbi

      end do

      end
