      subroutine problevskipel ( skip )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-11-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill elements of kprob part aa that must be skipped since they are
!     in the wrong part of the levelset function
!     kprob part aa must have been created and initialized
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     element
!     skip
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical skip

!     skip           o    If true elements must be skipped
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numlevelset, iinp14, ilev, iskip

!     iinp14         first position in array iinputkp corresponding to obstacles
!     ilev           Sequence number of level being considered
!     iskip          Defines how the level set nodes must be treated
!                    -1: all nodes outside phi>0 are prescribed
!                     0: no nodes are prescribed
!                     1: all nodes inside phi>0 are prescribed (not the
!                        boundary)
!     numlevelset    Number of different level set functions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRMARKOUTLEVSP Mark array imark for all elements that are completely
!                    outside the obstacle
!     PRMARKSKIPELM  Mark array imark for all elements that are completely
!                    inside the obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'problevskipel' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from problevskipel'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      numlevelset = iinputkp(48)
      iinp14 = intarprb(18)

      skip = .false.
      do ilev = 1, numlevelset
         iskip = iinputkp(iinp14+ilev-1)
         if ( iskip==1 .or. iskip==-1 ) then
            skip = .true.
            exit
         end if  ! ( iskip==1 .or. iskip==-1 )
      end do  ! ilev = 1, numlevelset

      if ( skip ) then

!     --- skip = true, put values in kprobaa

         do ilev = 1, numlevelset
            iskip = iinputkp(iinp14+ilev-1)
            if ( iskip==-1 ) then

!           --- iskip = -1
!               Skip all elements inside obstacle

               call prmarkskipelm ( kprobaa, ielemobst, ilev )

            else if ( iskip==1 ) then

!           --- iskip = 1
!               Skip all elements outside obstacle

               call prmarkoutlevsp ( kprobaa, ielemobst, ilev, nelem )

            end if  ! ( iskip==-1 )

         end do  ! ilev = 1, numlevelset

      end if  ! ( skip )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End problevskipel'

      end if  ! ( debug )
      call erclos ( 'problevskipel' )

      end
