      subroutine probnatblevs1 ( natlevset, iinp, &
                                 ipointobst, iposgrp, ikprbi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-12-2011 Use inizerolev
!        version  1.0  date 11-10-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob part i with elements for
!     natural boundary conditions defined on zero levelset points
!     1d only
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     boundary_condition
!     natural
!     problem_definition
!     level_set
!     natural_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer natlevset, iinp, &
              ipointobst(*), iposgrp(*), ikprbi

!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!     natlevset      i    number of parts where zero level set elements
!                         are defined
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer igrp, ibngrp, inum, i, istart, numbndobst

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     igrp           Boundary element group number read
!     inum           Number of numbers
!     istart         First position used at a part
!     numbndobst     Number of nodal points on the boundary of the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIZEROLEV     Compute number of nodes in zero level set and starting
!                    position in ipointobst
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probnatblevs1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probnatblevs1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      call inizerolev ( ipointobst, istart, numbndobst, 1 )

      if ( debug ) &
         write(irefwr,1) 'numbndobst, istart', numbndobst, istart
      do igrp = 1, natlevset

         ibngrp = iinputkp(iinp+1)         ! boundary group
         inum = max(1,iinputkp(iinp+5) )   ! number of curves
         iinp = iinp+6+inum
         ikprbi = iposgrp(ibngrp)        ! starting address in kprob part i
         if ( debug ) &
              write(irefwr,1) 'igrp, ibngrp, ikprbi, inum, iinp', &
                               igrp, ibngrp, ikprbi, inum, iinp

!        --- For all points on boundary

         do i = 1, numbndobst

!        --- Fill kprob part i

            kprobi(ikprbi+1) = ipointobst(istart+i)
            ikprbi = ikprbi+1

         end do  ! i = 1, numbndobst

         if ( debug ) then
            call prinin1 ( kprobi(iposgrp(ibngrp)+1), numbndobst, &
                           1, 'kprob nat elements' )

         end if  ! ( debug )

!        --- Raise iposgrp

         iposgrp(ibngrp) = iposgrp(ibngrp)+numbndobst

      end do  ! igrp = 1, natlevset

1000  call erclos ( 'probnatblevs1' )
      if ( debug ) then

!     --- Debug information

         call prinin ( kprobb, kprobloc(12), 'kprob part b' )
         write(irefwr,*) 'End probnatblevs1'

      end if  ! ( debug )

      end
