      subroutine probparblocks
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill some information about blocks in array blockinfor
!
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: length1
      integer :: numneigh, iwork(iacnodes), iseq(iacnodes), i, iblock, &
                 istart, numunks

!     i              Counting variable
!     iblock         Block sequence number
!     iseq           Work array for sorting
!     istart         First position used at a part
!     iwork          integer work array
!     length1        Help parameter to indicate the length of arrays
!     numneigh       number of neighbor blocks
!     numunks        number of unknowns along the interface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     In array blockinfor we store the following information
!     - number of neighbor blocks with lower sequence number
!     - number of neighbor blocks with higher sequence number
!     Followed by the corresponding block numbers (sorted)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparblocks' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparblocks'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get starting addresses

      blockinfor => kprobpart%blockinfor
      blockunkinfo => kprobpart%blockunkinfo
      sendinfo => kprobpart%sendinfo
      length1 = blockunkinfo(1)+3

      if ( associated(kprobpart%blockinfor) ) &
         deallocate(kprobpart%blockinfor)
      allocate(kprobpart%blockinfor(length1))
      blockinfor => kprobpart%blockinfor

      numneigh = blockunkinfo(1)  ! number of neighbor blocks

      if ( debug ) write(irefwr,1) 'numneigh', numneigh

!     --- Store neighbor blocks in iwork

      do i = 1, numneigh
         iwork(i) = blockunkinfo(2*i)
      end do  ! i = 1, numneigh

!     --- Sort array iwork

      call chsort ( iwork, iseq, numneigh )

!     --- Store information in array blockinfor

      blockinfor(1) = 0  ! number of neighbors with smaller number
      do i = 1, numneigh
         iblock = iwork(iseq(i))
         blockinfor(2+i) = iblock
         if ( iblock<inodenr )  blockinfor(1) = blockinfor(1)+1
      end do  ! i = 1, numneigh

      blockinfor(2) = numneigh-blockinfor(1)  ! number of neighbors with higher
                                              ! number

!     --- Store number of unknowns of all neighbors with larger number
!         into blockinfor(3+numeigh)

      numunks = 0
      do i = blockinfor(1)+1, numneigh
         iblock = iwork(iseq(i))
         istart = blockunkinfo(2*i+1)
         numunks = numunks+sendinfo(istart)
      end do  ! i = blockinfor(1)+1, numneigh

      blockinfor(3+numneigh) = numunks

1000  if ( debug ) then

!     --- Debug information


         write(irefwr,1) 'numunks', numunks
         call prinin ( blockinfor, length1,  'blockinfor' )
         write(irefwr,*) 'End probparblocks'

      end if  ! ( debug )
      call erclos ( 'probparblocks' )

      end subroutine probparblocks
