      subroutine probskipelm
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-10-2011 Make iwork local
!        version  1.2  date 08-12-2004 Use subroutine prmarkskipelm
!        version  1.1  date 11-01-2004 New definition ipointobst
!
!   copyright (c) 2000-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob part aa with information concerning the elements to be
!     skipped
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     element
!     skip
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, inpelm, itypeskip, iobst, i1, i2, ifirst, ilast, &
              icurnr, ielgrp, iinp, ikelmc, inode, ipart, iielem, &
              istar3, istart, numpart, numbndobst, numinnobst, numpntobst
      integer, allocatable :: iwork(:)
      logical found, found1

!     found          If true already a possibility has been found
!     found1         If true already a possibility has been found
!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icurnr         Curve sequence number
!     ielem          Element number
!     ielgrp         element group number
!     ifirst         First curve (surface) number read
!     iielem         Relative element number in element group
!     iinp           last position used in array iinputkp
!     ikelmc         position in kmesh part c
!     ilast          Last curve (surface) number read
!     inode          Node sequence number
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     ipart          Sequence number of part
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     itypeskip      Defines which elements must be skipped
!                    1: skip all elements inside an obstacle
!                    2: skip all elements that are in the neighbourhood of a
!                       boundary and also on a given curve
!     iwork          integer work array of length npoint
!                    iwork(i) = 1, means that node i is at a given curve
!                    iwork(i) = 2, means that node i is in the
!                       neighbourhood of a boundary of an obstacle
!                    Otherwise iwork(i) = 0
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numpart        Number of parts read
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRMARKSKIPELM  Mark array imark for all elements that are completely
!                    inside the obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     185   first curve number (icrv1) is out of range
!     186   last curve number (icrv2) is out of range
!    1576   Value of integer parameter incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     iinp := iinp13
!     Clear array kprobaa
!     numpart := iinputkp(iinp)
!     iinp := iinp+1
!     for i := 1 (1) numpart do
!        itypeskip := iinputkp(iinp)
!        if ( itypeskip=1 ) then
!           iobst := iinputkp(iinp+1)
!           Set kprobaa equal to 1 for all elements that are in the obstacle
!        else if ( itypeskip=2 ) then
!           iobst := iinputkp(iinp+1)
!           ifirst := iinputkp(iinp+2)
!           ilast := iinputkp(iinp+2)
!           Clear array iwork
!           Make iwork equal to 1 for nodes in curves ifirst to ilast
!           Make iwork equal to 2 for all nodes in and near the boundary of
!              iobst
!           For all elements do
!              if ( all nodes in element satisfy iwork(node)>0  and
!                   at least one satisfies iwork(node) = 1 ) then
!                 Set kprobaa(element) equal to 1
!        iinp := iinp+5
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probskipelm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probskipelm'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- iinp := iinp13
!         numpart := iinputkp(iinp)
!         iinp := iinp+1

      iinp = intarprb(17)
      numpart = iinputkp(iinp)
      iinp = iinp+1
      if ( debug ) write(irefwr,1) 'npoint, numpart', npoint, numpart

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- for i := 1 (1) numpart do

      do ipart = 1, numpart

!     --- itypeskip := iinputkp(iinp)

         itypeskip = iinputkp(iinp)
         if ( debug ) write(irefwr,1) 'itypeskip', itypeskip

         if ( itypeskip==1 ) then

!        --- itypeskip=1
!            iobst := iinputkp(iinp+1)
!            Set kprobaa equal to 1 for all elements that are in the obstacle

            iobst = iinputkp(iinp+1)
            call prmarkskipelm ( kprobaa, ielemobst, iobst )

         else if ( itypeskip==2 ) then

!        --- itypeskip=2
!            iobst := iinputkp(iinp+1)
!            ifirst := iinputkp(iinp+2)
!            ilast := iinputkp(iinp+2)
!            Clear array iwork

            iobst  = iinputkp(iinp+1)
            ifirst = iinputkp(iinp+2)
            ilast  = iinputkp(iinp+3)

            if ( debug ) &
               write(irefwr,1) 'iobst, ifirst, ilast', iobst, ifirst, ilast
            if ( ifirst<1 .or. ifirst>ncurvs ) then

!           --- ifirst out of range

               call errint ( ifirst, 1 )
               call errint ( ncurvs, 2 )
               call errsub ( 185, 2, 0, 0 )

            end if
            if ( ilast<ifirst .or. ilast>ncurvs ) then

!           --- ifirst out of range

               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( ncurvs, 3 )
               call errsub ( 186, 3, 0, 0 )

            end if
            iwork = 0

!           --- Make iwork equal to 1 for nodes in curves ifirst to ilast

            istar3 = kmeshm(2)-1
            do icurnr = ifirst, ilast

               i1 = kmeshm(4+icurnr)
               i2 = kmeshm(5+icurnr)-1
               do i = i1, i2
                  inode = kmeshm(istar3+i)
                  if ( inode>0 ) iwork(inode) = 1
               end do

            end do

!           --- Make iwork equal to 2 for all nodes in and near the boundary of
!               iobst

            istart = ipointobst(iobst)
            numpntobst = ipointobst(istart)
            numinnobst = ipointobst(istart+1)
            numbndobst = ipointobst(istart+2)
            istart = istart+3+numpntobst
            iwork(ipointobst(istart+1:istart+numbndobst+numinnobst)) = 2
            if ( debug ) &
               write(irefwr,1) 'numpntobst, numinnobst, numbndobst', &
                                numpntobst, numinnobst, numbndobst

!           --- For all elements do

            ielem = 0
            ikelmc = 0
            do ielgrp = 1, nelgrp

               inelem = Topgrp(ielgrp)%inelem
               inpelm = Topgrp(ielgrp)%inpelm
               ishape = kmeshf(ielgrp)
               if ( debug ) &
                  write(irefwr,1) 'ielgrp, inelem, ishape', &
                                   ielgrp, inelem, ishape
               if ( ishape<3 .or. ishape>10 ) then

!              --- element is not two-dimensional: skip

                  ikelmc = ikelmc+inpelm*inelem
                  ielem = ielem+inelem

               else

                  do iielem = 1, inelem

                     ielem = ielem+1

!                    --- if ( all nodes in element satisfy iwork(node)>0  and
!                          at least one satisfies iwork(node) = 1 ) then
!                        Set kprobaa(element) equal to 1

                     found = .false.
                     found1 = .false.
                     do i = 1, inpelm
                        inode = kmeshc(ikelmc+i)
                        if ( iwork(inode)<=0 ) go to 100
                        if ( iwork(inode)==1 ) found = .true.
                        if ( iwork(inode)==2 ) found1 = .true.
                     end do
                     if ( found .and. found1 ) kprobaa(ielem) = 1

100                  ikelmc = ikelmc+inpelm

                  end do

               end if

            end do

         else

!        --- Option has not been implemented

            call errchr ( 'itype_skip', 1 )
            call errchr ( 'iinputkp', 1 )
            call errint ( itypeskip, 1 )
            call errint ( iinp, 2 )
            call errsub ( 1576, 2, 0, 2 )

         end if

         iinp = iinp+5

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobaa, nelem, 'kprobaa' )
         write(irefwr,*) 'End probskipelm'

      end if  ! ( debug )
      call erclos ( 'probskipelm' )

      end subroutine probskipelm
