      subroutine prord1 ( iinput, maxlen, iinp, iinputlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version 16.4  date 20-10-2015 Split into various subroutines
!        version 16.3  date 02-09-2014 error message if numwis and itype>99
!        version 16.2  date 02-03-2012 Jump in case of error
!
!   copyright (c) 1982-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read cards for definition of type numbers of boundary elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, iinp, iinputlc(maxlen)

!     iinp      i/o   Last position used in IINPUT
!     iinput     o    integer input array
!     iinputlc        integer work array to store the input read locally
!                     This input is copied into iinput afterwards in the correct
!                     sequence
!                     iinputlc is filled as follows:
!                     Per boundary element group:
!                     Pos. 1-20:  type numbers
!                     If type numbers all in the range 1 to 99
!                     followed by pos 21: npelm
!                     followed by extra information read.
!                     This information is stored as read:
!                        type of information:
!                          -1 = end
!                           1 = numdegfd
!                           2 = vec
!                           3 = number
!                         This is followed by number of numbers read, followed
!                         by the numbers itself
!                         In case of vec and number the first of the number read
!                         is the sequence number
!                     If type number = >10000 followed by
!                        iextra
!                        4*iextra positions with information
!     maxlen     i    Maximum amount of contiguous space in IBUFFR after a
!                     starting address
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: ncodsb=1, nncode3=4
      integer itype3(nncode3), ncode3(nncode3), ncodmx3(nncode3), &
              nscods(ncodsb)
      double precision tabmin3(nncode3), tabmax3(nncode3)
      character (len=11) codesb(ncodsb), code3(nncode3)
      integer k, inum, ibngrp, icheck, jinp, &
              itypnr, i, iextra, iwork(1301), &
              inpelm, ntype, numnat, iinplc, numveclc, nsear

!     code3          Table with keywords
!     codesb         Table with subkeywords
!     i              Counting variable
!     ibngrp         boundary element group number
!     icheck         Indication if all type numbers are smaller than 100 (0) or
!                    not (1)
!     iextra         if >0 extra arrays of special structure must be defined
!     iinplc         Last position used in array iinputlc
!     inpelm         Number of points in an element
!     inum           Number of numbers read in a record
!     itype3         Indicates the type of action corresponding to keywords
!                    in code3
!     itypnr         Type number of element
!     iwork          integer work array of length 1301 to store
!                    starting addresses
!                    The positions 1 .. 1000 contains the starting addresses
!                    of all boundary element groups in array iinputlc
!                    Pos. 1001 contains the starting address of numdegfd
!                    for type numbers between 1 and 99
!                    Pos. 1001+ivec contains the starting address of
!                    vectors of special structure ivec
!                    for type numbers between 1 and 99
!                    Pos. 1001+Numvec+isear contains the starting address
!                    of physical degrees of freedom isear
!                    for type numbers between 1 and 99
!     jinp           last position used in array iinput with respect to iprec
!     k              Counting variable
!     ncode3         Table containing the number of significant letters in code3
!     ncodmx3        See ncodmx, now for code3
!     ncodsb         Number of entries in codesb
!     nncode3        Number of entries in code3
!     nscods         Number of sub keywords
!     nsear          Number of specific degrees of freedom
!     ntype          number of types  (maximal 20)
!     numnat         number of boundary element groups
!     numveclc         Number of vectors of special structure
!                    table(icode) gets the value 1 etc.
!     tabmax3        Contains maximum values allowed in table for code3
!     tabmin3        Contains minimum values allowed in table for code3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRORD1COPY     Copy array iinputlc into iinput for boundary elements
!     PRORD1TYPE     Read information of type numbers for boundary elements
!     PRORD1USER     Read information for boundary user elements
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognized:
!    For all boundary element groups:
!    bngrp i = ( type = n1, [n2, n3, ...] )
!       if all types positive and<100 possibly followed by:
!          numdegfd = ...  (default: 1)
!          vec j = ...
!          number = ...
!          npelm = ..
!       if type>10000 followed by mod(itype,10000) records of the shape:
!          n_vertex=., n_midside_point=., n_centroid=., n_face=.
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!     550   array length too short
!    1092   Type number does not exist
!    3202   Type number not between 1 and 99 for numerical analysis lab
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code3 / 'N_VERTEX', 'N_MIDSIDE_', 'N_CENTROID', 'N_FACE' /
      data ncode3  / 5, 5, 5, 5 /
      data ncodmx3 / 8,15,10, 6 /
      data itype3  / 0, 0, 0, 0 /
      data tabmin3 / 0d0, 0d0, 0d0, 0d0 /
      data tabmax3 / 1d2, 1d2, 1d2, 1d2 /
      data codesb / 'BNGRP' /
      data nscods /  3 /

!     code2    Table of local keywords
!              1: NUMDEGFD
!              2: VEC
!              3: NUMBER
!              4: NPELM
!     code3    Table of local keywords
!              1: N_VERTEX
!              2: N_MIDSIDE_POINT
!              3: N_CENTROID
!              4: N_FACE
!     codesb   Table of local subkeywords
!              1: BNGRP
!              2: NUMBER
!              3: TYPE
!              4: VEC
! ======================================================================
!
      call eropen ( 'prord1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord1'
         write(irefwr,1) 'iinp', iinp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Extend codemain with local subkeywords

      numnat = 0
      numveclc = 0
      nsear  = 0

!     --- Copy information into codesub

      call sepcopycodesub ( codesb, ncodsb, nscods )

!     --- read next numnat cards

      ityprd = 1
      iinplc = 0
      ntype = iinput(1)
      iwork(1:1000) = 0
      icode = -100

200   if ( icode==-100 .or. icode==nncodemain+1 ) then

!     --- new boundary element group
!         Read information of types


         icode = 0

         call prord1type ( iinplc, iinputlc, iwork, &
                           inum, ibngrp, numnat, ntype )

         if ( debug ) &
            write(irefwr,1) 'inum, numnat, iinplc', &
                             inum, numnat, iinplc

         if ( icode>0 ) go to 200   ! Main keyword found

!        --- check if all type numbers are larger than 99 or not

         icheck = 0
         do k = 1, inum
            itypnr = iinputlc(iinplc+k)

            if ( debug ) write(irefwr,1) 'k, itypnr', k, itypnr

            if ( numwis .and. (itypnr<1 .or. itypnr>99) ) then

!           --- Error type number not between 1 and 99 for the
!               numerical analysis lab

               call errint ( itypnr, 1 )
               call errsub ( 3202, 1, 0, 0 )
               go to 1000

            else if ( itypnr>=100 ) then
               icheck = 1
            else if ( itypnr<=0 ) then

!           --- Non-positive type number not allowed

               call errint ( itypnr, 1 )
               call errint ( ibngrp, 2 )
               call errsub ( 1092, 2, 0, 0 )
               go to 1000

            end if
         end do
         iextra = itypnr/10000
         iinplc = iinplc+20
         if ( iinplc>maxlen ) then

!        --- Too much information, i.e. maxlen too short

            call errchr ( 'iinputlc', 1 )
            call errint ( iinplc, 1 )
            call errint ( maxlen, 2 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

         if ( debug ) write(irefwr,1) 'itypnr', itypnr

         inpelm = -1
         iinplc = iinplc+1
         jinp = iinplc
         iinputlc(jinp) = inpelm

         if ( icheck==0 ) then

!        --- all elements are user elements

            call prord1user ( iinplc, iinputlc, numveclc, nsear, jinp, ibngrp )
            if ( icode>0 ) go to 200

         else if ( iextra>0 ) then

!        --- iextra extra records required

            iinputlc(iinplc+1) = iextra
            iinplc = iinplc+1

            call sepcopycode ( code3, nncode3, ncode3, ncodmx3 )
            call sepcopytype ( nncode3, itype3, tabmin3, tabmax3 )
            nmain = 0
            iretur = icode
            do i = 1, iextra
               table(1:nncode3) = 0d0
               ityprd = 1
               call sepreadsetkeyw
               do k = 1, nncode3
                  iinputlc(iinplc+k) = nint(table(k))
               end do
               iinplc = iinplc + nncode3
            end do
            icode = iretur

         else

!        --- Standard situation, the next record must be read

            icode = -100

         end if
         go to 200

      end if

      if ( iinplc>maxlen ) then

!     --- Too much information, i.e. maxlen too short

         call errchr ( 'iinputlc', 1 )
         call errint ( iinplc, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      if ( ierror/=0 ) go to 1000

!     --- Copy array iinputlc into iinput and rinputlc into rinput

      call prord1copy ( iwork, iinput, iinp, iinputlc, &
                        ntype, nncode3, numveclc, nsear, numnat )

      if ( iinp>maxlen ) then
         call errint ( iinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
      end if
      iinput(1) = ntype
      iinput(2) = numnat

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp', iinp
         call prinin ( iinput, iinp, 'iinput' )
         write(irefwr,*) 'End prord1'

      end if  ! ( debug )
      call erclos ( 'prord1' )

      end subroutine prord1
