      subroutine prord2contctp ( iinputlc, iinplc, itype, natconpt, &
                                 table )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-04-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iinputlc with information of natural boundary conditions
!     for contact_points records
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     natural_boundary_condition
!     contact
!     point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputlc(*), iinplc, itype, natconpt
      double precision table(*)

!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   Work array in which the input read is stored before
!                         copying in iinput
!                         See subroutine prord2
!     itype          o    Type of record found
!     natconpt      i/o   number of parts where contact points are defined as
!                         bound els.
!     table          i    Contains values of items read corresponding to code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ishape

!     ishape         Shape number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord2contctp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord2contctp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itype = 8
      natconpt = natconpt + 1
      ishape = nint(table(5))
      iinputlc(iinplc+1) = itype
      iinputlc(iinplc+2) = ishape
      iinputlc(iinplc+3) = nint(table(8))
      iinplc = iinplc+3

1000  call erclos ( 'prord2contctp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prord2contctp'

      end if  ! ( debug )

      end
