      subroutine prord2curvep ( iinputlc, iinplc, itype, natcrpnt, &
                                istartpnt, table, tabmax, tabout, &
                                lencrvpt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-03-2012 Extra parameter lencrvpt
!        version  1.0  date 14-04-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iinputlc with information of natural boundary conditions
!     for curv_points records
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     natural_boundary_condition
!     curve
!     point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputlc(*), iinplc, itype, natcrpnt, istartpnt, lencrvpt
      double precision tabmax(*), tabout(*), table(*)

!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   Work array in which the input read is stored before
!                         copying in iinput
!                         See subroutine prord2
!     istartpnt      i    Start position of keyword p in table
!     itype          o    Type of record found
!     lencrvpt      i/o   Length of curve_points part in iinput for boundary
!                         elements
!     natcrpnt      i/o   Number of curves where point boundary elements
!                         are given
!     table          i    Contains values of items read corresponding to code
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, iposut

!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     334   record with curves contains wrong number of curve numbers
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord2curvep' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord2curvep'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itype = 11
      natcrpnt = natcrpnt+1

      inum = nint(tabmax(istartpnt+1))
      if ( inum==0 ) then

!     --- inum = 0, no curves found

         call errint ( inum, 1 )
         call errint ( 2, 2 )
         call errsub ( 334, 2, 0, 0 )
         go to 1000

      end if  ! ( inum==0 )

      iposut = nint(table(istartpnt+1)) ! starting address in tabout
      iinputlc(iinplc+1) = itype
      iinputlc(iinplc+2) = inum
      iinplc = iinplc+2
      iinputlc(iinplc+1:iinplc+inum) = &
         nint(tabout(iposut:iposut+inum-1))
      iinplc = iinplc+inum
      lencrvpt = lencrvpt+2+inum

1000  call erclos ( 'prord2curvep' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prord2curvep'

      end if  ! ( debug )

      end
