      subroutine prord2point ( iinputlc, iinplc, itype, natpnt, table, &
                               istartpnt, lenpnt, tabmax, tabout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-04-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iinputlc with information of natural boundary conditions
!     for point records
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     point
!     natural_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputlc(*), iinplc, itype, natpnt, istartpnt, lenpnt
      double precision table(*), tabmax(*), tabout(*)

!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   Work array in which the input read is stored before
!                         copying in iinput
!                         See subroutine prord2
!     istartpnt      i    Start position of keyword p in table
!     itype          o    Type of record found
!     lenpnt         i    Length of points part
!     natpnt        i/o   number of parts where point elements are defined
!                         for natural boundary conditions
!     table          i    Contains the numbers read for the various keywords
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, iposut

!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     169   Number of points read in point generation record is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord2point' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord2point'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itype = 2
      natpnt = natpnt+1
      inum = nint(tabmax(istartpnt))
      if ( inum==0 ) then
         call errint ( inum, 1 )
         call errint ( 20, 2 )
         call errsub ( 169, 2, 0, 0 )
      else
         iposut = nint(table(istartpnt))
      end if

      iinputlc(iinplc+1) = itype
      iinputlc(iinplc+2) = inum
      iinplc = iinplc+2
      iinputlc(iinplc+1:iinplc+inum) = &
         nint(tabout(iposut:iposut+inum-1))
      iinplc = iinplc+inum
      lenpnt = lenpnt+inum+2

1000  call erclos ( 'prord2point' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prord2point'

      end if  ! ( debug )

      end
