      subroutine prordlevelset ( iinput, maxlen, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 23-08-2011 Extra debug
!        version  1.1  date 02-04-2008 Fortran 90
!        version  1.0  date 13-08-2004
!
!   copyright (c) 2004-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information following levelset and store in iinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, iinp

!     iinp          i/o   Last position used in IINPUT
!     iinput        i/o   integer array to be filled with information concerning
!                         number of level sets
!     maxlen         i    Maximal space available for array IINPUT
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=4
      integer itype1(nncode1), ncode1(nncode1), ncodmx1(nncode1), i, jinp, &
              numpart, icount
      double precision tabmin1(nncode1), tabmax1(nncode1)
      character (len=13) code1(nncode1)

!     code1          Table with keywords
!     i              Counting variable
!     icount         Counter
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     jinp           first position used in array iinput
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
!     numpart        Nuber of times the keyword levelset has been found
!     tabmax1        Contains maximum values allowed in table for code1
!     tabmin1        Contains minimum values allowed in table for code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognized:
!       levelset i, text
!       with text: all, positive, negative
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!    2590   mandatory keyword not found
!    2592   mutually exclusive keywords of a set of three were found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Replace keyword levelset in codemain temporarily by fake keyword
!     Read and store information until main keyword is found
!     Set keyword levelset back
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'LEVELSET', 'ALL', 'POSITIVE_PART', 'NEGATIVE_PART' /
      data ncode1  / 5,  3,  3,  3 /
      data ncodmx1 / 8,  3, 13, 13 /
      data itype1  / 0,  2,  2,  2 /
      data tabmin1 / 1d0, 0d0, 0d0, 0d0 /
      data tabmax1 / 1d0, 0d0, 0d0, 0d0 /

!     code1    Table of local keywords
!              1: LEVELSET
!              2: ALL
!              3: POSITIVE
!              4: NEGATIVE
! ======================================================================
!
      call eropen ( 'prordlevelset' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordlevelset'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      nmain = nncodemain

!     --- Replace keyword levelset in codemain temporarily by fake keyword

      codemain(17) = 'YYYYYYYY'
      lastitem = 0  ! set lastitem back to 0, so that levelset is reread

!     --- Read and store information until main keyword is found

      iretur = 0
      icode = 0
      numpart = 0
      jinp = iinp+1
      iinp = jinp
      do while ( icode==0 )

         table(1:nncode) = -1d20
         call sepreadsetkeyw
         icode = iretur

         if ( icode==0 ) then

!        --- icode = 0, not a main keyword found yet

            numpart = numpart+1
            if ( table(1)<1d0 ) then

!           --- Keyword level set not found

               call errchr ( code1(1)(1:ncodmx1(1)), 1 )
               call errsub ( 2590, 0, 0, 1 )
               call errcrd

            else

!           --- store sequence number in iinput

               iinput(iinp+1) = nint(table(1))

            end if  ! ( table(1)<1d0 )
            iinput(iinp+2) = 0
            icount = 0
            if ( table(2)>-1d10 ) then

!           --- all found

               icount = 1

            end if  ! ( table(2)>-1d10 ) then
            if ( table(3)>-1d10 ) then

!           --- positive found

               icount = icount+1
               iinput(iinp+2) = 1

            end if  ! ( table(2)>-1d10 ) then
            if ( table(4)>-1d10 ) then

!           --- negative found

               icount = icount+1
               iinput(iinp+2) = -1

            end if  ! ( table(2)>-1d10 ) then

            if ( icount>1 ) then

!           --- Too many keywords

               do i = 1, 3
                  call errchr ( code1(i+1)(1:ncodmx1(i+1)), i )
               end do  ! i = 1, 3
               call errsub ( 2592, 0, 0, 3 )
               call errcrd

            end if  ! ( icount>1 )

            iinp = iinp+2

         end if  ! ( icode==0 ) then

      end do  ! while ( icode==0 )
      iinput(jinp) = numpart

      if ( iinp>=maxlen ) then

!     --- iinp>maxlen, too few space in IINPUT

         call errint ( iinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
         call errcrd

      end if

!     --- Set keyword levelset back

      codemain(17) = code1(1)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prordlevelset'

      end if
      call erclos ( 'prordlevelset' )

      end
