      subroutine prordnumlevs ( icode, iinput, codemn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-08-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information concerning num_levelset and store in iinput(48)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode, iinput(*)
      character *(*) codemn(*)

!     codemn         i    Table with main keywords
!     icode          o    Return code, see prorea
!     iinput        i/o   integer array to be filled with information concerning
!                         number of level sets
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numlevelset

!     numlevelset    Number of different level set functions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1628   Keyword found more than one for the same problem
!    1870   Integer number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prordnumlevs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordnumlevs'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      if ( iinput(48).gt.0 ) then

!     --- Keyword NUM_LEVELSET has been read before

         call errchr ( codemn(icode), 1 )
         call errwar ( 1628, 0, 0, 1 )
         call errcrd

      end if  ! ( iinput(48).gt.0 )
      numlevelset = 1

!     --- NUM_LEVELSET found, read number

      call asknxt

      if ( ityprd.ne.2 ) then

!     --- Not an integer found

         call errchr ( codemn(icode), 1 )
         call errsub ( 574, 0, 0, 1 )
         call errcrd

      else

!     --- integer found

         numlevelset = nint(realrd)
         if ( numlevelset.lt.1 .or. numlevelset.gt.1 ) then

!        --- numlevelset out of range

             call errint ( numlevelset, 1 )
             call errint ( 1, 2 )
             call errint ( 1, 3 )
             call errchr ( codemn(icode), 1 )
             call errwar ( 1870, 3, 0, 1 )
             call errcrd

         end if

      end if
      ityprd = 1
      icode = 0
      iinput(48) = numlevelset

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prordnumlevs'

      end if
      call erclos ( 'prordnumlevs' )

      end
