      subroutine prorea ( iinput, lengin, lprob, kread, &
                          rinput, lengrl, maxlen, maxrl, iseqprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version 14.9  date 06-12-2015 new style of storage
!        version 14.8  date 30-10-2014 Types must be written completely
!        version 14.7  date 06-08-2014 Extra debug
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read cards for subroutine probdf and store in arrays iinput/rinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     fictitious_domain
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      use sepmoduledebug
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxlen, iinput(maxlen), lengin, &
              lprob, lengrl, maxrl, kread, iseqprob
      double precision rinput(maxrl)

!     iinput         o    integer input array to be filled by PROREA:
!                         Defined length maxlen
!                         See subroutine PRINTPROBINPUT
!     iseqprob       o    Sequence number of problem input
!     kread          i    Indication if first record has been read (>0)
!                         or not (0)
!                         Possible values:
!                         0:  first record has not been read
!                         1:  first record has been read but not the problem
!                             number
!                         >10:  first record has been read as well as the
!                             problem number
!                             The problem number is equal to kread-10
!     lengin         o    Length of integer array iinput
!     lengrl         o    Length of real array rinput
!                         (in double precision variables)
!     lprob          o    Number of problems that have been read
!     maxlen         i    Declarwed length of array iinput
!     maxrl          i    Maximum number of reals that are allowed in the input
!     rinput         o    Real input array to be filled by PROREA:
!                         Defined length maxrl
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nseq=15
      integer ipiinp, iprinp, lenloc, maxloc, iseq(nseq,2), &
              iinp, ih, nprob, leniinpfp, &
              iprinplc, maxrllc, iprint, iprob, &
              iinputlc(maxlen), newstyle

      logical finished, firstrecord

!     finished       If true the while loop has finished
!     firstrecord    if true the first record has not been interpreted yet
!     ih             Help variable to store a constant temporarily
!     iinp           Last position used in IINPUT
!     iinputlc       Work array to store local array iinput
!     ipiinp         Starting address of array IINPUT in IBUFFR
!     iprinp         Starting address of array RINPUT in RINPUT
!     iprinplc       Starting address of array RINPUTLC in IBUFFR
!     iprint         Print level
!     iprob          Problem number
!     iseq           Array containing the start and end addresses of array
!                    iinput for the various parts in the sequence:
!                     1:  type numbers for internal elements
!                     2:  type numbers for boundary elements
!                     3:  information about boundary elements
!                     4:  information about essential boundary conditions
!                     5:  information about local transforms
!                     6:  information about unknown constants
!                     7:  information about renumbering
!                     8:  type numbers for global elements
!                     9:  information about global elements
!                    10:  information about global renumbering
!                    11:  type numbers for fictitious elements
!                    12:  information about fictitious elements
!                    13:  information about elements to be skipped
!                    14:  information about level set functions
!                    15:  information about periodical boundary conditions
!     leniinpfp      Length of first part of iinput for probdf
!     lenloc         Computed length of array RINPUT for one problem
!     maxloc         Copy of maxrl, used to compute the free space for each
!                    space separately
!     maxrllc        Maximum length available for array rinputlc
!     newstyle       if 1 keyword newstyle has been found
!     nprob          Number of various problems
!     nseq           Length of array iseq
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINTPROBINPUT Print the contents of array iinput with respect to problem
!                    definition
!     PRORD0         Read records for definition of type numbers of internal
!                    element
!     PRORD1         Read records for natural boundary conditions
!     PRORD2         Read records for boundary elements
!     PRORD3         Read records for essential boundary conditions
!     PRORD4         Read records for local transformations
!     PRORD5         Read records for boundary conditions of the type u equals
!                    unknown constant
!     PRORD6         Read numbers for renumbering of unknowns
!     PRORD7         read cards for definition of type numbers of global
!                    elements
!     PRORD8         read cards for global elements
!     PRORD9         read cards for global renumbering
!     PRORDFICTELM   read cards for definition of type numbers of fictitious
!                    elements
!     PRORDFICTUNK   read cards for fictitious elements
!     PRORDLEVELSET  Read information following levelset and store in iinput
!     PRORDNATBOUN   Fill type numbers for natural boundary conditions in iinput
!     PRORDNUMLEVS   Read information concerning num_levelset
!     PRORDPERBC     Read records for definition of periodical boundary
!                    conditions
!     PRORDPRINTLEV  Read information concerning print_level and store in iprint
!     PRORDR         Rearrange array IINPUT
!     PRORDSKIPELM   read cards for skip elements
!     PROREADEFAULT  Defines array iinput for probdf in case of default problem
!     SEPCOPYCODEMN1 Copy codemn2 and related arrays into codemain
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the standard input file through the subroutines
!    PRORD.
!    The following input is recognized
!    PROBLEM
!       TYPES
!          Records as described in PRORD0
!       NATBOUNCOND
!          Records as described in PRORD1
!       BOUNELEMENTS
!          Records as described in PRORD2
!       ESSBOUNDCOND
!          Records as described in PRORD3
!       LOCALTRANSFORM
!          Records as described in PRORD4
!       UNKNOWNCONSTANT
!          Records as described in PRORD5
!       REORDER/RENUMBER
!          Records as described in PRORD6
!       GLOBAL_UNKNOWNS
!          Records as described in PRORD7
!       GLOBAL_ELEMENTS
!          Records as described in PRORD8
!       FICTITIOUS_UNKNOWNS
!          Records as described in PRORDFICTUNK
!       FICTITIOUS_ELEMENTS
!          Records as described in PRORDFICTELM
!       SKIP_ELEMENTS
!          Records as described in PRORDSKIPELM
!       PRINT_LEVEL = k
!       NUM_LEVELSET = i
!       LEVELSET  i, all /negative_part / positive_part
!    END
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      18   first input record of probdf is not problem
!      29   Too few space for standard problems in IINPUT
!     694   No problem number read in succeeding problem record.
!     695   Problem number read is incorrect
!    1628   Keyword found more than one for the same problem
!    3098   Value of parameter not allowed
!    3203   BOUNELEMENTS found before NATURAL_BOUNDARY_CONDITIONS and TYPES
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prorea' )
      debug = debug_prorea
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prorea'
         write(irefwr,1) 'kread, iseqprob', kread, iseqprob
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set some constants

      ipiinp = 1
      lengin = 0
      iprinp = 1
      lengrl = 0
      nprob  = 0
      leniinpfp = 60
      newstyle = 0   ! at this moment old style

!     --- Extract information from kread

      if ( kread==0 ) then

!     --- kread = 0, problem has not been read

         icode = 0
         iprob = 0

      else if ( kread==1 ) then

!     --- kread = 1, problem has been read, but not problem number

         icode = 7
         iprob = 0

      else

!     --- kread>1, problem has been read, problem number = kread-10

         icode = 7
         iprob = kread-10
         if ( iprob/=1 ) then

!        --- problem number not equal to 1

            call errint ( iprob, 1 )
            call errint ( 1, 2 )
            call errsub ( 695, 2, 0, 0 )
            call errcrd

         end if

      end if

      maxloc = maxrl

!     --- First read the text PROBLEM

      if ( icode==0 ) then

!     --- icode = 0

         ityprd = 1
         call asknxt
         if ( debug ) then
            write(irefwr,1) 'ityprd', ityprd
            if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )
         if ( ityprd/=4 .or. string(1:3)/='PRO' ) then
            call errsub ( 18, 0, 0, 0 )
            call errcrd
         end if

      end if
      iprint = 0
      if ( ierror/=0 ) go to 1000

!     --- New problem

200   nprob  = nprob + 1
      if ( debug ) then
         write(irefwr,1) 'nprob, ipiinp, iprob', nprob, ipiinp, iprob
         write(irefwr,1) 'lastitem/1', lastitem
      end if  ! ( debug )

!     --- Read rest of information following keyword problem

      table = 0d0
      iretur = -10
      firstrecord = .true.  ! first record has not been interpreted

      nncode = 2
      code(1) = 'SEQUENCE_NUMBER'
      code(2) = 'DEFAULT'
      ncode(1:2) = 3
      ncodmx(1) = 15
      ncodmx(2) = 7
      itype(1) = 0
      itype(2) = 2
      tabmin(1) = 1d0
      tabmin(2) = 0d0
      tabmax(1) = 1d1
      tabmax(2) = 0d0
      nmain = 0

      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

      iseqprob = nint(table(1))  ! sequence number input
      if ( debug ) then
         write(irefwr,1) 'iseqprob, ityprd', iseqprob, ityprd
         call prinin ( iseqkeyw, nseqkeyw, 'iseqkeyw' )
      end if  ! ( debug )

      if ( table(2)>0d0 ) then

!     --- keyword DEFAULT found after PROBLEM
!         The default problem definition is used and the rest of the input is
!         skipped

         call proreadefault ( iinput, nelgrp, lengin, lengrl )
         go to 1000

      end if  ! ( table(2)>0d0 )

      if ( iprob==0 ) then

!     --- Problem has not been read yet

         if ( nseqkeyw>0 .and. iseqkeyw(1)<0 ) then

!        --- number found, check if number read is equal to problem number

            ih = nint(tabout(-iseqkeyw(1)))
            if ( nprob/=ih ) then

!           --- problem numlevelset, numreadnumber not equal to nprob

               call errint ( ih, 1 )
               call errint ( nprob, 2 )
               call errsub ( 695, 2, 0, 0 )
               call errcrd

            end if  ! ( nprob/=ih )
            if ( debug ) write(irefwr,1) 'nprob, ih', nprob, ih

         else

!        --- no number found

            if ( nprob>1 ) then

!           --- nprob>1, there are more problems, hence
!               error: no problem number found

               call errint ( nprob, 1 )
               call errsub ( 694, 1, 0, 0 )
               call errcrd

            end if

         end if  ! ( iseqkeyw(1)<0 )

      end if  ! ( iprob==0 )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iseq

      iseq = 0
      icode = 0
      iinp = leniinpfp
      iinput(ipiinp+1:ipiinp+leniinpfp) = 0
      iinput(ipiinp) = 1

      call sepcopycodemn1 ( codeprob, nncodeprob, nscodeprob, ncodeprob )

!     --- Read and interpret next record

      finished = .false.
      do while ( .not. finished )

!     --- Loop to interpret next record

         if ( icode==0 ) then
            call asknxt
            ierpri = 0
            icode = sepmainkeyw()
         end if
         if ( debug ) write(irefwr,1) 'icode, iinp', icode, iinp

!        --- Read next of input depending on icode

         select case (icode)

         case(1)

!        --- NATBOUNCOND found

            if ( iseq(2,1)/=0 ) then

!           --- Natbouncond found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(2,1) = iinp
            call prord1 ( iinput(ipiinp), maxlen, iinp, iinputlc )
            iseq(2,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/1', iinp
            firstrecord = .false.  ! first record interpreted

         case(2)

!        --- essential boundary conditions

            if ( iseq(4,1)/=0 ) then

!           --- ESSBOUND found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(4,1) = iinp
            call prord3 ( iinput(ipiinp), maxlen, iinp, iinputlc )
            iseq(4,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/2', iinp
            firstrecord = .false.

         case(3)

!        --- END found

            finished = .true.
            if ( debug ) write(irefwr,1) 'iinp/3', iinp

         case(4)

!        --- local transformations

            if ( iseq(5,1)/=0 ) then

!           --- LOCALTRANSFORM found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(5,1) = iinp
            call prord4 ( iinput(ipiinp), maxlen, &
                          rinput, lenloc, maxloc, iinp, iinputlc )
            iseq(5,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/4', iinp
            firstrecord = .false.

         case(5)

!        --- boundary conditions of the type u equals unknown constant

            if ( iseq(6,1)/=0 ) then

!           --- UNKNOWNCONSTANT found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(6,1) = iinp
            call prord5 ( iinput(ipiinp), maxlen, iinp )
            iseq(6,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/5', iinp
            firstrecord = .false.

         case(6,19)

!        --- Reorder (used to be renumber)
!            Read numbers for renumbering of unknowns

            if ( iseq(7,1)/=0 ) then

!           --- REORDER found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(7,1) = iinp
            call prord6 ( iinput(ipiinp), maxlen, iinp )
            iseq(7,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/6', iinp
            firstrecord = .false.

         case(7)

!        --- PROBLEM found

            finished = .true.
            lastitem = 1

         case(8)

!        --- TYPES found

            if ( iseq(1,1)/=0 ) then

!           --- Types found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd
               go to 1000

            end if
            iseq(1,1) = iinp
            maxrllc = 2*nelgrp
            maxloc = maxloc-maxrllc
            iprinplc = iprinp+maxloc

            call prord0 ( iinput(ipiinp), rinput(iprinp), &
                          lenloc, maxlen, maxloc, iinp, iinputlc, &
                          rinput(iprinplc), maxrllc )
            maxloc = maxloc+maxrllc
            iseq(1,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/8', iinp
            firstrecord = .false.

         case(9)

!        --- Boundary elements found

            if ( iseq(3,1)/=0 ) then

!           --- Bounelements found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(3,1) = iinp
            call prord2 ( iinput(ipiinp), maxlen, iinp, iinputlc )
            iseq(3,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/9', iinp

            if ( iseq(2,1)==0 ) then

!           --- Boundary elements found, but not natbouncond
!               Fill type numbers for natural boundary conditions

               if ( iseq(1,1)==0 ) then

!              --- Type numbers have not been read before boundary elements

                  call errsub ( 3203, 0, 0, 0 )
                  call errcrd
                  go to 1000

               end if  ! ( iseq(1,1)==0 )

               iseq(2,1) = iinp
               call prordnatboun ( iinput(ipiinp), iinp, nelgrp, &
                                   iinput(ipiinp+iseq(1,1)) )
               iseq(2,2) = iinp

            end if  ! ( iseq(2,1)==0 )
            firstrecord = .false.

         case(10)

!        --- GLOBAL_UNKNOWNS found

            if ( iseq(8,1)/=0 ) then

!           --- GLOBAL_UNKNOWNS found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(8,1) = iinp
            call prord7 ( iinput(ipiinp), maxlen, iinp, iinputlc )
            iseq(8,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/10', iinp
            firstrecord = .false.

         case(11)

!        --- GLOBAL_ELEMENTS found

            if ( iseq(9,1)/=0 ) then

!           --- GLOBAL_ELEMENTS found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(9,1) = iinp
            call prord8 ( iinput(ipiinp), maxlen, iinp, iinputlc )
            iseq(9,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/11', iinp
            firstrecord = .false.

         case(12)

!        --- GLOBAL_RENUMBERING found

            if ( iseq(10,1)/=0 ) then

!           --- GLOBAL_RENUMBERING found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(10,1) = iinp
            call prord9 ( iinput(ipiinp), maxlen, iinp )
            iseq(10,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/12', iinp
            firstrecord = .false.

         case(13)

!        --- PRINT_LEVEL found, read number

            call prordprintlev ( icode, iprint, codemain )
            firstrecord = .false.

         case(14)

!        --- FICTITIOUS_UNKNOWNS found

            if ( iseq(11,1)/=0 ) then

!           --- FICTITIOUS_UNKNOWNS found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(11,1) = iinp
            call prordfictunk ( iinput(ipiinp), maxlen, iinp, iinputlc )
            iseq(11,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/14', iinp
            firstrecord = .false.

         case(15)

!        --- FICTITIOUS_ELEMENTS found

            if ( iseq(12,1)/=0 ) then

!           --- FICTITIOUS_ELEMENTS found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(12,1) = iinp
            call prordfictelm ( iinput(ipiinp), maxlen, iinp, iinputlc )
            iseq(12,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/15', iinp
            firstrecord = .false.

         case(16)

!        --- SKIP_ELEMENTS found

            if ( iseq(13,1)/=0 ) then

!           --- SKIP_ELEMENTS found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(13,1) = iinp
            call prordskipelm ( iinput(ipiinp), maxlen, iinp )
            iseq(13,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/16', iinp
            firstrecord = .false.

         case(17)

!        --- LEVELSET found

            if ( iseq(14,1)/=0 ) then

!           --- LEVELSET found more than once

               call errchr ( codemain(icode), 1 )
               call errsub ( 1628, 0, 0, 1 )
               call errcrd

            end if
            iseq(14,1) = iinp
            call prordlevelset ( iinput(ipiinp), maxlen, iinp )
            iseq(14,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/17', iinp
            firstrecord = .false.

         case(18)

!        --- NUM_LEVELSET found

            call prordnumlevs ( icode, iinput(ipiinp), codemain )
            firstrecord = .false.

         case(20)

!        --- PERIODICAL_BOUNDARY_CONDITIONS found

            if ( debug ) write(irefwr,1) 'iinp/18', iinp
            iseq(15,1) = iinp
            call prordperbc ( iinput(ipiinp), maxlen, iinp, rinput, &
                              lenloc, maxrl )
            iseq(15,2) = iinp
            if ( debug ) write(irefwr,1) 'iinp/19', iinp
            firstrecord = .false.

         case(21)

!        --- NEWSTYLE found

            newstyle = 1   ! temporarily
            firstrecord = .false.
            ityprd = 1
            icode = 0

         case(22)

!        --- OLDSTYLE found

            newstyle = 0   ! temporarily
            firstrecord = .false.
            ityprd = 1
            icode = 0

         case default

!        --- keyword not recognized

            if ( debug ) write(irefwr,*) 'firstrecord', firstrecord
            if ( firstrecord ) then

!           --- First record not yet found, assume types

               iseq(1,1) = iinp
               maxrllc = 2*nelgrp
               maxloc = maxloc-maxrllc
               iprinplc = iprinp+maxloc
               icode = -11                 ! record has been read before

               call prord0 ( iinput(ipiinp), rinput(iprinp), &
                             lenloc, maxlen, maxloc, iinp, iinputlc, &
                             rinput(iprinplc), maxrllc )
               maxloc = maxloc+maxrllc
               iseq(1,2) = iinp

            else

!           --- First record found, keyword not recognized

               call errint ( icode, 1 )
               call errchr ( 'icode', 1 )
               call errsub ( 3098, 1, 0, 1 )
               go to 1000

            end if  ! ( firstrecord )

         end select  ! (icode)
         if ( ierror/=0 ) go to 1000

      end do  ! while ( .not. finished )

!     --- rearrange array iinput

      if ( iinp>maxlen-iinp ) then
         call errint ( iinp, 1 )
         call errint ( maxlen-iinp, 2 )
         call errsub ( 29, 2, 0, 0 )
      end if

!     --- rearrange array iinput

      call prordr ( iinput(ipiinp), iinput(ipiinp+iinp), &
                    iseq, iinp, nprob )
      if ( ierror/=0 ) go to 1000
      iinput(ipiinp+58) = newstyle   ! temporarily

      if ( icode==7 ) then

!     --- icode = 7, New problem: restart

         if ( debug ) &
            write(irefwr,1) 'iinp, maxlen, ipiinp', &
                             iinp, maxlen, ipiinp
         maxlen = maxlen - iinp
         ipiinp = ipiinp + iinp
         iprinp = iprinp + lenloc
         maxloc = maxloc - lenloc
         go to 200

      else

!     --- End reached, compute length of iinput

         lengin = ipiinp + iinp - 1
         lengrl = iprinp + lenloc - 1

      end if

1000  lprob = nprob
      iinput(35) = iprint

      if ( debug ) then

!     --- debug is true, print iinput and rinput

         call prinin ( iinput, lengin, 'iinput end of prorea' )
         call prinrl ( rinput, lengrl, 'rinput end of prorea' )
         call printprobinput ( iinput, nprob, rinput )
         write(irefwr,*) 'End prorea'

      end if
      call erclos ( 'prorea' )

      end subroutine prorea
