      subroutine sepcommatpm ( np, nu, npm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create matrix structure for pressure mass matrix
!     Old subroutine incommatpm
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: np, nu, npm

!     np             i    Number of pressure unknowns
!     npm            o    Size of second part of structure of pressure mass
!                         matrix
!     nu             i    Number of velocity unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ibegin, iend, j
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     ibegin         Starting value for loop counter
!     iend           End value for loop counter
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPINTMATPM    Transform symmetric compact storage scheme to
!                    row-compact storage
!                    Special application PM-matrix for simple
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatpm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatpm'
         call sepprincompstr ( np,  irowsii(nu+1), icolsii )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      allocate ( IM%rowsPM(np+1), IM%diagPM(np) )
      irowsPM => IM%rowsPM

      npm = 0
      irowsPM(1) = 0
      ibegin = irowsii(nu+1)+1
      do i = 1, np
         iend = irowsii(nu+1+i)

         do j = ibegin, iend
            if ( icolsii(j)>nu ) then

!           --- column number larger than nu+1

               npm = npm+1
               icolsPM(npm) = icolsii(j)-nu

            end if

         end do  ! j = ibegin, iend
         npm = npm+1
         icolsPM(npm) = i
         ibegin = iend+1
         irowsPM(i+1) = npm

      end do  ! i = 1, np

      if ( debug ) then
         call sepprincompstr ( np,  irowsPM, icolsPM )
         write(irefwr,1) 'nu, np, npm', nu, np, npm
      end if
      allocate ( IM%colsPM(npm) )

      IM%colsPM = icolsPM(1:npm)
      deallocate ( icolsPM )

      call sepintmatpm

1000  if ( debug ) then

!     --- Debug information

         call sepprincompstr ( np,  IM%rowsPM, IM%colsPM )
         write(irefwr,*) 'End sepcommatpm'

      end if  ! ( debug )
      call erclos ( 'sepcommatpm' )

      end subroutine sepcommatpm
