      subroutine sepedg01
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates the edges part 1 pointers and initialization
!     stores number of connecting elements in ncone
!     Old subroutine topedg01
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipd, ielgrp, ielem

!     ielem          Element number
!     ielgrp         element group number
!     ipd            pointer in kmeshaad
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPEDG011      Create the first part of the edges array for one
!                    element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepedg01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepedg01'
  1      format ( a, 1x, (10i6) )

      end if

      ipd = nelem + 1
      ielem = 0
      if ( debug ) write(irefwr,1) 'nelem, nelgrp', nelem, nelgrp

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf ( ielgrp )

!        --- Fetch number of edges per element

         if ( ishape==-10001 ) then
            nedpel = 0
         else if ( ishape==-9 ) then
            nedpel = kmesha(1+ielgrp)-kmesha(ielgrp)-1
         else
            nedpel = sepgetnedpel(ielgrp)
         end if
         if ( nedpel==-1 ) nedpel = 0
         if ( debug ) &
            write(irefwr,1) 'ielgrp, inelem, ishape, nedpel', &
                             ielgrp, inelem, ishape, nedpel

!        --- Loop over the elements in this group

         call sepedg011 ( ielem, ipd )
         if ( debug ) write(irefwr,*) 'ipd ', ipd

      end do

!     --- Finish the job

      length = ipd - 1

      if ( debug ) then

!     --- Debug information

         call prinin ( nmeshd, nelem, 'nmeshd part 1' )
         write(irefwr,*) 'End sepedg01'

      end if
      call erclos ( 'sepedg01' )

      end
