      subroutine sepelgrpedges ( edgeelems, edgelist, lenedgelist )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-04-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a list of all edges corresponding to 2 element groups
!     Old subroutine topelgrpedges
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer edgeelems(*), edgelist(*), lenedgelist

!     edgeelems      i    Contains the element numbers an edge is connected with
!                         in the following way:
!                         Position 1 to nedges contains starting addresses
!                         for each edge
!                         For each edge from the starting position the number
!                         of elements is stored followed by the element numbers
!     edgelist       o    List of edges that correspioond to 2 element groups
!     lenedgelist    o    Number of elements in array edgelist
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer elgrps(nelem), icount, iedge, istart, numelms, &
              i, ielgrp, nel, ielem1, ielem2

!     elgrps         Array of length nelem containg the element group sequence
!                    number for each element
!     i              Counting variable
!     icount         Counter
!     iedge          edge sequence number
!     ielem1         First element number in edge
!     ielem2         Second element number in edge
!     ielgrp         element group number
!     istart         First position used at a part
!     nel            number of elements
!     numelms        Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array elgrps with elements group numbers of each element
!     icount := 0
!     for iedge := 1 (1) nedges do
!        if ( number of elements corresponding to edge = 2 ) then
!           if ( both elements correspond to different element groups ) then
!              icount := icount+1
!              edgelist(icount) := iedge
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelgrpedges' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelgrpedges'

      end if

!     --- Fill array elgrps with elements group numbers of each element

      icount = 0
      if ( nelgrp==1 ) then

!     --- Only one element group: return

         icount = 0
         go to 1000

      end if
      nel = 0
      do ielgrp = 1, nelgrp
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         if ( debug ) &
            write(irefwr,*) 'element group topel ', ielgrp, ishape
         if ( ishape==-10001 .or. ishape.eq.109 .or. &
              ishape==134 .or. ishape.eq.50 .or. ishape.eq.-9 ) then

!        --- Special element that mus be skipped

            do i = nel+1, nel+inelem
               elgrps(i) = 0
            end do

         else

!        --- Standard case

            do i = nel+1, nel+inelem
               elgrps(i) = ielgrp
            end do

         end if  ! ( ishape==-10001 ... )

         nel = nel+inelem

      end do  ! ielgrp = 1, nelgrp

!     --- for iedge := 1 (1) nedges do

      do iedge = 1, nedges

         istart = edgeelems(iedge)         ! starting address of iedge
         numelms =  edgeelems(istart)      ! number of elements corresponding to
                                           ! iedge
         if ( numelms==2 ) then

!        --- number of elements corresponding to edge = 2

            ielem1 = edgeelems(istart+1)
            ielem2 = edgeelems(istart+2)
            if ( elgrps(ielem1)/=elgrps(ielem2) .and. &
                 elgrps(ielem1)/=0 .and. elgrps(ielem2).ne.0 ) then

!           --- both elements correspond to different element groups
!               they do not belong to a special element
!               icount := icount+1
!               edgelist(icount) := iedge

               icount = icount+1
               if ( icount>lenedgelist ) then

!              --- Array edgelist too small

                  call errint ( icount, 1 )
                  call errint ( lenedgelist, 2 )
                  call errchr ( 'edgelist', 1 )
                  call errsub ( 550, 2, 0, 1 )
                  go to 1000

               end if
               edgelist(icount) = iedge

            end if  ! ( elgrps(ielem1)/=elgrps(ielem2) )

         end if  ! ( numelms==2 )

      end do  ! iedge = 1, nedges
      lenedgelist = icount

1000  if ( debug ) then

!     --- Debug information

         call prinin ( edgelist, lenedgelist, &
                       'edges belonging to two groups' )
         write(irefwr,*) 'End sepelgrpedges'

      end if
      call erclos ( 'sepelgrpedges' )

      end
