      subroutine sepelgrpnodes
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 16-07-2012 Remove iwork from parameter list
!        version  1.0  date 29-11-2001
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part ae with all nodes corresponding to element groups
!     Old subroutine topelgrpnodes
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element_group
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ikelmc, i, istart_ielgrp, &
              ielgrp, istart, irelem, nodp, lenielgrpnods
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     istart         First position used at a part
!     istart_ielgrp  Starting position of nodes for element group
!     iwork          integer work array of length npoint
!                    iwork is used to indicate if a nodal point has been
!                    stored for the element group (>0) or not (0)
!     lenielgrpnods  Length of array ielgrp_nodes
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nelgrp := number of element groups
!     Clear array ielgrp_nodes positions 1 to nelgrp
!     istart_ielgrp := nelgrp+1
!     For ielgrp := 1 (1) nelgrp do
!        inelem := number of elements in element group
!        inpelm := number of nodes per element in element group
!        ielgrp_nodes(ielgrp) := istart_ielgrp
!        istart := istart_ielgrp
!        Clear array iwork positions 1 to npoint
!        For irelem := 1 (1) inelem do
!           For i := 1 (1) inpelm do
!              nodp := i-th node in element
!              if ( iwork(nodp)=0 ) then
!                 iwork(nodp) := 1
!                 istart := istart+1
!                 ielgrp_nodes(istart) := nodp
!        ielgrp_nodes(istart_ielgrp) := istart-istart_ielgrp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelgrpnodes' )
      ikelmc = 0
      lenielgrpnods = 3*(npoint+nelgrp)

      allocate ( iwork(npoint), ielgrp_nodes(lenielgrpnods), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+lenielgrpnods, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array ielgrp_nodes positions 1 to nelgrp

      ielgrp_nodes(1:nelgrp) = 0

!     --- istart_ielgrp := nelgrp+1

      istart_ielgrp = nelgrp+1

!     --- For ielgrp := 1 (1) nelgrp do

      do ielgrp = 1, nelgrp

!     --- inelem := number of elements in element group
!         inpelm := number of nodes per element in element group

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm

!        --- ielgrp_nodes(ielgrp) := istart_ielgrp
!            istart := istart_ielgrp

         ielgrp_nodes(ielgrp) = istart_ielgrp
         istart = istart_ielgrp

!        --- Clear array iwork positions 1 to npoint

         iwork = 0

!        --- For irelem := 1 (1) inelem do

         do irelem = 1, inelem

!        --- For i := 1 (1) inpelm do

            do i = 1, inpelm

!           --- nodp := i-th node in element

               nodp = kmeshc(ikelmc+i)
               if ( iwork(nodp)==0 ) then

!              --- iwork(nodp)=0
!                  iwork(nodp) := 1
!                  istart := istart+1
!                  ielgrp_nodes(istart) := nodp

                  iwork(nodp) = 1
                  istart = istart+1
                  ielgrp_nodes(istart) = nodp

               end if

            end do
            ikelmc = ikelmc+inpelm

         end do

!        --- ielgrp_nodes(istart_ielgrp) := istart-istart_ielgrp

         ielgrp_nodes(istart_ielgrp) = istart-istart_ielgrp
         istart_ielgrp = istart+1
         if ( istart>lenielgrpnods ) then

!        --- lenielgrpnods too small

            call errint ( istart, 1 )
            call errint ( lenielgrpnods, 2 )
            call errchr ( 'ielgrp_nodes', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

      end do

      lenielgrpnods = istart
      if ( km(iseqkm)%kmesh(55)>0 ) deallocate(km(iseqkm)%ielgrp_nodes)
      allocate ( km(iseqkm)%ielgrp_nodes(lenielgrpnods) )
      km(iseqkm)%kmesh(55) = lenielgrpnods
      kelmae => km(iseqkm)%kmesh(55)
      km(iseqkm)%ielgrp_nodes = ielgrp_nodes(1:lenielgrpnods)

      deallocate ( iwork, ielgrp_nodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      ielgrp_nodes => km(iseqkm)%ielgrp_nodes

1000  call erclos ( 'sepelgrpnodes' )

      end
