      subroutine sepelmedges ( edgeelems, lenedgeelems )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-04-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all elements corresponding to the edges in array edgeelems
!     Old subroutine topelmedges
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer edgeelems(*), lenedgeelems

!     edgeelems      o    Contains the element numbers an edge is connected with
!                         in the following way:
!                         Position 1 to nedges contains starting addresses
!                         for each edge
!                         For each edge from the starting position the number
!                         of elements is stored followed by the element numbers
!     lenedgeelems  i/o   Length of array edgeelems
!                         At input estimated length
!                         At output computed length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, numedges, j, numelems, jstart, iedge

!     i              Counting variable
!     iedge          edge sequence number
!     istart         First position used at a part
!     j              Counting variable
!     jstart         starting position
!     numedges       number of edges
!     numelems       Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract parameters from array kmesh
!     Clear array edgeelems(1:nedges)
!     Fill number of elements coresponding to each edge in first part of
!     edgeelems
!     Fill starting addresses in edgeelems and clear part 2
!     Fill array edgeelems part 2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmedges' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmedges'
         write(irefwr,1) 'nelem, nedges', nelem, nedges
  1      format ( a, 1x, (10i6) )

      end if


!     --- Clear array edgeelems(1:nedges)

      edgeelems(1:nedges) = 0

!     --- Fill number of elements coresponding to each edge in first part of
!         edgeelems

      do i = 1, nelem
         istart = nmeshd(i)
         numedges = nmeshd(istart)
         do j = 1, numedges
            iedge = abs(nmeshd(istart+j))
            edgeelems(iedge) = edgeelems(iedge)+1
         end do
      end do  ! i = 1, nelem

!     --- Fill starting addresses in edgeelems and clear part 2

      istart = nedges+1
      do iedge = 1, nedges
         numelems = edgeelems(iedge)
         edgeelems(iedge) = istart
         istart = istart+numelems+1
      end do
      istart = istart-1
      if ( istart>lenedgeelems ) then

!     --- Array edgeelems too small

         call errint ( istart, 1 )
         call errint ( lenedgeelems, 2 )
         call errchr ( 'edgeelems', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      lenedgeelems = istart
      do i = 1, nedges
         edgeelems(edgeelems(i)) = 0
      end do

!     --- Fill array edgeelems part 2

      do i = 1, nelem
         istart = nmeshd(i)
         numedges = nmeshd(istart)
         do j = 1, numedges
            iedge = abs(nmeshd(istart+j))
            jstart = edgeelems(iedge)
            numelems = edgeelems(jstart)+1
            edgeelems(jstart) = numelems
            edgeelems(jstart+numelems) = i
         end do
      end do  ! i = 1, nelem

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'elements corresponding to edges'
         call prinnmeshx ( nedges, edgeelems )
         write(irefwr,*) 'End sepelmedges'

      end if
      call erclos ( 'sepelmedges' )

      end
