      subroutine sepfaceinfo ( ierrors, nerrors )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write extra information concerning faces that are coupled to more than
!     two internal elements
!     So extra information concerning an incorrect mesh is printed
!     Old subroutine topfaceinfo
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     error_manipulation
!     mesh
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ierrors(*), nerrors

!     ierrors        i    Array containing face numbers of faces that are
!                         marked as incorrect
!     nerrors        i    Number of faces stored in ierrors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iface, ielem, j, ipstart

!     i              Counting variable
!     ielem          Element number
!     iface          Face sequence number
!     ipstart        Starting address in array
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPPRINELMNOD1 Print all nodes corresponding to an element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) nerrors do
!        iface := ierrors(i)
!        write all elements corresponding to face
!        write all nodes corresponding to element
!        write coordinates of these nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfaceinfo' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfaceinfo'

      end if
      if ( ierror/=0 ) go to 1000

!     --- For i := 1 (1) nerrors do

      do i = 1, nerrors

!     --- iface := ierrors(i)
!         write all elements corresponding to face
!         write all nodes corresponding to element
!         write coordinates of these nodes

         iface = ierrors(i)
         ipstart = nmeshf(iface)
         write(irefwr,*) 'face number ', iface
         call prinin ( nmeshf(ipstart+1), nmeshf(ipstart), 'elements' )
         do j = 1, nmeshf(ipstart)
            ielem = nmeshf(ipstart+j)
            call sepprinelmnod1 ( ielem )
         end do  ! j = 1, nmeshf(ipstart)

      end do  ! i = 1, nerrors

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfaceinfo'

      end if
      call erclos ( 'sepfaceinfo' )

      end
