      subroutine sepfillkmeshe1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute length of kmesh part e2
!     Old subroutine topnp0
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     node
!     neighbor
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, i, lenkneigh, &
              isum, ikelm, ih, neli, &
              isum1, j, ielem, ikelm2, k, nodp
      integer, allocatable :: iwork(:), kneigh(:)

!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     ikelm
!     ikelm2
!     inpelm         number of points per element
!     isum           Integer variable to compute a sum
!     isum1
!     iwork          integer work array  see toplm1 and toplm2
!     j              Counting variable
!     k              Counting variable
!     kneigh         Contains neigboring nodes
!     lenkneigh      Declared length of array kneigh
!     neli           number of elements coupled to a node
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
!     SEPPOINTKMSHC  Fill starting addresses of each element in kmeshc
!                    into kmeshcpoint
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1381   Too few space available in array
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmeshe1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmeshe1'
         call ertrace

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- iwork will be cleared

      lenkneigh = npoint

      allocate ( iwork(npoint), kmeshcpoint(nelem+1), kneigh(lenkneigh), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npoint+nelem+1+lenkneigh, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iwork=0

!     --- the nodal points are considered
!         kmeshcpoint will be filled with pointers to kmesh part c

      call seppointkmshc

!     --- fill kmesh parts e1 and e

      isum=0
      ikelm=0
      do i=1,npoint
         kmshe1(i)=isum
         ih=kmshd1(i+1)
         neli=ih-ikelm
         isum1=0
         iwork(i)=2
         do j=1,neli
            ielem=kmshd2(ikelm+j)
            ikelm2=kmeshcpoint(ielem)
            inpelm=kmeshcpoint(ielem+1)-ikelm2
            do k=1,inpelm
               nodp=kmeshc(ikelm2+k)
               if(iwork(nodp)==0) then

!              --- new nodal point

                  iwork(nodp)=1
                  isum=isum+1
                  isum1=isum1+1
                  if(isum1>lenkneigh) then
                     call errint ( isum, 1 )
                     call errint ( lenkneigh, 2 )
                     call errsub ( 1381, 2, 0, 0 )
                     go to 1000
                  end if
                  kneigh(isum1)=nodp
               end if
            end do
         end do

!        --- positions in iwork with iwork(i)>0 will be cleared

         iwork(i)=0
         iwork(kneigh(1:isum1))=0
         ikelm=ih
      end do
      kmshe1(npoint+1)=isum

      deallocate ( iwork, kneigh, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmshe1, npoint+1, 'kmeshe1' )
         write(irefwr,*) 'End sepfillkmeshe1'

      end if  ! ( debug )
      call erclos ( 'sepfillkmeshe1' )

      end
