      subroutine sepfillkmshd1 ( ielem, ielgrp, markelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 12-06-2015 New parameter list
!        version  1.1  date 18-06-2014 Extra debug
!        version  1.0  date 28-08-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill part of kmeshd1 for one element group
!     Only elements marked in markelem are considered
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     parallel
!     mesh
!     element
!     node
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: markelem(*), ielgrp
      integer, intent(inout) :: ielem

!     ielem         i/o   Last element number used
!     ielgrp         i    element group number
!     markelem       i    Work array to mark elements that must be used
!                         0: element is skipped
!                         1: element is used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irelem, nodp, j, inpelm

!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     j              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmshd1' )
      debug = .false. .and. ioutp>=0
      kmeshc2d => Topgrp(ielgrp)%topol_elg
      inpelm = Topgrp(ielgrp)%inpelm

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmshd1'
         call ertrace
         write(irefwr,1) 'ielgrp, inpelm, inelem, ielem ', &
                          ielgrp, inpelm, Topgrp(ielgrp)%inelem, ielem
         call prinin1 ( kmeshc2d, Topgrp(ielgrp)%inelem, &
                        inpelm, 'kmeshc2d' )
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do irelem = 1, Topgrp(ielgrp)%inelem

!     --- Loop over nodes in element

         if ( markelem(ielem+irelem)==1 ) then

!        --- Element is marked

            do j = 1, inpelm
               nodp = kmeshc2d(j,irelem)
               kmshd1(nodp) = kmshd1(nodp)+1
            end do  ! j = 1, Topgrp(ielgrp)%inpelm

         end if  ! ( markelem(ielem+irelem)==1 )

      end do  ! irelem = 1, Topgrp(ielgrp)%inelem

      ielem = ielem+Topgrp(ielgrp)%inelem

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillkmshd1'

      end if  ! ( debug )
      call erclos ( 'sepfillkmshd1' )

      end subroutine sepfillkmshd1
