      subroutine sepfillkmshd2 ( ielgrp, iwork, ielem, markelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 12-06-2015 New parameter list
!        version  1.1  date 07-02-2014 Change format
!        version  1.0  date 28-08-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmshd2 for one element group
!     Only elements marked in markelem are considered
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     parallel
!     mesh
!     element
!     node
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ielgrp, markelem(*)
      integer, intent(inout) :: ielem, iwork(*)

!     ielem         i/o   Element number
!     ielgrp         i    element group number
!     iwork         i/o   integer work array
!                         Must be filled with 1 at first call
!                         iwork(i) contains the next sequence number of an
!                         element to be stored in kmeshd
!     markelem       i    Work array to mark elements that must be used
!                         0: element is skipped
!                         1: element is used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: irelem, j, nodp, ih, inpelm

!     ih             Help variable to store a constant temporarily
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     j              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmshd2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmshd2'
         call ertrace
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kmeshc2d => Topgrp(ielgrp)%topol_elg
      inpelm = Topgrp(ielgrp)%inpelm

      do irelem = 1, Topgrp(ielgrp)%inelem

         ielem = ielem+1

         if ( markelem(ielem)==1 ) then

!        --- Element is marked
!            Loop over nodes in element

            do j = 1, inpelm

               nodp = kmeshc2d(j,irelem)
               ih = iwork(nodp)
               if ( debug ) &
                  write(irefwr,1) 'ielem, nodp, ih, kmeshd(nodp)', &
                                   ielem, nodp, ih, kmshd1(nodp)
               kmshd2(ih+kmshd1(nodp)) = ielem
               iwork(nodp) = ih+1

            end do  ! j = 1, inpelm

         end if  ! ( markelem(ielem+irelem)==1 )

      end do  ! irelem = 1, Topgrp(ielgrp)%inelem

1000  call erclos ( 'sepfillkmshd2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillkmshd2'

      end if  ! ( debug )

      end subroutine sepfillkmshd2
