      subroutine sepfillmatstr
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 11-12-2017 New storage of intmat
!        version  2.0  date 06-01-2016 Use sepmodulematstr
!        version  1.0  date 27-12-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill parameters in module sepmodulematstr
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipstart
      integer :: icounter
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     icounter       Counter for number of times this subroutine is called
!     ipstart        Starting address of array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data icounter /1/
! ======================================================================
!
      call eropen ( 'sepfillmatstr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillmatstr'
         write(irefwr,1) 'intmat, iprob', intmat, iprob
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'icounter', icounter

      IM => intmt(intmat)

!     --- Skip filling if this is the same array as before

      iseqim = intmat
      if ( IM%icounter==icounter ) go to 900
      icounter = icounter+1
      IM%icounter = icounter
      if ( debug ) write(irefwr,1) 'icounter/2', icounter

!     --- Get information and associate sub arrays

      iprob = IM%iprob
      irowsii => IM%rowsii
      icolsii => IM%colsii
      irowsip => IM%rowsip
      icolsip => IM%colsip
      irowsp => IM%rowspp
      icolsp => IM%colspp
      irowsic => IM%rowsic
      icolsic => IM%colsic
      irowsfi => IM%rowsfi
      icolsfi => IM%colsfi
      irowsif => IM%rowsif
      icolsif => IM%colsif
      irowsout => IM%rowsout
      icolsout => IM%colsout
      storeinfo => IM%storeinfo

      intdia => irowsii
      nrusol = IM%nrusol

      if ( .not. IM%direct .and. (.not. IM%symmetric .or. IM%mumps) ) &
         intdia => IM%diag

      if ( debug ) write(irefwr,*) 'parallel ', parallel

      if ( parallel ) then

         kprobpar => kprobpart%kprobpar
         typecoupl = IM%typecoupl
         if ( typecoupl==1 ) then
            indprh = kprobpar(4) ! Dirichlet-Dirichlet coupling
            kprobh => kprobhdd
            kprobhinv => kprobhddinv
         end if  ! ( typecoupl==1 )

         recunknowns => IM%recunknowns
         sendunknowns => IM%sendunknowns
         nglob = kprobpar(2)   ! number of global degrees of freedom

      else

!     --- Serial case

         nnz => irowsii
         onnz => icolsii

      end if  ! ( parallel )

      if ( IM%simple ) then

!     --- Simple method

        nrusolvel = IM%nu
        nrusolpress = IM%np

        irowsQ => IM%rowsQ
        icolsQ => IM%colsQ
        irowsG => IM%rowsG
        icolsG => IM%colsG
        irowsDG => IM%rowsDG
        icolsDG => IM%colsDG
        irowsPM => IM%rowsPM
        icolsPM => IM%colsPM
        intdiaQ => IM%diagQ
        sizeqmat = size(IM%colsQ)
        sizedgmat = size(IM%colsDG)
        sizedmat = size(IM%colsG)
        sizepmmat = size(IM%colsPM)

      end if  ! ( IM%simple )

 900  if ( parallel ) then

!       --- Allocate some global arrays

        allocate ( sendwork(nglob), xglob(nglob), stat = error )

        if ( error/=0 ) call eralloc ( error, 2*nglob, 'sendwork' )
        if ( ierror/=0 ) go to 1000

!       --- Initialize xglob

        xglob = 0d0

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irowsii, 5, 'irowsii' )
         write(irefwr,*) 'End sepfillmatstr'

      end if  ! ( debug )
      call erclos ( 'sepfillmatstr' )

      end subroutine sepfillmatstr
