      subroutine sepintmat4 ( nsol, irowsnew, icolsnew, lencolsnew, &
                              rows, cols, lencols, diag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-04-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill data structure in case of symmetric compact matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nsol, lencolsnew, lencols
      integer, intent(in) :: rows(nsol+1), cols(lencols)
      integer, intent(out) :: irowsnew(nsol+1), icolsnew(lencolsnew), &
                              diag(nsol)

!     cols           i    array with column numbers
!     diag           i    array with positions of diagonal
!     icolsnew       o    New array icolsii
!     irowsnew       o    New array irowsii
!     lencols        i    size of array cols
!     lencolsnew     i    size of array icolsnew
!     nsol           i    number of unknowns with respect to these arrays
!     rows           i    Number of non-zero entries of matrix per row
!                         accumulated (rows(1)=0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ncols, icount

!     i              Counting variable
!     icount         Counter
!     ncols          number of non-zero columns in row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPPRINCOMPSTR print part of array intmat corresponding to internal
!                    unknowns. Compact storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintmat4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintmat4'
         call sepprincompstr ( nsol, rows, cols )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icount = 0
      do i = 1, nsol
         irowsnew(i) = icount
         icount = icount+1
         icolsnew(icount) = i         !diagonal
         diag(i) = icount
         ncols = rows(i+1)-rows(i)  ! number of columns
         icolsnew(icount+1:icount+ncols) = cols(rows(i)+1:rows(i+1))
         icount = icount+ncols
      end do
      irowsnew(nsol+1) = icount

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irowsnew, nsol+1, 'irowsnew' )
         call prinin ( icolsnew, lencolsnew, 'icolsnew' )
         write(irefwr,*) 'End sepintmat4'

      end if  ! ( debug )
      call erclos ( 'sepintmat4' )

      end subroutine sepintmat4
