      subroutine sepobstmarkbt ( lenipointobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill ipointobst part 4, i.e. the set of boundary nodes of an obstacle
!     that are only inside obstacle elements
!     Old subroutine mshobstmarkbt
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenipointobst

!     lenipointobst i/o   length of array ipointobst
!                         At input: present length
!                         At output: new length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, istart, lenextra, lastpos, firstpos
      integer, allocatable :: ipointobstcp(:)

!     firstpos       First position to be copied
!     iobst          Obstacle sequence number
!     ipointobstcp   Copy of old array ipointobst
!     istart         First position used at a part
!     lastpos        Last position to be copied
!     lenextra       Extra length of array ipointobst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPOBSTMARKBT1 Fill ipointobst part 4, i.e. the set of boundary nodes of
!                    an obstacle that are only inside obstacle elements
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy array ipointobst to ipointobstcp
!     For iobst := 1 (1) numobst do
!        istart := ipointobst(iobst)
!        Fill ipointobst from istart
!        Raise extra length
!        Copy next obstacle from ipointobstcp to ipointobst
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepobstmarkbt' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepobstmarkbt'
         call prinin ( ipointobst, numobst, 'ipointobst' )
         call sepprinipobst ( ipointobst )

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( ipointobstcp(lenipointobst), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lenipointobst, 'ipointobstcp' )
      if ( ierror/=0 ) go to 1000

!     --- Copy array ipointobst to ipointobstcp

      ipointobstcp(1:lenipointobst) = ipointobst(1:lenipointobst)

      lenextra = 0  ! extra length of array ipointobst

!     --- For iobst := 1 (1) numobst do

      do iobst = 1, numobst

!     --- istart := ipointobst(iobst)
!         Fill ipointobst from istart

         istart = ipointobst(iobst)
         call sepobstmarkbt1 ( ipointobst(istart) )

!        --- Raise extra length

         lenextra = lenextra+ipointobst(istart+3)

!        --- Copy next obstacle from ipointobstcp to ipointobst

         if ( iobst<numobst .and. lenextra>0 ) then

!        --- This is not the last obstacle,
!            copy information of next obstacle to ipointobst
!            First raise starting position

            ipointobst(iobst+1) = ipointobst(iobst+1)+lenextra

!           --- Compute last position of obstacle in ipointobstcp

            if ( iobst+1<numobst ) then
               lastpos = ipointobstcp(iobst+2)-1
            else
               lastpos = lenipointobst
            end if  ! ( iobst+1<numobst)

!           --- Copy

            firstpos = ipointobstcp(iobst+1)
            ipointobst(firstpos+lenextra:lastpos+lenextra) = &
               ipointobstcp(firstpos:lastpos)

         end if  ! ( iobst<numobst .and. lenextra>0 )

      end do  ! iobst = 1, numobst

!     --- Raise lenipointobst

      lenipointobst = lenipointobst+lenextra

      deallocate ( ipointobstcp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ipointobstcp' )

1000  if ( debug ) then

!     --- Debug information

         call sepprinipobst ( ipointobst )
         write(irefwr,*) 'End sepobstmarkbt'

      end if
      call erclos ( 'sepobstmarkbt' )

      end
