      subroutine topbv3free ( maxren, irenum, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprobh for free unknowns
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     renumbering
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr1
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: maxren, irenum(*), iwork(*)

!     irenum         i    This array contains the kmeshk of the renumbering of
!                         all unknowns per point
!                         So if iren(i)=j then unknown i is numbered in the j-th
!                         level
!                         Unknowns with sequence number larger than maxren
!                         correspond always to the largest level
!     iwork         i/o   Output of subroutine TOPBV2
!                         iwork(i)>0, means essential boundary condition for
!                                     unknown i
!                         iwork(i)=0, means unknown i is free
!                         iwork(i)<0, means unknown i corresponds to
!                         periodical bc.
!     maxren         i    Maximum number of unknowns that have a special
!                         numbering
!                         If <0, the unknowns must be renumbered per level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icount, i, inodp, inunkp, j, inusol

!     i              Counting variable
!     icount         Counter
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKPROBHR Fill part of kprobh corresponding to free unknowns in case
!                    of renumbering of nodes
!     TOPBV3FREE1    Fill kprobh for free unknowns (global renumbering)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv3free' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv3free'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'kelmj, maxren, nunkp, iprob, n', &
                          kelmj, maxren, nunkp, iprob

      inunkp = nunkp

      if ( kelmj==0 .and. maxren==0 ) then

!     --- kelmj = 0, and maxren=0  natural sequence

         icount = 1
         do i = 1, nusol
            if ( iwork(i)==0 ) then
               kprobh(i) = icount
               icount = icount+1
            end if
         end do

      else if ( maxren==0 ) then

!     --- kelmj>0 and maxren=0   new sequence

         call sepfillkprobhr ( iwork )

      else

!     --- Global renumbering over all nodes or kmeshk

         call topbv3free1 ( maxren, irenum, iwork )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topbv3free'

      end if  ! ( debug )
      call erclos ( 'topbv3free' )

      end subroutine topbv3free
