      subroutine topedg05 ( nmeshd, nmeshh, nedges, melped, lpnmph, &
                            nmeshc, kmshd1, kmshd2 )
! ======================================================================
!
!        programmer     Bas van Rens
!        version 1.0    date 23-08-1998 Renamed nmed05
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   creates the nmesh part h
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer nmeshd(*), nmeshh(*), nedges, melped, lpnmph, &
              nmeshc(*), kmshd1(*), kmshd2(*)

!     kmshd1     i    first part of kmesh part d, contains the number of
!                     elements corresponding to each nodal point accumulated
!                     kmshd1(1) = 0.
!                     Length: npoint+1
!     kmshd2     i    second part of kmesh part d
!                     Contains all the element numbers the nodal points are
!                     lying in sequentially
!     lpnmph     o    length of nmeshh
!     melped     o    maximum number of elements connected to one edges
!     nedges     i    total number of edges
!     nmeshc     i    nmesh part c
!     nmeshd     i    nmesh part d
!     nmeshh     o    nmesh part h
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iph, iedge, ipd, nedpel, jedpel, jedge, nelinode, &
              jelped, ielem, inode, i, ipkmd2, ipc

!     i          Counting variable
!     iedge      edge sequence number
!     ielem      Element number
!     inode      Node sequence number
!     ipc        pointer in nmeshc
!     ipd        pointer in kmeshaad
!     iph        pointer in nmeshh
!     ipkmd2     pointer in kmeshd2
!     jedge      local edge number
!     jedpel     edge in an element
!     jelped     number of elements on edge
!     nedpel     number of edges in one element
!     nelinode   number of elements coupled to node
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iph = nedges + 1
      melped = 0

!     --- Loop over the edges

      do iedge = 1, nedges

         nmeshh(iedge) = iph

!        --- get the first node in iedge

         ipc = nmeshc(iedge)
         inode = nmeshc(ipc + 1)

         jelped = 0

!        --- Loop over the elements connected to node inode

         ipkmd2 = kmshd1(inode)
         nelinode = kmshd1(inode+1)-ipkmd2

         do i = 1, nelinode

!        --- Fetch number of edges in this element

            ielem = kmshd2(ipkmd2+i)

            ipd = nmeshd(ielem)
            nedpel = nmeshd(ipd)

!           --- Loop over the edges of this element

            do jedpel = 1, nedpel

!           --- Fetch the edge labels

               jedge = nmeshd(ipd+jedpel)

!              --- Compare with current edge and store

               if ( abs(jedge).eq.iedge ) then
                  jelped = jelped + 1
                  nmeshh(iph+jelped) = ielem

!                 --- Edge found go on with next element

                  goto 100
               end if

!              --- Update pointer to part 2 and update melped

            end do
100      end do
         nmeshh(iph) = jelped
         iph = iph + jelped + 1
         if ( jelped.gt.melped ) melped = jelped
      end do

!     --- Finish this job

      lpnmph = iph - 1
      end
