      subroutine topelmfaces ( nouterfaces, outerfaces, kmeshc, inpelm, &
                               nmeshg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshc with surface elements corresponding to the faces
!     stored in array outerfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     face
!     outer_boundary
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nouterfaces, outerfaces(*), inpelm, nmeshg(*), &
              kmeshc(inpelm,nouterfaces)

!     inpelm         i    number of points per element
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nmeshg         i    nmesh part g
!     nouterfaces    i    Number of outer faces (length of outerfaces)
!     outerfaces     i    Array containing the face numbers of the outer faces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iface, i, j, ipg

!     i              Counting variable
!     iface          Face sequence number
!     ipg            pointer in kmeshaag
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all faces in array outerfaces do
!        Store element in kmeshc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topelmfaces' )

!     --- For all faces in array outerfaces do

      do i = 1, nouterfaces

!     --- Store element in kmeshc

         iface = outerfaces(i)
         ipg = nmeshg(iface)
         do j = 1, inpelm
            kmeshc(j,i) = nmeshg(ipg+j)
         end do

      end do

      call erclos ( 'topelmfaces' )
      end
