      subroutine topelmspfaces ( nouterfaces, outerfaces, kmeshc, &
                                 nmeshg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all vertices of the spectral surface elements stored in the
!     outer surfaces into an array kmeshc consisting of quadrileaterals
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     face
!     outer_boundary
!     topology
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nouterfaces, outerfaces(*), nmeshg(*), &
              kmeshc(4,nouterfaces)

!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nmeshg         i    nmesh part g
!     nouterfaces    i    Number of outer faces (length of outerfaces)
!     outerfaces     i    Array containing the face numbers of the outer faces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iface, i, nspecp, ipg
      double precision a

!     a              Help variable to store quantity
!     i              Counting variable
!     iface          Face sequence number
!     ipg            pointer in kmeshaag
!     nspecp         number of spectral points in one direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all faces in array outerfaces do
!        Store element in kmeshc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topelmspfaces' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topelmspfaces'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Detect number of points per edge

      iface = outerfaces(1)
      ipg = nmeshg(iface)
      a = nmeshg(ipg)
      nspecp = nint(sqrt(a))

!     --- For all faces in array outerfaces do

      do i = 1, nouterfaces

!     --- Store element in kmeshc

         iface = outerfaces(i)
         ipg = nmeshg(iface)
         kmeshc(1,i) = nmeshg(ipg+1)
         kmeshc(2,i) = nmeshg(ipg+nspecp)
         kmeshc(3,i) = nmeshg(ipg+nspecp*nspecp)
         kmeshc(4,i) = nmeshg(ipg+nspecp*(nspecp-1)+1)

      end do

1000  call erclos ( 'topelmspfaces' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topelmspfaces'

      end if

      end
