      subroutine topelnodes2 ( npoint, inpelm, nelem, kmshd1, kmshd2, &
                               kmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill second part of kmesh part d
!     kmesh part d contains the elements corresponding to each node
!     The first part has already been filled in topelnodes1
!     This is the same routine as toplm2, but now restricted to one
!     element group only
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, inpelm, nelem, kmshd1(*), &
              kmeshc(inpelm,nelem), kmshd2(*)

!     inpelm         i    number of points per element
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmshd1         i    first part of kmesh part d, contains the number of
!                         elements corresponding to each nodal point accumulated
!                         kmshd1(1) = 0.
!                         Length: npoint+1
!     kmshd2         o    second part of kmesh part d
!                         Contains all the element numbers the nodal points are
!                         lying in sequentially
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nodp, ih, ielem, iwork(npoint)

!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     iwork          integer work array of length npoint
!                    iwork(i) contains the next sequence number of an
!                    element to be stored in kmeshd
!     j              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     initialize parameters
!     Set iwork to 1
!     Fill kmesh part d2 in kmshd2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set iwork to 1

      do i = 1, npoint
         iwork(i) = 1
      end do

!     --- Loop over elements in element group

      do ielem = 1, nelem

!     --- Loop over nodes in element

         do j = 1, inpelm

            nodp = kmeshc(j,ielem)
            ih = iwork(nodp)
            kmshd2(ih+kmshd1(nodp)) = ielem
            iwork(nodp) = ih+1

         end do

      end do

      end
