      subroutine topfaces02 ( nmeshd, nmesha, nmeshb, nmeshh, medpfa, &
                              nfaces, length, nelem, nfapel, ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-12-2002 Extension with prisms
!        version  1.0  date 06-05-2002
!
!   copyright (c) -200-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates the faces part 2 actual face labels
!     This subroutine is restricted to one volume only
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     face
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmesha(*), nmeshb(*), nmeshd(*), nmeshh(*), &
              medpfa, nfaces, length, nelem, nfapel, ishape

!     ishape         i    Shape number of elements
!     length         o    length of nmesh part c
!     medpfa         i    maximum number of edges per face
!     nelem          i    Number of elements
!     nfaces         o    Number of faces
!     nfapel         i    number of faces in one element
!     nmesha         o    nmesh part a
!     nmeshb         o    nmesh part b
!     nmeshd         i    nmesh part d
!     nmeshh         i    nmesh part h
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipa, ipb, irelem, ielem, nedpfa(6), i, ipd, jface, iface, &
              j, k, locedges(4,6), ihelp(12)

!     i              Counting variable
!     ielem          Element number
!     iface          global face number
!     ihelp          integer work array of maximum length 2*nfapel
!     ipa            pointer in kmeshaaa
!     ipb            pointer in kmeshaab
!     ipd            pointer in kmeshaad
!     irelem         Relative element sequence number with respect to
!                    element group
!     j              Counting variable
!     jface          local face number in element
!     k              Counting variable
!     locedges       Array of size 4 x 6 containing the local edge
!                    numbers of each face of a volume element
!     nedpfa         number of edges per face
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     TOPFA03        Determine local edge numbers for local face number
!     TOPFACES04     Fill part of nmesh part a
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ielem = 0
      nfaces = 0

!     --- Get the local edges of this face and store in locedges

      do jface = 1, nfapel
         call topfa03 ( jface, ishape, nedpfa(jface), &
                        locedges(1,jface) )
      end do

!     --- Loop over the elements

      do irelem = 1, nelem
         ielem = ielem + 1
         ipa = nmesha(ielem)

!        --- Loop over the faces

         do jface = 1, nfapel
            if ( nmesha(ipa+jface).eq.0 ) then

!           --- New face found

               nfaces = nfaces + 1
               nmeshb(nfaces) = 0

!              --- Store this face in ihelp for a while

               ipd = nmeshd(ielem)
               do i = 1, nedpfa(jface)
                  j = abs(locedges(i,jface))
                  k = locedges(i,jface)/j
                  ihelp(i) = k*nmeshd(ipd+j)
               end do

!              --- Find all elements with this face and fill nmesha part 2

               call topfaces04 ( nmeshd, nmeshh, nmesha, ihelp, &
                                 nedpfa(jface), nfaces, ihelp(medpfa+1), &
                                 nfapel, locedges )
            end if
         end do

      end do

!     --- Redo this but now fill nmeshb

      ipb = nfaces + 1
      ielem = 0

!     --- Loop over the elements

      do irelem = 1, nelem
         ielem = ielem + 1
         ipa = nmesha(ielem)
         ipd = nmeshd(ielem)

!        --- Loop over the faces

         do jface = 1, nfapel

!        --- Get the label of this face from nmesha

            iface = abs(nmesha(ipa+jface))

            if ( nmeshb(iface).eq.0 ) then

!           --- New face found
!               Store this face in nmeshb

               nmeshb(iface) = ipb
               nmeshb(ipb) = nedpfa(jface)
               do i = 1, nedpfa(jface)
                  j = abs(locedges(i,jface))
                  k = locedges(i,jface)/j
                  nmeshb(ipb+i) = k*nmeshd(ipd+j)
               end do
               ipb = ipb + nedpfa(jface) + 1
            end if
         end do

      end do

      length = ipb - 1

      end
