      subroutine topfaces04 ( nmeshd, nmeshh, nmesha, edges, &
                              nedpfa, iface, ihelp, nfapel, locedges )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.0  date 23-08-1998 renamed nmfa04
!
!   copyright (c) 1998-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   creates the faces part 2 actual face labels
!
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     face
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshd(*), nmeshh(*), nmesha(*), edges(*), &
              nedpfa, iface, ihelp(*), nfapel, locedges(4,6)

!     edges          i    edges of the face to create
!     iface          i    Face sequence number of input face
!     ihelp               integer work array of length medpfa>=nedpfa
!     locedges       i    Contains the local edge sequence numbers of the
!                         faces of a volume
!     nedpfa         i    number of edges per face
!     nfapel         i    number of faces in one element
!     nmesha         o    nmesh part a
!     nmeshd         i    nmesh part d
!     nmeshh         i    nmesh part h
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ie1, iph, nelped, ielped, ielem, jedge, ipd, &
              nedpel, iedpel, i, j, ipa, jface, nt, k

!     i              Counting variable
!     ie1            label of first edge on the face
!     iedpel         edge number in element
!     ielem          Element number
!     ielped         element in one edge
!     ipa            pointer in kmeshaaa
!     ipd            pointer in kmeshaad
!     iph            pointer in kmeshaah
!     j              Counting variable
!     jedge          edge number in element
!     jface          face number in element
!     k              Counting variable
!     nedpel         number of edges in one element
!     nelped         number of elements connected to an edge
!     nt             counter for edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ie1 = abs(edges(1))
      iph = nmeshh(ie1)
      nelped = nmeshh(iph)

!     --- Loop over all elements connected with this edge

      do ielped = 1, nelped
         ielem = nmeshh(iph+ielped)

!        --- Fetch number of edges in this element

         ipd = nmeshd(ielem)
         nedpel = nmeshd(ipd)

!        --- Loop over the (other) edges in this face (they all must be in
!            ielem)

         do i = 2, nedpfa
            jedge = abs(edges(i))

!           --- Check if also this edge is in the element

            do iedpel = 1, nedpel

               if ( abs(nmeshd(ipd+iedpel)).eq.jedge ) then
                  goto 50
               end if
            end do

            goto 500

 50      end do

!        --- This element contains all edges in iedge fetch number of faces in
!            ielem

         ipa = nmesha(ielem)

!        --- Loop over the faces

         do jface = 1, nfapel

!        --- Store this face in ihelp for a while

            ipd = nmeshd(ielem)
            do i = 1, nedpfa
               j = abs(locedges(i,jface))
               k = locedges(i,jface)/j
               ihelp(i) = k*nmeshd(ipd+j)
            end do

!           --- Compare this face with the input face stored in array edges

            nt = 0
            do i = 1, nedpfa
               do j = 1, nedpfa
                  if ( abs(ihelp(i)).eq.abs(edges(j)) ) nt = nt + 1
               end do
            end do

!           --- Try next face if faces do not correspond

            if ( nt.ne.nedpfa ) then
               goto 100

            else

!           --- This elements contains the input face: store and try next
!               element

               nmesha(ipa+jface) = iface
               goto 500

            end if
100      end do

500   end do
      end
