      subroutine toplm1obst ( nelem, npoint, kmeshdobst, kmeshcobst, &
                              isum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-11-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kmesh part d1 with pointers to kmesh part d2 for an obstacle
!     kmesh part d contains the elements corresponding to each node
!     The second part will be filled in toplm2obst
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, npoint, kmeshdobst(*), kmeshcobst(3,*), isum

!     isum           o    computed length of of part 2 of kmesh part d
!     kmeshcobst     i    kmesh part c, contains the nodal points
!                         corresponding to the elements of the obstacle
!     kmeshdobst     o    KMESH part d, only the first part is filled
!     nelem          i    Number of elements of the obstacle
!     npoint         i    Number of nodal points of the obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nodp, ih, ielem

!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     j              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     clear kmesh part d1
!     Store number of elements corresponding to nodes in kmesh d part 1
!     Fill kmesh d part 1 by addition
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toplm1obst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toplm1obst'
         call ertrace

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- clear kmesh part d1

      kmeshdobst(1:npoint) = 0

!     --- Store number of elements corresponding to nodes in kmesh d part 1
!         Loop over elements

      do ielem = 1, nelem

!     --- Loop over nodes in element

         do j = 1, 3
            nodp = kmeshcobst(j,ielem)
            kmeshdobst(nodp) = kmeshdobst(nodp)+1
         end do

      end do

!     --- Fill kmesh d part 1 by addition

      isum = 0
      do i = 1,npoint
         ih = kmeshdobst(i)
         kmeshdobst(i) = isum
         isum = isum+ih
      end do

      kmeshdobst(npoint+1) = isum

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toplm1obst'

      end if  ! ( debug )
      call erclos ( 'toplm1obst' )

      end subroutine toplm1obst
