      subroutine topol04 ( nnodelgr, nodelgrp, nnelgnod )
! ======================================================================
!
!        programmer     Guus Segal
!        version  2.0  date 25-02-2018
!        version  1.0  date 22-07-1997
!
!   copyright (c) 1997-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array nnelgnod with number of nodes corresponding
!    to an element group
! **********************************************************************
!
!                       KEYWORDS
!
!  topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodelgr(*), nodelgrp(*), nnelgnod(0:*)

!     nnelgnod   o    Contains the number of nodes corresponding
!                     to the element groups
!     nnodelgr   i    Contains the number of element groups corresponding
!                     to the nodes
!     nodelgrp   i    Contains the element group sequence numbers corresponding
!                     to the nodes
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielgrp, nodp

!     i          General loop parameter
!     ielgrp     element group number
!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array nnelgnod
!     for nodp := 1 (1) npoint do
!        for i := nnodelgr(i)+1 (1) nnodelgr(i+1) do
!           ielgrp := nodelgrp(i)
!           nnelgnod(ielgrp) := nnelgnod(ielgrp)+1
!     accumulate nnelgnod
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- Clear array nnelgnod

      nnelgnod(1:nelgrp) = 0

!     --- loop over nodes

      do nodp = 1, npoint

!     --- loop over element groups corresponding to node

         do i = nnodelgr(nodp)+1, nnodelgr(nodp+1)

            ielgrp = nodelgrp(i)
            nnelgnod(ielgrp) = nnelgnod(ielgrp)+1

         end do

      end do

!     --- accumulate nnelgnod

      nnelgnod(0) = 0
      do i = 1, nelgrp
         nnelgnod(i) = nnelgnod(i)+nnelgnod(i-1)
      end do
      end
