      subroutine topouterfac ( nmeshf, nfaces, nouterfaces, outerfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all outer faces of a volume by storing only those faces
!     corresponding to one element
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     surface
!     volume
!     face
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshf(*), nfaces, nouterfaces, outerfaces(*)

!     nfaces         i    Number of faces
!     nmeshf         i    nmesh part f
!     nouterfaces    o    Number of outer faces (length of outerfaces)
!     outerfaces     o    Array containing the face numbers of the outer faces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iface, ipf, nelem

!     iface          Face sequence number
!     ipf            pointer in kmeshaaf
!     nelem          Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nouterfaces := 0
!     For all faces do
!        if ( face corresponds to one element only ) then
!           raise nouterfaces
!           store face number in outerfaces
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topouterfac' )

!     --- nouterfaces := 0

      nouterfaces = 0

!     --- For all faces do

      do iface = 1, nfaces

         ipf = nmeshf(iface)
         nelem = nmeshf(ipf)
         if ( nelem.eq.1 ) then

!        --- face corresponds to one element only
!            raise nouterfaces
!            store face number in outerfaces

            nouterfaces = nouterfaces+1
            outerfaces(nouterfaces) = iface

         end if

      end do

      call erclos ( 'topouterfac' )
      end
