      subroutine elemsubr ( npelm, x, y, nunk_pel, elem_mat, &
                            elem_vec, elem_mass, uold, itype )
! ======================================================================
!
!        programmer     Guus Segal
!        version 2.0    date 28-08-2014
!        version 1.0    date 15-10-2004
!
!
!   copyright (c) 2004-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Special user element subroutine to be used by SEPRAN in the case of
!     the Numerical Analysis Lab of Delft University of Technology
! **********************************************************************
!
!                       KEYWORDS
!
!    matrix
!    vector
!    element
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer, intent(in) :: npelm, nunk_pel, itype
      double precision, intent(in) :: x(npelm), y(npelm), &
                                      uold(nunk_pel)
      double precision, intent(out) :: elem_mat(nunk_pel,nunk_pel), &
                                       elem_vec(nunk_pel), &
                                       elem_mass(nunk_pel)

!     elem_mass   o   In this two-dimensional array the student must store the
!                     element mass matrix, provided the mass matrix must be
!                     computed, in the following way:
!                     elem_mass(i,j) = s_{ij} ; i,j = 1(1)nunk_pel
!                     This matrix should only be filled if a mass matrix is
!                     required, for example for time-dependent problems.
!     elem_mat    o   In this two-dimensional array the student must store the
!                     element matrix, in the following way:
!                     elem_mat(i,j) = s_{ij} ; i,j = 1(1)nunk_pel.
!                     The degrees of freedom in an element are stored
!                     sequentially, first all degrees of freedom corresponding
!                     to the first point, then to the second, etcetera.
!     elem_vec    o   In this array the student must store the element vector, &
!                     in the following way:
!                     elem_vec(i) = f_i; i = 1(1)nunk_pel
!                     It concerns the derived quantity that must be computed
!     itype       i   Type number of element.
!                     This parameter is defined in the input block PROBLEM
!     npelm       i   Number of points per element
!     nunk_pel    i   number of degrees of freedom in the element
!     uold        i   In this array the old solution, as indicated by V1, &
!                     is stored. This solution may contain the boundary
!                     conditions only, if the array has been created by
!                     prescribe_boundary_conditions, but also a starting vector
!                     if V1 has been created by create or even the previous
!                     solution in an iteration process if nonlinear_equations
!                     is used.
!     x           i   Contains the x-coordinates of the nodes of the element using
!                     the local node numbering
!     y           i   Contains the y-coordinates of the nodes of the element using
!                     the local node numbering
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
!      Dit gedeelte moet door de practicant worden ingevuld
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!    none
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    none
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The element matrix, element right-hand side and if the problem so
!    requires the element mass matrix are filled by the user, depending on
!    the parameter itype
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Hier moet de practicant statements neer zetten om de elementmatrix
!         en elementvector te vullen

      call errchr ( 'elemsubr', 1 )
      call errsub ( 349, 0, 0, 1 )
      call instop
      end
