      subroutine fnhooke ( eps, matpar, s, se )
! ======================================================================
!
!        programmer     Marcel Rutten
!        version 1.0    date 21-01-1998
!
!
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
! fill estimate of 2nd Piola Kirchoff stress plus tangential stiffness
! matrix for one integration point. (hookean material)
! **********************************************************************
!
!                       KEYWORDS
!
!      elasticity_equation
!      solid
!      material_behaviour
!      hookean
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision eps(6), matpar(21), s(6), se(6,6)

!     eps      i     Green-Lagrange strain
!     matpar   i     material properties
!     s        o     2nd Piola Kirchhoff stress estimate
!     se       o     tangential stiffness matrix
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision e1, nu1, a0, a1, a2, a3

!     a0
!     a1
!     a2
!     a3
!     e1
!     i          Counting variable
!     j          Counting variable
!     nu1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

!     --- linear (Hookean) material properties

      E1 = matpar(1)
      nu1 = matpar(2)
      a0 = E1 / ( 1d0 + nu1 )
      a1 = a0 * ( 1d0 - nu1 ) /( 1d0 - 2d0*nu1)
      a2 = a0 * nu1 /( 1d0 - 2d0*nu1)
      a3 = a0 / 2d0

      do i=1,3
         se(i,i) = a1
         se(i+3,i+3) = a3
      end do

      se(1,2) = a2
      se(2,1) = a2
      se(2,3) = a2
      se(3,2) = a2
      se(1,3) = a2
      se(3,1) = a2

      do i=1,3
         do j=1,3
            s(i) = s(i) + se(i,j)*eps(j)
         end do
         s(i+3) = se(i+3,i+3)*eps(i+3)
      end do

      end
