      subroutine el0603 ( iuser, user, array )
! ======================================================================
!
!        programmer    Luc Timmermans
!        version  1.2  date 23-08-2001 New call to elusr3
!        version  1.1  date 23-01-1996
!        version  1.0  date 13-07-1993
!
!   copyright (c) 1993-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     this subroutine performs the initialisations for subroutine ELM603
!     or ELD603 in the case that IFIRST = 0
!     the parameters in the common blocks celint and celiar get a start
!     value
!     array array is initialised with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*)
      double precision user(*), array(*)

!     array      o    Real work array in which the values of the coefficients
!                     in the integration points are stored in the sequence:
!                     1: alpha
!                     the coefficient requires exact m (number of
!                     integration points) positions
!     iuser      i    Integer user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
!     user       i    Real user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, i

!     i          Counting variable
!     ncoef      Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCOMM    Initializes the commons celp and celint with standard values
!     ELUSR3    Extracts information concerning the coefficients from IUSER
!               and USER.
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     ERRWAR    Print warnings
!     INSTOP    Stop the program
!     PRINLG    Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!    1600   alpha = 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- fill some standard values in the common block celint

      call eropen ('el0603')
      call elcomm

!     --- set parameters depending on the type numbers

      if ( itype.eq.603 ) then

!     --- itype = 603, one-dimensional spectral element

         n = inpelm
         m = inpelm
         mn = n*m
         jdercn = 1
         nvert = 2
         ndimlc = 2
      end if

!     --- fill the arrays IND, MST and ARRAY with information of
!         the coefficients

      ncoef = 1
      if (ncoef.gt.0) then
         ipos = iuser(ielgrp + 5)
         if (ipos.lt.7) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub (42, 5, 0, 0)
            go to 1000
         end if
         nuser = 0
         do i = 1 , ncoef
            call elusr3 (i, array(1+(i-1)*m), iuser, user, i)
         end do

!        --- test array lengths, number of nodes and number of
!            degrees of freedom

         call prinlg ( 'iuser', ipos-1, user, iuser)
         call prinlg ( 'user', -nuser, user, iuser)
      end if

!     --- compute jconsf
!         first check if alpha is unequal to zero

      if (ind(1).eq.0) then

!     --- alpha is zero

         call errwar (1600, 0, 0, 0)
         jconsf = 0
      else if ((ind(1).lt.0) .or. (ind(1).eq.2002)) then

!     --- alpha is constant (per element)

         jconsf = 1
      else

!     --- general

         jconsf = 2
      end if
1000  if (ierror.ne.0) call instop
      call erclos ('el0603')
      end
