      subroutine el0608 ( iuser, user, array )
! ======================================================================
!
!        programmer    Peter Minev
!        version  1.2  date 23-08-2001 New call to elusr3
!        version  1.1  date 28-01-1996 Layout
!        version  1.0  date 18-11-1994
!
!   copyright (c) 1994-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     this subroutine performs the initialisations for subroutine ELM608
!     in the case that IFIRST = 0
!     the parameters in the common blocks celint and celiar get a start
!     value
!     array array is initialised with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*)
      double precision user(*), array(*)

!     array   o      real work array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                    1: alpha
!                    the coefficient requires exact m (number of
!                    integration points) positions
!     iuser   i      user array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     user    i      user array in which real information of the
!                    coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef
      double precision rm

!     ncoef         number of coefficients in the equation
!     rm            real parameter to calculate mone
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCOMM    Initializes the commons celp and celint with standard values
!     ELUSR3    Extracts information concerning the coefficients from IUSER
!               and USER.
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INSTOP    Stop the program
!     PRINLG    Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!    1697   combination n and mone not correct
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- fill some standard values in the common block celint

      call eropen ('el0608')
      call elcomm

!     --- set parameters depending on the type numbers

      if ( itype.eq.608 ) then

!     --- itype = 608, two-dimensional spectral element in 3D

         n = inpelm
         m = inpelm
         rm = m
         mone = nint(sqrt(rm)+.5d0)
         if ( mone*mone.ne.n ) then
            call errint ( ndimlc, 1 )
            call errint ( n, 2 )
            call errint ( mone**ndimlc, 3 )
            call errsub ( 1697, 3, 0, 0 )
         end if
         mn = n*m
         jdercn = 1
         nvert = 4
         ndimlc = 3
      end if

!     --- fill the arrays IND, MST and ARRAY with information of
!         the coefficients

      ncoef = 1
      if (ncoef.gt.0) then
         ipos = iuser(ielgrp + 5)
         if (ipos.lt.7) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub (42, 5, 0, 0)
            go to 1000
         end if
         nuser = 0
         call elusr3 (1, array(1), iuser, user, 1)

!        --- test array lengths, number of nodes and number of
!            degrees of freedom

         call prinlg ( 'iuser', ipos-1, user, iuser)
         call prinlg ( 'user', -nuser, user, iuser)
      end if

!     --- first check if alpha is unequal to zero

      if ( ind(1).eq.0 ) then

!     --- alpha is zero

         jconsf = 0
      else if ( ind(1).lt.0 .or. ind(1).eq.2002 ) then

!     --- alpha is constant (per element)

         jconsf = 1
      else

!     --- general

         jconsf = 2
      end if
1000  if (ierror.ne.0) call instop
      call erclos ('el0608')
      end
