      subroutine el0612(iuser, user, isub, isup, array, &
                        ichois, icheld)
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  1.2  date 23-08-2001 New call to elusr3
!        version  1.1  date 05-05-1999    Include common blocks
!        version  1.0  date 05-12-1996
!
!   copyright (c) 1996-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     this subroutine performs the initialisations for subroutine ELM612
!     or ELD612 in the case that IFIRST = 0
!     the parameters in the common blocks celint and celiar get a start
!     value
!     array array is initialised with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), isub, isup, icheld, ichois
      double precision user(*), array(*)

!     array    o     real work array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                    1: eta
!                    the coefficients 1-1 require exact n (number of
!                    nodal points) positions
!     icheld   i     choice parameter to distinguish between the various
!                    possibilities of this subroutine
!                    0:   Call by ELM612
!                    >0:  Call by ELD612 (not yet done)
!     ichois   o     output choice parameter
!     isub     o     integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup     o     integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iuser    i     user array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     user     i      user array in which real information of the
!                    coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, i
      double precision rn

!     i             counting variable
!     ncoef         number of coefficients in the equation
!     rn            double precision value of inpelm
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCOMM  Initializes the commons celp and celint with standard values
!     ELUSR3  Extracts information concerning the coefficients from IUSER
!             and USER.
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
!     PRINLG  Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- fill some standard values in the common block celint

      call eropen ('el0612')
      call elcomm
      ncomp = 3

!     --- set parameters depending on the type numbers

      n = inpelm
      m = inpelm
      rn = n
      mone = int(sqrt(rn))
      jdercn = 1
      nvert = 4
      ndimlc = 2

!     --- fill the arrays IND, MST and ARRAY with information of the
!     coefficient

      ncoef = 2
      if ( ncoef.gt.0 ) then
         ipos = iuser(ielgrp + 5)
         if (ipos.lt.7) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub (42, 5, 0, 0)
            go to 1000
         end if
         nuser = 0
         do i = 1 , ncoef
            call elusr3 (i, array(1+(i-1)*n), iuser, user, i)
         end do

!        --- compute ichois

         isub = 1
         isup = 1
         ichois = iuser (8)

!        --- test array lengths, number of nodes and number of degrees of
!            --- freedom

         call prinlg ( 'iuser', ipos-1, user, iuser)
         call prinlg ( 'user', -nuser, user, iuser)
      end if
1000  if (ierror.ne.0) call instop
      call erclos ('el0612')
      end
