      subroutine el0800beta ( kfirst, jwcons )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute jzero corresponding to zeroth order part of the equation
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kfirst, jwcons

!     jwcons         i    Indication if the weight function is constant
!                         (1) or not (2)
!     kfirst         o    This parameter indicates the smallest velocity
!                         component that is not equal to zero in the convective
!                         terms
!                         For example kfirst = 2, means that the y- &
!                         component of the velocity is unequal to zero
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800beta' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800beta'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ind(10)==0 ) then

!     --- beta = 0

         jzero = 0

      else if ( ind(10)<0 .or. ind(10)==2002 ) then

!     --- beta is constant (per element)

         jzero = jwcons

      else

!     --- General

         jzero = 2

      end if
      if ( debug ) &
         write(irefwr,1) 'jsecnd, kfirst, jzero, idim', &
                          jsecnd, kfirst, jzero, idim

1000  call erclos ( 'el0800beta' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0800beta'

      end if  ! ( debug )

      end
