      subroutine el0800chk ( metup, mrule, linear, jelp, jheat, &
                             jwcons, massparm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 20-01-2012
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check combination of upwind and jcart
!     Adapt irule and jwcons if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer metup, mrule, jelp, jheat, jwcons, massparm
      logical linear

!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     jheat          i    Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed (>0) or not ( = 0)
!                         If jheat = 1, then the coefficient for the mass matrix
!                         is constant, otherwise (jheat = 2) it is variable
!     jwcons        i/o   Indication if the weight function is constant
!                         (1) or not (2)
!     linear         i    Indicates whether the element is a linear simplex
!                         (true) or not (false)
!     massparm       i    Indicates the largest value of jheat during the
!                         time integration
!     metup          i    Upwind parameter
!     mrule          i    Type of integration rule as given by the user
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1785   Upwinding has only been implemented for Cartesian and axi-symmetric
!           co-ordinates systems
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800chk' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800chk'
         write(irefwr,1) 'metup, jcart', metup, jcart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( metup>0 .and. jcart>2 ) then
         call errint ( jcart-1, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1785, 4, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      if ( mod(metup,20)>=9 .and. mod(metup,20)<=11 ) then

!     --- special upwind (9: linear, satisfying maximum principle)
!         Always use 1 point Gauss rule

         irule = 2

      else if ( mrule==0 .and. metup>0 ) then

!     --- mrule = 0, and upwind
!         A different integration rule is applied

         if ( imas==1 ) then
            irule = 1
         else if ( jelp==3 .and. mod(metup,20)>6 .or. &
                   jelp==11 .or. jelp==21 ) then
            irule = 2
         else if ( jelp>=3 .and. jelp<=6 ) then
            irule = 3
         end if

      end if

      if ( linear ) then
         jwcons = min(jcart,2)
      else
         jwcons = 2
      end if
      if ( icheld>0 .and. mrule==0 ) irule = 1

      if ( debug ) &
         write(irefwr,1) 'icheld, irule, mrule, jelp, jheat', &
                          icheld, irule, mrule, jelp, jheat

      if ( massparm==2 .and.  mrule==0 .and. &
           (jelp<=5 .or. jelp>=7) ) then

!     --- Non-diagonal mass matrix, shapes 1-5 and >=7
!         Integration rule must be at least 3 to avoid non-positive
!         definite mass matrices

         if ( jelp==2 ) then

!        --- Quadratic 1d, use irule = 4

            irule = 4

         else

!        --- Other cases, use irule = 3

            irule = max ( irule, 3 )

         end if

      else if ( massparm==2 .and. jelp<=6 .and. mrule==0 ) then

!     --- Non-diagonal mass matrix, shape 6
!         Integration rule must be at least 2 to avoid non-positive
!         definite mass matrices

         irule = max ( irule, 2 )

      end if

1000  call erclos ( 'el0800chk' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'irule, jdiag', irule, jdiag
         write(irefwr,*) 'End el0800chk'

      end if  ! ( debug )

      end
