      subroutine el0800rule ( kchois, interpol, ishift, iuser, mrule )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 18-12-2014 Use elgetintcoef
!        version  1.1  date 20-06-2013 Use el0800ruledef
!
!   copyright (c) 2010-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the parameters irule and jcart from array iuser
!     positions ipos and ipos+1
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     numerical_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kchois, interpol, ishift, iuser(*), mrule

!     interpol      i/o   If 0 the standard interpolation is applied, &
!                         if 1 the interpolation is restricted to linear
!                         subelements
!     ishift        i/o   Shift in coefficient sequence number
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     kchois         i    Indication what type of parameters must be computed
!                         See calling subroutine
!     mrule          o    Type of integration rule as given by the user
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nncode
      parameter (nncode=3)
      integer :: ncode(nncode)
      character (len=13) :: code (nncode)

!     code           Table with keywords corresponding to string coefficients
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     nncode         Number of entries in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal, elgetintcoef

!     EL0800RULEDEF  Compute the default integration rule depending
!                    on the shape number
!     ELGETINTCOEF   Give elgetintcoef the actual integer value corresponding to
!                    iuser(ipos)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'CARTESIAN', 'AXI_SYMMETRIC', 'POLAR' /
      data ncode / 9,13, 5 /
! ======================================================================
!
      call eropen ( 'el0800rule' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800rule'
         write(irefwr,1) 'ishape', ishape
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      mrule = inirepscal ( iuser(ipos) )
      interpol = mrule/100
      mrule = mod(mrule,100)
      if ( debug ) write(irefwr,1) 'mrule', mrule
      if ( mrule==0 ) then
         call el0800ruledef
      else
         irule = mrule
      end if
      if ( debug ) write(irefwr,1) 'irule/1', irule
      if ( ishape==9 ) then
         if ( irule<3 .or. irule>4 ) irule = 3
      else if ( ishape==10 ) then
         if ( irule<2 .or. irule>3 ) irule = 2
      end if
      if ( debug ) write(irefwr,1) 'irule/2', irule
      ipos = ipos + 1
      ishift = ishift+1
      if ( mrule<0 .or. mrule>40 ) then
         call errint ( mrule, 1 )
         if ( kchois==0 ) then
            call errint ( 3, 2 )
         else
            call errint ( 1, 2 )
         end if
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if

      jcart = elgetintcoef ( iuser, ipos, ishift, code, nncode, ncode, &
                             1, 'coordinate_system'  )

      if ( jcart<0 .or. mod(jcart,10)>2 .or. &
           ndim>2 .and. jcart/10>4 ) then
         call errint ( jcart, 1 )
         if ( kchois==0 ) then
            call errint ( 4, 2 )
         else
            call errint ( 2, 2 )
         end if
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1767, 5, 0, 0 )
      end if
      jcart = jcart + 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'm, n, irule, jcart', m, n, irule, jcart
         write(irefwr,*) 'End el0800rule'

      end if  ! ( debug )
      call erclos ( 'el0800rule' )

      end
