      subroutine el0814 ( iuser, jelp, itypln )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 16-02-2015 use elgetintcoef
!        version  1.0  date 15-12-2010
!
!   copyright (c) 2010-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM814 or
!     ELD814 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     minimal_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), jelp, itypln

!     itypln         o    Indicates the type of linearization to be performed
!                         Possible values:
!                         0:  Picard
!                         1:  Newton
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, ishift, interpol, jwcons, mrule
      logical linear

!     interpol       If 0 the standard interpolation is applied, &
!                    if 1 the interpolation is restricted to linear
!                    subelements
!     ishift         Shift in coefficient sequence number
!     jwcons         Indication if the weight function is constant
!                    (1) or not (2)
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: elfilllinear

!     EL0800ELP      Set parameters depending on shape of element
!     EL0800INM      Compute the parameter m for an element
!     EL0800RULE     Fill the parameters irule and jcart from array iuser
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELFILLLINEAR   Set elfilllinear equal to value following linearization
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERROR3065      Print error 3065 if necessary
!     ERROR42        Print error message 42
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is given in elm814
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0814' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0814'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set a number of defaults

      call elcomm
      ishift = 0
      jderiv = 1

!     --- Set parameters depending on the shape numbers

      call el0800elp ( linear, jelp )

      mn = m * n

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      ipos = iuser ( ielgrp + 5 )
      if ( ipos<7 ) call error42 ( ielgrp, ipos, ielem, itype )

      ipos = ipos+2   ! First two parameters not yet defined
      ishift = ishift+2

!     --- Compute irule and jcart (3 and 4)

      call el0800rule ( 0, interpol, ishift, iuser, mrule )
      if ( irule/=1 ) then

!     --- irule>1, Gauss rule  compute m

         call el0800inm ( m, jelp, jwcons, irule, igausp, jdiag, &
                          iphi )
         mn = m * n

      end if  ! ( irule/=1 )

      itypln = elfilllinear ( iuser, ipos, ishift, 1 )

      call error3065 ( itypln, 0, 1, 'type of linearization', &
                       ishift )

      if ( jelp==4 .and. jcart==1 .and. &
           icheld<=0 .and. irule==1 ) kstep = 2

      if ( igausp==1 .and. mod(iweigh,10)==0 ) iweigh = iweigh+1

      if ( itypln==0 ) notvec = 1  ! no right-hand side if Picard

      notmas = 1

1000  call erclos ( 'el0814' )
      if ( debug ) then

!     --- Debug information

         call prinin ( ind, ncoef, 'ind' )
         call prinin ( ist, ncoef, 'ist' )
         write(irefwr,1) 'igausp, m, n, iweigh, iphi', &
                          igausp, m, n, iweigh, iphi
         write(irefwr,*) 'End el0814'

      end if  ! ( debug )

      end
