      subroutine el10011 ( ix, dphidx, dudx, elemvc, elemwg, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-03-2013 Correction declaration
!        version  1.0  date 08-08-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    actual filling of derivatives and weights in elemvc and elemwg
!    with respect to subroutine ELD800
!    icheld = 1: d c /d_ix
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ix, ndim
      double precision  dudx(m,ndimlc), dphidx(n,m,*), &
                        elemwg(n), elemvc(n)

!     dphidx   i     Array of length n x m x ndim  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dudx     i     Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                           j   k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     elemvc   o     element vector containing the derivatives for subroutine
!                    deriva
!     elemwg   o     element vector containing the weights for subroutine
!                    deriva
!     ix       i     Indication of the type of derivative to be computed.
!                    When icheld=1 or 11, du/d ix is computed
!     ndim     i     Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el10011' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el10011'
         write(irefwr,1) 'jdercn, n, ix', jdercn, n, ix
         call prinrl ( dudx, n, 'dudx' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jdercn==0 ) then

!     --- Derivatives are constant

         elemwg = abs( dphidx(1:n,1,ix) )
         elemvc = dudx(1,1) * elemwg

      else if ( ndim==2 .and. n>=6 ) then

!     --- quadratic elements in R2

         do i = 1, n-1, 2
            elemwg(i) = abs( dphidx(i,i,ix) )
            elemvc(i) = dudx(i,1) * elemwg(i)
         end do
         do i = 2, n, 2
            elemvc(i) = dudx(i,1)
         end do
         elemvc(n) = dudx(n,1)

      else

!     --- General case

         elemvc = dudx(1:n,1)

      end if

1000  call erclos ( 'el10011' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el10011'

      end if  ! ( debug )

      end
