      subroutine el100111 ( dudx, elemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-08-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    actual filling of derivatives in elemvc
!    with respect to subroutine ELD800
!    icheld = 11: d c /d_ix defined per element
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision  dudx(n), elemvc

!     dudx     i     Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                           j   k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     elemvc   o     element vector containing the derivatives for subroutine
!                    deriva
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el100111' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el100111'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jdercn==0 ) then

!     --- Derivatives are constant

         elemvc = dudx(1)

      else

!     --- General element

         elemvc = sum(dudx)/n

      end if

1000  call erclos ( 'el100111' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el100111'

      end if  ! ( debug )

      end
