      subroutine el10012 ( icheld, dphidx, dudx, elemvc, elemwg, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 16-04-2014 Debug
!        version  1.0  date 08-08-2011
!
!   copyright (c) 2011-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    actual filling of derivatives and weights in elemvc and elemwg
!    with respect to subroutine ELD800
!    icheld = 2, 3, 5: gradient
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheld, ndim
      double precision  dudx(m,ndimlc), dphidx(n,m,ndimlc), &
                        elemwg(ndimlc,n), elemvc(ndimlc,n)

!     dphidx   i     Array of length n x m x ndim  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dudx     i     Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                           j   k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     elemvc   o     element vector containing the derivatives for subroutine
!                    deriva
!     elemwg   o     element vector containing the weights for subroutine
!                    deriva
!     icheld   i     Choice parameter to distinguish between the various
!                    possibilities of this subroutine
!                    Definitions:
!                    2:   grad u
!                    3:   - grad u
!                    5:   elemvc(1,*)=du/dy   elemvc(2,*)=-du/dx (2D only)
!     ndim     i     Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision h

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     j              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el10012' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el10012'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jdercn==0 ) then

!     --- Derivatives are constant

         do j = 1, ndimlc
            elemwg(j,1:n) = abs( dphidx(1:n,1,j) )
            elemvc(j,1:n) = dudx(1,j) * elemwg(j,1:n)
         end do

      else if ( ndim==2 .and. n>=6 ) then

!     --- quadratic elements in R2

         do j = 1, ndimlc
            do i = 1, n-1, 2
               elemwg(j,i) = abs( dphidx(i,i,j) )
               elemvc(j,i) = dudx(i,j) * elemwg(j,i)
            end do
            do i = 2, n, 2
               elemvc(j,i) = dudx(i,j)
            end do
            elemvc(j,n) = dudx(n,j)
         end do

      else

!     --- General element

         do j = 1, ndimlc
            do i = 1, n
               elemvc(j,i) = dudx(i,j)
            end do
         end do

      end if

      if ( icheld==3 ) then

!     --- icheld = 3,  change sign

         elemvc = -elemvc

      else if ( icheld==5 ) then

!     --- icheld  = 5, change sign and sequence  (2D only)

         do i = 1, n
            h = elemvc(1,i)
            elemvc(1,i) = elemvc(2,i)
            elemvc(2,i) = - h
            h = elemwg(1,i)
            elemwg(1,i) = elemwg(2,i)
            elemwg(2,i) = h
         end do

      end if

1000  call erclos ( 'el10012' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el10012'

      end if  ! ( debug )

      end
