      subroutine el1003 ( dcdx, u, fold, g, fnew, ndim )
! ======================================================================
!
!        programmer     Guus Segal
!        version  1.0   date 12-04-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute right-hand side function with respect to
!    newton linearization of nonlinear convective terms for element 800
! **********************************************************************
!
!                       KEYWORDS
!
!    element_vector
! **********************************************************************
!
!                       COMMON BLOCKS
!
      use sepmodulecactl
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim
      double precision fold(m), g(m), u(m,ndim), dcdx(m,ndim), fnew(m)

!     dcdx       i    Real m x ndim x ncomp array in which the values of the
!                     derivatives of the solution vector in the integration
!                     points are stored in the sequence:
!                     dc / dx  (x ) = dcdx ( k, j, i )
!                       i    j   k
!     fnew       o    Computed right-hand side function
!     fold       i    Old right-hand side function
!     g          i    Function of gradient of solution
!     ndim       i    Dimension of the space.
!     u          i    Dg / D gradc, i.e. coeeficient for convection
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k
      double precision sum

!     i       Counting variable
!     k       Counting variable
!     sum     Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      new     old
!     f     = f     - g + grad(c) . dg/dgradc
!      i       i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      do k = 1, m
         sum = 0d0
         do i = 1, ndim
            sum = sum + u(k,i)*dcdx(k,i)
         end do
         fnew(k) = fold(k) - g(k) + sum
      end do
      end
