      subroutine el1005 ( icheld, ix, jdegfd, u, dudx, x, ndcomp, &
                          elemvc, mdecl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 04-09-2006 Debug statements
!        version  4.0  date 12-05-1999 Extra parameter mdecl
!        version  3.2  date 14-03-1999 Use integration points
!        version  3.1  date 03-09-1994 extension with icheld=7
!        version  3.0  date 26-02-1994 delete superfluous statements and
!                                      parameters
!        version  2.0  date 22-07-1993 Extension with icheld=2,3,4,5, &
!                                      new parameter list
!        version  1.0  date 07-07-1992
!
!   copyright (c) 1994-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives and weights in elemvc and elemwg
!     in the case of a vector quantity
!     Filling is performed for all nodal points
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheld, ix, jdegfd, ndcomp, mdecl
      double precision  dudx(mdecl,ndimlc,*), u(mdecl,*), &
                        elemvc(ndcomp,*), x(mdecl,*)

!     dudx     i     Real m x ndimlc x ncomp array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j, i )
!                      i    j   k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     elemvc   o     element vector containing the derivatives for subroutine
!                    deriva
!     icheld   i     Choice parameter to distinguish between the various
!                    possibilities of this subroutine
!                    Definitions:
!                    1:   compute the derivative of jdegfd th unknown in ix
!                         direction
!                         ix = 1    elemvc(1,*) = du jdegfd / dx
!                         ix = 2    elemvc(1,*) = du jdegfd / dy
!                         ix = 3    elemvc(1,*) = du jdegfd / dz
!                    2:   grad u
!                         Derivatives are stored in the sequence first all
!                         derivatives for node 1 then for node 2 etc.
!                         The definition of grad u depends on jcart
!                         jcart = 1:
!                         du1/dx1, du2/dx1, du3/dx1, du1/dx2, ...
!                         jcart = 2, ndimlc=2:
!                         du1/dx1, du2/dx1, du1/dx2, du2/dx2
!                         jcart = 2, ndimlc=3:
!                         du1/dx1, du2/dx1, du3/dx1, du1/rdx2-u2/r, &
!                         du2/rdx2+u1/r, du1/rdx3, du1/dx3, du2/dx3, du3/dx3
!                         if r=0, at this moment:
!                         du1/dx1, du2/dx1, du3/dx1, 0, 0, 0
!                         du1/dx3, du2/dx3, du3/dx3
!                         jcart = 3:
!                         du1/dx1, du2/dx1, du1/rdx2-u2/r, du2/rdx2+u1/r
!                         if r=0, at this moment:
!                         du1/dx1, du2/dx1, 0, 0
!                         jcart = 4:
!                         du1/dx1, du3/dx1, du2/dx1, -u3/r, u1/r,0
!                         du1/dx2, du3/dx2, du2/dx2
!                         if r=0, at this moment:
!                         du1/dx1, du3/dx1, du2/dx1, 0, 0, 0, &
!                         du1/dx2, du3/dx2, du2/dx2
!                    3:   - grad u
!                    4:   div u
!                         jcart = 1: du1/dx1 + du2/dx2 + du3/dx3
!                         jcart = 2 or 4: du1/dx1 + u1/r + du2/dx2 (ndimlc=2)
!                                    du1/dx1 + (u1+du2/dx2)/r+du3/dx3 (ndimlc=3)
!                         if r=0, at this moment:
!                                    2du1/dx1 + du2/dx2 (ndimlc=2)
!                                    2du1/dx1 + du3/dx3 (ndimlc=3)
!                         jcart = 3: du1/dx1 + (u1 + du2/dx2)/r
!                         if r=0, at this moment:
!                                    2du1/dx1
!                    5:   curl u
!                         jcart = 1, ndimlc=2: du2/dx1 - du1/dx2
!                         jcart = 1, ndimlc=3: du3/dx2 - du2/dx3
!                                            du1/dx3 - du3/dx1
!                                            du2/dx1 - du1/dx2
!                         jcart = 2, ndimlc=2: du2/dx1 - du1/dx2
!                         jcart = 2, ndimlc=3: du3/rdx2 - du2/dx3
!                                            du1/dx3 - du3/dx1
!                                            du2/dx1 + (u2-du1/dx2)/r
!                         if r=0, at this moment:
!                                            - du2/dx3
!                                            du1/dx3 - du3/dx1
!                                            2du2/dx1
!                         jcart = 3: du2/dx1 + (u2-du1/dx2)/r
!                         if r=0, at this moment: 2du2/dx1
!                         jcart = 4: -du3/dx2
!                                     du1/dx2 - du2/dx1
!                                     du3/dx1 + u3/r
!                         if r=0, at this moment:
!                                    -du3/dx2
!                                     du1/dx2 - du2/dx1
!                                    2du3/dx1
!                    7:   strain tensor
!                    21-27: See 1-7, however, the element vector refers to
!                         the vertices only
!     ix       i     Indication of the type of derivative to be computed.
!                    When icheld=1 or 11, du/d ix is computed
!     jdegfd   i     The derivative du jdegfd / d ix is computed
!     mdecl    i     Formal declaration parameter for the arrays (=m)
!     ndcomp   i     Indicates the number of quantities that must be filled
!                    in elemvc and elemwg per nodal point
!     u        i     array of length m x ncomp in which the values of
!                    u in the integration points are stored the sequence:
!                    u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                     i   j
!     x        i     array containing the coordinates of the nodal points
!                    according to:
!                    x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                     i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k

!     i              General loop variable
!     j              General loop variable
!     k              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el1005' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el1005'
         write(irefwr,1) 'ndcomp, icheld', ndcomp, icheld
         write(irefwr,1) 'jcart, ndimlc, m, n, irule', &
                          jcart, ndimlc, m, n, irule
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( icheld==1 ) then

!     --- icheld = 1, compute derivative du jdegfd /d ix

         do i = 1, m
            elemvc(1,i) = dudx(i,ix,jdegfd)
         end do

      else if ( icheld<=3 ) then

!     --- icheld = 2, 3, compute gradient

         if ( jcart==1 ) then

!        --- Cartesian co-ordinates

            do i = 1, ndimlc
               do j = 1, ndimlc
                  do k = 1, m
                     elemvc(i+(j-1)*ndimlc,k) = dudx(k,j,i)
                  end do
               end do
            end do
         else if ( jcart==2 ) then

!        --- Cylinder co-ordinates

            if ( ndimlc==2 ) then

!           --- ndimlc = 2

               do k = 1, m
                  elemvc(1,k) = dudx(k,1,1)
                  elemvc(2,k) = dudx(k,1,2)
                  elemvc(3,k) = dudx(k,2,1)
                  elemvc(4,k) = dudx(k,2,2)
               end do
            else

!           --- ndimlc = 3

               do k = 1, m
                  elemvc(1,k) = dudx(k,1,1)
                  elemvc(2,k) = dudx(k,1,2)
                  elemvc(3,k) = dudx(k,1,3)
                  if ( abs(x(1,k))>epsmac ) then
                     elemvc(4,k) = ( dudx(k,2,1) - u(k,2) )/x(1,k)
                     elemvc(5,k) = ( dudx(k,2,2) + u(k,1) )/x(1,k)
                     elemvc(6,k) = dudx(k,3,1)/x(1,k)
                  else
                     elemvc(4,k) = 0d0
                     elemvc(5,k) = 0d0
                     elemvc(6,k) = 0d0
                  end if
                  elemvc(7,k) = dudx(k,3,1)
                  elemvc(8,k) = dudx(k,3,2)
                  elemvc(9,k) = dudx(k,3,3)
               end do
            end if
         else if ( jcart==3 ) then

!        --- Polar co-ordinates

            do k = 1, m
               elemvc(1,k) = dudx(k,1,1)
               elemvc(2,k) = dudx(k,1,2)
               if ( abs(x(k,1))>epsmac ) then
                  elemvc(3,k) = (dudx(k,2,1) - u(k,2))/x(k,1)
                  elemvc(4,k) = (dudx(k,2,2) + u(k,1))/x(k,1)
               else
                  elemvc(3,k) = -dudx(k,1,2)
                  elemvc(4,k) = dudx(k,1,1)
               end if
            end do
         else

!        --- Cylindrical co-ordinates with swirl

            do k = 1, m
               elemvc(1,k) = dudx(k,1,1)
               elemvc(2,k) = dudx(k,1,3)
               elemvc(3,k) = dudx(k,1,2)
               if ( abs(x(k,1))>epsmac ) then
                  elemvc(4,k) = - u(k,3) / x(k,1)
                  elemvc(5,k) = u(k,1) / x(k,1)
               else
                  elemvc(4,k) = 0d0
                  elemvc(5,k) = 0d0
               end if
               elemvc(6,k) = 0d0
               elemvc(7,k) = dudx(k,2,1)
               elemvc(8,k) = dudx(k,2,3)
               elemvc(9,k) = dudx(k,2,2)
            end do
         end if
         if ( icheld==3 ) then

!        --- icheld = 3, change sign

            do k = 1, m
               do j = 1, ncomp
                  elemvc(j,k) = -elemvc(j,k)
               end do
            end do
         end if

      else if ( icheld==4 ) then

!     --- icheld = 4, compute divergence

         if ( jcart==1 ) then

!        --- Cartesian co-ordinates

            if ( ndimlc==1 ) then

!           --- ndimlc = 1

               do k = 1, m
                  elemvc(1,k) = dudx(k,1,1)
               end do
            else if ( ndimlc==2 ) then

!           --- ndimlc = 2

               do k = 1, m
                  elemvc(1,k) = dudx(k,1,1) + dudx(k,2,2)
               end do
            else

!           --- ndimlc = 3

               do k = 1, m
                  elemvc(1,k) = dudx(k,1,1) + dudx(k,2,2) + dudx(k,3,3)
               end do
            end if
         else if ( jcart==2 .or. jcart==4 ) then

!        --- Cylinder co-ordinates

            if ( ndimlc==2 ) then

!           --- ndimlc = 2

               do k = 1, m
                  if ( abs(x(k,1))>epsmac ) then
                     elemvc(1,k) = dudx(k,1,1) + u(k,1)/x(k,1) + &
                                   dudx(k,2,2)
                  else
                     elemvc(1,k) = 2d0*dudx(k,1,1) + dudx(k,2,2)
                  end if
               end do
            else

!           --- ndimlc = 3

               do k = 1, m
                  if ( abs(x(k,1))>epsmac ) then
                     elemvc(1,k) = dudx(k,1,1) + &
                                   (u(k,1)+dudx(k,2,2))/x(k,1) + &
                                   dudx(k,3,3)
                  else
                     elemvc(1,k) = 2d0*dudx(k,1,1) + dudx(k,3,3)
                  end if
               end do
            end if
         else

!        --- Polar co-ordinates

            do k = 1, m
               if ( abs(x(k,1))>epsmac ) then
                  elemvc(1,k) = dudx(k,1,1) + &
                                (u(k,1)+dudx(k,2,2))/x(k,1)
               else
                  elemvc(1,k) = 2d0*dudx(k,1,1)
               end if
            end do
         end if

      else if ( icheld==5 ) then

!     --- icheld = 5, compute curl

         if ( jcart==1 ) then

!        --- Cartesian co-ordinates

            if ( ndimlc==2 ) then

!           --- ndimlc = 2

               do k = 1, m
                  elemvc(1,k) = dudx(k,1,2) - dudx(k,2,1)
               end do
            else

!           --- ndimlc = 3

               do k = 1, m
                  elemvc(1,k) = dudx(k,2,3) - dudx(k,3,2)
                  elemvc(2,k) = dudx(k,3,1) - dudx(k,1,3)
                  elemvc(3,k) = dudx(k,1,2) - dudx(k,2,1)
               end do
            end if
         else if ( jcart==2 ) then

!        --- Cylinder co-ordinates

            if ( ndimlc==2 ) then

!           --- ndimlc = 2

               do k = 1, m
                  elemvc(1,k) = dudx(k,1,2) - dudx(k,2,1)
               end do
            else

!           --- ndimlc = 3

               do k = 1, m
                  elemvc(2,k) = dudx(k,3,1) - dudx(k,1,3)
                  if ( abs(x(k,1))>epsmac ) then
                     elemvc(1,k) = dudx(k,2,3)/x(k,1) - dudx(k,3,2)
                     elemvc(3,k) = dudx(k,1,2) + &
                                   (u(k,2)- dudx(k,2,1))/x(k,1)
                  else
                     elemvc(1,k) = - dudx(k,3,2)
                     elemvc(3,k) = 2d0*dudx(k,1,2)
                  end if
               end do
            end if
         else if ( jcart==3 ) then

!        --- Polar co-ordinates

            do k = 1, m
               if ( abs(x(k,1))>epsmac ) then
                  elemvc(1,k) = dudx(k,1,2)+(u(k,2)-dudx(k,2,1))/x(k,1)
               else
                  elemvc(1,k) = 2d0*dudx(k,1,2)
               end if
            end do
         else

!        --- Cylinder co-ordinates with swirl

            do k = 1, m
               elemvc(1,k) = - dudx(k,1,3)
               elemvc(2,k) =   dudx(k,2,1) - dudx(k,1,2)
               if ( abs(x(k,1))>epsmac ) then
                  elemvc(3,k) = dudx(k,1,3)+u(k,3)/x(k,1)
               else
                  elemvc(3,k) = 2d0*dudx(k,1,3)
               end if
            end do
        end if

      else

!     --- icheld = 6, compute stress tensor

         if ( jcart==1 ) then

!        --- Cartesian co-ordinates

            if ( ndimlc==2 ) then

!           --- ndimlc = 2

               do k = 1, m
                  elemvc(1,k) = dudx(k,1,1)
                  elemvc(2,k) = dudx(k,2,2)
                  elemvc(4,k) = dudx(k,2,1)+dudx(k,1,2)
               end do
            else

!           --- ndimlc = 3

               do k = 1, m
                  elemvc(1,k) = dudx(k,1,1)
                  elemvc(2,k) = dudx(k,2,2)
                  elemvc(3,k) = dudx(k,3,3)
                  elemvc(4,k) = dudx(k,2,1)+dudx(k,1,2)
                  elemvc(5,k) = dudx(k,3,2)+dudx(k,2,3)
                  elemvc(6,k) = dudx(k,1,3)+dudx(k,3,1)
               end do
            end if
         else if ( jcart==2 ) then

!        --- Cylinder co-ordinates

            if ( ndimlc==2 ) then

!           --- ndimlc = 2

               do k = 1, m
                  elemvc(1,k) = dudx(k,1,1)
                  elemvc(2,k) = dudx(k,2,2)
                  if ( abs(x(1,k))>epsmac ) then
                     elemvc(3,k) = u(k,1)/x(k,1)
                  else
                     elemvc(3,k) = 0d0
                  end if
                  elemvc(4,k) = dudx(k,1,2)+dudx(k,2,1)
               end do
            end if
         end if
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el1005'

      end if  ! ( debug )
      call erclos ( 'el1005' )

      end
