      subroutine el1007 ( elemnp, elemvc, ndcomp, ishape )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.1    date 13-01-1997 Extension of shape numbers
!        version 1.0    date 28-07-1993
!
!   copyright (c) 1993-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for vertices only in the case that the vector
!     has already been filled in all nodal points in array elemnp
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!    element_vector
!     vertex
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer ndcomp, ishape
      double precision  elemnp(ndcomp,*), elemvc(ndcomp,*)

!     elemnp     i    element vector containing the derivatives for subroutine
!                     deriv in all nodal points
!     elemvc     o    element vector containing the derivatives for subroutine
!                     deriv in all vertices
!     ishape     i    Shape number of elements
!     ndcomp     i    Indicates the number of quantities that must be filled
!                     in elemvc and elemwg per nodal point
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, index1(4), index2(8)
      data index1 / 1, 3, 5, 10 /, index2 /1, 3, 7, 9, 19, 21, 25, 27/

!     i        General loop variable
!     index1   Array containing the indices of the vertices of a
!              quadratic tetrahedron
!     index2   Array containing the indices of the vertices of a
!              quadratic hexahedron
!     j        General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      if ( nvert.eq.n .or. ishape.eq.9 .or. ishape.eq.10 .or. &
           ishape.ge.17 ) then

!     --- number vertices is equal to number of nodes: copy

         do j = 1, ndcomp
            do i = 1, n
               elemvc(j,i) = elemnp(j,i)
            end do
         end do

      else if ( ishape.lt.10 ) then

!     --- ishape = 4, 6 or 7 (2d elements)

         do j = 1, ndcomp
            do i = 1, n-1 , 2
               elemvc(j,(i+1)/2) = elemnp(j,i)
            end do
         end do

      else if ( ishape.eq.14 ) then

!     --- ishape = 14, quadratic hexahedron

         do j = 1, ndcomp
            do i = 1, 8
               elemvc(j,i) = elemnp(j,index2(i))
            end do
         end do

      else

!     --- ishape = 12, 15 or 16 quadratic tetrahedron

         do j = 1, ndcomp
            do i = 1, 4
               elemvc(j,i) = elemnp(j,index1(i))
            end do
         end do

       end if

      end
