      subroutine el1009 ( g, f, w, dphidx, ndim, factor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 07-11-2006 Extra parameter factor
!        version  3.0  date 07-08-2005 Extra parameter ndim
!        version  2.0  date 07-11-1993 Complete new subroutine, &
!                                      with new meaning)
!        version  1.0  date 10-06-1988
!
!   copyright (c) 1988-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element vector of the following shape
!        f  = f   + factor / g . grad phi  dx
!         i    i           e -            i
!                                   k              k
!           = f  + sum sum  w   g (x )  dphi /dx (x ) factor
!              i    k   l    k   l          i   l
!     EL1009 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim
      double precision g(m,*), f(n), w(m), dphidx(n,m,*), factor

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     f             i/o   Element vector to be filled
!                         At input F may already have been filled partly
!                         depending on the value of jadd in common celint
!     factor         i    Multiplication factor
!                         factor times g is used
!     g              i    array of length mxndim in which the values of
!                         g in the integration points are stored
!     ndim           i    Dimension of the space.
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision sum

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Check if vector must be added to an existing vector or not
!         If not set vector F equal to zero

      if ( jadd==0 ) then

!     --- jadd = 0, set vector equal to zero

         do i = 1, n
            f(i) = 0d0
         end do

      end if  ! ( jadd==0 )

!     --- Add vector to F

      if ( jdercn==0 ) then

!     --- jdercn=0, derivatives of basis functions constant

         do j = 1, ndim
            do i = 1, n
               sum = 0d0
               do k = 1, m
                  sum = sum + g(k,j) * dphidx(i,1,j) * w(k)
               end do
               f(i) = f(i) + sum*factor
            end do
         end do

      else

!     --- jdercn=1, derivatives of basis functions are variable

         do j = 1, ndim
            do i = 1, n
               sum = 0d0
               do k = 1, m
                  sum = sum + g(k,j) * dphidx(i,k,j) * w(k)
               end do
               f(i) = f(i) + sum*factor
            end do
         end do

      end if

      end
