      subroutine el2003(icho,jdiagloc,g,elemvc,w,phi,nloc,mloc,work,econ)
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.3  date 03-04-1999 Include commons
!        version  4.2  date 22-12-1992 JvdZ: remove phiint, delta;
!                                      include 'SPcommon/cactl/; expl. decl.
!        version  4.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  4.0  date 08-10-1986
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute right hand side vector for an element vector.
!    real case
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icho, jdiagloc, nloc, mloc
      logical econ
      double precision elemvc(*), g(*), w(*), work(*), phi(*)

!     econ       i    indication whether some integer parameters may be saved
!                     for the next calls of el2001 (true) or not (false)
!                     When el2001 is called more than once in an element
!                     subroutine econ = .false. should be used
!     elemvc    i/o   Element vector
!     g          i    array containing the function g
!                     in the integration points
!     icho       i    choice parameter. Possibilities:
!                     icho = ichois + 100 * ispec
!                     ispec:
!                       0  normal case
!                       1  quadratic triangle with 3-point newton cotes rule
!                     ichois:
!                      1   g and w constant
!                      2   general
!                      4,5 see 1,2;   the right-hand-side vector is added to an
!                                     already existing vector
!     jdiagloc      i    Indicates if the basis functions satisfy phi i(j) = 0, &
!                     if i#j (jdiagloc=1) or not (jdiagloc=2)
!     m          i    number of integration points
!     n          i    number of basis functions
!     phi        i    array of length n * m, values of shape
!                     functions in integration points in the sequence:
!                            k
!                     phi (xg ) = phi (i,k)
!                        i
!                     array phi is only filled when gauss integration is applied
!     w          i    array of length m containing the weights for integration
!     work       i    Work array of length m
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, im, k
      double precision h, sum
      integer ichois,kbot,ksteploc
      save    ichois,kbot,ksteploc

!     h          Help variable to store a constant temporarily
!     i          Counting variable
!     ichois     Choice parameter
!     im         (i-1) x m
!     k          Counting variable
!     kbot
!     ksteploc
!     sum        Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if(.not. econ .or. ifirst.eq.0) then

!     --- ifirst = 0  compute ichois, kbot and ksteploc

         if(icho.gt.100) then
            ichois=icho-100
            kbot=2
            ksteploc=2
         else
            ichois=icho
            kbot=1
            ksteploc=1
         end if
      end if
      if(jdiagloc.eq.1) then

!     --- jdiagloc = 1   Newton Cotes rule

         goto( 50, 110, 110, 130, 150, 150 ), ichois

!        --- ichois = 1  g and w constant

 50         h=g(1)*w(1)
            do 100 i=1,nloc
100            elemvc(i)=h
            return

!           --- ichois = 2

110         do 120 i=1,nloc
120            elemvc(i)=w(i)*g(i)
            return

!           --- ichois = 4  g and w constant

130         h=g(1)*w(1)
            do 140 i=1,nloc
140            elemvc(i)=elemvc(i)+h
            return

!           --- ichois = 5

150         do 160 i=kbot,nloc,ksteploc
160            elemvc(i)=elemvc(i)+w(i)*g(i)
            return
      else

!     --- jdiagloc = 2   Gauss rule

         goto ( 200, 300, 300, 400, 500, 500 ), ichois

!        --- ichois = 1  g and w constant

200         h=g(1)*w(1)
            im=0
            do 280 i=1,nloc
               sum=0d0
               do 260 k=1,mloc
260               sum=sum+phi(im+k)
               elemvc(i)=h*sum
280            im=im+mloc
            return

!           --- ichois = 2  general

300         do 310 k=1,mloc
310            work(k)=w(k)*g(k)
            im=0
            do 380 i=1,nloc
               sum=0d0
               do 360 k=1,mloc
360               sum=sum+work(k)*phi(im+k)
               elemvc(i)=sum
380            im=im+mloc
            return

!           --- ichois = 4  g and w constant

400         h=g(1)*w(1)
            im=0
            do 480 i=1,nloc
               sum=0d0
               do 460 k=1,mloc
460               sum=sum+phi(im+k)
               elemvc(i)=elemvc(i)+h*sum
480            im=im+mloc
            return

!           --- ichois = 5  general

500         do 510 k=1,mloc
510            work(k)=w(k)*g(k)
            im=0
            do 580 i=1,nloc
               sum=0d0
               do 560 k=1,mloc
560               sum=sum+work(k)*phi(im+k)
               elemvc(i)=elemvc(i)+sum
580            im=im+mloc
            return
      end if
      end
