      subroutine el6002 ( beta, s, w, mdecl, det )
! ======================================================================
!
!        programmer     Luc Timmermans, Patrick Anderson
!        version 3.2    date  25-01-1996  Layout
!        version 3.1    date  29-11-1995  Addition of determinant
!        version 3.0    date  17-01-1991  source
!        version 2.0    date  07-12-1990  test version
!        version 1.0    date  11-06-1990  first draft
!
!   copyright (c) 1990-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!***********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to spectral element stiffness matrix
!     corresponding to the Helmholtz part of the equation, i.e.
!
!          s   =  s   + /  beta phi  phi de
!           ij     ij   e          j    i
!
!                              i
!              =  s   +  beta(x ) w(i) delta  * det
!                  ij                       ij
!
!             i
!     where  x  is the integration point
!
!     EL6002 is called by the element subroutine ELM600
!            it must be preceded by the subroutines ELP640 and ELP641
!
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
!***********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer mdecl
      double precision beta(mdecl), s(mdecl,mdecl), w(mdecl),det

!     beta     i     array of length mdecl containing the values of the
!                    third coefficient in the integration points
!     det       o   transformationof the determinant
!     mdecl    i     number of integration/interpolation points
!     s       i/o    array of length mdecl x mdecl containing the spectral
!                    element stiffness matrix, at input s may already have
!                    been filled depending on the value of jadd in the common
!                    block celint
!                    since SEPRAN assumes an one-dimensional storage, s
!                    must be stored transposed
!     w        i     array of length mdecl containing the weights for
!                    numerical integration
!***********************************************************************
!
!                     COMMON BLOCKS
!
!***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, j

!     i          Counting variable
!     j          Counting variable
! ********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================

!     * check if Helmholtz matrix must be added to an existing matrix
!       or not, if not set s equal to zero

      if (jadd.eq.0) then
!        * jadd = 0, set s equal to zero

         do 200 i = 1, mdecl

            do 100 j = 1, mdecl

               s(i,j) = 0d0
100         continue
200      continue

      end if

!     * add Helmholtz matrix to s

      do 300 i = 1, mdecl

         s(i,i) = s(i,i) + beta(i)*w(i)*det
300   continue

      end
