      subroutine el6009 ( a, f, w, mdecl )
! ======================================================================
!
!        programmer     Luc Timmermans
!        version 1.1    date  25-01-1996  Layout
!        version 1.0    date  11-06-1993
!
!
!
!   copyright (c) 1993-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to the spectral element vector
!     corresponding to nonhomogeneous Neumann conditions
!
!         f  =  f   +  / a phi  de
!          i     i     e      i
!
!                              i
!            =  f   +  w(i) a(x )
!                i
!
!            i
!     where x  is the integration point
!
!     EL6009 is called by the element subroutine ELM603
!            it must be preceded by the subroutines ELP652 and ELP651
!***********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! ********************************************************************
!
!                    INPUT / OUTPUT PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer mdecl
      double precision a(mdecl), f(mdecl), w(mdecl)

!     a        i   array of length mdecl containing the values of the
!                  boundary condition in the integration points
!     f       i/o  array of length mdecl containing the spectral element
!                  vector
!     mdecl    i   number of integration/interpolation points
!     w        i   array of length mdecl containing the weights for
!                  numerical integration
!***********************************************************************
!
!                 COMMON BLOCKS
!
!***********************************************************************
!
!                  LOCAL PARAMETERS
!
      integer i

!     i     counting variable
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- initialize f to zero

      do 50 i = 1, mdecl
         f(i) = 0d0
50    continue

      if ( jconsf.eq.1 ) then

!     --- jconsf = 1, a constant

         do 100 i = 1, mdecl
            f(i) = f(i) + a(1)*w(i)
100      continue

      end if

      if ( jconsf.eq.2 ) then

!     --- jconsf = 2, general

         do 200 i = 1, mdecl
            f(i) = f(i) + a(i)*w(i)
200      continue

      end if

      end
