      subroutine el6023(g, f, w1, w2, det, modecl, mdecl)
!
!***********************************************************************
!
!     programmer Luc Timmermans
!     version 2.0   date  07-11-1991  change in parameterlist
!     version 1.0   date  20-09-1991  test version
!
!***********************************************************************
!
!     compute contributions to the spectral element vector
!     corresponding to the righthand side of the equation
!
!     EL6023 is called by the element subroutine ELM602
!            it must be preceded by the subroutines ELP640 and ELP642
!
!***********************************************************************
!
!     INPUT/OUTPUT PARAMETERS
!
      use sepmodulecactl
      implicit none

      integer modecl, mdecl
      double precision g(mdecl), f(mdecl), w1(modecl), w2(modecl), &
                       det(modecl,modecl)
!
!     g        i   array of length mdecl containing the values of the
!                  fourth coefficient (righthand side) in the integration
!     node
!     f        o   array of length mdecl containing the spectral element
!                  vector
!     w1       i   array of length modecl containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     w2       i   array of length modecl containing the weights for
!                  numerical integration in the y-direction corresponding
!                  to the reference element
!     det      i   array of length modecl x modecl containing the
!                  transformation determinant
!     modecl   i   number of integration points in one dimension
!     mdecl    i   number of integration points in the element
!
!***********************************************************************
!
!     PARAMETERS IN COMMON BLOCKS
!
!***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, j, ij
      double precision gcon, help, swap
!
!     i        counting variable
!     gcon     constant value in the case that  g is a constant function
!     help     help parameter
!     swap     swap parameter
!
!***********************************************************************

      if (jconsf.eq.1) then
!        * g is constant (per element)

         gcon = g(1)
         do 400 i = 1, modecl
            help = gcon*w1(i)
            do 300 j = 1, modecl

               ij = i + (j - 1)*modecl
               f(ij) = help*w2(j)*det(i,j)
300         continue
400      continue

      else
!        * g is variable

         do 600 i = 1, modecl
            do 500 j = 1, modecl

               ij = i + (j - 1)*modecl
               f(ij) = w1(i)*w2(j)*det(i,j)*g(ij)
500         continue
600      continue

      end if

      if (modecl.eq.2) then

         swap = f(3)
         f(3) = f(4)
         f(4) = swap
      end if

      end
