      subroutine el6029 ( gamma, t, w1, w2, d1, d2, det, p1, p2, p3, &
                          p4, work, modecl, mdecl )
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.1    date  25-01-1996  Layout
!        version 1.0    date  20-09-1995
!
!
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     EL6029 is called by the element subroutine ELM606
!            it must be preceded by the subroutines ELP640 and ELP642
!***********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! ********************************************************************
!
!                    INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer modecl, mdecl
      double precision gamma(mdecl), t(mdecl,mdecl), w1(modecl), &
                       w2(modecl), d1(modecl,modecl), &
                       d2(modecl,modecl), det(modecl,modecl), &
                       p1(modecl,modecl), p2(modecl,modecl), &
                       p3(modecl,modecl), p4(modecl,modecl), &
                       work(mdecl)

!     d1       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     d2       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the y-direction
!     det      i   array of length modecl x modecl containing the
!                  transformation determinant
!     gamma    i   array of length mdecl
!     mdecl    i   number of integration points in the element
!     modecl   i   number of integration points in one dimension
!     p1       i   arrays of length modecl x modecl containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     t        o   array of length mdecl x mdecl containing the spectral
!                  element stiffness matrix, since SEPRAN assumes an
!                  one-dimensional storage, s must be stored transposed
!     w1       i   array of length modecl containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     w2       i   array of length modecl containing the weights for
!                  numerical integration in the y-direction corresponding
!                  to the reference element
!     work         work array of length mdecl, to store products temporarily
!***********************************************************************
!
!                    COMMON BLOCKS
!
!***********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer i, j, k, l, ij, kl, kj, lj, il, ik
      double precision sum, h1

!     h1         Help variable to compute a product
!     i          Counting variable
!     ij         position in the matrix s
!     ik         position in the matrix s
!     il         position in the matrix s
!     j          Counting variable
!     k          Counting variable
!     kj         position in the matrix s
!     kl         position in the matrix s
!     l          General loop variable
!     lj         position in the matrix s
!     sum        Help parameter to compute a sum
!***********************************************************************
!
!                      SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
!***********************************************************************
!
!                     ERROR MESSAGES
!
!***********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- first initialize s as zero

      do 200 i = 1, mdecl
         do 100 j = 1, mdecl
            t(i,j) = 0d0
100      continue
200   continue

      do 1100 i = 1, modecl
         ij = i
         do 1000 j = 1, modecl
            h1 = - p3(i,j)*p4(i,j) - p4(i,j)*p3(i,j)
            work(ij) = h1*gamma(ij)*(w1(i)*w2(j)/det(i,j))*(-0.5d0)
            ij = ij + modecl
1000     continue
1100  continue

!     --- add first factor of diffusion matrix to t

      do 1500 i=1, modecl
         ij = i
         do 1400 j = 1, modecl
            do 1300 k = 1, modecl
               kj = k + (j - 1)*modecl
               sum = 0d0
               do 1200 l = 1, modecl
                  lj = l + (j - 1)*modecl
                  sum = sum + work(lj)*d1(l,k)*d1(l,i)
1200           continue
               t(ij,kj) = t(ij,kj) + sum
1300        continue
            ij = ij + modecl
1400     continue
1500  continue

!     --- store second help factor in work
!
      do 1700 i = 1, modecl
         ij = i
         do 1600 j = 1, modecl
            h1 =   p3(i,j)*p2(i,j) + p4(i,j)*p1(i,j)
            work(ij) = h1*gamma(ij)*(w1(i)*w2(j)/det(i,j))*(-0.5d0)
            ij = ij + modecl
1600     continue
1700  continue

!     --- add second factor of diffusion matrix to t

      do 2100 i = 1, modecl
         ij = i
         do 2000 j = 1, modecl
            do 1900 k = 1, modecl
               kl = k
               do 1800 l = 1, modecl
                  il = i + (l - 1)*modecl
                  t(ij,kl) = t(ij,kl) + work(il)*d1(i,k)*d2(l,j)
               kl = kl + modecl
1800           continue
1900        continue
            ij = ij + modecl
2000     continue
2100  continue

!     --- add third factor of diffusion matrix to t

      do 2500 i = 1, modecl
         ij = i
         do 2400 j = 1, modecl
            do 2300 k = 1, modecl
               kj = k + (j - 1)*modecl
               kl = k
               h1 = work(kj)*d1(k,i)
               do 2200 l = 1, modecl
                  t(ij,kl) = t(ij,kl) + h1*d2(j,l)
                  kl = kl  + modecl
2200           continue
2300        continue
            ij = ij + modecl
2400     continue
2500  continue

!     --- store third help factor in work

      do 2700 i = 1, modecl
         ij = i
         do 2600 j = 1, modecl
            h1 = - p1(i,j)*p2(i,j) - p2(i,j)*p1(i,j)
            work(ij) = h1*gamma(ij)*(w1(i)*w2(j)/det(i,j))*(-0.5d0)
            ij = ij + modecl
2600     continue
2700  continue

!     --- add fourth factor of diffusion matrix to t

      do 3100 i = 1, modecl
         ij = i
         do 3000 j = 1, modecl
            do 2900 k = 1, modecl
               ik = i + (k - 1)*modecl
               sum = 0d0
               do 2800 l = 1, modecl
                  il = i + (l - 1)*modecl
                  sum = sum + work(il)*d2(l,k)*d2(l,j)
2800           continue
               t(ij,ik) = t(ij,ik) + sum
2900        continue
            ij = ij + modecl
3000     continue
3100  continue

      end
