      subroutine el6041 ( u, s, w1, d1, p1, p2, &
                          p3, p4, work, modecl, mdecl )
! ======================================================================
!
!        programmer     Luc Timmermans
!        version 1.1    date  25-01-1996 layout
!        version 5.0    date  27-09-1994 equal number of nodes in each
!                                        direction - P. Minev
!        version 4.0    date  08-03-1993 local vector of TG scheme is
!                                        loaded - P. Minev
!        version 3.0    date  21-03-1992 complete rewrite
!        version 2.0    date  07-11-1991 change in parameterlist
!        version 1.0    date  23-09-1991
!
!
!
!   copyright (c) 1991-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to spectral element stiffness matrix
!     corresponding to the convection part of the equation
!
!     EL6041 is called by the element subroutine ELM600
!***********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! ********************************************************************
!
!                INPUT / OUTPUT PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer modecl, mdecl
      double precision u(mdecl,2), s(mdecl,mdecl), w1(modecl), &
                       d1(modecl,modecl), &
                       p1(modecl,modecl), p2(modecl,modecl), &
                       p3(modecl,modecl), p4(modecl,modecl),work(mdecl)

!     d1       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     mdecl    i   number of integration points in the element
!     modecl   i   number of integration points in the one dimension
!     p1       i   arrays of length modecl x modecl containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     s        o   array of length mdecl x mdecl containing the spectral
!                  element stiffness matrix, since SEPRAN assumes an
!                  one-dimensional storage, s must be stored transposed
!     u        i   array of length mdecl x 2 containing the values of the
!                  convection coefficient in the integration points
!     w1       i   array of length modecl containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     work         work array of length mdecl, to store products temporarily
!***********************************************************************
!
!                          COMMON BLOCKS
!
!***********************************************************************
!
!                   LOCAL PARAMETERS
!
      integer i, j, k, ij, ik, kj
      double precision help
!     help       help variable to compute a product
!     i          Counting variable
!     ij         position in the matrix s
!     ik         position in the matrix s
!     j          Counting variable
!     k          Counting variable
!     kj         position in the matrix s
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- first check if convection matrix must be added to an existing
!         matrix or not, if not set s equal to zero
!
      if (jadd.eq.0) then

!     --- jadd = 0, set s equal to zero

         do 200 i = 1, mdecl
            do 300 j = 1, mdecl
               s(i,j) = 0d0
300         continue
200      continue

      end if

!     --- store first help factor in work

      do 1400 i = 1, modecl

         ij = i
         do 1300 j = 1, modecl

            help = u(ij,1)*p4(i,j) - u(ij,2)*p3(i,j)
            work(ij) = help*w1(i)*w1(j)
            ij = ij + modecl
1300     continue
1400  continue

!     --- add first factor of convection matrix to s

      do 1700 i = 1, modecl

         ij = i
         do 1600 j = 1, modecl

            do 1500 k = 1, modecl

               kj = k + (j - 1)*modecl
               s(kj,ij) = s(kj,ij) + work(ij)*d1(i,k)
1500        continue

            ij = ij + modecl
1600     continue
1700  continue

!     --- store second help factor in work

      do 1900 i = 1, modecl

         ij = i
         do 1800 j = 1, modecl

            help = u(ij,2)*p1(i,j) - u(ij,1)*p2(i,j)
            work(ij) = help*w1(i)*w1(j)
            ij = ij + modecl
1800     continue
1900  continue

!     --- add second factor of convection matrix to s

      do 2200 i = 1, modecl

         ij = i
         do 2100 j = 1, modecl

            do 2000 k = 1, modecl

               ik = i + (k - 1)*modecl
             s(ik,ij) = s(ik,ij) + work(ij)*d1(j,k)
2000        continue

            ij = ij + modecl
2100     continue
2200  continue

      end
