      subroutine el6051 ( f, elemwg, old, dx, p1, p2, p3, p4, &
                          det, work, mone, ichois )
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.3    date  25-02-1997  Addition error message
!        version 1.2    date  25-06-1996  Correction in use elemwg
!        version 1.1    date  25-01-1996  Layout
!        version 1.0    date  10-11-1995
!
!
!   copyright (c) 1995-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the strong gradient of a scalar in the case of spectral
!     elements;
! **********************************************************************
!
!                       KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer mone, ichois
      double precision f(mone,mone), elemwg(mone,mone), &
                       old(mone,mone), &
                       dx(mone,mone), det(mone,mone), &
                       p1(mone,mone), p2(mone,mone), &
                       p3(mone,mone), p4(mone,mone), &
                       work(mone,mone)

!     det     i   jacobian od the transform of the element
!     dx      i   array of length mone x mone contaning the values of the
!                 derivatives of the basisfunctions
!     elemwg  o   element vector containing the weights for subroutine deriva
!     f       o   array of length 2*mv containing the gradient
!     ichois  i   Choice parameter
!     mone    i   number of velocity points in the one dimension
!     old     i   array of length mv containing the scalar
!     p1      i   arrays of length mone x mone containing the partial
!                 derivatives of the coordinates due to transformation for
!                 the pressure
!     p2      i   See p1
!     p3      i   See p1
!     p4      i   See p1
!     work    -   work array, length mv
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k

!     i            counting variable
!     j            counting variable
!     k            counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1619   Component of gradient can not be computed
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

      if ( ichois.lt.1 .or. ichois.gt.2 ) then
         call eropen ( 'el6051' )
         call errint ( ichois, 1 )
         call errsub ( 1619, 1, 0, 0 )
         call erclos ( 'el6051' )
         go to 5000
      end if

      do 600 i = 1, mone

         do 500 j = 1, mone

!        --- calculating the first sum

         work(i,j) = 0d0
         do 400 k = 1, mone
            work(i,j) = work(i,j) + old(k,j)*dx(i,k)
400      continue

!        --- first component of the gradient

         if (ichois.eq.1) f(i,j) = work(i,j)*p4(i,j)

!        --- second component of the gradient

         if (ichois.eq.2) f(i,j) = -work(i,j)* p3(i,j)

!        --- calculating the second sum

         work(i,j) = 0d0
         do 300 k = 1, mone
            work(i,j) = work(i,j) + old(i,k)*dx(j,k)
300      continue

!        --- first component of the gradient

         if (ichois.eq.1) then
            f(i,j) = (f(i,j) - work(i,j)*p2(i,j))/det(i,j)
            elemwg(i,j) = 1.0d0
         end if

!        --- second component of the gradient

         if (ichois.eq.2) then
            f(i,j) = (f(i,j) + work(i,j)*p1(i,j))/det(i,j)
            elemwg(i,j) = 1.0d0
         end if
500      continue
600   continue

5000  continue
      end
