      subroutine el6055 ( f, elemwg, old, dxp, det, p1, p2, p3, p4, &
                          p5, p6, p7, p8, p9, mone, numr )
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.0    date   03-02-1997
!
!   copyrigth (c) 1997-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rigths reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   COMPUTE STRONG GRADIENTS IN CASE OF SPECTRAL ELEMENTS 3D
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer mone, numr
      double precision f(mone,mone,mone), old(mone,mone,mone), &
                       elemwg(mone,mone,mone), det(mone,mone,mone), &
                       p1(mone,mone,mone), p2(mone,mone,mone), &
                       p3(mone,mone,mone), p4(mone,mone,mone), &
                       p5(mone,mone,mone), p6(mone,mone,mone), &
                       p7(mone,mone,mone), p8(mone,mone,mone), &
                       p9(mone,mone,mone), dxp(mone,mone)

!     det      i   ?
!     dxp      i   array of length mone x mone contaning the values of the
!                  derivatives of the basisfunctions for the pressure in the
!                  pressure points in the x-direction
!     elemwg   o   ?
!     f       i/o  array of length m containing the righthandside
!                  or the residual
!     mone     i   number of points in one dimension
!     numr     i   indicates which component of the gradient to be computed
!                  numr = i indicates the i-th component
!     old      i   array of length m containing the old pressure
!     p1       i   arrays of length m containing the elements of the invert
!                  jacobian of the local transform onto the standard
!                  quadrilateral
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     p5       i   See p1
!     p6       i   See p1
!     p7       i   See p1
!     p8       i   See p1
!     p9       i   See p1
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l
      double precision sum1, sum2, sum3

!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     l          General loop variable
!     sum1       Help parameter to compute a sum
!     sum2       Help parameter to compute a sum
!     sum3       Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1619   Component of gradient can not be computed
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( numr.lt.1 .or. numr.gt.3 ) then
         call eropen ( 'el6055' )
         call errint ( numr, 1 )
         call errsub ( 1619, 1, 0, 0 )
         call erclos ( 'el6055' )
         go to 5000
      end if
      go to ( 1, 2, 3) numr

!     --- first component of the gradient

  1   do 400 i = 1, mone
         do 500 j = 1, mone
            do 600 k = 1, mone

!           --- calculate the first factor of the derivative

               sum1 = 0d0
               do 700 l = 1, mone
                  sum1 = sum1 + dxp(i,l)*old(l,j,k)
700            continue
               sum1 = p1(i,j,k)*sum1

!              --- calculate the second factor of the derivative

               sum2 = 0d0
               do 800 l = 1, mone
                  sum2 = sum2 + dxp(j,l)*old(i,l,k)
800            continue
               sum2 = p4(i,j,k)*sum2

!              --- calculate the third factor of the derivative

               sum3 = 0d0
               do 900 l = 1, mone
                  sum3 = sum3 + dxp(k,l)*old(i,j,l)
900            continue
               sum3 = p7(i,j,k)*sum3

               f(i,j,k) = (sum1 + sum2 + sum3)/det(i,j,k)
               elemwg(i,j,k) = 1.0d0

600         continue
500      continue
400   continue

      go to 5000

!     --- second component of the gradient

  2   do 1000 i = 1, mone
         do 1100 j = 1, mone
            do 1200 k = 1, mone

!           --- calculate the first factor of the derivative

               sum1 = 0d0
               do 1300 l = 1, mone
                  sum1 = sum1 + dxp(i,l)*old(l,j,k)
1300           continue
               sum1 = p2(i,j,k)*sum1

!              --- calculate the second factor of the derivative

               sum2 = 0d0
               do 1400 l = 1, mone
                  sum2 = sum2 + dxp(j,l)*old(i,l,k)
1400           continue
               sum2 = p5(i,j,k)*sum2

!              --- calculate the third factor of the derivative

               sum3 = 0d0
               do 1500 l = 1, mone
                  sum3 = sum3 + dxp(k,l)*old(i,j,l)
1500           continue
               sum3 = p8(i,j,k)*sum3

               f(i,j,k) = (sum1 + sum2 + sum3)/det(i,j,k)
               elemwg(i,j,k) = 1.0d0

1200        continue
1100     continue
1000  continue

      go to 5000

!     --- third component of the gradient

  3   do 2000 i = 1, mone
         do 2100 j = 1, mone
            do 2200 k = 1, mone

!           --- calculate the first factor of the derivative

               sum1 = 0d0
               do 2300 l = 1, mone
                  sum1 = sum1 + dxp(i,l)*old(l,j,k)
2300           continue
               sum1 = p3(i,j,k)*sum1

!              --- calculate the second factor of the derivative

               sum2 = 0d0
               do 2400 l = 1, mone
                  sum2 = sum2 + dxp(j,l)*old(i,l,k)
2400           continue
               sum2 = p6(i,j,k)*sum2

!              --- calculate the third factor of the derivative

               sum3 = 0d0
               do 2500 l = 1, mone
                  sum3 = sum3 + dxp(k,l)*old(i,j,l)
2500           continue
               sum3 = p9(i,j,k)*sum3

               f(i,j,k) = (sum1 + sum2 + sum3)/det(i,j,k)
               elemwg(i,j,k) = 1.0d0

2200        continue
2100     continue
2000  continue

5000  continue

      end
