      subroutine el6066(u1, u2, u3, d1, q1, q2, q3, q4, &
                        q5, q6, q7, q8, q9, det, m1, f, ipelwg, &
                        delta, csmago)
! ======================================================================
!
!        programmer     Michel Speetjens
!        version 1.1    date   05-08-1996 Minor changes. P. Anderson
!        version 1.0    date   01-12-1995
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!         Computes eddy viscosity for LES according to the
!         Smagorinsky model.
!         It must be preceded by the subroutine ELP642
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!    large_eddy
!    Smagorinsky
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer m1
      double precision f(m1,m1,m1), ipelwg(m1,m1,m1), &
                       u1(m1,m1,m1),u2(m1,m1,m1), &
                       u3(m1,m1,m1), d1(m1,m1), &
                       det(m1,m1,m1), &
                       q1(m1,m1,m1), q2(m1,m1,m1), &
                       q3(m1,m1,m1), q4(m1,m1,m1), &
                       q5(m1,m1,m1), q6(m1,m1,m1), &
                       q7(m1,m1,m1), q8(m1,m1,m1), &
                       q9(m1,m1,m1)
      double precision delta, csmago

!     csmago       Smagorinsky constant
!     d1       i   array of length m x m contaning the values of the
!                  derivatives of the basisfunctions in the 1 direction
!     delta        LES filter size
!     det      i   array of length m x m x m containing the
!                  transformation determinant
!     f        o   array of length m x m x m containing the computed
!                  eddy viscosity
!     ipelwg
!     m1       i   number of integration points in one direction
!     q1       i   array of length m x m x m containing the partial
!                  derivatives of the coordinates due to
!                  3-D transformation
!     q2       i   See q1
!     q3       i   See q1
!     q4       i   See q1
!     q5       i   See q1
!     q6       i   See q1
!     q7       i   See q1
!     q8       i   See q1
!     q9       i   See q1
!     u1       i   array of length m x m x m containing the solution of
!                  the previous iteration step in the nodal points
!                  (first component)
!     u2       i   array of length m x m x m containing the solution of
!                  the previous iteration step in the nodal points
!                  (second component)
!     u3       i   array of length m x m x m containing the solution of
!                  the previous iteration step in the nodal points
!                  (third component)
! **********************************************************************
!
!                       COMMON BLOCKS
!
      integer ierror, nerror
      common /cconst/ ierror, nerror
      save /cconst/

!                 /cconst/
!    Error codes and number of errors to print
!    ierror  i/o  Error number of last error occurred
!                 0        No error present
!                 >0        Last run-time error
!    nerror   -   Number of error messages printed
!  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - &
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, p
      double precision sum1, sum2, sum3, sum4, sum5, sum6, sum7, &
             sum8,  sum9, h1, h2, h3 , h4, h5, h6, c

!     c          Help variable
!     h1         Help variable
!     h2         Help variable
!     h3         Help variable
!     h4         Help variable
!     h5         Help variable
!     h6         Help variable
!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     p          Counting variable
!     sum1       Parameter to compute a sum
!     sum2       Parameter to compute a sum
!     sum3       Parameter to compute a sum
!     sum4       Parameter to compute a sum
!     sum5       Parameter to compute a sum
!     sum6       Parameter to compute a sum
!     sum7       Parameter to compute a sum
!     sum8       Parameter to compute a sum
!     sum9       Parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('el6066')

      c = (delta*csmago)**2

      do 100 i = 1 , m1
         do 110 j = 1 , m1
            do 120 k = 1 , m1

               ipelwg(i,j,k) = 1d0

         sum1 = 0.0d0
         sum2 = 0.0d0
         sum3 = 0.0d0
         sum4 = 0.0d0
         sum5 = 0.0d0
         sum6 = 0.0d0
         sum7 = 0.0d0
         sum8 = 0.0d0
         sum9 = 0.0d0

         do 130 p = 1, m1

            sum1 = sum1 + d1(i,p)*u1(p,j,k)
            sum2 = sum2 + d1(j,p)*u1(i,p,k)
            sum3 = sum3 + d1(k,p)*u1(i,j,p)
            sum4 = sum4 + d1(i,p)*u2(p,j,k)
            sum5 = sum5 + d1(j,p)*u2(i,p,k)
            sum6 = sum6 + d1(k,p)*u2(i,j,p)
            sum7 = sum7 + d1(i,p)*u3(p,j,k)
            sum8 = sum8 + d1(j,p)*u3(i,p,k)
            sum9 = sum9 + d1(k,p)*u3(i,j,p)

130      continue

         h1 = q1(i,j,k)*sum1 + q4(i,j,k)*sum2 + q7(i,j,k)*sum3
         h2 = q2(i,j,k)*sum4 + q5(i,j,k)*sum5 + q8(i,j,k)*sum6
         h3 = q3(i,j,k)*sum7 + q6(i,j,k)*sum8 + q9(i,j,k)*sum9
         h4 = q1(i,j,k)*sum4 + q4(i,j,k)*sum5 + q7(i,j,k)*sum6 &
              + q2(i,j,k)*sum1 + q5(i,j,k)*sum2 + q8(i,j,k)*sum3
         h5 = q1(i,j,k)*sum7 + q4(i,j,k)*sum8 + q7(i,j,k)*sum9 &
           + q3(i,j,k)*sum1 + q6(i,j,k)*sum2 + q9(i,j,k)*sum3
         h6 = q2(i,j,k)*sum7 + q5(i,j,k)*sum8 + q8(i,j,k)*sum9 &
           + q3(i,j,k)*sum4 + q6(i,j,k)*sum5 + q9(i,j,k)*sum6
             f(i,j,k) = 2.0d0*(h1**2 + h2**2 + h3**2 + &
               (h4**2 + h5**2 + h6**2)/2.0d0)
         f(i,j,k) = c*sqrt(f(i,j,k))/det(i,j,k)

120      continue
110   continue
100   continue

      call erclos('el6066')

      if (ierror.ne.0) call instop

      end
