      subroutine el6093 ( u1, u2, u3, w1, d1, p1, p2, p3, p4, &
                          p5, p6, p7, p8, p9, mod, f )
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   29-09-1994
!
!   copyrigth (c) 1994-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rigths reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute the divergence residual in the case of spectral elements
!     3-D case
!
!     EL6093 is called by the element subroutine ELD607
!            it must be preceded by the subroutines ELP640 and ELP646
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer mod
      double precision u1(mod,mod,mod),u2(mod,mod,mod),u3(mod,mod,mod), &
                       w1(mod), d1(mod,mod), &
                       p1(mod,mod,mod), p2(mod,mod,mod), &
                       p3(mod,mod,mod), p4(mod,mod,mod), &
                       p5(mod,mod,mod), p6(mod,mod,mod), &
                       p7(mod,mod,mod), p8(mod,mod,mod), &
                       p9(mod,mod,mod), f(mod,mod,mod)

!     d1       i   array of length mod x mod contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     f       i/o  array of length mod**3 containing the rigthhand-side vector
!     mod      i   number of integration points in the one dimension
!     p1       i   arrays of length mod x mod containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     p5       i   See p1
!     p6       i   See p1
!     p7       i   See p1
!     p8       i   See p1
!     p9       i   See p1
!     u1       i   array of length mod x mod x mod containing the values
!                  of the first velocity component in the integration
!     node
!     u2       i   array of length mod x mod x mod containing the values
!                  of the second velocity component in the integration
!     node
!     u3       i   array of length mod x mod x mod containing the values
!                  of the third velocity component in the integration
!     node
!     w1       i   array of length mod containing the weigths for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!***********************************************************************
!
!                     COMMON BLOCKS
!
!***********************************************************************
!
!                     LOCAL PARAMETERS
!
      integer i, j, k, l
      double precision sum, sum1, sum2, sum3

!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     l          General loop variable
!     sum        Help parameter to compute a sum
!     sum1       Help parameter to compute a sum
!     sum2       Help parameter to compute a sum
!     sum3       Help parameter to compute a sum
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      do 1700 i = 1, mod
         do 1600 j = 1, mod
            do 1500 k = 1, mod
               sum1 = 0d0
               sum2 = 0d0
               sum3 = 0d0
               do 1550 l = 1, mod
                  sum1 = sum1 + d1(i,l)*u1(l,j,k)
                  sum2 = sum2 + d1(i,l)*u2(l,j,k)
                  sum3 = sum3 + d1(i,l)*u3(l,j,k)
1550           continue
               sum = sum1*p1(i,j,k) + &
                     sum2*p2(i,j,k) + &
                     sum3*p3(i,j,k)

               sum1 = 0d0
               sum2 = 0d0
               sum3 = 0d0
               do 1650 l = 1, mod
                  sum1 = sum1 + d1(j,l)*u1(i,l,k)
                  sum2 = sum2 + d1(j,l)*u2(i,l,k)
                  sum3 = sum3 + d1(j,l)*u3(i,l,k)
1650           continue
               sum = sum + sum1*p4(i,j,k) + &
                           sum2*p5(i,j,k) + &
                           sum3*p6(i,j,k)

               sum1 = 0d0
               sum2 = 0d0
               sum3 = 0d0
               do 1750 l = 1, mod
                  sum1 = sum1 + d1(k,l)*u1(i,j,l)
                  sum2 = sum2 + d1(k,l)*u2(i,j,l)
                  sum3 = sum3 + d1(k,l)*u3(i,j,l)
1750           continue
               sum = sum + sum1*p7(i,j,k) + &
                           sum2*p8(i,j,k) + &
                           sum3*p9(i,j,k)
               f(i,j,k) = w1(i)*w1(j)*w1(k)*sum
1500        continue
1600     continue
1700  continue

      end
