      subroutine el6095( u1, u2, u3, lambda, f, old, w1, d1, p1, p2, &
                         p3, p4, p5, p6, p7, p8, p9, mod )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  1.2  date 24-05-2001 Include common
!        version  1.1  date 30-10-1997 GH: Error in F corrected
!        version  1.0  date 28-02-1997
!
!   copyright (c) 1997-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the convection residual in the case of
!     3-D case visco elastic solver.
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!   none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mod
      double precision u1(mod,mod,mod),u2(mod,mod,mod),u3(mod,mod,mod), &
                       w1(mod), d1(mod,mod), lambda, &
                       p1(mod,mod,mod), p2(mod,mod,mod), &
                       p3(mod,mod,mod), p4(mod,mod,mod), &
                       p5(mod,mod,mod), p6(mod,mod,mod), &
                       p7(mod,mod,mod), p8(mod,mod,mod), &
                       p9(mod,mod,mod), f(mod,mod,mod), old(mod,mod,mod)

!     d1       i   Array of length MOD x MOD contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     f       i/o  Array containing the rigthhand-side vector
!     lambda
!     mod      i   Number of integration points in the one dimension
!     old      i   Array containing the old solution
!     p1       i   Arrays of length MOD x MOD containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     p5       i   See p1
!     p6       i   See p1
!     p7       i   See p1
!     p8       i   See p1
!     p9       i   See p1
!     u1       i   Array of length MOD**3 containing the values of the
!                  convection coefficient of LHSD in the integration points
!     u2       i   See u1
!     u3       i   See u1
!     w1       i   Array of length MOD containing the weigths for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l
      double precision sum, sum1, sum2, sum3

!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     l          General loop variable
!     sum        Help parameter to compute a sum
!     sum1       Help parameter to compute a sum
!     sum2       Help parameter to compute a sum
!     sum3       Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i = 1, mod
         do j = 1, mod
            do k = 1, mod
               sum = 0d0
               do l = 1, mod
                  sum = sum + d1(i,l)*old(l,j,k)
               end do
               sum1 = sum*p1(i,j,k)
               sum2 = sum*p2(i,j,k)
               sum3 = sum*p3(i,j,k)
               sum = 0d0
               do l = 1, mod
                  sum = sum + d1(j,l)*old(i,l,k)
               end do
               sum1 = sum1 + sum*p4(i,j,k)
               sum2 = sum2 + sum*p5(i,j,k)
               sum3 = sum3 + sum*p6(i,j,k)
               sum = 0d0
               do l = 1, mod
                  sum = sum + d1(k,l)*old(i,j,l)
               end do
               sum1 = sum1 + sum*p7(i,j,k)
               sum2 = sum2 + sum*p8(i,j,k)
               sum3 = sum3 + sum*p9(i,j,k)
               f(i,j,k) = f(i,j,k) + lambda*w1(i)*w1(j)*w1(k)* &
                          ( u1(i,j,k)*sum1 +  u2(i,j,k)*sum2 + &
                                              u3(i,j,k)*sum3 )
            end do
         end do
      end do
      end
