      subroutine el6213( coor, index1, x, inter, iinter, moneloc, mloc )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  1.0  date 30-10-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Determine element number for set of arbitrary points
!
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), moneloc, mloc, iinter(*)
      double precision coor(2,*), x(mloc,2), inter(4,*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     iinter i/o   Integer array containing information of points
!                  are already interpolated or not
!     index1  i    Array of length inpelm containing the point numbers
!                  of the nodal points in the element
!     inter  i/o   Array containing on input the (x,y) co-ordinates of the
!                  points to be interpolated such that
!                  x = inter(1,*)
!                  y = inter(2,*)
!                  Output is given in fourth column of inter
!     m       i    number of integration points
!     moneloc    i    number of integration points in one dimension
!     x       i    Array containing the co-ordinate of the nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ij, indx
      double precision lox, loy, rbx, rby, xlocal(1,2)

!     i          Counting variable
!     ij         help parameter
!     indx       help parameter
!     j          Counting variable
!     lox        Minimum x - value
!     loy        Minimum y - value
!     n
!     rbx        Maximum x - value
!     rby        Maximum y - value
!     xlocal
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if (moneloc.ne.2) then
         write (*,*) 'Only first order elements allowed'
         call instop
      end if

!     --- store global integration points

      do 150 i = 1, moneloc
         ij = i
         do 100 j = 1, moneloc
            x(ij,1) = coor(1, index1(ij))
            x(ij,2) = coor(2, index1(ij))
            ij = ij + moneloc
100      continue
150   continue

!     --- get corners of bounding box correspondig to numbering
!         of a standard quadratic element
!         --- determine bounding box of element

      lox  = min(x(1,1),x(2,1),x(3,1),x(4,1))
      rbx  = max(x(1,1),x(2,1),x(3,1),x(4,1))
      loy  = min(x(1,2),x(2,2),x(3,2),x(4,2))
      rby  = max(x(1,2),x(2,2),x(3,2),x(4,2))
      indx=10

!     --- check of any points have to be interpolated

200   if ( (iinter(indx).eq.0) .and. (iinter(1).gt.iinter(2)) &
           .and. ((indx-10).lt.iinter(1)) ) then
         xlocal(1,1)=inter(1,indx)
         xlocal(1,2)=inter(2,indx)

!        --- check if point is in bounding box

         if ( xlocal(1,1).ge.lox .and. &
              xlocal(1,1).le.rbx .and. &
              xlocal(1,2).ge.loy .and. &
              xlocal(1,2).le.rby ) then
           inter(3,indx)=ielem
           iinter(indx)=ielem
           iinter(2) = iinter(2) + 1
           indx = indx + 1
         else
           indx = indx + 1
           goto 200
         end if
      else
         indx = indx + 1
         if ( (indx-10).lt.iinter(1) ) goto 200
      end if
      if ( (indx-10).lt.iinter(1) ) goto 200
      end
