      subroutine el6220 ( coor, index1, x, sol, phix, phiy, phiz, &
                          inter, iinter, numold, mone, m )
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.3    date   30-05-1996 Multiple solutions
!        version 1.2    date   19-03-1996 Correction local param
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   10-08-1995
!
!   copyrigth (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rigths reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!       3D Interpolation of (multiple) solution(s) in an arbitrary point
!       ONLY WORKS WITH ODD NUMBER OF SIDEPOINTS
! **********************************************************************
!
!                       KEYWORDS
!
!    3d
!     spectral
!    interpolation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer index1(*), numold, mone, m,iinter(*)
      double precision coor(3,*), x(m,3), sol(mone*mone*mone,numold), &
                       phix(mone),phiy(mone),phiz(mone), inter(6,*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     iinter     i    ?
!     index1     i    Array of length inpelm containing the point numbers
!                     of the nodal points in the element
!     inter      i    ?
!     m          i    number of integration points
!     mone       i    number of integration points in one dimension
!     numold     i    Number of vectors that are stored in VECOLD
!     phix       i    Array of length n x m x ndim  containing the derivatives
!                     of the basis functions in the sequence:
!                     d phi / dx (xg ) = phix (i,k,1);
!                          i        k
!                     d phi / dy (xg ) = phix (i,k,2);
!                          i        k
!                     d phi / dz (xg ) = phix (i,k,3);
!                          i        k
!                     If the element is a linear tetrahedron (n=4), then the
!                     derivatives are constant and only k = 1 is filled.
!     phiy       i    As phix
!     phiz       i    As phiy
!     sol        i    Solution Array for numold solutions (max.3)
!     x          i    x co-ordinate of point
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, ii, ijk,ind, ntrial,m2,m3
      double precision  lox,loy,loz,rbx,rby,rbz,tolf,swap, &
                        crn(20,3), xlocal(1,3),rdum(50), &
                        f1,f2,f3,j11,j12,j13,j21,j22,j23, &
                        j31,j32,j33, u, v, w, y, &
                        errorf,errorx,delta1,delta2,delta3, &
                        bndval,sum(3)

!     bndval     Parameter used to chech bounds local par.
!     crn        Boundingbox corners
!     delta1     Newton update
!     delta2     Newton update
!     delta3     Newton update
!     errorf     Error in function f
!     errorx     Error in x
!     f1         First component vector function f
!     f2         Second component vector function f
!     f3         Third component vector function f
!     i          Counting variable
!     ii         Counting variable
!     ijk        Counting variable
!     ind        Index of current point
!     j          Counting variable
!     j11        Jacobian component vector function f
!     j12        Jacobian component vector function f
!     j13        Jacobian component vector function f
!     j21        Jacobian component vector function f
!     j22        Jacobian component vector function f
!     j23        Jacobian component vector function f
!     j31        Jacobian component vector function f
!     j32        Jacobian component vector function f
!     j33        Jacobian component vector function f
!     k          Counting variable
!     l          General loop variable
!     lox        Minimum value for x in current element
!     loy        Minimum value for y in current element
!     loz        Minimum value for z in current element
!     m2         Mone * Mone
!     m3         Mone * MOne * Mone
!     ntrial     Maximum number of Newton iterations
!     rbx        Maximum value for x in current element
!     rby        Maximum value for y in current element
!     rbz        Maximum value for z in current element
!     rdum       dummy variable
!     sum        Help parameter to compute a sum
!     swap       swap parameter
!     tolf       Tolerance for function f
!     u
!     v
!     w
!     xlocal     Local coordinate array
!     y
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision fnc3f1,fnc3f2,fnc3f3,fn3j11,fn3j12,fn3j13, &
                       fn3j21,fn3j22,fn3j23,fn3j31,fn3j32,fn3j33

!     ELP648    compute quantities for one-dimensional spectral reference
!               element [-1,1]
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     FN3J11    Jacobian component vector function
!     FN3J12    Jacobian component vector function
!     FN3J13    Jacobian component vector function
!     FN3J21    Jacobian component vector function
!     FN3J22    Jacobian component vector function
!     FN3J23    Jacobian component vector function
!     FN3J31    Jacobian component vector function
!     FN3J32    Jacobian component vector function
!     FN3J33    Jacobian component vector function
!     FNC3F1
!     FNC3F2
!     FNC3F3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1620   Number of side points even
! **********************************************************************
!
!                       PSEUDO CODE
! **********************************************************************
!
!             STATEMENT FUNCTIONS
!
      fnc3f1(u,v,w,y) = y-(1+u)*(1-v)*(1-w)*(-2+u-v-w)*crn(1,1)/8d0 - &
         (1+u)*(1+v)*(1-w)*(-2+u+v-w)*crn(2,1)/8d0 - &
         (1-u)*(1+v)*(1-w)*(-2-u+v-w)*crn(3,1)/8d0 - &
         (1-u)*(1-v)*(1-w)*(-2-u-v-w)*crn(4,1)/8d0 - &
         (1+u)*(1-v)*(1+w)*(-2+u-v+w)*crn(5,1)/8d0 - &
         (1+u)*(1+v)*(1+w)*(-2+u+v+w)*crn(6,1)/8d0 - &
         (1-u)*(1+v)*(1+w)*(-2-u+v+w)*crn(7,1)/8d0 - &
         (1-u)*(1-v)*(1+w)*(-2-u-v+w)*crn(8,1)/8d0 - &
         (1+u)*(1-v**2)*(1-w)*crn(9,1)/4d0 - &
         (1-u**2)*(1+v)*(1-w)*crn(10,1)/4d0 - &
         (1-u)*(1-v**2)*(1-w)*crn(11,1)/4d0 - &
         (1-u**2)*(1-v)*(1-w)*crn(12,1)/4d0 - &
         (1+u)*(1-v)*(1-w**2)*crn(13,1)/4d0 - &
         (1+u)*(1+v)*(1-w**2)*crn(14,1)/4d0 - &
         (1-u)*(1+v)*(1-w**2)*crn(15,1)/4d0 - &
         (1-u)*(1-v)*(1-w**2)*crn(16,1)/4d0 - &
         (1+u)*(1-v**2)*(1+w)*crn(17,1)/4d0 - &
         (1-u**2)*(1+v)*(1+w)*crn(18,1)/4d0 - &
         (1-u)*(1-v**2)*(1+w)*crn(19,1)/4d0 - &
         (1-u**2)*(1-v)*(1+w)*crn(20,1)/4d0

      fnc3f2(u,v,w,y) = y-(1+u)*(1-v)*(1-w)*(-2+u-v-w)*crn(1,2)/8d0 - &
         (1+u)*(1+v)*(1-w)*(-2+u+v-w)*crn(2,2)/8d0 - &
         (1-u)*(1+v)*(1-w)*(-2-u+v-w)*crn(3,2)/8d0 - &
         (1-u)*(1-v)*(1-w)*(-2-u-v-w)*crn(4,2)/8d0 - &
         (1+u)*(1-v)*(1+w)*(-2+u-v+w)*crn(5,2)/8d0 - &
         (1+u)*(1+v)*(1+w)*(-2+u+v+w)*crn(6,2)/8d0 - &
         (1-u)*(1+v)*(1+w)*(-2-u+v+w)*crn(7,2)/8d0 - &
         (1-u)*(1-v)*(1+w)*(-2-u-v+w)*crn(8,2)/8d0 - &
         (1+u)*(1-v**2)*(1-w)*crn(9,2)/4d0 - &
         (1-u**2)*(1+v)*(1-w)*crn(10,2)/4d0 - &
         (1-u)*(1-v**2)*(1-w)*crn(11,2)/4d0 - &
         (1-u**2)*(1-v)*(1-w)*crn(12,2)/4d0 - &
         (1+u)*(1-v)*(1-w**2)*crn(13,2)/4d0 - &
         (1+u)*(1+v)*(1-w**2)*crn(14,2)/4d0 - &
         (1-u)*(1+v)*(1-w**2)*crn(15,2)/4d0 - &
        (1-u)*(1-v)*(1-w**2)*crn(16,2)/4d0 - &
         (1+u)*(1-v**2)*(1+w)*crn(17,2)/4d0 - &
        (1-u**2)*(1+v)*(1+w)*crn(18,2)/4d0 - &
         (1-u)*(1-v**2)*(1+w)*crn(19,2)/4d0 - &
        (1-u**2)*(1-v)*(1+w)*crn(20,2)/4d0

      fnc3f3(u,v,w,y) = y-(1+u)*(1-v)*(1-w)*(-2+u-v-w)*crn(1,3)/8d0 - &
        (1+u)*(1+v)*(1-w)*(-2+u+v-w)*crn(2,3)/8d0 - &
        (1-u)*(1+v)*(1-w)*(-2-u+v-w)*crn(3,3)/8d0 - &
        (1-u)*(1-v)*(1-w)*(-2-u-v-w)*crn(4,3)/8d0 - &
        (1+u)*(1-v)*(1+w)*(-2+u-v+w)*crn(5,3)/8d0 - &
        (1+u)*(1+v)*(1+w)*(-2+u+v+w)*crn(6,3)/8d0 - &
        (1-u)*(1+v)*(1+w)*(-2-u+v+w)*crn(7,3)/8d0 - &
        (1-u)*(1-v)*(1+w)*(-2-u-v+w)*crn(8,3)/8d0 - &
        (1+u)*(1-v**2)*(1-w)*crn(9,3)/4d0 - &
        (1-u**2)*(1+v)*(1-w)*crn(10,3)/4d0 - &
        (1-u)*(1-v**2)*(1-w)*crn(11,3)/4d0 - &
        (1-u**2)*(1-v)*(1-w)*crn(12,3)/4d0 - &
        (1+u)*(1-v)*(1-w**2)*crn(13,3)/4d0 - &
        (1+u)*(1+v)*(1-w**2)*crn(14,3)/4d0 - &
        (1-u)*(1+v)*(1-w**2)*crn(15,3)/4d0 - &
        (1-u)*(1-v)*(1-w**2)*crn(16,3)/4d0 - &
        (1+u)*(1-v**2)*(1+w)*crn(17,3)/4d0 - &
        (1-u**2)*(1+v)*(1+w)*crn(18,3)/4d0 - &
        (1-u)*(1-v**2)*(1+w)*crn(19,3)/4d0 - &
        (1-u**2)*(1-v)*(1+w)*crn(20,3)/4d0

      fn3j11(u,v,w) = 0.125d0*(-(1+u)*(1-v)*(1-w)*crn(1,1) - &
        (1-v)*(1-w)*(-2+u-v-w)*crn(1,1)-(1+u)*(1+v)*(1-w)*crn(2,1) - &
        (1+v)*(1-w)*(-2+u+v-w)*crn(2,1)+(1-u)*(1+v)*(1-w)*crn(3,1) + &
        (1+v)*(1-w)*(-2-u+v-w)*crn(3,1)+(1-u)*(1-v)*(1-w)*crn(4,1) + &
        (1-v)*(1-w)*(-2-u-v-w)*crn(4,1)-(1+u)*(1-v)*(1+w)*crn(5,1) - &
        (1-v)*(1+w)*(-2+u-v+w)*crn(5,1)-(1+u)*(1+v)*(1+w)*crn(6,1) - &
        (1+v)*(1+w)*(-2+u+v+w)*crn(6,1)+(1-u)*(1+v)*(1+w)*crn(7,1) + &
        (1+v)*(1+w)*(-2-u+v+w)*crn(7,1)+(1-u)*(1-v)*(1+w)*crn(8,1) + &
        (1-v)*(1+w)*(-2-u-v+w)*crn(8,1) ) - &
        (1-v**2)*(1-w)*crn(9,1)/4d0+u*(1+v)*(1-w)*crn(10,1)/2 + &
        (1-v**2)*(1-w)*crn(11,1)/4d0+u*(1-v)*(1-w)*crn(12,1)/2d0 - &
        (1-v)*(1-w**2)*crn(13,1)/4d0-(1+v)*(1-w**2)*crn(14,1)/4d0 + &
        (1+v)*(1-w**2)*crn(15,1)/4d0+(1-v)*(1-w**2)*crn(16,1)/4d0 - &
        (1-v**2)*(1+w)*crn(17,1)/4d0+u*(1+v)*(1+w)*crn(18,1)/2d0 + &
        (1-v**2)*(1+w)*crn(19,1)/4d0+u*(1-v)*(1+w)*crn(20,1)/2d0

      fn3j12(u,v,w) = 0.125d0*((1+u)*(1-v)*(1-w)*crn(1,1)+ &
        (1+u)*(1-w)*(-2+u-v-w)*crn(1,1)-(1+u)*(1+v)*(1-w)*crn(2,1)- &
        (1+u)*(1-w)*(-2+u+v-w)*crn(2,1)-(1-u)*(1+v)*(1-w)*crn(3,1)- &
        (1-u)*(1-w)*(-2-u+v-w)*crn(3,1)+(1-u)*(1-v)*(1-w)*crn(4,1)+ &
        (1-u)*(1-w)*(-2-u-v-w)*crn(4,1)+(1+u)*(1-v)*(1+w)*crn(5,1)+ &
        (1+u)*(1+w)*(-2+u-v+w)*crn(5,1)-(1+u)*(1+v)*(1+w)*crn(6,1)- &
        (1+u)*(1+w)*(-2+u+v+w)*crn(6,1)-(1-u)*(1+v)*(1+w)*crn(7,1)- &
        (1-u)*(1+w)*(-2-u+v+w)*crn(7,1)+(1-u)*(1-v)*(1+w)*crn(8,1)+ &
        (1-u)*(1 +w)*(-2-u-v+w)*crn(8,1))+ &
        (1+u)*v*(1- w)*crn(9,1)/2d0-(1-u**2)*(1-w)*crn(10,1)/4d0+ &
        (1-u)*v*(1-w)*crn(11,1)/2d0+(1-u**2)*(1-w)*crn(12,1)/4d0+ &
        (1+u)*(1-w**2)*crn(13,1)/4d0- (1+u)*(1-w**2)*crn(14,1)/4d0- &
        (1-u)*(1-w**2)*crn(15,1)/4d0+(1-u)*(1-w**2)*crn(16,1)/4d0+ &
        (1+u)*v*(1+w)*crn(17,1)/2d0-(1-u**2)*(1+w)*crn(18,1)/4d0+ &
        (1-u)*v*(1+w)*crn(19,1)/2d0+(1-u**2)*(1+w)*crn(20,1)/4d0

      fn3j13(u,v,w) = 0.125d0*((1+u)*(1-v)*(1-w)*crn(1,1)+ &
        (1+u)*(1-v)*(-2+u-v-w)*crn(1,1)+(1+u)*(1+v)*(1-w)*crn(2,1)+ &
        (1+u)*(1+v)*(-2+u+v-w)*crn(2,1)+(1-u)*(1+v)*(1-w)*crn(3,1)+ &
        (1-u)*(1+v)*(-2-u+v-w)*crn(3,1)+(1-u)*(1-v)*(1-w)*crn(4,1)+ &
        (1-u)*(1-v)*(-2-u-v-w)*crn(4,1)-(1+u)*(1-v)*(1+w)*crn(5,1)- &
        (1+u)*(1-v)*(-2+u-v+w)*crn(5,1)-(1+u)*(1+v)*(1+w)*crn(6,1)- &
        (1+u)*(1+v)*(-2+u+v+w)*crn(6,1)-(1-u)*(1+v)*(1+w)*crn(7,1)- &
        (1-u)*(1+v)*(-2-u+v+w)*crn(7,1)-(1-u)*(1-v)*(1+w)*crn(8,1)- &
        (1-u)*(1-v)*(-2-u-v+w)*crn(8,1) ) + &
        (1+u)*(1-v**2)*crn(9,1)/4d0+(1-u**2)*(1+v)*crn(10,1)/4d0 + &
        (1-u)*(1-v**2)*crn(11,1)/4d0+(1-u**2)*(1-v)*crn(12,1)/4d0 + &
        (1+u)*(1-v)*w*crn(13,1)/2d0+(1+u)*(1+v)*w*crn(14,1)/2d0 + &
        (1-u)*(1+v)*w*crn(15,1)/2d0+(1-u)*(1-v)*w*crn(16,1)/2d0 - &
        (1+u)*(1-v**2)*crn(17,1)/4d0-(1- u**2)*(1+v)*crn(18,1)/4d0 - &
        (1-u)*(1-v**2)*crn(19,1)/4d0 -(1- u**2)*(1-v)*crn(20,1)/4d0

      fn3j21(u,v,w) = 0.125d0*( -(1+u)*(1-v)*(1-w)*crn(1,2)- &
        (1-v)*(1-w)*(-2+u-v-w)*crn(1,2)-(1+u)*(1+v)*(1-w)*crn(2,2)- &
        (1+v)*(1-w)*(-2+u+v-w)*crn(2,2)+(1-u)*(1+v)*(1-w)*crn(3,2)+ &
        (1+v)*(1-w)*(-2-u+v-w)*crn(3,2)+(1-u)*(1-v)*(1-w)*crn(4,2)+ &
        (1-v)*(1-w)*(-2-u-v-w)*crn(4,2)-(1+u)*(1-v)*(1+w)*crn(5,2)- &
        (1-v)*(1+w)*(-2+u-v+w)*crn(5,2)-(1+u)*(1+v)*(1+w)*crn(6,2)- &
        (1+v)*(1+w)*(-2+u+v+w)*crn(6,2)+(1-u)*(1+v)*(1+w)*crn(7,2)+ &
        (1+v)*(1+w)*(-2-u+v+w)*crn(7,2)+(1-u)*(1-v)*(1+w)*crn(8,2)+ &
        (1-v)*(1+w)*(-2-u-v+w)*crn(8,2))- &
        (1-v**2)*(1-w)*crn(9,2)/4d0+u*(1+v)*(1-w)*crn(10,2)/2d0 + &
        (1-v**2)*(1-w)*crn(11,2)/4d0+u*(1-v)*(1-w)*crn(12,2)/2d0 - &
        (1-v)*(1-w**2)*crn(13,2)/4d0-(1+v)*(1-w**2)*crn(14,2)/4d0 + &
        (1+v)*(1-w**2)*crn(15,2)/4d0+(1-v)*(1-w**2)*crn(16,2)/4d0 - &
        (1-v**2)*(1+w)*crn(17,2)/4d0+u*(1+v)*(1+w)*crn(18,2)/2d0 + &
        (1-v**2)*(1+w)*crn(19,2)/4d0+u*(1-v)*(1+w)*crn(20,2)/2d0

      fn3j22(u,v,w) = 0.125d0*((1+u)*(1-v)*(1-w)*crn(1,2)+ &
        (1+u)*(1-w)*(-2+u-v-w)*crn(1,2)-(1+u)*(1+v)*(1-w)*crn(2,2)- &
        (1+u)*(1-w)*(-2+u+v-w)*crn(2,2)-(1-u)*(1+v)*(1-w)*crn(3,2)- &
        (1-u)*(1-w)*(-2-u+v-w)*crn(3,2)+(1-u)*(1-v)*(1-w)*crn(4,2)+ &
        (1-u)*(1-w)*(-2-u-v-w)*crn(4,2)+(1+u)*(1-v)*(1+w)*crn(5,2)+ &
        (1+u)*(1+w)*(-2+u-v+w)*crn(5,2)-(1+u)*(1+v)*(1+w)*crn(6,2)- &
        (1+u)*(1+w)*(-2+u+v+w)*crn(6,2)-(1-u)*(1+v)*(1+w)*crn(7,2)- &
        (1-u)*(1+w)*(-2-u+v+w)*crn(7,2)+(1-u)*(1-v)*(1+w)*crn(8,2)+ &
        (1-u)*(1+w)*(-2-u-v+w)*crn(8,2) )+ &
        (1+u)*v*(1-w)*crn(9,2)/2d0-(1-u**2)*(1-w)*crn(10,2)/4d0 + &
        (1-u)*v*(1-w)*crn(11,2)/2d0+(1-u**2)*(1-w)*crn(12,2)/4d0 + &
        (1+u)*(1-w**2)*crn(13,2)/4d0-(1+u)*(1-w**2)*crn(14,2)/4d0 - &
        (1-u)*(1-w**2)*crn(15,2)/4d0+(1-u)*(1-w**2)*crn(16,2)/4d0 + &
        (1+u)*v*(1+w)*crn(17,2)/2d0-(1-u**2)*(1+w)*crn(18,2)/4d0 + &
        (1-u)*v*(1+w)*crn(19,2)/2d0+(1-u**2)*(1+w)*crn(20,2)/4d0

      fn3j23(u,v,w) = 0.125d0*((1+u)*(1-v)*(1-w)*crn(1,2)+ &
        (1+u)*(1-v)*(-2+u-v-w)*crn(1,2)+(1+u)*(1+v)*(1-w)*crn(2,2)+ &
        (1+u)*(1+v)*(-2+u+v-w)*crn(2,2)+(1-u)*(1+v)*(1-w)*crn(3,2)+ &
        (1-u)*(1+v)*(-2-u+v-w)*crn(3,2)+(1-u)*(1-v)*(1-w)*crn(4,2)+ &
        (1-u)*(1-v)*(-2-u-v-w)*crn(4,2)-(1+u)*(1-v)*(1+w)*crn(5,2)- &
        (1+u)*(1-v)*(-2+u-v+w)*crn(5,2)-(1+u)*(1+v)*(1+w)*crn(6,2)- &
        (1+u)*(1+v)*(-2+u+v+w)*crn(6,2)-(1-u)*(1+v)*(1+w)*crn(7,2)- &
        (1-u)*(1+v)*(-2-u+v+w)*crn(7,2)-(1-u)*(1-v)*(1+w)*crn(8,2)- &
        (1-u)*(1-v)*(-2-u-v+w)*crn(8,2) )+ &
        (1+u)*(1-v**2)*crn(9,2)/4d0+(1-u**2)*(1+v)*crn(10,2)/4d0 + &
        (1-u)*(1-v**2)*crn(11,2)/4d0+(1-u**2)*(1-v)*crn(12,2)/4d0 + &
        (1+u)*(1-v)*w*crn(13,2)/2d0 +(1+u)*(1+v)*w*crn(14,2)/2d0 + &
        (1-u)*(1+v)*w*crn(15,2)/2d0 +(1-u)*(1-v)*w*crn(16,2)/2d0 - &
        (1+u)*(1-v**2)*crn(17,2)/4d0-(1-u**2)*(1+v)*crn(18,2)/4d0 - &
        (1-u)*(1-v**2)*crn(19,2)/4d0-(1-u**2)*(1-v)*crn(20,2)/4d0

      fn3j31(u,v,w) = 0.125d0*(-(1+u)*(1-v)*(1-w)*crn(1,3)- &
        (1-v)*(1-w)*(-2+u-v-w)*crn(1,3)-(1+u)*(1+v)*(1-w)*crn(2,3)- &
        (1+v)*(1-w)*(-2+u+v-w)*crn(2,3)+(1-u)*(1+v)*(1-w)*crn(3,3)+ &
        (1+v)*(1-w)*(-2-u+v-w)*crn(3,3)+(1-u)*(1-v)*(1-w)*crn(4,3)+ &
        (1-v)*(1-w)*(-2-u-v-w)*crn(4,3)-(1+u)*(1-v)*(1+w)*crn(5,3)- &
        (1-v)*(1+w)*(-2+u-v+w)*crn(5,3)-(1+u)*(1+v)*(1+w)*crn(6,3)- &
        (1+v)*(1+w)*(-2+u+v+w)*crn(6,3)+(1-u)*(1+v)*(1+w)*crn(7,3)+ &
        (1+v)*(1+w)*(-2-u+v+w)*crn(7,3)+(1-u)*(1-v)*(1+w)*crn(8,3)+ &
        (1-v)*(1+w)*(-2-u-v+w)*crn(8,3) )- &
        (1-v**2)*(1-w)*crn(9,3)/4d0+u*(1+v)*(1-w)*crn(10,3)/2d0 + &
        (1-v**2)*(1-w)*crn(11,3)/4d0+u*(1-v)*(1-w)*crn(12,3)/2d0 - &
        (1-v)*(1-w**2)*crn(13,3)/4d0-(1+v)*(1-w**2)*crn(14,3)/4d0 + &
        (1+v)*(1-w**2)*crn(15,3)/4d0+(1-v)*(1-w**2)*crn(16,3)/4d0 - &
        (1-v**2)*(1+w)*crn(17,3)/4d0+u*(1+v)*(1+w)*crn(18,3)/2d0 + &
        (1-v**2)*(1+w)*crn(19,3)/4d0+u*(1-v)*(1+w)*crn(20,3)/2d0

      fn3j32(u,v,w) = 0.125d0*((1+u)*(1-v)*(1-w)*crn(1,3)+ &
        (1+u)*(1-w)*(-2+u-v-w)*crn(1,3)-(1+u)*(1+v)*(1-w)*crn(2,3)- &
        (1+u)*(1-w)*(-2+u+v-w)*crn(2,3)-(1-u)*(1+v)*(1-w)*crn(3,3)- &
        (1-u)*(1-w)*(-2-u+v-w)*crn(3,3)+(1-u)*(1-v)*(1-w)*crn(4,3)+ &
        (1-u)*(1-w)*(-2-u-v-w)*crn(4,3)+(1+u)*(1-v)*(1+w)*crn(5,3)+ &
        (1+u)*(1+w)*(-2+u-v+w)*crn(5,3)-(1+u)*(1+v)*(1+w)*crn(6,3)- &
        (1+u)*(1+w)*(-2+u+v+w)*crn(6,3)-(1-u)*(1+v)*(1+w)*crn(7,3)- &
        (1-u)*(1+w)*(-2-u+v+w)*crn(7,3)+(1-u)*(1-v)*(1+w)*crn(8,3)+ &
        (1-u)*(1+w)*(-2-u-v+w)*crn(8,3) )+ &
        (1+u)*v*(1-w)*crn(9,3)/2d0 -(1-u**2)*(1-w)*crn(10,3)/4d0 + &
        (1-u)*v*(1-w)*crn(11,3)/2d0+(1-u**2)*(1-w)*crn(12,3)/4d0 + &
        (1+u)*(1-w**2)*crn(13,3)/4d0-(1+u)*(1-w**2)*crn(14,3)/4d0 - &
        (1-u)*(1-w**2)*crn(15,3)/4d0+(1-u)*(1-w**2)*crn(16,3)/4d0 + &
        (1+u)*v*(1+w)*crn(17,3)/2d0-(1-u**2)*(1+w)*crn(18,3)/4d0 + &
        (1-u)*v*(1+w)*crn(19,3)/2d0+(1-u**2)*(1+w)*crn(20,3)/4d0

      fn3j33(u,v,w) = 0.125d0*((1+u)*(1-v)*(1-w)*crn(1,3)+ &
        (1+u)*(1-v)*(-2+u-v-w)*crn(1,3)+(1+u)*(1+v)*(1-w)*crn(2,3)+ &
        (1+u)*(1+v)*(-2+u+v-w)*crn(2,3)+(1-u)*(1+v)*(1-w)*crn(3,3)+ &
        (1-u)*(1+v)*(-2-u+v-w)*crn(3,3)+(1-u)*(1-v)*(1-w)*crn(4,3)+ &
        (1-u)*(1-v)*(-2-u-v-w)*crn(4,3)-(1+u)*(1-v)*(1+w)*crn(5,3)- &
        (1+u)*(1-v)*(-2+u-v+w)*crn(5,3)-(1+u)*(1+v)*(1+w)*crn(6,3)- &
        (1+u)*(1+v)*(-2+u+v+w)*crn(6,3)-(1-u)*(1+v)*(1+w)*crn(7,3)- &
        (1-u)*(1+v)*(-2-u+v+w)*crn(7,3)-(1-u)*(1-v)*(1+w)*crn(8,3)- &
        (1-u)*(1-v)*(-2-u-v+w)*crn(8,3) )+ &
        (1+u)*(1-v**2)*crn(9,3)/4d0+ (1-u**2)*(1+v)*crn(10,3)/4d0 + &
        (1-u)*(1-v**2)*crn(11,3)/4d0+(1-u**2)*(1-v)*crn(12,3)/4d0 + &
        (1+u)*(1-v)*w*crn(13,3)/2d0+ (1+u)*(1+v)*w*crn(14,3)/2d0 + &
        (1-u)*(1+v)*w*crn(15,3)/2d0+(1-u)*(1-v)*w*crn(16,3)/2d0 - &
        (1+u)*(1-v**2)*crn(17,3)/4d0-(1-u**2)*(1+v)*crn(18,3)/4d0 - &
        (1-u)*(1-v**2)*crn(19,3)/4d0- (1-u**2)*(1-v)*crn(20,3)/4d0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      m2 = mone * mone
      m3 = m2 * mone
      bndval = 1.0d0 + 1.0d-7

!     --- check if mone is odd

      if ( mod(mone,2).eq.0 ) then
         call eropen ( 'el6220' )
         call errint ( mone, 1 )
         call errsub ( 1620, 1, 0, 0 )
         call erclos ( 'el6220' )
         go to 1000
      end if

!     --- store global integration points

      do 150 i = 1, mone
         do 100 j = 1, mone
           do 110 k = 1, mone
              ijk = i+(j-1)*mone+(k-1)*m2
             x(ijk,1) = coor(1, index1(ijk))
             x(ijk,2) = coor(2, index1(ijk))
             x(ijk,3) = coor(3, index1(ijk))
 110        continue
 100     continue
 150   continue

!      --- get corners of bounding box correspondig to numbering
!          of a standard quadratic brick element

      crn(1,1) = x(1,1)
      crn(1,2) = x(1,2)
      crn(1,3) = x(1,3)
      crn(2,1) = x(mone,1)
      crn(2,2) = x(mone,2)
      crn(2,3) = x(mone,3)
      crn(3,1) = x(m2,1)
      crn(3,2) = x(m2,2)
      crn(3,3) = x(m2,3)
      crn(4,1) = x(m2-mone+1,1)
      crn(4,2) = x(m2-mone+1,2)
      crn(4,3) = x(m2-mone+1,3)
      crn(5,1) = x(m2*(mone-1)+1,1)
      crn(5,2) = x(m2*(mone-1)+1,2)
      crn(5,3) = x(m2*(mone-1)+1,3)
      crn(6,1) = x(m3-m2+mone,1)
      crn(6,2) = x(m3-m2+mone,2)
      crn(6,3) = x(m3-m2+mone,3)
      crn(7,1) = x(m3,1)
      crn(7,2) = x(m3,2)
      crn(7,3) = x(m3,3)
      crn(8,1) = x(m3-mone+1,1)
      crn(8,2) = x(m3-mone+1,2)
      crn(8,3) = x(m3-mone+1,3)
      crn(9,1) = x((mone+1)/2,1)
      crn(9,2) = x((mone+1)/2,2)
      crn(9,3) = x((mone+1)/2,3)
      crn(10,1) = x(mone*(mone+1)/2,1)
      crn(10,2) = x(mone*(mone+1)/2,2)
      crn(10,3) = x(mone*(mone+1)/2,3)
      crn(11,1) = x(m2-(mone+1)/2+1,1)
      crn(11,2) = x(m2-(mone+1)/2+1,2)
      crn(11,3) = x(m2-(mone+1)/2+1,3)
      crn(12,1) = x(mone*(mone+1)/2-mone+1,1)
      crn(12,2) = x(mone*(mone+1)/2-mone+1,2)
      crn(12,3) = x(mone*(mone+1)/2-mone+1,3)
      crn(13,1) = x(m2*(mone-1)/2+1,1)
      crn(13,2) = x(m2*(mone-1)/2+1,2)
      crn(13,3) = x(m2*(mone-1)/2+1,3)
      crn(14,1) = x(m2*(mone-1)/2+mone,1)
      crn(14,2) = x(m2*(mone-1)/2+mone,2)
      crn(14,3) = x(m2*(mone-1)/2+mone,3)
      crn(15,1) = x(m2*(mone-1)/2+m2,1)
      crn(15,2) = x(m2*(mone-1)/2+m2,2)
      crn(15,3) = x(m2*(mone-1)/2+m2,3)
      crn(16,1) = x(m2*(mone-1)/2+m2-mone+1,1)
      crn(16,2) = x(m2*(mone-1)/2+m2-mone+1,2)
      crn(16,3) = x(m2*(mone-1)/2+m2-mone+1,3)
      crn(17,1) = x(m3-m2+(mone+1)/2,1)
      crn(17,2) = x(m3-m2+(mone+1)/2,2)
      crn(17,3) = x(m3-m2+(mone+1)/2,3)
      crn(18,1) = x(m3-m2+mone*(mone+1)/2,1)
      crn(18,2) = x(m3-m2+mone*(mone+1)/2,2)
      crn(18,3) = x(m3-m2+mone*(mone+1)/2,3)
      crn(19,1) = x(m3-(mone+1)/2+1,1)
      crn(19,2) = x(m3-(mone+1)/2+1,2)
      crn(19,3) = x(m3-(mone+1)/2+1,3)
      crn(20,1) = x(m3-m2+mone*(mone+1)/2-mone+1,1)
      crn(20,2) = x(m3-m2+mone*(mone+1)/2-mone+1,2)
      crn(20,3) = x(m3-m2+mone*(mone+1)/2-mone+1,3)

!     --- determine bounding box of element

      lox  = min(crn(1,1),crn(2,1),crn(3,1),crn(4,1), &
                 crn(5,1),crn(6,1),crn(7,1),crn(8,1), &
                 crn(9,1),crn(10,1),crn(11,1),crn(12,1), &
                 crn(13,1),crn(14,1),crn(15,1),crn(16,1), &
                 crn(17,1),crn(18,1),crn(19,1),crn(20,1))
      rbx  = max(crn(1,1),crn(2,1),crn(3,1),crn(4,1), &
                 crn(5,1),crn(6,1),crn(7,1),crn(8,1), &
                 crn(9,1),crn(10,1),crn(11,1),crn(12,1), &
                 crn(13,1),crn(14,1),crn(15,1),crn(16,1), &
                 crn(17,1),crn(18,1),crn(19,1),crn(20,1))
      loy  = min(crn(1,2),crn(2,2),crn(3,2),crn(4,2), &
                 crn(5,2),crn(6,2),crn(7,2),crn(8,2), &
                 crn(9,2),crn(10,2),crn(11,2),crn(12,2), &
                 crn(13,2),crn(14,2),crn(15,2),crn(16,2), &
                 crn(17,2),crn(18,2),crn(19,2),crn(20,2))
      rby  = max(crn(1,2),crn(2,2),crn(3,2),crn(4,2), &
                 crn(5,2),crn(6,2),crn(7,2),crn(8,2), &
                 crn(9,2),crn(10,2),crn(11,2),crn(12,2), &
                 crn(13,2),crn(14,2),crn(15,2),crn(16,2), &
                 crn(17,2),crn(18,2),crn(19,2),crn(20,2))
      loz  = min(crn(1,3),crn(2,3),crn(3,3),crn(4,3), &
                 crn(5,3),crn(6,3),crn(7,3),crn(8,3), &
                 crn(9,3),crn(10,3),crn(11,3),crn(12,3), &
                 crn(13,3),crn(14,3),crn(15,3),crn(16,3), &
                 crn(17,3),crn(18,3),crn(19,3),crn(20,3))
      rbz  = max(crn(1,3),crn(2,3),crn(3,3),crn(4,3), &
                 crn(5,3),crn(6,3),crn(7,3),crn(8,3), &
                 crn(9,3),crn(10,3),crn(11,3),crn(12,3), &
                 crn(13,3),crn(14,3),crn(15,3),crn(16,3), &
                 crn(17,3),crn(18,3),crn(19,3),crn(20,3))
      ind=10

!     --- check of any points have to be interpolated

   5  if ( (iinter(ind).eq.0) .and. (iinter(1).gt.iinter(2)) &
           .and. ((ind-10).lt.iinter(1)) ) &
      then
          xlocal(1,1)=inter(1,ind)
          xlocal(1,2)=inter(2,ind)
          xlocal(1,3)=inter(3,ind)

!         --- check if point is in bounding box

          if (xlocal(1,1).ge.lox .and. &
              xlocal(1,1).le.rbx .and. &
              xlocal(1,2).ge.loy .and. &
              xlocal(1,2).le.rby .and. &
              xlocal(1,3).ge.loz .and. &
              xlocal(1,3).le.rbz) &
          then

!         --- get initial guess local coordinates

             xlocal(1,1) = (lox+rbx-2*xlocal(1,1))/ &
                           (lox-rbx)
             xlocal(1,2) = (loy+rby-2*xlocal(1,2))/ &
                           (rby-loy)
             xlocal(1,3) = (loz+rbz-2*xlocal(1,3))/ &
                           (loz-rbz)
             ntrial = 35
             tolf   = 1.0d-8
             do 10, k=1, ntrial

!            --- f1 is x - (sumu over x_i N_i)

               f1 = fnc3f1(xlocal(1,1),xlocal(1,2),xlocal(1,3), &
                          inter(1,ind))

!              --- f2 is y - (sumu over x_i N_i)

               f2 = fnc3f2(xlocal(1,1),xlocal(1,2),xlocal(1,3), &
                          inter(2,ind))

!              --- f3 is z - (sumu over x_i N_i)

               f3 = fnc3f3(xlocal(1,1),xlocal(1,2),xlocal(1,3), &
                          inter(3,ind))

!              --- determine derivative of (f1 f2 f3)

               j11 = fn3j11(xlocal(1,1),xlocal(1,2),xlocal(1,3))
               j12 = fn3j12(xlocal(1,1),xlocal(1,2),xlocal(1,3))
               j13 = fn3j13(xlocal(1,1),xlocal(1,2),xlocal(1,3))
               j21 = fn3j21(xlocal(1,1),xlocal(1,2),xlocal(1,3))
               j22 = fn3j22(xlocal(1,1),xlocal(1,2),xlocal(1,3))
               j23 = fn3j23(xlocal(1,1),xlocal(1,2),xlocal(1,3))
               j31 = fn3j31(xlocal(1,1),xlocal(1,2),xlocal(1,3))
               j32 = fn3j32(xlocal(1,1),xlocal(1,2),xlocal(1,3))
               j33 = fn3j33(xlocal(1,1),xlocal(1,2),xlocal(1,3))

!              --- determine the error in (f1 f2 f3)

               errorf = abs(f1)+abs(f2)+abs(f3)

!              --- determine the Newton-update

               delta1 = (-(f3*j13*j22)+f3*j12*j23+f2*j13*j32- &
                           f1*j23*j32-f2*j12*j33+f1*j22*j33)/ &
                         (j13*j22*j31-j12*j23*j31-j13*j21*j32+ &
                          j11*j23*j32+j12*j21*j33-j11*j22*j33)
               delta2 = (f3*j13*j21-f3*j11*j23-f2*j13*j31+ &
                         f1*j23*j31+f2*j11*j33-f1*j21*j33)/ &
                        (j13*j22*j31-j12*j23*j31-j13*j21*j32+ &
                        j11*j23*j32+j12*j21*j33-j11*j22*j33)
               delta3 = (-(f3*j12*j21)+f3*j11*j22+f2*j12*j31 - &
                           f1*j22*j31-f2*j11*j32+f1*j21*j32)/ &
                         (j13*j22*j31-j12*j23*j31-j13*j21*j32+ &
                          j11*j23*j32+j12*j21*j33-j11*j22*j33)
               errorx = abs(delta1)+abs(delta2)+abs(delta3)

!              --- correct the local coordiates

               xlocal(1,1) = xlocal(1,1)+delta1
               xlocal(1,2) = xlocal(1,2)+delta2
               xlocal(1,3) = xlocal(1,3)+delta3

!              --- decide whether to stop iterating

               if (errorf.le.tolf) goto 11
               if (errorx.le.tolf) goto 11

   10        continue

!            --- determine if the point really is in the element

   11        if ((abs(xlocal(1,1)).gt.bndval).or. &
                 (abs(xlocal(1,2)).gt.bndval).or. &
                 (abs(xlocal(1,3)).gt.bndval)) &
             then
              ind = ind + 1
              goto 5
            end if

!           --- calculate basis functions

             swap = xlocal(1,1)
             xlocal(1,1)= xlocal(1,2)
             xlocal(1,2)= -swap
             call elp648(xlocal(1,1),rdum,phix,mone)
             call elp648(xlocal(1,2),rdum,phiy,mone)
             call elp648(xlocal(1,3),rdum,phiz,mone)

!            --- calculate interpolant

             sum(1) = 0.0d0
             sum(2) = 0.0d0
             sum(3) = 0.0d0
             k=1
             do 40, l=1,mone
                do 50, j=1,mone
                  do 60, i=1,mone
                     do 70, ii=1,numold
                        sum(ii)=sum(ii)+ &
                           sol(k,ii)*phix(i)*phiy(j)*phiz(l)
   70                continue
                     k=k+1
   60             continue
   50           continue
   40        continue

             do 80, ii=1,numold
                inter(3+ii,ind)=sum(ii)
   80        continue
             iinter(ind)=1
             iinter(2) = iinter(2) + 1
             ind = ind + 1
          else
          ind = ind + 1
          goto 5
          end if
       else
       ind = ind + 1
       if ( (ind-10).lt.iinter(1) ) goto 5
      end if
       if ( (ind-10).lt.iinter(1) ) goto 5

1000  continue

      end
