      subroutine eld651 ( icount, Amin1B, BTAmin, F2, elemvc )
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.1   date   91-03-21
!        version 1.0   date   90-11-01
!
!
!   copyright (c) 1991  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!   minus the element vector is added to F
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      implicit none

      integer icount
      double precision Amin1B(*), BTAmin, F2(*), elemvc(*)

!
!     icount   i     number of unknowns per element
!
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
!                                         -1
!     Amin1B   i  real vector containing A  B
!
!                                T -1
!     BTAmin i  real equal to B A  B
!
!     F2       i  real vector: F  = - / f dx = f * opp(S)
!                               2     S
!
!     elemvc  i/o real vector containing:
!                          -1   -1   T -1  -1  -1  T
!                 at in ( A  - A  B(B A  B)  (A  B) ) vecold
!                 at out F is added
!
! *********************************************************************
!
!               LOCAL PARAMETERS

      integer i

!
!     i           loop variable
!
! ********************************************************************
!
!                              METHOD
!
!
!       Differential equation:
!
!                    u = - A grad phi
!
!                    div u = f
!
!
!       leads to:
!                     -1
!              U   = A  ( F  - B Phi - C Labda )
!                          1
!
!                       T -1   -1   T -1
!              Phi = ( B A  B )  ( B A  ( F  - C Labda ) - F )
!                                          1                2
!
!              D Labda = F
!
!       the rhs of the first equation is computed
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!       none
!
! ********************************************************************

      call eropen ( 'eld651' )

      do 10 i = 1, icount
         elemvc(i) = Amin1B(i) / BTAmin * F2(i) - elemvc(i)
10    continue

      call erclos ( 'eld651' )

      end
