      subroutine eldgrint
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 31-08-2007 New call to el0250in
!        version  2.0  date 15-09-2004 Use case statement
!        version  1.2  date 17-01-2001 Extension with type 800
!
!   copyright (c) 1999-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute number of integration points per element as function of icheld
!   Help subroutine for deriv
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kchois, metup, mrule, ncoef, &
              jelp, ishift, interpol, iqmat, iqmat1, &
              iseq, jheat, jwcons, igprob
      logical linear

!     igprob         dummy
!     interpol       dummy
!     iqmat          dummy
!     iqmat1         dummy
!     iseq           dummy
!     ishift         dummy
!     jelp           dummy
!     jheat          dummy
!     jwcons         dummy
!     kchois         dummy
!     linear         dummy
!     metup          dummy
!     mrule          dummy
!     ncoef          dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0250IN       Compute the number of integration points for elements of
!                    type 250
!     EL0800IN       Compute the number of integration points for elements of
!                    type 800
!     EL0900IN       Compute the number of integration points for elements of
!                    type 900
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2042   Arrays of special structure per element per integration point
!           not available for this type number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldgrint' )
      n = inpelm
      m = n
      irule = 1
      select case ( itype )

         case (:-1, 1:249, 251:799, 801:809, 811:899, 903: )

!        --- Type < 0 or > 999
!            These types are not available

            call errint ( itype, 1 )
            call errint ( ielgrp, 2 )
            call errsub ( 2042, 2, 0, 0)

         case (250)

!        --- structural equations

            jheat = 0
            call el0250in ( iuser, jheat, igprob )

         case (800,810)

!        --- Second order elliptic equations

            ishift = 0
            iseq = 1
            call el0800in ( iuser, kchois, metup, mrule, ncoef, linear, &
                            jelp, ishift, interpol, iqmat, iqmat1, &
                            iseq, jheat, jwcons )

         case (900:902)

!        --- fluid flow

            call el0900in ( iuser )

      end select  ! case ( itype )

      call erclos ( 'eldgrint' )
      end
