      subroutine eldiscint31 ( phicut, index1, n, k)
! ======================================================================
!
!        programmer    iohn Brusche
!        version  1.0  date 07-12-2005
!
!   copyright (c) 2005-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill matrix phicut with phi values for 3D second order Newton-Cotes
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          n, k, index1(4,k)
      double precision phicut(n,10,*)

!     index1         i    indices of the vertices of the tetrahedra
!     k              i    number of tetrahedra
!     n              i    number of basis functions
!     phicut       i/o   phi values for intersected elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, l
      double precision phi(n,8,k)

!     i              Counting variable
!     j              Counting variable
!     l              Counting variable
!     phi            Copy of phicut
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ2        Print an integer two-dimensional array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint31' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint31'
         call prinin2(index1, 4, k, 'index array')
         call prinrl(phicut,40, 'phicut')

      end if
      if ( ierror/=0 ) go to 1000

!     --- initialize phi

      do i = 1, n
         do j = 1, 8
            do l = 1, k
               phi(i,j,l) = 0d0
            end do
         end do
      end do

!     --- Copy phicut as filled for 1st order Newton Cotes

      do i = 1, n
         if (k==2) then
            do j = 1, 7
               do l = 1, k
                  phi(i,j,l) = phicut(i,j,1)
               end do
            end do
         else
            do j = 1, 8
               do l = 1, k
                  phi(i,j,l) = phicut(i,j,1)
               end do
            end do
         end if
      end do

!     --- Fill phicut for 2nd order Newton Cotes

      do i = 1, n
         do j = 1, k
            phicut(i,1,j) = phi(i,index1(1,j),j)
            phicut(i,2,j) = phi(i,index1(2,j),j)
            phicut(i,3,j) = phi(i,index1(3,j),j)
            phicut(i,4,j) = phi(i,index1(4,j),j)
            phicut(i,5,j) = 0.5d0*(phi(i,index1(1,j),j) + &
                                 phi(i,index1(2,j),j))
            phicut(i,6,j) = 0.5d0*(phi(i,index1(1,j),j) + &
                                 phi(i,index1(3,j),j))
            phicut(i,7,j) = 0.5d0*(phi(i,index1(1,j),j) + &
                                 phi(i,index1(4,j),j))
            phicut(i,8,j) = 0.5d0*(phi(i,index1(2,j),j) + &
                                 phi(i,index1(3,j),j))
            phicut(i,9,j) = 0.5d0*(phi(i,index1(2,j),j) + &
                                 phi(i,index1(4,j),j))
            phicut(i,10,j) = 0.5d0*(phi(i,index1(3,j),j) + &
                                 phi(i,index1(4,j),j))
         end do
      end do

1000  call erclos ( 'eldiscint31' )

      if ( debug ) then

!     --- Debug information

         do k = 1, 2
            do i = 1, 4
               do j = 1, 10
                  write(irefwr,*) 'phicut(',i,',',j,',',k,') ', &
                     phicut(i,j,k)
               end do
            end do
         end do

         write(irefwr,*) 'End eldiscint31'

      end if

      end
