      subroutine elfillcoeffin1 ( coeffintpnts, coeffnodes, ndegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 17-07-2011
!
!   copyright (c) 2011-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coefficients computed in nodes in integration points
!     interpolation in subelements must be used
!     jdiag = 2, nodes and integration points differ
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     element
!     fill
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndegfd
      double precision coeffnodes(n,ndegfd), coeffintpnts(m,ndegfd)

!     coeffintpnts  i/o   Real array in which the values of the
!                         coefficients in the integration points are stored
!     coeffnodes     i    Real array in which the values of the
!                         coefficients in the nodal points are stored
!     ndegfd         i    Number of degrees of freedom per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision maxnode, minnode

!     maxnode        Maximum value of coefficient in nodes of element
!     minnode        Minimum value of coefficient in nodes of element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFILLCOEFFIN2 Fill coefficients computed in nodes in integration points
!                    interpolation in subelements must be used
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillcoeffin1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfillcoeffin1'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      maxnode = maxval(coeffnodes)*1.001d0
      minnode = minval(coeffnodes)*0.999d0

      if ( debug ) then
         write(irefwr,2) 'maxnode, minnode', maxnode, minnode
         write(irefwr,2) 'maxint, minint', &
            maxval(coeffintpnts), minval(coeffintpnts)
      end if  ! ( debug )

!     --- jump in case there is  no overshoot

      if ( maxval(coeffintpnts)<=maxnode .and. &
           minval(coeffintpnts)>=minnode ) go to 1000

      select case ( ishape )

         case ( 4, 7 )

            call elfillcoeffin2 ( coeffintpnts, coeffnodes, ndegfd )

         case default

!        --- Other cases: not yet implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )

      end select  ! case ( ishape )

1000  call erclos ( 'elfillcoeffin1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elfillcoeffin1'

      end if  ! ( debug )

      end
