      subroutine elfillugauss ( u, phi, ugauss, n, ncomp, m, jsol, &
                                jdiag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-08-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill the old solution in Gauss points
!    It is assumed that the solution itself has been already filled
! **********************************************************************
!
!                       KEYWORDS
!
!     fill_solution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, ncomp, m, jsol, jdiag
      double precision u(n,ncomp), phi(n,m), ugauss(m,ncomp)

!     jdiag          i    Indicates if the basis functions satisfy phii(j = 0, &
!                         if i#j (jdiag=1) or not (jdiag=2)
!     jsol           i    Indication if the values of the local vector in the
!                         nodal points or the integration points must be
!                         computed (1) or not (0)
!     m              i    number of integration points
!     n              i    number of basis functions for each component
!     ncomp          i    Number of components of solution vector
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     u              i    Real n x ncomp array in which the values of the
!                         solution vector in the nodal points of the element
!                         are stored in the sequence:
!                         u_i(x_j) = u(j,i)  j = 1(1)n, i = 1 (1) ncomp
!     ugauss         o    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         u_i(x_j) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                         ugauss is only filled if jsol # 0 in common celint
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillugauss' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfillugauss'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jsol==1 .and. jdiag/=1 ) then

!     --- Compute solution in Gauss points
!                   n
!         ug (k) = sum   u (i) phi (k)
!           l      i=1    l       i

         do j = 1, ncomp
            do i = 1, m
               ugauss(i,j) = ddot ( n, u(1,j), 1, phi(1,i), 1 )
            end do
         end do

      else if ( jsol/=0 ) then

!     --- jsol = 1,  Copy u into ugauss

         do j = 1, ncomp
            do i = 1, n
               ugauss(i,j) = u(i,j)
            end do
         end do

      end if

1000  call erclos ( 'elfillugauss' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elfillugauss'

      end if  ! ( debug )

      end
