      function elgetintcoef ( iuser, ipos, ishift, code, nncode, ncode, &
                              isubtract, keywordlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-01-2015 Change for non-existing string
!        version  1.0  date 15-12-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Give elgetintcoef the actual integer value corresponding to iuser(ipos)
!     Raise ipos and ishift
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     scalar
!     string
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleread
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iuser(*), nncode, ncode(nncode), isubtract
      integer, intent(inout) :: ipos, ishift
      character (len=*) :: code(nncode), keywordlc
      integer :: elgetintcoef

!     code           i    Table with subkeywords corresponding to keywordlc
!     elgetintcoef   o    Value of integer parameter
!     ipos          i/o   Position in array iuser
!                         Is raised by 1
!     ishift        i/o   Shift of specific variable with respect to iuser
!                         Is raised by 1
!     isubtract      i    Value which must be subtracted from the sequence
!                         number in code to get the correct parameter
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     keywordlc      i    contains the main keyword corresponding to the
!                         parameter
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     nncode         i    Number of entries in code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq, i

!     i              Counting variable
!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inirepscal

!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2543   string not recognized as subkeyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elgetintcoef' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elgetintcoef'
         write(irefwr,1) 'ipos, ishift', ipos, ishift
         write(irefwr,1) 'iuser(ipos)', iuser(ipos)
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      elgetintcoef = 0

      if ( iuser(ipos)>100100000 ) then

!     --- Parameter refers to string

         iseq = iuser(ipos)-100100000        ! sequence number
         string = texts(instring(iseq))      ! value of string
         nchar =  index ( string, ' ' )-1    ! number of characters

         if ( nchar==0 ) then

!        --- No string found, return with 0 because not yet filled

            elgetintcoef = 0
            ipos = ipos + 1
            ishift = ishift+1
            go to 1000

         end if  ! ( nchar==0 )

         call askupc ( string(1:nchar) )     ! replace by capitals

         if ( debug ) then
            write(irefwr,1) 'iseq, nchar', iseq, nchar
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

!        --- Replace the string by a number

         iseq = 0
         do i = 1, nncode

            if ( code(i)(1:ncode(i))==string(1:nchar) ) then

!           --- string is equal to code i

               iseq = i
               exit

            end if  ! ( code(i)(1:ncode(i))==string(1:nchar) )

         end do  ! i = 1, nncode

         if ( iseq==0 ) then

!        --- string has wrong value

            call errchr ( string(1:nchar), 1 )
            call errchr ( keywordlc, 2 )
            do i = 1, nncode
               call errchr ( code(i)(1:ncode(i)), i+2 )
            end do  ! i = 1, nncode
            call errsub ( 2543, 0, 0, nncode+2 )

         end if  ! ( iseq==0 )

         elgetintcoef = iseq-isubtract

      else

!     --- Parameter refers to integer

         elgetintcoef = inirepscal ( iuser(ipos) )

      end if  ! ( iuser(ipos)>100100000)

      ipos = ipos + 1
      ishift = ishift+1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ipos, ishift', ipos, ishift
         write(irefwr,1) 'elgetintcoef', elgetintcoef
         write(irefwr,*) 'End elgetintcoef'

      end if  ! ( debug )
      call erclos ( 'elgetintcoef' )

      end function elgetintcoef
