      subroutine elm608 ( swork )
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   18-11-1994
!
!
!   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     fill spectral element vector for inhomogeneous Neumann
!     condition; 2D surface in 3D space.
!
!     ELM608 is a help subroutine for subroutine BUILD
!     it is called through the intermediate subroutine EL0000
!
!     so:
!
!     BUILD
!       -  loop over spectral elements
!          -  EL0000
!             -  ELM608
!
!     itype = 608  :   two dimensional spectral element in 3D space
!                      (m nodal points)
! ********************************************************************
!
!                      KEYWORDS
!
!   element_matrix
!   element_vector
!   elliptic_equation
!   parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmodulecactl
      implicit none
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      double precision swork(*)

!     swork          spectral element work array
!***********************************************************************
!
!                        COMMON BLOCKS

!***********************************************************************
!
!                LOCAL PARAMETERS

      integer i

!     i          Counting variable
!***********************************************************************
!
!                    I/O
!
!***********************************************************************
!
!               SUBROUTINES CALLED
!
!     EL0608    initialize parameters for subroutine ELM608
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!***********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!                              - &
! ======================================================================
!
      call eropen('elm608')

      call el0608( E%iuser, E%user, swork(1) )

!     --- this ia a dummy routine which only initialize
!         the matrix and vector
!         set E%mat to zero

      E%mat = 0d0
      E%mmat = 0d0
      E%vec = 0d0

      call erclos('elm608')

      end
