      subroutine elm652 ( ielgrp, user, iuser, x, inpelm, A, J, ndim )
!
! ====================================================================
!
!
!        programmer    Arno van Duin
!        version 1.2   date   29-01-1992 error in call to FUNCCF corrected
!        version 1.1   date   19-03-1991 adjusted for NDIM = 3
!        version 1.0   date   11-01-1990
!
!
!   copyright (c) 1990,1992  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!   the matrix A is computed,  A  = / (C v ) . v  dx
!                               ij  S     i     j
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS
!
      implicit none

      integer ielgrp, inpelm, iuser(*), ndim
      double precision A(*), J, user(*), x(*)
!
!                 (see E.F. Kaasschieter:
!                             Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
!     A       o    real matrix: A  = / (C v ) . v  dx, C is the inverse of the
!                                ij  S     i     j
!                  hydraulic conductivity tensor, v are the basis functions
!
!     ielgrp  i    element group number
!
!     inpelm  i    number of points per element
!
!     iuser   i    Integer user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!
!     J       o    real, determinant of the mapping to a reference triangle
!
!     ndim    i    The dimension of space
!
!     user    i    Real user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!
!     x       i    Array containing the coordinates of the midpoints of the
!                  edges of an element
!
! *********************************************************************
!
!               LOCAL PARAMETERS
!
      integer i, k, istart, jcount
      double precision atens(9), ctens(9), Bmat(9), Cdak(6), &
             A11, A12, A13, A21, A22, A23, A31, A32, A33, xc(3)
!
!                           ^  ^   ^
!     (A  )   real equals / v  v  dx
!       kl ij             ^  ik jl
!                         S
!
!     atens   real matrix containing the hydraulic conductivity tensor
!                                                                        ^
!     Bmat    real matrix containing the mapping to a reference triangle S
!
!                          ^    T
!     Cdak    real matrix: C = B C B / J   ( C = ctens, B = Bmat )
!
!     ctens   real matrix containing the inverse of atens
!
!     i       loop variable
!
!     istart  integer, see SEPRAN Standard Problems 2.1.1
!
!     jcount  Help parameter to compute the centroid of the element
!
!     k       loop variable
!
!     xc      array with the coordinates of the centre point of the element
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
      double precision elm662, funccf
!
!     ELM659  computes the inverse of tensor A
!     ELM660  computes the transformation matrix B
!     ELM661  computes the local inverse of A
!     ELM662  computes the integral of the basis functions over a reference
!             element
!     ERCLOS  delete this routine from call tree
!     EROPEN  add this routine to call tree
!     FUNCCF  get coefficient
!
! ********************************************************************
!
!                              METHOD
!
!
!       Differential equation:
!
!                    u = - A grad phi
!
!                    div u = f
!
!
!       leads to:
!                     -1
!              U   = A  ( F  - B Phi - C Labda )
!                          1
!
!                       T -1   -1   T -1
!              Phi = ( B A  B )  ( B A  ( F  - C Labda ) - F )
!                                          1                2
!
!              D Labda = F
!
!       the matrix A is computed
!
! ====================================================================

      call eropen ( 'elm652' )

!   * Compute the coordinates of the centre point of the element

      do 100 i = 1, ndim
         xc(i) = 0d0
100   continue
      if ( ndim.eq.2 ) jcount = 3
      if ( ndim.eq.3 ) jcount = 4
      do 120 i = 1, jcount
         do 110 k = 1, ndim
            xc(k) = xc(k) + x(ndim*(i-1)+k)/jcount
110      continue
120   continue

!   * Compute atens

      istart = iuser(ielgrp + 5)
      if ( ndim.eq.2 ) then
         if ( iuser(istart).eq.0 ) then
            atens(1) = 0d0
         else if ( iuser(istart).lt.-5 ) then
            atens(1) = user(-iuser(istart))
         else
            atens(1) = funccf ( iuser(istart), xc(1), xc(2), 0d0 )
         end if
         if ( iuser(istart+1).eq.0 ) then
            atens(2) = 0d0
         else if ( iuser(istart+1).lt.-5 ) then
            atens(2) = user(-iuser(istart+1))
         else
            atens(2) = funccf ( iuser(istart+1), xc(1), xc(2), 0d0 )
         end if
         if ( iuser(istart+2).eq.0 ) then
            atens(4) = 0d0
         else if ( iuser(istart+2).lt.-5 ) then
            atens(4) = user(-iuser(istart+2))
         else
            atens(4) = funccf ( iuser(istart+2), xc(1), xc(2), 0d0 )
         end if
         atens(3) = atens(2)
      else
         if ( iuser(istart).eq.0 ) then
            atens(1) = 0d0
         else if ( iuser(istart).lt.-5 ) then
            atens(1) = user(-iuser(istart))
         else
            atens(1) = funccf ( iuser(istart), xc(1), xc(2), xc(3) )
         end if
         if ( iuser(istart+1).eq.0 ) then
            atens(2) = 0d0
         else if ( iuser(istart+1).lt.-5 ) then
            atens(2) = user(-iuser(istart+1))
         else
            atens(2) = funccf ( iuser(istart+1), xc(1), xc(2), xc(3) )
         end if
         if ( iuser(istart+2).eq.0 ) then
            atens(3) = 0d0
         else if ( iuser(istart+2).lt.-5 ) then
            atens(3) = user(-iuser(istart+2))
         else
            atens(3) = funccf ( iuser(istart+2), xc(1), xc(2), xc(3) )
         end if
         if ( iuser(istart+3).eq.0 ) then
            atens(5) = 0d0
         else if ( iuser(istart+3).lt.-5 ) then
            atens(5) = user(-iuser(istart+3))
         else
            atens(5) = funccf ( iuser(istart+3), xc(1), xc(2), xc(3) )
         end if
         if ( iuser(istart+4).eq.0 ) then
            atens(6) = 0d0
         else if ( iuser(istart+4).lt.-5 ) then
            atens(6) = user(-iuser(istart+4))
         else
            atens(6) = funccf ( iuser(istart+4), xc(1), xc(2), xc(3) )
         end if
         if ( iuser(istart+5).eq.0 ) then
            atens(9) = 0d0
         else if ( iuser(istart+5).lt.-5 ) then
            atens(9) = user(-iuser(istart+5))
         else
            atens(9) = funccf ( iuser(istart+5), xc(1), xc(2), xc(3) )
         end if
         atens(4) = atens(2)
         atens(7) = atens(3)
         atens(8) = atens(6)
      end if

!   * Compute ctens

      call elm659 ( atens, ctens, ndim )

!   * Compute Bmat as well as J = det(Bmat)

      call elm660 ( x, Bmat, J, ndim )

!   * Compute Cdak

      call elm661 ( Bmat, ctens, J, Cdak, ndim )


      if ( ndim.eq.2 ) then

!                 S  ^       ^       ^
!      * Compute A = C  A  + C  A  + C  ( A  + A  )
!                     11 11   22 22   12   12   21

         do 220 i = 1, inpelm/2
            do 210 k = 1, inpelm/2
               A11 = elm662 ( i, 1, k, 1, ndim )
               A22 = elm662 ( i, 2, k, 2, ndim )
               A12 = elm662 ( i, 1, k, 2, ndim )
               A21 = elm662 ( i, 2, k, 1, ndim )
               A((i-1)*inpelm/2+k) = Cdak(1) *   A11 + &
                                     Cdak(3) *   A22 + &
                                     Cdak(2) * ( A12 + A21 )
210         continue
220      continue

      else

!      Compute
!
!  S  ^       ^       ^       ^                ^                ^
! A = C  A  + C  A  + C  A  + C  ( A  + A  ) + C  ( A  + A  ) + C  ( A  + A  )
!      11 11   22 22   33 33   12   12   21     32   23   32     31   31   13
!

         do 240 i = 1, 4
            do 230 k = 1, 4
               A11 = elm662 ( i, 1, k, 1, ndim )
               A12 = elm662 ( i, 1, k, 2, ndim )
               A13 = elm662 ( i, 1, k, 3, ndim )
               A21 = elm662 ( i, 2, k, 1, ndim )
               A22 = elm662 ( i, 2, k, 2, ndim )
               A23 = elm662 ( i, 2, k, 3, ndim )
               A31 = elm662 ( i, 3, k, 1, ndim )
               A32 = elm662 ( i, 3, k, 2, ndim )
               A33 = elm662 ( i, 3, k, 3, ndim )
               A((i-1)*4+k) = Cdak(1) *     A11       + &
                              Cdak(3) *     A22       + &
                              Cdak(5) *     A33       + &
                              Cdak(2) * ( A12 + A21 ) + &
                              Cdak(4) * ( A23 + A32 ) + &
                              Cdak(6) * ( A31 + A13 )
230         continue
240      continue

      end if

      call erclos ( 'elm652' )

      end
