      subroutine elm657 ( ielgrp, iuser, user, J, icount, x, F2)
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.1   date   91-03-20 adjusted for NDIM = 3
!        version 1.0   date   90-11-01
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!   the element matrix is filled
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      implicit none

      integer ielgrp, icount, iuser(*)
      double precision user(*), J, F2(icount), x(*)

!
!     ielgrp   i     element group number
!
!     icount   i     number of unknowns per element
!
!     iuser    i     Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!
!     user     i     Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
!     F2          real vector: F  = - / f dx = f * opp(S)
!                               2     S
!
!     x           real array containing the coordinates of the midpoints of the
!                 edges of the element
!
!     J           real, determinant of the mapping to a reference triangle
!
! *********************************************************************
!
!               LOCAL PARAMETERS

      integer i, istart
      double precision funccf

!
!     funccf      function value : value of the coefficients
!
!     i           loop variable
!
!     istart      see SEPRAN Standard Problems 2.1.1
!
! ********************************************************************
!
!                              METHOD
!
!
!       Differential equation:
!
!                    u = - A grad phi
!
!                    div u = f
!
!
!       leads to:
!                     -1
!              U   = A  ( F  - B Phi - C Labda )
!                          1
!
!                       T -1   -1   T -1
!              Phi = ( B A  B )  ( B A  ( F  - C Labda ) - F )
!                                          1                2
!
!              D Labda = F
!
!       the vector F2 is filled
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!       none
!
! ********************************************************************

      call eropen ( 'elm657' )

      istart = iuser(5+ielgrp)
      do 10 i = 1, icount
         if ( iuser(istart+6).eq.0) then
            F2(i) = 0d0
         else if ( iuser(istart+6).lt.-5) then
            F2(i) = - user(-iuser(istart+6)) * J / 2
         else
            F2(i) = funccf ( iuser(istart+6), x(1), x(2), 0d0 )
            F2(i) = - F2(i) * J / 2
         end if
10    continue

      call erclos ( 'elm657' )

      end
