      subroutine elm660 ( x, Bmat, J, ndim )
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.1   date   91-03-19 adjusted for NDIM = 3
!        version 1.0   date   90-11-01
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!   the mapping matrix to a reference triangle is computed
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      implicit none

      integer ndim
      double precision Bmat(*), J, x(*)

!
!     ndim     i     the dimension of space
!
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
!     Bmat     o     real matrix containing the mapping to a reference
!                    triangle S
!
!     det      o     real, determinant of the mapping to a reference triangle
!
!     x        i     real array containing the coordinates of the midpoints of
!                    the edges of the element
!
! *********************************************************************
!
!               LOCAL PARAMETERS
!
!       none
!
! ********************************************************************
!
!                              METHOD
!
!     ndim = 2
!                ( x1 - x2   x1 - x3 )
!       Bmat = 2 ( y1 - y2   y1 - y3 ),    J = det ( Bmat )
!
!       (x1,y1), (x2,y2), (x3,y3) are he midpoints of the edges
!
!     ndim = 3
!                ( x1 - x2   x1 - x3   x1 - x4 )
!       Bmat = 3 ( y1 - y2   y1 - y3   y1 - y4 ),    J = det ( Bmat )
!                ( z1 - z2   z1 - z3   z1 - z4 )
!
!       (x1,y1,z1), (x2,y2,z2), (x3,y3,z3), (x4,y4,z4)  are the
!       midpoints of the edges
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!       none
!
! ********************************************************************

      call eropen ( 'elm660' )

      if ( ndim.eq.2 ) then
         Bmat(1) = 2 * ( x(1) - x(3) )
         Bmat(2) = 2 * ( x(2) - x(4) )
         Bmat(3) = 2 * ( x(1) - x(5) )
         Bmat(4) = 2 * ( x(2) - x(6) )
         J = Bmat(1) * Bmat(4) - Bmat(2) * Bmat(3)
      else
         Bmat(1) = 3 * ( x(1) - x(4) )
         Bmat(2) = 3 * ( x(2) - x(5) )
         Bmat(3) = 3 * ( x(3) - x(6) )
         Bmat(4) = 3 * ( x(1) - x(7) )
         Bmat(5) = 3 * ( x(2) - x(8) )
         Bmat(6) = 3 * ( x(3) - x(9) )
         Bmat(7) = 3 * ( x(1) - x(10))
         Bmat(8) = 3 * ( x(2) - x(11))
         Bmat(9) = 3 * ( x(3) - x(12))
         J = Bmat(1)*(Bmat(5) * Bmat(9) - Bmat(6) * Bmat(8))+ &
             Bmat(4)*(Bmat(8) * Bmat(3) - Bmat(9) * Bmat(2))+ &
             Bmat(7)*(Bmat(2) * Bmat(6) - Bmat(3) * Bmat(5))
      end if

      call erclos ( 'elm660' )

      end
