      subroutine elm661 ( Bmat, ctens, J, Cdak, ndim )
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.1   date   91-03-19 adjusted for NDIM = 3
!        version 1.0   date   90-11-01
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!              ^
!   the matrix C is computed
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      implicit none

      integer ndim
      double precision Bmat(ndim,ndim), ctens(ndim,ndim), J, Cdak(6)

!
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!                                                                             ^
!     Bmat     i   real matrix containing the mapping to a reference triangle S
!
!                               ^    T
!     Cdak     o   real matrix: C = B C B / J   ( C = ctens, B = Bmat )
!
!     ctens    i   real matrix containing the inverse of atens
!
!     J        i   real, determinant of the mapping to a reference triangle
!
!     ndim     i   the dimension of space
!
! *********************************************************************
!
!               LOCAL PARAMETERS
!
!     none
!
! ********************************************************************
!
!                              METHOD
!     ^    T
!     C = Bmat Ctens Bmat / J
!
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!       none
!
! ********************************************************************

      call eropen ( 'elm661' )

      if ( ndim.eq.2 ) then
         Cdak(1) = ( Bmat(1,1) * ( ctens(1,1) * Bmat(1,1) + &
                                   ctens(1,2) * Bmat(2,1) ) + &
                     Bmat(2,1) * ( ctens(2,1) * Bmat(1,1) + &
                                   ctens(2,2) * Bmat(2,1) ) ) / J
         Cdak(2) = ( Bmat(1,1) * ( ctens(1,1) * Bmat(1,2) + &
                                   ctens(1,2) * Bmat(2,2) ) + &
                     Bmat(2,1) * ( ctens(2,1) * Bmat(1,2) + &
                                   ctens(2,2) * Bmat(2,2) ) ) / J
         Cdak(3) = ( Bmat(1,2) * ( ctens(1,1) * Bmat(1,2) + &
                                   ctens(1,2) * Bmat(2,2) ) + &
                     Bmat(2,2) * ( ctens(2,1) * Bmat(1,2) + &
                                   ctens(2,2) * Bmat(2,2) ) ) / J
      else
         Cdak(1) = ( Bmat(1,1) * ( ctens(1,1) * Bmat(1,1) + &
                                   ctens(1,2) * Bmat(2,1) + &
                                   ctens(1,3) * Bmat(3,1) ) + &
                     Bmat(2,1) * ( ctens(2,1) * Bmat(1,1) + &
                                   ctens(2,2) * Bmat(2,1) + &
                                   ctens(2,3) * Bmat(3,1) ) + &
                     Bmat(3,1) * ( ctens(3,1) * Bmat(1,1) + &
                                   ctens(3,2) * Bmat(2,1) + &
                                   ctens(3,3) * Bmat(3,1) ) ) / J
         Cdak(2) = ( Bmat(1,1) * ( ctens(1,1) * Bmat(1,2) + &
                                   ctens(1,2) * Bmat(2,2) + &
                                   ctens(1,3) * Bmat(3,2) ) + &
                     Bmat(2,1) * ( ctens(2,1) * Bmat(1,2) + &
                                   ctens(2,2) * Bmat(2,2) + &
                                   ctens(2,3) * Bmat(3,2) ) + &
                     Bmat(3,1) * ( ctens(3,1) * Bmat(1,2) + &
                                   ctens(3,2) * Bmat(2,2) + &
                                   ctens(3,3) * Bmat(3,2) ) ) / J
         Cdak(3) = ( Bmat(1,2) * ( ctens(1,1) * Bmat(1,2) + &
                                   ctens(1,2) * Bmat(2,2) + &
                                   ctens(1,3) * Bmat(3,2) ) + &
                     Bmat(2,2) * ( ctens(2,1) * Bmat(1,2) + &
                                   ctens(2,2) * Bmat(2,2) + &
                                   ctens(2,3) * Bmat(3,2) ) + &
                     Bmat(3,2) * ( ctens(3,1) * Bmat(1,2) + &
                                   ctens(3,2) * Bmat(2,2) + &
                                   ctens(3,3) * Bmat(3,2) ) ) / J
         Cdak(4) = ( Bmat(1,2) * ( ctens(1,1) * Bmat(1,3) + &
                                   ctens(1,2) * Bmat(2,3) + &
                                   ctens(1,3) * Bmat(3,3) ) + &
                     Bmat(2,2) * ( ctens(2,1) * Bmat(1,3) + &
                                   ctens(2,2) * Bmat(2,3) + &
                                   ctens(2,3) * Bmat(3,3) ) + &
                     Bmat(3,2) * ( ctens(3,1) * Bmat(1,3) + &
                                   ctens(3,2) * Bmat(2,3) + &
                                   ctens(3,3) * Bmat(3,3) ) ) / J
         Cdak(5) = ( Bmat(1,3) * ( ctens(1,1) * Bmat(1,3) + &
                                   ctens(1,2) * Bmat(2,3) + &
                                   ctens(1,3) * Bmat(3,3) ) + &
                     Bmat(2,3) * ( ctens(2,1) * Bmat(1,3) + &
                                   ctens(2,2) * Bmat(2,3) + &
                                   ctens(2,3) * Bmat(3,3) ) + &
                     Bmat(3,3) * ( ctens(3,1) * Bmat(1,3) + &
                                   ctens(3,2) * Bmat(2,3) + &
                                   ctens(3,3) * Bmat(3,3) ) ) / J
         Cdak(6) = ( Bmat(1,3) * ( ctens(1,1) * Bmat(1,1) + &
                                   ctens(1,2) * Bmat(2,1) + &
                                   ctens(1,3) * Bmat(3,1) ) + &
                     Bmat(2,3) * ( ctens(2,1) * Bmat(1,1) + &
                                   ctens(2,2) * Bmat(2,1) + &
                                   ctens(2,3) * Bmat(3,1) ) + &
                     Bmat(3,3) * ( ctens(3,1) * Bmat(1,1) + &
                                   ctens(3,2) * Bmat(2,1) + &
                                   ctens(3,3) * Bmat(3,1) ) ) / J
      end if

      call erclos ( 'elm661' )

      end
