      subroutine elm800vecsec ( elemvec, lambda, dphidx, w, gradt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution of second derivative of function to
!     right-hand side vector
!     Add to existing right-hand side vector
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision elemvec(icount), lambda(m), dphidx(n,m,*), &
                       w(m), gradt(m,ndimlc)

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     elemvec        o    Element vector
!     gradt          i    gradient of t in intergation points
!     lambda         i    Contains the coefficient lambda
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, j, i
      double precision sum, h

!     h              Help variable to store a constant
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The term to be added to the right-hand side is:
!
!     / div ( lambda grad T ) phi_i d Omega
!
!     After integration by parts (Gauss theorem) this becomes:
!
!     / - lambda grad T . grad phi_i d Omega + / lambda grad T . n phi_i d Gamma
!
!     In this subroutine we compute the volume integral
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm800vecsec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm800vecsec'
         call prinrl ( lambda, m, 'lambda' )
         call prinrl ( elemvec, icount, 'elemvc_in' )
         call prinrl1 ( gradt, ndimlc, m, 'dt/dx' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jdercn==1 ) then

!     --- Derivatives are not constant
!         elemvec(i) = sum_m lambda gradT . grad phi_i

         do i = 1, n
            sum = 0d0
            do j = 1, ndimlc
               do k = 1, m
                  sum = sum - w(k)*lambda(k)*gradt(k,j)*dphidx(i,k,j)
               end do  ! k = 1, m
            end do  ! j = 1, ndimlc
            elemvec(i) = elemvec(i)+sum
         end do  ! i = 1, n

      else

!     --- Derivatives are constant
!         elemvec(i) = sum_m lambda gradT . grad phi_i

         do i = 1, n
            sum = 0d0
            do j = 1, ndimlc
               h = gradt(1,j)*dphidx(i,1,j)
               do k = 1, m
                  sum = sum - w(k)*lambda(k)*h
               end do  ! k = 1, m
            end do  ! j = 1, ndimlc
            elemvec(i) = elemvec(i)+sum
         end do  ! i = 1, n

      end if  ! ( jdercn==1 )

1000  call erclos ( 'elm800vecsec' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( elemvec, icount, 'elemvc_out' )
         write(irefwr,*) 'End elm800vecsec'

      end if  ! ( debug )

      end
