      subroutine elm815parms ( iuser )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 12-08-2013 Remove parameters
!        version  1.0  date 23-01-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute parameters irule, m and n for type elements of type 800
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*)

!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ishift, mrule, jelp, jheat, ipend, &
              metup, interpol, iqmat, iqmat1
      logical :: linear

!                    they are not
!     interpol       dummy
!     ipend          Last position in array
!     iqmat          dummy
!     iqmat1         dummy
!     ishift         dummy
!     jelp           dummy
!     jheat          dummy
!     jwcons         dummy
!     linear         dummy
!     metup          dummy
!     mrule          Type of integration rule as given by the user
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0800INMAT    Compute the number of integration points for this element
!                    group using array iuser
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm815parms' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm815parms'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize parameters in commons

      call elcomm

!     --- Set parameters depending on the shape numbers

      call el0800inmat ( iuser, metup, mrule, linear, &
                         jelp, ishift, interpol, iqmat, iqmat1, &
                         jheat, jwcons )
      if ( debug ) then
         write(irefwr,1) 'metup, mrule, jelp, iqmat', &
                          metup, mrule, jelp, iqmat
         write(irefwr,1) 'iqmat1, jheat, jwcons', &
                          iqmat1, jheat, jwcons
         write(irefwr,1) 'n, m', n, m
      end if  ! ( debug )

1000  call erclos ( 'elm815parms' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm815parms'

      end if  ! ( debug )

      end
