      subroutine elmassenthal ( elemvc, vecloc, phi, w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 23-09-2004
!
!   copyright (c) 2004-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the element-wise inner product of the mass matrix and a vector,
!     where the vector is defined per integration point
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     enthalpy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision elemvc(*), phi(n,*), w(*), &
                       vecloc(numold,maxunk,*)

!     elemvc         o    Element vector, contains enthalpy in integration
!     node
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
!     w              i    weights
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision enthalpy(m), sum

!     enthalpy       enthalpy vector in integration points
!     i              Counting variable
!     j              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute M H by  / sum_j phi_i,j H_j
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmassenthal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmassenthal'
         write(irefwr,*) 'iseqenth, numold, maxunk ', &
                          iseqenth, numold, maxunk

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Copy enthalpy

      do j = 1, m
         enthalpy(j) = vecloc(iseqenth,1,j)
      end do  ! j = 1, m

!     --- Compute MH

      do i = 1, n
         sum = 0d0
         do j = 1, m
            sum = sum+w(j)*phi(i,j)*enthalpy(j)
         end do
         elemvc(i) = sum
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( enthalpy, m, 'H' )
         call prinrl1 ( phi, m, n, 'phi' )
         call prinrl ( w, m, 'w' )
         call prinrl ( elemvc, n, 'elemvc' )
         write(irefwr,*) 'End elmassenthal'

      end if
      call erclos ( 'elmassenthal' )

      end
