      subroutine elmmapvelsurf ( uvel, qmat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map velocity vector onto surface using array qmat
!
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     velocity
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision uvel(m,*), qmat(3,3,m)

!     qmat           i    transformation matrix global -> local coordinates
!                         in case of a 3D element
!     uvel          i/o   Contains the velocity vector
!                         At input 3d vector, at output 2d (mapped)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision u1, u2, norm
      integer k

!     k              Counting variable
!     norm           Length of vector
!     u1             Contains the first component of the mapped velocity
!                    vector in the t_1 direction
!     u2             Contains the first component of the mapped velocity
!                    vector in the t_2 direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmmapvelsurf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmmapvelsurf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- uvel := u . s

      do k = 1, m
         u1 = uvel(k,1)*qmat(2,1,k) + &
              uvel(k,2)*qmat(2,2,k) + &
              uvel(k,3)*qmat(2,3,k)
         u2 = uvel(k,1)*qmat(3,1,k) + &
              uvel(k,2)*qmat(3,2,k) + &
              uvel(k,3)*qmat(3,3,k)
         norm = sqrt(u1**2+u2**2)
         uvel(k,1) = u1/norm
         uvel(k,2) = u2/norm
      end do  ! k = 1, m

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elmmapvelsurf'

      end if  ! ( debug )
      call erclos ( 'elmmapvelsurf' )

      end
