      subroutine elp611curved ( dxdxi, x, phider, w, dphidx, index1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 12-06-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill dxdxi and dphidx for a curved 1d element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(3)
      double precision dxdxi(3,m), x(ndim,n), phider(3,m), w(m), &
                       dphidx(3,m)

!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dxdxi          o    work array of length 3xm to store the values of
!                         dx/dxi, dy/dxi and dz/dxi in the integration points
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phider         i    Contains the derivatives of the basis functions in the
!                         reference element
!                         phider(i,j) = d phi_i/ ds (x^j)
!     w              o    array of length m containing the weights for
!                         integration
!     x              i    contains the coordinates of the nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR0    Print error message errornr for element
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp611curved' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp611curved'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute dx/dxi = sum_i x_i d phi_i / dxi

      do k = 1, ndim
         dxdxi(k,1:m) = sum(x(1:3,k)*phider(1:3,k))
      end do  ! k = 1, ndim

!     --- Compute weights

      do k = 1, m

         w(k) = sqrt(dot_product(dxdxi(:,k),dxdxi(:,k)))

         if ( abs(w(k))<=0d0 ) then

!        --- Error 105: h is zero

            call errorelvar0 ( 105 )
            call prinin(index1,2,'node numbers')
            call prinrl2(x,2,ndim,'co-ordinates')
            go to 1000

         end if  ! ( abs(w(k))<=0d0 )

      end do  ! k = 1, m

      if ( ideriv==1 ) then

!     --- Compute derivatives

         do k = 1, 3
            dphidx(k,1:m) = phider(k,1:m) / w(1:m)
         end do

      end if

1000  call erclos ( 'elp611curved' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp611curved'

      end if  ! ( debug )

      end
