      subroutine elp620intpol ( coor, index1, phi, xcoord, ycoord )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-08-2000 Adaptation error message
!        version  1.0  date 05-04-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the values of the basis function in a specific point
!    Linear triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(2,*), phi(*), xcoord, ycoord
      integer index1(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length 3 containing the values of the basis
!                         functions in the point xcoord, ycoord
!     xcoord         i    x-coordinate of point
!     ycoord         i    y-coordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x(2,3), e(3,0:2), delta
      integer j

!     delta          jacobian of element
!     e              real array containing quantities to compute the
!                    derivatives of the basis functions
!     j              Counting variable
!     x              array containing the co-ordinates of the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-diemsional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The three vertices define three linear basis functions phi_i(x,y)
!    with:
!    phi_i(x,y) = a_0 + a_1 x + a_2 y
!    a_1 = dphi_i/dx;  a_2 = dphi_i/dy
!    The computation of these derivatives is copied from elp620
!    a_0 = 1 - a_1 x_i - a_2 y_i, because phi_i(x_i,y_i) = 1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do j = 1, 3
         x(1,j) = coor(1,index1(j))
         x(2,j) = coor(2,index1(j))
      end do
      e(1,1) = x(1,3) - x(1,2)
      e(2,1) = x(1,1) - x(1,3)
      e(3,1) = x(1,2) - x(1,1)
      e(1,2) = x(2,2) - x(2,3)
      e(2,2) = x(2,3) - x(2,1)
      e(3,2) = x(2,1) - x(2,2)
      delta = e(3,2)*e(1,1) - e(1,2)*e(3,1)
      if ( abs(delta).le.0d0 ) then
         call eropen ( 'elp620intpol' )
         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 105, 3, 0, 0 )
         call prinin(index1,3,'node numbers')
         call prinrl2(x,3,2,'co-ordinates')
         call erclos ( 'elp620intpol' )
         go to 1000
      end if
      do j = 1, 3
         e(j,1) = e(j,1)/delta
         e(j,2) = e(j,2)/delta
         e(j,0) = 1d0-e(j,2)*x(1,j)-e(j,1)*x(2,j)
         phi(j) = e(j,0)+e(j,2)*xcoord+e(j,1)*ycoord
      end do

1000  end
