      subroutine elp620transfo ( x, jtransfo )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 12-06-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute x as function of jtransfo
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jtransfo
      double precision x(n,ndim)

!     jtransfo       i    Parameter jtransform as defined in jcart
!     x             i/o   Contains coordinates of nodes
!                         At input the original coordinates
!                         At output the transformed oned
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kx, ky, k
      double precision alpha(3), rr

!     alpha          Contains angles in case of transformation 4
!     k              Counting variable
!     kx             integer help parameter to determine the correct angle
!     ky             integer help parameter to determine the correct angle
!     rr             Radius of cylinder in case of transformation 4
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2147   Parameter out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp620transfo' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp620transfo'
         write(irefwr,1) 'jtransfo', jtransfo
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case (jtransfo)
      case(2)

!     --- Transform: xsi := y, eta := z

         x(:,1) = x(:,2)
         x(:,2) = x(:,3)

      case(3)

!     --- Transform: xsi := x, eta := z

         x(:,2) = x(:,3)

      case(4)

!     --- Transform: xsi := R*atan2(y,x), eta := z, R := sqrt(x**2+y**2)
!         Elements with nodes approx. (-1,0,z) can cause trouble because of
!         atan2(e,-1) = pi and atan2(-e,-1) = -pi !!!

         kx = 0
         ky = 0
         do k = 1, n
            if ( x(k,1)>0.0d0 ) then
               kx = kx + 1
            else
               kx = kx - 1
            end if
            if ( x(k,2)>=0.0d0 ) then
               ky = ky + 1
            else
               ky = ky - 1
            end if
            alpha(k) = atan2(x(k,2), x(k,1))
         end do
         if ( kx==-n .and. abs(ky)<n ) then
            do k = 1, n
               if (alpha(k)<0.0d0) then
                  alpha(k) = alpha(k) + 2d0*pi
               end if
            end do
         end if

         do k = 1, n
             rr = sqrt(x(k,1)**2 + x(k,2)**2)
             x(k,1) = rr*alpha(k)
             x(k,2) = x(k,3)
         end do

      case default

!     --- Incorrect value of jtransfo

         call errint ( jtransfo, 1 )
         call errint ( 2, 2 )
         call errint ( 4, 3 )
         call errchr ( 'jtransfo', 1 )
         call errsub ( 2147, 3, 0, 1 )

      end select  ! case (jtransfo)

1000  call erclos ( 'elp620transfo' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp620transfo'

      end if  ! ( debug )

      end
