      subroutine elp621deriv ( intrule, dphidx, dxidx, dphidxi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 07-08-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute derivatives of basis functions in case of
!     bi-linear quadrilateral
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     derivative
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intrule
      double precision dphidx(4,m,ndim), dxidx(m,2,2), dphidxi(n,m,2)

!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dphidxi        i    Array containing dphi_i/dxi in a point
!     dxidx          i    Contains the derivatives d xi/ dx
!                         dxidx(k,i,j) = dxi_i/dx_j (x^k)
!     intrule        i    Actual integration rule
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i

!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp621deriv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp621deriv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( intrule==1 ) then

!     --- intrule = 1  Newton-Cotes rule
!         Use the fact that dphidxi is either 0 or 1

         dphidx(1,1,1:2) = -dxidx(1,1,1:2)-dxidx(1,2,1:2)
         dphidx(2,1,1:2) = dxidx(1,1,1:2)
         dphidx(3,1,1:2) = 0d0
         dphidx(4,1,1:2) = dxidx(1,2,1:2)

         dphidx(1,2,1:2) = -dxidx(2,1,1:2)
         dphidx(2,2,1:2) = dxidx(2,1,1:2)-dxidx(2,2,1:2)
         dphidx(3,2,1:2) = dxidx(2,2,1:2)
         dphidx(4,2,1:2) = 0d0

         dphidx(1,3,1:2) = 0d0
         dphidx(2,3,1:2) = -dxidx(3,2,1:2)
         dphidx(3,3,1:2) = dxidx(3,1,1:2)+dxidx(3,2,1:2)
         dphidx(4,3,1:2) = -dxidx(3,1,1:2)

         dphidx(1,4,1:2) = -dxidx(4,2,1:2)
         dphidx(2,4,1:2) = 0d0
         dphidx(3,4,1:2) = dxidx(4,1,1:2)
         dphidx(4,4,1:2) = -dxidx(4,1,1:2)+dxidx(4,2,1:2)

      else

!     --- intrule>1  Gauss rule

         do i = 1, n

            do k = 1, m

               dphidx(i,k,1:2) = dphidxi(i,k,1)*dxidx(k,1,1:2) + &
                                 dphidxi(i,k,2)*dxidx(k,2,1:2)

            end do  ! k = 1, m

         end do  ! i = 1, n

      end if  ! ( intrule>1 )

1000  call erclos ( 'elp621deriv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp621deriv'

      end if  ! ( debug )

      end
