      subroutine elp630first ( iweigo, phi, intrule, wh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for a linear tetrahedron
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iweigo, intrule
      double precision phi(4,m), wh(m)

!     intrule        o    Actual integration rule
!     iweigo         o    if 1 the weights must be filled
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     wh             o    Contains the weights in the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision h
      character (len=18) text

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Linear tetrahedron' /
! ======================================================================
!
      call eropen ( 'elp630first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp630first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iweigo = mod(iweigh,10)

      if ( jcart/=1 ) then

!     --- Error 1769: Co-ordinate system not available

         call errorelvar ( jcart-1, 1769, text, 0 )

      end if
      if ( intrule<1 .or. intrule>6 ) then

!     --- Error 1770: Integration rule not available

         call errorelvar ( intrule, 1770, text, 1 )
         intrule = 1

      end if
      if ( intrule==1 .and. m/=4 .or. &
           intrule==2 .and. m/=1 .or. &
           intrule==3 .and. m/=4 .or. &
           intrule==4 .and. m/=5 .or. &
           intrule==5 .and. m/=10 .or. &
           intrule==6 .and. m/=8 ) then

!     --- Error 1424: m and integration rule do not match

         call errorelvar2 ( m, intrule, 1424, text, 0 )

      end if

!     --- Fill array phi

      select case ( intrule )

      case(1)

!     --- intrule = 1  Newton Cotes rule

         phi = 0d0
         do i = 1, 4
            phi(i,i) = 1d0
         end do
         h = 1d0/24d0
         wh = h

      case(2)

!     --- intrule = 2  one-point Gauss rule

         phi(1:4,1) = alam6(1:4)
         wh = wgs6

      case(3)

!     --- intrule = 3  four-point Gauss rule

         phi(:,1:m) = alam7(:,1:m)
         wh = wgs7

      case(4)

!     --- intrule = 4  five-point Gauss rule

         phi(:,1:m) = alam8(:,1:m)
         wh = wgs8

      case(5)

!     --- intrule = 5 10 point Second order Newton Cotes rule

         phi = 0d0
         do i = 1, 4
            phi(i,i) = 1d0
         end do
         phi(1,5) = 0.5d0
         phi(2,5:6) = 0.5d0
         phi(3,6:7) = 0.5d0
         phi(1,7:8) = 0.5d0
         phi(4,8) = 0.5d0
         phi(2,9) = 0.5d0
         phi(4,9) = 0.5d0
         phi(3:4,10) = 0.5d0

         h = 1d0/120d0
         wh(1:4) = -h
         h = 4d0*h
         wh(5:10) = h

      case(6)

!     --- intrule = 6: 8 point Newton Cotes rule

         phi = 0d0
         do i = 1, 4
            phi(i,i) = 1d0
         end do
         h = 1d0/3d0
         phi(1:3,5) = h
         phi(1:2,6) = h
         phi(4,6) = h
         phi(1,7) = h
         phi(3:4,7) = h
         phi(2:4,8) = h

         h = 1d0/240d0
         wh(1:4) = h
         h = 9d0*h
         wh(5:8) = h

      end select  ! case ( intrule )

!     --- Set igausp to 1 in case of integration rules larger than 1

      if ( intrule>1 .and. iweigo==1 ) igausp = 1

1000  call erclos ( 'elp630first' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp630first'

      end if  ! ( debug )

      end
