      subroutine elp642 ( coor, index1, x, d1, det, p1, p2, p3, p4, &
                          mone, m )
! ======================================================================
!
!        programmer     Luc Timmermans
!        version 4.1    date   28-01-1996 Layout
!        version 4.0    date   27-09-1994 removal of d2 - P. Minev
!        version 3.0    date   23-03-1992 removal of transformed gradient
!        version 2.0    date   07-11-1991 change in parameterlist
!        version 1.1    date   18-10-1991 adjustments to array x
!        version 1.0    date   18-09-1991
!
!   copyright (c) 1991-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!***********************************************************************
!
!                         DESCRIPTION
!
!     store global integration points
!     compute transformation determinant and partial derivatives
!
!     ELP642 is called by the element subroutine ELM602
!
!***********************************************************************
!
!                      KEYWORDS
!
!     spectral
!***********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer index1(*), mone, m
      double precision coor(2,*), x(m,2), d1(mone,mone), &
                       det(mone,mone), p1(mone,mone), p2(mone,mone), &
                       p3(mone,mone), p4(mone,mone)

!     coor      i   array of length npoint containing the coordinates
!                   of the nodal points with respect to the global
!                   nummering
!
!                       x  = coor(1,i); y  = coor(2,i)
!                        i               i
!     d1        i   array of length mone x mone contaning the values of the
!                   derivatives of the basisfunctions in the x-direction
!     det       o   array of length mone x mone containing the transformation
!                   determinant
!     index1    i   array of length inpelm containing the point numbers
!                   of the nodal points in the element
!     m         i   number of integration points in the element
!     mone      i   number of intregation points in one dimension
!     p1        o   array of length mone x mone containing the partial
!                   derivatives of the coordinates due to transformation
!     p2        o   array of length mone x mone containing the partial
!                   derivatives of the coordinates due to transformation
!     p3        o   array of length mone x mone containing the partial
!                   derivatives of the coordinates due to transformation
!     p4        o   array of length mone x mone containing the partial
!                   derivatives of the coordinates due to transformation
!     x         o   array containing the coordinates of the integration
!                   points in the element with respect to the local
!                   nummering, length m x 2
!
!                         x(ij,k) = coor(k,index1(ij))
!
!                         where ij = i + (j-1)*k
!***********************************************************************
!
!                      COMMON BLOCKS
!
!***********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer i, j, k, l, ij, kj, il
      double precision h1, h2, h3, h4, swap

!     h1     variable to store a sum or product temporarily
!     h2     variable to store a sum or product temporarily
!     h3     variable to store a sum or product temporarily
!     h4     variable to store a sum or product temporarily
!     i      general loop variable
!     ij     help parameter
!     il     help parameter
!     j      general loop variable
!     k      general loop variable
!     kj     help parameter
!     l      general loop variable
!     swap    swap parameter
!***********************************************************************
!
!                 SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  print error messages
!***********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!     1603   transformation determinant equal to zero
!***********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- store global integration points

      do 150 i = 1, mone

         ij = i
         do 100 j = 1, mone

            x(ij,1) = coor(1, index1(ij))
            x(ij,2) = coor(2, index1(ij))
            ij = ij + mone
100      continue
150   continue

      if (mone.eq.2) then

!     --- mone = 2, different nodal numbering

         swap = x(3,1)
         x(3,1) = x(4,1)
         x(4,1) = swap
         swap = x(3,2)
         x(3,2) = x(4,2)
         x(4,2) = swap
      end if

!     --- compute transformation determinant
!         first compute partial derivatives

      do 350 i = 1, mone
         do 300 j = 1, mone

            h1 = 0d0
            h3 = 0d0
            do 200 k = 1, mone

               kj = k + (j-1)*mone
               h1 = h1 + x(kj,1)*d1(i,k)
               h3 = h3 + x(kj,2)*d1(i,k)
200         continue

            p1(i,j) = h1
            p2(i,j) = h3
            h2 = 0d0
            h4 = 0d0
            do 250 l = 1, mone

               il = i + (l-1)*mone
               h2 = h2 + x(il,1)*d1(j,l)
               h4 = h4 + x(il,2)*d1(j,l)
250         continue

            p3(i,j) = h2
            p4(i,j) = h4
            det(i,j) = p1(i,j)*p4(i,j) - p2(i,j)*p3(i,j)

!           --- check if determinant equal to zero

            if (det(i,j).lt.1d-12) then
               print *,'det(i,j) = ',det(i,j)
               call eropen ('elp642')
               call errsub ( 1603, 0, 0, 0)
               call erclos ('elp642')
               go to 1000
            end if

300      continue
350   continue

1000  continue

      end
