      subroutine elp644( coor, index1, x, wref, w, d, m)
! ======================================================================
!
!        programmer     Luc Timmermans
!        version 2.1    date   28-01-1996 Layout
!        version 2.0    date   15-12-1994 general curve is introduced - &
!                                         P. Minev
!        version 1.0    date   12-08-1993
!
!   copyright (c) 1993-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!***********************************************************************
!
!                         DESCRIPTION
!
!     store integration points
!     transform weights and derivatives from the reference element
!     line elements only
!
!     ELP644 is called by the subroutine ELM603 or ELM605
! **********************************************************************
!
!                      KEYWORDS
!
!     spectral
! **********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer index1(*), m
      double precision coor(*), x(m,2), wref(m), w(m), d(m,m)

!     coor     i     array of length npoint containing the co-ordinates of
!                    the nodal points with respect to the global numbering
!                    x  = coor (1,i)
!                     i
!                    y  = coor (2,i)
!                     i
!     d        i     array of length m*m containing the values of derivatives
!                    of the basic functions at the
!     index1   i     array containing the point numbers of the nodal points
!                    in the elements, length inpelm
!     m        i     number of integration points
!     w        o     array of length m containing the modified
!                    weights for numerical integration (element dependent)
!     wref     i     array of length m containing the weights for numerical
!                    integration corresponding to the reference element
!     x       o      array containing the coordinates of the integration
!                    points in the element, length m x 2
!                    x  = x(i,1)  i = 1(1)m
!                     i
!                    y  = x(i,2)  i = 1(1)m
!                     i
!***********************************************************************
!
!                   COMMON BLOCKS
!
!***********************************************************************
!
!                   LOCAL PARAMETERS

      integer i, j
      double precision trans, xp, yp

!     i          counting variable
!     j          counting variable
!     trans      transformation Jacobian
!     xp         (dx/dt)**2
!     yp         (dy/dt)**2
!***********************************************************************
!
!                  SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  print error messages
!***********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                   ERROR MESSAGES
!
!    1603    transformation jacobian equal to zero
!***********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- compute kchois
!         store coordinates of the nodal points in array x

      do 100 i = 1, m
         x(i,1) = coor(1+2*(index1(i)-1))
         x(i,2) = coor(2+2*(index1(i)-1))
100   continue

!     --- compute transformation Jacobian
!         first compute dx/dt and dy/dt

      do 200 i = 1, m
         xp = 0d0
         yp = 0d0
         do 300 j = 1, m
            xp = xp + x(j,1)*d(i,j)
            yp = yp + x(j,2)*d(i,j)
300      continue
         trans = sqrt(xp**2 + yp**2)

!        --- check value of Jacobian

         if (trans.lt.1d-4) then
            call eropen ( 'elp644' )
            call errsub ( 1603, 0, 0, 0)
            call erclos ( 'elp644' )
            go to 1000
         end if

!        --- compute weights

         w(i) = trans*wref(i)

200   continue

1000  continue

      end
