      subroutine elp649(coor, index1, x, d1, det, mone, m)
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   15-12-1994
!
!   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!***********************************************************************
!
!                         DESCRIPTION
!
!     store global integration points
!     compute transformation determinant and partial derivatives
!
!     ELP649 is called by the element subroutine ELM608
!***********************************************************************
!
!                      KEYWORDS
!
!     spectral
!***********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer index1(*), mone, m
      double precision coor(3,*), x(m,3), d1(mone,mone), &
                       det(mone,mone)

!     coor      i   array of length npoint containing the coordinates
!                   of the nodal points with respect to the global
!                   nummering
!                       x  = coor(1,i); y  = coor(2,i)
!                        i               i
!     d1        i   array of length mone x mone contaning the values of the
!                   derivatives of the basisfunctions in the x-direction
!     det       o   array of length mone x mone containing the transformation
!                   determinant
!     index1    i   array of length inpelm containing the point numbers
!                   of the nodal points in the element
!     m         i   number of integration points in the element
!     mone      i   number of intregation points in one dimension
!     x         o   array containing the coordinates of the integration
!                   points in the element with respect to the local
!                   nummering, length m x 2
!                         x(ij,k) = coor(k,index1(ij))
!                         where ij = i + (j-1)*k
!***********************************************************************
!
!                       COMMON BLOCKS
!
!***********************************************************************
!
!                      LOCAL PARAMETERS
!
      integer i, j, k, ij, kj, ik
      double precision h1, h2, h3, h4, h5, h6, swap, e, f, g

!     e          help parameter to keep product of the derivatives
!     f          help parameter to keep product of the derivatives
!     g          help parameter to keep product of the derivatives
!     h1         variable to store a sum or product temporarily
!     h2         variable to store a sum or product temporarily
!     h3         variable to store a sum or product temporarily
!     h4         variable to store a sum or product temporarily
!     h5         variable to store a sum or product temporarily
!     h6         variable to store a sum or product temporarily
!     i          Counting variable
!     ij         help parameter
!     ik         help parameter
!     j          Counting variable
!     k          Counting variable
!     kj         help parameter
!     swap       swap parameter
!***********************************************************************
!
!                 SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  print error messages
!***********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!     1603   transformation determinant equal to zero
!***********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- store global integration points

      do 150 i = 1, mone

         do 100 j = 1, mone

            ij = i+(j-1)*mone
            x(ij,1) = coor(1, index1(ij))
            x(ij,2) = coor(2, index1(ij))
            x(ij,3) = coor(3, index1(ij))

100      continue
150   continue

      if (mone.eq.2) then

!     --- mone = 2, different nodal numbering

         swap = x(3,1)
         x(3,1) = x(4,1)
         x(4,1) = swap
         swap = x(3,2)
         x(3,2) = x(4,2)
         x(4,2) = swap
         swap = x(3,3)
         x(3,3) = x(4,3)
         x(4,3) = swap
      end if

!     --- compute transformation determinant
!         first compute partial derivatives

      do 350 i = 1, mone
         do 300 j = 1, mone

            h1 = 0d0
            h3 = 0d0
            h2 = 0d0
            h4 = 0d0
            h5 = 0d0
            h6 = 0d0
            do 200 k = 1, mone

               kj = k + (j-1)*mone
               h1 = h1 + x(kj,1)*d1(i,k)
               h3 = h3 + x(kj,2)*d1(i,k)
               h5 = h5 + x(kj,3)*d1(i,k)

               ik = i + (k-1)*mone
               h2 = h2 + x(ik,1)*d1(j,k)
               h4 = h4 + x(ik,2)*d1(j,k)
               h6 = h6 + x(ik,3)*d1(j,k)

200         continue

            e = h1*h1 + h3*h3 + h5*h5
            f = h1*h2 + h3*h4 + h5*h6
            g = h2*h2 + h4*h4 + h6*h6

            det(i,j) = sqrt( e*g - f*f )

!           --- check if determinant equal to zero

            if (det(i,j).lt.1d-10) then

               call eropen ('elp649')
               call errsub ( 1603, 0, 0, 0)
               call erclos ('elp649')
               go to 1000
            end if

300      continue
350   continue

1000  continue

      end
