      subroutine elpmaptang ( dphidx, qmat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-06-2013 correction debug
!        version  1.0  date 25-02-2009
!
!   copyright (c) 2009-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map three-dimensional array dphidx onto array dphids on surface
!     using mapping array qmat
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     basis_function
!     element
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision dphidx(n,m,*), qmat(3,3,m)

!     dphidx        i/o   Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         At output dphids i.e. the two components
!                         in tangential directions are stored
!     qmat           i    transformation matrix global -> local coordinates
!                         in case of a 3D element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k
      double precision dphids(m,2)

!     dphids         Contains the derivatives of the basis functions
!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elpmaptang' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elpmaptang'
         call prinrl1 ( dphidx(1,1,1), m, n, 'dphidx' )
         call prinrl1 ( dphidx(1,1,2), m, n, 'dphidy' )
         call prinrl1 ( dphidx(1,1,3), m, n, 'dphidz' )
         call prinrl1 ( qmat(1,1,1), m, 9, 'qmat' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- dphids = grad phi . s

      do i = 1, n
         do k = 1, m
            dphids(k,1) = dphidx(i,k,1) * qmat(2,1,k) + &
                          dphidx(i,k,2) * qmat(2,2,k) + &
                          dphidx(i,k,3) * qmat(2,3,k)
            dphids(k,2) = dphidx(i,k,1) * qmat(3,1,k) + &
                          dphidx(i,k,2) * qmat(3,2,k) + &
                          dphidx(i,k,3) * qmat(3,3,k)
         end do  ! k = 1, m
         dphidx(i,1:m,1) = dphids(1:m,1)
         dphidx(i,1:m,2) = dphids(1:m,2)
      end do  ! i = 1, n

1000  call erclos ( 'elpmaptang' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( dphidx(1,1,1), m, n, 'dphidx' )
         call prinrl1 ( dphidx(1,1,2), m, n, 'dphidy' )
         write(irefwr,*) 'End elpmaptang'

      end if  ! ( debug )

      end
