      subroutine elup03 ( s, dphidx, phi, psi, w, a1, a2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 04-07-2003 Extra parameters a1, a2
!        version  2.0  date 21-09-1993 Removal of dimension parameters
!        version  1.0  date 03-03-1993
!
!   copyright (c) 1993-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Add contribution of second derivative part to matrix in the case of
!   upwinding
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elliptic_equation
!     upwinding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision s(n,n), phi(n,m), dphidx(n,m,2), psi(n,m), w(m), &
                       a1(9,9), a2(9,9)

!     a1             Work array to store parts of the computation of the
!                    second derivative term
!     a2             Work array to store parts of the computation of the
!                    second derivative term
!     dphidx   i     Array of length n x m x 2 containing the values of the
!                    derivatives of the basis functions in the integration
!     node
!     phi      i     Array of length n x m containing the values of the basis
!                    functions in the integration points
!     psi      o     Array of length n x m containing the values of the modified
!     s       i/o    Array of length n x n containing the element
!                    matrix
!     w        i     Array of length m containing the weights for the
!                    numerical integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l
      double precision sum, sum1

!     i      Counting variable
!     j      Counting variable
!     k      Counting variable
!     l      Counting variable
!     sum    Parameter to compute a sum
!     sum1   Parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     - &
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        The contribution of the second derivative upwind terms is:
!        sum  d/dx  alpha   dphi / dx   (psi -phi )
!         kl      k      kl     j    l      i    i
!        In order to avoid the computation of second derivatives of basis
!        functions we apply the following approximation:
!          klj                             n    klj
!         T    = alpha   dphi / dx     ~  sum  T     phi
!                     kl     j    l    ~  m=1   m       m
!                   klj     n    klj
!        So d/dx   T    ~  sum  T     d phi / dx
!               k       ~  m=1   m         m    k
!     For i := 1 (1) n do
!        For j := 1 (1) n do
!            For l := 1 (1) n do
!               a1(l) := alpha11 dphi j / dx1 + alpha12 dphi j / dx2   (xl)
!               a2(l) := alpha12 dphi j / dx1 + alpha22 dphi j / dx2   (xl)
!            For k := 1 (1) m do
!               sec_der jk := sum a1(l) dphi l / dx1 + a2(l) dphi l /dx2  (xk)
!                              l
!               sij := sij - wk sec_der jk ( psi i (xk) - phi i (xk) )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i = 1, n

!     --- second derivative part

         do j = 1, n
            sum = 0d0
            do k = kstep, m, kstep
               sum1 = 0d0
               do l = 1, n
                  sum1 = sum1 + a1(l,j)*dphidx(l,k,1)+ &
                                a2(l,j)*dphidx(l,k,2)
               end do
               sum = sum - (psi(i,k)-phi(i,k)) * sum1 * w(k)
            end do
            s(j,i) = s(j,i) + sum
         end do
      end do
      end
