      subroutine sepel0152cf
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill complex coefficients for elements of type 152
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     coefficient
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPELALLOCCCF  Allocate complex coeffs and related arrays for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0152cf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0152cf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepelallocccf ( 0 ) ! allocate coefficient arrays (complex)

      do i = 1, E%ncoef
         E%ccoeffsnode(1:B%n,i) = E%ccoeffs(1,i)
      end do  ! i = 1, E%ncoef

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

1000  if ( debug ) then

!     --- Debug information

         call prinin ( E%ich, E%ncoef, 'ich' )
         call prinin ( E%ind, E%ncoef, 'ind' )
         write(irefwr,*) 'End sepel0152cf'

      end if  ! ( debug )
      call erclos ( 'sepel0152cf' )

      end subroutine sepel0152cf
