      subroutine sepel0300
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Performs initializations for subroutine sepelm300
!     Old subroutine el0300
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     incompressible
!     element_matrix
!     element_vector
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmoduleelem
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ncoef

!     i              Counting variable
!     ncoef          Size of coefficient arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPADAPRULE700 Adapt integration rule in case of elements for
!                    elliptic equations
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELFILLCOF   Fill all real coefficients in E%coeffs
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0300' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0300'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset

!     --- Get information of coefficients
!         First type of bearing and type of linearization

      E%typebear = sepelgetintcf ( codebear, nncodebear, ncodebear, &
                                   1, 'type_of_bearing' )
      E%typelinear = sepelgetintcf ( codelinear, nncodelinear, &
                                     ncodelinear, 1, codegen(10) )
      if ( E%iuser(E%ipos-1)>100100000 ) E%typelinear = E%typelinear-1

!     --- Next quadrature rule and coordinate system

      call sepgetrule
      call sepgetcoorsys

      E%ipos = E%ipos+1       ! skip one position
      E%ishift = E%ishift+1

      if ( debug ) &
         write(irefwr,1) 'typebear, typelinear, jcart, irule', &
                          E%typebear, E%typelinear, B%cart, B%rule

!     --- Adapt integration rule for specific cases

      call sepadaprule700

!     --- Get information of basis functions

      call sepelbasefnch

!     --- Fill all real coefficients

      E%ncoef = 15
      E%indh = 1
      E%indvisc = 2
      E%indsqueeze = 3
      E%indconst = 4
      E%indpress = 5
      E%indvel = 6
      E%indrho = 9
      E%indrt = 10
      E%indsource = 11
      E%induppvel = 14
      E%indgamma = 16 ! parameter to store gamma to be used in sepelvecgamma
      E%inddiff = 19  ! parameter to store h^3/(12 mu)
      E%iqmat = 2     ! transformation parameter
      B%fillqmat = .true.  ! fill transformation matrix
      ncoef = 19      ! extended in order to h^3/(12 mu) as well as
                      ! gamma as coefficients
      n = B%n
      m = B%m
      ncoef = 19

      allocate ( E%coeffs(m,ncoef), E%ind(ncoef), E%ist(ncoef), &
                 E%mst(ncoef), E%ivecnums(ncoef), E%ich(ncoef), &
                 E%coeffsnode(n,ncoef), stat = error )
      if ( error/=0 ) call eralloc ( error, m*ncoef, 'coeffs' )
      if ( ierror/=0 ) go to 1000

      E%ich = 0  ! clear ich
      E%ind = 0  ! clear ind
      E%coeffs = 0d0

!     --- Trick to use sepelvecgamma in sepelbearrhs

      E%gamma => E%coeffs(:,E%indgamma:E%indgamma+1)
      if ( E%ndim==1 ) then
         E%ind(E%indgamma:E%indgamma) = 1
      else
         E%ind(E%indgamma:E%indgamma+1) = 1
      end if  ! ( E%ndim==1 )

      call sepelfillcof ( 1 )

      do i = 1, E%ncoef
         E%coeffsnode(1:B%n,i) = E%coeffs(1,i)
      end do  ! i = 1, E%ncoef
      if ( E%ind(E%indrho)==0 ) E%coeffs(:,E%indrho) = 1d0

      if ( E%typebear==1 ) then

!     --- compressible bearing, use previous solution

         indvector(iseqin) = 1
         E%compold = 1
         if ( E%typelinear==1 ) E%filloldderiv = .true.  ! Newton linearization

      end if  ! ( E%typebear==1 )

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'E%compold', E%compold
         write(irefwr,*) 'End sepel0300'

      end if  ! ( debug )
      call erclos ( 'sepel0300' )

      end subroutine sepel0300
