      subroutine sepelallocvecs
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Allocate locvecs in oldsols for all vectors that are used
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulefunc
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k

!     k              general loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelallocvecs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelallocvecs'
         call prinin ( indvector, numoldvecs, 'indvector' )
         write(irefwr,1) 'iseqprevsol, n', iseqprevsol, B%n
         write(irefwr,*) 'filloldderiv ', E%filloldderiv
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do k = 1, numoldvecs

         if ( indvector(k)>0 ) then

!        --- indvector(k)>0, hence vector k must be filled

            if ( debug ) &
               write(irefwr,1) 'allocate: k, m, maxunk', &
                                k, B%m, E%maxunk
            allocate ( oldsols(k)%locvecs(B%m,E%maxunk), &
                       oldsols(k)%locvecsnode(B%n,E%maxunk), &
                       oldsols(k)%loccvecs(B%m,E%maxunk), &
                       oldsols(k)%loccvecsnode(B%n,E%maxunk), &
                       stat = error )
            if ( error/=0 ) call eralloc ( error, (B%m+B%n)*E%maxunk, 'locvecs' )
            if ( ierror/=0 ) go to 1000

            if ( k==iseqprevsol .and. &
                 (E%filloldderiv .or. E%filloldvec) ) then

!           --- The derivatives of the old vector are also used

               allocate ( gradoldvec(B%m,ndim,nunkp), &
                          gradoldcvec(B%m,ndim,nunkp), &
                          stat = error )
               if ( error/=0 ) &
                  call eralloc ( error, B%m*nunkp*ndim, 'gradoldvec' )
               if ( ierror/=0 ) go to 1000

!              --- quantities for module sepmodulefunc

               gradprev => gradoldvec(:,:,1)
               prevsolution => oldsols(k)%locvecs(:,1)

            end if  ! ( k==iseqprevsol .and. E%filloldderiv )

         else

!        --- indvector(k)=0, hence vector k must not be filled

            nullify ( oldsols(k)%locvecs, oldsols(k)%loccvecs )

         end if  ! ( indvector(k)>0 )

      end do  ! k = 1, numoldvecs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelallocvecs'

      end if  ! ( debug )
      call erclos ( 'sepelallocvecs' )

      end subroutine sepelallocvecs
