      subroutine sepelbaseupw
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the modified basis functions psi due to upwind
!     See old subroutine elm800upwbase
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     upwinding
!     basis_function
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPELPUPW01    Compute modified basis functions for upwind elements
!                    One-dimensional elements
!     SEPELPUPW01TIM Compute modified basis functions psi in case of upwind of
!                    type 12 (1d)
!     SEPELPUPW03    Compute modified basis functions psi in case of upwind
!                    Two dimensional elements (triangles)
!     SEPELPUPW03TIM Compute modified basis functions psi in case of upwind of
!                    type 12. Two dimensional elements (triangles)
!     SEPELPUPW03NL  Compute modified basis functions for upwind elements
!                    Two dimensional elements (triangles)
!                    Non-linear method
!     SEPELPUPW05    Compute modified basis functions psi in case of upwind
!                    Two dimensional elements (quadrilaterals)
!     SEPELPUPW11    Compute modified basis functions psi in case of upwind
!                    Three dimensional elements (tetrahedrons)
!     SEPELPUPW11TIM Compute modified basis functions psi in case of upwind of
!                    type 12. Three dimensional elements (tetrahedrons)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
!    3189   option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelbaseupw' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelbaseupw'
         write(irefwr,1) 'n, m, type_basisfunc, methodupw', &
                          B%n, B%m, B%type_basisfunc, E%methodupw
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%first==0 ) then

!     --- First call: allocate basis functions and related quantities

         allocate ( B%psi(B%n,B%m), stat = error )
         if ( error/=0 ) call eralloc ( error, B%m*B%n, 'psi' )

      end if  ! ( E%first==0 )

      B%secondupw = .false.  ! second order upwind term is not relevant

      select case (B%type_basisfunc)

      case(1, 2)

!     --- case 1: one-dimensional elements

         if ( E%methodupw==12 ) then

!        --- linear elements, methodupw = 12

            call sepelpupw01tim

         else

!        --- methodupw # 12

            call sepelpupw01

         end if  ! ( E%methodupw==12 )

      case(3)

         if ( E%methodupw==12 ) then

!        --- linear elements, methodupw = 12

            call sepelpupw03tim

         else if ( E%methodupw>=9 .and. E%methodupw<=11 ) then

!        --- methodupw = 9, 10, 11: not yet implemented

            call sepelpupw03nl

         else

!        --- methodupw = 1 to 8

            call sepelpupw03

         end if  ! ( E%methodupw==12 )

      case(4)

         call sepelpupw03

      case(5, 6, 9, 10)

         call sepelpupw05

      case(11:14, 21:22)

         if ( E%methodupw==12 ) then

!        --- linear elements, methodupw = 12

            call sepelpupw11tim

         else

!        --- methodupw = 1 to 8

            call sepelpupw11

         end if  ! ( E%methodupw==12 )

      case default

!        --- Shape number not available

         call errint ( B%type_basisfunc, 1 )
         call errsub ( 800, 1, 0, 0 )

      end select  ! case (B%type_basisfunc)

1000  call erclos ( 'sepelbaseupw' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%psi, B%m, B%n,'psi' )
         write(irefwr,*) 'End sepelbaseupw'

      end if  ! ( debug )

      end subroutine sepelbaseupw
