      subroutine sepelbearcons
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute constant part of matrix for Reynolds equation
!     Old subroutine elbearcons
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     incompressible
!     element_matrix
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: coeff(B%m)
      double precision, pointer :: cons(:), pold(:)

!     coeff          Help array to store the computed coefficient
!     cons           Contains the variable k in the equations
!     pold           Contains the pressure in the prior iteration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPELMASSMAT1  Fill the element mass matrix; one unknown per node
!                    Consistent mass matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     This subroutine computes the part
!
!     / coeff phi_i phi_j d Omega
!
!     where coeff depends on the coefficients, type of bearing and
!     linearization
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelbearcons' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelbearcons'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      cons => E%coeffs(:,E%indconst)
      pold => E%ugauss(:,1)

      if ( E%typebear==0 ) then

!     --- incompressible bearing

         coeff = cons

      else if ( E%typelinear==1 ) then

!     --- compressible bearing, Newton linearization

         coeff = 2d0*cons*pold

      else

!     --- compressible bearing, Picard linearization

         coeff = cons*pold

      end if
      E%submat => E%mat
      call sepelmassmat1 ( coeff )

1000  call erclos ( 'sepelbearcons' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%mat, B%n, B%n, 'elemmat')
         write(irefwr,*) 'End sepelbearcons'

      end if  ! ( debug )

      end subroutine sepelbearcons
