      subroutine sepelcdiffus
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for diffusion equation
!     It is assumed that diffusion coefficients are stored in coefficients
!     1 to 6
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINCOMPL1     Print 2d complex vector
!     SEPELCDIFFDIAG Fill complex element matrix for diffusion equation
!                    The diffusion is a diagonal tensor
!     SEPELCDIFFTENS Fill complex element matrix for diffusion equation
!                    The diffusion is a tensor
!     SEPELCLAPLACE  Compute the complex element matrix for the Laplace equation
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelcdiffus' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelcdiffus'
         write(irefwr,1) 'E%typediff', E%typediff
         write(irefwr,*) 'B%fillqmat ', B%fillqmat
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%fillqmat ) E%typediff = 3

      select case(E%typediff)

      case(1)

!     --- Diffusion is scalar

         call sepelclaplace ( E%ccoeffs(:,1) )

      case(2)

!     --- Diffusion is diagonal tensor

         call sepelcdiffdiag ( E%ccoeffs )

      case(3)

!     --- Diffusion is tensor

         call sepelcdifftens ( E%ccoeffs )

      end select  ! case(E%typediff)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelcdiffus'
         call princompl1 ( E%cmat, B%n, B%n, 'element_matrix/diffusion' )

      end if  ! ( debug )
      call erclos ( 'sepelcdiffus' )

      end subroutine sepelcdiffus
