      subroutine sepeld08
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives in E%vec
!     with respect to subroutine sepeld705
!     icheld = 8: smoother for to0090
!     Old subroutine eld118/eld100
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
!     SEPELD081      Fill smoother for to0090 (ndim=1)
!     SEPELD083      Fill smoother for to0090 (shape 3)
!     SEPELD084      Fill smoother for to0090 (shape 4)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     406   Value of icheld not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld08' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld08'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( E%ishape )

      case(1, 2)
         call sepeld081
      case(3)
         call sepeld083 ( E%wvec, E%vec )
      case(4)
         call sepeld084 ( E%wvec, E%vec )
      case default
         call errint ( icheld, 1 )
         call errint ( E%itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )
      end select  ! case ( B%ishape )

1000  call erclos ( 'sepeld08' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%vec, n, ndim, 'elemvc' )
         call prinrl1 ( E%wvec, n, ndim, 'elemwg' )
         write(irefwr,*) 'End sepeld08'

      end if  ! ( debug )

      end subroutine sepeld08
