      subroutine sepeld32 ( elemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives in elemvc
!     with respect to subroutine
!     icheld = 32, 33, 35: gradient no weights
!     Old subroutine el100132
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: elemvc(ndimlc,B%n)

!     elemvc         o    Element vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: h
      double precision, pointer :: dudx(:,:), dphidx(:,:,:)

!     dphidx         Reference to array B%phider
!                    (derivatives of basis functions in nodes)
!     dudx           Reference to array gradoldvec
!                    (derivatives of solution in nodes)
!     h        Help variable to store a constant temporarily
!     i        General loop variable
!     j        General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

      dudx => gradoldvec(:,:,1)
      dphidx => B%phider(:,:,:)
      n = B%n

      if ( B%derivconst ) then

!     --- Derivatives are constant

         do j = 1, ndimlc
            do i = 1, n
               elemvc(j,i) = dudx(1,j)
            end do
         end do

      else if ( ndim==2 .and. n>=6 ) then

!     --- quadratic elements in R2

         do j = 1, ndimlc
            do i = 1, n-1, 2
               elemvc(j,i) = dudx(i,j)
            end do
            do i = 2, n, 2
               elemvc(j,i) = dudx(i,j)
            end do
            elemvc(j,n) = dudx(n,j)
         end do

      else

!     --- General element

         do j = 1, ndimlc
            do i = 1, n
               elemvc(j,i) = dudx(i,j)
            end do
         end do

      end if

      if ( icheld==33 ) then

!     --- icheld = 33,  change sign

         do j = 1, ndimlc
            do i = 1, n
               elemvc(j,i) = - elemvc(j,i)
            end do
         end do

      else if ( icheld==35 ) then

!     --- icheld = 35, change sign and sequence  (2D only)

         do i = 1, n
            h = elemvc(1,i)
            elemvc(1,i) = elemvc(2,i)
            elemvc(2,i) = - h
         end do

      end if

      end
