      subroutine sepeld807_45
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine whether an element should be refined (ielvec = 1) or not
!     (ielvec = 0). An element will NOT be refined only when the enthalpy
!     values (in the integration points) are all larger than rho*L or all
!     less than zero.
!     Old subroutine eldetrefelem
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     enthalpy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleelm
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, neg
      double precision :: Tm
      double precision, pointer :: temperature(:)


!     Tm             melting temperature
!     i              Counting variable
!     neg            Counting variable
!     temperature    temperature vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld807_45' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld807_45'
         write(irefwr,1) 'ielem, iseqtemp', ielem, iseqtemp
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      Tm = E%coeffs(1,8)

!     --- Fill temperature

      temperature =>  oldsols(iseqtemp)%locvecs(1:m,1)

!        --- Count the number of temperature values less than Tm

      neg = 0

      do i = 1, m
         if ( temperature(i)<Tm ) then
            neg = neg + 1
         end if
      end do

!     --- If at least one of the nodes has 'opposite' sign, mlocark
!         it to be refined

      if ( neg==m .or. neg==0 ) then
         E%ielvec(1) = 0
      else
         E%ielvec(1) = 1
      end if

1000  call erclos ( 'sepeld807_45' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'itype', itype
         write(irefwr,2) 'Tm', Tm
         call prinrl ( temperature, m, 'temperature' )
         call prinin ( E%ielvec, 1, 'E%ielvec' )
         write(irefwr,*) 'End sepeld807_45'

      end if

      end
