      subroutine sepeldc061 ( elemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives and weights in elemvc and elemwg
!     with respect to subroutine sepeld705
!     icheld = 6, 16, 26  Compute - A gradient
!     ndim = 1
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex :: elemvc(ndimlc,B%n)

!     elemvc         o    Element vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double complex, pointer :: dudx(:), a11(:)

!     a11            Diffusion coeffecient a_11
!                    they are not
!     dudx           Reference to array gradoldvec
!                    (derivatives of solution in nodes)
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The formula to be computed is:
!     g1 = - a11 ux
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldc061' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldc061'
         write(irefwr,*) 'derivconst ', B%derivconst

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dudx => gradoldcvec(:,1,1)
      a11 => E%ccoeffs(:,1)

      if ( B%derivconst ) then

!     --- Derivatives are constant

         if ( icheld==6 .or. icheld==36 ) then
            do i = 1, n
               elemvc(1,i) = -dudx(1) * a11(i)
            end do
         else
            elemvc(1,1) = -dudx(1)*sum(a11(:))/n
         end if

      else

!     --- General case

         if ( icheld==6 .or. icheld==36 ) then
            elemvc(1,:) = -dudx(:) * a11(:)
         else
            elemvc(1,1) = -sum(dudx(:)*a11(:))/n
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldc061'

      end if  ! ( debug )
      call erclos ( 'sepeldc061' )

      end subroutine sepeldc061
