      subroutine sepeldiffusion
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for diffusion equation
!     It is assumed that diffusion coefficients are stored in coefficients
!     1 to 6
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPELDIFFUPW   Compute part of upwind discretization related to diffusion
!     SEPELDIFFUS1   Fill subelement matrix for diffusion equation
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldiffusion' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldiffusion'
         write(irefwr,1) 'E%typediff', E%typediff
         write(irefwr,*) 'B%fillqmat ', B%fillqmat
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%fillqmat ) E%typediff = 3

      call sepeldiffus1 ( E%coeffs, E%ind, E%typediff )

      if ( B%fillpsi .and. E%methodupw>0 .and. E%applyupw<1 ) then

!     --- Upwind must be applied to diffusion term

         call sepeldiffupw

      end if  ! ( E%methodupw>0 .and. E%applyupw<1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldiffusion'
         call prinrl1 ( E%mat, B%n, B%n, 'element_matrix/diffusion' )

      end if  ! ( debug )
      call erclos ( 'sepeldiffusion' )

      end subroutine sepeldiffusion
