      subroutine sepelfillcvar
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill space dependent coefficients if necessary
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     fill
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecomio
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      logical :: fillvecloc
      double complex :: vecloc(1:numoldvecs,1:E%maxunk,1:B%m)

!     fillvecloc     if true vecloc has been filled
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     vecloc         Work array in which all old solution vectors for the
!                    quadrature points are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
!     SEPELFILLCVAR1 Fill one space dependent complex coefficient
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelfillcvar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelfillcvar'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      fillvecloc = .false.  ! vecloc has not been filled

!     --- Loop over all possible space dependent coefficients

      do i = E%isubcfvar, E%isupcfvar

         if ( debug ) &
            write(irefwr,1) 'i, E%ind(i), E%ist(i)', &
                             i, E%ind(i), E%ist(i)

         if ( E%ind(i)>0 .or. E%ist(i)>0 ) then

!        --- Coefficient is variable

            if ( E%ind(i)>10000 .and. .not. fillvecloc ) then

!           --- vecloc has not been filled

               do k = 1, numoldvecs
                  if ( indvector(k)==1 ) then
                     do j = 1, E%maxunk
                        vecloc(k,j,1:B%m) = oldsols(k)%loccvecs(1:B%m,j)
                     end do  ! j = 1, 1:E%maxunk
                  end if  ! ( indvector(k)==1 )
               end do  ! k = 1, numoldvecs
               fillvecloc = .true.

            end if  ! ( E%ind(i)>10000 .and. .not. fillvecloc )

            call sepelfillcvar1 ( i, vecloc )

         end if  ! ( E%ind(i)>0 .or. E%ist(i)>0 )

      end do  ! i = E%isubcfvar, E%isupcfvar

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( B%xyz, E%npelm, E%ndim, 'x' )
         call prinrl2 ( B%xyzg, B%m, E%ndim, 'xg' )
         call prinrl1 ( E%coeffs, E%isupcfvar, B%m, 'coefficients' )
         write(irefwr,*) 'End sepelfillcvar'

      end if  ! ( debug )
      call erclos ( 'sepelfillcvar' )

      end subroutine sepelfillcvar
