      subroutine sepeli0705
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform initialization for subroutine sepeli705
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_integral
!     integral
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      use sepmoduleelm
      use sepmoduleelem
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPADAPRULE700 Adapt integration rule in case of elements for
!                    elliptic equations
!     SEPEL0705CF    Fill real coefficients for elements of type 705
!     SEPELALLOCVCOL Allocate locvecs in oldsols for one vector
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeli0705' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeli0705'
         write(irefwr,1) 'icheli', icheli
         write(irefwr,1) 'iseqin, iseqi1, iseqi2', iseqin, iseqi1, iseqi2
         write(irefwr,1) 'iseqprevsol', iseqprevsol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Default values

      B%cart = 1
      B%rule = 1
      E%methodupw = 0
      E%applyupw = 0
      E%compold = 0
      E%filloldderiv = .false.
      E%ncoef = 1

      if ( icheli==7 ) then
         E%ncoef = 0  ! No coefficients required
         B%fillderiv = .false.
      else
         nunkp = oldsols(iseqprevsol)%nunkp
         indvector = 0
      end if  ! ( icheli==7 )
      if ( icheli>=3 .and. icheli<=6 ) E%filloldderiv = .true.

      if ( E%ncoef>0 ) then

!     --- set parameters for coeffs

         call sepeliposset
         if ( debug ) write(irefwr,1) 'E%ipos', E%ipos

!        --- Next quadrature rule and coordinate system

         call sepgetrule

         call sepgetcoorsys
         E%ipos = E%ipos+1  ! skip one position
         if ( debug ) write(irefwr,1) 'E%ipos/2', E%ipos

      else

!     --- No coefficients

         E%isupcfvar = -1
         E%isubcfvar = 1

      end if  ! ( E%ncoef>0 )

!     --- Adapt integration rule for specific cases

      call sepadaprule700

!     --- Get information of basis functions

      call sepelbasefnch

      if ( E%ncoef>0 ) then

!     --- Number of coefficients > 0

         call sepel0705cf  ! fill coefficients

         if ( debug ) then
            write(irefwr,1) 'E%ipos/3', E%ipos
            call prinin ( E%ind, E%ncoef, 'ind' )
         end if  ! ( debug )

         if ( E%ind(1)==0 ) then

!        --- coefficient = 0, set to 1

            E%ind(1) = -1
            E%coeffs(:,1) = 1d0

         end if  ! ( E%ind(1)==0 )

      end if  ! ( E%ncoef>0 )n

      if ( E%compold==0 ) then
         if ( icheli>=2 .and. icheli<=6 ) E%compold = 1
      end if  ! ( E%compold==0 )
      if ( icheli/=7 ) call sepelallocvcol! allocate oldsols%locvecs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icheli, ncoef', icheli, E%ncoef
         write(irefwr,1) 'irule, cart', B%rule, B%cart
         write(irefwr,*) 'filloldderiv ', E%filloldderiv
         write(irefwr,*) 'End sepeli0705'

      end if  ! ( debug )
      call erclos ( 'sepeli0705' )

      end subroutine sepeli0705
