      subroutine sepelm150cf
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coefficients for complex general elliptic equation
!     One unknown per point
!     Type 150
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINCOMPL      Print 1d complex vector
!     PRINCOMPL1     Print 2d complex vector
!     SEPELFILLCVAR  Fill complex space dependent coefficients
!     SEPELMULTCRHO  Multiply complex velocity by rho cp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm150cf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm150cf'
         write(irefwr,1) 'E%isubcfvar, E%isupcfvar', E%isubcfvar, E%isupcfvar
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%isupcfvar>=E%isubcfvar ) then

!     --- Fill variable coefficients if necessary

         call sepelfillcvar

!        --- Compute rho cp if necessary

         if ( E%ind(E%indrho)/=0 .or. E%ind(E%indcp)/=0 ) then
            E%ccoeffs(:,E%indrhocp) = E%ccoeffs(:,E%indrho)*E%ccoeffs(:,E%indcp)
            E%ccoeffsnode(:,E%indrhocp) = &
               E%ccoeffsnode(:,E%indrho)*E%ccoeffsnode(:,E%indcp)
            E%coeffs(:,E%indrho) = real(E%ccoeffs(:,E%indrho),kind=8)
            E%coeffs(:,E%indrhocp) = real(E%ccoeffs(:,E%indrhocp),kind=8)
         end if  ! ( E%ind(E%indrho)/=0 .or. E%ind(E%indcp)/=0 )

         if ( debug ) call princompl ( E%ccoeffs(:,E%indsource), B%m, 'source' )

      end if  ! ( E%isupcfvar>=E%isubcfvar )

      if ( debug ) call princompl1 ( E%ccoeffs, E%ncoef, B%m, 'coeffs' )

      if ( E%ncoef>6 ) then

!     --- More than 6 coefficients and not bearing
!         Multiply velocity by rho cp if necessary

         if ( E%ind(E%indrhocp)/=0 ) call sepelmultcrho

      end if  ! ( E%ncoef>6 .and. itype/=300 .and. itype/=325 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelm150cf'

      end if  ! ( debug )
      call erclos ( 'sepelm150cf' )

      end subroutine sepelm150cf
