      subroutine sepelmaselas
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the element mass matrix for linear elasticity problems
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     mass_matrix
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: diagmass(:), rho(:)
      integer :: i

!     diagmass       Contains the element matrix in case of a diagonal mass
!                    matrix
!     i              Counting variable
!     rho            density rho cp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELMASSMAT2  Fill element mass matrix for linear elastic problem
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmaselas' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmaselas'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      rho => E%coeffs(:,E%indrho)

      if ( E%imas==1 ) then

!     --- imas = 1, diagonal mass matrix

         diagmass => E%mmat(1:E%ndim*n:E%ndim,1)
         diagmass = rho*B%w
         do i = 2, E%ndim
            E%mmat(i:E%ndim*n:E%ndim,1) = diagmass
         end do  ! i = 2, E%ndim

      else if ( E%imas==2 ) then

!     --- imas = 2, non-diagonal mass matrix

         E%mmat = 0d0
         call sepelmassmat2 ( rho, E%mmat )

      end if  ! ( E%imas==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmaselas'

      end if  ! ( debug )
      call erclos ( 'sepelmaselas' )

      end subroutine sepelmaselas
