      subroutine sepelmlinelas
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for linear elasticity problem
!     Old subroutine el8102
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: s(:,:)
      double precision :: dw(E%dimD,E%dimD,B%m)
      integer :: i, j

!     dw             Dmat * w
!     i              Counting variable
!     j              Counting variable
!     s              Sub element matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPELMLINELAS1 Fill part of element mass matrix for linear elastic problem
!     SEPELMLINELAS2 Fill part of element mass matrix for linear elastic problem
!     SEPELMLINELAS3D Fill element matrix for linear elasticity problem(3d)
!     SEPFILLBMAT    Fill B-matrix for linear elasticity problem
!     SEPFILLDMAT    Fill D-matrix for linear elasticity problem
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmlinelas' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmlinelas'
         write(irefwr,*) 'dmatconst', E%dmatconst
         call prinrl ( B%w, B%m, 'w' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill D matrix if not constant

      if ( .not. E%dmatconst ) call sepfilldmat

!     --- Multiply D by w and store in dw

      do i = 1, E%dimD
         do j = 1, E%dimD
            Dw(i,j,:) = B%w(:)*E%Dmat(i,j,:)
         end do  ! j = 1, E%dimD
      end do  ! i = 1, E%dimD

      select case ( E%ndim )

         case(2)

!        --- ndim = 2, fill Bmat

            call sepfillbmat

!           --- Fill submatrices S_11, S_12 and S_22

            do i = 1, 2
               call sepelmlinelas1 ( i, Dw )
            end do  ! i = 1, 2
            call sepelmlinelas2 ( 2, 1, Dw )

         case(3)

!        --- ndim = 3

            call sepelmlinelas3d ( Dw )

      end select  ! case ( E%ndim )

!     --- Store lower part

      s => E%mat
      do i = 1, E%icount
         do j = 1, i-1
            s(j,i) = s(i,j)
         end do  ! j = 1, i-1
      end do  ! i = 1, E%icount

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmlinelas'

      end if  ! ( debug )
      call erclos ( 'sepelmlinelas' )

      end subroutine sepelmlinelas
