      subroutine sepelmlinelas2 ( l1, l2, Dw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill part of element mass matrix for linear elastic problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: Dw(E%dimD,E%dimD,B%m)
      integer :: l1, l2

!     Dw             i    Dmat * w
!     l1             i    sequence number of sub matrix (l1,l2)
!     l2             i    sequence number of sub matrix (l1,l2)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: S(B%n,B%n), h, som
      integer :: ip, iq, i, nd, j

!     S               Sub element matrix to be filled
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     ip             loop variable
!     iq             loop variable
!     j              Counting variable
!     nd             Dimension of D-matrix
!     som            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmlinelas2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmlinelas2'
         write(irefwr,1) 'l1, l2', l1, l2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nd = E%dimD

      if ( B%derivconst ) then

!     --- Derivatives of basis functions are constant

         do i = 1, n
            do j = 1, n
               som = 0d0
               do ip = 1, nd
                  do iq = 1, nd
                     h = sum(Dw(ip,iq,:))
                     som = som+h*E%Bmat(ip,l1,i,1)*E%Bmat(iq,l2,j,1)
                  end do  ! iq = 1, nd
               end do  ! ip = 1, nd
               s(i,j) = som
            end do  ! j = 1, i
         end do  ! i = 1, n

      else

!     --- Derivatives of basis functions are not constant

         do i = 1, n
            do j = 1, n
               som = 0d0
               do ip = 1, nd
                  do iq = 1, nd
                     som = som+sum(E%Bmat(ip,l1,i,:)*Dw(ip,iq,:)*&
                                   E%Bmat(iq,l2,j,:))
                  end do  ! iq = 1, nd
               end do  ! ip = 1, nd
               s(i,j) = som
            end do  ! j = 1, i
         end do  ! i = 1, n

      end if  ! ( B%derivconst )

      nd = E%ndim

      do i = 1, n
         do j = 1, i-1
            E%mat(l1+(i-1)*nd,l2+(j-1)*nd) = S(i,j)
            E%mat(l2+(i-1)*nd,l1+(j-1)*nd) = S(j,i)
         end do  ! j = 1, i-1
         E%mat(l1+(i-1)*nd,l2+(i-1)*nd) = S(i,i)
      end do  ! i = 1, n

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( s, n, n, 's')
         write(irefwr,*) 'End sepelmlinelas2'

      end if  ! ( debug )
      call erclos ( 'sepelmlinelas2' )

      end subroutine sepelmlinelas2
