      subroutine sepeloldcvecs2 ( ndegfd, vecloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array vecloc in case of vectors of type 129
!     ndegfd is set to 0
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     coefficient
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: ndegfd
      double complex, intent(out) :: vecloc(m,*)

!     ndegfd         o    Number of degrees of freedom per point
!     vecloc         o    Local vector defined per element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, k

!     j              General loop variable (over degrees of freedom)
!     k              General loop variable (over integration points)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeloldcvecs2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeloldcvecs2'
         call prinin ( index4n, 1, 'index4n' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndegfd = index4n(1)
      if ( debug ) call prinin ( index3n, ndegfd*m, 'index3n' )
      if ( m==index4n(2) ) then

!     --- Standard case integration points available

         do j = 1, ndegfd
            do k = 1, m
               vecloc(k,j) = cvecold(index3n((k-1)*ndegfd+j))
            end do  ! k = 1, m
         end do  ! j = 1, ndegfd

      else

!     --- Boundary elements, no integration points available

         do j = 1, E%maxunk
            vecloc(1:m,j) = 0d0
         end do  ! j = 1, E%maxunk

      end if  ! ( m==index4n(2) )
      ndegfd = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeloldcvecs2'

      end if  ! ( debug )
      call erclos ( 'sepeloldcvecs2' )

      end subroutine sepeloldcvecs2
