      subroutine sepelp01first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-06-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     First order 1d element (2 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: lambda(B%m)
      character (len=22), parameter :: text = 'Linear 1D line element'

!     lambda         Contains values of lambda_1 in quadrature points
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp01first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp01first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case (B%rule)

      case (1)

!     --- Newton Cotes rule

         if ( B%m/=2 ) call errorelvar2 ( B%m, B%rule, 1424, text, 0 )
         if ( ierror/=0 ) go to 1000
         lambda(1) = 0d0
         lambda(2) = 1d0
         B%wc = 0.5d0

         B%jdiag = 1   ! basis functions satisfy phi_i(x_j) = delta_ij

      case (2:5)

!     --- Gauss rules

         call sepelplambda1d ( lambda, text, B%rule, B%m, B%wc )

      case default

!     --- Quadrature not available

         call errint ( B%rule, 1 )
         call errsub ( 1497, 1, 0, 0)
         go to 1000

      end select  ! case (B%rule)

      B%lam(1,:) = 1d0-lambda(:)
      B%lam(2,:) = lambda(:)
      B%derivconst = .true.  ! derivatives of basis functions constant

!     --- Fill basis functions in quadrature points

      B%phic(1,:) = B%lam(1,:)
      B%phic(2,:) = B%lam(2,:)

      B%phi(1,:) = B%lam(1,:)
      B%phi(2,:) = B%lam(2,:)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp01first'

      end if  ! ( debug )
      call erclos ( 'sepelp01first' )

      end subroutine sepelp01first
