      subroutine sepelp11first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Linear tetrahedron (4 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: lambda(4,B%m)
      character (len=18), parameter :: text = 'Linear tetrahedron'
      integer :: k

!     k              Loop variable
!     lambda         Contains values of lambda in quadrature points
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     SEPELPLAMBDA3D Fill the three-dimensional linear basis functions in case
!                    of a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1424   m does not correspond to irule
!    1497   Type of integration rule not yet available
!    1769   Type of co-ordinate system incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp11first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp11first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%cart>1 ) call errorelvar ( B%cart-1, 1769, text, 0 )

      select case (B%rule)

      case (1)

!     --- Newton Cotes rule

         if ( B%m/=4 ) call errorelvar2 ( B%m, B%rule, 1424, text, 0 )
         if ( ierror/=0 ) go to 1000
         lambda = 0d0
         do k = 1, 4
            lambda(k,k) = 1d0
         end do  ! k = 1, 4
         B%wc = 1d0/24d0

         B%jdiag = 1   ! basis functions satisfy phi_i(x_j) = delta_ij

      case (2:6)

!     --- Gauss rules

         call sepelplambda3d ( lambda, text )

      case default

!     --- Quadrature not available

         call errint ( B%rule, 1 )
         call errsub ( 1497, 1, 0, 0)
         go to 1000

      end select  ! case (B%rule)

      B%derivconst = .true.  ! derivatives of basis functions constant

!     --- Fill basis functions in quadrature points

      B%lam = lambda
      B%phic = lambda
      B%phi = lambda
      B%lamderc = 0d0
      B%lamderc(1,:,:) = -1d0
      B%lamderc(2,:,1) = 1d0
      B%lamderc(3,:,2) = 1d0
      B%lamderc(4,:,3) = 1d0
      B%lamder => B%phider

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp11first'

      end if  ! ( debug )
      call erclos ( 'sepelp11first' )

      end subroutine sepelp11first
