      subroutine sepelp14
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     3d Tri-Quadratic hexahedron (27 nodes)
!     Old subroutine elp633
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     hexahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: jac(B%m)
      integer :: i

!     jac            Array of length m containing the values of the
!                    Jacobian in the vertices or for the Gauss integration
!                    rules work array to store some help variables
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELP13JACOB  Compute Jacobian and derivatives of basis functions
!                    3d Tri-linear hexahedron (8 nodes)
!     SEPELP14FIRST  compute some element independent quantities
!                    3d Tri-quadratic hexahedron (27 nodes)
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp14' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp14'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%first==0 ) call sepelp14first

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp13jacob ( jac )

!     --- Fill weights

      B%w = B%wc*abs(jac(1:B%m))

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w contains the weights
!         w2 the diagonal of the mass matrix

         do i = 1, B%n
            B%w2(i) = sum(B%w*B%phi(i,:)**2)
         end do  ! i = 1, B%n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  call erclos ( 'sepelp14' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( B%w, B%m, 'weight' )
         call prinrl1 ( B%phider(1,1,1), B%m, B%n, 'dphi/dx' )
         call prinrl1 ( B%phider(1,1,2), B%m, B%n, 'dphi/dy' )
         call prinrl1 ( B%phider(1,1,3), B%m, B%n, 'dphi/dz' )

         write(irefwr,*) 'End sepelp14'

      end if  ! ( debug )

      end subroutine sepelp14
