      subroutine sepelp30first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-06-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Third order 1d hermite element (2 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: lambda(B%m)
      character (len=25), parameter :: text = 'Linear 1D hermite element'

!     lambda         Contains values of lambda_1 in quadrature points
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp30first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp30first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case (B%rule)

      case (1)

!     --- 3 point Gauss rule

         call sepelplambda1d ( lambda, text, 4, B%m, B%wc )
         B%fillgauss = .true.

      case (2:3)

!     --- 4 or 5 point Gauss rule

         call sepelplambda1d ( lambda, text, 4+B%rule, B%m, B%wc )
         B%fillgauss = .true.

      case default

!     --- Quadrature not available

         call errint ( B%rule, 1 )
         call errsub ( 1497, 1, 0, 0)
         go to 1000

      end select  ! case (B%rule)

      B%lam(1,:) = 1d0-lambda(:)
      B%lam(2,:) = lambda(:)

      B%phic(2,:) = B%lam(1,:)**2*B%lam(2,:)
      B%phic(4,:) = B%lam(2,:)**2*B%lam(1,:)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp30first'

      end if  ! ( debug )
      call erclos ( 'sepelp30first' )

      end subroutine sepelp30first
