      subroutine sepelp3d2dder
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the derivatives dphi/dx for a 2d element in 3d
!     Old subroutine elp3dto2dderiv
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     2d
!     3d
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision :: dxidx(m,2,3)
      double precision, pointer :: dxdxi(:,:,:), jac(:), dphidx(:,:,:)
      double precision, pointer :: dphidxi(:,:,:)

!     dphidx         Contains values of dphi/dx in quadrature points
!     dphidxi        Contains values of dphi/dxi in quadrature points
!     dxdxi          Contains the derivatives d x/ dxi
!     dxidx          Contains the derivatives d xi/ dx
!                    dxidx(k,i,j) = dxi_i/dx_j (x^k)
!     i              Counting variable
!     j              Counting variable
!     jac            Jacobian in the quadrature points
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp3d2dder' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp3d2dder'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dxdxi => B%dxdxig
      dphidx => B%phider
      jac => B%jac
      dphidxi => B%phiderc

      m = B%m
      n = B%n
      kstep = B%kstep

      do k = kstep, m, kstep
         dxdxi(k,:,3) = dxdxi(k,:,3)/jac(k)
      end do

      do k = kstep, m, kstep
         dxidx(k,1,1) = (dxdxi(k,2,2)*dxdxi(k,3,3) - &
                         dxdxi(k,2,3)*dxdxi(k,3,2))/jac(k)
         dxidx(k,1,2) = (dxdxi(k,2,3)*dxdxi(k,3,1) - &
                         dxdxi(k,2,1)*dxdxi(k,3,3))/jac(k)
         dxidx(k,1,3) = (dxdxi(k,2,1)*dxdxi(k,3,2) - &
                         dxdxi(k,2,2)*dxdxi(k,3,1))/jac(k)
         dxidx(k,2,1) = (dxdxi(k,3,2)*dxdxi(k,1,3) - &
                         dxdxi(k,3,3)*dxdxi(k,1,2))/jac(k)
         dxidx(k,2,2) = (dxdxi(k,3,3)*dxdxi(k,1,1) - &
                         dxdxi(k,3,1)*dxdxi(k,1,3))/jac(k)
         dxidx(k,2,3) = (dxdxi(k,3,1)*dxdxi(k,1,2) - &
                         dxdxi(k,3,2)*dxdxi(k,1,1))/jac(k)
      end do

      do j = 1, 3
         do k = kstep, m, kstep
            do i = 1, n
               dphidx(i,k,j) = dphidxi(i,k,1)*dxidx(k,1,j) + &
                               dphidxi(i,k,2)*dxidx(k,2,j)
            end do
         end do
      end do

1000  call erclos ( 'sepelp3d2dder' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp3d2dder'

      end if  ! ( debug )

      end
