      subroutine sepelpupw01tim
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute modified basis functions psi in case of upwind of type 12
!     Only linear 1d elements are implemented
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     upwind
!     basis_function
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k
      double precision :: normu(B%m), beta(B%m), h
      double precision, pointer :: phix(:,:)

!     beta           factor beta (see method)
!     h              jacobian of element
!     k              Counting variable
!     normu          norm of u-vector
!     phix           x derivative of basis function phi
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3082   This type of upwind has not been implemented for this shape
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The upwind basis function is given by
!
!     psi_i = phi_i + beta u . grad phi_i
!
!     with beta = 1 / 2 sqrt(Delta t^-2 + h^-2 ||u||^2)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpupw01tim' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpupw01tim'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)

      if ( B%type_basisfunc/=1 ) then

!     --- Error: shape # 1

         call errint ( 12, 1 )
         call errint ( B%type_basisfunc, 2 )
         call errsub ( 3082, 2, 0, 0 )
         go to 1000

      end if  ! ( B%type_basisfunc/=3 )

!     --- Compute h

      h = sum(B%w)
      if ( debug ) write(irefwr,2) 'h', h

!     --- Compute beta
!         beta = 1 / 2 sqrt(Delta t^-2 + h^-2 ||u||^2)

      normu = E%uvel(:)**2
      if ( debug ) write(irefwr,2) 'normu', normu
      beta = 1d0 / (2d0*sqrt(1d0/tstep**2+normu/h**2))

!     --- Compute psi
!         psi_i = phi_i + beta u . grad phi_i

      do k = 1, B%m
         B%psi(1:B%n,k) = B%phi(1:B%n,k) + beta(k)*E%uvel(k)*phix(1:B%n,k)
      end do  ! k = 1, B%m

1000  call erclos ( 'sepelpupw01tim' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelpupw01tim'

      end if  ! ( debug )

      end subroutine sepelpupw01tim
