      subroutine sepeltypcrhocp
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information about density rho and rho cp
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     density
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeltypcrhocp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeltypcrhocp'
         write(irefwr,1) 'indrho, indcp, indrhocp', &
                          E%indrho, E%indcp, E%indrhocp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%ind(E%indrho)==0 ) then

!     --- rho = 0, make it equal to 1

         E%ccoeffs(:,E%indrho) = 1d0
         E%ind(E%indrho) = -1

      end if  ! ( E%ind(E%indrho)==0 )

      if ( E%ind(E%indcp)==0 ) then

!     --- cp = 0, make it equal to 1

         E%ccoeffs(:,E%indcp) = 1d0
         E%ind(E%indcp) = -1

      end if  ! ( E%ind(E%indcp)==0 )

!     --- fill rho cp

      E%ccoeffs(:,E%indrhocp) = E%ccoeffs(:,E%indrho)*E%ccoeffs(:,E%indcp)
      E%ind(E%indrhocp) = -1
      E%coeffs(:,E%indrho) = real(E%ccoeffs(:,E%indrho),kind=8)
      E%coeffs(:,E%indrhocp) = real(E%ccoeffs(:,E%indrhocp),kind=8)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeltypcrhocp'

      end if  ! ( debug )
      call erclos ( 'sepeltypcrhocp' )

      end subroutine sepeltypcrhocp
