      subroutine sepeltypecdiff
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute typediff
!     Complex version
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     diffusion
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: eps
      double complex :: a11, a22

!     a11            Diffusion coefficient a11
!     a22            Diffusion coefficient a22
!     eps            Local accuracy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeltypecdiff' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeltypecdiff'
         call prinin ( E%ind, 6, 'ind' )
         write(irefwr,1) 'ndim', E%ndim
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%ind(2)/=0 .or. E%ind(3)/=0 .or. E%ind(5)/=0 ) then

         E%typediff = 3

      else if ( E%ind(1)/=0 .or. E%ind(4)/=0 .or. E%ind(6)/=0 ) then

         select case (E%ndim)
         case(1)

            E%typediff = 1

         case(2)

            if ( E%ind(1)==E%ind(4) .and. E%ind(1)<2003 ) then
               E%typediff = 1
            else
               E%typediff = 2
               if ( E%ind(1)<0 .and. E%ind(4)<0 ) then

!              --- a_11 constant and a_22 constant, check if they are the same

                  a11 = cmplx(E%user(-E%ind(1)),E%user(-E%ind(1)+1),kind=8)
                  a22 = cmplx(E%user(-E%ind(4)),E%user(-E%ind(4)+1),kind=8)
                  eps = abs(a11)*1d-11
                  if ( abs(a11-a22)<=eps ) &
                     E%typediff = 1
               end if  ! ( E%ind(1)<0 .and. E%ind(4)<0 )
            end if  ! ( E%ind(1)==E%ind(4) )

         case(3)

            if ( E%ind(1)==E%ind(4) .and. E%ind(4)==E%ind(6) .and. &
                 E%ind(1)<2003 ) then
               E%typediff = 1
            else
               E%typediff = 2
            end if  ! ( E%ind(1)==E%ind(4) .and. E%ind(4)==E%ind(6) )

         end select  ! case (E%ndim)

      end if  ! ( E%ind(2)/=0 .or. E%ind(3)/=0 .or. E%ind(5)/=0 )

!     --- Set logical diffusion

      E%diffusion = E%ind(1)/=0 .or. E%ind(2)/=0 .or. E%ind(3)/=0 .or. &
                    E%ind(4)/=0 .or. E%ind(5)/=0 .or. E%ind(6)/=0

      E%cdifftensor => E%ccoeffs(:,1:6)
      E%cdiffvec => E%ccoeffs(:,1)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'typediff', E%typediff
         write(irefwr,*) 'End sepeltypecdiff'

      end if  ! ( debug )
      call erclos ( 'sepeltypecdiff' )

      end subroutine sepeltypecdiff
