      subroutine sepeltypediff1 ( ind, typediff, diffusion )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute typediff
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     diffusion
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ind(6)
      integer, intent(out) :: typediff
      logical, intent(out) :: diffusion

!     diffusion      o    if true diffusion must be computed
!     ind            i    Indicates if coefficients are 0, constant or variable
!     typediff       o    Defines type of diffusion tensor
!                         1:  diffusion is a scalar
!                         2:  diffusion is a diagonal tensor
!                         3:  diffusion is a symmetric tensor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: eps

!     eps            Local accuracy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeltypediff1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeltypediff1'
         call prinin ( ind, 6, 'ind' )
         write(irefwr,1) 'ndim', E%ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ind(2)/=0 .or. ind(3)/=0 .or. ind(5)/=0 ) then

         typediff = 3

      else if ( ind(1)/=0 .or. ind(4)/=0 .or. ind(6)/=0 ) then

         select case (E%ndim)
         case(1)

            typediff = 1

         case(2)

            if ( ind(1)==ind(4) .and. ind(1)<2003 ) then
               typediff = 1
            else
               typediff = 2
               if ( ind(1)<0 .and. ind(4)<0 ) then

!              --- a_11 constant and a_22 constant, check if they are the same

                  eps = E%user(-ind(1))*1d-11
                  if ( abs(E%user(-ind(1))-E%user(-ind(4)))<=eps ) &
                     typediff = 1
               end if  ! ( ind(1)<0 .and. ind(4)<0 )
            end if  ! ( ind(1)==ind(4) )

         case(3)

            if ( ind(1)==ind(4) .and. ind(4)==ind(6) .and. &
                 ind(1)<2003 ) then
               typediff = 1
            else
               typediff = 2
            end if  ! ( ind(1)==ind(4) .and. ind(4)==ind(6) )

         end select  ! case (E%ndim)

      end if  ! ( ind(2)/=0 .or. ind(3)/=0 .or. ind(5)/=0 )

!     --- Set logical diffusion

      diffusion = ind(1)/=0 .or. ind(2)/=0 .or. ind(3)/=0 .or. &
                  ind(4)/=0 .or. ind(5)/=0 .or. ind(6)/=0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeltypediff1'

      end if  ! ( debug )
      call erclos ( 'sepeltypediff1' )

      end subroutine sepeltypediff1
