      subroutine sepelvlinelas
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for linear elasticity problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: v(:)
      integer :: i

!     i              Counting variable
!     v              subpart of element vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELINSTRAIN  Compute contribution to element vector due to
!                    initial strain
!     SEPELINSTRESS  Compute contribution to element vector due to
!                    initial stress
!     SEPELLOAD1     Fill the element vector with a load
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelvlinelas' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelvlinelas'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%vec = 0d0

      do i = 1, E%ndim

         if ( E%ind(22+i)/=0 ) then

!        --- F_i not equal to 0

            E%subvec => E%vec(i:E%icount:E%ndim)

            call sepelload1 ( E%coeffs(:,22+i), B%jdiag, B%phi )

         end if  ! ( E%ind(27+i)/=0 )

      end do  ! i = 1, E%ndim

      if ( E%initial_strain ) call sepelinstrain

      if ( E%initial_stress ) call sepelinstress

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelvlinelas'

      end if  ! ( debug )
      call erclos ( 'sepelvlinelas' )

      end subroutine sepelvlinelas
