      subroutine sepelvlinelasb
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for boundary element corresponding to
!     linear elastic problem
!     Old subroutine el8008
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: v(:), qmat(:,:)
      double precision :: work(m,3)
      integer :: i

!     i              Counting variable
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     v              sub vector of element vector
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPELLOAD1     Fill the element vector with a load
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelvlinelasb' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelvlinelasb'
         write(irefwr,1) 'iload, ndim', E%iload, E%ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( E%iload )

      case(2)

!     --- iload = 2, Classical case
!         Loop over the ndim components

         do i = 1, E%ndim

            if ( E%ind(i)/=0 ) then

!           --- F_i not equal to 0


               E%subvec => E%vec(i:E%icount:E%ndim)
               call sepelload1 ( E%coeffs(:,i), B%jdiag, B%phi )

            end if  ! ( E%ind(27+i)/=0 )

         end do  ! i = 1, E%ndim

      case (3)

!     --- iload = 3, T and theta given (2D only)

         if ( E%ind(2)<=0 ) then

!        --- theta constant

            work(:,1) = E%coeffs(:,1) * E%costet
            work(:,2) = E%coeffs(:,2) * E%sintet

         else

!        --- theta variable

            work(:,1) = E%coeffs(:,1) * cos(E%coeffs(:,1))
            work(:,2) = E%coeffs(:,2) * sin(E%coeffs(:,2))

         end if

         E%subvec => E%vec(1:E%icount:E%ndim)
         call sepelload1 ( work(1,1), B%jdiag, B%phi )
         E%subvec => E%vec(2:E%icount:E%ndim)
         call sepelload1 ( work(1,2), B%jdiag, B%phi )

      case(4)

!     --- iload = 4, Normal and tangential load given
!         Loop over the ndim components
!         See el8008


         if ( E%ndim==2 ) then

!        --- ndim = 2, compute x and y-component and store in work(3-4)

            work(:,1) = B%tang(:,1)*E%coeffs(:,2) + B%tang(:,2)*E%coeffs(:,1)
            work(:,2) = B%tang(:,2)*E%coeffs(:,2) - B%tang(:,1)*E%coeffs(:,1)

         else

!        --- ndim = 3
!            transform loads according to qmat:
!            At this stage, normal loads ( pressure ) only

           qmat => B%qmat(1,:,:)
           work(:,1) = qmat(1,:) * E%coeffs(:,1)
           work(:,2) = qmat(2,:) * E%coeffs(:,1)
           work(:,3) = qmat(3,:) * E%coeffs(:,1)

         end if

         do i = 1, E%ndim

            E%subvec => E%vec(i:E%icount:E%ndim)
            call sepelload1 ( work(1,i), B%jdiag, B%phi )

         end do  ! i = 1, E%ndim

      end select  ! case ( iload )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, E%icount, 'elemvc' )
         write(irefwr,*) 'End sepelvlinelasb'

      end if  ! ( debug )
      call erclos ( 'sepelvlinelasb' )

      end subroutine sepelvlinelasb
