      subroutine el0201 ( isequ, iseqp )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.3  date 24-01-2003 Extension with shape 16
!        version  2.2  date 19-05-2003 New shapes for type 204 (RvL)
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM200
!     or ELD200 in the case that IFIRST = 0
!     Fills sequence of displacement and pressure dof's depending on
!     element shapenumber
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isequ(*), iseqp(*)

!     iseqp   o    Sequence for pressure dof's
!     isequ   o    Sequence for displacement dof's
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i       Counting variable
!     j       Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      goto (200, 201, 202, 202, 204, 204) itype-199

!     --- Standard (ITYPE=200)

 200     if ( ishape.eq. 4) then

!        --- Quadratic triangle

         do i = 1, 2*6
            isequ(i) = i
         end do
         goto 299

!        --- Biquadratic quadrilateral

         else if ( ishape.eq. 6) then
         do i = 1, 2*9
            isequ(i) = i
         end do
         goto 299

!        --- Extended quadratic triangle

         else if ( ishape.eq. 7) then
         do i = 1, 2*7
            isequ(i) = i
         end do
         goto 299

!        --- Quadratic Tetrahedron

         else if ( ishape.eq.12 ) then
         do i = 1, 3*10
            isequ(i) = i
         end do
         goto 299

!        --- Quadratic hexahedron

      else if ( ishape.eq.14 ) then
         do i = 1, 3*27
            isequ(i) = i
         end do
         goto 299

!        --- Extended Quadratic Tetrahedron

      else if ( ishape.eq.16 ) then
         do i = 1, 3*15
            isequ(i) = i
         end do
         goto 299

      end if

!     --- Crouzeix-Raviart element (ITYPE=201)

!         -- Biquadratic quadrilateral

 201  if ( ishape.eq.6 ) then
         do i = 1, 2*9
            isequ(i) = i
         end do
         do i = 1, 3
            iseqp(i) = 2*9+i
         end do
        goto 299

!       --- Extended quadratic triangle

      else if ( ishape.eq.7 ) then
         do i = 1, 2*7
            isequ(i) = i
         end do
         do i = 1, 3
            iseqp(i) = 2*7+i
         end do
        goto 299

!       --- Quadratic hexahedron

      else if ( ishape.eq.14 ) then
         do i = 1, 3*14
            isequ(i) = i
         end do
         do i = 3*14+1, 3*27
            isequ(i) = i+4
         end do
         do i = 1, 4
            iseqp(i) = 3*14+i
         end do
         goto 299

!        --- Extended quadratic tetrahedron

      else if ( ishape.eq.16 ) then
         do i = 1, 3*15
            isequ(i) = i
         end do
         do i = 1, 4
            iseqp(i) = 3*15+i
         end do
         goto 299
      end if

!     --- Taylor-Hood element (ITYPE=202/203)
!         -- Biquadratic quadrilateral

 202    if (ishape.eq.6) then
         isequ(1)=1
         isequ(2)=2
         do j=1,4
         isequ(2+j)=3+j
         isequ(6+j)=8+j
         isequ(10+j)=13+j
         isequ(14+j)=18+j
         end do
         iseqp(1)=3
         iseqp(2)=8
         iseqp(3)=13
         iseqp(4)=18
         goto 299

!        --- Extended quadratic triangle

        else if ( ishape.eq.7 ) then
         isequ(1)=1
         isequ(2)=2
         do j=1,4
         isequ(2+j)=3+j
         isequ(6+j)=8+j
         isequ(10+j)=13+j
         end do
         iseqp(1)=3
         iseqp(2)=8
         iseqp(3)=13
         goto 299

!        --- Quadratic hexahedron

        else if ( ishape.eq.14 ) then
         do j = 1, 3
            do i = 1, 2
               isequ((i-1)*5+j) = (i-1)*6+j
               isequ((i-1)*5+18+j) = (i-1)*6+20+j
               isequ((i-1)*5+54+j) = (i-1)*6+58+j
               isequ((i-1)*5+72+j) = (i-1)*6+78+j
            end do
            isequ(3+j) = 4+j
            isequ(6+j) = 7+j
            isequ(21+j) = 24+j
            isequ(24+j) = 27+j
            isequ(57+j) = 62+j
            isequ(60+j) = 65+j
            isequ(75+j) = 82+j
            isequ(78+j) = 85+j
         end do
         do i = 10, 18
            isequ(i) = i+2
         end do
         do i = 28, 54
            isequ(i) = i+4
         end do
         do i = 64, 72
            isequ(i) = i+6
         end do

         iseqp(1) = 4
         iseqp(2) = 11
         iseqp(3) = 24
         iseqp(4) = 31
         iseqp(5) = 62
         iseqp(6) = 69
         iseqp(7) = 82
         iseqp(8) = 89
         goto 299

!        --- Extended quadratic tetrahedron

      else if ( ishape.eq.16 ) then

         do j = 1, 3
               isequ(j) = j
         end do
         do j = 1, 6
               isequ(3+j) = 4+j
         end do
         do j = 1, 6
               isequ(9+j) = 11+j
         end do
         do j = 1, 15
               isequ(15+j) = 18+j
         end do
         do j = 1, 18
               isequ(30+j) = 34+j
         end do
         iseqp(1) = 4
         iseqp(2) = 11
         iseqp(3) = 21
         iseqp(4) = 31

         goto 299

      end if

!     --- Taylor-Hood element with 4-phase extension (ITYPE=204)

!         --- Quadratic triangle

 204  if (ishape.eq.4) then
         isequ(1)=1
         isequ(2)=2
         do i=1,4
            isequ(2+i)=i+7
            isequ(6+i)=i+16
         end do
         isequ(11)=26
         isequ(12)=27
         iseqp(1)=3
         iseqp(2)=12
         iseqp(3)=21
         go to 299

!        --- Biquadratic quadrilateral

      else if ( ishape.eq.6 ) then
         isequ(1)=1
         isequ(2)=2
         do i=1,4
            isequ(2+i)=i+7
            isequ(6+i)=i+16
            isequ(10+i)=i+25
            isequ(14+i)=i+34
         end do
         iseqp(1)=3
         iseqp(2)=12
         iseqp(3)=21
         iseqp(4)=30
         go to 299

!        --- Extended quadratic triangle

      else if ( ishape.eq.7 ) then
         isequ(1)=1
         isequ(2)=2
         do i=1,4
            isequ(2+i)=i+7
            isequ(6+i)=i+16
            isequ(10+i)=i+25
         end do
         iseqp(1)=3
         iseqp(2)=12
         iseqp(3)=21
         go to 299

!        --- Quadratic hexahedron

      else if (ishape.eq.14) then
         do i = 1, 3
            isequ(i) = i
         end do
         do i = 1, 6
            isequ(3+i) = 8+i
            isequ(21+i) = 36+i
            isequ(57+i) = 82+i
            isequ(75+i) = 110+i
         end do
         do i = 1, 12
            isequ(9+i) = 19+i
            isequ(63+i) = 93+i
         end do
         do i = 1,30
            isequ(27+i)=47+i
         end do
         iseqp(1) = 4
         iseqp(2) = 15
         iseqp(3) = 32
         iseqp(4) = 43
         iseqp(5) = 78
         iseqp(6) = 89
         iseqp(7) = 106
         iseqp(8) = 117
         goto 299

!        --- Extended quadratic tetrahedron

      else if (ishape.eq.16) then
         do i = 1, 3
            isequ(i) = i
         end do
         do i = 1, 6
            isequ(3+i) = 8+i
            isequ(9+i) = 19+i
         end do
         do i = 1, 15
            isequ(15+i) = 30+i
         end do
         do i = 1, 15
            isequ(30+i) = 50+i
         end do
         iseqp(1)=4
         iseqp(2)=15
         iseqp(3)=26
         iseqp(4)=46
         goto 299
      end if

 299  end
