      subroutine el0400( iuser, user, jelp, isub, isup, method, &
                         mconv, ktime, modelv, irotat )
! ======================================================================
!
!        programmer    Jaap van der Zanden and Guus Segal
!        version  2.12 date 20-09-2002 Set ndimlc
!        version  2.11 date 23-08-2001 New call to elusr3
!        version  2.10 date 18-12-1998 Include commons
!        version  2.9  date 14-07-1996 correction ind(lengthmst)
!        version  2.8  date 12-12-1995 add modelv=5,6,7; tune celwrk
!        version  2.7  date 26-12-1994 New commons cmessc and cmessg
!        version  2.6  date 22-12-1993 improve error report
!        version  2.5  date 10-12-1993 save common; add /cmcdpr/
!        version  2.4  date 23-06-1992 New common cmessg
!        version  2.3  date 12-04-1992 Extension for call of elflr7
!        version  2.2  date 29-01-1991 Correction Ind(50)=1 for ktime.gt.0
!                                      Peter Slikkerveer
!        version  2.1  date 30-11-1990 Correction for modelv
!        version  2.0  date 24-01-1990 Extension for type number 406
!        version  1.0  date 02-04-1989 Preceding version: EL4029
!
!   copyright (c) 1989-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initialisations for subroutine ELM400
!     in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialised with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     flow
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      double precision x, w, dphidx, xgauss, phi, array, amas, amasps, &
                       u, ugauss, un, dudx, trans, transl, qphixx, &
                       etaeff, secinv, dvisd2, omega, sigma, rho, cn, &
                       clam, f, eij, dlamdx, penal, psi, dpsidx, dum, &
                       dum1, visprm
      common /celwrk/ x(14), w(7), dphidx(98), xgauss(14), phi(49), &
                      array(56), u(21), ugauss(21), un(21), dudx(42), &
                      trans(24), transl(1), qphixx(147), etaeff(7), &
                      secinv(7), dvisd2(7), omega, sigma, rho, &
                      cn, clam, psi(42), f(21), dpsidx(84), amasps(36), &
                      penal(14), amas(49), dum1(210), visprm(3), &
                      eij(6), dlamdx(6), dum(488)
      save   /celwrk/

!                     /celwrk/
!          Work common block to store local arrays
!          CELWRK contains exactly 1500 real positions, the last 500
!          are reserved for the ELP subroutines
!          The following parameters are used in CELWRK
!     amasps         Mass matrix corresponding to basis funtions psi
!     array          8 x m array to store the coefficients for each
!                    integration point
!                    Contents:
!                    Pos 1-m:       - &
!                    Pos m+1-2m:    rho
!                    Pos 2m+1-3m:   - &
!                    Pos 3m+1-4m:   omega
!                    Pos 4m+1-5m:   f 1
!                    Pos 5m+1-6m:   f 2
!                    Pos 6m+1-7m:   f 3
!                    Pos 7m+1-8m:   eta
!     clam           Parameter lambda for Generalised Newtonian fluids
!     cn             Parameter n for Generalised Newtonian fluids
!     dlamdx         Array of 6 positions containing the derivatives of the
!                    barycentric co-ordinates
!     dphidx         Array of length n x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                      in node k
!     dpsidx         Array of length n-1 x m x 2  containing the derivatives
!                    of the special basis functions corresponding to the
!                    nodal points in the sequence:
!                    d psi / dx = dpsidx (i,k,1);  d psi / dy = dpsidx (i,k,2);
!                         i                             i
!                      in node k
!     dudx           Array of length 4 x m containing the derivatives of the
!                    preceding velocity in the integration points in the
!                    sequence:
!                    1    - m:  du/dx
!                    m+1  - 2m: du/dy
!                    2m+1 - 3m: dv/dx
!                    3m+1 - 4m: dv/dy
!     dum            488 positions reserved for the ELP subroutines
!     dum1           work space free for later use
!     dvisd2         Array of length m to store the derivative of the viscosity
!                    with respect to the second invariant of the
!                    stress tensor in the integration points
!     eij            Array of 6 positions containing the factors e
!     etaeff         Array of length m to store the effective viscosity in
!                    the integration points
!     f              Array of length 3n to store the right-hand side vector
!                    temporarily
!     omega          Angular velocity of rotating system
!     phi      o     Array of length n x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i,k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     psi            Array of length n-1 x m containing the values of the
!                    basis functions corresponding to the nodal points
!                    in the Gauss points in the sequence:
!                           k
!                    psi (xg ) = psi (i,k)
!                       i
!                    Array psi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     qphixx         Array of length 3 x n x n to store the integrals
!                    corresponding to the second derivative terms on a
!                    reference element
!     rho            Density of the fluid
!     secinv         Array of length m to store the second invariant of the
!                    stress tensor in the integration points
!     sigma          Inverse of parameter epsilon in Penalty function method
!     trans          Transformation array for the velocity in the centroid
!                    length: 2 * inpelm
!     transl         Not yet used
!     x              array of length n x 2 containing the x and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);
!                     i              i
!     xgauss   o     Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
!     u              Array of length 3n to store the preceding velocity in
!                    the nodal points
!     ugauss         Array of length 3m to store the preceding velocity in
!                    the integration points
!     un             Array of length 3n to store the velocity at a preceding
!                    time level in the nodal points
!     visprm         Viscosity model parameters for modelv=6,7
!     w              array of length m containing the weights for the
!                    numerical integration
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - &
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*)
      integer iuser(*), jelp, isub, isup, method, mconv, ktime, modelv, &
              irotat

!     irotat         Parameter indicating if the co-ordinate system rotates (10
!                    or not (0)
!     isub    o      Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup    o      Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     ktime          Parameter indicating the type of time-dependence
!                    Possible values:
!                    0:  Steady state
!                    1:  Instationary equations theta method
!                  +-2:  Pressure correction scheme
!                    3:  Mass matrix only
!                    4:  Iterative instationary equations theta method
!     iuser    i     User array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     jelp    o      Indication of the type of basis function subroutine must
!                    be called by ELM400. Possible vaues:
!                    1:  Subroutine ELP220  (Seven-point extended triangle)
!     mconv          With this parameter the type of linearization of the
!                    convective terms is indicated
!                    Possible values:
!                    0:  No convective terms (Stokes)
!                    1:  Standard Picard linearization
!                    2:  Newton linearization
!                    3:  Picard linearization in the wrong way
!                        (small convergence region, slow convergence)
!                    4:  Picard linearization (only effect on right-hand side)
!     method  o      Indicates the type of method that is used to solve the
!                    Navier-Stokes equations
!                    Possible values:
!                    1:   Penalty function method
!     modelv         Parameter indicating the type of viscosity model used
!                    Possible values:
!                    1:  Newtonian liquid
!                    2:  Power law liquid
!                    3:  Carreau model liquid
!                    4:  Plastico-viscous liquid
!                    5:  Papanastasiou plastico-viscous model
!                    6:  Bi-viscosity plastico-viscous model
!                    7:  Carreau-Yasuda model
!                  100:  User defined effective viscosity (based on grad(v))
!                  101:  User defined effective viscosity (based on secinv)
!                  102:  User defined effective viscosity (x,u,secinv)
!     user    i      User array in which real information of the
!                    coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, isubcf, isupcf, nparm, ishift
      logical funchk

!    funchk        Indicates whether there are coefficients that must be
!    i             Counting variable
!    ishift        Shift in coefficient sequence number
!    isubcf        Under bound for coefficients check
!    isupcf        Upper bound for coefficients check
!    j             Counting variable
!    nparm         Number of parameters stored in arrays IUSER and USER, &
!                  until eta
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCOMM  Initializes the commons celp and celint with standard values
!     ELUSR3  Extracts information concerning the coefficients from IUSER
!             and USER.
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRCHR  Put character in error message
!     ERREAL  Put real in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
!     PRINLG  Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     238  Omega # 0, whereas axi-symmetric co-ordinates are used
!    1300  INPELM has wrong value
!    1301  ICOUNT has wrong value
!    1302  IUSER(6) smaller than 7
!    1303  Parameter i is a function
!    1304  Paramater i  is non-positive
!    1305  MODELV has wrong value
!    1306  Eta, cn, or lambda has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0400' )
      call elcomm
      ncomp = 2
      ndimlc = 2
      ind(lengthmst) = 0
      ishift = 0

!     --- Set parameters depending on the type numbers

      if ( itype.eq.400 .or. itype.eq.402 .or. itype.eq.404 &
           .or. itype.eq.406 ) then

!     --- itype = 400,402,404,406:  Seven-point extended triangle

         n      =  7
         m      =  7
         mn     = 49
         jelp   =  1
         method =  1
         jtrans =  1
         if ( itype.eq.406 ) then
            ncomp = 3
            jdiag = 2
         else
            ncomp = 2
         end if
         if ( inpelm.ne.n-1 ) then

!        --- Error 1300:  inpelm incorrect

            call errint ( inpelm, 1 )
            call errint ( itype, 2 )
            call errint ( n-1, 3 )
            call errint ( ielgrp, 4 )
            call errint ( ielem, 5 )
            call errsub ( 1300, 5, 0, 0 )
         end if
         if (icount.ne.ncomp*(n-1) ) then

!        --- Error 1301:  icount incorrect

            call errint ( icount, 1 )
            call errint ( itype, 2 )
            call errint ( ncomp*(n-1), 3 )
            call errint ( ielgrp, 4 )
            call errint ( ielem, 5 )
            call errsub ( 1301, 5, 0, 0 )
         end if
         if ( itype.eq.400 ) then

!        --- itype = 400:  Cartesian co-ordinates

            jcart = 1

         else if ( itype.eq.402 ) then

!        --- itype = 402:  Polar co-ordinates

            jcart = 3

         else if ( itype.eq.404 .or. itype.eq.406 ) then

!        --- itype = 404, 406:  Axi-symmetric co-ordinates

            jcart = 2

         end if

      end if

!     --- Fill the arrays IND, MST and ARRAY with information of the
!     coefficient

      ipos = iuser ( ielgrp + 5 )
      if ( ipos.lt.7 ) then

!     --- Error 1302:  ipos < 7

         call errint ( ielgrp+5, 1 )
         call errint ( ipos, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errint ( ielem, 5 )
         call errsub ( 1302, 5, 0, 0 )
      end if
      nuser = 0
      nparm = ncomp+5
      do i = 1, nparm
         call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
      end do
      ishift = ishift+nparm

!     --- Check the parameters
!         Sigma = 1 / epsilon

      if ( ind(1).gt.0 ) then

!     --- Error 1303:  sigma is a function

         call errint ( 1, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errchr ( 'epsilon', 1 )
         call errsub ( 1303, 4, 0, 1 )

      else if ( array(1).le.0d0 ) then

!     --- Error 1304: Epsilon is not positive

         call errint ( 1, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call erreal ( array(1), 1 )
         call errchr ( 'epsilon', 1 )
         call errsub ( 1304, 4, 1, 1 )
      else

         sigma = 1d0 / array(1)

      end if

!     --- Rho

      if ( ind(2).gt.0 ) then

!     --- Error 1303: Rho is a function

         call errint ( 2, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errchr ( 'rho', 1 )
         call errsub ( 1303, 4, 0, 1 )

      else if ( array(1+m).le.0d0 ) then

!     --- Error 1304: Rho is not positive

         call errint ( 2, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call erreal ( array(1+m), 1 )
         call errchr ( 'rho', 1 )
         call errsub ( 1304, 4, 1, 1 )
      else

         rho = array(1+m)

      end if

!     --- Compute mconv and ktime

      mconv = ind(3)
      if ( mconv.ge.100 ) then
         mconv = mconv - 100
         ktime = 4
      else if ( mconv.ge.30) then
         ktime = 3
         mconv = 0
      else if ( mconv.ge.10) then
         mconv = mconv-10
         ktime = 1
         if ( mconv.ge.10) then
           mconv = mconv-10
           ktime = 2
           if ( mconv.ge.5) then
              mconv = mconv-5
              ktime = -2
           end if
         end if
      end if
      if ( mconv.gt.0 .or. ktime.gt.0 .and. ktime.ne.3 ) &
         ind(lengthmst) = 1

      if ( mconv.lt.0 .or. mconv.gt.4 ) then

!     --- Error 1305: mconv has wrong value

         call errint ( 3, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errint ( mconv, 5 )
         call errchr ( 'mconv', 1 )
         call errsub ( 1305, 5, 0, 1 )

      end if

!     --- Omega

      if ( ind(4).gt.0 ) then

!     --- Error 1303: Omega is a function

         call errint ( 4, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errchr ( 'omega', 1 )
         call errsub ( 1303, 4, 0, 1 )

      else

         omega = array(1+3*m)
         if ( abs(omega).lt.epsmac ) then
            irotat = 0
         else
            irotat = 1
            if ( itype.eq.404 ) then

!           --- 238:  irotat = 1, whereas itype = 404

               call errint ( ielgrp, 1 )
               call errint ( ielem, 2 )
               call errsub ( 238, 2, 0, 0 )

            end if

         end if

      end if

!     --- Modelv

      modelv = ind(nparm)
      ind(nparm) = 0
      if ( modelv.le.0 .or. modelv.gt.7 .and. modelv.lt.100 .or. &
           modelv.gt.102 ) then

!     --- Error 1305: modelv has wrong value

         call errint ( nparm, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errint ( modelv, 5 )
         call errchr ( 'modelv', 1 )
         call errsub ( 1305, 5, 0, 1 )

      else if ( modelv.le.7 ) then

!     --- 1 <= modelv <= 7,  Read eta

         ishift = ishift+1
         call elusr3 ( nparm, array(1+(nparm-1)*m), iuser, user, &
                       ishift )
         if ( ind(nparm).eq.0 .or. ind(nparm).lt.0 .and. &
              array(1+(nparm-1)*m).lt.0 ) then

!        --- Error 1304: Eta is not positive

            call errint ( nparm, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call erreal ( array(1+(nparm-1)*m), 1 )
            call errchr ( 'eta (0)', 1 )
            call errsub ( 1304, 4, 1, 1 )

         end if

!        --- for modelv > 1 more parameters are necessary for the viscous part

         if ( modelv.ge.2 ) then

!        --- modelv = 2,3, -- 7, .i.e. Power-law, Carreau or Plastico-Viscous
!            models
!            Read parameter n, store in array(1 to m)

            ind(lengthmst) = 1
            ishift = ishift+1
            call elusr3 ( 1, array, iuser, user, ishift )
            cn = array(1)
            if ( ind(1).gt.0 ) then

!           --- Error 1303: n is a function

               call errint ( nparm+1, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errchr ( 'n (Power)', 1 )
               call errsub ( 1303, 4, 0, 1 )

            end if
            if ( cn.lt.0d0 .or. &
                (modelv.ne.5 .and. cn.gt.5d0) ) then

!           --- Error 1306: cn has wrong value

               call errint ( nparm+1, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call erreal ( cn, 1 )
               call errchr ( 'n (Power)', 1 )
               call errsub ( 1306, 4, 1, 1 )

            end if

            if ( modelv.ge.3 ) then

!           --- third parameter of Carreau models or Plastico viscous models

               ishift = ishift+1
               call elusr3 ( 1, array, iuser, user, ishift )
               clam = array(1)
               if ( ind(1).gt.0 ) then

!              --- Error 1303: lambda is a function

                  call errint ( nparm+2, 1 )
                  call errint ( itype, 2 )
                  call errint ( ielgrp, 3 )
                  call errint ( ielem, 4 )
                  call errchr ( 'lambda', 1 )
                  call errsub ( 1303, 4, 0, 1 )

               end if
               if ( clam.lt.0 ) then

!              --- Error 1306: lambda has wrong value

                  call errint ( nparm+2, 1 )
                  call errint ( itype, 2 )
                  call errint ( ielgrp, 3 )
                  call errint ( ielem, 4 )
                  call erreal ( clam, 1 )
                  call errchr ( 'lambda', 1 )
                  call errsub ( 1306, 4, 1, 1 )

               end if

               if ( modelv.ge.6 ) then

!              --- fourth parameter of bi-viscosity  Plastico viscous model
!                                   or Carreau-Yasuda model

                  ishift = ishift+1
                  call elusr3 ( 1, array, iuser, user, ishift )
                  visprm(1) = array(1)
                  if ( ind(1).gt.0 ) then

!                 --- Error 1303: visprm(1) is a function

                     call errint ( nparm+3, 1 )
                     call errint ( itype, 2 )
                     call errint ( ielgrp, 3 )
                     call errint ( ielem, 4 )
                     call errchr ( '4-th viscosity model parameter', 1 )
                     call errsub ( 1303, 4, 0, 1 )

                  end if
                  if ( visprm(1).lt.0 ) then

!                 --- Error 1306: visprm(1) has wrong value

                     call errint ( nparm+3, 1 )
                     call errint ( itype, 2 )
                     call errint ( ielgrp, 3 )
                     call errint ( ielem, 4 )
                     call erreal ( visprm(1), 1 )
                     call errchr ( '4-th viscosity model parameter', 1 )
                     call errsub ( 1306, 4, 1, 1 )

                  end if
                  if ( modelv.ge.7 ) then

!                 --- fifth  parameter of Carreau-Yasuda model

                     ishift = ishift+1
                     call elusr3 ( 1, array, iuser, user, ishift )
                     visprm(2) = array(1)
                     if ( ind(1).gt.0 ) then

!                    --- Error 1303: visprm(2) is a function

                        call errint ( nparm+4, 1 )
                        call errint ( itype, 2 )
                        call errint ( ielgrp, 3 )
                        call errint ( ielem, 4 )
                        call errchr ( '5-th viscosity model parameter', &
                              1 )
                        call errsub ( 1303, 4, 0, 1 )

                     end if
                     if ( visprm(2).lt.0 ) then

!                    --- Error 1306: visprm(2) has wrong value

                        call errint ( nparm+4, 1 )
                        call errint ( itype, 2 )
                        call errint ( ielgrp, 3 )
                        call errint ( ielem, 4 )
                        call erreal ( visprm(2), 1 )
                        call errchr ( '5-th viscosity model parameter', &
                              1 )
                        call errsub ( 1306, 4, 1, 1 )

                     end if
                  end if

               end if

            end if

         end if

      end if
      if ( modelv.gt.1 .or. mconv.gt.0 ) jderiv = 1

!     --- Test array lengths

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      if ( ktime.eq.3 ) then
         isub = nparm+1
         isup = nparm
      else
         if ( matrix ) then
            isupcf = nparm
         else
            isupcf = nparm-1
         end if
         if ( vector ) then
            isubcf = 5
         else
            isubcf = 6
         end if

         do i = isubcf , isupcf
            isub = i

!           --- If ind(i) > 0 then paramater i corresponds to a function

            if ( ind(i).gt.0 ) goto 170

         end do

         isub = isupcf + 1

170      isup = isupcf
         do i = isupcf, isub , -1
            isup = i
            if ( ind(i).gt.0 ) goto 190

         end do

      end if

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters

190   do i = isub, isup
         ist(i) = 0
      end do
      funchk = .false.
      do i = isub, isup
         if ( ind(i).gt.0 .and. ind(i).lt.2000 ) then

!        --- only the function calls are checked

            do j = i+1, isup
               if ( ind(j).eq.ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i).gt.1000 .and. ind(i).le.2000 ) ind(lengthmst) = 1
         if ( ind(i).gt.10000 ) funchk = .true.
      end do

      if ( modelv.gt.1 ) ind(lengthmst) = 1
      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

      if ( ierror.ne.0 ) call instop
      call erclos ( 'el0400' )

      end
