      subroutine el0906 ( ind1, jelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 04-08-2006
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize some constants with respect to subroutine elm906
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ind1(*), jelp

!     ind1           o    Array containing the relative position if the vertices
!                         in index1
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0906' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0906'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill standard values in common blocks

      call elcomm
      if ( ishape==4 ) then

!     --- shape = 4, quadratic triangle

         jelp = 3     ! linear triangle for pressure
         ind1(1) = 1  ! first vertex
         ind1(2) = 3  ! second vertex
         ind1(3) = 5  ! third vertex
         n = 3
         m = 3

      else if ( ishape==6 ) then

!     --- shape = 6, bi-quadratic quadrilateral

         jelp = 5     ! bi-linear quadrilateral for pressure
         ind1(1) = 1  ! first vertex
         ind1(2) = 3  ! second vertex
         ind1(3) = 5  ! third vertex
         ind1(4) = 7  ! fourth vertex
         n = 4
         m = 4

      else if ( ishape==12 ) then

!     --- shape = 12, quadratic tetrahedron

         jelp = 11    ! linear tetrahedron for pressure
         ind1(1) = 1  ! first vertex
         ind1(2) = 3  ! second vertex
         ind1(3) = 5  ! third vertex
         ind1(4) =10  ! fourth vertex
         n = 4
         m = 4

      else if ( ishape==14 ) then

!     --- shape = 14, quadratic hexahedron

         jelp = 13    ! linear hexahedron for pressure
         ind1(1) = 1  ! first vertex
         ind1(2) = 3  ! second vertex
         ind1(3) = 9  ! third vertex
         ind1(4) = 7  ! fourth vertex
         ind1(5) = 19 ! first vertex
         ind1(6) = 21 ! second vertex
         ind1(7) = 27 ! third vertex
         ind1(8) = 25 ! fourth vertex
         n = 8
         m = 8

      else

!     --- shape number not yet implemented

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if  ! ( ishape==4 )
      npsi = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0906'

      end if  ! ( debug )
      call erclos ( 'el0906' )

      end
