      subroutine el4010 ( icheld, ix, jd, itcoor, x, u, dudx, ip, nloc, &
                          ndim, elemvc )
! ======================================================================
!
!        programmer   jaap van der zanden
!        version 2.6  date 18-12-1998 include commons
!        version 2.5  date 22-12-1992 improve error report
!        version 2.4  date 21-11-1992 replace ertrap; expl. decl.
!        version 2.3  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version 2.2  date 05-05-1989 removal of ierror
!        version 2.1  date 12-01-1989 limit r ==> 0
!        version 2.0  date 16-01-1987
!
!
!   copyright (c) 1984-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     this subroutine computes derived quantities for icheld=2,3,4
!          (d v_i/d x_j, divergence, curl)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!   curl
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
      integer icheld, ix, jd, itcoor, ip, nloc, ndim
      double precision x(*), u(*), dudx(*), elemvc(*)

!     dudx       i    Real m x ndim  array in which the values of the
!                     derivatives of the solution vector in the integration
!                     points are stored in the sequence:
!                     du / dx  (x ) = dudx ( k, j )
!                       j     k
!                     If jdercn = 0 in common block celint, then only
!                     dudx (1,*) is filled because dudx is then a constant
!                     per element
!                     dudx is only filled if jderiv > 0
!     elemvc     o    Element vector
!     icheld     i    choice parameter for action to be selected
!                      =2  derivative of u with respect to ix
!                      =3  divergence of u
!                      =4  curl of u
!     ip         i    sequence number of nodal point
!     itcoor     i    type of co-ordinate system
!     ix         i    sequence number of space co-ordinate
!     jd         i    sequence number of component of u
!     n          i    number of nodal points of element
!     ndim       i    Dimension of the space.
!     u          i    (velocity) vector
!     x          i    o-ordinates of nodal points of the element
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, jd1, ncomploc
      double precision xip
      integer n2,n3,n4,n5,n6,n7,n8
      save    n2,n3,n4,n5,n6,n7,n8

!     i1
!     jd1
!     n2
!     n3
!     n4
!     n5
!     n6
!     n7
!     n8
!     ncomploc
!     xip
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     214   parameter idgfd1 has a non-positive value
!     218   Degree of freedom does not exist
!     262   ix has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial; for x_1=r=0 the div and curl are computed as if for a
!            divergence free vector u
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- test

      if (ifirst.eq.0) then
         ncomploc=ndim
         if (itcoor.eq.3) ncomploc=3
         if ((itype.ge.420).and.(itype.le.422)) ncomploc=ncomploc+1
         if (icheld.eq.2) then
            if (jd.lt.1) then
               call eropen('el4010')
               call errint(jd,1)
               call errsub ( 214, 1, 0, 0)
               call erclos('el4010')
            else if (jd.gt.ncomploc) then
               call eropen('el4010')
               call errint(jd,1)
               call errint(ncomploc,2)
               call errsub ( 218, 2, 0, 0)
               call erclos('el4010')
            end if
            if ( ix.lt.1 .or. ix.gt.ndim ) then
               call eropen('el4010')
               call errint(ix,1)
               call errint(ndim,2)
               call errint(itype,3)
               call errint(ielgrp,4)
               call errsub ( 262, 4, 0, 0)
               call erclos('el4010')
            end if
         else if (icheld.eq.3) then
            if (ndim.eq.2) then
               n3=3*nloc
            else if (ndim.eq.3) then
               n4=4*nloc
               n8=n4+n4
            end if
         else if (icheld.eq.4) then
            n2=nloc+nloc
            if (itcoor.eq.3) then
               n4=n2+n2
               n5=n4+nloc
            else if (ndim.eq.3) then
               n3=n2+nloc
               n5=n3+n2
               n6=n5+nloc
               n7=n6+nloc
            end if
         end if
      end if

!     --- actual computations

      if (icheld.eq.2) then

!     --- d u(jcomp)/d x(idim) in point ip

         jd1=jd
         if (itcoor.eq.3) then
            if (jd.eq.2) then
               jd1=3
            else if (jd.eq.3) then
               jd1=2
            end if
         end if
         i1=((jd1-1)*ndim+ix-1)*nloc+ip
         if ((itcoor.eq.0).or.(itcoor.eq.1).or.(itcoor.eq.3).or. &
            ((itcoor.eq.2).and.(ix.eq.1)).or. &
            ((itcoor.eq.4).and.((ix.eq.1).or.(ix.eq.3)))) then
            elemvc(1)=dudx(i1)
         else
            xip=x(ip)
            if (abs(xip).gt.epsmac) then
               elemvc(1)=dudx(i1)/xip
            else
               elemvc(1)=0.0d0
            end if
         end if
      else if (icheld.eq.3) then

!     --- div u

         if (ndim.eq.2) then
            if (itcoor.eq.0) then
               elemvc(1)=dudx(ip)+dudx(n3+ip)
            else if ((itcoor.eq.1).or.(itcoor.eq.3)) then
               xip=x(ip)
               if (abs(xip).gt.epsmac) then
                  elemvc(1)=dudx(ip)+u(ip)/xip+dudx(n3+ip)
               else

!              --- for r=0 apply the div u = 0 condition

                  elemvc(1)=dudx(ip)*2+dudx(n3+ip)
               end if
            else if (itcoor.eq.2) then
               xip=x(ip)
               if (abs(xip).gt.epsmac) then
                  elemvc(1)=dudx(ip)+(u(ip)+dudx(n3+ip))/xip
               else

!              --- for r=0 apply the div u = 0 condition

                  elemvc(1)=dudx(ip)*2
               end if
            end if
         else if (ndim.eq.3) then
            if (itcoor.eq.0) then
               elemvc(1)=dudx(ip)+dudx(n4+ip)+dudx(n8+ip)
            else if (itcoor.eq.4) then
               xip=x(ip)
               if (abs(xip).gt.epsmac) then
                  elemvc(1)=dudx(ip)+(u(ip)+dudx(n3+ip))/xip+dudx(n8+ip)
               else

!              --- for r=0 apply the div u = 0 condition

                  elemvc(1)=dudx(ip)*2+dudx(n8+ip)
               end if
            end if
         end if
      else if (icheld.eq.4) then

!     --- curl u

         if (ndim.eq.2) then
            if ((itcoor.eq.0).or.(itcoor.eq.1)) then
               elemvc(1)=dudx(n2+ip)-dudx(nloc+ip)
            else if (itcoor.eq.2) then
               xip=x(ip)
               if (abs(xip).gt.epsmac) then
                  elemvc(1)=dudx(n2+ip)+(u(nloc+ip)-dudx(nloc+ip))/xip
               else

!              --- for r=0 apply the div u = 0 condition

                  elemvc(1)=dudx(n2+ip)*2
               end if
            else if (itcoor.eq.3) then
               xip=x(ip)
               elemvc(1)=-dudx(n5+ip)
               elemvc(2)=dudx(nloc+ip)-dudx(n2+ip)
               if (abs(xip).gt.epsmac) then
                  elemvc(3)=dudx(n4+ip)+u(n2+ip)/xip
               else

!              --- for r=0 apply the div u = 0 condition

                  elemvc(3)=dudx(n4+ip)*2
               end if
            end if
         else if (ndim.eq.3) then
            if (itcoor.eq.0) then
               elemvc(1)=dudx(n7+ip)-dudx(n5+ip)
               elemvc(2)=dudx(n2+ip)-dudx(n6+ip)
               elemvc(3)=dudx(n3+ip)-dudx(nloc+ip)
            else if (itcoor.eq.4) then
               xip=x(ip)
               elemvc(2)=dudx(n2+ip)-dudx(n6+ip)
               if (abs(xip).gt.epsmac) then
                  elemvc(1)=dudx(n7+ip)/xip-dudx(n5+ip)
                  elemvc(3)=dudx(n3+ip)+(u(nloc+ip)-dudx(nloc+ip))/xip
               else

!              --- for r=0 apply the div u = 0 condition

                  elemvc(1)=-dudx(n5+ip)
                  elemvc(3)=dudx(n3+ip)*2
               end if
            end if
         end if
      end if
      end
