      subroutine el4022 (ichvis,itcoor,etha,stress,w,xg,phig,dphidx, &
                         nstart,nloc,mloc,lrowloc,psig,dpsidx,n1, &
                         dldx,qdphix,delta,work)
!
! ======================================================================
!
!        programmer   jaap van der zanden
!
!        version 3.3  date   22-12-1992 save common; expl. decl
!        version 3.2  date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 3.1  date   19-01-1987 save
!        version 3    date   20-05-1985 itcoor=3
!        version 2    date   06-05-1985 simplified parameter list
!        version 1    date   08-01-1985
!
!
!   copyright (c) 1984, 1992  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
      use sepmodulecactl
      implicit none
      integer ichvis, itcoor, nstart, nloc, mloc, lrowloc, n1
      double precision etha(*), stress(*), w(*), xg(*), phig(*), &
             dphidx(*), psig(*), dpsidx(*), dldx(*), qdphix(*), delta, &
             work(*)
! **********************************************************************
!
!   DESCRIPTION
!
!    the element matrix due to viscous stresses is computed
!        for various models
!
!
! **********************************************************************
!
!     for variables see elp270 , elm400 , el4000
!
!     stress   element matrix to be initialised with viscous stresses
!
!     work     help array for intermediate storage of weights
!
!     nstart     number of test function to start with integration
!                (1 : navier-stokes; 7 : pressure gradient)
!
!     n          number of basis functions
!
!     lrowloc       total length of stress element matrix
!
! **********************************************************************
!
!     the matrix stress is filled only with the viscous stress part
!        if there are other variables it has to be supplemented
!
!     stress(i,j) (lrowloc*lrowloc) = 2*sum1(i,j) (n*n) + sum2 + sum3 +2*sum4
!                           (for itcoor=0)
!
!     stress(i,j) (lrowloc*lrowloc) = similar + 2*sum5 (for itcoor=1)
!
!     in   = (i-1)*n     irow  = (i-1)*lrowloc      n2=n+n
!     jn   = (j-1)*n     nln  = n*lrowloc
!                        nln1 = nln+n
! **********************************************************************
!
!   ERROR MESSAGES
!
!     None.
! **********************************************************************
!
!   SUBROUTINES  CALLED
!
!     None.
! **********************************************************************
!
!   LOCAL  VARIABLES
!
! **********************************************************************
      integer i, in, irow, i1, i2, j, jn, jrow, jrow0, j1, k, &
              len, n1m
      double precision detadx, detady, dexex, dexey, deyey, &
                       dksidx, dksidy, dkxex, dkxey, dkxkx, &
                       dkxky , dkyex , dkyey , dkyky , &
                       dpxik , dpxjk , dpyik , dpyjk , h, hq, &
                       phik, phjk, q11, q12, q21, q22, &
                       sum1, sum2, sum3, sum4, sum5, sum6, sum7, sum8, &
                       sum9, xk
      integer n2,nm,nm2,nln,nln1
      save    n2,nm,nm2,nln,nln1
! ======================================================================
      if (ifirst.eq.0) then
         n2=nloc+nloc
         nm=nloc*mloc
         nm2=nm+nm
         nln=nloc*lrowloc
         nln1=nloc*(lrowloc+1)
      end if
      if (ichvis.eq.0) then
         h=delta*etha(1)
      else
         do 1 i=1,mloc
            work(i)=etha(i)*w(i)
1        continue
      end if
      if ((ichvis.eq.0).and.(itcoor.eq.0) &
          .and.((nloc.eq.6).or.(nloc.eq.7)).and.(nloc.eq.mloc)) then
! ----------------------------------------------------------------------
!
!*    special case : newtonian liquid with constant viscosity
!     and straigth triangles for cartesian coordinates
!
!
!*    dksidx = d ksi / dx  ,  dksidy = d ksi / dy
!     detadx = d eta / dx  ,  detady = d eta / dy
!
!     dkxkx=delta*etha*dkx*dkx, etc.
!     dkxey=delta*etha*dkx*dey, etc.
!
!     matrix stress(i,j) for viscous stresses is composed of
!
!     2*etha*dphi(i)/dx * du(j)/dx  i=1,..,n;j=1,..,n
!       ==> qdphixi,j)
!
!     etha*(dphi(i)/dy+dphi(i+n)/dx) * du(j)/dy
!       ==> qdphix(nm2+i,j)+qdphix(nm+i,j)
!       (remark that phi(i+n)=phi(i) and that stress(i,j) is lrowloc*lrowloc)
!
!     etha*(dphi(i+7)/dx + dphi(i)/dy) * dv(j+7)/dx
!       ==> qdphix(i,j) + qdphix(nm+(j,i))
!       (remark that u(j+n)=v(j) --> phi(j) )
!
!     2*viscous*dphi(i+n)/dy*du(j+n)/dy
!       ==> qdphix(nm2+(i,j))
!
!     qdphix(i,j)=integr(dphi(i)/dx * dphi(j)/dx) on the reference elem.
!
         dksidx=dldx(2)
         dksidy=dldx(5)
         detadx=dldx(3)
         detady=dldx(6)
         dkxkx=h*dksidx*dksidx
         dkxky=h*dksidx*dksidy
         dkxex=h*dksidx*detadx
         dkxey=h*dksidx*detady
         dkyky=h*dksidy*dksidy
         dkyex=h*dksidy*detadx
         dkyey=h*dksidy*detady
         dexex=h*detadx*detadx
         dexey=h*detadx*detady
         deyey=h*detady*detady
         do 20 i=nstart,nloc
            in=(i-1)*mloc
            irow=(i-1)*lrowloc
            jn=0
            jrow=0
            do 10 j=1,i
               q11=qdphix(in+j)
               q12=qdphix(nm+in+j)
               q21=qdphix(nm+jn+i)
               q22=qdphix(nm2+in+j)
               hq=q12+q21
               sum1=dkxkx*q11+dkxex*hq+dexex*q22
               sum2=dkyky*q11+dkyey*hq+deyey*q22
               sum3=dkxky*q11+dexey*q22
               sum4=sum3+dkxey*q12+dkyex*q21
               sum3=sum3+dkyex*q12+dkxey*q21
               q11=2*sum1+sum2
               q22=2*sum2+sum1
               stress(irow+j)=q11
               stress(irow+j+nln1)=q22
               stress(irow+j+nloc)=sum3
               stress(irow+j+nln)=sum4
!
!*   if i.eq.j prevent refilling the matrix stress
!
               if (j.ne.i) then
                  stress(jrow+i)=q11
                  stress(jrow+i+nln1)=q22
                  stress(jrow+i+nln)=sum3
                  stress(jrow+i+nloc)=sum4
               end if
               jn=jn+mloc
               jrow=jrow+lrowloc
10          continue
20       continue
      else if (itcoor.eq.0) then
!
!*    viscosity not constant, or element is curved (itcoor=0), &
!               or element is not a triangle
!
      do 120 i=nstart,nloc
         in=(i-1)*mloc
         irow=(i-1)*lrowloc
         jn=0
         jrow=0
         do 110 j=1,i
            sum1=0.0d0
            sum2=0.0d0
            sum3=0.0d0
            sum4=0.0d0
            do 105 k=1,mloc
               dpxik=dphidx(in+k)
               dpxjk=dphidx(jn+k)
               dpyik=dphidx(nm+in+k)
               dpyjk=dphidx(nm+jn+k)
               h=work(k)
               sum1=sum1+h*dpxik*dpxjk
               sum2=sum2+h*dpyik*dpyjk
               sum3=sum3+h*dpyik*dpxjk
               sum4=sum4+h*dpxik*dpyjk
105         continue
            q11=2*sum1+sum2
            q22=2*sum2+sum1
            stress(irow+j)=q11
            stress(irow+j+nln1)=q22
            stress(irow+j+nloc)=sum3
            stress(irow+j+nln)=sum4
!
!*   if i.eq.j prevent refilling the matrix stress
!
            if (j.ne.i) then
               stress(jrow+i)=q11
               stress(jrow+i+nln1)=q22
               stress(jrow+i+nln)=sum3
               stress(jrow+i+nloc)=sum4
            end if
            jn=jn+mloc
            jrow=jrow+lrowloc
110      continue
120   continue
      else if ((ichvis.eq.1).and.((itcoor.eq.1).or.(itcoor.eq.3))) then
!
!*    axisymmetrical coordinates (no swirl part)
!
      do 220 i=nstart,nloc
         in=(i-1)*mloc
         irow=(i-1)*lrowloc
         jn=0
         jrow=0
         do 210 j=1,i
            sum1=0.0d0
            sum2=0.0d0
            sum3=0.0d0
            sum4=0.0d0
            sum5=0.0d0
            do 205 k=1,mloc
               dpxik=dphidx(in+k)
               dpxjk=dphidx(jn+k)
               dpyik=dphidx(nm+in+k)
               dpyjk=dphidx(nm+jn+k)
               h=work(k)
               sum1=sum1+h*dpxik*dpxjk
               sum2=sum2+h*dpyik*dpyjk
               sum3=sum3+h*dpyik*dpxjk
               sum4=sum4+h*dpxik*dpyjk
               sum5=sum5+h*phig(in+k)*phig(jn+k)/(xg(k)*xg(k))
205         continue
            q11=2*(sum1+sum5)+sum2
            q22=2*sum2+sum1
            stress(irow+j)=q11
            stress(irow+j+nln1)=q22
            stress(irow+j+nloc)=sum3
            stress(irow+j+nln)=sum4
!
!*   if i.eq.j prevent refilling the matrix stress
!
            if (j.ne.i) then
               stress(jrow+i)=q11
               stress(jrow+i+nln1)=q22
               stress(jrow+i+nln)=sum3
               stress(jrow+i+nloc)=sum4
            end if
            jn=jn+mloc
            jrow=jrow+lrowloc
210      continue
220   continue
      if ((itcoor.eq.3).and.(nstart.eq.1)) then
!
!*    axisymmetrical coordinates (swirl part)
!
         in=0
         irow=n2*lrowloc+n2
         jrow0=irow
         n1m=n1*mloc
         do 260 i=1,n1
         jrow=jrow0
            jn=0
            do 250 j=1,i
            sum1=0.0d0
               do 245 k=1,mloc
                  dpxik=dpsidx(in+k)
                  dpxjk=dpsidx(jn+k)
                  dpyik=dpsidx(n1m+in+k)
                  dpyjk=dpsidx(n1m+jn+k)
                  h=work(k)
                  xk=xg(k)
                  sum1=sum1+h*((dpxik-psig(in+k)/xk) &
                              *(dpxjk-psig(jn+k)/xk) &
                              +dpyik*dpyjk)
245            continue
               stress(irow+j)=sum1
!
!*   if i.eq.j prevent refilling the matrix stress
!
               if (j.ne.i) then
                  stress(jrow+i)=sum1
               end if
               jn=jn+mloc
               jrow=jrow+lrowloc
250         continue
            in=in+mloc
            irow=irow+lrowloc
260      continue
      end if
      else if ((ichvis.eq.1).and.(itcoor.eq.2)) then
!
!*    polar coordinates
!
      do 320 i=nstart,nloc
         in=(i-1)*mloc
         irow=(i-1)*lrowloc
         jn=0
         jrow=0
         do 310 j=1,i
            sum1=0.0d0
            sum2=0.0d0
            sum3=0.0d0
            sum4=0.0d0
            sum5=0.0d0
            sum6=0.0d0
            sum7=0.0d0
            sum8=0.0d0
            sum9=0.0d0
            do 305 k=1,mloc
               dpxik=dphidx(in+k)
               dpxjk=dphidx(jn+k)
               xk=xg(k)
               dpyik=dphidx(nm+in+k)/xk
               dpyjk=dphidx(nm+jn+k)/xk
               phik=phig(in+k)/xk
               phjk=phig(jn+k)/xk
               h=work(k)
               sum1=sum1+h*dpxik*dpxjk
               sum2=sum2+h*dpyik*dpyjk
               sum3=sum3+h*dpyik*dpxjk
               sum4=sum4+h*dpxik*dpyjk
               sum5=sum5+h*phik*phjk
               sum6=sum6+h*phik*dpxjk
               sum7=sum7+h*dpxik*phjk
               sum8=sum8+h*phik*dpyjk
               sum9=sum9+h*dpyik*phjk
305         continue
            q11=2*(sum1+sum5)+sum2
            q22=2*sum2+sum1+sum5-sum6-sum7
            stress(irow+j)=q11
            stress(irow+j+nln1)=q22
            stress(irow+j+nloc)=sum3+2*sum8-sum9
            stress(irow+j+nln)=sum4-sum8+2*sum9
!
!*   if i.eq.j prevent refilling the matrix stress
!
            if (j.ne.i) then
               stress(jrow+i)=q11
               stress(jrow+i+nln1)=q22
               stress(jrow+i+nln)=sum3+2*sum8-sum9
               stress(jrow+i+nloc)=sum4-sum8+2*sum9
            end if
            jn=jn+mloc
            jrow=jrow+lrowloc
310      continue
320   continue
      end if
!
!**  set elements of stress outside 2n rows of length 2n equal to 0
!                  on rows (   1 : n2   ) and colums (     n2 : lrowloc)
!
      if (nstart.ne.1) return
      len=lrowloc-n2
      if (len.eq.0) return
      irow=n2
      do 1000 i=1,n2
         do 990 j=1,len
            stress(irow+j)=0.0d0
990      continue
         irow=irow+lrowloc
1000  continue
      if (itcoor.eq.3) then
!
!*  axisymmetrical coordinates with swirl (extra zeros)
!                  on rows (n2+1 : n2+n1) and colums (      1 : n2)
!                                         and colums (n2+n1+1 : lrowloc)
!
         len=lrowloc-n2-n1
         irow=n2*lrowloc
         j1=n2+n1+1
         do 1030 i=1,nloc
            do 1020 j=1,n2
               stress(irow+j)=0.0d0
1020        continue
            if (len.gt.0) then
               do 1021 j=j1,lrowloc
                  stress(irow+j)=0.0d0
1021           continue
            end if
            irow=irow+lrowloc
1030     continue
      end if
      if (len.gt.0) then
!
!*  zeros on all columns of rows (n2+n1+1 : lrowloc)
!
         i1=(lrowloc-len)*lrowloc+1
         i2=lrowloc*lrowloc
         do 1040 i=i1,i2
            stress(i)=0.0d0
1040     continue
      end if
      end
