      subroutine el4036(itcoor,w,wh,xg,phig,dphidx,uold,duoldx, &
                        dvisd2,elmatr,ndim,nloc,mloc, &
                        psig,dpsidx,n1)
!
! =====================================================================
!        programmer   jaap van der zanden
!
!        version 4.3  date   23-12-1992 expl. decl; save common
!        version 1.2  date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 4.1  date   19-01-1987 save
!
!
!        programmer   hans damsteegt
!        version 1    date   01-10-1983 afep
!
!
!   copyright (c) 1984,1992  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from the "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! *********************************************************************
      use sepmodulecactl
      implicit none
      integer itcoor, ndim, nloc, mloc, n1
      double precision w(*), wh(*), xg(*), phig(*), uold(*), dphidx(*), &
             duoldx(*), elmatr(*), dvisd2(*), psig(*), dpsidx(*)
! *********************************************************************
!
!   DESCRIPTION
!
!     element stress matrix for generalized newtonian fluid
!
!             extra terms for newton linearization of viscosity
!
! *********************************************************************
!
!   INPUT / OUTPUT  PARAMETERS
!
!        element matrix  (2-component case)
!
!        i1            = ( i - 1 ) * nl
!        j1            = ( j - 1 ) * nl
!        (i,j)         = ( i - 1 ) * nl + j
!        (i+n,j+nloc)     = ( i - 1 ) * nl + j + nln1
!        (i+n,j)       = ( i - 1 ) * nl + j + nln
!        (i,j+nloc)       = ( i - 1 ) * nl + j + n
!
!        element matrix  is symmetrical
!
!        elmatr(j,i)     = elmatr(i,j)
!        elmatr(j+n,i+nloc) = elmatr(i+n,j+nloc)
!        elmatr(i+n,j)   = elmatr(j,i+nloc)
!        elmatr(i,j+nloc)   = elmatr(j+n,i)
!
!        interpolation polynomials and their derivatives
!
!        (i,k)        = ( i - 1 ) * m + k = ihi + k
!        (j,k)        = ( j - 1 ) * m + k = ihj + k
!
! *********************************************************************
!
!   COMMON  BLOCKS
!
! *********************************************************************
! *********************************************************************
!
!   ERROR MESSAGES
!
!     None.
! *********************************************************************
!
!   SUBROUTINES  CALLED
!
!     None.
! *********************************************************************
!
!   LOCAL  VARIABLES
!
! *********************************************************************
      integer i, ihi, ihj, i1, j, j1, k
      double precision a11, a12, a13, a22, a23, a33, &
                       de1, de2, de3, dd1, dd2, dd3, &
                       g1i, g2i, g3i, g1j, g2j, g3j, g33, &
                       g11i, g12i, g13i, g23i, g33i, &
                       g11j, g12j, g13j, g23j, g33j, sum11, sum12, &
                       sum13, sum21, sum22, sum23, sum31, sum32, sum33, &
                       whk
      integer n2,m2,m3,m4,m5,m6,m7,m8,lr,nm,n1m,n2m,nln,nln1,nln2, &
           n2ln,n2ln1,n2ln2
      save    n2,m2,m3,m4,m5,m6,m7,m8,lr,nm,n1m,n2m,nln,nln1,nln2, &
           n2ln,n2ln1,n2ln2
! =====================================================================
      do 100 k=1,mloc
         wh(k)=w(k)*dvisd2(k)
100   continue
         if (ifirst.eq.0) then
         nm=nloc*mloc
         m2=mloc+mloc
         m3=m2+mloc
         n2=nloc+nloc
         if ((ndim.eq.2).and.(itcoor.ne.3)) then
            lr=n2
         else if ((ndim.eq.2).and.(itcoor.eq.3)) then
            lr=nloc+nloc+n1
            n1m=n1*mloc
         else if (ndim.eq.3) then
            lr=nloc+nloc+nloc
            n2m=n2*mloc
         end if
         nln=nloc*lr
         nln1=nln+nloc
         if ((ndim.eq.3).or.(itcoor.eq.3)) then
            nln2=nln1+nloc
            n2ln=n2*lr
            n2ln1=n2ln+nloc
            n2ln2=n2ln+n2
            m4=m3+mloc
            m5=m4+mloc
         end if
         if (ndim.eq.3) then
            m6=m5+mloc
            m7=m6+mloc
            m8=m7+mloc
         end if
      end if
      if ((ndim.eq.2).and.(itcoor.eq.0)) then
         i1=0
         ihi=0
         do 400 i=1,nloc
            ihj=0
            j1=0
            do 300 j=1,i
               sum11=0
               sum22=0
               sum12=0
               sum21=0
               do 200 k=1,mloc
                  g1i=dphidx(ihi+k)
                  g2i=dphidx(nm+ihi+k)
                  g1j=dphidx(ihj+k)
                  g2j=dphidx(nm+ihj+k)
                  a11=2*duoldx(k)
                  a22=2*duoldx(m3+k)
                  a12=duoldx(mloc+k)+duoldx(m2+k)
                  dd1=g1i*a11+g2i*a12
                  dd2=g1i*a12+g2i*a22
                  de1=g1j*a11+g2j*a12
                  de2=g1j*a12+g2j*a22
                  whk=wh(k)
                  sum11=sum11+dd1*de1*whk
                  sum22=sum22+dd2*de2*whk
                  sum12=sum12+dd1*de2*whk
                  sum21=sum21+dd2*de1*whk
200            continue
!
!  * fill array elmatr (symmetrical)
!
               elmatr(i1+j)      =sum11
               elmatr(i1+j+nln1) =sum22
               elmatr(i1+j+nloc)    =sum12
               elmatr(i1+j+nln)  =sum21
               if (i.ne.j) then
                  elmatr(j1+i)      =sum11
                  elmatr(j1+i+nln1) =sum22
                  elmatr(j1+i+nloc)    =sum21
                  elmatr(j1+i+nln)  =sum12
               end if
               ihj=ihj+mloc
               j1=j1+lr
300         continue
            i1=i1+lr
            ihi=ihi+mloc
400      continue
      else if ((ndim.eq.3).and.(itcoor.eq.0)) then
         i1=0
         ihi=0
         do 700 i=1,nloc
            ihj=0
            j1=0
            do 600 j=1,i
               sum11=0
               sum12=0
               sum13=0
               sum21=0
               sum22=0
               sum23=0
               sum31=0
               sum32=0
               sum33=0
               do 500 k=1,mloc
                  g1i=dphidx(ihi+k)
                  g2i=dphidx(nm+ihi+k)
                  g3i=dphidx(n2m+ihi+k)
                  g1j=dphidx(ihj+k)
                  g2j=dphidx(nm+ihj+k)
                  g3j=dphidx(n2m+ihj+k)
                  a11=2*duoldx(k)
                  a22=2*duoldx(m4+k)
                  a33=2*duoldx(m8+k)
                  a12=duoldx(mloc+k)+duoldx(m3+k)
                  a13=duoldx(m2+k)+duoldx(m6+k)
                  a23=duoldx(m5+k)+duoldx(m7+k)
                  dd1=g1i*a11+g2i*a12+g3i*a13
                  dd2=g1i*a12+g2i*a22+g3i*a23
                  dd3=g1i*a13+g2i*a23+g3i*a33
                  de1=g1j*a11+g2j*a12+g3j*a13
                  de2=g1j*a12+g2j*a22+g3j*a23
                  de3=g1j*a13+g2j*a23+g3j*a33
                  whk=wh(k)
                  sum11=sum11+dd1*de1*whk
                  sum12=sum12+dd1*de2*whk
                  sum13=sum13+dd1*de3*whk
                  sum21=sum21+dd2*de1*whk
                  sum22=sum22+dd2*de2*whk
                  sum23=sum23+dd2*de3*whk
                  sum31=sum31+dd3*de1*whk
                  sum32=sum32+dd3*de2*whk
                  sum33=sum33+dd3*de3*whk
500            continue
!
!  * fill array elmatr (symmetrical)
!
               elmatr(i1+j)      =sum11
               elmatr(i1+j+nloc)    =sum12
               elmatr(i1+j+n2)   =sum13
               elmatr(i1+j+nln)  =sum21
               elmatr(i1+j+nln1) =sum22
               elmatr(i1+j+nln2) =sum23
               elmatr(i1+j+n2ln) =sum31
               elmatr(i1+j+n2ln1)=sum32
               elmatr(i1+j+n2ln2)=sum33
               if (i.ne.j) then
                  elmatr(j1+i)      =sum11
                  elmatr(j1+i+nloc)    =sum12
                  elmatr(j1+i+n2)   =sum13
                  elmatr(j1+i+nln)  =sum21
                  elmatr(j1+i+nln1) =sum22
                  elmatr(j1+i+nln2) =sum23
                  elmatr(j1+i+n2ln) =sum31
                  elmatr(j1+i+n2ln1)=sum32
                  elmatr(j1+i+n2ln2)=sum33
               end if
               ihj=ihj+mloc
               j1=j1+lr
600         continue
            i1=i1+lr
            ihi=ihi+mloc
700      continue
      else if (itcoor.eq.1) then
!
!  * compute terms of matrix for axisymmetric coordinates (no swirl)
!
         i1=0
         ihi=0
         do 1030 i=1,nloc
            ihj=0
            j1=0
            do 1020 j=1,i
               sum11=0
               sum22=0
               sum12=0
               sum21=0
               do 1010 k=1,mloc
                  g1i=dphidx(ihi+k)
                  g2i=dphidx(nm+ihi+k)
                  g1j=dphidx(ihj+k)
                  g2j=dphidx(nm+ihj+k)
                  a11=2*duoldx(k)
                  a22=2*duoldx(m3+k)
                  a12=duoldx(mloc+k)+duoldx(m2+k)
                  g33=2*uold(k)/(xg(k)**2)
                  dd1=g1i*a11+g2i*a12+phig(ihi+k)*g33
                  dd2=g2i*a22+g1i*a12
                  de1=g1j*a11+g2j*a12+phig(ihj+k)*g33
                  de2=g2j*a22+g1j*a12
                  whk=wh(k)
                  sum11=sum11+dd1*de1*whk
                  sum22=sum22+dd2*de2*whk
                  sum12=sum12+dd1*de2*whk
                  sum21=sum21+dd2*de1*whk
1010           continue
!
!  * fill array elmatr
!
               elmatr(i1+j)      =sum11
               elmatr(i1+j+nln1) =sum22
               elmatr(i1+j+nloc)=sum12
               elmatr(i1+j+nln)  =sum21
               if (i.ne.j) then
                  elmatr(j1+i)      =sum11
                  elmatr(j1+i+nln1) =sum22
                  elmatr(j1+i+nloc)=sum21
                  elmatr(j1+i+nln)  =sum12
               end if
               ihj=ihj+mloc
               j1=j1+lr
1020        continue
            i1=i1+lr
            ihi=ihi+mloc
1030     continue
      else if (itcoor.eq.3) then
!
!*  axisymmetrical coordinates with swirl
!
         i1=0
         ihi=0
         do 1060 i=1,nloc
            ihj=0
            j1=0
            do 1050 j=1,i
               sum11=0
               sum12=0
               sum13=0
               sum21=0
               sum22=0
               sum23=0
               sum31=0
               sum32=0
               sum33=0
               do 1040 k=1,mloc
                  g11i=dphidx(ihi+k)
                  g12i=dphidx(nm+ihi+k)
                  g33i=phig(ihi+k)
                  g11j=dphidx(ihj+k)
                  g12j=dphidx(nm+ihj+k)
                  g33j=phig(ihj+k)
                  a11=2*duoldx(k)
                  a12=duoldx(mloc+k)+duoldx(m2+k)
                  a22=2*duoldx(m3+k)
                  a33=2*uold(k)/xg(k)
                  dd1=g11i*a11+g12i*a12+g33i*a33
                  dd2=g11i*a12+g12i*a22
                  de1=g11j*a11+g12j*a12+g33j*a33
                  de2=g11j*a12+g12j*a22
                  whk=wh(k)
                  sum11=sum11+dd1*de1*whk
                  sum12=sum12+dd1*de2*whk
                  sum21=sum21+dd2*de1*whk
                  sum22=sum22+dd2*de2*whk
                  if (i.le.n1) then
                     g13i=dpsidx(ihi+k)-psig(ihi+k)/xg(k)
                     g23i=dpsidx(n1m+ihi+k)
                     g13j=dpsidx(ihj+k)-psig(ihj+k)/xg(k)
                     g23j=dpsidx(n1m+ihj+k)
                     a13=duoldx(m4+k)-uold(k)/xg(k)
                     a23=duoldx(m5+k)
                     dd3=g13i*a13+g23i*a23
                     de3=g13j*a13+g23j*a23
                     sum13=sum13+dd1*de3*whk
                     sum23=sum23+dd2*de3*whk
                     sum31=sum31+dd3*de1*whk
                     sum32=sum32+dd3*de2*whk
                     sum33=sum33+dd3*de3*whk
                  end if
1040           continue
!
!  * fill array elmatr (symmetrical)
!
               elmatr(i1+j)      =sum11
               elmatr(i1+j+nloc)    =sum12
               elmatr(i1+j+nln)  =sum21
               elmatr(i1+j+nln1) =sum22
               if (i.le.n1) then
                  elmatr(i1+j+n2)   =sum13
                  elmatr(i1+j+nln2) =sum23
                  elmatr(i1+j+n2ln) =sum31
                  elmatr(i1+j+n2ln1)=sum32
                  elmatr(i1+j+n2ln2)=sum33
               end if
               if (i.ne.j) then
                  elmatr(j1+i)      =sum11
                  elmatr(j1+i+nloc)    =sum12
                  elmatr(j1+i+nln)  =sum21
                  elmatr(j1+i+nln1) =sum22
                  if (i.le.n1) then
                     elmatr(j1+i+n2)   =sum13
                     elmatr(j1+i+nln2) =sum23
                     elmatr(j1+i+n2ln) =sum31
                     elmatr(j1+i+n2ln1)=sum32
                     elmatr(j1+i+n2ln2)=sum33
                  end if
               end if
               ihj=ihj+mloc
               j1=j1+lr
1050         continue
            i1=i1+lr
            ihi=ihi+mloc
1060      continue
      end if
      end
