      subroutine el4039(ichois,itcoor,ip,mloc,ndim, &
                        xg,u,dudx,secinv,gradv)
!
! ======================================================================
!
!        programmer   jaap van der zanden
!        version 1.3  date   23-12-1992 remove name; expl. declarations
!        version 1.2  date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 1.1  date   15-01-1987 save
!
!
!   copyright (c) 1986,1992  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!   DESCRIPTION
!
!     compute gradient of velocity
!     for ichois=1 also second invariant
!
! **********************************************************************
!
!   KEYWORDS
!
! *********************************************************************
      use sepmodulecomio
      use sepmodulecactl
      implicit none
      integer ichois, itcoor, ip, mloc, ndim
      double precision xg(*), u(*), dudx(*), gradv(*)
! **********************************************************************
!
!   INPUT / OUTPUT  PARAMETERS
!
! **********************************************************************
!
!   COMMON  BLOCKS
!
! *********************************************************************
!
!  LOCAL  VARIABLES
!
      double precision secinv, xgk
      integer m2,m3,m4,m5,m6,m7,m8
      save    m2,m3,m4,m5,m6,m7,m8
! *********************************************************************
!
!   SUBROUTINES  CALLED
!
!   none
!
! *********************************************************************
!
!   I / O
!
!   none
!
! *********************************************************************
!
!   ERROR  MESSAGES
!
!   none
!
! *********************************************************************
!
!   PSEUDO  CODE
!
!   trivial
!
! =====================================================================
      if (ifirst.eq.0) then
         m2=mloc+mloc
         m3=m2+mloc
         if (itcoor.eq.3) then
            m4=m3+mloc
            m5=m4+mloc
         else if (ndim.eq.3) then
            m4=m3+mloc
            m5=m4+mloc
            m6=m5+mloc
            m7=m6+mloc
            m8=m7+mloc
         end if
      end if
      if ((itcoor.eq.0).and.(ndim.eq.2)) then
!
!*   2-dim cartesian coordinates
!
         gradv(1)=dudx(ip)
         gradv(2)=dudx(m2+ip)
         gradv(3)=dudx(mloc+ip)
         gradv(4)=dudx(m3+ip)
         if (ichois.eq.1) &
            secinv=2*(gradv(1)*gradv(1)+gradv(4)*gradv(4)) &
                     +(gradv(2)+gradv(3))*(gradv(2)+gradv(3))
      else if ((itcoor.eq.1).and.(ndim.eq.2)) then
!
!*   axisymmetrical coordinates (no swirl)
!
         gradv(1)=dudx(ip)
         gradv(2)=dudx(m2+ip)
         gradv(3)=dudx(mloc+ip)
         gradv(4)=dudx(m3+ip)
         xgk=xg(ip)
         if (abs(xgk).gt.epsmac) then
            gradv(5)=u(ip)/xgk
         else
            gradv(5)=0.0d0
         end if
         if (ichois.eq.1) &
            secinv=2*(gradv(1)*gradv(1)+gradv(4)*gradv(4) &
                     +gradv(5)*gradv(5)) &
                    +(gradv(2)+gradv(3))*(gradv(2)+gradv(3))
      else if ((itcoor.eq.2).and.(ndim.eq.2)) then
!
!*   2-dim polar coordinates
!
         xgk=xg(ip)
         gradv(1)=dudx(ip)
         gradv(2)=dudx(m2+ip)
         if (abs(xgk).gt.epsmac) then
            gradv(3)=(dudx(mloc+ip)-u(mloc+ip))/xgk
            gradv(4)=(dudx(m3+ip)+u(ip))/xgk
         else
            gradv(3)=0.0d0
            gradv(4)=0.0d0
         end if
         if (ichois.eq.1) &
               secinv=2*(gradv(1)*gradv(1)+gradv(4)*gradv(4)) &
                         +(gradv(2)+gradv(3))*(gradv(2)+gradv(3))
      else if ((itcoor.eq.3).and.(ndim.eq.2)) then
!
!*   axisymmetrical coordinates (with swirl)
!
         xgk=xg(ip)
         gradv(1)=dudx(ip)
         gradv(2)=dudx(m4+ip)
         gradv(3)=dudx(m2+ip)
         if (abs(xgk).gt.epsmac) then
            gradv(4)=-u(m2+ip)/xgk
            gradv(5)=u(ip)/xgk
         else
            gradv(4)=0.0d0
            gradv(5)=0.0d0
         end if
         gradv(6)=0.0d0
         gradv(7)=dudx(mloc+ip)
         gradv(8)=dudx(m5+ip)
         gradv(9)=dudx(m3+ip)
         if (ichois.eq.1) &
            secinv=2*(gradv(1)*gradv(1)+gradv(5)*gradv(5) &
                     +gradv(9)*gradv(9)) &
                    +(gradv(2)+gradv(4))*(gradv(2)+gradv(4)) &
                    +(gradv(3)+gradv(7))*(gradv(3)+gradv(7)) &
                    + gradv(8)*gradv(8)
      else if ((itcoor.eq.0).and.(ndim.eq.3)) then
!
!*   3-dim cartesian coordinates
!
         gradv(1)=dudx(ip)
         gradv(2)=dudx(m3+ip)
         gradv(3)=dudx(m6+ip)
         gradv(4)=dudx(mloc+ip)
         gradv(5)=dudx(m4+ip)
         gradv(6)=dudx(m7+ip)
         gradv(7)=dudx(m2+ip)
         gradv(8)=dudx(m5+ip)
         gradv(9)=dudx(m8+ip)
         if (ichois.eq.1) &
            secinv=2*(gradv(1)*gradv(1)+gradv(5)*gradv(5) &
                     +gradv(9)*gradv(9)) &
                    +(gradv(2)+gradv(4))*(gradv(2)+gradv(4)) &
                    +(gradv(3)+gradv(7))*(gradv(3)+gradv(7)) &
                    +(gradv(6)+gradv(8))*(gradv(6)+gradv(8))
      else if ((itcoor.eq.4).and.(ndim.eq.3)) then
!
!*   3-dim cylindrical coordinates
!
         xgk=xg(ip)
         gradv(1)=dudx(ip)
         gradv(2)=dudx(m3+ip)
         gradv(3)=dudx(m6+ip)
         if (abs(xgk).gt.epsmac) then
            gradv(4)=(dudx(mloc+ip)-u(mloc+ip))/xgk
            gradv(5)=(dudx(m4+ip)+u(ip))/xgk
            gradv(6)=dudx(m7+ip)/xgk
         else
            gradv(4)=0.0d0
            gradv(5)=0.0d0
            gradv(6)=0.0d0
         end if
         gradv(7)=dudx(m2+ip)
         gradv(8)=dudx(m5+ip)
         gradv(9)=dudx(m8+ip)
         if (ichois.eq.1) &
            secinv=2*(gradv(1)*gradv(1)+gradv(5)*gradv(5) &
                     +gradv(9)*gradv(9)) &
                    +(gradv(2)+gradv(4))*(gradv(2)+gradv(4)) &
                    +(gradv(3)+gradv(7))*(gradv(3)+gradv(7)) &
                    +(gradv(6)+gradv(8))*(gradv(6)+gradv(8))
      end if
      end
