      subroutine el4043(ichois,itcoor,ip,mloc,ndim, &
                        modelv,etha,cn,cl,ct,temp1,param, &
                        xg,u,dudx,viscos,gradv,derivq)
!
! =====================================================================
!
!   programmer   jaap van der zanden
!
!   version 2.4  date   22-11-1992 remove name, secinv
!   version 2.3  date   22-08-1991 Zdenek: no dimension, cdc, ce
!
!
!   copyright (c) 1985,1992  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
      use sepmodulecomio
      use sepmodulecactl
      implicit none
      integer ichois,itcoor,ip,mloc,ndim,modelv
      double precision cn, cl, ct, temp1, viscos, etha(*), &
             param(*), xg(*), u(*), dudx(*), &
             gradv(*), derivq(*)
! **********************************************************************
!
!   DESCRIPTION
!
!     this subroutine tests and computes model-dependent
!     quantities for a generalized newtonian liquid
!                    and plastico-viscous liquids
!                    in nodal point ip (.le.m)
!
! **********************************************************************
!
!   KEYWORDS
!
! **********************************************************************
!
!   INPUT / OUTPUT  PARAMETERS
!
!     ichois=1 extra stress tensor
!            2 gradient of velocity tensor
!            3 elongation
!            4 deformation
!            5 sqrt(secinv)
!            6 gradient of temperature
!
!     available are :  modelv=1, newtonian model
!                      modelv=2, power law model
!                      modelv=3, 3 parameter carreau model
!                      modelv=4, 3 parameter plastico-viscous liquid
!                               cn=1 : bingham model
!                               cn=2 : casson model
!                      modelv=100 , user defined effective viscosity
!                                   based on gradv,temp
!                      modelv=101 , user defined effective viscosity
!                                   based on second invariant,temp
!                      modelv=102 , user defined effective viscosity
!                                   based on x,u,second invariant,temp
!
!
!     etha(i)   viscosity parameter etha0 in integration points
!     cn        power (law model) parameter n
!     cl        carreau model parameter lambda, or
!               bingham (plastico-viscous) model parameter s
!     ct,temp1  parameters for temperature dependence
!     viscos    computed effective viscosity
!     gradv     gradient of velocity tensor in ip
!               ichois=1-4  gradient of velocity is computed
!     derivq    ichois=1 computed stress tensor in ip
!                     =2 gradient of velocity in ip
!                     =3 elongation in ip
!                     =4 shear rate in ip (vector in 3-dim or swirl)
!                     =5 sqrt(secinv)
!                     =6 temperature gradient in ip
!
!
!**********************************************************************
!
!    COMMON  BLOCKS
!
! ********************************************************************
!
!    LOCAL  VARIABLES
!
      integer ich44,i,it,m2
      double precision u1, u2, u3, visch(9), sech(9), detdsc(9)
      save m2
! **********************************************************************
!
!   SUBROUTINES  CALLED
!
!   EL4040  Compute viscosity for a non-isothermal newtonian liquid
!   ERCLOS  reset old name of calling tree
!   EROPEN  add name of routine to calling tree
!
! **********************************************************************
!
!   I / O
!
!   none
!
! **********************************************************************
!
!   ERROR  MESSAGES
!
! **********************************************************************
!
!   PSEUDO  CODE
!
!   trivial
!
! =====================================================================
!
!*  computation of grad(v) for all values of 0<ichois<6
!
      if (ifirst.eq.0) then
         m2=mloc+mloc
         if (itcoor.eq.3) then
         else if (ndim.eq.3) then
         end if
      end if
      call eropen('el4043')
      if ((ichois.gt.0).and.(ichois.lt.6)) then
         if (ichois.eq.2) then
            ich44=11
            call el4040(ich44,itcoor,ip,mloc,ndim,modelv,etha,cn,cl, &
                        ct,temp1,param,xg,u,dudx,visch,derivq, &
                        sech,detdsc)
            viscos=visch(ip)
            call erclos('el4043')
            return
         else
            ich44=21
            if (ichois.eq.1) ich44=11
            call el4040(ich44,itcoor,ip,mloc,ndim,modelv,etha,cn,cl, &
                        ct,temp1,param,xg,u,dudx,visch,gradv, &
                        sech,detdsc)
            viscos=visch(ip)
            if (ichois.eq.5) then
               derivq(1)=sqrt(sech(ip))
               call erclos('el4043')
               return
            end if
         end if
      end if
      if (ichois.eq.1) then
!
!*  compute stress tensor in ip
!
         if (((itcoor.eq.0).or.(itcoor.eq.2)).and.(ndim.eq.2)) then
            derivq(1)=2*viscos*gradv(1)
            derivq(2)=viscos*(gradv(2)+gradv(3))
            derivq(3)=2*viscos*gradv(4)
         else if ((itcoor.eq.1).and.(ndim.eq.2)) then
            derivq(1)=2*viscos*gradv(1)
            derivq(2)=viscos*(gradv(2)+gradv(3))
            derivq(3)=2*viscos*gradv(4)
            derivq(4)=2*viscos*gradv(5)
         else if (((itcoor.eq.3).and.(ndim.eq.2)).or.(ndim.eq.3)) then
            derivq(1)=2*viscos*gradv(1)
            derivq(2)=viscos*(gradv(2)+gradv(4))
            derivq(3)=viscos*(gradv(3)+gradv(7))
            derivq(4)=2*viscos*gradv(5)
            derivq(5)=viscos*(gradv(6)+gradv(8))
            derivq(6)=2*viscos*gradv(9)
         end if
      else if (ichois.eq.2) then
         if (( itcoor.eq.0 .or. itcoor.eq.2) .and. ndim.eq.2 ) then
            it=4
         else if ( itcoor.eq.1 .and. ndim.eq.2 ) then
            it=5
         else if ( itcoor.eq.3 .or. ndim.eq.3 ) then
            it=9
         end if
         do 10 i=1,it
            derivq(i)=gradv(i)
10       continue
      else if (ichois.eq.3) then
!
!*  elongation
!
         u1=u(ip)
         u2=u(ip+mloc)
         if ((ndim.eq.2).and.(itcoor.ne.3)) then
            if ((abs(u1).le.sqreps).and.(abs(u2).le.sqreps)) then
               derivq(1)=0.0d0
            else
               derivq(1)=(u1*gradv(1)*u1 &
                         +u1*(gradv(2)+gradv(3))*u2 &
                         +u2*gradv(4)*u2)/(u1*u1+u2*u2)
            end if
         else if ((ndim.eq.3).or.(itcoor.eq.3)) then
            if (ndim.eq.3) then
               u3=u(ip+m2)
            else if ((ndim.eq.2).and.(itcoor.eq.3)) then
               u2=u(ip+m2)
               u3=u(ip+mloc)
            end if
            if ((abs(u1).le.sqreps).and.(abs(u2).le.sqreps).and. &
                (abs(u3).le.sqreps)) then
               derivq(1)=0.0d0
            else
               derivq(1)=(u1*gradv(1)*u1 &
                         +u1*(gradv(2)+gradv(4))*u2 &
                         +u1*(gradv(3)+gradv(7))*u3 &
                         +u2*gradv(5)*u2 &
                         +u2*(gradv(6)+gradv(8))*u3 &
                         +u3*gradv(9)*u3)/(u1*u1+u2*u2+u3*u3)
            end if
         end if
      else if (ichois.eq.4) then
!
!*  deformation
!
         if ((ndim.eq.2).and.(itcoor.ne.3)) then
            u1=u(ip)
            u2=u(ip+mloc)
            if ((abs(u1).lt.sqreps).and.(abs(u2).lt.sqreps)) then
!
!*  zero velocity, take velocity in centroid for directions
!
               u1=u(mloc)
               u2=u(mloc+mloc)
               if ((abs(u1).lt.sqreps).and.(abs(u2).lt.sqreps)) then
                  derivq(1)=0.0d0
               else
                  derivq(1)=(2*u1*u2*(-gradv(1)+gradv(4)) &
                            +(u1*u1-u2*u2)*(gradv(2)+gradv(3))) &
                            /(u1*u1+u2*u2)
               end if
            else
               derivq(1)=(2*u1*u2*(-gradv(1)+gradv(4)) &
                         +(u1*u1-u2*u2)*(gradv(2)+gradv(3))) &
                         /(u1*u1+u2*u2)
            end if
         else if ((ndim.eq.3).or.(itcoor.eq.3)) then
            derivq(1)=gradv(2)+gradv(4)
            derivq(2)=gradv(3)+gradv(7)
            derivq(3)=gradv(6)+gradv(8)
         end if
      else if (ichois.eq.6) then
         it=ndim*ndim*mloc
         if (itcoor.eq.3) it=it+mloc+mloc
         derivq(1)=dudx(it+ip)
         if ((itcoor.eq.2).or.(itcoor.eq.4)) then
            if (xg(ip).gt.0.0d0) then
               derivq(2)=dudx(it+mloc+ip)/xg(ip)
            else
               derivq(2)=0.0d0
            end if
         else
            derivq(2)=dudx(it+mloc+ip)
         end if
         if (ndim.eq.3) derivq(3)=dudx(it+mloc+mloc+ip)
      end if
      call erclos('el4043')
      end
