      subroutine el8204 (s, w, psi, Ni, ci, Jd, Jn, work, ncoef, npres)
! ======================================================================
!
!        programmer    Raoul van Loon/Chris van Oijen
!        version  1.1  date 19-05-2003  Adjustment for dWdN(2)
!        version  1.0  date 29-05-2000
!
!   copyright (c) 1993-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the part of the element matrix, that is correlated to
!     the fluid flux out and into the element and its right hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoef, npres
      double precision s(npres,npres,6), w(*), ci(3,*), Ni(3,*), Jd(*), &
                       Jn(*), psi(npres,m), work(*)

!     ci             o    Array of 3 x m long that contains the concentrations.
!                         ci(1,i) fixed charge concentration
!                         ci(2,i) cation concentration
!                         ci(3,i) anion concentration
!     jd             i    Determinant of deformation gradient
!     jn             i    Determinant of deformation gradient at incr. n
!     ncoef          i    Number of coefficients in the equation
!     ni             i    Upper boundary in the computation for the loop
!                         variable i
!     npres          i    Number of pressure parameters per element
!     psi            i    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     s              o    Element matrix to be filled
!     w              i    array of length m containing the weights for
!                         integration
!     work           i    Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, maxcoef
      parameter (maxcoef=50)
      double precision coef(maxcoef)
      double precision dWdN(6), Cab(6), Vp, Vm, R, Temp, Gamma

!     cab            Inverse of dWdN
!     coef           Element coefficients
!     dwdn           Hessian of density energy function W
!     gamma          Osmotic coefficient
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     maxcoef        Number of coefficients
!     r              Gas constant
!     temp           Temperature
!     vm             Molar volume of anions
!     vp             Molar volume of cations
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELSYMINV       Compute the inverse of a symmetric stored 3 x 3 matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear the matrix

      do k = 1, 6
         do i = 1, npres
            do j = 1, npres
               s(i,j,k) = 0d0
            end do
         end do
      end do

!     --- Clear the matrix

      do i = 1, 6
         Cab(i) = 0d0
      end do

!     --- Loop over the integration points

      do k = 1, m

!     --- Get coefficients

         do i = 1, ncoef
            coef(i) = work(1+m*(i-1))
         end do
         Vp = coef(31-5)
         Vm = coef(32-5)
         Gamma = coef(36-5)
         Temp = coef(42-5)
         R = coef(43-5)

!        --- Fill in Cab
!            The matrix is stored as a symmetric matrix with 6
!            components

         dWdN(1) = R*Temp*Gamma*(ci(2,k)+ci(3,k))/Ni(1,k)
         dWdN(2) = (R*Temp)/(Vp*Ni(2,k))
         dWdN(3) = (R*Temp)/(Vm*Ni(3,k))
         dWdN(4) = -(R*Temp*Gamma)/(Vp*Ni(1,k))
         dWdN(5) = 0d0
         dWdN(6) = -(R*Temp*Gamma)/(Vm*Ni(1,k))
         call elsyminv (dWdN, Cab)

!        --- T
!            --- assemble psi * (1/J) * Cab * psi

         do l = 1, 6
            do i = 1, npres
               do j = 1, npres
                  s(j,i,l) = s(j,i,l) + w(k)*(1d0/(Jd(k)*Jn(k)))* &
                             psi(j,k)*Cab(l)*psi(i,k)
               end do
            end do
         end do

      end do

!     --- do k =1 , 6
!         do i = 1, npres
!            write(*,*) (s(i,j,k), j=1,npres)
!         end do
!         write(*,*)
!         end do

      end
