      subroutine el8210 ( rhsd, xg, x0, w, Fd, Jd, Fn, Jn, p, B, usrvec, &
                          nd, imatlaw, iusrvc, iusrflg, work, xg0, &
                          ncoef, index1, nsig, ichfib, maxusr )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 07-07-2007 Extra parameters ichfib, maxusr
!        version  2.0  date 13-04-2005 Extra parameters for axi-symmetry (RvL)
!                                      extra parameter for el8300
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   compute the stress part of the element vector for general nonlinear
!   solid element
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imatlaw, iusrvc, iusrflg, nd, ncoef, index1(*), nsig, &
              ichfib, maxusr
      double precision B(nd,ndim,n,m), w(m), rhsd(ndim,n), Jd(m), &
                       Fd(m,ndim,ndim),  Fn(m,ndim,ndim), Jn(m), &
                       p(m), xg(m,ndim), x0(ndim,m), work(*), &
                       usrvec(m,*), xg0(m,ndim)

!     b              i    Array containing the B-matrix
!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             i    Deformation gradient at incr. n Fn(k,j,i) = F_ij(x^k)
!     imatlaw        i    Type of material constitutive equation to be used
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     iusrflg        i    ?
!     iusrvc         i    if>0, a user defined vector with iusrvec dof's
!                         present
!     jd             i    Determinant of deformation gradient
!     jn             i    Determinant of deformation gradient at incr. n
!     ncoef          i    Number of coefficients in the equation
!     nd             i    Dimension of tangent stiffness matrix
!     nsig           i    ?
!     p              i    Array containing the pressure
!     rhsd           o    Element vector to be filled
!     usrvec         i    User vector
!     w              i    array of length m containing the weights for
!                         integration
!     work           i    Work array
!     x0             i    coordinates of undeformed mesh
!     xg             i    coordinates of integration points
!     xg0            i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, maxcoef
      parameter (maxcoef=50)
      double precision sigma(nd,n), s_wrk(nsig), &
                       Fn_wrk(ndim,ndim), D_t(81), coef(maxcoef), &
                       Fd_wrk(ndim, ndim), corr0(ndim), corr(ndim), &
                       usr_wrk(maxusr), corrg0(ndim)

!     coef           Local coefficient array
!     corr           coordinates of current integration point
!     corr0          coordinates in undeformed configuration
!     corrg0         ?
!     d_t            Tangent stiffness matrix material part
!     fd_wrk         F_delta work array
!     fn_wrk         F_n work array
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     maxcoef        Maximum number of coefficients
!     s_wrk          Stress work array
!     sigma          Stress
!     usr_wrk        Uservector work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL8218         Compute element vector from -B' * sigma
!     EL8300         fills extra stress tensor and tangential stiffness matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Loop over the integration points

      do k = 1, m

!     --- compute coordinates of current integration point

         do i = 1, ndim
            corr0(i) = x0(i,k)
            corrg0(i) = xg0(k,i)
            corr(i) = xg(k,i)
         end do

!        --- compute deformation gradient in current integration point

         do j = 1, ndim
            do i = 1, ndim
               Fd_wrk(i,j) = Fd(k,j,i)
               Fn_wrk(i,j) = Fn(k,j,i)
            end do
         end do

!        --- compute user vector in current integration point

         if (iusrvc>=1) then
            do i = 1, iusrvc
               usr_wrk(i) = usrvec(k,i)
            end do
         end if

!        --- get coefficients in current integration point

         do i = 1, ncoef
            coef(i) = work(k+m*(i-1))
         end do

!        --- compute sigma

         call el8300 ( Fd_wrk, Jd(k), Fn_wrk, Jn(k), p(k), usr_wrk, &
                       coef, D_t, s_wrk, corr0, corr, corrg0, index1(k), &
                       imatlaw, iusrvc, iusrflg, nd, 1, isubr, ichfib )

        if (ndim==2) then

           sigma(1,k)=s_wrk(1)
           sigma(2,k)=s_wrk(2)
           sigma(3,k)=s_wrk(3)
           sigma(4,k)=s_wrk(3)

!          --- axisymmetric case

           if (jcart==2) then
              sigma(5,k)=s_wrk(4)
           end if

        else if (ndim==3) then

           do i = 1, 9
              if (i>6) then
                 sigma(i,k) = s_wrk(i-3)
              else
                 sigma(i,k) = s_wrk(i)
              end if
           end do

        end if

      end do

!     --- assemble element vector  -B^T  * sigma

      call el8218 (rhsd, B, sigma, w, nd)

      end
