      subroutine el8213 (f, w, psi, Ni, Nn, Jd, Jn, npres)
! ======================================================================
!
!        programmer    Raoul van Loon/Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 19-05-2003  Different shapes possible
!        version  1.0  date 29-05-2000
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the part of the element matrix, that is correlated to
!     the fluid flux out and into the element and its right hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres
      double precision f(3,*), w(*), psi(npres,m), Ni(3,*), Nn(3,*), &
                       Jd(*), Jn(*)

!     f              o    Right hand side part correlated to the fluxes
!     jd             i    Determinant of deformation gradient
!     jn             i    Determinant of deformation gradient at incr. n
!     ni             i    Volume fractions in the nodal points
!                         Ni(1,i) Volume fraction of the fluid
!                         Ni(2,i) Volume fraction of the cations
!                         Ni(3,i) Volume fraction of the anions
!     nn             i    Old volume fractions in the nodal points
!                         Nn(1,i) Volume fraction of the fluid
!                         Nn(2,i) Volume fraction of the cations
!                         Nn(3,i) Volume fraction of the anions
!     npres          i    Number of pressure parameters per element
!     psi            i    basis functions for the pressure
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, idx(8)

!     i              Counting variable
!     idx            Indices of vertices
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1, 3, 7, 9, 19, 21, 25, 27 /
! ======================================================================
!
!     --- Loop over the integration points

      if (ishape.eq.4.or.ishape.eq.6.or.ishape.eq.7) then
         idx(1)=1
         idx(2)=3
         idx(3)=5
         idx(4)=7
      end if
      do k = 1, m
         do i = 1, npres
            do j = 1, npres
               do l = 1, 3
                  f(l,i) = f(l,i) + w(k)*(1d0/(Jd(k)*Jn(k)))* &
                                    psi(i,k)*psi(j,k)* &
                                    (Ni(l,idx(j)) - Nn(l,idx(j)))
               end do
            end do
         end do
      end do

      end
