      subroutine el8232 (elemmt, s, div, ms, isequ, iseqp, npres)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 16-11-2000
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Assemble element stiffness matrix for general nonlinear solid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres, isequ(*), iseqp(*)
      double precision elemmt(icount,icount), s(ndim*n,ndim*n), &
                       div(ndim*n,npres), ms(npres,npres)

!     div     i    Divergence matrix
!                  div(i,k) corresponds to the k-th row and i-th column
!                  of the matrix, The k refers to the basis functions of the
!                  pressure, i to the basis functions of the velocity in the
!                  sequence of the solution vector
!     elemmt  o    Element matrix to be filled
!                  Only the part corresponding to div and grad is filled
!     iseqp   i    Sequence for pressure dof's
!     isequ   i    Sequence for displacement dof's
!     ms      i    Mass matrix
!     npres   i    Number of pressure unknowns
!     s       i    Element matrix without the gradient and divergence part
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i       Counting variable
!     j       Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The element matrix has the following structure
!         | Suu  Sup |
!     S = |          |
!         | Spu   M  |
!     with Suu the material stiffness matrix and Sup and Spu
!     the divergence matrix
!     Depending of the element type the submatrices are filled
!     corresponding to the correct dof.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill material stiffness part  Suu

      do i = 1, ndim*n
         do j = 1, ndim*n
            elemmt(isequ(i),isequ(j)) = s(i,j)
         end do
      end do

!     --- Fill Sup

      do i = 1, npres
         do j = 1, ndim*n
            elemmt(isequ(j),iseqp(i)) = -div(j,i)
         end do
      end do

!     --- Fill Spu

      do i = 1, npres
         do j = 1, ndim*n
            elemmt(iseqp(i),isequ(j)) = -div(j,i)
         end do
      end do

!     --- Fill M

      do i = 1, npres
         do j = 1, npres
            elemmt(iseqp(i),iseqp(j)) = -ms(i,j)
         end do
      end do

      end
