      subroutine el8235 (elemvc, R, U, Q, T, z, isequ, iseqp, &
                         iseqmu, iseqksi, npres)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 19-05-2003  Some adjustments (RvL)
!        version  1.0  date 16-10-2000  Derived from el1006
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Assemble element vector for general nonlinear solid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres, isequ(*), iseqp(*), iseqmu(3,*), iseqksi(*)
      double precision elemvc(*), R(*), U(*), Q(3,*), &
                       T(3,*), z(*)

!     elemvc         o    Element vector
!     iseqksi        i    Sequence for electric potential dof's
!     iseqmu         i    Sequence for the chemical potential dof's
!                         Per nodal point the sequence number is given for
!                         1 the fixed charge density, &
!                         2 the cation concentration, &
!                         3 the antion concentration
!     iseqp          i    Sequence for pressure dof's
!     isequ          i    Sequence for displacement dof's
!     npres          i    Number of pressure parameters per element
!     q              i    Right hand side connected to the flux part
!     r              i    Right hand side connected to the mechanical part
!     t              i    Right hand side connected to the mass balance
!     u              i    Right hand side connected to the saturation part
!     z              i    Help array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k

!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The element vector has the following structure
!
!         | fu |
!     f = |    |
!         | fp |
!
!     with fu the material rhs and fp the divergence rhs.
!
!     Depending of the element type the submatrices are filled
!     corresponding to the correct dof.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill in  R

      do i = 1, ndim*n
         elemvc(isequ(i)) = -R(i)
      end do

!     --- Fill in U - sum(Q)

      do i = 1, npres
         elemvc(iseqp(i)) =  U(i)
         do k = 1, 3
            elemvc(iseqp(i)) = elemvc(iseqp(i)) - Q(k,i)
         end do
      end do

!     --- Fill in Q + sum(T)

      do i = 1, npres
         do k = 1, 3
            elemvc(iseqmu(k,i)) = Q(k,i) + T(k,i)
         end do
      end do

!     --- Fill in -sum(z*Q)

      do i = 1, npres
         do k = 1, 3
            elemvc(iseqksi(i)) = elemvc(iseqksi(i)) - z(k)*Q(k,i)
         end do
      end do

      do i=1, icount
         elemvc(i)=-elemvc(i)
      end do

      end
