      subroutine el8249 ( F_new, F, Fn, xg0, xg, nrow )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 31-08-2007 Extra parameter nrow
!        version  2.0  date 13-04-2005 Add integer nsig for axi-symmetry (RvL)
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute deformation gradient for general nonlinear solid
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nrow
      double precision F_new(nrow,nrow,m), F(m,ndim,ndim), &
                       Fn(m,ndim,ndim), xg0(m,ndim), xg(m,ndim)

!     F              i    Deformation gradient F_delta
!     F_new          o    Deformation gradient F
!     Fn             i    Deformation gradient Fn
!     nrow           i    first and second dimension of F_new
!                         In the Cartesian case ndim
!                         In the axisymmetric case ndim+1
!     xg             i    Array of length 27x3 containing the coordinates of
!                         the Gauss points in the reference element for a
!                         27 point Gauss rule
!     xg0            i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision F_wrk(ndim,ndim), Fn_wrk(ndim,ndim), &
                       F_tmp(ndim,ndim)

!     f_tmp          ?
!     f_wrk          Work array for deformation gradient
!     fn_wrk         ?
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DGEMM          BLAS routine for matrix multiplication general.
!                    (NAG: f06yaf)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el8249' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el8249'
         write(irefwr,1) 'm, ndim, jcart', m, ndim, jcart
         call prinrl1 ( F, ndim**2, m, 'F' )
         call prinrl1 ( Fn, ndim**2, m, 'Fn' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over the integration points

      do k = 1, m

!     --- compute deformation gradient in current integration point

         do j = 1, ndim
            do i = 1, ndim
               F_wrk(i,j) = F(k,j,i)
            end do
         end do

!        --- compute deformation gradient Fn in current integration point

         do j = 1, ndim
            do i = 1, ndim
               Fn_wrk(i,j) = Fn(k,j,i)
            end do
         end do

!        --- compute F_new = F * Fn

         call dgemm ('n', 'n', ndim, ndim, ndim, 1d0, F_wrk, &
                      ndim, Fn_wrk, ndim, 0d0, F_tmp, ndim)

!        --- store F_new (store transposed!)

         do j = 1, ndim
            do i = 1, ndim
               F_new(i,j,k) = F_tmp(j,i)
            end do
         end do

         if (jcart==2) then

            do j=1,ndim
               F_new(ndim+1,j,k)=0d0
               F_new(j,ndim+1,k)=0d0
            end do

!           --- F_theta_theta = r/r_0

            F_new(ndim+1,ndim+1,k)=xg(k,1)/xg0(k,1)

         end if

      end do

1000  call erclos ( 'el8249' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( F_new, m, nrow**2, 'F_new' )
         write(irefwr,*) 'End el8249'

      end if  ! ( debug )

      end
