      subroutine el8300 (Fd, Jd, Fn, Jn, p, usrvec, coef, D_t, sigma, &
                         coor0, coor, coorg0, nodpnt, imatlaw, &
                         iusrvc, iusrflg, nd, ichoice, isubr, ichfib )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  5.0  date 07-07-2007 Extra parameter ichfib
!        version  4.0  date 13-04-2005 Add integer nsig for axi-symmetry (RvL)
!        version  3.0  date 01-11-2002 extra parameters
!                                      extra parameter for elmul01/03
!        version  2.0  date 02-06-2001 Complete revision
!        version  1.4  date 17-05-2001 Added coor0
!        version  1.3  date 07-11-2000 Added user vector and nodal
!                                      point number
!        version  1.2  date 18-10-2000 Added determinant of F and Fn
!        version  1.1  date 03-04-2000 Added coordinates
!        version  1.0  date 15-04-1999
!
!   copyright (c) 1999-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fills stress tensor and tangential stiffness matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imatlaw, iusrvc, iusrflg, ichoice, nodpnt, isubr, nd, &
              ichfib
      double precision Fd(*), Fn(*), Jd, Jn, p, usrvec(*), coef(*), &
                       D_t(*), sigma(*), coor0(*), coor(*), coorg0(*)

!     coef           i    Array with element coefficients
!     coor           i    Array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coor0          i    Array with element coordinates on undeformed mesh
!     coorg0         i    ?
!     d_t            o    Tangential stiffness matrix
!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             i    Deformation gradient at incr. n Fn(k,j,i) = F_ij(x^k)
!     ichfib         i    Indicates which fiber subroutine has to be used
!                         See el0200
!     ichoice        i    Integer choice parameters for the intermediate file
!     imatlaw        i    Type of material constitutive equation to be used
!     isubr          i    Indicates the type of calling subroutine
!     iusrflg        i    User flag
!     iusrvc         i    Number of dof's in usrvec
!     jd             i    Determinant of deformation gradient
!     jn             i    Determinant of deformation gradient at incr. n
!     nd             i    Dimension of tangent stiffness matrix
!     nodpnt         i    Nodal point number
!     p              i    Hydrostatic pressure
!     sigma          o    Stress tensor
!     usrvec         i    User defined vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL8301         ?
!     EL8302         ?
!     EL8304         ?
!     EL8306         ?
!     EL8308         ?
!     EL8310         Compressible Neo-Hookean constitutive law
!     EL8311         Incompressible Neo-Hookean constitutive law
!     EL8311_1       Incompressible Neo-Hookean constitutive law
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FNUSERMAT      user function for material properties
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     2723           ?
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Special case if imatlaw = 99

      if (imatlaw==99) goto 199

!     --- Jump to the appropriate material routine

      if (imatlaw<10) then

!     --- Elastic materials

         goto (101, 102, 103, 104, 998, &
               106, 998, 108, 998), imatlaw

      else if (imatlaw<20) then

!     --- Composite materials

         goto (110, 111, 998, 998, 998, &
               998, 998, 998, 998, 998), imatlaw-9

      else if (imatlaw<30) then

!     --- Hypoelastic materials

         goto (998, 998, 998, 998, 998, &
               998, 998, 998, 998, 998), imatlaw-9

      end if

!     --- ELASTIC MATERIALS

!         --- compressible neo-hookean

 101  call el8301 (Fd, Jd, Fn, Jn, p, coef, D_t, sigma, &
                   ichoice, nd, coor, coorg0)
      goto 999

!     --- incompressible neo-hookean

 102  call el8302 (Fd, Fn, p, coef, D_t, sigma, &
                   ichoice, nd, coor, coorg0)
      goto 999

!     --- compressible Mooney-Rivlin

 103  goto 199

!     --- incompressible Mooney-Rivlin

 104  call el8304 (Fd, Fn, p, coef, D_t, sigma, ichoice)
      goto 999

!     --- incompressible Yeoh

 106  call el8306 (Fd, Fn, p, coef, D_t, sigma, ichoice, &
                   nd)
      goto 999

!     --- incompressible general hyperelastic material

 108  call el8308 (Fd, Fn, p, coef, D_t, sigma, ichoice, &
                   nd)
      goto 999

!     --- COMPOSITE MATERIALS
!         matrix fiber composite

 110  call el8310 (Fd, Jd, Fn, Jn, p, usrvec, coef, D_t, sigma, &
                   coor0, coor, nodpnt, iusrvc, iusrflg, ichoice, &
                   isubr, nd, ichfib)
      goto 999

 111  if ( ichfib==0 ) then

!     --- ichfib = 0, original call

         call el8311 ( Fd, Jd, Fn, Jn, p, usrvec, coef, D_t, sigma, &
                       coor0, coor, nodpnt, iusrvc, iusrflg, ichoice, &
                       isubr, nd )

      else

!     --- ichfib = 1, new call, using more generations

         call el8311_1 ( Fd, Jd, Fn, Jn, p, usrvec, coef, D_t, sigma, &
                         coor0, coor, nodpnt, iusrvc, iusrflg, ichoice, &
                         isubr, nd )

      end if  ! ( ichfib==0 )
      goto 999

!     --- HYPOELASTIC MATERIALS

!         --- USER DEFINED MATERIAL

 199  call fnusermat (Fd, Jd, Fn, Jn, p, usrvec, coef, D_t, sigma, &
           coor0, coor, nodpnt, iusrvc, iusrflg, ichoice, &
           isubr)
      goto 999

!     --- No such material law

 998  call eropen ('el8300')
      call errint (imatlaw, 1)
      call errsub (2723, 1, 0, 0)
      call erclos ('el8300')

 999  end
