      subroutine eld900address ( jcheld, mult )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 27-02-2010 Complete new subroutine
!        version  1.0  date 08-02-2006
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the parameter jcheld
!     Compute parameter mult for length of elemvc/wg
!     Compute starting addresses of subarrays in array coeffsarray
!     Help subroutine for eld900
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     navier_stokes_equation
!     initialization
!     start
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jcheld, mult

!     jcheld         o    if ( icheld<= 10 ) then icheld else icheld-20
!     mult           o    Multiplication factor for the number of unknowns
!                         per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eld900address' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eld900address'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute jcheld as function of icheld

      if ( icheld<=11 ) then
         jcheld = icheld
      else if ( icheld<=16 ) then
         jcheld = icheld-10
      else if ( icheld==17 ) then
         jcheld = 11
      else if ( icheld<=19 ) then
         jcheld = icheld-10
      else if ( icheld==20 ) then
         jcheld = 7
      else if ( icheld<=31 ) then
         jcheld = icheld-20
      else if ( icheld<=42 ) then
         jcheld = icheld-31
      else
         jcheld = icheld-42
      end if

!     --- Compute parameter mult for length of elemvc/wg

      if ( jcheld==1 .or. jcheld==4 .or. jcheld==7 .or. &
           jcheld==8 .or. jcheld==10 .or. jcheld==11 ) then

!     --- icheld = 1, 4, 6, 8, 10 or 11
!         1 unknown per point

         mult = 1

      else if ( jcheld==2 .or. jcheld==3 ) then

!     --- icheld = 2 or 3
!         ndim**2 unknowns per point

         mult = ndim**2

      else if ( jcheld==6 ) then

!     --- icheld = 6
!         lower triangle of tensor

         mult = 6

      else if ( ndim==2 ) then

!     --- icheld = 5 or 9 and ndim = 2, &
!         one unknown per point

         mult = 1

      else

!     --- icheld = 5 or 9 and ndim = 3, &
!         three unknowns per point

         mult = 3

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eld900address'

      end if
      call erclos ( 'eld900address' )

      end
