      subroutine elm204
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.6  date 29-05-2014 remove call to time subroutine
!        version  2.5  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for general nonlinear solids
!     ELM200 is a help subroutine for subroutine BUILD (TO0050)
!     it is called through the intermediate subroutine elmsubs
!     So:
!     BUILD
!     TO0050
!       -  Loop over elements
!          -  ELMSUBS
!             - ELM200
!    The subroutine is able to handle the following element shapes and cases:
!    3D:  27 node brick
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmoduleelm
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxnp, maxnu, maxnmu, maxnksi, maxusr, maxcoef, iseqnr
      parameter (maxnu = 27, maxnp= 8, maxnmu = 3, maxnksi = 1, &
                 maxusr = 100, maxcoef = 45, iseqnr = 1)
      integer isub, isup, jelp, nsig
      double precision rdum(1)
      save isub, isup, jelp
      integer i, j, igprob, imatlaw, iusrvc, iusrflg, ncoef, &
              nd, ndisp, npres
      integer isequ(3*maxnu), iseqp(maxnp), iseqmu(maxnmu*maxnp), &
              iseqksi(maxnksi*maxnp), mdiv
      save igprob, imatlaw, iusrvc, iusrflg, ncoef, nd, &
           ndisp, npres, isequ, iseqp, iseqmu, iseqksi
      double precision coor0(ndim,maxnu), coorn(ndim,maxnu), &
                       coor1(ndim,maxnu), qmat(3*maxnu), &
                       pp(maxnp,maxnp), rhsdiv(maxnp), fdiv(maxnu), &
                       kappa(maxnu), dum(27)
      integer, pointer :: index3n1(:)
      double precision, pointer :: vecoldlc(:), vecoldlc1(:)
      integer ipx, ipxg, ipw, ipphix, ippsix, ipphi0, ipphin, ipphi, &
              ipdphi0, ipdphin, ipdphi, ippsi, ipdpsi, ipcoef, &
              ipu, ipug, ipFn, ipxg0, ipend, &
              ipJn, ipFd, ipJd, ipp, ipp8, ippn, ippn8, ipmu, ipmu8, &
              ipmun, ipmun8, ipN, ipN8, &
              ipNn, ipNn8, ipc, ipc8, ipcn, ipcn8, ipz, &
              ipusrvc, ipB, ipCab, ipKab, ipelS, ipelR, ipelL, &
              ipelU, ipelQ, ipelT, ipwork
      save ipx, ipxg, ipw, ipphix, ippsix, ipphi0, ipphin, ipphi, &
           ipdphi0, ipdphin, ipdphi, ippsi, ipdpsi, ipcoef, &
           ipu, ipug, ipFn, &
           ipJn, ipFd, ipJd, ipp, ipp8, ippn, ippn8, ipmu, ipmu8, &
           ipmun, ipmun8, ipN, ipN8, &
           ipNn, ipNn8, ipc, ipc8, ipcn, ipcn8, ipz, &
           ipusrvc, ipB, ipCab, ipKab, ipelS, ipelR, ipelL, &
           ipelU, ipelQ, ipelT, ipwork

!     coor0          Coordinates in undeformed configuration
!     coor1          Coordinates in current configuration
!     coorn          Coordinates in reference configuration
!     dum            Dummy
!     fdiv           Dummy
!     i              Counting variable
!     igprob         Type of stress strain-relation
!     imatlaw        Type of material constitutive equation to be used
!     ipb            Pointer to B-matrix
!     ipc            Pointer to concentrations
!     ipc8           Pointer to concentrations in vertices
!     ipcab          Pointer to
!     ipcn           Pointer to old concentrations
!     ipcn8          Pointer to old concentrations in vertices
!     ipcoef         Pointer to coefficients
!     ipdphi         Pointer to dphi/dx on current configuration
!     ipdphi0        Pointer to dphi/dx on undeformed configuration
!     ipdphin        Pointer to dphi/dx on reference configuration
!     ipdpsi         Pointer to dpsi/dx on reference configuration
!     ipell          Pointer to element divergence matrix
!     ipelq          Pointer to flux rhs
!     ipelr          Pointer to element rhs
!     ipels          Pointer to element material stiffness matrix
!     ipelt          Pointer to element permeability rhs
!     ipelu          Pointer to element divergence rhs
!     ipend          Last position in array
!     ipfd           Pointer to F_delta
!     ipfn           Pointer to F_n
!     ipjd           Pointer to J_delta
!     ipjn           Pointer to J_n
!     ipkab          Pointer to permeability matrix
!     ipmu           Pointer to the chemical potentials
!     ipmu8          Pointer to the chemical potentials in vertices
!     ipmun          Pointer to the old chemical potentials
!     ipmun8         Pointer to the old chemical potentials in vertices
!     ipn            Pointer to old hydrostatic pressure
!     ipn8           Pointer to old hydrostatic pressure in vertices
!     ipnn           ?
!     ipnn8          ?
!     ipp            Pointer to hydrostatic pressure
!     ipp8           Pointer to hydrostatic pressure in vertices
!     ipphi          Pointer to basisfunctions on current configuration
!     ipphi0         Pointer to basisfunctions on undeformed configuration
!     ipphin         Pointer to basisfunctions on reference configuration
!     ipphix         Pointer to phiksi
!     ippn           Pointer to hydrostatic pressure on reference configuration
!     ippn8          Pointer to hydrostatic pressure on reference configuration
!                    in the vertices
!     ippsi          Pointer to linear basis functions
!     ippsix         Pointer to psiksi
!     ipu            Pointer to nodal displacements
!     ipug           Pointer to displacements in integration points
!     ipusrvc        Pointer to user vector
!     ipw            Pointer to weighing functions
!     ipwork         Pointer to work array
!     ipx            Pointer to element coordinates
!     ipxg           Pointer to integration point coordinates
!     ipxg0          ?
!     ipz            Pointer to help array
!     iseqksi        Sequence for ksi dof's
!     iseqmu         Sequence for chemical potential dof's
!     iseqnr         Sequence number for first old solution vector
!     iseqp          Sequence for pressure dof's
!     isequ          Sequence for displacement dof's
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iusrflg        ?
!     iusrvc         if>0, a user defined vector is used
!     j              Counting variable
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP624  (Extended quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     kappa          Dummy
!     maxcoef        Maximum number of coefficients
!     maxnksi        Number of electric potentials
!     maxnmu         Number of chemical potentials (fluid, cations, anions)
!     maxnp          Maximum number of pressure dof's
!     maxnu          Maximum number of displacement dof's
!     maxusr         Maximum number of uservector dof's
!     mdiv           Defines if divergence right-hand side is used (1)
!                    or not (0)
!     ncoef          Number of coefficients in the equation
!     nd             Dimension of tangent stiffness matrix
!     ndisp          Number of displacement dof's
!     npres          Number of pressure dof's
!     nsig           ?
!     pp             dummy
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     rdum           dummy variable
!     rhsdiv         Divergence right-hand side (if mdiv=1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0204         performs the initializations for subroutine ELM204/ELD204
!     EL1004         Compute gradient and divergence matrix (Taylor-Hood)
!     EL2604         Get old displacement vector and deformation gradient for
!                    general nonlinear solid
!     EL2614         Add displacement field to element coordinates in buffer
!     EL2720         Fill matrix B for general nonlinear solid element
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3010         Fill small matrix elemmt into larger matrix s
!     EL8200         compute the stress part of the element matrix for general
!                    nonlinear solid elements
!     EL8204         Compute the part of the element matrix, that is correlated
!                    to the fluid flux out and into the element
!     EL8205         Compute the part of the element matrix, that is correlated
!                    to the fluid flux out and into the element
!     EL8210         compute the stress part of the element vector for general
!                    nonlinear solid element
!     EL8211         compute the divergence part of the element vector
!     EL8213         Compute the part of the element matrix, that is correlated
!                    to the fluid flux out and into the element
!     EL8214         Compute the part of the element matrix, that is correlated
!                    to the fluid flux out and into the element
!     EL8234         Assemble element stiffness matrix for general nonlinear
!                    solid
!     EL8235         Assemble element vector for general nonlinear solid
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays E%iuser and USER
!     Usually they are filled by FILCOF
!
!     The sequence is:
!
!    +-------+-------------------------------------------------------------+ &
!    | coef  | description                                                 |
!    +-------+-------------------------------------------------------------+ &
!    |   1   | information about the  time-integration                     |
!    |       +---------+----+----------------------------------------------+ &
!    |                 |    | not used                                     |
!    +-------+---------+----+----------------------------------------------+ &
!    |   2   | type of stress-strain relation                              |
!    |       +---------+----+----------------------------------------------+ &
!    |                 |  0 | full 3D                                      |
!    +-------+---------+----+----------------------------------------------+ &
!    |   3   | type of numerical integration                               |
!    |       +---------+----+----------------------------------------------+ &
!    |                 |  0 | default value, defined by element            |
!    |                 |  1 | Newton-Cotes                                 |
!    |                 |  3 | Gauss                                        |
!    +-------+---------+----+----------------------------------------------+ &
!    |   4   | constitutive law  coef = imatlaw                            |
!    |       +---------+----+----------------------------------------------+ &
!    |         imatlaw |  1 | compressible elastic solid                   |
!    |                 |  2 | incompressible elastic solid                 |
!    |                 |  4 | incompressible Mooney-Rivlin                 |
!    |                 | 10 | composite material                           |
!    |                 | 11 | composite material                           |
!    |                 | 99 | user-defined material                        |
!    +-------+---------+----+----------------------------------------------+ &
!    |   5   | user flags    coef = iusrvc + 100*iusrflg                   |
!    |       +---------+----+----------------------------------------------+ &
!    |         iusrvec |  0 | user vector is not filled                    |
!    |                 |  * | user is vector is filled and vector has      |
!    |                 |    | iusrvec degrees of freedom                   |
!    |         iusrflg |  * | user defined flags                           |
!    +-------+---------+---------------------------------------------------+ &
!    |   6   | density                                                     |
!    +-------+-------------------------------------------------------------+ &
!    |   7   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |   8   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |   9   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!
!    +---------------------------------------------------------------------+ &
!    |    VECTORS NEEDED                                                   |
!    +---------------------------------------------------------------------+ &
!    | The following vectors are needed (in this order)                    |
!    +----+-----------+----------------------------------------------------+ &
!    |  1 |  u        |  incremental solution                              |
!    |  2 |  un       |  old solution at increment n                       |
!    |  3 |  usrvec   |  user defined vector  (if iusrvec>=1)            |
!    +----+-----------+----------------------------------------------------+ &
!
!
!    +---------------------------------------------------------------------+ &
!    |    CONSTITUTIVE EQUATION PARAMETERS                                 |
!    +---------------------------------------------------------------------+ &
!
!    +---------------------------------------------------------------------+ &
!    |    compressible Neo-Hookean (COEF4 = 1)                             |
!    +-------+-------+-----------------------------------------------------+ &
!            |  10   | shear modulus G                                     |
!            |  11   | bulk modulus K                                      |
!    +-------+-------+-----------------------------------------------------+ &
!    |    incompressible Neo-Hookean (COEF4 = 2)                           |
!    +-------+-------+-----------------------------------------------------+ &
!            |  10   | shear modulus G                                     |
!    +-------+-------+-----------------------------------------------------+ &
!    |    incompressible Mooney-Rivlin (COEF4 = 4)                         |
!    +-------+-------+-----------------------------------------------------+ &
!            |  10   | material parameter c0                               |
!            |  11   | material parameter c1                               |
!    +-------+-------+-----------------------------------------------------+ &
!    |    composite (COEF4 = 10, 11)                                       |
!    +-------+-------+-----------------------------------------------------+ &
!            | 10-29 | see above for matrix material                       |
!            +-------+-----------------------------------------------------+ &
!            | 30-34 | composite and fiber type                            |
!            |       +---+----+--------------------------------------------+ &
!            |           | 30 | type of constitutive equation for matrix   |
!            |           | 31 | not used                                   |
!            |           | 32 | number of layers                           |
!            |           | 33 | not used                                   |
!            |           | 34 | theta (only for COEF4 = 11)                |
!            +-------+---+----+--------------------------------------------+ &
!            | 35-45 | fiber material parameters                           |
!    +-------+-------+-----------------------------------------------------+ &
!    |    user defined material (COEF4 = 99)                               |
!    +-------+-------+-----------------------------------------------------+ &
!            | 10-29 | user material parameters                            |
!            +-------+-----------------------------------------------------+ &
!
!
!    +---------------------------------------------------------------------+ &
!    |    2-PHASE MATERIAL MODEL                                           |
!    +---------------------------------------------------------------------+ &
!
!    +-------+-------------------------------------------------------------+ &
!    | 10-19 | see above for solid phase                                   |
!    +-------+-------------------------------------------------------------+ &
!    | 20-25 | permeability                                                |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 20 | permeability coefficient K11                       |
!    |           | 21 | permeability coefficient K22                       |
!    |           | 22 | permeability coefficient K33                       |
!    |           | 23 | permeability coefficient K12                       |
!    |           | 24 | permeability coefficient K23                       |
!    |           | 25 | permeability coefficient K31                       |
!    +-------+---+----+----------------------------------------------------+ &
!    | 26-27 | time integration                                            |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 26 | delta t                                            |
!    |           | 27 | theta (0=explicit, 1=implicit)                     |
!    +-------+---+----+----------------------------------------------------+ &
!    | 28    | misc                                                        |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 28 | fluid volume fraction                              |
!    +-----------+----+----------------------------------------------------+ &
!
!
!    +---------------------------------------------------------------------+ &
!    |    4-PHASE MATERIAL MODEL                                           |
!    +---------------------------------------------------------------------+ &
!
!    +-------+-------------------------------------------------------------+ &
!    | 10-19 | see above for solid phase                                   |
!    +-------+-------------------------------------------------------------+ &
!    | 20-25 | permeability                                                |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 20 | permeability coefficient K11                       |
!    |           | 21 | permeability coefficient K22                       |
!    |           | 22 | permeability coefficient K33                       |
!    |           | 23 | permeability coefficient K12                       |
!    |           | 24 | permeability coefficient K23                       |
!    |           | 25 | permeability coefficient K31                       |
!    +-------+---+----+----------------------------------------------------+ &
!    | 26-27 | time integration                                            |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 26 | delta t                                            |
!    |           | 27 | theta (0=explicit, 1=implicit)                     |
!    +-------+---+----+----------------------------------------------------+ &
!    | 28-34 | ion properties                                              |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 28 | D+  diffusion coefficient positive ions            |
!    |           | 29 | D-  diffusion coefficient negative ions            |
!    |           | 30 | r                                                  |
!    |           | 31 | V+  molar volume fraction pos. ions                |
!    |           | 32 | V-  molar volume fraction neg. ions                |
!    |           | 33 | z+  valent pos. ions                               |
!    |           | 34 | z-  valent neg. ions                               |
!    +-------+---+----+----------------------------------------------------+ &
!    | 35-45 | misc                                                        |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 35 | cfc    fixed charged density                       |
!    |           | 36 | gamma  osmotic coefficient                         |
!    |           | 37 | phif   volume fraction fluid                       |
!    |           | 38 | cin+   internal concentration pos. ions            |
!    |           | 39 | cin-   internal concentration neg. ions            |
!    |           | 40 | cext+  external concentration pos. ions            |
!    |           | 41 | cext-  external concentration neg. ions            |
!    |           | 42 | T      temperature                                 |
!    |           | 43 | R      constant                                    |
!    |           | 44 | F      constant                                    |
!    |           | 45 | lambda Initial deformation caused by the swollen   |
!    |           |    |        state of the sample at the start of the     |
!    |           |    |        computation                                 |
!    +-----------+----+----------------------------------------------------+ &
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The following system is build:
!     |  S      -L         0           0       |  |  du  |   |     R       |
!     |                                        |  |      | = |             |
!     |   T                                    |  |      |   |             |
!     | -L   -sum(Cab)    Cab     -sum(Cab)*X  |  |  dp  |   | U - sum(Q)  |
!     |                                        |  |      |   |             |
!     |                                        |  |      |   |             |
!     |  0      Cab     -Kab-Cab    Cab*X      |  | dmu  |   | Q + sum(T)  |
!     |                                        |  |      |   |             |
!     |                                        |  |      |   |             |
!     |  0  -sum(Cab)*X   Cab*X  -sum(X*Cab*X) |  | dksi |   | -sum(X*Cab) |
!     with X a scalar.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('elm204')
      if (ifirst==0) then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         call el0204 (E%iuser, E%user, jelp, igprob, imatlaw, &
                      iusrvc, iusrflg, nd, isub, isup, work, &
                      npres, ndisp, isequ, iseqp, iseqmu, iseqksi, &
                      ncoef)
         nsig = 0  !!!!!! must be corrected

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser)

!        --- Fill pointers in array work

         ipcoef  = 1
         ipx     = ipcoef  + maxnu*maxcoef
         ipxg0   = ipx     + maxnu*ndim
         ipxg    = ipxg0   + maxnu*ndim
         ipw     = ipxg    + maxnu*ndim
         ipphix  = ipw     + maxnu
         ippsix  = ipphix  + maxnu*maxnu*ndim
         ipphi0  = ippsix  + maxnp*maxnu*ndim
         ipphin  = ipphi0  + maxnu*maxnu
         ipphi   = ipphin  + maxnu*maxnu
         ipdphi0 = ipphi   + maxnu*maxnu
         ipdphin = ipdphi0 + maxnu*maxnu*ndim
         ipdphi  = ipdphin + maxnu*maxnu*ndim
         ippsi   = ipdphi  + maxnu*maxnu*ndim
         ipdpsi  = ippsi   + maxnp*maxnu
         ipu     = ipdpsi  + maxnp*maxnu*ndim
         ipug    = ipu     + maxnu*ndim
         ipFn    = ipug    + maxnu*ndim
         ipJn    = ipFn    + maxnu*ndim*ndim
         ipFd    = ipJn    + maxnu
         ipJd    = ipFd    + maxnu*ndim*ndim
         ipp     = ipJd    + maxnu
         ipp8    = ipp     + maxnu
         ippn    = ipp8    + maxnp
         ippn8   = ippn    + maxnu
         ipmu    = ippn8   + maxnp
         ipmu8   = ipmu    + maxnu*maxnmu
         ipmun   = ipmu8   + maxnp*maxnmu
         ipmun8  = ipmun   + maxnu*maxnmu
         ipN     = ipmun8  + maxnp*maxnmu
         ipN8    = ipN     + maxnu*maxnmu
         ipNn    = ipN8    + maxnp*maxnmu
         ipNn8   = ipNn    + maxnu*maxnmu
         ipc     = ipNn8   + maxnp*maxnmu
         ipc8    = ipc     + maxnu*maxnmu
         ipcn    = ipc8    + maxnp*maxnmu
         ipcn8   = ipcn    + maxnu*maxnmu
         ipz     = ipcn8   + maxnp*maxnmu
         ipusrvc = ipz     + maxnp*maxnmu
         ipB     = ipusrvc + maxnu*maxusr
         ipCab   = ipB     + (ndim*ndim)*ndim*maxnu*maxnu
         ipKab   = ipCab   + 6*maxnp*maxnp
         ipelS   = ipKab   + 6*maxnp*maxnp
         ipelL   = ipelS   + (maxnu*ndim)**2
         ipelR   = ipelL   + (maxnu*ndim)*maxnp
         ipelU   = ipelR   + maxnu*ndim
         ipelQ   = ipelU   + maxnp
         ipelT   = ipelQ   + maxnmu*maxnp
         ipwork  = ipelT   + maxnmu*maxnp
         ipend   = ipwork  + max((n+1)*m,n**2+n+m)

!        --- Check length of array coeffarr (declared in elmgrinf)

         call inichkarlen ( ipend, 71830, 'coeffarr' )

      end if
      mdiv = 0

!     --- Get coordinates of deformed mesh on increment n (coorn) and
!         compute current and initial coordinates (coor1 and coor0)

      iseqtype = oldsols(iseqnr+1)%iseqtype
      index3n1 => indoldsols(iseqtype)%index3
      vecoldlc1 => oldsols(iseqnr+1)%oldvecs
      iseqtype = oldsols(iseqnr)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(iseqnr)%oldvecs
      do i = 1, n
         do j = 1 , ndim
            coorn(j,i) = coor(j,index1(i))
            coor1(j,i) = coorn(j,i) + &
                         vecoldlc(index3n(isequ((i-1)*ndim+j)))
            coor0(j,i) = coorn(j,i) - &
                         vecoldlc1(index3n1(isequ((i-1)*ndim+j)))
         end do
      end do

!     --- Compute basis functions on reference and deformed mesh

      call elm800basefn ( coor0, coeffarr(ipx), coeffarr(ipw), coeffarr(ipdphi0), &
                          coeffarr(ipxg), coeffarr(ipphi0), index1, qmat, rdum, &
                          jelp, coeffarr(ipphix), coeffarr(ippsi), coeffarr(ipdpsi), &
                          coeffarr(ippsix))
      call elm800basefn ( coorn, coeffarr(ipx), coeffarr(ipw), coeffarr(ipdphin), &
                          coeffarr(ipxg), coeffarr(ipphin), index1, qmat, rdum, &
                          jelp, coeffarr(ipphix), coeffarr(ippsi), coeffarr(ipdpsi), &
                          coeffarr(ippsix))
      call elm800basefn ( coor1, coeffarr(ipx), coeffarr(ipw), coeffarr(ipdphi), &
                          coeffarr(ipxg), coeffarr(ipphi), index1, qmat, rdum, &
                          jelp, coeffarr(ipphix), coeffarr(ippsi), coeffarr(ipdpsi), &
                          coeffarr(ippsix))
      if (ierror/=0) goto 1000

!     --- Get all old vectors

      call el2604 ( coeffarr(ipphi0), coeffarr(ipphin), coeffarr(ipphi), &
                    coeffarr(ipdphi0), coeffarr(ipdphin), &
                    coeffarr(ippsi), coeffarr(ipu), coeffarr(ipug), &
                    coeffarr(ipFd), coeffarr(ipJd), coeffarr(ipFn), coeffarr(ipJn), &
                    coeffarr(ipp), coeffarr(ipp8), coeffarr(ippn), coeffarr(ippn8), &
                    coeffarr(ipmu), coeffarr(ipmu8), coeffarr(ipmun), coeffarr(ipmun8), &
                    coeffarr(ipusrvc), iusrvc, iseqnr, isequ, iseqp, iseqmu, &
                    npres, coeffarr(ipc), coeffarr(ipcn), ncoef, coeffarr(ipcoef))

!     --- Fill variable coefficients if necessary

      do i = isub, isup
         if (ind(i)>0 .or. ist(i)>0) then

!        --- Coefficient is variable

            call elflr71 (i, E%user, coeffarr, coeffarr(ipx), &
                         coeffarr(ipxg), m, n, coeffarr(ipphi), &
                         coeffarr(ipwork), 1, index1, &
                         E%iuser, coeffarr(ipug), work )
         end if
      end do

!     --- Compute B-matrix

      call el2720 (coeffarr(ipB), coeffarr(ipdphi), nd, coeffarr(ipphi), coeffarr(ipxg))

!     --- Compute quantities such as momentaneous volume fraction, &
!         internal concentration and some constants

      call el2614 (coeffarr(ipN), coeffarr(ipNn), coeffarr(ipc), coeffarr(ipcn), &
                   coeffarr(ipJd), coeffarr(ipJn), coeffarr(ipz), coeffarr(ipcoef), &
                   ncoef, npres)

!     --- Fill parts of matrix if necessary

      if (matrix) then

!     --- Compute stiffness matrix

         call el8200 ( coeffarr(ipelS), coeffarr(ipxg), coor0, coeffarr(ipw), &
                       coeffarr(ipFd), coeffarr(ipJd), coeffarr(ipFn), coeffarr(ipJn), &
                       coeffarr(ipp), coeffarr(ipB), coeffarr(ipusrvc), nd, imatlaw, &
                       iusrvc, iusrflg, coeffarr(ipcoef), coeffarr(ipxg0), &
                       ncoef, index1, nsig, 0, maxusr )

!        --- Compute divergence matrix

         call el1004 ( coeffarr(ipelL), rdum, coeffarr(ippsi), coeffarr(ipdphi), &
                       coeffarr(ipphi), coeffarr(ipw), coeffarr(ipxg), 0, rdum, &
                       pp, kappa, rhsdiv, fdiv, mdiv, dum )

!        --- Compute saturation matrix

         call el8204 (coeffarr(ipCab), coeffarr(ipw), coeffarr(ippsi), coeffarr(ipN), &
                      coeffarr(ipc), coeffarr(ipJd), coeffarr(ipJn), coeffarr(ipcoef), &
                      ncoef, npres)

!        --- Compute permeability matrix

         call el8205 (coeffarr(ipKab), coeffarr(ipw), coeffarr(ipdpsi), coeffarr(ipcn), &
                      coeffarr(ipcoef), ncoef, npres)
      end if

!     --- Fill parts of vector if necessary

      if (vector .and. notvec==0) then

!     --- First set element vector equal to zero

         do i = 1, icount
            elemvc(i) = 0d0
         end do
         do i = 1, ndim*n
            coeffarr(ipelR+i-1) = 0d0
         end do
         do i = 1, npres
            coeffarr(ipelU+i-1) = 0d0
         end do
         do i = 1, 3*npres
            coeffarr(ipelQ+i-1) = 0d0
            coeffarr(ipelT+i-1) = 0d0
         end do

!        --- Compute stiffness vector

         call el8210 ( coeffarr(ipelR), coeffarr(ipxg), coor0, coeffarr(ipw), &
                       coeffarr(ipFd), coeffarr(ipJd), coeffarr(ipFn), coeffarr(ipJn), &
                       coeffarr(ipp), coeffarr(ipB), coeffarr(ipusrvc), nd, imatlaw, &
                       iusrvc, iusrflg, coeffarr(ipcoef), coeffarr(ipxg0), &
                       ncoef, index1, nsig, 0, maxusr )

!        --- Compute divergence vector

         call el8211 (coeffarr(ipelU), coeffarr(ipw), coeffarr(ippsi), &
                      coeffarr(ipJd), coeffarr(ipJn), npres, 1)

!        --- Compute saturation vector

         call el8213 (coeffarr(ipelQ), coeffarr(ipw), coeffarr(ippsi), coeffarr(ipN), &
                      coeffarr(ipNn), coeffarr(ipJd), coeffarr(ipJn), npres)

!        --- Compute permeability vector

         call el8214 (coeffarr(ipelT), coeffarr(ipw), coeffarr(ipKab), coeffarr(ipdpsi), &
                      coeffarr(ipcn), coeffarr(ipFd), coeffarr(ipJd), coeffarr(ipmu8), &
                      coeffarr(ipmun8), coeffarr(ipcoef), ncoef, npres)
      end if

!     --- Fill parts of mass matrix if necessary

      if (notmas==0) then

!     --- notmas = 0,  compute mass matrix

         if (imas>0) then
            elemms = 0d0
         else
            jadd = 0
         end if
         if (imas>1) then

!        --- non-diagonal mass matrix

            jzero = 2
            call el3002 (coeffarr, coeffarr(ipwork), coeffarr(ipw), coeffarr(ipphi), &
                         m, n, n )
         else

!        --- diagonal mass matrix

            jconsf = imas
            call el3003 (coeffarr, coeffarr(ipwork+n*n), coeffarr(ipw), coeffarr(ipphi), &
                         m, n )
         end if
         if (imas==1) then

!        --- imas = 1, build diagonal mass matrix

            do i = 1, n
               elemms(1,i) = coeffarr(ipwork+n*n+i-1)
               elemms(2,i) = coeffarr(ipwork+n*n+i-1)
               elemms(3,i) = coeffarr(ipwork+n*n+i-1)
            end do
         else

!        --- imas = 2, build non-diagonal mass matrix

            jadd = 0
            call el3010 (elemms, coeffarr(ipwork), 3*n, n, 1, 1, 3, &
                         .false., 1d0)
            call el3010 (elemms, coeffarr(ipwork), 3*n, n, 2, 2, 3, &
                         .false., 1d0)
            call el3010 (elemms, coeffarr(ipwork), 3*n, n, 3, 3, 3, &
                         .false., 1d0)
         end if
      end if

!     --- Assemble total element stiffness matrix

      if (matrix) then
         call el8234 (elemmt, coeffarr(ipelS), coeffarr(ipelL), coeffarr(ipCab), &
                      coeffarr(ipKab), coeffarr(ipz), isequ, iseqp, iseqmu, &
                      iseqksi, npres)
      end if

!     --- Assemble total element vector

      if (vector) then
         call el8235 (elemvc, coeffarr(ipelR), coeffarr(ipelU), coeffarr(ipelQ), &
                      coeffarr(ipelT), coeffarr(ipz), isequ, iseqp, iseqmu, &
                      iseqksi, npres)
      end if

1000  call erclos ('elm204')

      end
