      subroutine elm406 ( work, vecloc, elemmt1d )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  7.0  date 06-01-2016 Use sepmoduleelem
!        version  6.9  date 09-03-2013 Replace elflr7 by elflr71
!        version  6.8  date 12-12-2012 Long integers
!
!   copyright (c) 1984-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector two-dimensional Navier-Stokes equations
!     with swirl
!     extended quadratic triangle for velocities (itype=406)
!              quadratic triangle for swirl
!     itype=406 :    axisymmetric coordinates  7-point triangle with swirl
!                    Penalty function approach
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     flow
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduletime
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      double precision x, w, dphidx, xgauss, phi, array, amasps, &
                       u, ugauss, un, dudx, trans, transl, qphixx, &
                       etaeff, secinv, dvisd2, omega, sigmalc, rho, cn, &
                       clam, f, eij, dlamdx, psi, dpsidx, dum, dum1, &
                       penal, amas, visprm
      common /celwrk/ x(14), w(7), dphidx(98), xgauss(14), phi(49), &
                      array(56), u(21), ugauss(21), un(21), dudx(42), &
                      trans(24), transl(1), qphixx(147), etaeff(7), &
                      secinv(7), dvisd2(7), omega, sigmalc, rho, &
                      cn, clam, psi(42), f(21), dpsidx(84), amasps(36), &
                      penal(14), amas(49), dum1(210), visprm(3), &
                      eij(6), dlamdx(6), dum(488)
      save   /celwrk/

!                     /celwrk/
!          Work common block to store local arrays
!          CELWRK contains exactly 1500 real positions, the last 500
!          are reserved for the ELP subroutines
!          The following parameters are used in CELWRK
!     amas           Mass matrix corresponding to basis funtions phi
!     amasps         Mass matrix corresponding to basis funtions psi
!     array          8 x m array to store the coefficients for each
!                    integration point
!                    Contents:
!                    Pos 1-m:       - &
!                    Pos m+1-2m:    rho
!                    Pos 2m+1-3m:   - &
!                    Pos 3m+1-4m:   omega
!                    Pos 4m+1-5m:   f 1
!                    Pos 5m+1-6m:   f 2
!                    Pos 6m+1-7m:   f 3
!                    Pos 7m+1-8m:   eta
!     clam           Parameter lambda for Generalised Newtonian fluids
!     cn             Parameter n for Generalised Newtonian fluids
!     dlamdx         Array of 6 positions containing the derivatives of the
!                    barycentric co-ordinates
!     dphidx         Array of length n x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                      in node k
!     dpsidx         Array of length n-1 x m x 2  containing the derivatives
!                    of the special basis functions corresponding to the
!                    nodal points in the sequence:
!                    d psi / dx = dpsidx (i,k,1);  d psi / dy = dpsidx (i,k,2);
!                         i                             i
!                      in node k
!     dudx           Array of length 4 x m containing the derivatives of the
!                    preceding velocity in the integration points in the
!                    sequence:
!                    1    - m:  du/dx
!                    m+1  - 2m: du/dy
!                    2m+1 - 3m: dv/dx
!                    3m+1 - 4m: dv/dy
!     dum            488 positions reserved for the ELP subroutines
!     dum1           work space free for later use
!     dvisd2         Array of length m to store the derivative of the viscosity
!                    with respect to the second invariant of the
!                    stress tensor in the integration points
!     eij            Array of 6 positions containing the factors e
!     etaeff         Array of length m to store the effective viscosity in
!                    the integration points
!     f              Array of length 3n to store the right-hand side vector
!                    temporarily  (not yet used)
!     omega          Angular velocity of rotating system
!     penal          Help array to store the penalty vector
!     phi            Array of length n x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i, k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     psi            Array of length n-1 x m containing the values of the
!                    basis functions corresponding to the nodal points
!                    in the Gauss points in the sequence:
!                           k
!                    psi (xg ) = psi (i, k)
!                       i
!                    Array psi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     qphixx         Array of length 3 x n x n to store the integrals
!                    corresponding to the second derivative terms on a
!                    reference element
!     rho            Density of the fluid
!     secinv         Array of length m to store the second invariant of the
!                    stress tensor in the integration points
!     sigmalc          Inverse of parameter epsilon in Penalty function method
!     trans          Transformation array for the velocity in the centroid
!                    length: 2 * inpelm
!     transl         Not yet used
!     x              array of length n x 2 containing the x and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i, 1);  y  = x (i, 2);
!                     i              i
!     xgauss   o     Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i, 1);  yg  = xgauss (i, 2);
!                      i                    i
!     u              Array of length 3n to store the preceding velocity in
!                    the nodal points
!     ugauss         Array of length 3m to store the preceding velocity in
!                    the integration points
!     un             Array of length 3n to store the velocity at a preceding
!                    time level in the nodal points
!     visprm         Viscosity model parameters for modelv=6,7
!     w              array of length m containing the weights for the
!                    numerical integration
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), work(*), elemmt1d(*)

!     vecloc         Work array in which all old solution vectors for the
!                    integration points are stored
!     work           Work array to be used by this subroutine
!                    At this moment a working space of 800 suffices
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer induv(18), indunk(3)
      integer method, itcoor, npuv, npuv2, nstart, jtcoor, &
              ltime, mconv, irotat, modelv, iche, ich220, &
              ichint, icurve, ich221, ichh, ichiv, i, j, isign, &
              ipwork, isub, isup, jelp, iptim, kderiv, number, ntemp
      double precision delta, rhoh(1), thetalc, tsteplc
      save method, itcoor, npuv, npuv2, nstart, jtcoor, ipwork, &
           ltime, mconv, irotat, modelv, isub, isup, &
           rhoh, iche, induv, indunk
      data induv/1,13,7,2,14,8,3,15,9,4,16,10,5,17,11,6,18,12/, &
           indunk / 1, 3, 2/

!     delta      jacobian of element
!     i          Counting variable
!     ich220     ?
!     ich221     ?
!     iche       ?
!     ichh       ?
!     ichint     ?
!     ichiv      ?
!     icurve     Type of curve
!     indunk     Defines the new sequence of the unknowns in a global way
!     induv      Defines the sequence of the unknowns in array u
!                The storage in array USOL is u , u   , u
!                                              r   psi   z
!                However, in array u and dudx the new sequence
!                u , u , u    is used
!                 r   z   psi
!                induv defines the new sequence for each node
!     iptim      Starting address for time-dependent solution in array work
!     ipwork     Starting address for right-hand side part in array work
!                The first part of array work is reserved for the element
!                matrix
!     irotat     Parameter indicating if the co-ordinate system rotates (10
!                or not (0)
!     isign      Help parameters to check the sign of the Jacobian in the
!                integration points
!     isub       Integer help parameter for do-loops to indicate the
!                smallest coefficient number that is dependent on space
!     isup       Integer help parameter for do-loops to indicate the
!                largest coefficient number that is dependent on space
!     itcoor     ?
!     j          Counting variable
!     jelp       Indication of the type of basis function subroutine must
!                be called by ELM400. Possible vaues:
!                1:  Subroutine ELP220  (Seven-point extended triangle)
!     jtcoor     ?
!     kderiv     ?
!     ltime      Parameter indicating the type of time-dependence
!                Possible values:
!                0:  Steady state
!                1:  Instationary equations theta method
!                +-2:  Pressure correction scheme
!                3:  Mass matrix only
!                4:  Iterative instationary equations theta method
!     mconv      With this parameter the type of linearization of the
!                convective terms is indicated
!                Possible values:
!                0:  No convective terms (Stokes)
!                1:  Standard Picard linearization
!                2:  Newton linearization
!                3:  Picard linearization in the wrong way
!                    (small convergence region, slow convergence)
!                4:  Picard linearization (only effect on right-hand side)
!     method     Indicates the type of method that is used to solve the
!                Navier-Stokes equations
!                Possible values:
!                1:   Penalty function method
!     modelv     Defines the type of viscosity model
!                Possible values:
!                  1:  newtonian liquid
!                  2:  power law liquid
!                  3:  carreau liquid
!                  4:  plastico-viscous liquid
!                  5:  Papanastasiou plastico-viscous model
!                  6:  bi-viscosity plastico-viscous model
!                  7:  Carreau-Yasuda model
!                100:  user provided model depending on grad u
!                101:  user provided model depending on the second invariant
!                102:  user provided model depending on the second invariant, &
!                      the co-ordinates and the velocity
!     npuv       ?
!     npuv2      ?
!     nstart     Number of positions in array ISTART
!     ntemp      Help varaible to store n temporarily
!     number     Sequence number of solution array in E%islold/vecold
!     rhoh       ?
!     thetalc    Local value of theta
!                In this case ctimen is filled in the incorrect old way
!     tsteplc    Local value of theta
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY   Copy double precision dx to double precision dy
!     EL0400  Compute element independent quantities
!     EL2002  Compute part of matrix corresponding to zeroth order derivatives
!     EL2009  Reorder the contents of the element matrix or element vector
!     EL2015  Compute matrix vector multiplication
!     EL2016  Compute sum of to matrices
!     EL3007  Fills old solution and derivatives
!     EL3009  Fill series of old solutions in array vecloc
!     EL4022  Compute viscous matrix for 2-dim incompressible flow
!     EL4023  Compute convective and/or coriolis acceleration (matrix)
!     EL4024  Eliminate velocity from centroid (uc, vc)
!     EL4025  Compute (transformed) penalty vector
!     EL4028  Compute  viscosity for a generalized newtonian liquid
!     EL4030  Compute (driving) external forces for RHS vector
!     EL4033  Compute convective acceleration for RHS vector
!     EL4037  Compute vector due to newton linearization of viscosity
!     ELFLR71 Fill variable coefficients in array ARRAY
!     ELIND0  Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP220  Shape functions for 7-point triangle (7 integration points)
!     ELP221  Shape functions for 6 point triangle (7 integration points)
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRCHR  Put character in error message
!     ERREAL  Put real in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1304   For coefficient only positive constants are allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm406' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm406'
         write(irefwr,1) 'ielem', ielem
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      thetalc = t
      tsteplc = tout
      if ( ifirst==0 ) then

!     --- ifirst = 0, compute element independent quantities

         ltime = 0
         call el0400 ( E%iuser, E%user, jelp, isub, isup, method, mconv, &
                       ltime, modelv, irotat )
         ipwork = 1 + (3*inpelm+2)**2

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

!        --- Set jsol temporarily equal to -1 in order to indicate that the
!            old versions of the ELP subroutines are used

        jsol = -jsol

!       --- Set some constants, these correspond to the old version
!           npuv    number of points for velocity u1, u2 including centroid
!           nstart  number of test function to start with
!           lrow    total number of degrees of freedom in element

         npuv=inpelm+1
         npuv2=npuv+npuv
         lrow=npuv2 + inpelm
         nstart=1
         itcoor = 3
         iptim = npuv*npuv+1
         nstart = 1
         jtcoor = 1

         if(ltime==1 .or. ltime==4) then
            rhoh(1) = rho/(thetalc*tsteplc)
         else if(ltime==3) then
            rhoh(1) = rho
         end if
         if ( modelv==2 .and. cn>1d0 ) then
            iche = 2
         else
            iche = 1
         end if
         ich220 = jtcoor+30
         if (method==0) ich220= ich220+40

      end if

!     --- compute basis functions, derivatives of basis functions, etc.

      if ( debug ) write(irefwr,*) 'before elp220'
      call elp220 ( ich220, ichint, coor, index1, isign, &
                    icurve, delta, x, xgauss, eij, dlamdx, w, phi, &
                    dphidx, trans, transl, qphixx )

!     --- swirl, quadratic basis functions

      ich221 = 11
      if ( debug ) write(irefwr,*) 'before elp221'
      call elp221 ( ich221, ichh, inpelm, coor, index1, &
                    isign, icurve, delta, x, xgauss, w, psi, dpsidx )

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      number = 0
      if ( ltime==4 ) then

!     --- ltime = 4, Iterative time-dependent method
!                    Both un and u n+1 * are necessary

         kderiv = jderiv
         jderiv = 0
         jind = 0
         ncomp = 3
         call el3007 ( un, work, phi, dphidx, m, n, ndimlc, &
                       ncomp, induv, trans, ugauss, jtrans, numold, 1 )
         jderiv = kderiv
         number = 2

      else if ( ind(lengthmst)==1 ) then

         number = 1

      end if
      if ( number>0 ) then

!     --- Number>0, fill old solution

         jderiv = -jderiv
         jind = -1
         ncomp = 2
         call el3007 ( u, dudx, phi, dphidx, m, n, ndimlc, 3, &
                       indunk, trans, ugauss, jtrans, numold, number )
         ncomp = 1
         jtrans = 0
         ntemp = n
         n = inpelm
         call el3007 ( u(2*ntemp+1), dudx(4*m+1), psi, dpsidx, &
                       m, ntemp, ndimlc, 3, indunk(3), trans, &
                       ugauss(2*m+1), jtrans, numold, number )
         n = ntemp
         ncomp = 3
         jtrans = 1
         jderiv = -jderiv

      end if
      if ( debug ) write(irefwr,*) 'before el3009'

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, phi, n )

      do 150 i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, E%user, array, x, xgauss, &
                          m, n, phi, 1, index1, &
                          E%iuser, ugauss, vecloc )
            if ( i==8 ) then

!           --- Coefficient 8, i.e. eta
!               Test if eta is positive

                do 110 j = 1, m
                   if ( array(j+(i-1)*m)<=0 ) then

!                  --- Error 1304: Eta is not positive

                      call errint ( i, 1 )
                      call errint ( itype, 2 )
                      call errint ( ielgrp, 3 )
                      call errint ( ielem, 4 )
                      call erreal ( array(j+(i-1)*m), 1 )
                      call errchr ( 'eta (0)', 1 )
                      call errsub ( 1304, 4, 1, 1 )

                   end if

110             continue

             end if

         end if

150   continue

      if (ind(8)/=0) then
         ichiv = 1
      else
         ichiv = ichint
      end if

!     --- compute effective viscosity

      if ( debug ) then
         write(irefwr,1) 'iche, itcoor, npuv, ndimlc, modelv', &
                          iche, itcoor, npuv, ndimlc, modelv
         write(irefwr,2) 'cn, clam, visprm', cn, clam, visprm
      end if  ! ( debug )

      if (matrix.or.modelv>1.and.modelv<=7.and.ltime/=3) &
         call el4028(iche, itcoor, npuv, ndimlc, &
                     modelv, array(1+7*m), cn, clam, visprm, &
                     work(ipwork), xgauss, ugauss, dudx, etaeff, &
                     secinv, dvisd2 )

      if ( debug ) write(irefwr,1) 'ltime', ltime

      if(ltime>=1) then
         call el2002(6, ichint+1, rhoh, E%mat, w, phi, npuv, &
             npuv, npuv, work(ipwork), work, delta, .true. )
         call el2002(6, ichint+1, rhoh, amasps, w, psi, 6, 7, &
             6, work(ipwork), work, delta, .true. )
         if(ltime==3) goto 1100
         call el2015 ( 1, ichint+1, E%mat, u, 1d0, elemmt1d(iptim), npuv, &
                       npuv, 1 )
         call el2015 ( 1, ichint+1, E%mat, u(1+npuv), 1d0, elemmt1d &
                       (iptim+npuv), npuv, npuv, 1)
         call el2015 ( 1, ichint+1, amasps, u(1+npuv2), 1d0, elemmt1d &
                       (iptim+npuv2), inpelm, inpelm, 1 )

!        --- copy m uold in array from pos. ipu

         call dcopy(lrow, elemmt1d(iptim), 1, u, 1)
      end if

!     --- if (matrix) compute element matrix

      if (.not.matrix) goto 2000

!     --- compute viscous stresses and initialize element matrix
!             s

      if ( modelv==2 .and. cn>1d0 ) then

!     --- compute terms due to newton linearization of viscosity
!             (for special cases only)

         do 999 i=1,npuv
            etaeff(i)=etaeff(i)+dvisd2(i)*secinv(i)
999      continue
      end if
      if ( debug ) write(irefwr,*) 'before el4022'
      call el4022 ( ichiv, itcoor, etaeff, work, w, xgauss, phi, &
                    dphidx, nstart, npuv, npuv, lrow, psi, dpsidx, &
                    inpelm, dlamdx, qphixx, delta, work(ipwork) )

!     --- compute penalty matrix if required, and add it

      if ( debug ) write(irefwr,*) 'before el4025'
      if (method==1) &
          call el4025 ( jtcoor, ichint, 1, work, work(ipwork), sigmalc, &
                        isign, delta, eij, w, xgauss, phi, dphidx, npuv, &
                        npuv, ncomp-1, lrow )

!     --- add convective and/or coriolis acceleration if required

      if ( debug ) write(irefwr,*) 'before el4023'
      if ( mconv>0 .and. mconv/=4 .or. irotat==1 ) &
         call el4023 ( ichint, itcoor, mconv, rho, irotat, omega, &
                       ugauss, dudx, w, xgauss, phi, dphidx, work, &
                       npuv, npuv, lrow, psi, dpsidx, inpelm )
1100  if(ltime==1 .or. ltime==4 .and. thetalc>0d0) then
         call el2016 ( 1, ichint+1, work, E%mat, 1d0, 1d0, npuv, lrow )
         call el2016 ( 1, ichint+1, work(1+npuv*lrow+npuv), E%mat, &
                       1d0, 1d0, npuv, lrow )
         call el2016 ( 1, ichint+1, work(1+npuv2*lrow+npuv2), amasps, &
                       1d0, 1d0, inpelm, lrow )
      else if (ltime==3) then
         work(1:lrow*lrow) = 0d0
         call el2016 ( 1, ichint+1, work, E%mat, 0d0, 1d0, npuv, lrow )
         call el2016 ( 1, ichint+1, work(1+npuv*lrow+npuv), E%mat, 0d0, &
                       1d0, npuv, lrow )
         call el2016 ( 1, ichint+1, work(1+npuv2*lrow+npuv2), amasps, &
                       0d0, 1d0, inpelm, lrow )
      end if

!     --- eliminate velocity from centroid (uc, vc) in point npuv
!               i.e. transform matrix : r(transp)*smatrix*r

      if ( debug ) write(irefwr,*) 'before el4024'
      call el4024 ( 1, work, work(ipwork), trans, work(ipwork), ndimlc, &
                    npuv, lrow )

!     --- reorder the contents of the element matrix and put it into E%mat

      if ( debug ) write(irefwr,*) 'before el2009'
      call el2009 ( 1, E%mat, elemvc, work, work(ipwork), induv, &
                    icount, 0 )

!     --- compute element vector

2000  if (.not.vector) go to 1000

!     --- fill (driving) external forces into vector (initialize it)

      call el4030 ( ichint, itcoor, array(1+4*m), phi, psi, rho, w, &
                    work(ipwork), nstart, npuv, npuv, inpelm, ndimlc, &
                    lrow )

!     --- add terms due to newton linearization of viscosity
!             work(ipwork)

      if ( modelv==2 .and. cn>1d0 ) &
         call el4037 ( itcoor, w, dum1, xgauss, phi, dphidx, ugauss, &
                       dudx, secinv, dvisd2, work(ipwork), ndimlc, npuv, &
                       npuv, psi, dpsidx, inpelm )

!     --- compute influence of convective acceleration for mconv = 2

      if ( mconv==2 .or. mconv==4 ) then
         call el4033 ( ichint, itcoor, rho, ugauss, dudx, w, xgauss, &
                       phi, work(ipwork), nstart, npuv, npuv, ndimlc, &
                       psi, inpelm )
      end if

!     --- eliminate velocity basis functions from centroid from vector

      call el4024 ( 2, work, work(ipwork), trans, work, ndimlc, npuv, &
                    lrow )

!     --- reorder the contents of the element vector and put it into elemvc

      call el2009 ( 2, E%mat, elemvc, work, work(ipwork), induv, &
                    icount, 0 )
1000  if ( debug ) then

!     --- Debug information

   !      call prinrl ( elemvc, icount, 'elemvc' )
   !      call prinrl1 ( E%mat, icount, icount, 'E%mat' )
         write(irefwr,*) 'End elm406'

      end if  ! ( debug )
      call erclos ( 'elm406' )

      end subroutine elm406
