      subroutine elm903parms1 ( iuser, interpol, ioldm, ishift, jcheld, &
                                jelp, jtime, kchois, nveloc, mcont, &
                                modelrhsd, modelv, mrule, ncoef, &
                                thetdt, compress )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 20-02-2010
!
!   copyright (c) 2010-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute parameters irule, m and n for type elements of type 903
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), interpol, ioldm, ishift, jcheld, jelp, jtime, &
              kchois, nveloc, mcont, modelrhsd, modelv, mrule, ncoef
      double precision thetdt
      logical compress

!     compress       o    If true compressible has been read
!     interpol       o    If 0 the standard interpolation is applied, &
!                         if 1 the interpolation is restricted to linear
!                         subelements
!     ioldm          o    Sequence number of velocity at end of previous time
!                         step in input vector.
!                         If 0 no iterative procedure is adopted for dynamical
!                         analyses.
!     ishift         o    Shift of specific variable with respect to diagonal
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jcheld         o    if ( icheld<= 10 ) then icheld else icheld-20
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     jtime          o    Integer parameter indicating if the mass matrix for
!                         the time-dependent equation must be computed (>0)
!                         or not ( = 0)
!     kchois         o    Indication what type of parameters must be computed
!     mcont          o    Defines the type of continuity equation
!     modelrhsd      o    Defines how the stress tensor is treated in the rhs
!     modelv         o    Defines the type of viscosity model
!     mrule          o    Type of integration rule as given by the user
!     ncoef          o    Number of coefficients in the equation
!     nveloc         o    Number of velocity parameters
!     thetdt         o    1/(theta dt) for the case of the theta method
!                         immediately applied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqpres(8), iseqvel(81)

!     iseqpres       Sequence number of pressure with respect to array isol
!     iseqvel        Array containing the positions of the velocities in
!                    the element vector in the sequence required by el2005
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0900COEFF1   Process first 5 parameters of array iuser in case of
!                    elements of type 900
!     EL0900CONS     Set some constants for subroutines el0900/3/5
!     EL0900INIT     Make some initialization for subroutines el0900/3/5
!                    depending on icheld
!     EL0903RULE     Set integration parameters with respect to elements
!                    of type 903
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm903parms1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm903parms1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize parameters in commons

      call elcomm

!     --- initialize some other parameters

      call el0900cons ( jtime, jconsf, ncomp, ndim, kchois, &
                        modelv, modelrhsd, mcont, mrule, jcheld, &
                        icheld, compress )
      if ( ierror/=0 ) go to 1000
      jsecnd = 2   ! is only used in case of upwind
                   ! viscosity is a scalar

!                  --- extra initializations

      jind = 0
      n = inpelm
      jtrans = 0
      jelp = ishape
      nvert = n
      ishift = 0

!     --- Compute kchois

      call el0900init ( icheld, ncoef, compress, itype, jcheld, &
                        ielgrp, ix, ndim, jdegfd, ncomp, jderiv, &
                        kchois, icount, inpelm )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'ncoef, kchois, jelp, n', &
                          ncoef, kchois, jelp, n

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      call el0900coeff1 ( ncoef, kchois, iuser, jtime, &
                          thetdt, jcheld, compress, ioldm, &
                          modelrhsd, modelv, interpol, &
                          ishift, mrule )
      if ( ierror/=0 ) go to 1000

!     --- Set parameters depending on the shape numbers

      call el0903rule ( mrule, irule, compress, ishape, icheld, &
                        n, m, jtrans, jelp, nveloc, ndim, &
                        iphi, igausp, jdiag, nvert, ncoef, &
                        jcart, mn, jdercn, ielem, ielgrp, &
                        itype, kchois, imas, iseqvel, iseqpres, &
                        npsi )

1000  call erclos ( 'elm903parms1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm903parms1'

      end if  ! ( debug )

      end
