      subroutine elm903upw ( sp, phi, psiupw, dpsidx, w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-05-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Update stiffness matrix with upwind term due to viscosity
!     Update gradient matrix with upwind term
! **********************************************************************
!
!                       KEYWORDS
!
!     upwinding
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision sp(ndimlc*n,*), phi(n,m), psiupw(n,m), &
                       dpsidx(nvert,m,ndimlc), w(m)

!     dpsidx         i    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                              i        k
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psiupw         i    Array of size n*m containing the adapted basis
!                         functions for upwind
!     sp            i/o   Gradient of pressure matrix
!                         At input it contains the original grad matrix
!                         At outut it is adapted due to upwind
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELM903GPUPW    Update gradient matrix with upwind term due to pressure
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm903upw' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm903upw'
         call prinrl2 ( sp, ndimlc*n, npsi, 'Grad-matrix/1' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Update the gradient matrix with the upwind term

      call elm903gpupw ( sp, phi, psiupw, dpsidx, w )

!     --- Update the stiffness matrix with the upwind term due to viscosity

!         call

1000  call erclos ( 'elm903upw' )
      if ( debug ) then

!     --- Debug information

         call prinrl2 ( sp, ndimlc*n, npsi, 'Grad-matrix/2' )
         write(irefwr,*) 'End elm903upw'

      end if  ! ( debug )

      end
