      subroutine elmfrict2d ( elemmt, icount, ist, n, stat, usc, mu )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-07-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the element stiffness matrix of a friction element (2D)
!     use the Brekelmans approach
! **********************************************************************
!
!                       KEYWORDS
!
!     friction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icount, ist, stat
      double precision elemmt(icount,icount), n(2), usc(2), mu

!     elemmt        i/o   Element matrix
!     icount         i    Number of unknowns in element
!     ist            i    Start of first Lagrange multiplier in element
!     mu             i    friction coefficient
!     n              i    normal vector
!     stat           i    Status of point
!     usc            i    Vector used to compute the tangential directions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision t(2), ut1

!     t              tangential vector
!     ut1            Component in first tangential direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmfrict2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmfrict2d'

      end if
      if ( ierror/=0 ) go to 1000

!     --- This gives the effect of the reaction forces at the momentum
!         equations.

      elemmt(ist+1,1) = -1d0
      elemmt(ist+2,2) = -1d0

      t(1) = n(2)
      t(2) = -n(1)

      if (stat==1 .or. stat==0) then

!     --- stick, so the extra equations model that all velocities
!         are zero

         elemmt(    1,ist+1) = 1d0
         elemmt(    2,ist+1) = 0d0
         elemmt(    1,ist+2) = 0d0
         elemmt(    2,ist+2) = 1d0

      else if (stat==2 ) then

!     --- slip, the tangential force is a constant times the normal
!         force. The normal velocity is zero.

         ut1 = sign(1d0,usc(1)*t(1) + usc(2)*t(2))
         elemmt(ist+1,ist+1) = -ut1*t(1) + n(1)*mu
         elemmt(ist+2,ist+1) = -ut1*t(2) + n(2)*mu
         elemmt(    1,ist+2) = n(1)
         elemmt(    2,ist+2) = n(2)

      else if (stat==3) then

!     --- release, the tangential force is zero and the normal
!         velocity is zero.

         elemmt(ist+1,ist+1) = t(1)
         elemmt(ist+2,ist+1) = t(2)
         elemmt(1    ,ist+2) = n(1)
         elemmt(2    ,ist+2) = n(2)

      else

         write(*,*)'ERROR: this value of stat is not implemented'
         call instop

      end if

1000  call erclos ( 'elmfrict2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elmfrict2d'

      end if

      end
