      subroutine elp273(itcoor,x,y,wint,e1,e2,dldx1,dldx2,sign, &
                        delta,w,x1g,x2g,baricc,mint)
! ======================================================================
!
!        programmer     Jaap van der Zanden
!        version 5.4    date 13-02-1999 Include commons
!        version 5.3    date 26-12-1994 New commons cmessc and cmessg
!        version 5.2    date 23-06-1992 New common cmessg
!        version 5.1    date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version 5.0    date 19-02-1991 explicit decl; error messg
!        version 4.0    date 15-02-1988 Factor 2 pi for axisymmetric co-ord
!        version 3.0    date 28-02-1986
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!             straight triangle with mint integration points
!                               and either 6 or 7 nodal points
!     compute derivative of baricentric coordinates (itcoor=0)
!     revise weights
!     compute x1g,x2g  (itcoor > 0)
!     for itcoor see elp220 or elp221
! **********************************************************************
!
!                       KEYWORDS
!
!    2d
!    basis_function
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmoduleconsts
      use sepmodulecactl
      implicit none
      integer itcoor,mint
      double precision sign, delta, x(*), y(*), wint(*), e1(*), e2(*), &
             dldx1(*), dldx2(*), w(*), x1g(*), x2g(*), baricc(*)

!     baricc  i    ?
!     delta   i    jacobian of element
!     dldx1   i    ?
!     dldx2   i    ?
!     e1      i    ?
!     e2      i    ?
!     itcoor  i    ?
!     mint    i    ?
!     sign    i    ?
!     w       i    array of length m containing the weights for integration
!     wint    i    ?
!     x       i
!     x1g     i    ?
!     x2g     i    ?
!     y       i
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i,kk
      double precision ddelta,h,zeta1,zeta2,zeta3

!     ddelta  ?
!     h       Help variable to store a constant temporarily
!     i       Counting variable
!     kk      ?
!     zeta1   ?
!     zeta2   ?
!     zeta3   ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      e1(1)=y(3)-y(5)
      e1(2)=y(5)-y(1)
      e1(3)=y(1)-y(3)
      e2(1)=x(5)-x(3)
      e2(2)=x(1)-x(5)
      e2(3)=x(3)-x(1)
      delta=e1(3)*e2(1)-e1(1)*e2(3)

      if(delta) 30,20,40

!     --- error 105:  delta=0

20    call eropen( 'elp273' )
      call errint ( ielem, 1 )
      call errint ( itype, 2 )
      call errint ( ielgrp, 3 )
      call errsub ( 105, 3, 0, 0)
      call erclos( 'elp273' )

!     --- delta < 0

30    sign=-1
      ddelta=-delta
      goto 50

!     --- delta > 0

40    sign=1
      ddelta=delta

50    do i=1,3
         dldx1(i)=e1(i)/delta
         dldx2(i)=e2(i)/delta
      end do
      delta=ddelta
      if (itcoor.eq.0) then

!     --- newton-cotes integration

         do i=1,mint
            w(i)=wint(i)*delta
         end do
      else

!     --- gauss integration

         kk=1
         h=delta
         if ( itcoor.eq.1 ) h = h * 2d0* pi
         do i=1,mint
            zeta1=baricc(kk)
            zeta2=baricc(kk+1)
            zeta3=baricc(kk+2)
            x1g(i)=zeta1*x(1)+zeta2*x(3)+zeta3*x(5)
            x2g(i)=zeta1*y(1)+zeta2*y(3)+zeta3*y(5)
            if (itcoor.eq.3) then
               w(i)=wint(i)*h
            else
               w(i)=wint(i)*x1g(i)*h
            end if
            kk=kk+3
         end do
      end if
      end
