      subroutine presgrad3d ( p, psix, pg )
! ======================================================================
!
!        programmer    Dragan Vidovic
!        version  1.1  date 09-03-2013 Correction declaration
!        version  1.0  date 23-08-2005
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the the pressure gradient for the
!     incompressible Navier-Stokes equations
!     The pressure gradient for 6 points is stored in a 1x12 array pg
! **********************************************************************
!
!                       KEYWORDS
!
!     navier_stokes_equation
!     pressure_correction
!     pressure
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision psix(npsi,ndimlc), p(npsi), pg(ndimlc,*)

!     p              i    Pressure in three nodes
!     pg             o    Pressure gradient in six points
!     psix           i    Array of length n/2 x m x ndim  containing the
!                         derivatives of the linear basis functions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision pgl
      integer i, j

!     i             counter
!     j             counter
!     pgl           local holder for the partial derivative
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The pressure gradient in an element is computed from the pressure in
!     three nodes:
!
!     Gp = sum_j p(j) psix(j,1,.)
!
!     This gradient is copied to all six points.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i=1,ndimlc
         pgl = 0d0
         do j=1,npsi
            pgl = pgl + psix(j,i)*p(j)
         end do
         do j = 1, n
            pg(i,j) = pgl
         end do
      end do

      end
