      subroutine sepel0900div
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set npress and icent and allocate phipress, gradmat and divmat
!     Type 900: Navier-Stokes penalty function formulation
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0900div' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0900div'
         write(irefwr,1) 'n, m, npelm, ishape', n, m, E%npelm, E%ishape
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%icent = B%n
      if ( E%npelm==27 ) E%icent = 14
      E%npress = 1
      if ( itype==902 ) then
         if ( E%ishape==4 .or. E%ishape==6 .or. E%ishape==7 .or. &
              E%ishape==14 .or. E%ishape==16 ) E%npress = E%ndim+1
      end if  ! ( itype==902 )

      if ( debug ) write(irefwr,1) 'E%npress, E%ndim', E%npress, E%ndim

      allocate ( B%phipress(npsi,m), E%gradmat(E%ndim*B%n,E%ndim+1), &
                 E%rhsdiv(npsi), E%pressmat(E%npress,E%npress), &
                 E%rhsdivtr(npsi), E%transdiv(E%ndim*(B%n-1),E%ndim), &
                 E%gradv(9,m), E%a1(9,m), E%secinv(m), E%detdsc(m), &
                 E%submat(E%ndim*B%n,E%ndim*B%n), &
                 E%submass(E%ndim*B%n,E%ndim*B%n), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, m*E%npress+E%ndim*n*E%npress, &
                        'phipress' )

      B%phipress(1,:) = 1d0
      E%gradmat = 0d0
      E%pressmat = 0d0
      E%rhsdiv = 0d0
      E%rhsdivtr = 0d0
      E%gradv = 0d0

      if ( jtrans==1 .or. itype==902 .and. E%npelm==27 ) then

!     --- transformation, use subvec

         allocate ( E%subvec(E%ndim*B%n) )

      else

!     --- no transformation, vec

         E%subvec => E%vec

      end if  ! ( jtrans==1 .or. itype==902 .and. E%npelm==27 )

      if ( E%typecontinuity==0 .or. E%typecontinuity==4 ) then

!     --- typecontinuity = 0, 4, div matrix is transpose of grad matrix

         E%divmat => E%gradmat
         E%transpress => E%transdiv

      else

!     --- typecontinuity # 0, allocate div matrix

         allocate ( E%divmat(E%ndim*B%n,E%ndim+1), &
                    E%transpress(E%ndim*(B%n-1),E%ndim), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, E%ndim*n*E%npress, 'divmat' )
         E%divmat = 0d0
         E%transpress = 0d0

      end if  ! ( E%typecontinuity==0 )

      E%useetaeff = E%modelvisc==1 .or. E%modelvisc==5 .or. E%modelvisc==9

      if ( E%useetaeff ) then

!     --- etaeff = eta

         E%etaeff => E%coeffs(:,E%indeta)

      else

!     --- etaeff # eta

         allocate ( E%etaeff(m) )

      end if  ! ( E%useetaeff )
      E%useetaeff = .not. E%useetaeff

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel0900div'

      end if  ! ( debug )
      call erclos ( 'sepel0900div' )

      end subroutine sepel0900div
