      subroutine sepel900rhsnwt
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element vector for Navier-Stokes element due to newton
!     Old subroutine el4029
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision :: work(m)
      double precision, pointer :: elemvc1(:), elemvc2(:), elemvc3(:)
      double precision, pointer :: phix(:,:), phiy(:,:), phiz(:,:)
      double precision, pointer :: dudx(:,:,:)

!     dudx           derivatives of the solution vector in the quadrature
!                    point
!     elemvc1        first part of element vector
!     elemvc2        second part of element vector
!     elemvc3        third part of element vector
!     i              Counting variable
!     phix           dphi/dx
!     phiy           dphi/dy
!     phiz           dphi/dz
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     226   Possibility for power-law model has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900rhsnwt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900rhsnwt'
         call prinrl2 ( E%subvec, 2, n, 'elemvc' )
         call prinrl ( B%w, m, 'w' )
         call prinrl ( E%detdsc, m, 'detdsc' )
         call prinrl ( E%secinv, m, 'secinv' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      elemvc1 => E%subvec(1:E%ndim*n:E%ndim)
      elemvc2 => E%subvec(2:E%ndim*n:E%ndim)
      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      dudx => gradoldvec

      work = B%w * E%detdsc * E%secinv

      if ( debug ) call prinrl ( work, m, 'work' )

      if ( B%cart==1 .and. E%ndim==2 ) then

!     --- E%ndim = 2 and B%cart = 1, Cartesian co-ordinates

         do i = 1, n
            elemvc1(i) = elemvc1(i) + &
               sum(work*(2d0*phix(i,:)*dudx(:,1,1)+ &
                             phiy(i,:)*(dudx(:,1,2)+dudx(:,2,1)) ))
            elemvc2(i) = elemvc2(i) + &
               sum(work*(2d0*phiy(i,:)*dudx(:,2,2)+ &
                             phix(i,:)*(dudx(:,1,2)+dudx(:,2,1)) ))
        end do  ! i = 1, n

      else if ( E%ndim==3 ) then

!     --- E%ndim = 3 and B%cart = 1, Cartesian co-ordinates

         elemvc3 => E%subvec(3:E%ndim*n:E%ndim)
         phiz => B%phider(:,:,3)

         do i = 1, n
            elemvc1(i) = elemvc1(i) + &
               sum(work*(2d0*phix(i,:)*dudx(:,1,1)+ &
                             phiy(i,:)*(dudx(:,1,2)+dudx(:,2,1)) + &
                             phiz(i,:)*(dudx(:,1,3)+dudx(:,3,1)) ))
            elemvc2(i) = elemvc2(i) + &
               sum(work*(2d0*phiy(i,:)*dudx(:,2,2)+ &
                             phix(i,:)*(dudx(:,1,2)+dudx(:,2,1)) + &
                             phiz(i,:)*(dudx(:,2,3)+dudx(:,3,2)) ))
            elemvc3(i) = elemvc3(i) + &
               sum(work*(2d0*phiz(i,:)*dudx(:,3,3)+ &
                             phix(i,:)*(dudx(:,1,3)+dudx(:,3,1)) + &
                             phiy(i,:)*(dudx(:,2,3)+dudx(:,3,2)) ))
         end do  ! i = 1, n

      else

!     --- This possibility has not yet been implemented

         call errint ( B%cart, 1 )
         call errint ( E%ndim, 2 )
         call errsub ( 226, 2, 0, 0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( E%subvec, 2, n, 'elemvc' )
         write(irefwr,*) 'End sepel900rhsnwt'

      end if  ! ( debug )
      call erclos ( 'sepel900rhsnwt' )

      end subroutine sepel900rhsnwt
