      subroutine sepela1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the A1 tensor, defined by grad v + grad v^T
!     Navier-Stokes equations
!     old subroutine sepelnsa1
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     gradient
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: gradv(:,:), a1(:,:)

!     a1             A1 tensor defined as grad v + grad v^T
!     gradv          Array containing the gradient vector in
!                    the quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The A1 tensor is defined as grad v + grad v^T
!     This tensor is symmetric
!     The sequence is the same as for the stress tensor, hence
!     2d cartesian:  A1_xx, A1_yy, 0, A1_xy
!     3d cartesian:  A1_xx, A1_yy, A1_zz, A1_xy, A1_yz, A1_zx
!     Axisymmetric coordinates: A1_rr, A1_zz, A1_phi_phi, A1_rz,
!                               A1_z_phi, A1_phi_r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepela1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepela1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      a1 => E%a1
      gradv => E%gradv

      if ( E%ndim==2 .and. (B%cart==1 .or. B%cart==3) ) then

!     --- E%ndim = 2, Cartesian or Polar co-ordinates

         a1(1,:) = 2d0*gradv(1,:)
         a1(2,:) = 2d0*gradv(4,:)
         a1(4,:) = gradv(2,:)+gradv(3,:)

      else if ( E%ndim==2 .and. B%cart==2 ) then

!     --- E%ndim = 2, Cylinder co-ordinates

         a1(1,:) = 2d0*gradv(1,:)
         a1(2,:) = 2d0*gradv(4,:)
         a1(3,:) = 2d0*gradv(5,:)
         a1(4,:) = gradv(2,:)+gradv(3,:)

      else

!     --- E%ndim = 3

         a1(1,:) = 2d0*gradv(1,:)
         a1(2,:) = 2d0*gradv(5,:)
         a1(3,:) = 2d0*gradv(9,:)
         a1(4,:) = gradv(2,:)+gradv(4,:)
         a1(5,:) = gradv(6,:)+gradv(8,:)
         a1(6,:) = gradv(3,:)+gradv(7,:)

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepela1'

      end if  ! ( debug )
      call erclos ( 'sepela1' )

      end subroutine sepela1
