      subroutine sepeleffvisc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the effective viscosity depending on the
!     viscosity model used
!     Old subroutine sepelvisc
! **********************************************************************
!
!                       KEYWORDS
!
!     viscosity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELA1        Compute the A1 tensor, defined by grad v + grad v^T
!     SEPELEFFVISC1  computes the effective viscosity depending on the
!                    viscosity model used
!     SEPELGRADV     Compute gradient tensor in case of Navier-Stokes
!     SEPELSECINV    Compute second inverse in case of Navier-Stokes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeleffvisc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeleffvisc'
         write(irefwr,1) 'modelv', E%modelvisc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%modelvisc/=1 .and. E%modelvisc/=10 ) then

!     --- Compute gradv tensor

         call sepelgradv

         if ( E%modelvisc/=100 ) then

!        --- Compute a1 tensor as well as second inverse

            call sepela1
            call sepelsecinv

         end if  ! ( E%modelvisc/=100 ) then

      end if  ! ( E%modelvisc/=1 .and. E%modelvisc/=5 .and. E%modelvisc/=10 )

      call sepeleffvisc1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeleffvisc'

      end if  ! ( debug )
      call erclos ( 'sepeleffvisc' )

      end subroutine sepeleffvisc
