      subroutine sepelgettime
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extract time parameter from iuser
!
! **********************************************************************
!
!                       KEYWORDS
!
!     navier_stokes_equation
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmoduleelem
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision thetdt

!     thetdt          1/(theta dt) for the case of the theta method
!                     immediately applied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     988   itime = 1 in combination with tstep<=0 not allowed
!    1541   The combination imas = 1 and ishape=4 is not permitted
!    2438   jtime > 0 and imas > 0 ( double use of mass matrix)
!    2779   linear elements and time dependent not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelgettime' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelgettime'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%typetime = inirepscal ( E%iuser(E%ipos) ) ! type of time integration

      if ( E%typetime>0 ) then
         E%compold = 1
         E%filloldderiv = .true.
      end if  ! ( E%typetime>0 )

      E%seq_second_old_sol = E%typetime/10
      E%typetime = mod(E%typetime,10)
      E%ipos = E%ipos+1
      E%ishift = E%ishift+1

      if ( imas==0 ) notmas = 1
      if ( E%typetime==0 ) then

         E%typetime = 1+(imas-1)*10

      else

         if ( itype==903 ) E%typetime = 101
         if ( tstep<=0d0 ) then

!        --- E%typetime>0 and tstep=0

            call erreal ( tstep, 1 )
            call errsub ( 988, 0, 1, 0 )

         else

            thetdt = 1d0 / (theta * tstep)

         end if
         if ( imas/=0 ) then

!        --- imas>0, hence a mass matrix must be built
!            However, also the mass matrix is used in the stiffness
!            matrix. This means a double use of mass matrix and hence
!            an error message is given.

            call errint ( E%typetime , 1 )
            call errint ( imas, 2 )
            call errsub ( 2438, 2, 0, 0 )

         end if
         E%typetime = 101

      end if
      if ( imas==1 .and. (E%ishape==4 .or. E%ishape==12) ) then

!     --- error E%ishape=4 or 12 may not be combined with a diagonal mass
!         matrix

         call errint ( E%ishape, 1 )
         call errint ( 1, 2 )
         call errint ( imas, 3 )
         call errsub ( 1541, 3, 0, 0 )

      end if

      if ( itype==903 .and. E%typetime>0 .and. &
          (E%ishape==3 .or. E%ishape==11) ) then

!     --- error linear elements and time dependent not yet implemented
!         for type 903

         call errint ( E%ishape, 1 )
         call errint ( itype, 2 )
         call errsub ( 2779, 2, 0, 0 )

      end if  ! ( E%typetime>0 .and. (E%ishape==3 .or. E%ishape==11) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelgettime'

      end if  ! ( debug )
      call erclos ( 'sepelgettime' )

      end subroutine sepelgettime
