      subroutine sepelm900matal
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store extra element matrices which are necessary for
!     schur complement type methods
!     Old subroutine elm900matal
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmoduleelm
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, j
      double precision :: kappa(m), g(m)
      double precision, pointer :: s(:)

!     g              work array
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     kappa          Cis used to store the inverse of the viscosity
!     s              sub element matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900matal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900matal'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store weighted pressure mass matrix, diagonal of pressure mass
!         matrix and diagonal of velocity mass matri
!         Set nvel, i.e. number of velocity points

      if ( debug ) call prinrl ( B%w2, n, 'w part 2' )

!     --- Fill diagonal of velocity mass matrix

      if ( E%ishape==14 ) then

!     --- 27 point element

         do i = 1, E%ndim
            s => E%dvelmmat(i:3*14:3)
            s = B%w2(1:14)
            s => E%dvelmmat(i+3*15:3*n:3)
            s = B%w2(15:27)
         end do  ! i = 1, ndim

      else

!     --- Standard element

         do i = 1, E%ndim
            s => E%dvelmmat(i:E%ndim*n:E%ndim)
            s = B%w2(1:n)
         end do  ! i = 1, ndim

      end if  ! ( E%ishape==14 )
      if ( debug ) then
         write(irefwr,1) 'n, icount, npres', &
                          n, icount, E%npress
         call prinrl ( E%dvelmmat, icount-E%npress, 'vel_mass_mat' )
      end if  ! ( debug )

!     --- Fill weighted pressure mass matrix

      kappa = B%w/E%etaeff
      do j = 1, E%npress

         g = kappa*B%phipress(j,:)

         do i = 1, E%npress
            E%wpressmmat(i,j) = sum ( g*B%phipress(i,:) )
         end do  ! i = 1, E%npress

      end do  ! j = 1, E%npress
      if ( debug ) &
         call prinrl1 ( E%wpressmmat, E%npress, E%npress, 'wpress_mass_mat' )

!     --- Fill diagonal of pressure mass matrix

      do i = 1, E%npress
         E%dpressmmat(i) = sum(B%w*B%phipress(i,:)**2)
      end do  ! i = 1, E%npress
      if ( debug ) &
         call prinrl ( E%dpressmmat, E%npress, 'press_mass_mat' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelm900matal'

      end if  ! ( debug )
      call erclos ( 'sepelm900matal' )

      end subroutine sepelm900matal
