      subroutine sepelm900trans
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the transformation matrix
!     Compute the velocity in the centroid and the derivative of u
!     Set matelim
!     Old subroutine elm900trans
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, n1

!     i              Counting variable
!     j              Counting variable
!     n1             n-1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPELTRANS     Compute the transformation matrix to be used in the
!                    elimination process of the centroid point for
!                    incompressible Navier-Stokes
!     SEPELUDERIV    Compute the derivative of a given function in one element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900trans' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900trans'
         write(irefwr,1) 'mcont, modelv', E%typecontinuity, E%modelvisc
         call prinrl1 ( E%gradmat, E%ndim+1, E%ndim*n, 'sp' )
         call prinrl1 ( E%divmat, E%ndim+1, E%ndim*n, 'div' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%trans => E%transdiv
      E%divgrmat => E%divmat
      call sepeltrans ( E%mdiv )

      if ( debug ) &
         call prinrl1 ( E%transdiv, E%ndim, E%ndim*(n-1), 'transdiv' )

      if ( E%typecontinuity/=0 .and. E%typecontinuity/=4 ) then
         E%trans => E%transpress
         E%divgrmat => E%gradmat
         call sepeltrans ( 0 )
      end if  ! ( E%typecontinuity/=0 .and. E%typecontinuity/=4 )

      if ( debug ) then
         call prinrl1 ( E%transpress, E%ndim, E%ndim*(n-1), 'transpress' )
         write(irefwr,1) 'E%typeconvect, E%modelvisc, iseqprevsol', &
                          E%typeconvect, E%modelvisc, iseqprevsol
         write(irefwr,1) 'indvector(iseqprevsol)', indvector(iseqprevsol)
      end if  ! ( debug )

      if ( E%filloldvec ) then

!     --- It is necessary to extract the velocity and to compute the
!         derivatives of u
!         Compute extra unknowns by transformation
!                  n-1
!         u (n) =  sum  trans (l)  u (l)
!          k       l=1       k      k

         n1 = n-1
         do j = 1, E%ndim
            uprev(n,j) = 0d0
            do i = 1, E%ndim
               uprev(n,j) = uprev(n,j) + &
                   dot_product ( E%transdiv(i:n1*E%ndim:E%ndim,j), &
                                 uprev(1:n1,i) )
            end do
         end do

!        --- correct by right-hand side

         if ( E%mdiv==1 ) uprev(n,:) = uprev(n,:) - E%rhsdivtr

         if ( B%jdiag/=1 ) then

!        --- Compute solution in Gauss points

            do j = 1, E%ndim
               do i = 1, m
                  E%ugauss(i,j) = dot_product ( uprev(:,j), B%phi(:,i) )
               end do
            end do

         end if  ! ( B%jdiag/=1 )

!        --- Fill derivative of prior vector and store in gradoldvec

         call sepeluderiv

      end if  ! ( E%filloldvec )

      if ( .not. matrix .and. E%mdiv/=0 ) then

!     --- The matrix is not build, but the right-hand side is
!         The right-hand side may contain a part due to compressibility
!         In that case we have to build the element matrix, since it is
!         used for elimination
!         This is only necessary in case rhsdivtr not zero

         do i = 1, E%ndim

            if ( abs(E%rhsdivtr(i))>0d0 )then

!           --- rhsdivtr#0, set matelim true

               E%matelim = .true.
               exit

            end if  ! ( abs(E%rhsdivtr(i))>0d0 )

         end do  ! i = 1, E%ndim

      end if  ! ( .not. matrix .and. E%mdiv/=0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelm900trans'

      end if  ! ( debug )
      call erclos ( 'sepelm900trans' )

      end subroutine sepelm900trans
