      subroutine sepelm900vecf
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create element vector for Navier-Stokes equations
!     using separate subvectors
!     Old subroutine elm900vec
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPELM900VECDI penalty right-hand side for the incompressible
!                    Navier-Stokes equations
!     SEPELM900VECEL Eliminate the centroid point for the incompressible
!                    Navier-Stokes equations in case of an extended
!                    quadratic triangle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900vecf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900vecf'
         write(irefwr,1) 'jtrans, mdiv, inpelm, npres, itype', &
                          jtrans, E%mdiv, E%npelm, E%npress, itype
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Eliminate centroid
!         The resulting vector is stored in elemvc

      call sepelm900vecel

!     --- divergence right-hand side

      if ( E%mdiv==1 ) call  sepelm900vecdi

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, icount, 'elemvc' )
         write(irefwr,*) 'End sepelm900vecf'

      end if  ! ( debug )
      call erclos ( 'sepelm900vecf' )

      end subroutine sepelm900vecf
