      subroutine sepelnpress
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute npress (number of pressure unknowns)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     initialization
!     pressure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: npress

!     npress         Number of pressure unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnpress' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnpress'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npress = 1

      if ( itype==903 ) then

         select case ( E%ishape )
            case(3,5,11)
               npress = inpelm
            case(4,10)
               npress = 3
            case(6,9,12,18)
               npress = 4
            case(14,17)
               npress = 8
         end select  ! case ( E%ishape )

      else if ( itype==902 ) then

         if ( E%ishape==6 .or. E%ishape==7 .or.E%ishape==14 .or. &
              E%ishape==16 ) &
            npress = E%ndim+1

      end if  ! ( itype==903 )

      E%npress = max(E%npress,npress)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelnpress'

      end if  ! ( debug )
      call erclos ( 'sepelnpress' )

      end subroutine sepelnpress
