      subroutine el3034 ( elemmt, Kxx, Kxg, Kgx, Kgg, &
                          isequ, iseqgam)
! ======================================================================
!
!        programmer    Martijn Cox & Niels Driessen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 04-07-2004 derived from el3033
!
!   copyright (c) 2004-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Assemble element stiffness matrix for contact element
!    elm211 (CR) and elm212 (TH)
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isequ(n*ndim), iseqgam(n)
      double precision Kxx(n*ndim,n*ndim), &
                       Kxg(ndim*n,n), Kgx(n,n*ndim), Kgg(n,n), &
                       elemmt(icount,icount)

!     elemmt        i/o   Element matrix to be filled
!     iseqgam        i    Sequence for contact pressure dof's
!     isequ          i    Sequence for displacement dof's
!     kgg            i    Matrix
!     kgx            i    Matrix
!     kxg            i    Matrix
!     kxx            i    Matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision matr_tmp(icount,icount)

!     i              counting variable
!     j              counting variable
!     matr_tmp       array of size icount x icount where elemmt
!                    is stored temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initialise the element matrix

      do i = 1, icount
         do j = 1, icount
            matr_tmp(i,j) = 0d0
            elemmt(i,j) = 0d0
         end do
      end do

!     --- fill in the element matrix, based on isequ

      do i = 1, n*ndim
         do j = 1, n*ndim
            matr_tmp(isequ(i),isequ(j)) = Kxx(i,j)
         end do
      end do

!     --- fill in the element matrix, based on isequ and iseqgam

      do i = 1, n
         do j = 1, n*ndim
            matr_tmp(iseqgam(i),isequ(j)) = Kgx(i,j)
            matr_tmp(isequ(j),iseqgam(i)) = Kxg(j,i)
         end do
      end do

!     --- fill in the element matrix, based on iseqgam

      do i = 1, n
         do j = 1, n
            matr_tmp(iseqgam(i),iseqgam(j)) = Kgg(i,j)
         end do
      end do

!     --- store transposed

      do i = 1, icount
         do j = 1, icount
            elemmt(i,j) = matr_tmp(j,i)
         end do
      end do

      end
