      subroutine eldu2dfg ( f, detf, xgauss, ugauss, nip, ndim, jcart )
! ======================================================================
!
!        programmer    Sander Gielen
!        version  1.3  date 14-09-2005 Roy Kerckhoffs: repaired
!                                      computation of detf
!        version  1.2  date 21-01-1998 Marcel Rutten,addition of 2D
!        version  1.1  date 12-11-1997 added detf computation
!        version  1.0  date 28-03-1996
!
!   copyright (c) 1996-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     make deformation gradient from displacement gradient
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nip, ndim, jcart
      double precision f(nip,ndim,ndim), detf(nip), ugauss(nip,ndim), &
                       xgauss(nip,ndim)

!     detf       o    determinant of f
!     f         i/o   At input:  dudx(k,j,i)
!                     At output:
!                     Deformation gradient f(k,j,i) = F_ij( x^k)
!     jcart      i    indication whether the element is a triangle with
!                     cartesian coordinates (1)
!                     or a circular triangle with cylinder symmetric
!                     coordinates (2)
!     ndim       i    Dimension of the space.
!     nip        i    number of integration points
!     ugauss     i    Preceding solution in Gauss points
!     xgauss     i    array of length m x ndim containing the co-ordinates of
!                     the gauss points.  array xgauss is only filled when gauss
!                     integration is applied.
!                     xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                     zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k

!     j          Counting variable
!     k          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                        c
!            F = (grad u) + I
!     grad u is already stored conjugated in dfgrad
!     see EL3007
!          k
!     F  (x ) is stored as: f(k,j,i)  k: integration point
!      ij                           i,j: components
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do k = 1, nip
         do j = 1, ndim

            f(k,j,j) = f(k,j,j) + 1d0

         end do

      end do

!     --- Determinant of deformation gradient:

      if ( ndim==3 ) then

        do k = 1, nip
           detf(k) = &
                   f(k,1,1)*( f(k,2,2)*f(k,3,3) - f(k,2,3)*f(k,3,2) ) - &
                   f(k,1,2)*( f(k,2,1)*f(k,3,3) - f(k,2,3)*f(k,3,1) ) + &
                   f(k,1,3)*( f(k,2,1)*f(k,3,2) - f(k,2,2)*f(k,3,1) )
        end do

      else if ( ndim==2 ) then

        do k = 1, nip

          detf(k) = f(k,1,1)*f(k,2,2) - f(k,1,2)*f(k,2,1)

!         --- Extra term for axisymmetric case:

          if ( jcart==2 ) &
            detf(k) = detf(k) * (1d0 + ugauss(k,1)/xgauss(k,1))

        end do

      end if

      end
