      subroutine elmul04 (a, b, out, opt)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  1.0  date 28-05-2001
!
!   copyright (c) 1996-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    multiply symmetric stored matrices, result is reduced stored
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     rotation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision a(6), b(6), out(6)
      integer opt

!     a          i    matrix to be multiplied
!     b          i    matrix to be multiplied
!     opt        i    option: 0: a*b
!                             1: a*b'
!                             2: a'*b
!     out        o    result
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision work(3,3), work2(3,3), work3(3,3)
      integer i

!     i          Counting variable
!     work       Work array
!     work2      Work array
!     work3      Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DGEMM     BLAS routine for matrix multiplication general.
!               (NAG: f06yaf)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- put a in work

      do i=1,3
         work(i,i) = a(i)
      end do
      work(1,2) = a(4)
      work(2,1) = a(4)
      work(2,3) = a(5)
      work(3,2) = a(5)
      work(1,3) = a(6)
      work(3,1) = a(6)

!     --- put b in work

      do i=1,3
         work2(i,i) = b(i)
      end do
      work2(1,2) = b(4)
      work2(2,1) = b(4)
      work2(2,3) = b(5)
      work2(3,2) = b(5)
      work2(1,3) = b(6)
      work2(3,1) = b(6)
      if (opt.eq.0) then

!     --- opt = 0
!         compute work2=a*b

         call dgemm ('n', 'n', 3, 3, 3, 1d0, work, 3, work2, 3, 0d0, &
                     work3, 3)
      else if (opt.eq.1) then

!     --- opt = 1
!         compute work2=a*b'

         call dgemm ('n', 't', 3, 3, 3, 1d0, work, 3, work2, 3, 0d0, &
                     work3, 3)
      else if (opt.eq.3) then

!     --- opt = 2
!         compute work2=a'*b

         call dgemm ('t', 'n', 3, 3, 3, 1d0, work, 3, work2, 3, 0d0, &
                     work3, 3)
      end if
      do i=1,3
         out(i) = work3(i,i)
      end do
      out(4) = work3(1,2)
      out(5) = work3(2,3)
      out(6) = work3(1,3)
      end
