      subroutine elrotsym ( s, q, opt )
! ======================================================================
!
!        programmer    Sander Gielen
!        version  1.0  date 28-03-1996
!
!   copyright (c) 1996-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    rotate symmetrical matrix s (reduced stored), using matrix q
!    result is again symmetric
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     rotation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision s(6), q(3,3)
      integer opt

!     opt        i    option: 0: rotate from global to local coordinates
!                             1: rotate from local to global coordinates
!     q          i    rotation matrix
!                     every column is a local vector expressed in global
!                     coordinates
!     s         i/o   matrix to rotate stored symmetrical
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision work(3,3), work2(3,3)
      integer i

!     i          Counting variable
!     work       Work array
!     work2      Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DGEMM     BLAS routine for matrix multiplication general.
!               (NAG: f06yaf)
!     DSYMM     BLAS routine for matrix multiplication with a symmetric
!               matrix.  (NAG: f06ycf)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       s = q' s q   (opt=0) Global to local
!       s = q  s q'  (opt=1) Local to global
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- put s in work

      do 100, i=1,3
         work(i,i) = s(i)
100   continue
      work(1,2) = s(4)
      work(2,1) = s(4)
      work(2,3) = s(5)
      work(3,2) = s(5)
      work(1,3) = s(6)
      work(3,1) = s(6)
      if ( opt==0 ) then

!     --- opt = 0
!         compute work2=work.q

         call dsymm( 'l', 'u', 3, 3, 1d0, work, 3, q, 3, 0d0, work2, 3)

!        --- compute work=q'.work2

         call dgemm( 't', 'n', 3, 3, 3, 1d0, q, 3, work2, 3, 0d0, &
                       work, 3)
      else

!     --- opt = 1
!         compute work2=q.work

         call dsymm( 'r', 'u', 3, 3, 1d0, work, 3, q, 3, 0d0, work2, 3)

!        --- compute work=work2.q'

         call dgemm( 'n', 't', 3, 3, 3, 1d0, work2, 3, q, 3, 0d0, &
                      work, 3)
      end if

!     --- get transformed s out work

      do 110, i=1,3
         s(i) = work(i,i)
110   continue
      s(4) = work(1,2)
      s(5) = work(2,3)
      s(6) = work(1,3)
      end
