      subroutine elsgmgls ( e, f, detf, detfr, nip )
! ======================================================================
!
!        programmer     Sander Gielen
!        version 1.0    date   12-11-1997 derived from ELDF2GLS
!
!   copyright (c) 1997-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Help routine for nonlinear solid analysis.
!     Computes MODIFIED Green-Lagrange strains from the deformation
!     gradient in case of Souza method
! **********************************************************************
!
!                       KEYWORDS
!
!    mechanics
!    non_linear
!    strain
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer nip
      double precision e(6,nip), f(nip,3,3), detf(nip), detfr

!     detf        i     determinant of ORIGINAL f
!     detfr       i     determinant of f at reduced integration point
!     e           o     MODIFIED Green-Lagrange strains e(i,k)
!                       k: intpt; i: component
!                                                             k
!     f           i     Deformation gradient f(k,j,i) = F  ( x )
!                                                        ij
!     nip         i     number of integration points
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision cr

!     cr         correction term for modified strains.
!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     with F is modified deformation
!                   c
!            E = ( F *F - I )/2
!
!            E  = ( F  F  - delta  )/2 = (f(p,i,k)*f(p,j,k) - delta(i,j))/2
!             ij     ki kj       ij
!
!     with F modified deformation
!
!            detf = DET(F)
!
!     modification is taken into account with "cr" variable
!
!     As E is symmetric the components are stored in the sequence
!     e(1,k) = E11;       e(2,k) = E22;       e(3,k) = E33
!     e(4,k) = E12 = E21; e(5,k) = E23 = E32; e(6,k) = E13 = E31
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

      do k=1,nip

         cr = ( detfr/detf(k) )**(2d0/3d0)

         do j=1,3
            e(j,k) = ( f(k,j,1)*f(k,j,1) + f(k,j,2)*f(k,j,2) + &
                       f(k,j,3)*f(k,j,3) )*cr*0.5d0 - 0.5d0
            i = j+1
            if ( j.eq.3 ) i = 1
            e(j+3,k) = ( f(k,j,1)*f(k,i,1) + f(k,j,2)*f(k,i,2) + &
                         f(k,j,3)*f(k,i,3) )*cr*0.5d0
         end do

      end do

      end
