      subroutine sepfillbbmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill matrix B^b for plate problem
!     Old subroutine el8109
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
!     thick_plate
!     plate
!     bending
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: dphidx(:,:,:)
      integer :: i

!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                         d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   The matrix b^b is defined by:
!
!           | 0     0    -dphi/dx |
!     B^b = | 0  dphi/dy     0    |
!           | 0  dphi/dx -dphi/dy |
!
!   Mark that the definition of theta differs from the Hughes definition in
!   case of plates
!   It is however, the same as in the case of shells
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillbbmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillbbmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidx => B%phider

      do i = 1, n
         E%Bb(1,3,i,:) =-dphidx(i,:,1)
         E%Bb(2,2,i,:) = dphidx(i,:,2)
         E%Bb(3,2,i,:) = dphidx(i,:,1)
         E%Bb(3,3,i,:) =-dphidx(i,:,2)
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillbbmat'

      end if  ! ( debug )
      call erclos ( 'sepfillbbmat' )

      end subroutine sepfillbbmat
