      subroutine askvecderiv ( inpsep, istart, table, tabmax, tabout,
     +                         nprob, number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-05-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret input for derivatives following vi = derivatives
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/cinout'
      include 'SPcommon/cscalvec'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:8), istart, nprob, number
      double precision table(*), tabmax(*), tabout(*)

!     inpsep         o    The integer input read is stored in array INPSEP of
!                         length 9
!                         Array INPSEP is filled as follows:
!                         0: Type of function to be called
!                             8:  derivatives
!                         1: Sequence number corresponding to input for
!                            coefficients
!                            If iseqcf = -1, the next one will be used
!                         2: IPROB   Problem sequence number
!                         3: IVECTOR Sequence number output vector
!                                    (first vector)
!                            If ivector = -1, the same value as iprob
!                            will be used
!                            If iseqdr = -1, the next one will be used
!                            If ivector = -2, no output vector is used
!                         4: ISEQDR Sequence number derivatives
!                            if -10: no block is read
!                         5: ISCALAR Sequence number scalar, is only used if
!                            boundary integrals are computed.
!                            Otherwise iscalar should be 0
!                            If iscalar>0, ivector must be -2
!                         6: ICHELD Choice parameter
!                         7: ISEQINPVEC Sequence number input vector
!                         8: IVEC Type of output vector
!     istart        i/o   Next free position in inpsep
!     nprob          i    Number of various problems
!     number         i    Vector sequence number
!     table          i    Contains the numbers read for the various keywords
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, iscal, iposut, m, maxtable
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iposut         Position in array tabout
!     iscal          Sequence number of scalar
!     ivec           Sequence number of vector
!     m              number of values stored in tabout for keyword
!     maxtable       Maximum value found in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1506   Both VECTOR and SCALAR given in DERIVATIVES
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecderiv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecderiv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iposut = nint(table(3))
      m = nint(tabmax(3))
      ivec = -1
      if ( m>0 ) ivec = nint(tabout(iposut))   ! input vector

      iposut = nint(table(8))
      m = nint(tabmax(8))
      iscal = -1
      if ( m>0 ) iscal = nint(tabout(iposut))   ! scalar

      if ( ivec==-1 ) then

!     --- Vector not given

         if ( iscal==-1 ) then
            ivec = nprob+1
         else
            ivec = -2
         end if

      else

!     --- Vector has been given, so scalar is not allowed

         if ( iscal>0 ) then
            call errsub ( 1506, 0, 0, 0 )
            call errcrd
         else
            iscal = 0
         end if
      end if

      maxtable = nint(max(table(3), table(31)))

      inpsep(0) = 8   ! icode for derivatives
      inpsep(1) = nint(table(32))   ! seq_coef
      inpsep(2) = nint(table(18))   ! problem
      inpsep(3) = number            ! output vector
      inpsep(4) = nint(table(33))   ! seq_deriv
      if ( inpsep(4)<0 .and. maxtable>0 ) inpsep(4) = -10  ! Use derivatives
      inpsep(5) = iscal             ! scalar
      inpsep(6) = max(1,nint(table(31)))   ! icheld
      inpsep(7) = ivec              ! input vector
      inpsep(8) = 0                 ! type of output vector
      istart = istart + 9
      if ( number>0 .and. number<=maxvectors ) ivectors(number) = 1

1000  call erclos ( 'askvecderiv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askvecderiv'

      end if  ! ( debug )

      end
