      subroutine erpres ( ibuffr, buffer, kmesh, kprob, isol, inpsep,
     +                    maxvec, numvec, istart, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-07-2011 Major revision
!        version  3.0  date 04-10-2010 Extra parameters
!        version  2.2  date 16-08-2004 keywords
!
!   copyright (c) 1993-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call subroutine PRESBC for the essential boundary conditions
!     Fill array IINPRS with the values stored in INPSEP
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/clinsl'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kprob(*), isol(5,*), inpsep(*), maxvec, numvec,
     +        ibuffr(*), istart
      double precision buffer(*), rinsep(*)

!     buffer        i/o   Real buffer array in which all large data is stored
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         i    The integer input read is stored in array INPSEP
!                         1  ISEQNR     Sequence number corresponding to input
!                                       If iseqnr = -1, the next one will be
!                                       used
!                         2  IPROB      Problem sequence number
!                                       Not used
!                         3  IVECTOR    Vector sequence number (first vector)
!                                       If ivector = -1, the same value as iprob
!                                        will be used
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
!     istart        i/o   Starting address in array INPVEC
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     maxvec         i    Maximum number of vectors per time-step permitted
!     numvec        i/o   Largest sequence number of vector that has been filled
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, ivec, iseqnr, ivect1, iinprs(4), icall,
     +        maxcal, istartsepin, ipblc2, ipblc3, mmiinput,
     +        mmrinput, mmiinputinf, ipiinput, ipiinputinf, iprinput,
     +        istrin, nreal, numint
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     icall          Input sequence number for present subroutine
!     iinprs         Input array defined by the user
!                    With this array the user may indicate which actions
!                    he wants to be performed by PRESBC
!                    IINPRS must be filled as follows:
!                    Pos. 1: Last entry number that has been filled by the
!                            user. If 0 or 1 only defaults are used
!                         2: IREAD Defines how the input must be read
!                                  Possible values:
!                                  0:  All SEPRAN input has been read by
!                                      subroutine SEPSTN until
!                                      END_OF_SEPRAN_INPUT
!                                      or end of file has been found
!                                      This input is used
!                                  1:  The input is read as described for PRESDF
!                                  2:  The last input read as described for
!                                      PRESDF is used
!                                      (equivalent with PRESTM ICTIME=1)
!                            Default value: 0
!                         3: ISEQNR, This parameter is only used if IREAD=0
!                            It indicates the sequence number of the input
!                            for essential boundary conditions to be used
!                            Default value: 1
!     ipblc2         Starting address of Block 2 of ISEPIN
!     ipblc3         Starting address of Block 3 of ISEPIN
!     ipiinput       Starting address of array iinput
!     ipiinputinf    Starting address of array iinputinf
!     iprinput       Starting address of array rinput
!     iseq           Sequence number read
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     istartsepin    Starting address in array ISEPIN
!     istrin         Starting address of information for this subroutine
!                    in RINSEP
!     ivec           Sequence number of vector of special structure
!     ivect1         Sequence number of first vector
!     maxcal         Maximum value of icall
!     mmiinput       Memory management sequence number with respect to
!                    iinput
!     mmiinputinf    Memory management sequence number with respect to
!                    iinputinf
!     mmrinput       Memory management sequence number with respect to
!                    rinput
!     nreal          Number of reals stored
!     numint         Number of integers stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inigettext, inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERPRES1        Store input arrays into arrays in ibuffr such that they
!                    can be seen by the standard SEPRAN read subroutines
!     ERPRES2        Remove the input arrays in arrays in ibuffr that were used
!                    to store the input
!     INI0152        Compute information concerning input read by readal
!     INI066TEXT     delete temporary array
!     INI080         Create permanent space in array BUFFER
!     INI081         Create permanent space in array IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRESBCBF       Read essential boundary conditions
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ivect1 /0/
! ======================================================================
!
      call eropen ( 'erpres' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erpres'
         write(irefwr,1) 'istart', istart
         call prinin ( inpsep, 4, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqnr = 1
      iseq   = inpsep(1)
      if ( iseq>0 ) iseqnr = iseq

      ivect1 = min ( ivect1 + 1, maxvec )
      ivec   = inpsep(3)
      if ( ivec/=-1 ) ivect1 = ivec
      iinprs(1) = 4
      iinprs(2) = 0
      iinprs(3) = iseqnr
      iinprs(4) = ivect1

      if ( iseq/=-10 ) then

!     --- Standard case, use sequence number

         call presbcbf ( ibuffr, buffer, kmesh, kprob, isol,
     +                   iinprs )
         istart = istart + 4

         numvec = max(numvec,iinprs(4))

      else

!     --- New case, all input in inpsep and rinsep
!         Create a new entry with respect to the input file and store
!         the input temporarily at these positions

         mmiinput = 0
         mmrinput = 0
         mmiinputinf = 0

!        --- Create space for the input arrays

         numint = inpsep(4)
         nreal = inpsep(5)
         istrin = inpsep(6)
         call ini081 ( ibuffr, mmiinput, numint+7, 'iinput_tmp' )

         if ( nreal>0 ) then

!        --- There are real arrays

            call ini080 ( ibuffr, mmrinput, nreal, 'rinput_tmp' )
            call ini081 ( ibuffr, mmiinputinf, nreal, 'inputinf_tmp' )

         end if  ! ( nreal>0 )

!        --- Compute certain start addresses

         call ini0152 ( ibuffr, isepin, icall, maxcal, istartsepin,
     +                  ipblc2, ipblc3, 5 )
         if ( debug )
     +      write(irefwr,1) 'icall, maxcal, istartsepin',
     +                       icall, maxcal, istartsepin

         ipiinput = inigettext ( mmiinput, 'iinput_tmp' )
         if ( nreal>0 ) then

!        --- There are real arrays

            iprinput = inidgttext ( mmrinput, 'rinput_tmp' )
            ipiinputinf = inigettext ( mmiinputinf, 'iinputinf_tmp' )

         else

!        --- There are no real arrays

            iprinput = 1
            ipiinputinf = 1

         end if  ! ( nreal>0 )

!        --- Fill input in temporary input block

         call erpres1 ( ibuffr(ipblc2), ibuffr(ipblc3), 5, iseqnr,
     +                  mmiinput, mmrinput, mmiinputinf,
     +                  ibuffr(ipiinput), buffer(iprinput),
     +                  ibuffr(ipiinputinf), numint, nreal, inpsep(7),
     +                  rinsep(istrin), inpsep(7+numint), inpsep(2) )

         iinprs(3) = iseqnr  ! store sequence number of temporary block

!        --- Fill boundary conditions

         call presbcbf ( ibuffr, buffer, kmesh, kprob, isol,
     +                   iinprs )

         numvec = max(numvec,inpsep(3))
         istart = istart + 7 + numint+nreal

!        --- Remove arrays

         if ( nreal>0 ) then

!        --- There are real arrays

            call ini066text ( ibuffr, mmiinputinf, 'inputinf_tmp' )
            call ini066text ( ibuffr, mmrinput, 'rinput_tmp' )

         end if  ! ( nreal>0 )
         call ini066text ( ibuffr, mmiinput, 'iinput_tmp' )

!        --- Recompute starting addresses

         call ini0152 ( ibuffr, isepin, icall, maxcal, istartsepin,
     +                  ipblc2, ipblc3, 5 )

!        --- Remove temporary input block

         call erpres2 ( ibuffr(ipblc2), ibuffr(ipblc3), 5 )

      end if  ! ( iseq/=-10 )

1000  call erclos ( 'erpres' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End erpres'

      end if  ! ( debug )

      end
