      subroutine intx02ext1 ( code, ncode, nncode, itype, table, codemn,
     +                        nmcode, nmain, iretur, tabmin, tabmax,
     +                        tabout, nout, strout, nstrng, ncodmx,
     +                        iseqkeyw, nseqkeyw, errorcode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-07-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read rest of record and check if keywords from the code table followed
!     by a number are available
!     It is assumed that either the record has alrady been read or that ityprd
!     is set equal to 1
!     The subroutine starts with reading the next item but reads only one record
!     If a keyword from the code table is found, the value following it is
!     read and put in the right position of table
!     It is checked if the value read is within a given range
!     If an item is not recognized an error message is given
!     If a main keyword is found or end of record the subroutine returns
!     This is the old subroutine intx02, extended with a sequence array
!     defining in which sequence options are found
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconst'
      include 'SPcommon/cmcdpi'
      include 'SPcommon/csepch'
      include 'SPcommon/cseprd'
      include 'SPcommon/cseprdnew'
      include 'SPcommon/creadinfo'
      include 'SPcommon/cinout'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nncode, ncode(nncode), itype(*), nmain, nmcode(*),
     +        iretur, nout, nstrng, ncodmx(nncode), iseqkeyw(*),
     +        nseqkeyw, ispeclsav, errorcode
      character (len=*) code(nncode), codemn(*), strout(nstrng)
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(nout)

!     code           i    Table containing the texts to be compared
!                         n texts have been stored in code
!     codemn         i    Table with main keywords
!     errorcode     i/o   If 0 at input an error message is given if a keyword
!                         is not recognized
!                         If > 0, the subroutine does not give an error message,
!                         but instead returns with -1
!     iretur        i/o   Return code. Possible values
!                         -10000  At input means that intx02ext1 returns if
!                              the first item read is not a keyword
!                              The return code will be -10001 in that case
!                         -11  At input means that asknxt has already been
!                              called before
!                         -10  At input means that no error message is given if
!                              a single real or integer is found
!                              The corresponding value is stored in tabout, next
!                              position
!                              iretur gets the value -10-number of numbers found
!                         -2   End of file has been reached
!                         -1   An error has been detected
!                          0   Normal end of subroutine
!                         >0   Main keyword number iretur has been found
!     iseqkeyw       o    Array of size nseqkeyw that contains the sequence in
!                         which keywords are found in a record
!                         isekeyw(i)>0, means i-th keyword found is the
!                                       iseqkeyw(i)-th keyword in the table
!                         isekeyw(i)<0, means keyword refers to number found
!                                       -isekeyw(i) refers to position in
!                                       tabout
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!                         Possible values (short description):
!                         -1, keyword, skip
!                          0, keyword = integer
!                          1, keyword = real
!                          2, keyword, Set table to 1
!                          3, keyword or keyword = integer
!                          4, keyword or keyword = integer,
!                             indicate both situations
!                          5, keyword = keyword
!                          6, keyword = 'text'
!                          8, keyword = real, repeatedly
!                          9, ci or -ci, i.e. keyword provided by sign
!                             followed by integer number repeatedly
!                         10, keyword iseq1 = j1, keyword iseq2 = j2, ...
!                         11, keyword = i1, i2, i3, ...  all positive integer
!                         12, keyword iseq1 = j1, keyword iseq2 = j2, ...
!                             or keyword = j1 (iseq1 is set to 1)
!                         13, keyword = keyword1 [keyword2], the last may
!                             be empty
!                         14, c1, -c3, c4, -c9, c5, i.e. list of keywords
!                             followed by sequence number (or c1 to c2)
!                         15, c1 v1 c2 v2 List of keyword followed by sequence
!                             number followed by another keyword followed by
!                             sequence number and this repeatedly
!                         16, keyword = real
!                             or keyword (value = real)
!                             or keyword (func = i)
!                             or keyword (time_func = i)
!                         17, keyword = ((x1,y1,..), (x2,y2,..)  ... (xn,yn,..))
!                         18, keyword = P1, P2, ... (or P1 to P2)
!                         19, keyword = ( x1, y1, z1 )
!                         20, keyword or keyword keyword = i1, i2, ...
!                         21, keyword (ax+by+cz=d)
!                         22, keyword [keyword1] [keyword2] = ( x1, y1, z1 )
!                         23, keyword keyword  set tabel to 1
!                         24, keyword [keyword1] [keyword2] = ( i1, i2, i3 )
!                         25, keyword iseq1 = real
!                             or keyword iseq1 (value = real)
!                             or keyword iseq1 (func = i)
!                             or keyword iseq1 (time_func = i)
!                         26, keyword = complex
!                             or keyword (value = complex)
!                             or keyword (func = i)
!                             or keyword (time_func = i)
!                             with complex = real or (real,real)
!                         27, keyword = series of:
!                             real
!                             or keyword (value = real)
!                             or keyword (func = i)
!                             or keyword (time_func = i)
!                         28, keyword = i1, i2, i3, ...  all integer
!                         29, keyword iseq = [keyword1] i1, i2, i3, ...  all
!                             integer
!                         30, keyword iseq1 = c1, keyword iseq2 = c2, ...
!                         31, keyword [iseq] = c1, c2, ... or c1 to c2
!                             for example curves 1 = c1 to c2 or curves = c1
!                         32, keyword [(i1, i2, ..)] = sj to sn or
!                             keyword = sj to sn[, nodes =(i1, i2, ..)]
!                         33, keyword iseq = ( i1, i2, i3, ... ) all integer
!                         34, keyword iseq1 = j1, keyword iseq2 = j2, ...
!                             Different input/output than 10
!                         35, keyword iseq1 = real keyword iseq2 = real
!                             or keyword iseq1 (value = real)
!                             or keyword iseq1 (func = i)
!                             or keyword iseq1 (time_func = i)
!                             or keyword iseq2 (value = real)
!                             or keyword iseq2 (func = i)
!                             or keyword iseq2 (time_func = i)
!                         36, keyword = integer or %name
!                         37, keyword iseq [to iseq1] = [( i1, i2, i3, ... )]
!                             all integer
!                         38, keyword = integer or vector name
!                         39, keyword = integer or scalar name
!                         40, keyword = integer or vector name repeatedly
!                         41, keyword iseq [to iseq1] = [keyword1] i1, i2, i3,
!                             ...  all integer
!                         42, keyword iseq [to iseq1]
!                         Possible values (long description):
!                         itype(i) = -1, no number, the information must be
!                                       skipped
!                                       input:  code(i) keyword
!                                       output: -
!                         itype(i) = 0, number i is of type integer
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value
!                                               tabmax(i) maximum value
!                                       output: table(i)  value
!                         itype(i) = 1, number i is of type real
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value
!                                               tabmax(i) maximum value
!                                       output: table(i)  value
!                         itype(i) = 2, no number but in table the value 1 must
!                                       be set
!                                       input:  code(i) keyword
!                                       output: table(i): 1
!                         itype(i) = 3, either an integer number or no number
!                                       at all
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value
!                                               tabmax(i) maximum value
!                                       output: table(i)  value or nothing
!                         itype(i) = 4, either an integer number or no number
!                                       at all
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value
!                                               tabmax(i) maximum value
!                                       output: table(i) = -1000 keyword without
!                                                          number
!                                               table(i) = iposut keyword with
!                                                          number
!                                               tabout(iposut) = value
!                         itype(i) = 5, number is given in the form of a keyword
!                                       input:  code(i) keyword
!                                               tabmin(i) starting position
!                                                         allowed
!                                                         keywords in codemn
!                                               tabmax(i) end position allowed
!                                                         keywords in codemn
!                                               codemn(tabmin(i) to tabmax(i))
!                                       output: table(i) = relative position of
!                                                          keyword found in
!                                                          codemn i.e. from 1 to
!                                                          tabmax(i)-tabmin(i)+1
!                         itype(i) = 6, text between quotes
!                                       input:  code(i) keyword
!                                       output: table(i) kposut
!                                               tabmax(i) number of characters
!                                               in text
!                                               strout(kposut) = string
!                         itype(i) = 8, special situation of keyword followed
!                                       by number, repeatedly
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value of
!                                                         reals
!                                               tabmax(i) maximum value of
!                                                         reals
!                                       output: table(i) = iposut
!                                               tabmax(i) = m
!                                                   (number of reals read)
!                                               tabout(iposut to iposut+m-1) =
!                                                   values
!                         itype(i) = 9, special situation of keyword followed
!                                       by integer number, repeatedly
!                                       the keyword may be provided by a sign,
!                                       like -c8
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value of
!                                                         integers
!                                               tabmax(i) maximum value of
!                                                         integers
!                                       output: table(i) = iposut
!                                               tabmax(i) = m
!                                                   (number of integers read)
!                                               tabout(iposut to iposut+m-1) =
!                                                   values
!                         itype(i) =10, special situation of keyword followed
!                                       by sequence number followed by number,
!                                       for example inpvc i = j
!                                       In this case the number positions
!                                       in the arrays code itype etc. is equal
!                                       to the range of the sequence numbers
!                                       input:  code(i to i+max-1) keyword
!                                               max is tabmax(i)-tabmin(i)+1
!                                               tabmin(i) minimum value i
!                                               tabmax(i) maximum value i
!                                               tabmin(i+1) minimum value j
!                                               tabmax(i+1) maximum value j
!                                               itype(i+1) = 0: j is integer
!                                               itype(i+1) = 1: j is real
!                                       output: table(i to i+max-1) contain the
!                                               values of j for respectively
!                                               k=tabmin(i) to k=tabmax(i)
!                         itype(i) =11, special situation of keyword followed by
!                                       series of positive integer numbers
!                                       Brackets are treated as separator
!                                       input:  code(i) keyword
!                                               tabmin(i) maximum number of
!                                                         integers
!                                               tabmax(i) maximum value of
!                                                         integers
!                                       output: table(i) = iposut
!                                               tabmax(i) = m
!                                                    (number of integers read)
!                                               tabout(iposut to iposut+m-1) =
!                                                     values
!                         itype(i) =12, special situation of keyword followed
!                                       by sequence number followed by number,
!                                       for example inpvc i = j or alternatively
!                                       keyword followed by number
!                                       for example inpvc = j. i = 1 in that
!                                       case
!                                       Rest see itype(i) = 10
!                         itype(i) =13, special situation of keyword followed
!                                       by one or two keywords, for example:
!                                       renumber = cuthill profile
!                                       input:  code(i) keyword
!                                               code(i+1) same keyword
!                                               itype(i+1) ipostyp
!                                               itype(ipostyp+2*j-2) ipmn1_j
!                                               itype(ipostyp+2*j-1) ipmn2_j
!                                               tabmin(i) ipmn1_1
!                                               tabmax(i) ipmn2_1
!                                               codemn(ipmn1_1 to ipmn2_1)
!                                        contains the list of first keywords
!                                               allowed codemn(ipmn1_j to
!                                               ipmn2_j) contains the list
!                                               of second keywords allowed
!                                               corresponding to first keyword
!                                               "j", where j is the relative
!                                               seq. number
!                                               If to first keyword j no second
!                                               keyword corresponds ipmn1_j and
!                                               ipmn2_j must be 0
!                                       output: table(i) = j1
!                                               table(i+1) = j2
!                                               j1, j2 denote the relative ref.
!                                               number corresponding to first
!                                               resp. second keyword
!                         itype(i) =14, special situation of keyword followed
!                                       by number, followed by same keyword
!                                       followed by number and so on, for
!                                       example: p1, p3, p5, p8, p2
!                                       The keywords may be preceded by a minus
!                                       sign, hence c1, c3, -c5, c8 etc.,
!                                       in which the sequence number
!                                       gets the minus sign
!                                       input:  code(i) keyword
!                                               tabmin(i) maximum number of
!                                                         integers
!                                               tabmax(i) maximum value of
!                                                         integers
!                                       output: table(i) = iposut
!                                               tabmax(i) = m
!                                                   (number of integers read)
!                                               tabout(iposut to iposut+m-1) =
!                                                   values
!                         itype(i) =15, special situation of keyword followed
!                                       by number, followed by same keyword
!                                       followed by number etc.
!                                       However, in this case it is possible
!                                       that the keyword number combination
!                                       is followed by other keywords, before it
!                                       is followed by the next keyword number
!                                       combination
!                                       Example: c1 v1 + c2 v2
!                                       In this case both c and v get type
!                                       number 15
!                                       In this case in tabout exactly tabmin(i)
!                                       positions are reserved. The rest of the
!                                       input is exactly the same as for
!                                       itype(i) = 14
!                         itype(i) =16, special situation of keyword followed
!                                       by either a value or the possibilities
!                                       value = v, func = ifunc, or
!                                       time_func = ifunc
!                                       input:  code(i) keyword
!                                       output: table(i) = ifunc (>0) or
!                                               -iposut
!                                               tabout(iposut) = value
!                                               if time_func = ifunc is given
!                                               table(i) gets the value
!                                               -ifunc-100
!                         itype(i) =17, special situation of keyword followed by
!                                       series of series of numbers between
!                                       brackets. Each series of local numbers
!                                       itself must be between brackets.
!                                       The number o fnumbers in the inner
!                                       series must be constant.
!                                       input:  code(i) keyword
!                                               tabmin(i) number of numbers in
!                                                         inner series
!                                               tabmax(i) maximum number of
!                                                         series
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m-1)
!                                               values
!                                               tabmax(i) m
!                         itype(i) =18, special situation of keyword
!                                       followed by a series of the same
!                                       keywords followed by sequence numbers
!                                       hence we have the situation keyword
!                                       followed by possibility 14
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m-1)
!                                               values
!                                               tabmax(i) m
!                         itype(i) =19, special situation of keyword followed by
!                                       a series of numbers possibly enclosed by
!                                       brackets
!                                       input:  code(i) keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m-1)
!                                               values
!                                               tabmax(i) m
!                         itype(i) =20, special situation of keyword which has
!                                       to be set or the same keyword followed
!                                       by another special keyword
!                                       followed by integers
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i)  iposut
!                                               or -100000 if subkeyword is not
!                                               present
!                                               tabout(iposut to iposut+m-1)
!                                               values
!                                               tabmax(i) m
!                         itype(i) =21, special situation of keyword followed by
!                                       (ax+by+cz=d), with a, b, c and d numbers
!                                       if a, b or c is omitted 1 or -1
!                                       is supposed
!                                       At output table(i) contains the starting
!                                       position of the 4 numbers a,b,c and d in
!                                       that sequence in tabout
!                         itype(i) =22, special situation of keyword followed by
!                                       possibly some other special keywords
!                                       followed by a series of numbers
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m)
!                                               values read
!                                               tabmax(i) m
!                         itype(i) =23, special situation of keyword followed
!                                       by a special keyword that must
!                                       be skipped
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special
!                                               sub keyword
!                                       output: table(i): 1
!                         itype(i) =24, special situation of keyword followed by
!                                       possibly some other special keywords
!                                       followed by a series of positive
!                                       integers
!                                       See 22
!                         itype(i) =25, special situation of keyword followed by
!                                       sequence number followed by either a
!                                       value or the possibilities
!                                       value = v, func = ifunc,
!                                       or time_func = ifunc (See 10 and 16)
!                                       input:  code(i to i+max-1) keyword
!                                               max is tabmax(i)-tabmin(i)+1
!                                               tabmin(i) minimum value i
!                                               tabmax(i) maximum value i
!                                       output: table(k) = ifunc (>0)
!                                               or -iposut
!                                               tabout(iposut) = value
!                                               if time_func = ifunc is given
!                                               table(k) gets the value
!                                               -ifunc-100 for respectively
!                                               k=tabmin(i) to k=tabmax(i)
!                         itype(i) =26, special situation of keyword followed
!                                       by either a value or the possibilities
!                                       value = v, func = ifunc, or
!                                       time_func = ifunc with v complex
!                                       input:  code(i) keyword
!                                       output: table(i) = ifunc (>0) or -iposut
!                                               tabout(iposut,iposut+1) = value
!                                               if time_func = ifunc is given
!                                               table(i) gets the value
!                                               -ifunc-100
!                         itype(i) =27, special situation of keyword followed
!                                       by a series of: value or value = v,
!                                       func = ifunc, or time_func = ifunc
!                                       input:  code(i) keyword
!                                               tabmin(i) number of values
!                                               expected
!                                       output: table(i) = iposut
!                                               tabout(iposut+2*i-2) = ifunc_i
!                                               if time_func = ifunc is given
!                                               -ifunc_i-100
!                                               if value is given then 0
!                                               tabout(iposut+2*i-1) = value
!                         itype(i) =28, special situation of keyword followed by
!                                       series of integer numbers. See 11
!                         itype(i) =29, special situation of keyword followed by
!                                       sequence number possibly followed by
!                                       another keyword followed by integers
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                                         |numbers|
!                                       output: table(i) = iposut
!                                               tabmax(i) = m (number of
!                                               integers read)
!                                               tabout(iposut to iposut+m-1) =
!                                               values
!                         itype(i) =30, special situation of keyword followed
!                                       by sequence number followed by
!                                       keyword and integer number,
!                                       for example obs i = cj or alternatively
!                                       keyword followed by keyword and integer
!                                       number
!                                       for example obs = cj. i = 1 in that case
!                                       input:  itype(i+1) = iposmain
!                                               codemn(iposmain) special
!                                               sub keyword
!                                       Rest see itype(i) = 10, 12
!                         itype(i) =31, special situation of keyword followed by
!                                       possibly an integer followed by
!                                       followed by a series of the same
!                                       keywords followed by sequence numbers
!                                       hence we have the situation keyword
!                                       possibly followed by an integer followed
!                                       by possibility 18
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special
!                                               sub keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m-1)
!                                               values
!                                               tabmax(i) m
!                                               tabmin(i) integer after keyword
!                                                         if not found: 0
!                         itype(i) =32, special situation of keyword followed by
!                                       possibly a series of integers
!                                       followed by one or two of the same
!                                       keywords followed by sequence numbers
!                                       followed by possibly the text nodes
!                                       followed by integers
!                                       If nodes is found the first series may
!                                       not be used
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read in the
!                                               nodes part
!                                               tabmax(i) maximum value of the
!                                               numbers after the sub keyword
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m+1)
!                                               values
!                                               the first 2 contain the number
!                                               after the subkeyword
!                                               the last m the values
!                                               after nodes
!                                               tabmax(i) m
!                         itype(i) =33, special situation of keyword followed by
!                                       sequence number followed by integers
!                                       input:  code(i) keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i) = iposut
!                                               tabmax(i) = m
!                                               (number of integers read)
!                                               tabout(iposut to iposut+m-1) =
!                                               values
!                         itype(i) =34, special situation of keyword followed
!                                       by sequence number followed by number,
!                                       for example inpvc i = j
!                                       In this case 2 positions
!                                       in the arrays code itype etc. are used
!                                       i must be between 1 and max
!                                       input:  code(i to i+1) keyword
!                                               tabmax(i) maximum value i
!                                               tabmin(i+1) minimum value j
!                                               tabmax(i+1) maximum value j
!                                               itype(i+1) = 0: j is integer
!                                               itype(i+1) = 1: j is real
!                                               table(i)<=0
!                                       output: table(i) iposut
!                                               tabout(iposut to iposut+max-1)
!                                               values of j
!                                               If sequence is not read
!                                               tabout(iposut+i-1) = -1d30
!                         itype(i) =35, special situation of keyword followed by
!                                       sequence number followed by either a
!                                       value or the possibilities
!                                       value = v, func = ifunc,
!                                       or time_func = ifunc (See 10 and 16)
!                                       then
!                                       followed by the same keyword followed by
!                                       sequence number followed by either a
!                                       value or the possibilities
!                                       value = v, func = ifunc,
!                                       or time_func = ifunc (See 10 and 16)
!                                       then
!                                       input:  code(i ) keyword
!                                               tabmin(i) minimum value i
!                                               tabmax(i) maximum value i
!                                       output: table(i) = iposut
!                                               tabout(iposut to iposut+3) =
!                                               values
!                         itype(i) =36, number i is of type integer or %name
!                                       input:  code(i) keyword
!                                               tabmin(i) minimum value
!                                               tabmax(i) maximum value
!                                       output: table(i)  value
!                         itype(i) =37, special situation of keyword followed by
!                                       sequence number to other sequence number
!                                       followed by integers between brackets
!                                       The second sequence number may absent as
!                                       well as the numbers between brackets
!                                       input:  code(i) keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                               |numbers|
!                                       output: table(i) = iposut
!                                               tabmax(i) = m (number of
!                                               integers read)
!                                               tabout(iposut to iposut+m-1) =
!                                               values
!                                               The first two always refer to
!                                        the sequence numbers
!                         itype(i) = 38, number i is vector name or number
!                                       input:  code(i) keyword
!                                       output: table(i)  value
!                         itype(i) = 39, number i is scalar name or number
!                                       input:  code(i) keyword
!                                       output: table(i)  value
!                         itype(i) = 40, number i is vector name or number
!                                        may appear more than once
!                                       input:  code(i) keyword
!                                       output: table(i)  iposut
!                                               tabout(iposut to iposut+m-1) =
!                                               values
!                                               tabmax(i) number of vectors
!                         itype(i) =41, special situation of keyword followed by
!                                       sequence number possibly followed by
!                                       another keyword followed by integers
!                                       input:  code(i) keyword
!                                               table(i) -iposmain
!                                               codemn(iposmain) special sub
!                                               keyword
!                                               tabmin(i) maximum number of
!                                               numbers that may be read
!                                               tabmax(i) maximum value of
!                                                         |numbers|
!                                       output: table(i) = iposut
!                                               tabmax(i) = m (number of
!                                               integers read)
!                                               tabout(iposut to iposut+m-1) =
!                                               values
!                                               The first two always refer to
!                                               the sequence numbers
!                         itype(i) =42, special situation of keyword followed by
!                                       sequence number or iseq1 to iseq2
!                                       input:  code(i) keyword
!                                               tabmax(i) maximum value of
!                                                         |numbers|
!                                       output: table(i) = iposut
!                                               tabout(iposut to iposut+1) =
!                                               values
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     ncodmx         i    Table containing the maximum number of characters in
!                         the keyword table code. If the keyword contains more
!                         characters a warning is given.
!     nmain          i    Number of main keywords stored in codemn
!                         codemn itself may be larger because of the possibility
!                         keyword = text
!                         If nmain<0 and a main keyword has been found,
!                         icolum is reset to the preceding value, which implies
!                         that the main keyword may be read again
!     nmcode         i    Table containing the number of significant characters
!                         in the main keyword table codemn, including those of
!                         the extra texts
!     nncode         i    Number of texts stored in code
!     nout           i    Length of array tabout
!     nseqkeyw       o    Number of entries in iseqkeyw
!     nstrng         i    Length of array strout
!     strout         o    Contains series of strings, see itype
!     table         i/o   Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax         i    Contains the maximum values allowed in the table
!                         In case of the possibility keyword = text it contains
!                         the starting positions of the texts in codemn
!     tabmin         i    Contains the minimum values allowed in the table
!                         In case of the possibility keyword = text it contains
!                         the end positions of the texts in codemn
!     tabout         o    Contains series of numbers, see itype
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished, readnx, debug, keywordfound, vectorfound
      integer maxcode, maxmultiple, maxrepeat
      parameter ( maxcode = 200, maxmultiple = 5, maxrepeat = 1000 )
      integer icode, i, iposut, minusg, ispecs, ihelp(maxcode),
     +        kposut, jump, icountkey(maxcode), nmax, m, iseq,
     +        pointer(nncode), numberkeyw(maxmultiple), icount, k
      double precision valuekeyw(maxrepeat, maxmultiple)
      character (len=240) subkey

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     i              Counting variable
!     icode          Position number of keyword found in table code
!     icount         Counter, number of keywords with itype = 8 or 9
!     icountkey      Array that counts the number of times a keyword has been
!                    found
!     ihelp          Help array to store number of appearances of keywords that
!                    may be used more than once or number of numbers read
!     iposut         last position used in array tabout
!     iseq           Sequence number
!     ispeclsav      Help parameter to save the value of ispecl
!     ispecs         Help variable to save the value of ispecl
!     jump           if 0 no error has been found, if 1 array tabout
!                    is too small
!     k              Counting variable
!     keywordfound   If true a keyword has been found
!     kposut         last position used in array strout
!     m              Number of integers read
!     maxcode        Maximum length of array code allowed
!     maxmultiple    Declared size of array pointer
!     maxrepeat      Declared size of first entry of array numberkeyw
!     minusg         Indicates if a minus sign has been found (-1) or not (1)
!     nmax           maximum number of numbers allowed
!     numberkeyw     Array of size maxmultiple containing the number of values
!                    stored in array valuekeyw for each type
!                    The sequence is defined by array pointer
!     pointer        array of size n containing the sequence numbers of
!                    all keywords with respect to arrays numberkeyw and
!                    valuekeyw
!                    if 0, no reference is necessary
!     readnx         Indicates if the next item has not been read (true) or has
!                    been (false)
!     subkey         Sub (second) keyword
!     valuekeyw      Array of size maxrepeat x maxmultiple containing
!                    all values corresponding to the special keywords of
!                    type 8 and 9
!                    valuekeyw(i,j) corresponds to ith value for j-th keyword
!                    where the sequence of pointer is used
!     vectorfound    If true the last item read is a vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INTX0201       Read keyword or keyword = number
!     INTX0205       Read keyword followed by one or two keywords
!     INTX0206       Read keyword = text
!     INTX0208       Read keyword = number repeatedly
!     INTX0209       Read keyword = number repeatedly
!     INTX0210       Read keyword followed by sequence number and number
!                    or number
!     INTX0211       Read keyword followed by a series of positive integers
!     INTX0215       Read keyword followed by an integer, repeatedly
!     INTX0216       Read keyword followed by a value or value = v, or func = i
!     INTX0217       Read keyword followed by a series of series of numbers
!     INTX0218       Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 14, 18 and 31
!     INTX0219       Read keyword followed by a series of numbers
!     INTX0220       Read single keyword inwhich case the table must be set
!                    to -100000 or keyword followed by agiven other keyword
!                    and a series of integers
!     INTX0221       Read keyword followed by (ax+by+cz=d)
!     INTX0222       Read keyword followed by possibly one or two special
!                    keywords that are skipped and then by a series of numbers
!     INTX0223       Read keyword followed by possibly a special keyword that
!                    is skipped
!                    table(icode) is set equal to 1
!     INTX0225       Read keyword followed by a sequence number followed by
!                    a value(1) or value(1) = v, or func = i
!     INTX0227       Read keyword followed bya series of values or value = v,
!                    or func = i
!     INTX0229       Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 29, 33 and 37
!     INTX0232       Read keyword like:
!                    surfaces (i1, i2, ... in ) = s1 to s2 or
!                    surfaces = s1 to s2, nodes (i1, i2, ... in )
!     INTX0235       Read keyword followed by a sequence number followed by
!                              a value or value = v, or func = i
!                              and again the same keyword followed  by
!                              a sequence number followed by
!                              a value or value = v, or func = i
!     INTX0238       Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 38
!     INTX0239       Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 39
!     INTX02EXPR     Check if string read corresponds to expression
!     INTX02KEYW     Check if string read is (main) keyword
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READNEXTITEM   Read next item and interpret
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     648   string could not be recognized
!    1439   Number of items more than 100
!    1440   Keyword occurs too many times
!    1468   Array tabout too small
!    1469   Array strout too small
!    1628   Keyword found more than one for the same problem
!    1868   Text not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
!    finished := false
!    Clear array ihelp
!    While ( not finished ) do
!       read next item
!       if ( end of file found ) then
!          iretur := -2
!          finished := true
!       else if ( end of record found ) then
!          iretur := 0
!          finished := true
!       else if ( item # keyword ) then
!          Error message
!          iretur := -1
!          finished := true
!       else if ( item = main keyword ) then
!          set iretur
!          finished := true
!       else if ( item = keyword from code ) then
!          icode := sequence number of keyword in code table
!          if ( itype(icode) # 2 and # -1 ) then
!             read next item
!             if ( itype(icode) = 0, 1, 3 or 4 and next item is a number ) then
!                Check type of number
!                Fill number in table
!             else if ( itype(icode) = 5 or 13 and next item is a text ) then
!                Check type of text
!                Fill number in table
!                if ( itype(icode) = 13 ) then
!                   If ( second keyword should be present ) then
!                      Read next keyword
!                      Check type of text
!                      Fill number in table
!             else if ( itype(icode) = 6 ) then
!                Read and store text
!             else if ( itype(icode) = 3 or 4 and no number found ) then
!                skip
!             else if ( itype(icode) = 10, 12, 30 and 34
!                       and next item is integer number ) then
!                Check sequence number
!                Read next item
!                Check if this is a number in the correct range
!                Fill number in table
!             else if ( itype(icode) = 11 or 28 or 29 ) then
!                Read and check series of integers between brackets
!                Store integers in tabout
!                Set table(i) and tabmax(i)
!             else if ( itype(icode) = 14, 18 or 31 ) then
!                if ( itype(icode) = 18 or 31 ) then
!                   Set keyword to subkeyword
!                While ( next item = keyword with number icode ) do
!                   Read next item (must be integer) and store in tabout
!                   Read next item
!                Set table(i) and tabmax(i)
!             else if ( itype(icode) = 15 ) then
!                if ( ihelp(icode)=0 ) then
!                   Fill table(icode)
!                Read next item (must be integer) and store in tabout
!                Raise ihelp
!             else if ( itype(icode) = 16 ) then
!                Read next item
!                if ( item = 'value' or 'func' or 'time_func' ) then
!                   read next item
!                Fill table and if necessary tabout
!             else if ( itype(icode) = 17 ) then
!                Read series of numbers until new item is found
!                Fill table and tabout
!             else if ( itype(icode) = 19 ) then
!                Read series of numbers until new item is found
!                Fill table and tabout
!             else if ( itype(icode) = 20 ) then
!                Check if keyword is followed by special keyword
!                If ( not special keyword found ) then
!                   Set table(icode)
!                else
!                   Read and store integers
!             else if ( itype(icode) = 21 ) then
!                Read ax+by+sz=d
!                Store result in posout
!             else if ( itype(icode) = 22 or 24 ) then
!                Read next item
!                if ( next item = special keyword ) then
!                   read next item
!                if ( next item = special keyword ) then
!                   read next item
!                Read numbers and store in posout
!             else if ( itype(icode) = 23 ) then
!                Read next item
!                If ( next item is special keyword ) then
!                   skip
!                Set table(icode)
!             else if ( itype(icode) = 25 and next item is integer number ) then
!                Check sequence number
!                Read next item
!                if ( item = 'value', 'func' or 'time_func' ) then
!                   read next item
!                Fill table and if necessary tabout
!             else if ( itype(icode) = 26 ) then
!                Repeat until different item is found
!                   Read next item
!                   if ( item = 'value' or 'func' or 'time_func' ) then
!                      read next item
!                   Fill tabout and tabmax
!             else if ( itype(icode) = 32 ) then
!                Read information after keyword
!                Set table(icode) and fill tabmax and tabout
!             else
!                error message
!                iretur := -1
!                finished := true
!          else if ( itype(icode) = 2 ) then
!             Put a 1 in the table
!       else ( keyword not recognized )
!          iretur := -1
!          finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx02ext1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx02ext1'
         write(irefwr,1) 'iretur, nncode, ityprd, lastitem',
     +                    iretur, nncode, ityprd, lastitem
         write(irefwr,1) 'istop, nout', istop, nout
         call prinin ( itype, nncode, 'itype' )
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      finished = .false.
      readnx = .false.
      keywordfound = .false.
      ispeclsav = ispecl  ! save ispecl
      if ( iretur/=-11 ) then
         if ( typeitems(lastitem+1)==13 .or.
     +        typeitems(lastitem+1)==16 ) lastitem = lastitem+1  ! = or : found
         if ( typeitems(lastitem+1)==5 ) lastitem = lastitem+1   ! ( found
         if ( debug ) write(irefwr,1) 'lastitem/1', lastitem
         if ( typeitems(lastitem+1)==3 ) ispecl = 9 ! text found, include number
         if ( debug )
     +      write(irefwr,1) 'typeitems(lastitem+1)',
     +                       typeitems(lastitem+1)
         call asknxt
      end if  ! ( iretur/=-11 )
      ispecl = ispeclsav  ! reset ispecl
      if ( debug ) write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
      iposut = 0
      kposut = 0
      nseqkeyw = 0
      ispecs = ispecl
      if ( nncode>maxcode ) then
         call errint ( nncode, 1 )
         call errint ( maxcode, 2 )
         call errsub ( 1439, 2, 0, 0 )
         go to 1000
      end if

      ihelp(1:nncode) = 0
      icountkey(1:nncode) = 0

      if ( nncode>lentable ) then

!     --- lentable too small

         call errint ( nncode, 1 )
         call errint ( lentable, 2 )
         call errchr ( 'infotable', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if
      if ( nout>lentabout ) then

!     --- lentabout too small

         call errint ( nout, 1 )
         call errint ( lentabout, 2 )
         call errchr ( 'infotabout', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if

      infotable(1:nncode) = 0
      infotabout(1:nout) = 0

      if ( ierror/=0 ) go to 1000

!     --- Initialize array pointer

      icount = 0
      do i = 1, nncode

         if ( itype(i)>=8 .and. itype(i)<=9 ) then

!        --- itype(i) = 8 or 9, hence this keyword may be present
!            more than once, raise icount and store in pointer

            icount = icount + 1
            pointer(i) = icount

         else

!        --- itype(i) # 6 or 8, hence this keyword is used only once
!            set pointer to 0

            pointer(i) = 0

         end if  ! ( itype(i)>=8 .and. itype(i)<=9 )

      end do  ! i = 1, n
      if ( debug ) then
         write(irefwr,1) 'icount', icount
         call prinin ( pointer, nncode, 'pointer' )
         write(irefwr,*) 'finished ', finished
      end if  ! ( debug )

!     --- Check array length

      call inichkarlen ( icount, maxmultiple, 'numberkeyw' )
      if ( ierror/=0 ) go to 1000

!     --- Initialize array numberkeyw

      numberkeyw(1:icount) = 0

      do while ( .not. finished )

!     --- read next item

         if ( ierror/=0 ) go to 1000
         minusg = 1
         ispecl = 4
         vectorfound = .false.
         if ( debug ) then
            write(irefwr,*) 'readnx ', readnx
            write(irefwr,1) 'istop, lastitem/1a', istop, lastitem
            write(irefwr,1) 'iposut', iposut
         end if  ! ( debug )

         if ( readnx .or. ityprd==20 ) then

!        --- Read next item, might be an expression
!            However keywords have higher priority than reals

            call readnextitem ( code, ncode, nncode, codemn, nmcode,
     +                          nmain, iretur, ncodmx, finished, icode )

         end if
         if ( debug ) then
            write(irefwr,1) 'ityprd, istop, lastitem/2',
     +                       ityprd, istop, lastitem
            if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )
         if ( ierror/=0 ) go to 1000

         if ( ityprd==11 .and. (itype(1)==16 .or. itype(1)==26 ) )
     +      then

!        --- Special case of a scalar found in combination with a
!            coefficient

            table(1) = nint(realrd)
            nseqkeyw = nseqkeyw+1
            iseqkeyw(nseqkeyw) = 0
            readnx = .true.
            go to 1000

         end if
         readnx = .true.
         if ( ityprd==6 ) then

!        --- minus sign found

            minusg = -1
            call asknxt
            if ( debug ) then
               write(irefwr,1) 'ityprd, istop, lastitem/2a',
     +                          ityprd, istop, lastitem
               if ( ityprd==4 )
     +            write(irefwr,*) 'string ', string(1:nchar)
            end if  ! ( debug )

         end if
110      if ( ityprd==7 ) then
            call asknxt
            if ( ierror/=0 ) go to 1000
            if ( istop/=2 .and. lastitem==nitems .and. ityprd==7 )
     +         ityprd = 1
            go to 110
         end if
         if ( debug ) write(irefwr,1) 'iretur, ityprd', iretur, ityprd
         if ( ierror/=0 ) go to 1000

         if ( ityprd==0 ) then

!        --- end of file found

            if ( iretur>-10 ) iretur = -2
            finished = .true.

         else if ( ityprd==1 ) then

!        --- end of record found

            if ( iretur>-10 ) iretur = 0
            finished = .true.

         else if ( iretur<=-10 .and. iretur>-10000 .and.
     +           (ityprd==2 .or. ityprd==3 .or. ityprd==11) ) then

!        --- iretur = -10 and number found

            iposut = iposut+1
            tabout(iposut) = realrd
            iretur = iretur-1
            nseqkeyw = nseqkeyw+1
            iseqkeyw(nseqkeyw) = -iposut
            if ( debug )
     +         write(irefwr,1) 'iretur, iposut', iretur, iposut
            if ( iposut>nout ) go to 950

         else if ( ityprd/=4 ) then

!        --- item # keyword

            if ( .not. keywordfound .and. iretur==-10000 ) then

!           --- iretur = -10000, so we must return with iretur = -10001
!               minus sign must be read again

               if ( minusg==-1 ) lastitem = lastitem-1
               lastitem = lastitem-1  ! last item must be reread
               iretur = -10001
               go to 1000

            end if  ! ( iretur==-10001 )
            do i = 1, abs(nmain)
               call errchr ( codemn(i), i )
            end do
            do i = 1, nncode
               call errchr ( code(i)(1:ncodmx(i)), i+abs(nmain) )
            end do
            call errsub ( 1868, 0, 0, abs(nmain)+nncode )
            call errcrd
            iretur = -1
            finished = .true.

         else

!        --- Keyword found, check if it is a main keyword

            call intx02keyw ( code, ncode, nncode, codemn, nmcode,
     +                        nmain, iretur, ncodmx, finished,
     +                        icode )
            if ( debug ) then
               write(irefwr,*) 'after intx02keyw'
               write(irefwr,1) 'icode, lastitem, iretur',
     +                          icode, lastitem, iretur
               write(irefwr,1) 'ityprd, iposut', ityprd, iposut
               write(irefwr,2) 'realrd', realrd
               write(irefwr,*) 'finished', finished
            end if  ! ( debug )
            if ( ierror/=0 ) go to 1000
            if ( finished ) go to 900   ! main keyword found
            if ( ityprd==2 ) vectorfound = .true.

            if ( icode>0 ) go to 200    ! keyword found

!           --- keyword not recognized

            if ( debug ) then
               write(irefwr,*) 'keywordfound', keywordfound
               write(irefwr,1) 'iretur', iretur
            end if  ! ( debug )

            if ( .not. keywordfound .and. iretur==-10000 ) then

!           --- iretur = -10000, so we must return with iretur = -10001
!               minus sign must be read again

               if ( minusg==-1 ) lastitem = lastitem-1
               lastitem = lastitembef  ! last item must be reread
               iretur = -10001
               go to 1000

            end if  ! ( iretur==-10000 )
            if ( iretur<=-10 .and. nchar<=32 ) then

!           --- iretur<=-10, check if it is an expression or constant

               if ( debug ) write(irefwr,1) 'nout, iposut', nout, iposut
               call intx02expr ( nncode, itype, table, iretur, tabout,
     +                           nout, iseqkeyw, nseqkeyw, readnx,
     +                           iposut, finished )
               if ( ierror/=0 ) go to 1000
               if ( debug )
     +            write(irefwr,1) 'ityprd, minusg', ityprd, minusg

               if ( (ityprd==2 .or. ityprd==3) .and. minusg==-1 ) then

!              --- Number found, provided with minus sign

                  minusg = 1
                  tabout(-iseqkeyw(nseqkeyw)) = -realrd

               end if  ! ( ityprd==2 .or. ityprd==3 .and. minusg==-1 )

               if ( debug ) then
                  write(irefwr,*) 'after intx02expr'
                  write(irefwr,1) 'iretur, lastitem', iretur, lastitem
                  write(irefwr,1) 'ityprd, nseqkeyw', ityprd, nseqkeyw
                  write(irefwr,1) '-iseqkeyw(nseqkeyw), nout, iposut',
     +                             -iseqkeyw(nseqkeyw), nout, iposut
                  write(irefwr,2) 'realrd', realrd
                  write(irefwr,2) 'tabout', tabout(-iseqkeyw(nseqkeyw))
                  write(irefwr,*) 'finished', finished
               end if  ! ( debug )
               if ( ierror/=0 ) go to 1000
               if ( finished ) then

!              --- End of expression

                  if ( lastitem>=nitems ) then

!                 --- end of record found

                     ityprd = 1

                  else

!                 --- There is more to read

                     finished = .false.

                  end if  ! ( lastitem>=nitems ) then
                  go to 900   ! expression found

               end if  ! ( finished )

            end if  ! ( iretur<=-10 )

!           --- When we reach this stage, the word is not recognized

            if ( errorcode>0 ) then
               errorcode = -1
               go to 1000  ! jump in case errorcode>0
            end if  ! ( errorcode>0 )

            call errchr ( string(1:nchar), 1 )
            do i = 1, abs(nmain)
               call errchr ( codemn(i), 1+i )
            end do
            do i = 1, nncode
               call errchr ( code(i), 1+i+abs(nmain) )
            end do
            call errsub ( 648, 0, 0, 1+abs(nmain)+nncode )
            call errcrd
            iretur = -1
            finished = .true.
            go to 1000

!           --- keyword found, read next item
!               Store sequence number of keyword into iseqkeyw

200         nseqkeyw = nseqkeyw+1
            iseqkeyw(nseqkeyw) = icode
            keywordfound = .true.
            if ( ierror/=0 ) go to 1000
            if ( debug ) then
              write(irefwr,1) 'nseqkeyw, icode, itype(icode), minusg',
     +                         nseqkeyw, icode, itype(icode), minusg
              write(irefwr,1) 'istop, lastitem/3', istop, lastitem
            end if  ! ( debug )
            if ( ierror/=0 ) go to 1000

            select case ( itype(icode) )

               case(:-1,7)

!              --- itype(icode) = -1,7: skip

               case(0,1,3,4,36)

!              --- itype(icode) = 0, 1, 3, 4, 36: Read keyword or
!                  keyword = number

                  call intx0201 ( icountkey, icode, table, nout,
     +                            tabout, tabmin, tabmax, code,
     +                            finished, readnx, itype, iposut )
                  if ( iposut>nout ) go to 950

               case(2)

!              --- itype(icode) = 2: set table = 1

                  table(icode) = 1d0
                  if ( icountkey(icode)/=0 ) then
                      call errchr ( code(icode), 1 )
                      call errsub ( 1628, 0, 0, 1 )
                  else
                     icountkey(icode) = icountkey(icode)+1
                  end if

               case(5,13)

!              --- itype(icode) = 5 or 13, hence the keyword must be followed by
!                  one or two keywords

                  call intx0205 ( icountkey, icode, table, iretur,
     +                            tabmin, tabmax, code, finished,
     +                            readnx, itype, codemn, nmcode )

               case(6)

!              --- itype(icode) = 6, text

                  call intx0206 ( icountkey, icode, table,
     +                            tabmax, code, finished,
     +                            kposut, nstrng, strout, iretur )
                  if ( kposut>nstrng ) go to 960

               case(8)

!              --- itype(icode) = 8, keyword = real, repeatedly

                  iseq = pointer(icode)
                  call inichkarlen ( numberkeyw(iseq)+1, maxrepeat,
     +                               'valuekeyw' )
                  if ( ierror/=0 ) go to 1000
                  call intx0208 ( code, icode, numberkeyw(iseq),
     +                            valuekeyw(1,iseq), tabmin, tabmax,
     +                            readnx, finished )

               case(9)

!              --- itype(icode) = 9, keyword = positive integer, repeatedly
!                  keyword may be provided with minus sign

                  iseq = pointer(icode)
                  call inichkarlen ( numberkeyw(iseq)+1, maxrepeat,
     +                               'valuekeyw' )
                  if ( ierror/=0 ) go to 1000
                  call intx0209 ( code, icode, numberkeyw(iseq),
     +                            valuekeyw(1,iseq),
     +                            tabmax, readnx, finished, minusg )

               case(10, 12, 30, 34)

!              --- itype(icode) = 10, hence the keyword must be followed by
!                  a sequence number and a number
!                  or itype(icode) = 12, 30:
!                  the sequence number may be not present

                  call intx0210 ( icode, table, nout, tabout, iretur,
     +                            tabmin, tabmax, code, finished,
     +                            readnx, itype, iposut, codemn,
     +                            nmcode, subkey, vectorfound )

               case(11, 28)

!              --- itype(icode) = 11 or 28,
!                  hence the keyword must be followed by
!                  a series of (positive) integers

                  call intx0211 ( icountkey, icode, table, nout,
     +                            tabout, tabmin, tabmax, code, readnx,
     +                            itype, iposut, ihelp )
                  if ( iposut>nout ) go to 950

               case(14,18,31)

!              --- itype(icode) = 14, 18, 31 hence the keyword must be followed
!                  by a positive integer and possibly the same keyword with
!                  another integer and so on

                  call intx0218 ( icode, code, ncode, itype, table,
     +                            codemn, nmcode, tabmin, tabmax,
     +                            tabout, nout, jump, iposut,
     +                            ihelp, finished, readnx, minusg )
                  if ( jump/=0 ) go to 950

               case(15)

!              --- itype(icode) = 15, hence the keyword must be followed by
!                  an integer. This situation may be repeated

                  call intx0215 ( icode, table, nout, tabout, minusg,
     +                            tabmin, tabmax, code, finished,
     +                            iposut, ihelp )
                  if ( debug ) then
                     write(irefwr,*) 'after intx0215'
                     write(irefwr,1) 'iposut', iposut
                     write(irefwr,2) 'tabout', tabout(iposut-1)
                  end if  ! ( debug )
                  if ( iposut>nout ) go to 950

               case(16,26)

!              --- itype(icode) = 16, hence the keyword must be followed by
!                                     a value or value = v, or func = i

                  call intx0216 ( icountkey, icode, table, nout,
     +                            tabout, code, itype, iposut )
                  if ( iposut>nout ) go to 950

               case(17)

!              --- itype(icode) = 17, hence the keyword must be followed by
!                                     a series of series of numbers

                  call intx0217 ( icountkey, icode, table, nout,
     +                            tabout, tabmin, tabmax, code,
     +                            finished, readnx, iposut, ihelp,
     +                            nmax )
                  if ( iposut>nout ) go to 950

               case(19)

!              --- itype(icode) = 19, hence the keyword must be followed by
!                                     a series of numbers

                  if ( debug ) call prinrl ( tabmax, icode, 'tabmax' )
                  call intx0219 ( icountkey, icode, table, nout,
     +                            tabout, tabmin, tabmax, code, readnx,
     +                            iposut, ihelp, nmax )
                  if ( nout<nmax+1 ) go to 950
                  if ( iposut>nout ) go to 950

               case(20)

!              --- itype(icode) = 20, hence the keyword is a single keyword in
!                                     which case the table must be set to
!                                     -100000 or the keyword is followed by a
!                                     given other keyword and a series of
!                                     integers

                  call intx0220 ( icountkey, icode, table, nout,
     +                            tabout, tabmin, tabmax, code, readnx,
     +                            nmcode, itype, iposut, ihelp, subkey,
     +                            codemn )

               case(21)

!              --- itype(icode) = 21, hence the keyword is followed by
!                                     (ax+by+cz=d)

                  call  intx0221 ( icountkey, icode, table, nout,
     +                             tabout, code, iposut )
                  if ( iposut>nout ) go to 950

               case(22, 24)

!              --- itype(icode) = 22, 24 hence the keyword is followed by
!                                        possibly one or two special keywords
!                                        that are skipped and then by a
!                                        series of numbers

                  call intx0222 ( icountkey, icode, table, nout,
     +                            tabout, tabmin, tabmax, code, readnx,
     +                            nmcode, itype, iposut, subkey,
     +                            codemn, ihelp, nmax )

               case(23)

!              --- itype(icode) = 23, hence the keyword is followed by possibly
!                                     a special keyword that is skipped
!                                     table(icode) is set equal to 1

                  call intx0223 ( icountkey, icode, table, code,
     +                            readnx, subkey, codemn, nmcode )

               case(25)

!              --- itype(icode) = 25, hence the keyword must be followed by
!                                     a sequence number followed by
!                                     a value or value = v, or func = i

                  call intx0225 ( icode, table, nout, tabout, tabmin,
     +                            tabmax, code, finished, iretur,
     +                            iposut )
                  if ( iposut>nout ) go to 950

               case(27)

!              --- itype(icode) = 27, hence the keyword must be followed by
!                                     a series of values or value = v,
!                                     or func = i

                  call intx0227 ( icountkey, icode, table, nout,
     +                            tabout, tabmin, code, iposut )
                  if ( iposut+2>nout ) go to 950

               case(29,33,37,41,42)

!              --- itype(icode) = 29, 31, 33, 41, 42
!                  hence the keyword must be followed by a sequence number
!                  possibly a subkeyword and then a series of integers

                  call intx0229 ( icode, code, itype, table, codemn,
     +                            nmcode, tabmin, tabmax, tabout, nout,
     +                            jump, iposut, ihelp, finished,
     +                            readnx, iretur )
                  if ( jump/=0 ) go to 950

               case(32)

!              --- type = 32, keyword like:
!                  surfaces (i1, i2, ... in ) = s1 to s2 or
!                  surfaces = s1 to s2, nodes (i1, i2, ... in )

                  call intx0232 ( icode, table, nout, tabout, codemn,
     +                            tabmin, tabmax, code, finished,
     +                            readnx, iposut, ihelp, subkey,
     +                            nmcode, m )
                  if ( iposut+m+2>nout ) go to 950

               case(35)

!              --- itype(icode) = 35, hence the keyword must be followed by
!                                     a sequence number followed by
!                                     a value or value = v, or func = i
!                                     and again the same keyword followed  by
!                                     a sequence number followed by
!                                     a value or value = v, or func = i

                  call intx0235 ( icode, table, nout, tabout,
     +                            tabmin, tabmax, code, finished,
     +                            iposut, iretur )
                  if ( iposut>nout ) go to 950

               case(38, 40)

!              --- itype(icode) = 38, hence the keyword must be followed by
!                  an integer number or a vector name

                  call intx0238 ( icountkey, icode, code, table,
     +                            tabout, nout, iposut, tabmax, itype,
     +                            readnx )

               case(39)

!              --- itype(icode) = 39, hence the keyword must be followed by
!                  an integer number or a scalar name

                  call intx0239 ( icountkey, icode, code, table )

            end select  ! case ( itype(icode) )
            if ( ierror/=0 ) go to 1000

         end if

900   end do  ! while ( .not. finished )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'icount', icount
      if ( icount>0 ) then

!     --- icount>0, hence there are keywords of type 8 and 9
!         Fill information read into table, tabmax and tabout

         if ( debug ) call prinin ( numberkeyw, icount, 'numberkeyw' )
         do i = 1, nncode

            if ( pointer(i)>0 ) then

!           --- Sequence number found

               iseq = pointer(i)
               if ( numberkeyw(iseq)>0 ) then

!              --- keyword appears more than once

                  tabmax(i) =  numberkeyw(iseq)  ! number of numbers read
                  table(i) = iposut+1  ! refers to starting position in tabout
                  if ( iposut+numberkeyw(iseq)>nout ) go to 950

!                 --- Copy information to tabout

                  do k = 1, numberkeyw(iseq)
                     tabout(iposut+k) = valuekeyw(k,iseq)
                  end do  ! k = 1, numberkeyw(iseq)
                  iposut = iposut+numberkeyw(iseq)

               end if  ! ( numberkeyw(iseq)>0 )

            end if  ! ( pointer(i)>0 )

         end do  ! i = 1, icount

      end if  ! ( icount>0 )

      do i = 1, nncode

         if ( ihelp(i)>0 ) then

!        --- ihelp(i)>0, fill tabmax(i)

            tabmax(i) = ihelp(i)
            if ( (itype(i)==29 .or. itype(i)==33 ) .and.
     +            ihelp(i)>nint(tabmin(i))+1 .or.
     +           (itype(i)<29 .or. itype(i)==30 .or.
     +            itype(i)==32 .or. itype(i)>33 ) .and.
     +            ihelp(i)>nint(tabmin(i)) ) then

!           --- Error: more numbers found than permitted

               call errchr ( code(i), 1 )
               call errint ( ihelp(i), 1 )
               call errint ( nint(tabmin(i)), 2 )
               call errsub ( 1440, 2, 0, 1 )
               call errcrd

            end if

         else if ( itype(i)==11 .or. itype(i)==14 .or.
     +             itype(i)==15 .or. itype(i)>17 .and.
     +             itype(i)<=20 .or. itype(i)==22 .or.
     +             itype(i)==24 .or. itype(i)==28 ) then

            tabmax(i) = 0d0

         end if

      end do

      go to 1000

!     --- Error: tabout too small

950   call errint ( nout, 1 )
      call errsub ( 1468, 1, 0, 0 )
      go to 1000

!     --- Error: strout too small

960   call errint ( nstrng, 1 )
      call errsub ( 1469, 1, 0, 0 )

1000  call erclos ( 'intx02ext1' )
      if ( ierror/=0 ) iretur = -1
      ispecl = ispecs
      if ( debug ) then
         write(irefwr,1) 'istop', istop
         write(irefwr,*) 'End intx02ext1'
      end if  ! ( debug )

      end
