      subroutine readcr ( numvec, istart, inpcre, rincre, nstart,
     +                    leninp, lenrin, isubr, kmesh, kmeshn, iseqnr,
     +                    kprob, inforinpcre )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.7  date 06-07-2011 New call to readc0
!        version  6.6  date 10-03-2008 Correction reading number following
!                                      type output
!        version  6.5  date 15-08-2007 Extra debug statements
!        version  6.4  date 24-01-2007 Jump in case of error
!
!   copyright (c) 1989-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Reads the standard input file for subroutine CREATE and stores the
!   information in the arrays INPCRE and RINCRE
!   REMARK:
!   If the description of INPCRE is changed this effects the subroutines:
!   READCR, ER0023 and ER0049
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrk'
      include 'SPcommon/cconst'
      include 'SPcommon/csepch'
      include 'SPcommon/cseprd'
      include 'SPcommon/cmcdpi'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numvec, nstart, istart(nstart), inpcre(*), leninp, lenrin,
     +        isubr, kmesh(*), kmeshn(*), iseqnr, kprob(*),
     +        inforinpcre(*)
      double precision rincre(*)

!     inforinpcre    o    Array of length of rincre that contains a 0 if the
!                         number stored in rincre is a real and a reference
!                         to scalars if it is a scalar
!     inpcre         o    The integer input read is stored in array INPCRE from
!                         position 1. Hence in the call INPCRE(ISTART), with
!                         ISTART the starting position must be used.
!                         INPCRE will be filled in correspondence with array
!                         INPCRE for subroutine CREATV
!     iseqnr        i/o   Sequence number with respect to the storage in array
!                         ISEPIN
!     istart         o    Array containing the starting addresses of the various
!                         vectors in INPCRE
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                         1:  CREATE
!                         2:  PRESDF
!                         3:  ERCREA in the case of create vector
!                         4:  ERCREA in the case of essential boundary
!                             conditions
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     kprob          i    Standard SEPRAN array, containing information of the
!                         problem
!     leninp         i    At input declared length of array INPRCE
!                         At output computed length of array INPCRE
!     lenrin         i    At input declared length of array RINCRE
!                         At output computed length of array RINCRE
!     nstart         i    Declared length of array ISTART
!     numvec         o    Number of SEPRAN arrays to be filled
!     rincre        i/o   Array RINCRE for subroutine CREATV
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nmain, nncode, nextra, nncod1
      parameter ( nmain=3, nncode=6, nextra=6, nncod1=2 )
      integer lstpos, maxrin, i, ipos, icode, nmcode(nmain+nextra),
     +        itype(nncode), icompl, inum, iseq, iprob, nintcr, ncases,
     +        iretur, nmcodmx(nmain), ncode(nncode), nprob,
     +        ncodmx(nncode), ncode1(nncod1), ncodmx1(nncod1),
     +        itype1(nncod1), jprob
      character (len=9) codemn(nmain+nextra)
      character (len=1) strout(1)
      character (len=15) code(nncode)
      character (len=11) code1(nncod1)
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(1), tabmn1(nncod1), tabmx1(nncod1)
      logical first, debug

!     code           Table with keywords
!     code1          Table with keywords
!     codemn         Table with main keywords
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     first          Indication if the first record is read (true) or not
!                    (false)
!     i              Counting variable
!     icode          Return code with respect to subroutine INTX01
!                    Indicates position in array code or code1
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     inum           Number of numbers read in a record
!     ipos           First position in array INPCRE for a new vector
!     iprob          Actual problem number
!     iretur         Return value of subroutine INTX02
!                    Possible values:
!                    -2:  end of file has been reached
!                    -1:  error has been found
!                     0:  normal return
!     iseq           Sequence number of vector to be created
!     itype          Table containing information about the numbers behind
!                    the keywords in the table code
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     jprob          Actual problem number
!     lstpos         Last position used in array INPCRE
!     maxrin         Last position used in array RINCRE
!     ncases         Number of cases for one vector
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx         Table containing the maximal number of  characters in
!                    the keyword table code
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     nextra         Number of extra positions in codemn to store subsubkeywords
!     nintcr         Number of integers stored in INPCRE by subroutine READCR
!     nmain          Number of entries in codemn
!     nmcode         Table containing the number of significant characters
!                    in the main keyword table codemn, including those of
!                    the extra texts
!     nmcodmx        Maximum number of letters in main keywords
!     nncod1         Number of entries in code1
!     nncode         Number of entries in code
!     nprob          Number of various problems
!     strout         Contains series of strings
!     table          Contains the numbers read for the various keywords
!                    Must be filled with the default values at input
!                    If keyword = text is considered, the number to be put
!                    in table is equal to the relative position in the
!                    table codemn, starting with 1
!                    Hence if the text is equal to codemn(tabmin(icode))
!                    table(icode) gets the value 1 etc.
!     tabmax         Contains maximum values allowed in table
!     tabmin         Contains minimum values allowed in table
!     tabmn1         Minimum value of items read corresponding to code1
!     tabmx1         Maximum value of items read corresponding to code1
!     tabout         Contains series of numbers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer intx00

!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTX00         Compare text in string with table (variable length)
!     INTX02         Interpret a record containing keywords followed by numbers
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READC0         Read standard data records
!     READSR         Read a series of numbers from the input (texts are
!                    neglected)
! **********************************************************************
!
!                       I/O
!
!     READCR reads input from the standard input file
!     The input must be of the following shape:
!     If ISUBR=1 then
!        CREATE [COMPLEX] VECTOR [i] [PROBLEM j]    (mandatory)
!        followed by data records of the following shape (optional)
!          TYPE = [SOLUTION VECTOR] [VECTOR OF SPECIAL STRUCTURE [Vj] ]
!                 [VECTOR DEFINED PER ELEMENT [alpha] ]
!                 [SPECIAL_VECTOR_PER_NODE_PER_ELEMENT [alpha] ]
!                 [CAPACITY_VECTOR, NELECTRODES = n [NPHYS]]
!          DATA RECORDS that can be read by subroutine READC0 (See READC0)
!        END (mandatory)
!     If ISUBR=2 then
!        ESSENTIAL [COMPLEX] BOUNDARY CONDITIONS [[PROBLEM] j]
!                  [VECTOR i] (mandatory)
!        followed by data records of the following shape (optional)
!          DATA RECORDS that can be read by subroutine READC0 (See READC0)
!        END (mandatory)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     352   First record for CREATE is not CREATE VECTOR
!     575   First record is not essential boundary conditions
!     883   Parameter i in CREATE [COMPLEX] VECTOR [i], has already been used in
!           a preceding CREATE record. i must be unique.
!     884   Internal error in subroutine READCR.
!           Array INPCRE is too small, please contact SEPRA
!     885   No word found after TYPE = VECTOR
!     886   More than one type record found for the same vector corresponding to
!           CREATE VECTOR
!     887   Wrong character found after TYPE = VECTOR
!     888   Wrong character found after TYPE =
!    1538   Too much stand-alone numbers found
!    2287   Sequence number found in other record than the first one
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'COMPLEX', 'VECTOR', 'PROBLEM', 'SEQUENCE_NUMBER',
     +            'CONDITIONS', 'BOUNDARY' /
      data codemn / 'CREATE', 'ESSENTIAL', 'END', 6*' ' /
      data code1 / 'NELECTRODES', 'NDEGFD' /
      data ncode / 4, 2, 4, 3, 4, 4 /,
     +     itype / 2, 3, 0, 0, 3, -1 /,
     +     table / 0d0, 1d0, 1d0, 1d0, 1d0, 0d0 /,
     +     tabmin /0d0, 1d0, 1d0, 1d0, 1d0, 0d0 /,
     +     tabmax /1d0, 10000d0, 1d0, 1d0, 1d0, 0d0 /
     +     nmcodmx / 6, 9, 3 /
     +     nmcode / 5, 5, 3, 6*0 /
     +     ncodmx / 7, 6, 7,15, 10, 8 /
      data ncode1 / 5, 4 /,
     +     itype1 / 0, 0 /,
     +     tabmn1 /1d0, 1d0 /,
     +     tabmx1 /1d2, 1d2 /
     +     ncodmx1 / 11, 6 /

!     code    Keywords recognized
!             Possible values:
!             1:  COMPLEX
!             2:  VECTOR
!             3:  PROBLEM
!             4:  SEQUENCE_NUMBER
!             5:  CONDITIONS
!             6:  BOUNDARY
!     codemn  Main keywords recognized
!             Possible values:
!             1:  CREATE
!             2:  ESSENTIAL
!             3:  END
!     code1   Subkeywords recognized
!             Possible values:
!             1:  NELECTRODES
!             2:  NDEGFD
! ======================================================================
!
      call eropen ( 'readcr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readcr'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Initialize some parameters

      do i = 1, nstart
         istart(i) = 0
      end do
      numvec = 0
      irecnr = 0
      first  = .true.
      lstpos = 2
      maxrin = 0
      if ( isubr<=2 ) then
         inpcre(1) = -isubr
      else
         inpcre(1) = -(isubr-2)
      end if
      iseq = 0
      ncases = 0
      nprob = kprob(40)
      if ( debug ) write(irefwr,1) 'nprob, isubr', nprob, isubr

!     --- read first record

      if ( .not. first .or. isubr<=2 ) then
         ityprd = 1
         call asknxt
      end if
      icode = intx00 ( codemn, nmcode, nmain, 1, nmcodmx )
      if ( ierror/=0 ) go to 1000
      if ( debug ) then
         write(irefwr,*) 'first ', first
         write(irefwr,1) 'icode', icode
      end if  ! ( debug )

!     --- Analyse record

100   if ( first ) then

!     --- First record, check if this record is of the type create vector

        if ( isubr==1 .and. icode/=1 ) then

!       --- wrong input record:  stop

            call errsub ( 352, 0, 0, 0 )
            call errcrd
            go to 1000

        else if ( isubr==2 .and. icode/=2 ) then

!       --- wrong input record:  stop

            call errsub ( 575, 0, 0, 0 )
            call errcrd
            go to 1000

         end if

      end if
      if ( (isubr==1 .or. isubr==3 ) .and. icode==1 .or.
     +     (isubr==2 .or. isubr==4 ) .and. icode==2 ) then

!     --- Create vector or essential boundary conditions found

         if ( iseq/=0 ) then

!        --- iseq # 0, preceding vector exists
!            Fill position 5 for this vector

            if ( ncases==0 ) ncases = -1
            inpcre(ipos+4) = ncases
            ncases = max(0,ncases)

         end if
         if ( debug )
     +      write(irefwr,1) 'iseq, ncases, iseqnr', iseq, ncases, iseqnr

!        --- Check rest of record
!            Read next item

         tabmax(2) = nstart
         tabmax(3) = nprob
         tabmax(4) = 100d0
         tabmax(5) = nprob
         table(2)  = -1d0
         table(3)  = -1d0
         table(4)  = iseqnr
         table(5)  = -1d0
         iretur = -10
         call intx02 ( code, ncode, nncode, itype, table, codemn,
     +                 nmcode, 0, iretur, tabmin, tabmax, tabout, 1,
     +                 strout, 1, ncodmx )
         if ( ierror/=0 ) go to 1000

         icompl = nint(table(1) )
         if ( first ) then

!        --- First record number: set iseqnr

            iseqnr = nint(table(4) )

         else if ( nint(table(4))/=iseqnr ) then

!        --- not the first record to be read, but sequence number found

            iseqnr = nint(table(4) )
            call errint ( iseqnr, 1 )
            call errsub ( 2287, 1, 0, 0 )
            call errcrd

         end if

         if ( isubr==1 .or. isubr==3 ) then
            iseq  = nint(table(2))
            if ( iseq==-1 ) iseq = 1
            iprob = max(nint(table(3)),1)
         else
            iseq = iseq+1
            if ( nint(table(2))>0 ) iseq = nint(table(2))
            iprob = min(nprob,iseq+iseqnr-1)
            if ( nint(table(3))/=-1 ) then
               iprob = nint(table(3))
            else if ( table(5)>=0 ) then
               iprob = nint(table(5))
            end if
         end if
         if ( iretur<-10 ) then
            call errint ( -iretur-10, 1 )
            call errint ( 0, 2 )
            call errsub ( 1538, 2, 0, 0 )
            call errcrd
         end if
         iretur = 0
         if ( debug )
     +      write(irefwr,1) 'iseqnr, iprob, lstpos, icompl',
     +                       iseqnr, iprob, lstpos, icompl

!        --- Store information in ISTART and INPCRE

         if ( istart(iseq)/=0 ) then

!        --- Error 883:  ISEQ has already been used

            call errint ( iseq, 1 )
            call errsub ( 883, 1, 0, 0 )
            call errcrd

         else

!        --- Fill information

            ipos = lstpos+1
            if ( ipos+5>leninp ) then

!           --- Leninp too small

               call errint ( leninp, 1 )
               call errint ( ipos+5, 2 )
               call errsub ( 884, 2, 0, 0 )
               go to 1000

            end if

            istart(iseq) = ipos
            numvec = max ( numvec, iseq )

!           --- Set defaults

            inpcre(ipos  ) = 1
            inpcre(ipos+1) = iprob
            inpcre(ipos+2) = 1
            inpcre(ipos+3) = icompl
            inpcre(ipos+4) = 0
            lstpos = lstpos + 5
            ncases = 0
            if ( debug ) then
               write(irefwr,1) 'ipos, iprob, icompl, lstpos',
     +                          ipos, iprob, icompl, lstpos
               call prinin ( inpcre(ipos), 5, 'inpcre_loc' )
            end if  ! ( debug )

         end if

      end if
      first = .false.

!     --- Read rest of record: type, info or main keyword

      call asknxt
      icode = intx00 ( codemn, nmcode, nmain, 0, nmcodmx )
      if ( debug ) write(irefwr,1) 'icode, ityprd', icode, ityprd
      if ( ierror/=0 ) go to 1000

      if ( icode==0 .and. ityprd==4 .and. string(1:4)=='TYPE'  )
     +   then

!     --- TYPE record found, analyse rest of record

         if ( inpcre(ipos)/=1 ) then

!        --- error 886: more than one type record

            call errsub ( 886, 0, 0, 0 )
            call errcrd
            go to 200

         end if

!        --- Read next word

         call asknxt
         if ( ityprd/=4 ) then

!        --- No word found after TYPE

            call errsub ( 885, 0, 0, 0 )
            call errcrd
            go to 200

         else if ( string(1:3)=='SOL') then

!        --- Solution found

            inpcre(ipos) = 1

         else if ( string(1:3)=='VEC') then

!        --- Vector of special structure
!            Read next keyword

            call asknxt  ! keyword of
            if ( ityprd/=4 ) then

!           --- wrong character found

               call errsub ( 887, 0, 0, 0 )
               call errcrd
               go to 200

            else if ( string(1:2)=='DE' ) then

!           --- Vector of special structure defined per element

               inpcre(ipos) = 3
               call asknxt  ! keyword per
               if ( ityprd==4 ) call asknxt  ! keyword element
               if ( ityprd==4 ) call asknxt  ! number
               if ( ityprd==2 )
     +            inpcre(ipos+2) = nint(realrd)  ! sequence number found

            else

!           --- Vector of special structure

               call asknxt  ! keyword special
               if ( ityprd==4 ) call asknxt  ! keyword structure
               if ( ityprd==4 ) call asknxt  ! keyword v
               if ( ityprd==4 ) call asknxt  ! number
               if ( ityprd==2 )
     +            inpcre(ipos+2) = nint(realrd)  ! sequence number found
               inpcre(ipos) = 2

            end if

         else if ( string(1:3)=='SPE') then

!        --- Vector of special structure defined per node per element (type 126)

            inpcre(ipos) = 4

!           --- Read one number

            call readsr ( work, 1, inum )
            if ( inum==1 ) inpcre(ipos+2) = nint(work(1))

         else if ( string(1:8)=='CAPACITY') then

!        --- Vector of type capacity (801)
!            Check rest of record

            table(1)  = 1d0
            table(2)  = 1d0
            iretur = -10
            call intx02 ( code1, ncode1, nncod1, itype1, table, codemn,
     +                    nmcode, 0, iretur, tabmn1, tabmx1, tabout, 1,
     +                    strout, 1, ncodmx1 )
            inpcre(ipos) = 5
            inpcre(ipos+2) = nint(table(1))+1000*(nint(table(2))-1)

         else

!        --- Wrong character found after TYPE =

            call errsub ( 888, 0, 0, 0 )
            call errcrd
            go to 200

         end if
         ityprd = 1
         if ( ierror/=0 ) go to 1000

      else

!     --- Not type found, reset icolum, kcolum

         if ( icode==0 ) istop = 3

      end if

200   if ( icode==0 ) then

!     --- Not a main keyword, check rest of record

210      ncases = ncases+1
         jprob = 0
         call readc0 ( inpcre(lstpos+1), rincre, nintcr, maxrin, icode,
     +                 icompl, kmesh, codemn, nmain, nmcode, kmeshn,
     +                 nextra, inforinpcre, jprob )
         if ( ierror/=0 ) go to 1000
         if ( debug ) then
            write(irefwr,1) 'nintcr, lstpos, icompl/2',
     +                       nintcr, lstpos, icompl
            call prinin ( inpcre(lstpos+1), nintcr, 'inpcre_loc2' )
         end if  ! ( debug )
         lstpos = lstpos + nintcr
         if ( icode==0 ) go to 210
         ncases = ncases-1

      end if
      if ( ierror/=0 ) go to 1000

      if ( icode<3 ) go to 100

!     --- END record found
!         Fill position 4

      inpcre(ipos+4) = ncases

!     --- Check if there are any "empty records"
!         Loop over all vectors

      do i = 1, numvec

         if ( istart(i)==0 ) then

!        --- istart(i) =0, vector i has not been filled
!            Put default values into INPCRE

            istart(i) = lstpos + 1
            if ( lstpos+6>leninp ) then

!           --- Leninp too small

               call errint ( leninp, 1 )
               call errint ( ipos+5, 2 )
               call errsub ( 884, 2, 0, 0 )
               go to 1000

            end if
            inpcre(lstpos+1) = 0
            inpcre(lstpos+2) = 1
            inpcre(lstpos+3) = 1
            inpcre(lstpos+4) = 0
            inpcre(lstpos+5) = -1
            lstpos = lstpos + 5

         end if

      end do

      inpcre(2) = numvec
      leninp = lstpos
      lenrin = maxrin

1000  call erclos ( 'readcr' )

      if ( debug ) then

!     --- Debug information

         call prinin ( inpcre, lstpos, 'inpcre' )
         call prinrl ( rincre, maxrin, 'rincre' )
         write(irefwr,*) 'End readcr'

      end if

      end
