      subroutine readscbouncond ( ibuffr, istart, inpsep,
     +                            tabout, icode, nout, strout, nprob,
     +                            numvec, rinsep, istrin, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-07-2011 Extra parameter ibuffr
!        version  1.1  date 26-06-2011 extension with zero_level_set
!        version  1.0  date 04-10-2010
!
!   copyright (c) 2010-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following standard keywords with respect
!     to subroutine readsc
!             1:  PRESCRIBE_BOUNDARY_CONDITIONS
!     The input is stored in array inpsep and the real array rinsep
!     If this subroutine is adapted also adapt ini016
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     output
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/comcons1'
      include 'SPcommon/comcons2'
      include 'SPcommon/cuscons'
      include 'SPcommon/cusname'
      include 'SPcommon/cpost5'
      include 'SPcommon/cseprdnew'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*),  istart, inpsep(*), icode, nout, nprob, numvec,
     +        istrin, kmesh(*)
      double precision tabout(*), rinsep(*)
      character (len=*) strout(*)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     icode          i    Indicates which of the options in readsc is found
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         In the standard case this part contains:
!                         2  ISEQNR   Sequence number corresponding to input
!                                     If iseqnr = -10, position 5 ... are used
!                                     See subroutine readscboun1
!                         3  IPROB    Problem sequence number
!                                     If iprob = -1, the next one will be used
!                         4  IVECT    Vector sequence number (first vector)
!                                     If ivector = -1, the same value as iprob
!                                     will be used
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nout           i    Number of entries in tabout
!     nprob          i    Number of various problems
!     numvec        i/o   Number of vectors stored in isol
!     rinsep        i/o   Array containing the real input for this subroutine
!     strout        i/o   Contains series of strings
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer errorcode, istartbef, istrinbef
      logical debug, outfnd

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     errorcode      If 0 at input an error message is given if a keyword
!                    is not recognized
!                    If>0, the subroutine does not give an error message,
!                    but instead returns with -1
!     istartbef      Value of istart at input
!     istrinbef      Value of istrin at input
!     outfnd         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READSCBOUN1    Read input following prescribe_boundary_conditions vector
!                    No sequence number found
!     READSCSTANDARD Read keywords after a number of standard keywords
! **********************************************************************
!
!                       I/O
!
!       PRESCRIBE_BOUNDARY_CONDITIONS, options (all in one line)
!           where options may have the following contents
!               SEQUENCE_NUMBER = s  (Default the next one)
!               VECTOR = s           (Default the same as problem)
!               PROBLEM = s          (Default 1)
!               VALUE = v            (No default)
!               FUNC = f             (No default)
!               QUADRATIC, MAX = a, MEAN_VALUE = v, BOUNDARY_LAYER_WIDTH = b,
!                 DIFFERENCE = delta_T
!               HALF_QUADRATIC, MAX = a
!               POINTS = (Pi, Pj, Pk )
!               CURVES = (Ci, Cj, Ck )
!               SURFACES = (Si, Sj, Sk )
!               ZERO_LEVELSET i
!               OLD_VECTOR = m
!               SEQ_VECTORS = V1, V2, ...
!      In case of CREATE_VECTOR, extra
!               OUTPUT_VECTOR = ivec
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscbouncond' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscbouncond'
         write(irefwr,1) 'istart, istrin', istart, istrin
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,1) 'lastitem, istart', lastitem, istart
         istartbef = istart
         istrinbef = istrin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First we read the standard keywords and check if there are
!         no other keywords

      errorcode = 1   ! return when an unknown keyword is found
      call readscstandard ( istart, inpsep, tabout, outfnd, icode,
     +                      nout, strout, nprob, numvec, errorcode )
      if ( debug )
     +   write(irefwr,1) 'errorcode, lastitem', errorcode, lastitem

      if ( errorcode<0 ) then

!     --- errorcode = -1, hence other keyword found
!         read rest of keywords with standard routine

         call readscboun1 ( ibuffr, icode, kmesh, inpsep,
     +                      istart, rinsep, istrin, nprob )

      end if  ! ( errorcode<0 )

1000  call erclos ( 'readscbouncond' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         call prinin ( inpsep(istartbef), istart-istartbef, 'inpsep' )
         if ( istrin>istrinbef )
     +     call prinrl ( rinsep(istrinbef), istrin-istrinbef, 'rinsep' )
         write(irefwr,*) 'End readscbouncond'

      end if  ! ( debug )

      end
