c ********************************************************************
c *
c *   example for the solution of the Boussinesq equations
c *   Square cavity problem  (Free convection)
c *   Flow by temperature difference
c *   programmer Guus Segal
c *   date 04-04-87
c *
c ********************************************************************
c *
      program freeconv
      use sepmoduleoldrouts
      implicit double precision (a-h,o-z)
      integer :: kmesh,kprob,iuser(100),intmat, ipress,isol,istrm
      double precision :: user(100)
c
      kmesh=0
      kprob=0
      iuser = 0
      intmat = 0
      isol = 0
      istrm = 0
      user = 0
      iuser(1)=100
      user(1)=100
      iuser(3)=0
      user(3)=0
c
c ********************************************************************
c
c   *  start of program  (standard for each sepran program)
c
c ********************************************************************
c
      print *,'PROGRAM BOUSS'
      istart=1
      irotat=1
      ioutp=0
      itime=0
      call start(istart,irotat,ioutp,itime)
c
      ichois=0
      call mesh(ichois,iinput,rinput,kmesh)
c
      ichois=0
      call probdf(ichois,kprob,kmesh,iinput)
c
      jmetod=2
      call commat(jmetod,kmesh,kprob,intmat)
c
c ********************************************************************
c
c   *  fill boundary conditions
c
c ********************************************************************
c
c
c   *   first set all boundary conditions equal to zero
c
      idim=0
      call bvalue(ichois,idim,kmesh,kprob,isol,value,icrv1,icrv2,idegfd,
     .            jstap)
c
c     temperature 1 at wall c8:
c

      value=1d0
      ichois=0
      idim=2
      icrv1=8
      icrv2=8
      idegfd=3
      jstap=0
      call bvalue(ichois,idim,kmesh,kprob,isol,value,icrv1,icrv2,idegfd,
     .            jstap)
c
c ********************************************************************
c
c   *    solve the system of non-linear equations
c
c ********************************************************************
c
      itertp=1
      maxitr=10
      eps=1d-4
      iprint=-2
      ichois=-1
      call nonlin(itertp,maxitr,eps,iprint,iwork,work,intmat,
     .            ichois,iuser,user,kmesh,kprob,isol,ielhlp)
c
c*  proceed with post-processing of the solution
c
      call prinrv(isol,kmesh,kprob,4,0,'velocity and temperature')
c
c*********************************************************************
c
c*   compute the pressure
c
c*********************************************************************
c
c     iuser,user as required by nonlin (is still present)
c
      icheld=1
      call deriva(ichois,icheld,ix,jdegfd,ivec,ipress,kmesh,kprob,
     .            isol,isol,iuser,user,ielhlp)
      call prinrv(ipress,kmesh,kprob,4,0,' pressure')
c
c*********************************************************************
c
c*   compute the stream function  , set strm(1)=0.0d0
c
c*********************************************************************
      ichels=1
      jpoint=1
      strm1=0d0
      call stream(ichels,ivec,istrm,jpoint,strm1,kmesh,
     .            kprob,isol)
c******************************************************************
c
c**   plot velocity vectors
c
c******************************************************************
c
      facvec=0d0
      call plotvc(1,2,isol,isol,kmesh,kprob,20d0,1d0,facvec)
c
c******************************************************************
c
c*  plot contour lines for the pressure, the stream function and
c   the temperature
c
c******************************************************************
c
      ncntln=0
      number=1
      call plotc1(number,kmesh,kprob,ipress,contln,ncntln,
     .         20d0,1d0,0)
      call plotc1(number,kmesh,kprob,istrm,contln,ncntln,
     .         20d0,1d0,0)
c
      number=3
      call plotc1(number,kmesh,kprob,isol,contln,ncntln,
     .         20d0,1d0,0)
      print *,'END PROGRAM BOUSS'
      call finish(istop)
      end
c
c=====
c
      subroutine filnln(niter,iuser,user,kmesh,kprob,isol,relax,differ)
      implicit double precision (a-h,o-z)
      integer :: iuser(*),kmesh,kprob,isol,iwork(20)
      double precision :: work(20),user(*)
c
c
c      1: penalty function parameter
c      2: density of fluid
c      3: type of linearization of convective terms
c                (0 = no convective terms, stokes flow)
c                (1 = picard)
c                (2 = newton)
c      4: angular velocity of rotating system
c      5: volume expansion coefficient
c      6: reference temperature
c      7: heat capacity at constant pressure
c      8: thermal conductivity
c      9: body force in x-direction
c     10: body force in y-direction
c     11: heat source per unit mass
c     12: type of constitutive equation (1 = eta constant)
c     13: eta
c     14: ct
c     15: reference temperature T1
c
      do 100 i=1,15
         iwork(i)=0
         work(i)=0d0
100   continue
c
c    *  define Rayleigh and Prandtl number
c
      ra=1d3
      pr=.71d0
      work(1)=1d-8
c
c   *  the pressure is of order 1000
c
      work(2)=1d0
      iwork(3)=min(niter,2)
      work(5)=ra/pr/9.81d0
      work(7)=1d0
      work(8)=1d0/pr
      iwork(12)=1
      work(13)=1d0
      iparm=15
c
      call fil100(1,iuser,user,kprob,iparm,iwork,work)
c
      end
