      program wheel

!      --- Main program for the Wheel-problem:
!          An elastic layer (the "tire") is fixed to a rigid cylinder
!          (the "hub").
!          This hub is compressed downwards and the tire is pressed onto the
!         "road".  The contact area increases for increasing load.
!          This main program is necessary because of the variable boundary
!          conditions

      implicit none

      integer, allocatable, dimension (:) :: ibuffr
      integer (kind=8) pbuffr
      integer error
      parameter ( pbuffr=25000000)
      allocate(ibuffr(pbuffr), stat = error)
      if (error /= 0) then
         ! space for these arrays could not be allocated
         print *, "error: (wheel) could not allocate space."
         stop
      end if  ! (error /= 0)

      call freebsub ( ibuffr, ibuffr, pbuffr )
      end

!     --- Function subroutine for the boundary conditions

      function funcbc( ichoice, x, y, z)
      implicit none

      integer ichoice
      double precision funcbc, x, y, z

      integer ifirst
      double precision Ru
      double precision getconst
      save Ru
      data ifirst /0/

      if ( ifirst==0 ) then

!     --- ifirst = 0, first call of funcbc
!         Get the values of some constants

         ifirst = 1
         Ru = getconst ( 'Ru' )

      end if

      if (ichoice==11) then

!     --- ichoice = 11, contact surface
!         In order to restrict the solution to z = -Ru it is necessary
!         to set the displacement equal to -Ru - z

        funcbc =  -Ru - z

      end if

      end

      subroutine funcvect ( ichoice, ndim, coor, numnodes,
     +                      uold, nuold, result, nphys )
      implicit none

      integer ichoice, ndim, numnodes, nuold, nphys
      double precision coor(ndim, numnodes),
     +                 uold( numnodes, nphys, nuold),
     +                 result( numnodes, *)

      integer k, ifirst
      double precision Ru
      save Ru
      double precision getconst
      data ifirst /0/

      if ( ifirst==0 ) then

!     --- ifirst = 0, first call of funcbc
!         Get the values of some constants

         ifirst = 1
         Ru = getconst ( 'Ru' )

      end if

      if (ichoice==31) then

!     --- ichoice = 31, contact distance = u_z + z + Ru

        do k = 1, numnodes
           result(k,1) = (coor(3,k) + uold(k,3,1)) + Ru
        end do

      else if (ichoice==41) then

!     --- ichoice = 41, contact force is third component of reaction force

        do k = 1, numnodes
           result(k,1) = uold(k,3,1)
        end do

      end if

      end
