      subroutine el0800 ( iuser, user, jelp, kfirst, jheat, isub,
     +                    isup, array, iseq, itypln, iseqtemp,
     +                    iseqenth, imass_mat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.4  date 12-08-2011 extra debug
!        version  5.3  date 10-08-2011 Extension with icheld = 7
!        version  5.2  date 09-03-2009 Type numbers 813
!
!   copyright (c) 1992-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM800 or
!     ELD800 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/cellog'
      include 'SPcommon/cactl'
      include 'SPcommon/celint'
      include 'SPcommon/celiar'
      include 'SPcommon/celp'
      include 'SPcommon/cinforel'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, kfirst, jheat, isub, isup,
     +        iseq, itypln, iseqtemp, iseqenth, imass_mat

!     array          o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!                         in the sequence:
!                              1D                  2D            3D
!                         1:   alpha 11         alpha 11       alpha 11
!                         2:                    alpha 12       alpha 12
!                         3:                    alpha 22       alpha 13
!                         4:                                   alpha 22
!                         5:                                   alpha 23
!                         6:                                   alpha 33
!                         7:   u 1              u 1            u 1
!                         8:                    u 2            u 2
!                         9:                                   u 3
!                        10:   beta             beta           beta
!                        11:   f                f              f
!                        12:   rho cp           rho cp         rho cp
!                        13:   gamma_1          gamma_1        gamma_1
!                        14:                    gamma_2        gamma_2
!                        15:                                   gamma_3
!                        16:   g_extra          g_extra        g_extra
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     imass_mat      o    If 0 the standard mass matrix must be computed
!                         otherwise a diffusion term must be added to
!                         this matrix
!     iseq           i    Sequence number of vector in VECOLD/ISLOLD
!                         In case of a call by ELI800 it is the parameter JDEGFD
!     iseqenth       i    Sequence number of enthalpy with respect to array isol
!     iseqtemp       i    Sequence number of temperature in solution vector
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     itypln         o    Indicates the type of lineariation to be performed
!                         Possible values:
!                         Type 800:
!                            1: Standard equation
!                            2: convective term is |ux| + |uy| + |uz|
!                            3: convective term is g(grad(c)),
!                               Newton linearization
!                         Type 803:
!                            1: Picard
!                            2: Newton
!     iuser          i    User array in which integer information of the
!                         coefficients is stored (See Standard Problems)
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELM800. Possible vaues:
!                         1:  Subroutine ELP610  (Linear line element)
!                         2:  Subroutine ELP611  (Quadratic line element)
!                         3:  Subroutine ELP620  (Linear triangle)
!                         4:  Subroutine ELP622  (Quadratic triangle)
!                         5:  Subroutine ELP621  (Bilinear quadrilateral)
!                         6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                        11:  Subroutine ELP630  (Linear tetrahedron)
!                        12:  Subroutine ELP631  (Quadratic tetrahedron)
!                        13:  Subroutine ELP632  (Trilinear brick)
!                        14:  Subroutine ELP633  (Triquadratic  brick)
!     jheat          o    Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed
!                         Possible values:
!                         0:  no mass matrix
!                         1:  diagonal mass matrix with constant coefficient
!                         2:  diagonal mass matrix with variable coefficient
!                        11:  full mass matrix with constant coefficient
!                        12:  full mass matrix with variable coefficient
!     kfirst         o    This parameter indicates the smallest velocity
!                         component that is not equal to zero in the convective
!                         terms
!                         For example kfirst = 2, means that the y-
!                         component of the velocity is unequal to zero
!     user           i    User array in which real information of the
!                         coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, i, j, isubcf, icoef, kchois, ishift, mrule,
     +        ichtrn, metup, interpol, iqmat, iqmat1
      logical debug, linear, funchk
      double precision rho, cs, cl, mu

!     cl             Specific heat of the liquid (used in enthalpy method)
!     cs             Specific heat of the solid (used in enthalpy method)
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     ichtrn         Defines the direction of the computation of an integral
!     icoef          Coefficient sequence number
!     interpol       If 0 the standard interpolation is applied,
!                    if 1 the interpolation is restricted to linear subelements
!     iqmat          Indication whether a transformation has to be performed if
!                    the elements are of a lower dimension than the dimension of
!                    the coordinates.  If 0 no tranformation is performed
!                    (standard case) if 1 then the velocities are mapped to
!                    the dimension of the elment
!     iqmat1         Indication whether the right-hand side has to be
!                    transformed
!     ishift         Shift in coefficient sequence number
!     isubcf         Under bound for coefficients check
!     j              Counting variable
!     jwcons         Indication if the weight function is constant (1)
!                    or not (2)
!     kchois         Indication what type of parameters must be computed
!                    Possible values:
!                    0:  Standard method, the calling subroutine is ELM800
!                        It is supposed that an element matrix must be computed
!                    1:  Subroutine ELD800 calls this subroutine, only
!                        derivatives must be computed
!                    2:  Subroutine ELD800 or ELI800 calls this subroutine,
!                        a user function defined as first parameter in
!                        IUSER/USER must be filled in elemvc (icheld=9)
!                    3:  Subroutine ELD800 calls this subroutine,
!                        alpha times the gradient must be stored in ELEMVC
!                        The first 3 parameters of IUSER/USER are used
!     linear         Indicates whether the element is a linear simplex (true)
!                    or not (false)
!     metup          Upwind parameter
!     mrule          Type of integration rule as given by the user
!     mu             relaxation parameter for the Newton enthalpy method
!     ncoef          Number of coefficients in the equation
!     rho            density rho (used in enthalpy method)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     EL0800DER      Compute some parameters related to derivatives or
!                    integrals
!     EL0800IN       Compute the number of integration points for elements of
!                    type 800
!     EL1002         Compute JSECND for diffusion terms
!     EL1013         Compute KFIRST and IDIM for convection terms
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     In the case of a call by BUILD:
!     1:  Indicates if the velocities and the diffusion tensor must be
!         transformed from 3D to a local lower dimensional system (10,20)
!         or not (0)
!         If icoef1 = 20, then also the right-hand side is mapped
!         If icoef1 = 100, a special case for the update of the coordinates
!         of a free surface by a convection equation is triggered
!     2:  Type of upwinding
!         Possible values:
!         0: no upwinding
!         1: Brooks and Hughes: classical upwind scheme
!         2: Brooks and Hughes: Il'in scheme
!         3: Brooks and Hughes: Doubly assymptotic approximation
!         4: Brooks and Hughes: Critical approximation
!         5: Hughes approximation
!         6: Time-dependent approximation
!         7: Hughes et al: Discontinuity capturing 1
!         8: Hughes et al: Discontinuity capturing 2
!         9: Mizukami triangles satisfying maximum principle
!        10: Mizukami triangles satisfying maximum principle,
!            activate flip-flop detection
!        11: Mizukami triangles satisfying maximum principle,
!            use flip-flop detection
!        12: Time-dependent approximation according to Tornberg and Enquist
!       21-32: See metupw-20, however, only the convective term
!            is adapted
!       >99: See metupw-100, however the mass matrix is not
!            weighted by the new basis functions
!         Possible values for icompress:
!         0: The convective term has the shape rho u . grad c
!                                                  -
!         1: The convective term has the shape div ( rho u c )
!                                                        -
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         This parameter consists of 2 parts: jcart and jtransform
!         according to jcart + 10*jtransform
!         Possible values:
!         jcart:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!         2: Polar co-orindates
!         jtransform: transformation from 3d to 2d
!         0:  standard, no special transformation
!         1:  transformation from 3d to 2d, with z constant
!             first coordinate becomes x, second y
!         2:  transformation from 3d to 2d, with x constant
!             first coordinate becomes y, second z
!         3:  transformation from 3d to 2d, with y constant
!             first coordinate becomes x, second z
!         4:  transformation from 3d to 2d, using cylinder co-ordinates
!             xsi = atan2(x,y), eta = z
!     5:  This parameter depends on the type number to be used
!         icoef5 consists of 2 parts: icoef5_orig + 10*imas_mat
!
!         icoef5_orig:
!
!         In case of type number 803 it indicates the type of linearization:
!         Possible values:
!         0,1: Picard
!         2:   Newton
!         In case of type number 800 it indicates if the special possibility
!         with respect to the first derivatives are considered
!         0,1: standard
!         2:   The absolute values of the first derivatives are used:
!              Hence |ux|+|uy|+|uz|
!              In this case the problem is non-linear and an initial estimate
!              is necessary
!         3:   The convective terms are of the form g(grad c), with g
!              a given differentiable function.
!              The derivative of g to grad c is known
!              This is also a non-linear problem
!         4-6: Special case, a convection equation along the surface is
!              solved where the main direction is defined in the itypln-3
!              direction
!              The convection equation to be solved is:
!              w dh/ds = u_i ( s, e_i )
!              h is the displacement in e_i direction
!              w is the velocity in the main direction (not i)
!              s is the direction of the projection of e_main onto the free
!              surface
!              Hence the right-hand side is defined by the velocity
!
!         imass_mat:
!
!         indicates how the mass matrix must be computed
!         0:   Standard way
!         1:   Mass matrix is defined by standard mass matrix +
!              coef21 * second derivative term (i.e. extra diffusion
!              in time derivative
!         2:   The term - coef22 grad u phi_i grad phi_j is added
!              (convective term)
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!    12:  u 1       (1D/2D/3D)
!    13:  u 2       (2D/3D)
!    14:  u 3       (3D)
!    15:  beta      (1D/2D/3D)
!    16:  f         (1D/2D/3D)
!    17:  rho cp    (1D/2D/3D)
!    18:  gamma_1
!    19:  gamma_2
!    20:  gamma_3
!    21:  C_1       Coefficient for diffusion in time derivative (imass_mat>=1)
!    22:  C_2       Coefficient for convection in time derivative (imass_mat=2)
!         It indicates the vector sequence number of the vector to
!     In the case of a call by INTEGR:
!     1:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     2:  Type of co-ordinate system
!         This parameter consists of 2 parts: jcart and jtransform
!         according to jcart + 10*jtransform
!         Possible values:
!         jcart:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!         jtransform: transformation from 3d to 2d
!         0:  standard, no special transformation
!         1:  transformation from 3d to 2d, with z constant
!             first coordinate becomes x, second y
!         2:  transformation from 3d to 2d, with x constant
!             first coordinate becomes y, second z
!         3:  transformation from 3d to 2d, with y constant
!             first coordinate becomes x, second z
!         4:  transformation from 3d to 2d, using cylinder co-ordinates
!             xsi = atan2(x,y), eta = z
!     3:  Not yet used (must be 0)
!     4:  f
!     In the case of a call by DERIV:
!     1:  ISEQ:  Sequence number of vector in VECOLD/ISLOLD from which UOLD and
!         derivatives must be computed.
!         If 0 or 1 the first vector is used
!         If more vectors are needed, these vectors are supposed to be stored
!         sequentially, where the first one is indicated by ISEQ
!     2-5:  Not yet used
!         In the cases ICHELD = 6, 16 the positions 6-11 contain
!         information about the matrix A:
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!         In the case ICHELD = 21 position 6 contains information about the
!         function f to be filled:
!     6:  Function f
!         In the case ICHELD = 24,25 the positions are filled as follows
!         6-11: alpha, 12-14: u1, 15: F0, 16,17: f, 18: rho,
!         19-21:gamma, 22: h, 23-24: u2
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     117   mass matrix not consistent, wheras diffusion is part of the
!           time derivative
!     264   jdegfd has wrong value
!     341   Icheli has wrong value
!     406   Value of icheld not allowed
!     771   Rho is non-positive
!    1305   Coefficient has wrong value
!    1527   Parameter in input out of range
!    1541   The combination imas = 1 and ishape=4 is not permitted
!    1695   Possibility icoef. = . has not been implemented for this shape
!    1863   Value of icheli has not yet been implemented for this element shape
!    2394   this type of element has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill some standard values in the common blocks celp and celint

      if ( icheld<0 ) then
         ichtrn = -icheld/10
      else
         ichtrn = 0
      end if

!     --- Set a number of defaults

      iweigh = 1
      ideriv = 1
      irule  = 1
      igausp = 0
      iphi   = 0
      jcart  = 1
      icoor  = 1
      ncomp  = 1
      jdiag  = 1
      idim   = 1
      jderiv = 0
      jtrans = 0
      jind   = 1
      kstep  = 1
      itypln = 0
      jheat  = 0
      interpol = 0
      iqmat = 0
      iqmat1 = 0
      ishift = 0
      imass_mat = 0
      if ( itype==325 ) iweigh = 21

!     --- Set parameters depending on the shape numbers

      call el0800in ( iuser, kchois, metup, mrule, ncoef, linear,
     +                jelp, ishift, interpol, iqmat, iqmat1,
     +                iseq, jheat, jwcons )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'kchois, metup, mrule, ncoef, jelp',
     +                    kchois, metup, mrule, ncoef, jelp
         write(irefwr,1) 'ishift, interpol, iqmat, iqmat1',
     +                    ishift, interpol, iqmat, iqmat1
         write(irefwr,1) 'iseq, jheat, jwcons, jsecnd',
     +                    iseq, jheat, jwcons, jsecnd

      end if  ! ( debug )

!     --- if derivatives are computed set iseq equal to iseqi1

      if ( icheld>=0 ) iseq = iseqi1

!     --- set some parameters with respect to derivatives

      call el0800der ( icheld, iweigh, jderiv, ndimlc, ideriv,
     +                 icheli, ix )

      if ( icheld==4 .or. icheld>=8 .and. icheld<=10 .or.
     +     icheld==14 .or. icheld>=17 .and. icheld<=20 .or.
     +     icheld>=26 .and. icheld<=30 .or. icheld==34 .or.
     +     icheld>=37 .and. icheld<=40 .or.
     +     itype/=810 .and. icheld>40 .or.
     +     itype==810 .and. icheld>45 ) then

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )

      else if ( icheli>8 .or. ichtrn>ndim ) then

         call errint ( -icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 341, 3, 0, 0 )

      end if
      if ( icheli>=3 .and. icheli<=5 ) then

         ix = icheli-2
         if ( ix>ndim ) then
            call errint ( -icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 341, 3, 0, 0 )
         end if

      end if
      if ( icheli==6 .and. (iseq<=0 .or. iseq>ndim) ) then
         call errint ( iseq, 1 )
         call errint ( ndim, 2 )
         call errsub ( 264, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      if ( icheli==6 .or. icheld==7 ) ncomp = ndim

      mn = m * n

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      if ( kchois==0 ) then
         if ( jheat==0 ) notmas = 1
      end if  ! ( kchois==0 ) then

      if ( debug )
     +   write(irefwr,1) 'n, m, ncoef, icheld', n, m, ncoef, icheld

      if ( ncoef>0 ) then

         nuser = 0

         isub = 1
         if ( itype==800 .or. itype==803 .or. itype==811 .or.
     +        itype==812 ) then

!        --- Type 800 or 803, determine the type of linearization
!            It concerns icoef5

            itypln = inirepscal ( iuser(ipos) )  ! icoef5
            imass_mat = itypln/10                ! imass_mat (standard 0)
            itypln = max(1,itypln-10*imass_mat)  ! original itypln
            if ( debug )
     +         write(irefwr,1) 'itypln, imass_mat', itypln, imass_mat
            if ( itypln<1 .or. itypln>6 .or.
     +           itype==803 .and. itypln>2 ) then
               if ( itype==803 ) then
                  call errchr ( 'type of linearization', 1 )
               else
                  call errchr ( 'type of special equation', 1 )
               end if
               call errint ( 5, 1 )
               call errint ( itypln, 2 )
               call errint ( 1, 3 )
               call errint ( 2, 4 )
               call errint ( ielem, 5 )
               call errint ( itype, 6 )
               call errint ( ielgrp, 7 )
               call errsub ( 1527, 7, 0, 1 )
               go to 1000
            end if
            if ( imass_mat>2 ) then

!           --- imass_mat out of range

               call errint ( 5, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errint ( imass_mat, 5 )
               call errchr ( 'imass_mat', 1 )
               call errsub ( 1305, 5, 0, 1 )
               go to 1000

            end if  ! ( imass_mat>2 )
            if ( itype==803 ) then

!           --- itype = 803

               isub = 7
               idim = 0

            else if ( itypln==2 ) then

!           --- itypln = 2

               if ( .not. linear ) then
                  call errint ( 5, 1 )
                  call errint ( 2, 2 )
                  call errint ( ishape, 3 )
                  call errsub ( 1695, 3, 0, 0 )
               end if
               kfirst = 1
               idim = ndim

            else if ( itypln>3 ) then

!           --- itypln = 4, 5, 6
!               Special case, a convection equation along the surface is
!               solved where the main direction is defined in the itypln-3
!               direction
!               The convection equation to be solved is:
!               w dh/ds = u_i ( s, e_i )
!               h is the displacement in e_i direction
!               w is the velocity in the main direction (not i)
!               s is the direction of the projection of e_main onto the free
!               surface
!               Hence the right-hand side is defined by the velocity

               iqmat = itypln+2     ! main direction + 5
               iweigh = iweigh + 10*iqmat
               if ( ishape<1 .or. ishape>3 .and. ishape/=5 )
     +            then

!              --- Error 2394: this type of element has not yet been implemented

                  call errint ( ishape, 1 )
                  call errint ( ielem, 2 )
                  call errint ( itype, 3 )
                  call errint ( ielgrp, 4 )
                  call errint ( 5, 5 )
                  call errint ( itypln, 6 )
                  call errsub ( 2394, 6, 0, 0 )

               end if

            end if

         else if ( itype==810 .and. icheld>=42 .or. itype==813 ) then

!        --- itype = 810, enthalpy equation by newton method

            iseqtemp = iuser(ipos)

         end if  ! ( itype==800 .or. itype==803 )
         if ( debug ) write(irefwr,1) 'jsecnd/2', jsecnd

         if ( imass_mat>=1 ) then

!        --- Special case of time derivative containing also diffusive part

            if ( kchois==0 ) then
               if ( imass_mat==1 ) then

!              --- Only the diffusion part is added

                  ncoef = 16

               else

!              --- Also the convection part is added

                  ncoef = 17
                  jderiv = 1     ! we need to compute grad u

               end if  ! ( imass_mat==1 )

            end if  ! ( kchois==0 )

            if ( imas/=2 ) then

!           --- Error, mass matrix is not consistent

               call errint ( itype, 1 )
               call errint ( ielgrp, 2 )
               call errsub ( 117, 2, 0, 0 )

            end if  ! ( jheat<10 )

         end if  ! ( imass_mat==1 )

         ipos = ipos + 1
         ishift = ishift+2-isub

         if ( debug )
     +      write(irefwr,1) 'isub, ncoef, ipos', isub, ncoef, ipos
         do i = isub, ncoef
            call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
         end do
         if ( debug ) then
            call prinin ( ind, ncoef, 'ind' )
            call prinin ( ist, ncoef, 'ist' )
         end if  ! ( debug )

!        --- Set coefficient of mass matrix to mu for itype = 810

         if ( itype==810 .and. icheld==0 ) then

!        --- itype = 810, enthalpy equation by newton method
!            The density rho (coeff 12/17) is overwritten by the constant mu

            ind(12) = -1
            rho = array(1+11*m)
            cs  = array(1+12*m)
            cl  = array(1+13*m)
            mu = rho * min(cs, cl)
            do i = 1, m
               array(i+11*m) = mu
            end do

         else if ( ind(12)==0 .and. itype==800 .and.
     +             (icheld==24 .or. icheld==25) ) then

!        --- Type 800 compute shear for Reynolds equation
!            Set rho = 1

            ind(13)=-1
            do i = 1, m
               array(i+12*m) = 1d0
            end do

         end if  ! ( itype==810 .and. icheld==0 )

         if ( ierror/=0 ) go to 1000
         if ( itype==810 .and. icheld>=42 ) then

!        --- itype = 810, enthalpy equation by newton method

            iseqenth = iuser(ipos)
            ipos = ipos+1

         end if  ! ( itype==810 )

!        --- Test array lengths, number of nodes and number of degrees
!            of freedom

         call prinlg ( 'iuser', ipos-1, user, iuser )
         call prinlg ( 'user', -nuser, user, iuser )
         if ( icheld>=41 ) then

!        --- icheld>=41, We do not use coefficients 1 to 11

            ind(1:11) = 0

         end if  ! ( icheld==41 )

      end if  ! ( ncoef>0 )
      if ( ierror/=0 ) go to 1000

      if ( itype==810 .and. icheld==44 ) then

!     --- Special case, itype 810, icheld = 44, we use isolprec

         iseqtemp = 1
         iseqenth = 2

      end if  ! ( itype==810 .and. icheld==44 )

!     --- Next consider all terms carefully in order to get the cheapest
!         computation

      if ( kchois==0 .and. (matrix .or. metup>0) .or. kchois==3)
     +     then

!     --- matrix = true  The element matrix must be computed.
!         or metupw>0:  Upwinding
!                         Compute jsecnd, idim and jzero
!         First consider the diffusion tensor:

         if ( icheld>=40 ) then

!        --- icheld = 40, nothing to be done

         else if ( itype==800 .or. itype>=811 ) then

!        --- itype = 800

            call el1002 ( 0, 5, jwcons, jelp, iqmat, array, ndim )
            if ( itypln>=2 .and. itypln<=3 ) jderiv = 1

         else

!        --- itype = 803, set jsecnd

            jsecnd = 2
            jderiv = 1

         end if
         if ( kchois/=3 .and. itype/=803 ) then

!        --- Next consider the Convective terms:
!            compute idim and first component idim

            if ( itypln>3 ) then

!           --- Special case for 3d convection equation in R^2
!               only one component is used
!               The coordinate system is transformed such that this is
!               always the first direction along the surface

               kfirst = 1
               idim = 1

            else if ( itypln<2 ) then

!           --- Standard case

               call el1013 ( 6, 5, kfirst, jelp, iqmat, ndim )

            else

!           --- Non-linear convection

               kfirst = 1
               idim = ndim

            end if

!           --- Constant part of the equation:
!               compute jzero

            if ( ind(10)==0 ) then

!           --- beta = 0

               jzero = 0

            else if ( ind(10)<0 .or. ind(10)==2002 ) then

!           --- beta is constant (per element)

               jzero = jwcons

            else

!           --- General

               jzero = 2

            end if

         end if
         if ( debug )
     +      write(irefwr,1) 'jsecnd, kfirst, jzero, idim',
     +                       jsecnd, kfirst, jzero, idim

      end if
      if ( ierror/=0 ) go to 1000

      if ( kchois==0 .and. vector .or. kchois==2 ) then

!     --- Vector = true, the element vector must be computed
!         or kchois = 2, the first parameter must be computed
!         Compute jconsf

         if ( kchois==0 ) then

            icoef = 11

         else

            icoef = 1

         end if
         if ( ind(icoef)==0 ) then

!        --- F = 0, no right-hand side

            if ( (itype==800 .and. itypln>=2 .or.
     +            itypln==1) .and. itypln<4 ) notvec = 0

         else if ( ind(icoef)<0 .or. ind(icoef)==2002 ) then

!        --- F is constant (per element)

            jconsf = jwcons

         else

!        --- General

            jconsf = 2

         end if

      end if

      if ( jheat>0 ) then

!     --- jheat>0, i.e. compute mass matrix

         if ( jheat==1 .and. jelp==4 .and. irule==1 ) then

!        --- Error: jelp = 4 and diagonal mass matrix
!            In this case the mass amtrix is singular so not allowed

            call errint ( ishape, 1 )
            call errint ( irule, 2 )
            call errint ( jheat, 3 )
            call errwar ( 1541, 3, 0, 0 )

         end if  ! ( jheat==1 .and. jelp==4 )

         if ( ind(12)<0 .and. array(12*m)<=0 ) then

!        --- Error rho non-positive

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 771, 3, 0, 0 )

         else if ( ind(12)==0 ) then

!        --- Warning, rho = 0.
!            rho is made equal to 1

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errwar ( 771, 3, 0, 0 )
            ind(12) = -1
            do i = 1, n
               array(12*m-1+i) = 1d0
            end do
            jheat = jwcons + 10*(jheat-1)

         else if ( ind(12)<0 .or. ind(12)==2002 ) then

            jheat = jwcons + 10*(jheat-1)

         else

            jheat = 2 + 10*(jheat-1)

         end if

      end if
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,1) 'jheat', jheat

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      if ( matrix .and. kchois==0 .or. kchois>=2 ) then
         isubcf = isub
      else
         isubcf = 11
      end if

      do i = isubcf , ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190

      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(50)

190   do i = isub, isup
         ist(i) = 0
      end do
      ind(50) = 0
      funchk = .false.
      do i = isub, isup
         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(50) = 1
         if ( ind(i)>10000 ) funchk = .true.
      end do

      if ( icheli>1 .and. icheli/=7 .or. icheld>0 .and.
     +     icheld/=21 ) ind(50) = 1
      if ( icheld==41 ) then

!     --- Special case of enthalpy computation from temperature (type 810)

         ind(50) = 1

      else if ( icheld>=42 .and. icheld<=45 ) then

!     --- Special case of right-hand side for enthalpy computation
!         using newton (type 810)

         ind(50) = 2
         iphi = 1
         iweigh = 1

      else if ( itype==803 .or. mod(metup,20)>=7 .and.
     +     mod(metup,20)<=11 .or. itypln>=2 ) then

         ind(50) = 1

      else if ( kchois==0 .and. imass_mat==2 ) then

!     --- This option requires grad u and hence also u

         ind(50) = 1

      end if  ! ( icheld==41 )

      if ( icheld==0 .and. (itype==800 .or. itype>=811) .and.
     +     (mod(metup,20)>=7 .and. mod(metup,20)<=11) ) then
         jsol = 1
         jderiv = 1
      end if
      if ( funchk ) ind(50) = ind(50) + 2

      if ( jelp==4 .and. jcart==1 .and. metup==0 .and.
     +     icheld<=0 .and. irule==1 ) kstep = 2

      if ( ichtrn>0 ) then

!     --- ichtrn>0, special integral

         if ( jelp/=1 ) then
            call errint ( -icheld, 1 )
            call errint ( jelp, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 1863, 5, 0, 0 )
         else
            jtrans = jtrans + 100*ichtrn
         end if

      end if
      if (iqmat1==1) iweigh = iweigh + 20
      if ( igausp==1 .and. mod(iweigh,10)==0 ) iweigh = iweigh+1
1000  irule = irule + 100*interpol

      call erclos ( 'el0800' )
      if ( debug ) then

!     --- Debug information

         call prinin ( ind, ncoef, 'ind' )
         call prinin ( ist, ncoef, 'ist' )
         write(irefwr,1) 'isub, isup, igausp, m, n, iweigh, iphi',
     +                    isub, isup, igausp, m, n, iweigh, iphi
         write(irefwr,1) 'jsecnd', jsecnd
         write(irefwr,*) 'End el0800'

      end if  ! ( debug )

      end
