! ***********************************************************************
!
!      File:  exam8-1-4b.f
!
!      Contents:  Main program for the test example described
!                 in the SEPRAN manual standard problems 8-1-4
!                 Artificial analytical example
!                 This program shows a user written version
!                 Since a function subroutine is used for the solution,
!                 it is not possible to used sepcomp
!
!      Usage:     Compile and link this program with the SEPRAN libraries
!                 seplink exam8-1-4
!
!                 Run this program with input: exam8-1-4.prb
!
!                 exam8-1-4b < exam8-1-4b.prb > exam8-1-4b.out
!
!      version 1.0    date   24-07-96
!
! ***********************************************************************
!
!     --- Main program (written by a user)

      program exam814b
      use sepmodulecomio
      implicit none

      integer lengprob, lengmesh
      parameter ( lengprob=8000, lengmesh=1000)
      integer idum, kmeshsp(lengmesh), kprob(lengprob), intmat(5),
     +        iinbld(11),matr(5), isol(5),iuser(1000), iexact(5),
     +        iinvec(6), kmeshel(lengmesh), irhsdel(5), isolel(5),
     +        irhsdsp(5), iinstr(2), iinprs(1), iincommt(2), iincrt(1),
     +        iinfil(1)

      double precision user(100), error1, view(3)

      kmeshsp(1) = lengmesh
      kmeshel(1) = lengmesh
      kprob(1) = lengprob
      iuser(1) = 100
      user(1) = 100
      kmeshsp(3) = 0
      kmeshel(3) = 0
      kprob(3) = 0
      iuser(3) = 0
      user(3) = 0

!     --- start the program

      iinstr(1) = 2
      iinstr(2) = 1
      call sepstn ( kmeshsp, kprob, intmat, iinstr )

!     --- create finite element mesh from spectral mesh

      call femesh ( kmeshsp, kmeshel, 10, 1 )

!     --- plot the mesh

      view(1)=10d0
      view(2)=10d0
      view(3)=10d0
      call plotm3 ( 0, kmeshel, idum, 16d0, 1d0, view )

!     --- prescribe the boundary conditions

      iinprs(1) = 0
      call presbc ( kmeshsp, kprob, isolel, iinprs )

!     --- define the solution method for the finite element grid

      iincommt(1) = 2
      iincommt(2) = 1
      call matstruc ( iincommt, kmeshel, kprob, intmat )

!     --- create the exact solution

      iincrt(1) = 0
      call creatn ( iincrt, kmeshsp, kprob, iexact )

!     --- Fill information of coefficients in iuser and user

      iinfil(1) = 0
      call fillcf ( iuser, user, kprob, kmeshsp, iinfil )

!     --- build and solve the finite element system

      iinbld(1) = 0
      call build ( iinbld, matr, intmat, kmeshel, kprob, irhsdel, idum,
     +             isolel, isolel, iuser, user )

      call solve ( 1, matr, isolel, irhsdel, intmat, kprob )

!     --- build spectral right-hand side

      iinbld(1) = 4
      iinbld(2) = 2
      iinbld(3) = 1
      iinbld(4) = 0
      call build( iinbld, matr, intmat, kmeshsp, kprob, irhsdsp, idum,
     +            isolel, isolel, iuser, user )

!     --- solve spectral system using conjugate gradient solver
!         with finite element preconditioning

      call pcgrad ( isolel, isol, matr, kmeshsp, kprob, intmat, irhsdsp,
     +              iuser, user, 1d-15 )

!     --- calculate the error in the spectral solution

      iinvec(1) = 6
      iinvec(2) = 5
      iinvec(3) = 0
      iinvec(4) = 1
      iinvec(5) = 1
      iinvec(6) = 3
      call manvec( iinvec, error1, isol, iexact, idum, kmeshsp, kprob )
      write ( irefwr, * ) ' Error =', error1

!     --- stop the program

      call finish( 0 )
      end

!     --- function FUNCBC for essential boundary conditions

      double precision function funcbc(ichois,x,y,z)
      implicit none
      integer ichois
      double precision x,y,z
      funcbc = sin(x)*sin(y)*sin(z)
      end

!     --- function FUNC for exact solution

      double precision function func(ichois,x,y,z)
      implicit none
      integer ichois
      double precision x,y,z

      func = sin(x)*sin(y)*sin(z)
      end

!     --- function FUNCCF for right-hand side

      double precision function funccf(ichois,x,y,z)
      implicit none
      integer ichois
      double precision x,y,z

      funccf = 4.0d0*sin(x)*sin(y)*sin(z)
      end
