/*
 * This file replaces the formal Mrm UIL stuff, that caused too much
 * trouble on several platforms, made the thing largely unmaintainable
 * and dependent on a sepview.uil file being installed.
 * 
 * It allows the program to work fine with plain Motif 1.2 as well - which is
 * available on most systems - and also makes it possible to make a standalone
 * binary, independent of i.e. Sepran or Motif-2.0 being installed.
 * 
 * Kees Lemmens, June 1999
 */

#include <X11/Xlib.h>
#include <Xm/ArrowB.h>
#include <Xm/CascadeB.h>
#include <Xm/DrawingA.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/MainW.h>
#include <Xm/MainWP.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/Text.h>

#include "sepview.h"

/* Used callbacks (in sepview.c) : 
 * 
 * - wpop  ();
 * - unwpop();
 * - job   ();
 */

/* Simple routines to define fonts and colors
 * (which is as usual very cumbersome under Motif :-)
 */

XmFontList GetFont(char *fontstr,Display *dsp)
{
   XmFontListEntry entry;
   
   if( dsp != NULL)
   {
      entry = XmFontListEntryLoad(dsp,fontstr,XmFONT_IS_FONT,"fonz");
      
      return XmFontListAppendEntry(NULL,entry);
   }
   else
     return NULL;
}

/* The GetColor routine was taken from the SUN Motif Programmer guide : */

Pixel GetColor(char *colstr,Widget top)
{
   XrmValue from,to;

   from.size = strlen(colstr) + 1;
   if(from.size < sizeof(String))
     from.size = sizeof(String);

   from.addr = colstr;
   to.addr = NULL;
   XtConvert(top,XmRString, &from,XmRPixel,&to);
   if(to.addr != NULL)
     return (Pixel) *(Pixel *) to.addr;
   else
     return (XtArgVal) NULL;
}

void setNiceColors(Widget widgar[],Widget top)
{
   Pixel red = GetColor("red",top);
   
   XtVaSetValues(widgar[sepPreviousButton], XmNforeground, red, NULL);
   XtVaSetValues(widgar[sepNextButton]    , XmNforeground, red, NULL);   
   XtVaSetValues(widgar[sepQuitButton]    , XmNforeground, red, NULL);   
}

Widget createHelpMenu(Widget menuBar,Widget top)
{
   Widget helpButton,helpMenu,manButton,manPopup,manualText,helpCancelButton;
     
   helpButton = XtVaCreateManagedWidget
     ("helpButton",xmCascadeButtonWidgetClass, menuBar,
      
      XmNmnemonic        , 72, /* H:72 */
      XmNlabelString     , XmStringCreate("Help",XmSTRING_DEFAULT_CHARSET),
      NULL);
   
   mark(helpButton,sepHelp);
   
   helpMenu = XmCreatePulldownMenu(menuBar,"helpMenu",NULL,0);
   /* Connect menu to button : */  
   XtVaSetValues(helpButton, XmNsubMenuId, helpMenu, NULL);
   
   manButton = XtVaCreateManagedWidget
     ("manButton",xmPushButtonGadgetClass,helpMenu,
      XmNsubMenuId       , helpMenu,
      XmNlabelString     , XmStringCreateLocalized("Manual"),
      XmNacceleratorText , XmStringCreateLocalized("Alt+H"),
      XmNaccelerator     , "Alt <Key> h: wpop(sepManForm)",
      NULL);
      XtAddCallback(manButton,XmNactivateCallback,
		 (XtCallbackProc)wpop,(XtPointer)sepManForm);

   manPopup = XmCreateFormDialog(top,"manPopup",NULL,0);

   XtVaSetValues
     (manPopup,
      XmNdialogTitle  , XmStringCreateLocalized("Sepview manual page"),
      XmNautoUnmanage , FALSE,
      XmNwidth        , 670,
      XmNheight       , 650,
      NULL);

   mark(manPopup,sepManForm);
   
   manualText = XmCreateScrolledText(manPopup,"manualText",NULL,0);
   /* XtValues are set in displayMan() */
   mark(manualText,sepManText);
   XtManageChild(manualText);
   
   helpCancelButton = XtVaCreateManagedWidget
     ("helpCancelButton",xmPushButtonWidgetClass,manPopup,
      XmNbottomAttachment , XmATTACH_FORM, /* XmATTACH_WIDGET, */
      XmNbottomOffset     , MEDIUM_OFFSETV,
      XmNlabelString      , XmStringCreateLocalized("Cancel"),
      XmNleftOffset       , MEDIUM_OFFSET,
      XmNrightOffset      , MEDIUM_OFFSET,
      XmNleftAttachment   , XmATTACH_FORM,
      XmNrightAttachment  , XmATTACH_FORM,
      NULL);
   
   XtAddCallback(helpCancelButton,XmNactivateCallback,
		 (XtCallbackProc)unwpop,(XtPointer)sepManForm);

   return helpButton;
}

Widget createViewMenu(Widget menuBar,Widget top)
{
   Widget viewButton,viewMenu,colorSubMenu;
   Widget zoomOutButton,zoomResetButton,autoScaleButton,
     redrawButton,colorMenuButton,colorDefaultButton,colorFileButton;
   
   viewButton = XtVaCreateManagedWidget
     ("viewButton",xmCascadeButtonWidgetClass, menuBar,
      XmNmnemonic        , 86, /* V:86 */
      XmNlabelString     , XmStringCreateLocalized("View"),
      NULL);
   
   viewMenu = XmCreatePulldownMenu(menuBar,"viewMenu",NULL,0);
   /* Connect menu to button : */  
   XtVaSetValues(viewButton, XmNsubMenuId, viewMenu, NULL);
   
   zoomOutButton = XtVaCreateManagedWidget
     ("zoomOut",xmPushButtonGadgetClass,viewMenu,
      XmNsubMenuId   , viewMenu,
      XmNlabelString , XmStringCreateLocalized("Zoom Out"),
      XmNsensitive   , FALSE,
      XmNacceleratorText , XmStringCreateLocalized("Alt+Z"),
      XmNaccelerator , "Alt <Key> z: job(sepZoomOut)",
      NULL);
   
   mark(zoomOutButton,sepZoomOut);
   XtAddCallback(zoomOutButton,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepZoomOut);
   
   zoomResetButton = XtVaCreateManagedWidget
     ("zoomReset",xmPushButtonGadgetClass,viewMenu,
      XmNsubMenuId   , viewMenu,
      XmNlabelString , XmStringCreateLocalized("Zoom Reset"),
      /* XmNsensitive   , FALSE, */
      XmNacceleratorText , XmStringCreateLocalized("Ctrl+Z"),
      XmNaccelerator , "Ctrl <Key> z: job(sepZoomReset)",
      NULL);
   
   mark(zoomResetButton,sepZoomReset);
   XtAddCallback(zoomResetButton,XmNactivateCallback,
		  (XtCallbackProc)job,(XtPointer)sepZoomReset);
   
   autoScaleButton = XtVaCreateManagedWidget
     ("autoScale",xmPushButtonGadgetClass,viewMenu,
      XmNsubMenuId   , viewMenu,
      XmNlabelString , XmStringCreateLocalized("Autoscale"),
      NULL);

   XtAddCallback(autoScaleButton,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepAutoScale);
   
   redrawButton = XtVaCreateManagedWidget
     ("redraw",xmPushButtonGadgetClass,viewMenu,
      XmNsubMenuId   , viewMenu,
      XmNlabelString , XmStringCreateLocalized("Redraw"),
      XmNacceleratorText , XmStringCreateLocalized("Ctrl+R"),
      XmNaccelerator , "Ctrl <Key> r: job(sepDrawArea)",
      NULL);
   
   XtAddCallback(redrawButton,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepDrawArea);

   
   colorMenuButton = XtVaCreateManagedWidget
     ("colorMenu",xmCascadeButtonWidgetClass,viewMenu,
      XmNlabelString , XmStringCreateLocalized("Colors"),
      NULL);
   
   colorSubMenu = XmCreatePulldownMenu(viewMenu,"colorSubMenu",NULL,0);
   /* Connect submenu to button : */   
   XtVaSetValues(colorMenuButton, XmNsubMenuId, colorSubMenu, NULL);

   colorDefaultButton = XtVaCreateManagedWidget
     ("colorDefault",xmPushButtonWidgetClass,colorSubMenu,
      XmNlabelString , XmStringCreateLocalized("Default colors"),
      /* XmNsensitive   , FALSE, */
      XmNacceleratorText , XmStringCreateLocalized("Ctrl+D"),
      XmNaccelerator , "Ctrl <Key> d: job(sepColorDefault)",
      NULL);
   
   mark(colorDefaultButton,sepColorDefault);
   XtAddCallback(colorDefaultButton,XmNactivateCallback,
		 (XtCallbackProc)wpop,(XtPointer)sepColorDefault);

   colorFileButton = XtVaCreateManagedWidget
     ("colorFile",xmPushButtonWidgetClass,colorSubMenu,
      XmNlabelString , XmStringCreateLocalized("Colorfile..."),
      XmNacceleratorText , XmStringCreateLocalized("Ctrl+C"),
      XmNaccelerator , "Ctrl <Key> c: wpop(sepColorOpen)",
      NULL);
   
   XtAddCallback(colorFileButton,XmNactivateCallback,
		 (XtCallbackProc)wpop,(XtPointer)sepColorOpen);
       
   return viewButton;
}

Widget createExportMenu(Widget menuBar,Widget top)
{
   Widget exportButton,exportMenu, gifOutputButton, gifSubMenu,
     ppmOutputButton, ppmSubMenu;
   Widget gifThisFrame,gifAllFrames,ppmThisFrame,ppmAllFrames;
   Widget psOutputButton, hpglOutputButton, figOutputButton;
   
   exportButton = XtVaCreateManagedWidget
     ("exportButton",xmCascadeButtonWidgetClass, menuBar,
      XmNmnemonic        , 69, /* 'E' */
      XmNlabelString     , XmStringCreateLocalized("Export"),
      NULL);

   exportMenu = XmCreatePulldownMenu(menuBar,"exportMenu",NULL,0);
   /* Connect menu to button : */  
   XtVaSetValues(exportButton, XmNsubMenuId, exportMenu, NULL);

   psOutputButton = XtVaCreateManagedWidget
     ("psOutput",xmPushButtonWidgetClass,exportMenu,
      XmNlabelString , XmStringCreateLocalized("Postscript"),
      XmNacceleratorText , XmStringCreateLocalized("Ctrl+P"),
      XmNaccelerator , "Ctrl <Key> p: job(sepPsOutputButton)",
      NULL);

   mark(psOutputButton,sepPsOutputButton);
   XtAddCallback(psOutputButton,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepPsOutputButton);
   
   
   hpglOutputButton = XtVaCreateManagedWidget
     ("hpglOutput",xmPushButtonWidgetClass,exportMenu,
      XmNlabelString , XmStringCreateLocalized("HPGL"),
      XmNacceleratorText , XmStringCreateLocalized("Ctrl+H"),
      XmNaccelerator , "Ctrl <Key> h: job(sepHpglOutputButton)",
      NULL);

   mark(hpglOutputButton,sepHpglOutputButton);
   XtAddCallback(hpglOutputButton,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepHpglOutputButton);

   
   figOutputButton = XtVaCreateManagedWidget
     ("figOutput",xmPushButtonWidgetClass,exportMenu,
      XmNlabelString , XmStringCreateLocalized("Fig"),
      NULL);
   
   mark(figOutputButton,sepFigOutputButton);
   XtAddCallback(figOutputButton,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepFigOutputButton);
   
    
   gifOutputButton = XtVaCreateManagedWidget
     ("gifOutput",xmCascadeButtonWidgetClass,exportMenu,
      XmNlabelString , XmStringCreateLocalized("GIF"),
      NULL);

    gifSubMenu = XmCreatePulldownMenu(exportMenu,"gifSubMenu",NULL,0);
   /* Connect submenu to button : */   
   XtVaSetValues(gifOutputButton, XmNsubMenuId, gifSubMenu, NULL);
 
 
   gifThisFrame = XtVaCreateManagedWidget
     ("gifThisFrame",xmPushButtonGadgetClass,gifSubMenu,
      XmNsubMenuId   , gifSubMenu,
      XmNlabelString , XmStringCreateLocalized("This frame"),
      NULL);
   mark(gifThisFrame,sepGIFOutputButton);
   XtAddCallback(gifThisFrame,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepGIFOutputButton);

   gifAllFrames = XtVaCreateManagedWidget
     ("gifAllFrames",xmPushButtonGadgetClass,gifSubMenu,
      XmNsubMenuId   , gifSubMenu,
      XmNlabelString , XmStringCreateLocalized("All frames"),
      NULL);
   mark(gifAllFrames,sepGIFOutputall);
   XtAddCallback(gifAllFrames,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepGIFOutputall);


   ppmOutputButton = XtVaCreateManagedWidget
     ("ppmOutput",xmCascadeButtonWidgetClass,exportMenu,
      XmNlabelString , XmStringCreateLocalized("PPM"),
      NULL);

    ppmSubMenu = XmCreatePulldownMenu(exportMenu,"ppmSubMenu",NULL,0);
   /* Connect submenu to button : */   
   XtVaSetValues(ppmOutputButton, XmNsubMenuId, ppmSubMenu, NULL);
 
 
   ppmThisFrame = XtVaCreateManagedWidget
     ("ppmThisFrame",xmPushButtonGadgetClass,ppmSubMenu,
      XmNsubMenuId   , ppmSubMenu,
      XmNlabelString , XmStringCreateLocalized("This frame"),
      NULL);
   mark(ppmThisFrame,sepPPMOutputButton);
   XtAddCallback(ppmThisFrame,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepPPMOutputButton);

   ppmAllFrames = XtVaCreateManagedWidget
     ("ppmAllFrames",xmPushButtonGadgetClass,ppmSubMenu,
      XmNsubMenuId   , ppmSubMenu,
      XmNlabelString , XmStringCreateLocalized("All frames"),
      NULL);
   mark(ppmAllFrames,sepPPMOutputall);
   XtAddCallback(ppmAllFrames,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepPPMOutputall);

   return exportButton;
}

Widget createFileMenu(Widget menuBar,Widget top)
{
   Widget fileButton,fileMenu,quitSubButton,quitSubMenu;
   Widget fileOpenButton,fileQuitButton;
   
   fileButton = XtVaCreateManagedWidget
     ("fileButton",xmCascadeButtonWidgetClass, menuBar,
      XmNmnemonic    , 70, /* 'F' */
      XmNlabelString     , XmStringCreateLocalized("File"),
      NULL);
   
   fileMenu = XmCreatePulldownMenu(menuBar,"fileMenu",NULL,0);
   /* Connect menu to button : */
   XtVaSetValues(fileButton, XmNsubMenuId, fileMenu, NULL);
   
   fileOpenButton = XtVaCreateManagedWidget
     ("fileOpen",xmPushButtonGadgetClass,fileMenu,
      XmNsubMenuId   , fileMenu,
      XmNlabelString , XmStringCreateLocalized("Select..."),
      XmNacceleratorText , XmStringCreateLocalized("Ctrl+F"),
      XmNaccelerator , "Ctrl <Key> f: wpop(sepFileOpen)",
      NULL);
   
   XtAddCallback(fileOpenButton,XmNactivateCallback,
		 (XtCallbackProc)wpop,(XtPointer)sepFileOpen);

   quitSubButton = XtVaCreateManagedWidget
     ("quitSubButton",xmCascadeButtonWidgetClass,fileMenu,
      XmNlabelString , XmStringCreateLocalized("Quit"),
      NULL);

   quitSubMenu = XmCreatePulldownMenu(fileMenu,"quitSubMenu",NULL,0);
   /* Connect submenu to button : */   
   XtVaSetValues(quitSubButton, XmNsubMenuId, quitSubMenu, NULL);

   fileQuitButton = XtVaCreateManagedWidget
     ("fileQuit",xmPushButtonWidgetClass,quitSubMenu,
      XmNsubMenuId   , quitSubMenu,
      XmNlabelString , XmStringCreateLocalized("  OK  "),
      NULL);
   
   mark(fileQuitButton,sepQuitButton);
   
   XtAddCallback(fileQuitButton,XmNactivateCallback,
		 (XtCallbackProc)unwpop,(XtPointer)sepMainWindow);
   
   return fileButton;
}

Widget createDrawFrame(Widget workForm)
{
   Widget drawFrame,drawArea;
   
   drawFrame =XtVaCreateManagedWidget
     ("drawFrame",xmFrameWidgetClass,workForm,
      NULL);
   XtVaSetValues
     (drawFrame,
      XmNtopAttachment    , XmATTACH_FORM,
      XmNbottomAttachment , XmATTACH_FORM,
      XmNbottomOffset     , 35,
      XmNleftAttachment   , XmATTACH_FORM,
      XmNrightAttachment  , XmATTACH_FORM,
      NULL);
   
   drawArea = XmCreateDrawingArea(drawFrame,"drawArea",NULL,0);
   XtVaSetValues
     (drawArea,
      XmNresizePolicy     ,XmRESIZE_ANY,
      NULL);
   
   mark(drawArea,sepDrawArea);
   XtAddCallback(drawArea,XmNexposeCallback,
		  (XtCallbackProc)job,(XtPointer)sepDrawArea);
   XtAddCallback(drawArea,XmNresizeCallback,
		  (XtCallbackProc)job,(XtPointer)sepDrawArea);
   XtManageChild(drawArea);

   return drawFrame;
}

Widget createVersionControl(Widget mainWindow)
{
   Widget niceForm,niceFrame,versionSelect;
   Widget previousButton,playButton,nextButton;

   niceForm = XtVaCreateManagedWidget
     ("niceForm",xmFormWidgetClass,mainWindow,
      NULL);
   
   niceFrame = XtVaCreateManagedWidget
     ("niceForm",xmFrameWidgetClass,niceForm,
      XmNshadowType      , XmSHADOW_OUT,
      XmNshadowThickness , 2,
      NULL);

   versionSelect = XtVaCreateManagedWidget
     ("versionSelect",xmRowColumnWidgetClass,niceFrame,
      XmNorientation     , XmHORIZONTAL,
      XmNisHomogeneous   , FALSE,
      NULL);
   
   previousButton = XtVaCreateManagedWidget
     ("previousButton",xmArrowButtonWidgetClass,versionSelect,
      XmNarrowDirection   , XmARROW_LEFT,
      XmNwidth            , 60,
      XmNsensitive        , FALSE,
      XmNshadowThickness  , 0,
      NULL);
   mark(previousButton,sepPreviousButton);
   XtAddCallback(previousButton,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepPreviousButton);

   
   playButton = XtVaCreateManagedWidget
     ("playButton",xmPushButtonWidgetClass,versionSelect,
      XmNwidth            , 60,
      XmNlabelString      , XmStringCreate("PLAY",XmSTRING_DEFAULT_CHARSET),
      XmNsensitive        , FALSE,
      NULL);
   mark(playButton,sepPlayButton);
   XtAddCallback(playButton,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepPlayButton);
   
   
   nextButton = XtVaCreateManagedWidget
     ("nextButton",xmArrowButtonWidgetClass,versionSelect,
      XmNarrowDirection   , XmARROW_RIGHT,
      XmNwidth            , 60,
      XmNsensitive        , FALSE,
      XmNshadowThickness  , 0,
      NULL);
   mark(nextButton,sepNextButton);
   XtAddCallback(nextButton,XmNactivateCallback,
		 (XtCallbackProc)job,(XtPointer)sepNextButton);

   return niceForm;
}
  
Widget createStatusForm(Widget mainWindow)
{
   Widget statusForm,filelabel,colorlabel,niceForm;
   
   statusForm = XtVaCreateManagedWidget
     ("statusForm",xmFormWidgetClass,mainWindow,
      XmNrightAttachment  , XmATTACH_FORM,
      XmNleftAttachment   , XmATTACH_FORM,
      XmNheight           , 30,
      XmNbottomAttachment , XmATTACH_FORM,
      NULL);

   colorlabel = XtVaCreateManagedWidget
     ("colorlabel",xmLabelWidgetClass,statusForm,
      XmNrightAttachment  , XmATTACH_FORM,
      XmNtopAttachment    , XmATTACH_FORM,
      XmNbottomAttachment , XmATTACH_FORM,
      XmNalignment        , XmALIGNMENT_CENTER,
      XmNmarginWidth      , 10,
      XmNlabelString      , XmStringCreateLocalized("Colors:"),
/*      XmNfontList         , GetFont("*-italic-12",XtDisplay(mainWindow)),
*/      XmNrightOffset      , NICESPACE,
      NULL);
   
   mark(colorlabel,sepColorStatus);

   filelabel = XtVaCreateManagedWidget
     ("filelabel",xmLabelWidgetClass,statusForm,
      
      XmNleftAttachment   , XmATTACH_FORM,
      XmNtopAttachment    , XmATTACH_FORM,
      XmNbottomAttachment , XmATTACH_FORM,
      XmNalignment        , XmALIGNMENT_CENTER,
      XmNmarginWidth      , 10,
      XmNlabelString      , XmStringCreateLocalized("DataFile:"),
/*      XmNfontList         , GetFont("*-italic-12",XtDisplay(mainWindow)),
*/      XmNleftOffset       , NICESPACE,
      NULL);
   
   mark(filelabel,sepStatus);

   niceForm   = createVersionControl(statusForm);
   XtVaSetValues
     (niceForm,
      XmNbottomAttachment , XmATTACH_FORM,
      XmNleftAttachment   , XmATTACH_POSITION,
      XmNleftPosition     , 35,
      XmNalignment        , XmALIGNMENT_BEGINNING,
      NULL);

   return statusForm;
}

void createAllDialogs(Widget top)
{
   Widget errorMessage,infoMessage;
   Widget fileOpen,colorOpen;
   
   fileOpen = XmCreateFileSelectionDialog(top,"fileOpen",NULL,0);
   XtVaSetValues
     (fileOpen,
      XmNdialogTitle  ,XmStringCreateLocalized("Sepview - choose your data file"),
      XmNdirMask           , XmStringCreateLocalized("*plot*[0-9]"),
      /* Workaround for Xm 1.2 fileselector bug : */
      /* XmNpattern           , XmStringCreateLocalized("*plot*[0-9]"), */
      XmNfilterLabelString , XmStringCreateLocalized("Sepran data files"),
      XmNmustMatch         , TRUE,
      NULL);
   mark(fileOpen,sepFileOpen);
   XtAddCallback(fileOpen,XmNokCallback,
		 (XtCallbackProc)unwpop,(XtPointer)sepFileOpen);
   XtAddCallback(fileOpen,XmNcancelCallback,
		 (XtCallbackProc)unwpop,(XtPointer)sepNoFileOpen);


   colorOpen = XmCreateFileSelectionDialog(top,"colorOpen",NULL,0);
   XtVaSetValues
     (colorOpen,
      XmNdialogTitle  , XmStringCreateLocalized("Sepview - choose your color file"),
      XmNdirMask           , XmStringCreateLocalized("*col"),
      XmNfilterLabelString , XmStringCreateLocalized("Sepran color files"),
      XmNmustMatch         , TRUE,
      NULL);
   mark(colorOpen,sepColorOpen);
   XtAddCallback(colorOpen,XmNokCallback,
		 (XtCallbackProc)unwpop,(XtPointer)sepColorOpen);
   XtAddCallback(colorOpen,XmNcancelCallback,
		 (XtCallbackProc)unwpop,(XtPointer)sepNoColorOpen);

   
   errorMessage = XmCreateErrorDialog(top,"errorMessage",NULL,0);
   XtVaSetValues
     (errorMessage,
      XmNdialogTitle    , XmStringCreateLocalized("Sepview error message"),
      XmNdialogStyle    , XmDIALOG_FULL_APPLICATION_MODAL,
      XmNnoResize       , TRUE,
      XmNmessageString  , XmStringCreateLocalized("hi folks"),
      NULL);
   mark(errorMessage,sepErrorMessage);
   
   
   infoMessage =  XmCreateInformationDialog(top,"infoMessage",NULL,0);
   XtVaSetValues
     (infoMessage,
      XmNdialogTitle    , XmStringCreateLocalized("Sepview info message"),
      XmNdialogStyle    , XmDIALOG_FULL_APPLICATION_MODAL,
      XmNnoResize       , TRUE,
      XmNmessageString  , XmStringCreateLocalized("hi folks"),
      NULL);
   mark(infoMessage,sepInfoMessage);
}

void createAllWidgets(Widget top)
{
   Widget mainWindow,menuBar,workForm,statusForm,drawFrame;
   Widget File,Export,View,Help;
   
   mainWindow = XtVaCreateManagedWidget
     ("mainWindow",xmMainWindowWidgetClass,top,
      XmNheight, 600,
      XmNwidth , 600,
      NULL);
   mark(mainWindow,sepMainWindow);
    
   /* Start Menu Bar */
   menuBar = XmCreateMenuBar(mainWindow,"menuBar",NULL,0);
   
   File   = createFileMenu  (menuBar,top);
   Export = createExportMenu(menuBar,top);
   View   = createViewMenu  (menuBar,top);
   Help   = createHelpMenu  (menuBar,top);
   
   XtVaSetValues(menuBar,
		 XmNorientation     , XmHORIZONTAL,
		 XmNspacing         , 15,
		 XmNtopAttachment   , XmATTACH_FORM,
		 XmNrightAttachment , XmATTACH_FORM,
		 XmNleftAttachment  , XmATTACH_FORM,
		 XmNmenuHelpWidget  , Help,
		 NULL);
   mark(menuBar,sepMenuBar);
   XtManageChild(menuBar); /* must be done separately with a menubar ... */
   /* End Menu Bar */


   workForm =XtVaCreateManagedWidget
     ("workForm",xmFormWidgetClass,mainWindow,
      NULL);
   XtVaSetValues
     (workForm,
      XmNtopAttachment    , XmATTACH_WIDGET,
      XmNtopWidget        , menuBar,
      XmNbottomAttachment , XmATTACH_FORM,
      XmNleftAttachment   , XmATTACH_FORM,
      XmNrightAttachment  , XmATTACH_FORM,
      NULL);
   
   /* niceForm   = createVersionControl(mainWindow); */
   drawFrame  = createDrawFrame(workForm);
   statusForm = createStatusForm(workForm);
   
   /*KL: found this in the SUN manual, without it the canvas wont show !! */
   XtVaSetValues(mainWindow, XmNworkWindow, workForm,NULL);
   
   createAllDialogs(top);
}

