      subroutine sepdifiar ( intmt1, intmt2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the difference of the matrix structures stored in
!     intmt1 and intmt2
!     Old subroutine difiar
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemain
      use sepmodulekprob
      use sepmodulematstr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: intmt1, intmt2

!     intmt1         i    first array intmat
!     intmt2         i    second array intmat
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k
      integer, pointer :: irowsii2(:), icolsii2(:)
      type (intmatrix), pointer :: IM1, IM2

!     IM1            Refers to intmt(intmt1)
!     IM2            Refers to intmt(intmt2)
!     icolsii2       array icolsii corresponding to intmt2
!     irowsii2       array irowsii corresponding to intmt2
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INCMAR         Compare contents of array INTMT1 and INTMT2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1232   INTMAT1 and INTMAT2 correspond to different problems
!    3098   Value of parameter not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepdifiar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepdifiar'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM1 => intmt(intmt1)
      IM2 => intmt(intmt2)
      if ( IM1%symmetric ) then
         call errint ( IM1%jmethod, 1 )
         call errsub ( 3098, 1, 0, 0 )
      end if  ! ( IM1%jmethod/=9 )
      if ( IM2%symmetric ) then
         call errint ( IM2%jmethod, 1 )
         call errsub ( 3098, 1, 0, 0 )
      end if  ! ( IM2%jmethod/=9 )

      iprob = IM1%iprob
      iseqkp = IM1%iseqkp
      kprobloc => kp(iseqkp)%kprobsub(iprob)%kprobloc
      if ( IM2%iprob/=iprob ) then
         call errint ( iprob, 1 )
         call errint ( IM2%iprob, 2 )
         call errsub ( 1232, 2, 0, 0 )
      end if
      if ( ierror/= 0 ) go to 1000

!     --- Compute constants

      nbound = kprobloc(10) + kprobloc(34)
      nrusol = kprobloc(66)
      irowsii => IM1%rowsii
      icolsii => IM1%colsii
      irowsii2 => IM2%rowsii
      icolsii2 => IM2%colsii

!     --- Compare first part of arrays INTMT1 and INTMT2

      call incmar ( nrusol, irowsii, icolsii, irowsii2, icolsii2 , 0, &
                    ' INTMAT part 1 (part interal unknowns)' )
      if ( nbound>0 ) then

!     --- NBOUND>0

         irowsip => IM1%rowsip
         icolsip => IM1%colsip
         irowsii2 => IM2%rowsip
         icolsii2 => IM2%colsip

!        --- Compare second part of arrays INTMT1 and INTMT2

         call incmar ( nbound, irowsip, icolsip, irowsii2, icolsii2, nrusol, &
                       ' INTMAT part 2 (part boundary conditions)' )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepdifiar'

      end if  ! ( debug )
      call erclos ( 'sepdifiar' )

      end subroutine sepdifiar
