      subroutine sepdifrar ( matr1, matr2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the difference of the matrices referred to by matr1/2
!     Old subroutine difrar
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmoduleintmat
      use sepmodulemain
      use sepmodulekprob
      use sepmodulematstr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: matr1, matr2

!     matr1          i    first array matr
!     matr2          i    second array matr
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, intmatref
      double precision :: amin, amax, eps
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     amax           Maximum value in matrix
!     amin           Minimum value in matrix
!     eps            Local accuracy
!     intmatref      refers to array intmat corresponding to matrices
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCMRR         Compare two matrices with one another
!     INCMVC         Compare two vectors with one another
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepdifrar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepdifrar'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check input

      intmatref = realmt(matr1)%intmatref
      IM => intmt(intmatref)
      iprob = intmt(intmatref)%iprob
      iseqkp = IM%iseqkp
      if ( ierror/= 0 ) go to 1000

!     --- Compute constants

      kprobloc => kp(iseqkp)%kprobsub(iprob)%kprobloc
      nbound = kprobloc(10) + kprobloc(34)
      nrusol = kprobloc(66)

!     --- Activate arrays

      S1_ii => realmt(matr1)%S_ii
      S2_ii => realmt(matr2)%S_ii


!     --- Compare first part of arrays MATR1 and MATR2 (diagonal)
!           First compute minimum and maximum value of diagonal

      amin = minval(S1_ii(1:nrusol))
      amax = maxval(S1_ii(1:nrusol))
      eps = max ( abs(amin), abs(amax) ) * epsmac * 1d2
      call incmvc ( nrusol, S1_ii, S2_ii, eps, 'diagonal of MATRIX' )

!     --- Compare rest of arrays MATR1 and MATR2 (eps is not changed)

      call incmrr ( nrusol, IM%rowsii, IM%colsii, &
                    S1_ii(1+nrusol), S2_ii(1+nrusol), 0, eps, &
                    ' MATRIX part 1 (part internal unknowns)' )
      if ( nbound>0 ) then

!     --- NBOUND>0

         S1_ii => realmt(matr1)%S_ip
         S2_ii => realmt(matr2)%S_ip
         irowsip => IM%rowsip
         icolsip => IM%colsip

!        --- Compare second part of matrices MATR1 and MATR2

         call incmrr ( nbound, irowsip, icolsip, &
                       S1_ii, S2_ii, nrusol, eps, &
                       ' MATRIX part 2 (part boundary conditions)' )

         S1_ii => realmt(matr1)%S_pi
         S2_ii => realmt(matr2)%S_pi

!        --- Compare second part of matrices MATR1 and MATR2

         call incmrr ( nbound, irowsip, icolsip, &
                       S1_ii, S2_ii, nrusol, eps, &
                       ' MATRIX part 2 (part boundary conditions)' )

         S1_ii => realmt(matr1)%S_pp
         S2_ii => realmt(matr2)%S_pp

!        --- Compare second part of matrices MATR1 and MATR2

         call incmrr ( nbound, irowsip, icolsip, &
                       S1_ii, S2_ii, nrusol, eps, &
                       ' MATRIX part 2 (part boundary conditions)' )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepdifrar'

      end if  ! ( debug )
      call erclos ( 'sepdifrar' )

      end subroutine sepdifrar
