      subroutine seppetscmonitor ( sepksp, iter, rnorm, dummy,
     .                             ierrpetsc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-08-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to monitor convergence of iterative method
!     petsc only
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
!#include <petsc/finclude/petscsysdef.h>
!#include <petsc/finclude/petscvecdef.h>
!#include <petsc/finclude/petsckspdef.h>
#include <petsc/finclude/petscksp.h>
      use sepmodulecomio
      use petscsys
      use petscvec
      use petscksp
      implicit none


! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      KSP              sepksp
      integer          iter, dummy, ierrpetsc
      double precision rnorm
      
!     dummy               Dummy parameter
!     ierrpetsc      i    Error parameter
!     iter           i    Iteration sequence number
!     sepksp         i    solver context
!     rnorm          i    Norm of residual
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppetscmonitor' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppetscmonitor'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,100) iter+1, rnorm
100   format ( i7, 6x, ss, 1pe10.2e2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppetscmonitor'

      end if  ! ( debug )
      call erclos ( 'seppetscmonitor' )

      end
