      subroutine seppetscsol3 ( reason )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-08-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print error message because solver has stopped iterating
!
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     linear_solver
!     error_manipulation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: reason

!     reason         i    Reason why KSP stopped the iteration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3192   Petsc solver stopped converging because KSP_DIVERGED_NULL
!    3193   Petsc solver stopped converging because KSP_DIVERGED_ITS
!    3194   Petsc solver stopped converging because KSP_DIVERGED_DTOL
!    3195   Petsc solver stopped converging because KSP_DIVERGED_BREAKDOWN
!    3196   Petsc solver stopped converging because KSP_DIVERGED_BREAKDOWN_BICG
!    3197   Petsc solver stopped converging because KSP_DIVERGED_NONSYMMETRIC
!    3198   Petsc solver stopped converging because KSP_DIVERGED_INDEFINITE_PC
!    3199   Petsc solver stopped converging because KSP_DIVERGED_NANORINF
!    3200   Petsc solver stopped converging because KSP_DIVERGED_INDEFINITE_MAT
!    3201   Petsc solver stopped converging
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppetscsol3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppetscsol3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( reason<=-2 .and. reason>=-10 ) then

!     --- -10 <= reason <= -2, standard error

         call errsub ( 3190-reason, 0, 0, 0 )

      else

!     --- reason = -1 or < -10, error not known

         call errint ( reason, 1 )
         call errsub ( 3201, 1, 0, 0 )

      end if  ! ( reason<=-2 .and. reason>=-10 )

1000  call erclos ( 'seppetscsol3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppetscsol3'

      end if  ! ( debug )

      end subroutine seppetscsol3
