      subroutine topq10 ( kmesh, nelem )

! =======================================================================
!
!
!         programmer   Niek Praagman
!
!         version 1.0  date 31-03-1993
!
!
!   copyright (c) 1993,1993  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Compute number of 3-point triangles or 4-point tetrahedrons of mesh
!     to be created if function has been given on elements (i.e. type 116)
!
! ***********************************************************************
!
!     KEYWORDS
!
!     3d
!     2d
!     subdivision
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer kmesh(*), nelem

!     kmesh     i   standard SEPRAN mesharray
!
!     nelem     o   number of elements to be created in new mesh
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer nelgrp, inelem, kelmf, kelmg, ielgrp, meshtp, ndim
!
!     ielgrp     loop variable for element groups
!
!     inelem     number of elements in group
!
!     kelmf      pointer to kmesh part f
!
!     kelmg      pointer to kmesh part g
!
!     meshtp     type of element (SEPRAN)
!
!     ndim       dimension
!
!     nelgrp     number of element groups
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     Run through all groups
!
!     Compute number of elements of group
!
!     Determine number of new triangles or tetrahedrons
!
!     Add to total number
!
! =======================================================================

!     Determine the needed values

      ndim   = kmesh(6)
      nelgrp = kmesh(5)
      inelem = kmesh(9)
      kelmf  = kmesh(20)
      kelmg  = kmesh(21)

      nelem    =  0

!     Run through all the groups

      do 1000 ielgrp = 1 , nelgrp

!         Determine type of element of group ielgrp

          meshtp = kmesh( kelmf + ielgrp - 1 )

          if ( nelgrp.gt.1 ) inelem = kmesh(kelmg+ielgrp-1)

!         Compute dependent on ndim

          if ( ndim.eq.2 ) then

             if ( meshtp.eq.3 ) then

!              triangle with three nodes

               nelem = nelem + inelem

             else if ( meshtp.eq.4 .or. meshtp.eq.7 ) then

!              triangle with six or seven nodes

               nelem = nelem + inelem

             else if ( meshtp.eq.5 ) then

!              quadrangle with four nodes

               nelem = nelem + 2 * inelem

             else if ( meshtp.eq.6 ) then

!              quadrangle with nine nodes

               nelem = nelem + 2 * inelem

             end if

          else if ( ndim.eq.3 ) then

             if ( meshtp.eq.11 .or. meshtp.eq.12 ) then

!              tetrahedron with four or ten nodes

               nelem = nelem + inelem

             else if ( meshtp.eq.15 .or. meshtp.eq.16 ) then

!              tetrahedron with 14 or 15 nodes

               nelem = nelem + inelem

             else if ( meshtp.eq.13 .or. meshtp.eq.14 ) then

!              hexahedron with 8 or 27 nodes

               nelem = nelem + 6 * inelem

             end if

          end if

1000  continue

      end
