      subroutine topq11 ( kmesh, kmeshc, kelem, ivul, ihulp, iorig )

! =======================================================================
!
!
!         programmer   Niek Praagman
!
!         version 1.0  date 31-03-1993
!
!
!   copyright (c) 1993,1993  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Fill new helparray kelem of 3-point triangles or 4-point tetrahe- &
!     drons if function has been given on elements (type 116)
!
! ***********************************************************************
!
!     KEYWORDS
!
!     2d
!     3d
!     subdivision
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer kmesh(*), kmeshc(*), kelem(*), ivul, ihulp(*), iorig(*)

!     ihulp     o   helparray for originating element group numbers
!
!     iorig     o   helparray with the original element numbers
!
!     ivul      i   indicator whether ihulp has to be filled or not
!
!     kelem     o   new element array with linear triangles or
!                   tetrahedrons only
!
!     kmesh     i   Standard SEPRAN array
!
!     kmeshc    i   KMESH part c : the elements
!
! ***********************************************************************
!
!     LOCAL PARAMETERS

      integer i, nelgrp, kelma, inpelm, inelem, kelmf, kelmg, ipl, &
              ioelem, ielold, ikelem, ielgrp, meshtp, iel, ndim

!     i         loop variable
!
!     iel       loop variable elements
!
!     ielgrp    loop variable element groups
!
!     ielold    old element counter
!
!     ikelem    pointer in new array kelem
!
!     inelem    number of elements of group
!
!     inpelm    number of points in element
!
!     ioelem    new element counter
!
!     ipl       pointer in old element array kmeshc
!
!     kelma     pointer to kmesh part a
!
!     kelmf     pointer to kmesh part f
!
!     kelmg     pointer to kmesh part g
!
!     meshtp    type of element (SEPRAN)
!
!     ndim      dimension
!
!     nelgrp    number of element groups in mesh
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     Run through all groups and divide all old elements in either
!     triangles (2d) or tetrahedrons (3d)
!
! =======================================================================

!     Set the variables

      ndim   = kmesh(6)
      nelgrp = kmesh(5)
      kelma  = kmesh(15)
      inpelm = kmesh(4)
      inelem = kmesh(9)
      kelmf  = kmesh(20)
      kelmg  = kmesh(21)

      ipl      =  0
      ikelem   =  0
      ioelem   =  0
      ielold   =  0
      if ( ivul.eq.1 ) ihulp(1) = 0

!     Run through all the groups

      do 1000 ielgrp = 1 , nelgrp

!         Determine type of element of group ielgrp

          meshtp = kmesh( kelmf + ielgrp - 1 )

!         Number of points in element

          if ( kelma.ne.0 ) inpelm=kmesh(kelma+ielgrp)- &
               kmesh(kelma+ielgrp-1)

          if ( nelgrp.gt.1 ) inelem=kmesh(kelmg+ielgrp-1)

          if ( ndim.eq.2 ) then

             if ( meshtp.lt.3 .or. meshtp.gt.7 ) then

!              Group not interesting, skip and goto end

               ipl = ipl + inpelm * inelem
               ielold = ielold + inelem

             else

!              Run through all elements and divide

               do 200 iel = 1 , inelem

                   ielold = ielold + 1

                   if ( meshtp.eq.3 ) then

!                     triangle with three nodes

                      kelem( ikelem+1 ) = kmeshc( ipl+1 )
                      kelem( ikelem+2 ) = kmeshc( ipl+2 )
                      kelem( ikelem+3 ) = kmeshc( ipl+3 )

                      ikelem = ikelem + 3
                      iorig(ioelem+1) = ielold
                      ioelem = ioelem + 1

                   else if ( meshtp.eq.4 .or. meshtp.eq.7 ) then

!                     triangle with six or seven nodes

                      kelem( ikelem+1 ) = kmeshc( ipl+1 )
                      kelem( ikelem+2 ) = kmeshc( ipl+3 )
                      kelem( ikelem+3 ) = kmeshc( ipl+5 )

                      ikelem = ikelem + 3
                      iorig(ioelem+1) = ielold
                      ioelem = ioelem + 1

                   else if ( meshtp.eq.5 ) then

!                     quadrangle with four nodes

                      kelem( ikelem+1 ) = kmeshc( ipl+1 )
                      kelem( ikelem+2 ) = kmeshc( ipl+2 )
                      kelem( ikelem+3 ) = kmeshc( ipl+3 )

                      kelem( ikelem+4 ) = kmeshc( ipl+1 )
                      kelem( ikelem+5 ) = kmeshc( ipl+3 )
                      kelem( ikelem+6 ) = kmeshc( ipl+4 )

                      ikelem = ikelem + 6
                      iorig(ioelem+1) = ielold
                      iorig(ioelem+2) = ielold
                      ioelem = ioelem + 2

                   else if ( meshtp.eq.6 ) then

!                     quadrangle with nine nodes

                      kelem( ikelem+1 ) = kmeshc( ipl+1 )
                      kelem( ikelem+2 ) = kmeshc( ipl+3 )
                      kelem( ikelem+3 ) = kmeshc( ipl+5 )

                      kelem( ikelem+4 ) = kmeshc( ipl+1 )
                      kelem( ikelem+5 ) = kmeshc( ipl+5 )
                      kelem( ikelem+6 ) = kmeshc( ipl+7 )

                      ikelem = ikelem + 6

                      iorig(ioelem+1) = ielold
                      iorig(ioelem+2) = ielold
                      ioelem = ioelem + 2

                   end if

                   ipl = ipl + inpelm

200            continue

             end if

          else if ( ndim.eq.3 ) then

             if ( meshtp.lt.11 .or. meshtp.gt.16 ) then

!              Group not interesting, skip and goto end

               ipl = ipl + inpelm * inelem
               ielold = ielold + inelem

             else

!              Run through all elements and divide

               do 600 iel = 1 , inelem

                   if ( meshtp.eq.11 ) then

!                     tetrahedron with four nodes

                      kelem( ikelem+1 ) = kmeshc( ipl+1 )
                      kelem( ikelem+2 ) = kmeshc( ipl+2 )
                      kelem( ikelem+3 ) = kmeshc( ipl+3 )
                      kelem( ikelem+4 ) = kmeshc( ipl+4 )

                      ikelem = ikelem + 4

                      iorig( ioelem+1 ) = ielold
                      ioelem = ioelem + 1

                   else if ( meshtp.eq.12 .or. meshtp.eq.15 .or. &
                             meshtp.eq.16 ) then

!                     tetrahedron with 10, 14 or 15 nodes

                      kelem(ikelem+1) = kmeshc(ipl+1)
                      kelem(ikelem+2) = kmeshc(ipl+3)
                      kelem(ikelem+3) = kmeshc(ipl+5)
                      kelem(ikelem+4) = kmeshc(ipl+10)

                      ikelem = ikelem + 4

                      iorig( ioelem+1 ) = ielold
                      ioelem = ioelem + 1

                   else if ( meshtp.eq.13 ) then

!                     hexahedron with eight nodes

                      kelem( ikelem+ 1 ) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 2 ) = kmeshc( ipl+ 6)
                      kelem( ikelem+ 3 ) = kmeshc( ipl+ 2)
                      kelem( ikelem+ 4 ) = kmeshc( ipl+ 7)

                      kelem( ikelem+ 5 ) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 6 ) = kmeshc( ipl+ 2)
                      kelem( ikelem+ 7 ) = kmeshc( ipl+ 3)
                      kelem( ikelem+ 8 ) = kmeshc( ipl+ 7)

                      kelem( ikelem+ 9 ) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 10) = kmeshc( ipl+ 3)
                      kelem( ikelem+ 11) = kmeshc( ipl+ 4)
                      kelem( ikelem+ 12) = kmeshc( ipl+ 7)

                      kelem( ikelem+ 13) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 14) = kmeshc( ipl+ 4)
                      kelem( ikelem+ 15) = kmeshc( ipl+ 8)
                      kelem( ikelem+ 16) = kmeshc( ipl+ 7)

                      kelem( ikelem+ 17) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 18) = kmeshc( ipl+ 8)
                      kelem( ikelem+ 19) = kmeshc( ipl+ 5)
                      kelem( ikelem+ 20) = kmeshc( ipl+ 7)

                      kelem( ikelem+ 21) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 22) = kmeshc( ipl+ 5)
                      kelem( ikelem+ 23) = kmeshc( ipl+ 6)
                      kelem( ikelem+ 24) = kmeshc( ipl+ 7)

                      ikelem = ikelem + 24

                      do 550 i = 1 , 6
                          iorig( ioelem+i ) = ielold
550                   continue

                      ioelem = ioelem + 6

                   else if ( meshtp.eq.14 ) then

!                     hexahedron with 27 nodes

                      kelem( ikelem+ 1 ) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 2 ) = kmeshc( ipl+21)
                      kelem( ikelem+ 3 ) = kmeshc( ipl+ 3)
                      kelem( ikelem+ 4 ) = kmeshc( ipl+27)

                      kelem( ikelem+ 5 ) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 6 ) = kmeshc( ipl+ 3)
                      kelem( ikelem+ 7 ) = kmeshc( ipl+ 9)
                      kelem( ikelem+ 8 ) = kmeshc( ipl+27)

                      kelem( ikelem+ 9 ) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 10) = kmeshc( ipl+ 9)
                      kelem( ikelem+ 11) = kmeshc( ipl+ 7)
                      kelem( ikelem+ 12) = kmeshc( ipl+27)

                      kelem( ikelem+ 13) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 14) = kmeshc( ipl+ 7)
                      kelem( ikelem+ 15) = kmeshc( ipl+25)
                      kelem( ikelem+ 16) = kmeshc( ipl+27)

                      kelem( ikelem+ 17) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 18) = kmeshc( ipl+25)
                      kelem( ikelem+ 19) = kmeshc( ipl+19)
                      kelem( ikelem+ 20) = kmeshc( ipl+27)

                      kelem( ikelem+ 21) = kmeshc( ipl+ 1)
                      kelem( ikelem+ 22) = kmeshc( ipl+19)
                      kelem( ikelem+ 23) = kmeshc( ipl+21)
                      kelem( ikelem+ 24) = kmeshc( ipl+27)

                      ikelem = ikelem + 24

                      do 560 i = 1 , 6
                          iorig( ioelem+i ) = ielold
560                   continue

                      ioelem = ioelem + 6

                   end if

                   ipl = ipl + inpelm

600            continue

             end if

          end if

!         Store number of triangles or tetrahedrons

          if ( ivul.eq.1 ) then

             if ( ndim.eq.2 ) ihulp(ielgrp+1) = ikelem / 3
             if ( ndim.eq.3 ) ihulp(ielgrp+1) = ikelem / 4

          end if

1000  continue

      end
