      subroutine topqu3 ( kmesh, kmeshc, kmesht, itrian, nelgrp, &
                          neleml, kline )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.8  date 16-02-2005 Correction connection elements
!                                      and hexahedrons
!        version  1.7  date 28-05-1997 Extension with hexahedrons
!        version  1.6  date 23-12-1996 Extension with 4-point triangles
!        version  1.5  date 26-07-1993 Correction ctypdv
!        version  1.4  date 21-05-1993 Linear and quadratic line elements
!        version  1.3  date 07-04-1993 3D added, np
!        version  1.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.1  date 09-05-1989 Small error correction
!        version  1.0  date 20-06-1988 Developed at Toshiba PC
!
!   copyright (c) 1988-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Divide quadratic triangular or quadrilateral elements (2 and 3D)
!     into elements of linear shape. Use info of array ITRIAN. If needed
!     divide line elements into linear elements.
! **********************************************************************
!
!                       KEYWORDS
!
!     subdivision
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/ctyp3df90'
      include 'SPcommon/ctypdvf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kmeshc(*), kmesht(*), nelgrp, itrian(nelgrp), &
              neleml, kline(*)

!     itrian  i   Input array of length NELGRP in which information of the
!                 various element groups is stored with respect to the
!                 subdivision into linear triangles.
!                 itrian(ielgrp) corresponds to the ielgrp th element
!                 group and its value has the following meaning:
!                 -3:   Subdivision of a quadratic line element
!                 -2:   Store linear line element
!                 -1:   Skip the element group
!                  0:   No subdivision required
!                  1:   Subdivision of a quadrilateral with 4 points into 2
!                       triangles
!                  2:   Subdivision of a triangle with 6 or 7 points into 4
!                       triangles
!                  3:   Subdivision of a triangle with 6 or 7 points into 1
!                       triangle
!                  4:   Subdivision of a quadrilateral with 9 points into 8
!                       triangles
!                  5:   Subdivision of a quadrilateral with 9 points into 8
!                       triangles. The centroid must be computed by
!                       interpolation
!                  6:   Subdivision of a quadrilateral with 9 points into 2
!                       triangles.
!                  7:   Subdivision of a tetrahedron with 10, 14 or 15 points
!                       into 8 tetrahedrons
!                  8:   Subdivision of a tetrahedron with 10, 14 or 15 points
!                       into 1 tetrahedron
!                  9:   Subdivision of a hexahedron with 8 points into 6
!                       tetrahedrons
!                 10:   Subdivision of a hexahedron with 27 points into 48
!                       tetrahedrons
!                 11:   Subdivision of a hexahedron with 27 points into
!                       6 tetrahedrons
!                 12:   Subdivision of a hexahedron with 27 points into 6
!                       hexahedrons
!                 13:   Subdivision of a hexahedron with 27 points into 1
!                       hexahedron
!     kline   o   Array containing "line"elements
!     kmesh   i   Standard SEPRAN array containing information concerning
!                 the mesh
!     kmeshc  i   Old array KMESH part c, containing the nodes of the
!                 elements. KMESHC must be subdivided
!     kmesht  o   Subdivided KMESH part c
!     neleml  o   Number of line elements in kline
!     nelgrp  i   Number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kelma, inpelm, inelem, kelmf, kelmg, i, ielgrp, &
              kelmc, kelmt, istart, ishape, iielem, nsubel, &
              j, k, isub12(64), isub13(8)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ielgrp         Element group number (loop variable)
!     iielem         Relative element number with respect to element group
!     inelem         Number of elements in an element group
!     inpelm         Number of nodes in an element
!     ishape         Shape number of element
!     istart         Start address minus one of information of an element either
!                    in array ISHSUB or in array ITETRA
!     isub12         Defines subdivision in case itrian(ielgrp)=12
!     isub13         Defines subdivision in case itrian(ielgrp)=13
!     j              Count helpvariable
!     k              Count helpvariable
!     kelma          Start adress of KMESH part a in array KMESH.
!                    KMESH part a contains the number of nodes per element of a
!                    specific element group accumulated.
!                    If KELMA = 0, then KMESH part a has not been filled
!     kelmc          Last position used in array KMESHC
!     kelmf          Start adress of KMESH part f in array KMESH.
!                    KMESH part f contains the shape numbers of the elements
!                    in an element group
!     kelmg          Start adress of KMESH part g in array KMESH.
!                    KMESH part g contains the number of elements of
!                    an element group
!                    KMESH part g is only filled if NELGRP > 1
!     kelmt          Last position used in array KMESHT
!     nsubel         Product of number of subelements and number of points in a
!                    subelement of one element to be divided
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all element groups
!       Compute for each group shape number, number of nodes in standard
!       element and number of elements
!       Check whether elements must be subdivided
!       If yes then perform subdivision
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data isub12 / 1,  2,  5,  4, 10, 11, 14, 13, &
                    2,  3,  6,  5, 11, 12, 15, 14, &
                    4,  5,  8,  7, 13, 14, 17, 16, &
                    5,  6,  9,  8, 14, 15, 18, 17, &
                   10, 11, 14, 13, 19, 20, 23, 22, &
                   11, 12, 15, 14, 20, 21, 24, 23, &
                   13, 14, 17, 16, 22, 23, 26, 25, &
                   14, 15, 18, 17, 23, 24, 27, 26 /
      data isub13 / 1,  3,  9,  7, 19, 21, 27, 25 /
! ======================================================================
!
      call eropen ( 'topqu3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topqu3'
         call prinin ( itrian, nelgrp, 'itrian' )
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Fill some constants

      inpelm = kmesh(4)
      inelem = kmesh(9)
      kelma  = kmesh(15)
      kelmf  = kmesh(20)
      kelmg  = kmesh(21)
      if ( debug ) write(irefwr,100) 'inpelm, inelem', inpelm, inelem

      kelmc  = 0
      neleml = 0
      kelmt  = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Compute shape number, number of nodes and number of elements
!         of element group IELGRP

         ishape = kmesh ( kelmf + ielgrp - 1 )
         if ( kelma.ne.0 ) inpelm = kmesh ( kelma + ielgrp ) - &
                                    kmesh ( kelma + ielgrp - 1 )
         if ( nelgrp.gt.1 ) inelem = kmesh ( kelmg + ielgrp - 1 )
         if ( debug ) &
            write(irefwr,100) 'ielgrp, ishape, inelem', &
                               ielgrp, ishape, inelem

!        --- Loop over elements, distinguish between elements to be subdivided
!            and elements that remain unchanged
!            The action depends on array itrian

         if ( itrian(ielgrp).eq.-3 ) then

!        --- Quadratic line element, store vertices only

            do iielem = 1 , inelem

                j = 2 * neleml + 2 * ( iielem - 1 )
                k =     kelmc  + 3 * ( iielem - 1 )

                kline( j + 1 ) = kmeshc( k + 1 )
                kline( j + 2 ) = kmeshc( k + 3 )

            end do

            neleml = neleml + inelem

         else if ( itrian(ielgrp).eq.-2 ) then

!        --- Linear line element

            j = 2 * neleml

            do iielem = 1, 2*inelem

                kline( j + iielem ) = kmeshc( kelmc + iielem )

            end do

            neleml = neleml + inelem

         else if ( itrian(ielgrp).eq.0 ) then

!        --- Linear triangular element

            do i = 1, 3
               do iielem = 1,inelem

                  kmesht ( kelmt + (iielem-1)*3 + i ) = &
                     kmeshc ( kelmc + (iielem-1)*inpelm + i )

               end do
            end do

            kelmt = kelmt + 3 * inelem

         else if ( itrian(ielgrp).gt.0 ) then

!        --- Quadratic element or quadrilateral found, subdivide

            istart = ishstr(ishape)
            if ( itrian(ielgrp).eq.-1 ) then
               nsubel =   0
            else if ( itrian(ielgrp).eq.1 ) then
               nsubel =   6
               istart =  ishstr(5)
            else if ( itrian(ielgrp).eq. 2 ) then
               nsubel =  12
               istart =  ishstr(4)
            else if ( itrian(ielgrp).eq. 3 ) then
               nsubel =   3
               istart =  ishstr(4) + 12
            else if ( itrian(ielgrp).eq. 4 .or. &
                      itrian(ielgrp).eq. 5 ) then
               nsubel =  24
               istart =  istart + 20
            else if ( itrian(ielgrp).eq. 6 ) then
               nsubel =   6
               istart =  istart + 44
            else if ( itrian(ielgrp).eq. 7 ) then
               nsubel =  32
               istart =   8
            else if ( itrian(ielgrp).eq. 8 ) then
               nsubel =   4
               istart =   4
            else if ( itrian(ielgrp).eq. 9 ) then
               nsubel =  24
               istart =  40
            else if ( itrian(ielgrp).eq.10 ) then
               nsubel = 192
               istart =  88
            else if ( itrian(ielgrp).eq.11 ) then
               nsubel =  24
               istart =  64
            end if

!           --- In 3d use itetra from ctyp3d and in 2d ishsub from
!               ctypdv

            if ( itrian(ielgrp).eq.12 ) then

!           --- subdivide hexahedron into 8 hexahedrae

              do i = 1, 64
                 do iielem = 1,inelem

                    kmesht(kelmt+(iielem-1)*64+i) = &
                    kmeshc(kelmc+(iielem-1)*inpelm+isub12(i))

                 end do
              end do

            else if ( itrian(ielgrp).eq.13 ) then

!           --- subdivide hexahedron into 1 hexahedron

              do i = 1, 8
                 do iielem = 1,inelem

                    kmesht(kelmt+(iielem-1)*8+i) = &
                    kmeshc(kelmc+(iielem-1)*inpelm+isub13(i))

                 end do
              end do

            else if ( itrian(ielgrp).gt.6 ) then

              do i = 1, nsubel
                 do iielem = 1,inelem

                    kmesht(kelmt+(iielem-1)*nsubel+i) = &
                    kmeshc(kelmc+(iielem-1)*inpelm+itetra(istart+i))

                 end do
              end do

              kelmt = kelmt + nsubel * inelem

            else

              do i = 1, nsubel
                 do iielem = 1, inelem

                    kmesht(kelmt+(iielem-1)*nsubel+i) = &
                    kmeshc(kelmc+(iielem-1)*inpelm+ishsub(istart+i))

                 end do
              end do

              kelmt = kelmt + nsubel * inelem

            end if

         end if

         kelmc = kelmc + inpelm * inelem

      end do

1000  call erclos ( 'topqu3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'neleml, kelmt', neleml, kelmt
         write(irefwr,*) 'End topqu3'

      end if

      end
