      subroutine writbs ( ichois, inum, numarr, namevc, iarray, ihelp )
! ====================================================================== &
!
!        programmer    Niek Praagman
!        version  5.2  date 29-05-2014 remove call to time subroutine
!        version  5.1  date 12-08-1999 New call to erclmn
!        version  5.0  date 03-04-1999 New body
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     writbs writes arrays to  background storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     backing_storage
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuf
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character *(*) namevc
      integer ichois, inum, numarr, iarray(*), ihelp(*)

!     iarray  i    Array to be written on backing storage.
!                  IARRAY may be both a real or an integer array.
!                  SEPRAN arrays are always integer arrays.
!     ichois  i    Choice parameter. This parameter indicates whether the array
!                  is a SEPRAN array ( ICHOIS = 0 ) or not ( ICHOIS # 0 ).
!                  When IARRAY corresponds to a user array, the value of
!                  |ICHOIS| denotes the length of this array, and the sign of
!                  ICHOIS denotes whether it is a real array (ICHOIS<0) or an
!                  integer array (ICHOIS>0).
!     ihelp   i    integer extra array needed.
!                  Extra arrays necessary for SEPRAN arrays:
!                    IARRAY is KMESH:   IHELP is dummy
!                    IARRAY is KPROB:   IHELP is KMESH
!                    IARRAY is ISOL:    IHELP is KPROB
!                    IARRAY is INTMAT:  IHELP is KPROB
!                    IARRAY is MATR:    IHELP is dummy
!     inum    i    Choice parameter. Possibilities:
!                  When INUM<=0 the array is written on the next free
!                  positions on backing storage.
!                  At most 500 arrays can be written on the permanent file.
!                  When INUM = -1 the subroutine prints a text of the following
!                  form
!                  Array "NAME" is stored as array number "NUMARR"
!                  where "NAME" denotes the value of NAME and "NUMARR" denotes
!                  the value of NUMARR.
!                  When INUM>0  the array is written on positions of a
!                  preceding written array, with array number INUM.
!                  See also NUMARR.
!                  The length of the new and old array may be different.
!                  This possibility may also be used when there is no preceding
!                  array with this number.
!     namevc  i    Name of the array to be written to backing storage.
!                  Name is a character constant of at most 10 characters.
!                  NAME is only used for printing purposes, when INUM = -1.
!     numarr  o    Each array that is written by WRITBS gets a sequence number, &
!                  the first array gets number 1, the second number 2 etc.
!                  At output NUMARR gets the value of this sequence number.
!                  Therefore NUMARR must be a variable and not a constant.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     WRITBSBF       Actual body of writbs
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'writbs' )
      call writbsbf ( ibuffr, buffer, ichois, inum, numarr, &
                      namevc, iarray, ihelp )
      call erclos ( 'writbs' )
      end
